// This prevents pollution of the global namespace
var SQL = (function () {
// The Module object: Our interface to the outside world. We import
// and export values on it, and do the work to get that through
// closure compiler if necessary. There are various ways Module can be used:
// 1. Not defined. We create it here
// 2. A function parameter, function(Module) { ..generated code.. }
// 3. pre-run appended it, var Module = {}; ..generated code..
// 4. External script tag defines var Module.
// We need to do an eval in order to handle the closure compiler
// case, where this code here is minified but Module was defined
// elsewhere (e.g. case 4 above). We also need to check if Module
// already exists (e.g. case 3 above).
// Note that if you want to run closure, and also to use Module
// after the generated code, you will need to define   var Module = {};
// before the code. Then that object will be used in the code, and you
// can continue to use Module afterwards as well.
var Module;
if (!Module) Module = (typeof Module !== 'undefined' ? Module : null) || {};

// Sometimes an existing Module object exists with properties
// meant to overwrite the default module functionality. Here
// we collect those properties and reapply _after_ we configure
// the current environment's defaults to avoid having to be so
// defensive during initialization.
var moduleOverrides = {};
for (var key in Module) {
  if (Module.hasOwnProperty(key)) {
    moduleOverrides[key] = Module[key];
  }
}

// The environment setup code below is customized to use Module.
// *** Environment setup code ***
var ENVIRONMENT_IS_WEB = typeof window === 'object';
// Three configurations we can be running in:
// 1) We could be the application main() thread running in the main JS UI thread. (ENVIRONMENT_IS_WORKER == false and ENVIRONMENT_IS_PTHREAD == false)
// 2) We could be the application main() thread proxied to worker. (with Emscripten -s PROXY_TO_WORKER=1) (ENVIRONMENT_IS_WORKER == true, ENVIRONMENT_IS_PTHREAD == false)
// 3) We could be an application pthread running in a worker. (ENVIRONMENT_IS_WORKER == true and ENVIRONMENT_IS_PTHREAD == true)
var ENVIRONMENT_IS_WORKER = typeof importScripts === 'function';
var ENVIRONMENT_IS_NODE = typeof process === 'object' && typeof require === 'function' && !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_WORKER;
var ENVIRONMENT_IS_SHELL = !ENVIRONMENT_IS_WEB && !ENVIRONMENT_IS_NODE && !ENVIRONMENT_IS_WORKER;

if (ENVIRONMENT_IS_NODE) {
  // Expose functionality in the same simple way that the shells work
  // Note that we pollute the global namespace here, otherwise we break in node
  if (!Module['print']) Module['print'] = function print(x) {
    process['stdout'].write(x + '\n');
  };
  if (!Module['printErr']) Module['printErr'] = function printErr(x) {
    process['stderr'].write(x + '\n');
  };

  var nodeFS = require('fs');
  var nodePath = require('path');

  Module['read'] = function read(filename, binary) {
    filename = nodePath['normalize'](filename);
    var ret = nodeFS['readFileSync'](filename);
    // The path is absolute if the normalized version is the same as the resolved.
    if (!ret && filename != nodePath['resolve'](filename)) {
      filename = path.join(__dirname, '..', 'src', filename);
      ret = nodeFS['readFileSync'](filename);
    }
    if (ret && !binary) ret = ret.toString();
    return ret;
  };

  Module['readBinary'] = function readBinary(filename) {
    var ret = Module['read'](filename, true);
    if (!ret.buffer) {
      ret = new Uint8Array(ret);
    }
    assert(ret.buffer);
    return ret;
  };

  Module['load'] = function load(f) {
    globalEval(read(f));
  };

  if (!Module['thisProgram']) {
    if (process['argv'].length > 1) {
      Module['thisProgram'] = process['argv'][1].replace(/\\/g, '/');
    } else {
      Module['thisProgram'] = 'unknown-program';
    }
  }

  Module['arguments'] = process['argv'].slice(2);

  if (typeof module !== 'undefined') {
    module['exports'] = Module;
  }

  process['on']('uncaughtException', function(ex) {
    // suppress ExitStatus exceptions from showing an error
    if (!(ex instanceof ExitStatus)) {
      throw ex;
    }
  });

  Module['inspect'] = function () { return '[Emscripten Module object]'; };
}
else if (ENVIRONMENT_IS_SHELL) {
  if (!Module['print']) Module['print'] = print;
  if (typeof printErr != 'undefined') Module['printErr'] = printErr; // not present in v8 or older sm

  if (typeof read != 'undefined') {
    Module['read'] = read;
  } else {
    Module['read'] = function read() { throw 'no read() available (jsc?)' };
  }

  Module['readBinary'] = function readBinary(f) {
    if (typeof readbuffer === 'function') {
      return new Uint8Array(readbuffer(f));
    }
    var data = read(f, 'binary');
    assert(typeof data === 'object');
    return data;
  };

  if (typeof scriptArgs != 'undefined') {
    Module['arguments'] = scriptArgs;
  } else if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

}
else if (ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) {
  Module['read'] = function read(url) {
    var xhr = new XMLHttpRequest();
    xhr.open('GET', url, false);
    xhr.send(null);
    return xhr.responseText;
  };

  if (typeof arguments != 'undefined') {
    Module['arguments'] = arguments;
  }

  if (typeof console !== 'undefined') {
    if (!Module['print']) Module['print'] = function print(x) {
      console.log(x);
    };
    if (!Module['printErr']) Module['printErr'] = function printErr(x) {
      console.log(x);
    };
  } else {
    // Probably a worker, and without console.log. We can do very little here...
    var TRY_USE_DUMP = false;
    if (!Module['print']) Module['print'] = (TRY_USE_DUMP && (typeof(dump) !== "undefined") ? (function(x) {
      dump(x);
    }) : (function(x) {
      // self.postMessage(x); // enable this if you want stdout to be sent as messages
    }));
  }

  if (ENVIRONMENT_IS_WORKER) {
    Module['load'] = importScripts;
  }

  if (typeof Module['setWindowTitle'] === 'undefined') {
    Module['setWindowTitle'] = function(title) { document.title = title };
  }
}
else {
  // Unreachable because SHELL is dependant on the others
  throw 'Unknown runtime environment. Where are we?';
}

function globalEval(x) {
  eval.call(null, x);
}
if (!Module['load'] && Module['read']) {
  Module['load'] = function load(f) {
    globalEval(Module['read'](f));
  };
}
if (!Module['print']) {
  Module['print'] = function(){};
}
if (!Module['printErr']) {
  Module['printErr'] = Module['print'];
}
if (!Module['arguments']) {
  Module['arguments'] = [];
}
if (!Module['thisProgram']) {
  Module['thisProgram'] = './this.program';
}

// *** Environment setup code ***

// Closure helpers
Module.print = Module['print'];
Module.printErr = Module['printErr'];

// Callbacks
Module['preRun'] = [];
Module['postRun'] = [];

// Merge back in the overrides
for (var key in moduleOverrides) {
  if (moduleOverrides.hasOwnProperty(key)) {
    Module[key] = moduleOverrides[key];
  }
}



// === Preamble library stuff ===

// Documentation for the public APIs defined in this file must be updated in: 
//    site/source/docs/api_reference/preamble.js.rst
// A prebuilt local version of the documentation is available at: 
//    site/build/text/docs/api_reference/preamble.js.txt
// You can also build docs locally as HTML or other formats in site/
// An online HTML version (which may be of a different version of Emscripten)
//    is up at http://kripken.github.io/emscripten-site/docs/api_reference/preamble.js.html

//========================================
// Runtime code shared with compiler
//========================================

var Runtime = {
  setTempRet0: function (value) {
    tempRet0 = value;
  },
  getTempRet0: function () {
    return tempRet0;
  },
  stackSave: function () {
    return STACKTOP;
  },
  stackRestore: function (stackTop) {
    STACKTOP = stackTop;
  },
  getNativeTypeSize: function (type) {
    switch (type) {
      case 'i1': case 'i8': return 1;
      case 'i16': return 2;
      case 'i32': return 4;
      case 'i64': return 8;
      case 'float': return 4;
      case 'double': return 8;
      default: {
        if (type[type.length-1] === '*') {
          return Runtime.QUANTUM_SIZE; // A pointer
        } else if (type[0] === 'i') {
          var bits = parseInt(type.substr(1));
          assert(bits % 8 === 0);
          return bits/8;
        } else {
          return 0;
        }
      }
    }
  },
  getNativeFieldSize: function (type) {
    return Math.max(Runtime.getNativeTypeSize(type), Runtime.QUANTUM_SIZE);
  },
  STACK_ALIGN: 16,
  prepVararg: function (ptr, type) {
    if (type === 'double' || type === 'i64') {
      // move so the load is aligned
      if (ptr & 7) {
        assert((ptr & 7) === 4);
        ptr += 4;
      }
    } else {
      assert((ptr & 3) === 0);
    }
    return ptr;
  },
  getAlignSize: function (type, size, vararg) {
    // we align i64s and doubles on 64-bit boundaries, unlike x86
    if (!vararg && (type == 'i64' || type == 'double')) return 8;
    if (!type) return Math.min(size, 8); // align structures internally to 64 bits
    return Math.min(size || (type ? Runtime.getNativeFieldSize(type) : 0), Runtime.QUANTUM_SIZE);
  },
  dynCall: function (sig, ptr, args) {
    if (args && args.length) {
      if (!args.splice) args = Array.prototype.slice.call(args);
      args.splice(0, 0, ptr);
      return Module['dynCall_' + sig].apply(null, args);
    } else {
      return Module['dynCall_' + sig].call(null, ptr);
    }
  },
  functionPointers: [null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null,null],
  addFunction: function (func) {
    for (var i = 0; i < Runtime.functionPointers.length; i++) {
      if (!Runtime.functionPointers[i]) {
        Runtime.functionPointers[i] = func;
        return 2*(1 + i);
      }
    }
    throw 'Finished up all reserved function pointers. Use a higher value for RESERVED_FUNCTION_POINTERS.';
  },
  removeFunction: function (index) {
    Runtime.functionPointers[(index-2)/2] = null;
  },
  warnOnce: function (text) {
    if (!Runtime.warnOnce.shown) Runtime.warnOnce.shown = {};
    if (!Runtime.warnOnce.shown[text]) {
      Runtime.warnOnce.shown[text] = 1;
      Module.printErr(text);
    }
  },
  funcWrappers: {},
  getFuncWrapper: function (func, sig) {
    assert(sig);
    if (!Runtime.funcWrappers[sig]) {
      Runtime.funcWrappers[sig] = {};
    }
    var sigCache = Runtime.funcWrappers[sig];
    if (!sigCache[func]) {
      sigCache[func] = function dynCall_wrapper() {
        return Runtime.dynCall(sig, func, arguments);
      };
    }
    return sigCache[func];
  },
  getCompilerSetting: function (name) {
    throw 'You must build with -s RETAIN_COMPILER_SETTINGS=1 for Runtime.getCompilerSetting or emscripten_get_compiler_setting to work';
  },
  stackAlloc: function (size) { var ret = STACKTOP;STACKTOP = (STACKTOP + size)|0;STACKTOP = (((STACKTOP)+15)&-16); return ret; },
  staticAlloc: function (size) { var ret = STATICTOP;STATICTOP = (STATICTOP + size)|0;STATICTOP = (((STATICTOP)+15)&-16); return ret; },
  dynamicAlloc: function (size) { var ret = DYNAMICTOP;DYNAMICTOP = (DYNAMICTOP + size)|0;DYNAMICTOP = (((DYNAMICTOP)+15)&-16); if (DYNAMICTOP >= TOTAL_MEMORY) { var success = enlargeMemory(); if (!success) { DYNAMICTOP = ret;  return 0; } }; return ret; },
  alignMemory: function (size,quantum) { var ret = size = Math.ceil((size)/(quantum ? quantum : 16))*(quantum ? quantum : 16); return ret; },
  makeBigInt: function (low,high,unsigned) { var ret = (unsigned ? ((+((low>>>0)))+((+((high>>>0)))*4294967296.0)) : ((+((low>>>0)))+((+((high|0)))*4294967296.0))); return ret; },
  GLOBAL_BASE: 8,
  QUANTUM_SIZE: 4,
  __dummy__: 0
}



Module["Runtime"] = Runtime;



//========================================
// Runtime essentials
//========================================

var __THREW__ = 0; // Used in checking for thrown exceptions.

var ABORT = false; // whether we are quitting the application. no code should run after this. set in exit() and abort()
var EXITSTATUS = 0;

var undef = 0;
// tempInt is used for 32-bit signed values or smaller. tempBigInt is used
// for 32-bit unsigned values or more than 32 bits. TODO: audit all uses of tempInt
var tempValue, tempInt, tempBigInt, tempInt2, tempBigInt2, tempPair, tempBigIntI, tempBigIntR, tempBigIntS, tempBigIntP, tempBigIntD, tempDouble, tempFloat;
var tempI64, tempI64b;
var tempRet0, tempRet1, tempRet2, tempRet3, tempRet4, tempRet5, tempRet6, tempRet7, tempRet8, tempRet9;

function assert(condition, text) {
  if (!condition) {
    abort('Assertion failed: ' + text);
  }
}

var globalScope = this;

// Returns the C function with a specified identifier (for C++, you need to do manual name mangling)
function getCFunc(ident) {
  var func = Module['_' + ident]; // closure exported function
  if (!func) {
    try {
      func = eval('_' + ident); // explicit lookup
    } catch(e) {}
  }
  assert(func, 'Cannot call unknown function ' + ident + ' (perhaps LLVM optimizations or closure removed it?)');
  return func;
}

var cwrap, ccall;
(function(){
  var JSfuncs = {
    // Helpers for cwrap -- it can't refer to Runtime directly because it might
    // be renamed by closure, instead it calls JSfuncs['stackSave'].body to find
    // out what the minified function name is.
    'stackSave': function() {
      Runtime.stackSave()
    },
    'stackRestore': function() {
      Runtime.stackRestore()
    },
    // type conversion from js to c
    'arrayToC' : function(arr) {
      var ret = Runtime.stackAlloc(arr.length);
      writeArrayToMemory(arr, ret);
      return ret;
    },
    'stringToC' : function(str) {
      var ret = 0;
      if (str !== null && str !== undefined && str !== 0) { // null string
        // at most 4 bytes per UTF-8 code point, +1 for the trailing '\0'
        ret = Runtime.stackAlloc((str.length << 2) + 1);
        writeStringToMemory(str, ret);
      }
      return ret;
    }
  };
  // For fast lookup of conversion functions
  var toC = {'string' : JSfuncs['stringToC'], 'array' : JSfuncs['arrayToC']};

  // C calling interface. 
  ccall = function ccallFunc(ident, returnType, argTypes, args, opts) {
    var func = getCFunc(ident);
    var cArgs = [];
    var stack = 0;
    if (args) {
      for (var i = 0; i < args.length; i++) {
        var converter = toC[argTypes[i]];
        if (converter) {
          if (stack === 0) stack = Runtime.stackSave();
          cArgs[i] = converter(args[i]);
        } else {
          cArgs[i] = args[i];
        }
      }
    }
    var ret = func.apply(null, cArgs);
    if (returnType === 'string') ret = Pointer_stringify(ret);
    if (stack !== 0) {
      if (opts && opts.async) {
        EmterpreterAsync.asyncFinalizers.push(function() {
          Runtime.stackRestore(stack);
        });
        return;
      }
      Runtime.stackRestore(stack);
    }
    return ret;
  }

  var sourceRegex = /^function\s*\(([^)]*)\)\s*{\s*([^*]*?)[\s;]*(?:return\s*(.*?)[;\s]*)?}$/;
  function parseJSFunc(jsfunc) {
    // Match the body and the return value of a javascript function source
    var parsed = jsfunc.toString().match(sourceRegex).slice(1);
    return {arguments : parsed[0], body : parsed[1], returnValue: parsed[2]}
  }
  var JSsource = {};
  for (var fun in JSfuncs) {
    if (JSfuncs.hasOwnProperty(fun)) {
      // Elements of toCsource are arrays of three items:
      // the code, and the return value
      JSsource[fun] = parseJSFunc(JSfuncs[fun]);
    }
  }

  
  cwrap = function cwrap(ident, returnType, argTypes) {
    argTypes = argTypes || [];
    var cfunc = getCFunc(ident);
    // When the function takes numbers and returns a number, we can just return
    // the original function
    var numericArgs = argTypes.every(function(type){ return type === 'number'});
    var numericRet = (returnType !== 'string');
    if ( numericRet && numericArgs) {
      return cfunc;
    }
    // Creation of the arguments list (["$1","$2",...,"$nargs"])
    var argNames = argTypes.map(function(x,i){return '$'+i});
    var funcstr = "(function(" + argNames.join(',') + ") {";
    var nargs = argTypes.length;
    if (!numericArgs) {
      // Generate the code needed to convert the arguments from javascript
      // values to pointers
      funcstr += 'var stack = ' + JSsource['stackSave'].body + ';';
      for (var i = 0; i < nargs; i++) {
        var arg = argNames[i], type = argTypes[i];
        if (type === 'number') continue;
        var convertCode = JSsource[type + 'ToC']; // [code, return]
        funcstr += 'var ' + convertCode.arguments + ' = ' + arg + ';';
        funcstr += convertCode.body + ';';
        funcstr += arg + '=' + convertCode.returnValue + ';';
      }
    }

    // When the code is compressed, the name of cfunc is not literally 'cfunc' anymore
    var cfuncname = parseJSFunc(function(){return cfunc}).returnValue;
    // Call the function
    funcstr += 'var ret = ' + cfuncname + '(' + argNames.join(',') + ');';
    if (!numericRet) { // Return type can only by 'string' or 'number'
      // Convert the result to a string
      var strgfy = parseJSFunc(function(){return Pointer_stringify}).returnValue;
      funcstr += 'ret = ' + strgfy + '(ret);';
    }
    if (!numericArgs) {
      // If we had a stack, restore it
      funcstr += JSsource['stackRestore'].body.replace('()', '(stack)') + ';';
    }
    funcstr += 'return ret})';
    return eval(funcstr);
  };
})();
Module["ccall"] = ccall;
Module["cwrap"] = cwrap;

function setValue(ptr, value, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': HEAP8[((ptr)>>0)]=value; break;
      case 'i8': HEAP8[((ptr)>>0)]=value; break;
      case 'i16': HEAP16[((ptr)>>1)]=value; break;
      case 'i32': HEAP32[((ptr)>>2)]=value; break;
      case 'i64': (tempI64 = [value>>>0,(tempDouble=value,(+(Math_abs(tempDouble))) >= 1.0 ? (tempDouble > 0.0 ? ((Math_min((+(Math_floor((tempDouble)/4294967296.0))), 4294967295.0))|0)>>>0 : (~~((+(Math_ceil((tempDouble - +(((~~(tempDouble)))>>>0))/4294967296.0)))))>>>0) : 0)],HEAP32[((ptr)>>2)]=tempI64[0],HEAP32[(((ptr)+(4))>>2)]=tempI64[1]); break;
      case 'float': HEAPF32[((ptr)>>2)]=value; break;
      case 'double': HEAPF64[((ptr)>>3)]=value; break;
      default: abort('invalid type for setValue: ' + type);
    }
}
Module["setValue"] = setValue;


function getValue(ptr, type, noSafe) {
  type = type || 'i8';
  if (type.charAt(type.length-1) === '*') type = 'i32'; // pointers are 32-bit
    switch(type) {
      case 'i1': return HEAP8[((ptr)>>0)];
      case 'i8': return HEAP8[((ptr)>>0)];
      case 'i16': return HEAP16[((ptr)>>1)];
      case 'i32': return HEAP32[((ptr)>>2)];
      case 'i64': return HEAP32[((ptr)>>2)];
      case 'float': return HEAPF32[((ptr)>>2)];
      case 'double': return HEAPF64[((ptr)>>3)];
      default: abort('invalid type for setValue: ' + type);
    }
  return null;
}
Module["getValue"] = getValue;

var ALLOC_NORMAL = 0; // Tries to use _malloc()
var ALLOC_STACK = 1; // Lives for the duration of the current function call
var ALLOC_STATIC = 2; // Cannot be freed
var ALLOC_DYNAMIC = 3; // Cannot be freed except through sbrk
var ALLOC_NONE = 4; // Do not allocate
Module["ALLOC_NORMAL"] = ALLOC_NORMAL;
Module["ALLOC_STACK"] = ALLOC_STACK;
Module["ALLOC_STATIC"] = ALLOC_STATIC;
Module["ALLOC_DYNAMIC"] = ALLOC_DYNAMIC;
Module["ALLOC_NONE"] = ALLOC_NONE;

// allocate(): This is for internal use. You can use it yourself as well, but the interface
//             is a little tricky (see docs right below). The reason is that it is optimized
//             for multiple syntaxes to save space in generated code. So you should
//             normally not use allocate(), and instead allocate memory using _malloc(),
//             initialize it with setValue(), and so forth.
// @slab: An array of data, or a number. If a number, then the size of the block to allocate,
//        in *bytes* (note that this is sometimes confusing: the next parameter does not
//        affect this!)
// @types: Either an array of types, one for each byte (or 0 if no type at that position),
//         or a single type which is used for the entire block. This only matters if there
//         is initial data - if @slab is a number, then this does not matter at all and is
//         ignored.
// @allocator: How to allocate memory, see ALLOC_*
function allocate(slab, types, allocator, ptr) {
  var zeroinit, size;
  if (typeof slab === 'number') {
    zeroinit = true;
    size = slab;
  } else {
    zeroinit = false;
    size = slab.length;
  }

  var singleType = typeof types === 'string' ? types : null;

  var ret;
  if (allocator == ALLOC_NONE) {
    ret = ptr;
  } else {
    ret = [_malloc, Runtime.stackAlloc, Runtime.staticAlloc, Runtime.dynamicAlloc][allocator === undefined ? ALLOC_STATIC : allocator](Math.max(size, singleType ? 1 : types.length));
  }

  if (zeroinit) {
    var ptr = ret, stop;
    assert((ret & 3) == 0);
    stop = ret + (size & ~3);
    for (; ptr < stop; ptr += 4) {
      HEAP32[((ptr)>>2)]=0;
    }
    stop = ret + size;
    while (ptr < stop) {
      HEAP8[((ptr++)>>0)]=0;
    }
    return ret;
  }

  if (singleType === 'i8') {
    if (slab.subarray || slab.slice) {
      HEAPU8.set(slab, ret);
    } else {
      HEAPU8.set(new Uint8Array(slab), ret);
    }
    return ret;
  }

  var i = 0, type, typeSize, previousType;
  while (i < size) {
    var curr = slab[i];

    if (typeof curr === 'function') {
      curr = Runtime.getFunctionIndex(curr);
    }

    type = singleType || types[i];
    if (type === 0) {
      i++;
      continue;
    }

    if (type == 'i64') type = 'i32'; // special case: we have one i32 here, and one i32 later

    setValue(ret+i, curr, type);

    // no need to look up size unless type changes, so cache it
    if (previousType !== type) {
      typeSize = Runtime.getNativeTypeSize(type);
      previousType = type;
    }
    i += typeSize;
  }

  return ret;
}
Module["allocate"] = allocate;

// Allocate memory during any stage of startup - static memory early on, dynamic memory later, malloc when ready
function getMemory(size) {
  if (!staticSealed) return Runtime.staticAlloc(size);
  if ((typeof _sbrk !== 'undefined' && !_sbrk.called) || !runtimeInitialized) return Runtime.dynamicAlloc(size);
  return _malloc(size);
}
Module["getMemory"] = getMemory;

function Pointer_stringify(ptr, /* optional */ length) {
  if (length === 0 || !ptr) return '';
  // TODO: use TextDecoder
  // Find the length, and check for UTF while doing so
  var hasUtf = 0;
  var t;
  var i = 0;
  while (1) {
    t = HEAPU8[(((ptr)+(i))>>0)];
    hasUtf |= t;
    if (t == 0 && !length) break;
    i++;
    if (length && i == length) break;
  }
  if (!length) length = i;

  var ret = '';

  if (hasUtf < 128) {
    var MAX_CHUNK = 1024; // split up into chunks, because .apply on a huge string can overflow the stack
    var curr;
    while (length > 0) {
      curr = String.fromCharCode.apply(String, HEAPU8.subarray(ptr, ptr + Math.min(length, MAX_CHUNK)));
      ret = ret ? ret + curr : curr;
      ptr += MAX_CHUNK;
      length -= MAX_CHUNK;
    }
    return ret;
  }
  return Module['UTF8ToString'](ptr);
}
Module["Pointer_stringify"] = Pointer_stringify;

// Given a pointer 'ptr' to a null-terminated ASCII-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function AsciiToString(ptr) {
  var str = '';
  while (1) {
    var ch = HEAP8[((ptr++)>>0)];
    if (!ch) return str;
    str += String.fromCharCode(ch);
  }
}
Module["AsciiToString"] = AsciiToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in ASCII form. The copy will require at most str.length+1 bytes of space in the HEAP.

function stringToAscii(str, outPtr) {
  return writeAsciiToMemory(str, outPtr, false);
}
Module["stringToAscii"] = stringToAscii;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the given array that contains uint8 values, returns
// a copy of that string as a Javascript String object.

function UTF8ArrayToString(u8Array, idx) {
  var u0, u1, u2, u3, u4, u5;

  var str = '';
  while (1) {
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    u0 = u8Array[idx++];
    if (!u0) return str;
    if (!(u0 & 0x80)) { str += String.fromCharCode(u0); continue; }
    u1 = u8Array[idx++] & 63;
    if ((u0 & 0xE0) == 0xC0) { str += String.fromCharCode(((u0 & 31) << 6) | u1); continue; }
    u2 = u8Array[idx++] & 63;
    if ((u0 & 0xF0) == 0xE0) {
      u0 = ((u0 & 15) << 12) | (u1 << 6) | u2;
    } else {
      u3 = u8Array[idx++] & 63;
      if ((u0 & 0xF8) == 0xF0) {
        u0 = ((u0 & 7) << 18) | (u1 << 12) | (u2 << 6) | u3;
      } else {
        u4 = u8Array[idx++] & 63;
        if ((u0 & 0xFC) == 0xF8) {
          u0 = ((u0 & 3) << 24) | (u1 << 18) | (u2 << 12) | (u3 << 6) | u4;
        } else {
          u5 = u8Array[idx++] & 63;
          u0 = ((u0 & 1) << 30) | (u1 << 24) | (u2 << 18) | (u3 << 12) | (u4 << 6) | u5;
        }
      }
    }
    if (u0 < 0x10000) {
      str += String.fromCharCode(u0);
    } else {
      var ch = u0 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    }
  }
}
Module["UTF8ArrayToString"] = UTF8ArrayToString;

// Given a pointer 'ptr' to a null-terminated UTF8-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF8ToString(ptr) {
  return UTF8ArrayToString(HEAPU8,ptr);
}
Module["UTF8ToString"] = UTF8ToString;

// Copies the given Javascript String object 'str' to the given byte array at address 'outIdx',
// encoded in UTF8 form and null-terminated. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outU8Array: the array to copy to. Each index in this array is assumed to be one 8-byte element.
//   outIdx: The starting offset in the array to begin the copying.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=1, only the null terminator will be written and nothing else.
//                    maxBytesToWrite=0 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8Array(str, outU8Array, outIdx, maxBytesToWrite) {
  if (!(maxBytesToWrite > 0)) // Parameter maxBytesToWrite is not optional. Negative values, 0, null, undefined and false each don't write out any bytes.
    return 0;

  var startIdx = outIdx;
  var endIdx = outIdx + maxBytesToWrite - 1; // -1 for string null terminator.
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    // For UTF8 byte structure, see http://en.wikipedia.org/wiki/UTF-8#Description and https://www.ietf.org/rfc/rfc2279.txt and https://tools.ietf.org/html/rfc3629
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      if (outIdx >= endIdx) break;
      outU8Array[outIdx++] = u;
    } else if (u <= 0x7FF) {
      if (outIdx + 1 >= endIdx) break;
      outU8Array[outIdx++] = 0xC0 | (u >> 6);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0xFFFF) {
      if (outIdx + 2 >= endIdx) break;
      outU8Array[outIdx++] = 0xE0 | (u >> 12);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x1FFFFF) {
      if (outIdx + 3 >= endIdx) break;
      outU8Array[outIdx++] = 0xF0 | (u >> 18);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else if (u <= 0x3FFFFFF) {
      if (outIdx + 4 >= endIdx) break;
      outU8Array[outIdx++] = 0xF8 | (u >> 24);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    } else {
      if (outIdx + 5 >= endIdx) break;
      outU8Array[outIdx++] = 0xFC | (u >> 30);
      outU8Array[outIdx++] = 0x80 | ((u >> 24) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 18) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 12) & 63);
      outU8Array[outIdx++] = 0x80 | ((u >> 6) & 63);
      outU8Array[outIdx++] = 0x80 | (u & 63);
    }
  }
  // Null-terminate the pointer to the buffer.
  outU8Array[outIdx] = 0;
  return outIdx - startIdx;
}
Module["stringToUTF8Array"] = stringToUTF8Array;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF8 form. The copy will require at most str.length*4+1 bytes of space in the HEAP.
// Use the function lengthBytesUTF8() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF8(str, outPtr, maxBytesToWrite) {
  return stringToUTF8Array(str, HEAPU8,outPtr, maxBytesToWrite);
}
Module["stringToUTF8"] = stringToUTF8;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF8 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF8(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! So decode UTF16->UTF32->UTF8.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var u = str.charCodeAt(i); // possibly a lead surrogate
    if (u >= 0xD800 && u <= 0xDFFF) u = 0x10000 + ((u & 0x3FF) << 10) | (str.charCodeAt(++i) & 0x3FF);
    if (u <= 0x7F) {
      ++len;
    } else if (u <= 0x7FF) {
      len += 2;
    } else if (u <= 0xFFFF) {
      len += 3;
    } else if (u <= 0x1FFFFF) {
      len += 4;
    } else if (u <= 0x3FFFFFF) {
      len += 5;
    } else {
      len += 6;
    }
  }
  return len;
}
Module["lengthBytesUTF8"] = lengthBytesUTF8;

// Given a pointer 'ptr' to a null-terminated UTF16LE-encoded string in the emscripten HEAP, returns
// a copy of that string as a Javascript String object.

function UTF16ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var codeUnit = HEAP16[(((ptr)+(i*2))>>1)];
    if (codeUnit == 0)
      return str;
    ++i;
    // fromCharCode constructs a character from a UTF-16 code unit, so we can pass the UTF16 string right through.
    str += String.fromCharCode(codeUnit);
  }
}
Module["UTF16ToString"] = UTF16ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF16 form. The copy will require at most str.length*4+2 bytes of space in the HEAP.
// Use the function lengthBytesUTF16() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=2, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<2 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF16(str, outPtr, maxBytesToWrite) {
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 2) return 0;
  maxBytesToWrite -= 2; // Null terminator.
  var startPtr = outPtr;
  var numCharsToWrite = (maxBytesToWrite < str.length*2) ? (maxBytesToWrite / 2) : str.length;
  for (var i = 0; i < numCharsToWrite; ++i) {
    // charCodeAt returns a UTF-16 encoded code unit, so it can be directly written to the HEAP.
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    HEAP16[((outPtr)>>1)]=codeUnit;
    outPtr += 2;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP16[((outPtr)>>1)]=0;
  return outPtr - startPtr;
}
Module["stringToUTF16"] = stringToUTF16;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF16(str) {
  return str.length*2;
}
Module["lengthBytesUTF16"] = lengthBytesUTF16;

function UTF32ToString(ptr) {
  var i = 0;

  var str = '';
  while (1) {
    var utf32 = HEAP32[(((ptr)+(i*4))>>2)];
    if (utf32 == 0)
      return str;
    ++i;
    // Gotcha: fromCharCode constructs a character from a UTF-16 encoded code (pair), not from a Unicode code point! So encode the code point to UTF-16 for constructing.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    if (utf32 >= 0x10000) {
      var ch = utf32 - 0x10000;
      str += String.fromCharCode(0xD800 | (ch >> 10), 0xDC00 | (ch & 0x3FF));
    } else {
      str += String.fromCharCode(utf32);
    }
  }
}
Module["UTF32ToString"] = UTF32ToString;

// Copies the given Javascript String object 'str' to the emscripten HEAP at address 'outPtr',
// null-terminated and encoded in UTF32 form. The copy will require at most str.length*4+4 bytes of space in the HEAP.
// Use the function lengthBytesUTF32() to compute the exact number of bytes (excluding null terminator) that this function will write.
// Parameters:
//   str: the Javascript string to copy.
//   outPtr: Byte address in Emscripten HEAP where to write the string to.
//   maxBytesToWrite: The maximum number of bytes this function can write to the array. This count should include the null 
//                    terminator, i.e. if maxBytesToWrite=4, only the null terminator will be written and nothing else.
//                    maxBytesToWrite<4 does not write any bytes to the output, not even the null terminator.
// Returns the number of bytes written, EXCLUDING the null terminator.

function stringToUTF32(str, outPtr, maxBytesToWrite) {
  // Backwards compatibility: if max bytes is not specified, assume unsafe unbounded write is allowed.
  if (maxBytesToWrite === undefined) {
    maxBytesToWrite = 0x7FFFFFFF;
  }
  if (maxBytesToWrite < 4) return 0;
  var startPtr = outPtr;
  var endPtr = startPtr + maxBytesToWrite - 4;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i); // possibly a lead surrogate
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) {
      var trailSurrogate = str.charCodeAt(++i);
      codeUnit = 0x10000 + ((codeUnit & 0x3FF) << 10) | (trailSurrogate & 0x3FF);
    }
    HEAP32[((outPtr)>>2)]=codeUnit;
    outPtr += 4;
    if (outPtr + 4 > endPtr) break;
  }
  // Null-terminate the pointer to the HEAP.
  HEAP32[((outPtr)>>2)]=0;
  return outPtr - startPtr;
}
Module["stringToUTF32"] = stringToUTF32;

// Returns the number of bytes the given Javascript string takes if encoded as a UTF16 byte array, EXCLUDING the null terminator byte.

function lengthBytesUTF32(str) {
  var len = 0;
  for (var i = 0; i < str.length; ++i) {
    // Gotcha: charCodeAt returns a 16-bit word that is a UTF-16 encoded code unit, not a Unicode code point of the character! We must decode the string to UTF-32 to the heap.
    // See http://unicode.org/faq/utf_bom.html#utf16-3
    var codeUnit = str.charCodeAt(i);
    if (codeUnit >= 0xD800 && codeUnit <= 0xDFFF) ++i; // possibly a lead surrogate, so skip over the tail surrogate.
    len += 4;
  }

  return len;
}
Module["lengthBytesUTF32"] = lengthBytesUTF32;

function demangle(func) {
  var hasLibcxxabi = !!Module['___cxa_demangle'];
  if (hasLibcxxabi) {
    try {
      var buf = _malloc(func.length);
      writeStringToMemory(func.substr(1), buf);
      var status = _malloc(4);
      var ret = Module['___cxa_demangle'](buf, 0, 0, status);
      if (getValue(status, 'i32') === 0 && ret) {
        return Pointer_stringify(ret);
      }
      // otherwise, libcxxabi failed, we can try ours which may return a partial result
    } catch(e) {
      // failure when using libcxxabi, we can try ours which may return a partial result
    } finally {
      if (buf) _free(buf);
      if (status) _free(status);
      if (ret) _free(ret);
    }
  }
  var i = 3;
  // params, etc.
  var basicTypes = {
    'v': 'void',
    'b': 'bool',
    'c': 'char',
    's': 'short',
    'i': 'int',
    'l': 'long',
    'f': 'float',
    'd': 'double',
    'w': 'wchar_t',
    'a': 'signed char',
    'h': 'unsigned char',
    't': 'unsigned short',
    'j': 'unsigned int',
    'm': 'unsigned long',
    'x': 'long long',
    'y': 'unsigned long long',
    'z': '...'
  };
  var subs = [];
  var first = true;
  function dump(x) {
    //return;
    if (x) Module.print(x);
    Module.print(func);
    var pre = '';
    for (var a = 0; a < i; a++) pre += ' ';
    Module.print (pre + '^');
  }
  function parseNested() {
    i++;
    if (func[i] === 'K') i++; // ignore const
    var parts = [];
    while (func[i] !== 'E') {
      if (func[i] === 'S') { // substitution
        i++;
        var next = func.indexOf('_', i);
        var num = func.substring(i, next) || 0;
        parts.push(subs[num] || '?');
        i = next+1;
        continue;
      }
      if (func[i] === 'C') { // constructor
        parts.push(parts[parts.length-1]);
        i += 2;
        continue;
      }
      var size = parseInt(func.substr(i));
      var pre = size.toString().length;
      if (!size || !pre) { i--; break; } // counter i++ below us
      var curr = func.substr(i + pre, size);
      parts.push(curr);
      subs.push(curr);
      i += pre + size;
    }
    i++; // skip E
    return parts;
  }
  function parse(rawList, limit, allowVoid) { // main parser
    limit = limit || Infinity;
    var ret = '', list = [];
    function flushList() {
      return '(' + list.join(', ') + ')';
    }
    var name;
    if (func[i] === 'N') {
      // namespaced N-E
      name = parseNested().join('::');
      limit--;
      if (limit === 0) return rawList ? [name] : name;
    } else {
      // not namespaced
      if (func[i] === 'K' || (first && func[i] === 'L')) i++; // ignore const and first 'L'
      var size = parseInt(func.substr(i));
      if (size) {
        var pre = size.toString().length;
        name = func.substr(i + pre, size);
        i += pre + size;
      }
    }
    first = false;
    if (func[i] === 'I') {
      i++;
      var iList = parse(true);
      var iRet = parse(true, 1, true);
      ret += iRet[0] + ' ' + name + '<' + iList.join(', ') + '>';
    } else {
      ret = name;
    }
    paramLoop: while (i < func.length && limit-- > 0) {
      //dump('paramLoop');
      var c = func[i++];
      if (c in basicTypes) {
        list.push(basicTypes[c]);
      } else {
        switch (c) {
          case 'P': list.push(parse(true, 1, true)[0] + '*'); break; // pointer
          case 'R': list.push(parse(true, 1, true)[0] + '&'); break; // reference
          case 'L': { // literal
            i++; // skip basic type
            var end = func.indexOf('E', i);
            var size = end - i;
            list.push(func.substr(i, size));
            i += size + 2; // size + 'EE'
            break;
          }
          case 'A': { // array
            var size = parseInt(func.substr(i));
            i += size.toString().length;
            if (func[i] !== '_') throw '?';
            i++; // skip _
            list.push(parse(true, 1, true)[0] + ' [' + size + ']');
            break;
          }
          case 'E': break paramLoop;
          default: ret += '?' + c; break paramLoop;
        }
      }
    }
    if (!allowVoid && list.length === 1 && list[0] === 'void') list = []; // avoid (void)
    if (rawList) {
      if (ret) {
        list.push(ret + '?');
      }
      return list;
    } else {
      return ret + flushList();
    }
  }
  var parsed = func;
  try {
    // Special-case the entry point, since its name differs from other name mangling.
    if (func == 'Object._main' || func == '_main') {
      return 'main()';
    }
    if (typeof func === 'number') func = Pointer_stringify(func);
    if (func[0] !== '_') return func;
    if (func[1] !== '_') return func; // C function
    if (func[2] !== 'Z') return func;
    switch (func[3]) {
      case 'n': return 'operator new()';
      case 'd': return 'operator delete()';
    }
    parsed = parse();
  } catch(e) {
    parsed += '?';
  }
  if (parsed.indexOf('?') >= 0 && !hasLibcxxabi) {
    Runtime.warnOnce('warning: a problem occurred in builtin C++ name demangling; build with  -s DEMANGLE_SUPPORT=1  to link in libcxxabi demangling');
  }
  return parsed;
}

function demangleAll(text) {
  return text.replace(/__Z[\w\d_]+/g, function(x) { var y = demangle(x); return x === y ? x : (x + ' [' + y + ']') });
}

function jsStackTrace() {
  var err = new Error();
  if (!err.stack) {
    // IE10+ special cases: It does have callstack info, but it is only populated if an Error object is thrown,
    // so try that as a special-case.
    try {
      throw new Error(0);
    } catch(e) {
      err = e;
    }
    if (!err.stack) {
      return '(no stack trace available)';
    }
  }
  return err.stack.toString();
}

function stackTrace() {
  return demangleAll(jsStackTrace());
}
Module["stackTrace"] = stackTrace;

// Memory management

var PAGE_SIZE = 4096;

function alignMemoryPage(x) {
  if (x % 4096 > 0) {
    x += (4096 - (x % 4096));
  }
  return x;
}

var HEAP;
var HEAP8, HEAPU8, HEAP16, HEAPU16, HEAP32, HEAPU32, HEAPF32, HEAPF64;

var STATIC_BASE = 0, STATICTOP = 0, staticSealed = false; // static area
var STACK_BASE = 0, STACKTOP = 0, STACK_MAX = 0; // stack area
var DYNAMIC_BASE = 0, DYNAMICTOP = 0; // dynamic area handled by sbrk


function abortOnCannotGrowMemory() {
  abort('Cannot enlarge memory arrays. Either (1) compile with  -s TOTAL_MEMORY=X  with X higher than the current value ' + TOTAL_MEMORY + ', (2) compile with  -s ALLOW_MEMORY_GROWTH=1  which adjusts the size at runtime but prevents some optimizations, (3) set Module.TOTAL_MEMORY to a higher value before the program runs, or if you want malloc to return NULL (0) instead of this abort, compile with  -s ABORTING_MALLOC=0 ');
}

function enlargeMemory() {
  abortOnCannotGrowMemory();
}


var TOTAL_STACK = Module['TOTAL_STACK'] || 5242880;
var TOTAL_MEMORY = Module['TOTAL_MEMORY'] || 16777216;

var totalMemory = 64*1024;
while (totalMemory < TOTAL_MEMORY || totalMemory < 2*TOTAL_STACK) {
  if (totalMemory < 16*1024*1024) {
    totalMemory *= 2;
  } else {
    totalMemory += 16*1024*1024
  }
}
if (totalMemory !== TOTAL_MEMORY) {
  TOTAL_MEMORY = totalMemory;
}

// Initialize the runtime's memory
// check for full engine support (use string 'subarray' to avoid closure compiler confusion)
assert(typeof Int32Array !== 'undefined' && typeof Float64Array !== 'undefined' && !!(new Int32Array(1)['subarray']) && !!(new Int32Array(1)['set']),
       'JS engine does not provide full typed array support');

var buffer;



buffer = new ArrayBuffer(TOTAL_MEMORY);
HEAP8 = new Int8Array(buffer);
HEAP16 = new Int16Array(buffer);
HEAP32 = new Int32Array(buffer);
HEAPU8 = new Uint8Array(buffer);
HEAPU16 = new Uint16Array(buffer);
HEAPU32 = new Uint32Array(buffer);
HEAPF32 = new Float32Array(buffer);
HEAPF64 = new Float64Array(buffer);


// Endianness check (note: assumes compiler arch was little-endian)
HEAP32[0] = 255;
assert(HEAPU8[0] === 255 && HEAPU8[3] === 0, 'Typed arrays 2 must be run on a little-endian system');

Module['HEAP'] = HEAP;
Module['buffer'] = buffer;
Module['HEAP8'] = HEAP8;
Module['HEAP16'] = HEAP16;
Module['HEAP32'] = HEAP32;
Module['HEAPU8'] = HEAPU8;
Module['HEAPU16'] = HEAPU16;
Module['HEAPU32'] = HEAPU32;
Module['HEAPF32'] = HEAPF32;
Module['HEAPF64'] = HEAPF64;

function callRuntimeCallbacks(callbacks) {
  while(callbacks.length > 0) {
    var callback = callbacks.shift();
    if (typeof callback == 'function') {
      callback();
      continue;
    }
    var func = callback.func;
    if (typeof func === 'number') {
      if (callback.arg === undefined) {
        Runtime.dynCall('v', func);
      } else {
        Runtime.dynCall('vi', func, [callback.arg]);
      }
    } else {
      func(callback.arg === undefined ? null : callback.arg);
    }
  }
}

var __ATPRERUN__  = []; // functions called before the runtime is initialized
var __ATINIT__    = []; // functions called during startup
var __ATMAIN__    = []; // functions called when main() is to be run
var __ATEXIT__    = []; // functions called during shutdown
var __ATPOSTRUN__ = []; // functions called after the runtime has exited

var runtimeInitialized = false;
var runtimeExited = false;


function preRun() {
  // compatibility - merge in anything from Module['preRun'] at this time
  if (Module['preRun']) {
    if (typeof Module['preRun'] == 'function') Module['preRun'] = [Module['preRun']];
    while (Module['preRun'].length) {
      addOnPreRun(Module['preRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPRERUN__);
}

function ensureInitRuntime() {
  if (runtimeInitialized) return;
  runtimeInitialized = true;
  callRuntimeCallbacks(__ATINIT__);
}

function preMain() {
  callRuntimeCallbacks(__ATMAIN__);
}

function exitRuntime() {
  callRuntimeCallbacks(__ATEXIT__);
  runtimeExited = true;
}

function postRun() {
  // compatibility - merge in anything from Module['postRun'] at this time
  if (Module['postRun']) {
    if (typeof Module['postRun'] == 'function') Module['postRun'] = [Module['postRun']];
    while (Module['postRun'].length) {
      addOnPostRun(Module['postRun'].shift());
    }
  }
  callRuntimeCallbacks(__ATPOSTRUN__);
}

function addOnPreRun(cb) {
  __ATPRERUN__.unshift(cb);
}
Module["addOnPreRun"] = addOnPreRun;

function addOnInit(cb) {
  __ATINIT__.unshift(cb);
}
Module["addOnInit"] = addOnInit;

function addOnPreMain(cb) {
  __ATMAIN__.unshift(cb);
}
Module["addOnPreMain"] = addOnPreMain;

function addOnExit(cb) {
  __ATEXIT__.unshift(cb);
}
Module["addOnExit"] = addOnExit;

function addOnPostRun(cb) {
  __ATPOSTRUN__.unshift(cb);
}
Module["addOnPostRun"] = addOnPostRun;

// Tools


function intArrayFromString(stringy, dontAddNull, length /* optional */) {
  var len = length > 0 ? length : lengthBytesUTF8(stringy)+1;
  var u8array = new Array(len);
  var numBytesWritten = stringToUTF8Array(stringy, u8array, 0, u8array.length);
  if (dontAddNull) u8array.length = numBytesWritten;
  return u8array;
}
Module["intArrayFromString"] = intArrayFromString;

function intArrayToString(array) {
  var ret = [];
  for (var i = 0; i < array.length; i++) {
    var chr = array[i];
    if (chr > 0xFF) {
      chr &= 0xFF;
    }
    ret.push(String.fromCharCode(chr));
  }
  return ret.join('');
}
Module["intArrayToString"] = intArrayToString;

function writeStringToMemory(string, buffer, dontAddNull) {
  var array = intArrayFromString(string, dontAddNull);
  var i = 0;
  while (i < array.length) {
    var chr = array[i];
    HEAP8[(((buffer)+(i))>>0)]=chr;
    i = i + 1;
  }
}
Module["writeStringToMemory"] = writeStringToMemory;

function writeArrayToMemory(array, buffer) {
  for (var i = 0; i < array.length; i++) {
    HEAP8[((buffer++)>>0)]=array[i];
  }
}
Module["writeArrayToMemory"] = writeArrayToMemory;

function writeAsciiToMemory(str, buffer, dontAddNull) {
  for (var i = 0; i < str.length; ++i) {
    HEAP8[((buffer++)>>0)]=str.charCodeAt(i);
  }
  // Null-terminate the pointer to the HEAP.
  if (!dontAddNull) HEAP8[((buffer)>>0)]=0;
}
Module["writeAsciiToMemory"] = writeAsciiToMemory;

function unSign(value, bits, ignore) {
  if (value >= 0) {
    return value;
  }
  return bits <= 32 ? 2*Math.abs(1 << (bits-1)) + value // Need some trickery, since if bits == 32, we are right at the limit of the bits JS uses in bitshifts
                    : Math.pow(2, bits)         + value;
}
function reSign(value, bits, ignore) {
  if (value <= 0) {
    return value;
  }
  var half = bits <= 32 ? Math.abs(1 << (bits-1)) // abs is needed if bits == 32
                        : Math.pow(2, bits-1);
  if (value >= half && (bits <= 32 || value > half)) { // for huge values, we can hit the precision limit and always get true here. so don't do that
                                                       // but, in general there is no perfect solution here. With 64-bit ints, we get rounding and errors
                                                       // TODO: In i64 mode 1, resign the two parts separately and safely
    value = -2*half + value; // Cannot bitshift half, as it may be at the limit of the bits JS uses in bitshifts
  }
  return value;
}


// check for imul support, and also for correctness ( https://bugs.webkit.org/show_bug.cgi?id=126345 )
if (!Math['imul'] || Math['imul'](0xffffffff, 5) !== -5) Math['imul'] = function imul(a, b) {
  var ah  = a >>> 16;
  var al = a & 0xffff;
  var bh  = b >>> 16;
  var bl = b & 0xffff;
  return (al*bl + ((ah*bl + al*bh) << 16))|0;
};
Math.imul = Math['imul'];


if (!Math['clz32']) Math['clz32'] = function(x) {
  x = x >>> 0;
  for (var i = 0; i < 32; i++) {
    if (x & (1 << (31 - i))) return i;
  }
  return 32;
};
Math.clz32 = Math['clz32']

var Math_abs = Math.abs;
var Math_cos = Math.cos;
var Math_sin = Math.sin;
var Math_tan = Math.tan;
var Math_acos = Math.acos;
var Math_asin = Math.asin;
var Math_atan = Math.atan;
var Math_atan2 = Math.atan2;
var Math_exp = Math.exp;
var Math_log = Math.log;
var Math_sqrt = Math.sqrt;
var Math_ceil = Math.ceil;
var Math_floor = Math.floor;
var Math_pow = Math.pow;
var Math_imul = Math.imul;
var Math_fround = Math.fround;
var Math_min = Math.min;
var Math_clz32 = Math.clz32;

// A counter of dependencies for calling run(). If we need to
// do asynchronous work before running, increment this and
// decrement it. Incrementing must happen in a place like
// PRE_RUN_ADDITIONS (used by emcc to add file preloading).
// Note that you can add dependencies in preRun, even though
// it happens right before run - run will be postponed until
// the dependencies are met.
var runDependencies = 0;
var runDependencyWatcher = null;
var dependenciesFulfilled = null; // overridden to take different actions when all run dependencies are fulfilled

function getUniqueRunDependency(id) {
  return id;
}

function addRunDependency(id) {
  runDependencies++;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
}
Module["addRunDependency"] = addRunDependency;

function removeRunDependency(id) {
  runDependencies--;
  if (Module['monitorRunDependencies']) {
    Module['monitorRunDependencies'](runDependencies);
  }
  if (runDependencies == 0) {
    if (runDependencyWatcher !== null) {
      clearInterval(runDependencyWatcher);
      runDependencyWatcher = null;
    }
    if (dependenciesFulfilled) {
      var callback = dependenciesFulfilled;
      dependenciesFulfilled = null;
      callback(); // can add another dependenciesFulfilled
    }
  }
}
Module["removeRunDependency"] = removeRunDependency;

Module["preloadedImages"] = {}; // maps url to image data
Module["preloadedAudios"] = {}; // maps url to audio data



var memoryInitializer = null;



// === Body ===

var ASM_CONSTS = [];




STATIC_BASE = 8;

STATICTOP = STATIC_BASE + 48688;
  /* global initializers */  __ATINIT__.push();
  

/* memory initializer */ allocate([1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,254,255,255,127,0,0,0,0,128,0,0,0,244,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,0,0,0,0,0,0,0,0,0,0,0,250,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,254,255,255,127,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,50,64,0,0,1,0,1,0,130,0,0,0,55,64,0,0,1,0,1,0,131,0,0,0,60,64,0,0,1,0,1,0,132,0,0,0,65,64,0,0,2,0,1,0,133,0,0,0,70,64,0,0,2,0,1,0,133,0,0,0,76,64,0,0,1,0,1,0,134,0,0,0,82,64,0,0,1,0,1,0,135,0,0,0,88,64,0,0,1,0,1,0,136,0,0,0,94,64,0,0,2,0,1,0,137,0,0,0,105,64,0,0,1,0,1,0,138,0,0,0,113,64,0,0,1,0,1,0,139,0,0,0,121,64,0,0,1,0,1,0,140,0,0,0,125,64,0,0,1,0,1,0,141,0,0,0,129,64,0,0,1,0,1,0,142,0,0,0,133,64,0,0,1,0,1,0,143,0,0,0,137,64,0,0,1,0,1,0,144,0,0,0,142,64,0,0,1,0,1,0,145,0,0,0,147,64,0,0,1,0,1,0,146,0,0,0,152,64,0,0,1,0,1,0,147,0,0,0,157,64,0,0,1,0,1,0,148,0,0,0,143,98,0,0,1,0,1,0,149,0,0,0,161,64,0,0,1,0,1,0,150,0,0,0,167,64,0,0,2,0,1,0,151,0,0,0,173,64,0,0,1,0,1,0,152,0,0,0,178,64,0,0,1,0,1,0,153,0,0,0,183,64,0,0,1,0,1,0,154,0,0,0,190,64,0,0,1,0,1,0,155,0,0,0,195,64,0,0,1,0,1,0,156,0,0,0,201,64,0,0,0,0,1,1,157,0,0,0,204,64,0,0,2,0,1,0,158,0,0,0,214,64,0,0,2,0,1,0,159,0,0,0,214,64,0,0,3,0,1,0,159,0,0,0,224,64,0,0,2,0,1,0,160,0,0,0,232,64,0,0,2,0,1,0,161,0,0,0,241,64,0,0,1,0,1,0,162,0,0,0,249,64,0,0,1,0,1,0,163,0,0,0,0,65,0,0,2,0,1,0,164,0,0,0,5,65,0,0,2,0,1,0,165,0,0,0,10,65,0,0,2,0,1,0,166,0,0,0,15,65,0,0,2,0,1,0,167,0,0,0,25,65,0,0,1,0,0,0,168,0,0,0,130,0,0,0,31,65,0,0,1,0,0,0,168,0,0,0,131,0,0,0,221,101,0,0,1,0,0,0,169,0,0,0,132,0,0,0,40,65,0,0,1,0,0,0,169,0,0,0,133,0,0,0,47,65,0,0,1,0,0,0,169,0,0,0,134,0,0,0,62,65,0,0,1,0,0,0,169,0,0,0,135,0,0,0,0,0,0,0,128,48,0,0,128,32,14,0,128,32,200,3,0,0,0,0,128,255,255,255,0,248,255,255,0,0,255,255,1,0,0,0,0,0,0,0,45,178,0,0,66,178,0,0,78,178,0,0,90,178,0,0,114,178,0,0,125,178,0,0,145,178,0,0,159,178,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,21,172,0,0,204,24,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,26,172,0,0,208,24,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,36,172,0,0,212,24,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,3,0,0,0,44,0,0,0,0,2,0,0,0,0,0,0,49,172,0,0,204,24,0,0,130,0,0,0,130,0,0,0,130,0,0,0,131,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,131,0,0,0,130,0,0,0,131,0,0,0,132,0,0,0,132,0,0,0,133,0,0,0,133,0,0,0,134,0,0,0,0,0,0,64,0,0,0,0,0,0,0,0,234,3,0,0,0,0,8,0,235,3,0,0,0,0,0,1,236,3,0,0,0,0,0,64,237,3,0,0,0,0,64,0,0,202,154,59,0,202,154,59,208,7,0,0,232,3,0,0,244,1,0,0,168,97,0,0,127,0,0,0,10,0,0,0,80,195,0,0,231,3,0,0,232,3,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,0,0,130,0,0,0,131,0,0,0,135,0,0,0,130,0,0,0,131,0,0,0,136,0,0,0,132,0,0,0,131,0,0,0,133,0,0,0,134,0,0,0,134,0,0,0,137,0,0,0,132,0,0,0,135,0,0,0,136,0,0,0,137,0,0,0,138,0,0,0,132,0,0,0,138,0,0,0,139,0,0,0,140,0,0,0,141,0,0,0,0,0,0,0,132,0,0,0,132,0,0,0,142,0,0,0,139,0,0,0,139,0,0,0,143,0,0,0,140,0,0,0,133,0,0,0,141,0,0,0,142,0,0,0,135,0,0,0,144,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,140,81,0,0,184,81,0,0,252,81,0,0,24,82,0,0,53,82,0,0,80,82,0,0,108,82,0,0,133,82,0,0,157,82,0,0,222,82,0,0,25,83,0,0,93,83,0,0,141,83,0,0,253,83,0,0,132,84,0,0,184,84,0,0,250,84,0,0,37,85,0,0,96,85,0,0,135,85,0,0,179,85,0,0,220,85,0,0,11,86,0,0,53,86,0,0,226,71,0,0,226,71,0,0,91,86,0,0,146,86,0,0,207,86,0,0,70,87,0,0,173,87,0,0,228,87,0,0,30,88,0,0,137,88,0,0,216,88,0,0,25,89,0,0,85,89,0,0,133,89,0,0,239,89,0,0,56,90,0,0,6,80,0,0,170,0,0,0,14,80,0,0,171,0,0,0,32,80,0,0,172,0,0,0,22,80,0,0,173,0,0,0,0,93,0,0,2,0,4,0,3,93,0,0,3,1,3,0,7,93,0,0,3,1,2,0,11,93,0,0,4,0,1,0,22,80,0,0,9,0,0,0,236,96,0,0,6,0,0,0,99,94,0,0,8,0,0,0,108,94,0,0,10,0,0,0,243,96,0,0,5,0,0,0,37,94,0,0,7,0,0,0,249,96,0,0,10,0,0,0,4,97,0,0,10,0,0,0,0,0,0,0,136,0,0,0,143,0,0,0,133,0,0,0,144,0,0,0,133,0,0,0,0,0,0,0,0,0,0,0,137,0,0,0,145,0,0,0,134,0,0,0,146,0,0,0,134,0,0,0,0,0,0,0,0,0,0,0,135,0,0,0,135,0,0,0,145,0,0,0,147,0,0,0,147,0,0,0,146,0,0,0,148,0,0,0,134,0,0,0,149,0,0,0,150,0,0,0,138,0,0,0,147,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,139,0,0,0,151,0,0,0,135,0,0,0,152,0,0,0,136,0,0,0,0,0,0,0,255,255,255,255,255,255,0,252,1,0,0,248,1,0,0,248,48,0,0,0,7,232,0,0,6,108,1,0,47,236,1,0,7,172,2,0,1,208,2,0,3,216,2,0,1,236,2,0,1,252,2,0,1,92,3,0,1,220,3,0,4,8,11,0,14,72,11,0,7,148,11,0,1,180,11,0,129,188,11,0,1,212,13,0,1,248,13,0,2,16,14,0,1,28,14,0,1,216,15,0,8,8,18,0,6,104,21,0,2,36,22,0,1,60,22,0,55,68,22,0,2,204,23,0,5,0,24,0,22,24,24,0,2,120,24,0,21,44,25,0,4,168,25,0,1,192,25,0,1,80,27,0,15,88,27,0,7,156,27,0,2,244,27,0,14,0,28,0,1,60,28,0,1,68,28,0,27,192,28,0,11,152,30,0,9,172,31,0,4,216,31,0,4,88,32,0,9,108,32,0,3,148,32,0,5,164,32,0,15,192,32,0,3,100,33,0,1,120,33,0,27,144,35,0,4,0,36,0,3,232,36,0,18,248,36,0,7,68,37,0,4,136,37,0,1,192,37,0,3,4,38,0,1,240,38,0,7,248,38,0,2,28,39,0,3,44,39,0,1,92,39,0,2,136,39,0,2,200,39,0,2,232,39,0,3,4,40,0,1,240,40,0,5,248,40,0,2,28,41,0,3,44,41,0,1,68,41,0,2,192,41,0,1,212,41,0,3,4,42,0,1,240,42,0,8,248,42,0,3,28,43,0,3,44,43,0,2,136,43,0,2,192,43,0,3,4,44,0,1,240,44,0,7,248,44,0,2,28,45,0,3,44,45,0,2,88,45,0,2,136,45,0,1,192,45,0,1,8,46,0,5,248,46,0,3,24,47,0,4,40,47,0,1,92,47,0,8,204,47,0,3,4,48,0,7,248,48,0,3,24,49,0,4,40,49,0,2,84,49,0,2,136,49,0,1,252,49,0,2,8,50,0,1,240,50,0,7,248,50,0,3,24,51,0,4,40,51,0,2,84,51,0,2,136,51,0,2,8,52,0,7,248,52,0,3,24,53,0,4,40,53,0,1,92,53,0,2,136,53,0,1,228,53,0,2,8,54,0,1,40,55,0,6,60,55,0,1,88,55,0,8,96,55,0,3,200,55,0,1,196,56,0,7,208,56,0,1,252,56,0,9,28,57,0,2,104,57,0,1,196,58,0,6,208,58,0,2,236,58,0,6,32,59,0,31,4,60,0,12,208,60,0,23,196,61,0,11,52,62,0,36,100,62,0,15,248,62,0,13,56,63,0,20,172,64,0,6,40,65,0,4,88,65,0,3,120,65,0,3,136,65,0,7,156,65,0,4,196,65,0,12,8,66,0,1,60,66,0,6,104,66,0,1,236,67,0,12,116,77,0,10,64,78,0,1,0,80,0,2,180,89,0,1,0,90,0,2,108,90,0,3,172,91,0,3,72,92,0,5,200,92,0,2,72,93,0,2,200,93,0,35,208,94,0,4,96,95,0,1,116,95,0,15,0,96,0,1,164,98,0,12,128,100,0,12,192,100,0,1,0,101,0,2,16,101,0,17,192,102,0,2,32,103,0,34,120,103,0,5,92,104,0,2,120,104,0,10,84,105,0,29,128,105,0,1,252,105,0,7,128,106,0,6,160,106,0,5,0,108,0,17,208,108,0,35,104,109,0,3,0,110,0,13,132,110,0,14,152,111,0,4,240,111,0,20,144,112,0,5,236,112,0,2,248,113,0,8,0,115,0,25,64,115,0,1,180,115,0,3,200,115,0,39,0,119,0,4,240,119,0,1,244,126,0,3,252,126,0,3,52,127,0,3,116,127,0,3,180,127,0,2,244,127,0,101,0,128,0,6,168,129,0,5,232,129,0,5,40,130,0,26,128,130,0,33,64,131,0,2,0,132,0,4,12,132,0,2,32,132,0,1,80,132,0,3,88,132,0,6,120,132,0,1,148,132,0,1,156,132,0,1,164,132,0,1,184,132,0,2,232,132,0,5,0,133,0,4,40,133,0,1,60,133,0,100,66,134,0,39,0,144,0,11,0,145,0,78,112,146,0,0,2,148,0,117,4,156,0,185,83,158,0,10,64,173,0,6,148,179,0,3,188,179,0,4,228,179,0,2,248,179,0,1,192,181,0,1,252,181,0,79,128,183,0,12,192,184,0,26,0,186,0,89,108,186,0,214,0,188,0,12,192,191,0,5,0,192,0,25,32,192,0,7,168,192,0,2,216,192,0,3,244,192,0,4,100,194,0,1,128,194,0,1,236,195,0,2,64,198,0,10,88,198,0,36,0,199,0,31,0,200,0,30,168,200,0,1,64,201,0,32,128,201,0,39,40,202,0,63,0,203,0,0,1,204,0,64,0,55,1,55,64,146,2,2,248,147,2,3,52,152,2,16,188,153,2,1,124,154,2,8,192,155,2,23,0,156,2,2,128,156,2,2,36,158,2,1,8,160,2,1,24,160,2,1,44,160,2,9,140,160,2,4,216,160,2,4,208,161,2,2,0,162,2,17,208,162,2,2,56,163,2,18,128,163,2,3,224,163,2,10,152,164,2,13,28,165,2,1,124,165,2,4,0,166,2,27,204,166,2,2,120,167,2,14,164,168,2,1,12,169,2,2,48,169,2,4,112,169,2,3,220,169,2,1,236,169,2,1,192,170,2,3,200,170,2,2,220,170,2,2,248,170,2,1,4,171,2,2,120,171,2,7,172,171,2,2,212,171,2,11,140,175,2,1,0,96,3,2,252,109,3,2,252,111,3,1,252,127,3,1,120,236,3,1,164,236,3,16,200,238,3,2,248,244,3,2,240,247,3,26,0,248,3,7,128,248,3,35,192,248,3,19,80,249,3,4,160,249,3,1,252,251,3,15,4,252,3,7,104,252,3,6,236,252,3,11,108,253,3,7,128,255,3,7,160,255,3,5,228,255,3,3,0,4,4,9,220,4,4,17,228,5,4,12,64,6,4,46,64,7,4,1,124,14,4,1,64,15,4,1,92,33,4,1,124,36,4,1,252,36,4,3,4,40,4,2,20,40,4,4,48,40,4,3,224,40,4,1,252,40,4,9,64,41,4,1,252,41,4,7,228,44,4,3,0,64,4,22,224,64,4,3,0,66,4,18,192,66,4,3,0,68,4,14,156,68,4,4,0,69,4,3,0,70,4,14,204,70,4,4,20,71,4,13,172,90,4,4,192,145,4,46,68,189,5,4,60,190,5,246,0,64,7,39,0,68,7,181,164,68,7,70,0,72,7,87,0,76,7,1,4,91,7,1,108,91,7,1,236,91,7,1,84,92,7,1,212,92,7,1,60,93,7,1,188,93,7,1,36,94,7,1,164,94,7,1,12,95,7,2,192,187,7,44,0,192,7,100,192,192,7,15,128,194,7,14,196,194,7,15,4,195,7,15,68,195,7,31,64,196,7,60,192,196,7,43,192,197,7,29,152,199,7,43,64,200,7,9,0,201,7,2,64,201,7,33,0,204,7,6,192,204,7,70,220,204,7,20,0,206,7,37,128,206,7,5,24,207,7,17,128,207,7,63,0,208,7,1,0,209,7,182,8,209,7,4,228,211,7,62,0,212,7,4,0,213,7,24,64,213,7,70,236,215,7,11,20,217,7,70,0,218,7,116,0,220,7,1,4,0,56,96,128,0,56,240,0,4,56,0,0,0,0,1,0,0,0,153,0,0,0,135,0,0,0,136,0,0,0,140,0,0,0,148,0,0,0,149,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,54,102,0,0,0,0,2,0,61,102,0,0,0,0,4,0,0,0,0,0,0,0,0,0,37,102,0,0,1,0,0,0,40,102,0,0,2,0,0,0,43,102,0,0,6,0,0,0,47,102,0,0,128,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,154,0,0,0,136,0,0,0,141,0,0,0,130,0,0,0,155,0,0,0,142,0,0,0,174,0,0,0,130,0,0,0,131,0,0,0,137,0,0,0,138,0,0,0,156,0,0,0,139,0,0,0,150,0,0,0,157,0,0,0,158,0,0,0,159,0,0,0,140,0,0,0,0,0,0,0,9,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,145,133,0,0,1,0,0,0,226,77,0,0,1,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,175,0,0,0,0,0,0,0,185,120,0,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,176,0,0,0,0,0,0,0,175,120,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,177,0,0,0,0,0,0,0,155,120,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,195,78,0,0,248,120,0,0,5,121,0,0,0,0,0,0,18,121,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,178,0,0,0,0,0,0,0,85,122,0,0,0,0,0,0,3,0,1,0,0,0,0,0,0,0,0,0,179,0,0,0,0,0,0,0,210,122,0,0,0,0,0,0,23,130,0,0,0,0,0,0,8,0,0,0,247,125,0,0,1,1,0,0,0,0,0,0,38,130,0,0,2,0,0,0,0,0,16,0,54,130,0,0,3,0,0,0,0,0,0,0,114,125,0,0,4,1,0,0,0,0,0,0,23,126,0,0,5,0,0,0,0,0,0,0,67,130,0,0,6,0,0,0,0,0,0,0,87,130,0,0,2,0,0,0,0,0,0,32,103,130,0,0,2,0,0,0,16,0,0,0,124,130,0,0,7,0,0,0,0,0,0,0,139,130,0,0,8,0,0,0,0,0,0,0,155,130,0,0,2,0,0,0,128,0,0,0,169,130,0,0,0,2,0,0,15,0,0,0,182,130,0,0,10,1,0,0,0,0,0,0,196,130,0,0,11,1,0,0,0,0,0,0,215,130,0,0,2,0,0,0,0,0,0,2,234,130,0,0,2,0,0,0,0,1,0,0,126,127,0,0,12,0,0,0,0,0,0,0,1,131,0,0,13,1,0,0,0,0,0,0,19,131,0,0,14,1,0,0,0,0,0,0,36,131,0,0,2,0,0,0,0,0,8,0,49,131,0,0,0,2,0,0,0,0,0,0,64,131,0,0,2,0,0,0,4,0,0,0,82,131,0,0,2,0,0,0,8,0,0,0,92,131,0,0,2,0,0,0,0,32,0,0,117,131,0,0,15,1,0,0,0,0,0,0,136,131,0,0,16,1,0,0,0,0,0,0,147,131,0,0,17,1,0,0,0,0,0,0,158,131,0,0,16,1,0,0,1,0,0,0,221,126,0,0,18,1,0,0,0,0,0,0,215,125,0,0,19,1,0,0,0,0,0,0,228,125,0,0,20,0,0,0,0,0,0,0,170,131,0,0,2,0,0,0,0,128,0,0,202,125,0,0,22,0,0,0,0,0,0,0,189,131,0,0,23,1,0,0,0,0,0,0,35,126,0,0,24,0,0,0,0,0,0,0,204,131,0,0,23,1,0,0,0,0,0,0,161,125,0,0,25,0,0,0,0,0,0,0,215,131,0,0,2,0,0,0,0,0,0,4,226,131,0,0,18,1,0,0,0,0,0,0,238,131,0,0,2,0,0,0,0,64,0,0,255,131,0,0,2,0,0,0,0,0,4,0,18,132,0,0,2,0,0,0,0,0,2,0,44,132,0,0,0,0,0,0,1,0,0,0,171,125,0,0,26,0,0,0,0,0,0,0,59,132,0,0,2,0,0,0,64,0,0,0,78,132,0,0,27,0,0,0,0,0,0,0,244,127,0,0,28,0,0,0,0,0,0,0,92,132,0,0,29,1,0,0,0,0,0,0,102,126,0,0,30,1,0,0,0,0,0,0,98,132,0,0,31,1,0,0,0,0,0,0,45,126,0,0,32,0,0,0,0,0,0,0,56,126,0,0,33,0,0,0,0,0,0,0,4,128,0,0,34,0,0,0,0,0,0,0,109,132,0,0,0,0,0,0,6,0,0,0,217,127,0,0,35,0,0,0,0,0,0,0,122,132,0,0,36,1,0,0,0,0,0,0,137,132,0,0,2,0,0,0,0,8,1,0,121,129,0,0,76,129,0,0,153,129,0,0,158,129,0,0,166,129,0,0,90,129,0,0,225,128,0,0,134,129,0,0,140,129,0,0,146,129,0,0,115,129,0,0,121,129,0,0,76,129,0,0,9,94,0,0,125,129,0,0,130,129,0,0,38,129,0,0,76,129,0,0,93,129,0,0,100,129,0,0,107,129,0,0,86,129,0,0,88,129,0,0,90,129,0,0,38,129,0,0,76,129,0,0,81,129,0,0,38,129,0,0,76,129,0,0,35,129,0,0,38,129,0,0,225,128,0,0,42,129,0,0,47,129,0,0,50,129,0,0,60,129,0,0,70,129,0,0,225,128,0,0,169,96,0,0,231,128,0,0,238,128,0,0,30,128,0,0,1,0,0,0,35,128,0,0,1,0,0,0,41,128,0,0,2,0,0,0,50,128,0,0,3,0,0,0,59,128,0,0,2,0,0,0,67,128,0,0,3,0,0,0,75,128,0,0,0,0,0,0,82,128,0,0,0,0,0,0,0,0,0,0,0,0,0,0,12,128,0,0,143,98,0,0,17,128,0,0,0,130,0,0,7,130,0,0,15,130,0,0,208,127,0,0,47,102,0,0,19,130,0,0,70,129,0,0,64,0,0,0,97,128,0,0,66,0,0,0,92,128,0,0,65,0,0,0,25,136,0,0,67,0,0,0,248,116,0,0,3,0,0,0,244,116,0,0,3,0,0,0,226,71,0,0,19,146,0,0,25,146,0,0,30,146,0,0,35,146,0,0,151,146,0,0,157,146,0,0,165,146,0,0,216,154,0,0,221,154,0,0,228,154,0,0,231,154,0,0,234,154,0,0,237,154,0,0,240,154,0,0,243,154,0,0,251,154,0,0,243,96,0,0,42,129,0,0,4,155,0,0,148,163,0,0,157,163,0,0,164,163,0,0,170,163,0,0,253,163,0,0,7,164,0,0,18,164,0,0,30,164,0,0,41,164,0,0,52,164,0,0,63,164,0,0,68,164,0,0,73,164,0,0,84,164,0,0,96,164,0,0,103,164,0,0,111,164,0,0,119,164,0,0,124,164,0,0,130,164,0,0,144,164,0,0,150,164,0,0,160,164,0,0,165,164,0,0,169,164,0,0,174,164,0,0,177,164,0,0,183,164,0,0,190,164,0,0,197,164,0,0,204,164,0,0,211,164,0,0,214,164,0,0,218,164,0,0,229,164,0,0,238,164,0,0,244,164,0,0,254,164,0,0,8,165,0,0,15,165,0,0,23,165,0,0,26,165,0,0,29,165,0,0,32,165,0,0,35,165,0,0,38,165,0,0,41,165,0,0,51,165,0,0,58,165,0,0,64,165,0,0,74,165,0,0,85,165,0,0,89,165,0,0,98,165,0,0,107,165,0,0,114,165,0,0,124,165,0,0,131,165,0,0,136,165,0,0,143,165,0,0,154,165,0,0,159,165,0,0,166,165,0,0,172,165,0,0,178,165,0,0,184,165,0,0,190,165,0,0,201,165,0,0,212,165,0,0,220,165,0,0,229,165,0,0,235,165,0,0,245,165,0,0,2,166,0,0,13,166,0,0,19,166,0,0,24,166,0,0,31,166,0,0,44,166,0,0,55,166,0,0,60,166,0,0,68,166,0,0,74,166,0,0,81,166,0,0,86,166,0,0,95,166,0,0,100,166,0,0,109,166,0,0,114,166,0,0,119,166,0,0,125,166,0,0,133,166,0,0,143,166,0,0,151,166,0,0,161,166,0,0,170,166,0,0,177,166,0,0,190,166,0,0,195,166,0,0,207,166,0,0,215,166,0,0,222,166,0,0,230,166,0,0,239,166,0,0,250,166,0,0,0,167,0,0,11,167,0,0,21,167,0,0,31,167,0,0,40,167,0,0,50,167,0,0,64,167,0,0,78,167,0,0,89,167,0,0,102,167,0,0,113,167,0,0,119,167,0,0,131,167,0,0,140,167,0,0,149,167,0,0,156,167,0,0,166,167,0,0,173,167,0,0,184,167,0,0,198,167,0,0,209,167,0,0,216,167,0,0,224,167,0,0,230,167,0,0,238,167,0,0,251,167,0,0,5,168,0,0,15,168,0,0,20,168,0,0,29,168,0,0,37,168,0,0,43,168,0,0,48,168,0,0,60,168,0,0,72,168,0,0,84,168,0,0,96,168,0,0,109,168,0,0,119,168,0,0,129,168,0,0,141,168,0,0,153,168,0,0,163,168,0,0,169,168,0,0,179,168,0,0,186,168,0,0,198,168,0,0,207,168,0,0,215,168,0,0,224,168,0,0,231,168,0,0,241,168,0,0,248,168,0,0,0,169,0,0,9,169,0,0,15,169,0,0,23,169,0,0,31,169,0,0,41,169,0,0,50,169,0,0,61,169,0,0,66,169,0,0,150,169,0,0,163,169,0,0,0,0,0,0,199,169,0,0,224,169,0,0,255,169,0,0,18,170,0,0,184,97,0,0,43,170,0,0,80,170,0,0,92,170,0,0,107,170,0,0,140,170,0,0,158,170,0,0,183,170,0,0,212,170,0,0,229,170,0,0,73,157,0,0,250,70,0,0,252,170,0,0,14,171,0,0,32,171,0,0,71,171,0,0,102,171,0,0,123,171,0,0,155,171,0,0,189,171,0,0,151,0,0,0,152,0,0,0,153,0,0,0,1,0,0,0,160,0,0,0,137,0,0,0,138,0,0,0,143,0,0,0,154,0,0,0,155,0,0,0,156,0,0,0,157,0,0,0,158,0,0,0,144,0,0,0,161,0,0,0,162,0,0,0,0,0,0,0,136,0,0,0,141,0,0,0,159,0,0,0,139,0,0,0,137,0,0,0,96,172,0,0,145,0,0,0,0,0,0,0,59,172,0,0,163,0,0,0,0,0,0,0,226,101,0,0,160,0,0,0,0,0,0,0,101,172,0,0,161,0,0,0,0,0,0,0,108,172,0,0,162,0,0,0,0,0,0,0,113,172,0,0,163,0,0,0,0,0,0,0,119,172,0,0,164,0,0,0,0,0,0,0,129,172,0,0,146,0,0,0,0,0,0,0,135,172,0,0,147,0,0,0,0,0,0,0,140,172,0,0,0,0,0,0,0,0,0,0,146,172,0,0,0,0,0,0,0,0,0,0,154,172,0,0,148,0,0,0,0,0,0,0,160,172,0,0,0,0,0,0,0,0,0,0,167,172,0,0,0,0,0,0,0,0,0,0,176,172,0,0,165,0,0,0,0,0,0,0,183,172,0,0,0,0,0,0,0,0,0,0,193,172,0,0,164,0,0,0,0,0,0,0,200,172,0,0,166,0,0,0,0,0,0,0,214,172,0,0,167,0,0,0,0,0,0,0,220,172,0,0,165,0,0,0,0,0,0,0,226,172,0,0,149,0,0,0,0,0,0,0,233,172,0,0,130,0,0,0,0,0,0,0,241,172,0,0,137,0,0,0,0,0,0,0,246,172,0,0,168,0,0,0,0,0,0,0,253,172,0,0,0,0,0,0,0,0,0,0,4,173,0,0,131,0,0,0,0,0,0,0,16,173,0,0,150,0,0,0,0,0,0,0,25,173,0,0,169,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,122,173,0,0,131,173,0,0,140,173,0,0,145,173,0,0,3,0,0,0,166,0,0,0,137,0,0,0,138,0,0,0,143,0,0,0,154,0,0,0,155,0,0,0,170,0,0,0,171,0,0,0,172,0,0,0,144,0,0,0,161,0,0,0,162,0,0,0,0,0,0,0,136,0,0,0,141,0,0,0,159,0,0,0,139,0,0,0,137,0,0,0,0,0,0,0,3,0,0,0,167,0,0,0,137,0,0,0,138,0,0,0,143,0,0,0,154,0,0,0,155,0,0,0,173,0,0,0,174,0,0,0,175,0,0,0,144,0,0,0,161,0,0,0,162,0,0,0,140,0,0,0,136,0,0,0,141,0,0,0,159,0,0,0,139,0,0,0,137,0,0,0,0,0,0,0,1,0,1,32,0,0,0,0,0,0,0,0,180,0,0,0,0,0,0,0,17,175,0,0,0,0,0,0,1,0,1,32,0,0,0,0,0,0,0,0,181,0,0,0,0,0,0,0,43,175,0,0,0,0,0,0,1,0,1,12,0,0,0,0,0,0,0,0,182,0,0,0,0,0,0,0,68,175,0,0,0,0,0,0,2,0,1,12,0,0,0,0,0,0,0,0,182,0,0,0,0,0,0,0,77,175,0,0,0,0,0,0,1,0,1,12,0,0,0,0,0,0,0,0,182,0,0,0,0,0,0,0,88,175,0,0,0,0,0,0,1,0,1,8,1,0,0,0,0,0,0,0,183,0,0,0,0,0,0,0,95,175,0,0,0,0,0,0,2,0,1,8,1,0,0,0,0,0,0,0,183,0,0,0,0,0,0,0,95,175,0,0,0,0,0,0,1,0,1,8,2,0,0,0,0,0,0,0,183,0,0,0,0,0,0,0,101,175,0,0,0,0,0,0,2,0,1,8,2,0,0,0,0,0,0,0,183,0,0,0,0,0,0,0,101,175,0,0,0,0,0,0,1,0,1,8,3,0,0,0,0,0,0,0,183,0,0,0,0,0,0,0,107,175,0,0,0,0,0,0,2,0,1,8,3,0,0,0,0,0,0,0,183,0,0,0,0,0,0,0,107,175,0,0,0,0,0,0,255,0,33,8,0,0,0,0,0,0,0,0,184,0,0,0,0,0,0,0,124,139,0,0,0,0,0,0,0,0,33,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,124,139,0,0,0,0,0,0,1,0,33,16,0,0,0,0,0,0,0,0,185,0,0,0,142,0,0,0,124,139,0,0,0,0,0,0,255,0,33,8,1,0,0,0,0,0,0,0,184,0,0,0,0,0,0,0,128,139,0,0,0,0,0,0,0,0,33,8,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,128,139,0,0,0,0,0,0,1,0,33,16,1,0,0,0,0,0,0,0,185,0,0,0,142,0,0,0,128,139,0,0,0,0,0,0,1,0,129,8,0,0,0,0,0,0,0,0,186,0,0,0,0,0,0,0,112,175,0,0,0,0,0,0,1,0,65,8,0,0,0,0,0,0,0,0,187,0,0,0,0,0,0,0,119,175,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,188,0,0,0,0,0,0,0,126,175,0,0,0,0,0,0,255,0,1,8,0,0,0,0,0,0,0,0,189,0,0,0,0,0,0,0,132,175,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,190,0,0,0,0,0,0,0,139,175,0,0,0,0,0,0,255,0,1,8,0,0,0,0,0,0,0,0,191,0,0,0,0,0,0,0,147,175,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,192,0,0,0,0,0,0,0,152,175,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,193,0,0,0,0,0,0,0,156,175,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,193,0,0,0,0,0,0,0,156,175,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,194,0,0,0,0,0,0,0,162,175,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,195,0,0,0,0,0,0,0,168,175,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,196,0,0,0,0,0,0,0,174,175,0,0,0,0,0,0,2,0,1,10,0,0,0,0,0,0,0,0,182,0,0,0,0,0,0,0,178,175,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,197,0,0,0,0,0,0,0,185,175,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,198,0,0,0,0,0,0,0,192,175,0,0,0,0,0,0,2,0,33,8,0,0,0,0,0,0,0,0,199,0,0,0,0,0,0,0,203,175,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,182,0,0,0,0,0,0,0,210,175,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,200,0,0,0,0,0,0,0,225,175,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,201,0,0,0,0,0,0,0,242,175,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,202,0,0,0,0,0,0,0,253,175,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,203,0,0,0,0,0,0,0,3,176,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,204,0,0,0,0,0,0,0,21,176,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,205,0,0,0,0,0,0,0,29,176,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,206,0,0,0,0,0,0,0,43,176,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,207,0,0,0,0,0,0,0,51,176,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,208,0,0,0,0,0,0,0,60,176,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,208,0,0,0,0,0,0,0,60,176,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,209,0,0,0,143,0,0,0,67,176,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,209,0,0,0,144,0,0,0,71,176,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,209,0,0,0,145,0,0,0,77,176,0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,210,0,0,0,146,0,0,0,81,176,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,210,0,0,0,146,0,0,0,81,176,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0,211,0,0,0,147,0,0,0,87,176,0,0,0,0,0,0,2,0,1,0,0,0,0,0,0,0,0,0,211,0,0,0,147,0,0,0,87,176,0,0,0,0,0,0,2,0,13,8,218,71,0,0,0,0,0,0,212,0,0,0,0,0,0,0,97,128,0,0,0,0,0,0,2,0,5,8,222,71,0,0,0,0,0,0,212,0,0,0,0,0,0,0,92,128,0,0,0,0,0,0,3,0,5,8,222,71,0,0,0,0,0,0,212,0,0,0,0,0,0,0,92,128,0,0,0,0,0,0,1,0,1,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,176,0,0,0,0,0,0,0,0,1,8,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,100,176,0,0,0,0,0,0,255,0,1,10,0,0,0,0,0,0,0,0,182,0,0,0,0,0,0,0,100,176,0,0,0,0,0,0,101,138,0,0,255,0,1,32,0,0,0,0,0,0,0,0,213,0,0,0,0,0,0,0,165,176,0,0,0,0,0,0,255,0,1,32,0,0,0,0,0,0,0,0,214,0,0,0,0,0,0,0,175,176,0,0,0,0,0,0,255,0,1,32,0,0,0,0,0,0,0,0,215,0,0,0,0,0,0,0,180,176,0,0,0,0,0,0,255,0,1,32,0,0,0,0,0,0,0,0,216,0,0,0,0,0,0,0,185,176,0,0,0,0,0,0,255,0,1,32,0,0,0,0,0,0,0,0,217,0,0,0,0,0,0,0,194,176,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,218,0,0,0,0,0,0,0,203,176,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,219,0,0,0,0,0,0,0,216,176,0,0,0,0,0,0,0,0,1,32,0,0,0,0,0,0,0,0,220,0,0,0,0,0,0,0,234,176,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,221,0,0,0,0,0,0,0,209,177,0,0,0,0,0,0,2,0,1,8,0,0,0,0,0,0,0,0,222,0,0,0,0,0,0,0,229,177,0,0,0,0,0,0,3,0,1,8,0,0,0,0,0,0,0,0,223,0,0,0,0,0,0,0,251,177,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,212,34,0,0,0,0,0,0,5,0,0,0,0,0,0,0,0,0,0,0,168,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,151,0,0,0,152,0,0,0,39,186,0,0,0,4,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,10,255,255,255,255,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,2,0,2,0,8,0,9,0,14,0,16,0,20,0,23,0,25,0,25,0,29,0,33,0,36,0,41,0,46,0,48,0,53,0,54,0,59,0,62,0,65,0,67,0,69,0,78,0,81,0,86,0,91,0,95,0,96,0,101,0,105,0,109,0,117,0,122,0,128,0,136,0,142,0,152,0,159,0,162,0,162,0,165,0,167,0,167,0,171,0,176,0,179,0,184,0,184,0,188,0,192,0,199,0,204,0,209,0,212,0,218,0,221,0,225,0,234,0,240,0,240,0,240,0,243,0,246,0,250,0,251,0,255,0,5,1,9,1,16,1,22,1,34,1,40,1,49,1,51,1,57,1,62,1,64,1,71,1,76,1,81,1,87,1,93,1,98,1,102,1,105,1,111,1,115,1,122,1,124,1,131,1,133,1,135,1,144,1,148,1,154,1,160,1,168,1,173,1,173,1,189,1,196,1,203,1,204,1,211,1,215,1,219,1,223,1,227,1,230,1,232,1,234,1,240,1,244,1,252,1,1,2,9,2,12,2,17,2,22,2,28,2,32,2,37,2,33,0,32,0,30,0,28,0,26,0,0,0,2,0,3,0,5,0,6,0,7,0,8,0,9,0,65,0,14,26,181,0,64,1,192,0,14,23,216,0,14,7,0,1,1,48,50,1,1,6,57,1,1,16,74,1,1,46,120,1,116,1,121,1,1,6,127,1,104,1,129,1,50,1,130,1,1,4,134,1,44,1,135,1,0,1,137,1,42,2,139,1,0,1,142,1,32,1,143,1,38,1,144,1,40,1,145,1,0,1,147,1,42,1,148,1,46,1,150,1,52,1,151,1,48,1,152,1,0,1,156,1,52,1,157,1,54,1,159,1,56,1,160,1,1,6,166,1,60,1,167,1,0,1,169,1,60,1,172,1,0,1,174,1,60,1,175,1,0,1,177,1,58,2,179,1,1,4,183,1,62,1,184,1,0,1,188,1,0,1,196,1,2,1,197,1,0,1,199,1,2,1,200,1,0,1,202,1,2,1,203,1,1,18,222,1,1,18,241,1,2,1,242,1,1,4,246,1,122,1,247,1,134,1,248,1,1,40,32,2,110,1,34,2,1,18,58,2,70,1,59,2,0,1,61,2,108,1,62,2,68,1,65,2,0,1,67,2,106,1,68,2,28,1,69,2,30,1,70,2,1,10,69,3,36,1,112,3,1,4,118,3,0,1,134,3,18,1,136,3,16,3,140,3,26,1,142,3,24,2,145,3,14,17,163,3,14,9,194,3,0,1,207,3,4,1,208,3,140,1,209,3,142,1,213,3,146,1,214,3,144,1,216,3,1,24,240,3,136,1,241,3,138,1,244,3,130,1,245,3,128,1,247,3,0,1,249,3,152,1,250,3,0,1,253,3,110,3,0,4,34,16,16,4,14,32,96,4,1,34,138,4,1,54,192,4,6,1,193,4,1,14,208,4,1,88,49,5,22,38,160,16,66,38,199,16,66,1,205,16,66,1,0,30,1,150,155,30,132,1,158,30,96,1,160,30,1,96,8,31,150,8,24,31,150,6,40,31,150,8,56,31,150,8,72,31,150,6,89,31,151,8,104,31,150,8,136,31,150,8,152,31,150,8,168,31], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE);
/* memory initializer */ allocate([150,8,184,31,150,2,186,31,126,2,188,31,148,1,190,31,100,1,200,31,124,4,204,31,148,1,216,31,150,2,218,31,120,2,232,31,150,2,234,31,118,2,236,31,152,1,248,31,112,2,250,31,114,2,252,31,148,1,38,33,98,1,42,33,92,1,43,33,94,1,50,33,12,1,96,33,8,16,131,33,0,1,182,36,10,26,0,44,22,47,96,44,0,1,98,44,88,1,99,44,102,1,100,44,90,1,103,44,1,6,109,44,84,1,110,44,86,1,111,44,80,1,112,44,82,1,114,44,0,1,117,44,0,1,126,44,78,2,128,44,1,100,235,44,1,4,242,44,0,1,64,166,1,46,128,166,1,24,34,167,1,14,50,167,1,62,121,167,1,4,125,167,76,1,126,167,1,10,139,167,0,1,141,167,74,1,144,167,1,4,160,167,1,10,170,167,72,1,33,255,14,26,1,0,2,0,8,0,15,0,16,0,26,0,28,0,32,0,37,0,38,0,40,0,48,0,63,0,64,0,69,0,71,0,79,0,80,0,116,0,202,0,203,0,205,0,206,0,207,0,209,0,210,0,211,0,213,0,214,0,217,0,218,0,219,0,7,3,96,28,40,42,43,42,188,90,216,90,252,117,193,213,225,213,226,213,228,213,3,214,9,214,25,214,65,223,186,223,65,226,163,226,251,227,26,241,244,254,61,255,93,255,126,255,128,255,130,255,135,255,144,255,156,255,159,255,170,255,182,255,192,255,196,255,198,255,200,255,202,255,208,255,226,255,231,255,234,255,241,255,247,255,248,255,249,255,0,0,5,7,56,7,67,7,99,7,136,7,148,7,203,7,232,7,248,7,12,8,62,8,120,8,158,8,216,8,238,8,40,9,79,9,168,9,184,9,212,9,36,10,108,10,172,10,222,10,26,11,78,11,138,11,168,11,184,11,212,11,8,13,128,13,112,14,128,14,144,14,160,14,56,15,72,15,88,15,128,15,168,15,200,15,10,16,42,16,74,16,106,16,138,16,170,16,200,16,216,16,248,16,56,17,72,17,120,17,152,17,4,24,54,24,120,24,136,24,216,24,29,25,105,25,129,25,8,240,28,240,94,240,152,240,202,240,248,240,8,241,30,241,88,241,104,241,140,241,184,241,218,241,252,241,46,242,170,242,202,242,248,242,10,243,94,243,156,243,234,243,14,244,72,244,90,244,120,244,140,244,176,244,184,244,192,244,200,244,10,245,204,245,74,246,106,246,42,247,158,247,227,255,54,2,13,2,93,2,207,255,51,1,235,1,21,2,156,2,179,1,89,2,132,2,148,0,235,2,18,3,27,3,163,1,20,3,59,3,22,3,198,1,64,3,121,3,239,1,56,3,222,2,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,15,3,130,3,137,3,139,3,143,3,153,3,165,3,168,3,172,3,175,3,179,3,182,3,184,3,187,3,190,3,194,3,197,3,201,3,206,3,209,3,212,3,216,3,220,3,223,3,225,3,228,3,231,3,234,3,238,3,242,3,250,3,253,3,0,4,4,4,8,4,10,4,12,4,16,4,22,4,27,4,34,4,38,4,40,4,44,4,46,4,49,4,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,76,0,87,3,36,0,11,2,235,0,160,1,9,3,76,0,22,1,76,0,76,0,76,0,76,0,188,2,188,2,188,2,150,0,220,0,147,0,217,0,221,0,50,1,50,1,99,2,5,0,23,2,44,2,108,2,208,2,104,3,129,3,116,0,96,3,93,1,11,4,13,4,148,1,23,4,224,3,127,255,26,4,236,1,62,0,210,2,111,3,48,4,65,4,40,3,42,4,70,4,71,4,72,4,73,4,74,4,8,3,30,4,45,2,57,0,112,0,131,0,167,0,182,0,250,0,16,1,35,1,75,1,108,1,182,1,241,1,5,2,79,2,141,2,178,2,227,2,7,3,30,3,124,3,140,3,156,3,162,3,247,3,39,4,45,4,99,1,16,3,31,3,213,3,77,4,158,3,127,4,137,4,138,4,177,3,140,4,142,4,104,4,144,4,147,4,148,4,250,0,149,4,150,4,151,4,154,4,156,4,157,4,64,4,78,4,95,4,100,4,102,4,158,3,107,4,115,4,164,4,116,4,105,4,106,4,79,4,120,4,83,4,155,4,132,4,143,4,158,4,110,4,98,4,159,4,160,4,126,4,129,4,173,4,87,4,178,4,179,4,99,4,101,4,181,4,123,4,161,4,145,4,162,4,166,4,167,4,168,4,189,4,193,4,169,4,133,4,172,4,174,4,170,4,196,4,194,4,121,4,130,4,205,4,207,4,209,4,192,4,213,4,216,4,217,4,220,4,198,4,203,4,206,4,208,4,199,4,211,4,212,4,221,4,225,4,202,4,226,4,230,4,175,4,177,4,180,4,183,4,185,4,187,4,190,4,188,4,231,4,184,4,235,4,191,4,232,4,176,4,182,4,236,4,223,4,237,4,239,4,238,4,242,4,254,4,2,5,12,5,14,5,17,5,18,5,19,5,20,5,197,4,200,4,204,4,8,5,11,5,252,4,253,4,15,5,69,1,64,3,95,1,57,3,5,0,203,0,203,0,51,3,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,45,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,209,3,209,3,56,3,56,3,58,3,179,3,98,1,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,82,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,95,0,95,0,94,0,94,0,94,0,93,0,95,1,23,3,209,3,209,3,69,1,94,0,94,0,94,0,93,0,95,1,24,3,75,0,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,194,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,53,5,155,0,155,0,2,0,69,1,19,1,146,0,132,0,52,0,52,0,93,0,95,1,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,101,0,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,190,3,190,3,69,1,12,1,172,1,157,1,155,1,61,0,240,2,240,2,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,60,0,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,14,1,73,1,17,1,21,1,191,3,192,3,250,0,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,45,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,170,3,46,5,186,2,194,2,46,5,242,0,156,1,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,91,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,170,3,47,5,128,1,187,2,47,5,125,1,123,1,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,189,2,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,92,0,89,0,178,0,65,3,168,3,117,1,188,2,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,119,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,252,4,179,3,98,1,50,3,168,3,227,2,227,2,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,230,0,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,201,3,227,0,92,0,89,0,178,0,117,1,44,1,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,153,3,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,193,1,191,1,191,1,191,1,147,0,225,2,225,2,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,40,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,163,1,231,0,190,3,190,3,158,0,25,0,166,1,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,194,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,187,1,224,0,224,0,164,1,190,3,190,3,194,3,69,1,52,0,52,0,191,3,192,3,176,0,159,1,78,0,99,0,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,123,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,172,1,162,1,42,1,191,3,192,3,194,3,81,0,99,0,88,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,205,2,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,75,3,75,3,87,3,90,3,228,3,62,1,87,1,123,1,100,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,194,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,69,1,94,1,94,1,94,1,4,1,121,1,84,1,161,3,52,0,52,0,90,0,74,3,74,3,86,3,89,3,78,3,78,3,97,0,97,0,98,0,98,0,98,0,98,0,105,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,86,0,189,1,79,3,3,0,179,4,105,1,104,1,122,1,88,1,45,3,190,3,190,3,20,5,86,0,189,1,217,2,3,0,212,0,169,0,31,1,149,1,26,1,148,1,199,0,232,0,194,1,44,1,248,2,83,0,84,0,24,1,245,0,6,1,109,1,251,0,85,0,96,1,96,1,92,0,89,0,178,0,83,0,84,0,242,0,156,1,52,0,52,0,192,1,85,0,96,1,96,1,246,0,191,3,192,3,194,0,199,1,158,2,146,1,143,1,142,1,192,1,243,0,221,0,114,0,178,1,8,3,105,1,194,1,141,1,12,1,235,2,224,0,224,0,132,0,132,0,198,0,64,3,178,1,196,1,195,1,172,1,171,1,51,3,159,1,222,2,201,2,132,0,52,0,52,0,64,3,12,1,196,1,195,1,222,2,194,0,51,3,107,1,146,1,143,1,142,1,194,1,247,4,247,4,23,0,190,3,190,3,86,0,189,1,141,1,3,0,228,0,173,1,127,3,56,3,56,3,58,3,59,3,19,0,203,0,208,2,52,0,52,0,172,1,152,1,183,1,249,0,56,3,56,3,58,3,59,3,19,0,229,0,147,1,153,0,83,0,84,0,249,2,177,0,241,0,194,1,209,2,85,0,96,1,96,1,120,0,157,0,191,3,192,3,58,0,209,3,153,1,99,1,74,1,192,1,12,1,172,1,174,1,64,1,22,3,32,0,32,0,86,0,189,1,8,3,3,0,85,1,98,0,98,0,98,0,98,0,178,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,64,3,120,0,196,1,195,1,45,3,119,3,51,3,83,0,84,0,209,3,45,3,132,0,154,1,152,3,85,0,96,1,96,1,132,0,151,1,21,3,190,3,190,3,92,0,89,0,178,0,149,3,192,1,6,1,114,1,5,1,82,0,146,3,80,0,6,1,114,1,5,1,8,3,56,3,56,3,58,3,59,3,19,0,166,3,178,1,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,64,3,74,0,196,1,195,1,190,3,190,3,51,3,191,3,192,3,120,0,92,0,89,0,178,0,177,3,2,0,150,3,197,3,12,1,1,0,208,3,76,0,189,1,250,2,3,0,196,2,133,3,133,3,131,1,190,3,190,3,245,2,151,3,115,1,228,2,10,3,244,2,1,1,56,3,56,3,58,3,59,3,19,0,161,1,229,2,194,1,24,0,191,3,192,3,83,0,84,0,113,1,190,3,190,3,177,0,226,0,85,0,96,1,96,1,117,3,59,1,58,1,57,1,215,0,55,1,10,0,10,0,171,2,192,1,93,1,92,1,191,3,192,3,141,3,9,3,157,0,120,0,190,3,190,3,81,1,8,3,160,1,199,2,54,1,194,1,178,1,194,1,65,1,194,1,23,3,103,0,200,0,175,0,194,1,191,3,192,3,140,3,64,3,24,3,196,1,195,1,9,0,9,0,51,3,10,0,10,0,52,0,52,0,51,0,51,0,180,0,204,2,248,0,10,0,10,0,171,0,170,0,167,0,83,1,191,3,192,3,247,0,216,3,190,2,190,2,194,1,203,2,233,0,174,2,214,3,121,3,215,3,182,0,146,3,56,3,56,3,58,3,59,3,19,0,183,0,0,1,167,1,132,0,181,0,138,1,10,0,10,0,121,3,123,3,237,2,190,3,190,3,149,3,12,1,217,3,198,0,217,3,93,1,92,1,169,1,159,1,43,1,49,3,64,3,70,1,57,3,120,0,76,1,133,0,51,3,12,1,98,0,98,0,98,0,98,0,91,0,96,0,96,0,96,0,96,0,95,0,95,0,94,0,94,0,94,0,93,0,95,1,157,0,42,3,115,1,126,1,103,1,191,3,192,3,102,1,12,1,194,1,150,3,112,1,68,1,56,3,56,3,58,3,194,1,197,2,194,1,8,1,124,1,121,3,194,1,109,3,234,2,253,0,151,3,255,0,177,1,36,0,36,0,234,0,194,1,234,0,120,0,13,1,37,0,37,0,12,0,12,0,78,1,16,1,27,0,27,0,194,1,74,1,118,0,194,1,162,0,230,2,24,1,194,1,38,0,38,0,194,1,217,3,100,1,217,3,194,1,197,2,186,4,194,1,132,0,194,1,39,0,39,0,194,1,40,0,40,0,194,1,106,1,41,0,41,0,194,1,42,0,42,0,194,1,254,0,28,0,28,0,194,1,29,0,29,0,31,0,31,0,194,1,43,0,43,0,194,1,44,0,44,0,194,1,202,2,45,0,45,0,194,1,11,0,11,0,255,2,194,1,46,0,46,0,194,1,12,1,194,1,105,0,105,0,194,1,47,0,47,0,194,1,48,0,48,0,194,1,237,0,33,0,33,0,194,1,172,0,49,0,49,0,194,1,50,0,50,0,34,0,34,0,18,1,122,0,122,0,194,1,123,0,123,0,194,1,124,0,124,0,194,1,130,3,56,0,56,0,194,1,129,3,35,0,35,0,194,1,11,1,194,1,49,3,194,1,49,3,106,0,106,0,194,1,53,0,53,0,129,1,107,0,107,0,194,1,49,3,108,0,108,0,49,3,194,1,104,0,104,0,121,0,121,0,119,0,119,0,194,1,117,0,112,0,112,0,194,1,20,1,194,1,225,0,111,0,111,0,194,1,218,2,194,1,109,0,109,0,194,1,161,2,162,2,163,2,144,3,110,0,110,0,61,1,230,3,55,0,55,0,57,0,57,0,180,2,75,1,54,0,54,0,26,0,26,0,184,2,30,0,30,0,61,1,169,3,197,0,196,0,195,0,79,1,25,1,80,1,190,1,75,1,233,2,177,2,180,1,184,1,188,1,120,0,72,0,130,1,223,0,175,0,89,1,245,2,165,3,20,0,30,1,63,1,244,2,47,3,116,1,118,1,202,0,202,0,202,0,7,1,139,1,29,1,74,0,208,0,21,0,184,2,207,2,206,2,116,3,120,0,120,0,120,0,120,0,120,0,242,2,22,1,60,3,77,0,74,0,214,2,215,2,17,3,15,3,112,3,202,0,231,3,208,0,126,3,125,3,126,3,125,3,182,2,48,3,251,2,116,0,6,3,10,5,175,1,176,1,46,1,231,3,134,1,47,1,55,3,185,2,179,2,168,2,159,0,33,1,167,2,116,3,169,2,184,3,35,1,218,0,37,1,7,0,60,1,60,3,173,0,37,3,3,1,108,1,252,0,143,3,120,1,201,2,39,1,179,1,52,1,168,0,187,3,225,3,135,0,144,1,222,3,28,1,114,3,113,3,205,0,160,3,158,3,59,0,77,1,62,0,144,0,156,0,130,0,72,0,34,3,110,1,111,1,137,1,137,0,185,0,189,0,160,0,139,0,127,1,67,0,128,3,140,0,141,0,142,0,148,0,133,1,44,3,7,3,10,1,219,0,190,0,154,0,135,1,145,3,108,3,15,1,150,1,191,0,66,1,170,2,221,2,192,0,86,1,220,2,212,2,219,2,199,2,211,2,165,1,193,2,71,0,67,1,6,0,204,0,3,3,32,1,79,0,41,1,90,1,4,3,192,2,34,1,27,1,191,2,2,3,36,1,38,1,199,3,239,0,1,3,102,0,94,3,182,1,170,1,240,0,168,1,186,1,73,0,213,0,176,2,238,0,22,0,197,1,185,3,214,0,217,0,216,0,198,1,165,2,164,2,159,2,241,2,125,0,115,0,235,0,126,0,157,2,97,1,166,0,127,0,244,0,179,0,101,1,50,1,48,1,49,1,51,1,113,0,124,3,71,1,122,3,43,3,72,1,134,0,128,0,136,0,138,0,231,2,2,1,139,3,184,0,143,0,129,0,142,3,186,0,63,0,64,0,145,0,187,0,138,3,65,0,8,0,66,0,13,0,188,0,202,0,131,3,9,1,149,0,219,3,132,1,150,0,173,2,161,0,136,1,29,1,193,0,23,1,140,1,151,0,145,1,68,0,14,0,15,0,210,2,69,0,236,0,63,3,131,0,62,3,92,3,70,0,239,2,16,0,158,1,243,2,4,0,174,0,220,0,222,0,16,3,201,0,152,0,11,3,77,0,74,0,17,0,18,0,107,3,93,3,91,3,148,3,96,3,147,3,207,0,206,0,174,3,163,0,181,1,180,3,175,3,164,0,209,0,234,3,185,1,95,3,165,0,210,0,61,3,183,2,87,0,56,1,211,0,12,5,11,5,53,1,16,16,2,16,40,0,87,2,136,3,100,2,248,2,248,2,248,2,248,2,213,2,237,255,16,0,16,0,100,0,248,2,248,2,248,2,248,2,248,2,248,2,248,2,108,3,108,3,61,2,30,2,207,2,88,2,61,0,137,0,172,0,207,0,242,0,21,1,56,1,91,1,126,1,161,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,203,1,238,1,203,1,17,2,52,2,52,2,193,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,88,3,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,248,2,219,3,234,2,234,2,234,2,234,2,234,2,33,3,23,0,32,0,181,3,193,3,211,3,196,3,196,3,181,3,73,0,113,0,205,255,31,6,31,6,31,6,24,2,24,2,24,2,99,0,99,0,45,3,45,3,155,2,205,0,240,0,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,76,1,243,3,166,1,166,1,113,0,30,0,30,0,30,0,30,0,30,0,30,0,31,6,31,6,31,6,154,3,162,255,162,255,128,1,101,2,60,3,164,1,253,2,36,3,83,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,160,2,160,2,160,2,181,3,181,3,145,2,181,3,181,3,181,3,238,255,181,3,181,3,226,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,181,3,4,3,94,4,200,2,200,2,200,2,42,3,45,0,1,3,195,4,109,4,162,1,162,1,57,2,109,4,57,2,62,3,95,2,151,2,114,3,162,1,181,2,114,3,114,3,80,3,128,4,41,4,6,5,214,4,214,4,7,5,7,5,214,4,64,5,61,5,215,4,73,5,73,5,73,5,73,5,214,4,75,5,215,4,64,5,61,5,61,5,215,4,214,4,75,5,219,4,32,5,214,4,214,4,75,5,90,5,214,4,75,5,214,4,75,5,90,5,10,5,10,5,10,5,47,5,90,5,10,5,21,5,10,5,47,5,10,5,10,5,4,5,24,5,4,5,24,5,4,5,24,5,4,5,24,5,214,4,111,5,214,4,0,5,90,5,86,5,86,5,90,5,22,5,28,5,30,5,29,5,215,4,134,5,136,5,151,5,151,5,160,5,160,5,160,5,160,5,31,6,31,6,31,6,31,6,31,6,31,6,31,6,31,6,7,2,210,3,186,4,201,4,104,0,117,4,165,4,222,4,224,4,227,4,228,4,229,4,233,4,234,4,249,4,235,3,163,4,13,5,146,4,248,4,255,4,210,4,1,5,152,4,153,4,9,5,218,4,171,4,173,5,175,5,157,5,39,5,167,5,89,5,172,5,166,5,168,5,72,5,65,5,84,5,74,5,178,5,76,5,183,5,199,5,79,5,77,5,169,5,170,5,174,5,176,5,92,5,148,5,141,5,87,5,209,5,207,5,192,5,108,5,78,5,137,5,190,5,139,5,133,5,149,5,115,5,200,5,203,5,206,5,114,5,122,5,208,5,150,5,210,5,211,5,205,5,212,5,152,5,177,5,214,5,158,5,171,5,215,5,217,5,218,5,216,5,127,5,222,5,223,5,225,5,219,5,126,5,226,5,227,5,195,5,188,5,231,5,130,5,229,5,193,5,230,5,194,5,236,5,229,5,237,5,238,5,239,5,240,5,241,5,243,5,252,5,244,5,246,5,245,5,247,5,248,5,250,5,251,5,247,5,253,5,255,5,0,6,1,6,3,6,156,5,161,5,162,5,163,5,7,6,11,6,13,6,1,5,247,4,247,4,247,4,179,4,179,4,179,4,179,4,247,4,72,4,101,4,101,4,231,4,52,5,52,5,52,5,52,5,52,5,52,5,178,4,52,5,52,5,52,5,52,5,247,4,76,4,107,4,52,5,52,5,52,5,52,5,180,4,181,4,52,5,52,5,52,5,230,4,232,4,117,4,116,4,115,4,114,4,213,4,88,4,112,4,105,4,109,4,180,4,174,4,175,4,173,4,177,4,181,4,52,5,108,4,143,4,158,4,142,4,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,152,4,157,4,164,4,156,4,153,4,145,4,144,4,146,4,147,4,52,5,251,3,43,4,52,5,52,5,52,5,148,4,52,5,149,4,161,4,160,4,159,4,238,4,9,5,8,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,1,5,247,4,1,4,1,4,52,5,247,4,247,4,247,4,247,4,247,4,247,4,243,4,76,4,67,4,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,235,4,233,4,52,5,194,4,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,72,4,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,3,5,52,5,208,4,72,4,72,4,72,4,74,4,56,4,66,4,236,3,111,4,90,4,90,4,41,5,111,4,41,5,18,4,23,5,15,4,101,4,90,4,176,4,101,4,101,4,73,4,66,4,52,5,44,5,81,4,81,4,43,5,43,5,81,4,122,4,46,4,111,4,52,4,52,4,52,4,52,4,81,4,248,3,111,4,122,4,46,4,46,4,111,4,81,4,248,3,212,4,38,5,81,4,81,4,248,3,187,4,81,4,248,3,81,4,248,3,187,4,44,4,44,4,44,4,33,4,187,4,44,4,18,4,44,4,33,4,44,4,44,4,94,4,89,4,94,4,89,4,94,4,89,4,94,4,89,4,81,4,182,4,81,4,52,5,187,4,191,4,191,4,187,4,106,4,95,4,104,4,102,4,111,4,254,3,36,4,6,5,6,5,2,5,2,5,2,5,2,5,49,5,49,5,243,4,18,5,18,5,20,4,20,4,18,5,52,5,52,5,52,5,52,5,52,5,52,5,13,5,52,5,196,4,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,128,4,52,5,232,3,240,4,52,5,52,5,239,4,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,40,5,52,5,52,5,52,5,52,5,52,5,52,5,211,4,210,4,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,58,4,52,5,52,5,52,5,27,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,103,4,52,5,96,4,52,5,52,5,31,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,52,5,249,4,52,5,52,5,52,5,248,4,52,5,52,5,52,5,52,5,52,5,130,4,52,5,129,4,133,4,52,5,242,3,52,5,16,0,2,2,12,0,14,0,24,0,31,0,59,0,15,39,97,99,111,115,0,97,115,105,110,0,97,116,97,110,0,97,116,110,50,0,97,116,97,110,50,0,97,99,111,115,104,0,97,115,105,110,104,0,97,116,97,110,104,0,100,105,102,102,101,114,101,110,99,101,0,100,101,103,114,101,101,115,0,114,97,100,105,97,110,115,0,99,111,115,0,115,105,110,0,116,97,110,0,99,111,116,0,99,111,115,104,0,115,105,110,104,0,116,97,110,104,0,99,111,116,104,0,101,120,112,0,108,111,103,49,48,0,112,111,119,101,114,0,115,105,103,110,0,115,113,114,116,0,115,113,117,97,114,101,0,99,101,105,108,0,102,108,111,111,114,0,112,105,0,114,101,112,108,105,99,97,116,101,0,99,104,97,114,105,110,100,101,120,0,108,101,102,116,115,116,114,0,114,105,103,104,116,115,116,114,0,114,101,118,101,114,115,101,0,112,114,111,112,101,114,0,112,97,100,108,0,112,97,100,114,0,112,97,100,99,0,115,116,114,102,105,108,116,101,114,0,115,116,100,101,118,0,118,97,114,105,97,110,99,101,0,109,101,100,105,97,110,0,108,111,119,101,114,95,113,117,97,114,116,105,108,101,0,117,112,112,101,114,95,113,117,97,114,116,105,108,101,0,102,111,114,32,110,111,100,101,0,97,114,103,99,61,61,49,0,99,47,101,120,116,101,110,115,105,111,110,45,102,117,110,99,116,105,111,110,115,46,99,0,97,99,111,115,70,117,110,99,0,97,115,105,110,70,117,110,99,0,97,116,97,110,70,117,110,99,0,97,114,103,99,61,61,50,0,97,116,110,50,70,117,110,99,0,97,99,111,115,104,70,117,110,99,0,97,115,105,110,104,70,117,110,99,0,97,116,97,110,104,70,117,110,99,0,100,105,102,102,101,114,101,110,99,101,70,117,110,99,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,2,3,0,1,2,0,0,2,2,4,5,5,0,1,2,6,2,3,0,1,0,2,0,2,0,0,0,0,0,0,0,1,2,3,0,1,2,0,0,2,2,4,5,5,0,1,2,6,2,3,0,1,0,2,0,2,0,0,0,0,0,63,48,48,48,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,4,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,4,114,97,100,50,100,101,103,70,117,110,99,0,100,101,103,50,114,97,100,70,117,110,99,0,99,111,115,70,117,110,99,0,115,105,110,70,117,110,99,0,116,97,110,70,117,110,99,0,99,111,116,70,117,110,99,0,99,111,115,104,70,117,110,99,0,115,105,110,104,70,117,110,99,0,116,97,110,104,70,117,110,99,0,99,111,116,104,70,117,110,99,0,101,120,112,70,117,110,99,0,108,111,103,70,117,110,99,0,108,111,103,49,48,70,117,110,99,0,112,111,119,101,114,70,117,110,99,0,115,105,103,110,70,117,110,99,0,115,113,114,116,70,117,110,99,0,115,113,117,97,114,101,70,117,110,99,0,99,101,105,108,70,117,110,99,0,102,108,111,111,114,70,117,110,99,0,100,111,109,97,105,110,32,101,114,114,111,114,0,97,114,103,99,61,61,51,32,124,124,97,114,103,99,61,61,50,0,99,104,97,114,105,110,100,101,120,70,117,110,99,0,108,101,102,116,70,117,110,99,0,114,105,103,104,116,70,117,110,99,0,49,61,61,97,114,103,99,0,114,101,118,101,114,115,101,70,117,110,99,0,112,114,111,112,101,114,70,117,110,99,0,112,97,100,108,70,117,110,99,0,122,60,61,122,84,101,114,109,0,115,113,108,105,116,101,51,85,116,102,56,67,104,97,114,76,101,110,0,112,97,100,114,70,117,110,99,0,112,97,100,99,70,117,110,99,0,115,116,114,102,105,108,116,101,114,70,117,110,99,0,118,97,114,105,97,110,99,101,83,116,101,112,0,109,111,100,101,83,116,101,112,0,51,46,49,53,46,49,0,83,81,76,73,84,69,95,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,54,55,56,57,58,59,60,61,62,63,64,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,91,92,93,94,95,96,97,98,99,100,101,102,103,104,105,106,107,108,109,110,111,112,113,114,115,116,117,118,119,120,121,122,123,124,125,126,127,128,129,130,131,132,133,134,135,136,137,138,139,140,141,142,143,144,145,146,147,148,149,150,151,152,153,154,155,156,157,158,159,160,161,162,163,164,165,166,167,168,169,170,171,172,173,174,175,176,177,178,179,180,181,182,183,184,185,186,187,188,189,190,191,192,193,194,195,196,197,198,199,200,201,202,203,204,205,206,207,208,209,210,211,212,213,214,215,216,217,218,219,220,221,222,223,224,225,226,227,228,229,230,231,232,233,234,235,236,237,238,239,240,241,242,243,244,245,246,247,248,249,250,251,252,253,254,255,0,1,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,4,5,3,5,1,5,1,5,2,5,2,5,1,5,1,5,4,5,3,5,1,5,1,5,2,5,2,5,1,5,1,5,115,116,114,105,110,103,32,111,114,32,98,108,111,98,32,116,111,111,32,98,105,103,0,99,97,110,110,111,116,32,111,112,101,110,32,118,105,114,116,117,97,108,32,116,97,98,108,101,58,32,37,115,0,99,97,110,110,111,116,32,111,112,101,110,32,116,97,98,108,101,32,119,105,116,104,111,117,116,32,114,111,119,105,100,58,32,37,115,0,99,97,110,110,111,116,32,111,112,101,110,32,118,105,101,119,58,32,37,115,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,58,32,34,37,115,34,0,102,111,114,101,105,103,110,32,107,101,121,0,105,110,100,101,120,101,100,0,99,97,110,110,111,116,32,111,112,101,110,32,37,115,32,99,111,108,117,109,110,32,102,111,114,32,119,114,105,116,105,110,103,0,151,0,0,0,104,0,0,0,82,1,1,0,33,0,7,1,96,0,0,1,87,1,0,0,13,0,2,0,111,0,0,0,75,0,0,0,37,115,0,42,63,91,0,37,95,0,1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,128,0,64,0,0,128,0,0,0,0,0,0,0,0,12,12,12,12,12,12,12,12,12,12,0,0,0,0,0,0,0,10,10,10,10,10,10,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,128,0,0,0,64,128,42,42,42,42,42,42,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,34,0,0,0,0,0,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,64,1,4,3,2,5,116,114,105,103,103,101,114,0,116,101,109,112,0,50,48,49,54,45,49,49,45,48,52,32,49,50,58,48,56,58,52,57,32,49,49,51,54,56,54,51,99,55,54,53,55,54,49,49,48,101,55,49,48,100,100,53,100,54,57,97,98,54,98,102,51,52,55,99,54,53,101,51,54,0,117,110,107,110,111,119,110,32,100,97,116,97,98,97,115,101,58,32,37,115,0,73,78,84,69,71,69,82,0,57,50,50,51,51,55,50,48,51,54,56,53,52,55,55,53,56,48,0,2,2,3,5,3,4,5,4,0,1,2,4,9,12,15,20,111,110,111,102,102,97,108,115,101,121,101,115,116,114,117,101,120,116,114,97,102,117,108,108,0,1,0,0,0,1,1,3,2,76,105,117,74,0,45,0,0,82,0,77,0,0,42,12,78,15,0,116,85,54,112,0,19,0,0,121,0,119,115,0,22,93,0,9,0,0,70,71,0,69,6,0,48,90,102,0,118,101,0,0,44,0,103,24,0,17,0,122,53,23,0,5,110,25,96,0,0,124,106,60,123,57,28,55,0,91,0,100,26,0,99,0,0,0,95,92,97,88,109,14,39,108,0,81,0,18,89,111,32,0,120,80,113,62,46,84,0,0,94,40,59,114,0,36,0,0,29,0,86,63,64,0,20,61,0,56,7,7,5,4,6,4,5,3,6,7,3,6,6,7,7,3,8,2,6,5,4,4,3,10,4,6,11,6,2,7,5,5,9,6,9,9,7,10,10,4,6,2,3,9,4,2,6,5,7,4,5,7,6,6,5,6,5,5,9,7,7,3,2,4,4,7,3,6,4,7,6,12,6,9,4,6,5,4,7,6,5,6,7,5,4,5,6,5,7,3,7,13,2,2,4,6,6,8,5,17,12,7,8,8,2,4,4,4,4,4,2,2,6,5,8,5,8,3,5,5,6,4,9,3,82,69,73,78,68,69,88,69,68,69,83,67,65,80,69,65,67,72,69,67,75,69,89,66,69,70,79,82,69,73,71,78,79,82,69,71,69,88,80,76,65,73,78,83,84,69,65,68,68,65,84,65,66,65,83,69,76,69,67,84,65,66,76,69,70,84,72,69,78,68,69,70,69,82,82,65,66,76,69,76,83,69,88,67,69,80,84,82,65,78,83,65,67,84,73,79,78,65,84,85,82,65,76,84,69,82,65,73,83,69,88,67,76,85,83,73,86,69,88,73,83,84,83,65,86,69,80,79,73,78,84,69,82,83,69,67,84,82,73,71,71,69,82,69,70,69,82,69,78,67,69,83,67,79,78,83,84,82,65,73,78,84,79,70,70,83,69,84,69,77,80,79,82,65,82,89,85,78,73,81,85,69,82,89,87,73,84,72,79,85,84,69,82,69,76,69,65,83,69,65,84,84,65,67,72,65,86,73,78,71,82,79,85,80,68,65,84,69,66,69,71,73,78,78,69,82,69,67,85,82,83,73,86,69,66,69,84,87,69,69,78,79,84,78,85,76,76,73,75,69,67,65,83,67,65,68,69,76,69,84,69,67,65,83,69,67,79,76,76,65,84,69,67,82,69,65,84,69,67,85,82,82,69,78,84,95,68,65,84,69,68,69,84,65,67,72,73,77,77,69,68,73,65,84,69,74,79,73,78,83,69,82,84,77,65,84,67,72,80,76,65,78,65,76,89,90,69,80,82,65,71,77,65,66,79,82,84,86,65,76,85,69,83,86,73,82,84,85,65,76,73,77,73,84,87,72,69,78,87,72,69,82,69,78,65,77,69,65,70,84,69,82,69,80,76,65,67,69,65,78,68,69,70,65,85,76,84,65,85,84,79,73,78,67,82,69,77,69,78,84,67,65,83,84,67,79,76,85,77,78,67,79,77,77,73,84,67,79,78,70,76,73,67,84,67,82,79,83,83,67,85,82,82,69,78,84,95,84,73,77,69,83,84,65,77,80,82,73,77,65,82,89,68,69,70,69,82,82,69,68,73,83,84,73,78,67,84,68,82,79,80,70,65,73,76,70,82,79,77,70,85,76,76,71,76,79,66,89,73,70,73,83,78,85,76,76,79,82,68,69,82,69,83,84,82,73,67,84,82,73,71,72,84,82,79,76,76,66,65,67,75,82,79,87,85,78,73,79,78,85,83,73,78,71,86,65,67,85,85,77,86,73,69,87,73,78,73,84,73,65,76,76,89,93,56,140,70,42,72,104,79,63,113,74,75,31,2,77,142,69,24,119,16,98,138,11,112,139,117,6,58,107,98,141,83,9,20,13,118,88,105,99,131,81,80,111,21,21,27,103,3,25,92,98,14,62,128,127,110,5,98,84,32,35,19,78,101,31,65,61,109,136,53,17,95,71,8,124,108,30,4,60,82,57,120,91,129,137,130,94,59,85,28,100,106,15,33,66,67,10,68,98,95,95,102,7,121,29,114,73,123,98,31,64,18,34,126,86,98,12,87,115,125,89,90,76,116,0,0,0,0,4,0,0,0,0,0,0,0,0,0,2,0,0,0,0,0,0,13,0,0,0,0,0,7,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,33,0,21,0,0,0,0,0,50,0,43,3,47,0,0,0,0,30,0,58,0,38,0,0,0,1,66,0,0,67,0,41,0,0,0,0,0,0,49,65,0,0,0,0,31,52,16,34,10,0,0,0,0,0,0,0,11,72,79,0,8,0,104,98,0,107,0,87,0,75,51,0,27,37,73,83,0,35,68,0,0,95,82,79,87,73,68,95,0,82,79,87,73,68,0,79,73,68,0,115,113,108,105,116,101,95,116,101,109,112,95,109,97,115,116,101,114,0,115,113,108,105,116,101,95,109,97,115,116,101,114,0,49,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,116,121,112,101,32,116,101,120,116,44,110,97,109,101,32,116,101,120,116,44,116,98,108,95,110,97,109,101,32,116,101,120,116,44,114,111,111,116,112,97,103,101,32,105,110,116,101,103,101,114,44,115,113,108,32,116,101,120,116,41,0,97,116,116,97,99,104,101,100,32,100,97,116,97,98,97,115,101,115,32,109,117,115,116,32,117,115,101,32,116,104,101,32,115,97,109,101,32,116,101,120,116,32,101,110,99,111,100,105,110,103,32,97,115,32,109,97,105,110,32,100,97,116,97,98,97,115,101,0,117,110,115,117,112,112,111,114,116,101,100,32,102,105,108,101,32,102,111,114,109,97,116,0,83,69,76,69,67,84,32,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,70,82,79,77,32,34,37,119,34,46,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,0,115,113,108,105,116,101,95,115,116,97,116,49,0,83,69,76,69,67,84,32,116,98,108,44,105,100,120,44,115,116,97,116,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,115,116,97,116,49,0,117,110,111,114,100,101,114,101,100,42,0,115,122,61,91,48,45,57,93,42,0,110,111,115,107,105,112,115,99,97,110,42,0,99,114,101,97,116,101,32,0,105,110,118,97,108,105,100,32,114,111,111,116,112,97,103,101,0,63,0,109,97,108,102,111,114,109,101,100,32,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,40,37,115,41,0,37,122,32,45,32,37,115,0,100,97,116,97,98,97,115,101,32,99,111,114,114,117,112,116,105,111,110,0,37,115,32,97,116,32,108,105,110,101,32,37,100,32,111,102,32,91,37,46,49,48,115,93,0,117,110,97,98,108,101,32,116,111,32,100,101,108,101,116,101,47,109,111,100,105,102,121,32,99,111,108,108,97,116,105,111,110,32,115,101,113,117,101,110,99,101,32,100,117,101,32,116,111,32,97,99,116,105,118,101,32,115,116,97,116,101,109,101,110,116,115,0,78,79,67,65,83,69,0,82,84,82,73,77,0,109,97,105,110,0,115,105,109,112,108,101,0,112,111,114,116,101,114,0,117,110,105,99,111,100,101,54,49,0,102,116,115,51,95,116,111,107,101,110,105,122,101,114,0,115,110], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+10240);
/* memory initializer */ allocate([105,112,112,101,116,0,111,102,102,115,101,116,115,0,109,97,116,99,104,105,110,102,111,0,111,112,116,105,109,105,122,101,0,102,116,115,51,0,102,116,115,52,0,102,116,115,51,116,111,107,101,110,105,122,101,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,105,110,112,117,116,44,32,116,111,107,101,110,44,32,115,116,97,114,116,44,32,101,110,100,44,32,112,111,115,105,116,105,111,110,41,0,117,110,107,110,111,119,110,32,116,111,107,101,110,105,122,101,114,58,32,37,115,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,99,111,110,116,101,110,116,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,100,111,99,115,105,122,101,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,116,97,116,39,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,115,116,97,116,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,115,101,103,109,101,110,116,115,39,59,0,65,76,84,69,82,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,32,32,82,69,78,65,77,69,32,84,79,32,39,37,113,95,115,101,103,100,105,114,39,59,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,87,72,69,82,69,32,114,111,119,105,100,32,61,32,63,0,83,69,76,69,67,84,32,78,79,84,32,69,88,73,83,84,83,40,83,69,76,69,67,84,32,100,111,99,105,100,32,70,82,79,77,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,87,72,69,82,69,32,114,111,119,105,100,33,61,63,41,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,116,97,116,39,0,83,69,76,69,67,84,32,37,115,32,87,72,69,82,69,32,114,111,119,105,100,61,63,0,83,69,76,69,67,84,32,40,83,69,76,69,67,84,32,109,97,120,40,105,100,120,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,41,32,43,32,49,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,40,98,108,111,99,107,105,100,44,32,98,108,111,99,107,41,32,86,65,76,85,69,83,40,63,44,32,63,41,0,83,69,76,69,67,84,32,99,111,97,108,101,115,99,101,40,40,83,69,76,69,67,84,32,109,97,120,40,98,108,111,99,107,105,100,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,41,32,43,32,49,44,32,49,41,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,86,65,76,85,69,83,40,63,44,63,44,63,44,63,44,63,44,63,41,0,83,69,76,69,67,84,32,105,100,120,44,32,115,116,97,114,116,95,98,108,111,99,107,44,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,44,32,101,110,100,95,98,108,111,99,107,44,32,114,111,111,116,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,79,82,68,69,82,32,66,89,32,105,100,120,32,65,83,67,0,83,69,76,69,67,84,32,105,100,120,44,32,115,116,97,114,116,95,98,108,111,99,107,44,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,44,32,101,110,100,95,98,108,111,99,107,44,32,114,111,111,116,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,79,82,68,69,82,32,66,89,32,108,101,118,101,108,32,68,69,83,67,44,32,105,100,120,32,65,83,67,0,83,69,76,69,67,84,32,99,111,117,110,116,40,42,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,0,83,69,76,69,67,84,32,109,97,120,40,108,101,118,101,108,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,32,87,72,69,82,69,32,98,108,111,99,107,105,100,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,32,86,65,76,85,69,83,40,37,115,41,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,87,72,69,82,69,32,100,111,99,105,100,32,61,32,63,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,86,65,76,85,69,83,40,63,44,63,41,0,83,69,76,69,67,84,32,115,105,122,101,32,70,82,79,77,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,32,87,72,69,82,69,32,100,111,99,105,100,61,63,0,83,69,76,69,67,84,32,118,97,108,117,101,32,70,82,79,77,32,37,81,46,39,37,113,95,115,116,97,116,39,32,87,72,69,82,69,32,105,100,61,63,0,82,69,80,76,65,67,69,32,73,78,84,79,32,37,81,46,39,37,113,95,115,116,97,116,39,32,86,65,76,85,69,83,40,63,44,63,41,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,0,83,69,76,69,67,84,32,63,32,85,78,73,79,78,32,83,69,76,69,67,84,32,108,101,118,101,108,32,47,32,40,49,48,50,52,32,42,32,63,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,83,69,76,69,67,84,32,108,101,118,101,108,44,32,99,111,117,110,116,40,42,41,32,65,83,32,99,110,116,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,32,32,71,82,79,85,80,32,66,89,32,108,101,118,101,108,32,72,65,86,73,78,71,32,99,110,116,62,61,63,32,32,79,82,68,69,82,32,66,89,32,40,108,101,118,101,108,32,37,37,32,49,48,50,52,41,32,65,83,67,32,76,73,77,73,84,32,49,0,83,69,76,69,67,84,32,50,32,42,32,116,111,116,97,108,40,49,32,43,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,32,45,32,115,116,97,114,116,95,98,108,111,99,107,41,32,32,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,60,32,63,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,61,32,63,0,85,80,68,65,84,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,105,100,120,32,61,32,63,32,87,72,69,82,69,32,108,101,118,101,108,61,63,32,65,78,68,32,105,100,120,61,63,0,83,69,76,69,67,84,32,105,100,120,44,32,115,116,97,114,116,95,98,108,111,99,107,44,32,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,44,32,101,110,100,95,98,108,111,99,107,44,32,114,111,111,116,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,61,32,63,0,85,80,68,65,84,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,115,116,97,114,116,95,98,108,111,99,107,32,61,32,63,44,32,114,111,111,116,32,61,32,63,87,72,69,82,69,32,108,101,118,101,108,32,61,32,63,32,65,78,68,32,105,100,120,32,61,32,63,0,83,69,76,69,67,84,32,49,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,32,87,72,69,82,69,32,98,108,111,99,107,105,100,61,63,32,65,78,68,32,98,108,111,99,107,32,73,83,32,78,85,76,76,0,83,69,76,69,67,84,32,105,100,120,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,61,63,32,79,82,68,69,82,32,66,89,32,49,32,65,83,67,0,83,69,76,69,67,84,32,109,97,120,40,32,108,101,118,101,108,32,37,37,32,49,48,50,52,32,41,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,83,69,76,69,67,84,32,108,101,118,101,108,44,32,105,100,120,44,32,101,110,100,95,98,108,111,99,107,32,70,82,79,77,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,87,72,69,82,69,32,108,101,118,101,108,32,66,69,84,87,69,69,78,32,63,32,65,78,68,32,63,32,79,82,68,69,82,32,66,89,32,108,101,118,101,108,32,68,69,83,67,44,32,105,100,120,32,65,83,67,0,85,80,68,65,84,69,32,79,82,32,70,65,73,76,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,108,101,118,101,108,61,45,49,44,105,100,120,61,63,32,87,72,69,82,69,32,108,101,118,101,108,61,63,32,65,78,68,32,105,100,120,61,63,0,85,80,68,65,84,69,32,79,82,32,70,65,73,76,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,32,83,69,84,32,108,101,118,101,108,61,63,32,87,72,69,82,69,32,108,101,118,101,108,61,45,49,0,37,108,108,100,32,37,108,108,100,0,37,115,95,115,101,103,109,101,110,116,115,0,98,108,111,99,107,0,83,69,76,69,67,84,32,49,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,39,37,113,95,115,116,97,116,39,0,112,99,120,0,83,69,76,69,67,84,32,37,115,32,87,72,69,82,69,32,114,111,119,105,100,32,61,32,63,0,117,110,114,101,99,111,103,110,105,122,101,100,32,109,97,116,99,104,105,110,102,111,32,114,101,113,117,101,115,116,58,32,37,99,0,105,108,108,101,103,97,108,32,102,105,114,115,116,32,97,114,103,117,109,101,110,116,32,116,111,32,37,115,0,73,110,100,101,120,32,111,112,116,105,109,105,122,101,100,0,73,110,100,101,120,32,97,108,114,101,97,100,121,32,111,112,116,105,109,97,108,0,83,65,86,69,80,79,73,78,84,32,102,116,115,51,0,82,69,76,69,65,83,69,32,102,116,115,51,0,82,79,76,76,66,65,67,75,32,84,79,32,102,116,115,51,0,37,100,32,37,100,32,37,100,32,37,100,32,0,60,98,62,0,60,47,98,62,0,60,98,62,46,46,46,60,47,98,62,0,119,114,111,110,103,32,110,117,109,98,101,114,32,111,102,32,97,114,103,117,109,101,110,116,115,32,116,111,32,102,117,110,99,116,105,111,110,32,115,110,105,112,112,101,116,40,41,0,114,101,98,117,105,108,100,0,105,110,116,101,103,114,105,116,121,45,99,104,101,99,107,0,109,101,114,103,101,61,0,97,117,116,111,109,101,114,103,101,61,0,67,82,69,65,84,69,32,84,65,66,76,69,32,73,70,32,78,79,84,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,116,97,116,39,40,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,44,32,118,97,108,117,101,32,66,76,79,66,41,59,0,83,69,76,69,67,84,32,37,115,0,68,69,83,67,0,65,83,67,0,83,69,76,69,67,84,32,37,115,32,87,72,69,82,69,32,114,111,119,105,100,32,66,69,84,87,69,69,78,32,37,108,108,100,32,65,78,68,32,37,108,108,100,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,32,37,115,0,83,69,76,69,67,84,32,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,32,37,115,0,70,84,83,32,101,120,112,114,101,115,115,105,111,110,32,116,114,101,101,32,105,115,32,116,111,111,32,108,97,114,103,101,32,40,109,97,120,105,109,117,109,32,100,101,112,116,104,32,37,100,41,0,109,97,108,102,111,114,109,101,100,32,77,65,84,67,72,32,101,120,112,114,101,115,115,105,111,110,58,32,91,37,115,93,0,79,82,0,65,78,68,0,78,79,84,0,78,69,65,82,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,0,68,82,79,80,32,84,65,66,76,69,32,73,70,32,69,88,73,83,84,83,32,37,81,46,39,37,113,95,115,116,97,116,39,0,116,111,107,101,110,105,122,101,0,117,110,114,101,99,111,103,110,105,122,101,100,32,112,97,114,97,109,101,116,101,114,58,32,37,115,0,117,110,114,101,99,111,103,110,105,122,101,100,32,109,97,116,99,104,105,110,102,111,58,32,37,115,0,97,115,99,0,100,101,115,99,0,117,110,114,101,99,111,103,110,105,122,101,100,32,111,114,100,101,114,58,32,37,115,0,99,111,110,116,101,110,116,0,101,114,114,111,114,32,112,97,114,115,105,110,103,32,112,114,101,102,105,120,32,112,97,114,97,109,101,116,101,114,58,32,37,115,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,58,32,37,115,0,99,111,109,112,114,101,115,115,0,117,110,99,111,109,112,114,101,115,115,0,109,105,115,115,105,110,103,32,37,115,32,112,97,114,97,109,101,116,101,114,32,105,110,32,102,116,115,52,32,99,111,110,115,116,114,117,99,116,111,114,0,95,95,108,97,110,103,105,100,0,37,81,44,32,0,37,122,37,81,44,32,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,37,115,32,37,81,32,72,73,68,68,69,78,44,32,100,111,99,105,100,32,72,73,68,68,69,78,44,32,37,81,32,72,73,68,68,69,78,41,0,80,82,65,71,77,65,32,37,81,46,112,97,103,101,95,115,105,122,101,0,100,111,99,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,0,37,122,44,32,39,99,37,100,37,113,39,0,37,122,44,32,108,97,110,103,105,100,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,99,111,110,116,101,110,116,39,40,37,115,41,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,109,101,110,116,115,39,40,98,108,111,99,107,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,44,32,98,108,111,99,107,32,66,76,79,66,41,59,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,115,101,103,100,105,114,39,40,108,101,118,101,108,32,73,78,84,69,71,69,82,44,105,100,120,32,73,78,84,69,71,69,82,44,115,116,97,114,116,95,98,108,111,99,107,32,73,78,84,69,71,69,82,44,108,101,97,118,101,115,95,101,110,100,95,98,108,111,99,107,32,73,78,84,69,71,69,82,44,101,110,100,95,98,108,111,99,107,32,73,78,84,69,71,69,82,44,114,111,111,116,32,66,76,79,66,44,80,82,73,77,65,82,89,32,75,69,89,40,108,101,118,101,108,44,32,105,100,120,41,41,59,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,39,37,113,95,100,111,99,115,105,122,101,39,40,100,111,99,105,100,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,44,32,115,105,122,101,32,66,76,79,66,41,59,0,44,37,115,40,63,41,0,44,32,63,0,37,115,37,115,0,100,111,99,105,100,0,44,37,115,40,120,46,39,99,37,100,37,113,39,41,0,44,32,120,46,37,81,0,108,97,110,103,105,100,0,114,111,119,105,100,0,44,32,120,46,39,37,113,39,0,95,99,111,110,116,101,110,116,0,32,70,82,79,77,32,39,37,113,39,46,39,37,113,37,115,39,32,65,83,32,120,0,83,69,76,69,67,84,32,42,32,70,82,79,77,32,37,81,46,37,81,0,112,114,101,102,105,120,0,111,114,100,101,114,0,108,97,110,103,117,97,103,101,105,100,0,110,111,116,105,110,100,101,120,101,100,0,117,110,107,110,111,119,110,32,116,111,107,101,110,105,122,101,114,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,97,114,103,117,109,101,110,116,32,116,121,112,101,32,109,105,115,109,97,116,99,104,0,111,117,116,32,111,102,32,109,101,109,111,114,121,0,102,116,115,51,116,111,107,101,110,105,122,101,32,100,105,115,97,98,108,101,100,0,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,115,101,115,115,0,115,115,0,115,101,105,0,105,0,100,101,101,0,101,101,0,103,110,105,0,100,101,0,116,97,0,97,116,101,0,108,98,0,98,108,101,0,122,105,0,105,122,101,0,108,97,110,111,105,116,97,0,108,97,110,111,105,116,0,116,105,111,110,0,105,99,110,101,0,101,110,99,101,0,105,99,110,97,0,97,110,99,101,0,114,101,122,105,0,105,103,111,108,0,108,111,103,0,105,108,98,0,105,108,108,97,0,97,108,0,105,108,116,110,101,0,101,110,116,0,105,108,101,0,101,0,105,108,115,117,111,0,111,117,115,0,110,111,105,116,97,122,105,0,110,111,105,116,97,0,114,111,116,97,0,109,115,105,108,97,0,115,115,101,110,101,118,105,0,105,118,101,0,115,115,101,110,108,117,102,0,102,117,108,0,115,115,101,110,115,117,111,0,105,116,105,108,97,0,105,116,105,118,105,0,105,116,105,108,105,98,0,101,116,97,99,105,0,105,99,0,101,118,105,116,97,0,101,122,105,108,97,0,105,116,105,99,105,0,108,97,99,105,0,108,117,102,0,115,115,101,110,0,116,110,101,109,101,0,116,110,101,109,0,116,110,101,0,110,111,105,0,101,116,97,0,105,116,105,0,0,1,1,1,0,1,1,1,0,1,1,1,1,1,0,1,1,1,1,1,0,1,1,1,2,1,102,116,115,52,97,117,120,0,42,0,67,82,69,65,84,69,32,84,65,66,76,69,32,120,40,116,101,114,109,44,32,99,111,108,44,32,100,111,99,117,109,101,110,116,115,44,32,111,99,99,117,114,114,101,110,99,101,115,44,32,108,97,110,103,117,97,103,101,105,100,32,72,73,68,68,69,78,41,0,105,110,118,97,108,105,100,32,97,114,103,117,109,101,110,116,115,32,116,111,32,102,116,115,52,97,117,120,32,99,111,110,115,116,114,117,99,116,111,114,0,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,16,17,18,19,20,21,22,23,24,25,26,27,28,29,30,31,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15,0,1,2,3,4,5,6,7,0,1,2,3,0,1,0,0,0,97,99,101,105,110,111,117,121,121,97,99,100,101,101,103,104,105,106,107,108,110,111,114,115,116,117,117,119,121,122,111,117,97,105,111,117,103,107,111,106,103,110,97,101,105,111,114,117,115,116,104,97,101,111,121,0,0,0,0,0,0,0,0,97,98,100,100,101,102,103,104,104,105,107,108,108,109,110,112,114,114,115,116,117,118,119,119,120,121,122,104,116,119,121,97,101,105,111,117,121,114,101,109,111,118,101,95,100,105,97,99,114,105,116,105,99,115,61,49,0,114,101,109,111,118,101,95,100,105,97,99,114,105,116,105,99,115,61,48,0,116,111,107,101,110,99,104,97,114,115,61,0,115,101,112,97,114,97,116,111,114,115,61,0,97,117,116,111,109,97,116,105,99,32,101,120,116,101,110,115,105,111,110,32,108,111,97,100,105,110,103,32,102,97,105,108,101,100,58,32,37,115,0,77,65,84,67,72,0,58,109,101,109,111,114,121,58,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,114,101,99,111,118,101,114,101,100,32,37,100,32,112,97,103,101,115,32,102,114,111,109,32,37,115,0,217,213,5,249,32,161,99,215,99,97,110,110,111,116,32,108,105,109,105,116,32,87,65,76,32,115,105,122,101,58,32,37,115,0,99,97,110,110,111,116,32,111,112,101,110,32,102,105,108,101,0,114,101,99,111,118,101,114,101,100,32,37,100,32,102,114,97,109,101,115,32,102,114,111,109,32,87,65,76,32,102,105,108,101,32,37,115,0,45,106,111,117,114,110,97,108,0,0,45,119,97,108,0,0,110,111,108,111,99,107,0,105,109,109,117,116,97,98,108,101,0,0,102,105,108,101,58,0,108,111,99,97,108,104,111,115,116,0,105,110,118,97,108,105,100,32,117,114,105,32,97,117,116,104,111,114,105,116,121,58,32,37,46,42,115,0,118,102,115,0,99,97,99,104,101,0,109,111,100,101,0,97,99,99,101,115,115,0,110,111,32,115,117,99,104,32,37,115,32,109,111,100,101,58,32,37,115,0,37,115,32,109,111,100,101,32,110,111,116,32,97,108,108,111,119,101,100,58,32,37,115,0,110,111,32,115,117,99,104,32,118,102,115,58,32,37,115,0,114,111,0,114,119,0,114,119,99,0,109,101,109,111,114,121,0,115,104,97,114,101,100,0,112,114,105,118,97,116,101,0,117,110,97,98,108,101,32,116,111,32,117,115,101,32,102,117,110,99,116,105,111,110,32,37,115,32,105,110,32,116,104,101,32,114,101,113,117,101,115,116,101,100,32,99,111,110,116,101,120,116,0,117,110,97,98,108,101,32,116,111,32,100,101,108,101,116,101,47,109,111,100,105,102,121,32,117,115,101,114,45,102,117,110,99,116,105,111,110,32,100,117,101,32,116,111,32,97,99,116,105,118,101,32,115,116,97,116,101,109,101,110,116,115,0,117,110,97,98,108,101,32,116,111,32,99,108,111,115,101,32,100,117,101,32,116,111,32,117,110,102,105,110,97,108,105,122,101,100,32,115,116,97,116,101,109,101,110,116,115,32,111,114,32,117,110,102,105,110,105,115,104,101,100,32,98,97,99,107,117,112,115,0,102,97,105,108,101,100,32,109,101,109,111,114,121,32,114,101,115,105,122,101,32,37,117,32,116,111,32,37,117,32,98,121,116,101,115,0,102,97,105,108,101,100,32,116,111,32,97,108,108,111,99,97,116,101,32,37,117,32,98,121,116,101,115,32,111,102,32,109,101,109,111,114,121,0,117,110,114,101,99,111,103,110,105,122,101,100,32,116,111,107,101,110,58,32,34,37,84,34,0,110,101,97,114,32,34,37,84,34,58,32,115,121,110,116,97,120,32,101,114,114,111,114,0,147,1,147,3,148,1,149,3,150,0,150,1,150,1,150,1,149,2,149,2,149,2,149,2,149,3,149,5,154,6,156,1,158,0,158,3,157,1,157,0,155,5,155,2,162,0,162,2,164,2,166,0,166,4,166,6,167,2,171,2,171,2,171,4,171,3,171,3,171,2,171,3,171,5,171,2,171,4,171,4,171,1,171,2,176,0,176,1,178,0,178,2,180,2,180,3,180,3,180,3,181,2,181,2,181,1,181,1,181,2,179,3,179,2,182,0,182,2,182,2,161,0,184,1,185,2,185,7,185,5,185,5,185,10,188,0,174,0,174,3,189,0,189,2,190,1,190,1,149,4,192,2,192,0,149,9,149,4,149,1,163,2,194,3,197,1,197,2,197,1,195,9,206,4,206,5,198,1,198,1,198,0,209,0,199,3,199,2,199,4,210,2,210,0,200,0,200,2,212,2,212,0,211,7,211,9,211,7,211,7,159,0,159,2,193,2,213,1,213,2,213,3,213,4,215,2,215,0,214,0,214,3,214,2,216,4,216,0,204,0,204,3,186,4,186,2,175,1,175,1,175,0,202,0,202,3,203,0,203,2,205,0,205,2,205,4,205,4,149,6,201,0,201,2,149,8,218,5,218,7,218,3,218,5,149,6,149,7,219,2,219,1,220,0,220,3,217,3,217,1,173,3,172,1,173,1,173,1,173,3,173,5,172,1,172,1,172,1,173,1,173,3,173,6,173,5,173,4,172,1,173,5,173,3,173,3,173,3,173,3,173,3,173,3,173,3,173,3,221,1,221,2,173,3,173,5,173,2,173,3,173,3,173,4,173,2,173,2,173,2,173,2,222,1,222,2,173,5,223,1,223,2,173,5,173,3,173,5,173,5,173,4,173,5,226,5,226,4,227,2,227,0,225,1,225,0,208,0,207,3,207,1,224,0,224,3,149,12,228,1,228,0,177,0,177,3,187,5,187,3,229,0,229,2,149,4,149,1,149,2,149,3,149,5,149,6,149,5,149,6,169,2,170,2,149,5,231,11,233,1,233,1,233,2,233,0,234,1,234,1,234,3,236,0,236,2,232,3,232,2,238,3,239,3,239,2,237,7,237,5,237,5,237,1,173,4,173,6,191,1,191,1,191,1,149,4,149,6,149,3,241,0,241,2,149,1,149,3,149,1,149,3,149,6,149,7,242,1,149,1,149,4,244,8,246,0,247,1,247,3,248,1,196,0,196,2,196,3,250,6,250,8,144,1,145,2,145,1,146,1,146,3,147,0,151,0,151,1,151,2,153,1,153,0,149,2,160,4,160,2,152,1,152,1,152,1,166,1,167,1,168,1,168,1,165,2,165,0,171,2,161,2,183,3,183,1,184,0,188,1,190,1,194,1,195,1,209,2,210,1,173,1,208,1,230,1,230,1,230,1,230,1,230,1,169,1,235,0,235,3,238,1,239,0,240,1,240,0,243,0,243,1,245,1,245,3,246,2,249,0,249,4,249,2,117,110,107,110,111,119,110,32,116,97,98,108,101,32,111,112,116,105,111,110,58,32,37,46,42,115,0,115,101,116,32,108,105,115,116,0,116,111,111,32,109,97,110,121,32,97,114,103,117,109,101,110,116,115,32,111,110,32,102,117,110,99,116,105,111,110,32,37,84,0,113,117,97,108,105,102,105,101,100,32,116,97,98,108,101,32,110,97,109,101,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,73,78,83,69,82,84,44,32,85,80,68,65,84,69,44,32,97,110,100,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,116,104,101,32,73,78,68,69,88,69,68,32,66,89,32,99,108,97,117,115,101,32,105,115,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,85,80,68,65,84,69,32,111,114,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,116,104,101,32,78,79,84,32,73,78,68,69,88,69,68,32,99,108,97,117,115,101,32,105,115,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,85,80,68,65,84,69,32,111,114,32,68,69,76,69,84,69,32,115,116,97,116,101,109,101,110,116,115,32,119,105,116,104,105,110,32,116,114,105,103,103,101,114,115,0,100,117,112,108,105,99,97,116,101,32,87,73,84,72,32,116,97,98,108,101,32,110,97,109,101,58,32,37,115,0,110,111,116,32,97,117,116,104,111,114,105,122,101,100,0,97,117,116,104,111,114,105,122,101,114,32,109,97,108,102,117,110,99,116,105,111,110,0,67,82,69,65,84,69,32,86,73,82,84,85,65,76,32,84,65,66,76,69,32,37,84,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,116,121,112,101,61,39,116,97,98,108,101,39,44,32,110,97,109,101,61,37,81,44,32,116,98,108,95,110,97,109,101,61,37,81,44,32,114,111,111,116,112,97,103,101,61,48,44,32,115,113,108,61,37,81,32,87,72,69,82,69,32,114,111,119,105,100,61,35,37,100,0,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,61,39,116,97,98,108,101,39,0,118,105,114,116,117,97,108,32,116,97,98,108,101,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,67,97,110,110,111,116,32,97,100,100,32,97,32,99,111,108,117,109,110,32,116,111,32,97,32,118,105,101,119,0,115,113,108,105,116,101,95,97,108,116,101,114,116,97,98,95,37,115,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,97,32,116,101,109,112,111,114,97,114,121,32,100,97,116,97,98,97,115,101,32,102,105,108,101,32,102,111,114,32,115,116,111,114,105,110,103,32,116,101,109,112,111,114,97,114,121,32,116,97,98,108,101,115,0,115,113,108,105,116,101,95,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,67,97,110,110,111,116,32,97,100,100,32,97,32,80,82,73,77,65,82,89,32,75,69,89,32,99,111,108,117,109,110,0,67,97,110,110,111,116,32,97,100,100,32,97,32,85,78,73,81,85,69,32,99,111,108,117,109,110,0,67,97,110,110,111,116,32,97,100,100,32,97,32,82,69,70,69,82,69,78,67,69,83,32,99,111,108,117,109,110,32,119,105,116,104,32,110,111,110,45,78,85,76,76,32,100,101,102,97,117,108,116,32,118,97,108,117,101,0,67,97,110,110,111,116,32,97,100,100,32,97,32,78,79,84,32,78,85,76,76,32,99,111,108,117,109,110,32,119,105,116,104,32,100,101,102,97,117,108,116,32,118,97,108,117,101,32,78,85,76,76,0,67,97,110,110,111,116,32,97,100,100,32,97,32,99,111,108,117,109,110,32,119,105,116,104,32,110,111,110,45,99,111,110,115,116,97,110,116,32,100,101,102,97,117,108,116,0,85,80,68,65,84,69,32,34,37,119,34,46,37,115,32,83,69,84,32,115,113,108,32,61,32,115,117,98,115,116,114,40,115,113,108,44,49,44,37,100,41,32,124,124,32,39,44,32,39,32,124,124,32,37,81,32,124,124,32,115,117,98,115,116,114,40,115,113,108,44,37,100,41,32,87,72,69,82,69,32,116,121,112,101,32,61,32,39,116,97,98,108,101,39,32,65,78,68,32,110,97,109,101,32,61,32,37,81,0,116,98,108,95,110,97,109,101,61,37,81,0,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,65,78,68,32,40,37,115,41,0,110,97,109,101,61,37,81,0,37,115,32,79,82,32,110,97,109,101,61,37,81,0,45,0,37,108,108,100,0,37,33,46,49,53,103,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,110,111,116,104,101,114,32,116,97,98,108,101,32,111,114,32,105,110,100,101,120,32,119,105,116,104,32,116,104,105,115,32,110,97,109,101,58,32,37,115,0,118,105,101,119,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,97,108,116,101,114,101,100,0,85,80,68,65,84,69,32,34,37,119,34,46,37,115,32,83,69,84,32,115,113,108,32,61,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,112,97,114,101,110,116,40,115,113,108,44,32,37,81,44,32,37,81,41,32,87,72,69,82,69,32,37,115,59,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,115,113,108,32,61,32,67,65,83,69,32,87,72,69,78,32,116,121,112,101,32,61,32,39,116,114,105,103,103,101,114,39,32,84,72,69,78,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,40,115,113,108,44,32,37,81,41,69,76,83,69,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,97,98,108,101,40,115,113,108,44,32,37,81,41,32,69,78,68,44,32,116,98,108,95,110,97,109,101,32,61,32,37,81,44,32,110,97,109,101,32,61,32,67,65,83,69,32,87,72,69,78,32,116,121,112,101,61,39,116,97,98,108,101,39,32,84,72,69,78,32,37,81,32,87,72,69,78,32,110,97,109,101,32,76,73,75,69,32,39,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,37,37,39,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,32,84,72,69,78,32,39,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,95,39,32,124,124,32,37,81,32,124,124,32,115,117,98,115,116,114,40,110,97,109,101,44,37,100,43,49,56,41,32,69,76,83,69,32,110,97,109,101,32,69,78,68,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,37,81,32,67,79,76,76,65,84,69,32,110,111,99,97,115,101,32,65,78,68,32,40,116,121,112,101,61,39,116,97,98,108,101,39,32,79,82,32,116,121,112,101,61,39,105,110,100,101,120,39,32,79,82,32,116,121,112,101,61,39,116,114,105,103,103,101,114,39,41,59,0,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,0,85,80,68,65,84,69,32,34,37,119,34,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,32,115,101,116,32,110,97,109,101,32,61,32,37,81,32,87,72,69,82,69,32,110,97,109,101,32,61,32,37,81,0,85,80,68,65,84,69,32,115,113,108,105,116,101,95,116,101,109,112,95,109,97,115,116,101,114,32,83,69,84,32,115,113,108,32,61,32,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,40,115,113,108,44,32,37,81,41,44,32,116,98,108,95,110,97,109,101,32,61,32,37,81,32,87,72,69,82,69,32,37,115,59,0,118,105,101,119,32,37,115,32,105,115,32,99,105,114,99,117,108,97,114,108,121,32,100,101,102,105,110,101,100,0,110,111,32,115,117,99,104,32,99,111,108,108,97,116,105,111,110,32,115,101,113,117,101,110,99,101,58,32,37,115,0,3,2,1,37,46,42,122,58,37,117,0,97,32,71,82,79,85,80,32,66,89,32,99,108,97,117,115,101,32,105,115,32,114,101,113,117,105,114,101,100,32,98,101,102,111,114,101,32,72,65,86,73,78,71,0,79,82,68,69,82,0,71,82,79,85,80,0,97,103,103,114,101,103,97,116,101,32,102,117,110,99,116,105,111,110,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,116,104,101,32,71,82,79,85,80,32,66,89,32,99,108,97,117,115,101,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,79,82,68,69,82,32,66,89,32,99,108,97,117,115,101,0,37,114,32,79,82,68,69,82,32,66,89,32,116,101,114,109,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,97,110,121,32,99,111,108,117,109,110,32,105,110,32,116,104,101,32,114,101,115,117,108,116,32,115,101,116,0,37,114,32,37,115,32,66,89,32,116,101,114,109,32,111,117,116,32,111,102,32,114,97,110,103,101,32,45,32,115,104,111,117,108,100,32,98,101,32,98,101,116,119,101,101,110,32,49,32,97,110,100,32,37,100,0,97,108,108,32,86,65,76,85,69,83,32,109,117,115,116,32,104,97,118,101,32,116,104,101,32,115,97,109,101,32,110,117,109,98,101,114,32,111,102,32,116,101,114,109,115,0,83,69,76,69,67,84,115,32,116,111,32,116,104,101,32,108,101,102,116,32,97,110,100,32,114,105,103,104,116,32,111,102,32,37,115,32,100,111,32,110,111,116,32,104,97,118,101,32,116,104,101,32,115,97,109,101,32,110,117,109,98,101,114,32,111,102,32,114,101,115,117,108,116,32,99,111,108,117,109,110,115,0,85,78,73,79,78,32,65,76,76,0,73,78,84,69,82,83,69,67,84,0,69,88,67,69,80,84,0,85,78,73,79,78,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,37,115,32,66,89,32,99,108,97,117,115,101,0,69,120,112,114,101,115,115,105,111,110,32,116,114,101,101,32,105,115,32,116,111,111,32,108,97,114,103,101,32,40,109,97,120,105,109,117,109,32,100,101,112,116,104,32,37,100,41,0,116,104,101,32,34,46,34,32,111,112,101,114,97,116,111,114,0,115,101,99,111,110,100,32,97,114,103,117,109,101,110,116,32,116,111,32,108,105,107,101,108,105,104,111,111,100,40,41,32,109,117,115,116,32,98,101,32,97,32,99,111,110,115,116,97,110,116,32,98,101,116,119,101,101,110,32,48,46,48,32,97,110,100,32,49,46,48,0,110,111,116,32,97,117,116,104,111,114,105,122,101,100,32,116,111,32,117,115,101,32,102,117,110,99,116,105,111,110,58,32,37,115,0,110,111,110,45,100,101,116,101,114,109,105,110,105,115,116,105,99,32,102,117,110,99,116,105,111,110,115,0,109,105,115,117,115,101,32,111,102,32,97,103,103,114,101,103,97,116,101,32,102,117,110,99,116,105,111,110,32,37,46,42,115,40,41,0,110,111,32,115,117,99,104,32,102,117,110,99,116,105,111,110,58,32,37,46,42,115,0,119,114,111,110,103,32,110,117,109,98,101,114,32,111,102,32,97,114,103,117,109,101,110,116,115,32,116,111,32,102,117,110,99,116,105,111,110,32,37,46,42,115,40,41,0,115,117,98,113,117,101,114,105,101,115,0,112,97,114,97,109,101,116,101,114,115,0,114,111,119,32,118,97,108,117,101,32,109,105,115,117,115,101,100,0,112,97,114,116,105,97,108,32,105,110,100,101,120,32,87,72,69,82,69,32,99,108,97,117,115,101,115,0,105,110,100,101,120,32,101,120,112,114,101,115,115,105,111,110,115,0,67,72,69,67,75,32,99,111,110,115,116,114,97,105,110,116,115,0,37,115,32,112,114,111,104,105,98,105,116,101,100,32,105,110,32,37,115,0,110,101,119,0,111,108,100,0,109,105,115,117,115,101,32,111,102,32,97,108,105,97,115,101,100,32,97,103,103,114,101,103,97,116,101,32,37,115,0,110,111,32,115,117,99,104,32,99,111,108,117,109,110,0,97,109,98,105,103,117,111,117,115,32,99,111,108,117,109,110,32,110,97,109,101,0,37,115,58,32,37,115,46,37,115,46,37,115,0,37,115,58,32,37,115,46,37,115,0,37,115,58,32,37,115,0,97,99,99,101,115,115,32,116,111,32,37,115,46,37,115,46,37,115,32,105,115,32,112,114,111,104,105,98,105,116,101,100,0,97,99,99,101,115,115,32,116,111,32,37,115,46,37,115,32,105,115,32,112,114,111,104,105,98,105,116,101,100,0,115,113,108,105,116,101,95,115,113,95,37,112,0,116,111,111,32,109,97,110,121,32,114,101,102,101,114,101,110,99,101,115,32,116,111,32,34,37,115,34,58,32,109,97,120,32,54,53,53,51,53,0,37,115,46,37,115,0,37,115,46,37,115,46,37,115,0,110,111,32,115,117,99,104,32,116,97,98,108,101,58,32,37,115,0,110,111,32,116,97,98,108,101,115,32,115,112,101,99,105,102,105,101,100,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,105,110,32,114,101,115,117,108,116,32,115,101,116,0,97,32,78,65,84,85,82,65,76,32,106,111,105,110,32,109,97,121,32,110,111,116,32,104,97,118,101,32,97,110,32,79,78,32,111,114,32,85,83,73,78,71,32,99,108,97,117,115,101,0,99,97,110,110,111,116,32,104,97,118,101,32,98,111,116,104,32,79,78,32,97,110,100,32,85,83,73,78,71,32,99,108,97,117,115,101,115,32,105,110,32,116,104,101,32,115,97,109,101,32,106,111,105,110,0,99,97,110,110,111,116,32,106,111,105,110,32,117,115,105,110,103,32,99,111,108,117,109,110,32,37,115,32,45,32,99,111,108,117,109,110,32,110,111,116,32,112,114,101,115,101,110,116,32,105,110,32,98,111,116,104,32,116,97,98,108,101,115,0,110,111,32,115,117,99,104,32,105,110,100,101,120,58,32,37,115,0,39,37,115,39,32,105,115,32,110,111,116,32,97,32,102,117,110,99,116,105,111,110,0,109,117,108,116,105,112,108,101,32,114,101,102,101,114,101,110,99,101,115,32,116,111,32,114,101,99,117,114,115,105,118,101,32,116,97,98,108,101,58,32,37,115,0,99,105,114,99,117,108,97,114,32,114,101,102,101,114,101,110,99,101,58,32,37,115,0,116,97,98,108,101,32,37,115,32,104,97,115,32,37,100,32,118,97,108,117,101,115,32,102,111,114,32,37,100,32,99,111,108,117,109,110,115,0,109,117,108,116,105,112,108,101,32,114,101,99,117,114,115,105,118,101,32,114,101,102,101,114,101,110,99,101,115,58,32,37,115,0,114,101,99,117,114,115,105,118,101,32,114,101,102,101,114,101,110,99,101,32,105,110,32,97,32,115,117,98,113,117,101,114,121,58,32,37,115,0,110,111,32,115,117,99,104,32,109,111,100,117,108,101,58,32,37,115,0,118,116,97,98,108,101,32,99,111,110,115,116,114,117,99,116,111,114,32,99,97,108,108,101,100,32,114,101,99,117,114,115,105,118,101,108,121,58,32,37,115,0,118,116,97,98,108,101,32,99,111,110,115,116,114,117,99,116,111,114,32,102,97,105,108,101,100,58,32,37,115,0,118,116,97,98,108,101], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+20480);
/* memory initializer */ allocate([32,99,111,110,115,116,114,117,99,116,111,114,32,100,105,100,32,110,111,116,32,100,101,99,108,97,114,101,32,115,99,104,101,109,97,58,32,37,115,0,104,105,100,100,101,110,0,111,98,106,101,99,116,32,110,97,109,101,32,114,101,115,101,114,118,101,100,32,102,111,114,32,105,110,116,101,114,110,97,108,32,117,115,101,58,32,37,115,0,99,111,114,114,117,112,116,32,100,97,116,97,98,97,115,101,0,117,110,107,110,111,119,110,32,100,97,116,97,98,97,115,101,32,37,84,0,105,100,120,0,116,98,108,0,115,113,108,105,116,101,95,37,0,66,66,66,0,115,116,97,116,95,103,101,116,0,37,108,108,117,0,32,37,108,108,117,0,115,116,97,116,95,112,117,115,104,0,115,116,97,116,95,105,110,105,116,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,37,115,40,37,115,41,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,37,115,61,37,81,0,116,98,108,44,105,100,120,44,115,116,97,116,0,115,113,108,105,116,101,95,115,116,97,116,51,0,115,113,108,105,116,101,95,115,116,97,116,52,0,117,110,97,98,108,101,32,116,111,32,105,100,101,110,116,105,102,121,32,116,104,101,32,111,98,106,101,99,116,32,116,111,32,98,101,32,114,101,105,110,100,101,120,101,100,0,105,110,100,101,120,32,39,37,113,39,0,44,32,0,109,105,115,117,115,101,32,111,102,32,97,103,103,114,101,103,97,116,101,58,32,37,115,40,41,0,117,110,107,110,111,119,110,32,102,117,110,99,116,105,111,110,58,32,37,115,40,41,0,82,65,73,83,69,40,41,32,109,97,121,32,111,110,108,121,32,98,101,32,117,115,101,100,32,119,105,116,104,105,110,32,97,32,116,114,105,103,103,101,114,45,112,114,111,103,114,97,109,0,85,83,73,78,71,32,73,78,68,69,88,32,37,115,32,70,79,82,32,73,78,45,79,80,69,82,65,84,79,82,0,67,79,82,82,69,76,65,84,69,68,32,0,76,73,83,84,0,83,67,65,76,65,82,0,69,88,69,67,85,84,69,32,37,115,37,115,32,83,85,66,81,85,69,82,89,32,37,100,0,115,117,98,45,115,101,108,101,99,116,32,114,101,116,117,114,110,115,32,37,100,32,99,111,108,117,109,110,115,32,45,32,101,120,112,101,99,116,101,100,32,37,100,0,48,120,0,104,101,120,32,108,105,116,101,114,97,108,32,116,111,111,32,98,105,103,58,32,37,115,0,115,113,108,105,116,101,95,100,101,116,97,99,104,0,110,111,32,115,117,99,104,32,100,97,116,97,98,97,115,101,58,32,37,115,0,99,97,110,110,111,116,32,100,101,116,97,99,104,32,100,97,116,97,98,97,115,101,32,37,115,0,99,97,110,110,111,116,32,68,69,84,65,67,72,32,100,97,116,97,98,97,115,101,32,119,105,116,104,105,110,32,116,114,97,110,115,97,99,116,105,111,110,0,100,97,116,97,98,97,115,101,32,37,115,32,105,115,32,108,111,99,107,101,100,0,115,113,108,105,116,101,95,97,116,116,97,99,104,0,116,111,111,32,109,97,110,121,32,97,116,116,97,99,104,101,100,32,100,97,116,97,98,97,115,101,115,32,45,32,109,97,120,32,37,100,0,99,97,110,110,111,116,32,65,84,84,65,67,72,32,100,97,116,97,98,97,115,101,32,119,105,116,104,105,110,32,116,114,97,110,115,97,99,116,105,111,110,0,100,97,116,97,98,97,115,101,32,37,115,32,105,115,32,97,108,114,101,97,100,121,32,105,110,32,117,115,101,0,100,97,116,97,98,97,115,101,32,105,115,32,97,108,114,101,97,100,121,32,97,116,116,97,99,104,101,100,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,100,97,116,97,98,97,115,101,58,32,37,115,0,110,111,32,115,117,99,104,32,116,114,105,103,103,101,114,58,32,37,83,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,110,97,109,101,61,37,81,32,65,78,68,32,116,121,112,101,61,39,116,114,105,103,103,101,114,39,0,116,101,109,112,111,114,97,114,121,32,116,114,105,103,103,101,114,32,109,97,121,32,110,111,116,32,104,97,118,101,32,113,117,97,108,105,102,105,101,100,32,110,97,109,101,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,116,114,105,103,103,101,114,115,32,111,110,32,118,105,114,116,117,97,108,32,116,97,98,108,101,115,0,116,114,105,103,103,101,114,32,37,84,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,116,114,105,103,103,101,114,32,111,110,32,115,121,115,116,101,109,32,116,97,98,108,101,0,66,69,70,79,82,69,0,65,70,84,69,82,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,37,115,32,116,114,105,103,103,101,114,32,111,110,32,118,105,101,119,58,32,37,83,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,73,78,83,84,69,65,68,32,79,70,32,116,114,105,103,103,101,114,32,111,110,32,116,97,98,108,101,58,32,37,83,0,37,115,32,37,84,32,99,97,110,110,111,116,32,114,101,102,101,114,101,110,99,101,32,111,98,106,101,99,116,115,32,105,110,32,100,97,116,97,98,97,115,101,32,37,115,0,37,115,32,99,97,110,110,111,116,32,117,115,101,32,118,97,114,105,97,98,108,101,115,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,37,115,32,86,65,76,85,69,83,40,39,116,114,105,103,103,101,114,39,44,37,81,44,37,81,44,48,44,39,67,82,69,65,84,69,32,84,82,73,71,71,69,82,32,37,113,39,41,0,116,121,112,101,61,39,116,114,105,103,103,101,114,39,32,65,78,68,32,110,97,109,101,61,39,37,113,39,0,45,37,84,0,114,101,115,117,108,116,0,99,97,99,104,101,95,115,105,122,101,0,2,0,0,0,101,0,1,3,66,1,8,0,76,0,2,0,48,1,2,1,66,1,8,0,76,0,1,0,161,0,0,0,87,1,1,0,112,97,103,101,95,115,105,122,101,0,115,101,99,117,114,101,95,100,101,108,101,116,101,0,110,111,114,109,97,108,0,101,120,99,108,117,115,105,118,101,0,108,111,99,107,105,110,103,95,109,111,100,101,0,106,111,117,114,110,97,108,95,109,111,100,101,0,106,111,117,114,110,97,108,95,115,105,122,101,95,108,105,109,105,116,0,97,117,116,111,95,118,97,99,117,117,109,0,2,0,1,0,101,0,1,4,21,1,0,0,75,0,2,0,102,0,7,0,99,97,99,104,101,95,115,112,105,108,108,0,109,109,97,112,95,115,105,122,101,0,116,101,109,112,95,115,116,111,114,101,0,116,101,109,112,95,115,116,111,114,101,95,100,105,114,101,99,116,111,114,121,0,110,111,116,32,97,32,119,114,105,116,97,98,108,101,32,100,105,114,101,99,116,111,114,121,0,115,121,110,99,104,114,111,110,111,117,115,0,83,97,102,101,116,121,32,108,101,118,101,108,32,109,97,121,32,110,111,116,32,98,101,32,99,104,97,110,103,101,100,32,105,110,115,105,100,101,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,105,115,115,105,115,105,0,115,115,105,105,0,115,105,105,0,105,105,115,0,105,115,105,0,105,115,105,115,105,0,105,115,115,0,105,115,0,105,105,115,115,115,115,115,115,0,78,79,78,69,0,115,105,0,105,110,116,101,103,114,105,116,121,95,99,104,101,99,107,0,42,42,42,32,105,110,32,100,97,116,97,98,97,115,101,32,37,115,32,42,42,42,10,0,78,85,76,76,32,118,97,108,117,101,32,105,110,32,37,115,46,37,115,0,114,111,119,32,0,32,109,105,115,115,105,110,103,32,102,114,111,109,32,105,110,100,101,120,32,0,110,111,110,45,117,110,105,113,117,101,32,101,110,116,114,121,32,105,110,32,105,110,100,101,120,32,0,119,114,111,110,103,32,35,32,111,102,32,101,110,116,114,105,101,115,32,105,110,32,105,110,100,101,120,32,0,91,1,0,0,21,1,4,0,97,0,3,0,87,3,1,0,111,107,0,101,110,99,111,100,105,110,103,0,117,110,115,117,112,112,111,114,116,101,100,32,101,110,99,111,100,105,110,103,58,32,37,115,0,2,0,1,0,102,0,0,0,2,0,0,0,101,0,1,0,87,1,1,0,99,111,109,112,105,108,101,95,111,112,116,105,111,110,0,102,117,108,108,0,114,101,115,116,97,114,116,0,116,114,117,110,99,97,116,101,0,119,97,108,95,97,117,116,111,99,104,101,99,107,112,111,105,110,116,0,116,105,109,101,111,117,116,0,115,111,102,116,95,104,101,97,112,95,108,105,109,105,116,0,116,104,114,101,97,100,115,0,98,117,115,121,0,99,104,101,99,107,112,111,105,110,116,101,100,0,85,84,70,56,0,85,84,70,45,56,0,85,84,70,45,49,54,108,101,0,85,84,70,45,49,54,98,101,0,85,84,70,49,54,108,101,0,85,84,70,49,54,98,101,0,85,84,70,45,49,54,0,85,84,70,49,54,0,37,95,0,0,108,105,107,101,0,103,108,111,98,0,76,73,75,69,32,111,114,32,71,76,79,66,32,112,97,116,116,101,114,110,32,116,111,111,32,99,111,109,112,108,101,120,0,69,83,67,65,80,69,32,101,120,112,114,101,115,115,105,111,110,32,109,117,115,116,32,98,101,32,97,32,115,105,110,103,108,101,32,99,104,97,114,97,99,116,101,114,0,102,111,114,101,105,103,110,32,107,101,121,32,109,105,115,109,97,116,99,104,32,45,32,34,37,119,34,32,114,101,102,101,114,101,110,99,105,110,103,32,34,37,119,34,0,116,97,98,108,101,0,112,97,114,101,110,116,0,102,107,105,100,0,83,69,84,32,78,85,76,76,0,83,69,84,32,68,69,70,65,85,76,84,0,67,65,83,67,65,68,69,0,82,69,83,84,82,73,67,84,0,78,79,32,65,67,84,73,79,78,0,105,100,0,115,101,113,0,102,114,111,109,0,116,111,0,111,110,95,117,112,100,97,116,101,0,111,110,95,100,101,108,101,116,101,0,109,97,116,99,104,0,110,97,109,101,0,102,105,108,101,0,99,0,117,0,112,107,0,117,110,105,113,117,101,0,111,114,105,103,105,110,0,112,97,114,116,105,97,108,0,115,101,113,110,111,0,99,105,100,0,99,111,108,108,0,107,101,121,0,105,110,100,101,120,0,119,105,100,116,104,0,104,101,105,103,104,116,0,116,121,112,101,0,110,111,116,110,117,108,108,0,100,102,108,116,95,118,97,108,117,101,0,116,101,109,112,111,114,97,114,121,32,115,116,111,114,97,103,101,32,99,97,110,110,111,116,32,98,101,32,99,104,97,110,103,101,100,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,110,111,110,101,0,105,110,99,114,101,109,101,110,116,97,108,0,100,101,108,101,116,101,0,112,101,114,115,105,115,116,0,111,102,102,0,119,97,108,0,97,112,112,108,105,99,97,116,105,111,110,95,105,100,0,97,117,116,111,109,97,116,105,99,95,105,110,100,101,120,0,98,117,115,121,95,116,105,109,101,111,117,116,0,99,97,115,101,95,115,101,110,115,105,116,105,118,101,95,108,105,107,101,0,99,101,108,108,95,115,105,122,101,95,99,104,101,99,107,0,99,104,101,99,107,112,111,105,110,116,95,102,117,108,108,102,115,121,110,99,0,99,111,108,108,97,116,105,111,110,95,108,105,115,116,0,99,111,109,112,105,108,101,95,111,112,116,105,111,110,115,0,99,111,117,110,116,95,99,104,97,110,103,101,115,0,100,97,116,97,95,118,101,114,115,105,111,110,0,100,97,116,97,98,97,115,101,95,108,105,115,116,0,100,101,102,97,117,108,116,95,99,97,99,104,101,95,115,105,122,101,0,100,101,102,101,114,95,102,111,114,101,105,103,110,95,107,101,121,115,0,101,109,112,116,121,95,114,101,115,117,108,116,95,99,97,108,108,98,97,99,107,115,0,102,111,114,101,105,103,110,95,107,101,121,95,99,104,101,99,107,0,102,111,114,101,105,103,110,95,107,101,121,95,108,105,115,116,0,102,111,114,101,105,103,110,95,107,101,121,115,0,102,114,101,101,108,105,115,116,95,99,111,117,110,116,0,102,117,108,108,95,99,111,108,117,109,110,95,110,97,109,101,115,0,102,117,108,108,102,115,121,110,99,0,105,103,110,111,114,101,95,99,104,101,99,107,95,99,111,110,115,116,114,97,105,110,116,115,0,105,110,99,114,101,109,101,110,116,97,108,95,118,97,99,117,117,109,0,105,110,100,101,120,95,105,110,102,111,0,105,110,100,101,120,95,108,105,115,116,0,105,110,100,101,120,95,120,105,110,102,111,0,108,101,103,97,99,121,95,102,105,108,101,95,102,111,114,109,97,116,0,109,97,120,95,112,97,103,101,95,99,111,117,110,116,0,112,97,103,101,95,99,111,117,110,116,0,113,117,101,114,121,95,111,110,108,121,0,113,117,105,99,107,95,99,104,101,99,107,0,114,101,97,100,95,117,110,99,111,109,109,105,116,116,101,100,0,114,101,99,117,114,115,105,118,101,95,116,114,105,103,103,101,114,115,0,114,101,118,101,114,115,101,95,117,110,111,114,100,101,114,101,100,95,115,101,108,101,99,116,115,0,115,99,104,101,109,97,95,118,101,114,115,105,111,110,0,115,104,111,114,116,95,99,111,108,117,109,110,95,110,97,109,101,115,0,115,104,114,105,110,107,95,109,101,109,111,114,121,0,115,116,97,116,115,0,116,97,98,108,101,95,105,110,102,111,0,117,115,101,114,95,118,101,114,115,105,111,110,0,119,97,108,95,99,104,101,99,107,112,111,105,110,116,0,119,114,105,116,97,98,108,101,95,115,99,104,101,109,97,0,110,111,32,115,117,99,104,32,105,110,100,101,120,58,32,37,83,0,105,110,100,101,120,32,97,115,115,111,99,105,97,116,101,100,32,119,105,116,104,32,85,78,73,81,85,69,32,111,114,32,80,82,73,77,65,82,89,32,75,69,89,32,99,111,110,115,116,114,97,105,110,116,32,99,97,110,110,111,116,32,98,101,32,100,114,111,112,112,101,100,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,110,97,109,101,61,37,81,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,114,111,111,116,112,97,103,101,61,37,100,32,87,72,69,82,69,32,35,37,100,32,65,78,68,32,114,111,111,116,112,97,103,101,61,35,37,100,0,115,113,108,105,116,101,95,115,116,97,116,37,100,0,115,121,110,116,97,120,32,101,114,114,111,114,32,97,102,116,101,114,32,99,111,108,117,109,110,32,110,97,109,101,32,34,37,46,42,115,34,0,48,0,118,97,114,105,97,98,108,101,32,110,117,109,98,101,114,32,109,117,115,116,32,98,101,32,98,101,116,119,101,101,110,32,63,49,32,97,110,100,32,63,37,100,0,116,111,111,32,109,97,110,121,32,83,81,76,32,118,97,114,105,97,98,108,101,115,0,116,97,98,108,101,32,37,83,32,104,97,115,32,110,111,32,99,111,108,117,109,110,32,110,97,109,101,100,32,37,115,0,116,97,98,108,101,32,37,83,32,104,97,115,32,37,100,32,99,111,108,117,109,110,115,32,98,117,116,32,37,100,32,118,97,108,117,101,115,32,119,101,114,101,32,115,117,112,112,108,105,101,100,0,37,100,32,118,97,108,117,101,115,32,102,111,114,32,37,100,32,99,111,108,117,109,110,115,0,114,111,119,115,32,105,110,115,101,114,116,101,100,0,35,0,2,0,114,0,0,0,99,0,2,0,115,0,0,0,111,0,0,0,97,116,32,109,111,115,116,32,37,100,32,116,97,98,108,101,115,32,105,110,32,97,32,106,111,105,110,0,26,24,23,25,0,0,57,53,26,23,25,24,61,59,58,60,7,6,57,53,83,69,65,82,67,72,0,83,67,65,78,0,32,83,85,66,81,85,69,82,89,32,37,100,0,32,84,65,66,76,69,32,37,115,0,32,65,83,32,37,115,0,80,82,73,77,65,82,89,32,75,69,89,0,65,85,84,79,77,65,84,73,67,32,80,65,82,84,73,65,76,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,0,65,85,84,79,77,65,84,73,67,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,0,67,79,86,69,82,73,78,71,32,73,78,68,69,88,32,37,115,0,73,78,68,69,88,32,37,115,0,32,85,83,73,78,71,32,0,61,0,62,63,32,65,78,68,32,114,111,119,105,100,60,0,62,0,60,0,32,85,83,73,78,71,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,32,40,114,111,119,105,100,37,115,63,41,0,32,86,73,82,84,85,65,76,32,84,65,66,76,69,32,73,78,68,69,88,32,37,100,58,37,115,0,32,40,0,32,65,78,68,32,0,37,115,61,63,0,65,78,89,40,37,115,41,0,41,0,40,0,44,0,60,101,120,112,114,62,0,97,117,116,111,109,97,116,105,99,32,105,110,100,101,120,32,111,110,32,37,115,40,37,115,41,0,97,117,116,111,45,105,110,100,101,120,0,110,111,32,113,117,101,114,121,32,115,111,108,117,116,105,111,110,0,10,10,9,9,8,8,7,7,7,6,6,6,5,5,5,4,4,4,4,3,3,3,3,3,3,2,2,2,2,2,2,2,37,115,46,120,66,101,115,116,73,110,100,101,120,32,109,97,108,102,117,110,99,116,105,111,110,0,41,39,66,73,78,65,82,89,0,114,101,103,101,120,112,0,116,111,111,32,109,97,110,121,32,97,114,103,117,109,101,110,116,115,32,111,110,32,37,115,40,41,32,45,32,109,97,120,32,37,100,0,45,45,32,84,82,73,71,71,69,82,32,37,115,0,0,0,0,1,1,1,1,1,0,16,0,1,0,1,1,1,3,3,1,18,1,3,3,9,9,9,9,38,38,9,9,9,9,9,3,3,11,11,11,11,11,11,1,38,38,38,38,38,38,38,38,38,38,1,18,1,1,1,1,1,1,1,35,11,1,1,3,3,3,1,1,1,2,2,8,0,16,16,16,16,0,16,16,0,0,16,16,0,0,0,0,2,2,2,0,0,0,16,0,0,16,16,0,0,0,0,0,0,0,0,0,0,0,16,16,0,0,0,0,0,0,0,0,16,0,4,4,0,0,16,16,0,16,0,16,16,0,0,0,0,0,0,6,16,0,4,26,0,0,0,0,0,0,0,0,0,0,0,16,16,0,0,0,0,1,2,3,4,6,8,8,0,0,0,0,0,0,1,1,2,2,3,3,4,4,5,5,6,6,7,7,8,8,9,9,10,10,11,11,12,12,13,13,14,14,15,15,16,16,17,17,18,18,19,19,20,20,21,21,22,22,23,23,24,24,25,25,26,26,27,27,28,28,29,29,30,30,31,31,32,32,33,33,34,34,35,35,36,36,37,37,38,38,39,39,40,40,41,41,42,42,43,43,44,44,45,45,46,46,47,47,48,48,49,49,50,50,51,51,52,52,53,53,54,54,55,55,56,56,57,57,70,79,82,69,73,71,78,32,75,69,89,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,37,115,46,114,111,119,105,100,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,109,111,100,105,102,105,101,100,0,99,97,110,110,111,116,32,109,111,100,105,102,121,32,37,115,32,98,101,99,97,117,115,101,32,105,116,32,105,115,32,97,32,118,105,101,119,0,37,100,32,99,111,108,117,109,110,115,32,97,115,115,105,103,110,101,100,32,37,100,32,118,97,108,117,101,115,0,114,111,119,115,32,117,112,100,97,116,101,100,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,105,110,32,37,115,0,114,111,119,115,32,100,101,108,101,116,101,100,0,0,7,4,6,4,40,10,5,32,14,5,48,19,4,56,23,5,1,28,5,3,110,97,116,117,114,97,108,101,102,116,111,117,116,101,114,105,103,104,116,102,117,108,108,105,110,110,101,114,99,114,111,115,115,0,32,0,117,110,107,110,111,119,110,32,111,114,32,117,110,115,117,112,112,111,114,116,101,100,32,106,111,105,110,32,116,121,112,101,58,32,37,84,32,37,84,37,115,37,84,0,82,73,71,72,84,32,97,110,100,32,70,85,76,76,32,79,85,84,69,82,32,74,79,73,78,115,32,97,114,101,32,110,111,116,32,99,117,114,114,101,110,116,108,121,32,115,117,112,112,111,114,116,101,100,0,79,78,0,85,83,73,78,71,0,97,32,74,79,73,78,32,99,108,97,117,115,101,32,105,115,32,114,101,113,117,105,114,101,100,32,98,101,102,111,114,101,32,37,115,0,116,111,111,32,109,97,110,121,32,116,101,114,109,115,32,105,110,32,99,111,109,112,111,117,110,100,32,83,69,76,69,67,84,0,101,120,112,101,99,116,101,100,32,37,100,32,99,111,108,117,109,110,115,32,102,111,114,32,39,37,115,39,32,98,117,116,32,103,111,116,32,37,100,0,68,73,83,84,73,78,67,84,0,71,82,79,85,80,32,66,89,0,82,73,71,72,84,32,80,65,82,84,32,79,70,32,79,82,68,69,82,32,66,89,0,79,82,68,69,82,32,66,89,0,99,111,108,117,109,110,37,100,0,109,105,110,0,109,97,120,0,32,85,83,73,78,71,32,67,79,86,69,82,73,78,71,32,73,78,68,69,88,32,0,83,67,65,78,32,84,65,66,76,69,32,37,115,37,115,37,115,0,68,73,83,84,73,78,67,84,32,97,103,103,114,101,103,97,116,101,115,32,109,117,115,116,32,104,97,118,101,32,101,120,97,99,116,108,121,32,111,110,101,32,97,114,103,117,109,101,110,116,0,85,83,69,32,84,69,77,80,32,66,45,84,82,69,69,32,70,79,82,32,37,115,0,79,82,68,69,82,32,66,89,32,99,108,97,117,115,101,32,115,104,111,117,108,100,32,99,111,109,101,32,97,102,116,101,114,32,37,115,32,110,111,116,32,98,101,102,111,114,101,0,76,73,77,73,84,32,99,108,97,117,115,101,32,115,104,111,117,108,100,32,99,111,109,101,32,97,102,116,101,114,32,37,115,32,110,111,116,32,98,101,102,111,114,101,0,85,83,73,78,71,32,84,69,77,80,32,66,45,84,82,69,69,32,0,67,79,77,80,79,85,78,68,32,83,85,66,81,85,69,82,73,69,83,32,37,100,32,65,78,68,32,37,100,32,37,115,40,37,115,41,0,114,101,99,117,114,115,105,118,101,32,97,103,103,114,101,103,97,116,101,32,113,117,101,114,105,101,115,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,112,97,114,97,109,101,116,101,114,115,32,97,114,101,32,110,111,116,32,97,108,108,111,119,101,100,32,105,110,32,118,105,101,119,115,0,118,105,101,119,0,115,113,108,105,116,101,95,115,116,97,116,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,100,114,111,112,112,101,100,0,117,115,101,32,68,82,79,80,32,84,65,66,76,69,32,116,111,32,100,101,108,101,116,101,32,116,97,98,108,101,32,37,115,0,117,115,101,32,68,82,79,80,32,86,73,69,87,32,116,111,32,100,101,108,101,116,101,32,118,105,101,119,32,37,115,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,32,87,72,69,82,69,32,110,97,109,101,61,37,81,0,68,69,76,69,84,69,32,70,82,79,77,32,37,81,46,37,115,32,87,72,69,82,69,32,116,98,108,95,110,97,109,101,61,37,81,32,97,110,100,32,116,121,112,101,33,61,39,116,114,105,103,103,101,114,39,0,102,111,114,101,105,103,110,32,107,101,121,32,111,110,32,37,115,32,115,104,111,117,108,100,32,114,101,102,101,114,101,110,99,101,32,111,110,108,121,32,111,110,101,32,99,111,108,117,109,110,32,111,102,32,116,97,98,108,101,32,37,84,0,110,117,109,98,101,114,32,111,102,32,99,111,108,117,109,110,115,32,105,110,32,102,111,114,101,105,103,110,32,107,101,121,32,100,111,101,115,32,110,111,116,32,109,97,116,99,104,32,116,104,101,32,110,117,109,98,101,114,32,111,102,32,99,111,108,117,109,110,115,32,105,110,32,116,104,101,32,114,101,102,101,114,101,110,99,101,100,32,116,97,98,108,101,0,117,110,107,110,111,119,110,32,99,111,108,117,109,110,32,34,37,115,34,32,105,110,32,102,111,114,101,105,103,110,32,107,101,121,32,100,101,102,105,110,105,116,105,111,110,0,99,97,110,110,111,116,32,99,114,101,97,116,101,32,97,32,84,69,77,80,32,105,110,100,101,120,32,111,110,32,110,111,110,45,84,69,77,80,32,116,97,98,108,101,32,34,37,115,34,0,97,108,116,101,114,116,97,98,95,0,116,97,98,108,101,32,37,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,118,105,101,119,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,118,105,114,116,117,97,108,32,116,97,98,108,101,115,32,109,97,121,32,110,111,116,32,98,101,32,105,110,100,101,120,101,100,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,32,116,97,98,108,101,32,110,97,109,101,100,32,37,115,0,105,110,100,101,120,32,37,115,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,115,113,108,105,116,101,95,97,117,116,111,105,110,100,101,120,95,37,115,95,37,100,0,101,120,112,114,101,115,115,105,111,110,115,32,112,114,111,104,105,98,105,116,101,100,32,105,110,32,80,82,73,77,65,82,89,32,75,69,89,32,97,110,100,32,85,78,73,81,85,69,32,99,111,110,115,116,114,97,105,110,116,115,0,99,111,110,102,108,105,99,116,105,110,103,32,79,78,32,67,79,78,70,76,73,67,84,32,99,108,97,117,115,101,115,32,115,112,101,99,105,102,105,101,100,0,32,85,78,73,81,85,69,0,67,82,69,65,84,69,37,115,32,73,78,68,69,88,32,37,46,42,115,0,73,78,83,69,82,84,32,73,78,84,79,32,37,81,46,37,115,32,86,65,76,85,69,83,40,39,105,110,100,101,120,39,44,37,81,44,37,81,44,35,37,100,44,37,81,41,59,0,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,61,39,105,110,100,101,120,39,0,116,97,98,108,101,32,34,37,115,34,32,104,97,115,32,109,111,114,101,32,116,104,97,110,32,111,110,101,32,112,114,105,109,97,114,121,32,107,101,121,0,65,85,84,79,73,78,67,82,69,77,69,78,84,32,105,115,32,111,110,108,121,32,97,108,108,111,119,101,100,32,111,110,32,97,110,32,73,78,84,69,71,69,82,32,80,82,73,77,65,82,89,32,75,69,89,0,100,101,102,97,117,108,116,32,118,97,108,117,101,32,111,102,32,99,111,108,117,109,110,32,91,37,115,93,32,105,115,32,110,111,116,32,99,111,110,115,116,97,110,116,0,116,111,111,32,109,97,110,121,32,99,111,108,117,109,110,115,32,111,110,32,37,115,0,100,117,112,108,105,99,97,116,101,32,99,111,108,117,109,110,32,110,97,109,101,58,32,37,115,0,65,85,84,79,73,78,67,82,69,77,69,78,84,32,110,111,116,32,97,108,108,111,119,101,100,32,111,110,32,87,73,84,72,79,85,84,32,82,79,87,73,68,32,116,97,98,108,101,115,0,80,82,73,77,65,82,89,32,75,69,89,32,109,105,115,115,105,110,103,32,111,110,32,116,97,98,108,101,32,37,115,0,84,65,66,76,69,0,86,73,69,87,0,67,82,69,65,84,69,32,37,115,32,37,46,42,115,0,85,80,68,65,84,69,32,37,81,46,37,115,32,83,69,84,32,116,121,112,101,61,39,37,115,39,44,32,110,97,109,101,61,37,81,44,32,116,98,108,95,110,97,109,101,61,37,81,44,32,114,111,111,116,112,97,103,101,61,35,37,100,44,32,115,113,108,61,37,81,32,87,72,69,82,69,32,114,111,119,105,100,61,35,37,100,0,67,82,69,65,84,69,32,84,65,66,76,69,32,37,81,46,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,40,110,97,109,101,44,115,101,113,41,0,116,98,108,95,110,97,109,101,61,39,37,113,39,32,65,78,68,32,116,121,112,101,33,61,39,116,114,105,103,103,101,114,39,0,10,32,32,0,44,10,32,32,0,10,41,0,67,82,69,65,84,69,32,84,65,66,76,69,32,0,32,84,69,88,84,0,32,78,85,77,0,32,73,78,84,0,32,82,69,65,76,0,116,101,109,112,111,114,97,114,121,32,116,97,98,108,101,32,110,97,109,101,32,109,117,115,116,32,98,101,32,117,110,113,117,97,108,105,102,105,101,100,0,2,4,8,6,116,97,98,108,101,32,37,84,32,97,108,114,101,97,100,121,32,101,120,105,115,116,115,0,116,104,101,114,101,32,105,115,32,97,108,114,101,97,100,121,32,97,110,32,105,110,100,101,120,32,110,97,109,101,100,32,37,115,0,6,0,0,0,0,0,66,69,71,73,78,0,82,69,76,69,65,83,69,0,82,79,76,76,66,65,67,75,0,67,79,77,77,73,84,0,79,0,0,0,57,0,9,0,96,0,0,0,36,0,7,0,123,0,0,0,96,0,1,0,13,0,9,0,7,0,2,0,76,0,0,0,111,0,0,0,112,97,114,115,101,114,32,115,116,97,99,107,32,111,118,101,114,102,108,111,119,0,19,95,53,97,22,24,24,101,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,19,55,55,132,133,134,1,2,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,187,43,44,45,46,47,48,49,50,51,52,53,47,48,49,50,51,52,53,61,97,97,19,49,50,51,52,53,70,26,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,144,145,146,147,19,16,22,92,172,173,52,53,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,81,43,44,45,46,47,48,49,50,51,52,53,55,56,19,152,207,208,115,24,117,118,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,79,43,44,45,46,47,48,49,50,51,52,53,19,88,157,90,23,97,98,193,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,19,22,23,172,23,26,119,120,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,187,43,44,45,46,47,48,49,50,51,52,53,19,22,23,228,23,26,231,152,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,172,43,44,45,46,47,48,49,50,51,52,53,19,221,222,223,23,96,152,172,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,19,0,1,2,23,96,190,191,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,238,43,44,45,46,47,48,49,50,51,52,53,19,185,218,221,222,223,152,152,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,241,43,44,45,46,47,48,49,50,51,52,53,19,152,168,169,170,22,190,191,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,19,19,218,55,56,24,22,152,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,250,194,195,56,55,56,55,19,172,173,97,98,152,206,138,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,19,207,208,152,97,98,97,138,27,28,29,30,31,32,33,34,35,36,37,38,39,40,41,181,43,44,45,46,47,48,49,50,51,52,53,19,30,31,32,33,247,248,19,152,28,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,19,168,169,170,238,19,53,152,172,173,29,30,31,32,33,34,35,36,37,38,39,40,41,152,43,44,45,46,47,48,49,50,51,52,53,19,20,101,22,23,169,170,56,207,85,55,56,23,19,20,26,22,99,100,101,102,103,104,105,238,152,152,210,47,48,112,152,108,109,110,54,55,56,221,222,223,47,48,119,120,172,173,66,54,55,56,152,97,98,99,148,149,102,103,104,66,154,23,156,83,26,230,152,113,152,163,194,195,92,92,30,95,83,97,98,207,208,101,206,179,180,92,172,173,95,152,97,98,188,99,101,219,102,103,104,152,119,120,196,55,56,19,20,113,22,193,163,11,132,133,134,135,136,24,65,172,173,207,208,250,152,132,133,134,135,136,193,78,84,47,48,49,98,199,152,86,54,55,56,196,152,97,98,209,55,163,244,107,66,152,207,208,164,175,172,173,19,20,124,22,111,38,39,40,41,83,43,44,45,46,47,48,49,50,51,52,53,95,196,97,98,85,152,101,47,48,97,85,92,207,193,54,55,56,92,49,175,55,56,221,222,223,12,66,108,109,110,137,163,139,108,109,110,26,132,133,134,135,136,152,83,43,44,45,46,47,48,49,50,51,52,53,95,26,97,98,55,56,101,97,98,196,221,222,223,146,147,57,171,152,22,26,19,20,49,22,179,108,109,110,55,56,116,73,219,75,124,121,152,132,133,134,135,136,163,85,152,232,97,98,47,48,237,55,56,98,5,54,55,56,193,10,11,12,13,14,172,173,17,66,47,48,97,98,152,124,152,196,55,56,186,124,152,106,160,152,83,152,164,152,61,22,211,212,152,97,98,152,95,70,97,98,172,173,101,172,173,172,173,172,173,60,181,62,172,173,47,48,123,186,97,98,71,100,55,56,152,181,186,21,107,152,109,82,163,132,133,134,135,136,89,16,207,92,93,19,172,173,169,170,195,55,56,12,152,132,30,134,47,48,186,206,225,152,95,114,97,196,245,246,101,152,38,39,40,41,42,43,44,45,46,47,48,49,50,51,52,53,152,163,219,152,141,97,98,193,152,152,57,91,164,132,133,134,152,55,152,152,237,230,152,103,193,88,73,90,75,172,173,183,152,185,196,152,172,173,172,173,217,152,172,173,152,107,22,152,24,193,112,152,172,173,152,132,242,134,152,97,140,152,92,152,172,173,152,172,173,152,100,172,173,152,172,173,152,140,172,173,152,172,173,172,173,152,172,173,152,172,173,152,152,172,173,152,172,173,213,152,172,173,152,152,152,172,173,152,172,173,152,172,173,152,210,172,173,152,26,172,173,152,172,173,172,173,152,172,173,152,172,173,152,172,173,152,59,172,173,152,63,172,173,152,193,152,152,152,152,172,173,152,172,173,77,172,173,152,152,172,173,152,152,172,173,172,173,172,173,152,22,172,173,152,152,152,22,172,173,152,152,152,172,173,152,7,8,9,163,172,173,22,23,172,173,172,173,166,167,172,173,172,173,55,172,173,22,23,108,109,110,217,152,217,166,167,163,163,163,163,163,196,130,217,211,212,217,116,23,22,101,26,121,23,23,23,26,26,26,23,23,112,26,26,37,97,100,101,55,196,196,196,196,196,23,23,55,26,26,7,8,23,152,23,26,96,26,132,132,134,134,23,152,152,26,152,122,152,191,152,96,234,152,152,152,152,152,197,210,152,97,152,152,210,233,210,198,150,97,184,201,239,214,214,201,239,180,214,227,200,198,155,67,243,176,69,175,175,175,122,159,159,240,159,240,22,220,27,130,201,18,159,18,189,158,158,220,192,159,137,236,192,192,192,189,74,189,159,235,159,158,22,177,201,201,159,107,158,177,159,174,158,76,174,182,174,106,182,125,174,107,177,22,159,216,215,137,159,53,216,176,215,174,174,216,215,215,174,229,216,129,224,177,126,229,127,177,128,25,162,226,26,161,13,153,6,153,151,151,151,151,205,165,178,178,165,4,3,22,165,142,15,94,202,204,203,201,16,23,249,23,120,249,246,111,131,123,20,16,1,125,123,111,56,64,37,37,131,122,1,37,5,37,22,107,26,80,140,80,87,72,107,20,24,19,112,105,23,79,22,79,22,22,22,58,22,79,23,68,23,23,26,116,22,26,23,22,122,23,23,56,64,22,124,26,26,64,64,23,23,23,23,11,23,22,26,23,22,24,1,23,22,26,251,24,23,22,122,23,23,22,15,122,122,122,23,0,0,55,55,55,55,0,55,55,55,0,55,55,55,55,0,0,0,55,0,0,55,0,0,0,55,0,0,0,0,55,55,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,55,27,27,27,27,27,27,27,27,27,7,7,27,7,7,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,27,7,15,8,5,4,22,24,8,17,18,21,20,23,11,26,16,3,3,3,3,3,3,3,3,3,3,5,19,12,14,13,6,5,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,9,27,27,27,1,8,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0,1,1,27,10,27,25,27,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,105,110,118,97,108,105,100,0,65,80,73,32,99,97,108,108,32,119,105,116,104,32,37,115,32,100,97,116,97,98,97,115,101,32,99,111,110,110,101,99,116,105,111,110,32,112,111,105,110,116,101,114,0,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,105,115,32,108,111,99,107,101,100,58,32,37,115,0,115,116,97,116,101,109,101,110,116,32,116,111,111,32,108,111,110,103,0,97,100,100,114,0,111,112,99,111,100,101,0,112,49,0,112,50,0,112,51,0,112,52,0,112,53,0,99,111,109,109,101,110,116,0,115,101,108,101,99,116,105,100,0,100,101,116,97,105,108,0,78,85,76,76,0,117,110,111,112,101,110,101,100,0,114,101,97,108,0,105,110,116,101,103,101,114,0,110,117,108,108,0,99,97,110,110,111,116,32,111,112,101,110,32,118,97,108,117,101,32,111,102,32,116,121,112,101,32,37,115,0,110,111,32,115,117,99,104,32,114,111,119,105,100,58,32,37,108,108,100,0,110,111,32,115,117,99,104,32,118,105,101,119,0,110,111,32,115,117,99,104,32,116,97,98,108,101,0,45,45,32,0,39,37,46,42,113,39,0,122,101,114,111,98,108,111,98,40,37,100,41,0,120,39,0,37,48,50,120,0,39,0,98,105,110,100,32,111,110,32,97,32,98,117,115,121,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,58,32,91,37,115,93,0,37,115,32,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,37,122,58,32,37,115,0,97,98,111,114,116,32,97,116,32,37,100,32,105,110,32,91,37,115,93,58,32,37,115,0,0,0,0,0,1,2,0,2,2,0,1,2,1,1,1,2,1,2,0,0,0,0,0,0,0,0,99,97,110,110,111,116,32,111,112,101,110,32,115,97,118,101,112,111,105,110,116,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,110,111,32,115,117,99,104,32,115,97,118,101,112,111,105,110,116,58,32,37,115,0,99,97,110,110,111,116,32,114,101,108,101,97,115,101,32,115,97,118,101,112,111,105,110,116,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,99,97,110,110,111,116,32,99,111,109,109,105,116,32,116,114,97,110,115,97,99,116,105,111,110,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,99,97,110,110,111,116,32,114,111,108,108,98,97,99,107,32,45,32,110,111,32,116,114,97,110,115,97,99,116,105,111,110,32,105,115,32,97,99,116,105,118,101,0,99,97,110,110,111,116,32,99,111,109,109,105,116,32,45,32,110,111,32,116,114,97,110,115,97,99,116,105,111,110,32,105,115,32,97,99,116,105,118,101,0,99,97,110,110,111,116,32,115,116,97,114,116,32,97,32,116,114,97,110,115,97,99,116,105,111,110,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,100,97,116,97,98,97,115,101,32,115,99,104,101,109,97,32,104,97,115,32,99,104,97,110,103,101,100,0,83,69,76,69,67,84,32,110,97,109,101,44,32,114,111,111,116,112,97,103,101,44,32,115,113,108,32,70,82,79,77,32,39,37,113,39,46,37,115,32,87,72,69,82,69,32,37,115,32,79,82,68,69,82,32,66,89,32,114,111,119,105,100,0,116,111,111,32,109,97,110,121,32,108,101,118,101,108,115,32,111,102,32,116,114,105,103,103,101,114,32,114,101,99,117,114,115,105,111,110,0,105,110,116,111,0,111,117,116,32,111,102,0,99,97,110,110,111,116,32,99,104,97,110,103,101,32,37,115,32,119,97,108,32,109,111,100,101,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,100,97,116,97,98,97,115,101,32,116,97,98,108,101,32,105,115,32,108,111,99,107,101,100,58,32,37,115,0,115,116,97,116,101,109,101,110,116,32,97,98,111,114,116,115,32,97,116,32,37,100,58,32,91,37,115,93,32,37,115,0,99,97,110,110,111,116,32,86,65,67,85,85,77,32,102,114,111,109,32,119,105,116,104,105,110,32,97,32,116,114,97,110,115,97,99,116,105,111,110,0,99,97,110,110,111,116,32,86,65,67,85,85,77,32,45,32,83,81,76,32,115,116,97,116,101,109,101,110,116,115,32,105,110,32,112,114,111,103,114,101,115,115,0,65,84,84,65,67,72,39,39,65,83,32,118,97,99,117,117,109,95,100,98,0,83,69,76,69,67,84,32,115,113,108,32,70,82,79,77,32,34,37,119,34,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,61,39,116,97,98,108,101,39,65,78,68,32,110,97,109,101,60,62,39,115,113,108,105,116,101,95,115,101,113,117,101,110,99,101,39,32,65,78,68,32,99,111,97,108,101,115,99,101,40,114,111,111,116,112,97,103,101,44,49,41,62,48,0,83,69,76,69,67,84,32,115,113,108,32,70,82,79,77,32,34,37,119,34,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,61,39,105,110,100,101,120,39,32,65,78,68,32,108,101,110,103,116,104,40,115,113,108,41,62,49,48,0,83,69,76,69,67,84,39,73,78,83,69,82,84,32,73,78,84,79,32,118,97,99,117,117,109,95,100,98,46,39,124,124,113,117,111,116,101,40,110,97,109,101,41,124,124,39,32,83,69,76,69,67,84,42,70,82,79,77,34,37,119,34,46,39,124,124,113,117,111,116,101,40,110,97,109,101,41,70,82,79,77,32,118,97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,61,39,116,97,98,108,101,39,65,78,68,32,99,111,97,108,101,115,99,101,40,114,111,111,116,112,97,103,101,44,49,41,62,48,0,73,78,83,69,82,84,32,73,78,84,79,32,118], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+30720);
/* memory initializer */ allocate([97,99,117,117,109,95,100,98,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,83,69,76,69,67,84,42,70,82,79,77,32,34,37,119,34,46,115,113,108,105,116,101,95,109,97,115,116,101,114,32,87,72,69,82,69,32,116,121,112,101,32,73,78,40,39,118,105,101,119,39,44,39,116,114,105,103,103,101,114,39,41,32,79,82,40,116,121,112,101,61,39,116,97,98,108,101,39,65,78,68,32,114,111,111,116,112,97,103,101,61,48,41,0,1,1,3,0,5,0,6,0,8,0,77,97,105,110,32,102,114,101,101,108,105,115,116,58,32,0,80,97,103,101,32,37,100,32,105,115,32,110,101,118,101,114,32,117,115,101,100,0,80,111,105,110,116,101,114,32,109,97,112,32,112,97,103,101,32,37,100,32,105,115,32,114,101,102,101,114,101,110,99,101,100,0,10,0,80,97,103,101,32,37,100,58,32,0,117,110,97,98,108,101,32,116,111,32,103,101,116,32,116,104,101,32,112,97,103,101,46,32,101,114,114,111,114,32,99,111,100,101,61,37,100,0,98,116,114,101,101,73,110,105,116,80,97,103,101,40,41,32,114,101,116,117,114,110,115,32,101,114,114,111,114,32,99,111,100,101,32,37,100,0,79,110,32,116,114,101,101,32,112,97,103,101,32,37,100,32,99,101,108,108,32,37,100,58,32,0,79,110,32,112,97,103,101,32,37,100,32,97,116,32,114,105,103,104,116,32,99,104,105,108,100,58,32,0,79,102,102,115,101,116,32,37,100,32,111,117,116,32,111,102,32,114,97,110,103,101,32,37,100,46,46,37,100,0,69,120,116,101,110,100,115,32,111,102,102,32,101,110,100,32,111,102,32,112,97,103,101,0,82,111,119,105,100,32,37,108,108,100,32,111,117,116,32,111,102,32,111,114,100,101,114,0,67,104,105,108,100,32,112,97,103,101,32,100,101,112,116,104,32,100,105,102,102,101,114,115,0,77,117,108,116,105,112,108,101,32,117,115,101,115,32,102,111,114,32,98,121,116,101,32,37,117,32,111,102,32,112,97,103,101,32,37,100,0,70,114,97,103,109,101,110,116,97,116,105,111,110,32,111,102,32,37,100,32,98,121,116,101,115,32,114,101,112,111,114,116,101,100,32,97,115,32,37,100,32,111,110,32,112,97,103,101,32,37,100,0,105,110,118,97,108,105,100,32,112,97,103,101,32,110,117,109,98,101,114,32,37,100,0,50,110,100,32,114,101,102,101,114,101,110,99,101,32,116,111,32,112,97,103,101,32,37,100,0,70,97,105,108,101,100,32,116,111,32,114,101,97,100,32,112,116,114,109,97,112,32,107,101,121,61,37,100,0,66,97,100,32,112,116,114,32,109,97,112,32,101,110,116,114,121,32,107,101,121,61,37,100,32,101,120,112,101,99,116,101,100,61,40,37,100,44,37,100,41,32,103,111,116,61,40,37,100,44,37,100,41,0,37,100,32,111,102,32,37,100,32,112,97,103,101,115,32,109,105,115,115,105,110,103,32,102,114,111,109,32,111,118,101,114,102,108,111,119,32,108,105,115,116,32,115,116,97,114,116,105,110,103,32,97,116,32,37,100,0,102,97,105,108,101,100,32,116,111,32,103,101,116,32,112,97,103,101,32,37,100,0,102,114,101,101,108,105,115,116,32,108,101,97,102,32,99,111,117,110,116,32,116,111,111,32,98,105,103,32,111,110,32,112,97,103,101,32,37,100,0,102,114,101,101,45,112,97,103,101,32,99,111,117,110,116,32,105,110,32,104,101,97,100,101,114,32,105,115,32,116,111,111,32,115,109,97,108,108,0,0,1,2,3,4,6,8,83,81,76,105,116,101,32,102,111,114,109,97,116,32,51,0,1,1,0,1,0,0,37,115,45,109,106,88,88,88,88,88,88,57,88,88,122,0,77,74,32,100,101,108,101,116,101,58,32,37,115,0,77,74,32,99,111,108,108,105,100,101,58,32,37,115,0,45,109,106,37,48,54,88,57,37,48,50,88,0,78,79,84,32,78,85,76,76,0,85,78,73,81,85,69,0,67,72,69,67,75,0,70,79,82,69,73,71,78,32,75,69,89,0,37,46,50,120,0,107,40,37,100,0,66,0,44,37,115,37,115,0,40,37,46,50,48,115,41,0,37,115,40,37,100,41,0,37,100,0,37,46,49,54,103,0,40,98,108,111,98,41,0,118,116,97,98,58,37,112,0,44,37,100,0,93,0,112,114,111,103,114,97,109,0,83,97,118,101,112,111,105,110,116,0,65,117,116,111,67,111,109,109,105,116,0,84,114,97,110,115,97,99,116,105,111,110,0,83,111,114,116,101,114,78,101,120,116,0,80,114,101,118,73,102,79,112,101,110,0,78,101,120,116,73,102,79,112,101,110,0,80,114,101,118,0,78,101,120,116,0,67,104,101,99,107,112,111,105,110,116,0,74,111,117,114,110,97,108,77,111,100,101,0,86,97,99,117,117,109,0,86,70,105,108,116,101,114,0,86,85,112,100,97,116,101,0,71,111,116,111,0,71,111,115,117,98,0,73,110,105,116,67,111,114,111,117,116,105,110,101,0,89,105,101,108,100,0,77,117,115,116,66,101,73,110,116,0,74,117,109,112,0,78,111,116,0,79,110,99,101,0,73,102,0,73,102,78,111,116,0,83,101,101,107,76,84,0,83,101,101,107,76,69,0,83,101,101,107,71,69,0,83,101,101,107,71,84,0,79,114,0,65,110,100,0,78,111,67,111,110,102,108,105,99,116,0,78,111,116,70,111,117,110,100,0,70,111,117,110,100,0,83,101,101,107,82,111,119,105,100,0,78,111,116,69,120,105,115,116,115,0,73,115,78,117,108,108,0,78,111,116,78,117,108,108,0,78,101,0,69,113,0,71,116,0,76,101,0,76,116,0,71,101,0,69,108,115,101,78,111,116,69,113,0,66,105,116,65,110,100,0,66,105,116,79,114,0,83,104,105,102,116,76,101,102,116,0,83,104,105,102,116,82,105,103,104,116,0,65,100,100,0,83,117,98,116,114,97,99,116,0,77,117,108,116,105,112,108,121,0,68,105,118,105,100,101,0,82,101,109,97,105,110,100,101,114,0,67,111,110,99,97,116,0,76,97,115,116,0,66,105,116,78,111,116,0,83,111,114,116,101,114,83,111,114,116,0,83,111,114,116,0,82,101,119,105,110,100,0,73,100,120,76,69,0,73,100,120,71,84,0,73,100,120,76,84,0,73,100,120,71,69,0,82,111,119,83,101,116,82,101,97,100,0,82,111,119,83,101,116,84,101,115,116,0,80,114,111,103,114,97,109,0,70,107,73,102,90,101,114,111,0,73,102,80,111,115,0,73,102,78,111,116,90,101,114,111,0,68,101,99,114,74,117,109,112,90,101,114,111,0,73,110,99,114,86,97,99,117,117,109,0,86,78,101,120,116,0,73,110,105,116,0,82,101,116,117,114,110,0,69,110,100,67,111,114,111,117,116,105,110,101,0,72,97,108,116,73,102,78,117,108,108,0,72,97,108,116,0,73,110,116,101,103,101,114,0,73,110,116,54,52,0,83,116,114,105,110,103,0,78,117,108,108,0,83,111,102,116,78,117,108,108,0,66,108,111,98,0,86,97,114,105,97,98,108,101,0,77,111,118,101,0,67,111,112,121,0,83,67,111,112,121,0,73,110,116,67,111,112,121,0,82,101,115,117,108,116,82,111,119,0,67,111,108,108,83,101,113,0,70,117,110,99,116,105,111,110,48,0,70,117,110,99,116,105,111,110,0,65,100,100,73,109,109,0,82,101,97,108,65,102,102,105,110,105,116,121,0,67,97,115,116,0,80,101,114,109,117,116,97,116,105,111,110,0,67,111,109,112,97,114,101,0,67,111,108,117,109,110,0,83,116,114,105,110,103,56,0,65,102,102,105,110,105,116,121,0,77,97,107,101,82,101,99,111,114,100,0,67,111,117,110,116,0,82,101,97,100,67,111,111,107,105,101,0,83,101,116,67,111,111,107,105,101,0,82,101,111,112,101,110,73,100,120,0,79,112,101,110,82,101,97,100,0,79,112,101,110,87,114,105,116,101,0,79,112,101,110,65,117,116,111,105,110,100,101,120,0,79,112,101,110,69,112,104,101,109,101,114,97,108,0,83,111,114,116,101,114,79,112,101,110,0,83,101,113,117,101,110,99,101,84,101,115,116,0,79,112,101,110,80,115,101,117,100,111,0,67,108,111,115,101,0,67,111,108,117,109,110,115,85,115,101,100,0,83,101,113,117,101,110,99,101,0,78,101,119,82,111,119,105,100,0,73,110,115,101,114,116,0,73,110,115,101,114,116,73,110,116,0,68,101,108,101,116,101,0,82,101,115,101,116,67,111,117,110,116,0,83,111,114,116,101,114,67,111,109,112,97,114,101,0,83,111,114,116,101,114,68,97,116,97,0,82,111,119,75,101,121,0,82,111,119,68,97,116,97,0,82,111,119,105,100,0,78,117,108,108,82,111,119,0,83,111,114,116,101,114,73,110,115,101,114,116,0,73,100,120,73,110,115,101,114,116,0,73,100,120,68,101,108,101,116,101,0,83,101,101,107,0,73,100,120,82,111,119,105,100,0,68,101,115,116,114,111,121,0,67,108,101,97,114,0,82,101,97,108,0,82,101,115,101,116,83,111,114,116,101,114,0,67,114,101,97,116,101,73,110,100,101,120,0,67,114,101,97,116,101,84,97,98,108,101,0,80,97,114,115,101,83,99,104,101,109,97,0,76,111,97,100,65,110,97,108,121,115,105,115,0,68,114,111,112,84,97,98,108,101,0,68,114,111,112,73,110,100,101,120,0,68,114,111,112,84,114,105,103,103,101,114,0,73,110,116,101,103,114,105,116,121,67,107,0,82,111,119,83,101,116,65,100,100,0,80,97,114,97,109,0,70,107,67,111,117,110,116,101,114,0,77,101,109,77,97,120,0,79,102,102,115,101,116,76,105,109,105,116,0,65,103,103,83,116,101,112,48,0,65,103,103,83,116,101,112,0,65,103,103,70,105,110,97,108,0,69,120,112,105,114,101,0,84,97,98,108,101,76,111,99,107,0,86,66,101,103,105,110,0,86,67,114,101,97,116,101,0,86,68,101,115,116,114,111,121,0,86,79,112,101,110,0,86,67,111,108,117,109,110,0,86,82,101,110,97,109,101,0,80,97,103,101,99,111,117,110,116,0,77,97,120,80,103,99,110,116,0,67,117,114,115,111,114,72,105,110,116,0,78,111,111,112,0,69,120,112,108,97,105,110,0,65,80,73,32,99,97,108,108,101,100,32,119,105,116,104,32,78,85,76,76,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,0,117,110,107,110,111,119,110,32,101,114,114,111,114,0,97,98,111,114,116,32,100,117,101,32,116,111,32,82,79,76,76,66,65,67,75,0,110,111,116,32,97,110,32,101,114,114,111,114,0,83,81,76,32,108,111,103,105,99,32,101,114,114,111,114,32,111,114,32,109,105,115,115,105,110,103,32,100,97,116,97,98,97,115,101,0,97,99,99,101,115,115,32,112,101,114,109,105,115,115,105,111,110,32,100,101,110,105,101,100,0,99,97,108,108,98,97,99,107,32,114,101,113,117,101,115,116,101,100,32,113,117,101,114,121,32,97,98,111,114,116,0,100,97,116,97,98,97,115,101,32,105,115,32,108,111,99,107,101,100,0,100,97,116,97,98,97,115,101,32,116,97,98,108,101,32,105,115,32,108,111,99,107,101,100,0,97,116,116,101,109,112,116,32,116,111,32,119,114,105,116,101,32,97,32,114,101,97,100,111,110,108,121,32,100,97,116,97,98,97,115,101,0,105,110,116,101,114,114,117,112,116,101,100,0,100,105,115,107,32,73,47,79,32,101,114,114,111,114,0,100,97,116,97,98,97,115,101,32,100,105,115,107,32,105,109,97,103,101,32,105,115,32,109,97,108,102,111,114,109,101,100,0,117,110,107,110,111,119,110,32,111,112,101,114,97,116,105,111,110,0,100,97,116,97,98,97,115,101,32,111,114,32,100,105,115,107,32,105,115,32,102,117,108,108,0,117,110,97,98,108,101,32,116,111,32,111,112,101,110,32,100,97,116,97,98,97,115,101,32,102,105,108,101,0,108,111,99,107,105,110,103,32,112,114,111,116,111,99,111,108,0,116,97,98,108,101,32,99,111,110,116,97,105,110,115,32,110,111,32,100,97,116,97,0,99,111,110,115,116,114,97,105,110,116,32,102,97,105,108,101,100,0,100,97,116,97,116,121,112,101,32,109,105,115,109,97,116,99,104,0,108,105,98,114,97,114,121,32,114,111,117,116,105,110,101,32,99,97,108,108,101,100,32,111,117,116,32,111,102,32,115,101,113,117,101,110,99,101,0,108,97,114,103,101,32,102,105,108,101,32,115,117,112,112,111,114,116,32,105,115,32,100,105,115,97,98,108,101,100,0,97,117,116,104,111,114,105,122,97,116,105,111,110,32,100,101,110,105,101,100,0,97,117,120,105,108,105,97,114,121,32,100,97,116,97,98,97,115,101,32,102,111,114,109,97,116,32,101,114,114,111,114,0,98,105,110,100,32,111,114,32,99,111,108,117,109,110,32,105,110,100,101,120,32,111,117,116,32,111,102,32,114,97,110,103,101,0,102,105,108,101,32,105,115,32,101,110,99,114,121,112,116,101,100,32,111,114,32,105,115,32,110,111,116,32,97,32,100,97,116,97,98,97,115,101,0,65,80,73,32,99,97,108,108,101,100,32,119,105,116,104,32,102,105,110,97,108,105,122,101,100,32,112,114,101,112,97,114,101,100,32,115,116,97,116,101,109,101,110,116,0,64,32,32,0,117,110,105,120,0,117,110,105,120,45,110,111,110,101,0,117,110,105,120,45,100,111,116,102,105,108,101,0,117,110,105,120,45,101,120,99,108,0,99,108,111,115,101,0,111,115,95,117,110,105,120,46,99,58,37,100,58,32,40,37,100,41,32,37,115,40,37,115,41,32,45,32,37,115,0,111,112,101,110,0,103,101,116,99,119,100,0,115,116,97,116,0,102,115,116,97,116,0,102,116,114,117,110,99,97,116,101,0,102,99,110,116,108,0,114,101,97,100,0,112,114,101,97,100,0,112,114,101,97,100,54,52,0,119,114,105,116,101,0,112,119,114,105,116,101,0,112,119,114,105,116,101,54,52,0,102,99,104,109,111,100,0,102,97,108,108,111,99,97,116,101,0,117,110,108,105,110,107,0,111,112,101,110,68,105,114,101,99,116,111,114,121,0,109,107,100,105,114,0,114,109,100,105,114,0,102,99,104,111,119,110,0,103,101,116,101,117,105,100,0,109,109,97,112,0,109,117,110,109,97,112,0,109,114,101,109,97,112,0,103,101,116,112,97,103,101,115,105,122,101,0,114,101,97,100,108,105,110,107,0,108,115,116,97,116,0,97,116,116,101,109,112,116,32,116,111,32,111,112,101,110,32,34,37,115,34,32,97,115,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,32,37,100,0,47,100,101,118,47,110,117,108,108,0,37,115,47,101,116,105,108,113,115,95,37,108,108,120,37,99,0,83,81,76,73,84,69,95,84,77,80,68,73,82,0,84,77,80,68,73,82,0,47,118,97,114,47,116,109,112,0,47,117,115,114,47,116,109,112,0,47,116,109,112,0,46,0,102,117,108,108,95,102,115,121,110,99,0,47,100,101,118,47,117,114,97,110,100,111,109,0,102,115,121,110,99,0,112,115,111,119,0,37,115,46,108,111,99,107,0,99,97,110,110,111,116,32,102,115,116,97,116,32,100,98,32,102,105,108,101,32,37,115,0,102,105,108,101,32,117,110,108,105,110,107,101,100,32,119,104,105,108,101,32,111,112,101,110,58,32,37,115,0,109,117,108,116,105,112,108,101,32,108,105,110,107,115,32,116,111,32,102,105,108,101,58,32,37,115,0,102,105,108,101,32,114,101,110,97,109,101,100,32,119,104,105,108,101,32,111,112,101,110,58,32,37,115,0,37,115,45,115,104,109,0,114,101,97,100,111,110,108,121,95,115,104,109,0,109,111,100,101,111,102,0,37,0,100,10,1,0,0,0,115,0,4,5,0,0,103,0,1,3,30,0,122,0,4,6,0,0,113,0,4,9,0,0,81,0,4,10,0,0,119,0,4,14,0,0,99,0,0,8,0,0,111,8,0,0,0,2,117,10,0,0,0,0,120,16,0,0,16,1,88,16,0,0,0,4,102,0,1,1,0,0,101,0,1,2,30,0,69,0,1,2,14,0,71,0,1,3,14,0,105,10,1,0,0,0,110,0,0,4,0,0,37,0,0,7,0,0,112,16,0,13,0,1,84,0,2,11,0,0,83,0,2,12,0,0,114,10,3,15,0,0,116,104,115,116,110,100,114,100,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,48,49,50,51,52,53,54,55,56,57,97,98,99,100,101,102,0,45,120,48,0,88,48,0,78,97,78,0,73,110,102,0,40,78,85,76,76,41,0,115,113,108,105,116,101,95,99,111,109,112,105,108,101,111,112,116,105,111,110,95,117,115,101,100,0,115,113,108,105,116,101,95,99,111,109,112,105,108,101,111,112,116,105,111,110,95,103,101,116,0,117,110,108,105,107,101,108,121,0,108,105,107,101,108,105,104,111,111,100,0,108,105,107,101,108,121,0,108,116,114,105,109,0,114,116,114,105,109,0,116,114,105,109,0,116,121,112,101,111,102,0,108,101,110,103,116,104,0,105,110,115,116,114,0,112,114,105,110,116,102,0,117,110,105,99,111,100,101,0,99,104,97,114,0,97,98,115,0,114,111,117,110,100,0,117,112,112,101,114,0,108,111,119,101,114,0,104,101,120,0,105,102,110,117,108,108,0,114,97,110,100,111,109,0,114,97,110,100,111,109,98,108,111,98,0,110,117,108,108,105,102,0,115,113,108,105,116,101,95,118,101,114,115,105,111,110,0,115,113,108,105,116,101,95,115,111,117,114,99,101,95,105,100,0,115,113,108,105,116,101,95,108,111,103,0,113,117,111,116,101,0,108,97,115,116,95,105,110,115,101,114,116,95,114,111,119,105,100,0,99,104,97,110,103,101,115,0,116,111,116,97,108,95,99,104,97,110,103,101,115,0,114,101,112,108,97,99,101,0,122,101,114,111,98,108,111,98,0,115,117,98,115,116,114,0,115,117,109,0,116,111,116,97,108,0,97,118,103,0,99,111,117,110,116,0,103,114,111,117,112,95,99,111,110,99,97,116,0,99,111,97,108,101,115,99,101,0,105,110,116,101,103,101,114,32,111,118,101,114,102,108,111,119,0,37,33,46,50,48,101,0,48,49,50,51,52,53,54,55,56,57,65,66,67,68,69,70,37,46,42,102,0,116,101,120,116,0,98,108,111,98,0,1,106,117,108,105,97,110,100,97,121,0,100,97,116,101,0,116,105,109,101,0,100,97,116,101,116,105,109,101,0,115,116,114,102,116,105,109,101,0,99,117,114,114,101,110,116,95,116,105,109,101,0,99,117,114,114,101,110,116,95,116,105,109,101,115,116,97,109,112,0,99,117,114,114,101,110,116,95,100,97,116,101,0,37,48,50,100,0,37,48,54,46,51,102,0,37,48,51,100,0,37,48,52,100,0,108,111,99,97,108,116,105,109,101,0,117,110,105,120,101,112,111,99,104,0,117,116,99,0,119,101,101,107,100,97,121,32,0,115,116,97,114,116,32,111,102,32,0,109,111,110,116,104,0,121,101,97,114,0,100,97,121,0,104,111,117,114,0,109,105,110,117,116,101,0,115,101,99,111,110,100,0,50,48,99,58,50,48,101,0,50,48,101,0,50,48,98,58,50,48,101,0,108,111,99,97,108,32,116,105,109,101,32,117,110,97,118,97,105,108,97,98,108,101,0,110,111,119,0,52,48,102,45,50,49,97,45,50,49,100,0,37,48,52,100,45,37,48,50,100,45,37,48,50,100,32,37,48,50,100,58,37,48,50,100,58,37,48,50,100,0,37,48,50,100,58,37,48,50,100,58,37,48,50,100,0,37,48,52,100,45,37,48,50,100,45,37,48,50,100,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,97,98,108,101,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,116,114,105,103,103,101,114,0,115,113,108,105,116,101,95,114,101,110,97,109,101,95,112,97,114,101,110,116,0,37,115,37,46,42,115,34,37,119,34,0,37,46,42,115,34,37,119,34,37,115,0,109,105,115,117,115,101,0,67,79,77,80,73,76,69,82,61,99,108,97,110,103,45,51,46,55,46,48,0,68,73,83,65,66,76,69,95,76,70,83,0,69,78,65,66,76,69,95,70,84,83,51,0,69,78,65,66,76,69,95,70,84,83,51,95,80,65,82,69,78,84,72,69,83,73,83,0,73,78,84,54,52,95,84,89,80,69,0,79,77,73,84,95,76,79,65,68,95,69,88,84,69,78,83,73,79,78,0,83,89,83,84,69,77,95,77,65,76,76,79,67,0,84,72,82,69,65,68,83,65,70,69,61,48,0,84,33,34,25,13,1,2,3,17,75,28,12,16,4,11,29,18,30,39,104,110,111,112,113,98,32,5,6,15,19,20,21,26,8,22,7,40,36,23,24,9,10,14,27,31,37,35,131,130,125,38,42,43,60,61,62,63,67,71,74,77,88,89,90,91,92,93,94,95,96,97,99,100,101,102,103,105,106,107,108,114,115,116,121,122,123,124,0,73,108,108,101,103,97,108,32,98,121,116,101,32,115,101,113,117,101,110,99,101,0,68,111,109,97,105,110,32,101,114,114,111,114,0,82,101,115,117,108,116,32,110,111,116,32,114,101,112,114,101,115,101,110,116,97,98,108,101,0,78,111,116,32,97,32,116,116,121,0,80,101,114,109,105,115,115,105,111,110,32,100,101,110,105,101,100,0,79,112,101,114,97,116,105,111,110,32,110,111,116,32,112,101,114,109,105,116,116,101,100,0,78,111,32,115,117,99,104,32,102,105,108,101,32,111,114,32,100,105,114,101,99,116,111,114,121,0,78,111,32,115,117,99,104,32,112,114,111,99,101,115,115,0,70,105,108,101,32,101,120,105,115,116,115,0,86,97,108,117,101,32,116,111,111,32,108,97,114,103,101,32,102,111,114,32,100,97,116,97,32,116,121,112,101,0,78,111,32,115,112,97,99,101,32,108,101,102,116,32,111,110,32,100,101,118,105,99,101,0,79,117,116,32,111,102,32,109,101,109,111,114,121,0,82,101,115,111,117,114,99,101,32,98,117,115,121,0,73,110,116,101,114,114,117,112,116,101,100,32,115,121,115,116,101,109,32,99,97,108,108,0,82,101,115,111,117,114,99,101,32,116,101,109,112,111,114,97,114,105,108,121,32,117,110,97,118,97,105,108,97,98,108,101,0,73,110,118,97,108,105,100,32,115,101,101,107,0,67,114,111,115,115,45,100,101,118,105,99,101,32,108,105,110,107,0,82,101,97,100,45,111,110,108,121,32,102,105,108,101,32,115,121,115,116,101,109,0,68,105,114,101,99,116,111,114,121,32,110,111,116,32,101,109,112,116,121,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,112,101,101,114,0,79,112,101,114,97,116,105,111,110,32,116,105,109,101,100,32,111,117,116,0,67,111,110,110,101,99,116,105,111,110,32,114,101,102,117,115,101,100,0,72,111,115,116,32,105,115,32,100,111,119,110,0,72,111,115,116,32,105,115,32,117,110,114,101,97,99,104,97,98,108,101,0,65,100,100,114,101,115,115,32,105,110,32,117,115,101,0,66,114,111,107,101,110,32,112,105,112,101,0,73,47,79,32,101,114,114,111,114,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,32,111,114,32,97,100,100,114,101,115,115,0,66,108,111,99,107,32,100,101,118,105,99,101,32,114,101,113,117,105,114,101,100,0,78,111,32,115,117,99,104,32,100,101,118,105,99,101,0,78,111,116,32,97,32,100,105,114,101,99,116,111,114,121,0,73,115,32,97,32,100,105,114,101,99,116,111,114,121,0,84,101,120,116,32,102,105,108,101,32,98,117,115,121,0,69,120,101,99,32,102,111,114,109,97,116,32,101,114,114,111,114,0,73,110,118,97,108,105,100,32,97,114,103,117,109,101,110,116,0,65,114,103,117,109,101,110,116,32,108,105,115,116,32,116,111,111,32,108,111,110,103,0,83,121,109,98,111,108,105,99,32,108,105,110,107,32,108,111,111,112,0,70,105,108,101,110,97,109,101,32,116,111,111,32,108,111,110,103,0,84,111,111,32,109,97,110,121,32,111,112,101,110,32,102,105,108,101,115,32,105,110,32,115,121,115,116,101,109,0,78,111,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,115,32,97,118,97,105,108,97,98,108,101,0,66,97,100,32,102,105,108,101,32,100,101,115,99,114,105,112,116,111,114,0,78,111,32,99,104,105,108,100,32,112,114,111,99,101,115,115,0,66,97,100,32,97,100,100,114,101,115,115,0,70,105,108,101,32,116,111,111,32,108,97,114,103,101,0,84,111,111,32,109,97,110,121,32,108,105,110,107,115,0,78,111,32,108,111,99,107,115,32,97,118,97,105,108,97,98,108,101,0,82,101,115,111,117,114,99,101,32,100,101,97,100,108,111,99,107,32,119,111,117,108,100,32,111,99,99,117,114,0,83,116,97,116,101,32,110,111,116,32,114,101,99,111,118,101,114,97,98,108,101,0,80,114,101,118,105,111,117,115,32,111,119,110,101,114,32,100,105,101,100,0,79,112,101,114,97,116,105,111,110,32,99,97,110,99,101,108,101,100,0,70,117,110,99,116,105,111,110,32,110,111,116,32,105,109,112,108,101,109,101,110,116,101,100,0,78,111,32,109,101,115,115,97,103,101,32,111,102,32,100,101,115,105,114,101,100,32,116,121,112,101,0,73,100,101,110,116,105,102,105,101,114,32,114,101,109,111,118,101,100,0,68,101,118,105,99,101,32,110,111,116,32,97,32,115,116,114,101,97,109,0,78,111,32,100,97,116,97,32,97,118,97,105,108,97,98,108,101,0,68,101,118,105,99,101,32,116,105,109,101,111,117,116,0,79,117,116,32,111,102,32,115,116,114,101,97,109,115,32,114,101,115,111,117,114,99,101,115,0,76,105,110,107,32,104,97,115,32,98,101,101,110,32,115,101,118,101,114,101,100,0,80,114,111,116,111,99,111,108,32,101,114,114,111,114,0,66,97,100,32,109,101,115,115,97,103,101,0,70,105,108,101,32,100,101,115,99,114,105,112,116,111,114,32,105,110,32,98,97,100,32,115,116,97,116,101,0,78,111,116,32,97,32,115,111,99,107,101,116,0,68,101,115,116,105,110,97,116,105,111,110,32,97,100,100,114,101,115,115,32,114,101,113,117,105,114,101,100,0,77,101,115,115,97,103,101,32,116,111,111,32,108,97,114,103,101,0,80,114,111,116,111,99,111,108,32,119,114,111,110,103,32,116,121,112,101,32,102,111,114,32,115,111,99,107,101,116,0,80,114,111,116,111,99,111,108,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,80,114,111,116,111,99,111,108,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,83,111,99,107,101,116,32,116,121,112,101,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,78,111,116,32,115,117,112,112,111,114,116,101,100,0,80,114,111,116,111,99,111,108,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,0,65,100,100,114,101,115,115,32,102,97,109,105,108,121,32,110,111,116,32,115,117,112,112,111,114,116,101,100,32,98,121,32,112,114,111,116,111,99,111,108,0,65,100,100,114,101,115,115,32,110,111,116,32,97,118,97,105,108,97,98,108,101,0,78,101,116,119,111,114,107,32,105,115,32,100,111,119,110,0,78,101,116,119,111,114,107,32,117,110,114,101,97,99,104,97,98,108,101,0,67,111,110,110,101,99,116,105,111,110,32,114,101,115,101,116,32,98,121,32,110,101,116,119,111,114,107,0,67,111,110,110,101,99,116,105,111,110,32,97,98,111,114,116,101,100,0,78,111,32,98,117,102,102,101,114,32,115,112,97,99,101,32,97,118,97,105,108,97,98,108,101,0,83,111,99,107,101,116,32,105,115,32,99,111,110,110,101,99,116,101,100,0,83,111,99,107,101,116,32,110,111,116,32,99,111,110,110,101,99,116,101,100,0,67,97,110,110,111,116,32,115,101,110,100,32,97,102,116,101,114,32,115,111,99,107,101,116,32,115,104,117,116,100,111,119,110,0,79,112,101,114,97,116,105,111,110,32,97,108,114,101,97,100,121,32,105,110,32,112,114,111,103,114,101,115,115,0,79,112,101,114,97,116,105,111,110,32,105,110,32,112,114,111,103,114,101,115,115,0,83,116,97,108,101,32,102,105,108,101,32,104,97,110,100,108,101,0,82,101,109,111,116,101,32,73,47,79,32,101,114,114,111,114,0,81,117,111,116,97,32,101,120,99,101,101,100,101,100,0,78,111,32,109,101,100,105,117,109,32,102,111,117,110,100,0,87,114,111,110,103,32,109,101,100,105,117,109,32,116,121,112,101,0,78,111,32,101,114,114,111,114,32,105,110,102,111,114,109,97,116,105,111,110,0,0,47,112,114,111,99,47,115,101,108,102,47,102,100,47], "i8", ALLOC_NONE, Runtime.GLOBAL_BASE+40960);





/* no memory initializer */
var tempDoublePtr = Runtime.alignMemory(allocate(12, "i8", ALLOC_STATIC), 8);

assert(tempDoublePtr % 8 == 0);

function copyTempFloat(ptr) { // functions, because inlining this code increases code size too much

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

}

function copyTempDouble(ptr) {

  HEAP8[tempDoublePtr] = HEAP8[ptr];

  HEAP8[tempDoublePtr+1] = HEAP8[ptr+1];

  HEAP8[tempDoublePtr+2] = HEAP8[ptr+2];

  HEAP8[tempDoublePtr+3] = HEAP8[ptr+3];

  HEAP8[tempDoublePtr+4] = HEAP8[ptr+4];

  HEAP8[tempDoublePtr+5] = HEAP8[ptr+5];

  HEAP8[tempDoublePtr+6] = HEAP8[ptr+6];

  HEAP8[tempDoublePtr+7] = HEAP8[ptr+7];

}

// {{PRE_LIBRARY}}


   
  Module["_i64Subtract"] = _i64Subtract;

  var _DtoILow=true;

   
  Module["_i64Add"] = _i64Add;

  
  
  
  var ERRNO_CODES={EPERM:1,ENOENT:2,ESRCH:3,EINTR:4,EIO:5,ENXIO:6,E2BIG:7,ENOEXEC:8,EBADF:9,ECHILD:10,EAGAIN:11,EWOULDBLOCK:11,ENOMEM:12,EACCES:13,EFAULT:14,ENOTBLK:15,EBUSY:16,EEXIST:17,EXDEV:18,ENODEV:19,ENOTDIR:20,EISDIR:21,EINVAL:22,ENFILE:23,EMFILE:24,ENOTTY:25,ETXTBSY:26,EFBIG:27,ENOSPC:28,ESPIPE:29,EROFS:30,EMLINK:31,EPIPE:32,EDOM:33,ERANGE:34,ENOMSG:42,EIDRM:43,ECHRNG:44,EL2NSYNC:45,EL3HLT:46,EL3RST:47,ELNRNG:48,EUNATCH:49,ENOCSI:50,EL2HLT:51,EDEADLK:35,ENOLCK:37,EBADE:52,EBADR:53,EXFULL:54,ENOANO:55,EBADRQC:56,EBADSLT:57,EDEADLOCK:35,EBFONT:59,ENOSTR:60,ENODATA:61,ETIME:62,ENOSR:63,ENONET:64,ENOPKG:65,EREMOTE:66,ENOLINK:67,EADV:68,ESRMNT:69,ECOMM:70,EPROTO:71,EMULTIHOP:72,EDOTDOT:73,EBADMSG:74,ENOTUNIQ:76,EBADFD:77,EREMCHG:78,ELIBACC:79,ELIBBAD:80,ELIBSCN:81,ELIBMAX:82,ELIBEXEC:83,ENOSYS:38,ENOTEMPTY:39,ENAMETOOLONG:36,ELOOP:40,EOPNOTSUPP:95,EPFNOSUPPORT:96,ECONNRESET:104,ENOBUFS:105,EAFNOSUPPORT:97,EPROTOTYPE:91,ENOTSOCK:88,ENOPROTOOPT:92,ESHUTDOWN:108,ECONNREFUSED:111,EADDRINUSE:98,ECONNABORTED:103,ENETUNREACH:101,ENETDOWN:100,ETIMEDOUT:110,EHOSTDOWN:112,EHOSTUNREACH:113,EINPROGRESS:115,EALREADY:114,EDESTADDRREQ:89,EMSGSIZE:90,EPROTONOSUPPORT:93,ESOCKTNOSUPPORT:94,EADDRNOTAVAIL:99,ENETRESET:102,EISCONN:106,ENOTCONN:107,ETOOMANYREFS:109,EUSERS:87,EDQUOT:122,ESTALE:116,ENOTSUP:95,ENOMEDIUM:123,EILSEQ:84,EOVERFLOW:75,ECANCELED:125,ENOTRECOVERABLE:131,EOWNERDEAD:130,ESTRPIPE:86};
  
  var ERRNO_MESSAGES={0:"Success",1:"Not super-user",2:"No such file or directory",3:"No such process",4:"Interrupted system call",5:"I/O error",6:"No such device or address",7:"Arg list too long",8:"Exec format error",9:"Bad file number",10:"No children",11:"No more processes",12:"Not enough core",13:"Permission denied",14:"Bad address",15:"Block device required",16:"Mount device busy",17:"File exists",18:"Cross-device link",19:"No such device",20:"Not a directory",21:"Is a directory",22:"Invalid argument",23:"Too many open files in system",24:"Too many open files",25:"Not a typewriter",26:"Text file busy",27:"File too large",28:"No space left on device",29:"Illegal seek",30:"Read only file system",31:"Too many links",32:"Broken pipe",33:"Math arg out of domain of func",34:"Math result not representable",35:"File locking deadlock error",36:"File or path name too long",37:"No record locks available",38:"Function not implemented",39:"Directory not empty",40:"Too many symbolic links",42:"No message of desired type",43:"Identifier removed",44:"Channel number out of range",45:"Level 2 not synchronized",46:"Level 3 halted",47:"Level 3 reset",48:"Link number out of range",49:"Protocol driver not attached",50:"No CSI structure available",51:"Level 2 halted",52:"Invalid exchange",53:"Invalid request descriptor",54:"Exchange full",55:"No anode",56:"Invalid request code",57:"Invalid slot",59:"Bad font file fmt",60:"Device not a stream",61:"No data (for no delay io)",62:"Timer expired",63:"Out of streams resources",64:"Machine is not on the network",65:"Package not installed",66:"The object is remote",67:"The link has been severed",68:"Advertise error",69:"Srmount error",70:"Communication error on send",71:"Protocol error",72:"Multihop attempted",73:"Cross mount point (not really error)",74:"Trying to read unreadable message",75:"Value too large for defined data type",76:"Given log. name not unique",77:"f.d. invalid for this operation",78:"Remote address changed",79:"Can   access a needed shared lib",80:"Accessing a corrupted shared lib",81:".lib section in a.out corrupted",82:"Attempting to link in too many libs",83:"Attempting to exec a shared library",84:"Illegal byte sequence",86:"Streams pipe error",87:"Too many users",88:"Socket operation on non-socket",89:"Destination address required",90:"Message too long",91:"Protocol wrong type for socket",92:"Protocol not available",93:"Unknown protocol",94:"Socket type not supported",95:"Not supported",96:"Protocol family not supported",97:"Address family not supported by protocol family",98:"Address already in use",99:"Address not available",100:"Network interface is not configured",101:"Network is unreachable",102:"Connection reset by network",103:"Connection aborted",104:"Connection reset by peer",105:"No buffer space available",106:"Socket is already connected",107:"Socket is not connected",108:"Can't send after socket shutdown",109:"Too many references",110:"Connection timed out",111:"Connection refused",112:"Host is down",113:"Host is unreachable",114:"Socket already connected",115:"Connection already in progress",116:"Stale file handle",122:"Quota exceeded",123:"No medium (in tape drive)",125:"Operation canceled",130:"Previous owner died",131:"State not recoverable"};
  
  function ___setErrNo(value) {
      if (Module['___errno_location']) HEAP32[((Module['___errno_location']())>>2)]=value;
      return value;
    }
  
  var PATH={splitPath:function (filename) {
        var splitPathRe = /^(\/?|)([\s\S]*?)((?:\.{1,2}|[^\/]+?|)(\.[^.\/]*|))(?:[\/]*)$/;
        return splitPathRe.exec(filename).slice(1);
      },normalizeArray:function (parts, allowAboveRoot) {
        // if the path tries to go above the root, `up` ends up > 0
        var up = 0;
        for (var i = parts.length - 1; i >= 0; i--) {
          var last = parts[i];
          if (last === '.') {
            parts.splice(i, 1);
          } else if (last === '..') {
            parts.splice(i, 1);
            up++;
          } else if (up) {
            parts.splice(i, 1);
            up--;
          }
        }
        // if the path is allowed to go above the root, restore leading ..s
        if (allowAboveRoot) {
          for (; up--; up) {
            parts.unshift('..');
          }
        }
        return parts;
      },normalize:function (path) {
        var isAbsolute = path.charAt(0) === '/',
            trailingSlash = path.substr(-1) === '/';
        // Normalize the path
        path = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), !isAbsolute).join('/');
        if (!path && !isAbsolute) {
          path = '.';
        }
        if (path && trailingSlash) {
          path += '/';
        }
        return (isAbsolute ? '/' : '') + path;
      },dirname:function (path) {
        var result = PATH.splitPath(path),
            root = result[0],
            dir = result[1];
        if (!root && !dir) {
          // No dirname whatsoever
          return '.';
        }
        if (dir) {
          // It has a dirname, strip trailing slash
          dir = dir.substr(0, dir.length - 1);
        }
        return root + dir;
      },basename:function (path) {
        // EMSCRIPTEN return '/'' for '/', not an empty string
        if (path === '/') return '/';
        var lastSlash = path.lastIndexOf('/');
        if (lastSlash === -1) return path;
        return path.substr(lastSlash+1);
      },extname:function (path) {
        return PATH.splitPath(path)[3];
      },join:function () {
        var paths = Array.prototype.slice.call(arguments, 0);
        return PATH.normalize(paths.join('/'));
      },join2:function (l, r) {
        return PATH.normalize(l + '/' + r);
      },resolve:function () {
        var resolvedPath = '',
          resolvedAbsolute = false;
        for (var i = arguments.length - 1; i >= -1 && !resolvedAbsolute; i--) {
          var path = (i >= 0) ? arguments[i] : FS.cwd();
          // Skip empty and invalid entries
          if (typeof path !== 'string') {
            throw new TypeError('Arguments to path.resolve must be strings');
          } else if (!path) {
            return ''; // an invalid portion invalidates the whole thing
          }
          resolvedPath = path + '/' + resolvedPath;
          resolvedAbsolute = path.charAt(0) === '/';
        }
        // At this point the path should be resolved to a full absolute path, but
        // handle relative paths to be safe (might happen when process.cwd() fails)
        resolvedPath = PATH.normalizeArray(resolvedPath.split('/').filter(function(p) {
          return !!p;
        }), !resolvedAbsolute).join('/');
        return ((resolvedAbsolute ? '/' : '') + resolvedPath) || '.';
      },relative:function (from, to) {
        from = PATH.resolve(from).substr(1);
        to = PATH.resolve(to).substr(1);
        function trim(arr) {
          var start = 0;
          for (; start < arr.length; start++) {
            if (arr[start] !== '') break;
          }
          var end = arr.length - 1;
          for (; end >= 0; end--) {
            if (arr[end] !== '') break;
          }
          if (start > end) return [];
          return arr.slice(start, end - start + 1);
        }
        var fromParts = trim(from.split('/'));
        var toParts = trim(to.split('/'));
        var length = Math.min(fromParts.length, toParts.length);
        var samePartsLength = length;
        for (var i = 0; i < length; i++) {
          if (fromParts[i] !== toParts[i]) {
            samePartsLength = i;
            break;
          }
        }
        var outputParts = [];
        for (var i = samePartsLength; i < fromParts.length; i++) {
          outputParts.push('..');
        }
        outputParts = outputParts.concat(toParts.slice(samePartsLength));
        return outputParts.join('/');
      }};
  
  var TTY={ttys:[],init:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // currently, FS.init does not distinguish if process.stdin is a file or TTY
        //   // device, it always assumes it's a TTY device. because of this, we're forcing
        //   // process.stdin to UTF8 encoding to at least make stdin reading compatible
        //   // with text files until FS.init can be refactored.
        //   process['stdin']['setEncoding']('utf8');
        // }
      },shutdown:function () {
        // https://github.com/kripken/emscripten/pull/1555
        // if (ENVIRONMENT_IS_NODE) {
        //   // inolen: any idea as to why node -e 'process.stdin.read()' wouldn't exit immediately (with process.stdin being a tty)?
        //   // isaacs: because now it's reading from the stream, you've expressed interest in it, so that read() kicks off a _read() which creates a ReadReq operation
        //   // inolen: I thought read() in that case was a synchronous operation that just grabbed some amount of buffered data if it exists?
        //   // isaacs: it is. but it also triggers a _read() call, which calls readStart() on the handle
        //   // isaacs: do process.stdin.pause() and i'd think it'd probably close the pending call
        //   process['stdin']['pause']();
        // }
      },register:function (dev, ops) {
        TTY.ttys[dev] = { input: [], output: [], ops: ops };
        FS.registerDevice(dev, TTY.stream_ops);
      },stream_ops:{open:function (stream) {
          var tty = TTY.ttys[stream.node.rdev];
          if (!tty) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          stream.tty = tty;
          stream.seekable = false;
        },close:function (stream) {
          // flush any pending line data
          stream.tty.ops.flush(stream.tty);
        },flush:function (stream) {
          stream.tty.ops.flush(stream.tty);
        },read:function (stream, buffer, offset, length, pos /* ignored */) {
          if (!stream.tty || !stream.tty.ops.get_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          var bytesRead = 0;
          for (var i = 0; i < length; i++) {
            var result;
            try {
              result = stream.tty.ops.get_char(stream.tty);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            if (result === undefined && bytesRead === 0) {
              throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
            }
            if (result === null || result === undefined) break;
            bytesRead++;
            buffer[offset+i] = result;
          }
          if (bytesRead) {
            stream.node.timestamp = Date.now();
          }
          return bytesRead;
        },write:function (stream, buffer, offset, length, pos) {
          if (!stream.tty || !stream.tty.ops.put_char) {
            throw new FS.ErrnoError(ERRNO_CODES.ENXIO);
          }
          for (var i = 0; i < length; i++) {
            try {
              stream.tty.ops.put_char(stream.tty, buffer[offset+i]);
            } catch (e) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
          }
          if (length) {
            stream.node.timestamp = Date.now();
          }
          return i;
        }},default_tty_ops:{get_char:function (tty) {
          if (!tty.input.length) {
            var result = null;
            if (ENVIRONMENT_IS_NODE) {
              // we will read data by chunks of BUFSIZE
              var BUFSIZE = 256;
              var buf = new Buffer(BUFSIZE);
              var bytesRead = 0;
  
              var fd = process.stdin.fd;
              // Linux and Mac cannot use process.stdin.fd (which isn't set up as sync)
              var usingDevice = false;
              try {
                fd = fs.openSync('/dev/stdin', 'r');
                usingDevice = true;
              } catch (e) {}
  
              bytesRead = fs.readSync(fd, buf, 0, BUFSIZE, null);
  
              if (usingDevice) { fs.closeSync(fd); }
              if (bytesRead > 0) {
                result = buf.slice(0, bytesRead).toString('utf-8');
              } else {
                result = null;
              }
  
            } else if (typeof window != 'undefined' &&
              typeof window.prompt == 'function') {
              // Browser.
              result = window.prompt('Input: ');  // returns null on cancel
              if (result !== null) {
                result += '\n';
              }
            } else if (typeof readline == 'function') {
              // Command line.
              result = readline();
              if (result !== null) {
                result += '\n';
              }
            }
            if (!result) {
              return null;
            }
            tty.input = intArrayFromString(result, true);
          }
          return tty.input.shift();
        },put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val); // val == 0 would cut text output off in the middle.
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['print'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }},default_tty1_ops:{put_char:function (tty, val) {
          if (val === null || val === 10) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          } else {
            if (val != 0) tty.output.push(val);
          }
        },flush:function (tty) {
          if (tty.output && tty.output.length > 0) {
            Module['printErr'](UTF8ArrayToString(tty.output, 0));
            tty.output = [];
          }
        }}};
  
  var MEMFS={ops_table:null,mount:function (mount) {
        return MEMFS.createNode(null, '/', 16384 | 511 /* 0777 */, 0);
      },createNode:function (parent, name, mode, dev) {
        if (FS.isBlkdev(mode) || FS.isFIFO(mode)) {
          // no supported
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (!MEMFS.ops_table) {
          MEMFS.ops_table = {
            dir: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                lookup: MEMFS.node_ops.lookup,
                mknod: MEMFS.node_ops.mknod,
                rename: MEMFS.node_ops.rename,
                unlink: MEMFS.node_ops.unlink,
                rmdir: MEMFS.node_ops.rmdir,
                readdir: MEMFS.node_ops.readdir,
                symlink: MEMFS.node_ops.symlink
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek
              }
            },
            file: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: {
                llseek: MEMFS.stream_ops.llseek,
                read: MEMFS.stream_ops.read,
                write: MEMFS.stream_ops.write,
                allocate: MEMFS.stream_ops.allocate,
                mmap: MEMFS.stream_ops.mmap,
                msync: MEMFS.stream_ops.msync
              }
            },
            link: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr,
                readlink: MEMFS.node_ops.readlink
              },
              stream: {}
            },
            chrdev: {
              node: {
                getattr: MEMFS.node_ops.getattr,
                setattr: MEMFS.node_ops.setattr
              },
              stream: FS.chrdev_stream_ops
            }
          };
        }
        var node = FS.createNode(parent, name, mode, dev);
        if (FS.isDir(node.mode)) {
          node.node_ops = MEMFS.ops_table.dir.node;
          node.stream_ops = MEMFS.ops_table.dir.stream;
          node.contents = {};
        } else if (FS.isFile(node.mode)) {
          node.node_ops = MEMFS.ops_table.file.node;
          node.stream_ops = MEMFS.ops_table.file.stream;
          node.usedBytes = 0; // The actual number of bytes used in the typed array, as opposed to contents.buffer.byteLength which gives the whole capacity.
          // When the byte data of the file is populated, this will point to either a typed array, or a normal JS array. Typed arrays are preferred
          // for performance, and used by default. However, typed arrays are not resizable like normal JS arrays are, so there is a small disk size
          // penalty involved for appending file writes that continuously grow a file similar to std::vector capacity vs used -scheme.
          node.contents = null; 
        } else if (FS.isLink(node.mode)) {
          node.node_ops = MEMFS.ops_table.link.node;
          node.stream_ops = MEMFS.ops_table.link.stream;
        } else if (FS.isChrdev(node.mode)) {
          node.node_ops = MEMFS.ops_table.chrdev.node;
          node.stream_ops = MEMFS.ops_table.chrdev.stream;
        }
        node.timestamp = Date.now();
        // add the new node to the parent
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },getFileDataAsRegularArray:function (node) {
        if (node.contents && node.contents.subarray) {
          var arr = [];
          for (var i = 0; i < node.usedBytes; ++i) arr.push(node.contents[i]);
          return arr; // Returns a copy of the original data.
        }
        return node.contents; // No-op, the file contents are already in a JS array. Return as-is.
      },getFileDataAsTypedArray:function (node) {
        if (!node.contents) return new Uint8Array;
        if (node.contents.subarray) return node.contents.subarray(0, node.usedBytes); // Make sure to not return excess unused bytes.
        return new Uint8Array(node.contents);
      },expandFileStorage:function (node, newCapacity) {
        // If we are asked to expand the size of a file that already exists, revert to using a standard JS array to store the file
        // instead of a typed array. This makes resizing the array more flexible because we can just .push() elements at the back to
        // increase the size.
        if (node.contents && node.contents.subarray && newCapacity > node.contents.length) {
          node.contents = MEMFS.getFileDataAsRegularArray(node);
          node.usedBytes = node.contents.length; // We might be writing to a lazy-loaded file which had overridden this property, so force-reset it.
        }
  
        if (!node.contents || node.contents.subarray) { // Keep using a typed array if creating a new storage, or if old one was a typed array as well.
          var prevCapacity = node.contents ? node.contents.buffer.byteLength : 0;
          if (prevCapacity >= newCapacity) return; // No need to expand, the storage was already large enough.
          // Don't expand strictly to the given requested limit if it's only a very small increase, but instead geometrically grow capacity.
          // For small filesizes (<1MB), perform size*2 geometric increase, but for large sizes, do a much more conservative size*1.125 increase to
          // avoid overshooting the allocation cap by a very large margin.
          var CAPACITY_DOUBLING_MAX = 1024 * 1024;
          newCapacity = Math.max(newCapacity, (prevCapacity * (prevCapacity < CAPACITY_DOUBLING_MAX ? 2.0 : 1.125)) | 0);
          if (prevCapacity != 0) newCapacity = Math.max(newCapacity, 256); // At minimum allocate 256b for each file when expanding.
          var oldContents = node.contents;
          node.contents = new Uint8Array(newCapacity); // Allocate new storage.
          if (node.usedBytes > 0) node.contents.set(oldContents.subarray(0, node.usedBytes), 0); // Copy old data over to the new storage.
          return;
        }
        // Not using a typed array to back the file storage. Use a standard JS array instead.
        if (!node.contents && newCapacity > 0) node.contents = [];
        while (node.contents.length < newCapacity) node.contents.push(0);
      },resizeFileStorage:function (node, newSize) {
        if (node.usedBytes == newSize) return;
        if (newSize == 0) {
          node.contents = null; // Fully decommit when requesting a resize to zero.
          node.usedBytes = 0;
          return;
        }
        if (!node.contents || node.contents.subarray) { // Resize a typed array if that is being used as the backing store.
          var oldContents = node.contents;
          node.contents = new Uint8Array(new ArrayBuffer(newSize)); // Allocate new storage.
          if (oldContents) {
            node.contents.set(oldContents.subarray(0, Math.min(newSize, node.usedBytes))); // Copy old data over to the new storage.
          }
          node.usedBytes = newSize;
          return;
        }
        // Backing with a JS array.
        if (!node.contents) node.contents = [];
        if (node.contents.length > newSize) node.contents.length = newSize;
        else while (node.contents.length < newSize) node.contents.push(0);
        node.usedBytes = newSize;
      },node_ops:{getattr:function (node) {
          var attr = {};
          // device numbers reuse inode numbers.
          attr.dev = FS.isChrdev(node.mode) ? node.id : 1;
          attr.ino = node.id;
          attr.mode = node.mode;
          attr.nlink = 1;
          attr.uid = 0;
          attr.gid = 0;
          attr.rdev = node.rdev;
          if (FS.isDir(node.mode)) {
            attr.size = 4096;
          } else if (FS.isFile(node.mode)) {
            attr.size = node.usedBytes;
          } else if (FS.isLink(node.mode)) {
            attr.size = node.link.length;
          } else {
            attr.size = 0;
          }
          attr.atime = new Date(node.timestamp);
          attr.mtime = new Date(node.timestamp);
          attr.ctime = new Date(node.timestamp);
          // NOTE: In our implementation, st_blocks = Math.ceil(st_size/st_blksize),
          //       but this is not required by the standard.
          attr.blksize = 4096;
          attr.blocks = Math.ceil(attr.size / attr.blksize);
          return attr;
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
          if (attr.size !== undefined) {
            MEMFS.resizeFileStorage(node, attr.size);
          }
        },lookup:function (parent, name) {
          throw FS.genericErrors[ERRNO_CODES.ENOENT];
        },mknod:function (parent, name, mode, dev) {
          return MEMFS.createNode(parent, name, mode, dev);
        },rename:function (old_node, new_dir, new_name) {
          // if we're overwriting a directory at new_name, make sure it's empty.
          if (FS.isDir(old_node.mode)) {
            var new_node;
            try {
              new_node = FS.lookupNode(new_dir, new_name);
            } catch (e) {
            }
            if (new_node) {
              for (var i in new_node.contents) {
                throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
              }
            }
          }
          // do the internal rewiring
          delete old_node.parent.contents[old_node.name];
          old_node.name = new_name;
          new_dir.contents[new_name] = old_node;
          old_node.parent = new_dir;
        },unlink:function (parent, name) {
          delete parent.contents[name];
        },rmdir:function (parent, name) {
          var node = FS.lookupNode(parent, name);
          for (var i in node.contents) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
          }
          delete parent.contents[name];
        },readdir:function (node) {
          var entries = ['.', '..']
          for (var key in node.contents) {
            if (!node.contents.hasOwnProperty(key)) {
              continue;
            }
            entries.push(key);
          }
          return entries;
        },symlink:function (parent, newname, oldpath) {
          var node = MEMFS.createNode(parent, newname, 511 /* 0777 */ | 40960, 0);
          node.link = oldpath;
          return node;
        },readlink:function (node) {
          if (!FS.isLink(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return node.link;
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          var contents = stream.node.contents;
          if (position >= stream.node.usedBytes) return 0;
          var size = Math.min(stream.node.usedBytes - position, length);
          assert(size >= 0);
          if (size > 8 && contents.subarray) { // non-trivial, and typed array
            buffer.set(contents.subarray(position, position + size), offset);
          } else {
            for (var i = 0; i < size; i++) buffer[offset + i] = contents[position + i];
          }
          return size;
        },write:function (stream, buffer, offset, length, position, canOwn) {
          if (!length) return 0;
          var node = stream.node;
          node.timestamp = Date.now();
  
          if (buffer.subarray && (!node.contents || node.contents.subarray)) { // This write is from a typed array to a typed array?
            if (canOwn) { // Can we just reuse the buffer we are given?
              node.contents = buffer.subarray(offset, offset + length);
              node.usedBytes = length;
              return length;
            } else if (node.usedBytes === 0 && position === 0) { // If this is a simple first write to an empty file, do a fast set since we don't need to care about old data.
              node.contents = new Uint8Array(buffer.subarray(offset, offset + length));
              node.usedBytes = length;
              return length;
            } else if (position + length <= node.usedBytes) { // Writing to an already allocated and used subrange of the file?
              node.contents.set(buffer.subarray(offset, offset + length), position);
              return length;
            }
          }
  
          // Appending to an existing file and we need to reallocate, or source data did not come as a typed array.
          MEMFS.expandFileStorage(node, position+length);
          if (node.contents.subarray && buffer.subarray) node.contents.set(buffer.subarray(offset, offset + length), position); // Use typed array write if available.
          else {
            for (var i = 0; i < length; i++) {
             node.contents[position + i] = buffer[offset + i]; // Or fall back to manual write if not.
            }
          }
          node.usedBytes = Math.max(node.usedBytes, position+length);
          return length;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.usedBytes;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        },allocate:function (stream, offset, length) {
          MEMFS.expandFileStorage(stream.node, offset + length);
          stream.node.usedBytes = Math.max(stream.node.usedBytes, offset + length);
        },mmap:function (stream, buffer, offset, length, position, prot, flags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          var ptr;
          var allocated;
          var contents = stream.node.contents;
          // Only make a new copy when MAP_PRIVATE is specified.
          if ( !(flags & 2) &&
                (contents.buffer === buffer || contents.buffer === buffer.buffer) ) {
            // We can't emulate MAP_SHARED when the file is not backed by the buffer
            // we're mapping to (e.g. the HEAP buffer).
            allocated = false;
            ptr = contents.byteOffset;
          } else {
            // Try to avoid unnecessary slices.
            if (position > 0 || position + length < stream.node.usedBytes) {
              if (contents.subarray) {
                contents = contents.subarray(position, position + length);
              } else {
                contents = Array.prototype.slice.call(contents, position, position + length);
              }
            }
            allocated = true;
            ptr = _malloc(length);
            if (!ptr) {
              throw new FS.ErrnoError(ERRNO_CODES.ENOMEM);
            }
            buffer.set(contents, ptr);
          }
          return { ptr: ptr, allocated: allocated };
        },msync:function (stream, buffer, offset, length, mmapFlags) {
          if (!FS.isFile(stream.node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
          }
          if (mmapFlags & 2) {
            // MAP_PRIVATE calls need not to be synced back to underlying fs
            return 0;
          }
  
          var bytesWritten = MEMFS.stream_ops.write(stream, buffer, 0, length, offset, false);
          // should we check if bytesWritten and length are the same?
          return 0;
        }}};
  
  var IDBFS={dbs:{},indexedDB:function () {
        if (typeof indexedDB !== 'undefined') return indexedDB;
        var ret = null;
        if (typeof window === 'object') ret = window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
        assert(ret, 'IDBFS used, but indexedDB not supported');
        return ret;
      },DB_VERSION:21,DB_STORE_NAME:"FILE_DATA",mount:function (mount) {
        // reuse all of the core MEMFS functionality
        return MEMFS.mount.apply(null, arguments);
      },syncfs:function (mount, populate, callback) {
        IDBFS.getLocalSet(mount, function(err, local) {
          if (err) return callback(err);
  
          IDBFS.getRemoteSet(mount, function(err, remote) {
            if (err) return callback(err);
  
            var src = populate ? remote : local;
            var dst = populate ? local : remote;
  
            IDBFS.reconcile(src, dst, callback);
          });
        });
      },getDB:function (name, callback) {
        // check the cache first
        var db = IDBFS.dbs[name];
        if (db) {
          return callback(null, db);
        }
  
        var req;
        try {
          req = IDBFS.indexedDB().open(name, IDBFS.DB_VERSION);
        } catch (e) {
          return callback(e);
        }
        req.onupgradeneeded = function(e) {
          var db = e.target.result;
          var transaction = e.target.transaction;
  
          var fileStore;
  
          if (db.objectStoreNames.contains(IDBFS.DB_STORE_NAME)) {
            fileStore = transaction.objectStore(IDBFS.DB_STORE_NAME);
          } else {
            fileStore = db.createObjectStore(IDBFS.DB_STORE_NAME);
          }
  
          if (!fileStore.indexNames.contains('timestamp')) {
            fileStore.createIndex('timestamp', 'timestamp', { unique: false });
          }
        };
        req.onsuccess = function() {
          db = req.result;
  
          // add to the cache
          IDBFS.dbs[name] = db;
          callback(null, db);
        };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },getLocalSet:function (mount, callback) {
        var entries = {};
  
        function isRealDir(p) {
          return p !== '.' && p !== '..';
        };
        function toAbsolute(root) {
          return function(p) {
            return PATH.join2(root, p);
          }
        };
  
        var check = FS.readdir(mount.mountpoint).filter(isRealDir).map(toAbsolute(mount.mountpoint));
  
        while (check.length) {
          var path = check.pop();
          var stat;
  
          try {
            stat = FS.stat(path);
          } catch (e) {
            return callback(e);
          }
  
          if (FS.isDir(stat.mode)) {
            check.push.apply(check, FS.readdir(path).filter(isRealDir).map(toAbsolute(path)));
          }
  
          entries[path] = { timestamp: stat.mtime };
        }
  
        return callback(null, { type: 'local', entries: entries });
      },getRemoteSet:function (mount, callback) {
        var entries = {};
  
        IDBFS.getDB(mount.mountpoint, function(err, db) {
          if (err) return callback(err);
  
          var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readonly');
          transaction.onerror = function(e) {
            callback(this.error);
            e.preventDefault();
          };
  
          var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
          var index = store.index('timestamp');
  
          index.openKeyCursor().onsuccess = function(event) {
            var cursor = event.target.result;
  
            if (!cursor) {
              return callback(null, { type: 'remote', db: db, entries: entries });
            }
  
            entries[cursor.primaryKey] = { timestamp: cursor.key };
  
            cursor.continue();
          };
        });
      },loadLocalEntry:function (path, callback) {
        var stat, node;
  
        try {
          var lookup = FS.lookupPath(path);
          node = lookup.node;
          stat = FS.stat(path);
        } catch (e) {
          return callback(e);
        }
  
        if (FS.isDir(stat.mode)) {
          return callback(null, { timestamp: stat.mtime, mode: stat.mode });
        } else if (FS.isFile(stat.mode)) {
          // Performance consideration: storing a normal JavaScript array to a IndexedDB is much slower than storing a typed array.
          // Therefore always convert the file contents to a typed array first before writing the data to IndexedDB.
          node.contents = MEMFS.getFileDataAsTypedArray(node);
          return callback(null, { timestamp: stat.mtime, mode: stat.mode, contents: node.contents });
        } else {
          return callback(new Error('node type not supported'));
        }
      },storeLocalEntry:function (path, entry, callback) {
        try {
          if (FS.isDir(entry.mode)) {
            FS.mkdir(path, entry.mode);
          } else if (FS.isFile(entry.mode)) {
            FS.writeFile(path, entry.contents, { encoding: 'binary', canOwn: true });
          } else {
            return callback(new Error('node type not supported'));
          }
  
          FS.chmod(path, entry.mode);
          FS.utime(path, entry.timestamp, entry.timestamp);
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },removeLocalEntry:function (path, callback) {
        try {
          var lookup = FS.lookupPath(path);
          var stat = FS.stat(path);
  
          if (FS.isDir(stat.mode)) {
            FS.rmdir(path);
          } else if (FS.isFile(stat.mode)) {
            FS.unlink(path);
          }
        } catch (e) {
          return callback(e);
        }
  
        callback(null);
      },loadRemoteEntry:function (store, path, callback) {
        var req = store.get(path);
        req.onsuccess = function(event) { callback(null, event.target.result); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },storeRemoteEntry:function (store, path, entry, callback) {
        var req = store.put(entry, path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },removeRemoteEntry:function (store, path, callback) {
        var req = store.delete(path);
        req.onsuccess = function() { callback(null); };
        req.onerror = function(e) {
          callback(this.error);
          e.preventDefault();
        };
      },reconcile:function (src, dst, callback) {
        var total = 0;
  
        var create = [];
        Object.keys(src.entries).forEach(function (key) {
          var e = src.entries[key];
          var e2 = dst.entries[key];
          if (!e2 || e.timestamp > e2.timestamp) {
            create.push(key);
            total++;
          }
        });
  
        var remove = [];
        Object.keys(dst.entries).forEach(function (key) {
          var e = dst.entries[key];
          var e2 = src.entries[key];
          if (!e2) {
            remove.push(key);
            total++;
          }
        });
  
        if (!total) {
          return callback(null);
        }
  
        var errored = false;
        var completed = 0;
        var db = src.type === 'remote' ? src.db : dst.db;
        var transaction = db.transaction([IDBFS.DB_STORE_NAME], 'readwrite');
        var store = transaction.objectStore(IDBFS.DB_STORE_NAME);
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= total) {
            return callback(null);
          }
        };
  
        transaction.onerror = function(e) {
          done(this.error);
          e.preventDefault();
        };
  
        // sort paths in ascending order so directory entries are created
        // before the files inside them
        create.sort().forEach(function (path) {
          if (dst.type === 'local') {
            IDBFS.loadRemoteEntry(store, path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeLocalEntry(path, entry, done);
            });
          } else {
            IDBFS.loadLocalEntry(path, function (err, entry) {
              if (err) return done(err);
              IDBFS.storeRemoteEntry(store, path, entry, done);
            });
          }
        });
  
        // sort paths in descending order so files are deleted before their
        // parent directories
        remove.sort().reverse().forEach(function(path) {
          if (dst.type === 'local') {
            IDBFS.removeLocalEntry(path, done);
          } else {
            IDBFS.removeRemoteEntry(store, path, done);
          }
        });
      }};
  
  var NODEFS={isWindows:false,staticInit:function () {
        NODEFS.isWindows = !!process.platform.match(/^win/);
      },mount:function (mount) {
        assert(ENVIRONMENT_IS_NODE);
        return NODEFS.createNode(null, '/', NODEFS.getMode(mount.opts.root), 0);
      },createNode:function (parent, name, mode, dev) {
        if (!FS.isDir(mode) && !FS.isFile(mode) && !FS.isLink(mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node = FS.createNode(parent, name, mode);
        node.node_ops = NODEFS.node_ops;
        node.stream_ops = NODEFS.stream_ops;
        return node;
      },getMode:function (path) {
        var stat;
        try {
          stat = fs.lstatSync(path);
          if (NODEFS.isWindows) {
            // On Windows, directories return permission bits 'rw-rw-rw-', even though they have 'rwxrwxrwx', so
            // propagate write bits to execute bits.
            stat.mode = stat.mode | ((stat.mode & 146) >> 1);
          }
        } catch (e) {
          if (!e.code) throw e;
          throw new FS.ErrnoError(ERRNO_CODES[e.code]);
        }
        return stat.mode;
      },realPath:function (node) {
        var parts = [];
        while (node.parent !== node) {
          parts.push(node.name);
          node = node.parent;
        }
        parts.push(node.mount.opts.root);
        parts.reverse();
        return PATH.join.apply(null, parts);
      },flagsToPermissionStringMap:{0:"r",1:"r+",2:"r+",64:"r",65:"r+",66:"r+",129:"rx+",193:"rx+",514:"w+",577:"w",578:"w+",705:"wx",706:"wx+",1024:"a",1025:"a",1026:"a+",1089:"a",1090:"a+",1153:"ax",1154:"ax+",1217:"ax",1218:"ax+",4096:"rs",4098:"rs+"},flagsToPermissionString:function (flags) {
        flags &= ~0100000 /*O_LARGEFILE*/; // Ignore this flag from musl, otherwise node.js fails to open the file.
        if (flags in NODEFS.flagsToPermissionStringMap) {
          return NODEFS.flagsToPermissionStringMap[flags];
        } else {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
      },node_ops:{getattr:function (node) {
          var path = NODEFS.realPath(node);
          var stat;
          try {
            stat = fs.lstatSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          // node.js v0.10.20 doesn't report blksize and blocks on Windows. Fake them with default blksize of 4096.
          // See http://support.microsoft.com/kb/140365
          if (NODEFS.isWindows && !stat.blksize) {
            stat.blksize = 4096;
          }
          if (NODEFS.isWindows && !stat.blocks) {
            stat.blocks = (stat.size+stat.blksize-1)/stat.blksize|0;
          }
          return {
            dev: stat.dev,
            ino: stat.ino,
            mode: stat.mode,
            nlink: stat.nlink,
            uid: stat.uid,
            gid: stat.gid,
            rdev: stat.rdev,
            size: stat.size,
            atime: stat.atime,
            mtime: stat.mtime,
            ctime: stat.ctime,
            blksize: stat.blksize,
            blocks: stat.blocks
          };
        },setattr:function (node, attr) {
          var path = NODEFS.realPath(node);
          try {
            if (attr.mode !== undefined) {
              fs.chmodSync(path, attr.mode);
              // update the common node structure mode as well
              node.mode = attr.mode;
            }
            if (attr.timestamp !== undefined) {
              var date = new Date(attr.timestamp);
              fs.utimesSync(path, date, date);
            }
            if (attr.size !== undefined) {
              fs.truncateSync(path, attr.size);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },lookup:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          var mode = NODEFS.getMode(path);
          return NODEFS.createNode(parent, name, mode);
        },mknod:function (parent, name, mode, dev) {
          var node = NODEFS.createNode(parent, name, mode, dev);
          // create the backing node for this in the fs root as well
          var path = NODEFS.realPath(node);
          try {
            if (FS.isDir(node.mode)) {
              fs.mkdirSync(path, node.mode);
            } else {
              fs.writeFileSync(path, '', { mode: node.mode });
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return node;
        },rename:function (oldNode, newDir, newName) {
          var oldPath = NODEFS.realPath(oldNode);
          var newPath = PATH.join2(NODEFS.realPath(newDir), newName);
          try {
            fs.renameSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },unlink:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.unlinkSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },rmdir:function (parent, name) {
          var path = PATH.join2(NODEFS.realPath(parent), name);
          try {
            fs.rmdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readdir:function (node) {
          var path = NODEFS.realPath(node);
          try {
            return fs.readdirSync(path);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },symlink:function (parent, newName, oldPath) {
          var newPath = PATH.join2(NODEFS.realPath(parent), newName);
          try {
            fs.symlinkSync(oldPath, newPath);
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },readlink:function (node) {
          var path = NODEFS.realPath(node);
          try {
            path = fs.readlinkSync(path);
            path = NODEJS_PATH.relative(NODEJS_PATH.resolve(node.mount.opts.root), path);
            return path;
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        }},stream_ops:{open:function (stream) {
          var path = NODEFS.realPath(stream.node);
          try {
            if (FS.isFile(stream.node.mode)) {
              stream.nfd = fs.openSync(path, NODEFS.flagsToPermissionString(stream.flags));
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },close:function (stream) {
          try {
            if (FS.isFile(stream.node.mode) && stream.nfd) {
              fs.closeSync(stream.nfd);
            }
          } catch (e) {
            if (!e.code) throw e;
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
        },read:function (stream, buffer, offset, length, position) {
          if (length === 0) return 0; // node errors on 0 length reads
          // FIXME this is terrible.
          var nbuffer = new Buffer(length);
          var res;
          try {
            res = fs.readSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          if (res > 0) {
            for (var i = 0; i < res; i++) {
              buffer[offset + i] = nbuffer[i];
            }
          }
          return res;
        },write:function (stream, buffer, offset, length, position) {
          // FIXME this is terrible.
          var nbuffer = new Buffer(buffer.subarray(offset, offset + length));
          var res;
          try {
            res = fs.writeSync(stream.nfd, nbuffer, 0, length, position);
          } catch (e) {
            throw new FS.ErrnoError(ERRNO_CODES[e.code]);
          }
          return res;
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              try {
                var stat = fs.fstatSync(stream.nfd);
                position += stat.size;
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES[e.code]);
              }
            }
          }
  
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
  
          return position;
        }}};
  
  var WORKERFS={DIR_MODE:16895,FILE_MODE:33279,reader:null,mount:function (mount) {
        assert(ENVIRONMENT_IS_WORKER);
        if (!WORKERFS.reader) WORKERFS.reader = new FileReaderSync();
        var root = WORKERFS.createNode(null, '/', WORKERFS.DIR_MODE, 0);
        var createdParents = {};
        function ensureParent(path) {
          // return the parent node, creating subdirs as necessary
          var parts = path.split('/');
          var parent = root;
          for (var i = 0; i < parts.length-1; i++) {
            var curr = parts.slice(0, i+1).join('/');
            if (!createdParents[curr]) {
              createdParents[curr] = WORKERFS.createNode(parent, curr, WORKERFS.DIR_MODE, 0);
            }
            parent = createdParents[curr];
          }
          return parent;
        }
        function base(path) {
          var parts = path.split('/');
          return parts[parts.length-1];
        }
        // We also accept FileList here, by using Array.prototype
        Array.prototype.forEach.call(mount.opts["files"] || [], function(file) {
          WORKERFS.createNode(ensureParent(file.name), base(file.name), WORKERFS.FILE_MODE, 0, file, file.lastModifiedDate);
        });
        (mount.opts["blobs"] || []).forEach(function(obj) {
          WORKERFS.createNode(ensureParent(obj["name"]), base(obj["name"]), WORKERFS.FILE_MODE, 0, obj["data"]);
        });
        (mount.opts["packages"] || []).forEach(function(pack) {
          pack['metadata'].files.forEach(function(file) {
            var name = file.filename.substr(1); // remove initial slash
            WORKERFS.createNode(ensureParent(name), base(name), WORKERFS.FILE_MODE, 0, pack['blob'].slice(file.start, file.end));
          });
        });
        return root;
      },createNode:function (parent, name, mode, dev, contents, mtime) {
        var node = FS.createNode(parent, name, mode);
        node.mode = mode;
        node.node_ops = WORKERFS.node_ops;
        node.stream_ops = WORKERFS.stream_ops;
        node.timestamp = (mtime || new Date).getTime();
        assert(WORKERFS.FILE_MODE !== WORKERFS.DIR_MODE);
        if (mode === WORKERFS.FILE_MODE) {
          node.size = contents.size;
          node.contents = contents;
        } else {
          node.size = 4096;
          node.contents = {};
        }
        if (parent) {
          parent.contents[name] = node;
        }
        return node;
      },node_ops:{getattr:function (node) {
          return {
            dev: 1,
            ino: undefined,
            mode: node.mode,
            nlink: 1,
            uid: 0,
            gid: 0,
            rdev: undefined,
            size: node.size,
            atime: new Date(node.timestamp),
            mtime: new Date(node.timestamp),
            ctime: new Date(node.timestamp),
            blksize: 4096,
            blocks: Math.ceil(node.size / 4096),
          };
        },setattr:function (node, attr) {
          if (attr.mode !== undefined) {
            node.mode = attr.mode;
          }
          if (attr.timestamp !== undefined) {
            node.timestamp = attr.timestamp;
          }
        },lookup:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        },mknod:function (parent, name, mode, dev) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },rename:function (oldNode, newDir, newName) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },unlink:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },rmdir:function (parent, name) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },readdir:function (node) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },symlink:function (parent, newName, oldPath) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        },readlink:function (node) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }},stream_ops:{read:function (stream, buffer, offset, length, position) {
          if (position >= stream.node.size) return 0;
          var chunk = stream.node.contents.slice(position, position + length);
          var ab = WORKERFS.reader.readAsArrayBuffer(chunk);
          buffer.set(new Uint8Array(ab), offset);
          return chunk.size;
        },write:function (stream, buffer, offset, length, position) {
          throw new FS.ErrnoError(ERRNO_CODES.EIO);
        },llseek:function (stream, offset, whence) {
          var position = offset;
          if (whence === 1) {  // SEEK_CUR.
            position += stream.position;
          } else if (whence === 2) {  // SEEK_END.
            if (FS.isFile(stream.node.mode)) {
              position += stream.node.size;
            }
          }
          if (position < 0) {
            throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
          }
          return position;
        }}};
  
  var _stdin=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stdout=allocate(1, "i32*", ALLOC_STATIC);
  
  var _stderr=allocate(1, "i32*", ALLOC_STATIC);var FS={root:null,mounts:[],devices:[null],streams:[],nextInode:1,nameTable:null,currentPath:"/",initialized:false,ignorePermissions:true,trackingDelegate:{},tracking:{openFlags:{READ:1,WRITE:2}},ErrnoError:null,genericErrors:{},filesystems:null,handleFSError:function (e) {
        if (!(e instanceof FS.ErrnoError)) throw e + ' : ' + stackTrace();
        return ___setErrNo(e.errno);
      },lookupPath:function (path, opts) {
        path = PATH.resolve(FS.cwd(), path);
        opts = opts || {};
  
        if (!path) return { path: '', node: null };
  
        var defaults = {
          follow_mount: true,
          recurse_count: 0
        };
        for (var key in defaults) {
          if (opts[key] === undefined) {
            opts[key] = defaults[key];
          }
        }
  
        if (opts.recurse_count > 8) {  // max recursive lookup of 8
          throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
        }
  
        // split the path
        var parts = PATH.normalizeArray(path.split('/').filter(function(p) {
          return !!p;
        }), false);
  
        // start at the root
        var current = FS.root;
        var current_path = '/';
  
        for (var i = 0; i < parts.length; i++) {
          var islast = (i === parts.length-1);
          if (islast && opts.parent) {
            // stop resolving
            break;
          }
  
          current = FS.lookupNode(current, parts[i]);
          current_path = PATH.join2(current_path, parts[i]);
  
          // jump to the mount's root node if this is a mountpoint
          if (FS.isMountpoint(current)) {
            if (!islast || (islast && opts.follow_mount)) {
              current = current.mounted.root;
            }
          }
  
          // by default, lookupPath will not follow a symlink if it is the final path component.
          // setting opts.follow = true will override this behavior.
          if (!islast || opts.follow) {
            var count = 0;
            while (FS.isLink(current.mode)) {
              var link = FS.readlink(current_path);
              current_path = PATH.resolve(PATH.dirname(current_path), link);
  
              var lookup = FS.lookupPath(current_path, { recurse_count: opts.recurse_count });
              current = lookup.node;
  
              if (count++ > 40) {  // limit max consecutive symlinks to 40 (SYMLOOP_MAX).
                throw new FS.ErrnoError(ERRNO_CODES.ELOOP);
              }
            }
          }
        }
  
        return { path: current_path, node: current };
      },getPath:function (node) {
        var path;
        while (true) {
          if (FS.isRoot(node)) {
            var mount = node.mount.mountpoint;
            if (!path) return mount;
            return mount[mount.length-1] !== '/' ? mount + '/' + path : mount + path;
          }
          path = path ? node.name + '/' + path : node.name;
          node = node.parent;
        }
      },hashName:function (parentid, name) {
        var hash = 0;
  
  
        for (var i = 0; i < name.length; i++) {
          hash = ((hash << 5) - hash + name.charCodeAt(i)) | 0;
        }
        return ((parentid + hash) >>> 0) % FS.nameTable.length;
      },hashAddNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        node.name_next = FS.nameTable[hash];
        FS.nameTable[hash] = node;
      },hashRemoveNode:function (node) {
        var hash = FS.hashName(node.parent.id, node.name);
        if (FS.nameTable[hash] === node) {
          FS.nameTable[hash] = node.name_next;
        } else {
          var current = FS.nameTable[hash];
          while (current) {
            if (current.name_next === node) {
              current.name_next = node.name_next;
              break;
            }
            current = current.name_next;
          }
        }
      },lookupNode:function (parent, name) {
        var err = FS.mayLookup(parent);
        if (err) {
          throw new FS.ErrnoError(err, parent);
        }
        var hash = FS.hashName(parent.id, name);
        for (var node = FS.nameTable[hash]; node; node = node.name_next) {
          var nodeName = node.name;
          if (node.parent.id === parent.id && nodeName === name) {
            return node;
          }
        }
        // if we failed to find it in the cache, call into the VFS
        return FS.lookup(parent, name);
      },createNode:function (parent, name, mode, rdev) {
        if (!FS.FSNode) {
          FS.FSNode = function(parent, name, mode, rdev) {
            if (!parent) {
              parent = this;  // root node sets parent to itself
            }
            this.parent = parent;
            this.mount = parent.mount;
            this.mounted = null;
            this.id = FS.nextInode++;
            this.name = name;
            this.mode = mode;
            this.node_ops = {};
            this.stream_ops = {};
            this.rdev = rdev;
          };
  
          FS.FSNode.prototype = {};
  
          // compatibility
          var readMode = 292 | 73;
          var writeMode = 146;
  
          // NOTE we must use Object.defineProperties instead of individual calls to
          // Object.defineProperty in order to make closure compiler happy
          Object.defineProperties(FS.FSNode.prototype, {
            read: {
              get: function() { return (this.mode & readMode) === readMode; },
              set: function(val) { val ? this.mode |= readMode : this.mode &= ~readMode; }
            },
            write: {
              get: function() { return (this.mode & writeMode) === writeMode; },
              set: function(val) { val ? this.mode |= writeMode : this.mode &= ~writeMode; }
            },
            isFolder: {
              get: function() { return FS.isDir(this.mode); }
            },
            isDevice: {
              get: function() { return FS.isChrdev(this.mode); }
            }
          });
        }
  
        var node = new FS.FSNode(parent, name, mode, rdev);
  
        FS.hashAddNode(node);
  
        return node;
      },destroyNode:function (node) {
        FS.hashRemoveNode(node);
      },isRoot:function (node) {
        return node === node.parent;
      },isMountpoint:function (node) {
        return !!node.mounted;
      },isFile:function (mode) {
        return (mode & 61440) === 32768;
      },isDir:function (mode) {
        return (mode & 61440) === 16384;
      },isLink:function (mode) {
        return (mode & 61440) === 40960;
      },isChrdev:function (mode) {
        return (mode & 61440) === 8192;
      },isBlkdev:function (mode) {
        return (mode & 61440) === 24576;
      },isFIFO:function (mode) {
        return (mode & 61440) === 4096;
      },isSocket:function (mode) {
        return (mode & 49152) === 49152;
      },flagModes:{"r":0,"rs":1052672,"r+":2,"w":577,"wx":705,"xw":705,"w+":578,"wx+":706,"xw+":706,"a":1089,"ax":1217,"xa":1217,"a+":1090,"ax+":1218,"xa+":1218},modeStringToFlags:function (str) {
        var flags = FS.flagModes[str];
        if (typeof flags === 'undefined') {
          throw new Error('Unknown file open mode: ' + str);
        }
        return flags;
      },flagsToPermissionString:function (flag) {
        var perms = ['r', 'w', 'rw'][flag & 3];
        if ((flag & 512)) {
          perms += 'w';
        }
        return perms;
      },nodePermissions:function (node, perms) {
        if (FS.ignorePermissions) {
          return 0;
        }
        // return 0 if any user, group or owner bits are set.
        if (perms.indexOf('r') !== -1 && !(node.mode & 292)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('w') !== -1 && !(node.mode & 146)) {
          return ERRNO_CODES.EACCES;
        } else if (perms.indexOf('x') !== -1 && !(node.mode & 73)) {
          return ERRNO_CODES.EACCES;
        }
        return 0;
      },mayLookup:function (dir) {
        var err = FS.nodePermissions(dir, 'x');
        if (err) return err;
        if (!dir.node_ops.lookup) return ERRNO_CODES.EACCES;
        return 0;
      },mayCreate:function (dir, name) {
        try {
          var node = FS.lookupNode(dir, name);
          return ERRNO_CODES.EEXIST;
        } catch (e) {
        }
        return FS.nodePermissions(dir, 'wx');
      },mayDelete:function (dir, name, isdir) {
        var node;
        try {
          node = FS.lookupNode(dir, name);
        } catch (e) {
          return e.errno;
        }
        var err = FS.nodePermissions(dir, 'wx');
        if (err) {
          return err;
        }
        if (isdir) {
          if (!FS.isDir(node.mode)) {
            return ERRNO_CODES.ENOTDIR;
          }
          if (FS.isRoot(node) || FS.getPath(node) === FS.cwd()) {
            return ERRNO_CODES.EBUSY;
          }
        } else {
          if (FS.isDir(node.mode)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return 0;
      },mayOpen:function (node, flags) {
        if (!node) {
          return ERRNO_CODES.ENOENT;
        }
        if (FS.isLink(node.mode)) {
          return ERRNO_CODES.ELOOP;
        } else if (FS.isDir(node.mode)) {
          if ((flags & 2097155) !== 0 ||  // opening for write
              (flags & 512)) {
            return ERRNO_CODES.EISDIR;
          }
        }
        return FS.nodePermissions(node, FS.flagsToPermissionString(flags));
      },MAX_OPEN_FDS:4096,nextfd:function (fd_start, fd_end) {
        fd_start = fd_start || 0;
        fd_end = fd_end || FS.MAX_OPEN_FDS;
        for (var fd = fd_start; fd <= fd_end; fd++) {
          if (!FS.streams[fd]) {
            return fd;
          }
        }
        throw new FS.ErrnoError(ERRNO_CODES.EMFILE);
      },getStream:function (fd) {
        return FS.streams[fd];
      },createStream:function (stream, fd_start, fd_end) {
        if (!FS.FSStream) {
          FS.FSStream = function(){};
          FS.FSStream.prototype = {};
          // compatibility
          Object.defineProperties(FS.FSStream.prototype, {
            object: {
              get: function() { return this.node; },
              set: function(val) { this.node = val; }
            },
            isRead: {
              get: function() { return (this.flags & 2097155) !== 1; }
            },
            isWrite: {
              get: function() { return (this.flags & 2097155) !== 0; }
            },
            isAppend: {
              get: function() { return (this.flags & 1024); }
            }
          });
        }
        // clone it, so we can return an instance of FSStream
        var newStream = new FS.FSStream();
        for (var p in stream) {
          newStream[p] = stream[p];
        }
        stream = newStream;
        var fd = FS.nextfd(fd_start, fd_end);
        stream.fd = fd;
        FS.streams[fd] = stream;
        return stream;
      },closeStream:function (fd) {
        FS.streams[fd] = null;
      },chrdev_stream_ops:{open:function (stream) {
          var device = FS.getDevice(stream.node.rdev);
          // override node's stream ops with the device's
          stream.stream_ops = device.stream_ops;
          // forward the open call
          if (stream.stream_ops.open) {
            stream.stream_ops.open(stream);
          }
        },llseek:function () {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }},major:function (dev) {
        return ((dev) >> 8);
      },minor:function (dev) {
        return ((dev) & 0xff);
      },makedev:function (ma, mi) {
        return ((ma) << 8 | (mi));
      },registerDevice:function (dev, ops) {
        FS.devices[dev] = { stream_ops: ops };
      },getDevice:function (dev) {
        return FS.devices[dev];
      },getMounts:function (mount) {
        var mounts = [];
        var check = [mount];
  
        while (check.length) {
          var m = check.pop();
  
          mounts.push(m);
  
          check.push.apply(check, m.mounts);
        }
  
        return mounts;
      },syncfs:function (populate, callback) {
        if (typeof(populate) === 'function') {
          callback = populate;
          populate = false;
        }
  
        var mounts = FS.getMounts(FS.root.mount);
        var completed = 0;
  
        function done(err) {
          if (err) {
            if (!done.errored) {
              done.errored = true;
              return callback(err);
            }
            return;
          }
          if (++completed >= mounts.length) {
            callback(null);
          }
        };
  
        // sync all mounts
        mounts.forEach(function (mount) {
          if (!mount.type.syncfs) {
            return done(null);
          }
          mount.type.syncfs(mount, populate, done);
        });
      },mount:function (type, opts, mountpoint) {
        var root = mountpoint === '/';
        var pseudo = !mountpoint;
        var node;
  
        if (root && FS.root) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        } else if (!root && !pseudo) {
          var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
          mountpoint = lookup.path;  // use the absolute path
          node = lookup.node;
  
          if (FS.isMountpoint(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
          }
  
          if (!FS.isDir(node.mode)) {
            throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
          }
        }
  
        var mount = {
          type: type,
          opts: opts,
          mountpoint: mountpoint,
          mounts: []
        };
  
        // create a root node for the fs
        var mountRoot = type.mount(mount);
        mountRoot.mount = mount;
        mount.root = mountRoot;
  
        if (root) {
          FS.root = mountRoot;
        } else if (node) {
          // set as a mountpoint
          node.mounted = mount;
  
          // add the new mount to the current mount's children
          if (node.mount) {
            node.mount.mounts.push(mount);
          }
        }
  
        return mountRoot;
      },unmount:function (mountpoint) {
        var lookup = FS.lookupPath(mountpoint, { follow_mount: false });
  
        if (!FS.isMountpoint(lookup.node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
  
        // destroy the nodes for this mount, and all its child mounts
        var node = lookup.node;
        var mount = node.mounted;
        var mounts = FS.getMounts(mount);
  
        Object.keys(FS.nameTable).forEach(function (hash) {
          var current = FS.nameTable[hash];
  
          while (current) {
            var next = current.name_next;
  
            if (mounts.indexOf(current.mount) !== -1) {
              FS.destroyNode(current);
            }
  
            current = next;
          }
        });
  
        // no longer a mountpoint
        node.mounted = null;
  
        // remove this mount from the child mounts
        var idx = node.mount.mounts.indexOf(mount);
        assert(idx !== -1);
        node.mount.mounts.splice(idx, 1);
      },lookup:function (parent, name) {
        return parent.node_ops.lookup(parent, name);
      },mknod:function (path, mode, dev) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        if (!name || name === '.' || name === '..') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.mayCreate(parent, name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.mknod) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.mknod(parent, name, mode, dev);
      },create:function (path, mode) {
        mode = mode !== undefined ? mode : 438 /* 0666 */;
        mode &= 4095;
        mode |= 32768;
        return FS.mknod(path, mode, 0);
      },mkdir:function (path, mode) {
        mode = mode !== undefined ? mode : 511 /* 0777 */;
        mode &= 511 | 512;
        mode |= 16384;
        return FS.mknod(path, mode, 0);
      },mkdev:function (path, mode, dev) {
        if (typeof(dev) === 'undefined') {
          dev = mode;
          mode = 438 /* 0666 */;
        }
        mode |= 8192;
        return FS.mknod(path, mode, dev);
      },symlink:function (oldpath, newpath) {
        if (!PATH.resolve(oldpath)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var lookup = FS.lookupPath(newpath, { parent: true });
        var parent = lookup.node;
        if (!parent) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        var newname = PATH.basename(newpath);
        var err = FS.mayCreate(parent, newname);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.symlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return parent.node_ops.symlink(parent, newname, oldpath);
      },rename:function (old_path, new_path) {
        var old_dirname = PATH.dirname(old_path);
        var new_dirname = PATH.dirname(new_path);
        var old_name = PATH.basename(old_path);
        var new_name = PATH.basename(new_path);
        // parents must exist
        var lookup, old_dir, new_dir;
        try {
          lookup = FS.lookupPath(old_path, { parent: true });
          old_dir = lookup.node;
          lookup = FS.lookupPath(new_path, { parent: true });
          new_dir = lookup.node;
        } catch (e) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        if (!old_dir || !new_dir) throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        // need to be part of the same mount
        if (old_dir.mount !== new_dir.mount) {
          throw new FS.ErrnoError(ERRNO_CODES.EXDEV);
        }
        // source must exist
        var old_node = FS.lookupNode(old_dir, old_name);
        // old path should not be an ancestor of the new path
        var relative = PATH.relative(old_path, new_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        // new path should not be an ancestor of the old path
        relative = PATH.relative(new_path, old_dirname);
        if (relative.charAt(0) !== '.') {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTEMPTY);
        }
        // see if the new path already exists
        var new_node;
        try {
          new_node = FS.lookupNode(new_dir, new_name);
        } catch (e) {
          // not fatal
        }
        // early out if nothing needs to change
        if (old_node === new_node) {
          return;
        }
        // we'll need to delete the old entry
        var isdir = FS.isDir(old_node.mode);
        var err = FS.mayDelete(old_dir, old_name, isdir);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        // need delete permissions if we'll be overwriting.
        // need create permissions if new doesn't already exist.
        err = new_node ?
          FS.mayDelete(new_dir, new_name, isdir) :
          FS.mayCreate(new_dir, new_name);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!old_dir.node_ops.rename) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(old_node) || (new_node && FS.isMountpoint(new_node))) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        // if we are going to change the parent, check write permissions
        if (new_dir !== old_dir) {
          err = FS.nodePermissions(old_dir, 'w');
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        try {
          if (FS.trackingDelegate['willMovePath']) {
            FS.trackingDelegate['willMovePath'](old_path, new_path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
        // remove the node from the lookup hash
        FS.hashRemoveNode(old_node);
        // do the underlying fs rename
        try {
          old_dir.node_ops.rename(old_node, new_dir, new_name);
        } catch (e) {
          throw e;
        } finally {
          // add the node back to the hash (in case node_ops.rename
          // changed its name)
          FS.hashAddNode(old_node);
        }
        try {
          if (FS.trackingDelegate['onMovePath']) FS.trackingDelegate['onMovePath'](old_path, new_path);
        } catch(e) {
          console.log("FS.trackingDelegate['onMovePath']('"+old_path+"', '"+new_path+"') threw an exception: " + e.message);
        }
      },rmdir:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, true);
        if (err) {
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.rmdir) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.rmdir(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        if (!node.node_ops.readdir) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        return node.node_ops.readdir(node);
      },unlink:function (path) {
        var lookup = FS.lookupPath(path, { parent: true });
        var parent = lookup.node;
        var name = PATH.basename(path);
        var node = FS.lookupNode(parent, name);
        var err = FS.mayDelete(parent, name, false);
        if (err) {
          // POSIX says unlink should set EPERM, not EISDIR
          if (err === ERRNO_CODES.EISDIR) err = ERRNO_CODES.EPERM;
          throw new FS.ErrnoError(err);
        }
        if (!parent.node_ops.unlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isMountpoint(node)) {
          throw new FS.ErrnoError(ERRNO_CODES.EBUSY);
        }
        try {
          if (FS.trackingDelegate['willDeletePath']) {
            FS.trackingDelegate['willDeletePath'](path);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['willDeletePath']('"+path+"') threw an exception: " + e.message);
        }
        parent.node_ops.unlink(parent, name);
        FS.destroyNode(node);
        try {
          if (FS.trackingDelegate['onDeletePath']) FS.trackingDelegate['onDeletePath'](path);
        } catch(e) {
          console.log("FS.trackingDelegate['onDeletePath']('"+path+"') threw an exception: " + e.message);
        }
      },readlink:function (path) {
        var lookup = FS.lookupPath(path);
        var link = lookup.node;
        if (!link) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!link.node_ops.readlink) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        return PATH.resolve(FS.getPath(link.parent), link.node_ops.readlink(link));
      },stat:function (path, dontFollow) {
        var lookup = FS.lookupPath(path, { follow: !dontFollow });
        var node = lookup.node;
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        if (!node.node_ops.getattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        return node.node_ops.getattr(node);
      },lstat:function (path) {
        return FS.stat(path, true);
      },chmod:function (path, mode, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          mode: (mode & 4095) | (node.mode & ~4095),
          timestamp: Date.now()
        });
      },lchmod:function (path, mode) {
        FS.chmod(path, mode, true);
      },fchmod:function (fd, mode) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chmod(stream.node, mode);
      },chown:function (path, uid, gid, dontFollow) {
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: !dontFollow });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        node.node_ops.setattr(node, {
          timestamp: Date.now()
          // we ignore the uid / gid for now
        });
      },lchown:function (path, uid, gid) {
        FS.chown(path, uid, gid, true);
      },fchown:function (fd, uid, gid) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        FS.chown(stream.node, uid, gid);
      },truncate:function (path, len) {
        if (len < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var node;
        if (typeof path === 'string') {
          var lookup = FS.lookupPath(path, { follow: true });
          node = lookup.node;
        } else {
          node = path;
        }
        if (!node.node_ops.setattr) {
          throw new FS.ErrnoError(ERRNO_CODES.EPERM);
        }
        if (FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!FS.isFile(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var err = FS.nodePermissions(node, 'w');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        node.node_ops.setattr(node, {
          size: len,
          timestamp: Date.now()
        });
      },ftruncate:function (fd, len) {
        var stream = FS.getStream(fd);
        if (!stream) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        FS.truncate(stream.node, len);
      },utime:function (path, atime, mtime) {
        var lookup = FS.lookupPath(path, { follow: true });
        var node = lookup.node;
        node.node_ops.setattr(node, {
          timestamp: Math.max(atime, mtime)
        });
      },open:function (path, flags, mode, fd_start, fd_end) {
        if (path === "") {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        flags = typeof flags === 'string' ? FS.modeStringToFlags(flags) : flags;
        mode = typeof mode === 'undefined' ? 438 /* 0666 */ : mode;
        if ((flags & 64)) {
          mode = (mode & 4095) | 32768;
        } else {
          mode = 0;
        }
        var node;
        if (typeof path === 'object') {
          node = path;
        } else {
          path = PATH.normalize(path);
          try {
            var lookup = FS.lookupPath(path, {
              follow: !(flags & 131072)
            });
            node = lookup.node;
          } catch (e) {
            // ignore
          }
        }
        // perhaps we need to create the node
        var created = false;
        if ((flags & 64)) {
          if (node) {
            // if O_CREAT and O_EXCL are set, error out if the node already exists
            if ((flags & 128)) {
              throw new FS.ErrnoError(ERRNO_CODES.EEXIST);
            }
          } else {
            // node doesn't exist, try to create it
            node = FS.mknod(path, mode, 0);
            created = true;
          }
        }
        if (!node) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOENT);
        }
        // can't truncate a device
        if (FS.isChrdev(node.mode)) {
          flags &= ~512;
        }
        // if asked only for a directory, then this must be one
        if ((flags & 65536) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        // check permissions, if this is not a file we just created now (it is ok to
        // create and write to a file with read-only permissions; it is read-only
        // for later use)
        if (!created) {
          var err = FS.mayOpen(node, flags);
          if (err) {
            throw new FS.ErrnoError(err);
          }
        }
        // do truncation if necessary
        if ((flags & 512)) {
          FS.truncate(node, 0);
        }
        // we've already handled these, don't pass down to the underlying vfs
        flags &= ~(128 | 512);
  
        // register the stream with the filesystem
        var stream = FS.createStream({
          node: node,
          path: FS.getPath(node),  // we want the absolute path to the node
          flags: flags,
          seekable: true,
          position: 0,
          stream_ops: node.stream_ops,
          // used by the file family libc calls (fopen, fwrite, ferror, etc.)
          ungotten: [],
          error: false
        }, fd_start, fd_end);
        // call the new stream's open function
        if (stream.stream_ops.open) {
          stream.stream_ops.open(stream);
        }
        if (Module['logReadFiles'] && !(flags & 1)) {
          if (!FS.readFiles) FS.readFiles = {};
          if (!(path in FS.readFiles)) {
            FS.readFiles[path] = 1;
            Module['printErr']('read file: ' + path);
          }
        }
        try {
          if (FS.trackingDelegate['onOpenFile']) {
            var trackingFlags = 0;
            if ((flags & 2097155) !== 1) {
              trackingFlags |= FS.tracking.openFlags.READ;
            }
            if ((flags & 2097155) !== 0) {
              trackingFlags |= FS.tracking.openFlags.WRITE;
            }
            FS.trackingDelegate['onOpenFile'](path, trackingFlags);
          }
        } catch(e) {
          console.log("FS.trackingDelegate['onOpenFile']('"+path+"', flags) threw an exception: " + e.message);
        }
        return stream;
      },close:function (stream) {
        if (stream.getdents) stream.getdents = null; // free readdir state
        try {
          if (stream.stream_ops.close) {
            stream.stream_ops.close(stream);
          }
        } catch (e) {
          throw e;
        } finally {
          FS.closeStream(stream.fd);
        }
      },llseek:function (stream, offset, whence) {
        if (!stream.seekable || !stream.stream_ops.llseek) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        stream.position = stream.stream_ops.llseek(stream, offset, whence);
        stream.ungotten = [];
        return stream.position;
      },read:function (stream, buffer, offset, length, position) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.read) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesRead = stream.stream_ops.read(stream, buffer, offset, length, position);
        if (!seeking) stream.position += bytesRead;
        return bytesRead;
      },write:function (stream, buffer, offset, length, position, canOwn) {
        if (length < 0 || position < 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (FS.isDir(stream.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.EISDIR);
        }
        if (!stream.stream_ops.write) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if (stream.flags & 1024) {
          // seek to the end before writing in append mode
          FS.llseek(stream, 0, 2);
        }
        var seeking = true;
        if (typeof position === 'undefined') {
          position = stream.position;
          seeking = false;
        } else if (!stream.seekable) {
          throw new FS.ErrnoError(ERRNO_CODES.ESPIPE);
        }
        var bytesWritten = stream.stream_ops.write(stream, buffer, offset, length, position, canOwn);
        if (!seeking) stream.position += bytesWritten;
        try {
          if (stream.path && FS.trackingDelegate['onWriteToFile']) FS.trackingDelegate['onWriteToFile'](stream.path);
        } catch(e) {
          console.log("FS.trackingDelegate['onWriteToFile']('"+path+"') threw an exception: " + e.message);
        }
        return bytesWritten;
      },allocate:function (stream, offset, length) {
        if (offset < 0 || length <= 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EINVAL);
        }
        if ((stream.flags & 2097155) === 0) {
          throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        }
        if (!FS.isFile(stream.node.mode) && !FS.isDir(node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        if (!stream.stream_ops.allocate) {
          throw new FS.ErrnoError(ERRNO_CODES.EOPNOTSUPP);
        }
        stream.stream_ops.allocate(stream, offset, length);
      },mmap:function (stream, buffer, offset, length, position, prot, flags) {
        // TODO if PROT is PROT_WRITE, make sure we have write access
        if ((stream.flags & 2097155) === 1) {
          throw new FS.ErrnoError(ERRNO_CODES.EACCES);
        }
        if (!stream.stream_ops.mmap) {
          throw new FS.ErrnoError(ERRNO_CODES.ENODEV);
        }
        return stream.stream_ops.mmap(stream, buffer, offset, length, position, prot, flags);
      },msync:function (stream, buffer, offset, length, mmapFlags) {
        if (!stream || !stream.stream_ops.msync) {
          return 0;
        }
        return stream.stream_ops.msync(stream, buffer, offset, length, mmapFlags);
      },munmap:function (stream) {
        return 0;
      },ioctl:function (stream, cmd, arg) {
        if (!stream.stream_ops.ioctl) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTTY);
        }
        return stream.stream_ops.ioctl(stream, cmd, arg);
      },readFile:function (path, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'r';
        opts.encoding = opts.encoding || 'binary';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var ret;
        var stream = FS.open(path, opts.flags);
        var stat = FS.stat(path);
        var length = stat.size;
        var buf = new Uint8Array(length);
        FS.read(stream, buf, 0, length, 0);
        if (opts.encoding === 'utf8') {
          ret = UTF8ArrayToString(buf, 0);
        } else if (opts.encoding === 'binary') {
          ret = buf;
        }
        FS.close(stream);
        return ret;
      },writeFile:function (path, data, opts) {
        opts = opts || {};
        opts.flags = opts.flags || 'w';
        opts.encoding = opts.encoding || 'utf8';
        if (opts.encoding !== 'utf8' && opts.encoding !== 'binary') {
          throw new Error('Invalid encoding type "' + opts.encoding + '"');
        }
        var stream = FS.open(path, opts.flags, opts.mode);
        if (opts.encoding === 'utf8') {
          var buf = new Uint8Array(lengthBytesUTF8(data)+1);
          var actualNumBytes = stringToUTF8Array(data, buf, 0, buf.length);
          FS.write(stream, buf, 0, actualNumBytes, 0, opts.canOwn);
        } else if (opts.encoding === 'binary') {
          FS.write(stream, data, 0, data.length, 0, opts.canOwn);
        }
        FS.close(stream);
      },cwd:function () {
        return FS.currentPath;
      },chdir:function (path) {
        var lookup = FS.lookupPath(path, { follow: true });
        if (!FS.isDir(lookup.node.mode)) {
          throw new FS.ErrnoError(ERRNO_CODES.ENOTDIR);
        }
        var err = FS.nodePermissions(lookup.node, 'x');
        if (err) {
          throw new FS.ErrnoError(err);
        }
        FS.currentPath = lookup.path;
      },createDefaultDirectories:function () {
        FS.mkdir('/tmp');
        FS.mkdir('/home');
        FS.mkdir('/home/web_user');
      },createDefaultDevices:function () {
        // create /dev
        FS.mkdir('/dev');
        // setup /dev/null
        FS.registerDevice(FS.makedev(1, 3), {
          read: function() { return 0; },
          write: function(stream, buffer, offset, length, pos) { return length; }
        });
        FS.mkdev('/dev/null', FS.makedev(1, 3));
        // setup /dev/tty and /dev/tty1
        // stderr needs to print output using Module['printErr']
        // so we register a second tty just for it.
        TTY.register(FS.makedev(5, 0), TTY.default_tty_ops);
        TTY.register(FS.makedev(6, 0), TTY.default_tty1_ops);
        FS.mkdev('/dev/tty', FS.makedev(5, 0));
        FS.mkdev('/dev/tty1', FS.makedev(6, 0));
        // setup /dev/[u]random
        var random_device;
        if (typeof crypto !== 'undefined') {
          // for modern web browsers
          var randomBuffer = new Uint8Array(1);
          random_device = function() { crypto.getRandomValues(randomBuffer); return randomBuffer[0]; };
        } else if (ENVIRONMENT_IS_NODE) {
          // for nodejs
          random_device = function() { return require('crypto').randomBytes(1)[0]; };
        } else {
          // default for ES5 platforms
          random_device = function() { return (Math.random()*256)|0; };
        }
        FS.createDevice('/dev', 'random', random_device);
        FS.createDevice('/dev', 'urandom', random_device);
        // we're not going to emulate the actual shm device,
        // just create the tmp dirs that reside in it commonly
        FS.mkdir('/dev/shm');
        FS.mkdir('/dev/shm/tmp');
      },createSpecialDirectories:function () {
        // create /proc/self/fd which allows /proc/self/fd/6 => readlink gives the name of the stream for fd 6 (see test_unistd_ttyname)
        FS.mkdir('/proc');
        FS.mkdir('/proc/self');
        FS.mkdir('/proc/self/fd');
        FS.mount({
          mount: function() {
            var node = FS.createNode('/proc/self', 'fd', 16384 | 0777, 73);
            node.node_ops = {
              lookup: function(parent, name) {
                var fd = +name;
                var stream = FS.getStream(fd);
                if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
                var ret = {
                  parent: null,
                  mount: { mountpoint: 'fake' },
                  node_ops: { readlink: function() { return stream.path } }
                };
                ret.parent = ret; // make it look like a simple root node
                return ret;
              }
            };
            return node;
          }
        }, {}, '/proc/self/fd');
      },createStandardStreams:function () {
        // TODO deprecate the old functionality of a single
        // input / output callback and that utilizes FS.createDevice
        // and instead require a unique set of stream ops
  
        // by default, we symlink the standard streams to the
        // default tty devices. however, if the standard streams
        // have been overwritten we create a unique device for
        // them instead.
        if (Module['stdin']) {
          FS.createDevice('/dev', 'stdin', Module['stdin']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdin');
        }
        if (Module['stdout']) {
          FS.createDevice('/dev', 'stdout', null, Module['stdout']);
        } else {
          FS.symlink('/dev/tty', '/dev/stdout');
        }
        if (Module['stderr']) {
          FS.createDevice('/dev', 'stderr', null, Module['stderr']);
        } else {
          FS.symlink('/dev/tty1', '/dev/stderr');
        }
  
        // open default streams for the stdin, stdout and stderr devices
        var stdin = FS.open('/dev/stdin', 'r');
        assert(stdin.fd === 0, 'invalid handle for stdin (' + stdin.fd + ')');
  
        var stdout = FS.open('/dev/stdout', 'w');
        assert(stdout.fd === 1, 'invalid handle for stdout (' + stdout.fd + ')');
  
        var stderr = FS.open('/dev/stderr', 'w');
        assert(stderr.fd === 2, 'invalid handle for stderr (' + stderr.fd + ')');
      },ensureErrnoError:function () {
        if (FS.ErrnoError) return;
        FS.ErrnoError = function ErrnoError(errno, node) {
          //Module.printErr(stackTrace()); // useful for debugging
          this.node = node;
          this.setErrno = function(errno) {
            this.errno = errno;
            for (var key in ERRNO_CODES) {
              if (ERRNO_CODES[key] === errno) {
                this.code = key;
                break;
              }
            }
          };
          this.setErrno(errno);
          this.message = ERRNO_MESSAGES[errno];
        };
        FS.ErrnoError.prototype = new Error();
        FS.ErrnoError.prototype.constructor = FS.ErrnoError;
        // Some errors may happen quite a bit, to avoid overhead we reuse them (and suffer a lack of stack info)
        [ERRNO_CODES.ENOENT].forEach(function(code) {
          FS.genericErrors[code] = new FS.ErrnoError(code);
          FS.genericErrors[code].stack = '<generic error, no stack>';
        });
      },staticInit:function () {
        FS.ensureErrnoError();
  
        FS.nameTable = new Array(4096);
  
        FS.mount(MEMFS, {}, '/');
  
        FS.createDefaultDirectories();
        FS.createDefaultDevices();
        FS.createSpecialDirectories();
  
        FS.filesystems = {
          'MEMFS': MEMFS,
          'IDBFS': IDBFS,
          'NODEFS': NODEFS,
          'WORKERFS': WORKERFS,
        };
      },init:function (input, output, error) {
        assert(!FS.init.initialized, 'FS.init was previously called. If you want to initialize later with custom parameters, remove any earlier calls (note that one is automatically added to the generated code)');
        FS.init.initialized = true;
  
        FS.ensureErrnoError();
  
        // Allow Module.stdin etc. to provide defaults, if none explicitly passed to us here
        Module['stdin'] = input || Module['stdin'];
        Module['stdout'] = output || Module['stdout'];
        Module['stderr'] = error || Module['stderr'];
  
        FS.createStandardStreams();
      },quit:function () {
        FS.init.initialized = false;
        // force-flush all streams, so we get musl std streams printed out
        var fflush = Module['_fflush'];
        if (fflush) fflush(0);
        // close all of our streams
        for (var i = 0; i < FS.streams.length; i++) {
          var stream = FS.streams[i];
          if (!stream) {
            continue;
          }
          FS.close(stream);
        }
      },getMode:function (canRead, canWrite) {
        var mode = 0;
        if (canRead) mode |= 292 | 73;
        if (canWrite) mode |= 146;
        return mode;
      },joinPath:function (parts, forceRelative) {
        var path = PATH.join.apply(null, parts);
        if (forceRelative && path[0] == '/') path = path.substr(1);
        return path;
      },absolutePath:function (relative, base) {
        return PATH.resolve(base, relative);
      },standardizePath:function (path) {
        return PATH.normalize(path);
      },findObject:function (path, dontResolveLastLink) {
        var ret = FS.analyzePath(path, dontResolveLastLink);
        if (ret.exists) {
          return ret.object;
        } else {
          ___setErrNo(ret.error);
          return null;
        }
      },analyzePath:function (path, dontResolveLastLink) {
        // operate from within the context of the symlink's target
        try {
          var lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          path = lookup.path;
        } catch (e) {
        }
        var ret = {
          isRoot: false, exists: false, error: 0, name: null, path: null, object: null,
          parentExists: false, parentPath: null, parentObject: null
        };
        try {
          var lookup = FS.lookupPath(path, { parent: true });
          ret.parentExists = true;
          ret.parentPath = lookup.path;
          ret.parentObject = lookup.node;
          ret.name = PATH.basename(path);
          lookup = FS.lookupPath(path, { follow: !dontResolveLastLink });
          ret.exists = true;
          ret.path = lookup.path;
          ret.object = lookup.node;
          ret.name = lookup.node.name;
          ret.isRoot = lookup.path === '/';
        } catch (e) {
          ret.error = e.errno;
        };
        return ret;
      },createFolder:function (parent, name, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.mkdir(path, mode);
      },createPath:function (parent, path, canRead, canWrite) {
        parent = typeof parent === 'string' ? parent : FS.getPath(parent);
        var parts = path.split('/').reverse();
        while (parts.length) {
          var part = parts.pop();
          if (!part) continue;
          var current = PATH.join2(parent, part);
          try {
            FS.mkdir(current);
          } catch (e) {
            // ignore EEXIST
          }
          parent = current;
        }
        return current;
      },createFile:function (parent, name, properties, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(canRead, canWrite);
        return FS.create(path, mode);
      },createDataFile:function (parent, name, data, canRead, canWrite, canOwn) {
        var path = name ? PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name) : parent;
        var mode = FS.getMode(canRead, canWrite);
        var node = FS.create(path, mode);
        if (data) {
          if (typeof data === 'string') {
            var arr = new Array(data.length);
            for (var i = 0, len = data.length; i < len; ++i) arr[i] = data.charCodeAt(i);
            data = arr;
          }
          // make sure we can write to the file
          FS.chmod(node, mode | 146);
          var stream = FS.open(node, 'w');
          FS.write(stream, data, 0, data.length, 0, canOwn);
          FS.close(stream);
          FS.chmod(node, mode);
        }
        return node;
      },createDevice:function (parent, name, input, output) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        var mode = FS.getMode(!!input, !!output);
        if (!FS.createDevice.major) FS.createDevice.major = 64;
        var dev = FS.makedev(FS.createDevice.major++, 0);
        // Create a fake device that a set of stream ops to emulate
        // the old behavior.
        FS.registerDevice(dev, {
          open: function(stream) {
            stream.seekable = false;
          },
          close: function(stream) {
            // flush any pending line data
            if (output && output.buffer && output.buffer.length) {
              output(10);
            }
          },
          read: function(stream, buffer, offset, length, pos /* ignored */) {
            var bytesRead = 0;
            for (var i = 0; i < length; i++) {
              var result;
              try {
                result = input();
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
              if (result === undefined && bytesRead === 0) {
                throw new FS.ErrnoError(ERRNO_CODES.EAGAIN);
              }
              if (result === null || result === undefined) break;
              bytesRead++;
              buffer[offset+i] = result;
            }
            if (bytesRead) {
              stream.node.timestamp = Date.now();
            }
            return bytesRead;
          },
          write: function(stream, buffer, offset, length, pos) {
            for (var i = 0; i < length; i++) {
              try {
                output(buffer[offset+i]);
              } catch (e) {
                throw new FS.ErrnoError(ERRNO_CODES.EIO);
              }
            }
            if (length) {
              stream.node.timestamp = Date.now();
            }
            return i;
          }
        });
        return FS.mkdev(path, mode, dev);
      },createLink:function (parent, name, target, canRead, canWrite) {
        var path = PATH.join2(typeof parent === 'string' ? parent : FS.getPath(parent), name);
        return FS.symlink(target, path);
      },forceLoadFile:function (obj) {
        if (obj.isDevice || obj.isFolder || obj.link || obj.contents) return true;
        var success = true;
        if (typeof XMLHttpRequest !== 'undefined') {
          throw new Error("Lazy loading should have been performed (contents set) in createLazyFile, but it was not. Lazy loading only works in web workers. Use --embed-file or --preload-file in emcc on the main thread.");
        } else if (Module['read']) {
          // Command-line.
          try {
            // WARNING: Can't read binary files in V8's d8 or tracemonkey's js, as
            //          read() will try to parse UTF8.
            obj.contents = intArrayFromString(Module['read'](obj.url), true);
            obj.usedBytes = obj.contents.length;
          } catch (e) {
            success = false;
          }
        } else {
          throw new Error('Cannot load without read() or XMLHttpRequest.');
        }
        if (!success) ___setErrNo(ERRNO_CODES.EIO);
        return success;
      },createLazyFile:function (parent, name, url, canRead, canWrite) {
        // Lazy chunked Uint8Array (implements get and length from Uint8Array). Actual getting is abstracted away for eventual reuse.
        function LazyUint8Array() {
          this.lengthKnown = false;
          this.chunks = []; // Loaded chunks. Index is the chunk number
        }
        LazyUint8Array.prototype.get = function LazyUint8Array_get(idx) {
          if (idx > this.length-1 || idx < 0) {
            return undefined;
          }
          var chunkOffset = idx % this.chunkSize;
          var chunkNum = (idx / this.chunkSize)|0;
          return this.getter(chunkNum)[chunkOffset];
        }
        LazyUint8Array.prototype.setDataGetter = function LazyUint8Array_setDataGetter(getter) {
          this.getter = getter;
        }
        LazyUint8Array.prototype.cacheLength = function LazyUint8Array_cacheLength() {
          // Find length
          var xhr = new XMLHttpRequest();
          xhr.open('HEAD', url, false);
          xhr.send(null);
          if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
          var datalength = Number(xhr.getResponseHeader("Content-length"));
          var header;
          var hasByteServing = (header = xhr.getResponseHeader("Accept-Ranges")) && header === "bytes";
          var chunkSize = 1024*1024; // Chunk size in bytes
  
          if (!hasByteServing) chunkSize = datalength;
  
          // Function to get a range from the remote URL.
          var doXHR = (function(from, to) {
            if (from > to) throw new Error("invalid range (" + from + ", " + to + ") or no bytes requested!");
            if (to > datalength-1) throw new Error("only " + datalength + " bytes available! programmer error!");
  
            // TODO: Use mozResponseArrayBuffer, responseStream, etc. if available.
            var xhr = new XMLHttpRequest();
            xhr.open('GET', url, false);
            if (datalength !== chunkSize) xhr.setRequestHeader("Range", "bytes=" + from + "-" + to);
  
            // Some hints to the browser that we want binary data.
            if (typeof Uint8Array != 'undefined') xhr.responseType = 'arraybuffer';
            if (xhr.overrideMimeType) {
              xhr.overrideMimeType('text/plain; charset=x-user-defined');
            }
  
            xhr.send(null);
            if (!(xhr.status >= 200 && xhr.status < 300 || xhr.status === 304)) throw new Error("Couldn't load " + url + ". Status: " + xhr.status);
            if (xhr.response !== undefined) {
              return new Uint8Array(xhr.response || []);
            } else {
              return intArrayFromString(xhr.responseText || '', true);
            }
          });
          var lazyArray = this;
          lazyArray.setDataGetter(function(chunkNum) {
            var start = chunkNum * chunkSize;
            var end = (chunkNum+1) * chunkSize - 1; // including this byte
            end = Math.min(end, datalength-1); // if datalength-1 is selected, this is the last block
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") {
              lazyArray.chunks[chunkNum] = doXHR(start, end);
            }
            if (typeof(lazyArray.chunks[chunkNum]) === "undefined") throw new Error("doXHR failed!");
            return lazyArray.chunks[chunkNum];
          });
  
          this._length = datalength;
          this._chunkSize = chunkSize;
          this.lengthKnown = true;
        }
        if (typeof XMLHttpRequest !== 'undefined') {
          if (!ENVIRONMENT_IS_WORKER) throw 'Cannot do synchronous binary XHRs outside webworkers in modern browsers. Use --embed-file or --preload-file in emcc';
          var lazyArray = new LazyUint8Array();
          Object.defineProperty(lazyArray, "length", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._length;
              }
          });
          Object.defineProperty(lazyArray, "chunkSize", {
              get: function() {
                  if(!this.lengthKnown) {
                      this.cacheLength();
                  }
                  return this._chunkSize;
              }
          });
  
          var properties = { isDevice: false, contents: lazyArray };
        } else {
          var properties = { isDevice: false, url: url };
        }
  
        var node = FS.createFile(parent, name, properties, canRead, canWrite);
        // This is a total hack, but I want to get this lazy file code out of the
        // core of MEMFS. If we want to keep this lazy file concept I feel it should
        // be its own thin LAZYFS proxying calls to MEMFS.
        if (properties.contents) {
          node.contents = properties.contents;
        } else if (properties.url) {
          node.contents = null;
          node.url = properties.url;
        }
        // Add a function that defers querying the file size until it is asked the first time.
        Object.defineProperty(node, "usedBytes", {
            get: function() { return this.contents.length; }
        });
        // override each stream op with one that tries to force load the lazy file first
        var stream_ops = {};
        var keys = Object.keys(node.stream_ops);
        keys.forEach(function(key) {
          var fn = node.stream_ops[key];
          stream_ops[key] = function forceLoadLazyFile() {
            if (!FS.forceLoadFile(node)) {
              throw new FS.ErrnoError(ERRNO_CODES.EIO);
            }
            return fn.apply(null, arguments);
          };
        });
        // use a custom read function
        stream_ops.read = function stream_ops_read(stream, buffer, offset, length, position) {
          if (!FS.forceLoadFile(node)) {
            throw new FS.ErrnoError(ERRNO_CODES.EIO);
          }
          var contents = stream.node.contents;
          if (position >= contents.length)
            return 0;
          var size = Math.min(contents.length - position, length);
          assert(size >= 0);
          if (contents.slice) { // normal array
            for (var i = 0; i < size; i++) {
              buffer[offset + i] = contents[position + i];
            }
          } else {
            for (var i = 0; i < size; i++) { // LazyUint8Array from sync binary XHR
              buffer[offset + i] = contents.get(position + i);
            }
          }
          return size;
        };
        node.stream_ops = stream_ops;
        return node;
      },createPreloadedFile:function (parent, name, url, canRead, canWrite, onload, onerror, dontCreateFile, canOwn, preFinish) {
        Browser.init();
        // TODO we should allow people to just pass in a complete filename instead
        // of parent and name being that we just join them anyways
        var fullname = name ? PATH.resolve(PATH.join2(parent, name)) : parent;
        var dep = getUniqueRunDependency('cp ' + fullname); // might have several active requests for the same fullname
        function processData(byteArray) {
          function finish(byteArray) {
            if (preFinish) preFinish();
            if (!dontCreateFile) {
              FS.createDataFile(parent, name, byteArray, canRead, canWrite, canOwn);
            }
            if (onload) onload();
            removeRunDependency(dep);
          }
          var handled = false;
          Module['preloadPlugins'].forEach(function(plugin) {
            if (handled) return;
            if (plugin['canHandle'](fullname)) {
              plugin['handle'](byteArray, fullname, finish, function() {
                if (onerror) onerror();
                removeRunDependency(dep);
              });
              handled = true;
            }
          });
          if (!handled) finish(byteArray);
        }
        addRunDependency(dep);
        if (typeof url == 'string') {
          Browser.asyncLoad(url, function(byteArray) {
            processData(byteArray);
          }, onerror);
        } else {
          processData(url);
        }
      },indexedDB:function () {
        return window.indexedDB || window.mozIndexedDB || window.webkitIndexedDB || window.msIndexedDB;
      },DB_NAME:function () {
        return 'EM_FS_' + window.location.pathname;
      },DB_VERSION:20,DB_STORE_NAME:"FILE_DATA",saveFilesToDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = function openRequest_onupgradeneeded() {
          console.log('creating db');
          var db = openRequest.result;
          db.createObjectStore(FS.DB_STORE_NAME);
        };
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          var transaction = db.transaction([FS.DB_STORE_NAME], 'readwrite');
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var putRequest = files.put(FS.analyzePath(path).object.contents, path);
            putRequest.onsuccess = function putRequest_onsuccess() { ok++; if (ok + fail == total) finish() };
            putRequest.onerror = function putRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      },loadFilesFromDB:function (paths, onload, onerror) {
        onload = onload || function(){};
        onerror = onerror || function(){};
        var indexedDB = FS.indexedDB();
        try {
          var openRequest = indexedDB.open(FS.DB_NAME(), FS.DB_VERSION);
        } catch (e) {
          return onerror(e);
        }
        openRequest.onupgradeneeded = onerror; // no database to load from
        openRequest.onsuccess = function openRequest_onsuccess() {
          var db = openRequest.result;
          try {
            var transaction = db.transaction([FS.DB_STORE_NAME], 'readonly');
          } catch(e) {
            onerror(e);
            return;
          }
          var files = transaction.objectStore(FS.DB_STORE_NAME);
          var ok = 0, fail = 0, total = paths.length;
          function finish() {
            if (fail == 0) onload(); else onerror();
          }
          paths.forEach(function(path) {
            var getRequest = files.get(path);
            getRequest.onsuccess = function getRequest_onsuccess() {
              if (FS.analyzePath(path).exists) {
                FS.unlink(path);
              }
              FS.createDataFile(PATH.dirname(path), PATH.basename(path), getRequest.result, true, true, true);
              ok++;
              if (ok + fail == total) finish();
            };
            getRequest.onerror = function getRequest_onerror() { fail++; if (ok + fail == total) finish() };
          });
          transaction.onerror = onerror;
        };
        openRequest.onerror = onerror;
      }};var SYSCALLS={DEFAULT_POLLMASK:5,mappings:{},umask:511,calculateAt:function (dirfd, path) {
        if (path[0] !== '/') {
          // relative path
          var dir;
          if (dirfd === -100) {
            dir = FS.cwd();
          } else {
            var dirstream = FS.getStream(dirfd);
            if (!dirstream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
            dir = dirstream.path;
          }
          path = PATH.join2(dir, path);
        }
        return path;
      },doStat:function (func, path, buf) {
        try {
          var stat = func(path);
        } catch (e) {
          if (e && e.node && PATH.normalize(path) !== PATH.normalize(FS.getPath(e.node))) {
            // an error occurred while trying to look up the path; we should just report ENOTDIR
            return -ERRNO_CODES.ENOTDIR;
          }
          throw e;
        }
        HEAP32[((buf)>>2)]=stat.dev;
        HEAP32[(((buf)+(4))>>2)]=0;
        HEAP32[(((buf)+(8))>>2)]=stat.ino;
        HEAP32[(((buf)+(12))>>2)]=stat.mode;
        HEAP32[(((buf)+(16))>>2)]=stat.nlink;
        HEAP32[(((buf)+(20))>>2)]=stat.uid;
        HEAP32[(((buf)+(24))>>2)]=stat.gid;
        HEAP32[(((buf)+(28))>>2)]=stat.rdev;
        HEAP32[(((buf)+(32))>>2)]=0;
        HEAP32[(((buf)+(36))>>2)]=stat.size;
        HEAP32[(((buf)+(40))>>2)]=4096;
        HEAP32[(((buf)+(44))>>2)]=stat.blocks;
        HEAP32[(((buf)+(48))>>2)]=(stat.atime.getTime() / 1000)|0;
        HEAP32[(((buf)+(52))>>2)]=0;
        HEAP32[(((buf)+(56))>>2)]=(stat.mtime.getTime() / 1000)|0;
        HEAP32[(((buf)+(60))>>2)]=0;
        HEAP32[(((buf)+(64))>>2)]=(stat.ctime.getTime() / 1000)|0;
        HEAP32[(((buf)+(68))>>2)]=0;
        HEAP32[(((buf)+(72))>>2)]=stat.ino;
        return 0;
      },doMsync:function (addr, stream, len, flags) {
        var buffer = new Uint8Array(HEAPU8.subarray(addr, addr + len));
        FS.msync(stream, buffer, 0, len, flags);
      },doMkdir:function (path, mode) {
        // remove a trailing slash, if one - /a/b/ has basename of '', but
        // we want to create b in the context of this function
        path = PATH.normalize(path);
        if (path[path.length-1] === '/') path = path.substr(0, path.length-1);
        FS.mkdir(path, mode, 0);
        return 0;
      },doMknod:function (path, mode, dev) {
        // we don't want this in the JS API as it uses mknod to create all nodes.
        switch (mode & 61440) {
          case 32768:
          case 8192:
          case 24576:
          case 4096:
          case 49152:
            break;
          default: return -ERRNO_CODES.EINVAL;
        }
        FS.mknod(path, mode, dev);
        return 0;
      },doReadlink:function (path, buf, bufsize) {
        if (bufsize <= 0) return -ERRNO_CODES.EINVAL;
        var ret = FS.readlink(path);
        ret = ret.slice(0, Math.max(0, bufsize));
        writeStringToMemory(ret, buf, true);
        return ret.length;
      },doAccess:function (path, amode) {
        if (amode & ~7) {
          // need a valid mode
          return -ERRNO_CODES.EINVAL;
        }
        var node;
        var lookup = FS.lookupPath(path, { follow: true });
        node = lookup.node;
        var perms = '';
        if (amode & 4) perms += 'r';
        if (amode & 2) perms += 'w';
        if (amode & 1) perms += 'x';
        if (perms /* otherwise, they've just passed F_OK */ && FS.nodePermissions(node, perms)) {
          return -ERRNO_CODES.EACCES;
        }
        return 0;
      },doDup:function (path, flags, suggestFD) {
        var suggest = FS.getStream(suggestFD);
        if (suggest) FS.close(suggest);
        return FS.open(path, flags, 0, suggestFD, suggestFD).fd;
      },doReadv:function (stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[(((iov)+(i*8))>>2)];
          var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
          var curr = FS.read(stream, HEAP8,ptr, len, offset);
          if (curr < 0) return -1;
          ret += curr;
          if (curr < len) break; // nothing more to read
        }
        return ret;
      },doWritev:function (stream, iov, iovcnt, offset) {
        var ret = 0;
        for (var i = 0; i < iovcnt; i++) {
          var ptr = HEAP32[(((iov)+(i*8))>>2)];
          var len = HEAP32[(((iov)+(i*8 + 4))>>2)];
          var curr = FS.write(stream, HEAP8,ptr, len, offset);
          if (curr < 0) return -1;
          ret += curr;
        }
        return ret;
      },varargs:0,get:function (varargs) {
        SYSCALLS.varargs += 4;
        var ret = HEAP32[(((SYSCALLS.varargs)-(4))>>2)];
        return ret;
      },getStr:function () {
        var ret = Pointer_stringify(SYSCALLS.get());
        return ret;
      },getStreamFromFD:function () {
        var stream = FS.getStream(SYSCALLS.get());
        if (!stream) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        return stream;
      },getSocketFromFD:function () {
        var socket = SOCKFS.getSocket(SYSCALLS.get());
        if (!socket) throw new FS.ErrnoError(ERRNO_CODES.EBADF);
        return socket;
      },getSocketAddress:function (allowNull) {
        var addrp = SYSCALLS.get(), addrlen = SYSCALLS.get();
        if (allowNull && addrp === 0) return null;
        var info = __read_sockaddr(addrp, addrlen);
        if (info.errno) throw new FS.ErrnoError(info.errno);
        info.addr = DNS.lookup_addr(info.addr) || info.addr;
        return info;
      },get64:function () {
        var low = SYSCALLS.get(), high = SYSCALLS.get();
        if (low >= 0) assert(high === 0);
        else assert(high === -1);
        return low;
      },getZero:function () {
        assert(SYSCALLS.get() === 0);
      }};function ___syscall192(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // mmap2
      var addr = SYSCALLS.get(), len = SYSCALLS.get(), prot = SYSCALLS.get(), flags = SYSCALLS.get(), fd = SYSCALLS.get(), off = SYSCALLS.get()
      off <<= 12; // undo pgoffset
      var ptr;
      var allocated = false;
      if (fd === -1) {
        ptr = _malloc(len);
        if (!ptr) return -ERRNO_CODES.ENOMEM;
        _memset(ptr, 0, len);
        allocated = true;
      } else {
        var info = FS.getStream(fd);
        if (!info) return -ERRNO_CODES.EBADF;
        var res = FS.mmap(info, HEAPU8, addr, len, off, prot, flags);
        ptr = res.ptr;
        allocated = res.allocated;
      }
      SYSCALLS.mappings[ptr] = { malloc: ptr, len: len, allocated: allocated, fd: fd, flags: flags };
      return ptr;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall195(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_stat64
      var path = SYSCALLS.getStr(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.stat, path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall194(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // ftruncate64
      var fd = SYSCALLS.get(), zero = SYSCALLS.getZero(), length = SYSCALLS.get64();
      FS.ftruncate(fd, length);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall197(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_fstat64
      var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.stat, stream.path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall196(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // SYS_lstat64
      var path = SYSCALLS.getStr(), buf = SYSCALLS.get();
      return SYSCALLS.doStat(FS.lstat, path, buf);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  var EmterpreterAsync=undefined;function ___syscall118(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fsync
      var stream = SYSCALLS.getStreamFromFD();
      return 0; // we can't do anything synchronously; the in-memory FS is already synced to
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _sysconf(name) {
      // long sysconf(int name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/sysconf.html
      switch(name) {
        case 30: return PAGE_SIZE;
        case 85: return totalMemory / PAGE_SIZE;
        case 132:
        case 133:
        case 12:
        case 137:
        case 138:
        case 15:
        case 235:
        case 16:
        case 17:
        case 18:
        case 19:
        case 20:
        case 149:
        case 13:
        case 10:
        case 236:
        case 153:
        case 9:
        case 21:
        case 22:
        case 159:
        case 154:
        case 14:
        case 77:
        case 78:
        case 139:
        case 80:
        case 81:
        case 82:
        case 68:
        case 67:
        case 164:
        case 11:
        case 29:
        case 47:
        case 48:
        case 95:
        case 52:
        case 51:
        case 46:
          return 200809;
        case 79:
          return 0;
        case 27:
        case 246:
        case 127:
        case 128:
        case 23:
        case 24:
        case 160:
        case 161:
        case 181:
        case 182:
        case 242:
        case 183:
        case 184:
        case 243:
        case 244:
        case 245:
        case 165:
        case 178:
        case 179:
        case 49:
        case 50:
        case 168:
        case 169:
        case 175:
        case 170:
        case 171:
        case 172:
        case 97:
        case 76:
        case 32:
        case 173:
        case 35:
          return -1;
        case 176:
        case 177:
        case 7:
        case 155:
        case 8:
        case 157:
        case 125:
        case 126:
        case 92:
        case 93:
        case 129:
        case 130:
        case 131:
        case 94:
        case 91:
          return 1;
        case 74:
        case 60:
        case 69:
        case 70:
        case 4:
          return 1024;
        case 31:
        case 42:
        case 72:
          return 32;
        case 87:
        case 26:
        case 33:
          return 2147483647;
        case 34:
        case 1:
          return 47839;
        case 38:
        case 36:
          return 99;
        case 43:
        case 37:
          return 2048;
        case 0: return 2097152;
        case 3: return 65536;
        case 28: return 32768;
        case 44: return 32767;
        case 75: return 16384;
        case 39: return 1000;
        case 89: return 700;
        case 71: return 256;
        case 40: return 255;
        case 2: return 100;
        case 180: return 64;
        case 25: return 20;
        case 5: return 16;
        case 6: return 6;
        case 73: return 4;
        case 84: {
          if (typeof navigator === 'object') return navigator['hardwareConcurrency'] || 1;
          return 1;
        }
      }
      ___setErrNo(ERRNO_CODES.EINVAL);
      return -1;
    }

  
  function _usleep(useconds) {
      // int usleep(useconds_t useconds);
      // http://pubs.opengroup.org/onlinepubs/000095399/functions/usleep.html
      // We're single-threaded, so use a busy loop. Super-ugly.
      var msec = useconds / 1000;
      if ((ENVIRONMENT_IS_WEB || ENVIRONMENT_IS_WORKER) && self['performance'] && self['performance']['now']) {
        var start = self['performance']['now']();
        while (self['performance']['now']() - start < msec) {
          // Do nothing.
        }
      } else {
        var start = Date.now();
        while (Date.now() - start < msec) {
          // Do nothing.
        }
      }
      return 0;
    }
  Module["_usleep"] = _usleep;function _nanosleep(rqtp, rmtp) {
      // int nanosleep(const struct timespec  *rqtp, struct timespec *rmtp);
      var seconds = HEAP32[((rqtp)>>2)];
      var nanoseconds = HEAP32[(((rqtp)+(4))>>2)];
      if (rmtp !== 0) {
        HEAP32[((rmtp)>>2)]=0;
        HEAP32[(((rmtp)+(4))>>2)]=0;
      }
      return _usleep((seconds * 1e6) + (nanoseconds / 1000));
    }

  function ___syscall94(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fchmod
      var fd = SYSCALLS.get(), mode = SYSCALLS.get();
      FS.fchmod(fd, mode);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall91(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // munmap
      var addr = SYSCALLS.get(), len = SYSCALLS.get();
      // TODO: support unmmap'ing parts of allocations
      var info = SYSCALLS.mappings[addr];
      if (!info) return 0;
      if (len === info.len) {
        var stream = FS.getStream(info.fd);
        SYSCALLS.doMsync(addr, stream, len, info.flags)
        FS.munmap(stream);
        SYSCALLS.mappings[addr] = null;
        if (info.allocated) {
          _free(info.malloc);
        }
      }
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall212(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // chown32
      var path = SYSCALLS.getStr(), owner = SYSCALLS.get(), group = SYSCALLS.get();
      FS.chown(path, owner, group);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall54(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // ioctl
      var stream = SYSCALLS.getStreamFromFD(), op = SYSCALLS.get();
      switch (op) {
        case 21505: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return 0;
        }
        case 21506: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return 0; // no-op, not actually adjusting terminal settings
        }
        case 21519: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          var argp = SYSCALLS.get();
          HEAP32[((argp)>>2)]=0;
          return 0;
        }
        case 21520: {
          if (!stream.tty) return -ERRNO_CODES.ENOTTY;
          return -ERRNO_CODES.EINVAL; // not supported
        }
        case 21531: {
          var argp = SYSCALLS.get();
          return FS.ioctl(stream, op, argp);
        }
        default: abort('bad ioctl syscall ' + op);
      }
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  var ___tm_current=allocate(44, "i8", ALLOC_STATIC);
  
  
  var ___tm_timezone=allocate(intArrayFromString("GMT"), "i8", ALLOC_STATIC);
  
  
  var _tzname=allocate(8, "i32*", ALLOC_STATIC);
  
  var _daylight=allocate(1, "i32*", ALLOC_STATIC);
  
  var _timezone=allocate(1, "i32*", ALLOC_STATIC);function _tzset() {
      // TODO: Use (malleable) environment variables instead of system settings.
      if (_tzset.called) return;
      _tzset.called = true;
  
      HEAP32[((_timezone)>>2)]=-(new Date()).getTimezoneOffset() * 60;
  
      var winter = new Date(2000, 0, 1);
      var summer = new Date(2000, 6, 1);
      HEAP32[((_daylight)>>2)]=Number(winter.getTimezoneOffset() != summer.getTimezoneOffset());
  
      function extractZone(date) {
        var match = date.toTimeString().match(/\(([A-Za-z ]+)\)$/);
        return match ? match[1] : "GMT";
      };
      var winterName = extractZone(winter);
      var summerName = extractZone(summer);
      var winterNamePtr = allocate(intArrayFromString(winterName), 'i8', ALLOC_NORMAL);
      var summerNamePtr = allocate(intArrayFromString(summerName), 'i8', ALLOC_NORMAL);
      if (summer.getTimezoneOffset() < winter.getTimezoneOffset()) {
        // Northern hemisphere
        HEAP32[((_tzname)>>2)]=winterNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=summerNamePtr;
      } else {
        HEAP32[((_tzname)>>2)]=summerNamePtr;
        HEAP32[(((_tzname)+(4))>>2)]=winterNamePtr;
      }
    }function _localtime_r(time, tmPtr) {
      _tzset();
      var date = new Date(HEAP32[((time)>>2)]*1000);
      HEAP32[((tmPtr)>>2)]=date.getSeconds();
      HEAP32[(((tmPtr)+(4))>>2)]=date.getMinutes();
      HEAP32[(((tmPtr)+(8))>>2)]=date.getHours();
      HEAP32[(((tmPtr)+(12))>>2)]=date.getDate();
      HEAP32[(((tmPtr)+(16))>>2)]=date.getMonth();
      HEAP32[(((tmPtr)+(20))>>2)]=date.getFullYear()-1900;
      HEAP32[(((tmPtr)+(24))>>2)]=date.getDay();
  
      var start = new Date(date.getFullYear(), 0, 1);
      var yday = ((date.getTime() - start.getTime()) / (1000 * 60 * 60 * 24))|0;
      HEAP32[(((tmPtr)+(28))>>2)]=yday;
      HEAP32[(((tmPtr)+(36))>>2)]=-(date.getTimezoneOffset() * 60);
  
      // DST is in December in South
      var summerOffset = new Date(2000, 6, 1).getTimezoneOffset();
      var winterOffset = start.getTimezoneOffset();
      var dst = (date.getTimezoneOffset() == Math.min(winterOffset, summerOffset))|0;
      HEAP32[(((tmPtr)+(32))>>2)]=dst;
  
      var zonePtr = HEAP32[(((_tzname)+(dst ? Runtime.QUANTUM_SIZE : 0))>>2)];
      HEAP32[(((tmPtr)+(40))>>2)]=zonePtr;
  
      return tmPtr;
    }function _localtime(time) {
      return _localtime_r(time, ___tm_current);
    }

   
  Module["_bitshift64Lshr"] = _bitshift64Lshr;

  function ___syscall33(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // access
      var path = SYSCALLS.getStr(), amode = SYSCALLS.get();
      return SYSCALLS.doAccess(path, amode);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _BDtoIHigh=true;

  function _pthread_cleanup_push(routine, arg) {
      __ATEXIT__.push(function() { Runtime.dynCall('vi', routine, [arg]) })
      _pthread_cleanup_push.level = __ATEXIT__.length;
    }

  
  
  
  
  var _environ=allocate(1, "i32*", ALLOC_STATIC);var ___environ=_environ;function ___buildEnvironment(env) {
      // WARNING: Arbitrary limit!
      var MAX_ENV_VALUES = 64;
      var TOTAL_ENV_SIZE = 1024;
  
      // Statically allocate memory for the environment.
      var poolPtr;
      var envPtr;
      if (!___buildEnvironment.called) {
        ___buildEnvironment.called = true;
        // Set default values. Use string keys for Closure Compiler compatibility.
        ENV['USER'] = ENV['LOGNAME'] = 'web_user';
        ENV['PATH'] = '/';
        ENV['PWD'] = '/';
        ENV['HOME'] = '/home/web_user';
        ENV['LANG'] = 'C';
        ENV['_'] = Module['thisProgram'];
        // Allocate memory.
        poolPtr = allocate(TOTAL_ENV_SIZE, 'i8', ALLOC_STATIC);
        envPtr = allocate(MAX_ENV_VALUES * 4,
                          'i8*', ALLOC_STATIC);
        HEAP32[((envPtr)>>2)]=poolPtr;
        HEAP32[((_environ)>>2)]=envPtr;
      } else {
        envPtr = HEAP32[((_environ)>>2)];
        poolPtr = HEAP32[((envPtr)>>2)];
      }
  
      // Collect key=value lines.
      var strings = [];
      var totalSize = 0;
      for (var key in env) {
        if (typeof env[key] === 'string') {
          var line = key + '=' + env[key];
          strings.push(line);
          totalSize += line.length;
        }
      }
      if (totalSize > TOTAL_ENV_SIZE) {
        throw new Error('Environment size exceeded TOTAL_ENV_SIZE!');
      }
  
      // Make new.
      var ptrSize = 4;
      for (var i = 0; i < strings.length; i++) {
        var line = strings[i];
        writeAsciiToMemory(line, poolPtr);
        HEAP32[(((envPtr)+(i * ptrSize))>>2)]=poolPtr;
        poolPtr += line.length + 1;
      }
      HEAP32[(((envPtr)+(strings.length * ptrSize))>>2)]=0;
    }var ENV={};function _getenv(name) {
      // char *getenv(const char *name);
      // http://pubs.opengroup.org/onlinepubs/009695399/functions/getenv.html
      if (name === 0) return 0;
      name = Pointer_stringify(name);
      if (!ENV.hasOwnProperty(name)) return 0;
  
      if (_getenv.ret) _free(_getenv.ret);
      _getenv.ret = allocate(intArrayFromString(ENV[name]), 'i8', ALLOC_NORMAL);
      return _getenv.ret;
    }

  function _gettimeofday(ptr) {
      var now = Date.now();
      HEAP32[((ptr)>>2)]=(now/1000)|0; // seconds
      HEAP32[(((ptr)+(4))>>2)]=((now % 1000)*1000)|0; // microseconds
      return 0;
    }

  
  function _emscripten_memcpy_big(dest, src, num) {
      HEAPU8.set(HEAPU8.subarray(src, src+num), dest);
      return dest;
    } 
  Module["_memcpy"] = _memcpy;

  var _llvm_pow_f64=Math_pow;

  function _sbrk(bytes) {
      // Implement a Linux-like 'memory area' for our 'process'.
      // Changes the size of the memory area by |bytes|; returns the
      // address of the previous top ('break') of the memory area
      // We control the "dynamic" memory - DYNAMIC_BASE to DYNAMICTOP
      var self = _sbrk;
      if (!self.called) {
        DYNAMICTOP = alignMemoryPage(DYNAMICTOP); // make sure we start out aligned
        self.called = true;
        assert(Runtime.dynamicAlloc);
        self.alloc = Runtime.dynamicAlloc;
        Runtime.dynamicAlloc = function() { abort('cannot dynamically allocate, sbrk now has control') };
      }
      var ret = DYNAMICTOP;
      if (bytes != 0) {
        var success = self.alloc(bytes);
        if (!success) return -1 >>> 0; // sbrk failure code
      }
      return ret;  // Previous break location.
    }

   
  Module["_memmove"] = _memmove;

  var _BItoD=true;

  function ___syscall85(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // readlink
      var path = SYSCALLS.getStr(), buf = SYSCALLS.get(), bufsize = SYSCALLS.get();
      return SYSCALLS.doReadlink(path, buf, bufsize);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall207(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fchown32
      var fd = SYSCALLS.get(), owner = SYSCALLS.get(), group = SYSCALLS.get();
      FS.fchown(fd, owner, group);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  
  function ___syscall202(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // getgid32
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }function ___syscall201() {
  return ___syscall202.apply(null, arguments)
  }

  function ___syscall40(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // rmdir
      var path = SYSCALLS.getStr();
      FS.rmdir(path);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___assert_fail(condition, filename, line, func) {
      ABORT = true;
      throw 'Assertion failed: ' + Pointer_stringify(condition) + ', at: ' + [filename ? Pointer_stringify(filename) : 'unknown filename', line, func ? Pointer_stringify(func) : 'unknown function'] + ' at ' + stackTrace();
    }

  
  var PROCINFO={ppid:1,pid:42,sid:42,pgid:42};function ___syscall20(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // getpid
      return PROCINFO.pid;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _acos=Math_acos;

   
  Module["_memset"] = _memset;

  var _BDtoILow=true;

  function ___syscall39(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // mkdir
      var path = SYSCALLS.getStr(), mode = SYSCALLS.get();
      return SYSCALLS.doMkdir(path, mode);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _DtoIHigh=true;

   
  Module["_bitshift64Shl"] = _bitshift64Shl;

  function _abort() {
      Module['abort']();
    }

  var _tan=Math_tan;

  function ___lock() {}

  function ___unlock() {}

  var _asin=Math_asin;

  var _floor=Math_floor;

  var _sqrt=Math_sqrt;

  
  
  function _emscripten_set_main_loop_timing(mode, value) {
      Browser.mainLoop.timingMode = mode;
      Browser.mainLoop.timingValue = value;
  
      if (!Browser.mainLoop.func) {
        return 1; // Return non-zero on failure, can't set timing mode when there is no main loop.
      }
  
      if (mode == 0 /*EM_TIMING_SETTIMEOUT*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setTimeout() {
          setTimeout(Browser.mainLoop.runner, value); // doing this each time means that on exception, we stop
        };
        Browser.mainLoop.method = 'timeout';
      } else if (mode == 1 /*EM_TIMING_RAF*/) {
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_rAF() {
          Browser.requestAnimationFrame(Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'rAF';
      } else if (mode == 2 /*EM_TIMING_SETIMMEDIATE*/) {
        if (!window['setImmediate']) {
          // Emulate setImmediate. (note: not a complete polyfill, we don't emulate clearImmediate() to keep code size to minimum, since not needed)
          var setImmediates = [];
          var emscriptenMainLoopMessageId = '__emcc';
          function Browser_setImmediate_messageHandler(event) {
            if (event.source === window && event.data === emscriptenMainLoopMessageId) {
              event.stopPropagation();
              setImmediates.shift()();
            }
          }
          window.addEventListener("message", Browser_setImmediate_messageHandler, true);
          window['setImmediate'] = function Browser_emulated_setImmediate(func) {
            setImmediates.push(func);
            window.postMessage(emscriptenMainLoopMessageId, "*");
          }
        }
        Browser.mainLoop.scheduler = function Browser_mainLoop_scheduler_setImmediate() {
          window['setImmediate'](Browser.mainLoop.runner);
        };
        Browser.mainLoop.method = 'immediate';
      }
      return 0;
    }function _emscripten_set_main_loop(func, fps, simulateInfiniteLoop, arg, noSetTiming) {
      Module['noExitRuntime'] = true;
  
      assert(!Browser.mainLoop.func, 'emscripten_set_main_loop: there can only be one main loop function at once: call emscripten_cancel_main_loop to cancel the previous one before setting a new one with different parameters.');
  
      Browser.mainLoop.func = func;
      Browser.mainLoop.arg = arg;
  
      var thisMainLoopId = Browser.mainLoop.currentlyRunningMainloop;
  
      Browser.mainLoop.runner = function Browser_mainLoop_runner() {
        if (ABORT) return;
        if (Browser.mainLoop.queue.length > 0) {
          var start = Date.now();
          var blocker = Browser.mainLoop.queue.shift();
          blocker.func(blocker.arg);
          if (Browser.mainLoop.remainingBlockers) {
            var remaining = Browser.mainLoop.remainingBlockers;
            var next = remaining%1 == 0 ? remaining-1 : Math.floor(remaining);
            if (blocker.counted) {
              Browser.mainLoop.remainingBlockers = next;
            } else {
              // not counted, but move the progress along a tiny bit
              next = next + 0.5; // do not steal all the next one's progress
              Browser.mainLoop.remainingBlockers = (8*remaining + next)/9;
            }
          }
          console.log('main loop blocker "' + blocker.name + '" took ' + (Date.now() - start) + ' ms'); //, left: ' + Browser.mainLoop.remainingBlockers);
          Browser.mainLoop.updateStatus();
          setTimeout(Browser.mainLoop.runner, 0);
          return;
        }
  
        // catch pauses from non-main loop sources
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Implement very basic swap interval control
        Browser.mainLoop.currentFrameNumber = Browser.mainLoop.currentFrameNumber + 1 | 0;
        if (Browser.mainLoop.timingMode == 1/*EM_TIMING_RAF*/ && Browser.mainLoop.timingValue > 1 && Browser.mainLoop.currentFrameNumber % Browser.mainLoop.timingValue != 0) {
          // Not the scheduled time to render this frame - skip.
          Browser.mainLoop.scheduler();
          return;
        }
  
        // Signal GL rendering layer that processing of a new frame is about to start. This helps it optimize
        // VBO double-buffering and reduce GPU stalls.
  
        if (Browser.mainLoop.method === 'timeout' && Module.ctx) {
          Module.printErr('Looks like you are rendering without using requestAnimationFrame for the main loop. You should use 0 for the frame rate in emscripten_set_main_loop in order to use requestAnimationFrame, as that can greatly improve your frame rates!');
          Browser.mainLoop.method = ''; // just warn once per call to set main loop
        }
  
        Browser.mainLoop.runIter(function() {
          if (typeof arg !== 'undefined') {
            Runtime.dynCall('vi', func, [arg]);
          } else {
            Runtime.dynCall('v', func);
          }
        });
  
        // catch pauses from the main loop itself
        if (thisMainLoopId < Browser.mainLoop.currentlyRunningMainloop) return;
  
        // Queue new audio data. This is important to be right after the main loop invocation, so that we will immediately be able
        // to queue the newest produced audio samples.
        // TODO: Consider adding pre- and post- rAF callbacks so that GL.newRenderingFrameStarted() and SDL.audio.queueNewAudioData()
        //       do not need to be hardcoded into this function, but can be more generic.
        if (typeof SDL === 'object' && SDL.audio && SDL.audio.queueNewAudioData) SDL.audio.queueNewAudioData();
  
        Browser.mainLoop.scheduler();
      }
  
      if (!noSetTiming) {
        if (fps && fps > 0) _emscripten_set_main_loop_timing(0/*EM_TIMING_SETTIMEOUT*/, 1000.0 / fps);
        else _emscripten_set_main_loop_timing(1/*EM_TIMING_RAF*/, 1); // Do rAF by rendering each frame (no decimating)
  
        Browser.mainLoop.scheduler();
      }
  
      if (simulateInfiniteLoop) {
        throw 'SimulateInfiniteLoop';
      }
    }var Browser={mainLoop:{scheduler:null,method:"",currentlyRunningMainloop:0,func:null,arg:0,timingMode:0,timingValue:0,currentFrameNumber:0,queue:[],pause:function () {
          Browser.mainLoop.scheduler = null;
          Browser.mainLoop.currentlyRunningMainloop++; // Incrementing this signals the previous main loop that it's now become old, and it must return.
        },resume:function () {
          Browser.mainLoop.currentlyRunningMainloop++;
          var timingMode = Browser.mainLoop.timingMode;
          var timingValue = Browser.mainLoop.timingValue;
          var func = Browser.mainLoop.func;
          Browser.mainLoop.func = null;
          _emscripten_set_main_loop(func, 0, false, Browser.mainLoop.arg, true /* do not set timing and call scheduler, we will do it on the next lines */);
          _emscripten_set_main_loop_timing(timingMode, timingValue);
          Browser.mainLoop.scheduler();
        },updateStatus:function () {
          if (Module['setStatus']) {
            var message = Module['statusMessage'] || 'Please wait...';
            var remaining = Browser.mainLoop.remainingBlockers;
            var expected = Browser.mainLoop.expectedBlockers;
            if (remaining) {
              if (remaining < expected) {
                Module['setStatus'](message + ' (' + (expected - remaining) + '/' + expected + ')');
              } else {
                Module['setStatus'](message);
              }
            } else {
              Module['setStatus']('');
            }
          }
        },runIter:function (func) {
          if (ABORT) return;
          if (Module['preMainLoop']) {
            var preRet = Module['preMainLoop']();
            if (preRet === false) {
              return; // |return false| skips a frame
            }
          }
          try {
            func();
          } catch (e) {
            if (e instanceof ExitStatus) {
              return;
            } else {
              if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
              throw e;
            }
          }
          if (Module['postMainLoop']) Module['postMainLoop']();
        }},isFullScreen:false,pointerLock:false,moduleContextCreatedCallbacks:[],workers:[],init:function () {
        if (!Module["preloadPlugins"]) Module["preloadPlugins"] = []; // needs to exist even in workers
  
        if (Browser.initted) return;
        Browser.initted = true;
  
        try {
          new Blob();
          Browser.hasBlobConstructor = true;
        } catch(e) {
          Browser.hasBlobConstructor = false;
          console.log("warning: no blob constructor, cannot create blobs with mimetypes");
        }
        Browser.BlobBuilder = typeof MozBlobBuilder != "undefined" ? MozBlobBuilder : (typeof WebKitBlobBuilder != "undefined" ? WebKitBlobBuilder : (!Browser.hasBlobConstructor ? console.log("warning: no BlobBuilder") : null));
        Browser.URLObject = typeof window != "undefined" ? (window.URL ? window.URL : window.webkitURL) : undefined;
        if (!Module.noImageDecoding && typeof Browser.URLObject === 'undefined') {
          console.log("warning: Browser does not support creating object URLs. Built-in browser image decoding will not be available.");
          Module.noImageDecoding = true;
        }
  
        // Support for plugins that can process preloaded files. You can add more of these to
        // your app by creating and appending to Module.preloadPlugins.
        //
        // Each plugin is asked if it can handle a file based on the file's name. If it can,
        // it is given the file's raw data. When it is done, it calls a callback with the file's
        // (possibly modified) data. For example, a plugin might decompress a file, or it
        // might create some side data structure for use later (like an Image element, etc.).
  
        var imagePlugin = {};
        imagePlugin['canHandle'] = function imagePlugin_canHandle(name) {
          return !Module.noImageDecoding && /\.(jpg|jpeg|png|bmp)$/i.test(name);
        };
        imagePlugin['handle'] = function imagePlugin_handle(byteArray, name, onload, onerror) {
          var b = null;
          if (Browser.hasBlobConstructor) {
            try {
              b = new Blob([byteArray], { type: Browser.getMimetype(name) });
              if (b.size !== byteArray.length) { // Safari bug #118630
                // Safari's Blob can only take an ArrayBuffer
                b = new Blob([(new Uint8Array(byteArray)).buffer], { type: Browser.getMimetype(name) });
              }
            } catch(e) {
              Runtime.warnOnce('Blob constructor present but fails: ' + e + '; falling back to blob builder');
            }
          }
          if (!b) {
            var bb = new Browser.BlobBuilder();
            bb.append((new Uint8Array(byteArray)).buffer); // we need to pass a buffer, and must copy the array to get the right data range
            b = bb.getBlob();
          }
          var url = Browser.URLObject.createObjectURL(b);
          var img = new Image();
          img.onload = function img_onload() {
            assert(img.complete, 'Image ' + name + ' could not be decoded');
            var canvas = document.createElement('canvas');
            canvas.width = img.width;
            canvas.height = img.height;
            var ctx = canvas.getContext('2d');
            ctx.drawImage(img, 0, 0);
            Module["preloadedImages"][name] = canvas;
            Browser.URLObject.revokeObjectURL(url);
            if (onload) onload(byteArray);
          };
          img.onerror = function img_onerror(event) {
            console.log('Image ' + url + ' could not be decoded');
            if (onerror) onerror();
          };
          img.src = url;
        };
        Module['preloadPlugins'].push(imagePlugin);
  
        var audioPlugin = {};
        audioPlugin['canHandle'] = function audioPlugin_canHandle(name) {
          return !Module.noAudioDecoding && name.substr(-4) in { '.ogg': 1, '.wav': 1, '.mp3': 1 };
        };
        audioPlugin['handle'] = function audioPlugin_handle(byteArray, name, onload, onerror) {
          var done = false;
          function finish(audio) {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = audio;
            if (onload) onload(byteArray);
          }
          function fail() {
            if (done) return;
            done = true;
            Module["preloadedAudios"][name] = new Audio(); // empty shim
            if (onerror) onerror();
          }
          if (Browser.hasBlobConstructor) {
            try {
              var b = new Blob([byteArray], { type: Browser.getMimetype(name) });
            } catch(e) {
              return fail();
            }
            var url = Browser.URLObject.createObjectURL(b); // XXX we never revoke this!
            var audio = new Audio();
            audio.addEventListener('canplaythrough', function() { finish(audio) }, false); // use addEventListener due to chromium bug 124926
            audio.onerror = function audio_onerror(event) {
              if (done) return;
              console.log('warning: browser could not fully decode audio ' + name + ', trying slower base64 approach');
              function encode64(data) {
                var BASE = 'ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789+/';
                var PAD = '=';
                var ret = '';
                var leftchar = 0;
                var leftbits = 0;
                for (var i = 0; i < data.length; i++) {
                  leftchar = (leftchar << 8) | data[i];
                  leftbits += 8;
                  while (leftbits >= 6) {
                    var curr = (leftchar >> (leftbits-6)) & 0x3f;
                    leftbits -= 6;
                    ret += BASE[curr];
                  }
                }
                if (leftbits == 2) {
                  ret += BASE[(leftchar&3) << 4];
                  ret += PAD + PAD;
                } else if (leftbits == 4) {
                  ret += BASE[(leftchar&0xf) << 2];
                  ret += PAD;
                }
                return ret;
              }
              audio.src = 'data:audio/x-' + name.substr(-3) + ';base64,' + encode64(byteArray);
              finish(audio); // we don't wait for confirmation this worked - but it's worth trying
            };
            audio.src = url;
            // workaround for chrome bug 124926 - we do not always get oncanplaythrough or onerror
            Browser.safeSetTimeout(function() {
              finish(audio); // try to use it even though it is not necessarily ready to play
            }, 10000);
          } else {
            return fail();
          }
        };
        Module['preloadPlugins'].push(audioPlugin);
  
        // Canvas event setup
  
        var canvas = Module['canvas'];
        function pointerLockChange() {
          Browser.pointerLock = document['pointerLockElement'] === canvas ||
                                document['mozPointerLockElement'] === canvas ||
                                document['webkitPointerLockElement'] === canvas ||
                                document['msPointerLockElement'] === canvas;
        }
        if (canvas) {
          // forced aspect ratio can be enabled by defining 'forcedAspectRatio' on Module
          // Module['forcedAspectRatio'] = 4 / 3;
          
          canvas.requestPointerLock = canvas['requestPointerLock'] ||
                                      canvas['mozRequestPointerLock'] ||
                                      canvas['webkitRequestPointerLock'] ||
                                      canvas['msRequestPointerLock'] ||
                                      function(){};
          canvas.exitPointerLock = document['exitPointerLock'] ||
                                   document['mozExitPointerLock'] ||
                                   document['webkitExitPointerLock'] ||
                                   document['msExitPointerLock'] ||
                                   function(){}; // no-op if function does not exist
          canvas.exitPointerLock = canvas.exitPointerLock.bind(document);
  
  
          document.addEventListener('pointerlockchange', pointerLockChange, false);
          document.addEventListener('mozpointerlockchange', pointerLockChange, false);
          document.addEventListener('webkitpointerlockchange', pointerLockChange, false);
          document.addEventListener('mspointerlockchange', pointerLockChange, false);
  
          if (Module['elementPointerLock']) {
            canvas.addEventListener("click", function(ev) {
              if (!Browser.pointerLock && canvas.requestPointerLock) {
                canvas.requestPointerLock();
                ev.preventDefault();
              }
            }, false);
          }
        }
      },createContext:function (canvas, useWebGL, setInModule, webGLContextAttributes) {
        if (useWebGL && Module.ctx && canvas == Module.canvas) return Module.ctx; // no need to recreate GL context if it's already been created for this canvas.
  
        var ctx;
        var contextHandle;
        if (useWebGL) {
          // For GLES2/desktop GL compatibility, adjust a few defaults to be different to WebGL defaults, so that they align better with the desktop defaults.
          var contextAttributes = {
            antialias: false,
            alpha: false
          };
  
          if (webGLContextAttributes) {
            for (var attribute in webGLContextAttributes) {
              contextAttributes[attribute] = webGLContextAttributes[attribute];
            }
          }
  
          contextHandle = GL.createContext(canvas, contextAttributes);
          if (contextHandle) {
            ctx = GL.getContext(contextHandle).GLctx;
          }
          // Set the background of the WebGL canvas to black
          canvas.style.backgroundColor = "black";
        } else {
          ctx = canvas.getContext('2d');
        }
  
        if (!ctx) return null;
  
        if (setInModule) {
          if (!useWebGL) assert(typeof GLctx === 'undefined', 'cannot set in module if GLctx is used, but we are a non-GL context that would replace it');
  
          Module.ctx = ctx;
          if (useWebGL) GL.makeContextCurrent(contextHandle);
          Module.useWebGL = useWebGL;
          Browser.moduleContextCreatedCallbacks.forEach(function(callback) { callback() });
          Browser.init();
        }
        return ctx;
      },destroyContext:function (canvas, useWebGL, setInModule) {},fullScreenHandlersInstalled:false,lockPointer:undefined,resizeCanvas:undefined,requestFullScreen:function (lockPointer, resizeCanvas, vrDevice) {
        Browser.lockPointer = lockPointer;
        Browser.resizeCanvas = resizeCanvas;
        Browser.vrDevice = vrDevice;
        if (typeof Browser.lockPointer === 'undefined') Browser.lockPointer = true;
        if (typeof Browser.resizeCanvas === 'undefined') Browser.resizeCanvas = false;
        if (typeof Browser.vrDevice === 'undefined') Browser.vrDevice = null;
  
        var canvas = Module['canvas'];
        function fullScreenChange() {
          Browser.isFullScreen = false;
          var canvasContainer = canvas.parentNode;
          if ((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
               document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
               document['fullScreenElement'] || document['fullscreenElement'] ||
               document['msFullScreenElement'] || document['msFullscreenElement'] ||
               document['webkitCurrentFullScreenElement']) === canvasContainer) {
            canvas.cancelFullScreen = document['cancelFullScreen'] ||
                                      document['mozCancelFullScreen'] ||
                                      document['webkitCancelFullScreen'] ||
                                      document['msExitFullscreen'] ||
                                      document['exitFullscreen'] ||
                                      function() {};
            canvas.cancelFullScreen = canvas.cancelFullScreen.bind(document);
            if (Browser.lockPointer) canvas.requestPointerLock();
            Browser.isFullScreen = true;
            if (Browser.resizeCanvas) Browser.setFullScreenCanvasSize();
          } else {
            
            // remove the full screen specific parent of the canvas again to restore the HTML structure from before going full screen
            canvasContainer.parentNode.insertBefore(canvas, canvasContainer);
            canvasContainer.parentNode.removeChild(canvasContainer);
            
            if (Browser.resizeCanvas) Browser.setWindowedCanvasSize();
          }
          if (Module['onFullScreen']) Module['onFullScreen'](Browser.isFullScreen);
          Browser.updateCanvasDimensions(canvas);
        }
  
        if (!Browser.fullScreenHandlersInstalled) {
          Browser.fullScreenHandlersInstalled = true;
          document.addEventListener('fullscreenchange', fullScreenChange, false);
          document.addEventListener('mozfullscreenchange', fullScreenChange, false);
          document.addEventListener('webkitfullscreenchange', fullScreenChange, false);
          document.addEventListener('MSFullscreenChange', fullScreenChange, false);
        }
  
        // create a new parent to ensure the canvas has no siblings. this allows browsers to optimize full screen performance when its parent is the full screen root
        var canvasContainer = document.createElement("div");
        canvas.parentNode.insertBefore(canvasContainer, canvas);
        canvasContainer.appendChild(canvas);
  
        // use parent of canvas as full screen root to allow aspect ratio correction (Firefox stretches the root to screen size)
        canvasContainer.requestFullScreen = canvasContainer['requestFullScreen'] ||
                                            canvasContainer['mozRequestFullScreen'] ||
                                            canvasContainer['msRequestFullscreen'] ||
                                           (canvasContainer['webkitRequestFullScreen'] ? function() { canvasContainer['webkitRequestFullScreen'](Element['ALLOW_KEYBOARD_INPUT']) } : null);
  
        if (vrDevice) {
          canvasContainer.requestFullScreen({ vrDisplay: vrDevice });
        } else {
          canvasContainer.requestFullScreen();
        }
      },nextRAF:0,fakeRequestAnimationFrame:function (func) {
        // try to keep 60fps between calls to here
        var now = Date.now();
        if (Browser.nextRAF === 0) {
          Browser.nextRAF = now + 1000/60;
        } else {
          while (now + 2 >= Browser.nextRAF) { // fudge a little, to avoid timer jitter causing us to do lots of delay:0
            Browser.nextRAF += 1000/60;
          }
        }
        var delay = Math.max(Browser.nextRAF - now, 0);
        setTimeout(func, delay);
      },requestAnimationFrame:function requestAnimationFrame(func) {
        if (typeof window === 'undefined') { // Provide fallback to setTimeout if window is undefined (e.g. in Node.js)
          Browser.fakeRequestAnimationFrame(func);
        } else {
          if (!window.requestAnimationFrame) {
            window.requestAnimationFrame = window['requestAnimationFrame'] ||
                                           window['mozRequestAnimationFrame'] ||
                                           window['webkitRequestAnimationFrame'] ||
                                           window['msRequestAnimationFrame'] ||
                                           window['oRequestAnimationFrame'] ||
                                           Browser.fakeRequestAnimationFrame;
          }
          window.requestAnimationFrame(func);
        }
      },safeCallback:function (func) {
        return function() {
          if (!ABORT) return func.apply(null, arguments);
        };
      },allowAsyncCallbacks:true,queuedAsyncCallbacks:[],pauseAsyncCallbacks:function () {
        Browser.allowAsyncCallbacks = false;
      },resumeAsyncCallbacks:function () { // marks future callbacks as ok to execute, and synchronously runs any remaining ones right now
        Browser.allowAsyncCallbacks = true;
        if (Browser.queuedAsyncCallbacks.length > 0) {
          var callbacks = Browser.queuedAsyncCallbacks;
          Browser.queuedAsyncCallbacks = [];
          callbacks.forEach(function(func) {
            func();
          });
        }
      },safeRequestAnimationFrame:function (func) {
        return Browser.requestAnimationFrame(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        });
      },safeSetTimeout:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setTimeout(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } else {
            Browser.queuedAsyncCallbacks.push(func);
          }
        }, timeout);
      },safeSetInterval:function (func, timeout) {
        Module['noExitRuntime'] = true;
        return setInterval(function() {
          if (ABORT) return;
          if (Browser.allowAsyncCallbacks) {
            func();
          } // drop it on the floor otherwise, next interval will kick in
        }, timeout);
      },getMimetype:function (name) {
        return {
          'jpg': 'image/jpeg',
          'jpeg': 'image/jpeg',
          'png': 'image/png',
          'bmp': 'image/bmp',
          'ogg': 'audio/ogg',
          'wav': 'audio/wav',
          'mp3': 'audio/mpeg'
        }[name.substr(name.lastIndexOf('.')+1)];
      },getUserMedia:function (func) {
        if(!window.getUserMedia) {
          window.getUserMedia = navigator['getUserMedia'] ||
                                navigator['mozGetUserMedia'];
        }
        window.getUserMedia(func);
      },getMovementX:function (event) {
        return event['movementX'] ||
               event['mozMovementX'] ||
               event['webkitMovementX'] ||
               0;
      },getMovementY:function (event) {
        return event['movementY'] ||
               event['mozMovementY'] ||
               event['webkitMovementY'] ||
               0;
      },getMouseWheelDelta:function (event) {
        var delta = 0;
        switch (event.type) {
          case 'DOMMouseScroll': 
            delta = event.detail;
            break;
          case 'mousewheel': 
            delta = event.wheelDelta;
            break;
          case 'wheel': 
            delta = event['deltaY'];
            break;
          default:
            throw 'unrecognized mouse wheel event: ' + event.type;
        }
        return delta;
      },mouseX:0,mouseY:0,mouseMovementX:0,mouseMovementY:0,touches:{},lastTouches:{},calculateMouseEvent:function (event) { // event should be mousemove, mousedown or mouseup
        if (Browser.pointerLock) {
          // When the pointer is locked, calculate the coordinates
          // based on the movement of the mouse.
          // Workaround for Firefox bug 764498
          if (event.type != 'mousemove' &&
              ('mozMovementX' in event)) {
            Browser.mouseMovementX = Browser.mouseMovementY = 0;
          } else {
            Browser.mouseMovementX = Browser.getMovementX(event);
            Browser.mouseMovementY = Browser.getMovementY(event);
          }
          
          // check if SDL is available
          if (typeof SDL != "undefined") {
          	Browser.mouseX = SDL.mouseX + Browser.mouseMovementX;
          	Browser.mouseY = SDL.mouseY + Browser.mouseMovementY;
          } else {
          	// just add the mouse delta to the current absolut mouse position
          	// FIXME: ideally this should be clamped against the canvas size and zero
          	Browser.mouseX += Browser.mouseMovementX;
          	Browser.mouseY += Browser.mouseMovementY;
          }        
        } else {
          // Otherwise, calculate the movement based on the changes
          // in the coordinates.
          var rect = Module["canvas"].getBoundingClientRect();
          var cw = Module["canvas"].width;
          var ch = Module["canvas"].height;
  
          // Neither .scrollX or .pageXOffset are defined in a spec, but
          // we prefer .scrollX because it is currently in a spec draft.
          // (see: http://www.w3.org/TR/2013/WD-cssom-view-20131217/)
          var scrollX = ((typeof window.scrollX !== 'undefined') ? window.scrollX : window.pageXOffset);
          var scrollY = ((typeof window.scrollY !== 'undefined') ? window.scrollY : window.pageYOffset);
  
          if (event.type === 'touchstart' || event.type === 'touchend' || event.type === 'touchmove') {
            var touch = event.touch;
            if (touch === undefined) {
              return; // the "touch" property is only defined in SDL
  
            }
            var adjustedX = touch.pageX - (scrollX + rect.left);
            var adjustedY = touch.pageY - (scrollY + rect.top);
  
            adjustedX = adjustedX * (cw / rect.width);
            adjustedY = adjustedY * (ch / rect.height);
  
            var coords = { x: adjustedX, y: adjustedY };
            
            if (event.type === 'touchstart') {
              Browser.lastTouches[touch.identifier] = coords;
              Browser.touches[touch.identifier] = coords;
            } else if (event.type === 'touchend' || event.type === 'touchmove') {
              var last = Browser.touches[touch.identifier];
              if (!last) last = coords;
              Browser.lastTouches[touch.identifier] = last;
              Browser.touches[touch.identifier] = coords;
            } 
            return;
          }
  
          var x = event.pageX - (scrollX + rect.left);
          var y = event.pageY - (scrollY + rect.top);
  
          // the canvas might be CSS-scaled compared to its backbuffer;
          // SDL-using content will want mouse coordinates in terms
          // of backbuffer units.
          x = x * (cw / rect.width);
          y = y * (ch / rect.height);
  
          Browser.mouseMovementX = x - Browser.mouseX;
          Browser.mouseMovementY = y - Browser.mouseY;
          Browser.mouseX = x;
          Browser.mouseY = y;
        }
      },xhrLoad:function (url, onload, onerror) {
        var xhr = new XMLHttpRequest();
        xhr.open('GET', url, true);
        xhr.responseType = 'arraybuffer';
        xhr.onload = function xhr_onload() {
          if (xhr.status == 200 || (xhr.status == 0 && xhr.response)) { // file URLs can return 0
            onload(xhr.response);
          } else {
            onerror();
          }
        };
        xhr.onerror = onerror;
        xhr.send(null);
      },asyncLoad:function (url, onload, onerror, noRunDep) {
        Browser.xhrLoad(url, function(arrayBuffer) {
          assert(arrayBuffer, 'Loading data file "' + url + '" failed (no arrayBuffer).');
          onload(new Uint8Array(arrayBuffer));
          if (!noRunDep) removeRunDependency('al ' + url);
        }, function(event) {
          if (onerror) {
            onerror();
          } else {
            throw 'Loading data file "' + url + '" failed.';
          }
        });
        if (!noRunDep) addRunDependency('al ' + url);
      },resizeListeners:[],updateResizeListeners:function () {
        var canvas = Module['canvas'];
        Browser.resizeListeners.forEach(function(listener) {
          listener(canvas.width, canvas.height);
        });
      },setCanvasSize:function (width, height, noUpdates) {
        var canvas = Module['canvas'];
        Browser.updateCanvasDimensions(canvas, width, height);
        if (!noUpdates) Browser.updateResizeListeners();
      },windowedWidth:0,windowedHeight:0,setFullScreenCanvasSize:function () {
        // check if SDL is available   
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags | 0x00800000; // set SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },setWindowedCanvasSize:function () {
        // check if SDL is available       
        if (typeof SDL != "undefined") {
        	var flags = HEAPU32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)];
        	flags = flags & ~0x00800000; // clear SDL_FULLSCREEN flag
        	HEAP32[((SDL.screen+Runtime.QUANTUM_SIZE*0)>>2)]=flags
        }
        Browser.updateResizeListeners();
      },updateCanvasDimensions:function (canvas, wNative, hNative) {
        if (wNative && hNative) {
          canvas.widthNative = wNative;
          canvas.heightNative = hNative;
        } else {
          wNative = canvas.widthNative;
          hNative = canvas.heightNative;
        }
        var w = wNative;
        var h = hNative;
        if (Module['forcedAspectRatio'] && Module['forcedAspectRatio'] > 0) {
          if (w/h < Module['forcedAspectRatio']) {
            w = Math.round(h * Module['forcedAspectRatio']);
          } else {
            h = Math.round(w / Module['forcedAspectRatio']);
          }
        }
        if (((document['webkitFullScreenElement'] || document['webkitFullscreenElement'] ||
             document['mozFullScreenElement'] || document['mozFullscreenElement'] ||
             document['fullScreenElement'] || document['fullscreenElement'] ||
             document['msFullScreenElement'] || document['msFullscreenElement'] ||
             document['webkitCurrentFullScreenElement']) === canvas.parentNode) && (typeof screen != 'undefined')) {
           var factor = Math.min(screen.width / w, screen.height / h);
           w = Math.round(w * factor);
           h = Math.round(h * factor);
        }
        if (Browser.resizeCanvas) {
          if (canvas.width  != w) canvas.width  = w;
          if (canvas.height != h) canvas.height = h;
          if (typeof canvas.style != 'undefined') {
            canvas.style.removeProperty( "width");
            canvas.style.removeProperty("height");
          }
        } else {
          if (canvas.width  != wNative) canvas.width  = wNative;
          if (canvas.height != hNative) canvas.height = hNative;
          if (typeof canvas.style != 'undefined') {
            if (w != wNative || h != hNative) {
              canvas.style.setProperty( "width", w + "px", "important");
              canvas.style.setProperty("height", h + "px", "important");
            } else {
              canvas.style.removeProperty( "width");
              canvas.style.removeProperty("height");
            }
          }
        }
      },wgetRequests:{},nextWgetRequestHandle:0,getNextWgetRequestHandle:function () {
        var handle = Browser.nextWgetRequestHandle;
        Browser.nextWgetRequestHandle++;
        return handle;
      }};

  var _sin=Math_sin;

  var _atan=Math_atan;

  function ___syscall15(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // chmod
      var path = SYSCALLS.getStr(), mode = SYSCALLS.get();
      FS.chmod(path, mode);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall10(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // unlink
      var path = SYSCALLS.getStr();
      FS.unlink(path);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall183(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // getcwd
      var buf = SYSCALLS.get(), size = SYSCALLS.get();
      if (size === 0) return -ERRNO_CODES.EINVAL;
      var cwd = FS.cwd();
      if (size < cwd.length + 1) return -ERRNO_CODES.ERANGE;
      writeAsciiToMemory(cwd, buf);
      return buf;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _SItoD=true;

  function ___syscall3(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // read
      var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get(), count = SYSCALLS.get();
      return FS.read(stream, HEAP8,buf, count);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _ceil=Math_ceil;

  function ___syscall5(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // open
      var pathname = SYSCALLS.getStr(), flags = SYSCALLS.get(), mode = SYSCALLS.get() // optional TODO
      var stream = FS.open(pathname, flags, mode);
      return stream.fd;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall4(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // write
      var stream = SYSCALLS.getStreamFromFD(), buf = SYSCALLS.get(), count = SYSCALLS.get();
      return FS.write(stream, HEAP8,buf, count);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall6(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // close
      var stream = SYSCALLS.getStreamFromFD();
      FS.close(stream);
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  var _cos=Math_cos;

  function _pthread_cleanup_pop() {
      assert(_pthread_cleanup_push.level == __ATEXIT__.length, 'cannot pop if something else added meanwhile!');
      __ATEXIT__.pop();
      _pthread_cleanup_push.level = __ATEXIT__.length;
    }

  var _atan2=Math_atan2;

  var _log=Math_log;

  var _exp=Math_exp;

  function _time(ptr) {
      var ret = (Date.now()/1000)|0;
      if (ptr) {
        HEAP32[((ptr)>>2)]=ret;
      }
      return ret;
    }

  function _pthread_self() {
      //FIXME: assumes only a single thread
      return 0;
    }

  function ___syscall140(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // llseek
      var stream = SYSCALLS.getStreamFromFD(), offset_high = SYSCALLS.get(), offset_low = SYSCALLS.get(), result = SYSCALLS.get(), whence = SYSCALLS.get();
      var offset = offset_low;
      assert(offset_high === 0);
      FS.llseek(stream, offset, whence);
      HEAP32[((result)>>2)]=stream.position;
      if (stream.getdents && offset === 0 && whence === 0) stream.getdents = null; // reset readdir state
      return 0;
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function _utimes(path, times) {
      var time;
      if (times) {
        var offset = 8 + 0;
        time = HEAP32[(((times)+(offset))>>2)] * 1000;
        offset = 8 + 4;
        time += HEAP32[(((times)+(offset))>>2)] / 1000;
      } else {
        time = Date.now();
      }
      path = Pointer_stringify(path);
      try {
        FS.utime(path, time, time);
        return 0;
      } catch (e) {
        FS.handleFSError(e);
        return -1;
      }
    }

  function ___syscall146(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // writev
      var stream = SYSCALLS.getStreamFromFD(), iov = SYSCALLS.get(), iovcnt = SYSCALLS.get();
      return SYSCALLS.doWritev(stream, iov, iovcnt);
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }

  function ___syscall221(which, varargs) {SYSCALLS.varargs = varargs;
  try {
   // fcntl64
      var stream = SYSCALLS.getStreamFromFD(), cmd = SYSCALLS.get();
      switch (cmd) {
        case 0: {
          var arg = SYSCALLS.get();
          if (arg < 0) {
            return -ERRNO_CODES.EINVAL;
          }
          var newStream;
          newStream = FS.open(stream.path, stream.flags, 0, arg);
          return newStream.fd;
        }
        case 1:
        case 2:
          return 0;  // FD_CLOEXEC makes no sense for a single process.
        case 3:
          return stream.flags;
        case 4: {
          var arg = SYSCALLS.get();
          stream.flags |= arg;
          return 0;
        }
        case 12:
        case 12: {
          var arg = SYSCALLS.get();
          var offset = 0;
          // We're always unlocked.
          HEAP16[(((arg)+(offset))>>1)]=2;
          return 0;
        }
        case 13:
        case 14:
        case 13:
        case 14:
          return 0; // Pretend that the locking is successful.
        case 16:
        case 8:
          return -ERRNO_CODES.EINVAL; // These are for sockets. We don't have them fully implemented yet.
        case 9:
          // musl trusts getown return values, due to a bug where they must be, as they overlap with errors. just return -1 here, so fnctl() returns that, and we set errno ourselves.
          ___setErrNo(ERRNO_CODES.EINVAL);
          return -1;
        default: {
          return -ERRNO_CODES.EINVAL;
        }
      }
    } catch (e) {
    if (typeof FS === 'undefined' || !(e instanceof FS.ErrnoError)) abort(e);
    return -e.errno;
  }
  }
FS.staticInit();__ATINIT__.unshift(function() { if (!Module["noFSInit"] && !FS.init.initialized) FS.init() });__ATMAIN__.push(function() { FS.ignorePermissions = false });__ATEXIT__.push(function() { FS.quit() });Module["FS_createFolder"] = FS.createFolder;Module["FS_createPath"] = FS.createPath;Module["FS_createDataFile"] = FS.createDataFile;Module["FS_createPreloadedFile"] = FS.createPreloadedFile;Module["FS_createLazyFile"] = FS.createLazyFile;Module["FS_createLink"] = FS.createLink;Module["FS_createDevice"] = FS.createDevice;Module["FS_unlink"] = FS.unlink;
__ATINIT__.unshift(function() { TTY.init() });__ATEXIT__.push(function() { TTY.shutdown() });
if (ENVIRONMENT_IS_NODE) { var fs = require("fs"); var NODEJS_PATH = require("path"); NODEFS.staticInit(); }
___buildEnvironment(ENV);
Module["requestFullScreen"] = function Module_requestFullScreen(lockPointer, resizeCanvas, vrDevice) { Browser.requestFullScreen(lockPointer, resizeCanvas, vrDevice) };
  Module["requestAnimationFrame"] = function Module_requestAnimationFrame(func) { Browser.requestAnimationFrame(func) };
  Module["setCanvasSize"] = function Module_setCanvasSize(width, height, noUpdates) { Browser.setCanvasSize(width, height, noUpdates) };
  Module["pauseMainLoop"] = function Module_pauseMainLoop() { Browser.mainLoop.pause() };
  Module["resumeMainLoop"] = function Module_resumeMainLoop() { Browser.mainLoop.resume() };
  Module["getUserMedia"] = function Module_getUserMedia() { Browser.getUserMedia() }
  Module["createContext"] = function Module_createContext(canvas, useWebGL, setInModule, webGLContextAttributes) { return Browser.createContext(canvas, useWebGL, setInModule, webGLContextAttributes) }
STACK_BASE = STACKTOP = Runtime.alignMemory(STATICTOP);

staticSealed = true; // seal the static portion of memory

STACK_MAX = STACK_BASE + TOTAL_STACK;

DYNAMIC_BASE = DYNAMICTOP = Runtime.alignMemory(STACK_MAX);

assert(DYNAMIC_BASE < TOTAL_MEMORY, "TOTAL_MEMORY not big enough for stack");

 var cttz_i8 = allocate([8,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,7,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,6,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,5,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0,4,0,1,0,2,0,1,0,3,0,1,0,2,0,1,0], "i8", ALLOC_DYNAMIC);


function invoke_iiii(index,a1,a2,a3) {
  try {
    return Module["dynCall_iiii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiii(index,a1,a2,a3) {
    return Runtime.functionPointers[index](a1,a2,a3);
}

function invoke_i(index) {
  try {
    return Module["dynCall_i"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_i(index) {
    return Runtime.functionPointers[index]();
}

function invoke_vi(index,a1) {
  try {
    Module["dynCall_vi"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_vi(index,a1) {
    Runtime.functionPointers[index](a1);
}

function invoke_vii(index,a1,a2) {
  try {
    Module["dynCall_vii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_vii(index,a1,a2) {
    Runtime.functionPointers[index](a1,a2);
}

function invoke_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    return Module["dynCall_iiiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
    return Runtime.functionPointers[index](a1,a2,a3,a4,a5,a6);
}

function invoke_ii(index,a1) {
  try {
    return Module["dynCall_ii"](index,a1);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_ii(index,a1) {
    return Runtime.functionPointers[index](a1);
}

function invoke_viii(index,a1,a2,a3) {
  try {
    Module["dynCall_viii"](index,a1,a2,a3);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_viii(index,a1,a2,a3) {
    Runtime.functionPointers[index](a1,a2,a3);
}

function invoke_v(index) {
  try {
    Module["dynCall_v"](index);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_v(index) {
    Runtime.functionPointers[index]();
}

function invoke_iiiii(index,a1,a2,a3,a4) {
  try {
    return Module["dynCall_iiiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiiii(index,a1,a2,a3,a4) {
    return Runtime.functionPointers[index](a1,a2,a3,a4);
}

function invoke_viiiiii(index,a1,a2,a3,a4,a5,a6) {
  try {
    Module["dynCall_viiiiii"](index,a1,a2,a3,a4,a5,a6);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_viiiiii(index,a1,a2,a3,a4,a5,a6) {
    Runtime.functionPointers[index](a1,a2,a3,a4,a5,a6);
}

function invoke_iii(index,a1,a2) {
  try {
    return Module["dynCall_iii"](index,a1,a2);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iii(index,a1,a2) {
    return Runtime.functionPointers[index](a1,a2);
}

function invoke_iiiiii(index,a1,a2,a3,a4,a5) {
  try {
    return Module["dynCall_iiiiii"](index,a1,a2,a3,a4,a5);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_iiiiii(index,a1,a2,a3,a4,a5) {
    return Runtime.functionPointers[index](a1,a2,a3,a4,a5);
}

function invoke_viiii(index,a1,a2,a3,a4) {
  try {
    Module["dynCall_viiii"](index,a1,a2,a3,a4);
  } catch(e) {
    if (typeof e !== 'number' && e !== 'longjmp') throw e;
    asm["setThrew"](1, 0);
  }
}

function jsCall_viiii(index,a1,a2,a3,a4) {
    Runtime.functionPointers[index](a1,a2,a3,a4);
}

Module.asmGlobalArg = { "Math": Math, "Int8Array": Int8Array, "Int16Array": Int16Array, "Int32Array": Int32Array, "Uint8Array": Uint8Array, "Uint16Array": Uint16Array, "Uint32Array": Uint32Array, "Float32Array": Float32Array, "Float64Array": Float64Array, "NaN": NaN, "Infinity": Infinity };

Module.asmLibraryArg = { "abort": abort, "assert": assert, "invoke_iiii": invoke_iiii, "jsCall_iiii": jsCall_iiii, "invoke_i": invoke_i, "jsCall_i": jsCall_i, "invoke_vi": invoke_vi, "jsCall_vi": jsCall_vi, "invoke_vii": invoke_vii, "jsCall_vii": jsCall_vii, "invoke_iiiiiii": invoke_iiiiiii, "jsCall_iiiiiii": jsCall_iiiiiii, "invoke_ii": invoke_ii, "jsCall_ii": jsCall_ii, "invoke_viii": invoke_viii, "jsCall_viii": jsCall_viii, "invoke_v": invoke_v, "jsCall_v": jsCall_v, "invoke_iiiii": invoke_iiiii, "jsCall_iiiii": jsCall_iiiii, "invoke_viiiiii": invoke_viiiiii, "jsCall_viiiiii": jsCall_viiiiii, "invoke_iii": invoke_iii, "jsCall_iii": jsCall_iii, "invoke_iiiiii": invoke_iiiiii, "jsCall_iiiiii": jsCall_iiiiii, "invoke_viiii": invoke_viiii, "jsCall_viiii": jsCall_viiii, "_pthread_cleanup_pop": _pthread_cleanup_pop, "___syscall85": ___syscall85, "_sin": _sin, "_exp": _exp, "_utimes": _utimes, "_llvm_pow_f64": _llvm_pow_f64, "_acos": _acos, "___syscall54": ___syscall54, "_pthread_cleanup_push": _pthread_cleanup_push, "_abort": _abort, "___syscall40": ___syscall40, "___syscall118": ___syscall118, "___setErrNo": ___setErrNo, "___syscall20": ___syscall20, "___syscall183": ___syscall183, "___assert_fail": ___assert_fail, "_usleep": _usleep, "_asin": _asin, "___buildEnvironment": ___buildEnvironment, "___syscall202": ___syscall202, "___syscall91": ___syscall91, "_cos": _cos, "___syscall39": ___syscall39, "_localtime_r": _localtime_r, "_tzset": _tzset, "_emscripten_set_main_loop_timing": _emscripten_set_main_loop_timing, "_sbrk": _sbrk, "___syscall192": ___syscall192, "___syscall197": ___syscall197, "___syscall196": ___syscall196, "___syscall195": ___syscall195, "___syscall194": ___syscall194, "___syscall212": ___syscall212, "_sysconf": _sysconf, "___syscall94": ___syscall94, "_nanosleep": _nanosleep, "_emscripten_memcpy_big": _emscripten_memcpy_big, "___syscall221": ___syscall221, "_floor": _floor, "_atan": _atan, "___syscall201": ___syscall201, "_pthread_self": _pthread_self, "_getenv": _getenv, "___syscall33": ___syscall33, "_log": _log, "___unlock": ___unlock, "___syscall140": ___syscall140, "___syscall15": ___syscall15, "_emscripten_set_main_loop": _emscripten_set_main_loop, "___syscall10": ___syscall10, "_tan": _tan, "___syscall3": ___syscall3, "___lock": ___lock, "___syscall6": ___syscall6, "___syscall5": ___syscall5, "___syscall4": ___syscall4, "_time": _time, "_atan2": _atan2, "_gettimeofday": _gettimeofday, "_ceil": _ceil, "_sqrt": _sqrt, "_localtime": _localtime, "___syscall207": ___syscall207, "___syscall146": ___syscall146, "STACKTOP": STACKTOP, "STACK_MAX": STACK_MAX, "tempDoublePtr": tempDoublePtr, "ABORT": ABORT, "cttz_i8": cttz_i8 };
// EMSCRIPTEN_START_ASM
var asm = (function(global, env, buffer) {
  'use asm';
  
  
  var HEAP8 = new global.Int8Array(buffer);
  var HEAP16 = new global.Int16Array(buffer);
  var HEAP32 = new global.Int32Array(buffer);
  var HEAPU8 = new global.Uint8Array(buffer);
  var HEAPU16 = new global.Uint16Array(buffer);
  var HEAPU32 = new global.Uint32Array(buffer);
  var HEAPF32 = new global.Float32Array(buffer);
  var HEAPF64 = new global.Float64Array(buffer);


  var STACKTOP=env.STACKTOP|0;
  var STACK_MAX=env.STACK_MAX|0;
  var tempDoublePtr=env.tempDoublePtr|0;
  var ABORT=env.ABORT|0;
  var cttz_i8=env.cttz_i8|0;

  var __THREW__ = 0;
  var threwValue = 0;
  var setjmpId = 0;
  var undef = 0;
  var nan = global.NaN, inf = global.Infinity;
  var tempInt = 0, tempBigInt = 0, tempBigIntP = 0, tempBigIntS = 0, tempBigIntR = 0.0, tempBigIntI = 0, tempBigIntD = 0, tempValue = 0, tempDouble = 0.0;

  var tempRet0 = 0;
  var tempRet1 = 0;
  var tempRet2 = 0;
  var tempRet3 = 0;
  var tempRet4 = 0;
  var tempRet5 = 0;
  var tempRet6 = 0;
  var tempRet7 = 0;
  var tempRet8 = 0;
  var tempRet9 = 0;
  var Math_floor=global.Math.floor;
  var Math_abs=global.Math.abs;
  var Math_sqrt=global.Math.sqrt;
  var Math_pow=global.Math.pow;
  var Math_cos=global.Math.cos;
  var Math_sin=global.Math.sin;
  var Math_tan=global.Math.tan;
  var Math_acos=global.Math.acos;
  var Math_asin=global.Math.asin;
  var Math_atan=global.Math.atan;
  var Math_atan2=global.Math.atan2;
  var Math_exp=global.Math.exp;
  var Math_log=global.Math.log;
  var Math_ceil=global.Math.ceil;
  var Math_imul=global.Math.imul;
  var Math_min=global.Math.min;
  var Math_clz32=global.Math.clz32;
  var abort=env.abort;
  var assert=env.assert;
  var invoke_iiii=env.invoke_iiii;
  var jsCall_iiii=env.jsCall_iiii;
  var invoke_i=env.invoke_i;
  var jsCall_i=env.jsCall_i;
  var invoke_vi=env.invoke_vi;
  var jsCall_vi=env.jsCall_vi;
  var invoke_vii=env.invoke_vii;
  var jsCall_vii=env.jsCall_vii;
  var invoke_iiiiiii=env.invoke_iiiiiii;
  var jsCall_iiiiiii=env.jsCall_iiiiiii;
  var invoke_ii=env.invoke_ii;
  var jsCall_ii=env.jsCall_ii;
  var invoke_viii=env.invoke_viii;
  var jsCall_viii=env.jsCall_viii;
  var invoke_v=env.invoke_v;
  var jsCall_v=env.jsCall_v;
  var invoke_iiiii=env.invoke_iiiii;
  var jsCall_iiiii=env.jsCall_iiiii;
  var invoke_viiiiii=env.invoke_viiiiii;
  var jsCall_viiiiii=env.jsCall_viiiiii;
  var invoke_iii=env.invoke_iii;
  var jsCall_iii=env.jsCall_iii;
  var invoke_iiiiii=env.invoke_iiiiii;
  var jsCall_iiiiii=env.jsCall_iiiiii;
  var invoke_viiii=env.invoke_viiii;
  var jsCall_viiii=env.jsCall_viiii;
  var _pthread_cleanup_pop=env._pthread_cleanup_pop;
  var ___syscall85=env.___syscall85;
  var _sin=env._sin;
  var _exp=env._exp;
  var _utimes=env._utimes;
  var _llvm_pow_f64=env._llvm_pow_f64;
  var _acos=env._acos;
  var ___syscall54=env.___syscall54;
  var _pthread_cleanup_push=env._pthread_cleanup_push;
  var _abort=env._abort;
  var ___syscall40=env.___syscall40;
  var ___syscall118=env.___syscall118;
  var ___setErrNo=env.___setErrNo;
  var ___syscall20=env.___syscall20;
  var ___syscall183=env.___syscall183;
  var ___assert_fail=env.___assert_fail;
  var _usleep=env._usleep;
  var _asin=env._asin;
  var ___buildEnvironment=env.___buildEnvironment;
  var ___syscall202=env.___syscall202;
  var ___syscall91=env.___syscall91;
  var _cos=env._cos;
  var ___syscall39=env.___syscall39;
  var _localtime_r=env._localtime_r;
  var _tzset=env._tzset;
  var _emscripten_set_main_loop_timing=env._emscripten_set_main_loop_timing;
  var _sbrk=env._sbrk;
  var ___syscall192=env.___syscall192;
  var ___syscall197=env.___syscall197;
  var ___syscall196=env.___syscall196;
  var ___syscall195=env.___syscall195;
  var ___syscall194=env.___syscall194;
  var ___syscall212=env.___syscall212;
  var _sysconf=env._sysconf;
  var ___syscall94=env.___syscall94;
  var _nanosleep=env._nanosleep;
  var _emscripten_memcpy_big=env._emscripten_memcpy_big;
  var ___syscall221=env.___syscall221;
  var _floor=env._floor;
  var _atan=env._atan;
  var ___syscall201=env.___syscall201;
  var _pthread_self=env._pthread_self;
  var _getenv=env._getenv;
  var ___syscall33=env.___syscall33;
  var _log=env._log;
  var ___unlock=env.___unlock;
  var ___syscall140=env.___syscall140;
  var ___syscall15=env.___syscall15;
  var _emscripten_set_main_loop=env._emscripten_set_main_loop;
  var ___syscall10=env.___syscall10;
  var _tan=env._tan;
  var ___syscall3=env.___syscall3;
  var ___lock=env.___lock;
  var ___syscall6=env.___syscall6;
  var ___syscall5=env.___syscall5;
  var ___syscall4=env.___syscall4;
  var _time=env._time;
  var _atan2=env._atan2;
  var _gettimeofday=env._gettimeofday;
  var _ceil=env._ceil;
  var _sqrt=env._sqrt;
  var _localtime=env._localtime;
  var ___syscall207=env.___syscall207;
  var ___syscall146=env.___syscall146;
  var tempFloat = 0.0;

// EMSCRIPTEN_START_FUNCS
function stackAlloc(size) {
  size = size|0;
  var ret = 0;
  ret = STACKTOP;
  STACKTOP = (STACKTOP + size)|0;
  STACKTOP = (STACKTOP + 15)&-16;

  return ret|0;
}
function stackSave() {
  return STACKTOP|0;
}
function stackRestore(top) {
  top = top|0;
  STACKTOP = top;
}
function establishStackSpace(stackBase, stackMax) {
  stackBase = stackBase|0;
  stackMax = stackMax|0;
  STACKTOP = stackBase;
  STACK_MAX = stackMax;
}

function setThrew(threw, value) {
  threw = threw|0;
  value = value|0;
  if ((__THREW__|0) == 0) {
    __THREW__ = threw;
    threwValue = value;
  }
}
function copyTempFloat(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
}
function copyTempDouble(ptr) {
  ptr = ptr|0;
  HEAP8[tempDoublePtr>>0] = HEAP8[ptr>>0];
  HEAP8[tempDoublePtr+1>>0] = HEAP8[ptr+1>>0];
  HEAP8[tempDoublePtr+2>>0] = HEAP8[ptr+2>>0];
  HEAP8[tempDoublePtr+3>>0] = HEAP8[ptr+3>>0];
  HEAP8[tempDoublePtr+4>>0] = HEAP8[ptr+4>>0];
  HEAP8[tempDoublePtr+5>>0] = HEAP8[ptr+5>>0];
  HEAP8[tempDoublePtr+6>>0] = HEAP8[ptr+6>>0];
  HEAP8[tempDoublePtr+7>>0] = HEAP8[ptr+7>>0];
}

function setTempRet0(value) {
  value = value|0;
  tempRet0 = value;
}
function getTempRet0() {
  return tempRet0|0;
}

function _RegisterExtensionFunctions($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pArg = 0;
 var $pArg1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $pArg = sp + 4|0;
 $pArg1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = ($1>>>0)<(40);
  if (!($2)) {
   break;
  }
  HEAP32[$pArg>>2] = 0;
  $3 = HEAP32[$i>>2]|0;
  $4 = (360 + (($3*12)|0)|0);
  $5 = ((($4)) + 5|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  switch ($7|0) {
  case 1:  {
   $8 = HEAP32[$0>>2]|0;
   HEAP32[$pArg>>2] = $8;
   break;
  }
  case 2:  {
   HEAP32[$pArg>>2] = (-1);
   break;
  }
  default: {
  }
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  $11 = (360 + (($10*12)|0)|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$i>>2]|0;
  $14 = (360 + (($13*12)|0)|0);
  $15 = ((($14)) + 4|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = HEAP32[$i>>2]|0;
  $19 = (360 + (($18*12)|0)|0);
  $20 = ((($19)) + 6|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = HEAP32[$pArg>>2]|0;
  $24 = HEAP32[$i>>2]|0;
  $25 = (360 + (($24*12)|0)|0);
  $26 = ((($25)) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3_create_function($9,$12,$17,$22,$23,$27,0,0)|0);
  $28 = HEAP32[$i>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$i>>2] = $29;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $30 = HEAP32[$i>>2]|0;
  $31 = ($30>>>0)<(6);
  if (!($31)) {
   break;
  }
  HEAP32[$pArg1>>2] = 0;
  $32 = HEAP32[$i>>2]|0;
  $33 = (840 + ($32<<4)|0);
  $34 = ((($33)) + 5|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  switch ($36|0) {
  case 1:  {
   $37 = HEAP32[$0>>2]|0;
   HEAP32[$pArg1>>2] = $37;
   break;
  }
  case 2:  {
   HEAP32[$pArg1>>2] = (-1);
   break;
  }
  default: {
  }
  }
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$i>>2]|0;
  $40 = (840 + ($39<<4)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$i>>2]|0;
  $43 = (840 + ($42<<4)|0);
  $44 = ((($43)) + 4|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = HEAP32[$pArg1>>2]|0;
  $48 = HEAP32[$i>>2]|0;
  $49 = (840 + ($48<<4)|0);
  $50 = ((($49)) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$i>>2]|0;
  $53 = (840 + ($52<<4)|0);
  $54 = ((($53)) + 12|0);
  $55 = HEAP32[$54>>2]|0;
  (_sqlite3_create_function($38,$41,$46,1,$47,0,$51,$55)|0);
  $56 = HEAP32[$i>>2]|0;
  $57 = (($56) + 1)|0;
  HEAP32[$i>>2] = $57;
 }
 STACKTOP = sp;return 0;
}
function _acosFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),378,(16758|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_acos((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _asinFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),379,(16767|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_asin((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _atanFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),380,(16776|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_atan((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _atn2Func($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $r1 = sp + 8|0;
 $r2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$r1>>3] = 0.0;
 HEAPF64[$r2>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),570,(16793|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = ($7|0)==(5);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_type($11)|0);
  $13 = ($12|0)==(5);
  if (!($13)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (+_sqlite3_value_double($16));
   HEAPF64[$r1>>3] = $17;
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (+_sqlite3_value_double($20));
   HEAPF64[$r2>>3] = $21;
   $22 = HEAP32[$0>>2]|0;
   $23 = +HEAPF64[$r1>>3];
   $24 = +HEAPF64[$r2>>3];
   $25 = (+Math_atan2((+$23),(+$24)));
   _sqlite3_result_double($22,$25);
   STACKTOP = sp;return;
  }
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_null($14);
 STACKTOP = sp;return;
}
function _acoshFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),394,(16802|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_acosh($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _asinhFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),402,(16812|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_asinh($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _atanhFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),410,(16822|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_atanh($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _differenceFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rVal = 0, $zIn1 = 0, $zIn2 = 0, $zR1 = 0, $zR2 = 0, $zResult1 = 0, $zResult2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $zResult1 = sp + 48|0;
 $zResult2 = sp + 40|0;
 $zR1 = sp + 20|0;
 $zR2 = sp + 16|0;
 $rVal = sp + 12|0;
 $i = sp + 8|0;
 $zIn1 = sp + 4|0;
 $zIn2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$zR1>>2] = $zResult1;
 HEAP32[$zR2>>2] = $zResult2;
 HEAP32[$rVal>>2] = 0;
 HEAP32[$i>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),1680,(16832|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = ($7|0)==(5);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_type($11)|0);
  $13 = ($12|0)==(5);
  if (!($13)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3_value_text($16)|0);
   HEAP32[$zIn1>>2] = $17;
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3_value_text($20)|0);
   HEAP32[$zIn2>>2] = $21;
   $22 = HEAP32[$zIn1>>2]|0;
   $23 = HEAP32[$zR1>>2]|0;
   _soundex($22,$23);
   $24 = HEAP32[$zIn2>>2]|0;
   $25 = HEAP32[$zR2>>2]|0;
   _soundex($24,$25);
   HEAP32[$i>>2] = 0;
   while(1) {
    $26 = HEAP32[$i>>2]|0;
    $27 = ($26|0)<(4);
    if (!($27)) {
     break;
    }
    $28 = HEAP32[$zR1>>2]|0;
    $29 = (_sqlite3ReadUtf8($28)|0);
    $30 = HEAP32[$zR2>>2]|0;
    $31 = (_sqlite3ReadUtf8($30)|0);
    $32 = ($29|0)==($31|0);
    if ($32) {
     $33 = HEAP32[$rVal>>2]|0;
     $34 = (($33) + 1)|0;
     HEAP32[$rVal>>2] = $34;
    }
    while(1) {
     $35 = HEAP32[$zR1>>2]|0;
     $36 = ((($35)) + 1|0);
     HEAP32[$zR1>>2] = $36;
     $37 = HEAP8[$36>>0]|0;
     $38 = $37 << 24 >> 24;
     $39 = 192 & $38;
     $40 = ($39|0)==(128);
     if (!($40)) {
      break;
     }
    }
    while(1) {
     $41 = HEAP32[$zR2>>2]|0;
     $42 = ((($41)) + 1|0);
     HEAP32[$zR2>>2] = $42;
     $43 = HEAP8[$42>>0]|0;
     $44 = $43 << 24 >> 24;
     $45 = 192 & $44;
     $46 = ($45|0)==(128);
     if (!($46)) {
      break;
     }
    }
    $47 = HEAP32[$i>>2]|0;
    $48 = (($47) + 1)|0;
    HEAP32[$i>>2] = $48;
   }
   $49 = HEAP32[$0>>2]|0;
   $50 = HEAP32[$rVal>>2]|0;
   _sqlite3_result_int($49,$50);
   STACKTOP = sp;return;
  }
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_null($14);
 STACKTOP = sp;return;
}
function _rad2degFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),498,(17236|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_rad2deg($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _deg2radFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),499,(17248|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_deg2rad($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _cosFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),420,(17260|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_cos((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _sinFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),419,(17268|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_sin((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _tanFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),421,(17276|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_tan((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _cotFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),422,(17284|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_cot($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _coshFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),446,(17292|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_cosh($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _sinhFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),438,(17301|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_sinh($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _tanhFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),454,(17310|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_tanh($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _cothFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),456,(17319|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_coth($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _expFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),474,(17328|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_exp((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _logFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),472,(17336|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_log((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _log10Func($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),473,(17344|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+_log10($13));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _powerFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $r2 = 0, $val = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $r1 = sp + 16|0;
 $r2 = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$r1>>3] = 0.0;
 HEAPF64[$r2>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),546,(17354|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = ($7|0)==(5);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_type($11)|0);
  $13 = ($12|0)==(5);
  if (!($13)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (+_sqlite3_value_double($16));
   HEAPF64[$r1>>3] = $17;
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (+_sqlite3_value_double($20));
   HEAPF64[$r2>>3] = $21;
   $22 = (___errno_location()|0);
   HEAP32[$22>>2] = 0;
   $23 = +HEAPF64[$r1>>3];
   $24 = +HEAPF64[$r2>>3];
   $25 = (+Math_pow((+$23),(+$24)));
   HEAPF64[$val>>3] = $25;
   $26 = (___errno_location()|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)==(0);
   $29 = HEAP32[$0>>2]|0;
   if ($28) {
    $30 = +HEAPF64[$val>>3];
    _sqlite3_result_double($29,$30);
    STACKTOP = sp;return;
   } else {
    $31 = (___errno_location()|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = (_strerror($32)|0);
    $34 = (___errno_location()|0);
    $35 = HEAP32[$34>>2]|0;
    _sqlite3_result_error($29,$33,$35);
    STACKTOP = sp;return;
   }
  }
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_null($14);
 STACKTOP = sp;return;
}
function _signFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0.0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0;
 var $63 = 0, $64 = 0, $65 = 0.0, $7 = 0, $8 = 0, $9 = 0, $iVal = 0, $rVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = $iVal;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(1);
 if (!($8)) {
  ___assert_fail((16726|0),(16734|0),590,(17364|0));
  // unreachable;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 switch ($11|0) {
 case 1:  {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_value_int64($13)|0);
  $15 = tempRet0;
  $16 = $iVal;
  $17 = $16;
  HEAP32[$17>>2] = $14;
  $18 = (($16) + 4)|0;
  $19 = $18;
  HEAP32[$19>>2] = $15;
  $20 = $iVal;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)>(0);
  $27 = ($22>>>0)>(0);
  $28 = ($25|0)==(0);
  $29 = $28 & $27;
  $30 = $26 | $29;
  if ($30) {
   $40 = 1;
  } else {
   $31 = $iVal;
   $32 = $31;
   $33 = HEAP32[$32>>2]|0;
   $34 = (($31) + 4)|0;
   $35 = $34;
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)<(0);
   $38 = $37 ? -1 : 0;
   $40 = $38;
  }
  $39 = ($40|0)<(0);
  $41 = $39 << 31 >> 31;
  $42 = $iVal;
  $43 = $42;
  HEAP32[$43>>2] = $40;
  $44 = (($42) + 4)|0;
  $45 = $44;
  HEAP32[$45>>2] = $41;
  $46 = HEAP32[$0>>2]|0;
  $47 = $iVal;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  _sqlite3_result_int64($46,$49,$52);
  STACKTOP = sp;return;
  break;
 }
 case 5:  {
  $53 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($53);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $54 = HEAP32[$2>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = (+_sqlite3_value_double($55));
  HEAPF64[$rVal>>3] = $56;
  $57 = +HEAPF64[$rVal>>3];
  $58 = $57 > 0.0;
  if ($58) {
   $63 = 1;
  } else {
   $59 = +HEAPF64[$rVal>>3];
   $60 = $59 < 0.0;
   $61 = $60 ? -1 : 0;
   $63 = $61;
  }
  $62 = (+($63|0));
  HEAPF64[$rVal>>3] = $62;
  $64 = HEAP32[$0>>2]|0;
  $65 = +HEAPF64[$rVal>>3];
  _sqlite3_result_double($64,$65);
  STACKTOP = sp;return;
 }
 }
}
function _sqrtFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $rVal = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rVal = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),375,(17373|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $cond = ($7|0)==(5);
 if ($cond) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($8);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (+_sqlite3_value_double($10));
 HEAPF64[$rVal>>3] = $11;
 $12 = (___errno_location()|0);
 HEAP32[$12>>2] = 0;
 $13 = +HEAPF64[$rVal>>3];
 $14 = (+Math_sqrt((+$13)));
 HEAPF64[$val>>3] = $14;
 $15 = (___errno_location()|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if ($17) {
  $19 = +HEAPF64[$val>>3];
  _sqlite3_result_double($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = (___errno_location()|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_strerror($21)|0);
  $23 = (___errno_location()|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_result_error($18,$22,$24);
  STACKTOP = sp;return;
 }
}
function _squareFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iVal = 0, $rVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $iVal = sp + 8|0;
 $rVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = $iVal;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 HEAPF64[$rVal>>3] = 0.0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(1);
 if (!($8)) {
  ___assert_fail((16726|0),(16734|0),514,(17382|0));
  // unreachable;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 switch ($11|0) {
 case 1:  {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_value_int64($13)|0);
  $15 = tempRet0;
  $16 = $iVal;
  $17 = $16;
  HEAP32[$17>>2] = $14;
  $18 = (($16) + 4)|0;
  $19 = $18;
  HEAP32[$19>>2] = $15;
  $20 = HEAP32[$0>>2]|0;
  $21 = $iVal;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = $iVal;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (___muldi3(($23|0),($26|0),($29|0),($32|0))|0);
  $34 = tempRet0;
  _sqlite3_result_int64($20,$33,$34);
  STACKTOP = sp;return;
  break;
 }
 case 5:  {
  $35 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($35);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $36 = HEAP32[$2>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (+_sqlite3_value_double($37));
  HEAPF64[$rVal>>3] = $38;
  $39 = HEAP32[$0>>2]|0;
  $40 = +HEAPF64[$rVal>>3];
  $41 = +HEAPF64[$rVal>>3];
  $42 = $40 * $41;
  _sqlite3_result_double($39,$42);
  STACKTOP = sp;return;
 }
 }
}
function _ceilFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0.0, $33 = 0.0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iVal = 0, $iVal1 = 0, $rVal = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $rVal = sp + 16|0;
 $iVal = sp + 8|0;
 $iVal1 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = $iVal;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(1);
 if (!($8)) {
  ___assert_fail((16726|0),(16734|0),620,(17393|0));
  // unreachable;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 switch ($11|0) {
 case 1:  {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_value_int64($13)|0);
  $15 = tempRet0;
  $16 = $iVal1;
  $17 = $16;
  HEAP32[$17>>2] = $14;
  $18 = (($16) + 4)|0;
  $19 = $18;
  HEAP32[$19>>2] = $15;
  $20 = HEAP32[$0>>2]|0;
  $21 = $iVal1;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  _sqlite3_result_int64($20,$23,$26);
  STACKTOP = sp;return;
  break;
 }
 case 5:  {
  $27 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($27);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = (+_sqlite3_value_double($29));
  HEAPF64[$rVal>>3] = $30;
  $31 = HEAP32[$0>>2]|0;
  $32 = +HEAPF64[$rVal>>3];
  $33 = (+Math_ceil((+$32)));
  $34 = (~~$33)>>>0;
  $35 = +Math_abs($33) >= 1.0 ? $33 > 0.0 ? (~~+Math_min(+Math_floor($33 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($33 - +(~~$33 >>> 0)) / 4294967296.0) >>> 0 : 0;
  _sqlite3_result_int64($31,$34,$35);
  STACKTOP = sp;return;
 }
 }
}
function _floorFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0.0, $33 = 0.0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iVal = 0, $iVal1 = 0, $rVal = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $rVal = sp + 16|0;
 $iVal = sp + 8|0;
 $iVal1 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAPF64[$rVal>>3] = 0.0;
 $3 = $iVal;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(1);
 if (!($8)) {
  ___assert_fail((16726|0),(16734|0),645,(17402|0));
  // unreachable;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 switch ($11|0) {
 case 1:  {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_value_int64($13)|0);
  $15 = tempRet0;
  $16 = $iVal1;
  $17 = $16;
  HEAP32[$17>>2] = $14;
  $18 = (($16) + 4)|0;
  $19 = $18;
  HEAP32[$19>>2] = $15;
  $20 = HEAP32[$0>>2]|0;
  $21 = $iVal1;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  _sqlite3_result_int64($20,$23,$26);
  STACKTOP = sp;return;
  break;
 }
 case 5:  {
  $27 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($27);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = (+_sqlite3_value_double($29));
  HEAPF64[$rVal>>3] = $30;
  $31 = HEAP32[$0>>2]|0;
  $32 = +HEAPF64[$rVal>>3];
  $33 = (+Math_floor((+$32)));
  $34 = (~~$33)>>>0;
  $35 = +Math_abs($33) >= 1.0 ? $33 > 0.0 ? (~~+Math_min(+Math_floor($33 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($33 - +(~~$33 >>> 0)) / 4294967296.0) >>> 0 : 0;
  _sqlite3_result_int64($31,$34,$35);
  STACKTOP = sp;return;
 }
 }
}
function _piFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3_result_double($3,3.1415926535897931);
 STACKTOP = sp;return;
}
function _replicateFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCount = 0, $nLen = 0, $nTLen = 0, $or$cond = 0, $z = 0, $zo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $z = sp + 36|0;
 $zo = sp + 32|0;
 $iCount = sp + 24|0;
 $nLen = sp + 16|0;
 $nTLen = sp + 8|0;
 $i = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = $i;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)!=(2);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 $12 = (5)==($11|0);
 if ($12) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_int64($15)|0);
 $17 = tempRet0;
 $18 = $iCount;
 $19 = $18;
 HEAP32[$19>>2] = $16;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $17;
 $22 = $iCount;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)<(0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($29,17412,-1);
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3_value_bytes($31)|0);
 $33 = ($32|0)<(0);
 $34 = $33 << 31 >> 31;
 $35 = $nLen;
 $36 = $35;
 HEAP32[$36>>2] = $32;
 $37 = (($35) + 4)|0;
 $38 = $37;
 HEAP32[$38>>2] = $34;
 $39 = $nLen;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($39) + 4)|0;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = $iCount;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($45) + 4)|0;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (___muldi3(($41|0),($44|0),($47|0),($50|0))|0);
 $52 = tempRet0;
 $53 = $nTLen;
 $54 = $53;
 HEAP32[$54>>2] = $51;
 $55 = (($53) + 4)|0;
 $56 = $55;
 HEAP32[$56>>2] = $52;
 $57 = $nTLen;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (_i64Add(($59|0),($62|0),1,0)|0);
 $64 = tempRet0;
 $65 = (_sqlite3_malloc($63)|0);
 HEAP32[$z>>2] = $65;
 $66 = $nLen;
 $67 = $66;
 $68 = HEAP32[$67>>2]|0;
 $69 = (($66) + 4)|0;
 $70 = $69;
 $71 = HEAP32[$70>>2]|0;
 $72 = (_i64Add(($68|0),($71|0),1,0)|0);
 $73 = tempRet0;
 $74 = (_sqlite3_malloc($72)|0);
 HEAP32[$zo>>2] = $74;
 $75 = HEAP32[$z>>2]|0;
 $76 = ($75|0)!=(0|0);
 $77 = HEAP32[$zo>>2]|0;
 $78 = ($77|0)!=(0|0);
 $or$cond = $76 & $78;
 if ($or$cond) {
  $86 = HEAP32[$zo>>2]|0;
  $87 = HEAP32[$2>>2]|0;
  $88 = HEAP32[$87>>2]|0;
  $89 = (_sqlite3_value_text($88)|0);
  (_strcpy($86,$89)|0);
  $90 = $i;
  $91 = $90;
  HEAP32[$91>>2] = 0;
  $92 = (($90) + 4)|0;
  $93 = $92;
  HEAP32[$93>>2] = 0;
  while(1) {
   $94 = $i;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($94) + 4)|0;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = $iCount;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($100) + 4)|0;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $106 = ($99|0)<($105|0);
   $107 = ($96>>>0)<($102>>>0);
   $108 = ($99|0)==($105|0);
   $109 = $108 & $107;
   $110 = $106 | $109;
   if (!($110)) {
    break;
   }
   $111 = HEAP32[$z>>2]|0;
   $112 = $i;
   $113 = $112;
   $114 = HEAP32[$113>>2]|0;
   $115 = (($112) + 4)|0;
   $116 = $115;
   $117 = HEAP32[$116>>2]|0;
   $118 = $nLen;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   $124 = (___muldi3(($114|0),($117|0),($120|0),($123|0))|0);
   $125 = tempRet0;
   $126 = (($111) + ($124)|0);
   $127 = HEAP32[$zo>>2]|0;
   (_strcpy($126,$127)|0);
   $128 = $i;
   $129 = $128;
   $130 = HEAP32[$129>>2]|0;
   $131 = (($128) + 4)|0;
   $132 = $131;
   $133 = HEAP32[$132>>2]|0;
   $134 = (_i64Add(($130|0),($133|0),1,0)|0);
   $135 = tempRet0;
   $136 = $i;
   $137 = $136;
   HEAP32[$137>>2] = $134;
   $138 = (($136) + 4)|0;
   $139 = $138;
   HEAP32[$139>>2] = $135;
  }
  $140 = HEAP32[$0>>2]|0;
  $141 = HEAP32[$z>>2]|0;
  _sqlite3_result_text($140,$141,-1,(-1));
  $142 = HEAP32[$z>>2]|0;
  _sqlite3_free($142);
  $143 = HEAP32[$zo>>2]|0;
  _sqlite3_free($143);
  STACKTOP = sp;return;
 }
 $79 = HEAP32[$0>>2]|0;
 _sqlite3_result_error_nomem($79);
 $80 = HEAP32[$z>>2]|0;
 $81 = ($80|0)!=(0|0);
 if ($81) {
  $82 = HEAP32[$z>>2]|0;
  _sqlite3_free($82);
 }
 $83 = HEAP32[$zo>>2]|0;
 $84 = ($83|0)!=(0|0);
 if (!($84)) {
  STACKTOP = sp;return;
 }
 $85 = HEAP32[$zo>>2]|0;
 _sqlite3_free($85);
 STACKTOP = sp;return;
}
function _charindexFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rVal = 0, $s = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z1 = sp + 12|0;
 $z2 = sp + 8|0;
 $s = sp + 4|0;
 $rVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$s>>2] = 0;
 HEAP32[$rVal>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(3);
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(2);
 $or$cond = $4 | $6;
 if (!($or$cond)) {
  ___assert_fail((17425|0),(16734|0),1038,(17443|0));
  // unreachable;
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_type($8)|0);
 $10 = (5)==($9|0);
 if (!($10)) {
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_value_type($13)|0);
  $15 = (5)==($14|0);
  if (!($15)) {
   $17 = HEAP32[$2>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3_value_text($18)|0);
   HEAP32[$z1>>2] = $19;
   $20 = HEAP32[$z1>>2]|0;
   $21 = ($20|0)==(0|0);
   if ($21) {
    STACKTOP = sp;return;
   }
   $22 = HEAP32[$2>>2]|0;
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (_sqlite3_value_text($24)|0);
   HEAP32[$z2>>2] = $25;
   $26 = HEAP32[$1>>2]|0;
   $27 = ($26|0)==(3);
   if ($27) {
    $28 = HEAP32[$2>>2]|0;
    $29 = ((($28)) + 8|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (_sqlite3_value_int($30)|0);
    $32 = (($31) - 1)|0;
    HEAP32[$s>>2] = $32;
    $33 = HEAP32[$s>>2]|0;
    $34 = ($33|0)<(0);
    $$ = $34 ? 0 : $32;
    HEAP32[$s>>2] = $$;
   } else {
    HEAP32[$s>>2] = 0;
   }
   $35 = HEAP32[$z1>>2]|0;
   $36 = HEAP32[$z2>>2]|0;
   $37 = HEAP32[$s>>2]|0;
   $38 = (__substr($35,$36,$37,0)|0);
   HEAP32[$rVal>>2] = $38;
   $39 = HEAP32[$0>>2]|0;
   $40 = HEAP32[$rVal>>2]|0;
   $41 = (($40) + 1)|0;
   _sqlite3_result_int($39,$41);
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 _sqlite3_result_null($16);
 STACKTOP = sp;return;
}
function _leftFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $cc = 0, $l = 0, $rz = 0, $z = 0, $zt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $c = sp + 20|0;
 $cc = sp + 16|0;
 $l = sp + 12|0;
 $z = sp + 8|0;
 $zt = sp + 4|0;
 $rz = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$c>>2] = 0;
 HEAP32[$cc>>2] = 0;
 HEAP32[$l>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),1073,(17457|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = (5)==($7|0);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_type($11)|0);
  $13 = (5)==($12|0);
  if (!($13)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3_value_text($16)|0);
   HEAP32[$z>>2] = $17;
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3_value_int($20)|0);
   HEAP32[$l>>2] = $21;
   $22 = HEAP32[$z>>2]|0;
   HEAP32[$zt>>2] = $22;
   L7: while(1) {
    $23 = HEAP32[$zt>>2]|0;
    $24 = (_sqlite3ReadUtf8($23)|0);
    $25 = ($24|0)!=(0);
    if (!($25)) {
     break;
    }
    $26 = HEAP32[$c>>2]|0;
    $27 = (($26) + 1)|0;
    HEAP32[$c>>2] = $27;
    $28 = HEAP32[$l>>2]|0;
    $29 = ($26|0)<($28|0);
    if (!($29)) {
     break;
    }
    while(1) {
     $30 = HEAP32[$zt>>2]|0;
     $31 = ((($30)) + 1|0);
     HEAP32[$zt>>2] = $31;
     $32 = HEAP8[$31>>0]|0;
     $33 = $32&255;
     $34 = 192 & $33;
     $35 = ($34|0)==(128);
     if (!($35)) {
      continue L7;
     }
    }
   }
   $36 = HEAP32[$zt>>2]|0;
   $37 = HEAP32[$z>>2]|0;
   $38 = $36;
   $39 = $37;
   $40 = (($38) - ($39))|0;
   HEAP32[$cc>>2] = $40;
   $41 = HEAP32[$zt>>2]|0;
   $42 = HEAP32[$z>>2]|0;
   $43 = $41;
   $44 = $42;
   $45 = (($43) - ($44))|0;
   $46 = (($45) + 1)|0;
   $47 = (_sqlite3_malloc($46)|0);
   HEAP32[$rz>>2] = $47;
   $48 = HEAP32[$rz>>2]|0;
   $49 = ($48|0)!=(0|0);
   if ($49) {
    $51 = HEAP32[$rz>>2]|0;
    $52 = HEAP32[$z>>2]|0;
    $53 = HEAP32[$zt>>2]|0;
    $54 = HEAP32[$z>>2]|0;
    $55 = $53;
    $56 = $54;
    $57 = (($55) - ($56))|0;
    (_strncpy($51,$52,$57)|0);
    $58 = HEAP32[$rz>>2]|0;
    $59 = HEAP32[$cc>>2]|0;
    $60 = (($58) + ($59)|0);
    HEAP8[$60>>0] = 0;
    $61 = HEAP32[$0>>2]|0;
    $62 = HEAP32[$rz>>2]|0;
    _sqlite3_result_text($61,$62,-1,(-1));
    $63 = HEAP32[$rz>>2]|0;
    _sqlite3_free($63);
    STACKTOP = sp;return;
   } else {
    $50 = HEAP32[$0>>2]|0;
    _sqlite3_result_error_nomem($50);
    STACKTOP = sp;return;
   }
  }
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_null($14);
 STACKTOP = sp;return;
}
function _rightFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $cc = 0, $l = 0, $rz = 0, $z = 0, $ze = 0, $zt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $l = sp + 24|0;
 $c = sp + 20|0;
 $cc = sp + 16|0;
 $z = sp + 12|0;
 $zt = sp + 8|0;
 $ze = sp + 4|0;
 $rz = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$l>>2] = 0;
 HEAP32[$c>>2] = 0;
 HEAP32[$cc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),1113,(17466|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = (5)==($7|0);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_type($11)|0);
  $13 = (5)==($12|0);
  if (!($13)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3_value_text($16)|0);
   HEAP32[$z>>2] = $17;
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3_value_int($20)|0);
   HEAP32[$l>>2] = $21;
   $22 = HEAP32[$z>>2]|0;
   HEAP32[$zt>>2] = $22;
   while(1) {
    $23 = HEAP32[$zt>>2]|0;
    $24 = (_sqlite3ReadUtf8($23)|0);
    $25 = ($24|0)!=(0);
    if (!($25)) {
     break;
    }
    while(1) {
     $26 = HEAP32[$zt>>2]|0;
     $27 = ((($26)) + 1|0);
     HEAP32[$zt>>2] = $27;
     $28 = HEAP8[$27>>0]|0;
     $29 = $28 << 24 >> 24;
     $30 = 192 & $29;
     $31 = ($30|0)==(128);
     if (!($31)) {
      break;
     }
    }
    $32 = HEAP32[$c>>2]|0;
    $33 = (($32) + 1)|0;
    HEAP32[$c>>2] = $33;
   }
   $34 = HEAP32[$zt>>2]|0;
   HEAP32[$ze>>2] = $34;
   $35 = HEAP32[$z>>2]|0;
   HEAP32[$zt>>2] = $35;
   $36 = HEAP32[$c>>2]|0;
   $37 = HEAP32[$l>>2]|0;
   $38 = (($36) - ($37))|0;
   HEAP32[$cc>>2] = $38;
   $39 = HEAP32[$cc>>2]|0;
   $40 = ($39|0)<(0);
   $$ = $40 ? 0 : $38;
   HEAP32[$cc>>2] = $$;
   L13: while(1) {
    $41 = HEAP32[$cc>>2]|0;
    $42 = (($41) + -1)|0;
    HEAP32[$cc>>2] = $42;
    $43 = ($41|0)>(0);
    if (!($43)) {
     break;
    }
    while(1) {
     $44 = HEAP32[$zt>>2]|0;
     $45 = ((($44)) + 1|0);
     HEAP32[$zt>>2] = $45;
     $46 = HEAP8[$45>>0]|0;
     $47 = $46 << 24 >> 24;
     $48 = 192 & $47;
     $49 = ($48|0)==(128);
     if (!($49)) {
      continue L13;
     }
    }
   }
   $50 = HEAP32[$ze>>2]|0;
   $51 = HEAP32[$zt>>2]|0;
   $52 = $50;
   $53 = $51;
   $54 = (($52) - ($53))|0;
   $55 = (($54) + 1)|0;
   $56 = (_sqlite3_malloc($55)|0);
   HEAP32[$rz>>2] = $56;
   $57 = HEAP32[$rz>>2]|0;
   $58 = ($57|0)!=(0|0);
   if ($58) {
    $60 = HEAP32[$rz>>2]|0;
    $61 = HEAP32[$zt>>2]|0;
    (_strcpy($60,$61)|0);
    $62 = HEAP32[$0>>2]|0;
    $63 = HEAP32[$rz>>2]|0;
    _sqlite3_result_text($62,$63,-1,(-1));
    $64 = HEAP32[$rz>>2]|0;
    _sqlite3_free($64);
    STACKTOP = sp;return;
   } else {
    $59 = HEAP32[$0>>2]|0;
    _sqlite3_result_error_nomem($59);
    STACKTOP = sp;return;
   }
  }
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_null($14);
 STACKTOP = sp;return;
}
function _reverseFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $l = 0, $rz = 0, $rzt = 0, $z = 0;
 var $zt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $z = sp + 20|0;
 $zt = sp + 16|0;
 $rz = sp + 12|0;
 $rzt = sp + 8|0;
 $l = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$l>>2] = 0;
 HEAP32[$i>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (1)==($3|0);
 if (!($4)) {
  ___assert_fail((17476|0),(16734|0),1323,(17484|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = (5)==($7|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_text($11)|0);
 HEAP32[$z>>2] = $12;
 $13 = HEAP32[$z>>2]|0;
 $14 = (_strlen($13)|0);
 HEAP32[$l>>2] = $14;
 $15 = HEAP32[$l>>2]|0;
 $16 = (($15) + 1)|0;
 $17 = (_sqlite3_malloc($16)|0);
 HEAP32[$rz>>2] = $17;
 $18 = HEAP32[$rz>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $20 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($20);
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$rz>>2]|0;
 $22 = HEAP32[$l>>2]|0;
 $23 = (($21) + ($22)|0);
 HEAP32[$rzt>>2] = $23;
 $24 = HEAP32[$rzt>>2]|0;
 $25 = ((($24)) + -1|0);
 HEAP32[$rzt>>2] = $25;
 HEAP8[$24>>0] = 0;
 $26 = HEAP32[$z>>2]|0;
 HEAP32[$zt>>2] = $26;
 L12: while(1) {
  $27 = HEAP32[$zt>>2]|0;
  $28 = (_sqlite3ReadUtf8($27)|0);
  $29 = ($28|0)!=(0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$zt>>2]|0;
  HEAP32[$z>>2] = $30;
  while(1) {
   $31 = HEAP32[$zt>>2]|0;
   $32 = ((($31)) + 1|0);
   HEAP32[$zt>>2] = $32;
   $33 = HEAP8[$32>>0]|0;
   $34 = $33 << 24 >> 24;
   $35 = 192 & $34;
   $36 = ($35|0)==(128);
   if (!($36)) {
    break;
   }
  }
  HEAP32[$i>>2] = 1;
  while(1) {
   $37 = HEAP32[$zt>>2]|0;
   $38 = HEAP32[$i>>2]|0;
   $39 = (0 - ($38))|0;
   $40 = (($37) + ($39)|0);
   $41 = HEAP32[$z>>2]|0;
   $42 = ($40>>>0)>=($41>>>0);
   if (!($42)) {
    continue L12;
   }
   $43 = HEAP32[$zt>>2]|0;
   $44 = HEAP32[$i>>2]|0;
   $45 = (0 - ($44))|0;
   $46 = (($43) + ($45)|0);
   $47 = HEAP8[$46>>0]|0;
   $48 = HEAP32[$rzt>>2]|0;
   $49 = ((($48)) + -1|0);
   HEAP32[$rzt>>2] = $49;
   HEAP8[$48>>0] = $47;
   $50 = HEAP32[$i>>2]|0;
   $51 = (($50) + 1)|0;
   HEAP32[$i>>2] = $51;
  }
 }
 $52 = HEAP32[$0>>2]|0;
 $53 = HEAP32[$rz>>2]|0;
 _sqlite3_result_text($52,$53,-1,(-1));
 $54 = HEAP32[$rz>>2]|0;
 _sqlite3_free($54);
 STACKTOP = sp;return;
}
function _properFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $c = 0, $r = 0, $z = 0, $zo = 0, $zt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z = sp + 12|0;
 $zo = sp + 8|0;
 $zt = sp + 4|0;
 $r = sp + 28|0;
 $c = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$c>>2] = 1;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),724,(17496|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = (5)==($7|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_text($11)|0);
 HEAP32[$z>>2] = $12;
 $13 = HEAP32[$z>>2]|0;
 $14 = (_sqlite3StrDup($13)|0);
 HEAP32[$zo>>2] = $14;
 $15 = HEAP32[$zo>>2]|0;
 $16 = ($15|0)!=(0|0);
 if (!($16)) {
  $17 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($17);
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$zo>>2]|0;
 HEAP32[$zt>>2] = $18;
 while(1) {
  $19 = HEAP32[$z>>2]|0;
  $20 = ((($19)) + 1|0);
  HEAP32[$z>>2] = $20;
  $21 = HEAP8[$19>>0]|0;
  HEAP8[$r>>0] = $21;
  $22 = $21 << 24 >> 24;
  $23 = ($22|0)!=(0);
  if (!($23)) {
   break;
  }
  $24 = HEAP8[$r>>0]|0;
  $25 = $24 << 24 >> 24;
  $26 = (_isblank($25)|0);
  $27 = ($26|0)!=(0);
  if ($27) {
   HEAP32[$c>>2] = 1;
  } else {
   $28 = HEAP32[$c>>2]|0;
   $29 = ($28|0)==(1);
   $30 = HEAP8[$r>>0]|0;
   $31 = $30 << 24 >> 24;
   if ($29) {
    $32 = (_toupper($31)|0);
    $33 = $32&255;
    HEAP8[$r>>0] = $33;
   } else {
    $34 = (_tolower($31)|0);
    $35 = $34&255;
    HEAP8[$r>>0] = $35;
   }
   HEAP32[$c>>2] = 0;
  }
  $36 = HEAP8[$r>>0]|0;
  $37 = HEAP32[$zt>>2]|0;
  $38 = ((($37)) + 1|0);
  HEAP32[$zt>>2] = $38;
  HEAP8[$37>>0] = $36;
 }
 $39 = HEAP32[$zt>>2]|0;
 HEAP8[$39>>0] = 0;
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$zo>>2]|0;
 _sqlite3_result_text($40,$41,-1,(-1));
 $42 = HEAP32[$zo>>2]|0;
 _sqlite3_free($42);
 STACKTOP = sp;return;
}
function _padlFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $12 = 0;
 var $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $ilen = 0, $zi = 0, $zl = 0, $zo = 0;
 var $zt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $ilen = sp + 8|0;
 $zl = sp;
 $i = sp + 28|0;
 $zi = sp + 24|0;
 $zo = sp + 20|0;
 $zt = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$i>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),771,(17507|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = ($7|0)==(5);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_text($11)|0);
 HEAP32[$zi>>2] = $12;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_int64($15)|0);
 $17 = tempRet0;
 $18 = $ilen;
 $19 = $18;
 HEAP32[$19>>2] = $16;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $17;
 $22 = $ilen;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)<(0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($29,17412,-1);
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$zi>>2]|0;
 $31 = (_sqlite3Utf8CharLen($30,-1)|0);
 $32 = ($31|0)<(0);
 $33 = $32 << 31 >> 31;
 $34 = $zl;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = (($34) + 4)|0;
 $37 = $36;
 HEAP32[$37>>2] = $33;
 $38 = $zl;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = $ilen;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + 4)|0;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = ($43|0)>($49|0);
 $51 = ($40>>>0)>=($46>>>0);
 $52 = ($43|0)==($49|0);
 $53 = $52 & $51;
 $54 = $50 | $53;
 $55 = HEAP32[$zi>>2]|0;
 do {
  if ($54) {
   $56 = (_sqlite3StrDup($55)|0);
   HEAP32[$zo>>2] = $56;
   $57 = HEAP32[$zo>>2]|0;
   $58 = ($57|0)!=(0|0);
   $59 = HEAP32[$0>>2]|0;
   if ($58) {
    $60 = HEAP32[$zo>>2]|0;
    _sqlite3_result_text($59,$60,-1,(-1));
    break;
   }
   _sqlite3_result_error_nomem($59);
   STACKTOP = sp;return;
  } else {
   $61 = (_strlen($55)|0);
   $62 = $ilen;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Add(($61|0),0,($64|0),($67|0))|0);
   $69 = tempRet0;
   $70 = $zl;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($70) + 4)|0;
   $74 = $73;
   $75 = HEAP32[$74>>2]|0;
   $76 = (_i64Subtract(($68|0),($69|0),($72|0),($75|0))|0);
   $77 = tempRet0;
   $78 = (_i64Add(($76|0),($77|0),1,0)|0);
   $79 = tempRet0;
   $80 = (_sqlite3_malloc($78)|0);
   HEAP32[$zo>>2] = $80;
   $81 = HEAP32[$zo>>2]|0;
   $82 = ($81|0)!=(0|0);
   if (!($82)) {
    $83 = HEAP32[$0>>2]|0;
    _sqlite3_result_error_nomem($83);
    STACKTOP = sp;return;
   }
   $84 = HEAP32[$zo>>2]|0;
   HEAP32[$zt>>2] = $84;
   HEAP32[$i>>2] = 1;
   while(1) {
    $85 = HEAP32[$i>>2]|0;
    $86 = ($85|0)<(0);
    $87 = $86 << 31 >> 31;
    $88 = $zl;
    $89 = $88;
    $90 = HEAP32[$89>>2]|0;
    $91 = (($88) + 4)|0;
    $92 = $91;
    $93 = HEAP32[$92>>2]|0;
    $94 = (_i64Add(($85|0),($87|0),($90|0),($93|0))|0);
    $95 = tempRet0;
    $96 = $ilen;
    $97 = $96;
    $98 = HEAP32[$97>>2]|0;
    $99 = (($96) + 4)|0;
    $100 = $99;
    $101 = HEAP32[$100>>2]|0;
    $102 = ($95|0)<($101|0);
    $103 = ($94>>>0)<=($98>>>0);
    $104 = ($95|0)==($101|0);
    $105 = $104 & $103;
    $106 = $102 | $105;
    $107 = HEAP32[$zt>>2]|0;
    if (!($106)) {
     break;
    }
    $108 = ((($107)) + 1|0);
    HEAP32[$zt>>2] = $108;
    HEAP8[$107>>0] = 32;
    $109 = HEAP32[$i>>2]|0;
    $110 = (($109) + 1)|0;
    HEAP32[$i>>2] = $110;
   }
   $111 = HEAP32[$zi>>2]|0;
   (_strcpy($107,$111)|0);
  }
 } while(0);
 $112 = HEAP32[$0>>2]|0;
 $113 = HEAP32[$zo>>2]|0;
 _sqlite3_result_text($112,$113,-1,(-1));
 $114 = HEAP32[$zo>>2]|0;
 _sqlite3_free($114);
 STACKTOP = sp;return;
}
function _padrFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $ilen = 0, $zi = 0, $zl = 0, $zll = 0, $zo = 0, $zt = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $ilen = sp + 16|0;
 $zl = sp + 8|0;
 $zll = sp;
 $i = sp + 36|0;
 $zi = sp + 32|0;
 $zo = sp + 28|0;
 $zt = sp + 24|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$i>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),825,(17544|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = ($7|0)==(5);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_text($11)|0);
 HEAP32[$zi>>2] = $12;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_int64($15)|0);
 $17 = tempRet0;
 $18 = $ilen;
 $19 = $18;
 HEAP32[$19>>2] = $16;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $17;
 $22 = $ilen;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)<(0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($29,17412,-1);
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$zi>>2]|0;
 $31 = (_sqlite3Utf8CharLen($30,-1)|0);
 $32 = ($31|0)<(0);
 $33 = $32 << 31 >> 31;
 $34 = $zl;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = (($34) + 4)|0;
 $37 = $36;
 HEAP32[$37>>2] = $33;
 $38 = $zl;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = $ilen;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + 4)|0;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = ($43|0)>($49|0);
 $51 = ($40>>>0)>=($46>>>0);
 $52 = ($43|0)==($49|0);
 $53 = $52 & $51;
 $54 = $50 | $53;
 $55 = HEAP32[$zi>>2]|0;
 do {
  if ($54) {
   $56 = (_sqlite3StrDup($55)|0);
   HEAP32[$zo>>2] = $56;
   $57 = HEAP32[$zo>>2]|0;
   $58 = ($57|0)!=(0|0);
   $59 = HEAP32[$0>>2]|0;
   if ($58) {
    $60 = HEAP32[$zo>>2]|0;
    _sqlite3_result_text($59,$60,-1,(-1));
    break;
   }
   _sqlite3_result_error_nomem($59);
   STACKTOP = sp;return;
  } else {
   $61 = (_strlen($55)|0);
   $62 = $zll;
   $63 = $62;
   HEAP32[$63>>2] = $61;
   $64 = (($62) + 4)|0;
   $65 = $64;
   HEAP32[$65>>2] = 0;
   $66 = $zll;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = (($66) + 4)|0;
   $70 = $69;
   $71 = HEAP32[$70>>2]|0;
   $72 = $ilen;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (_i64Add(($68|0),($71|0),($74|0),($77|0))|0);
   $79 = tempRet0;
   $80 = $zl;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $83 = (($80) + 4)|0;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = (_i64Subtract(($78|0),($79|0),($82|0),($85|0))|0);
   $87 = tempRet0;
   $88 = (_i64Add(($86|0),($87|0),1,0)|0);
   $89 = tempRet0;
   $90 = (_sqlite3_malloc($88)|0);
   HEAP32[$zo>>2] = $90;
   $91 = HEAP32[$zo>>2]|0;
   $92 = ($91|0)!=(0|0);
   if (!($92)) {
    $93 = HEAP32[$0>>2]|0;
    _sqlite3_result_error_nomem($93);
    STACKTOP = sp;return;
   }
   $94 = HEAP32[$zo>>2]|0;
   $95 = HEAP32[$zi>>2]|0;
   $96 = (_strcpy($94,$95)|0);
   $97 = $zll;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = (($97) + 4)|0;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($96) + ($99)|0);
   HEAP32[$zt>>2] = $103;
   HEAP32[$i>>2] = 1;
   while(1) {
    $104 = HEAP32[$i>>2]|0;
    $105 = ($104|0)<(0);
    $106 = $105 << 31 >> 31;
    $107 = $zl;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = (_i64Add(($104|0),($106|0),($109|0),($112|0))|0);
    $114 = tempRet0;
    $115 = $ilen;
    $116 = $115;
    $117 = HEAP32[$116>>2]|0;
    $118 = (($115) + 4)|0;
    $119 = $118;
    $120 = HEAP32[$119>>2]|0;
    $121 = ($114|0)<($120|0);
    $122 = ($113>>>0)<=($117>>>0);
    $123 = ($114|0)==($120|0);
    $124 = $123 & $122;
    $125 = $121 | $124;
    $126 = HEAP32[$zt>>2]|0;
    if (!($125)) {
     break;
    }
    $127 = ((($126)) + 1|0);
    HEAP32[$zt>>2] = $127;
    HEAP8[$126>>0] = 32;
    $128 = HEAP32[$i>>2]|0;
    $129 = (($128) + 1)|0;
    HEAP32[$i>>2] = $129;
   }
   HEAP8[$126>>0] = 0;
  }
 } while(0);
 $130 = HEAP32[$0>>2]|0;
 $131 = HEAP32[$zo>>2]|0;
 _sqlite3_result_text($130,$131,-1,(-1));
 $132 = HEAP32[$zo>>2]|0;
 _sqlite3_free($132);
 STACKTOP = sp;return;
}
function _padcFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $ilen = 0, $zi = 0, $zl = 0, $zll = 0, $zo = 0, $zt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $ilen = sp + 16|0;
 $zl = sp + 8|0;
 $zll = sp;
 $i = sp + 36|0;
 $zi = sp + 32|0;
 $zo = sp + 28|0;
 $zt = sp + 24|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$i>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),880,(17553|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = ($7|0)==(5);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_text($11)|0);
 HEAP32[$zi>>2] = $12;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_int64($15)|0);
 $17 = tempRet0;
 $18 = $ilen;
 $19 = $18;
 HEAP32[$19>>2] = $16;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $17;
 $22 = $ilen;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)<(0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($29,17412,-1);
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$zi>>2]|0;
 $31 = (_sqlite3Utf8CharLen($30,-1)|0);
 $32 = ($31|0)<(0);
 $33 = $32 << 31 >> 31;
 $34 = $zl;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = (($34) + 4)|0;
 $37 = $36;
 HEAP32[$37>>2] = $33;
 $38 = $zl;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = $ilen;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + 4)|0;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = ($43|0)>($49|0);
 $51 = ($40>>>0)>=($46>>>0);
 $52 = ($43|0)==($49|0);
 $53 = $52 & $51;
 $54 = $50 | $53;
 $55 = HEAP32[$zi>>2]|0;
 do {
  if ($54) {
   $56 = (_sqlite3StrDup($55)|0);
   HEAP32[$zo>>2] = $56;
   $57 = HEAP32[$zo>>2]|0;
   $58 = ($57|0)!=(0|0);
   $59 = HEAP32[$0>>2]|0;
   if ($58) {
    $60 = HEAP32[$zo>>2]|0;
    _sqlite3_result_text($59,$60,-1,(-1));
    break;
   }
   _sqlite3_result_error_nomem($59);
   STACKTOP = sp;return;
  } else {
   $61 = (_strlen($55)|0);
   $62 = $zll;
   $63 = $62;
   HEAP32[$63>>2] = $61;
   $64 = (($62) + 4)|0;
   $65 = $64;
   HEAP32[$65>>2] = 0;
   $66 = $zll;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = (($66) + 4)|0;
   $70 = $69;
   $71 = HEAP32[$70>>2]|0;
   $72 = $ilen;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (_i64Add(($68|0),($71|0),($74|0),($77|0))|0);
   $79 = tempRet0;
   $80 = $zl;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $83 = (($80) + 4)|0;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = (_i64Subtract(($78|0),($79|0),($82|0),($85|0))|0);
   $87 = tempRet0;
   $88 = (_i64Add(($86|0),($87|0),1,0)|0);
   $89 = tempRet0;
   $90 = (_sqlite3_malloc($88)|0);
   HEAP32[$zo>>2] = $90;
   $91 = HEAP32[$zo>>2]|0;
   $92 = ($91|0)!=(0|0);
   if (!($92)) {
    $93 = HEAP32[$0>>2]|0;
    _sqlite3_result_error_nomem($93);
    STACKTOP = sp;return;
   }
   $94 = HEAP32[$zo>>2]|0;
   HEAP32[$zt>>2] = $94;
   HEAP32[$i>>2] = 1;
   while(1) {
    $95 = HEAP32[$i>>2]|0;
    $96 = $95<<1;
    $97 = ($96|0)<(0);
    $98 = $97 << 31 >> 31;
    $99 = $zl;
    $100 = $99;
    $101 = HEAP32[$100>>2]|0;
    $102 = (($99) + 4)|0;
    $103 = $102;
    $104 = HEAP32[$103>>2]|0;
    $105 = (_i64Add(($96|0),($98|0),($101|0),($104|0))|0);
    $106 = tempRet0;
    $107 = $ilen;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = ($106|0)<($112|0);
    $114 = ($105>>>0)<=($109>>>0);
    $115 = ($106|0)==($112|0);
    $116 = $115 & $114;
    $117 = $113 | $116;
    $118 = HEAP32[$zt>>2]|0;
    if (!($117)) {
     break;
    }
    $119 = ((($118)) + 1|0);
    HEAP32[$zt>>2] = $119;
    HEAP8[$118>>0] = 32;
    $120 = HEAP32[$i>>2]|0;
    $121 = (($120) + 1)|0;
    HEAP32[$i>>2] = $121;
   }
   $122 = HEAP32[$zi>>2]|0;
   (_strcpy($118,$122)|0);
   $123 = $zll;
   $124 = $123;
   $125 = HEAP32[$124>>2]|0;
   $126 = (($123) + 4)|0;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = HEAP32[$zt>>2]|0;
   $130 = (($129) + ($125)|0);
   HEAP32[$zt>>2] = $130;
   while(1) {
    $131 = HEAP32[$i>>2]|0;
    $132 = ($131|0)<(0);
    $133 = $132 << 31 >> 31;
    $134 = $zl;
    $135 = $134;
    $136 = HEAP32[$135>>2]|0;
    $137 = (($134) + 4)|0;
    $138 = $137;
    $139 = HEAP32[$138>>2]|0;
    $140 = (_i64Add(($131|0),($133|0),($136|0),($139|0))|0);
    $141 = tempRet0;
    $142 = $ilen;
    $143 = $142;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($142) + 4)|0;
    $146 = $145;
    $147 = HEAP32[$146>>2]|0;
    $148 = ($141|0)<($147|0);
    $149 = ($140>>>0)<=($144>>>0);
    $150 = ($141|0)==($147|0);
    $151 = $150 & $149;
    $152 = $148 | $151;
    $153 = HEAP32[$zt>>2]|0;
    if (!($152)) {
     break;
    }
    $154 = ((($153)) + 1|0);
    HEAP32[$zt>>2] = $154;
    HEAP8[$153>>0] = 32;
    $155 = HEAP32[$i>>2]|0;
    $156 = (($155) + 1)|0;
    HEAP32[$i>>2] = $156;
   }
   HEAP8[$153>>0] = 0;
  }
 } while(0);
 $157 = HEAP32[$0>>2]|0;
 $158 = HEAP32[$zo>>2]|0;
 _sqlite3_result_text($157,$158,-1,(-1));
 $159 = HEAP32[$zo>>2]|0;
 _sqlite3_free($159);
 STACKTOP = sp;return;
}
function _strfilterFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $c1 = 0, $c2 = 0, $z1 = 0, $z21 = 0, $z22 = 0, $zi1 = 0, $zi2 = 0, $zo = 0, $zot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $zi1 = sp + 32|0;
 $zi2 = sp + 28|0;
 $z1 = sp + 24|0;
 $z21 = sp + 20|0;
 $z22 = sp + 16|0;
 $zo = sp + 12|0;
 $zot = sp + 8|0;
 $c1 = sp + 4|0;
 $c2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$c1>>2] = 0;
 HEAP32[$c2>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 if (!($4)) {
  ___assert_fail((16785|0),(16734|0),939,(17562|0));
  // unreachable;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_type($6)|0);
 $8 = ($7|0)==(5);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_type($11)|0);
  $13 = ($12|0)==(5);
  if (!($13)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3_value_text($16)|0);
   HEAP32[$zi1>>2] = $17;
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3_value_text($20)|0);
   HEAP32[$zi2>>2] = $21;
   $22 = HEAP32[$zi1>>2]|0;
   $23 = (_strlen($22)|0);
   $24 = (($23) + 1)|0;
   $25 = (_sqlite3_malloc($24)|0);
   HEAP32[$zo>>2] = $25;
   $26 = HEAP32[$zo>>2]|0;
   $27 = ($26|0)!=(0|0);
   if (!($27)) {
    $28 = HEAP32[$0>>2]|0;
    _sqlite3_result_error_nomem($28);
    STACKTOP = sp;return;
   }
   $29 = HEAP32[$zo>>2]|0;
   HEAP32[$zot>>2] = $29;
   $30 = HEAP32[$zi1>>2]|0;
   HEAP32[$z1>>2] = $30;
   L11: while(1) {
    $31 = HEAP32[$z1>>2]|0;
    $32 = (_sqlite3ReadUtf8($31)|0);
    HEAP32[$c1>>2] = $32;
    $33 = ($32|0)!=(0);
    if (!($33)) {
     break;
    }
    $34 = HEAP32[$zi2>>2]|0;
    HEAP32[$z21>>2] = $34;
    L14: while(1) {
     $35 = HEAP32[$z21>>2]|0;
     $36 = (_sqlite3ReadUtf8($35)|0);
     HEAP32[$c2>>2] = $36;
     $37 = ($36|0)!=(0);
     if (!($37)) {
      break;
     }
     $38 = HEAP32[$c2>>2]|0;
     $39 = HEAP32[$c1>>2]|0;
     $40 = ($38|0)!=($39|0);
     if (!($40)) {
      break;
     }
     while(1) {
      $41 = HEAP32[$z21>>2]|0;
      $42 = ((($41)) + 1|0);
      HEAP32[$z21>>2] = $42;
      $43 = HEAP8[$42>>0]|0;
      $44 = $43 << 24 >> 24;
      $45 = 192 & $44;
      $46 = ($45|0)==(128);
      if (!($46)) {
       continue L14;
      }
     }
    }
    $47 = HEAP32[$c2>>2]|0;
    $48 = ($47|0)!=(0);
    if ($48) {
     $49 = HEAP32[$z21>>2]|0;
     HEAP32[$z22>>2] = $49;
     while(1) {
      $50 = HEAP32[$z22>>2]|0;
      $51 = ((($50)) + 1|0);
      HEAP32[$z22>>2] = $51;
      $52 = HEAP8[$51>>0]|0;
      $53 = $52 << 24 >> 24;
      $54 = 192 & $53;
      $55 = ($54|0)==(128);
      if (!($55)) {
       break;
      }
     }
     $56 = HEAP32[$zot>>2]|0;
     $57 = HEAP32[$z21>>2]|0;
     $58 = HEAP32[$z22>>2]|0;
     $59 = HEAP32[$z21>>2]|0;
     $60 = $58;
     $61 = $59;
     $62 = (($60) - ($61))|0;
     (_strncpy($56,$57,$62)|0);
     $63 = HEAP32[$z22>>2]|0;
     $64 = HEAP32[$z21>>2]|0;
     $65 = $63;
     $66 = $64;
     $67 = (($65) - ($66))|0;
     $68 = HEAP32[$zot>>2]|0;
     $69 = (($68) + ($67)|0);
     HEAP32[$zot>>2] = $69;
    }
    while(1) {
     $70 = HEAP32[$z1>>2]|0;
     $71 = ((($70)) + 1|0);
     HEAP32[$z1>>2] = $71;
     $72 = HEAP8[$71>>0]|0;
     $73 = $72 << 24 >> 24;
     $74 = 192 & $73;
     $75 = ($74|0)==(128);
     if (!($75)) {
      continue L11;
     }
    }
   }
   $76 = HEAP32[$zot>>2]|0;
   HEAP8[$76>>0] = 0;
   $77 = HEAP32[$0>>2]|0;
   $78 = HEAP32[$zo>>2]|0;
   _sqlite3_result_text($77,$78,-1,(-1));
   $79 = HEAP32[$zo>>2]|0;
   _sqlite3_free($79);
   STACKTOP = sp;return;
  }
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_null($14);
 STACKTOP = sp;return;
}
function _varianceStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0.0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0.0;
 var $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0.0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $delta = 0, $p = 0, $x = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $p = sp + 16|0;
 $delta = sp + 8|0;
 $x = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(1);
 if (!($4)) {
  ___assert_fail((16726|0),(16734|0),1395,(17576|0));
  // unreachable;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3_aggregate_context($5,24)|0);
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_numeric_type($8)|0);
 $10 = (5)!=($9|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = $12;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (_i64Add(($15|0),($18|0),1,0)|0);
 $20 = tempRet0;
 $21 = $12;
 $22 = $21;
 HEAP32[$22>>2] = $19;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = (+_sqlite3_value_double($26));
 HEAPF64[$x>>3] = $27;
 $28 = +HEAPF64[$x>>3];
 $29 = HEAP32[$p>>2]|0;
 $30 = +HEAPF64[$29>>3];
 $31 = $28 - $30;
 HEAPF64[$delta>>3] = $31;
 $32 = +HEAPF64[$delta>>3];
 $33 = HEAP32[$p>>2]|0;
 $34 = ((($33)) + 16|0);
 $35 = $34;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (+($37>>>0)) + (4294967296.0*(+($40|0)));
 $42 = $32 / $41;
 $43 = HEAP32[$p>>2]|0;
 $44 = +HEAPF64[$43>>3];
 $45 = $44 + $42;
 HEAPF64[$43>>3] = $45;
 $46 = +HEAPF64[$delta>>3];
 $47 = +HEAPF64[$x>>3];
 $48 = HEAP32[$p>>2]|0;
 $49 = +HEAPF64[$48>>3];
 $50 = $47 - $49;
 $51 = $46 * $50;
 $52 = HEAP32[$p>>2]|0;
 $53 = ((($52)) + 8|0);
 $54 = +HEAPF64[$53>>3];
 $55 = $54 + $51;
 HEAPF64[$53>>3] = $55;
 STACKTOP = sp;return;
}
function _stdevFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 16|0);
  $7 = $6;
  $8 = $7;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($7) + 4)|0;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>(0);
  $14 = ($9>>>0)>(1);
  $15 = ($12|0)==(0);
  $16 = $15 & $14;
  $17 = $13 | $16;
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = +HEAPF64[$20>>3];
   $22 = HEAP32[$p>>2]|0;
   $23 = ((($22)) + 16|0);
   $24 = $23;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + 4)|0;
   $28 = $27;
   $29 = HEAP32[$28>>2]|0;
   $30 = (_i64Subtract(($26|0),($29|0),1,0)|0);
   $31 = tempRet0;
   $32 = (+($30>>>0)) + (4294967296.0*(+($31|0)));
   $33 = $21 / $32;
   $34 = (+Math_sqrt((+$33)));
   _sqlite3_result_double($18,$34);
   STACKTOP = sp;return;
  }
 }
 $35 = HEAP32[$0>>2]|0;
 _sqlite3_result_double($35,0.0);
 STACKTOP = sp;return;
}
function _varianceFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0.0, $33 = 0.0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 16|0);
  $7 = $6;
  $8 = $7;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($7) + 4)|0;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>(0);
  $14 = ($9>>>0)>(1);
  $15 = ($12|0)==(0);
  $16 = $15 & $14;
  $17 = $13 | $16;
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = +HEAPF64[$20>>3];
   $22 = HEAP32[$p>>2]|0;
   $23 = ((($22)) + 16|0);
   $24 = $23;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + 4)|0;
   $28 = $27;
   $29 = HEAP32[$28>>2]|0;
   $30 = (_i64Subtract(($26|0),($29|0),1,0)|0);
   $31 = tempRet0;
   $32 = (+($30>>>0)) + (4294967296.0*(+($31|0)));
   $33 = $21 / $32;
   _sqlite3_result_double($18,$33);
   STACKTOP = sp;return;
  }
 }
 $34 = HEAP32[$0>>2]|0;
 _sqlite3_result_double($34,0.0);
 STACKTOP = sp;return;
}
function _modeStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0.0, $94 = 0, $95 = 0.0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $dptr = 0, $iptr = 0, $p = 0, $type = 0, $xd = 0, $xi = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $p = sp + 52|0;
 $xi = sp + 8|0;
 $xd = sp;
 $iptr = sp + 48|0;
 $dptr = sp + 44|0;
 $type = sp + 40|0;
 $3 = sp + 28|0;
 $4 = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $5 = $xi;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 HEAPF64[$xd>>3] = 0.0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ($9|0)==(1);
 if (!($10)) {
  ___assert_fail((16726|0),(16734|0),1418,(17589|0));
  // unreachable;
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3_value_numeric_type($12)|0);
 HEAP32[$type>>2] = $13;
 $14 = HEAP32[$type>>2]|0;
 $15 = ($14|0)==(5);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = (_sqlite3_aggregate_context($16,64)|0);
 HEAP32[$p>>2] = $17;
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 56|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (0|0)==($20|0);
 do {
  if ($21) {
   $22 = (_calloc(1,12)|0);
   $23 = HEAP32[$p>>2]|0;
   $24 = ((($23)) + 56|0);
   HEAP32[$24>>2] = $22;
   $25 = HEAP32[$type>>2]|0;
   $26 = ($25|0)==(1);
   $27 = HEAP32[$p>>2]|0;
   if ($26) {
    $28 = ((($27)) + 56|0);
    $29 = HEAP32[$28>>2]|0;
    _map_make($3,176);
    ;HEAP32[$29>>2]=HEAP32[$3>>2]|0;HEAP32[$29+4>>2]=HEAP32[$3+4>>2]|0;HEAP32[$29+8>>2]=HEAP32[$3+8>>2]|0;
    $30 = HEAP32[$p>>2]|0;
    $31 = ((($30)) + 48|0);
    $32 = $31;
    $33 = $32;
    HEAP32[$33>>2] = 0;
    $34 = (($32) + 4)|0;
    $35 = $34;
    HEAP32[$35>>2] = 0;
    break;
   } else {
    $36 = ((($27)) + 48|0);
    $37 = $36;
    $38 = $37;
    HEAP32[$38>>2] = 1;
    $39 = (($37) + 4)|0;
    $40 = $39;
    HEAP32[$40>>2] = 0;
    $41 = HEAP32[$p>>2]|0;
    $42 = ((($41)) + 56|0);
    $43 = HEAP32[$42>>2]|0;
    _map_make($4,177);
    ;HEAP32[$43>>2]=HEAP32[$4>>2]|0;HEAP32[$43+4>>2]=HEAP32[$4+4>>2]|0;HEAP32[$43+8>>2]=HEAP32[$4+8>>2]|0;
    break;
   }
  }
 } while(0);
 $44 = HEAP32[$p>>2]|0;
 $45 = ((($44)) + 16|0);
 $46 = $45;
 $47 = $46;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($46) + 4)|0;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = (_i64Add(($48|0),($51|0),1,0)|0);
 $53 = tempRet0;
 $54 = $45;
 $55 = $54;
 HEAP32[$55>>2] = $52;
 $56 = (($54) + 4)|0;
 $57 = $56;
 HEAP32[$57>>2] = $53;
 $58 = HEAP32[$p>>2]|0;
 $59 = ((($58)) + 48|0);
 $60 = $59;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (($60) + 4)|0;
 $64 = $63;
 $65 = HEAP32[$64>>2]|0;
 $66 = (0)==($62|0);
 $67 = (0)==($65|0);
 $68 = $66 & $67;
 $69 = HEAP32[$2>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 if ($68) {
  $71 = (_sqlite3_value_int64($70)|0);
  $72 = tempRet0;
  $73 = $xi;
  $74 = $73;
  HEAP32[$74>>2] = $71;
  $75 = (($73) + 4)|0;
  $76 = $75;
  HEAP32[$76>>2] = $72;
  $77 = (_calloc(1,8)|0);
  HEAP32[$iptr>>2] = $77;
  $78 = $xi;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = HEAP32[$iptr>>2]|0;
  $85 = $84;
  $86 = $85;
  HEAP32[$86>>2] = $80;
  $87 = (($85) + 4)|0;
  $88 = $87;
  HEAP32[$88>>2] = $83;
  $89 = HEAP32[$p>>2]|0;
  $90 = ((($89)) + 56|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = HEAP32[$iptr>>2]|0;
  _map_insert($91,$92);
  STACKTOP = sp;return;
 } else {
  $93 = (+_sqlite3_value_double($70));
  HEAPF64[$xd>>3] = $93;
  $94 = (_calloc(1,8)|0);
  HEAP32[$dptr>>2] = $94;
  $95 = +HEAPF64[$xd>>3];
  $96 = HEAP32[$dptr>>2]|0;
  HEAPF64[$96>>3] = $95;
  $97 = HEAP32[$p>>2]|0;
  $98 = ((($97)) + 56|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = HEAP32[$dptr>>2]|0;
  _map_insert($99,$100);
  STACKTOP = sp;return;
 }
}
function _modeFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$p>>2]|0;
 _map_iterate($11,131,$12);
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 _map_destroy($15);
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 56|0);
 $18 = HEAP32[$17>>2]|0;
 _free($18);
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 40|0);
 $21 = $20;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (1)==($23|0);
 $28 = (0)==($26|0);
 $29 = $27 & $28;
 if (!($29)) {
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 48|0);
 $32 = $31;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (0)==($34|0);
 $39 = (0)==($37|0);
 $40 = $38 & $39;
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$p>>2]|0;
 if ($40) {
  $43 = $42;
  $44 = $43;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($43) + 4)|0;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  _sqlite3_result_int64($41,$45,$48);
  STACKTOP = sp;return;
 } else {
  $49 = ((($42)) + 8|0);
  $50 = +HEAPF64[$49>>3];
  _sqlite3_result_double($41,$50);
  STACKTOP = sp;return;
 }
}
function _medianFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (+($9>>>0)) + (4294967296.0*(+($12|0)));
 $14 = $13 / 2.0;
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 24|0);
 HEAPF64[$16>>3] = $14;
 $17 = HEAP32[$0>>2]|0;
 __medianFinalize($17);
 STACKTOP = sp;return;
}
function _lower_quartileFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (+($9>>>0)) + (4294967296.0*(+($12|0)));
 $14 = $13 / 4.0;
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 24|0);
 HEAPF64[$16>>3] = $14;
 $17 = HEAP32[$0>>2]|0;
 __medianFinalize($17);
 STACKTOP = sp;return;
}
function _upper_quartileFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (___muldi3(($9|0),($12|0),3,0)|0);
 $14 = tempRet0;
 $15 = (+($13>>>0)) + (4294967296.0*(+($14|0)));
 $16 = $15 / 4.0;
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 24|0);
 HEAPF64[$18>>3] = $16;
 $19 = HEAP32[$0>>2]|0;
 __medianFinalize($19);
 STACKTOP = sp;return;
}
function _map_make($agg$result,$cmp) {
 $agg$result = $agg$result|0;
 $cmp = $cmp|0;
 var $0 = 0, $1 = 0, $2 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $r = sp;
 HEAP32[$0>>2] = $cmp;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($r)) + 4|0);
 HEAP32[$2>>2] = $1;
 HEAP32[$r>>2] = 0;
 ;HEAP32[$agg$result>>2]=HEAP32[$r>>2]|0;HEAP32[$agg$result+4>>2]=HEAP32[$r+4>>2]|0;HEAP32[$agg$result+8>>2]=HEAP32[$r+8>>2]|0;
 STACKTOP = sp;return;
}
function _xcalloc($nmemb,$size,$s) {
 $nmemb = $nmemb|0;
 $size = $size|0;
 $s = $s|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $ret = sp;
 HEAP32[$0>>2] = $nmemb;
 HEAP32[$1>>2] = $size;
 HEAP32[$2>>2] = $s;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_calloc($3,$4)|0);
 HEAP32[$ret>>2] = $5;
 $6 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _xfree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _free($1);
 STACKTOP = sp;return;
}
function _node_insert($n,$cmp,$e) {
 $n = $n|0;
 $cmp = $cmp|0;
 $e = $e|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $nn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $c = sp + 4|0;
 $nn = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $cmp;
 HEAP32[$2>>2] = $e;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = (_xcalloc(1,24,16717)|0);
  HEAP32[$nn>>2] = $6;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$nn>>2]|0;
  $9 = ((($8)) + 8|0);
  HEAP32[$9>>2] = $7;
  $10 = HEAP32[$nn>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = $11;
  $13 = $12;
  HEAP32[$13>>2] = 1;
  $14 = (($12) + 4)|0;
  $15 = $14;
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$nn>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  HEAP32[$17>>2] = $16;
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (FUNCTION_TABLE_iii[$18 & 255]($22,$23)|0);
 HEAP32[$c>>2] = $24;
 $25 = HEAP32[$c>>2]|0;
 $26 = (0)==($25|0);
 if ($26) {
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = $29;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_i64Add(($32|0),($35|0),1,0)|0);
  $37 = tempRet0;
  $38 = $29;
  $39 = $38;
  HEAP32[$39>>2] = $36;
  $40 = (($38) + 4)|0;
  $41 = $40;
  HEAP32[$41>>2] = $37;
  $42 = HEAP32[$2>>2]|0;
  _xfree($42);
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$c>>2]|0;
 $44 = ($43|0)>(0);
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 if ($44) {
  $47 = HEAP32[$1>>2]|0;
  $48 = HEAP32[$2>>2]|0;
  _node_insert($46,$47,$48);
  STACKTOP = sp;return;
 } else {
  $49 = ((($46)) + 4|0);
  $50 = HEAP32[$1>>2]|0;
  $51 = HEAP32[$2>>2]|0;
  _node_insert($49,$50,$51);
  STACKTOP = sp;return;
 }
}
function _map_insert($m,$e) {
 $m = $m|0;
 $e = $e|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $m;
 HEAP32[$1>>2] = $e;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 _node_insert($2,$5,$6);
 STACKTOP = sp;return;
}
function _node_iterate($n,$iter,$p) {
 $n = $n|0;
 $iter = $iter|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $iter;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  _node_iterate($9,$10,$11);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = $17;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 FUNCTION_TABLE_viiii[$12 & 255]($15,$20,$23,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  STACKTOP = sp;return;
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 _node_iterate($31,$32,$33);
 STACKTOP = sp;return;
}
function _map_iterate($m,$iter,$p) {
 $m = $m|0;
 $iter = $iter|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $m;
 HEAP32[$1>>2] = $iter;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 _node_iterate($4,$5,$6);
 STACKTOP = sp;return;
}
function _node_destroy($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 $2 = (0|0)!=($1|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 _xfree($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  _node_destroy($10);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  _node_destroy($17);
 }
 $18 = HEAP32[$0>>2]|0;
 _xfree($18);
 STACKTOP = sp;return;
}
function _map_destroy($m) {
 $m = $m|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $m;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _node_destroy($2);
 STACKTOP = sp;return;
}
function _int_cmp($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aa = 0, $bb = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $aa = sp + 8|0;
 $bb = sp;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $b;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = $aa;
 $11 = $10;
 HEAP32[$11>>2] = $6;
 $12 = (($10) + 4)|0;
 $13 = $12;
 HEAP32[$13>>2] = $9;
 $14 = HEAP32[$2>>2]|0;
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $bb;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = $aa;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = $bb;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = ($27|0)==($33|0);
 $38 = ($30|0)==($36|0);
 $39 = $37 & $38;
 if ($39) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $40 = $aa;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($40) + 4)|0;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 $46 = $bb;
 $47 = $46;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($46) + 4)|0;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = ($45|0)<($51|0);
 $53 = ($42>>>0)<($48>>>0);
 $54 = ($45|0)==($51|0);
 $55 = $54 & $53;
 $56 = $52 | $55;
 if ($56) {
  HEAP32[$0>>2] = -1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 } else {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return (0)|0;
}
function _double_cmp($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0.0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0.0, $9 = 0, $aa = 0, $bb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $aa = sp + 8|0;
 $bb = sp;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $b;
 $3 = HEAP32[$1>>2]|0;
 $4 = +HEAPF64[$3>>3];
 HEAPF64[$aa>>3] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = +HEAPF64[$5>>3];
 HEAPF64[$bb>>3] = $6;
 $7 = +HEAPF64[$aa>>3];
 $8 = +HEAPF64[$bb>>3];
 $9 = $7 == $8;
 do {
  if ($9) {
   HEAP32[$0>>2] = 0;
  } else {
   $10 = +HEAPF64[$aa>>3];
   $11 = +HEAPF64[$bb>>3];
   $12 = $10 < $11;
   if ($12) {
    HEAP32[$0>>2] = -1;
    break;
   } else {
    HEAP32[$0>>2] = 1;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _soundex($zIn,$zResult) {
 $zIn = $zIn|0;
 $zResult = $zResult|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $code = 0, $i = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $code = sp;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $zResult;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = (($3) + ($2)|0);
  $5 = HEAP8[$4>>0]|0;
  $6 = $5&255;
  $7 = ($6|0)!=(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + ($8)|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = (_isalpha($12)|0);
  $14 = ($13|0)!=(0);
  $15 = $14 ^ 1;
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$i>>2] = $17;
 }
 $18 = HEAP32[$i>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = (($19) + ($18)|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = ($21<<24>>24)!=(0);
 if (!($22)) {
  $67 = HEAP32[$1>>2]|0;
  (_strcpy($67,16975)|0);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$i>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = (($24) + ($23)|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = (_toupper($27)|0);
 $29 = $28&255;
 $30 = HEAP32[$1>>2]|0;
 HEAP8[$30>>0] = $29;
 HEAP32[$j>>2] = 1;
 while(1) {
  $31 = HEAP32[$j>>2]|0;
  $32 = ($31|0)<(4);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$0>>2]|0;
  $35 = (($34) + ($33)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = ($37|0)!=(0);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$0>>2]|0;
  $41 = (($40) + ($39)|0);
  $42 = HEAP8[$41>>0]|0;
  $43 = $42&255;
  $44 = $43 & 127;
  $45 = (16847 + ($44)|0);
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  HEAP32[$code>>2] = $47;
  $48 = HEAP32[$code>>2]|0;
  $49 = ($48|0)>(0);
  if ($49) {
   $50 = HEAP32[$code>>2]|0;
   $51 = (($50) + 48)|0;
   $52 = $51&255;
   $53 = HEAP32[$j>>2]|0;
   $54 = (($53) + 1)|0;
   HEAP32[$j>>2] = $54;
   $55 = HEAP32[$1>>2]|0;
   $56 = (($55) + ($53)|0);
   HEAP8[$56>>0] = $52;
  }
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$i>>2] = $58;
 }
 while(1) {
  $59 = HEAP32[$j>>2]|0;
  $60 = ($59|0)<(4);
  $61 = HEAP32[$j>>2]|0;
  if (!($60)) {
   break;
  }
  $62 = (($61) + 1)|0;
  HEAP32[$j>>2] = $62;
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + ($61)|0);
  HEAP8[$64>>0] = 48;
 }
 $65 = HEAP32[$1>>2]|0;
 $66 = (($65) + ($61)|0);
 HEAP8[$66>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3ReadUtf8($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $xtra = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $c = sp + 4|0;
 $xtra = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 1|0);
 HEAP32[$0>>2] = $2;
 $3 = HEAP8[$1>>0]|0;
 $4 = $3&255;
 HEAP32[$c>>2] = $4;
 $5 = HEAP32[$c>>2]|0;
 $6 = (16980 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$xtra>>2] = $8;
 $9 = HEAP32[$xtra>>2]|0;
 switch ($9|0) {
 case 4:  {
  HEAP32[$c>>2] = 65533;
  $48 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($48|0);
  break;
 }
 case 3:  {
  $10 = HEAP32[$c>>2]|0;
  $11 = $10 << 6;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$0>>2] = $13;
  $14 = HEAP8[$12>>0]|0;
  $15 = $14&255;
  $16 = (($11) + ($15))|0;
  HEAP32[$c>>2] = $16;
  label = 4;
  break;
 }
 case 2:  {
  label = 4;
  break;
 }
 case 1:  {
  break;
 }
 default: {
  $48 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 }
 if ((label|0) == 4) {
  $17 = HEAP32[$c>>2]|0;
  $18 = $17 << 6;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 1|0);
  HEAP32[$0>>2] = $20;
  $21 = HEAP8[$19>>0]|0;
  $22 = $21&255;
  $23 = (($18) + ($22))|0;
  HEAP32[$c>>2] = $23;
 }
 $24 = HEAP32[$c>>2]|0;
 $25 = $24 << 6;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 1|0);
 HEAP32[$0>>2] = $27;
 $28 = HEAP8[$26>>0]|0;
 $29 = $28&255;
 $30 = (($25) + ($29))|0;
 HEAP32[$c>>2] = $30;
 $31 = HEAP32[$xtra>>2]|0;
 $32 = (936 + ($31<<2)|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$c>>2]|0;
 $35 = (($34) - ($33))|0;
 HEAP32[$c>>2] = $35;
 $36 = HEAP32[$xtra>>2]|0;
 $37 = (952 + ($36<<2)|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$c>>2]|0;
 $40 = $38 & $39;
 $41 = ($40|0)==(0);
 if (!($41)) {
  $42 = HEAP32[$c>>2]|0;
  $43 = $42 & -2048;
  $44 = ($43|0)==(55296);
  if (!($44)) {
   $45 = HEAP32[$c>>2]|0;
   $46 = $45 & -2;
   $47 = ($46|0)==(65534);
   if (!($47)) {
    $48 = HEAP32[$c>>2]|0;
    STACKTOP = sp;return ($48|0);
   }
  }
 }
 HEAP32[$c>>2] = 65533;
 $48 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _rad2deg($x) {
 $x = +$x;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAPF64[$0>>3] = $x;
 $1 = +HEAPF64[$0>>3];
 $2 = 180.0 * $1;
 $3 = $2 / 3.1415926535897931;
 STACKTOP = sp;return (+$3);
}
function _deg2rad($x) {
 $x = +$x;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAPF64[$0>>3] = $x;
 $1 = +HEAPF64[$0>>3];
 $2 = $1 * 3.1415926535897931;
 $3 = $2 / 180.0;
 STACKTOP = sp;return (+$3);
}
function _cot($x) {
 $x = +$x;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAPF64[$0>>3] = $x;
 $1 = +HEAPF64[$0>>3];
 $2 = (+Math_tan((+$1)));
 $3 = 1.0 / $2;
 STACKTOP = sp;return (+$3);
}
function _coth($x) {
 $x = +$x;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAPF64[$0>>3] = $x;
 $1 = +HEAPF64[$0>>3];
 $2 = (+_tanh($1));
 $3 = 1.0 / $2;
 STACKTOP = sp;return (+$3);
}
function __substr($z1,$z2,$s,$p) {
 $z1 = $z1|0;
 $z2 = $z2|0;
 $s = $s|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $c = 0, $c1 = 0, $c2 = 0, $or$cond = 0, $or$cond4 = 0, $rVal = 0, $zt1 = 0, $zt2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $c = sp + 20|0;
 $rVal = sp + 16|0;
 $zt1 = sp + 12|0;
 $zt2 = sp + 8|0;
 $c1 = sp + 4|0;
 $c2 = sp;
 HEAP32[$1>>2] = $z1;
 HEAP32[$2>>2] = $z2;
 HEAP32[$3>>2] = $s;
 HEAP32[$4>>2] = $p;
 HEAP32[$c>>2] = 0;
 HEAP32[$rVal>>2] = -1;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6 << 24 >> 24;
 $8 = (0)==($7|0);
 if ($8) {
  HEAP32[$0>>2] = -1;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 L4: while(1) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3ReadUtf8($9)|0);
  $11 = ($10|0)!=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$c>>2]|0;
  $13 = (($12) + 1)|0;
  HEAP32[$c>>2] = $13;
  $14 = HEAP32[$3>>2]|0;
  $15 = ($12|0)<($14|0);
  if (!($15)) {
   break;
  }
  while(1) {
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 1|0);
   HEAP32[$2>>2] = $17;
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   $20 = 192 & $19;
   $21 = ($20|0)==(128);
   if (!($21)) {
    continue L4;
   }
  }
 }
 HEAP32[$c>>2] = 0;
 while(1) {
  $22 = HEAP32[$2>>2]|0;
  $23 = (_sqlite3ReadUtf8($22)|0);
  $24 = ($23|0)!=(0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$1>>2]|0;
  HEAP32[$zt1>>2] = $25;
  $26 = HEAP32[$2>>2]|0;
  HEAP32[$zt2>>2] = $26;
  while(1) {
   $27 = HEAP32[$zt1>>2]|0;
   $28 = (_sqlite3ReadUtf8($27)|0);
   HEAP32[$c1>>2] = $28;
   $29 = HEAP32[$zt2>>2]|0;
   $30 = (_sqlite3ReadUtf8($29)|0);
   HEAP32[$c2>>2] = $30;
   while(1) {
    $31 = HEAP32[$zt1>>2]|0;
    $32 = ((($31)) + 1|0);
    HEAP32[$zt1>>2] = $32;
    $33 = HEAP8[$32>>0]|0;
    $34 = $33 << 24 >> 24;
    $35 = 192 & $34;
    $36 = ($35|0)==(128);
    if (!($36)) {
     break;
    }
   }
   while(1) {
    $37 = HEAP32[$zt2>>2]|0;
    $38 = ((($37)) + 1|0);
    HEAP32[$zt2>>2] = $38;
    $39 = HEAP8[$38>>0]|0;
    $40 = $39 << 24 >> 24;
    $41 = 192 & $40;
    $42 = ($41|0)==(128);
    if (!($42)) {
     break;
    }
   }
   $43 = HEAP32[$c1>>2]|0;
   $44 = HEAP32[$c2>>2]|0;
   $45 = ($43|0)==($44|0);
   $46 = HEAP32[$c1>>2]|0;
   $47 = ($46|0)!=(0);
   $or$cond = $45 & $47;
   $48 = HEAP32[$c2>>2]|0;
   $49 = ($48|0)!=(0);
   $or$cond4 = $or$cond & $49;
   if (!($or$cond4)) {
    break;
   }
  }
  $50 = HEAP32[$c1>>2]|0;
  $51 = ($50|0)==(0);
  if ($51) {
   label = 14;
   break;
  }
  while(1) {
   $53 = HEAP32[$2>>2]|0;
   $54 = ((($53)) + 1|0);
   HEAP32[$2>>2] = $54;
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = 192 & $56;
   $58 = ($57|0)==(128);
   if (!($58)) {
    break;
   }
  }
  $59 = HEAP32[$c>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$c>>2] = $60;
 }
 if ((label|0) == 14) {
  $52 = HEAP32[$c>>2]|0;
  HEAP32[$rVal>>2] = $52;
 }
 $61 = HEAP32[$4>>2]|0;
 $62 = ($61|0)!=(0|0);
 if ($62) {
  $63 = HEAP32[$2>>2]|0;
  $64 = HEAP32[$4>>2]|0;
  HEAP32[$64>>2] = $63;
 }
 $65 = HEAP32[$rVal>>2]|0;
 $66 = ($65|0)>=(0);
 $67 = HEAP32[$rVal>>2]|0;
 $68 = HEAP32[$3>>2]|0;
 $69 = (($67) + ($68))|0;
 $70 = $66 ? $69 : $67;
 HEAP32[$0>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($71|0);
}
function _sqlite3StrDup($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_strlen($1)|0);
 $3 = (($2) + 1)|0;
 $4 = (_sqlite3_malloc($3)|0);
 HEAP32[$res>>2] = $4;
 $5 = HEAP32[$res>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_strcpy($5,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _sqlite3Utf8CharLen($z,$nByte) {
 $z = $z|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $r = sp + 4|0;
 $zTerm = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $nByte;
 HEAP32[$r>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (($5) + ($4)|0);
  HEAP32[$zTerm>>2] = $6;
 } else {
  HEAP32[$zTerm>>2] = (-1);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$zTerm>>2]|0;
 $9 = ($7>>>0)<=($8>>>0);
 if (!($9)) {
  ___assert_fail((17516|0),(16734|0),316,(17525|0));
  // unreachable;
 }
 while(1) {
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP8[$10>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   label = 10;
   break;
  }
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$zTerm>>2]|0;
  $16 = ($14>>>0)<($15>>>0);
  if (!($16)) {
   label = 10;
   break;
  }
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = (16980 + ($19)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (($22) + 1)|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + ($23)|0);
  HEAP32[$0>>2] = $25;
  $26 = HEAP32[$r>>2]|0;
  $27 = (($26) + 1)|0;
  HEAP32[$r>>2] = $27;
 }
 if ((label|0) == 10) {
  $28 = HEAP32[$r>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _modeIterate($e,$0,$1,$pp) {
 $e = $e|0;
 $0 = $0|0;
 $1 = $1|0;
 $pp = $pp|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0.0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0.0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ed = 0, $ei = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 16|0;
 $4 = sp + 28|0;
 $ei = sp + 8|0;
 $ed = sp;
 $p = sp + 24|0;
 HEAP32[$2>>2] = $e;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pp;
 $9 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 48|0);
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (0)==($14|0);
 $19 = (0)==($17|0);
 $20 = $18 & $19;
 $21 = HEAP32[$2>>2]|0;
 if ($20) {
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)<(0);
  $24 = $23 << 31 >> 31;
  $25 = $ei;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = (($25) + 4)|0;
  $28 = $27;
  HEAP32[$28>>2] = $24;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 32|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = $3;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = ($33|0)==($39|0);
  $44 = ($36|0)==($42|0);
  $45 = $43 & $44;
  $46 = HEAP32[$p>>2]|0;
  if ($45) {
   $47 = ((($46)) + 40|0);
   $48 = $47;
   $49 = $48;
   $50 = HEAP32[$49>>2]|0;
   $51 = (($48) + 4)|0;
   $52 = $51;
   $53 = HEAP32[$52>>2]|0;
   $54 = (_i64Add(($50|0),($53|0),1,0)|0);
   $55 = tempRet0;
   $56 = $47;
   $57 = $56;
   HEAP32[$57>>2] = $54;
   $58 = (($56) + 4)|0;
   $59 = $58;
   HEAP32[$59>>2] = $55;
   STACKTOP = sp;return;
  }
  $60 = ((($46)) + 32|0);
  $61 = $60;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (($61) + 4)|0;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = $3;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($66|0)<($72|0);
  $74 = ($63>>>0)<($69>>>0);
  $75 = ($66|0)==($72|0);
  $76 = $75 & $74;
  $77 = $73 | $76;
  if (!($77)) {
   STACKTOP = sp;return;
  }
  $78 = $ei;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = HEAP32[$p>>2]|0;
  $85 = $84;
  $86 = $85;
  HEAP32[$86>>2] = $80;
  $87 = (($85) + 4)|0;
  $88 = $87;
  HEAP32[$88>>2] = $83;
  $89 = $3;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = HEAP32[$p>>2]|0;
  $96 = ((($95)) + 32|0);
  $97 = $96;
  $98 = $97;
  HEAP32[$98>>2] = $91;
  $99 = (($97) + 4)|0;
  $100 = $99;
  HEAP32[$100>>2] = $94;
  $101 = HEAP32[$p>>2]|0;
  $102 = ((($101)) + 40|0);
  $103 = $102;
  $104 = $103;
  HEAP32[$104>>2] = 1;
  $105 = (($103) + 4)|0;
  $106 = $105;
  HEAP32[$106>>2] = 0;
  STACKTOP = sp;return;
 } else {
  $107 = +HEAPF64[$21>>3];
  HEAPF64[$ed>>3] = $107;
  $108 = HEAP32[$p>>2]|0;
  $109 = ((($108)) + 32|0);
  $110 = $109;
  $111 = $110;
  $112 = HEAP32[$111>>2]|0;
  $113 = (($110) + 4)|0;
  $114 = $113;
  $115 = HEAP32[$114>>2]|0;
  $116 = $3;
  $117 = $116;
  $118 = HEAP32[$117>>2]|0;
  $119 = (($116) + 4)|0;
  $120 = $119;
  $121 = HEAP32[$120>>2]|0;
  $122 = ($112|0)==($118|0);
  $123 = ($115|0)==($121|0);
  $124 = $122 & $123;
  $125 = HEAP32[$p>>2]|0;
  if ($124) {
   $126 = ((($125)) + 40|0);
   $127 = $126;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = (($127) + 4)|0;
   $131 = $130;
   $132 = HEAP32[$131>>2]|0;
   $133 = (_i64Add(($129|0),($132|0),1,0)|0);
   $134 = tempRet0;
   $135 = $126;
   $136 = $135;
   HEAP32[$136>>2] = $133;
   $137 = (($135) + 4)|0;
   $138 = $137;
   HEAP32[$138>>2] = $134;
   STACKTOP = sp;return;
  }
  $139 = ((($125)) + 32|0);
  $140 = $139;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = (($140) + 4)|0;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = $3;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (($146) + 4)|0;
  $150 = $149;
  $151 = HEAP32[$150>>2]|0;
  $152 = ($145|0)<($151|0);
  $153 = ($142>>>0)<($148>>>0);
  $154 = ($145|0)==($151|0);
  $155 = $154 & $153;
  $156 = $152 | $155;
  if (!($156)) {
   STACKTOP = sp;return;
  }
  $157 = +HEAPF64[$ed>>3];
  $158 = HEAP32[$p>>2]|0;
  $159 = ((($158)) + 8|0);
  HEAPF64[$159>>3] = $157;
  $160 = $3;
  $161 = $160;
  $162 = HEAP32[$161>>2]|0;
  $163 = (($160) + 4)|0;
  $164 = $163;
  $165 = HEAP32[$164>>2]|0;
  $166 = HEAP32[$p>>2]|0;
  $167 = ((($166)) + 32|0);
  $168 = $167;
  $169 = $168;
  HEAP32[$169>>2] = $162;
  $170 = (($168) + 4)|0;
  $171 = $170;
  HEAP32[$171>>2] = $165;
  $172 = HEAP32[$p>>2]|0;
  $173 = ((($172)) + 40|0);
  $174 = $173;
  $175 = $174;
  HEAP32[$175>>2] = 1;
  $176 = (($174) + 4)|0;
  $177 = $176;
  HEAP32[$177>>2] = 0;
  STACKTOP = sp;return;
 }
}
function __medianFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0.0, $62 = 0.0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0.0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0.0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 60|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 56|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$p>>2]|0;
 _map_iterate($13,132,$14);
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 56|0);
 $17 = HEAP32[$16>>2]|0;
 _map_destroy($17);
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 56|0);
 $20 = HEAP32[$19>>2]|0;
 _free($20);
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 48|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (0)==($25|0);
 $30 = (0)==($28|0);
 $31 = $29 & $30;
 if (!($31)) {
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$p>>2]|0;
  $65 = ((($64)) + 8|0);
  $66 = +HEAPF64[$65>>3];
  $67 = HEAP32[$p>>2]|0;
  $68 = ((($67)) + 40|0);
  $69 = $68;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (+($71>>>0)) + (4294967296.0*(+($74|0)));
  $76 = $66 / $75;
  _sqlite3_result_double($63,$76);
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 40|0);
 $34 = $33;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (1)==($36|0);
 $41 = (0)==($39|0);
 $42 = $40 & $41;
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$p>>2]|0;
 $45 = $44;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($45) + 4)|0;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 if ($42) {
  _sqlite3_result_int64($43,$47,$50);
  STACKTOP = sp;return;
 } else {
  $51 = (+($47>>>0)) + (4294967296.0*(+($50|0)));
  $52 = $51 * 1.0;
  $53 = HEAP32[$p>>2]|0;
  $54 = ((($53)) + 40|0);
  $55 = $54;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($55) + 4)|0;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (+($57>>>0)) + (4294967296.0*(+($60|0)));
  $62 = $52 / $61;
  _sqlite3_result_double($43,$62);
  STACKTOP = sp;return;
 }
}
function _medianIterate($e,$0,$1,$pp) {
 $e = $e|0;
 $0 = $0|0;
 $1 = $1|0;
 $pp = $pp|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0.0, $125 = 0.0, $126 = 0, $127 = 0, $128 = 0.0, $129 = 0.0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $16 = 0.0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0.0, $26 = 0, $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0.0, $66 = 0.0, $67 = 0, $68 = 0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ed = 0, $ei = 0, $iL = 0, $iR = 0, $il = 0, $ir = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 56|0;
 $3 = sp + 32|0;
 $4 = sp + 52|0;
 $ei = sp + 24|0;
 $ed = sp + 16|0;
 $iL = sp + 8|0;
 $iR = sp;
 $il = sp + 48|0;
 $ir = sp + 44|0;
 $p = sp + 40|0;
 HEAP32[$2>>2] = $e;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pp;
 $9 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 60|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = +HEAPF64[$15>>3];
 HEAPF64[$iL>>3] = $16;
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (+($21>>>0)) + (4294967296.0*(+($24|0)));
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 24|0);
 $28 = +HEAPF64[$27>>3];
 $29 = $25 - $28;
 HEAPF64[$iR>>3] = $29;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 32|0);
 $32 = $31;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = $3;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (_i64Add(($34|0),($37|0),($40|0),($43|0))|0);
 $45 = tempRet0;
 HEAP32[$il>>2] = $44;
 $46 = HEAP32[$p>>2]|0;
 $47 = ((($46)) + 16|0);
 $48 = $47;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$p>>2]|0;
 $55 = ((($54)) + 32|0);
 $56 = $55;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($56) + 4)|0;
 $60 = $59;
 $61 = HEAP32[$60>>2]|0;
 $62 = (_i64Subtract(($50|0),($53|0),($58|0),($61|0))|0);
 $63 = tempRet0;
 HEAP32[$ir>>2] = $62;
 $64 = HEAP32[$il>>2]|0;
 $65 = (+($64|0));
 $66 = +HEAPF64[$iL>>3];
 $67 = $65 >= $66;
 do {
  if ($67) {
   $68 = HEAP32[$ir>>2]|0;
   $69 = (+($68|0));
   $70 = +HEAPF64[$iR>>3];
   $71 = $69 >= $70;
   $72 = HEAP32[$p>>2]|0;
   if (!($71)) {
    $130 = ((($72)) + 60|0);
    HEAP32[$130>>2] = 1;
    break;
   }
   $73 = ((($72)) + 40|0);
   $74 = $73;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($74) + 4)|0;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = (_i64Add(($76|0),($79|0),1,0)|0);
   $81 = tempRet0;
   $82 = $73;
   $83 = $82;
   HEAP32[$83>>2] = $80;
   $84 = (($82) + 4)|0;
   $85 = $84;
   HEAP32[$85>>2] = $81;
   $86 = HEAP32[$p>>2]|0;
   $87 = ((($86)) + 48|0);
   $88 = $87;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (($88) + 4)|0;
   $92 = $91;
   $93 = HEAP32[$92>>2]|0;
   $94 = (0)==($90|0);
   $95 = (0)==($93|0);
   $96 = $94 & $95;
   $97 = HEAP32[$2>>2]|0;
   if ($96) {
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)<(0);
    $100 = $99 << 31 >> 31;
    $101 = $ei;
    $102 = $101;
    HEAP32[$102>>2] = $98;
    $103 = (($101) + 4)|0;
    $104 = $103;
    HEAP32[$104>>2] = $100;
    $105 = $ei;
    $106 = $105;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($105) + 4)|0;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = HEAP32[$p>>2]|0;
    $112 = $111;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = (($112) + 4)|0;
    $116 = $115;
    $117 = HEAP32[$116>>2]|0;
    $118 = (_i64Add(($114|0),($117|0),($107|0),($110|0))|0);
    $119 = tempRet0;
    $120 = $111;
    $121 = $120;
    HEAP32[$121>>2] = $118;
    $122 = (($120) + 4)|0;
    $123 = $122;
    HEAP32[$123>>2] = $119;
    break;
   } else {
    $124 = +HEAPF64[$97>>3];
    HEAPF64[$ed>>3] = $124;
    $125 = +HEAPF64[$ed>>3];
    $126 = HEAP32[$p>>2]|0;
    $127 = ((($126)) + 8|0);
    $128 = +HEAPF64[$127>>3];
    $129 = $128 + $125;
    HEAPF64[$127>>3] = $129;
    break;
   }
  }
 } while(0);
 $131 = $3;
 $132 = $131;
 $133 = HEAP32[$132>>2]|0;
 $134 = (($131) + 4)|0;
 $135 = $134;
 $136 = HEAP32[$135>>2]|0;
 $137 = HEAP32[$p>>2]|0;
 $138 = ((($137)) + 32|0);
 $139 = $138;
 $140 = $139;
 $141 = HEAP32[$140>>2]|0;
 $142 = (($139) + 4)|0;
 $143 = $142;
 $144 = HEAP32[$143>>2]|0;
 $145 = (_i64Add(($141|0),($144|0),($133|0),($136|0))|0);
 $146 = tempRet0;
 $147 = $138;
 $148 = $147;
 HEAP32[$148>>2] = $145;
 $149 = (($147) + 4)|0;
 $150 = $149;
 HEAP32[$150>>2] = $146;
 STACKTOP = sp;return;
}
function _sqlite3_compileoption_used($zOptName) {
 $zOptName = $zOptName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $zOptName;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_strnicmp($2,17605,7)|0);
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 7|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 HEAP32[$n>>2] = $8;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(8);
  if (!($10)) {
   label = 9;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$i>>2]|0;
  $13 = (976 + ($12<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$n>>2]|0;
  $16 = (_sqlite3_strnicmp($11,$14,$15)|0);
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$n>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = (976 + ($19<<2)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18)|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = (_sqlite3IsIdChar($23)|0);
   $25 = ($24|0)==(0);
   if ($25) {
    label = 7;
    break;
   }
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = (($26) + 1)|0;
  HEAP32[$i>>2] = $27;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _sqlite3_strnicmp($zLeft,$zRight,$N) {
 $zLeft = $zLeft|0;
 $zRight = $zRight|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $a = sp + 4|0;
 $b = sp;
 HEAP32[$1>>2] = $zLeft;
 HEAP32[$2>>2] = $zRight;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 if ($5) {
  $7 = ($6|0)!=(0|0);
  $8 = $7 ? -1 : 0;
  HEAP32[$0>>2] = $8;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $9 = ($6|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 1;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$a>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$b>>2] = $11;
 while(1) {
  $12 = HEAP32[$3>>2]|0;
  $13 = (($12) + -1)|0;
  HEAP32[$3>>2] = $13;
  $14 = ($12|0)>(0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$a>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$a>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = (17613 + ($21)|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  $25 = HEAP32[$b>>2]|0;
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = (17613 + ($27)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29&255;
  $31 = ($24|0)==($30|0);
  if (!($31)) {
   break;
  }
  $32 = HEAP32[$a>>2]|0;
  $33 = ((($32)) + 1|0);
  HEAP32[$a>>2] = $33;
  $34 = HEAP32[$b>>2]|0;
  $35 = ((($34)) + 1|0);
  HEAP32[$b>>2] = $35;
 }
 $36 = HEAP32[$3>>2]|0;
 $37 = ($36|0)<(0);
 if ($37) {
  $51 = 0;
 } else {
  $38 = HEAP32[$a>>2]|0;
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = (17613 + ($40)|0);
  $42 = HEAP8[$41>>0]|0;
  $43 = $42&255;
  $44 = HEAP32[$b>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = (17613 + ($46)|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = $48&255;
  $50 = (($43) - ($49))|0;
  $51 = $50;
 }
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3_compileoption_get($N) {
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(8);
 $or$cond = $3 & $5;
 if ($or$cond) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (976 + ($6<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _sqlite3_status64($op,$pCurrent,$pHighwater,$resetFlag) {
 $op = $op|0;
 $pCurrent = $pCurrent|0;
 $pHighwater = $pHighwater|0;
 $resetFlag = $resetFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $pMutex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pMutex = sp;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pCurrent;
 HEAP32[$3>>2] = $pHighwater;
 HEAP32[$4>>2] = $resetFlag;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<(0);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)>=(10);
 $or$cond = $6 | $8;
 if ($or$cond) {
  $9 = (_sqlite3MisuseError(18365)|0);
  HEAP32[$0>>2] = $9;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = (17869 + ($10)|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12 << 24 >> 24;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = (_sqlite3Pcache1Mutex()|0);
  $17 = $15;
 } else {
  $16 = (_sqlite3MallocMutex()|0);
  $17 = $16;
 }
 HEAP32[$pMutex>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = (1008 + ($18<<2)|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = $21;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((1048) + ($26<<2)|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = $29;
 $31 = $30;
 HEAP32[$31>>2] = $28;
 $32 = (($30) + 4)|0;
 $33 = $32;
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$4>>2]|0;
 $35 = ($34|0)!=(0);
 if ($35) {
  $36 = HEAP32[$1>>2]|0;
  $37 = (1008 + ($36<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = ((1048) + ($39<<2)|0);
  HEAP32[$40>>2] = $38;
 }
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3_vfs_find($zVfs) {
 $zVfs = $zVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $or$cond = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVfs = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $zVfs;
 HEAP32[$pVfs>>2] = 0;
 $2 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $5 = HEAP32[1088>>2]|0;
 HEAP32[$pVfs>>2] = $5;
 while(1) {
  $6 = HEAP32[$pVfs>>2]|0;
  $7 = ($6|0)==(0|0);
  $8 = HEAP32[$1>>2]|0;
  $9 = ($8|0)==(0|0);
  $or$cond = $7 | $9;
  if ($or$cond) {
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$pVfs>>2]|0;
  $12 = ((($11)) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_strcmp($10,$13)|0);
  $15 = ($14|0)==(0);
  if ($15) {
   break;
  }
  $16 = HEAP32[$pVfs>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$pVfs>>2] = $18;
 }
 $19 = HEAP32[$pVfs>>2]|0;
 HEAP32[$0>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3_initialize() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 $1 = HEAP32[(236)>>2]|0;
 $2 = ($1|0)!=(0);
 if ($2) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $5;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[(244)>>2] = 1;
 $6 = HEAP32[(248)>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MallocInit()|0);
  HEAP32[$rc>>2] = $8;
 }
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 if ($10) {
  HEAP32[(248)>>2] = 1;
  $11 = HEAP32[(260)>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   HEAP32[(260)>>2] = (8);
   $13 = HEAP32[(12)>>2]|0;
   $14 = ($13|0)==(0);
   $15 = HEAP32[(260)>>2]|0;
   $16 = ($15|0)!=(0|0);
   $or$cond = $14 | $16;
   if (!($or$cond)) {
    HEAP32[$rc>>2] = 7;
   }
  }
 }
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if ($18) {
  $19 = HEAP32[(256)>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[(256)>>2] = $20;
 }
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $23;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $24 = HEAP32[(236)>>2]|0;
 $25 = ($24|0)==(0);
 $26 = HEAP32[(240)>>2]|0;
 $27 = ($26|0)==(0);
 $or$cond3 = $25 & $27;
 if ($or$cond3) {
  HEAP32[(240)>>2] = 1;
  dest=1092; stop=dest+92|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  _sqlite3RegisterBuiltinFunctions();
  $28 = HEAP32[(252)>>2]|0;
  $29 = ($28|0)==(0);
  if ($29) {
   $30 = (_sqlite3PcacheInitialize()|0);
   HEAP32[$rc>>2] = $30;
  }
  $31 = HEAP32[$rc>>2]|0;
  $32 = ($31|0)==(0);
  if ($32) {
   HEAP32[(252)>>2] = 1;
   $33 = (_sqlite3OsInit()|0);
   HEAP32[$rc>>2] = $33;
  }
  $34 = HEAP32[$rc>>2]|0;
  $35 = ($34|0)==(0);
  if ($35) {
   $36 = HEAP32[(212)>>2]|0;
   $37 = HEAP32[(216)>>2]|0;
   $38 = HEAP32[(220)>>2]|0;
   _sqlite3PCacheBufferSetup($36,$37,$38);
   HEAP32[(236)>>2] = 1;
  }
  HEAP32[(240)>>2] = 0;
 }
 $39 = HEAP32[(256)>>2]|0;
 $40 = (($39) + -1)|0;
 HEAP32[(256)>>2] = $40;
 $41 = HEAP32[(256)>>2]|0;
 $42 = ($41|0)<=(0);
 if ($42) {
  HEAP32[(260)>>2] = 0;
 }
 $43 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3_vfs_register($pVfs,$makeDflt) {
 $pVfs = $pVfs|0;
 $makeDflt = $makeDflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $makeDflt;
 $3 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $6;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $7 = HEAP32[$1>>2]|0;
 _vfsUnlink($7);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[1088>>2]|0;
 $11 = ($10|0)==(0|0);
 $or$cond = $9 | $11;
 $12 = HEAP32[1088>>2]|0;
 if ($or$cond) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 12|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$1>>2]|0;
  HEAP32[1088>>2] = $15;
 } else {
  $16 = ((($12)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 12|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[1088>>2]|0;
  $22 = ((($21)) + 12|0);
  HEAP32[$22>>2] = $20;
 }
 HEAP32[$0>>2] = 0;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3_release_memory($n) {
 $n = $n|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $n;
 STACKTOP = sp;return 0;
}
function _sqlite3_soft_heap_limit64($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $excess = 0, $nUsed = 0, $priorLimit = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 24|0;
 $priorLimit = sp + 16|0;
 $excess = sp + 8|0;
 $nUsed = sp;
 $rc = sp + 40|0;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = $2;
  $12 = $11;
  HEAP32[$12>>2] = -1;
  $13 = (($11) + 4)|0;
  $14 = $13;
  HEAP32[$14>>2] = -1;
  $130 = $2;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  tempRet0 = ($135);
  STACKTOP = sp;return ($132|0);
 }
 $15 = (296);
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $priorLimit;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = $3;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)<(0);
 if ($31) {
  $32 = $priorLimit;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = $2;
  $39 = $38;
  HEAP32[$39>>2] = $34;
  $40 = (($38) + 4)|0;
  $41 = $40;
  HEAP32[$41>>2] = $37;
  $130 = $2;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  tempRet0 = ($135);
  STACKTOP = sp;return ($132|0);
 }
 $42 = $3;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (296);
 $49 = $48;
 HEAP32[$49>>2] = $44;
 $50 = (($48) + 4)|0;
 $51 = $50;
 HEAP32[$51>>2] = $47;
 $52 = (_sqlite3StatusValue(0)|0);
 $53 = tempRet0;
 $54 = $nUsed;
 $55 = $54;
 HEAP32[$55>>2] = $52;
 $56 = (($54) + 4)|0;
 $57 = $56;
 HEAP32[$57>>2] = $53;
 $58 = $3;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)>(0);
 $65 = ($60>>>0)>(0);
 $66 = ($63|0)==(0);
 $67 = $66 & $65;
 $68 = $64 | $67;
 if ($68) {
  $69 = $3;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = $nUsed;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = ($74|0)<($80|0);
  $82 = ($71>>>0)<=($77>>>0);
  $83 = ($74|0)==($80|0);
  $84 = $83 & $82;
  $85 = $81 | $84;
  $87 = $85;
 } else {
  $87 = 0;
 }
 $86 = $87&1;
 HEAP32[(316)>>2] = $86;
 $88 = (_sqlite3_memory_used()|0);
 $89 = tempRet0;
 $90 = $3;
 $91 = $90;
 $92 = HEAP32[$91>>2]|0;
 $93 = (($90) + 4)|0;
 $94 = $93;
 $95 = HEAP32[$94>>2]|0;
 $96 = (_i64Subtract(($88|0),($89|0),($92|0),($95|0))|0);
 $97 = tempRet0;
 $98 = $excess;
 $99 = $98;
 HEAP32[$99>>2] = $96;
 $100 = (($98) + 4)|0;
 $101 = $100;
 HEAP32[$101>>2] = $97;
 $102 = $excess;
 $103 = $102;
 $104 = HEAP32[$103>>2]|0;
 $105 = (($102) + 4)|0;
 $106 = $105;
 $107 = HEAP32[$106>>2]|0;
 $108 = ($107|0)>(0);
 $109 = ($104>>>0)>(0);
 $110 = ($107|0)==(0);
 $111 = $110 & $109;
 $112 = $108 | $111;
 if ($112) {
  $113 = $excess;
  $114 = $113;
  $115 = HEAP32[$114>>2]|0;
  $116 = (($113) + 4)|0;
  $117 = $116;
  $118 = HEAP32[$117>>2]|0;
  $119 = $115 & 2147483647;
  (_sqlite3_release_memory($119)|0);
 }
 $120 = $priorLimit;
 $121 = $120;
 $122 = HEAP32[$121>>2]|0;
 $123 = (($120) + 4)|0;
 $124 = $123;
 $125 = HEAP32[$124>>2]|0;
 $126 = $2;
 $127 = $126;
 HEAP32[$127>>2] = $122;
 $128 = (($126) + 4)|0;
 $129 = $128;
 HEAP32[$129>>2] = $125;
 $130 = $2;
 $131 = $130;
 $132 = HEAP32[$131>>2]|0;
 $133 = (($130) + 4)|0;
 $134 = $133;
 $135 = HEAP32[$134>>2]|0;
 tempRet0 = ($135);
 STACKTOP = sp;return ($132|0);
}
function _sqlite3_memory_used() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $mx = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $res = sp + 8|0;
 $mx = sp;
 (_sqlite3_status64(0,$res,$mx,0)|0);
 $0 = $res;
 $1 = $0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($0) + 4)|0;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 tempRet0 = ($5);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_malloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $n;
 $2 = (_sqlite3_initialize()|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<=(0);
 if ($5) {
  $10 = 0;
 } else {
  $6 = HEAP32[$1>>2]|0;
  $7 = ($6|0)<(0);
  $8 = $7 << 31 >> 31;
  $9 = (_sqlite3Malloc($6,$8)|0);
  $10 = $9;
 }
 HEAP32[$0>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3_malloc64($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = (_sqlite3_initialize()|0);
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$2>>2] = 0;
  $17 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($17|0);
 } else {
  $10 = $3;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (_sqlite3Malloc($12,$15)|0);
  HEAP32[$2>>2] = $16;
  $17 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3_free($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[8>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_sqlite3MallocSize($5)|0);
  _sqlite3StatusDown(0,$6);
  _sqlite3StatusDown(9,1);
  $7 = HEAP32[(52)>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$7 & 255]($8);
  STACKTOP = sp;return;
 } else {
  $9 = HEAP32[(52)>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$9 & 255]($10);
  STACKTOP = sp;return;
 }
}
function _sqlite3_realloc($pOld,$n) {
 $pOld = $pOld|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pOld;
 HEAP32[$2>>2] = $n;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  HEAP32[$2>>2] = 0;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)<(0);
 $10 = $9 << 31 >> 31;
 $11 = (_sqlite3Realloc($7,$8,$10)|0);
 HEAP32[$0>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3_realloc64($pOld,$0,$1) {
 $pOld = $pOld|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $pOld;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = (_sqlite3_initialize()|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$2>>2] = 0;
  $19 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $11 = HEAP32[$3>>2]|0;
  $12 = $4;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($12) + 4)|0;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3Realloc($11,$14,$17)|0);
  HEAP32[$2>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3_vmprintf($zFormat,$ap) {
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, $z = 0, $zBase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $z = sp + 28|0;
 $zBase = sp + 44|0;
 $acc = sp;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  _sqlite3StrAccumInit($acc,0,$zBase,70,1000000000);
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  _sqlite3VXPrintf($acc,$5,$6);
  $7 = (_sqlite3StrAccumFinish($acc)|0);
  HEAP32[$z>>2] = $7;
  $8 = HEAP32[$z>>2]|0;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _sqlite3_mprintf($zFormat,$varargs) {
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$1>>2] = $zFormat;
 $2 = (_sqlite3_initialize()|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 } else {
  HEAP32[$ap>>2] = $varargs;
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3_vmprintf($4,$ap)|0);
  HEAP32[$z>>2] = $5;
  $6 = HEAP32[$z>>2]|0;
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 return (0)|0;
}
function _sqlite3_vsnprintf($n,$zBuf,$zFormat,$ap) {
 $n = $n|0;
 $zBuf = $zBuf|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $acc = sp;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$3>>2] = $zFormat;
 HEAP32[$4>>2] = $ap;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$2>>2]|0;
 if ($6) {
  HEAP32[$0>>2] = $7;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  _sqlite3StrAccumInit($acc,0,$7,$8,0);
  $9 = HEAP32[$3>>2]|0;
  $10 = HEAP32[$4>>2]|0;
  _sqlite3VXPrintf($acc,$9,$10);
  $11 = (_sqlite3StrAccumFinish($acc)|0);
  HEAP32[$0>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _sqlite3_snprintf($n,$zBuf,$zFormat,$varargs) {
 $n = $n|0;
 $zBuf = $zBuf|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $z = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $zBuf;
 HEAP32[$2>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3_vsnprintf($3,$4,$5,$ap)|0);
 HEAP32[$z>>2] = $6;
 $7 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_log($iErrCode,$zFormat,$varargs) {
 $iErrCode = $iErrCode|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $iErrCode;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[(264)>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _renderLogMsg($4,$5,$ap);
 STACKTOP = sp;return;
}
function _sqlite3_randomness($N,$pBuf) {
 $N = $N|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $i = 0, $k = 0, $or$cond = 0, $t = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $t = sp + 272|0;
 $zBuf = sp + 4|0;
 $i = sp;
 $k = sp + 16|0;
 HEAP32[$0>>2] = $N;
 HEAP32[$1>>2] = $pBuf;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$zBuf>>2] = $2;
 $3 = (_sqlite3_initialize()|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(0|0);
 $or$cond = $6 | $8;
 if ($or$cond) {
  HEAP8[17879>>0] = 0;
  STACKTOP = sp;return;
 }
 $9 = HEAP8[17879>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if (!($10)) {
  HEAP8[(17881)>>0] = 0;
  HEAP8[(17880)>>0] = 0;
  $11 = (_sqlite3_vfs_find(0)|0);
  (_sqlite3OsRandomness($11,256,$k)|0);
  HEAP32[$i>>2] = 0;
  while(1) {
   $12 = HEAP32[$i>>2]|0;
   $13 = ($12|0)<(256);
   if (!($13)) {
    break;
   }
   $14 = HEAP32[$i>>2]|0;
   $15 = $14&255;
   $16 = HEAP32[$i>>2]|0;
   $17 = ((17882) + ($16)|0);
   HEAP8[$17>>0] = $15;
   $18 = HEAP32[$i>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$i>>2] = $19;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $20 = HEAP32[$i>>2]|0;
   $21 = ($20|0)<(256);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$i>>2]|0;
   $23 = ((17882) + ($22)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = HEAP32[$i>>2]|0;
   $27 = (($k) + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28 << 24 >> 24;
   $30 = (($25) + ($29))|0;
   $31 = HEAP8[(17881)>>0]|0;
   $32 = $31&255;
   $33 = (($32) + ($30))|0;
   $34 = $33&255;
   HEAP8[(17881)>>0] = $34;
   $35 = HEAP8[(17881)>>0]|0;
   $36 = $35&255;
   $37 = ((17882) + ($36)|0);
   $38 = HEAP8[$37>>0]|0;
   HEAP8[$t>>0] = $38;
   $39 = HEAP32[$i>>2]|0;
   $40 = ((17882) + ($39)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = HEAP8[(17881)>>0]|0;
   $43 = $42&255;
   $44 = ((17882) + ($43)|0);
   HEAP8[$44>>0] = $41;
   $45 = HEAP8[$t>>0]|0;
   $46 = HEAP32[$i>>2]|0;
   $47 = ((17882) + ($46)|0);
   HEAP8[$47>>0] = $45;
   $48 = HEAP32[$i>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[$i>>2] = $49;
  }
  HEAP8[17879>>0] = 1;
 }
 while(1) {
  $50 = HEAP8[(17880)>>0]|0;
  $51 = (($50) + 1)<<24>>24;
  HEAP8[(17880)>>0] = $51;
  $52 = HEAP8[(17880)>>0]|0;
  $53 = $52&255;
  $54 = ((17882) + ($53)|0);
  $55 = HEAP8[$54>>0]|0;
  HEAP8[$t>>0] = $55;
  $56 = HEAP8[$t>>0]|0;
  $57 = $56&255;
  $58 = HEAP8[(17881)>>0]|0;
  $59 = $58&255;
  $60 = (($59) + ($57))|0;
  $61 = $60&255;
  HEAP8[(17881)>>0] = $61;
  $62 = HEAP8[(17881)>>0]|0;
  $63 = $62&255;
  $64 = ((17882) + ($63)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = HEAP8[(17880)>>0]|0;
  $67 = $66&255;
  $68 = ((17882) + ($67)|0);
  HEAP8[$68>>0] = $65;
  $69 = HEAP8[$t>>0]|0;
  $70 = HEAP8[(17881)>>0]|0;
  $71 = $70&255;
  $72 = ((17882) + ($71)|0);
  HEAP8[$72>>0] = $69;
  $73 = HEAP8[(17880)>>0]|0;
  $74 = $73&255;
  $75 = ((17882) + ($74)|0);
  $76 = HEAP8[$75>>0]|0;
  $77 = $76&255;
  $78 = HEAP8[$t>>0]|0;
  $79 = $78&255;
  $80 = (($79) + ($77))|0;
  $81 = $80&255;
  HEAP8[$t>>0] = $81;
  $82 = HEAP8[$t>>0]|0;
  $83 = $82&255;
  $84 = ((17882) + ($83)|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = HEAP32[$zBuf>>2]|0;
  $87 = ((($86)) + 1|0);
  HEAP32[$zBuf>>2] = $87;
  HEAP8[$86>>0] = $85;
  $88 = HEAP32[$0>>2]|0;
  $89 = (($88) + -1)|0;
  HEAP32[$0>>2] = $89;
  $90 = ($89|0)!=(0);
  if (!($90)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3_stricmp($zLeft,$zRight) {
 $zLeft = $zLeft|0;
 $zRight = $zRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $zLeft;
 HEAP32[$2>>2] = $zRight;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 do {
  if ($4) {
   $6 = ($5|0)!=(0|0);
   $7 = $6 ? -1 : 0;
   HEAP32[$0>>2] = $7;
  } else {
   $8 = ($5|0)==(0|0);
   if ($8) {
    HEAP32[$0>>2] = 1;
    break;
   } else {
    $9 = HEAP32[$1>>2]|0;
    $10 = HEAP32[$2>>2]|0;
    $11 = (_sqlite3StrICmp($9,$10)|0);
    HEAP32[$0>>2] = $11;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3_os_init() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $i = sp;
 HEAP32[$i>>2] = 0;
 while(1) {
  $0 = HEAP32[$i>>2]|0;
  $1 = ($0>>>0)<(4);
  if (!($1)) {
   break;
  }
  $2 = HEAP32[$i>>2]|0;
  $3 = (1184 + (($2*88)|0)|0);
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)==(0);
  $6 = $5&1;
  (_sqlite3_vfs_register($3,$6)|0);
  $7 = HEAP32[$i>>2]|0;
  $8 = (($7) + 1)|0;
  HEAP32[$i>>2] = $8;
 }
 STACKTOP = sp;return 0;
}
function _sqlite3_backup_step($p,$nPage) {
 $p = $p|0;
 $nPage = $nPage|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bCloseTrans = 0, $destMode = 0, $iEnd = 0, $iOff = 0, $iPg = 0, $iSize = 0, $iSrcPg = 0, $iSrcPg2 = 0, $ii = 0, $nDestTruncate = 0, $nDstPage = 0, $nSrcPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond6 = 0, $pDestPager = 0;
 var $pFile = 0, $pPg = 0, $pSrcPager = 0, $pSrcPg = 0, $pSrcPg1 = 0, $pgszDest = 0, $pgszSrc = 0, $ratio = 0, $rc = 0, $zData = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $rc = sp + 100|0;
 $destMode = sp + 96|0;
 $pgszSrc = sp + 92|0;
 $pgszDest = sp + 88|0;
 $pSrcPager = sp + 84|0;
 $pDestPager = sp + 80|0;
 $ii = sp + 76|0;
 $nSrcPage = sp + 72|0;
 $bCloseTrans = sp + 68|0;
 $iSrcPg = sp + 64|0;
 $pSrcPg = sp + 60|0;
 $nDestTruncate = sp + 56|0;
 $ratio = sp + 52|0;
 $iSize = sp + 16|0;
 $pFile = sp + 48|0;
 $iPg = sp + 44|0;
 $nDstPage = sp + 40|0;
 $iOff = sp + 8|0;
 $iEnd = sp;
 $pPg = sp + 36|0;
 $pSrcPg1 = sp + 32|0;
 $iSrcPg2 = sp + 28|0;
 $zData = sp + 24|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nPage;
 HEAP32[$pgszSrc>>2] = 0;
 HEAP32[$pgszDest>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = (_isFatalError($8)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $422 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($422|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3BtreePager($13)|0);
 HEAP32[$pSrcPager>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3BtreePager($17)|0);
 HEAP32[$pDestPager>>2] = $18;
 HEAP32[$nSrcPage>>2] = -1;
 HEAP32[$bCloseTrans>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0|0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ((($26)) + 20|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(2);
  if ($30) {
   HEAP32[$rc>>2] = 5;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  HEAP32[$rc>>2] = 0;
 }
 $31 = HEAP32[$rc>>2]|0;
 $32 = ($31|0)==(0);
 if ($32) {
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 24|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (_sqlite3BtreeIsInReadTrans($35)|0);
  $37 = (0)==($36|0);
  if ($37) {
   $38 = HEAP32[$0>>2]|0;
   $39 = ((($38)) + 24|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (_sqlite3BtreeBeginTrans($40,0)|0);
   HEAP32[$rc>>2] = $41;
   HEAP32[$bCloseTrans>>2] = 1;
  }
 }
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 12|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)==(0);
 $46 = HEAP32[$rc>>2]|0;
 $47 = ($46|0)==(0);
 $or$cond = $45 & $47;
 if ($or$cond) {
  $48 = HEAP32[$0>>2]|0;
  $49 = (_setDestPgsz($48)|0);
  $50 = ($49|0)==(7);
  if ($50) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $51 = HEAP32[$rc>>2]|0;
 $52 = (0)==($51|0);
 if ($52) {
  $53 = HEAP32[$0>>2]|0;
  $54 = ((($53)) + 12|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)==(0);
  if ($56) {
   $57 = HEAP32[$0>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3BtreeBeginTrans($59,2)|0);
   HEAP32[$rc>>2] = $60;
   $61 = (0)==($60|0);
   if ($61) {
    $62 = HEAP32[$0>>2]|0;
    $63 = ((($62)) + 12|0);
    HEAP32[$63>>2] = 1;
    $64 = HEAP32[$0>>2]|0;
    $65 = ((($64)) + 4|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = HEAP32[$0>>2]|0;
    $68 = ((($67)) + 8|0);
    _sqlite3BtreeGetMeta($66,1,$68);
   }
  }
 }
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 24|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (_sqlite3BtreeGetPageSize($71)|0);
 HEAP32[$pgszSrc>>2] = $72;
 $73 = HEAP32[$0>>2]|0;
 $74 = ((($73)) + 4|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = (_sqlite3BtreeGetPageSize($75)|0);
 HEAP32[$pgszDest>>2] = $76;
 $77 = HEAP32[$0>>2]|0;
 $78 = ((($77)) + 4|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = (_sqlite3BtreePager($79)|0);
 $81 = (_sqlite3PagerGetJournalMode($80)|0);
 HEAP32[$destMode>>2] = $81;
 $82 = HEAP32[$rc>>2]|0;
 $83 = (0)==($82|0);
 $84 = HEAP32[$destMode>>2]|0;
 $85 = ($84|0)==(5);
 $or$cond3 = $83 & $85;
 if ($or$cond3) {
  $86 = HEAP32[$pgszSrc>>2]|0;
  $87 = HEAP32[$pgszDest>>2]|0;
  $88 = ($86|0)!=($87|0);
  if ($88) {
   HEAP32[$rc>>2] = 8;
  }
 }
 $89 = HEAP32[$0>>2]|0;
 $90 = ((($89)) + 24|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = (_sqlite3BtreeLastPage($91)|0);
 HEAP32[$nSrcPage>>2] = $92;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $93 = HEAP32[$1>>2]|0;
  $94 = ($93|0)<(0);
  if (!($94)) {
   $95 = HEAP32[$ii>>2]|0;
   $96 = HEAP32[$1>>2]|0;
   $97 = ($95|0)<($96|0);
   if (!($97)) {
    break;
   }
  }
  $98 = HEAP32[$0>>2]|0;
  $99 = ((($98)) + 16|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = HEAP32[$nSrcPage>>2]|0;
  $102 = ($100>>>0)<=($101>>>0);
  if (!($102)) {
   break;
  }
  $103 = HEAP32[$rc>>2]|0;
  $104 = ($103|0)!=(0);
  $105 = $104 ^ 1;
  if (!($105)) {
   break;
  }
  $106 = HEAP32[$0>>2]|0;
  $107 = ((($106)) + 16|0);
  $108 = HEAP32[$107>>2]|0;
  HEAP32[$iSrcPg>>2] = $108;
  $109 = HEAP32[$iSrcPg>>2]|0;
  $110 = HEAP32[1536>>2]|0;
  $111 = HEAP32[$0>>2]|0;
  $112 = ((($111)) + 24|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ((($113)) + 4|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = ((($115)) + 32|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = (($110>>>0) / ($117>>>0))&-1;
  $119 = (($118) + 1)|0;
  $120 = ($109|0)!=($119|0);
  if ($120) {
   $121 = HEAP32[$pSrcPager>>2]|0;
   $122 = HEAP32[$iSrcPg>>2]|0;
   $123 = (_sqlite3PagerGet($121,$122,$pSrcPg,2)|0);
   HEAP32[$rc>>2] = $123;
   $124 = HEAP32[$rc>>2]|0;
   $125 = ($124|0)==(0);
   if ($125) {
    $126 = HEAP32[$0>>2]|0;
    $127 = HEAP32[$iSrcPg>>2]|0;
    $128 = HEAP32[$pSrcPg>>2]|0;
    $129 = (_sqlite3PagerGetData($128)|0);
    $130 = (_backupOnePage($126,$127,$129,0)|0);
    HEAP32[$rc>>2] = $130;
    $131 = HEAP32[$pSrcPg>>2]|0;
    _sqlite3PagerUnref($131);
   }
  }
  $132 = HEAP32[$0>>2]|0;
  $133 = ((($132)) + 16|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = (($134) + 1)|0;
  HEAP32[$133>>2] = $135;
  $136 = HEAP32[$ii>>2]|0;
  $137 = (($136) + 1)|0;
  HEAP32[$ii>>2] = $137;
 }
 $138 = HEAP32[$rc>>2]|0;
 $139 = ($138|0)==(0);
 do {
  if ($139) {
   $140 = HEAP32[$nSrcPage>>2]|0;
   $141 = HEAP32[$0>>2]|0;
   $142 = ((($141)) + 36|0);
   HEAP32[$142>>2] = $140;
   $143 = HEAP32[$nSrcPage>>2]|0;
   $144 = (($143) + 1)|0;
   $145 = HEAP32[$0>>2]|0;
   $146 = ((($145)) + 16|0);
   $147 = HEAP32[$146>>2]|0;
   $148 = (($144) - ($147))|0;
   $149 = HEAP32[$0>>2]|0;
   $150 = ((($149)) + 32|0);
   HEAP32[$150>>2] = $148;
   $151 = HEAP32[$0>>2]|0;
   $152 = ((($151)) + 16|0);
   $153 = HEAP32[$152>>2]|0;
   $154 = HEAP32[$nSrcPage>>2]|0;
   $155 = ($153>>>0)>($154>>>0);
   if ($155) {
    HEAP32[$rc>>2] = 101;
    break;
   }
   $156 = HEAP32[$0>>2]|0;
   $157 = ((($156)) + 40|0);
   $158 = HEAP32[$157>>2]|0;
   $159 = ($158|0)!=(0);
   if (!($159)) {
    $160 = HEAP32[$0>>2]|0;
    _attachBackupObject($160);
   }
  }
 } while(0);
 $161 = HEAP32[$rc>>2]|0;
 $162 = ($161|0)==(101);
 if ($162) {
  $163 = HEAP32[$nSrcPage>>2]|0;
  $164 = ($163|0)==(0);
  if ($164) {
   $165 = HEAP32[$0>>2]|0;
   $166 = ((($165)) + 4|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = (_sqlite3BtreeNewDb($167)|0);
   HEAP32[$rc>>2] = $168;
   HEAP32[$nSrcPage>>2] = 1;
  }
  $169 = HEAP32[$rc>>2]|0;
  $170 = ($169|0)==(0);
  $171 = HEAP32[$rc>>2]|0;
  $172 = ($171|0)==(101);
  $or$cond6 = $170 | $172;
  if ($or$cond6) {
   $173 = HEAP32[$0>>2]|0;
   $174 = ((($173)) + 4|0);
   $175 = HEAP32[$174>>2]|0;
   $176 = HEAP32[$0>>2]|0;
   $177 = ((($176)) + 8|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = (($178) + 1)|0;
   $180 = (_sqlite3BtreeUpdateMeta($175,1,$179)|0);
   HEAP32[$rc>>2] = $180;
  }
  $181 = HEAP32[$rc>>2]|0;
  $182 = ($181|0)==(0);
  if ($182) {
   $183 = HEAP32[$0>>2]|0;
   $184 = HEAP32[$183>>2]|0;
   $185 = ($184|0)!=(0|0);
   if ($185) {
    $186 = HEAP32[$0>>2]|0;
    $187 = HEAP32[$186>>2]|0;
    _sqlite3ResetAllSchemasOfConnection($187);
   }
   $188 = HEAP32[$destMode>>2]|0;
   $189 = ($188|0)==(5);
   if ($189) {
    $190 = HEAP32[$0>>2]|0;
    $191 = ((($190)) + 4|0);
    $192 = HEAP32[$191>>2]|0;
    $193 = (_sqlite3BtreeSetVersion($192,2)|0);
    HEAP32[$rc>>2] = $193;
   }
  }
  $194 = HEAP32[$rc>>2]|0;
  $195 = ($194|0)==(0);
  if ($195) {
   $196 = HEAP32[$pgszSrc>>2]|0;
   $197 = HEAP32[$pgszDest>>2]|0;
   $198 = ($196|0)<($197|0);
   if ($198) {
    $199 = HEAP32[$pgszDest>>2]|0;
    $200 = HEAP32[$pgszSrc>>2]|0;
    $201 = (($199|0) / ($200|0))&-1;
    HEAP32[$ratio>>2] = $201;
    $202 = HEAP32[$nSrcPage>>2]|0;
    $203 = HEAP32[$ratio>>2]|0;
    $204 = (($202) + ($203))|0;
    $205 = (($204) - 1)|0;
    $206 = HEAP32[$ratio>>2]|0;
    $207 = (($205|0) / ($206|0))&-1;
    HEAP32[$nDestTruncate>>2] = $207;
    $208 = HEAP32[$nDestTruncate>>2]|0;
    $209 = HEAP32[1536>>2]|0;
    $210 = HEAP32[$0>>2]|0;
    $211 = ((($210)) + 4|0);
    $212 = HEAP32[$211>>2]|0;
    $213 = ((($212)) + 4|0);
    $214 = HEAP32[$213>>2]|0;
    $215 = ((($214)) + 32|0);
    $216 = HEAP32[$215>>2]|0;
    $217 = (($209>>>0) / ($216>>>0))&-1;
    $218 = (($217) + 1)|0;
    $219 = ($208|0)==($218|0);
    if ($219) {
     $220 = HEAP32[$nDestTruncate>>2]|0;
     $221 = (($220) + -1)|0;
     HEAP32[$nDestTruncate>>2] = $221;
    }
   } else {
    $222 = HEAP32[$nSrcPage>>2]|0;
    $223 = HEAP32[$pgszSrc>>2]|0;
    $224 = HEAP32[$pgszDest>>2]|0;
    $225 = (($223|0) / ($224|0))&-1;
    $226 = Math_imul($222, $225)|0;
    HEAP32[$nDestTruncate>>2] = $226;
   }
   $227 = HEAP32[$pgszSrc>>2]|0;
   $228 = HEAP32[$pgszDest>>2]|0;
   $229 = ($227|0)<($228|0);
   if ($229) {
    $230 = HEAP32[$pgszSrc>>2]|0;
    $231 = ($230|0)<(0);
    $232 = $231 << 31 >> 31;
    $233 = HEAP32[$nSrcPage>>2]|0;
    $234 = ($233|0)<(0);
    $235 = $234 << 31 >> 31;
    $236 = (___muldi3(($230|0),($232|0),($233|0),($235|0))|0);
    $237 = tempRet0;
    $238 = $iSize;
    $239 = $238;
    HEAP32[$239>>2] = $236;
    $240 = (($238) + 4)|0;
    $241 = $240;
    HEAP32[$241>>2] = $237;
    $242 = HEAP32[$pDestPager>>2]|0;
    $243 = (_sqlite3PagerFile($242)|0);
    HEAP32[$pFile>>2] = $243;
    $244 = HEAP32[$pDestPager>>2]|0;
    _sqlite3PagerPagecount($244,$nDstPage);
    $245 = HEAP32[$nDestTruncate>>2]|0;
    HEAP32[$iPg>>2] = $245;
    while(1) {
     $246 = HEAP32[$rc>>2]|0;
     $247 = ($246|0)==(0);
     if (!($247)) {
      break;
     }
     $248 = HEAP32[$iPg>>2]|0;
     $249 = HEAP32[$nDstPage>>2]|0;
     $250 = ($248>>>0)<=($249>>>0);
     if (!($250)) {
      break;
     }
     $251 = HEAP32[$iPg>>2]|0;
     $252 = HEAP32[1536>>2]|0;
     $253 = HEAP32[$0>>2]|0;
     $254 = ((($253)) + 4|0);
     $255 = HEAP32[$254>>2]|0;
     $256 = ((($255)) + 4|0);
     $257 = HEAP32[$256>>2]|0;
     $258 = ((($257)) + 32|0);
     $259 = HEAP32[$258>>2]|0;
     $260 = (($252>>>0) / ($259>>>0))&-1;
     $261 = (($260) + 1)|0;
     $262 = ($251|0)!=($261|0);
     do {
      if ($262) {
       $263 = HEAP32[$pDestPager>>2]|0;
       $264 = HEAP32[$iPg>>2]|0;
       $265 = (_sqlite3PagerGet($263,$264,$pPg,0)|0);
       HEAP32[$rc>>2] = $265;
       $266 = HEAP32[$rc>>2]|0;
       $267 = ($266|0)==(0);
       if (!($267)) {
        break;
       }
       $268 = HEAP32[$pPg>>2]|0;
       $269 = (_sqlite3PagerWrite($268)|0);
       HEAP32[$rc>>2] = $269;
       $270 = HEAP32[$pPg>>2]|0;
       _sqlite3PagerUnref($270);
      }
     } while(0);
     $271 = HEAP32[$iPg>>2]|0;
     $272 = (($271) + 1)|0;
     HEAP32[$iPg>>2] = $272;
    }
    $273 = HEAP32[$rc>>2]|0;
    $274 = ($273|0)==(0);
    if ($274) {
     $275 = HEAP32[$pDestPager>>2]|0;
     $276 = (_sqlite3PagerCommitPhaseOne($275,0,1)|0);
     HEAP32[$rc>>2] = $276;
    }
    $277 = HEAP32[1536>>2]|0;
    $278 = HEAP32[$pgszDest>>2]|0;
    $279 = (($277) + ($278))|0;
    $280 = ($279|0)<(0);
    $281 = $280 << 31 >> 31;
    $282 = $iSize;
    $283 = $282;
    $284 = HEAP32[$283>>2]|0;
    $285 = (($282) + 4)|0;
    $286 = $285;
    $287 = HEAP32[$286>>2]|0;
    $288 = ($281|0)<($287|0);
    $289 = ($279>>>0)<($284>>>0);
    $290 = ($281|0)==($287|0);
    $291 = $290 & $289;
    $292 = $288 | $291;
    if ($292) {
     $293 = HEAP32[1536>>2]|0;
     $294 = HEAP32[$pgszDest>>2]|0;
     $295 = (($293) + ($294))|0;
     $296 = ($295|0)<(0);
     $297 = $296 << 31 >> 31;
     $306 = $295;$309 = $297;
    } else {
     $298 = $iSize;
     $299 = $298;
     $300 = HEAP32[$299>>2]|0;
     $301 = (($298) + 4)|0;
     $302 = $301;
     $303 = HEAP32[$302>>2]|0;
     $306 = $300;$309 = $303;
    }
    $304 = $iEnd;
    $305 = $304;
    HEAP32[$305>>2] = $306;
    $307 = (($304) + 4)|0;
    $308 = $307;
    HEAP32[$308>>2] = $309;
    $310 = HEAP32[1536>>2]|0;
    $311 = HEAP32[$pgszSrc>>2]|0;
    $312 = (($310) + ($311))|0;
    $313 = ($312|0)<(0);
    $314 = $313 << 31 >> 31;
    $315 = $iOff;
    $316 = $315;
    HEAP32[$316>>2] = $312;
    $317 = (($315) + 4)|0;
    $318 = $317;
    HEAP32[$318>>2] = $314;
    while(1) {
     $319 = HEAP32[$rc>>2]|0;
     $320 = ($319|0)==(0);
     if (!($320)) {
      break;
     }
     $321 = $iOff;
     $322 = $321;
     $323 = HEAP32[$322>>2]|0;
     $324 = (($321) + 4)|0;
     $325 = $324;
     $326 = HEAP32[$325>>2]|0;
     $327 = $iEnd;
     $328 = $327;
     $329 = HEAP32[$328>>2]|0;
     $330 = (($327) + 4)|0;
     $331 = $330;
     $332 = HEAP32[$331>>2]|0;
     $333 = ($326|0)<($332|0);
     $334 = ($323>>>0)<($329>>>0);
     $335 = ($326|0)==($332|0);
     $336 = $335 & $334;
     $337 = $333 | $336;
     if (!($337)) {
      break;
     }
     HEAP32[$pSrcPg1>>2] = 0;
     $338 = $iOff;
     $339 = $338;
     $340 = HEAP32[$339>>2]|0;
     $341 = (($338) + 4)|0;
     $342 = $341;
     $343 = HEAP32[$342>>2]|0;
     $344 = HEAP32[$pgszSrc>>2]|0;
     $345 = ($344|0)<(0);
     $346 = $345 << 31 >> 31;
     $347 = (___divdi3(($340|0),($343|0),($344|0),($346|0))|0);
     $348 = tempRet0;
     $349 = (_i64Add(($347|0),($348|0),1,0)|0);
     $350 = tempRet0;
     HEAP32[$iSrcPg2>>2] = $349;
     $351 = HEAP32[$pSrcPager>>2]|0;
     $352 = HEAP32[$iSrcPg2>>2]|0;
     $353 = (_sqlite3PagerGet($351,$352,$pSrcPg1,0)|0);
     HEAP32[$rc>>2] = $353;
     $354 = HEAP32[$rc>>2]|0;
     $355 = ($354|0)==(0);
     if ($355) {
      $356 = HEAP32[$pSrcPg1>>2]|0;
      $357 = (_sqlite3PagerGetData($356)|0);
      HEAP32[$zData>>2] = $357;
      $358 = HEAP32[$pFile>>2]|0;
      $359 = HEAP32[$zData>>2]|0;
      $360 = HEAP32[$pgszSrc>>2]|0;
      $361 = $iOff;
      $362 = $361;
      $363 = HEAP32[$362>>2]|0;
      $364 = (($361) + 4)|0;
      $365 = $364;
      $366 = HEAP32[$365>>2]|0;
      $367 = (_sqlite3OsWrite($358,$359,$360,$363,$366)|0);
      HEAP32[$rc>>2] = $367;
     }
     $368 = HEAP32[$pSrcPg1>>2]|0;
     _sqlite3PagerUnref($368);
     $369 = HEAP32[$pgszSrc>>2]|0;
     $370 = ($369|0)<(0);
     $371 = $370 << 31 >> 31;
     $372 = $iOff;
     $373 = $372;
     $374 = HEAP32[$373>>2]|0;
     $375 = (($372) + 4)|0;
     $376 = $375;
     $377 = HEAP32[$376>>2]|0;
     $378 = (_i64Add(($374|0),($377|0),($369|0),($371|0))|0);
     $379 = tempRet0;
     $380 = $iOff;
     $381 = $380;
     HEAP32[$381>>2] = $378;
     $382 = (($380) + 4)|0;
     $383 = $382;
     HEAP32[$383>>2] = $379;
    }
    $384 = HEAP32[$rc>>2]|0;
    $385 = ($384|0)==(0);
    if ($385) {
     $386 = HEAP32[$pFile>>2]|0;
     $387 = $iSize;
     $388 = $387;
     $389 = HEAP32[$388>>2]|0;
     $390 = (($387) + 4)|0;
     $391 = $390;
     $392 = HEAP32[$391>>2]|0;
     $393 = (_backupTruncateFile($386,$389,$392)|0);
     HEAP32[$rc>>2] = $393;
    }
    $394 = HEAP32[$rc>>2]|0;
    $395 = ($394|0)==(0);
    if ($395) {
     $396 = HEAP32[$pDestPager>>2]|0;
     $397 = (_sqlite3PagerSync($396,0)|0);
     HEAP32[$rc>>2] = $397;
    }
   } else {
    $398 = HEAP32[$pDestPager>>2]|0;
    $399 = HEAP32[$nDestTruncate>>2]|0;
    _sqlite3PagerTruncateImage($398,$399);
    $400 = HEAP32[$pDestPager>>2]|0;
    $401 = (_sqlite3PagerCommitPhaseOne($400,0,0)|0);
    HEAP32[$rc>>2] = $401;
   }
   $402 = HEAP32[$rc>>2]|0;
   $403 = (0)==($402|0);
   if ($403) {
    $404 = HEAP32[$0>>2]|0;
    $405 = ((($404)) + 4|0);
    $406 = HEAP32[$405>>2]|0;
    $407 = (_sqlite3BtreeCommitPhaseTwo($406,0)|0);
    HEAP32[$rc>>2] = $407;
    $408 = (0)==($407|0);
    $$ = $408 ? 101 : $407;
    HEAP32[$rc>>2] = $$;
   }
  }
 }
 $409 = HEAP32[$bCloseTrans>>2]|0;
 $410 = ($409|0)!=(0);
 if ($410) {
  $411 = HEAP32[$0>>2]|0;
  $412 = ((($411)) + 24|0);
  $413 = HEAP32[$412>>2]|0;
  (_sqlite3BtreeCommitPhaseOne($413,0)|0);
  $414 = HEAP32[$0>>2]|0;
  $415 = ((($414)) + 24|0);
  $416 = HEAP32[$415>>2]|0;
  (_sqlite3BtreeCommitPhaseTwo($416,0)|0);
 }
 $417 = HEAP32[$rc>>2]|0;
 $418 = ($417|0)==(3082);
 if ($418) {
  HEAP32[$rc>>2] = 7;
 }
 $419 = HEAP32[$rc>>2]|0;
 $420 = HEAP32[$0>>2]|0;
 $421 = ((($420)) + 28|0);
 HEAP32[$421>>2] = $419;
 $422 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($422|0);
}
function _sqlite3_backup_finish($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pSrcDb = 0, $pp = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pp = sp + 8|0;
 $pSrcDb = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pSrcDb>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 24|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + -1)|0;
  HEAP32[$16>>2] = $18;
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 40|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 24|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3BtreePager($25)|0);
  $27 = (_sqlite3PagerBackupPtr($26)|0);
  HEAP32[$pp>>2] = $27;
  while(1) {
   $28 = HEAP32[$pp>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = ($29|0)!=($30|0);
   if (!($31)) {
    break;
   }
   $32 = HEAP32[$pp>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = ((($33)) + 44|0);
   HEAP32[$pp>>2] = $34;
  }
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 44|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$pp>>2]|0;
  HEAP32[$38>>2] = $37;
 }
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 (_sqlite3BtreeRollback($41,0,0)|0);
 $42 = HEAP32[$1>>2]|0;
 $43 = ((($42)) + 28|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)==(101);
 if ($45) {
  $49 = 0;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 28|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = $48;
 }
 HEAP32[$rc>>2] = $49;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0|0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$rc>>2]|0;
  _sqlite3Error($54,$55);
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  _sqlite3LeaveMutexAndCloseZombie($57);
 }
 $58 = HEAP32[$1>>2]|0;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)!=(0|0);
 if ($60) {
  $61 = HEAP32[$1>>2]|0;
  _sqlite3_free($61);
 }
 $62 = HEAP32[$pSrcDb>>2]|0;
 _sqlite3LeaveMutexAndCloseZombie($62);
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _sqlite3_finalize($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $v = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pStmt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$rc>>2] = 0;
 } else {
  $4 = HEAP32[$1>>2]|0;
  HEAP32[$v>>2] = $4;
  $5 = HEAP32[$v>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$db>>2] = $6;
  $7 = HEAP32[$v>>2]|0;
  $8 = (_vdbeSafety($7)|0);
  $9 = ($8|0)!=(0);
  if ($9) {
   $10 = (_sqlite3MisuseError(75223)|0);
   HEAP32[$0>>2] = $10;
   $33 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($33|0);
  }
  $11 = HEAP32[$v>>2]|0;
  $12 = ((($11)) + 128|0);
  $13 = $12;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)>(0);
  $20 = ($15>>>0)>(0);
  $21 = ($18|0)==(0);
  $22 = $21 & $20;
  $23 = $19 | $22;
  if ($23) {
   $24 = HEAP32[$db>>2]|0;
   $25 = HEAP32[$v>>2]|0;
   _invokeProfileCallback($24,$25);
  }
  $26 = HEAP32[$v>>2]|0;
  $27 = (_sqlite3VdbeFinalize($26)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$db>>2]|0;
  $29 = HEAP32[$rc>>2]|0;
  $30 = (_sqlite3ApiExit($28,$29)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$db>>2]|0;
  _sqlite3LeaveMutexAndCloseZombie($31);
 }
 $32 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $32;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3_reset($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $v = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  HEAP32[$rc>>2] = 0;
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ((($6)) + 128|0);
 $8 = $7;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)>(0);
 $15 = ($10>>>0)>(0);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 if ($18) {
  $19 = HEAP32[$db>>2]|0;
  $20 = HEAP32[$v>>2]|0;
  _invokeProfileCallback($19,$20);
 }
 $21 = HEAP32[$v>>2]|0;
 $22 = (_sqlite3VdbeReset($21)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$v>>2]|0;
 _sqlite3VdbeRewind($23);
 $24 = HEAP32[$db>>2]|0;
 $25 = HEAP32[$rc>>2]|0;
 $26 = (_sqlite3ApiExit($24,$25)|0);
 HEAP32[$rc>>2] = $26;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3_clear_bindings($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$p>>2]|0;
  $4 = ((($3)) + 16|0);
  $5 = HEAP16[$4>>1]|0;
  $6 = $5 << 16 >> 16;
  $7 = ($2|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 116|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*40)|0)|0);
  _sqlite3VdbeMemRelease($12);
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 116|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*40)|0)|0);
  $18 = ((($17)) + 8|0);
  HEAP16[$18>>1] = 1;
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 144|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = ($23&65535) >>> 9;
 $25 = $24 & 1;
 $26 = $25&65535;
 $27 = ($26|0)!=(0);
 if (!($27)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $28 = HEAP32[$p>>2]|0;
 $29 = ((($28)) + 196|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)!=(0);
 if (!($31)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 144|0);
 $34 = HEAP16[$33>>1]|0;
 $35 = $34 & -2;
 $36 = $35 | 1;
 HEAP16[$33>>1] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3_value_blob($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pVal;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 18;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $33 = HEAP32[$1>>2]|0;
  $34 = (_sqlite3_value_text($33)|0);
  HEAP32[$0>>2] = $34;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & 16384;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$p>>2]|0;
  $16 = (_sqlite3VdbeMemExpandBlob($15)|0);
  $18 = $16;
 } else {
  $18 = 0;
 }
 $17 = ($18|0)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 8|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 $23 = $22 | 16;
 $24 = $23&65535;
 HEAP16[$20>>1] = $24;
 $25 = HEAP32[$p>>2]|0;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $31;
 } else {
  $32 = 0;
 }
 HEAP32[$0>>2] = $32;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3_value_text($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3ValueText($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_value_bytes($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3ValueBytes($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_value_double($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (+_sqlite3VdbeRealValue($1));
 STACKTOP = sp;return (+$2);
}
function _sqlite3_value_int($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_value_int64($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 tempRet0 = ($3);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_value_type($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 31;
 $6 = (18138 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_result_blob($pCtx,$z,$n,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $xDel;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 _setResultStrOrError($4,$5,$6,0,$7);
 STACKTOP = sp;return;
}
function _sqlite3_result_blob64($pCtx,$z,$0,$1,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 $xDel = $xDel|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $z;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $xDel;
 $10 = $4;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15>>>0)>(0);
 $17 = ($12>>>0)>(2147483647);
 $18 = ($15|0)==(0);
 $19 = $18 & $17;
 $20 = $16 | $19;
 if ($20) {
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$5>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  (_invokeValueDestructor($21,$22,$23)|0);
  STACKTOP = sp;return;
 } else {
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = $4;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$5>>2]|0;
  _setResultStrOrError($24,$25,$28,0,$32);
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_double($pCtx,$rVal) {
 $pCtx = $pCtx|0;
 $rVal = +$rVal;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAPF64[$1>>3] = $rVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = +HEAPF64[$1>>3];
 _sqlite3VdbeMemSetDouble($3,$4);
 STACKTOP = sp;return;
}
function _sqlite3_result_error($pCtx,$z,$n) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = 1;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeMemSetStr($8,$9,$10,1,(-1))|0);
 STACKTOP = sp;return;
}
function _sqlite3_result_int($pCtx,$iVal) {
 $pCtx = $pCtx|0;
 $iVal = $iVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $iVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(0);
 $6 = $5 << 31 >> 31;
 _sqlite3VdbeMemSetInt64($3,$4,$6);
 STACKTOP = sp;return;
}
function _sqlite3_result_int64($pCtx,$0,$1) {
 $pCtx = $pCtx|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pCtx;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 _sqlite3VdbeMemSetInt64($9,$12,$15);
 STACKTOP = sp;return;
}
function _sqlite3_result_null($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 STACKTOP = sp;return;
}
function _sqlite3_result_text($pCtx,$z,$n,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $xDel;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 _setResultStrOrError($4,$5,$6,1,$7);
 STACKTOP = sp;return;
}
function _sqlite3_result_text64($pCtx,$z,$0,$1,$xDel,$enc) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 $xDel = $xDel|0;
 $enc = $enc|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 8|0;
 $6 = sp + 20|0;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $z;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $xDel;
 HEAP8[$6>>0] = $enc;
 $11 = HEAP8[$6>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(4);
 if ($13) {
  $14 = HEAP8[968>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = ($15|0)==(0);
  $17 = $16 ? 3 : 2;
  $18 = $17&255;
  HEAP8[$6>>0] = $18;
 }
 $19 = $4;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24>>>0)>(0);
 $26 = ($21>>>0)>(2147483647);
 $27 = ($24|0)==(0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  (_invokeValueDestructor($30,$31,$32)|0);
  STACKTOP = sp;return;
 } else {
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = $4;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP8[$6>>0]|0;
  $42 = HEAP32[$5>>2]|0;
  _setResultStrOrError($33,$34,$37,$41,$42);
  STACKTOP = sp;return;
 }
}
function _sqlite3_result_value($pCtx,$pValue) {
 $pCtx = $pCtx|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pValue;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeMemCopy($3,$4)|0);
 STACKTOP = sp;return;
}
function _sqlite3_result_zeroblob64($pCtx,$0,$1) {
 $pCtx = $pCtx|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pOut = sp + 8|0;
 HEAP32[$3>>2] = $pCtx;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pOut>>2] = $10;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$pOut>>2]|0;
 $18 = ((($17)) + 32|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 96|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)<(0);
 $23 = $22 << 31 >> 31;
 $24 = ($16>>>0)>($23>>>0);
 $25 = ($13>>>0)>($21>>>0);
 $26 = ($16|0)==($23|0);
 $27 = $26 & $25;
 $28 = $24 | $27;
 if ($28) {
  HEAP32[$2>>2] = 18;
  $37 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($37|0);
 } else {
  $29 = HEAP32[$3>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  $31 = $4;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  _sqlite3VdbeMemSetZeroBlob($30,$33);
  HEAP32[$2>>2] = 0;
  $37 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return (0)|0;
}
function _sqlite3_result_error_code($pCtx,$errCode) {
 $pCtx = $pCtx|0;
 $errCode = $errCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $errCode;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 1;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = (_sqlite3ErrStr($16)|0);
 (_sqlite3VdbeMemSetStr($15,$17,-1,1,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3_result_error_toobig($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 HEAP32[$2>>2] = 18;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 25|0);
 HEAP8[$4>>0] = 1;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 (_sqlite3VdbeMemSetStr($6,18170,-1,1,0)|0);
 STACKTOP = sp;return;
}
function _sqlite3_result_error_nomem($pCtx) {
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCtx;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = 7;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 25|0);
 HEAP8[$6>>0] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 _sqlite3OomFault($10);
 STACKTOP = sp;return;
}
function _sqlite3_step($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $db = 0, $rc = 0, $rc2 = 0, $savedPc = 0, $v = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $rc2 = sp + 20|0;
 $v = sp + 16|0;
 $cnt = sp + 12|0;
 $db = sp + 8|0;
 $savedPc = sp + 4|0;
 $zErr = sp;
 HEAP32[$1>>2] = $pStmt;
 HEAP32[$rc>>2] = 0;
 HEAP32[$rc2>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$v>>2] = $2;
 HEAP32[$cnt>>2] = 0;
 $3 = HEAP32[$v>>2]|0;
 $4 = (_vdbeSafetyNotNull($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = (_sqlite3MisuseError(75775)|0);
  HEAP32[$0>>2] = $6;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ((($9)) + 144|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11 & -3;
 HEAP16[$10>>1] = $12;
 while(1) {
  $13 = HEAP32[$v>>2]|0;
  $14 = (_sqlite3Step($13)|0);
  HEAP32[$rc>>2] = $14;
  $15 = ($14|0)==(17);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$cnt>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$cnt>>2] = $17;
  $18 = ($16|0)<(50);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$v>>2]|0;
  $20 = ((($19)) + 36|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$savedPc>>2] = $21;
  $22 = HEAP32[$v>>2]|0;
  $23 = (_sqlite3Reprepare($22)|0);
  HEAP32[$rc>>2] = $23;
  HEAP32[$rc2>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)!=(0);
  if ($25) {
   break;
  }
  $26 = HEAP32[$1>>2]|0;
  (_sqlite3_reset($26)|0);
  $27 = HEAP32[$savedPc>>2]|0;
  $28 = ($27|0)>=(0);
  if (!($28)) {
   continue;
  }
  $29 = HEAP32[$v>>2]|0;
  $30 = ((($29)) + 144|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $31 & -3;
  $33 = $32 | 2;
  HEAP16[$30>>1] = $33;
 }
 $34 = HEAP32[$rc2>>2]|0;
 $35 = ($34|0)!=(0);
 do {
  if ($35) {
   $36 = HEAP32[$db>>2]|0;
   $37 = ((($36)) + 244|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3_value_text($38)|0);
   HEAP32[$zErr>>2] = $39;
   $40 = HEAP32[$db>>2]|0;
   $41 = HEAP32[$v>>2]|0;
   $42 = ((($41)) + 108|0);
   $43 = HEAP32[$42>>2]|0;
   _sqlite3DbFree($40,$43);
   $44 = HEAP32[$db>>2]|0;
   $45 = ((($44)) + 69|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = ($46<<24>>24)!=(0);
   if ($47) {
    $56 = HEAP32[$v>>2]|0;
    $57 = ((($56)) + 108|0);
    HEAP32[$57>>2] = 0;
    HEAP32[$rc>>2] = 7;
    $58 = HEAP32[$v>>2]|0;
    $59 = ((($58)) + 40|0);
    HEAP32[$59>>2] = 7;
    break;
   } else {
    $48 = HEAP32[$db>>2]|0;
    $49 = HEAP32[$zErr>>2]|0;
    $50 = (_sqlite3DbStrDup($48,$49)|0);
    $51 = HEAP32[$v>>2]|0;
    $52 = ((($51)) + 108|0);
    HEAP32[$52>>2] = $50;
    $53 = HEAP32[$rc2>>2]|0;
    $54 = HEAP32[$v>>2]|0;
    $55 = ((($54)) + 40|0);
    HEAP32[$55>>2] = $53;
    break;
   }
  }
 } while(0);
 $60 = HEAP32[$db>>2]|0;
 $61 = HEAP32[$rc>>2]|0;
 $62 = (_sqlite3ApiExit($60,$61)|0);
 HEAP32[$rc>>2] = $62;
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _sqlite3_user_data($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3_context_db_handle($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 32|0);
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3_aggregate_context($p,$nByte) {
 $p = $p|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nByte;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 8192;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = HEAP32[$2>>2]|0;
  $13 = (_createAggContext($11,$12)|0);
  HEAP32[$0>>2] = $13;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $14 = ((($11)) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _sqlite3_column_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pVm = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pVm>>2] = $1;
 $2 = HEAP32[$pVm>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $8 = 0;
  STACKTOP = sp;return ($8|0);
 }
 $4 = HEAP32[$pVm>>2]|0;
 $5 = ((($4)) + 140|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_data_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pVm = sp;
 HEAP32[$1>>2] = $pStmt;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pVm>>2] = $2;
 $3 = HEAP32[$pVm>>2]|0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = HEAP32[$pVm>>2]|0;
  $6 = ((($5)) + 104|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(0|0);
  if (!($8)) {
   $9 = HEAP32[$pVm>>2]|0;
   $10 = ((($9)) + 140|0);
   $11 = HEAP16[$10>>1]|0;
   $12 = $11&65535;
   HEAP32[$0>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($13|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3_column_blob($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_bytes($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_bytes($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_double($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0.0, $6 = 0, $7 = 0.0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (+_sqlite3_value_double($4));
 HEAPF64[$val>>3] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = +HEAPF64[$val>>3];
 STACKTOP = sp;return (+$7);
}
function _sqlite3_column_int($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_int64($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $val = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_int64($4)|0);
 $6 = tempRet0;
 $7 = $val;
 $8 = $7;
 HEAP32[$8>>2] = $5;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[$0>>2]|0;
 _columnMallocFailure($11);
 $12 = $val;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 tempRet0 = ($17);
 STACKTOP = sp;return ($14|0);
}
function _sqlite3_column_text($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $val = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$val>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$val>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_value($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pOut = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 HEAP32[$pOut>>2] = $4;
 $5 = HEAP32[$pOut>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 2048;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $23 = HEAP32[$0>>2]|0;
  _columnMallocFailure($23);
  $24 = HEAP32[$pOut>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $11 = HEAP32[$pOut>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -2049;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$pOut>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 | 4096;
 $22 = $21&65535;
 HEAP16[$18>>1] = $22;
 $23 = HEAP32[$0>>2]|0;
 _columnMallocFailure($23);
 $24 = HEAP32[$pOut>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3_column_type($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $iType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iType = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnMem($2,$3)|0);
 $5 = (_sqlite3_value_type($4)|0);
 HEAP32[$iType>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _columnMallocFailure($6);
 $7 = HEAP32[$iType>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_column_name($pStmt,$N) {
 $pStmt = $pStmt|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_columnName($2,$3,169,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3_bind_blob($pStmt,$i,$zData,$nData,$xDel) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_bindText($5,$6,$7,$8,$9,0)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3_bind_double($pStmt,$i,$rValue) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $rValue = +$rValue;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAPF64[$2>>3] = $rValue;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_vdbeUnbind($4,$5)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 116|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*40)|0)|0);
 $15 = +HEAPF64[$2>>3];
 _sqlite3VdbeMemSetDouble($14,$15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3_bind_int($p,$i,$iValue) {
 $p = $p|0;
 $i = $i|0;
 $iValue = $iValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $iValue;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3_bind_int64($3,$4,$5,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_bind_int64($pStmt,$i,$0,$1) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $pStmt;
 HEAP32[$3>>2] = $i;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (_vdbeUnbind($10,$11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 116|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + (($16*40)|0)|0);
 $21 = $4;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 _sqlite3VdbeMemSetInt64($20,$23,$26);
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3_bind_null($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_vdbeUnbind($3,$4)|0);
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3_bind_text($pStmt,$i,$zData,$nData,$xDel) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_bindText($5,$6,$7,$8,$9,1)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3_bind_value($pStmt,$i,$pValue) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pValue;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3_value_type($3)|0);
 switch ($4|0) {
 case 1:  {
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = $7;
  $9 = $8;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($8) + 4)|0;
  $12 = $11;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_bind_int64($5,$6,$10,$13)|0);
  HEAP32[$rc>>2] = $14;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
  break;
 }
 case 2:  {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = +HEAPF64[$17>>3];
  $19 = (_sqlite3_bind_double($15,$16,$18)|0);
  HEAP32[$rc>>2] = $19;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
  break;
 }
 case 4:  {
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22&65535;
  $24 = $23 & 16384;
  $25 = ($24|0)!=(0);
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  if ($25) {
   $29 = HEAP32[$28>>2]|0;
   $30 = (_sqlite3_bind_zeroblob($26,$27,$29)|0);
   HEAP32[$rc>>2] = $30;
   $52 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($52|0);
  } else {
   $31 = ((($28)) + 16|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$2>>2]|0;
   $34 = ((($33)) + 12|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3_bind_blob($26,$27,$32,$35,(-1))|0);
   HEAP32[$rc>>2] = $36;
   $52 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
  break;
 }
 case 3:  {
  $37 = HEAP32[$0>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = ((($39)) + 16|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 12|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 10|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = (_bindText($37,$38,$41,$44,(-1),$47)|0);
  HEAP32[$rc>>2] = $48;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
  break;
 }
 default: {
  $49 = HEAP32[$0>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = (_sqlite3_bind_null($49,$50)|0);
  HEAP32[$rc>>2] = $51;
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 }
 return (0)|0;
}
function _sqlite3_bind_zeroblob($pStmt,$i,$n) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_vdbeUnbind($4,$5)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 116|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*40)|0)|0);
 $15 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemSetZeroBlob($14,$15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3_bind_parameter_count($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $8 = 0;
  STACKTOP = sp;return ($8|0);
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6 << 16 >> 16;
 $8 = $7;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_bind_parameter_index($pStmt,$zName) {
 $pStmt = $pStmt|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 $6 = (_sqlite3VdbeParameterIndex($2,$3,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3_sql($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $7 = 0;
  STACKTOP = sp;return ($7|0);
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 176|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3_value_numeric_type($pVal) {
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $pMem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $eType = sp + 4|0;
 $pMem = sp;
 HEAP32[$0>>2] = $pVal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_value_type($1)|0);
 HEAP32[$eType>>2] = $2;
 $3 = HEAP32[$eType>>2]|0;
 $4 = ($3|0)==(3);
 if (!($4)) {
  $9 = HEAP32[$eType>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$pMem>>2] = $5;
 $6 = HEAP32[$pMem>>2]|0;
 _applyNumericAffinity($6,0);
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_value_type($7)|0);
 HEAP32[$eType>>2] = $8;
 $9 = HEAP32[$eType>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3_blob_open($db,$zDb,$zTable,$zColumn,$0,$1,$flags,$ppBlob) {
 $db = $db|0;
 $zDb = $zDb|0;
 $zTable = $zTable|0;
 $zColumn = $zColumn|0;
 $0 = $0|0;
 $1 = $1|0;
 $flags = $flags|0;
 $ppBlob = $ppBlob|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aOp = 0, $iCol = 0, $iDb = 0, $j = 0, $j1 = 0;
 var $nAttempt = 0, $pBlob = 0, $pFKey = 0, $pIdx = 0, $pParse = 0, $pTab = 0, $rc = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $zErr = 0, $zFault = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $5 = sp + 120|0;
 $6 = sp;
 $7 = sp + 116|0;
 $8 = sp + 112|0;
 $nAttempt = sp + 108|0;
 $iCol = sp + 104|0;
 $rc = sp + 100|0;
 $zErr = sp + 96|0;
 $pTab = sp + 92|0;
 $pParse = sp + 88|0;
 $pBlob = sp + 84|0;
 $zFault = sp + 80|0;
 $pIdx = sp + 76|0;
 $pFKey = sp + 72|0;
 $j = sp + 68|0;
 $j1 = sp + 64|0;
 $v = sp + 60|0;
 $iDb = sp + 56|0;
 $aOp = sp + 52|0;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $zDb;
 HEAP32[$4>>2] = $zTable;
 HEAP32[$5>>2] = $zColumn;
 $9 = $6;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$7>>2] = $flags;
 HEAP32[$8>>2] = $ppBlob;
 HEAP32[$nAttempt>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$zErr>>2] = 0;
 HEAP32[$pParse>>2] = 0;
 HEAP32[$pBlob>>2] = 0;
 $13 = HEAP32[$8>>2]|0;
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$7>>2]|0;
 $15 = ($14|0)!=(0);
 $16 = $15 ^ 1;
 $17 = $16 ^ 1;
 $18 = $17&1;
 HEAP32[$7>>2] = $18;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_sqlite3DbMallocZero($19,36,0)|0);
 HEAP32[$pBlob>>2] = $20;
 $21 = HEAP32[$pBlob>>2]|0;
 $22 = ($21|0)!=(0|0);
 L1: do {
  if ($22) {
   $23 = HEAP32[$2>>2]|0;
   $24 = (_sqlite3DbMallocRaw($23,480,0)|0);
   HEAP32[$pParse>>2] = $24;
   $25 = HEAP32[$pParse>>2]|0;
   $26 = ($25|0)!=(0|0);
   if ($26) {
    while(1) {
     $27 = HEAP32[$pParse>>2]|0;
     _memset(($27|0),0,480)|0;
     $28 = HEAP32[$2>>2]|0;
     $29 = HEAP32[$pParse>>2]|0;
     HEAP32[$29>>2] = $28;
     $30 = HEAP32[$2>>2]|0;
     $31 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($30,$31);
     HEAP32[$zErr>>2] = 0;
     $32 = HEAP32[$2>>2]|0;
     _sqlite3BtreeEnterAll($32);
     $33 = HEAP32[$pParse>>2]|0;
     $34 = HEAP32[$4>>2]|0;
     $35 = HEAP32[$3>>2]|0;
     $36 = (_sqlite3LocateTable($33,0,$34,$35)|0);
     HEAP32[$pTab>>2] = $36;
     $37 = HEAP32[$pTab>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$pTab>>2]|0;
      $40 = ((($39)) + 42|0);
      $41 = HEAP8[$40>>0]|0;
      $42 = $41&255;
      $43 = $42 & 16;
      $44 = ($43|0)!=(0);
      if ($44) {
       HEAP32[$pTab>>2] = 0;
       $45 = HEAP32[$pParse>>2]|0;
       $46 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer>>2] = $46;
       _sqlite3ErrorMsg($45,18193,$vararg_buffer);
      }
     }
     $47 = HEAP32[$pTab>>2]|0;
     $48 = ($47|0)!=(0|0);
     if ($48) {
      $49 = HEAP32[$pTab>>2]|0;
      $50 = ((($49)) + 42|0);
      $51 = HEAP8[$50>>0]|0;
      $52 = $51&255;
      $53 = $52 & 32;
      $54 = ($53|0)==(0);
      if (!($54)) {
       HEAP32[$pTab>>2] = 0;
       $55 = HEAP32[$pParse>>2]|0;
       $56 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer1>>2] = $56;
       _sqlite3ErrorMsg($55,18223,$vararg_buffer1);
      }
     }
     $57 = HEAP32[$pTab>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$pTab>>2]|0;
      $60 = ((($59)) + 12|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)!=(0|0);
      if ($62) {
       HEAP32[$pTab>>2] = 0;
       $63 = HEAP32[$pParse>>2]|0;
       $64 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $64;
       _sqlite3ErrorMsg($63,18259,$vararg_buffer4);
      }
     }
     $65 = HEAP32[$pTab>>2]|0;
     $66 = ($65|0)!=(0|0);
     if (!($66)) {
      label = 13;
      break;
     }
     $78 = HEAP32[$pTab>>2]|0;
     $79 = HEAP32[$pBlob>>2]|0;
     $80 = ((($79)) + 32|0);
     HEAP32[$80>>2] = $78;
     $81 = HEAP32[$2>>2]|0;
     $82 = HEAP32[$pTab>>2]|0;
     $83 = ((($82)) + 64|0);
     $84 = HEAP32[$83>>2]|0;
     $85 = (_sqlite3SchemaToIndex($81,$84)|0);
     $86 = HEAP32[$2>>2]|0;
     $87 = ((($86)) + 16|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = (($88) + ($85<<4)|0);
     $90 = HEAP32[$89>>2]|0;
     $91 = HEAP32[$pBlob>>2]|0;
     $92 = ((($91)) + 28|0);
     HEAP32[$92>>2] = $90;
     HEAP32[$iCol>>2] = 0;
     while(1) {
      $93 = HEAP32[$iCol>>2]|0;
      $94 = HEAP32[$pTab>>2]|0;
      $95 = ((($94)) + 34|0);
      $96 = HEAP16[$95>>1]|0;
      $97 = $96 << 16 >> 16;
      $98 = ($93|0)<($97|0);
      if (!($98)) {
       break;
      }
      $99 = HEAP32[$iCol>>2]|0;
      $100 = HEAP32[$pTab>>2]|0;
      $101 = ((($100)) + 4|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = (($102) + ($99<<4)|0);
      $104 = HEAP32[$103>>2]|0;
      $105 = HEAP32[$5>>2]|0;
      $106 = (_sqlite3StrICmp($104,$105)|0);
      $107 = ($106|0)==(0);
      if ($107) {
       break;
      }
      $108 = HEAP32[$iCol>>2]|0;
      $109 = (($108) + 1)|0;
      HEAP32[$iCol>>2] = $109;
     }
     $110 = HEAP32[$iCol>>2]|0;
     $111 = HEAP32[$pTab>>2]|0;
     $112 = ((($111)) + 34|0);
     $113 = HEAP16[$112>>1]|0;
     $114 = $113 << 16 >> 16;
     $115 = ($110|0)==($114|0);
     if ($115) {
      label = 21;
      break;
     }
     $121 = HEAP32[$7>>2]|0;
     $122 = ($121|0)!=(0);
     if ($122) {
      HEAP32[$zFault>>2] = 0;
      $123 = HEAP32[$2>>2]|0;
      $124 = ((($123)) + 24|0);
      $125 = HEAP32[$124>>2]|0;
      $126 = $125 & 524288;
      $127 = ($126|0)!=(0);
      L26: do {
       if ($127) {
        $128 = HEAP32[$pTab>>2]|0;
        $129 = ((($128)) + 16|0);
        $130 = HEAP32[$129>>2]|0;
        HEAP32[$pFKey>>2] = $130;
        while(1) {
         $131 = HEAP32[$pFKey>>2]|0;
         $132 = ($131|0)!=(0|0);
         if (!($132)) {
          break L26;
         }
         HEAP32[$j>>2] = 0;
         while(1) {
          $133 = HEAP32[$j>>2]|0;
          $134 = HEAP32[$pFKey>>2]|0;
          $135 = ((($134)) + 20|0);
          $136 = HEAP32[$135>>2]|0;
          $137 = ($133|0)<($136|0);
          if (!($137)) {
           break;
          }
          $138 = HEAP32[$j>>2]|0;
          $139 = HEAP32[$pFKey>>2]|0;
          $140 = ((($139)) + 36|0);
          $141 = (($140) + ($138<<3)|0);
          $142 = HEAP32[$141>>2]|0;
          $143 = HEAP32[$iCol>>2]|0;
          $144 = ($142|0)==($143|0);
          if ($144) {
           HEAP32[$zFault>>2] = 18301;
          }
          $145 = HEAP32[$j>>2]|0;
          $146 = (($145) + 1)|0;
          HEAP32[$j>>2] = $146;
         }
         $147 = HEAP32[$pFKey>>2]|0;
         $148 = ((($147)) + 4|0);
         $149 = HEAP32[$148>>2]|0;
         HEAP32[$pFKey>>2] = $149;
        }
       }
      } while(0);
      $150 = HEAP32[$pTab>>2]|0;
      $151 = ((($150)) + 8|0);
      $152 = HEAP32[$151>>2]|0;
      HEAP32[$pIdx>>2] = $152;
      while(1) {
       $153 = HEAP32[$pIdx>>2]|0;
       $154 = ($153|0)!=(0|0);
       if (!($154)) {
        break;
       }
       HEAP32[$j1>>2] = 0;
       while(1) {
        $155 = HEAP32[$j1>>2]|0;
        $156 = HEAP32[$pIdx>>2]|0;
        $157 = ((($156)) + 50|0);
        $158 = HEAP16[$157>>1]|0;
        $159 = $158&65535;
        $160 = ($155|0)<($159|0);
        if (!($160)) {
         break;
        }
        $161 = HEAP32[$j1>>2]|0;
        $162 = HEAP32[$pIdx>>2]|0;
        $163 = ((($162)) + 4|0);
        $164 = HEAP32[$163>>2]|0;
        $165 = (($164) + ($161<<1)|0);
        $166 = HEAP16[$165>>1]|0;
        $167 = $166 << 16 >> 16;
        $168 = HEAP32[$iCol>>2]|0;
        $169 = ($167|0)==($168|0);
        if ($169) {
         label = 38;
        } else {
         $170 = HEAP32[$j1>>2]|0;
         $171 = HEAP32[$pIdx>>2]|0;
         $172 = ((($171)) + 4|0);
         $173 = HEAP32[$172>>2]|0;
         $174 = (($173) + ($170<<1)|0);
         $175 = HEAP16[$174>>1]|0;
         $176 = $175 << 16 >> 16;
         $177 = ($176|0)==(-2);
         if ($177) {
          label = 38;
         }
        }
        if ((label|0) == 38) {
         label = 0;
         HEAP32[$zFault>>2] = 18313;
        }
        $178 = HEAP32[$j1>>2]|0;
        $179 = (($178) + 1)|0;
        HEAP32[$j1>>2] = $179;
       }
       $180 = HEAP32[$pIdx>>2]|0;
       $181 = ((($180)) + 20|0);
       $182 = HEAP32[$181>>2]|0;
       HEAP32[$pIdx>>2] = $182;
      }
      $183 = HEAP32[$zFault>>2]|0;
      $184 = ($183|0)!=(0|0);
      if ($184) {
       label = 42;
       break;
      }
     }
     $190 = HEAP32[$pParse>>2]|0;
     $191 = (_sqlite3VdbeCreate($190)|0);
     $192 = HEAP32[$pBlob>>2]|0;
     $193 = ((($192)) + 20|0);
     HEAP32[$193>>2] = $191;
     $194 = HEAP32[$pBlob>>2]|0;
     $195 = ((($194)) + 20|0);
     $196 = HEAP32[$195>>2]|0;
     $197 = ($196|0)!=(0|0);
     if ($197) {
      $198 = HEAP32[$pBlob>>2]|0;
      $199 = ((($198)) + 20|0);
      $200 = HEAP32[$199>>2]|0;
      HEAP32[$v>>2] = $200;
      $201 = HEAP32[$2>>2]|0;
      $202 = HEAP32[$pTab>>2]|0;
      $203 = ((($202)) + 64|0);
      $204 = HEAP32[$203>>2]|0;
      $205 = (_sqlite3SchemaToIndex($201,$204)|0);
      HEAP32[$iDb>>2] = $205;
      $206 = HEAP32[$v>>2]|0;
      $207 = HEAP32[$iDb>>2]|0;
      $208 = HEAP32[$7>>2]|0;
      $209 = HEAP32[$pTab>>2]|0;
      $210 = ((($209)) + 64|0);
      $211 = HEAP32[$210>>2]|0;
      $212 = HEAP32[$211>>2]|0;
      $213 = HEAP32[$pTab>>2]|0;
      $214 = ((($213)) + 64|0);
      $215 = HEAP32[$214>>2]|0;
      $216 = ((($215)) + 4|0);
      $217 = HEAP32[$216>>2]|0;
      (_sqlite3VdbeAddOp4Int($206,2,$207,$208,$212,$217)|0);
      $218 = HEAP32[$v>>2]|0;
      _sqlite3VdbeChangeP5($218,1);
      $219 = HEAP32[$v>>2]|0;
      $220 = (_sqlite3VdbeAddOpList($219,9,18355,0)|0);
      HEAP32[$aOp>>2] = $220;
      $221 = HEAP32[$v>>2]|0;
      $222 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($221,$222);
      $223 = HEAP32[$2>>2]|0;
      $224 = ((($223)) + 69|0);
      $225 = HEAP8[$224>>0]|0;
      $226 = $225&255;
      $227 = ($226|0)==(0);
      if ($227) {
       $228 = HEAP32[$iDb>>2]|0;
       $229 = HEAP32[$aOp>>2]|0;
       $230 = ((($229)) + 4|0);
       HEAP32[$230>>2] = $228;
       $231 = HEAP32[$pTab>>2]|0;
       $232 = ((($231)) + 28|0);
       $233 = HEAP32[$232>>2]|0;
       $234 = HEAP32[$aOp>>2]|0;
       $235 = ((($234)) + 8|0);
       HEAP32[$235>>2] = $233;
       $236 = HEAP32[$7>>2]|0;
       $237 = HEAP32[$aOp>>2]|0;
       $238 = ((($237)) + 12|0);
       HEAP32[$238>>2] = $236;
       $239 = HEAP32[$v>>2]|0;
       $240 = HEAP32[$pTab>>2]|0;
       $241 = HEAP32[$240>>2]|0;
       _sqlite3VdbeChangeP4($239,1,$241,0);
      }
      $242 = HEAP32[$2>>2]|0;
      $243 = ((($242)) + 69|0);
      $244 = HEAP8[$243>>0]|0;
      $245 = $244&255;
      $246 = ($245|0)==(0);
      if ($246) {
       $247 = HEAP32[$7>>2]|0;
       $248 = ($247|0)!=(0);
       if ($248) {
        $249 = HEAP32[$aOp>>2]|0;
        $250 = ((($249)) + 20|0);
        HEAP8[$250>>0] = 105;
       }
       $251 = HEAP32[$pTab>>2]|0;
       $252 = ((($251)) + 28|0);
       $253 = HEAP32[$252>>2]|0;
       $254 = HEAP32[$aOp>>2]|0;
       $255 = ((($254)) + 20|0);
       $256 = ((($255)) + 8|0);
       HEAP32[$256>>2] = $253;
       $257 = HEAP32[$iDb>>2]|0;
       $258 = HEAP32[$aOp>>2]|0;
       $259 = ((($258)) + 20|0);
       $260 = ((($259)) + 12|0);
       HEAP32[$260>>2] = $257;
       $261 = HEAP32[$aOp>>2]|0;
       $262 = ((($261)) + 20|0);
       $263 = ((($262)) + 1|0);
       HEAP8[$263>>0] = -14;
       $264 = HEAP32[$pTab>>2]|0;
       $265 = ((($264)) + 34|0);
       $266 = HEAP16[$265>>1]|0;
       $267 = $266 << 16 >> 16;
       $268 = (($267) + 1)|0;
       $269 = HEAP32[$aOp>>2]|0;
       $270 = ((($269)) + 20|0);
       $271 = ((($270)) + 16|0);
       HEAP32[$271>>2] = $268;
       $272 = HEAP32[$pTab>>2]|0;
       $273 = ((($272)) + 34|0);
       $274 = HEAP16[$273>>1]|0;
       $275 = $274 << 16 >> 16;
       $276 = HEAP32[$aOp>>2]|0;
       $277 = ((($276)) + 80|0);
       $278 = ((($277)) + 8|0);
       HEAP32[$278>>2] = $275;
       $279 = HEAP32[$pParse>>2]|0;
       $280 = ((($279)) + 400|0);
       HEAP16[$280>>1] = 1;
       $281 = HEAP32[$pParse>>2]|0;
       $282 = ((($281)) + 44|0);
       HEAP32[$282>>2] = 1;
       $283 = HEAP32[$pParse>>2]|0;
       $284 = ((($283)) + 40|0);
       HEAP32[$284>>2] = 1;
       $285 = HEAP32[$v>>2]|0;
       $286 = HEAP32[$pParse>>2]|0;
       _sqlite3VdbeMakeReady($285,$286);
      }
     }
     $287 = HEAP32[$7>>2]|0;
     $288 = HEAP32[$pBlob>>2]|0;
     HEAP32[$288>>2] = $287;
     $289 = HEAP32[$iCol>>2]|0;
     $290 = HEAP32[$pBlob>>2]|0;
     $291 = ((($290)) + 12|0);
     HEAP32[$291>>2] = $289;
     $292 = HEAP32[$2>>2]|0;
     $293 = HEAP32[$pBlob>>2]|0;
     $294 = ((($293)) + 24|0);
     HEAP32[$294>>2] = $292;
     $295 = HEAP32[$2>>2]|0;
     $296 = ((($295)) + 69|0);
     $297 = HEAP8[$296>>0]|0;
     $298 = ($297<<24>>24)!=(0);
     if ($298) {
      break L1;
     }
     $299 = HEAP32[$pBlob>>2]|0;
     $300 = ((($299)) + 20|0);
     $301 = HEAP32[$300>>2]|0;
     $302 = $6;
     $303 = $302;
     $304 = HEAP32[$303>>2]|0;
     $305 = (($302) + 4)|0;
     $306 = $305;
     $307 = HEAP32[$306>>2]|0;
     (_sqlite3_bind_int64($301,1,$304,$307)|0);
     $308 = HEAP32[$pBlob>>2]|0;
     $309 = $6;
     $310 = $309;
     $311 = HEAP32[$310>>2]|0;
     $312 = (($309) + 4)|0;
     $313 = $312;
     $314 = HEAP32[$313>>2]|0;
     $315 = (_blobSeekToRow($308,$311,$314,$zErr)|0);
     HEAP32[$rc>>2] = $315;
     $316 = HEAP32[$nAttempt>>2]|0;
     $317 = (($316) + 1)|0;
     HEAP32[$nAttempt>>2] = $317;
     $318 = ($317|0)<(50);
     $319 = HEAP32[$rc>>2]|0;
     $320 = ($319|0)==(17);
     $321 = $318 ? $320 : 0;
     if (!($321)) {
      break L1;
     }
    }
    if ((label|0) == 13) {
     $67 = HEAP32[$pParse>>2]|0;
     $68 = ((($67)) + 4|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = ($69|0)!=(0|0);
     if ($70) {
      $71 = HEAP32[$2>>2]|0;
      $72 = HEAP32[$zErr>>2]|0;
      _sqlite3DbFree($71,$72);
      $73 = HEAP32[$pParse>>2]|0;
      $74 = ((($73)) + 4|0);
      $75 = HEAP32[$74>>2]|0;
      HEAP32[$zErr>>2] = $75;
      $76 = HEAP32[$pParse>>2]|0;
      $77 = ((($76)) + 4|0);
      HEAP32[$77>>2] = 0;
     }
     HEAP32[$rc>>2] = 1;
     break;
    }
    else if ((label|0) == 21) {
     $116 = HEAP32[$2>>2]|0;
     $117 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($116,$117);
     $118 = HEAP32[$2>>2]|0;
     $119 = HEAP32[$5>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $119;
     $120 = (_sqlite3MPrintf($118,18280,$vararg_buffer7)|0);
     HEAP32[$zErr>>2] = $120;
     HEAP32[$rc>>2] = 1;
     break;
    }
    else if ((label|0) == 42) {
     $185 = HEAP32[$2>>2]|0;
     $186 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($185,$186);
     $187 = HEAP32[$2>>2]|0;
     $188 = HEAP32[$zFault>>2]|0;
     HEAP32[$vararg_buffer10>>2] = $188;
     $189 = (_sqlite3MPrintf($187,18321,$vararg_buffer10)|0);
     HEAP32[$zErr>>2] = $189;
     HEAP32[$rc>>2] = 1;
     break;
    }
   }
  }
 } while(0);
 $322 = HEAP32[$rc>>2]|0;
 $323 = ($322|0)==(0);
 if ($323) {
  $324 = HEAP32[$2>>2]|0;
  $325 = ((($324)) + 69|0);
  $326 = HEAP8[$325>>0]|0;
  $327 = $326&255;
  $328 = ($327|0)==(0);
  if ($328) {
   $329 = HEAP32[$pBlob>>2]|0;
   $330 = HEAP32[$8>>2]|0;
   HEAP32[$330>>2] = $329;
   $342 = HEAP32[$2>>2]|0;
   $343 = HEAP32[$rc>>2]|0;
   $344 = HEAP32[$zErr>>2]|0;
   $345 = ($344|0)!=(0|0);
   $346 = $345 ? 18391 : 0;
   $347 = HEAP32[$zErr>>2]|0;
   HEAP32[$vararg_buffer13>>2] = $347;
   _sqlite3ErrorWithMsg($342,$343,$346,$vararg_buffer13);
   $348 = HEAP32[$2>>2]|0;
   $349 = HEAP32[$zErr>>2]|0;
   _sqlite3DbFree($348,$349);
   $350 = HEAP32[$pParse>>2]|0;
   _sqlite3ParserReset($350);
   $351 = HEAP32[$2>>2]|0;
   $352 = HEAP32[$pParse>>2]|0;
   _sqlite3DbFree($351,$352);
   $353 = HEAP32[$2>>2]|0;
   $354 = HEAP32[$rc>>2]|0;
   $355 = (_sqlite3ApiExit($353,$354)|0);
   HEAP32[$rc>>2] = $355;
   $356 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($356|0);
  }
 }
 $331 = HEAP32[$pBlob>>2]|0;
 $332 = ($331|0)!=(0|0);
 if ($332) {
  $333 = HEAP32[$pBlob>>2]|0;
  $334 = ((($333)) + 20|0);
  $335 = HEAP32[$334>>2]|0;
  $336 = ($335|0)!=(0|0);
  if ($336) {
   $337 = HEAP32[$pBlob>>2]|0;
   $338 = ((($337)) + 20|0);
   $339 = HEAP32[$338>>2]|0;
   (_sqlite3VdbeFinalize($339)|0);
  }
 }
 $340 = HEAP32[$2>>2]|0;
 $341 = HEAP32[$pBlob>>2]|0;
 _sqlite3DbFree($340,$341);
 $342 = HEAP32[$2>>2]|0;
 $343 = HEAP32[$rc>>2]|0;
 $344 = HEAP32[$zErr>>2]|0;
 $345 = ($344|0)!=(0|0);
 $346 = $345 ? 18391 : 0;
 $347 = HEAP32[$zErr>>2]|0;
 HEAP32[$vararg_buffer13>>2] = $347;
 _sqlite3ErrorWithMsg($342,$343,$346,$vararg_buffer13);
 $348 = HEAP32[$2>>2]|0;
 $349 = HEAP32[$zErr>>2]|0;
 _sqlite3DbFree($348,$349);
 $350 = HEAP32[$pParse>>2]|0;
 _sqlite3ParserReset($350);
 $351 = HEAP32[$2>>2]|0;
 $352 = HEAP32[$pParse>>2]|0;
 _sqlite3DbFree($351,$352);
 $353 = HEAP32[$2>>2]|0;
 $354 = HEAP32[$rc>>2]|0;
 $355 = (_sqlite3ApiExit($353,$354)|0);
 HEAP32[$rc>>2] = $355;
 $356 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($356|0);
}
function _sqlite3_blob_close($pBlob) {
 $pBlob = $pBlob|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pBlob;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 24|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$db>>2] = $6;
  $7 = HEAP32[$p>>2]|0;
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3_finalize($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($11,$12);
  $13 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  HEAP32[$rc>>2] = 0;
  $13 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _sqlite3_blob_read($pBlob,$z,$n,$iOffset) {
 $pBlob = $pBlob|0;
 $z = $z|0;
 $n = $n|0;
 $iOffset = $iOffset|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pBlob;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $iOffset;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_blobReadWrite($4,$5,$6,$7,138)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_blob_bytes($pBlob) {
 $pBlob = $pBlob|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBlob;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $11 = 0;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $11 = 0;
  STACKTOP = sp;return ($11|0);
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3_blob_reopen($pBlob,$0,$1) {
 $pBlob = $pBlob|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp;
 $rc = sp + 24|0;
 $p = sp + 20|0;
 $db = sp + 16|0;
 $zErr = sp + 12|0;
 HEAP32[$3>>2] = $pBlob;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = (_sqlite3MisuseError(84814)|0);
  HEAP32[$2>>2] = $12;
  $42 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)==(0|0);
 if ($19) {
  HEAP32[$rc>>2] = 4;
 } else {
  $20 = HEAP32[$p>>2]|0;
  $21 = $4;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = (_blobSeekToRow($20,$23,$26,$zErr)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$rc>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$db>>2]|0;
   $31 = HEAP32[$rc>>2]|0;
   $32 = HEAP32[$zErr>>2]|0;
   $33 = ($32|0)!=(0|0);
   $34 = $33 ? 18391 : 0;
   $35 = HEAP32[$zErr>>2]|0;
   HEAP32[$vararg_buffer>>2] = $35;
   _sqlite3ErrorWithMsg($30,$31,$34,$vararg_buffer);
   $36 = HEAP32[$db>>2]|0;
   $37 = HEAP32[$zErr>>2]|0;
   _sqlite3DbFree($36,$37);
  }
 }
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$rc>>2]|0;
 $40 = (_sqlite3ApiExit($38,$39)|0);
 HEAP32[$rc>>2] = $40;
 $41 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $41;
 $42 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3_strglob($zGlobPattern,$zString) {
 $zGlobPattern = $zGlobPattern|0;
 $zString = $zString|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $zGlobPattern;
 HEAP32[$1>>2] = $zString;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_patternCompare($2,$3,18394,91)|0);
 $5 = ($4|0)==(0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3_strlike($zPattern,$zStr,$esc) {
 $zPattern = $zPattern|0;
 $zStr = $zStr|0;
 $esc = $esc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $zPattern;
 HEAP32[$1>>2] = $zStr;
 HEAP32[$2>>2] = $esc;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_patternCompare($3,$4,18398,$5)|0);
 $7 = ($6|0)==(0);
 $8 = $7&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_exec($db,$zSql,$xCallback,$pArg,$pzErrMsg) {
 $db = $db|0;
 $zSql = $zSql|0;
 $xCallback = $xCallback|0;
 $pArg = $pArg|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $azCols = 0, $azVals = 0, $callbackIsInit = 0, $i = 0, $nCol = 0, $nErrMsg = 0, $or$cond = 0, $or$cond3 = 0;
 var $pStmt = 0, $rc = 0, $zLeftover = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $rc = sp + 32|0;
 $zLeftover = sp + 28|0;
 $pStmt = sp + 24|0;
 $azCols = sp + 20|0;
 $callbackIsInit = sp + 16|0;
 $nCol = sp + 12|0;
 $azVals = sp + 8|0;
 $i = sp + 4|0;
 $nErrMsg = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $xCallback;
 HEAP32[$4>>2] = $pArg;
 HEAP32[$5>>2] = $pzErrMsg;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 HEAP32[$azCols>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3SafetyCheckOk($6)|0);
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = (_sqlite3MisuseError(109597)|0);
  HEAP32[$0>>2] = $9;
  $151 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($151|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$2>>2] = 18402;
 }
 $12 = HEAP32[$1>>2]|0;
 _sqlite3Error($12,0);
 L8: while(1) {
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   break;
  }
  HEAP32[$azVals>>2] = 0;
  HEAP32[$pStmt>>2] = 0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (_sqlite3_prepare_v2($19,$20,-1,$pStmt,$zLeftover)|0);
  HEAP32[$rc>>2] = $21;
  $22 = HEAP32[$rc>>2]|0;
  $23 = ($22|0)!=(0);
  if ($23) {
   continue;
  }
  $24 = HEAP32[$pStmt>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   $26 = HEAP32[$zLeftover>>2]|0;
   HEAP32[$2>>2] = $26;
   continue;
  }
  HEAP32[$callbackIsInit>>2] = 0;
  $27 = HEAP32[$pStmt>>2]|0;
  $28 = (_sqlite3_column_count($27)|0);
  HEAP32[$nCol>>2] = $28;
  while(1) {
   $29 = HEAP32[$pStmt>>2]|0;
   $30 = (_sqlite3_step($29)|0);
   HEAP32[$rc>>2] = $30;
   $31 = HEAP32[$3>>2]|0;
   $32 = ($31|0)!=(0|0);
   do {
    if ($32) {
     $33 = HEAP32[$rc>>2]|0;
     $34 = (100)==($33|0);
     if (!($34)) {
      $35 = HEAP32[$rc>>2]|0;
      $36 = (101)!=($35|0);
      $37 = HEAP32[$callbackIsInit>>2]|0;
      $38 = ($37|0)!=(0);
      $or$cond = $36 | $38;
      if ($or$cond) {
       break;
      }
      $39 = HEAP32[$1>>2]|0;
      $40 = ((($39)) + 24|0);
      $41 = HEAP32[$40>>2]|0;
      $42 = $41 & 256;
      $43 = ($42|0)!=(0);
      if (!($43)) {
       break;
      }
     }
     $44 = HEAP32[$callbackIsInit>>2]|0;
     $45 = ($44|0)!=(0);
     if (!($45)) {
      $46 = HEAP32[$1>>2]|0;
      $47 = HEAP32[$nCol>>2]|0;
      $48 = $47<<1;
      $49 = $48<<2;
      $50 = (($49) + 1)|0;
      $51 = (_sqlite3DbMallocZero($46,$50,0)|0);
      HEAP32[$azCols>>2] = $51;
      $52 = HEAP32[$azCols>>2]|0;
      $53 = ($52|0)==(0|0);
      if ($53) {
       break L8;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $54 = HEAP32[$i>>2]|0;
       $55 = HEAP32[$nCol>>2]|0;
       $56 = ($54|0)<($55|0);
       if (!($56)) {
        break;
       }
       $57 = HEAP32[$pStmt>>2]|0;
       $58 = HEAP32[$i>>2]|0;
       $59 = (_sqlite3_column_name($57,$58)|0);
       $60 = HEAP32[$i>>2]|0;
       $61 = HEAP32[$azCols>>2]|0;
       $62 = (($61) + ($60<<2)|0);
       HEAP32[$62>>2] = $59;
       $63 = HEAP32[$i>>2]|0;
       $64 = (($63) + 1)|0;
       HEAP32[$i>>2] = $64;
      }
      HEAP32[$callbackIsInit>>2] = 1;
     }
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)==(100);
     L32: do {
      if ($66) {
       $67 = HEAP32[$nCol>>2]|0;
       $68 = HEAP32[$azCols>>2]|0;
       $69 = (($68) + ($67<<2)|0);
       HEAP32[$azVals>>2] = $69;
       HEAP32[$i>>2] = 0;
       while(1) {
        $70 = HEAP32[$i>>2]|0;
        $71 = HEAP32[$nCol>>2]|0;
        $72 = ($70|0)<($71|0);
        if (!($72)) {
         break L32;
        }
        $73 = HEAP32[$pStmt>>2]|0;
        $74 = HEAP32[$i>>2]|0;
        $75 = (_sqlite3_column_text($73,$74)|0);
        $76 = HEAP32[$i>>2]|0;
        $77 = HEAP32[$azVals>>2]|0;
        $78 = (($77) + ($76<<2)|0);
        HEAP32[$78>>2] = $75;
        $79 = HEAP32[$i>>2]|0;
        $80 = HEAP32[$azVals>>2]|0;
        $81 = (($80) + ($79<<2)|0);
        $82 = HEAP32[$81>>2]|0;
        $83 = ($82|0)!=(0|0);
        if (!($83)) {
         $84 = HEAP32[$pStmt>>2]|0;
         $85 = HEAP32[$i>>2]|0;
         $86 = (_sqlite3_column_type($84,$85)|0);
         $87 = ($86|0)!=(5);
         if ($87) {
          label = 27;
          break L8;
         }
        }
        $89 = HEAP32[$i>>2]|0;
        $90 = (($89) + 1)|0;
        HEAP32[$i>>2] = $90;
       }
      }
     } while(0);
     $91 = HEAP32[$3>>2]|0;
     $92 = HEAP32[$4>>2]|0;
     $93 = HEAP32[$nCol>>2]|0;
     $94 = HEAP32[$azVals>>2]|0;
     $95 = HEAP32[$azCols>>2]|0;
     $96 = (FUNCTION_TABLE_iiiii[$91 & 255]($92,$93,$94,$95)|0);
     $97 = ($96|0)!=(0);
     if ($97) {
      label = 30;
      break L8;
     }
    }
   } while(0);
   $100 = HEAP32[$rc>>2]|0;
   $101 = ($100|0)!=(100);
   if ($101) {
    break;
   }
  }
  $102 = HEAP32[$pStmt>>2]|0;
  $103 = (_sqlite3VdbeFinalize($102)|0);
  HEAP32[$rc>>2] = $103;
  HEAP32[$pStmt>>2] = 0;
  $104 = HEAP32[$zLeftover>>2]|0;
  HEAP32[$2>>2] = $104;
  while(1) {
   $105 = HEAP32[$2>>2]|0;
   $106 = HEAP8[$105>>0]|0;
   $107 = $106&255;
   $108 = (18403 + ($107)|0);
   $109 = HEAP8[$108>>0]|0;
   $110 = $109&255;
   $111 = $110 & 1;
   $112 = ($111|0)!=(0);
   if (!($112)) {
    break;
   }
   $113 = HEAP32[$2>>2]|0;
   $114 = ((($113)) + 1|0);
   HEAP32[$2>>2] = $114;
  }
  $115 = HEAP32[$1>>2]|0;
  $116 = HEAP32[$azCols>>2]|0;
  _sqlite3DbFree($115,$116);
  HEAP32[$azCols>>2] = 0;
 }
 if ((label|0) == 27) {
  $88 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($88);
 }
 else if ((label|0) == 30) {
  HEAP32[$rc>>2] = 4;
  $98 = HEAP32[$pStmt>>2]|0;
  (_sqlite3VdbeFinalize($98)|0);
  HEAP32[$pStmt>>2] = 0;
  $99 = HEAP32[$1>>2]|0;
  _sqlite3Error($99,4);
 }
 $117 = HEAP32[$pStmt>>2]|0;
 $118 = ($117|0)!=(0|0);
 if ($118) {
  $119 = HEAP32[$pStmt>>2]|0;
  (_sqlite3VdbeFinalize($119)|0);
 }
 $120 = HEAP32[$1>>2]|0;
 $121 = HEAP32[$azCols>>2]|0;
 _sqlite3DbFree($120,$121);
 $122 = HEAP32[$1>>2]|0;
 $123 = HEAP32[$rc>>2]|0;
 $124 = (_sqlite3ApiExit($122,$123)|0);
 HEAP32[$rc>>2] = $124;
 $125 = HEAP32[$rc>>2]|0;
 $126 = ($125|0)!=(0);
 $127 = HEAP32[$5>>2]|0;
 $128 = ($127|0)!=(0|0);
 $or$cond3 = $126 & $128;
 do {
  if ($or$cond3) {
   $129 = HEAP32[$1>>2]|0;
   $130 = (_sqlite3_errmsg($129)|0);
   $131 = (_sqlite3Strlen30($130)|0);
   $132 = (1 + ($131))|0;
   HEAP32[$nErrMsg>>2] = $132;
   $133 = HEAP32[$nErrMsg>>2]|0;
   $134 = ($133|0)<(0);
   $135 = $134 << 31 >> 31;
   $136 = (_sqlite3Malloc($133,$135)|0);
   $137 = HEAP32[$5>>2]|0;
   HEAP32[$137>>2] = $136;
   $138 = HEAP32[$5>>2]|0;
   $139 = HEAP32[$138>>2]|0;
   $140 = ($139|0)!=(0|0);
   if ($140) {
    $141 = HEAP32[$5>>2]|0;
    $142 = HEAP32[$141>>2]|0;
    $143 = HEAP32[$1>>2]|0;
    $144 = (_sqlite3_errmsg($143)|0);
    $145 = HEAP32[$nErrMsg>>2]|0;
    _memcpy(($142|0),($144|0),($145|0))|0;
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    $146 = HEAP32[$1>>2]|0;
    _sqlite3Error($146,7);
    break;
   }
  } else {
   $147 = HEAP32[$5>>2]|0;
   $148 = ($147|0)!=(0|0);
   if ($148) {
    $149 = HEAP32[$5>>2]|0;
    HEAP32[$149>>2] = 0;
   }
  }
 } while(0);
 $150 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $150;
 $151 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($151|0);
}
function _sqlite3_prepare_v2($db,$zSql,$nBytes,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $ppStmt;
 HEAP32[$4>>2] = $pzTail;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3LockAndPrepare($5,$6,$7,1,0,$8,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3_errmsg($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $4 = (_sqlite3ErrStr(7)|0);
  HEAP32[$0>>2] = $4;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3SafetyCheckSickOrOk($5)|0);
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MisuseError(140046)|0);
  $9 = (_sqlite3ErrStr($8)|0);
  HEAP32[$0>>2] = $9;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 69|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  $14 = (_sqlite3ErrStr(7)|0);
  HEAP32[$z>>2] = $14;
 } else {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 244|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3_value_text($17)|0);
  HEAP32[$z>>2] = $18;
  $19 = HEAP32[$z>>2]|0;
  $20 = ($19|0)==(0|0);
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 52|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3ErrStr($23)|0);
   HEAP32[$z>>2] = $24;
  }
 }
 $25 = HEAP32[$z>>2]|0;
 HEAP32[$0>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3_prepare($db,$zSql,$nBytes,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $ppStmt;
 HEAP32[$4>>2] = $pzTail;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3LockAndPrepare($5,$6,$7,0,0,$8,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3_create_module($db,$zName,$pModule,$pAux) {
 $db = $db|0;
 $zName = $zName|0;
 $pModule = $pModule|0;
 $pAux = $pAux|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pModule;
 HEAP32[$3>>2] = $pAux;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_createModule($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_create_module_v2($db,$zName,$pModule,$pAux,$xDestroy) {
 $db = $db|0;
 $zName = $zName|0;
 $pModule = $pModule|0;
 $pAux = $pAux|0;
 $xDestroy = $xDestroy|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pModule;
 HEAP32[$3>>2] = $pAux;
 HEAP32[$4>>2] = $xDestroy;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_createModule($5,$6,$7,$8,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3_declare_vtab($db,$zCreateTable) {
 $db = $db|0;
 $zCreateTable = $zCreateTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $pCtx = 0, $pIdx = 0, $pNew = 0, $pParse = 0, $pTab = 0, $rc = 0, $vararg_buffer = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pCtx = sp + 28|0;
 $pParse = sp + 24|0;
 $rc = sp + 20|0;
 $pTab = sp + 16|0;
 $zErr = sp + 12|0;
 $pNew = sp + 8|0;
 $pIdx = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zCreateTable;
 HEAP32[$rc>>2] = 0;
 HEAP32[$zErr>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 336|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pCtx>>2] = $5;
 $6 = HEAP32[$pCtx>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$pCtx>>2]|0;
  $9 = ((($8)) + 12|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   $14 = HEAP32[$pCtx>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$pTab>>2] = $16;
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3DbMallocZero($17,480,0)|0);
   HEAP32[$pParse>>2] = $18;
   $19 = HEAP32[$pParse>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    HEAP32[$rc>>2] = 7;
   } else {
    $21 = HEAP32[$pParse>>2]|0;
    $22 = ((($21)) + 410|0);
    HEAP8[$22>>0] = 1;
    $23 = HEAP32[$1>>2]|0;
    $24 = HEAP32[$pParse>>2]|0;
    HEAP32[$24>>2] = $23;
    $25 = HEAP32[$pParse>>2]|0;
    $26 = ((($25)) + 136|0);
    HEAP32[$26>>2] = 1;
    $27 = HEAP32[$pParse>>2]|0;
    $28 = HEAP32[$2>>2]|0;
    $29 = (_sqlite3RunParser($27,$28,$zErr)|0);
    $30 = (0)==($29|0);
    if ($30) {
     $31 = HEAP32[$pParse>>2]|0;
     $32 = ((($31)) + 440|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($33|0)!=(0|0);
     if ($34) {
      $35 = HEAP32[$1>>2]|0;
      $36 = ((($35)) + 69|0);
      $37 = HEAP8[$36>>0]|0;
      $38 = ($37<<24>>24)!=(0);
      if ($38) {
       label = 18;
      } else {
       $39 = HEAP32[$pParse>>2]|0;
       $40 = ((($39)) + 440|0);
       $41 = HEAP32[$40>>2]|0;
       $42 = ((($41)) + 12|0);
       $43 = HEAP32[$42>>2]|0;
       $44 = ($43|0)!=(0|0);
       if ($44) {
        label = 18;
       } else {
        $45 = HEAP32[$pParse>>2]|0;
        $46 = ((($45)) + 440|0);
        $47 = HEAP32[$46>>2]|0;
        $48 = ((($47)) + 42|0);
        $49 = HEAP8[$48>>0]|0;
        $50 = $49&255;
        $51 = $50 & 16;
        $52 = ($51|0)==(0);
        if ($52) {
         $53 = HEAP32[$pTab>>2]|0;
         $54 = ((($53)) + 4|0);
         $55 = HEAP32[$54>>2]|0;
         $56 = ($55|0)!=(0|0);
         if (!($56)) {
          $57 = HEAP32[$pParse>>2]|0;
          $58 = ((($57)) + 440|0);
          $59 = HEAP32[$58>>2]|0;
          HEAP32[$pNew>>2] = $59;
          $60 = HEAP32[$pNew>>2]|0;
          $61 = ((($60)) + 4|0);
          $62 = HEAP32[$61>>2]|0;
          $63 = HEAP32[$pTab>>2]|0;
          $64 = ((($63)) + 4|0);
          HEAP32[$64>>2] = $62;
          $65 = HEAP32[$pNew>>2]|0;
          $66 = ((($65)) + 34|0);
          $67 = HEAP16[$66>>1]|0;
          $68 = HEAP32[$pTab>>2]|0;
          $69 = ((($68)) + 34|0);
          HEAP16[$69>>1] = $67;
          $70 = HEAP32[$pNew>>2]|0;
          $71 = ((($70)) + 42|0);
          $72 = HEAP8[$71>>0]|0;
          $73 = $72&255;
          $74 = $73 & 96;
          $75 = HEAP32[$pTab>>2]|0;
          $76 = ((($75)) + 42|0);
          $77 = HEAP8[$76>>0]|0;
          $78 = $77&255;
          $79 = $78 | $74;
          $80 = $79&255;
          HEAP8[$76>>0] = $80;
          $81 = HEAP32[$pNew>>2]|0;
          $82 = ((($81)) + 34|0);
          HEAP16[$82>>1] = 0;
          $83 = HEAP32[$pNew>>2]|0;
          $84 = ((($83)) + 4|0);
          HEAP32[$84>>2] = 0;
          $85 = HEAP32[$pNew>>2]|0;
          $86 = ((($85)) + 42|0);
          $87 = HEAP8[$86>>0]|0;
          $88 = $87&255;
          $89 = $88 & 32;
          $90 = ($89|0)==(0);
          if (!($90)) {
           $91 = HEAP32[$pCtx>>2]|0;
           $92 = HEAP32[$91>>2]|0;
           $93 = ((($92)) + 4|0);
           $94 = HEAP32[$93>>2]|0;
           $95 = HEAP32[$94>>2]|0;
           $96 = ((($95)) + 52|0);
           $97 = HEAP32[$96>>2]|0;
           $98 = ($97|0)!=(0|0);
           if ($98) {
            HEAP32[$rc>>2] = 1;
           }
          }
          $99 = HEAP32[$pNew>>2]|0;
          $100 = ((($99)) + 8|0);
          $101 = HEAP32[$100>>2]|0;
          HEAP32[$pIdx>>2] = $101;
          $102 = HEAP32[$pIdx>>2]|0;
          $103 = ($102|0)!=(0|0);
          if ($103) {
           $104 = HEAP32[$pIdx>>2]|0;
           $105 = HEAP32[$pTab>>2]|0;
           $106 = ((($105)) + 8|0);
           HEAP32[$106>>2] = $104;
           $107 = HEAP32[$pNew>>2]|0;
           $108 = ((($107)) + 8|0);
           HEAP32[$108>>2] = 0;
           $109 = HEAP32[$pTab>>2]|0;
           $110 = HEAP32[$pIdx>>2]|0;
           $111 = ((($110)) + 12|0);
           HEAP32[$111>>2] = $109;
          }
         }
         $112 = HEAP32[$pCtx>>2]|0;
         $113 = ((($112)) + 12|0);
         HEAP32[$113>>2] = 1;
        } else {
         label = 18;
        }
       }
      }
     } else {
      label = 18;
     }
    } else {
     label = 18;
    }
    if ((label|0) == 18) {
     $114 = HEAP32[$1>>2]|0;
     $115 = HEAP32[$zErr>>2]|0;
     $116 = ($115|0)!=(0|0);
     $117 = $116 ? 18391 : 0;
     $118 = HEAP32[$zErr>>2]|0;
     HEAP32[$vararg_buffer>>2] = $118;
     _sqlite3ErrorWithMsg($114,1,$117,$vararg_buffer);
     $119 = HEAP32[$1>>2]|0;
     $120 = HEAP32[$zErr>>2]|0;
     _sqlite3DbFree($119,$120);
     HEAP32[$rc>>2] = 1;
    }
    $121 = HEAP32[$pParse>>2]|0;
    $122 = ((($121)) + 410|0);
    HEAP8[$122>>0] = 0;
    $123 = HEAP32[$pParse>>2]|0;
    $124 = ((($123)) + 8|0);
    $125 = HEAP32[$124>>2]|0;
    $126 = ($125|0)!=(0|0);
    if ($126) {
     $127 = HEAP32[$pParse>>2]|0;
     $128 = ((($127)) + 8|0);
     $129 = HEAP32[$128>>2]|0;
     (_sqlite3VdbeFinalize($129)|0);
    }
    $130 = HEAP32[$1>>2]|0;
    $131 = HEAP32[$pParse>>2]|0;
    $132 = ((($131)) + 440|0);
    $133 = HEAP32[$132>>2]|0;
    _sqlite3DeleteTable($130,$133);
    $134 = HEAP32[$pParse>>2]|0;
    _sqlite3ParserReset($134);
    $135 = HEAP32[$1>>2]|0;
    $136 = HEAP32[$pParse>>2]|0;
    _sqlite3DbFree($135,$136);
   }
   $137 = HEAP32[$1>>2]|0;
   $138 = HEAP32[$rc>>2]|0;
   $139 = (_sqlite3ApiExit($137,$138)|0);
   HEAP32[$rc>>2] = $139;
   $140 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $140;
   $141 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($141|0);
  }
 }
 $12 = HEAP32[$1>>2]|0;
 _sqlite3Error($12,21);
 $13 = (_sqlite3MisuseError(123278)|0);
 HEAP32[$0>>2] = $13;
 $141 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($141|0);
}
function _sqlite3_vtab_on_conflict($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 74|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = (($4) - 1)|0;
 $6 = (18659 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3_vtab_config($db,$op,$varargs) {
 $db = $db|0;
 $op = $op|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $arglist_current = 0, $arglist_next = 0, $cond = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$1>>2]|0;
 $cond = ($2|0)==(1);
 do {
  if ($cond) {
   $3 = HEAP32[$0>>2]|0;
   $4 = ((($3)) + 336|0);
   $5 = HEAP32[$4>>2]|0;
   HEAP32[$p>>2] = $5;
   $6 = HEAP32[$p>>2]|0;
   $7 = ($6|0)!=(0|0);
   if ($7) {
    $arglist_current = HEAP32[$ap>>2]|0;
    $9 = $arglist_current;
    $10 = ((0) + 4|0);
    $expanded2 = $10;
    $expanded = (($expanded2) - 1)|0;
    $11 = (($9) + ($expanded))|0;
    $12 = ((0) + 4|0);
    $expanded6 = $12;
    $expanded5 = (($expanded6) - 1)|0;
    $expanded4 = $expanded5 ^ -1;
    $13 = $11 & $expanded4;
    $14 = $13;
    $15 = HEAP32[$14>>2]|0;
    $arglist_next = ((($14)) + 4|0);
    HEAP32[$ap>>2] = $arglist_next;
    $16 = $15&255;
    $17 = HEAP32[$p>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = ((($18)) + 16|0);
    HEAP8[$19>>0] = $16;
    break;
   } else {
    $8 = (_sqlite3MisuseError(123749)|0);
    HEAP32[$rc>>2] = $8;
    break;
   }
  } else {
   $20 = (_sqlite3MisuseError(123757)|0);
   HEAP32[$rc>>2] = $20;
  }
 } while(0);
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $25 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$rc>>2]|0;
 _sqlite3Error($23,$24);
 $25 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3_libversion() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return (17598|0);
}
function _sqlite3_sourceid() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return (18677|0);
}
function _sqlite3_libversion_number() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 3015001;
}
function _sqlite3_config($op,$varargs) {
 $op = $op|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ap = 0, $arglist_current = 0, $arglist_current12 = 0, $arglist_current15 = 0, $arglist_current18 = 0, $arglist_current21 = 0, $arglist_current24 = 0, $arglist_current27 = 0, $arglist_current3 = 0;
 var $arglist_current30 = 0, $arglist_current33 = 0, $arglist_current36 = 0, $arglist_current39 = 0, $arglist_current42 = 0, $arglist_current45 = 0, $arglist_current48 = 0, $arglist_current51 = 0, $arglist_current54 = 0, $arglist_current57 = 0, $arglist_current6 = 0, $arglist_current60 = 0, $arglist_current63 = 0, $arglist_current9 = 0, $arglist_next = 0, $arglist_next10 = 0, $arglist_next13 = 0, $arglist_next16 = 0, $arglist_next19 = 0, $arglist_next22 = 0;
 var $arglist_next25 = 0, $arglist_next28 = 0, $arglist_next31 = 0, $arglist_next34 = 0, $arglist_next37 = 0, $arglist_next4 = 0, $arglist_next40 = 0, $arglist_next43 = 0, $arglist_next46 = 0, $arglist_next49 = 0, $arglist_next52 = 0, $arglist_next55 = 0, $arglist_next58 = 0, $arglist_next61 = 0, $arglist_next64 = 0, $arglist_next7 = 0, $expanded = 0, $expanded100 = 0, $expanded102 = 0, $expanded103 = 0;
 var $expanded104 = 0, $expanded106 = 0, $expanded107 = 0, $expanded109 = 0, $expanded11 = 0, $expanded110 = 0, $expanded111 = 0, $expanded113 = 0, $expanded114 = 0, $expanded116 = 0, $expanded117 = 0, $expanded118 = 0, $expanded12 = 0, $expanded120 = 0, $expanded121 = 0, $expanded123 = 0, $expanded124 = 0, $expanded125 = 0, $expanded127 = 0, $expanded128 = 0;
 var $expanded13 = 0, $expanded130 = 0, $expanded131 = 0, $expanded132 = 0, $expanded134 = 0, $expanded135 = 0, $expanded137 = 0, $expanded138 = 0, $expanded139 = 0, $expanded141 = 0, $expanded142 = 0, $expanded144 = 0, $expanded145 = 0, $expanded146 = 0, $expanded148 = 0, $expanded149 = 0, $expanded15 = 0, $expanded151 = 0, $expanded152 = 0, $expanded153 = 0;
 var $expanded16 = 0, $expanded18 = 0, $expanded19 = 0, $expanded2 = 0, $expanded20 = 0, $expanded22 = 0, $expanded23 = 0, $expanded25 = 0, $expanded26 = 0, $expanded27 = 0, $expanded29 = 0, $expanded30 = 0, $expanded32 = 0, $expanded33 = 0, $expanded34 = 0, $expanded36 = 0, $expanded37 = 0, $expanded39 = 0, $expanded4 = 0, $expanded40 = 0;
 var $expanded41 = 0, $expanded43 = 0, $expanded44 = 0, $expanded46 = 0, $expanded47 = 0, $expanded48 = 0, $expanded5 = 0, $expanded50 = 0, $expanded51 = 0, $expanded53 = 0, $expanded54 = 0, $expanded55 = 0, $expanded57 = 0, $expanded58 = 0, $expanded6 = 0, $expanded60 = 0, $expanded61 = 0, $expanded62 = 0, $expanded64 = 0, $expanded65 = 0;
 var $expanded67 = 0, $expanded68 = 0, $expanded69 = 0, $expanded71 = 0, $expanded72 = 0, $expanded74 = 0, $expanded75 = 0, $expanded76 = 0, $expanded78 = 0, $expanded79 = 0, $expanded8 = 0, $expanded81 = 0, $expanded82 = 0, $expanded83 = 0, $expanded85 = 0, $expanded86 = 0, $expanded88 = 0, $expanded89 = 0, $expanded9 = 0, $expanded90 = 0;
 var $expanded92 = 0, $expanded93 = 0, $expanded95 = 0, $expanded96 = 0, $expanded97 = 0, $expanded99 = 0, $mxMmap = 0, $or$cond = 0, $rc = 0, $szMmap = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $ap = sp + 24|0;
 $rc = sp + 16|0;
 $szMmap = sp + 8|0;
 $mxMmap = sp;
 HEAP32[$1>>2] = $op;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[(236)>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = (_sqlite3MisuseError(138190)|0);
  HEAP32[$0>>2] = $4;
  $270 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($270|0);
 }
 HEAP32[$ap>>2] = $varargs;
 $5 = HEAP32[$1>>2]|0;
 do {
  switch ($5|0) {
  case 4:  {
   $arglist_current = HEAP32[$ap>>2]|0;
   $6 = $arglist_current;
   $7 = ((0) + 4|0);
   $expanded2 = $7;
   $expanded = (($expanded2) - 1)|0;
   $8 = (($6) + ($expanded))|0;
   $9 = ((0) + 4|0);
   $expanded6 = $9;
   $expanded5 = (($expanded6) - 1)|0;
   $expanded4 = $expanded5 ^ -1;
   $10 = $8 & $expanded4;
   $11 = $10;
   $12 = HEAP32[$11>>2]|0;
   $arglist_next = ((($11)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next;
   ;HEAP32[(48)>>2]=HEAP32[$12>>2]|0;HEAP32[(48)+4>>2]=HEAP32[$12+4>>2]|0;HEAP32[(48)+8>>2]=HEAP32[$12+8>>2]|0;HEAP32[(48)+12>>2]=HEAP32[$12+12>>2]|0;HEAP32[(48)+16>>2]=HEAP32[$12+16>>2]|0;HEAP32[(48)+20>>2]=HEAP32[$12+20>>2]|0;HEAP32[(48)+24>>2]=HEAP32[$12+24>>2]|0;HEAP32[(48)+28>>2]=HEAP32[$12+28>>2]|0;
   break;
  }
  case 5:  {
   $13 = HEAP32[(48)>>2]|0;
   $14 = ($13|0)==(0|0);
   if ($14) {
    _sqlite3MemSetDefault();
   }
   $arglist_current3 = HEAP32[$ap>>2]|0;
   $15 = $arglist_current3;
   $16 = ((0) + 4|0);
   $expanded9 = $16;
   $expanded8 = (($expanded9) - 1)|0;
   $17 = (($15) + ($expanded8))|0;
   $18 = ((0) + 4|0);
   $expanded13 = $18;
   $expanded12 = (($expanded13) - 1)|0;
   $expanded11 = $expanded12 ^ -1;
   $19 = $17 & $expanded11;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $arglist_next4 = ((($20)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next4;
   ;HEAP32[$21>>2]=HEAP32[(48)>>2]|0;HEAP32[$21+4>>2]=HEAP32[(48)+4>>2]|0;HEAP32[$21+8>>2]=HEAP32[(48)+8>>2]|0;HEAP32[$21+12>>2]=HEAP32[(48)+12>>2]|0;HEAP32[$21+16>>2]=HEAP32[(48)+16>>2]|0;HEAP32[$21+20>>2]=HEAP32[(48)+20>>2]|0;HEAP32[$21+24>>2]=HEAP32[(48)+24>>2]|0;HEAP32[$21+28>>2]=HEAP32[(48)+28>>2]|0;
   break;
  }
  case 9:  {
   $arglist_current6 = HEAP32[$ap>>2]|0;
   $22 = $arglist_current6;
   $23 = ((0) + 4|0);
   $expanded16 = $23;
   $expanded15 = (($expanded16) - 1)|0;
   $24 = (($22) + ($expanded15))|0;
   $25 = ((0) + 4|0);
   $expanded20 = $25;
   $expanded19 = (($expanded20) - 1)|0;
   $expanded18 = $expanded19 ^ -1;
   $26 = $24 & $expanded18;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $arglist_next7 = ((($27)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next7;
   HEAP32[8>>2] = $28;
   break;
  }
  case 6:  {
   $arglist_current9 = HEAP32[$ap>>2]|0;
   $29 = $arglist_current9;
   $30 = ((0) + 4|0);
   $expanded23 = $30;
   $expanded22 = (($expanded23) - 1)|0;
   $31 = (($29) + ($expanded22))|0;
   $32 = ((0) + 4|0);
   $expanded27 = $32;
   $expanded26 = (($expanded27) - 1)|0;
   $expanded25 = $expanded26 ^ -1;
   $33 = $31 & $expanded25;
   $34 = $33;
   $35 = HEAP32[$34>>2]|0;
   $arglist_next10 = ((($34)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next10;
   HEAP32[(200)>>2] = $35;
   $arglist_current12 = HEAP32[$ap>>2]|0;
   $36 = $arglist_current12;
   $37 = ((0) + 4|0);
   $expanded30 = $37;
   $expanded29 = (($expanded30) - 1)|0;
   $38 = (($36) + ($expanded29))|0;
   $39 = ((0) + 4|0);
   $expanded34 = $39;
   $expanded33 = (($expanded34) - 1)|0;
   $expanded32 = $expanded33 ^ -1;
   $40 = $38 & $expanded32;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $arglist_next13 = ((($41)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next13;
   HEAP32[(204)>>2] = $42;
   $arglist_current15 = HEAP32[$ap>>2]|0;
   $43 = $arglist_current15;
   $44 = ((0) + 4|0);
   $expanded37 = $44;
   $expanded36 = (($expanded37) - 1)|0;
   $45 = (($43) + ($expanded36))|0;
   $46 = ((0) + 4|0);
   $expanded41 = $46;
   $expanded40 = (($expanded41) - 1)|0;
   $expanded39 = $expanded40 ^ -1;
   $47 = $45 & $expanded39;
   $48 = $47;
   $49 = HEAP32[$48>>2]|0;
   $arglist_next16 = ((($48)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next16;
   HEAP32[(208)>>2] = $49;
   break;
  }
  case 7:  {
   $arglist_current18 = HEAP32[$ap>>2]|0;
   $50 = $arglist_current18;
   $51 = ((0) + 4|0);
   $expanded44 = $51;
   $expanded43 = (($expanded44) - 1)|0;
   $52 = (($50) + ($expanded43))|0;
   $53 = ((0) + 4|0);
   $expanded48 = $53;
   $expanded47 = (($expanded48) - 1)|0;
   $expanded46 = $expanded47 ^ -1;
   $54 = $52 & $expanded46;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $arglist_next19 = ((($55)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next19;
   HEAP32[(212)>>2] = $56;
   $arglist_current21 = HEAP32[$ap>>2]|0;
   $57 = $arglist_current21;
   $58 = ((0) + 4|0);
   $expanded51 = $58;
   $expanded50 = (($expanded51) - 1)|0;
   $59 = (($57) + ($expanded50))|0;
   $60 = ((0) + 4|0);
   $expanded55 = $60;
   $expanded54 = (($expanded55) - 1)|0;
   $expanded53 = $expanded54 ^ -1;
   $61 = $59 & $expanded53;
   $62 = $61;
   $63 = HEAP32[$62>>2]|0;
   $arglist_next22 = ((($62)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next22;
   HEAP32[(216)>>2] = $63;
   $arglist_current24 = HEAP32[$ap>>2]|0;
   $64 = $arglist_current24;
   $65 = ((0) + 4|0);
   $expanded58 = $65;
   $expanded57 = (($expanded58) - 1)|0;
   $66 = (($64) + ($expanded57))|0;
   $67 = ((0) + 4|0);
   $expanded62 = $67;
   $expanded61 = (($expanded62) - 1)|0;
   $expanded60 = $expanded61 ^ -1;
   $68 = $66 & $expanded60;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $arglist_next25 = ((($69)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next25;
   HEAP32[(220)>>2] = $70;
   break;
  }
  case 24:  {
   $71 = (_sqlite3HeaderSizeBtree()|0);
   $72 = (_sqlite3HeaderSizePcache()|0);
   $73 = (($71) + ($72))|0;
   $74 = (_sqlite3HeaderSizePcache1()|0);
   $75 = (($73) + ($74))|0;
   $arglist_current27 = HEAP32[$ap>>2]|0;
   $76 = $arglist_current27;
   $77 = ((0) + 4|0);
   $expanded65 = $77;
   $expanded64 = (($expanded65) - 1)|0;
   $78 = (($76) + ($expanded64))|0;
   $79 = ((0) + 4|0);
   $expanded69 = $79;
   $expanded68 = (($expanded69) - 1)|0;
   $expanded67 = $expanded68 ^ -1;
   $80 = $78 & $expanded67;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $arglist_next28 = ((($81)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next28;
   HEAP32[$82>>2] = $75;
   break;
  }
  case 14:  {
   break;
  }
  case 15:  {
   HEAP32[$rc>>2] = 1;
   break;
  }
  case 18:  {
   $arglist_current30 = HEAP32[$ap>>2]|0;
   $83 = $arglist_current30;
   $84 = ((0) + 4|0);
   $expanded72 = $84;
   $expanded71 = (($expanded72) - 1)|0;
   $85 = (($83) + ($expanded71))|0;
   $86 = ((0) + 4|0);
   $expanded76 = $86;
   $expanded75 = (($expanded76) - 1)|0;
   $expanded74 = $expanded75 ^ -1;
   $87 = $85 & $expanded74;
   $88 = $87;
   $89 = HEAP32[$88>>2]|0;
   $arglist_next31 = ((($88)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next31;
   dest=(116); src=$89; stop=dest+52|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   break;
  }
  case 19:  {
   $90 = HEAP32[(124)>>2]|0;
   $91 = ($90|0)==(0|0);
   if ($91) {
    _sqlite3PCacheSetDefault();
   }
   $arglist_current33 = HEAP32[$ap>>2]|0;
   $92 = $arglist_current33;
   $93 = ((0) + 4|0);
   $expanded79 = $93;
   $expanded78 = (($expanded79) - 1)|0;
   $94 = (($92) + ($expanded78))|0;
   $95 = ((0) + 4|0);
   $expanded83 = $95;
   $expanded82 = (($expanded83) - 1)|0;
   $expanded81 = $expanded82 ^ -1;
   $96 = $94 & $expanded81;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $arglist_next34 = ((($97)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next34;
   dest=$98; src=(116); stop=dest+52|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   break;
  }
  case 13:  {
   $arglist_current36 = HEAP32[$ap>>2]|0;
   $99 = $arglist_current36;
   $100 = ((0) + 4|0);
   $expanded86 = $100;
   $expanded85 = (($expanded86) - 1)|0;
   $101 = (($99) + ($expanded85))|0;
   $102 = ((0) + 4|0);
   $expanded90 = $102;
   $expanded89 = (($expanded90) - 1)|0;
   $expanded88 = $expanded89 ^ -1;
   $103 = $101 & $expanded88;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $arglist_next37 = ((($104)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next37;
   HEAP32[(36)>>2] = $105;
   $arglist_current39 = HEAP32[$ap>>2]|0;
   $106 = $arglist_current39;
   $107 = ((0) + 4|0);
   $expanded93 = $107;
   $expanded92 = (($expanded93) - 1)|0;
   $108 = (($106) + ($expanded92))|0;
   $109 = ((0) + 4|0);
   $expanded97 = $109;
   $expanded96 = (($expanded97) - 1)|0;
   $expanded95 = $expanded96 ^ -1;
   $110 = $108 & $expanded95;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $arglist_next40 = ((($111)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next40;
   HEAP32[(40)>>2] = $112;
   break;
  }
  case 16:  {
   $arglist_current42 = HEAP32[$ap>>2]|0;
   $113 = $arglist_current42;
   $114 = ((0) + 4|0);
   $expanded100 = $114;
   $expanded99 = (($expanded100) - 1)|0;
   $115 = (($113) + ($expanded99))|0;
   $116 = ((0) + 4|0);
   $expanded104 = $116;
   $expanded103 = (($expanded104) - 1)|0;
   $expanded102 = $expanded103 ^ -1;
   $117 = $115 & $expanded102;
   $118 = $117;
   $119 = HEAP32[$118>>2]|0;
   $arglist_next43 = ((($118)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next43;
   HEAP32[(264)>>2] = $119;
   $arglist_current45 = HEAP32[$ap>>2]|0;
   $120 = $arglist_current45;
   $121 = ((0) + 4|0);
   $expanded107 = $121;
   $expanded106 = (($expanded107) - 1)|0;
   $122 = (($120) + ($expanded106))|0;
   $123 = ((0) + 4|0);
   $expanded111 = $123;
   $expanded110 = (($expanded111) - 1)|0;
   $expanded109 = $expanded110 ^ -1;
   $124 = $122 & $expanded109;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $arglist_next46 = ((($125)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next46;
   HEAP32[(268)>>2] = $126;
   break;
  }
  case 17:  {
   $arglist_current48 = HEAP32[$ap>>2]|0;
   $127 = $arglist_current48;
   $128 = ((0) + 4|0);
   $expanded114 = $128;
   $expanded113 = (($expanded114) - 1)|0;
   $129 = (($127) + ($expanded113))|0;
   $130 = ((0) + 4|0);
   $expanded118 = $130;
   $expanded117 = (($expanded118) - 1)|0;
   $expanded116 = $expanded117 ^ -1;
   $131 = $129 & $expanded116;
   $132 = $131;
   $133 = HEAP32[$132>>2]|0;
   $arglist_next49 = ((($132)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next49;
   HEAP32[(20)>>2] = $133;
   break;
  }
  case 20:  {
   $arglist_current51 = HEAP32[$ap>>2]|0;
   $134 = $arglist_current51;
   $135 = ((0) + 4|0);
   $expanded121 = $135;
   $expanded120 = (($expanded121) - 1)|0;
   $136 = (($134) + ($expanded120))|0;
   $137 = ((0) + 4|0);
   $expanded125 = $137;
   $expanded124 = (($expanded125) - 1)|0;
   $expanded123 = $expanded124 ^ -1;
   $138 = $136 & $expanded123;
   $139 = $138;
   $140 = HEAP32[$139>>2]|0;
   $arglist_next52 = ((($139)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next52;
   HEAP32[(24)>>2] = $140;
   break;
  }
  case 22:  {
   $arglist_current54 = HEAP32[$ap>>2]|0;
   $141 = $arglist_current54;
   $142 = ((0) + 8|0);
   $expanded128 = $142;
   $expanded127 = (($expanded128) - 1)|0;
   $143 = (($141) + ($expanded127))|0;
   $144 = ((0) + 8|0);
   $expanded132 = $144;
   $expanded131 = (($expanded132) - 1)|0;
   $expanded130 = $expanded131 ^ -1;
   $145 = $143 & $expanded130;
   $146 = $145;
   $147 = $146;
   $148 = $147;
   $149 = HEAP32[$148>>2]|0;
   $150 = (($147) + 4)|0;
   $151 = $150;
   $152 = HEAP32[$151>>2]|0;
   $arglist_next55 = ((($146)) + 8|0);
   HEAP32[$ap>>2] = $arglist_next55;
   $153 = $szMmap;
   $154 = $153;
   HEAP32[$154>>2] = $149;
   $155 = (($153) + 4)|0;
   $156 = $155;
   HEAP32[$156>>2] = $152;
   $arglist_current57 = HEAP32[$ap>>2]|0;
   $157 = $arglist_current57;
   $158 = ((0) + 8|0);
   $expanded135 = $158;
   $expanded134 = (($expanded135) - 1)|0;
   $159 = (($157) + ($expanded134))|0;
   $160 = ((0) + 8|0);
   $expanded139 = $160;
   $expanded138 = (($expanded139) - 1)|0;
   $expanded137 = $expanded138 ^ -1;
   $161 = $159 & $expanded137;
   $162 = $161;
   $163 = $162;
   $164 = $163;
   $165 = HEAP32[$164>>2]|0;
   $166 = (($163) + 4)|0;
   $167 = $166;
   $168 = HEAP32[$167>>2]|0;
   $arglist_next58 = ((($162)) + 8|0);
   HEAP32[$ap>>2] = $arglist_next58;
   $169 = $mxMmap;
   $170 = $169;
   HEAP32[$170>>2] = $165;
   $171 = (($169) + 4)|0;
   $172 = $171;
   HEAP32[$172>>2] = $168;
   $173 = $mxMmap;
   $174 = $173;
   $175 = HEAP32[$174>>2]|0;
   $176 = (($173) + 4)|0;
   $177 = $176;
   $178 = HEAP32[$177>>2]|0;
   $179 = ($178|0)<(0);
   $180 = $mxMmap;
   $181 = $180;
   $182 = HEAP32[$181>>2]|0;
   $183 = (($180) + 4)|0;
   $184 = $183;
   $185 = HEAP32[$184>>2]|0;
   $186 = ($185|0)>(0);
   $187 = ($182>>>0)>(0);
   $188 = ($185|0)==(0);
   $189 = $188 & $187;
   $190 = $186 | $189;
   $or$cond = $179 | $190;
   $191 = $or$cond ? 0 : $165;
   $192 = $or$cond ? 0 : $168;
   $193 = $mxMmap;
   $194 = $193;
   HEAP32[$194>>2] = $191;
   $195 = (($193) + 4)|0;
   $196 = $195;
   HEAP32[$196>>2] = $192;
   $197 = $szMmap;
   $198 = $197;
   $199 = HEAP32[$198>>2]|0;
   $200 = (($197) + 4)|0;
   $201 = $200;
   $202 = HEAP32[$201>>2]|0;
   $203 = ($202|0)<(0);
   if ($203) {
    $204 = $szMmap;
    $205 = $204;
    HEAP32[$205>>2] = 0;
    $206 = (($204) + 4)|0;
    $207 = $206;
    HEAP32[$207>>2] = 0;
   }
   $208 = $szMmap;
   $209 = $208;
   $210 = HEAP32[$209>>2]|0;
   $211 = (($208) + 4)|0;
   $212 = $211;
   $213 = HEAP32[$212>>2]|0;
   $214 = $mxMmap;
   $215 = $214;
   $216 = HEAP32[$215>>2]|0;
   $217 = (($214) + 4)|0;
   $218 = $217;
   $219 = HEAP32[$218>>2]|0;
   $220 = ($213|0)>($219|0);
   $221 = ($210>>>0)>($216>>>0);
   $222 = ($213|0)==($219|0);
   $223 = $222 & $221;
   $224 = $220 | $223;
   if ($224) {
    $225 = $mxMmap;
    $226 = $225;
    $227 = HEAP32[$226>>2]|0;
    $228 = (($225) + 4)|0;
    $229 = $228;
    $230 = HEAP32[$229>>2]|0;
    $231 = $szMmap;
    $232 = $231;
    HEAP32[$232>>2] = $227;
    $233 = (($231) + 4)|0;
    $234 = $233;
    HEAP32[$234>>2] = $230;
   }
   $235 = $mxMmap;
   $236 = $235;
   $237 = HEAP32[$236>>2]|0;
   $238 = (($235) + 4)|0;
   $239 = $238;
   $240 = HEAP32[$239>>2]|0;
   $241 = (192);
   $242 = $241;
   HEAP32[$242>>2] = $237;
   $243 = (($241) + 4)|0;
   $244 = $243;
   HEAP32[$244>>2] = $240;
   $245 = $szMmap;
   $246 = $245;
   $247 = HEAP32[$246>>2]|0;
   $248 = (($245) + 4)|0;
   $249 = $248;
   $250 = HEAP32[$249>>2]|0;
   $251 = (184);
   $252 = $251;
   HEAP32[$252>>2] = $247;
   $253 = (($251) + 4)|0;
   $254 = $253;
   HEAP32[$254>>2] = $250;
   break;
  }
  case 25:  {
   $arglist_current60 = HEAP32[$ap>>2]|0;
   $255 = $arglist_current60;
   $256 = ((0) + 4|0);
   $expanded142 = $256;
   $expanded141 = (($expanded142) - 1)|0;
   $257 = (($255) + ($expanded141))|0;
   $258 = ((0) + 4|0);
   $expanded146 = $258;
   $expanded145 = (($expanded146) - 1)|0;
   $expanded144 = $expanded145 ^ -1;
   $259 = $257 & $expanded144;
   $260 = $259;
   $261 = HEAP32[$260>>2]|0;
   $arglist_next61 = ((($260)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next61;
   HEAP32[(232)>>2] = $261;
   break;
  }
  case 26:  {
   $arglist_current63 = HEAP32[$ap>>2]|0;
   $262 = $arglist_current63;
   $263 = ((0) + 4|0);
   $expanded149 = $263;
   $expanded148 = (($expanded149) - 1)|0;
   $264 = (($262) + ($expanded148))|0;
   $265 = ((0) + 4|0);
   $expanded153 = $265;
   $expanded152 = (($expanded153) - 1)|0;
   $expanded151 = $expanded152 ^ -1;
   $266 = $264 & $expanded151;
   $267 = $266;
   $268 = HEAP32[$267>>2]|0;
   $arglist_next64 = ((($267)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next64;
   HEAP32[(44)>>2] = $268;
   break;
  }
  default: {
   HEAP32[$rc>>2] = 1;
  }
  }
 } while(0);
 $269 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $269;
 $270 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($270|0);
}
function _sqlite3_db_release_memory($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pBt = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pBt>>2] = $13;
  $14 = HEAP32[$pBt>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_sqlite3BtreePager($16)|0);
   HEAP32[$pPager>>2] = $17;
   $18 = HEAP32[$pPager>>2]|0;
   _sqlite3PagerShrink($18);
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 STACKTOP = sp;return 0;
}
function _sqlite3_db_config($db,$op,$varargs) {
 $db = $db|0;
 $op = $op|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ap = 0, $arglist_current = 0, $arglist_current11 = 0, $arglist_current14 = 0, $arglist_current2 = 0, $arglist_current5 = 0, $arglist_current8 = 0, $arglist_next = 0, $arglist_next12 = 0, $arglist_next15 = 0, $arglist_next3 = 0, $arglist_next6 = 0, $arglist_next9 = 0, $cnt = 0;
 var $expanded = 0, $expanded10 = 0, $expanded12 = 0, $expanded13 = 0, $expanded14 = 0, $expanded16 = 0, $expanded17 = 0, $expanded19 = 0, $expanded20 = 0, $expanded21 = 0, $expanded23 = 0, $expanded24 = 0, $expanded26 = 0, $expanded27 = 0, $expanded28 = 0, $expanded3 = 0, $expanded30 = 0, $expanded31 = 0, $expanded33 = 0, $expanded34 = 0;
 var $expanded35 = 0, $expanded37 = 0, $expanded38 = 0, $expanded40 = 0, $expanded41 = 0, $expanded42 = 0, $expanded5 = 0, $expanded6 = 0, $expanded7 = 0, $expanded9 = 0, $i = 0, $oldFlags = 0, $onoff = 0, $pBuf = 0, $pRes = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $ap = sp + 32|0;
 $rc = sp + 28|0;
 $pBuf = sp + 24|0;
 $sz = sp + 20|0;
 $cnt = sp + 16|0;
 $i = sp + 12|0;
 $onoff = sp + 8|0;
 $pRes = sp + 4|0;
 $oldFlags = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$1>>2]|0;
 switch ($2|0) {
 case 1000:  {
  $arglist_current = HEAP32[$ap>>2]|0;
  $3 = $arglist_current;
  $4 = ((0) + 4|0);
  $expanded3 = $4;
  $expanded = (($expanded3) - 1)|0;
  $5 = (($3) + ($expanded))|0;
  $6 = ((0) + 4|0);
  $expanded7 = $6;
  $expanded6 = (($expanded7) - 1)|0;
  $expanded5 = $expanded6 ^ -1;
  $7 = $5 & $expanded5;
  $8 = $7;
  $9 = HEAP32[$8>>2]|0;
  $arglist_next = ((($8)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$12>>2] = $9;
  HEAP32[$rc>>2] = 0;
  $105 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($105|0);
  break;
 }
 case 1001:  {
  $arglist_current2 = HEAP32[$ap>>2]|0;
  $13 = $arglist_current2;
  $14 = ((0) + 4|0);
  $expanded10 = $14;
  $expanded9 = (($expanded10) - 1)|0;
  $15 = (($13) + ($expanded9))|0;
  $16 = ((0) + 4|0);
  $expanded14 = $16;
  $expanded13 = (($expanded14) - 1)|0;
  $expanded12 = $expanded13 ^ -1;
  $17 = $15 & $expanded12;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $arglist_next3 = ((($18)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next3;
  HEAP32[$pBuf>>2] = $19;
  $arglist_current5 = HEAP32[$ap>>2]|0;
  $20 = $arglist_current5;
  $21 = ((0) + 4|0);
  $expanded17 = $21;
  $expanded16 = (($expanded17) - 1)|0;
  $22 = (($20) + ($expanded16))|0;
  $23 = ((0) + 4|0);
  $expanded21 = $23;
  $expanded20 = (($expanded21) - 1)|0;
  $expanded19 = $expanded20 ^ -1;
  $24 = $22 & $expanded19;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $arglist_next6 = ((($25)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next6;
  HEAP32[$sz>>2] = $26;
  $arglist_current8 = HEAP32[$ap>>2]|0;
  $27 = $arglist_current8;
  $28 = ((0) + 4|0);
  $expanded24 = $28;
  $expanded23 = (($expanded24) - 1)|0;
  $29 = (($27) + ($expanded23))|0;
  $30 = ((0) + 4|0);
  $expanded28 = $30;
  $expanded27 = (($expanded28) - 1)|0;
  $expanded26 = $expanded27 ^ -1;
  $31 = $29 & $expanded26;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $arglist_next9 = ((($32)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next9;
  HEAP32[$cnt>>2] = $33;
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$pBuf>>2]|0;
  $36 = HEAP32[$sz>>2]|0;
  $37 = HEAP32[$cnt>>2]|0;
  $38 = (_setupLookaside($34,$35,$36,$37)|0);
  HEAP32[$rc>>2] = $38;
  $105 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($105|0);
  break;
 }
 default: {
  HEAP32[$rc>>2] = 1;
  HEAP32[$i>>2] = 0;
  while(1) {
   $39 = HEAP32[$i>>2]|0;
   $40 = ($39>>>0)<(4);
   if (!($40)) {
    label = 17;
    break;
   }
   $41 = HEAP32[$i>>2]|0;
   $42 = (1548 + ($41<<3)|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = ($43|0)==($44|0);
   if ($45) {
    break;
   }
   $103 = HEAP32[$i>>2]|0;
   $104 = (($103) + 1)|0;
   HEAP32[$i>>2] = $104;
  }
  if ((label|0) == 17) {
   $105 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($105|0);
  }
  $arglist_current11 = HEAP32[$ap>>2]|0;
  $46 = $arglist_current11;
  $47 = ((0) + 4|0);
  $expanded31 = $47;
  $expanded30 = (($expanded31) - 1)|0;
  $48 = (($46) + ($expanded30))|0;
  $49 = ((0) + 4|0);
  $expanded35 = $49;
  $expanded34 = (($expanded35) - 1)|0;
  $expanded33 = $expanded34 ^ -1;
  $50 = $48 & $expanded33;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $arglist_next12 = ((($51)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next12;
  HEAP32[$onoff>>2] = $52;
  $arglist_current14 = HEAP32[$ap>>2]|0;
  $53 = $arglist_current14;
  $54 = ((0) + 4|0);
  $expanded38 = $54;
  $expanded37 = (($expanded38) - 1)|0;
  $55 = (($53) + ($expanded37))|0;
  $56 = ((0) + 4|0);
  $expanded42 = $56;
  $expanded41 = (($expanded42) - 1)|0;
  $expanded40 = $expanded41 ^ -1;
  $57 = $55 & $expanded40;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $arglist_next15 = ((($58)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next15;
  HEAP32[$pRes>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  $61 = ((($60)) + 24|0);
  $62 = HEAP32[$61>>2]|0;
  HEAP32[$oldFlags>>2] = $62;
  $63 = HEAP32[$onoff>>2]|0;
  $64 = ($63|0)>(0);
  if ($64) {
   $65 = HEAP32[$i>>2]|0;
   $66 = (1548 + ($65<<3)|0);
   $67 = ((($66)) + 4|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = HEAP32[$0>>2]|0;
   $70 = ((($69)) + 24|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = $71 | $68;
   HEAP32[$70>>2] = $72;
  } else {
   $73 = HEAP32[$onoff>>2]|0;
   $74 = ($73|0)==(0);
   if ($74) {
    $75 = HEAP32[$i>>2]|0;
    $76 = (1548 + ($75<<3)|0);
    $77 = ((($76)) + 4|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = $78 ^ -1;
    $80 = HEAP32[$0>>2]|0;
    $81 = ((($80)) + 24|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = $82 & $79;
    HEAP32[$81>>2] = $83;
   }
  }
  $84 = HEAP32[$oldFlags>>2]|0;
  $85 = HEAP32[$0>>2]|0;
  $86 = ((($85)) + 24|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = ($84|0)!=($87|0);
  if ($88) {
   $89 = HEAP32[$0>>2]|0;
   _sqlite3ExpirePreparedStatements($89);
  }
  $90 = HEAP32[$pRes>>2]|0;
  $91 = ($90|0)!=(0|0);
  if ($91) {
   $92 = HEAP32[$0>>2]|0;
   $93 = ((($92)) + 24|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = HEAP32[$i>>2]|0;
   $96 = (1548 + ($95<<3)|0);
   $97 = ((($96)) + 4|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = $94 & $98;
   $100 = ($99|0)!=(0);
   $101 = $100&1;
   $102 = HEAP32[$pRes>>2]|0;
   HEAP32[$102>>2] = $101;
  }
  HEAP32[$rc>>2] = 0;
  $105 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($105|0);
 }
 }
 return (0)|0;
}
function _sqlite3_last_insert_rowid($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = $2;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 tempRet0 = ($8);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3_changes($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 88|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_total_changes($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 92|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3_close($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3Close($1,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_close_v2($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3Close($1,1)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3_busy_handler($db,$xBusy,$pArg) {
 $db = $db|0;
 $xBusy = $xBusy|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $xBusy;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 380|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 380|0);
 $9 = ((($8)) + 4|0);
 HEAP32[$9>>2] = $6;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 380|0);
 $12 = ((($11)) + 8|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 428|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return 0;
}
function _sqlite3_busy_timeout($db,$ms) {
 $db = $db|0;
 $ms = $ms|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $ms;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 $4 = HEAP32[$0>>2]|0;
 if ($3) {
  $5 = HEAP32[$0>>2]|0;
  (_sqlite3_busy_handler($4,178,$5)|0);
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 428|0);
  HEAP32[$8>>2] = $6;
  STACKTOP = sp;return 0;
 } else {
  (_sqlite3_busy_handler($4,0,0)|0);
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _sqlite3_create_function($db,$zFunc,$nArg,$enc,$p,$xSFunc,$xStep,$xFinal) {
 $db = $db|0;
 $zFunc = $zFunc|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $p = $p|0;
 $xSFunc = $xSFunc|0;
 $xStep = $xStep|0;
 $xFinal = $xFinal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $7 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $nArg;
 HEAP32[$3>>2] = $enc;
 HEAP32[$4>>2] = $p;
 HEAP32[$5>>2] = $xSFunc;
 HEAP32[$6>>2] = $xStep;
 HEAP32[$7>>2] = $xFinal;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = HEAP32[$5>>2]|0;
 $14 = HEAP32[$6>>2]|0;
 $15 = HEAP32[$7>>2]|0;
 $16 = (_sqlite3_create_function_v2($8,$9,$10,$11,$12,$13,$14,$15,0)|0);
 STACKTOP = sp;return ($16|0);
}
function _sqlite3_create_function_v2($db,$zFunc,$nArg,$enc,$p,$xSFunc,$xStep,$xFinal,$xDestroy) {
 $db = $db|0;
 $zFunc = $zFunc|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $p = $p|0;
 $xSFunc = $xSFunc|0;
 $xStep = $xStep|0;
 $xFinal = $xFinal|0;
 $xDestroy = $xDestroy|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $7 = sp + 12|0;
 $8 = sp + 8|0;
 $rc = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $nArg;
 HEAP32[$3>>2] = $enc;
 HEAP32[$4>>2] = $p;
 HEAP32[$5>>2] = $xSFunc;
 HEAP32[$6>>2] = $xStep;
 HEAP32[$7>>2] = $xFinal;
 HEAP32[$8>>2] = $xDestroy;
 HEAP32[$rc>>2] = 1;
 HEAP32[$pArg>>2] = 0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$0>>2]|0;
   $12 = (_sqlite3DbMallocZero($11,12,0)|0);
   HEAP32[$pArg>>2] = $12;
   $13 = HEAP32[$pArg>>2]|0;
   $14 = ($13|0)!=(0|0);
   $15 = HEAP32[$8>>2]|0;
   if ($14) {
    $17 = HEAP32[$pArg>>2]|0;
    $18 = ((($17)) + 4|0);
    HEAP32[$18>>2] = $15;
    $19 = HEAP32[$4>>2]|0;
    $20 = HEAP32[$pArg>>2]|0;
    $21 = ((($20)) + 8|0);
    HEAP32[$21>>2] = $19;
    label = 5;
    break;
   } else {
    $16 = HEAP32[$4>>2]|0;
    FUNCTION_TABLE_vi[$15 & 255]($16);
    break;
   }
  } else {
   label = 5;
  }
 } while(0);
 if ((label|0) == 5) {
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  $27 = HEAP32[$5>>2]|0;
  $28 = HEAP32[$6>>2]|0;
  $29 = HEAP32[$7>>2]|0;
  $30 = HEAP32[$pArg>>2]|0;
  $31 = (_sqlite3CreateFunc($22,$23,$24,$25,$26,$27,$28,$29,$30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$pArg>>2]|0;
  $33 = ($32|0)!=(0|0);
  if ($33) {
   $34 = HEAP32[$pArg>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[$8>>2]|0;
    $38 = HEAP32[$4>>2]|0;
    FUNCTION_TABLE_vi[$37 & 255]($38);
    $39 = HEAP32[$0>>2]|0;
    $40 = HEAP32[$pArg>>2]|0;
    _sqlite3DbFree($39,$40);
   }
  }
 }
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$rc>>2]|0;
 $43 = (_sqlite3ApiExit($41,$42)|0);
 HEAP32[$rc>>2] = $43;
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3_overload_function($db,$zName,$nArg) {
 $db = $db|0;
 $zName = $zName|0;
 $nArg = $nArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $nArg;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3FindFunction($3,$4,$5,1,0)|0);
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_sqlite3CreateFunc($8,$9,$10,1,0,224,0,0,0)|0);
  HEAP32[$rc>>2] = $11;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$rc>>2]|0;
 $14 = (_sqlite3ApiExit($12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3_wal_autocheckpoint($db,$nFrame) {
 $db = $db|0;
 $nFrame = $nFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nFrame;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 $4 = HEAP32[$0>>2]|0;
 if ($3) {
  $5 = HEAP32[$1>>2]|0;
  $6 = $5;
  (_sqlite3_wal_hook($4,139,$6)|0);
  STACKTOP = sp;return 0;
 } else {
  (_sqlite3_wal_hook($4,0,0)|0);
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _sqlite3_wal_hook($db,$xCallback,$pArg) {
 $db = $db|0;
 $xCallback = $xCallback|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $xCallback;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 228|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pRet>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 224|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 228|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3_wal_checkpoint_v2($db,$zDb,$eMode,$pnLog,$pnCkpt) {
 $db = $db|0;
 $zDb = $zDb|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $rc = sp + 8|0;
 $iDb = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $eMode;
 HEAP32[$4>>2] = $pnLog;
 HEAP32[$5>>2] = $pnCkpt;
 HEAP32[$iDb>>2] = 10;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$4>>2]|0;
  HEAP32[$8>>2] = -1;
 }
 $9 = HEAP32[$5>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$5>>2]|0;
  HEAP32[$11>>2] = -1;
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = ($12|0)<(0);
 $14 = HEAP32[$3>>2]|0;
 $15 = ($14|0)>(3);
 $or$cond = $13 | $15;
 if ($or$cond) {
  HEAP32[$0>>2] = 21;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP8[$18>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$2>>2]|0;
   $24 = (_sqlite3FindDbName($22,$23)|0);
   HEAP32[$iDb>>2] = $24;
  }
 }
 $25 = HEAP32[$iDb>>2]|0;
 $26 = ($25|0)<(0);
 if ($26) {
  HEAP32[$rc>>2] = 1;
  $27 = HEAP32[$1>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $28;
  _sqlite3ErrorWithMsg($27,1,18738,$vararg_buffer);
 } else {
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 380|0);
  $31 = ((($30)) + 8|0);
  HEAP32[$31>>2] = 0;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$iDb>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  $36 = HEAP32[$5>>2]|0;
  $37 = (_sqlite3Checkpoint($32,$33,$34,$35,$36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$rc>>2]|0;
  _sqlite3Error($38,$39);
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$rc>>2]|0;
 $42 = (_sqlite3ApiExit($40,$41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3_wal_checkpoint($db,$zDb) {
 $db = $db|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3_wal_checkpoint_v2($2,$3,0,0,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3_errcode($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3SafetyCheckSickOrOk($4)|0);
  $6 = ($5|0)!=(0);
  if (!($6)) {
   $7 = (_sqlite3MisuseError(140115)|0);
   HEAP32[$0>>2] = $7;
   $22 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 69|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if (!($14)) {
   $15 = HEAP32[$1>>2]|0;
   $16 = ((($15)) + 52|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 56|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $17 & $20;
   HEAP32[$0>>2] = $21;
   $22 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 HEAP32[$0>>2] = 7;
 $22 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3_limit($db,$limitId,$newLimit) {
 $db = $db|0;
 $limitId = $limitId|0;
 $newLimit = $newLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $oldLimit = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $oldLimit = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $limitId;
 HEAP32[$3>>2] = $newLimit;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)<(0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>=(12);
 $or$cond = $5 | $7;
 if ($or$cond) {
  HEAP32[$0>>2] = -1;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 96|0);
 $11 = (($10) + ($8<<2)|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$oldLimit>>2] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ($13|0)>=(0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (1580 + ($16<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)>($18|0);
  if ($19) {
   $20 = HEAP32[$2>>2]|0;
   $21 = (1580 + ($20<<2)|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$3>>2] = $22;
  }
  $23 = HEAP32[$3>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 96|0);
  $27 = (($26) + ($24<<2)|0);
  HEAP32[$27>>2] = $23;
 }
 $28 = HEAP32[$oldLimit>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3_open($zFilename,$ppDb) {
 $zFilename = $zFilename|0;
 $ppDb = $ppDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $zFilename;
 HEAP32[$1>>2] = $ppDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_openDatabase($2,$3,6,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3_file_control($db,$zDbName,$op,$pArg) {
 $db = $db|0;
 $zDbName = $zDbName|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $pBtree = 0, $pPager = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pBtree = sp + 8|0;
 $pPager = sp + 4|0;
 $fd = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDbName;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pArg;
 HEAP32[$rc>>2] = 1;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3DbNameToBtree($4,$5)|0);
 HEAP32[$pBtree>>2] = $6;
 $7 = HEAP32[$pBtree>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $9 = HEAP32[$pBtree>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$pBtree>>2]|0;
 $11 = (_sqlite3BtreePager($10)|0);
 HEAP32[$pPager>>2] = $11;
 $12 = HEAP32[$pPager>>2]|0;
 $13 = (_sqlite3PagerFile($12)|0);
 HEAP32[$fd>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)==(7);
 if ($15) {
  $16 = HEAP32[$fd>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = ($18|0)==(27);
 if ($19) {
  $20 = HEAP32[$pPager>>2]|0;
  $21 = (_sqlite3PagerVfs($20)|0);
  $22 = HEAP32[$3>>2]|0;
  HEAP32[$22>>2] = $21;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)==(28);
 if ($24) {
  $25 = HEAP32[$pPager>>2]|0;
  $26 = (_sqlite3PagerJrnlFile($25)|0);
  $27 = HEAP32[$3>>2]|0;
  HEAP32[$27>>2] = $26;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $28 = HEAP32[$fd>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  $31 = HEAP32[$fd>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  $34 = (_sqlite3OsFileControl($31,$32,$33)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  HEAP32[$rc>>2] = 12;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _sqlite3_uri_parameter($zFilename,$zParam) {
 $zFilename = $zFilename|0;
 $zParam = $zParam|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $zParam;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 $or$cond = $4 | $6;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 $9 = (($8) + 1)|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) + ($9)|0);
 HEAP32[$1>>2] = $11;
 while(1) {
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if (!($14)) {
   label = 8;
   break;
  }
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (_strcmp($15,$16)|0);
  HEAP32[$x>>2] = $17;
  $18 = HEAP32[$1>>2]|0;
  $19 = (_sqlite3Strlen30($18)|0);
  $20 = (($19) + 1)|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (($21) + ($20)|0);
  HEAP32[$1>>2] = $22;
  $23 = HEAP32[$x>>2]|0;
  $24 = ($23|0)==(0);
  $25 = HEAP32[$1>>2]|0;
  if ($24) {
   label = 6;
   break;
  }
  $26 = (_sqlite3Strlen30($25)|0);
  $27 = (($26) + 1)|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + ($27)|0);
  HEAP32[$1>>2] = $29;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $25;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _sqlite3_uri_boolean($zFilename,$zParam,$bDflt) {
 $zFilename = $zFilename|0;
 $zParam = $zParam|0;
 $bDflt = $bDflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $zFilename;
 HEAP32[$1>>2] = $zParam;
 HEAP32[$2>>2] = $bDflt;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3_uri_parameter($3,$4)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = $7&1;
 HEAP32[$2>>2] = $8;
 $9 = HEAP32[$z>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$z>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = $12&255;
  $14 = (_sqlite3GetBoolean($11,$13)|0);
  $15 = $14&255;
  $17 = $15;
  STACKTOP = sp;return ($17|0);
 } else {
  $16 = HEAP32[$2>>2]|0;
  $17 = $16;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3Strlen30($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 0;
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_strlen($4)|0);
  $6 = 1073741823 & $5;
  HEAP32[$0>>2] = $6;
 }
 $7 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3IsIdChar($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $c;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 $3 = (18403 + ($2)|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 70;
 $7 = ($6|0)!=(0);
 $8 = $7&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3MisuseError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_reportError(21,$1,45606)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3Pcache1Mutex() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(1712)>>2]|0;
 return ($0|0);
}
function _sqlite3MallocMutex() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[288>>2]|0;
 return ($0|0);
}
function _sqlite3BtreeEnterAll($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$p>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$p>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ((($19)) + 4|0);
   HEAP32[$20>>2] = $16;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 STACKTOP = sp;return;
}
function _sqlite3BtreePager($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3DeleteTrigger($db,$pTrigger) {
 $db = $db|0;
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTrigger;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3DeleteTriggerStep($4,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 _sqlite3DbFree($8,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3DbFree($11,$14);
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3ExprDelete($15,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 16|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3IdListDelete($19,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($23,$24);
 STACKTOP = sp;return;
}
function _sqlite3DeleteTable($db,$pTable) {
 $db = $db|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 456|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 36|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = (($12) + -1)<<16>>16;
  HEAP16[$11>>1] = $13;
  $14 = $13&65535;
  $15 = ($14|0)>(0);
  if ($15) {
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 _deleteTable($16,$17);
 STACKTOP = sp;return;
}
function _sqlite3VdbeClearObject($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $i = 0, $pNext = 0, $pSub = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pSub = sp + 8|0;
 $pNext = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 100|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 140|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8<<1;
 _releaseMemArray($4,$9);
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 200|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pSub>>2] = $12;
 while(1) {
  $13 = HEAP32[$pSub>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$pSub>>2]|0;
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pNext>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$pSub>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$pSub>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  _vdbeFreeOpArray($18,$20,$23);
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$pSub>>2]|0;
  _sqlite3DbFree($24,$25);
  $26 = HEAP32[$pNext>>2]|0;
  HEAP32[$pSub>>2] = $26;
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(381479589);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 116|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 16|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36 << 16 >> 16;
  _releaseMemArray($33,$37);
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 18|0);
  $40 = HEAP16[$39>>1]|0;
  $41 = $40 << 16 >> 16;
  $42 = (($41) - 1)|0;
  HEAP32[$i>>2] = $42;
  while(1) {
   $43 = HEAP32[$i>>2]|0;
   $44 = ($43|0)>=(0);
   $45 = HEAP32[$0>>2]|0;
   if (!($44)) {
    break;
   }
   $46 = HEAP32[$i>>2]|0;
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 120|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + ($46<<2)|0);
   $51 = HEAP32[$50>>2]|0;
   _sqlite3DbFree($45,$51);
   $52 = HEAP32[$i>>2]|0;
   $53 = (($52) + -1)|0;
   HEAP32[$i>>2] = $53;
  }
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 120|0);
  $56 = HEAP32[$55>>2]|0;
  _sqlite3DbFree($45,$56);
  $57 = HEAP32[$0>>2]|0;
  $58 = HEAP32[$1>>2]|0;
  $59 = ((($58)) + 180|0);
  $60 = HEAP32[$59>>2]|0;
  _sqlite3DbFree($57,$60);
 }
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$1>>2]|0;
 $63 = ((($62)) + 88|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 136|0);
 $67 = HEAP32[$66>>2]|0;
 _vdbeFreeOpArray($61,$64,$67);
 $68 = HEAP32[$0>>2]|0;
 $69 = HEAP32[$1>>2]|0;
 $70 = ((($69)) + 100|0);
 $71 = HEAP32[$70>>2]|0;
 _sqlite3DbFree($68,$71);
 $72 = HEAP32[$0>>2]|0;
 $73 = HEAP32[$1>>2]|0;
 $74 = ((($73)) + 176|0);
 $75 = HEAP32[$74>>2]|0;
 _sqlite3DbFree($72,$75);
 STACKTOP = sp;return;
}
function _sqlite3DbFree($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBuf = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 456|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  if ($9) {
   _measureAllocationSize($10,$11);
   STACKTOP = sp;return;
  }
  $12 = (_isLookaside($10,$11)|0);
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   HEAP32[$pBuf>>2] = $14;
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 256|0);
   $17 = ((($16)) + 28|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$pBuf>>2]|0;
   HEAP32[$19>>2] = $18;
   $20 = HEAP32[$pBuf>>2]|0;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 256|0);
   $23 = ((($22)) + 28|0);
   HEAP32[$23>>2] = $20;
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 256|0);
   $26 = ((($25)) + 8|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + -1)|0;
   HEAP32[$26>>2] = $28;
   STACKTOP = sp;return;
  }
 }
 $29 = HEAP32[$1>>2]|0;
 _sqlite3_free($29);
 STACKTOP = sp;return;
}
function _sqlite3MallocInit() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pSlot = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $n = sp + 8|0;
 $sz = sp + 4|0;
 $pSlot = sp;
 $0 = HEAP32[(48)>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  _sqlite3MemSetDefault();
 }
 ;HEAP32[288>>2]=0|0;HEAP32[288+4>>2]=0|0;HEAP32[288+8>>2]=0|0;HEAP32[288+12>>2]=0|0;HEAP32[288+16>>2]=0|0;HEAP32[288+20>>2]=0|0;HEAP32[288+24>>2]=0|0;HEAP32[288+28>>2]=0|0;
 HEAP32[288>>2] = (8);
 $2 = HEAP32[(200)>>2]|0;
 $3 = ($2|0)!=(0|0);
 $4 = HEAP32[(204)>>2]|0;
 $5 = ($4|0)>=(100);
 $or$cond = $3 & $5;
 $6 = HEAP32[(208)>>2]|0;
 $7 = ($6|0)>(0);
 $or$cond3 = $or$cond & $7;
 if ($or$cond3) {
  $8 = HEAP32[(204)>>2]|0;
  $9 = $8 & -8;
  HEAP32[$sz>>2] = $9;
  $10 = HEAP32[$sz>>2]|0;
  HEAP32[(204)>>2] = $10;
  $11 = HEAP32[(200)>>2]|0;
  HEAP32[$pSlot>>2] = $11;
  $12 = HEAP32[(208)>>2]|0;
  HEAP32[$n>>2] = $12;
  $13 = HEAP32[$pSlot>>2]|0;
  HEAP32[(308)>>2] = $13;
  $14 = HEAP32[$n>>2]|0;
  HEAP32[(312)>>2] = $14;
  HEAP32[$i>>2] = 0;
  while(1) {
   $15 = HEAP32[$i>>2]|0;
   $16 = HEAP32[$n>>2]|0;
   $17 = (($16) - 1)|0;
   $18 = ($15|0)<($17|0);
   if (!($18)) {
    break;
   }
   $19 = HEAP32[$sz>>2]|0;
   $20 = HEAP32[$pSlot>>2]|0;
   $21 = (($20) + ($19)|0);
   $22 = HEAP32[$pSlot>>2]|0;
   HEAP32[$22>>2] = $21;
   $23 = HEAP32[$pSlot>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$pSlot>>2] = $24;
   $25 = HEAP32[$i>>2]|0;
   $26 = (($25) + 1)|0;
   HEAP32[$i>>2] = $26;
  }
  $27 = HEAP32[$pSlot>>2]|0;
  HEAP32[$27>>2] = 0;
  $28 = HEAP32[$pSlot>>2]|0;
  $29 = ((($28)) + 4|0);
  HEAP32[(304)>>2] = $29;
 } else {
  HEAP32[(304)>>2] = 0;
  HEAP32[(200)>>2] = 0;
  HEAP32[(204)>>2] = 0;
  HEAP32[(208)>>2] = 0;
 }
 $30 = HEAP32[(212)>>2]|0;
 $31 = ($30|0)==(0|0);
 $32 = HEAP32[(216)>>2]|0;
 $33 = ($32|0)<(512);
 $or$cond5 = $31 | $33;
 $34 = HEAP32[(220)>>2]|0;
 $35 = ($34|0)<=(0);
 $or$cond7 = $or$cond5 | $35;
 if ($or$cond7) {
  HEAP32[(212)>>2] = 0;
  HEAP32[(216)>>2] = 0;
 }
 $36 = HEAP32[(68)>>2]|0;
 $37 = HEAP32[(76)>>2]|0;
 $38 = (FUNCTION_TABLE_ii[$36 & 255]($37)|0);
 HEAP32[$rc>>2] = $38;
 $39 = HEAP32[$rc>>2]|0;
 $40 = ($39|0)!=(0);
 if (!($40)) {
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 ;HEAP32[288>>2]=0|0;HEAP32[288+4>>2]=0|0;HEAP32[288+8>>2]=0|0;HEAP32[288+12>>2]=0|0;HEAP32[288+16>>2]=0|0;HEAP32[288+20>>2]=0|0;HEAP32[288+24>>2]=0|0;HEAP32[288+28>>2]=0|0;
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3RegisterBuiltinFunctions() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 _sqlite3AlterFunctions();
 _sqlite3RegisterDateTimeFunctions();
 _sqlite3InsertBuiltinFuncs(6956,57);
 return;
}
function _sqlite3PcacheInitialize() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(124)>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  _sqlite3PCacheSetDefault();
 }
 $2 = HEAP32[(124)>>2]|0;
 $3 = HEAP32[(120)>>2]|0;
 $4 = (FUNCTION_TABLE_ii[$2 & 255]($3)|0);
 return ($4|0);
}
function _sqlite3OsInit() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 $1 = (_sqlite3_malloc(10)|0);
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  HEAP32[$0>>2] = 7;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 } else {
  $4 = HEAP32[$p>>2]|0;
  _sqlite3_free($4);
  $5 = (_sqlite3_os_init()|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 return (0)|0;
}
function _sqlite3PCacheBufferSetup($pBuf,$sz,$n) {
 $pBuf = $pBuf|0;
 $sz = $sz|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBuf;
 HEAP32[$1>>2] = $sz;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[(1680)>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$2>>2] = 0;
  HEAP32[$1>>2] = 0;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = $7 & -8;
 HEAP32[$1>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[(1692)>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[(1720)>>2] = $10;
 HEAP32[(1696)>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)>(90);
 if ($12) {
  $16 = 10;
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13|0) / 10)&-1;
  $15 = (($14) + 1)|0;
  $16 = $15;
 }
 HEAP32[(1700)>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 HEAP32[(1704)>>2] = $17;
 HEAP32[(1716)>>2] = 0;
 HEAP32[(1724)>>2] = 0;
 while(1) {
  $18 = HEAP32[$2>>2]|0;
  $19 = (($18) + -1)|0;
  HEAP32[$2>>2] = $19;
  $20 = ($18|0)!=(0);
  $21 = HEAP32[$0>>2]|0;
  if (!($20)) {
   break;
  }
  HEAP32[$p>>2] = $21;
  $22 = HEAP32[(1716)>>2]|0;
  $23 = HEAP32[$p>>2]|0;
  HEAP32[$23>>2] = $22;
  $24 = HEAP32[$p>>2]|0;
  HEAP32[(1716)>>2] = $24;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + ($25)|0);
  HEAP32[$0>>2] = $27;
 }
 HEAP32[(1708)>>2] = $21;
 STACKTOP = sp;return;
}
function _vfsUnlink($pVfs) {
 $pVfs = $pVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVfs;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[1088>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($3|0)==($4|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[1088>>2] = $8;
  STACKTOP = sp;return;
 }
 $9 = HEAP32[1088>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[1088>>2]|0;
 HEAP32[$p>>2] = $11;
 while(1) {
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$p>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = ($18|0)!=($19|0);
   $31 = $20;
  } else {
   $31 = 0;
  }
  $21 = HEAP32[$p>>2]|0;
  $22 = ((($21)) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  if (!($31)) {
   break;
  }
  HEAP32[$p>>2] = $23;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ($23|0)==($24|0);
 if (!($25)) {
  STACKTOP = sp;return;
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$p>>2]|0;
 $30 = ((($29)) + 12|0);
 HEAP32[$30>>2] = $28;
 STACKTOP = sp;return;
}
function _sqlite3StatusValue($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = (1008 + ($1<<2)|0);
 $3 = HEAP32[$2>>2]|0;
 tempRet0 = (0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3Malloc($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $p = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 $7 = $2;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9|0)==(0);
 $14 = ($12|0)==(0);
 $15 = $13 & $14;
 $16 = $2;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21>>>0)>(0);
 $23 = ($18>>>0)>=(2147483392);
 $24 = ($21|0)==(0);
 $25 = $24 & $23;
 $26 = $22 | $25;
 $or$cond = $15 | $26;
 if ($or$cond) {
  HEAP32[$p>>2] = 0;
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $27 = HEAP32[8>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = $2;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  (_mallocWithAlarm($31,$p)|0);
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 } else {
  $35 = HEAP32[(48)>>2]|0;
  $36 = $2;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (FUNCTION_TABLE_ii[$35 & 255]($38)|0);
  HEAP32[$p>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 return (0)|0;
}
function _sqlite3MallocSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[(60)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (FUNCTION_TABLE_ii[$1 & 255]($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3StatusDown($op,$N) {
 $op = $op|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (1008 + ($3<<2)|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) - ($2))|0;
 HEAP32[$4>>2] = $6;
 STACKTOP = sp;return;
}
function _sqlite3Realloc($pOld,$0,$1) {
 $pOld = $pOld|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $12 = 0;
 var $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nDiff = 0, $nNew = 0, $nOld = 0, $or$cond = 0, $pNew = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp;
 $nOld = sp + 20|0;
 $nNew = sp + 16|0;
 $nDiff = sp + 12|0;
 $pNew = sp + 8|0;
 HEAP32[$3>>2] = $pOld;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 if ($10) {
  $17 = (_sqlite3Malloc($13,$16)|0);
  HEAP32[$2>>2] = $17;
  $116 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $18 = ($13|0)==(0);
 $19 = ($16|0)==(0);
 $20 = $18 & $19;
 if ($20) {
  $21 = HEAP32[$3>>2]|0;
  _sqlite3_free($21);
  HEAP32[$2>>2] = 0;
  $116 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $22 = $4;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27>>>0)>(0);
 $29 = ($24>>>0)>=(2147483392);
 $30 = ($27|0)==(0);
 $31 = $30 & $29;
 $32 = $28 | $31;
 if ($32) {
  HEAP32[$2>>2] = 0;
  $116 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $33 = HEAP32[$3>>2]|0;
 $34 = (_sqlite3MallocSize($33)|0);
 HEAP32[$nOld>>2] = $34;
 $35 = HEAP32[(64)>>2]|0;
 $36 = $4;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (FUNCTION_TABLE_ii[$35 & 255]($38)|0);
 HEAP32[$nNew>>2] = $42;
 $43 = HEAP32[$nOld>>2]|0;
 $44 = HEAP32[$nNew>>2]|0;
 $45 = ($43|0)==($44|0);
 do {
  if ($45) {
   $46 = HEAP32[$3>>2]|0;
   HEAP32[$pNew>>2] = $46;
  } else {
   $47 = HEAP32[8>>2]|0;
   $48 = ($47|0)!=(0);
   if (!($48)) {
    $111 = HEAP32[(56)>>2]|0;
    $112 = HEAP32[$3>>2]|0;
    $113 = HEAP32[$nNew>>2]|0;
    $114 = (FUNCTION_TABLE_iii[$111 & 255]($112,$113)|0);
    HEAP32[$pNew>>2] = $114;
    break;
   }
   $49 = $4;
   $50 = $49;
   $51 = HEAP32[$50>>2]|0;
   $52 = (($49) + 4)|0;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   _sqlite3StatusHighwater(5,$51);
   $55 = HEAP32[$nNew>>2]|0;
   $56 = HEAP32[$nOld>>2]|0;
   $57 = (($55) - ($56))|0;
   HEAP32[$nDiff>>2] = $57;
   $58 = (_sqlite3StatusValue(0)|0);
   $59 = tempRet0;
   $60 = (296);
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = (($60) + 4)|0;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = HEAP32[$nDiff>>2]|0;
   $67 = ($66|0)<(0);
   $68 = $67 << 31 >> 31;
   $69 = (_i64Subtract(($62|0),($65|0),($66|0),($68|0))|0);
   $70 = tempRet0;
   $71 = ($59|0)>($70|0);
   $72 = ($58>>>0)>=($69>>>0);
   $73 = ($59|0)==($70|0);
   $74 = $73 & $72;
   $75 = $71 | $74;
   if ($75) {
    $76 = HEAP32[$nDiff>>2]|0;
    _sqlite3MallocAlarm($76);
   }
   $77 = HEAP32[(56)>>2]|0;
   $78 = HEAP32[$3>>2]|0;
   $79 = HEAP32[$nNew>>2]|0;
   $80 = (FUNCTION_TABLE_iii[$77 & 255]($78,$79)|0);
   HEAP32[$pNew>>2] = $80;
   $81 = HEAP32[$pNew>>2]|0;
   $82 = ($81|0)==(0|0);
   $83 = (296);
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = (($83) + 4)|0;
   $87 = $86;
   $88 = HEAP32[$87>>2]|0;
   $89 = ($88|0)>(0);
   $90 = ($85>>>0)>(0);
   $91 = ($88|0)==(0);
   $92 = $91 & $90;
   $93 = $89 | $92;
   $or$cond = $82 & $93;
   if ($or$cond) {
    $94 = $4;
    $95 = $94;
    $96 = HEAP32[$95>>2]|0;
    $97 = (($94) + 4)|0;
    $98 = $97;
    $99 = HEAP32[$98>>2]|0;
    _sqlite3MallocAlarm($96);
    $100 = HEAP32[(56)>>2]|0;
    $101 = HEAP32[$3>>2]|0;
    $102 = HEAP32[$nNew>>2]|0;
    $103 = (FUNCTION_TABLE_iii[$100 & 255]($101,$102)|0);
    HEAP32[$pNew>>2] = $103;
   }
   $104 = HEAP32[$pNew>>2]|0;
   $105 = ($104|0)!=(0|0);
   if ($105) {
    $106 = HEAP32[$pNew>>2]|0;
    $107 = (_sqlite3MallocSize($106)|0);
    HEAP32[$nNew>>2] = $107;
    $108 = HEAP32[$nNew>>2]|0;
    $109 = HEAP32[$nOld>>2]|0;
    $110 = (($108) - ($109))|0;
    _sqlite3StatusUp(0,$110);
   }
  }
 } while(0);
 $115 = HEAP32[$pNew>>2]|0;
 HEAP32[$2>>2] = $115;
 $116 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($116|0);
}
function _sqlite3StrAccumInit($p,$db,$zBase,$n,$mx) {
 $p = $p|0;
 $db = $db|0;
 $zBase = $zBase|0;
 $n = $n|0;
 $mx = $mx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zBase;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $mx;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $5;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$9>>2] = $5;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 16|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 20|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 24|0);
 HEAP8[$21>>0] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 25|0);
 HEAP8[$23>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VXPrintf($pAccum,$fmt,$ap) {
 $pAccum = $pAccum|0;
 $fmt = $fmt|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0;
 var $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0;
 var $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0;
 var $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0;
 var $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0;
 var $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0;
 var $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0;
 var $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0;
 var $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0;
 var $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0;
 var $502 = 0, $503 = 0.0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0.0, $512 = 0, $513 = 0, $514 = 0.0, $515 = 0, $516 = 0.0, $517 = 0.0, $518 = 0, $519 = 0, $52 = 0;
 var $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0.0, $536 = 0.0, $537 = 0, $538 = 0;
 var $539 = 0, $54 = 0, $540 = 0.0, $541 = 0.0, $542 = 0.0, $543 = 0.0, $544 = 0, $545 = 0, $546 = 0.0, $547 = 0, $548 = 0.0, $549 = 0.0, $55 = 0, $550 = 0.0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0.0, $556 = 0.0;
 var $557 = 0, $558 = 0, $559 = 0.0, $56 = 0, $560 = 0.0, $561 = 0.0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0.0, $567 = 0.0, $568 = 0, $569 = 0, $57 = 0, $570 = 0.0, $571 = 0.0, $572 = 0.0, $573 = 0, $574 = 0;
 var $575 = 0, $576 = 0, $577 = 0.0, $578 = 0.0, $579 = 0, $58 = 0, $580 = 0, $581 = 0.0, $582 = 0.0, $583 = 0.0, $584 = 0, $585 = 0.0, $586 = 0.0, $587 = 0, $588 = 0, $589 = 0.0, $59 = 0, $590 = 0, $591 = 0.0, $592 = 0.0;
 var $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0;
 var $610 = 0, $611 = 0.0, $612 = 0.0, $613 = 0.0, $614 = 0.0, $615 = 0, $616 = 0.0, $617 = 0.0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0;
 var $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0;
 var $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0;
 var $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0;
 var $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0;
 var $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0;
 var $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0;
 var $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0;
 var $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0;
 var $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0;
 var $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0;
 var $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0;
 var $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0;
 var $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0;
 var $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0;
 var $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0;
 var $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0;
 var $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0;
 var $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0;
 var $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0;
 var $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0;
 var $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $arglist_current = 0, $arglist_current12 = 0, $arglist_current15 = 0, $arglist_current18 = 0, $arglist_current21 = 0, $arglist_current24 = 0, $arglist_current27 = 0, $arglist_current30 = 0, $arglist_current33 = 0;
 var $arglist_current36 = 0, $arglist_current39 = 0, $arglist_current42 = 0, $arglist_current45 = 0, $arglist_current48 = 0, $arglist_current51 = 0, $arglist_next = 0, $arglist_next13 = 0, $arglist_next16 = 0, $arglist_next19 = 0, $arglist_next22 = 0, $arglist_next25 = 0, $arglist_next28 = 0, $arglist_next31 = 0, $arglist_next34 = 0, $arglist_next37 = 0, $arglist_next40 = 0, $arglist_next43 = 0, $arglist_next46 = 0, $arglist_next49 = 0;
 var $arglist_next52 = 0, $bArgList = 0, $base = 0, $buf = 0, $bufpt = 0, $c = 0, $ch = 0, $cset = 0, $done = 0, $e2 = 0, $escarg = 0, $exp = 0, $expanded = 0, $expanded101 = 0, $expanded102 = 0, $expanded104 = 0, $expanded105 = 0, $expanded106 = 0, $expanded108 = 0, $expanded109 = 0;
 var $expanded111 = 0, $expanded112 = 0, $expanded113 = 0, $expanded115 = 0, $expanded116 = 0, $expanded118 = 0, $expanded119 = 0, $expanded120 = 0, $expanded122 = 0, $expanded123 = 0, $expanded125 = 0, $expanded126 = 0, $expanded127 = 0, $expanded129 = 0, $expanded130 = 0, $expanded132 = 0, $expanded133 = 0, $expanded134 = 0, $expanded136 = 0, $expanded137 = 0;
 var $expanded139 = 0, $expanded140 = 0, $expanded141 = 0, $expanded143 = 0, $expanded144 = 0, $expanded146 = 0, $expanded147 = 0, $expanded148 = 0, $expanded150 = 0, $expanded151 = 0, $expanded153 = 0, $expanded154 = 0, $expanded155 = 0, $expanded53 = 0, $expanded55 = 0, $expanded56 = 0, $expanded57 = 0, $expanded59 = 0, $expanded60 = 0, $expanded62 = 0;
 var $expanded63 = 0, $expanded64 = 0, $expanded66 = 0, $expanded67 = 0, $expanded69 = 0, $expanded70 = 0, $expanded71 = 0, $expanded73 = 0, $expanded74 = 0, $expanded76 = 0, $expanded77 = 0, $expanded78 = 0, $expanded80 = 0, $expanded81 = 0, $expanded83 = 0, $expanded84 = 0, $expanded85 = 0, $expanded87 = 0, $expanded88 = 0, $expanded90 = 0;
 var $expanded91 = 0, $expanded92 = 0, $expanded94 = 0, $expanded95 = 0, $expanded97 = 0, $expanded98 = 0, $expanded99 = 0, $flag_alternateform = 0, $flag_altform2 = 0, $flag_blanksign = 0, $flag_dp = 0, $flag_leftjustify = 0, $flag_long = 0, $flag_longlong = 0, $flag_plussign = 0, $flag_rtz = 0, $flag_zeropad = 0, $i = 0, $i2 = 0, $idx = 0;
 var $infop = 0, $isnull = 0, $j = 0, $k = 0, $k3 = 0, $length = 0, $longvalue = 0, $n = 0, $nOut = 0, $nPad = 0, $needQuote = 0, $nsd = 0, $or$cond = 0, $or$cond10 = 0, $or$cond4 = 0, $or$cond6 = 0, $pArgList = 0, $pItem = 0, $pSrc = 0, $pToken = 0;
 var $pre = 0, $precision = 0, $prefix = 0, $px = 0, $q = 0, $realvalue = 0, $rounder = 0, $scale = 0, $useIntern = 0, $v = 0, $width = 0, $wx = 0, $x = 0, $x1 = 0, $xtype = 0, $zExtra = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 272|0;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $c = sp + 164|0;
 $bufpt = sp + 160|0;
 $precision = sp + 156|0;
 $length = sp + 152|0;
 $idx = sp + 148|0;
 $width = sp + 144|0;
 $flag_leftjustify = sp + 268|0;
 $flag_plussign = sp + 267|0;
 $flag_blanksign = sp + 266|0;
 $flag_alternateform = sp + 265|0;
 $flag_altform2 = sp + 264|0;
 $flag_zeropad = sp + 263|0;
 $flag_long = sp + 262|0;
 $flag_longlong = sp + 261|0;
 $done = sp + 260|0;
 $xtype = sp + 259|0;
 $bArgList = sp + 258|0;
 $useIntern = sp + 257|0;
 $prefix = sp + 256|0;
 $longvalue = sp + 32|0;
 $realvalue = sp + 24|0;
 $infop = sp + 140|0;
 $zOut = sp + 136|0;
 $nOut = sp + 132|0;
 $zExtra = sp + 128|0;
 $exp = sp + 124|0;
 $e2 = sp + 120|0;
 $nsd = sp + 116|0;
 $rounder = sp + 16|0;
 $flag_dp = sp + 255|0;
 $flag_rtz = sp + 254|0;
 $pArgList = sp + 112|0;
 $buf = sp + 184|0;
 $wx = sp + 108|0;
 $px = sp + 104|0;
 $v = sp + 8|0;
 $x = sp + 100|0;
 $cset = sp + 96|0;
 $base = sp + 183|0;
 $pre = sp + 92|0;
 $x1 = sp + 182|0;
 $scale = sp;
 $i = sp + 88|0;
 $nPad = sp + 84|0;
 $i2 = sp + 80|0;
 $j = sp + 76|0;
 $k = sp + 72|0;
 $n = sp + 68|0;
 $isnull = sp + 64|0;
 $needQuote = sp + 60|0;
 $ch = sp + 181|0;
 $q = sp + 180|0;
 $escarg = sp + 56|0;
 $pToken = sp + 52|0;
 $pSrc = sp + 48|0;
 $k3 = sp + 44|0;
 $pItem = sp + 40|0;
 HEAP32[$0>>2] = $pAccum;
 HEAP32[$1>>2] = $fmt;
 HEAP32[$2>>2] = $ap;
 HEAP8[$xtype>>0] = 16;
 HEAP32[$zExtra>>2] = 0;
 HEAP32[$pArgList>>2] = 0;
 HEAP32[$bufpt>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 25|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 25|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = $10 & 2;
  $12 = $11&255;
  HEAP8[$bArgList>>0] = $12;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $arglist_current = HEAP32[$15>>2]|0;
   $16 = $arglist_current;
   $17 = ((0) + 4|0);
   $expanded53 = $17;
   $expanded = (($expanded53) - 1)|0;
   $18 = (($16) + ($expanded))|0;
   $19 = ((0) + 4|0);
   $expanded57 = $19;
   $expanded56 = (($expanded57) - 1)|0;
   $expanded55 = $expanded56 ^ -1;
   $20 = $18 & $expanded55;
   $21 = $20;
   $22 = HEAP32[$21>>2]|0;
   $arglist_next = ((($21)) + 4|0);
   HEAP32[$15>>2] = $arglist_next;
   HEAP32[$pArgList>>2] = $22;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 25|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $26 & 1;
  $28 = $27&255;
  HEAP8[$useIntern>>0] = $28;
 } else {
  HEAP8[$useIntern>>0] = 0;
  HEAP8[$bArgList>>0] = 0;
 }
 L7: while(1) {
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP8[$29>>0]|0;
  $31 = $30 << 24 >> 24;
  HEAP32[$c>>2] = $31;
  $32 = ($31|0)!=(0);
  if (!($32)) {
   label = 268;
   break;
  }
  $33 = HEAP32[$c>>2]|0;
  $34 = ($33|0)!=(37);
  if ($34) {
   $35 = HEAP32[$1>>2]|0;
   HEAP32[$bufpt>>2] = $35;
   while(1) {
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 1|0);
    HEAP32[$1>>2] = $37;
    $38 = HEAP32[$1>>2]|0;
    $39 = HEAP8[$38>>0]|0;
    $40 = $39 << 24 >> 24;
    $41 = ($40|0)!=(0);
    if (!($41)) {
     break;
    }
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP8[$42>>0]|0;
    $44 = $43 << 24 >> 24;
    $45 = ($44|0)!=(37);
    if (!($45)) {
     break;
    }
   }
   $46 = HEAP32[$0>>2]|0;
   $47 = HEAP32[$bufpt>>2]|0;
   $48 = HEAP32[$1>>2]|0;
   $49 = HEAP32[$bufpt>>2]|0;
   $50 = $48;
   $51 = $49;
   $52 = (($50) - ($51))|0;
   _sqlite3StrAccumAppend($46,$47,$52);
   $53 = HEAP32[$1>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54 << 24 >> 24;
   $56 = ($55|0)==(0);
   if ($56) {
    label = 268;
    break;
   }
  }
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 1|0);
  HEAP32[$1>>2] = $58;
  $59 = HEAP8[$58>>0]|0;
  $60 = $59 << 24 >> 24;
  HEAP32[$c>>2] = $60;
  $61 = ($60|0)==(0);
  if ($61) {
   label = 13;
   break;
  }
  HEAP8[$flag_zeropad>>0] = 0;
  HEAP8[$flag_altform2>>0] = 0;
  HEAP8[$flag_alternateform>>0] = 0;
  HEAP8[$flag_blanksign>>0] = 0;
  HEAP8[$flag_plussign>>0] = 0;
  HEAP8[$flag_leftjustify>>0] = 0;
  HEAP8[$done>>0] = 0;
  while(1) {
   $63 = HEAP32[$c>>2]|0;
   switch ($63|0) {
   case 45:  {
    HEAP8[$flag_leftjustify>>0] = 1;
    break;
   }
   case 43:  {
    HEAP8[$flag_plussign>>0] = 1;
    break;
   }
   case 32:  {
    HEAP8[$flag_blanksign>>0] = 1;
    break;
   }
   case 35:  {
    HEAP8[$flag_alternateform>>0] = 1;
    break;
   }
   case 33:  {
    HEAP8[$flag_altform2>>0] = 1;
    break;
   }
   case 48:  {
    HEAP8[$flag_zeropad>>0] = 1;
    break;
   }
   default: {
    HEAP8[$done>>0] = 1;
   }
   }
   $64 = HEAP8[$done>>0]|0;
   $65 = ($64<<24>>24)!=(0);
   if ($65) {
    break;
   }
   $66 = HEAP32[$1>>2]|0;
   $67 = ((($66)) + 1|0);
   HEAP32[$1>>2] = $67;
   $68 = HEAP8[$67>>0]|0;
   $69 = $68 << 24 >> 24;
   HEAP32[$c>>2] = $69;
   $70 = ($69|0)!=(0);
   if (!($70)) {
    break;
   }
  }
  $71 = HEAP32[$c>>2]|0;
  $72 = ($71|0)==(42);
  if ($72) {
   $73 = HEAP8[$bArgList>>0]|0;
   $74 = ($73<<24>>24)!=(0);
   if ($74) {
    $75 = HEAP32[$pArgList>>2]|0;
    $76 = (_getIntArg($75)|0);
    $77 = tempRet0;
    HEAP32[$width>>2] = $76;
   } else {
    $78 = HEAP32[$2>>2]|0;
    $arglist_current12 = HEAP32[$78>>2]|0;
    $79 = $arglist_current12;
    $80 = ((0) + 4|0);
    $expanded60 = $80;
    $expanded59 = (($expanded60) - 1)|0;
    $81 = (($79) + ($expanded59))|0;
    $82 = ((0) + 4|0);
    $expanded64 = $82;
    $expanded63 = (($expanded64) - 1)|0;
    $expanded62 = $expanded63 ^ -1;
    $83 = $81 & $expanded62;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $arglist_next13 = ((($84)) + 4|0);
    HEAP32[$78>>2] = $arglist_next13;
    HEAP32[$width>>2] = $85;
   }
   $86 = HEAP32[$width>>2]|0;
   $87 = ($86|0)<(0);
   if ($87) {
    HEAP8[$flag_leftjustify>>0] = 1;
    $88 = HEAP32[$width>>2]|0;
    $89 = ($88|0)>=(-2147483647);
    $90 = HEAP32[$width>>2]|0;
    $91 = (0 - ($90))|0;
    $92 = $89 ? $91 : 0;
    HEAP32[$width>>2] = $92;
   }
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 1|0);
   HEAP32[$1>>2] = $94;
   $95 = HEAP8[$94>>0]|0;
   $96 = $95 << 24 >> 24;
   HEAP32[$c>>2] = $96;
  } else {
   HEAP32[$wx>>2] = 0;
   while(1) {
    $97 = HEAP32[$c>>2]|0;
    $98 = ($97|0)>=(48);
    $99 = HEAP32[$c>>2]|0;
    $100 = ($99|0)<=(57);
    $101 = $98 ? $100 : 0;
    $102 = HEAP32[$wx>>2]|0;
    if (!($101)) {
     break;
    }
    $103 = ($102*10)|0;
    $104 = HEAP32[$c>>2]|0;
    $105 = (($103) + ($104))|0;
    $106 = (($105) - 48)|0;
    HEAP32[$wx>>2] = $106;
    $107 = HEAP32[$1>>2]|0;
    $108 = ((($107)) + 1|0);
    HEAP32[$1>>2] = $108;
    $109 = HEAP8[$108>>0]|0;
    $110 = $109 << 24 >> 24;
    HEAP32[$c>>2] = $110;
   }
   $111 = $102 & 2147483647;
   HEAP32[$width>>2] = $111;
  }
  $112 = HEAP32[$c>>2]|0;
  $113 = ($112|0)==(46);
  do {
   if ($113) {
    $114 = HEAP32[$1>>2]|0;
    $115 = ((($114)) + 1|0);
    HEAP32[$1>>2] = $115;
    $116 = HEAP8[$115>>0]|0;
    $117 = $116 << 24 >> 24;
    HEAP32[$c>>2] = $117;
    $118 = HEAP32[$c>>2]|0;
    $119 = ($118|0)==(42);
    if (!($119)) {
     HEAP32[$px>>2] = 0;
     while(1) {
      $144 = HEAP32[$c>>2]|0;
      $145 = ($144|0)>=(48);
      $146 = HEAP32[$c>>2]|0;
      $147 = ($146|0)<=(57);
      $148 = $145 ? $147 : 0;
      $149 = HEAP32[$px>>2]|0;
      if (!($148)) {
       break;
      }
      $150 = ($149*10)|0;
      $151 = HEAP32[$c>>2]|0;
      $152 = (($150) + ($151))|0;
      $153 = (($152) - 48)|0;
      HEAP32[$px>>2] = $153;
      $154 = HEAP32[$1>>2]|0;
      $155 = ((($154)) + 1|0);
      HEAP32[$1>>2] = $155;
      $156 = HEAP8[$155>>0]|0;
      $157 = $156 << 24 >> 24;
      HEAP32[$c>>2] = $157;
     }
     $158 = $149 & 2147483647;
     HEAP32[$precision>>2] = $158;
     break;
    }
    $120 = HEAP8[$bArgList>>0]|0;
    $121 = ($120<<24>>24)!=(0);
    if ($121) {
     $122 = HEAP32[$pArgList>>2]|0;
     $123 = (_getIntArg($122)|0);
     $124 = tempRet0;
     HEAP32[$precision>>2] = $123;
    } else {
     $125 = HEAP32[$2>>2]|0;
     $arglist_current15 = HEAP32[$125>>2]|0;
     $126 = $arglist_current15;
     $127 = ((0) + 4|0);
     $expanded67 = $127;
     $expanded66 = (($expanded67) - 1)|0;
     $128 = (($126) + ($expanded66))|0;
     $129 = ((0) + 4|0);
     $expanded71 = $129;
     $expanded70 = (($expanded71) - 1)|0;
     $expanded69 = $expanded70 ^ -1;
     $130 = $128 & $expanded69;
     $131 = $130;
     $132 = HEAP32[$131>>2]|0;
     $arglist_next16 = ((($131)) + 4|0);
     HEAP32[$125>>2] = $arglist_next16;
     HEAP32[$precision>>2] = $132;
    }
    $133 = HEAP32[$1>>2]|0;
    $134 = ((($133)) + 1|0);
    HEAP32[$1>>2] = $134;
    $135 = HEAP8[$134>>0]|0;
    $136 = $135 << 24 >> 24;
    HEAP32[$c>>2] = $136;
    $137 = HEAP32[$precision>>2]|0;
    $138 = ($137|0)<(0);
    if ($138) {
     $139 = HEAP32[$precision>>2]|0;
     $140 = ($139|0)>=(-2147483647);
     $141 = HEAP32[$precision>>2]|0;
     $142 = (0 - ($141))|0;
     $143 = $140 ? $142 : -1;
     HEAP32[$precision>>2] = $143;
    }
   } else {
    HEAP32[$precision>>2] = -1;
   }
  } while(0);
  $159 = HEAP32[$c>>2]|0;
  $160 = ($159|0)==(108);
  do {
   if ($160) {
    HEAP8[$flag_long>>0] = 1;
    $161 = HEAP32[$1>>2]|0;
    $162 = ((($161)) + 1|0);
    HEAP32[$1>>2] = $162;
    $163 = HEAP8[$162>>0]|0;
    $164 = $163 << 24 >> 24;
    HEAP32[$c>>2] = $164;
    $165 = HEAP32[$c>>2]|0;
    $166 = ($165|0)==(108);
    if ($166) {
     HEAP8[$flag_longlong>>0] = 1;
     $167 = HEAP32[$1>>2]|0;
     $168 = ((($167)) + 1|0);
     HEAP32[$1>>2] = $168;
     $169 = HEAP8[$168>>0]|0;
     $170 = $169 << 24 >> 24;
     HEAP32[$c>>2] = $170;
     break;
    } else {
     HEAP8[$flag_longlong>>0] = 0;
     break;
    }
   } else {
    HEAP8[$flag_longlong>>0] = 0;
    HEAP8[$flag_long>>0] = 0;
   }
  } while(0);
  HEAP32[$infop>>2] = 44615;
  HEAP8[$xtype>>0] = 16;
  HEAP32[$idx>>2] = 0;
  while(1) {
   $171 = HEAP32[$idx>>2]|0;
   $172 = ($171|0)<(23);
   if (!($172)) {
    break;
   }
   $173 = HEAP32[$c>>2]|0;
   $174 = HEAP32[$idx>>2]|0;
   $175 = (44615 + (($174*6)|0)|0);
   $176 = HEAP8[$175>>0]|0;
   $177 = $176 << 24 >> 24;
   $178 = ($173|0)==($177|0);
   $179 = HEAP32[$idx>>2]|0;
   if ($178) {
    label = 56;
    break;
   }
   $193 = (($179) + 1)|0;
   HEAP32[$idx>>2] = $193;
  }
  if ((label|0) == 56) {
   label = 0;
   $180 = (44615 + (($179*6)|0)|0);
   HEAP32[$infop>>2] = $180;
   $181 = HEAP8[$useIntern>>0]|0;
   $182 = $181&255;
   $183 = ($182|0)!=(0);
   if (!($183)) {
    $184 = HEAP32[$infop>>2]|0;
    $185 = ((($184)) + 2|0);
    $186 = HEAP8[$185>>0]|0;
    $187 = $186&255;
    $188 = $187 & 2;
    $189 = ($188|0)==(0);
    if (!($189)) {
     label = 268;
     break;
    }
   }
   $190 = HEAP32[$infop>>2]|0;
   $191 = ((($190)) + 3|0);
   $192 = HEAP8[$191>>0]|0;
   HEAP8[$xtype>>0] = $192;
  }
  $194 = HEAP8[$xtype>>0]|0;
  $195 = $194&255;
  L79: do {
   switch ($195|0) {
   case 13:  {
    HEAP8[$flag_longlong>>0] = 0;
    HEAP8[$flag_long>>0] = 1;
    label = 62;
    break;
   }
   case 0: case 15:  {
    label = 62;
    break;
   }
   case 3: case 2: case 1:  {
    $500 = HEAP8[$bArgList>>0]|0;
    $501 = ($500<<24>>24)!=(0);
    if ($501) {
     $502 = HEAP32[$pArgList>>2]|0;
     $503 = (+_getDoubleArg($502));
     HEAPF64[$realvalue>>3] = $503;
    } else {
     $504 = HEAP32[$2>>2]|0;
     $arglist_current30 = HEAP32[$504>>2]|0;
     $505 = $arglist_current30;
     $506 = ((0) + 8|0);
     $expanded102 = $506;
     $expanded101 = (($expanded102) - 1)|0;
     $507 = (($505) + ($expanded101))|0;
     $508 = ((0) + 8|0);
     $expanded106 = $508;
     $expanded105 = (($expanded106) - 1)|0;
     $expanded104 = $expanded105 ^ -1;
     $509 = $507 & $expanded104;
     $510 = $509;
     $511 = +HEAPF64[$510>>3];
     $arglist_next31 = ((($510)) + 8|0);
     HEAP32[$504>>2] = $arglist_next31;
     HEAPF64[$realvalue>>3] = $511;
    }
    $512 = HEAP32[$precision>>2]|0;
    $513 = ($512|0)<(0);
    if ($513) {
     HEAP32[$precision>>2] = 6;
    }
    $514 = +HEAPF64[$realvalue>>3];
    $515 = $514 < 0.0;
    do {
     if ($515) {
      $516 = +HEAPF64[$realvalue>>3];
      $517 = -$516;
      HEAPF64[$realvalue>>3] = $517;
      HEAP8[$prefix>>0] = 45;
     } else {
      $518 = HEAP8[$flag_plussign>>0]|0;
      $519 = ($518<<24>>24)!=(0);
      if ($519) {
       HEAP8[$prefix>>0] = 43;
       break;
      }
      $520 = HEAP8[$flag_blanksign>>0]|0;
      $521 = ($520<<24>>24)!=(0);
      if ($521) {
       HEAP8[$prefix>>0] = 32;
       break;
      } else {
       HEAP8[$prefix>>0] = 0;
       break;
      }
     }
    } while(0);
    $522 = HEAP8[$xtype>>0]|0;
    $523 = $522&255;
    $524 = ($523|0)==(3);
    $525 = HEAP32[$precision>>2]|0;
    $526 = ($525|0)>(0);
    $or$cond = $524 & $526;
    if ($or$cond) {
     $527 = HEAP32[$precision>>2]|0;
     $528 = (($527) + -1)|0;
     HEAP32[$precision>>2] = $528;
    }
    $529 = HEAP32[$precision>>2]|0;
    $530 = $529 & 4095;
    HEAP32[$idx>>2] = $530;
    HEAPF64[$rounder>>3] = 0.5;
    while(1) {
     $531 = HEAP32[$idx>>2]|0;
     $532 = ($531|0)>(0);
     if (!($532)) {
      break;
     }
     $533 = HEAP32[$idx>>2]|0;
     $534 = (($533) + -1)|0;
     HEAP32[$idx>>2] = $534;
     $535 = +HEAPF64[$rounder>>3];
     $536 = $535 * 0.10000000000000001;
     HEAPF64[$rounder>>3] = $536;
    }
    $537 = HEAP8[$xtype>>0]|0;
    $538 = $537&255;
    $539 = ($538|0)==(1);
    if ($539) {
     $540 = +HEAPF64[$rounder>>3];
     $541 = +HEAPF64[$realvalue>>3];
     $542 = $541 + $540;
     HEAPF64[$realvalue>>3] = $542;
    }
    HEAP32[$exp>>2] = 0;
    $543 = +HEAPF64[$realvalue>>3];
    $544 = (_sqlite3IsNaN($543)|0);
    $545 = ($544|0)!=(0);
    if ($545) {
     HEAP32[$bufpt>>2] = 44802;
     HEAP32[$length>>2] = 3;
     break L79;
    }
    $546 = +HEAPF64[$realvalue>>3];
    $547 = $546 > 0.0;
    if ($547) {
     HEAPF64[$scale>>3] = 1.0;
     while(1) {
      $548 = +HEAPF64[$realvalue>>3];
      $549 = +HEAPF64[$scale>>3];
      $550 = 1.0E+100 * $549;
      $551 = $548 >= $550;
      $552 = HEAP32[$exp>>2]|0;
      $553 = ($552|0)<=(350);
      $554 = $551 ? $553 : 0;
      if (!($554)) {
       break;
      }
      $555 = +HEAPF64[$scale>>3];
      $556 = $555 * 1.0E+100;
      HEAPF64[$scale>>3] = $556;
      $557 = HEAP32[$exp>>2]|0;
      $558 = (($557) + 100)|0;
      HEAP32[$exp>>2] = $558;
     }
     while(1) {
      $559 = +HEAPF64[$realvalue>>3];
      $560 = +HEAPF64[$scale>>3];
      $561 = 1.0E+10 * $560;
      $562 = $559 >= $561;
      $563 = HEAP32[$exp>>2]|0;
      $564 = ($563|0)<=(350);
      $565 = $562 ? $564 : 0;
      if (!($565)) {
       break;
      }
      $566 = +HEAPF64[$scale>>3];
      $567 = $566 * 1.0E+10;
      HEAPF64[$scale>>3] = $567;
      $568 = HEAP32[$exp>>2]|0;
      $569 = (($568) + 10)|0;
      HEAP32[$exp>>2] = $569;
     }
     while(1) {
      $570 = +HEAPF64[$realvalue>>3];
      $571 = +HEAPF64[$scale>>3];
      $572 = 10.0 * $571;
      $573 = $570 >= $572;
      $574 = HEAP32[$exp>>2]|0;
      $575 = ($574|0)<=(350);
      $576 = $573 ? $575 : 0;
      $577 = +HEAPF64[$scale>>3];
      if (!($576)) {
       break;
      }
      $578 = $577 * 10.0;
      HEAPF64[$scale>>3] = $578;
      $579 = HEAP32[$exp>>2]|0;
      $580 = (($579) + 1)|0;
      HEAP32[$exp>>2] = $580;
     }
     $581 = +HEAPF64[$realvalue>>3];
     $582 = $581 / $577;
     HEAPF64[$realvalue>>3] = $582;
     while(1) {
      $583 = +HEAPF64[$realvalue>>3];
      $584 = $583 < 1.0E-8;
      if (!($584)) {
       break;
      }
      $585 = +HEAPF64[$realvalue>>3];
      $586 = $585 * 1.0E+8;
      HEAPF64[$realvalue>>3] = $586;
      $587 = HEAP32[$exp>>2]|0;
      $588 = (($587) - 8)|0;
      HEAP32[$exp>>2] = $588;
     }
     while(1) {
      $589 = +HEAPF64[$realvalue>>3];
      $590 = $589 < 1.0;
      if (!($590)) {
       break;
      }
      $591 = +HEAPF64[$realvalue>>3];
      $592 = $591 * 10.0;
      HEAPF64[$realvalue>>3] = $592;
      $593 = HEAP32[$exp>>2]|0;
      $594 = (($593) + -1)|0;
      HEAP32[$exp>>2] = $594;
     }
     $595 = HEAP32[$exp>>2]|0;
     $596 = ($595|0)>(350);
     if ($596) {
      HEAP32[$bufpt>>2] = $buf;
      $597 = HEAP8[$prefix>>0]|0;
      HEAP8[$buf>>0] = $597;
      $598 = HEAP8[$prefix>>0]|0;
      $599 = $598 << 24 >> 24;
      $600 = ($599|0)!=(0);
      $601 = $600&1;
      $602 = (($buf) + ($601)|0);
      ;HEAP8[$602>>0]=HEAP8[44806>>0]|0;HEAP8[$602+1>>0]=HEAP8[44806+1>>0]|0;HEAP8[$602+2>>0]=HEAP8[44806+2>>0]|0;HEAP8[$602+3>>0]=HEAP8[44806+3>>0]|0;
      $603 = HEAP8[$prefix>>0]|0;
      $604 = $603 << 24 >> 24;
      $605 = ($604|0)!=(0);
      $606 = $605&1;
      $607 = (3 + ($606))|0;
      HEAP32[$length>>2] = $607;
      break L79;
     }
    }
    HEAP32[$bufpt>>2] = $buf;
    $608 = HEAP8[$xtype>>0]|0;
    $609 = $608&255;
    $610 = ($609|0)!=(1);
    if ($610) {
     $611 = +HEAPF64[$rounder>>3];
     $612 = +HEAPF64[$realvalue>>3];
     $613 = $612 + $611;
     HEAPF64[$realvalue>>3] = $613;
     $614 = +HEAPF64[$realvalue>>3];
     $615 = $614 >= 10.0;
     if ($615) {
      $616 = +HEAPF64[$realvalue>>3];
      $617 = $616 * 0.10000000000000001;
      HEAPF64[$realvalue>>3] = $617;
      $618 = HEAP32[$exp>>2]|0;
      $619 = (($618) + 1)|0;
      HEAP32[$exp>>2] = $619;
     }
    }
    $620 = HEAP8[$xtype>>0]|0;
    $621 = $620&255;
    $622 = ($621|0)==(3);
    L137: do {
     if ($622) {
      $623 = HEAP8[$flag_alternateform>>0]|0;
      $624 = ($623<<24>>24)!=(0);
      $625 = $624 ^ 1;
      $626 = $625&1;
      $627 = $626&255;
      HEAP8[$flag_rtz>>0] = $627;
      $628 = HEAP32[$exp>>2]|0;
      $629 = ($628|0)<(-4);
      do {
       if (!($629)) {
        $630 = HEAP32[$exp>>2]|0;
        $631 = HEAP32[$precision>>2]|0;
        $632 = ($630|0)>($631|0);
        if ($632) {
         break;
        }
        $633 = HEAP32[$precision>>2]|0;
        $634 = HEAP32[$exp>>2]|0;
        $635 = (($633) - ($634))|0;
        HEAP32[$precision>>2] = $635;
        HEAP8[$xtype>>0] = 1;
        break L137;
       }
      } while(0);
      HEAP8[$xtype>>0] = 2;
     } else {
      $636 = HEAP8[$flag_altform2>>0]|0;
      HEAP8[$flag_rtz>>0] = $636;
     }
    } while(0);
    $637 = HEAP8[$xtype>>0]|0;
    $638 = $637&255;
    $639 = ($638|0)==(2);
    if ($639) {
     HEAP32[$e2>>2] = 0;
    } else {
     $640 = HEAP32[$exp>>2]|0;
     HEAP32[$e2>>2] = $640;
    }
    $641 = HEAP32[$e2>>2]|0;
    $642 = ($641|0)>(0);
    $643 = HEAP32[$e2>>2]|0;
    $644 = $642 ? $643 : 0;
    $645 = ($644|0)<(0);
    $646 = $645 << 31 >> 31;
    $647 = HEAP32[$precision>>2]|0;
    $648 = ($647|0)<(0);
    $649 = $648 << 31 >> 31;
    $650 = (_i64Add(($644|0),($646|0),($647|0),($649|0))|0);
    $651 = tempRet0;
    $652 = HEAP32[$width>>2]|0;
    $653 = ($652|0)<(0);
    $654 = $653 << 31 >> 31;
    $655 = (_i64Add(($650|0),($651|0),($652|0),($654|0))|0);
    $656 = tempRet0;
    $657 = ($656|0)>(0);
    $658 = ($655>>>0)>(55);
    $659 = ($656|0)==(0);
    $660 = $659 & $658;
    $661 = $657 | $660;
    if ($661) {
     $662 = HEAP32[$e2>>2]|0;
     $663 = ($662|0)>(0);
     $664 = HEAP32[$e2>>2]|0;
     $665 = $663 ? $664 : 0;
     $666 = ($665|0)<(0);
     $667 = $666 << 31 >> 31;
     $668 = HEAP32[$precision>>2]|0;
     $669 = ($668|0)<(0);
     $670 = $669 << 31 >> 31;
     $671 = (_i64Add(($665|0),($667|0),($668|0),($670|0))|0);
     $672 = tempRet0;
     $673 = HEAP32[$width>>2]|0;
     $674 = ($673|0)<(0);
     $675 = $674 << 31 >> 31;
     $676 = (_i64Add(($671|0),($672|0),($673|0),($675|0))|0);
     $677 = tempRet0;
     $678 = (_i64Add(($676|0),($677|0),15,0)|0);
     $679 = tempRet0;
     $680 = (_sqlite3Malloc($678,$679)|0);
     HEAP32[$zExtra>>2] = $680;
     HEAP32[$bufpt>>2] = $680;
     $681 = HEAP32[$bufpt>>2]|0;
     $682 = ($681|0)==(0|0);
     if ($682) {
      label = 161;
      break L7;
     }
    }
    $684 = HEAP32[$bufpt>>2]|0;
    HEAP32[$zOut>>2] = $684;
    $685 = HEAP8[$flag_altform2>>0]|0;
    $686 = $685&255;
    $687 = ($686*10)|0;
    $688 = (16 + ($687))|0;
    HEAP32[$nsd>>2] = $688;
    $689 = HEAP32[$precision>>2]|0;
    $690 = ($689|0)>(0);
    $691 = $690 ? 1 : 0;
    $692 = HEAP8[$flag_alternateform>>0]|0;
    $693 = $692&255;
    $694 = $691 | $693;
    $695 = HEAP8[$flag_altform2>>0]|0;
    $696 = $695&255;
    $697 = $694 | $696;
    $698 = $697&255;
    HEAP8[$flag_dp>>0] = $698;
    $699 = HEAP8[$prefix>>0]|0;
    $700 = ($699<<24>>24)!=(0);
    if ($700) {
     $701 = HEAP8[$prefix>>0]|0;
     $702 = HEAP32[$bufpt>>2]|0;
     $703 = ((($702)) + 1|0);
     HEAP32[$bufpt>>2] = $703;
     HEAP8[$702>>0] = $701;
    }
    $704 = HEAP32[$e2>>2]|0;
    $705 = ($704|0)<(0);
    L155: do {
     if ($705) {
      $706 = HEAP32[$bufpt>>2]|0;
      $707 = ((($706)) + 1|0);
      HEAP32[$bufpt>>2] = $707;
      HEAP8[$706>>0] = 48;
     } else {
      while(1) {
       $708 = HEAP32[$e2>>2]|0;
       $709 = ($708|0)>=(0);
       if (!($709)) {
        break L155;
       }
       $710 = (_et_getdigit($realvalue,$nsd)|0);
       $711 = HEAP32[$bufpt>>2]|0;
       $712 = ((($711)) + 1|0);
       HEAP32[$bufpt>>2] = $712;
       HEAP8[$711>>0] = $710;
       $713 = HEAP32[$e2>>2]|0;
       $714 = (($713) + -1)|0;
       HEAP32[$e2>>2] = $714;
      }
     }
    } while(0);
    $715 = HEAP8[$flag_dp>>0]|0;
    $716 = ($715<<24>>24)!=(0);
    if ($716) {
     $717 = HEAP32[$bufpt>>2]|0;
     $718 = ((($717)) + 1|0);
     HEAP32[$bufpt>>2] = $718;
     HEAP8[$717>>0] = 46;
    }
    $719 = HEAP32[$e2>>2]|0;
    $720 = (($719) + 1)|0;
    HEAP32[$e2>>2] = $720;
    while(1) {
     $721 = HEAP32[$e2>>2]|0;
     $722 = ($721|0)<(0);
     if (!($722)) {
      break;
     }
     $723 = HEAP32[$bufpt>>2]|0;
     $724 = ((($723)) + 1|0);
     HEAP32[$bufpt>>2] = $724;
     HEAP8[$723>>0] = 48;
     $725 = HEAP32[$precision>>2]|0;
     $726 = (($725) + -1)|0;
     HEAP32[$precision>>2] = $726;
     $727 = HEAP32[$e2>>2]|0;
     $728 = (($727) + 1)|0;
     HEAP32[$e2>>2] = $728;
    }
    while(1) {
     $729 = HEAP32[$precision>>2]|0;
     $730 = (($729) + -1)|0;
     HEAP32[$precision>>2] = $730;
     $731 = ($729|0)>(0);
     if (!($731)) {
      break;
     }
     $732 = (_et_getdigit($realvalue,$nsd)|0);
     $733 = HEAP32[$bufpt>>2]|0;
     $734 = ((($733)) + 1|0);
     HEAP32[$bufpt>>2] = $734;
     HEAP8[$733>>0] = $732;
    }
    $735 = HEAP8[$flag_rtz>>0]|0;
    $736 = $735&255;
    $737 = ($736|0)!=(0);
    do {
     if ($737) {
      $738 = HEAP8[$flag_dp>>0]|0;
      $739 = $738&255;
      $740 = ($739|0)!=(0);
      if (!($740)) {
       break;
      }
      while(1) {
       $741 = HEAP32[$bufpt>>2]|0;
       $742 = ((($741)) + -1|0);
       $743 = HEAP8[$742>>0]|0;
       $744 = $743 << 24 >> 24;
       $745 = ($744|0)==(48);
       $746 = HEAP32[$bufpt>>2]|0;
       $747 = ((($746)) + -1|0);
       if (!($745)) {
        break;
       }
       HEAP32[$bufpt>>2] = $747;
       HEAP8[$747>>0] = 0;
      }
      $748 = HEAP8[$747>>0]|0;
      $749 = $748 << 24 >> 24;
      $750 = ($749|0)==(46);
      if (!($750)) {
       break;
      }
      $751 = HEAP8[$flag_altform2>>0]|0;
      $752 = ($751<<24>>24)!=(0);
      $753 = HEAP32[$bufpt>>2]|0;
      if ($752) {
       $754 = ((($753)) + 1|0);
       HEAP32[$bufpt>>2] = $754;
       HEAP8[$753>>0] = 48;
       break;
      } else {
       $755 = ((($753)) + -1|0);
       HEAP32[$bufpt>>2] = $755;
       HEAP8[$755>>0] = 0;
       break;
      }
     }
    } while(0);
    $756 = HEAP8[$xtype>>0]|0;
    $757 = $756&255;
    $758 = ($757|0)==(2);
    if ($758) {
     $759 = HEAP32[$infop>>2]|0;
     $760 = ((($759)) + 4|0);
     $761 = HEAP8[$760>>0]|0;
     $762 = $761&255;
     $763 = (44762 + ($762)|0);
     $764 = HEAP8[$763>>0]|0;
     $765 = HEAP32[$bufpt>>2]|0;
     $766 = ((($765)) + 1|0);
     HEAP32[$bufpt>>2] = $766;
     HEAP8[$765>>0] = $764;
     $767 = HEAP32[$exp>>2]|0;
     $768 = ($767|0)<(0);
     $769 = HEAP32[$bufpt>>2]|0;
     $770 = ((($769)) + 1|0);
     HEAP32[$bufpt>>2] = $770;
     if ($768) {
      HEAP8[$769>>0] = 45;
      $771 = HEAP32[$exp>>2]|0;
      $772 = (0 - ($771))|0;
      HEAP32[$exp>>2] = $772;
     } else {
      HEAP8[$769>>0] = 43;
     }
     $773 = HEAP32[$exp>>2]|0;
     $774 = ($773|0)>=(100);
     if ($774) {
      $775 = HEAP32[$exp>>2]|0;
      $776 = (($775|0) / 100)&-1;
      $777 = (($776) + 48)|0;
      $778 = $777&255;
      $779 = HEAP32[$bufpt>>2]|0;
      $780 = ((($779)) + 1|0);
      HEAP32[$bufpt>>2] = $780;
      HEAP8[$779>>0] = $778;
      $781 = HEAP32[$exp>>2]|0;
      $782 = (($781|0) % 100)&-1;
      HEAP32[$exp>>2] = $782;
     }
     $783 = HEAP32[$exp>>2]|0;
     $784 = (($783|0) / 10)&-1;
     $785 = (($784) + 48)|0;
     $786 = $785&255;
     $787 = HEAP32[$bufpt>>2]|0;
     $788 = ((($787)) + 1|0);
     HEAP32[$bufpt>>2] = $788;
     HEAP8[$787>>0] = $786;
     $789 = HEAP32[$exp>>2]|0;
     $790 = (($789|0) % 10)&-1;
     $791 = (($790) + 48)|0;
     $792 = $791&255;
     $793 = HEAP32[$bufpt>>2]|0;
     $794 = ((($793)) + 1|0);
     HEAP32[$bufpt>>2] = $794;
     HEAP8[$793>>0] = $792;
    }
    $795 = HEAP32[$bufpt>>2]|0;
    HEAP8[$795>>0] = 0;
    $796 = HEAP32[$bufpt>>2]|0;
    $797 = HEAP32[$zOut>>2]|0;
    $798 = $796;
    $799 = $797;
    $800 = (($798) - ($799))|0;
    HEAP32[$length>>2] = $800;
    $801 = HEAP32[$zOut>>2]|0;
    HEAP32[$bufpt>>2] = $801;
    $802 = HEAP8[$flag_zeropad>>0]|0;
    $803 = $802&255;
    $804 = ($803|0)==(0);
    $805 = HEAP8[$flag_leftjustify>>0]|0;
    $806 = ($805<<24>>24)!=(0);
    $or$cond4 = $804 | $806;
    if ($or$cond4) {
     break L79;
    }
    $807 = HEAP32[$length>>2]|0;
    $808 = HEAP32[$width>>2]|0;
    $809 = ($807|0)<($808|0);
    if (!($809)) {
     break L79;
    }
    $810 = HEAP32[$width>>2]|0;
    $811 = HEAP32[$length>>2]|0;
    $812 = (($810) - ($811))|0;
    HEAP32[$nPad>>2] = $812;
    $813 = HEAP32[$width>>2]|0;
    HEAP32[$i>>2] = $813;
    while(1) {
     $814 = HEAP32[$i>>2]|0;
     $815 = HEAP32[$nPad>>2]|0;
     $816 = ($814|0)>=($815|0);
     if (!($816)) {
      break;
     }
     $817 = HEAP32[$i>>2]|0;
     $818 = HEAP32[$nPad>>2]|0;
     $819 = (($817) - ($818))|0;
     $820 = HEAP32[$bufpt>>2]|0;
     $821 = (($820) + ($819)|0);
     $822 = HEAP8[$821>>0]|0;
     $823 = HEAP32[$i>>2]|0;
     $824 = HEAP32[$bufpt>>2]|0;
     $825 = (($824) + ($823)|0);
     HEAP8[$825>>0] = $822;
     $826 = HEAP32[$i>>2]|0;
     $827 = (($826) + -1)|0;
     HEAP32[$i>>2] = $827;
    }
    $828 = HEAP8[$prefix>>0]|0;
    $829 = $828 << 24 >> 24;
    $830 = ($829|0)!=(0);
    $831 = $830&1;
    HEAP32[$i>>2] = $831;
    while(1) {
     $832 = HEAP32[$nPad>>2]|0;
     $833 = (($832) + -1)|0;
     HEAP32[$nPad>>2] = $833;
     $834 = ($832|0)!=(0);
     if (!($834)) {
      break;
     }
     $835 = HEAP32[$i>>2]|0;
     $836 = (($835) + 1)|0;
     HEAP32[$i>>2] = $836;
     $837 = HEAP32[$bufpt>>2]|0;
     $838 = (($837) + ($835)|0);
     HEAP8[$838>>0] = 48;
    }
    $839 = HEAP32[$width>>2]|0;
    HEAP32[$length>>2] = $839;
    break;
   }
   case 4:  {
    $840 = HEAP8[$bArgList>>0]|0;
    $841 = ($840<<24>>24)!=(0);
    if (!($841)) {
     $842 = HEAP32[$0>>2]|0;
     $843 = ((($842)) + 12|0);
     $844 = HEAP32[$843>>2]|0;
     $845 = HEAP32[$2>>2]|0;
     $arglist_current33 = HEAP32[$845>>2]|0;
     $846 = $arglist_current33;
     $847 = ((0) + 4|0);
     $expanded109 = $847;
     $expanded108 = (($expanded109) - 1)|0;
     $848 = (($846) + ($expanded108))|0;
     $849 = ((0) + 4|0);
     $expanded113 = $849;
     $expanded112 = (($expanded113) - 1)|0;
     $expanded111 = $expanded112 ^ -1;
     $850 = $848 & $expanded111;
     $851 = $850;
     $852 = HEAP32[$851>>2]|0;
     $arglist_next34 = ((($851)) + 4|0);
     HEAP32[$845>>2] = $arglist_next34;
     HEAP32[$852>>2] = $844;
    }
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   case 7:  {
    HEAP8[$buf>>0] = 37;
    HEAP32[$bufpt>>2] = $buf;
    HEAP32[$length>>2] = 1;
    break;
   }
   case 8:  {
    $853 = HEAP8[$bArgList>>0]|0;
    $854 = ($853<<24>>24)!=(0);
    if ($854) {
     $855 = HEAP32[$pArgList>>2]|0;
     $856 = (_getTextArg($855)|0);
     HEAP32[$bufpt>>2] = $856;
     $857 = HEAP32[$bufpt>>2]|0;
     $858 = ($857|0)!=(0|0);
     if ($858) {
      $859 = HEAP32[$bufpt>>2]|0;
      $860 = HEAP8[$859>>0]|0;
      $861 = $860 << 24 >> 24;
      $862 = $861;
     } else {
      $862 = 0;
     }
     HEAP32[$c>>2] = $862;
    } else {
     $863 = HEAP32[$2>>2]|0;
     $arglist_current36 = HEAP32[$863>>2]|0;
     $864 = $arglist_current36;
     $865 = ((0) + 4|0);
     $expanded116 = $865;
     $expanded115 = (($expanded116) - 1)|0;
     $866 = (($864) + ($expanded115))|0;
     $867 = ((0) + 4|0);
     $expanded120 = $867;
     $expanded119 = (($expanded120) - 1)|0;
     $expanded118 = $expanded119 ^ -1;
     $868 = $866 & $expanded118;
     $869 = $868;
     $870 = HEAP32[$869>>2]|0;
     $arglist_next37 = ((($869)) + 4|0);
     HEAP32[$863>>2] = $arglist_next37;
     HEAP32[$c>>2] = $870;
    }
    $871 = HEAP32[$precision>>2]|0;
    $872 = ($871|0)>(1);
    if ($872) {
     $873 = HEAP32[$precision>>2]|0;
     $874 = (($873) - 1)|0;
     $875 = HEAP32[$width>>2]|0;
     $876 = (($875) - ($874))|0;
     HEAP32[$width>>2] = $876;
     $877 = HEAP32[$width>>2]|0;
     $878 = ($877|0)<=(1);
     $879 = HEAP8[$flag_leftjustify>>0]|0;
     $880 = ($879<<24>>24)!=(0);
     $or$cond6 = $878 | $880;
     if (!($or$cond6)) {
      $881 = HEAP32[$0>>2]|0;
      $882 = HEAP32[$width>>2]|0;
      $883 = (($882) - 1)|0;
      _sqlite3AppendChar($881,$883,32);
      HEAP32[$width>>2] = 0;
     }
     $884 = HEAP32[$0>>2]|0;
     $885 = HEAP32[$precision>>2]|0;
     $886 = (($885) - 1)|0;
     $887 = HEAP32[$c>>2]|0;
     $888 = $887&255;
     _sqlite3AppendChar($884,$886,$888);
    }
    HEAP32[$length>>2] = 1;
    $889 = HEAP32[$c>>2]|0;
    $890 = $889&255;
    HEAP8[$buf>>0] = $890;
    HEAP32[$bufpt>>2] = $buf;
    break;
   }
   case 6: case 5:  {
    $891 = HEAP8[$bArgList>>0]|0;
    $892 = ($891<<24>>24)!=(0);
    if ($892) {
     $893 = HEAP32[$pArgList>>2]|0;
     $894 = (_getTextArg($893)|0);
     HEAP32[$bufpt>>2] = $894;
     HEAP8[$xtype>>0] = 5;
    } else {
     $895 = HEAP32[$2>>2]|0;
     $arglist_current39 = HEAP32[$895>>2]|0;
     $896 = $arglist_current39;
     $897 = ((0) + 4|0);
     $expanded123 = $897;
     $expanded122 = (($expanded123) - 1)|0;
     $898 = (($896) + ($expanded122))|0;
     $899 = ((0) + 4|0);
     $expanded127 = $899;
     $expanded126 = (($expanded127) - 1)|0;
     $expanded125 = $expanded126 ^ -1;
     $900 = $898 & $expanded125;
     $901 = $900;
     $902 = HEAP32[$901>>2]|0;
     $arglist_next40 = ((($901)) + 4|0);
     HEAP32[$895>>2] = $arglist_next40;
     HEAP32[$bufpt>>2] = $902;
    }
    $903 = HEAP32[$bufpt>>2]|0;
    $904 = ($903|0)==(0|0);
    if ($904) {
     HEAP32[$bufpt>>2] = 18402;
    } else {
     $905 = HEAP8[$xtype>>0]|0;
     $906 = $905&255;
     $907 = ($906|0)==(6);
     if ($907) {
      $908 = HEAP32[$bufpt>>2]|0;
      HEAP32[$zExtra>>2] = $908;
     }
    }
    $909 = HEAP32[$precision>>2]|0;
    $910 = ($909|0)>=(0);
    if (!($910)) {
     $922 = HEAP32[$bufpt>>2]|0;
     $923 = (_sqlite3Strlen30($922)|0);
     HEAP32[$length>>2] = $923;
     break L79;
    }
    HEAP32[$length>>2] = 0;
    while(1) {
     $911 = HEAP32[$length>>2]|0;
     $912 = HEAP32[$precision>>2]|0;
     $913 = ($911|0)<($912|0);
     if (!($913)) {
      break L79;
     }
     $914 = HEAP32[$length>>2]|0;
     $915 = HEAP32[$bufpt>>2]|0;
     $916 = (($915) + ($914)|0);
     $917 = HEAP8[$916>>0]|0;
     $918 = $917 << 24 >> 24;
     $919 = ($918|0)!=(0);
     if (!($919)) {
      break L79;
     }
     $920 = HEAP32[$length>>2]|0;
     $921 = (($920) + 1)|0;
     HEAP32[$length>>2] = $921;
    }
    break;
   }
   case 14: case 10: case 9:  {
    $924 = HEAP8[$xtype>>0]|0;
    $925 = $924&255;
    $926 = ($925|0)==(14);
    $927 = $926 ? 34 : 39;
    $928 = $927&255;
    HEAP8[$q>>0] = $928;
    $929 = HEAP8[$bArgList>>0]|0;
    $930 = ($929<<24>>24)!=(0);
    if ($930) {
     $931 = HEAP32[$pArgList>>2]|0;
     $932 = (_getTextArg($931)|0);
     HEAP32[$escarg>>2] = $932;
    } else {
     $933 = HEAP32[$2>>2]|0;
     $arglist_current42 = HEAP32[$933>>2]|0;
     $934 = $arglist_current42;
     $935 = ((0) + 4|0);
     $expanded130 = $935;
     $expanded129 = (($expanded130) - 1)|0;
     $936 = (($934) + ($expanded129))|0;
     $937 = ((0) + 4|0);
     $expanded134 = $937;
     $expanded133 = (($expanded134) - 1)|0;
     $expanded132 = $expanded133 ^ -1;
     $938 = $936 & $expanded132;
     $939 = $938;
     $940 = HEAP32[$939>>2]|0;
     $arglist_next43 = ((($939)) + 4|0);
     HEAP32[$933>>2] = $arglist_next43;
     HEAP32[$escarg>>2] = $940;
    }
    $941 = HEAP32[$escarg>>2]|0;
    $942 = ($941|0)==(0|0);
    $943 = $942&1;
    HEAP32[$isnull>>2] = $943;
    $944 = HEAP32[$isnull>>2]|0;
    $945 = ($944|0)!=(0);
    if ($945) {
     $946 = HEAP8[$xtype>>0]|0;
     $947 = $946&255;
     $948 = ($947|0)==(10);
     $949 = $948 ? 39691 : 44810;
     HEAP32[$escarg>>2] = $949;
    }
    $950 = HEAP32[$precision>>2]|0;
    HEAP32[$k>>2] = $950;
    HEAP32[$n>>2] = 0;
    HEAP32[$i2>>2] = 0;
    while(1) {
     $951 = HEAP32[$k>>2]|0;
     $952 = ($951|0)!=(0);
     if (!($952)) {
      break;
     }
     $953 = HEAP32[$i2>>2]|0;
     $954 = HEAP32[$escarg>>2]|0;
     $955 = (($954) + ($953)|0);
     $956 = HEAP8[$955>>0]|0;
     HEAP8[$ch>>0] = $956;
     $957 = $956 << 24 >> 24;
     $958 = ($957|0)!=(0);
     if (!($958)) {
      break;
     }
     $959 = HEAP8[$ch>>0]|0;
     $960 = $959 << 24 >> 24;
     $961 = HEAP8[$q>>0]|0;
     $962 = $961 << 24 >> 24;
     $963 = ($960|0)==($962|0);
     if ($963) {
      $964 = HEAP32[$n>>2]|0;
      $965 = (($964) + 1)|0;
      HEAP32[$n>>2] = $965;
     }
     $966 = HEAP32[$i2>>2]|0;
     $967 = (($966) + 1)|0;
     HEAP32[$i2>>2] = $967;
     $968 = HEAP32[$k>>2]|0;
     $969 = (($968) + -1)|0;
     HEAP32[$k>>2] = $969;
    }
    $970 = HEAP32[$isnull>>2]|0;
    $971 = ($970|0)!=(0);
    if ($971) {
     $976 = 0;
    } else {
     $972 = HEAP8[$xtype>>0]|0;
     $973 = $972&255;
     $974 = ($973|0)==(10);
     $976 = $974;
    }
    $975 = $976&1;
    HEAP32[$needQuote>>2] = $975;
    $977 = HEAP32[$i2>>2]|0;
    $978 = (($977) + 3)|0;
    $979 = HEAP32[$n>>2]|0;
    $980 = (($979) + ($978))|0;
    HEAP32[$n>>2] = $980;
    $981 = HEAP32[$n>>2]|0;
    $982 = ($981|0)>(70);
    if ($982) {
     $983 = HEAP32[$n>>2]|0;
     $984 = ($983|0)<(0);
     $985 = $984 << 31 >> 31;
     $986 = (_sqlite3Malloc($983,$985)|0);
     HEAP32[$zExtra>>2] = $986;
     HEAP32[$bufpt>>2] = $986;
     $987 = HEAP32[$bufpt>>2]|0;
     $988 = ($987|0)==(0|0);
     if ($988) {
      label = 241;
      break L7;
     }
    } else {
     HEAP32[$bufpt>>2] = $buf;
    }
    HEAP32[$j>>2] = 0;
    $990 = HEAP32[$needQuote>>2]|0;
    $991 = ($990|0)!=(0);
    if ($991) {
     $992 = HEAP8[$q>>0]|0;
     $993 = HEAP32[$j>>2]|0;
     $994 = (($993) + 1)|0;
     HEAP32[$j>>2] = $994;
     $995 = HEAP32[$bufpt>>2]|0;
     $996 = (($995) + ($993)|0);
     HEAP8[$996>>0] = $992;
    }
    $997 = HEAP32[$i2>>2]|0;
    HEAP32[$k>>2] = $997;
    HEAP32[$i2>>2] = 0;
    while(1) {
     $998 = HEAP32[$i2>>2]|0;
     $999 = HEAP32[$k>>2]|0;
     $1000 = ($998|0)<($999|0);
     if (!($1000)) {
      break;
     }
     $1001 = HEAP32[$i2>>2]|0;
     $1002 = HEAP32[$escarg>>2]|0;
     $1003 = (($1002) + ($1001)|0);
     $1004 = HEAP8[$1003>>0]|0;
     HEAP8[$ch>>0] = $1004;
     $1005 = HEAP32[$j>>2]|0;
     $1006 = (($1005) + 1)|0;
     HEAP32[$j>>2] = $1006;
     $1007 = HEAP32[$bufpt>>2]|0;
     $1008 = (($1007) + ($1005)|0);
     HEAP8[$1008>>0] = $1004;
     $1009 = HEAP8[$ch>>0]|0;
     $1010 = $1009 << 24 >> 24;
     $1011 = HEAP8[$q>>0]|0;
     $1012 = $1011 << 24 >> 24;
     $1013 = ($1010|0)==($1012|0);
     if ($1013) {
      $1014 = HEAP8[$ch>>0]|0;
      $1015 = HEAP32[$j>>2]|0;
      $1016 = (($1015) + 1)|0;
      HEAP32[$j>>2] = $1016;
      $1017 = HEAP32[$bufpt>>2]|0;
      $1018 = (($1017) + ($1015)|0);
      HEAP8[$1018>>0] = $1014;
     }
     $1019 = HEAP32[$i2>>2]|0;
     $1020 = (($1019) + 1)|0;
     HEAP32[$i2>>2] = $1020;
    }
    $1021 = HEAP32[$needQuote>>2]|0;
    $1022 = ($1021|0)!=(0);
    if ($1022) {
     $1023 = HEAP8[$q>>0]|0;
     $1024 = HEAP32[$j>>2]|0;
     $1025 = (($1024) + 1)|0;
     HEAP32[$j>>2] = $1025;
     $1026 = HEAP32[$bufpt>>2]|0;
     $1027 = (($1026) + ($1024)|0);
     HEAP8[$1027>>0] = $1023;
    }
    $1028 = HEAP32[$j>>2]|0;
    $1029 = HEAP32[$bufpt>>2]|0;
    $1030 = (($1029) + ($1028)|0);
    HEAP8[$1030>>0] = 0;
    $1031 = HEAP32[$j>>2]|0;
    HEAP32[$length>>2] = $1031;
    break;
   }
   case 11:  {
    $1032 = HEAP32[$2>>2]|0;
    $arglist_current45 = HEAP32[$1032>>2]|0;
    $1033 = $arglist_current45;
    $1034 = ((0) + 4|0);
    $expanded137 = $1034;
    $expanded136 = (($expanded137) - 1)|0;
    $1035 = (($1033) + ($expanded136))|0;
    $1036 = ((0) + 4|0);
    $expanded141 = $1036;
    $expanded140 = (($expanded141) - 1)|0;
    $expanded139 = $expanded140 ^ -1;
    $1037 = $1035 & $expanded139;
    $1038 = $1037;
    $1039 = HEAP32[$1038>>2]|0;
    $arglist_next46 = ((($1038)) + 4|0);
    HEAP32[$1032>>2] = $arglist_next46;
    HEAP32[$pToken>>2] = $1039;
    $1040 = HEAP32[$pToken>>2]|0;
    $1041 = ($1040|0)!=(0|0);
    if ($1041) {
     $1042 = HEAP32[$pToken>>2]|0;
     $1043 = ((($1042)) + 4|0);
     $1044 = HEAP32[$1043>>2]|0;
     $1045 = ($1044|0)!=(0);
     if ($1045) {
      $1046 = HEAP32[$0>>2]|0;
      $1047 = HEAP32[$pToken>>2]|0;
      $1048 = HEAP32[$1047>>2]|0;
      $1049 = HEAP32[$pToken>>2]|0;
      $1050 = ((($1049)) + 4|0);
      $1051 = HEAP32[$1050>>2]|0;
      _sqlite3StrAccumAppend($1046,$1048,$1051);
     }
    }
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   case 12:  {
    $1052 = HEAP32[$2>>2]|0;
    $arglist_current48 = HEAP32[$1052>>2]|0;
    $1053 = $arglist_current48;
    $1054 = ((0) + 4|0);
    $expanded144 = $1054;
    $expanded143 = (($expanded144) - 1)|0;
    $1055 = (($1053) + ($expanded143))|0;
    $1056 = ((0) + 4|0);
    $expanded148 = $1056;
    $expanded147 = (($expanded148) - 1)|0;
    $expanded146 = $expanded147 ^ -1;
    $1057 = $1055 & $expanded146;
    $1058 = $1057;
    $1059 = HEAP32[$1058>>2]|0;
    $arglist_next49 = ((($1058)) + 4|0);
    HEAP32[$1052>>2] = $arglist_next49;
    HEAP32[$pSrc>>2] = $1059;
    $1060 = HEAP32[$2>>2]|0;
    $arglist_current51 = HEAP32[$1060>>2]|0;
    $1061 = $arglist_current51;
    $1062 = ((0) + 4|0);
    $expanded151 = $1062;
    $expanded150 = (($expanded151) - 1)|0;
    $1063 = (($1061) + ($expanded150))|0;
    $1064 = ((0) + 4|0);
    $expanded155 = $1064;
    $expanded154 = (($expanded155) - 1)|0;
    $expanded153 = $expanded154 ^ -1;
    $1065 = $1063 & $expanded153;
    $1066 = $1065;
    $1067 = HEAP32[$1066>>2]|0;
    $arglist_next52 = ((($1066)) + 4|0);
    HEAP32[$1060>>2] = $arglist_next52;
    HEAP32[$k3>>2] = $1067;
    $1068 = HEAP32[$k3>>2]|0;
    $1069 = HEAP32[$pSrc>>2]|0;
    $1070 = ((($1069)) + 8|0);
    $1071 = (($1070) + (($1068*72)|0)|0);
    HEAP32[$pItem>>2] = $1071;
    $1072 = HEAP32[$pItem>>2]|0;
    $1073 = ((($1072)) + 4|0);
    $1074 = HEAP32[$1073>>2]|0;
    $1075 = ($1074|0)!=(0|0);
    if ($1075) {
     $1076 = HEAP32[$0>>2]|0;
     $1077 = HEAP32[$pItem>>2]|0;
     $1078 = ((($1077)) + 4|0);
     $1079 = HEAP32[$1078>>2]|0;
     _sqlite3StrAccumAppendAll($1076,$1079);
     $1080 = HEAP32[$0>>2]|0;
     _sqlite3StrAccumAppend($1080,44433,1);
    }
    $1081 = HEAP32[$0>>2]|0;
    $1082 = HEAP32[$pItem>>2]|0;
    $1083 = ((($1082)) + 8|0);
    $1084 = HEAP32[$1083>>2]|0;
    _sqlite3StrAccumAppendAll($1081,$1084);
    HEAP32[$width>>2] = 0;
    HEAP32[$length>>2] = 0;
    break;
   }
   default: {
    label = 268;
    break L7;
   }
   }
  } while(0);
  if ((label|0) == 62) {
   label = 0;
   $196 = HEAP32[$infop>>2]|0;
   $197 = ((($196)) + 2|0);
   $198 = HEAP8[$197>>0]|0;
   $199 = $198&255;
   $200 = $199 & 1;
   $201 = ($200|0)!=(0);
   $202 = HEAP8[$bArgList>>0]|0;
   $203 = ($202<<24>>24)!=(0);
   do {
    if ($201) {
     do {
      if ($203) {
       $204 = HEAP32[$pArgList>>2]|0;
       $205 = (_getIntArg($204)|0);
       $206 = tempRet0;
       $207 = $v;
       $208 = $207;
       HEAP32[$208>>2] = $205;
       $209 = (($207) + 4)|0;
       $210 = $209;
       HEAP32[$210>>2] = $206;
      } else {
       $211 = HEAP8[$flag_longlong>>0]|0;
       $212 = ($211<<24>>24)!=(0);
       $213 = HEAP32[$2>>2]|0;
       if ($212) {
        $arglist_current18 = HEAP32[$213>>2]|0;
        $214 = $arglist_current18;
        $215 = ((0) + 8|0);
        $expanded74 = $215;
        $expanded73 = (($expanded74) - 1)|0;
        $216 = (($214) + ($expanded73))|0;
        $217 = ((0) + 8|0);
        $expanded78 = $217;
        $expanded77 = (($expanded78) - 1)|0;
        $expanded76 = $expanded77 ^ -1;
        $218 = $216 & $expanded76;
        $219 = $218;
        $220 = $219;
        $221 = $220;
        $222 = HEAP32[$221>>2]|0;
        $223 = (($220) + 4)|0;
        $224 = $223;
        $225 = HEAP32[$224>>2]|0;
        $arglist_next19 = ((($219)) + 8|0);
        HEAP32[$213>>2] = $arglist_next19;
        $226 = $v;
        $227 = $226;
        HEAP32[$227>>2] = $222;
        $228 = (($226) + 4)|0;
        $229 = $228;
        HEAP32[$229>>2] = $225;
        break;
       } else {
        $arglist_current21 = HEAP32[$213>>2]|0;
        $230 = $arglist_current21;
        $231 = ((0) + 4|0);
        $expanded81 = $231;
        $expanded80 = (($expanded81) - 1)|0;
        $232 = (($230) + ($expanded80))|0;
        $233 = ((0) + 4|0);
        $expanded85 = $233;
        $expanded84 = (($expanded85) - 1)|0;
        $expanded83 = $expanded84 ^ -1;
        $234 = $232 & $expanded83;
        $235 = $234;
        $236 = HEAP32[$235>>2]|0;
        $arglist_next22 = ((($235)) + 4|0);
        HEAP32[$213>>2] = $arglist_next22;
        $237 = ($236|0)<(0);
        $238 = $237 << 31 >> 31;
        $239 = $v;
        $240 = $239;
        HEAP32[$240>>2] = $236;
        $241 = (($239) + 4)|0;
        $242 = $241;
        HEAP32[$242>>2] = $238;
        break;
       }
      }
     } while(0);
     $243 = $v;
     $244 = $243;
     $245 = HEAP32[$244>>2]|0;
     $246 = (($243) + 4)|0;
     $247 = $246;
     $248 = HEAP32[$247>>2]|0;
     $249 = ($248|0)<(0);
     $250 = $v;
     $251 = $250;
     $252 = HEAP32[$251>>2]|0;
     $253 = (($250) + 4)|0;
     $254 = $253;
     $255 = HEAP32[$254>>2]|0;
     if ($249) {
      $256 = ($252|0)==(0);
      $257 = ($255|0)==(-2147483648);
      $258 = $256 & $257;
      if ($258) {
       $259 = $longvalue;
       $260 = $259;
       HEAP32[$260>>2] = 0;
       $261 = (($259) + 4)|0;
       $262 = $261;
       HEAP32[$262>>2] = -2147483648;
      } else {
       $263 = $v;
       $264 = $263;
       $265 = HEAP32[$264>>2]|0;
       $266 = (($263) + 4)|0;
       $267 = $266;
       $268 = HEAP32[$267>>2]|0;
       $269 = (_i64Subtract(0,0,($265|0),($268|0))|0);
       $270 = tempRet0;
       $271 = $longvalue;
       $272 = $271;
       HEAP32[$272>>2] = $269;
       $273 = (($271) + 4)|0;
       $274 = $273;
       HEAP32[$274>>2] = $270;
      }
      HEAP8[$prefix>>0] = 45;
      break;
     }
     $275 = $longvalue;
     $276 = $275;
     HEAP32[$276>>2] = $252;
     $277 = (($275) + 4)|0;
     $278 = $277;
     HEAP32[$278>>2] = $255;
     $279 = HEAP8[$flag_plussign>>0]|0;
     $280 = ($279<<24>>24)!=(0);
     if ($280) {
      HEAP8[$prefix>>0] = 43;
      break;
     }
     $281 = HEAP8[$flag_blanksign>>0]|0;
     $282 = ($281<<24>>24)!=(0);
     if ($282) {
      HEAP8[$prefix>>0] = 32;
      break;
     } else {
      HEAP8[$prefix>>0] = 0;
      break;
     }
    } else {
     do {
      if ($203) {
       $283 = HEAP32[$pArgList>>2]|0;
       $284 = (_getIntArg($283)|0);
       $285 = tempRet0;
       $286 = $longvalue;
       $287 = $286;
       HEAP32[$287>>2] = $284;
       $288 = (($286) + 4)|0;
       $289 = $288;
       HEAP32[$289>>2] = $285;
      } else {
       $290 = HEAP8[$flag_longlong>>0]|0;
       $291 = ($290<<24>>24)!=(0);
       $292 = HEAP32[$2>>2]|0;
       if ($291) {
        $arglist_current24 = HEAP32[$292>>2]|0;
        $293 = $arglist_current24;
        $294 = ((0) + 8|0);
        $expanded88 = $294;
        $expanded87 = (($expanded88) - 1)|0;
        $295 = (($293) + ($expanded87))|0;
        $296 = ((0) + 8|0);
        $expanded92 = $296;
        $expanded91 = (($expanded92) - 1)|0;
        $expanded90 = $expanded91 ^ -1;
        $297 = $295 & $expanded90;
        $298 = $297;
        $299 = $298;
        $300 = $299;
        $301 = HEAP32[$300>>2]|0;
        $302 = (($299) + 4)|0;
        $303 = $302;
        $304 = HEAP32[$303>>2]|0;
        $arglist_next25 = ((($298)) + 8|0);
        HEAP32[$292>>2] = $arglist_next25;
        $305 = $longvalue;
        $306 = $305;
        HEAP32[$306>>2] = $301;
        $307 = (($305) + 4)|0;
        $308 = $307;
        HEAP32[$308>>2] = $304;
        break;
       } else {
        $arglist_current27 = HEAP32[$292>>2]|0;
        $309 = $arglist_current27;
        $310 = ((0) + 4|0);
        $expanded95 = $310;
        $expanded94 = (($expanded95) - 1)|0;
        $311 = (($309) + ($expanded94))|0;
        $312 = ((0) + 4|0);
        $expanded99 = $312;
        $expanded98 = (($expanded99) - 1)|0;
        $expanded97 = $expanded98 ^ -1;
        $313 = $311 & $expanded97;
        $314 = $313;
        $315 = HEAP32[$314>>2]|0;
        $arglist_next28 = ((($314)) + 4|0);
        HEAP32[$292>>2] = $arglist_next28;
        $316 = $longvalue;
        $317 = $316;
        HEAP32[$317>>2] = $315;
        $318 = (($316) + 4)|0;
        $319 = $318;
        HEAP32[$319>>2] = 0;
        break;
       }
      }
     } while(0);
     HEAP8[$prefix>>0] = 0;
    }
   } while(0);
   $320 = $longvalue;
   $321 = $320;
   $322 = HEAP32[$321>>2]|0;
   $323 = (($320) + 4)|0;
   $324 = $323;
   $325 = HEAP32[$324>>2]|0;
   $326 = ($322|0)==(0);
   $327 = ($325|0)==(0);
   $328 = $326 & $327;
   if ($328) {
    HEAP8[$flag_alternateform>>0] = 0;
   }
   $329 = HEAP8[$flag_zeropad>>0]|0;
   $330 = $329&255;
   $331 = ($330|0)!=(0);
   if ($331) {
    $332 = HEAP32[$precision>>2]|0;
    $333 = HEAP32[$width>>2]|0;
    $334 = HEAP8[$prefix>>0]|0;
    $335 = $334 << 24 >> 24;
    $336 = ($335|0)!=(0);
    $337 = $336&1;
    $338 = (($333) - ($337))|0;
    $339 = ($332|0)<($338|0);
    if ($339) {
     $340 = HEAP32[$width>>2]|0;
     $341 = HEAP8[$prefix>>0]|0;
     $342 = $341 << 24 >> 24;
     $343 = ($342|0)!=(0);
     $344 = $343&1;
     $345 = (($340) - ($344))|0;
     HEAP32[$precision>>2] = $345;
    }
   }
   $346 = HEAP32[$precision>>2]|0;
   $347 = ($346|0)<(60);
   if ($347) {
    HEAP32[$nOut>>2] = 70;
    HEAP32[$zOut>>2] = $buf;
   } else {
    $348 = HEAP32[$precision>>2]|0;
    $349 = (($348) + 10)|0;
    HEAP32[$nOut>>2] = $349;
    $350 = HEAP32[$nOut>>2]|0;
    $351 = ($350|0)<(0);
    $352 = $351 << 31 >> 31;
    $353 = (_sqlite3Malloc($350,$352)|0);
    HEAP32[$zExtra>>2] = $353;
    HEAP32[$zOut>>2] = $353;
    $354 = HEAP32[$zOut>>2]|0;
    $355 = ($354|0)==(0|0);
    if ($355) {
     label = 92;
     break;
    }
   }
   $357 = HEAP32[$nOut>>2]|0;
   $358 = (($357) - 1)|0;
   $359 = HEAP32[$zOut>>2]|0;
   $360 = (($359) + ($358)|0);
   HEAP32[$bufpt>>2] = $360;
   $361 = HEAP8[$xtype>>0]|0;
   $362 = $361&255;
   $363 = ($362|0)==(15);
   if ($363) {
    $364 = $longvalue;
    $365 = $364;
    $366 = HEAP32[$365>>2]|0;
    $367 = (($364) + 4)|0;
    $368 = $367;
    $369 = HEAP32[$368>>2]|0;
    $370 = (___uremdi3(($366|0),($369|0),10,0)|0);
    $371 = tempRet0;
    HEAP32[$x>>2] = $370;
    $372 = HEAP32[$x>>2]|0;
    $373 = ($372|0)>=(4);
    if ($373) {
     label = 96;
    } else {
     $374 = $longvalue;
     $375 = $374;
     $376 = HEAP32[$375>>2]|0;
     $377 = (($374) + 4)|0;
     $378 = $377;
     $379 = HEAP32[$378>>2]|0;
     $380 = (___udivdi3(($376|0),($379|0),10,0)|0);
     $381 = tempRet0;
     $382 = (___uremdi3(($380|0),($381|0),10,0)|0);
     $383 = tempRet0;
     $384 = ($382|0)==(1);
     $385 = ($383|0)==(0);
     $386 = $384 & $385;
     if ($386) {
      label = 96;
     }
    }
    if ((label|0) == 96) {
     label = 0;
     HEAP32[$x>>2] = 0;
    }
    $387 = HEAP32[$x>>2]|0;
    $388 = $387<<1;
    $389 = (($388) + 1)|0;
    $390 = (44753 + ($389)|0);
    $391 = HEAP8[$390>>0]|0;
    $392 = HEAP32[$bufpt>>2]|0;
    $393 = ((($392)) + -1|0);
    HEAP32[$bufpt>>2] = $393;
    HEAP8[$393>>0] = $391;
    $394 = HEAP32[$x>>2]|0;
    $395 = $394<<1;
    $396 = (44753 + ($395)|0);
    $397 = HEAP8[$396>>0]|0;
    $398 = HEAP32[$bufpt>>2]|0;
    $399 = ((($398)) + -1|0);
    HEAP32[$bufpt>>2] = $399;
    HEAP8[$399>>0] = $397;
   }
   $400 = HEAP32[$infop>>2]|0;
   $401 = ((($400)) + 4|0);
   $402 = HEAP8[$401>>0]|0;
   $403 = $402&255;
   $404 = (44762 + ($403)|0);
   HEAP32[$cset>>2] = $404;
   $405 = HEAP32[$infop>>2]|0;
   $406 = ((($405)) + 1|0);
   $407 = HEAP8[$406>>0]|0;
   HEAP8[$base>>0] = $407;
   while(1) {
    $408 = $longvalue;
    $409 = $408;
    $410 = HEAP32[$409>>2]|0;
    $411 = (($408) + 4)|0;
    $412 = $411;
    $413 = HEAP32[$412>>2]|0;
    $414 = HEAP8[$base>>0]|0;
    $415 = $414&255;
    $416 = (___uremdi3(($410|0),($413|0),($415|0),0)|0);
    $417 = tempRet0;
    $418 = HEAP32[$cset>>2]|0;
    $419 = (($418) + ($416)|0);
    $420 = HEAP8[$419>>0]|0;
    $421 = HEAP32[$bufpt>>2]|0;
    $422 = ((($421)) + -1|0);
    HEAP32[$bufpt>>2] = $422;
    HEAP8[$422>>0] = $420;
    $423 = $longvalue;
    $424 = $423;
    $425 = HEAP32[$424>>2]|0;
    $426 = (($423) + 4)|0;
    $427 = $426;
    $428 = HEAP32[$427>>2]|0;
    $429 = HEAP8[$base>>0]|0;
    $430 = $429&255;
    $431 = (___udivdi3(($425|0),($428|0),($430|0),0)|0);
    $432 = tempRet0;
    $433 = $longvalue;
    $434 = $433;
    HEAP32[$434>>2] = $431;
    $435 = (($433) + 4)|0;
    $436 = $435;
    HEAP32[$436>>2] = $432;
    $437 = $longvalue;
    $438 = $437;
    $439 = HEAP32[$438>>2]|0;
    $440 = (($437) + 4)|0;
    $441 = $440;
    $442 = HEAP32[$441>>2]|0;
    $443 = ($442>>>0)>(0);
    $444 = ($439>>>0)>(0);
    $445 = ($442|0)==(0);
    $446 = $445 & $444;
    $447 = $443 | $446;
    if (!($447)) {
     break;
    }
   }
   $448 = HEAP32[$nOut>>2]|0;
   $449 = (($448) - 1)|0;
   $450 = HEAP32[$zOut>>2]|0;
   $451 = (($450) + ($449)|0);
   $452 = HEAP32[$bufpt>>2]|0;
   $453 = $451;
   $454 = $452;
   $455 = (($453) - ($454))|0;
   HEAP32[$length>>2] = $455;
   $456 = HEAP32[$precision>>2]|0;
   $457 = HEAP32[$length>>2]|0;
   $458 = (($456) - ($457))|0;
   HEAP32[$idx>>2] = $458;
   while(1) {
    $459 = HEAP32[$idx>>2]|0;
    $460 = ($459|0)>(0);
    if (!($460)) {
     break;
    }
    $461 = HEAP32[$bufpt>>2]|0;
    $462 = ((($461)) + -1|0);
    HEAP32[$bufpt>>2] = $462;
    HEAP8[$462>>0] = 48;
    $463 = HEAP32[$idx>>2]|0;
    $464 = (($463) + -1)|0;
    HEAP32[$idx>>2] = $464;
   }
   $465 = HEAP8[$prefix>>0]|0;
   $466 = ($465<<24>>24)!=(0);
   if ($466) {
    $467 = HEAP8[$prefix>>0]|0;
    $468 = HEAP32[$bufpt>>2]|0;
    $469 = ((($468)) + -1|0);
    HEAP32[$bufpt>>2] = $469;
    HEAP8[$469>>0] = $467;
   }
   $470 = HEAP8[$flag_alternateform>>0]|0;
   $471 = $470&255;
   $472 = ($471|0)!=(0);
   L345: do {
    if ($472) {
     $473 = HEAP32[$infop>>2]|0;
     $474 = ((($473)) + 5|0);
     $475 = HEAP8[$474>>0]|0;
     $476 = $475&255;
     $477 = ($476|0)!=(0);
     if ($477) {
      $478 = HEAP32[$infop>>2]|0;
      $479 = ((($478)) + 5|0);
      $480 = HEAP8[$479>>0]|0;
      $481 = $480&255;
      $482 = (44795 + ($481)|0);
      HEAP32[$pre>>2] = $482;
      while(1) {
       $483 = HEAP32[$pre>>2]|0;
       $484 = HEAP8[$483>>0]|0;
       HEAP8[$x1>>0] = $484;
       $485 = $484 << 24 >> 24;
       $486 = ($485|0)!=(0);
       if (!($486)) {
        break L345;
       }
       $487 = HEAP8[$x1>>0]|0;
       $488 = HEAP32[$bufpt>>2]|0;
       $489 = ((($488)) + -1|0);
       HEAP32[$bufpt>>2] = $489;
       HEAP8[$489>>0] = $487;
       $490 = HEAP32[$pre>>2]|0;
       $491 = ((($490)) + 1|0);
       HEAP32[$pre>>2] = $491;
      }
     }
    }
   } while(0);
   $492 = HEAP32[$nOut>>2]|0;
   $493 = (($492) - 1)|0;
   $494 = HEAP32[$zOut>>2]|0;
   $495 = (($494) + ($493)|0);
   $496 = HEAP32[$bufpt>>2]|0;
   $497 = $495;
   $498 = $496;
   $499 = (($497) - ($498))|0;
   HEAP32[$length>>2] = $499;
  }
  $1085 = HEAP32[$length>>2]|0;
  $1086 = HEAP32[$width>>2]|0;
  $1087 = (($1086) - ($1085))|0;
  HEAP32[$width>>2] = $1087;
  $1088 = HEAP32[$width>>2]|0;
  $1089 = ($1088|0)<=(0);
  $1090 = HEAP8[$flag_leftjustify>>0]|0;
  $1091 = ($1090<<24>>24)!=(0);
  $or$cond10 = $1089 | $1091;
  if (!($or$cond10)) {
   $1092 = HEAP32[$0>>2]|0;
   $1093 = HEAP32[$width>>2]|0;
   _sqlite3AppendChar($1092,$1093,32);
  }
  $1094 = HEAP32[$0>>2]|0;
  $1095 = HEAP32[$bufpt>>2]|0;
  $1096 = HEAP32[$length>>2]|0;
  _sqlite3StrAccumAppend($1094,$1095,$1096);
  $1097 = HEAP32[$width>>2]|0;
  $1098 = ($1097|0)>(0);
  if ($1098) {
   $1099 = HEAP8[$flag_leftjustify>>0]|0;
   $1100 = $1099&255;
   $1101 = ($1100|0)!=(0);
   if ($1101) {
    $1102 = HEAP32[$0>>2]|0;
    $1103 = HEAP32[$width>>2]|0;
    _sqlite3AppendChar($1102,$1103,32);
   }
  }
  $1104 = HEAP32[$zExtra>>2]|0;
  $1105 = ($1104|0)!=(0|0);
  if ($1105) {
   $1106 = HEAP32[$0>>2]|0;
   $1107 = HEAP32[$1106>>2]|0;
   $1108 = HEAP32[$zExtra>>2]|0;
   _sqlite3DbFree($1107,$1108);
   HEAP32[$zExtra>>2] = 0;
  }
  $1109 = HEAP32[$1>>2]|0;
  $1110 = ((($1109)) + 1|0);
  HEAP32[$1>>2] = $1110;
 }
 if ((label|0) == 13) {
  $62 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($62,44613,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 92) {
  $356 = HEAP32[$0>>2]|0;
  _setStrAccumError($356,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 161) {
  $683 = HEAP32[$0>>2]|0;
  _setStrAccumError($683,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 241) {
  $989 = HEAP32[$0>>2]|0;
  _setStrAccumError($989,1);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 268) {
  STACKTOP = sp;return;
 }
}
function _sqlite3StrAccumFinish($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$0>>2]|0;
   $6 = ((($5)) + 12|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = HEAP32[$0>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (($10) + ($7)|0);
   HEAP8[$11>>0] = 0;
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 20|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14>>>0)>(0);
   if ($15) {
    $16 = HEAP32[$0>>2]|0;
    $17 = ((($16)) + 25|0);
    $18 = HEAP8[$17>>0]|0;
    $19 = $18&255;
    $20 = $19 & 4;
    $21 = ($20|0)!=(0);
    if (!($21)) {
     $22 = HEAP32[$0>>2]|0;
     $23 = HEAP32[$22>>2]|0;
     $24 = HEAP32[$0>>2]|0;
     $25 = ((($24)) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = (($26) + 1)|0;
     $28 = (_sqlite3DbMallocRaw($23,$27,0)|0);
     $29 = HEAP32[$0>>2]|0;
     $30 = ((($29)) + 8|0);
     HEAP32[$30>>2] = $28;
     $31 = HEAP32[$0>>2]|0;
     $32 = ((($31)) + 8|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($33|0)!=(0|0);
     $35 = HEAP32[$0>>2]|0;
     if ($34) {
      $36 = ((($35)) + 8|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = HEAP32[$0>>2]|0;
      $39 = ((($38)) + 4|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = HEAP32[$0>>2]|0;
      $42 = ((($41)) + 12|0);
      $43 = HEAP32[$42>>2]|0;
      $44 = (($43) + 1)|0;
      _memcpy(($37|0),($40|0),($44|0))|0;
      $45 = HEAP32[$0>>2]|0;
      $46 = ((($45)) + 25|0);
      $47 = HEAP8[$46>>0]|0;
      $48 = $47&255;
      $49 = $48 | 4;
      $50 = $49&255;
      HEAP8[$46>>0] = $50;
      break;
     } else {
      _setStrAccumError($35,1);
      break;
     }
    }
   }
  }
 } while(0);
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 8|0);
 $53 = HEAP32[$52>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _renderLogMsg($iErrCode,$zFormat,$ap) {
 $iErrCode = $iErrCode|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $acc = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $acc = sp;
 $zMsg = sp + 40|0;
 HEAP32[$0>>2] = $iErrCode;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 _sqlite3StrAccumInit($acc,0,$zMsg,210,0);
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 _sqlite3VXPrintf($acc,$3,$4);
 $5 = HEAP32[(264)>>2]|0;
 $6 = HEAP32[(268)>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3StrAccumFinish($acc)|0);
 FUNCTION_TABLE_viii[$5 & 255]($6,$7,$8);
 STACKTOP = sp;return;
}
function _sqlite3OsRandomness($pVfs,$nByte,$zBufOut) {
 $pVfs = $pVfs|0;
 $nByte = $nByte|0;
 $zBufOut = $zBufOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $nByte;
 HEAP32[$2>>2] = $zBufOut;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (FUNCTION_TABLE_iiii[$5 & 255]($6,$7,$8)|0);
 STACKTOP = sp;return ($9|0);
}
function _sqlite3StrICmp($zLeft,$zRight) {
 $zLeft = $zLeft|0;
 $zRight = $zRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $a = sp + 8|0;
 $b = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $zLeft;
 HEAP32[$1>>2] = $zRight;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$a>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$b>>2] = $3;
 while(1) {
  $4 = HEAP32[$a>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5&255;
  $7 = (17613 + ($6)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  $10 = HEAP32[$b>>2]|0;
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = (17613 + ($12)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = (($9) - ($15))|0;
  HEAP32[$c>>2] = $16;
  $17 = HEAP32[$c>>2]|0;
  $18 = ($17|0)!=(0);
  if ($18) {
   label = 5;
   break;
  }
  $19 = HEAP32[$a>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = ($21|0)==(0);
  if ($22) {
   label = 5;
   break;
  }
  $23 = HEAP32[$a>>2]|0;
  $24 = ((($23)) + 1|0);
  HEAP32[$a>>2] = $24;
  $25 = HEAP32[$b>>2]|0;
  $26 = ((($25)) + 1|0);
  HEAP32[$b>>2] = $26;
 }
 if ((label|0) == 5) {
  $27 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3ErrorWithMsg($db,$err_code,$zFormat,$varargs) {
 $db = $db|0;
 $err_code = $err_code|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $z = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $err_code;
 HEAP32[$2>>2] = $zFormat;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 52|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3SystemError($6,$7);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(0|0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$1>>2]|0;
  _sqlite3Error($10,$11);
  STACKTOP = sp;return;
 }
 $12 = ((($10)) + 244|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  $15 = HEAP32[$0>>2]|0;
  $16 = (_sqlite3ValueNew($15)|0);
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 244|0);
  HEAP32[$18>>2] = $16;
  $19 = ($16|0)!=(0|0);
  if (!($19)) {
   STACKTOP = sp;return;
  }
 }
 HEAP32[$ap>>2] = $varargs;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (_sqlite3VMPrintf($20,$21,$ap)|0);
 HEAP32[$z>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 244|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$z>>2]|0;
 _sqlite3ValueSetStr($25,-1,$26,1,170);
 STACKTOP = sp;return;
}
function _sqlite3MallocZero($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $p = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 $7 = $2;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3Malloc($9,$12)|0);
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $23 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = $2;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 _memset(($16|0),0,($19|0))|0;
 $23 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3Error($db,$err_code) {
 $db = $db|0;
 $err_code = $err_code|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $err_code;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 52|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 244|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   STACKTOP = sp;return;
  }
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 _sqlite3ErrorFinish($11,$12);
 STACKTOP = sp;return;
}
function _sqlite3BtreeEnter($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = $2;
 STACKTOP = sp;return;
}
function _isFatalError($rc) {
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $rc;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0);
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(5);
 $or$cond = $2 & $4;
 if (!($or$cond)) {
  $8 = 0;
  $7 = $8&1;
  STACKTOP = sp;return ($7|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)!=(6);
 $8 = $6;
 $7 = $8&1;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3BtreeIsInReadTrans($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BtreeBeginTrans($p,$wrflag) {
 $p = $p|0;
 $wrflag = $wrflag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond9 = 0, $pBlock = 0, $pBt = 0, $pIter = 0, $pPage1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pBt = sp + 16|0;
 $rc = sp + 12|0;
 $pBlock = sp + 8|0;
 $pIter = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $wrflag;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(2);
 do {
  if (!($10)) {
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 8|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(1);
   $16 = HEAP32[$1>>2]|0;
   $17 = ($16|0)!=(0);
   $or$cond = $15 | $17;
   if ($or$cond) {
    $18 = HEAP32[$pBt>>2]|0;
    $19 = ((($18)) + 22|0);
    $20 = HEAP16[$19>>1]|0;
    $21 = $20&65535;
    $22 = $21 & 1;
    $23 = ($22|0)!=(0);
    $24 = HEAP32[$1>>2]|0;
    $25 = ($24|0)!=(0);
    $or$cond3 = $23 & $25;
    if ($or$cond3) {
     HEAP32[$rc>>2] = 8;
     break;
    }
    HEAP32[$pBlock>>2] = 0;
    $26 = HEAP32[$1>>2]|0;
    $27 = ($26|0)!=(0);
    if ($27) {
     $28 = HEAP32[$pBt>>2]|0;
     $29 = ((($28)) + 20|0);
     $30 = HEAP8[$29>>0]|0;
     $31 = $30&255;
     $32 = ($31|0)==(2);
     if ($32) {
      label = 8;
     } else {
      label = 7;
     }
    } else {
     label = 7;
    }
    L9: do {
     if ((label|0) == 7) {
      $33 = HEAP32[$pBt>>2]|0;
      $34 = ((($33)) + 22|0);
      $35 = HEAP16[$34>>1]|0;
      $36 = $35&65535;
      $37 = $36 & 64;
      $38 = ($37|0)!=(0);
      if ($38) {
       label = 8;
      } else {
       $43 = HEAP32[$1>>2]|0;
       $44 = ($43|0)>(1);
       if ($44) {
        $45 = HEAP32[$pBt>>2]|0;
        $46 = ((($45)) + 72|0);
        $47 = HEAP32[$46>>2]|0;
        HEAP32[$pIter>>2] = $47;
        while(1) {
         $48 = HEAP32[$pIter>>2]|0;
         $49 = ($48|0)!=(0|0);
         if (!($49)) {
          break L9;
         }
         $50 = HEAP32[$pIter>>2]|0;
         $51 = HEAP32[$50>>2]|0;
         $52 = HEAP32[$0>>2]|0;
         $53 = ($51|0)!=($52|0);
         $54 = HEAP32[$pIter>>2]|0;
         if ($53) {
          break;
         }
         $57 = ((($54)) + 12|0);
         $58 = HEAP32[$57>>2]|0;
         HEAP32[$pIter>>2] = $58;
        }
        $55 = HEAP32[$54>>2]|0;
        $56 = HEAP32[$55>>2]|0;
        HEAP32[$pBlock>>2] = $56;
       }
      }
     }
    } while(0);
    if ((label|0) == 8) {
     $39 = HEAP32[$pBt>>2]|0;
     $40 = ((($39)) + 76|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     HEAP32[$pBlock>>2] = $42;
    }
    $59 = HEAP32[$pBlock>>2]|0;
    $60 = ($59|0)!=(0|0);
    if ($60) {
     HEAP32[$rc>>2] = 262;
     break;
    }
    $61 = HEAP32[$0>>2]|0;
    $62 = (_querySharedCacheTableLock($61,1,1)|0);
    HEAP32[$rc>>2] = $62;
    $63 = HEAP32[$rc>>2]|0;
    $64 = (0)!=($63|0);
    if (!($64)) {
     $65 = HEAP32[$pBt>>2]|0;
     $66 = ((($65)) + 22|0);
     $67 = HEAP16[$66>>1]|0;
     $68 = $67&65535;
     $69 = $68 & -9;
     $70 = $69&65535;
     HEAP16[$66>>1] = $70;
     $71 = HEAP32[$pBt>>2]|0;
     $72 = ((($71)) + 44|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = ($73|0)==(0);
     if ($74) {
      $75 = HEAP32[$pBt>>2]|0;
      $76 = ((($75)) + 22|0);
      $77 = HEAP16[$76>>1]|0;
      $78 = $77&65535;
      $79 = $78 | 8;
      $80 = $79&65535;
      HEAP16[$76>>1] = $80;
     }
     while(1) {
      $81 = HEAP32[$pBt>>2]|0;
      $82 = ((($81)) + 12|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = ($83|0)==(0|0);
      if ($84) {
       $85 = HEAP32[$pBt>>2]|0;
       $86 = (_lockBtree($85)|0);
       HEAP32[$rc>>2] = $86;
       $87 = (0)==($86|0);
       if ($87) {
        continue;
       }
      }
      $88 = HEAP32[$rc>>2]|0;
      $89 = ($88|0)==(0);
      $90 = HEAP32[$1>>2]|0;
      $91 = ($90|0)!=(0);
      $or$cond5 = $89 & $91;
      do {
       if ($or$cond5) {
        $92 = HEAP32[$pBt>>2]|0;
        $93 = ((($92)) + 22|0);
        $94 = HEAP16[$93>>1]|0;
        $95 = $94&65535;
        $96 = $95 & 1;
        $97 = ($96|0)!=(0);
        if ($97) {
         HEAP32[$rc>>2] = 8;
         break;
        }
        $98 = HEAP32[$pBt>>2]|0;
        $99 = HEAP32[$98>>2]|0;
        $100 = HEAP32[$1>>2]|0;
        $101 = ($100|0)>(1);
        $102 = $101&1;
        $103 = HEAP32[$0>>2]|0;
        $104 = HEAP32[$103>>2]|0;
        $105 = (_sqlite3TempInMemory($104)|0);
        $106 = (_sqlite3PagerBegin($99,$102,$105)|0);
        HEAP32[$rc>>2] = $106;
        $107 = HEAP32[$rc>>2]|0;
        $108 = ($107|0)==(0);
        if ($108) {
         $109 = HEAP32[$pBt>>2]|0;
         $110 = (_newDatabase($109)|0);
         HEAP32[$rc>>2] = $110;
        }
       }
      } while(0);
      $111 = HEAP32[$rc>>2]|0;
      $112 = ($111|0)!=(0);
      if ($112) {
       $113 = HEAP32[$pBt>>2]|0;
       _unlockBtreeIfUnused($113);
      }
      $114 = HEAP32[$rc>>2]|0;
      $115 = $114 & 255;
      $116 = ($115|0)==(5);
      if (!($116)) {
       break;
      }
      $117 = HEAP32[$pBt>>2]|0;
      $118 = ((($117)) + 20|0);
      $119 = HEAP8[$118>>0]|0;
      $120 = $119&255;
      $121 = ($120|0)==(0);
      if (!($121)) {
       break;
      }
      $122 = HEAP32[$pBt>>2]|0;
      $123 = (_btreeInvokeBusyHandler($122)|0);
      $124 = ($123|0)!=(0);
      if (!($124)) {
       break;
      }
     }
     $125 = HEAP32[$rc>>2]|0;
     $126 = ($125|0)==(0);
     if ($126) {
      $127 = HEAP32[$0>>2]|0;
      $128 = ((($127)) + 8|0);
      $129 = HEAP8[$128>>0]|0;
      $130 = $129&255;
      $131 = ($130|0)==(0);
      if ($131) {
       $132 = HEAP32[$pBt>>2]|0;
       $133 = ((($132)) + 40|0);
       $134 = HEAP32[$133>>2]|0;
       $135 = (($134) + 1)|0;
       HEAP32[$133>>2] = $135;
       $136 = HEAP32[$0>>2]|0;
       $137 = ((($136)) + 9|0);
       $138 = HEAP8[$137>>0]|0;
       $139 = ($138<<24>>24)!=(0);
       if ($139) {
        $140 = HEAP32[$0>>2]|0;
        $141 = ((($140)) + 32|0);
        $142 = ((($141)) + 8|0);
        HEAP8[$142>>0] = 1;
        $143 = HEAP32[$pBt>>2]|0;
        $144 = ((($143)) + 72|0);
        $145 = HEAP32[$144>>2]|0;
        $146 = HEAP32[$0>>2]|0;
        $147 = ((($146)) + 32|0);
        $148 = ((($147)) + 12|0);
        HEAP32[$148>>2] = $145;
        $149 = HEAP32[$0>>2]|0;
        $150 = ((($149)) + 32|0);
        $151 = HEAP32[$pBt>>2]|0;
        $152 = ((($151)) + 72|0);
        HEAP32[$152>>2] = $150;
       }
      }
      $153 = HEAP32[$1>>2]|0;
      $154 = ($153|0)!=(0);
      $155 = $154 ? 2 : 1;
      $156 = $155&255;
      $157 = HEAP32[$0>>2]|0;
      $158 = ((($157)) + 8|0);
      HEAP8[$158>>0] = $156;
      $159 = HEAP32[$0>>2]|0;
      $160 = ((($159)) + 8|0);
      $161 = HEAP8[$160>>0]|0;
      $162 = $161&255;
      $163 = HEAP32[$pBt>>2]|0;
      $164 = ((($163)) + 20|0);
      $165 = HEAP8[$164>>0]|0;
      $166 = $165&255;
      $167 = ($162|0)>($166|0);
      if ($167) {
       $168 = HEAP32[$0>>2]|0;
       $169 = ((($168)) + 8|0);
       $170 = HEAP8[$169>>0]|0;
       $171 = HEAP32[$pBt>>2]|0;
       $172 = ((($171)) + 20|0);
       HEAP8[$172>>0] = $170;
      }
      $173 = HEAP32[$1>>2]|0;
      $174 = ($173|0)!=(0);
      if ($174) {
       $175 = HEAP32[$pBt>>2]|0;
       $176 = ((($175)) + 12|0);
       $177 = HEAP32[$176>>2]|0;
       HEAP32[$pPage1>>2] = $177;
       $178 = HEAP32[$0>>2]|0;
       $179 = HEAP32[$pBt>>2]|0;
       $180 = ((($179)) + 76|0);
       HEAP32[$180>>2] = $178;
       $181 = HEAP32[$pBt>>2]|0;
       $182 = ((($181)) + 22|0);
       $183 = HEAP16[$182>>1]|0;
       $184 = $183&65535;
       $185 = $184 & -33;
       $186 = $185&65535;
       HEAP16[$182>>1] = $186;
       $187 = HEAP32[$1>>2]|0;
       $188 = ($187|0)>(1);
       if ($188) {
        $189 = HEAP32[$pBt>>2]|0;
        $190 = ((($189)) + 22|0);
        $191 = HEAP16[$190>>1]|0;
        $192 = $191&65535;
        $193 = $192 | 32;
        $194 = $193&65535;
        HEAP16[$190>>1] = $194;
       }
       $195 = HEAP32[$pBt>>2]|0;
       $196 = ((($195)) + 44|0);
       $197 = HEAP32[$196>>2]|0;
       $198 = HEAP32[$pPage1>>2]|0;
       $199 = ((($198)) + 56|0);
       $200 = HEAP32[$199>>2]|0;
       $201 = ((($200)) + 28|0);
       $202 = (_sqlite3Get4byte($201)|0);
       $203 = ($197|0)!=($202|0);
       if ($203) {
        $204 = HEAP32[$pPage1>>2]|0;
        $205 = ((($204)) + 72|0);
        $206 = HEAP32[$205>>2]|0;
        $207 = (_sqlite3PagerWrite($206)|0);
        HEAP32[$rc>>2] = $207;
        $208 = HEAP32[$rc>>2]|0;
        $209 = ($208|0)==(0);
        if ($209) {
         $210 = HEAP32[$pPage1>>2]|0;
         $211 = ((($210)) + 56|0);
         $212 = HEAP32[$211>>2]|0;
         $213 = ((($212)) + 28|0);
         $214 = HEAP32[$pBt>>2]|0;
         $215 = ((($214)) + 44|0);
         $216 = HEAP32[$215>>2]|0;
         _sqlite3Put4byte($213,$216);
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $217 = HEAP32[$rc>>2]|0;
 $218 = ($217|0)==(0);
 $219 = HEAP32[$1>>2]|0;
 $220 = ($219|0)!=(0);
 $or$cond9 = $218 & $220;
 if (!($or$cond9)) {
  $228 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($228|0);
 }
 $221 = HEAP32[$pBt>>2]|0;
 $222 = HEAP32[$221>>2]|0;
 $223 = HEAP32[$0>>2]|0;
 $224 = HEAP32[$223>>2]|0;
 $225 = ((($224)) + 432|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = (_sqlite3PagerOpenSavepoint($222,$226)|0);
 HEAP32[$rc>>2] = $227;
 $228 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($228|0);
}
function _setDestPgsz($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3BtreeGetPageSize($6)|0);
 $8 = (_sqlite3BtreeSetPageSize($3,$7,-1,0)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3BtreeGetMeta($p,$idx,$pMeta) {
 $p = $p|0;
 $idx = $idx|0;
 $pMeta = $pMeta|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $pMeta;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(15);
 if ($8) {
  $9 = HEAP32[$pBt>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (_sqlite3PagerDataVersion($10)|0);
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 20|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11) + ($14))|0;
  $16 = HEAP32[$2>>2]|0;
  HEAP32[$16>>2] = $15;
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$1>>2]|0;
  $18 = $17<<2;
  $19 = (36 + ($18))|0;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 56|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($19)|0);
  $26 = (_sqlite3Get4byte($25)|0);
  $27 = HEAP32[$2>>2]|0;
  HEAP32[$27>>2] = $26;
  STACKTOP = sp;return;
 }
}
function _sqlite3BtreeGetPageSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3PagerGetJournalMode($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 5|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3BtreeLastPage($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_btreePagecount($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3PagerGet($pPager,$pgno,$ppPage,$flags) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bMmapOk = 0, $iFrame = 0, $noContent = 0, $or$cond = 0, $or$cond11 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pBase = 0, $pData = 0, $pPg = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $rc = sp + 24|0;
 $pPg = sp + 20|0;
 $iFrame = sp + 16|0;
 $noContent = sp + 12|0;
 $bMmapOk = sp + 8|0;
 $pData = sp + 4|0;
 $pBase = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pPg>>2] = 0;
 HEAP32[$iFrame>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1;
 HEAP32[$noContent>>2] = $6;
 $7 = 0;
 HEAP32[$bMmapOk>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8>>>0)<=(1);
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0);
 $or$cond = $9 & $11;
 if ($or$cond) {
  $12 = (_sqlite3CorruptError(51704)|0);
  HEAP32[$0>>2] = $12;
  $206 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($206|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0);
 do {
  if ($16) {
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 44|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$rc>>2] = $19;
   label = 24;
  } else {
   $20 = HEAP32[$bMmapOk>>2]|0;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = (_sqlite3PagerUseWal($22)|0);
    $24 = ($23|0)!=(0);
    if ($24) {
     $25 = HEAP32[$1>>2]|0;
     $26 = ((($25)) + 216|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$2>>2]|0;
     $29 = (_sqlite3WalFindFrame($27,$28,$iFrame)|0);
     HEAP32[$rc>>2] = $29;
     $30 = HEAP32[$rc>>2]|0;
     $31 = ($30|0)!=(0);
     if ($31) {
      break;
     }
    }
   }
   $32 = HEAP32[$bMmapOk>>2]|0;
   $33 = ($32|0)!=(0);
   $34 = HEAP32[$iFrame>>2]|0;
   $35 = ($34|0)==(0);
   $or$cond3 = $33 & $35;
   if ($or$cond3) {
    HEAP32[$pData>>2] = 0;
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 64|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$2>>2]|0;
    $40 = (($39) - 1)|0;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 160|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (___muldi3(($40|0),0,($43|0),($45|0))|0);
    $47 = tempRet0;
    $48 = HEAP32[$1>>2]|0;
    $49 = ((($48)) + 160|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (_sqlite3OsFetch($38,$46,$47,$50,$pData)|0);
    HEAP32[$rc>>2] = $51;
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)==(0);
    $54 = HEAP32[$pData>>2]|0;
    $55 = ($54|0)!=(0|0);
    $or$cond5 = $53 & $55;
    if ($or$cond5) {
     $56 = HEAP32[$1>>2]|0;
     $57 = ((($56)) + 17|0);
     $58 = HEAP8[$57>>0]|0;
     $59 = $58&255;
     $60 = ($59|0)>(1);
     if ($60) {
      label = 12;
     } else {
      $61 = HEAP32[$1>>2]|0;
      $62 = ((($61)) + 13|0);
      $63 = HEAP8[$62>>0]|0;
      $64 = $63&255;
      $65 = ($64|0)!=(0);
      if ($65) {
       label = 12;
      }
     }
     if ((label|0) == 12) {
      $66 = HEAP32[$1>>2]|0;
      $67 = HEAP32[$2>>2]|0;
      $68 = (_sqlite3PagerLookup($66,$67)|0);
      HEAP32[$pPg>>2] = $68;
     }
     $69 = HEAP32[$pPg>>2]|0;
     $70 = ($69|0)==(0|0);
     $71 = HEAP32[$1>>2]|0;
     if ($70) {
      $72 = HEAP32[$2>>2]|0;
      $73 = HEAP32[$pData>>2]|0;
      $74 = (_pagerAcquireMapPage($71,$72,$73,$pPg)|0);
      HEAP32[$rc>>2] = $74;
     } else {
      $75 = ((($71)) + 64|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = HEAP32[$2>>2]|0;
      $78 = (($77) - 1)|0;
      $79 = HEAP32[$1>>2]|0;
      $80 = ((($79)) + 160|0);
      $81 = HEAP32[$80>>2]|0;
      $82 = ($81|0)<(0);
      $83 = $82 << 31 >> 31;
      $84 = (___muldi3(($78|0),0,($81|0),($83|0))|0);
      $85 = tempRet0;
      $86 = HEAP32[$pData>>2]|0;
      (_sqlite3OsUnfetch($76,$84,$85,$86)|0);
     }
     $87 = HEAP32[$pPg>>2]|0;
     $88 = ($87|0)!=(0|0);
     if ($88) {
      $89 = HEAP32[$pPg>>2]|0;
      $90 = HEAP32[$3>>2]|0;
      HEAP32[$90>>2] = $89;
      HEAP32[$0>>2] = 0;
      $206 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($206|0);
     }
    }
    $91 = HEAP32[$rc>>2]|0;
    $92 = ($91|0)!=(0);
    if ($92) {
     break;
    }
   }
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 212|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = HEAP32[$2>>2]|0;
   $97 = (_sqlite3PcacheFetch($95,$96,3)|0);
   HEAP32[$pBase>>2] = $97;
   $98 = HEAP32[$pBase>>2]|0;
   $99 = ($98|0)==(0|0);
   if ($99) {
    $100 = HEAP32[$1>>2]|0;
    $101 = ((($100)) + 212|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = HEAP32[$2>>2]|0;
    $104 = (_sqlite3PcacheFetchStress($102,$103,$pBase)|0);
    HEAP32[$rc>>2] = $104;
    $105 = HEAP32[$rc>>2]|0;
    $106 = ($105|0)!=(0);
    if ($106) {
     break;
    }
    $107 = HEAP32[$pBase>>2]|0;
    $108 = ($107|0)==(0|0);
    if ($108) {
     $109 = HEAP32[$3>>2]|0;
     HEAP32[$109>>2] = 0;
     HEAP32[$pPg>>2] = 0;
     HEAP32[$rc>>2] = 7;
     break;
    }
   }
   $110 = HEAP32[$1>>2]|0;
   $111 = ((($110)) + 212|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = HEAP32[$2>>2]|0;
   $114 = HEAP32[$pBase>>2]|0;
   $115 = (_sqlite3PcacheFetchFinish($112,$113,$114)|0);
   $116 = HEAP32[$3>>2]|0;
   HEAP32[$116>>2] = $115;
   HEAP32[$pPg>>2] = $115;
   label = 24;
  }
 } while(0);
 do {
  if ((label|0) == 24) {
   $117 = HEAP32[$rc>>2]|0;
   $118 = ($117|0)!=(0);
   if ($118) {
    HEAP32[$pPg>>2] = 0;
    break;
   }
   $119 = HEAP32[$pPg>>2]|0;
   $120 = ((($119)) + 16|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = ($121|0)==(0|0);
   $123 = HEAP32[$noContent>>2]|0;
   $124 = ($123|0)!=(0);
   $or$cond7 = $122 | $124;
   $125 = HEAP32[$1>>2]|0;
   if (!($or$cond7)) {
    $126 = ((($125)) + 192|0);
    $127 = HEAP32[$126>>2]|0;
    $128 = (($127) + 1)|0;
    HEAP32[$126>>2] = $128;
    HEAP32[$0>>2] = 0;
    $206 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($206|0);
   }
   $129 = HEAP32[$pPg>>2]|0;
   $130 = ((($129)) + 16|0);
   HEAP32[$130>>2] = $125;
   $131 = HEAP32[$2>>2]|0;
   $132 = ($131>>>0)>(2147483647);
   if (!($132)) {
    $133 = HEAP32[$2>>2]|0;
    $134 = HEAP32[1536>>2]|0;
    $135 = HEAP32[$1>>2]|0;
    $136 = ((($135)) + 160|0);
    $137 = HEAP32[$136>>2]|0;
    $138 = (($134|0) / ($137|0))&-1;
    $139 = (($138) + 1)|0;
    $140 = ($133|0)==($139|0);
    if (!($140)) {
     $142 = HEAP32[$1>>2]|0;
     $143 = ((($142)) + 64|0);
     $144 = HEAP32[$143>>2]|0;
     $145 = HEAP32[$144>>2]|0;
     $146 = ($145|0)!=(0|0);
     if ($146) {
      $147 = HEAP32[$1>>2]|0;
      $148 = ((($147)) + 28|0);
      $149 = HEAP32[$148>>2]|0;
      $150 = HEAP32[$2>>2]|0;
      $151 = ($149>>>0)<($150>>>0);
      $152 = HEAP32[$noContent>>2]|0;
      $153 = ($152|0)!=(0);
      $or$cond9 = $151 | $153;
      if ($or$cond9) {
       label = 33;
      } else {
       $178 = HEAP32[$1>>2]|0;
       $179 = (_sqlite3PagerUseWal($178)|0);
       $180 = ($179|0)!=(0);
       $181 = HEAP32[$bMmapOk>>2]|0;
       $182 = ($181|0)==(0);
       $or$cond11 = $180 & $182;
       if ($or$cond11) {
        $183 = HEAP32[$1>>2]|0;
        $184 = ((($183)) + 216|0);
        $185 = HEAP32[$184>>2]|0;
        $186 = HEAP32[$2>>2]|0;
        $187 = (_sqlite3WalFindFrame($185,$186,$iFrame)|0);
        HEAP32[$rc>>2] = $187;
        $188 = HEAP32[$rc>>2]|0;
        $189 = ($188|0)!=(0);
        if ($189) {
         break;
        }
       }
       $190 = HEAP32[$1>>2]|0;
       $191 = ((($190)) + 192|0);
       $192 = ((($191)) + 4|0);
       $193 = HEAP32[$192>>2]|0;
       $194 = (($193) + 1)|0;
       HEAP32[$192>>2] = $194;
       $195 = HEAP32[$pPg>>2]|0;
       $196 = HEAP32[$iFrame>>2]|0;
       $197 = (_readDbPage($195,$196)|0);
       HEAP32[$rc>>2] = $197;
       $198 = HEAP32[$rc>>2]|0;
       $199 = ($198|0)!=(0);
       if ($199) {
        break;
       }
      }
     } else {
      label = 33;
     }
     if ((label|0) == 33) {
      $154 = HEAP32[$2>>2]|0;
      $155 = HEAP32[$1>>2]|0;
      $156 = ((($155)) + 164|0);
      $157 = HEAP32[$156>>2]|0;
      $158 = ($154>>>0)>($157>>>0);
      if ($158) {
       HEAP32[$rc>>2] = 13;
       break;
      }
      $159 = HEAP32[$noContent>>2]|0;
      $160 = ($159|0)!=(0);
      if ($160) {
       _sqlite3BeginBenignMalloc();
       $161 = HEAP32[$2>>2]|0;
       $162 = HEAP32[$1>>2]|0;
       $163 = ((($162)) + 32|0);
       $164 = HEAP32[$163>>2]|0;
       $165 = ($161>>>0)<=($164>>>0);
       if ($165) {
        $166 = HEAP32[$1>>2]|0;
        $167 = ((($166)) + 60|0);
        $168 = HEAP32[$167>>2]|0;
        $169 = HEAP32[$2>>2]|0;
        (_sqlite3BitvecSet($168,$169)|0);
       }
       $170 = HEAP32[$1>>2]|0;
       $171 = HEAP32[$2>>2]|0;
       (_addToSavepointBitvecs($170,$171)|0);
       _sqlite3EndBenignMalloc();
      }
      $172 = HEAP32[$pPg>>2]|0;
      $173 = ((($172)) + 4|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = HEAP32[$1>>2]|0;
      $176 = ((($175)) + 160|0);
      $177 = HEAP32[$176>>2]|0;
      _memset(($174|0),0,($177|0))|0;
     }
     HEAP32[$0>>2] = 0;
     $206 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($206|0);
    }
   }
   $141 = (_sqlite3CorruptError(51793)|0);
   HEAP32[$rc>>2] = $141;
  }
 } while(0);
 $200 = HEAP32[$pPg>>2]|0;
 $201 = ($200|0)!=(0|0);
 if ($201) {
  $202 = HEAP32[$pPg>>2]|0;
  _sqlite3PcacheDrop($202);
 }
 $203 = HEAP32[$1>>2]|0;
 _pagerUnlockIfUnused($203);
 $204 = HEAP32[$3>>2]|0;
 HEAP32[$204>>2] = 0;
 $205 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $205;
 $206 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($206|0);
}
function _sqlite3PagerGetData($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _backupOnePage($p,$iSrcPg,$zSrcData,$bUpdate) {
 $p = $p|0;
 $iSrcPg = $iSrcPg|0;
 $zSrcData = $zSrcData|0;
 $bUpdate = $bUpdate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iDest = 0, $iEnd = 0, $iOff = 0, $nCopy = 0, $nDestPgsz = 0, $nSrcPgsz = 0, $or$cond = 0, $pDestPager = 0, $pDestPg = 0, $rc = 0;
 var $zDestData = 0, $zIn = 0, $zOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $pDestPager = sp + 52|0;
 $nSrcPgsz = sp + 48|0;
 $nDestPgsz = sp + 44|0;
 $nCopy = sp + 40|0;
 $iEnd = sp + 8|0;
 $rc = sp + 36|0;
 $iOff = sp;
 $pDestPg = sp + 32|0;
 $iDest = sp + 28|0;
 $zIn = sp + 24|0;
 $zDestData = sp + 20|0;
 $zOut = sp + 16|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iSrcPg;
 HEAP32[$2>>2] = $zSrcData;
 HEAP32[$3>>2] = $bUpdate;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3BtreePager($6)|0);
 HEAP32[$pDestPager>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3BtreeGetPageSize($10)|0);
 HEAP32[$nSrcPgsz>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3BtreeGetPageSize($14)|0);
 HEAP32[$nDestPgsz>>2] = $15;
 $16 = HEAP32[$nSrcPgsz>>2]|0;
 $17 = HEAP32[$nDestPgsz>>2]|0;
 $18 = ($16|0)<($17|0);
 $19 = HEAP32[$nSrcPgsz>>2]|0;
 $20 = HEAP32[$nDestPgsz>>2]|0;
 $21 = $18 ? $19 : $20;
 HEAP32[$nCopy>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$nSrcPgsz>>2]|0;
 $24 = ($23|0)<(0);
 $25 = $24 << 31 >> 31;
 $26 = (___muldi3(($22|0),0,($23|0),($25|0))|0);
 $27 = tempRet0;
 $28 = $iEnd;
 $29 = $28;
 HEAP32[$29>>2] = $26;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $27;
 HEAP32[$rc>>2] = 0;
 $32 = HEAP32[$nSrcPgsz>>2]|0;
 $33 = HEAP32[$nDestPgsz>>2]|0;
 $34 = ($32|0)!=($33|0);
 if ($34) {
  $35 = HEAP32[$pDestPager>>2]|0;
  $36 = (_sqlite3PagerIsMemdb($35)|0);
  $37 = ($36|0)!=(0);
  if ($37) {
   HEAP32[$rc>>2] = 8;
  }
 }
 $38 = $iEnd;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$nSrcPgsz>>2]|0;
 $45 = ($44|0)<(0);
 $46 = $45 << 31 >> 31;
 $47 = (_i64Subtract(($40|0),($43|0),($44|0),($46|0))|0);
 $48 = tempRet0;
 $49 = $iOff;
 $50 = $49;
 HEAP32[$50>>2] = $47;
 $51 = (($49) + 4)|0;
 $52 = $51;
 HEAP32[$52>>2] = $48;
 while(1) {
  $53 = HEAP32[$rc>>2]|0;
  $54 = ($53|0)==(0);
  if (!($54)) {
   label = 14;
   break;
  }
  $55 = $iOff;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($55) + 4)|0;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = $iEnd;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (($61) + 4)|0;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = ($60|0)<($66|0);
  $68 = ($57>>>0)<($63>>>0);
  $69 = ($60|0)==($66|0);
  $70 = $69 & $68;
  $71 = $67 | $70;
  if (!($71)) {
   label = 14;
   break;
  }
  HEAP32[$pDestPg>>2] = 0;
  $72 = $iOff;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$nDestPgsz>>2]|0;
  $79 = ($78|0)<(0);
  $80 = $79 << 31 >> 31;
  $81 = (___divdi3(($74|0),($77|0),($78|0),($80|0))|0);
  $82 = tempRet0;
  $83 = (($81) + 1)|0;
  HEAP32[$iDest>>2] = $83;
  $84 = HEAP32[$iDest>>2]|0;
  $85 = HEAP32[1536>>2]|0;
  $86 = HEAP32[$0>>2]|0;
  $87 = ((($86)) + 4|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = ((($88)) + 4|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = ((($90)) + 32|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($85>>>0) / ($92>>>0))&-1;
  $94 = (($93) + 1)|0;
  $95 = ($84|0)==($94|0);
  if (!($95)) {
   $96 = HEAP32[$pDestPager>>2]|0;
   $97 = HEAP32[$iDest>>2]|0;
   $98 = (_sqlite3PagerGet($96,$97,$pDestPg,0)|0);
   HEAP32[$rc>>2] = $98;
   $99 = (0)==($98|0);
   if ($99) {
    $100 = HEAP32[$pDestPg>>2]|0;
    $101 = (_sqlite3PagerWrite($100)|0);
    HEAP32[$rc>>2] = $101;
    $102 = (0)==($101|0);
    if ($102) {
     $103 = $iOff;
     $104 = $103;
     $105 = HEAP32[$104>>2]|0;
     $106 = (($103) + 4)|0;
     $107 = $106;
     $108 = HEAP32[$107>>2]|0;
     $109 = HEAP32[$nSrcPgsz>>2]|0;
     $110 = ($109|0)<(0);
     $111 = $110 << 31 >> 31;
     $112 = (___remdi3(($105|0),($108|0),($109|0),($111|0))|0);
     $113 = tempRet0;
     $114 = HEAP32[$2>>2]|0;
     $115 = (($114) + ($112)|0);
     HEAP32[$zIn>>2] = $115;
     $116 = HEAP32[$pDestPg>>2]|0;
     $117 = (_sqlite3PagerGetData($116)|0);
     HEAP32[$zDestData>>2] = $117;
     $118 = $iOff;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = (($118) + 4)|0;
     $122 = $121;
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$nDestPgsz>>2]|0;
     $125 = ($124|0)<(0);
     $126 = $125 << 31 >> 31;
     $127 = (___remdi3(($120|0),($123|0),($124|0),($126|0))|0);
     $128 = tempRet0;
     $129 = HEAP32[$zDestData>>2]|0;
     $130 = (($129) + ($127)|0);
     HEAP32[$zOut>>2] = $130;
     $131 = HEAP32[$zOut>>2]|0;
     $132 = HEAP32[$zIn>>2]|0;
     $133 = HEAP32[$nCopy>>2]|0;
     _memcpy(($131|0),($132|0),($133|0))|0;
     $134 = HEAP32[$pDestPg>>2]|0;
     $135 = (_sqlite3PagerGetExtra($134)|0);
     HEAP8[$135>>0] = 0;
     $136 = $iOff;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = ($138|0)==(0);
     $143 = ($141|0)==(0);
     $144 = $142 & $143;
     $145 = HEAP32[$3>>2]|0;
     $146 = ($145|0)==(0);
     $or$cond = $144 & $146;
     if ($or$cond) {
      $147 = HEAP32[$zOut>>2]|0;
      $148 = ((($147)) + 28|0);
      $149 = HEAP32[$0>>2]|0;
      $150 = ((($149)) + 24|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = (_sqlite3BtreeLastPage($151)|0);
      _sqlite3Put4byte($148,$152);
     }
    }
   }
   $153 = HEAP32[$pDestPg>>2]|0;
   _sqlite3PagerUnref($153);
  }
  $154 = HEAP32[$nDestPgsz>>2]|0;
  $155 = ($154|0)<(0);
  $156 = $155 << 31 >> 31;
  $157 = $iOff;
  $158 = $157;
  $159 = HEAP32[$158>>2]|0;
  $160 = (($157) + 4)|0;
  $161 = $160;
  $162 = HEAP32[$161>>2]|0;
  $163 = (_i64Add(($159|0),($162|0),($154|0),($156|0))|0);
  $164 = tempRet0;
  $165 = $iOff;
  $166 = $165;
  HEAP32[$166>>2] = $163;
  $167 = (($165) + 4)|0;
  $168 = $167;
  HEAP32[$168>>2] = $164;
 }
 if ((label|0) == 14) {
  $169 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 return (0)|0;
}
function _sqlite3PagerUnref($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _sqlite3PagerUnrefNotNull($3);
 STACKTOP = sp;return;
}
function _attachBackupObject($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3BtreePager($3)|0);
 $5 = (_sqlite3PagerBackupPtr($4)|0);
 HEAP32[$pp>>2] = $5;
 $6 = HEAP32[$pp>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 44|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$pp>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 40|0);
 HEAP32[$13>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3BtreeNewDb($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 44|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_newDatabase($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3BtreeUpdateMeta($p,$idx,$iMeta) {
 $p = $p|0;
 $idx = $idx|0;
 $iMeta = $iMeta|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pP1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pBt = sp + 8|0;
 $pP1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $iMeta;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pP1>>2] = $11;
 $12 = HEAP32[$pBt>>2]|0;
 $13 = ((($12)) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 72|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3PagerWrite($16)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if (!($19)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = $20<<2;
 $22 = (36 + ($21))|0;
 $23 = HEAP32[$pP1>>2]|0;
 $24 = (($23) + ($22)|0);
 $25 = HEAP32[$2>>2]|0;
 _sqlite3Put4byte($24,$25);
 $26 = HEAP32[$1>>2]|0;
 $27 = ($26|0)==(7);
 if (!($27)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = $28&255;
 $30 = HEAP32[$pBt>>2]|0;
 $31 = ((($30)) + 18|0);
 HEAP8[$31>>0] = $29;
 $32 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3ResetAllSchemasOfConnection($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  HEAP32[$pDb>>2] = $11;
  $12 = HEAP32[$pDb>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$pDb>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   _sqlite3SchemaClear($18);
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 24|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 & -3;
 HEAP32[$22>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($25);
 $26 = HEAP32[$0>>2]|0;
 _sqlite3CollapseDatabaseArray($26);
 STACKTOP = sp;return;
}
function _sqlite3BtreeSetVersion($pBtree,$iVersion) {
 $pBtree = $pBtree|0;
 $iVersion = $iVersion|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $aData = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pBt = sp + 8|0;
 $rc = sp + 4|0;
 $aData = sp;
 HEAP32[$0>>2] = $pBtree;
 HEAP32[$1>>2] = $iVersion;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$pBt>>2]|0;
 $6 = ((($5)) + 22|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & -17;
 $10 = $9&65535;
 HEAP16[$6>>1] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)==(1);
 if ($12) {
  $13 = HEAP32[$pBt>>2]|0;
  $14 = ((($13)) + 22|0);
  $15 = HEAP16[$14>>1]|0;
  $16 = $15&65535;
  $17 = $16 | 16;
  $18 = $17&65535;
  HEAP16[$14>>1] = $18;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = (_sqlite3BtreeBeginTrans($19,0)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 do {
  if ($22) {
   $23 = HEAP32[$pBt>>2]|0;
   $24 = ((($23)) + 12|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ((($25)) + 56|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$aData>>2] = $27;
   $28 = HEAP32[$aData>>2]|0;
   $29 = ((($28)) + 18|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = HEAP32[$1>>2]|0;
   $33 = $32&255;
   $34 = $33&255;
   $35 = ($31|0)!=($34|0);
   if (!($35)) {
    $36 = HEAP32[$aData>>2]|0;
    $37 = ((($36)) + 19|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = HEAP32[$1>>2]|0;
    $41 = $40&255;
    $42 = $41&255;
    $43 = ($39|0)!=($42|0);
    if (!($43)) {
     break;
    }
   }
   $44 = HEAP32[$0>>2]|0;
   $45 = (_sqlite3BtreeBeginTrans($44,2)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)==(0);
   if ($47) {
    $48 = HEAP32[$pBt>>2]|0;
    $49 = ((($48)) + 12|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = ((($50)) + 72|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (_sqlite3PagerWrite($52)|0);
    HEAP32[$rc>>2] = $53;
    $54 = HEAP32[$rc>>2]|0;
    $55 = ($54|0)==(0);
    if ($55) {
     $56 = HEAP32[$1>>2]|0;
     $57 = $56&255;
     $58 = HEAP32[$aData>>2]|0;
     $59 = ((($58)) + 18|0);
     HEAP8[$59>>0] = $57;
     $60 = HEAP32[$1>>2]|0;
     $61 = $60&255;
     $62 = HEAP32[$aData>>2]|0;
     $63 = ((($62)) + 19|0);
     HEAP8[$63>>0] = $61;
    }
   }
  }
 } while(0);
 $64 = HEAP32[$pBt>>2]|0;
 $65 = ((($64)) + 22|0);
 $66 = HEAP16[$65>>1]|0;
 $67 = $66&65535;
 $68 = $67 & -17;
 $69 = $68&65535;
 HEAP16[$65>>1] = $69;
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _sqlite3PagerFile($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerPagecount($pPager,$pnPage) {
 $pPager = $pPager|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pnPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$5>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3PagerWrite($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pPager = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$pPager>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$pPager>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$0>>2] = $11;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 24|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 4;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$pPager>>2]|0;
  $19 = ((($18)) + 28|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20>>>0)>=($23>>>0);
  if ($24) {
   $25 = HEAP32[$pPager>>2]|0;
   $26 = ((($25)) + 104|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$1>>2]|0;
    $30 = (_subjournalPageIfRequired($29)|0);
    HEAP32[$0>>2] = $30;
    $41 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($41|0);
   } else {
    HEAP32[$0>>2] = 0;
    $41 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($41|0);
   }
  }
 }
 $31 = HEAP32[$pPager>>2]|0;
 $32 = ((($31)) + 156|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$pPager>>2]|0;
 $35 = ((($34)) + 160|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($33>>>0)>($36>>>0);
 $38 = HEAP32[$1>>2]|0;
 if ($37) {
  $39 = (_pagerWriteLargeSector($38)|0);
  HEAP32[$0>>2] = $39;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 } else {
  $40 = (_pager_write($38)|0);
  HEAP32[$0>>2] = $40;
  $41 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 return (0)|0;
}
function _sqlite3PagerCommitPhaseOne($pPager,$zMaster,$noSync) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 $noSync = $noSync|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nNew = 0, $or$cond = 0, $pList = 0;
 var $pPageOne = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pList = sp + 8|0;
 $pPageOne = sp + 4|0;
 $nNew = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$3>>2] = $noSync;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $114 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($114|0);
 }
 $11 = (_sqlite3FaultSim(400)|0);
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$0>>2] = 10;
  $114 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($114|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 17|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)<(3);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $114 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($114|0);
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = (_pagerFlushOnCommit($18,1)|0);
 $20 = (0)==($19|0);
 $21 = HEAP32[$1>>2]|0;
 do {
  if ($20) {
   $22 = ((($21)) + 96|0);
   $23 = HEAP32[$22>>2]|0;
   _sqlite3BackupRestart($23);
  } else {
   $24 = (_sqlite3PagerUseWal($21)|0);
   $25 = ($24|0)!=(0);
   $26 = HEAP32[$1>>2]|0;
   if ($25) {
    $27 = ((($26)) + 212|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (_sqlite3PcacheDirtyList($28)|0);
    HEAP32[$pList>>2] = $29;
    HEAP32[$pPageOne>>2] = 0;
    $30 = HEAP32[$pList>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     $32 = HEAP32[$1>>2]|0;
     $33 = (_sqlite3PagerGet($32,1,$pPageOne,0)|0);
     HEAP32[$rc>>2] = $33;
     $34 = HEAP32[$pPageOne>>2]|0;
     HEAP32[$pList>>2] = $34;
     $35 = HEAP32[$pList>>2]|0;
     $36 = ((($35)) + 12|0);
     HEAP32[$36>>2] = 0;
    }
    $37 = HEAP32[$pList>>2]|0;
    $38 = ($37|0)!=(0|0);
    if ($38) {
     $39 = HEAP32[$1>>2]|0;
     $40 = HEAP32[$pList>>2]|0;
     $41 = HEAP32[$1>>2]|0;
     $42 = ((($41)) + 28|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = (_pagerWalFrames($39,$40,$43,1)|0);
     HEAP32[$rc>>2] = $44;
    }
    $45 = HEAP32[$pPageOne>>2]|0;
    _sqlite3PagerUnref($45);
    $46 = HEAP32[$rc>>2]|0;
    $47 = ($46|0)==(0);
    if (!($47)) {
     break;
    }
    $48 = HEAP32[$1>>2]|0;
    $49 = ((($48)) + 212|0);
    $50 = HEAP32[$49>>2]|0;
    _sqlite3PcacheCleanAll($50);
    break;
   }
   $51 = (_pager_incr_changecounter($26,0)|0);
   HEAP32[$rc>>2] = $51;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)!=(0);
   if (!($53)) {
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$2>>2]|0;
    $56 = (_writeMasterJournal($54,$55)|0);
    HEAP32[$rc>>2] = $56;
    $57 = HEAP32[$rc>>2]|0;
    $58 = ($57|0)!=(0);
    if (!($58)) {
     $59 = HEAP32[$1>>2]|0;
     $60 = (_syncJournal($59,0)|0);
     HEAP32[$rc>>2] = $60;
     $61 = HEAP32[$rc>>2]|0;
     $62 = ($61|0)!=(0);
     if (!($62)) {
      $63 = HEAP32[$1>>2]|0;
      $64 = HEAP32[$1>>2]|0;
      $65 = ((($64)) + 212|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = (_sqlite3PcacheDirtyList($66)|0);
      $68 = (_pager_write_pagelist($63,$67)|0);
      HEAP32[$rc>>2] = $68;
      $69 = HEAP32[$rc>>2]|0;
      $70 = ($69|0)!=(0);
      if (!($70)) {
       $71 = HEAP32[$1>>2]|0;
       $72 = ((($71)) + 212|0);
       $73 = HEAP32[$72>>2]|0;
       _sqlite3PcacheCleanAll($73);
       $74 = HEAP32[$1>>2]|0;
       $75 = ((($74)) + 28|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = HEAP32[$1>>2]|0;
       $78 = ((($77)) + 36|0);
       $79 = HEAP32[$78>>2]|0;
       $80 = ($76>>>0)>($79>>>0);
       if ($80) {
        $81 = HEAP32[$1>>2]|0;
        $82 = ((($81)) + 28|0);
        $83 = HEAP32[$82>>2]|0;
        $84 = HEAP32[$1>>2]|0;
        $85 = ((($84)) + 28|0);
        $86 = HEAP32[$85>>2]|0;
        $87 = HEAP32[1536>>2]|0;
        $88 = HEAP32[$1>>2]|0;
        $89 = ((($88)) + 160|0);
        $90 = HEAP32[$89>>2]|0;
        $91 = (($87|0) / ($90|0))&-1;
        $92 = (($91) + 1)|0;
        $93 = ($86|0)==($92|0);
        $94 = $93&1;
        $95 = (($83) - ($94))|0;
        HEAP32[$nNew>>2] = $95;
        $96 = HEAP32[$1>>2]|0;
        $97 = HEAP32[$nNew>>2]|0;
        $98 = (_pager_truncate($96,$97)|0);
        HEAP32[$rc>>2] = $98;
        $99 = HEAP32[$rc>>2]|0;
        $100 = ($99|0)!=(0);
        $101 = HEAP32[$3>>2]|0;
        $102 = ($101|0)!=(0);
        $or$cond = $100 | $102;
        if ($or$cond) {
         break;
        }
       } else {
        $$old = HEAP32[$3>>2]|0;
        $$old1 = ($$old|0)!=(0);
        if ($$old1) {
         break;
        }
       }
       $103 = HEAP32[$1>>2]|0;
       $104 = HEAP32[$2>>2]|0;
       $105 = (_sqlite3PagerSync($103,$104)|0);
       HEAP32[$rc>>2] = $105;
      }
     }
    }
   }
  }
 } while(0);
 $106 = HEAP32[$rc>>2]|0;
 $107 = ($106|0)==(0);
 if ($107) {
  $108 = HEAP32[$1>>2]|0;
  $109 = (_sqlite3PagerUseWal($108)|0);
  $110 = ($109|0)!=(0);
  if (!($110)) {
   $111 = HEAP32[$1>>2]|0;
   $112 = ((($111)) + 17|0);
   HEAP8[$112>>0] = 5;
  }
 }
 $113 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $113;
 $114 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($114|0);
}
function _sqlite3OsWrite($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $id;
 HEAP32[$3>>2] = $pBuf;
 HEAP32[$4>>2] = $amt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = $5;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiiii[$13 & 255]($14,$15,$16,$19,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _backupTruncateFile($pFile,$0,$1) {
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iCurrent = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 8|0;
 $iCurrent = sp;
 $rc = sp + 16|0;
 HEAP32[$2>>2] = $pFile;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3OsFileSize($8,$iCurrent)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $12 = $iCurrent;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = $3;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($17|0)>($23|0);
 $25 = ($14>>>0)>($20>>>0);
 $26 = ($17|0)==($23|0);
 $27 = $26 & $25;
 $28 = $24 | $27;
 if (!($28)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = $3;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (_sqlite3OsTruncate($29,$32,$35)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3PagerSync($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $zMaster;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  HEAP32[$pArg>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$pArg>>2]|0;
  $12 = (_sqlite3OsFileControl($10,21,$11)|0);
  HEAP32[$rc>>2] = $12;
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(12);
  $$ = $14 ? 0 : $12;
  HEAP32[$rc>>2] = $$;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 7|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if ($20) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 64|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 12|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = (_sqlite3OsSync($23,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3PagerTruncateImage($pPager,$nPage) {
 $pPager = $pPager|0;
 $nPage = $nPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 28|0);
 HEAP32[$4>>2] = $2;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCommitPhaseTwo($p,$bCleanup) {
 $p = $p|0;
 $bCleanup = $bCleanup|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $bCleanup;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $33 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $8 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(2);
 do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$pBt>>2] = $16;
   $17 = HEAP32[$pBt>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3PagerCommitPhaseTwo($18)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)!=(0);
   $22 = HEAP32[$2>>2]|0;
   $23 = ($22|0)==(0);
   $or$cond = $21 & $23;
   if (!($or$cond)) {
    $25 = HEAP32[$1>>2]|0;
    $26 = ((($25)) + 20|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (($27) + -1)|0;
    HEAP32[$26>>2] = $28;
    $29 = HEAP32[$pBt>>2]|0;
    $30 = ((($29)) + 20|0);
    HEAP8[$30>>0] = 1;
    $31 = HEAP32[$pBt>>2]|0;
    _btreeClearHasContent($31);
    break;
   }
   $24 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $24;
   $33 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($33|0);
  }
 } while(0);
 $32 = HEAP32[$1>>2]|0;
 _btreeEndTransaction($32);
 HEAP32[$0>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3BtreeCommitPhaseOne($p,$zMaster) {
 $p = $p|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pBt>>2] = $10;
  $11 = HEAP32[$1>>2]|0;
  _sqlite3BtreeEnter($11);
  $12 = HEAP32[$pBt>>2]|0;
  $13 = ((($12)) + 17|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = ($14<<24>>24)!=(0);
  if ($15) {
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_autoVacuumCommit($16)|0);
   HEAP32[$rc>>2] = $17;
   $18 = HEAP32[$rc>>2]|0;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $20;
    $35 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
  }
  $21 = HEAP32[$pBt>>2]|0;
  $22 = ((($21)) + 19|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = ($23<<24>>24)!=(0);
  if ($24) {
   $25 = HEAP32[$pBt>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$pBt>>2]|0;
   $28 = ((($27)) + 44|0);
   $29 = HEAP32[$28>>2]|0;
   _sqlite3PagerTruncateImage($26,$29);
  }
  $30 = HEAP32[$pBt>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (_sqlite3PagerCommitPhaseOne($31,$32,0)|0);
  HEAP32[$rc>>2] = $33;
 }
 $34 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3PagerBackupPtr($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 96|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3BtreeRollback($p,$tripCode,$writeOnly) {
 $p = $p|0;
 $tripCode = $tripCode|0;
 $writeOnly = $writeOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nPage = 0, $pBt = 0, $pPage1 = 0, $rc = 0, $rc2 = 0, $rc21 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $rc = sp + 20|0;
 $pBt = sp + 16|0;
 $pPage1 = sp + 12|0;
 $rc2 = sp + 8|0;
 $rc21 = sp + 4|0;
 $nPage = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $tripCode;
 HEAP32[$2>>2] = $writeOnly;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$pBt>>2]|0;
  $10 = (_saveAllCursors($9,0,0)|0);
  HEAP32[$1>>2] = $10;
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   HEAP32[$2>>2] = 0;
  }
 } else {
  HEAP32[$rc>>2] = 0;
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_sqlite3BtreeTripAllCursors($15,$16,$17)|0);
  HEAP32[$rc2>>2] = $18;
  $19 = HEAP32[$rc2>>2]|0;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $21;
  }
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = $24&255;
 $26 = ($25|0)==(2);
 if (!($26)) {
  $52 = HEAP32[$0>>2]|0;
  _btreeEndTransaction($52);
  $53 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $27 = HEAP32[$pBt>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (_sqlite3PagerRollback($28)|0);
 HEAP32[$rc21>>2] = $29;
 $30 = HEAP32[$rc21>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$rc21>>2]|0;
  HEAP32[$rc>>2] = $32;
 }
 $33 = HEAP32[$pBt>>2]|0;
 $34 = (_btreeGetPage($33,1,$pPage1,0)|0);
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$pPage1>>2]|0;
  $37 = ((($36)) + 56|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ((($38)) + 28|0);
  $40 = (_sqlite3Get4byte($39)|0);
  HEAP32[$nPage>>2] = $40;
  $41 = HEAP32[$nPage>>2]|0;
  $42 = ($41|0)==(0);
  if ($42) {
   $43 = HEAP32[$pBt>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   _sqlite3PagerPagecount($44,$nPage);
  }
  $45 = HEAP32[$nPage>>2]|0;
  $46 = HEAP32[$pBt>>2]|0;
  $47 = ((($46)) + 44|0);
  HEAP32[$47>>2] = $45;
  $48 = HEAP32[$pPage1>>2]|0;
  _releasePage($48);
 }
 $49 = HEAP32[$pBt>>2]|0;
 $50 = ((($49)) + 20|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$pBt>>2]|0;
 _btreeClearHasContent($51);
 $52 = HEAP32[$0>>2]|0;
 _btreeEndTransaction($52);
 $53 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _sqlite3LeaveMutexAndCloseZombie($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $j = 0, $p = 0, $pColl = 0, $pDb = 0, $pMod = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $i = sp + 24|0;
 $j = sp + 20|0;
 $pDb = sp + 16|0;
 $pNext = sp + 12|0;
 $p = sp + 8|0;
 $pColl = sp + 4|0;
 $pMod = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 84|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(1691352191);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = (_connectionIsBusy($5)|0);
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 _sqlite3RollbackAll($8,0);
 $9 = HEAP32[$0>>2]|0;
 _sqlite3CloseSavepoints($9);
 HEAP32[$j>>2] = 0;
 while(1) {
  $10 = HEAP32[$j>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$j>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  HEAP32[$pDb>>2] = $19;
  $20 = HEAP32[$pDb>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$pDb>>2]|0;
   $25 = ((($24)) + 4|0);
   $26 = HEAP32[$25>>2]|0;
   (_sqlite3BtreeClose($26)|0);
   $27 = HEAP32[$pDb>>2]|0;
   $28 = ((($27)) + 4|0);
   HEAP32[$28>>2] = 0;
   $29 = HEAP32[$j>>2]|0;
   $30 = ($29|0)!=(1);
   if ($30) {
    $31 = HEAP32[$pDb>>2]|0;
    $32 = ((($31)) + 12|0);
    HEAP32[$32>>2] = 0;
   }
  }
  $33 = HEAP32[$j>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$j>>2] = $34;
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 16|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ((($37)) + 16|0);
 $39 = ((($38)) + 12|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = ($40|0)!=(0|0);
 if ($41) {
  $42 = HEAP32[$0>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ((($44)) + 16|0);
  $46 = ((($45)) + 12|0);
  $47 = HEAP32[$46>>2]|0;
  _sqlite3SchemaClear($47);
 }
 $48 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($48);
 $49 = HEAP32[$0>>2]|0;
 _sqlite3CollapseDatabaseArray($49);
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 348|0);
 $52 = ((($51)) + 8|0);
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$i>>2] = $53;
 while(1) {
  $54 = HEAP32[$i>>2]|0;
  $55 = ($54|0)!=(0|0);
  if (!($55)) {
   break;
  }
  $56 = HEAP32[$i>>2]|0;
  $57 = ((($56)) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  HEAP32[$p>>2] = $58;
  while(1) {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$p>>2]|0;
   _functionDestroy($59,$60);
   $61 = HEAP32[$p>>2]|0;
   $62 = ((($61)) + 8|0);
   $63 = HEAP32[$62>>2]|0;
   HEAP32[$pNext>>2] = $63;
   $64 = HEAP32[$0>>2]|0;
   $65 = HEAP32[$p>>2]|0;
   _sqlite3DbFree($64,$65);
   $66 = HEAP32[$pNext>>2]|0;
   HEAP32[$p>>2] = $66;
   $67 = HEAP32[$p>>2]|0;
   $68 = ($67|0)!=(0|0);
   if (!($68)) {
    break;
   }
  }
  $69 = HEAP32[$i>>2]|0;
  $70 = HEAP32[$69>>2]|0;
  HEAP32[$i>>2] = $70;
 }
 $71 = HEAP32[$0>>2]|0;
 $72 = ((($71)) + 348|0);
 _sqlite3HashClear($72);
 $73 = HEAP32[$0>>2]|0;
 $74 = ((($73)) + 364|0);
 $75 = ((($74)) + 8|0);
 $76 = HEAP32[$75>>2]|0;
 HEAP32[$i>>2] = $76;
 while(1) {
  $77 = HEAP32[$i>>2]|0;
  $78 = ($77|0)!=(0|0);
  if (!($78)) {
   break;
  }
  $79 = HEAP32[$i>>2]|0;
  $80 = ((($79)) + 8|0);
  $81 = HEAP32[$80>>2]|0;
  HEAP32[$pColl>>2] = $81;
  HEAP32[$j>>2] = 0;
  while(1) {
   $82 = HEAP32[$j>>2]|0;
   $83 = ($82|0)<(3);
   if (!($83)) {
    break;
   }
   $84 = HEAP32[$j>>2]|0;
   $85 = HEAP32[$pColl>>2]|0;
   $86 = (($85) + (($84*20)|0)|0);
   $87 = ((($86)) + 16|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = ($88|0)!=(0|0);
   if ($89) {
    $90 = HEAP32[$j>>2]|0;
    $91 = HEAP32[$pColl>>2]|0;
    $92 = (($91) + (($90*20)|0)|0);
    $93 = ((($92)) + 16|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = HEAP32[$j>>2]|0;
    $96 = HEAP32[$pColl>>2]|0;
    $97 = (($96) + (($95*20)|0)|0);
    $98 = ((($97)) + 8|0);
    $99 = HEAP32[$98>>2]|0;
    FUNCTION_TABLE_vi[$94 & 255]($99);
   }
   $100 = HEAP32[$j>>2]|0;
   $101 = (($100) + 1)|0;
   HEAP32[$j>>2] = $101;
  }
  $102 = HEAP32[$0>>2]|0;
  $103 = HEAP32[$pColl>>2]|0;
  _sqlite3DbFree($102,$103);
  $104 = HEAP32[$i>>2]|0;
  $105 = HEAP32[$104>>2]|0;
  HEAP32[$i>>2] = $105;
 }
 $106 = HEAP32[$0>>2]|0;
 $107 = ((($106)) + 364|0);
 _sqlite3HashClear($107);
 $108 = HEAP32[$0>>2]|0;
 $109 = ((($108)) + 320|0);
 $110 = ((($109)) + 8|0);
 $111 = HEAP32[$110>>2]|0;
 HEAP32[$i>>2] = $111;
 while(1) {
  $112 = HEAP32[$i>>2]|0;
  $113 = ($112|0)!=(0|0);
  if (!($113)) {
   break;
  }
  $114 = HEAP32[$i>>2]|0;
  $115 = ((($114)) + 8|0);
  $116 = HEAP32[$115>>2]|0;
  HEAP32[$pMod>>2] = $116;
  $117 = HEAP32[$pMod>>2]|0;
  $118 = ((($117)) + 12|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = ($119|0)!=(0|0);
  if ($120) {
   $121 = HEAP32[$pMod>>2]|0;
   $122 = ((($121)) + 12|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = HEAP32[$pMod>>2]|0;
   $125 = ((($124)) + 8|0);
   $126 = HEAP32[$125>>2]|0;
   FUNCTION_TABLE_vi[$123 & 255]($126);
  }
  $127 = HEAP32[$0>>2]|0;
  $128 = HEAP32[$pMod>>2]|0;
  _sqlite3VtabEponymousTableClear($127,$128);
  $129 = HEAP32[$0>>2]|0;
  $130 = HEAP32[$pMod>>2]|0;
  _sqlite3DbFree($129,$130);
  $131 = HEAP32[$i>>2]|0;
  $132 = HEAP32[$131>>2]|0;
  HEAP32[$i>>2] = $132;
 }
 $133 = HEAP32[$0>>2]|0;
 $134 = ((($133)) + 320|0);
 _sqlite3HashClear($134);
 $135 = HEAP32[$0>>2]|0;
 _sqlite3Error($135,0);
 $136 = HEAP32[$0>>2]|0;
 $137 = ((($136)) + 244|0);
 $138 = HEAP32[$137>>2]|0;
 _sqlite3ValueFree($138);
 $139 = HEAP32[$0>>2]|0;
 $140 = ((($139)) + 84|0);
 HEAP32[$140>>2] = -1254786768;
 $141 = HEAP32[$0>>2]|0;
 $142 = HEAP32[$0>>2]|0;
 $143 = ((($142)) + 16|0);
 $144 = HEAP32[$143>>2]|0;
 $145 = ((($144)) + 16|0);
 $146 = ((($145)) + 12|0);
 $147 = HEAP32[$146>>2]|0;
 _sqlite3DbFree($141,$147);
 $148 = HEAP32[$0>>2]|0;
 $149 = ((($148)) + 84|0);
 HEAP32[$149>>2] = -1623446221;
 $150 = HEAP32[$0>>2]|0;
 $151 = ((($150)) + 256|0);
 $152 = ((($151)) + 6|0);
 $153 = HEAP8[$152>>0]|0;
 $154 = ($153<<24>>24)!=(0);
 if ($154) {
  $155 = HEAP32[$0>>2]|0;
  $156 = ((($155)) + 256|0);
  $157 = ((($156)) + 32|0);
  $158 = HEAP32[$157>>2]|0;
  _sqlite3_free($158);
 }
 $159 = HEAP32[$0>>2]|0;
 _sqlite3_free($159);
 STACKTOP = sp;return;
}
function _vdbeSafety($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  _sqlite3_log(21,44004,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $5 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($5|0);
 } else {
  HEAP32[$0>>2] = 0;
  $5 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($5|0);
 }
 return (0)|0;
}
function _invokeProfileCallback($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $iElapse = 0, $iNow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $iNow = sp + 8|0;
 $iElapse = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 (_sqlite3OsCurrentTimeInt64($3,$iNow)|0);
 $4 = $iNow;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 128|0);
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (_i64Subtract(($6|0),($9|0),($14|0),($17|0))|0);
 $19 = tempRet0;
 $20 = (___muldi3(($18|0),($19|0),1000000,0)|0);
 $21 = tempRet0;
 $22 = $iElapse;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 192|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 192|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 196|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 176|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = $iElapse;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($39) + 4)|0;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  FUNCTION_TABLE_viiii[$32 & 255]($35,$38,$41,$44);
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 76|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47&255;
 $49 = $48 & 2;
 $50 = ($49|0)!=(0);
 if (!($50)) {
  $58 = HEAP32[$1>>2]|0;
  $59 = ((($58)) + 128|0);
  $60 = $59;
  $61 = $60;
  HEAP32[$61>>2] = 0;
  $62 = (($60) + 4)|0;
  $63 = $62;
  HEAP32[$63>>2] = 0;
  STACKTOP = sp;return;
 }
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 184|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 188|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$1>>2]|0;
 (FUNCTION_TABLE_iiiii[$53 & 255](2,$56,$57,$iElapse)|0);
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 128|0);
 $60 = $59;
 $61 = $60;
 HEAP32[$61>>2] = 0;
 $62 = (($60) + 4)|0;
 $63 = $62;
 HEAP32[$63>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeFinalize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(770837923);
 if ($4) {
  label = 3;
 } else {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(832317811);
  if ($8) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $9 = HEAP32[$0>>2]|0;
  $10 = (_sqlite3VdbeReset($9)|0);
  HEAP32[$rc>>2] = $10;
 }
 $11 = HEAP32[$0>>2]|0;
 _sqlite3VdbeDelete($11);
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3ApiExit($db,$rc) {
 $db = $db|0;
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $rc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 69|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(3082);
 $or$cond = $7 | $9;
 if ($or$cond) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_apiOomError($10)|0);
  HEAP32[$0>>2] = $11;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 } else {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 56|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $12 & $15;
  HEAP32[$0>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3VdbeReset($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeHalt($3)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>=(0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  (_sqlite3VdbeTransferError($8)|0);
  $9 = HEAP32[$db>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 108|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3DbFree($9,$12);
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 108|0);
  HEAP32[$14>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 144|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = ($17&65535) >>> 5;
  $19 = $18 & 1;
  $20 = $19&65535;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 144|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24 & -2;
   $26 = $25 | 1;
   HEAP16[$23>>1] = $26;
  }
 } else {
  $27 = ((($8)) + 40|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = ((($30)) + 144|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = $32 & 1;
   $34 = $33&65535;
   $35 = ($34|0)!=(0);
   if ($35) {
    $36 = HEAP32[$db>>2]|0;
    $37 = HEAP32[$0>>2]|0;
    $38 = ((($37)) + 40|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$0>>2]|0;
    $41 = ((($40)) + 108|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = ($42|0)!=(0|0);
    $44 = $43 ? 18391 : 0;
    $45 = HEAP32[$0>>2]|0;
    $46 = ((($45)) + 108|0);
    $47 = HEAP32[$46>>2]|0;
    HEAP32[$vararg_buffer>>2] = $47;
    _sqlite3ErrorWithMsg($36,$39,$44,$vararg_buffer);
    $48 = HEAP32[$db>>2]|0;
    $49 = HEAP32[$0>>2]|0;
    $50 = ((($49)) + 108|0);
    $51 = HEAP32[$50>>2]|0;
    _sqlite3DbFree($48,$51);
    $52 = HEAP32[$0>>2]|0;
    $53 = ((($52)) + 108|0);
    HEAP32[$53>>2] = 0;
   }
  }
 }
 $54 = HEAP32[$0>>2]|0;
 _Cleanup($54);
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 56|0);
 $57 = $56;
 $58 = $57;
 HEAP32[$58>>2] = 0;
 $59 = (($57) + 4)|0;
 $60 = $59;
 HEAP32[$60>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 20|0);
 HEAP32[$62>>2] = 1224384374;
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 40|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$db>>2]|0;
 $67 = ((($66)) + 56|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = $65 & $68;
 STACKTOP = sp;return ($69|0);
}
function _sqlite3VdbeRewind($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 HEAP32[$2>>2] = 770837923;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 36|0);
 HEAP32[$4>>2] = -1;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 40|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 142|0);
 HEAP8[$8>>0] = 2;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 44|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 32|0);
 HEAP32[$12>>2] = 1;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 143|0);
 HEAP8[$14>>0] = -1;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 48|0);
 HEAP32[$16>>2] = 0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 64|0);
 $19 = $18;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 24|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   STACKTOP = sp;return;
  }
 }
 $11 = HEAP32[$0>>2]|0;
 _vdbeMemClear($11);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemExpandBlob($pMem) {
 $pMem = $pMem|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + ($6))|0;
 HEAP32[$nByte>>2] = $7;
 $8 = HEAP32[$nByte>>2]|0;
 $9 = ($8|0)<=(0);
 $$ = $9 ? 1 : $7;
 HEAP32[$nByte>>2] = $$;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$nByte>>2]|0;
 $12 = (_sqlite3VdbeMemGrow($10,$11,1)|0);
 $13 = ($12|0)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 7;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16)|0);
  $21 = HEAP32[$1>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  _memset(($20|0),0,($22|0))|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 12|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + ($24))|0;
  HEAP32[$26>>2] = $28;
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 8|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $31&65535;
  $33 = $32 & -16897;
  $34 = $33&65535;
  HEAP16[$30>>1] = $34;
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _sqlite3ValueText($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$1>>2]|0;
   $6 = ((($5)) + 8|0);
   $7 = HEAP16[$6>>1]|0;
   $8 = $7&65535;
   $9 = $8 & 514;
   $10 = ($9|0)==(514);
   if ($10) {
    $11 = HEAP32[$1>>2]|0;
    $12 = ((($11)) + 10|0);
    $13 = HEAP8[$12>>0]|0;
    $14 = $13&255;
    $15 = HEAP8[$2>>0]|0;
    $16 = $15&255;
    $17 = ($14|0)==($16|0);
    if ($17) {
     $18 = HEAP32[$1>>2]|0;
     $19 = ((($18)) + 16|0);
     $20 = HEAP32[$19>>2]|0;
     HEAP32[$0>>2] = $20;
     break;
    }
   }
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 8|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23&65535;
   $25 = $24 & 1;
   $26 = ($25|0)!=(0);
   if ($26) {
    HEAP32[$0>>2] = 0;
    break;
   } else {
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP8[$2>>0]|0;
    $29 = (_valueToText($27,$28)|0);
    HEAP32[$0>>2] = $29;
    break;
   }
  } else {
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3ValueBytes($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $p = sp;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 2;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 10|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = HEAP8[$2>>0]|0;
  $15 = $14&255;
  $16 = ($13|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$p>>2]|0;
   $18 = ((($17)) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$0>>2] = $19;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & 16;
 $25 = ($24|0)!=(0);
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 8|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 if ($25) {
  $30 = $29 & 16384;
  $31 = ($30|0)!=(0);
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  if ($31) {
   $35 = HEAP32[$p>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = (($34) + ($36))|0;
   HEAP32[$0>>2] = $37;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  } else {
   HEAP32[$0>>2] = $34;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
 } else {
  $38 = $29 & 1;
  $39 = ($38|0)!=(0);
  if ($39) {
   HEAP32[$0>>2] = 0;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  } else {
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP8[$2>>0]|0;
   $42 = (_valueBytes($40,$41)|0);
   HEAP32[$0>>2] = $42;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
 }
 return (0)|0;
}
function _sqlite3VdbeRealValue($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0.0, $38 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0.0, $val = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 16|0;
 $val = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 8;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = +HEAPF64[$8>>3];
  HEAPF64[$0>>3] = $9;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 $10 = ((($8)) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & 4;
 $14 = ($13|0)!=(0);
 $15 = HEAP32[$1>>2]|0;
 if ($14) {
  $16 = $15;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (+($18>>>0)) + (4294967296.0*(+($21|0)));
  HEAPF64[$0>>3] = $22;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 $23 = ((($15)) + 8|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = $25 & 18;
 $27 = ($26|0)!=(0);
 if ($27) {
  HEAPF64[$val>>3] = 0.0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 10|0);
  $36 = HEAP8[$35>>0]|0;
  (_sqlite3AtoF($30,$val,$33,$36)|0);
  $37 = +HEAPF64[$val>>3];
  HEAPF64[$0>>3] = $37;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 } else {
  HEAPF64[$0>>3] = 0.0;
  $38 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$38);
 }
 return +(0.0);
}
function _sqlite3VdbeIntValue($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $flags = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 20|0;
 $flags = sp + 16|0;
 $value = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 HEAP32[$flags>>2] = $5;
 $6 = HEAP32[$flags>>2]|0;
 $7 = $6 & 4;
 $8 = ($7|0)!=(0);
 do {
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = $9;
   $11 = $10;
   $12 = HEAP32[$11>>2]|0;
   $13 = (($10) + 4)|0;
   $14 = $13;
   $15 = HEAP32[$14>>2]|0;
   $16 = $0;
   $17 = $16;
   HEAP32[$17>>2] = $12;
   $18 = (($16) + 4)|0;
   $19 = $18;
   HEAP32[$19>>2] = $15;
  } else {
   $20 = HEAP32[$flags>>2]|0;
   $21 = $20 & 8;
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = +HEAPF64[$23>>3];
    $25 = (_doubleToInt64($24)|0);
    $26 = tempRet0;
    $27 = $0;
    $28 = $27;
    HEAP32[$28>>2] = $25;
    $29 = (($27) + 4)|0;
    $30 = $29;
    HEAP32[$30>>2] = $26;
    break;
   }
   $31 = HEAP32[$flags>>2]|0;
   $32 = $31 & 18;
   $33 = ($32|0)!=(0);
   if ($33) {
    $34 = $value;
    $35 = $34;
    HEAP32[$35>>2] = 0;
    $36 = (($34) + 4)|0;
    $37 = $36;
    HEAP32[$37>>2] = 0;
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 16|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$1>>2]|0;
    $45 = ((($44)) + 10|0);
    $46 = HEAP8[$45>>0]|0;
    (_sqlite3Atoi64($40,$value,$43,$46)|0);
    $47 = $value;
    $48 = $47;
    $49 = HEAP32[$48>>2]|0;
    $50 = (($47) + 4)|0;
    $51 = $50;
    $52 = HEAP32[$51>>2]|0;
    $53 = $0;
    $54 = $53;
    HEAP32[$54>>2] = $49;
    $55 = (($53) + 4)|0;
    $56 = $55;
    HEAP32[$56>>2] = $52;
    break;
   } else {
    $57 = $0;
    $58 = $57;
    HEAP32[$58>>2] = 0;
    $59 = (($57) + 4)|0;
    $60 = $59;
    HEAP32[$60>>2] = 0;
    break;
   }
  }
 } while(0);
 $61 = $0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 tempRet0 = ($66);
 STACKTOP = sp;return ($63|0);
}
function _sqlite3VdbeMemMakeWriteable($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 18;
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP16[$9>>1]|0;
   $11 = $10&65535;
   $12 = $11 & 16384;
   $13 = ($12|0)!=(0);
   if ($13) {
    $14 = HEAP32[$1>>2]|0;
    $15 = (_sqlite3VdbeMemExpandBlob($14)|0);
    $16 = ($15|0)!=(0);
    if ($16) {
     HEAP32[$0>>2] = 7;
     $62 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($62|0);
    }
   }
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 24|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0);
   if (!($20)) {
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 16|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 20|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ($23|0)!=($26|0);
    if (!($27)) {
     break;
    }
   }
   $28 = HEAP32[$1>>2]|0;
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 12|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + 2)|0;
   $33 = (_sqlite3VdbeMemGrow($28,$32,1)|0);
   $34 = ($33|0)!=(0);
   if (!($34)) {
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 16|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + ($37)|0);
    HEAP8[$41>>0] = 0;
    $42 = HEAP32[$1>>2]|0;
    $43 = ((($42)) + 12|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + 1)|0;
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 16|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + ($45)|0);
    HEAP8[$49>>0] = 0;
    $50 = HEAP32[$1>>2]|0;
    $51 = ((($50)) + 8|0);
    $52 = HEAP16[$51>>1]|0;
    $53 = $52&65535;
    $54 = $53 | 512;
    $55 = $54&65535;
    HEAP16[$51>>1] = $55;
    break;
   }
   HEAP32[$0>>2] = 7;
   $62 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($62|0);
  }
 } while(0);
 $56 = HEAP32[$1>>2]|0;
 $57 = ((($56)) + 8|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = $59 & -4097;
 $61 = $60&65535;
 HEAP16[$57>>1] = $61;
 HEAP32[$0>>2] = 0;
 $62 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3ValueFree($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($6,$7);
 STACKTOP = sp;return;
}
function _setResultStrOrError($pCtx,$z,$n,$enc,$xDel) {
 $pCtx = $pCtx|0;
 $z = $z|0;
 $n = $n|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $4 = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP8[$3>>0]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_sqlite3VdbeMemSetStr($6,$7,$8,$9,$10)|0);
 $12 = ($11|0)==(18);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 _sqlite3_result_error_toobig($13);
 STACKTOP = sp;return;
}
function _invokeValueDestructor($p,$xDel,$pCtx) {
 $p = $p|0;
 $xDel = $xDel|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $xDel;
 HEAP32[$2>>2] = $pCtx;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==((-1)|0);
 $or$cond = $4 | $6;
 if (!($or$cond)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$7 & 255]($8);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return 18;
 }
 $11 = HEAP32[$2>>2]|0;
 _sqlite3_result_error_toobig($11);
 STACKTOP = sp;return 18;
}
function _sqlite3VdbeMemSetDouble($pMem,$val) {
 $pMem = $pMem|0;
 $val = +$val;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0.0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMem;
 HEAPF64[$1>>3] = $val;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($2);
 $3 = +HEAPF64[$1>>3];
 $4 = (_sqlite3IsNaN($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = +HEAPF64[$1>>3];
 $7 = HEAP32[$0>>2]|0;
 HEAPF64[$7>>3] = $6;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP16[$9>>1] = 8;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemSetStr($pMem,$z,$n,$enc,$xDel) {
 $pMem = $pMem|0;
 $z = $z|0;
 $n = $n|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $flags = 0;
 var $iLimit = 0, $nAlloc = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 34|0;
 $5 = sp + 12|0;
 $nByte = sp + 8|0;
 $iLimit = sp + 4|0;
 $flags = sp + 32|0;
 $nAlloc = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP8[$4>>0] = $enc;
 HEAP32[$5>>2] = $xDel;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$nByte>>2] = $6;
 HEAP16[$flags>>1] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 if (!($8)) {
  _sqlite3VdbeMemSetNull($9);
  HEAP32[$0>>2] = 0;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 }
 $10 = ((($9)) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 96|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$iLimit>>2] = $17;
 } else {
  HEAP32[$iLimit>>2] = 1000000000;
 }
 $18 = HEAP8[$4>>0]|0;
 $19 = $18&255;
 $20 = ($19|0)==(0);
 $21 = $20 ? 16 : 2;
 $22 = $21&65535;
 HEAP16[$flags>>1] = $22;
 $23 = HEAP32[$nByte>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP8[$4>>0]|0;
  $26 = $25&255;
  $27 = ($26|0)==(1);
  L11: do {
   if ($27) {
    $28 = HEAP32[$2>>2]|0;
    $29 = (_sqlite3Strlen30($28)|0);
    HEAP32[$nByte>>2] = $29;
    $30 = HEAP32[$nByte>>2]|0;
    $31 = HEAP32[$iLimit>>2]|0;
    $32 = ($30|0)>($31|0);
    if ($32) {
     $33 = HEAP32[$iLimit>>2]|0;
     $34 = (($33) + 1)|0;
     HEAP32[$nByte>>2] = $34;
    }
   } else {
    HEAP32[$nByte>>2] = 0;
    while(1) {
     $35 = HEAP32[$nByte>>2]|0;
     $36 = HEAP32[$iLimit>>2]|0;
     $37 = ($35|0)<=($36|0);
     if (!($37)) {
      break L11;
     }
     $38 = HEAP32[$nByte>>2]|0;
     $39 = HEAP32[$2>>2]|0;
     $40 = (($39) + ($38)|0);
     $41 = HEAP8[$40>>0]|0;
     $42 = $41 << 24 >> 24;
     $43 = HEAP32[$nByte>>2]|0;
     $44 = (($43) + 1)|0;
     $45 = HEAP32[$2>>2]|0;
     $46 = (($45) + ($44)|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = $47 << 24 >> 24;
     $49 = $42 | $48;
     $50 = ($49|0)!=(0);
     if (!($50)) {
      break L11;
     }
     $51 = HEAP32[$nByte>>2]|0;
     $52 = (($51) + 2)|0;
     HEAP32[$nByte>>2] = $52;
    }
   }
  } while(0);
  $53 = HEAP16[$flags>>1]|0;
  $54 = $53&65535;
  $55 = $54 | 512;
  $56 = $55&65535;
  HEAP16[$flags>>1] = $56;
 }
 $57 = HEAP32[$5>>2]|0;
 $58 = ($57|0)==((-1)|0);
 do {
  if ($58) {
   $59 = HEAP32[$nByte>>2]|0;
   HEAP32[$nAlloc>>2] = $59;
   $60 = HEAP16[$flags>>1]|0;
   $61 = $60&65535;
   $62 = $61 & 512;
   $63 = ($62|0)!=(0);
   if ($63) {
    $64 = HEAP8[$4>>0]|0;
    $65 = $64&255;
    $66 = ($65|0)==(1);
    $67 = $66 ? 1 : 2;
    $68 = HEAP32[$nAlloc>>2]|0;
    $69 = (($68) + ($67))|0;
    HEAP32[$nAlloc>>2] = $69;
   }
   $70 = HEAP32[$nByte>>2]|0;
   $71 = HEAP32[$iLimit>>2]|0;
   $72 = ($70|0)>($71|0);
   if ($72) {
    HEAP32[$0>>2] = 18;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
   $73 = HEAP32[$1>>2]|0;
   $74 = HEAP32[$nAlloc>>2]|0;
   $75 = ($74|0)>(32);
   $76 = HEAP32[$nAlloc>>2]|0;
   $77 = $75 ? $76 : 32;
   $78 = (_sqlite3VdbeMemClearAndResize($73,$77)|0);
   $79 = ($78|0)!=(0);
   if (!($79)) {
    $80 = HEAP32[$1>>2]|0;
    $81 = ((($80)) + 16|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = HEAP32[$2>>2]|0;
    $84 = HEAP32[$nAlloc>>2]|0;
    _memcpy(($82|0),($83|0),($84|0))|0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  } else {
   $85 = HEAP32[$5>>2]|0;
   $86 = ($85|0)==(170|0);
   $87 = HEAP32[$1>>2]|0;
   _sqlite3VdbeMemRelease($87);
   $88 = HEAP32[$2>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = ((($89)) + 16|0);
   HEAP32[$90>>2] = $88;
   if ($86) {
    $91 = HEAP32[$1>>2]|0;
    $92 = ((($91)) + 20|0);
    HEAP32[$92>>2] = $88;
    $93 = HEAP32[$1>>2]|0;
    $94 = ((($93)) + 32|0);
    $95 = HEAP32[$94>>2]|0;
    $96 = HEAP32[$1>>2]|0;
    $97 = ((($96)) + 20|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = (_sqlite3DbMallocSize($95,$98)|0);
    $100 = HEAP32[$1>>2]|0;
    $101 = ((($100)) + 24|0);
    HEAP32[$101>>2] = $99;
    break;
   } else {
    $102 = HEAP32[$5>>2]|0;
    $103 = HEAP32[$1>>2]|0;
    $104 = ((($103)) + 36|0);
    HEAP32[$104>>2] = $102;
    $105 = HEAP32[$5>>2]|0;
    $106 = ($105|0)==(0|0);
    $107 = $106 ? 2048 : 1024;
    $108 = HEAP16[$flags>>1]|0;
    $109 = $108&65535;
    $110 = $109 | $107;
    $111 = $110&65535;
    HEAP16[$flags>>1] = $111;
    break;
   }
  }
 } while(0);
 $112 = HEAP32[$nByte>>2]|0;
 $113 = HEAP32[$1>>2]|0;
 $114 = ((($113)) + 12|0);
 HEAP32[$114>>2] = $112;
 $115 = HEAP16[$flags>>1]|0;
 $116 = HEAP32[$1>>2]|0;
 $117 = ((($116)) + 8|0);
 HEAP16[$117>>1] = $115;
 $118 = HEAP8[$4>>0]|0;
 $119 = $118&255;
 $120 = ($119|0)==(0);
 $121 = HEAP8[$4>>0]|0;
 $122 = $121&255;
 $123 = $120 ? 1 : $122;
 $124 = $123&255;
 $125 = HEAP32[$1>>2]|0;
 $126 = ((($125)) + 10|0);
 HEAP8[$126>>0] = $124;
 $127 = HEAP32[$1>>2]|0;
 $128 = ((($127)) + 10|0);
 $129 = HEAP8[$128>>0]|0;
 $130 = $129&255;
 $131 = ($130|0)!=(1);
 if ($131) {
  $132 = HEAP32[$1>>2]|0;
  $133 = (_sqlite3VdbeMemHandleBom($132)|0);
  $134 = ($133|0)!=(0);
  if ($134) {
   HEAP32[$0>>2] = 7;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
 }
 $135 = HEAP32[$nByte>>2]|0;
 $136 = HEAP32[$iLimit>>2]|0;
 $137 = ($135|0)>($136|0);
 if ($137) {
  HEAP32[$0>>2] = 18;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 } else {
  HEAP32[$0>>2] = 0;
  $138 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($138|0);
 }
 return (0)|0;
}
function _sqlite3VdbeMemSetInt64($pMem,$0,$1) {
 $pMem = $pMem|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pMem;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 9312;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$2>>2]|0;
  $15 = $3;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  _vdbeReleaseAndSetInt64($14,$17,$20);
  STACKTOP = sp;return;
 } else {
  $21 = $3;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = $27;
  $29 = $28;
  HEAP32[$29>>2] = $23;
  $30 = (($28) + 4)|0;
  $31 = $30;
  HEAP32[$31>>2] = $26;
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 8|0);
  HEAP16[$33>>1] = 4;
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeMemSetNull($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  _vdbeMemClearExternAndSetNull($7);
  STACKTOP = sp;return;
 } else {
  $8 = ((($7)) + 8|0);
  HEAP16[$8>>1] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeMemCopy($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 9312;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  _vdbeMemClearExternAndSetNull($8);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 ;HEAP32[$9>>2]=HEAP32[$10>>2]|0;HEAP32[$9+4>>2]=HEAP32[$10+4>>2]|0;HEAP32[$9+8>>2]=HEAP32[$10+8>>2]|0;HEAP32[$9+12>>2]=HEAP32[$10+12>>2]|0;HEAP32[$9+16>>2]=HEAP32[$10+16>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -1025;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & 18;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26 & 2048;
 $28 = (0)==($27|0);
 if (!($28)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 8|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31&65535;
 $33 = $32 | 4096;
 $34 = $33&65535;
 HEAP16[$30>>1] = $34;
 $35 = HEAP32[$0>>2]|0;
 $36 = (_sqlite3VdbeMemMakeWriteable($35)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3VdbeMemSetZeroBlob($pMem,$n) {
 $pMem = $pMem|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $n;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 HEAP16[$4>>1] = 16400;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 12|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7|0)<(0);
 if ($8) {
  HEAP32[$1>>2] = 0;
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 10|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 16|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3ErrStr($rc) {
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cond = 0, $or$cond = 0, $zErr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $zErr = sp;
 HEAP32[$0>>2] = $rc;
 HEAP32[$zErr>>2] = 43378;
 $1 = HEAP32[$0>>2]|0;
 $cond = ($1|0)==(516);
 if ($cond) {
  HEAP32[$zErr>>2] = 43392;
 } else {
  $2 = HEAP32[$0>>2]|0;
  $3 = $2 & 255;
  HEAP32[$0>>2] = $3;
  $4 = HEAP32[$0>>2]|0;
  $5 = ($4|0)>=(0);
  $6 = HEAP32[$0>>2]|0;
  $7 = ($6|0)<(27);
  $or$cond = $5 & $7;
  if ($or$cond) {
   $8 = HEAP32[$0>>2]|0;
   $9 = (6240 + ($8<<2)|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = (6240 + ($12<<2)|0);
    $14 = HEAP32[$13>>2]|0;
    HEAP32[$zErr>>2] = $14;
   }
  }
 }
 $15 = HEAP32[$zErr>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3OomFault($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 69|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 70|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 69|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 168|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)>(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 248|0);
  HEAP32[$18>>2] = 1;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 256|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + 1)|0;
 HEAP32[$20>>2] = $22;
 STACKTOP = sp;return;
}
function _vdbeSafetyNotNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  _sqlite3_log(21,43338,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_vdbeSafety($4)|0);
  HEAP32[$0>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 return (0)|0;
}
function _sqlite3Step($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(770837923);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  (_sqlite3_reset($6)|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 69|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 $13 = HEAP32[$1>>2]|0;
 if ($12) {
  $14 = ((($13)) + 40|0);
  HEAP32[$14>>2] = 7;
  HEAP32[$0>>2] = 7;
  $163 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($163|0);
 }
 $15 = ((($13)) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)<=(0);
 if ($17) {
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 144|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20 & 1;
  $22 = $21&65535;
  $23 = ($22|0)!=(0);
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 40|0);
   HEAP32[$25>>2] = 17;
   HEAP32[$rc>>2] = 1;
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 36|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)<(0);
  if ($29) {
   $30 = HEAP32[$db>>2]|0;
   $31 = ((($30)) + 156|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = ($32|0)==(0);
   if ($33) {
    $34 = HEAP32[$db>>2]|0;
    $35 = ((($34)) + 248|0);
    HEAP32[$35>>2] = 0;
   }
   $36 = HEAP32[$db>>2]|0;
   $37 = ((($36)) + 192|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($38|0)!=(0|0);
   if ($39) {
    label = 13;
   } else {
    $40 = HEAP32[$db>>2]|0;
    $41 = ((($40)) + 76|0);
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = $43 & 2;
    $45 = ($44|0)!=(0);
    if ($45) {
     label = 13;
    }
   }
   if ((label|0) == 13) {
    $46 = HEAP32[$db>>2]|0;
    $47 = ((($46)) + 148|0);
    $48 = ((($47)) + 5|0);
    $49 = HEAP8[$48>>0]|0;
    $50 = ($49<<24>>24)!=(0);
    if (!($50)) {
     $51 = HEAP32[$1>>2]|0;
     $52 = ((($51)) + 176|0);
     $53 = HEAP32[$52>>2]|0;
     $54 = ($53|0)!=(0|0);
     if ($54) {
      $55 = HEAP32[$db>>2]|0;
      $56 = HEAP32[$55>>2]|0;
      $57 = HEAP32[$1>>2]|0;
      $58 = ((($57)) + 128|0);
      (_sqlite3OsCurrentTimeInt64($56,$58)|0);
     }
    }
   }
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 156|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + 1)|0;
   HEAP32[$60>>2] = $62;
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 144|0);
   $65 = HEAP16[$64>>1]|0;
   $66 = ($65&65535) >>> 7;
   $67 = $66 & 1;
   $68 = $67&65535;
   $69 = ($68|0)==(0);
   if ($69) {
    $70 = HEAP32[$db>>2]|0;
    $71 = ((($70)) + 164|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (($72) + 1)|0;
    HEAP32[$71>>2] = $73;
   }
   $74 = HEAP32[$1>>2]|0;
   $75 = ((($74)) + 144|0);
   $76 = HEAP16[$75>>1]|0;
   $77 = ($76&65535) >>> 8;
   $78 = $77 & 1;
   $79 = $78&65535;
   $80 = ($79|0)!=(0);
   if ($80) {
    $81 = HEAP32[$db>>2]|0;
    $82 = ((($81)) + 160|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = (($83) + 1)|0;
    HEAP32[$82>>2] = $84;
   }
   $85 = HEAP32[$1>>2]|0;
   $86 = ((($85)) + 36|0);
   HEAP32[$86>>2] = 0;
  }
  $87 = HEAP32[$1>>2]|0;
  $88 = ((($87)) + 144|0);
  $89 = HEAP16[$88>>1]|0;
  $90 = ($89&65535) >>> 2;
  $91 = $90 & 3;
  $92 = $91&65535;
  $93 = ($92|0)!=(0);
  if ($93) {
   $94 = HEAP32[$1>>2]|0;
   $95 = (_sqlite3VdbeList($94)|0);
   HEAP32[$rc>>2] = $95;
  } else {
   $96 = HEAP32[$db>>2]|0;
   $97 = ((($96)) + 168|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = (($98) + 1)|0;
   HEAP32[$97>>2] = $99;
   $100 = HEAP32[$1>>2]|0;
   $101 = (_sqlite3VdbeExec($100)|0);
   HEAP32[$rc>>2] = $101;
   $102 = HEAP32[$db>>2]|0;
   $103 = ((($102)) + 168|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = (($104) + -1)|0;
   HEAP32[$103>>2] = $105;
  }
  $106 = HEAP32[$rc>>2]|0;
  $107 = ($106|0)!=(100);
  if ($107) {
   $108 = HEAP32[$1>>2]|0;
   $109 = ((($108)) + 128|0);
   $110 = $109;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $113 = (($110) + 4)|0;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = ($115|0)>(0);
   $117 = ($112>>>0)>(0);
   $118 = ($115|0)==(0);
   $119 = $118 & $117;
   $120 = $116 | $119;
   if ($120) {
    $121 = HEAP32[$db>>2]|0;
    $122 = HEAP32[$1>>2]|0;
    _invokeProfileCallback($121,$122);
   }
  }
  $123 = HEAP32[$rc>>2]|0;
  $124 = ($123|0)==(101);
  if ($124) {
   $125 = HEAP32[$db>>2]|0;
   $126 = (_doWalCallbacks($125)|0);
   $127 = HEAP32[$1>>2]|0;
   $128 = ((($127)) + 40|0);
   HEAP32[$128>>2] = $126;
   $129 = HEAP32[$1>>2]|0;
   $130 = ((($129)) + 40|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = ($131|0)!=(0);
   if ($132) {
    HEAP32[$rc>>2] = 1;
   }
  }
  $133 = HEAP32[$rc>>2]|0;
  $134 = HEAP32[$db>>2]|0;
  $135 = ((($134)) + 52|0);
  HEAP32[$135>>2] = $133;
  $136 = HEAP32[$1>>2]|0;
  $137 = HEAP32[$136>>2]|0;
  $138 = HEAP32[$1>>2]|0;
  $139 = ((($138)) + 40|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = (_sqlite3ApiExit($137,$140)|0);
  $142 = (7)==($141|0);
  if ($142) {
   $143 = HEAP32[$1>>2]|0;
   $144 = ((($143)) + 40|0);
   HEAP32[$144>>2] = 7;
  }
 }
 $145 = HEAP32[$1>>2]|0;
 $146 = ((($145)) + 144|0);
 $147 = HEAP16[$146>>1]|0;
 $148 = ($147&65535) >>> 9;
 $149 = $148 & 1;
 $150 = $149&65535;
 $151 = ($150|0)!=(0);
 $152 = HEAP32[$rc>>2]|0;
 $153 = ($152|0)!=(100);
 $or$cond = $151 & $153;
 $154 = HEAP32[$rc>>2]|0;
 $155 = ($154|0)!=(101);
 $or$cond3 = $or$cond & $155;
 if ($or$cond3) {
  $156 = HEAP32[$1>>2]|0;
  $157 = (_sqlite3VdbeTransferError($156)|0);
  HEAP32[$rc>>2] = $157;
 }
 $158 = HEAP32[$rc>>2]|0;
 $159 = HEAP32[$db>>2]|0;
 $160 = ((($159)) + 56|0);
 $161 = HEAP32[$160>>2]|0;
 $162 = $158 & $161;
 HEAP32[$0>>2] = $162;
 $163 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($163|0);
}
function _sqlite3Reprepare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $pNew = 0, $rc = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $pNew = sp + 8|0;
 $zSql = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3_sql($2)|0);
 HEAP32[$zSql>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3VdbeDb($4)|0);
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$zSql>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3LockAndPrepare($6,$7,-1,0,$8,$pNew,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $16 = HEAP32[$pNew>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  _sqlite3VdbeSwap($16,$17);
  $18 = HEAP32[$pNew>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  (_sqlite3TransferBindings($18,$19)|0);
  $20 = HEAP32[$pNew>>2]|0;
  _sqlite3VdbeResetStepResult($20);
  $21 = HEAP32[$pNew>>2]|0;
  (_sqlite3VdbeFinalize($21)|0);
  HEAP32[$0>>2] = 0;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(7);
 if ($13) {
  $14 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($14);
 }
 $15 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $15;
 $22 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3DbStrDup($db,$z) {
 $db = $db|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $n = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zNew = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $z;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 $7 = (($6) + 1)|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3DbMallocRaw($8,$9,$11)|0);
 HEAP32[$zNew>>2] = $12;
 $13 = HEAP32[$zNew>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$zNew>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$n>>2]|0;
  _memcpy(($15|0),($16|0),($17|0))|0;
 }
 $18 = HEAP32[$zNew>>2]|0;
 HEAP32[$0>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _createAggContext($p,$nByte) {
 $p = $p|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pMem = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nByte;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pMem>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<=(0);
 $7 = HEAP32[$pMem>>2]|0;
 if ($6) {
  _sqlite3VdbeMemSetNull($7);
  $8 = HEAP32[$pMem>>2]|0;
  $9 = ((($8)) + 16|0);
  HEAP32[$9>>2] = 0;
 } else {
  $10 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemClearAndResize($7,$10)|0);
  $11 = HEAP32[$pMem>>2]|0;
  $12 = ((($11)) + 8|0);
  HEAP16[$12>>1] = 8192;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$pMem>>2]|0;
  HEAP32[$16>>2] = $15;
  $17 = HEAP32[$pMem>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0|0);
  if ($20) {
   $21 = HEAP32[$pMem>>2]|0;
   $22 = ((($21)) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   _memset(($23|0),0,($24|0))|0;
  }
 }
 $25 = HEAP32[$pMem>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3DbMallocZero($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $db;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3DbMallocRaw($8,$11,$14)|0);
 HEAP32[$p>>2] = $15;
 $16 = HEAP32[$p>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $25 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $18 = HEAP32[$p>>2]|0;
 $19 = $3;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 _memset(($18|0),0,($21|0))|0;
 $25 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _columnMem($pStmt,$i) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pOut = 0, $pVm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pVm = sp + 4|0;
 $pOut = sp;
 HEAP32[$1>>2] = $pStmt;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pVm>>2] = $3;
 $4 = HEAP32[$pVm>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = (_columnNullValue()|0);
  HEAP32[$0>>2] = $6;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $7 = HEAP32[$pVm>>2]|0;
 $8 = ((($7)) + 104|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$pVm>>2]|0;
  $13 = ((($12)) + 140|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = ($11|0)<($15|0);
  $17 = HEAP32[$2>>2]|0;
  $18 = ($17|0)>=(0);
  $or$cond = $16 & $18;
  if ($or$cond) {
   $19 = HEAP32[$2>>2]|0;
   $20 = HEAP32[$pVm>>2]|0;
   $21 = ((($20)) + 104|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + (($19*40)|0)|0);
   HEAP32[$pOut>>2] = $23;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $24 = HEAP32[$pVm>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  _sqlite3Error($25,25);
  $26 = (_columnNullValue()|0);
  HEAP32[$pOut>>2] = $26;
 }
 $27 = HEAP32[$pOut>>2]|0;
 HEAP32[$0>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _columnMallocFailure($pStmt) {
 $pStmt = $pStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pStmt;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3ApiExit($5,$8)|0);
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 40|0);
 HEAP32[$11>>2] = $9;
 STACKTOP = sp;return;
}
function _columnName($pStmt,$N,$xFunc,$useType) {
 $pStmt = $pStmt|0;
 $N = $N|0;
 $xFunc = $xFunc|0;
 $useType = $useType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $or$cond = 0, $p = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $ret = sp + 12|0;
 $p = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $xFunc;
 HEAP32[$3>>2] = $useType;
 HEAP32[$ret>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_column_count($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$n>>2]|0;
 $11 = ($9|0)<($10|0);
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)>=(0);
 $or$cond = $11 & $13;
 if (!($or$cond)) {
  $31 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$n>>2]|0;
 $16 = Math_imul($14, $15)|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + ($16))|0;
 HEAP32[$1>>2] = $18;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 100|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + (($20*40)|0)|0);
 $25 = (FUNCTION_TABLE_ii[$19 & 255]($24)|0);
 HEAP32[$ret>>2] = $25;
 $26 = HEAP32[$db>>2]|0;
 $27 = ((($26)) + 69|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = ($28<<24>>24)!=(0);
 if (!($29)) {
  $31 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $30 = HEAP32[$db>>2]|0;
 _sqlite3OomClear($30);
 HEAP32[$ret>>2] = 0;
 $31 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _bindText($pStmt,$i,$zData,$nData,$xDel,$encoding) {
 $pStmt = $pStmt|0;
 $i = $i|0;
 $zData = $zData|0;
 $nData = $nData|0;
 $xDel = $xDel|0;
 $encoding = $encoding|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $pVar = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 32|0;
 $p = sp + 8|0;
 $pVar = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $zData;
 HEAP32[$3>>2] = $nData;
 HEAP32[$4>>2] = $xDel;
 HEAP8[$5>>0] = $encoding;
 $6 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_vdbeUnbind($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $45 = HEAP32[$4>>2]|0;
  $46 = ($45|0)!=(0|0);
  $47 = HEAP32[$4>>2]|0;
  $48 = ($47|0)!=((-1)|0);
  $or$cond = $46 & $48;
  if (!($or$cond)) {
   $51 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($51|0);
  }
  $49 = HEAP32[$4>>2]|0;
  $50 = HEAP32[$2>>2]|0;
  FUNCTION_TABLE_vi[$49 & 255]($50);
  $51 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $51 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (($14) - 1)|0;
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 116|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + (($15*40)|0)|0);
 HEAP32[$pVar>>2] = $19;
 $20 = HEAP32[$pVar>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = HEAP8[$5>>0]|0;
 $24 = HEAP32[$4>>2]|0;
 $25 = (_sqlite3VdbeMemSetStr($20,$21,$22,$23,$24)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP8[$5>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = HEAP32[$pVar>>2]|0;
   $32 = HEAP32[$p>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = ((($33)) + 66|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = $35&255;
   $37 = (_sqlite3VdbeChangeEncoding($31,$36)|0);
   HEAP32[$rc>>2] = $37;
  }
 }
 $38 = HEAP32[$p>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$rc>>2]|0;
 _sqlite3Error($39,$40);
 $41 = HEAP32[$p>>2]|0;
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$rc>>2]|0;
 $44 = (_sqlite3ApiExit($42,$43)|0);
 HEAP32[$rc>>2] = $44;
 $51 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($51|0);
}
function _vdbeUnbind($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $pVar = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pVar = sp + 4|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_vdbeSafetyNotNull($3)|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = (_sqlite3MisuseError(76348)|0);
  HEAP32[$0>>2] = $6;
  $68 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(770837923);
 if (!($10)) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)>=(0);
  if (!($14)) {
   $21 = HEAP32[$2>>2]|0;
   $22 = ($21|0)<(1);
   if (!($22)) {
    $23 = HEAP32[$2>>2]|0;
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 16|0);
    $26 = HEAP16[$25>>1]|0;
    $27 = $26 << 16 >> 16;
    $28 = ($23|0)>($27|0);
    if (!($28)) {
     $31 = HEAP32[$2>>2]|0;
     $32 = (($31) + -1)|0;
     HEAP32[$2>>2] = $32;
     $33 = HEAP32[$2>>2]|0;
     $34 = HEAP32[$1>>2]|0;
     $35 = ((($34)) + 116|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = (($36) + (($33*40)|0)|0);
     HEAP32[$pVar>>2] = $37;
     $38 = HEAP32[$pVar>>2]|0;
     _sqlite3VdbeMemRelease($38);
     $39 = HEAP32[$pVar>>2]|0;
     $40 = ((($39)) + 8|0);
     HEAP16[$40>>1] = 1;
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     _sqlite3Error($42,0);
     $43 = HEAP32[$1>>2]|0;
     $44 = ((($43)) + 144|0);
     $45 = HEAP16[$44>>1]|0;
     $46 = ($45&65535) >>> 9;
     $47 = $46 & 1;
     $48 = $47&65535;
     $49 = ($48|0)!=(0);
     do {
      if ($49) {
       $50 = HEAP32[$2>>2]|0;
       $51 = ($50|0)<(32);
       if ($51) {
        $52 = HEAP32[$1>>2]|0;
        $53 = ((($52)) + 196|0);
        $54 = HEAP32[$53>>2]|0;
        $55 = HEAP32[$2>>2]|0;
        $56 = 1 << $55;
        $57 = $54 & $56;
        $58 = ($57|0)!=(0);
        if (!($58)) {
         label = 12;
        }
       } else {
        label = 12;
       }
       if ((label|0) == 12) {
        $59 = HEAP32[$1>>2]|0;
        $60 = ((($59)) + 196|0);
        $61 = HEAP32[$60>>2]|0;
        $62 = ($61|0)==(-1);
        if (!($62)) {
         break;
        }
       }
       $63 = HEAP32[$1>>2]|0;
       $64 = ((($63)) + 144|0);
       $65 = HEAP16[$64>>1]|0;
       $66 = $65 & -2;
       $67 = $66 | 1;
       HEAP16[$64>>1] = $67;
      }
     } while(0);
     HEAP32[$0>>2] = 0;
     $68 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($68|0);
    }
   }
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   _sqlite3Error($30,25);
   HEAP32[$0>>2] = 25;
   $68 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($68|0);
  }
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 _sqlite3Error($16,21);
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 176|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer>>2] = $19;
 _sqlite3_log(21,39833,$vararg_buffer);
 $20 = (_sqlite3MisuseError(76356)|0);
 HEAP32[$0>>2] = $20;
 $68 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($68|0);
}
function _sqlite3VdbeParameterIndex($p,$zName,$nName) {
 $p = $p|0;
 $zName = $zName|0;
 $nName = $nName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $z = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $nName;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0|0);
 L5: do {
  if ($7) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $8 = HEAP32[$i>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = ((($9)) + 18|0);
    $11 = HEAP16[$10>>1]|0;
    $12 = $11 << 16 >> 16;
    $13 = ($8|0)<($12|0);
    if (!($13)) {
     break L5;
    }
    $14 = HEAP32[$i>>2]|0;
    $15 = HEAP32[$1>>2]|0;
    $16 = ((($15)) + 120|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (($17) + ($14<<2)|0);
    $19 = HEAP32[$18>>2]|0;
    HEAP32[$z>>2] = $19;
    $20 = HEAP32[$z>>2]|0;
    $21 = ($20|0)!=(0|0);
    if ($21) {
     $22 = HEAP32[$z>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = HEAP32[$3>>2]|0;
     $25 = (_strncmp($22,$23,$24)|0);
     $26 = ($25|0)==(0);
     if ($26) {
      $27 = HEAP32[$3>>2]|0;
      $28 = HEAP32[$z>>2]|0;
      $29 = (($28) + ($27)|0);
      $30 = HEAP8[$29>>0]|0;
      $31 = $30 << 24 >> 24;
      $32 = ($31|0)==(0);
      if ($32) {
       break;
      }
     }
    }
    $35 = HEAP32[$i>>2]|0;
    $36 = (($35) + 1)|0;
    HEAP32[$i>>2] = $36;
   }
   $33 = HEAP32[$i>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$0>>2] = $34;
   $37 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3TransferBindings($pFromStmt,$pToStmt) {
 $pFromStmt = $pFromStmt|0;
 $pToStmt = $pToStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pFrom = 0, $pTo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pFrom = sp + 8|0;
 $pTo = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pFromStmt;
 HEAP32[$1>>2] = $pToStmt;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFrom>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pTo>>2] = $3;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$pFrom>>2]|0;
  $6 = ((($5)) + 16|0);
  $7 = HEAP16[$6>>1]|0;
  $8 = $7 << 16 >> 16;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pTo>>2]|0;
  $12 = ((($11)) + 116|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + (($10*40)|0)|0);
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$pFrom>>2]|0;
  $17 = ((($16)) + 116|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + (($15*40)|0)|0);
  _sqlite3VdbeMemMove($14,$19);
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 STACKTOP = sp;return 0;
}
function _sqlite3VdbeExpandSql($p,$zRawSql) {
 $p = $p|0;
 $zRawSql = $zRawSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0.0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $enc = 0, $i = 0, $idx = 0, $n = 0, $nOut = 0, $nOut1 = 0, $nToken = 0, $nextIndex = 0, $out = 0, $pVar = 0, $utf8 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer11 = 0, $vararg_buffer4 = 0, $vararg_buffer8 = 0, $vararg_ptr7 = 0, $zBase = 0, $zStart = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer11 = sp + 72|0;
 $vararg_buffer8 = sp + 64|0;
 $vararg_buffer4 = sp + 56|0;
 $vararg_buffer1 = sp + 48|0;
 $vararg_buffer = sp + 40|0;
 $0 = sp + 148|0;
 $1 = sp + 144|0;
 $db = sp + 140|0;
 $idx = sp + 136|0;
 $nextIndex = sp + 132|0;
 $n = sp + 128|0;
 $nToken = sp + 124|0;
 $i = sp + 120|0;
 $pVar = sp + 116|0;
 $out = sp + 88|0;
 $utf8 = sp;
 $zBase = sp + 156|0;
 $zStart = sp + 84|0;
 $nOut = sp + 80|0;
 $enc = sp + 152|0;
 $nOut1 = sp + 76|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zRawSql;
 HEAP32[$idx>>2] = 0;
 HEAP32[$nextIndex>>2] = 1;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 96|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3StrAccumInit($out,0,$zBase,100,$6);
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 168|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)>(1);
 L1: do {
  if ($10) {
   while(1) {
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP8[$11>>0]|0;
    $13 = ($12<<24>>24)!=(0);
    if (!($13)) {
     break L1;
    }
    $14 = HEAP32[$1>>2]|0;
    HEAP32[$zStart>>2] = $14;
    while(1) {
     $15 = HEAP32[$1>>2]|0;
     $16 = ((($15)) + 1|0);
     HEAP32[$1>>2] = $16;
     $17 = HEAP8[$15>>0]|0;
     $18 = $17 << 24 >> 24;
     $19 = ($18|0)!=(10);
     if (!($19)) {
      break;
     }
     $20 = HEAP32[$1>>2]|0;
     $21 = HEAP8[$20>>0]|0;
     $22 = $21 << 24 >> 24;
     $23 = ($22|0)!=(0);
     if (!($23)) {
      break;
     }
    }
    _sqlite3StrAccumAppend($out,39799,3);
    $24 = HEAP32[$zStart>>2]|0;
    $25 = HEAP32[$1>>2]|0;
    $26 = HEAP32[$zStart>>2]|0;
    $27 = $25;
    $28 = $26;
    $29 = (($27) - ($28))|0;
    _sqlite3StrAccumAppend($out,$24,$29);
   }
  } else {
   $30 = HEAP32[$0>>2]|0;
   $31 = ((($30)) + 16|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = $32 << 16 >> 16;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$1>>2]|0;
    $36 = HEAP32[$1>>2]|0;
    $37 = (_sqlite3Strlen30($36)|0);
    _sqlite3StrAccumAppend($out,$35,$37);
    break;
   }
   while(1) {
    $38 = HEAP32[$1>>2]|0;
    $39 = HEAP8[$38>>0]|0;
    $40 = ($39<<24>>24)!=(0);
    if (!($40)) {
     break L1;
    }
    $41 = HEAP32[$1>>2]|0;
    $42 = (_findNextHostParameter($41,$nToken)|0);
    HEAP32[$n>>2] = $42;
    $43 = HEAP32[$1>>2]|0;
    $44 = HEAP32[$n>>2]|0;
    _sqlite3StrAccumAppend($out,$43,$44);
    $45 = HEAP32[$n>>2]|0;
    $46 = HEAP32[$1>>2]|0;
    $47 = (($46) + ($45)|0);
    HEAP32[$1>>2] = $47;
    $48 = HEAP32[$nToken>>2]|0;
    $49 = ($48|0)==(0);
    if ($49) {
     break L1;
    }
    $50 = HEAP32[$1>>2]|0;
    $51 = HEAP8[$50>>0]|0;
    $52 = $51 << 24 >> 24;
    $53 = ($52|0)==(63);
    do {
     if ($53) {
      $54 = HEAP32[$nToken>>2]|0;
      $55 = ($54|0)>(1);
      if ($55) {
       $56 = HEAP32[$1>>2]|0;
       $57 = ((($56)) + 1|0);
       (_sqlite3GetInt32($57,$idx)|0);
       break;
      } else {
       $58 = HEAP32[$nextIndex>>2]|0;
       HEAP32[$idx>>2] = $58;
       break;
      }
     } else {
      $59 = HEAP32[$0>>2]|0;
      $60 = HEAP32[$1>>2]|0;
      $61 = HEAP32[$nToken>>2]|0;
      $62 = (_sqlite3VdbeParameterIndex($59,$60,$61)|0);
      HEAP32[$idx>>2] = $62;
     }
    } while(0);
    $63 = HEAP32[$nToken>>2]|0;
    $64 = HEAP32[$1>>2]|0;
    $65 = (($64) + ($63)|0);
    HEAP32[$1>>2] = $65;
    $66 = HEAP32[$idx>>2]|0;
    $67 = (($66) + 1)|0;
    HEAP32[$nextIndex>>2] = $67;
    $68 = HEAP32[$idx>>2]|0;
    $69 = (($68) - 1)|0;
    $70 = HEAP32[$0>>2]|0;
    $71 = ((($70)) + 116|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (($72) + (($69*40)|0)|0);
    HEAP32[$pVar>>2] = $73;
    $74 = HEAP32[$pVar>>2]|0;
    $75 = ((($74)) + 8|0);
    $76 = HEAP16[$75>>1]|0;
    $77 = $76&65535;
    $78 = $77 & 1;
    $79 = ($78|0)!=(0);
    if ($79) {
     _sqlite3StrAccumAppend($out,39691,4);
     continue;
    }
    $80 = HEAP32[$pVar>>2]|0;
    $81 = ((($80)) + 8|0);
    $82 = HEAP16[$81>>1]|0;
    $83 = $82&65535;
    $84 = $83 & 4;
    $85 = ($84|0)!=(0);
    $86 = HEAP32[$pVar>>2]|0;
    if ($85) {
     $87 = $86;
     $88 = $87;
     $89 = HEAP32[$88>>2]|0;
     $90 = (($87) + 4)|0;
     $91 = $90;
     $92 = HEAP32[$91>>2]|0;
     $93 = $vararg_buffer;
     $94 = $93;
     HEAP32[$94>>2] = $89;
     $95 = (($93) + 4)|0;
     $96 = $95;
     HEAP32[$96>>2] = $92;
     _sqlite3XPrintf($out,28266,$vararg_buffer);
     continue;
    }
    $97 = ((($86)) + 8|0);
    $98 = HEAP16[$97>>1]|0;
    $99 = $98&65535;
    $100 = $99 & 8;
    $101 = ($100|0)!=(0);
    $102 = HEAP32[$pVar>>2]|0;
    if ($101) {
     $103 = +HEAPF64[$102>>3];
     HEAPF64[$vararg_buffer1>>3] = $103;
     _sqlite3XPrintf($out,28271,$vararg_buffer1);
     continue;
    }
    $104 = ((($102)) + 8|0);
    $105 = HEAP16[$104>>1]|0;
    $106 = $105&65535;
    $107 = $106 & 2;
    $108 = ($107|0)!=(0);
    if ($108) {
     $109 = HEAP32[$db>>2]|0;
     $110 = ((($109)) + 66|0);
     $111 = HEAP8[$110>>0]|0;
     HEAP8[$enc>>0] = $111;
     $112 = HEAP8[$enc>>0]|0;
     $113 = $112&255;
     $114 = ($113|0)!=(1);
     if ($114) {
      dest=$utf8; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
      $115 = HEAP32[$db>>2]|0;
      $116 = ((($utf8)) + 32|0);
      HEAP32[$116>>2] = $115;
      $117 = HEAP32[$pVar>>2]|0;
      $118 = ((($117)) + 16|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = HEAP32[$pVar>>2]|0;
      $121 = ((($120)) + 12|0);
      $122 = HEAP32[$121>>2]|0;
      $123 = HEAP8[$enc>>0]|0;
      (_sqlite3VdbeMemSetStr($utf8,$119,$122,$123,0)|0);
      $124 = (_sqlite3VdbeChangeEncoding($utf8,1)|0);
      $125 = (7)==($124|0);
      if ($125) {
       $126 = ((($out)) + 24|0);
       HEAP8[$126>>0] = 1;
       $127 = ((($out)) + 16|0);
       HEAP32[$127>>2] = 0;
      }
      HEAP32[$pVar>>2] = $utf8;
     }
     $128 = HEAP32[$pVar>>2]|0;
     $129 = ((($128)) + 12|0);
     $130 = HEAP32[$129>>2]|0;
     HEAP32[$nOut>>2] = $130;
     $131 = HEAP32[$nOut>>2]|0;
     $132 = HEAP32[$pVar>>2]|0;
     $133 = ((($132)) + 16|0);
     $134 = HEAP32[$133>>2]|0;
     HEAP32[$vararg_buffer4>>2] = $131;
     $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $134;
     _sqlite3XPrintf($out,39803,$vararg_buffer4);
     $135 = HEAP8[$enc>>0]|0;
     $136 = $135&255;
     $137 = ($136|0)!=(1);
     if (!($137)) {
      continue;
     }
     _sqlite3VdbeMemRelease($utf8);
     continue;
    } else {
     $138 = HEAP32[$pVar>>2]|0;
     $139 = ((($138)) + 8|0);
     $140 = HEAP16[$139>>1]|0;
     $141 = $140&65535;
     $142 = $141 & 16384;
     $143 = ($142|0)!=(0);
     if ($143) {
      $144 = HEAP32[$pVar>>2]|0;
      $145 = HEAP32[$144>>2]|0;
      HEAP32[$vararg_buffer8>>2] = $145;
      _sqlite3XPrintf($out,39810,$vararg_buffer8);
      continue;
     }
     _sqlite3StrAccumAppend($out,39823,2);
     $146 = HEAP32[$pVar>>2]|0;
     $147 = ((($146)) + 12|0);
     $148 = HEAP32[$147>>2]|0;
     HEAP32[$nOut1>>2] = $148;
     HEAP32[$i>>2] = 0;
     while(1) {
      $149 = HEAP32[$i>>2]|0;
      $150 = HEAP32[$nOut1>>2]|0;
      $151 = ($149|0)<($150|0);
      if (!($151)) {
       break;
      }
      $152 = HEAP32[$i>>2]|0;
      $153 = HEAP32[$pVar>>2]|0;
      $154 = ((($153)) + 16|0);
      $155 = HEAP32[$154>>2]|0;
      $156 = (($155) + ($152)|0);
      $157 = HEAP8[$156>>0]|0;
      $158 = $157 << 24 >> 24;
      $159 = $158 & 255;
      HEAP32[$vararg_buffer11>>2] = $159;
      _sqlite3XPrintf($out,39826,$vararg_buffer11);
      $160 = HEAP32[$i>>2]|0;
      $161 = (($160) + 1)|0;
      HEAP32[$i>>2] = $161;
     }
     _sqlite3StrAccumAppend($out,39831,1);
     continue;
    }
   }
  }
 } while(0);
 $162 = ((($out)) + 24|0);
 $163 = HEAP8[$162>>0]|0;
 $164 = ($163<<24>>24)!=(0);
 if (!($164)) {
  $165 = (_sqlite3StrAccumFinish($out)|0);
  STACKTOP = sp;return ($165|0);
 }
 _sqlite3StrAccumReset($out);
 $165 = (_sqlite3StrAccumFinish($out)|0);
 STACKTOP = sp;return ($165|0);
}
function _applyNumericAffinity($pRec,$bTryForInt) {
 $pRec = $pRec|0;
 $bTryForInt = $bTryForInt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $enc = 0, $iValue = 0, $rValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rValue = sp + 8|0;
 $iValue = sp;
 $enc = sp + 24|0;
 HEAP32[$0>>2] = $pRec;
 HEAP32[$1>>2] = $bTryForInt;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 10|0);
 $4 = HEAP8[$3>>0]|0;
 HEAP8[$enc>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP8[$enc>>0]|0;
 $12 = (_sqlite3AtoF($7,$rValue,$10,$11)|0);
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP8[$enc>>0]|0;
 $21 = (_sqlite3Atoi64($16,$iValue,$19,$20)|0);
 $22 = (0)==($21|0);
 if ($22) {
  $23 = $iValue;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = $29;
  $31 = $30;
  HEAP32[$31>>2] = $25;
  $32 = (($30) + 4)|0;
  $33 = $32;
  HEAP32[$33>>2] = $28;
  $34 = HEAP32[$0>>2]|0;
  $35 = ((($34)) + 8|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 | 4;
  $39 = $38&65535;
  HEAP16[$35>>1] = $39;
  STACKTOP = sp;return;
 }
 $40 = +HEAPF64[$rValue>>3];
 $41 = HEAP32[$0>>2]|0;
 HEAPF64[$41>>3] = $40;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 8|0);
 $44 = HEAP16[$43>>1]|0;
 $45 = $44&65535;
 $46 = $45 | 8;
 $47 = $46&65535;
 HEAP16[$43>>1] = $47;
 $48 = HEAP32[$1>>2]|0;
 $49 = ($48|0)!=(0);
 if (!($49)) {
  STACKTOP = sp;return;
 }
 $50 = HEAP32[$0>>2]|0;
 _sqlite3VdbeIntegerAffinity($50);
 STACKTOP = sp;return;
}
function _sqlite3DbMallocRaw($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $p = sp + 8|0;
 HEAP32[$3>>2] = $db;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  $12 = $4;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (($12) + 4)|0;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3DbMallocRawNN($11,$14,$17)|0);
  HEAP32[$2>>2] = $18;
  $27 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3Malloc($21,$24)|0);
  HEAP32[$p>>2] = $25;
  $26 = HEAP32[$p>>2]|0;
  HEAP32[$2>>2] = $26;
  $27 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3LocateTable($pParse,$flags,$zName,$zDbase) {
 $pParse = $pParse|0;
 $flags = $flags|0;
 $zName = $zName|0;
 $zDbase = $zDbase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pMod = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $zMsg = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $p = sp + 32|0;
 $zMsg = sp + 28|0;
 $pMod = sp + 24|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $zName;
 HEAP32[$4>>2] = $zDbase;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_sqlite3FindTable($9,$10,$11)|0);
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  $15 = HEAP32[$2>>2]|0;
  $16 = $15 & 1;
  $17 = ($16|0)!=(0);
  $18 = $17 ? 39772 : 39785;
  HEAP32[$zMsg>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = (_sqlite3FindDbName($20,$21)|0);
  $23 = ($22|0)<(1);
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = ((($25)) + 320|0);
   $27 = HEAP32[$3>>2]|0;
   $28 = (_sqlite3HashFind($26,$27)|0);
   HEAP32[$pMod>>2] = $28;
   $29 = HEAP32[$pMod>>2]|0;
   $30 = ($29|0)!=(0|0);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = HEAP32[$pMod>>2]|0;
    $33 = (_sqlite3VtabEponymousTableInit($31,$32)|0);
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = HEAP32[$pMod>>2]|0;
     $36 = ((($35)) + 16|0);
     $37 = HEAP32[$36>>2]|0;
     HEAP32[$0>>2] = $37;
     $51 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($51|0);
    }
   }
  }
  $38 = HEAP32[$2>>2]|0;
  $39 = $38 & 2;
  $40 = ($39|0)==(0);
  if ($40) {
   $41 = HEAP32[$4>>2]|0;
   $42 = ($41|0)!=(0|0);
   $43 = HEAP32[$1>>2]|0;
   $44 = HEAP32[$zMsg>>2]|0;
   if ($42) {
    $45 = HEAP32[$4>>2]|0;
    $46 = HEAP32[$3>>2]|0;
    HEAP32[$vararg_buffer>>2] = $44;
    $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr1>>2] = $45;
    $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
    HEAP32[$vararg_ptr2>>2] = $46;
    _sqlite3ErrorMsg($43,30029,$vararg_buffer);
   } else {
    $47 = HEAP32[$3>>2]|0;
    HEAP32[$vararg_buffer3>>2] = $44;
    $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
    HEAP32[$vararg_ptr6>>2] = $47;
    _sqlite3ErrorMsg($43,30039,$vararg_buffer3);
   }
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 17|0);
   HEAP8[$49>>0] = 1;
  }
 }
 $50 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $50;
 $51 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($51|0);
}
function _sqlite3ErrorMsg($pParse,$zFormat,$varargs) {
 $pParse = $pParse|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $db = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $zMsg = sp + 24|0;
 $ap = sp + 8|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$db>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VMPrintf($4,$5,$ap)|0);
 HEAP32[$zMsg>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 73|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$zMsg>>2]|0;
  _sqlite3DbFree($11,$12);
  STACKTOP = sp;return;
 } else {
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 36|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$14>>2] = $16;
  $17 = HEAP32[$db>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  _sqlite3DbFree($17,$20);
  $21 = HEAP32[$zMsg>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 4|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 12|0);
  HEAP32[$25>>2] = 1;
  STACKTOP = sp;return;
 }
}
function _sqlite3SchemaToIndex($db,$pSchema) {
 $db = $db|0;
 $pSchema = $pSchema|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSchema;
 HEAP32[$i>>2] = -1000000;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $20 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<4)|0);
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   label = 6;
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 6) {
  $20 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _sqlite3MPrintf($db,$zFormat,$varargs) {
 $db = $db|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ap = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VMPrintf($2,$3,$ap)|0);
 HEAP32[$z>>2] = $4;
 $5 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3VdbeCreate($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (_sqlite3DbMallocRawNN($4,208,0)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 88|0);
 dest=$9; stop=dest+120|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $10 = HEAP32[$db>>2]|0;
 $11 = HEAP32[$p>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0|0);
 if ($15) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$db>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ((($19)) + 4|0);
  HEAP32[$20>>2] = $16;
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 8|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 4|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$p>>2]|0;
 $29 = HEAP32[$db>>2]|0;
 $30 = ((($29)) + 4|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$p>>2]|0;
 $32 = ((($31)) + 20|0);
 HEAP32[$32>>2] = 381479589;
 $33 = HEAP32[$1>>2]|0;
 $34 = HEAP32[$p>>2]|0;
 $35 = ((($34)) + 12|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $36;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3VdbeAddOp4Int($p,$op,$p1,$p2,$p3,$p4) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $p4 = $p4|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $p4;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_sqlite3VdbeAddOp3($6,$7,$8,$9,$10)|0);
 HEAP32[$addr>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$addr>>2]|0;
 $14 = HEAP32[$5>>2]|0;
 $15 = $14;
 _sqlite3VdbeChangeP4($12,$13,$15,-14);
 $16 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3VdbeChangeP5($p,$p5) {
 $p = $p|0;
 $p5 = $p5|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $p5;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 136|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP8[$1>>0]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 136|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 88|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*20)|0)|0);
 $15 = ((($14)) + 3|0);
 HEAP8[$15>>0] = $6;
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOpList($p,$nOp,$aOp,$iLineno) {
 $p = $p|0;
 $nOp = $nOp|0;
 $aOp = $aOp|0;
 $iLineno = $iLineno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $i = 0, $pFirst = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $i = sp + 8|0;
 $pOut = sp + 4|0;
 $pFirst = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nOp;
 HEAP32[$3>>2] = $aOp;
 HEAP32[$4>>2] = $iLineno;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 136|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($7) + ($8))|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($9|0)>($14|0);
 if ($15) {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_growOpArray($16,$17)|0);
  $19 = ($18|0)!=(0);
  if ($19) {
   HEAP32[$0>>2] = 0;
   $89 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($89|0);
  }
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 136|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 88|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + (($22*20)|0)|0);
 HEAP32[$pOut>>2] = $26;
 HEAP32[$pFirst>>2] = $26;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP8[$30>>0]|0;
  $32 = HEAP32[$pOut>>2]|0;
  HEAP8[$32>>0] = $31;
  $33 = HEAP32[$3>>2]|0;
  $34 = ((($33)) + 1|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35 << 24 >> 24;
  $37 = HEAP32[$pOut>>2]|0;
  $38 = ((($37)) + 4|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$3>>2]|0;
  $40 = ((($39)) + 2|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = HEAP32[$pOut>>2]|0;
  $44 = ((($43)) + 8|0);
  HEAP32[$44>>2] = $42;
  $45 = HEAP32[$3>>2]|0;
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  $48 = (34898 + ($47)|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = $50 & 1;
  $52 = ($51|0)!=(0);
  if ($52) {
   $53 = HEAP32[$3>>2]|0;
   $54 = ((($53)) + 2|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = ($56|0)>(0);
   if ($57) {
    $58 = HEAP32[$1>>2]|0;
    $59 = ((($58)) + 136|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$pOut>>2]|0;
    $62 = ((($61)) + 8|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = (($63) + ($60))|0;
    HEAP32[$62>>2] = $64;
   }
  }
  $65 = HEAP32[$3>>2]|0;
  $66 = ((($65)) + 3|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67 << 24 >> 24;
  $69 = HEAP32[$pOut>>2]|0;
  $70 = ((($69)) + 12|0);
  HEAP32[$70>>2] = $68;
  $71 = HEAP32[$pOut>>2]|0;
  $72 = ((($71)) + 1|0);
  HEAP8[$72>>0] = 0;
  $73 = HEAP32[$pOut>>2]|0;
  $74 = ((($73)) + 16|0);
  HEAP32[$74>>2] = 0;
  $75 = HEAP32[$pOut>>2]|0;
  $76 = ((($75)) + 3|0);
  HEAP8[$76>>0] = 0;
  $77 = HEAP32[$i>>2]|0;
  $78 = (($77) + 1)|0;
  HEAP32[$i>>2] = $78;
  $79 = HEAP32[$3>>2]|0;
  $80 = ((($79)) + 4|0);
  HEAP32[$3>>2] = $80;
  $81 = HEAP32[$pOut>>2]|0;
  $82 = ((($81)) + 20|0);
  HEAP32[$pOut>>2] = $82;
 }
 $83 = HEAP32[$2>>2]|0;
 $84 = HEAP32[$1>>2]|0;
 $85 = ((($84)) + 136|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = (($86) + ($83))|0;
 HEAP32[$85>>2] = $87;
 $88 = HEAP32[$pFirst>>2]|0;
 HEAP32[$0>>2] = $88;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _sqlite3VdbeUsesBtree($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$1>>2]|0;
 $3 = 1 << $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 148|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 | $3;
 HEAP32[$5>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(1);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($10<<4)|0);
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3BtreeSharable($17)|0);
 $19 = ($18|0)!=(0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = 1 << $20;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 152|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = $24 | $21;
 HEAP32[$23>>2] = $25;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP4($p,$addr,$zP4,$n) {
 $p = $p|0;
 $addr = $addr|0;
 $zP4 = $zP4|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pOp = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $zP4;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 69|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 if ($9) {
  $10 = HEAP32[$3>>2]|0;
  $11 = ($10|0)!=(-10);
  if (!($11)) {
   STACKTOP = sp;return;
  }
  $12 = HEAP32[$db>>2]|0;
  $13 = HEAP32[$3>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  _freeP4($12,$13,$14);
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ($15|0)<(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 136|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - 1)|0;
  HEAP32[$1>>2] = $20;
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 88|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + (($21*20)|0)|0);
 HEAP32[$pOp>>2] = $25;
 $26 = HEAP32[$3>>2]|0;
 $27 = ($26|0)>=(0);
 if (!($27)) {
  $28 = HEAP32[$pOp>>2]|0;
  $29 = ((($28)) + 1|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = $30 << 24 >> 24;
  $32 = ($31|0)!=(0);
  if (!($32)) {
   $37 = HEAP32[$3>>2]|0;
   $38 = ($37|0)==(-14);
   $39 = HEAP32[$2>>2]|0;
   if ($38) {
    $40 = $39;
    $41 = HEAP32[$pOp>>2]|0;
    $42 = ((($41)) + 16|0);
    HEAP32[$42>>2] = $40;
    $43 = HEAP32[$pOp>>2]|0;
    $44 = ((($43)) + 1|0);
    HEAP8[$44>>0] = -14;
    STACKTOP = sp;return;
   }
   $45 = ($39|0)!=(0|0);
   if (!($45)) {
    STACKTOP = sp;return;
   }
   $46 = HEAP32[$2>>2]|0;
   $47 = HEAP32[$pOp>>2]|0;
   $48 = ((($47)) + 16|0);
   HEAP32[$48>>2] = $46;
   $49 = HEAP32[$3>>2]|0;
   $50 = $49&255;
   $51 = HEAP32[$pOp>>2]|0;
   $52 = ((($51)) + 1|0);
   HEAP8[$52>>0] = $50;
   $53 = HEAP32[$3>>2]|0;
   $54 = ($53|0)==(-10);
   if (!($54)) {
    STACKTOP = sp;return;
   }
   $55 = HEAP32[$2>>2]|0;
   _sqlite3VtabLock($55);
   STACKTOP = sp;return;
  }
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = HEAP32[$pOp>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 _vdbeChangeP4Full($33,$34,$35,$36);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMakeReady($p,$pParse) {
 $p = $p|0;
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $n = 0, $nArg = 0, $nCursor = 0, $nMem = 0, $nVar = 0, $or$cond = 0;
 var $or$cond3 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $db = sp + 32|0;
 $nVar = sp + 28|0;
 $nMem = sp + 24|0;
 $nCursor = sp + 20|0;
 $nArg = sp + 16|0;
 $n = sp + 12|0;
 $x = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 400|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6 << 16 >> 16;
 HEAP32[$nVar>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 44|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$nMem>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$nCursor>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 108|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$nArg>>2] = $16;
 $17 = HEAP32[$nCursor>>2]|0;
 $18 = HEAP32[$nMem>>2]|0;
 $19 = (($18) + ($17))|0;
 HEAP32[$nMem>>2] = $19;
 $20 = HEAP32[$nCursor>>2]|0;
 $21 = ($20|0)==(0);
 $22 = HEAP32[$nMem>>2]|0;
 $23 = ($22|0)>(0);
 $or$cond = $21 & $23;
 if ($or$cond) {
  $24 = HEAP32[$nMem>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$nMem>>2] = $25;
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 136|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28*20)|0;
 $30 = (($29) + 7)|0;
 $31 = $30 & -8;
 HEAP32[$n>>2] = $31;
 $32 = HEAP32[$n>>2]|0;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 88|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + ($32)|0);
 HEAP32[$x>>2] = $36;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 52|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$n>>2]|0;
 $41 = (($39) - ($40))|0;
 $42 = $41 & -8;
 $43 = ((($x)) + 4|0);
 HEAP32[$43>>2] = $42;
 $44 = HEAP32[$0>>2]|0;
 _resolveP2Values($44,$nArg);
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 20|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47&255;
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 21|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = ($53|0)!=(0);
  $56 = $54;
 } else {
  $56 = 0;
 }
 $55 = $56&1;
 $57 = $55&255;
 $58 = $57&255;
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 144|0);
 $61 = $58&65535;
 $62 = HEAP16[$60>>1]|0;
 $63 = $61 & 1;
 $64 = ($63 << 6)&65535;
 $65 = $62 & -65;
 $66 = $65 | $64;
 HEAP16[$60>>1] = $66;
 $67 = HEAP32[$1>>2]|0;
 $68 = ((($67)) + 409|0);
 $69 = HEAP8[$68>>0]|0;
 $70 = $69&255;
 $71 = ($70|0)!=(0);
 $72 = HEAP32[$nMem>>2]|0;
 $73 = ($72|0)<(10);
 $or$cond3 = $71 & $73;
 if ($or$cond3) {
  HEAP32[$nMem>>2] = 10;
 }
 $74 = HEAP32[$0>>2]|0;
 $75 = ((($74)) + 144|0);
 $76 = HEAP16[$75>>1]|0;
 $77 = $76 & -2;
 HEAP16[$75>>1] = $77;
 while(1) {
  $78 = ((($x)) + 8|0);
  HEAP32[$78>>2] = 0;
  $79 = HEAP32[$0>>2]|0;
  $80 = ((($79)) + 92|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = HEAP32[$nMem>>2]|0;
  $83 = ($82*40)|0;
  $84 = (_allocSpace($x,$81,$83)|0);
  $85 = HEAP32[$0>>2]|0;
  $86 = ((($85)) + 92|0);
  HEAP32[$86>>2] = $84;
  $87 = HEAP32[$0>>2]|0;
  $88 = ((($87)) + 116|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = HEAP32[$nVar>>2]|0;
  $91 = ($90*40)|0;
  $92 = (_allocSpace($x,$89,$91)|0);
  $93 = HEAP32[$0>>2]|0;
  $94 = ((($93)) + 116|0);
  HEAP32[$94>>2] = $92;
  $95 = HEAP32[$0>>2]|0;
  $96 = ((($95)) + 96|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = HEAP32[$nArg>>2]|0;
  $99 = $98<<2;
  $100 = (_allocSpace($x,$97,$99)|0);
  $101 = HEAP32[$0>>2]|0;
  $102 = ((($101)) + 96|0);
  HEAP32[$102>>2] = $100;
  $103 = HEAP32[$0>>2]|0;
  $104 = ((($103)) + 112|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = HEAP32[$nCursor>>2]|0;
  $107 = $106<<2;
  $108 = (_allocSpace($x,$105,$107)|0);
  $109 = HEAP32[$0>>2]|0;
  $110 = ((($109)) + 112|0);
  HEAP32[$110>>2] = $108;
  $111 = ((($x)) + 8|0);
  $112 = HEAP32[$111>>2]|0;
  $113 = ($112|0)==(0);
  if ($113) {
   break;
  }
  $114 = HEAP32[$db>>2]|0;
  $115 = ((($x)) + 8|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = ($116|0)<(0);
  $118 = $117 << 31 >> 31;
  $119 = (_sqlite3DbMallocRawNN($114,$116,$118)|0);
  $120 = HEAP32[$0>>2]|0;
  $121 = ((($120)) + 180|0);
  HEAP32[$121>>2] = $119;
  HEAP32[$x>>2] = $119;
  $122 = ((($x)) + 8|0);
  $123 = HEAP32[$122>>2]|0;
  $124 = ((($x)) + 4|0);
  HEAP32[$124>>2] = $123;
  $125 = HEAP32[$db>>2]|0;
  $126 = ((($125)) + 69|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = ($127<<24>>24)!=(0);
  $129 = $128 ^ 1;
  if (!($129)) {
   break;
  }
 }
 $130 = HEAP32[$1>>2]|0;
 $131 = ((($130)) + 404|0);
 $132 = HEAP32[$131>>2]|0;
 $133 = $132&65535;
 $134 = HEAP32[$0>>2]|0;
 $135 = ((($134)) + 18|0);
 HEAP16[$135>>1] = $133;
 $136 = HEAP32[$1>>2]|0;
 $137 = ((($136)) + 428|0);
 $138 = HEAP32[$137>>2]|0;
 $139 = HEAP32[$0>>2]|0;
 $140 = ((($139)) + 120|0);
 HEAP32[$140>>2] = $138;
 $141 = HEAP32[$1>>2]|0;
 $142 = ((($141)) + 404|0);
 HEAP32[$142>>2] = 0;
 $143 = HEAP32[$1>>2]|0;
 $144 = ((($143)) + 428|0);
 HEAP32[$144>>2] = 0;
 $145 = HEAP32[$1>>2]|0;
 $146 = ((($145)) + 409|0);
 $147 = HEAP8[$146>>0]|0;
 $148 = $147&255;
 $149 = HEAP32[$0>>2]|0;
 $150 = ((($149)) + 144|0);
 $151 = $148&65535;
 $152 = HEAP16[$150>>1]|0;
 $153 = $151 & 3;
 $154 = ($153 << 2)&65535;
 $155 = $152 & -13;
 $156 = $155 | $154;
 HEAP16[$150>>1] = $156;
 $157 = HEAP32[$db>>2]|0;
 $158 = ((($157)) + 69|0);
 $159 = HEAP8[$158>>0]|0;
 $160 = ($159<<24>>24)!=(0);
 if ($160) {
  $161 = HEAP32[$0>>2]|0;
  $162 = ((($161)) + 16|0);
  HEAP16[$162>>1] = 0;
  $163 = HEAP32[$0>>2]|0;
  $164 = ((($163)) + 28|0);
  HEAP32[$164>>2] = 0;
  $165 = HEAP32[$0>>2]|0;
  $166 = ((($165)) + 24|0);
  HEAP32[$166>>2] = 0;
  $192 = HEAP32[$0>>2]|0;
  _sqlite3VdbeRewind($192);
  STACKTOP = sp;return;
 } else {
  $167 = HEAP32[$nCursor>>2]|0;
  $168 = HEAP32[$0>>2]|0;
  $169 = ((($168)) + 28|0);
  HEAP32[$169>>2] = $167;
  $170 = HEAP32[$nVar>>2]|0;
  $171 = $170&65535;
  $172 = HEAP32[$0>>2]|0;
  $173 = ((($172)) + 16|0);
  HEAP16[$173>>1] = $171;
  $174 = HEAP32[$0>>2]|0;
  $175 = ((($174)) + 116|0);
  $176 = HEAP32[$175>>2]|0;
  $177 = HEAP32[$nVar>>2]|0;
  $178 = HEAP32[$db>>2]|0;
  _initMemArray($176,$177,$178,1);
  $179 = HEAP32[$nMem>>2]|0;
  $180 = HEAP32[$0>>2]|0;
  $181 = ((($180)) + 24|0);
  HEAP32[$181>>2] = $179;
  $182 = HEAP32[$0>>2]|0;
  $183 = ((($182)) + 92|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = HEAP32[$nMem>>2]|0;
  $186 = HEAP32[$db>>2]|0;
  _initMemArray($184,$185,$186,128);
  $187 = HEAP32[$0>>2]|0;
  $188 = ((($187)) + 112|0);
  $189 = HEAP32[$188>>2]|0;
  $190 = HEAP32[$nCursor>>2]|0;
  $191 = $190<<2;
  _memset(($189|0),0,($191|0))|0;
  $192 = HEAP32[$0>>2]|0;
  _sqlite3VdbeRewind($192);
  STACKTOP = sp;return;
 }
}
function _blobSeekToRow($p,$0,$1,$pzErr) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pzErr = $pzErr|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pC = 0, $rc = 0;
 var $type = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp + 24|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 52|0;
 $3 = sp;
 $4 = sp + 48|0;
 $rc = sp + 44|0;
 $zErr = sp + 40|0;
 $v = sp + 36|0;
 $pC = sp + 32|0;
 $type = sp + 28|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pzErr;
 HEAP32[$zErr>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = $3;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$v>>2]|0;
 $19 = ((($18)) + 116|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20;
 $22 = $21;
 HEAP32[$22>>2] = $14;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $17;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 20|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (_sqlite3_step($27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)==(100);
 do {
  if ($30) {
   $31 = HEAP32[$v>>2]|0;
   $32 = ((($31)) + 112|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pC>>2] = $34;
   $35 = HEAP32[$2>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$pC>>2]|0;
   $39 = ((($38)) + 80|0);
   $40 = (($39) + ($37<<2)|0);
   $41 = HEAP32[$40>>2]|0;
   HEAP32[$type>>2] = $41;
   $42 = HEAP32[$type>>2]|0;
   $43 = ($42>>>0)<(12);
   $44 = HEAP32[$2>>2]|0;
   if (!($43)) {
    $59 = ((($44)) + 12|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$pC>>2]|0;
    $62 = ((($61)) + 12|0);
    $63 = HEAP16[$62>>1]|0;
    $64 = $63 << 16 >> 16;
    $65 = (($60) + ($64))|0;
    $66 = HEAP32[$pC>>2]|0;
    $67 = ((($66)) + 80|0);
    $68 = (($67) + ($65<<2)|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = HEAP32[$2>>2]|0;
    $71 = ((($70)) + 8|0);
    HEAP32[$71>>2] = $69;
    $72 = HEAP32[$type>>2]|0;
    $73 = (_sqlite3VdbeSerialTypeLen($72)|0);
    $74 = HEAP32[$2>>2]|0;
    $75 = ((($74)) + 4|0);
    HEAP32[$75>>2] = $73;
    $76 = HEAP32[$pC>>2]|0;
    $77 = ((($76)) + 16|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = HEAP32[$2>>2]|0;
    $80 = ((($79)) + 16|0);
    HEAP32[$80>>2] = $78;
    $81 = HEAP32[$2>>2]|0;
    $82 = ((($81)) + 16|0);
    $83 = HEAP32[$82>>2]|0;
    _sqlite3BtreeIncrblobCursor($83);
    break;
   }
   $45 = ((($44)) + 24|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$type>>2]|0;
   $48 = ($47|0)==(0);
   if ($48) {
    $52 = 39718;
   } else {
    $49 = HEAP32[$type>>2]|0;
    $50 = ($49|0)==(7);
    $51 = $50 ? 39705 : 39710;
    $52 = $51;
   }
   HEAP32[$vararg_buffer>>2] = $52;
   $53 = (_sqlite3MPrintf($46,39723,$vararg_buffer)|0);
   HEAP32[$zErr>>2] = $53;
   HEAP32[$rc>>2] = 1;
   $54 = HEAP32[$2>>2]|0;
   $55 = ((($54)) + 20|0);
   $56 = HEAP32[$55>>2]|0;
   (_sqlite3_finalize($56)|0);
   $57 = HEAP32[$2>>2]|0;
   $58 = ((($57)) + 20|0);
   HEAP32[$58>>2] = 0;
  }
 } while(0);
 $84 = HEAP32[$rc>>2]|0;
 $85 = ($84|0)==(100);
 if ($85) {
  HEAP32[$rc>>2] = 0;
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $86 = HEAP32[$2>>2]|0;
 $87 = ((($86)) + 20|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = ($88|0)!=(0|0);
 if (!($89)) {
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $90 = HEAP32[$2>>2]|0;
 $91 = ((($90)) + 20|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = (_sqlite3_finalize($92)|0);
 HEAP32[$rc>>2] = $93;
 $94 = HEAP32[$2>>2]|0;
 $95 = ((($94)) + 20|0);
 HEAP32[$95>>2] = 0;
 $96 = HEAP32[$rc>>2]|0;
 $97 = ($96|0)==(0);
 $98 = HEAP32[$2>>2]|0;
 $99 = ((($98)) + 24|0);
 $100 = HEAP32[$99>>2]|0;
 if ($97) {
  $101 = $3;
  $102 = $101;
  $103 = HEAP32[$102>>2]|0;
  $104 = (($101) + 4)|0;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = $vararg_buffer1;
  $108 = $107;
  HEAP32[$108>>2] = $103;
  $109 = (($107) + 4)|0;
  $110 = $109;
  HEAP32[$110>>2] = $106;
  $111 = (_sqlite3MPrintf($100,39752,$vararg_buffer1)|0);
  HEAP32[$zErr>>2] = $111;
  HEAP32[$rc>>2] = 1;
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 } else {
  $112 = HEAP32[$2>>2]|0;
  $113 = ((($112)) + 24|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = (_sqlite3_errmsg($114)|0);
  HEAP32[$vararg_buffer4>>2] = $115;
  $116 = (_sqlite3MPrintf($100,18391,$vararg_buffer4)|0);
  HEAP32[$zErr>>2] = $116;
  $117 = HEAP32[$zErr>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  HEAP32[$118>>2] = $117;
  $119 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 return (0)|0;
}
function _sqlite3ParserReset($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 76|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3DbFree($5,$8);
 $9 = HEAP32[$db>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 80|0);
 $12 = HEAP32[$11>>2]|0;
 _sqlite3ExprListDelete($9,$12);
 $13 = HEAP32[$db>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 24|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = HEAP32[$db>>2]|0;
  $20 = ((($19)) + 256|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) - ($18))|0;
  HEAP32[$20>>2] = $22;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 24|0);
 HEAP8[$24>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeData($pCur,$offset,$amt,$pBuf) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $offset;
 HEAP32[$3>>2] = $amt;
 HEAP32[$4>>2] = $pBuf;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 66|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 4;
  $26 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 66|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)>=(3);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (_btreeRestoreCursorPosition($15)|0);
  $17 = $16;
 } else {
  $17 = 0;
 }
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = HEAP32[$4>>2]|0;
  $24 = (_accessPayload($20,$21,$22,$23,0)|0);
  HEAP32[$rc>>2] = $24;
 }
 $25 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _blobReadWrite($pBlob,$z,$n,$iOffset,$xCall) {
 $pBlob = $pBlob|0;
 $z = $z|0;
 $n = $n|0;
 $iOffset = $iOffset|0;
 $xCall = $xCall|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $p = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 $v = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pBlob;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $iOffset;
 HEAP32[$5>>2] = $xCall;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $9 = (_sqlite3MisuseError(84716)|0);
  HEAP32[$0>>2] = $9;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$v>>2] = $15;
 $16 = HEAP32[$3>>2]|0;
 $17 = ($16|0)<(0);
 $18 = HEAP32[$4>>2]|0;
 $19 = ($18|0)<(0);
 $or$cond = $17 | $19;
 do {
  if ($or$cond) {
   label = 5;
  } else {
   $20 = HEAP32[$4>>2]|0;
   $21 = ($20|0)<(0);
   $22 = $21 << 31 >> 31;
   $23 = HEAP32[$3>>2]|0;
   $24 = ($23|0)<(0);
   $25 = $24 << 31 >> 31;
   $26 = (_i64Add(($20|0),($22|0),($23|0),($25|0))|0);
   $27 = tempRet0;
   $28 = HEAP32[$p>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)<(0);
   $32 = $31 << 31 >> 31;
   $33 = ($27|0)>($32|0);
   $34 = ($26>>>0)>($30>>>0);
   $35 = ($27|0)==($32|0);
   $36 = $35 & $34;
   $37 = $33 | $36;
   if ($37) {
    label = 5;
   } else {
    $38 = HEAP32[$v>>2]|0;
    $39 = ($38|0)==(0|0);
    if ($39) {
     HEAP32[$rc>>2] = 4;
     break;
    }
    $40 = HEAP32[$p>>2]|0;
    $41 = ((($40)) + 16|0);
    $42 = HEAP32[$41>>2]|0;
    _sqlite3BtreeEnterCursor($42);
    $43 = HEAP32[$5>>2]|0;
    $44 = HEAP32[$p>>2]|0;
    $45 = ((($44)) + 16|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$4>>2]|0;
    $48 = HEAP32[$p>>2]|0;
    $49 = ((($48)) + 8|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($47) + ($50))|0;
    $52 = HEAP32[$3>>2]|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (FUNCTION_TABLE_iiiii[$43 & 255]($46,$51,$52,$53)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)==(4);
    if ($56) {
     $57 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeFinalize($57)|0);
     $58 = HEAP32[$p>>2]|0;
     $59 = ((($58)) + 20|0);
     HEAP32[$59>>2] = 0;
     break;
    } else {
     $60 = HEAP32[$rc>>2]|0;
     $61 = HEAP32[$v>>2]|0;
     $62 = ((($61)) + 40|0);
     HEAP32[$62>>2] = $60;
     break;
    }
   }
  }
 } while(0);
 if ((label|0) == 5) {
  HEAP32[$rc>>2] = 1;
 }
 $63 = HEAP32[$db>>2]|0;
 $64 = HEAP32[$rc>>2]|0;
 _sqlite3Error($63,$64);
 $65 = HEAP32[$db>>2]|0;
 $66 = HEAP32[$rc>>2]|0;
 $67 = (_sqlite3ApiExit($65,$66)|0);
 HEAP32[$rc>>2] = $67;
 $68 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $68;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _sqlite3ExpirePreparedStatements($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 144|0);
  $8 = HEAP16[$7>>1]|0;
  $9 = $8 & -2;
  $10 = $9 | 1;
  HEAP16[$7>>1] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
 }
 STACKTOP = sp;return;
}
function _patternCompare($zPattern,$zString,$pInfo,$matchOther) {
 $zPattern = $zPattern|0;
 $zString = $zString|0;
 $pInfo = $pInfo|0;
 $matchOther = $matchOther|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c2 = 0, $cx = 0, $invert = 0, $matchAll = 0, $matchOne = 0, $noCase = 0, $or$cond = 0;
 var $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $prior_c = 0, $seen = 0, $zEscaped = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $c = sp + 32|0;
 $c2 = sp + 28|0;
 $matchOne = sp + 24|0;
 $matchAll = sp + 20|0;
 $noCase = sp + 56|0;
 $zEscaped = sp + 16|0;
 $cx = sp + 12|0;
 $prior_c = sp + 8|0;
 $seen = sp + 4|0;
 $invert = sp;
 HEAP32[$1>>2] = $zPattern;
 HEAP32[$2>>2] = $zString;
 HEAP32[$3>>2] = $pInfo;
 HEAP32[$4>>2] = $matchOther;
 $5 = HEAP32[$3>>2]|0;
 $6 = ((($5)) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$matchOne>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$matchAll>>2] = $11;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 3|0);
 $14 = HEAP8[$13>>0]|0;
 HEAP8[$noCase>>0] = $14;
 HEAP32[$zEscaped>>2] = 0;
 L1: while(1) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)<(128);
  if ($18) {
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 1|0);
   HEAP32[$1>>2] = $20;
   $21 = HEAP8[$19>>0]|0;
   $22 = $21&255;
   $24 = $22;
  } else {
   $23 = (_sqlite3Utf8Read($1)|0);
   $24 = $23;
  }
  HEAP32[$c>>2] = $24;
  $25 = ($24|0)!=(0);
  if (!($25)) {
   label = 83;
   break;
  }
  $26 = HEAP32[$c>>2]|0;
  $27 = HEAP32[$matchAll>>2]|0;
  $28 = ($26|0)==($27|0);
  if ($28) {
   label = 7;
   break;
  }
  $145 = HEAP32[$c>>2]|0;
  $146 = HEAP32[$4>>2]|0;
  $147 = ($145|0)==($146|0);
  do {
   if ($147) {
    $148 = HEAP32[$3>>2]|0;
    $149 = ((($148)) + 2|0);
    $150 = HEAP8[$149>>0]|0;
    $151 = $150&255;
    $152 = ($151|0)==(0);
    if ($152) {
     $153 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c>>2] = $153;
     $154 = HEAP32[$c>>2]|0;
     $155 = ($154|0)==(0);
     if ($155) {
      label = 49;
      break L1;
     }
     $156 = HEAP32[$1>>2]|0;
     HEAP32[$zEscaped>>2] = $156;
     break;
    }
    HEAP32[$prior_c>>2] = 0;
    HEAP32[$seen>>2] = 0;
    HEAP32[$invert>>2] = 0;
    $157 = (_sqlite3Utf8Read($2)|0);
    HEAP32[$c>>2] = $157;
    $158 = HEAP32[$c>>2]|0;
    $159 = ($158|0)==(0);
    if ($159) {
     label = 52;
     break L1;
    }
    $160 = (_sqlite3Utf8Read($1)|0);
    HEAP32[$c2>>2] = $160;
    $161 = HEAP32[$c2>>2]|0;
    $162 = ($161|0)==(94);
    if ($162) {
     HEAP32[$invert>>2] = 1;
     $163 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $163;
    }
    $164 = HEAP32[$c2>>2]|0;
    $165 = ($164|0)==(93);
    if ($165) {
     $166 = HEAP32[$c>>2]|0;
     $167 = ($166|0)==(93);
     if ($167) {
      HEAP32[$seen>>2] = 1;
     }
     $168 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $168;
    }
    while(1) {
     $169 = HEAP32[$c2>>2]|0;
     $170 = ($169|0)!=(0);
     $171 = HEAP32[$c2>>2]|0;
     $172 = ($171|0)!=(93);
     $173 = $170 ? $172 : 0;
     $174 = HEAP32[$c2>>2]|0;
     if (!($173)) {
      break;
     }
     $175 = ($174|0)==(45);
     if ($175) {
      $176 = HEAP32[$1>>2]|0;
      $177 = HEAP8[$176>>0]|0;
      $178 = $177&255;
      $179 = ($178|0)!=(93);
      if ($179) {
       $180 = HEAP32[$1>>2]|0;
       $181 = HEAP8[$180>>0]|0;
       $182 = $181&255;
       $183 = ($182|0)!=(0);
       $184 = HEAP32[$prior_c>>2]|0;
       $185 = ($184>>>0)>(0);
       $or$cond = $183 & $185;
       if ($or$cond) {
        $186 = (_sqlite3Utf8Read($1)|0);
        HEAP32[$c2>>2] = $186;
        $187 = HEAP32[$c>>2]|0;
        $188 = HEAP32[$prior_c>>2]|0;
        $189 = ($187>>>0)>=($188>>>0);
        if ($189) {
         $190 = HEAP32[$c>>2]|0;
         $191 = HEAP32[$c2>>2]|0;
         $192 = ($190>>>0)<=($191>>>0);
         if ($192) {
          HEAP32[$seen>>2] = 1;
         }
        }
        HEAP32[$prior_c>>2] = 0;
       } else {
        label = 67;
       }
      } else {
       label = 67;
      }
     } else {
      label = 67;
     }
     if ((label|0) == 67) {
      label = 0;
      $193 = HEAP32[$c>>2]|0;
      $194 = HEAP32[$c2>>2]|0;
      $195 = ($193|0)==($194|0);
      if ($195) {
       HEAP32[$seen>>2] = 1;
      }
      $196 = HEAP32[$c2>>2]|0;
      HEAP32[$prior_c>>2] = $196;
     }
     $197 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c2>>2] = $197;
    }
    $198 = ($174|0)==(0);
    if ($198) {
     label = 73;
     break L1;
    }
    $199 = HEAP32[$seen>>2]|0;
    $200 = HEAP32[$invert>>2]|0;
    $201 = $199 ^ $200;
    $202 = ($201|0)==(0);
    if ($202) {
     label = 73;
     break L1;
    } else {
     continue L1;
    }
   }
  } while(0);
  $203 = HEAP32[$2>>2]|0;
  $204 = HEAP8[$203>>0]|0;
  $205 = $204&255;
  $206 = ($205|0)<(128);
  if ($206) {
   $207 = HEAP32[$2>>2]|0;
   $208 = ((($207)) + 1|0);
   HEAP32[$2>>2] = $208;
   $209 = HEAP8[$207>>0]|0;
   $210 = $209&255;
   $212 = $210;
  } else {
   $211 = (_sqlite3Utf8Read($2)|0);
   $212 = $211;
  }
  HEAP32[$c2>>2] = $212;
  $213 = HEAP32[$c>>2]|0;
  $214 = HEAP32[$c2>>2]|0;
  $215 = ($213|0)==($214|0);
  if ($215) {
   continue;
  }
  $216 = HEAP8[$noCase>>0]|0;
  $217 = $216&255;
  $218 = ($217|0)!=(0);
  if ($218) {
   $219 = HEAP32[$c>>2]|0;
   $220 = $219&255;
   $221 = $220&255;
   $222 = (17613 + ($221)|0);
   $223 = HEAP8[$222>>0]|0;
   $224 = $223&255;
   $225 = HEAP32[$c2>>2]|0;
   $226 = $225&255;
   $227 = $226&255;
   $228 = (17613 + ($227)|0);
   $229 = HEAP8[$228>>0]|0;
   $230 = $229&255;
   $231 = ($224|0)==($230|0);
   $232 = HEAP32[$c>>2]|0;
   $233 = ($232>>>0)<(128);
   $or$cond3 = $231 & $233;
   $234 = HEAP32[$c2>>2]|0;
   $235 = ($234>>>0)<(128);
   $or$cond5 = $or$cond3 & $235;
   if ($or$cond5) {
    continue;
   }
  }
  $236 = HEAP32[$c>>2]|0;
  $237 = HEAP32[$matchOne>>2]|0;
  $238 = ($236|0)==($237|0);
  if (!($238)) {
   label = 82;
   break;
  }
  $239 = HEAP32[$1>>2]|0;
  $240 = HEAP32[$zEscaped>>2]|0;
  $241 = ($239|0)!=($240|0);
  $242 = HEAP32[$c2>>2]|0;
  $243 = ($242|0)!=(0);
  $or$cond7 = $241 & $243;
  if (!($or$cond7)) {
   label = 82;
   break;
  }
 }
 if ((label|0) == 7) {
  while(1) {
   label = 0;
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = ($31|0)<(128);
   if ($32) {
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 1|0);
    HEAP32[$1>>2] = $34;
    $35 = HEAP8[$33>>0]|0;
    $36 = $35&255;
    $38 = $36;
   } else {
    $37 = (_sqlite3Utf8Read($1)|0);
    $38 = $37;
   }
   HEAP32[$c>>2] = $38;
   $39 = HEAP32[$matchAll>>2]|0;
   $40 = ($38|0)==($39|0);
   if ($40) {
    $250 = 1;
   } else {
    $41 = HEAP32[$c>>2]|0;
    $42 = HEAP32[$matchOne>>2]|0;
    $43 = ($41|0)==($42|0);
    $250 = $43;
   }
   $44 = HEAP32[$c>>2]|0;
   if (!($250)) {
    break;
   }
   $45 = HEAP32[$matchOne>>2]|0;
   $46 = ($44|0)==($45|0);
   if (!($46)) {
    label = 7;
    continue;
   }
   $47 = (_sqlite3Utf8Read($2)|0);
   $48 = ($47|0)==(0);
   if ($48) {
    label = 15;
    break;
   } else {
    label = 7;
   }
  }
  if ((label|0) == 15) {
   HEAP32[$0>>2] = 0;
   $249 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($249|0);
  }
  $49 = ($44|0)==(0);
  if ($49) {
   HEAP32[$0>>2] = 1;
   $249 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($249|0);
  }
  $50 = HEAP32[$c>>2]|0;
  $51 = HEAP32[$4>>2]|0;
  $52 = ($50|0)==($51|0);
  do {
   if ($52) {
    $53 = HEAP32[$3>>2]|0;
    $54 = ((($53)) + 2|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = ($56|0)==(0);
    if ($57) {
     $58 = (_sqlite3Utf8Read($1)|0);
     HEAP32[$c>>2] = $58;
     $59 = HEAP32[$c>>2]|0;
     $60 = ($59|0)==(0);
     if (!($60)) {
      break;
     }
     HEAP32[$0>>2] = 0;
     $249 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($249|0);
    }
    L83: while(1) {
     $61 = HEAP32[$2>>2]|0;
     $62 = HEAP8[$61>>0]|0;
     $63 = $62&255;
     $64 = ($63|0)!=(0);
     if ($64) {
      $65 = HEAP32[$1>>2]|0;
      $66 = ((($65)) + -1|0);
      $67 = HEAP32[$2>>2]|0;
      $68 = HEAP32[$3>>2]|0;
      $69 = HEAP32[$4>>2]|0;
      $70 = (_patternCompare($66,$67,$68,$69)|0);
      $71 = ($70|0)==(0);
      $251 = $71;
     } else {
      $251 = 0;
     }
     $72 = HEAP32[$2>>2]|0;
     if (!($251)) {
      break;
     }
     $73 = ((($72)) + 1|0);
     HEAP32[$2>>2] = $73;
     $74 = HEAP8[$72>>0]|0;
     $75 = $74&255;
     $76 = ($75|0)>=(192);
     if (!($76)) {
      continue;
     }
     while(1) {
      $77 = HEAP32[$2>>2]|0;
      $78 = HEAP8[$77>>0]|0;
      $79 = $78&255;
      $80 = $79 & 192;
      $81 = ($80|0)==(128);
      if (!($81)) {
       continue L83;
      }
      $82 = HEAP32[$2>>2]|0;
      $83 = ((($82)) + 1|0);
      HEAP32[$2>>2] = $83;
     }
    }
    $84 = HEAP8[$72>>0]|0;
    $85 = $84&255;
    $86 = ($85|0)!=(0);
    $87 = $86&1;
    HEAP32[$0>>2] = $87;
    $249 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($249|0);
   }
  } while(0);
  $88 = HEAP32[$c>>2]|0;
  $89 = ($88>>>0)<=(128);
  L95: do {
   if ($89) {
    $90 = HEAP8[$noCase>>0]|0;
    $91 = ($90<<24>>24)!=(0);
    $92 = HEAP32[$c>>2]|0;
    if ($91) {
     $93 = HEAP32[$c>>2]|0;
     $94 = $93&255;
     $95 = $94&255;
     $96 = (18403 + ($95)|0);
     $97 = HEAP8[$96>>0]|0;
     $98 = $97&255;
     $99 = $98 & 32;
     $100 = $99 ^ -1;
     $101 = $92 & $100;
     HEAP32[$cx>>2] = $101;
     $102 = HEAP32[$c>>2]|0;
     $103 = $102&255;
     $104 = $103&255;
     $105 = (17613 + ($104)|0);
     $106 = HEAP8[$105>>0]|0;
     $107 = $106&255;
     HEAP32[$c>>2] = $107;
    } else {
     HEAP32[$cx>>2] = $92;
    }
    while(1) {
     $108 = HEAP32[$2>>2]|0;
     $109 = ((($108)) + 1|0);
     HEAP32[$2>>2] = $109;
     $110 = HEAP8[$108>>0]|0;
     $111 = $110&255;
     HEAP32[$c2>>2] = $111;
     $112 = ($111|0)!=(0);
     if (!($112)) {
      break L95;
     }
     $113 = HEAP32[$c2>>2]|0;
     $114 = HEAP32[$c>>2]|0;
     $115 = ($113|0)!=($114|0);
     if ($115) {
      $116 = HEAP32[$c2>>2]|0;
      $117 = HEAP32[$cx>>2]|0;
      $118 = ($116|0)!=($117|0);
      if ($118) {
       continue;
      }
     }
     $119 = HEAP32[$1>>2]|0;
     $120 = HEAP32[$2>>2]|0;
     $121 = HEAP32[$3>>2]|0;
     $122 = HEAP32[$4>>2]|0;
     $123 = (_patternCompare($119,$120,$121,$122)|0);
     $124 = ($123|0)!=(0);
     if ($124) {
      break;
     }
    }
    HEAP32[$0>>2] = 1;
    $249 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($249|0);
   } else {
    while(1) {
     $125 = HEAP32[$2>>2]|0;
     $126 = HEAP8[$125>>0]|0;
     $127 = $126&255;
     $128 = ($127|0)<(128);
     if ($128) {
      $129 = HEAP32[$2>>2]|0;
      $130 = ((($129)) + 1|0);
      HEAP32[$2>>2] = $130;
      $131 = HEAP8[$129>>0]|0;
      $132 = $131&255;
      $134 = $132;
     } else {
      $133 = (_sqlite3Utf8Read($2)|0);
      $134 = $133;
     }
     HEAP32[$c2>>2] = $134;
     $135 = ($134|0)!=(0);
     if (!($135)) {
      break L95;
     }
     $136 = HEAP32[$c2>>2]|0;
     $137 = HEAP32[$c>>2]|0;
     $138 = ($136|0)!=($137|0);
     if ($138) {
      continue;
     }
     $139 = HEAP32[$1>>2]|0;
     $140 = HEAP32[$2>>2]|0;
     $141 = HEAP32[$3>>2]|0;
     $142 = HEAP32[$4>>2]|0;
     $143 = (_patternCompare($139,$140,$141,$142)|0);
     $144 = ($143|0)!=(0);
     if ($144) {
      break;
     }
    }
    HEAP32[$0>>2] = 1;
    $249 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($249|0);
   }
  } while(0);
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 49) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 52) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 73) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 82) {
  HEAP32[$0>>2] = 0;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 else if ((label|0) == 83) {
  $244 = HEAP32[$2>>2]|0;
  $245 = HEAP8[$244>>0]|0;
  $246 = $245&255;
  $247 = ($246|0)==(0);
  $248 = $247&1;
  HEAP32[$0>>2] = $248;
  $249 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($249|0);
 }
 return (0)|0;
}
function _sqlite3SafetyCheckOk($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $magic = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $magic = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 do {
  if ($3) {
   _logBadConnection(39691);
   HEAP32[$0>>2] = 0;
  } else {
   $4 = HEAP32[$1>>2]|0;
   $5 = ((($4)) + 84|0);
   $6 = HEAP32[$5>>2]|0;
   HEAP32[$magic>>2] = $6;
   $7 = HEAP32[$magic>>2]|0;
   $8 = ($7|0)!=(-1607883113);
   if (!($8)) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $9 = HEAP32[$1>>2]|0;
   $10 = (_sqlite3SafetyCheckSickOrOk($9)|0);
   $11 = ($10|0)!=(0);
   if ($11) {
    _logBadConnection(39696);
   }
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3LockAndPrepare($db,$zSql,$nBytes,$saveSqlFlag,$pOld,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $saveSqlFlag = $saveSqlFlag|0;
 $pOld = $pOld|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 8|0;
 $7 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $nBytes;
 HEAP32[$4>>2] = $saveSqlFlag;
 HEAP32[$5>>2] = $pOld;
 HEAP32[$6>>2] = $ppStmt;
 HEAP32[$7>>2] = $pzTail;
 $8 = HEAP32[$6>>2]|0;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3SafetyCheckOk($9)|0);
 $11 = ($10|0)==(0);
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(0|0);
 $or$cond = $11 | $13;
 if ($or$cond) {
  $14 = (_sqlite3MisuseError(114197)|0);
  HEAP32[$0>>2] = $14;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $15 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnterAll($15);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$6>>2]|0;
 $22 = HEAP32[$7>>2]|0;
 $23 = (_sqlite3Prepare($16,$17,$18,$19,$20,$21,$22)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(17);
 if ($25) {
  $26 = HEAP32[$6>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3_finalize($27)|0);
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$4>>2]|0;
  $32 = HEAP32[$5>>2]|0;
  $33 = HEAP32[$6>>2]|0;
  $34 = HEAP32[$7>>2]|0;
  $35 = (_sqlite3Prepare($28,$29,$30,$31,$32,$33,$34)|0);
  HEAP32[$rc>>2] = $35;
 }
 $36 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $36;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3SafetyCheckSickOrOk($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $magic = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $magic = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 84|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$magic>>2] = $4;
 $5 = HEAP32[$magic>>2]|0;
 $6 = ($5|0)!=(1266094736);
 $7 = HEAP32[$magic>>2]|0;
 $8 = ($7|0)!=(-1607883113);
 $or$cond = $6 & $8;
 $9 = HEAP32[$magic>>2]|0;
 $10 = ($9|0)!=(-264537850);
 $or$cond3 = $or$cond & $10;
 if ($or$cond3) {
  _logBadConnection(39538);
  HEAP32[$0>>2] = 0;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 } else {
  HEAP32[$0>>2] = 1;
  $11 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 return (0)|0;
}
function _createModule($db,$zName,$pModule,$pAux,$xDestroy) {
 $db = $db|0;
 $zName = $zName|0;
 $pModule = $pModule|0;
 $pAux = $pAux|0;
 $xDestroy = $xDestroy|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $or$cond = 0, $pDel = 0, $pMod = 0, $rc = 0, $zCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $rc = sp + 16|0;
 $nName = sp + 12|0;
 $pMod = sp + 8|0;
 $pDel = sp + 4|0;
 $zCopy = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pModule;
 HEAP32[$3>>2] = $pAux;
 HEAP32[$4>>2] = $xDestroy;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3Strlen30($5)|0);
 HEAP32[$nName>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 320|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3HashFind($8,$9)|0);
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = (_sqlite3MisuseError(122590)|0);
  HEAP32[$rc>>2] = $12;
 } else {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$nName>>2]|0;
  $15 = (20 + ($14))|0;
  $16 = (($15) + 1)|0;
  $17 = (_sqlite3DbMallocRawNN($13,$16,0)|0);
  HEAP32[$pMod>>2] = $17;
  $18 = HEAP32[$pMod>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pMod>>2]|0;
   $21 = ((($20)) + 20|0);
   HEAP32[$zCopy>>2] = $21;
   $22 = HEAP32[$zCopy>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = HEAP32[$nName>>2]|0;
   $25 = (($24) + 1)|0;
   _memcpy(($22|0),($23|0),($25|0))|0;
   $26 = HEAP32[$zCopy>>2]|0;
   $27 = HEAP32[$pMod>>2]|0;
   $28 = ((($27)) + 4|0);
   HEAP32[$28>>2] = $26;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$pMod>>2]|0;
   HEAP32[$30>>2] = $29;
   $31 = HEAP32[$3>>2]|0;
   $32 = HEAP32[$pMod>>2]|0;
   $33 = ((($32)) + 8|0);
   HEAP32[$33>>2] = $31;
   $34 = HEAP32[$4>>2]|0;
   $35 = HEAP32[$pMod>>2]|0;
   $36 = ((($35)) + 12|0);
   HEAP32[$36>>2] = $34;
   $37 = HEAP32[$pMod>>2]|0;
   $38 = ((($37)) + 16|0);
   HEAP32[$38>>2] = 0;
   $39 = HEAP32[$0>>2]|0;
   $40 = ((($39)) + 320|0);
   $41 = HEAP32[$zCopy>>2]|0;
   $42 = HEAP32[$pMod>>2]|0;
   $43 = (_sqlite3HashInsert($40,$41,$42)|0);
   HEAP32[$pDel>>2] = $43;
   $44 = HEAP32[$pDel>>2]|0;
   $45 = ($44|0)!=(0|0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    _sqlite3OomFault($46);
    $47 = HEAP32[$0>>2]|0;
    $48 = HEAP32[$pDel>>2]|0;
    _sqlite3DbFree($47,$48);
   }
  }
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$rc>>2]|0;
 $51 = (_sqlite3ApiExit($49,$50)|0);
 HEAP32[$rc>>2] = $51;
 $52 = HEAP32[$rc>>2]|0;
 $53 = ($52|0)!=(0);
 $54 = HEAP32[$4>>2]|0;
 $55 = ($54|0)!=(0|0);
 $or$cond = $53 & $55;
 if (!($or$cond)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $56 = HEAP32[$4>>2]|0;
 $57 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$56 & 255]($57);
 $58 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _sqlite3RunParser($pParse,$zSql,$pzErrMsg) {
 $pParse = $pParse|0;
 $zSql = $zSql|0;
 $pzErrMsg = $pzErrMsg|0;
 var $$byval_copy = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $lastTokenParsed = 0;
 var $mxSqlLen = 0, $nErr = 0, $p = 0, $p1 = 0, $pEngine = 0, $tokenType = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $$byval_copy = sp + 72|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $nErr = sp + 52|0;
 $i = sp + 48|0;
 $pEngine = sp + 44|0;
 $tokenType = sp + 40|0;
 $lastTokenParsed = sp + 36|0;
 $db = sp + 32|0;
 $mxSqlLen = sp + 28|0;
 $p = sp + 24|0;
 $p1 = sp + 20|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zSql;
 HEAP32[$3>>2] = $pzErrMsg;
 HEAP32[$nErr>>2] = 0;
 HEAP32[$lastTokenParsed>>2] = -1;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 96|0);
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$mxSqlLen>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 156|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$db>>2]|0;
  $15 = ((($14)) + 248|0);
  HEAP32[$15>>2] = 0;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 12|0);
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 436|0);
 HEAP32[$20>>2] = $18;
 HEAP32[$i>>2] = 0;
 $21 = (_sqlite3ParserAlloc(179)|0);
 HEAP32[$pEngine>>2] = $21;
 $22 = HEAP32[$pEngine>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($24);
  HEAP32[$0>>2] = 7;
  $235 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($235|0);
 }
 L7: while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (($26) + ($25)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($29|0)!=(0);
  do {
   if ($30) {
    $31 = HEAP32[$i>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = (($32) + ($31)|0);
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 392|0);
    HEAP32[$35>>2] = $33;
    $36 = HEAP32[$i>>2]|0;
    $37 = HEAP32[$2>>2]|0;
    $38 = (($37) + ($36)|0);
    $39 = (_sqlite3GetToken($38,$tokenType)|0);
    $40 = HEAP32[$1>>2]|0;
    $41 = ((($40)) + 392|0);
    $42 = ((($41)) + 4|0);
    HEAP32[$42>>2] = $39;
    $43 = HEAP32[$1>>2]|0;
    $44 = ((($43)) + 392|0);
    $45 = ((($44)) + 4|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$i>>2]|0;
    $48 = (($47) + ($46))|0;
    HEAP32[$i>>2] = $48;
    $49 = HEAP32[$i>>2]|0;
    $50 = HEAP32[$mxSqlLen>>2]|0;
    $51 = ($49|0)>($50|0);
    if ($51) {
     label = 7;
     break L7;
    }
   } else {
    $54 = HEAP32[$lastTokenParsed>>2]|0;
    $55 = ($54|0)==(1);
    if ($55) {
     HEAP32[$tokenType>>2] = 0;
     break;
    }
    $56 = HEAP32[$lastTokenParsed>>2]|0;
    $57 = ($56|0)==(0);
    if ($57) {
     break L7;
    }
    HEAP32[$tokenType>>2] = 1;
   }
  } while(0);
  $58 = HEAP32[$tokenType>>2]|0;
  $59 = ($58|0)>=(162);
  if ($59) {
   $60 = HEAP32[$db>>2]|0;
   $61 = ((($60)) + 248|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = ($62|0)!=(0);
   if ($63) {
    label = 14;
    break;
   }
   $66 = HEAP32[$tokenType>>2]|0;
   $67 = ($66|0)==(163);
   if ($67) {
    label = 16;
    break;
   } else {
    continue;
   }
  } else {
   $71 = HEAP32[$pEngine>>2]|0;
   $72 = HEAP32[$tokenType>>2]|0;
   $73 = HEAP32[$1>>2]|0;
   $74 = ((($73)) + 392|0);
   $75 = HEAP32[$1>>2]|0;
   ;HEAP32[$$byval_copy>>2]=HEAP32[$74>>2]|0;HEAP32[$$byval_copy+4>>2]=HEAP32[$74+4>>2]|0;
   _sqlite3Parser($71,$72,$$byval_copy,$75);
   $76 = HEAP32[$tokenType>>2]|0;
   HEAP32[$lastTokenParsed>>2] = $76;
   $77 = HEAP32[$1>>2]|0;
   $78 = ((($77)) + 12|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = ($79|0)!=(0);
   if ($80) {
    break;
   }
   $81 = HEAP32[$db>>2]|0;
   $82 = ((($81)) + 69|0);
   $83 = HEAP8[$82>>0]|0;
   $84 = $83&255;
   $85 = ($84|0)!=(0);
   if ($85) {
    break;
   } else {
    continue;
   }
  }
 }
 if ((label|0) == 7) {
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 12|0);
  HEAP32[$53>>2] = 18;
 }
 else if ((label|0) == 14) {
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 12|0);
  HEAP32[$65>>2] = 9;
 }
 else if ((label|0) == 16) {
  $68 = HEAP32[$1>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 392|0);
  HEAP32[$vararg_buffer>>2] = $70;
  _sqlite3ErrorMsg($68,26437,$vararg_buffer);
 }
 $86 = HEAP32[$i>>2]|0;
 $87 = HEAP32[$2>>2]|0;
 $88 = (($87) + ($86)|0);
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 436|0);
 HEAP32[$90>>2] = $88;
 $91 = HEAP32[$pEngine>>2]|0;
 _sqlite3ParserFree($91,148);
 $92 = HEAP32[$db>>2]|0;
 $93 = ((($92)) + 69|0);
 $94 = HEAP8[$93>>0]|0;
 $95 = ($94<<24>>24)!=(0);
 if ($95) {
  $96 = HEAP32[$1>>2]|0;
  $97 = ((($96)) + 12|0);
  HEAP32[$97>>2] = 7;
 }
 $98 = HEAP32[$1>>2]|0;
 $99 = ((($98)) + 12|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = ($100|0)!=(0);
 if ($101) {
  $102 = HEAP32[$1>>2]|0;
  $103 = ((($102)) + 12|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = ($104|0)!=(101);
  if ($105) {
   $106 = HEAP32[$1>>2]|0;
   $107 = ((($106)) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = ($108|0)==(0|0);
   if ($109) {
    $110 = HEAP32[$db>>2]|0;
    $111 = HEAP32[$1>>2]|0;
    $112 = ((($111)) + 12|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = (_sqlite3ErrStr($113)|0);
    HEAP32[$vararg_buffer1>>2] = $114;
    $115 = (_sqlite3MPrintf($110,18391,$vararg_buffer1)|0);
    $116 = HEAP32[$1>>2]|0;
    $117 = ((($116)) + 4|0);
    HEAP32[$117>>2] = $115;
   }
  }
 }
 $118 = HEAP32[$1>>2]|0;
 $119 = ((($118)) + 4|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = ($120|0)!=(0|0);
 if ($121) {
  $122 = HEAP32[$1>>2]|0;
  $123 = ((($122)) + 4|0);
  $124 = HEAP32[$123>>2]|0;
  $125 = HEAP32[$3>>2]|0;
  HEAP32[$125>>2] = $124;
  $126 = HEAP32[$1>>2]|0;
  $127 = ((($126)) + 12|0);
  $128 = HEAP32[$127>>2]|0;
  $129 = HEAP32[$3>>2]|0;
  $130 = HEAP32[$129>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $130;
  _sqlite3_log($128,18391,$vararg_buffer4);
  $131 = HEAP32[$1>>2]|0;
  $132 = ((($131)) + 4|0);
  HEAP32[$132>>2] = 0;
  $133 = HEAP32[$nErr>>2]|0;
  $134 = (($133) + 1)|0;
  HEAP32[$nErr>>2] = $134;
 }
 $135 = HEAP32[$1>>2]|0;
 $136 = ((($135)) + 8|0);
 $137 = HEAP32[$136>>2]|0;
 $138 = ($137|0)!=(0|0);
 if ($138) {
  $139 = HEAP32[$1>>2]|0;
  $140 = ((($139)) + 36|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ($141|0)>(0);
  if ($142) {
   $143 = HEAP32[$1>>2]|0;
   $144 = ((($143)) + 18|0);
   $145 = HEAP8[$144>>0]|0;
   $146 = $145&255;
   $147 = ($146|0)==(0);
   if ($147) {
    $148 = HEAP32[$1>>2]|0;
    $149 = ((($148)) + 8|0);
    $150 = HEAP32[$149>>2]|0;
    _sqlite3VdbeDelete($150);
    $151 = HEAP32[$1>>2]|0;
    $152 = ((($151)) + 8|0);
    HEAP32[$152>>2] = 0;
   }
  }
 }
 $153 = HEAP32[$1>>2]|0;
 $154 = ((($153)) + 18|0);
 $155 = HEAP8[$154>>0]|0;
 $156 = $155&255;
 $157 = ($156|0)==(0);
 if ($157) {
  $158 = HEAP32[$db>>2]|0;
  $159 = HEAP32[$1>>2]|0;
  $160 = ((($159)) + 116|0);
  $161 = HEAP32[$160>>2]|0;
  _sqlite3DbFree($158,$161);
  $162 = HEAP32[$1>>2]|0;
  $163 = ((($162)) + 116|0);
  HEAP32[$163>>2] = 0;
  $164 = HEAP32[$1>>2]|0;
  $165 = ((($164)) + 112|0);
  HEAP32[$165>>2] = 0;
 }
 $166 = HEAP32[$1>>2]|0;
 $167 = ((($166)) + 460|0);
 $168 = HEAP32[$167>>2]|0;
 _sqlite3_free($168);
 $169 = HEAP32[$1>>2]|0;
 $170 = ((($169)) + 410|0);
 $171 = HEAP8[$170>>0]|0;
 $172 = ($171<<24>>24)!=(0);
 if (!($172)) {
  $173 = HEAP32[$db>>2]|0;
  $174 = HEAP32[$1>>2]|0;
  $175 = ((($174)) + 440|0);
  $176 = HEAP32[$175>>2]|0;
  _sqlite3DeleteTable($173,$176);
 }
 $177 = HEAP32[$1>>2]|0;
 $178 = ((($177)) + 476|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = ($179|0)!=(0|0);
 if ($180) {
  $181 = HEAP32[$db>>2]|0;
  $182 = HEAP32[$1>>2]|0;
  $183 = ((($182)) + 476|0);
  $184 = HEAP32[$183>>2]|0;
  _sqlite3WithDelete($181,$184);
 }
 $185 = HEAP32[$db>>2]|0;
 $186 = HEAP32[$1>>2]|0;
 $187 = ((($186)) + 444|0);
 $188 = HEAP32[$187>>2]|0;
 _sqlite3DeleteTrigger($185,$188);
 $189 = HEAP32[$1>>2]|0;
 $190 = ((($189)) + 404|0);
 $191 = HEAP32[$190>>2]|0;
 $192 = (($191) - 1)|0;
 HEAP32[$i>>2] = $192;
 while(1) {
  $193 = HEAP32[$i>>2]|0;
  $194 = ($193|0)>=(0);
  $195 = HEAP32[$db>>2]|0;
  if (!($194)) {
   break;
  }
  $196 = HEAP32[$i>>2]|0;
  $197 = HEAP32[$1>>2]|0;
  $198 = ((($197)) + 428|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = (($199) + ($196<<2)|0);
  $201 = HEAP32[$200>>2]|0;
  _sqlite3DbFree($195,$201);
  $202 = HEAP32[$i>>2]|0;
  $203 = (($202) + -1)|0;
  HEAP32[$i>>2] = $203;
 }
 $204 = HEAP32[$1>>2]|0;
 $205 = ((($204)) + 428|0);
 $206 = HEAP32[$205>>2]|0;
 _sqlite3DbFree($195,$206);
 while(1) {
  $207 = HEAP32[$1>>2]|0;
  $208 = ((($207)) + 120|0);
  $209 = HEAP32[$208>>2]|0;
  $210 = ($209|0)!=(0|0);
  if (!($210)) {
   break;
  }
  $211 = HEAP32[$1>>2]|0;
  $212 = ((($211)) + 120|0);
  $213 = HEAP32[$212>>2]|0;
  HEAP32[$p>>2] = $213;
  $214 = HEAP32[$p>>2]|0;
  $215 = HEAP32[$214>>2]|0;
  $216 = HEAP32[$1>>2]|0;
  $217 = ((($216)) + 120|0);
  HEAP32[$217>>2] = $215;
  $218 = HEAP32[$db>>2]|0;
  $219 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($218,$219);
 }
 while(1) {
  $220 = HEAP32[$1>>2]|0;
  $221 = ((($220)) + 464|0);
  $222 = HEAP32[$221>>2]|0;
  $223 = ($222|0)!=(0|0);
  if (!($223)) {
   break;
  }
  $224 = HEAP32[$1>>2]|0;
  $225 = ((($224)) + 464|0);
  $226 = HEAP32[$225>>2]|0;
  HEAP32[$p1>>2] = $226;
  $227 = HEAP32[$p1>>2]|0;
  $228 = ((($227)) + 68|0);
  $229 = HEAP32[$228>>2]|0;
  $230 = HEAP32[$1>>2]|0;
  $231 = ((($230)) + 464|0);
  HEAP32[$231>>2] = $229;
  $232 = HEAP32[$db>>2]|0;
  $233 = HEAP32[$p1>>2]|0;
  _sqlite3DeleteTable($232,$233);
 }
 $234 = HEAP32[$nErr>>2]|0;
 HEAP32[$0>>2] = $234;
 $235 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($235|0);
}
function _sqlite3ValueNew($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3DbMallocZero($1,40,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $10 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 32|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3ValueSetStr($v,$n,$z,$enc,$xDel) {
 $v = $v|0;
 $n = $n|0;
 $z = $z|0;
 $enc = $enc|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $4 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $z;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $xDel;
 $5 = HEAP32[$0>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP8[$3>>0]|0;
 $11 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeMemSetStr($7,$8,$9,$10,$11)|0);
 STACKTOP = sp;return;
}
function _sqlite3MemSetDefault() {
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = 4220;
 (_sqlite3_config(4,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _sqlite3HeaderSizeBtree() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 88;
}
function _sqlite3HeaderSizePcache() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 40;
}
function _sqlite3HeaderSizePcache1() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 32;
}
function _sqlite3PCacheSetDefault() {
 var $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = 4168;
 (_sqlite3_config(18,$vararg_buffer)|0);
 STACKTOP = sp;return;
}
function _sqlite3PagerShrink($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 212|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3PcacheShrink($3);
 STACKTOP = sp;return;
}
function _sqlite3BtreeIsInTrans($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _setupLookaside($db,$pBuf,$sz,$cnt) {
 $db = $db|0;
 $pBuf = $pBuf|0;
 $sz = $sz|0;
 $cnt = $cnt|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $or$cond = 0, $p = 0, $pStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pStart = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pBuf;
 HEAP32[$3>>2] = $sz;
 HEAP32[$4>>2] = $cnt;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 256|0);
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 5;
  $103 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($103|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 256|0);
 $12 = ((($11)) + 6|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = ($13<<24>>24)!=(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 256|0);
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3_free($18);
 }
 $19 = HEAP32[$3>>2]|0;
 $20 = $19 & -8;
 HEAP32[$3>>2] = $20;
 $21 = HEAP32[$3>>2]|0;
 $22 = ($21|0)<=(4);
 $$ = $22 ? 0 : $20;
 HEAP32[$3>>2] = $$;
 $23 = HEAP32[$4>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  HEAP32[$4>>2] = 0;
 }
 $25 = HEAP32[$3>>2]|0;
 $26 = ($25|0)==(0);
 $27 = HEAP32[$4>>2]|0;
 $28 = ($27|0)==(0);
 $or$cond = $26 | $28;
 do {
  if ($or$cond) {
   HEAP32[$3>>2] = 0;
   HEAP32[$pStart>>2] = 0;
  } else {
   $29 = HEAP32[$2>>2]|0;
   $30 = ($29|0)==(0|0);
   if (!($30)) {
    $43 = HEAP32[$2>>2]|0;
    HEAP32[$pStart>>2] = $43;
    break;
   }
   _sqlite3BeginBenignMalloc();
   $31 = HEAP32[$3>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = Math_imul($31, $32)|0;
   $34 = ($33|0)<(0);
   $35 = $34 << 31 >> 31;
   $36 = (_sqlite3Malloc($33,$35)|0);
   HEAP32[$pStart>>2] = $36;
   _sqlite3EndBenignMalloc();
   $37 = HEAP32[$pStart>>2]|0;
   $38 = ($37|0)!=(0|0);
   if ($38) {
    $39 = HEAP32[$pStart>>2]|0;
    $40 = (_sqlite3MallocSize($39)|0);
    $41 = HEAP32[$3>>2]|0;
    $42 = (($40|0) / ($41|0))&-1;
    HEAP32[$4>>2] = $42;
   }
  }
 } while(0);
 $44 = HEAP32[$pStart>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 256|0);
 $47 = ((($46)) + 32|0);
 HEAP32[$47>>2] = $44;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 256|0);
 $50 = ((($49)) + 28|0);
 HEAP32[$50>>2] = 0;
 $51 = HEAP32[$3>>2]|0;
 $52 = $51&65535;
 $53 = HEAP32[$1>>2]|0;
 $54 = ((($53)) + 256|0);
 $55 = ((($54)) + 4|0);
 HEAP16[$55>>1] = $52;
 $56 = HEAP32[$pStart>>2]|0;
 $57 = ($56|0)!=(0|0);
 if ($57) {
  $58 = HEAP32[$pStart>>2]|0;
  HEAP32[$p>>2] = $58;
  $59 = HEAP32[$4>>2]|0;
  $60 = (($59) - 1)|0;
  HEAP32[$i>>2] = $60;
  while(1) {
   $61 = HEAP32[$i>>2]|0;
   $62 = ($61|0)>=(0);
   if (!($62)) {
    break;
   }
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 256|0);
   $65 = ((($64)) + 28|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[$p>>2]|0;
   HEAP32[$67>>2] = $66;
   $68 = HEAP32[$p>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 256|0);
   $71 = ((($70)) + 28|0);
   HEAP32[$71>>2] = $68;
   $72 = HEAP32[$3>>2]|0;
   $73 = HEAP32[$p>>2]|0;
   $74 = (($73) + ($72)|0);
   HEAP32[$p>>2] = $74;
   $75 = HEAP32[$i>>2]|0;
   $76 = (($75) + -1)|0;
   HEAP32[$i>>2] = $76;
  }
  $77 = HEAP32[$p>>2]|0;
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 256|0);
  $80 = ((($79)) + 36|0);
  HEAP32[$80>>2] = $77;
  $81 = HEAP32[$1>>2]|0;
  $82 = ((($81)) + 256|0);
  HEAP32[$82>>2] = 0;
  $83 = HEAP32[$2>>2]|0;
  $84 = ($83|0)==(0|0);
  $85 = $84 ? 1 : 0;
  $86 = $85&255;
  $87 = HEAP32[$1>>2]|0;
  $88 = ((($87)) + 256|0);
  $89 = ((($88)) + 6|0);
  HEAP8[$89>>0] = $86;
 } else {
  $90 = HEAP32[$1>>2]|0;
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 256|0);
  $93 = ((($92)) + 32|0);
  HEAP32[$93>>2] = $90;
  $94 = HEAP32[$1>>2]|0;
  $95 = HEAP32[$1>>2]|0;
  $96 = ((($95)) + 256|0);
  $97 = ((($96)) + 36|0);
  HEAP32[$97>>2] = $94;
  $98 = HEAP32[$1>>2]|0;
  $99 = ((($98)) + 256|0);
  HEAP32[$99>>2] = 1;
  $100 = HEAP32[$1>>2]|0;
  $101 = ((($100)) + 256|0);
  $102 = ((($101)) + 6|0);
  HEAP8[$102>>0] = 0;
 }
 HEAP32[$0>>2] = 0;
 $103 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($103|0);
}
function _sqlite3Close($db,$forceZombie) {
 $db = $db|0;
 $forceZombie = $forceZombie|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $forceZombie;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  HEAP32[$0>>2] = 0;
  $33 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3SafetyCheckSickOrOk($5)|0);
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $8 = (_sqlite3MisuseError(138867)|0);
  HEAP32[$0>>2] = $8;
  $33 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 76|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 8;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 184|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 188|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  (FUNCTION_TABLE_iiiii[$17 & 255](8,$20,$21,0)|0);
 }
 $22 = HEAP32[$1>>2]|0;
 _disconnectAllVtab($22);
 $23 = HEAP32[$1>>2]|0;
 (_sqlite3VtabRollback($23)|0);
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)!=(0);
 if (!($25)) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (_connectionIsBusy($26)|0);
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$1>>2]|0;
   _sqlite3ErrorWithMsg($29,5,26295,$vararg_buffer);
   HEAP32[$0>>2] = 5;
   $33 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($33|0);
  }
 }
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 84|0);
 HEAP32[$31>>2] = 1691352191;
 $32 = HEAP32[$1>>2]|0;
 _sqlite3LeaveMutexAndCloseZombie($32);
 HEAP32[$0>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqliteDefaultBusyCallback($ptr,$count) {
 $ptr = $ptr|0;
 $count = $count|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $timeout = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 $timeout = sp;
 HEAP32[$1>>2] = $ptr;
 HEAP32[$2>>2] = $count;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 428|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$timeout>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = (($7) + 1)|0;
 $9 = ($8*1000)|0;
 $10 = HEAP32[$timeout>>2]|0;
 $11 = ($9|0)>($10|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 } else {
  $12 = HEAP32[$db>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3OsSleep($13,1000000)|0);
  HEAP32[$0>>2] = 1;
  $14 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($14|0);
 }
 return (0)|0;
}
function _sqlite3CreateFunc($db,$zFunctionName,$nArg,$enc,$pUserData,$xSFunc,$xStep,$xFinal,$pDestructor) {
 $db = $db|0;
 $zFunctionName = $zFunctionName|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $pUserData = $pUserData|0;
 $xSFunc = $xSFunc|0;
 $xStep = $xStep|0;
 $xFinal = $xFinal|0;
 $pDestructor = $pDestructor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $extraFlags = 0, $nName = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond3 = 0, $or$cond3$not = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond7$not = 0, $or$cond9 = 0, $p = 0;
 var $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $8 = sp + 24|0;
 $9 = sp + 20|0;
 $p = sp + 16|0;
 $nName = sp + 12|0;
 $extraFlags = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zFunctionName;
 HEAP32[$3>>2] = $nArg;
 HEAP32[$4>>2] = $enc;
 HEAP32[$5>>2] = $pUserData;
 HEAP32[$6>>2] = $xSFunc;
 HEAP32[$7>>2] = $xStep;
 HEAP32[$8>>2] = $xFinal;
 HEAP32[$9>>2] = $pDestructor;
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 do {
  if (!($11)) {
   $12 = HEAP32[$6>>2]|0;
   $13 = ($12|0)!=(0|0);
   if ($13) {
    $14 = HEAP32[$8>>2]|0;
    $15 = ($14|0)!=(0|0);
    $16 = HEAP32[$7>>2]|0;
    $17 = ($16|0)!=(0|0);
    $or$cond = $15 | $17;
    if ($or$cond) {
     break;
    }
   }
   $18 = HEAP32[$6>>2]|0;
   $19 = ($18|0)==(0|0);
   $20 = HEAP32[$8>>2]|0;
   $21 = ($20|0)!=(0|0);
   $or$cond3 = $19 & $21;
   $or$cond3$not = $or$cond3 ^ 1;
   $22 = HEAP32[$7>>2]|0;
   $23 = ($22|0)!=(0|0);
   $or$cond5 = $or$cond3$not | $23;
   if ($or$cond5) {
    $24 = HEAP32[$6>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$8>>2]|0;
    $27 = ($26|0)!=(0|0);
    $or$cond7 = $25 | $27;
    $or$cond7$not = $or$cond7 ^ 1;
    $28 = HEAP32[$7>>2]|0;
    $29 = ($28|0)!=(0|0);
    $or$cond9 = $or$cond7$not & $29;
    $30 = HEAP32[$3>>2]|0;
    $31 = ($30|0)<(-1);
    $or$cond11 = $or$cond9 | $31;
    $32 = HEAP32[$3>>2]|0;
    $33 = ($32|0)>(127);
    $or$cond13 = $or$cond11 | $33;
    if (!($or$cond13)) {
     $34 = HEAP32[$2>>2]|0;
     $35 = (_sqlite3Strlen30($34)|0);
     HEAP32[$nName>>2] = $35;
     $36 = (255)<($35|0);
     if (!($36)) {
      $38 = HEAP32[$4>>2]|0;
      $39 = $38 & 2048;
      HEAP32[$extraFlags>>2] = $39;
      $40 = HEAP32[$4>>2]|0;
      $41 = $40 & 7;
      HEAP32[$4>>2] = $41;
      $42 = HEAP32[$4>>2]|0;
      $43 = ($42|0)==(4);
      do {
       if ($43) {
        $44 = HEAP8[968>>0]|0;
        $45 = $44 << 24 >> 24;
        $46 = ($45|0)==(0);
        $47 = $46 ? 3 : 2;
        HEAP32[$4>>2] = $47;
       } else {
        $48 = HEAP32[$4>>2]|0;
        $49 = ($48|0)==(5);
        if ($49) {
         $50 = HEAP32[$1>>2]|0;
         $51 = HEAP32[$2>>2]|0;
         $52 = HEAP32[$3>>2]|0;
         $53 = HEAP32[$extraFlags>>2]|0;
         $54 = 1 | $53;
         $55 = HEAP32[$5>>2]|0;
         $56 = HEAP32[$6>>2]|0;
         $57 = HEAP32[$7>>2]|0;
         $58 = HEAP32[$8>>2]|0;
         $59 = HEAP32[$9>>2]|0;
         $60 = (_sqlite3CreateFunc($50,$51,$52,$54,$55,$56,$57,$58,$59)|0);
         HEAP32[$rc>>2] = $60;
         $61 = HEAP32[$rc>>2]|0;
         $62 = ($61|0)==(0);
         if ($62) {
          $63 = HEAP32[$1>>2]|0;
          $64 = HEAP32[$2>>2]|0;
          $65 = HEAP32[$3>>2]|0;
          $66 = HEAP32[$extraFlags>>2]|0;
          $67 = 2 | $66;
          $68 = HEAP32[$5>>2]|0;
          $69 = HEAP32[$6>>2]|0;
          $70 = HEAP32[$7>>2]|0;
          $71 = HEAP32[$8>>2]|0;
          $72 = HEAP32[$9>>2]|0;
          $73 = (_sqlite3CreateFunc($63,$64,$65,$67,$68,$69,$70,$71,$72)|0);
          HEAP32[$rc>>2] = $73;
         }
         $74 = HEAP32[$rc>>2]|0;
         $75 = ($74|0)!=(0);
         if (!($75)) {
          HEAP32[$4>>2] = 3;
          break;
         }
         $76 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $76;
         $147 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($147|0);
        }
       }
      } while(0);
      $77 = HEAP32[$1>>2]|0;
      $78 = HEAP32[$2>>2]|0;
      $79 = HEAP32[$3>>2]|0;
      $80 = HEAP32[$4>>2]|0;
      $81 = $80&255;
      $82 = (_sqlite3FindFunction($77,$78,$79,$81,0)|0);
      HEAP32[$p>>2] = $82;
      $83 = HEAP32[$p>>2]|0;
      $84 = ($83|0)!=(0|0);
      do {
       if ($84) {
        $85 = HEAP32[$p>>2]|0;
        $86 = ((($85)) + 2|0);
        $87 = HEAP16[$86>>1]|0;
        $88 = $87&65535;
        $89 = $88 & 3;
        $90 = HEAP32[$4>>2]|0;
        $91 = ($89|0)==($90|0);
        if ($91) {
         $92 = HEAP32[$p>>2]|0;
         $93 = HEAP8[$92>>0]|0;
         $94 = $93 << 24 >> 24;
         $95 = HEAP32[$3>>2]|0;
         $96 = ($94|0)==($95|0);
         if ($96) {
          $97 = HEAP32[$1>>2]|0;
          $98 = ((($97)) + 156|0);
          $99 = HEAP32[$98>>2]|0;
          $100 = ($99|0)!=(0);
          $101 = HEAP32[$1>>2]|0;
          if (!($100)) {
           _sqlite3ExpirePreparedStatements($101);
           break;
          }
          _sqlite3ErrorWithMsg($101,5,26232,$vararg_buffer);
          HEAP32[$0>>2] = 5;
          $147 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($147|0);
         }
        }
       }
      } while(0);
      $102 = HEAP32[$1>>2]|0;
      $103 = HEAP32[$2>>2]|0;
      $104 = HEAP32[$3>>2]|0;
      $105 = HEAP32[$4>>2]|0;
      $106 = $105&255;
      $107 = (_sqlite3FindFunction($102,$103,$104,$106,1)|0);
      HEAP32[$p>>2] = $107;
      $108 = HEAP32[$p>>2]|0;
      $109 = ($108|0)!=(0|0);
      if (!($109)) {
       HEAP32[$0>>2] = 7;
       $147 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($147|0);
      }
      $110 = HEAP32[$1>>2]|0;
      $111 = HEAP32[$p>>2]|0;
      _functionDestroy($110,$111);
      $112 = HEAP32[$9>>2]|0;
      $113 = ($112|0)!=(0|0);
      if ($113) {
       $114 = HEAP32[$9>>2]|0;
       $115 = HEAP32[$114>>2]|0;
       $116 = (($115) + 1)|0;
       HEAP32[$114>>2] = $116;
      }
      $117 = HEAP32[$9>>2]|0;
      $118 = HEAP32[$p>>2]|0;
      $119 = ((($118)) + 24|0);
      HEAP32[$119>>2] = $117;
      $120 = HEAP32[$p>>2]|0;
      $121 = ((($120)) + 2|0);
      $122 = HEAP16[$121>>1]|0;
      $123 = $122&65535;
      $124 = $123 & 3;
      $125 = HEAP32[$extraFlags>>2]|0;
      $126 = $124 | $125;
      $127 = $126&65535;
      $128 = HEAP32[$p>>2]|0;
      $129 = ((($128)) + 2|0);
      HEAP16[$129>>1] = $127;
      $130 = HEAP32[$6>>2]|0;
      $131 = ($130|0)!=(0|0);
      $132 = HEAP32[$6>>2]|0;
      $133 = HEAP32[$7>>2]|0;
      $134 = $131 ? $132 : $133;
      $135 = HEAP32[$p>>2]|0;
      $136 = ((($135)) + 12|0);
      HEAP32[$136>>2] = $134;
      $137 = HEAP32[$8>>2]|0;
      $138 = HEAP32[$p>>2]|0;
      $139 = ((($138)) + 16|0);
      HEAP32[$139>>2] = $137;
      $140 = HEAP32[$5>>2]|0;
      $141 = HEAP32[$p>>2]|0;
      $142 = ((($141)) + 4|0);
      HEAP32[$142>>2] = $140;
      $143 = HEAP32[$3>>2]|0;
      $144 = $143&65535;
      $145 = $144&255;
      $146 = HEAP32[$p>>2]|0;
      HEAP8[$146>>0] = $145;
      HEAP32[$0>>2] = 0;
      $147 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($147|0);
     }
    }
   }
  }
 } while(0);
 $37 = (_sqlite3MisuseError(139436)|0);
 HEAP32[$0>>2] = $37;
 $147 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($147|0);
}
function _sqlite3FindFunction($db,$zName,$nArg,$enc,$createFlag) {
 $db = $db|0;
 $zName = $zName|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bestScore = 0, $h = 0, $nName = 0, $or$cond = 0;
 var $p = 0, $pBest = 0, $pOther = 0, $score = 0, $score1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 49|0;
 $5 = sp + 48|0;
 $p = sp + 28|0;
 $pBest = sp + 24|0;
 $bestScore = sp + 20|0;
 $h = sp + 16|0;
 $nName = sp + 12|0;
 $score = sp + 8|0;
 $score1 = sp + 4|0;
 $pOther = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $nArg;
 HEAP8[$4>>0] = $enc;
 HEAP8[$5>>0] = $createFlag;
 HEAP32[$pBest>>2] = 0;
 HEAP32[$bestScore>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3Strlen30($6)|0);
 HEAP32[$nName>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 348|0);
 $10 = HEAP32[$2>>2]|0;
 $11 = (_sqlite3HashFind($9,$10)|0);
 HEAP32[$p>>2] = $11;
 while(1) {
  $12 = HEAP32[$p>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$p>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = HEAP8[$4>>0]|0;
  $17 = (_matchQuality($14,$15,$16)|0);
  HEAP32[$score>>2] = $17;
  $18 = HEAP32[$score>>2]|0;
  $19 = HEAP32[$bestScore>>2]|0;
  $20 = ($18|0)>($19|0);
  if ($20) {
   $21 = HEAP32[$p>>2]|0;
   HEAP32[$pBest>>2] = $21;
   $22 = HEAP32[$score>>2]|0;
   HEAP32[$bestScore>>2] = $22;
  }
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$p>>2] = $25;
 }
 $26 = HEAP8[$5>>0]|0;
 $27 = ($26<<24>>24)!=(0);
 L8: do {
  if (!($27)) {
   $28 = HEAP32[$pBest>>2]|0;
   $29 = ($28|0)==(0|0);
   if (!($29)) {
    $30 = HEAP32[$1>>2]|0;
    $31 = ((($30)) + 24|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 & 2097152;
    $34 = ($33|0)!=(0);
    if (!($34)) {
     break;
    }
   }
   HEAP32[$bestScore>>2] = 0;
   $35 = HEAP32[$2>>2]|0;
   $36 = HEAP8[$35>>0]|0;
   $37 = $36&255;
   $38 = (17613 + ($37)|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = HEAP32[$nName>>2]|0;
   $42 = (($40) + ($41))|0;
   $43 = (($42|0) % 23)&-1;
   HEAP32[$h>>2] = $43;
   $44 = HEAP32[$h>>2]|0;
   $45 = HEAP32[$2>>2]|0;
   $46 = (_functionSearch($44,$45)|0);
   HEAP32[$p>>2] = $46;
   while(1) {
    $47 = HEAP32[$p>>2]|0;
    $48 = ($47|0)!=(0|0);
    if (!($48)) {
     break L8;
    }
    $49 = HEAP32[$p>>2]|0;
    $50 = HEAP32[$3>>2]|0;
    $51 = HEAP8[$4>>0]|0;
    $52 = (_matchQuality($49,$50,$51)|0);
    HEAP32[$score1>>2] = $52;
    $53 = HEAP32[$score1>>2]|0;
    $54 = HEAP32[$bestScore>>2]|0;
    $55 = ($53|0)>($54|0);
    if ($55) {
     $56 = HEAP32[$p>>2]|0;
     HEAP32[$pBest>>2] = $56;
     $57 = HEAP32[$score1>>2]|0;
     HEAP32[$bestScore>>2] = $57;
    }
    $58 = HEAP32[$p>>2]|0;
    $59 = ((($58)) + 8|0);
    $60 = HEAP32[$59>>2]|0;
    HEAP32[$p>>2] = $60;
   }
  }
 } while(0);
 $61 = HEAP8[$5>>0]|0;
 $62 = $61&255;
 $63 = ($62|0)!=(0);
 $64 = HEAP32[$bestScore>>2]|0;
 $65 = ($64|0)<(6);
 $or$cond = $63 & $65;
 do {
  if ($or$cond) {
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$nName>>2]|0;
   $68 = (28 + ($67))|0;
   $69 = (($68) + 1)|0;
   $70 = (_sqlite3DbMallocZero($66,$69,0)|0);
   HEAP32[$pBest>>2] = $70;
   $71 = ($70|0)!=(0|0);
   if ($71) {
    $72 = HEAP32[$pBest>>2]|0;
    $73 = ((($72)) + 28|0);
    $74 = HEAP32[$pBest>>2]|0;
    $75 = ((($74)) + 20|0);
    HEAP32[$75>>2] = $73;
    $76 = HEAP32[$3>>2]|0;
    $77 = $76&65535;
    $78 = $77&255;
    $79 = HEAP32[$pBest>>2]|0;
    HEAP8[$79>>0] = $78;
    $80 = HEAP8[$4>>0]|0;
    $81 = $80&255;
    $82 = HEAP32[$pBest>>2]|0;
    $83 = ((($82)) + 2|0);
    HEAP16[$83>>1] = $81;
    $84 = HEAP32[$pBest>>2]|0;
    $85 = ((($84)) + 28|0);
    $86 = HEAP32[$2>>2]|0;
    $87 = HEAP32[$nName>>2]|0;
    $88 = (($87) + 1)|0;
    _memcpy(($85|0),($86|0),($88|0))|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 348|0);
    $91 = HEAP32[$pBest>>2]|0;
    $92 = ((($91)) + 20|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = HEAP32[$pBest>>2]|0;
    $95 = (_sqlite3HashInsert($90,$93,$94)|0);
    HEAP32[$pOther>>2] = $95;
    $96 = HEAP32[$pOther>>2]|0;
    $97 = HEAP32[$pBest>>2]|0;
    $98 = ($96|0)==($97|0);
    if (!($98)) {
     $102 = HEAP32[$pOther>>2]|0;
     $103 = HEAP32[$pBest>>2]|0;
     $104 = ((($103)) + 8|0);
     HEAP32[$104>>2] = $102;
     break;
    }
    $99 = HEAP32[$1>>2]|0;
    $100 = HEAP32[$pBest>>2]|0;
    _sqlite3DbFree($99,$100);
    $101 = HEAP32[$1>>2]|0;
    _sqlite3OomFault($101);
    HEAP32[$0>>2] = 0;
    $115 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($115|0);
   }
  }
 } while(0);
 $105 = HEAP32[$pBest>>2]|0;
 $106 = ($105|0)!=(0|0);
 do {
  if ($106) {
   $107 = HEAP32[$pBest>>2]|0;
   $108 = ((($107)) + 12|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = ($109|0)!=(0|0);
   if (!($110)) {
    $111 = HEAP8[$5>>0]|0;
    $112 = $111&255;
    $113 = ($112|0)!=(0);
    if (!($113)) {
     break;
    }
   }
   $114 = HEAP32[$pBest>>2]|0;
   HEAP32[$0>>2] = $114;
   $115 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($115|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $115 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($115|0);
}
function _sqlite3InvalidFunction($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zName = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$zName>>2] = $7;
 $8 = HEAP32[$zName>>2]|0;
 HEAP32[$vararg_buffer>>2] = $8;
 $9 = (_sqlite3_mprintf(26181,$vararg_buffer)|0);
 HEAP32[$zErr>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$zErr>>2]|0;
 _sqlite3_result_error($10,$11,-1);
 $12 = HEAP32[$zErr>>2]|0;
 _sqlite3_free($12);
 STACKTOP = sp;return;
}
function _sqlite3WalDefaultHook($pClientData,$db,$zDb,$nFrame) {
 $pClientData = $pClientData|0;
 $db = $db|0;
 $zDb = $zDb|0;
 $nFrame = $nFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pClientData;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $nFrame;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = $5;
 $7 = ($4|0)>=($6|0);
 if (!($7)) {
  STACKTOP = sp;return 0;
 }
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3_wal_checkpoint($8,$9)|0);
 _sqlite3EndBenignMalloc();
 STACKTOP = sp;return 0;
}
function _sqlite3FindDbName($db,$zName) {
 $db = $db|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$i>>2] = -1;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $24 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 20|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) - 1)|0;
 HEAP32[$i>>2] = $7;
 $8 = HEAP32[$i>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + ($8<<4)|0);
 HEAP32[$pDb>>2] = $12;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = ($13|0)>=(0);
  if (!($14)) {
   label = 6;
   break;
  }
  $15 = HEAP32[$pDb>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = (_sqlite3StrICmp($16,$17)|0);
  $19 = (0)==($18|0);
  if ($19) {
   label = 6;
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$i>>2] = $21;
  $22 = HEAP32[$pDb>>2]|0;
  $23 = ((($22)) + -16|0);
  HEAP32[$pDb>>2] = $23;
 }
 if ((label|0) == 6) {
  $24 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3Checkpoint($db,$iDb,$eMode,$pnLog,$pnCkpt) {
 $db = $db|0;
 $iDb = $iDb|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bBusy = 0;
 var $i = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $bBusy = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $pnLog;
 HEAP32[$4>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bBusy>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(0);
  $12 = $9 ? $11 : 0;
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ($13|0)==($14|0);
  $16 = HEAP32[$1>>2]|0;
  $17 = ($16|0)==(10);
  $or$cond = $15 | $17;
  if ($or$cond) {
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18<<4)|0);
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$2>>2]|0;
   $26 = HEAP32[$3>>2]|0;
   $27 = HEAP32[$4>>2]|0;
   $28 = (_sqlite3BtreeCheckpoint($24,$25,$26,$27)|0);
   HEAP32[$rc>>2] = $28;
   HEAP32[$3>>2] = 0;
   HEAP32[$4>>2] = 0;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(5);
   if ($30) {
    HEAP32[$bBusy>>2] = 1;
    HEAP32[$rc>>2] = 0;
   }
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
 }
 $33 = HEAP32[$rc>>2]|0;
 $34 = ($33|0)==(0);
 $35 = HEAP32[$bBusy>>2]|0;
 $36 = ($35|0)!=(0);
 $or$cond3 = $34 & $36;
 $37 = HEAP32[$rc>>2]|0;
 $38 = $or$cond3 ? 5 : $37;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3OomClear($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 69|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 168|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 69|0);
 HEAP8[$11>>0] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 248|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 256|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + -1)|0;
 HEAP32[$15>>2] = $17;
 STACKTOP = sp;return;
}
function _openDatabase($zFilename,$ppDb,$flags,$zVfs) {
 $zFilename = $zFilename|0;
 $ppDb = $ppDb|0;
 $flags = $flags|0;
 $zVfs = $zVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $db = 0, $isThreadsafe = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, $zErrMsg = 0, $zOpen = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $db = sp + 20|0;
 $rc = sp + 16|0;
 $isThreadsafe = sp + 12|0;
 $zOpen = sp + 8|0;
 $zErrMsg = sp + 4|0;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $ppDb;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $zVfs;
 HEAP32[$zOpen>>2] = 0;
 HEAP32[$zErrMsg>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = (_sqlite3_initialize()|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $239 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($239|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = $10 & 7;
 $12 = 1 << $11;
 $13 = $12 & 70;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = (_sqlite3MisuseError(140621)|0);
  HEAP32[$0>>2] = $15;
  $239 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($239|0);
 }
 $16 = HEAP32[(12)>>2]|0;
 $17 = ($16|0)==(0);
 do {
  if ($17) {
   HEAP32[$isThreadsafe>>2] = 0;
  } else {
   $18 = HEAP32[$3>>2]|0;
   $19 = $18 & 32768;
   $20 = ($19|0)!=(0);
   if ($20) {
    HEAP32[$isThreadsafe>>2] = 0;
    break;
   }
   $21 = HEAP32[$3>>2]|0;
   $22 = $21 & 65536;
   $23 = ($22|0)!=(0);
   if ($23) {
    HEAP32[$isThreadsafe>>2] = 1;
    break;
   } else {
    $24 = HEAP32[(16)>>2]|0;
    HEAP32[$isThreadsafe>>2] = $24;
    break;
   }
  }
 } while(0);
 $25 = HEAP32[$3>>2]|0;
 $26 = $25 & 262144;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$3>>2]|0;
  $29 = $28 & -131073;
  HEAP32[$3>>2] = $29;
 } else {
  $30 = HEAP32[(228)>>2]|0;
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$3>>2]|0;
   $33 = $32 | 131072;
   HEAP32[$3>>2] = $33;
  }
 }
 $34 = HEAP32[$3>>2]|0;
 $35 = $34 & -655129;
 HEAP32[$3>>2] = $35;
 $36 = (_sqlite3MallocZero(464,0)|0);
 HEAP32[$db>>2] = $36;
 $37 = HEAP32[$db>>2]|0;
 $38 = ($37|0)==(0|0);
 do {
  if (!($38)) {
   $39 = HEAP32[$isThreadsafe>>2]|0;
   $40 = ($39|0)!=(0);
   if ($40) {
    $41 = HEAP32[$db>>2]|0;
    $42 = ((($41)) + 12|0);
    HEAP32[$42>>2] = (8);
    $43 = HEAP32[$db>>2]|0;
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)==(0|0);
    if ($46) {
     $47 = HEAP32[$db>>2]|0;
     _sqlite3_free($47);
     HEAP32[$db>>2] = 0;
     break;
    }
   }
   $48 = HEAP32[$db>>2]|0;
   $49 = ((($48)) + 56|0);
   HEAP32[$49>>2] = 255;
   $50 = HEAP32[$db>>2]|0;
   $51 = ((($50)) + 20|0);
   HEAP32[$51>>2] = 2;
   $52 = HEAP32[$db>>2]|0;
   $53 = ((($52)) + 84|0);
   HEAP32[$53>>2] = -264537850;
   $54 = HEAP32[$db>>2]|0;
   $55 = ((($54)) + 392|0);
   $56 = HEAP32[$db>>2]|0;
   $57 = ((($56)) + 16|0);
   HEAP32[$57>>2] = $55;
   $58 = HEAP32[$db>>2]|0;
   $59 = ((($58)) + 96|0);
   dest=$59; src=1580; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   $60 = HEAP32[$db>>2]|0;
   $61 = ((($60)) + 96|0);
   $62 = ((($61)) + 44|0);
   HEAP32[$62>>2] = 0;
   $63 = HEAP32[$db>>2]|0;
   $64 = ((($63)) + 67|0);
   HEAP8[$64>>0] = 1;
   $65 = HEAP32[$db>>2]|0;
   $66 = ((($65)) + 72|0);
   HEAP8[$66>>0] = -1;
   $67 = (184);
   $68 = $67;
   $69 = HEAP32[$68>>2]|0;
   $70 = (($67) + 4)|0;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$db>>2]|0;
   $74 = ((($73)) + 40|0);
   $75 = $74;
   $76 = $75;
   HEAP32[$76>>2] = $69;
   $77 = (($75) + 4)|0;
   $78 = $77;
   HEAP32[$78>>2] = $72;
   $79 = HEAP32[$db>>2]|0;
   $80 = ((($79)) + 80|0);
   HEAP32[$80>>2] = 0;
   $81 = HEAP32[$db>>2]|0;
   $82 = ((($81)) + 144|0);
   HEAP32[$82>>2] = 2147483647;
   $83 = HEAP32[$db>>2]|0;
   $84 = ((($83)) + 24|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = $85 | 17825888;
   HEAP32[$84>>2] = $86;
   $87 = HEAP32[$db>>2]|0;
   $88 = ((($87)) + 364|0);
   _sqlite3HashInit($88);
   $89 = HEAP32[$db>>2]|0;
   $90 = ((($89)) + 320|0);
   _sqlite3HashInit($90);
   $91 = HEAP32[$db>>2]|0;
   (_createCollation($91,34834,1,0,141,0)|0);
   $92 = HEAP32[$db>>2]|0;
   (_createCollation($92,34834,3,0,141,0)|0);
   $93 = HEAP32[$db>>2]|0;
   (_createCollation($93,34834,2,0,141,0)|0);
   $94 = HEAP32[$db>>2]|0;
   (_createCollation($94,20429,1,0,142,0)|0);
   $95 = HEAP32[$db>>2]|0;
   (_createCollation($95,20436,1,(1),141,0)|0);
   $96 = HEAP32[$db>>2]|0;
   $97 = ((($96)) + 69|0);
   $98 = HEAP8[$97>>0]|0;
   $99 = ($98<<24>>24)!=(0);
   if (!($99)) {
    $100 = HEAP32[$db>>2]|0;
    $101 = (_sqlite3FindCollSeq($100,1,34834,0)|0);
    $102 = HEAP32[$db>>2]|0;
    $103 = ((($102)) + 8|0);
    HEAP32[$103>>2] = $101;
    $104 = HEAP32[$3>>2]|0;
    $105 = HEAP32[$db>>2]|0;
    $106 = ((($105)) + 48|0);
    HEAP32[$106>>2] = $104;
    $107 = HEAP32[$4>>2]|0;
    $108 = HEAP32[$1>>2]|0;
    $109 = HEAP32[$db>>2]|0;
    $110 = (_sqlite3ParseUri($107,$108,$3,$109,$zOpen,$zErrMsg)|0);
    HEAP32[$rc>>2] = $110;
    $111 = HEAP32[$rc>>2]|0;
    $112 = ($111|0)!=(0);
    if ($112) {
     $113 = HEAP32[$rc>>2]|0;
     $114 = ($113|0)==(7);
     if ($114) {
      $115 = HEAP32[$db>>2]|0;
      _sqlite3OomFault($115);
     }
     $116 = HEAP32[$db>>2]|0;
     $117 = HEAP32[$rc>>2]|0;
     $118 = HEAP32[$zErrMsg>>2]|0;
     $119 = ($118|0)!=(0|0);
     $120 = $119 ? 18391 : 0;
     $121 = HEAP32[$zErrMsg>>2]|0;
     HEAP32[$vararg_buffer>>2] = $121;
     _sqlite3ErrorWithMsg($116,$117,$120,$vararg_buffer);
     $122 = HEAP32[$zErrMsg>>2]|0;
     _sqlite3_free($122);
     break;
    }
    $123 = HEAP32[$db>>2]|0;
    $124 = HEAP32[$123>>2]|0;
    $125 = HEAP32[$zOpen>>2]|0;
    $126 = HEAP32[$db>>2]|0;
    $127 = HEAP32[$db>>2]|0;
    $128 = ((($127)) + 16|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = ((($129)) + 4|0);
    $131 = HEAP32[$3>>2]|0;
    $132 = $131 | 256;
    $133 = (_sqlite3BtreeOpen($124,$125,$126,$130,0,$132)|0);
    HEAP32[$rc>>2] = $133;
    $134 = HEAP32[$rc>>2]|0;
    $135 = ($134|0)!=(0);
    if ($135) {
     $136 = HEAP32[$rc>>2]|0;
     $137 = ($136|0)==(3082);
     if ($137) {
      HEAP32[$rc>>2] = 7;
     }
     $138 = HEAP32[$db>>2]|0;
     $139 = HEAP32[$rc>>2]|0;
     _sqlite3Error($138,$139);
     break;
    }
    $140 = HEAP32[$db>>2]|0;
    $141 = ((($140)) + 16|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ((($142)) + 4|0);
    $144 = HEAP32[$143>>2]|0;
    _sqlite3BtreeEnter($144);
    $145 = HEAP32[$db>>2]|0;
    $146 = HEAP32[$db>>2]|0;
    $147 = ((($146)) + 16|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = ((($148)) + 4|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = (_sqlite3SchemaGet($145,$150)|0);
    $152 = HEAP32[$db>>2]|0;
    $153 = ((($152)) + 16|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = ((($154)) + 12|0);
    HEAP32[$155>>2] = $151;
    $156 = HEAP32[$db>>2]|0;
    $157 = ((($156)) + 69|0);
    $158 = HEAP8[$157>>0]|0;
    $159 = ($158<<24>>24)!=(0);
    if (!($159)) {
     $160 = HEAP32[$db>>2]|0;
     $161 = ((($160)) + 16|0);
     $162 = HEAP32[$161>>2]|0;
     $163 = ((($162)) + 12|0);
     $164 = HEAP32[$163>>2]|0;
     $165 = ((($164)) + 77|0);
     $166 = HEAP8[$165>>0]|0;
     $167 = HEAP32[$db>>2]|0;
     $168 = ((($167)) + 66|0);
     HEAP8[$168>>0] = $166;
    }
    $169 = HEAP32[$db>>2]|0;
    $170 = (_sqlite3SchemaGet($169,0)|0);
    $171 = HEAP32[$db>>2]|0;
    $172 = ((($171)) + 16|0);
    $173 = HEAP32[$172>>2]|0;
    $174 = ((($173)) + 16|0);
    $175 = ((($174)) + 12|0);
    HEAP32[$175>>2] = $170;
    $176 = HEAP32[$db>>2]|0;
    $177 = ((($176)) + 16|0);
    $178 = HEAP32[$177>>2]|0;
    HEAP32[$178>>2] = 20442;
    $179 = HEAP32[$db>>2]|0;
    $180 = ((($179)) + 16|0);
    $181 = HEAP32[$180>>2]|0;
    $182 = ((($181)) + 8|0);
    HEAP8[$182>>0] = 3;
    $183 = HEAP32[$db>>2]|0;
    $184 = ((($183)) + 16|0);
    $185 = HEAP32[$184>>2]|0;
    $186 = ((($185)) + 16|0);
    HEAP32[$186>>2] = 18672;
    $187 = HEAP32[$db>>2]|0;
    $188 = ((($187)) + 16|0);
    $189 = HEAP32[$188>>2]|0;
    $190 = ((($189)) + 16|0);
    $191 = ((($190)) + 8|0);
    HEAP8[$191>>0] = 1;
    $192 = HEAP32[$db>>2]|0;
    $193 = ((($192)) + 84|0);
    HEAP32[$193>>2] = -1607883113;
    $194 = HEAP32[$db>>2]|0;
    $195 = ((($194)) + 69|0);
    $196 = HEAP8[$195>>0]|0;
    $197 = ($196<<24>>24)!=(0);
    if (!($197)) {
     $198 = HEAP32[$db>>2]|0;
     _sqlite3Error($198,0);
     $199 = HEAP32[$db>>2]|0;
     _sqlite3RegisterPerConnectionBuiltinFunctions($199);
     $200 = HEAP32[$db>>2]|0;
     $201 = (_sqlite3_errcode($200)|0);
     HEAP32[$rc>>2] = $201;
     $202 = HEAP32[$rc>>2]|0;
     $203 = ($202|0)==(0);
     if ($203) {
      $204 = HEAP32[$db>>2]|0;
      _sqlite3AutoLoadExtensions($204);
      $205 = HEAP32[$db>>2]|0;
      $206 = (_sqlite3_errcode($205)|0);
      HEAP32[$rc>>2] = $206;
      $207 = HEAP32[$rc>>2]|0;
      $208 = ($207|0)!=(0);
      if ($208) {
       break;
      }
     }
     $209 = HEAP32[$db>>2]|0;
     $210 = ((($209)) + 69|0);
     $211 = HEAP8[$210>>0]|0;
     $212 = ($211<<24>>24)==(0);
     $213 = HEAP32[$rc>>2]|0;
     $214 = ($213|0)==(0);
     $or$cond = $212 & $214;
     if ($or$cond) {
      $215 = HEAP32[$db>>2]|0;
      $216 = (_sqlite3Fts3Init($215)|0);
      HEAP32[$rc>>2] = $216;
     }
     $217 = HEAP32[$rc>>2]|0;
     $218 = ($217|0)!=(0);
     if ($218) {
      $219 = HEAP32[$db>>2]|0;
      $220 = HEAP32[$rc>>2]|0;
      _sqlite3Error($219,$220);
     }
     $221 = HEAP32[$db>>2]|0;
     $222 = HEAP32[(36)>>2]|0;
     $223 = HEAP32[(40)>>2]|0;
     (_setupLookaside($221,0,$222,$223)|0);
     $224 = HEAP32[$db>>2]|0;
     (_sqlite3_wal_autocheckpoint($224,1000)|0);
    }
   }
  }
 } while(0);
 $225 = HEAP32[$db>>2]|0;
 $226 = (_sqlite3_errcode($225)|0);
 HEAP32[$rc>>2] = $226;
 $227 = HEAP32[$rc>>2]|0;
 $228 = ($227|0)==(7);
 if ($228) {
  $229 = HEAP32[$db>>2]|0;
  (_sqlite3_close($229)|0);
  HEAP32[$db>>2] = 0;
 } else {
  $230 = HEAP32[$rc>>2]|0;
  $231 = ($230|0)!=(0);
  if ($231) {
   $232 = HEAP32[$db>>2]|0;
   $233 = ((($232)) + 84|0);
   HEAP32[$233>>2] = 1266094736;
  }
 }
 $234 = HEAP32[$db>>2]|0;
 $235 = HEAP32[$2>>2]|0;
 HEAP32[$235>>2] = $234;
 $236 = HEAP32[$zOpen>>2]|0;
 _sqlite3_free($236);
 $237 = HEAP32[$rc>>2]|0;
 $238 = $237 & 255;
 HEAP32[$0>>2] = $238;
 $239 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($239|0);
}
function _createCollation($db,$zName,$enc,$pCtx,$xCompare,$xDel) {
 $db = $db|0;
 $zName = $zName|0;
 $enc = $enc|0;
 $pCtx = $pCtx|0;
 $xCompare = $xCompare|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $aColl = 0, $enc2 = 0, $j = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pColl = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 48|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $pColl = sp + 20|0;
 $enc2 = sp + 16|0;
 $aColl = sp + 12|0;
 $j = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zName;
 HEAP8[$3>>0] = $enc;
 HEAP32[$4>>2] = $pCtx;
 HEAP32[$5>>2] = $xCompare;
 HEAP32[$6>>2] = $xDel;
 $7 = HEAP8[$3>>0]|0;
 $8 = $7&255;
 HEAP32[$enc2>>2] = $8;
 $9 = HEAP32[$enc2>>2]|0;
 $10 = ($9|0)==(4);
 $11 = HEAP32[$enc2>>2]|0;
 $12 = ($11|0)==(8);
 $or$cond = $10 | $12;
 if ($or$cond) {
  $13 = HEAP8[968>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = ($14|0)==(0);
  $16 = $15 ? 3 : 2;
  HEAP32[$enc2>>2] = $16;
 }
 $17 = HEAP32[$enc2>>2]|0;
 $18 = ($17|0)<(1);
 $19 = HEAP32[$enc2>>2]|0;
 $20 = ($19|0)>(3);
 $or$cond3 = $18 | $20;
 if ($or$cond3) {
  $21 = (_sqlite3MisuseError(140172)|0);
  HEAP32[$0>>2] = $21;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$enc2>>2]|0;
 $24 = $23&255;
 $25 = HEAP32[$2>>2]|0;
 $26 = (_sqlite3FindCollSeq($22,$24,$25,0)|0);
 HEAP32[$pColl>>2] = $26;
 $27 = HEAP32[$pColl>>2]|0;
 $28 = ($27|0)!=(0|0);
 L8: do {
  if ($28) {
   $29 = HEAP32[$pColl>>2]|0;
   $30 = ((($29)) + 12|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 156|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($35|0)!=(0);
    $37 = HEAP32[$1>>2]|0;
    if ($36) {
     _sqlite3ErrorWithMsg($37,5,20361,$vararg_buffer);
     HEAP32[$0>>2] = 5;
     $102 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($102|0);
    }
    _sqlite3ExpirePreparedStatements($37);
    $38 = HEAP32[$pColl>>2]|0;
    $39 = ((($38)) + 4|0);
    $40 = HEAP8[$39>>0]|0;
    $41 = $40&255;
    $42 = $41 & -9;
    $43 = HEAP32[$enc2>>2]|0;
    $44 = ($42|0)==($43|0);
    if ($44) {
     $45 = HEAP32[$1>>2]|0;
     $46 = ((($45)) + 364|0);
     $47 = HEAP32[$2>>2]|0;
     $48 = (_sqlite3HashFind($46,$47)|0);
     HEAP32[$aColl>>2] = $48;
     HEAP32[$j>>2] = 0;
     while(1) {
      $49 = HEAP32[$j>>2]|0;
      $50 = ($49|0)<(3);
      if (!($50)) {
       break L8;
      }
      $51 = HEAP32[$j>>2]|0;
      $52 = HEAP32[$aColl>>2]|0;
      $53 = (($52) + (($51*20)|0)|0);
      HEAP32[$p>>2] = $53;
      $54 = HEAP32[$p>>2]|0;
      $55 = ((($54)) + 4|0);
      $56 = HEAP8[$55>>0]|0;
      $57 = $56&255;
      $58 = HEAP32[$pColl>>2]|0;
      $59 = ((($58)) + 4|0);
      $60 = HEAP8[$59>>0]|0;
      $61 = $60&255;
      $62 = ($57|0)==($61|0);
      if ($62) {
       $63 = HEAP32[$p>>2]|0;
       $64 = ((($63)) + 16|0);
       $65 = HEAP32[$64>>2]|0;
       $66 = ($65|0)!=(0|0);
       if ($66) {
        $67 = HEAP32[$p>>2]|0;
        $68 = ((($67)) + 16|0);
        $69 = HEAP32[$68>>2]|0;
        $70 = HEAP32[$p>>2]|0;
        $71 = ((($70)) + 8|0);
        $72 = HEAP32[$71>>2]|0;
        FUNCTION_TABLE_vi[$69 & 255]($72);
       }
       $73 = HEAP32[$p>>2]|0;
       $74 = ((($73)) + 12|0);
       HEAP32[$74>>2] = 0;
      }
      $75 = HEAP32[$j>>2]|0;
      $76 = (($75) + 1)|0;
      HEAP32[$j>>2] = $76;
     }
    }
   }
  }
 } while(0);
 $77 = HEAP32[$1>>2]|0;
 $78 = HEAP32[$enc2>>2]|0;
 $79 = $78&255;
 $80 = HEAP32[$2>>2]|0;
 $81 = (_sqlite3FindCollSeq($77,$79,$80,1)|0);
 HEAP32[$pColl>>2] = $81;
 $82 = HEAP32[$pColl>>2]|0;
 $83 = ($82|0)==(0|0);
 if ($83) {
  HEAP32[$0>>2] = 7;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 } else {
  $84 = HEAP32[$5>>2]|0;
  $85 = HEAP32[$pColl>>2]|0;
  $86 = ((($85)) + 12|0);
  HEAP32[$86>>2] = $84;
  $87 = HEAP32[$4>>2]|0;
  $88 = HEAP32[$pColl>>2]|0;
  $89 = ((($88)) + 8|0);
  HEAP32[$89>>2] = $87;
  $90 = HEAP32[$6>>2]|0;
  $91 = HEAP32[$pColl>>2]|0;
  $92 = ((($91)) + 16|0);
  HEAP32[$92>>2] = $90;
  $93 = HEAP32[$enc2>>2]|0;
  $94 = HEAP8[$3>>0]|0;
  $95 = $94&255;
  $96 = $95 & 8;
  $97 = $93 | $96;
  $98 = $97&255;
  $99 = HEAP32[$pColl>>2]|0;
  $100 = ((($99)) + 4|0);
  HEAP8[$100>>0] = $98;
  $101 = HEAP32[$1>>2]|0;
  _sqlite3Error($101,0);
  HEAP32[$0>>2] = 0;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 return (0)|0;
}
function _sqlite3Init($db,$pzErrMsg) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $commit_internal = 0, $i = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $rc = sp + 4|0;
 $commit_internal = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pzErrMsg;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 $7 = $6 ^ 1;
 $8 = $7&1;
 HEAP32[$commit_internal>>2] = $8;
 HEAP32[$rc>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 148|0);
 $11 = ((($10)) + 5|0);
 HEAP8[$11>>0] = 1;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 77|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 66|0);
 HEAP8[$20>>0] = $18;
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$rc>>2]|0;
  $22 = ($21|0)==(0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28<<4)|0);
  $33 = ((($32)) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ((($34)) + 78|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 & 1;
  $39 = ($38|0)==(1);
  $40 = HEAP32[$i>>2]|0;
  $41 = ($40|0)==(1);
  $or$cond = $39 | $41;
  if (!($or$cond)) {
   $42 = HEAP32[$0>>2]|0;
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = (_sqlite3InitOne($42,$43,$44)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$0>>2]|0;
    $49 = HEAP32[$i>>2]|0;
    _sqlite3ResetOneSchema($48,$49);
   }
  }
  $50 = HEAP32[$i>>2]|0;
  $51 = (($50) + 1)|0;
  HEAP32[$i>>2] = $51;
 }
 $52 = HEAP32[$rc>>2]|0;
 $53 = ($52|0)==(0);
 if ($53) {
  $54 = HEAP32[$0>>2]|0;
  $55 = ((($54)) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ((($56)) + 16|0);
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = ((($59)) + 78|0);
  $61 = HEAP16[$60>>1]|0;
  $62 = $61&65535;
  $63 = $62 & 1;
  $64 = ($63|0)==(1);
  if (!($64)) {
   $65 = HEAP32[$0>>2]|0;
   $66 = HEAP32[$1>>2]|0;
   $67 = (_sqlite3InitOne($65,1,$66)|0);
   HEAP32[$rc>>2] = $67;
   $68 = HEAP32[$rc>>2]|0;
   $69 = ($68|0)!=(0);
   if ($69) {
    $70 = HEAP32[$0>>2]|0;
    _sqlite3ResetOneSchema($70,1);
   }
  }
 }
 $71 = HEAP32[$0>>2]|0;
 $72 = ((($71)) + 148|0);
 $73 = ((($72)) + 5|0);
 HEAP8[$73>>0] = 0;
 $74 = HEAP32[$rc>>2]|0;
 $75 = ($74|0)==(0);
 $76 = HEAP32[$commit_internal>>2]|0;
 $77 = ($76|0)!=(0);
 $or$cond3 = $75 & $77;
 if (!($or$cond3)) {
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $78 = HEAP32[$0>>2]|0;
 _sqlite3CommitInternalChanges($78);
 $79 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _sqlite3FindTable($db,$zName,$zDatabase) {
 $db = $db|0;
 $zName = $zName|0;
 $zDatabase = $zDatabase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $j = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $p = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $zDatabase;
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)<(2);
  $10 = HEAP32[$i>>2]|0;
  $11 = $10 ^ 1;
  $12 = $9 ? $11 : $10;
  HEAP32[$j>>2] = $12;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)==(0|0);
  if ($14) {
   label = 5;
  } else {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$j>>2]|0;
   $17 = HEAP32[$0>>2]|0;
   $18 = ((($17)) + 16|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + ($16<<4)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (_sqlite3StrICmp($15,$21)|0);
   $23 = ($22|0)==(0);
   if ($23) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $24 = HEAP32[$j>>2]|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + ($24<<4)|0);
   $29 = ((($28)) + 12|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ((($30)) + 8|0);
   $32 = HEAP32[$1>>2]|0;
   $33 = (_sqlite3HashFind($31,$32)|0);
   HEAP32[$p>>2] = $33;
   $34 = HEAP32[$p>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    label = 7;
    break;
   }
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 if ((label|0) == 7) {
  $38 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 return (0)|0;
}
function _sqlite3IsRowid($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3StrICmp($2,19887)|0);
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 1;
  } else {
   $5 = HEAP32[$1>>2]|0;
   $6 = (_sqlite3StrICmp($5,19895)|0);
   $7 = ($6|0)==(0);
   if ($7) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = (_sqlite3StrICmp($8,19901)|0);
   $10 = ($9|0)==(0);
   if ($10) {
    HEAP32[$0>>2] = 1;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3ColumnType($pCol,$zDflt) {
 $pCol = $pCol|0;
 $zDflt = $zDflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pCol;
 HEAP32[$2>>2] = $zDflt;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 15|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = $6 & 4;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $9;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 } else {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_strlen($13)|0);
  $15 = (($11) + ($14)|0);
  $16 = ((($15)) + 1|0);
  HEAP32[$0>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3OsSleep($pVfs,$nMicro) {
 $pVfs = $pVfs|0;
 $nMicro = $nMicro|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $nMicro;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$4 & 255]($5,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _sqlite3DbNameToBtree($db,$zDbName) {
 $db = $db|0;
 $zDbName = $zDbName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zDbName;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 8;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if ($15) {
   $16 = HEAP32[$2>>2]|0;
   $17 = ($16|0)==(0|0);
   if ($17) {
    label = 6;
    break;
   }
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + ($19<<4)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (_sqlite3StrICmp($18,$24)|0);
   $26 = ($25|0)==(0);
   if ($26) {
    label = 6;
    break;
   }
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 if ((label|0) == 6) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<4)|0);
  $32 = ((($31)) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$0>>2] = $33;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 return (0)|0;
}
function _sqlite3PagerVfs($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3PagerJrnlFile($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ((($5)) + 216|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3WalFile($7)|0);
  $11 = $8;
  STACKTOP = sp;return ($11|0);
 } else {
  $9 = ((($5)) + 68|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10;
  STACKTOP = sp;return ($11|0);
 }
 return (0)|0;
}
function _sqlite3OsFileControl($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (FUNCTION_TABLE_iiii[$6 & 255]($7,$8,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _sqlite3FaultSim($iTest) {
 $iTest = $iTest|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $xCallback = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $xCallback = sp;
 HEAP32[$0>>2] = $iTest;
 $1 = HEAP32[(272)>>2]|0;
 HEAP32[$xCallback>>2] = $1;
 $2 = HEAP32[$xCallback>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $7 = 0;
  STACKTOP = sp;return ($7|0);
 }
 $4 = HEAP32[$xCallback>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 255]($5)|0);
 $7 = $6;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3BtreeSetPageSize($p,$pageSize,$nReserve,$iFix) {
 $p = $p|0;
 $pageSize = $pageSize|0;
 $nReserve = $nReserve|0;
 $iFix = $iFix|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pageSize;
 HEAP32[$3>>2] = $nReserve;
 HEAP32[$4>>2] = $iFix;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$pBt>>2]|0;
 $10 = ((($9)) + 22|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & 2;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 8;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $15 = HEAP32[$3>>2]|0;
 $16 = ($15|0)<(0);
 if ($16) {
  $17 = HEAP32[$pBt>>2]|0;
  $18 = ((($17)) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = ((($20)) + 36|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19) - ($22))|0;
  HEAP32[$3>>2] = $23;
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)>=(512);
 $26 = HEAP32[$2>>2]|0;
 $27 = ($26|0)<=(65536);
 $or$cond = $25 & $27;
 if ($or$cond) {
  $28 = HEAP32[$2>>2]|0;
  $29 = (($28) - 1)|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = $29 & $30;
  $32 = ($31|0)==(0);
  if ($32) {
   $33 = HEAP32[$2>>2]|0;
   $34 = HEAP32[$pBt>>2]|0;
   $35 = ((($34)) + 32|0);
   HEAP32[$35>>2] = $33;
   $36 = HEAP32[$pBt>>2]|0;
   _freeTempSpace($36);
  }
 }
 $37 = HEAP32[$pBt>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$pBt>>2]|0;
 $40 = ((($39)) + 32|0);
 $41 = HEAP32[$3>>2]|0;
 $42 = (_sqlite3PagerSetPagesize($38,$40,$41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$pBt>>2]|0;
 $44 = ((($43)) + 32|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$3>>2]|0;
 $47 = $46&65535;
 $48 = $47&65535;
 $49 = (($45) - ($48))|0;
 $50 = HEAP32[$pBt>>2]|0;
 $51 = ((($50)) + 36|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$4>>2]|0;
 $53 = ($52|0)!=(0);
 if ($53) {
  $54 = HEAP32[$pBt>>2]|0;
  $55 = ((($54)) + 22|0);
  $56 = HEAP16[$55>>1]|0;
  $57 = $56&65535;
  $58 = $57 | 2;
  $59 = $58&65535;
  HEAP16[$55>>1] = $59;
 }
 $60 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $60;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3KeywordCode($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $id = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $id = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 HEAP32[$id>>2] = 55;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 (_keywordCode($2,$3,$id)|0);
 $4 = HEAP32[$id>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3ScratchMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3StatusHighwater(8,$1);
 $2 = HEAP32[(312)>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[(204)>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ($4|0)>=($5|0);
  if ($6) {
   $7 = HEAP32[(308)>>2]|0;
   HEAP32[$p>>2] = $7;
   $8 = HEAP32[(308)>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   HEAP32[(308)>>2] = $9;
   $10 = HEAP32[(312)>>2]|0;
   $11 = (($10) + -1)|0;
   HEAP32[(312)>>2] = $11;
   _sqlite3StatusUp(3,1);
   $22 = HEAP32[$p>>2]|0;
   STACKTOP = sp;return ($22|0);
  }
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_sqlite3Malloc($12,$14)|0);
 HEAP32[$p>>2] = $15;
 $16 = HEAP32[8>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = HEAP32[$p>>2]|0;
 $19 = ($18|0)!=(0|0);
 $or$cond = $17 & $19;
 if (!($or$cond)) {
  $22 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = (_sqlite3MallocSize($20)|0);
 _sqlite3StatusUp(4,$21);
 $22 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3ScratchFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iSize = 0, $pSlot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pSlot = sp + 4|0;
 $iSize = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = $3;
 $5 = HEAP32[(200)>>2]|0;
 $6 = $5;
 $7 = ($4>>>0)>=($6>>>0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = $8;
  $10 = HEAP32[(304)>>2]|0;
  $11 = $10;
  $12 = ($9>>>0)<($11>>>0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   HEAP32[$pSlot>>2] = $13;
   $14 = HEAP32[(308)>>2]|0;
   $15 = HEAP32[$pSlot>>2]|0;
   HEAP32[$15>>2] = $14;
   $16 = HEAP32[$pSlot>>2]|0;
   HEAP32[(308)>>2] = $16;
   $17 = HEAP32[(312)>>2]|0;
   $18 = (($17) + 1)|0;
   HEAP32[(312)>>2] = $18;
   _sqlite3StatusDown(3,1);
   STACKTOP = sp;return;
  }
 }
 $19 = HEAP32[8>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  $22 = (_sqlite3MallocSize($21)|0);
  HEAP32[$iSize>>2] = $22;
  $23 = HEAP32[$iSize>>2]|0;
  _sqlite3StatusDown(4,$23);
  $24 = HEAP32[$iSize>>2]|0;
  _sqlite3StatusDown(0,$24);
  _sqlite3StatusDown(9,1);
  $25 = HEAP32[(52)>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$25 & 255]($26);
  STACKTOP = sp;return;
 } else {
  $27 = HEAP32[(52)>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  FUNCTION_TABLE_vi[$27 & 255]($28);
  STACKTOP = sp;return;
 }
}
function _sqlite3GetBoolean($z,$dflt) {
 $z = $z|0;
 $dflt = $dflt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $z;
 HEAP8[$1>>0] = $dflt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = (_getSafetyLevel($2,1,$3)|0);
 $5 = $4&255;
 $6 = ($5|0)!=(0);
 $7 = $6&1;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3DecOrHexToI64($z,$pOut) {
 $z = $z|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $i = 0, $k = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $u = sp;
 $i = sp + 12|0;
 $k = sp + 8|0;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(48);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 1|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)==(120);
   if (!($11)) {
    $12 = HEAP32[$1>>2]|0;
    $13 = ((($12)) + 1|0);
    $14 = HEAP8[$13>>0]|0;
    $15 = $14 << 24 >> 24;
    $16 = ($15|0)==(88);
    if (!($16)) {
     break;
    }
   }
   $17 = $u;
   $18 = $17;
   HEAP32[$18>>2] = 0;
   $19 = (($17) + 4)|0;
   $20 = $19;
   HEAP32[$20>>2] = 0;
   HEAP32[$i>>2] = 2;
   while(1) {
    $21 = HEAP32[$i>>2]|0;
    $22 = HEAP32[$1>>2]|0;
    $23 = (($22) + ($21)|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24 << 24 >> 24;
    $26 = ($25|0)==(48);
    $27 = HEAP32[$i>>2]|0;
    if (!($26)) {
     break;
    }
    $28 = (($27) + 1)|0;
    HEAP32[$i>>2] = $28;
   }
   HEAP32[$k>>2] = $27;
   while(1) {
    $29 = HEAP32[$k>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = (($30) + ($29)|0);
    $32 = HEAP8[$31>>0]|0;
    $33 = $32&255;
    $34 = (18403 + ($33)|0);
    $35 = HEAP8[$34>>0]|0;
    $36 = $35&255;
    $37 = $36 & 8;
    $38 = ($37|0)!=(0);
    if (!($38)) {
     break;
    }
    $39 = $u;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = (($39) + 4)|0;
    $43 = $42;
    $44 = HEAP32[$43>>2]|0;
    $45 = (___muldi3(($41|0),($44|0),16,0)|0);
    $46 = tempRet0;
    $47 = HEAP32[$k>>2]|0;
    $48 = HEAP32[$1>>2]|0;
    $49 = (($48) + ($47)|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = $50 << 24 >> 24;
    $52 = (_sqlite3HexToInt($51)|0);
    $53 = $52&255;
    $54 = (_i64Add(($45|0),($46|0),($53|0),0)|0);
    $55 = tempRet0;
    $56 = $u;
    $57 = $56;
    HEAP32[$57>>2] = $54;
    $58 = (($56) + 4)|0;
    $59 = $58;
    HEAP32[$59>>2] = $55;
    $60 = HEAP32[$k>>2]|0;
    $61 = (($60) + 1)|0;
    HEAP32[$k>>2] = $61;
   }
   $62 = HEAP32[$2>>2]|0;
   ;HEAP32[$62>>2]=HEAP32[$u>>2]|0;HEAP32[$62+4>>2]=HEAP32[$u+4>>2]|0;
   $63 = HEAP32[$k>>2]|0;
   $64 = HEAP32[$1>>2]|0;
   $65 = (($64) + ($63)|0);
   $66 = HEAP8[$65>>0]|0;
   $67 = $66 << 24 >> 24;
   $68 = ($67|0)==(0);
   if ($68) {
    $69 = HEAP32[$k>>2]|0;
    $70 = HEAP32[$i>>2]|0;
    $71 = (($69) - ($70))|0;
    $72 = ($71|0)<=(16);
    $74 = $72;
   } else {
    $74 = 0;
   }
   $73 = $74 ? 0 : 1;
   HEAP32[$0>>2] = $73;
   $80 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($80|0);
  }
 } while(0);
 $75 = HEAP32[$1>>2]|0;
 $76 = HEAP32[$2>>2]|0;
 $77 = HEAP32[$1>>2]|0;
 $78 = (_sqlite3Strlen30($77)|0);
 $79 = (_sqlite3Atoi64($75,$76,$78,1)|0);
 HEAP32[$0>>2] = $79;
 $80 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($80|0);
}
function _sqlite3BtreeGetFilename($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3PagerFilename($4,1)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3BtreeIsReadonly($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 22|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 $9 = $8&1;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3PagerFilename($pPager,$nullIfMemDb) {
 $pPager = $pPager|0;
 $nullIfMemDb = $nullIfMemDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nullIfMemDb;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 16|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)!=(0);
  if ($8) {
   $12 = 18402;
   STACKTOP = sp;return ($12|0);
  }
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 176|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3HexToInt($h) {
 $h = $h|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $h;
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 >> 6;
 $3 = 1 & $2;
 $4 = ($3*9)|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + ($4))|0;
 HEAP32[$0>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = $7 & 15;
 $9 = $8&255;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3Atoi64($zNum,$pNum,$length,$enc) {
 $zNum = $zNum|0;
 $pNum = $pNum|0;
 $length = $length|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $incr = 0, $neg = 0, $nonNum = 0, $or$cond = 0, $u = 0, $zEnd = 0, $zStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 52|0;
 $incr = sp + 32|0;
 $u = sp;
 $neg = sp + 28|0;
 $i = sp + 24|0;
 $c = sp + 20|0;
 $nonNum = sp + 16|0;
 $zStart = sp + 12|0;
 $zEnd = sp + 8|0;
 HEAP32[$1>>2] = $zNum;
 HEAP32[$2>>2] = $pNum;
 HEAP32[$3>>2] = $length;
 HEAP8[$4>>0] = $enc;
 $5 = $u;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 HEAP32[$neg>>2] = 0;
 HEAP32[$c>>2] = 0;
 HEAP32[$nonNum>>2] = 0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (($9) + ($10)|0);
 HEAP32[$zEnd>>2] = $11;
 $12 = HEAP8[$4>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(1);
 if ($14) {
  HEAP32[$incr>>2] = 1;
 } else {
  HEAP32[$incr>>2] = 2;
  $15 = HEAP8[$4>>0]|0;
  $16 = $15&255;
  $17 = (3 - ($16))|0;
  HEAP32[$i>>2] = $17;
  while(1) {
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$3>>2]|0;
   $20 = ($18|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = (($22) + ($21)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24 << 24 >> 24;
   $26 = ($25|0)==(0);
   if (!($26)) {
    break;
   }
   $27 = HEAP32[$i>>2]|0;
   $28 = (($27) + 2)|0;
   HEAP32[$i>>2] = $28;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = ($29|0)<($30|0);
  $32 = $31&1;
  HEAP32[$nonNum>>2] = $32;
  $33 = HEAP32[$i>>2]|0;
  $34 = $33 ^ 1;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35) + ($34)|0);
  HEAP32[$zEnd>>2] = $36;
  $37 = HEAP8[$4>>0]|0;
  $38 = $37&255;
  $39 = $38 & 1;
  $40 = HEAP32[$1>>2]|0;
  $41 = (($40) + ($39)|0);
  HEAP32[$1>>2] = $41;
 }
 while(1) {
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$zEnd>>2]|0;
  $44 = ($42>>>0)<($43>>>0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$1>>2]|0;
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  $48 = (18403 + ($47)|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = $50 & 1;
  $52 = ($51|0)!=(0);
  if (!($52)) {
   break;
  }
  $53 = HEAP32[$incr>>2]|0;
  $54 = HEAP32[$1>>2]|0;
  $55 = (($54) + ($53)|0);
  HEAP32[$1>>2] = $55;
 }
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$zEnd>>2]|0;
 $58 = ($56>>>0)<($57>>>0);
 do {
  if ($58) {
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP8[$59>>0]|0;
   $61 = $60 << 24 >> 24;
   $62 = ($61|0)==(45);
   if ($62) {
    HEAP32[$neg>>2] = 1;
    $63 = HEAP32[$incr>>2]|0;
    $64 = HEAP32[$1>>2]|0;
    $65 = (($64) + ($63)|0);
    HEAP32[$1>>2] = $65;
    break;
   }
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP8[$66>>0]|0;
   $68 = $67 << 24 >> 24;
   $69 = ($68|0)==(43);
   if ($69) {
    $70 = HEAP32[$incr>>2]|0;
    $71 = HEAP32[$1>>2]|0;
    $72 = (($71) + ($70)|0);
    HEAP32[$1>>2] = $72;
   }
  }
 } while(0);
 $73 = HEAP32[$1>>2]|0;
 HEAP32[$zStart>>2] = $73;
 while(1) {
  $74 = HEAP32[$1>>2]|0;
  $75 = HEAP32[$zEnd>>2]|0;
  $76 = ($74>>>0)<($75>>>0);
  if (!($76)) {
   break;
  }
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP8[$77>>0]|0;
  $79 = $78 << 24 >> 24;
  $80 = ($79|0)==(48);
  if (!($80)) {
   break;
  }
  $81 = HEAP32[$incr>>2]|0;
  $82 = HEAP32[$1>>2]|0;
  $83 = (($82) + ($81)|0);
  HEAP32[$1>>2] = $83;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $84 = HEAP32[$i>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = (($85) + ($84)|0);
  $87 = HEAP32[$zEnd>>2]|0;
  $88 = ($86>>>0)<($87>>>0);
  if ($88) {
   $89 = HEAP32[$i>>2]|0;
   $90 = HEAP32[$1>>2]|0;
   $91 = (($90) + ($89)|0);
   $92 = HEAP8[$91>>0]|0;
   $93 = $92 << 24 >> 24;
   HEAP32[$c>>2] = $93;
   $94 = ($93|0)>=(48);
   if ($94) {
    $95 = HEAP32[$c>>2]|0;
    $96 = ($95|0)<=(57);
    $182 = $96;
   } else {
    $182 = 0;
   }
  } else {
   $182 = 0;
  }
  $97 = $u;
  $98 = $97;
  $99 = HEAP32[$98>>2]|0;
  $100 = (($97) + 4)|0;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  if (!($182)) {
   break;
  }
  $103 = (___muldi3(($99|0),($102|0),10,0)|0);
  $104 = tempRet0;
  $105 = HEAP32[$c>>2]|0;
  $106 = ($105|0)<(0);
  $107 = $106 << 31 >> 31;
  $108 = (_i64Add(($103|0),($104|0),($105|0),($107|0))|0);
  $109 = tempRet0;
  $110 = (_i64Subtract(($108|0),($109|0),48,0)|0);
  $111 = tempRet0;
  $112 = $u;
  $113 = $112;
  HEAP32[$113>>2] = $110;
  $114 = (($112) + 4)|0;
  $115 = $114;
  HEAP32[$115>>2] = $111;
  $116 = HEAP32[$incr>>2]|0;
  $117 = HEAP32[$i>>2]|0;
  $118 = (($117) + ($116))|0;
  HEAP32[$i>>2] = $118;
 }
 $119 = ($102>>>0)>(2147483647);
 $120 = ($99>>>0)>(4294967295);
 $121 = ($102|0)==(2147483647);
 $122 = $121 & $120;
 $123 = $119 | $122;
 $124 = HEAP32[$neg>>2]|0;
 $125 = ($124|0)!=(0);
 do {
  if ($123) {
   $126 = $125 ? 0 : -1;
   $127 = $125 ? -2147483648 : 2147483647;
   $128 = HEAP32[$2>>2]|0;
   $129 = $128;
   $130 = $129;
   HEAP32[$130>>2] = $126;
   $131 = (($129) + 4)|0;
   $132 = $131;
   HEAP32[$132>>2] = $127;
  } else {
   $133 = $u;
   $134 = $133;
   $135 = HEAP32[$134>>2]|0;
   $136 = (($133) + 4)|0;
   $137 = $136;
   $138 = HEAP32[$137>>2]|0;
   if ($125) {
    $139 = (_i64Subtract(0,0,($135|0),($138|0))|0);
    $140 = tempRet0;
    $141 = HEAP32[$2>>2]|0;
    $142 = $141;
    $143 = $142;
    HEAP32[$143>>2] = $139;
    $144 = (($142) + 4)|0;
    $145 = $144;
    HEAP32[$145>>2] = $140;
    break;
   } else {
    $146 = HEAP32[$2>>2]|0;
    $147 = $146;
    $148 = $147;
    HEAP32[$148>>2] = $135;
    $149 = (($147) + 4)|0;
    $150 = $149;
    HEAP32[$150>>2] = $138;
    break;
   }
  }
 } while(0);
 $151 = HEAP32[$i>>2]|0;
 $152 = HEAP32[$1>>2]|0;
 $153 = (($152) + ($151)|0);
 $154 = HEAP32[$zEnd>>2]|0;
 $155 = ($153>>>0)<($154>>>0);
 do {
  if (!($155)) {
   $156 = HEAP32[$i>>2]|0;
   $157 = ($156|0)==(0);
   if ($157) {
    $158 = HEAP32[$zStart>>2]|0;
    $159 = HEAP32[$1>>2]|0;
    $160 = ($158|0)==($159|0);
    if ($160) {
     break;
    }
   }
   $161 = HEAP32[$i>>2]|0;
   $162 = HEAP32[$incr>>2]|0;
   $163 = ($162*19)|0;
   $164 = ($161|0)>($163|0);
   $165 = HEAP32[$nonNum>>2]|0;
   $166 = ($165|0)!=(0);
   $or$cond = $164 | $166;
   if (!($or$cond)) {
    $167 = HEAP32[$i>>2]|0;
    $168 = HEAP32[$incr>>2]|0;
    $169 = ($168*19)|0;
    $170 = ($167|0)<($169|0);
    if ($170) {
     HEAP32[$0>>2] = 0;
     $181 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($181|0);
    }
    $171 = HEAP32[$1>>2]|0;
    $172 = HEAP32[$incr>>2]|0;
    $173 = (_compare2pow63($171,$172)|0);
    HEAP32[$c>>2] = $173;
    $174 = HEAP32[$c>>2]|0;
    $175 = ($174|0)<(0);
    if ($175) {
     HEAP32[$0>>2] = 0;
     $181 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($181|0);
    }
    $176 = HEAP32[$c>>2]|0;
    $177 = ($176|0)>(0);
    if ($177) {
     HEAP32[$0>>2] = 1;
     $181 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($181|0);
    } else {
     $178 = HEAP32[$neg>>2]|0;
     $179 = ($178|0)!=(0);
     $180 = $179 ? 0 : 2;
     HEAP32[$0>>2] = $180;
     $181 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($181|0);
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 1;
 $181 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($181|0);
}
function _compare2pow63($zNum,$incr) {
 $zNum = $zNum|0;
 $incr = $incr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $pow63 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $c = sp + 8|0;
 $i = sp + 4|0;
 $pow63 = sp;
 HEAP32[$0>>2] = $zNum;
 HEAP32[$1>>2] = $incr;
 HEAP32[$c>>2] = 0;
 HEAP32[$pow63>>2] = 18767;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$c>>2]|0;
  $3 = ($2|0)==(0);
  $4 = HEAP32[$i>>2]|0;
  $5 = ($4|0)<(18);
  $6 = $3 ? $5 : 0;
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = Math_imul($7, $8)|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = (($10) + ($9)|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$pow63>>2]|0;
  $16 = (($15) + ($14)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 << 24 >> 24;
  $19 = (($13) - ($18))|0;
  $20 = ($19*10)|0;
  HEAP32[$c>>2] = $20;
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 $23 = HEAP32[$c>>2]|0;
 $24 = ($23|0)==(0);
 if (!($24)) {
  $32 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = ($25*18)|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = (($27) + ($26)|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = $29 << 24 >> 24;
 $31 = (($30) - 56)|0;
 HEAP32[$c>>2] = $31;
 $32 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _getSafetyLevel($z,$omitFull,$dflt) {
 $z = $z|0;
 $omitFull = $omitFull|0;
 $dflt = $dflt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 17|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 16|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $omitFull;
 HEAP8[$3>>0] = $dflt;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (18403 + ($6)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & 4;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$1>>2]|0;
 if ($11) {
  $13 = (_sqlite3Atoi($12)|0);
  $14 = $13&255;
  HEAP8[$0>>0] = $14;
  $46 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($46|0);
 }
 $15 = (_sqlite3Strlen30($12)|0);
 HEAP32[$n>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = ($16|0)<(8);
  if (!($17)) {
   label = 11;
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (18786 + ($18)|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = HEAP32[$n>>2]|0;
  $23 = ($21|0)==($22|0);
  if ($23) {
   $24 = HEAP32[$i>>2]|0;
   $25 = (18794 + ($24)|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = (18802 + ($27)|0);
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP32[$n>>2]|0;
   $31 = (_sqlite3_strnicmp($28,$29,$30)|0);
   $32 = ($31|0)==(0);
   if ($32) {
    $33 = HEAP32[$2>>2]|0;
    $34 = ($33|0)!=(0);
    if (!($34)) {
     label = 9;
     break;
    }
    $35 = HEAP32[$i>>2]|0;
    $36 = (18827 + ($35)|0);
    $37 = HEAP8[$36>>0]|0;
    $38 = $37&255;
    $39 = ($38|0)<=(1);
    if ($39) {
     label = 9;
     break;
    }
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 if ((label|0) == 9) {
  $40 = HEAP32[$i>>2]|0;
  $41 = (18827 + ($40)|0);
  $42 = HEAP8[$41>>0]|0;
  HEAP8[$0>>0] = $42;
  $46 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($46|0);
 }
 else if ((label|0) == 11) {
  $45 = HEAP8[$3>>0]|0;
  HEAP8[$0>>0] = $45;
  $46 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($46|0);
 }
 return (0)|0;
}
function _sqlite3Atoi($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$x>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = HEAP32[$0>>2]|0;
  (_sqlite3GetInt32($3,$x)|0);
 }
 $4 = HEAP32[$x>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3GetInt32($zNum,$pValue) {
 $zNum = $zNum|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $neg = 0, $or$cond = 0, $u = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $v = sp;
 $i = sp + 20|0;
 $c = sp + 16|0;
 $neg = sp + 12|0;
 $u = sp + 8|0;
 HEAP32[$1>>2] = $zNum;
 HEAP32[$2>>2] = $pValue;
 $3 = $v;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 HEAP32[$neg>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)==(45);
 do {
  if ($10) {
   HEAP32[$neg>>2] = 1;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 1|0);
   HEAP32[$1>>2] = $12;
  } else {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   $16 = ($15|0)==(43);
   $17 = HEAP32[$1>>2]|0;
   if ($16) {
    $18 = ((($17)) + 1|0);
    HEAP32[$1>>2] = $18;
    break;
   }
   $19 = HEAP8[$17>>0]|0;
   $20 = $19 << 24 >> 24;
   $21 = ($20|0)==(48);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 1|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24 << 24 >> 24;
    $26 = ($25|0)==(120);
    if (!($26)) {
     $27 = HEAP32[$1>>2]|0;
     $28 = ((($27)) + 1|0);
     $29 = HEAP8[$28>>0]|0;
     $30 = $29 << 24 >> 24;
     $31 = ($30|0)==(88);
     if (!($31)) {
      break;
     }
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 2|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = (18403 + ($35)|0);
    $37 = HEAP8[$36>>0]|0;
    $38 = $37&255;
    $39 = $38 & 8;
    $40 = ($39|0)!=(0);
    if ($40) {
     HEAP32[$u>>2] = 0;
     $41 = HEAP32[$1>>2]|0;
     $42 = ((($41)) + 2|0);
     HEAP32[$1>>2] = $42;
     while(1) {
      $43 = HEAP32[$1>>2]|0;
      $44 = HEAP8[$43>>0]|0;
      $45 = $44 << 24 >> 24;
      $46 = ($45|0)==(48);
      if (!($46)) {
       break;
      }
      $47 = HEAP32[$1>>2]|0;
      $48 = ((($47)) + 1|0);
      HEAP32[$1>>2] = $48;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $49 = HEAP32[$i>>2]|0;
      $50 = HEAP32[$1>>2]|0;
      $51 = (($50) + ($49)|0);
      $52 = HEAP8[$51>>0]|0;
      $53 = $52&255;
      $54 = (18403 + ($53)|0);
      $55 = HEAP8[$54>>0]|0;
      $56 = $55&255;
      $57 = $56 & 8;
      $58 = ($57|0)!=(0);
      $59 = HEAP32[$i>>2]|0;
      $60 = ($59|0)<(8);
      $61 = $58 ? $60 : 0;
      $62 = HEAP32[$u>>2]|0;
      if (!($61)) {
       break;
      }
      $63 = $62<<4;
      $64 = HEAP32[$i>>2]|0;
      $65 = HEAP32[$1>>2]|0;
      $66 = (($65) + ($64)|0);
      $67 = HEAP8[$66>>0]|0;
      $68 = $67 << 24 >> 24;
      $69 = (_sqlite3HexToInt($68)|0);
      $70 = $69&255;
      $71 = (($63) + ($70))|0;
      HEAP32[$u>>2] = $71;
      $72 = HEAP32[$i>>2]|0;
      $73 = (($72) + 1)|0;
      HEAP32[$i>>2] = $73;
     }
     $74 = $62 & -2147483648;
     $75 = ($74|0)==(0);
     if ($75) {
      $76 = HEAP32[$i>>2]|0;
      $77 = HEAP32[$1>>2]|0;
      $78 = (($77) + ($76)|0);
      $79 = HEAP8[$78>>0]|0;
      $80 = $79&255;
      $81 = (18403 + ($80)|0);
      $82 = HEAP8[$81>>0]|0;
      $83 = $82&255;
      $84 = $83 & 8;
      $85 = ($84|0)==(0);
      if ($85) {
       $86 = HEAP32[$2>>2]|0;
       ;HEAP32[$86>>2]=HEAP32[$u>>2]|0;
       HEAP32[$0>>2] = 1;
       $162 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($162|0);
      }
     }
     HEAP32[$0>>2] = 0;
     $162 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($162|0);
    }
   }
  }
 } while(0);
 while(1) {
  $87 = HEAP32[$1>>2]|0;
  $88 = HEAP8[$87>>0]|0;
  $89 = $88 << 24 >> 24;
  $90 = ($89|0)==(48);
  if (!($90)) {
   break;
  }
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 1|0);
  HEAP32[$1>>2] = $92;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $93 = HEAP32[$i>>2]|0;
  $94 = ($93|0)<(11);
  if (!($94)) {
   break;
  }
  $95 = HEAP32[$i>>2]|0;
  $96 = HEAP32[$1>>2]|0;
  $97 = (($96) + ($95)|0);
  $98 = HEAP8[$97>>0]|0;
  $99 = $98 << 24 >> 24;
  $100 = (($99) - 48)|0;
  HEAP32[$c>>2] = $100;
  $101 = ($100|0)>=(0);
  $102 = HEAP32[$c>>2]|0;
  $103 = ($102|0)<=(9);
  $or$cond = $101 & $103;
  if (!($or$cond)) {
   break;
  }
  $104 = $v;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = (___muldi3(($106|0),($109|0),10,0)|0);
  $111 = tempRet0;
  $112 = HEAP32[$c>>2]|0;
  $113 = ($112|0)<(0);
  $114 = $113 << 31 >> 31;
  $115 = (_i64Add(($110|0),($111|0),($112|0),($114|0))|0);
  $116 = tempRet0;
  $117 = $v;
  $118 = $117;
  HEAP32[$118>>2] = $115;
  $119 = (($117) + 4)|0;
  $120 = $119;
  HEAP32[$120>>2] = $116;
  $121 = HEAP32[$i>>2]|0;
  $122 = (($121) + 1)|0;
  HEAP32[$i>>2] = $122;
 }
 $123 = HEAP32[$i>>2]|0;
 $124 = ($123|0)>(10);
 if ($124) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $125 = $v;
 $126 = $125;
 $127 = HEAP32[$126>>2]|0;
 $128 = (($125) + 4)|0;
 $129 = $128;
 $130 = HEAP32[$129>>2]|0;
 $131 = HEAP32[$neg>>2]|0;
 $132 = ($131|0)<(0);
 $133 = $132 << 31 >> 31;
 $134 = (_i64Subtract(($127|0),($130|0),($131|0),($133|0))|0);
 $135 = tempRet0;
 $136 = ($135|0)>(0);
 $137 = ($134>>>0)>(2147483647);
 $138 = ($135|0)==(0);
 $139 = $138 & $137;
 $140 = $136 | $139;
 if ($140) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $141 = HEAP32[$neg>>2]|0;
 $142 = ($141|0)!=(0);
 if ($142) {
  $143 = $v;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (($143) + 4)|0;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (_i64Subtract(0,0,($145|0),($148|0))|0);
  $150 = tempRet0;
  $151 = $v;
  $152 = $151;
  HEAP32[$152>>2] = $149;
  $153 = (($151) + 4)|0;
  $154 = $153;
  HEAP32[$154>>2] = $150;
 }
 $155 = $v;
 $156 = $155;
 $157 = HEAP32[$156>>2]|0;
 $158 = (($155) + 4)|0;
 $159 = $158;
 $160 = HEAP32[$159>>2]|0;
 $161 = HEAP32[$2>>2]|0;
 HEAP32[$161>>2] = $157;
 HEAP32[$0>>2] = 1;
 $162 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _sqlite3StatusHighwater($op,$X) {
 $op = $op|0;
 $X = $X|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $newValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $newValue = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $X;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$newValue>>2] = $2;
 $3 = HEAP32[$newValue>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((1048) + ($4<<2)|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3>>>0)>($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$newValue>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((1048) + ($9<<2)|0);
 HEAP32[$10>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3StatusUp($op,$N) {
 $op = $op|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $op;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (1008 + ($3<<2)|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2))|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (1008 + ($7<<2)|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((1048) + ($10<<2)|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($9>>>0)>($12>>>0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = (1008 + ($14<<2)|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((1048) + ($17<<2)|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _keywordCode($z,$n,$pType) {
 $z = $z|0;
 $n = $n|0;
 $pType = $pType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $zKW = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $zKW = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $pType;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>=(2);
 if (!($4)) {
  $73 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = (17613 + ($7)|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = $10<<2;
 $12 = HEAP32[$1>>2]|0;
 $13 = (($12) - 1)|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = (($14) + ($13)|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = (17613 + ($17)|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20*3)|0;
 $22 = $11 ^ $21;
 $23 = HEAP32[$1>>2]|0;
 $24 = $22 ^ $23;
 $25 = (($24|0) % 127)&-1;
 HEAP32[$i>>2] = $25;
 $26 = HEAP32[$i>>2]|0;
 $27 = (18835 + ($26)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = (($29) - 1)|0;
 HEAP32[$i>>2] = $30;
 while(1) {
  $31 = HEAP32[$i>>2]|0;
  $32 = ($31|0)>=(0);
  if (!($32)) {
   label = 13;
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = (18962 + ($33)|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35&255;
  $37 = HEAP32[$1>>2]|0;
  $38 = ($36|0)!=($37|0);
  if (!($38)) {
   HEAP32[$j>>2] = 0;
   $39 = HEAP32[$i>>2]|0;
   $40 = (9524 + ($39<<1)|0);
   $41 = HEAP16[$40>>1]|0;
   $42 = $41&65535;
   $43 = (19086 + ($42)|0);
   HEAP32[$zKW>>2] = $43;
   while(1) {
    $44 = HEAP32[$j>>2]|0;
    $45 = HEAP32[$1>>2]|0;
    $46 = ($44|0)<($45|0);
    if ($46) {
     $47 = HEAP32[$j>>2]|0;
     $48 = HEAP32[$0>>2]|0;
     $49 = (($48) + ($47)|0);
     $50 = HEAP8[$49>>0]|0;
     $51 = $50 << 24 >> 24;
     $52 = $51 & -33;
     $53 = HEAP32[$j>>2]|0;
     $54 = HEAP32[$zKW>>2]|0;
     $55 = (($54) + ($53)|0);
     $56 = HEAP8[$55>>0]|0;
     $57 = $56 << 24 >> 24;
     $58 = ($52|0)==($57|0);
     $74 = $58;
    } else {
     $74 = 0;
    }
    $59 = HEAP32[$j>>2]|0;
    if (!($74)) {
     break;
    }
    $60 = (($59) + 1)|0;
    HEAP32[$j>>2] = $60;
   }
   $61 = HEAP32[$1>>2]|0;
   $62 = ($59|0)<($61|0);
   if (!($62)) {
    break;
   }
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = (19763 + ($68)|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = $70&255;
  $72 = (($71) - 1)|0;
  HEAP32[$i>>2] = $72;
 }
 if ((label|0) == 13) {
  $73 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $63 = HEAP32[$i>>2]|0;
 $64 = (19639 + ($63)|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = $65&255;
 $67 = HEAP32[$2>>2]|0;
 HEAP32[$67>>2] = $66;
 $73 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _freeTempSpace($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + -4|0);
 HEAP32[$6>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 80|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3PageFree($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 80|0);
 HEAP32[$13>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3PagerSetPagesize($pPager,$pPageSize,$nReserve) {
 $pPager = $pPager|0;
 $pPageSize = $pPageSize|0;
 $nReserve = $nReserve|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nByte = 0, $or$cond = 0, $pNew = 0, $pageSize = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $pageSize = sp + 12|0;
 $pNew = sp + 8|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pPageSize;
 HEAP32[$2>>2] = $nReserve;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pageSize>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(0);
 if ($9) {
  label = 3;
 } else {
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 28|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)==(0);
  if ($13) {
   label = 3;
  }
 }
 do {
  if ((label|0) == 3) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 212|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3PcacheRefCount($16)|0);
   $18 = ($17|0)==(0);
   $19 = HEAP32[$pageSize>>2]|0;
   $20 = ($19|0)!=(0);
   $or$cond = $18 & $20;
   if ($or$cond) {
    $21 = HEAP32[$pageSize>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 160|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($21|0)!=($24|0);
    if ($25) {
     HEAP32[$pNew>>2] = 0;
     $26 = $nByte;
     $27 = $26;
     HEAP32[$27>>2] = 0;
     $28 = (($26) + 4)|0;
     $29 = $28;
     HEAP32[$29>>2] = 0;
     $30 = HEAP32[$0>>2]|0;
     $31 = ((($30)) + 17|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = $32&255;
     $34 = ($33|0)>(0);
     if ($34) {
      $35 = HEAP32[$0>>2]|0;
      $36 = ((($35)) + 64|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = HEAP32[$37>>2]|0;
      $39 = ($38|0)!=(0|0);
      if ($39) {
       $40 = HEAP32[$0>>2]|0;
       $41 = ((($40)) + 64|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = (_sqlite3OsFileSize($42,$nByte)|0);
       HEAP32[$rc>>2] = $43;
      }
     }
     $44 = HEAP32[$rc>>2]|0;
     $45 = ($44|0)==(0);
     if ($45) {
      $46 = HEAP32[$pageSize>>2]|0;
      $47 = (_sqlite3PageMalloc($46)|0);
      HEAP32[$pNew>>2] = $47;
      $48 = HEAP32[$pNew>>2]|0;
      $49 = ($48|0)!=(0|0);
      if (!($49)) {
       HEAP32[$rc>>2] = 7;
      }
     }
     $50 = HEAP32[$rc>>2]|0;
     $51 = ($50|0)==(0);
     if ($51) {
      $52 = HEAP32[$0>>2]|0;
      _pager_reset($52);
      $53 = HEAP32[$0>>2]|0;
      $54 = ((($53)) + 212|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$pageSize>>2]|0;
      $57 = (_sqlite3PcacheSetPageSize($55,$56)|0);
      HEAP32[$rc>>2] = $57;
     }
     $58 = HEAP32[$rc>>2]|0;
     $59 = ($58|0)==(0);
     if ($59) {
      $60 = HEAP32[$0>>2]|0;
      $61 = ((($60)) + 208|0);
      $62 = HEAP32[$61>>2]|0;
      _sqlite3PageFree($62);
      $63 = HEAP32[$pNew>>2]|0;
      $64 = HEAP32[$0>>2]|0;
      $65 = ((($64)) + 208|0);
      HEAP32[$65>>2] = $63;
      $66 = $nByte;
      $67 = $66;
      $68 = HEAP32[$67>>2]|0;
      $69 = (($66) + 4)|0;
      $70 = $69;
      $71 = HEAP32[$70>>2]|0;
      $72 = HEAP32[$pageSize>>2]|0;
      $73 = (_i64Add(($68|0),($71|0),($72|0),0)|0);
      $74 = tempRet0;
      $75 = (_i64Subtract(($73|0),($74|0),1,0)|0);
      $76 = tempRet0;
      $77 = HEAP32[$pageSize>>2]|0;
      $78 = (___divdi3(($75|0),($76|0),($77|0),0)|0);
      $79 = tempRet0;
      $80 = HEAP32[$0>>2]|0;
      $81 = ((($80)) + 28|0);
      HEAP32[$81>>2] = $78;
      $82 = HEAP32[$pageSize>>2]|0;
      $83 = HEAP32[$0>>2]|0;
      $84 = ((($83)) + 160|0);
      HEAP32[$84>>2] = $82;
      break;
     } else {
      $85 = HEAP32[$pNew>>2]|0;
      _sqlite3PageFree($85);
      break;
     }
    }
   }
  }
 } while(0);
 $86 = HEAP32[$0>>2]|0;
 $87 = ((($86)) + 160|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = HEAP32[$1>>2]|0;
 HEAP32[$89>>2] = $88;
 $90 = HEAP32[$rc>>2]|0;
 $91 = ($90|0)==(0);
 if (!($91)) {
  $103 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($103|0);
 }
 $92 = HEAP32[$2>>2]|0;
 $93 = ($92|0)<(0);
 if ($93) {
  $94 = HEAP32[$0>>2]|0;
  $95 = ((($94)) + 150|0);
  $96 = HEAP16[$95>>1]|0;
  $97 = $96 << 16 >> 16;
  HEAP32[$2>>2] = $97;
 }
 $98 = HEAP32[$2>>2]|0;
 $99 = $98&65535;
 $100 = HEAP32[$0>>2]|0;
 $101 = ((($100)) + 150|0);
 HEAP16[$101>>1] = $99;
 $102 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($102);
 $103 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($103|0);
}
function _sqlite3PcacheRefCount($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3OsFileSize($id,$pSize) {
 $id = $id|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3PageMalloc($sz) {
 $sz = $sz|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $sz;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_pcache1Alloc($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _pager_reset($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 108|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 96|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3BackupRestart($7);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 212|0);
 $10 = HEAP32[$9>>2]|0;
 _sqlite3PcacheClear($10);
 STACKTOP = sp;return;
}
function _sqlite3PcacheSetPageSize($pCache,$szPage) {
 $pCache = $pCache|0;
 $szPage = $szPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $szPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[(132)>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 28|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + 40)|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = (FUNCTION_TABLE_iiii[$7 & 255]($8,$12,$16)|0);
  HEAP32[$pNew>>2] = $17;
  $18 = HEAP32[$pNew>>2]|0;
  $19 = ($18|0)==(0|0);
  if ($19) {
   HEAP32[$0>>2] = 7;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
  $20 = HEAP32[(136)>>2]|0;
  $21 = HEAP32[$pNew>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = (_numberOfCachePages($22)|0);
  FUNCTION_TABLE_vii[$20 & 255]($21,$23);
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 44|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[(160)>>2]|0;
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 44|0);
   $31 = HEAP32[$30>>2]|0;
   FUNCTION_TABLE_vi[$28 & 255]($31);
  }
  $32 = HEAP32[$pNew>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 44|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$2>>2]|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 24|0);
  HEAP32[$37>>2] = $35;
 }
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3PageFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _pcache1Free($1);
 STACKTOP = sp;return;
}
function _pagerFixMaplimit($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 STACKTOP = sp;return;
}
function _pcache1Free($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nFreed = 0, $pSlot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pSlot = sp + 4|0;
 $nFreed = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = $3;
 $5 = HEAP32[(1704)>>2]|0;
 $6 = $5;
 $7 = ($4>>>0)>=($6>>>0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = $8;
  $10 = HEAP32[(1708)>>2]|0;
  $11 = $10;
  $12 = ($9>>>0)<($11>>>0);
  if ($12) {
   _sqlite3StatusDown(1,1);
   $13 = HEAP32[$0>>2]|0;
   HEAP32[$pSlot>>2] = $13;
   $14 = HEAP32[(1716)>>2]|0;
   $15 = HEAP32[$pSlot>>2]|0;
   HEAP32[$15>>2] = $14;
   $16 = HEAP32[$pSlot>>2]|0;
   HEAP32[(1716)>>2] = $16;
   $17 = HEAP32[(1720)>>2]|0;
   $18 = (($17) + 1)|0;
   HEAP32[(1720)>>2] = $18;
   $19 = HEAP32[(1720)>>2]|0;
   $20 = HEAP32[(1700)>>2]|0;
   $21 = ($19|0)<($20|0);
   $22 = $21&1;
   HEAP32[(1724)>>2] = $22;
   STACKTOP = sp;return;
  }
 }
 HEAP32[$nFreed>>2] = 0;
 $23 = HEAP32[$0>>2]|0;
 $24 = (_sqlite3MallocSize($23)|0);
 HEAP32[$nFreed>>2] = $24;
 $25 = HEAP32[$nFreed>>2]|0;
 _sqlite3StatusDown(2,$25);
 $26 = HEAP32[$0>>2]|0;
 _sqlite3_free($26);
 STACKTOP = sp;return;
}
function _numberOfCachePages($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(0);
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 if ($5) {
  HEAP32[$0>>2] = $8;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $9 = ($8|0)<(0);
  $10 = $9 << 31 >> 31;
  $11 = (___muldi3(-1024,-1,($8|0),($10|0))|0);
  $12 = tempRet0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 24|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($15) + ($18))|0;
  $20 = ($19|0)<(0);
  $21 = $20 << 31 >> 31;
  $22 = (___divdi3(($11|0),($12|0),($19|0),($21|0))|0);
  $23 = tempRet0;
  HEAP32[$0>>2] = $22;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3BackupRestart($pBackup) {
 $pBackup = $pBackup|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBackup;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 while(1) {
  $2 = HEAP32[$p>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 16|0);
  HEAP32[$5>>2] = 1;
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$p>>2] = $8;
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheClear($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3PcacheTruncate($1,0);
 STACKTOP = sp;return;
}
function _sqlite3PcacheTruncate($pCache,$pgno) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, $pPage1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $p = sp + 8|0;
 $pNext = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 while(1) {
  $8 = HEAP32[$p>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pNext>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15>>>0)>($16>>>0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   _sqlite3PcacheMakeClean($18);
  }
  $19 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $19;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0);
  if ($25) {
   $26 = HEAP32[(144)>>2]|0;
   $27 = HEAP32[$0>>2]|0;
   $28 = ((($27)) + 44|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (FUNCTION_TABLE_iiii[$26 & 255]($29,1,0)|0);
   HEAP32[$pPage1>>2] = $30;
   $31 = HEAP32[$pPage1>>2]|0;
   $32 = ($31|0)!=(0|0);
   if ($32) {
    $33 = HEAP32[$pPage1>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 24|0);
    $37 = HEAP32[$36>>2]|0;
    _memset(($34|0),0,($37|0))|0;
    HEAP32[$1>>2] = 1;
   }
  }
 }
 $38 = HEAP32[(156)>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 44|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 $43 = (($42) + 1)|0;
 FUNCTION_TABLE_vii[$38 & 255]($41,$43);
 STACKTOP = sp;return;
}
function _sqlite3PcacheMakeClean($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($7,1);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & -15;
 $13 = $12&65535;
 HEAP16[$9>>1] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 | 1;
 $19 = $18&65535;
 HEAP16[$15>>1] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 26|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)==(0);
 if (!($24)) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$0>>2]|0;
 _pcacheUnpin($25);
 STACKTOP = sp;return;
}
function _pcacheManageDirtyList($pPage,$addRemove) {
 $pPage = $pPage|0;
 $addRemove = $addRemove|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $p = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP8[$1>>0] = $addRemove;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP8[$1>>0]|0;
 $6 = $5&255;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ($11|0)==($12|0);
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 36|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$p>>2]|0;
   $18 = ((($17)) + 8|0);
   HEAP32[$18>>2] = $16;
  }
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)!=(0|0);
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 36|0);
  $25 = HEAP32[$24>>2]|0;
  if ($22) {
   $26 = HEAP32[$0>>2]|0;
   $27 = ((($26)) + 32|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ((($28)) + 36|0);
   HEAP32[$29>>2] = $25;
  } else {
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 4|0);
   HEAP32[$31>>2] = $25;
  }
  $32 = HEAP32[$0>>2]|0;
  $33 = ((($32)) + 36|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)!=(0|0);
  $36 = HEAP32[$0>>2]|0;
  $37 = ((($36)) + 32|0);
  $38 = HEAP32[$37>>2]|0;
  if ($35) {
   $39 = HEAP32[$0>>2]|0;
   $40 = ((($39)) + 36|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = ((($41)) + 32|0);
   HEAP32[$42>>2] = $38;
  } else {
   $43 = HEAP32[$p>>2]|0;
   HEAP32[$43>>2] = $38;
   $44 = HEAP32[$p>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)==(0|0);
   if ($46) {
    $47 = HEAP32[$p>>2]|0;
    $48 = ((($47)) + 33|0);
    HEAP8[$48>>0] = 2;
   }
  }
  $49 = HEAP32[$0>>2]|0;
  $50 = ((($49)) + 32|0);
  HEAP32[$50>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  $52 = ((($51)) + 36|0);
  HEAP32[$52>>2] = 0;
 }
 $53 = HEAP8[$1>>0]|0;
 $54 = $53&255;
 $55 = $54 & 2;
 $56 = ($55|0)!=(0);
 if (!($56)) {
  STACKTOP = sp;return;
 }
 $57 = HEAP32[$p>>2]|0;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 32|0);
 HEAP32[$60>>2] = $58;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 32|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)!=(0|0);
 $65 = HEAP32[$0>>2]|0;
 if ($64) {
  $66 = HEAP32[$0>>2]|0;
  $67 = ((($66)) + 32|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = ((($68)) + 36|0);
  HEAP32[$69>>2] = $65;
 } else {
  $70 = HEAP32[$p>>2]|0;
  $71 = ((($70)) + 4|0);
  HEAP32[$71>>2] = $65;
  $72 = HEAP32[$p>>2]|0;
  $73 = ((($72)) + 32|0);
  $74 = HEAP8[$73>>0]|0;
  $75 = ($74<<24>>24)!=(0);
  if ($75) {
   $76 = HEAP32[$p>>2]|0;
   $77 = ((($76)) + 33|0);
   HEAP8[$77>>0] = 1;
  }
 }
 $78 = HEAP32[$0>>2]|0;
 $79 = HEAP32[$p>>2]|0;
 HEAP32[$79>>2] = $78;
 $80 = HEAP32[$p>>2]|0;
 $81 = ((($80)) + 8|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = ($82|0)!=(0|0);
 if ($83) {
  STACKTOP = sp;return;
 }
 $84 = HEAP32[$0>>2]|0;
 $85 = ((($84)) + 24|0);
 $86 = HEAP16[$85>>1]|0;
 $87 = $86&65535;
 $88 = $87 & 8;
 $89 = (0)==($88|0);
 if (!($89)) {
  STACKTOP = sp;return;
 }
 $90 = HEAP32[$0>>2]|0;
 $91 = HEAP32[$p>>2]|0;
 $92 = ((($91)) + 8|0);
 HEAP32[$92>>2] = $90;
 STACKTOP = sp;return;
}
function _pcacheUnpin($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[(148)>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 44|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 FUNCTION_TABLE_viii[$7 & 255]($12,$14,0);
 STACKTOP = sp;return;
}
function _pcache1Alloc($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $sz = sp;
 HEAP32[$0>>2] = $nByte;
 HEAP32[$p>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[(1692)>>2]|0;
 $3 = ($1|0)<=($2|0);
 if ($3) {
  $4 = HEAP32[(1716)>>2]|0;
  HEAP32[$p>>2] = $4;
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if ($6) {
   $7 = HEAP32[(1716)>>2]|0;
   $8 = HEAP32[$7>>2]|0;
   HEAP32[(1716)>>2] = $8;
   $9 = HEAP32[(1720)>>2]|0;
   $10 = (($9) + -1)|0;
   HEAP32[(1720)>>2] = $10;
   $11 = HEAP32[(1720)>>2]|0;
   $12 = HEAP32[(1700)>>2]|0;
   $13 = ($11|0)<($12|0);
   $14 = $13&1;
   HEAP32[(1724)>>2] = $14;
   $15 = HEAP32[$0>>2]|0;
   _sqlite3StatusHighwater(7,$15);
   _sqlite3StatusUp(1,1);
  }
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = ($16|0)==(0|0);
 if (!($17)) {
  $28 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = (_sqlite3Malloc($18,$20)|0);
 HEAP32[$p>>2] = $21;
 $22 = HEAP32[$p>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $28 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = (_sqlite3MallocSize($24)|0);
 HEAP32[$sz>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 _sqlite3StatusHighwater(7,$26);
 $27 = HEAP32[$sz>>2]|0;
 _sqlite3StatusUp(2,$27);
 $28 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3BitvecCreate($iSize) {
 $iSize = $iSize|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $iSize;
 $1 = (_sqlite3MallocZero(512,0)|0);
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $6 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BitvecSet($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiValues = 0, $bin = 0, $h = 0, $j = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $h = sp + 16|0;
 $bin = sp + 12|0;
 $j = sp + 8|0;
 $rc = sp + 4|0;
 $aiValues = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$2>>2] = $6;
 while(1) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8>>>0)>(4000);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($14>>>0) / ($17>>>0))&-1;
  HEAP32[$bin>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($19>>>0) % ($22>>>0))&-1;
  HEAP32[$2>>2] = $23;
  $24 = HEAP32[$bin>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 12|0);
  $27 = (($26) + ($24<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 8|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3BitvecCreate($32)|0);
   $34 = HEAP32[$bin>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = (($36) + ($34<<2)|0);
   HEAP32[$37>>2] = $33;
   $38 = HEAP32[$bin>>2]|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 12|0);
   $41 = (($40) + ($38<<2)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==(0|0);
   if ($43) {
    label = 8;
    break;
   }
  }
  $44 = HEAP32[$bin>>2]|0;
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 12|0);
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$1>>2] = $48;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 7;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50>>>0)<=(4000);
 $52 = HEAP32[$2>>2]|0;
 if ($51) {
  $53 = $52 & 7;
  $54 = 1 << $53;
  $55 = HEAP32[$2>>2]|0;
  $56 = (($55>>>0) / 8)&-1;
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = (($58) + ($56)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = $61 | $54;
  $63 = $62&255;
  HEAP8[$59>>0] = $63;
  HEAP32[$0>>2] = 0;
  $144 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($144|0);
 }
 $64 = (($52) + 1)|0;
 HEAP32[$2>>2] = $64;
 $65 = $52;
 $66 = (($65>>>0) % 125)&-1;
 HEAP32[$h>>2] = $66;
 $67 = HEAP32[$h>>2]|0;
 $68 = HEAP32[$1>>2]|0;
 $69 = ((($68)) + 12|0);
 $70 = (($69) + ($67<<2)|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)!=(0);
 L20: do {
  if ($72) {
   while(1) {
    $77 = HEAP32[$h>>2]|0;
    $78 = HEAP32[$1>>2]|0;
    $79 = ((($78)) + 12|0);
    $80 = (($79) + ($77<<2)|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = HEAP32[$2>>2]|0;
    $83 = ($81|0)==($82|0);
    if ($83) {
     break;
    }
    $84 = HEAP32[$h>>2]|0;
    $85 = (($84) + 1)|0;
    HEAP32[$h>>2] = $85;
    $86 = HEAP32[$h>>2]|0;
    $87 = ($86>>>0)>=(125);
    $$ = $87 ? 0 : $85;
    HEAP32[$h>>2] = $$;
    $88 = HEAP32[$h>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 12|0);
    $91 = (($90) + ($88<<2)|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)!=(0);
    if (!($93)) {
     label = 17;
     break L20;
    }
   }
   HEAP32[$0>>2] = 0;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  } else {
   $73 = HEAP32[$1>>2]|0;
   $74 = ((($73)) + 4|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75>>>0)<(124);
   if (!($76)) {
    label = 17;
   }
  }
 } while(0);
 if ((label|0) == 17) {
  $94 = HEAP32[$1>>2]|0;
  $95 = ((($94)) + 4|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96>>>0)>=(62);
  if ($97) {
   $98 = (_sqlite3DbMallocRaw(0,500,0)|0);
   HEAP32[$aiValues>>2] = $98;
   $99 = HEAP32[$aiValues>>2]|0;
   $100 = ($99|0)==(0|0);
   if ($100) {
    HEAP32[$0>>2] = 7;
    $144 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($144|0);
   }
   $101 = HEAP32[$aiValues>>2]|0;
   $102 = HEAP32[$1>>2]|0;
   $103 = ((($102)) + 12|0);
   _memcpy(($101|0),($103|0),500)|0;
   $104 = HEAP32[$1>>2]|0;
   $105 = ((($104)) + 12|0);
   _memset(($105|0),0,500)|0;
   $106 = HEAP32[$1>>2]|0;
   $107 = HEAP32[$106>>2]|0;
   $108 = (($107) + 125)|0;
   $109 = (($108) - 1)|0;
   $110 = (($109>>>0) / 125)&-1;
   $111 = HEAP32[$1>>2]|0;
   $112 = ((($111)) + 8|0);
   HEAP32[$112>>2] = $110;
   $113 = HEAP32[$1>>2]|0;
   $114 = HEAP32[$2>>2]|0;
   $115 = (_sqlite3BitvecSet($113,$114)|0);
   HEAP32[$rc>>2] = $115;
   HEAP32[$j>>2] = 0;
   while(1) {
    $116 = HEAP32[$j>>2]|0;
    $117 = ($116>>>0)<(125);
    if (!($117)) {
     break;
    }
    $118 = HEAP32[$j>>2]|0;
    $119 = HEAP32[$aiValues>>2]|0;
    $120 = (($119) + ($118<<2)|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = ($121|0)!=(0);
    if ($122) {
     $123 = HEAP32[$1>>2]|0;
     $124 = HEAP32[$j>>2]|0;
     $125 = HEAP32[$aiValues>>2]|0;
     $126 = (($125) + ($124<<2)|0);
     $127 = HEAP32[$126>>2]|0;
     $128 = (_sqlite3BitvecSet($123,$127)|0);
     $129 = HEAP32[$rc>>2]|0;
     $130 = $129 | $128;
     HEAP32[$rc>>2] = $130;
    }
    $131 = HEAP32[$j>>2]|0;
    $132 = (($131) + 1)|0;
    HEAP32[$j>>2] = $132;
   }
   $133 = HEAP32[$aiValues>>2]|0;
   _sqlite3DbFree(0,$133);
   $134 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $134;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  }
 }
 $135 = HEAP32[$1>>2]|0;
 $136 = ((($135)) + 4|0);
 $137 = HEAP32[$136>>2]|0;
 $138 = (($137) + 1)|0;
 HEAP32[$136>>2] = $138;
 $139 = HEAP32[$2>>2]|0;
 $140 = HEAP32[$h>>2]|0;
 $141 = HEAP32[$1>>2]|0;
 $142 = ((($141)) + 12|0);
 $143 = (($142) + ($140<<2)|0);
 HEAP32[$143>>2] = $139;
 HEAP32[$0>>2] = 0;
 $144 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($144|0);
}
function _sqlite3BitvecClear($p,$i,$pBuf) {
 $p = $p|0;
 $i = $i|0;
 $pBuf = $pBuf|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $aiValues = 0, $bin = 0;
 var $h = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $bin = sp + 12|0;
 $j = sp + 8|0;
 $aiValues = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pBuf;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$1>>2] = $6;
 while(1) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11>>>0) / ($14>>>0))&-1;
  HEAP32[$bin>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($16>>>0) % ($19>>>0))&-1;
  HEAP32[$1>>2] = $20;
  $21 = HEAP32[$bin>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = (($23) + ($21<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$0>>2] = $25;
  $26 = HEAP32[$0>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   label = 16;
   break;
  }
 }
 if ((label|0) == 16) {
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29>>>0)<=(4000);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = $31 & 7;
  $33 = 1 << $32;
  $34 = $33 ^ -1;
  $35 = HEAP32[$1>>2]|0;
  $36 = (($35>>>0) / 8)&-1;
  $37 = HEAP32[$0>>2]|0;
  $38 = ((($37)) + 12|0);
  $39 = (($38) + ($36)|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $42 = $41 & $34;
  $43 = $42&255;
  HEAP8[$39>>0] = $43;
  STACKTOP = sp;return;
 }
 $44 = HEAP32[$2>>2]|0;
 HEAP32[$aiValues>>2] = $44;
 $45 = HEAP32[$aiValues>>2]|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 12|0);
 _memcpy(($45|0),($47|0),500)|0;
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 12|0);
 _memset(($49|0),0,500)|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 4|0);
 HEAP32[$51>>2] = 0;
 HEAP32[$j>>2] = 0;
 while(1) {
  $52 = HEAP32[$j>>2]|0;
  $53 = ($52>>>0)<(125);
  if (!($53)) {
   break;
  }
  $54 = HEAP32[$j>>2]|0;
  $55 = HEAP32[$aiValues>>2]|0;
  $56 = (($55) + ($54<<2)|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ($57|0)!=(0);
  if ($58) {
   $59 = HEAP32[$j>>2]|0;
   $60 = HEAP32[$aiValues>>2]|0;
   $61 = (($60) + ($59<<2)|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = (($63) + 1)|0;
   $65 = ($62|0)!=($64|0);
   if ($65) {
    $66 = HEAP32[$j>>2]|0;
    $67 = HEAP32[$aiValues>>2]|0;
    $68 = (($67) + ($66<<2)|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) - 1)|0;
    $71 = $70;
    $72 = (($71>>>0) % 125)&-1;
    HEAP32[$h>>2] = $72;
    $73 = HEAP32[$0>>2]|0;
    $74 = ((($73)) + 4|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$74>>2] = $76;
    while(1) {
     $77 = HEAP32[$h>>2]|0;
     $78 = HEAP32[$0>>2]|0;
     $79 = ((($78)) + 12|0);
     $80 = (($79) + ($77<<2)|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = ($81|0)!=(0);
     if (!($82)) {
      break;
     }
     $83 = HEAP32[$h>>2]|0;
     $84 = (($83) + 1)|0;
     HEAP32[$h>>2] = $84;
     $85 = HEAP32[$h>>2]|0;
     $86 = ($85>>>0)>=(125);
     $$ = $86 ? 0 : $84;
     HEAP32[$h>>2] = $$;
    }
    $87 = HEAP32[$j>>2]|0;
    $88 = HEAP32[$aiValues>>2]|0;
    $89 = (($88) + ($87<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$h>>2]|0;
    $92 = HEAP32[$0>>2]|0;
    $93 = ((($92)) + 12|0);
    $94 = (($93) + ($91<<2)|0);
    HEAP32[$94>>2] = $90;
   }
  }
  $95 = HEAP32[$j>>2]|0;
  $96 = (($95) + 1)|0;
  HEAP32[$j>>2] = $96;
 }
 STACKTOP = sp;return;
}
function _sqlite3BitvecTest($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3BitvecTestNotNull($4,$5)|0);
 $7 = ($6|0)!=(0);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3BitvecSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3BitvecDestroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 L4: do {
  if ($6) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $7 = HEAP32[$i>>2]|0;
    $8 = ($7>>>0)<(125);
    if (!($8)) {
     break L4;
    }
    $9 = HEAP32[$i>>2]|0;
    $10 = HEAP32[$0>>2]|0;
    $11 = ((($10)) + 12|0);
    $12 = (($11) + ($9<<2)|0);
    $13 = HEAP32[$12>>2]|0;
    _sqlite3BitvecDestroy($13);
    $14 = HEAP32[$i>>2]|0;
    $15 = (($14) + 1)|0;
    HEAP32[$i>>2] = $15;
   }
  }
 } while(0);
 $16 = HEAP32[$0>>2]|0;
 _sqlite3_free($16);
 STACKTOP = sp;return;
}
function _sqlite3BitvecTestNotNull($p,$i) {
 $p = $p|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $bin = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $bin = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($5>>>0)>=($7>>>0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 while(1) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13>>>0) / ($16>>>0))&-1;
  HEAP32[$bin>>2] = $17;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($18>>>0) % ($21>>>0))&-1;
  HEAP32[$2>>2] = $22;
  $23 = HEAP32[$bin>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = (($25) + ($23<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$1>>2] = $27;
  $28 = HEAP32[$1>>2]|0;
  $29 = ($28|0)!=(0|0);
  if (!($29)) {
   label = 5;
   break;
  }
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 0;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 $30 = HEAP32[$1>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31>>>0)<=(4000);
 $33 = HEAP32[$2>>2]|0;
 if ($32) {
  $34 = (($33>>>0) / 8)&-1;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = (($36) + ($34)|0);
  $38 = HEAP8[$37>>0]|0;
  $39 = $38&255;
  $40 = HEAP32[$2>>2]|0;
  $41 = $40 & 7;
  $42 = 1 << $41;
  $43 = $39 & $42;
  $44 = ($43|0)!=(0);
  $45 = $44&1;
  HEAP32[$0>>2] = $45;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 $46 = (($33) + 1)|0;
 HEAP32[$2>>2] = $46;
 $47 = $33;
 $48 = (($47>>>0) % 125)&-1;
 HEAP32[$h>>2] = $48;
 while(1) {
  $49 = HEAP32[$h>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 12|0);
  $52 = (($51) + ($49<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($53|0)!=(0);
  if (!($54)) {
   label = 13;
   break;
  }
  $55 = HEAP32[$h>>2]|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 12|0);
  $58 = (($57) + ($55<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = ($59|0)==($60|0);
  if ($61) {
   label = 11;
   break;
  }
  $62 = HEAP32[$h>>2]|0;
  $63 = (($62) + 1)|0;
  $64 = (($63>>>0) % 125)&-1;
  HEAP32[$h>>2] = $64;
 }
 if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 return (0)|0;
}
function _sqlite3WalFile($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3HashFind($pH,$pKey) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $elem = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pKey;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_findElementWithHash($2,$3,$h)|0);
 HEAP32[$elem>>2] = $4;
 $5 = HEAP32[$elem>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = 0;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$elem>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9;
 STACKTOP = sp;return ($10|0);
}
function _findElementWithHash($pH,$pKey,$pHash) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $pHash = $pHash|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $count = 0, $elem = 0, $h = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $elem = sp + 12|0;
 $count = sp + 8|0;
 $h = sp + 4|0;
 $pEntry = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $pHash;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$2>>2]|0;
  $9 = (_strHash($8)|0);
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9>>>0) % ($11>>>0))&-1;
  HEAP32[$h>>2] = $12;
  $13 = HEAP32[$h>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<3)|0);
  HEAP32[$pEntry>>2] = $17;
  $18 = HEAP32[$pEntry>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$elem>>2] = $20;
  $21 = HEAP32[$pEntry>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$count>>2] = $22;
 } else {
  HEAP32[$h>>2] = 0;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$elem>>2] = $25;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$count>>2] = $28;
 }
 $29 = HEAP32[$h>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 HEAP32[$30>>2] = $29;
 while(1) {
  $31 = HEAP32[$count>>2]|0;
  $32 = (($31) + -1)|0;
  HEAP32[$count>>2] = $32;
  $33 = ($31|0)!=(0);
  if (!($33)) {
   label = 9;
   break;
  }
  $34 = HEAP32[$elem>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = (_sqlite3StrICmp($36,$37)|0);
  $39 = ($38|0)==(0);
  $40 = HEAP32[$elem>>2]|0;
  if ($39) {
   label = 7;
   break;
  }
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$elem>>2] = $41;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = $40;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _strHash($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $h = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $h = sp;
 $c = sp + 8|0;
 HEAP32[$0>>2] = $z;
 HEAP32[$h>>2] = 0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ((($1)) + 1|0);
  HEAP32[$0>>2] = $2;
  $3 = HEAP8[$1>>0]|0;
  HEAP8[$c>>0] = $3;
  $4 = $3&255;
  $5 = ($4|0)!=(0);
  if (!($5)) {
   break;
  }
  $6 = HEAP8[$c>>0]|0;
  $7 = $6&255;
  $8 = (17613 + ($7)|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = HEAP32[$h>>2]|0;
  $12 = (($11) + ($10))|0;
  HEAP32[$h>>2] = $12;
  $13 = HEAP32[$h>>2]|0;
  $14 = Math_imul($13, -1640531535)|0;
  HEAP32[$h>>2] = $14;
 }
 $15 = HEAP32[$h>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3InitOne($db,$iDb,$pzErrMsg) {
 $db = $db|0;
 $iDb = $iDb|0;
 $pzErrMsg = $pzErrMsg|0;
 var $$ = 0, $$1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $azArg = 0, $encoding = 0, $i = 0, $initData = 0, $meta = 0, $openedTransaction = 0, $or$cond = 0, $pDb = 0, $rc = 0, $size = 0, $vararg_buffer = 0, $vararg_ptr3 = 0, $xAuth = 0, $zMasterName = 0;
 var $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $rc = sp + 92|0;
 $i = sp + 88|0;
 $size = sp + 84|0;
 $pDb = sp + 80|0;
 $azArg = sp + 64|0;
 $meta = sp + 40|0;
 $initData = sp + 24|0;
 $zMasterName = sp + 20|0;
 $openedTransaction = sp + 16|0;
 $encoding = sp + 112|0;
 $zSql = sp + 12|0;
 $xAuth = sp + 8|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pzErrMsg;
 HEAP32[$openedTransaction>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(1);
 $6 = $5 ? 19905 : 19924;
 HEAP32[$zMasterName>>2] = $6;
 HEAP32[$azArg>>2] = $6;
 $7 = ((($azArg)) + 4|0);
 HEAP32[$7>>2] = 19938;
 $8 = ((($azArg)) + 8|0);
 HEAP32[$8>>2] = 19940;
 $9 = ((($azArg)) + 12|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$initData>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($initData)) + 8|0);
 HEAP32[$12>>2] = $11;
 $13 = ((($initData)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$3>>2]|0;
 $15 = ((($initData)) + 4|0);
 HEAP32[$15>>2] = $14;
 (_sqlite3InitCallback($initData,3,$azArg,0)|0);
 $16 = ((($initData)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = ((($initData)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$rc>>2] = $20;
 } else {
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  HEAP32[$pDb>>2] = $25;
  $26 = HEAP32[$pDb>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   $30 = HEAP32[$2>>2]|0;
   $31 = ($30|0)==(1);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 16|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = ((($34)) + 16|0);
    $36 = ((($35)) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ((($37)) + 78|0);
    $39 = HEAP16[$38>>1]|0;
    $40 = $39&65535;
    $41 = $40 | 1;
    $42 = $41&65535;
    HEAP16[$38>>1] = $42;
   }
   HEAP32[$0>>2] = 0;
   $243 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($243|0);
  }
  $43 = HEAP32[$pDb>>2]|0;
  $44 = ((($43)) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  _sqlite3BtreeEnter($45);
  $46 = HEAP32[$pDb>>2]|0;
  $47 = ((($46)) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (_sqlite3BtreeIsInReadTrans($48)|0);
  $50 = ($49|0)!=(0);
  do {
   if ($50) {
    label = 11;
   } else {
    $51 = HEAP32[$pDb>>2]|0;
    $52 = ((($51)) + 4|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (_sqlite3BtreeBeginTrans($53,0)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)!=(0);
    if ($56) {
     $57 = HEAP32[$3>>2]|0;
     $58 = HEAP32[$1>>2]|0;
     $59 = HEAP32[$rc>>2]|0;
     $60 = (_sqlite3ErrStr($59)|0);
     _sqlite3SetString($57,$58,$60);
     break;
    } else {
     HEAP32[$openedTransaction>>2] = 1;
     label = 11;
     break;
    }
   }
  } while(0);
  L16: do {
   if ((label|0) == 11) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $61 = HEAP32[$i>>2]|0;
     $62 = ($61|0)<(5);
     if (!($62)) {
      break;
     }
     $63 = HEAP32[$pDb>>2]|0;
     $64 = ((($63)) + 4|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$i>>2]|0;
     $67 = (($66) + 1)|0;
     $68 = HEAP32[$i>>2]|0;
     $69 = (($meta) + ($68<<2)|0);
     _sqlite3BtreeGetMeta($65,$67,$69);
     $70 = HEAP32[$i>>2]|0;
     $71 = (($70) + 1)|0;
     HEAP32[$i>>2] = $71;
    }
    $72 = HEAP32[$meta>>2]|0;
    $73 = HEAP32[$pDb>>2]|0;
    $74 = ((($73)) + 12|0);
    $75 = HEAP32[$74>>2]|0;
    HEAP32[$75>>2] = $72;
    $76 = ((($meta)) + 16|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = ($77|0)!=(0);
    $79 = HEAP32[$2>>2]|0;
    do {
     if ($78) {
      $80 = ($79|0)==(0);
      $81 = ((($meta)) + 16|0);
      $82 = HEAP32[$81>>2]|0;
      if ($80) {
       $83 = $82&255;
       $84 = $83&255;
       $85 = $84 & 3;
       $86 = $85&255;
       HEAP8[$encoding>>0] = $86;
       $87 = HEAP8[$encoding>>0]|0;
       $88 = $87&255;
       $89 = ($88|0)==(0);
       $$ = $89 ? 1 : $86;
       HEAP8[$encoding>>0] = $$;
       $90 = HEAP8[$encoding>>0]|0;
       $91 = HEAP32[$1>>2]|0;
       $92 = ((($91)) + 66|0);
       HEAP8[$92>>0] = $90;
       break;
      }
      $93 = HEAP32[$1>>2]|0;
      $94 = ((($93)) + 66|0);
      $95 = HEAP8[$94>>0]|0;
      $96 = $95&255;
      $97 = ($82|0)!=($96|0);
      if ($97) {
       $98 = HEAP32[$3>>2]|0;
       $99 = HEAP32[$1>>2]|0;
       _sqlite3SetString($98,$99,20016);
       HEAP32[$rc>>2] = 1;
       break L16;
      }
     } else {
      $100 = HEAP32[$1>>2]|0;
      $101 = ((($100)) + 16|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = (($102) + ($79<<4)|0);
      $104 = ((($103)) + 12|0);
      $105 = HEAP32[$104>>2]|0;
      $106 = ((($105)) + 78|0);
      $107 = HEAP16[$106>>1]|0;
      $108 = $107&65535;
      $109 = $108 | 4;
      $110 = $109&65535;
      HEAP16[$106>>1] = $110;
     }
    } while(0);
    $111 = HEAP32[$1>>2]|0;
    $112 = ((($111)) + 66|0);
    $113 = HEAP8[$112>>0]|0;
    $114 = HEAP32[$pDb>>2]|0;
    $115 = ((($114)) + 12|0);
    $116 = HEAP32[$115>>2]|0;
    $117 = ((($116)) + 77|0);
    HEAP8[$117>>0] = $113;
    $118 = HEAP32[$pDb>>2]|0;
    $119 = ((($118)) + 12|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = ((($120)) + 80|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = ($122|0)==(0);
    if ($123) {
     $124 = ((($meta)) + 8|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = (_sqlite3AbsInt32($125)|0);
     HEAP32[$size>>2] = $126;
     $127 = HEAP32[$size>>2]|0;
     $128 = ($127|0)==(0);
     $$1 = $128 ? -2000 : $126;
     HEAP32[$size>>2] = $$1;
     $129 = HEAP32[$size>>2]|0;
     $130 = HEAP32[$pDb>>2]|0;
     $131 = ((($130)) + 12|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = ((($132)) + 80|0);
     HEAP32[$133>>2] = $129;
     $134 = HEAP32[$pDb>>2]|0;
     $135 = ((($134)) + 4|0);
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$pDb>>2]|0;
     $138 = ((($137)) + 12|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ((($139)) + 80|0);
     $141 = HEAP32[$140>>2]|0;
     (_sqlite3BtreeSetCacheSize($136,$141)|0);
    }
    $142 = ((($meta)) + 4|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = $143&255;
    $145 = HEAP32[$pDb>>2]|0;
    $146 = ((($145)) + 12|0);
    $147 = HEAP32[$146>>2]|0;
    $148 = ((($147)) + 76|0);
    HEAP8[$148>>0] = $144;
    $149 = HEAP32[$pDb>>2]|0;
    $150 = ((($149)) + 12|0);
    $151 = HEAP32[$150>>2]|0;
    $152 = ((($151)) + 76|0);
    $153 = HEAP8[$152>>0]|0;
    $154 = $153&255;
    $155 = ($154|0)==(0);
    if ($155) {
     $156 = HEAP32[$pDb>>2]|0;
     $157 = ((($156)) + 12|0);
     $158 = HEAP32[$157>>2]|0;
     $159 = ((($158)) + 76|0);
     HEAP8[$159>>0] = 1;
    }
    $160 = HEAP32[$pDb>>2]|0;
    $161 = ((($160)) + 12|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = ((($162)) + 76|0);
    $164 = HEAP8[$163>>0]|0;
    $165 = $164&255;
    $166 = ($165|0)>(4);
    if ($166) {
     $167 = HEAP32[$3>>2]|0;
     $168 = HEAP32[$1>>2]|0;
     _sqlite3SetString($167,$168,20084);
     HEAP32[$rc>>2] = 1;
     break;
    }
    $169 = HEAP32[$2>>2]|0;
    $170 = ($169|0)==(0);
    if ($170) {
     $171 = ((($meta)) + 4|0);
     $172 = HEAP32[$171>>2]|0;
     $173 = ($172|0)>=(4);
     if ($173) {
      $174 = HEAP32[$1>>2]|0;
      $175 = ((($174)) + 24|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = $176 & -32769;
      HEAP32[$175>>2] = $177;
     }
    }
    $178 = HEAP32[$1>>2]|0;
    $179 = HEAP32[$2>>2]|0;
    $180 = HEAP32[$1>>2]|0;
    $181 = ((($180)) + 16|0);
    $182 = HEAP32[$181>>2]|0;
    $183 = (($182) + ($179<<4)|0);
    $184 = HEAP32[$183>>2]|0;
    $185 = HEAP32[$zMasterName>>2]|0;
    HEAP32[$vararg_buffer>>2] = $184;
    $vararg_ptr3 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr3>>2] = $185;
    $186 = (_sqlite3MPrintf($178,20108,$vararg_buffer)|0);
    HEAP32[$zSql>>2] = $186;
    $187 = HEAP32[$1>>2]|0;
    $188 = ((($187)) + 296|0);
    $189 = HEAP32[$188>>2]|0;
    HEAP32[$xAuth>>2] = $189;
    $190 = HEAP32[$1>>2]|0;
    $191 = ((($190)) + 296|0);
    HEAP32[$191>>2] = 0;
    $192 = HEAP32[$1>>2]|0;
    $193 = HEAP32[$zSql>>2]|0;
    $194 = (_sqlite3_exec($192,$193,140,$initData,0)|0);
    HEAP32[$rc>>2] = $194;
    $195 = HEAP32[$xAuth>>2]|0;
    $196 = HEAP32[$1>>2]|0;
    $197 = ((($196)) + 296|0);
    HEAP32[$197>>2] = $195;
    $198 = HEAP32[$rc>>2]|0;
    $199 = ($198|0)==(0);
    if ($199) {
     $200 = ((($initData)) + 12|0);
     $201 = HEAP32[$200>>2]|0;
     HEAP32[$rc>>2] = $201;
    }
    $202 = HEAP32[$1>>2]|0;
    $203 = HEAP32[$zSql>>2]|0;
    _sqlite3DbFree($202,$203);
    $204 = HEAP32[$rc>>2]|0;
    $205 = ($204|0)==(0);
    if ($205) {
     $206 = HEAP32[$1>>2]|0;
     $207 = HEAP32[$2>>2]|0;
     (_sqlite3AnalysisLoad($206,$207)|0);
    }
    $208 = HEAP32[$1>>2]|0;
    $209 = ((($208)) + 69|0);
    $210 = HEAP8[$209>>0]|0;
    $211 = ($210<<24>>24)!=(0);
    if ($211) {
     HEAP32[$rc>>2] = 7;
     $212 = HEAP32[$1>>2]|0;
     _sqlite3ResetAllSchemasOfConnection($212);
    }
    $213 = HEAP32[$rc>>2]|0;
    $214 = ($213|0)==(0);
    if (!($214)) {
     $215 = HEAP32[$1>>2]|0;
     $216 = ((($215)) + 24|0);
     $217 = HEAP32[$216>>2]|0;
     $218 = $217 & 65536;
     $219 = ($218|0)!=(0);
     if (!($219)) {
      break;
     }
    }
    $220 = HEAP32[$2>>2]|0;
    $221 = HEAP32[$1>>2]|0;
    $222 = ((($221)) + 16|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = (($223) + ($220<<4)|0);
    $225 = ((($224)) + 12|0);
    $226 = HEAP32[$225>>2]|0;
    $227 = ((($226)) + 78|0);
    $228 = HEAP16[$227>>1]|0;
    $229 = $228&65535;
    $230 = $229 | 1;
    $231 = $230&65535;
    HEAP16[$227>>1] = $231;
    HEAP32[$rc>>2] = 0;
   }
  } while(0);
  $232 = HEAP32[$openedTransaction>>2]|0;
  $233 = ($232|0)!=(0);
  if ($233) {
   $234 = HEAP32[$pDb>>2]|0;
   $235 = ((($234)) + 4|0);
   $236 = HEAP32[$235>>2]|0;
   (_sqlite3BtreeCommit($236)|0);
  }
 }
 $237 = HEAP32[$rc>>2]|0;
 $238 = ($237|0)==(7);
 $239 = HEAP32[$rc>>2]|0;
 $240 = ($239|0)==(3082);
 $or$cond = $238 | $240;
 if ($or$cond) {
  $241 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($241);
 }
 $242 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $242;
 $243 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($243|0);
}
function _sqlite3ResetOneSchema($db,$iDb) {
 $db = $db|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDb = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2<<4)|0);
 HEAP32[$pDb>>2] = $6;
 $7 = HEAP32[$pDb>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 _sqlite3SchemaClear($9);
 $10 = HEAP32[$1>>2]|0;
 $11 = ($10|0)!=(1);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP32[$pDb>>2] = $15;
 $16 = HEAP32[$pDb>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3SchemaClear($18);
 STACKTOP = sp;return;
}
function _sqlite3CommitInternalChanges($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 & -3;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3SchemaClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pElem = 0, $pSchema = 0, $pTab = 0, $temp1 = 0, $temp2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $temp1 = sp + 32|0;
 $temp2 = sp + 16|0;
 $pElem = sp + 8|0;
 $pSchema = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pSchema>>2] = $1;
 $2 = HEAP32[$pSchema>>2]|0;
 $3 = ((($2)) + 8|0);
 ;HEAP32[$temp1>>2]=HEAP32[$3>>2]|0;HEAP32[$temp1+4>>2]=HEAP32[$3+4>>2]|0;HEAP32[$temp1+8>>2]=HEAP32[$3+8>>2]|0;HEAP32[$temp1+12>>2]=HEAP32[$3+12>>2]|0;
 $4 = HEAP32[$pSchema>>2]|0;
 $5 = ((($4)) + 40|0);
 ;HEAP32[$temp2>>2]=HEAP32[$5>>2]|0;HEAP32[$temp2+4>>2]=HEAP32[$5+4>>2]|0;HEAP32[$temp2+8>>2]=HEAP32[$5+8>>2]|0;HEAP32[$temp2+12>>2]=HEAP32[$5+12>>2]|0;
 $6 = HEAP32[$pSchema>>2]|0;
 $7 = ((($6)) + 40|0);
 _sqlite3HashInit($7);
 $8 = HEAP32[$pSchema>>2]|0;
 $9 = ((($8)) + 24|0);
 _sqlite3HashClear($9);
 $10 = ((($temp2)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pElem>>2] = $11;
 while(1) {
  $12 = HEAP32[$pElem>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pElem>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  _sqlite3DeleteTrigger(0,$16);
  $17 = HEAP32[$pElem>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$pElem>>2] = $18;
 }
 _sqlite3HashClear($temp2);
 $19 = HEAP32[$pSchema>>2]|0;
 $20 = ((($19)) + 8|0);
 _sqlite3HashInit($20);
 $21 = ((($temp1)) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pElem>>2] = $22;
 while(1) {
  $23 = HEAP32[$pElem>>2]|0;
  $24 = ($23|0)!=(0|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$pElem>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pTab>>2] = $27;
  $28 = HEAP32[$pTab>>2]|0;
  _sqlite3DeleteTable(0,$28);
  $29 = HEAP32[$pElem>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pElem>>2] = $30;
 }
 _sqlite3HashClear($temp1);
 $31 = HEAP32[$pSchema>>2]|0;
 $32 = ((($31)) + 56|0);
 _sqlite3HashClear($32);
 $33 = HEAP32[$pSchema>>2]|0;
 $34 = ((($33)) + 72|0);
 HEAP32[$34>>2] = 0;
 $35 = HEAP32[$pSchema>>2]|0;
 $36 = ((($35)) + 78|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37&65535;
 $39 = $38 & 1;
 $40 = ($39|0)!=(0);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$pSchema>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + 1)|0;
 HEAP32[$42>>2] = $44;
 $45 = HEAP32[$pSchema>>2]|0;
 $46 = ((($45)) + 78|0);
 $47 = HEAP16[$46>>1]|0;
 $48 = $47&65535;
 $49 = $48 & -2;
 $50 = $49&65535;
 HEAP16[$46>>1] = $50;
 STACKTOP = sp;return;
}
function _sqlite3HashInit($pNew) {
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pNew;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3HashClear($pH) {
 $pH = $pH|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $elem = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $elem = sp + 4|0;
 $next_elem = sp;
 HEAP32[$0>>2] = $pH;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$elem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3_free($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 12|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 HEAP32[$11>>2] = 0;
 while(1) {
  $12 = HEAP32[$elem>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$elem>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$next_elem>>2] = $15;
  $16 = HEAP32[$elem>>2]|0;
  _sqlite3_free($16);
  $17 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 4|0);
 HEAP32[$19>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3InitCallback($pInit,$argc,$argv,$NotUsed) {
 $pInit = $pInit|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $pData = 0, $pIndex = 0;
 var $pStmt = 0, $rc = 0, $saved_iDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $pData = sp + 20|0;
 $db = sp + 16|0;
 $iDb = sp + 12|0;
 $rc = sp + 8|0;
 $saved_iDb = sp + 44|0;
 $pStmt = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $pInit;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $NotUsed;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pData>>2] = $5;
 $6 = HEAP32[$pData>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$pData>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$iDb>>2] = $10;
 $11 = HEAP32[$iDb>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<4)|0);
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 78|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & -5;
 $22 = $21&65535;
 HEAP16[$18>>1] = $22;
 $23 = HEAP32[$db>>2]|0;
 $24 = ((($23)) + 69|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = ($25<<24>>24)!=(0);
 if ($26) {
  $27 = HEAP32[$pData>>2]|0;
  $28 = HEAP32[$3>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  _corruptSchema($27,$29,0);
  HEAP32[$0>>2] = 1;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 $30 = HEAP32[$3>>2]|0;
 $31 = ($30|0)==(0|0);
 if ($31) {
  HEAP32[$0>>2] = 0;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 $32 = HEAP32[$3>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0|0);
 L9: do {
  if ($35) {
   $36 = HEAP32[$pData>>2]|0;
   $37 = HEAP32[$3>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   _corruptSchema($36,$38,0);
  } else {
   $39 = HEAP32[$3>>2]|0;
   $40 = ((($39)) + 8|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (_sqlite3_strnicmp($41,20250,7)|0);
   $43 = ($42|0)==(0);
   if (!($43)) {
    $97 = HEAP32[$3>>2]|0;
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)==(0|0);
    do {
     if (!($99)) {
      $100 = HEAP32[$3>>2]|0;
      $101 = ((($100)) + 8|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = ($102|0)!=(0|0);
      if ($103) {
       $104 = HEAP32[$3>>2]|0;
       $105 = ((($104)) + 8|0);
       $106 = HEAP32[$105>>2]|0;
       $107 = HEAP8[$106>>0]|0;
       $108 = $107 << 24 >> 24;
       $109 = ($108|0)!=(0);
       if ($109) {
        break;
       }
      }
      $113 = HEAP32[$db>>2]|0;
      $114 = HEAP32[$3>>2]|0;
      $115 = HEAP32[$114>>2]|0;
      $116 = HEAP32[$iDb>>2]|0;
      $117 = HEAP32[$db>>2]|0;
      $118 = ((($117)) + 16|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = (($119) + ($116<<4)|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = (_sqlite3FindIndex($113,$115,$121)|0);
      HEAP32[$pIndex>>2] = $122;
      $123 = HEAP32[$pIndex>>2]|0;
      $124 = ($123|0)==(0|0);
      if ($124) {
       break L9;
      }
      $125 = HEAP32[$3>>2]|0;
      $126 = ((($125)) + 4|0);
      $127 = HEAP32[$126>>2]|0;
      $128 = HEAP32[$pIndex>>2]|0;
      $129 = ((($128)) + 44|0);
      $130 = (_sqlite3GetInt32($127,$129)|0);
      $131 = ($130|0)==(0);
      if (!($131)) {
       break L9;
      }
      $132 = HEAP32[$pData>>2]|0;
      $133 = HEAP32[$3>>2]|0;
      $134 = HEAP32[$133>>2]|0;
      _corruptSchema($132,$134,20258);
      break L9;
     }
    } while(0);
    $110 = HEAP32[$pData>>2]|0;
    $111 = HEAP32[$3>>2]|0;
    $112 = HEAP32[$111>>2]|0;
    _corruptSchema($110,$112,0);
    break;
   }
   $44 = HEAP32[$db>>2]|0;
   $45 = ((($44)) + 148|0);
   $46 = ((($45)) + 4|0);
   $47 = HEAP8[$46>>0]|0;
   HEAP8[$saved_iDb>>0] = $47;
   $48 = HEAP32[$iDb>>2]|0;
   $49 = $48&255;
   $50 = HEAP32[$db>>2]|0;
   $51 = ((($50)) + 148|0);
   $52 = ((($51)) + 4|0);
   HEAP8[$52>>0] = $49;
   $53 = HEAP32[$3>>2]|0;
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (_sqlite3Atoi($55)|0);
   $57 = HEAP32[$db>>2]|0;
   $58 = ((($57)) + 148|0);
   HEAP32[$58>>2] = $56;
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 148|0);
   $61 = ((($60)) + 6|0);
   HEAP8[$61>>0] = 0;
   $62 = HEAP32[$db>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = ((($63)) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   (_sqlite3_prepare($62,$65,-1,$pStmt,0)|0);
   $66 = HEAP32[$db>>2]|0;
   $67 = ((($66)) + 52|0);
   $68 = HEAP32[$67>>2]|0;
   HEAP32[$rc>>2] = $68;
   $69 = HEAP8[$saved_iDb>>0]|0;
   $70 = HEAP32[$db>>2]|0;
   $71 = ((($70)) + 148|0);
   $72 = ((($71)) + 4|0);
   HEAP8[$72>>0] = $69;
   $73 = HEAP32[$rc>>2]|0;
   $74 = (0)!=($73|0);
   do {
    if ($74) {
     $75 = HEAP32[$db>>2]|0;
     $76 = ((($75)) + 148|0);
     $77 = ((($76)) + 6|0);
     $78 = HEAP8[$77>>0]|0;
     $79 = ($78<<24>>24)!=(0);
     if (!($79)) {
      $80 = HEAP32[$rc>>2]|0;
      $81 = HEAP32[$pData>>2]|0;
      $82 = ((($81)) + 12|0);
      HEAP32[$82>>2] = $80;
      $83 = HEAP32[$rc>>2]|0;
      $84 = ($83|0)==(7);
      if ($84) {
       $85 = HEAP32[$db>>2]|0;
       _sqlite3OomFault($85);
       break;
      }
      $86 = HEAP32[$rc>>2]|0;
      $87 = ($86|0)!=(9);
      if ($87) {
       $88 = HEAP32[$rc>>2]|0;
       $89 = $88 & 255;
       $90 = ($89|0)!=(6);
       if ($90) {
        $91 = HEAP32[$pData>>2]|0;
        $92 = HEAP32[$3>>2]|0;
        $93 = HEAP32[$92>>2]|0;
        $94 = HEAP32[$db>>2]|0;
        $95 = (_sqlite3_errmsg($94)|0);
        _corruptSchema($91,$93,$95);
       }
      }
     }
    }
   } while(0);
   $96 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_finalize($96)|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $135 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _sqlite3SetString($pz,$db,$zNew) {
 $pz = $pz|0;
 $db = $db|0;
 $zNew = $zNew|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pz;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 _sqlite3DbFree($3,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (_sqlite3DbStrDup($6,$7)|0);
 $9 = HEAP32[$0>>2]|0;
 HEAP32[$9>>2] = $8;
 STACKTOP = sp;return;
}
function _sqlite3AbsInt32($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 $4 = HEAP32[$1>>2]|0;
 do {
  if ($3) {
   HEAP32[$0>>2] = $4;
  } else {
   $5 = ($4|0)==(-2147483648);
   if ($5) {
    HEAP32[$0>>2] = 2147483647;
    break;
   } else {
    $6 = HEAP32[$1>>2]|0;
    $7 = (0 - ($6))|0;
    HEAP32[$0>>2] = $7;
    break;
   }
  }
 } while(0);
 $8 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3BtreeSetCacheSize($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3PagerSetCachesize($7,$8);
 STACKTOP = sp;return 0;
}
function _sqlite3AnalysisLoad($db,$iDb) {
 $db = $db|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $i = 0, $pIdx = 0, $pIdx1 = 0, $rc = 0, $sInfo = 0, $vararg_buffer = 0;
 var $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $sInfo = sp + 24|0;
 $i = sp + 20|0;
 $zSql = sp + 16|0;
 $rc = sp + 12|0;
 $pIdx = sp + 8|0;
 $pIdx1 = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2<<4)|0);
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$i>>2] = $11;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pIdx>>2] = $16;
  $17 = HEAP32[$pIdx>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP16[$19>>1] = 0;
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$i>>2] = $21;
 }
 $22 = HEAP32[$0>>2]|0;
 HEAP32[$sInfo>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($23<<4)|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ((($sInfo)) + 4|0);
 HEAP32[$29>>2] = $28;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($sInfo)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (_sqlite3FindTable($30,20163,$32)|0);
 $34 = ($33|0)!=(0|0);
 do {
  if ($34) {
   $35 = HEAP32[$0>>2]|0;
   $36 = ((($sInfo)) + 4|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$vararg_buffer>>2] = $37;
   $38 = (_sqlite3MPrintf($35,20176,$vararg_buffer)|0);
   HEAP32[$zSql>>2] = $38;
   $39 = HEAP32[$zSql>>2]|0;
   $40 = ($39|0)==(0|0);
   if ($40) {
    HEAP32[$rc>>2] = 7;
    break;
   } else {
    $41 = HEAP32[$0>>2]|0;
    $42 = HEAP32[$zSql>>2]|0;
    $43 = (_sqlite3_exec($41,$42,141,$sInfo,0)|0);
    HEAP32[$rc>>2] = $43;
    $44 = HEAP32[$0>>2]|0;
    $45 = HEAP32[$zSql>>2]|0;
    _sqlite3DbFree($44,$45);
    break;
   }
  }
 } while(0);
 $46 = HEAP32[$1>>2]|0;
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 16|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + ($46<<4)|0);
 $51 = ((($50)) + 12|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ((($52)) + 24|0);
 $54 = ((($53)) + 8|0);
 $55 = HEAP32[$54>>2]|0;
 HEAP32[$i>>2] = $55;
 while(1) {
  $56 = HEAP32[$i>>2]|0;
  $57 = ($56|0)!=(0|0);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$i>>2]|0;
  $59 = ((($58)) + 8|0);
  $60 = HEAP32[$59>>2]|0;
  HEAP32[$pIdx1>>2] = $60;
  $61 = HEAP32[$pIdx1>>2]|0;
  $62 = ((($61)) + 8|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP16[$63>>1]|0;
  $65 = $64 << 16 >> 16;
  $66 = ($65|0)==(0);
  if ($66) {
   $67 = HEAP32[$pIdx1>>2]|0;
   _sqlite3DefaultRowEst($67);
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  HEAP32[$i>>2] = $69;
 }
 $70 = HEAP32[$rc>>2]|0;
 $71 = ($70|0)==(7);
 if (!($71)) {
  $73 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $72 = HEAP32[$0>>2]|0;
 _sqlite3OomFault($72);
 $73 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _sqlite3BtreeCommit($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3BtreeCommitPhaseOne($2,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $8 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3BtreeCommitPhaseTwo($6,0)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _analysisLoader($pData,$argc,$argv,$NotUsed) {
 $pData = $pData|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $aiRowEst = 0, $fakeIdx = 0, $nCol = 0, $pIndex = 0, $pInfo = 0, $pTable = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $pInfo = sp + 76|0;
 $pIndex = sp + 72|0;
 $pTable = sp + 68|0;
 $z = sp + 64|0;
 $aiRowEst = sp + 60|0;
 $nCol = sp + 56|0;
 $fakeIdx = sp;
 HEAP32[$1>>2] = $pData;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $NotUsed;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pInfo>>2] = $5;
 $6 = HEAP32[$3>>2]|0;
 $7 = ($6|0)==(0|0);
 if (!($7)) {
  $8 = HEAP32[$3>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0|0);
  if (!($10)) {
   $11 = HEAP32[$3>>2]|0;
   $12 = ((($11)) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)==(0|0);
   if (!($14)) {
    $15 = HEAP32[$pInfo>>2]|0;
    $16 = HEAP32[$15>>2]|0;
    $17 = HEAP32[$3>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$pInfo>>2]|0;
    $20 = ((($19)) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = (_sqlite3FindTable($16,$18,$21)|0);
    HEAP32[$pTable>>2] = $22;
    $23 = HEAP32[$pTable>>2]|0;
    $24 = ($23|0)==(0|0);
    if ($24) {
     HEAP32[$0>>2] = 0;
     $89 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($89|0);
    }
    $25 = HEAP32[$3>>2]|0;
    $26 = ((($25)) + 4|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = ($27|0)==(0|0);
    do {
     if ($28) {
      HEAP32[$pIndex>>2] = 0;
     } else {
      $29 = HEAP32[$3>>2]|0;
      $30 = HEAP32[$29>>2]|0;
      $31 = HEAP32[$3>>2]|0;
      $32 = ((($31)) + 4|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = (_sqlite3_stricmp($30,$33)|0);
      $35 = ($34|0)==(0);
      if ($35) {
       $36 = HEAP32[$pTable>>2]|0;
       $37 = (_sqlite3PrimaryKeyIndex($36)|0);
       HEAP32[$pIndex>>2] = $37;
       break;
      } else {
       $38 = HEAP32[$pInfo>>2]|0;
       $39 = HEAP32[$38>>2]|0;
       $40 = HEAP32[$3>>2]|0;
       $41 = ((($40)) + 4|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = HEAP32[$pInfo>>2]|0;
       $44 = ((($43)) + 4|0);
       $45 = HEAP32[$44>>2]|0;
       $46 = (_sqlite3FindIndex($39,$42,$45)|0);
       HEAP32[$pIndex>>2] = $46;
       break;
      }
     }
    } while(0);
    $47 = HEAP32[$3>>2]|0;
    $48 = ((($47)) + 8|0);
    $49 = HEAP32[$48>>2]|0;
    HEAP32[$z>>2] = $49;
    $50 = HEAP32[$pIndex>>2]|0;
    $51 = ($50|0)!=(0|0);
    if ($51) {
     HEAP32[$aiRowEst>>2] = 0;
     $52 = HEAP32[$pIndex>>2]|0;
     $53 = ((($52)) + 50|0);
     $54 = HEAP16[$53>>1]|0;
     $55 = $54&65535;
     $56 = (($55) + 1)|0;
     HEAP32[$nCol>>2] = $56;
     $57 = HEAP32[$pIndex>>2]|0;
     $58 = ((($57)) + 55|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59 & -5;
     HEAP8[$58>>0] = $60;
     $61 = HEAP32[$z>>2]|0;
     $62 = HEAP32[$nCol>>2]|0;
     $63 = HEAP32[$aiRowEst>>2]|0;
     $64 = HEAP32[$pIndex>>2]|0;
     $65 = ((($64)) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$pIndex>>2]|0;
     _decodeIntArray($61,$62,$63,$66,$67);
     $68 = HEAP32[$pIndex>>2]|0;
     $69 = ((($68)) + 36|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($70|0)==(0|0);
     if ($71) {
      $72 = HEAP32[$pIndex>>2]|0;
      $73 = ((($72)) + 8|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = HEAP16[$74>>1]|0;
      $76 = HEAP32[$pTable>>2]|0;
      $77 = ((($76)) + 38|0);
      HEAP16[$77>>1] = $75;
     }
    } else {
     $78 = HEAP32[$pTable>>2]|0;
     $79 = ((($78)) + 40|0);
     $80 = HEAP16[$79>>1]|0;
     $81 = ((($fakeIdx)) + 48|0);
     HEAP16[$81>>1] = $80;
     $82 = HEAP32[$z>>2]|0;
     $83 = HEAP32[$pTable>>2]|0;
     $84 = ((($83)) + 38|0);
     _decodeIntArray($82,1,0,$84,$fakeIdx);
     $85 = ((($fakeIdx)) + 48|0);
     $86 = HEAP16[$85>>1]|0;
     $87 = HEAP32[$pTable>>2]|0;
     $88 = ((($87)) + 40|0);
     HEAP16[$88>>1] = $86;
    }
    HEAP32[$0>>2] = 0;
    $89 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($89|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $89 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($89|0);
}
function _sqlite3DefaultRowEst($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $a = 0, $aVal = 0, $i = 0, $nCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $aVal = sp + 16|0;
 $a = sp + 8|0;
 $nCopy = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pIdx;
 ;HEAP16[$aVal>>1]=HEAP16[9772>>1]|0;HEAP16[$aVal+2>>1]=HEAP16[9772+2>>1]|0;HEAP16[$aVal+4>>1]=HEAP16[9772+4>>1]|0;HEAP16[$aVal+6>>1]=HEAP16[9772+6>>1]|0;HEAP16[$aVal+8>>1]=HEAP16[9772+8>>1]|0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$a>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 50|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = (5)<($7|0);
 if ($8) {
  $13 = 5;
 } else {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 50|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = $12;
 }
 HEAP32[$nCopy>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 38|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = HEAP32[$a>>2]|0;
 HEAP16[$19>>1] = $18;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 36|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$a>>2]|0;
  $25 = HEAP16[$24>>1]|0;
  $26 = $25 << 16 >> 16;
  $27 = (($26) - 10)|0;
  $28 = $27&65535;
  HEAP16[$24>>1] = $28;
 }
 $29 = HEAP32[$a>>2]|0;
 $30 = HEAP16[$29>>1]|0;
 $31 = $30 << 16 >> 16;
 $32 = ($31|0)<(33);
 if ($32) {
  $33 = HEAP32[$a>>2]|0;
  HEAP16[$33>>1] = 33;
 }
 $34 = HEAP32[$a>>2]|0;
 $35 = ((($34)) + 2|0);
 $36 = HEAP32[$nCopy>>2]|0;
 $37 = $36<<1;
 _memcpy(($35|0),($aVal|0),($37|0))|0;
 $38 = HEAP32[$nCopy>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$i>>2] = $39;
 while(1) {
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$0>>2]|0;
  $42 = ((($41)) + 50|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43&65535;
  $45 = ($40|0)<=($44|0);
  if (!($45)) {
   break;
  }
  $46 = HEAP32[$i>>2]|0;
  $47 = HEAP32[$a>>2]|0;
  $48 = (($47) + ($46<<1)|0);
  HEAP16[$48>>1] = 23;
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 54|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = ($54|0)!=(0);
 if (!($55)) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 50|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = HEAP32[$a>>2]|0;
 $61 = (($60) + ($59<<1)|0);
 HEAP16[$61>>1] = 0;
 STACKTOP = sp;return;
}
function _sqlite3PrimaryKeyIndex($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   label = 5;
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 55|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 & 3;
  $10 = $9&255;
  $11 = ($10|0)==(2);
  $12 = $11 ^ 1;
  if (!($12)) {
   label = 5;
   break;
  }
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$p>>2] = $15;
 }
 if ((label|0) == 5) {
  $16 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _sqlite3FindIndex($db,$zName,$zDb) {
 $db = $db|0;
 $zName = $zName|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $j = 0, $p = 0, $pSchema = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pSchema = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 7;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)<(2);
  $10 = HEAP32[$i>>2]|0;
  $11 = $10 ^ 1;
  $12 = $9 ? $11 : $10;
  HEAP32[$j>>2] = $12;
  $13 = HEAP32[$j>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  $18 = ((($17)) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pSchema>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $22 = HEAP32[$2>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + ($23<<4)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (_sqlite3StrICmp($22,$28)|0);
   $30 = ($29|0)!=(0);
   if (!($30)) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $31 = HEAP32[$pSchema>>2]|0;
   $32 = ((($31)) + 24|0);
   $33 = HEAP32[$1>>2]|0;
   $34 = (_sqlite3HashFind($32,$33)|0);
   HEAP32[$p>>2] = $34;
   $35 = HEAP32[$p>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    label = 7;
    break;
   }
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 if ((label|0) == 7) {
  $39 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 return (0)|0;
}
function _decodeIntArray($zIntArray,$nOut,$aOut,$aLog,$pIndex) {
 $zIntArray = $zIntArray|0;
 $nOut = $nOut|0;
 $aOut = $aOut|0;
 $aLog = $aLog|0;
 $pIndex = $pIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $c = 0, $i = 0, $v = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $z = sp + 12|0;
 $c = sp + 8|0;
 $i = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $zIntArray;
 HEAP32[$1>>2] = $nOut;
 HEAP32[$2>>2] = $aOut;
 HEAP32[$3>>2] = $aLog;
 HEAP32[$4>>2] = $pIndex;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$z>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)!=(0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  HEAP32[$v>>2] = 0;
  while(1) {
   $13 = HEAP32[$z>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   HEAP32[$c>>2] = $15;
   $16 = ($15|0)>=(48);
   $17 = HEAP32[$c>>2]|0;
   $18 = ($17|0)<=(57);
   $19 = $16 ? $18 : 0;
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$v>>2]|0;
   $21 = ($20*10)|0;
   $22 = HEAP32[$c>>2]|0;
   $23 = (($21) + ($22))|0;
   $24 = (($23) - 48)|0;
   HEAP32[$v>>2] = $24;
   $25 = HEAP32[$z>>2]|0;
   $26 = ((($25)) + 1|0);
   HEAP32[$z>>2] = $26;
  }
  $27 = HEAP32[$v>>2]|0;
  $28 = (_sqlite3LogEst($27,0)|0);
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = (($30) + ($29<<1)|0);
  HEAP16[$31>>1] = $28;
  $32 = HEAP32[$z>>2]|0;
  $33 = HEAP8[$32>>0]|0;
  $34 = $33 << 24 >> 24;
  $35 = ($34|0)==(32);
  if ($35) {
   $36 = HEAP32[$z>>2]|0;
   $37 = ((($36)) + 1|0);
   HEAP32[$z>>2] = $37;
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[$i>>2] = $39;
 }
 $40 = HEAP32[$4>>2]|0;
 $41 = ((($40)) + 55|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42 & -5;
 HEAP8[$41>>0] = $43;
 $44 = HEAP32[$4>>2]|0;
 $45 = ((($44)) + 55|0);
 $46 = HEAP8[$45>>0]|0;
 $47 = $46 & -65;
 HEAP8[$45>>0] = $47;
 L13: while(1) {
  $48 = HEAP32[$z>>2]|0;
  $49 = HEAP8[$48>>0]|0;
  $50 = ($49<<24>>24)!=(0);
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$z>>2]|0;
  $52 = (_sqlite3_strglob(20217,$51)|0);
  $53 = ($52|0)==(0);
  do {
   if ($53) {
    $54 = HEAP32[$4>>2]|0;
    $55 = ((($54)) + 55|0);
    $56 = HEAP8[$55>>0]|0;
    $57 = $56 & -5;
    $58 = $57 | 4;
    HEAP8[$55>>0] = $58;
   } else {
    $59 = HEAP32[$z>>2]|0;
    $60 = (_sqlite3_strglob(20228,$59)|0);
    $61 = ($60|0)==(0);
    $62 = HEAP32[$z>>2]|0;
    if ($61) {
     $63 = ((($62)) + 3|0);
     $64 = (_sqlite3Atoi($63)|0);
     $65 = ($64|0)<(0);
     $66 = $65 << 31 >> 31;
     $67 = (_sqlite3LogEst($64,$66)|0);
     $68 = HEAP32[$4>>2]|0;
     $69 = ((($68)) + 48|0);
     HEAP16[$69>>1] = $67;
     break;
    }
    $70 = (_sqlite3_strglob(20238,$62)|0);
    $71 = ($70|0)==(0);
    if ($71) {
     $72 = HEAP32[$4>>2]|0;
     $73 = ((($72)) + 55|0);
     $74 = HEAP8[$73>>0]|0;
     $75 = $74 & -65;
     $76 = $75 | 64;
     HEAP8[$73>>0] = $76;
    }
   }
  } while(0);
  while(1) {
   $77 = HEAP32[$z>>2]|0;
   $78 = HEAP8[$77>>0]|0;
   $79 = $78 << 24 >> 24;
   $80 = ($79|0)!=(0);
   if (!($80)) {
    break;
   }
   $81 = HEAP32[$z>>2]|0;
   $82 = HEAP8[$81>>0]|0;
   $83 = $82 << 24 >> 24;
   $84 = ($83|0)!=(32);
   if (!($84)) {
    break;
   }
   $85 = HEAP32[$z>>2]|0;
   $86 = ((($85)) + 1|0);
   HEAP32[$z>>2] = $86;
  }
  while(1) {
   $87 = HEAP32[$z>>2]|0;
   $88 = HEAP8[$87>>0]|0;
   $89 = $88 << 24 >> 24;
   $90 = ($89|0)==(32);
   if (!($90)) {
    continue L13;
   }
   $91 = HEAP32[$z>>2]|0;
   $92 = ((($91)) + 1|0);
   HEAP32[$z>>2] = $92;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3LogEst($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 10|0;
 $3 = sp;
 $y = sp + 8|0;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP16[$y>>1] = 40;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13>>>0)<(0);
 $15 = ($10>>>0)<(8);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 L1: do {
  if ($18) {
   $19 = $3;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $22 = (($19) + 4)|0;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24>>>0)<(0);
   $26 = ($21>>>0)<(2);
   $27 = ($24|0)==(0);
   $28 = $27 & $26;
   $29 = $25 | $28;
   if ($29) {
    HEAP16[$2>>1] = 0;
    $126 = HEAP16[$2>>1]|0;
    STACKTOP = sp;return ($126|0);
   } else {
    while(1) {
     $30 = $3;
     $31 = $30;
     $32 = HEAP32[$31>>2]|0;
     $33 = (($30) + 4)|0;
     $34 = $33;
     $35 = HEAP32[$34>>2]|0;
     $36 = ($35>>>0)<(0);
     $37 = ($32>>>0)<(8);
     $38 = ($35|0)==(0);
     $39 = $38 & $37;
     $40 = $36 | $39;
     if (!($40)) {
      break L1;
     }
     $41 = HEAP16[$y>>1]|0;
     $42 = $41 << 16 >> 16;
     $43 = (($42) - 10)|0;
     $44 = $43&65535;
     HEAP16[$y>>1] = $44;
     $45 = $3;
     $46 = $45;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($45) + 4)|0;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (_bitshift64Shl(($47|0),($50|0),1)|0);
     $52 = tempRet0;
     $53 = $3;
     $54 = $53;
     HEAP32[$54>>2] = $51;
     $55 = (($53) + 4)|0;
     $56 = $55;
     HEAP32[$56>>2] = $52;
    }
   }
  } else {
   while(1) {
    $57 = $3;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = (($57) + 4)|0;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = ($62>>>0)>(0);
    $64 = ($59>>>0)>(255);
    $65 = ($62|0)==(0);
    $66 = $65 & $64;
    $67 = $63 | $66;
    if (!($67)) {
     break;
    }
    $68 = HEAP16[$y>>1]|0;
    $69 = $68 << 16 >> 16;
    $70 = (($69) + 40)|0;
    $71 = $70&65535;
    HEAP16[$y>>1] = $71;
    $72 = $3;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = (_bitshift64Lshr(($74|0),($77|0),4)|0);
    $79 = tempRet0;
    $80 = $3;
    $81 = $80;
    HEAP32[$81>>2] = $78;
    $82 = (($80) + 4)|0;
    $83 = $82;
    HEAP32[$83>>2] = $79;
   }
   while(1) {
    $84 = $3;
    $85 = $84;
    $86 = HEAP32[$85>>2]|0;
    $87 = (($84) + 4)|0;
    $88 = $87;
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89>>>0)>(0);
    $91 = ($86>>>0)>(15);
    $92 = ($89|0)==(0);
    $93 = $92 & $91;
    $94 = $90 | $93;
    if (!($94)) {
     break L1;
    }
    $95 = HEAP16[$y>>1]|0;
    $96 = $95 << 16 >> 16;
    $97 = (($96) + 10)|0;
    $98 = $97&65535;
    HEAP16[$y>>1] = $98;
    $99 = $3;
    $100 = $99;
    $101 = HEAP32[$100>>2]|0;
    $102 = (($99) + 4)|0;
    $103 = $102;
    $104 = HEAP32[$103>>2]|0;
    $105 = (_bitshift64Lshr(($101|0),($104|0),1)|0);
    $106 = tempRet0;
    $107 = $3;
    $108 = $107;
    HEAP32[$108>>2] = $105;
    $109 = (($107) + 4)|0;
    $110 = $109;
    HEAP32[$110>>2] = $106;
   }
  }
 } while(0);
 $111 = $3;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = $113 & 7;
 $118 = (9782 + ($117<<1)|0);
 $119 = HEAP16[$118>>1]|0;
 $120 = $119 << 16 >> 16;
 $121 = HEAP16[$y>>1]|0;
 $122 = $121 << 16 >> 16;
 $123 = (($120) + ($122))|0;
 $124 = (($123) - 10)|0;
 $125 = $124&65535;
 HEAP16[$2>>1] = $125;
 $126 = HEAP16[$2>>1]|0;
 STACKTOP = sp;return ($126|0);
}
function _sqlite3PagerSetCachesize($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 212|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _sqlite3PcacheSetCachesize($4,$5);
 STACKTOP = sp;return;
}
function _sqlite3PcacheSetCachesize($pCache,$mxPage) {
 $pCache = $pCache|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[(136)>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_numberOfCachePages($9)|0);
 FUNCTION_TABLE_vii[$5 & 255]($8,$10);
 STACKTOP = sp;return;
}
function _corruptSchema($pData,$zObj,$zExtra) {
 $pData = $pData|0;
 $zObj = $zObj|0;
 $zExtra = $zExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $db = sp + 20|0;
 $z = sp + 16|0;
 HEAP32[$0>>2] = $pData;
 HEAP32[$1>>2] = $zObj;
 HEAP32[$2>>2] = $zExtra;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 69|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  $9 = HEAP32[$db>>2]|0;
  $10 = ((($9)) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 & 65536;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ($14|0)==(0|0);
   if ($15) {
    HEAP32[$1>>2] = 20275;
   }
   $16 = HEAP32[$db>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   HEAP32[$vararg_buffer>>2] = $17;
   $18 = (_sqlite3MPrintf($16,20277,$vararg_buffer)|0);
   HEAP32[$z>>2] = $18;
   $19 = HEAP32[$2>>2]|0;
   $20 = ($19|0)!=(0|0);
   if ($20) {
    $21 = HEAP32[$db>>2]|0;
    $22 = HEAP32[$z>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $22;
    $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $23;
    $24 = (_sqlite3MPrintf($21,20308,$vararg_buffer1)|0);
    HEAP32[$z>>2] = $24;
   }
   $25 = HEAP32[$db>>2]|0;
   $26 = HEAP32[$0>>2]|0;
   $27 = ((($26)) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   _sqlite3DbFree($25,$29);
   $30 = HEAP32[$z>>2]|0;
   $31 = HEAP32[$0>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$33>>2] = $30;
  }
 }
 $34 = HEAP32[$db>>2]|0;
 $35 = ((($34)) + 69|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = ($37|0)!=(0);
 if ($38) {
  $42 = 7;
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 12|0);
  HEAP32[$41>>2] = $42;
  STACKTOP = sp;return;
 }
 $39 = (_sqlite3CorruptError(113554)|0);
 $42 = $39;
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 12|0);
 HEAP32[$41>>2] = $42;
 STACKTOP = sp;return;
}
function _sqlite3CorruptError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_reportError(11,$1,20316)|0);
 STACKTOP = sp;return ($2|0);
}
function _reportError($iErr,$lineno,$zType) {
 $iErr = $iErr|0;
 $lineno = $lineno|0;
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 HEAP32[$0>>2] = $iErr;
 HEAP32[$1>>2] = $lineno;
 HEAP32[$2>>2] = $zType;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3_sourceid()|0);
 $7 = ((($6)) + 20|0);
 HEAP32[$vararg_buffer>>2] = $4;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $5;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $7;
 _sqlite3_log($3,20336,$vararg_buffer);
 $8 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3FindCollSeq($db,$enc,$zName,$create) {
 $db = $db|0;
 $enc = $enc|0;
 $zName = $zName|0;
 $create = $create|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 16|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $db;
 HEAP8[$1>>0] = $enc;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $create;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$3>>2]|0;
  $9 = (_findCollSeqEntry($6,$7,$8)|0);
  HEAP32[$pColl>>2] = $9;
 } else {
  $10 = ((($6)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pColl>>2] = $11;
 }
 $12 = HEAP32[$pColl>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $19 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $14 = HEAP8[$1>>0]|0;
 $15 = $14&255;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$pColl>>2]|0;
 $18 = (($17) + (($16*20)|0)|0);
 HEAP32[$pColl>>2] = $18;
 $19 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _findCollSeqEntry($db,$zName,$create) {
 $db = $db|0;
 $zName = $zName|0;
 $create = $create|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $or$cond = 0, $pColl = 0, $pDel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pColl = sp + 8|0;
 $nName = sp + 4|0;
 $pDel = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $create;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 364|0);
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3HashFind($4,$5)|0);
 HEAP32[$pColl>>2] = $6;
 $7 = HEAP32[$pColl>>2]|0;
 $8 = (0|0)==($7|0);
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0);
 $or$cond = $8 & $10;
 if (!($or$cond)) {
  $58 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 HEAP32[$nName>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$nName>>2]|0;
 $15 = (60 + ($14))|0;
 $16 = (($15) + 1)|0;
 $17 = (_sqlite3DbMallocZero($13,$16,0)|0);
 HEAP32[$pColl>>2] = $17;
 $18 = HEAP32[$pColl>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $58 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 HEAP32[$pDel>>2] = 0;
 $20 = HEAP32[$pColl>>2]|0;
 $21 = ((($20)) + 60|0);
 $22 = HEAP32[$pColl>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$pColl>>2]|0;
 $24 = ((($23)) + 4|0);
 HEAP8[$24>>0] = 1;
 $25 = HEAP32[$pColl>>2]|0;
 $26 = ((($25)) + 60|0);
 $27 = HEAP32[$pColl>>2]|0;
 $28 = ((($27)) + 20|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pColl>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = ((($30)) + 4|0);
 HEAP8[$31>>0] = 2;
 $32 = HEAP32[$pColl>>2]|0;
 $33 = ((($32)) + 60|0);
 $34 = HEAP32[$pColl>>2]|0;
 $35 = ((($34)) + 40|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pColl>>2]|0;
 $37 = ((($36)) + 40|0);
 $38 = ((($37)) + 4|0);
 HEAP8[$38>>0] = 3;
 $39 = HEAP32[$pColl>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$nName>>2]|0;
 _memcpy(($40|0),($41|0),($42|0))|0;
 $43 = HEAP32[$nName>>2]|0;
 $44 = HEAP32[$pColl>>2]|0;
 $45 = HEAP32[$44>>2]|0;
 $46 = (($45) + ($43)|0);
 HEAP8[$46>>0] = 0;
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 364|0);
 $49 = HEAP32[$pColl>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$pColl>>2]|0;
 $52 = (_sqlite3HashInsert($48,$50,$51)|0);
 HEAP32[$pDel>>2] = $52;
 $53 = HEAP32[$pDel>>2]|0;
 $54 = ($53|0)!=(0|0);
 if (!($54)) {
  $58 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $55 = HEAP32[$0>>2]|0;
 _sqlite3OomFault($55);
 $56 = HEAP32[$0>>2]|0;
 $57 = HEAP32[$pDel>>2]|0;
 _sqlite3DbFree($56,$57);
 HEAP32[$pColl>>2] = 0;
 $58 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _sqlite3HashInsert($pH,$pKey,$data) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $data = $data|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_elem = 0, $old_data = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $h = sp + 12|0;
 $elem = sp + 8|0;
 $new_elem = sp + 4|0;
 $old_data = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $data;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_findElementWithHash($4,$5,$h)|0);
 HEAP32[$elem>>2] = $6;
 $7 = HEAP32[$elem>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$elem>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$old_data>>2] = $11;
  $12 = HEAP32[$3>>2]|0;
  $13 = ($12|0)==(0|0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$elem>>2]|0;
   $16 = HEAP32[$h>>2]|0;
   _removeElementGivenHash($14,$15,$16);
  } else {
   $17 = HEAP32[$3>>2]|0;
   $18 = HEAP32[$elem>>2]|0;
   $19 = ((($18)) + 8|0);
   HEAP32[$19>>2] = $17;
   $20 = HEAP32[$2>>2]|0;
   $21 = HEAP32[$elem>>2]|0;
   $22 = ((($21)) + 12|0);
   HEAP32[$22>>2] = $20;
  }
  $23 = HEAP32[$old_data>>2]|0;
  HEAP32[$0>>2] = $23;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $24 = HEAP32[$3>>2]|0;
 $25 = ($24|0)==(0|0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $26 = (_sqlite3Malloc(16,0)|0);
 HEAP32[$new_elem>>2] = $26;
 $27 = HEAP32[$new_elem>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$3>>2]|0;
  HEAP32[$0>>2] = $29;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$new_elem>>2]|0;
 $32 = ((($31)) + 12|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$3>>2]|0;
 $34 = HEAP32[$new_elem>>2]|0;
 $35 = ((($34)) + 8|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + 1)|0;
 HEAP32[$37>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42>>>0)>=(10);
 if ($43) {
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = $48<<1;
  $50 = ($46>>>0)>($49>>>0);
  if ($50) {
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$1>>2]|0;
   $53 = ((($52)) + 4|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = $54<<1;
   $56 = (_rehash($51,$55)|0);
   $57 = ($56|0)!=(0);
   if ($57) {
    $58 = HEAP32[$2>>2]|0;
    $59 = (_strHash($58)|0);
    $60 = HEAP32[$1>>2]|0;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59>>>0) % ($61>>>0))&-1;
    HEAP32[$h>>2] = $62;
   }
  }
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 12|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0|0);
 if ($67) {
  $68 = HEAP32[$h>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 12|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + ($68<<3)|0);
  $74 = $72;
 } else {
  $74 = 0;
 }
 $73 = HEAP32[$new_elem>>2]|0;
 _insertElement($63,$74,$73);
 HEAP32[$0>>2] = 0;
 $75 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _removeElementGivenHash($pH,$elem,$h) {
 $pH = $pH|0;
 $elem = $elem|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pEntry = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $elem;
 HEAP32[$2>>2] = $h;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 if ($6) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$11>>2] = $8;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  HEAP32[$13>>2] = $8;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  HEAP32[$22>>2] = $19;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<3)|0);
  HEAP32[$pEntry>>2] = $31;
  $32 = HEAP32[$pEntry>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ($34|0)==($35|0);
  if ($36) {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$pEntry>>2]|0;
   $40 = ((($39)) + 4|0);
   HEAP32[$40>>2] = $38;
  }
  $41 = HEAP32[$pEntry>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + -1)|0;
  HEAP32[$41>>2] = $43;
 }
 $44 = HEAP32[$1>>2]|0;
 _sqlite3_free($44);
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 4|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + -1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 4|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)==(0);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$0>>2]|0;
 _sqlite3HashClear($53);
 STACKTOP = sp;return;
}
function _rehash($pH,$new_size) {
 $pH = $pH|0;
 $new_size = $new_size|0;
 var $$new_size = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_ht = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $new_ht = sp + 12|0;
 $elem = sp + 8|0;
 $next_elem = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $new_size;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<3;
 $5 = ($4>>>0)>(1024);
 $$new_size = $5 ? 128 : $new_size;
 HEAP32[$2>>2] = $$new_size;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($6|0)==($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 _sqlite3BeginBenignMalloc();
 $10 = HEAP32[$2>>2]|0;
 $11 = $10<<3;
 $12 = (_sqlite3Malloc($11,0)|0);
 HEAP32[$new_ht>>2] = $12;
 _sqlite3EndBenignMalloc();
 $13 = HEAP32[$new_ht>>2]|0;
 $14 = ($13|0)==(0|0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3_free($17);
 $18 = HEAP32[$new_ht>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 12|0);
 HEAP32[$20>>2] = $18;
 $21 = HEAP32[$new_ht>>2]|0;
 $22 = (_sqlite3MallocSize($21)|0);
 $23 = (($22>>>0) / 8)&-1;
 HEAP32[$2>>2] = $23;
 $24 = HEAP32[$1>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$new_ht>>2]|0;
 $26 = HEAP32[$2>>2]|0;
 $27 = $26<<3;
 _memset(($25|0),0,($27|0))|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$elem>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 8|0);
 HEAP32[$32>>2] = 0;
 while(1) {
  $33 = HEAP32[$elem>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$elem>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_strHash($37)|0);
  $39 = HEAP32[$2>>2]|0;
  $40 = (($38>>>0) % ($39>>>0))&-1;
  HEAP32[$h>>2] = $40;
  $41 = HEAP32[$elem>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$next_elem>>2] = $42;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$h>>2]|0;
  $45 = HEAP32[$new_ht>>2]|0;
  $46 = (($45) + ($44<<3)|0);
  $47 = HEAP32[$elem>>2]|0;
  _insertElement($43,$46,$47);
  $48 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $48;
 }
 HEAP32[$0>>2] = 1;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _insertElement($pH,$pEntry,$pNew) {
 $pH = $pH|0;
 $pEntry = $pEntry|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pHead = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pHead = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pEntry;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0);
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = $10;
  } else {
   $11 = 0;
  }
  HEAP32[$pHead>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$12>>2] = $14;
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 4|0);
  HEAP32[$17>>2] = $15;
 } else {
  HEAP32[$pHead>>2] = 0;
 }
 $18 = HEAP32[$pHead>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($46|0)!=(0|0);
  if ($47) {
   $48 = HEAP32[$2>>2]|0;
   $49 = HEAP32[$0>>2]|0;
   $50 = ((($49)) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ((($51)) + 4|0);
   HEAP32[$52>>2] = $48;
  }
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 4|0);
  HEAP32[$54>>2] = 0;
  $55 = HEAP32[$2>>2]|0;
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 8|0);
  HEAP32[$57>>2] = $55;
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$pHead>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 HEAP32[$21>>2] = $20;
 $22 = HEAP32[$pHead>>2]|0;
 $23 = ((($22)) + 4|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 4|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$pHead>>2]|0;
 $28 = ((($27)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 $31 = HEAP32[$2>>2]|0;
 if ($30) {
  $32 = HEAP32[$pHead>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$34>>2] = $31;
 } else {
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP32[$36>>2] = $31;
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$pHead>>2]|0;
 $39 = ((($38)) + 4|0);
 HEAP32[$39>>2] = $37;
 STACKTOP = sp;return;
}
function _sqlite3BeginBenignMalloc() {
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[1728>>2]|0;
 $1 = ($0|0)!=(0|0);
 if (!($1)) {
  return;
 }
 $2 = HEAP32[1728>>2]|0;
 FUNCTION_TABLE_v[$2 & 255]();
 return;
}
function _sqlite3EndBenignMalloc() {
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(1732)>>2]|0;
 $1 = ($0|0)!=(0|0);
 if (!($1)) {
  return;
 }
 $2 = HEAP32[(1732)>>2]|0;
 FUNCTION_TABLE_v[$2 & 255]();
 return;
}
function _binCollFunc($padFlag,$nKey1,$pKey1,$nKey2,$pKey2) {
 $padFlag = $padFlag|0;
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $nKey2 = $nKey2|0;
 $pKey2 = $pKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $padFlag;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey2;
 HEAP32[$4>>2] = $pKey2;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = ($5|0)<($6|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = $7 ? $8 : $9;
 HEAP32[$n>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = HEAP32[$n>>2]|0;
 $14 = (_memcmp($11,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$n>>2]|0;
  $21 = (($19) + ($20)|0);
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$n>>2]|0;
  $24 = (($22) - ($23))|0;
  $25 = (_allSpaces($21,$24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   $27 = HEAP32[$4>>2]|0;
   $28 = HEAP32[$n>>2]|0;
   $29 = (($27) + ($28)|0);
   $30 = HEAP32[$3>>2]|0;
   $31 = HEAP32[$n>>2]|0;
   $32 = (($30) - ($31))|0;
   $33 = (_allSpaces($29,$32)|0);
   $34 = ($33|0)!=(0);
   if ($34) {
    $38 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 }
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = (($35) - ($36))|0;
 HEAP32[$rc>>2] = $37;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _nocaseCollatingFunc($NotUsed,$nKey1,$pKey1,$nKey2,$pKey2) {
 $NotUsed = $NotUsed|0;
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $nKey2 = $nKey2|0;
 $pKey2 = $pKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $r = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey2;
 HEAP32[$4>>2] = $pKey2;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)<($8|0);
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = $9 ? $10 : $11;
 $13 = (_sqlite3_strnicmp($5,$6,$12)|0);
 HEAP32[$r>>2] = $13;
 $14 = HEAP32[$r>>2]|0;
 $15 = (0)==($14|0);
 if (!($15)) {
  $19 = HEAP32[$r>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($16) - ($17))|0;
 HEAP32[$r>>2] = $18;
 $19 = HEAP32[$r>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3ParseUri($zDefaultVfs,$zUri,$pFlags,$ppVfs,$pzFile,$pzErrMsg) {
 $zDefaultVfs = $zDefaultVfs|0;
 $zUri = $zUri|0;
 $pFlags = $pFlags|0;
 $ppVfs = $ppVfs|0;
 $pzFile = $pzFile|0;
 $pzErrMsg = $pzErrMsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMode = 0, $c = 0, $eState = 0, $flags = 0, $i = 0, $iIn = 0, $iOut = 0, $limit = 0, $mask = 0, $mode = 0, $nByte = 0, $nOpt = 0;
 var $nUri = 0, $nVal = 0, $octet = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer17 = 0, $vararg_buffer21 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr20 = 0, $z = 0, $zFile = 0, $zModeType = 0, $zOpt = 0, $zVal = 0, $zVfs = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer21 = sp + 32|0;
 $vararg_buffer17 = sp + 24|0;
 $vararg_buffer13 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $5 = sp + 120|0;
 $6 = sp + 116|0;
 $rc = sp + 112|0;
 $flags = sp + 108|0;
 $zVfs = sp + 104|0;
 $zFile = sp + 100|0;
 $c = sp + 144|0;
 $nUri = sp + 96|0;
 $zOpt = sp + 92|0;
 $eState = sp + 88|0;
 $iIn = sp + 84|0;
 $iOut = sp + 80|0;
 $nByte = sp;
 $octet = sp + 76|0;
 $nOpt = sp + 72|0;
 $zVal = sp + 68|0;
 $nVal = sp + 64|0;
 $aMode = sp + 60|0;
 $zModeType = sp + 56|0;
 $mask = sp + 52|0;
 $limit = sp + 48|0;
 $i = sp + 44|0;
 $mode = sp + 40|0;
 $z = sp + 36|0;
 HEAP32[$1>>2] = $zDefaultVfs;
 HEAP32[$2>>2] = $zUri;
 HEAP32[$3>>2] = $pFlags;
 HEAP32[$4>>2] = $ppVfs;
 HEAP32[$5>>2] = $pzFile;
 HEAP32[$6>>2] = $pzErrMsg;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$flags>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$zVfs>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_sqlite3Strlen30($10)|0);
 HEAP32[$nUri>>2] = $11;
 $12 = HEAP32[$flags>>2]|0;
 $13 = $12 & 64;
 $14 = ($13|0)!=(0);
 $15 = HEAP32[(20)>>2]|0;
 $16 = ($15|0)!=(0);
 $or$cond = $14 | $16;
 $17 = HEAP32[$nUri>>2]|0;
 $18 = ($17|0)>=(5);
 $or$cond3 = $or$cond & $18;
 L1: do {
  if ($or$cond3) {
   $19 = HEAP32[$2>>2]|0;
   $20 = (_memcmp($19,26023,5)|0);
   $21 = ($20|0)==(0);
   if ($21) {
    HEAP32[$iOut>>2] = 0;
    $22 = HEAP32[$nUri>>2]|0;
    $23 = (($22) + 2)|0;
    $24 = ($23|0)<(0);
    $25 = $24 << 31 >> 31;
    $26 = $nByte;
    $27 = $26;
    HEAP32[$27>>2] = $23;
    $28 = (($26) + 4)|0;
    $29 = $28;
    HEAP32[$29>>2] = $25;
    $30 = HEAP32[$flags>>2]|0;
    $31 = $30 | 64;
    HEAP32[$flags>>2] = $31;
    HEAP32[$iIn>>2] = 0;
    while(1) {
     $32 = HEAP32[$iIn>>2]|0;
     $33 = HEAP32[$nUri>>2]|0;
     $34 = ($32|0)<($33|0);
     if (!($34)) {
      break;
     }
     $35 = HEAP32[$iIn>>2]|0;
     $36 = HEAP32[$2>>2]|0;
     $37 = (($36) + ($35)|0);
     $38 = HEAP8[$37>>0]|0;
     $39 = $38 << 24 >> 24;
     $40 = ($39|0)==(38);
     $41 = $40&1;
     $42 = ($41|0)<(0);
     $43 = $42 << 31 >> 31;
     $44 = $nByte;
     $45 = $44;
     $46 = HEAP32[$45>>2]|0;
     $47 = (($44) + 4)|0;
     $48 = $47;
     $49 = HEAP32[$48>>2]|0;
     $50 = (_i64Add(($46|0),($49|0),($41|0),($43|0))|0);
     $51 = tempRet0;
     $52 = $nByte;
     $53 = $52;
     HEAP32[$53>>2] = $50;
     $54 = (($52) + 4)|0;
     $55 = $54;
     HEAP32[$55>>2] = $51;
     $56 = HEAP32[$iIn>>2]|0;
     $57 = (($56) + 1)|0;
     HEAP32[$iIn>>2] = $57;
    }
    $58 = $nByte;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = (($58) + 4)|0;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (_sqlite3_malloc64($60,$63)|0);
    HEAP32[$zFile>>2] = $64;
    $65 = HEAP32[$zFile>>2]|0;
    $66 = ($65|0)!=(0|0);
    if (!($66)) {
     HEAP32[$0>>2] = 7;
     $386 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($386|0);
    }
    HEAP32[$iIn>>2] = 5;
    $67 = HEAP32[$2>>2]|0;
    $68 = ((($67)) + 5|0);
    $69 = HEAP8[$68>>0]|0;
    $70 = $69 << 24 >> 24;
    $71 = ($70|0)==(47);
    do {
     if ($71) {
      $72 = HEAP32[$2>>2]|0;
      $73 = ((($72)) + 6|0);
      $74 = HEAP8[$73>>0]|0;
      $75 = $74 << 24 >> 24;
      $76 = ($75|0)==(47);
      if ($76) {
       HEAP32[$iIn>>2] = 7;
       while(1) {
        $77 = HEAP32[$iIn>>2]|0;
        $78 = HEAP32[$2>>2]|0;
        $79 = (($78) + ($77)|0);
        $80 = HEAP8[$79>>0]|0;
        $81 = $80 << 24 >> 24;
        $82 = ($81|0)!=(0);
        if ($82) {
         $83 = HEAP32[$iIn>>2]|0;
         $84 = HEAP32[$2>>2]|0;
         $85 = (($84) + ($83)|0);
         $86 = HEAP8[$85>>0]|0;
         $87 = $86 << 24 >> 24;
         $88 = ($87|0)!=(47);
         $387 = $88;
        } else {
         $387 = 0;
        }
        $89 = HEAP32[$iIn>>2]|0;
        if (!($387)) {
         break;
        }
        $90 = (($89) + 1)|0;
        HEAP32[$iIn>>2] = $90;
       }
       $91 = ($89|0)!=(7);
       if ($91) {
        $92 = HEAP32[$iIn>>2]|0;
        $93 = ($92|0)!=(16);
        if (!($93)) {
         $94 = HEAP32[$2>>2]|0;
         $95 = ((($94)) + 7|0);
         $96 = (_memcmp(26029,$95,9)|0);
         $97 = ($96|0)!=(0);
         if (!($97)) {
          break;
         }
        }
        $98 = HEAP32[$iIn>>2]|0;
        $99 = (($98) - 7)|0;
        $100 = HEAP32[$2>>2]|0;
        $101 = ((($100)) + 7|0);
        HEAP32[$vararg_buffer>>2] = $99;
        $vararg_ptr12 = ((($vararg_buffer)) + 4|0);
        HEAP32[$vararg_ptr12>>2] = $101;
        $102 = (_sqlite3_mprintf(26039,$vararg_buffer)|0);
        $103 = HEAP32[$6>>2]|0;
        HEAP32[$103>>2] = $102;
        HEAP32[$rc>>2] = 1;
        break L1;
       }
      }
     }
    } while(0);
    HEAP32[$eState>>2] = 0;
    L27: while(1) {
     $104 = HEAP32[$iIn>>2]|0;
     $105 = HEAP32[$2>>2]|0;
     $106 = (($105) + ($104)|0);
     $107 = HEAP8[$106>>0]|0;
     HEAP8[$c>>0] = $107;
     $108 = $107 << 24 >> 24;
     $109 = ($108|0)!=(0);
     if (!($109)) {
      break;
     }
     $110 = HEAP8[$c>>0]|0;
     $111 = $110 << 24 >> 24;
     $112 = ($111|0)!=(35);
     if (!($112)) {
      break;
     }
     $113 = HEAP32[$iIn>>2]|0;
     $114 = (($113) + 1)|0;
     HEAP32[$iIn>>2] = $114;
     $115 = HEAP8[$c>>0]|0;
     $116 = $115 << 24 >> 24;
     $117 = ($116|0)==(37);
     do {
      if ($117) {
       $118 = HEAP32[$iIn>>2]|0;
       $119 = HEAP32[$2>>2]|0;
       $120 = (($119) + ($118)|0);
       $121 = HEAP8[$120>>0]|0;
       $122 = $121&255;
       $123 = (18403 + ($122)|0);
       $124 = HEAP8[$123>>0]|0;
       $125 = $124&255;
       $126 = $125 & 8;
       $127 = ($126|0)!=(0);
       if ($127) {
        $128 = HEAP32[$iIn>>2]|0;
        $129 = (($128) + 1)|0;
        $130 = HEAP32[$2>>2]|0;
        $131 = (($130) + ($129)|0);
        $132 = HEAP8[$131>>0]|0;
        $133 = $132&255;
        $134 = (18403 + ($133)|0);
        $135 = HEAP8[$134>>0]|0;
        $136 = $135&255;
        $137 = $136 & 8;
        $138 = ($137|0)!=(0);
        if ($138) {
         $139 = HEAP32[$iIn>>2]|0;
         $140 = (($139) + 1)|0;
         HEAP32[$iIn>>2] = $140;
         $141 = HEAP32[$2>>2]|0;
         $142 = (($141) + ($139)|0);
         $143 = HEAP8[$142>>0]|0;
         $144 = $143 << 24 >> 24;
         $145 = (_sqlite3HexToInt($144)|0);
         $146 = $145&255;
         $147 = $146 << 4;
         HEAP32[$octet>>2] = $147;
         $148 = HEAP32[$iIn>>2]|0;
         $149 = (($148) + 1)|0;
         HEAP32[$iIn>>2] = $149;
         $150 = HEAP32[$2>>2]|0;
         $151 = (($150) + ($148)|0);
         $152 = HEAP8[$151>>0]|0;
         $153 = $152 << 24 >> 24;
         $154 = (_sqlite3HexToInt($153)|0);
         $155 = $154&255;
         $156 = HEAP32[$octet>>2]|0;
         $157 = (($156) + ($155))|0;
         HEAP32[$octet>>2] = $157;
         $158 = HEAP32[$octet>>2]|0;
         $159 = ($158|0)==(0);
         if (!($159)) {
          $189 = HEAP32[$octet>>2]|0;
          $190 = $189&255;
          HEAP8[$c>>0] = $190;
          break;
         }
         while(1) {
          $160 = HEAP32[$iIn>>2]|0;
          $161 = HEAP32[$2>>2]|0;
          $162 = (($161) + ($160)|0);
          $163 = HEAP8[$162>>0]|0;
          HEAP8[$c>>0] = $163;
          $164 = $163 << 24 >> 24;
          $165 = ($164|0)!=(0);
          if (!($165)) {
           continue L27;
          }
          $166 = HEAP8[$c>>0]|0;
          $167 = $166 << 24 >> 24;
          $168 = ($167|0)!=(35);
          if (!($168)) {
           continue L27;
          }
          $169 = HEAP32[$eState>>2]|0;
          $170 = ($169|0)!=(0);
          if (!($170)) {
           $171 = HEAP8[$c>>0]|0;
           $172 = $171 << 24 >> 24;
           $173 = ($172|0)!=(63);
           if (!($173)) {
            continue L27;
           }
          }
          $174 = HEAP32[$eState>>2]|0;
          $175 = ($174|0)!=(1);
          if (!($175)) {
           $176 = HEAP8[$c>>0]|0;
           $177 = $176 << 24 >> 24;
           $178 = ($177|0)!=(61);
           if (!($178)) {
            continue L27;
           }
           $179 = HEAP8[$c>>0]|0;
           $180 = $179 << 24 >> 24;
           $181 = ($180|0)!=(38);
           if (!($181)) {
            continue L27;
           }
          }
          $182 = HEAP32[$eState>>2]|0;
          $183 = ($182|0)!=(2);
          if (!($183)) {
           $184 = HEAP8[$c>>0]|0;
           $185 = $184 << 24 >> 24;
           $186 = ($185|0)!=(38);
           if (!($186)) {
            continue L27;
           }
          }
          $187 = HEAP32[$iIn>>2]|0;
          $188 = (($187) + 1)|0;
          HEAP32[$iIn>>2] = $188;
         }
        } else {
         label = 37;
        }
       } else {
        label = 37;
       }
      } else {
       label = 37;
      }
     } while(0);
     L51: do {
      if ((label|0) == 37) {
       label = 0;
       $191 = HEAP32[$eState>>2]|0;
       $192 = ($191|0)==(1);
       do {
        if ($192) {
         $193 = HEAP8[$c>>0]|0;
         $194 = $193 << 24 >> 24;
         $195 = ($194|0)==(38);
         if (!($195)) {
          $196 = HEAP8[$c>>0]|0;
          $197 = $196 << 24 >> 24;
          $198 = ($197|0)==(61);
          if (!($198)) {
           break;
          }
         }
         $199 = HEAP32[$iOut>>2]|0;
         $200 = (($199) - 1)|0;
         $201 = HEAP32[$zFile>>2]|0;
         $202 = (($201) + ($200)|0);
         $203 = HEAP8[$202>>0]|0;
         $204 = $203 << 24 >> 24;
         $205 = ($204|0)==(0);
         if ($205) {
          while(1) {
           $206 = HEAP32[$iIn>>2]|0;
           $207 = HEAP32[$2>>2]|0;
           $208 = (($207) + ($206)|0);
           $209 = HEAP8[$208>>0]|0;
           $210 = $209 << 24 >> 24;
           $211 = ($210|0)!=(0);
           if (!($211)) {
            continue L27;
           }
           $212 = HEAP32[$iIn>>2]|0;
           $213 = HEAP32[$2>>2]|0;
           $214 = (($213) + ($212)|0);
           $215 = HEAP8[$214>>0]|0;
           $216 = $215 << 24 >> 24;
           $217 = ($216|0)!=(35);
           if (!($217)) {
            continue L27;
           }
           $218 = HEAP32[$iIn>>2]|0;
           $219 = (($218) - 1)|0;
           $220 = HEAP32[$2>>2]|0;
           $221 = (($220) + ($219)|0);
           $222 = HEAP8[$221>>0]|0;
           $223 = $222 << 24 >> 24;
           $224 = ($223|0)!=(38);
           if (!($224)) {
            continue L27;
           }
           $225 = HEAP32[$iIn>>2]|0;
           $226 = (($225) + 1)|0;
           HEAP32[$iIn>>2] = $226;
          }
         } else {
          $227 = HEAP8[$c>>0]|0;
          $228 = $227 << 24 >> 24;
          $229 = ($228|0)==(38);
          if ($229) {
           $230 = HEAP32[$iOut>>2]|0;
           $231 = (($230) + 1)|0;
           HEAP32[$iOut>>2] = $231;
           $232 = HEAP32[$zFile>>2]|0;
           $233 = (($232) + ($230)|0);
           HEAP8[$233>>0] = 0;
          } else {
           HEAP32[$eState>>2] = 2;
          }
          HEAP8[$c>>0] = 0;
          break L51;
         }
        }
       } while(0);
       $234 = HEAP32[$eState>>2]|0;
       $235 = ($234|0)==(0);
       if ($235) {
        $236 = HEAP8[$c>>0]|0;
        $237 = $236 << 24 >> 24;
        $238 = ($237|0)==(63);
        if (!($238)) {
         label = 51;
        }
       } else {
        label = 51;
       }
       if ((label|0) == 51) {
        label = 0;
        $239 = HEAP32[$eState>>2]|0;
        $240 = ($239|0)==(2);
        if (!($240)) {
         break;
        }
        $241 = HEAP8[$c>>0]|0;
        $242 = $241 << 24 >> 24;
        $243 = ($242|0)==(38);
        if (!($243)) {
         break;
        }
       }
       HEAP8[$c>>0] = 0;
       HEAP32[$eState>>2] = 1;
      }
     } while(0);
     $244 = HEAP8[$c>>0]|0;
     $245 = HEAP32[$iOut>>2]|0;
     $246 = (($245) + 1)|0;
     HEAP32[$iOut>>2] = $246;
     $247 = HEAP32[$zFile>>2]|0;
     $248 = (($247) + ($245)|0);
     HEAP8[$248>>0] = $244;
    }
    $249 = HEAP32[$eState>>2]|0;
    $250 = ($249|0)==(1);
    if ($250) {
     $251 = HEAP32[$iOut>>2]|0;
     $252 = (($251) + 1)|0;
     HEAP32[$iOut>>2] = $252;
     $253 = HEAP32[$zFile>>2]|0;
     $254 = (($253) + ($251)|0);
     HEAP8[$254>>0] = 0;
    }
    $255 = HEAP32[$iOut>>2]|0;
    $256 = (($255) + 1)|0;
    HEAP32[$iOut>>2] = $256;
    $257 = HEAP32[$zFile>>2]|0;
    $258 = (($257) + ($255)|0);
    HEAP8[$258>>0] = 0;
    $259 = HEAP32[$iOut>>2]|0;
    $260 = (($259) + 1)|0;
    HEAP32[$iOut>>2] = $260;
    $261 = HEAP32[$zFile>>2]|0;
    $262 = (($261) + ($259)|0);
    HEAP8[$262>>0] = 0;
    $263 = HEAP32[$zFile>>2]|0;
    $264 = (_sqlite3Strlen30($263)|0);
    $265 = (($264) + 1)|0;
    $266 = HEAP32[$zFile>>2]|0;
    $267 = (($266) + ($265)|0);
    HEAP32[$zOpt>>2] = $267;
    while(1) {
     $268 = HEAP32[$zOpt>>2]|0;
     $269 = HEAP8[$268>>0]|0;
     $270 = ($269<<24>>24)!=(0);
     if (!($270)) {
      label = 84;
      break L1;
     }
     $271 = HEAP32[$zOpt>>2]|0;
     $272 = (_sqlite3Strlen30($271)|0);
     HEAP32[$nOpt>>2] = $272;
     $273 = HEAP32[$nOpt>>2]|0;
     $274 = (($273) + 1)|0;
     $275 = HEAP32[$zOpt>>2]|0;
     $276 = (($275) + ($274)|0);
     HEAP32[$zVal>>2] = $276;
     $277 = HEAP32[$zVal>>2]|0;
     $278 = (_sqlite3Strlen30($277)|0);
     HEAP32[$nVal>>2] = $278;
     $279 = HEAP32[$nOpt>>2]|0;
     $280 = ($279|0)==(3);
     if ($280) {
      $281 = HEAP32[$zOpt>>2]|0;
      $282 = (_memcmp(26067,$281,3)|0);
      $283 = ($282|0)==(0);
      if ($283) {
       $284 = HEAP32[$zVal>>2]|0;
       HEAP32[$zVfs>>2] = $284;
      } else {
       label = 62;
      }
     } else {
      label = 62;
     }
     if ((label|0) == 62) {
      label = 0;
      HEAP32[$aMode>>2] = 0;
      HEAP32[$zModeType>>2] = 0;
      HEAP32[$mask>>2] = 0;
      HEAP32[$limit>>2] = 0;
      $285 = HEAP32[$nOpt>>2]|0;
      $286 = ($285|0)==(5);
      if ($286) {
       $287 = HEAP32[$zOpt>>2]|0;
       $288 = (_memcmp(26071,$287,5)|0);
       $289 = ($288|0)==(0);
       if ($289) {
        HEAP32[$mask>>2] = 393216;
        HEAP32[$aMode>>2] = 4104;
        $290 = HEAP32[$mask>>2]|0;
        HEAP32[$limit>>2] = $290;
        HEAP32[$zModeType>>2] = 26071;
       }
      }
      $291 = HEAP32[$nOpt>>2]|0;
      $292 = ($291|0)==(4);
      if ($292) {
       $293 = HEAP32[$zOpt>>2]|0;
       $294 = (_memcmp(26077,$293,4)|0);
       $295 = ($294|0)==(0);
       if ($295) {
        HEAP32[$mask>>2] = 135;
        HEAP32[$aMode>>2] = 4128;
        $296 = HEAP32[$mask>>2]|0;
        $297 = HEAP32[$flags>>2]|0;
        $298 = $296 & $297;
        HEAP32[$limit>>2] = $298;
        HEAP32[$zModeType>>2] = 26082;
       }
      }
      $299 = HEAP32[$aMode>>2]|0;
      $300 = ($299|0)!=(0|0);
      if ($300) {
       HEAP32[$mode>>2] = 0;
       HEAP32[$i>>2] = 0;
       while(1) {
        $301 = HEAP32[$i>>2]|0;
        $302 = HEAP32[$aMode>>2]|0;
        $303 = (($302) + ($301<<3)|0);
        $304 = HEAP32[$303>>2]|0;
        $305 = ($304|0)!=(0|0);
        if (!($305)) {
         break;
        }
        $306 = HEAP32[$i>>2]|0;
        $307 = HEAP32[$aMode>>2]|0;
        $308 = (($307) + ($306<<3)|0);
        $309 = HEAP32[$308>>2]|0;
        HEAP32[$z>>2] = $309;
        $310 = HEAP32[$nVal>>2]|0;
        $311 = HEAP32[$z>>2]|0;
        $312 = (_sqlite3Strlen30($311)|0);
        $313 = ($310|0)==($312|0);
        if ($313) {
         $314 = HEAP32[$zVal>>2]|0;
         $315 = HEAP32[$z>>2]|0;
         $316 = HEAP32[$nVal>>2]|0;
         $317 = (_memcmp($314,$315,$316)|0);
         $318 = (0)==($317|0);
         if ($318) {
          label = 73;
          break;
         }
        }
        $324 = HEAP32[$i>>2]|0;
        $325 = (($324) + 1)|0;
        HEAP32[$i>>2] = $325;
       }
       if ((label|0) == 73) {
        label = 0;
        $319 = HEAP32[$i>>2]|0;
        $320 = HEAP32[$aMode>>2]|0;
        $321 = (($320) + ($319<<3)|0);
        $322 = ((($321)) + 4|0);
        $323 = HEAP32[$322>>2]|0;
        HEAP32[$mode>>2] = $323;
       }
       $326 = HEAP32[$mode>>2]|0;
       $327 = ($326|0)==(0);
       if ($327) {
        label = 76;
        break;
       }
       $332 = HEAP32[$mode>>2]|0;
       $333 = $332 & -129;
       $334 = HEAP32[$limit>>2]|0;
       $335 = ($333|0)>($334|0);
       if ($335) {
        label = 78;
        break;
       }
       $340 = HEAP32[$flags>>2]|0;
       $341 = HEAP32[$mask>>2]|0;
       $342 = $341 ^ -1;
       $343 = $340 & $342;
       $344 = HEAP32[$mode>>2]|0;
       $345 = $343 | $344;
       HEAP32[$flags>>2] = $345;
      }
     }
     $346 = HEAP32[$nVal>>2]|0;
     $347 = (($346) + 1)|0;
     $348 = HEAP32[$zVal>>2]|0;
     $349 = (($348) + ($347)|0);
     HEAP32[$zOpt>>2] = $349;
    }
    if ((label|0) == 76) {
     $328 = HEAP32[$zModeType>>2]|0;
     $329 = HEAP32[$zVal>>2]|0;
     HEAP32[$vararg_buffer13>>2] = $328;
     $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
     HEAP32[$vararg_ptr16>>2] = $329;
     $330 = (_sqlite3_mprintf(26089,$vararg_buffer13)|0);
     $331 = HEAP32[$6>>2]|0;
     HEAP32[$331>>2] = $330;
     HEAP32[$rc>>2] = 1;
     break;
    }
    else if ((label|0) == 78) {
     $336 = HEAP32[$zModeType>>2]|0;
     $337 = HEAP32[$zVal>>2]|0;
     HEAP32[$vararg_buffer17>>2] = $336;
     $vararg_ptr20 = ((($vararg_buffer17)) + 4|0);
     HEAP32[$vararg_ptr20>>2] = $337;
     $338 = (_sqlite3_mprintf(26109,$vararg_buffer17)|0);
     $339 = HEAP32[$6>>2]|0;
     HEAP32[$339>>2] = $338;
     HEAP32[$rc>>2] = 3;
     break;
    }
   } else {
    label = 81;
   }
  } else {
   label = 81;
  }
 } while(0);
 do {
  if ((label|0) == 81) {
   $350 = HEAP32[$nUri>>2]|0;
   $351 = (($350) + 2)|0;
   $352 = ($351|0)<(0);
   $353 = $352 << 31 >> 31;
   $354 = (_sqlite3_malloc64($351,$353)|0);
   HEAP32[$zFile>>2] = $354;
   $355 = HEAP32[$zFile>>2]|0;
   $356 = ($355|0)!=(0|0);
   if ($356) {
    $357 = HEAP32[$zFile>>2]|0;
    $358 = HEAP32[$2>>2]|0;
    $359 = HEAP32[$nUri>>2]|0;
    _memcpy(($357|0),($358|0),($359|0))|0;
    $360 = HEAP32[$nUri>>2]|0;
    $361 = HEAP32[$zFile>>2]|0;
    $362 = (($361) + ($360)|0);
    HEAP8[$362>>0] = 0;
    $363 = HEAP32[$nUri>>2]|0;
    $364 = (($363) + 1)|0;
    $365 = HEAP32[$zFile>>2]|0;
    $366 = (($365) + ($364)|0);
    HEAP8[$366>>0] = 0;
    $367 = HEAP32[$flags>>2]|0;
    $368 = $367 & -65;
    HEAP32[$flags>>2] = $368;
    label = 84;
    break;
   }
   HEAP32[$0>>2] = 7;
   $386 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($386|0);
  }
 } while(0);
 if ((label|0) == 84) {
  $369 = HEAP32[$zVfs>>2]|0;
  $370 = (_sqlite3_vfs_find($369)|0);
  $371 = HEAP32[$4>>2]|0;
  HEAP32[$371>>2] = $370;
  $372 = HEAP32[$4>>2]|0;
  $373 = HEAP32[$372>>2]|0;
  $374 = ($373|0)==(0|0);
  if ($374) {
   $375 = HEAP32[$zVfs>>2]|0;
   HEAP32[$vararg_buffer21>>2] = $375;
   $376 = (_sqlite3_mprintf(26133,$vararg_buffer21)|0);
   $377 = HEAP32[$6>>2]|0;
   HEAP32[$377>>2] = $376;
   HEAP32[$rc>>2] = 1;
  }
 }
 $378 = HEAP32[$rc>>2]|0;
 $379 = ($378|0)!=(0);
 if ($379) {
  $380 = HEAP32[$zFile>>2]|0;
  _sqlite3_free($380);
  HEAP32[$zFile>>2] = 0;
 }
 $381 = HEAP32[$flags>>2]|0;
 $382 = HEAP32[$3>>2]|0;
 HEAP32[$382>>2] = $381;
 $383 = HEAP32[$zFile>>2]|0;
 $384 = HEAP32[$5>>2]|0;
 HEAP32[$384>>2] = $383;
 $385 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $385;
 $386 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($386|0);
}
function _sqlite3BtreeOpen($pVfs,$zFilename,$db,$ppBtree,$flags,$vfsFlags) {
 $pVfs = $pVfs|0;
 $zFilename = $zFilename|0;
 $db = $db|0;
 $ppBtree = $ppBtree|0;
 $flags = $flags|0;
 $vfsFlags = $vfsFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $iDb = 0, $isMemdb = 0, $isTempDb = 0, $mutexOpen = 0, $nFilename = 0, $nFullPathname = 0, $nReserve = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pBt = 0, $pExisting = 0, $pSib = 0, $rc = 0, $zDbHeader = 0, $zFullPathname = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $pBt = sp + 48|0;
 $p = sp + 44|0;
 $mutexOpen = sp + 40|0;
 $rc = sp + 36|0;
 $nReserve = sp + 180|0;
 $zDbHeader = sp + 80|0;
 $isTempDb = sp + 32|0;
 $isMemdb = sp + 28|0;
 $nFilename = sp + 24|0;
 $nFullPathname = sp + 20|0;
 $zFullPathname = sp + 16|0;
 $iDb = sp + 12|0;
 $pExisting = sp + 8|0;
 $i = sp + 4|0;
 $pSib = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zFilename;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $ppBtree;
 HEAP32[$5>>2] = $flags;
 HEAP32[$6>>2] = $vfsFlags;
 HEAP32[$pBt>>2] = 0;
 HEAP32[$mutexOpen>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  $14 = 1;
 } else {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(0);
  $14 = $12;
 }
 $13 = $14&1;
 HEAP32[$isTempDb>>2] = $13;
 $15 = HEAP32[$2>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$2>>2]|0;
  $18 = (_strcmp($17,25837)|0);
  $19 = ($18|0)==(0);
  if ($19) {
   $29 = 1;
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 do {
  if ((label|0) == 5) {
   $20 = HEAP32[$isTempDb>>2]|0;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$3>>2]|0;
    $23 = (_sqlite3TempInMemory($22)|0);
    $24 = ($23|0)!=(0);
    if ($24) {
     $29 = 1;
     break;
    }
   }
   $25 = HEAP32[$6>>2]|0;
   $26 = $25 & 128;
   $27 = ($26|0)!=(0);
   $29 = $27;
  }
 } while(0);
 $28 = $29&1;
 HEAP32[$isMemdb>>2] = $28;
 $30 = HEAP32[$isMemdb>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$5>>2]|0;
  $33 = $32 | 2;
  HEAP32[$5>>2] = $33;
 }
 $34 = HEAP32[$6>>2]|0;
 $35 = $34 & 256;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$isMemdb>>2]|0;
  $38 = ($37|0)!=(0);
  $39 = HEAP32[$isTempDb>>2]|0;
  $40 = ($39|0)!=(0);
  $or$cond = $38 | $40;
  if ($or$cond) {
   $41 = HEAP32[$6>>2]|0;
   $42 = $41 & -257;
   $43 = $42 | 512;
   HEAP32[$6>>2] = $43;
  }
 }
 $44 = (_sqlite3MallocZero(48,0)|0);
 HEAP32[$p>>2] = $44;
 $45 = HEAP32[$p>>2]|0;
 $46 = ($45|0)!=(0|0);
 if (!($46)) {
  HEAP32[$0>>2] = 7;
  $399 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($399|0);
 }
 $47 = HEAP32[$p>>2]|0;
 $48 = ((($47)) + 8|0);
 HEAP8[$48>>0] = 0;
 $49 = HEAP32[$3>>2]|0;
 $50 = HEAP32[$p>>2]|0;
 HEAP32[$50>>2] = $49;
 $51 = HEAP32[$p>>2]|0;
 $52 = HEAP32[$p>>2]|0;
 $53 = ((($52)) + 32|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$p>>2]|0;
 $55 = ((($54)) + 32|0);
 $56 = ((($55)) + 4|0);
 HEAP32[$56>>2] = 1;
 $57 = HEAP32[$isTempDb>>2]|0;
 $58 = ($57|0)==(0);
 do {
  if ($58) {
   $59 = HEAP32[$isMemdb>>2]|0;
   $60 = ($59|0)==(0);
   if (!($60)) {
    $61 = HEAP32[$6>>2]|0;
    $62 = $61 & 64;
    $63 = ($62|0)!=(0);
    if (!($63)) {
     break;
    }
   }
   $64 = HEAP32[$6>>2]|0;
   $65 = $64 & 131072;
   $66 = ($65|0)!=(0);
   if ($66) {
    $67 = HEAP32[$2>>2]|0;
    $68 = (_sqlite3Strlen30($67)|0);
    $69 = (($68) + 1)|0;
    HEAP32[$nFilename>>2] = $69;
    $70 = HEAP32[$1>>2]|0;
    $71 = ((($70)) + 8|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (($72) + 1)|0;
    HEAP32[$nFullPathname>>2] = $73;
    $74 = HEAP32[$nFullPathname>>2]|0;
    $75 = HEAP32[$nFilename>>2]|0;
    $76 = ($74|0)>($75|0);
    $77 = HEAP32[$nFullPathname>>2]|0;
    $78 = HEAP32[$nFilename>>2]|0;
    $79 = $76 ? $77 : $78;
    $80 = ($79|0)<(0);
    $81 = $80 << 31 >> 31;
    $82 = (_sqlite3Malloc($79,$81)|0);
    HEAP32[$zFullPathname>>2] = $82;
    $83 = HEAP32[$p>>2]|0;
    $84 = ((($83)) + 9|0);
    HEAP8[$84>>0] = 1;
    $85 = HEAP32[$zFullPathname>>2]|0;
    $86 = ($85|0)!=(0|0);
    if (!($86)) {
     $87 = HEAP32[$p>>2]|0;
     _sqlite3_free($87);
     HEAP32[$0>>2] = 7;
     $399 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($399|0);
    }
    $88 = HEAP32[$isMemdb>>2]|0;
    $89 = ($88|0)!=(0);
    if ($89) {
     $90 = HEAP32[$zFullPathname>>2]|0;
     $91 = HEAP32[$2>>2]|0;
     $92 = HEAP32[$nFilename>>2]|0;
     _memcpy(($90|0),($91|0),($92|0))|0;
    } else {
     $93 = HEAP32[$1>>2]|0;
     $94 = HEAP32[$2>>2]|0;
     $95 = HEAP32[$nFullPathname>>2]|0;
     $96 = HEAP32[$zFullPathname>>2]|0;
     $97 = (_sqlite3OsFullPathname($93,$94,$95,$96)|0);
     HEAP32[$rc>>2] = $97;
     $98 = HEAP32[$rc>>2]|0;
     $99 = ($98|0)!=(0);
     if ($99) {
      $100 = HEAP32[$zFullPathname>>2]|0;
      _sqlite3_free($100);
      $101 = HEAP32[$p>>2]|0;
      _sqlite3_free($101);
      $102 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $102;
      $399 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($399|0);
     }
    }
    $103 = HEAP32[4024>>2]|0;
    HEAP32[$pBt>>2] = $103;
    while(1) {
     $104 = HEAP32[$pBt>>2]|0;
     $105 = ($104|0)!=(0|0);
     if (!($105)) {
      break;
     }
     $106 = HEAP32[$zFullPathname>>2]|0;
     $107 = HEAP32[$pBt>>2]|0;
     $108 = HEAP32[$107>>2]|0;
     $109 = (_sqlite3PagerFilename($108,0)|0);
     $110 = (_strcmp($106,$109)|0);
     $111 = (0)==($110|0);
     if ($111) {
      $112 = HEAP32[$pBt>>2]|0;
      $113 = HEAP32[$112>>2]|0;
      $114 = (_sqlite3PagerVfs($113)|0);
      $115 = HEAP32[$1>>2]|0;
      $116 = ($114|0)==($115|0);
      if ($116) {
       label = 29;
       break;
      }
     }
     $148 = HEAP32[$pBt>>2]|0;
     $149 = ((($148)) + 68|0);
     $150 = HEAP32[$149>>2]|0;
     HEAP32[$pBt>>2] = $150;
    }
    do {
     if ((label|0) == 29) {
      $117 = HEAP32[$3>>2]|0;
      $118 = ((($117)) + 20|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = (($119) - 1)|0;
      HEAP32[$iDb>>2] = $120;
      while(1) {
       $121 = HEAP32[$iDb>>2]|0;
       $122 = ($121|0)>=(0);
       if (!($122)) {
        label = 35;
        break;
       }
       $123 = HEAP32[$iDb>>2]|0;
       $124 = HEAP32[$3>>2]|0;
       $125 = ((($124)) + 16|0);
       $126 = HEAP32[$125>>2]|0;
       $127 = (($126) + ($123<<4)|0);
       $128 = ((($127)) + 4|0);
       $129 = HEAP32[$128>>2]|0;
       HEAP32[$pExisting>>2] = $129;
       $130 = HEAP32[$pExisting>>2]|0;
       $131 = ($130|0)!=(0|0);
       if ($131) {
        $132 = HEAP32[$pExisting>>2]|0;
        $133 = ((($132)) + 4|0);
        $134 = HEAP32[$133>>2]|0;
        $135 = HEAP32[$pBt>>2]|0;
        $136 = ($134|0)==($135|0);
        if ($136) {
         break;
        }
       }
       $139 = HEAP32[$iDb>>2]|0;
       $140 = (($139) + -1)|0;
       HEAP32[$iDb>>2] = $140;
      }
      if ((label|0) == 35) {
       $141 = HEAP32[$pBt>>2]|0;
       $142 = HEAP32[$p>>2]|0;
       $143 = ((($142)) + 4|0);
       HEAP32[$143>>2] = $141;
       $144 = HEAP32[$pBt>>2]|0;
       $145 = ((($144)) + 64|0);
       $146 = HEAP32[$145>>2]|0;
       $147 = (($146) + 1)|0;
       HEAP32[$145>>2] = $147;
       break;
      }
      $137 = HEAP32[$zFullPathname>>2]|0;
      _sqlite3_free($137);
      $138 = HEAP32[$p>>2]|0;
      _sqlite3_free($138);
      HEAP32[$0>>2] = 19;
      $399 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($399|0);
     }
    } while(0);
    $151 = HEAP32[$zFullPathname>>2]|0;
    _sqlite3_free($151);
   }
  }
 } while(0);
 $152 = HEAP32[$pBt>>2]|0;
 $153 = ($152|0)==(0|0);
 do {
  if ($153) {
   $154 = (_sqlite3MallocZero(84,0)|0);
   HEAP32[$pBt>>2] = $154;
   $155 = HEAP32[$pBt>>2]|0;
   $156 = ($155|0)==(0|0);
   if ($156) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $157 = HEAP32[$1>>2]|0;
   $158 = HEAP32[$pBt>>2]|0;
   $159 = HEAP32[$2>>2]|0;
   $160 = HEAP32[$5>>2]|0;
   $161 = HEAP32[$6>>2]|0;
   $162 = (_sqlite3PagerOpen($157,$158,$159,88,$160,$161,149)|0);
   HEAP32[$rc>>2] = $162;
   $163 = HEAP32[$rc>>2]|0;
   $164 = ($163|0)==(0);
   if ($164) {
    $165 = HEAP32[$pBt>>2]|0;
    $166 = HEAP32[$165>>2]|0;
    $167 = HEAP32[$3>>2]|0;
    $168 = ((($167)) + 40|0);
    $169 = $168;
    $170 = $169;
    $171 = HEAP32[$170>>2]|0;
    $172 = (($169) + 4)|0;
    $173 = $172;
    $174 = HEAP32[$173>>2]|0;
    _sqlite3PagerSetMmapLimit($166,$171,$174);
    $175 = HEAP32[$pBt>>2]|0;
    $176 = HEAP32[$175>>2]|0;
    $177 = (_sqlite3PagerReadFileheader($176,100,$zDbHeader)|0);
    HEAP32[$rc>>2] = $177;
   }
   $178 = HEAP32[$rc>>2]|0;
   $179 = ($178|0)!=(0);
   if (!($179)) {
    $180 = HEAP32[$5>>2]|0;
    $181 = $180&255;
    $182 = HEAP32[$pBt>>2]|0;
    $183 = ((($182)) + 16|0);
    HEAP8[$183>>0] = $181;
    $184 = HEAP32[$3>>2]|0;
    $185 = HEAP32[$pBt>>2]|0;
    $186 = ((($185)) + 4|0);
    HEAP32[$186>>2] = $184;
    $187 = HEAP32[$pBt>>2]|0;
    $188 = HEAP32[$187>>2]|0;
    $189 = HEAP32[$pBt>>2]|0;
    _sqlite3PagerSetBusyhandler($188,171,$189);
    $190 = HEAP32[$pBt>>2]|0;
    $191 = HEAP32[$p>>2]|0;
    $192 = ((($191)) + 4|0);
    HEAP32[$192>>2] = $190;
    $193 = HEAP32[$pBt>>2]|0;
    $194 = ((($193)) + 8|0);
    HEAP32[$194>>2] = 0;
    $195 = HEAP32[$pBt>>2]|0;
    $196 = ((($195)) + 12|0);
    HEAP32[$196>>2] = 0;
    $197 = HEAP32[$pBt>>2]|0;
    $198 = HEAP32[$197>>2]|0;
    $199 = (_sqlite3PagerIsreadonly($198)|0);
    $200 = ($199<<24>>24)!=(0);
    if ($200) {
     $201 = HEAP32[$pBt>>2]|0;
     $202 = ((($201)) + 22|0);
     $203 = HEAP16[$202>>1]|0;
     $204 = $203&65535;
     $205 = $204 | 1;
     $206 = $205&65535;
     HEAP16[$202>>1] = $206;
    }
    $207 = ((($zDbHeader)) + 16|0);
    $208 = HEAP8[$207>>0]|0;
    $209 = $208&255;
    $210 = $209 << 8;
    $211 = ((($zDbHeader)) + 17|0);
    $212 = HEAP8[$211>>0]|0;
    $213 = $212&255;
    $214 = $213 << 16;
    $215 = $210 | $214;
    $216 = HEAP32[$pBt>>2]|0;
    $217 = ((($216)) + 32|0);
    HEAP32[$217>>2] = $215;
    $218 = HEAP32[$pBt>>2]|0;
    $219 = ((($218)) + 32|0);
    $220 = HEAP32[$219>>2]|0;
    $221 = ($220>>>0)<(512);
    if ($221) {
     label = 49;
    } else {
     $222 = HEAP32[$pBt>>2]|0;
     $223 = ((($222)) + 32|0);
     $224 = HEAP32[$223>>2]|0;
     $225 = ($224>>>0)>(65536);
     if ($225) {
      label = 49;
     } else {
      $226 = HEAP32[$pBt>>2]|0;
      $227 = ((($226)) + 32|0);
      $228 = HEAP32[$227>>2]|0;
      $229 = (($228) - 1)|0;
      $230 = HEAP32[$pBt>>2]|0;
      $231 = ((($230)) + 32|0);
      $232 = HEAP32[$231>>2]|0;
      $233 = $229 & $232;
      $234 = ($233|0)!=(0);
      if ($234) {
       label = 49;
      } else {
       $245 = ((($zDbHeader)) + 20|0);
       $246 = HEAP8[$245>>0]|0;
       HEAP8[$nReserve>>0] = $246;
       $247 = HEAP32[$pBt>>2]|0;
       $248 = ((($247)) + 22|0);
       $249 = HEAP16[$248>>1]|0;
       $250 = $249&65535;
       $251 = $250 | 2;
       $252 = $251&65535;
       HEAP16[$248>>1] = $252;
       $253 = ((($zDbHeader)) + 52|0);
       $254 = (_sqlite3Get4byte($253)|0);
       $255 = ($254|0)!=(0);
       $256 = $255 ? 1 : 0;
       $257 = $256&255;
       $258 = HEAP32[$pBt>>2]|0;
       $259 = ((($258)) + 17|0);
       HEAP8[$259>>0] = $257;
       $260 = ((($zDbHeader)) + 64|0);
       $261 = (_sqlite3Get4byte($260)|0);
       $262 = ($261|0)!=(0);
       $263 = $262 ? 1 : 0;
       $264 = $263&255;
       $265 = HEAP32[$pBt>>2]|0;
       $266 = ((($265)) + 18|0);
       HEAP8[$266>>0] = $264;
      }
     }
    }
    if ((label|0) == 49) {
     $235 = HEAP32[$pBt>>2]|0;
     $236 = ((($235)) + 32|0);
     HEAP32[$236>>2] = 0;
     $237 = HEAP32[$2>>2]|0;
     $238 = ($237|0)==(0|0);
     $239 = HEAP32[$isMemdb>>2]|0;
     $240 = ($239|0)!=(0);
     $or$cond3 = $238 | $240;
     if (!($or$cond3)) {
      $241 = HEAP32[$pBt>>2]|0;
      $242 = ((($241)) + 17|0);
      HEAP8[$242>>0] = 0;
      $243 = HEAP32[$pBt>>2]|0;
      $244 = ((($243)) + 18|0);
      HEAP8[$244>>0] = 0;
     }
     HEAP8[$nReserve>>0] = 0;
    }
    $267 = HEAP32[$pBt>>2]|0;
    $268 = HEAP32[$267>>2]|0;
    $269 = HEAP32[$pBt>>2]|0;
    $270 = ((($269)) + 32|0);
    $271 = HEAP8[$nReserve>>0]|0;
    $272 = $271&255;
    $273 = (_sqlite3PagerSetPagesize($268,$270,$272)|0);
    HEAP32[$rc>>2] = $273;
    $274 = HEAP32[$rc>>2]|0;
    $275 = ($274|0)!=(0);
    if (!($275)) {
     $276 = HEAP32[$pBt>>2]|0;
     $277 = ((($276)) + 32|0);
     $278 = HEAP32[$277>>2]|0;
     $279 = HEAP8[$nReserve>>0]|0;
     $280 = $279&255;
     $281 = (($278) - ($280))|0;
     $282 = HEAP32[$pBt>>2]|0;
     $283 = ((($282)) + 36|0);
     HEAP32[$283>>2] = $281;
     $284 = HEAP32[$pBt>>2]|0;
     $285 = ((($284)) + 64|0);
     HEAP32[$285>>2] = 1;
     $286 = HEAP32[$p>>2]|0;
     $287 = ((($286)) + 9|0);
     $288 = HEAP8[$287>>0]|0;
     $289 = ($288<<24>>24)!=(0);
     if ($289) {
      $290 = HEAP32[4024>>2]|0;
      $291 = HEAP32[$pBt>>2]|0;
      $292 = ((($291)) + 68|0);
      HEAP32[$292>>2] = $290;
      $293 = HEAP32[$pBt>>2]|0;
      HEAP32[4024>>2] = $293;
      label = 56;
     } else {
      label = 56;
     }
    }
   }
  } else {
   label = 56;
  }
 } while(0);
 if ((label|0) == 56) {
  $294 = HEAP32[$p>>2]|0;
  $295 = ((($294)) + 9|0);
  $296 = HEAP8[$295>>0]|0;
  $297 = ($296<<24>>24)!=(0);
  L85: do {
   if ($297) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $298 = HEAP32[$i>>2]|0;
     $299 = HEAP32[$3>>2]|0;
     $300 = ((($299)) + 20|0);
     $301 = HEAP32[$300>>2]|0;
     $302 = ($298|0)<($301|0);
     if (!($302)) {
      break L85;
     }
     $303 = HEAP32[$i>>2]|0;
     $304 = HEAP32[$3>>2]|0;
     $305 = ((($304)) + 16|0);
     $306 = HEAP32[$305>>2]|0;
     $307 = (($306) + ($303<<4)|0);
     $308 = ((($307)) + 4|0);
     $309 = HEAP32[$308>>2]|0;
     HEAP32[$pSib>>2] = $309;
     $310 = ($309|0)!=(0|0);
     if ($310) {
      $311 = HEAP32[$pSib>>2]|0;
      $312 = ((($311)) + 9|0);
      $313 = HEAP8[$312>>0]|0;
      $314 = $313&255;
      $315 = ($314|0)!=(0);
      if ($315) {
       break;
      }
     }
     $375 = HEAP32[$i>>2]|0;
     $376 = (($375) + 1)|0;
     HEAP32[$i>>2] = $376;
    }
    while(1) {
     $316 = HEAP32[$pSib>>2]|0;
     $317 = ((($316)) + 28|0);
     $318 = HEAP32[$317>>2]|0;
     $319 = ($318|0)!=(0|0);
     if (!($319)) {
      break;
     }
     $320 = HEAP32[$pSib>>2]|0;
     $321 = ((($320)) + 28|0);
     $322 = HEAP32[$321>>2]|0;
     HEAP32[$pSib>>2] = $322;
    }
    $323 = HEAP32[$p>>2]|0;
    $324 = ((($323)) + 4|0);
    $325 = HEAP32[$324>>2]|0;
    $326 = $325;
    $327 = HEAP32[$pSib>>2]|0;
    $328 = ((($327)) + 4|0);
    $329 = HEAP32[$328>>2]|0;
    $330 = $329;
    $331 = ($326>>>0)<($330>>>0);
    if ($331) {
     $332 = HEAP32[$pSib>>2]|0;
     $333 = HEAP32[$p>>2]|0;
     $334 = ((($333)) + 24|0);
     HEAP32[$334>>2] = $332;
     $335 = HEAP32[$p>>2]|0;
     $336 = ((($335)) + 28|0);
     HEAP32[$336>>2] = 0;
     $337 = HEAP32[$p>>2]|0;
     $338 = HEAP32[$pSib>>2]|0;
     $339 = ((($338)) + 28|0);
     HEAP32[$339>>2] = $337;
     break;
    }
    while(1) {
     $340 = HEAP32[$pSib>>2]|0;
     $341 = ((($340)) + 24|0);
     $342 = HEAP32[$341>>2]|0;
     $343 = ($342|0)!=(0|0);
     if ($343) {
      $344 = HEAP32[$pSib>>2]|0;
      $345 = ((($344)) + 24|0);
      $346 = HEAP32[$345>>2]|0;
      $347 = ((($346)) + 4|0);
      $348 = HEAP32[$347>>2]|0;
      $349 = $348;
      $350 = HEAP32[$p>>2]|0;
      $351 = ((($350)) + 4|0);
      $352 = HEAP32[$351>>2]|0;
      $353 = $352;
      $354 = ($349>>>0)<($353>>>0);
      $400 = $354;
     } else {
      $400 = 0;
     }
     $355 = HEAP32[$pSib>>2]|0;
     $356 = ((($355)) + 24|0);
     $357 = HEAP32[$356>>2]|0;
     if (!($400)) {
      break;
     }
     HEAP32[$pSib>>2] = $357;
    }
    $358 = HEAP32[$p>>2]|0;
    $359 = ((($358)) + 24|0);
    HEAP32[$359>>2] = $357;
    $360 = HEAP32[$pSib>>2]|0;
    $361 = HEAP32[$p>>2]|0;
    $362 = ((($361)) + 28|0);
    HEAP32[$362>>2] = $360;
    $363 = HEAP32[$p>>2]|0;
    $364 = ((($363)) + 24|0);
    $365 = HEAP32[$364>>2]|0;
    $366 = ($365|0)!=(0|0);
    if ($366) {
     $367 = HEAP32[$p>>2]|0;
     $368 = HEAP32[$p>>2]|0;
     $369 = ((($368)) + 24|0);
     $370 = HEAP32[$369>>2]|0;
     $371 = ((($370)) + 28|0);
     HEAP32[$371>>2] = $367;
    }
    $372 = HEAP32[$p>>2]|0;
    $373 = HEAP32[$pSib>>2]|0;
    $374 = ((($373)) + 24|0);
    HEAP32[$374>>2] = $372;
   }
  } while(0);
  $377 = HEAP32[$p>>2]|0;
  $378 = HEAP32[$4>>2]|0;
  HEAP32[$378>>2] = $377;
 }
 $379 = HEAP32[$rc>>2]|0;
 $380 = ($379|0)!=(0);
 if ($380) {
  $381 = HEAP32[$pBt>>2]|0;
  $382 = ($381|0)!=(0|0);
  if ($382) {
   $383 = HEAP32[$pBt>>2]|0;
   $384 = HEAP32[$383>>2]|0;
   $385 = ($384|0)!=(0|0);
   if ($385) {
    $386 = HEAP32[$pBt>>2]|0;
    $387 = HEAP32[$386>>2]|0;
    (_sqlite3PagerClose($387)|0);
   }
  }
  $388 = HEAP32[$pBt>>2]|0;
  _sqlite3_free($388);
  $389 = HEAP32[$p>>2]|0;
  _sqlite3_free($389);
  $390 = HEAP32[$4>>2]|0;
  HEAP32[$390>>2] = 0;
 } else {
  $391 = HEAP32[$p>>2]|0;
  $392 = (_sqlite3BtreeSchema($391,0,0)|0);
  $393 = ($392|0)==(0|0);
  if ($393) {
   $394 = HEAP32[$p>>2]|0;
   $395 = ((($394)) + 4|0);
   $396 = HEAP32[$395>>2]|0;
   $397 = HEAP32[$396>>2]|0;
   _sqlite3PagerSetCachesize($397,-2000);
  }
 }
 $398 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $398;
 $399 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($399|0);
}
function _sqlite3SchemaGet($db,$pBt) {
 $db = $db|0;
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pBt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (_sqlite3BtreeSchema($4,84,150)|0);
  HEAP32[$p>>2] = $5;
 } else {
  $6 = (_sqlite3DbMallocZero(0,84,0)|0);
  HEAP32[$p>>2] = $6;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$0>>2]|0;
  _sqlite3OomFault($9);
  $25 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 76|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = (0)==($13|0);
 if (!($14)) {
  $25 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 8|0);
 _sqlite3HashInit($16);
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 24|0);
 _sqlite3HashInit($18);
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 40|0);
 _sqlite3HashInit($20);
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 56|0);
 _sqlite3HashInit($22);
 $23 = HEAP32[$p>>2]|0;
 $24 = ((($23)) + 77|0);
 HEAP8[$24>>0] = 1;
 $25 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3RegisterPerConnectionBuiltinFunctions($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_overload_function($1,25831,2)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)==(7);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 _sqlite3OomFault($5);
 STACKTOP = sp;return;
}
function _sqlite3AutoLoadExtensions($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $go = 0, $i = 0, $pThunk = 0, $rc = 0, $vararg_buffer = 0, $xInit = 0, $zErrmsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $i = sp + 24|0;
 $go = sp + 20|0;
 $rc = sp + 16|0;
 $xInit = sp + 12|0;
 $zErrmsg = sp + 8|0;
 $pThunk = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$go>>2] = 1;
 $1 = HEAP32[1540>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$go>>2]|0;
  $4 = ($3|0)!=(0);
  if (!($4)) {
   break;
  }
  HEAP32[$pThunk>>2] = 0;
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[1540>>2]|0;
  $7 = ($5>>>0)>=($6>>>0);
  if ($7) {
   HEAP32[$xInit>>2] = 0;
   HEAP32[$go>>2] = 0;
  } else {
   $8 = HEAP32[$i>>2]|0;
   $9 = HEAP32[(1544)>>2]|0;
   $10 = (($9) + ($8<<2)|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$xInit>>2] = $11;
  }
  HEAP32[$zErrmsg>>2] = 0;
  $12 = HEAP32[$xInit>>2]|0;
  $13 = ($12|0)!=(0|0);
  if ($13) {
   $14 = HEAP32[$xInit>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$pThunk>>2]|0;
   $17 = (FUNCTION_TABLE_iiii[$14 & 255]($15,$zErrmsg,$16)|0);
   HEAP32[$rc>>2] = $17;
   $18 = ($17|0)!=(0);
   if ($18) {
    $19 = HEAP32[$0>>2]|0;
    $20 = HEAP32[$rc>>2]|0;
    $21 = HEAP32[$zErrmsg>>2]|0;
    HEAP32[$vararg_buffer>>2] = $21;
    _sqlite3ErrorWithMsg($19,$20,25792,$vararg_buffer);
    HEAP32[$go>>2] = 0;
   }
  }
  $22 = HEAP32[$zErrmsg>>2]|0;
  _sqlite3_free($22);
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _sqlite3Fts3Init($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $pHash = 0, $pPorter = 0, $pSimple = 0, $pUnicode = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $pHash = sp + 12|0;
 $pSimple = sp + 8|0;
 $pPorter = sp + 4|0;
 $pUnicode = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pHash>>2] = 0;
 HEAP32[$pSimple>>2] = 0;
 HEAP32[$pPorter>>2] = 0;
 HEAP32[$pUnicode>>2] = 0;
 _sqlite3Fts3UnicodeTokenizer($pUnicode);
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3Fts3InitAux($2)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $6;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 _sqlite3Fts3SimpleTokenizerModule($pSimple);
 _sqlite3Fts3PorterTokenizerModule($pPorter);
 $7 = (_sqlite3_malloc(20)|0);
 HEAP32[$pHash>>2] = $7;
 $8 = HEAP32[$pHash>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$pHash>>2]|0;
  _sqlite3Fts3HashInit($10,1,1);
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = HEAP32[$pHash>>2]|0;
   $14 = HEAP32[$pSimple>>2]|0;
   $15 = (_sqlite3Fts3HashInsert($13,20447,7,$14)|0);
   $16 = ($15|0)!=(0|0);
   if (!($16)) {
    $17 = HEAP32[$pHash>>2]|0;
    $18 = HEAP32[$pPorter>>2]|0;
    $19 = (_sqlite3Fts3HashInsert($17,20454,7,$18)|0);
    $20 = ($19|0)!=(0|0);
    if (!($20)) {
     $21 = HEAP32[$pHash>>2]|0;
     $22 = HEAP32[$pUnicode>>2]|0;
     $23 = (_sqlite3Fts3HashInsert($21,20461,10,$22)|0);
     $24 = ($23|0)!=(0|0);
     if (!($24)) {
      break;
     }
    }
   }
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $25 = HEAP32[$rc>>2]|0;
 $26 = (0)==($25|0);
 if ($26) {
  $27 = HEAP32[$1>>2]|0;
  $28 = HEAP32[$pHash>>2]|0;
  $29 = (_sqlite3Fts3InitHashTable($27,$28,20471)|0);
  HEAP32[$rc>>2] = $29;
  $30 = (0)==($29|0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = (_sqlite3_overload_function($31,20486,-1)|0);
   HEAP32[$rc>>2] = $32;
   $33 = (0)==($32|0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = (_sqlite3_overload_function($34,20494,1)|0);
    HEAP32[$rc>>2] = $35;
    $36 = (0)==($35|0);
    if ($36) {
     $37 = HEAP32[$1>>2]|0;
     $38 = (_sqlite3_overload_function($37,20502,1)|0);
     HEAP32[$rc>>2] = $38;
     $39 = (0)==($38|0);
     if ($39) {
      $40 = HEAP32[$1>>2]|0;
      $41 = (_sqlite3_overload_function($40,20502,2)|0);
      HEAP32[$rc>>2] = $41;
      $42 = (0)==($41|0);
      if ($42) {
       $43 = HEAP32[$1>>2]|0;
       $44 = (_sqlite3_overload_function($43,20512,1)|0);
       HEAP32[$rc>>2] = $44;
       $45 = (0)==($44|0);
       if ($45) {
        $46 = HEAP32[$1>>2]|0;
        $47 = HEAP32[$pHash>>2]|0;
        $48 = (_sqlite3_create_module_v2($46,20521,1736,$47,151)|0);
        HEAP32[$rc>>2] = $48;
        $49 = HEAP32[$rc>>2]|0;
        $50 = ($49|0)==(0);
        if ($50) {
         $51 = HEAP32[$1>>2]|0;
         $52 = HEAP32[$pHash>>2]|0;
         $53 = (_sqlite3_create_module_v2($51,20526,1736,$52,0)|0);
         HEAP32[$rc>>2] = $53;
        }
        $54 = HEAP32[$rc>>2]|0;
        $55 = ($54|0)==(0);
        if ($55) {
         $56 = HEAP32[$1>>2]|0;
         $57 = HEAP32[$pHash>>2]|0;
         $58 = (_sqlite3Fts3InitTok($56,$57)|0);
         HEAP32[$rc>>2] = $58;
        }
        $59 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $59;
        $65 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($65|0);
       }
      }
     }
    }
   }
  }
 }
 $60 = HEAP32[$pHash>>2]|0;
 $61 = ($60|0)!=(0|0);
 if ($61) {
  $62 = HEAP32[$pHash>>2]|0;
  _sqlite3Fts3HashClear($62);
  $63 = HEAP32[$pHash>>2]|0;
  _sqlite3_free($63);
 }
 $64 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $64;
 $65 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($65|0);
}
function _sqlite3Fts3UnicodeTokenizer($ppModule) {
 $ppModule = $ppModule|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $ppModule;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$1>>2] = 2356;
 STACKTOP = sp;return;
}
function _sqlite3Fts3InitAux($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_create_module($1,25443,2264,0)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3Fts3SimpleTokenizerModule($ppModule) {
 $ppModule = $ppModule|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $ppModule;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$1>>2] = 2236;
 STACKTOP = sp;return;
}
function _sqlite3Fts3PorterTokenizerModule($ppModule) {
 $ppModule = $ppModule|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $ppModule;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$1>>2] = 2208;
 STACKTOP = sp;return;
}
function _sqlite3Fts3HashInit($pNew,$keyClass,$copyKey) {
 $pNew = $pNew|0;
 $keyClass = $keyClass|0;
 $copyKey = $copyKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pNew;
 HEAP8[$1>>0] = $keyClass;
 HEAP8[$2>>0] = $copyKey;
 $3 = HEAP8[$1>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 HEAP8[$4>>0] = $3;
 $5 = HEAP8[$2>>0]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 1|0);
 HEAP8[$7>>0] = $5;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 4|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP32[$15>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3HashInsert($pH,$pKey,$nKey,$data) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 $data = $data|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $elem = 0, $h = 0, $hraw = 0, $new_elem = 0;
 var $old_data = 0, $or$cond = 0, $xHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $hraw = sp + 20|0;
 $h = sp + 16|0;
 $elem = sp + 12|0;
 $new_elem = sp + 8|0;
 $xHash = sp + 4|0;
 $old_data = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $nKey;
 HEAP32[$4>>2] = $data;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6 << 24 >> 24;
 $8 = (_ftsHashFunction($7)|0);
 HEAP32[$xHash>>2] = $8;
 $9 = HEAP32[$xHash>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iii[$9 & 255]($10,$11)|0);
 HEAP32[$hraw>>2] = $12;
 $13 = HEAP32[$hraw>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) - 1)|0;
 $18 = $13 & $17;
 HEAP32[$h>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$h>>2]|0;
 $23 = (_fts3FindElementByHash($19,$20,$21,$22)|0);
 HEAP32[$elem>>2] = $23;
 $24 = HEAP32[$elem>>2]|0;
 $25 = ($24|0)!=(0|0);
 if ($25) {
  $26 = HEAP32[$elem>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$old_data>>2] = $28;
  $29 = HEAP32[$4>>2]|0;
  $30 = ($29|0)==(0|0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP32[$elem>>2]|0;
   $33 = HEAP32[$h>>2]|0;
   _fts3RemoveElementByHash($31,$32,$33);
  } else {
   $34 = HEAP32[$4>>2]|0;
   $35 = HEAP32[$elem>>2]|0;
   $36 = ((($35)) + 8|0);
   HEAP32[$36>>2] = $34;
  }
  $37 = HEAP32[$old_data>>2]|0;
  HEAP32[$0>>2] = $37;
  $115 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($115|0);
 }
 $38 = HEAP32[$4>>2]|0;
 $39 = ($38|0)==(0|0);
 if ($39) {
  HEAP32[$0>>2] = 0;
  $115 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($115|0);
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 12|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = HEAP32[$1>>2]|0;
  $45 = (_fts3Rehash($44,8)|0);
  $46 = ($45|0)!=(0);
  if (!($46)) {
   label = 10;
  }
 } else {
  label = 10;
 }
 do {
  if ((label|0) == 10) {
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$1>>2]|0;
   $51 = ((($50)) + 12|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($49|0)>=($52|0);
   if ($53) {
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 12|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = $57<<1;
    $59 = (_fts3Rehash($54,$58)|0);
    $60 = ($59|0)!=(0);
    if ($60) {
     break;
    }
   }
   $64 = (_fts3HashMalloc(20)|0);
   HEAP32[$new_elem>>2] = $64;
   $65 = HEAP32[$new_elem>>2]|0;
   $66 = ($65|0)==(0|0);
   if ($66) {
    $67 = HEAP32[$4>>2]|0;
    HEAP32[$0>>2] = $67;
    $115 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($115|0);
   }
   $68 = HEAP32[$1>>2]|0;
   $69 = ((($68)) + 1|0);
   $70 = HEAP8[$69>>0]|0;
   $71 = $70 << 24 >> 24;
   $72 = ($71|0)!=(0);
   $73 = HEAP32[$2>>2]|0;
   $74 = ($73|0)!=(0|0);
   $or$cond = $72 & $74;
   do {
    if ($or$cond) {
     $75 = HEAP32[$3>>2]|0;
     $76 = (_fts3HashMalloc($75)|0);
     $77 = HEAP32[$new_elem>>2]|0;
     $78 = ((($77)) + 12|0);
     HEAP32[$78>>2] = $76;
     $79 = HEAP32[$new_elem>>2]|0;
     $80 = ((($79)) + 12|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = ($81|0)==(0|0);
     $83 = HEAP32[$new_elem>>2]|0;
     if (!($82)) {
      $85 = ((($83)) + 12|0);
      $86 = HEAP32[$85>>2]|0;
      $87 = HEAP32[$2>>2]|0;
      $88 = HEAP32[$3>>2]|0;
      _memcpy(($86|0),($87|0),($88|0))|0;
      break;
     }
     _fts3HashFree($83);
     $84 = HEAP32[$4>>2]|0;
     HEAP32[$0>>2] = $84;
     $115 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($115|0);
    } else {
     $89 = HEAP32[$2>>2]|0;
     $90 = HEAP32[$new_elem>>2]|0;
     $91 = ((($90)) + 12|0);
     HEAP32[$91>>2] = $89;
    }
   } while(0);
   $92 = HEAP32[$3>>2]|0;
   $93 = HEAP32[$new_elem>>2]|0;
   $94 = ((($93)) + 16|0);
   HEAP32[$94>>2] = $92;
   $95 = HEAP32[$1>>2]|0;
   $96 = ((($95)) + 4|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = (($97) + 1)|0;
   HEAP32[$96>>2] = $98;
   $99 = HEAP32[$hraw>>2]|0;
   $100 = HEAP32[$1>>2]|0;
   $101 = ((($100)) + 12|0);
   $102 = HEAP32[$101>>2]|0;
   $103 = (($102) - 1)|0;
   $104 = $99 & $103;
   HEAP32[$h>>2] = $104;
   $105 = HEAP32[$1>>2]|0;
   $106 = HEAP32[$h>>2]|0;
   $107 = HEAP32[$1>>2]|0;
   $108 = ((($107)) + 16|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + ($106<<3)|0);
   $111 = HEAP32[$new_elem>>2]|0;
   _fts3HashInsertElement($105,$110,$111);
   $112 = HEAP32[$4>>2]|0;
   $113 = HEAP32[$new_elem>>2]|0;
   $114 = ((($113)) + 8|0);
   HEAP32[$114>>2] = $112;
   HEAP32[$0>>2] = 0;
   $115 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($115|0);
  }
 } while(0);
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 4|0);
 HEAP32[$62>>2] = 0;
 $63 = HEAP32[$4>>2]|0;
 HEAP32[$0>>2] = $63;
 $115 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($115|0);
}
function _sqlite3Fts3InitHashTable($db,$pHash,$zName) {
 $db = $db|0;
 $pHash = $pHash|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $any = 0, $p = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $p = sp + 4|0;
 $any = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $zName;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 HEAP32[$any>>2] = 5;
 $4 = HEAP32[$rc>>2]|0;
 $5 = (0)==($4|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$p>>2]|0;
  $9 = (_sqlite3_create_function($6,$7,1,5,$8,225,0,0)|0);
  HEAP32[$rc>>2] = $9;
 }
 $10 = HEAP32[$rc>>2]|0;
 $11 = (0)==($10|0);
 if (!($11)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$p>>2]|0;
 $15 = (_sqlite3_create_function($12,$13,2,5,$14,225,0,0)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _hashDestroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $pHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pHash = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pHash>>2] = $1;
 $2 = HEAP32[$pHash>>2]|0;
 _sqlite3Fts3HashClear($2);
 $3 = HEAP32[$pHash>>2]|0;
 _sqlite3_free($3);
 STACKTOP = sp;return;
}
function _sqlite3Fts3InitTok($db,$pHash) {
 $db = $db|0;
 $pHash = $pHash|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pHash;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3_create_module($2,20531,1828,$3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3Fts3HashClear($pH) {
 $pH = $pH|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $next_elem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $elem = sp + 4|0;
 $next_elem = sp;
 HEAP32[$0>>2] = $pH;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$elem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 _fts3HashFree($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 16|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 HEAP32[$12>>2] = 0;
 while(1) {
  $13 = HEAP32[$elem>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$elem>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$next_elem>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 1|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$elem>>2]|0;
   $23 = ((($22)) + 12|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0|0);
   if ($25) {
    $26 = HEAP32[$elem>>2]|0;
    $27 = ((($26)) + 12|0);
    $28 = HEAP32[$27>>2]|0;
    _fts3HashFree($28);
   }
  }
  $29 = HEAP32[$elem>>2]|0;
  _fts3HashFree($29);
  $30 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $30;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 4|0);
 HEAP32[$32>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3HashFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return;
}
function _fts3tokConnectMethod($db,$pHash,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pHash = $pHash|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $azArg = 0, $azDequote = 0, $nDequote = 0, $pMod = 0, $pTab = 0, $pTok = 0, $rc = 0, $zModule = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $pTab = sp + 28|0;
 $pMod = sp + 24|0;
 $pTok = sp + 20|0;
 $rc = sp + 16|0;
 $azDequote = sp + 12|0;
 $nDequote = sp + 8|0;
 $zModule = sp + 4|0;
 $azArg = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pHash;
 HEAP32[$3>>2] = $argc;
 HEAP32[$4>>2] = $argv;
 HEAP32[$5>>2] = $ppVtab;
 HEAP32[$6>>2] = $pzErr;
 HEAP32[$pTab>>2] = 0;
 HEAP32[$pMod>>2] = 0;
 HEAP32[$pTok>>2] = 0;
 HEAP32[$azDequote>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3_declare_vtab($7,20544)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $11;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) - 3)|0;
 HEAP32[$nDequote>>2] = $13;
 $14 = HEAP32[$nDequote>>2]|0;
 $15 = HEAP32[$4>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = (_fts3tokDequoteArray($14,$16,$azDequote)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$nDequote>>2]|0;
  $21 = ($20|0)<(1);
  if ($21) {
   HEAP32[$zModule>>2] = 20447;
  } else {
   $22 = HEAP32[$azDequote>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$zModule>>2] = $23;
  }
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$zModule>>2]|0;
  $26 = HEAP32[$6>>2]|0;
  $27 = (_fts3tokQueryTokenizer($24,$25,$pMod,$26)|0);
  HEAP32[$rc>>2] = $27;
 }
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)==(0);
 if ($29) {
  $30 = HEAP32[$azDequote>>2]|0;
  $31 = ((($30)) + 4|0);
  HEAP32[$azArg>>2] = $31;
  $32 = HEAP32[$pMod>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$nDequote>>2]|0;
  $36 = ($35|0)>(1);
  $37 = HEAP32[$nDequote>>2]|0;
  $38 = (($37) - 1)|0;
  $39 = $36 ? $38 : 0;
  $40 = HEAP32[$azArg>>2]|0;
  $41 = (FUNCTION_TABLE_iiii[$34 & 255]($39,$40,$pTok)|0);
  HEAP32[$rc>>2] = $41;
 }
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = (_sqlite3_malloc(20)|0);
  HEAP32[$pTab>>2] = $44;
  $45 = HEAP32[$pTab>>2]|0;
  $46 = ($45|0)==(0|0);
  if ($46) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)==(0);
 if ($48) {
  $49 = HEAP32[$pTab>>2]|0;
  ;HEAP32[$49>>2]=0|0;HEAP32[$49+4>>2]=0|0;HEAP32[$49+8>>2]=0|0;HEAP32[$49+12>>2]=0|0;HEAP32[$49+16>>2]=0|0;
  $50 = HEAP32[$pMod>>2]|0;
  $51 = HEAP32[$pTab>>2]|0;
  $52 = ((($51)) + 12|0);
  HEAP32[$52>>2] = $50;
  $53 = HEAP32[$pTok>>2]|0;
  $54 = HEAP32[$pTab>>2]|0;
  $55 = ((($54)) + 16|0);
  HEAP32[$55>>2] = $53;
  $56 = HEAP32[$pTab>>2]|0;
  $57 = HEAP32[$5>>2]|0;
  HEAP32[$57>>2] = $56;
 } else {
  $58 = HEAP32[$pTok>>2]|0;
  $59 = ($58|0)!=(0|0);
  if ($59) {
   $60 = HEAP32[$pMod>>2]|0;
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$pTok>>2]|0;
   (FUNCTION_TABLE_ii[$62 & 255]($63)|0);
  }
 }
 $64 = HEAP32[$azDequote>>2]|0;
 _sqlite3_free($64);
 $65 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $65;
 $66 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($66|0);
}
function _fts3tokBestIndexMethod($pVTab,$pInfo) {
 $pVTab = $pVTab|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $pInfo;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   label = 8;
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*12)|0)|0);
  $12 = ((($11)) + 5|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[$i>>2]|0;
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (($19) + (($16*12)|0)|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(0);
   if ($22) {
    $23 = HEAP32[$i>>2]|0;
    $24 = HEAP32[$2>>2]|0;
    $25 = ((($24)) + 4|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = (($26) + (($23*12)|0)|0);
    $28 = ((($27)) + 4|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(2);
    if ($31) {
     label = 6;
     break;
    }
   }
  }
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$i>>2] = $48;
 }
 if ((label|0) == 6) {
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 20|0);
  HEAP32[$33>>2] = 1;
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 16|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<3)|0);
  HEAP32[$38>>2] = 1;
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = ((($40)) + 16|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + ($39<<3)|0);
  $44 = ((($43)) + 4|0);
  HEAP8[$44>>0] = 1;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 40|0);
  HEAPF64[$46>>3] = 1.0;
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 8) {
  $49 = HEAP32[$2>>2]|0;
  $50 = ((($49)) + 20|0);
  HEAP32[$50>>2] = 0;
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 return (0)|0;
}
function _fts3tokDisconnectMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pTab>>2] = $1;
 $2 = HEAP32[$pTab>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 (FUNCTION_TABLE_ii[$6 & 255]($9)|0);
 $10 = HEAP32[$pTab>>2]|0;
 _sqlite3_free($10);
 STACKTOP = sp;return 0;
}
function _fts3tokOpenMethod($pVTab,$ppCsr) {
 $pVTab = $pVTab|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $ppCsr;
 $3 = (_sqlite3_malloc(36)|0);
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$pCsr>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 7;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  $6 = HEAP32[$pCsr>>2]|0;
  dest=$6; stop=dest+36|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $7 = HEAP32[$pCsr>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  HEAP32[$8>>2] = $7;
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _fts3tokCloseMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 _fts3tokResetCursor($2);
 $3 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($3);
 STACKTOP = sp;return 0;
}
function _fts3tokFilterMethod($pCursor,$idxNum,$idxStr,$nVal,$apVal) {
 $pCursor = $pCursor|0;
 $idxNum = $idxNum|0;
 $idxStr = $idxStr|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $pCsr = 0, $pTab = 0, $rc = 0, $zByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $rc = sp + 16|0;
 $pCsr = sp + 12|0;
 $pTab = sp + 8|0;
 $zByte = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $idxNum;
 HEAP32[$3>>2] = $idxStr;
 HEAP32[$4>>2] = $nVal;
 HEAP32[$5>>2] = $apVal;
 HEAP32[$rc>>2] = 1;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTab>>2] = $8;
 $9 = HEAP32[$pCsr>>2]|0;
 _fts3tokResetCursor($9);
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(1);
 do {
  if ($11) {
   $12 = HEAP32[$5>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = (_sqlite3_value_text($13)|0);
   HEAP32[$zByte>>2] = $14;
   $15 = HEAP32[$5>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3_value_bytes($16)|0);
   HEAP32[$nByte>>2] = $17;
   $18 = HEAP32[$nByte>>2]|0;
   $19 = (($18) + 1)|0;
   $20 = (_sqlite3_malloc($19)|0);
   $21 = HEAP32[$pCsr>>2]|0;
   $22 = ((($21)) + 4|0);
   HEAP32[$22>>2] = $20;
   $23 = HEAP32[$pCsr>>2]|0;
   $24 = ((($23)) + 4|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)==(0|0);
   if ($26) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $27 = HEAP32[$pCsr>>2]|0;
   $28 = ((($27)) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$zByte>>2]|0;
   $31 = HEAP32[$nByte>>2]|0;
   _memcpy(($29|0),($30|0),($31|0))|0;
   $32 = HEAP32[$nByte>>2]|0;
   $33 = HEAP32[$pCsr>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32)|0);
   HEAP8[$36>>0] = 0;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ((($39)) + 12|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$pTab>>2]|0;
   $43 = ((($42)) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$pCsr>>2]|0;
   $46 = ((($45)) + 4|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$nByte>>2]|0;
   $49 = HEAP32[$pCsr>>2]|0;
   $50 = ((($49)) + 8|0);
   $51 = (FUNCTION_TABLE_iiiii[$41 & 255]($44,$47,$48,$50)|0);
   HEAP32[$rc>>2] = $51;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = HEAP32[$pTab>>2]|0;
    $55 = ((($54)) + 16|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = HEAP32[$pCsr>>2]|0;
    $58 = ((($57)) + 8|0);
    $59 = HEAP32[$58>>2]|0;
    HEAP32[$59>>2] = $56;
   }
  }
 } while(0);
 $60 = HEAP32[$rc>>2]|0;
 $61 = ($60|0)!=(0);
 if ($61) {
  $62 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $62;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 } else {
  $63 = HEAP32[$1>>2]|0;
  $64 = (_fts3tokNextMethod($63)|0);
  HEAP32[$0>>2] = $64;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 return (0)|0;
}
function _fts3tokNextMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pCsr = sp + 8|0;
 $pTab = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTab>>2] = $3;
 $4 = HEAP32[$pCsr>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 $8 = HEAP32[$pTab>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pCsr>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$pCsr>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$pCsr>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$pCsr>>2]|0;
 $21 = ((($20)) + 24|0);
 $22 = HEAP32[$pCsr>>2]|0;
 $23 = ((($22)) + 28|0);
 $24 = HEAP32[$pCsr>>2]|0;
 $25 = ((($24)) + 32|0);
 $26 = (FUNCTION_TABLE_iiiiiii[$12 & 255]($15,$17,$19,$21,$23,$25)|0);
 HEAP32[$rc>>2] = $26;
 $27 = HEAP32[$rc>>2]|0;
 $28 = ($27|0)!=(0);
 if (!($28)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $29 = HEAP32[$pCsr>>2]|0;
 _fts3tokResetCursor($29);
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(101);
 if (!($31)) {
  $32 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 HEAP32[$rc>>2] = 0;
 $32 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _fts3tokEofMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _fts3tokColumnMethod($pCursor,$pCtx,$iCol) {
 $pCursor = $pCursor|0;
 $pCtx = $pCtx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pCtx;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 switch ($4|0) {
 case 0:  {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$pCsr>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  _sqlite3_result_text($5,$8,-1,(-1));
  STACKTOP = sp;return 0;
  break;
 }
 case 1:  {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$pCsr>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$pCsr>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3_result_text($9,$12,$15,(-1));
  STACKTOP = sp;return 0;
  break;
 }
 case 2:  {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pCsr>>2]|0;
  $18 = ((($17)) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3_result_int($16,$19);
  STACKTOP = sp;return 0;
  break;
 }
 case 3:  {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$pCsr>>2]|0;
  $22 = ((($21)) + 28|0);
  $23 = HEAP32[$22>>2]|0;
  _sqlite3_result_int($20,$23);
  STACKTOP = sp;return 0;
  break;
 }
 default: {
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$pCsr>>2]|0;
  $26 = ((($25)) + 32|0);
  $27 = HEAP32[$26>>2]|0;
  _sqlite3_result_int($24,$27);
  STACKTOP = sp;return 0;
 }
 }
 return (0)|0;
}
function _fts3tokRowidMethod($pCursor,$pRowid) {
 $pCursor = $pCursor|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pRowid;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$pCsr>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = HEAP32[$1>>2]|0;
 $9 = $8;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $7;
 STACKTOP = sp;return 0;
}
function _fts3tokResetCursor($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$pTab>>2] = $6;
  $7 = HEAP32[$pTab>>2]|0;
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  (FUNCTION_TABLE_ii[$11 & 255]($14)|0);
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 8|0);
  HEAP32[$16>>2] = 0;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 _sqlite3_free($19);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 16|0);
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 20|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 24|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 28|0);
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 32|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 12|0);
 HEAP32[$33>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3tokDequoteArray($argc,$argv,$pazDequote) {
 $argc = $argc|0;
 $argv = $argv|0;
 $pazDequote = $pazDequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $azDequote = 0, $i = 0, $n = 0;
 var $nByte = 0, $pSpace = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $rc = sp + 20|0;
 $i = sp + 16|0;
 $nByte = sp + 12|0;
 $azDequote = sp + 8|0;
 $pSpace = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $argc;
 HEAP32[$1>>2] = $argv;
 HEAP32[$2>>2] = $pazDequote;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  HEAP32[$5>>2] = 0;
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 HEAP32[$nByte>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($10) + ($9<<2)|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_strlen($12)|0);
  $14 = (($13) + 1)|0;
  $15 = HEAP32[$nByte>>2]|0;
  $16 = (($15) + ($14))|0;
  HEAP32[$nByte>>2] = $16;
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = $19<<2;
 $21 = HEAP32[$nByte>>2]|0;
 $22 = (($20) + ($21))|0;
 $23 = (_sqlite3_malloc($22)|0);
 HEAP32[$azDequote>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$azDequote>>2]|0;
 $26 = ($25|0)==(0|0);
 if ($26) {
  HEAP32[$rc>>2] = 7;
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$azDequote>>2]|0;
 $29 = (($28) + ($27<<2)|0);
 HEAP32[$pSpace>>2] = $29;
 HEAP32[$i>>2] = 0;
 while(1) {
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$0>>2]|0;
  $32 = ($30|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = (($34) + ($33<<2)|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (_strlen($36)|0);
  HEAP32[$n>>2] = $37;
  $38 = HEAP32[$pSpace>>2]|0;
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$azDequote>>2]|0;
  $41 = (($40) + ($39<<2)|0);
  HEAP32[$41>>2] = $38;
  $42 = HEAP32[$pSpace>>2]|0;
  $43 = HEAP32[$i>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + ($43<<2)|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$n>>2]|0;
  $48 = (($47) + 1)|0;
  _memcpy(($42|0),($46|0),($48|0))|0;
  $49 = HEAP32[$pSpace>>2]|0;
  _sqlite3Fts3Dequote($49);
  $50 = HEAP32[$n>>2]|0;
  $51 = (($50) + 1)|0;
  $52 = HEAP32[$pSpace>>2]|0;
  $53 = (($52) + ($51)|0);
  HEAP32[$pSpace>>2] = $53;
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 $56 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _fts3tokQueryTokenizer($pHash,$zName,$pp,$pzErr) {
 $pHash = $pHash|0;
 $zName = $zName|0;
 $pp = $pp|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nName = 0;
 var $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 $nName = sp + 4|0;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $pp;
 HEAP32[$4>>2] = $pzErr;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_strlen($5)|0);
 HEAP32[$nName>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$nName>>2]|0;
 $10 = (($9) + 1)|0;
 $11 = (_sqlite3Fts3HashFind($7,$8,$10)|0);
 HEAP32[$p>>2] = $11;
 $12 = HEAP32[$p>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $14 = HEAP32[$4>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $15;
  _sqlite3Fts3ErrMsg($14,20595,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _sqlite3Fts3HashFind($pH,$pKey,$nKey) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pElem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pElem = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pKey;
 HEAP32[$2>>2] = $nKey;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3Fts3HashFindElem($3,$4,$5)|0);
 HEAP32[$pElem>>2] = $6;
 $7 = HEAP32[$pElem>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $12 = 0;
  STACKTOP = sp;return ($12|0);
 }
 $9 = HEAP32[$pElem>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = $11;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3Fts3ErrMsg($pzErr,$zFormat,$varargs) {
 $pzErr = $pzErr|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $pzErr;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3_vmprintf($4,$ap)|0);
 $6 = HEAP32[$0>>2]|0;
 HEAP32[$6>>2] = $5;
 STACKTOP = sp;return;
}
function _sqlite3Fts3HashFindElem($pH,$pKey,$nKey) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $xHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $h = sp + 4|0;
 $xHash = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $nKey;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if (!($5)) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 16|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  if (!($9)) {
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP8[$10>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = (_ftsHashFunction($12)|0);
   HEAP32[$xHash>>2] = $13;
   $14 = HEAP32[$xHash>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP32[$3>>2]|0;
   $17 = (FUNCTION_TABLE_iii[$14 & 255]($15,$16)|0);
   HEAP32[$h>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = HEAP32[$h>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 12|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) - 1)|0;
   $26 = $21 & $25;
   $27 = (_fts3FindElementByHash($18,$19,$20,$26)|0);
   HEAP32[$0>>2] = $27;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _ftsHashFunction($keyClass) {
 $keyClass = $keyClass|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $keyClass;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(1);
 if ($3) {
  HEAP32[$0>>2] = 180;
 } else {
  HEAP32[$0>>2] = 181;
 }
 $4 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _fts3FindElementByHash($pH,$pKey,$nKey,$h) {
 $pH = $pH|0;
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $count = 0, $elem = 0, $pEntry = 0, $xCompare = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $elem = sp + 12|0;
 $count = sp + 8|0;
 $xCompare = sp + 4|0;
 $pEntry = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $nKey;
 HEAP32[$4>>2] = $h;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 L1: do {
  if ($8) {
   $9 = HEAP32[$4>>2]|0;
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 16|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) + ($9<<3)|0);
   HEAP32[$pEntry>>2] = $13;
   $14 = HEAP32[$pEntry>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$elem>>2] = $16;
   $17 = HEAP32[$pEntry>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$count>>2] = $18;
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP8[$19>>0]|0;
   $21 = $20 << 24 >> 24;
   $22 = (_ftsCompareFunction($21)|0);
   HEAP32[$xCompare>>2] = $22;
   while(1) {
    $23 = HEAP32[$count>>2]|0;
    $24 = (($23) + -1)|0;
    HEAP32[$count>>2] = $24;
    $25 = ($23|0)!=(0);
    $26 = HEAP32[$elem>>2]|0;
    $27 = ($26|0)!=(0|0);
    $28 = $25 ? $27 : 0;
    if (!($28)) {
     break L1;
    }
    $29 = HEAP32[$xCompare>>2]|0;
    $30 = HEAP32[$elem>>2]|0;
    $31 = ((($30)) + 12|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$elem>>2]|0;
    $34 = ((($33)) + 16|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    $37 = HEAP32[$3>>2]|0;
    $38 = (FUNCTION_TABLE_iiiii[$29 & 255]($32,$35,$36,$37)|0);
    $39 = ($38|0)==(0);
    $40 = HEAP32[$elem>>2]|0;
    if ($39) {
     break;
    }
    $41 = HEAP32[$40>>2]|0;
    HEAP32[$elem>>2] = $41;
   }
   HEAP32[$0>>2] = $40;
   $42 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($42|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $42 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _ftsCompareFunction($keyClass) {
 $keyClass = $keyClass|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $keyClass;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(1);
 if ($3) {
  HEAP32[$0>>2] = 142;
 } else {
  HEAP32[$0>>2] = 143;
 }
 $4 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _fts3StrCompare($pKey1,$n1,$pKey2,$n2) {
 $pKey1 = $pKey1|0;
 $n1 = $n1|0;
 $pKey2 = $pKey2|0;
 $n2 = $n2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $n1;
 HEAP32[$3>>2] = $pKey2;
 HEAP32[$4>>2] = $n2;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($5|0)!=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 1;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_strncmp($8,$9,$10)|0);
  HEAP32[$0>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _fts3BinCompare($pKey1,$n1,$pKey2,$n2) {
 $pKey1 = $pKey1|0;
 $n1 = $n1|0;
 $pKey2 = $pKey2|0;
 $n2 = $n2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $n1;
 HEAP32[$3>>2] = $pKey2;
 HEAP32[$4>>2] = $n2;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($5|0)!=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 1;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_memcmp($8,$9,$10)|0);
  HEAP32[$0>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _fts3StrHash($pKey,$nKey) {
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $h = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $z = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $pKey;
 HEAP32[$1>>2] = $nKey;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $2;
 HEAP32[$h>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)<=(0);
 if ($4) {
  $5 = HEAP32[$z>>2]|0;
  $6 = (_strlen($5)|0);
  HEAP32[$1>>2] = $6;
 }
 while(1) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ($7|0)>(0);
  $9 = HEAP32[$h>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = $9 << 3;
  $11 = HEAP32[$h>>2]|0;
  $12 = $10 ^ $11;
  $13 = HEAP32[$z>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP32[$z>>2] = $14;
  $15 = HEAP8[$13>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = $12 ^ $16;
  HEAP32[$h>>2] = $17;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + -1)|0;
  HEAP32[$1>>2] = $19;
 }
 $20 = $9 & 2147483647;
 STACKTOP = sp;return ($20|0);
}
function _fts3BinHash($pKey,$nKey) {
 $pKey = $pKey|0;
 $nKey = $nKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $h = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $pKey;
 HEAP32[$1>>2] = $nKey;
 HEAP32[$h>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $2;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = (($3) + -1)|0;
  HEAP32[$1>>2] = $4;
  $5 = ($3|0)>(0);
  $6 = HEAP32[$h>>2]|0;
  if (!($5)) {
   break;
  }
  $7 = $6 << 3;
  $8 = HEAP32[$h>>2]|0;
  $9 = $7 ^ $8;
  $10 = HEAP32[$z>>2]|0;
  $11 = ((($10)) + 1|0);
  HEAP32[$z>>2] = $11;
  $12 = HEAP8[$10>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = $9 ^ $13;
  HEAP32[$h>>2] = $14;
 }
 $15 = $6 & 2147483647;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3Fts3Dequote($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iIn = 0, $iOut = 0, $quote = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $quote = sp + 12|0;
 $iIn = sp + 4|0;
 $iOut = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP8[$1>>0]|0;
 HEAP8[$quote>>0] = $2;
 $3 = HEAP8[$quote>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(91);
 if (!($5)) {
  $6 = HEAP8[$quote>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)==(39);
  if (!($8)) {
   $9 = HEAP8[$quote>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)==(34);
   if (!($11)) {
    $12 = HEAP8[$quote>>0]|0;
    $13 = $12 << 24 >> 24;
    $14 = ($13|0)==(96);
    if (!($14)) {
     STACKTOP = sp;return;
    }
   }
  }
 }
 HEAP32[$iIn>>2] = 1;
 HEAP32[$iOut>>2] = 0;
 $15 = HEAP8[$quote>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)==(91);
 if ($17) {
  HEAP8[$quote>>0] = 93;
 }
 while(1) {
  $18 = HEAP32[$iIn>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + ($18)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = ($21<<24>>24)!=(0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$iIn>>2]|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26 << 24 >> 24;
  $28 = HEAP8[$quote>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($27|0)==($29|0);
  $31 = HEAP32[$iIn>>2]|0;
  $32 = (($31) + 1)|0;
  if (!($30)) {
   HEAP32[$iIn>>2] = $32;
   $47 = HEAP32[$0>>2]|0;
   $48 = (($47) + ($31)|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = HEAP32[$iOut>>2]|0;
   $51 = (($50) + 1)|0;
   HEAP32[$iOut>>2] = $51;
   $52 = HEAP32[$0>>2]|0;
   $53 = (($52) + ($50)|0);
   HEAP8[$53>>0] = $49;
   continue;
  }
  $33 = HEAP32[$0>>2]|0;
  $34 = (($33) + ($32)|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = $35 << 24 >> 24;
  $37 = HEAP8[$quote>>0]|0;
  $38 = $37 << 24 >> 24;
  $39 = ($36|0)!=($38|0);
  if ($39) {
   break;
  }
  $40 = HEAP8[$quote>>0]|0;
  $41 = HEAP32[$iOut>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$iOut>>2] = $42;
  $43 = HEAP32[$0>>2]|0;
  $44 = (($43) + ($41)|0);
  HEAP8[$44>>0] = $40;
  $45 = HEAP32[$iIn>>2]|0;
  $46 = (($45) + 2)|0;
  HEAP32[$iIn>>2] = $46;
 }
 $54 = HEAP32[$iOut>>2]|0;
 $55 = HEAP32[$0>>2]|0;
 $56 = (($55) + ($54)|0);
 HEAP8[$56>>0] = 0;
 STACKTOP = sp;return;
}
function _fts3CreateMethod($db,$pAux,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pAux = $pAux|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pAux;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $ppVtab;
 HEAP32[$5>>2] = $pzErr;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = HEAP32[$5>>2]|0;
 $12 = (_fts3InitVtab(1,$6,$7,$8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _fts3ConnectMethod($db,$pAux,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pAux = $pAux|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pAux;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $ppVtab;
 HEAP32[$5>>2] = $pzErr;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$4>>2]|0;
 $11 = HEAP32[$5>>2]|0;
 $12 = (_fts3InitVtab(0,$6,$7,$8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _fts3BestIndexMethod($pVTab,$pInfo) {
 $pVTab = $pVTab|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDocid = 0, $i = 0, $iCons = 0, $iDocidGe = 0, $iDocidLe = 0, $iIdx = 0, $iLangidCons = 0, $or$cond = 0, $p = 0, $pCons = 0, $pOrder = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $p = sp + 36|0;
 $i = sp + 32|0;
 $iCons = sp + 28|0;
 $iLangidCons = sp + 24|0;
 $iDocidGe = sp + 20|0;
 $iDocidLe = sp + 16|0;
 $iIdx = sp + 12|0;
 $bDocid = sp + 8|0;
 $pCons = sp + 4|0;
 $pOrder = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 HEAP32[$iCons>>2] = -1;
 HEAP32[$iLangidCons>>2] = -1;
 HEAP32[$iDocidGe>>2] = -1;
 HEAP32[$iDocidLe>>2] = -1;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 20|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 40|0);
 HEAPF64[$7>>3] = 5.0E+6;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*12)|0)|0);
  HEAP32[$pCons>>2] = $16;
  $17 = HEAP32[$pCons>>2]|0;
  $18 = ((($17)) + 5|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)==(0);
  $22 = HEAP32[$pCons>>2]|0;
  L4: do {
   if ($21) {
    $23 = ((($22)) + 4|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24&255;
    $26 = ($25|0)==(64);
    if ($26) {
     label = 5;
     break L1;
    }
   } else {
    $32 = HEAP32[$22>>2]|0;
    $33 = ($32|0)<(0);
    if ($33) {
     $42 = 1;
    } else {
     $34 = HEAP32[$pCons>>2]|0;
     $35 = HEAP32[$34>>2]|0;
     $36 = HEAP32[$p>>2]|0;
     $37 = ((($36)) + 24|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = (($38) + 1)|0;
     $40 = ($35|0)==($39|0);
     $42 = $40;
    }
    $41 = $42&1;
    HEAP32[$bDocid>>2] = $41;
    $43 = HEAP32[$iCons>>2]|0;
    $44 = ($43|0)<(0);
    if ($44) {
     $45 = HEAP32[$pCons>>2]|0;
     $46 = ((($45)) + 4|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = $47&255;
     $49 = ($48|0)==(2);
     $50 = HEAP32[$bDocid>>2]|0;
     $51 = ($50|0)!=(0);
     $or$cond = $49 & $51;
     if ($or$cond) {
      $52 = HEAP32[$2>>2]|0;
      $53 = ((($52)) + 20|0);
      HEAP32[$53>>2] = 1;
      $54 = HEAP32[$2>>2]|0;
      $55 = ((($54)) + 40|0);
      HEAPF64[$55>>3] = 1.0;
      $56 = HEAP32[$i>>2]|0;
      HEAP32[$iCons>>2] = $56;
     }
    }
    $57 = HEAP32[$pCons>>2]|0;
    $58 = ((($57)) + 4|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = ($60|0)==(64);
    if ($61) {
     $62 = HEAP32[$pCons>>2]|0;
     $63 = HEAP32[$62>>2]|0;
     $64 = ($63|0)>=(0);
     if ($64) {
      $65 = HEAP32[$pCons>>2]|0;
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$p>>2]|0;
      $68 = ((($67)) + 24|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = ($66|0)<=($69|0);
      if ($70) {
       $71 = HEAP32[$pCons>>2]|0;
       $72 = HEAP32[$71>>2]|0;
       $73 = (2 + ($72))|0;
       $74 = HEAP32[$2>>2]|0;
       $75 = ((($74)) + 20|0);
       HEAP32[$75>>2] = $73;
       $76 = HEAP32[$2>>2]|0;
       $77 = ((($76)) + 40|0);
       HEAPF64[$77>>3] = 2.0;
       $78 = HEAP32[$i>>2]|0;
       HEAP32[$iCons>>2] = $78;
      }
     }
    }
    $79 = HEAP32[$pCons>>2]|0;
    $80 = ((($79)) + 4|0);
    $81 = HEAP8[$80>>0]|0;
    $82 = $81&255;
    $83 = ($82|0)==(2);
    if ($83) {
     $84 = HEAP32[$pCons>>2]|0;
     $85 = HEAP32[$84>>2]|0;
     $86 = HEAP32[$p>>2]|0;
     $87 = ((($86)) + 24|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = (($88) + 2)|0;
     $90 = ($85|0)==($89|0);
     if ($90) {
      $91 = HEAP32[$i>>2]|0;
      HEAP32[$iLangidCons>>2] = $91;
     }
    }
    $92 = HEAP32[$bDocid>>2]|0;
    $93 = ($92|0)!=(0);
    if ($93) {
     $94 = HEAP32[$pCons>>2]|0;
     $95 = ((($94)) + 4|0);
     $96 = HEAP8[$95>>0]|0;
     $97 = $96&255;
     switch ($97|0) {
     case 4: case 32:  {
      $98 = HEAP32[$i>>2]|0;
      HEAP32[$iDocidGe>>2] = $98;
      break L4;
      break;
     }
     case 16: case 8:  {
      $99 = HEAP32[$i>>2]|0;
      HEAP32[$iDocidLe>>2] = $99;
      break L4;
      break;
     }
     default: {
      break L4;
     }
     }
    }
   }
  } while(0);
  $100 = HEAP32[$i>>2]|0;
  $101 = (($100) + 1)|0;
  HEAP32[$i>>2] = $101;
 }
 if ((label|0) == 5) {
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 20|0);
  HEAP32[$28>>2] = 0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 40|0);
  HEAPF64[$30>>3] = 1.0000000000000001E+50;
  $31 = HEAP32[$2>>2]|0;
  _fts3SetEstimatedRows($31,0,262144);
  HEAP32[$0>>2] = 0;
  $186 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($186|0);
 }
 $102 = HEAP32[$2>>2]|0;
 $103 = ((($102)) + 20|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = ($104|0)==(1);
 if ($105) {
  $106 = HEAP32[$2>>2]|0;
  _fts3SetUniqueFlag($106);
 }
 HEAP32[$iIdx>>2] = 1;
 $107 = HEAP32[$iCons>>2]|0;
 $108 = ($107|0)>=(0);
 if ($108) {
  $109 = HEAP32[$iIdx>>2]|0;
  $110 = (($109) + 1)|0;
  HEAP32[$iIdx>>2] = $110;
  $111 = HEAP32[$iCons>>2]|0;
  $112 = HEAP32[$2>>2]|0;
  $113 = ((($112)) + 16|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = (($114) + ($111<<3)|0);
  HEAP32[$115>>2] = $109;
  $116 = HEAP32[$iCons>>2]|0;
  $117 = HEAP32[$2>>2]|0;
  $118 = ((($117)) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = (($119) + ($116<<3)|0);
  $121 = ((($120)) + 4|0);
  HEAP8[$121>>0] = 1;
 }
 $122 = HEAP32[$iLangidCons>>2]|0;
 $123 = ($122|0)>=(0);
 if ($123) {
  $124 = HEAP32[$2>>2]|0;
  $125 = ((($124)) + 20|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = $126 | 65536;
  HEAP32[$125>>2] = $127;
  $128 = HEAP32[$iIdx>>2]|0;
  $129 = (($128) + 1)|0;
  HEAP32[$iIdx>>2] = $129;
  $130 = HEAP32[$iLangidCons>>2]|0;
  $131 = HEAP32[$2>>2]|0;
  $132 = ((($131)) + 16|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = (($133) + ($130<<3)|0);
  HEAP32[$134>>2] = $128;
 }
 $135 = HEAP32[$iDocidGe>>2]|0;
 $136 = ($135|0)>=(0);
 if ($136) {
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 20|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = $139 | 131072;
  HEAP32[$138>>2] = $140;
  $141 = HEAP32[$iIdx>>2]|0;
  $142 = (($141) + 1)|0;
  HEAP32[$iIdx>>2] = $142;
  $143 = HEAP32[$iDocidGe>>2]|0;
  $144 = HEAP32[$2>>2]|0;
  $145 = ((($144)) + 16|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = (($146) + ($143<<3)|0);
  HEAP32[$147>>2] = $141;
 }
 $148 = HEAP32[$iDocidLe>>2]|0;
 $149 = ($148|0)>=(0);
 if ($149) {
  $150 = HEAP32[$2>>2]|0;
  $151 = ((($150)) + 20|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = $152 | 262144;
  HEAP32[$151>>2] = $153;
  $154 = HEAP32[$iIdx>>2]|0;
  $155 = (($154) + 1)|0;
  HEAP32[$iIdx>>2] = $155;
  $156 = HEAP32[$iDocidLe>>2]|0;
  $157 = HEAP32[$2>>2]|0;
  $158 = ((($157)) + 16|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = (($159) + ($156<<3)|0);
  HEAP32[$160>>2] = $154;
 }
 $161 = HEAP32[$2>>2]|0;
 $162 = ((($161)) + 8|0);
 $163 = HEAP32[$162>>2]|0;
 $164 = ($163|0)==(1);
 do {
  if ($164) {
   $165 = HEAP32[$2>>2]|0;
   $166 = ((($165)) + 12|0);
   $167 = HEAP32[$166>>2]|0;
   HEAP32[$pOrder>>2] = $167;
   $168 = HEAP32[$pOrder>>2]|0;
   $169 = HEAP32[$168>>2]|0;
   $170 = ($169|0)<(0);
   if (!($170)) {
    $171 = HEAP32[$pOrder>>2]|0;
    $172 = HEAP32[$171>>2]|0;
    $173 = HEAP32[$p>>2]|0;
    $174 = ((($173)) + 24|0);
    $175 = HEAP32[$174>>2]|0;
    $176 = (($175) + 1)|0;
    $177 = ($172|0)==($176|0);
    if (!($177)) {
     break;
    }
   }
   $178 = HEAP32[$pOrder>>2]|0;
   $179 = ((($178)) + 4|0);
   $180 = HEAP8[$179>>0]|0;
   $181 = ($180<<24>>24)!=(0);
   $182 = HEAP32[$2>>2]|0;
   $183 = ((($182)) + 24|0);
   if ($181) {
    HEAP32[$183>>2] = 23624;
   } else {
    HEAP32[$183>>2] = 23629;
   }
   $184 = HEAP32[$2>>2]|0;
   $185 = ((($184)) + 32|0);
   HEAP32[$185>>2] = 1;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $186 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($186|0);
}
function _fts3DisconnectMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = ($2|0)<(40);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 56|0);
  $7 = (($6) + ($4<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  (_sqlite3_finalize($8)|0);
  $9 = HEAP32[$i>>2]|0;
  $10 = (($9) + 1)|0;
  HEAP32[$i>>2] = $10;
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = ((($11)) + 240|0);
 $13 = HEAP32[$12>>2]|0;
 _sqlite3_free($13);
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 216|0);
 $16 = HEAP32[$15>>2]|0;
 _sqlite3_free($16);
 $17 = HEAP32[$p>>2]|0;
 $18 = ((($17)) + 220|0);
 $19 = HEAP32[$18>>2]|0;
 _sqlite3_free($19);
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 40|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3_free($22);
 $23 = HEAP32[$p>>2]|0;
 $24 = ((($23)) + 44|0);
 $25 = HEAP32[$24>>2]|0;
 _sqlite3_free($25);
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 36|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ((($29)) + 8|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 36|0);
 $34 = HEAP32[$33>>2]|0;
 (FUNCTION_TABLE_ii[$31 & 255]($34)|0);
 $35 = HEAP32[$p>>2]|0;
 _sqlite3_free($35);
 STACKTOP = sp;return 0;
}
function _fts3DestroyMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer14 = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer14 = sp + 32|0;
 $vararg_buffer10 = sp + 24|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $p = sp + 52|0;
 $rc = sp + 48|0;
 $zDb = sp + 44|0;
 $db = sp + 40|0;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$zDb>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[$db>>2]|0;
  $13 = HEAP32[$zDb>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $16;
  _fts3DbExec($rc,$12,23824,$vararg_buffer);
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$zDb>>2]|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $18;
 $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $21;
 _fts3DbExec($rc,$17,23861,$vararg_buffer2);
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$zDb>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 20|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$vararg_buffer6>>2] = $23;
 $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
 HEAP32[$vararg_ptr9>>2] = $26;
 _fts3DbExec($rc,$22,23899,$vararg_buffer6);
 $27 = HEAP32[$db>>2]|0;
 $28 = HEAP32[$zDb>>2]|0;
 $29 = HEAP32[$p>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$vararg_buffer10>>2] = $28;
 $vararg_ptr13 = ((($vararg_buffer10)) + 4|0);
 HEAP32[$vararg_ptr13>>2] = $31;
 _fts3DbExec($rc,$27,23935,$vararg_buffer10);
 $32 = HEAP32[$db>>2]|0;
 $33 = HEAP32[$zDb>>2]|0;
 $34 = HEAP32[$p>>2]|0;
 $35 = ((($34)) + 20|0);
 $36 = HEAP32[$35>>2]|0;
 HEAP32[$vararg_buffer14>>2] = $33;
 $vararg_ptr17 = ((($vararg_buffer14)) + 4|0);
 HEAP32[$vararg_ptr17>>2] = $36;
 _fts3DbExec($rc,$32,23972,$vararg_buffer14);
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  $40 = (_fts3DisconnectMethod($39)|0);
  $42 = $40;
  STACKTOP = sp;return ($42|0);
 } else {
  $41 = HEAP32[$rc>>2]|0;
  $42 = $41;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _fts3OpenMethod($pVTab,$ppCsr) {
 $pVTab = $pVTab|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pCsr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $ppCsr;
 $3 = (_sqlite3_malloc(96)|0);
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$4>>2] = $3;
 $5 = HEAP32[$pCsr>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$pCsr>>2]|0;
  dest=$7; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  HEAP32[$0>>2] = 0;
  $8 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($8|0);
 } else {
  HEAP32[$0>>2] = 7;
  $8 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 return (0)|0;
}
function _fts3CloseMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 (_sqlite3_finalize($4)|0);
 $5 = HEAP32[$pCsr>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3Fts3ExprFree($7);
 $8 = HEAP32[$pCsr>>2]|0;
 _sqlite3Fts3FreeDeferredTokens($8);
 $9 = HEAP32[$pCsr>>2]|0;
 $10 = ((($9)) + 44|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3_free($11);
 $12 = HEAP32[$pCsr>>2]|0;
 $13 = ((($12)) + 92|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3Fts3MIBufferFree($14);
 $15 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($15);
 STACKTOP = sp;return 0;
}
function _fts3FilterMethod($pCursor,$idxNum,$idxStr,$nVal,$apVal) {
 $pCursor = $pCursor|0;
 $idxNum = $idxNum|0;
 $idxStr = $idxStr|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eSearch = 0, $iCol = 0, $iIdx = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pCons = 0, $pCsr = 0;
 var $pDocidGe = 0, $pDocidLe = 0, $pLangid = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $zQuery = 0, $zSql = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer7 = sp + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $4 = sp + 92|0;
 $5 = sp + 88|0;
 $rc = sp + 84|0;
 $zSql = sp + 80|0;
 $eSearch = sp + 76|0;
 $p = sp + 72|0;
 $pCsr = sp + 68|0;
 $pCons = sp + 64|0;
 $pLangid = sp + 60|0;
 $pDocidGe = sp + 56|0;
 $pDocidLe = sp + 52|0;
 $iIdx = sp + 48|0;
 $iCol = sp + 44|0;
 $zQuery = sp + 40|0;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $idxNum;
 HEAP32[$3>>2] = $idxStr;
 HEAP32[$4>>2] = $nVal;
 HEAP32[$5>>2] = $apVal;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $8;
 HEAP32[$pCons>>2] = 0;
 HEAP32[$pLangid>>2] = 0;
 HEAP32[$pDocidGe>>2] = 0;
 HEAP32[$pDocidLe>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = $9 & 65535;
 HEAP32[$eSearch>>2] = $10;
 HEAP32[$iIdx>>2] = 0;
 $11 = HEAP32[$eSearch>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$iIdx>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$iIdx>>2] = $14;
  $15 = HEAP32[$5>>2]|0;
  $16 = (($15) + ($13<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pCons>>2] = $17;
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = $18 & 65536;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$iIdx>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$iIdx>>2] = $22;
  $23 = HEAP32[$5>>2]|0;
  $24 = (($23) + ($21<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pLangid>>2] = $25;
 }
 $26 = HEAP32[$2>>2]|0;
 $27 = $26 & 131072;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = HEAP32[$iIdx>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$iIdx>>2] = $30;
  $31 = HEAP32[$5>>2]|0;
  $32 = (($31) + ($29<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$pDocidGe>>2] = $33;
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = $34 & 262144;
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$iIdx>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$iIdx>>2] = $38;
  $39 = HEAP32[$5>>2]|0;
  $40 = (($39) + ($37<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$pDocidLe>>2] = $41;
 }
 $42 = HEAP32[$pCsr>>2]|0;
 $43 = ((($42)) + 8|0);
 $44 = HEAP32[$43>>2]|0;
 (_sqlite3_finalize($44)|0);
 $45 = HEAP32[$pCsr>>2]|0;
 $46 = ((($45)) + 44|0);
 $47 = HEAP32[$46>>2]|0;
 _sqlite3_free($47);
 $48 = HEAP32[$pCsr>>2]|0;
 $49 = ((($48)) + 92|0);
 $50 = HEAP32[$49>>2]|0;
 _sqlite3Fts3MIBufferFree($50);
 $51 = HEAP32[$pCsr>>2]|0;
 $52 = ((($51)) + 12|0);
 $53 = HEAP32[$52>>2]|0;
 _sqlite3Fts3ExprFree($53);
 $54 = HEAP32[$1>>2]|0;
 $55 = ((($54)) + 4|0);
 dest=$55; stop=dest+92|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $56 = HEAP32[$pDocidGe>>2]|0;
 $57 = (_fts3DocidRange($56,0,-2147483648)|0);
 $58 = tempRet0;
 $59 = HEAP32[$pCsr>>2]|0;
 $60 = ((($59)) + 72|0);
 $61 = $60;
 $62 = $61;
 HEAP32[$62>>2] = $57;
 $63 = (($61) + 4)|0;
 $64 = $63;
 HEAP32[$64>>2] = $58;
 $65 = HEAP32[$pDocidLe>>2]|0;
 $66 = (_fts3DocidRange($65,-1,2147483647)|0);
 $67 = tempRet0;
 $68 = HEAP32[$pCsr>>2]|0;
 $69 = ((($68)) + 80|0);
 $70 = $69;
 $71 = $70;
 HEAP32[$71>>2] = $66;
 $72 = (($70) + 4)|0;
 $73 = $72;
 HEAP32[$73>>2] = $67;
 $74 = HEAP32[$3>>2]|0;
 $75 = ($74|0)!=(0|0);
 if ($75) {
  $76 = HEAP32[$3>>2]|0;
  $77 = HEAP8[$76>>0]|0;
  $78 = $77 << 24 >> 24;
  $79 = ($78|0)==(68);
  $80 = $79&1;
  $81 = $80&255;
  $82 = HEAP32[$pCsr>>2]|0;
  $83 = ((($82)) + 52|0);
  HEAP8[$83>>0] = $81;
 } else {
  $84 = HEAP32[$p>>2]|0;
  $85 = ((($84)) + 231|0);
  $86 = HEAP8[$85>>0]|0;
  $87 = HEAP32[$pCsr>>2]|0;
  $88 = ((($87)) + 52|0);
  HEAP8[$88>>0] = $86;
 }
 $89 = HEAP32[$eSearch>>2]|0;
 $90 = $89&65535;
 $91 = HEAP32[$pCsr>>2]|0;
 $92 = ((($91)) + 4|0);
 HEAP16[$92>>1] = $90;
 $93 = HEAP32[$eSearch>>2]|0;
 $94 = ($93|0)!=(1);
 $95 = HEAP32[$eSearch>>2]|0;
 $96 = ($95|0)!=(0);
 $or$cond = $94 & $96;
 do {
  if ($or$cond) {
   $97 = HEAP32[$eSearch>>2]|0;
   $98 = (($97) - 2)|0;
   HEAP32[$iCol>>2] = $98;
   $99 = HEAP32[$pCons>>2]|0;
   $100 = (_sqlite3_value_text($99)|0);
   HEAP32[$zQuery>>2] = $100;
   $101 = HEAP32[$zQuery>>2]|0;
   $102 = ($101|0)==(0|0);
   if ($102) {
    $103 = HEAP32[$pCons>>2]|0;
    $104 = (_sqlite3_value_type($103)|0);
    $105 = ($104|0)!=(5);
    if ($105) {
     HEAP32[$0>>2] = 7;
     $231 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($231|0);
    }
   }
   $106 = HEAP32[$pCsr>>2]|0;
   $107 = ((($106)) + 16|0);
   HEAP32[$107>>2] = 0;
   $108 = HEAP32[$pLangid>>2]|0;
   $109 = ($108|0)!=(0|0);
   if ($109) {
    $110 = HEAP32[$pLangid>>2]|0;
    $111 = (_sqlite3_value_int($110)|0);
    $112 = HEAP32[$pCsr>>2]|0;
    $113 = ((($112)) + 16|0);
    HEAP32[$113>>2] = $111;
   }
   $114 = HEAP32[$p>>2]|0;
   $115 = ((($114)) + 36|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = HEAP32[$pCsr>>2]|0;
   $118 = ((($117)) + 16|0);
   $119 = HEAP32[$118>>2]|0;
   $120 = HEAP32[$p>>2]|0;
   $121 = ((($120)) + 28|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = HEAP32[$p>>2]|0;
   $124 = ((($123)) + 228|0);
   $125 = HEAP8[$124>>0]|0;
   $126 = $125&255;
   $127 = HEAP32[$p>>2]|0;
   $128 = ((($127)) + 24|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = HEAP32[$iCol>>2]|0;
   $131 = HEAP32[$zQuery>>2]|0;
   $132 = HEAP32[$pCsr>>2]|0;
   $133 = ((($132)) + 12|0);
   $134 = HEAP32[$p>>2]|0;
   $135 = ((($134)) + 8|0);
   $136 = (_sqlite3Fts3ExprParse($116,$119,$122,$126,$129,$130,$131,-1,$133,$135)|0);
   HEAP32[$rc>>2] = $136;
   $137 = HEAP32[$rc>>2]|0;
   $138 = ($137|0)!=(0);
   if ($138) {
    $139 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $139;
    $231 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($231|0);
   }
   $140 = HEAP32[$pCsr>>2]|0;
   $141 = (_fts3EvalStart($140)|0);
   HEAP32[$rc>>2] = $141;
   $142 = HEAP32[$p>>2]|0;
   _sqlite3Fts3SegmentsClose($142);
   $143 = HEAP32[$rc>>2]|0;
   $144 = ($143|0)!=(0);
   if (!($144)) {
    $146 = HEAP32[$pCsr>>2]|0;
    $147 = ((($146)) + 44|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = HEAP32[$pCsr>>2]|0;
    $150 = ((($149)) + 40|0);
    HEAP32[$150>>2] = $148;
    $151 = HEAP32[$pCsr>>2]|0;
    $152 = ((($151)) + 32|0);
    $153 = $152;
    $154 = $153;
    HEAP32[$154>>2] = 0;
    $155 = (($153) + 4)|0;
    $156 = $155;
    HEAP32[$156>>2] = 0;
    break;
   }
   $145 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $145;
   $231 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($231|0);
  }
 } while(0);
 $157 = HEAP32[$eSearch>>2]|0;
 $158 = ($157|0)==(0);
 do {
  if ($158) {
   $159 = HEAP32[$pDocidGe>>2]|0;
   $160 = ($159|0)!=(0|0);
   $161 = HEAP32[$pDocidLe>>2]|0;
   $162 = ($161|0)!=(0|0);
   $or$cond3 = $160 | $162;
   $163 = HEAP32[$p>>2]|0;
   $164 = ((($163)) + 216|0);
   $165 = HEAP32[$164>>2]|0;
   $166 = HEAP32[$pCsr>>2]|0;
   if ($or$cond3) {
    $167 = ((($166)) + 72|0);
    $168 = $167;
    $169 = $168;
    $170 = HEAP32[$169>>2]|0;
    $171 = (($168) + 4)|0;
    $172 = $171;
    $173 = HEAP32[$172>>2]|0;
    $174 = HEAP32[$pCsr>>2]|0;
    $175 = ((($174)) + 80|0);
    $176 = $175;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (($176) + 4)|0;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = HEAP32[$pCsr>>2]|0;
    $183 = ((($182)) + 52|0);
    $184 = HEAP8[$183>>0]|0;
    $185 = $184&255;
    $186 = ($185|0)!=(0);
    $187 = $186 ? 23624 : 23629;
    HEAP32[$vararg_buffer>>2] = $165;
    $vararg_ptr4 = ((($vararg_buffer)) + 8|0);
    $188 = $vararg_ptr4;
    $189 = $188;
    HEAP32[$189>>2] = $170;
    $190 = (($188) + 4)|0;
    $191 = $190;
    HEAP32[$191>>2] = $173;
    $vararg_ptr5 = ((($vararg_buffer)) + 16|0);
    $192 = $vararg_ptr5;
    $193 = $192;
    HEAP32[$193>>2] = $178;
    $194 = (($192) + 4)|0;
    $195 = $194;
    HEAP32[$195>>2] = $181;
    $vararg_ptr6 = ((($vararg_buffer)) + 24|0);
    HEAP32[$vararg_ptr6>>2] = $187;
    $196 = (_sqlite3_mprintf(23633,$vararg_buffer)|0);
    HEAP32[$zSql>>2] = $196;
   } else {
    $197 = ((($166)) + 52|0);
    $198 = HEAP8[$197>>0]|0;
    $199 = $198&255;
    $200 = ($199|0)!=(0);
    $201 = $200 ? 23624 : 23629;
    HEAP32[$vararg_buffer7>>2] = $165;
    $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
    HEAP32[$vararg_ptr10>>2] = $201;
    $202 = (_sqlite3_mprintf(23695,$vararg_buffer7)|0);
    HEAP32[$zSql>>2] = $202;
   }
   $203 = HEAP32[$zSql>>2]|0;
   $204 = ($203|0)!=(0|0);
   if ($204) {
    $205 = HEAP32[$p>>2]|0;
    $206 = ((($205)) + 12|0);
    $207 = HEAP32[$206>>2]|0;
    $208 = HEAP32[$zSql>>2]|0;
    $209 = HEAP32[$pCsr>>2]|0;
    $210 = ((($209)) + 8|0);
    $211 = (_sqlite3_prepare_v2($207,$208,-1,$210,0)|0);
    HEAP32[$rc>>2] = $211;
    $212 = HEAP32[$zSql>>2]|0;
    _sqlite3_free($212);
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    break;
   }
  } else {
   $213 = HEAP32[$eSearch>>2]|0;
   $214 = ($213|0)==(1);
   if ($214) {
    $215 = HEAP32[$pCsr>>2]|0;
    $216 = HEAP32[$pCsr>>2]|0;
    $217 = ((($216)) + 8|0);
    $218 = (_fts3CursorSeekStmt($215,$217)|0);
    HEAP32[$rc>>2] = $218;
    $219 = HEAP32[$rc>>2]|0;
    $220 = ($219|0)==(0);
    if ($220) {
     $221 = HEAP32[$pCsr>>2]|0;
     $222 = ((($221)) + 8|0);
     $223 = HEAP32[$222>>2]|0;
     $224 = HEAP32[$pCons>>2]|0;
     $225 = (_sqlite3_bind_value($223,1,$224)|0);
     HEAP32[$rc>>2] = $225;
    }
   }
  }
 } while(0);
 $226 = HEAP32[$rc>>2]|0;
 $227 = ($226|0)!=(0);
 if ($227) {
  $228 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $228;
  $231 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($231|0);
 } else {
  $229 = HEAP32[$1>>2]|0;
  $230 = (_fts3NextMethod($229)|0);
  HEAP32[$0>>2] = $230;
  $231 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($231|0);
 }
 return (0)|0;
}
function _fts3NextMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)==(1);
 if (!($6)) {
  $7 = HEAP32[$pCsr>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($10|0)==(0);
  if (!($11)) {
   $33 = HEAP32[$0>>2]|0;
   $34 = (_fts3EvalNext($33)|0);
   HEAP32[$rc>>2] = $34;
   $35 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($35|0);
  }
 }
 $12 = HEAP32[$pCsr>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3_step($14)|0);
 $16 = (100)!=($15|0);
 $17 = HEAP32[$pCsr>>2]|0;
 if ($16) {
  $18 = ((($17)) + 6|0);
  HEAP8[$18>>0] = 1;
  $19 = HEAP32[$pCsr>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3_reset($21)|0);
  HEAP32[$rc>>2] = $22;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  $23 = ((($17)) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3_column_int64($24,0)|0);
  $26 = tempRet0;
  $27 = HEAP32[$pCsr>>2]|0;
  $28 = ((($27)) + 32|0);
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $25;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $26;
  HEAP32[$rc>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _fts3EofMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 6|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _fts3ColumnMethod($pCursor,$pCtx,$iCol) {
 $pCursor = $pCursor|0;
 $pCtx = $pCtx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $iLangid = 0, $p = 0, $pCsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $pCsr = sp + 8|0;
 $p = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pCtx;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 $11 = ($6|0)==($10|0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$pCsr>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = $14;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  _sqlite3_result_int64($12,$17,$20);
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 24|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($21|0)==($24|0);
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  _sqlite3_result_blob($26,$pCsr,4,(-1));
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$p>>2]|0;
 $29 = ((($28)) + 24|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($30) + 2)|0;
 $32 = ($27|0)==($31|0);
 if ($32) {
  $33 = HEAP32[$pCsr>>2]|0;
  $34 = ((($33)) + 12|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0|0);
  if ($36) {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$pCsr>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)<(0);
   $42 = $41 << 31 >> 31;
   _sqlite3_result_int64($37,$40,$42);
   $81 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 $43 = HEAP32[$pCsr>>2]|0;
 $44 = (_fts3CursorSeek(0,$43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(0);
 if (!($46)) {
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $47 = HEAP32[$2>>2]|0;
 $48 = HEAP32[$p>>2]|0;
 $49 = ((($48)) + 24|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) + 2)|0;
 $52 = ($47|0)==($51|0);
 if ($52) {
  HEAP32[$iLangid>>2] = 0;
  $53 = HEAP32[$p>>2]|0;
  $54 = ((($53)) + 44|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0|0);
  if ($56) {
   $57 = HEAP32[$pCsr>>2]|0;
   $58 = ((($57)) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$p>>2]|0;
   $61 = ((($60)) + 24|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + 1)|0;
   $64 = (_sqlite3_column_int($59,$63)|0);
   HEAP32[$iLangid>>2] = $64;
  }
  $65 = HEAP32[$1>>2]|0;
  $66 = HEAP32[$iLangid>>2]|0;
  _sqlite3_result_int($65,$66);
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 } else {
  $67 = HEAP32[$pCsr>>2]|0;
  $68 = ((($67)) + 8|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = (_sqlite3_data_count($69)|0);
  $71 = HEAP32[$2>>2]|0;
  $72 = (($71) + 1)|0;
  $73 = ($70|0)>($72|0);
  if (!($73)) {
   $81 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
  $74 = HEAP32[$1>>2]|0;
  $75 = HEAP32[$pCsr>>2]|0;
  $76 = ((($75)) + 8|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  $79 = (($78) + 1)|0;
  $80 = (_sqlite3_column_value($77,$79)|0);
  _sqlite3_result_value($74,$80);
  $81 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 return (0)|0;
}
function _fts3RowidMethod($pCursor,$pRowid) {
 $pCursor = $pCursor|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pRowid;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$pCsr>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _fts3UpdateMethod($pVtab,$nArg,$apVal,$pRowid) {
 $pVtab = $pVtab|0;
 $nArg = $nArg|0;
 $apVal = $apVal|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $nArg;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$3>>2] = $pRowid;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3Fts3UpdateMethod($4,$5,$6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _fts3BeginMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 52|0);
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$p>>2]|0;
 $5 = (_fts3SetHasStat($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _fts3SyncMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $A = 0;
 var $mxLevel = 0, $nMinMerge = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $nMinMerge = sp + 16|0;
 $p = sp + 12|0;
 $rc = sp + 8|0;
 $mxLevel = sp + 4|0;
 $A = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$nMinMerge>>2] = 64;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = (_sqlite3Fts3PendingTermsFlush($2)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 52|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8>>>0)>(4);
  if ($9) {
   $10 = HEAP32[$p>>2]|0;
   $11 = ((($10)) + 48|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = ($12|0)!=(0);
   if ($13) {
    $14 = HEAP32[$p>>2]|0;
    $15 = ((($14)) + 48|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($16|0)!=(255);
    if ($17) {
     HEAP32[$mxLevel>>2] = 0;
     $18 = HEAP32[$p>>2]|0;
     $19 = (_sqlite3Fts3MaxLevel($18,$mxLevel)|0);
     HEAP32[$rc>>2] = $19;
     $20 = HEAP32[$p>>2]|0;
     $21 = ((($20)) + 52|0);
     $22 = HEAP32[$21>>2]|0;
     $23 = HEAP32[$mxLevel>>2]|0;
     $24 = Math_imul($22, $23)|0;
     HEAP32[$A>>2] = $24;
     $25 = HEAP32[$A>>2]|0;
     $26 = (($25|0) / 2)&-1;
     $27 = HEAP32[$A>>2]|0;
     $28 = (($27) + ($26))|0;
     HEAP32[$A>>2] = $28;
     $29 = HEAP32[$A>>2]|0;
     $30 = ($29|0)>(64);
     if ($30) {
      $31 = HEAP32[$p>>2]|0;
      $32 = HEAP32[$A>>2]|0;
      $33 = HEAP32[$p>>2]|0;
      $34 = ((($33)) + 48|0);
      $35 = HEAP32[$34>>2]|0;
      $36 = (_sqlite3Fts3Incrmerge($31,$32,$35)|0);
      HEAP32[$rc>>2] = $36;
     }
    }
   }
  }
 }
 $37 = HEAP32[$p>>2]|0;
 _sqlite3Fts3SegmentsClose($37);
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3CommitMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVtab;
 STACKTOP = sp;return 0;
}
function _fts3RollbackMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $2 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 _sqlite3Fts3PendingTermsClear($2);
 STACKTOP = sp;return 0;
}
function _fts3FindFunctionMethod($pVtab,$nArg,$zName,$pxFunc,$ppArg) {
 $pVtab = $pVtab|0;
 $nArg = $nArg|0;
 $zName = $zName|0;
 $pxFunc = $pxFunc|0;
 $ppArg = $ppArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aOverload = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 40|0;
 $aOverload = sp + 8|0;
 $i = sp;
 HEAP32[$1>>2] = $pVtab;
 HEAP32[$2>>2] = $nArg;
 HEAP32[$3>>2] = $zName;
 HEAP32[$4>>2] = $pxFunc;
 HEAP32[$5>>2] = $ppArg;
 ;HEAP32[$aOverload>>2]=HEAP32[2080>>2]|0;HEAP32[$aOverload+4>>2]=HEAP32[2080+4>>2]|0;HEAP32[$aOverload+8>>2]=HEAP32[2080+8>>2]|0;HEAP32[$aOverload+12>>2]=HEAP32[2080+12>>2]|0;HEAP32[$aOverload+16>>2]=HEAP32[2080+16>>2]|0;HEAP32[$aOverload+20>>2]=HEAP32[2080+20>>2]|0;HEAP32[$aOverload+24>>2]=HEAP32[2080+24>>2]|0;HEAP32[$aOverload+28>>2]=HEAP32[2080+28>>2]|0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = ($6|0)<(4);
  if (!($7)) {
   label = 6;
   break;
  }
  $8 = HEAP32[$3>>2]|0;
  $9 = HEAP32[$i>>2]|0;
  $10 = (($aOverload) + ($9<<3)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (_strcmp($8,$11)|0);
  $13 = ($12|0)==(0);
  $14 = HEAP32[$i>>2]|0;
  if ($13) {
   label = 4;
   break;
  }
  $19 = (($14) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 4) {
  $15 = (($aOverload) + ($14<<3)|0);
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$4>>2]|0;
  HEAP32[$18>>2] = $17;
  HEAP32[$0>>2] = 1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _fts3RenameMethod($pVtab,$zName) {
 $pVtab = $pVtab|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer18 = 0, $vararg_buffer3 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr21 = 0;
 var $vararg_ptr22 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer18 = sp + 64|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $p = sp + 84|0;
 $db = sp + 80|0;
 $rc = sp + 76|0;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = (_fts3SetHasStat($6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = HEAP32[$p>>2]|0;
  $11 = (_sqlite3Fts3PendingTermsFlush($10)|0);
  HEAP32[$rc>>2] = $11;
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 40|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  $16 = HEAP32[$db>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $19;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $22;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $23;
  _fts3DbExec($rc,$16,20617,$vararg_buffer);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 230|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = ($26<<24>>24)!=(0);
 if ($27) {
  $28 = HEAP32[$db>>2]|0;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $31;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = $34;
  $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
  HEAP32[$vararg_ptr7>>2] = $35;
  _fts3DbExec($rc,$28,20670,$vararg_buffer3);
 }
 $36 = HEAP32[$p>>2]|0;
 $37 = ((($36)) + 229|0);
 $38 = HEAP8[$37>>0]|0;
 $39 = ($38<<24>>24)!=(0);
 if ($39) {
  $40 = HEAP32[$db>>2]|0;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$p>>2]|0;
  $45 = ((($44)) + 20|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer8>>2] = $43;
  $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
  HEAP32[$vararg_ptr11>>2] = $46;
  $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
  HEAP32[$vararg_ptr12>>2] = $47;
  _fts3DbExec($rc,$40,20723,$vararg_buffer8);
 }
 $48 = HEAP32[$db>>2]|0;
 $49 = HEAP32[$p>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$p>>2]|0;
 $53 = ((($52)) + 20|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer13>>2] = $51;
 $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
 HEAP32[$vararg_ptr16>>2] = $54;
 $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
 HEAP32[$vararg_ptr17>>2] = $55;
 _fts3DbExec($rc,$48,20770,$vararg_buffer13);
 $56 = HEAP32[$db>>2]|0;
 $57 = HEAP32[$p>>2]|0;
 $58 = ((($57)) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = HEAP32[$p>>2]|0;
 $61 = ((($60)) + 20|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer18>>2] = $59;
 $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
 HEAP32[$vararg_ptr21>>2] = $62;
 $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
 HEAP32[$vararg_ptr22>>2] = $63;
 _fts3DbExec($rc,$56,20824,$vararg_buffer18);
 $64 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _fts3SavepointMethod($pVtab,$iSavepoint) {
 $pVtab = $pVtab|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 232|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 if (!($6)) {
  $9 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_fts3SyncMethod($7)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _fts3ReleaseMethod($pVtab,$iSavepoint) {
 $pVtab = $pVtab|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $iSavepoint;
 STACKTOP = sp;return 0;
}
function _fts3RollbackToMethod($pVtab,$iSavepoint) {
 $pVtab = $pVtab|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $iSavepoint;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 _sqlite3Fts3PendingTermsClear($3);
 STACKTOP = sp;return 0;
}
function _sqlite3Fts3PendingTermsClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pElem = 0, $pHash = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $pElem = sp + 8|0;
 $pHash = sp + 4|0;
 $pList = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 248|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 252|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + (($6*24)|0)|0);
  $11 = ((($10)) + 4|0);
  HEAP32[$pHash>>2] = $11;
  $12 = HEAP32[$pHash>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pElem>>2] = $14;
  while(1) {
   $15 = HEAP32[$pElem>>2]|0;
   $16 = ($15|0)!=(0|0);
   if (!($16)) {
    break;
   }
   $17 = HEAP32[$pElem>>2]|0;
   $18 = ((($17)) + 8|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$pList>>2] = $19;
   $20 = HEAP32[$pList>>2]|0;
   _fts3PendingListDelete($20);
   $21 = HEAP32[$pElem>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pElem>>2] = $22;
  }
  $23 = HEAP32[$pHash>>2]|0;
  _sqlite3Fts3HashClear($23);
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 260|0);
 HEAP32[$27>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3PendingListDelete($pList) {
 $pList = $pList|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pList;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return;
}
function _fts3SetHasStat($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bHasStat = 0, $pStmt = 0, $rc = 0;
 var $vararg_buffer = 0, $vararg_ptr1 = 0, $zFmt = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $rc = sp + 24|0;
 $zFmt = sp + 20|0;
 $zSql = sp + 16|0;
 $pStmt = sp + 12|0;
 $bHasStat = sp + 8|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 229|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(2);
 if (!($5)) {
  $36 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 HEAP32[$zFmt>>2] = 23181;
 $6 = HEAP32[$zFmt>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $12;
 $13 = (_sqlite3_mprintf($6,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $13;
 $14 = HEAP32[$zSql>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  HEAP32[$rc>>2] = 7;
  $36 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 HEAP32[$pStmt>>2] = 0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$zSql>>2]|0;
 $20 = (_sqlite3_prepare_v2($18,$19,-1,$pStmt,0)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = HEAP32[$pStmt>>2]|0;
  $24 = (_sqlite3_step($23)|0);
  $25 = ($24|0)==(100);
  $26 = $25&1;
  HEAP32[$bHasStat>>2] = $26;
  $27 = HEAP32[$pStmt>>2]|0;
  $28 = (_sqlite3_finalize($27)|0);
  HEAP32[$rc>>2] = $28;
  $29 = HEAP32[$rc>>2]|0;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = HEAP32[$bHasStat>>2]|0;
   $32 = $31&255;
   $33 = HEAP32[$0>>2]|0;
   $34 = ((($33)) + 229|0);
   HEAP8[$34>>0] = $32;
  }
 }
 $35 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($35);
 $36 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3Fts3PendingTermsFlush($p) {
 $p = $p|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$rc>>2]|0;
  $2 = ($1|0)==(0);
  if ($2) {
   $3 = HEAP32[$i>>2]|0;
   $4 = HEAP32[$0>>2]|0;
   $5 = ((($4)) + 248|0);
   $6 = HEAP32[$5>>2]|0;
   $7 = ($3|0)<($6|0);
   $59 = $7;
  } else {
   $59 = 0;
  }
  $8 = HEAP32[$0>>2]|0;
  if (!($59)) {
   break;
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 272|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$i>>2]|0;
  $13 = (_fts3SegmentMerge($8,$11,$12,-1)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  $15 = ($14|0)==(101);
  $$ = $15 ? 0 : $13;
  HEAP32[$rc>>2] = $$;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$i>>2] = $17;
 }
 _sqlite3Fts3PendingTermsClear($8);
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if (!($19)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 229|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 48|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(255);
 if (!($28)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 52|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31>>>0)>(0);
 if (!($32)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 HEAP32[$pStmt>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 $34 = (_fts3SqlStmt($33,22,$pStmt,0)|0);
 HEAP32[$rc>>2] = $34;
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)==(0);
 if (!($36)) {
  $58 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $37 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_bind_int($37,1,2)|0);
 $38 = HEAP32[$pStmt>>2]|0;
 $39 = (_sqlite3_step($38)|0);
 HEAP32[$rc>>2] = $39;
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(100);
 if ($41) {
  $42 = HEAP32[$pStmt>>2]|0;
  $43 = (_sqlite3_column_int($42,0)|0);
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 48|0);
  HEAP32[$45>>2] = $43;
  $46 = HEAP32[$0>>2]|0;
  $47 = ((($46)) + 48|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)==(1);
  if ($49) {
   $50 = HEAP32[$0>>2]|0;
   $51 = ((($50)) + 48|0);
   HEAP32[$51>>2] = 8;
  }
 } else {
  $52 = HEAP32[$rc>>2]|0;
  $53 = ($52|0)==(101);
  if ($53) {
   $54 = HEAP32[$0>>2]|0;
   $55 = ((($54)) + 48|0);
   HEAP32[$55>>2] = 0;
  }
 }
 $56 = HEAP32[$pStmt>>2]|0;
 $57 = (_sqlite3_reset($56)|0);
 HEAP32[$rc>>2] = $57;
 $58 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _fts3DbExec($pRc,$db,$zFormat,$varargs) {
 $pRc = $pRc|0;
 $db = $db|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $ap = sp + 8|0;
 $zSql = sp;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $zFormat;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3_vmprintf($6,$ap)|0);
 HEAP32[$zSql>>2] = $7;
 $8 = HEAP32[$zSql>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  HEAP32[$10>>2] = 7;
  STACKTOP = sp;return;
 } else {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$zSql>>2]|0;
  $13 = (_sqlite3_exec($11,$12,0,0,0)|0);
  $14 = HEAP32[$0>>2]|0;
  HEAP32[$14>>2] = $13;
  $15 = HEAP32[$zSql>>2]|0;
  _sqlite3_free($15);
  STACKTOP = sp;return;
 }
}
function _fts3SegmentMerge($p,$iLangid,$iIndex,$iLevel) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bIgnoreEmpty = 0, $csr = 0, $filter = 0, $iIdx = 0, $iMaxLevel = 0, $iNewLevel = 0, $or$cond = 0, $pWriter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $rc = sp + 104|0;
 $iIdx = sp + 100|0;
 $iNewLevel = sp + 8|0;
 $pWriter = sp + 96|0;
 $filter = sp + 80|0;
 $csr = sp + 24|0;
 $bIgnoreEmpty = sp + 16|0;
 $iMaxLevel = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$iIdx>>2] = 0;
 $4 = $iNewLevel;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 HEAP32[$pWriter>>2] = 0;
 HEAP32[$bIgnoreEmpty>>2] = 0;
 $8 = $iMaxLevel;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = (_sqlite3Fts3SegReaderCursor($12,$13,$14,$15,0,0,1,0,$csr)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)!=(0);
 do {
  if (!($18)) {
   $19 = ((($csr)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)==(0);
   if (!($21)) {
    $22 = HEAP32[$3>>2]|0;
    $23 = ($22|0)!=(-1);
    if ($23) {
     $24 = HEAP32[$0>>2]|0;
     $25 = HEAP32[$1>>2]|0;
     $26 = HEAP32[$2>>2]|0;
     $27 = (_fts3SegmentMaxLevel($24,$25,$26,$iMaxLevel)|0);
     HEAP32[$rc>>2] = $27;
     $28 = HEAP32[$rc>>2]|0;
     $29 = ($28|0)!=(0);
     if ($29) {
      break;
     }
    }
    $30 = HEAP32[$3>>2]|0;
    $31 = ($30|0)==(-2);
    if ($31) {
     $32 = ((($csr)) + 4|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($33|0)==(1);
     if ($34) {
      $35 = HEAP32[$csr>>2]|0;
      $36 = HEAP32[$35>>2]|0;
      $37 = ((($36)) + 56|0);
      $38 = HEAP32[$37>>2]|0;
      $39 = ($38|0)!=(0|0);
      $40 = $39&1;
      $41 = (0)==($40|0);
      if ($41) {
       HEAP32[$rc>>2] = 101;
       break;
      }
     }
     $42 = $iMaxLevel;
     $43 = $42;
     $44 = HEAP32[$43>>2]|0;
     $45 = (($42) + 4)|0;
     $46 = $45;
     $47 = HEAP32[$46>>2]|0;
     $48 = $iNewLevel;
     $49 = $48;
     HEAP32[$49>>2] = $44;
     $50 = (($48) + 4)|0;
     $51 = $50;
     HEAP32[$51>>2] = $47;
     HEAP32[$bIgnoreEmpty>>2] = 1;
    } else {
     $52 = HEAP32[$0>>2]|0;
     $53 = HEAP32[$1>>2]|0;
     $54 = HEAP32[$2>>2]|0;
     $55 = HEAP32[$3>>2]|0;
     $56 = (($55) + 1)|0;
     $57 = (_getAbsoluteLevel($52,$53,$54,$56)|0);
     $58 = tempRet0;
     $59 = $iNewLevel;
     $60 = $59;
     HEAP32[$60>>2] = $57;
     $61 = (($59) + 4)|0;
     $62 = $61;
     HEAP32[$62>>2] = $58;
     $63 = HEAP32[$0>>2]|0;
     $64 = HEAP32[$1>>2]|0;
     $65 = HEAP32[$2>>2]|0;
     $66 = HEAP32[$3>>2]|0;
     $67 = (($66) + 1)|0;
     $68 = (_fts3AllocateSegdirIdx($63,$64,$65,$67,$iIdx)|0);
     HEAP32[$rc>>2] = $68;
     $69 = HEAP32[$3>>2]|0;
     $70 = ($69|0)!=(-1);
     if ($70) {
      $71 = $iNewLevel;
      $72 = $71;
      $73 = HEAP32[$72>>2]|0;
      $74 = (($71) + 4)|0;
      $75 = $74;
      $76 = HEAP32[$75>>2]|0;
      $77 = $iMaxLevel;
      $78 = $77;
      $79 = HEAP32[$78>>2]|0;
      $80 = (($77) + 4)|0;
      $81 = $80;
      $82 = HEAP32[$81>>2]|0;
      $83 = ($76|0)>($82|0);
      $84 = ($73>>>0)>($79>>>0);
      $85 = ($76|0)==($82|0);
      $86 = $85 & $84;
      $87 = $83 | $86;
      $89 = $87;
     } else {
      $89 = 0;
     }
     $88 = $89&1;
     HEAP32[$bIgnoreEmpty>>2] = $88;
    }
    $90 = HEAP32[$rc>>2]|0;
    $91 = ($90|0)!=(0);
    if (!($91)) {
     ;HEAP32[$filter>>2]=0|0;HEAP32[$filter+4>>2]=0|0;HEAP32[$filter+8>>2]=0|0;HEAP32[$filter+12>>2]=0|0;
     $92 = ((($filter)) + 12|0);
     HEAP32[$92>>2] = 1;
     $93 = HEAP32[$bIgnoreEmpty>>2]|0;
     $94 = ($93|0)!=(0);
     $95 = $94 ? 2 : 0;
     $96 = ((($filter)) + 12|0);
     $97 = HEAP32[$96>>2]|0;
     $98 = $97 | $95;
     HEAP32[$96>>2] = $98;
     $99 = HEAP32[$0>>2]|0;
     $100 = (_sqlite3Fts3SegReaderStart($99,$csr,$filter)|0);
     HEAP32[$rc>>2] = $100;
     while(1) {
      $101 = HEAP32[$rc>>2]|0;
      $102 = (0)==($101|0);
      if (!($102)) {
       break;
      }
      $103 = HEAP32[$0>>2]|0;
      $104 = (_sqlite3Fts3SegReaderStep($103,$csr)|0);
      HEAP32[$rc>>2] = $104;
      $105 = HEAP32[$rc>>2]|0;
      $106 = ($105|0)!=(100);
      if ($106) {
       break;
      }
      $107 = HEAP32[$0>>2]|0;
      $108 = ((($csr)) + 40|0);
      $109 = HEAP32[$108>>2]|0;
      $110 = ((($csr)) + 44|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = ((($csr)) + 48|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = ((($csr)) + 52|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = (_fts3SegWriterAdd($107,$pWriter,1,$109,$111,$113,$115)|0);
      HEAP32[$rc>>2] = $116;
     }
     $117 = HEAP32[$rc>>2]|0;
     $118 = ($117|0)!=(0);
     if (!($118)) {
      $119 = HEAP32[$3>>2]|0;
      $120 = ($119|0)!=(-1);
      if ($120) {
       $121 = HEAP32[$0>>2]|0;
       $122 = HEAP32[$1>>2]|0;
       $123 = HEAP32[$2>>2]|0;
       $124 = HEAP32[$3>>2]|0;
       $125 = HEAP32[$csr>>2]|0;
       $126 = ((($csr)) + 4|0);
       $127 = HEAP32[$126>>2]|0;
       $128 = (_fts3DeleteSegdir($121,$122,$123,$124,$125,$127)|0);
       HEAP32[$rc>>2] = $128;
       $129 = HEAP32[$rc>>2]|0;
       $130 = ($129|0)==(0);
       $131 = HEAP32[$pWriter>>2]|0;
       $132 = ($131|0)!=(0|0);
       $or$cond = $130 & $132;
       if (!($or$cond)) {
        break;
       }
      } else {
       $$old = HEAP32[$pWriter>>2]|0;
       $$old1 = ($$old|0)!=(0|0);
       if (!($$old1)) {
        break;
       }
      }
      $133 = HEAP32[$0>>2]|0;
      $134 = HEAP32[$pWriter>>2]|0;
      $135 = $iNewLevel;
      $136 = $135;
      $137 = HEAP32[$136>>2]|0;
      $138 = (($135) + 4)|0;
      $139 = $138;
      $140 = HEAP32[$139>>2]|0;
      $141 = HEAP32[$iIdx>>2]|0;
      $142 = (_fts3SegWriterFlush($133,$134,$137,$140,$141)|0);
      HEAP32[$rc>>2] = $142;
      $143 = HEAP32[$rc>>2]|0;
      $144 = ($143|0)==(0);
      if ($144) {
       $145 = HEAP32[$3>>2]|0;
       $146 = ($145|0)==(-1);
       if (!($146)) {
        $147 = $iNewLevel;
        $148 = $147;
        $149 = HEAP32[$148>>2]|0;
        $150 = (($147) + 4)|0;
        $151 = $150;
        $152 = HEAP32[$151>>2]|0;
        $153 = $iMaxLevel;
        $154 = $153;
        $155 = HEAP32[$154>>2]|0;
        $156 = (($153) + 4)|0;
        $157 = $156;
        $158 = HEAP32[$157>>2]|0;
        $159 = ($152|0)<($158|0);
        $160 = ($149>>>0)<($155>>>0);
        $161 = ($152|0)==($158|0);
        $162 = $161 & $160;
        $163 = $159 | $162;
        if (!($163)) {
         break;
        }
       }
       $164 = HEAP32[$0>>2]|0;
       $165 = $iNewLevel;
       $166 = $165;
       $167 = HEAP32[$166>>2]|0;
       $168 = (($165) + 4)|0;
       $169 = $168;
       $170 = HEAP32[$169>>2]|0;
       $171 = HEAP32[$pWriter>>2]|0;
       $172 = ((($171)) + 56|0);
       $173 = $172;
       $174 = $173;
       $175 = HEAP32[$174>>2]|0;
       $176 = (($173) + 4)|0;
       $177 = $176;
       $178 = HEAP32[$177>>2]|0;
       $179 = (_fts3PromoteSegments($164,$167,$170,$175,$178)|0);
       HEAP32[$rc>>2] = $179;
      }
     }
    }
   }
  }
 } while(0);
 $180 = HEAP32[$pWriter>>2]|0;
 _fts3SegWriterFree($180);
 _sqlite3Fts3SegReaderFinish($csr);
 $181 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($181|0);
}
function _fts3SqlStmt($p,$eStmt,$pp,$apVal) {
 $p = $p|0;
 $eStmt = $eStmt|0;
 $pp = $pp|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $azSql = 0, $i = 0, $nParam = 0, $pStmt = 0, $rc = 0;
 var $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr9 = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 228|0;
 $1 = sp + 224|0;
 $2 = sp + 220|0;
 $3 = sp + 216|0;
 $azSql = sp + 56|0;
 $rc = sp + 48|0;
 $pStmt = sp + 44|0;
 $zSql = sp + 40|0;
 $i = sp + 36|0;
 $nParam = sp + 32|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eStmt;
 HEAP32[$2>>2] = $pp;
 HEAP32[$3>>2] = $apVal;
 _memcpy(($azSql|0),(1920|0),160)|0;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = (($6) + ($4<<2)|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pStmt>>2] = $8;
 $9 = HEAP32[$pStmt>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if (!($10)) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ($11|0)==(18);
   $13 = HEAP32[$1>>2]|0;
   do {
    if ($12) {
     $14 = (($azSql) + ($13<<2)|0);
     $15 = HEAP32[$14>>2]|0;
     $16 = HEAP32[$0>>2]|0;
     $17 = ((($16)) + 16|0);
     $18 = HEAP32[$17>>2]|0;
     $19 = HEAP32[$0>>2]|0;
     $20 = ((($19)) + 20|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = HEAP32[$0>>2]|0;
     $23 = ((($22)) + 220|0);
     $24 = HEAP32[$23>>2]|0;
     HEAP32[$vararg_buffer>>2] = $18;
     $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr1>>2] = $21;
     $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
     HEAP32[$vararg_ptr2>>2] = $24;
     $25 = (_sqlite3_mprintf($15,$vararg_buffer)|0);
     HEAP32[$zSql>>2] = $25;
    } else {
     $26 = ($13|0)==(7);
     $27 = HEAP32[$1>>2]|0;
     $28 = (($azSql) + ($27<<2)|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = HEAP32[$0>>2]|0;
     if ($26) {
      $31 = ((($30)) + 216|0);
      $32 = HEAP32[$31>>2]|0;
      HEAP32[$vararg_buffer3>>2] = $32;
      $33 = (_sqlite3_mprintf($29,$vararg_buffer3)|0);
      HEAP32[$zSql>>2] = $33;
      break;
     } else {
      $34 = ((($30)) + 16|0);
      $35 = HEAP32[$34>>2]|0;
      $36 = HEAP32[$0>>2]|0;
      $37 = ((($36)) + 20|0);
      $38 = HEAP32[$37>>2]|0;
      HEAP32[$vararg_buffer6>>2] = $35;
      $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
      HEAP32[$vararg_ptr9>>2] = $38;
      $39 = (_sqlite3_mprintf($29,$vararg_buffer6)|0);
      HEAP32[$zSql>>2] = $39;
      break;
     }
    }
   } while(0);
   $40 = HEAP32[$zSql>>2]|0;
   $41 = ($40|0)!=(0|0);
   if ($41) {
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 12|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = HEAP32[$zSql>>2]|0;
    $46 = (_sqlite3_prepare_v2($44,$45,-1,$pStmt,0)|0);
    HEAP32[$rc>>2] = $46;
    $47 = HEAP32[$zSql>>2]|0;
    _sqlite3_free($47);
    $48 = HEAP32[$pStmt>>2]|0;
    $49 = HEAP32[$1>>2]|0;
    $50 = HEAP32[$0>>2]|0;
    $51 = ((($50)) + 56|0);
    $52 = (($51) + ($49<<2)|0);
    HEAP32[$52>>2] = $48;
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    break;
   }
  }
 } while(0);
 $53 = HEAP32[$3>>2]|0;
 $54 = ($53|0)!=(0|0);
 if (!($54)) {
  $72 = HEAP32[$pStmt>>2]|0;
  $73 = HEAP32[$2>>2]|0;
  HEAP32[$73>>2] = $72;
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 $55 = HEAP32[$pStmt>>2]|0;
 $56 = (_sqlite3_bind_parameter_count($55)|0);
 HEAP32[$nParam>>2] = $56;
 HEAP32[$i>>2] = 0;
 while(1) {
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)==(0);
  if (!($58)) {
   label = 15;
   break;
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$nParam>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   label = 15;
   break;
  }
  $62 = HEAP32[$pStmt>>2]|0;
  $63 = HEAP32[$i>>2]|0;
  $64 = (($63) + 1)|0;
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$3>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (_sqlite3_bind_value($62,$64,$68)|0);
  HEAP32[$rc>>2] = $69;
  $70 = HEAP32[$i>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$i>>2] = $71;
 }
 if ((label|0) == 15) {
  $72 = HEAP32[$pStmt>>2]|0;
  $73 = HEAP32[$2>>2]|0;
  HEAP32[$73>>2] = $72;
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 return (0)|0;
}
function _sqlite3Fts3SegReaderCursor($p,$iLangid,$iIndex,$iLevel,$zTerm,$nTerm,$isPrefix,$isScan,$pCsr) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $isPrefix = $isPrefix|0;
 $isScan = $isScan|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 8|0;
 $7 = sp + 4|0;
 $8 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $isPrefix;
 HEAP32[$7>>2] = $isScan;
 HEAP32[$8>>2] = $pCsr;
 $9 = HEAP32[$8>>2]|0;
 dest=$9; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$6>>2]|0;
 $17 = HEAP32[$7>>2]|0;
 $18 = HEAP32[$8>>2]|0;
 $19 = (_fts3SegReaderCursor($10,$11,$12,$13,$14,$15,$16,$17,$18)|0);
 STACKTOP = sp;return ($19|0);
}
function _fts3SegmentMaxLevel($p,$iLangid,$iIndex,$pnMax) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $pnMax = $pnMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $iIndex;
 HEAP32[$4>>2] = $pnMax;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_fts3SqlStmt($5,15,$pStmt,0)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $10 = HEAP32[$pStmt>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_getAbsoluteLevel($11,$12,$13,0)|0);
 $15 = tempRet0;
 (_sqlite3_bind_int64($10,1,$14,$15)|0);
 $16 = HEAP32[$pStmt>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$3>>2]|0;
 $20 = (_getAbsoluteLevel($17,$18,$19,1023)|0);
 $21 = tempRet0;
 (_sqlite3_bind_int64($16,2,$20,$21)|0);
 $22 = HEAP32[$pStmt>>2]|0;
 $23 = (_sqlite3_step($22)|0);
 $24 = (100)==($23|0);
 if ($24) {
  $25 = HEAP32[$pStmt>>2]|0;
  $26 = (_sqlite3_column_int64($25,0)|0);
  $27 = tempRet0;
  $28 = HEAP32[$4>>2]|0;
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $27;
 }
 $33 = HEAP32[$pStmt>>2]|0;
 $34 = (_sqlite3_reset($33)|0);
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _getAbsoluteLevel($p,$iLangid,$iIndex,$iLevel) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iBase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $iBase = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(0);
 $6 = $5 << 31 >> 31;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 248|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (___muldi3(($4|0),($6|0),($9|0),($11|0))|0);
 $13 = tempRet0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)<(0);
 $16 = $15 << 31 >> 31;
 $17 = (_i64Add(($12|0),($13|0),($14|0),($16|0))|0);
 $18 = tempRet0;
 $19 = (___muldi3(($17|0),($18|0),1024,0)|0);
 $20 = tempRet0;
 $21 = $iBase;
 $22 = $21;
 HEAP32[$22>>2] = $19;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = $iBase;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$3>>2]|0;
 $32 = ($31|0)<(0);
 $33 = $32 << 31 >> 31;
 $34 = (_i64Add(($27|0),($30|0),($31|0),($33|0))|0);
 $35 = tempRet0;
 tempRet0 = ($35);
 STACKTOP = sp;return ($34|0);
}
function _fts3AllocateSegdirIdx($p,$iLangid,$iIndex,$iLevel,$piIdx) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $piIdx = $piIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iNext = 0, $pNextIdx = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $rc = sp + 8|0;
 $pNextIdx = sp + 4|0;
 $iNext = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $piIdx;
 HEAP32[$iNext>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_fts3SqlStmt($5,8,$pNextIdx,0)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$pNextIdx>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$3>>2]|0;
  $14 = (_getAbsoluteLevel($10,$11,$12,$13)|0);
  $15 = tempRet0;
  (_sqlite3_bind_int64($9,1,$14,$15)|0);
  $16 = HEAP32[$pNextIdx>>2]|0;
  $17 = (_sqlite3_step($16)|0);
  $18 = (100)==($17|0);
  if ($18) {
   $19 = HEAP32[$pNextIdx>>2]|0;
   $20 = (_sqlite3_column_int($19,0)|0);
   HEAP32[$iNext>>2] = $20;
  }
  $21 = HEAP32[$pNextIdx>>2]|0;
  $22 = (_sqlite3_reset($21)|0);
  HEAP32[$rc>>2] = $22;
 }
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)==(0);
 if (!($24)) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $25 = HEAP32[$iNext>>2]|0;
 $26 = ($25|0)>=(16);
 if ($26) {
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = (_fts3SegmentMerge($27,$28,$29,$30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$4>>2]|0;
  HEAP32[$32>>2] = 0;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  $33 = HEAP32[$iNext>>2]|0;
  $34 = HEAP32[$4>>2]|0;
  HEAP32[$34>>2] = $33;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _sqlite3Fts3SegReaderStart($p,$pCsr,$pFilter) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 $pFilter = $pFilter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pFilter;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_fts3SegReaderStart($6,$7,$9,$12)|0);
 STACKTOP = sp;return ($13|0);
}
function _sqlite3Fts3SegReaderStep($p,$pCsr) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $aNew = 0, $apSegment = 0;
 var $i = 0, $iDelta = 0, $iDocid = 0, $iPrev = 0, $isColFilter = 0, $isFirst = 0, $isIgnoreEmpty = 0, $isPrefix = 0, $isRequirePos = 0, $isScan = 0, $j = 0, $nByte = 0, $nDoclist = 0, $nList = 0, $nMerge = 0, $nSegment = 0, $nWrite = 0, $or$cond = 0, $or$cond10 = 0, $or$cond3 = 0;
 var $or$cond5 = 0, $or$cond8 = 0, $pFilter = 0, $pList = 0, $pSeg = 0, $rc = 0, $xCmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $rc = sp + 108|0;
 $isIgnoreEmpty = sp + 104|0;
 $isRequirePos = sp + 100|0;
 $isColFilter = sp + 96|0;
 $isPrefix = sp + 92|0;
 $isScan = sp + 88|0;
 $isFirst = sp + 84|0;
 $apSegment = sp + 80|0;
 $nSegment = sp + 76|0;
 $pFilter = sp + 72|0;
 $xCmp = sp + 68|0;
 $nMerge = sp + 64|0;
 $i = sp + 60|0;
 $pSeg = sp + 56|0;
 $nDoclist = sp + 52|0;
 $iPrev = sp + 16|0;
 $j = sp + 48|0;
 $pList = sp + 44|0;
 $nList = sp + 40|0;
 $nByte = sp + 36|0;
 $iDocid = sp + 8|0;
 $iDelta = sp;
 $aNew = sp + 32|0;
 $a = sp + 28|0;
 $nWrite = sp + 24|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7 & 2;
 HEAP32[$isIgnoreEmpty>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 1;
 HEAP32[$isRequirePos>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $19 & 4;
 HEAP32[$isColFilter>>2] = $20;
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 & 8;
 HEAP32[$isPrefix>>2] = $26;
 $27 = HEAP32[$2>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ((($29)) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = $31 & 16;
 HEAP32[$isScan>>2] = $32;
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ((($35)) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $37 & 32;
 HEAP32[$isFirst>>2] = $38;
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 HEAP32[$apSegment>>2] = $40;
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$nSegment>>2] = $43;
 $44 = HEAP32[$2>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 HEAP32[$pFilter>>2] = $46;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 231|0);
 $49 = HEAP8[$48>>0]|0;
 $50 = $49&255;
 $51 = ($50|0)!=(0);
 $52 = $51 ? 182 : 183;
 HEAP32[$xCmp>>2] = $52;
 $53 = HEAP32[$2>>2]|0;
 $54 = ((($53)) + 4|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = ($55|0)==(0);
 if ($56) {
  HEAP32[$0>>2] = 0;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 L4: while(1) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $57 = HEAP32[$i>>2]|0;
   $58 = HEAP32[$2>>2]|0;
   $59 = ((($58)) + 8|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ($57|0)<($60|0);
   if (!($61)) {
    break;
   }
   $62 = HEAP32[$i>>2]|0;
   $63 = HEAP32[$apSegment>>2]|0;
   $64 = (($63) + ($62<<2)|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$pSeg>>2] = $65;
   $66 = HEAP32[$pSeg>>2]|0;
   $67 = ((($66)) + 4|0);
   $68 = HEAP8[$67>>0]|0;
   $69 = ($68<<24>>24)!=(0);
   if ($69) {
    $70 = HEAP32[$pSeg>>2]|0;
    _fts3SegReaderSetEof($70);
   } else {
    $71 = HEAP32[$1>>2]|0;
    $72 = HEAP32[$pSeg>>2]|0;
    $73 = (_fts3SegReaderNext($71,$72,0)|0);
    HEAP32[$rc>>2] = $73;
   }
   $74 = HEAP32[$rc>>2]|0;
   $75 = ($74|0)!=(0);
   if ($75) {
    label = 9;
    break L4;
   }
   $77 = HEAP32[$i>>2]|0;
   $78 = (($77) + 1)|0;
   HEAP32[$i>>2] = $78;
  }
  $79 = HEAP32[$apSegment>>2]|0;
  $80 = HEAP32[$nSegment>>2]|0;
  $81 = HEAP32[$2>>2]|0;
  $82 = ((($81)) + 8|0);
  $83 = HEAP32[$82>>2]|0;
  _fts3SegReaderSort($79,$80,$83,184);
  $84 = HEAP32[$2>>2]|0;
  $85 = ((($84)) + 8|0);
  HEAP32[$85>>2] = 0;
  $86 = HEAP32[$apSegment>>2]|0;
  $87 = HEAP32[$86>>2]|0;
  $88 = ((($87)) + 40|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = ($89|0)==(0|0);
  if ($90) {
   label = 61;
   break;
  }
  $91 = HEAP32[$apSegment>>2]|0;
  $92 = HEAP32[$91>>2]|0;
  $93 = ((($92)) + 60|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = HEAP32[$2>>2]|0;
  $96 = ((($95)) + 44|0);
  HEAP32[$96>>2] = $94;
  $97 = HEAP32[$apSegment>>2]|0;
  $98 = HEAP32[$97>>2]|0;
  $99 = ((($98)) + 64|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = HEAP32[$2>>2]|0;
  $102 = ((($101)) + 40|0);
  HEAP32[$102>>2] = $100;
  $103 = HEAP32[$pFilter>>2]|0;
  $104 = HEAP32[$103>>2]|0;
  $105 = ($104|0)==(0|0);
  $106 = HEAP32[$isScan>>2]|0;
  $107 = ($106|0)!=(0);
  $or$cond = $105 | $107;
  if (!($or$cond)) {
   $108 = HEAP32[$2>>2]|0;
   $109 = ((($108)) + 44|0);
   $110 = HEAP32[$109>>2]|0;
   $111 = HEAP32[$pFilter>>2]|0;
   $112 = ((($111)) + 4|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = ($110|0)<($113|0);
   if ($114) {
    label = 61;
    break;
   }
   $115 = HEAP32[$isPrefix>>2]|0;
   $116 = ($115|0)!=(0);
   if (!($116)) {
    $117 = HEAP32[$2>>2]|0;
    $118 = ((($117)) + 44|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = HEAP32[$pFilter>>2]|0;
    $121 = ((($120)) + 4|0);
    $122 = HEAP32[$121>>2]|0;
    $123 = ($119|0)>($122|0);
    if ($123) {
     label = 61;
     break;
    }
   }
   $124 = HEAP32[$2>>2]|0;
   $125 = ((($124)) + 40|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = HEAP32[$pFilter>>2]|0;
   $128 = HEAP32[$127>>2]|0;
   $129 = HEAP32[$pFilter>>2]|0;
   $130 = ((($129)) + 4|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = (_memcmp($126,$128,$131)|0);
   $133 = ($132|0)!=(0);
   if ($133) {
    label = 61;
    break;
   }
  }
  HEAP32[$nMerge>>2] = 1;
  while(1) {
   $134 = HEAP32[$nMerge>>2]|0;
   $135 = HEAP32[$nSegment>>2]|0;
   $136 = ($134|0)<($135|0);
   if ($136) {
    $137 = HEAP32[$nMerge>>2]|0;
    $138 = HEAP32[$apSegment>>2]|0;
    $139 = (($138) + ($137<<2)|0);
    $140 = HEAP32[$139>>2]|0;
    $141 = ((($140)) + 40|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ($142|0)!=(0|0);
    if ($143) {
     $144 = HEAP32[$nMerge>>2]|0;
     $145 = HEAP32[$apSegment>>2]|0;
     $146 = (($145) + ($144<<2)|0);
     $147 = HEAP32[$146>>2]|0;
     $148 = ((($147)) + 60|0);
     $149 = HEAP32[$148>>2]|0;
     $150 = HEAP32[$2>>2]|0;
     $151 = ((($150)) + 44|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = ($149|0)==($152|0);
     if ($153) {
      $154 = HEAP32[$2>>2]|0;
      $155 = ((($154)) + 40|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = HEAP32[$nMerge>>2]|0;
      $158 = HEAP32[$apSegment>>2]|0;
      $159 = (($158) + ($157<<2)|0);
      $160 = HEAP32[$159>>2]|0;
      $161 = ((($160)) + 64|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = HEAP32[$2>>2]|0;
      $164 = ((($163)) + 44|0);
      $165 = HEAP32[$164>>2]|0;
      $166 = (_memcmp($156,$162,$165)|0);
      $167 = (0)==($166|0);
      $476 = $167;
     } else {
      $476 = 0;
     }
    } else {
     $476 = 0;
    }
   } else {
    $476 = 0;
   }
   $168 = HEAP32[$nMerge>>2]|0;
   if (!($476)) {
    break;
   }
   $169 = (($168) + 1)|0;
   HEAP32[$nMerge>>2] = $169;
  }
  $170 = ($168|0)!=(1);
  $171 = HEAP32[$isIgnoreEmpty>>2]|0;
  $172 = ($171|0)!=(0);
  $or$cond3 = $170 | $172;
  $173 = HEAP32[$isFirst>>2]|0;
  $174 = ($173|0)!=(0);
  $or$cond5 = $or$cond3 | $174;
  do {
   if ($or$cond5) {
    label = 32;
   } else {
    $175 = HEAP32[$1>>2]|0;
    $176 = ((($175)) + 231|0);
    $177 = HEAP8[$176>>0]|0;
    $178 = $177&255;
    $179 = ($178|0)==(0);
    if (!($179)) {
     $180 = HEAP32[$apSegment>>2]|0;
     $181 = HEAP32[$180>>2]|0;
     $182 = ((($181)) + 56|0);
     $183 = HEAP32[$182>>2]|0;
     $184 = ($183|0)!=(0|0);
     $185 = $184&1;
     $186 = ($185|0)==(0);
     if (!($186)) {
      label = 32;
      break;
     }
    }
    $187 = HEAP32[$apSegment>>2]|0;
    $188 = HEAP32[$187>>2]|0;
    $189 = ((($188)) + 76|0);
    $190 = HEAP32[$189>>2]|0;
    $191 = HEAP32[$2>>2]|0;
    $192 = ((($191)) + 52|0);
    HEAP32[$192>>2] = $190;
    $193 = HEAP32[$apSegment>>2]|0;
    $194 = HEAP32[$193>>2]|0;
    $195 = ((($194)) + 56|0);
    $196 = HEAP32[$195>>2]|0;
    $197 = ($196|0)!=(0|0);
    if ($197) {
     $198 = HEAP32[$2>>2]|0;
     $199 = HEAP32[$apSegment>>2]|0;
     $200 = HEAP32[$199>>2]|0;
     $201 = ((($200)) + 72|0);
     $202 = HEAP32[$201>>2]|0;
     $203 = HEAP32[$2>>2]|0;
     $204 = ((($203)) + 52|0);
     $205 = HEAP32[$204>>2]|0;
     $206 = (_fts3MsrBufferData($198,$202,$205)|0);
     HEAP32[$rc>>2] = $206;
     $207 = HEAP32[$2>>2]|0;
     $208 = ((($207)) + 16|0);
     $209 = HEAP32[$208>>2]|0;
     $210 = HEAP32[$2>>2]|0;
     $211 = ((($210)) + 48|0);
     HEAP32[$211>>2] = $209;
    } else {
     $212 = HEAP32[$apSegment>>2]|0;
     $213 = HEAP32[$212>>2]|0;
     $214 = ((($213)) + 72|0);
     $215 = HEAP32[$214>>2]|0;
     $216 = HEAP32[$2>>2]|0;
     $217 = ((($216)) + 48|0);
     HEAP32[$217>>2] = $215;
    }
    $218 = HEAP32[$rc>>2]|0;
    $219 = ($218|0)==(0);
    if ($219) {
     HEAP32[$rc>>2] = 100;
    }
   }
  } while(0);
  if ((label|0) == 32) {
   label = 0;
   HEAP32[$nDoclist>>2] = 0;
   $220 = $iPrev;
   $221 = $220;
   HEAP32[$221>>2] = 0;
   $222 = (($220) + 4)|0;
   $223 = $222;
   HEAP32[$223>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $224 = HEAP32[$i>>2]|0;
    $225 = HEAP32[$nMerge>>2]|0;
    $226 = ($224|0)<($225|0);
    if (!($226)) {
     break;
    }
    $227 = HEAP32[$1>>2]|0;
    $228 = HEAP32[$i>>2]|0;
    $229 = HEAP32[$apSegment>>2]|0;
    $230 = (($229) + ($228<<2)|0);
    $231 = HEAP32[$230>>2]|0;
    (_fts3SegReaderFirstDocid($227,$231)|0);
    $232 = HEAP32[$i>>2]|0;
    $233 = (($232) + 1)|0;
    HEAP32[$i>>2] = $233;
   }
   $234 = HEAP32[$apSegment>>2]|0;
   $235 = HEAP32[$nMerge>>2]|0;
   $236 = HEAP32[$nMerge>>2]|0;
   $237 = HEAP32[$xCmp>>2]|0;
   _fts3SegReaderSort($234,$235,$236,$237);
   while(1) {
    $238 = HEAP32[$apSegment>>2]|0;
    $239 = HEAP32[$238>>2]|0;
    $240 = ((($239)) + 80|0);
    $241 = HEAP32[$240>>2]|0;
    $242 = ($241|0)!=(0|0);
    if (!($242)) {
     break;
    }
    HEAP32[$pList>>2] = 0;
    HEAP32[$nList>>2] = 0;
    $243 = HEAP32[$apSegment>>2]|0;
    $244 = HEAP32[$243>>2]|0;
    $245 = ((($244)) + 88|0);
    $246 = $245;
    $247 = $246;
    $248 = HEAP32[$247>>2]|0;
    $249 = (($246) + 4)|0;
    $250 = $249;
    $251 = HEAP32[$250>>2]|0;
    $252 = $iDocid;
    $253 = $252;
    HEAP32[$253>>2] = $248;
    $254 = (($252) + 4)|0;
    $255 = $254;
    HEAP32[$255>>2] = $251;
    $256 = HEAP32[$1>>2]|0;
    $257 = HEAP32[$apSegment>>2]|0;
    $258 = HEAP32[$257>>2]|0;
    (_fts3SegReaderNextDocid($256,$258,$pList,$nList)|0);
    HEAP32[$j>>2] = 1;
    while(1) {
     $259 = HEAP32[$j>>2]|0;
     $260 = HEAP32[$nMerge>>2]|0;
     $261 = ($259|0)<($260|0);
     if (!($261)) {
      break;
     }
     $262 = HEAP32[$j>>2]|0;
     $263 = HEAP32[$apSegment>>2]|0;
     $264 = (($263) + ($262<<2)|0);
     $265 = HEAP32[$264>>2]|0;
     $266 = ((($265)) + 80|0);
     $267 = HEAP32[$266>>2]|0;
     $268 = ($267|0)!=(0|0);
     if (!($268)) {
      break;
     }
     $269 = HEAP32[$j>>2]|0;
     $270 = HEAP32[$apSegment>>2]|0;
     $271 = (($270) + ($269<<2)|0);
     $272 = HEAP32[$271>>2]|0;
     $273 = ((($272)) + 88|0);
     $274 = $273;
     $275 = $274;
     $276 = HEAP32[$275>>2]|0;
     $277 = (($274) + 4)|0;
     $278 = $277;
     $279 = HEAP32[$278>>2]|0;
     $280 = $iDocid;
     $281 = $280;
     $282 = HEAP32[$281>>2]|0;
     $283 = (($280) + 4)|0;
     $284 = $283;
     $285 = HEAP32[$284>>2]|0;
     $286 = ($276|0)==($282|0);
     $287 = ($279|0)==($285|0);
     $288 = $286 & $287;
     if (!($288)) {
      break;
     }
     $289 = HEAP32[$1>>2]|0;
     $290 = HEAP32[$j>>2]|0;
     $291 = HEAP32[$apSegment>>2]|0;
     $292 = (($291) + ($290<<2)|0);
     $293 = HEAP32[$292>>2]|0;
     (_fts3SegReaderNextDocid($289,$293,0,0)|0);
     $294 = HEAP32[$j>>2]|0;
     $295 = (($294) + 1)|0;
     HEAP32[$j>>2] = $295;
    }
    $296 = HEAP32[$isColFilter>>2]|0;
    $297 = ($296|0)!=(0);
    if ($297) {
     $298 = HEAP32[$pFilter>>2]|0;
     $299 = ((($298)) + 8|0);
     $300 = HEAP32[$299>>2]|0;
     _fts3ColumnFilter($300,0,$pList,$nList);
    }
    $301 = HEAP32[$isIgnoreEmpty>>2]|0;
    $302 = ($301|0)==(0);
    $303 = HEAP32[$nList>>2]|0;
    $304 = ($303|0)>(0);
    $or$cond8 = $302 | $304;
    do {
     if ($or$cond8) {
      $305 = HEAP32[$1>>2]|0;
      $306 = ((($305)) + 231|0);
      $307 = HEAP8[$306>>0]|0;
      $308 = $307&255;
      $309 = ($308|0)!=(0);
      $310 = HEAP32[$nDoclist>>2]|0;
      $311 = ($310|0)>(0);
      $or$cond10 = $309 & $311;
      if ($or$cond10) {
       $312 = $iPrev;
       $313 = $312;
       $314 = HEAP32[$313>>2]|0;
       $315 = (($312) + 4)|0;
       $316 = $315;
       $317 = HEAP32[$316>>2]|0;
       $318 = $iDocid;
       $319 = $318;
       $320 = HEAP32[$319>>2]|0;
       $321 = (($318) + 4)|0;
       $322 = $321;
       $323 = HEAP32[$322>>2]|0;
       $324 = (_i64Subtract(($314|0),($317|0),($320|0),($323|0))|0);
       $325 = tempRet0;
       $326 = $iDelta;
       $327 = $326;
       HEAP32[$327>>2] = $324;
       $328 = (($326) + 4)|0;
       $329 = $328;
       HEAP32[$329>>2] = $325;
      } else {
       $330 = $iDocid;
       $331 = $330;
       $332 = HEAP32[$331>>2]|0;
       $333 = (($330) + 4)|0;
       $334 = $333;
       $335 = HEAP32[$334>>2]|0;
       $336 = $iPrev;
       $337 = $336;
       $338 = HEAP32[$337>>2]|0;
       $339 = (($336) + 4)|0;
       $340 = $339;
       $341 = HEAP32[$340>>2]|0;
       $342 = (_i64Subtract(($332|0),($335|0),($338|0),($341|0))|0);
       $343 = tempRet0;
       $344 = $iDelta;
       $345 = $344;
       HEAP32[$345>>2] = $342;
       $346 = (($344) + 4)|0;
       $347 = $346;
       HEAP32[$347>>2] = $343;
      }
      $348 = $iDelta;
      $349 = $348;
      $350 = HEAP32[$349>>2]|0;
      $351 = (($348) + 4)|0;
      $352 = $351;
      $353 = HEAP32[$352>>2]|0;
      $354 = (_sqlite3Fts3VarintLen($350,$353)|0);
      $355 = HEAP32[$isRequirePos>>2]|0;
      $356 = ($355|0)!=(0);
      $357 = HEAP32[$nList>>2]|0;
      $358 = (($357) + 1)|0;
      $359 = $356 ? $358 : 0;
      $360 = (($354) + ($359))|0;
      HEAP32[$nByte>>2] = $360;
      $361 = HEAP32[$nDoclist>>2]|0;
      $362 = HEAP32[$nByte>>2]|0;
      $363 = (($361) + ($362))|0;
      $364 = HEAP32[$2>>2]|0;
      $365 = ((($364)) + 20|0);
      $366 = HEAP32[$365>>2]|0;
      $367 = ($363|0)>($366|0);
      if ($367) {
       $368 = HEAP32[$nDoclist>>2]|0;
       $369 = HEAP32[$nByte>>2]|0;
       $370 = (($368) + ($369))|0;
       $371 = $370<<1;
       $372 = HEAP32[$2>>2]|0;
       $373 = ((($372)) + 20|0);
       HEAP32[$373>>2] = $371;
       $374 = HEAP32[$2>>2]|0;
       $375 = ((($374)) + 16|0);
       $376 = HEAP32[$375>>2]|0;
       $377 = HEAP32[$2>>2]|0;
       $378 = ((($377)) + 20|0);
       $379 = HEAP32[$378>>2]|0;
       $380 = (_sqlite3_realloc($376,$379)|0);
       HEAP32[$aNew>>2] = $380;
       $381 = HEAP32[$aNew>>2]|0;
       $382 = ($381|0)!=(0|0);
       if (!($382)) {
        label = 50;
        break L4;
       }
       $383 = HEAP32[$aNew>>2]|0;
       $384 = HEAP32[$2>>2]|0;
       $385 = ((($384)) + 16|0);
       HEAP32[$385>>2] = $383;
      }
      $386 = HEAP32[$isFirst>>2]|0;
      $387 = ($386|0)!=(0);
      $388 = HEAP32[$nDoclist>>2]|0;
      $389 = HEAP32[$2>>2]|0;
      $390 = ((($389)) + 16|0);
      $391 = HEAP32[$390>>2]|0;
      $392 = (($391) + ($388)|0);
      if ($387) {
       HEAP32[$a>>2] = $392;
       $393 = $iDelta;
       $394 = $393;
       $395 = HEAP32[$394>>2]|0;
       $396 = (($393) + 4)|0;
       $397 = $396;
       $398 = HEAP32[$397>>2]|0;
       $399 = HEAP32[$pList>>2]|0;
       $400 = HEAP32[$nList>>2]|0;
       $401 = HEAP32[$a>>2]|0;
       $402 = (_sqlite3Fts3FirstFilter($395,$398,$399,$400,$401)|0);
       HEAP32[$nWrite>>2] = $402;
       $403 = HEAP32[$nWrite>>2]|0;
       $404 = ($403|0)!=(0);
       if (!($404)) {
        break;
       }
       $405 = $iDocid;
       $406 = $405;
       $407 = HEAP32[$406>>2]|0;
       $408 = (($405) + 4)|0;
       $409 = $408;
       $410 = HEAP32[$409>>2]|0;
       $411 = $iPrev;
       $412 = $411;
       HEAP32[$412>>2] = $407;
       $413 = (($411) + 4)|0;
       $414 = $413;
       HEAP32[$414>>2] = $410;
       $415 = HEAP32[$nWrite>>2]|0;
       $416 = HEAP32[$nDoclist>>2]|0;
       $417 = (($416) + ($415))|0;
       HEAP32[$nDoclist>>2] = $417;
       break;
      } else {
       $418 = $iDelta;
       $419 = $418;
       $420 = HEAP32[$419>>2]|0;
       $421 = (($418) + 4)|0;
       $422 = $421;
       $423 = HEAP32[$422>>2]|0;
       $424 = (_sqlite3Fts3PutVarint($392,$420,$423)|0);
       $425 = HEAP32[$nDoclist>>2]|0;
       $426 = (($425) + ($424))|0;
       HEAP32[$nDoclist>>2] = $426;
       $427 = $iDocid;
       $428 = $427;
       $429 = HEAP32[$428>>2]|0;
       $430 = (($427) + 4)|0;
       $431 = $430;
       $432 = HEAP32[$431>>2]|0;
       $433 = $iPrev;
       $434 = $433;
       HEAP32[$434>>2] = $429;
       $435 = (($433) + 4)|0;
       $436 = $435;
       HEAP32[$436>>2] = $432;
       $437 = HEAP32[$isRequirePos>>2]|0;
       $438 = ($437|0)!=(0);
       if (!($438)) {
        break;
       }
       $439 = HEAP32[$nDoclist>>2]|0;
       $440 = HEAP32[$2>>2]|0;
       $441 = ((($440)) + 16|0);
       $442 = HEAP32[$441>>2]|0;
       $443 = (($442) + ($439)|0);
       $444 = HEAP32[$pList>>2]|0;
       $445 = HEAP32[$nList>>2]|0;
       _memcpy(($443|0),($444|0),($445|0))|0;
       $446 = HEAP32[$nList>>2]|0;
       $447 = HEAP32[$nDoclist>>2]|0;
       $448 = (($447) + ($446))|0;
       HEAP32[$nDoclist>>2] = $448;
       $449 = HEAP32[$nDoclist>>2]|0;
       $450 = (($449) + 1)|0;
       HEAP32[$nDoclist>>2] = $450;
       $451 = HEAP32[$2>>2]|0;
       $452 = ((($451)) + 16|0);
       $453 = HEAP32[$452>>2]|0;
       $454 = (($453) + ($449)|0);
       HEAP8[$454>>0] = 0;
       break;
      }
     }
    } while(0);
    $455 = HEAP32[$apSegment>>2]|0;
    $456 = HEAP32[$nMerge>>2]|0;
    $457 = HEAP32[$j>>2]|0;
    $458 = HEAP32[$xCmp>>2]|0;
    _fts3SegReaderSort($455,$456,$457,$458);
   }
   $459 = HEAP32[$nDoclist>>2]|0;
   $460 = ($459|0)>(0);
   if ($460) {
    $461 = HEAP32[$2>>2]|0;
    $462 = ((($461)) + 16|0);
    $463 = HEAP32[$462>>2]|0;
    $464 = HEAP32[$2>>2]|0;
    $465 = ((($464)) + 48|0);
    HEAP32[$465>>2] = $463;
    $466 = HEAP32[$nDoclist>>2]|0;
    $467 = HEAP32[$2>>2]|0;
    $468 = ((($467)) + 52|0);
    HEAP32[$468>>2] = $466;
    HEAP32[$rc>>2] = 100;
   }
  }
  $469 = HEAP32[$nMerge>>2]|0;
  $470 = HEAP32[$2>>2]|0;
  $471 = ((($470)) + 8|0);
  HEAP32[$471>>2] = $469;
  $472 = HEAP32[$rc>>2]|0;
  $473 = ($472|0)==(0);
  if (!($473)) {
   label = 61;
   break;
  }
 }
 if ((label|0) == 9) {
  $76 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $76;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 else if ((label|0) == 50) {
  HEAP32[$0>>2] = 7;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 else if ((label|0) == 61) {
  $474 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $474;
  $475 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($475|0);
 }
 return (0)|0;
}
function _fts3SegWriterAdd($p,$ppWriter,$isCopyTerm,$zTerm,$nTerm,$aDoclist,$nDoclist) {
 $p = $p|0;
 $ppWriter = $ppWriter|0;
 $isCopyTerm = $isCopyTerm|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $nData = 0, $nPrefix = 0, $nReq = 0, $nSuffix = 0, $pStmt = 0;
 var $pWriter = 0, $rc = 0, $rc1 = 0, $zNew = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $6 = sp + 44|0;
 $7 = sp + 40|0;
 $nPrefix = sp + 36|0;
 $nSuffix = sp + 32|0;
 $nReq = sp + 28|0;
 $nData = sp + 24|0;
 $pWriter = sp + 20|0;
 $rc = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc1 = sp + 8|0;
 $aNew = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $ppWriter;
 HEAP32[$3>>2] = $isCopyTerm;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $aDoclist;
 HEAP32[$7>>2] = $nDoclist;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pWriter>>2] = $9;
 $10 = HEAP32[$pWriter>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $12 = (_sqlite3_malloc(64)|0);
  HEAP32[$pWriter>>2] = $12;
  $13 = HEAP32[$pWriter>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   HEAP32[$0>>2] = 7;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $15 = HEAP32[$pWriter>>2]|0;
  dest=$15; stop=dest+64|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $16 = HEAP32[$pWriter>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  HEAP32[$17>>2] = $16;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 224|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3_malloc($20)|0);
  $22 = HEAP32[$pWriter>>2]|0;
  $23 = ((($22)) + 48|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$pWriter>>2]|0;
  $25 = ((($24)) + 48|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   HEAP32[$0>>2] = 7;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 224|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$pWriter>>2]|0;
  $32 = ((($31)) + 40|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$1>>2]|0;
  $34 = (_fts3SqlStmt($33,10,$pStmt,0)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $37;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $38 = HEAP32[$pStmt>>2]|0;
  $39 = (_sqlite3_step($38)|0);
  $40 = (100)==($39|0);
  if ($40) {
   $41 = HEAP32[$pStmt>>2]|0;
   $42 = (_sqlite3_column_int64($41,0)|0);
   $43 = tempRet0;
   $44 = HEAP32[$pWriter>>2]|0;
   $45 = ((($44)) + 16|0);
   $46 = $45;
   $47 = $46;
   HEAP32[$47>>2] = $42;
   $48 = (($46) + 4)|0;
   $49 = $48;
   HEAP32[$49>>2] = $43;
   $50 = HEAP32[$pWriter>>2]|0;
   $51 = ((($50)) + 16|0);
   $52 = $51;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   $55 = (($52) + 4)|0;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = HEAP32[$pWriter>>2]|0;
   $59 = ((($58)) + 8|0);
   $60 = $59;
   $61 = $60;
   HEAP32[$61>>2] = $54;
   $62 = (($60) + 4)|0;
   $63 = $62;
   HEAP32[$63>>2] = $57;
  }
  $64 = HEAP32[$pStmt>>2]|0;
  $65 = (_sqlite3_reset($64)|0);
  HEAP32[$rc>>2] = $65;
  $66 = HEAP32[$rc>>2]|0;
  $67 = ($66|0)!=(0);
  if ($67) {
   $68 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $68;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
 }
 $69 = HEAP32[$pWriter>>2]|0;
 $70 = ((($69)) + 44|0);
 $71 = HEAP32[$70>>2]|0;
 HEAP32[$nData>>2] = $71;
 $72 = HEAP32[$pWriter>>2]|0;
 $73 = ((($72)) + 24|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = HEAP32[$pWriter>>2]|0;
 $76 = ((($75)) + 28|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = HEAP32[$4>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = (_fts3PrefixCompress($74,$77,$78,$79)|0);
 HEAP32[$nPrefix>>2] = $80;
 $81 = HEAP32[$5>>2]|0;
 $82 = HEAP32[$nPrefix>>2]|0;
 $83 = (($81) - ($82))|0;
 HEAP32[$nSuffix>>2] = $83;
 $84 = HEAP32[$nPrefix>>2]|0;
 $85 = ($84|0)<(0);
 $86 = $85 << 31 >> 31;
 $87 = (_sqlite3Fts3VarintLen($84,$86)|0);
 $88 = HEAP32[$nSuffix>>2]|0;
 $89 = ($88|0)<(0);
 $90 = $89 << 31 >> 31;
 $91 = (_sqlite3Fts3VarintLen($88,$90)|0);
 $92 = (($87) + ($91))|0;
 $93 = HEAP32[$nSuffix>>2]|0;
 $94 = (($92) + ($93))|0;
 $95 = HEAP32[$7>>2]|0;
 $96 = ($95|0)<(0);
 $97 = $96 << 31 >> 31;
 $98 = (_sqlite3Fts3VarintLen($95,$97)|0);
 $99 = (($94) + ($98))|0;
 $100 = HEAP32[$7>>2]|0;
 $101 = (($99) + ($100))|0;
 HEAP32[$nReq>>2] = $101;
 $102 = HEAP32[$nData>>2]|0;
 $103 = ($102|0)>(0);
 do {
  if ($103) {
   $104 = HEAP32[$nData>>2]|0;
   $105 = HEAP32[$nReq>>2]|0;
   $106 = (($104) + ($105))|0;
   $107 = HEAP32[$1>>2]|0;
   $108 = ((($107)) + 224|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = ($106|0)>($109|0);
   if ($110) {
    $111 = HEAP32[$1>>2]|0;
    $112 = HEAP32[$pWriter>>2]|0;
    $113 = ((($112)) + 16|0);
    $114 = $113;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($114) + 4)|0;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = (_i64Add(($116|0),($119|0),1,0)|0);
    $121 = tempRet0;
    $122 = $113;
    $123 = $122;
    HEAP32[$123>>2] = $120;
    $124 = (($122) + 4)|0;
    $125 = $124;
    HEAP32[$125>>2] = $121;
    $126 = HEAP32[$pWriter>>2]|0;
    $127 = ((($126)) + 48|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$nData>>2]|0;
    $130 = (_fts3WriteSegment($111,$116,$119,$128,$129)|0);
    HEAP32[$rc1>>2] = $130;
    $131 = HEAP32[$rc1>>2]|0;
    $132 = ($131|0)!=(0);
    if ($132) {
     $133 = HEAP32[$rc1>>2]|0;
     HEAP32[$0>>2] = $133;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
    $134 = HEAP32[$1>>2]|0;
    $135 = ((($134)) + 52|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = (($136) + 1)|0;
    HEAP32[$135>>2] = $137;
    $138 = HEAP32[$1>>2]|0;
    $139 = HEAP32[$pWriter>>2]|0;
    $140 = HEAP32[$3>>2]|0;
    $141 = HEAP32[$4>>2]|0;
    $142 = HEAP32[$nPrefix>>2]|0;
    $143 = (($142) + 1)|0;
    $144 = (_fts3NodeAddTerm($138,$139,$140,$141,$143)|0);
    HEAP32[$rc1>>2] = $144;
    $145 = HEAP32[$rc1>>2]|0;
    $146 = ($145|0)!=(0);
    if (!($146)) {
     HEAP32[$nData>>2] = 0;
     $148 = HEAP32[$pWriter>>2]|0;
     $149 = ((($148)) + 28|0);
     HEAP32[$149>>2] = 0;
     HEAP32[$nPrefix>>2] = 0;
     $150 = HEAP32[$5>>2]|0;
     HEAP32[$nSuffix>>2] = $150;
     $151 = HEAP32[$5>>2]|0;
     $152 = ($151|0)<(0);
     $153 = $152 << 31 >> 31;
     $154 = (_sqlite3Fts3VarintLen($151,$153)|0);
     $155 = (1 + ($154))|0;
     $156 = HEAP32[$5>>2]|0;
     $157 = (($155) + ($156))|0;
     $158 = HEAP32[$7>>2]|0;
     $159 = ($158|0)<(0);
     $160 = $159 << 31 >> 31;
     $161 = (_sqlite3Fts3VarintLen($158,$160)|0);
     $162 = (($157) + ($161))|0;
     $163 = HEAP32[$7>>2]|0;
     $164 = (($162) + ($163))|0;
     HEAP32[$nReq>>2] = $164;
     break;
    }
    $147 = HEAP32[$rc1>>2]|0;
    HEAP32[$0>>2] = $147;
    $293 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($293|0);
   }
  }
 } while(0);
 $165 = HEAP32[$nReq>>2]|0;
 $166 = ($165|0)<(0);
 $167 = $166 << 31 >> 31;
 $168 = HEAP32[$pWriter>>2]|0;
 $169 = ((($168)) + 56|0);
 $170 = $169;
 $171 = $170;
 $172 = HEAP32[$171>>2]|0;
 $173 = (($170) + 4)|0;
 $174 = $173;
 $175 = HEAP32[$174>>2]|0;
 $176 = (_i64Add(($172|0),($175|0),($165|0),($167|0))|0);
 $177 = tempRet0;
 $178 = $169;
 $179 = $178;
 HEAP32[$179>>2] = $176;
 $180 = (($178) + 4)|0;
 $181 = $180;
 HEAP32[$181>>2] = $177;
 $182 = HEAP32[$nReq>>2]|0;
 $183 = HEAP32[$pWriter>>2]|0;
 $184 = ((($183)) + 40|0);
 $185 = HEAP32[$184>>2]|0;
 $186 = ($182|0)>($185|0);
 do {
  if ($186) {
   $187 = HEAP32[$pWriter>>2]|0;
   $188 = ((($187)) + 48|0);
   $189 = HEAP32[$188>>2]|0;
   $190 = HEAP32[$nReq>>2]|0;
   $191 = (_sqlite3_realloc($189,$190)|0);
   HEAP32[$aNew>>2] = $191;
   $192 = HEAP32[$aNew>>2]|0;
   $193 = ($192|0)!=(0|0);
   if ($193) {
    $194 = HEAP32[$aNew>>2]|0;
    $195 = HEAP32[$pWriter>>2]|0;
    $196 = ((($195)) + 48|0);
    HEAP32[$196>>2] = $194;
    $197 = HEAP32[$nReq>>2]|0;
    $198 = HEAP32[$pWriter>>2]|0;
    $199 = ((($198)) + 40|0);
    HEAP32[$199>>2] = $197;
    break;
   }
   HEAP32[$0>>2] = 7;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
 } while(0);
 $200 = HEAP32[$nData>>2]|0;
 $201 = HEAP32[$pWriter>>2]|0;
 $202 = ((($201)) + 48|0);
 $203 = HEAP32[$202>>2]|0;
 $204 = (($203) + ($200)|0);
 $205 = HEAP32[$nPrefix>>2]|0;
 $206 = ($205|0)<(0);
 $207 = $206 << 31 >> 31;
 $208 = (_sqlite3Fts3PutVarint($204,$205,$207)|0);
 $209 = HEAP32[$nData>>2]|0;
 $210 = (($209) + ($208))|0;
 HEAP32[$nData>>2] = $210;
 $211 = HEAP32[$nData>>2]|0;
 $212 = HEAP32[$pWriter>>2]|0;
 $213 = ((($212)) + 48|0);
 $214 = HEAP32[$213>>2]|0;
 $215 = (($214) + ($211)|0);
 $216 = HEAP32[$nSuffix>>2]|0;
 $217 = ($216|0)<(0);
 $218 = $217 << 31 >> 31;
 $219 = (_sqlite3Fts3PutVarint($215,$216,$218)|0);
 $220 = HEAP32[$nData>>2]|0;
 $221 = (($220) + ($219))|0;
 HEAP32[$nData>>2] = $221;
 $222 = HEAP32[$nData>>2]|0;
 $223 = HEAP32[$pWriter>>2]|0;
 $224 = ((($223)) + 48|0);
 $225 = HEAP32[$224>>2]|0;
 $226 = (($225) + ($222)|0);
 $227 = HEAP32[$nPrefix>>2]|0;
 $228 = HEAP32[$4>>2]|0;
 $229 = (($228) + ($227)|0);
 $230 = HEAP32[$nSuffix>>2]|0;
 _memcpy(($226|0),($229|0),($230|0))|0;
 $231 = HEAP32[$nSuffix>>2]|0;
 $232 = HEAP32[$nData>>2]|0;
 $233 = (($232) + ($231))|0;
 HEAP32[$nData>>2] = $233;
 $234 = HEAP32[$nData>>2]|0;
 $235 = HEAP32[$pWriter>>2]|0;
 $236 = ((($235)) + 48|0);
 $237 = HEAP32[$236>>2]|0;
 $238 = (($237) + ($234)|0);
 $239 = HEAP32[$7>>2]|0;
 $240 = ($239|0)<(0);
 $241 = $240 << 31 >> 31;
 $242 = (_sqlite3Fts3PutVarint($238,$239,$241)|0);
 $243 = HEAP32[$nData>>2]|0;
 $244 = (($243) + ($242))|0;
 HEAP32[$nData>>2] = $244;
 $245 = HEAP32[$nData>>2]|0;
 $246 = HEAP32[$pWriter>>2]|0;
 $247 = ((($246)) + 48|0);
 $248 = HEAP32[$247>>2]|0;
 $249 = (($248) + ($245)|0);
 $250 = HEAP32[$6>>2]|0;
 $251 = HEAP32[$7>>2]|0;
 _memcpy(($249|0),($250|0),($251|0))|0;
 $252 = HEAP32[$nData>>2]|0;
 $253 = HEAP32[$7>>2]|0;
 $254 = (($252) + ($253))|0;
 $255 = HEAP32[$pWriter>>2]|0;
 $256 = ((($255)) + 44|0);
 HEAP32[$256>>2] = $254;
 $257 = HEAP32[$3>>2]|0;
 $258 = ($257|0)!=(0);
 if ($258) {
  $259 = HEAP32[$5>>2]|0;
  $260 = HEAP32[$pWriter>>2]|0;
  $261 = ((($260)) + 32|0);
  $262 = HEAP32[$261>>2]|0;
  $263 = ($259|0)>($262|0);
  do {
   if ($263) {
    $264 = HEAP32[$pWriter>>2]|0;
    $265 = ((($264)) + 36|0);
    $266 = HEAP32[$265>>2]|0;
    $267 = HEAP32[$5>>2]|0;
    $268 = $267<<1;
    $269 = (_sqlite3_realloc($266,$268)|0);
    HEAP32[$zNew>>2] = $269;
    $270 = HEAP32[$zNew>>2]|0;
    $271 = ($270|0)!=(0|0);
    if ($271) {
     $272 = HEAP32[$5>>2]|0;
     $273 = $272<<1;
     $274 = HEAP32[$pWriter>>2]|0;
     $275 = ((($274)) + 32|0);
     HEAP32[$275>>2] = $273;
     $276 = HEAP32[$zNew>>2]|0;
     $277 = HEAP32[$pWriter>>2]|0;
     $278 = ((($277)) + 36|0);
     HEAP32[$278>>2] = $276;
     $279 = HEAP32[$zNew>>2]|0;
     $280 = HEAP32[$pWriter>>2]|0;
     $281 = ((($280)) + 24|0);
     HEAP32[$281>>2] = $279;
     break;
    }
    HEAP32[$0>>2] = 7;
    $293 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($293|0);
   }
  } while(0);
  $282 = HEAP32[$pWriter>>2]|0;
  $283 = ((($282)) + 24|0);
  $284 = HEAP32[$283>>2]|0;
  $285 = HEAP32[$4>>2]|0;
  $286 = HEAP32[$5>>2]|0;
  _memcpy(($284|0),($285|0),($286|0))|0;
 } else {
  $287 = HEAP32[$4>>2]|0;
  $288 = HEAP32[$pWriter>>2]|0;
  $289 = ((($288)) + 24|0);
  HEAP32[$289>>2] = $287;
 }
 $290 = HEAP32[$5>>2]|0;
 $291 = HEAP32[$pWriter>>2]|0;
 $292 = ((($291)) + 28|0);
 HEAP32[$292>>2] = $290;
 HEAP32[$0>>2] = 0;
 $293 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($293|0);
}
function _fts3DeleteSegdir($p,$iLangid,$iIndex,$iLevel,$apSegment,$nReader) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $apSegment = $apSegment|0;
 $nReader = $nReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pDelete = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $pDelete = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $iIndex;
 HEAP32[$4>>2] = $iLevel;
 HEAP32[$5>>2] = $apSegment;
 HEAP32[$6>>2] = $nReader;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDelete>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$6>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$5>>2]|0;
  $15 = (($14) + ($13<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_fts3DeleteSegment($12,$16)|0);
  HEAP32[$rc>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $22;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $23 = HEAP32[$4>>2]|0;
 $24 = ($23|0)==(-2);
 $25 = HEAP32[$1>>2]|0;
 if ($24) {
  $26 = (_fts3SqlStmt($25,26,$pDelete,0)|0);
  HEAP32[$rc>>2] = $26;
  $27 = HEAP32[$rc>>2]|0;
  $28 = ($27|0)==(0);
  if ($28) {
   $29 = HEAP32[$pDelete>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   $33 = (_getAbsoluteLevel($30,$31,$32,0)|0);
   $34 = tempRet0;
   (_sqlite3_bind_int64($29,1,$33,$34)|0);
   $35 = HEAP32[$pDelete>>2]|0;
   $36 = HEAP32[$1>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = HEAP32[$3>>2]|0;
   $39 = (_getAbsoluteLevel($36,$37,$38,1023)|0);
   $40 = tempRet0;
   (_sqlite3_bind_int64($35,2,$39,$40)|0);
  }
 } else {
  $41 = (_fts3SqlStmt($25,16,$pDelete,0)|0);
  HEAP32[$rc>>2] = $41;
  $42 = HEAP32[$rc>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$pDelete>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$2>>2]|0;
   $47 = HEAP32[$3>>2]|0;
   $48 = HEAP32[$4>>2]|0;
   $49 = (_getAbsoluteLevel($45,$46,$47,$48)|0);
   $50 = tempRet0;
   (_sqlite3_bind_int64($44,1,$49,$50)|0);
  }
 }
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$pDelete>>2]|0;
  (_sqlite3_step($53)|0);
  $54 = HEAP32[$pDelete>>2]|0;
  $55 = (_sqlite3_reset($54)|0);
  HEAP32[$rc>>2] = $55;
 }
 $56 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $56;
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _fts3SegWriterFlush($p,$pWriter,$0,$1,$iIdx) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $iLast = 0, $iLastLeaf = 0, $nRoot = 0, $rc = 0, $zRoot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 16|0;
 $5 = sp + 36|0;
 $rc = sp + 32|0;
 $iLast = sp + 8|0;
 $iLastLeaf = sp;
 $zRoot = sp + 28|0;
 $nRoot = sp + 24|0;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pWriter;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $iIdx;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $114 = HEAP32[$2>>2]|0;
  $115 = $4;
  $116 = $115;
  $117 = HEAP32[$116>>2]|0;
  $118 = (($115) + 4)|0;
  $119 = $118;
  $120 = HEAP32[$119>>2]|0;
  $121 = HEAP32[$5>>2]|0;
  $122 = HEAP32[$3>>2]|0;
  $123 = ((($122)) + 56|0);
  $124 = $123;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = (($124) + 4)|0;
  $128 = $127;
  $129 = HEAP32[$128>>2]|0;
  $130 = HEAP32[$3>>2]|0;
  $131 = ((($130)) + 48|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = HEAP32[$3>>2]|0;
  $134 = ((($133)) + 44|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (_fts3WriteSegdir($114,$117,$120,$121,0,0,0,0,0,0,$126,$129,$132,$135)|0);
  HEAP32[$rc>>2] = $136;
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 52|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = (($139) + 1)|0;
  HEAP32[$138>>2] = $140;
  $141 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 $13 = $iLast;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 HEAP32[$zRoot>>2] = 0;
 HEAP32[$nRoot>>2] = 0;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = $iLastLeaf;
 $26 = $25;
 HEAP32[$26>>2] = $21;
 $27 = (($25) + 4)|0;
 $28 = $27;
 HEAP32[$28>>2] = $24;
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 $31 = ((($30)) + 16|0);
 $32 = $31;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (_i64Add(($34|0),($37|0),1,0)|0);
 $39 = tempRet0;
 $40 = $31;
 $41 = $40;
 HEAP32[$41>>2] = $38;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $39;
 $44 = HEAP32[$3>>2]|0;
 $45 = ((($44)) + 48|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$3>>2]|0;
 $48 = ((($47)) + 44|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (_fts3WriteSegment($29,$34,$37,$46,$49)|0);
 HEAP32[$rc>>2] = $50;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$2>>2]|0;
  $54 = HEAP32[$3>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$3>>2]|0;
  $57 = ((($56)) + 8|0);
  $58 = $57;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$3>>2]|0;
  $65 = ((($64)) + 16|0);
  $66 = $65;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($66) + 4)|0;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (_fts3NodeWrite($53,$55,1,$60,$63,$68,$71,$iLast,$zRoot,$nRoot)|0);
  HEAP32[$rc>>2] = $72;
 }
 $73 = HEAP32[$rc>>2]|0;
 $74 = ($73|0)==(0);
 if (!($74)) {
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 52|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = (($139) + 1)|0;
  HEAP32[$138>>2] = $140;
  $141 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 $75 = HEAP32[$2>>2]|0;
 $76 = $4;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = (($76) + 4)|0;
 $80 = $79;
 $81 = HEAP32[$80>>2]|0;
 $82 = HEAP32[$5>>2]|0;
 $83 = HEAP32[$3>>2]|0;
 $84 = ((($83)) + 8|0);
 $85 = $84;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = $iLastLeaf;
 $92 = $91;
 $93 = HEAP32[$92>>2]|0;
 $94 = (($91) + 4)|0;
 $95 = $94;
 $96 = HEAP32[$95>>2]|0;
 $97 = $iLast;
 $98 = $97;
 $99 = HEAP32[$98>>2]|0;
 $100 = (($97) + 4)|0;
 $101 = $100;
 $102 = HEAP32[$101>>2]|0;
 $103 = HEAP32[$3>>2]|0;
 $104 = ((($103)) + 56|0);
 $105 = $104;
 $106 = $105;
 $107 = HEAP32[$106>>2]|0;
 $108 = (($105) + 4)|0;
 $109 = $108;
 $110 = HEAP32[$109>>2]|0;
 $111 = HEAP32[$zRoot>>2]|0;
 $112 = HEAP32[$nRoot>>2]|0;
 $113 = (_fts3WriteSegdir($75,$78,$81,$82,$87,$90,$93,$96,$99,$102,$107,$110,$111,$112)|0);
 HEAP32[$rc>>2] = $113;
 $137 = HEAP32[$2>>2]|0;
 $138 = ((($137)) + 52|0);
 $139 = HEAP32[$138>>2]|0;
 $140 = (($139) + 1)|0;
 HEAP32[$138>>2] = $140;
 $141 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($141|0);
}
function _fts3PromoteSegments($p,$0,$1,$2,$3) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bOk = 0, $dummy = 0, $iIdx = 0, $iLast = 0, $nLimit = 0;
 var $nSize = 0, $pRange = 0, $pUpdate1 = 0, $pUpdate2 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $4 = sp + 72|0;
 $5 = sp + 40|0;
 $6 = sp + 32|0;
 $rc = sp + 68|0;
 $pRange = sp + 64|0;
 $bOk = sp + 60|0;
 $iLast = sp + 24|0;
 $nLimit = sp + 16|0;
 $nSize = sp + 8|0;
 $dummy = sp;
 $iIdx = sp + 56|0;
 $pUpdate1 = sp + 52|0;
 $pUpdate2 = sp + 48|0;
 HEAP32[$4>>2] = $p;
 $7 = $5;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 HEAP32[$12>>2] = $2;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $15 = HEAP32[$4>>2]|0;
 $16 = (_fts3SqlStmt($15,37,$pRange,0)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if (!($18)) {
  $158 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 HEAP32[$bOk>>2] = 0;
 $19 = $5;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (___divdi3(($21|0),($24|0),1024,0)|0);
 $26 = tempRet0;
 $27 = (_i64Add(($25|0),($26|0),1,0)|0);
 $28 = tempRet0;
 $29 = (___muldi3(($27|0),($28|0),1024,0)|0);
 $30 = tempRet0;
 $31 = (_i64Subtract(($29|0),($30|0),1,0)|0);
 $32 = tempRet0;
 $33 = $iLast;
 $34 = $33;
 HEAP32[$34>>2] = $31;
 $35 = (($33) + 4)|0;
 $36 = $35;
 HEAP32[$36>>2] = $32;
 $37 = $6;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) + 4)|0;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (___muldi3(($39|0),($42|0),3,0)|0);
 $44 = tempRet0;
 $45 = (___divdi3(($43|0),($44|0),2,0)|0);
 $46 = tempRet0;
 $47 = $nLimit;
 $48 = $47;
 HEAP32[$48>>2] = $45;
 $49 = (($47) + 4)|0;
 $50 = $49;
 HEAP32[$50>>2] = $46;
 $51 = HEAP32[$pRange>>2]|0;
 $52 = $5;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = (_i64Add(($54|0),($57|0),1,0)|0);
 $59 = tempRet0;
 (_sqlite3_bind_int64($51,1,$58,$59)|0);
 $60 = HEAP32[$pRange>>2]|0;
 $61 = $iLast;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 (_sqlite3_bind_int64($60,2,$63,$66)|0);
 while(1) {
  $67 = HEAP32[$pRange>>2]|0;
  $68 = (_sqlite3_step($67)|0);
  $69 = (100)==($68|0);
  if (!($69)) {
   break;
  }
  $70 = $nSize;
  $71 = $70;
  HEAP32[$71>>2] = 0;
  $72 = (($70) + 4)|0;
  $73 = $72;
  HEAP32[$73>>2] = 0;
  $74 = HEAP32[$pRange>>2]|0;
  _fts3ReadEndBlockField($74,2,$dummy,$nSize);
  $75 = $nSize;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = ($80|0)<(0);
  $82 = ($77>>>0)<=(0);
  $83 = ($80|0)==(0);
  $84 = $83 & $82;
  $85 = $81 | $84;
  if ($85) {
   label = 6;
   break;
  }
  $86 = $nSize;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = (($86) + 4)|0;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = $nLimit;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($91|0)>($97|0);
  $99 = ($88>>>0)>($94>>>0);
  $100 = ($91|0)==($97|0);
  $101 = $100 & $99;
  $102 = $98 | $101;
  if ($102) {
   label = 6;
   break;
  }
  HEAP32[$bOk>>2] = 1;
 }
 if ((label|0) == 6) {
  HEAP32[$bOk>>2] = 0;
 }
 $103 = HEAP32[$pRange>>2]|0;
 $104 = (_sqlite3_reset($103)|0);
 HEAP32[$rc>>2] = $104;
 $105 = HEAP32[$bOk>>2]|0;
 $106 = ($105|0)!=(0);
 if (!($106)) {
  $158 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 HEAP32[$iIdx>>2] = 0;
 HEAP32[$pUpdate1>>2] = 0;
 HEAP32[$pUpdate2>>2] = 0;
 $107 = HEAP32[$rc>>2]|0;
 $108 = ($107|0)==(0);
 if ($108) {
  $109 = HEAP32[$4>>2]|0;
  $110 = (_fts3SqlStmt($109,38,$pUpdate1,0)|0);
  HEAP32[$rc>>2] = $110;
 }
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)==(0);
 if ($112) {
  $113 = HEAP32[$4>>2]|0;
  $114 = (_fts3SqlStmt($113,39,$pUpdate2,0)|0);
  HEAP32[$rc>>2] = $114;
 }
 $115 = HEAP32[$rc>>2]|0;
 $116 = ($115|0)==(0);
 L21: do {
  if ($116) {
   $117 = HEAP32[$pRange>>2]|0;
   $118 = $5;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   (_sqlite3_bind_int64($117,1,$120,$123)|0);
   while(1) {
    $124 = HEAP32[$pRange>>2]|0;
    $125 = (_sqlite3_step($124)|0);
    $126 = (100)==($125|0);
    if (!($126)) {
     break L21;
    }
    $127 = HEAP32[$pUpdate1>>2]|0;
    $128 = HEAP32[$iIdx>>2]|0;
    $129 = (($128) + 1)|0;
    HEAP32[$iIdx>>2] = $129;
    (_sqlite3_bind_int($127,1,$128)|0);
    $130 = HEAP32[$pUpdate1>>2]|0;
    $131 = HEAP32[$pRange>>2]|0;
    $132 = (_sqlite3_column_int($131,0)|0);
    (_sqlite3_bind_int($130,2,$132)|0);
    $133 = HEAP32[$pUpdate1>>2]|0;
    $134 = HEAP32[$pRange>>2]|0;
    $135 = (_sqlite3_column_int($134,1)|0);
    (_sqlite3_bind_int($133,3,$135)|0);
    $136 = HEAP32[$pUpdate1>>2]|0;
    (_sqlite3_step($136)|0);
    $137 = HEAP32[$pUpdate1>>2]|0;
    $138 = (_sqlite3_reset($137)|0);
    HEAP32[$rc>>2] = $138;
    $139 = HEAP32[$rc>>2]|0;
    $140 = ($139|0)!=(0);
    if ($140) {
     break;
    }
   }
   $141 = HEAP32[$pRange>>2]|0;
   (_sqlite3_reset($141)|0);
  }
 } while(0);
 $142 = HEAP32[$rc>>2]|0;
 $143 = ($142|0)==(0);
 if ($143) {
  $144 = HEAP32[$pRange>>2]|0;
  $145 = (_sqlite3_reset($144)|0);
  HEAP32[$rc>>2] = $145;
 }
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)==(0);
 if (!($147)) {
  $158 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $148 = HEAP32[$pUpdate2>>2]|0;
 $149 = $5;
 $150 = $149;
 $151 = HEAP32[$150>>2]|0;
 $152 = (($149) + 4)|0;
 $153 = $152;
 $154 = HEAP32[$153>>2]|0;
 (_sqlite3_bind_int64($148,1,$151,$154)|0);
 $155 = HEAP32[$pUpdate2>>2]|0;
 (_sqlite3_step($155)|0);
 $156 = HEAP32[$pUpdate2>>2]|0;
 $157 = (_sqlite3_reset($156)|0);
 HEAP32[$rc>>2] = $157;
 $158 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($158|0);
}
function _fts3SegWriterFree($pWriter) {
 $pWriter = $pWriter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWriter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3_free($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 36|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3_free($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 _fts3NodeFree($10);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3_free($11);
 STACKTOP = sp;return;
}
function _sqlite3Fts3SegReaderFinish($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($8<<2)|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3Fts3SegReaderFree($12);
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 _sqlite3_free($16);
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = HEAP32[$18>>2]|0;
 _sqlite3_free($19);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 HEAP32[$22>>2] = 0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 16|0);
 HEAP32[$24>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3SegReaderFree($pReader) {
 $pReader = $pReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pReader;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $21 = HEAP32[$0>>2]|0;
  _sqlite3_free($21);
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 64|0);
  $9 = HEAP32[$8>>2]|0;
  _sqlite3_free($9);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 5|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 40|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_free($17);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 52|0);
 $20 = HEAP32[$19>>2]|0;
 (_sqlite3_blob_close($20)|0);
 $21 = HEAP32[$0>>2]|0;
 _sqlite3_free($21);
 STACKTOP = sp;return;
}
function _fts3NodeFree($pTree) {
 $pTree = $pTree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pRight = sp;
 HEAP32[$0>>2] = $pTree;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 _fts3NodeFree($7);
 while(1) {
  $8 = HEAP32[$p>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pRight>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 36|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$p>>2]|0;
  $17 = ((($16)) + 40|0);
  $18 = ($15|0)!=($17|0);
  if ($18) {
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 36|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3_free($21);
  }
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 28|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3_free($24);
  $25 = HEAP32[$p>>2]|0;
  _sqlite3_free($25);
  $26 = HEAP32[$pRight>>2]|0;
  HEAP32[$p>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _fts3ReadEndBlockField($pStmt,$iCol,$piEndBlock,$pnByte) {
 $pStmt = $pStmt|0;
 $iCol = $iCol|0;
 $piEndBlock = $piEndBlock|0;
 $pnByte = $pnByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $iMul = 0, $iVal = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $zText = sp + 16|0;
 $i = sp + 12|0;
 $iMul = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pStmt;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$2>>2] = $piEndBlock;
 HEAP32[$3>>2] = $pnByte;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3_column_text($4,$5)|0);
 HEAP32[$zText>>2] = $6;
 $7 = HEAP32[$zText>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 HEAP32[$iMul>>2] = 1;
 $9 = $iVal;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$zText>>2]|0;
  $15 = (($14) + ($13)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)>=(48);
  if ($18) {
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$zText>>2]|0;
   $21 = (($20) + ($19)|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)<=(57);
   $120 = $24;
  } else {
   $120 = 0;
  }
  $25 = $iVal;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  if (!($120)) {
   break;
  }
  $31 = (___muldi3(($27|0),($30|0),10,0)|0);
  $32 = tempRet0;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$zText>>2]|0;
  $35 = (($34) + ($33)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = (($37) - 48)|0;
  $39 = ($38|0)<(0);
  $40 = $39 << 31 >> 31;
  $41 = (_i64Add(($31|0),($32|0),($38|0),($40|0))|0);
  $42 = tempRet0;
  $43 = $iVal;
  $44 = $43;
  HEAP32[$44>>2] = $41;
  $45 = (($43) + 4)|0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$i>>2] = $48;
 }
 $49 = HEAP32[$2>>2]|0;
 $50 = $49;
 $51 = $50;
 HEAP32[$51>>2] = $27;
 $52 = (($50) + 4)|0;
 $53 = $52;
 HEAP32[$53>>2] = $30;
 while(1) {
  $54 = HEAP32[$i>>2]|0;
  $55 = HEAP32[$zText>>2]|0;
  $56 = (($55) + ($54)|0);
  $57 = HEAP8[$56>>0]|0;
  $58 = $57&255;
  $59 = ($58|0)==(32);
  if (!($59)) {
   break;
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 $62 = $iVal;
 $63 = $62;
 HEAP32[$63>>2] = 0;
 $64 = (($62) + 4)|0;
 $65 = $64;
 HEAP32[$65>>2] = 0;
 $66 = HEAP32[$i>>2]|0;
 $67 = HEAP32[$zText>>2]|0;
 $68 = (($67) + ($66)|0);
 $69 = HEAP8[$68>>0]|0;
 $70 = $69&255;
 $71 = ($70|0)==(45);
 if ($71) {
  $72 = HEAP32[$i>>2]|0;
  $73 = (($72) + 1)|0;
  HEAP32[$i>>2] = $73;
  HEAP32[$iMul>>2] = -1;
 }
 while(1) {
  $74 = HEAP32[$i>>2]|0;
  $75 = HEAP32[$zText>>2]|0;
  $76 = (($75) + ($74)|0);
  $77 = HEAP8[$76>>0]|0;
  $78 = $77&255;
  $79 = ($78|0)>=(48);
  if ($79) {
   $80 = HEAP32[$i>>2]|0;
   $81 = HEAP32[$zText>>2]|0;
   $82 = (($81) + ($80)|0);
   $83 = HEAP8[$82>>0]|0;
   $84 = $83&255;
   $85 = ($84|0)<=(57);
   $121 = $85;
  } else {
   $121 = 0;
  }
  $86 = $iVal;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = (($86) + 4)|0;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  if (!($121)) {
   break;
  }
  $92 = (___muldi3(($88|0),($91|0),10,0)|0);
  $93 = tempRet0;
  $94 = HEAP32[$i>>2]|0;
  $95 = HEAP32[$zText>>2]|0;
  $96 = (($95) + ($94)|0);
  $97 = HEAP8[$96>>0]|0;
  $98 = $97&255;
  $99 = (($98) - 48)|0;
  $100 = ($99|0)<(0);
  $101 = $100 << 31 >> 31;
  $102 = (_i64Add(($92|0),($93|0),($99|0),($101|0))|0);
  $103 = tempRet0;
  $104 = $iVal;
  $105 = $104;
  HEAP32[$105>>2] = $102;
  $106 = (($104) + 4)|0;
  $107 = $106;
  HEAP32[$107>>2] = $103;
  $108 = HEAP32[$i>>2]|0;
  $109 = (($108) + 1)|0;
  HEAP32[$i>>2] = $109;
 }
 $110 = HEAP32[$iMul>>2]|0;
 $111 = ($110|0)<(0);
 $112 = $111 << 31 >> 31;
 $113 = (___muldi3(($88|0),($91|0),($110|0),($112|0))|0);
 $114 = tempRet0;
 $115 = HEAP32[$3>>2]|0;
 $116 = $115;
 $117 = $116;
 HEAP32[$117>>2] = $113;
 $118 = (($116) + 4)|0;
 $119 = $118;
 HEAP32[$119>>2] = $114;
 STACKTOP = sp;return;
}
function _fts3WriteSegment($p,$0,$1,$z,$n) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $z = $z|0;
 $n = $n|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $z;
 HEAP32[$5>>2] = $n;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_fts3SqlStmt($10,9,$pStmt,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $14 = HEAP32[$pStmt>>2]|0;
 $15 = $3;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 (_sqlite3_bind_int64($14,1,$17,$20)|0);
 $21 = HEAP32[$pStmt>>2]|0;
 $22 = HEAP32[$4>>2]|0;
 $23 = HEAP32[$5>>2]|0;
 (_sqlite3_bind_blob($21,2,$22,$23,0)|0);
 $24 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_step($24)|0);
 $25 = HEAP32[$pStmt>>2]|0;
 $26 = (_sqlite3_reset($25)|0);
 HEAP32[$rc>>2] = $26;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _fts3NodeWrite($p,$pTree,$iHeight,$0,$1,$2,$3,$piLast,$paRoot,$pnRoot) {
 $p = $p|0;
 $pTree = $pTree|0;
 $iHeight = $iHeight|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 $piLast = $piLast|0;
 $paRoot = $paRoot|0;
 $pnRoot = $pnRoot|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iNextFree = 0, $iNextLeaf = 0, $nStart = 0, $nStart1 = 0, $nWrite = 0, $pIter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 24|0;
 $8 = sp + 16|0;
 $9 = sp + 60|0;
 $10 = sp + 56|0;
 $11 = sp + 52|0;
 $rc = sp + 48|0;
 $nStart = sp + 44|0;
 $pIter = sp + 40|0;
 $iNextFree = sp + 8|0;
 $iNextLeaf = sp;
 $nStart1 = sp + 36|0;
 $nWrite = sp + 32|0;
 HEAP32[$4>>2] = $p;
 HEAP32[$5>>2] = $pTree;
 HEAP32[$6>>2] = $iHeight;
 $12 = $7;
 $13 = $12;
 HEAP32[$13>>2] = $0;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $1;
 $16 = $8;
 $17 = $16;
 HEAP32[$17>>2] = $2;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $3;
 HEAP32[$9>>2] = $piLast;
 HEAP32[$10>>2] = $paRoot;
 HEAP32[$11>>2] = $pnRoot;
 HEAP32[$rc>>2] = 0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  $23 = HEAP32[$5>>2]|0;
  $24 = HEAP32[$6>>2]|0;
  $25 = $7;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (_fts3TreeFinishNode($23,$24,$27,$30)|0);
  HEAP32[$nStart>>2] = $31;
  $32 = $8;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (_i64Subtract(($34|0),($37|0),1,0)|0);
  $39 = tempRet0;
  $40 = HEAP32[$9>>2]|0;
  $41 = $40;
  $42 = $41;
  HEAP32[$42>>2] = $38;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $39;
  $45 = HEAP32[$5>>2]|0;
  $46 = ((($45)) + 32|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$nStart>>2]|0;
  $49 = (($47) - ($48))|0;
  $50 = HEAP32[$11>>2]|0;
  HEAP32[$50>>2] = $49;
  $51 = HEAP32[$nStart>>2]|0;
  $52 = HEAP32[$5>>2]|0;
  $53 = ((($52)) + 36|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51)|0);
  $56 = HEAP32[$10>>2]|0;
  HEAP32[$56>>2] = $55;
  $169 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $57 = $8;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = $iNextFree;
 $64 = $63;
 HEAP32[$64>>2] = $59;
 $65 = (($63) + 4)|0;
 $66 = $65;
 HEAP32[$66>>2] = $62;
 $67 = $7;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($67) + 4)|0;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = $iNextLeaf;
 $74 = $73;
 HEAP32[$74>>2] = $69;
 $75 = (($73) + 4)|0;
 $76 = $75;
 HEAP32[$76>>2] = $72;
 $77 = HEAP32[$5>>2]|0;
 $78 = ((($77)) + 8|0);
 $79 = HEAP32[$78>>2]|0;
 HEAP32[$pIter>>2] = $79;
 while(1) {
  $80 = HEAP32[$pIter>>2]|0;
  $81 = ($80|0)!=(0|0);
  $82 = HEAP32[$rc>>2]|0;
  $83 = ($82|0)==(0);
  $84 = $81 ? $83 : 0;
  if (!($84)) {
   break;
  }
  $85 = HEAP32[$pIter>>2]|0;
  $86 = HEAP32[$6>>2]|0;
  $87 = $iNextLeaf;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  $93 = (_fts3TreeFinishNode($85,$86,$89,$92)|0);
  HEAP32[$nStart1>>2] = $93;
  $94 = HEAP32[$pIter>>2]|0;
  $95 = ((($94)) + 32|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = HEAP32[$nStart1>>2]|0;
  $98 = (($96) - ($97))|0;
  HEAP32[$nWrite>>2] = $98;
  $99 = HEAP32[$4>>2]|0;
  $100 = $iNextFree;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (($100) + 4)|0;
  $104 = $103;
  $105 = HEAP32[$104>>2]|0;
  $106 = HEAP32[$nStart1>>2]|0;
  $107 = HEAP32[$pIter>>2]|0;
  $108 = ((($107)) + 36|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = (($109) + ($106)|0);
  $111 = HEAP32[$nWrite>>2]|0;
  $112 = (_fts3WriteSegment($99,$102,$105,$110,$111)|0);
  HEAP32[$rc>>2] = $112;
  $113 = $iNextFree;
  $114 = $113;
  $115 = HEAP32[$114>>2]|0;
  $116 = (($113) + 4)|0;
  $117 = $116;
  $118 = HEAP32[$117>>2]|0;
  $119 = (_i64Add(($115|0),($118|0),1,0)|0);
  $120 = tempRet0;
  $121 = $iNextFree;
  $122 = $121;
  HEAP32[$122>>2] = $119;
  $123 = (($121) + 4)|0;
  $124 = $123;
  HEAP32[$124>>2] = $120;
  $125 = HEAP32[$pIter>>2]|0;
  $126 = ((($125)) + 12|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = (($127) + 1)|0;
  $129 = ($128|0)<(0);
  $130 = $129 << 31 >> 31;
  $131 = $iNextLeaf;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = (($131) + 4)|0;
  $135 = $134;
  $136 = HEAP32[$135>>2]|0;
  $137 = (_i64Add(($133|0),($136|0),($128|0),($130|0))|0);
  $138 = tempRet0;
  $139 = $iNextLeaf;
  $140 = $139;
  HEAP32[$140>>2] = $137;
  $141 = (($139) + 4)|0;
  $142 = $141;
  HEAP32[$142>>2] = $138;
  $143 = HEAP32[$pIter>>2]|0;
  $144 = ((($143)) + 4|0);
  $145 = HEAP32[$144>>2]|0;
  HEAP32[$pIter>>2] = $145;
 }
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)==(0);
 if (!($147)) {
  $169 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $148 = HEAP32[$4>>2]|0;
 $149 = HEAP32[$5>>2]|0;
 $150 = HEAP32[$149>>2]|0;
 $151 = HEAP32[$6>>2]|0;
 $152 = (($151) + 1)|0;
 $153 = $8;
 $154 = $153;
 $155 = HEAP32[$154>>2]|0;
 $156 = (($153) + 4)|0;
 $157 = $156;
 $158 = HEAP32[$157>>2]|0;
 $159 = $iNextFree;
 $160 = $159;
 $161 = HEAP32[$160>>2]|0;
 $162 = (($159) + 4)|0;
 $163 = $162;
 $164 = HEAP32[$163>>2]|0;
 $165 = HEAP32[$9>>2]|0;
 $166 = HEAP32[$10>>2]|0;
 $167 = HEAP32[$11>>2]|0;
 $168 = (_fts3NodeWrite($148,$150,$152,$155,$158,$161,$164,$165,$166,$167)|0);
 HEAP32[$rc>>2] = $168;
 $169 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _fts3WriteSegdir($p,$0,$1,$iIdx,$2,$3,$4,$5,$6,$7,$8,$9,$zRoot,$nRoot) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $2 = $2|0;
 $3 = $3|0;
 $4 = $4|0;
 $5 = $5|0;
 $6 = $6|0;
 $7 = $7|0;
 $8 = $8|0;
 $9 = $9|0;
 $zRoot = $zRoot|0;
 $nRoot = $nRoot|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $12 = 0, $13 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pStmt = 0, $rc = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp + 40|0;
 $10 = sp + 84|0;
 $11 = sp + 80|0;
 $12 = sp + 32|0;
 $13 = sp + 76|0;
 $14 = sp + 24|0;
 $15 = sp + 16|0;
 $16 = sp + 8|0;
 $17 = sp;
 $18 = sp + 72|0;
 $19 = sp + 68|0;
 $pStmt = sp + 64|0;
 $rc = sp + 60|0;
 $zEnd = sp + 56|0;
 HEAP32[$11>>2] = $p;
 $20 = $12;
 $21 = $20;
 HEAP32[$21>>2] = $0;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = $1;
 HEAP32[$13>>2] = $iIdx;
 $24 = $14;
 $25 = $24;
 HEAP32[$25>>2] = $2;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $3;
 $28 = $15;
 $29 = $28;
 HEAP32[$29>>2] = $4;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $5;
 $32 = $16;
 $33 = $32;
 HEAP32[$33>>2] = $6;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $7;
 $36 = $17;
 $37 = $36;
 HEAP32[$37>>2] = $8;
 $38 = (($36) + 4)|0;
 $39 = $38;
 HEAP32[$39>>2] = $9;
 HEAP32[$18>>2] = $zRoot;
 HEAP32[$19>>2] = $nRoot;
 $40 = HEAP32[$11>>2]|0;
 $41 = (_fts3SqlStmt($40,11,$pStmt,0)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = HEAP32[$pStmt>>2]|0;
  $45 = $12;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($45) + 4)|0;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  (_sqlite3_bind_int64($44,1,$47,$50)|0);
  $51 = HEAP32[$pStmt>>2]|0;
  $52 = HEAP32[$13>>2]|0;
  (_sqlite3_bind_int($51,2,$52)|0);
  $53 = HEAP32[$pStmt>>2]|0;
  $54 = $14;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($54) + 4)|0;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  (_sqlite3_bind_int64($53,3,$56,$59)|0);
  $60 = HEAP32[$pStmt>>2]|0;
  $61 = $15;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (($61) + 4)|0;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  (_sqlite3_bind_int64($60,4,$63,$66)|0);
  $67 = $17;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($69|0)==(0);
  $74 = ($72|0)==(0);
  $75 = $73 & $74;
  do {
   if ($75) {
    $76 = HEAP32[$pStmt>>2]|0;
    $77 = $16;
    $78 = $77;
    $79 = HEAP32[$78>>2]|0;
    $80 = (($77) + 4)|0;
    $81 = $80;
    $82 = HEAP32[$81>>2]|0;
    (_sqlite3_bind_int64($76,5,$79,$82)|0);
   } else {
    $83 = $16;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $86 = (($83) + 4)|0;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = $17;
    $90 = $89;
    $91 = HEAP32[$90>>2]|0;
    $92 = (($89) + 4)|0;
    $93 = $92;
    $94 = HEAP32[$93>>2]|0;
    $95 = $vararg_buffer;
    $96 = $95;
    HEAP32[$96>>2] = $85;
    $97 = (($95) + 4)|0;
    $98 = $97;
    HEAP32[$98>>2] = $88;
    $vararg_ptr1 = ((($vararg_buffer)) + 8|0);
    $99 = $vararg_ptr1;
    $100 = $99;
    HEAP32[$100>>2] = $91;
    $101 = (($99) + 4)|0;
    $102 = $101;
    HEAP32[$102>>2] = $94;
    $103 = (_sqlite3_mprintf(23153,$vararg_buffer)|0);
    HEAP32[$zEnd>>2] = $103;
    $104 = HEAP32[$zEnd>>2]|0;
    $105 = ($104|0)!=(0|0);
    if ($105) {
     $106 = HEAP32[$pStmt>>2]|0;
     $107 = HEAP32[$zEnd>>2]|0;
     (_sqlite3_bind_text($106,5,$107,-1,148)|0);
     break;
    }
    HEAP32[$10>>2] = 7;
    $115 = HEAP32[$10>>2]|0;
    STACKTOP = sp;return ($115|0);
   }
  } while(0);
  $108 = HEAP32[$pStmt>>2]|0;
  $109 = HEAP32[$18>>2]|0;
  $110 = HEAP32[$19>>2]|0;
  (_sqlite3_bind_blob($108,6,$109,$110,0)|0);
  $111 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($111)|0);
  $112 = HEAP32[$pStmt>>2]|0;
  $113 = (_sqlite3_reset($112)|0);
  HEAP32[$rc>>2] = $113;
 }
 $114 = HEAP32[$rc>>2]|0;
 HEAP32[$10>>2] = $114;
 $115 = HEAP32[$10>>2]|0;
 STACKTOP = sp;return ($115|0);
}
function _fts3TreeFinishNode($pTree,$iHeight,$0,$1) {
 $pTree = $pTree|0;
 $iHeight = $iHeight|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nStart = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $nStart = sp + 8|0;
 HEAP32[$2>>2] = $pTree;
 HEAP32[$3>>2] = $iHeight;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3Fts3VarintLen($11,$14)|0);
 $16 = (10 - ($15))|0;
 HEAP32[$nStart>>2] = $16;
 $17 = HEAP32[$3>>2]|0;
 $18 = $17&255;
 $19 = HEAP32[$nStart>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 36|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + ($19)|0);
 HEAP8[$23>>0] = $18;
 $24 = HEAP32[$nStart>>2]|0;
 $25 = (($24) + 1)|0;
 $26 = HEAP32[$2>>2]|0;
 $27 = ((($26)) + 36|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25)|0);
 $30 = $4;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 (_sqlite3Fts3PutVarint($29,$32,$35)|0);
 $36 = HEAP32[$nStart>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3Fts3VarintLen($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $i = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = (($7) + 1)|0;
  HEAP32[$i>>2] = $8;
  $9 = $2;
  $10 = $9;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9) + 4)|0;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = (_bitshift64Lshr(($11|0),($14|0),7)|0);
  $16 = tempRet0;
  $17 = $2;
  $18 = $17;
  HEAP32[$18>>2] = $15;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $21 = $2;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)!=(0);
  $28 = ($26|0)!=(0);
  $29 = $27 | $28;
  if (!($29)) {
   break;
  }
 }
 $30 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3Fts3PutVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $q = 0, $vu = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 8|0;
 $q = sp + 16|0;
 $vu = sp;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[$q>>2] = $8;
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = $vu;
 $16 = $15;
 HEAP32[$16>>2] = $11;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = $14;
 while(1) {
  $19 = $vu;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = $21 & 127;
  $26 = $25 | 128;
  $27 = $26&255;
  $28 = HEAP32[$q>>2]|0;
  $29 = ((($28)) + 1|0);
  HEAP32[$q>>2] = $29;
  HEAP8[$28>>0] = $27;
  $30 = $vu;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_bitshift64Lshr(($32|0),($35|0),7)|0);
  $37 = tempRet0;
  $38 = $vu;
  $39 = $38;
  HEAP32[$39>>2] = $36;
  $40 = (($38) + 4)|0;
  $41 = $40;
  HEAP32[$41>>2] = $37;
  $42 = $vu;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = ($44|0)!=(0);
  $49 = ($47|0)!=(0);
  $50 = $48 | $49;
  if (!($50)) {
   break;
  }
 }
 $51 = HEAP32[$q>>2]|0;
 $52 = ((($51)) + -1|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = $54 & 127;
 $56 = $55&255;
 HEAP8[$52>>0] = $56;
 $57 = HEAP32[$q>>2]|0;
 $58 = HEAP32[$2>>2]|0;
 $59 = $57;
 $60 = $58;
 $61 = (($59) - ($60))|0;
 STACKTOP = sp;return ($61|0);
}
function _fts3DeleteSegment($p,$pSeg) {
 $p = $p|0;
 $pSeg = $pSeg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDelete = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pDelete = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pSeg;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)!=(0);
 $11 = ($9|0)!=(0);
 $12 = $10 | $11;
 if (!($12)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = (_fts3SqlStmt($13,17,$pDelete,0)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if (!($16)) {
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $17 = HEAP32[$pDelete>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = $19;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 (_sqlite3_bind_int64($17,1,$22,$25)|0);
 $26 = HEAP32[$pDelete>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 24|0);
 $29 = $28;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (($29) + 4)|0;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 (_sqlite3_bind_int64($26,2,$31,$34)|0);
 $35 = HEAP32[$pDelete>>2]|0;
 (_sqlite3_step($35)|0);
 $36 = HEAP32[$pDelete>>2]|0;
 $37 = (_sqlite3_reset($36)|0);
 HEAP32[$rc>>2] = $37;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3PrefixCompress($zPrev,$nPrev,$zNext,$nNext) {
 $zPrev = $zPrev|0;
 $nPrev = $nPrev|0;
 $zNext = $zNext|0;
 $nNext = $nNext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $zPrev;
 HEAP32[$1>>2] = $nPrev;
 HEAP32[$2>>2] = $zNext;
 HEAP32[$3>>2] = $nNext;
 HEAP32[$n>>2] = 0;
 while(1) {
  $4 = HEAP32[$n>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$n>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = HEAP32[$n>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (($13) + ($12)|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = ($11|0)==($16|0);
  if (!($17)) {
   label = 5;
   break;
  }
  $18 = HEAP32[$n>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$n>>2] = $19;
 }
 if ((label|0) == 5) {
  $20 = HEAP32[$n>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _fts3NodeAddTerm($p,$ppTree,$isCopyTerm,$zTerm,$nTerm) {
 $p = $p|0;
 $ppTree = $ppTree|0;
 $isCopyTerm = $isCopyTerm|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nData = 0, $nPrefix = 0, $nReq = 0, $nSuffix = 0, $pNew = 0, $pParent = 0, $pTree = 0, $rc = 0, $zNew = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $pTree = sp + 32|0;
 $rc = sp + 28|0;
 $pNew = sp + 24|0;
 $nData = sp + 20|0;
 $nReq = sp + 16|0;
 $nPrefix = sp + 12|0;
 $nSuffix = sp + 8|0;
 $zNew = sp + 4|0;
 $pParent = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $ppTree;
 HEAP32[$3>>2] = $isCopyTerm;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTree>>2] = $7;
 $8 = HEAP32[$pTree>>2]|0;
 $9 = ($8|0)!=(0|0);
 do {
  if ($9) {
   $10 = HEAP32[$pTree>>2]|0;
   $11 = ((($10)) + 32|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$nData>>2] = $12;
   $13 = HEAP32[$nData>>2]|0;
   HEAP32[$nReq>>2] = $13;
   $14 = HEAP32[$pTree>>2]|0;
   $15 = ((($14)) + 16|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pTree>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = HEAP32[$5>>2]|0;
   $22 = (_fts3PrefixCompress($16,$19,$20,$21)|0);
   HEAP32[$nPrefix>>2] = $22;
   $23 = HEAP32[$5>>2]|0;
   $24 = HEAP32[$nPrefix>>2]|0;
   $25 = (($23) - ($24))|0;
   HEAP32[$nSuffix>>2] = $25;
   $26 = HEAP32[$nPrefix>>2]|0;
   $27 = ($26|0)<(0);
   $28 = $27 << 31 >> 31;
   $29 = (_sqlite3Fts3VarintLen($26,$28)|0);
   $30 = HEAP32[$nSuffix>>2]|0;
   $31 = ($30|0)<(0);
   $32 = $31 << 31 >> 31;
   $33 = (_sqlite3Fts3VarintLen($30,$32)|0);
   $34 = (($29) + ($33))|0;
   $35 = HEAP32[$nSuffix>>2]|0;
   $36 = (($34) + ($35))|0;
   $37 = HEAP32[$nReq>>2]|0;
   $38 = (($37) + ($36))|0;
   HEAP32[$nReq>>2] = $38;
   $39 = HEAP32[$nReq>>2]|0;
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 224|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($39|0)<=($42|0);
   if (!($43)) {
    $44 = HEAP32[$pTree>>2]|0;
    $45 = ((($44)) + 16|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = ($46|0)!=(0|0);
    if ($47) {
     break;
    }
   }
   $48 = HEAP32[$nReq>>2]|0;
   $49 = HEAP32[$1>>2]|0;
   $50 = ((($49)) + 224|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($48|0)>($51|0);
   if ($52) {
    $53 = HEAP32[$nReq>>2]|0;
    $54 = (_sqlite3_malloc($53)|0);
    $55 = HEAP32[$pTree>>2]|0;
    $56 = ((($55)) + 36|0);
    HEAP32[$56>>2] = $54;
    $57 = HEAP32[$pTree>>2]|0;
    $58 = ((($57)) + 36|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = ($59|0)!=(0|0);
    if (!($60)) {
     HEAP32[$0>>2] = 7;
     $207 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($207|0);
    }
   }
   $61 = HEAP32[$pTree>>2]|0;
   $62 = ((($61)) + 16|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($63|0)!=(0|0);
   if ($64) {
    $65 = HEAP32[$nData>>2]|0;
    $66 = HEAP32[$pTree>>2]|0;
    $67 = ((($66)) + 36|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + ($65)|0);
    $70 = HEAP32[$nPrefix>>2]|0;
    $71 = ($70|0)<(0);
    $72 = $71 << 31 >> 31;
    $73 = (_sqlite3Fts3PutVarint($69,$70,$72)|0);
    $74 = HEAP32[$nData>>2]|0;
    $75 = (($74) + ($73))|0;
    HEAP32[$nData>>2] = $75;
   }
   $76 = HEAP32[$nData>>2]|0;
   $77 = HEAP32[$pTree>>2]|0;
   $78 = ((($77)) + 36|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + ($76)|0);
   $81 = HEAP32[$nSuffix>>2]|0;
   $82 = ($81|0)<(0);
   $83 = $82 << 31 >> 31;
   $84 = (_sqlite3Fts3PutVarint($80,$81,$83)|0);
   $85 = HEAP32[$nData>>2]|0;
   $86 = (($85) + ($84))|0;
   HEAP32[$nData>>2] = $86;
   $87 = HEAP32[$nData>>2]|0;
   $88 = HEAP32[$pTree>>2]|0;
   $89 = ((($88)) + 36|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (($90) + ($87)|0);
   $92 = HEAP32[$nPrefix>>2]|0;
   $93 = HEAP32[$4>>2]|0;
   $94 = (($93) + ($92)|0);
   $95 = HEAP32[$nSuffix>>2]|0;
   _memcpy(($91|0),($94|0),($95|0))|0;
   $96 = HEAP32[$nData>>2]|0;
   $97 = HEAP32[$nSuffix>>2]|0;
   $98 = (($96) + ($97))|0;
   $99 = HEAP32[$pTree>>2]|0;
   $100 = ((($99)) + 32|0);
   HEAP32[$100>>2] = $98;
   $101 = HEAP32[$pTree>>2]|0;
   $102 = ((($101)) + 12|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = (($103) + 1)|0;
   HEAP32[$102>>2] = $104;
   $105 = HEAP32[$3>>2]|0;
   $106 = ($105|0)!=(0);
   if ($106) {
    $107 = HEAP32[$pTree>>2]|0;
    $108 = ((($107)) + 24|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = HEAP32[$5>>2]|0;
    $111 = ($109|0)<($110|0);
    do {
     if ($111) {
      $112 = HEAP32[$pTree>>2]|0;
      $113 = ((($112)) + 28|0);
      $114 = HEAP32[$113>>2]|0;
      $115 = HEAP32[$5>>2]|0;
      $116 = $115<<1;
      $117 = (_sqlite3_realloc($114,$116)|0);
      HEAP32[$zNew>>2] = $117;
      $118 = HEAP32[$zNew>>2]|0;
      $119 = ($118|0)!=(0|0);
      if ($119) {
       $120 = HEAP32[$5>>2]|0;
       $121 = $120<<1;
       $122 = HEAP32[$pTree>>2]|0;
       $123 = ((($122)) + 24|0);
       HEAP32[$123>>2] = $121;
       $124 = HEAP32[$zNew>>2]|0;
       $125 = HEAP32[$pTree>>2]|0;
       $126 = ((($125)) + 28|0);
       HEAP32[$126>>2] = $124;
       break;
      }
      HEAP32[$0>>2] = 7;
      $207 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($207|0);
     }
    } while(0);
    $127 = HEAP32[$pTree>>2]|0;
    $128 = ((($127)) + 28|0);
    $129 = HEAP32[$128>>2]|0;
    $130 = HEAP32[$pTree>>2]|0;
    $131 = ((($130)) + 16|0);
    HEAP32[$131>>2] = $129;
    $132 = HEAP32[$pTree>>2]|0;
    $133 = ((($132)) + 16|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = HEAP32[$4>>2]|0;
    $136 = HEAP32[$5>>2]|0;
    _memcpy(($134|0),($135|0),($136|0))|0;
    $137 = HEAP32[$5>>2]|0;
    $138 = HEAP32[$pTree>>2]|0;
    $139 = ((($138)) + 20|0);
    HEAP32[$139>>2] = $137;
   } else {
    $140 = HEAP32[$4>>2]|0;
    $141 = HEAP32[$pTree>>2]|0;
    $142 = ((($141)) + 16|0);
    HEAP32[$142>>2] = $140;
    $143 = HEAP32[$5>>2]|0;
    $144 = HEAP32[$pTree>>2]|0;
    $145 = ((($144)) + 20|0);
    HEAP32[$145>>2] = $143;
   }
   HEAP32[$0>>2] = 0;
   $207 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($207|0);
  }
 } while(0);
 $146 = HEAP32[$1>>2]|0;
 $147 = ((($146)) + 224|0);
 $148 = HEAP32[$147>>2]|0;
 $149 = (40 + ($148))|0;
 $150 = (_sqlite3_malloc($149)|0);
 HEAP32[$pNew>>2] = $150;
 $151 = HEAP32[$pNew>>2]|0;
 $152 = ($151|0)!=(0|0);
 if (!($152)) {
  HEAP32[$0>>2] = 7;
  $207 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($207|0);
 }
 $153 = HEAP32[$pNew>>2]|0;
 dest=$153; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $154 = HEAP32[$pNew>>2]|0;
 $155 = ((($154)) + 32|0);
 HEAP32[$155>>2] = 11;
 $156 = HEAP32[$pNew>>2]|0;
 $157 = ((($156)) + 40|0);
 $158 = HEAP32[$pNew>>2]|0;
 $159 = ((($158)) + 36|0);
 HEAP32[$159>>2] = $157;
 $160 = HEAP32[$pTree>>2]|0;
 $161 = ($160|0)!=(0|0);
 if ($161) {
  $162 = HEAP32[$pTree>>2]|0;
  $163 = HEAP32[$162>>2]|0;
  HEAP32[$pParent>>2] = $163;
  $164 = HEAP32[$1>>2]|0;
  $165 = HEAP32[$3>>2]|0;
  $166 = HEAP32[$4>>2]|0;
  $167 = HEAP32[$5>>2]|0;
  $168 = (_fts3NodeAddTerm($164,$pParent,$165,$166,$167)|0);
  HEAP32[$rc>>2] = $168;
  $169 = HEAP32[$pTree>>2]|0;
  $170 = HEAP32[$169>>2]|0;
  $171 = ($170|0)==(0|0);
  if ($171) {
   $172 = HEAP32[$pParent>>2]|0;
   $173 = HEAP32[$pTree>>2]|0;
   HEAP32[$173>>2] = $172;
  }
  $174 = HEAP32[$pNew>>2]|0;
  $175 = HEAP32[$pTree>>2]|0;
  $176 = ((($175)) + 4|0);
  HEAP32[$176>>2] = $174;
  $177 = HEAP32[$pTree>>2]|0;
  $178 = ((($177)) + 8|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = HEAP32[$pNew>>2]|0;
  $181 = ((($180)) + 8|0);
  HEAP32[$181>>2] = $179;
  $182 = HEAP32[$pParent>>2]|0;
  $183 = HEAP32[$pNew>>2]|0;
  HEAP32[$183>>2] = $182;
  $184 = HEAP32[$pTree>>2]|0;
  $185 = ((($184)) + 28|0);
  $186 = HEAP32[$185>>2]|0;
  $187 = HEAP32[$pNew>>2]|0;
  $188 = ((($187)) + 28|0);
  HEAP32[$188>>2] = $186;
  $189 = HEAP32[$pTree>>2]|0;
  $190 = ((($189)) + 24|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = HEAP32[$pNew>>2]|0;
  $193 = ((($192)) + 24|0);
  HEAP32[$193>>2] = $191;
  $194 = HEAP32[$pTree>>2]|0;
  $195 = ((($194)) + 28|0);
  HEAP32[$195>>2] = 0;
 } else {
  $196 = HEAP32[$pNew>>2]|0;
  $197 = HEAP32[$pNew>>2]|0;
  $198 = ((($197)) + 8|0);
  HEAP32[$198>>2] = $196;
  $199 = HEAP32[$1>>2]|0;
  $200 = HEAP32[$3>>2]|0;
  $201 = HEAP32[$4>>2]|0;
  $202 = HEAP32[$5>>2]|0;
  $203 = (_fts3NodeAddTerm($199,$pNew,$200,$201,$202)|0);
  HEAP32[$rc>>2] = $203;
 }
 $204 = HEAP32[$pNew>>2]|0;
 $205 = HEAP32[$2>>2]|0;
 HEAP32[$205>>2] = $204;
 $206 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $206;
 $207 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($207|0);
}
function _fts3SegReaderDoclistCmpRev($pLhs,$pRhs) {
 $pLhs = $pLhs|0;
 $pRhs = $pRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pLhs;
 HEAP32[$1>>2] = $pRhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 80|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = $5&1;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 80|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = $10&1;
 $12 = (($6) - ($11))|0;
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 88|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 88|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($19|0)==($27|0);
 $32 = ($22|0)==($30|0);
 $33 = $31 & $32;
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) - ($37))|0;
  HEAP32[$rc>>2] = $38;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 88|0);
  $41 = $40;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) + 4)|0;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 88|0);
  $49 = $48;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = ($46|0)<($54|0);
  $56 = ($43>>>0)<($51>>>0);
  $57 = ($46|0)==($54|0);
  $58 = $57 & $56;
  $59 = $55 | $58;
  $60 = $59 ? 1 : -1;
  HEAP32[$rc>>2] = $60;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _fts3SegReaderDoclistCmp($pLhs,$pRhs) {
 $pLhs = $pLhs|0;
 $pRhs = $pRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pLhs;
 HEAP32[$1>>2] = $pRhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 80|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = $5&1;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 80|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = $10&1;
 $12 = (($6) - ($11))|0;
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 88|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 88|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = ($19|0)==($27|0);
 $32 = ($22|0)==($30|0);
 $33 = $31 & $32;
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) - ($37))|0;
  HEAP32[$rc>>2] = $38;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 88|0);
  $41 = $40;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) + 4)|0;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 88|0);
  $49 = $48;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = ($46|0)>($54|0);
  $56 = ($43>>>0)>($51>>>0);
  $57 = ($46|0)==($54|0);
  $58 = $57 & $56;
  $59 = $55 | $58;
  $60 = $59 ? 1 : -1;
  HEAP32[$rc>>2] = $60;
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _fts3SegReaderSetEof($pSeg) {
 $pSeg = $pSeg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pSeg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 5|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 if ($5) {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 40|0);
  HEAP32[$15>>2] = 0;
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3_free($8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 (_sqlite3_blob_close($11)|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 52|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 40|0);
 HEAP32[$15>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3SegReaderNext($p,$pReader,$bIncr) {
 $p = $p|0;
 $pReader = $pReader|0;
 $bIncr = $bIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCopy = 0, $nCopy = 0, $nNew = 0, $nPrefix = 0, $nSuffix = 0, $or$cond = 0;
 var $pElem = 0, $pList = 0, $pNext = 0, $rc = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $rc = sp + 36|0;
 $pNext = sp + 32|0;
 $nPrefix = sp + 28|0;
 $nSuffix = sp + 24|0;
 $pElem = sp + 20|0;
 $aCopy = sp + 16|0;
 $pList = sp + 12|0;
 $nCopy = sp + 8|0;
 $nNew = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pReader;
 HEAP32[$3>>2] = $bIncr;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 72|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$2>>2]|0;
 if ($7) {
  $11 = ((($8)) + 76|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 72|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12)|0);
  HEAP32[$pNext>>2] = $16;
 } else {
  $9 = ((($8)) + 40|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pNext>>2] = $10;
 }
 $17 = HEAP32[$pNext>>2]|0;
 $18 = ($17|0)!=(0|0);
 if ($18) {
  $19 = HEAP32[$pNext>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 44|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 40|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + ($22)|0);
  $27 = ($19>>>0)>=($26>>>0);
  if ($27) {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 56|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)!=(0|0);
  $32 = HEAP32[$2>>2]|0;
  if ($31) {
   $33 = ((($32)) + 56|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$pElem>>2] = $35;
   $36 = HEAP32[$2>>2]|0;
   $37 = ((($36)) + 40|0);
   $38 = HEAP32[$37>>2]|0;
   _sqlite3_free($38);
   $39 = HEAP32[$2>>2]|0;
   $40 = ((($39)) + 40|0);
   HEAP32[$40>>2] = 0;
   $41 = HEAP32[$pElem>>2]|0;
   $42 = ($41|0)!=(0|0);
   do {
    if ($42) {
     $43 = HEAP32[$pElem>>2]|0;
     $44 = ((($43)) + 8|0);
     $45 = HEAP32[$44>>2]|0;
     HEAP32[$pList>>2] = $45;
     $46 = HEAP32[$pList>>2]|0;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($47) + 1)|0;
     HEAP32[$nCopy>>2] = $48;
     $49 = HEAP32[$pElem>>2]|0;
     $50 = ((($49)) + 12|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = HEAP32[$2>>2]|0;
     $53 = ((($52)) + 64|0);
     HEAP32[$53>>2] = $51;
     $54 = HEAP32[$pElem>>2]|0;
     $55 = ((($54)) + 16|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = HEAP32[$2>>2]|0;
     $58 = ((($57)) + 60|0);
     HEAP32[$58>>2] = $56;
     $59 = HEAP32[$nCopy>>2]|0;
     $60 = (_sqlite3_malloc($59)|0);
     HEAP32[$aCopy>>2] = $60;
     $61 = HEAP32[$aCopy>>2]|0;
     $62 = ($61|0)!=(0|0);
     if ($62) {
      $63 = HEAP32[$aCopy>>2]|0;
      $64 = HEAP32[$pList>>2]|0;
      $65 = ((($64)) + 4|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$nCopy>>2]|0;
      _memcpy(($63|0),($66|0),($67|0))|0;
      $68 = HEAP32[$nCopy>>2]|0;
      $69 = HEAP32[$2>>2]|0;
      $70 = ((($69)) + 76|0);
      HEAP32[$70>>2] = $68;
      $71 = HEAP32[$2>>2]|0;
      $72 = ((($71)) + 44|0);
      HEAP32[$72>>2] = $68;
      $73 = HEAP32[$aCopy>>2]|0;
      $74 = HEAP32[$2>>2]|0;
      $75 = ((($74)) + 72|0);
      HEAP32[$75>>2] = $73;
      $76 = HEAP32[$2>>2]|0;
      $77 = ((($76)) + 40|0);
      HEAP32[$77>>2] = $73;
      $78 = HEAP32[$2>>2]|0;
      $79 = ((($78)) + 56|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = ((($80)) + 4|0);
      HEAP32[$79>>2] = $81;
      break;
     }
     HEAP32[$0>>2] = 7;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
   } while(0);
   HEAP32[$0>>2] = 0;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  _fts3SegReaderSetEof($32);
  $82 = HEAP32[$2>>2]|0;
  $83 = ((($82)) + 32|0);
  $84 = $83;
  $85 = $84;
  $86 = HEAP32[$85>>2]|0;
  $87 = (($84) + 4)|0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = HEAP32[$2>>2]|0;
  $91 = ((($90)) + 16|0);
  $92 = $91;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($89|0)>($97|0);
  $99 = ($86>>>0)>=($94>>>0);
  $100 = ($89|0)==($97|0);
  $101 = $100 & $99;
  $102 = $98 | $101;
  if ($102) {
   HEAP32[$0>>2] = 0;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $103 = HEAP32[$1>>2]|0;
  $104 = HEAP32[$2>>2]|0;
  $105 = ((($104)) + 32|0);
  $106 = $105;
  $107 = $106;
  $108 = HEAP32[$107>>2]|0;
  $109 = (($106) + 4)|0;
  $110 = $109;
  $111 = HEAP32[$110>>2]|0;
  $112 = (_i64Add(($108|0),($111|0),1,0)|0);
  $113 = tempRet0;
  $114 = $105;
  $115 = $114;
  HEAP32[$115>>2] = $112;
  $116 = (($114) + 4)|0;
  $117 = $116;
  HEAP32[$117>>2] = $113;
  $118 = HEAP32[$2>>2]|0;
  $119 = ((($118)) + 40|0);
  $120 = HEAP32[$2>>2]|0;
  $121 = ((($120)) + 44|0);
  $122 = HEAP32[$3>>2]|0;
  $123 = ($122|0)!=(0);
  $124 = HEAP32[$2>>2]|0;
  $125 = ((($124)) + 48|0);
  $126 = $123 ? $125 : 0;
  $127 = (_sqlite3Fts3ReadBlock($103,$112,$113,$119,$121,$126)|0);
  HEAP32[$rc>>2] = $127;
  $128 = HEAP32[$rc>>2]|0;
  $129 = ($128|0)!=(0);
  if ($129) {
   $130 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $130;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
  $131 = HEAP32[$3>>2]|0;
  $132 = ($131|0)!=(0);
  if ($132) {
   $133 = HEAP32[$2>>2]|0;
   $134 = ((($133)) + 48|0);
   $135 = HEAP32[$134>>2]|0;
   $136 = HEAP32[$2>>2]|0;
   $137 = ((($136)) + 44|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = ($135|0)<($138|0);
   if ($139) {
    $140 = HEAP32[$1>>2]|0;
    $141 = ((($140)) + 244|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = HEAP32[$2>>2]|0;
    $144 = ((($143)) + 52|0);
    HEAP32[$144>>2] = $142;
    $145 = HEAP32[$1>>2]|0;
    $146 = ((($145)) + 244|0);
    HEAP32[$146>>2] = 0;
   }
  }
  $147 = HEAP32[$2>>2]|0;
  $148 = ((($147)) + 40|0);
  $149 = HEAP32[$148>>2]|0;
  HEAP32[$pNext>>2] = $149;
 }
 $150 = HEAP32[$2>>2]|0;
 $151 = HEAP32[$pNext>>2]|0;
 $152 = (_fts3SegReaderRequire($150,$151,20)|0);
 HEAP32[$rc>>2] = $152;
 $153 = HEAP32[$rc>>2]|0;
 $154 = ($153|0)!=(0);
 if ($154) {
  $155 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $155;
  $293 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($293|0);
 }
 $156 = HEAP32[$pNext>>2]|0;
 $157 = HEAP8[$156>>0]|0;
 $158 = $157&255;
 $159 = $158 & 128;
 $160 = ($159|0)!=(0);
 $161 = HEAP32[$pNext>>2]|0;
 if ($160) {
  $162 = (_sqlite3Fts3GetVarint32($161,$nPrefix)|0);
  $167 = $162;
 } else {
  $163 = HEAP8[$161>>0]|0;
  $164 = $163&255;
  HEAP32[$nPrefix>>2] = $164;
  $167 = 1;
 }
 $165 = HEAP32[$pNext>>2]|0;
 $166 = (($165) + ($167)|0);
 HEAP32[$pNext>>2] = $166;
 $168 = HEAP32[$pNext>>2]|0;
 $169 = HEAP8[$168>>0]|0;
 $170 = $169&255;
 $171 = $170 & 128;
 $172 = ($171|0)!=(0);
 $173 = HEAP32[$pNext>>2]|0;
 if ($172) {
  $174 = (_sqlite3Fts3GetVarint32($173,$nSuffix)|0);
  $179 = $174;
 } else {
  $175 = HEAP8[$173>>0]|0;
  $176 = $175&255;
  HEAP32[$nSuffix>>2] = $176;
  $179 = 1;
 }
 $177 = HEAP32[$pNext>>2]|0;
 $178 = (($177) + ($179)|0);
 HEAP32[$pNext>>2] = $178;
 $180 = HEAP32[$nPrefix>>2]|0;
 $181 = ($180|0)<(0);
 $182 = HEAP32[$nSuffix>>2]|0;
 $183 = ($182|0)<=(0);
 $or$cond = $181 | $183;
 if (!($or$cond)) {
  $184 = HEAP32[$nSuffix>>2]|0;
  $185 = HEAP32[$pNext>>2]|0;
  $186 = (($185) + ($184)|0);
  $187 = HEAP32[$2>>2]|0;
  $188 = ((($187)) + 44|0);
  $189 = HEAP32[$188>>2]|0;
  $190 = HEAP32[$2>>2]|0;
  $191 = ((($190)) + 40|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = (($192) + ($189)|0);
  $194 = ($186>>>0)>($193>>>0);
  if (!($194)) {
   $195 = HEAP32[$nPrefix>>2]|0;
   $196 = HEAP32[$nSuffix>>2]|0;
   $197 = (($195) + ($196))|0;
   $198 = HEAP32[$2>>2]|0;
   $199 = ((($198)) + 68|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = ($197|0)>($200|0);
   do {
    if ($201) {
     $202 = HEAP32[$nPrefix>>2]|0;
     $203 = HEAP32[$nSuffix>>2]|0;
     $204 = (($202) + ($203))|0;
     $205 = $204<<1;
     HEAP32[$nNew>>2] = $205;
     $206 = HEAP32[$2>>2]|0;
     $207 = ((($206)) + 64|0);
     $208 = HEAP32[$207>>2]|0;
     $209 = HEAP32[$nNew>>2]|0;
     $210 = (_sqlite3_realloc($208,$209)|0);
     HEAP32[$zNew>>2] = $210;
     $211 = HEAP32[$zNew>>2]|0;
     $212 = ($211|0)!=(0|0);
     if ($212) {
      $213 = HEAP32[$zNew>>2]|0;
      $214 = HEAP32[$2>>2]|0;
      $215 = ((($214)) + 64|0);
      HEAP32[$215>>2] = $213;
      $216 = HEAP32[$nNew>>2]|0;
      $217 = HEAP32[$2>>2]|0;
      $218 = ((($217)) + 68|0);
      HEAP32[$218>>2] = $216;
      break;
     }
     HEAP32[$0>>2] = 7;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
   } while(0);
   $219 = HEAP32[$2>>2]|0;
   $220 = HEAP32[$pNext>>2]|0;
   $221 = HEAP32[$nSuffix>>2]|0;
   $222 = (($221) + 10)|0;
   $223 = (_fts3SegReaderRequire($219,$220,$222)|0);
   HEAP32[$rc>>2] = $223;
   $224 = HEAP32[$rc>>2]|0;
   $225 = ($224|0)!=(0);
   if ($225) {
    $226 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $226;
    $293 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($293|0);
   }
   $227 = HEAP32[$nPrefix>>2]|0;
   $228 = HEAP32[$2>>2]|0;
   $229 = ((($228)) + 64|0);
   $230 = HEAP32[$229>>2]|0;
   $231 = (($230) + ($227)|0);
   $232 = HEAP32[$pNext>>2]|0;
   $233 = HEAP32[$nSuffix>>2]|0;
   _memcpy(($231|0),($232|0),($233|0))|0;
   $234 = HEAP32[$nPrefix>>2]|0;
   $235 = HEAP32[$nSuffix>>2]|0;
   $236 = (($234) + ($235))|0;
   $237 = HEAP32[$2>>2]|0;
   $238 = ((($237)) + 60|0);
   HEAP32[$238>>2] = $236;
   $239 = HEAP32[$nSuffix>>2]|0;
   $240 = HEAP32[$pNext>>2]|0;
   $241 = (($240) + ($239)|0);
   HEAP32[$pNext>>2] = $241;
   $242 = HEAP32[$pNext>>2]|0;
   $243 = HEAP8[$242>>0]|0;
   $244 = $243&255;
   $245 = $244 & 128;
   $246 = ($245|0)!=(0);
   $247 = HEAP32[$pNext>>2]|0;
   if ($246) {
    $248 = HEAP32[$2>>2]|0;
    $249 = ((($248)) + 76|0);
    $250 = (_sqlite3Fts3GetVarint32($247,$249)|0);
    $257 = $250;
   } else {
    $251 = HEAP8[$247>>0]|0;
    $252 = $251&255;
    $253 = HEAP32[$2>>2]|0;
    $254 = ((($253)) + 76|0);
    HEAP32[$254>>2] = $252;
    $257 = 1;
   }
   $255 = HEAP32[$pNext>>2]|0;
   $256 = (($255) + ($257)|0);
   HEAP32[$pNext>>2] = $256;
   $258 = HEAP32[$pNext>>2]|0;
   $259 = HEAP32[$2>>2]|0;
   $260 = ((($259)) + 72|0);
   HEAP32[$260>>2] = $258;
   $261 = HEAP32[$2>>2]|0;
   $262 = ((($261)) + 80|0);
   HEAP32[$262>>2] = 0;
   $263 = HEAP32[$2>>2]|0;
   $264 = ((($263)) + 76|0);
   $265 = HEAP32[$264>>2]|0;
   $266 = HEAP32[$2>>2]|0;
   $267 = ((($266)) + 72|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = (($268) + ($265)|0);
   $270 = HEAP32[$2>>2]|0;
   $271 = ((($270)) + 44|0);
   $272 = HEAP32[$271>>2]|0;
   $273 = HEAP32[$2>>2]|0;
   $274 = ((($273)) + 40|0);
   $275 = HEAP32[$274>>2]|0;
   $276 = (($275) + ($272)|0);
   $277 = ($269>>>0)>($276>>>0);
   do {
    if (!($277)) {
     $278 = HEAP32[$2>>2]|0;
     $279 = ((($278)) + 48|0);
     $280 = HEAP32[$279>>2]|0;
     $281 = ($280|0)==(0);
     if ($281) {
      $282 = HEAP32[$2>>2]|0;
      $283 = ((($282)) + 76|0);
      $284 = HEAP32[$283>>2]|0;
      $285 = (($284) - 1)|0;
      $286 = HEAP32[$2>>2]|0;
      $287 = ((($286)) + 72|0);
      $288 = HEAP32[$287>>2]|0;
      $289 = (($288) + ($285)|0);
      $290 = HEAP8[$289>>0]|0;
      $291 = $290 << 24 >> 24;
      $292 = ($291|0)!=(0);
      if ($292) {
       break;
      }
     }
     HEAP32[$0>>2] = 0;
     $293 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($293|0);
    }
   } while(0);
   HEAP32[$0>>2] = 267;
   $293 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($293|0);
  }
 }
 HEAP32[$0>>2] = 267;
 $293 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($293|0);
}
function _fts3SegReaderCmp($pLhs,$pRhs) {
 $pLhs = $pLhs|0;
 $pRhs = $pRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $rc2 = sp;
 HEAP32[$0>>2] = $pLhs;
 HEAP32[$1>>2] = $pRhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 40|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  if ($9) {
   $10 = HEAP32[$0>>2]|0;
   $11 = ((($10)) + 60|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 60|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (($12) - ($15))|0;
   HEAP32[$rc2>>2] = $16;
   $17 = HEAP32[$rc2>>2]|0;
   $18 = ($17|0)<(0);
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 64|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 64|0);
   $24 = HEAP32[$23>>2]|0;
   if ($18) {
    $25 = HEAP32[$0>>2]|0;
    $26 = ((($25)) + 60|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (_memcmp($21,$24,$27)|0);
    HEAP32[$rc>>2] = $28;
   } else {
    $29 = HEAP32[$1>>2]|0;
    $30 = ((($29)) + 60|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (_memcmp($21,$24,$31)|0);
    HEAP32[$rc>>2] = $32;
   }
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $35;
   }
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $36 = HEAP32[$0>>2]|0;
  $37 = ((($36)) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)==(0|0);
  $40 = $39&1;
  $41 = HEAP32[$1>>2]|0;
  $42 = ((($41)) + 40|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($43|0)==(0|0);
  $45 = $44&1;
  $46 = (($40) - ($45))|0;
  HEAP32[$rc>>2] = $46;
 }
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)==(0);
 if (!($48)) {
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) - ($52))|0;
 HEAP32[$rc>>2] = $53;
 $54 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _fts3SegReaderSort($apSegment,$nSegment,$nSuspect,$xCmp) {
 $apSegment = $apSegment|0;
 $nSegment = $nSegment|0;
 $nSuspect = $nSuspect|0;
 $xCmp = $xCmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $apSegment;
 HEAP32[$1>>2] = $nSegment;
 HEAP32[$2>>2] = $nSuspect;
 HEAP32[$3>>2] = $xCmp;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($4|0)==($5|0);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  $8 = (($7) + -1)|0;
  HEAP32[$2>>2] = $8;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = (($9) - 1)|0;
 HEAP32[$i>>2] = $10;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)>=(0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  HEAP32[$j>>2] = $13;
  while(1) {
   $14 = HEAP32[$j>>2]|0;
   $15 = HEAP32[$1>>2]|0;
   $16 = (($15) - 1)|0;
   $17 = ($14|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$3>>2]|0;
   $19 = HEAP32[$j>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = (($20) + ($19<<2)|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = (($23) + 1)|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = (($25) + ($24<<2)|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (FUNCTION_TABLE_iii[$18 & 255]($22,$27)|0);
   $29 = ($28|0)<(0);
   if ($29) {
    break;
   }
   $30 = HEAP32[$j>>2]|0;
   $31 = (($30) + 1)|0;
   $32 = HEAP32[$0>>2]|0;
   $33 = (($32) + ($31<<2)|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pTmp>>2] = $34;
   $35 = HEAP32[$j>>2]|0;
   $36 = HEAP32[$0>>2]|0;
   $37 = (($36) + ($35<<2)|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = HEAP32[$j>>2]|0;
   $40 = (($39) + 1)|0;
   $41 = HEAP32[$0>>2]|0;
   $42 = (($41) + ($40<<2)|0);
   HEAP32[$42>>2] = $38;
   $43 = HEAP32[$pTmp>>2]|0;
   $44 = HEAP32[$j>>2]|0;
   $45 = HEAP32[$0>>2]|0;
   $46 = (($45) + ($44<<2)|0);
   HEAP32[$46>>2] = $43;
   $47 = HEAP32[$j>>2]|0;
   $48 = (($47) + 1)|0;
   HEAP32[$j>>2] = $48;
  }
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + -1)|0;
  HEAP32[$i>>2] = $50;
 }
 STACKTOP = sp;return;
}
function _fts3MsrBufferData($pMsr,$pList,$nList) {
 $pMsr = $pMsr|0;
 $pList = $pList|0;
 $nList = $nList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $pMsr;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $nList;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($4|0)>($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$3>>2]|0;
   $10 = $9<<1;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 20|0);
   HEAP32[$12>>2] = $10;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 20|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3_realloc($15,$18)|0);
   HEAP32[$pNew>>2] = $19;
   $20 = HEAP32[$pNew>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$pNew>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 16|0);
    HEAP32[$24>>2] = $22;
    break;
   }
   HEAP32[$0>>2] = 7;
   $30 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 } while(0);
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 _memcpy(($27|0),($28|0),($29|0))|0;
 HEAP32[$0>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _fts3SegReaderFirstDocid($pTab,$pReader) {
 $pTab = $pTab|0;
 $pReader = $pReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bEof = 0, $n = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $bEof = sp + 16|0;
 $n = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $pReader;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 231|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 56|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(0|0);
  if ($10) {
   HEAP8[$bEof>>0] = 0;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 88|0);
   $13 = $12;
   $14 = $13;
   HEAP32[$14>>2] = 0;
   $15 = (($13) + 4)|0;
   $16 = $15;
   HEAP32[$16>>2] = 0;
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 84|0);
   HEAP32[$18>>2] = 0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 72|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 76|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 80|0);
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 88|0);
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 84|0);
   _sqlite3Fts3DoclistPrev(0,$21,$24,$26,$28,$30,$bEof);
   $51 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($51|0);
  }
 }
 $31 = HEAP32[$1>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 72|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (_fts3SegReaderRequire($31,$34,10)|0);
 HEAP32[$rc>>2] = $35;
 $36 = HEAP32[$rc>>2]|0;
 $37 = ($36|0)==(0);
 if (!($37)) {
  $51 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 72|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 88|0);
 $43 = (_sqlite3Fts3GetVarint($40,$42)|0);
 HEAP32[$n>>2] = $43;
 $44 = HEAP32[$n>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 72|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44)|0);
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 80|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($51|0);
}
function _fts3SegReaderNextDocid($pTab,$pReader,$ppOffsetList,$pnOffsetList) {
 $pTab = $pTab|0;
 $pReader = $pReader|0;
 $ppOffsetList = $ppOffsetList|0;
 $pnOffsetList = $pnOffsetList|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bEof = 0, $c = 0, $iDelta = 0, $p = 0, $pEnd = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $rc = sp + 16|0;
 $p = sp + 12|0;
 $c = sp + 41|0;
 $bEof = sp + 40|0;
 $pEnd = sp + 8|0;
 $iDelta = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pReader;
 HEAP32[$3>>2] = $ppOffsetList;
 HEAP32[$4>>2] = $pnOffsetList;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 HEAP8[$c>>0] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 231|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(0);
 do {
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = ((($13)) + 56|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    HEAP8[$bEof>>0] = 0;
    $17 = HEAP32[$3>>2]|0;
    $18 = ($17|0)!=(0|0);
    if ($18) {
     $19 = HEAP32[$2>>2]|0;
     $20 = ((($19)) + 80|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = HEAP32[$3>>2]|0;
     HEAP32[$22>>2] = $21;
     $23 = HEAP32[$2>>2]|0;
     $24 = ((($23)) + 84|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = (($25) - 1)|0;
     $27 = HEAP32[$4>>2]|0;
     HEAP32[$27>>2] = $26;
    }
    $28 = HEAP32[$2>>2]|0;
    $29 = ((($28)) + 72|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    $32 = ((($31)) + 76|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = ((($34)) + 88|0);
    $36 = HEAP32[$2>>2]|0;
    $37 = ((($36)) + 84|0);
    _sqlite3Fts3DoclistPrev(0,$30,$33,$p,$35,$37,$bEof);
    $38 = HEAP8[$bEof>>0]|0;
    $39 = ($38<<24>>24)!=(0);
    if ($39) {
     $40 = HEAP32[$2>>2]|0;
     $41 = ((($40)) + 80|0);
     HEAP32[$41>>2] = 0;
     break;
    } else {
     $42 = HEAP32[$p>>2]|0;
     $43 = HEAP32[$2>>2]|0;
     $44 = ((($43)) + 80|0);
     HEAP32[$44>>2] = $42;
     break;
    }
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
 } while(0);
 do {
  if ((label|0) == 8) {
   $45 = HEAP32[$2>>2]|0;
   $46 = ((($45)) + 76|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = ((($48)) + 72|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($50) + ($47)|0);
   HEAP32[$pEnd>>2] = $51;
   while(1) {
    $52 = HEAP32[$p>>2]|0;
    $53 = HEAP8[$52>>0]|0;
    $54 = $53 << 24 >> 24;
    $55 = HEAP8[$c>>0]|0;
    $56 = $55 << 24 >> 24;
    $57 = $54 | $56;
    $58 = ($57|0)!=(0);
    if ($58) {
     $59 = HEAP32[$p>>2]|0;
     $60 = ((($59)) + 1|0);
     HEAP32[$p>>2] = $60;
     $61 = HEAP8[$59>>0]|0;
     $62 = $61 << 24 >> 24;
     $63 = $62 & 128;
     $64 = $63&255;
     HEAP8[$c>>0] = $64;
     continue;
    }
    $65 = HEAP32[$2>>2]|0;
    $66 = ((($65)) + 52|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = ($67|0)==(0|0);
    if ($68) {
     break;
    }
    $69 = HEAP32[$p>>2]|0;
    $70 = HEAP32[$2>>2]|0;
    $71 = ((($70)) + 48|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = HEAP32[$2>>2]|0;
    $74 = ((($73)) + 40|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (($75) + ($72)|0);
    $77 = ($69>>>0)<($76>>>0);
    if ($77) {
     break;
    }
    $78 = HEAP32[$2>>2]|0;
    $79 = (_fts3SegReaderIncrRead($78)|0);
    HEAP32[$rc>>2] = $79;
    $80 = HEAP32[$rc>>2]|0;
    $81 = ($80|0)!=(0);
    if ($81) {
     label = 14;
     break;
    }
   }
   if ((label|0) == 14) {
    $82 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $82;
    $153 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($153|0);
   }
   $83 = HEAP32[$p>>2]|0;
   $84 = ((($83)) + 1|0);
   HEAP32[$p>>2] = $84;
   $85 = HEAP32[$3>>2]|0;
   $86 = ($85|0)!=(0|0);
   if ($86) {
    $87 = HEAP32[$2>>2]|0;
    $88 = ((($87)) + 80|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$3>>2]|0;
    HEAP32[$90>>2] = $89;
    $91 = HEAP32[$p>>2]|0;
    $92 = HEAP32[$2>>2]|0;
    $93 = ((($92)) + 80|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = $91;
    $96 = $94;
    $97 = (($95) - ($96))|0;
    $98 = (($97) - 1)|0;
    $99 = HEAP32[$4>>2]|0;
    HEAP32[$99>>2] = $98;
   }
   while(1) {
    $100 = HEAP32[$p>>2]|0;
    $101 = HEAP32[$pEnd>>2]|0;
    $102 = ($100>>>0)<($101>>>0);
    if ($102) {
     $103 = HEAP32[$p>>2]|0;
     $104 = HEAP8[$103>>0]|0;
     $105 = $104 << 24 >> 24;
     $106 = ($105|0)==(0);
     $154 = $106;
    } else {
     $154 = 0;
    }
    $107 = HEAP32[$p>>2]|0;
    if (!($154)) {
     break;
    }
    $108 = ((($107)) + 1|0);
    HEAP32[$p>>2] = $108;
   }
   $109 = HEAP32[$pEnd>>2]|0;
   $110 = ($107>>>0)>=($109>>>0);
   $111 = HEAP32[$2>>2]|0;
   if ($110) {
    $112 = ((($111)) + 80|0);
    HEAP32[$112>>2] = 0;
    break;
   }
   $113 = HEAP32[$p>>2]|0;
   $114 = (_fts3SegReaderRequire($111,$113,10)|0);
   HEAP32[$rc>>2] = $114;
   $115 = HEAP32[$rc>>2]|0;
   $116 = ($115|0)==(0);
   if ($116) {
    $117 = HEAP32[$p>>2]|0;
    $118 = HEAP32[$p>>2]|0;
    $119 = (_sqlite3Fts3GetVarint($118,$iDelta)|0);
    $120 = (($117) + ($119)|0);
    $121 = HEAP32[$2>>2]|0;
    $122 = ((($121)) + 80|0);
    HEAP32[$122>>2] = $120;
    $123 = HEAP32[$1>>2]|0;
    $124 = ((($123)) + 231|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = ($125<<24>>24)!=(0);
    $127 = $iDelta;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = HEAP32[$2>>2]|0;
    $134 = ((($133)) + 88|0);
    $135 = $134;
    $136 = $135;
    $137 = HEAP32[$136>>2]|0;
    $138 = (($135) + 4)|0;
    $139 = $138;
    $140 = HEAP32[$139>>2]|0;
    if ($126) {
     $141 = (_i64Subtract(($137|0),($140|0),($129|0),($132|0))|0);
     $142 = tempRet0;
     $143 = $134;
     $144 = $143;
     HEAP32[$144>>2] = $141;
     $145 = (($143) + 4)|0;
     $146 = $145;
     HEAP32[$146>>2] = $142;
     break;
    } else {
     $147 = (_i64Add(($137|0),($140|0),($129|0),($132|0))|0);
     $148 = tempRet0;
     $149 = $134;
     $150 = $149;
     HEAP32[$150>>2] = $147;
     $151 = (($149) + 4)|0;
     $152 = $151;
     HEAP32[$152>>2] = $148;
     break;
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $153 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($153|0);
}
function _fts3ColumnFilter($iCol,$bZero,$ppList,$pnList) {
 $iCol = $iCol|0;
 $bZero = $bZero|0;
 $ppList = $ppList|0;
 $pnList = $pnList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $c = 0, $iCurrent = 0, $nList = 0;
 var $p = 0, $pEnd = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pList = sp + 16|0;
 $nList = sp + 12|0;
 $pEnd = sp + 8|0;
 $iCurrent = sp + 4|0;
 $p = sp;
 $c = sp + 36|0;
 HEAP32[$0>>2] = $iCol;
 HEAP32[$1>>2] = $bZero;
 HEAP32[$2>>2] = $ppList;
 HEAP32[$3>>2] = $pnList;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pList>>2] = $5;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nList>>2] = $7;
 $8 = HEAP32[$nList>>2]|0;
 $9 = HEAP32[$pList>>2]|0;
 $10 = (($9) + ($8)|0);
 HEAP32[$pEnd>>2] = $10;
 HEAP32[$iCurrent>>2] = 0;
 $11 = HEAP32[$pList>>2]|0;
 HEAP32[$p>>2] = $11;
 while(1) {
  HEAP8[$c>>0] = 0;
  while(1) {
   $12 = HEAP32[$p>>2]|0;
   $13 = HEAP32[$pEnd>>2]|0;
   $14 = ($12>>>0)<($13>>>0);
   if (!($14)) {
    break;
   }
   $15 = HEAP8[$c>>0]|0;
   $16 = $15 << 24 >> 24;
   $17 = HEAP32[$p>>2]|0;
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   $20 = $16 | $19;
   $21 = $20 & 254;
   $22 = ($21|0)!=(0);
   if (!($22)) {
    break;
   }
   $23 = HEAP32[$p>>2]|0;
   $24 = ((($23)) + 1|0);
   HEAP32[$p>>2] = $24;
   $25 = HEAP8[$23>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = $26 & 128;
   $28 = $27&255;
   HEAP8[$c>>0] = $28;
  }
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$iCurrent>>2]|0;
  $31 = ($29|0)==($30|0);
  $32 = HEAP32[$p>>2]|0;
  $33 = HEAP32[$pList>>2]|0;
  $34 = $32;
  $35 = $33;
  $36 = (($34) - ($35))|0;
  if ($31) {
   label = 7;
   break;
  }
  $37 = HEAP32[$nList>>2]|0;
  $38 = (($37) - ($36))|0;
  HEAP32[$nList>>2] = $38;
  $39 = HEAP32[$p>>2]|0;
  HEAP32[$pList>>2] = $39;
  $40 = HEAP32[$nList>>2]|0;
  $41 = ($40|0)==(0);
  if ($41) {
   break;
  }
  $42 = HEAP32[$pList>>2]|0;
  $43 = ((($42)) + 1|0);
  HEAP32[$p>>2] = $43;
  $44 = HEAP32[$p>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $46 & 128;
  $48 = ($47|0)!=(0);
  $49 = HEAP32[$p>>2]|0;
  if ($48) {
   $50 = (_sqlite3Fts3GetVarint32($49,$iCurrent)|0);
   $55 = $50;
  } else {
   $51 = HEAP8[$49>>0]|0;
   $52 = $51&255;
   HEAP32[$iCurrent>>2] = $52;
   $55 = 1;
  }
  $53 = HEAP32[$p>>2]|0;
  $54 = (($53) + ($55)|0);
  HEAP32[$p>>2] = $54;
 }
 if ((label|0) == 7) {
  HEAP32[$nList>>2] = $36;
 }
 $56 = HEAP32[$1>>2]|0;
 $57 = ($56|0)!=(0);
 if (!($57)) {
  $73 = HEAP32[$pList>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  HEAP32[$74>>2] = $73;
  $75 = HEAP32[$nList>>2]|0;
  $76 = HEAP32[$3>>2]|0;
  HEAP32[$76>>2] = $75;
  STACKTOP = sp;return;
 }
 $58 = HEAP32[$nList>>2]|0;
 $59 = HEAP32[$pList>>2]|0;
 $60 = (($59) + ($58)|0);
 $61 = HEAP32[$pEnd>>2]|0;
 $62 = ($60|0)!=($61|0);
 if (!($62)) {
  $73 = HEAP32[$pList>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  HEAP32[$74>>2] = $73;
  $75 = HEAP32[$nList>>2]|0;
  $76 = HEAP32[$3>>2]|0;
  HEAP32[$76>>2] = $75;
  STACKTOP = sp;return;
 }
 $63 = HEAP32[$nList>>2]|0;
 $64 = HEAP32[$pList>>2]|0;
 $65 = (($64) + ($63)|0);
 $66 = HEAP32[$pEnd>>2]|0;
 $67 = HEAP32[$nList>>2]|0;
 $68 = HEAP32[$pList>>2]|0;
 $69 = (($68) + ($67)|0);
 $70 = $66;
 $71 = $69;
 $72 = (($70) - ($71))|0;
 _memset(($65|0),0,($72|0))|0;
 $73 = HEAP32[$pList>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 HEAP32[$74>>2] = $73;
 $75 = HEAP32[$nList>>2]|0;
 $76 = HEAP32[$3>>2]|0;
 HEAP32[$76>>2] = $75;
 STACKTOP = sp;return;
}
function _sqlite3Fts3FirstFilter($0,$1,$pList,$nList,$pOut) {
 $0 = $0|0;
 $1 = $1|0;
 $pList = $pList|0;
 $nList = $nList|0;
 $pOut = $pOut|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $bWritten = 0, $iCol = 0, $nOut = 0, $p = 0, $pEnd = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 8|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $nOut = sp + 28|0;
 $bWritten = sp + 24|0;
 $p = sp + 20|0;
 $pEnd = sp + 16|0;
 $iCol = sp;
 $6 = $2;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$3>>2] = $pList;
 HEAP32[$4>>2] = $nList;
 HEAP32[$5>>2] = $pOut;
 HEAP32[$nOut>>2] = 0;
 HEAP32[$bWritten>>2] = 0;
 $10 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) + ($11)|0);
 HEAP32[$pEnd>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)!=(1);
 if ($17) {
  $18 = HEAP32[$p>>2]|0;
  $19 = HEAP8[$18>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)==(2);
  if ($21) {
   $22 = HEAP32[$nOut>>2]|0;
   $23 = HEAP32[$5>>2]|0;
   $24 = (($23) + ($22)|0);
   $25 = $2;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (($25) + 4)|0;
   $29 = $28;
   $30 = HEAP32[$29>>2]|0;
   $31 = (_sqlite3Fts3PutVarint($24,$27,$30)|0);
   $32 = HEAP32[$nOut>>2]|0;
   $33 = (($32) + ($31))|0;
   HEAP32[$nOut>>2] = $33;
   $34 = HEAP32[$nOut>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$nOut>>2] = $35;
   $36 = HEAP32[$5>>2]|0;
   $37 = (($36) + ($34)|0);
   HEAP8[$37>>0] = 2;
   HEAP32[$bWritten>>2] = 1;
  }
  _fts3ColumnlistCopy(0,$p);
 }
 while(1) {
  $38 = HEAP32[$p>>2]|0;
  $39 = HEAP32[$pEnd>>2]|0;
  $40 = ($38>>>0)<($39>>>0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$p>>2]|0;
  $42 = HEAP8[$41>>0]|0;
  $43 = $42 << 24 >> 24;
  $44 = ($43|0)==(1);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$p>>2]|0;
  $46 = ((($45)) + 1|0);
  HEAP32[$p>>2] = $46;
  $47 = HEAP32[$p>>2]|0;
  $48 = (_sqlite3Fts3GetVarint($47,$iCol)|0);
  $49 = HEAP32[$p>>2]|0;
  $50 = (($49) + ($48)|0);
  HEAP32[$p>>2] = $50;
  $51 = HEAP32[$p>>2]|0;
  $52 = HEAP8[$51>>0]|0;
  $53 = $52 << 24 >> 24;
  $54 = ($53|0)==(2);
  if ($54) {
   $55 = HEAP32[$bWritten>>2]|0;
   $56 = ($55|0)==(0);
   if ($56) {
    $57 = HEAP32[$nOut>>2]|0;
    $58 = HEAP32[$5>>2]|0;
    $59 = (($58) + ($57)|0);
    $60 = $2;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = (($60) + 4)|0;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (_sqlite3Fts3PutVarint($59,$62,$65)|0);
    $67 = HEAP32[$nOut>>2]|0;
    $68 = (($67) + ($66))|0;
    HEAP32[$nOut>>2] = $68;
    HEAP32[$bWritten>>2] = 1;
   }
   $69 = HEAP32[$nOut>>2]|0;
   $70 = (($69) + 1)|0;
   HEAP32[$nOut>>2] = $70;
   $71 = HEAP32[$5>>2]|0;
   $72 = (($71) + ($69)|0);
   HEAP8[$72>>0] = 1;
   $73 = HEAP32[$nOut>>2]|0;
   $74 = HEAP32[$5>>2]|0;
   $75 = (($74) + ($73)|0);
   $76 = $iCol;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (_sqlite3Fts3PutVarint($75,$78,$81)|0);
   $83 = HEAP32[$nOut>>2]|0;
   $84 = (($83) + ($82))|0;
   HEAP32[$nOut>>2] = $84;
   $85 = HEAP32[$nOut>>2]|0;
   $86 = (($85) + 1)|0;
   HEAP32[$nOut>>2] = $86;
   $87 = HEAP32[$5>>2]|0;
   $88 = (($87) + ($85)|0);
   HEAP8[$88>>0] = 2;
  }
  _fts3ColumnlistCopy(0,$p);
 }
 $89 = HEAP32[$bWritten>>2]|0;
 $90 = ($89|0)!=(0);
 if (!($90)) {
  $95 = HEAP32[$nOut>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $91 = HEAP32[$nOut>>2]|0;
 $92 = (($91) + 1)|0;
 HEAP32[$nOut>>2] = $92;
 $93 = HEAP32[$5>>2]|0;
 $94 = (($93) + ($91)|0);
 HEAP8[$94>>0] = 0;
 $95 = HEAP32[$nOut>>2]|0;
 STACKTOP = sp;return ($95|0);
}
function _fts3ColumnlistCopy($pp,$ppPoslist) {
 $pp = $pp|0;
 $ppPoslist = $ppPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0;
 var $n = 0, $p = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pEnd = sp + 8|0;
 $c = sp + 20|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $ppPoslist;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pEnd>>2] = $3;
 HEAP8[$c>>0] = 0;
 while(1) {
  $4 = HEAP32[$pEnd>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = HEAP8[$c>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = $6 | $8;
  $10 = 254 & $9;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pEnd>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$pEnd>>2] = $13;
  $14 = HEAP8[$12>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = $15 & 128;
  $17 = $16&255;
  HEAP8[$c>>0] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $37 = HEAP32[$pEnd>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  HEAP32[$38>>2] = $37;
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$pEnd>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = $20;
 $24 = $22;
 $25 = (($23) - ($24))|0;
 HEAP32[$n>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$p>>2] = $27;
 $28 = HEAP32[$p>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$n>>2]|0;
 _memcpy(($28|0),($30|0),($31|0))|0;
 $32 = HEAP32[$n>>2]|0;
 $33 = HEAP32[$p>>2]|0;
 $34 = (($33) + ($32)|0);
 HEAP32[$p>>2] = $34;
 $35 = HEAP32[$p>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 HEAP32[$36>>2] = $35;
 $37 = HEAP32[$pEnd>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 HEAP32[$38>>2] = $37;
 STACKTOP = sp;return;
}
function _sqlite3Fts3GetVarint($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $c = 0, $pStart = 0, $shift = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pStart = sp + 24|0;
 $a = sp + 20|0;
 $b = sp + 8|0;
 $shift = sp + 16|0;
 $c = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pStart>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 1|0);
 HEAP32[$1>>2] = $5;
 $6 = HEAP8[$4>>0]|0;
 $7 = $6 << 24 >> 24;
 HEAP32[$a>>2] = $7;
 $8 = HEAP32[$a>>2]|0;
 $9 = $8 & 128;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$a>>2]|0;
 if ($10) {
  $12 = HEAP32[$2>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $11;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = 0;
  HEAP32[$0>>2] = 1;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $17 = $11 & 127;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 1|0);
 HEAP32[$1>>2] = $19;
 $20 = HEAP8[$18>>0]|0;
 $21 = $20 << 24 >> 24;
 $22 = $21 << 7;
 $23 = $17 | $22;
 HEAP32[$a>>2] = $23;
 $24 = HEAP32[$a>>2]|0;
 $25 = $24 & 16384;
 $26 = ($25|0)==(0);
 $27 = HEAP32[$a>>2]|0;
 if ($26) {
  $28 = HEAP32[$2>>2]|0;
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $27;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = 0;
  HEAP32[$0>>2] = 2;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $33 = $27 & 16383;
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($34)) + 1|0);
 HEAP32[$1>>2] = $35;
 $36 = HEAP8[$34>>0]|0;
 $37 = $36 << 24 >> 24;
 $38 = $37 << 14;
 $39 = $33 | $38;
 HEAP32[$a>>2] = $39;
 $40 = HEAP32[$a>>2]|0;
 $41 = $40 & 2097152;
 $42 = ($41|0)==(0);
 $43 = HEAP32[$a>>2]|0;
 if ($42) {
  $44 = HEAP32[$2>>2]|0;
  $45 = $44;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = 0;
  HEAP32[$0>>2] = 3;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $49 = $43 & 2097151;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 1|0);
 HEAP32[$1>>2] = $51;
 $52 = HEAP8[$50>>0]|0;
 $53 = $52 << 24 >> 24;
 $54 = $53 << 21;
 $55 = $49 | $54;
 HEAP32[$a>>2] = $55;
 $56 = HEAP32[$a>>2]|0;
 $57 = $56 & 268435456;
 $58 = ($57|0)==(0);
 $59 = HEAP32[$a>>2]|0;
 if ($58) {
  $60 = HEAP32[$2>>2]|0;
  $61 = $60;
  $62 = $61;
  HEAP32[$62>>2] = $59;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = 0;
  HEAP32[$0>>2] = 4;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $65 = $59 & 268435455;
 $66 = $b;
 $67 = $66;
 HEAP32[$67>>2] = $65;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = 0;
 HEAP32[$shift>>2] = 28;
 while(1) {
  $70 = HEAP32[$shift>>2]|0;
  $71 = ($70|0)<=(63);
  if (!($71)) {
   break;
  }
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 1|0);
  HEAP32[$1>>2] = $73;
  $74 = HEAP8[$72>>0]|0;
  $75 = $74 << 24 >> 24;
  $76 = ($75|0)<(0);
  $77 = $76 << 31 >> 31;
  $78 = $c;
  $79 = $78;
  HEAP32[$79>>2] = $75;
  $80 = (($78) + 4)|0;
  $81 = $80;
  HEAP32[$81>>2] = $77;
  $82 = $c;
  $83 = $82;
  $84 = HEAP32[$83>>2]|0;
  $85 = (($82) + 4)|0;
  $86 = $85;
  $87 = HEAP32[$86>>2]|0;
  $88 = $84 & 127;
  $89 = HEAP32[$shift>>2]|0;
  $90 = (_bitshift64Shl(($88|0),0,($89|0))|0);
  $91 = tempRet0;
  $92 = $b;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = (_i64Add(($94|0),($97|0),($90|0),($91|0))|0);
  $99 = tempRet0;
  $100 = $b;
  $101 = $100;
  HEAP32[$101>>2] = $98;
  $102 = (($100) + 4)|0;
  $103 = $102;
  HEAP32[$103>>2] = $99;
  $104 = $c;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = $106 & 128;
  $111 = ($110|0)==(0);
  $112 = (0)==(0);
  $113 = $111 & $112;
  if ($113) {
   break;
  }
  $114 = HEAP32[$shift>>2]|0;
  $115 = (($114) + 7)|0;
  HEAP32[$shift>>2] = $115;
 }
 $116 = $b;
 $117 = $116;
 $118 = HEAP32[$117>>2]|0;
 $119 = (($116) + 4)|0;
 $120 = $119;
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$2>>2]|0;
 $123 = $122;
 $124 = $123;
 HEAP32[$124>>2] = $118;
 $125 = (($123) + 4)|0;
 $126 = $125;
 HEAP32[$126>>2] = $121;
 $127 = HEAP32[$1>>2]|0;
 $128 = HEAP32[$pStart>>2]|0;
 $129 = $127;
 $130 = $128;
 $131 = (($129) - ($130))|0;
 HEAP32[$0>>2] = $131;
 $132 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _sqlite3Fts3GetVarint32($p,$pi) {
 $p = $p|0;
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $a = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pi;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 1|0);
 HEAP32[$1>>2] = $4;
 $5 = HEAP8[$3>>0]|0;
 $6 = $5 << 24 >> 24;
 HEAP32[$a>>2] = $6;
 $7 = HEAP32[$a>>2]|0;
 $8 = $7 & 127;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 1|0);
 HEAP32[$1>>2] = $10;
 $11 = HEAP8[$9>>0]|0;
 $12 = $11 << 24 >> 24;
 $13 = $12 << 7;
 $14 = $8 | $13;
 HEAP32[$a>>2] = $14;
 $15 = HEAP32[$a>>2]|0;
 $16 = $15 & 16384;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$a>>2]|0;
 if ($17) {
  $19 = HEAP32[$2>>2]|0;
  HEAP32[$19>>2] = $18;
  HEAP32[$0>>2] = 2;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $20 = $18 & 16383;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 1|0);
 HEAP32[$1>>2] = $22;
 $23 = HEAP8[$21>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = $24 << 14;
 $26 = $20 | $25;
 HEAP32[$a>>2] = $26;
 $27 = HEAP32[$a>>2]|0;
 $28 = $27 & 2097152;
 $29 = ($28|0)==(0);
 $30 = HEAP32[$a>>2]|0;
 if ($29) {
  $31 = HEAP32[$2>>2]|0;
  HEAP32[$31>>2] = $30;
  HEAP32[$0>>2] = 3;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $32 = $30 & 2097151;
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 1|0);
 HEAP32[$1>>2] = $34;
 $35 = HEAP8[$33>>0]|0;
 $36 = $35 << 24 >> 24;
 $37 = $36 << 21;
 $38 = $32 | $37;
 HEAP32[$a>>2] = $38;
 $39 = HEAP32[$a>>2]|0;
 $40 = $39 & 268435456;
 $41 = ($40|0)==(0);
 $42 = HEAP32[$a>>2]|0;
 if ($41) {
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  HEAP32[$0>>2] = 4;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 } else {
  $44 = $42 & 268435455;
  HEAP32[$a>>2] = $44;
  $45 = HEAP32[$a>>2]|0;
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP8[$46>>0]|0;
  $48 = $47 << 24 >> 24;
  $49 = $48 & 15;
  $50 = $49 << 28;
  $51 = $45 | $50;
  $52 = HEAP32[$2>>2]|0;
  HEAP32[$52>>2] = $51;
  HEAP32[$0>>2] = 5;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 return (0)|0;
}
function _sqlite3Fts3DoclistPrev($bDescIdx,$aDoclist,$nDoclist,$ppIter,$piDocid,$pnList,$pbEof) {
 $bDescIdx = $bDescIdx|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 $ppIter = $ppIter|0;
 $piDocid = $piDocid|0;
 $pnList = $pnList|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iDelta = 0, $iDelta2 = 0, $iDocid = 0, $iMul = 0, $iMul1 = 0, $p = 0, $pDocid = 0, $pEnd = 0, $pNext = 0, $pSave = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $p = sp + 48|0;
 $iDocid = sp + 16|0;
 $pNext = sp + 44|0;
 $pDocid = sp + 40|0;
 $pEnd = sp + 36|0;
 $iMul = sp + 32|0;
 $iDelta = sp + 8|0;
 $iMul1 = sp + 28|0;
 $iDelta2 = sp;
 $pSave = sp + 24|0;
 HEAP32[$0>>2] = $bDescIdx;
 HEAP32[$1>>2] = $aDoclist;
 HEAP32[$2>>2] = $nDoclist;
 HEAP32[$3>>2] = $ppIter;
 HEAP32[$4>>2] = $piDocid;
 HEAP32[$5>>2] = $pnList;
 HEAP32[$6>>2] = $pbEof;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = ($9|0)==(0|0);
 if (!($10)) {
  $81 = HEAP32[$0>>2]|0;
  $82 = ($81|0)!=(0);
  $83 = $82 ? -1 : 1;
  HEAP32[$iMul1>>2] = $83;
  $84 = HEAP32[$1>>2]|0;
  _fts3GetReverseVarint($p,$84,$iDelta2);
  $85 = HEAP32[$iMul1>>2]|0;
  $86 = ($85|0)<(0);
  $87 = $86 << 31 >> 31;
  $88 = $iDelta2;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (___muldi3(($85|0),($87|0),($90|0),($93|0))|0);
  $95 = tempRet0;
  $96 = HEAP32[$4>>2]|0;
  $97 = $96;
  $98 = $97;
  $99 = HEAP32[$98>>2]|0;
  $100 = (($97) + 4)|0;
  $101 = $100;
  $102 = HEAP32[$101>>2]|0;
  $103 = (_i64Subtract(($99|0),($102|0),($94|0),($95|0))|0);
  $104 = tempRet0;
  $105 = $96;
  $106 = $105;
  HEAP32[$106>>2] = $103;
  $107 = (($105) + 4)|0;
  $108 = $107;
  HEAP32[$108>>2] = $104;
  $109 = HEAP32[$p>>2]|0;
  $110 = HEAP32[$1>>2]|0;
  $111 = ($109|0)==($110|0);
  if ($111) {
   $112 = HEAP32[$6>>2]|0;
   HEAP8[$112>>0] = 1;
  } else {
   $113 = HEAP32[$p>>2]|0;
   HEAP32[$pSave>>2] = $113;
   $114 = HEAP32[$1>>2]|0;
   _fts3ReversePoslist($114,$p);
   $115 = HEAP32[$pSave>>2]|0;
   $116 = HEAP32[$p>>2]|0;
   $117 = $115;
   $118 = $116;
   $119 = (($117) - ($118))|0;
   $120 = HEAP32[$5>>2]|0;
   HEAP32[$120>>2] = $119;
  }
  $121 = HEAP32[$p>>2]|0;
  $122 = HEAP32[$3>>2]|0;
  HEAP32[$122>>2] = $121;
  STACKTOP = sp;return;
 }
 $11 = $iDocid;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 HEAP32[$pNext>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 HEAP32[$pDocid>>2] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + ($16)|0);
 HEAP32[$pEnd>>2] = $18;
 HEAP32[$iMul>>2] = 1;
 while(1) {
  $19 = HEAP32[$pDocid>>2]|0;
  $20 = HEAP32[$pEnd>>2]|0;
  $21 = ($19>>>0)<($20>>>0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$pDocid>>2]|0;
  $23 = (_sqlite3Fts3GetVarint($22,$iDelta)|0);
  $24 = HEAP32[$pDocid>>2]|0;
  $25 = (($24) + ($23)|0);
  HEAP32[$pDocid>>2] = $25;
  $26 = HEAP32[$iMul>>2]|0;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  $29 = $iDelta;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (___muldi3(($26|0),($28|0),($31|0),($34|0))|0);
  $36 = tempRet0;
  $37 = $iDocid;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Add(($39|0),($42|0),($35|0),($36|0))|0);
  $44 = tempRet0;
  $45 = $iDocid;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  $49 = HEAP32[$pDocid>>2]|0;
  HEAP32[$pNext>>2] = $49;
  _fts3PoslistCopy(0,$pDocid);
  while(1) {
   $50 = HEAP32[$pDocid>>2]|0;
   $51 = HEAP32[$pEnd>>2]|0;
   $52 = ($50>>>0)<($51>>>0);
   if (!($52)) {
    break;
   }
   $53 = HEAP32[$pDocid>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54 << 24 >> 24;
   $56 = ($55|0)==(0);
   if (!($56)) {
    break;
   }
   $57 = HEAP32[$pDocid>>2]|0;
   $58 = ((($57)) + 1|0);
   HEAP32[$pDocid>>2] = $58;
  }
  $59 = HEAP32[$0>>2]|0;
  $60 = ($59|0)!=(0);
  $61 = $60 ? -1 : 1;
  HEAP32[$iMul>>2] = $61;
 }
 $62 = HEAP32[$pEnd>>2]|0;
 $63 = HEAP32[$pNext>>2]|0;
 $64 = $62;
 $65 = $63;
 $66 = (($64) - ($65))|0;
 $67 = HEAP32[$5>>2]|0;
 HEAP32[$67>>2] = $66;
 $68 = HEAP32[$pNext>>2]|0;
 $69 = HEAP32[$3>>2]|0;
 HEAP32[$69>>2] = $68;
 $70 = $iDocid;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = (($70) + 4)|0;
 $74 = $73;
 $75 = HEAP32[$74>>2]|0;
 $76 = HEAP32[$4>>2]|0;
 $77 = $76;
 $78 = $77;
 HEAP32[$78>>2] = $72;
 $79 = (($77) + 4)|0;
 $80 = $79;
 HEAP32[$80>>2] = $75;
 STACKTOP = sp;return;
}
function _fts3SegReaderIncrRead($pReader) {
 $pReader = $pReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $nRead = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $nRead = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pReader;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 48|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($3) - ($6))|0;
 $8 = ($7|0)<(4096);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 48|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11) - ($14))|0;
  $16 = $15;
 } else {
  $16 = 4096;
 }
 HEAP32[$nRead>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 40|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22)|0);
 $27 = HEAP32[$nRead>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 48|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3_blob_read($19,$26,$27,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)==(0);
 if (!($33)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $34 = HEAP32[$nRead>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 48|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + ($34))|0;
 HEAP32[$36>>2] = $38;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 48|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 40|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41)|0);
 dest=$45; stop=dest+20|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 48|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 44|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($48|0)==($51|0);
 if (!($52)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 52|0);
 $55 = HEAP32[$54>>2]|0;
 (_sqlite3_blob_close($55)|0);
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 52|0);
 HEAP32[$57>>2] = 0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 48|0);
 HEAP32[$59>>2] = 0;
 $60 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _fts3SegReaderRequire($pReader,$pFrom,$nByte) {
 $pReader = $pReader|0;
 $pFrom = $pFrom|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pReader;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 52|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)!=(0|0);
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  $or$cond = $6 & $8;
  if (!($or$cond)) {
   label = 5;
   break;
  }
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 40|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $9;
  $14 = $12;
  $15 = (($13) - ($14))|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($15) + ($16))|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 48|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)>($20|0);
  if (!($21)) {
   label = 5;
   break;
  }
  $22 = HEAP32[$0>>2]|0;
  $23 = (_fts3SegReaderIncrRead($22)|0);
  HEAP32[$rc>>2] = $23;
 }
 if ((label|0) == 5) {
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _fts3PoslistCopy($pp,$ppPoslist) {
 $pp = $pp|0;
 $ppPoslist = $ppPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $n = 0;
 var $p = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pEnd = sp + 8|0;
 $c = sp + 20|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $ppPoslist;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pEnd>>2] = $3;
 HEAP8[$c>>0] = 0;
 while(1) {
  $4 = HEAP32[$pEnd>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = HEAP8[$c>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = $6 | $8;
  $10 = ($9|0)!=(0);
  $11 = HEAP32[$pEnd>>2]|0;
  $12 = ((($11)) + 1|0);
  HEAP32[$pEnd>>2] = $12;
  if (!($10)) {
   break;
  }
  $13 = HEAP8[$11>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = $14 & 128;
  $16 = $15&255;
  HEAP8[$c>>0] = $16;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  $36 = HEAP32[$pEnd>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  HEAP32[$37>>2] = $36;
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$pEnd>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = $19;
 $23 = $21;
 $24 = (($22) - ($23))|0;
 HEAP32[$n>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$p>>2] = $26;
 $27 = HEAP32[$p>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$n>>2]|0;
 _memcpy(($27|0),($29|0),($30|0))|0;
 $31 = HEAP32[$n>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = (($32) + ($31)|0);
 HEAP32[$p>>2] = $33;
 $34 = HEAP32[$p>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 HEAP32[$35>>2] = $34;
 $36 = HEAP32[$pEnd>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 HEAP32[$37>>2] = $36;
 STACKTOP = sp;return;
}
function _fts3GetReverseVarint($pp,$pStart,$pVal) {
 $pp = $pp|0;
 $pStart = $pStart|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iVal = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $iVal = sp;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pStart;
 HEAP32[$2>>2] = $pVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + -2|0);
 HEAP32[$p>>2] = $5;
 while(1) {
  $6 = HEAP32[$p>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ($6>>>0)>=($7>>>0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = $11 & 128;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + -1|0);
  HEAP32[$p>>2] = $15;
 }
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 1|0);
 HEAP32[$p>>2] = $17;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$p>>2]|0;
 (_sqlite3Fts3GetVarint($20,$iVal)|0);
 $21 = $iVal;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = $27;
 $29 = $28;
 HEAP32[$29>>2] = $23;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $26;
 STACKTOP = sp;return;
}
function _fts3ReversePoslist($pStart,$ppPoslist) {
 $pStart = $pStart|0;
 $ppPoslist = $ppPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 $c = sp + 12|0;
 HEAP32[$0>>2] = $pStart;
 HEAP32[$1>>2] = $ppPoslist;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + -2|0);
 HEAP32[$p>>2] = $4;
 HEAP8[$c>>0] = 0;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ($5>>>0)>($6>>>0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$p>>2]|0;
  $9 = ((($8)) + -1|0);
  HEAP32[$p>>2] = $9;
  $10 = HEAP8[$8>>0]|0;
  HEAP8[$c>>0] = $10;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(0);
  if (!($12)) {
   break;
  }
 }
 while(1) {
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ($13>>>0)>($14>>>0);
  if ($15) {
   $16 = HEAP32[$p>>2]|0;
   $17 = HEAP8[$16>>0]|0;
   $18 = $17 << 24 >> 24;
   $19 = $18 & 128;
   $20 = HEAP8[$c>>0]|0;
   $21 = $20 << 24 >> 24;
   $22 = $19 | $21;
   $23 = ($22|0)!=(0);
   $47 = $23;
  } else {
   $47 = 0;
  }
  $24 = HEAP32[$p>>2]|0;
  if (!($47)) {
   break;
  }
  $25 = ((($24)) + -1|0);
  HEAP32[$p>>2] = $25;
  $26 = HEAP8[$24>>0]|0;
  HEAP8[$c>>0] = $26;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ($24>>>0)>($27>>>0);
 if ($28) {
  label = 11;
 } else {
  $29 = HEAP8[$c>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = ($30|0)==(0);
  if ($31) {
   $32 = HEAP32[$1>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = ((($34)) + 2|0);
   $36 = ($33>>>0)>($35>>>0);
   if ($36) {
    label = 11;
   }
  }
 }
 if ((label|0) == 11) {
  $37 = HEAP32[$p>>2]|0;
  $38 = ((($37)) + 2|0);
  HEAP32[$p>>2] = $38;
 }
 while(1) {
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 1|0);
  HEAP32[$p>>2] = $40;
  $41 = HEAP8[$39>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = $42 & 128;
  $44 = ($43|0)!=(0);
  if (!($44)) {
   break;
  }
 }
 $45 = HEAP32[$p>>2]|0;
 $46 = HEAP32[$1>>2]|0;
 HEAP32[$46>>2] = $45;
 STACKTOP = sp;return;
}
function _sqlite3Fts3ReadBlock($p,$0,$1,$paBlob,$pnBlob,$pnLoad) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $paBlob = $paBlob|0;
 $pnBlob = $pnBlob|0;
 $pnLoad = $pnLoad|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aByte = 0, $nByte = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $rc = sp + 20|0;
 $nByte = sp + 16|0;
 $aByte = sp + 12|0;
 HEAP32[$3>>2] = $p;
 $8 = $4;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$5>>2] = $paBlob;
 HEAP32[$6>>2] = $pnBlob;
 HEAP32[$7>>2] = $pnLoad;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 244|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0|0);
 $16 = HEAP32[$3>>2]|0;
 if ($15) {
  $17 = ((($16)) + 244|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3_blob_reopen($18,$21,$24)|0);
  HEAP32[$rc>>2] = $25;
 } else {
  $26 = ((($16)) + 240|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (0|0)==($27|0);
  if ($28) {
   $29 = HEAP32[$3>>2]|0;
   $30 = ((($29)) + 20|0);
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$vararg_buffer>>2] = $31;
   $32 = (_sqlite3_mprintf(23163,$vararg_buffer)|0);
   $33 = HEAP32[$3>>2]|0;
   $34 = ((($33)) + 240|0);
   HEAP32[$34>>2] = $32;
   $35 = HEAP32[$3>>2]|0;
   $36 = ((($35)) + 240|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (0|0)==($37|0);
   if ($38) {
    HEAP32[$2>>2] = 7;
    $93 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($93|0);
   }
  }
  $39 = HEAP32[$3>>2]|0;
  $40 = ((($39)) + 12|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$3>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$3>>2]|0;
  $46 = ((($45)) + 240|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = $4;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($48) + 4)|0;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$3>>2]|0;
  $55 = ((($54)) + 244|0);
  $56 = (_sqlite3_blob_open($41,$44,$47,23175,$50,$53,0,$55)|0);
  HEAP32[$rc>>2] = $56;
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if ($58) {
  $59 = HEAP32[$3>>2]|0;
  $60 = ((($59)) + 244|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (_sqlite3_blob_bytes($61)|0);
  HEAP32[$nByte>>2] = $62;
  $63 = HEAP32[$nByte>>2]|0;
  $64 = HEAP32[$6>>2]|0;
  HEAP32[$64>>2] = $63;
  $65 = HEAP32[$5>>2]|0;
  $66 = ($65|0)!=(0|0);
  if ($66) {
   $67 = HEAP32[$nByte>>2]|0;
   $68 = (($67) + 20)|0;
   $69 = (_sqlite3_malloc($68)|0);
   HEAP32[$aByte>>2] = $69;
   $70 = HEAP32[$aByte>>2]|0;
   $71 = ($70|0)!=(0|0);
   if ($71) {
    $72 = HEAP32[$7>>2]|0;
    $73 = ($72|0)!=(0|0);
    $74 = HEAP32[$nByte>>2]|0;
    $75 = ($74|0)>(16384);
    $or$cond = $73 & $75;
    if ($or$cond) {
     HEAP32[$nByte>>2] = 4096;
     $76 = HEAP32[$nByte>>2]|0;
     $77 = HEAP32[$7>>2]|0;
     HEAP32[$77>>2] = $76;
    }
    $78 = HEAP32[$3>>2]|0;
    $79 = ((($78)) + 244|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = HEAP32[$aByte>>2]|0;
    $82 = HEAP32[$nByte>>2]|0;
    $83 = (_sqlite3_blob_read($80,$81,$82,0)|0);
    HEAP32[$rc>>2] = $83;
    $84 = HEAP32[$nByte>>2]|0;
    $85 = HEAP32[$aByte>>2]|0;
    $86 = (($85) + ($84)|0);
    dest=$86; stop=dest+20|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
    $87 = HEAP32[$rc>>2]|0;
    $88 = ($87|0)!=(0);
    if ($88) {
     $89 = HEAP32[$aByte>>2]|0;
     _sqlite3_free($89);
     HEAP32[$aByte>>2] = 0;
    }
   } else {
    HEAP32[$rc>>2] = 7;
   }
   $90 = HEAP32[$aByte>>2]|0;
   $91 = HEAP32[$5>>2]|0;
   HEAP32[$91>>2] = $90;
  }
 }
 $92 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $92;
 $93 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _fts3SegReaderStart($p,$pCsr,$zTerm,$nTerm) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nSeg = 0, $or$cond = 0, $pSeg = 0, $rc = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $i = sp + 16|0;
 $nSeg = sp + 12|0;
 $res = sp + 8|0;
 $pSeg = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$3>>2] = $zTerm;
 HEAP32[$4>>2] = $nTerm;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nSeg>>2] = $7;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $8 = HEAP32[$2>>2]|0;
  $9 = ((($8)) + 28|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10|0)==(0);
  if (!($11)) {
   label = 12;
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12|0)<($15|0);
  if (!($16)) {
   label = 12;
   break;
  }
  HEAP32[$res>>2] = 0;
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($17<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pSeg>>2] = $21;
  while(1) {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$pSeg>>2]|0;
   $24 = (_fts3SegReaderNext($22,$23,0)|0);
   HEAP32[$rc>>2] = $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)!=(0);
   if ($26) {
    label = 6;
    break L1;
   }
   $28 = HEAP32[$3>>2]|0;
   $29 = ($28|0)!=(0|0);
   if (!($29)) {
    break;
   }
   $30 = HEAP32[$pSeg>>2]|0;
   $31 = HEAP32[$3>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = (_fts3SegReaderTermCmp($30,$31,$32)|0);
   HEAP32[$res>>2] = $33;
   $34 = ($33|0)<(0);
   if (!($34)) {
    break;
   }
  }
  $35 = HEAP32[$pSeg>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = $37&255;
  $39 = ($38|0)!=(0);
  $40 = HEAP32[$res>>2]|0;
  $41 = ($40|0)!=(0);
  $or$cond = $39 & $41;
  if ($or$cond) {
   $42 = HEAP32[$pSeg>>2]|0;
   _fts3SegReaderSetEof($42);
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 if ((label|0) == 6) {
  $27 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $27;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 else if ((label|0) == 12) {
  $45 = HEAP32[$2>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$nSeg>>2]|0;
  $48 = HEAP32[$nSeg>>2]|0;
  _fts3SegReaderSort($46,$47,$48,184);
  HEAP32[$0>>2] = 0;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 return (0)|0;
}
function _fts3SegReaderTermCmp($pSeg,$zTerm,$nTerm) {
 $pSeg = $pSeg|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pSeg;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$res>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $29 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 60|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = ($9|0)>($10|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 64|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 if ($11) {
  $16 = HEAP32[$2>>2]|0;
  $17 = (_memcmp($14,$15,$16)|0);
  HEAP32[$res>>2] = $17;
 } else {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 60|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_memcmp($14,$15,$20)|0);
  HEAP32[$res>>2] = $21;
 }
 $22 = HEAP32[$res>>2]|0;
 $23 = ($22|0)==(0);
 if (!($23)) {
  $29 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 60|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (($26) - ($27))|0;
 HEAP32[$res>>2] = $28;
 $29 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _fts3SegReaderCursor($p,$iLangid,$iIndex,$iLevel,$zTerm,$nTerm,$isPrefix,$isScan,$pCsr) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $isPrefix = $isPrefix|0;
 $isScan = $isScan|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iEndBlock = 0, $iLeavesEndBlock = 0, $iLeavesEndBlock$ = 0, $iStartBlock = 0, $nRoot = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pSeg = 0, $pSeg1 = 0;
 var $pStmt = 0, $pi = 0, $rc = 0, $rc2 = 0, $zRoot = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $8 = sp + 56|0;
 $rc = sp + 52|0;
 $pStmt = sp + 48|0;
 $rc2 = sp + 44|0;
 $pSeg = sp + 40|0;
 $pSeg1 = sp + 36|0;
 $iStartBlock = sp + 16|0;
 $iLeavesEndBlock = sp + 8|0;
 $iEndBlock = sp;
 $nRoot = sp + 32|0;
 $zRoot = sp + 28|0;
 $pi = sp + 24|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $isPrefix;
 HEAP32[$7>>2] = $isScan;
 HEAP32[$8>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)<(0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 252|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   HEAP32[$pSeg>>2] = 0;
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$4>>2]|0;
   $18 = HEAP32[$5>>2]|0;
   $19 = HEAP32[$6>>2]|0;
   $20 = ($19|0)!=(0);
   $21 = HEAP32[$7>>2]|0;
   $22 = ($21|0)!=(0);
   $23 = $20 ? 1 : $22;
   $24 = $23&1;
   $25 = (_sqlite3Fts3SegReaderPending($15,$16,$17,$18,$24,$pSeg)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)==(0);
   $28 = HEAP32[$pSeg>>2]|0;
   $29 = ($28|0)!=(0|0);
   $or$cond = $27 & $29;
   if ($or$cond) {
    $30 = HEAP32[$8>>2]|0;
    $31 = HEAP32[$pSeg>>2]|0;
    $32 = (_fts3SegReaderCursorAppend($30,$31)|0);
    HEAP32[$rc>>2] = $32;
   }
  }
 }
 $33 = HEAP32[$3>>2]|0;
 $34 = ($33|0)!=(-1);
 L6: do {
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[$0>>2]|0;
    $38 = HEAP32[$1>>2]|0;
    $39 = HEAP32[$2>>2]|0;
    $40 = HEAP32[$3>>2]|0;
    $41 = (_sqlite3Fts3AllSegdirs($37,$38,$39,$40,$pStmt)|0);
    HEAP32[$rc>>2] = $41;
   }
   while(1) {
    $42 = HEAP32[$rc>>2]|0;
    $43 = ($42|0)==(0);
    if (!($43)) {
     break L6;
    }
    $44 = HEAP32[$pStmt>>2]|0;
    $45 = (_sqlite3_step($44)|0);
    HEAP32[$rc>>2] = $45;
    $46 = (100)==($45|0);
    if (!($46)) {
     break L6;
    }
    HEAP32[$pSeg1>>2] = 0;
    $47 = HEAP32[$pStmt>>2]|0;
    $48 = (_sqlite3_column_int64($47,1)|0);
    $49 = tempRet0;
    $50 = $iStartBlock;
    $51 = $50;
    HEAP32[$51>>2] = $48;
    $52 = (($50) + 4)|0;
    $53 = $52;
    HEAP32[$53>>2] = $49;
    $54 = HEAP32[$pStmt>>2]|0;
    $55 = (_sqlite3_column_int64($54,2)|0);
    $56 = tempRet0;
    $57 = $iLeavesEndBlock;
    $58 = $57;
    HEAP32[$58>>2] = $55;
    $59 = (($57) + 4)|0;
    $60 = $59;
    HEAP32[$60>>2] = $56;
    $61 = HEAP32[$pStmt>>2]|0;
    $62 = (_sqlite3_column_int64($61,3)|0);
    $63 = tempRet0;
    $64 = $iEndBlock;
    $65 = $64;
    HEAP32[$65>>2] = $62;
    $66 = (($64) + 4)|0;
    $67 = $66;
    HEAP32[$67>>2] = $63;
    $68 = HEAP32[$pStmt>>2]|0;
    $69 = (_sqlite3_column_bytes($68,4)|0);
    HEAP32[$nRoot>>2] = $69;
    $70 = HEAP32[$pStmt>>2]|0;
    $71 = (_sqlite3_column_blob($70,4)|0);
    HEAP32[$zRoot>>2] = $71;
    $72 = $iStartBlock;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = ($74|0)!=(0);
    $79 = ($77|0)!=(0);
    $80 = $78 | $79;
    $81 = HEAP32[$4>>2]|0;
    $82 = ($81|0)!=(0|0);
    $or$cond3 = $80 & $82;
    if ($or$cond3) {
     $83 = HEAP32[$6>>2]|0;
     $84 = ($83|0)!=(0);
     $iLeavesEndBlock$ = $84 ? $iLeavesEndBlock : 0;
     HEAP32[$pi>>2] = $iLeavesEndBlock$;
     $85 = HEAP32[$0>>2]|0;
     $86 = HEAP32[$4>>2]|0;
     $87 = HEAP32[$5>>2]|0;
     $88 = HEAP32[$zRoot>>2]|0;
     $89 = HEAP32[$nRoot>>2]|0;
     $90 = HEAP32[$pi>>2]|0;
     $91 = (_fts3SelectLeaf($85,$86,$87,$88,$89,$iStartBlock,$90)|0);
     HEAP32[$rc>>2] = $91;
     $92 = HEAP32[$rc>>2]|0;
     $93 = ($92|0)!=(0);
     if ($93) {
      break L6;
     }
     $94 = HEAP32[$6>>2]|0;
     $95 = ($94|0)==(0);
     $96 = HEAP32[$7>>2]|0;
     $97 = ($96|0)==(0);
     $or$cond5 = $95 & $97;
     if ($or$cond5) {
      $98 = $iStartBlock;
      $99 = $98;
      $100 = HEAP32[$99>>2]|0;
      $101 = (($98) + 4)|0;
      $102 = $101;
      $103 = HEAP32[$102>>2]|0;
      $104 = $iLeavesEndBlock;
      $105 = $104;
      HEAP32[$105>>2] = $100;
      $106 = (($104) + 4)|0;
      $107 = $106;
      HEAP32[$107>>2] = $103;
     }
    }
    $108 = HEAP32[$8>>2]|0;
    $109 = ((($108)) + 4|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = (($110) + 1)|0;
    $112 = HEAP32[$6>>2]|0;
    $113 = ($112|0)==(0);
    $114 = HEAP32[$7>>2]|0;
    $115 = ($114|0)==(0);
    $116 = $113 ? $115 : 0;
    $117 = $116&1;
    $118 = $iStartBlock;
    $119 = $118;
    $120 = HEAP32[$119>>2]|0;
    $121 = (($118) + 4)|0;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = $iLeavesEndBlock;
    $125 = $124;
    $126 = HEAP32[$125>>2]|0;
    $127 = (($124) + 4)|0;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = $iEndBlock;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = (($130) + 4)|0;
    $134 = $133;
    $135 = HEAP32[$134>>2]|0;
    $136 = HEAP32[$zRoot>>2]|0;
    $137 = HEAP32[$nRoot>>2]|0;
    $138 = (_sqlite3Fts3SegReaderNew($111,$117,$120,$123,$126,$129,$132,$135,$136,$137,$pSeg1)|0);
    HEAP32[$rc>>2] = $138;
    $139 = HEAP32[$rc>>2]|0;
    $140 = ($139|0)!=(0);
    if ($140) {
     break L6;
    }
    $141 = HEAP32[$8>>2]|0;
    $142 = HEAP32[$pSeg1>>2]|0;
    $143 = (_fts3SegReaderCursorAppend($141,$142)|0);
    HEAP32[$rc>>2] = $143;
   }
  }
 } while(0);
 $144 = HEAP32[$pStmt>>2]|0;
 $145 = (_sqlite3_reset($144)|0);
 HEAP32[$rc2>>2] = $145;
 $146 = HEAP32[$rc>>2]|0;
 $147 = ($146|0)==(101);
 if (!($147)) {
  $149 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($149|0);
 }
 $148 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $148;
 $149 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($149|0);
}
function _sqlite3Fts3SegReaderPending($p,$iIndex,$zTerm,$nTerm,$bPrefix,$ppReader) {
 $p = $p|0;
 $iIndex = $iIndex|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $bPrefix = $bPrefix|0;
 $ppReader = $ppReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $aElem = 0, $aElem2 = 0, $nAlloc = 0, $nByte = 0, $nElem = 0, $nKey = 0, $pE = 0;
 var $pHash = 0, $pReader = 0, $rc = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $pReader = sp + 40|0;
 $pE = sp + 36|0;
 $aElem = sp + 32|0;
 $nElem = sp + 28|0;
 $rc = sp + 24|0;
 $pHash = sp + 20|0;
 $nAlloc = sp + 16|0;
 $zKey = sp + 12|0;
 $nKey = sp + 8|0;
 $aElem2 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iIndex;
 HEAP32[$2>>2] = $zTerm;
 HEAP32[$3>>2] = $nTerm;
 HEAP32[$4>>2] = $bPrefix;
 HEAP32[$5>>2] = $ppReader;
 HEAP32[$pReader>>2] = 0;
 HEAP32[$aElem>>2] = 0;
 HEAP32[$nElem>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 252|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($6*24)|0)|0);
 $11 = ((($10)) + 4|0);
 HEAP32[$pHash>>2] = $11;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  HEAP32[$nAlloc>>2] = 0;
  $14 = HEAP32[$pHash>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pE>>2] = $16;
  while(1) {
   $17 = HEAP32[$pE>>2]|0;
   $18 = ($17|0)!=(0|0);
   if (!($18)) {
    break;
   }
   $19 = HEAP32[$pE>>2]|0;
   $20 = ((($19)) + 12|0);
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$zKey>>2] = $21;
   $22 = HEAP32[$pE>>2]|0;
   $23 = ((($22)) + 16|0);
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$nKey>>2] = $24;
   $25 = HEAP32[$3>>2]|0;
   $26 = ($25|0)==(0);
   if ($26) {
    label = 7;
   } else {
    $27 = HEAP32[$nKey>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = ($27|0)>=($28|0);
    if ($29) {
     $30 = HEAP32[$zKey>>2]|0;
     $31 = HEAP32[$2>>2]|0;
     $32 = HEAP32[$3>>2]|0;
     $33 = (_memcmp($30,$31,$32)|0);
     $34 = (0)==($33|0);
     if ($34) {
      label = 7;
     }
    }
   }
   if ((label|0) == 7) {
    label = 0;
    $35 = HEAP32[$nElem>>2]|0;
    $36 = HEAP32[$nAlloc>>2]|0;
    $37 = ($35|0)==($36|0);
    if ($37) {
     $38 = HEAP32[$nAlloc>>2]|0;
     $39 = (($38) + 16)|0;
     HEAP32[$nAlloc>>2] = $39;
     $40 = HEAP32[$aElem>>2]|0;
     $41 = HEAP32[$nAlloc>>2]|0;
     $42 = $41<<2;
     $43 = (_sqlite3_realloc($40,$42)|0);
     HEAP32[$aElem2>>2] = $43;
     $44 = HEAP32[$aElem2>>2]|0;
     $45 = ($44|0)!=(0|0);
     if (!($45)) {
      label = 9;
      break;
     }
     $46 = HEAP32[$aElem2>>2]|0;
     HEAP32[$aElem>>2] = $46;
    }
    $47 = HEAP32[$pE>>2]|0;
    $48 = HEAP32[$nElem>>2]|0;
    $49 = (($48) + 1)|0;
    HEAP32[$nElem>>2] = $49;
    $50 = HEAP32[$aElem>>2]|0;
    $51 = (($50) + ($48<<2)|0);
    HEAP32[$51>>2] = $47;
   }
   $52 = HEAP32[$pE>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$pE>>2] = $53;
  }
  if ((label|0) == 9) {
   HEAP32[$rc>>2] = 7;
   HEAP32[$nElem>>2] = 0;
  }
  $54 = HEAP32[$nElem>>2]|0;
  $55 = ($54|0)>(1);
  if ($55) {
   $56 = HEAP32[$aElem>>2]|0;
   $57 = HEAP32[$nElem>>2]|0;
   _qsort($56,$57,4,185);
  }
 } else {
  $58 = HEAP32[$pHash>>2]|0;
  $59 = HEAP32[$2>>2]|0;
  $60 = HEAP32[$3>>2]|0;
  $61 = (_sqlite3Fts3HashFindElem($58,$59,$60)|0);
  HEAP32[$pE>>2] = $61;
  $62 = HEAP32[$pE>>2]|0;
  $63 = ($62|0)!=(0|0);
  if ($63) {
   HEAP32[$aElem>>2] = $pE;
   HEAP32[$nElem>>2] = 1;
  }
 }
 $64 = HEAP32[$nElem>>2]|0;
 $65 = ($64|0)>(0);
 do {
  if ($65) {
   $66 = HEAP32[$nElem>>2]|0;
   $67 = (($66) + 1)|0;
   $68 = $67<<2;
   $69 = (96 + ($68))|0;
   HEAP32[$nByte>>2] = $69;
   $70 = HEAP32[$nByte>>2]|0;
   $71 = (_sqlite3_malloc($70)|0);
   HEAP32[$pReader>>2] = $71;
   $72 = HEAP32[$pReader>>2]|0;
   $73 = ($72|0)!=(0|0);
   if ($73) {
    $74 = HEAP32[$pReader>>2]|0;
    $75 = HEAP32[$nByte>>2]|0;
    _memset(($74|0),0,($75|0))|0;
    $76 = HEAP32[$pReader>>2]|0;
    HEAP32[$76>>2] = 2147483647;
    $77 = HEAP32[$pReader>>2]|0;
    $78 = ((($77)) + 96|0);
    $79 = HEAP32[$pReader>>2]|0;
    $80 = ((($79)) + 56|0);
    HEAP32[$80>>2] = $78;
    $81 = HEAP32[$pReader>>2]|0;
    $82 = ((($81)) + 56|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = HEAP32[$aElem>>2]|0;
    $85 = HEAP32[$nElem>>2]|0;
    $86 = $85<<2;
    _memcpy(($83|0),($84|0),($86|0))|0;
    break;
   } else {
    HEAP32[$rc>>2] = 7;
    break;
   }
  }
 } while(0);
 $87 = HEAP32[$4>>2]|0;
 $88 = ($87|0)!=(0);
 if (!($88)) {
  $90 = HEAP32[$pReader>>2]|0;
  $91 = HEAP32[$5>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($92|0);
 }
 $89 = HEAP32[$aElem>>2]|0;
 _sqlite3_free($89);
 $90 = HEAP32[$pReader>>2]|0;
 $91 = HEAP32[$5>>2]|0;
 HEAP32[$91>>2] = $90;
 $92 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($92|0);
}
function _fts3SegReaderCursorAppend($pCsr,$pNew) {
 $pCsr = $pCsr|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $apNew = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $apNew = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5|0) % 16)&-1;
 $7 = ($6|0)==(0);
 do {
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (($10) + 16)|0;
   $12 = $11<<2;
   HEAP32[$nByte>>2] = $12;
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$nByte>>2]|0;
   $16 = (_sqlite3_realloc($14,$15)|0);
   HEAP32[$apNew>>2] = $16;
   $17 = HEAP32[$apNew>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    $20 = HEAP32[$apNew>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    HEAP32[$21>>2] = $20;
    break;
   }
   $19 = HEAP32[$2>>2]|0;
   _sqlite3Fts3SegReaderFree($19);
   HEAP32[$0>>2] = 7;
   $30 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 } while(0);
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$24>>2] = $26;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($25<<2)|0);
 HEAP32[$29>>2] = $22;
 HEAP32[$0>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3Fts3AllSegdirs($p,$iLangid,$iIndex,$iLevel,$ppStmt) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $iLevel = $iLevel|0;
 $ppStmt = $ppStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $iLevel;
 HEAP32[$4>>2] = $ppStmt;
 HEAP32[$pStmt>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)<(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = (_fts3SqlStmt($7,13,$pStmt,0)|0);
  HEAP32[$rc>>2] = $8;
  $9 = HEAP32[$rc>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$pStmt>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = (_getAbsoluteLevel($12,$13,$14,0)|0);
   $16 = tempRet0;
   (_sqlite3_bind_int64($11,1,$15,$16)|0);
   $17 = HEAP32[$pStmt>>2]|0;
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = (_getAbsoluteLevel($18,$19,$20,1023)|0);
   $22 = tempRet0;
   (_sqlite3_bind_int64($17,2,$21,$22)|0);
  }
 } else {
  $23 = (_fts3SqlStmt($7,12,$pStmt,0)|0);
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(0);
  if ($25) {
   $26 = HEAP32[$pStmt>>2]|0;
   $27 = HEAP32[$0>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   $31 = (_getAbsoluteLevel($27,$28,$29,$30)|0);
   $32 = tempRet0;
   (_sqlite3_bind_int64($26,1,$31,$32)|0);
  }
 }
 $33 = HEAP32[$pStmt>>2]|0;
 $34 = HEAP32[$4>>2]|0;
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _fts3SelectLeaf($p,$zTerm,$nTerm,$zNode,$nNode,$piLeaf,$piLeaf2) {
 $p = $p|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $zNode = $zNode|0;
 $nNode = $nNode|0;
 $piLeaf = $piLeaf|0;
 $piLeaf2 = $piLeaf2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $iHeight = 0, $nBlob = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $zBlob = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $rc = sp + 12|0;
 $iHeight = sp + 8|0;
 $zBlob = sp + 4|0;
 $nBlob = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$3>>2] = $zNode;
 HEAP32[$4>>2] = $nNode;
 HEAP32[$5>>2] = $piLeaf;
 HEAP32[$6>>2] = $piLeaf2;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & 128;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$3>>2]|0;
 if ($11) {
  (_sqlite3Fts3GetVarint32($12,$iHeight)|0);
 } else {
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  HEAP32[$iHeight>>2] = $14;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = HEAP32[$6>>2]|0;
 $21 = (_fts3ScanInteriorNode($15,$16,$17,$18,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)==(0);
 $24 = HEAP32[$iHeight>>2]|0;
 $25 = ($24|0)>(1);
 $or$cond = $23 & $25;
 if (!($or$cond)) {
  $99 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($99|0);
 }
 HEAP32[$zBlob>>2] = 0;
 HEAP32[$nBlob>>2] = 0;
 $26 = HEAP32[$5>>2]|0;
 $27 = ($26|0)!=(0|0);
 $28 = HEAP32[$6>>2]|0;
 $29 = ($28|0)!=(0|0);
 $or$cond3 = $27 & $29;
 if ($or$cond3) {
  $30 = HEAP32[$5>>2]|0;
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$6>>2]|0;
  $38 = $37;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (($38) + 4)|0;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = ($33|0)!=($40|0);
  $45 = ($36|0)!=($43|0);
  $46 = $44 | $45;
  if ($46) {
   $47 = HEAP32[$0>>2]|0;
   $48 = HEAP32[$5>>2]|0;
   $49 = $48;
   $50 = $49;
   $51 = HEAP32[$50>>2]|0;
   $52 = (($49) + 4)|0;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   $55 = (_sqlite3Fts3ReadBlock($47,$51,$54,$zBlob,$nBlob,0)|0);
   HEAP32[$rc>>2] = $55;
   $56 = HEAP32[$rc>>2]|0;
   $57 = ($56|0)==(0);
   if ($57) {
    $58 = HEAP32[$0>>2]|0;
    $59 = HEAP32[$1>>2]|0;
    $60 = HEAP32[$2>>2]|0;
    $61 = HEAP32[$zBlob>>2]|0;
    $62 = HEAP32[$nBlob>>2]|0;
    $63 = HEAP32[$5>>2]|0;
    $64 = (_fts3SelectLeaf($58,$59,$60,$61,$62,$63,0)|0);
    HEAP32[$rc>>2] = $64;
   }
   $65 = HEAP32[$zBlob>>2]|0;
   _sqlite3_free($65);
   HEAP32[$5>>2] = 0;
   HEAP32[$zBlob>>2] = 0;
  }
 }
 $66 = HEAP32[$rc>>2]|0;
 $67 = ($66|0)==(0);
 if ($67) {
  $68 = HEAP32[$0>>2]|0;
  $69 = HEAP32[$5>>2]|0;
  $70 = ($69|0)!=(0|0);
  if ($70) {
   $71 = HEAP32[$5>>2]|0;
   $72 = $71;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $85 = $74;$86 = $77;
  } else {
   $78 = HEAP32[$6>>2]|0;
   $79 = $78;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (($79) + 4)|0;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = $81;$86 = $84;
  }
  $87 = (_sqlite3Fts3ReadBlock($68,$85,$86,$zBlob,$nBlob,0)|0);
  HEAP32[$rc>>2] = $87;
 }
 $88 = HEAP32[$rc>>2]|0;
 $89 = ($88|0)==(0);
 if ($89) {
  $90 = HEAP32[$0>>2]|0;
  $91 = HEAP32[$1>>2]|0;
  $92 = HEAP32[$2>>2]|0;
  $93 = HEAP32[$zBlob>>2]|0;
  $94 = HEAP32[$nBlob>>2]|0;
  $95 = HEAP32[$5>>2]|0;
  $96 = HEAP32[$6>>2]|0;
  $97 = (_fts3SelectLeaf($90,$91,$92,$93,$94,$95,$96)|0);
  HEAP32[$rc>>2] = $97;
 }
 $98 = HEAP32[$zBlob>>2]|0;
 _sqlite3_free($98);
 $99 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($99|0);
}
function _sqlite3Fts3SegReaderNew($iAge,$bLookup,$0,$1,$2,$3,$4,$5,$zRoot,$nRoot,$ppReader) {
 $iAge = $iAge|0;
 $bLookup = $bLookup|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 $4 = $4|0;
 $5 = $5|0;
 $zRoot = $zRoot|0;
 $nRoot = $nRoot|0;
 $ppReader = $ppReader|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $nExtra = 0, $pReader = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $6 = sp + 52|0;
 $7 = sp + 48|0;
 $8 = sp + 44|0;
 $9 = sp + 16|0;
 $10 = sp + 8|0;
 $11 = sp;
 $12 = sp + 40|0;
 $13 = sp + 36|0;
 $14 = sp + 32|0;
 $pReader = sp + 28|0;
 $nExtra = sp + 24|0;
 HEAP32[$7>>2] = $iAge;
 HEAP32[$8>>2] = $bLookup;
 $15 = $9;
 $16 = $15;
 HEAP32[$16>>2] = $0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = $1;
 $19 = $10;
 $20 = $19;
 HEAP32[$20>>2] = $2;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $3;
 $23 = $11;
 $24 = $23;
 HEAP32[$24>>2] = $4;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $5;
 HEAP32[$12>>2] = $zRoot;
 HEAP32[$13>>2] = $nRoot;
 HEAP32[$14>>2] = $ppReader;
 HEAP32[$nExtra>>2] = 0;
 $27 = $9;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = ($29|0)==(0);
 $34 = ($32|0)==(0);
 $35 = $33 & $34;
 if ($35) {
  $36 = HEAP32[$13>>2]|0;
  $37 = (($36) + 20)|0;
  HEAP32[$nExtra>>2] = $37;
 }
 $38 = HEAP32[$nExtra>>2]|0;
 $39 = (96 + ($38))|0;
 $40 = (_sqlite3_malloc($39)|0);
 HEAP32[$pReader>>2] = $40;
 $41 = HEAP32[$pReader>>2]|0;
 $42 = ($41|0)!=(0|0);
 if (!($42)) {
  HEAP32[$6>>2] = 7;
  $125 = HEAP32[$6>>2]|0;
  STACKTOP = sp;return ($125|0);
 }
 $43 = HEAP32[$pReader>>2]|0;
 dest=$43; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $44 = HEAP32[$7>>2]|0;
 $45 = HEAP32[$pReader>>2]|0;
 HEAP32[$45>>2] = $44;
 $46 = HEAP32[$8>>2]|0;
 $47 = ($46|0)!=(0);
 $48 = $47&1;
 $49 = $48&255;
 $50 = HEAP32[$pReader>>2]|0;
 $51 = ((($50)) + 4|0);
 HEAP8[$51>>0] = $49;
 $52 = $9;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$pReader>>2]|0;
 $59 = ((($58)) + 8|0);
 $60 = $59;
 $61 = $60;
 HEAP32[$61>>2] = $54;
 $62 = (($60) + 4)|0;
 $63 = $62;
 HEAP32[$63>>2] = $57;
 $64 = $10;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = (($64) + 4)|0;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$pReader>>2]|0;
 $71 = ((($70)) + 16|0);
 $72 = $71;
 $73 = $72;
 HEAP32[$73>>2] = $66;
 $74 = (($72) + 4)|0;
 $75 = $74;
 HEAP32[$75>>2] = $69;
 $76 = $11;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = (($76) + 4)|0;
 $80 = $79;
 $81 = HEAP32[$80>>2]|0;
 $82 = HEAP32[$pReader>>2]|0;
 $83 = ((($82)) + 24|0);
 $84 = $83;
 $85 = $84;
 HEAP32[$85>>2] = $78;
 $86 = (($84) + 4)|0;
 $87 = $86;
 HEAP32[$87>>2] = $81;
 $88 = HEAP32[$nExtra>>2]|0;
 $89 = ($88|0)!=(0);
 if ($89) {
  $90 = HEAP32[$pReader>>2]|0;
  $91 = ((($90)) + 96|0);
  $92 = HEAP32[$pReader>>2]|0;
  $93 = ((($92)) + 40|0);
  HEAP32[$93>>2] = $91;
  $94 = HEAP32[$pReader>>2]|0;
  $95 = ((($94)) + 5|0);
  HEAP8[$95>>0] = 1;
  $96 = HEAP32[$13>>2]|0;
  $97 = HEAP32[$pReader>>2]|0;
  $98 = ((($97)) + 44|0);
  HEAP32[$98>>2] = $96;
  $99 = HEAP32[$pReader>>2]|0;
  $100 = ((($99)) + 40|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = HEAP32[$12>>2]|0;
  $103 = HEAP32[$13>>2]|0;
  _memcpy(($101|0),($102|0),($103|0))|0;
  $104 = HEAP32[$13>>2]|0;
  $105 = HEAP32[$pReader>>2]|0;
  $106 = ((($105)) + 40|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($107) + ($104)|0);
  dest=$108; stop=dest+20|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 } else {
  $109 = $9;
  $110 = $109;
  $111 = HEAP32[$110>>2]|0;
  $112 = (($109) + 4)|0;
  $113 = $112;
  $114 = HEAP32[$113>>2]|0;
  $115 = (_i64Subtract(($111|0),($114|0),1,0)|0);
  $116 = tempRet0;
  $117 = HEAP32[$pReader>>2]|0;
  $118 = ((($117)) + 32|0);
  $119 = $118;
  $120 = $119;
  HEAP32[$120>>2] = $115;
  $121 = (($119) + 4)|0;
  $122 = $121;
  HEAP32[$122>>2] = $116;
 }
 $123 = HEAP32[$pReader>>2]|0;
 $124 = HEAP32[$14>>2]|0;
 HEAP32[$124>>2] = $123;
 HEAP32[$6>>2] = 0;
 $125 = HEAP32[$6>>2]|0;
 STACKTOP = sp;return ($125|0);
}
function _fts3ScanInteriorNode($zTerm,$nTerm,$zNode,$nNode,$piFirst,$piLast) {
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $zNode = $zNode|0;
 $nNode = $nNode|0;
 $piFirst = $piFirst|0;
 $piLast = $piLast|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cmp = 0, $iChild = 0, $isFirstTerm = 0, $nAlloc = 0;
 var $nBuffer = 0, $nPrefix = 0, $nSuffix = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $zBuffer = 0, $zCsr = 0, $zEnd = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $rc = sp + 48|0;
 $zCsr = sp + 44|0;
 $zEnd = sp + 40|0;
 $zBuffer = sp + 36|0;
 $nAlloc = sp + 32|0;
 $isFirstTerm = sp + 28|0;
 $iChild = sp;
 $cmp = sp + 24|0;
 $nSuffix = sp + 20|0;
 $nPrefix = sp + 16|0;
 $nBuffer = sp + 12|0;
 $zNew = sp + 8|0;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$3>>2] = $zNode;
 HEAP32[$4>>2] = $nNode;
 HEAP32[$5>>2] = $piFirst;
 HEAP32[$6>>2] = $piLast;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$zCsr>>2] = $7;
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP32[$zCsr>>2]|0;
 $10 = (($9) + ($8)|0);
 HEAP32[$zEnd>>2] = $10;
 HEAP32[$zBuffer>>2] = 0;
 HEAP32[$nAlloc>>2] = 0;
 HEAP32[$isFirstTerm>>2] = 1;
 $11 = HEAP32[$zCsr>>2]|0;
 $12 = (_sqlite3Fts3GetVarint($11,$iChild)|0);
 $13 = HEAP32[$zCsr>>2]|0;
 $14 = (($13) + ($12)|0);
 HEAP32[$zCsr>>2] = $14;
 $15 = HEAP32[$zCsr>>2]|0;
 $16 = (_sqlite3Fts3GetVarint($15,$iChild)|0);
 $17 = HEAP32[$zCsr>>2]|0;
 $18 = (($17) + ($16)|0);
 HEAP32[$zCsr>>2] = $18;
 $19 = HEAP32[$zCsr>>2]|0;
 $20 = HEAP32[$zEnd>>2]|0;
 $21 = ($19>>>0)>($20>>>0);
 if ($21) {
  HEAP32[$0>>2] = 267;
  $175 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($175|0);
 }
 while(1) {
  $22 = HEAP32[$zCsr>>2]|0;
  $23 = HEAP32[$zEnd>>2]|0;
  $24 = ($22>>>0)<($23>>>0);
  if (!($24)) {
   label = 28;
   break;
  }
  $25 = HEAP32[$5>>2]|0;
  $26 = ($25|0)!=(0|0);
  $27 = HEAP32[$6>>2]|0;
  $28 = ($27|0)!=(0|0);
  $29 = $26 ? 1 : $28;
  if (!($29)) {
   label = 28;
   break;
  }
  HEAP32[$nPrefix>>2] = 0;
  $30 = HEAP32[$isFirstTerm>>2]|0;
  $31 = ($30|0)!=(0);
  if (!($31)) {
   $32 = HEAP32[$zCsr>>2]|0;
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = $34 & 128;
   $36 = ($35|0)!=(0);
   $37 = HEAP32[$zCsr>>2]|0;
   if ($36) {
    $38 = (_sqlite3Fts3GetVarint32($37,$nPrefix)|0);
    $43 = $38;
   } else {
    $39 = HEAP8[$37>>0]|0;
    $40 = $39&255;
    HEAP32[$nPrefix>>2] = $40;
    $43 = 1;
   }
   $41 = HEAP32[$zCsr>>2]|0;
   $42 = (($41) + ($43)|0);
   HEAP32[$zCsr>>2] = $42;
  }
  HEAP32[$isFirstTerm>>2] = 0;
  $44 = HEAP32[$zCsr>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $46 & 128;
  $48 = ($47|0)!=(0);
  $49 = HEAP32[$zCsr>>2]|0;
  if ($48) {
   $50 = (_sqlite3Fts3GetVarint32($49,$nSuffix)|0);
   $55 = $50;
  } else {
   $51 = HEAP8[$49>>0]|0;
   $52 = $51&255;
   HEAP32[$nSuffix>>2] = $52;
   $55 = 1;
  }
  $53 = HEAP32[$zCsr>>2]|0;
  $54 = (($53) + ($55)|0);
  HEAP32[$zCsr>>2] = $54;
  $56 = HEAP32[$nPrefix>>2]|0;
  $57 = ($56|0)<(0);
  $58 = HEAP32[$nSuffix>>2]|0;
  $59 = ($58|0)<(0);
  $or$cond = $57 | $59;
  if ($or$cond) {
   label = 15;
   break;
  }
  $60 = HEAP32[$nSuffix>>2]|0;
  $61 = HEAP32[$zCsr>>2]|0;
  $62 = (($61) + ($60)|0);
  $63 = HEAP32[$zEnd>>2]|0;
  $64 = ($62>>>0)>($63>>>0);
  if ($64) {
   label = 15;
   break;
  }
  $65 = HEAP32[$nPrefix>>2]|0;
  $66 = HEAP32[$nSuffix>>2]|0;
  $67 = (($65) + ($66))|0;
  $68 = HEAP32[$nAlloc>>2]|0;
  $69 = ($67|0)>($68|0);
  if ($69) {
   $70 = HEAP32[$nPrefix>>2]|0;
   $71 = HEAP32[$nSuffix>>2]|0;
   $72 = (($70) + ($71))|0;
   $73 = $72<<1;
   HEAP32[$nAlloc>>2] = $73;
   $74 = HEAP32[$zBuffer>>2]|0;
   $75 = HEAP32[$nAlloc>>2]|0;
   $76 = (_sqlite3_realloc($74,$75)|0);
   HEAP32[$zNew>>2] = $76;
   $77 = HEAP32[$zNew>>2]|0;
   $78 = ($77|0)!=(0|0);
   if (!($78)) {
    label = 18;
    break;
   }
   $79 = HEAP32[$zNew>>2]|0;
   HEAP32[$zBuffer>>2] = $79;
  }
  $80 = HEAP32[$nPrefix>>2]|0;
  $81 = HEAP32[$zBuffer>>2]|0;
  $82 = (($81) + ($80)|0);
  $83 = HEAP32[$zCsr>>2]|0;
  $84 = HEAP32[$nSuffix>>2]|0;
  _memcpy(($82|0),($83|0),($84|0))|0;
  $85 = HEAP32[$nPrefix>>2]|0;
  $86 = HEAP32[$nSuffix>>2]|0;
  $87 = (($85) + ($86))|0;
  HEAP32[$nBuffer>>2] = $87;
  $88 = HEAP32[$nSuffix>>2]|0;
  $89 = HEAP32[$zCsr>>2]|0;
  $90 = (($89) + ($88)|0);
  HEAP32[$zCsr>>2] = $90;
  $91 = HEAP32[$1>>2]|0;
  $92 = HEAP32[$zBuffer>>2]|0;
  $93 = HEAP32[$nBuffer>>2]|0;
  $94 = HEAP32[$2>>2]|0;
  $95 = ($93|0)>($94|0);
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$nBuffer>>2]|0;
  $98 = $95 ? $96 : $97;
  $99 = (_memcmp($91,$92,$98)|0);
  HEAP32[$cmp>>2] = $99;
  $100 = HEAP32[$5>>2]|0;
  $101 = ($100|0)!=(0|0);
  do {
   if ($101) {
    $102 = HEAP32[$cmp>>2]|0;
    $103 = ($102|0)<(0);
    if (!($103)) {
     $104 = HEAP32[$cmp>>2]|0;
     $105 = ($104|0)==(0);
     if (!($105)) {
      break;
     }
     $106 = HEAP32[$nBuffer>>2]|0;
     $107 = HEAP32[$2>>2]|0;
     $108 = ($106|0)>($107|0);
     if (!($108)) {
      break;
     }
    }
    $109 = $iChild;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = (($109) + 4)|0;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = HEAP32[$5>>2]|0;
    $116 = $115;
    $117 = $116;
    HEAP32[$117>>2] = $111;
    $118 = (($116) + 4)|0;
    $119 = $118;
    HEAP32[$119>>2] = $114;
    HEAP32[$5>>2] = 0;
   }
  } while(0);
  $120 = HEAP32[$6>>2]|0;
  $121 = ($120|0)!=(0|0);
  $122 = HEAP32[$cmp>>2]|0;
  $123 = ($122|0)<(0);
  $or$cond3 = $121 & $123;
  if ($or$cond3) {
   $124 = $iChild;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($124) + 4)|0;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = HEAP32[$6>>2]|0;
   $131 = $130;
   $132 = $131;
   HEAP32[$132>>2] = $126;
   $133 = (($131) + 4)|0;
   $134 = $133;
   HEAP32[$134>>2] = $129;
   HEAP32[$6>>2] = 0;
  }
  $135 = $iChild;
  $136 = $135;
  $137 = HEAP32[$136>>2]|0;
  $138 = (($135) + 4)|0;
  $139 = $138;
  $140 = HEAP32[$139>>2]|0;
  $141 = (_i64Add(($137|0),($140|0),1,0)|0);
  $142 = tempRet0;
  $143 = $iChild;
  $144 = $143;
  HEAP32[$144>>2] = $141;
  $145 = (($143) + 4)|0;
  $146 = $145;
  HEAP32[$146>>2] = $142;
 }
 if ((label|0) == 15) {
  HEAP32[$rc>>2] = 267;
 }
 else if ((label|0) == 18) {
  HEAP32[$rc>>2] = 7;
 }
 else if ((label|0) == 28) {
  $147 = HEAP32[$5>>2]|0;
  $148 = ($147|0)!=(0|0);
  if ($148) {
   $149 = $iChild;
   $150 = $149;
   $151 = HEAP32[$150>>2]|0;
   $152 = (($149) + 4)|0;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = HEAP32[$5>>2]|0;
   $156 = $155;
   $157 = $156;
   HEAP32[$157>>2] = $151;
   $158 = (($156) + 4)|0;
   $159 = $158;
   HEAP32[$159>>2] = $154;
  }
  $160 = HEAP32[$6>>2]|0;
  $161 = ($160|0)!=(0|0);
  if ($161) {
   $162 = $iChild;
   $163 = $162;
   $164 = HEAP32[$163>>2]|0;
   $165 = (($162) + 4)|0;
   $166 = $165;
   $167 = HEAP32[$166>>2]|0;
   $168 = HEAP32[$6>>2]|0;
   $169 = $168;
   $170 = $169;
   HEAP32[$170>>2] = $164;
   $171 = (($169) + 4)|0;
   $172 = $171;
   HEAP32[$172>>2] = $167;
  }
 }
 $173 = HEAP32[$zBuffer>>2]|0;
 _sqlite3_free($173);
 $174 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $174;
 $175 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($175|0);
}
function _fts3CompareElemByTerm($lhs,$rhs) {
 $lhs = $lhs|0;
 $rhs = $rhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $n = 0, $n1 = 0, $n2 = 0, $z1 = 0, $z2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $z1 = sp + 20|0;
 $z2 = sp + 16|0;
 $n1 = sp + 12|0;
 $n2 = sp + 8|0;
 $n = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $lhs;
 HEAP32[$1>>2] = $rhs;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$z1>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$z2>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$n1>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$n2>>2] = $17;
 $18 = HEAP32[$n1>>2]|0;
 $19 = HEAP32[$n2>>2]|0;
 $20 = ($18|0)<($19|0);
 $21 = HEAP32[$n1>>2]|0;
 $22 = HEAP32[$n2>>2]|0;
 $23 = $20 ? $21 : $22;
 HEAP32[$n>>2] = $23;
 $24 = HEAP32[$z1>>2]|0;
 $25 = HEAP32[$z2>>2]|0;
 $26 = HEAP32[$n>>2]|0;
 $27 = (_memcmp($24,$25,$26)|0);
 HEAP32[$c>>2] = $27;
 $28 = HEAP32[$c>>2]|0;
 $29 = ($28|0)==(0);
 if (!($29)) {
  $33 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $30 = HEAP32[$n1>>2]|0;
 $31 = HEAP32[$n2>>2]|0;
 $32 = (($30) - ($31))|0;
 HEAP32[$c>>2] = $32;
 $33 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _fts3SnippetFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iCol = 0, $nToken = 0, $or$cond = 0, $or$cond3 = 0, $pCsr = 0, $zEllipsis = 0, $zEnd = 0, $zStart = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $pCsr = sp + 20|0;
 $zStart = sp + 16|0;
 $zEnd = sp + 12|0;
 $zEllipsis = sp + 8|0;
 $iCol = sp + 4|0;
 $nToken = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$zStart>>2] = 23427;
 HEAP32[$zEnd>>2] = 23431;
 HEAP32[$zEllipsis>>2] = 23436;
 HEAP32[$iCol>>2] = -1;
 HEAP32[$nToken>>2] = 15;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(6);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  _sqlite3_result_error($5,23447,-1);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_fts3FunctionArg($5,20486,$7,$pCsr)|0);
 $9 = ($8|0)!=(0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 switch ($10|0) {
 case 6:  {
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3_value_int($13)|0);
  HEAP32[$nToken>>2] = $14;
  label = 6;
  break;
 }
 case 5:  {
  label = 6;
  break;
 }
 case 4:  {
  label = 7;
  break;
 }
 case 3:  {
  label = 8;
  break;
 }
 case 2:  {
  label = 9;
  break;
 }
 default: {
 }
 }
 if ((label|0) == 6) {
  $15 = HEAP32[$2>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (_sqlite3_value_int($17)|0);
  HEAP32[$iCol>>2] = $18;
  label = 7;
 }
 if ((label|0) == 7) {
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3_value_text($21)|0);
  HEAP32[$zEllipsis>>2] = $22;
  label = 8;
 }
 if ((label|0) == 8) {
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3_value_text($25)|0);
  HEAP32[$zEnd>>2] = $26;
  label = 9;
 }
 if ((label|0) == 9) {
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (_sqlite3_value_text($29)|0);
  HEAP32[$zStart>>2] = $30;
 }
 $31 = HEAP32[$zEllipsis>>2]|0;
 $32 = ($31|0)!=(0|0);
 $33 = HEAP32[$zEnd>>2]|0;
 $34 = ($33|0)!=(0|0);
 $or$cond = $32 & $34;
 $35 = HEAP32[$zStart>>2]|0;
 $36 = ($35|0)!=(0|0);
 $or$cond3 = $or$cond & $36;
 if (!($or$cond3)) {
  $37 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($37);
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$nToken>>2]|0;
 $39 = ($38|0)==(0);
 $40 = HEAP32[$0>>2]|0;
 if ($39) {
  _sqlite3_result_text($40,18402,-1,0);
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$pCsr>>2]|0;
 $42 = (_fts3CursorSeek($40,$41)|0);
 $43 = (0)==($42|0);
 if (!($43)) {
  STACKTOP = sp;return;
 }
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$pCsr>>2]|0;
 $46 = HEAP32[$zStart>>2]|0;
 $47 = HEAP32[$zEnd>>2]|0;
 $48 = HEAP32[$zEllipsis>>2]|0;
 $49 = HEAP32[$iCol>>2]|0;
 $50 = HEAP32[$nToken>>2]|0;
 _sqlite3Fts3Snippet($44,$45,$46,$47,$48,$49,$50);
 STACKTOP = sp;return;
}
function _fts3OffsetsFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_fts3FunctionArg($3,20494,$5,$pCsr)|0);
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$pCsr>>2]|0;
 $10 = (_fts3CursorSeek($8,$9)|0);
 $11 = (0)==($10|0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$pCsr>>2]|0;
 _sqlite3Fts3Offsets($12,$13);
 STACKTOP = sp;return;
}
function _fts3OptimizeFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pCursor = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $p = sp + 4|0;
 $pCursor = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_fts3FunctionArg($3,20512,$5,$pCursor)|0);
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$pCursor>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = (_sqlite3Fts3Optimize($10)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 switch ($12|0) {
 case 0:  {
  $13 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($13,23331,-1,0);
  STACKTOP = sp;return;
  break;
 }
 case 101:  {
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($14,23347,-1,0);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($15,$16);
  STACKTOP = sp;return;
 }
 }
}
function _fts3MatchinfoFunc($pContext,$nVal,$apVal) {
 $pContext = $pContext|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $zArg = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pCsr = sp + 4|0;
 $zArg = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $nVal;
 HEAP32[$2>>2] = $apVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_fts3FunctionArg($3,20502,$5,$pCsr)|0);
 $7 = (0)==($6|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 HEAP32[$zArg>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)>(1);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_sqlite3_value_text($12)|0);
  HEAP32[$zArg>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$pCsr>>2]|0;
 $16 = HEAP32[$zArg>>2]|0;
 _sqlite3Fts3Matchinfo($14,$15,$16);
 STACKTOP = sp;return;
}
function _fts3FunctionArg($pContext,$zFunc,$pVal,$ppCsr) {
 $pContext = $pContext|0;
 $zFunc = $zFunc|0;
 $pVal = $pVal|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pRet = 0, $vararg_buffer = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pRet = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$1>>2] = $pContext;
 HEAP32[$2>>2] = $zFunc;
 HEAP32[$3>>2] = $pVal;
 HEAP32[$4>>2] = $ppCsr;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_sqlite3_value_type($5)|0);
 $7 = ($6|0)!=(4);
 if (!($7)) {
  $8 = HEAP32[$3>>2]|0;
  $9 = (_sqlite3_value_bytes($8)|0);
  $10 = ($9|0)!=(4);
  if (!($10)) {
   $16 = HEAP32[$3>>2]|0;
   $17 = (_sqlite3_value_blob($16)|0);
   ;HEAP8[$pRet>>0]=HEAP8[$17>>0]|0;HEAP8[$pRet+1>>0]=HEAP8[$17+1>>0]|0;HEAP8[$pRet+2>>0]=HEAP8[$17+2>>0]|0;HEAP8[$pRet+3>>0]=HEAP8[$17+3>>0]|0;
   $18 = HEAP32[$pRet>>2]|0;
   $19 = HEAP32[$4>>2]|0;
   HEAP32[$19>>2] = $18;
   HEAP32[$0>>2] = 0;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 $12 = (_sqlite3_mprintf(23302,$vararg_buffer)|0);
 HEAP32[$zErr>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$zErr>>2]|0;
 _sqlite3_result_error($13,$14,-1);
 $15 = HEAP32[$zErr>>2]|0;
 _sqlite3_free($15);
 HEAP32[$0>>2] = 1;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3Fts3Matchinfo($pContext,$pCsr,$zArg) {
 $pContext = $pContext|0;
 $pCsr = $pCsr|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, $zFormat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pTab = sp + 4|0;
 $zFormat = sp;
 HEAP32[$0>>2] = $pContext;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $zArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  HEAP32[$zFormat>>2] = $7;
 } else {
  HEAP32[$zFormat>>2] = 23237;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 $12 = HEAP32[$0>>2]|0;
 if ($11) {
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$zFormat>>2]|0;
  _fts3GetMatchinfo($12,$13,$14);
  $15 = HEAP32[$pTab>>2]|0;
  _sqlite3Fts3SegmentsClose($15);
  STACKTOP = sp;return;
 } else {
  _sqlite3_result_blob($12,18402,0,0);
  STACKTOP = sp;return;
 }
}
function _fts3GetMatchinfo($pCtx,$pCsr,$zArg) {
 $pCtx = $pCtx|0;
 $pCsr = $pCsr|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aOut = 0;
 var $bGlobal = 0, $i = 0, $n = 0, $nMatchinfo = 0, $pTab = 0, $rc = 0, $sInfo = 0, $xDestroyOut = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $sInfo = sp;
 $pTab = sp + 64|0;
 $rc = sp + 60|0;
 $bGlobal = sp + 56|0;
 $aOut = sp + 52|0;
 $xDestroyOut = sp + 48|0;
 $nMatchinfo = sp + 44|0;
 $i = sp + 40|0;
 $zErr = sp + 36|0;
 $n = sp + 32|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $zArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bGlobal>>2] = 0;
 HEAP32[$aOut>>2] = 0;
 HEAP32[$xDestroyOut>>2] = 0;
 ;HEAP32[$sInfo>>2]=0|0;HEAP32[$sInfo+4>>2]=0|0;HEAP32[$sInfo+8>>2]=0|0;HEAP32[$sInfo+12>>2]=0|0;HEAP32[$sInfo+16>>2]=0|0;HEAP32[$sInfo+20>>2]=0|0;HEAP32[$sInfo+24>>2]=0|0;HEAP32[$sInfo+28>>2]=0|0;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$sInfo>>2] = $5;
 $6 = HEAP32[$pTab>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($sInfo)) + 4|0);
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 92|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 92|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = (_strcmp($18,$19)|0);
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 92|0);
   $24 = HEAP32[$23>>2]|0;
   _sqlite3Fts3MIBufferFree($24);
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 92|0);
   HEAP32[$26>>2] = 0;
  }
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 92|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)==(0|0);
 if ($30) {
  HEAP32[$nMatchinfo>>2] = 0;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (_fts3ExprPhraseCount($33)|0);
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 20|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 20|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ((($sInfo)) + 8|0);
  HEAP32[$40>>2] = $39;
  HEAP32[$i>>2] = 0;
  while(1) {
   $41 = HEAP32[$i>>2]|0;
   $42 = HEAP32[$2>>2]|0;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = ($44<<24>>24)!=(0);
   if (!($45)) {
    break;
   }
   HEAP32[$zErr>>2] = 0;
   $46 = HEAP32[$pTab>>2]|0;
   $47 = HEAP32[$i>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = (($48) + ($47)|0);
   $50 = HEAP8[$49>>0]|0;
   $51 = (_fts3MatchinfoCheck($46,$50,$zErr)|0);
   $52 = ($51|0)!=(0);
   if ($52) {
    label = 8;
    break;
   }
   $56 = HEAP32[$i>>2]|0;
   $57 = HEAP32[$2>>2]|0;
   $58 = (($57) + ($56)|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = (_fts3MatchinfoSize($sInfo,$59)|0);
   $61 = HEAP32[$nMatchinfo>>2]|0;
   $62 = (($61) + ($60))|0;
   HEAP32[$nMatchinfo>>2] = $62;
   $63 = HEAP32[$i>>2]|0;
   $64 = (($63) + 1)|0;
   HEAP32[$i>>2] = $64;
  }
  if ((label|0) == 8) {
   $53 = HEAP32[$0>>2]|0;
   $54 = HEAP32[$zErr>>2]|0;
   _sqlite3_result_error($53,$54,-1);
   $55 = HEAP32[$zErr>>2]|0;
   _sqlite3_free($55);
   STACKTOP = sp;return;
  }
  $65 = HEAP32[$nMatchinfo>>2]|0;
  $66 = HEAP32[$2>>2]|0;
  $67 = (_fts3MIBufferNew($65,$66)|0);
  $68 = HEAP32[$1>>2]|0;
  $69 = ((($68)) + 92|0);
  HEAP32[$69>>2] = $67;
  $70 = HEAP32[$1>>2]|0;
  $71 = ((($70)) + 92|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($72|0)!=(0|0);
  if (!($73)) {
   HEAP32[$rc>>2] = 7;
  }
  $74 = HEAP32[$1>>2]|0;
  $75 = ((($74)) + 88|0);
  HEAP32[$75>>2] = 1;
  HEAP32[$bGlobal>>2] = 1;
 }
 $76 = HEAP32[$rc>>2]|0;
 $77 = ($76|0)==(0);
 if ($77) {
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 92|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = (_fts3MIBufferAlloc($80,$aOut)|0);
  HEAP32[$xDestroyOut>>2] = $81;
  $82 = HEAP32[$xDestroyOut>>2]|0;
  $83 = ($82|0)==(0|0);
  if ($83) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $84 = HEAP32[$rc>>2]|0;
 $85 = ($84|0)==(0);
 if ($85) {
  $86 = HEAP32[$aOut>>2]|0;
  $87 = ((($sInfo)) + 28|0);
  HEAP32[$87>>2] = $86;
  $88 = HEAP32[$1>>2]|0;
  $89 = ((($88)) + 20|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = ((($sInfo)) + 8|0);
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$1>>2]|0;
  $93 = HEAP32[$bGlobal>>2]|0;
  $94 = HEAP32[$2>>2]|0;
  $95 = (_fts3MatchinfoValues($92,$93,$sInfo,$94)|0);
  HEAP32[$rc>>2] = $95;
  $96 = HEAP32[$bGlobal>>2]|0;
  $97 = ($96|0)!=(0);
  if ($97) {
   $98 = HEAP32[$1>>2]|0;
   $99 = ((($98)) + 92|0);
   $100 = HEAP32[$99>>2]|0;
   _fts3MIBufferSetGlobal($100);
  }
 }
 $101 = HEAP32[$rc>>2]|0;
 $102 = ($101|0)!=(0);
 if (!($102)) {
  $109 = HEAP32[$1>>2]|0;
  $110 = ((($109)) + 92|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ((($111)) + 4|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = $113<<2;
  HEAP32[$n>>2] = $114;
  $115 = HEAP32[$0>>2]|0;
  $116 = HEAP32[$aOut>>2]|0;
  $117 = HEAP32[$n>>2]|0;
  $118 = HEAP32[$xDestroyOut>>2]|0;
  _sqlite3_result_blob($115,$116,$117,$118);
  STACKTOP = sp;return;
 }
 $103 = HEAP32[$0>>2]|0;
 $104 = HEAP32[$rc>>2]|0;
 _sqlite3_result_error_code($103,$104);
 $105 = HEAP32[$xDestroyOut>>2]|0;
 $106 = ($105|0)!=(0|0);
 if (!($106)) {
  STACKTOP = sp;return;
 }
 $107 = HEAP32[$xDestroyOut>>2]|0;
 $108 = HEAP32[$aOut>>2]|0;
 FUNCTION_TABLE_vi[$107 & 255]($108);
 STACKTOP = sp;return;
}
function _sqlite3Fts3SegmentsClose($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 244|0);
 $3 = HEAP32[$2>>2]|0;
 (_sqlite3_blob_close($3)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 244|0);
 HEAP32[$5>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3MIBufferFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 HEAP8[$3>>0] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 1|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 2|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)==(0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 _sqlite3_free($18);
 STACKTOP = sp;return;
}
function _fts3ExprPhraseCount($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $nPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nPhrase = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$nPhrase>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 (_fts3ExprIterate($1,153,$nPhrase)|0);
 $2 = HEAP32[$nPhrase>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _fts3MatchinfoCheck($pTab,$cArg,$pzErr) {
 $pTab = $pTab|0;
 $cArg = $cArg|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 16|0;
 $3 = sp + 4|0;
 HEAP32[$1>>2] = $pTab;
 HEAP8[$2>>0] = $cArg;
 HEAP32[$3>>2] = $pzErr;
 $4 = HEAP8[$2>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(112);
 do {
  if (!($6)) {
   $7 = HEAP8[$2>>0]|0;
   $8 = $7 << 24 >> 24;
   $9 = ($8|0)==(99);
   if (!($9)) {
    $10 = HEAP8[$2>>0]|0;
    $11 = $10 << 24 >> 24;
    $12 = ($11|0)==(110);
    if ($12) {
     $13 = HEAP32[$1>>2]|0;
     $14 = ((($13)) + 228|0);
     $15 = HEAP8[$14>>0]|0;
     $16 = $15&255;
     $17 = ($16|0)!=(0);
     if ($17) {
      break;
     }
    }
    $18 = HEAP8[$2>>0]|0;
    $19 = $18 << 24 >> 24;
    $20 = ($19|0)==(97);
    if ($20) {
     $21 = HEAP32[$1>>2]|0;
     $22 = ((($21)) + 228|0);
     $23 = HEAP8[$22>>0]|0;
     $24 = $23&255;
     $25 = ($24|0)!=(0);
     if ($25) {
      break;
     }
    }
    $26 = HEAP8[$2>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)==(108);
    if ($28) {
     $29 = HEAP32[$1>>2]|0;
     $30 = ((($29)) + 230|0);
     $31 = HEAP8[$30>>0]|0;
     $32 = $31&255;
     $33 = ($32|0)!=(0);
     if ($33) {
      break;
     }
    }
    $34 = HEAP8[$2>>0]|0;
    $35 = $34 << 24 >> 24;
    $36 = ($35|0)==(115);
    if (!($36)) {
     $37 = HEAP8[$2>>0]|0;
     $38 = $37 << 24 >> 24;
     $39 = ($38|0)==(120);
     if (!($39)) {
      $40 = HEAP8[$2>>0]|0;
      $41 = $40 << 24 >> 24;
      $42 = ($41|0)==(121);
      if (!($42)) {
       $43 = HEAP8[$2>>0]|0;
       $44 = $43 << 24 >> 24;
       $45 = ($44|0)==(98);
       if (!($45)) {
        $46 = HEAP32[$3>>2]|0;
        $47 = HEAP8[$2>>0]|0;
        $48 = $47 << 24 >> 24;
        HEAP32[$vararg_buffer>>2] = $48;
        _sqlite3Fts3ErrMsg($46,23267,$vararg_buffer);
        HEAP32[$0>>2] = 1;
        $49 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($49|0);
       }
      }
     }
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _fts3MatchinfoSize($pInfo,$cArg) {
 $pInfo = $pInfo|0;
 $cArg = $cArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $nVal = sp;
 HEAP32[$0>>2] = $pInfo;
 HEAP8[$1>>0] = $cArg;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2 << 24 >> 24;
 switch ($3|0) {
 case 99: case 112: case 110:  {
  HEAP32[$nVal>>2] = 1;
  break;
 }
 case 115: case 108: case 97:  {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$nVal>>2] = $6;
  break;
 }
 case 121:  {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = Math_imul($9, $12)|0;
  HEAP32[$nVal>>2] = $13;
  break;
 }
 case 98:  {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + 31)|0;
  $21 = (($20|0) / 32)&-1;
  $22 = Math_imul($16, $21)|0;
  HEAP32[$nVal>>2] = $22;
  break;
 }
 default: {
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = Math_imul($25, $28)|0;
  $30 = ($29*3)|0;
  HEAP32[$nVal>>2] = $30;
 }
 }
 $31 = HEAP32[$nVal>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _fts3MIBufferNew($nElem,$zMatchinfo) {
 $nElem = $nElem|0;
 $zMatchinfo = $zMatchinfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $nStr = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRet = sp + 8|0;
 $nByte = sp + 4|0;
 $nStr = sp;
 HEAP32[$0>>2] = $nElem;
 HEAP32[$1>>2] = $zMatchinfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = $2<<1;
 $4 = (($3) + 1)|0;
 $5 = $4<<2;
 $6 = (($5) + 20)|0;
 HEAP32[$nByte>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_strlen($7)|0);
 HEAP32[$nStr>>2] = $8;
 $9 = HEAP32[$nByte>>2]|0;
 $10 = HEAP32[$nStr>>2]|0;
 $11 = (($9) + ($10))|0;
 $12 = (($11) + 1)|0;
 $13 = (_sqlite3_malloc($12)|0);
 HEAP32[$pRet>>2] = $13;
 $14 = HEAP32[$pRet>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $54 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $16 = HEAP32[$pRet>>2]|0;
 $17 = HEAP32[$nByte>>2]|0;
 _memset(($16|0),0,($17|0))|0;
 $18 = HEAP32[$pRet>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$pRet>>2]|0;
 $22 = $20;
 $23 = $21;
 $24 = (($22) - ($23))|0;
 $25 = HEAP32[$pRet>>2]|0;
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$pRet>>2]|0;
 $28 = ((($27)) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = (($30) + 1)|0;
 $32 = $31<<2;
 $33 = (($29) + ($32))|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = (1 + ($34))|0;
 $36 = HEAP32[$pRet>>2]|0;
 $37 = ((($36)) + 16|0);
 $38 = (($37) + ($35<<2)|0);
 HEAP32[$38>>2] = $33;
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$pRet>>2]|0;
 $41 = ((($40)) + 4|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$pRet>>2]|0;
 $43 = HEAP32[$nByte>>2]|0;
 $44 = (($42) + ($43)|0);
 $45 = HEAP32[$pRet>>2]|0;
 $46 = ((($45)) + 12|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$pRet>>2]|0;
 $48 = ((($47)) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$nStr>>2]|0;
 $52 = (($51) + 1)|0;
 _memcpy(($49|0),($50|0),($52|0))|0;
 $53 = HEAP32[$pRet>>2]|0;
 HEAP8[$53>>0] = 1;
 $54 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _fts3MIBufferAlloc($p,$paOut) {
 $p = $p|0;
 $paOut = $paOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aOut = 0, $xRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $xRet = sp + 4|0;
 $aOut = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $paOut;
 HEAP32[$xRet>>2] = 0;
 HEAP32[$aOut>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 1|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$0>>2]|0;
 do {
  if ($6) {
   $8 = ((($7)) + 1|0);
   HEAP8[$8>>0] = 1;
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 16|0);
   $11 = ((($10)) + 4|0);
   HEAP32[$aOut>>2] = $11;
   HEAP32[$xRet>>2] = 152;
  } else {
   $12 = ((($7)) + 2|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)==(0);
   $16 = HEAP32[$0>>2]|0;
   if ($15) {
    $17 = ((($16)) + 2|0);
    HEAP8[$17>>0] = 1;
    $18 = HEAP32[$0>>2]|0;
    $19 = ((($18)) + 4|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + 2)|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 16|0);
    $24 = (($23) + ($21<<2)|0);
    HEAP32[$aOut>>2] = $24;
    HEAP32[$xRet>>2] = 152;
    break;
   }
   $25 = ((($16)) + 4|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = $26<<2;
   $28 = (_sqlite3_malloc($27)|0);
   HEAP32[$aOut>>2] = $28;
   $29 = HEAP32[$aOut>>2]|0;
   $30 = ($29|0)!=(0|0);
   if ($30) {
    HEAP32[$xRet>>2] = 148;
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = HEAP32[$aOut>>2]|0;
     $36 = HEAP32[$0>>2]|0;
     $37 = ((($36)) + 16|0);
     $38 = ((($37)) + 4|0);
     $39 = HEAP32[$0>>2]|0;
     $40 = ((($39)) + 4|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = $41<<2;
     _memcpy(($35|0),($38|0),($42|0))|0;
    }
   }
  }
 } while(0);
 $43 = HEAP32[$aOut>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 HEAP32[$44>>2] = $43;
 $45 = HEAP32[$xRet>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _fts3MatchinfoValues($pCsr,$bGlobal,$pInfo,$zArg) {
 $pCsr = $pCsr|0;
 $bGlobal = $bGlobal|0;
 $pInfo = $pInfo|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $a3 = 0, $i = 0, $iCol = 0, $iCol2 = 0, $iVal = 0, $nDoc = 0, $nDoc1 = 0, $nToken = 0, $nToken4 = 0, $nZero = 0, $pExpr = 0, $pSelect = 0, $pSelectDocsize = 0, $pTab = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $rc = sp + 76|0;
 $i = sp + 72|0;
 $pTab = sp + 68|0;
 $pSelect = sp + 64|0;
 $nDoc = sp + 24|0;
 $nDoc1 = sp + 16|0;
 $a = sp + 60|0;
 $iCol = sp + 56|0;
 $iVal = sp + 52|0;
 $nToken = sp + 8|0;
 $pSelectDocsize = sp + 48|0;
 $iCol2 = sp + 44|0;
 $a3 = sp + 40|0;
 $nToken4 = sp;
 $nZero = sp + 36|0;
 $pExpr = sp + 32|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $bGlobal;
 HEAP32[$2>>2] = $pInfo;
 HEAP32[$3>>2] = $zArg;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 HEAP32[$pSelect>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$rc>>2]|0;
  $7 = ($6|0)==(0);
  if (!($7)) {
   label = 31;
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  $10 = (($9) + ($8)|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)!=(0);
  if (!($13)) {
   label = 31;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + ($14)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 24|0);
  HEAP8[$19>>0] = $17;
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = (($21) + ($20)|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = $23 << 24 >> 24;
  L5: do {
   switch ($24|0) {
   case 112:  {
    $25 = HEAP32[$1>>2]|0;
    $26 = ($25|0)!=(0);
    if ($26) {
     $27 = HEAP32[$2>>2]|0;
     $28 = ((($27)) + 8|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = HEAP32[$2>>2]|0;
     $31 = ((($30)) + 28|0);
     $32 = HEAP32[$31>>2]|0;
     HEAP32[$32>>2] = $29;
    }
    break;
   }
   case 99:  {
    $33 = HEAP32[$1>>2]|0;
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = HEAP32[$2>>2]|0;
     $36 = ((($35)) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = HEAP32[$2>>2]|0;
     $39 = ((($38)) + 28|0);
     $40 = HEAP32[$39>>2]|0;
     HEAP32[$40>>2] = $37;
    }
    break;
   }
   case 110:  {
    $41 = HEAP32[$1>>2]|0;
    $42 = ($41|0)!=(0);
    if ($42) {
     $43 = $nDoc;
     $44 = $43;
     HEAP32[$44>>2] = 0;
     $45 = (($43) + 4)|0;
     $46 = $45;
     HEAP32[$46>>2] = 0;
     $47 = HEAP32[$pTab>>2]|0;
     $48 = (_fts3MatchinfoSelectDoctotal($47,$pSelect,$nDoc,0)|0);
     HEAP32[$rc>>2] = $48;
     $49 = $nDoc;
     $50 = $49;
     $51 = HEAP32[$50>>2]|0;
     $52 = (($49) + 4)|0;
     $53 = $52;
     $54 = HEAP32[$53>>2]|0;
     $55 = HEAP32[$2>>2]|0;
     $56 = ((($55)) + 28|0);
     $57 = HEAP32[$56>>2]|0;
     HEAP32[$57>>2] = $51;
    }
    break;
   }
   case 97:  {
    $58 = HEAP32[$1>>2]|0;
    $59 = ($58|0)!=(0);
    if ($59) {
     $60 = HEAP32[$pTab>>2]|0;
     $61 = (_fts3MatchinfoSelectDoctotal($60,$pSelect,$nDoc1,$a)|0);
     HEAP32[$rc>>2] = $61;
     $62 = HEAP32[$rc>>2]|0;
     $63 = ($62|0)==(0);
     if ($63) {
      HEAP32[$iCol>>2] = 0;
      while(1) {
       $64 = HEAP32[$iCol>>2]|0;
       $65 = HEAP32[$2>>2]|0;
       $66 = ((($65)) + 4|0);
       $67 = HEAP32[$66>>2]|0;
       $68 = ($64|0)<($67|0);
       if (!($68)) {
        break L5;
       }
       $69 = HEAP32[$a>>2]|0;
       $70 = (_sqlite3Fts3GetVarint($69,$nToken)|0);
       $71 = HEAP32[$a>>2]|0;
       $72 = (($71) + ($70)|0);
       HEAP32[$a>>2] = $72;
       $73 = $nToken;
       $74 = $73;
       $75 = HEAP32[$74>>2]|0;
       $76 = (($73) + 4)|0;
       $77 = $76;
       $78 = HEAP32[$77>>2]|0;
       $79 = $nDoc1;
       $80 = $79;
       $81 = HEAP32[$80>>2]|0;
       $82 = (($79) + 4)|0;
       $83 = $82;
       $84 = HEAP32[$83>>2]|0;
       $85 = (___divdi3(($81|0),($84|0),2,0)|0);
       $86 = tempRet0;
       $87 = (_i64Add(($75|0),0,($85|0),($86|0))|0);
       $88 = tempRet0;
       $89 = $nDoc1;
       $90 = $89;
       $91 = HEAP32[$90>>2]|0;
       $92 = (($89) + 4)|0;
       $93 = $92;
       $94 = HEAP32[$93>>2]|0;
       $95 = (___divdi3(($87|0),($88|0),($91|0),($94|0))|0);
       $96 = tempRet0;
       HEAP32[$iVal>>2] = $95;
       $97 = HEAP32[$iVal>>2]|0;
       $98 = HEAP32[$iCol>>2]|0;
       $99 = HEAP32[$2>>2]|0;
       $100 = ((($99)) + 28|0);
       $101 = HEAP32[$100>>2]|0;
       $102 = (($101) + ($98<<2)|0);
       HEAP32[$102>>2] = $97;
       $103 = HEAP32[$iCol>>2]|0;
       $104 = (($103) + 1)|0;
       HEAP32[$iCol>>2] = $104;
      }
     }
    }
    break;
   }
   case 108:  {
    HEAP32[$pSelectDocsize>>2] = 0;
    $105 = HEAP32[$pTab>>2]|0;
    $106 = HEAP32[$0>>2]|0;
    $107 = ((($106)) + 32|0);
    $108 = $107;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (($108) + 4)|0;
    $112 = $111;
    $113 = HEAP32[$112>>2]|0;
    $114 = (_sqlite3Fts3SelectDocsize($105,$110,$113,$pSelectDocsize)|0);
    HEAP32[$rc>>2] = $114;
    $115 = HEAP32[$rc>>2]|0;
    $116 = ($115|0)==(0);
    L27: do {
     if ($116) {
      $117 = HEAP32[$pSelectDocsize>>2]|0;
      $118 = (_sqlite3_column_blob($117,0)|0);
      HEAP32[$a3>>2] = $118;
      HEAP32[$iCol2>>2] = 0;
      while(1) {
       $119 = HEAP32[$iCol2>>2]|0;
       $120 = HEAP32[$2>>2]|0;
       $121 = ((($120)) + 4|0);
       $122 = HEAP32[$121>>2]|0;
       $123 = ($119|0)<($122|0);
       if (!($123)) {
        break L27;
       }
       $124 = HEAP32[$a3>>2]|0;
       $125 = (_sqlite3Fts3GetVarint($124,$nToken4)|0);
       $126 = HEAP32[$a3>>2]|0;
       $127 = (($126) + ($125)|0);
       HEAP32[$a3>>2] = $127;
       $128 = $nToken4;
       $129 = $128;
       $130 = HEAP32[$129>>2]|0;
       $131 = (($128) + 4)|0;
       $132 = $131;
       $133 = HEAP32[$132>>2]|0;
       $134 = HEAP32[$iCol2>>2]|0;
       $135 = HEAP32[$2>>2]|0;
       $136 = ((($135)) + 28|0);
       $137 = HEAP32[$136>>2]|0;
       $138 = (($137) + ($134<<2)|0);
       HEAP32[$138>>2] = $130;
       $139 = HEAP32[$iCol2>>2]|0;
       $140 = (($139) + 1)|0;
       HEAP32[$iCol2>>2] = $140;
      }
     }
    } while(0);
    $141 = HEAP32[$pSelectDocsize>>2]|0;
    (_sqlite3_reset($141)|0);
    break;
   }
   case 115:  {
    $142 = HEAP32[$0>>2]|0;
    $143 = (_fts3ExprLoadDoclists($142,0,0)|0);
    HEAP32[$rc>>2] = $143;
    $144 = HEAP32[$rc>>2]|0;
    $145 = ($144|0)==(0);
    if ($145) {
     $146 = HEAP32[$0>>2]|0;
     $147 = HEAP32[$2>>2]|0;
     $148 = (_fts3MatchinfoLcs($146,$147)|0);
     HEAP32[$rc>>2] = $148;
    }
    break;
   }
   case 121: case 98:  {
    $149 = HEAP32[$2>>2]|0;
    $150 = HEAP32[$i>>2]|0;
    $151 = HEAP32[$3>>2]|0;
    $152 = (($151) + ($150)|0);
    $153 = HEAP8[$152>>0]|0;
    $154 = (_fts3MatchinfoSize($149,$153)|0);
    $155 = $154<<2;
    HEAP32[$nZero>>2] = $155;
    $156 = HEAP32[$2>>2]|0;
    $157 = ((($156)) + 28|0);
    $158 = HEAP32[$157>>2]|0;
    $159 = HEAP32[$nZero>>2]|0;
    _memset(($158|0),0,($159|0))|0;
    $160 = HEAP32[$0>>2]|0;
    $161 = ((($160)) + 12|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = HEAP32[$2>>2]|0;
    _fts3ExprLHitGather($162,$163);
    break;
   }
   default: {
    $164 = HEAP32[$0>>2]|0;
    $165 = ((($164)) + 12|0);
    $166 = HEAP32[$165>>2]|0;
    HEAP32[$pExpr>>2] = $166;
    $167 = HEAP32[$0>>2]|0;
    $168 = (_fts3ExprLoadDoclists($167,0,0)|0);
    HEAP32[$rc>>2] = $168;
    $169 = HEAP32[$rc>>2]|0;
    $170 = ($169|0)!=(0);
    if (!($170)) {
     $171 = HEAP32[$1>>2]|0;
     $172 = ($171|0)!=(0);
     if ($172) {
      $173 = HEAP32[$0>>2]|0;
      $174 = ((($173)) + 24|0);
      $175 = HEAP32[$174>>2]|0;
      $176 = ($175|0)!=(0|0);
      if ($176) {
       $177 = HEAP32[$pTab>>2]|0;
       $178 = HEAP32[$2>>2]|0;
       $179 = ((($178)) + 16|0);
       $180 = (_fts3MatchinfoSelectDoctotal($177,$pSelect,$179,0)|0);
       HEAP32[$rc>>2] = $180;
       $181 = HEAP32[$rc>>2]|0;
       $182 = ($181|0)!=(0);
       if ($182) {
        break L5;
       }
      }
      $183 = HEAP32[$pExpr>>2]|0;
      $184 = HEAP32[$2>>2]|0;
      $185 = (_fts3ExprIterate($183,154,$184)|0);
      HEAP32[$rc>>2] = $185;
      $186 = HEAP32[$0>>2]|0;
      (_sqlite3Fts3EvalTestDeferred($186,$rc)|0);
      $187 = HEAP32[$rc>>2]|0;
      $188 = ($187|0)!=(0);
      if ($188) {
       break L5;
      }
     }
     $189 = HEAP32[$pExpr>>2]|0;
     $190 = HEAP32[$2>>2]|0;
     (_fts3ExprIterate($189,155,$190)|0);
    }
   }
   }
  } while(0);
  $191 = HEAP32[$2>>2]|0;
  $192 = HEAP32[$i>>2]|0;
  $193 = HEAP32[$3>>2]|0;
  $194 = (($193) + ($192)|0);
  $195 = HEAP8[$194>>0]|0;
  $196 = (_fts3MatchinfoSize($191,$195)|0);
  $197 = HEAP32[$2>>2]|0;
  $198 = ((($197)) + 28|0);
  $199 = HEAP32[$198>>2]|0;
  $200 = (($199) + ($196<<2)|0);
  HEAP32[$198>>2] = $200;
  $201 = HEAP32[$i>>2]|0;
  $202 = (($201) + 1)|0;
  HEAP32[$i>>2] = $202;
 }
 if ((label|0) == 31) {
  $203 = HEAP32[$pSelect>>2]|0;
  (_sqlite3_reset($203)|0);
  $204 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($204|0);
 }
 return (0)|0;
}
function _fts3MIBufferSetGlobal($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 HEAP32[$2>>2] = 1;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (2 + ($5))|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = (($8) + ($6<<2)|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15<<2;
 _memcpy(($9|0),($12|0),($16|0))|0;
 STACKTOP = sp;return;
}
function _fts3MatchinfoSelectDoctotal($pTab,$ppStmt,$pnDoc,$paLen) {
 $pTab = $pTab|0;
 $ppStmt = $ppStmt|0;
 $pnDoc = $pnDoc|0;
 $paLen = $paLen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $nDoc = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pStmt = sp + 16|0;
 $a = sp + 12|0;
 $nDoc = sp;
 $rc = sp + 8|0;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $ppStmt;
 HEAP32[$3>>2] = $pnDoc;
 HEAP32[$4>>2] = $paLen;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3Fts3SelectDoctotal($8,$9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $13;
   $46 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pStmt>>2] = $15;
 $16 = HEAP32[$pStmt>>2]|0;
 $17 = (_sqlite3_column_blob($16,0)|0);
 HEAP32[$a>>2] = $17;
 $18 = HEAP32[$a>>2]|0;
 $19 = (_sqlite3Fts3GetVarint($18,$nDoc)|0);
 $20 = HEAP32[$a>>2]|0;
 $21 = (($20) + ($19)|0);
 HEAP32[$a>>2] = $21;
 $22 = $nDoc;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = ($24|0)==(0);
 $29 = ($27|0)==(0);
 $30 = $28 & $29;
 if ($30) {
  HEAP32[$0>>2] = 267;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $31 = $nDoc;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$3>>2]|0;
 $38 = $37;
 $39 = $38;
 HEAP32[$39>>2] = $33;
 $40 = (($38) + 4)|0;
 $41 = $40;
 HEAP32[$41>>2] = 0;
 $42 = HEAP32[$4>>2]|0;
 $43 = ($42|0)!=(0|0);
 if ($43) {
  $44 = HEAP32[$a>>2]|0;
  $45 = HEAP32[$4>>2]|0;
  HEAP32[$45>>2] = $44;
 }
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _sqlite3Fts3SelectDocsize($pTab,$0,$1,$ppStmt) {
 $pTab = $pTab|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppStmt = $ppStmt|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $pTab;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $ppStmt;
 $9 = HEAP32[$2>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = (_fts3SelectDocsize($9,$12,$15,$16)|0);
 STACKTOP = sp;return ($17|0);
}
function _fts3ExprLoadDoclists($pCsr,$pnPhrase,$pnToken) {
 $pCsr = $pCsr|0;
 $pnPhrase = $pnPhrase|0;
 $pnToken = $pnToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var $sCtx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $sCtx = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pnPhrase;
 HEAP32[$2>>2] = $pnToken;
 ;HEAP32[$sCtx>>2]=0|0;HEAP32[$sCtx+4>>2]=0|0;HEAP32[$sCtx+8>>2]=0|0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$sCtx>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_fts3ExprIterate($6,156,$sCtx)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = ((($sCtx)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  HEAP32[$12>>2] = $11;
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $15 = ((($sCtx)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _fts3MatchinfoLcs($pCsr,$pInfo) {
 $pCsr = $pCsr|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aIter = 0, $i = 0, $iCol = 0, $nLcs = 0;
 var $nLive = 0, $nThisLcs = 0, $nToken = 0, $pAdv = 0, $pIt = 0, $pIter = 0, $pIter1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $aIter = sp + 44|0;
 $i = sp + 40|0;
 $iCol = sp + 36|0;
 $nToken = sp + 32|0;
 $pIter = sp + 28|0;
 $nLcs = sp + 24|0;
 $nLive = sp + 20|0;
 $rc = sp + 16|0;
 $pIt = sp + 12|0;
 $pAdv = sp + 8|0;
 $nThisLcs = sp + 4|0;
 $pIter1 = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pInfo;
 HEAP32[$nToken>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5<<4;
 $7 = (_sqlite3_malloc($6)|0);
 HEAP32[$aIter>>2] = $7;
 $8 = HEAP32[$aIter>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  HEAP32[$0>>2] = 7;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 $10 = HEAP32[$aIter>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13<<4;
 _memset(($10|0),0,($14|0))|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$aIter>>2]|0;
 (_fts3ExprIterate($17,157,$18)|0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($19|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$aIter>>2]|0;
  $26 = (($25) + ($24<<4)|0);
  HEAP32[$pIter>>2] = $26;
  $27 = HEAP32[$pIter>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 20|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ((($30)) + 64|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$nToken>>2]|0;
  $34 = (($33) - ($32))|0;
  HEAP32[$nToken>>2] = $34;
  $35 = HEAP32[$nToken>>2]|0;
  $36 = HEAP32[$pIter>>2]|0;
  $37 = ((($36)) + 4|0);
  HEAP32[$37>>2] = $35;
  $38 = HEAP32[$i>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[$i>>2] = $39;
 }
 HEAP32[$iCol>>2] = 0;
 L9: while(1) {
  $40 = HEAP32[$iCol>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 4|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   label = 33;
   break;
  }
  HEAP32[$nLcs>>2] = 0;
  HEAP32[$nLive>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $45 = HEAP32[$i>>2]|0;
   $46 = HEAP32[$2>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ($45|0)<($48|0);
   if (!($49)) {
    break;
   }
   $50 = HEAP32[$i>>2]|0;
   $51 = HEAP32[$aIter>>2]|0;
   $52 = (($51) + ($50<<4)|0);
   HEAP32[$pIt>>2] = $52;
   $53 = HEAP32[$1>>2]|0;
   $54 = HEAP32[$pIt>>2]|0;
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[$iCol>>2]|0;
   $57 = HEAP32[$pIt>>2]|0;
   $58 = ((($57)) + 8|0);
   $59 = (_sqlite3Fts3EvalPhrasePoslist($53,$55,$56,$58)|0);
   HEAP32[$rc>>2] = $59;
   $60 = HEAP32[$rc>>2]|0;
   $61 = ($60|0)!=(0);
   if ($61) {
    label = 11;
    break L9;
   }
   $63 = HEAP32[$pIt>>2]|0;
   $64 = ((($63)) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($65|0)!=(0|0);
   if ($66) {
    $67 = HEAP32[$pIt>>2]|0;
    $68 = ((($67)) + 4|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = HEAP32[$pIt>>2]|0;
    $71 = ((($70)) + 12|0);
    HEAP32[$71>>2] = $69;
    $72 = HEAP32[$i>>2]|0;
    $73 = HEAP32[$aIter>>2]|0;
    $74 = (($73) + ($72<<4)|0);
    (_fts3LcsIteratorAdvance($74)|0);
    $75 = HEAP32[$nLive>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$nLive>>2] = $76;
   }
   $77 = HEAP32[$i>>2]|0;
   $78 = (($77) + 1)|0;
   HEAP32[$i>>2] = $78;
  }
  while(1) {
   $79 = HEAP32[$nLive>>2]|0;
   $80 = ($79|0)>(0);
   if (!($80)) {
    break;
   }
   HEAP32[$pAdv>>2] = 0;
   HEAP32[$nThisLcs>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $81 = HEAP32[$i>>2]|0;
    $82 = HEAP32[$2>>2]|0;
    $83 = ((($82)) + 8|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = ($81|0)<($84|0);
    if (!($85)) {
     break;
    }
    $86 = HEAP32[$i>>2]|0;
    $87 = HEAP32[$aIter>>2]|0;
    $88 = (($87) + ($86<<4)|0);
    HEAP32[$pIter1>>2] = $88;
    $89 = HEAP32[$pIter1>>2]|0;
    $90 = ((($89)) + 8|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    if ($92) {
     HEAP32[$nThisLcs>>2] = 0;
    } else {
     $93 = HEAP32[$pAdv>>2]|0;
     $94 = ($93|0)==(0|0);
     if ($94) {
      label = 22;
     } else {
      $95 = HEAP32[$pIter1>>2]|0;
      $96 = ((($95)) + 12|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = HEAP32[$pAdv>>2]|0;
      $99 = ((($98)) + 12|0);
      $100 = HEAP32[$99>>2]|0;
      $101 = ($97|0)<($100|0);
      if ($101) {
       label = 22;
      }
     }
     if ((label|0) == 22) {
      label = 0;
      $102 = HEAP32[$pIter1>>2]|0;
      HEAP32[$pAdv>>2] = $102;
     }
     $103 = HEAP32[$nThisLcs>>2]|0;
     $104 = ($103|0)==(0);
     if ($104) {
      label = 25;
     } else {
      $105 = HEAP32[$pIter1>>2]|0;
      $106 = ((($105)) + 12|0);
      $107 = HEAP32[$106>>2]|0;
      $108 = HEAP32[$pIter1>>2]|0;
      $109 = ((($108)) + -16|0);
      $110 = ((($109)) + 12|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = ($107|0)==($111|0);
      if ($112) {
       label = 25;
      } else {
       HEAP32[$nThisLcs>>2] = 1;
      }
     }
     if ((label|0) == 25) {
      label = 0;
      $113 = HEAP32[$nThisLcs>>2]|0;
      $114 = (($113) + 1)|0;
      HEAP32[$nThisLcs>>2] = $114;
     }
     $115 = HEAP32[$nThisLcs>>2]|0;
     $116 = HEAP32[$nLcs>>2]|0;
     $117 = ($115|0)>($116|0);
     if ($117) {
      $118 = HEAP32[$nThisLcs>>2]|0;
      HEAP32[$nLcs>>2] = $118;
     }
    }
    $119 = HEAP32[$i>>2]|0;
    $120 = (($119) + 1)|0;
    HEAP32[$i>>2] = $120;
   }
   $121 = HEAP32[$pAdv>>2]|0;
   $122 = (_fts3LcsIteratorAdvance($121)|0);
   $123 = ($122|0)!=(0);
   if (!($123)) {
    continue;
   }
   $124 = HEAP32[$nLive>>2]|0;
   $125 = (($124) + -1)|0;
   HEAP32[$nLive>>2] = $125;
  }
  $126 = HEAP32[$nLcs>>2]|0;
  $127 = HEAP32[$iCol>>2]|0;
  $128 = HEAP32[$2>>2]|0;
  $129 = ((($128)) + 28|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = (($130) + ($127<<2)|0);
  HEAP32[$131>>2] = $126;
  $132 = HEAP32[$iCol>>2]|0;
  $133 = (($132) + 1)|0;
  HEAP32[$iCol>>2] = $133;
 }
 if ((label|0) == 11) {
  $62 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $62;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 else if ((label|0) == 33) {
  $134 = HEAP32[$aIter>>2]|0;
  _sqlite3_free($134);
  HEAP32[$0>>2] = 0;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 return (0)|0;
}
function _fts3ExprLHitGather($pExpr,$p) {
 $pExpr = $pExpr|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 32|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 32|0);
 $18 = $17;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($11|0)==($20|0);
 $25 = ($14|0)==($23|0);
 $26 = $24 & $25;
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 $31 = HEAP32[$0>>2]|0;
 if ($30) {
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  _fts3ExprLHitGather($33,$34);
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 16|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  _fts3ExprLHitGather($37,$38);
  STACKTOP = sp;return;
 } else {
  $39 = HEAP32[$1>>2]|0;
  _fts3ExprLHits($31,$39);
  STACKTOP = sp;return;
 }
}
function _fts3ExprGlobalHitsCb($pExpr,$iPhrase,$pCtx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $pCtx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ($7*3)|0;
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = Math_imul($8, $11)|0;
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 28|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<2)|0);
 $17 = (_sqlite3Fts3EvalPhraseStats($5,$6,$16)|0);
 STACKTOP = sp;return ($17|0);
}
function _fts3ExprIterate($pExpr,$x,$pCtx) {
 $pExpr = $pExpr|0;
 $x = $x|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $iPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $iPhrase = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $x;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$iPhrase>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_fts3ExprIterate2($3,$iPhrase,$4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3Fts3EvalTestDeferred($pCsr,$pRc) {
 $pCsr = $pCsr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bMiss = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $bMiss = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pRc;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$rc>>2] = $3;
 HEAP32[$bMiss>>2] = 0;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)==(0);
  $28 = HEAP32[$bMiss>>2]|0;
  $29 = ($28|0)!=(0);
  $30 = $27 ? $29 : 0;
  $31 = $30&1;
  STACKTOP = sp;return ($31|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = (_fts3CursorSeek(0,$10)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (_sqlite3Fts3CacheDeferredDoclists($14)|0);
   HEAP32[$rc>>2] = $15;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_fts3EvalTestExpr($16,$19,$rc)|0);
 $21 = (0)==($20|0);
 $22 = $21&1;
 HEAP32[$bMiss>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 _sqlite3Fts3FreeDeferredDoclists($23);
 $24 = HEAP32[$rc>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 HEAP32[$25>>2] = $24;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)==(0);
 $28 = HEAP32[$bMiss>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = $27 ? $29 : 0;
 $31 = $30&1;
 STACKTOP = sp;return ($31|0);
}
function _fts3ExprLocalHitsCb($pExpr,$iPhrase,$pCtx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iStart = 0, $p = 0, $pCsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $p = sp + 12|0;
 $iStart = sp + 8|0;
 $i = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = Math_imul($4, $7)|0;
 $9 = ($8*3)|0;
 HEAP32[$iStart>>2] = $9;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  $15 = HEAP32[$rc>>2]|0;
  $16 = ($15|0)==(0);
  $17 = $14 ? $16 : 0;
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$p>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$i>>2]|0;
  $22 = (_sqlite3Fts3EvalPhrasePoslist($19,$20,$21,$pCsr)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$pCsr>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = (_fts3ColumnlistCount($pCsr)|0);
   $26 = HEAP32[$iStart>>2]|0;
   $27 = HEAP32[$i>>2]|0;
   $28 = ($27*3)|0;
   $29 = (($26) + ($28))|0;
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 28|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (($32) + ($29<<2)|0);
   HEAP32[$33>>2] = $25;
  } else {
   $34 = HEAP32[$iStart>>2]|0;
   $35 = HEAP32[$i>>2]|0;
   $36 = ($35*3)|0;
   $37 = (($34) + ($36))|0;
   $38 = HEAP32[$p>>2]|0;
   $39 = ((($38)) + 28|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (($40) + ($37<<2)|0);
   HEAP32[$41>>2] = 0;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$i>>2] = $43;
 }
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3Fts3EvalPhrasePoslist($pCsr,$pExpr,$iCol,$ppOut) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $iCol = $iCol|0;
 $ppOut = $ppOut|0;
 var $$old = 0, $$old4 = 0, $$old5 = 0, $$old6 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0;
 var $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0;
 var $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0;
 var $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0;
 var $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0;
 var $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0;
 var $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0;
 var $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0;
 var $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0;
 var $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0;
 var $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0;
 var $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0;
 var $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0;
 var $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0;
 var $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDescDoclist = 0, $bEof = 0, $bEofSave = 0, $bMatch = 0, $bOr = 0, $bTreeEof = 0, $dummy = 0, $iDocid = 0, $iThis = 0, $or$cond = 0, $or$cond7 = 0, $p = 0;
 var $pIter = 0, $pNear = 0, $pPh = 0, $pPhrase = 0, $pTab = 0, $pTest = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $pPhrase = sp + 60|0;
 $pTab = sp + 56|0;
 $pIter = sp + 52|0;
 $iThis = sp + 48|0;
 $iDocid = sp;
 $rc = sp + 44|0;
 $bDescDoclist = sp + 40|0;
 $bOr = sp + 36|0;
 $bTreeEof = sp + 85|0;
 $p = sp + 32|0;
 $pNear = sp + 28|0;
 $bMatch = sp + 24|0;
 $bEofSave = sp + 20|0;
 $bEof = sp + 84|0;
 $pTest = sp + 16|0;
 $pPh = sp + 12|0;
 $dummy = sp + 8|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $iCol;
 HEAP32[$4>>2] = $ppOut;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pPhrase>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTab>>2] = $9;
 $10 = HEAP32[$4>>2]|0;
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pPhrase>>2]|0;
 $12 = ((($11)) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$pTab>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($13|0)<($16|0);
 if ($17) {
  $18 = HEAP32[$pPhrase>>2]|0;
  $19 = ((($18)) + 68|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = ($20|0)!=($21|0);
  if ($22) {
   HEAP32[$0>>2] = 0;
   $363 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($363|0);
  }
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 24|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = $iDocid;
 $32 = $31;
 HEAP32[$32>>2] = $27;
 $33 = (($31) + 4)|0;
 $34 = $33;
 HEAP32[$34>>2] = $30;
 $35 = HEAP32[$pPhrase>>2]|0;
 $36 = ((($35)) + 28|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$pIter>>2] = $37;
 $38 = $iDocid;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 32|0);
 $46 = $45;
 $47 = $46;
 $48 = HEAP32[$47>>2]|0;
 $49 = (($46) + 4)|0;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = ($40|0)!=($48|0);
 $53 = ($43|0)!=($51|0);
 $54 = $52 | $53;
 if ($54) {
  label = 6;
 } else {
  $55 = HEAP32[$2>>2]|0;
  $56 = ((($55)) + 32|0);
  $57 = HEAP8[$56>>0]|0;
  $58 = $57&255;
  $59 = ($58|0)!=(0);
  if ($59) {
   label = 6;
  }
 }
 do {
  if ((label|0) == 6) {
   HEAP32[$rc>>2] = 0;
   $60 = HEAP32[$pTab>>2]|0;
   $61 = ((($60)) + 231|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   HEAP32[$bDescDoclist>>2] = $63;
   HEAP32[$bOr>>2] = 0;
   HEAP8[$bTreeEof>>0] = 0;
   $64 = HEAP32[$2>>2]|0;
   HEAP32[$pNear>>2] = $64;
   $65 = HEAP32[$2>>2]|0;
   $66 = ((($65)) + 8|0);
   $67 = HEAP32[$66>>2]|0;
   HEAP32[$p>>2] = $67;
   while(1) {
    $68 = HEAP32[$p>>2]|0;
    $69 = ($68|0)!=(0|0);
    if (!($69)) {
     break;
    }
    $70 = HEAP32[$p>>2]|0;
    $71 = HEAP32[$70>>2]|0;
    $72 = ($71|0)==(4);
    if ($72) {
     HEAP32[$bOr>>2] = 1;
    }
    $73 = HEAP32[$p>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    $75 = ($74|0)==(1);
    if ($75) {
     $76 = HEAP32[$p>>2]|0;
     HEAP32[$pNear>>2] = $76;
    }
    $77 = HEAP32[$p>>2]|0;
    $78 = ((($77)) + 32|0);
    $79 = HEAP8[$78>>0]|0;
    $80 = ($79<<24>>24)!=(0);
    if ($80) {
     HEAP8[$bTreeEof>>0] = 1;
    }
    $81 = HEAP32[$p>>2]|0;
    $82 = ((($81)) + 8|0);
    $83 = HEAP32[$82>>2]|0;
    HEAP32[$p>>2] = $83;
   }
   $84 = HEAP32[$bOr>>2]|0;
   $85 = ($84|0)==(0);
   if ($85) {
    HEAP32[$0>>2] = 0;
    $363 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($363|0);
   }
   $86 = HEAP32[$pPhrase>>2]|0;
   $87 = ((($86)) + 40|0);
   $88 = HEAP32[$87>>2]|0;
   $89 = ($88|0)!=(0);
   L27: do {
    if ($89) {
     $90 = HEAP32[$pNear>>2]|0;
     $91 = ((($90)) + 32|0);
     $92 = HEAP8[$91>>0]|0;
     $93 = $92&255;
     HEAP32[$bEofSave>>2] = $93;
     $94 = HEAP32[$1>>2]|0;
     $95 = HEAP32[$pNear>>2]|0;
     _fts3EvalRestart($94,$95,$rc);
     L29: while(1) {
      $$old = HEAP32[$rc>>2]|0;
      $$old4 = ($$old|0)==(0);
      if (!($$old4)) {
       break L27;
      }
      while(1) {
       $96 = HEAP32[$pNear>>2]|0;
       $97 = ((($96)) + 32|0);
       $98 = HEAP8[$97>>0]|0;
       $99 = ($98<<24>>24)!=(0);
       $100 = $99 ^ 1;
       if (!($100)) {
        break L27;
       }
       $101 = HEAP32[$1>>2]|0;
       $102 = HEAP32[$pNear>>2]|0;
       _fts3EvalNextRow($101,$102,$rc);
       $103 = HEAP32[$bEofSave>>2]|0;
       $104 = ($103|0)==(0);
       if (!($104)) {
        continue L29;
       }
       $105 = HEAP32[$pNear>>2]|0;
       $106 = ((($105)) + 24|0);
       $107 = $106;
       $108 = $107;
       $109 = HEAP32[$108>>2]|0;
       $110 = (($107) + 4)|0;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = $iDocid;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (($113) + 4)|0;
       $117 = $116;
       $118 = HEAP32[$117>>2]|0;
       $119 = ($109|0)!=($115|0);
       $120 = ($112|0)!=($118|0);
       $121 = $119 | $120;
       $122 = HEAP32[$rc>>2]|0;
       $123 = ($122|0)==(0);
       $or$cond = $121 & $123;
       if (!($or$cond)) {
        break L27;
       }
      }
     }
    }
   } while(0);
   $124 = HEAP8[$bTreeEof>>0]|0;
   $125 = ($124<<24>>24)!=(0);
   $126 = HEAP32[$rc>>2]|0;
   $127 = ($126|0)==(0);
   $or$cond7 = $125 & $127;
   L36: do {
    if ($or$cond7) {
     while(1) {
      $128 = HEAP32[$pNear>>2]|0;
      $129 = ((($128)) + 32|0);
      $130 = HEAP8[$129>>0]|0;
      $131 = ($130<<24>>24)!=(0);
      $132 = $131 ^ 1;
      if (!($132)) {
       break L36;
      }
      $133 = HEAP32[$1>>2]|0;
      $134 = HEAP32[$pNear>>2]|0;
      _fts3EvalNextRow($133,$134,$rc);
      $$old5 = HEAP32[$rc>>2]|0;
      $$old6 = ($$old5|0)==(0);
      if (!($$old6)) {
       break;
      }
     }
    }
   } while(0);
   $135 = HEAP32[$rc>>2]|0;
   $136 = ($135|0)!=(0);
   if ($136) {
    $137 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $137;
    $363 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($363|0);
   }
   HEAP32[$bMatch>>2] = 1;
   $138 = HEAP32[$pNear>>2]|0;
   HEAP32[$p>>2] = $138;
   while(1) {
    $139 = HEAP32[$p>>2]|0;
    $140 = ($139|0)!=(0|0);
    if (!($140)) {
     break;
    }
    HEAP8[$bEof>>0] = 0;
    $141 = HEAP32[$p>>2]|0;
    HEAP32[$pTest>>2] = $141;
    $142 = HEAP32[$pTest>>2]|0;
    $143 = HEAP32[$142>>2]|0;
    $144 = ($143|0)==(1);
    if ($144) {
     $145 = HEAP32[$pTest>>2]|0;
     $146 = ((($145)) + 16|0);
     $147 = HEAP32[$146>>2]|0;
     HEAP32[$pTest>>2] = $147;
    }
    $148 = HEAP32[$pTest>>2]|0;
    $149 = ((($148)) + 20|0);
    $150 = HEAP32[$149>>2]|0;
    HEAP32[$pPh>>2] = $150;
    $151 = HEAP32[$pPh>>2]|0;
    $152 = ((($151)) + 48|0);
    $153 = HEAP32[$152>>2]|0;
    HEAP32[$pIter>>2] = $153;
    $154 = HEAP32[$pPh>>2]|0;
    $155 = ((($154)) + 56|0);
    $156 = $155;
    $157 = $156;
    $158 = HEAP32[$157>>2]|0;
    $159 = (($156) + 4)|0;
    $160 = $159;
    $161 = HEAP32[$160>>2]|0;
    $162 = $iDocid;
    $163 = $162;
    HEAP32[$163>>2] = $158;
    $164 = (($162) + 4)|0;
    $165 = $164;
    HEAP32[$165>>2] = $161;
    $166 = HEAP32[$1>>2]|0;
    $167 = ((($166)) + 52|0);
    $168 = HEAP8[$167>>0]|0;
    $169 = $168&255;
    $170 = HEAP32[$bDescDoclist>>2]|0;
    $171 = ($169|0)==($170|0);
    $172 = HEAP32[$pPh>>2]|0;
    $173 = ((($172)) + 4|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = ($174|0)!=(0);
    L51: do {
     if ($171) {
      if ($175) {
       $176 = HEAP32[$pIter>>2]|0;
       $177 = HEAP32[$pPh>>2]|0;
       $178 = HEAP32[$177>>2]|0;
       $179 = HEAP32[$pPh>>2]|0;
       $180 = ((($179)) + 4|0);
       $181 = HEAP32[$180>>2]|0;
       $182 = (($178) + ($181)|0);
       $183 = ($176>>>0)>=($182>>>0);
       $185 = $183;
      } else {
       $185 = 1;
      }
      $184 = $185&1;
      $186 = $184&255;
      HEAP8[$bEof>>0] = $186;
      while(1) {
       $187 = HEAP32[$pIter>>2]|0;
       $188 = ($187|0)==(0|0);
       if (!($188)) {
        $189 = HEAP32[$bDescDoclist>>2]|0;
        $190 = ($189|0)!=(0);
        $191 = $190 ? -1 : 1;
        $192 = ($191|0)<(0);
        $193 = $192 << 31 >> 31;
        $194 = $iDocid;
        $195 = $194;
        $196 = HEAP32[$195>>2]|0;
        $197 = (($194) + 4)|0;
        $198 = $197;
        $199 = HEAP32[$198>>2]|0;
        $200 = HEAP32[$1>>2]|0;
        $201 = ((($200)) + 32|0);
        $202 = $201;
        $203 = $202;
        $204 = HEAP32[$203>>2]|0;
        $205 = (($202) + 4)|0;
        $206 = $205;
        $207 = HEAP32[$206>>2]|0;
        $208 = (_i64Subtract(($196|0),($199|0),($204|0),($207|0))|0);
        $209 = tempRet0;
        (___muldi3(($191|0),($193|0),($208|0),($209|0))|0);
        $210 = tempRet0;
        $211 = ($210|0)<(0);
        if (!($211)) {
         break L51;
        }
       }
       $212 = HEAP8[$bEof>>0]|0;
       $213 = $212&255;
       $214 = ($213|0)==(0);
       if (!($214)) {
        break L51;
       }
       $215 = HEAP32[$bDescDoclist>>2]|0;
       $216 = HEAP32[$pPh>>2]|0;
       $217 = HEAP32[$216>>2]|0;
       $218 = HEAP32[$pPh>>2]|0;
       $219 = ((($218)) + 4|0);
       $220 = HEAP32[$219>>2]|0;
       _sqlite3Fts3DoclistNext($215,$217,$220,$pIter,$iDocid,$bEof);
      }
     } else {
      if ($175) {
       $221 = HEAP32[$pIter>>2]|0;
       $222 = ($221|0)!=(0|0);
       if ($222) {
        $223 = HEAP32[$pIter>>2]|0;
        $224 = HEAP32[$pPh>>2]|0;
        $225 = HEAP32[$224>>2]|0;
        $226 = ($223>>>0)<=($225>>>0);
        $228 = $226;
       } else {
        $228 = 0;
       }
      } else {
       $228 = 1;
      }
      $227 = $228&1;
      $229 = $227&255;
      HEAP8[$bEof>>0] = $229;
      while(1) {
       $230 = HEAP32[$pIter>>2]|0;
       $231 = ($230|0)==(0|0);
       if (!($231)) {
        $232 = HEAP32[$bDescDoclist>>2]|0;
        $233 = ($232|0)!=(0);
        $234 = $233 ? -1 : 1;
        $235 = ($234|0)<(0);
        $236 = $235 << 31 >> 31;
        $237 = $iDocid;
        $238 = $237;
        $239 = HEAP32[$238>>2]|0;
        $240 = (($237) + 4)|0;
        $241 = $240;
        $242 = HEAP32[$241>>2]|0;
        $243 = HEAP32[$1>>2]|0;
        $244 = ((($243)) + 32|0);
        $245 = $244;
        $246 = $245;
        $247 = HEAP32[$246>>2]|0;
        $248 = (($245) + 4)|0;
        $249 = $248;
        $250 = HEAP32[$249>>2]|0;
        $251 = (_i64Subtract(($239|0),($242|0),($247|0),($250|0))|0);
        $252 = tempRet0;
        $253 = (___muldi3(($234|0),($236|0),($251|0),($252|0))|0);
        $254 = tempRet0;
        $255 = ($254|0)>(0);
        $256 = ($253>>>0)>(0);
        $257 = ($254|0)==(0);
        $258 = $257 & $256;
        $259 = $255 | $258;
        if (!($259)) {
         break L51;
        }
       }
       $260 = HEAP8[$bEof>>0]|0;
       $261 = $260&255;
       $262 = ($261|0)==(0);
       if (!($262)) {
        break L51;
       }
       $263 = HEAP32[$bDescDoclist>>2]|0;
       $264 = HEAP32[$pPh>>2]|0;
       $265 = HEAP32[$264>>2]|0;
       $266 = HEAP32[$pPh>>2]|0;
       $267 = ((($266)) + 4|0);
       $268 = HEAP32[$267>>2]|0;
       _sqlite3Fts3DoclistPrev($263,$265,$268,$pIter,$iDocid,$dummy,$bEof);
      }
     }
    } while(0);
    $269 = HEAP32[$pIter>>2]|0;
    $270 = HEAP32[$pPh>>2]|0;
    $271 = ((($270)) + 48|0);
    HEAP32[$271>>2] = $269;
    $272 = $iDocid;
    $273 = $272;
    $274 = HEAP32[$273>>2]|0;
    $275 = (($272) + 4)|0;
    $276 = $275;
    $277 = HEAP32[$276>>2]|0;
    $278 = HEAP32[$pPh>>2]|0;
    $279 = ((($278)) + 56|0);
    $280 = $279;
    $281 = $280;
    HEAP32[$281>>2] = $274;
    $282 = (($280) + 4)|0;
    $283 = $282;
    HEAP32[$283>>2] = $277;
    $284 = HEAP8[$bEof>>0]|0;
    $285 = $284&255;
    $286 = ($285|0)!=(0);
    if ($286) {
     label = 50;
    } else {
     $287 = $iDocid;
     $288 = $287;
     $289 = HEAP32[$288>>2]|0;
     $290 = (($287) + 4)|0;
     $291 = $290;
     $292 = HEAP32[$291>>2]|0;
     $293 = HEAP32[$1>>2]|0;
     $294 = ((($293)) + 32|0);
     $295 = $294;
     $296 = $295;
     $297 = HEAP32[$296>>2]|0;
     $298 = (($295) + 4)|0;
     $299 = $298;
     $300 = HEAP32[$299>>2]|0;
     $301 = ($289|0)!=($297|0);
     $302 = ($292|0)!=($300|0);
     $303 = $301 | $302;
     if ($303) {
      label = 50;
     }
    }
    if ((label|0) == 50) {
     label = 0;
     HEAP32[$bMatch>>2] = 0;
    }
    $304 = HEAP32[$p>>2]|0;
    $305 = ((($304)) + 12|0);
    $306 = HEAP32[$305>>2]|0;
    HEAP32[$p>>2] = $306;
   }
   $307 = HEAP32[$bMatch>>2]|0;
   $308 = ($307|0)!=(0);
   if ($308) {
    $309 = HEAP32[$pPhrase>>2]|0;
    $310 = ((($309)) + 48|0);
    $311 = HEAP32[$310>>2]|0;
    HEAP32[$pIter>>2] = $311;
    break;
   } else {
    HEAP32[$pIter>>2] = 0;
    break;
   }
  }
 } while(0);
 $312 = HEAP32[$pIter>>2]|0;
 $313 = ($312|0)==(0|0);
 if ($313) {
  HEAP32[$0>>2] = 0;
  $363 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($363|0);
 }
 $314 = HEAP32[$pIter>>2]|0;
 $315 = HEAP8[$314>>0]|0;
 $316 = $315 << 24 >> 24;
 $317 = ($316|0)==(1);
 if ($317) {
  $318 = HEAP32[$pIter>>2]|0;
  $319 = ((($318)) + 1|0);
  HEAP32[$pIter>>2] = $319;
  $320 = HEAP32[$pIter>>2]|0;
  $321 = HEAP8[$320>>0]|0;
  $322 = $321&255;
  $323 = $322 & 128;
  $324 = ($323|0)!=(0);
  $325 = HEAP32[$pIter>>2]|0;
  if ($324) {
   $326 = (_sqlite3Fts3GetVarint32($325,$iThis)|0);
   $331 = $326;
  } else {
   $327 = HEAP8[$325>>0]|0;
   $328 = $327&255;
   HEAP32[$iThis>>2] = $328;
   $331 = 1;
  }
  $329 = HEAP32[$pIter>>2]|0;
  $330 = (($329) + ($331)|0);
  HEAP32[$pIter>>2] = $330;
 } else {
  HEAP32[$iThis>>2] = 0;
 }
 while(1) {
  $332 = HEAP32[$iThis>>2]|0;
  $333 = HEAP32[$3>>2]|0;
  $334 = ($332|0)<($333|0);
  if (!($334)) {
   break;
  }
  _fts3ColumnlistCopy(0,$pIter);
  $335 = HEAP32[$pIter>>2]|0;
  $336 = HEAP8[$335>>0]|0;
  $337 = $336 << 24 >> 24;
  $338 = ($337|0)==(0);
  if ($338) {
   label = 65;
   break;
  }
  $339 = HEAP32[$pIter>>2]|0;
  $340 = ((($339)) + 1|0);
  HEAP32[$pIter>>2] = $340;
  $341 = HEAP32[$pIter>>2]|0;
  $342 = HEAP8[$341>>0]|0;
  $343 = $342&255;
  $344 = $343 & 128;
  $345 = ($344|0)!=(0);
  $346 = HEAP32[$pIter>>2]|0;
  if ($345) {
   $347 = (_sqlite3Fts3GetVarint32($346,$iThis)|0);
   $352 = $347;
  } else {
   $348 = HEAP8[$346>>0]|0;
   $349 = $348&255;
   HEAP32[$iThis>>2] = $349;
   $352 = 1;
  }
  $350 = HEAP32[$pIter>>2]|0;
  $351 = (($350) + ($352)|0);
  HEAP32[$pIter>>2] = $351;
 }
 if ((label|0) == 65) {
  HEAP32[$0>>2] = 0;
  $363 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($363|0);
 }
 $353 = HEAP32[$pIter>>2]|0;
 $354 = HEAP8[$353>>0]|0;
 $355 = $354 << 24 >> 24;
 $356 = ($355|0)==(0);
 if ($356) {
  HEAP32[$pIter>>2] = 0;
 }
 $357 = HEAP32[$3>>2]|0;
 $358 = HEAP32[$iThis>>2]|0;
 $359 = ($357|0)==($358|0);
 $360 = HEAP32[$pIter>>2]|0;
 $361 = $359 ? $360 : 0;
 $362 = HEAP32[$4>>2]|0;
 HEAP32[$362>>2] = $361;
 HEAP32[$0>>2] = 0;
 $363 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($363|0);
}
function _fts3ColumnlistCount($ppCollist) {
 $ppCollist = $ppCollist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $c = 0, $nEntry = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pEnd = sp + 4|0;
 $c = sp + 12|0;
 $nEntry = sp;
 HEAP32[$0>>2] = $ppCollist;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pEnd>>2] = $2;
 HEAP8[$c>>0] = 0;
 HEAP32[$nEntry>>2] = 0;
 while(1) {
  $3 = HEAP32[$pEnd>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = HEAP8[$c>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = $5 | $7;
  $9 = 254 & $8;
  $10 = ($9|0)!=(0);
  $11 = HEAP32[$pEnd>>2]|0;
  if (!($10)) {
   break;
  }
  $12 = ((($11)) + 1|0);
  HEAP32[$pEnd>>2] = $12;
  $13 = HEAP8[$11>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = $14 & 128;
  $16 = $15&255;
  HEAP8[$c>>0] = $16;
  $17 = HEAP8[$c>>0]|0;
  $18 = ($17<<24>>24)!=(0);
  if ($18) {
   continue;
  }
  $19 = HEAP32[$nEntry>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$nEntry>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 HEAP32[$21>>2] = $11;
 $22 = HEAP32[$nEntry>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _fts3EvalRestart($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pPhrase = 0, $pToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pPhrase = sp + 8|0;
 $i = sp + 4|0;
 $pToken = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pPhrase>>2] = $10;
 $11 = HEAP32[$pPhrase>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$pPhrase>>2]|0;
  _fts3EvalInvalidatePoslist($13);
  $14 = HEAP32[$pPhrase>>2]|0;
  $15 = ((($14)) + 40|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $18 = HEAP32[$i>>2]|0;
    $19 = HEAP32[$pPhrase>>2]|0;
    $20 = ((($19)) + 64|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($18|0)<($21|0);
    if (!($22)) {
     break;
    }
    $23 = HEAP32[$i>>2]|0;
    $24 = HEAP32[$pPhrase>>2]|0;
    $25 = ((($24)) + 72|0);
    $26 = (($25) + (($23*24)|0)|0);
    HEAP32[$pToken>>2] = $26;
    $27 = HEAP32[$pToken>>2]|0;
    $28 = ((($27)) + 20|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($29|0)!=(0|0);
    if ($30) {
     $31 = HEAP32[$pToken>>2]|0;
     $32 = ((($31)) + 20|0);
     $33 = HEAP32[$32>>2]|0;
     (_sqlite3Fts3MsrIncrRestart($33)|0);
    }
    $34 = HEAP32[$i>>2]|0;
    $35 = (($34) + 1)|0;
    HEAP32[$i>>2] = $35;
   }
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$pPhrase>>2]|0;
   $38 = (_fts3EvalPhraseStart($36,0,$37)|0);
   $39 = HEAP32[$2>>2]|0;
   HEAP32[$39>>2] = $38;
  }
  $40 = HEAP32[$pPhrase>>2]|0;
  $41 = ((($40)) + 8|0);
  HEAP32[$41>>2] = 0;
  $42 = HEAP32[$pPhrase>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = $43;
  $45 = $44;
  HEAP32[$45>>2] = 0;
  $46 = (($44) + 4)|0;
  $47 = $46;
  HEAP32[$47>>2] = 0;
  $48 = HEAP32[$pPhrase>>2]|0;
  $49 = ((($48)) + 48|0);
  HEAP32[$49>>2] = 0;
 }
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = $51;
 $53 = $52;
 HEAP32[$53>>2] = 0;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = 0;
 $56 = HEAP32[$1>>2]|0;
 $57 = ((($56)) + 32|0);
 HEAP8[$57>>0] = 0;
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 33|0);
 HEAP8[$59>>0] = 0;
 $60 = HEAP32[$0>>2]|0;
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 12|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = HEAP32[$2>>2]|0;
 _fts3EvalRestart($60,$63,$64);
 $65 = HEAP32[$0>>2]|0;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 16|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = HEAP32[$2>>2]|0;
 _fts3EvalRestart($65,$68,$69);
 STACKTOP = sp;return;
}
function _fts3EvalNextRow($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDescDoclist = 0, $iCmp = 0, $iDiff = 0, $or$cond = 0, $or$cond6 = 0, $or$cond8 = 0, $pDl = 0, $pDl1 = 0, $pLeft = 0, $pLeft2 = 0;
 var $pLeft4 = 0, $pPhrase = 0, $pRight = 0, $pRight3 = 0, $pRight5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $bDescDoclist = sp + 52|0;
 $pLeft = sp + 48|0;
 $pRight = sp + 44|0;
 $iDiff = sp + 8|0;
 $pDl = sp + 40|0;
 $pDl1 = sp + 36|0;
 $pLeft2 = sp + 32|0;
 $pRight3 = sp + 28|0;
 $iCmp = sp;
 $pLeft4 = sp + 24|0;
 $pRight5 = sp + 20|0;
 $pPhrase = sp + 16|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 HEAP32[$bDescDoclist>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 33|0);
 HEAP8[$11>>0] = 1;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 switch ($13|0) {
 case 3: case 1:  {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pLeft>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pRight>>2] = $19;
  $20 = HEAP32[$pLeft>>2]|0;
  $21 = ((($20)) + 34|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = ($22<<24>>24)!=(0);
  if ($23) {
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$pRight>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($24,$25,$26);
   $27 = HEAP32[$pRight>>2]|0;
   $28 = ((($27)) + 24|0);
   $29 = $28;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($29) + 4)|0;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 24|0);
   $37 = $36;
   $38 = $37;
   HEAP32[$38>>2] = $31;
   $39 = (($37) + 4)|0;
   $40 = $39;
   HEAP32[$40>>2] = $34;
   $41 = HEAP32[$pRight>>2]|0;
   $42 = ((($41)) + 32|0);
   $43 = HEAP8[$42>>0]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = ((($44)) + 32|0);
   HEAP8[$45>>0] = $43;
   STACKTOP = sp;return;
  }
  $46 = HEAP32[$pRight>>2]|0;
  $47 = ((($46)) + 34|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = ($48<<24>>24)!=(0);
  $50 = HEAP32[$0>>2]|0;
  $51 = HEAP32[$pLeft>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  _fts3EvalNextRow($50,$51,$52);
  if ($49) {
   $53 = HEAP32[$pLeft>>2]|0;
   $54 = ((($53)) + 24|0);
   $55 = $54;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($55) + 4)|0;
   $59 = $58;
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$1>>2]|0;
   $62 = ((($61)) + 24|0);
   $63 = $62;
   $64 = $63;
   HEAP32[$64>>2] = $57;
   $65 = (($63) + 4)|0;
   $66 = $65;
   HEAP32[$66>>2] = $60;
   $67 = HEAP32[$pLeft>>2]|0;
   $68 = ((($67)) + 32|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = HEAP32[$1>>2]|0;
   $71 = ((($70)) + 32|0);
   HEAP8[$71>>0] = $69;
   STACKTOP = sp;return;
  }
  $72 = HEAP32[$0>>2]|0;
  $73 = HEAP32[$pRight>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  _fts3EvalNextRow($72,$73,$74);
  while(1) {
   $75 = HEAP32[$pLeft>>2]|0;
   $76 = ((($75)) + 32|0);
   $77 = HEAP8[$76>>0]|0;
   $78 = ($77<<24>>24)!=(0);
   if ($78) {
    break;
   }
   $79 = HEAP32[$pRight>>2]|0;
   $80 = ((($79)) + 32|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = ($81<<24>>24)!=(0);
   if ($82) {
    break;
   }
   $83 = HEAP32[$2>>2]|0;
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)==(0);
   if (!($85)) {
    break;
   }
   $86 = HEAP32[$bDescDoclist>>2]|0;
   $87 = ($86|0)!=(0);
   $88 = $87 ? -1 : 1;
   $89 = ($88|0)<(0);
   $90 = $89 << 31 >> 31;
   $91 = HEAP32[$pLeft>>2]|0;
   $92 = ((($91)) + 24|0);
   $93 = $92;
   $94 = $93;
   $95 = HEAP32[$94>>2]|0;
   $96 = (($93) + 4)|0;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$pRight>>2]|0;
   $100 = ((($99)) + 24|0);
   $101 = $100;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = (($101) + 4)|0;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = (_i64Subtract(($95|0),($98|0),($103|0),($106|0))|0);
   $108 = tempRet0;
   $109 = (___muldi3(($88|0),($90|0),($107|0),($108|0))|0);
   $110 = tempRet0;
   $111 = $iDiff;
   $112 = $111;
   HEAP32[$112>>2] = $109;
   $113 = (($111) + 4)|0;
   $114 = $113;
   HEAP32[$114>>2] = $110;
   $115 = $iDiff;
   $116 = $115;
   $117 = HEAP32[$116>>2]|0;
   $118 = (($115) + 4)|0;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = ($117|0)==(0);
   $122 = ($120|0)==(0);
   $123 = $121 & $122;
   if ($123) {
    break;
   }
   $124 = $iDiff;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($124) + 4)|0;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = ($129|0)<(0);
   $131 = HEAP32[$0>>2]|0;
   if ($130) {
    $132 = HEAP32[$pLeft>>2]|0;
    $133 = HEAP32[$2>>2]|0;
    _fts3EvalNextRow($131,$132,$133);
    continue;
   } else {
    $134 = HEAP32[$pRight>>2]|0;
    $135 = HEAP32[$2>>2]|0;
    _fts3EvalNextRow($131,$134,$135);
    continue;
   }
  }
  $136 = HEAP32[$pLeft>>2]|0;
  $137 = ((($136)) + 24|0);
  $138 = $137;
  $139 = $138;
  $140 = HEAP32[$139>>2]|0;
  $141 = (($138) + 4)|0;
  $142 = $141;
  $143 = HEAP32[$142>>2]|0;
  $144 = HEAP32[$1>>2]|0;
  $145 = ((($144)) + 24|0);
  $146 = $145;
  $147 = $146;
  HEAP32[$147>>2] = $140;
  $148 = (($146) + 4)|0;
  $149 = $148;
  HEAP32[$149>>2] = $143;
  $150 = HEAP32[$pLeft>>2]|0;
  $151 = ((($150)) + 32|0);
  $152 = HEAP8[$151>>0]|0;
  $153 = $152&255;
  $154 = ($153|0)!=(0);
  if ($154) {
   $161 = 1;
  } else {
   $155 = HEAP32[$pRight>>2]|0;
   $156 = ((($155)) + 32|0);
   $157 = HEAP8[$156>>0]|0;
   $158 = $157&255;
   $159 = ($158|0)!=(0);
   $161 = $159;
  }
  $160 = $161&1;
  $162 = $160&255;
  $163 = HEAP32[$1>>2]|0;
  $164 = ((($163)) + 32|0);
  HEAP8[$164>>0] = $162;
  $165 = HEAP32[$1>>2]|0;
  $166 = HEAP32[$165>>2]|0;
  $167 = ($166|0)==(1);
  if (!($167)) {
   STACKTOP = sp;return;
  }
  $168 = HEAP32[$1>>2]|0;
  $169 = ((($168)) + 32|0);
  $170 = HEAP8[$169>>0]|0;
  $171 = $170&255;
  $172 = ($171|0)!=(0);
  if (!($172)) {
   STACKTOP = sp;return;
  }
  $173 = HEAP32[$pRight>>2]|0;
  $174 = ((($173)) + 20|0);
  $175 = HEAP32[$174>>2]|0;
  $176 = ($175|0)!=(0|0);
  L35: do {
   if ($176) {
    $177 = HEAP32[$pRight>>2]|0;
    $178 = ((($177)) + 20|0);
    $179 = HEAP32[$178>>2]|0;
    $180 = HEAP32[$179>>2]|0;
    $181 = ($180|0)!=(0|0);
    if ($181) {
     $182 = HEAP32[$pRight>>2]|0;
     $183 = ((($182)) + 20|0);
     $184 = HEAP32[$183>>2]|0;
     HEAP32[$pDl>>2] = $184;
     while(1) {
      $185 = HEAP32[$2>>2]|0;
      $186 = HEAP32[$185>>2]|0;
      $187 = ($186|0)==(0);
      if (!($187)) {
       break L35;
      }
      $188 = HEAP32[$pRight>>2]|0;
      $189 = ((($188)) + 32|0);
      $190 = HEAP8[$189>>0]|0;
      $191 = $190&255;
      $192 = ($191|0)==(0);
      if (!($192)) {
       break L35;
      }
      $193 = HEAP32[$pDl>>2]|0;
      $194 = ((($193)) + 28|0);
      $195 = HEAP32[$194>>2]|0;
      $196 = HEAP32[$pDl>>2]|0;
      $197 = ((($196)) + 32|0);
      $198 = HEAP32[$197>>2]|0;
      _memset(($195|0),0,($198|0))|0;
      $199 = HEAP32[$0>>2]|0;
      $200 = HEAP32[$pRight>>2]|0;
      $201 = HEAP32[$2>>2]|0;
      _fts3EvalNextRow($199,$200,$201);
     }
    }
   }
  } while(0);
  $202 = HEAP32[$pLeft>>2]|0;
  $203 = ((($202)) + 20|0);
  $204 = HEAP32[$203>>2]|0;
  $205 = ($204|0)!=(0|0);
  if (!($205)) {
   STACKTOP = sp;return;
  }
  $206 = HEAP32[$pLeft>>2]|0;
  $207 = ((($206)) + 20|0);
  $208 = HEAP32[$207>>2]|0;
  $209 = HEAP32[$208>>2]|0;
  $210 = ($209|0)!=(0|0);
  if (!($210)) {
   STACKTOP = sp;return;
  }
  $211 = HEAP32[$pLeft>>2]|0;
  $212 = ((($211)) + 20|0);
  $213 = HEAP32[$212>>2]|0;
  HEAP32[$pDl1>>2] = $213;
  while(1) {
   $214 = HEAP32[$2>>2]|0;
   $215 = HEAP32[$214>>2]|0;
   $216 = ($215|0)==(0);
   if (!($216)) {
    label = 53;
    break;
   }
   $217 = HEAP32[$pLeft>>2]|0;
   $218 = ((($217)) + 32|0);
   $219 = HEAP8[$218>>0]|0;
   $220 = $219&255;
   $221 = ($220|0)==(0);
   if (!($221)) {
    label = 53;
    break;
   }
   $222 = HEAP32[$pDl1>>2]|0;
   $223 = ((($222)) + 28|0);
   $224 = HEAP32[$223>>2]|0;
   $225 = HEAP32[$pDl1>>2]|0;
   $226 = ((($225)) + 32|0);
   $227 = HEAP32[$226>>2]|0;
   _memset(($224|0),0,($227|0))|0;
   $228 = HEAP32[$0>>2]|0;
   $229 = HEAP32[$pLeft>>2]|0;
   $230 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($228,$229,$230);
  }
  if ((label|0) == 53) {
   STACKTOP = sp;return;
  }
  break;
 }
 case 4:  {
  $231 = HEAP32[$1>>2]|0;
  $232 = ((($231)) + 12|0);
  $233 = HEAP32[$232>>2]|0;
  HEAP32[$pLeft2>>2] = $233;
  $234 = HEAP32[$1>>2]|0;
  $235 = ((($234)) + 16|0);
  $236 = HEAP32[$235>>2]|0;
  HEAP32[$pRight3>>2] = $236;
  $237 = HEAP32[$bDescDoclist>>2]|0;
  $238 = ($237|0)!=(0);
  $239 = $238 ? -1 : 1;
  $240 = ($239|0)<(0);
  $241 = $240 << 31 >> 31;
  $242 = HEAP32[$pLeft2>>2]|0;
  $243 = ((($242)) + 24|0);
  $244 = $243;
  $245 = $244;
  $246 = HEAP32[$245>>2]|0;
  $247 = (($244) + 4)|0;
  $248 = $247;
  $249 = HEAP32[$248>>2]|0;
  $250 = HEAP32[$pRight3>>2]|0;
  $251 = ((($250)) + 24|0);
  $252 = $251;
  $253 = $252;
  $254 = HEAP32[$253>>2]|0;
  $255 = (($252) + 4)|0;
  $256 = $255;
  $257 = HEAP32[$256>>2]|0;
  $258 = (_i64Subtract(($246|0),($249|0),($254|0),($257|0))|0);
  $259 = tempRet0;
  $260 = (___muldi3(($239|0),($241|0),($258|0),($259|0))|0);
  $261 = tempRet0;
  $262 = $iCmp;
  $263 = $262;
  HEAP32[$263>>2] = $260;
  $264 = (($262) + 4)|0;
  $265 = $264;
  HEAP32[$265>>2] = $261;
  $266 = HEAP32[$pRight3>>2]|0;
  $267 = ((($266)) + 32|0);
  $268 = HEAP8[$267>>0]|0;
  $269 = $268&255;
  $270 = ($269|0)!=(0);
  do {
   if ($270) {
    label = 33;
   } else {
    $271 = HEAP32[$pLeft2>>2]|0;
    $272 = ((($271)) + 32|0);
    $273 = HEAP8[$272>>0]|0;
    $274 = $273&255;
    $275 = ($274|0)==(0);
    $276 = $iCmp;
    $277 = $276;
    $278 = HEAP32[$277>>2]|0;
    $279 = (($276) + 4)|0;
    $280 = $279;
    $281 = HEAP32[$280>>2]|0;
    $282 = ($281|0)<(0);
    $or$cond = $275 & $282;
    if ($or$cond) {
     label = 33;
    } else {
     $286 = HEAP32[$pLeft2>>2]|0;
     $287 = ((($286)) + 32|0);
     $288 = HEAP8[$287>>0]|0;
     $289 = $288&255;
     $290 = ($289|0)!=(0);
     if (!($290)) {
      $291 = HEAP32[$pRight3>>2]|0;
      $292 = ((($291)) + 32|0);
      $293 = HEAP8[$292>>0]|0;
      $294 = $293&255;
      $295 = ($294|0)==(0);
      $296 = $iCmp;
      $297 = $296;
      $298 = HEAP32[$297>>2]|0;
      $299 = (($296) + 4)|0;
      $300 = $299;
      $301 = HEAP32[$300>>2]|0;
      $302 = ($301|0)>(0);
      $303 = ($298>>>0)>(0);
      $304 = ($301|0)==(0);
      $305 = $304 & $303;
      $306 = $302 | $305;
      $or$cond6 = $295 & $306;
      if (!($or$cond6)) {
       $310 = HEAP32[$0>>2]|0;
       $311 = HEAP32[$pLeft2>>2]|0;
       $312 = HEAP32[$2>>2]|0;
       _fts3EvalNextRow($310,$311,$312);
       $313 = HEAP32[$0>>2]|0;
       $314 = HEAP32[$pRight3>>2]|0;
       $315 = HEAP32[$2>>2]|0;
       _fts3EvalNextRow($313,$314,$315);
       break;
      }
     }
     $307 = HEAP32[$0>>2]|0;
     $308 = HEAP32[$pRight3>>2]|0;
     $309 = HEAP32[$2>>2]|0;
     _fts3EvalNextRow($307,$308,$309);
    }
   }
  } while(0);
  if ((label|0) == 33) {
   $283 = HEAP32[$0>>2]|0;
   $284 = HEAP32[$pLeft2>>2]|0;
   $285 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($283,$284,$285);
  }
  $316 = HEAP32[$pLeft2>>2]|0;
  $317 = ((($316)) + 32|0);
  $318 = HEAP8[$317>>0]|0;
  $319 = $318&255;
  $320 = ($319|0)!=(0);
  if ($320) {
   $321 = HEAP32[$pRight3>>2]|0;
   $322 = ((($321)) + 32|0);
   $323 = HEAP8[$322>>0]|0;
   $324 = $323&255;
   $325 = ($324|0)!=(0);
   $327 = $325;
  } else {
   $327 = 0;
  }
  $326 = $327&1;
  $328 = $326&255;
  $329 = HEAP32[$1>>2]|0;
  $330 = ((($329)) + 32|0);
  HEAP8[$330>>0] = $328;
  $331 = HEAP32[$bDescDoclist>>2]|0;
  $332 = ($331|0)!=(0);
  $333 = $332 ? -1 : 1;
  $334 = ($333|0)<(0);
  $335 = $334 << 31 >> 31;
  $336 = HEAP32[$pLeft2>>2]|0;
  $337 = ((($336)) + 24|0);
  $338 = $337;
  $339 = $338;
  $340 = HEAP32[$339>>2]|0;
  $341 = (($338) + 4)|0;
  $342 = $341;
  $343 = HEAP32[$342>>2]|0;
  $344 = HEAP32[$pRight3>>2]|0;
  $345 = ((($344)) + 24|0);
  $346 = $345;
  $347 = $346;
  $348 = HEAP32[$347>>2]|0;
  $349 = (($346) + 4)|0;
  $350 = $349;
  $351 = HEAP32[$350>>2]|0;
  $352 = (_i64Subtract(($340|0),($343|0),($348|0),($351|0))|0);
  $353 = tempRet0;
  $354 = (___muldi3(($333|0),($335|0),($352|0),($353|0))|0);
  $355 = tempRet0;
  $356 = $iCmp;
  $357 = $356;
  HEAP32[$357>>2] = $354;
  $358 = (($356) + 4)|0;
  $359 = $358;
  HEAP32[$359>>2] = $355;
  $360 = HEAP32[$pRight3>>2]|0;
  $361 = ((($360)) + 32|0);
  $362 = HEAP8[$361>>0]|0;
  $363 = $362&255;
  $364 = ($363|0)!=(0);
  if (!($364)) {
   $365 = HEAP32[$pLeft2>>2]|0;
   $366 = ((($365)) + 32|0);
   $367 = HEAP8[$366>>0]|0;
   $368 = $367&255;
   $369 = ($368|0)==(0);
   $370 = $iCmp;
   $371 = $370;
   $372 = HEAP32[$371>>2]|0;
   $373 = (($370) + 4)|0;
   $374 = $373;
   $375 = HEAP32[$374>>2]|0;
   $376 = ($375|0)<(0);
   $or$cond8 = $369 & $376;
   if (!($or$cond8)) {
    $391 = HEAP32[$pRight3>>2]|0;
    $392 = ((($391)) + 24|0);
    $393 = $392;
    $394 = $393;
    $395 = HEAP32[$394>>2]|0;
    $396 = (($393) + 4)|0;
    $397 = $396;
    $398 = HEAP32[$397>>2]|0;
    $399 = HEAP32[$1>>2]|0;
    $400 = ((($399)) + 24|0);
    $401 = $400;
    $402 = $401;
    HEAP32[$402>>2] = $395;
    $403 = (($401) + 4)|0;
    $404 = $403;
    HEAP32[$404>>2] = $398;
    STACKTOP = sp;return;
   }
  }
  $377 = HEAP32[$pLeft2>>2]|0;
  $378 = ((($377)) + 24|0);
  $379 = $378;
  $380 = $379;
  $381 = HEAP32[$380>>2]|0;
  $382 = (($379) + 4)|0;
  $383 = $382;
  $384 = HEAP32[$383>>2]|0;
  $385 = HEAP32[$1>>2]|0;
  $386 = ((($385)) + 24|0);
  $387 = $386;
  $388 = $387;
  HEAP32[$388>>2] = $381;
  $389 = (($387) + 4)|0;
  $390 = $389;
  HEAP32[$390>>2] = $384;
  STACKTOP = sp;return;
  break;
 }
 case 2:  {
  $405 = HEAP32[$1>>2]|0;
  $406 = ((($405)) + 12|0);
  $407 = HEAP32[$406>>2]|0;
  HEAP32[$pLeft4>>2] = $407;
  $408 = HEAP32[$1>>2]|0;
  $409 = ((($408)) + 16|0);
  $410 = HEAP32[$409>>2]|0;
  HEAP32[$pRight5>>2] = $410;
  $411 = HEAP32[$pRight5>>2]|0;
  $412 = ((($411)) + 33|0);
  $413 = HEAP8[$412>>0]|0;
  $414 = $413&255;
  $415 = ($414|0)==(0);
  if ($415) {
   $416 = HEAP32[$0>>2]|0;
   $417 = HEAP32[$pRight5>>2]|0;
   $418 = HEAP32[$2>>2]|0;
   _fts3EvalNextRow($416,$417,$418);
  }
  $419 = HEAP32[$0>>2]|0;
  $420 = HEAP32[$pLeft4>>2]|0;
  $421 = HEAP32[$2>>2]|0;
  _fts3EvalNextRow($419,$420,$421);
  $422 = HEAP32[$pLeft4>>2]|0;
  $423 = ((($422)) + 32|0);
  $424 = HEAP8[$423>>0]|0;
  $425 = $424&255;
  $426 = ($425|0)==(0);
  L80: do {
   if ($426) {
    while(1) {
     $427 = HEAP32[$2>>2]|0;
     $428 = HEAP32[$427>>2]|0;
     $429 = ($428|0)!=(0);
     if ($429) {
      break L80;
     }
     $430 = HEAP32[$pRight5>>2]|0;
     $431 = ((($430)) + 32|0);
     $432 = HEAP8[$431>>0]|0;
     $433 = ($432<<24>>24)!=(0);
     if ($433) {
      break L80;
     }
     $434 = HEAP32[$bDescDoclist>>2]|0;
     $435 = ($434|0)!=(0);
     $436 = $435 ? -1 : 1;
     $437 = ($436|0)<(0);
     $438 = $437 << 31 >> 31;
     $439 = HEAP32[$pLeft4>>2]|0;
     $440 = ((($439)) + 24|0);
     $441 = $440;
     $442 = $441;
     $443 = HEAP32[$442>>2]|0;
     $444 = (($441) + 4)|0;
     $445 = $444;
     $446 = HEAP32[$445>>2]|0;
     $447 = HEAP32[$pRight5>>2]|0;
     $448 = ((($447)) + 24|0);
     $449 = $448;
     $450 = $449;
     $451 = HEAP32[$450>>2]|0;
     $452 = (($449) + 4)|0;
     $453 = $452;
     $454 = HEAP32[$453>>2]|0;
     $455 = (_i64Subtract(($443|0),($446|0),($451|0),($454|0))|0);
     $456 = tempRet0;
     $457 = (___muldi3(($436|0),($438|0),($455|0),($456|0))|0);
     $458 = tempRet0;
     $459 = ($458|0)>(0);
     $460 = ($457>>>0)>(0);
     $461 = ($458|0)==(0);
     $462 = $461 & $460;
     $463 = $459 | $462;
     if (!($463)) {
      break L80;
     }
     $464 = HEAP32[$0>>2]|0;
     $465 = HEAP32[$pRight5>>2]|0;
     $466 = HEAP32[$2>>2]|0;
     _fts3EvalNextRow($464,$465,$466);
    }
   }
  } while(0);
  $467 = HEAP32[$pLeft4>>2]|0;
  $468 = ((($467)) + 24|0);
  $469 = $468;
  $470 = $469;
  $471 = HEAP32[$470>>2]|0;
  $472 = (($469) + 4)|0;
  $473 = $472;
  $474 = HEAP32[$473>>2]|0;
  $475 = HEAP32[$1>>2]|0;
  $476 = ((($475)) + 24|0);
  $477 = $476;
  $478 = $477;
  HEAP32[$478>>2] = $471;
  $479 = (($477) + 4)|0;
  $480 = $479;
  HEAP32[$480>>2] = $474;
  $481 = HEAP32[$pLeft4>>2]|0;
  $482 = ((($481)) + 32|0);
  $483 = HEAP8[$482>>0]|0;
  $484 = HEAP32[$1>>2]|0;
  $485 = ((($484)) + 32|0);
  HEAP8[$485>>0] = $483;
  STACKTOP = sp;return;
  break;
 }
 default: {
  $486 = HEAP32[$1>>2]|0;
  $487 = ((($486)) + 20|0);
  $488 = HEAP32[$487>>2]|0;
  HEAP32[$pPhrase>>2] = $488;
  $489 = HEAP32[$pPhrase>>2]|0;
  _fts3EvalInvalidatePoslist($489);
  $490 = HEAP32[$0>>2]|0;
  $491 = HEAP32[$pPhrase>>2]|0;
  $492 = HEAP32[$1>>2]|0;
  $493 = ((($492)) + 32|0);
  $494 = (_fts3EvalPhraseNext($490,$491,$493)|0);
  $495 = HEAP32[$2>>2]|0;
  HEAP32[$495>>2] = $494;
  $496 = HEAP32[$pPhrase>>2]|0;
  $497 = ((($496)) + 16|0);
  $498 = $497;
  $499 = $498;
  $500 = HEAP32[$499>>2]|0;
  $501 = (($498) + 4)|0;
  $502 = $501;
  $503 = HEAP32[$502>>2]|0;
  $504 = HEAP32[$1>>2]|0;
  $505 = ((($504)) + 24|0);
  $506 = $505;
  $507 = $506;
  HEAP32[$507>>2] = $500;
  $508 = (($506) + 4)|0;
  $509 = $508;
  HEAP32[$509>>2] = $503;
  STACKTOP = sp;return;
 }
 }
}
function _sqlite3Fts3DoclistNext($bDescIdx,$aDoclist,$nDoclist,$ppIter,$piDocid,$pbEof) {
 $bDescIdx = $bDescIdx|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 $ppIter = $ppIter|0;
 $piDocid = $piDocid|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $iVar = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $p = sp + 8|0;
 $iVar = sp;
 HEAP32[$0>>2] = $bDescIdx;
 HEAP32[$1>>2] = $aDoclist;
 HEAP32[$2>>2] = $nDoclist;
 HEAP32[$3>>2] = $ppIter;
 HEAP32[$4>>2] = $piDocid;
 HEAP32[$5>>2] = $pbEof;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  $13 = (_sqlite3Fts3GetVarint($11,$12)|0);
  $14 = HEAP32[$p>>2]|0;
  $15 = (($14) + ($13)|0);
  HEAP32[$p>>2] = $15;
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  STACKTOP = sp;return;
 }
 _fts3PoslistCopy(0,$p);
 while(1) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + ($17)|0);
  $20 = ($16>>>0)<($19>>>0);
  if ($20) {
   $21 = HEAP32[$p>>2]|0;
   $22 = HEAP8[$21>>0]|0;
   $23 = $22 << 24 >> 24;
   $24 = ($23|0)==(0);
   $64 = $24;
  } else {
   $64 = 0;
  }
  $25 = HEAP32[$p>>2]|0;
  if (!($64)) {
   break;
  }
  $26 = ((($25)) + 1|0);
  HEAP32[$p>>2] = $26;
 }
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = (($28) + ($27)|0);
 $30 = ($25>>>0)>=($29>>>0);
 if ($30) {
  $31 = HEAP32[$5>>2]|0;
  HEAP8[$31>>0] = 1;
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  STACKTOP = sp;return;
 } else {
  $32 = HEAP32[$p>>2]|0;
  $33 = (_sqlite3Fts3GetVarint($32,$iVar)|0);
  $34 = HEAP32[$p>>2]|0;
  $35 = (($34) + ($33)|0);
  HEAP32[$p>>2] = $35;
  $36 = HEAP32[$0>>2]|0;
  $37 = ($36|0)!=(0);
  $38 = $37 ? -1 : 1;
  $39 = ($38|0)<(0);
  $40 = $39 << 31 >> 31;
  $41 = $iVar;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($41) + 4)|0;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (___muldi3(($38|0),($40|0),($43|0),($46|0))|0);
  $48 = tempRet0;
  $49 = HEAP32[$4>>2]|0;
  $50 = $49;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = (_i64Add(($52|0),($55|0),($47|0),($48|0))|0);
  $57 = tempRet0;
  $58 = $49;
  $59 = $58;
  HEAP32[$59>>2] = $56;
  $60 = (($58) + 4)|0;
  $61 = $60;
  HEAP32[$61>>2] = $57;
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  STACKTOP = sp;return;
 }
}
function _fts3EvalInvalidatePoslist($pPhrase) {
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPhrase;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 28|0);
  $7 = HEAP32[$6>>2]|0;
  _sqlite3_free($7);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 32|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 24|0);
 HEAP32[$13>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3EvalPhraseNext($pCsr,$p,$pbEof) {
 $pCsr = $pCsr|0;
 $p = $p|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDL = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pDL = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pDL>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_fts3EvalIncrPhraseNext($10,$11,$12)|0);
  HEAP32[$rc>>2] = $13;
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 $14 = ((($10)) + 52|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = HEAP32[$pTab>>2]|0;
 $18 = ((($17)) + 231|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($16|0)!=($20|0);
 if ($21) {
  $22 = HEAP32[$pDL>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0);
  if ($25) {
   $26 = HEAP32[$pTab>>2]|0;
   $27 = ((($26)) + 231|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = HEAP32[$pDL>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$pDL>>2]|0;
   $33 = ((($32)) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$pDL>>2]|0;
   $36 = ((($35)) + 8|0);
   $37 = HEAP32[$pDL>>2]|0;
   $38 = ((($37)) + 16|0);
   $39 = HEAP32[$pDL>>2]|0;
   $40 = ((($39)) + 32|0);
   $41 = HEAP32[$2>>2]|0;
   _sqlite3Fts3DoclistPrev($29,$31,$34,$36,$38,$40,$41);
   $42 = HEAP32[$pDL>>2]|0;
   $43 = ((($42)) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$pDL>>2]|0;
   $46 = ((($45)) + 28|0);
   HEAP32[$46>>2] = $44;
   $50 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($50|0);
  }
 }
 $47 = HEAP32[$pTab>>2]|0;
 $48 = HEAP32[$pDL>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 _fts3EvalDlPhraseNext($47,$48,$49);
 $50 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _fts3EvalIncrPhraseNext($pCsr,$p,$pbEof) {
 $pCsr = $pCsr|0;
 $p = $p|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $a = 0, $aDoclist = 0, $bDescDoclist = 0, $bEof = 0, $bMaxSet = 0, $i = 0, $iMax = 0, $nByte = 0, $nDist = 0, $nList = 0, $pDL = 0, $pL = 0, $pOut = 0, $pR = 0, $pTab = 0, $rc = 0, $res = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 172|0;
 $1 = sp + 168|0;
 $2 = sp + 164|0;
 $3 = sp + 160|0;
 $rc = sp + 156|0;
 $pDL = sp + 152|0;
 $pTab = sp + 148|0;
 $bEof = sp + 176|0;
 $bDescDoclist = sp + 144|0;
 $a = sp + 8|0;
 $bMaxSet = sp + 140|0;
 $iMax = sp;
 $i = sp + 136|0;
 $nList = sp + 132|0;
 $nByte = sp + 128|0;
 $aDoclist = sp + 124|0;
 $pL = sp + 120|0;
 $pR = sp + 116|0;
 $pOut = sp + 112|0;
 $nDist = sp + 108|0;
 $res = sp + 104|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$pDL>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTab>>2] = $6;
 HEAP8[$bEof>>0] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(1);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 40|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$pTab>>2]|0;
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 72|0);
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$pDL>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$pDL>>2]|0;
   $23 = ((($22)) + 28|0);
   $24 = HEAP32[$pDL>>2]|0;
   $25 = ((($24)) + 32|0);
   $26 = (_sqlite3Fts3MsrIncrNext($15,$19,$21,$23,$25)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$pDL>>2]|0;
   $28 = ((($27)) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0|0);
   if ($30) {
    HEAP8[$bEof>>0] = 1;
   }
  } else {
   label = 5;
  }
 } else {
  label = 5;
 }
 L5: do {
  if ((label|0) == 5) {
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 52|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   HEAP32[$bDescDoclist>>2] = $34;
   dest=$a; stop=dest+96|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   while(1) {
    $35 = HEAP8[$bEof>>0]|0;
    $36 = $35&255;
    $37 = ($36|0)==(0);
    if (!($37)) {
     break L5;
    }
    HEAP32[$bMaxSet>>2] = 0;
    $38 = $iMax;
    $39 = $38;
    HEAP32[$39>>2] = 0;
    $40 = (($38) + 4)|0;
    $41 = $40;
    HEAP32[$41>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $42 = HEAP32[$rc>>2]|0;
     $43 = ($42|0)==(0);
     if (!($43)) {
      break;
     }
     $44 = HEAP32[$i>>2]|0;
     $45 = HEAP32[$2>>2]|0;
     $46 = ((($45)) + 64|0);
     $47 = HEAP32[$46>>2]|0;
     $48 = ($44|0)<($47|0);
     if (!($48)) {
      break;
     }
     $49 = HEAP8[$bEof>>0]|0;
     $50 = $49&255;
     $51 = ($50|0)==(0);
     if (!($51)) {
      break;
     }
     $52 = HEAP32[$pTab>>2]|0;
     $53 = HEAP32[$2>>2]|0;
     $54 = HEAP32[$i>>2]|0;
     $55 = HEAP32[$i>>2]|0;
     $56 = (($a) + (($55*24)|0)|0);
     $57 = (_incrPhraseTokenNext($52,$53,$54,$56,$bEof)|0);
     HEAP32[$rc>>2] = $57;
     $58 = HEAP32[$i>>2]|0;
     $59 = (($a) + (($58*24)|0)|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = ($60|0)==(0);
     do {
      if ($61) {
       $62 = HEAP32[$bMaxSet>>2]|0;
       $63 = ($62|0)==(0);
       if (!($63)) {
        $64 = HEAP32[$bDescDoclist>>2]|0;
        $65 = ($64|0)!=(0);
        $66 = $65 ? -1 : 1;
        $67 = ($66|0)<(0);
        $68 = $67 << 31 >> 31;
        $69 = $iMax;
        $70 = $69;
        $71 = HEAP32[$70>>2]|0;
        $72 = (($69) + 4)|0;
        $73 = $72;
        $74 = HEAP32[$73>>2]|0;
        $75 = HEAP32[$i>>2]|0;
        $76 = (($a) + (($75*24)|0)|0);
        $77 = ((($76)) + 8|0);
        $78 = $77;
        $79 = $78;
        $80 = HEAP32[$79>>2]|0;
        $81 = (($78) + 4)|0;
        $82 = $81;
        $83 = HEAP32[$82>>2]|0;
        $84 = (_i64Subtract(($71|0),($74|0),($80|0),($83|0))|0);
        $85 = tempRet0;
        (___muldi3(($66|0),($68|0),($84|0),($85|0))|0);
        $86 = tempRet0;
        $87 = ($86|0)<(0);
        if (!($87)) {
         break;
        }
       }
       $88 = HEAP32[$i>>2]|0;
       $89 = (($a) + (($88*24)|0)|0);
       $90 = ((($89)) + 8|0);
       $91 = $90;
       $92 = $91;
       $93 = HEAP32[$92>>2]|0;
       $94 = (($91) + 4)|0;
       $95 = $94;
       $96 = HEAP32[$95>>2]|0;
       $97 = $iMax;
       $98 = $97;
       HEAP32[$98>>2] = $93;
       $99 = (($97) + 4)|0;
       $100 = $99;
       HEAP32[$100>>2] = $96;
       HEAP32[$bMaxSet>>2] = 1;
      }
     } while(0);
     $101 = HEAP32[$i>>2]|0;
     $102 = (($101) + 1)|0;
     HEAP32[$i>>2] = $102;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $103 = HEAP32[$i>>2]|0;
     $104 = HEAP32[$2>>2]|0;
     $105 = ((($104)) + 64|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = ($103|0)<($106|0);
     if (!($107)) {
      break;
     }
     while(1) {
      $108 = HEAP32[$rc>>2]|0;
      $109 = ($108|0)==(0);
      if (!($109)) {
       break;
      }
      $110 = HEAP8[$bEof>>0]|0;
      $111 = $110&255;
      $112 = ($111|0)==(0);
      if (!($112)) {
       break;
      }
      $113 = HEAP32[$i>>2]|0;
      $114 = (($a) + (($113*24)|0)|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = ($115|0)==(0);
      if (!($116)) {
       break;
      }
      $117 = HEAP32[$bDescDoclist>>2]|0;
      $118 = ($117|0)!=(0);
      $119 = $118 ? -1 : 1;
      $120 = ($119|0)<(0);
      $121 = $120 << 31 >> 31;
      $122 = HEAP32[$i>>2]|0;
      $123 = (($a) + (($122*24)|0)|0);
      $124 = ((($123)) + 8|0);
      $125 = $124;
      $126 = $125;
      $127 = HEAP32[$126>>2]|0;
      $128 = (($125) + 4)|0;
      $129 = $128;
      $130 = HEAP32[$129>>2]|0;
      $131 = $iMax;
      $132 = $131;
      $133 = HEAP32[$132>>2]|0;
      $134 = (($131) + 4)|0;
      $135 = $134;
      $136 = HEAP32[$135>>2]|0;
      $137 = (_i64Subtract(($127|0),($130|0),($133|0),($136|0))|0);
      $138 = tempRet0;
      (___muldi3(($119|0),($121|0),($137|0),($138|0))|0);
      $139 = tempRet0;
      $140 = ($139|0)<(0);
      if (!($140)) {
       break;
      }
      $141 = HEAP32[$pTab>>2]|0;
      $142 = HEAP32[$2>>2]|0;
      $143 = HEAP32[$i>>2]|0;
      $144 = HEAP32[$i>>2]|0;
      $145 = (($a) + (($144*24)|0)|0);
      $146 = (_incrPhraseTokenNext($141,$142,$143,$145,$bEof)|0);
      HEAP32[$rc>>2] = $146;
      $147 = HEAP32[$bDescDoclist>>2]|0;
      $148 = ($147|0)!=(0);
      $149 = $148 ? -1 : 1;
      $150 = ($149|0)<(0);
      $151 = $150 << 31 >> 31;
      $152 = HEAP32[$i>>2]|0;
      $153 = (($a) + (($152*24)|0)|0);
      $154 = ((($153)) + 8|0);
      $155 = $154;
      $156 = $155;
      $157 = HEAP32[$156>>2]|0;
      $158 = (($155) + 4)|0;
      $159 = $158;
      $160 = HEAP32[$159>>2]|0;
      $161 = $iMax;
      $162 = $161;
      $163 = HEAP32[$162>>2]|0;
      $164 = (($161) + 4)|0;
      $165 = $164;
      $166 = HEAP32[$165>>2]|0;
      $167 = (_i64Subtract(($157|0),($160|0),($163|0),($166|0))|0);
      $168 = tempRet0;
      $169 = (___muldi3(($149|0),($151|0),($167|0),($168|0))|0);
      $170 = tempRet0;
      $171 = ($170|0)>(0);
      $172 = ($169>>>0)>(0);
      $173 = ($170|0)==(0);
      $174 = $173 & $172;
      $175 = $171 | $174;
      if (!($175)) {
       continue;
      }
      $176 = HEAP32[$i>>2]|0;
      $177 = (($a) + (($176*24)|0)|0);
      $178 = ((($177)) + 8|0);
      $179 = $178;
      $180 = $179;
      $181 = HEAP32[$180>>2]|0;
      $182 = (($179) + 4)|0;
      $183 = $182;
      $184 = HEAP32[$183>>2]|0;
      $185 = $iMax;
      $186 = $185;
      HEAP32[$186>>2] = $181;
      $187 = (($185) + 4)|0;
      $188 = $187;
      HEAP32[$188>>2] = $184;
      HEAP32[$i>>2] = 0;
     }
     $189 = HEAP32[$i>>2]|0;
     $190 = (($189) + 1)|0;
     HEAP32[$i>>2] = $190;
    }
    $191 = HEAP8[$bEof>>0]|0;
    $192 = $191&255;
    $193 = ($192|0)==(0);
    if (!($193)) {
     continue;
    }
    HEAP32[$nList>>2] = 0;
    $194 = HEAP32[$2>>2]|0;
    $195 = ((($194)) + 64|0);
    $196 = HEAP32[$195>>2]|0;
    $197 = (($196) - 1)|0;
    $198 = (($a) + (($197*24)|0)|0);
    $199 = ((($198)) + 20|0);
    $200 = HEAP32[$199>>2]|0;
    HEAP32[$nByte>>2] = $200;
    $201 = HEAP32[$nByte>>2]|0;
    $202 = (($201) + 1)|0;
    $203 = (_sqlite3_malloc($202)|0);
    HEAP32[$aDoclist>>2] = $203;
    $204 = HEAP32[$aDoclist>>2]|0;
    $205 = ($204|0)!=(0|0);
    if (!($205)) {
     break;
    }
    $206 = HEAP32[$aDoclist>>2]|0;
    $207 = HEAP32[$2>>2]|0;
    $208 = ((($207)) + 64|0);
    $209 = HEAP32[$208>>2]|0;
    $210 = (($209) - 1)|0;
    $211 = (($a) + (($210*24)|0)|0);
    $212 = ((($211)) + 16|0);
    $213 = HEAP32[$212>>2]|0;
    $214 = HEAP32[$nByte>>2]|0;
    $215 = (($214) + 1)|0;
    _memcpy(($206|0),($213|0),($215|0))|0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $216 = HEAP32[$i>>2]|0;
     $217 = HEAP32[$2>>2]|0;
     $218 = ((($217)) + 64|0);
     $219 = HEAP32[$218>>2]|0;
     $220 = (($219) - 1)|0;
     $221 = ($216|0)<($220|0);
     if (!($221)) {
      break;
     }
     $222 = HEAP32[$i>>2]|0;
     $223 = (($a) + (($222*24)|0)|0);
     $224 = HEAP32[$223>>2]|0;
     $225 = ($224|0)==(0);
     if ($225) {
      $226 = HEAP32[$i>>2]|0;
      $227 = (($a) + (($226*24)|0)|0);
      $228 = ((($227)) + 16|0);
      $229 = HEAP32[$228>>2]|0;
      HEAP32[$pL>>2] = $229;
      $230 = HEAP32[$aDoclist>>2]|0;
      HEAP32[$pR>>2] = $230;
      $231 = HEAP32[$aDoclist>>2]|0;
      HEAP32[$pOut>>2] = $231;
      $232 = HEAP32[$2>>2]|0;
      $233 = ((($232)) + 64|0);
      $234 = HEAP32[$233>>2]|0;
      $235 = (($234) - 1)|0;
      $236 = HEAP32[$i>>2]|0;
      $237 = (($235) - ($236))|0;
      HEAP32[$nDist>>2] = $237;
      $238 = HEAP32[$nDist>>2]|0;
      $239 = (_fts3PoslistPhraseMerge($pOut,$238,0,1,$pL,$pR)|0);
      HEAP32[$res>>2] = $239;
      $240 = HEAP32[$res>>2]|0;
      $241 = ($240|0)==(0);
      if ($241) {
       break;
      }
      $242 = HEAP32[$pOut>>2]|0;
      $243 = HEAP32[$aDoclist>>2]|0;
      $244 = $242;
      $245 = $243;
      $246 = (($244) - ($245))|0;
      HEAP32[$nList>>2] = $246;
     }
     $247 = HEAP32[$i>>2]|0;
     $248 = (($247) + 1)|0;
     HEAP32[$i>>2] = $248;
    }
    $249 = HEAP32[$i>>2]|0;
    $250 = HEAP32[$2>>2]|0;
    $251 = ((($250)) + 64|0);
    $252 = HEAP32[$251>>2]|0;
    $253 = (($252) - 1)|0;
    $254 = ($249|0)==($253|0);
    if ($254) {
     label = 35;
     break;
    }
    $275 = HEAP32[$aDoclist>>2]|0;
    _sqlite3_free($275);
   }
   if ((label|0) == 35) {
    $255 = $iMax;
    $256 = $255;
    $257 = HEAP32[$256>>2]|0;
    $258 = (($255) + 4)|0;
    $259 = $258;
    $260 = HEAP32[$259>>2]|0;
    $261 = HEAP32[$pDL>>2]|0;
    $262 = ((($261)) + 16|0);
    $263 = $262;
    $264 = $263;
    HEAP32[$264>>2] = $257;
    $265 = (($263) + 4)|0;
    $266 = $265;
    HEAP32[$266>>2] = $260;
    $267 = HEAP32[$aDoclist>>2]|0;
    $268 = HEAP32[$pDL>>2]|0;
    $269 = ((($268)) + 28|0);
    HEAP32[$269>>2] = $267;
    $270 = HEAP32[$nList>>2]|0;
    $271 = HEAP32[$pDL>>2]|0;
    $272 = ((($271)) + 32|0);
    HEAP32[$272>>2] = $270;
    $273 = HEAP32[$pDL>>2]|0;
    $274 = ((($273)) + 24|0);
    HEAP32[$274>>2] = 1;
    break;
   }
   HEAP32[$0>>2] = 7;
   $279 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($279|0);
  }
 } while(0);
 $276 = HEAP8[$bEof>>0]|0;
 $277 = HEAP32[$3>>2]|0;
 HEAP8[$277>>0] = $276;
 $278 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $278;
 $279 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($279|0);
}
function _fts3EvalDlPhraseNext($pTab,$pDL,$pbEof) {
 $pTab = $pTab|0;
 $pDL = $pDL|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $iDelta = 0;
 var $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pIter = sp + 12|0;
 $pEnd = sp + 8|0;
 $iDelta = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $pDL;
 HEAP32[$2>>2] = $pbEof;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($5)|0);
 HEAP32[$pEnd>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 $13 = HEAP32[$1>>2]|0;
 if ($12) {
  $14 = ((($13)) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$pIter>>2] = $15;
 } else {
  $16 = HEAP32[$13>>2]|0;
  HEAP32[$pIter>>2] = $16;
 }
 $17 = HEAP32[$pIter>>2]|0;
 $18 = HEAP32[$pEnd>>2]|0;
 $19 = ($17>>>0)>=($18>>>0);
 if ($19) {
  $20 = HEAP32[$2>>2]|0;
  HEAP8[$20>>0] = 1;
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$pIter>>2]|0;
 $22 = (_sqlite3Fts3GetVarint($21,$iDelta)|0);
 $23 = HEAP32[$pIter>>2]|0;
 $24 = (($23) + ($22)|0);
 HEAP32[$pIter>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 231|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $29 = ($28|0)==(0);
 if ($29) {
  label = 8;
 } else {
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)==(0|0);
  if ($33) {
   label = 8;
  } else {
   $54 = $iDelta;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($54) + 4)|0;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   $61 = ((($60)) + 16|0);
   $62 = $61;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Subtract(($64|0),($67|0),($56|0),($59|0))|0);
   $69 = tempRet0;
   $70 = $61;
   $71 = $70;
   HEAP32[$71>>2] = $68;
   $72 = (($70) + 4)|0;
   $73 = $72;
   HEAP32[$73>>2] = $69;
  }
 }
 if ((label|0) == 8) {
  $34 = $iDelta;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 16|0);
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (_i64Add(($44|0),($47|0),($36|0),($39|0))|0);
  $49 = tempRet0;
  $50 = $41;
  $51 = $50;
  HEAP32[$51>>2] = $48;
  $52 = (($50) + 4)|0;
  $53 = $52;
  HEAP32[$53>>2] = $49;
 }
 $74 = HEAP32[$pIter>>2]|0;
 $75 = HEAP32[$1>>2]|0;
 $76 = ((($75)) + 28|0);
 HEAP32[$76>>2] = $74;
 _fts3PoslistCopy(0,$pIter);
 $77 = HEAP32[$pIter>>2]|0;
 $78 = HEAP32[$1>>2]|0;
 $79 = ((($78)) + 28|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $77;
 $82 = $80;
 $83 = (($81) - ($82))|0;
 $84 = HEAP32[$1>>2]|0;
 $85 = ((($84)) + 32|0);
 HEAP32[$85>>2] = $83;
 while(1) {
  $86 = HEAP32[$pIter>>2]|0;
  $87 = HEAP32[$pEnd>>2]|0;
  $88 = ($86>>>0)<($87>>>0);
  if ($88) {
   $89 = HEAP32[$pIter>>2]|0;
   $90 = HEAP8[$89>>0]|0;
   $91 = $90 << 24 >> 24;
   $92 = ($91|0)==(0);
   $98 = $92;
  } else {
   $98 = 0;
  }
  $93 = HEAP32[$pIter>>2]|0;
  if (!($98)) {
   break;
  }
  $94 = ((($93)) + 1|0);
  HEAP32[$pIter>>2] = $94;
 }
 $95 = HEAP32[$1>>2]|0;
 $96 = ((($95)) + 8|0);
 HEAP32[$96>>2] = $93;
 $97 = HEAP32[$2>>2]|0;
 HEAP8[$97>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3Fts3MsrIncrNext($p,$pMsr,$piDocid,$paPoslist,$pnPoslist) {
 $p = $p|0;
 $pMsr = $pMsr|0;
 $piDocid = $piDocid|0;
 $paPoslist = $paPoslist|0;
 $pnPoslist = $pnPoslist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apSegment = 0, $iDocid = 0, $j = 0, $nList = 0;
 var $nMerge = 0, $pList = 0, $pSeg = 0, $rc = 0, $xCmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 40|0;
 $nMerge = sp + 36|0;
 $apSegment = sp + 32|0;
 $xCmp = sp + 28|0;
 $pSeg = sp + 24|0;
 $rc = sp + 20|0;
 $pList = sp + 16|0;
 $nList = sp + 12|0;
 $j = sp + 8|0;
 $iDocid = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pMsr;
 HEAP32[$3>>2] = $piDocid;
 HEAP32[$4>>2] = $paPoslist;
 HEAP32[$5>>2] = $pnPoslist;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nMerge>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$apSegment>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 231|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 $16 = $15 ? 182 : 183;
 HEAP32[$xCmp>>2] = $16;
 $17 = HEAP32[$nMerge>>2]|0;
 $18 = ($17|0)==(0);
 if ($18) {
  $19 = HEAP32[$4>>2]|0;
  HEAP32[$19>>2] = 0;
  HEAP32[$0>>2] = 0;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 while(1) {
  $20 = HEAP32[$2>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$pSeg>>2] = $22;
  $23 = HEAP32[$pSeg>>2]|0;
  $24 = ((($23)) + 80|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)==(0|0);
  if ($26) {
   label = 4;
   break;
  }
  $28 = HEAP32[$apSegment>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 88|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = $iDocid;
  $38 = $37;
  HEAP32[$38>>2] = $33;
  $39 = (($37) + 4)|0;
  $40 = $39;
  HEAP32[$40>>2] = $36;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$apSegment>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = (_fts3SegReaderNextDocid($41,$43,$pList,$nList)|0);
  HEAP32[$rc>>2] = $44;
  HEAP32[$j>>2] = 1;
  while(1) {
   $45 = HEAP32[$rc>>2]|0;
   $46 = ($45|0)==(0);
   if (!($46)) {
    break;
   }
   $47 = HEAP32[$j>>2]|0;
   $48 = HEAP32[$nMerge>>2]|0;
   $49 = ($47|0)<($48|0);
   if (!($49)) {
    break;
   }
   $50 = HEAP32[$j>>2]|0;
   $51 = HEAP32[$apSegment>>2]|0;
   $52 = (($51) + ($50<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ((($53)) + 80|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ($55|0)!=(0|0);
   if (!($56)) {
    break;
   }
   $57 = HEAP32[$j>>2]|0;
   $58 = HEAP32[$apSegment>>2]|0;
   $59 = (($58) + ($57<<2)|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ((($60)) + 88|0);
   $62 = $61;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = $iDocid;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $71 = (($68) + 4)|0;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = ($64|0)==($70|0);
   $75 = ($67|0)==($73|0);
   $76 = $74 & $75;
   if (!($76)) {
    break;
   }
   $77 = HEAP32[$1>>2]|0;
   $78 = HEAP32[$j>>2]|0;
   $79 = HEAP32[$apSegment>>2]|0;
   $80 = (($79) + ($78<<2)|0);
   $81 = HEAP32[$80>>2]|0;
   $82 = (_fts3SegReaderNextDocid($77,$81,0,0)|0);
   HEAP32[$rc>>2] = $82;
   $83 = HEAP32[$j>>2]|0;
   $84 = (($83) + 1)|0;
   HEAP32[$j>>2] = $84;
  }
  $85 = HEAP32[$rc>>2]|0;
  $86 = ($85|0)!=(0);
  if ($86) {
   label = 12;
   break;
  }
  $88 = HEAP32[$2>>2]|0;
  $89 = HEAP32[$88>>2]|0;
  $90 = HEAP32[$nMerge>>2]|0;
  $91 = HEAP32[$j>>2]|0;
  $92 = HEAP32[$xCmp>>2]|0;
  _fts3SegReaderSort($89,$90,$91,$92);
  $93 = HEAP32[$nList>>2]|0;
  $94 = ($93|0)>(0);
  if ($94) {
   $95 = HEAP32[$apSegment>>2]|0;
   $96 = HEAP32[$95>>2]|0;
   $97 = ((($96)) + 56|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($98|0)!=(0|0);
   if ($99) {
    $100 = HEAP32[$2>>2]|0;
    $101 = HEAP32[$pList>>2]|0;
    $102 = HEAP32[$nList>>2]|0;
    $103 = (($102) + 1)|0;
    $104 = (_fts3MsrBufferData($100,$101,$103)|0);
    HEAP32[$rc>>2] = $104;
    $105 = HEAP32[$rc>>2]|0;
    $106 = ($105|0)!=(0);
    if ($106) {
     label = 16;
     break;
    }
    $108 = HEAP32[$2>>2]|0;
    $109 = ((($108)) + 16|0);
    $110 = HEAP32[$109>>2]|0;
    HEAP32[$pList>>2] = $110;
   }
  }
  $111 = HEAP32[$2>>2]|0;
  $112 = ((($111)) + 24|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($113|0)>=(0);
  if ($114) {
   $115 = HEAP32[$2>>2]|0;
   $116 = ((($115)) + 24|0);
   $117 = HEAP32[$116>>2]|0;
   _fts3ColumnFilter($117,1,$pList,$nList);
  }
  $118 = HEAP32[$nList>>2]|0;
  $119 = ($118|0)>(0);
  if ($119) {
   label = 21;
   break;
  }
 }
 if ((label|0) == 4) {
  $27 = HEAP32[$4>>2]|0;
  HEAP32[$27>>2] = 0;
 }
 else if ((label|0) == 12) {
  $87 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $87;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 else if ((label|0) == 16) {
  $107 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $107;
  $135 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 else if ((label|0) == 21) {
  $120 = HEAP32[$pList>>2]|0;
  $121 = HEAP32[$4>>2]|0;
  HEAP32[$121>>2] = $120;
  $122 = $iDocid;
  $123 = $122;
  $124 = HEAP32[$123>>2]|0;
  $125 = (($122) + 4)|0;
  $126 = $125;
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$3>>2]|0;
  $129 = $128;
  $130 = $129;
  HEAP32[$130>>2] = $124;
  $131 = (($129) + 4)|0;
  $132 = $131;
  HEAP32[$132>>2] = $127;
  $133 = HEAP32[$nList>>2]|0;
  $134 = HEAP32[$5>>2]|0;
  HEAP32[$134>>2] = $133;
 }
 HEAP32[$0>>2] = 0;
 $135 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _incrPhraseTokenNext($pTab,$pPhrase,$iToken,$p,$pbEof) {
 $pTab = $pTab|0;
 $pPhrase = $pPhrase|0;
 $iToken = $iToken|0;
 $p = $p|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pToken = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $pPhrase;
 HEAP32[$2>>2] = $iToken;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($7|0)==($8|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  _fts3EvalDlPhraseNext($10,$11,$12);
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 28|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = ((($16)) + 16|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 32|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = ((($21)) + 20|0);
  HEAP32[$22>>2] = $20;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = $24;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = $32;
  $34 = $33;
  HEAP32[$34>>2] = $27;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = $30;
  $62 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 72|0);
 $40 = (($39) + (($37*24)|0)|0);
 HEAP32[$pToken>>2] = $40;
 $41 = HEAP32[$pToken>>2]|0;
 $42 = ((($41)) + 20|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0|0);
 if (!($44)) {
  $61 = HEAP32[$3>>2]|0;
  HEAP32[$61>>2] = 1;
  $62 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$pToken>>2]|0;
 $47 = ((($46)) + 20|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$3>>2]|0;
 $50 = ((($49)) + 8|0);
 $51 = HEAP32[$3>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$3>>2]|0;
 $54 = ((($53)) + 20|0);
 $55 = (_sqlite3Fts3MsrIncrNext($45,$48,$50,$52,$54)|0);
 HEAP32[$rc>>2] = $55;
 $56 = HEAP32[$3>>2]|0;
 $57 = ((($56)) + 16|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = ($58|0)==(0|0);
 if (!($59)) {
  $62 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $60 = HEAP32[$4>>2]|0;
 HEAP8[$60>>0] = 1;
 $62 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _fts3PoslistPhraseMerge($pp,$nToken,$isSaveLeft,$isExact,$pp1,$pp2) {
 $pp = $pp|0;
 $nToken = $nToken|0;
 $isSaveLeft = $isSaveLeft|0;
 $isExact = $isExact|0;
 $pp1 = $pp1|0;
 $pp2 = $pp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iCol1 = 0, $iCol2 = 0, $iPos1 = 0, $iPos2 = 0, $iPrev = 0, $iSave = 0, $p = 0, $p1 = 0, $p2 = 0, $pSave = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $6 = sp + 56|0;
 $p = sp + 52|0;
 $p1 = sp + 48|0;
 $p2 = sp + 44|0;
 $iCol1 = sp + 40|0;
 $iCol2 = sp + 36|0;
 $pSave = sp + 32|0;
 $iPrev = sp + 24|0;
 $iPos1 = sp + 16|0;
 $iPos2 = sp + 8|0;
 $iSave = sp;
 HEAP32[$1>>2] = $pp;
 HEAP32[$2>>2] = $nToken;
 HEAP32[$3>>2] = $isSaveLeft;
 HEAP32[$4>>2] = $isExact;
 HEAP32[$5>>2] = $pp1;
 HEAP32[$6>>2] = $pp2;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$5>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p1>>2] = $10;
 $11 = HEAP32[$6>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p2>>2] = $12;
 HEAP32[$iCol1>>2] = 0;
 HEAP32[$iCol2>>2] = 0;
 $13 = HEAP32[$p1>>2]|0;
 $14 = HEAP8[$13>>0]|0;
 $15 = $14 << 24 >> 24;
 $16 = ($15|0)==(1);
 if ($16) {
  $17 = HEAP32[$p1>>2]|0;
  $18 = ((($17)) + 1|0);
  HEAP32[$p1>>2] = $18;
  $19 = HEAP32[$p1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = $21 & 128;
  $23 = ($22|0)!=(0);
  $24 = HEAP32[$p1>>2]|0;
  if ($23) {
   $25 = (_sqlite3Fts3GetVarint32($24,$iCol1)|0);
   $30 = $25;
  } else {
   $26 = HEAP8[$24>>0]|0;
   $27 = $26&255;
   HEAP32[$iCol1>>2] = $27;
   $30 = 1;
  }
  $28 = HEAP32[$p1>>2]|0;
  $29 = (($28) + ($30)|0);
  HEAP32[$p1>>2] = $29;
 }
 $31 = HEAP32[$p2>>2]|0;
 $32 = HEAP8[$31>>0]|0;
 $33 = $32 << 24 >> 24;
 $34 = ($33|0)==(1);
 if ($34) {
  $35 = HEAP32[$p2>>2]|0;
  $36 = ((($35)) + 1|0);
  HEAP32[$p2>>2] = $36;
  $37 = HEAP32[$p2>>2]|0;
  $38 = HEAP8[$37>>0]|0;
  $39 = $38&255;
  $40 = $39 & 128;
  $41 = ($40|0)!=(0);
  $42 = HEAP32[$p2>>2]|0;
  if ($41) {
   $43 = (_sqlite3Fts3GetVarint32($42,$iCol2)|0);
   $48 = $43;
  } else {
   $44 = HEAP8[$42>>0]|0;
   $45 = $44&255;
   HEAP32[$iCol2>>2] = $45;
   $48 = 1;
  }
  $46 = HEAP32[$p2>>2]|0;
  $47 = (($46) + ($48)|0);
  HEAP32[$p2>>2] = $47;
 }
 while(1) {
  $49 = HEAP32[$iCol1>>2]|0;
  $50 = HEAP32[$iCol2>>2]|0;
  $51 = ($49|0)==($50|0);
  if (!($51)) {
   $315 = HEAP32[$iCol1>>2]|0;
   $316 = HEAP32[$iCol2>>2]|0;
   $317 = ($315|0)<($316|0);
   if ($317) {
    _fts3ColumnlistCopy(0,$p1);
    $318 = HEAP32[$p1>>2]|0;
    $319 = HEAP8[$318>>0]|0;
    $320 = $319 << 24 >> 24;
    $321 = (0)==($320|0);
    if ($321) {
     break;
    }
    $322 = HEAP32[$p1>>2]|0;
    $323 = ((($322)) + 1|0);
    HEAP32[$p1>>2] = $323;
    $324 = HEAP32[$p1>>2]|0;
    $325 = HEAP8[$324>>0]|0;
    $326 = $325&255;
    $327 = $326 & 128;
    $328 = ($327|0)!=(0);
    $329 = HEAP32[$p1>>2]|0;
    if ($328) {
     $330 = (_sqlite3Fts3GetVarint32($329,$iCol1)|0);
     $335 = $330;
    } else {
     $331 = HEAP8[$329>>0]|0;
     $332 = $331&255;
     HEAP32[$iCol1>>2] = $332;
     $335 = 1;
    }
    $333 = HEAP32[$p1>>2]|0;
    $334 = (($333) + ($335)|0);
    HEAP32[$p1>>2] = $334;
    continue;
   } else {
    _fts3ColumnlistCopy(0,$p2);
    $336 = HEAP32[$p2>>2]|0;
    $337 = HEAP8[$336>>0]|0;
    $338 = $337 << 24 >> 24;
    $339 = (0)==($338|0);
    if ($339) {
     break;
    }
    $340 = HEAP32[$p2>>2]|0;
    $341 = ((($340)) + 1|0);
    HEAP32[$p2>>2] = $341;
    $342 = HEAP32[$p2>>2]|0;
    $343 = HEAP8[$342>>0]|0;
    $344 = $343&255;
    $345 = $344 & 128;
    $346 = ($345|0)!=(0);
    $347 = HEAP32[$p2>>2]|0;
    if ($346) {
     $348 = (_sqlite3Fts3GetVarint32($347,$iCol2)|0);
     $353 = $348;
    } else {
     $349 = HEAP8[$347>>0]|0;
     $350 = $349&255;
     HEAP32[$iCol2>>2] = $350;
     $353 = 1;
    }
    $351 = HEAP32[$p2>>2]|0;
    $352 = (($351) + ($353)|0);
    HEAP32[$p2>>2] = $352;
    continue;
   }
  }
  $52 = HEAP32[$p>>2]|0;
  HEAP32[$pSave>>2] = $52;
  $53 = $iPrev;
  $54 = $53;
  HEAP32[$54>>2] = 0;
  $55 = (($53) + 4)|0;
  $56 = $55;
  HEAP32[$56>>2] = 0;
  $57 = $iPos1;
  $58 = $57;
  HEAP32[$58>>2] = 0;
  $59 = (($57) + 4)|0;
  $60 = $59;
  HEAP32[$60>>2] = 0;
  $61 = $iPos2;
  $62 = $61;
  HEAP32[$62>>2] = 0;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = 0;
  $65 = HEAP32[$iCol1>>2]|0;
  $66 = ($65|0)!=(0);
  if ($66) {
   $67 = HEAP32[$p>>2]|0;
   $68 = ((($67)) + 1|0);
   HEAP32[$p>>2] = $68;
   HEAP8[$67>>0] = 1;
   $69 = HEAP32[$p>>2]|0;
   $70 = HEAP32[$iCol1>>2]|0;
   $71 = ($70|0)<(0);
   $72 = $71 << 31 >> 31;
   $73 = (_sqlite3Fts3PutVarint($69,$70,$72)|0);
   $74 = HEAP32[$p>>2]|0;
   $75 = (($74) + ($73)|0);
   HEAP32[$p>>2] = $75;
  }
  _fts3GetDeltaVarint($p1,$iPos1);
  $76 = $iPos1;
  $77 = $76;
  $78 = HEAP32[$77>>2]|0;
  $79 = (($76) + 4)|0;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (_i64Subtract(($78|0),($81|0),2,0)|0);
  $83 = tempRet0;
  $84 = $iPos1;
  $85 = $84;
  HEAP32[$85>>2] = $82;
  $86 = (($84) + 4)|0;
  $87 = $86;
  HEAP32[$87>>2] = $83;
  _fts3GetDeltaVarint($p2,$iPos2);
  $88 = $iPos2;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (_i64Subtract(($90|0),($93|0),2,0)|0);
  $95 = tempRet0;
  $96 = $iPos2;
  $97 = $96;
  HEAP32[$97>>2] = $94;
  $98 = (($96) + 4)|0;
  $99 = $98;
  HEAP32[$99>>2] = $95;
  while(1) {
   $100 = $iPos2;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($100) + 4)|0;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $106 = $iPos1;
   $107 = $106;
   $108 = HEAP32[$107>>2]|0;
   $109 = (($106) + 4)|0;
   $110 = $109;
   $111 = HEAP32[$110>>2]|0;
   $112 = HEAP32[$2>>2]|0;
   $113 = ($112|0)<(0);
   $114 = $113 << 31 >> 31;
   $115 = (_i64Add(($108|0),($111|0),($112|0),($114|0))|0);
   $116 = tempRet0;
   $117 = ($102|0)==($115|0);
   $118 = ($105|0)==($116|0);
   $119 = $117 & $118;
   if ($119) {
    label = 19;
   } else {
    $120 = HEAP32[$4>>2]|0;
    $121 = ($120|0)==(0);
    if ($121) {
     $122 = $iPos2;
     $123 = $122;
     $124 = HEAP32[$123>>2]|0;
     $125 = (($122) + 4)|0;
     $126 = $125;
     $127 = HEAP32[$126>>2]|0;
     $128 = $iPos1;
     $129 = $128;
     $130 = HEAP32[$129>>2]|0;
     $131 = (($128) + 4)|0;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = ($127|0)>($133|0);
     $135 = ($124>>>0)>($130>>>0);
     $136 = ($127|0)==($133|0);
     $137 = $136 & $135;
     $138 = $134 | $137;
     if ($138) {
      $139 = $iPos2;
      $140 = $139;
      $141 = HEAP32[$140>>2]|0;
      $142 = (($139) + 4)|0;
      $143 = $142;
      $144 = HEAP32[$143>>2]|0;
      $145 = $iPos1;
      $146 = $145;
      $147 = HEAP32[$146>>2]|0;
      $148 = (($145) + 4)|0;
      $149 = $148;
      $150 = HEAP32[$149>>2]|0;
      $151 = HEAP32[$2>>2]|0;
      $152 = ($151|0)<(0);
      $153 = $152 << 31 >> 31;
      $154 = (_i64Add(($147|0),($150|0),($151|0),($153|0))|0);
      $155 = tempRet0;
      $156 = ($144|0)<($155|0);
      $157 = ($141>>>0)<=($154>>>0);
      $158 = ($144|0)==($155|0);
      $159 = $158 & $157;
      $160 = $156 | $159;
      if ($160) {
       label = 19;
      }
     }
    }
   }
   if ((label|0) == 19) {
    label = 0;
    $161 = HEAP32[$3>>2]|0;
    $162 = ($161|0)!=(0);
    $163 = $iPos1;
    $164 = $163;
    $165 = HEAP32[$164>>2]|0;
    $166 = (($163) + 4)|0;
    $167 = $166;
    $168 = HEAP32[$167>>2]|0;
    $169 = $iPos2;
    $170 = $169;
    $171 = HEAP32[$170>>2]|0;
    $172 = (($169) + 4)|0;
    $173 = $172;
    $174 = HEAP32[$173>>2]|0;
    $175 = $162 ? $165 : $171;
    $176 = $162 ? $168 : $174;
    $177 = $iSave;
    $178 = $177;
    HEAP32[$178>>2] = $175;
    $179 = (($177) + 4)|0;
    $180 = $179;
    HEAP32[$180>>2] = $176;
    $181 = $iSave;
    $182 = $181;
    $183 = HEAP32[$182>>2]|0;
    $184 = (($181) + 4)|0;
    $185 = $184;
    $186 = HEAP32[$185>>2]|0;
    $187 = (_i64Add(($183|0),($186|0),2,0)|0);
    $188 = tempRet0;
    _fts3PutDeltaVarint($p,$iPrev,$187,$188);
    $189 = $iPrev;
    $190 = $189;
    $191 = HEAP32[$190>>2]|0;
    $192 = (($189) + 4)|0;
    $193 = $192;
    $194 = HEAP32[$193>>2]|0;
    $195 = (_i64Subtract(($191|0),($194|0),2,0)|0);
    $196 = tempRet0;
    $197 = $iPrev;
    $198 = $197;
    HEAP32[$198>>2] = $195;
    $199 = (($197) + 4)|0;
    $200 = $199;
    HEAP32[$200>>2] = $196;
    HEAP32[$pSave>>2] = 0;
   }
   $201 = HEAP32[$3>>2]|0;
   $202 = ($201|0)!=(0);
   if ($202) {
    label = 22;
   } else {
    $203 = $iPos2;
    $204 = $203;
    $205 = HEAP32[$204>>2]|0;
    $206 = (($203) + 4)|0;
    $207 = $206;
    $208 = HEAP32[$207>>2]|0;
    $209 = $iPos1;
    $210 = $209;
    $211 = HEAP32[$210>>2]|0;
    $212 = (($209) + 4)|0;
    $213 = $212;
    $214 = HEAP32[$213>>2]|0;
    $215 = HEAP32[$2>>2]|0;
    $216 = ($215|0)<(0);
    $217 = $216 << 31 >> 31;
    $218 = (_i64Add(($211|0),($214|0),($215|0),($217|0))|0);
    $219 = tempRet0;
    $220 = ($208|0)<($219|0);
    $221 = ($205>>>0)<=($218>>>0);
    $222 = ($208|0)==($219|0);
    $223 = $222 & $221;
    $224 = $220 | $223;
    if (!($224)) {
     label = 22;
    }
   }
   if ((label|0) == 22) {
    label = 0;
    $225 = $iPos2;
    $226 = $225;
    $227 = HEAP32[$226>>2]|0;
    $228 = (($225) + 4)|0;
    $229 = $228;
    $230 = HEAP32[$229>>2]|0;
    $231 = $iPos1;
    $232 = $231;
    $233 = HEAP32[$232>>2]|0;
    $234 = (($231) + 4)|0;
    $235 = $234;
    $236 = HEAP32[$235>>2]|0;
    $237 = ($230|0)<($236|0);
    $238 = ($227>>>0)<=($233>>>0);
    $239 = ($230|0)==($236|0);
    $240 = $239 & $238;
    $241 = $237 | $240;
    if (!($241)) {
     $259 = HEAP32[$p1>>2]|0;
     $260 = HEAP8[$259>>0]|0;
     $261 = $260 << 24 >> 24;
     $262 = $261 & 254;
     $263 = ($262|0)==(0);
     if ($263) {
      break;
     }
     _fts3GetDeltaVarint($p1,$iPos1);
     $264 = $iPos1;
     $265 = $264;
     $266 = HEAP32[$265>>2]|0;
     $267 = (($264) + 4)|0;
     $268 = $267;
     $269 = HEAP32[$268>>2]|0;
     $270 = (_i64Subtract(($266|0),($269|0),2,0)|0);
     $271 = tempRet0;
     $272 = $iPos1;
     $273 = $272;
     HEAP32[$273>>2] = $270;
     $274 = (($272) + 4)|0;
     $275 = $274;
     HEAP32[$275>>2] = $271;
     continue;
    }
   }
   $242 = HEAP32[$p2>>2]|0;
   $243 = HEAP8[$242>>0]|0;
   $244 = $243 << 24 >> 24;
   $245 = $244 & 254;
   $246 = ($245|0)==(0);
   if ($246) {
    break;
   }
   _fts3GetDeltaVarint($p2,$iPos2);
   $247 = $iPos2;
   $248 = $247;
   $249 = HEAP32[$248>>2]|0;
   $250 = (($247) + 4)|0;
   $251 = $250;
   $252 = HEAP32[$251>>2]|0;
   $253 = (_i64Subtract(($249|0),($252|0),2,0)|0);
   $254 = tempRet0;
   $255 = $iPos2;
   $256 = $255;
   HEAP32[$256>>2] = $253;
   $257 = (($255) + 4)|0;
   $258 = $257;
   HEAP32[$258>>2] = $254;
  }
  $276 = HEAP32[$pSave>>2]|0;
  $277 = ($276|0)!=(0|0);
  if ($277) {
   $278 = HEAP32[$pSave>>2]|0;
   HEAP32[$p>>2] = $278;
  }
  _fts3ColumnlistCopy(0,$p1);
  _fts3ColumnlistCopy(0,$p2);
  $279 = HEAP32[$p1>>2]|0;
  $280 = HEAP8[$279>>0]|0;
  $281 = $280 << 24 >> 24;
  $282 = (0)==($281|0);
  if ($282) {
   break;
  }
  $283 = HEAP32[$p2>>2]|0;
  $284 = HEAP8[$283>>0]|0;
  $285 = $284 << 24 >> 24;
  $286 = (0)==($285|0);
  if ($286) {
   break;
  }
  $287 = HEAP32[$p1>>2]|0;
  $288 = ((($287)) + 1|0);
  HEAP32[$p1>>2] = $288;
  $289 = HEAP32[$p1>>2]|0;
  $290 = HEAP8[$289>>0]|0;
  $291 = $290&255;
  $292 = $291 & 128;
  $293 = ($292|0)!=(0);
  $294 = HEAP32[$p1>>2]|0;
  if ($293) {
   $295 = (_sqlite3Fts3GetVarint32($294,$iCol1)|0);
   $300 = $295;
  } else {
   $296 = HEAP8[$294>>0]|0;
   $297 = $296&255;
   HEAP32[$iCol1>>2] = $297;
   $300 = 1;
  }
  $298 = HEAP32[$p1>>2]|0;
  $299 = (($298) + ($300)|0);
  HEAP32[$p1>>2] = $299;
  $301 = HEAP32[$p2>>2]|0;
  $302 = ((($301)) + 1|0);
  HEAP32[$p2>>2] = $302;
  $303 = HEAP32[$p2>>2]|0;
  $304 = HEAP8[$303>>0]|0;
  $305 = $304&255;
  $306 = $305 & 128;
  $307 = ($306|0)!=(0);
  $308 = HEAP32[$p2>>2]|0;
  if ($307) {
   $309 = (_sqlite3Fts3GetVarint32($308,$iCol2)|0);
   $314 = $309;
  } else {
   $310 = HEAP8[$308>>0]|0;
   $311 = $310&255;
   HEAP32[$iCol2>>2] = $311;
   $314 = 1;
  }
  $312 = HEAP32[$p2>>2]|0;
  $313 = (($312) + ($314)|0);
  HEAP32[$p2>>2] = $313;
 }
 _fts3PoslistCopy(0,$p2);
 _fts3PoslistCopy(0,$p1);
 $354 = HEAP32[$p1>>2]|0;
 $355 = HEAP32[$5>>2]|0;
 HEAP32[$355>>2] = $354;
 $356 = HEAP32[$p2>>2]|0;
 $357 = HEAP32[$6>>2]|0;
 HEAP32[$357>>2] = $356;
 $358 = HEAP32[$1>>2]|0;
 $359 = HEAP32[$358>>2]|0;
 $360 = HEAP32[$p>>2]|0;
 $361 = ($359|0)==($360|0);
 if ($361) {
  HEAP32[$0>>2] = 0;
  $366 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($366|0);
 } else {
  $362 = HEAP32[$p>>2]|0;
  $363 = ((($362)) + 1|0);
  HEAP32[$p>>2] = $363;
  HEAP8[$362>>0] = 0;
  $364 = HEAP32[$p>>2]|0;
  $365 = HEAP32[$1>>2]|0;
  HEAP32[$365>>2] = $364;
  HEAP32[$0>>2] = 1;
  $366 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($366|0);
 }
 return (0)|0;
}
function _fts3GetDeltaVarint($pp,$pVal) {
 $pp = $pp|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pVal;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3Fts3GetVarint($3,$iVal)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($4)|0);
 HEAP32[$5>>2] = $7;
 $8 = $iVal;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (_i64Add(($17|0),($20|0),($10|0),($13|0))|0);
 $22 = tempRet0;
 $23 = $14;
 $24 = $23;
 HEAP32[$24>>2] = $21;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 STACKTOP = sp;return;
}
function _fts3PutDeltaVarint($pp,$piPrev,$0,$1) {
 $pp = $pp|0;
 $piPrev = $piPrev|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$2>>2] = $pp;
 HEAP32[$3>>2] = $piPrev;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = $17;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (_i64Subtract(($13|0),($16|0),($20|0),($23|0))|0);
 $25 = tempRet0;
 $26 = (_sqlite3Fts3PutVarint($10,$24,$25)|0);
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + ($26)|0);
 HEAP32[$27>>2] = $29;
 $30 = $4;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = $36;
 $38 = $37;
 HEAP32[$38>>2] = $32;
 $39 = (($37) + 4)|0;
 $40 = $39;
 HEAP32[$40>>2] = $35;
 STACKTOP = sp;return;
}
function _sqlite3Fts3MsrIncrRestart($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 28|0);
 HEAP32[$4>>2] = 1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 80|0);
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($16<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ((($20)) + 84|0);
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($22<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ((($26)) + 88|0);
  $28 = $27;
  $29 = $28;
  HEAP32[$29>>2] = 0;
  $30 = (($28) + 4)|0;
  $31 = $30;
  HEAP32[$31>>2] = 0;
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 STACKTOP = sp;return 0;
}
function _fts3EvalPhraseStart($pCsr,$bOptOk,$p) {
 $pCsr = $pCsr|0;
 $bOptOk = $bOptOk|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bHaveIncr = 0, $bIncrOk = 0, $i = 0, $iCol = 0, $or$cond = 0, $pSegcsr = 0, $pTab = 0, $pToken = 0, $pToken1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $pTab = sp + 32|0;
 $rc = sp + 28|0;
 $i = sp + 24|0;
 $bHaveIncr = sp + 20|0;
 $bIncrOk = sp + 16|0;
 $pToken = sp + 12|0;
 $iCol = sp + 8|0;
 $pToken1 = sp + 4|0;
 $pSegcsr = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $bOptOk;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bHaveIncr>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 52|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = HEAP32[$pTab>>2]|0;
  $12 = ((($11)) + 231|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($10|0)==($14|0);
  if ($15) {
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 64|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)<=(4);
   if ($19) {
    $20 = HEAP32[$2>>2]|0;
    $21 = ((($20)) + 64|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ($22|0)>(0);
    $25 = $23;
   } else {
    $25 = 0;
   }
  } else {
   $25 = 0;
  }
 } else {
  $25 = 0;
 }
 $24 = $25&1;
 HEAP32[$bIncrOk>>2] = $24;
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$bIncrOk>>2]|0;
  $27 = ($26|0)==(1);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 64|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 72|0);
  $36 = (($35) + (($33*24)|0)|0);
  HEAP32[$pToken>>2] = $36;
  $37 = HEAP32[$pToken>>2]|0;
  $38 = ((($37)) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0);
  if ($40) {
   label = 11;
  } else {
   $41 = HEAP32[$pToken>>2]|0;
   $42 = ((($41)) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)!=(0|0);
   if ($44) {
    $45 = HEAP32[$pToken>>2]|0;
    $46 = ((($45)) + 20|0);
    $47 = HEAP32[$46>>2]|0;
    $48 = ((($47)) + 36|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = ($49|0)!=(0);
    if (!($50)) {
     label = 11;
    }
   }
  }
  if ((label|0) == 11) {
   label = 0;
   HEAP32[$bIncrOk>>2] = 0;
  }
  $51 = HEAP32[$pToken>>2]|0;
  $52 = ((($51)) + 20|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($53|0)!=(0|0);
  if ($54) {
   HEAP32[$bHaveIncr>>2] = 1;
  }
  $55 = HEAP32[$i>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$i>>2] = $56;
 }
 $57 = HEAP32[$bIncrOk>>2]|0;
 $58 = ($57|0)!=(0);
 $59 = HEAP32[$bHaveIncr>>2]|0;
 $60 = ($59|0)!=(0);
 $or$cond = $58 & $60;
 if (!($or$cond)) {
  $101 = HEAP32[$0>>2]|0;
  $102 = HEAP32[$2>>2]|0;
  $103 = (_fts3EvalPhraseLoad($101,$102)|0);
  HEAP32[$rc>>2] = $103;
  $104 = HEAP32[$2>>2]|0;
  $105 = ((($104)) + 40|0);
  HEAP32[$105>>2] = 0;
  $106 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $61 = HEAP32[$2>>2]|0;
 $62 = ((($61)) + 68|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = HEAP32[$pTab>>2]|0;
 $65 = ((($64)) + 24|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($63|0)>=($66|0);
 if ($67) {
  $71 = -1;
 } else {
  $68 = HEAP32[$2>>2]|0;
  $69 = ((($68)) + 68|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $70;
 }
 HEAP32[$iCol>>2] = $71;
 HEAP32[$i>>2] = 0;
 while(1) {
  $72 = HEAP32[$rc>>2]|0;
  $73 = ($72|0)==(0);
  if (!($73)) {
   break;
  }
  $74 = HEAP32[$i>>2]|0;
  $75 = HEAP32[$2>>2]|0;
  $76 = ((($75)) + 64|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = ($74|0)<($77|0);
  if (!($78)) {
   break;
  }
  $79 = HEAP32[$i>>2]|0;
  $80 = HEAP32[$2>>2]|0;
  $81 = ((($80)) + 72|0);
  $82 = (($81) + (($79*24)|0)|0);
  HEAP32[$pToken1>>2] = $82;
  $83 = HEAP32[$pToken1>>2]|0;
  $84 = ((($83)) + 20|0);
  $85 = HEAP32[$84>>2]|0;
  HEAP32[$pSegcsr>>2] = $85;
  $86 = HEAP32[$pSegcsr>>2]|0;
  $87 = ($86|0)!=(0|0);
  if ($87) {
   $88 = HEAP32[$pTab>>2]|0;
   $89 = HEAP32[$pSegcsr>>2]|0;
   $90 = HEAP32[$iCol>>2]|0;
   $91 = HEAP32[$pToken1>>2]|0;
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$pToken1>>2]|0;
   $94 = ((($93)) + 4|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = (_sqlite3Fts3MsrIncrStart($88,$89,$90,$92,$95)|0);
   HEAP32[$rc>>2] = $96;
  }
  $97 = HEAP32[$i>>2]|0;
  $98 = (($97) + 1)|0;
  HEAP32[$i>>2] = $98;
 }
 $99 = HEAP32[$2>>2]|0;
 $100 = ((($99)) + 40|0);
 HEAP32[$100>>2] = 1;
 $106 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($106|0);
}
function _sqlite3Fts3MsrIncrStart($p,$pCsr,$iCol,$zTerm,$nTerm) {
 $p = $p|0;
 $pCsr = $pCsr|0;
 $iCol = $iCol|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $i = 0, $nSegment = 0, $pSeg = 0, $rc = 0, $xCmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $i = sp + 16|0;
 $rc = sp + 12|0;
 $nSegment = sp + 8|0;
 $xCmp = sp + 4|0;
 $pSeg = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$3>>2] = $iCol;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nSegment>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 231|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)!=(0);
 $14 = $13 ? 182 : 183;
 HEAP32[$xCmp>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 $19 = (_fts3SegReaderStart($15,$16,$17,$18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  $22 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $22;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$nSegment>>2]|0;
  $25 = ($23|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($26<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pSeg>>2] = $30;
  $31 = HEAP32[$pSeg>>2]|0;
  $32 = ((($31)) + 40|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$pSeg>>2]|0;
  $36 = HEAP32[$4>>2]|0;
  $37 = HEAP32[$5>>2]|0;
  $38 = (_fts3SegReaderTermCmp($35,$36,$37)|0);
  $39 = ($38|0)!=(0);
  if ($39) {
   break;
  }
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = HEAP32[$i>>2]|0;
 $43 = HEAP32[$2>>2]|0;
 $44 = ((($43)) + 8|0);
 HEAP32[$44>>2] = $42;
 HEAP32[$i>>2] = 0;
 while(1) {
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = ((($46)) + 8|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($45|0)<($48|0);
  if (!($49)) {
   label = 13;
   break;
  }
  $50 = HEAP32[$1>>2]|0;
  $51 = HEAP32[$i>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($53) + ($51<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = (_fts3SegReaderFirstDocid($50,$55)|0);
  HEAP32[$rc>>2] = $56;
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)!=(0);
  if ($58) {
   label = 11;
   break;
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 if ((label|0) == 11) {
  $59 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $59;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 else if ((label|0) == 13) {
  $62 = HEAP32[$2>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$xCmp>>2]|0;
  _fts3SegReaderSort($63,$64,$65,$66);
  $67 = HEAP32[$3>>2]|0;
  $68 = HEAP32[$2>>2]|0;
  $69 = ((($68)) + 24|0);
  HEAP32[$69>>2] = $67;
  HEAP32[$0>>2] = 0;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 return (0)|0;
}
function _fts3EvalPhraseLoad($pCsr,$p) {
 $pCsr = $pCsr|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iToken = 0, $nThis = 0, $pTab = 0, $pThis = 0;
 var $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pTab = sp + 20|0;
 $iToken = sp + 16|0;
 $rc = sp + 12|0;
 $pToken = sp + 8|0;
 $nThis = sp + 4|0;
 $pThis = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTab>>2] = $3;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iToken>>2] = 0;
 while(1) {
  $4 = HEAP32[$rc>>2]|0;
  $5 = ($4|0)==(0);
  if (!($5)) {
   label = 8;
   break;
  }
  $6 = HEAP32[$iToken>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 64|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   label = 8;
   break;
  }
  $11 = HEAP32[$iToken>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 72|0);
  $14 = (($13) + (($11*24)|0)|0);
  HEAP32[$pToken>>2] = $14;
  $15 = HEAP32[$pToken>>2]|0;
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0|0);
  if ($18) {
   HEAP32[$nThis>>2] = 0;
   HEAP32[$pThis>>2] = 0;
   $19 = HEAP32[$pTab>>2]|0;
   $20 = HEAP32[$pToken>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 68|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_fts3TermSelect($19,$20,$23,$nThis,$pThis)|0);
   HEAP32[$rc>>2] = $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(0);
   if ($26) {
    $27 = HEAP32[$pTab>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = HEAP32[$iToken>>2]|0;
    $30 = HEAP32[$pThis>>2]|0;
    $31 = HEAP32[$nThis>>2]|0;
    $32 = (_fts3EvalPhraseMergeToken($27,$28,$29,$30,$31)|0);
    HEAP32[$rc>>2] = $32;
   }
  }
  $33 = HEAP32[$iToken>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$iToken>>2] = $34;
 }
 if ((label|0) == 8) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _fts3TermSelect($p,$pTok,$iColumn,$pnOut,$ppOut) {
 $p = $p|0;
 $pTok = $pTok|0;
 $iColumn = $iColumn|0;
 $pnOut = $pnOut|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $filter = 0, $i = 0, $pSegcsr = 0, $rc = 0, $tsc = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $3 = sp + 164|0;
 $4 = sp + 160|0;
 $rc = sp + 156|0;
 $pSegcsr = sp + 152|0;
 $tsc = sp + 24|0;
 $filter = sp + 8|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pTok;
 HEAP32[$2>>2] = $iColumn;
 HEAP32[$3>>2] = $pnOut;
 HEAP32[$4>>2] = $ppOut;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pSegcsr>>2] = $7;
 dest=$tsc; stop=dest+128|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 $12 = $11 ? 8 : 0;
 $13 = 3 | $12;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = $17 ? 32 : 0;
 $19 = $13 | $18;
 $20 = HEAP32[$2>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 24|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($20|0)<($23|0);
 $25 = $24 ? 4 : 0;
 $26 = $19 | $25;
 $27 = ((($filter)) + 12|0);
 HEAP32[$27>>2] = $26;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($filter)) + 8|0);
 HEAP32[$29>>2] = $28;
 $30 = HEAP32[$1>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$filter>>2] = $31;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ((($filter)) + 4|0);
 HEAP32[$35>>2] = $34;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pSegcsr>>2]|0;
 $38 = (_sqlite3Fts3SegReaderStart($36,$37,$filter)|0);
 HEAP32[$rc>>2] = $38;
 while(1) {
  $39 = HEAP32[$rc>>2]|0;
  $40 = (0)==($39|0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$pSegcsr>>2]|0;
  $43 = (_sqlite3Fts3SegReaderStep($41,$42)|0);
  HEAP32[$rc>>2] = $43;
  $44 = (100)==($43|0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$0>>2]|0;
  $46 = HEAP32[$pSegcsr>>2]|0;
  $47 = ((($46)) + 48|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$pSegcsr>>2]|0;
  $50 = ((($49)) + 52|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (_fts3TermSelectMerge($45,$tsc,$48,$51)|0);
  HEAP32[$rc>>2] = $52;
 }
 $53 = HEAP32[$rc>>2]|0;
 $54 = ($53|0)==(0);
 if ($54) {
  $55 = HEAP32[$0>>2]|0;
  $56 = (_fts3TermSelectFinishMerge($55,$tsc)|0);
  HEAP32[$rc>>2] = $56;
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if ($58) {
  $59 = HEAP32[$tsc>>2]|0;
  $60 = HEAP32[$4>>2]|0;
  HEAP32[$60>>2] = $59;
  $61 = ((($tsc)) + 64|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  $71 = HEAP32[$pSegcsr>>2]|0;
  _fts3SegReaderCursorFree($71);
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 20|0);
  HEAP32[$73>>2] = 0;
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $64 = HEAP32[$i>>2]|0;
  $65 = ($64|0)<(16);
  if (!($65)) {
   break;
  }
  $66 = HEAP32[$i>>2]|0;
  $67 = (($tsc) + ($66<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  _sqlite3_free($68);
  $69 = HEAP32[$i>>2]|0;
  $70 = (($69) + 1)|0;
  HEAP32[$i>>2] = $70;
 }
 $71 = HEAP32[$pSegcsr>>2]|0;
 _fts3SegReaderCursorFree($71);
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 20|0);
 HEAP32[$73>>2] = 0;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _fts3EvalPhraseMergeToken($pTab,$p,$iToken,$pList,$nList) {
 $pTab = $pTab|0;
 $p = $p|0;
 $iToken = $iToken|0;
 $pList = $pList|0;
 $nList = $nList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $nDiff = 0, $nLeft = 0, $nRight = 0, $pLeft = 0, $pRight = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $rc = sp + 20|0;
 $pLeft = sp + 16|0;
 $pRight = sp + 12|0;
 $nLeft = sp + 8|0;
 $nRight = sp + 4|0;
 $nDiff = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iToken;
 HEAP32[$3>>2] = $pList;
 HEAP32[$4>>2] = $nList;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)==(0|0);
 $7 = HEAP32[$1>>2]|0;
 do {
  if ($6) {
   $8 = HEAP32[$7>>2]|0;
   _sqlite3_free($8);
   $9 = HEAP32[$1>>2]|0;
   HEAP32[$9>>2] = 0;
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 4|0);
   HEAP32[$11>>2] = 0;
  } else {
   $12 = ((($7)) + 44|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)<(0);
   if ($14) {
    $15 = HEAP32[$3>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    HEAP32[$16>>2] = $15;
    $17 = HEAP32[$4>>2]|0;
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 4|0);
    HEAP32[$19>>2] = $17;
    break;
   }
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(0|0);
   if ($22) {
    $23 = HEAP32[$3>>2]|0;
    _sqlite3_free($23);
    break;
   }
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 44|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = ($26|0)<($27|0);
   $29 = HEAP32[$1>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   if ($28) {
    HEAP32[$pLeft>>2] = $30;
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$nLeft>>2] = $33;
    $34 = HEAP32[$3>>2]|0;
    HEAP32[$pRight>>2] = $34;
    $35 = HEAP32[$4>>2]|0;
    HEAP32[$nRight>>2] = $35;
    $36 = HEAP32[$2>>2]|0;
    $37 = HEAP32[$1>>2]|0;
    $38 = ((($37)) + 44|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($36) - ($39))|0;
    HEAP32[$nDiff>>2] = $40;
   } else {
    HEAP32[$pRight>>2] = $30;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 4|0);
    $43 = HEAP32[$42>>2]|0;
    HEAP32[$nRight>>2] = $43;
    $44 = HEAP32[$3>>2]|0;
    HEAP32[$pLeft>>2] = $44;
    $45 = HEAP32[$4>>2]|0;
    HEAP32[$nLeft>>2] = $45;
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 44|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = HEAP32[$2>>2]|0;
    $50 = (($48) - ($49))|0;
    HEAP32[$nDiff>>2] = $50;
   }
   $51 = HEAP32[$0>>2]|0;
   $52 = ((($51)) + 231|0);
   $53 = HEAP8[$52>>0]|0;
   $54 = $53&255;
   $55 = HEAP32[$nDiff>>2]|0;
   $56 = HEAP32[$pLeft>>2]|0;
   $57 = HEAP32[$nLeft>>2]|0;
   $58 = (_fts3DoclistPhraseMerge($54,$55,$56,$57,$pRight,$nRight)|0);
   HEAP32[$rc>>2] = $58;
   $59 = HEAP32[$pLeft>>2]|0;
   _sqlite3_free($59);
   $60 = HEAP32[$pRight>>2]|0;
   $61 = HEAP32[$1>>2]|0;
   HEAP32[$61>>2] = $60;
   $62 = HEAP32[$nRight>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 4|0);
   HEAP32[$64>>2] = $62;
  }
 } while(0);
 $65 = HEAP32[$2>>2]|0;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 44|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ($65|0)>($68|0);
 if (!($69)) {
  $73 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $70 = HEAP32[$2>>2]|0;
 $71 = HEAP32[$1>>2]|0;
 $72 = ((($71)) + 44|0);
 HEAP32[$72>>2] = $70;
 $73 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _fts3DoclistPhraseMerge($bDescDoclist,$nDist,$aLeft,$nLeft,$paRight,$pnRight) {
 $bDescDoclist = $bDescDoclist|0;
 $nDist = $nDist|0;
 $aLeft = $aLeft|0;
 $nLeft = $nLeft|0;
 $paRight = $paRight|0;
 $pnRight = $pnRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $aOut = 0, $aRight = 0, $bFirstOut = 0, $bFirstOutSave = 0, $i1 = 0, $i2 = 0, $iDiff = 0, $iPrev = 0, $iPrevSave = 0, $p = 0, $p1 = 0, $p2 = 0, $pEnd1 = 0, $pEnd2 = 0, $pSave = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp + 80|0;
 $i1 = sp + 32|0;
 $i2 = sp + 24|0;
 $iPrev = sp + 16|0;
 $aRight = sp + 76|0;
 $pEnd1 = sp + 72|0;
 $pEnd2 = sp + 68|0;
 $p1 = sp + 64|0;
 $p2 = sp + 60|0;
 $p = sp + 56|0;
 $bFirstOut = sp + 52|0;
 $aOut = sp + 48|0;
 $iDiff = sp + 8|0;
 $pSave = sp + 44|0;
 $iPrevSave = sp;
 $bFirstOutSave = sp + 40|0;
 HEAP32[$1>>2] = $bDescDoclist;
 HEAP32[$2>>2] = $nDist;
 HEAP32[$3>>2] = $aLeft;
 HEAP32[$4>>2] = $nLeft;
 HEAP32[$5>>2] = $paRight;
 HEAP32[$6>>2] = $pnRight;
 $7 = $i1;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = $i2;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = $iPrev;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$5>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$aRight>>2] = $20;
 $21 = HEAP32[$4>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = (($22) + ($21)|0);
 HEAP32[$pEnd1>>2] = $23;
 $24 = HEAP32[$6>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$aRight>>2]|0;
 $27 = (($26) + ($25)|0);
 HEAP32[$pEnd2>>2] = $27;
 $28 = HEAP32[$3>>2]|0;
 HEAP32[$p1>>2] = $28;
 $29 = HEAP32[$aRight>>2]|0;
 HEAP32[$p2>>2] = $29;
 HEAP32[$bFirstOut>>2] = 0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($30|0)!=(0);
 if ($31) {
  $32 = HEAP32[$6>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + 10)|0;
  $35 = (_sqlite3_malloc($34)|0);
  HEAP32[$aOut>>2] = $35;
  $36 = HEAP32[$aOut>>2]|0;
  $37 = ($36|0)==(0|0);
  if ($37) {
   HEAP32[$0>>2] = 7;
   $141 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($141|0);
  }
 } else {
  $38 = HEAP32[$aRight>>2]|0;
  HEAP32[$aOut>>2] = $38;
 }
 $39 = HEAP32[$aOut>>2]|0;
 HEAP32[$p>>2] = $39;
 $40 = HEAP32[$pEnd1>>2]|0;
 _fts3GetDeltaVarint3($p1,$40,0,$i1);
 $41 = HEAP32[$pEnd2>>2]|0;
 _fts3GetDeltaVarint3($p2,$41,0,$i2);
 while(1) {
  $42 = HEAP32[$p1>>2]|0;
  $43 = ($42|0)!=(0|0);
  $44 = HEAP32[$p2>>2]|0;
  $45 = ($44|0)!=(0|0);
  $46 = $43 ? $45 : 0;
  if (!($46)) {
   break;
  }
  $47 = HEAP32[$1>>2]|0;
  $48 = ($47|0)!=(0);
  $49 = $48 ? -1 : 1;
  $50 = ($49|0)<(0);
  $51 = $50 << 31 >> 31;
  $52 = $i1;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) + 4)|0;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = $i2;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = (_i64Subtract(($54|0),($57|0),($60|0),($63|0))|0);
  $65 = tempRet0;
  $66 = (___muldi3(($49|0),($51|0),($64|0),($65|0))|0);
  $67 = tempRet0;
  $68 = $iDiff;
  $69 = $68;
  HEAP32[$69>>2] = $66;
  $70 = (($68) + 4)|0;
  $71 = $70;
  HEAP32[$71>>2] = $67;
  $72 = $iDiff;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = ($74|0)==(0);
  $79 = ($77|0)==(0);
  $80 = $78 & $79;
  if ($80) {
   $81 = HEAP32[$p>>2]|0;
   HEAP32[$pSave>>2] = $81;
   $82 = $iPrev;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = (($82) + 4)|0;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = $iPrevSave;
   $89 = $88;
   HEAP32[$89>>2] = $84;
   $90 = (($88) + 4)|0;
   $91 = $90;
   HEAP32[$91>>2] = $87;
   $92 = HEAP32[$bFirstOut>>2]|0;
   HEAP32[$bFirstOutSave>>2] = $92;
   $93 = HEAP32[$1>>2]|0;
   $94 = $i1;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($94) + 4)|0;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   _fts3PutDeltaVarint3($p,$93,$iPrev,$bFirstOut,$96,$99);
   $100 = HEAP32[$2>>2]|0;
   $101 = (_fts3PoslistPhraseMerge($p,$100,0,1,$p1,$p2)|0);
   $102 = (0)==($101|0);
   if ($102) {
    $103 = HEAP32[$pSave>>2]|0;
    HEAP32[$p>>2] = $103;
    $104 = $iPrevSave;
    $105 = $104;
    $106 = HEAP32[$105>>2]|0;
    $107 = (($104) + 4)|0;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = $iPrev;
    $111 = $110;
    HEAP32[$111>>2] = $106;
    $112 = (($110) + 4)|0;
    $113 = $112;
    HEAP32[$113>>2] = $109;
    $114 = HEAP32[$bFirstOutSave>>2]|0;
    HEAP32[$bFirstOut>>2] = $114;
   }
   $115 = HEAP32[$pEnd1>>2]|0;
   $116 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p1,$115,$116,$i1);
   $117 = HEAP32[$pEnd2>>2]|0;
   $118 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p2,$117,$118,$i2);
   continue;
  } else {
   $119 = $iDiff;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = (($119) + 4)|0;
   $123 = $122;
   $124 = HEAP32[$123>>2]|0;
   $125 = ($124|0)<(0);
   if ($125) {
    _fts3PoslistCopy(0,$p1);
    $126 = HEAP32[$pEnd1>>2]|0;
    $127 = HEAP32[$1>>2]|0;
    _fts3GetDeltaVarint3($p1,$126,$127,$i1);
    continue;
   } else {
    _fts3PoslistCopy(0,$p2);
    $128 = HEAP32[$pEnd2>>2]|0;
    $129 = HEAP32[$1>>2]|0;
    _fts3GetDeltaVarint3($p2,$128,$129,$i2);
    continue;
   }
  }
 }
 $130 = HEAP32[$p>>2]|0;
 $131 = HEAP32[$aOut>>2]|0;
 $132 = $130;
 $133 = $131;
 $134 = (($132) - ($133))|0;
 $135 = HEAP32[$6>>2]|0;
 HEAP32[$135>>2] = $134;
 $136 = HEAP32[$1>>2]|0;
 $137 = ($136|0)!=(0);
 if ($137) {
  $138 = HEAP32[$aRight>>2]|0;
  _sqlite3_free($138);
  $139 = HEAP32[$aOut>>2]|0;
  $140 = HEAP32[$5>>2]|0;
  HEAP32[$140>>2] = $139;
 }
 HEAP32[$0>>2] = 0;
 $141 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($141|0);
}
function _fts3GetDeltaVarint3($pp,$pEnd,$bDescIdx,$pVal) {
 $pp = $pp|0;
 $pEnd = $pEnd|0;
 $bDescIdx = $bDescIdx|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pEnd;
 HEAP32[$2>>2] = $bDescIdx;
 HEAP32[$3>>2] = $pVal;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5>>>0)>=($6>>>0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  HEAP32[$8>>2] = 0;
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3Fts3GetVarint($9,$iVal)|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($10)|0);
 HEAP32[$11>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)!=(0);
 $16 = $iVal;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 if ($15) {
  $29 = (_i64Subtract(($25|0),($28|0),($18|0),($21|0))|0);
  $30 = tempRet0;
  $31 = $22;
  $32 = $31;
  HEAP32[$32>>2] = $29;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $30;
  STACKTOP = sp;return;
 } else {
  $35 = (_i64Add(($25|0),($28|0),($18|0),($21|0))|0);
  $36 = tempRet0;
  $37 = $22;
  $38 = $37;
  HEAP32[$38>>2] = $35;
  $39 = (($37) + 4)|0;
  $40 = $39;
  HEAP32[$40>>2] = $36;
  STACKTOP = sp;return;
 }
}
function _fts3PutDeltaVarint3($pp,$bDescIdx,$piPrev,$pbFirst,$0,$1) {
 $pp = $pp|0;
 $bDescIdx = $bDescIdx|0;
 $piPrev = $piPrev|0;
 $pbFirst = $pbFirst|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $iWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp + 8|0;
 $iWrite = sp;
 HEAP32[$2>>2] = $pp;
 HEAP32[$3>>2] = $bDescIdx;
 HEAP32[$4>>2] = $piPrev;
 HEAP32[$5>>2] = $pbFirst;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  label = 3;
 } else {
  $13 = HEAP32[$5>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0);
  if ($15) {
   label = 3;
  } else {
   $35 = HEAP32[$4>>2]|0;
   $36 = $35;
   $37 = $36;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($36) + 4)|0;
   $40 = $39;
   $41 = HEAP32[$40>>2]|0;
   $42 = $6;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = (_i64Subtract(($38|0),($41|0),($44|0),($47|0))|0);
   $49 = tempRet0;
   $50 = $iWrite;
   $51 = $50;
   HEAP32[$51>>2] = $48;
   $52 = (($50) + 4)|0;
   $53 = $52;
   HEAP32[$53>>2] = $49;
  }
 }
 if ((label|0) == 3) {
  $16 = $6;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  $23 = $22;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (_i64Subtract(($18|0),($21|0),($25|0),($28|0))|0);
  $30 = tempRet0;
  $31 = $iWrite;
  $32 = $31;
  HEAP32[$32>>2] = $29;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $30;
 }
 $54 = HEAP32[$2>>2]|0;
 $55 = HEAP32[$54>>2]|0;
 $56 = $iWrite;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = (($56) + 4)|0;
 $60 = $59;
 $61 = HEAP32[$60>>2]|0;
 $62 = (_sqlite3Fts3PutVarint($55,$58,$61)|0);
 $63 = HEAP32[$2>>2]|0;
 $64 = HEAP32[$63>>2]|0;
 $65 = (($64) + ($62)|0);
 HEAP32[$63>>2] = $65;
 $66 = $6;
 $67 = $66;
 $68 = HEAP32[$67>>2]|0;
 $69 = (($66) + 4)|0;
 $70 = $69;
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP32[$4>>2]|0;
 $73 = $72;
 $74 = $73;
 HEAP32[$74>>2] = $68;
 $75 = (($73) + 4)|0;
 $76 = $75;
 HEAP32[$76>>2] = $71;
 $77 = HEAP32[$5>>2]|0;
 HEAP32[$77>>2] = 1;
 STACKTOP = sp;return;
}
function _fts3TermSelectMerge($p,$pTS,$aDoclist,$nDoclist) {
 $p = $p|0;
 $pTS = $pTS|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $aMerge = 0, $aNew = 0, $iOut = 0, $nMerge = 0, $nNew = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $aMerge = sp + 20|0;
 $nMerge = sp + 16|0;
 $iOut = sp + 12|0;
 $aNew = sp + 8|0;
 $nNew = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pTS;
 HEAP32[$3>>2] = $aDoclist;
 HEAP32[$4>>2] = $nDoclist;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$4>>2]|0;
   $9 = (($8) + 10)|0;
   $10 = (($9) + 1)|0;
   $11 = (_sqlite3_malloc($10)|0);
   $12 = HEAP32[$2>>2]|0;
   HEAP32[$12>>2] = $11;
   $13 = HEAP32[$4>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 64|0);
   HEAP32[$15>>2] = $13;
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    $19 = HEAP32[$2>>2]|0;
    $20 = HEAP32[$19>>2]|0;
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$4>>2]|0;
    _memcpy(($20|0),($21|0),($22|0))|0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $88 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($88|0);
  } else {
   $23 = HEAP32[$3>>2]|0;
   HEAP32[$aMerge>>2] = $23;
   $24 = HEAP32[$4>>2]|0;
   HEAP32[$nMerge>>2] = $24;
   HEAP32[$iOut>>2] = 0;
   while(1) {
    $25 = HEAP32[$iOut>>2]|0;
    $26 = ($25|0)<(16);
    if (!($26)) {
     break L1;
    }
    $27 = HEAP32[$iOut>>2]|0;
    $28 = HEAP32[$2>>2]|0;
    $29 = (($28) + ($27<<2)|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     label = 8;
     break;
    }
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 231|0);
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = HEAP32[$aMerge>>2]|0;
    $46 = HEAP32[$nMerge>>2]|0;
    $47 = HEAP32[$iOut>>2]|0;
    $48 = HEAP32[$2>>2]|0;
    $49 = (($48) + ($47<<2)|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$iOut>>2]|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = ((($52)) + 64|0);
    $54 = (($53) + ($51<<2)|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (_fts3DoclistOrMerge($44,$45,$46,$50,$55,$aNew,$nNew)|0);
    HEAP32[$rc>>2] = $56;
    $57 = HEAP32[$rc>>2]|0;
    $58 = ($57|0)!=(0);
    $59 = HEAP32[$aMerge>>2]|0;
    $60 = HEAP32[$3>>2]|0;
    $61 = ($59|0)!=($60|0);
    if ($58) {
     break;
    }
    if ($61) {
     $64 = HEAP32[$aMerge>>2]|0;
     _sqlite3_free($64);
    }
    $65 = HEAP32[$iOut>>2]|0;
    $66 = HEAP32[$2>>2]|0;
    $67 = (($66) + ($65<<2)|0);
    $68 = HEAP32[$67>>2]|0;
    _sqlite3_free($68);
    $69 = HEAP32[$iOut>>2]|0;
    $70 = HEAP32[$2>>2]|0;
    $71 = (($70) + ($69<<2)|0);
    HEAP32[$71>>2] = 0;
    $72 = HEAP32[$aNew>>2]|0;
    HEAP32[$aMerge>>2] = $72;
    $73 = HEAP32[$nNew>>2]|0;
    HEAP32[$nMerge>>2] = $73;
    $74 = HEAP32[$iOut>>2]|0;
    $75 = (($74) + 1)|0;
    $76 = ($75|0)==(16);
    if ($76) {
     $77 = HEAP32[$aMerge>>2]|0;
     $78 = HEAP32[$iOut>>2]|0;
     $79 = HEAP32[$2>>2]|0;
     $80 = (($79) + ($78<<2)|0);
     HEAP32[$80>>2] = $77;
     $81 = HEAP32[$nMerge>>2]|0;
     $82 = HEAP32[$iOut>>2]|0;
     $83 = HEAP32[$2>>2]|0;
     $84 = ((($83)) + 64|0);
     $85 = (($84) + ($82<<2)|0);
     HEAP32[$85>>2] = $81;
    }
    $86 = HEAP32[$iOut>>2]|0;
    $87 = (($86) + 1)|0;
    HEAP32[$iOut>>2] = $87;
   }
   if ((label|0) == 8) {
    $32 = HEAP32[$aMerge>>2]|0;
    $33 = HEAP32[$iOut>>2]|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = (($34) + ($33<<2)|0);
    HEAP32[$35>>2] = $32;
    $36 = HEAP32[$nMerge>>2]|0;
    $37 = HEAP32[$iOut>>2]|0;
    $38 = HEAP32[$2>>2]|0;
    $39 = ((($38)) + 64|0);
    $40 = (($39) + ($37<<2)|0);
    HEAP32[$40>>2] = $36;
    break;
   }
   if ($61) {
    $62 = HEAP32[$aMerge>>2]|0;
    _sqlite3_free($62);
   }
   $63 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $63;
   $88 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($88|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $88 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($88|0);
}
function _fts3TermSelectFinishMerge($p,$pTS) {
 $p = $p|0;
 $pTS = $pTS|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aNew = 0, $aOut = 0, $i = 0, $nNew = 0, $nOut = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $aOut = sp + 20|0;
 $nOut = sp + 16|0;
 $i = sp + 12|0;
 $nNew = sp + 8|0;
 $aNew = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pTS;
 HEAP32[$aOut>>2] = 0;
 HEAP32[$nOut>>2] = 0;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = ($3|0)<(16);
  if (!($4)) {
   label = 10;
   break;
  }
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) + ($5<<2)|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)!=(0|0);
  do {
   if ($9) {
    $10 = HEAP32[$aOut>>2]|0;
    $11 = ($10|0)!=(0|0);
    if (!($11)) {
     $12 = HEAP32[$i>>2]|0;
     $13 = HEAP32[$2>>2]|0;
     $14 = (($13) + ($12<<2)|0);
     $15 = HEAP32[$14>>2]|0;
     HEAP32[$aOut>>2] = $15;
     $16 = HEAP32[$i>>2]|0;
     $17 = HEAP32[$2>>2]|0;
     $18 = ((($17)) + 64|0);
     $19 = (($18) + ($16<<2)|0);
     $20 = HEAP32[$19>>2]|0;
     HEAP32[$nOut>>2] = $20;
     $21 = HEAP32[$i>>2]|0;
     $22 = HEAP32[$2>>2]|0;
     $23 = (($22) + ($21<<2)|0);
     HEAP32[$23>>2] = 0;
     break;
    }
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 231|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = HEAP32[$i>>2]|0;
    $29 = HEAP32[$2>>2]|0;
    $30 = (($29) + ($28<<2)|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$i>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = ((($33)) + 64|0);
    $35 = (($34) + ($32<<2)|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$aOut>>2]|0;
    $38 = HEAP32[$nOut>>2]|0;
    $39 = (_fts3DoclistOrMerge($27,$31,$36,$37,$38,$aNew,$nNew)|0);
    HEAP32[$rc>>2] = $39;
    $40 = HEAP32[$rc>>2]|0;
    $41 = ($40|0)!=(0);
    if ($41) {
     label = 7;
     break L1;
    }
    $44 = HEAP32[$i>>2]|0;
    $45 = HEAP32[$2>>2]|0;
    $46 = (($45) + ($44<<2)|0);
    $47 = HEAP32[$46>>2]|0;
    _sqlite3_free($47);
    $48 = HEAP32[$aOut>>2]|0;
    _sqlite3_free($48);
    $49 = HEAP32[$i>>2]|0;
    $50 = HEAP32[$2>>2]|0;
    $51 = (($50) + ($49<<2)|0);
    HEAP32[$51>>2] = 0;
    $52 = HEAP32[$aNew>>2]|0;
    HEAP32[$aOut>>2] = $52;
    $53 = HEAP32[$nNew>>2]|0;
    HEAP32[$nOut>>2] = $53;
   }
  } while(0);
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 7) {
  $42 = HEAP32[$aOut>>2]|0;
  _sqlite3_free($42);
  $43 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $43;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 else if ((label|0) == 10) {
  $56 = HEAP32[$aOut>>2]|0;
  $57 = HEAP32[$2>>2]|0;
  HEAP32[$57>>2] = $56;
  $58 = HEAP32[$nOut>>2]|0;
  $59 = HEAP32[$2>>2]|0;
  $60 = ((($59)) + 64|0);
  HEAP32[$60>>2] = $58;
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _fts3SegReaderCursorFree($pSegcsr) {
 $pSegcsr = $pSegcsr|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pSegcsr;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegReaderFinish($1);
 $2 = HEAP32[$0>>2]|0;
 _sqlite3_free($2);
 STACKTOP = sp;return;
}
function _fts3DoclistOrMerge($bDescDoclist,$a1,$n1,$a2,$n2,$paOut,$pnOut) {
 $bDescDoclist = $bDescDoclist|0;
 $a1 = $a1|0;
 $n1 = $n1|0;
 $a2 = $a2|0;
 $n2 = $n2|0;
 $paOut = $paOut|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aOut = 0, $bFirstOut = 0, $i1 = 0, $i2 = 0, $iDiff = 0, $iPrev = 0, $or$cond = 0;
 var $or$cond3 = 0, $or$cond5 = 0, $p = 0, $p1 = 0, $p2 = 0, $pEnd1 = 0, $pEnd2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $i1 = sp + 24|0;
 $i2 = sp + 16|0;
 $iPrev = sp + 8|0;
 $pEnd1 = sp + 56|0;
 $pEnd2 = sp + 52|0;
 $p1 = sp + 48|0;
 $p2 = sp + 44|0;
 $p = sp + 40|0;
 $aOut = sp + 36|0;
 $bFirstOut = sp + 32|0;
 $iDiff = sp;
 HEAP32[$1>>2] = $bDescDoclist;
 HEAP32[$2>>2] = $a1;
 HEAP32[$3>>2] = $n1;
 HEAP32[$4>>2] = $a2;
 HEAP32[$5>>2] = $n2;
 HEAP32[$6>>2] = $paOut;
 HEAP32[$7>>2] = $pnOut;
 $8 = $i1;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = 0;
 $12 = $i2;
 $13 = $12;
 HEAP32[$13>>2] = 0;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = 0;
 $16 = $iPrev;
 $17 = $16;
 HEAP32[$17>>2] = 0;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (($21) + ($20)|0);
 HEAP32[$pEnd1>>2] = $22;
 $23 = HEAP32[$5>>2]|0;
 $24 = HEAP32[$4>>2]|0;
 $25 = (($24) + ($23)|0);
 HEAP32[$pEnd2>>2] = $25;
 $26 = HEAP32[$2>>2]|0;
 HEAP32[$p1>>2] = $26;
 $27 = HEAP32[$4>>2]|0;
 HEAP32[$p2>>2] = $27;
 HEAP32[$bFirstOut>>2] = 0;
 $28 = HEAP32[$6>>2]|0;
 HEAP32[$28>>2] = 0;
 $29 = HEAP32[$7>>2]|0;
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$3>>2]|0;
 $31 = HEAP32[$5>>2]|0;
 $32 = (($30) + ($31))|0;
 $33 = (($32) + 10)|0;
 $34 = (($33) - 1)|0;
 $35 = (_sqlite3_malloc($34)|0);
 HEAP32[$aOut>>2] = $35;
 $36 = HEAP32[$aOut>>2]|0;
 $37 = ($36|0)!=(0|0);
 if (!($37)) {
  HEAP32[$0>>2] = 7;
  $132 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($132|0);
 }
 $38 = HEAP32[$aOut>>2]|0;
 HEAP32[$p>>2] = $38;
 $39 = HEAP32[$pEnd1>>2]|0;
 _fts3GetDeltaVarint3($p1,$39,0,$i1);
 $40 = HEAP32[$pEnd2>>2]|0;
 _fts3GetDeltaVarint3($p2,$40,0,$i2);
 while(1) {
  $41 = HEAP32[$p1>>2]|0;
  $42 = ($41|0)!=(0|0);
  $43 = HEAP32[$p2>>2]|0;
  $44 = ($43|0)!=(0|0);
  $45 = $42 ? 1 : $44;
  if (!($45)) {
   break;
  }
  $46 = HEAP32[$1>>2]|0;
  $47 = ($46|0)!=(0);
  $48 = $47 ? -1 : 1;
  $49 = ($48|0)<(0);
  $50 = $49 << 31 >> 31;
  $51 = $i1;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = (($51) + 4)|0;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = $i2;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_i64Subtract(($53|0),($56|0),($59|0),($62|0))|0);
  $64 = tempRet0;
  $65 = (___muldi3(($48|0),($50|0),($63|0),($64|0))|0);
  $66 = tempRet0;
  $67 = $iDiff;
  $68 = $67;
  HEAP32[$68>>2] = $65;
  $69 = (($67) + 4)|0;
  $70 = $69;
  HEAP32[$70>>2] = $66;
  $71 = HEAP32[$p2>>2]|0;
  $72 = ($71|0)!=(0|0);
  $73 = HEAP32[$p1>>2]|0;
  $74 = ($73|0)!=(0|0);
  $or$cond = $72 & $74;
  $75 = $iDiff;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = ($77|0)==(0);
  $82 = ($80|0)==(0);
  $83 = $81 & $82;
  $or$cond3 = $or$cond & $83;
  if ($or$cond3) {
   $84 = HEAP32[$1>>2]|0;
   $85 = $i1;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = (($85) + 4)|0;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   _fts3PutDeltaVarint3($p,$84,$iPrev,$bFirstOut,$87,$90);
   _fts3PoslistMerge($p,$p1,$p2);
   $91 = HEAP32[$pEnd1>>2]|0;
   $92 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p1,$91,$92,$i1);
   $93 = HEAP32[$pEnd2>>2]|0;
   $94 = HEAP32[$1>>2]|0;
   _fts3GetDeltaVarint3($p2,$93,$94,$i2);
   continue;
  }
  $95 = HEAP32[$p2>>2]|0;
  $96 = ($95|0)!=(0|0);
  if ($96) {
   $97 = HEAP32[$p1>>2]|0;
   $98 = ($97|0)!=(0|0);
   $99 = $iDiff;
   $100 = $99;
   $101 = HEAP32[$100>>2]|0;
   $102 = (($99) + 4)|0;
   $103 = $102;
   $104 = HEAP32[$103>>2]|0;
   $105 = ($104|0)<(0);
   $or$cond5 = $98 & $105;
   if (!($or$cond5)) {
    $115 = HEAP32[$1>>2]|0;
    $116 = $i2;
    $117 = $116;
    $118 = HEAP32[$117>>2]|0;
    $119 = (($116) + 4)|0;
    $120 = $119;
    $121 = HEAP32[$120>>2]|0;
    _fts3PutDeltaVarint3($p,$115,$iPrev,$bFirstOut,$118,$121);
    _fts3PoslistCopy($p,$p2);
    $122 = HEAP32[$pEnd2>>2]|0;
    $123 = HEAP32[$1>>2]|0;
    _fts3GetDeltaVarint3($p2,$122,$123,$i2);
    continue;
   }
  }
  $106 = HEAP32[$1>>2]|0;
  $107 = $i1;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = (($107) + 4)|0;
  $111 = $110;
  $112 = HEAP32[$111>>2]|0;
  _fts3PutDeltaVarint3($p,$106,$iPrev,$bFirstOut,$109,$112);
  _fts3PoslistCopy($p,$p1);
  $113 = HEAP32[$pEnd1>>2]|0;
  $114 = HEAP32[$1>>2]|0;
  _fts3GetDeltaVarint3($p1,$113,$114,$i1);
 }
 $124 = HEAP32[$aOut>>2]|0;
 $125 = HEAP32[$6>>2]|0;
 HEAP32[$125>>2] = $124;
 $126 = HEAP32[$p>>2]|0;
 $127 = HEAP32[$aOut>>2]|0;
 $128 = $126;
 $129 = $127;
 $130 = (($128) - ($129))|0;
 $131 = HEAP32[$7>>2]|0;
 HEAP32[$131>>2] = $130;
 HEAP32[$0>>2] = 0;
 $132 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _fts3PoslistMerge($pp,$pp1,$pp2) {
 $pp = $pp|0;
 $pp1 = $pp1|0;
 $pp2 = $pp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i1 = 0, $i2 = 0, $iCol1 = 0, $iCol2 = 0, $iPrev = 0, $n = 0, $p = 0, $p1 = 0, $p2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $p = sp + 44|0;
 $p1 = sp + 40|0;
 $p2 = sp + 36|0;
 $iCol1 = sp + 32|0;
 $iCol2 = sp + 28|0;
 $i1 = sp + 16|0;
 $i2 = sp + 8|0;
 $iPrev = sp;
 $n = sp + 24|0;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pp1;
 HEAP32[$2>>2] = $pp2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p1>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p2>>2] = $8;
 L1: while(1) {
  $9 = HEAP32[$p1>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   $13 = HEAP32[$p2>>2]|0;
   $14 = HEAP8[$13>>0]|0;
   $15 = $14 << 24 >> 24;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    break;
   }
  }
  $17 = HEAP32[$p1>>2]|0;
  $18 = HEAP8[$17>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = ($19|0)==(1);
  $21 = HEAP32[$p1>>2]|0;
  do {
   if ($20) {
    $22 = ((($21)) + 1|0);
    $23 = HEAP8[$22>>0]|0;
    $24 = $23&255;
    $25 = $24 & 128;
    $26 = ($25|0)!=(0);
    $27 = HEAP32[$p1>>2]|0;
    $28 = ((($27)) + 1|0);
    if ($26) {
     (_sqlite3Fts3GetVarint32($28,$iCol1)|0);
     break;
    } else {
     $29 = HEAP8[$28>>0]|0;
     $30 = $29&255;
     HEAP32[$iCol1>>2] = $30;
     break;
    }
   } else {
    $31 = HEAP8[$21>>0]|0;
    $32 = $31 << 24 >> 24;
    $33 = ($32|0)==(0);
    if ($33) {
     HEAP32[$iCol1>>2] = 2147483647;
     break;
    } else {
     HEAP32[$iCol1>>2] = 0;
     break;
    }
   }
  } while(0);
  $34 = HEAP32[$p2>>2]|0;
  $35 = HEAP8[$34>>0]|0;
  $36 = $35 << 24 >> 24;
  $37 = ($36|0)==(1);
  $38 = HEAP32[$p2>>2]|0;
  do {
   if ($37) {
    $39 = ((($38)) + 1|0);
    $40 = HEAP8[$39>>0]|0;
    $41 = $40&255;
    $42 = $41 & 128;
    $43 = ($42|0)!=(0);
    $44 = HEAP32[$p2>>2]|0;
    $45 = ((($44)) + 1|0);
    if ($43) {
     (_sqlite3Fts3GetVarint32($45,$iCol2)|0);
     break;
    } else {
     $46 = HEAP8[$45>>0]|0;
     $47 = $46&255;
     HEAP32[$iCol2>>2] = $47;
     break;
    }
   } else {
    $48 = HEAP8[$38>>0]|0;
    $49 = $48 << 24 >> 24;
    $50 = ($49|0)==(0);
    if ($50) {
     HEAP32[$iCol2>>2] = 2147483647;
     break;
    } else {
     HEAP32[$iCol2>>2] = 0;
     break;
    }
   }
  } while(0);
  $51 = HEAP32[$iCol1>>2]|0;
  $52 = HEAP32[$iCol2>>2]|0;
  $53 = ($51|0)==($52|0);
  if (!($53)) {
   $168 = HEAP32[$iCol1>>2]|0;
   $169 = HEAP32[$iCol2>>2]|0;
   $170 = ($168|0)<($169|0);
   if ($170) {
    $171 = HEAP32[$iCol1>>2]|0;
    $172 = (_fts3PutColNumber($p,$171)|0);
    $173 = HEAP32[$p1>>2]|0;
    $174 = (($173) + ($172)|0);
    HEAP32[$p1>>2] = $174;
    _fts3ColumnlistCopy($p,$p1);
    continue;
   } else {
    $175 = HEAP32[$iCol2>>2]|0;
    $176 = (_fts3PutColNumber($p,$175)|0);
    $177 = HEAP32[$p2>>2]|0;
    $178 = (($177) + ($176)|0);
    HEAP32[$p2>>2] = $178;
    _fts3ColumnlistCopy($p,$p2);
    continue;
   }
  }
  $54 = $i1;
  $55 = $54;
  HEAP32[$55>>2] = 0;
  $56 = (($54) + 4)|0;
  $57 = $56;
  HEAP32[$57>>2] = 0;
  $58 = $i2;
  $59 = $58;
  HEAP32[$59>>2] = 0;
  $60 = (($58) + 4)|0;
  $61 = $60;
  HEAP32[$61>>2] = 0;
  $62 = $iPrev;
  $63 = $62;
  HEAP32[$63>>2] = 0;
  $64 = (($62) + 4)|0;
  $65 = $64;
  HEAP32[$65>>2] = 0;
  $66 = HEAP32[$iCol1>>2]|0;
  $67 = (_fts3PutColNumber($p,$66)|0);
  HEAP32[$n>>2] = $67;
  $68 = HEAP32[$n>>2]|0;
  $69 = HEAP32[$p1>>2]|0;
  $70 = (($69) + ($68)|0);
  HEAP32[$p1>>2] = $70;
  $71 = HEAP32[$n>>2]|0;
  $72 = HEAP32[$p2>>2]|0;
  $73 = (($72) + ($71)|0);
  HEAP32[$p2>>2] = $73;
  _fts3GetDeltaVarint($p1,$i1);
  _fts3GetDeltaVarint($p2,$i2);
  while(1) {
   $74 = $i1;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($74) + 4)|0;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = $i2;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $83 = (($80) + 4)|0;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = ($79|0)<($85|0);
   $87 = ($76>>>0)<($82>>>0);
   $88 = ($79|0)==($85|0);
   $89 = $88 & $87;
   $90 = $86 | $89;
   $91 = $i1;
   $92 = $91;
   $93 = HEAP32[$92>>2]|0;
   $94 = (($91) + 4)|0;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = $i2;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = (($97) + 4)|0;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = $90 ? $93 : $99;
   $104 = $90 ? $96 : $102;
   _fts3PutDeltaVarint($p,$iPrev,$103,$104);
   $105 = $iPrev;
   $106 = $105;
   $107 = HEAP32[$106>>2]|0;
   $108 = (($105) + 4)|0;
   $109 = $108;
   $110 = HEAP32[$109>>2]|0;
   $111 = (_i64Subtract(($107|0),($110|0),2,0)|0);
   $112 = tempRet0;
   $113 = $iPrev;
   $114 = $113;
   HEAP32[$114>>2] = $111;
   $115 = (($113) + 4)|0;
   $116 = $115;
   HEAP32[$116>>2] = $112;
   $117 = $i1;
   $118 = $117;
   $119 = HEAP32[$118>>2]|0;
   $120 = (($117) + 4)|0;
   $121 = $120;
   $122 = HEAP32[$121>>2]|0;
   $123 = $i2;
   $124 = $123;
   $125 = HEAP32[$124>>2]|0;
   $126 = (($123) + 4)|0;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = ($119|0)==($125|0);
   $130 = ($122|0)==($128|0);
   $131 = $129 & $130;
   do {
    if ($131) {
     _fts3ReadNextPos($p1,$i1);
     _fts3ReadNextPos($p2,$i2);
    } else {
     $132 = $i1;
     $133 = $132;
     $134 = HEAP32[$133>>2]|0;
     $135 = (($132) + 4)|0;
     $136 = $135;
     $137 = HEAP32[$136>>2]|0;
     $138 = $i2;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = (($138) + 4)|0;
     $142 = $141;
     $143 = HEAP32[$142>>2]|0;
     $144 = ($137|0)<($143|0);
     $145 = ($134>>>0)<($140>>>0);
     $146 = ($137|0)==($143|0);
     $147 = $146 & $145;
     $148 = $144 | $147;
     if ($148) {
      _fts3ReadNextPos($p1,$i1);
      break;
     } else {
      _fts3ReadNextPos($p2,$i2);
      break;
     }
    }
   } while(0);
   $149 = $i1;
   $150 = $149;
   $151 = HEAP32[$150>>2]|0;
   $152 = (($149) + 4)|0;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = ($151|0)!=(2147483647);
   $156 = ($154|0)!=(0);
   $157 = $155 | $156;
   $158 = $i2;
   $159 = $158;
   $160 = HEAP32[$159>>2]|0;
   $161 = (($158) + 4)|0;
   $162 = $161;
   $163 = HEAP32[$162>>2]|0;
   $164 = ($160|0)!=(2147483647);
   $165 = ($163|0)!=(0);
   $166 = $164 | $165;
   $167 = $157 ? 1 : $166;
   if (!($167)) {
    continue L1;
   }
  }
 }
 $179 = HEAP32[$p>>2]|0;
 $180 = ((($179)) + 1|0);
 HEAP32[$p>>2] = $180;
 HEAP8[$179>>0] = 0;
 $181 = HEAP32[$p>>2]|0;
 $182 = HEAP32[$0>>2]|0;
 HEAP32[$182>>2] = $181;
 $183 = HEAP32[$p1>>2]|0;
 $184 = ((($183)) + 1|0);
 $185 = HEAP32[$1>>2]|0;
 HEAP32[$185>>2] = $184;
 $186 = HEAP32[$p2>>2]|0;
 $187 = ((($186)) + 1|0);
 $188 = HEAP32[$2>>2]|0;
 HEAP32[$188>>2] = $187;
 STACKTOP = sp;return;
}
function _fts3PutColNumber($pp,$iCol) {
 $pp = $pp|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$n>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  $18 = HEAP32[$n>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 1|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)<(0);
 $10 = $9 << 31 >> 31;
 $11 = (_sqlite3Fts3PutVarint($7,$8,$10)|0);
 $12 = (1 + ($11))|0;
 HEAP32[$n>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 HEAP8[$13>>0] = 1;
 $14 = HEAP32[$n>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 $16 = (($15) + ($14)|0);
 $17 = HEAP32[$0>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _fts3ReadNextPos($pp,$pi) {
 $pp = $pp|0;
 $pi = $pi|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $pi;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = $5 & 254;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  _fts3GetDeltaVarint($8,$9);
  $10 = HEAP32[$1>>2]|0;
  $11 = $10;
  $12 = $11;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($11) + 4)|0;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (_i64Subtract(($13|0),($16|0),2,0)|0);
  $18 = tempRet0;
  $19 = $10;
  $20 = $19;
  HEAP32[$20>>2] = $17;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $18;
  STACKTOP = sp;return;
 } else {
  $23 = HEAP32[$1>>2]|0;
  $24 = $23;
  $25 = $24;
  HEAP32[$25>>2] = 2147483647;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = 0;
  STACKTOP = sp;return;
 }
}
function _fts3CursorSeek($pContext,$pCsr) {
 $pContext = $pContext|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$1>>2] = $pContext;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 7|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$pStmt>>2] = 0;
  $7 = HEAP32[$2>>2]|0;
  $8 = (_fts3CursorSeekStmt($7,$pStmt)|0);
  HEAP32[$rc>>2] = $8;
  $9 = HEAP32[$rc>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$2>>2]|0;
   $12 = ((($11)) + 8|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 32|0);
   $16 = $15;
   $17 = $16;
   $18 = HEAP32[$17>>2]|0;
   $19 = (($16) + 4)|0;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   (_sqlite3_bind_int64($13,1,$18,$21)|0);
   $22 = HEAP32[$2>>2]|0;
   $23 = ((($22)) + 7|0);
   HEAP8[$23>>0] = 0;
   $24 = HEAP32[$2>>2]|0;
   $25 = ((($24)) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (_sqlite3_step($26)|0);
   $28 = (100)==($27|0);
   if ($28) {
    HEAP32[$0>>2] = 0;
    $49 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($49|0);
   }
   $29 = HEAP32[$2>>2]|0;
   $30 = ((($29)) + 8|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (_sqlite3_reset($31)|0);
   HEAP32[$rc>>2] = $32;
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$2>>2]|0;
    $36 = HEAP32[$35>>2]|0;
    $37 = ((($36)) + 40|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ($38|0)==(0|0);
    if ($39) {
     HEAP32[$rc>>2] = 267;
     $40 = HEAP32[$2>>2]|0;
     $41 = ((($40)) + 6|0);
     HEAP8[$41>>0] = 1;
    }
   }
  }
 }
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)!=(0);
 $44 = HEAP32[$1>>2]|0;
 $45 = ($44|0)!=(0|0);
 $or$cond = $43 & $45;
 if ($or$cond) {
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($46,$47);
 }
 $48 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3Fts3CacheDeferredDoclists($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDocid = 0, $iDum1 = 0, $iDum2 = 0, $iPos = 0, $nToken = 0, $or$cond = 0, $p = 0;
 var $pDef = 0, $pModule = 0, $pPT = 0, $pT = 0, $pTC = 0, $rc = 0, $zText = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $rc = sp + 60|0;
 $i = sp + 56|0;
 $iDocid = sp;
 $pDef = sp + 52|0;
 $p = sp + 48|0;
 $pT = sp + 44|0;
 $pModule = sp + 40|0;
 $zText = sp + 36|0;
 $pTC = sp + 32|0;
 $zToken = sp + 28|0;
 $nToken = sp + 24|0;
 $iDum1 = sp + 20|0;
 $iDum2 = sp + 16|0;
 $iPos = sp + 12|0;
 $pPT = sp + 8|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $151 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($151|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pT>>2] = $9;
 $10 = HEAP32[$pT>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pModule>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3_column_int64($14,0)|0);
 $16 = tempRet0;
 $17 = $iDocid;
 $18 = $17;
 HEAP32[$18>>2] = $15;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $16;
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($21|0)<($24|0);
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)==(0);
  $28 = $25 ? $27 : 0;
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$p>>2]|0;
  $31 = ((($30)) + 32|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29)|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$0>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$i>>2]|0;
   $41 = (($40) + 1)|0;
   $42 = (_sqlite3_column_text($39,$41)|0);
   HEAP32[$zText>>2] = $42;
   HEAP32[$pTC>>2] = 0;
   $43 = HEAP32[$pT>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = ((($44)) + 16|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$zText>>2]|0;
   $48 = (_sqlite3Fts3OpenTokenizer($43,$46,$47,-1,$pTC)|0);
   HEAP32[$rc>>2] = $48;
   L9: while(1) {
    $49 = HEAP32[$rc>>2]|0;
    $50 = ($49|0)==(0);
    if (!($50)) {
     break;
    }
    HEAP32[$nToken>>2] = 0;
    HEAP32[$iDum1>>2] = 0;
    HEAP32[$iDum2>>2] = 0;
    HEAP32[$iPos>>2] = 0;
    $51 = HEAP32[$pModule>>2]|0;
    $52 = ((($51)) + 20|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = HEAP32[$pTC>>2]|0;
    $55 = (FUNCTION_TABLE_iiiiiii[$53 & 255]($54,$zToken,$nToken,$iDum1,$iDum2,$iPos)|0);
    HEAP32[$rc>>2] = $55;
    $56 = HEAP32[$0>>2]|0;
    $57 = ((($56)) + 24|0);
    $58 = HEAP32[$57>>2]|0;
    HEAP32[$pDef>>2] = $58;
    while(1) {
     $59 = HEAP32[$pDef>>2]|0;
     $60 = ($59|0)!=(0|0);
     $61 = HEAP32[$rc>>2]|0;
     $62 = ($61|0)==(0);
     $63 = $60 ? $62 : 0;
     if (!($63)) {
      continue L9;
     }
     $64 = HEAP32[$pDef>>2]|0;
     $65 = HEAP32[$64>>2]|0;
     HEAP32[$pPT>>2] = $65;
     $66 = HEAP32[$pDef>>2]|0;
     $67 = ((($66)) + 4|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = HEAP32[$p>>2]|0;
     $70 = ((($69)) + 24|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = ($68|0)>=($71|0);
     if ($72) {
      label = 11;
     } else {
      $73 = HEAP32[$pDef>>2]|0;
      $74 = ((($73)) + 4|0);
      $75 = HEAP32[$74>>2]|0;
      $76 = HEAP32[$i>>2]|0;
      $77 = ($75|0)==($76|0);
      if ($77) {
       label = 11;
      }
     }
     do {
      if ((label|0) == 11) {
       label = 0;
       $78 = HEAP32[$pPT>>2]|0;
       $79 = ((($78)) + 12|0);
       $80 = HEAP32[$79>>2]|0;
       $81 = ($80|0)==(0);
       $82 = HEAP32[$iPos>>2]|0;
       $83 = ($82|0)==(0);
       $or$cond = $81 | $83;
       if ($or$cond) {
        $84 = HEAP32[$pPT>>2]|0;
        $85 = ((($84)) + 4|0);
        $86 = HEAP32[$85>>2]|0;
        $87 = HEAP32[$nToken>>2]|0;
        $88 = ($86|0)==($87|0);
        if (!($88)) {
         $89 = HEAP32[$pPT>>2]|0;
         $90 = ((($89)) + 8|0);
         $91 = HEAP32[$90>>2]|0;
         $92 = ($91|0)!=(0);
         if (!($92)) {
          break;
         }
         $93 = HEAP32[$pPT>>2]|0;
         $94 = ((($93)) + 4|0);
         $95 = HEAP32[$94>>2]|0;
         $96 = HEAP32[$nToken>>2]|0;
         $97 = ($95|0)<($96|0);
         if (!($97)) {
          break;
         }
        }
        $98 = HEAP32[$zToken>>2]|0;
        $99 = HEAP32[$pPT>>2]|0;
        $100 = HEAP32[$99>>2]|0;
        $101 = HEAP32[$pPT>>2]|0;
        $102 = ((($101)) + 4|0);
        $103 = HEAP32[$102>>2]|0;
        $104 = (_memcmp($98,$100,$103)|0);
        $105 = (0)==($104|0);
        if ($105) {
         $106 = HEAP32[$pDef>>2]|0;
         $107 = ((($106)) + 12|0);
         $108 = $iDocid;
         $109 = $108;
         $110 = HEAP32[$109>>2]|0;
         $111 = (($108) + 4)|0;
         $112 = $111;
         $113 = HEAP32[$112>>2]|0;
         $114 = HEAP32[$i>>2]|0;
         $115 = ($114|0)<(0);
         $116 = $115 << 31 >> 31;
         $117 = HEAP32[$iPos>>2]|0;
         $118 = ($117|0)<(0);
         $119 = $118 << 31 >> 31;
         (_fts3PendingListAppend($107,$110,$113,$114,$116,$117,$119,$rc)|0);
        }
       }
      }
     } while(0);
     $120 = HEAP32[$pDef>>2]|0;
     $121 = ((($120)) + 8|0);
     $122 = HEAP32[$121>>2]|0;
     HEAP32[$pDef>>2] = $122;
    }
   }
   $123 = HEAP32[$pTC>>2]|0;
   $124 = ($123|0)!=(0|0);
   if ($124) {
    $125 = HEAP32[$pModule>>2]|0;
    $126 = ((($125)) + 16|0);
    $127 = HEAP32[$126>>2]|0;
    $128 = HEAP32[$pTC>>2]|0;
    (FUNCTION_TABLE_ii[$127 & 255]($128)|0);
   }
   $129 = HEAP32[$rc>>2]|0;
   $130 = ($129|0)==(101);
   if ($130) {
    HEAP32[$rc>>2] = 0;
   }
  }
  $131 = HEAP32[$i>>2]|0;
  $132 = (($131) + 1)|0;
  HEAP32[$i>>2] = $132;
 }
 $133 = HEAP32[$0>>2]|0;
 $134 = ((($133)) + 24|0);
 $135 = HEAP32[$134>>2]|0;
 HEAP32[$pDef>>2] = $135;
 while(1) {
  $136 = HEAP32[$pDef>>2]|0;
  $137 = ($136|0)!=(0|0);
  $138 = HEAP32[$rc>>2]|0;
  $139 = ($138|0)==(0);
  $140 = $137 ? $139 : 0;
  if (!($140)) {
   break;
  }
  $141 = HEAP32[$pDef>>2]|0;
  $142 = ((($141)) + 12|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = ($143|0)!=(0|0);
  if ($144) {
   $145 = HEAP32[$pDef>>2]|0;
   $146 = ((($145)) + 12|0);
   $147 = (_fts3PendingListAppendVarint($146,0,0)|0);
   HEAP32[$rc>>2] = $147;
  }
  $148 = HEAP32[$pDef>>2]|0;
  $149 = ((($148)) + 8|0);
  $150 = HEAP32[$149>>2]|0;
  HEAP32[$pDef>>2] = $150;
 }
 $151 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($151|0);
}
function _fts3EvalTestExpr($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bHit = 0, $bHit1 = 0;
 var $bHit2 = 0, $p = 0, $pPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $bHit = sp + 16|0;
 $p = sp + 12|0;
 $bHit1 = sp + 8|0;
 $bHit2 = sp + 4|0;
 $pPhrase = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 HEAP32[$bHit>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 switch ($7|0) {
 case 3: case 1:  {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_fts3EvalTestExpr($8,$11,$12)|0);
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (_fts3EvalTestExpr($15,$18,$19)|0);
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    $24 = (_fts3EvalNearTest($22,$23)|0);
    $25 = ($24|0)!=(0);
    $27 = $25;
   } else {
    $27 = 0;
   }
  } else {
   $27 = 0;
  }
  $26 = $27&1;
  HEAP32[$bHit>>2] = $26;
  $28 = HEAP32[$bHit>>2]|0;
  $29 = ($28|0)==(0);
  if (!($29)) {
   $217 = HEAP32[$bHit>>2]|0;
   STACKTOP = sp;return ($217|0);
  }
  $30 = HEAP32[$1>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)==(1);
  if (!($32)) {
   $217 = HEAP32[$bHit>>2]|0;
   STACKTOP = sp;return ($217|0);
  }
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 8|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)==(0|0);
  if (!($36)) {
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40|0)!=(1);
   if (!($41)) {
    $217 = HEAP32[$bHit>>2]|0;
    STACKTOP = sp;return ($217|0);
   }
  }
  $42 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $42;
  while(1) {
   $43 = HEAP32[$p>>2]|0;
   $44 = ((($43)) + 20|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)==(0|0);
   $47 = HEAP32[$p>>2]|0;
   if (!($46)) {
    break;
   }
   $48 = ((($47)) + 16|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ((($49)) + 24|0);
   $51 = $50;
   $52 = $51;
   $53 = HEAP32[$52>>2]|0;
   $54 = (($51) + 4)|0;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$0>>2]|0;
   $58 = ((($57)) + 32|0);
   $59 = $58;
   $60 = $59;
   $61 = HEAP32[$60>>2]|0;
   $62 = (($59) + 4)|0;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = ($53|0)==($61|0);
   $66 = ($56|0)==($64|0);
   $67 = $65 & $66;
   if ($67) {
    $68 = HEAP32[$p>>2]|0;
    $69 = ((($68)) + 16|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ((($70)) + 20|0);
    $72 = HEAP32[$71>>2]|0;
    _fts3EvalInvalidatePoslist($72);
   }
   $73 = HEAP32[$p>>2]|0;
   $74 = ((($73)) + 12|0);
   $75 = HEAP32[$74>>2]|0;
   HEAP32[$p>>2] = $75;
  }
  $76 = ((($47)) + 24|0);
  $77 = $76;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = (($77) + 4)|0;
  $81 = $80;
  $82 = HEAP32[$81>>2]|0;
  $83 = HEAP32[$0>>2]|0;
  $84 = ((($83)) + 32|0);
  $85 = $84;
  $86 = $85;
  $87 = HEAP32[$86>>2]|0;
  $88 = (($85) + 4)|0;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = ($79|0)==($87|0);
  $92 = ($82|0)==($90|0);
  $93 = $91 & $92;
  if (!($93)) {
   $217 = HEAP32[$bHit>>2]|0;
   STACKTOP = sp;return ($217|0);
  }
  $94 = HEAP32[$p>>2]|0;
  $95 = ((($94)) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  _fts3EvalInvalidatePoslist($96);
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
  break;
 }
 case 4:  {
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = ((($98)) + 12|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = HEAP32[$2>>2]|0;
  $102 = (_fts3EvalTestExpr($97,$100,$101)|0);
  HEAP32[$bHit1>>2] = $102;
  $103 = HEAP32[$0>>2]|0;
  $104 = HEAP32[$1>>2]|0;
  $105 = ((($104)) + 16|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = HEAP32[$2>>2]|0;
  $108 = (_fts3EvalTestExpr($103,$106,$107)|0);
  HEAP32[$bHit2>>2] = $108;
  $109 = HEAP32[$bHit1>>2]|0;
  $110 = ($109|0)!=(0);
  $111 = HEAP32[$bHit2>>2]|0;
  $112 = ($111|0)!=(0);
  $113 = $110 ? 1 : $112;
  $114 = $113&1;
  HEAP32[$bHit>>2] = $114;
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
  break;
 }
 case 2:  {
  $115 = HEAP32[$0>>2]|0;
  $116 = HEAP32[$1>>2]|0;
  $117 = ((($116)) + 12|0);
  $118 = HEAP32[$117>>2]|0;
  $119 = HEAP32[$2>>2]|0;
  $120 = (_fts3EvalTestExpr($115,$118,$119)|0);
  $121 = ($120|0)!=(0);
  if ($121) {
   $122 = HEAP32[$0>>2]|0;
   $123 = HEAP32[$1>>2]|0;
   $124 = ((($123)) + 16|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = HEAP32[$2>>2]|0;
   $127 = (_fts3EvalTestExpr($122,$125,$126)|0);
   $128 = ($127|0)!=(0);
   $129 = $128 ^ 1;
   $131 = $129;
  } else {
   $131 = 0;
  }
  $130 = $131&1;
  HEAP32[$bHit>>2] = $130;
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
  break;
 }
 default: {
  $132 = HEAP32[$0>>2]|0;
  $133 = ((($132)) + 24|0);
  $134 = HEAP32[$133>>2]|0;
  $135 = ($134|0)!=(0|0);
  do {
   if ($135) {
    $136 = HEAP32[$1>>2]|0;
    $137 = ((($136)) + 24|0);
    $138 = $137;
    $139 = $138;
    $140 = HEAP32[$139>>2]|0;
    $141 = (($138) + 4)|0;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = HEAP32[$0>>2]|0;
    $145 = ((($144)) + 32|0);
    $146 = $145;
    $147 = $146;
    $148 = HEAP32[$147>>2]|0;
    $149 = (($146) + 4)|0;
    $150 = $149;
    $151 = HEAP32[$150>>2]|0;
    $152 = ($140|0)==($148|0);
    $153 = ($143|0)==($151|0);
    $154 = $152 & $153;
    if (!($154)) {
     $155 = HEAP32[$1>>2]|0;
     $156 = ((($155)) + 34|0);
     $157 = HEAP8[$156>>0]|0;
     $158 = $157&255;
     $159 = ($158|0)!=(0);
     if (!($159)) {
      break;
     }
    }
    $160 = HEAP32[$1>>2]|0;
    $161 = ((($160)) + 20|0);
    $162 = HEAP32[$161>>2]|0;
    HEAP32[$pPhrase>>2] = $162;
    $163 = HEAP32[$1>>2]|0;
    $164 = ((($163)) + 34|0);
    $165 = HEAP8[$164>>0]|0;
    $166 = ($165<<24>>24)!=(0);
    if ($166) {
     $167 = HEAP32[$pPhrase>>2]|0;
     _fts3EvalInvalidatePoslist($167);
    }
    $168 = HEAP32[$0>>2]|0;
    $169 = HEAP32[$pPhrase>>2]|0;
    $170 = (_fts3EvalDeferredPhrase($168,$169)|0);
    $171 = HEAP32[$2>>2]|0;
    HEAP32[$171>>2] = $170;
    $172 = HEAP32[$pPhrase>>2]|0;
    $173 = ((($172)) + 28|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = ($174|0)!=(0|0);
    $176 = $175&1;
    HEAP32[$bHit>>2] = $176;
    $177 = HEAP32[$0>>2]|0;
    $178 = ((($177)) + 32|0);
    $179 = $178;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = (($179) + 4)|0;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = HEAP32[$1>>2]|0;
    $186 = ((($185)) + 24|0);
    $187 = $186;
    $188 = $187;
    HEAP32[$188>>2] = $181;
    $189 = (($187) + 4)|0;
    $190 = $189;
    HEAP32[$190>>2] = $184;
    $217 = HEAP32[$bHit>>2]|0;
    STACKTOP = sp;return ($217|0);
   }
  } while(0);
  $191 = HEAP32[$1>>2]|0;
  $192 = ((($191)) + 32|0);
  $193 = HEAP8[$192>>0]|0;
  $194 = $193&255;
  $195 = ($194|0)==(0);
  if ($195) {
   $196 = HEAP32[$1>>2]|0;
   $197 = ((($196)) + 24|0);
   $198 = $197;
   $199 = $198;
   $200 = HEAP32[$199>>2]|0;
   $201 = (($198) + 4)|0;
   $202 = $201;
   $203 = HEAP32[$202>>2]|0;
   $204 = HEAP32[$0>>2]|0;
   $205 = ((($204)) + 32|0);
   $206 = $205;
   $207 = $206;
   $208 = HEAP32[$207>>2]|0;
   $209 = (($206) + 4)|0;
   $210 = $209;
   $211 = HEAP32[$210>>2]|0;
   $212 = ($200|0)==($208|0);
   $213 = ($203|0)==($211|0);
   $214 = $212 & $213;
   $216 = $214;
  } else {
   $216 = 0;
  }
  $215 = $216&1;
  HEAP32[$bHit>>2] = $215;
  $217 = HEAP32[$bHit>>2]|0;
  STACKTOP = sp;return ($217|0);
 }
 }
 return (0)|0;
}
function _sqlite3Fts3FreeDeferredDoclists($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pDef = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pDef>>2] = $3;
 while(1) {
  $4 = HEAP32[$pDef>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$pDef>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  _fts3PendingListDelete($8);
  $9 = HEAP32[$pDef>>2]|0;
  $10 = ((($9)) + 12|0);
  HEAP32[$10>>2] = 0;
  $11 = HEAP32[$pDef>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pDef>>2] = $13;
 }
 STACKTOP = sp;return;
}
function _fts3EvalNearTest($pExpr,$pRc) {
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aPoslist = 0, $aTmp = 0, $nNear = 0, $nNear1 = 0, $nTmp = 0;
 var $nToken = 0, $or$cond = 0, $p = 0, $pPhrase = 0, $pPhrase2 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $res = sp + 36|0;
 $p = sp + 32|0;
 $nTmp = sp + 28|0;
 $aTmp = sp + 24|0;
 $aPoslist = sp + 20|0;
 $nToken = sp + 16|0;
 $pPhrase = sp + 12|0;
 $nNear = sp + 8|0;
 $nNear1 = sp + 4|0;
 $pPhrase2 = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $pRc;
 HEAP32[$res>>2] = 1;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(1);
 if (!($7)) {
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0|0);
 if (!($16)) {
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)!=(1);
  if (!($21)) {
   $134 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($134|0);
  }
 }
 HEAP32[$nTmp>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $22;
 while(1) {
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 12|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)!=(0|0);
  $27 = HEAP32[$p>>2]|0;
  if (!($26)) {
   break;
  }
  $28 = ((($27)) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 20|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ((($31)) + 32|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$nTmp>>2]|0;
  $35 = (($34) + ($33))|0;
  HEAP32[$nTmp>>2] = $35;
  $36 = HEAP32[$p>>2]|0;
  $37 = ((($36)) + 12|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$p>>2] = $38;
 }
 $39 = ((($27)) + 20|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = ((($40)) + 32|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$nTmp>>2]|0;
 $44 = (($43) + ($42))|0;
 HEAP32[$nTmp>>2] = $44;
 $45 = HEAP32[$nTmp>>2]|0;
 $46 = ($45|0)==(0);
 if ($46) {
  HEAP32[$res>>2] = 0;
  $134 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($134|0);
 }
 $47 = HEAP32[$nTmp>>2]|0;
 $48 = $47<<1;
 $49 = (_sqlite3_malloc($48)|0);
 HEAP32[$aTmp>>2] = $49;
 $50 = HEAP32[$aTmp>>2]|0;
 $51 = ($50|0)!=(0|0);
 L22: do {
  if ($51) {
   $53 = HEAP32[$p>>2]|0;
   $54 = ((($53)) + 20|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ((($55)) + 28|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$aPoslist>>2] = $57;
   $58 = HEAP32[$p>>2]|0;
   $59 = ((($58)) + 20|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ((($60)) + 64|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$nToken>>2] = $62;
   $63 = HEAP32[$p>>2]|0;
   $64 = ((($63)) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$p>>2] = $65;
   while(1) {
    $66 = HEAP32[$res>>2]|0;
    $67 = ($66|0)!=(0);
    $68 = HEAP32[$p>>2]|0;
    $69 = ($68|0)!=(0|0);
    $or$cond = $67 & $69;
    if (!($or$cond)) {
     break;
    }
    $70 = HEAP32[$p>>2]|0;
    $71 = HEAP32[$70>>2]|0;
    $72 = ($71|0)==(1);
    if (!($72)) {
     break;
    }
    $73 = HEAP32[$p>>2]|0;
    $74 = ((($73)) + 16|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = ((($75)) + 20|0);
    $77 = HEAP32[$76>>2]|0;
    HEAP32[$pPhrase>>2] = $77;
    $78 = HEAP32[$p>>2]|0;
    $79 = ((($78)) + 4|0);
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$nNear>>2] = $80;
    $81 = HEAP32[$nNear>>2]|0;
    $82 = HEAP32[$aTmp>>2]|0;
    $83 = HEAP32[$pPhrase>>2]|0;
    $84 = (_fts3EvalNearTrim($81,$82,$aPoslist,$nToken,$83)|0);
    HEAP32[$res>>2] = $84;
    $85 = HEAP32[$p>>2]|0;
    $86 = ((($85)) + 8|0);
    $87 = HEAP32[$86>>2]|0;
    HEAP32[$p>>2] = $87;
   }
   $88 = HEAP32[$0>>2]|0;
   $89 = ((($88)) + 16|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = ((($90)) + 20|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = ((($92)) + 28|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$aPoslist>>2] = $94;
   $95 = HEAP32[$0>>2]|0;
   $96 = ((($95)) + 16|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = ((($97)) + 20|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = ((($99)) + 64|0);
   $101 = HEAP32[$100>>2]|0;
   HEAP32[$nToken>>2] = $101;
   $102 = HEAP32[$0>>2]|0;
   $103 = ((($102)) + 12|0);
   $104 = HEAP32[$103>>2]|0;
   HEAP32[$p>>2] = $104;
   while(1) {
    $105 = HEAP32[$p>>2]|0;
    $106 = ($105|0)!=(0|0);
    $107 = HEAP32[$res>>2]|0;
    $108 = ($107|0)!=(0);
    $109 = $106 ? $108 : 0;
    if (!($109)) {
     break L22;
    }
    $110 = HEAP32[$p>>2]|0;
    $111 = ((($110)) + 8|0);
    $112 = HEAP32[$111>>2]|0;
    $113 = ((($112)) + 4|0);
    $114 = HEAP32[$113>>2]|0;
    HEAP32[$nNear1>>2] = $114;
    $115 = HEAP32[$p>>2]|0;
    $116 = HEAP32[$115>>2]|0;
    $117 = ($116|0)==(1);
    $118 = HEAP32[$p>>2]|0;
    if ($117) {
     $119 = ((($118)) + 16|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = ((($120)) + 20|0);
     $122 = HEAP32[$121>>2]|0;
     $125 = $122;
    } else {
     $123 = ((($118)) + 20|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = $124;
    }
    HEAP32[$pPhrase2>>2] = $125;
    $126 = HEAP32[$nNear1>>2]|0;
    $127 = HEAP32[$aTmp>>2]|0;
    $128 = HEAP32[$pPhrase2>>2]|0;
    $129 = (_fts3EvalNearTrim($126,$127,$aPoslist,$nToken,$128)|0);
    HEAP32[$res>>2] = $129;
    $130 = HEAP32[$p>>2]|0;
    $131 = ((($130)) + 12|0);
    $132 = HEAP32[$131>>2]|0;
    HEAP32[$p>>2] = $132;
   }
  } else {
   $52 = HEAP32[$1>>2]|0;
   HEAP32[$52>>2] = 7;
   HEAP32[$res>>2] = 0;
  }
 } while(0);
 $133 = HEAP32[$aTmp>>2]|0;
 _sqlite3_free($133);
 $134 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($134|0);
}
function _fts3EvalDeferredPhrase($pCsr,$pPhrase) {
 $pCsr = $pCsr|0;
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aOut = 0, $aOut3 = 0, $aPoslist = 0, $iPrev = 0, $iToken = 0, $nDistance = 0, $nList = 0, $nMaxUndeferred = 0, $nPoslist = 0, $p1 = 0;
 var $p11 = 0, $p2 = 0, $p22 = 0, $pDeferred = 0, $pList = 0, $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $iToken = sp + 64|0;
 $aPoslist = sp + 60|0;
 $nPoslist = sp + 56|0;
 $iPrev = sp + 52|0;
 $pToken = sp + 48|0;
 $pDeferred = sp + 44|0;
 $pList = sp + 40|0;
 $nList = sp + 36|0;
 $rc = sp + 32|0;
 $aOut = sp + 28|0;
 $p1 = sp + 24|0;
 $p2 = sp + 20|0;
 $nMaxUndeferred = sp + 16|0;
 $nDistance = sp + 12|0;
 $p11 = sp + 8|0;
 $p22 = sp + 4|0;
 $aOut3 = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pPhrase;
 HEAP32[$aPoslist>>2] = 0;
 HEAP32[$nPoslist>>2] = 0;
 HEAP32[$iPrev>>2] = -1;
 HEAP32[$iToken>>2] = 0;
 while(1) {
  $3 = HEAP32[$iToken>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = ((($4)) + 64|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 14;
   break;
  }
  $8 = HEAP32[$iToken>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 72|0);
  $11 = (($10) + (($8*24)|0)|0);
  HEAP32[$pToken>>2] = $11;
  $12 = HEAP32[$pToken>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pDeferred>>2] = $14;
  $15 = HEAP32[$pDeferred>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   $17 = HEAP32[$pDeferred>>2]|0;
   $18 = (_sqlite3Fts3DeferredTokenList($17,$pList,$nList)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)!=(0);
   if ($20) {
    label = 5;
    break;
   }
   $22 = HEAP32[$pList>>2]|0;
   $23 = ($22|0)==(0|0);
   $24 = HEAP32[$aPoslist>>2]|0;
   if ($23) {
    label = 7;
    break;
   }
   $29 = ($24|0)==(0|0);
   $30 = HEAP32[$pList>>2]|0;
   if ($29) {
    HEAP32[$aPoslist>>2] = $30;
    $31 = HEAP32[$nList>>2]|0;
    HEAP32[$nPoslist>>2] = $31;
   } else {
    HEAP32[$aOut>>2] = $30;
    $32 = HEAP32[$aPoslist>>2]|0;
    HEAP32[$p1>>2] = $32;
    $33 = HEAP32[$aOut>>2]|0;
    HEAP32[$p2>>2] = $33;
    $34 = HEAP32[$iToken>>2]|0;
    $35 = HEAP32[$iPrev>>2]|0;
    $36 = (($34) - ($35))|0;
    (_fts3PoslistPhraseMerge($aOut,$36,0,1,$p1,$p2)|0);
    $37 = HEAP32[$aPoslist>>2]|0;
    _sqlite3_free($37);
    $38 = HEAP32[$pList>>2]|0;
    HEAP32[$aPoslist>>2] = $38;
    $39 = HEAP32[$aOut>>2]|0;
    $40 = HEAP32[$aPoslist>>2]|0;
    $41 = $39;
    $42 = $40;
    $43 = (($41) - ($42))|0;
    HEAP32[$nPoslist>>2] = $43;
    $44 = HEAP32[$nPoslist>>2]|0;
    $45 = ($44|0)==(0);
    if ($45) {
     label = 11;
     break;
    }
   }
   $51 = HEAP32[$iToken>>2]|0;
   HEAP32[$iPrev>>2] = $51;
  }
  $52 = HEAP32[$iToken>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$iToken>>2] = $53;
 }
 if ((label|0) == 5) {
  $21 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $21;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 else if ((label|0) == 7) {
  _sqlite3_free($24);
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 28|0);
  HEAP32[$26>>2] = 0;
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 32|0);
  HEAP32[$28>>2] = 0;
  HEAP32[$0>>2] = 0;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 else if ((label|0) == 11) {
  $46 = HEAP32[$aPoslist>>2]|0;
  _sqlite3_free($46);
  $47 = HEAP32[$2>>2]|0;
  $48 = ((($47)) + 28|0);
  HEAP32[$48>>2] = 0;
  $49 = HEAP32[$2>>2]|0;
  $50 = ((($49)) + 32|0);
  HEAP32[$50>>2] = 0;
  HEAP32[$0>>2] = 0;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 else if ((label|0) == 14) {
  $54 = HEAP32[$iPrev>>2]|0;
  $55 = ($54|0)>=(0);
  do {
   if ($55) {
    $56 = HEAP32[$2>>2]|0;
    $57 = ((($56)) + 44|0);
    $58 = HEAP32[$57>>2]|0;
    HEAP32[$nMaxUndeferred>>2] = $58;
    $59 = HEAP32[$nMaxUndeferred>>2]|0;
    $60 = ($59|0)<(0);
    if ($60) {
     $61 = HEAP32[$aPoslist>>2]|0;
     $62 = HEAP32[$2>>2]|0;
     $63 = ((($62)) + 28|0);
     HEAP32[$63>>2] = $61;
     $64 = HEAP32[$nPoslist>>2]|0;
     $65 = HEAP32[$2>>2]|0;
     $66 = ((($65)) + 32|0);
     HEAP32[$66>>2] = $64;
     $67 = HEAP32[$1>>2]|0;
     $68 = ((($67)) + 32|0);
     $69 = $68;
     $70 = $69;
     $71 = HEAP32[$70>>2]|0;
     $72 = (($69) + 4)|0;
     $73 = $72;
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$2>>2]|0;
     $76 = ((($75)) + 16|0);
     $77 = $76;
     $78 = $77;
     HEAP32[$78>>2] = $71;
     $79 = (($77) + 4)|0;
     $80 = $79;
     HEAP32[$80>>2] = $74;
     $81 = HEAP32[$2>>2]|0;
     $82 = ((($81)) + 24|0);
     HEAP32[$82>>2] = 1;
     break;
    }
    $83 = HEAP32[$nMaxUndeferred>>2]|0;
    $84 = HEAP32[$iPrev>>2]|0;
    $85 = ($83|0)>($84|0);
    if ($85) {
     $86 = HEAP32[$aPoslist>>2]|0;
     HEAP32[$p11>>2] = $86;
     $87 = HEAP32[$2>>2]|0;
     $88 = ((($87)) + 28|0);
     $89 = HEAP32[$88>>2]|0;
     HEAP32[$p22>>2] = $89;
     $90 = HEAP32[$nMaxUndeferred>>2]|0;
     $91 = HEAP32[$iPrev>>2]|0;
     $92 = (($90) - ($91))|0;
     HEAP32[$nDistance>>2] = $92;
    } else {
     $93 = HEAP32[$2>>2]|0;
     $94 = ((($93)) + 28|0);
     $95 = HEAP32[$94>>2]|0;
     HEAP32[$p11>>2] = $95;
     $96 = HEAP32[$aPoslist>>2]|0;
     HEAP32[$p22>>2] = $96;
     $97 = HEAP32[$iPrev>>2]|0;
     $98 = HEAP32[$nMaxUndeferred>>2]|0;
     $99 = (($97) - ($98))|0;
     HEAP32[$nDistance>>2] = $99;
    }
    $100 = HEAP32[$nPoslist>>2]|0;
    $101 = (($100) + 8)|0;
    $102 = (_sqlite3_malloc($101)|0);
    HEAP32[$aOut3>>2] = $102;
    $103 = HEAP32[$aOut3>>2]|0;
    $104 = ($103|0)!=(0|0);
    if (!($104)) {
     $105 = HEAP32[$aPoslist>>2]|0;
     _sqlite3_free($105);
     HEAP32[$0>>2] = 7;
     $129 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($129|0);
    }
    $106 = HEAP32[$aOut3>>2]|0;
    $107 = HEAP32[$2>>2]|0;
    $108 = ((($107)) + 28|0);
    HEAP32[$108>>2] = $106;
    $109 = HEAP32[$nDistance>>2]|0;
    $110 = (_fts3PoslistPhraseMerge($aOut3,$109,0,1,$p11,$p22)|0);
    $111 = ($110|0)!=(0);
    if ($111) {
     $112 = HEAP32[$2>>2]|0;
     $113 = ((($112)) + 24|0);
     HEAP32[$113>>2] = 1;
     $114 = HEAP32[$aOut3>>2]|0;
     $115 = HEAP32[$2>>2]|0;
     $116 = ((($115)) + 28|0);
     $117 = HEAP32[$116>>2]|0;
     $118 = $114;
     $119 = $117;
     $120 = (($118) - ($119))|0;
     $121 = HEAP32[$2>>2]|0;
     $122 = ((($121)) + 32|0);
     HEAP32[$122>>2] = $120;
    } else {
     $123 = HEAP32[$aOut3>>2]|0;
     _sqlite3_free($123);
     $124 = HEAP32[$2>>2]|0;
     $125 = ((($124)) + 28|0);
     HEAP32[$125>>2] = 0;
     $126 = HEAP32[$2>>2]|0;
     $127 = ((($126)) + 32|0);
     HEAP32[$127>>2] = 0;
    }
    $128 = HEAP32[$aPoslist>>2]|0;
    _sqlite3_free($128);
   }
  } while(0);
  HEAP32[$0>>2] = 0;
  $129 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($129|0);
 }
 return (0)|0;
}
function _sqlite3Fts3DeferredTokenList($p,$ppData,$pnData) {
 $p = $p|0;
 $ppData = $ppData|0;
 $pnData = $pnData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $dummy = 0, $nSkip = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pRet = sp + 12|0;
 $nSkip = sp + 8|0;
 $dummy = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $ppData;
 HEAP32[$3>>2] = $pnData;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_malloc($13)|0);
 HEAP32[$pRet>>2] = $14;
 $15 = HEAP32[$pRet>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ((($19)) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3Fts3GetVarint($21,$dummy)|0);
  HEAP32[$nSkip>>2] = $22;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 12|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$nSkip>>2]|0;
  $28 = (($26) - ($27))|0;
  $29 = HEAP32[$3>>2]|0;
  HEAP32[$29>>2] = $28;
  $30 = HEAP32[$pRet>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  HEAP32[$31>>2] = $30;
  $32 = HEAP32[$pRet>>2]|0;
  $33 = HEAP32[$nSkip>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + ($33)|0);
  $40 = HEAP32[$3>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  _memcpy(($32|0),($39|0),($41|0))|0;
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 } else {
  HEAP32[$0>>2] = 7;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _fts3EvalNearTrim($nNear,$aTmp,$paPoslist,$pnToken,$pPhrase) {
 $nNear = $nNear|0;
 $aTmp = $aTmp|0;
 $paPoslist = $paPoslist|0;
 $pnToken = $pnToken|0;
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nNew = 0, $nParam1 = 0, $nParam2 = 0, $p2 = 0, $pOut = 0, $res = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $nParam1 = sp + 20|0;
 $nParam2 = sp + 16|0;
 $nNew = sp + 12|0;
 $p2 = sp + 8|0;
 $pOut = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $nNear;
 HEAP32[$1>>2] = $aTmp;
 HEAP32[$2>>2] = $paPoslist;
 HEAP32[$3>>2] = $pnToken;
 HEAP32[$4>>2] = $pPhrase;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$4>>2]|0;
 $7 = ((($6)) + 64|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($5) + ($8))|0;
 HEAP32[$nParam1>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + ($12))|0;
 HEAP32[$nParam2>>2] = $13;
 $14 = HEAP32[$4>>2]|0;
 $15 = ((($14)) + 28|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pOut>>2] = $16;
 HEAP32[$p2>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$nParam1>>2]|0;
 $19 = HEAP32[$nParam2>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = (_fts3PoslistNearMerge($pOut,$17,$18,$19,$20,$p2)|0);
 HEAP32[$res>>2] = $21;
 $22 = HEAP32[$res>>2]|0;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  $53 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $24 = HEAP32[$pOut>>2]|0;
 $25 = HEAP32[$4>>2]|0;
 $26 = ((($25)) + 28|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $24;
 $29 = $27;
 $30 = (($28) - ($29))|0;
 $31 = (($30) - 1)|0;
 HEAP32[$nNew>>2] = $31;
 $32 = HEAP32[$nNew>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 $34 = ((($33)) + 28|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + ($32)|0);
 $37 = HEAP32[$4>>2]|0;
 $38 = ((($37)) + 32|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$nNew>>2]|0;
 $41 = (($39) - ($40))|0;
 _memset(($36|0),0,($41|0))|0;
 $42 = HEAP32[$nNew>>2]|0;
 $43 = HEAP32[$4>>2]|0;
 $44 = ((($43)) + 32|0);
 HEAP32[$44>>2] = $42;
 $45 = HEAP32[$4>>2]|0;
 $46 = ((($45)) + 28|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$2>>2]|0;
 HEAP32[$48>>2] = $47;
 $49 = HEAP32[$4>>2]|0;
 $50 = ((($49)) + 64|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 HEAP32[$52>>2] = $51;
 $53 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _fts3PoslistNearMerge($pp,$aTmp,$nRight,$nLeft,$pp1,$pp2) {
 $pp = $pp|0;
 $aTmp = $aTmp|0;
 $nRight = $nRight|0;
 $nLeft = $nLeft|0;
 $pp1 = $pp1|0;
 $pp2 = $pp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aTmp2 = 0, $p1 = 0;
 var $p2 = 0, $pTmp1 = 0, $pTmp2 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $p1 = sp + 20|0;
 $p2 = sp + 16|0;
 $pTmp1 = sp + 12|0;
 $pTmp2 = sp + 8|0;
 $aTmp2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $aTmp;
 HEAP32[$2>>2] = $nRight;
 HEAP32[$3>>2] = $nLeft;
 HEAP32[$4>>2] = $pp1;
 HEAP32[$5>>2] = $pp2;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p1>>2] = $7;
 $8 = HEAP32[$5>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p2>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$pTmp1>>2] = $10;
 HEAP32[$res>>2] = 1;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = HEAP32[$5>>2]|0;
 (_fts3PoslistPhraseMerge($pTmp1,$11,0,0,$12,$13)|0);
 $14 = HEAP32[$pTmp1>>2]|0;
 HEAP32[$pTmp2>>2] = $14;
 HEAP32[$aTmp2>>2] = $14;
 $15 = HEAP32[$p1>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$p2>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$3>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$4>>2]|0;
 (_fts3PoslistPhraseMerge($pTmp2,$19,1,0,$20,$21)|0);
 $22 = HEAP32[$pTmp1>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ($22|0)!=($23|0);
 if ($24) {
  $25 = HEAP32[$pTmp2>>2]|0;
  $26 = HEAP32[$aTmp2>>2]|0;
  $27 = ($25|0)!=($26|0);
  if ($27) {
   $28 = HEAP32[$0>>2]|0;
   _fts3PoslistMerge($28,$1,$aTmp2);
   $37 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $29 = HEAP32[$pTmp1>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($29|0)!=($30|0);
 if ($31) {
  $32 = HEAP32[$0>>2]|0;
  _fts3PoslistCopy($32,$1);
  $37 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $33 = HEAP32[$pTmp2>>2]|0;
 $34 = HEAP32[$aTmp2>>2]|0;
 $35 = ($33|0)!=($34|0);
 if ($35) {
  $36 = HEAP32[$0>>2]|0;
  _fts3PoslistCopy($36,$aTmp2);
  $37 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($37|0);
 } else {
  HEAP32[$res>>2] = 0;
  $37 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return (0)|0;
}
function _sqlite3Fts3OpenTokenizer($pTokenizer,$iLangid,$z,$n,$ppCsr) {
 $pTokenizer = $pTokenizer|0;
 $iLangid = $iLangid|0;
 $z = $z|0;
 $n = $n|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, $pModule = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pModule = sp + 8|0;
 $pCsr = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pTokenizer;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $ppCsr;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pModule>>2] = $6;
 HEAP32[$pCsr>>2] = 0;
 $7 = HEAP32[$pModule>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (FUNCTION_TABLE_iiiii[$9 & 255]($10,$11,$12,$pCsr)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(0);
 if ($15) {
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pCsr>>2]|0;
  HEAP32[$17>>2] = $16;
  $18 = HEAP32[$pModule>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)>=(1);
  if ($20) {
   $21 = HEAP32[$pModule>>2]|0;
   $22 = ((($21)) + 24|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$pCsr>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = (FUNCTION_TABLE_iii[$23 & 255]($24,$25)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$rc>>2]|0;
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$pModule>>2]|0;
    $30 = ((($29)) + 16|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$pCsr>>2]|0;
    (FUNCTION_TABLE_ii[$31 & 255]($32)|0);
    HEAP32[$pCsr>>2] = 0;
   }
  }
 }
 $33 = HEAP32[$pCsr>>2]|0;
 $34 = HEAP32[$4>>2]|0;
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _fts3PendingListAppend($pp,$0,$1,$2,$3,$4,$5,$pRc) {
 $pp = $pp|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 $4 = $4|0;
 $5 = $5|0;
 $pRc = $pRc|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iDelta = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $6 = sp + 48|0;
 $7 = sp + 44|0;
 $8 = sp + 24|0;
 $9 = sp + 16|0;
 $10 = sp + 8|0;
 $11 = sp + 40|0;
 $p = sp + 36|0;
 $rc = sp + 32|0;
 $iDelta = sp;
 HEAP32[$7>>2] = $pp;
 $12 = $8;
 $13 = $12;
 HEAP32[$13>>2] = $0;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $1;
 $16 = $9;
 $17 = $16;
 HEAP32[$17>>2] = $2;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $3;
 $20 = $10;
 $21 = $20;
 HEAP32[$21>>2] = $4;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = $5;
 HEAP32[$11>>2] = $pRc;
 $24 = HEAP32[$7>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$p>>2] = $25;
 HEAP32[$rc>>2] = 0;
 $26 = HEAP32[$p>>2]|0;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = HEAP32[$p>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = $29;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = $8;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = ($32|0)!=($38|0);
  $43 = ($35|0)!=($41|0);
  $44 = $42 | $43;
  if ($44) {
   label = 3;
  } else {
   label = 9;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $45 = $8;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($45) + 4)|0;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$p>>2]|0;
  $52 = ($51|0)!=(0|0);
  if ($52) {
   $53 = HEAP32[$p>>2]|0;
   $54 = ((($53)) + 16|0);
   $55 = $54;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = (($55) + 4)|0;
   $59 = $58;
   $60 = HEAP32[$59>>2]|0;
   $61 = $57;$62 = $60;
  } else {
   $61 = 0;$62 = 0;
  }
  $63 = (_i64Subtract(($47|0),($50|0),($61|0),($62|0))|0);
  $64 = tempRet0;
  $65 = $iDelta;
  $66 = $65;
  HEAP32[$66>>2] = $63;
  $67 = (($65) + 4)|0;
  $68 = $67;
  HEAP32[$68>>2] = $64;
  $69 = HEAP32[$p>>2]|0;
  $70 = ($69|0)!=(0|0);
  if ($70) {
   $71 = HEAP32[$p>>2]|0;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($72) + 1)|0;
   HEAP32[$71>>2] = $73;
  }
  $74 = $iDelta;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (($74) + 4)|0;
  $78 = $77;
  $79 = HEAP32[$78>>2]|0;
  $80 = (_fts3PendingListAppendVarint($p,$76,$79)|0);
  HEAP32[$rc>>2] = $80;
  $81 = (0)!=($80|0);
  if (!($81)) {
   $82 = HEAP32[$p>>2]|0;
   $83 = ((($82)) + 24|0);
   $84 = $83;
   $85 = $84;
   HEAP32[$85>>2] = -1;
   $86 = (($84) + 4)|0;
   $87 = $86;
   HEAP32[$87>>2] = -1;
   $88 = HEAP32[$p>>2]|0;
   $89 = ((($88)) + 32|0);
   $90 = $89;
   $91 = $90;
   HEAP32[$91>>2] = 0;
   $92 = (($90) + 4)|0;
   $93 = $92;
   HEAP32[$93>>2] = 0;
   $94 = $8;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($94) + 4)|0;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = HEAP32[$p>>2]|0;
   $101 = ((($100)) + 16|0);
   $102 = $101;
   $103 = $102;
   HEAP32[$103>>2] = $96;
   $104 = (($102) + 4)|0;
   $105 = $104;
   HEAP32[$105>>2] = $99;
   label = 9;
  }
 }
 do {
  if ((label|0) == 9) {
   $106 = $9;
   $107 = $106;
   $108 = HEAP32[$107>>2]|0;
   $109 = (($106) + 4)|0;
   $110 = $109;
   $111 = HEAP32[$110>>2]|0;
   $112 = ($111|0)>(0);
   $113 = ($108>>>0)>(0);
   $114 = ($111|0)==(0);
   $115 = $114 & $113;
   $116 = $112 | $115;
   if ($116) {
    $117 = HEAP32[$p>>2]|0;
    $118 = ((($117)) + 24|0);
    $119 = $118;
    $120 = $119;
    $121 = HEAP32[$120>>2]|0;
    $122 = (($119) + 4)|0;
    $123 = $122;
    $124 = HEAP32[$123>>2]|0;
    $125 = $9;
    $126 = $125;
    $127 = HEAP32[$126>>2]|0;
    $128 = (($125) + 4)|0;
    $129 = $128;
    $130 = HEAP32[$129>>2]|0;
    $131 = ($121|0)!=($127|0);
    $132 = ($124|0)!=($130|0);
    $133 = $131 | $132;
    if ($133) {
     $134 = (_fts3PendingListAppendVarint($p,1,0)|0);
     HEAP32[$rc>>2] = $134;
     $135 = (0)!=($134|0);
     if ($135) {
      break;
     }
     $136 = $9;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = (_fts3PendingListAppendVarint($p,$138,$141)|0);
     HEAP32[$rc>>2] = $142;
     $143 = (0)!=($142|0);
     if ($143) {
      break;
     }
     $144 = $9;
     $145 = $144;
     $146 = HEAP32[$145>>2]|0;
     $147 = (($144) + 4)|0;
     $148 = $147;
     $149 = HEAP32[$148>>2]|0;
     $150 = HEAP32[$p>>2]|0;
     $151 = ((($150)) + 24|0);
     $152 = $151;
     $153 = $152;
     HEAP32[$153>>2] = $146;
     $154 = (($152) + 4)|0;
     $155 = $154;
     HEAP32[$155>>2] = $149;
     $156 = HEAP32[$p>>2]|0;
     $157 = ((($156)) + 32|0);
     $158 = $157;
     $159 = $158;
     HEAP32[$159>>2] = 0;
     $160 = (($158) + 4)|0;
     $161 = $160;
     HEAP32[$161>>2] = 0;
    }
   }
   $162 = $9;
   $163 = $162;
   $164 = HEAP32[$163>>2]|0;
   $165 = (($162) + 4)|0;
   $166 = $165;
   $167 = HEAP32[$166>>2]|0;
   $168 = ($167|0)>(0);
   $169 = ($164>>>0)>=(0);
   $170 = ($167|0)==(0);
   $171 = $170 & $169;
   $172 = $168 | $171;
   if ($172) {
    $173 = $10;
    $174 = $173;
    $175 = HEAP32[$174>>2]|0;
    $176 = (($173) + 4)|0;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (_i64Add(2,0,($175|0),($178|0))|0);
    $180 = tempRet0;
    $181 = HEAP32[$p>>2]|0;
    $182 = ((($181)) + 32|0);
    $183 = $182;
    $184 = $183;
    $185 = HEAP32[$184>>2]|0;
    $186 = (($183) + 4)|0;
    $187 = $186;
    $188 = HEAP32[$187>>2]|0;
    $189 = (_i64Subtract(($179|0),($180|0),($185|0),($188|0))|0);
    $190 = tempRet0;
    $191 = (_fts3PendingListAppendVarint($p,$189,$190)|0);
    HEAP32[$rc>>2] = $191;
    $192 = HEAP32[$rc>>2]|0;
    $193 = ($192|0)==(0);
    if ($193) {
     $194 = $10;
     $195 = $194;
     $196 = HEAP32[$195>>2]|0;
     $197 = (($194) + 4)|0;
     $198 = $197;
     $199 = HEAP32[$198>>2]|0;
     $200 = HEAP32[$p>>2]|0;
     $201 = ((($200)) + 32|0);
     $202 = $201;
     $203 = $202;
     HEAP32[$203>>2] = $196;
     $204 = (($202) + 4)|0;
     $205 = $204;
     HEAP32[$205>>2] = $199;
    }
   }
  }
 } while(0);
 $206 = HEAP32[$rc>>2]|0;
 $207 = HEAP32[$11>>2]|0;
 HEAP32[$207>>2] = $206;
 $208 = HEAP32[$p>>2]|0;
 $209 = HEAP32[$7>>2]|0;
 $210 = HEAP32[$209>>2]|0;
 $211 = ($208|0)!=($210|0);
 if ($211) {
  $212 = HEAP32[$p>>2]|0;
  $213 = HEAP32[$7>>2]|0;
  HEAP32[$213>>2] = $212;
  HEAP32[$6>>2] = 1;
  $214 = HEAP32[$6>>2]|0;
  STACKTOP = sp;return ($214|0);
 } else {
  HEAP32[$6>>2] = 0;
  $214 = HEAP32[$6>>2]|0;
  STACKTOP = sp;return ($214|0);
 }
 return (0)|0;
}
function _fts3PendingListAppendVarint($pp,$0,$1) {
 $pp = $pp|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $nNew = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $p = sp + 12|0;
 $nNew = sp + 8|0;
 HEAP32[$3>>2] = $pp;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$p>>2]|0;
 $12 = ($11|0)!=(0|0);
 do {
  if ($12) {
   $23 = HEAP32[$p>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + 10)|0;
   $26 = (($25) + 1)|0;
   $27 = HEAP32[$p>>2]|0;
   $28 = ((($27)) + 8|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($26|0)>($29|0);
   if ($30) {
    $31 = HEAP32[$p>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = $33<<1;
    HEAP32[$nNew>>2] = $34;
    $35 = HEAP32[$p>>2]|0;
    $36 = HEAP32[$nNew>>2]|0;
    $37 = (40 + ($36))|0;
    $38 = (_sqlite3_realloc($35,$37)|0);
    HEAP32[$p>>2] = $38;
    $39 = HEAP32[$p>>2]|0;
    $40 = ($39|0)!=(0|0);
    if ($40) {
     $44 = HEAP32[$nNew>>2]|0;
     $45 = HEAP32[$p>>2]|0;
     $46 = ((($45)) + 8|0);
     HEAP32[$46>>2] = $44;
     $47 = HEAP32[$p>>2]|0;
     $48 = ((($47)) + 40|0);
     $49 = HEAP32[$p>>2]|0;
     $50 = ((($49)) + 4|0);
     HEAP32[$50>>2] = $48;
     break;
    }
    $41 = HEAP32[$3>>2]|0;
    $42 = HEAP32[$41>>2]|0;
    _sqlite3_free($42);
    $43 = HEAP32[$3>>2]|0;
    HEAP32[$43>>2] = 0;
    HEAP32[$2>>2] = 7;
    $75 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($75|0);
   }
  } else {
   $13 = (_sqlite3_malloc(140)|0);
   HEAP32[$p>>2] = $13;
   $14 = HEAP32[$p>>2]|0;
   $15 = ($14|0)!=(0|0);
   if ($15) {
    $16 = HEAP32[$p>>2]|0;
    $17 = ((($16)) + 8|0);
    HEAP32[$17>>2] = 100;
    $18 = HEAP32[$p>>2]|0;
    $19 = ((($18)) + 40|0);
    $20 = HEAP32[$p>>2]|0;
    $21 = ((($20)) + 4|0);
    HEAP32[$21>>2] = $19;
    $22 = HEAP32[$p>>2]|0;
    HEAP32[$22>>2] = 0;
    break;
   }
   HEAP32[$2>>2] = 7;
   $75 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($75|0);
  }
 } while(0);
 $51 = HEAP32[$p>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = HEAP32[$p>>2]|0;
 $54 = ((($53)) + 4|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + ($52)|0);
 $57 = $4;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3Fts3PutVarint($56,$59,$62)|0);
 $64 = HEAP32[$p>>2]|0;
 $65 = HEAP32[$64>>2]|0;
 $66 = (($65) + ($63))|0;
 HEAP32[$64>>2] = $66;
 $67 = HEAP32[$p>>2]|0;
 $68 = HEAP32[$67>>2]|0;
 $69 = HEAP32[$p>>2]|0;
 $70 = ((($69)) + 4|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($71) + ($68)|0);
 HEAP8[$72>>0] = 0;
 $73 = HEAP32[$p>>2]|0;
 $74 = HEAP32[$3>>2]|0;
 HEAP32[$74>>2] = $73;
 HEAP32[$2>>2] = 0;
 $75 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _fts3CursorSeekStmt($pCsr,$ppStmt) {
 $pCsr = $pCsr|0;
 $ppStmt = $ppStmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $p = sp + 8|0;
 $zSql = sp + 4|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $ppStmt;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = HEAP32[$7>>2]|0;
   HEAP32[$p>>2] = $8;
   $9 = HEAP32[$p>>2]|0;
   $10 = ((($9)) + 216|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer>>2] = $11;
   $12 = (_sqlite3_mprintf(23241,$vararg_buffer)|0);
   HEAP32[$zSql>>2] = $12;
   $13 = HEAP32[$zSql>>2]|0;
   $14 = ($13|0)!=(0|0);
   if ($14) {
    $15 = HEAP32[$p>>2]|0;
    $16 = ((($15)) + 12|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$zSql>>2]|0;
    $19 = HEAP32[$1>>2]|0;
    $20 = ((($19)) + 8|0);
    $21 = (_sqlite3_prepare_v2($17,$18,-1,$20,0)|0);
    HEAP32[$rc>>2] = $21;
    $22 = HEAP32[$zSql>>2]|0;
    _sqlite3_free($22);
    break;
   }
   HEAP32[$0>>2] = 7;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 } while(0);
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$2>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _fts3ExprIterate2($pExpr,$piPhrase,$x,$pCtx) {
 $pExpr = $pExpr|0;
 $piPhrase = $piPhrase|0;
 $x = $x|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $or$cond = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $eType = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $piPhrase;
 HEAP32[$2>>2] = $x;
 HEAP32[$3>>2] = $pCtx;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$eType>>2] = $5;
 $6 = HEAP32[$eType>>2]|0;
 $7 = ($6|0)!=(5);
 if (!($7)) {
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  $31 = (FUNCTION_TABLE_iiii[$26 & 255]($27,$29,$30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$32>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_fts3ExprIterate2($10,$11,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 $17 = HEAP32[$eType>>2]|0;
 $18 = ($17|0)!=(2);
 $or$cond = $16 & $18;
 if (!($or$cond)) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 16|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$3>>2]|0;
 $25 = (_fts3ExprIterate2($21,$22,$23,$24)|0);
 HEAP32[$rc>>2] = $25;
 $35 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3Fts3EvalPhraseStats($pCsr,$pExpr,$aiOut) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $aiOut = $aiOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $iCol = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pTab = sp + 8|0;
 $rc = sp + 4|0;
 $iCol = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $aiOut;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 34|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(1);
  if ($14) {
   HEAP32[$iCol>>2] = 0;
   while(1) {
    $15 = HEAP32[$iCol>>2]|0;
    $16 = HEAP32[$pTab>>2]|0;
    $17 = ((($16)) + 24|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($15|0)<($18|0);
    if (!($19)) {
     break;
    }
    $20 = HEAP32[$0>>2]|0;
    $21 = ((($20)) + 64|0);
    $22 = $21;
    $23 = $22;
    $24 = HEAP32[$23>>2]|0;
    $25 = (($22) + 4)|0;
    $26 = $25;
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$iCol>>2]|0;
    $29 = ($28*3)|0;
    $30 = (($29) + 1)|0;
    $31 = HEAP32[$2>>2]|0;
    $32 = (($31) + ($30<<2)|0);
    HEAP32[$32>>2] = $24;
    $33 = HEAP32[$0>>2]|0;
    $34 = ((($33)) + 64|0);
    $35 = $34;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($35) + 4)|0;
    $39 = $38;
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$iCol>>2]|0;
    $42 = ($41*3)|0;
    $43 = (($42) + 2)|0;
    $44 = HEAP32[$2>>2]|0;
    $45 = (($44) + ($43<<2)|0);
    HEAP32[$45>>2] = $37;
    $46 = HEAP32[$iCol>>2]|0;
    $47 = (($46) + 1)|0;
    HEAP32[$iCol>>2] = $47;
   }
   $86 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($86|0);
  }
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$1>>2]|0;
 $50 = (_fts3EvalGatherStats($48,$49)|0);
 HEAP32[$rc>>2] = $50;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if (!($52)) {
  $86 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 HEAP32[$iCol>>2] = 0;
 while(1) {
  $53 = HEAP32[$iCol>>2]|0;
  $54 = HEAP32[$pTab>>2]|0;
  $55 = ((($54)) + 24|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($53|0)<($56|0);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$iCol>>2]|0;
  $59 = ($58*3)|0;
  $60 = (($59) + 1)|0;
  $61 = HEAP32[$1>>2]|0;
  $62 = ((($61)) + 40|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($63) + ($60<<2)|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$iCol>>2]|0;
  $67 = ($66*3)|0;
  $68 = (($67) + 1)|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = (($69) + ($68<<2)|0);
  HEAP32[$70>>2] = $65;
  $71 = HEAP32[$iCol>>2]|0;
  $72 = ($71*3)|0;
  $73 = (($72) + 2)|0;
  $74 = HEAP32[$1>>2]|0;
  $75 = ((($74)) + 40|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = (($76) + ($73<<2)|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = HEAP32[$iCol>>2]|0;
  $80 = ($79*3)|0;
  $81 = (($80) + 2)|0;
  $82 = HEAP32[$2>>2]|0;
  $83 = (($82) + ($81<<2)|0);
  HEAP32[$83>>2] = $78;
  $84 = HEAP32[$iCol>>2]|0;
  $85 = (($84) + 1)|0;
  HEAP32[$iCol>>2] = $85;
 }
 $86 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($86|0);
}
function _fts3EvalGatherStats($pCsr,$pExpr) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bEof = 0, $iDocid = 0, $iPrevId = 0, $p = 0, $pE = 0, $pRoot = 0, $pTab = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $rc = sp + 32|0;
 $pTab = sp + 28|0;
 $pRoot = sp + 24|0;
 $p = sp + 20|0;
 $iPrevId = sp + 8|0;
 $iDocid = sp;
 $bEof = sp + 48|0;
 $pE = sp + 16|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = HEAP32[$7>>2]|0;
   HEAP32[$pTab>>2] = $8;
   $9 = HEAP32[$1>>2]|0;
   $10 = ((($9)) + 32|0);
   $11 = $10;
   $12 = $11;
   $13 = HEAP32[$12>>2]|0;
   $14 = (($11) + 4)|0;
   $15 = $14;
   $16 = HEAP32[$15>>2]|0;
   $17 = $iPrevId;
   $18 = $17;
   HEAP32[$18>>2] = $13;
   $19 = (($17) + 4)|0;
   $20 = $19;
   HEAP32[$20>>2] = $16;
   $21 = HEAP32[$2>>2]|0;
   HEAP32[$pRoot>>2] = $21;
   while(1) {
    $22 = HEAP32[$pRoot>>2]|0;
    $23 = ((($22)) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($24|0)!=(0|0);
    if ($25) {
     $26 = HEAP32[$pRoot>>2]|0;
     $27 = ((($26)) + 8|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = HEAP32[$28>>2]|0;
     $30 = ($29|0)==(1);
     $188 = $30;
    } else {
     $188 = 0;
    }
    $31 = HEAP32[$pRoot>>2]|0;
    if (!($188)) {
     break;
    }
    $32 = ((($31)) + 8|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$pRoot>>2] = $33;
   }
   $34 = ((($31)) + 24|0);
   $35 = $34;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($35) + 4)|0;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = $iDocid;
   $42 = $41;
   HEAP32[$42>>2] = $37;
   $43 = (($41) + 4)|0;
   $44 = $43;
   HEAP32[$44>>2] = $40;
   $45 = HEAP32[$pRoot>>2]|0;
   $46 = ((($45)) + 32|0);
   $47 = HEAP8[$46>>0]|0;
   HEAP8[$bEof>>0] = $47;
   $48 = HEAP32[$pRoot>>2]|0;
   HEAP32[$p>>2] = $48;
   while(1) {
    $49 = HEAP32[$p>>2]|0;
    $50 = ($49|0)!=(0|0);
    if (!($50)) {
     break;
    }
    $51 = HEAP32[$p>>2]|0;
    $52 = HEAP32[$51>>2]|0;
    $53 = ($52|0)==(5);
    $54 = HEAP32[$p>>2]|0;
    if ($53) {
     $57 = $54;
    } else {
     $55 = ((($54)) + 16|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = $56;
    }
    HEAP32[$pE>>2] = $57;
    $58 = HEAP32[$pTab>>2]|0;
    $59 = ((($58)) + 24|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = ($60*3)|0;
    $62 = $61<<2;
    $63 = (_sqlite3_malloc($62)|0);
    $64 = HEAP32[$pE>>2]|0;
    $65 = ((($64)) + 40|0);
    HEAP32[$65>>2] = $63;
    $66 = HEAP32[$pE>>2]|0;
    $67 = ((($66)) + 40|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ($68|0)!=(0|0);
    if (!($69)) {
     label = 12;
     break;
    }
    $70 = HEAP32[$pE>>2]|0;
    $71 = ((($70)) + 40|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = HEAP32[$pTab>>2]|0;
    $74 = ((($73)) + 24|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = ($75*3)|0;
    $77 = $76<<2;
    _memset(($72|0),0,($77|0))|0;
    $78 = HEAP32[$p>>2]|0;
    $79 = ((($78)) + 12|0);
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$p>>2] = $80;
   }
   if ((label|0) == 12) {
    HEAP32[$0>>2] = 7;
    $187 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($187|0);
   }
   $81 = HEAP32[$1>>2]|0;
   $82 = HEAP32[$pRoot>>2]|0;
   _fts3EvalRestart($81,$82,$rc);
   while(1) {
    $83 = HEAP32[$1>>2]|0;
    $84 = ((($83)) + 6|0);
    $85 = HEAP8[$84>>0]|0;
    $86 = $85&255;
    $87 = ($86|0)==(0);
    $88 = HEAP32[$rc>>2]|0;
    $89 = ($88|0)==(0);
    $90 = $87 ? $89 : 0;
    if (!($90)) {
     break;
    }
    while(1) {
     $91 = HEAP32[$1>>2]|0;
     $92 = ((($91)) + 7|0);
     $93 = HEAP8[$92>>0]|0;
     $94 = $93&255;
     $95 = ($94|0)==(0);
     if ($95) {
      $96 = HEAP32[$1>>2]|0;
      $97 = ((($96)) + 8|0);
      $98 = HEAP32[$97>>2]|0;
      (_sqlite3_reset($98)|0);
     }
     $99 = HEAP32[$1>>2]|0;
     $100 = HEAP32[$pRoot>>2]|0;
     _fts3EvalNextRow($99,$100,$rc);
     $101 = HEAP32[$pRoot>>2]|0;
     $102 = ((($101)) + 32|0);
     $103 = HEAP8[$102>>0]|0;
     $104 = HEAP32[$1>>2]|0;
     $105 = ((($104)) + 6|0);
     HEAP8[$105>>0] = $103;
     $106 = HEAP32[$1>>2]|0;
     $107 = ((($106)) + 7|0);
     HEAP8[$107>>0] = 1;
     $108 = HEAP32[$1>>2]|0;
     $109 = ((($108)) + 88|0);
     HEAP32[$109>>2] = 1;
     $110 = HEAP32[$pRoot>>2]|0;
     $111 = ((($110)) + 24|0);
     $112 = $111;
     $113 = $112;
     $114 = HEAP32[$113>>2]|0;
     $115 = (($112) + 4)|0;
     $116 = $115;
     $117 = HEAP32[$116>>2]|0;
     $118 = HEAP32[$1>>2]|0;
     $119 = ((($118)) + 32|0);
     $120 = $119;
     $121 = $120;
     HEAP32[$121>>2] = $114;
     $122 = (($120) + 4)|0;
     $123 = $122;
     HEAP32[$123>>2] = $117;
     $124 = HEAP32[$1>>2]|0;
     $125 = ((($124)) + 6|0);
     $126 = HEAP8[$125>>0]|0;
     $127 = $126&255;
     $128 = ($127|0)==(0);
     if (!($128)) {
      break;
     }
     $129 = HEAP32[$pRoot>>2]|0;
     $130 = HEAP32[$129>>2]|0;
     $131 = ($130|0)==(1);
     if (!($131)) {
      break;
     }
     $132 = HEAP32[$1>>2]|0;
     $133 = (_sqlite3Fts3EvalTestDeferred($132,$rc)|0);
     $134 = ($133|0)!=(0);
     if (!($134)) {
      break;
     }
    }
    $135 = HEAP32[$rc>>2]|0;
    $136 = ($135|0)==(0);
    if (!($136)) {
     continue;
    }
    $137 = HEAP32[$1>>2]|0;
    $138 = ((($137)) + 6|0);
    $139 = HEAP8[$138>>0]|0;
    $140 = $139&255;
    $141 = ($140|0)==(0);
    if (!($141)) {
     continue;
    }
    $142 = HEAP32[$pRoot>>2]|0;
    _fts3EvalUpdateCounts($142);
   }
   $143 = HEAP32[$1>>2]|0;
   $144 = ((($143)) + 6|0);
   HEAP8[$144>>0] = 0;
   $145 = $iPrevId;
   $146 = $145;
   $147 = HEAP32[$146>>2]|0;
   $148 = (($145) + 4)|0;
   $149 = $148;
   $150 = HEAP32[$149>>2]|0;
   $151 = HEAP32[$1>>2]|0;
   $152 = ((($151)) + 32|0);
   $153 = $152;
   $154 = $153;
   HEAP32[$154>>2] = $147;
   $155 = (($153) + 4)|0;
   $156 = $155;
   HEAP32[$156>>2] = $150;
   $157 = HEAP8[$bEof>>0]|0;
   $158 = ($157<<24>>24)!=(0);
   if ($158) {
    $159 = HEAP8[$bEof>>0]|0;
    $160 = HEAP32[$pRoot>>2]|0;
    $161 = ((($160)) + 32|0);
    HEAP8[$161>>0] = $159;
    break;
   }
   $162 = HEAP32[$1>>2]|0;
   $163 = HEAP32[$pRoot>>2]|0;
   _fts3EvalRestart($162,$163,$rc);
   while(1) {
    $164 = HEAP32[$1>>2]|0;
    $165 = HEAP32[$pRoot>>2]|0;
    _fts3EvalNextRow($164,$165,$rc);
    $166 = HEAP32[$pRoot>>2]|0;
    $167 = ((($166)) + 24|0);
    $168 = $167;
    $169 = $168;
    $170 = HEAP32[$169>>2]|0;
    $171 = (($168) + 4)|0;
    $172 = $171;
    $173 = HEAP32[$172>>2]|0;
    $174 = $iDocid;
    $175 = $174;
    $176 = HEAP32[$175>>2]|0;
    $177 = (($174) + 4)|0;
    $178 = $177;
    $179 = HEAP32[$178>>2]|0;
    $180 = ($170|0)!=($176|0);
    $181 = ($173|0)!=($179|0);
    $182 = $180 | $181;
    $183 = HEAP32[$rc>>2]|0;
    $184 = ($183|0)==(0);
    $185 = $182 ? $184 : 0;
    if (!($185)) {
     break;
    }
   }
  }
 } while(0);
 $186 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $186;
 $187 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($187|0);
}
function _fts3EvalUpdateCounts($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $c = 0, $iCnt = 0, $iCol = 0, $p = 0, $pPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $pPhrase = sp + 12|0;
 $iCol = sp + 8|0;
 $p = sp + 4|0;
 $c = sp + 20|0;
 $iCnt = sp;
 HEAP32[$0>>2] = $pExpr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPhrase>>2] = $5;
 $6 = HEAP32[$pPhrase>>2]|0;
 $7 = ($6|0)!=(0|0);
 L4: do {
  if ($7) {
   $8 = HEAP32[$pPhrase>>2]|0;
   $9 = ((($8)) + 28|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    HEAP32[$iCol>>2] = 0;
    $12 = HEAP32[$pPhrase>>2]|0;
    $13 = ((($12)) + 28|0);
    $14 = HEAP32[$13>>2]|0;
    HEAP32[$p>>2] = $14;
    while(1) {
     HEAP8[$c>>0] = 0;
     HEAP32[$iCnt>>2] = 0;
     while(1) {
      $15 = HEAP32[$p>>2]|0;
      $16 = HEAP8[$15>>0]|0;
      $17 = $16 << 24 >> 24;
      $18 = HEAP8[$c>>0]|0;
      $19 = $18&255;
      $20 = $17 | $19;
      $21 = 254 & $20;
      $22 = ($21|0)!=(0);
      if (!($22)) {
       break;
      }
      $23 = HEAP8[$c>>0]|0;
      $24 = $23&255;
      $25 = $24 & 128;
      $26 = ($25|0)==(0);
      if ($26) {
       $27 = HEAP32[$iCnt>>2]|0;
       $28 = (($27) + 1)|0;
       HEAP32[$iCnt>>2] = $28;
      }
      $29 = HEAP32[$p>>2]|0;
      $30 = ((($29)) + 1|0);
      HEAP32[$p>>2] = $30;
      $31 = HEAP8[$29>>0]|0;
      $32 = $31 << 24 >> 24;
      $33 = $32 & 128;
      $34 = $33&255;
      HEAP8[$c>>0] = $34;
     }
     $35 = HEAP32[$iCnt>>2]|0;
     $36 = HEAP32[$iCol>>2]|0;
     $37 = ($36*3)|0;
     $38 = (($37) + 1)|0;
     $39 = HEAP32[$0>>2]|0;
     $40 = ((($39)) + 40|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = (($41) + ($38<<2)|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = (($43) + ($35))|0;
     HEAP32[$42>>2] = $44;
     $45 = HEAP32[$iCnt>>2]|0;
     $46 = ($45|0)>(0);
     $47 = $46&1;
     $48 = HEAP32[$iCol>>2]|0;
     $49 = ($48*3)|0;
     $50 = (($49) + 2)|0;
     $51 = HEAP32[$0>>2]|0;
     $52 = ((($51)) + 40|0);
     $53 = HEAP32[$52>>2]|0;
     $54 = (($53) + ($50<<2)|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (($55) + ($47))|0;
     HEAP32[$54>>2] = $56;
     $57 = HEAP32[$p>>2]|0;
     $58 = HEAP8[$57>>0]|0;
     $59 = $58 << 24 >> 24;
     $60 = ($59|0)==(0);
     if ($60) {
      break L4;
     }
     $61 = HEAP32[$p>>2]|0;
     $62 = ((($61)) + 1|0);
     HEAP32[$p>>2] = $62;
     $63 = HEAP32[$p>>2]|0;
     $64 = HEAP8[$63>>0]|0;
     $65 = $64&255;
     $66 = $65 & 128;
     $67 = ($66|0)!=(0);
     $68 = HEAP32[$p>>2]|0;
     if ($67) {
      $69 = (_sqlite3Fts3GetVarint32($68,$iCol)|0);
      $74 = $69;
     } else {
      $70 = HEAP8[$68>>0]|0;
      $71 = $70&255;
      HEAP32[$iCol>>2] = $71;
      $74 = 1;
     }
     $72 = HEAP32[$p>>2]|0;
     $73 = (($72) + ($74)|0);
     HEAP32[$p>>2] = $73;
    }
   }
  }
 } while(0);
 $75 = HEAP32[$0>>2]|0;
 $76 = ((($75)) + 12|0);
 $77 = HEAP32[$76>>2]|0;
 _fts3EvalUpdateCounts($77);
 $78 = HEAP32[$0>>2]|0;
 $79 = ((($78)) + 16|0);
 $80 = HEAP32[$79>>2]|0;
 _fts3EvalUpdateCounts($80);
 STACKTOP = sp;return;
}
function _fts3ExprLHits($pExpr,$p) {
 $pExpr = $pExpr|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $iCol = 0, $iStart = 0, $nHit = 0, $pIter = 0, $pPhrase = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pTab = sp + 20|0;
 $iStart = sp + 16|0;
 $pPhrase = sp + 12|0;
 $pIter = sp + 8|0;
 $iCol = sp + 4|0;
 $nHit = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pPhrase>>2] = $7;
 $8 = HEAP32[$pPhrase>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pIter>>2] = $10;
 HEAP32[$iCol>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13 << 24 >> 24;
 $15 = ($14|0)==(121);
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 36|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 if ($15) {
  $22 = Math_imul($18, $21)|0;
  HEAP32[$iStart>>2] = $22;
 } else {
  $23 = (($21) + 31)|0;
  $24 = (($23|0) / 32)&-1;
  $25 = Math_imul($18, $24)|0;
  HEAP32[$iStart>>2] = $25;
 }
 while(1) {
  $26 = (_fts3ColumnlistCount($pIter)|0);
  HEAP32[$nHit>>2] = $26;
  $27 = HEAP32[$pPhrase>>2]|0;
  $28 = ((($27)) + 68|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pTab>>2]|0;
  $31 = ((($30)) + 24|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($29|0)>=($32|0);
  if ($33) {
   label = 6;
  } else {
   $34 = HEAP32[$pPhrase>>2]|0;
   $35 = ((($34)) + 68|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$iCol>>2]|0;
   $38 = ($36|0)==($37|0);
   if ($38) {
    label = 6;
   }
  }
  do {
   if ((label|0) == 6) {
    label = 0;
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 24|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = $41 << 24 >> 24;
    $43 = ($42|0)==(121);
    $44 = HEAP32[$nHit>>2]|0;
    if ($43) {
     $45 = HEAP32[$iStart>>2]|0;
     $46 = HEAP32[$iCol>>2]|0;
     $47 = (($45) + ($46))|0;
     $48 = HEAP32[$1>>2]|0;
     $49 = ((($48)) + 28|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = (($50) + ($47<<2)|0);
     HEAP32[$51>>2] = $44;
     break;
    }
    $52 = ($44|0)!=(0);
    if ($52) {
     $53 = HEAP32[$iCol>>2]|0;
     $54 = $53 & 31;
     $55 = 1 << $54;
     $56 = HEAP32[$iStart>>2]|0;
     $57 = HEAP32[$iCol>>2]|0;
     $58 = (($57) + 1)|0;
     $59 = (($58|0) / 32)&-1;
     $60 = (($56) + ($59))|0;
     $61 = HEAP32[$1>>2]|0;
     $62 = ((($61)) + 28|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = (($63) + ($60<<2)|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = $65 | $55;
     HEAP32[$64>>2] = $66;
    }
   }
  } while(0);
  $67 = HEAP32[$pIter>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 << 24 >> 24;
  $70 = ($69|0)!=(1);
  if ($70) {
   break;
  }
  $71 = HEAP32[$pIter>>2]|0;
  $72 = ((($71)) + 1|0);
  HEAP32[$pIter>>2] = $72;
  $73 = HEAP32[$pIter>>2]|0;
  $74 = HEAP8[$73>>0]|0;
  $75 = $74&255;
  $76 = $75 & 128;
  $77 = ($76|0)!=(0);
  $78 = HEAP32[$pIter>>2]|0;
  if ($77) {
   $79 = (_sqlite3Fts3GetVarint32($78,$iCol)|0);
   $84 = $79;
  } else {
   $80 = HEAP8[$78>>0]|0;
   $81 = $80&255;
   HEAP32[$iCol>>2] = $81;
   $84 = 1;
  }
  $82 = HEAP32[$pIter>>2]|0;
  $83 = (($82) + ($84)|0);
  HEAP32[$pIter>>2] = $83;
 }
 STACKTOP = sp;return;
}
function _fts3MatchinfoLcsCb($pExpr,$iPhrase,$pCtx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $aIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $aIter = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $pCtx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$aIter>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$aIter>>2]|0;
 $7 = (($6) + ($5<<4)|0);
 HEAP32[$7>>2] = $4;
 STACKTOP = sp;return 0;
}
function _fts3LcsIteratorAdvance($pIter) {
 $pIter = $pIter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $iRead = 0, $or$cond = 0, $pRead = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $pRead = sp + 12|0;
 $iRead = sp;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $pIter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pRead>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$pRead>>2]|0;
 $5 = (_sqlite3Fts3GetVarint($4,$iRead)|0);
 $6 = HEAP32[$pRead>>2]|0;
 $7 = (($6) + ($5)|0);
 HEAP32[$pRead>>2] = $7;
 $8 = $iRead;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)==(0);
 $15 = ($13|0)==(0);
 $16 = $14 & $15;
 $17 = $iRead;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($19|0)==(1);
 $24 = ($22|0)==(0);
 $25 = $23 & $24;
 $or$cond = $16 | $25;
 if ($or$cond) {
  HEAP32[$pRead>>2] = 0;
  HEAP32[$rc>>2] = 1;
  $38 = HEAP32[$pRead>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 } else {
  $26 = $iRead;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (_i64Subtract(($28|0),($31|0),2,0)|0);
  $33 = tempRet0;
  $34 = HEAP32[$0>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + ($32))|0;
  HEAP32[$35>>2] = $37;
  $38 = HEAP32[$pRead>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 return (0)|0;
}
function _fts3ExprLoadDoclistsCb($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pPhrase = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pPhrase = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPhrase>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 $11 = HEAP32[$pPhrase>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13))|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _fts3SelectDocsize($pTab,$0,$1,$ppStmt) {
 $pTab = $pTab|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppStmt = $ppStmt|0;
 var $$ = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $4 = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $pTab;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $ppStmt;
 HEAP32[$pStmt>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,21,$pStmt,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = HEAP32[$pStmt>>2]|0;
   $14 = $3;
   $15 = $14;
   $16 = HEAP32[$15>>2]|0;
   $17 = (($14) + 4)|0;
   $18 = $17;
   $19 = HEAP32[$18>>2]|0;
   (_sqlite3_bind_int64($13,1,$16,$19)|0);
   $20 = HEAP32[$pStmt>>2]|0;
   $21 = (_sqlite3_step($20)|0);
   HEAP32[$rc>>2] = $21;
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)!=(100);
   if (!($23)) {
    $24 = HEAP32[$pStmt>>2]|0;
    $25 = (_sqlite3_column_type($24,0)|0);
    $26 = ($25|0)!=(4);
    if (!($26)) {
     HEAP32[$rc>>2] = 0;
     break;
    }
   }
   $27 = HEAP32[$pStmt>>2]|0;
   $28 = (_sqlite3_reset($27)|0);
   HEAP32[$rc>>2] = $28;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(0);
   $$ = $30 ? 267 : $28;
   HEAP32[$rc>>2] = $$;
   HEAP32[$pStmt>>2] = 0;
  }
 } while(0);
 $31 = HEAP32[$pStmt>>2]|0;
 $32 = HEAP32[$4>>2]|0;
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3Fts3SelectDoctotal($pTab,$ppStmt) {
 $pTab = $pTab|0;
 $ppStmt = $ppStmt|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $ppStmt;
 HEAP32[$pStmt>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,22,$pStmt,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 do {
  if ($5) {
   $6 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_bind_int($6,1,0)|0);
   $7 = HEAP32[$pStmt>>2]|0;
   $8 = (_sqlite3_step($7)|0);
   $9 = ($8|0)!=(100);
   if (!($9)) {
    $10 = HEAP32[$pStmt>>2]|0;
    $11 = (_sqlite3_column_type($10,0)|0);
    $12 = ($11|0)!=(4);
    if (!($12)) {
     break;
    }
   }
   $13 = HEAP32[$pStmt>>2]|0;
   $14 = (_sqlite3_reset($13)|0);
   HEAP32[$rc>>2] = $14;
   $15 = HEAP32[$rc>>2]|0;
   $16 = ($15|0)==(0);
   $$ = $16 ? 267 : $14;
   HEAP32[$rc>>2] = $$;
   HEAP32[$pStmt>>2] = 0;
  }
 } while(0);
 $17 = HEAP32[$pStmt>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _fts3MIBufferFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pBuf = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + -4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (0 - ($4))|0;
 $6 = (($1) + ($5)|0);
 HEAP32[$pBuf>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pBuf>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = ((($9)) + 4|0);
 $11 = ($7|0)==($10|0);
 $12 = HEAP32[$pBuf>>2]|0;
 if ($11) {
  $13 = ((($12)) + 1|0);
  HEAP8[$13>>0] = 0;
 } else {
  $14 = ((($12)) + 2|0);
  HEAP8[$14>>0] = 0;
 }
 $15 = HEAP32[$pBuf>>2]|0;
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$pBuf>>2]|0;
 $20 = ((($19)) + 1|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = ($22|0)==(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$pBuf>>2]|0;
 $25 = ((($24)) + 2|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = ($27|0)==(0);
 if (!($28)) {
  STACKTOP = sp;return;
 }
 $29 = HEAP32[$pBuf>>2]|0;
 _sqlite3_free($29);
 STACKTOP = sp;return;
}
function _fts3ExprPhraseCountCb($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 36|0);
 HEAP32[$8>>2] = $6;
 STACKTOP = sp;return 0;
}
function _sqlite3Fts3Optimize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $rc2 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3_exec($3,23369,0,0,0)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = (_fts3DoOptimize($7,1)|0);
   HEAP32[$rc>>2] = $8;
   $9 = HEAP32[$rc>>2]|0;
   $10 = ($9|0)==(0);
   $11 = HEAP32[$rc>>2]|0;
   $12 = ($11|0)==(101);
   $or$cond = $10 | $12;
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   if (!($or$cond)) {
    (_sqlite3_exec($15,23397,0,0,0)|0);
    $20 = HEAP32[$0>>2]|0;
    $21 = ((($20)) + 12|0);
    $22 = HEAP32[$21>>2]|0;
    (_sqlite3_exec($22,23384,0,0,0)|0);
    break;
   }
   $16 = (_sqlite3_exec($15,23384,0,0,0)|0);
   HEAP32[$rc2>>2] = $16;
   $17 = HEAP32[$rc2>>2]|0;
   $18 = ($17|0)!=(0);
   if ($18) {
    $19 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $19;
   }
  }
 } while(0);
 $23 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegmentsClose($23);
 $24 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _fts3DoOptimize($p,$bReturnDone) {
 $p = $p|0;
 $bReturnDone = $bReturnDone|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bSeenDone = 0, $i = 0, $iLangid = 0, $or$cond = 0, $or$cond3 = 0, $pAllLangid = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $bSeenDone = sp + 20|0;
 $rc = sp + 16|0;
 $pAllLangid = sp + 12|0;
 $rc2 = sp + 8|0;
 $i = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $bReturnDone;
 HEAP32[$bSeenDone>>2] = 0;
 HEAP32[$pAllLangid>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,27,$pAllLangid,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[$pAllLangid>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 272|0);
  $9 = HEAP32[$8>>2]|0;
  (_sqlite3_bind_int($6,1,$9)|0);
  $10 = HEAP32[$pAllLangid>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 248|0);
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3_bind_int($10,2,$13)|0);
  L3: while(1) {
   $14 = HEAP32[$pAllLangid>>2]|0;
   $15 = (_sqlite3_step($14)|0);
   $16 = ($15|0)==(100);
   $17 = HEAP32[$pAllLangid>>2]|0;
   if (!($16)) {
    break;
   }
   $18 = (_sqlite3_column_int($17,0)|0);
   HEAP32[$iLangid>>2] = $18;
   HEAP32[$i>>2] = 0;
   while(1) {
    $19 = HEAP32[$rc>>2]|0;
    $20 = ($19|0)==(0);
    if (!($20)) {
     continue L3;
    }
    $21 = HEAP32[$i>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 248|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = ($21|0)<($24|0);
    if (!($25)) {
     continue L3;
    }
    $26 = HEAP32[$0>>2]|0;
    $27 = HEAP32[$iLangid>>2]|0;
    $28 = HEAP32[$i>>2]|0;
    $29 = (_fts3SegmentMerge($26,$27,$28,-2)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$rc>>2]|0;
    $31 = ($30|0)==(101);
    if ($31) {
     HEAP32[$bSeenDone>>2] = 1;
     HEAP32[$rc>>2] = 0;
    }
    $32 = HEAP32[$i>>2]|0;
    $33 = (($32) + 1)|0;
    HEAP32[$i>>2] = $33;
   }
  }
  $34 = (_sqlite3_reset($17)|0);
  HEAP32[$rc2>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $37;
  }
 }
 $38 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegmentsClose($38);
 $39 = HEAP32[$0>>2]|0;
 _sqlite3Fts3PendingTermsClear($39);
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(0);
 $42 = HEAP32[$1>>2]|0;
 $43 = ($42|0)!=(0);
 $or$cond = $41 & $43;
 $44 = HEAP32[$bSeenDone>>2]|0;
 $45 = ($44|0)!=(0);
 $or$cond3 = $or$cond & $45;
 $46 = HEAP32[$rc>>2]|0;
 $47 = $or$cond3 ? 101 : $46;
 STACKTOP = sp;return ($47|0);
}
function _sqlite3Fts3Offsets($pCtx,$pCsr) {
 $pCtx = $pCtx|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $NDUMMY = 0, $ZDUMMY = 0, $aBuffer = 0, $i = 0, $iCol = 0, $iCurrent = 0, $iEnd = 0, $iMinPos = 0, $iStart = 0, $nDoc = 0, $nToken = 0, $pC = 0, $pMod = 0, $pT = 0, $pTab = 0, $pTerm = 0, $rc = 0, $res = 0, $sCtx = 0;
 var $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zDoc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer = sp + 32|0;
 $0 = sp + 132|0;
 $1 = sp + 128|0;
 $pTab = sp + 124|0;
 $pMod = sp + 120|0;
 $rc = sp + 116|0;
 $nToken = sp + 112|0;
 $iCol = sp + 108|0;
 $res = sp + 96|0;
 $sCtx = sp;
 $pC = sp + 92|0;
 $ZDUMMY = sp + 88|0;
 $NDUMMY = sp + 84|0;
 $iStart = sp + 80|0;
 $iEnd = sp + 76|0;
 $iCurrent = sp + 72|0;
 $zDoc = sp + 68|0;
 $nDoc = sp + 64|0;
 $i = sp + 60|0;
 $iMinPos = sp + 56|0;
 $pTerm = sp + 52|0;
 $pT = sp + 48|0;
 $aBuffer = sp + 136|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pCsr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTab>>2] = $3;
 $4 = HEAP32[$pTab>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pMod>>2] = $7;
 ;HEAP32[$res>>2]=0|0;HEAP32[$res+4>>2]=0|0;HEAP32[$res+8>>2]=0|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $12 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($12,18402,0,0);
  STACKTOP = sp;return;
 }
 ;HEAP32[$sCtx>>2]=0|0;HEAP32[$sCtx+4>>2]=0|0;HEAP32[$sCtx+8>>2]=0|0;HEAP32[$sCtx+12>>2]=0|0;HEAP32[$sCtx+16>>2]=0|0;HEAP32[$sCtx+20>>2]=0|0;HEAP32[$sCtx+24>>2]=0|0;HEAP32[$sCtx+28>>2]=0|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_fts3ExprLoadDoclists($13,0,$nToken)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)!=(0);
 L5: do {
  if (!($16)) {
   $17 = HEAP32[$nToken>>2]|0;
   $18 = ($17*12)|0;
   $19 = (_sqlite3_malloc($18)|0);
   $20 = ((($sCtx)) + 24|0);
   HEAP32[$20>>2] = $19;
   $21 = ((($sCtx)) + 24|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (0|0)==($22|0);
   if ($23) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 32|0);
   $26 = $25;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = ((($sCtx)) + 16|0);
   $33 = $32;
   $34 = $33;
   HEAP32[$34>>2] = $28;
   $35 = (($33) + 4)|0;
   $36 = $35;
   HEAP32[$36>>2] = $31;
   $37 = HEAP32[$1>>2]|0;
   HEAP32[$sCtx>>2] = $37;
   HEAP32[$iCol>>2] = 0;
   while(1) {
    $38 = HEAP32[$iCol>>2]|0;
    $39 = HEAP32[$pTab>>2]|0;
    $40 = ((($39)) + 24|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = ($38|0)<($41|0);
    if (!($42)) {
     break L5;
    }
    HEAP32[$NDUMMY>>2] = 0;
    HEAP32[$iStart>>2] = 0;
    HEAP32[$iEnd>>2] = 0;
    HEAP32[$iCurrent>>2] = 0;
    $43 = HEAP32[$iCol>>2]|0;
    $44 = ((($sCtx)) + 4|0);
    HEAP32[$44>>2] = $43;
    $45 = ((($sCtx)) + 8|0);
    HEAP32[$45>>2] = 0;
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 12|0);
    $48 = HEAP32[$47>>2]|0;
    (_fts3ExprIterate($48,158,$sCtx)|0);
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 8|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$iCol>>2]|0;
    $53 = (($52) + 1)|0;
    $54 = (_sqlite3_column_text($51,$53)|0);
    HEAP32[$zDoc>>2] = $54;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 8|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = HEAP32[$iCol>>2]|0;
    $59 = (($58) + 1)|0;
    $60 = (_sqlite3_column_bytes($57,$59)|0);
    HEAP32[$nDoc>>2] = $60;
    $61 = HEAP32[$zDoc>>2]|0;
    $62 = ($61|0)==(0|0);
    if ($62) {
     $63 = HEAP32[$1>>2]|0;
     $64 = ((($63)) + 8|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$iCol>>2]|0;
     $67 = (($66) + 1)|0;
     $68 = (_sqlite3_column_type($65,$67)|0);
     $69 = ($68|0)==(5);
     if (!($69)) {
      break;
     }
    } else {
     $70 = HEAP32[$pTab>>2]|0;
     $71 = ((($70)) + 36|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = HEAP32[$1>>2]|0;
     $74 = ((($73)) + 16|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = HEAP32[$zDoc>>2]|0;
     $77 = HEAP32[$nDoc>>2]|0;
     $78 = (_sqlite3Fts3OpenTokenizer($72,$75,$76,$77,$pC)|0);
     HEAP32[$rc>>2] = $78;
     $79 = HEAP32[$rc>>2]|0;
     $80 = ($79|0)!=(0);
     if ($80) {
      break L5;
     }
     $81 = HEAP32[$pMod>>2]|0;
     $82 = ((($81)) + 20|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = HEAP32[$pC>>2]|0;
     $85 = (FUNCTION_TABLE_iiiiiii[$83 & 255]($84,$ZDUMMY,$NDUMMY,$iStart,$iEnd,$iCurrent)|0);
     HEAP32[$rc>>2] = $85;
     while(1) {
      $86 = HEAP32[$rc>>2]|0;
      $87 = ($86|0)==(0);
      if (!($87)) {
       break;
      }
      HEAP32[$iMinPos>>2] = 2147483647;
      HEAP32[$pTerm>>2] = 0;
      HEAP32[$i>>2] = 0;
      while(1) {
       $88 = HEAP32[$i>>2]|0;
       $89 = HEAP32[$nToken>>2]|0;
       $90 = ($88|0)<($89|0);
       if (!($90)) {
        break;
       }
       $91 = HEAP32[$i>>2]|0;
       $92 = ((($sCtx)) + 24|0);
       $93 = HEAP32[$92>>2]|0;
       $94 = (($93) + (($91*12)|0)|0);
       HEAP32[$pT>>2] = $94;
       $95 = HEAP32[$pT>>2]|0;
       $96 = HEAP32[$95>>2]|0;
       $97 = ($96|0)!=(0|0);
       if ($97) {
        $98 = HEAP32[$pT>>2]|0;
        $99 = ((($98)) + 4|0);
        $100 = HEAP32[$99>>2]|0;
        $101 = HEAP32[$pT>>2]|0;
        $102 = ((($101)) + 8|0);
        $103 = HEAP32[$102>>2]|0;
        $104 = (($100) - ($103))|0;
        $105 = HEAP32[$iMinPos>>2]|0;
        $106 = ($104|0)<($105|0);
        if ($106) {
         $107 = HEAP32[$pT>>2]|0;
         $108 = ((($107)) + 4|0);
         $109 = HEAP32[$108>>2]|0;
         $110 = HEAP32[$pT>>2]|0;
         $111 = ((($110)) + 8|0);
         $112 = HEAP32[$111>>2]|0;
         $113 = (($109) - ($112))|0;
         HEAP32[$iMinPos>>2] = $113;
         $114 = HEAP32[$pT>>2]|0;
         HEAP32[$pTerm>>2] = $114;
        }
       }
       $115 = HEAP32[$i>>2]|0;
       $116 = (($115) + 1)|0;
       HEAP32[$i>>2] = $116;
      }
      $117 = HEAP32[$pTerm>>2]|0;
      $118 = ($117|0)!=(0|0);
      if (!($118)) {
       HEAP32[$rc>>2] = 101;
       continue;
      }
      $119 = HEAP32[$pTerm>>2]|0;
      $120 = HEAP32[$119>>2]|0;
      $121 = HEAP8[$120>>0]|0;
      $122 = $121 << 24 >> 24;
      $123 = 254 & $122;
      $124 = (0)==($123|0);
      $125 = HEAP32[$pTerm>>2]|0;
      if ($124) {
       HEAP32[$125>>2] = 0;
      } else {
       $126 = HEAP32[$pTerm>>2]|0;
       $127 = ((($126)) + 4|0);
       _fts3GetDeltaPosition($125,$127);
      }
      while(1) {
       $128 = HEAP32[$rc>>2]|0;
       $129 = ($128|0)==(0);
       if (!($129)) {
        break;
       }
       $130 = HEAP32[$iCurrent>>2]|0;
       $131 = HEAP32[$iMinPos>>2]|0;
       $132 = ($130|0)<($131|0);
       if (!($132)) {
        break;
       }
       $133 = HEAP32[$pMod>>2]|0;
       $134 = ((($133)) + 20|0);
       $135 = HEAP32[$134>>2]|0;
       $136 = HEAP32[$pC>>2]|0;
       $137 = (FUNCTION_TABLE_iiiiiii[$135 & 255]($136,$ZDUMMY,$NDUMMY,$iStart,$iEnd,$iCurrent)|0);
       HEAP32[$rc>>2] = $137;
      }
      $138 = HEAP32[$rc>>2]|0;
      $139 = ($138|0)==(0);
      if ($139) {
       $140 = HEAP32[$iCol>>2]|0;
       $141 = HEAP32[$pTerm>>2]|0;
       $142 = ((($sCtx)) + 24|0);
       $143 = HEAP32[$142>>2]|0;
       $144 = $141;
       $145 = $143;
       $146 = (($144) - ($145))|0;
       $147 = (($146|0) / 12)&-1;
       $148 = HEAP32[$iStart>>2]|0;
       $149 = HEAP32[$iEnd>>2]|0;
       $150 = HEAP32[$iStart>>2]|0;
       $151 = (($149) - ($150))|0;
       HEAP32[$vararg_buffer>>2] = $140;
       $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
       HEAP32[$vararg_ptr1>>2] = $147;
       $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
       HEAP32[$vararg_ptr2>>2] = $148;
       $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
       HEAP32[$vararg_ptr3>>2] = $151;
       (_sqlite3_snprintf(64,$aBuffer,23414,$vararg_buffer)|0);
       $152 = (_fts3StringAppend($res,$aBuffer,-1)|0);
       HEAP32[$rc>>2] = $152;
       continue;
      }
      $153 = HEAP32[$rc>>2]|0;
      $154 = ($153|0)==(101);
      if (!($154)) {
       continue;
      }
      $155 = HEAP32[$pTab>>2]|0;
      $156 = ((($155)) + 40|0);
      $157 = HEAP32[$156>>2]|0;
      $158 = ($157|0)==(0|0);
      if (!($158)) {
       continue;
      }
      HEAP32[$rc>>2] = 267;
     }
     $159 = HEAP32[$rc>>2]|0;
     $160 = ($159|0)==(101);
     if ($160) {
      HEAP32[$rc>>2] = 0;
     }
     $161 = HEAP32[$pMod>>2]|0;
     $162 = ((($161)) + 16|0);
     $163 = HEAP32[$162>>2]|0;
     $164 = HEAP32[$pC>>2]|0;
     (FUNCTION_TABLE_ii[$163 & 255]($164)|0);
     $165 = HEAP32[$rc>>2]|0;
     $166 = ($165|0)!=(0);
     if ($166) {
      break L5;
     }
    }
    $167 = HEAP32[$iCol>>2]|0;
    $168 = (($167) + 1)|0;
    HEAP32[$iCol>>2] = $168;
   }
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $169 = ((($sCtx)) + 24|0);
 $170 = HEAP32[$169>>2]|0;
 _sqlite3_free($170);
 $171 = HEAP32[$pTab>>2]|0;
 _sqlite3Fts3SegmentsClose($171);
 $172 = HEAP32[$rc>>2]|0;
 $173 = ($172|0)!=(0);
 $174 = HEAP32[$0>>2]|0;
 if ($173) {
  $175 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($174,$175);
  $176 = HEAP32[$res>>2]|0;
  _sqlite3_free($176);
  STACKTOP = sp;return;
 } else {
  $177 = HEAP32[$res>>2]|0;
  $178 = ((($res)) + 4|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = (($179) - 1)|0;
  _sqlite3_result_text($174,$177,$180,148);
  STACKTOP = sp;return;
 }
}
function _fts3ExprTermOffsetInit($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iPos = 0, $iTerm = 0, $nTerm = 0, $p = 0, $pList = 0, $pT = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $p = sp + 24|0;
 $nTerm = sp + 20|0;
 $iTerm = sp + 16|0;
 $pList = sp + 12|0;
 $iPos = sp + 8|0;
 $rc = sp + 4|0;
 $pT = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 HEAP32[$iPos>>2] = 0;
 $4 = HEAP32[$p>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3Fts3EvalPhrasePoslist($5,$6,$9,$pList)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 64|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$nTerm>>2] = $15;
 $16 = HEAP32[$pList>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  _fts3GetDeltaPosition($pList,$iPos);
 }
 HEAP32[$iTerm>>2] = 0;
 while(1) {
  $18 = HEAP32[$iTerm>>2]|0;
  $19 = HEAP32[$nTerm>>2]|0;
  $20 = ($18|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$p>>2]|0;
  $22 = ((($21)) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$22>>2] = $24;
  $25 = HEAP32[$p>>2]|0;
  $26 = ((($25)) + 24|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($23*12)|0)|0);
  HEAP32[$pT>>2] = $28;
  $29 = HEAP32[$nTerm>>2]|0;
  $30 = HEAP32[$iTerm>>2]|0;
  $31 = (($29) - ($30))|0;
  $32 = (($31) - 1)|0;
  $33 = HEAP32[$pT>>2]|0;
  $34 = ((($33)) + 8|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$pList>>2]|0;
  $36 = HEAP32[$pT>>2]|0;
  HEAP32[$36>>2] = $35;
  $37 = HEAP32[$iPos>>2]|0;
  $38 = HEAP32[$pT>>2]|0;
  $39 = ((($38)) + 4|0);
  HEAP32[$39>>2] = $37;
  $40 = HEAP32[$iTerm>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$iTerm>>2] = $41;
 }
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _fts3GetDeltaPosition($pp,$piPos) {
 $pp = $pp|0;
 $piPos = $piPos|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $iVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iVal = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $piPos;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 & 128;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 if ($7) {
  $10 = (_sqlite3Fts3GetVarint32($9,$iVal)|0);
  $16 = $10;
 } else {
  $11 = HEAP8[$9>>0]|0;
  $12 = $11&255;
  HEAP32[$iVal>>2] = $12;
  $16 = 1;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($16)|0);
 HEAP32[$13>>2] = $15;
 $17 = HEAP32[$iVal>>2]|0;
 $18 = (($17) - 2)|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($18))|0;
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _fts3StringAppend($pStr,$zAppend,$nAppend) {
 $pStr = $pStr|0;
 $zAppend = $zAppend|0;
 $nAppend = $nAppend|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nAlloc = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $nAlloc = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $pStr;
 HEAP32[$2>>2] = $zAppend;
 HEAP32[$3>>2] = $nAppend;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<(0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_strlen($6)|0);
  HEAP32[$3>>2] = $7;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (($10) + ($11))|0;
 $13 = (($12) + 1)|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($13|0)>=($16|0);
 do {
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$3>>2]|0;
   $22 = (($20) + ($21))|0;
   $23 = (($22) + 100)|0;
   HEAP32[$nAlloc>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$nAlloc>>2]|0;
   $27 = (_sqlite3_realloc($25,$26)|0);
   HEAP32[$zNew>>2] = $27;
   $28 = HEAP32[$zNew>>2]|0;
   $29 = ($28|0)!=(0|0);
   if ($29) {
    $30 = HEAP32[$zNew>>2]|0;
    $31 = HEAP32[$1>>2]|0;
    HEAP32[$31>>2] = $30;
    $32 = HEAP32[$nAlloc>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 8|0);
    HEAP32[$34>>2] = $32;
    break;
   }
   HEAP32[$0>>2] = 7;
   $54 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($54|0);
  }
 } while(0);
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($39) + ($37)|0);
 $41 = HEAP32[$2>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 _memcpy(($40|0),($41|0),($42|0))|0;
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 4|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + ($43))|0;
 HEAP32[$45>>2] = $47;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 4|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$1>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + ($50)|0);
 HEAP8[$53>>0] = 0;
 HEAP32[$0>>2] = 0;
 $54 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _sqlite3Fts3Snippet($pCtx,$pCsr,$zStart,$zEnd,$zEllipsis,$iCol,$nToken) {
 $pCtx = $pCtx|0;
 $pCsr = $pCsr|0;
 $zStart = $zStart|0;
 $zEnd = $zEnd|0;
 $zEllipsis = $zEllipsis|0;
 $iCol = $iCol|0;
 $nToken = $nToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aSnippet = 0, $i = 0, $iBestScore = 0, $iRead = 0;
 var $iS = 0, $iSnip = 0, $mCovered = 0, $mSeen = 0, $nFToken = 0, $nSnippet = 0, $or$cond = 0, $pFragment = 0, $pTab = 0, $rc = 0, $res = 0, $sF = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 212|0;
 $1 = sp + 208|0;
 $2 = sp + 204|0;
 $3 = sp + 200|0;
 $4 = sp + 196|0;
 $5 = sp + 192|0;
 $6 = sp + 188|0;
 $pTab = sp + 184|0;
 $rc = sp + 180|0;
 $i = sp + 176|0;
 $res = sp + 164|0;
 $nSnippet = sp + 160|0;
 $aSnippet = sp + 40|0;
 $nFToken = sp + 156|0;
 $iSnip = sp + 152|0;
 $mCovered = sp + 32|0;
 $mSeen = sp + 24|0;
 $iBestScore = sp + 148|0;
 $iRead = sp + 144|0;
 $pFragment = sp + 140|0;
 $sF = sp;
 $iS = sp + 136|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $zStart;
 HEAP32[$3>>2] = $zEnd;
 HEAP32[$4>>2] = $zEllipsis;
 HEAP32[$5>>2] = $iCol;
 HEAP32[$6>>2] = $nToken;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTab>>2] = $8;
 HEAP32[$rc>>2] = 0;
 ;HEAP32[$res>>2]=0|0;HEAP32[$res+4>>2]=0|0;HEAP32[$res+8>>2]=0|0;
 HEAP32[$nSnippet>>2] = 0;
 HEAP32[$nFToken>>2] = -1;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  $13 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($13,18402,0,0);
  STACKTOP = sp;return;
 }
 HEAP32[$nSnippet>>2] = 1;
 L5: while(1) {
  $14 = $mCovered;
  $15 = $14;
  HEAP32[$15>>2] = 0;
  $16 = (($14) + 4)|0;
  $17 = $16;
  HEAP32[$17>>2] = 0;
  $18 = $mSeen;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$6>>2]|0;
  $23 = ($22|0)>=(0);
  $24 = HEAP32[$6>>2]|0;
  if ($23) {
   $25 = HEAP32[$nSnippet>>2]|0;
   $26 = (($24) + ($25))|0;
   $27 = (($26) - 1)|0;
   $28 = HEAP32[$nSnippet>>2]|0;
   $29 = (($27|0) / ($28|0))&-1;
   HEAP32[$nFToken>>2] = $29;
  } else {
   $30 = Math_imul(-1, $24)|0;
   HEAP32[$nFToken>>2] = $30;
  }
  HEAP32[$iSnip>>2] = 0;
  while(1) {
   $31 = HEAP32[$iSnip>>2]|0;
   $32 = HEAP32[$nSnippet>>2]|0;
   $33 = ($31|0)<($32|0);
   if (!($33)) {
    break;
   }
   HEAP32[$iBestScore>>2] = -1;
   $34 = HEAP32[$iSnip>>2]|0;
   $35 = (($aSnippet) + (($34*24)|0)|0);
   HEAP32[$pFragment>>2] = $35;
   $36 = HEAP32[$pFragment>>2]|0;
   ;HEAP32[$36>>2]=0|0;HEAP32[$36+4>>2]=0|0;HEAP32[$36+8>>2]=0|0;HEAP32[$36+12>>2]=0|0;HEAP32[$36+16>>2]=0|0;HEAP32[$36+20>>2]=0|0;
   HEAP32[$iRead>>2] = 0;
   while(1) {
    $37 = HEAP32[$iRead>>2]|0;
    $38 = HEAP32[$pTab>>2]|0;
    $39 = ((($38)) + 24|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = ($37|0)<($40|0);
    if (!($41)) {
     break;
    }
    ;HEAP32[$sF>>2]=0|0;HEAP32[$sF+4>>2]=0|0;HEAP32[$sF+8>>2]=0|0;HEAP32[$sF+12>>2]=0|0;HEAP32[$sF+16>>2]=0|0;HEAP32[$sF+20>>2]=0|0;
    HEAP32[$iS>>2] = 0;
    $42 = HEAP32[$5>>2]|0;
    $43 = ($42|0)>=(0);
    if ($43) {
     $44 = HEAP32[$iRead>>2]|0;
     $45 = HEAP32[$5>>2]|0;
     $46 = ($44|0)!=($45|0);
     if (!($46)) {
      label = 13;
     }
    } else {
     label = 13;
    }
    if ((label|0) == 13) {
     label = 0;
     $47 = HEAP32[$nFToken>>2]|0;
     $48 = HEAP32[$1>>2]|0;
     $49 = HEAP32[$iRead>>2]|0;
     $50 = $mCovered;
     $51 = $50;
     $52 = HEAP32[$51>>2]|0;
     $53 = (($50) + 4)|0;
     $54 = $53;
     $55 = HEAP32[$54>>2]|0;
     $56 = (_fts3BestSnippet($47,$48,$49,$52,$55,$mSeen,$sF,$iS)|0);
     HEAP32[$rc>>2] = $56;
     $57 = HEAP32[$rc>>2]|0;
     $58 = ($57|0)!=(0);
     if ($58) {
      break L5;
     }
     $59 = HEAP32[$iS>>2]|0;
     $60 = HEAP32[$iBestScore>>2]|0;
     $61 = ($59|0)>($60|0);
     if ($61) {
      $62 = HEAP32[$pFragment>>2]|0;
      ;HEAP32[$62>>2]=HEAP32[$sF>>2]|0;HEAP32[$62+4>>2]=HEAP32[$sF+4>>2]|0;HEAP32[$62+8>>2]=HEAP32[$sF+8>>2]|0;HEAP32[$62+12>>2]=HEAP32[$sF+12>>2]|0;HEAP32[$62+16>>2]=HEAP32[$sF+16>>2]|0;HEAP32[$62+20>>2]=HEAP32[$sF+20>>2]|0;
      $63 = HEAP32[$iS>>2]|0;
      HEAP32[$iBestScore>>2] = $63;
     }
    }
    $64 = HEAP32[$iRead>>2]|0;
    $65 = (($64) + 1)|0;
    HEAP32[$iRead>>2] = $65;
   }
   $66 = HEAP32[$pFragment>>2]|0;
   $67 = ((($66)) + 8|0);
   $68 = $67;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $71 = (($68) + 4)|0;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = $mCovered;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (($74) + 4)|0;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = $76 | $70;
   $81 = $79 | $73;
   $82 = $mCovered;
   $83 = $82;
   HEAP32[$83>>2] = $80;
   $84 = (($82) + 4)|0;
   $85 = $84;
   HEAP32[$85>>2] = $81;
   $86 = HEAP32[$iSnip>>2]|0;
   $87 = (($86) + 1)|0;
   HEAP32[$iSnip>>2] = $87;
  }
  $88 = $mSeen;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = $mCovered;
  $95 = $94;
  $96 = HEAP32[$95>>2]|0;
  $97 = (($94) + 4)|0;
  $98 = $97;
  $99 = HEAP32[$98>>2]|0;
  $100 = ($90|0)==($96|0);
  $101 = ($93|0)==($99|0);
  $102 = $100 & $101;
  $103 = HEAP32[$nSnippet>>2]|0;
  $104 = ($103|0)==(4);
  $or$cond = $102 | $104;
  if ($or$cond) {
   label = 20;
   break;
  }
  $105 = HEAP32[$nSnippet>>2]|0;
  $106 = (($105) + 1)|0;
  HEAP32[$nSnippet>>2] = $106;
 }
 L27: do {
  if ((label|0) == 20) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $107 = HEAP32[$i>>2]|0;
    $108 = HEAP32[$nSnippet>>2]|0;
    $109 = ($107|0)<($108|0);
    $110 = HEAP32[$rc>>2]|0;
    $111 = ($110|0)==(0);
    $112 = $109 ? $111 : 0;
    if (!($112)) {
     break L27;
    }
    $113 = HEAP32[$1>>2]|0;
    $114 = HEAP32[$i>>2]|0;
    $115 = (($aSnippet) + (($114*24)|0)|0);
    $116 = HEAP32[$i>>2]|0;
    $117 = HEAP32[$i>>2]|0;
    $118 = HEAP32[$nSnippet>>2]|0;
    $119 = (($118) - 1)|0;
    $120 = ($117|0)==($119|0);
    $121 = $120&1;
    $122 = HEAP32[$nFToken>>2]|0;
    $123 = HEAP32[$2>>2]|0;
    $124 = HEAP32[$3>>2]|0;
    $125 = HEAP32[$4>>2]|0;
    $126 = (_fts3SnippetText($113,$115,$116,$121,$122,$123,$124,$125,$res)|0);
    HEAP32[$rc>>2] = $126;
    $127 = HEAP32[$i>>2]|0;
    $128 = (($127) + 1)|0;
    HEAP32[$i>>2] = $128;
   }
  }
 } while(0);
 $129 = HEAP32[$pTab>>2]|0;
 _sqlite3Fts3SegmentsClose($129);
 $130 = HEAP32[$rc>>2]|0;
 $131 = ($130|0)!=(0);
 $132 = HEAP32[$0>>2]|0;
 if ($131) {
  $133 = HEAP32[$rc>>2]|0;
  _sqlite3_result_error_code($132,$133);
  $134 = HEAP32[$res>>2]|0;
  _sqlite3_free($134);
  STACKTOP = sp;return;
 } else {
  $135 = HEAP32[$res>>2]|0;
  _sqlite3_result_text($132,$135,-1,148);
  STACKTOP = sp;return;
 }
}
function _fts3BestSnippet($nSnippet,$pCsr,$iCol,$0,$1,$pmSeen,$pFragment,$piScore) {
 $nSnippet = $nSnippet|0;
 $pCsr = $pCsr|0;
 $iCol = $iCol|0;
 $0 = $0|0;
 $1 = $1|0;
 $pmSeen = $pmSeen|0;
 $pFragment = $pFragment|0;
 $piScore = $piScore|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iBestScore = 0;
 var $iPos = 0, $iScore = 0, $mCover = 0, $mHighlite = 0, $nByte = 0, $nList = 0, $rc = 0, $sIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $6 = sp + 16|0;
 $7 = sp + 88|0;
 $8 = sp + 84|0;
 $9 = sp + 80|0;
 $rc = sp + 76|0;
 $nList = sp + 72|0;
 $sIter = sp + 48|0;
 $nByte = sp + 40|0;
 $iBestScore = sp + 36|0;
 $i = sp + 32|0;
 $iPos = sp + 28|0;
 $iScore = sp + 24|0;
 $mCover = sp + 8|0;
 $mHighlite = sp;
 HEAP32[$3>>2] = $nSnippet;
 HEAP32[$4>>2] = $pCsr;
 HEAP32[$5>>2] = $iCol;
 $10 = $6;
 $11 = $10;
 HEAP32[$11>>2] = $0;
 $12 = (($10) + 4)|0;
 $13 = $12;
 HEAP32[$13>>2] = $1;
 HEAP32[$7>>2] = $pmSeen;
 HEAP32[$8>>2] = $pFragment;
 HEAP32[$9>>2] = $piScore;
 HEAP32[$iBestScore>>2] = -1;
 ;HEAP32[$sIter>>2]=0|0;HEAP32[$sIter+4>>2]=0|0;HEAP32[$sIter+8>>2]=0|0;HEAP32[$sIter+12>>2]=0|0;HEAP32[$sIter+16>>2]=0|0;HEAP32[$sIter+20>>2]=0|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = (_fts3ExprLoadDoclists($14,$nList,0)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $18;
  $119 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $19 = HEAP32[$nList>>2]|0;
 $20 = ($19*24)|0;
 HEAP32[$nByte>>2] = $20;
 $21 = HEAP32[$nByte>>2]|0;
 $22 = (_sqlite3_malloc($21)|0);
 $23 = ((($sIter)) + 16|0);
 HEAP32[$23>>2] = $22;
 $24 = ((($sIter)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  HEAP32[$2>>2] = 7;
  $119 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($119|0);
 }
 $27 = ((($sIter)) + 16|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$nByte>>2]|0;
 _memset(($28|0),0,($29|0))|0;
 $30 = HEAP32[$4>>2]|0;
 HEAP32[$sIter>>2] = $30;
 $31 = HEAP32[$5>>2]|0;
 $32 = ((($sIter)) + 4|0);
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$3>>2]|0;
 $34 = ((($sIter)) + 8|0);
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$nList>>2]|0;
 $36 = ((($sIter)) + 12|0);
 HEAP32[$36>>2] = $35;
 $37 = ((($sIter)) + 20|0);
 HEAP32[$37>>2] = -1;
 $38 = HEAP32[$4>>2]|0;
 $39 = ((($38)) + 12|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (_fts3ExprIterate($40,159,$sIter)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $44 = HEAP32[$i>>2]|0;
   $45 = HEAP32[$nList>>2]|0;
   $46 = ($44|0)<($45|0);
   if (!($46)) {
    break;
   }
   $47 = HEAP32[$i>>2]|0;
   $48 = ((($sIter)) + 16|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = (($49) + (($47*24)|0)|0);
   $51 = ((($50)) + 12|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)!=(0|0);
   if ($53) {
    $54 = HEAP32[$i>>2]|0;
    $55 = (_bitshift64Shl(1,0,($54|0))|0);
    $56 = tempRet0;
    $57 = HEAP32[$7>>2]|0;
    $58 = $57;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = (($58) + 4)|0;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = $60 | $55;
    $65 = $63 | $56;
    $66 = $57;
    $67 = $66;
    HEAP32[$67>>2] = $64;
    $68 = (($66) + 4)|0;
    $69 = $68;
    HEAP32[$69>>2] = $65;
   }
   $70 = HEAP32[$i>>2]|0;
   $71 = (($70) + 1)|0;
   HEAP32[$i>>2] = $71;
  }
  $72 = HEAP32[$5>>2]|0;
  $73 = HEAP32[$8>>2]|0;
  HEAP32[$73>>2] = $72;
  while(1) {
   $74 = (_fts3SnippetNextCandidate($sIter)|0);
   $75 = ($74|0)!=(0);
   $76 = $75 ^ 1;
   if (!($76)) {
    break;
   }
   $77 = $6;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = (($77) + 4)|0;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   _fts3SnippetDetails($sIter,$79,$82,$iPos,$iScore,$mCover,$mHighlite);
   $83 = HEAP32[$iScore>>2]|0;
   $84 = HEAP32[$iBestScore>>2]|0;
   $85 = ($83|0)>($84|0);
   if (!($85)) {
    continue;
   }
   $86 = HEAP32[$iPos>>2]|0;
   $87 = HEAP32[$8>>2]|0;
   $88 = ((($87)) + 4|0);
   HEAP32[$88>>2] = $86;
   $89 = $mHighlite;
   $90 = $89;
   $91 = HEAP32[$90>>2]|0;
   $92 = (($89) + 4)|0;
   $93 = $92;
   $94 = HEAP32[$93>>2]|0;
   $95 = HEAP32[$8>>2]|0;
   $96 = ((($95)) + 16|0);
   $97 = $96;
   $98 = $97;
   HEAP32[$98>>2] = $91;
   $99 = (($97) + 4)|0;
   $100 = $99;
   HEAP32[$100>>2] = $94;
   $101 = $mCover;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = (($101) + 4)|0;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = HEAP32[$8>>2]|0;
   $108 = ((($107)) + 8|0);
   $109 = $108;
   $110 = $109;
   HEAP32[$110>>2] = $103;
   $111 = (($109) + 4)|0;
   $112 = $111;
   HEAP32[$112>>2] = $106;
   $113 = HEAP32[$iScore>>2]|0;
   HEAP32[$iBestScore>>2] = $113;
  }
  $114 = HEAP32[$iBestScore>>2]|0;
  $115 = HEAP32[$9>>2]|0;
  HEAP32[$115>>2] = $114;
 }
 $116 = ((($sIter)) + 16|0);
 $117 = HEAP32[$116>>2]|0;
 _sqlite3_free($117);
 $118 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $118;
 $119 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($119|0);
}
function _fts3SnippetText($pCsr,$pFragment,$iFragment,$isLast,$nSnippet,$zOpen,$zClose,$zEllipsis,$pOut) {
 $pCsr = $pCsr|0;
 $pFragment = $pFragment|0;
 $iFragment = $iFragment|0;
 $isLast = $isLast|0;
 $nSnippet = $nSnippet|0;
 $zOpen = $zOpen|0;
 $zClose = $zClose|0;
 $zEllipsis = $zEllipsis|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $DUMMY1 = 0, $ZDUMMY = 0, $hlmask = 0, $iBegin = 0, $iCol = 0, $iCurrent = 0, $iEnd = 0, $iFin = 0, $iPos = 0, $isHighlight = 0, $isShiftDone = 0, $n = 0, $nDoc = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pC = 0;
 var $pMod = 0, $pTab = 0, $rc = 0, $zDoc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $6 = sp + 88|0;
 $7 = sp + 84|0;
 $8 = sp + 80|0;
 $9 = sp + 76|0;
 $pTab = sp + 72|0;
 $rc = sp + 68|0;
 $zDoc = sp + 64|0;
 $nDoc = sp + 60|0;
 $iCurrent = sp + 56|0;
 $iEnd = sp + 52|0;
 $isShiftDone = sp + 48|0;
 $iPos = sp + 44|0;
 $hlmask = sp;
 $iCol = sp + 40|0;
 $pMod = sp + 36|0;
 $pC = sp + 32|0;
 $ZDUMMY = sp + 28|0;
 $DUMMY1 = sp + 24|0;
 $iBegin = sp + 20|0;
 $iFin = sp + 16|0;
 $isHighlight = sp + 12|0;
 $n = sp + 8|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pFragment;
 HEAP32[$3>>2] = $iFragment;
 HEAP32[$4>>2] = $isLast;
 HEAP32[$5>>2] = $nSnippet;
 HEAP32[$6>>2] = $zOpen;
 HEAP32[$7>>2] = $zClose;
 HEAP32[$8>>2] = $zEllipsis;
 HEAP32[$9>>2] = $pOut;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pTab>>2] = $11;
 HEAP32[$iCurrent>>2] = 0;
 HEAP32[$iEnd>>2] = 0;
 HEAP32[$isShiftDone>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$iPos>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = $hlmask;
 $24 = $23;
 HEAP32[$24>>2] = $19;
 $25 = (($23) + 4)|0;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + 1)|0;
 HEAP32[$iCol>>2] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$iCol>>2]|0;
 $34 = (_sqlite3_column_text($32,$33)|0);
 HEAP32[$zDoc>>2] = $34;
 $35 = HEAP32[$zDoc>>2]|0;
 $36 = ($35|0)==(0|0);
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$iCol>>2]|0;
 if ($36) {
  $41 = (_sqlite3_column_type($39,$40)|0);
  $42 = ($41|0)!=(5);
  if ($42) {
   HEAP32[$0>>2] = 7;
   $182 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($182|0);
  } else {
   HEAP32[$0>>2] = 0;
   $182 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($182|0);
  }
 }
 $43 = (_sqlite3_column_bytes($39,$40)|0);
 HEAP32[$nDoc>>2] = $43;
 $44 = HEAP32[$pTab>>2]|0;
 $45 = ((($44)) + 36|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pMod>>2] = $47;
 $48 = HEAP32[$pTab>>2]|0;
 $49 = ((($48)) + 36|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$zDoc>>2]|0;
 $55 = HEAP32[$nDoc>>2]|0;
 $56 = (_sqlite3Fts3OpenTokenizer($50,$53,$54,$55,$pC)|0);
 HEAP32[$rc>>2] = $56;
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $59;
  $182 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($182|0);
 }
 while(1) {
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(0);
  if (!($61)) {
   break;
  }
  HEAP32[$DUMMY1>>2] = -1;
  HEAP32[$iBegin>>2] = 0;
  HEAP32[$iFin>>2] = 0;
  HEAP32[$isHighlight>>2] = 0;
  $62 = HEAP32[$pMod>>2]|0;
  $63 = ((($62)) + 20|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = HEAP32[$pC>>2]|0;
  $66 = (FUNCTION_TABLE_iiiiiii[$64 & 255]($65,$ZDUMMY,$DUMMY1,$iBegin,$iFin,$iCurrent)|0);
  HEAP32[$rc>>2] = $66;
  $67 = HEAP32[$rc>>2]|0;
  $68 = ($67|0)!=(0);
  if ($68) {
   label = 9;
   break;
  }
  $76 = HEAP32[$iCurrent>>2]|0;
  $77 = HEAP32[$iPos>>2]|0;
  $78 = ($76|0)<($77|0);
  if ($78) {
   continue;
  }
  $79 = HEAP32[$isShiftDone>>2]|0;
  $80 = ($79|0)!=(0);
  if (!($80)) {
   $81 = HEAP32[$nDoc>>2]|0;
   $82 = HEAP32[$iBegin>>2]|0;
   $83 = (($81) - ($82))|0;
   HEAP32[$n>>2] = $83;
   $84 = HEAP32[$pTab>>2]|0;
   $85 = HEAP32[$1>>2]|0;
   $86 = ((($85)) + 16|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$5>>2]|0;
   $89 = HEAP32[$iBegin>>2]|0;
   $90 = HEAP32[$zDoc>>2]|0;
   $91 = (($90) + ($89)|0);
   $92 = HEAP32[$n>>2]|0;
   $93 = (_fts3SnippetShift($84,$87,$88,$91,$92,$iPos,$hlmask)|0);
   HEAP32[$rc>>2] = $93;
   HEAP32[$isShiftDone>>2] = 1;
   $94 = HEAP32[$rc>>2]|0;
   $95 = ($94|0)==(0);
   do {
    if ($95) {
     $96 = HEAP32[$iPos>>2]|0;
     $97 = ($96|0)>(0);
     $98 = HEAP32[$3>>2]|0;
     $99 = ($98|0)>(0);
     $or$cond = $97 | $99;
     if ($or$cond) {
      $100 = HEAP32[$9>>2]|0;
      $101 = HEAP32[$8>>2]|0;
      $102 = (_fts3StringAppend($100,$101,-1)|0);
      HEAP32[$rc>>2] = $102;
      break;
     }
     $103 = HEAP32[$iBegin>>2]|0;
     $104 = ($103|0)!=(0);
     if ($104) {
      $105 = HEAP32[$9>>2]|0;
      $106 = HEAP32[$zDoc>>2]|0;
      $107 = HEAP32[$iBegin>>2]|0;
      $108 = (_fts3StringAppend($105,$106,$107)|0);
      HEAP32[$rc>>2] = $108;
     }
    }
   } while(0);
   $109 = HEAP32[$rc>>2]|0;
   $110 = ($109|0)!=(0);
   if ($110) {
    continue;
   }
   $111 = HEAP32[$iCurrent>>2]|0;
   $112 = HEAP32[$iPos>>2]|0;
   $113 = ($111|0)<($112|0);
   if ($113) {
    continue;
   }
  }
  $114 = HEAP32[$iCurrent>>2]|0;
  $115 = HEAP32[$iPos>>2]|0;
  $116 = HEAP32[$5>>2]|0;
  $117 = (($115) + ($116))|0;
  $118 = ($114|0)>=($117|0);
  if ($118) {
   label = 21;
   break;
  }
  $124 = $hlmask;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = (($124) + 4)|0;
  $128 = $127;
  $129 = HEAP32[$128>>2]|0;
  $130 = HEAP32[$iCurrent>>2]|0;
  $131 = HEAP32[$iPos>>2]|0;
  $132 = (($130) - ($131))|0;
  $133 = (_bitshift64Shl(1,0,($132|0))|0);
  $134 = tempRet0;
  $135 = $126 & $133;
  $136 = $129 & $134;
  $137 = ($135|0)!=(0);
  $138 = ($136|0)!=(0);
  $139 = $137 | $138;
  $140 = $139&1;
  HEAP32[$isHighlight>>2] = $140;
  $141 = HEAP32[$iCurrent>>2]|0;
  $142 = HEAP32[$iPos>>2]|0;
  $143 = ($141|0)>($142|0);
  if ($143) {
   $144 = HEAP32[$9>>2]|0;
   $145 = HEAP32[$iEnd>>2]|0;
   $146 = HEAP32[$zDoc>>2]|0;
   $147 = (($146) + ($145)|0);
   $148 = HEAP32[$iBegin>>2]|0;
   $149 = HEAP32[$iEnd>>2]|0;
   $150 = (($148) - ($149))|0;
   $151 = (_fts3StringAppend($144,$147,$150)|0);
   HEAP32[$rc>>2] = $151;
  }
  $152 = HEAP32[$rc>>2]|0;
  $153 = ($152|0)==(0);
  $154 = HEAP32[$isHighlight>>2]|0;
  $155 = ($154|0)!=(0);
  $or$cond3 = $153 & $155;
  if ($or$cond3) {
   $156 = HEAP32[$9>>2]|0;
   $157 = HEAP32[$6>>2]|0;
   $158 = (_fts3StringAppend($156,$157,-1)|0);
   HEAP32[$rc>>2] = $158;
  }
  $159 = HEAP32[$rc>>2]|0;
  $160 = ($159|0)==(0);
  if ($160) {
   $161 = HEAP32[$9>>2]|0;
   $162 = HEAP32[$iBegin>>2]|0;
   $163 = HEAP32[$zDoc>>2]|0;
   $164 = (($163) + ($162)|0);
   $165 = HEAP32[$iFin>>2]|0;
   $166 = HEAP32[$iBegin>>2]|0;
   $167 = (($165) - ($166))|0;
   $168 = (_fts3StringAppend($161,$164,$167)|0);
   HEAP32[$rc>>2] = $168;
  }
  $169 = HEAP32[$rc>>2]|0;
  $170 = ($169|0)==(0);
  $171 = HEAP32[$isHighlight>>2]|0;
  $172 = ($171|0)!=(0);
  $or$cond5 = $170 & $172;
  if ($or$cond5) {
   $173 = HEAP32[$9>>2]|0;
   $174 = HEAP32[$7>>2]|0;
   $175 = (_fts3StringAppend($173,$174,-1)|0);
   HEAP32[$rc>>2] = $175;
  }
  $176 = HEAP32[$iFin>>2]|0;
  HEAP32[$iEnd>>2] = $176;
 }
 if ((label|0) == 9) {
  $69 = HEAP32[$rc>>2]|0;
  $70 = ($69|0)==(101);
  if ($70) {
   $71 = HEAP32[$9>>2]|0;
   $72 = HEAP32[$iEnd>>2]|0;
   $73 = HEAP32[$zDoc>>2]|0;
   $74 = (($73) + ($72)|0);
   $75 = (_fts3StringAppend($71,$74,-1)|0);
   HEAP32[$rc>>2] = $75;
  }
 }
 else if ((label|0) == 21) {
  $119 = HEAP32[$4>>2]|0;
  $120 = ($119|0)!=(0);
  if ($120) {
   $121 = HEAP32[$9>>2]|0;
   $122 = HEAP32[$8>>2]|0;
   $123 = (_fts3StringAppend($121,$122,-1)|0);
   HEAP32[$rc>>2] = $123;
  }
 }
 $177 = HEAP32[$pMod>>2]|0;
 $178 = ((($177)) + 16|0);
 $179 = HEAP32[$178>>2]|0;
 $180 = HEAP32[$pC>>2]|0;
 (FUNCTION_TABLE_ii[$179 & 255]($180)|0);
 $181 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $181;
 $182 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($182|0);
}
function _fts3SnippetShift($pTab,$iLangid,$nSnippet,$zDoc,$nDoc,$piPos,$pHlmask) {
 $pTab = $pTab|0;
 $iLangid = $iLangid|0;
 $nSnippet = $nSnippet|0;
 $zDoc = $zDoc|0;
 $nDoc = $nDoc|0;
 $piPos = $piPos|0;
 $pHlmask = $pHlmask|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $DUMMY1 = 0, $DUMMY2 = 0, $DUMMY3 = 0, $ZDUMMY = 0, $hlmask = 0, $iCurrent = 0, $nDesired = 0;
 var $nLeft = 0, $nRight = 0, $nShift = 0, $or$cond = 0, $pC = 0, $pMod = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp + 64|0;
 $6 = sp + 60|0;
 $7 = sp + 56|0;
 $hlmask = sp;
 $nLeft = sp + 52|0;
 $nRight = sp + 48|0;
 $nDesired = sp + 44|0;
 $nShift = sp + 40|0;
 $iCurrent = sp + 36|0;
 $rc = sp + 32|0;
 $pMod = sp + 28|0;
 $pC = sp + 24|0;
 $ZDUMMY = sp + 20|0;
 $DUMMY1 = sp + 16|0;
 $DUMMY2 = sp + 12|0;
 $DUMMY3 = sp + 8|0;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $nSnippet;
 HEAP32[$4>>2] = $zDoc;
 HEAP32[$5>>2] = $nDoc;
 HEAP32[$6>>2] = $piPos;
 HEAP32[$7>>2] = $pHlmask;
 $8 = HEAP32[$7>>2]|0;
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = $hlmask;
 $16 = $15;
 HEAP32[$16>>2] = $11;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = $14;
 $19 = $hlmask;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($21|0)!=(0);
 $26 = ($24|0)!=(0);
 $27 = $25 | $26;
 do {
  if ($27) {
   HEAP32[$nLeft>>2] = 0;
   while(1) {
    $28 = $hlmask;
    $29 = $28;
    $30 = HEAP32[$29>>2]|0;
    $31 = (($28) + 4)|0;
    $32 = $31;
    $33 = HEAP32[$32>>2]|0;
    $34 = HEAP32[$nLeft>>2]|0;
    $35 = (_bitshift64Shl(1,0,($34|0))|0);
    $36 = tempRet0;
    $37 = $30 & $35;
    $38 = $33 & $36;
    $39 = ($37|0)!=(0);
    $40 = ($38|0)!=(0);
    $41 = $39 | $40;
    $42 = $41 ^ 1;
    if (!($42)) {
     break;
    }
    $43 = HEAP32[$nLeft>>2]|0;
    $44 = (($43) + 1)|0;
    HEAP32[$nLeft>>2] = $44;
   }
   HEAP32[$nRight>>2] = 0;
   while(1) {
    $45 = $hlmask;
    $46 = $45;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($45) + 4)|0;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$3>>2]|0;
    $52 = (($51) - 1)|0;
    $53 = HEAP32[$nRight>>2]|0;
    $54 = (($52) - ($53))|0;
    $55 = (_bitshift64Shl(1,0,($54|0))|0);
    $56 = tempRet0;
    $57 = $47 & $55;
    $58 = $50 & $56;
    $59 = ($57|0)!=(0);
    $60 = ($58|0)!=(0);
    $61 = $59 | $60;
    $62 = $61 ^ 1;
    if (!($62)) {
     break;
    }
    $63 = HEAP32[$nRight>>2]|0;
    $64 = (($63) + 1)|0;
    HEAP32[$nRight>>2] = $64;
   }
   $65 = HEAP32[$nLeft>>2]|0;
   $66 = HEAP32[$nRight>>2]|0;
   $67 = (($65) - ($66))|0;
   $68 = (($67|0) / 2)&-1;
   HEAP32[$nDesired>>2] = $68;
   $69 = HEAP32[$nDesired>>2]|0;
   $70 = ($69|0)>(0);
   if ($70) {
    HEAP32[$iCurrent>>2] = 0;
    $71 = HEAP32[$1>>2]|0;
    $72 = ((($71)) + 36|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    HEAP32[$pMod>>2] = $74;
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 36|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$2>>2]|0;
    $79 = HEAP32[$4>>2]|0;
    $80 = HEAP32[$5>>2]|0;
    $81 = (_sqlite3Fts3OpenTokenizer($77,$78,$79,$80,$pC)|0);
    HEAP32[$rc>>2] = $81;
    $82 = HEAP32[$rc>>2]|0;
    $83 = ($82|0)!=(0);
    if ($83) {
     $84 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $84;
     $132 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($132|0);
    }
    while(1) {
     $85 = HEAP32[$rc>>2]|0;
     $86 = ($85|0)==(0);
     if (!($86)) {
      break;
     }
     $87 = HEAP32[$iCurrent>>2]|0;
     $88 = HEAP32[$3>>2]|0;
     $89 = HEAP32[$nDesired>>2]|0;
     $90 = (($88) + ($89))|0;
     $91 = ($87|0)<($90|0);
     if (!($91)) {
      break;
     }
     HEAP32[$DUMMY1>>2] = 0;
     HEAP32[$DUMMY2>>2] = 0;
     HEAP32[$DUMMY3>>2] = 0;
     $92 = HEAP32[$pMod>>2]|0;
     $93 = ((($92)) + 20|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = HEAP32[$pC>>2]|0;
     $96 = (FUNCTION_TABLE_iiiiiii[$94 & 255]($95,$ZDUMMY,$DUMMY1,$DUMMY2,$DUMMY3,$iCurrent)|0);
     HEAP32[$rc>>2] = $96;
    }
    $97 = HEAP32[$pMod>>2]|0;
    $98 = ((($97)) + 16|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = HEAP32[$pC>>2]|0;
    (FUNCTION_TABLE_ii[$99 & 255]($100)|0);
    $101 = HEAP32[$rc>>2]|0;
    $102 = ($101|0)!=(0);
    $103 = HEAP32[$rc>>2]|0;
    $104 = ($103|0)!=(101);
    $or$cond = $102 & $104;
    $105 = HEAP32[$rc>>2]|0;
    if ($or$cond) {
     HEAP32[$0>>2] = $105;
     $132 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($132|0);
    } else {
     $106 = ($105|0)==(101);
     $107 = $106&1;
     $108 = HEAP32[$iCurrent>>2]|0;
     $109 = (($107) + ($108))|0;
     $110 = HEAP32[$3>>2]|0;
     $111 = (($109) - ($110))|0;
     HEAP32[$nShift>>2] = $111;
     $112 = HEAP32[$nShift>>2]|0;
     $113 = ($112|0)>(0);
     if (!($113)) {
      break;
     }
     $114 = HEAP32[$nShift>>2]|0;
     $115 = HEAP32[$6>>2]|0;
     $116 = HEAP32[$115>>2]|0;
     $117 = (($116) + ($114))|0;
     HEAP32[$115>>2] = $117;
     $118 = $hlmask;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = (($118) + 4)|0;
     $122 = $121;
     $123 = HEAP32[$122>>2]|0;
     $124 = HEAP32[$nShift>>2]|0;
     $125 = (_bitshift64Lshr(($120|0),($123|0),($124|0))|0);
     $126 = tempRet0;
     $127 = HEAP32[$7>>2]|0;
     $128 = $127;
     $129 = $128;
     HEAP32[$129>>2] = $125;
     $130 = (($128) + 4)|0;
     $131 = $130;
     HEAP32[$131>>2] = $126;
     break;
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $132 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _fts3SnippetFindPositions($pExpr,$iPhrase,$ctx) {
 $pExpr = $pExpr|0;
 $iPhrase = $iPhrase|0;
 $ctx = $ctx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iFirst = 0, $p = 0, $pCsr = 0, $pPhrase = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $p = sp + 16|0;
 $pPhrase = sp + 12|0;
 $pCsr = sp + 8|0;
 $rc = sp + 4|0;
 $iFirst = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iPhrase;
 HEAP32[$2>>2] = $ctx;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + (($4*24)|0)|0);
 HEAP32[$pPhrase>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$pPhrase>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$p>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3Fts3EvalPhrasePoslist($16,$17,$20,$pCsr)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$pCsr>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $39 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 HEAP32[$iFirst>>2] = 0;
 $24 = HEAP32[$pCsr>>2]|0;
 $25 = HEAP32[$pPhrase>>2]|0;
 $26 = ((($25)) + 4|0);
 HEAP32[$26>>2] = $24;
 _fts3GetDeltaPosition($pCsr,$iFirst);
 $27 = HEAP32[$pCsr>>2]|0;
 $28 = HEAP32[$pPhrase>>2]|0;
 $29 = ((($28)) + 12|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$pCsr>>2]|0;
 $31 = HEAP32[$pPhrase>>2]|0;
 $32 = ((($31)) + 20|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$iFirst>>2]|0;
 $34 = HEAP32[$pPhrase>>2]|0;
 $35 = ((($34)) + 8|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$iFirst>>2]|0;
 $37 = HEAP32[$pPhrase>>2]|0;
 $38 = ((($37)) + 16|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($39|0);
}
function _fts3SnippetNextCandidate($pIter) {
 $pIter = $pIter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $i = 0, $iEnd = 0, $iStart = 0, $pPhrase = 0, $pPhrase1 = 0, $pPhrase2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $i = sp + 20|0;
 $pPhrase = sp + 16|0;
 $iStart = sp + 12|0;
 $iEnd = sp + 8|0;
 $pPhrase1 = sp + 4|0;
 $pPhrase2 = sp;
 HEAP32[$1>>2] = $pIter;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<(0);
 L1: do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 20|0);
   HEAP32[$7>>2] = 0;
   HEAP32[$i>>2] = 0;
   while(1) {
    $8 = HEAP32[$i>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = ((($9)) + 12|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($8|0)<($11|0);
    if (!($12)) {
     break L1;
    }
    $13 = HEAP32[$i>>2]|0;
    $14 = HEAP32[$1>>2]|0;
    $15 = ((($14)) + 16|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = (($16) + (($13*24)|0)|0);
    HEAP32[$pPhrase>>2] = $17;
    $18 = HEAP32[$pPhrase>>2]|0;
    $19 = ((($18)) + 12|0);
    $20 = HEAP32[$pPhrase>>2]|0;
    $21 = ((($20)) + 8|0);
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 8|0);
    $24 = HEAP32[$23>>2]|0;
    _fts3SnippetAdvance($19,$21,$24);
    $25 = HEAP32[$i>>2]|0;
    $26 = (($25) + 1)|0;
    HEAP32[$i>>2] = $26;
   }
  } else {
   HEAP32[$iEnd>>2] = 2147483647;
   HEAP32[$i>>2] = 0;
   while(1) {
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 12|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($27|0)<($30|0);
    if (!($31)) {
     break;
    }
    $32 = HEAP32[$i>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = ((($33)) + 16|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = (($35) + (($32*24)|0)|0);
    HEAP32[$pPhrase1>>2] = $36;
    $37 = HEAP32[$pPhrase1>>2]|0;
    $38 = ((($37)) + 12|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)!=(0|0);
    if ($40) {
     $41 = HEAP32[$pPhrase1>>2]|0;
     $42 = ((($41)) + 8|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = HEAP32[$iEnd>>2]|0;
     $45 = ($43|0)<($44|0);
     if ($45) {
      $46 = HEAP32[$pPhrase1>>2]|0;
      $47 = ((($46)) + 8|0);
      $48 = HEAP32[$47>>2]|0;
      HEAP32[$iEnd>>2] = $48;
     }
    }
    $49 = HEAP32[$i>>2]|0;
    $50 = (($49) + 1)|0;
    HEAP32[$i>>2] = $50;
   }
   $51 = HEAP32[$iEnd>>2]|0;
   $52 = ($51|0)==(2147483647);
   if ($52) {
    HEAP32[$0>>2] = 1;
    $84 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($84|0);
   }
   $53 = HEAP32[$iEnd>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = (($53) - ($56))|0;
   $58 = (($57) + 1)|0;
   HEAP32[$iStart>>2] = $58;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 20|0);
   HEAP32[$60>>2] = $58;
   HEAP32[$i>>2] = 0;
   while(1) {
    $61 = HEAP32[$i>>2]|0;
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 12|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = ($61|0)<($64|0);
    if (!($65)) {
     break L1;
    }
    $66 = HEAP32[$i>>2]|0;
    $67 = HEAP32[$1>>2]|0;
    $68 = ((($67)) + 16|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + (($66*24)|0)|0);
    HEAP32[$pPhrase2>>2] = $70;
    $71 = HEAP32[$pPhrase2>>2]|0;
    $72 = ((($71)) + 12|0);
    $73 = HEAP32[$pPhrase2>>2]|0;
    $74 = ((($73)) + 8|0);
    $75 = HEAP32[$iEnd>>2]|0;
    $76 = (($75) + 1)|0;
    _fts3SnippetAdvance($72,$74,$76);
    $77 = HEAP32[$pPhrase2>>2]|0;
    $78 = ((($77)) + 20|0);
    $79 = HEAP32[$pPhrase2>>2]|0;
    $80 = ((($79)) + 16|0);
    $81 = HEAP32[$iStart>>2]|0;
    _fts3SnippetAdvance($78,$80,$81);
    $82 = HEAP32[$i>>2]|0;
    $83 = (($82) + 1)|0;
    HEAP32[$i>>2] = $83;
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $84 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($84|0);
}
function _fts3SnippetDetails($pIter,$0,$1,$piToken,$piScore,$pmCover,$pmHighlight) {
 $pIter = $pIter|0;
 $0 = $0|0;
 $1 = $1|0;
 $piToken = $piToken|0;
 $piScore = $piScore|0;
 $pmCover = $pmCover|0;
 $pmHighlight = $pmHighlight|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCsr = 0, $iScore = 0, $iStart = 0, $j = 0, $mCover = 0, $mHighlight = 0, $mPhrase = 0, $mPos = 0, $pCsr = 0;
 var $pPhrase = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $2 = sp + 84|0;
 $3 = sp + 32|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $7 = sp + 68|0;
 $iStart = sp + 64|0;
 $iScore = sp + 60|0;
 $i = sp + 56|0;
 $mCover = sp + 24|0;
 $mHighlight = sp + 16|0;
 $pPhrase = sp + 52|0;
 $pCsr = sp + 48|0;
 $iCsr = sp + 44|0;
 $j = sp + 40|0;
 $mPhrase = sp + 8|0;
 $mPos = sp;
 HEAP32[$2>>2] = $pIter;
 $8 = $3;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$4>>2] = $piToken;
 HEAP32[$5>>2] = $piScore;
 HEAP32[$6>>2] = $pmCover;
 HEAP32[$7>>2] = $pmHighlight;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 20|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$iStart>>2] = $14;
 HEAP32[$iScore>>2] = 0;
 $15 = $mCover;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = 0;
 $19 = $mHighlight;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + (($28*24)|0)|0);
  HEAP32[$pPhrase>>2] = $32;
  $33 = HEAP32[$pPhrase>>2]|0;
  $34 = ((($33)) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0|0);
  L4: do {
   if ($36) {
    $37 = HEAP32[$pPhrase>>2]|0;
    $38 = ((($37)) + 20|0);
    $39 = HEAP32[$38>>2]|0;
    HEAP32[$pCsr>>2] = $39;
    $40 = HEAP32[$pPhrase>>2]|0;
    $41 = ((($40)) + 16|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$iCsr>>2] = $42;
    while(1) {
     $43 = HEAP32[$iCsr>>2]|0;
     $44 = HEAP32[$iStart>>2]|0;
     $45 = HEAP32[$2>>2]|0;
     $46 = ((($45)) + 8|0);
     $47 = HEAP32[$46>>2]|0;
     $48 = (($44) + ($47))|0;
     $49 = ($43|0)<($48|0);
     if (!($49)) {
      break L4;
     }
     $50 = HEAP32[$i>>2]|0;
     $51 = (_bitshift64Shl(1,0,($50|0))|0);
     $52 = tempRet0;
     $53 = $mPhrase;
     $54 = $53;
     HEAP32[$54>>2] = $51;
     $55 = (($53) + 4)|0;
     $56 = $55;
     HEAP32[$56>>2] = $52;
     $57 = HEAP32[$iCsr>>2]|0;
     $58 = HEAP32[$iStart>>2]|0;
     $59 = (($57) - ($58))|0;
     $60 = (_bitshift64Shl(1,0,($59|0))|0);
     $61 = tempRet0;
     $62 = $mPos;
     $63 = $62;
     HEAP32[$63>>2] = $60;
     $64 = (($62) + 4)|0;
     $65 = $64;
     HEAP32[$65>>2] = $61;
     $66 = $mCover;
     $67 = $66;
     $68 = HEAP32[$67>>2]|0;
     $69 = (($66) + 4)|0;
     $70 = $69;
     $71 = HEAP32[$70>>2]|0;
     $72 = $3;
     $73 = $72;
     $74 = HEAP32[$73>>2]|0;
     $75 = (($72) + 4)|0;
     $76 = $75;
     $77 = HEAP32[$76>>2]|0;
     $78 = $68 | $74;
     $79 = $71 | $77;
     $80 = $mPhrase;
     $81 = $80;
     $82 = HEAP32[$81>>2]|0;
     $83 = (($80) + 4)|0;
     $84 = $83;
     $85 = HEAP32[$84>>2]|0;
     $86 = $78 & $82;
     $87 = $79 & $85;
     $88 = ($86|0)!=(0);
     $89 = ($87|0)!=(0);
     $90 = $88 | $89;
     $91 = HEAP32[$iScore>>2]|0;
     if ($90) {
      $92 = (($91) + 1)|0;
      HEAP32[$iScore>>2] = $92;
     } else {
      $93 = (($91) + 1000)|0;
      HEAP32[$iScore>>2] = $93;
     }
     $94 = $mPhrase;
     $95 = $94;
     $96 = HEAP32[$95>>2]|0;
     $97 = (($94) + 4)|0;
     $98 = $97;
     $99 = HEAP32[$98>>2]|0;
     $100 = $mCover;
     $101 = $100;
     $102 = HEAP32[$101>>2]|0;
     $103 = (($100) + 4)|0;
     $104 = $103;
     $105 = HEAP32[$104>>2]|0;
     $106 = $102 | $96;
     $107 = $105 | $99;
     $108 = $mCover;
     $109 = $108;
     HEAP32[$109>>2] = $106;
     $110 = (($108) + 4)|0;
     $111 = $110;
     HEAP32[$111>>2] = $107;
     HEAP32[$j>>2] = 0;
     while(1) {
      $112 = HEAP32[$j>>2]|0;
      $113 = HEAP32[$pPhrase>>2]|0;
      $114 = HEAP32[$113>>2]|0;
      $115 = ($112|0)<($114|0);
      if (!($115)) {
       break;
      }
      $116 = $mPos;
      $117 = $116;
      $118 = HEAP32[$117>>2]|0;
      $119 = (($116) + 4)|0;
      $120 = $119;
      $121 = HEAP32[$120>>2]|0;
      $122 = HEAP32[$j>>2]|0;
      $123 = (_bitshift64Lshr(($118|0),($121|0),($122|0))|0);
      $124 = tempRet0;
      $125 = $mHighlight;
      $126 = $125;
      $127 = HEAP32[$126>>2]|0;
      $128 = (($125) + 4)|0;
      $129 = $128;
      $130 = HEAP32[$129>>2]|0;
      $131 = $127 | $123;
      $132 = $130 | $124;
      $133 = $mHighlight;
      $134 = $133;
      HEAP32[$134>>2] = $131;
      $135 = (($133) + 4)|0;
      $136 = $135;
      HEAP32[$136>>2] = $132;
      $137 = HEAP32[$j>>2]|0;
      $138 = (($137) + 1)|0;
      HEAP32[$j>>2] = $138;
     }
     $139 = HEAP32[$pCsr>>2]|0;
     $140 = HEAP8[$139>>0]|0;
     $141 = $140 << 24 >> 24;
     $142 = $141 & 254;
     $143 = (0)==($142|0);
     if ($143) {
      break L4;
     }
     _fts3GetDeltaPosition($pCsr,$iCsr);
    }
   }
  } while(0);
  $144 = HEAP32[$i>>2]|0;
  $145 = (($144) + 1)|0;
  HEAP32[$i>>2] = $145;
 }
 $146 = HEAP32[$iStart>>2]|0;
 $147 = HEAP32[$4>>2]|0;
 HEAP32[$147>>2] = $146;
 $148 = HEAP32[$iScore>>2]|0;
 $149 = HEAP32[$5>>2]|0;
 HEAP32[$149>>2] = $148;
 $150 = $mCover;
 $151 = $150;
 $152 = HEAP32[$151>>2]|0;
 $153 = (($150) + 4)|0;
 $154 = $153;
 $155 = HEAP32[$154>>2]|0;
 $156 = HEAP32[$6>>2]|0;
 $157 = $156;
 $158 = $157;
 HEAP32[$158>>2] = $152;
 $159 = (($157) + 4)|0;
 $160 = $159;
 HEAP32[$160>>2] = $155;
 $161 = $mHighlight;
 $162 = $161;
 $163 = HEAP32[$162>>2]|0;
 $164 = (($161) + 4)|0;
 $165 = $164;
 $166 = HEAP32[$165>>2]|0;
 $167 = HEAP32[$7>>2]|0;
 $168 = $167;
 $169 = $168;
 HEAP32[$169>>2] = $163;
 $170 = (($168) + 4)|0;
 $171 = $170;
 HEAP32[$171>>2] = $166;
 STACKTOP = sp;return;
}
function _fts3SnippetAdvance($ppIter,$piIter,$iNext) {
 $ppIter = $ppIter|0;
 $piIter = $piIter|0;
 $iNext = $iNext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iIter = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pIter = sp + 4|0;
 $iIter = sp;
 HEAP32[$0>>2] = $ppIter;
 HEAP32[$1>>2] = $piIter;
 HEAP32[$2>>2] = $iNext;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIter>>2] = $4;
 $5 = HEAP32[$pIter>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$iIter>>2] = $8;
 while(1) {
  $9 = HEAP32[$iIter>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pIter>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13 << 24 >> 24;
  $15 = $14 & 254;
  $16 = (0)==($15|0);
  if ($16) {
   label = 5;
   break;
  }
  _fts3GetDeltaPosition($pIter,$iIter);
 }
 if ((label|0) == 5) {
  HEAP32[$iIter>>2] = -1;
  HEAP32[$pIter>>2] = 0;
 }
 $17 = HEAP32[$iIter>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$pIter>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 HEAP32[$20>>2] = $19;
 STACKTOP = sp;return;
}
function _sqlite3Fts3MaxLevel($p,$pnMax) {
 $p = $p|0;
 $pnMax = $pnMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mxLevel = 0, $pStmt = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $mxLevel = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnMax;
 HEAP32[$mxLevel>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,36,$pStmt,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $13 = HEAP32[$mxLevel>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$14>>2] = $13;
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 $6 = HEAP32[$pStmt>>2]|0;
 $7 = (_sqlite3_step($6)|0);
 $8 = (100)==($7|0);
 if ($8) {
  $9 = HEAP32[$pStmt>>2]|0;
  $10 = (_sqlite3_column_int($9,0)|0);
  HEAP32[$mxLevel>>2] = $10;
 }
 $11 = HEAP32[$pStmt>>2]|0;
 $12 = (_sqlite3_reset($11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$mxLevel>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3Fts3Incrmerge($p,$nMerge,$nMin) {
 $p = $p|0;
 $nMerge = $nMerge|0;
 $nMin = $nMin|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDirtyHint = 0, $bIgnore = 0, $bUseHint = 0;
 var $hint = 0, $iAbsLevel = 0, $iHintAbsLevel = 0, $iIdx = 0, $nAlloc = 0, $nHint = 0, $nHintSeg = 0, $nKey = 0, $nMod = 0, $nRem = 0, $nSeg = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pCsr = 0, $pFilter = 0, $pFindLevel = 0, $pWriter = 0, $rc = 0;
 var $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $rc = sp + 96|0;
 $nRem = sp + 92|0;
 $pCsr = sp + 88|0;
 $pFilter = sp + 84|0;
 $pWriter = sp + 80|0;
 $nSeg = sp + 76|0;
 $iAbsLevel = sp + 16|0;
 $hint = sp + 64|0;
 $bDirtyHint = sp + 60|0;
 $nAlloc = sp + 56|0;
 $nMod = sp + 8|0;
 $pFindLevel = sp + 52|0;
 $bUseHint = sp + 48|0;
 $iIdx = sp + 44|0;
 $nHint = sp + 40|0;
 $iHintAbsLevel = sp;
 $nHintSeg = sp + 36|0;
 $bIgnore = sp + 32|0;
 $zKey = sp + 28|0;
 $nKey = sp + 24|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nMerge;
 HEAP32[$3>>2] = $nMin;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$nRem>>2] = $4;
 HEAP32[$nSeg>>2] = 0;
 $5 = $iAbsLevel;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 ;HEAP32[$hint>>2]=0|0;HEAP32[$hint+4>>2]=0|0;HEAP32[$hint+8>>2]=0|0;
 HEAP32[$bDirtyHint>>2] = 0;
 HEAP32[$nAlloc>>2] = 640;
 $9 = (_sqlite3_malloc(640)|0);
 HEAP32[$pWriter>>2] = $9;
 $10 = HEAP32[$pWriter>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  HEAP32[$0>>2] = 7;
  $316 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($316|0);
 }
 $12 = HEAP32[$pWriter>>2]|0;
 $13 = ((($12)) + 568|0);
 HEAP32[$pFilter>>2] = $13;
 $14 = HEAP32[$pFilter>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP32[$pCsr>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = (_fts3IncrmergeHintLoad($16,$hint)|0);
 HEAP32[$rc>>2] = $17;
 while(1) {
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)==(0);
  $20 = HEAP32[$nRem>>2]|0;
  $21 = ($20|0)>(0);
  $22 = $19 ? $21 : 0;
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 248|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = $25<<10;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  $29 = $nMod;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
  HEAP32[$pFindLevel>>2] = 0;
  HEAP32[$bUseHint>>2] = 0;
  HEAP32[$iIdx>>2] = 0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (_fts3SqlStmt($33,28,$pFindLevel,0)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$pFindLevel>>2]|0;
  $36 = HEAP32[$3>>2]|0;
  $37 = (2)>($36|0);
  $38 = HEAP32[$3>>2]|0;
  $39 = $37 ? 2 : $38;
  (_sqlite3_bind_int($35,1,$39)|0);
  $40 = HEAP32[$pFindLevel>>2]|0;
  $41 = (_sqlite3_step($40)|0);
  $42 = ($41|0)==(100);
  if ($42) {
   $43 = HEAP32[$pFindLevel>>2]|0;
   $44 = (_sqlite3_column_int64($43,0)|0);
   $45 = tempRet0;
   $46 = $iAbsLevel;
   $47 = $46;
   HEAP32[$47>>2] = $44;
   $48 = (($46) + 4)|0;
   $49 = $48;
   HEAP32[$49>>2] = $45;
   $50 = HEAP32[$pFindLevel>>2]|0;
   $51 = (_sqlite3_column_int($50,1)|0);
   HEAP32[$nSeg>>2] = $51;
  } else {
   HEAP32[$nSeg>>2] = -1;
  }
  $52 = HEAP32[$pFindLevel>>2]|0;
  $53 = (_sqlite3_reset($52)|0);
  HEAP32[$rc>>2] = $53;
  $54 = HEAP32[$rc>>2]|0;
  $55 = ($54|0)==(0);
  do {
   if ($55) {
    $56 = ((($hint)) + 4|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0);
    if ($58) {
     $59 = ((($hint)) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     HEAP32[$nHint>>2] = $60;
     $61 = $iHintAbsLevel;
     $62 = $61;
     HEAP32[$62>>2] = 0;
     $63 = (($61) + 4)|0;
     $64 = $63;
     HEAP32[$64>>2] = 0;
     HEAP32[$nHintSeg>>2] = 0;
     $65 = (_fts3IncrmergeHintPop($hint,$iHintAbsLevel,$nHintSeg)|0);
     HEAP32[$rc>>2] = $65;
     $66 = HEAP32[$nSeg>>2]|0;
     $67 = ($66|0)<(0);
     if (!($67)) {
      $68 = $iAbsLevel;
      $69 = $68;
      $70 = HEAP32[$69>>2]|0;
      $71 = (($68) + 4)|0;
      $72 = $71;
      $73 = HEAP32[$72>>2]|0;
      $74 = $nMod;
      $75 = $74;
      $76 = HEAP32[$75>>2]|0;
      $77 = (($74) + 4)|0;
      $78 = $77;
      $79 = HEAP32[$78>>2]|0;
      $80 = (___remdi3(($70|0),($73|0),($76|0),($79|0))|0);
      $81 = tempRet0;
      $82 = $iHintAbsLevel;
      $83 = $82;
      $84 = HEAP32[$83>>2]|0;
      $85 = (($82) + 4)|0;
      $86 = $85;
      $87 = HEAP32[$86>>2]|0;
      $88 = $nMod;
      $89 = $88;
      $90 = HEAP32[$89>>2]|0;
      $91 = (($88) + 4)|0;
      $92 = $91;
      $93 = HEAP32[$92>>2]|0;
      $94 = (___remdi3(($84|0),($87|0),($90|0),($93|0))|0);
      $95 = tempRet0;
      $96 = ($81|0)>($95|0);
      $97 = ($80>>>0)>=($94>>>0);
      $98 = ($81|0)==($95|0);
      $99 = $98 & $97;
      $100 = $96 | $99;
      if (!($100)) {
       $112 = HEAP32[$nHint>>2]|0;
       $113 = ((($hint)) + 4|0);
       HEAP32[$113>>2] = $112;
       break;
      }
     }
     $101 = $iHintAbsLevel;
     $102 = $101;
     $103 = HEAP32[$102>>2]|0;
     $104 = (($101) + 4)|0;
     $105 = $104;
     $106 = HEAP32[$105>>2]|0;
     $107 = $iAbsLevel;
     $108 = $107;
     HEAP32[$108>>2] = $103;
     $109 = (($107) + 4)|0;
     $110 = $109;
     HEAP32[$110>>2] = $106;
     $111 = HEAP32[$nHintSeg>>2]|0;
     HEAP32[$nSeg>>2] = $111;
     HEAP32[$bUseHint>>2] = 1;
     HEAP32[$bDirtyHint>>2] = 1;
    }
   }
  } while(0);
  $114 = HEAP32[$nSeg>>2]|0;
  $115 = ($114|0)<(0);
  if ($115) {
   break;
  }
  $116 = HEAP32[$pWriter>>2]|0;
  _memset(($116|0),0,640)|0;
  $117 = HEAP32[$pFilter>>2]|0;
  $118 = ((($117)) + 12|0);
  HEAP32[$118>>2] = 1;
  $119 = HEAP32[$rc>>2]|0;
  $120 = ($119|0)==(0);
  do {
   if ($120) {
    $121 = HEAP32[$1>>2]|0;
    $122 = $iAbsLevel;
    $123 = $122;
    $124 = HEAP32[$123>>2]|0;
    $125 = (($122) + 4)|0;
    $126 = $125;
    $127 = HEAP32[$126>>2]|0;
    $128 = (_fts3IncrmergeOutputIdx($121,$124,$127,$iIdx)|0);
    HEAP32[$rc>>2] = $128;
    $129 = HEAP32[$iIdx>>2]|0;
    $130 = ($129|0)==(0);
    if (!($130)) {
     $131 = HEAP32[$bUseHint>>2]|0;
     $132 = ($131|0)!=(0);
     $133 = HEAP32[$iIdx>>2]|0;
     $134 = ($133|0)==(1);
     $or$cond = $132 & $134;
     if (!($or$cond)) {
      break;
     }
    }
    HEAP32[$bIgnore>>2] = 0;
    $135 = HEAP32[$1>>2]|0;
    $136 = $iAbsLevel;
    $137 = $136;
    $138 = HEAP32[$137>>2]|0;
    $139 = (($136) + 4)|0;
    $140 = $139;
    $141 = HEAP32[$140>>2]|0;
    $142 = (_i64Add(($138|0),($141|0),1,0)|0);
    $143 = tempRet0;
    $144 = (_fts3SegmentIsMaxLevel($135,$142,$143,$bIgnore)|0);
    HEAP32[$rc>>2] = $144;
    $145 = HEAP32[$bIgnore>>2]|0;
    $146 = ($145|0)!=(0);
    if ($146) {
     $147 = HEAP32[$pFilter>>2]|0;
     $148 = ((($147)) + 12|0);
     $149 = HEAP32[$148>>2]|0;
     $150 = $149 | 2;
     HEAP32[$148>>2] = $150;
    }
   }
  } while(0);
  $151 = HEAP32[$rc>>2]|0;
  $152 = ($151|0)==(0);
  if ($152) {
   $153 = HEAP32[$1>>2]|0;
   $154 = $iAbsLevel;
   $155 = $154;
   $156 = HEAP32[$155>>2]|0;
   $157 = (($154) + 4)|0;
   $158 = $157;
   $159 = HEAP32[$158>>2]|0;
   $160 = HEAP32[$nSeg>>2]|0;
   $161 = HEAP32[$pCsr>>2]|0;
   $162 = (_fts3IncrmergeCsr($153,$156,$159,$160,$161)|0);
   HEAP32[$rc>>2] = $162;
  }
  $163 = HEAP32[$rc>>2]|0;
  $164 = (0)==($163|0);
  if ($164) {
   $165 = HEAP32[$pCsr>>2]|0;
   $166 = ((($165)) + 4|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = HEAP32[$nSeg>>2]|0;
   $169 = ($167|0)==($168|0);
   if ($169) {
    $170 = HEAP32[$1>>2]|0;
    $171 = HEAP32[$pCsr>>2]|0;
    $172 = HEAP32[$pFilter>>2]|0;
    $173 = (_sqlite3Fts3SegReaderStart($170,$171,$172)|0);
    HEAP32[$rc>>2] = $173;
    $174 = (0)==($173|0);
    if ($174) {
     $175 = HEAP32[$1>>2]|0;
     $176 = HEAP32[$pCsr>>2]|0;
     $177 = (_sqlite3Fts3SegReaderStep($175,$176)|0);
     HEAP32[$rc>>2] = $177;
     $178 = (100)==($177|0);
     if ($178) {
      $179 = HEAP32[$bUseHint>>2]|0;
      $180 = ($179|0)!=(0);
      $181 = HEAP32[$iIdx>>2]|0;
      $182 = ($181|0)>(0);
      $or$cond3 = $180 & $182;
      if ($or$cond3) {
       $183 = HEAP32[$pCsr>>2]|0;
       $184 = ((($183)) + 40|0);
       $185 = HEAP32[$184>>2]|0;
       HEAP32[$zKey>>2] = $185;
       $186 = HEAP32[$pCsr>>2]|0;
       $187 = ((($186)) + 44|0);
       $188 = HEAP32[$187>>2]|0;
       HEAP32[$nKey>>2] = $188;
       $189 = HEAP32[$1>>2]|0;
       $190 = $iAbsLevel;
       $191 = $190;
       $192 = HEAP32[$191>>2]|0;
       $193 = (($190) + 4)|0;
       $194 = $193;
       $195 = HEAP32[$194>>2]|0;
       $196 = HEAP32[$iIdx>>2]|0;
       $197 = (($196) - 1)|0;
       $198 = HEAP32[$zKey>>2]|0;
       $199 = HEAP32[$nKey>>2]|0;
       $200 = HEAP32[$pWriter>>2]|0;
       $201 = (_fts3IncrmergeLoad($189,$192,$195,$197,$198,$199,$200)|0);
       HEAP32[$rc>>2] = $201;
      } else {
       $202 = HEAP32[$1>>2]|0;
       $203 = $iAbsLevel;
       $204 = $203;
       $205 = HEAP32[$204>>2]|0;
       $206 = (($203) + 4)|0;
       $207 = $206;
       $208 = HEAP32[$207>>2]|0;
       $209 = HEAP32[$iIdx>>2]|0;
       $210 = HEAP32[$pCsr>>2]|0;
       $211 = HEAP32[$pWriter>>2]|0;
       $212 = (_fts3IncrmergeWriter($202,$205,$208,$209,$210,$211)|0);
       HEAP32[$rc>>2] = $212;
      }
      $213 = HEAP32[$rc>>2]|0;
      $214 = ($213|0)==(0);
      if ($214) {
       $215 = HEAP32[$pWriter>>2]|0;
       $216 = HEAP32[$215>>2]|0;
       $217 = ($216|0)!=(0);
       if ($217) {
        while(1) {
         $218 = HEAP32[$1>>2]|0;
         $219 = HEAP32[$pWriter>>2]|0;
         $220 = HEAP32[$pCsr>>2]|0;
         $221 = (_fts3IncrmergeAppend($218,$219,$220)|0);
         HEAP32[$rc>>2] = $221;
         $222 = HEAP32[$rc>>2]|0;
         $223 = ($222|0)==(0);
         if ($223) {
          $224 = HEAP32[$1>>2]|0;
          $225 = HEAP32[$pCsr>>2]|0;
          $226 = (_sqlite3Fts3SegReaderStep($224,$225)|0);
          HEAP32[$rc>>2] = $226;
         }
         $227 = HEAP32[$pWriter>>2]|0;
         $228 = ((($227)) + 4|0);
         $229 = HEAP32[$228>>2]|0;
         $230 = HEAP32[$nRem>>2]|0;
         $231 = ($229|0)>=($230|0);
         $232 = HEAP32[$rc>>2]|0;
         $233 = ($232|0)==(100);
         $or$cond5 = $231 & $233;
         if ($or$cond5) {
          HEAP32[$rc>>2] = 0;
         }
         $234 = HEAP32[$rc>>2]|0;
         $235 = ($234|0)==(100);
         if (!($235)) {
          break;
         }
        }
        $236 = HEAP32[$rc>>2]|0;
        $237 = ($236|0)==(0);
        if ($237) {
         $238 = HEAP32[$pWriter>>2]|0;
         $239 = ((($238)) + 4|0);
         $240 = HEAP32[$239>>2]|0;
         $241 = (1 + ($240))|0;
         $242 = HEAP32[$nRem>>2]|0;
         $243 = (($242) - ($241))|0;
         HEAP32[$nRem>>2] = $243;
         $244 = HEAP32[$1>>2]|0;
         $245 = $iAbsLevel;
         $246 = $245;
         $247 = HEAP32[$246>>2]|0;
         $248 = (($245) + 4)|0;
         $249 = $248;
         $250 = HEAP32[$249>>2]|0;
         $251 = HEAP32[$pCsr>>2]|0;
         $252 = (_fts3IncrmergeChomp($244,$247,$250,$251,$nSeg)|0);
         HEAP32[$rc>>2] = $252;
         $253 = HEAP32[$nSeg>>2]|0;
         $254 = ($253|0)!=(0);
         if ($254) {
          HEAP32[$bDirtyHint>>2] = 1;
          $255 = $iAbsLevel;
          $256 = $255;
          $257 = HEAP32[$256>>2]|0;
          $258 = (($255) + 4)|0;
          $259 = $258;
          $260 = HEAP32[$259>>2]|0;
          $261 = HEAP32[$nSeg>>2]|0;
          _fts3IncrmergeHintPush($hint,$257,$260,$261,$rc);
         }
        }
       }
      }
      $262 = HEAP32[$nSeg>>2]|0;
      $263 = ($262|0)!=(0);
      if ($263) {
       $264 = HEAP32[$pWriter>>2]|0;
       $265 = ((($264)) + 40|0);
       $266 = $265;
       $267 = $266;
       $268 = HEAP32[$267>>2]|0;
       $269 = (($266) + 4)|0;
       $270 = $269;
       $271 = HEAP32[$270>>2]|0;
       $272 = (___muldi3(($268|0),($271|0),-1,-1)|0);
       $273 = tempRet0;
       $274 = HEAP32[$pWriter>>2]|0;
       $275 = ((($274)) + 40|0);
       $276 = $275;
       $277 = $276;
       HEAP32[$277>>2] = $272;
       $278 = (($276) + 4)|0;
       $279 = $278;
       HEAP32[$279>>2] = $273;
      }
      $280 = HEAP32[$1>>2]|0;
      $281 = HEAP32[$pWriter>>2]|0;
      _fts3IncrmergeRelease($280,$281,$rc);
      $282 = HEAP32[$nSeg>>2]|0;
      $283 = ($282|0)==(0);
      if ($283) {
       $284 = HEAP32[$pWriter>>2]|0;
       $285 = ((($284)) + 48|0);
       $286 = HEAP8[$285>>0]|0;
       $287 = $286&255;
       $288 = ($287|0)==(0);
       if ($288) {
        $289 = HEAP32[$1>>2]|0;
        $290 = $iAbsLevel;
        $291 = $290;
        $292 = HEAP32[$291>>2]|0;
        $293 = (($290) + 4)|0;
        $294 = $293;
        $295 = HEAP32[$294>>2]|0;
        $296 = (_i64Add(($292|0),($295|0),1,0)|0);
        $297 = tempRet0;
        $298 = HEAP32[$pWriter>>2]|0;
        $299 = ((($298)) + 40|0);
        $300 = $299;
        $301 = $300;
        $302 = HEAP32[$301>>2]|0;
        $303 = (($300) + 4)|0;
        $304 = $303;
        $305 = HEAP32[$304>>2]|0;
        (_fts3PromoteSegments($289,$296,$297,$302,$305)|0);
       }
      }
     }
    }
   }
  }
  $306 = HEAP32[$pCsr>>2]|0;
  _sqlite3Fts3SegReaderFinish($306);
 }
 $307 = HEAP32[$bDirtyHint>>2]|0;
 $308 = ($307|0)!=(0);
 $309 = HEAP32[$rc>>2]|0;
 $310 = ($309|0)==(0);
 $or$cond7 = $308 & $310;
 if ($or$cond7) {
  $311 = HEAP32[$1>>2]|0;
  $312 = (_fts3IncrmergeHintStore($311,$hint)|0);
  HEAP32[$rc>>2] = $312;
 }
 $313 = HEAP32[$pWriter>>2]|0;
 _sqlite3_free($313);
 $314 = HEAP32[$hint>>2]|0;
 _sqlite3_free($314);
 $315 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $315;
 $316 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($316|0);
}
function _fts3IncrmergeHintLoad($p,$pHint) {
 $p = $p|0;
 $pHint = $pHint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHint = 0, $nHint = 0, $pSelect = 0, $rc = 0, $rc2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pSelect = sp + 16|0;
 $rc = sp + 12|0;
 $rc2 = sp + 8|0;
 $aHint = sp + 4|0;
 $nHint = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pHint;
 HEAP32[$pSelect>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_fts3SqlStmt($4,22,$pSelect,0)|0);
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 $7 = ($6|0)==(0);
 if (!($7)) {
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$pSelect>>2]|0;
 (_sqlite3_bind_int($8,1,1)|0);
 $9 = HEAP32[$pSelect>>2]|0;
 $10 = (_sqlite3_step($9)|0);
 $11 = (100)==($10|0);
 if ($11) {
  $12 = HEAP32[$pSelect>>2]|0;
  $13 = (_sqlite3_column_blob($12,0)|0);
  HEAP32[$aHint>>2] = $13;
  $14 = HEAP32[$pSelect>>2]|0;
  $15 = (_sqlite3_column_bytes($14,0)|0);
  HEAP32[$nHint>>2] = $15;
  $16 = HEAP32[$aHint>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$nHint>>2]|0;
   _blobGrowBuffer($18,$19,$rc);
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)==(0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$aHint>>2]|0;
    $25 = HEAP32[$nHint>>2]|0;
    _memcpy(($23|0),($24|0),($25|0))|0;
    $26 = HEAP32[$nHint>>2]|0;
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 4|0);
    HEAP32[$28>>2] = $26;
   }
  }
 }
 $29 = HEAP32[$pSelect>>2]|0;
 $30 = (_sqlite3_reset($29)|0);
 HEAP32[$rc2>>2] = $30;
 $31 = HEAP32[$rc>>2]|0;
 $32 = ($31|0)==(0);
 if (!($32)) {
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $33 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $33;
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _fts3IncrmergeHintPop($pHint,$piAbsLevel,$pnInput) {
 $pHint = $pHint|0;
 $piAbsLevel = $piAbsLevel|0;
 $pnInput = $pnInput|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $nHint = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $nHint = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pHint;
 HEAP32[$2>>2] = $piAbsLevel;
 HEAP32[$3>>2] = $pnInput;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$nHint>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 2)|0;
 HEAP32[$i>>2] = $10;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)>(0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = (($13) - 1)|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($14)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = $19 & 128;
  $21 = ($20|0)!=(0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + -1)|0;
  HEAP32[$i>>2] = $23;
 }
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = ($24|0)>(0);
  if ($25) {
   $26 = HEAP32[$i>>2]|0;
   $27 = (($26) - 1)|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + ($27)|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31 << 24 >> 24;
   $33 = $32 & 128;
   $34 = ($33|0)!=(0);
   $71 = $34;
  } else {
   $71 = 0;
  }
  $35 = HEAP32[$i>>2]|0;
  if (!($71)) {
   break;
  }
  $36 = (($35) + -1)|0;
  HEAP32[$i>>2] = $36;
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 4|0);
 HEAP32[$38>>2] = $35;
 $39 = HEAP32[$i>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + ($39)|0);
 $43 = HEAP32[$2>>2]|0;
 $44 = (_sqlite3Fts3GetVarint($42,$43)|0);
 $45 = HEAP32[$i>>2]|0;
 $46 = (($45) + ($44))|0;
 HEAP32[$i>>2] = $46;
 $47 = HEAP32[$i>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + ($47)|0);
 $51 = HEAP8[$50>>0]|0;
 $52 = $51&255;
 $53 = $52 & 128;
 $54 = ($53|0)!=(0);
 $55 = HEAP32[$i>>2]|0;
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + ($55)|0);
 if ($54) {
  $59 = HEAP32[$3>>2]|0;
  $60 = (_sqlite3Fts3GetVarint32($58,$59)|0);
  $66 = $60;
 } else {
  $61 = HEAP8[$58>>0]|0;
  $62 = $61&255;
  $63 = HEAP32[$3>>2]|0;
  HEAP32[$63>>2] = $62;
  $66 = 1;
 }
 $64 = HEAP32[$i>>2]|0;
 $65 = (($64) + ($66))|0;
 HEAP32[$i>>2] = $65;
 $67 = HEAP32[$i>>2]|0;
 $68 = HEAP32[$nHint>>2]|0;
 $69 = ($67|0)!=($68|0);
 if ($69) {
  HEAP32[$0>>2] = 267;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 } else {
  HEAP32[$0>>2] = 0;
  $70 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 return (0)|0;
}
function _fts3IncrmergeOutputIdx($p,$0,$1,$piIdx) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $piIdx = $piIdx|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOutputIdx = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $4 = sp + 16|0;
 $rc = sp + 12|0;
 $pOutputIdx = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $piIdx;
 HEAP32[$pOutputIdx>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,8,$pOutputIdx,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $28 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $13 = HEAP32[$pOutputIdx>>2]|0;
 $14 = $3;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 (_sqlite3_bind_int64($13,1,$20,$21)|0);
 $22 = HEAP32[$pOutputIdx>>2]|0;
 (_sqlite3_step($22)|0);
 $23 = HEAP32[$pOutputIdx>>2]|0;
 $24 = (_sqlite3_column_int($23,0)|0);
 $25 = HEAP32[$4>>2]|0;
 HEAP32[$25>>2] = $24;
 $26 = HEAP32[$pOutputIdx>>2]|0;
 $27 = (_sqlite3_reset($26)|0);
 HEAP32[$rc>>2] = $27;
 $28 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _fts3SegmentIsMaxLevel($p,$0,$1,$pbMax) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pbMax = $pbMax|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp;
 $5 = sp + 16|0;
 $pStmt = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$3>>2] = $p;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $pbMax;
 $10 = HEAP32[$3>>2]|0;
 $11 = (_fts3SqlStmt($10,15,$pStmt,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $14;
  $48 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $15 = HEAP32[$pStmt>>2]|0;
 $16 = $4;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (_i64Add(($18|0),($21|0),1,0)|0);
 $23 = tempRet0;
 (_sqlite3_bind_int64($15,1,$22,$23)|0);
 $24 = HEAP32[$pStmt>>2]|0;
 $25 = $4;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (___divdi3(($27|0),($30|0),1024,0)|0);
 $32 = tempRet0;
 $33 = (_i64Add(($31|0),($32|0),1,0)|0);
 $34 = tempRet0;
 $35 = (___muldi3(($33|0),($34|0),1024,0)|0);
 $36 = tempRet0;
 (_sqlite3_bind_int64($24,2,$35,$36)|0);
 $37 = HEAP32[$5>>2]|0;
 HEAP32[$37>>2] = 0;
 $38 = HEAP32[$pStmt>>2]|0;
 $39 = (_sqlite3_step($38)|0);
 $40 = (100)==($39|0);
 if ($40) {
  $41 = HEAP32[$pStmt>>2]|0;
  $42 = (_sqlite3_column_type($41,0)|0);
  $43 = ($42|0)==(5);
  $44 = $43&1;
  $45 = HEAP32[$5>>2]|0;
  HEAP32[$45>>2] = $44;
 }
 $46 = HEAP32[$pStmt>>2]|0;
 $47 = (_sqlite3_reset($46)|0);
 HEAP32[$2>>2] = $47;
 $48 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _fts3IncrmergeCsr($p,$0,$1,$nSeg,$pCsr) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $nSeg = $nSeg|0;
 $pCsr = $pCsr|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $nByte = 0, $pStmt = 0, $rc = 0, $rc2 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $rc = sp + 24|0;
 $pStmt = sp + 20|0;
 $nByte = sp + 16|0;
 $i = sp + 12|0;
 $rc2 = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $nSeg;
 HEAP32[$5>>2] = $pCsr;
 HEAP32[$pStmt>>2] = 0;
 $10 = HEAP32[$5>>2]|0;
 dest=$10; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $11 = HEAP32[$4>>2]|0;
 $12 = $11<<2;
 HEAP32[$nByte>>2] = $12;
 $13 = HEAP32[$nByte>>2]|0;
 $14 = (_sqlite3_malloc($13)|0);
 $15 = HEAP32[$5>>2]|0;
 HEAP32[$15>>2] = $14;
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$rc>>2] = 7;
 } else {
  $19 = HEAP32[$5>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$nByte>>2]|0;
  _memset(($20|0),0,($21|0))|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = (_fts3SqlStmt($22,12,$pStmt,0)|0);
  HEAP32[$rc>>2] = $23;
 }
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(0);
 if (!($25)) {
  $71 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $26 = HEAP32[$pStmt>>2]|0;
 $27 = $3;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 (_sqlite3_bind_int64($26,1,$29,$32)|0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)==(0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$pStmt>>2]|0;
  $36 = (_sqlite3_step($35)|0);
  $37 = ($36|0)==(100);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$4>>2]|0;
  $40 = ($38|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$pStmt>>2]|0;
  $43 = (_sqlite3_column_int64($42,1)|0);
  $44 = tempRet0;
  $45 = HEAP32[$pStmt>>2]|0;
  $46 = (_sqlite3_column_int64($45,2)|0);
  $47 = tempRet0;
  $48 = HEAP32[$pStmt>>2]|0;
  $49 = (_sqlite3_column_int64($48,3)|0);
  $50 = tempRet0;
  $51 = HEAP32[$pStmt>>2]|0;
  $52 = (_sqlite3_column_blob($51,4)|0);
  $53 = HEAP32[$pStmt>>2]|0;
  $54 = (_sqlite3_column_bytes($53,4)|0);
  $55 = HEAP32[$i>>2]|0;
  $56 = HEAP32[$5>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + ($55<<2)|0);
  $59 = (_sqlite3Fts3SegReaderNew($41,0,$43,$44,$46,$47,$49,$50,$52,$54,$58)|0);
  HEAP32[$rc>>2] = $59;
  $60 = HEAP32[$5>>2]|0;
  $61 = ((($60)) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = (($62) + 1)|0;
  HEAP32[$61>>2] = $63;
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 $66 = HEAP32[$pStmt>>2]|0;
 $67 = (_sqlite3_reset($66)|0);
 HEAP32[$rc2>>2] = $67;
 $68 = HEAP32[$rc>>2]|0;
 $69 = ($68|0)==(0);
 if (!($69)) {
  $71 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $70 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $70;
 $71 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($71|0);
}
function _fts3IncrmergeLoad($p,$0,$1,$iIdx,$zKey,$nKey,$pWriter) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $zKey = $zKey|0;
 $nKey = $nKey|0;
 $pWriter = $pWriter|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBlock = 0, $aLeaf = 0, $aRoot = 0, $bAppendable = 0, $i = 0, $iEnd = 0, $iLeafEnd = 0, $iStart = 0;
 var $nBlock = 0, $nHeight = 0, $nLeaf = 0, $nRoot = 0, $or$cond = 0, $or$cond3 = 0, $pNode = 0, $pSelect = 0, $rc = 0, $rc2 = 0, $reader = 0, $reader1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $2 = sp + 200|0;
 $3 = sp + 196|0;
 $4 = sp + 120|0;
 $5 = sp + 192|0;
 $6 = sp + 188|0;
 $7 = sp + 184|0;
 $8 = sp + 180|0;
 $rc = sp + 176|0;
 $pSelect = sp + 172|0;
 $iStart = sp + 112|0;
 $iLeafEnd = sp + 104|0;
 $iEnd = sp + 96|0;
 $aRoot = sp + 168|0;
 $nRoot = sp + 164|0;
 $rc2 = sp + 160|0;
 $bAppendable = sp + 156|0;
 $aLeaf = sp + 152|0;
 $nLeaf = sp + 148|0;
 $reader = sp + 48|0;
 $i = sp + 144|0;
 $nHeight = sp + 140|0;
 $pNode = sp + 136|0;
 $reader1 = sp;
 $aBlock = sp + 132|0;
 $nBlock = sp + 128|0;
 HEAP32[$3>>2] = $p;
 $9 = $4;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$5>>2] = $iIdx;
 HEAP32[$6>>2] = $zKey;
 HEAP32[$7>>2] = $nKey;
 HEAP32[$8>>2] = $pWriter;
 HEAP32[$pSelect>>2] = 0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_fts3SqlStmt($13,32,$pSelect,0)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = $iStart;
  $18 = $17;
  HEAP32[$18>>2] = 0;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = 0;
  $21 = $iLeafEnd;
  $22 = $21;
  HEAP32[$22>>2] = 0;
  $23 = (($21) + 4)|0;
  $24 = $23;
  HEAP32[$24>>2] = 0;
  $25 = $iEnd;
  $26 = $25;
  HEAP32[$26>>2] = 0;
  $27 = (($25) + 4)|0;
  $28 = $27;
  HEAP32[$28>>2] = 0;
  HEAP32[$aRoot>>2] = 0;
  HEAP32[$nRoot>>2] = 0;
  HEAP32[$bAppendable>>2] = 0;
  $29 = HEAP32[$pSelect>>2]|0;
  $30 = $4;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (_i64Add(($32|0),($35|0),1,0)|0);
  $37 = tempRet0;
  (_sqlite3_bind_int64($29,1,$36,$37)|0);
  $38 = HEAP32[$pSelect>>2]|0;
  $39 = HEAP32[$5>>2]|0;
  (_sqlite3_bind_int($38,2,$39)|0);
  $40 = HEAP32[$pSelect>>2]|0;
  $41 = (_sqlite3_step($40)|0);
  $42 = ($41|0)==(100);
  $43 = HEAP32[$pSelect>>2]|0;
  if (!($42)) {
   $116 = (_sqlite3_reset($43)|0);
   HEAP32[$2>>2] = $116;
   $393 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($393|0);
  }
  $44 = (_sqlite3_column_int64($43,1)|0);
  $45 = tempRet0;
  $46 = $iStart;
  $47 = $46;
  HEAP32[$47>>2] = $44;
  $48 = (($46) + 4)|0;
  $49 = $48;
  HEAP32[$49>>2] = $45;
  $50 = HEAP32[$pSelect>>2]|0;
  $51 = (_sqlite3_column_int64($50,2)|0);
  $52 = tempRet0;
  $53 = $iLeafEnd;
  $54 = $53;
  HEAP32[$54>>2] = $51;
  $55 = (($53) + 4)|0;
  $56 = $55;
  HEAP32[$56>>2] = $52;
  $57 = HEAP32[$pSelect>>2]|0;
  $58 = HEAP32[$8>>2]|0;
  $59 = ((($58)) + 40|0);
  _fts3ReadEndBlockField($57,3,$iEnd,$59);
  $60 = HEAP32[$8>>2]|0;
  $61 = ((($60)) + 40|0);
  $62 = $61;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)<(0);
  if ($68) {
   $69 = HEAP32[$8>>2]|0;
   $70 = ((($69)) + 40|0);
   $71 = $70;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = (($71) + 4)|0;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = (___muldi3(($73|0),($76|0),-1,-1)|0);
   $78 = tempRet0;
   $79 = HEAP32[$8>>2]|0;
   $80 = ((($79)) + 40|0);
   $81 = $80;
   $82 = $81;
   HEAP32[$82>>2] = $77;
   $83 = (($81) + 4)|0;
   $84 = $83;
   HEAP32[$84>>2] = $78;
  }
  $85 = HEAP32[$8>>2]|0;
  $86 = ((($85)) + 40|0);
  $87 = $86;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  $93 = ($89|0)==(0);
  $94 = ($92|0)==(0);
  $95 = $93 & $94;
  $96 = $95&1;
  $97 = $96&255;
  $98 = HEAP32[$8>>2]|0;
  $99 = ((($98)) + 48|0);
  HEAP8[$99>>0] = $97;
  $100 = HEAP32[$pSelect>>2]|0;
  $101 = (_sqlite3_column_bytes($100,4)|0);
  HEAP32[$nRoot>>2] = $101;
  $102 = HEAP32[$pSelect>>2]|0;
  $103 = (_sqlite3_column_blob($102,4)|0);
  HEAP32[$aRoot>>2] = $103;
  $104 = HEAP32[$3>>2]|0;
  $105 = $iEnd;
  $106 = $105;
  $107 = HEAP32[$106>>2]|0;
  $108 = (($105) + 4)|0;
  $109 = $108;
  $110 = HEAP32[$109>>2]|0;
  $111 = (_fts3IsAppendable($104,$107,$110,$bAppendable)|0);
  HEAP32[$rc>>2] = $111;
  $112 = HEAP32[$rc>>2]|0;
  $113 = ($112|0)==(0);
  $114 = HEAP32[$bAppendable>>2]|0;
  $115 = ($114|0)!=(0);
  $or$cond = $113 & $115;
  if ($or$cond) {
   HEAP32[$aLeaf>>2] = 0;
   HEAP32[$nLeaf>>2] = 0;
   $117 = HEAP32[$3>>2]|0;
   $118 = $iLeafEnd;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   $124 = (_sqlite3Fts3ReadBlock($117,$120,$123,$aLeaf,$nLeaf,0)|0);
   HEAP32[$rc>>2] = $124;
   $125 = HEAP32[$rc>>2]|0;
   $126 = ($125|0)==(0);
   if ($126) {
    $127 = HEAP32[$aLeaf>>2]|0;
    $128 = HEAP32[$nLeaf>>2]|0;
    $129 = (_nodeReaderInit($reader,$127,$128)|0);
    HEAP32[$rc>>2] = $129;
    while(1) {
     $130 = HEAP32[$rc>>2]|0;
     $131 = ($130|0)==(0);
     $132 = HEAP32[$reader>>2]|0;
     $133 = ($132|0)!=(0|0);
     $134 = $131 ? $133 : 0;
     if (!($134)) {
      break;
     }
     $135 = (_nodeReaderNext($reader)|0);
     HEAP32[$rc>>2] = $135;
    }
    $136 = HEAP32[$6>>2]|0;
    $137 = HEAP32[$7>>2]|0;
    $138 = ((($reader)) + 24|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = ((($reader)) + 24|0);
    $141 = ((($140)) + 4|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = (_fts3TermCmp($136,$137,$139,$142)|0);
    $144 = ($143|0)<=(0);
    if ($144) {
     HEAP32[$bAppendable>>2] = 0;
    }
    _nodeReaderRelease($reader);
   }
   $145 = HEAP32[$aLeaf>>2]|0;
   _sqlite3_free($145);
  }
  $146 = HEAP32[$rc>>2]|0;
  $147 = ($146|0)==(0);
  $148 = HEAP32[$bAppendable>>2]|0;
  $149 = ($148|0)!=(0);
  $or$cond3 = $147 & $149;
  L23: do {
   if ($or$cond3) {
    $150 = HEAP32[$aRoot>>2]|0;
    $151 = HEAP8[$150>>0]|0;
    $152 = $151 << 24 >> 24;
    HEAP32[$nHeight>>2] = $152;
    $153 = $iEnd;
    $154 = $153;
    $155 = HEAP32[$154>>2]|0;
    $156 = (($153) + 4)|0;
    $157 = $156;
    $158 = HEAP32[$157>>2]|0;
    $159 = $iStart;
    $160 = $159;
    $161 = HEAP32[$160>>2]|0;
    $162 = (($159) + 4)|0;
    $163 = $162;
    $164 = HEAP32[$163>>2]|0;
    $165 = (_i64Subtract(($155|0),($158|0),($161|0),($164|0))|0);
    $166 = tempRet0;
    $167 = (_i64Add(($165|0),($166|0),1,0)|0);
    $168 = tempRet0;
    $169 = (($167|0) / 16)&-1;
    $170 = HEAP32[$8>>2]|0;
    HEAP32[$170>>2] = $169;
    $171 = $iStart;
    $172 = $171;
    $173 = HEAP32[$172>>2]|0;
    $174 = (($171) + 4)|0;
    $175 = $174;
    $176 = HEAP32[$175>>2]|0;
    $177 = HEAP32[$8>>2]|0;
    $178 = ((($177)) + 24|0);
    $179 = $178;
    $180 = $179;
    HEAP32[$180>>2] = $173;
    $181 = (($179) + 4)|0;
    $182 = $181;
    HEAP32[$182>>2] = $176;
    $183 = $iEnd;
    $184 = $183;
    $185 = HEAP32[$184>>2]|0;
    $186 = (($183) + 4)|0;
    $187 = $186;
    $188 = HEAP32[$187>>2]|0;
    $189 = HEAP32[$8>>2]|0;
    $190 = ((($189)) + 32|0);
    $191 = $190;
    $192 = $191;
    HEAP32[$192>>2] = $185;
    $193 = (($191) + 4)|0;
    $194 = $193;
    HEAP32[$194>>2] = $188;
    $195 = $4;
    $196 = $195;
    $197 = HEAP32[$196>>2]|0;
    $198 = (($195) + 4)|0;
    $199 = $198;
    $200 = HEAP32[$199>>2]|0;
    $201 = HEAP32[$8>>2]|0;
    $202 = ((($201)) + 8|0);
    $203 = $202;
    $204 = $203;
    HEAP32[$204>>2] = $197;
    $205 = (($203) + 4)|0;
    $206 = $205;
    HEAP32[$206>>2] = $200;
    $207 = HEAP32[$5>>2]|0;
    $208 = HEAP32[$8>>2]|0;
    $209 = ((($208)) + 16|0);
    HEAP32[$209>>2] = $207;
    $210 = HEAP32[$nHeight>>2]|0;
    $211 = (($210) + 1)|0;
    HEAP32[$i>>2] = $211;
    while(1) {
     $212 = HEAP32[$i>>2]|0;
     $213 = ($212|0)<(16);
     if (!($213)) {
      break;
     }
     $214 = HEAP32[$8>>2]|0;
     $215 = ((($214)) + 24|0);
     $216 = $215;
     $217 = $216;
     $218 = HEAP32[$217>>2]|0;
     $219 = (($216) + 4)|0;
     $220 = $219;
     $221 = HEAP32[$220>>2]|0;
     $222 = HEAP32[$i>>2]|0;
     $223 = HEAP32[$8>>2]|0;
     $224 = HEAP32[$223>>2]|0;
     $225 = Math_imul($222, $224)|0;
     $226 = ($225|0)<(0);
     $227 = $226 << 31 >> 31;
     $228 = (_i64Add(($218|0),($221|0),($225|0),($227|0))|0);
     $229 = tempRet0;
     $230 = HEAP32[$i>>2]|0;
     $231 = HEAP32[$8>>2]|0;
     $232 = ((($231)) + 56|0);
     $233 = (($232) + ($230<<5)|0);
     $234 = $233;
     $235 = $234;
     HEAP32[$235>>2] = $228;
     $236 = (($234) + 4)|0;
     $237 = $236;
     HEAP32[$237>>2] = $229;
     $238 = HEAP32[$i>>2]|0;
     $239 = (($238) + 1)|0;
     HEAP32[$i>>2] = $239;
    }
    $240 = HEAP32[$nHeight>>2]|0;
    $241 = HEAP32[$8>>2]|0;
    $242 = ((($241)) + 56|0);
    $243 = (($242) + ($240<<5)|0);
    HEAP32[$pNode>>2] = $243;
    $244 = HEAP32[$8>>2]|0;
    $245 = ((($244)) + 24|0);
    $246 = $245;
    $247 = $246;
    $248 = HEAP32[$247>>2]|0;
    $249 = (($246) + 4)|0;
    $250 = $249;
    $251 = HEAP32[$250>>2]|0;
    $252 = HEAP32[$8>>2]|0;
    $253 = HEAP32[$252>>2]|0;
    $254 = HEAP32[$nHeight>>2]|0;
    $255 = Math_imul($253, $254)|0;
    $256 = ($255|0)<(0);
    $257 = $256 << 31 >> 31;
    $258 = (_i64Add(($248|0),($251|0),($255|0),($257|0))|0);
    $259 = tempRet0;
    $260 = HEAP32[$pNode>>2]|0;
    $261 = $260;
    $262 = $261;
    HEAP32[$262>>2] = $258;
    $263 = (($261) + 4)|0;
    $264 = $263;
    HEAP32[$264>>2] = $259;
    $265 = HEAP32[$pNode>>2]|0;
    $266 = ((($265)) + 20|0);
    $267 = HEAP32[$nRoot>>2]|0;
    $268 = HEAP32[$3>>2]|0;
    $269 = ((($268)) + 224|0);
    $270 = HEAP32[$269>>2]|0;
    $271 = ($267|0)>($270|0);
    if ($271) {
     $272 = HEAP32[$nRoot>>2]|0;
     $276 = $272;
    } else {
     $273 = HEAP32[$3>>2]|0;
     $274 = ((($273)) + 224|0);
     $275 = HEAP32[$274>>2]|0;
     $276 = $275;
    }
    _blobGrowBuffer($266,$276,$rc);
    $277 = HEAP32[$rc>>2]|0;
    $278 = ($277|0)==(0);
    if ($278) {
     $279 = HEAP32[$pNode>>2]|0;
     $280 = ((($279)) + 20|0);
     $281 = HEAP32[$280>>2]|0;
     $282 = HEAP32[$aRoot>>2]|0;
     $283 = HEAP32[$nRoot>>2]|0;
     _memcpy(($281|0),($282|0),($283|0))|0;
     $284 = HEAP32[$nRoot>>2]|0;
     $285 = HEAP32[$pNode>>2]|0;
     $286 = ((($285)) + 20|0);
     $287 = ((($286)) + 4|0);
     HEAP32[$287>>2] = $284;
    }
    $288 = HEAP32[$nHeight>>2]|0;
    HEAP32[$i>>2] = $288;
    while(1) {
     $289 = HEAP32[$i>>2]|0;
     $290 = ($289|0)>=(0);
     $291 = HEAP32[$rc>>2]|0;
     $292 = ($291|0)==(0);
     $293 = $290 ? $292 : 0;
     if (!($293)) {
      break L23;
     }
     $294 = HEAP32[$i>>2]|0;
     $295 = HEAP32[$8>>2]|0;
     $296 = ((($295)) + 56|0);
     $297 = (($296) + ($294<<5)|0);
     HEAP32[$pNode>>2] = $297;
     $298 = HEAP32[$pNode>>2]|0;
     $299 = ((($298)) + 20|0);
     $300 = HEAP32[$299>>2]|0;
     $301 = HEAP32[$pNode>>2]|0;
     $302 = ((($301)) + 20|0);
     $303 = ((($302)) + 4|0);
     $304 = HEAP32[$303>>2]|0;
     $305 = (_nodeReaderInit($reader1,$300,$304)|0);
     HEAP32[$rc>>2] = $305;
     while(1) {
      $306 = HEAP32[$reader1>>2]|0;
      $307 = ($306|0)!=(0|0);
      $308 = HEAP32[$rc>>2]|0;
      $309 = ($308|0)==(0);
      $310 = $307 ? $309 : 0;
      if (!($310)) {
       break;
      }
      $311 = (_nodeReaderNext($reader1)|0);
      HEAP32[$rc>>2] = $311;
     }
     $312 = HEAP32[$pNode>>2]|0;
     $313 = ((($312)) + 8|0);
     $314 = ((($reader1)) + 24|0);
     $315 = ((($314)) + 4|0);
     $316 = HEAP32[$315>>2]|0;
     _blobGrowBuffer($313,$316,$rc);
     $317 = HEAP32[$rc>>2]|0;
     $318 = ($317|0)==(0);
     if ($318) {
      $319 = HEAP32[$pNode>>2]|0;
      $320 = ((($319)) + 8|0);
      $321 = HEAP32[$320>>2]|0;
      $322 = ((($reader1)) + 24|0);
      $323 = HEAP32[$322>>2]|0;
      $324 = ((($reader1)) + 24|0);
      $325 = ((($324)) + 4|0);
      $326 = HEAP32[$325>>2]|0;
      _memcpy(($321|0),($323|0),($326|0))|0;
      $327 = ((($reader1)) + 24|0);
      $328 = ((($327)) + 4|0);
      $329 = HEAP32[$328>>2]|0;
      $330 = HEAP32[$pNode>>2]|0;
      $331 = ((($330)) + 8|0);
      $332 = ((($331)) + 4|0);
      HEAP32[$332>>2] = $329;
      $333 = HEAP32[$i>>2]|0;
      $334 = ($333|0)>(0);
      if ($334) {
       HEAP32[$aBlock>>2] = 0;
       HEAP32[$nBlock>>2] = 0;
       $335 = HEAP32[$i>>2]|0;
       $336 = (($335) - 1)|0;
       $337 = HEAP32[$8>>2]|0;
       $338 = ((($337)) + 56|0);
       $339 = (($338) + ($336<<5)|0);
       HEAP32[$pNode>>2] = $339;
       $340 = ((($reader1)) + 16|0);
       $341 = $340;
       $342 = $341;
       $343 = HEAP32[$342>>2]|0;
       $344 = (($341) + 4)|0;
       $345 = $344;
       $346 = HEAP32[$345>>2]|0;
       $347 = HEAP32[$pNode>>2]|0;
       $348 = $347;
       $349 = $348;
       HEAP32[$349>>2] = $343;
       $350 = (($348) + 4)|0;
       $351 = $350;
       HEAP32[$351>>2] = $346;
       $352 = HEAP32[$3>>2]|0;
       $353 = ((($reader1)) + 16|0);
       $354 = $353;
       $355 = $354;
       $356 = HEAP32[$355>>2]|0;
       $357 = (($354) + 4)|0;
       $358 = $357;
       $359 = HEAP32[$358>>2]|0;
       $360 = (_sqlite3Fts3ReadBlock($352,$356,$359,$aBlock,$nBlock,0)|0);
       HEAP32[$rc>>2] = $360;
       $361 = HEAP32[$pNode>>2]|0;
       $362 = ((($361)) + 20|0);
       $363 = HEAP32[$nBlock>>2]|0;
       $364 = HEAP32[$3>>2]|0;
       $365 = ((($364)) + 224|0);
       $366 = HEAP32[$365>>2]|0;
       $367 = ($363|0)>($366|0);
       if ($367) {
        $368 = HEAP32[$nBlock>>2]|0;
        $372 = $368;
       } else {
        $369 = HEAP32[$3>>2]|0;
        $370 = ((($369)) + 224|0);
        $371 = HEAP32[$370>>2]|0;
        $372 = $371;
       }
       _blobGrowBuffer($362,$372,$rc);
       $373 = HEAP32[$rc>>2]|0;
       $374 = ($373|0)==(0);
       if ($374) {
        $375 = HEAP32[$pNode>>2]|0;
        $376 = ((($375)) + 20|0);
        $377 = HEAP32[$376>>2]|0;
        $378 = HEAP32[$aBlock>>2]|0;
        $379 = HEAP32[$nBlock>>2]|0;
        _memcpy(($377|0),($378|0),($379|0))|0;
        $380 = HEAP32[$nBlock>>2]|0;
        $381 = HEAP32[$pNode>>2]|0;
        $382 = ((($381)) + 20|0);
        $383 = ((($382)) + 4|0);
        HEAP32[$383>>2] = $380;
       }
       $384 = HEAP32[$aBlock>>2]|0;
       _sqlite3_free($384);
      }
     }
     _nodeReaderRelease($reader1);
     $385 = HEAP32[$i>>2]|0;
     $386 = (($385) + -1)|0;
     HEAP32[$i>>2] = $386;
    }
   }
  } while(0);
  $387 = HEAP32[$pSelect>>2]|0;
  $388 = (_sqlite3_reset($387)|0);
  HEAP32[$rc2>>2] = $388;
  $389 = HEAP32[$rc>>2]|0;
  $390 = ($389|0)==(0);
  if ($390) {
   $391 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $391;
  }
 }
 $392 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $392;
 $393 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($393|0);
}
function _fts3IncrmergeWriter($p,$0,$1,$iIdx,$pCsr,$pWriter) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $pCsr = $pCsr|0;
 $pWriter = $pWriter|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $nLeafEst = 0, $pFirstBlock = 0, $pLeafEst = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $rc = sp + 24|0;
 $i = sp + 20|0;
 $nLeafEst = sp + 16|0;
 $pLeafEst = sp + 12|0;
 $pFirstBlock = sp + 8|0;
 HEAP32[$3>>2] = $p;
 $8 = $4;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$5>>2] = $iIdx;
 HEAP32[$6>>2] = $pCsr;
 HEAP32[$7>>2] = $pWriter;
 HEAP32[$nLeafEst>>2] = 0;
 HEAP32[$pLeafEst>>2] = 0;
 HEAP32[$pFirstBlock>>2] = 0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (_fts3SqlStmt($12,29,$pLeafEst,0)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(0);
 if ($15) {
  $16 = HEAP32[$pLeafEst>>2]|0;
  $17 = $4;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($17) + 4)|0;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  (_sqlite3_bind_int64($16,1,$19,$22)|0);
  $23 = HEAP32[$pLeafEst>>2]|0;
  $24 = HEAP32[$6>>2]|0;
  $25 = ((($24)) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  (_sqlite3_bind_int64($23,2,$26,$28)|0);
  $29 = HEAP32[$pLeafEst>>2]|0;
  $30 = (_sqlite3_step($29)|0);
  $31 = (100)==($30|0);
  if ($31) {
   $32 = HEAP32[$pLeafEst>>2]|0;
   $33 = (_sqlite3_column_int($32,0)|0);
   HEAP32[$nLeafEst>>2] = $33;
  }
  $34 = HEAP32[$pLeafEst>>2]|0;
  $35 = (_sqlite3_reset($34)|0);
  HEAP32[$rc>>2] = $35;
 }
 $36 = HEAP32[$rc>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $38;
  $152 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($152|0);
 }
 $39 = HEAP32[$3>>2]|0;
 $40 = (_fts3SqlStmt($39,10,$pFirstBlock,0)|0);
 HEAP32[$rc>>2] = $40;
 $41 = HEAP32[$rc>>2]|0;
 $42 = ($41|0)==(0);
 if ($42) {
  $43 = HEAP32[$pFirstBlock>>2]|0;
  $44 = (_sqlite3_step($43)|0);
  $45 = (100)==($44|0);
  if ($45) {
   $46 = HEAP32[$pFirstBlock>>2]|0;
   $47 = (_sqlite3_column_int64($46,0)|0);
   $48 = tempRet0;
   $49 = HEAP32[$7>>2]|0;
   $50 = ((($49)) + 24|0);
   $51 = $50;
   $52 = $51;
   HEAP32[$52>>2] = $47;
   $53 = (($51) + 4)|0;
   $54 = $53;
   HEAP32[$54>>2] = $48;
   $55 = HEAP32[$7>>2]|0;
   $56 = ((($55)) + 24|0);
   $57 = $56;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (($57) + 4)|0;
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = (_i64Subtract(($59|0),($62|0),1,0)|0);
   $64 = tempRet0;
   $65 = HEAP32[$7>>2]|0;
   $66 = ((($65)) + 32|0);
   $67 = $66;
   $68 = $67;
   HEAP32[$68>>2] = $63;
   $69 = (($67) + 4)|0;
   $70 = $69;
   HEAP32[$70>>2] = $64;
   $71 = HEAP32[$nLeafEst>>2]|0;
   $72 = $71<<4;
   $73 = ($72|0)<(0);
   $74 = $73 << 31 >> 31;
   $75 = HEAP32[$7>>2]|0;
   $76 = ((($75)) + 32|0);
   $77 = $76;
   $78 = $77;
   $79 = HEAP32[$78>>2]|0;
   $80 = (($77) + 4)|0;
   $81 = $80;
   $82 = HEAP32[$81>>2]|0;
   $83 = (_i64Add(($79|0),($82|0),($72|0),($74|0))|0);
   $84 = tempRet0;
   $85 = $76;
   $86 = $85;
   HEAP32[$86>>2] = $83;
   $87 = (($85) + 4)|0;
   $88 = $87;
   HEAP32[$88>>2] = $84;
  }
  $89 = HEAP32[$pFirstBlock>>2]|0;
  $90 = (_sqlite3_reset($89)|0);
  HEAP32[$rc>>2] = $90;
 }
 $91 = HEAP32[$rc>>2]|0;
 $92 = ($91|0)!=(0);
 if ($92) {
  $93 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $93;
  $152 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($152|0);
 }
 $94 = HEAP32[$3>>2]|0;
 $95 = HEAP32[$7>>2]|0;
 $96 = ((($95)) + 32|0);
 $97 = $96;
 $98 = $97;
 $99 = HEAP32[$98>>2]|0;
 $100 = (($97) + 4)|0;
 $101 = $100;
 $102 = HEAP32[$101>>2]|0;
 $103 = (_fts3WriteSegment($94,$99,$102,0,0)|0);
 HEAP32[$rc>>2] = $103;
 $104 = HEAP32[$rc>>2]|0;
 $105 = ($104|0)!=(0);
 if ($105) {
  $106 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $106;
  $152 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($152|0);
 }
 $107 = $4;
 $108 = $107;
 $109 = HEAP32[$108>>2]|0;
 $110 = (($107) + 4)|0;
 $111 = $110;
 $112 = HEAP32[$111>>2]|0;
 $113 = HEAP32[$7>>2]|0;
 $114 = ((($113)) + 8|0);
 $115 = $114;
 $116 = $115;
 HEAP32[$116>>2] = $109;
 $117 = (($115) + 4)|0;
 $118 = $117;
 HEAP32[$118>>2] = $112;
 $119 = HEAP32[$nLeafEst>>2]|0;
 $120 = HEAP32[$7>>2]|0;
 HEAP32[$120>>2] = $119;
 $121 = HEAP32[$5>>2]|0;
 $122 = HEAP32[$7>>2]|0;
 $123 = ((($122)) + 16|0);
 HEAP32[$123>>2] = $121;
 HEAP32[$i>>2] = 0;
 while(1) {
  $124 = HEAP32[$i>>2]|0;
  $125 = ($124|0)<(16);
  if (!($125)) {
   break;
  }
  $126 = HEAP32[$7>>2]|0;
  $127 = ((($126)) + 24|0);
  $128 = $127;
  $129 = $128;
  $130 = HEAP32[$129>>2]|0;
  $131 = (($128) + 4)|0;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = HEAP32[$i>>2]|0;
  $135 = HEAP32[$7>>2]|0;
  $136 = HEAP32[$135>>2]|0;
  $137 = Math_imul($134, $136)|0;
  $138 = ($137|0)<(0);
  $139 = $138 << 31 >> 31;
  $140 = (_i64Add(($130|0),($133|0),($137|0),($139|0))|0);
  $141 = tempRet0;
  $142 = HEAP32[$i>>2]|0;
  $143 = HEAP32[$7>>2]|0;
  $144 = ((($143)) + 56|0);
  $145 = (($144) + ($142<<5)|0);
  $146 = $145;
  $147 = $146;
  HEAP32[$147>>2] = $140;
  $148 = (($146) + 4)|0;
  $149 = $148;
  HEAP32[$149>>2] = $141;
  $150 = HEAP32[$i>>2]|0;
  $151 = (($150) + 1)|0;
  HEAP32[$i>>2] = $151;
 }
 HEAP32[$2>>2] = 0;
 $152 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($152|0);
}
function _fts3IncrmergeAppend($p,$pWriter,$pCsr) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aDoclist = 0, $nDoclist = 0, $nPrefix = 0;
 var $nSpace = 0, $nSuffix = 0, $nTerm = 0, $pLeaf = 0, $rc = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $zTerm = sp + 32|0;
 $nTerm = sp + 28|0;
 $aDoclist = sp + 24|0;
 $nDoclist = sp + 20|0;
 $rc = sp + 16|0;
 $nSpace = sp + 12|0;
 $nPrefix = sp + 8|0;
 $nSuffix = sp + 4|0;
 $pLeaf = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pWriter;
 HEAP32[$2>>2] = $pCsr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$zTerm>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nTerm>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 48|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$aDoclist>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 52|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$nDoclist>>2] = $14;
 HEAP32[$rc>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 56|0);
 HEAP32[$pLeaf>>2] = $16;
 $17 = HEAP32[$pLeaf>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$pLeaf>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$zTerm>>2]|0;
 $25 = HEAP32[$nTerm>>2]|0;
 $26 = (_fts3PrefixCompress($19,$23,$24,$25)|0);
 HEAP32[$nPrefix>>2] = $26;
 $27 = HEAP32[$nTerm>>2]|0;
 $28 = HEAP32[$nPrefix>>2]|0;
 $29 = (($27) - ($28))|0;
 HEAP32[$nSuffix>>2] = $29;
 $30 = HEAP32[$nPrefix>>2]|0;
 $31 = ($30|0)<(0);
 $32 = $31 << 31 >> 31;
 $33 = (_sqlite3Fts3VarintLen($30,$32)|0);
 HEAP32[$nSpace>>2] = $33;
 $34 = HEAP32[$nSuffix>>2]|0;
 $35 = ($34|0)<(0);
 $36 = $35 << 31 >> 31;
 $37 = (_sqlite3Fts3VarintLen($34,$36)|0);
 $38 = HEAP32[$nSuffix>>2]|0;
 $39 = (($37) + ($38))|0;
 $40 = HEAP32[$nSpace>>2]|0;
 $41 = (($40) + ($39))|0;
 HEAP32[$nSpace>>2] = $41;
 $42 = HEAP32[$nDoclist>>2]|0;
 $43 = ($42|0)<(0);
 $44 = $43 << 31 >> 31;
 $45 = (_sqlite3Fts3VarintLen($42,$44)|0);
 $46 = HEAP32[$nDoclist>>2]|0;
 $47 = (($45) + ($46))|0;
 $48 = HEAP32[$nSpace>>2]|0;
 $49 = (($48) + ($47))|0;
 HEAP32[$nSpace>>2] = $49;
 $50 = HEAP32[$pLeaf>>2]|0;
 $51 = ((($50)) + 20|0);
 $52 = ((($51)) + 4|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = ($53|0)>(0);
 if ($54) {
  $55 = HEAP32[$pLeaf>>2]|0;
  $56 = ((($55)) + 20|0);
  $57 = ((($56)) + 4|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$nSpace>>2]|0;
  $60 = (($58) + ($59))|0;
  $61 = HEAP32[$0>>2]|0;
  $62 = ((($61)) + 224|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($60|0)>($63|0);
  if ($64) {
   $65 = HEAP32[$0>>2]|0;
   $66 = HEAP32[$pLeaf>>2]|0;
   $67 = $66;
   $68 = $67;
   $69 = HEAP32[$68>>2]|0;
   $70 = (($67) + 4)|0;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$pLeaf>>2]|0;
   $74 = ((($73)) + 20|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$pLeaf>>2]|0;
   $77 = ((($76)) + 20|0);
   $78 = ((($77)) + 4|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (_fts3WriteSegment($65,$69,$72,$75,$79)|0);
   HEAP32[$rc>>2] = $80;
   $81 = HEAP32[$1>>2]|0;
   $82 = ((($81)) + 4|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = (($83) + 1)|0;
   HEAP32[$82>>2] = $84;
   $85 = HEAP32[$rc>>2]|0;
   $86 = ($85|0)==(0);
   if ($86) {
    $87 = HEAP32[$0>>2]|0;
    $88 = HEAP32[$1>>2]|0;
    $89 = HEAP32[$zTerm>>2]|0;
    $90 = HEAP32[$nPrefix>>2]|0;
    $91 = (($90) + 1)|0;
    $92 = (_fts3IncrmergePush($87,$88,$89,$91)|0);
    HEAP32[$rc>>2] = $92;
   }
   $93 = HEAP32[$pLeaf>>2]|0;
   $94 = $93;
   $95 = $94;
   $96 = HEAP32[$95>>2]|0;
   $97 = (($94) + 4)|0;
   $98 = $97;
   $99 = HEAP32[$98>>2]|0;
   $100 = (_i64Add(($96|0),($99|0),1,0)|0);
   $101 = tempRet0;
   $102 = $93;
   $103 = $102;
   HEAP32[$103>>2] = $100;
   $104 = (($102) + 4)|0;
   $105 = $104;
   HEAP32[$105>>2] = $101;
   $106 = HEAP32[$pLeaf>>2]|0;
   $107 = ((($106)) + 8|0);
   $108 = ((($107)) + 4|0);
   HEAP32[$108>>2] = 0;
   $109 = HEAP32[$pLeaf>>2]|0;
   $110 = ((($109)) + 20|0);
   $111 = ((($110)) + 4|0);
   HEAP32[$111>>2] = 0;
   $112 = HEAP32[$nTerm>>2]|0;
   HEAP32[$nSuffix>>2] = $112;
   HEAP32[$nSpace>>2] = 1;
   $113 = HEAP32[$nSuffix>>2]|0;
   $114 = ($113|0)<(0);
   $115 = $114 << 31 >> 31;
   $116 = (_sqlite3Fts3VarintLen($113,$115)|0);
   $117 = HEAP32[$nSuffix>>2]|0;
   $118 = (($116) + ($117))|0;
   $119 = HEAP32[$nSpace>>2]|0;
   $120 = (($119) + ($118))|0;
   HEAP32[$nSpace>>2] = $120;
   $121 = HEAP32[$nDoclist>>2]|0;
   $122 = ($121|0)<(0);
   $123 = $122 << 31 >> 31;
   $124 = (_sqlite3Fts3VarintLen($121,$123)|0);
   $125 = HEAP32[$nDoclist>>2]|0;
   $126 = (($124) + ($125))|0;
   $127 = HEAP32[$nSpace>>2]|0;
   $128 = (($127) + ($126))|0;
   HEAP32[$nSpace>>2] = $128;
  }
 }
 $129 = HEAP32[$nSpace>>2]|0;
 $130 = ($129|0)<(0);
 $131 = $130 << 31 >> 31;
 $132 = HEAP32[$1>>2]|0;
 $133 = ((($132)) + 40|0);
 $134 = $133;
 $135 = $134;
 $136 = HEAP32[$135>>2]|0;
 $137 = (($134) + 4)|0;
 $138 = $137;
 $139 = HEAP32[$138>>2]|0;
 $140 = (_i64Add(($136|0),($139|0),($129|0),($131|0))|0);
 $141 = tempRet0;
 $142 = $133;
 $143 = $142;
 HEAP32[$143>>2] = $140;
 $144 = (($142) + 4)|0;
 $145 = $144;
 HEAP32[$145>>2] = $141;
 $146 = HEAP32[$pLeaf>>2]|0;
 $147 = ((($146)) + 20|0);
 $148 = HEAP32[$pLeaf>>2]|0;
 $149 = ((($148)) + 20|0);
 $150 = ((($149)) + 4|0);
 $151 = HEAP32[$150>>2]|0;
 $152 = HEAP32[$nSpace>>2]|0;
 $153 = (($151) + ($152))|0;
 _blobGrowBuffer($147,$153,$rc);
 $154 = HEAP32[$rc>>2]|0;
 $155 = ($154|0)==(0);
 if (!($155)) {
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $156 = HEAP32[$pLeaf>>2]|0;
 $157 = ((($156)) + 20|0);
 $158 = ((($157)) + 4|0);
 $159 = HEAP32[$158>>2]|0;
 $160 = ($159|0)==(0);
 if ($160) {
  $161 = HEAP32[$pLeaf>>2]|0;
  $162 = ((($161)) + 20|0);
  $163 = ((($162)) + 4|0);
  HEAP32[$163>>2] = 1;
  $164 = HEAP32[$pLeaf>>2]|0;
  $165 = ((($164)) + 20|0);
  $166 = HEAP32[$165>>2]|0;
  HEAP8[$166>>0] = 0;
 }
 $167 = HEAP32[$pLeaf>>2]|0;
 $168 = ((($167)) + 20|0);
 $169 = HEAP32[$pLeaf>>2]|0;
 $170 = ((($169)) + 8|0);
 $171 = HEAP32[$zTerm>>2]|0;
 $172 = HEAP32[$nTerm>>2]|0;
 $173 = HEAP32[$aDoclist>>2]|0;
 $174 = HEAP32[$nDoclist>>2]|0;
 $175 = (_fts3AppendToNode($168,$170,$171,$172,$173,$174)|0);
 HEAP32[$rc>>2] = $175;
 $176 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($176|0);
}
function _fts3IncrmergeChomp($p,$0,$1,$pCsr,$pnRem) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pCsr = $pCsr|0;
 $pnRem = $pnRem|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $i = 0, $j = 0, $nRem = 0, $nTerm = 0, $pSeg = 0, $rc = 0, $zTerm = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $i = sp + 32|0;
 $nRem = sp + 28|0;
 $rc = sp + 24|0;
 $pSeg = sp + 20|0;
 $j = sp + 16|0;
 $zTerm = sp + 12|0;
 $nTerm = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $pCsr;
 HEAP32[$5>>2] = $pnRem;
 HEAP32[$nRem>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $10 = HEAP32[$4>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) - 1)|0;
 HEAP32[$i>>2] = $13;
 while(1) {
  $14 = HEAP32[$i>>2]|0;
  $15 = ($14|0)>=(0);
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)==(0);
  $18 = $15 ? $17 : 0;
  if (!($18)) {
   break;
  }
  HEAP32[$pSeg>>2] = 0;
  HEAP32[$j>>2] = 0;
  while(1) {
   $19 = HEAP32[$j>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)<($22|0);
   if (!($23)) {
    break;
   }
   $24 = HEAP32[$j>>2]|0;
   $25 = HEAP32[$4>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) + ($24<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   HEAP32[$pSeg>>2] = $28;
   $29 = HEAP32[$pSeg>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$i>>2]|0;
   $32 = ($30|0)==($31|0);
   if ($32) {
    break;
   }
   $33 = HEAP32[$j>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$j>>2] = $34;
  }
  $35 = HEAP32[$pSeg>>2]|0;
  $36 = ((($35)) + 40|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)==(0|0);
  if ($38) {
   $39 = HEAP32[$2>>2]|0;
   $40 = HEAP32[$pSeg>>2]|0;
   $41 = (_fts3DeleteSegment($39,$40)|0);
   HEAP32[$rc>>2] = $41;
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)==(0);
   if ($43) {
    $44 = HEAP32[$2>>2]|0;
    $45 = $3;
    $46 = $45;
    $47 = HEAP32[$46>>2]|0;
    $48 = (($45) + 4)|0;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$pSeg>>2]|0;
    $52 = HEAP32[$51>>2]|0;
    $53 = (_fts3RemoveSegdirEntry($44,$47,$50,$52)|0);
    HEAP32[$rc>>2] = $53;
   }
   $54 = HEAP32[$5>>2]|0;
   HEAP32[$54>>2] = 0;
  } else {
   $55 = HEAP32[$pSeg>>2]|0;
   $56 = ((($55)) + 64|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$zTerm>>2] = $57;
   $58 = HEAP32[$pSeg>>2]|0;
   $59 = ((($58)) + 60|0);
   $60 = HEAP32[$59>>2]|0;
   HEAP32[$nTerm>>2] = $60;
   $61 = HEAP32[$2>>2]|0;
   $62 = $3;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = HEAP32[$pSeg>>2]|0;
   $69 = HEAP32[$68>>2]|0;
   $70 = HEAP32[$zTerm>>2]|0;
   $71 = HEAP32[$nTerm>>2]|0;
   $72 = (_fts3TruncateSegment($61,$64,$67,$69,$70,$71)|0);
   HEAP32[$rc>>2] = $72;
   $73 = HEAP32[$nRem>>2]|0;
   $74 = (($73) + 1)|0;
   HEAP32[$nRem>>2] = $74;
  }
  $75 = HEAP32[$i>>2]|0;
  $76 = (($75) + -1)|0;
  HEAP32[$i>>2] = $76;
 }
 $77 = HEAP32[$rc>>2]|0;
 $78 = ($77|0)==(0);
 if (!($78)) {
  $92 = HEAP32[$nRem>>2]|0;
  $93 = HEAP32[$5>>2]|0;
  HEAP32[$93>>2] = $92;
  $94 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($94|0);
 }
 $79 = HEAP32[$nRem>>2]|0;
 $80 = HEAP32[$4>>2]|0;
 $81 = ((($80)) + 4|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = ($79|0)!=($82|0);
 if (!($83)) {
  $92 = HEAP32[$nRem>>2]|0;
  $93 = HEAP32[$5>>2]|0;
  HEAP32[$93>>2] = $92;
  $94 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($94|0);
 }
 $84 = HEAP32[$2>>2]|0;
 $85 = $3;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = (_fts3RepackSegdirLevel($84,$87,$90)|0);
 HEAP32[$rc>>2] = $91;
 $92 = HEAP32[$nRem>>2]|0;
 $93 = HEAP32[$5>>2]|0;
 HEAP32[$93>>2] = $92;
 $94 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($94|0);
}
function _fts3IncrmergeHintPush($pHint,$0,$1,$nInput,$pRc) {
 $pHint = $pHint|0;
 $0 = $0|0;
 $1 = $1|0;
 $nInput = $nInput|0;
 $pRc = $pRc|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $pHint;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $nInput;
 HEAP32[$5>>2] = $pRc;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + 20)|0;
 $15 = HEAP32[$5>>2]|0;
 _blobGrowBuffer($10,$14,$15);
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($21)|0);
 $25 = $3;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3Fts3PutVarint($24,$27,$30)|0);
 $32 = HEAP32[$2>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31))|0;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$2>>2]|0;
 $37 = ((($36)) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + ($38)|0);
 $42 = HEAP32[$4>>2]|0;
 $43 = ($42|0)<(0);
 $44 = $43 << 31 >> 31;
 $45 = (_sqlite3Fts3PutVarint($41,$42,$44)|0);
 $46 = HEAP32[$2>>2]|0;
 $47 = ((($46)) + 4|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + ($45))|0;
 HEAP32[$47>>2] = $49;
 STACKTOP = sp;return;
}
function _fts3IncrmergeRelease($p,$pWriter,$pRc) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iRoot = 0, $or$cond = 0, $pBlock = 0, $pNode = 0, $pNode1 = 0, $pRoot = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $i = sp + 24|0;
 $iRoot = sp + 20|0;
 $pRoot = sp + 16|0;
 $rc = sp + 12|0;
 $pNode = sp + 8|0;
 $pBlock = sp + 4|0;
 $pNode1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pWriter;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$rc>>2] = $4;
 HEAP32[$iRoot>>2] = 15;
 while(1) {
  $5 = HEAP32[$iRoot>>2]|0;
  $6 = ($5|0)>=(0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$iRoot>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 56|0);
  $10 = (($9) + ($7<<5)|0);
  HEAP32[$pNode>>2] = $10;
  $11 = HEAP32[$pNode>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)>(0);
  if ($15) {
   break;
  }
  $16 = HEAP32[$pNode>>2]|0;
  $17 = ((($16)) + 20|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3_free($18);
  $19 = HEAP32[$pNode>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  _sqlite3_free($21);
  $22 = HEAP32[$iRoot>>2]|0;
  $23 = (($22) + -1)|0;
  HEAP32[$iRoot>>2] = $23;
 }
 $24 = HEAP32[$iRoot>>2]|0;
 $25 = ($24|0)<(0);
 if ($25) {
  STACKTOP = sp;return;
 }
 $26 = HEAP32[$iRoot>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 56|0);
  $30 = ((($29)) + 32|0);
  $31 = ((($30)) + 20|0);
  HEAP32[$pBlock>>2] = $31;
  $32 = HEAP32[$pBlock>>2]|0;
  _blobGrowBuffer($32,11,$rc);
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)==(0);
  if ($34) {
   $35 = HEAP32[$pBlock>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   HEAP8[$36>>0] = 1;
   $37 = HEAP32[$pBlock>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   $39 = ((($38)) + 1|0);
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 56|0);
   $42 = $41;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = (_sqlite3Fts3PutVarint($39,$44,$47)|0);
   $49 = (1 + ($48))|0;
   $50 = HEAP32[$pBlock>>2]|0;
   $51 = ((($50)) + 4|0);
   HEAP32[$51>>2] = $49;
  }
  HEAP32[$iRoot>>2] = 1;
 }
 $52 = HEAP32[$iRoot>>2]|0;
 $53 = HEAP32[$1>>2]|0;
 $54 = ((($53)) + 56|0);
 $55 = (($54) + ($52<<5)|0);
 HEAP32[$pRoot>>2] = $55;
 HEAP32[$i>>2] = 0;
 while(1) {
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$iRoot>>2]|0;
  $58 = ($56|0)<($57|0);
  if (!($58)) {
   break;
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 56|0);
  $62 = (($61) + ($59<<5)|0);
  HEAP32[$pNode1>>2] = $62;
  $63 = HEAP32[$pNode1>>2]|0;
  $64 = ((($63)) + 20|0);
  $65 = ((($64)) + 4|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = ($66|0)>(0);
  $68 = HEAP32[$rc>>2]|0;
  $69 = ($68|0)==(0);
  $or$cond = $67 & $69;
  if ($or$cond) {
   $70 = HEAP32[$0>>2]|0;
   $71 = HEAP32[$pNode1>>2]|0;
   $72 = $71;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = HEAP32[$pNode1>>2]|0;
   $79 = ((($78)) + 20|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = HEAP32[$pNode1>>2]|0;
   $82 = ((($81)) + 20|0);
   $83 = ((($82)) + 4|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = (_fts3WriteSegment($70,$74,$77,$80,$84)|0);
   HEAP32[$rc>>2] = $85;
  }
  $86 = HEAP32[$pNode1>>2]|0;
  $87 = ((($86)) + 20|0);
  $88 = HEAP32[$87>>2]|0;
  _sqlite3_free($88);
  $89 = HEAP32[$pNode1>>2]|0;
  $90 = ((($89)) + 8|0);
  $91 = HEAP32[$90>>2]|0;
  _sqlite3_free($91);
  $92 = HEAP32[$i>>2]|0;
  $93 = (($92) + 1)|0;
  HEAP32[$i>>2] = $93;
 }
 $94 = HEAP32[$rc>>2]|0;
 $95 = ($94|0)==(0);
 if ($95) {
  $96 = HEAP32[$0>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  $98 = ((($97)) + 8|0);
  $99 = $98;
  $100 = $99;
  $101 = HEAP32[$100>>2]|0;
  $102 = (($99) + 4)|0;
  $103 = $102;
  $104 = HEAP32[$103>>2]|0;
  $105 = (_i64Add(($101|0),($104|0),1,0)|0);
  $106 = tempRet0;
  $107 = HEAP32[$1>>2]|0;
  $108 = ((($107)) + 16|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = HEAP32[$1>>2]|0;
  $111 = ((($110)) + 24|0);
  $112 = $111;
  $113 = $112;
  $114 = HEAP32[$113>>2]|0;
  $115 = (($112) + 4)|0;
  $116 = $115;
  $117 = HEAP32[$116>>2]|0;
  $118 = HEAP32[$1>>2]|0;
  $119 = ((($118)) + 56|0);
  $120 = $119;
  $121 = $120;
  $122 = HEAP32[$121>>2]|0;
  $123 = (($120) + 4)|0;
  $124 = $123;
  $125 = HEAP32[$124>>2]|0;
  $126 = HEAP32[$1>>2]|0;
  $127 = ((($126)) + 32|0);
  $128 = $127;
  $129 = $128;
  $130 = HEAP32[$129>>2]|0;
  $131 = (($128) + 4)|0;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = HEAP32[$1>>2]|0;
  $135 = ((($134)) + 48|0);
  $136 = HEAP8[$135>>0]|0;
  $137 = $136&255;
  $138 = ($137|0)==(0);
  if ($138) {
   $139 = HEAP32[$1>>2]|0;
   $140 = ((($139)) + 40|0);
   $141 = $140;
   $142 = $141;
   $143 = HEAP32[$142>>2]|0;
   $144 = (($141) + 4)|0;
   $145 = $144;
   $146 = HEAP32[$145>>2]|0;
   $154 = $143;$155 = $146;
  } else {
   $154 = 0;$155 = 0;
  }
  $147 = HEAP32[$pRoot>>2]|0;
  $148 = ((($147)) + 20|0);
  $149 = HEAP32[$148>>2]|0;
  $150 = HEAP32[$pRoot>>2]|0;
  $151 = ((($150)) + 20|0);
  $152 = ((($151)) + 4|0);
  $153 = HEAP32[$152>>2]|0;
  $156 = (_fts3WriteSegdir($96,$105,$106,$109,$114,$117,$122,$125,$130,$133,$154,$155,$149,$153)|0);
  HEAP32[$rc>>2] = $156;
 }
 $157 = HEAP32[$pRoot>>2]|0;
 $158 = ((($157)) + 20|0);
 $159 = HEAP32[$158>>2]|0;
 _sqlite3_free($159);
 $160 = HEAP32[$pRoot>>2]|0;
 $161 = ((($160)) + 8|0);
 $162 = HEAP32[$161>>2]|0;
 _sqlite3_free($162);
 $163 = HEAP32[$rc>>2]|0;
 $164 = HEAP32[$2>>2]|0;
 HEAP32[$164>>2] = $163;
 STACKTOP = sp;return;
}
function _fts3IncrmergeHintStore($p,$pHint) {
 $p = $p|0;
 $pHint = $pHint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pReplace = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pReplace = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pHint;
 HEAP32[$pReplace>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_fts3SqlStmt($2,23,$pReplace,0)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $6 = HEAP32[$pReplace>>2]|0;
 (_sqlite3_bind_int($6,1,1)|0);
 $7 = HEAP32[$pReplace>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 (_sqlite3_bind_blob($7,2,$9,$12,0)|0);
 $13 = HEAP32[$pReplace>>2]|0;
 (_sqlite3_step($13)|0);
 $14 = HEAP32[$pReplace>>2]|0;
 $15 = (_sqlite3_reset($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _blobGrowBuffer($pBlob,$nMin,$pRc) {
 $pBlob = $pBlob|0;
 $nMin = $nMin|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $nAlloc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nAlloc = sp + 4|0;
 $a = sp;
 HEAP32[$0>>2] = $pBlob;
 HEAP32[$1>>2] = $nMin;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)>($9|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$nAlloc>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$nAlloc>>2]|0;
 $15 = (_sqlite3_realloc($13,$14)|0);
 HEAP32[$a>>2] = $15;
 $16 = HEAP32[$a>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$nAlloc>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 8|0);
  HEAP32[$20>>2] = $18;
  $21 = HEAP32[$a>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  HEAP32[$22>>2] = $21;
  STACKTOP = sp;return;
 } else {
  $23 = HEAP32[$2>>2]|0;
  HEAP32[$23>>2] = 7;
  STACKTOP = sp;return;
 }
}
function _fts3RemoveSegdirEntry($p,$0,$1,$iIdx) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDelete = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp;
 $4 = sp + 16|0;
 $rc = sp + 12|0;
 $pDelete = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $iIdx;
 HEAP32[$pDelete>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,30,$pDelete,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $25 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $13 = HEAP32[$pDelete>>2]|0;
 $14 = $3;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 (_sqlite3_bind_int64($13,1,$16,$19)|0);
 $20 = HEAP32[$pDelete>>2]|0;
 $21 = HEAP32[$4>>2]|0;
 (_sqlite3_bind_int($20,2,$21)|0);
 $22 = HEAP32[$pDelete>>2]|0;
 (_sqlite3_step($22)|0);
 $23 = HEAP32[$pDelete>>2]|0;
 $24 = (_sqlite3_reset($23)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _fts3TruncateSegment($p,$0,$1,$iIdx,$zTerm,$nTerm) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $iIdx = $iIdx|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBlock = 0, $aRoot = 0, $block = 0, $iBlock = 0, $iNewStart = 0;
 var $iOldStart = 0, $nBlock = 0, $nRoot = 0, $or$cond = 0, $pChomp = 0, $pDel = 0, $pFetch = 0, $rc = 0, $rc2 = 0, $root = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $2 = sp + 104|0;
 $3 = sp + 24|0;
 $4 = sp + 100|0;
 $5 = sp + 96|0;
 $6 = sp + 92|0;
 $rc = sp + 88|0;
 $root = sp + 76|0;
 $block = sp + 64|0;
 $iBlock = sp + 16|0;
 $iNewStart = sp + 8|0;
 $iOldStart = sp;
 $pFetch = sp + 60|0;
 $rc2 = sp + 56|0;
 $aRoot = sp + 52|0;
 $nRoot = sp + 48|0;
 $aBlock = sp + 44|0;
 $nBlock = sp + 40|0;
 $pDel = sp + 36|0;
 $pChomp = sp + 32|0;
 HEAP32[$2>>2] = $p;
 $7 = $3;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$4>>2] = $iIdx;
 HEAP32[$5>>2] = $zTerm;
 HEAP32[$6>>2] = $nTerm;
 HEAP32[$rc>>2] = 0;
 ;HEAP32[$root>>2]=0|0;HEAP32[$root+4>>2]=0|0;HEAP32[$root+8>>2]=0|0;
 ;HEAP32[$block>>2]=0|0;HEAP32[$block+4>>2]=0|0;HEAP32[$block+8>>2]=0|0;
 $11 = $iBlock;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = $iNewStart;
 $16 = $15;
 HEAP32[$16>>2] = 0;
 $17 = (($15) + 4)|0;
 $18 = $17;
 HEAP32[$18>>2] = 0;
 $19 = $iOldStart;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 HEAP32[$pFetch>>2] = 0;
 $23 = HEAP32[$2>>2]|0;
 $24 = (_fts3SqlStmt($23,32,$pFetch,0)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 $26 = ($25|0)==(0);
 if ($26) {
  $27 = HEAP32[$pFetch>>2]|0;
  $28 = $3;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($28) + 4)|0;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  (_sqlite3_bind_int64($27,1,$30,$33)|0);
  $34 = HEAP32[$pFetch>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  (_sqlite3_bind_int($34,2,$35)|0);
  $36 = HEAP32[$pFetch>>2]|0;
  $37 = (_sqlite3_step($36)|0);
  $38 = (100)==($37|0);
  if ($38) {
   $39 = HEAP32[$pFetch>>2]|0;
   $40 = (_sqlite3_column_blob($39,4)|0);
   HEAP32[$aRoot>>2] = $40;
   $41 = HEAP32[$pFetch>>2]|0;
   $42 = (_sqlite3_column_bytes($41,4)|0);
   HEAP32[$nRoot>>2] = $42;
   $43 = HEAP32[$pFetch>>2]|0;
   $44 = (_sqlite3_column_int64($43,1)|0);
   $45 = tempRet0;
   $46 = $iOldStart;
   $47 = $46;
   HEAP32[$47>>2] = $44;
   $48 = (($46) + 4)|0;
   $49 = $48;
   HEAP32[$49>>2] = $45;
   $50 = HEAP32[$aRoot>>2]|0;
   $51 = HEAP32[$nRoot>>2]|0;
   $52 = HEAP32[$5>>2]|0;
   $53 = HEAP32[$6>>2]|0;
   $54 = (_fts3TruncateNode($50,$51,$root,$52,$53,$iBlock)|0);
   HEAP32[$rc>>2] = $54;
  }
  $55 = HEAP32[$pFetch>>2]|0;
  $56 = (_sqlite3_reset($55)|0);
  HEAP32[$rc2>>2] = $56;
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)==(0);
  if ($58) {
   $59 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $59;
  }
 }
 while(1) {
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(0);
  $62 = $iBlock;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = ($64|0)!=(0);
  $69 = ($67|0)!=(0);
  $70 = $68 | $69;
  $71 = $61 ? $70 : 0;
  if (!($71)) {
   break;
  }
  HEAP32[$aBlock>>2] = 0;
  HEAP32[$nBlock>>2] = 0;
  $72 = $iBlock;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = $iNewStart;
  $79 = $78;
  HEAP32[$79>>2] = $74;
  $80 = (($78) + 4)|0;
  $81 = $80;
  HEAP32[$81>>2] = $77;
  $82 = HEAP32[$2>>2]|0;
  $83 = $iBlock;
  $84 = $83;
  $85 = HEAP32[$84>>2]|0;
  $86 = (($83) + 4)|0;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = (_sqlite3Fts3ReadBlock($82,$85,$88,$aBlock,$nBlock,0)|0);
  HEAP32[$rc>>2] = $89;
  $90 = HEAP32[$rc>>2]|0;
  $91 = ($90|0)==(0);
  if ($91) {
   $92 = HEAP32[$aBlock>>2]|0;
   $93 = HEAP32[$nBlock>>2]|0;
   $94 = HEAP32[$5>>2]|0;
   $95 = HEAP32[$6>>2]|0;
   $96 = (_fts3TruncateNode($92,$93,$block,$94,$95,$iBlock)|0);
   HEAP32[$rc>>2] = $96;
  }
  $97 = HEAP32[$rc>>2]|0;
  $98 = ($97|0)==(0);
  if ($98) {
   $99 = HEAP32[$2>>2]|0;
   $100 = $iNewStart;
   $101 = $100;
   $102 = HEAP32[$101>>2]|0;
   $103 = (($100) + 4)|0;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $106 = HEAP32[$block>>2]|0;
   $107 = ((($block)) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = (_fts3WriteSegment($99,$102,$105,$106,$108)|0);
   HEAP32[$rc>>2] = $109;
  }
  $110 = HEAP32[$aBlock>>2]|0;
  _sqlite3_free($110);
 }
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)==(0);
 $113 = $iNewStart;
 $114 = $113;
 $115 = HEAP32[$114>>2]|0;
 $116 = (($113) + 4)|0;
 $117 = $116;
 $118 = HEAP32[$117>>2]|0;
 $119 = ($115|0)!=(0);
 $120 = ($118|0)!=(0);
 $121 = $119 | $120;
 $or$cond = $112 & $121;
 if ($or$cond) {
  HEAP32[$pDel>>2] = 0;
  $122 = HEAP32[$2>>2]|0;
  $123 = (_fts3SqlStmt($122,17,$pDel,0)|0);
  HEAP32[$rc>>2] = $123;
  $124 = HEAP32[$rc>>2]|0;
  $125 = ($124|0)==(0);
  if ($125) {
   $126 = HEAP32[$pDel>>2]|0;
   $127 = $iOldStart;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = (($127) + 4)|0;
   $131 = $130;
   $132 = HEAP32[$131>>2]|0;
   (_sqlite3_bind_int64($126,1,$129,$132)|0);
   $133 = HEAP32[$pDel>>2]|0;
   $134 = $iNewStart;
   $135 = $134;
   $136 = HEAP32[$135>>2]|0;
   $137 = (($134) + 4)|0;
   $138 = $137;
   $139 = HEAP32[$138>>2]|0;
   $140 = (_i64Subtract(($136|0),($139|0),1,0)|0);
   $141 = tempRet0;
   (_sqlite3_bind_int64($133,2,$140,$141)|0);
   $142 = HEAP32[$pDel>>2]|0;
   (_sqlite3_step($142)|0);
   $143 = HEAP32[$pDel>>2]|0;
   $144 = (_sqlite3_reset($143)|0);
   HEAP32[$rc>>2] = $144;
  }
 }
 $145 = HEAP32[$rc>>2]|0;
 $146 = ($145|0)==(0);
 if (!($146)) {
  $174 = HEAP32[$root>>2]|0;
  _sqlite3_free($174);
  $175 = HEAP32[$block>>2]|0;
  _sqlite3_free($175);
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 HEAP32[$pChomp>>2] = 0;
 $147 = HEAP32[$2>>2]|0;
 $148 = (_fts3SqlStmt($147,33,$pChomp,0)|0);
 HEAP32[$rc>>2] = $148;
 $149 = HEAP32[$rc>>2]|0;
 $150 = ($149|0)==(0);
 if (!($150)) {
  $174 = HEAP32[$root>>2]|0;
  _sqlite3_free($174);
  $175 = HEAP32[$block>>2]|0;
  _sqlite3_free($175);
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $151 = HEAP32[$pChomp>>2]|0;
 $152 = $iNewStart;
 $153 = $152;
 $154 = HEAP32[$153>>2]|0;
 $155 = (($152) + 4)|0;
 $156 = $155;
 $157 = HEAP32[$156>>2]|0;
 (_sqlite3_bind_int64($151,1,$154,$157)|0);
 $158 = HEAP32[$pChomp>>2]|0;
 $159 = HEAP32[$root>>2]|0;
 $160 = ((($root)) + 4|0);
 $161 = HEAP32[$160>>2]|0;
 (_sqlite3_bind_blob($158,2,$159,$161,0)|0);
 $162 = HEAP32[$pChomp>>2]|0;
 $163 = $3;
 $164 = $163;
 $165 = HEAP32[$164>>2]|0;
 $166 = (($163) + 4)|0;
 $167 = $166;
 $168 = HEAP32[$167>>2]|0;
 (_sqlite3_bind_int64($162,3,$165,$168)|0);
 $169 = HEAP32[$pChomp>>2]|0;
 $170 = HEAP32[$4>>2]|0;
 (_sqlite3_bind_int($169,4,$170)|0);
 $171 = HEAP32[$pChomp>>2]|0;
 (_sqlite3_step($171)|0);
 $172 = HEAP32[$pChomp>>2]|0;
 $173 = (_sqlite3_reset($172)|0);
 HEAP32[$rc>>2] = $173;
 $174 = HEAP32[$root>>2]|0;
 _sqlite3_free($174);
 $175 = HEAP32[$block>>2]|0;
 _sqlite3_free($175);
 $176 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($176|0);
}
function _fts3RepackSegdirLevel($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $aIdx = 0, $aNew = 0, $i = 0, $nAlloc = 0, $nIdx = 0, $pSelect = 0, $pUpdate = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp;
 $rc = sp + 40|0;
 $aIdx = sp + 36|0;
 $nIdx = sp + 32|0;
 $nAlloc = sp + 28|0;
 $i = sp + 24|0;
 $pSelect = sp + 20|0;
 $pUpdate = sp + 16|0;
 $rc2 = sp + 12|0;
 $aNew = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP32[$aIdx>>2] = 0;
 HEAP32[$nIdx>>2] = 0;
 HEAP32[$nAlloc>>2] = 0;
 HEAP32[$pSelect>>2] = 0;
 HEAP32[$pUpdate>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_fts3SqlStmt($8,35,$pSelect,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = HEAP32[$pSelect>>2]|0;
  $13 = $3;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  (_sqlite3_bind_int64($12,1,$15,$18)|0);
  while(1) {
   $19 = HEAP32[$pSelect>>2]|0;
   $20 = (_sqlite3_step($19)|0);
   $21 = (100)==($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$nIdx>>2]|0;
   $23 = HEAP32[$nAlloc>>2]|0;
   $24 = ($22|0)>=($23|0);
   if ($24) {
    $25 = HEAP32[$nAlloc>>2]|0;
    $26 = (($25) + 16)|0;
    HEAP32[$nAlloc>>2] = $26;
    $27 = HEAP32[$aIdx>>2]|0;
    $28 = HEAP32[$nAlloc>>2]|0;
    $29 = $28<<2;
    $30 = (_sqlite3_realloc($27,$29)|0);
    HEAP32[$aNew>>2] = $30;
    $31 = HEAP32[$aNew>>2]|0;
    $32 = ($31|0)!=(0|0);
    if (!($32)) {
     label = 6;
     break;
    }
    $33 = HEAP32[$aNew>>2]|0;
    HEAP32[$aIdx>>2] = $33;
   }
   $34 = HEAP32[$pSelect>>2]|0;
   $35 = (_sqlite3_column_int($34,0)|0);
   $36 = HEAP32[$nIdx>>2]|0;
   $37 = (($36) + 1)|0;
   HEAP32[$nIdx>>2] = $37;
   $38 = HEAP32[$aIdx>>2]|0;
   $39 = (($38) + ($36<<2)|0);
   HEAP32[$39>>2] = $35;
  }
  if ((label|0) == 6) {
   HEAP32[$rc>>2] = 7;
  }
  $40 = HEAP32[$pSelect>>2]|0;
  $41 = (_sqlite3_reset($40)|0);
  HEAP32[$rc2>>2] = $41;
  $42 = HEAP32[$rc>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $44;
  }
 }
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(0);
 if ($46) {
  $47 = HEAP32[$2>>2]|0;
  $48 = (_fts3SqlStmt($47,31,$pUpdate,0)|0);
  HEAP32[$rc>>2] = $48;
 }
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = HEAP32[$pUpdate>>2]|0;
  $52 = $3;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) + 4)|0;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  (_sqlite3_bind_int64($51,2,$54,$57)|0);
 }
 $58 = HEAP32[$2>>2]|0;
 $59 = ((($58)) + 232|0);
 HEAP8[$59>>0] = 1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(0);
  if (!($61)) {
   label = 21;
   break;
  }
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$nIdx>>2]|0;
  $64 = ($62|0)<($63|0);
  if (!($64)) {
   label = 21;
   break;
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$aIdx>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$i>>2]|0;
  $70 = ($68|0)!=($69|0);
  if ($70) {
   $71 = HEAP32[$pUpdate>>2]|0;
   $72 = HEAP32[$i>>2]|0;
   $73 = HEAP32[$aIdx>>2]|0;
   $74 = (($73) + ($72<<2)|0);
   $75 = HEAP32[$74>>2]|0;
   (_sqlite3_bind_int($71,3,$75)|0);
   $76 = HEAP32[$pUpdate>>2]|0;
   $77 = HEAP32[$i>>2]|0;
   (_sqlite3_bind_int($76,1,$77)|0);
   $78 = HEAP32[$pUpdate>>2]|0;
   (_sqlite3_step($78)|0);
   $79 = HEAP32[$pUpdate>>2]|0;
   $80 = (_sqlite3_reset($79)|0);
   HEAP32[$rc>>2] = $80;
  }
  $81 = HEAP32[$i>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$i>>2] = $82;
 }
 if ((label|0) == 21) {
  $83 = HEAP32[$2>>2]|0;
  $84 = ((($83)) + 232|0);
  HEAP8[$84>>0] = 0;
  $85 = HEAP32[$aIdx>>2]|0;
  _sqlite3_free($85);
  $86 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 return (0)|0;
}
function _fts3TruncateNode($aNode,$nNode,$pNew,$zTerm,$nTerm,$piBlock) {
 $aNode = $aNode|0;
 $nNode = $nNode|0;
 $pNew = $pNew|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $piBlock = $piBlock|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bLeaf = 0, $or$cond = 0, $prev = 0, $rc = 0, $reader = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $reader = sp;
 $prev = sp + 60|0;
 $rc = sp + 56|0;
 $bLeaf = sp + 52|0;
 $res = sp + 48|0;
 HEAP32[$1>>2] = $aNode;
 HEAP32[$2>>2] = $nNode;
 HEAP32[$3>>2] = $pNew;
 HEAP32[$4>>2] = $zTerm;
 HEAP32[$5>>2] = $nTerm;
 HEAP32[$6>>2] = $piBlock;
 ;HEAP32[$prev>>2]=0|0;HEAP32[$prev+4>>2]=0|0;HEAP32[$prev+8>>2]=0|0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)==(0);
 $11 = $10&1;
 HEAP32[$bLeaf>>2] = $11;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 _blobGrowBuffer($12,$13,$rc);
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $16;
  $111 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($111|0);
 }
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 4|0);
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = (_nodeReaderInit($reader,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 while(1) {
  $22 = HEAP32[$rc>>2]|0;
  $23 = ($22|0)==(0);
  $24 = HEAP32[$reader>>2]|0;
  $25 = ($24|0)!=(0|0);
  $26 = $23 ? $25 : 0;
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$3>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($29|0)==(0);
  if ($30) {
   $31 = ((($reader)) + 24|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = ((($reader)) + 24|0);
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$4>>2]|0;
   $37 = HEAP32[$5>>2]|0;
   $38 = (_fts3TermCmp($32,$35,$36,$37)|0);
   HEAP32[$res>>2] = $38;
   $39 = HEAP32[$res>>2]|0;
   $40 = ($39|0)<(0);
   if (!($40)) {
    $41 = HEAP32[$bLeaf>>2]|0;
    $42 = ($41|0)==(0);
    $43 = HEAP32[$res>>2]|0;
    $44 = ($43|0)==(0);
    $or$cond = $42 & $44;
    if (!($or$cond)) {
     $45 = HEAP32[$3>>2]|0;
     $46 = HEAP32[$1>>2]|0;
     $47 = HEAP8[$46>>0]|0;
     $48 = $47 << 24 >> 24;
     $49 = ((($reader)) + 16|0);
     $50 = $49;
     $51 = $50;
     $52 = HEAP32[$51>>2]|0;
     $53 = (($50) + 4)|0;
     $54 = $53;
     $55 = HEAP32[$54>>2]|0;
     _fts3StartNode($45,$48,$52,$55);
     $56 = ((($reader)) + 16|0);
     $57 = $56;
     $58 = $57;
     $59 = HEAP32[$58>>2]|0;
     $60 = (($57) + 4)|0;
     $61 = $60;
     $62 = HEAP32[$61>>2]|0;
     $63 = HEAP32[$6>>2]|0;
     $64 = $63;
     $65 = $64;
     HEAP32[$65>>2] = $59;
     $66 = (($64) + 4)|0;
     $67 = $66;
     HEAP32[$67>>2] = $62;
     label = 9;
    }
   }
  } else {
   label = 9;
  }
  if ((label|0) == 9) {
   label = 0;
   $68 = HEAP32[$3>>2]|0;
   $69 = ((($reader)) + 24|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = ((($reader)) + 24|0);
   $72 = ((($71)) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = ((($reader)) + 36|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ((($reader)) + 40|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (_fts3AppendToNode($68,$prev,$70,$73,$75,$77)|0);
   HEAP32[$rc>>2] = $78;
   $79 = HEAP32[$rc>>2]|0;
   $80 = ($79|0)!=(0);
   if ($80) {
    break;
   }
  }
  $81 = (_nodeReaderNext($reader)|0);
  HEAP32[$rc>>2] = $81;
 }
 $82 = HEAP32[$3>>2]|0;
 $83 = ((($82)) + 4|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = ($84|0)==(0);
 if ($85) {
  $86 = HEAP32[$3>>2]|0;
  $87 = HEAP32[$1>>2]|0;
  $88 = HEAP8[$87>>0]|0;
  $89 = $88 << 24 >> 24;
  $90 = ((($reader)) + 16|0);
  $91 = $90;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (($91) + 4)|0;
  $95 = $94;
  $96 = HEAP32[$95>>2]|0;
  _fts3StartNode($86,$89,$93,$96);
  $97 = ((($reader)) + 16|0);
  $98 = $97;
  $99 = $98;
  $100 = HEAP32[$99>>2]|0;
  $101 = (($98) + 4)|0;
  $102 = $101;
  $103 = HEAP32[$102>>2]|0;
  $104 = HEAP32[$6>>2]|0;
  $105 = $104;
  $106 = $105;
  HEAP32[$106>>2] = $100;
  $107 = (($105) + 4)|0;
  $108 = $107;
  HEAP32[$108>>2] = $103;
 }
 _nodeReaderRelease($reader);
 $109 = HEAP32[$prev>>2]|0;
 _sqlite3_free($109);
 $110 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $110;
 $111 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($111|0);
}
function _nodeReaderInit($p,$aNode,$nNode) {
 $p = $p|0;
 $aNode = $aNode|0;
 $nNode = $nNode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $aNode;
 HEAP32[$2>>2] = $nNode;
 $3 = HEAP32[$0>>2]|0;
 dest=$3; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 $13 = HEAP32[$0>>2]|0;
 if ($12) {
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 1|0);
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 16|0);
  $18 = (_sqlite3Fts3GetVarint($15,$17)|0);
  $19 = (1 + ($18))|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 8|0);
  HEAP32[$21>>2] = $19;
  $23 = HEAP32[$0>>2]|0;
  $24 = (_nodeReaderNext($23)|0);
  STACKTOP = sp;return ($24|0);
 } else {
  $22 = ((($13)) + 8|0);
  HEAP32[$22>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  $24 = (_nodeReaderNext($23)|0);
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _fts3TermCmp($zLhs,$nLhs,$zRhs,$nRhs) {
 $zLhs = $zLhs|0;
 $nLhs = $nLhs|0;
 $zRhs = $zRhs|0;
 $nRhs = $nRhs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $nCmp = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $nCmp = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $zLhs;
 HEAP32[$1>>2] = $nLhs;
 HEAP32[$2>>2] = $zRhs;
 HEAP32[$3>>2] = $nRhs;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($4|0)<($5|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = $6 ? $7 : $8;
 HEAP32[$nCmp>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$nCmp>>2]|0;
 $13 = (_memcmp($10,$11,$12)|0);
 HEAP32[$res>>2] = $13;
 $14 = HEAP32[$res>>2]|0;
 $15 = ($14|0)==(0);
 if (!($15)) {
  $19 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = (($16) - ($17))|0;
 HEAP32[$res>>2] = $18;
 $19 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _fts3StartNode($pNode,$iHeight,$0,$1) {
 $pNode = $pNode|0;
 $iHeight = $iHeight|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$2>>2] = $pNode;
 HEAP32[$3>>2] = $iHeight;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9&255;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP8[$12>>0] = $10;
 $13 = $4;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($15|0)!=(0);
 $20 = ($18|0)!=(0);
 $21 = $19 | $20;
 $22 = HEAP32[$2>>2]|0;
 if ($21) {
  $23 = HEAP32[$22>>2]|0;
  $24 = ((($23)) + 1|0);
  $25 = $4;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3Fts3PutVarint($24,$27,$30)|0);
  $32 = (1 + ($31))|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = ((($33)) + 4|0);
  HEAP32[$34>>2] = $32;
  STACKTOP = sp;return;
 } else {
  $35 = ((($22)) + 4|0);
  HEAP32[$35>>2] = 1;
  STACKTOP = sp;return;
 }
}
function _fts3AppendToNode($pNode,$pPrev,$zTerm,$nTerm,$aDoclist,$nDoclist) {
 $pNode = $pNode|0;
 $pPrev = $pPrev|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $aDoclist = $aDoclist|0;
 $nDoclist = $nDoclist|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bFirst = 0, $nPrefix = 0, $nSuffix = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $rc = sp + 12|0;
 $bFirst = sp + 8|0;
 $nPrefix = sp + 4|0;
 $nSuffix = sp;
 HEAP32[$1>>2] = $pNode;
 HEAP32[$2>>2] = $pPrev;
 HEAP32[$3>>2] = $zTerm;
 HEAP32[$4>>2] = $nTerm;
 HEAP32[$5>>2] = $aDoclist;
 HEAP32[$6>>2] = $nDoclist;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 $11 = $10&1;
 HEAP32[$bFirst>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 _blobGrowBuffer($12,$13,$rc);
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $16;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = HEAP32[$4>>2]|0;
 $24 = (_fts3PrefixCompress($18,$21,$22,$23)|0);
 HEAP32[$nPrefix>>2] = $24;
 $25 = HEAP32[$4>>2]|0;
 $26 = HEAP32[$nPrefix>>2]|0;
 $27 = (($25) - ($26))|0;
 HEAP32[$nSuffix>>2] = $27;
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$3>>2]|0;
 $31 = HEAP32[$4>>2]|0;
 _memcpy(($29|0),($30|0),($31|0))|0;
 $32 = HEAP32[$4>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 4|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$bFirst>>2]|0;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + ($39)|0);
  $43 = HEAP32[$nPrefix>>2]|0;
  $44 = ($43|0)<(0);
  $45 = $44 << 31 >> 31;
  $46 = (_sqlite3Fts3PutVarint($42,$43,$45)|0);
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 4|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($46))|0;
  HEAP32[$48>>2] = $50;
 }
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 4|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$1>>2]|0;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + ($53)|0);
 $57 = HEAP32[$nSuffix>>2]|0;
 $58 = ($57|0)<(0);
 $59 = $58 << 31 >> 31;
 $60 = (_sqlite3Fts3PutVarint($56,$57,$59)|0);
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 4|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = (($63) + ($60))|0;
 HEAP32[$62>>2] = $64;
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 4|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$1>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($69) + ($67)|0);
 $71 = HEAP32[$nPrefix>>2]|0;
 $72 = HEAP32[$3>>2]|0;
 $73 = (($72) + ($71)|0);
 $74 = HEAP32[$nSuffix>>2]|0;
 _memcpy(($70|0),($73|0),($74|0))|0;
 $75 = HEAP32[$nSuffix>>2]|0;
 $76 = HEAP32[$1>>2]|0;
 $77 = ((($76)) + 4|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = (($78) + ($75))|0;
 HEAP32[$77>>2] = $79;
 $80 = HEAP32[$5>>2]|0;
 $81 = ($80|0)!=(0|0);
 if ($81) {
  $82 = HEAP32[$1>>2]|0;
  $83 = ((($82)) + 4|0);
  $84 = HEAP32[$83>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = HEAP32[$85>>2]|0;
  $87 = (($86) + ($84)|0);
  $88 = HEAP32[$6>>2]|0;
  $89 = ($88|0)<(0);
  $90 = $89 << 31 >> 31;
  $91 = (_sqlite3Fts3PutVarint($87,$88,$90)|0);
  $92 = HEAP32[$1>>2]|0;
  $93 = ((($92)) + 4|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (($94) + ($91))|0;
  HEAP32[$93>>2] = $95;
  $96 = HEAP32[$1>>2]|0;
  $97 = ((($96)) + 4|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = HEAP32[$1>>2]|0;
  $100 = HEAP32[$99>>2]|0;
  $101 = (($100) + ($98)|0);
  $102 = HEAP32[$5>>2]|0;
  $103 = HEAP32[$6>>2]|0;
  _memcpy(($101|0),($102|0),($103|0))|0;
  $104 = HEAP32[$6>>2]|0;
  $105 = HEAP32[$1>>2]|0;
  $106 = ((($105)) + 4|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($107) + ($104))|0;
  HEAP32[$106>>2] = $108;
 }
 HEAP32[$0>>2] = 0;
 $109 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($109|0);
}
function _nodeReaderNext($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bFirst = 0, $nPrefix = 0, $nSuffix = 0;
 var $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $bFirst = sp + 12|0;
 $nPrefix = sp + 8|0;
 $nSuffix = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 $6 = $5&1;
 HEAP32[$bFirst>>2] = $6;
 HEAP32[$nPrefix>>2] = 0;
 HEAP32[$nSuffix>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)!=(0);
 $16 = ($14|0)!=(0);
 $17 = $15 | $16;
 $18 = HEAP32[$bFirst>>2]|0;
 $19 = ($18|0)==(0);
 $or$cond = $17 & $19;
 if ($or$cond) {
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = $21;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (_i64Add(($24|0),($27|0),1,0)|0);
  $29 = tempRet0;
  $30 = $21;
  $31 = $30;
  HEAP32[$31>>2] = $28;
  $32 = (($30) + 4)|0;
  $33 = $32;
  HEAP32[$33>>2] = $29;
 }
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = ($36|0)>=($39|0);
 if ($40) {
  $41 = HEAP32[$0>>2]|0;
  HEAP32[$41>>2] = 0;
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $42 = HEAP32[$bFirst>>2]|0;
 $43 = ($42|0)==(0);
 if ($43) {
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($46)|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 & 128;
  $53 = ($52|0)!=(0);
  $54 = HEAP32[$0>>2]|0;
  $55 = ((($54)) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$0>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($56)|0);
  if ($53) {
   $60 = (_sqlite3Fts3GetVarint32($59,$nPrefix)|0);
   $67 = $60;
  } else {
   $61 = HEAP8[$59>>0]|0;
   $62 = $61&255;
   HEAP32[$nPrefix>>2] = $62;
   $67 = 1;
  }
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 8|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + ($67))|0;
  HEAP32[$64>>2] = $66;
 }
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 8|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$0>>2]|0;
 $72 = HEAP32[$71>>2]|0;
 $73 = (($72) + ($70)|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 $76 = $75 & 128;
 $77 = ($76|0)!=(0);
 $78 = HEAP32[$0>>2]|0;
 $79 = ((($78)) + 8|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = HEAP32[$0>>2]|0;
 $82 = HEAP32[$81>>2]|0;
 $83 = (($82) + ($80)|0);
 if ($77) {
  $84 = (_sqlite3Fts3GetVarint32($83,$nSuffix)|0);
  $91 = $84;
 } else {
  $85 = HEAP8[$83>>0]|0;
  $86 = $85&255;
  HEAP32[$nSuffix>>2] = $86;
  $91 = 1;
 }
 $87 = HEAP32[$0>>2]|0;
 $88 = ((($87)) + 8|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + ($91))|0;
 HEAP32[$88>>2] = $90;
 $92 = HEAP32[$0>>2]|0;
 $93 = ((($92)) + 24|0);
 $94 = HEAP32[$nPrefix>>2]|0;
 $95 = HEAP32[$nSuffix>>2]|0;
 $96 = (($94) + ($95))|0;
 _blobGrowBuffer($93,$96,$rc);
 $97 = HEAP32[$rc>>2]|0;
 $98 = ($97|0)==(0);
 if (!($98)) {
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $99 = HEAP32[$nPrefix>>2]|0;
 $100 = HEAP32[$0>>2]|0;
 $101 = ((($100)) + 24|0);
 $102 = HEAP32[$101>>2]|0;
 $103 = (($102) + ($99)|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = ((($104)) + 8|0);
 $106 = HEAP32[$105>>2]|0;
 $107 = HEAP32[$0>>2]|0;
 $108 = HEAP32[$107>>2]|0;
 $109 = (($108) + ($106)|0);
 $110 = HEAP32[$nSuffix>>2]|0;
 _memcpy(($103|0),($109|0),($110|0))|0;
 $111 = HEAP32[$nPrefix>>2]|0;
 $112 = HEAP32[$nSuffix>>2]|0;
 $113 = (($111) + ($112))|0;
 $114 = HEAP32[$0>>2]|0;
 $115 = ((($114)) + 24|0);
 $116 = ((($115)) + 4|0);
 HEAP32[$116>>2] = $113;
 $117 = HEAP32[$nSuffix>>2]|0;
 $118 = HEAP32[$0>>2]|0;
 $119 = ((($118)) + 8|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = (($120) + ($117))|0;
 HEAP32[$119>>2] = $121;
 $122 = HEAP32[$0>>2]|0;
 $123 = ((($122)) + 16|0);
 $124 = $123;
 $125 = $124;
 $126 = HEAP32[$125>>2]|0;
 $127 = (($124) + 4)|0;
 $128 = $127;
 $129 = HEAP32[$128>>2]|0;
 $130 = ($126|0)==(0);
 $131 = ($129|0)==(0);
 $132 = $130 & $131;
 if (!($132)) {
  $176 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 $133 = HEAP32[$0>>2]|0;
 $134 = ((($133)) + 8|0);
 $135 = HEAP32[$134>>2]|0;
 $136 = HEAP32[$0>>2]|0;
 $137 = HEAP32[$136>>2]|0;
 $138 = (($137) + ($135)|0);
 $139 = HEAP8[$138>>0]|0;
 $140 = $139&255;
 $141 = $140 & 128;
 $142 = ($141|0)!=(0);
 $143 = HEAP32[$0>>2]|0;
 $144 = ((($143)) + 8|0);
 $145 = HEAP32[$144>>2]|0;
 $146 = HEAP32[$0>>2]|0;
 $147 = HEAP32[$146>>2]|0;
 $148 = (($147) + ($145)|0);
 if ($142) {
  $149 = HEAP32[$0>>2]|0;
  $150 = ((($149)) + 40|0);
  $151 = (_sqlite3Fts3GetVarint32($148,$150)|0);
  $160 = $151;
 } else {
  $152 = HEAP8[$148>>0]|0;
  $153 = $152&255;
  $154 = HEAP32[$0>>2]|0;
  $155 = ((($154)) + 40|0);
  HEAP32[$155>>2] = $153;
  $160 = 1;
 }
 $156 = HEAP32[$0>>2]|0;
 $157 = ((($156)) + 8|0);
 $158 = HEAP32[$157>>2]|0;
 $159 = (($158) + ($160))|0;
 HEAP32[$157>>2] = $159;
 $161 = HEAP32[$0>>2]|0;
 $162 = ((($161)) + 8|0);
 $163 = HEAP32[$162>>2]|0;
 $164 = HEAP32[$0>>2]|0;
 $165 = HEAP32[$164>>2]|0;
 $166 = (($165) + ($163)|0);
 $167 = HEAP32[$0>>2]|0;
 $168 = ((($167)) + 36|0);
 HEAP32[$168>>2] = $166;
 $169 = HEAP32[$0>>2]|0;
 $170 = ((($169)) + 40|0);
 $171 = HEAP32[$170>>2]|0;
 $172 = HEAP32[$0>>2]|0;
 $173 = ((($172)) + 8|0);
 $174 = HEAP32[$173>>2]|0;
 $175 = (($174) + ($171))|0;
 HEAP32[$173>>2] = $175;
 $176 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($176|0);
}
function _nodeReaderRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 STACKTOP = sp;return;
}
function _fts3IncrmergePush($p,$pWriter,$zTerm,$nTerm) {
 $p = $p|0;
 $pWriter = $pWriter|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iLayer = 0, $iNextPtr = 0, $iPtr = 0, $nPrefix = 0, $nSpace = 0, $nSuffix = 0;
 var $or$cond = 0, $pBlk = 0, $pNode = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $iPtr = sp + 8|0;
 $iLayer = sp + 40|0;
 $iNextPtr = sp;
 $pNode = sp + 36|0;
 $rc = sp + 32|0;
 $nPrefix = sp + 28|0;
 $nSuffix = sp + 24|0;
 $nSpace = sp + 20|0;
 $pBlk = sp + 16|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pWriter;
 HEAP32[$3>>2] = $zTerm;
 HEAP32[$4>>2] = $nTerm;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $iPtr;
 $14 = $13;
 HEAP32[$14>>2] = $9;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = $12;
 HEAP32[$iLayer>>2] = 1;
 while(1) {
  $17 = HEAP32[$iLayer>>2]|0;
  $18 = ($17|0)<(16);
  if (!($18)) {
   label = 16;
   break;
  }
  $19 = $iNextPtr;
  $20 = $19;
  HEAP32[$20>>2] = 0;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = 0;
  $23 = HEAP32[$iLayer>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = ((($24)) + 56|0);
  $26 = (($25) + ($23<<5)|0);
  HEAP32[$pNode>>2] = $26;
  HEAP32[$rc>>2] = 0;
  $27 = HEAP32[$pNode>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pNode>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = ((($31)) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  $36 = (_fts3PrefixCompress($29,$33,$34,$35)|0);
  HEAP32[$nPrefix>>2] = $36;
  $37 = HEAP32[$4>>2]|0;
  $38 = HEAP32[$nPrefix>>2]|0;
  $39 = (($37) - ($38))|0;
  HEAP32[$nSuffix>>2] = $39;
  $40 = HEAP32[$nPrefix>>2]|0;
  $41 = ($40|0)<(0);
  $42 = $41 << 31 >> 31;
  $43 = (_sqlite3Fts3VarintLen($40,$42)|0);
  HEAP32[$nSpace>>2] = $43;
  $44 = HEAP32[$nSuffix>>2]|0;
  $45 = ($44|0)<(0);
  $46 = $45 << 31 >> 31;
  $47 = (_sqlite3Fts3VarintLen($44,$46)|0);
  $48 = HEAP32[$nSuffix>>2]|0;
  $49 = (($47) + ($48))|0;
  $50 = HEAP32[$nSpace>>2]|0;
  $51 = (($50) + ($49))|0;
  HEAP32[$nSpace>>2] = $51;
  $52 = HEAP32[$pNode>>2]|0;
  $53 = ((($52)) + 8|0);
  $54 = ((($53)) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)==(0);
  if ($56) {
   label = 5;
  } else {
   $57 = HEAP32[$pNode>>2]|0;
   $58 = ((($57)) + 20|0);
   $59 = ((($58)) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$nSpace>>2]|0;
   $62 = (($60) + ($61))|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 224|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = ($62|0)<=($65|0);
   if ($66) {
    label = 5;
   } else {
    $164 = HEAP32[$1>>2]|0;
    $165 = HEAP32[$pNode>>2]|0;
    $166 = $165;
    $167 = $166;
    $168 = HEAP32[$167>>2]|0;
    $169 = (($166) + 4)|0;
    $170 = $169;
    $171 = HEAP32[$170>>2]|0;
    $172 = HEAP32[$pNode>>2]|0;
    $173 = ((($172)) + 20|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = HEAP32[$pNode>>2]|0;
    $176 = ((($175)) + 20|0);
    $177 = ((($176)) + 4|0);
    $178 = HEAP32[$177>>2]|0;
    $179 = (_fts3WriteSegment($164,$168,$171,$174,$178)|0);
    HEAP32[$rc>>2] = $179;
    $180 = HEAP32[$iLayer>>2]|0;
    $181 = $180&255;
    $182 = HEAP32[$pNode>>2]|0;
    $183 = ((($182)) + 20|0);
    $184 = HEAP32[$183>>2]|0;
    HEAP8[$184>>0] = $181;
    $185 = HEAP32[$pNode>>2]|0;
    $186 = ((($185)) + 20|0);
    $187 = HEAP32[$186>>2]|0;
    $188 = ((($187)) + 1|0);
    $189 = $iPtr;
    $190 = $189;
    $191 = HEAP32[$190>>2]|0;
    $192 = (($189) + 4)|0;
    $193 = $192;
    $194 = HEAP32[$193>>2]|0;
    $195 = (_i64Add(($191|0),($194|0),1,0)|0);
    $196 = tempRet0;
    $197 = (_sqlite3Fts3PutVarint($188,$195,$196)|0);
    $198 = (1 + ($197))|0;
    $199 = HEAP32[$pNode>>2]|0;
    $200 = ((($199)) + 20|0);
    $201 = ((($200)) + 4|0);
    HEAP32[$201>>2] = $198;
    $202 = HEAP32[$pNode>>2]|0;
    $203 = $202;
    $204 = $203;
    $205 = HEAP32[$204>>2]|0;
    $206 = (($203) + 4)|0;
    $207 = $206;
    $208 = HEAP32[$207>>2]|0;
    $209 = $iNextPtr;
    $210 = $209;
    HEAP32[$210>>2] = $205;
    $211 = (($209) + 4)|0;
    $212 = $211;
    HEAP32[$212>>2] = $208;
    $213 = HEAP32[$pNode>>2]|0;
    $214 = $213;
    $215 = $214;
    $216 = HEAP32[$215>>2]|0;
    $217 = (($214) + 4)|0;
    $218 = $217;
    $219 = HEAP32[$218>>2]|0;
    $220 = (_i64Add(($216|0),($219|0),1,0)|0);
    $221 = tempRet0;
    $222 = $213;
    $223 = $222;
    HEAP32[$223>>2] = $220;
    $224 = (($222) + 4)|0;
    $225 = $224;
    HEAP32[$225>>2] = $221;
    $226 = HEAP32[$pNode>>2]|0;
    $227 = ((($226)) + 8|0);
    $228 = ((($227)) + 4|0);
    HEAP32[$228>>2] = 0;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $67 = HEAP32[$pNode>>2]|0;
   $68 = ((($67)) + 20|0);
   HEAP32[$pBlk>>2] = $68;
   $69 = HEAP32[$pBlk>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)==(0);
   if ($72) {
    $73 = HEAP32[$pBlk>>2]|0;
    $74 = HEAP32[$1>>2]|0;
    $75 = ((($74)) + 224|0);
    $76 = HEAP32[$75>>2]|0;
    _blobGrowBuffer($73,$76,$rc);
    $77 = HEAP32[$rc>>2]|0;
    $78 = ($77|0)==(0);
    if ($78) {
     $79 = HEAP32[$iLayer>>2]|0;
     $80 = $79&255;
     $81 = HEAP32[$pBlk>>2]|0;
     $82 = HEAP32[$81>>2]|0;
     HEAP8[$82>>0] = $80;
     $83 = HEAP32[$pBlk>>2]|0;
     $84 = HEAP32[$83>>2]|0;
     $85 = ((($84)) + 1|0);
     $86 = $iPtr;
     $87 = $86;
     $88 = HEAP32[$87>>2]|0;
     $89 = (($86) + 4)|0;
     $90 = $89;
     $91 = HEAP32[$90>>2]|0;
     $92 = (_sqlite3Fts3PutVarint($85,$88,$91)|0);
     $93 = (1 + ($92))|0;
     $94 = HEAP32[$pBlk>>2]|0;
     $95 = ((($94)) + 4|0);
     HEAP32[$95>>2] = $93;
    }
   }
   $96 = HEAP32[$pBlk>>2]|0;
   $97 = HEAP32[$pBlk>>2]|0;
   $98 = ((($97)) + 4|0);
   $99 = HEAP32[$98>>2]|0;
   $100 = HEAP32[$nSpace>>2]|0;
   $101 = (($99) + ($100))|0;
   _blobGrowBuffer($96,$101,$rc);
   $102 = HEAP32[$pNode>>2]|0;
   $103 = ((($102)) + 8|0);
   $104 = HEAP32[$4>>2]|0;
   _blobGrowBuffer($103,$104,$rc);
   $105 = HEAP32[$rc>>2]|0;
   $106 = ($105|0)==(0);
   if ($106) {
    $107 = HEAP32[$pNode>>2]|0;
    $108 = ((($107)) + 8|0);
    $109 = ((($108)) + 4|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = ($110|0)!=(0);
    if ($111) {
     $112 = HEAP32[$pBlk>>2]|0;
     $113 = ((($112)) + 4|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = HEAP32[$pBlk>>2]|0;
     $116 = HEAP32[$115>>2]|0;
     $117 = (($116) + ($114)|0);
     $118 = HEAP32[$nPrefix>>2]|0;
     $119 = ($118|0)<(0);
     $120 = $119 << 31 >> 31;
     $121 = (_sqlite3Fts3PutVarint($117,$118,$120)|0);
     $122 = HEAP32[$pBlk>>2]|0;
     $123 = ((($122)) + 4|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = (($124) + ($121))|0;
     HEAP32[$123>>2] = $125;
    }
    $126 = HEAP32[$pBlk>>2]|0;
    $127 = ((($126)) + 4|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$pBlk>>2]|0;
    $130 = HEAP32[$129>>2]|0;
    $131 = (($130) + ($128)|0);
    $132 = HEAP32[$nSuffix>>2]|0;
    $133 = ($132|0)<(0);
    $134 = $133 << 31 >> 31;
    $135 = (_sqlite3Fts3PutVarint($131,$132,$134)|0);
    $136 = HEAP32[$pBlk>>2]|0;
    $137 = ((($136)) + 4|0);
    $138 = HEAP32[$137>>2]|0;
    $139 = (($138) + ($135))|0;
    HEAP32[$137>>2] = $139;
    $140 = HEAP32[$pBlk>>2]|0;
    $141 = ((($140)) + 4|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = HEAP32[$pBlk>>2]|0;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($144) + ($142)|0);
    $146 = HEAP32[$nPrefix>>2]|0;
    $147 = HEAP32[$3>>2]|0;
    $148 = (($147) + ($146)|0);
    $149 = HEAP32[$nSuffix>>2]|0;
    _memcpy(($145|0),($148|0),($149|0))|0;
    $150 = HEAP32[$nSuffix>>2]|0;
    $151 = HEAP32[$pBlk>>2]|0;
    $152 = ((($151)) + 4|0);
    $153 = HEAP32[$152>>2]|0;
    $154 = (($153) + ($150))|0;
    HEAP32[$152>>2] = $154;
    $155 = HEAP32[$pNode>>2]|0;
    $156 = ((($155)) + 8|0);
    $157 = HEAP32[$156>>2]|0;
    $158 = HEAP32[$3>>2]|0;
    $159 = HEAP32[$4>>2]|0;
    _memcpy(($157|0),($158|0),($159|0))|0;
    $160 = HEAP32[$4>>2]|0;
    $161 = HEAP32[$pNode>>2]|0;
    $162 = ((($161)) + 8|0);
    $163 = ((($162)) + 4|0);
    HEAP32[$163>>2] = $160;
   }
  }
  $229 = HEAP32[$rc>>2]|0;
  $230 = ($229|0)!=(0);
  $231 = $iNextPtr;
  $232 = $231;
  $233 = HEAP32[$232>>2]|0;
  $234 = (($231) + 4)|0;
  $235 = $234;
  $236 = HEAP32[$235>>2]|0;
  $237 = ($233|0)==(0);
  $238 = ($236|0)==(0);
  $239 = $237 & $238;
  $or$cond = $230 | $239;
  if ($or$cond) {
   label = 14;
   break;
  }
  $241 = $iNextPtr;
  $242 = $241;
  $243 = HEAP32[$242>>2]|0;
  $244 = (($241) + 4)|0;
  $245 = $244;
  $246 = HEAP32[$245>>2]|0;
  $247 = $iPtr;
  $248 = $247;
  HEAP32[$248>>2] = $243;
  $249 = (($247) + 4)|0;
  $250 = $249;
  HEAP32[$250>>2] = $246;
  $251 = HEAP32[$iLayer>>2]|0;
  $252 = (($251) + 1)|0;
  HEAP32[$iLayer>>2] = $252;
 }
 if ((label|0) == 14) {
  $240 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $240;
  $253 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($253|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $253 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($253|0);
 }
 return (0)|0;
}
function _fts3IsAppendable($p,$0,$1,$pbRes) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 $pbRes = $pbRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bRes = 0, $pCheck = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp;
 $4 = sp + 20|0;
 $bRes = sp + 16|0;
 $pCheck = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pbRes;
 HEAP32[$bRes>>2] = 0;
 HEAP32[$pCheck>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_fts3SqlStmt($9,34,$pCheck,0)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  $25 = HEAP32[$bRes>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  HEAP32[$26>>2] = $25;
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $13 = HEAP32[$pCheck>>2]|0;
 $14 = $3;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 (_sqlite3_bind_int64($13,1,$16,$19)|0);
 $20 = HEAP32[$pCheck>>2]|0;
 $21 = (_sqlite3_step($20)|0);
 $22 = (100)==($21|0);
 if ($22) {
  HEAP32[$bRes>>2] = 1;
 }
 $23 = HEAP32[$pCheck>>2]|0;
 $24 = (_sqlite3_reset($23)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$bRes>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3Fts3UpdateMethod($pVtab,$nArg,$apVal,$pRowid) {
 $pVtab = $pVtab|0;
 $nArg = $nArg|0;
 $apVal = $apVal|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aSzDel = 0, $aSzIns = 0, $bInsertDone = 0, $iLangid = 0, $isRemove = 0, $nChng = 0, $or$cond = 0, $p = 0, $pNewRowid = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $p = sp + 32|0;
 $rc = sp + 28|0;
 $isRemove = sp + 24|0;
 $aSzIns = sp + 20|0;
 $aSzDel = sp + 16|0;
 $nChng = sp + 12|0;
 $bInsertDone = sp + 8|0;
 $pNewRowid = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $pVtab;
 HEAP32[$1>>2] = $nArg;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$3>>2] = $pRowid;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $4;
 HEAP32[$rc>>2] = 0;
 HEAP32[$isRemove>>2] = 0;
 HEAP32[$aSzIns>>2] = 0;
 HEAP32[$aSzDel>>2] = 0;
 HEAP32[$nChng>>2] = 0;
 HEAP32[$bInsertDone>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)>(1);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_type($8)|0);
  $10 = ($9|0)==(5);
  if ($10) {
   $11 = HEAP32[$p>>2]|0;
   $12 = ((($11)) + 24|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (($13) + 2)|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = (($15) + ($14<<2)|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3_value_type($17)|0);
   $19 = ($18|0)!=(5);
   if ($19) {
    $20 = HEAP32[$p>>2]|0;
    $21 = HEAP32[$p>>2]|0;
    $22 = ((($21)) + 24|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = (($23) + 2)|0;
    $25 = HEAP32[$2>>2]|0;
    $26 = (($25) + ($24<<2)|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (_fts3SpecialInsert($20,$27)|0);
    HEAP32[$rc>>2] = $28;
    $206 = HEAP32[$aSzDel>>2]|0;
    _sqlite3_free($206);
    $207 = HEAP32[$p>>2]|0;
    _sqlite3Fts3SegmentsClose($207);
    $208 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($208|0);
   }
  }
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = ($29|0)>(1);
 if ($30) {
  $31 = HEAP32[$p>>2]|0;
  $32 = ((($31)) + 24|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (2 + ($33))|0;
  $35 = (($34) + 2)|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = (($36) + ($35<<2)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (_sqlite3_value_int($38)|0);
  $40 = ($39|0)<(0);
  if ($40) {
   HEAP32[$rc>>2] = 19;
   $206 = HEAP32[$aSzDel>>2]|0;
   _sqlite3_free($206);
   $207 = HEAP32[$p>>2]|0;
   _sqlite3Fts3SegmentsClose($207);
   $208 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($208|0);
  }
 }
 $41 = HEAP32[$p>>2]|0;
 $42 = ((($41)) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($43) + 1)|0;
 $45 = $44<<2;
 $46 = $45<<1;
 $47 = (_sqlite3_malloc($46)|0);
 HEAP32[$aSzDel>>2] = $47;
 $48 = HEAP32[$aSzDel>>2]|0;
 $49 = ($48|0)==(0|0);
 if ($49) {
  HEAP32[$rc>>2] = 7;
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $50 = HEAP32[$p>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($52) + 1)|0;
 $54 = HEAP32[$aSzDel>>2]|0;
 $55 = (($54) + ($53<<2)|0);
 HEAP32[$aSzIns>>2] = $55;
 $56 = HEAP32[$aSzDel>>2]|0;
 $57 = HEAP32[$p>>2]|0;
 $58 = ((($57)) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = (($59) + 1)|0;
 $61 = $60<<2;
 $62 = $61<<1;
 _memset(($56|0),0,($62|0))|0;
 $63 = HEAP32[$p>>2]|0;
 $64 = (_fts3Writelock($63)|0);
 HEAP32[$rc>>2] = $64;
 $65 = HEAP32[$rc>>2]|0;
 $66 = ($65|0)!=(0);
 if ($66) {
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $67 = HEAP32[$1>>2]|0;
 $68 = ($67|0)>(1);
 do {
  if ($68) {
   $69 = HEAP32[$p>>2]|0;
   $70 = ((($69)) + 40|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)==(0|0);
   if ($72) {
    $73 = HEAP32[$p>>2]|0;
    $74 = ((($73)) + 24|0);
    $75 = HEAP32[$74>>2]|0;
    $76 = (3 + ($75))|0;
    $77 = HEAP32[$2>>2]|0;
    $78 = (($77) + ($76<<2)|0);
    $79 = HEAP32[$78>>2]|0;
    HEAP32[$pNewRowid>>2] = $79;
    $80 = HEAP32[$pNewRowid>>2]|0;
    $81 = (_sqlite3_value_type($80)|0);
    $82 = ($81|0)==(5);
    if ($82) {
     $83 = HEAP32[$2>>2]|0;
     $84 = ((($83)) + 4|0);
     $85 = HEAP32[$84>>2]|0;
     HEAP32[$pNewRowid>>2] = $85;
    }
    $86 = HEAP32[$pNewRowid>>2]|0;
    $87 = (_sqlite3_value_type($86)|0);
    $88 = ($87|0)!=(5);
    if ($88) {
     $89 = HEAP32[$2>>2]|0;
     $90 = HEAP32[$89>>2]|0;
     $91 = (_sqlite3_value_type($90)|0);
     $92 = ($91|0)==(5);
     if (!($92)) {
      $93 = HEAP32[$2>>2]|0;
      $94 = HEAP32[$93>>2]|0;
      $95 = (_sqlite3_value_int64($94)|0);
      $96 = tempRet0;
      $97 = HEAP32[$pNewRowid>>2]|0;
      $98 = (_sqlite3_value_int64($97)|0);
      $99 = tempRet0;
      $100 = ($95|0)!=($98|0);
      $101 = ($96|0)!=($99|0);
      $102 = $100 | $101;
      if (!($102)) {
       break;
      }
     }
     $103 = HEAP32[$p>>2]|0;
     $104 = ((($103)) + 12|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (_sqlite3_vtab_on_conflict($105)|0);
     $107 = ($106|0)==(5);
     $108 = HEAP32[$p>>2]|0;
     if ($107) {
      $109 = HEAP32[$pNewRowid>>2]|0;
      $110 = HEAP32[$aSzDel>>2]|0;
      $111 = (_fts3DeleteByRowid($108,$109,$nChng,$110)|0);
      HEAP32[$rc>>2] = $111;
      break;
     } else {
      $112 = HEAP32[$2>>2]|0;
      $113 = HEAP32[$3>>2]|0;
      $114 = (_fts3InsertData($108,$112,$113)|0);
      HEAP32[$rc>>2] = $114;
      HEAP32[$bInsertDone>>2] = 1;
      break;
     }
    }
   }
  }
 } while(0);
 $115 = HEAP32[$rc>>2]|0;
 $116 = ($115|0)!=(0);
 if ($116) {
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $117 = HEAP32[$2>>2]|0;
 $118 = HEAP32[$117>>2]|0;
 $119 = (_sqlite3_value_type($118)|0);
 $120 = ($119|0)!=(5);
 if ($120) {
  $121 = HEAP32[$p>>2]|0;
  $122 = HEAP32[$2>>2]|0;
  $123 = HEAP32[$122>>2]|0;
  $124 = HEAP32[$aSzDel>>2]|0;
  $125 = (_fts3DeleteByRowid($121,$123,$nChng,$124)|0);
  HEAP32[$rc>>2] = $125;
  HEAP32[$isRemove>>2] = 1;
 }
 $126 = HEAP32[$1>>2]|0;
 $127 = ($126|0)>(1);
 $128 = HEAP32[$rc>>2]|0;
 $129 = ($128|0)==(0);
 $or$cond = $127 & $129;
 if ($or$cond) {
  $130 = HEAP32[$p>>2]|0;
  $131 = ((($130)) + 24|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = (2 + ($132))|0;
  $134 = (($133) + 2)|0;
  $135 = HEAP32[$2>>2]|0;
  $136 = (($135) + ($134<<2)|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = (_sqlite3_value_int($137)|0);
  HEAP32[$iLangid>>2] = $138;
  $139 = HEAP32[$bInsertDone>>2]|0;
  $140 = ($139|0)==(0);
  if ($140) {
   $141 = HEAP32[$p>>2]|0;
   $142 = HEAP32[$2>>2]|0;
   $143 = HEAP32[$3>>2]|0;
   $144 = (_fts3InsertData($141,$142,$143)|0);
   HEAP32[$rc>>2] = $144;
   $145 = HEAP32[$rc>>2]|0;
   $146 = ($145|0)==(19);
   if ($146) {
    $147 = HEAP32[$p>>2]|0;
    $148 = ((($147)) + 40|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = ($149|0)==(0|0);
    if ($150) {
     HEAP32[$rc>>2] = 267;
    }
   }
  }
  $151 = HEAP32[$rc>>2]|0;
  $152 = ($151|0)==(0);
  do {
   if ($152) {
    $153 = HEAP32[$isRemove>>2]|0;
    $154 = ($153|0)!=(0);
    if ($154) {
     $155 = HEAP32[$3>>2]|0;
     $156 = $155;
     $157 = $156;
     $158 = HEAP32[$157>>2]|0;
     $159 = (($156) + 4)|0;
     $160 = $159;
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$p>>2]|0;
     $163 = ((($162)) + 264|0);
     $164 = $163;
     $165 = $164;
     $166 = HEAP32[$165>>2]|0;
     $167 = (($164) + 4)|0;
     $168 = $167;
     $169 = HEAP32[$168>>2]|0;
     $170 = ($158|0)!=($166|0);
     $171 = ($161|0)!=($169|0);
     $172 = $170 | $171;
     if (!($172)) {
      break;
     }
    }
    $173 = HEAP32[$p>>2]|0;
    $174 = HEAP32[$iLangid>>2]|0;
    $175 = HEAP32[$3>>2]|0;
    $176 = $175;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (($176) + 4)|0;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = (_fts3PendingTermsDocid($173,0,$174,$178,$181)|0);
    HEAP32[$rc>>2] = $182;
   }
  } while(0);
  $183 = HEAP32[$rc>>2]|0;
  $184 = ($183|0)==(0);
  if ($184) {
   $185 = HEAP32[$p>>2]|0;
   $186 = HEAP32[$iLangid>>2]|0;
   $187 = HEAP32[$2>>2]|0;
   $188 = HEAP32[$aSzIns>>2]|0;
   $189 = (_fts3InsertTerms($185,$186,$187,$188)|0);
   HEAP32[$rc>>2] = $189;
  }
  $190 = HEAP32[$p>>2]|0;
  $191 = ((($190)) + 230|0);
  $192 = HEAP8[$191>>0]|0;
  $193 = ($192<<24>>24)!=(0);
  if ($193) {
   $194 = HEAP32[$p>>2]|0;
   $195 = HEAP32[$aSzIns>>2]|0;
   _fts3InsertDocsize($rc,$194,$195);
  }
  $196 = HEAP32[$nChng>>2]|0;
  $197 = (($196) + 1)|0;
  HEAP32[$nChng>>2] = $197;
 }
 $198 = HEAP32[$p>>2]|0;
 $199 = ((($198)) + 228|0);
 $200 = HEAP8[$199>>0]|0;
 $201 = ($200<<24>>24)!=(0);
 if (!($201)) {
  $206 = HEAP32[$aSzDel>>2]|0;
  _sqlite3_free($206);
  $207 = HEAP32[$p>>2]|0;
  _sqlite3Fts3SegmentsClose($207);
  $208 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($208|0);
 }
 $202 = HEAP32[$p>>2]|0;
 $203 = HEAP32[$aSzIns>>2]|0;
 $204 = HEAP32[$aSzDel>>2]|0;
 $205 = HEAP32[$nChng>>2]|0;
 _fts3UpdateDocTotals($rc,$202,$203,$204,$205);
 $206 = HEAP32[$aSzDel>>2]|0;
 _sqlite3_free($206);
 $207 = HEAP32[$p>>2]|0;
 _sqlite3Fts3SegmentsClose($207);
 $208 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($208|0);
}
function _fts3SpecialInsert($p,$pVal) {
 $p = $p|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nVal = 0, $rc = 0, $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $zVal = sp + 4|0;
 $nVal = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pVal;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3_value_text($3)|0);
 HEAP32[$zVal>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3_value_bytes($5)|0);
 HEAP32[$nVal>>2] = $6;
 $7 = HEAP32[$zVal>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  HEAP32[$0>>2] = 7;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $9 = HEAP32[$nVal>>2]|0;
 $10 = ($9|0)==(8);
 if ($10) {
  $11 = HEAP32[$zVal>>2]|0;
  $12 = (_sqlite3_strnicmp($11,20512,8)|0);
  $13 = (0)==($12|0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = (_fts3DoOptimize($14,0)|0);
   HEAP32[$rc>>2] = $15;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 do {
  if ((label|0) == 6) {
   $16 = HEAP32[$nVal>>2]|0;
   $17 = ($16|0)==(7);
   if ($17) {
    $18 = HEAP32[$zVal>>2]|0;
    $19 = (_sqlite3_strnicmp($18,23495,7)|0);
    $20 = (0)==($19|0);
    if ($20) {
     $21 = HEAP32[$1>>2]|0;
     $22 = (_fts3DoRebuild($21)|0);
     HEAP32[$rc>>2] = $22;
     break;
    }
   }
   $23 = HEAP32[$nVal>>2]|0;
   $24 = ($23|0)==(15);
   if ($24) {
    $25 = HEAP32[$zVal>>2]|0;
    $26 = (_sqlite3_strnicmp($25,23503,15)|0);
    $27 = (0)==($26|0);
    if ($27) {
     $28 = HEAP32[$1>>2]|0;
     $29 = (_fts3DoIntegrityCheck($28)|0);
     HEAP32[$rc>>2] = $29;
     break;
    }
   }
   $30 = HEAP32[$nVal>>2]|0;
   $31 = ($30|0)>(6);
   if ($31) {
    $32 = HEAP32[$zVal>>2]|0;
    $33 = (_sqlite3_strnicmp($32,23519,6)|0);
    $34 = (0)==($33|0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     $36 = HEAP32[$zVal>>2]|0;
     $37 = ((($36)) + 6|0);
     $38 = (_fts3DoIncrmerge($35,$37)|0);
     HEAP32[$rc>>2] = $38;
     break;
    }
   }
   $39 = HEAP32[$nVal>>2]|0;
   $40 = ($39|0)>(10);
   if ($40) {
    $41 = HEAP32[$zVal>>2]|0;
    $42 = (_sqlite3_strnicmp($41,23526,10)|0);
    $43 = (0)==($42|0);
    if ($43) {
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP32[$zVal>>2]|0;
     $46 = ((($45)) + 10|0);
     $47 = (_fts3DoAutoincrmerge($44,$46)|0);
     HEAP32[$rc>>2] = $47;
     break;
    }
   }
   HEAP32[$rc>>2] = 1;
  }
 } while(0);
 $48 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _fts3Writelock($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 260|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_fts3SqlStmt($5,16,$pStmt,0)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if ($8) {
   $9 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_bind_null($9,1)|0);
   $10 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_step($10)|0);
   $11 = HEAP32[$pStmt>>2]|0;
   $12 = (_sqlite3_reset($11)|0);
   HEAP32[$rc>>2] = $12;
  }
 }
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _fts3DeleteByRowid($p,$pRowid,$pnChng,$aSzDel) {
 $p = $p|0;
 $pRowid = $pRowid|0;
 $pnChng = $pnChng|0;
 $aSzDel = $aSzDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $bFound = 0, $isEmpty = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $bFound = sp + 4|0;
 $isEmpty = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pRowid;
 HEAP32[$2>>2] = $pnChng;
 HEAP32[$3>>2] = $aSzDel;
 HEAP32[$rc>>2] = 0;
 HEAP32[$bFound>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 _fts3DeleteTerms($rc,$4,$5,$6,$bFound);
 $7 = HEAP32[$bFound>>2]|0;
 $8 = ($7|0)!=(0);
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 $or$cond = $8 & $10;
 if (!($or$cond)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 HEAP32[$isEmpty>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (_fts3IsEmpty($11,$12,$isEmpty)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(0);
 if (!($15)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $16 = HEAP32[$isEmpty>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (_fts3DeleteAll($18,1)|0);
  HEAP32[$rc>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  HEAP32[$20>>2] = 0;
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 24|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + 1)|0;
  $26 = $25<<2;
  $27 = $26<<1;
  _memset(($21|0),0,($27|0))|0;
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) - 1)|0;
 $31 = HEAP32[$2>>2]|0;
 HEAP32[$31>>2] = $30;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 40|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0|0);
 if ($35) {
  $36 = HEAP32[$0>>2]|0;
  _fts3SqlExec($rc,$36,0,$1);
 }
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 230|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = ($39<<24>>24)!=(0);
 if (!($40)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $41 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$41,19,$1);
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _fts3InsertData($p,$apVal,$piDocid) {
 $p = $p|0;
 $apVal = $apVal|0;
 $piDocid = $piDocid|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $pContentInsert = 0, $pRowid = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $pContentInsert = sp + 4|0;
 $pRowid = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $apVal;
 HEAP32[$3>>2] = $piDocid;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = ((($8)) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 3)|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (($12) + ($11<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pRowid>>2] = $14;
  $15 = HEAP32[$pRowid>>2]|0;
  $16 = (_sqlite3_value_type($15)|0);
  $17 = ($16|0)==(5);
  if ($17) {
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$pRowid>>2] = $20;
  }
  $21 = HEAP32[$pRowid>>2]|0;
  $22 = (_sqlite3_value_type($21)|0);
  $23 = ($22|0)!=(1);
  if ($23) {
   HEAP32[$0>>2] = 19;
   $102 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($102|0);
  } else {
   $24 = HEAP32[$pRowid>>2]|0;
   $25 = (_sqlite3_value_int64($24)|0);
   $26 = tempRet0;
   $27 = HEAP32[$3>>2]|0;
   $28 = $27;
   $29 = $28;
   HEAP32[$29>>2] = $25;
   $30 = (($28) + 4)|0;
   $31 = $30;
   HEAP32[$31>>2] = $26;
   HEAP32[$0>>2] = 0;
   $102 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($102|0);
  }
 }
 $32 = HEAP32[$2>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = (_fts3SqlStmt($8,18,$pContentInsert,$33)|0);
 HEAP32[$rc>>2] = $34;
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 44|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$pContentInsert>>2]|0;
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 24|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + 2)|0;
   $46 = HEAP32[$1>>2]|0;
   $47 = ((($46)) + 24|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + 4)|0;
   $50 = HEAP32[$2>>2]|0;
   $51 = (($50) + ($49<<2)|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3_value_int($52)|0);
   $54 = (_sqlite3_bind_int($41,$45,$53)|0);
   HEAP32[$rc>>2] = $54;
  }
 }
 $55 = HEAP32[$rc>>2]|0;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $57;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 24|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (3 + ($60))|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = (($62) + ($61<<2)|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = (_sqlite3_value_type($64)|0);
 $66 = (5)!=($65|0);
 if ($66) {
  $67 = HEAP32[$2>>2]|0;
  $68 = HEAP32[$67>>2]|0;
  $69 = (_sqlite3_value_type($68)|0);
  $70 = (5)==($69|0);
  if ($70) {
   $71 = HEAP32[$2>>2]|0;
   $72 = ((($71)) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (_sqlite3_value_type($73)|0);
   $75 = (5)!=($74|0);
   if ($75) {
    HEAP32[$0>>2] = 1;
    $102 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($102|0);
   }
  }
  $76 = HEAP32[$pContentInsert>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 24|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (3 + ($79))|0;
  $81 = HEAP32[$2>>2]|0;
  $82 = (($81) + ($80<<2)|0);
  $83 = HEAP32[$82>>2]|0;
  $84 = (_sqlite3_bind_value($76,1,$83)|0);
  HEAP32[$rc>>2] = $84;
  $85 = HEAP32[$rc>>2]|0;
  $86 = ($85|0)!=(0);
  if ($86) {
   $87 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $87;
   $102 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($102|0);
  }
 }
 $88 = HEAP32[$pContentInsert>>2]|0;
 (_sqlite3_step($88)|0);
 $89 = HEAP32[$pContentInsert>>2]|0;
 $90 = (_sqlite3_reset($89)|0);
 HEAP32[$rc>>2] = $90;
 $91 = HEAP32[$1>>2]|0;
 $92 = ((($91)) + 12|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = (_sqlite3_last_insert_rowid($93)|0);
 $95 = tempRet0;
 $96 = HEAP32[$3>>2]|0;
 $97 = $96;
 $98 = $97;
 HEAP32[$98>>2] = $94;
 $99 = (($97) + 4)|0;
 $100 = $99;
 HEAP32[$100>>2] = $95;
 $101 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $101;
 $102 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _fts3PendingTermsDocid($p,$bDelete,$iLangid,$0,$1) {
 $p = $p|0;
 $bDelete = $bDelete|0;
 $iLangid = $iLangid|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp;
 $rc = sp + 8|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $bDelete;
 HEAP32[$5>>2] = $iLangid;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 264|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($16|0)<($24|0);
 $26 = ($13>>>0)<($21>>>0);
 $27 = ($16|0)==($24|0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 do {
  if ($29) {
   label = 6;
  } else {
   $30 = $6;
   $31 = $30;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($30) + 4)|0;
   $34 = $33;
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$3>>2]|0;
   $37 = ((($36)) + 264|0);
   $38 = $37;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = (($38) + 4)|0;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = ($32|0)==($40|0);
   $45 = ($35|0)==($43|0);
   $46 = $44 & $45;
   if ($46) {
    $47 = HEAP32[$3>>2]|0;
    $48 = ((($47)) + 276|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = ($49|0)==(0);
    if ($50) {
     label = 6;
     break;
    }
   }
   $51 = HEAP32[$3>>2]|0;
   $52 = ((($51)) + 272|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$5>>2]|0;
   $55 = ($53|0)!=($54|0);
   if ($55) {
    label = 6;
   } else {
    $56 = HEAP32[$3>>2]|0;
    $57 = ((($56)) + 260|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$3>>2]|0;
    $60 = ((($59)) + 256|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = ($58|0)>($61|0);
    if ($62) {
     label = 6;
    }
   }
  }
 } while(0);
 if ((label|0) == 6) {
  $63 = HEAP32[$3>>2]|0;
  $64 = (_sqlite3Fts3PendingTermsFlush($63)|0);
  HEAP32[$rc>>2] = $64;
  $65 = HEAP32[$rc>>2]|0;
  $66 = ($65|0)!=(0);
  if ($66) {
   $67 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $67;
   $86 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($86|0);
  }
 }
 $68 = $6;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (($68) + 4)|0;
 $72 = $71;
 $73 = HEAP32[$72>>2]|0;
 $74 = HEAP32[$3>>2]|0;
 $75 = ((($74)) + 264|0);
 $76 = $75;
 $77 = $76;
 HEAP32[$77>>2] = $70;
 $78 = (($76) + 4)|0;
 $79 = $78;
 HEAP32[$79>>2] = $73;
 $80 = HEAP32[$5>>2]|0;
 $81 = HEAP32[$3>>2]|0;
 $82 = ((($81)) + 272|0);
 HEAP32[$82>>2] = $80;
 $83 = HEAP32[$4>>2]|0;
 $84 = HEAP32[$3>>2]|0;
 $85 = ((($84)) + 276|0);
 HEAP32[$85>>2] = $83;
 HEAP32[$2>>2] = 0;
 $86 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($86|0);
}
function _fts3InsertTerms($p,$iLangid,$apVal,$aSz) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $apVal = $apVal|0;
 $aSz = $aSz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iCol = 0, $rc = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $iCol = sp + 8|0;
 $zText = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $apVal;
 HEAP32[$4>>2] = $aSz;
 HEAP32[$i>>2] = 2;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 24|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + 2)|0;
  $10 = ($5|0)<($9|0);
  if (!($10)) {
   label = 8;
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = (($11) - 2)|0;
  HEAP32[$iCol>>2] = $12;
  $13 = HEAP32[$iCol>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 32|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = ($19|0)==(0);
  if ($20) {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = (($22) + ($21<<2)|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (_sqlite3_value_text($24)|0);
   HEAP32[$zText>>2] = $25;
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = HEAP32[$zText>>2]|0;
   $29 = HEAP32[$iCol>>2]|0;
   $30 = HEAP32[$iCol>>2]|0;
   $31 = HEAP32[$4>>2]|0;
   $32 = (($31) + ($30<<2)|0);
   $33 = (_fts3PendingTermsAdd($26,$27,$28,$29,$32)|0);
   HEAP32[$rc>>2] = $33;
   $34 = HEAP32[$rc>>2]|0;
   $35 = ($34|0)!=(0);
   if ($35) {
    label = 5;
    break;
   }
   $37 = HEAP32[$i>>2]|0;
   $38 = HEAP32[$3>>2]|0;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (_sqlite3_value_bytes($40)|0);
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 24|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$4>>2]|0;
   $46 = (($45) + ($44<<2)|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (($47) + ($41))|0;
   HEAP32[$46>>2] = $48;
  }
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 if ((label|0) == 5) {
  $36 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $36;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $51 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($51|0);
 }
 return (0)|0;
}
function _fts3InsertDocsize($pRC,$p,$aSz) {
 $pRC = $pRC|0;
 $p = $p|0;
 $aSz = $aSz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $nBlob = 0, $pBlob = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pBlob = sp + 12|0;
 $nBlob = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $aSz;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8*10)|0;
 $10 = (_sqlite3_malloc($9)|0);
 HEAP32[$pBlob>>2] = $10;
 $11 = HEAP32[$pBlob>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  HEAP32[$13>>2] = 7;
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$pBlob>>2]|0;
 _fts3EncodeIntArray($16,$17,$18,$nBlob);
 $19 = HEAP32[$1>>2]|0;
 $20 = (_fts3SqlStmt($19,20,$pStmt,0)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = HEAP32[$pBlob>>2]|0;
  _sqlite3_free($23);
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  HEAP32[$25>>2] = $24;
  STACKTOP = sp;return;
 } else {
  $26 = HEAP32[$pStmt>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 264|0);
  $29 = $28;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  (_sqlite3_bind_int64($26,1,$31,$34)|0);
  $35 = HEAP32[$pStmt>>2]|0;
  $36 = HEAP32[$pBlob>>2]|0;
  $37 = HEAP32[$nBlob>>2]|0;
  (_sqlite3_bind_blob($35,2,$36,$37,148)|0);
  $38 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($38)|0);
  $39 = HEAP32[$pStmt>>2]|0;
  $40 = (_sqlite3_reset($39)|0);
  $41 = HEAP32[$0>>2]|0;
  HEAP32[$41>>2] = $40;
  STACKTOP = sp;return;
 }
}
function _fts3UpdateDocTotals($pRC,$p,$aSzIns,$aSzDel,$nChng) {
 $pRC = $pRC|0;
 $p = $p|0;
 $aSzIns = $aSzIns|0;
 $aSzDel = $aSzDel|0;
 $nChng = $nChng|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0;
 var $i = 0, $nBlob = 0, $nStat = 0, $pBlob = 0, $pStmt = 0, $rc = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $pBlob = sp + 28|0;
 $nBlob = sp + 24|0;
 $a = sp + 20|0;
 $pStmt = sp + 16|0;
 $i = sp + 12|0;
 $rc = sp + 8|0;
 $nStat = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $aSzIns;
 HEAP32[$3>>2] = $aSzDel;
 HEAP32[$4>>2] = $nChng;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 2)|0;
 HEAP32[$nStat>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$nStat>>2]|0;
 $13 = ($12*14)|0;
 $14 = (_sqlite3_malloc($13)|0);
 HEAP32[$a>>2] = $14;
 $15 = HEAP32[$a>>2]|0;
 $16 = ($15|0)==(0|0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  HEAP32[$17>>2] = 7;
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$nStat>>2]|0;
 $19 = HEAP32[$a>>2]|0;
 $20 = (($19) + ($18<<2)|0);
 HEAP32[$pBlob>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = (_fts3SqlStmt($21,22,$pStmt,0)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$a>>2]|0;
  _sqlite3_free($25);
  $26 = HEAP32[$rc>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  HEAP32[$27>>2] = $26;
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_bind_int($28,1,0)|0);
 $29 = HEAP32[$pStmt>>2]|0;
 $30 = (_sqlite3_step($29)|0);
 $31 = ($30|0)==(100);
 if ($31) {
  $32 = HEAP32[$nStat>>2]|0;
  $33 = HEAP32[$a>>2]|0;
  $34 = HEAP32[$pStmt>>2]|0;
  $35 = (_sqlite3_column_blob($34,0)|0);
  $36 = HEAP32[$pStmt>>2]|0;
  $37 = (_sqlite3_column_bytes($36,0)|0);
  _fts3DecodeIntArray($32,$33,$35,$37);
 } else {
  $38 = HEAP32[$a>>2]|0;
  $39 = HEAP32[$nStat>>2]|0;
  $40 = $39<<2;
  _memset(($38|0),0,($40|0))|0;
 }
 $41 = HEAP32[$pStmt>>2]|0;
 $42 = (_sqlite3_reset($41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$rc>>2]|0;
 $44 = ($43|0)!=(0);
 if ($44) {
  $45 = HEAP32[$a>>2]|0;
  _sqlite3_free($45);
  $46 = HEAP32[$rc>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  HEAP32[$47>>2] = $46;
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$4>>2]|0;
 $49 = ($48|0)<(0);
 if ($49) {
  $50 = HEAP32[$a>>2]|0;
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$4>>2]|0;
  $53 = (0 - ($52))|0;
  $54 = ($51>>>0)<($53>>>0);
  if ($54) {
   $55 = HEAP32[$a>>2]|0;
   HEAP32[$55>>2] = 0;
  } else {
   label = 14;
  }
 } else {
  label = 14;
 }
 if ((label|0) == 14) {
  $56 = HEAP32[$4>>2]|0;
  $57 = HEAP32[$a>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($56))|0;
  HEAP32[$57>>2] = $59;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$1>>2]|0;
  $62 = ((($61)) + 24|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($63) + 1)|0;
  $65 = ($60|0)<($64|0);
  if (!($65)) {
   break;
  }
  $66 = HEAP32[$i>>2]|0;
  $67 = (($66) + 1)|0;
  $68 = HEAP32[$a>>2]|0;
  $69 = (($68) + ($67<<2)|0);
  $70 = HEAP32[$69>>2]|0;
  HEAP32[$x>>2] = $70;
  $71 = HEAP32[$x>>2]|0;
  $72 = HEAP32[$i>>2]|0;
  $73 = HEAP32[$2>>2]|0;
  $74 = (($73) + ($72<<2)|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($71) + ($75))|0;
  $77 = HEAP32[$i>>2]|0;
  $78 = HEAP32[$3>>2]|0;
  $79 = (($78) + ($77<<2)|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = ($76>>>0)<($80>>>0);
  if ($81) {
   HEAP32[$x>>2] = 0;
  } else {
   $82 = HEAP32[$x>>2]|0;
   $83 = HEAP32[$i>>2]|0;
   $84 = HEAP32[$2>>2]|0;
   $85 = (($84) + ($83<<2)|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = (($82) + ($86))|0;
   $88 = HEAP32[$i>>2]|0;
   $89 = HEAP32[$3>>2]|0;
   $90 = (($89) + ($88<<2)|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($87) - ($91))|0;
   HEAP32[$x>>2] = $92;
  }
  $93 = HEAP32[$x>>2]|0;
  $94 = HEAP32[$i>>2]|0;
  $95 = (($94) + 1)|0;
  $96 = HEAP32[$a>>2]|0;
  $97 = (($96) + ($95<<2)|0);
  HEAP32[$97>>2] = $93;
  $98 = HEAP32[$i>>2]|0;
  $99 = (($98) + 1)|0;
  HEAP32[$i>>2] = $99;
 }
 $100 = HEAP32[$nStat>>2]|0;
 $101 = HEAP32[$a>>2]|0;
 $102 = HEAP32[$pBlob>>2]|0;
 _fts3EncodeIntArray($100,$101,$102,$nBlob);
 $103 = HEAP32[$1>>2]|0;
 $104 = (_fts3SqlStmt($103,23,$pStmt,0)|0);
 HEAP32[$rc>>2] = $104;
 $105 = HEAP32[$rc>>2]|0;
 $106 = ($105|0)!=(0);
 if ($106) {
  $107 = HEAP32[$a>>2]|0;
  _sqlite3_free($107);
  $108 = HEAP32[$rc>>2]|0;
  $109 = HEAP32[$0>>2]|0;
  HEAP32[$109>>2] = $108;
  STACKTOP = sp;return;
 } else {
  $110 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_bind_int($110,1,0)|0);
  $111 = HEAP32[$pStmt>>2]|0;
  $112 = HEAP32[$pBlob>>2]|0;
  $113 = HEAP32[$nBlob>>2]|0;
  (_sqlite3_bind_blob($111,2,$112,$113,0)|0);
  $114 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($114)|0);
  $115 = HEAP32[$pStmt>>2]|0;
  $116 = (_sqlite3_reset($115)|0);
  $117 = HEAP32[$0>>2]|0;
  HEAP32[$117>>2] = $116;
  $118 = HEAP32[$a>>2]|0;
  _sqlite3_free($118);
  STACKTOP = sp;return;
 }
}
function _fts3DecodeIntArray($N,$a,$zBuf,$nBuf) {
 $N = $N|0;
 $a = $a|0;
 $zBuf = $zBuf|0;
 $nBuf = $nBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $j = sp + 8|0;
 $x = sp;
 HEAP32[$0>>2] = $N;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$3>>2] = $nBuf;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$j>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = (_sqlite3Fts3GetVarint($9,$x)|0);
  $11 = HEAP32[$j>>2]|0;
  $12 = (($11) + ($10))|0;
  HEAP32[$j>>2] = $12;
  $13 = $x;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (($20) + ($19<<2)|0);
  HEAP32[$21>>2] = $15;
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 STACKTOP = sp;return;
}
function _fts3EncodeIntArray($N,$a,$zBuf,$pNBuf) {
 $N = $N|0;
 $a = $a|0;
 $zBuf = $zBuf|0;
 $pNBuf = $pNBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $N;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$3>>2] = $pNBuf;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ($4|0)<($5|0);
  $7 = HEAP32[$j>>2]|0;
  if (!($6)) {
   break;
  }
  $8 = HEAP32[$2>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = (($11) + ($10<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3Fts3PutVarint($9,$13,0)|0);
  $15 = HEAP32[$j>>2]|0;
  $16 = (($15) + ($14))|0;
  HEAP32[$j>>2] = $16;
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
 }
 $19 = HEAP32[$3>>2]|0;
 HEAP32[$19>>2] = $7;
 STACKTOP = sp;return;
}
function _fts3PendingTermsAdd($p,$iLangid,$zText,$iCol,$pnWord) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $zText = $zText|0;
 $iCol = $iCol|0;
 $pnWord = $pnWord|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $i = 0, $iEnd = 0, $iPos = 0, $iStart = 0, $nToken = 0, $nWord = 0, $or$cond = 0, $or$cond$not = 0, $or$cond3 = 0;
 var $pCsr = 0, $pIndex = 0, $pModule = 0, $pTokenizer = 0, $rc = 0, $xNext = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $5 = sp + 52|0;
 $rc = sp + 48|0;
 $iStart = sp + 44|0;
 $iEnd = sp + 40|0;
 $iPos = sp + 36|0;
 $nWord = sp + 32|0;
 $zToken = sp + 28|0;
 $nToken = sp + 24|0;
 $pTokenizer = sp + 20|0;
 $pModule = sp + 16|0;
 $pCsr = sp + 12|0;
 $xNext = sp + 8|0;
 $i = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $zText;
 HEAP32[$4>>2] = $iCol;
 HEAP32[$5>>2] = $pnWord;
 HEAP32[$iStart>>2] = 0;
 HEAP32[$iEnd>>2] = 0;
 HEAP32[$iPos>>2] = 0;
 HEAP32[$nWord>>2] = 0;
 HEAP32[$nToken>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 36|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTokenizer>>2] = $8;
 $9 = HEAP32[$pTokenizer>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pModule>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  $13 = HEAP32[$5>>2]|0;
  HEAP32[$13>>2] = 0;
  HEAP32[$0>>2] = 0;
  $90 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($90|0);
 }
 $14 = HEAP32[$pTokenizer>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = (_sqlite3Fts3OpenTokenizer($14,$15,$16,-1,$pCsr)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $90 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($90|0);
 }
 $21 = HEAP32[$pModule>>2]|0;
 $22 = ((($21)) + 20|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$xNext>>2] = $23;
 L9: while(1) {
  $24 = HEAP32[$rc>>2]|0;
  $25 = (0)==($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$xNext>>2]|0;
  $27 = HEAP32[$pCsr>>2]|0;
  $28 = (FUNCTION_TABLE_iiiiiii[$26 & 255]($27,$zToken,$nToken,$iStart,$iEnd,$iPos)|0);
  HEAP32[$rc>>2] = $28;
  $29 = (0)==($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$iPos>>2]|0;
  $31 = HEAP32[$nWord>>2]|0;
  $32 = ($30|0)>=($31|0);
  if ($32) {
   $33 = HEAP32[$iPos>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$nWord>>2] = $34;
  }
  $35 = HEAP32[$iPos>>2]|0;
  $36 = ($35|0)>=(0);
  $37 = HEAP32[$zToken>>2]|0;
  $38 = ($37|0)!=(0|0);
  $or$cond = $36 & $38;
  $or$cond$not = $or$cond ^ 1;
  $39 = HEAP32[$nToken>>2]|0;
  $40 = ($39|0)<=(0);
  $or$cond3 = $or$cond$not | $40;
  if ($or$cond3) {
   label = 11;
   break;
  }
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$4>>2]|0;
  $43 = HEAP32[$iPos>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 252|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ((($46)) + 4|0);
  $48 = HEAP32[$zToken>>2]|0;
  $49 = HEAP32[$nToken>>2]|0;
  $50 = (_fts3PendingTermsAddOne($41,$42,$43,$47,$48,$49)|0);
  HEAP32[$rc>>2] = $50;
  HEAP32[$i>>2] = 1;
  while(1) {
   $51 = HEAP32[$rc>>2]|0;
   $52 = ($51|0)==(0);
   if (!($52)) {
    continue L9;
   }
   $53 = HEAP32[$i>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 248|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = ($53|0)<($56|0);
   if (!($57)) {
    continue L9;
   }
   $58 = HEAP32[$i>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 252|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + (($58*24)|0)|0);
   HEAP32[$pIndex>>2] = $62;
   $63 = HEAP32[$nToken>>2]|0;
   $64 = HEAP32[$pIndex>>2]|0;
   $65 = HEAP32[$64>>2]|0;
   $66 = ($63|0)<($65|0);
   if (!($66)) {
    $67 = HEAP32[$1>>2]|0;
    $68 = HEAP32[$4>>2]|0;
    $69 = HEAP32[$iPos>>2]|0;
    $70 = HEAP32[$pIndex>>2]|0;
    $71 = ((($70)) + 4|0);
    $72 = HEAP32[$zToken>>2]|0;
    $73 = HEAP32[$pIndex>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    $75 = (_fts3PendingTermsAddOne($67,$68,$69,$71,$72,$74)|0);
    HEAP32[$rc>>2] = $75;
   }
   $76 = HEAP32[$i>>2]|0;
   $77 = (($76) + 1)|0;
   HEAP32[$i>>2] = $77;
  }
 }
 if ((label|0) == 11) {
  HEAP32[$rc>>2] = 1;
 }
 $78 = HEAP32[$pModule>>2]|0;
 $79 = ((($78)) + 16|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = HEAP32[$pCsr>>2]|0;
 (FUNCTION_TABLE_ii[$80 & 255]($81)|0);
 $82 = HEAP32[$nWord>>2]|0;
 $83 = HEAP32[$5>>2]|0;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($84) + ($82))|0;
 HEAP32[$83>>2] = $85;
 $86 = HEAP32[$rc>>2]|0;
 $87 = ($86|0)==(101);
 $88 = HEAP32[$rc>>2]|0;
 $89 = $87 ? 0 : $88;
 HEAP32[$0>>2] = $89;
 $90 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($90|0);
}
function _fts3PendingTermsAddOne($p,$iCol,$iPos,$pHash,$zToken,$nToken) {
 $p = $p|0;
 $iCol = $iCol|0;
 $iPos = $iPos|0;
 $pHash = $pHash|0;
 $zToken = $zToken|0;
 $nToken = $nToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $pList = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$2>>2] = $iPos;
 HEAP32[$3>>2] = $pHash;
 HEAP32[$4>>2] = $zToken;
 HEAP32[$5>>2] = $nToken;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = HEAP32[$5>>2]|0;
 $9 = (_sqlite3Fts3HashFind($6,$7,$8)|0);
 HEAP32[$pList>>2] = $9;
 $10 = HEAP32[$pList>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$pList>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$5>>2]|0;
  $15 = (($13) + ($14))|0;
  $16 = (($15) + 20)|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 260|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) - ($16))|0;
  HEAP32[$18>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 264|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = ($29|0)<(0);
 $31 = $30 << 31 >> 31;
 $32 = HEAP32[$2>>2]|0;
 $33 = ($32|0)<(0);
 $34 = $33 << 31 >> 31;
 $35 = (_fts3PendingListAppend($pList,$25,$28,$29,$31,$32,$34,$rc)|0);
 $36 = ($35|0)!=(0);
 if ($36) {
  $37 = HEAP32[$pList>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = HEAP32[$4>>2]|0;
  $40 = HEAP32[$5>>2]|0;
  $41 = HEAP32[$pList>>2]|0;
  $42 = (_sqlite3Fts3HashInsert($38,$39,$40,$41)|0);
  $43 = ($37|0)==($42|0);
  if ($43) {
   $44 = HEAP32[$pList>>2]|0;
   _sqlite3_free($44);
   HEAP32[$rc>>2] = 7;
  }
 }
 $45 = HEAP32[$rc>>2]|0;
 $46 = ($45|0)==(0);
 if (!($46)) {
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $47 = HEAP32[$pList>>2]|0;
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$5>>2]|0;
 $50 = (($48) + ($49))|0;
 $51 = (($50) + 20)|0;
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 260|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + ($51))|0;
 HEAP32[$53>>2] = $55;
 $56 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _fts3DeleteTerms($pRC,$p,$pRowid,$aSz,$pbFound) {
 $pRC = $pRC|0;
 $p = $p|0;
 $pRowid = $pRowid|0;
 $aSz = $aSz|0;
 $pbFound = $pbFound|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $i = 0, $iCol = 0, $iDocid = 0, $iLangid = 0, $pSelect = 0, $rc = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $rc = sp + 28|0;
 $pSelect = sp + 24|0;
 $i = sp + 20|0;
 $iLangid = sp + 16|0;
 $iDocid = sp;
 $iCol = sp + 12|0;
 $zText = sp + 8|0;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pRowid;
 HEAP32[$3>>2] = $aSz;
 HEAP32[$4>>2] = $pbFound;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (_fts3SqlStmt($8,7,$pSelect,$2)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 $12 = HEAP32[$pSelect>>2]|0;
 if ($11) {
  $13 = (_sqlite3_step($12)|0);
  $14 = (100)==($13|0);
  do {
   if ($14) {
    $15 = HEAP32[$1>>2]|0;
    $16 = HEAP32[$pSelect>>2]|0;
    $17 = (_langidFromSelect($15,$16)|0);
    HEAP32[$iLangid>>2] = $17;
    $18 = HEAP32[$pSelect>>2]|0;
    $19 = (_sqlite3_column_int64($18,0)|0);
    $20 = tempRet0;
    $21 = $iDocid;
    $22 = $21;
    HEAP32[$22>>2] = $19;
    $23 = (($21) + 4)|0;
    $24 = $23;
    HEAP32[$24>>2] = $20;
    $25 = HEAP32[$1>>2]|0;
    $26 = HEAP32[$iLangid>>2]|0;
    $27 = $iDocid;
    $28 = $27;
    $29 = HEAP32[$28>>2]|0;
    $30 = (($27) + 4)|0;
    $31 = $30;
    $32 = HEAP32[$31>>2]|0;
    $33 = (_fts3PendingTermsDocid($25,1,$26,$29,$32)|0);
    HEAP32[$rc>>2] = $33;
    HEAP32[$i>>2] = 1;
    while(1) {
     $34 = HEAP32[$rc>>2]|0;
     $35 = ($34|0)==(0);
     if (!($35)) {
      break;
     }
     $36 = HEAP32[$i>>2]|0;
     $37 = HEAP32[$1>>2]|0;
     $38 = ((($37)) + 24|0);
     $39 = HEAP32[$38>>2]|0;
     $40 = ($36|0)<=($39|0);
     if (!($40)) {
      break;
     }
     $41 = HEAP32[$i>>2]|0;
     $42 = (($41) - 1)|0;
     HEAP32[$iCol>>2] = $42;
     $43 = HEAP32[$iCol>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = ((($44)) + 32|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = (($46) + ($43)|0);
     $48 = HEAP8[$47>>0]|0;
     $49 = $48&255;
     $50 = ($49|0)==(0);
     if ($50) {
      $51 = HEAP32[$pSelect>>2]|0;
      $52 = HEAP32[$i>>2]|0;
      $53 = (_sqlite3_column_text($51,$52)|0);
      HEAP32[$zText>>2] = $53;
      $54 = HEAP32[$1>>2]|0;
      $55 = HEAP32[$iLangid>>2]|0;
      $56 = HEAP32[$zText>>2]|0;
      $57 = HEAP32[$iCol>>2]|0;
      $58 = HEAP32[$3>>2]|0;
      $59 = (($58) + ($57<<2)|0);
      $60 = (_fts3PendingTermsAdd($54,$55,$56,-1,$59)|0);
      HEAP32[$rc>>2] = $60;
      $61 = HEAP32[$pSelect>>2]|0;
      $62 = HEAP32[$i>>2]|0;
      $63 = (_sqlite3_column_bytes($61,$62)|0);
      $64 = HEAP32[$1>>2]|0;
      $65 = ((($64)) + 24|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$3>>2]|0;
      $68 = (($67) + ($66<<2)|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = (($69) + ($63))|0;
      HEAP32[$68>>2] = $70;
     }
     $71 = HEAP32[$i>>2]|0;
     $72 = (($71) + 1)|0;
     HEAP32[$i>>2] = $72;
    }
    $73 = HEAP32[$rc>>2]|0;
    $74 = ($73|0)!=(0);
    if (!($74)) {
     $78 = HEAP32[$4>>2]|0;
     HEAP32[$78>>2] = 1;
     break;
    }
    $75 = HEAP32[$pSelect>>2]|0;
    (_sqlite3_reset($75)|0);
    $76 = HEAP32[$rc>>2]|0;
    $77 = HEAP32[$0>>2]|0;
    HEAP32[$77>>2] = $76;
    STACKTOP = sp;return;
   }
  } while(0);
  $79 = HEAP32[$pSelect>>2]|0;
  $80 = (_sqlite3_reset($79)|0);
  HEAP32[$rc>>2] = $80;
 } else {
  (_sqlite3_reset($12)|0);
 }
 $81 = HEAP32[$rc>>2]|0;
 $82 = HEAP32[$0>>2]|0;
 HEAP32[$82>>2] = $81;
 STACKTOP = sp;return;
}
function _fts3IsEmpty($p,$pRowid,$pisEmpty) {
 $p = $p|0;
 $pRowid = $pRowid|0;
 $pisEmpty = $pisEmpty|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pRowid;
 HEAP32[$2>>2] = $pisEmpty;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  HEAP32[$7>>2] = 0;
  HEAP32[$rc>>2] = 0;
  $20 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = (_fts3SqlStmt($8,1,$pStmt,$1)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $20 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $12 = HEAP32[$pStmt>>2]|0;
 $13 = (_sqlite3_step($12)|0);
 $14 = (100)==($13|0);
 if ($14) {
  $15 = HEAP32[$pStmt>>2]|0;
  $16 = (_sqlite3_column_int($15,0)|0);
  $17 = HEAP32[$2>>2]|0;
  HEAP32[$17>>2] = $16;
 }
 $18 = HEAP32[$pStmt>>2]|0;
 $19 = (_sqlite3_reset($18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _fts3DeleteAll($p,$bContent) {
 $p = $p|0;
 $bContent = $bContent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $bContent;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3Fts3PendingTermsClear($2);
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  _fts3SqlExec($rc,$5,2,0);
 }
 $6 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$6,3,0);
 $7 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$7,4,0);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 230|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  _fts3SqlExec($rc,$12,5,0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 229|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = ($15<<24>>24)!=(0);
 if (!($16)) {
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $17 = HEAP32[$0>>2]|0;
 _fts3SqlExec($rc,$17,6,0);
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _fts3SqlExec($pRC,$p,$eStmt,$apVal) {
 $pRC = $pRC|0;
 $p = $p|0;
 $eStmt = $eStmt|0;
 $apVal = $apVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pStmt = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pRC;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $eStmt;
 HEAP32[$3>>2] = $apVal;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = (_fts3SqlStmt($7,$8,$pStmt,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($13)|0);
  $14 = HEAP32[$pStmt>>2]|0;
  $15 = (_sqlite3_reset($14)|0);
  HEAP32[$rc>>2] = $15;
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 HEAP32[$17>>2] = $16;
 STACKTOP = sp;return;
}
function _langidFromSelect($p,$pSelect) {
 $p = $p|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iLangid = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $iLangid = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pSelect;
 HEAP32[$iLangid>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $12 = HEAP32[$iLangid>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 $11 = (_sqlite3_column_int($6,$10)|0);
 HEAP32[$iLangid>>2] = $11;
 $12 = HEAP32[$iLangid>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _fts3DoRebuild($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aSz = 0, $aSzDel = 0, $aSzIns = 0, $iCol = 0, $iLangid = 0, $nByte = 0, $nEntry = 0, $pStmt = 0, $rc = 0, $rc2 = 0, $vararg_buffer = 0, $z = 0, $zSql = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $rc = sp + 48|0;
 $aSz = sp + 44|0;
 $aSzIns = sp + 40|0;
 $aSzDel = sp + 36|0;
 $pStmt = sp + 32|0;
 $nEntry = sp + 28|0;
 $zSql = sp + 24|0;
 $nByte = sp + 20|0;
 $iCol = sp + 16|0;
 $iLangid = sp + 12|0;
 $z = sp + 8|0;
 $rc2 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_fts3DeleteAll($1,0)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  $145 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($145|0);
 }
 HEAP32[$aSz>>2] = 0;
 HEAP32[$aSzIns>>2] = 0;
 HEAP32[$aSzDel>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 HEAP32[$nEntry>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 216|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$vararg_buffer>>2] = $7;
 $8 = (_sqlite3_mprintf(23614,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $8;
 $9 = HEAP32[$zSql>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$zSql>>2]|0;
  $15 = (_sqlite3_prepare_v2($13,$14,-1,$pStmt,0)|0);
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$zSql>>2]|0;
  _sqlite3_free($16);
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 do {
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 24|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + 1)|0;
   $23 = $22<<2;
   $24 = ($23*3)|0;
   HEAP32[$nByte>>2] = $24;
   $25 = HEAP32[$nByte>>2]|0;
   $26 = (_sqlite3_malloc($25)|0);
   HEAP32[$aSz>>2] = $26;
   $27 = HEAP32[$aSz>>2]|0;
   $28 = ($27|0)==(0|0);
   if ($28) {
    HEAP32[$rc>>2] = 7;
    break;
   } else {
    $29 = HEAP32[$aSz>>2]|0;
    $30 = HEAP32[$nByte>>2]|0;
    _memset(($29|0),0,($30|0))|0;
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 24|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 1)|0;
    $35 = HEAP32[$aSz>>2]|0;
    $36 = (($35) + ($34<<2)|0);
    HEAP32[$aSzIns>>2] = $36;
    $37 = HEAP32[$0>>2]|0;
    $38 = ((($37)) + 24|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + 1)|0;
    $41 = HEAP32[$aSzIns>>2]|0;
    $42 = (($41) + ($40<<2)|0);
    HEAP32[$aSzDel>>2] = $42;
    break;
   }
  }
 } while(0);
 L13: while(1) {
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)==(0);
  if ($44) {
   $45 = HEAP32[$pStmt>>2]|0;
   $46 = (_sqlite3_step($45)|0);
   $47 = (100)==($46|0);
   $146 = $47;
  } else {
   $146 = 0;
  }
  $48 = HEAP32[$0>>2]|0;
  if (!($146)) {
   break;
  }
  $49 = HEAP32[$pStmt>>2]|0;
  $50 = (_langidFromSelect($48,$49)|0);
  HEAP32[$iLangid>>2] = $50;
  $51 = HEAP32[$0>>2]|0;
  $52 = HEAP32[$iLangid>>2]|0;
  $53 = HEAP32[$pStmt>>2]|0;
  $54 = (_sqlite3_column_int64($53,0)|0);
  $55 = tempRet0;
  $56 = (_fts3PendingTermsDocid($51,0,$52,$54,$55)|0);
  HEAP32[$rc>>2] = $56;
  $57 = HEAP32[$aSz>>2]|0;
  $58 = HEAP32[$0>>2]|0;
  $59 = ((($58)) + 24|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (($60) + 1)|0;
  $62 = $61<<2;
  _memset(($57|0),0,($62|0))|0;
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)==(0);
   if (!($64)) {
    break;
   }
   $65 = HEAP32[$iCol>>2]|0;
   $66 = HEAP32[$0>>2]|0;
   $67 = ((($66)) + 24|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = ($65|0)<($68|0);
   if (!($69)) {
    break;
   }
   $70 = HEAP32[$iCol>>2]|0;
   $71 = HEAP32[$0>>2]|0;
   $72 = ((($71)) + 32|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($73) + ($70)|0);
   $75 = HEAP8[$74>>0]|0;
   $76 = $75&255;
   $77 = ($76|0)==(0);
   if ($77) {
    $78 = HEAP32[$pStmt>>2]|0;
    $79 = HEAP32[$iCol>>2]|0;
    $80 = (($79) + 1)|0;
    $81 = (_sqlite3_column_text($78,$80)|0);
    HEAP32[$z>>2] = $81;
    $82 = HEAP32[$0>>2]|0;
    $83 = HEAP32[$iLangid>>2]|0;
    $84 = HEAP32[$z>>2]|0;
    $85 = HEAP32[$iCol>>2]|0;
    $86 = HEAP32[$iCol>>2]|0;
    $87 = HEAP32[$aSz>>2]|0;
    $88 = (($87) + ($86<<2)|0);
    $89 = (_fts3PendingTermsAdd($82,$83,$84,$85,$88)|0);
    HEAP32[$rc>>2] = $89;
    $90 = HEAP32[$pStmt>>2]|0;
    $91 = HEAP32[$iCol>>2]|0;
    $92 = (($91) + 1)|0;
    $93 = (_sqlite3_column_bytes($90,$92)|0);
    $94 = HEAP32[$0>>2]|0;
    $95 = ((($94)) + 24|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$aSz>>2]|0;
    $98 = (($97) + ($96<<2)|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = (($99) + ($93))|0;
    HEAP32[$98>>2] = $100;
   }
   $101 = HEAP32[$iCol>>2]|0;
   $102 = (($101) + 1)|0;
   HEAP32[$iCol>>2] = $102;
  }
  $103 = HEAP32[$0>>2]|0;
  $104 = ((($103)) + 230|0);
  $105 = HEAP8[$104>>0]|0;
  $106 = ($105<<24>>24)!=(0);
  if ($106) {
   $107 = HEAP32[$0>>2]|0;
   $108 = HEAP32[$aSz>>2]|0;
   _fts3InsertDocsize($rc,$107,$108);
  }
  $109 = HEAP32[$rc>>2]|0;
  $110 = ($109|0)!=(0);
  if ($110) {
   $111 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_finalize($111)|0);
   HEAP32[$pStmt>>2] = 0;
   continue;
  }
  $112 = HEAP32[$nEntry>>2]|0;
  $113 = (($112) + 1)|0;
  HEAP32[$nEntry>>2] = $113;
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $114 = HEAP32[$iCol>>2]|0;
   $115 = HEAP32[$0>>2]|0;
   $116 = ((($115)) + 24|0);
   $117 = HEAP32[$116>>2]|0;
   $118 = ($114|0)<=($117|0);
   if (!($118)) {
    continue L13;
   }
   $119 = HEAP32[$iCol>>2]|0;
   $120 = HEAP32[$aSz>>2]|0;
   $121 = (($120) + ($119<<2)|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = HEAP32[$iCol>>2]|0;
   $124 = HEAP32[$aSzIns>>2]|0;
   $125 = (($124) + ($123<<2)|0);
   $126 = HEAP32[$125>>2]|0;
   $127 = (($126) + ($122))|0;
   HEAP32[$125>>2] = $127;
   $128 = HEAP32[$iCol>>2]|0;
   $129 = (($128) + 1)|0;
   HEAP32[$iCol>>2] = $129;
  }
 }
 $130 = ((($48)) + 228|0);
 $131 = HEAP8[$130>>0]|0;
 $132 = ($131<<24>>24)!=(0);
 if ($132) {
  $133 = HEAP32[$0>>2]|0;
  $134 = HEAP32[$aSzIns>>2]|0;
  $135 = HEAP32[$aSzDel>>2]|0;
  $136 = HEAP32[$nEntry>>2]|0;
  _fts3UpdateDocTotals($rc,$133,$134,$135,$136);
 }
 $137 = HEAP32[$aSz>>2]|0;
 _sqlite3_free($137);
 $138 = HEAP32[$pStmt>>2]|0;
 $139 = ($138|0)!=(0|0);
 if (!($139)) {
  $145 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($145|0);
 }
 $140 = HEAP32[$pStmt>>2]|0;
 $141 = (_sqlite3_finalize($140)|0);
 HEAP32[$rc2>>2] = $141;
 $142 = HEAP32[$rc>>2]|0;
 $143 = ($142|0)==(0);
 if (!($143)) {
  $145 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($145|0);
 }
 $144 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $144;
 $145 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($145|0);
}
function _fts3DoIntegrityCheck($p) {
 $p = $p|0;
 var $$ = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $bOk = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $bOk = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$bOk>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_fts3IntegrityCheck($1,$bOk)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)==(0);
 $5 = HEAP32[$bOk>>2]|0;
 $6 = ($5|0)==(0);
 $or$cond = $4 & $6;
 $$ = $or$cond ? 267 : $2;
 HEAP32[$rc>>2] = $$;
 $7 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _fts3DoIncrmerge($p,$zParam) {
 $p = $p|0;
 $zParam = $zParam|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nMerge = 0, $nMin = 0, $or$cond = 0, $rc = 0, $z = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $nMin = sp + 8|0;
 $nMerge = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zParam;
 HEAP32[$nMin>>2] = 8;
 HEAP32[$nMerge>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$z>>2] = $2;
 $3 = (_fts3Getint($z)|0);
 HEAP32[$nMerge>>2] = $3;
 $4 = HEAP32[$z>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = ($6|0)==(44);
 if ($7) {
  $8 = HEAP32[$z>>2]|0;
  $9 = ((($8)) + 1|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$z>>2]|0;
   $14 = ((($13)) + 1|0);
   HEAP32[$z>>2] = $14;
   $15 = (_fts3Getint($z)|0);
   HEAP32[$nMin>>2] = $15;
  }
 }
 $16 = HEAP32[$z>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17 << 24 >> 24;
 $19 = ($18|0)!=(0);
 $20 = HEAP32[$nMin>>2]|0;
 $21 = ($20|0)<(2);
 $or$cond = $19 | $21;
 if ($or$cond) {
  HEAP32[$rc>>2] = 1;
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 HEAP32[$rc>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 229|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = ($24<<24>>24)!=(0);
 if (!($25)) {
  $26 = HEAP32[$0>>2]|0;
  _sqlite3Fts3CreateStatTable($rc,$26);
 }
 $27 = HEAP32[$rc>>2]|0;
 $28 = ($27|0)==(0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$nMerge>>2]|0;
  $31 = HEAP32[$nMin>>2]|0;
  $32 = (_sqlite3Fts3Incrmerge($29,$30,$31)|0);
  HEAP32[$rc>>2] = $32;
 }
 $33 = HEAP32[$0>>2]|0;
 _sqlite3Fts3SegmentsClose($33);
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _fts3DoAutoincrmerge($p,$zParam) {
 $p = $p|0;
 $zParam = $zParam|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pStmt = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zParam;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $3 = (_fts3Getint($2)|0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 48|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 48|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(1);
 if ($9) {
  label = 3;
 } else {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 48|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>(16);
  if ($13) {
   label = 3;
  }
 }
 if ((label|0) == 3) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 48|0);
  HEAP32[$15>>2] = 8;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 229|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18<<24>>24)!=(0);
 if (!($19)) {
  $20 = HEAP32[$1>>2]|0;
  _sqlite3Fts3CreateStatTable($rc,$20);
  $21 = HEAP32[$rc>>2]|0;
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $23;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = (_fts3SqlStmt($24,23,$pStmt,0)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 } else {
  $29 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_bind_int($29,1,2)|0);
  $30 = HEAP32[$pStmt>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 48|0);
  $33 = HEAP32[$32>>2]|0;
  (_sqlite3_bind_int($30,2,$33)|0);
  $34 = HEAP32[$pStmt>>2]|0;
  (_sqlite3_step($34)|0);
  $35 = HEAP32[$pStmt>>2]|0;
  $36 = (_sqlite3_reset($35)|0);
  HEAP32[$rc>>2] = $36;
  $37 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $37;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 return (0)|0;
}
function _fts3Getint($pz) {
 $pz = $pz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $z = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pz;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$z>>2] = $2;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$z>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = ($5|0)>=(48);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$z>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)<=(57);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11*10)|0;
  $13 = HEAP32[$z>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP32[$z>>2] = $14;
  $15 = HEAP8[$13>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = (($12) + ($16))|0;
  $18 = (($17) - 48)|0;
  HEAP32[$i>>2] = $18;
 }
 if ((label|0) == 5) {
  $19 = HEAP32[$z>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  HEAP32[$20>>2] = $19;
  $21 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3Fts3CreateStatTable($pRc,$p) {
 $pRc = $pRc|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$vararg_buffer>>2] = $8;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 _fts3DbExec($2,$5,23537,$vararg_buffer);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 229|0);
 HEAP8[$16>>0] = 1;
 STACKTOP = sp;return;
}
function _fts3IntegrityCheck($p,$pbOk) {
 $p = $p|0;
 $pbOk = $pbOk|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum1 = 0, $cksum2 = 0, $i = 0, $i1 = 0, $iCol = 0, $iDocid = 0, $iDum1 = 0, $iDum2 = 0, $iLang = 0, $iLangid = 0;
 var $iPos = 0, $nText = 0, $nToken = 0, $pAllLangid = 0, $pModule = 0, $pStmt = 0, $pT = 0, $rc = 0, $rc2 = 0, $vararg_buffer = 0, $zSql = 0, $zText = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp + 24|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $rc = sp + 100|0;
 $cksum1 = sp + 16|0;
 $cksum2 = sp + 8|0;
 $pAllLangid = sp + 96|0;
 $rc2 = sp + 92|0;
 $iLangid = sp + 88|0;
 $i = sp + 84|0;
 $pModule = sp + 80|0;
 $pStmt = sp + 76|0;
 $zSql = sp + 72|0;
 $iDocid = sp;
 $iLang = sp + 68|0;
 $iCol = sp + 64|0;
 $zText = sp + 60|0;
 $nText = sp + 56|0;
 $pT = sp + 52|0;
 $zToken = sp + 48|0;
 $nToken = sp + 44|0;
 $iDum1 = sp + 40|0;
 $iDum2 = sp + 36|0;
 $iPos = sp + 32|0;
 $i1 = sp + 28|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pbOk;
 HEAP32[$rc>>2] = 0;
 $2 = $cksum1;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = $cksum2;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 HEAP32[$pAllLangid>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (_fts3SqlStmt($10,27,$pAllLangid,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$pAllLangid>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 272|0);
  $17 = HEAP32[$16>>2]|0;
  (_sqlite3_bind_int($14,1,$17)|0);
  $18 = HEAP32[$pAllLangid>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 248|0);
  $21 = HEAP32[$20>>2]|0;
  (_sqlite3_bind_int($18,2,$21)|0);
  L3: while(1) {
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$pAllLangid>>2]|0;
    $25 = (_sqlite3_step($24)|0);
    $26 = ($25|0)==(100);
    $228 = $26;
   } else {
    $228 = 0;
   }
   $27 = HEAP32[$pAllLangid>>2]|0;
   if (!($228)) {
    break;
   }
   $28 = (_sqlite3_column_int($27,0)|0);
   HEAP32[$iLangid>>2] = $28;
   HEAP32[$i>>2] = 0;
   while(1) {
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 248|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = ($29|0)<($32|0);
    if (!($33)) {
     continue L3;
    }
    $34 = $cksum1;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$0>>2]|0;
    $41 = HEAP32[$iLangid>>2]|0;
    $42 = HEAP32[$i>>2]|0;
    $43 = (_fts3ChecksumIndex($40,$41,$42,$rc)|0);
    $44 = tempRet0;
    $45 = $36 ^ $43;
    $46 = $39 ^ $44;
    $47 = $cksum1;
    $48 = $47;
    HEAP32[$48>>2] = $45;
    $49 = (($47) + 4)|0;
    $50 = $49;
    HEAP32[$50>>2] = $46;
    $51 = HEAP32[$i>>2]|0;
    $52 = (($51) + 1)|0;
    HEAP32[$i>>2] = $52;
   }
  }
  $53 = (_sqlite3_reset($27)|0);
  HEAP32[$rc2>>2] = $53;
  $54 = HEAP32[$rc>>2]|0;
  $55 = ($54|0)==(0);
  if ($55) {
   $56 = HEAP32[$rc2>>2]|0;
   HEAP32[$rc>>2] = $56;
  }
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if (!($58)) {
  $210 = $cksum1;
  $211 = $210;
  $212 = HEAP32[$211>>2]|0;
  $213 = (($210) + 4)|0;
  $214 = $213;
  $215 = HEAP32[$214>>2]|0;
  $216 = $cksum2;
  $217 = $216;
  $218 = HEAP32[$217>>2]|0;
  $219 = (($216) + 4)|0;
  $220 = $219;
  $221 = HEAP32[$220>>2]|0;
  $222 = ($212|0)==($218|0);
  $223 = ($215|0)==($221|0);
  $224 = $222 & $223;
  $225 = $224&1;
  $226 = HEAP32[$1>>2]|0;
  HEAP32[$226>>2] = $225;
  $227 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 36|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$61>>2]|0;
 HEAP32[$pModule>>2] = $62;
 HEAP32[$pStmt>>2] = 0;
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 216|0);
 $65 = HEAP32[$64>>2]|0;
 HEAP32[$vararg_buffer>>2] = $65;
 $66 = (_sqlite3_mprintf(23614,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $66;
 $67 = HEAP32[$zSql>>2]|0;
 $68 = ($67|0)!=(0|0);
 if ($68) {
  $69 = HEAP32[$0>>2]|0;
  $70 = ((($69)) + 12|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = HEAP32[$zSql>>2]|0;
  $73 = (_sqlite3_prepare_v2($71,$72,-1,$pStmt,0)|0);
  HEAP32[$rc>>2] = $73;
  $74 = HEAP32[$zSql>>2]|0;
  _sqlite3_free($74);
 } else {
  HEAP32[$rc>>2] = 7;
 }
 L21: while(1) {
  $75 = HEAP32[$rc>>2]|0;
  $76 = ($75|0)==(0);
  if ($76) {
   $77 = HEAP32[$pStmt>>2]|0;
   $78 = (_sqlite3_step($77)|0);
   $79 = (100)==($78|0);
   $229 = $79;
  } else {
   $229 = 0;
  }
  $80 = HEAP32[$pStmt>>2]|0;
  if (!($229)) {
   break;
  }
  $81 = (_sqlite3_column_int64($80,0)|0);
  $82 = tempRet0;
  $83 = $iDocid;
  $84 = $83;
  HEAP32[$84>>2] = $81;
  $85 = (($83) + 4)|0;
  $86 = $85;
  HEAP32[$86>>2] = $82;
  $87 = HEAP32[$0>>2]|0;
  $88 = HEAP32[$pStmt>>2]|0;
  $89 = (_langidFromSelect($87,$88)|0);
  HEAP32[$iLang>>2] = $89;
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $90 = HEAP32[$rc>>2]|0;
   $91 = ($90|0)==(0);
   if (!($91)) {
    continue L21;
   }
   $92 = HEAP32[$iCol>>2]|0;
   $93 = HEAP32[$0>>2]|0;
   $94 = ((($93)) + 24|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = ($92|0)<($95|0);
   if (!($96)) {
    continue L21;
   }
   $97 = HEAP32[$iCol>>2]|0;
   $98 = HEAP32[$0>>2]|0;
   $99 = ((($98)) + 32|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = (($100) + ($97)|0);
   $102 = HEAP8[$101>>0]|0;
   $103 = $102&255;
   $104 = ($103|0)==(0);
   if ($104) {
    $105 = HEAP32[$pStmt>>2]|0;
    $106 = HEAP32[$iCol>>2]|0;
    $107 = (($106) + 1)|0;
    $108 = (_sqlite3_column_text($105,$107)|0);
    HEAP32[$zText>>2] = $108;
    $109 = HEAP32[$pStmt>>2]|0;
    $110 = HEAP32[$iCol>>2]|0;
    $111 = (($110) + 1)|0;
    $112 = (_sqlite3_column_bytes($109,$111)|0);
    HEAP32[$nText>>2] = $112;
    HEAP32[$pT>>2] = 0;
    $113 = HEAP32[$0>>2]|0;
    $114 = ((($113)) + 36|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = HEAP32[$iLang>>2]|0;
    $117 = HEAP32[$zText>>2]|0;
    $118 = HEAP32[$nText>>2]|0;
    $119 = (_sqlite3Fts3OpenTokenizer($115,$116,$117,$118,$pT)|0);
    HEAP32[$rc>>2] = $119;
    L33: while(1) {
     $120 = HEAP32[$rc>>2]|0;
     $121 = ($120|0)==(0);
     if (!($121)) {
      break;
     }
     HEAP32[$nToken>>2] = 0;
     HEAP32[$iDum1>>2] = 0;
     HEAP32[$iDum2>>2] = 0;
     HEAP32[$iPos>>2] = 0;
     $122 = HEAP32[$pModule>>2]|0;
     $123 = ((($122)) + 20|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = HEAP32[$pT>>2]|0;
     $126 = (FUNCTION_TABLE_iiiiiii[$124 & 255]($125,$zToken,$nToken,$iDum1,$iDum2,$iPos)|0);
     HEAP32[$rc>>2] = $126;
     $127 = HEAP32[$rc>>2]|0;
     $128 = ($127|0)==(0);
     if (!($128)) {
      continue;
     }
     $129 = $cksum2;
     $130 = $129;
     $131 = HEAP32[$130>>2]|0;
     $132 = (($129) + 4)|0;
     $133 = $132;
     $134 = HEAP32[$133>>2]|0;
     $135 = HEAP32[$zToken>>2]|0;
     $136 = HEAP32[$nToken>>2]|0;
     $137 = HEAP32[$iLang>>2]|0;
     $138 = $iDocid;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = (($138) + 4)|0;
     $142 = $141;
     $143 = HEAP32[$142>>2]|0;
     $144 = HEAP32[$iCol>>2]|0;
     $145 = HEAP32[$iPos>>2]|0;
     $146 = (_fts3ChecksumEntry($135,$136,$137,0,$140,$143,$144,$145)|0);
     $147 = tempRet0;
     $148 = $131 ^ $146;
     $149 = $134 ^ $147;
     $150 = $cksum2;
     $151 = $150;
     HEAP32[$151>>2] = $148;
     $152 = (($150) + 4)|0;
     $153 = $152;
     HEAP32[$153>>2] = $149;
     HEAP32[$i1>>2] = 1;
     while(1) {
      $154 = HEAP32[$i1>>2]|0;
      $155 = HEAP32[$0>>2]|0;
      $156 = ((($155)) + 248|0);
      $157 = HEAP32[$156>>2]|0;
      $158 = ($154|0)<($157|0);
      if (!($158)) {
       continue L33;
      }
      $159 = HEAP32[$i1>>2]|0;
      $160 = HEAP32[$0>>2]|0;
      $161 = ((($160)) + 252|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = (($162) + (($159*24)|0)|0);
      $164 = HEAP32[$163>>2]|0;
      $165 = HEAP32[$nToken>>2]|0;
      $166 = ($164|0)<=($165|0);
      if ($166) {
       $167 = $cksum2;
       $168 = $167;
       $169 = HEAP32[$168>>2]|0;
       $170 = (($167) + 4)|0;
       $171 = $170;
       $172 = HEAP32[$171>>2]|0;
       $173 = HEAP32[$zToken>>2]|0;
       $174 = HEAP32[$i1>>2]|0;
       $175 = HEAP32[$0>>2]|0;
       $176 = ((($175)) + 252|0);
       $177 = HEAP32[$176>>2]|0;
       $178 = (($177) + (($174*24)|0)|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = HEAP32[$iLang>>2]|0;
       $181 = HEAP32[$i1>>2]|0;
       $182 = $iDocid;
       $183 = $182;
       $184 = HEAP32[$183>>2]|0;
       $185 = (($182) + 4)|0;
       $186 = $185;
       $187 = HEAP32[$186>>2]|0;
       $188 = HEAP32[$iCol>>2]|0;
       $189 = HEAP32[$iPos>>2]|0;
       $190 = (_fts3ChecksumEntry($173,$179,$180,$181,$184,$187,$188,$189)|0);
       $191 = tempRet0;
       $192 = $169 ^ $190;
       $193 = $172 ^ $191;
       $194 = $cksum2;
       $195 = $194;
       HEAP32[$195>>2] = $192;
       $196 = (($194) + 4)|0;
       $197 = $196;
       HEAP32[$197>>2] = $193;
      }
      $198 = HEAP32[$i1>>2]|0;
      $199 = (($198) + 1)|0;
      HEAP32[$i1>>2] = $199;
     }
    }
    $200 = HEAP32[$pT>>2]|0;
    $201 = ($200|0)!=(0|0);
    if ($201) {
     $202 = HEAP32[$pModule>>2]|0;
     $203 = ((($202)) + 16|0);
     $204 = HEAP32[$203>>2]|0;
     $205 = HEAP32[$pT>>2]|0;
     (FUNCTION_TABLE_ii[$204 & 255]($205)|0);
    }
    $206 = HEAP32[$rc>>2]|0;
    $207 = ($206|0)==(101);
    if ($207) {
     HEAP32[$rc>>2] = 0;
    }
   }
   $208 = HEAP32[$iCol>>2]|0;
   $209 = (($208) + 1)|0;
   HEAP32[$iCol>>2] = $209;
  }
 }
 (_sqlite3_finalize($80)|0);
 $210 = $cksum1;
 $211 = $210;
 $212 = HEAP32[$211>>2]|0;
 $213 = (($210) + 4)|0;
 $214 = $213;
 $215 = HEAP32[$214>>2]|0;
 $216 = $cksum2;
 $217 = $216;
 $218 = HEAP32[$217>>2]|0;
 $219 = (($216) + 4)|0;
 $220 = $219;
 $221 = HEAP32[$220>>2]|0;
 $222 = ($212|0)==($218|0);
 $223 = ($215|0)==($221|0);
 $224 = $222 & $223;
 $225 = $224&1;
 $226 = HEAP32[$1>>2]|0;
 HEAP32[$226>>2] = $225;
 $227 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($227|0);
}
function _fts3ChecksumIndex($p,$iLangid,$iIndex,$pRc) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum = 0, $csr = 0, $filter = 0, $iCol = 0, $iDocid = 0, $iPos = 0, $iVal = 0, $or$cond = 0, $pCsr = 0, $pEnd = 0, $rc = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $filter = sp + 112|0;
 $csr = sp + 56|0;
 $rc = sp + 48|0;
 $cksum = sp + 32|0;
 $pCsr = sp + 44|0;
 $pEnd = sp + 40|0;
 $iDocid = sp + 24|0;
 $iCol = sp + 16|0;
 $iPos = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $iIndex;
 HEAP32[$3>>2] = $pRc;
 $4 = $cksum;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 ;HEAP32[$filter>>2]=0|0;HEAP32[$filter+4>>2]=0|0;HEAP32[$filter+8>>2]=0|0;HEAP32[$filter+12>>2]=0|0;
 dest=$csr; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = ((($filter)) + 12|0);
 HEAP32[$8>>2] = 3;
 $9 = ((($filter)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 | 16;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (_sqlite3Fts3SegReaderCursor($12,$13,$14,-2,0,0,0,1,$csr)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = (_sqlite3Fts3SegReaderStart($18,$csr,$filter)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if (!($21)) {
  _sqlite3Fts3SegReaderFinish($csr);
  $179 = HEAP32[$rc>>2]|0;
  $180 = HEAP32[$3>>2]|0;
  HEAP32[$180>>2] = $179;
  $181 = $cksum;
  $182 = $181;
  $183 = HEAP32[$182>>2]|0;
  $184 = (($181) + 4)|0;
  $185 = $184;
  $186 = HEAP32[$185>>2]|0;
  tempRet0 = ($186);
  STACKTOP = sp;return ($183|0);
 }
 L6: while(1) {
  $22 = HEAP32[$0>>2]|0;
  $23 = (_sqlite3Fts3SegReaderStep($22,$csr)|0);
  HEAP32[$rc>>2] = $23;
  $24 = (100)==($23|0);
  if (!($24)) {
   break;
  }
  $25 = ((($csr)) + 48|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pCsr>>2] = $26;
  $27 = ((($csr)) + 52|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$pCsr>>2]|0;
  $30 = (($29) + ($28)|0);
  HEAP32[$pEnd>>2] = $30;
  $31 = $iDocid;
  $32 = $31;
  HEAP32[$32>>2] = 0;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = 0;
  $35 = $iCol;
  $36 = $35;
  HEAP32[$36>>2] = 0;
  $37 = (($35) + 4)|0;
  $38 = $37;
  HEAP32[$38>>2] = 0;
  $39 = $iPos;
  $40 = $39;
  HEAP32[$40>>2] = 0;
  $41 = (($39) + 4)|0;
  $42 = $41;
  HEAP32[$42>>2] = 0;
  $43 = HEAP32[$pCsr>>2]|0;
  $44 = (_sqlite3Fts3GetVarint($43,$iDocid)|0);
  $45 = HEAP32[$pCsr>>2]|0;
  $46 = (($45) + ($44)|0);
  HEAP32[$pCsr>>2] = $46;
  while(1) {
   $47 = HEAP32[$pCsr>>2]|0;
   $48 = HEAP32[$pEnd>>2]|0;
   $49 = ($47>>>0)<($48>>>0);
   if (!($49)) {
    continue L6;
   }
   $50 = $iVal;
   $51 = $50;
   HEAP32[$51>>2] = 0;
   $52 = (($50) + 4)|0;
   $53 = $52;
   HEAP32[$53>>2] = 0;
   $54 = HEAP32[$pCsr>>2]|0;
   $55 = (_sqlite3Fts3GetVarint($54,$iVal)|0);
   $56 = HEAP32[$pCsr>>2]|0;
   $57 = (($56) + ($55)|0);
   HEAP32[$pCsr>>2] = $57;
   $58 = HEAP32[$pCsr>>2]|0;
   $59 = HEAP32[$pEnd>>2]|0;
   $60 = ($58>>>0)<($59>>>0);
   if (!($60)) {
    continue;
   }
   $61 = $iVal;
   $62 = $61;
   $63 = HEAP32[$62>>2]|0;
   $64 = (($61) + 4)|0;
   $65 = $64;
   $66 = HEAP32[$65>>2]|0;
   $67 = ($63|0)==(0);
   $68 = ($66|0)==(0);
   $69 = $67 & $68;
   $70 = $iVal;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($70) + 4)|0;
   $74 = $73;
   $75 = HEAP32[$74>>2]|0;
   $76 = ($72|0)==(1);
   $77 = ($75|0)==(0);
   $78 = $76 & $77;
   $or$cond = $69 | $78;
   if (!($or$cond)) {
    $121 = $iVal;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = (($121) + 4)|0;
    $125 = $124;
    $126 = HEAP32[$125>>2]|0;
    $127 = (_i64Subtract(($123|0),($126|0),2,0)|0);
    $128 = tempRet0;
    $129 = $iPos;
    $130 = $129;
    $131 = HEAP32[$130>>2]|0;
    $132 = (($129) + 4)|0;
    $133 = $132;
    $134 = HEAP32[$133>>2]|0;
    $135 = (_i64Add(($131|0),($134|0),($127|0),($128|0))|0);
    $136 = tempRet0;
    $137 = $iPos;
    $138 = $137;
    HEAP32[$138>>2] = $135;
    $139 = (($137) + 4)|0;
    $140 = $139;
    HEAP32[$140>>2] = $136;
    $141 = $cksum;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = (($141) + 4)|0;
    $145 = $144;
    $146 = HEAP32[$145>>2]|0;
    $147 = ((($csr)) + 40|0);
    $148 = HEAP32[$147>>2]|0;
    $149 = ((($csr)) + 44|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = HEAP32[$1>>2]|0;
    $152 = HEAP32[$2>>2]|0;
    $153 = $iDocid;
    $154 = $153;
    $155 = HEAP32[$154>>2]|0;
    $156 = (($153) + 4)|0;
    $157 = $156;
    $158 = HEAP32[$157>>2]|0;
    $159 = $iCol;
    $160 = $159;
    $161 = HEAP32[$160>>2]|0;
    $162 = (($159) + 4)|0;
    $163 = $162;
    $164 = HEAP32[$163>>2]|0;
    $165 = $iPos;
    $166 = $165;
    $167 = HEAP32[$166>>2]|0;
    $168 = (($165) + 4)|0;
    $169 = $168;
    $170 = HEAP32[$169>>2]|0;
    $171 = (_fts3ChecksumEntry($148,$150,$151,$152,$155,$158,$161,$167)|0);
    $172 = tempRet0;
    $173 = $143 ^ $171;
    $174 = $146 ^ $172;
    $175 = $cksum;
    $176 = $175;
    HEAP32[$176>>2] = $173;
    $177 = (($175) + 4)|0;
    $178 = $177;
    HEAP32[$178>>2] = $174;
    continue;
   }
   $79 = $iCol;
   $80 = $79;
   HEAP32[$80>>2] = 0;
   $81 = (($79) + 4)|0;
   $82 = $81;
   HEAP32[$82>>2] = 0;
   $83 = $iPos;
   $84 = $83;
   HEAP32[$84>>2] = 0;
   $85 = (($83) + 4)|0;
   $86 = $85;
   HEAP32[$86>>2] = 0;
   $87 = $iVal;
   $88 = $87;
   $89 = HEAP32[$88>>2]|0;
   $90 = (($87) + 4)|0;
   $91 = $90;
   $92 = HEAP32[$91>>2]|0;
   $93 = ($89|0)!=(0);
   $94 = ($92|0)!=(0);
   $95 = $93 | $94;
   $96 = HEAP32[$pCsr>>2]|0;
   if ($95) {
    $97 = (_sqlite3Fts3GetVarint($96,$iCol)|0);
    $98 = HEAP32[$pCsr>>2]|0;
    $99 = (($98) + ($97)|0);
    HEAP32[$pCsr>>2] = $99;
    continue;
   } else {
    $100 = (_sqlite3Fts3GetVarint($96,$iVal)|0);
    $101 = HEAP32[$pCsr>>2]|0;
    $102 = (($101) + ($100)|0);
    HEAP32[$pCsr>>2] = $102;
    $103 = $iVal;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($103) + 4)|0;
    $107 = $106;
    $108 = HEAP32[$107>>2]|0;
    $109 = $iDocid;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = (($109) + 4)|0;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = (_i64Add(($111|0),($114|0),($105|0),($108|0))|0);
    $116 = tempRet0;
    $117 = $iDocid;
    $118 = $117;
    HEAP32[$118>>2] = $115;
    $119 = (($117) + 4)|0;
    $120 = $119;
    HEAP32[$120>>2] = $116;
    continue;
   }
  }
 }
 _sqlite3Fts3SegReaderFinish($csr);
 $179 = HEAP32[$rc>>2]|0;
 $180 = HEAP32[$3>>2]|0;
 HEAP32[$180>>2] = $179;
 $181 = $cksum;
 $182 = $181;
 $183 = HEAP32[$182>>2]|0;
 $184 = (($181) + 4)|0;
 $185 = $184;
 $186 = HEAP32[$185>>2]|0;
 tempRet0 = ($186);
 STACKTOP = sp;return ($183|0);
}
function _fts3ChecksumEntry($zTerm,$nTerm,$iLangid,$iIndex,$0,$1,$iCol,$iPos) {
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $iLangid = $iLangid|0;
 $iIndex = $iIndex|0;
 $0 = $0|0;
 $1 = $1|0;
 $iCol = $iCol|0;
 $iPos = $iPos|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 8|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $i = sp + 16|0;
 $ret = sp;
 HEAP32[$2>>2] = $zTerm;
 HEAP32[$3>>2] = $nTerm;
 HEAP32[$4>>2] = $iLangid;
 HEAP32[$5>>2] = $iIndex;
 $9 = $6;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$7>>2] = $iCol;
 HEAP32[$8>>2] = $iPos;
 $13 = $6;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = $ret;
 $20 = $19;
 HEAP32[$20>>2] = $15;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $18;
 $23 = $ret;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (_bitshift64Shl(($25|0),($28|0),3)|0);
 $30 = tempRet0;
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)<(0);
 $33 = $32 << 31 >> 31;
 $34 = (_i64Add(($29|0),($30|0),($31|0),($33|0))|0);
 $35 = tempRet0;
 $36 = $ret;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (_i64Add(($38|0),($41|0),($34|0),($35|0))|0);
 $43 = tempRet0;
 $44 = $ret;
 $45 = $44;
 HEAP32[$45>>2] = $42;
 $46 = (($44) + 4)|0;
 $47 = $46;
 HEAP32[$47>>2] = $43;
 $48 = $ret;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (_bitshift64Shl(($50|0),($53|0),3)|0);
 $55 = tempRet0;
 $56 = HEAP32[$5>>2]|0;
 $57 = ($56|0)<(0);
 $58 = $57 << 31 >> 31;
 $59 = (_i64Add(($54|0),($55|0),($56|0),($58|0))|0);
 $60 = tempRet0;
 $61 = $ret;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = (_i64Add(($63|0),($66|0),($59|0),($60|0))|0);
 $68 = tempRet0;
 $69 = $ret;
 $70 = $69;
 HEAP32[$70>>2] = $67;
 $71 = (($69) + 4)|0;
 $72 = $71;
 HEAP32[$72>>2] = $68;
 $73 = $ret;
 $74 = $73;
 $75 = HEAP32[$74>>2]|0;
 $76 = (($73) + 4)|0;
 $77 = $76;
 $78 = HEAP32[$77>>2]|0;
 $79 = (_bitshift64Shl(($75|0),($78|0),3)|0);
 $80 = tempRet0;
 $81 = HEAP32[$7>>2]|0;
 $82 = ($81|0)<(0);
 $83 = $82 << 31 >> 31;
 $84 = (_i64Add(($79|0),($80|0),($81|0),($83|0))|0);
 $85 = tempRet0;
 $86 = $ret;
 $87 = $86;
 $88 = HEAP32[$87>>2]|0;
 $89 = (($86) + 4)|0;
 $90 = $89;
 $91 = HEAP32[$90>>2]|0;
 $92 = (_i64Add(($88|0),($91|0),($84|0),($85|0))|0);
 $93 = tempRet0;
 $94 = $ret;
 $95 = $94;
 HEAP32[$95>>2] = $92;
 $96 = (($94) + 4)|0;
 $97 = $96;
 HEAP32[$97>>2] = $93;
 $98 = $ret;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = (($98) + 4)|0;
 $102 = $101;
 $103 = HEAP32[$102>>2]|0;
 $104 = (_bitshift64Shl(($100|0),($103|0),3)|0);
 $105 = tempRet0;
 $106 = HEAP32[$8>>2]|0;
 $107 = ($106|0)<(0);
 $108 = $107 << 31 >> 31;
 $109 = (_i64Add(($104|0),($105|0),($106|0),($108|0))|0);
 $110 = tempRet0;
 $111 = $ret;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = (($111) + 4)|0;
 $115 = $114;
 $116 = HEAP32[$115>>2]|0;
 $117 = (_i64Add(($113|0),($116|0),($109|0),($110|0))|0);
 $118 = tempRet0;
 $119 = $ret;
 $120 = $119;
 HEAP32[$120>>2] = $117;
 $121 = (($119) + 4)|0;
 $122 = $121;
 HEAP32[$122>>2] = $118;
 HEAP32[$i>>2] = 0;
 while(1) {
  $123 = HEAP32[$i>>2]|0;
  $124 = HEAP32[$3>>2]|0;
  $125 = ($123|0)<($124|0);
  $126 = $ret;
  $127 = $126;
  $128 = HEAP32[$127>>2]|0;
  $129 = (($126) + 4)|0;
  $130 = $129;
  $131 = HEAP32[$130>>2]|0;
  if (!($125)) {
   break;
  }
  $132 = (_bitshift64Shl(($128|0),($131|0),3)|0);
  $133 = tempRet0;
  $134 = HEAP32[$i>>2]|0;
  $135 = HEAP32[$2>>2]|0;
  $136 = (($135) + ($134)|0);
  $137 = HEAP8[$136>>0]|0;
  $138 = $137 << 24 >> 24;
  $139 = ($138|0)<(0);
  $140 = $139 << 31 >> 31;
  $141 = (_i64Add(($132|0),($133|0),($138|0),($140|0))|0);
  $142 = tempRet0;
  $143 = $ret;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (($143) + 4)|0;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (_i64Add(($145|0),($148|0),($141|0),($142|0))|0);
  $150 = tempRet0;
  $151 = $ret;
  $152 = $151;
  HEAP32[$152>>2] = $149;
  $153 = (($151) + 4)|0;
  $154 = $153;
  HEAP32[$154>>2] = $150;
  $155 = HEAP32[$i>>2]|0;
  $156 = (($155) + 1)|0;
  HEAP32[$i>>2] = $156;
 }
 tempRet0 = ($131);
 STACKTOP = sp;return ($128|0);
}
function _fts3EvalNext($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pExpr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pExpr = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pExpr>>2] = $3;
 $4 = HEAP32[$pExpr>>2]|0;
 $5 = ($4|0)==(0|0);
 L1: do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 6|0);
   HEAP8[$7>>0] = 1;
  } else {
   while(1) {
    $8 = HEAP32[$0>>2]|0;
    $9 = ((($8)) + 7|0);
    $10 = HEAP8[$9>>0]|0;
    $11 = $10&255;
    $12 = ($11|0)==(0);
    if ($12) {
     $13 = HEAP32[$0>>2]|0;
     $14 = ((($13)) + 8|0);
     $15 = HEAP32[$14>>2]|0;
     (_sqlite3_reset($15)|0);
    }
    $16 = HEAP32[$0>>2]|0;
    $17 = HEAP32[$pExpr>>2]|0;
    _fts3EvalNextRow($16,$17,$rc);
    $18 = HEAP32[$pExpr>>2]|0;
    $19 = ((($18)) + 32|0);
    $20 = HEAP8[$19>>0]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ((($21)) + 6|0);
    HEAP8[$22>>0] = $20;
    $23 = HEAP32[$0>>2]|0;
    $24 = ((($23)) + 7|0);
    HEAP8[$24>>0] = 1;
    $25 = HEAP32[$0>>2]|0;
    $26 = ((($25)) + 88|0);
    HEAP32[$26>>2] = 1;
    $27 = HEAP32[$pExpr>>2]|0;
    $28 = ((($27)) + 24|0);
    $29 = $28;
    $30 = $29;
    $31 = HEAP32[$30>>2]|0;
    $32 = (($29) + 4)|0;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 32|0);
    $37 = $36;
    $38 = $37;
    HEAP32[$38>>2] = $31;
    $39 = (($37) + 4)|0;
    $40 = $39;
    HEAP32[$40>>2] = $34;
    $41 = HEAP32[$0>>2]|0;
    $42 = ((($41)) + 6|0);
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = ($44|0)==(0);
    if (!($45)) {
     break L1;
    }
    $46 = HEAP32[$0>>2]|0;
    $47 = (_sqlite3Fts3EvalTestDeferred($46,$rc)|0);
    $48 = ($47|0)!=(0);
    if (!($48)) {
     break;
    }
   }
  }
 } while(0);
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if (!($50)) {
  $105 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($105|0);
 }
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 52|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = ($54|0)==(0);
 if ($55) {
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 32|0);
  $58 = $57;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$0>>2]|0;
  $65 = ((($64)) + 80|0);
  $66 = $65;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = (($66) + 4)|0;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = ($63|0)>($71|0);
  $73 = ($60>>>0)>($68>>>0);
  $74 = ($63|0)==($71|0);
  $75 = $74 & $73;
  $76 = $72 | $75;
  if (!($76)) {
   label = 10;
  }
 } else {
  label = 10;
 }
 if ((label|0) == 10) {
  $77 = HEAP32[$0>>2]|0;
  $78 = ((($77)) + 52|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = ($80|0)!=(0);
  if (!($81)) {
   $105 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($105|0);
  }
  $82 = HEAP32[$0>>2]|0;
  $83 = ((($82)) + 32|0);
  $84 = $83;
  $85 = $84;
  $86 = HEAP32[$85>>2]|0;
  $87 = (($84) + 4)|0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = HEAP32[$0>>2]|0;
  $91 = ((($90)) + 72|0);
  $92 = $91;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($89|0)<($97|0);
  $99 = ($86>>>0)<($94>>>0);
  $100 = ($89|0)==($97|0);
  $101 = $100 & $99;
  $102 = $98 | $101;
  if (!($102)) {
   $105 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($105|0);
  }
 }
 $103 = HEAP32[$0>>2]|0;
 $104 = ((($103)) + 6|0);
 HEAP8[$104>>0] = 1;
 $105 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($105|0);
}
function _sqlite3Fts3ExprFree($pDel) {
 $pDel = $pDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $p = 0, $pParent = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pParent = sp;
 HEAP32[$0>>2] = $pDel;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 while(1) {
  $2 = HEAP32[$p>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 12|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   $8 = HEAP32[$p>>2]|0;
   $9 = ((($8)) + 16|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if (!($11)) {
    break;
   }
  }
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  $16 = HEAP32[$p>>2]|0;
  if ($15) {
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $21 = $18;
  } else {
   $19 = ((($16)) + 16|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $20;
  }
  HEAP32[$p>>2] = $21;
 }
 L11: while(1) {
  $22 = HEAP32[$p>>2]|0;
  $23 = ($22|0)!=(0|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pParent>>2] = $26;
  $27 = HEAP32[$p>>2]|0;
  _fts3FreeExprNode($27);
  $28 = HEAP32[$pParent>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$p>>2]|0;
   $31 = HEAP32[$pParent>>2]|0;
   $32 = ((($31)) + 12|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($30|0)==($33|0);
   if ($34) {
    $35 = HEAP32[$pParent>>2]|0;
    $36 = ((($35)) + 16|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)!=(0|0);
    if ($38) {
     $39 = HEAP32[$pParent>>2]|0;
     $40 = ((($39)) + 16|0);
     $41 = HEAP32[$40>>2]|0;
     HEAP32[$p>>2] = $41;
     while(1) {
      $42 = HEAP32[$p>>2]|0;
      $43 = ($42|0)!=(0|0);
      if (!($43)) {
       continue L11;
      }
      $44 = HEAP32[$p>>2]|0;
      $45 = ((($44)) + 12|0);
      $46 = HEAP32[$45>>2]|0;
      $47 = ($46|0)!=(0|0);
      if (!($47)) {
       $48 = HEAP32[$p>>2]|0;
       $49 = ((($48)) + 16|0);
       $50 = HEAP32[$49>>2]|0;
       $51 = ($50|0)!=(0|0);
       if (!($51)) {
        continue L11;
       }
      }
      $52 = HEAP32[$p>>2]|0;
      $53 = ((($52)) + 12|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($54|0)!=(0|0);
      $56 = HEAP32[$p>>2]|0;
      if ($55) {
       $57 = ((($56)) + 12|0);
       $58 = HEAP32[$57>>2]|0;
       $61 = $58;
      } else {
       $59 = ((($56)) + 16|0);
       $60 = HEAP32[$59>>2]|0;
       $61 = $60;
      }
      HEAP32[$p>>2] = $61;
     }
    }
   }
  }
  $62 = HEAP32[$pParent>>2]|0;
  HEAP32[$p>>2] = $62;
 }
 STACKTOP = sp;return;
}
function _fts3DocidRange($pVal,$0,$1) {
 $pVal = $pVal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $4 = sp;
 $eType = sp + 16|0;
 HEAP32[$3>>2] = $pVal;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3_value_numeric_type($11)|0);
  HEAP32[$eType>>2] = $12;
  $13 = HEAP32[$eType>>2]|0;
  $14 = ($13|0)==(1);
  if ($14) {
   $15 = HEAP32[$3>>2]|0;
   $16 = (_sqlite3_value_int64($15)|0);
   $17 = tempRet0;
   $18 = $2;
   $19 = $18;
   HEAP32[$19>>2] = $16;
   $20 = (($18) + 4)|0;
   $21 = $20;
   HEAP32[$21>>2] = $17;
   $32 = $2;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = (($32) + 4)|0;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   tempRet0 = ($37);
   STACKTOP = sp;return ($34|0);
  }
 }
 $22 = $4;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = $2;
 $29 = $28;
 HEAP32[$29>>2] = $24;
 $30 = (($28) + 4)|0;
 $31 = $30;
 HEAP32[$31>>2] = $27;
 $32 = $2;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 tempRet0 = ($37);
 STACKTOP = sp;return ($34|0);
}
function _sqlite3Fts3ExprParse($pTokenizer,$iLangid,$azCol,$bFts4,$nCol,$iDefaultCol,$z,$n,$ppExpr,$pzErr) {
 $pTokenizer = $pTokenizer|0;
 $iLangid = $iLangid|0;
 $azCol = $azCol|0;
 $bFts4 = $bFts4|0;
 $nCol = $nCol|0;
 $iDefaultCol = $iDefaultCol|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $9 = sp + 16|0;
 $rc = sp + 12|0;
 HEAP32[$0>>2] = $pTokenizer;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $azCol;
 HEAP32[$3>>2] = $bFts4;
 HEAP32[$4>>2] = $nCol;
 HEAP32[$5>>2] = $iDefaultCol;
 HEAP32[$6>>2] = $z;
 HEAP32[$7>>2] = $n;
 HEAP32[$8>>2] = $ppExpr;
 HEAP32[$9>>2] = $pzErr;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$6>>2]|0;
 $17 = HEAP32[$7>>2]|0;
 $18 = HEAP32[$8>>2]|0;
 $19 = (_fts3ExprParseUnbalanced($10,$11,$12,$13,$14,$15,$16,$17,$18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  $22 = HEAP32[$8>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$8>>2]|0;
   $26 = (_fts3ExprBalance($25,12)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$rc>>2]|0;
   $28 = ($27|0)==(0);
   if ($28) {
    $29 = HEAP32[$8>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = (_fts3ExprCheckDepth($30,12)|0);
    HEAP32[$rc>>2] = $31;
   }
  }
 }
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)!=(0);
 if (!($33)) {
  $44 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $34 = HEAP32[$8>>2]|0;
 $35 = HEAP32[$34>>2]|0;
 _sqlite3Fts3ExprFree($35);
 $36 = HEAP32[$8>>2]|0;
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(18);
 if ($38) {
  $39 = HEAP32[$9>>2]|0;
  HEAP32[$vararg_buffer>>2] = 12;
  _sqlite3Fts3ErrMsg($39,23723,$vararg_buffer);
  HEAP32[$rc>>2] = 1;
  $44 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(1);
 if (!($41)) {
  $44 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $42 = HEAP32[$9>>2]|0;
 $43 = HEAP32[$6>>2]|0;
 HEAP32[$vararg_buffer1>>2] = $43;
 _sqlite3Fts3ErrMsg($42,23775,$vararg_buffer1);
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _fts3EvalStart($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $aTC = 0, $apOr = 0, $ii = 0, $nOr = 0, $nToken = 0, $or$cond = 0, $pTC = 0;
 var $pTab = 0, $ppOr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $pTab = sp + 32|0;
 $rc = sp + 28|0;
 $nToken = sp + 24|0;
 $nOr = sp + 20|0;
 $aTC = sp + 16|0;
 $apOr = sp + 12|0;
 $ii = sp + 8|0;
 $pTC = sp + 4|0;
 $ppOr = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pTab>>2] = $2;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nToken>>2] = 0;
 HEAP32[$nOr>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 _fts3EvalAllocateReaders($3,$6,$nToken,$nOr,$rc);
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 $9 = HEAP32[$nToken>>2]|0;
 $10 = ($9|0)>(1);
 $or$cond = $8 & $10;
 do {
  if ($or$cond) {
   $11 = HEAP32[$pTab>>2]|0;
   $12 = ((($11)) + 228|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = HEAP32[$nToken>>2]|0;
    $17 = ($16*24)|0;
    $18 = HEAP32[$nOr>>2]|0;
    $19 = $18<<2;
    $20 = $19<<1;
    $21 = (($17) + ($20))|0;
    $22 = (_sqlite3_malloc($21)|0);
    HEAP32[$aTC>>2] = $22;
    $23 = HEAP32[$nToken>>2]|0;
    $24 = HEAP32[$aTC>>2]|0;
    $25 = (($24) + (($23*24)|0)|0);
    HEAP32[$apOr>>2] = $25;
    $26 = HEAP32[$aTC>>2]|0;
    $27 = ($26|0)!=(0|0);
    if (!($27)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    $28 = HEAP32[$aTC>>2]|0;
    HEAP32[$pTC>>2] = $28;
    $29 = HEAP32[$apOr>>2]|0;
    HEAP32[$ppOr>>2] = $29;
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    _fts3EvalTokenCosts($30,0,$33,$pTC,$ppOr,$rc);
    $34 = HEAP32[$pTC>>2]|0;
    $35 = HEAP32[$aTC>>2]|0;
    $36 = $34;
    $37 = $35;
    $38 = (($36) - ($37))|0;
    $39 = (($38|0) / 24)&-1;
    HEAP32[$nToken>>2] = $39;
    $40 = HEAP32[$ppOr>>2]|0;
    $41 = HEAP32[$apOr>>2]|0;
    $42 = $40;
    $43 = $41;
    $44 = (($42) - ($43))|0;
    $45 = (($44|0) / 4)&-1;
    HEAP32[$nOr>>2] = $45;
    $46 = HEAP32[$rc>>2]|0;
    $47 = ($46|0)==(0);
    L7: do {
     if ($47) {
      $48 = HEAP32[$0>>2]|0;
      $49 = HEAP32[$aTC>>2]|0;
      $50 = HEAP32[$nToken>>2]|0;
      $51 = (_fts3EvalSelectDeferred($48,0,$49,$50)|0);
      HEAP32[$rc>>2] = $51;
      HEAP32[$ii>>2] = 0;
      while(1) {
       $52 = HEAP32[$rc>>2]|0;
       $53 = ($52|0)==(0);
       if (!($53)) {
        break L7;
       }
       $54 = HEAP32[$ii>>2]|0;
       $55 = HEAP32[$nOr>>2]|0;
       $56 = ($54|0)<($55|0);
       if (!($56)) {
        break L7;
       }
       $57 = HEAP32[$0>>2]|0;
       $58 = HEAP32[$ii>>2]|0;
       $59 = HEAP32[$apOr>>2]|0;
       $60 = (($59) + ($58<<2)|0);
       $61 = HEAP32[$60>>2]|0;
       $62 = HEAP32[$aTC>>2]|0;
       $63 = HEAP32[$nToken>>2]|0;
       $64 = (_fts3EvalSelectDeferred($57,$61,$62,$63)|0);
       HEAP32[$rc>>2] = $64;
       $65 = HEAP32[$ii>>2]|0;
       $66 = (($65) + 1)|0;
       HEAP32[$ii>>2] = $66;
      }
     }
    } while(0);
    $67 = HEAP32[$aTC>>2]|0;
    _sqlite3_free($67);
   }
  }
 } while(0);
 $68 = HEAP32[$0>>2]|0;
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 12|0);
 $71 = HEAP32[$70>>2]|0;
 _fts3EvalStartReaders($68,$71,$rc);
 $72 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _fts3EvalAllocateReaders($pCsr,$pExpr,$pnToken,$pnOr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pnToken = $pnToken|0;
 $pnOr = $pnOr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $nToken = 0, $pToken = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $nToken = sp + 8|0;
 $pToken = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pnToken;
 HEAP32[$3>>2] = $pnOr;
 HEAP32[$4>>2] = $pRc;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$4>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (0)==($8|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(5);
 $13 = HEAP32[$1>>2]|0;
 if (!($12)) {
  $53 = HEAP32[$13>>2]|0;
  $54 = ($53|0)==(4);
  $55 = $54&1;
  $56 = HEAP32[$3>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + ($55))|0;
  HEAP32[$56>>2] = $58;
  $59 = HEAP32[$0>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 12|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$2>>2]|0;
  $64 = HEAP32[$3>>2]|0;
  $65 = HEAP32[$4>>2]|0;
  _fts3EvalAllocateReaders($59,$62,$63,$64,$65);
  $66 = HEAP32[$0>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  $68 = ((($67)) + 16|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = HEAP32[$3>>2]|0;
  $72 = HEAP32[$4>>2]|0;
  _fts3EvalAllocateReaders($66,$69,$70,$71,$72);
  STACKTOP = sp;return;
 }
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$nToken>>2] = $17;
 $18 = HEAP32[$nToken>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($18))|0;
 HEAP32[$19>>2] = $21;
 HEAP32[$i>>2] = 0;
 while(1) {
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$nToken>>2]|0;
  $24 = ($22|0)<($23|0);
  if (!($24)) {
   label = 9;
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 20|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 72|0);
  $30 = (($29) + (($25*24)|0)|0);
  HEAP32[$pToken>>2] = $30;
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$pToken>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$pToken>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$pToken>>2]|0;
  $38 = ((($37)) + 8|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$pToken>>2]|0;
  $41 = ((($40)) + 20|0);
  $42 = (_fts3TermSegReaderCursor($31,$33,$36,$39,$41)|0);
  HEAP32[$rc>>2] = $42;
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)!=(0);
  if ($44) {
   label = 7;
   break;
  }
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$i>>2] = $48;
 }
 if ((label|0) == 7) {
  $45 = HEAP32[$rc>>2]|0;
  $46 = HEAP32[$4>>2]|0;
  HEAP32[$46>>2] = $45;
  STACKTOP = sp;return;
 }
 else if ((label|0) == 9) {
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 20|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = ((($51)) + 44|0);
  HEAP32[$52>>2] = -1;
  STACKTOP = sp;return;
 }
}
function _fts3EvalTokenCosts($pCsr,$pRoot,$pExpr,$ppTC,$ppOr,$pRc) {
 $pCsr = $pCsr|0;
 $pRoot = $pRoot|0;
 $pExpr = $pExpr|0;
 $ppTC = $ppTC|0;
 $ppOr = $ppOr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $i = 0;
 var $pPhrase = 0, $pTC = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $pPhrase = sp + 8|0;
 $i = sp + 4|0;
 $pTC = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $ppTC;
 HEAP32[$4>>2] = $ppOr;
 HEAP32[$5>>2] = $pRc;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(5);
 $12 = HEAP32[$2>>2]|0;
 if ($11) {
  $13 = ((($12)) + 20|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pPhrase>>2] = $14;
  HEAP32[$i>>2] = 0;
  while(1) {
   $15 = HEAP32[$5>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)==(0);
   if (!($17)) {
    label = 13;
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$pPhrase>>2]|0;
   $20 = ((($19)) + 64|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($18|0)<($21|0);
   if (!($22)) {
    label = 13;
    break;
   }
   $23 = HEAP32[$3>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = ((($24)) + 24|0);
   HEAP32[$23>>2] = $25;
   HEAP32[$pTC>>2] = $24;
   $26 = HEAP32[$pPhrase>>2]|0;
   $27 = HEAP32[$pTC>>2]|0;
   HEAP32[$27>>2] = $26;
   $28 = HEAP32[$i>>2]|0;
   $29 = HEAP32[$pTC>>2]|0;
   $30 = ((($29)) + 4|0);
   HEAP32[$30>>2] = $28;
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP32[$pTC>>2]|0;
   $33 = ((($32)) + 12|0);
   HEAP32[$33>>2] = $31;
   $34 = HEAP32[$i>>2]|0;
   $35 = HEAP32[$pPhrase>>2]|0;
   $36 = ((($35)) + 72|0);
   $37 = (($36) + (($34*24)|0)|0);
   $38 = HEAP32[$pTC>>2]|0;
   $39 = ((($38)) + 8|0);
   HEAP32[$39>>2] = $37;
   $40 = HEAP32[$pPhrase>>2]|0;
   $41 = ((($40)) + 68|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$pTC>>2]|0;
   $44 = ((($43)) + 20|0);
   HEAP32[$44>>2] = $42;
   $45 = HEAP32[$0>>2]|0;
   $46 = HEAP32[$pTC>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ((($48)) + 20|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$pTC>>2]|0;
   $52 = ((($51)) + 16|0);
   $53 = (_sqlite3Fts3MsrOvfl($45,$50,$52)|0);
   $54 = HEAP32[$5>>2]|0;
   HEAP32[$54>>2] = $53;
   $55 = HEAP32[$i>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$i>>2] = $56;
  }
  if ((label|0) == 13) {
   STACKTOP = sp;return;
  }
 }
 $57 = HEAP32[$12>>2]|0;
 $58 = ($57|0)!=(2);
 if (!($58)) {
  STACKTOP = sp;return;
 }
 $59 = HEAP32[$2>>2]|0;
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)==(4);
 if ($61) {
  $62 = HEAP32[$2>>2]|0;
  $63 = ((($62)) + 12|0);
  $64 = HEAP32[$63>>2]|0;
  HEAP32[$1>>2] = $64;
  $65 = HEAP32[$1>>2]|0;
  $66 = HEAP32[$4>>2]|0;
  $67 = HEAP32[$66>>2]|0;
  HEAP32[$67>>2] = $65;
  $68 = HEAP32[$4>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  $70 = ((($69)) + 4|0);
  HEAP32[$68>>2] = $70;
 }
 $71 = HEAP32[$0>>2]|0;
 $72 = HEAP32[$1>>2]|0;
 $73 = HEAP32[$2>>2]|0;
 $74 = ((($73)) + 12|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = HEAP32[$3>>2]|0;
 $77 = HEAP32[$4>>2]|0;
 $78 = HEAP32[$5>>2]|0;
 _fts3EvalTokenCosts($71,$72,$75,$76,$77,$78);
 $79 = HEAP32[$2>>2]|0;
 $80 = HEAP32[$79>>2]|0;
 $81 = ($80|0)==(4);
 if ($81) {
  $82 = HEAP32[$2>>2]|0;
  $83 = ((($82)) + 16|0);
  $84 = HEAP32[$83>>2]|0;
  HEAP32[$1>>2] = $84;
  $85 = HEAP32[$1>>2]|0;
  $86 = HEAP32[$4>>2]|0;
  $87 = HEAP32[$86>>2]|0;
  HEAP32[$87>>2] = $85;
  $88 = HEAP32[$4>>2]|0;
  $89 = HEAP32[$88>>2]|0;
  $90 = ((($89)) + 4|0);
  HEAP32[$88>>2] = $90;
 }
 $91 = HEAP32[$0>>2]|0;
 $92 = HEAP32[$1>>2]|0;
 $93 = HEAP32[$2>>2]|0;
 $94 = ((($93)) + 16|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = HEAP32[$3>>2]|0;
 $97 = HEAP32[$4>>2]|0;
 $98 = HEAP32[$5>>2]|0;
 _fts3EvalTokenCosts($91,$92,$95,$96,$97,$98);
 STACKTOP = sp;return;
}
function _fts3EvalSelectDeferred($pCsr,$pRoot,$aTC,$nTC) {
 $pCsr = $pCsr|0;
 $pRoot = $pRoot|0;
 $aTC = $aTC|0;
 $nTC = $nTC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iTC = 0, $ii = 0, $nCount = 0, $nDocSize = 0, $nList = 0, $nLoad4 = 0, $nMinEst = 0, $nOvfl = 0, $nToken = 0, $or$cond = 0, $pList = 0, $pTC = 0, $pTab = 0, $pToken = 0, $pToken1 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $pTab = sp + 56|0;
 $nDocSize = sp + 52|0;
 $rc = sp + 48|0;
 $ii = sp + 44|0;
 $nOvfl = sp + 40|0;
 $nToken = sp + 36|0;
 $nMinEst = sp + 32|0;
 $nLoad4 = sp + 28|0;
 $iTC = sp + 24|0;
 $pTC = sp + 20|0;
 $pToken = sp + 16|0;
 $pToken1 = sp + 12|0;
 $nList = sp + 8|0;
 $pList = sp + 4|0;
 $nCount = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pRoot;
 HEAP32[$3>>2] = $aTC;
 HEAP32[$4>>2] = $nTC;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTab>>2] = $6;
 HEAP32[$nDocSize>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nOvfl>>2] = 0;
 HEAP32[$nToken>>2] = 0;
 HEAP32[$nMinEst>>2] = 0;
 HEAP32[$nLoad4>>2] = 1;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ((($7)) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $11 = HEAP32[$ii>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  $13 = ($11|0)<($12|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$ii>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = (($15) + (($14*24)|0)|0);
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = ($18|0)==($19|0);
  if ($20) {
   $21 = HEAP32[$ii>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = (($22) + (($21*24)|0)|0);
   $24 = ((($23)) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$nOvfl>>2]|0;
   $27 = (($26) + ($25))|0;
   HEAP32[$nOvfl>>2] = $27;
   $28 = HEAP32[$nToken>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$nToken>>2] = $29;
  }
  $30 = HEAP32[$ii>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$ii>>2] = $31;
 }
 $32 = HEAP32[$nOvfl>>2]|0;
 $33 = ($32|0)==(0);
 $34 = HEAP32[$nToken>>2]|0;
 $35 = ($34|0)<(2);
 $or$cond = $33 | $35;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = (_fts3EvalAverageDocsize($36,$nDocSize)|0);
 HEAP32[$rc>>2] = $37;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $38 = HEAP32[$ii>>2]|0;
  $39 = HEAP32[$nToken>>2]|0;
  $40 = ($38|0)<($39|0);
  $41 = HEAP32[$rc>>2]|0;
  $42 = ($41|0)==(0);
  $43 = $40 ? $42 : 0;
  if (!($43)) {
   break;
  }
  HEAP32[$pTC>>2] = 0;
  HEAP32[$iTC>>2] = 0;
  while(1) {
   $44 = HEAP32[$iTC>>2]|0;
   $45 = HEAP32[$4>>2]|0;
   $46 = ($44|0)<($45|0);
   if (!($46)) {
    break;
   }
   $47 = HEAP32[$iTC>>2]|0;
   $48 = HEAP32[$3>>2]|0;
   $49 = (($48) + (($47*24)|0)|0);
   $50 = ((($49)) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($51|0)!=(0|0);
   do {
    if ($52) {
     $53 = HEAP32[$iTC>>2]|0;
     $54 = HEAP32[$3>>2]|0;
     $55 = (($54) + (($53*24)|0)|0);
     $56 = ((($55)) + 12|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = HEAP32[$2>>2]|0;
     $59 = ($57|0)==($58|0);
     if ($59) {
      $60 = HEAP32[$pTC>>2]|0;
      $61 = ($60|0)!=(0|0);
      if ($61) {
       $62 = HEAP32[$iTC>>2]|0;
       $63 = HEAP32[$3>>2]|0;
       $64 = (($63) + (($62*24)|0)|0);
       $65 = ((($64)) + 16|0);
       $66 = HEAP32[$65>>2]|0;
       $67 = HEAP32[$pTC>>2]|0;
       $68 = ((($67)) + 16|0);
       $69 = HEAP32[$68>>2]|0;
       $70 = ($66|0)<($69|0);
       if (!($70)) {
        break;
       }
      }
      $71 = HEAP32[$iTC>>2]|0;
      $72 = HEAP32[$3>>2]|0;
      $73 = (($72) + (($71*24)|0)|0);
      HEAP32[$pTC>>2] = $73;
     }
    }
   } while(0);
   $74 = HEAP32[$iTC>>2]|0;
   $75 = (($74) + 1)|0;
   HEAP32[$iTC>>2] = $75;
  }
  $76 = HEAP32[$ii>>2]|0;
  $77 = ($76|0)!=(0);
  if ($77) {
   $78 = HEAP32[$pTC>>2]|0;
   $79 = ((($78)) + 16|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = HEAP32[$nMinEst>>2]|0;
   $82 = HEAP32[$nLoad4>>2]|0;
   $83 = (($82|0) / 4)&-1;
   $84 = (($81) + ($83))|0;
   $85 = (($84) - 1)|0;
   $86 = HEAP32[$nLoad4>>2]|0;
   $87 = (($86|0) / 4)&-1;
   $88 = (($85|0) / ($87|0))&-1;
   $89 = HEAP32[$nDocSize>>2]|0;
   $90 = Math_imul($88, $89)|0;
   $91 = ($80|0)>=($90|0);
   if ($91) {
    $92 = HEAP32[$pTC>>2]|0;
    $93 = ((($92)) + 8|0);
    $94 = HEAP32[$93>>2]|0;
    HEAP32[$pToken>>2] = $94;
    $95 = HEAP32[$1>>2]|0;
    $96 = HEAP32[$pToken>>2]|0;
    $97 = HEAP32[$pTC>>2]|0;
    $98 = ((($97)) + 20|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = (_sqlite3Fts3DeferToken($95,$96,$99)|0);
    HEAP32[$rc>>2] = $100;
    $101 = HEAP32[$pToken>>2]|0;
    $102 = ((($101)) + 20|0);
    $103 = HEAP32[$102>>2]|0;
    _fts3SegReaderCursorFree($103);
    $104 = HEAP32[$pToken>>2]|0;
    $105 = ((($104)) + 20|0);
    HEAP32[$105>>2] = 0;
   } else {
    label = 23;
   }
  } else {
   label = 23;
  }
  do {
   if ((label|0) == 23) {
    label = 0;
    $106 = HEAP32[$ii>>2]|0;
    $107 = ($106|0)<(12);
    if ($107) {
     $108 = HEAP32[$nLoad4>>2]|0;
     $109 = $108<<2;
     HEAP32[$nLoad4>>2] = $109;
    }
    $110 = HEAP32[$ii>>2]|0;
    $111 = ($110|0)==(0);
    if (!($111)) {
     $112 = HEAP32[$pTC>>2]|0;
     $113 = HEAP32[$112>>2]|0;
     $114 = ((($113)) + 64|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = ($115|0)>(1);
     if (!($116)) {
      break;
     }
     $117 = HEAP32[$ii>>2]|0;
     $118 = HEAP32[$nToken>>2]|0;
     $119 = (($118) - 1)|0;
     $120 = ($117|0)!=($119|0);
     if (!($120)) {
      break;
     }
    }
    $121 = HEAP32[$pTC>>2]|0;
    $122 = ((($121)) + 8|0);
    $123 = HEAP32[$122>>2]|0;
    HEAP32[$pToken1>>2] = $123;
    HEAP32[$nList>>2] = 0;
    HEAP32[$pList>>2] = 0;
    $124 = HEAP32[$pTab>>2]|0;
    $125 = HEAP32[$pToken1>>2]|0;
    $126 = HEAP32[$pTC>>2]|0;
    $127 = ((($126)) + 20|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = (_fts3TermSelect($124,$125,$128,$nList,$pList)|0);
    HEAP32[$rc>>2] = $129;
    $130 = HEAP32[$rc>>2]|0;
    $131 = ($130|0)==(0);
    if ($131) {
     $132 = HEAP32[$pTab>>2]|0;
     $133 = HEAP32[$pTC>>2]|0;
     $134 = HEAP32[$133>>2]|0;
     $135 = HEAP32[$pTC>>2]|0;
     $136 = ((($135)) + 4|0);
     $137 = HEAP32[$136>>2]|0;
     $138 = HEAP32[$pList>>2]|0;
     $139 = HEAP32[$nList>>2]|0;
     $140 = (_fts3EvalPhraseMergeToken($132,$134,$137,$138,$139)|0);
     HEAP32[$rc>>2] = $140;
    }
    $141 = HEAP32[$rc>>2]|0;
    $142 = ($141|0)==(0);
    if ($142) {
     $143 = HEAP32[$pTC>>2]|0;
     $144 = HEAP32[$143>>2]|0;
     $145 = HEAP32[$144>>2]|0;
     $146 = HEAP32[$pTC>>2]|0;
     $147 = HEAP32[$146>>2]|0;
     $148 = ((($147)) + 4|0);
     $149 = HEAP32[$148>>2]|0;
     $150 = (_fts3DoclistCountDocids($145,$149)|0);
     HEAP32[$nCount>>2] = $150;
     $151 = HEAP32[$ii>>2]|0;
     $152 = ($151|0)==(0);
     if (!($152)) {
      $153 = HEAP32[$nCount>>2]|0;
      $154 = HEAP32[$nMinEst>>2]|0;
      $155 = ($153|0)<($154|0);
      if (!($155)) {
       break;
      }
     }
     $156 = HEAP32[$nCount>>2]|0;
     HEAP32[$nMinEst>>2] = $156;
    }
   }
  } while(0);
  $157 = HEAP32[$pTC>>2]|0;
  $158 = ((($157)) + 8|0);
  HEAP32[$158>>2] = 0;
  $159 = HEAP32[$ii>>2]|0;
  $160 = (($159) + 1)|0;
  HEAP32[$ii>>2] = $160;
 }
 $161 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $161;
 $162 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _fts3EvalStartReaders($pCsr,$pExpr,$pRc) {
 $pCsr = $pCsr|0;
 $pExpr = $pExpr|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $i = 0, $nToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nToken = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pRc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (0)==($6|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(5);
 if (!($10)) {
  $45 = HEAP32[$0>>2]|0;
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 12|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$2>>2]|0;
  _fts3EvalStartReaders($45,$48,$49);
  $50 = HEAP32[$0>>2]|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = ((($51)) + 16|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  _fts3EvalStartReaders($50,$53,$54);
  $55 = HEAP32[$1>>2]|0;
  $56 = ((($55)) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ((($57)) + 34|0);
  $59 = HEAP8[$58>>0]|0;
  $60 = $59&255;
  $61 = ($60|0)!=(0);
  if ($61) {
   $62 = HEAP32[$1>>2]|0;
   $63 = ((($62)) + 16|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = ((($64)) + 34|0);
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   $68 = ($67|0)!=(0);
   $70 = $68;
  } else {
   $70 = 0;
  }
  $69 = $70&1;
  $71 = $69&255;
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 34|0);
  HEAP8[$73>>0] = $71;
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 64|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$nToken>>2] = $15;
 $16 = HEAP32[$nToken>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$nToken>>2]|0;
   $20 = ($18|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 20|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ((($24)) + 72|0);
   $26 = (($25) + (($21*24)|0)|0);
   $27 = ((($26)) + 16|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)==(0|0);
   if ($29) {
    break;
   }
   $30 = HEAP32[$i>>2]|0;
   $31 = (($30) + 1)|0;
   HEAP32[$i>>2] = $31;
  }
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP32[$nToken>>2]|0;
  $34 = ($32|0)==($33|0);
  $35 = $34&1;
  $36 = $35&255;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 34|0);
  HEAP8[$38>>0] = $36;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 20|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (_fts3EvalPhraseStart($39,1,$42)|0);
 $44 = HEAP32[$2>>2]|0;
 HEAP32[$44>>2] = $43;
 STACKTOP = sp;return;
}
function _fts3EvalAverageDocsize($pCsr,$pnPage) {
 $pCsr = $pCsr|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $nByte = 0, $nDoc = 0, $or$cond = 0, $p = 0, $pEnd = 0, $pStmt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $rc = sp + 32|0;
 $p = sp + 28|0;
 $pStmt = sp + 24|0;
 $nDoc = sp + 8|0;
 $nByte = sp;
 $pEnd = sp + 20|0;
 $a = sp + 16|0;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pnPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$p>>2] = $8;
  $9 = $nDoc;
  $10 = $9;
  HEAP32[$10>>2] = 0;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = 0;
  $13 = $nByte;
  $14 = $13;
  HEAP32[$14>>2] = 0;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = 0;
  $17 = HEAP32[$p>>2]|0;
  $18 = (_sqlite3Fts3SelectDoctotal($17,$pStmt)|0);
  HEAP32[$rc>>2] = $18;
  $19 = HEAP32[$rc>>2]|0;
  $20 = ($19|0)!=(0);
  if ($20) {
   $21 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $21;
   $109 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($109|0);
  }
  $22 = HEAP32[$pStmt>>2]|0;
  $23 = (_sqlite3_column_blob($22,0)|0);
  HEAP32[$a>>2] = $23;
  $24 = HEAP32[$pStmt>>2]|0;
  $25 = (_sqlite3_column_bytes($24,0)|0);
  $26 = HEAP32[$a>>2]|0;
  $27 = (($26) + ($25)|0);
  HEAP32[$pEnd>>2] = $27;
  $28 = HEAP32[$a>>2]|0;
  $29 = (_sqlite3Fts3GetVarint($28,$nDoc)|0);
  $30 = HEAP32[$a>>2]|0;
  $31 = (($30) + ($29)|0);
  HEAP32[$a>>2] = $31;
  while(1) {
   $32 = HEAP32[$a>>2]|0;
   $33 = HEAP32[$pEnd>>2]|0;
   $34 = ($32>>>0)<($33>>>0);
   if (!($34)) {
    break;
   }
   $35 = HEAP32[$a>>2]|0;
   $36 = (_sqlite3Fts3GetVarint($35,$nByte)|0);
   $37 = HEAP32[$a>>2]|0;
   $38 = (($37) + ($36)|0);
   HEAP32[$a>>2] = $38;
  }
  $39 = $nDoc;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($39) + 4)|0;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($41|0)==(0);
  $46 = ($44|0)==(0);
  $47 = $45 & $46;
  $48 = $nByte;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($48) + 4)|0;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = ($50|0)==(0);
  $55 = ($53|0)==(0);
  $56 = $54 & $55;
  $or$cond = $47 | $56;
  if ($or$cond) {
   $57 = HEAP32[$pStmt>>2]|0;
   (_sqlite3_reset($57)|0);
   HEAP32[$0>>2] = 267;
   $109 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($109|0);
  }
  $58 = $nDoc;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 64|0);
  $66 = $65;
  $67 = $66;
  HEAP32[$67>>2] = $60;
  $68 = (($66) + 4)|0;
  $69 = $68;
  HEAP32[$69>>2] = $63;
  $70 = $nByte;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $nDoc;
  $77 = $76;
  $78 = HEAP32[$77>>2]|0;
  $79 = (($76) + 4)|0;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (___divdi3(($72|0),($75|0),($78|0),($81|0))|0);
  $83 = tempRet0;
  $84 = HEAP32[$p>>2]|0;
  $85 = ((($84)) + 236|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = ($86|0)<(0);
  $88 = $87 << 31 >> 31;
  $89 = (_i64Add(($82|0),($83|0),($86|0),($88|0))|0);
  $90 = tempRet0;
  $91 = HEAP32[$p>>2]|0;
  $92 = ((($91)) + 236|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = ($93|0)<(0);
  $95 = $94 << 31 >> 31;
  $96 = (___divdi3(($89|0),($90|0),($93|0),($95|0))|0);
  $97 = tempRet0;
  $98 = HEAP32[$1>>2]|0;
  $99 = ((($98)) + 60|0);
  HEAP32[$99>>2] = $96;
  $100 = HEAP32[$pStmt>>2]|0;
  $101 = (_sqlite3_reset($100)|0);
  HEAP32[$rc>>2] = $101;
  $102 = HEAP32[$rc>>2]|0;
  $103 = ($102|0)!=(0);
  if ($103) {
   $104 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $104;
   $109 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($109|0);
  }
 }
 $105 = HEAP32[$1>>2]|0;
 $106 = ((($105)) + 60|0);
 $107 = HEAP32[$106>>2]|0;
 $108 = HEAP32[$2>>2]|0;
 HEAP32[$108>>2] = $107;
 HEAP32[$0>>2] = 0;
 $109 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($109|0);
}
function _sqlite3Fts3DeferToken($pCsr,$pToken,$iCol) {
 $pCsr = $pCsr|0;
 $pToken = $pToken|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDeferred = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pDeferred = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pToken;
 HEAP32[$3>>2] = $iCol;
 $4 = (_sqlite3_malloc(16)|0);
 HEAP32[$pDeferred>>2] = $4;
 $5 = HEAP32[$pDeferred>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$pDeferred>>2]|0;
  ;HEAP32[$7>>2]=0|0;HEAP32[$7+4>>2]=0|0;HEAP32[$7+8>>2]=0|0;HEAP32[$7+12>>2]=0|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$pDeferred>>2]|0;
  HEAP32[$9>>2] = $8;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$pDeferred>>2]|0;
  $14 = ((($13)) + 8|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$3>>2]|0;
  $16 = HEAP32[$pDeferred>>2]|0;
  $17 = ((($16)) + 4|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP32[$pDeferred>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 24|0);
  HEAP32[$20>>2] = $18;
  $21 = HEAP32[$pDeferred>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 16|0);
  HEAP32[$23>>2] = $21;
  HEAP32[$0>>2] = 0;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  HEAP32[$0>>2] = 7;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _fts3DoclistCountDocids($aList,$nList) {
 $aList = $aList|0;
 $nList = $nList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $aEnd = 0, $nDoc = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $nDoc = sp + 8|0;
 $aEnd = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $aList;
 HEAP32[$1>>2] = $nList;
 HEAP32[$nDoc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $19 = HEAP32[$nDoc>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (($5) + ($4)|0);
 HEAP32[$aEnd>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $7;
 while(1) {
  $8 = HEAP32[$p>>2]|0;
  $9 = HEAP32[$aEnd>>2]|0;
  $10 = ($8>>>0)<($9>>>0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$nDoc>>2]|0;
  $12 = (($11) + 1)|0;
  HEAP32[$nDoc>>2] = $12;
  while(1) {
   $13 = HEAP32[$p>>2]|0;
   $14 = ((($13)) + 1|0);
   HEAP32[$p>>2] = $14;
   $15 = HEAP8[$13>>0]|0;
   $16 = $15 << 24 >> 24;
   $17 = $16 & 128;
   $18 = ($17|0)!=(0);
   if (!($18)) {
    break;
   }
  }
  _fts3PoslistCopy(0,$p);
 }
 $19 = HEAP32[$nDoc>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3Fts3MsrOvfl($pCsr,$pMsr,$pnOvfl) {
 $pCsr = $pCsr|0;
 $pMsr = $pMsr|0;
 $pnOvfl = $pnOvfl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $ii = 0, $jj = 0, $nBlob = 0;
 var $nOvfl = 0, $p = 0, $pReader = 0, $pgsz = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $p = sp + 32|0;
 $nOvfl = sp + 28|0;
 $ii = sp + 24|0;
 $rc = sp + 20|0;
 $pgsz = sp + 16|0;
 $pReader = sp + 12|0;
 $jj = sp;
 $nBlob = sp + 8|0;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $pMsr;
 HEAP32[$2>>2] = $pnOvfl;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 HEAP32[$nOvfl>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 236|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgsz>>2] = $7;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $8 = HEAP32[$rc>>2]|0;
  $9 = ($8|0)==(0);
  if (!($9)) {
   label = 13;
   break;
  }
  $10 = HEAP32[$ii>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   label = 13;
   break;
  }
  $15 = HEAP32[$ii>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($15<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pReader>>2] = $19;
  $20 = HEAP32[$pReader>>2]|0;
  $21 = ((($20)) + 56|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  L5: do {
   if (!($23)) {
    $24 = HEAP32[$pReader>>2]|0;
    $25 = ((($24)) + 5|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = ($27|0)!=(0);
    if (!($28)) {
     $29 = HEAP32[$pReader>>2]|0;
     $30 = ((($29)) + 8|0);
     $31 = $30;
     $32 = $31;
     $33 = HEAP32[$32>>2]|0;
     $34 = (($31) + 4)|0;
     $35 = $34;
     $36 = HEAP32[$35>>2]|0;
     $37 = $jj;
     $38 = $37;
     HEAP32[$38>>2] = $33;
     $39 = (($37) + 4)|0;
     $40 = $39;
     HEAP32[$40>>2] = $36;
     while(1) {
      $41 = $jj;
      $42 = $41;
      $43 = HEAP32[$42>>2]|0;
      $44 = (($41) + 4)|0;
      $45 = $44;
      $46 = HEAP32[$45>>2]|0;
      $47 = HEAP32[$pReader>>2]|0;
      $48 = ((($47)) + 16|0);
      $49 = $48;
      $50 = $49;
      $51 = HEAP32[$50>>2]|0;
      $52 = (($49) + 4)|0;
      $53 = $52;
      $54 = HEAP32[$53>>2]|0;
      $55 = ($46|0)<($54|0);
      $56 = ($43>>>0)<=($51>>>0);
      $57 = ($46|0)==($54|0);
      $58 = $57 & $56;
      $59 = $55 | $58;
      if (!($59)) {
       break L5;
      }
      $60 = HEAP32[$p>>2]|0;
      $61 = $jj;
      $62 = $61;
      $63 = HEAP32[$62>>2]|0;
      $64 = (($61) + 4)|0;
      $65 = $64;
      $66 = HEAP32[$65>>2]|0;
      $67 = (_sqlite3Fts3ReadBlock($60,$63,$66,0,$nBlob,0)|0);
      HEAP32[$rc>>2] = $67;
      $68 = HEAP32[$rc>>2]|0;
      $69 = ($68|0)!=(0);
      if ($69) {
       break L5;
      }
      $70 = HEAP32[$nBlob>>2]|0;
      $71 = (($70) + 35)|0;
      $72 = HEAP32[$pgsz>>2]|0;
      $73 = ($71|0)>($72|0);
      if ($73) {
       $74 = HEAP32[$nBlob>>2]|0;
       $75 = (($74) + 34)|0;
       $76 = HEAP32[$pgsz>>2]|0;
       $77 = (($75|0) / ($76|0))&-1;
       $78 = HEAP32[$nOvfl>>2]|0;
       $79 = (($78) + ($77))|0;
       HEAP32[$nOvfl>>2] = $79;
      }
      $80 = $jj;
      $81 = $80;
      $82 = HEAP32[$81>>2]|0;
      $83 = (($80) + 4)|0;
      $84 = $83;
      $85 = HEAP32[$84>>2]|0;
      $86 = (_i64Add(($82|0),($85|0),1,0)|0);
      $87 = tempRet0;
      $88 = $jj;
      $89 = $88;
      HEAP32[$89>>2] = $86;
      $90 = (($88) + 4)|0;
      $91 = $90;
      HEAP32[$91>>2] = $87;
     }
    }
   }
  } while(0);
  $92 = HEAP32[$ii>>2]|0;
  $93 = (($92) + 1)|0;
  HEAP32[$ii>>2] = $93;
 }
 if ((label|0) == 13) {
  $94 = HEAP32[$nOvfl>>2]|0;
  $95 = HEAP32[$2>>2]|0;
  HEAP32[$95>>2] = $94;
  $96 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 return (0)|0;
}
function _fts3TermSegReaderCursor($pCsr,$zTerm,$nTerm,$isPrefix,$ppSegcsr) {
 $pCsr = $pCsr|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $isPrefix = $isPrefix|0;
 $ppSegcsr = $ppSegcsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $bFound = 0, $i = 0, $p = 0;
 var $pSegcsr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pSegcsr = sp + 16|0;
 $rc = sp + 12|0;
 $i = sp + 8|0;
 $bFound = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $zTerm;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$3>>2] = $isPrefix;
 HEAP32[$4>>2] = $ppSegcsr;
 HEAP32[$rc>>2] = 7;
 $5 = (_sqlite3_malloc(56)|0);
 HEAP32[$pSegcsr>>2] = $5;
 $6 = HEAP32[$pSegcsr>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $94 = HEAP32[$pSegcsr>>2]|0;
  $95 = HEAP32[$4>>2]|0;
  HEAP32[$95>>2] = $94;
  $96 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 HEAP32[$bFound>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10|0)!=(0);
 L4: do {
  if ($11) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $12 = HEAP32[$bFound>>2]|0;
    $13 = ($12|0)==(0);
    if (!($13)) {
     break;
    }
    $14 = HEAP32[$i>>2]|0;
    $15 = HEAP32[$p>>2]|0;
    $16 = ((($15)) + 248|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ($14|0)<($17|0);
    if (!($18)) {
     break;
    }
    $19 = HEAP32[$i>>2]|0;
    $20 = HEAP32[$p>>2]|0;
    $21 = ((($20)) + 252|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + (($19*24)|0)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$2>>2]|0;
    $26 = ($24|0)==($25|0);
    if ($26) {
     HEAP32[$bFound>>2] = 1;
     $27 = HEAP32[$p>>2]|0;
     $28 = HEAP32[$0>>2]|0;
     $29 = ((($28)) + 16|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$i>>2]|0;
     $32 = HEAP32[$1>>2]|0;
     $33 = HEAP32[$2>>2]|0;
     $34 = HEAP32[$pSegcsr>>2]|0;
     $35 = (_sqlite3Fts3SegReaderCursor($27,$30,$31,-2,$32,$33,0,0,$34)|0);
     HEAP32[$rc>>2] = $35;
     $36 = HEAP32[$pSegcsr>>2]|0;
     $37 = ((($36)) + 36|0);
     HEAP32[$37>>2] = 1;
    }
    $38 = HEAP32[$i>>2]|0;
    $39 = (($38) + 1)|0;
    HEAP32[$i>>2] = $39;
   }
   HEAP32[$i>>2] = 1;
   while(1) {
    $40 = HEAP32[$bFound>>2]|0;
    $41 = ($40|0)==(0);
    if (!($41)) {
     break L4;
    }
    $42 = HEAP32[$i>>2]|0;
    $43 = HEAP32[$p>>2]|0;
    $44 = ((($43)) + 248|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($42|0)<($45|0);
    if (!($46)) {
     break L4;
    }
    $47 = HEAP32[$i>>2]|0;
    $48 = HEAP32[$p>>2]|0;
    $49 = ((($48)) + 252|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + (($47*24)|0)|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = (($53) + 1)|0;
    $55 = ($52|0)==($54|0);
    if ($55) {
     HEAP32[$bFound>>2] = 1;
     $56 = HEAP32[$p>>2]|0;
     $57 = HEAP32[$0>>2]|0;
     $58 = ((($57)) + 16|0);
     $59 = HEAP32[$58>>2]|0;
     $60 = HEAP32[$i>>2]|0;
     $61 = HEAP32[$1>>2]|0;
     $62 = HEAP32[$2>>2]|0;
     $63 = HEAP32[$pSegcsr>>2]|0;
     $64 = (_sqlite3Fts3SegReaderCursor($56,$59,$60,-2,$61,$62,1,0,$63)|0);
     HEAP32[$rc>>2] = $64;
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)==(0);
     if ($66) {
      $67 = HEAP32[$p>>2]|0;
      $68 = HEAP32[$0>>2]|0;
      $69 = ((($68)) + 16|0);
      $70 = HEAP32[$69>>2]|0;
      $71 = HEAP32[$1>>2]|0;
      $72 = HEAP32[$2>>2]|0;
      $73 = HEAP32[$pSegcsr>>2]|0;
      $74 = (_fts3SegReaderCursorAddZero($67,$70,$71,$72,$73)|0);
      HEAP32[$rc>>2] = $74;
     }
    }
    $75 = HEAP32[$i>>2]|0;
    $76 = (($75) + 1)|0;
    HEAP32[$i>>2] = $76;
   }
  }
 } while(0);
 $77 = HEAP32[$bFound>>2]|0;
 $78 = ($77|0)==(0);
 if (!($78)) {
  $94 = HEAP32[$pSegcsr>>2]|0;
  $95 = HEAP32[$4>>2]|0;
  HEAP32[$95>>2] = $94;
  $96 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($96|0);
 }
 $79 = HEAP32[$p>>2]|0;
 $80 = HEAP32[$0>>2]|0;
 $81 = ((($80)) + 16|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = HEAP32[$1>>2]|0;
 $84 = HEAP32[$2>>2]|0;
 $85 = HEAP32[$3>>2]|0;
 $86 = HEAP32[$pSegcsr>>2]|0;
 $87 = (_sqlite3Fts3SegReaderCursor($79,$82,0,-2,$83,$84,$85,0,$86)|0);
 HEAP32[$rc>>2] = $87;
 $88 = HEAP32[$3>>2]|0;
 $89 = ($88|0)!=(0);
 $90 = $89 ^ 1;
 $91 = $90&1;
 $92 = HEAP32[$pSegcsr>>2]|0;
 $93 = ((($92)) + 36|0);
 HEAP32[$93>>2] = $91;
 $94 = HEAP32[$pSegcsr>>2]|0;
 $95 = HEAP32[$4>>2]|0;
 HEAP32[$95>>2] = $94;
 $96 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($96|0);
}
function _fts3SegReaderCursorAddZero($p,$iLangid,$zTerm,$nTerm,$pCsr) {
 $p = $p|0;
 $iLangid = $iLangid|0;
 $zTerm = $zTerm|0;
 $nTerm = $nTerm|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLangid;
 HEAP32[$2>>2] = $zTerm;
 HEAP32[$3>>2] = $nTerm;
 HEAP32[$4>>2] = $pCsr;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_fts3SegReaderCursor($5,$6,0,-2,$7,$8,0,0,$9)|0);
 STACKTOP = sp;return ($10|0);
}
function _fts3ExprParseUnbalanced($pTokenizer,$iLangid,$azCol,$bFts4,$nCol,$iDefaultCol,$z,$n,$ppExpr) {
 $pTokenizer = $pTokenizer|0;
 $iLangid = $iLangid|0;
 $azCol = $azCol|0;
 $bFts4 = $bFts4|0;
 $nCol = $nCol|0;
 $iDefaultCol = $iDefaultCol|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nParsed = 0;
 var $rc = 0, $sParse = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $6 = sp + 56|0;
 $7 = sp + 52|0;
 $8 = sp + 48|0;
 $9 = sp + 44|0;
 $nParsed = sp + 40|0;
 $rc = sp + 36|0;
 $sParse = sp;
 HEAP32[$1>>2] = $pTokenizer;
 HEAP32[$2>>2] = $iLangid;
 HEAP32[$3>>2] = $azCol;
 HEAP32[$4>>2] = $bFts4;
 HEAP32[$5>>2] = $nCol;
 HEAP32[$6>>2] = $iDefaultCol;
 HEAP32[$7>>2] = $z;
 HEAP32[$8>>2] = $n;
 HEAP32[$9>>2] = $ppExpr;
 dest=$sParse; stop=dest+36|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$sParse>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($sParse)) + 4|0);
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = ((($sParse)) + 8|0);
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$5>>2]|0;
 $16 = ((($sParse)) + 16|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$6>>2]|0;
 $18 = ((($sParse)) + 20|0);
 HEAP32[$18>>2] = $17;
 $19 = HEAP32[$4>>2]|0;
 $20 = ((($sParse)) + 12|0);
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$7>>2]|0;
 $22 = ($21|0)==(0|0);
 if ($22) {
  $23 = HEAP32[$9>>2]|0;
  HEAP32[$23>>2] = 0;
  HEAP32[$0>>2] = 0;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $24 = HEAP32[$8>>2]|0;
 $25 = ($24|0)<(0);
 if ($25) {
  $26 = HEAP32[$7>>2]|0;
  $27 = (_strlen($26)|0);
  HEAP32[$8>>2] = $27;
 }
 $28 = HEAP32[$7>>2]|0;
 $29 = HEAP32[$8>>2]|0;
 $30 = HEAP32[$9>>2]|0;
 $31 = (_fts3ExprParse($sParse,$28,$29,$30,$nParsed)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)==(0);
 if ($33) {
  $34 = ((($sParse)) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   HEAP32[$rc>>2] = 1;
  }
 }
 $37 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3ExprBalance($pp,$nMaxDepth) {
 $pp = $pp|0;
 $nMaxDepth = $nMaxDepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apLeaf = 0, $eType = 0, $i = 0, $iLvl = 0, $or$cond = 0;
 var $p = 0, $pDel = 0, $pFree = 0, $pLeft = 0, $pParent = 0, $pRight = 0, $pRoot = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $rc = sp + 44|0;
 $pRoot = sp + 40|0;
 $pFree = sp + 36|0;
 $eType = sp + 32|0;
 $apLeaf = sp + 28|0;
 $i = sp + 24|0;
 $p = sp + 20|0;
 $iLvl = sp + 16|0;
 $pParent = sp + 12|0;
 $pDel = sp + 8|0;
 $pLeft = sp + 4|0;
 $pRight = sp;
 HEAP32[$0>>2] = $pp;
 HEAP32[$1>>2] = $nMaxDepth;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pRoot>>2] = $3;
 HEAP32[$pFree>>2] = 0;
 $4 = HEAP32[$pRoot>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$eType>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$rc>>2] = 1;
 }
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 do {
  if ($9) {
   $10 = HEAP32[$eType>>2]|0;
   $11 = ($10|0)==(3);
   $12 = HEAP32[$eType>>2]|0;
   $13 = ($12|0)==(4);
   $or$cond = $11 | $13;
   if (!($or$cond)) {
    $189 = HEAP32[$eType>>2]|0;
    $190 = ($189|0)==(2);
    if (!($190)) {
     break;
    }
    $191 = HEAP32[$pRoot>>2]|0;
    $192 = ((($191)) + 12|0);
    $193 = HEAP32[$192>>2]|0;
    HEAP32[$pLeft>>2] = $193;
    $194 = HEAP32[$pRoot>>2]|0;
    $195 = ((($194)) + 16|0);
    $196 = HEAP32[$195>>2]|0;
    HEAP32[$pRight>>2] = $196;
    $197 = HEAP32[$pRoot>>2]|0;
    $198 = ((($197)) + 12|0);
    HEAP32[$198>>2] = 0;
    $199 = HEAP32[$pRoot>>2]|0;
    $200 = ((($199)) + 16|0);
    HEAP32[$200>>2] = 0;
    $201 = HEAP32[$pLeft>>2]|0;
    $202 = ((($201)) + 8|0);
    HEAP32[$202>>2] = 0;
    $203 = HEAP32[$pRight>>2]|0;
    $204 = ((($203)) + 8|0);
    HEAP32[$204>>2] = 0;
    $205 = HEAP32[$1>>2]|0;
    $206 = (($205) - 1)|0;
    $207 = (_fts3ExprBalance($pLeft,$206)|0);
    HEAP32[$rc>>2] = $207;
    $208 = HEAP32[$rc>>2]|0;
    $209 = ($208|0)==(0);
    if ($209) {
     $210 = HEAP32[$1>>2]|0;
     $211 = (($210) - 1)|0;
     $212 = (_fts3ExprBalance($pRight,$211)|0);
     HEAP32[$rc>>2] = $212;
    }
    $213 = HEAP32[$rc>>2]|0;
    $214 = ($213|0)!=(0);
    if ($214) {
     $215 = HEAP32[$pRight>>2]|0;
     _sqlite3Fts3ExprFree($215);
     $216 = HEAP32[$pLeft>>2]|0;
     _sqlite3Fts3ExprFree($216);
     break;
    } else {
     $217 = HEAP32[$pLeft>>2]|0;
     $218 = HEAP32[$pRoot>>2]|0;
     $219 = ((($218)) + 12|0);
     HEAP32[$219>>2] = $217;
     $220 = HEAP32[$pRoot>>2]|0;
     $221 = HEAP32[$pLeft>>2]|0;
     $222 = ((($221)) + 8|0);
     HEAP32[$222>>2] = $220;
     $223 = HEAP32[$pRight>>2]|0;
     $224 = HEAP32[$pRoot>>2]|0;
     $225 = ((($224)) + 16|0);
     HEAP32[$225>>2] = $223;
     $226 = HEAP32[$pRoot>>2]|0;
     $227 = HEAP32[$pRight>>2]|0;
     $228 = ((($227)) + 8|0);
     HEAP32[$228>>2] = $226;
     break;
    }
   }
   $14 = HEAP32[$1>>2]|0;
   $15 = $14<<2;
   $16 = (_sqlite3_malloc($15)|0);
   HEAP32[$apLeaf>>2] = $16;
   $17 = HEAP32[$apLeaf>>2]|0;
   $18 = (0|0)==($17|0);
   if ($18) {
    HEAP32[$rc>>2] = 7;
   } else {
    $19 = HEAP32[$apLeaf>>2]|0;
    $20 = HEAP32[$1>>2]|0;
    $21 = $20<<2;
    _memset(($19|0),0,($21|0))|0;
   }
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$pRoot>>2]|0;
    HEAP32[$p>>2] = $24;
    while(1) {
     $25 = HEAP32[$p>>2]|0;
     $26 = HEAP32[$25>>2]|0;
     $27 = HEAP32[$eType>>2]|0;
     $28 = ($26|0)==($27|0);
     if (!($28)) {
      break;
     }
     $29 = HEAP32[$p>>2]|0;
     $30 = ((($29)) + 12|0);
     $31 = HEAP32[$30>>2]|0;
     HEAP32[$p>>2] = $31;
    }
    while(1) {
     $32 = HEAP32[$p>>2]|0;
     $33 = ((($32)) + 8|0);
     $34 = HEAP32[$33>>2]|0;
     HEAP32[$pParent>>2] = $34;
     $35 = HEAP32[$p>>2]|0;
     $36 = ((($35)) + 8|0);
     HEAP32[$36>>2] = 0;
     $37 = HEAP32[$pParent>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$pParent>>2]|0;
      $40 = ((($39)) + 12|0);
      HEAP32[$40>>2] = 0;
     } else {
      HEAP32[$pRoot>>2] = 0;
     }
     $41 = HEAP32[$1>>2]|0;
     $42 = (($41) - 1)|0;
     $43 = (_fts3ExprBalance($p,$42)|0);
     HEAP32[$rc>>2] = $43;
     $44 = HEAP32[$rc>>2]|0;
     $45 = ($44|0)!=(0);
     if ($45) {
      break;
     }
     HEAP32[$iLvl>>2] = 0;
     while(1) {
      $46 = HEAP32[$p>>2]|0;
      $47 = ($46|0)!=(0|0);
      if (!($47)) {
       break;
      }
      $48 = HEAP32[$iLvl>>2]|0;
      $49 = HEAP32[$1>>2]|0;
      $50 = ($48|0)<($49|0);
      if (!($50)) {
       break;
      }
      $51 = HEAP32[$iLvl>>2]|0;
      $52 = HEAP32[$apLeaf>>2]|0;
      $53 = (($52) + ($51<<2)|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($54|0)==(0|0);
      if ($55) {
       $56 = HEAP32[$p>>2]|0;
       $57 = HEAP32[$iLvl>>2]|0;
       $58 = HEAP32[$apLeaf>>2]|0;
       $59 = (($58) + ($57<<2)|0);
       HEAP32[$59>>2] = $56;
       HEAP32[$p>>2] = 0;
      } else {
       $60 = HEAP32[$iLvl>>2]|0;
       $61 = HEAP32[$apLeaf>>2]|0;
       $62 = (($61) + ($60<<2)|0);
       $63 = HEAP32[$62>>2]|0;
       $64 = HEAP32[$pFree>>2]|0;
       $65 = ((($64)) + 12|0);
       HEAP32[$65>>2] = $63;
       $66 = HEAP32[$p>>2]|0;
       $67 = HEAP32[$pFree>>2]|0;
       $68 = ((($67)) + 16|0);
       HEAP32[$68>>2] = $66;
       $69 = HEAP32[$pFree>>2]|0;
       $70 = HEAP32[$pFree>>2]|0;
       $71 = ((($70)) + 12|0);
       $72 = HEAP32[$71>>2]|0;
       $73 = ((($72)) + 8|0);
       HEAP32[$73>>2] = $69;
       $74 = HEAP32[$pFree>>2]|0;
       $75 = HEAP32[$pFree>>2]|0;
       $76 = ((($75)) + 16|0);
       $77 = HEAP32[$76>>2]|0;
       $78 = ((($77)) + 8|0);
       HEAP32[$78>>2] = $74;
       $79 = HEAP32[$pFree>>2]|0;
       HEAP32[$p>>2] = $79;
       $80 = HEAP32[$pFree>>2]|0;
       $81 = ((($80)) + 8|0);
       $82 = HEAP32[$81>>2]|0;
       HEAP32[$pFree>>2] = $82;
       $83 = HEAP32[$p>>2]|0;
       $84 = ((($83)) + 8|0);
       HEAP32[$84>>2] = 0;
       $85 = HEAP32[$iLvl>>2]|0;
       $86 = HEAP32[$apLeaf>>2]|0;
       $87 = (($86) + ($85<<2)|0);
       HEAP32[$87>>2] = 0;
      }
      $88 = HEAP32[$iLvl>>2]|0;
      $89 = (($88) + 1)|0;
      HEAP32[$iLvl>>2] = $89;
     }
     $90 = HEAP32[$p>>2]|0;
     $91 = ($90|0)!=(0|0);
     if ($91) {
      label = 24;
      break;
     }
     $93 = HEAP32[$pParent>>2]|0;
     $94 = ($93|0)==(0|0);
     if ($94) {
      break;
     }
     $95 = HEAP32[$pParent>>2]|0;
     $96 = ((($95)) + 16|0);
     $97 = HEAP32[$96>>2]|0;
     HEAP32[$p>>2] = $97;
     while(1) {
      $98 = HEAP32[$p>>2]|0;
      $99 = HEAP32[$98>>2]|0;
      $100 = HEAP32[$eType>>2]|0;
      $101 = ($99|0)==($100|0);
      if (!($101)) {
       break;
      }
      $102 = HEAP32[$p>>2]|0;
      $103 = ((($102)) + 12|0);
      $104 = HEAP32[$103>>2]|0;
      HEAP32[$p>>2] = $104;
     }
     $105 = HEAP32[$pParent>>2]|0;
     $106 = ((($105)) + 8|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = HEAP32[$pParent>>2]|0;
     $109 = ((($108)) + 16|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = ((($110)) + 8|0);
     HEAP32[$111>>2] = $107;
     $112 = HEAP32[$pParent>>2]|0;
     $113 = ((($112)) + 8|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = ($114|0)!=(0|0);
     $116 = HEAP32[$pParent>>2]|0;
     $117 = ((($116)) + 16|0);
     $118 = HEAP32[$117>>2]|0;
     if ($115) {
      $119 = HEAP32[$pParent>>2]|0;
      $120 = ((($119)) + 8|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ((($121)) + 12|0);
      HEAP32[$122>>2] = $118;
     } else {
      HEAP32[$pRoot>>2] = $118;
     }
     $123 = HEAP32[$pFree>>2]|0;
     $124 = HEAP32[$pParent>>2]|0;
     $125 = ((($124)) + 8|0);
     HEAP32[$125>>2] = $123;
     $126 = HEAP32[$pParent>>2]|0;
     HEAP32[$pFree>>2] = $126;
    }
    if ((label|0) == 24) {
     $92 = HEAP32[$p>>2]|0;
     _sqlite3Fts3ExprFree($92);
     HEAP32[$rc>>2] = 18;
    }
    $127 = HEAP32[$rc>>2]|0;
    $128 = ($127|0)==(0);
    L53: do {
     if ($128) {
      HEAP32[$p>>2] = 0;
      HEAP32[$i>>2] = 0;
      while(1) {
       $129 = HEAP32[$i>>2]|0;
       $130 = HEAP32[$1>>2]|0;
       $131 = ($129|0)<($130|0);
       if (!($131)) {
        break;
       }
       $132 = HEAP32[$i>>2]|0;
       $133 = HEAP32[$apLeaf>>2]|0;
       $134 = (($133) + ($132<<2)|0);
       $135 = HEAP32[$134>>2]|0;
       $136 = ($135|0)!=(0|0);
       do {
        if ($136) {
         $137 = HEAP32[$p>>2]|0;
         $138 = ($137|0)==(0|0);
         if ($138) {
          $139 = HEAP32[$i>>2]|0;
          $140 = HEAP32[$apLeaf>>2]|0;
          $141 = (($140) + ($139<<2)|0);
          $142 = HEAP32[$141>>2]|0;
          HEAP32[$p>>2] = $142;
          $143 = HEAP32[$p>>2]|0;
          $144 = ((($143)) + 8|0);
          HEAP32[$144>>2] = 0;
          break;
         } else {
          $145 = HEAP32[$p>>2]|0;
          $146 = HEAP32[$pFree>>2]|0;
          $147 = ((($146)) + 16|0);
          HEAP32[$147>>2] = $145;
          $148 = HEAP32[$i>>2]|0;
          $149 = HEAP32[$apLeaf>>2]|0;
          $150 = (($149) + ($148<<2)|0);
          $151 = HEAP32[$150>>2]|0;
          $152 = HEAP32[$pFree>>2]|0;
          $153 = ((($152)) + 12|0);
          HEAP32[$153>>2] = $151;
          $154 = HEAP32[$pFree>>2]|0;
          $155 = HEAP32[$pFree>>2]|0;
          $156 = ((($155)) + 12|0);
          $157 = HEAP32[$156>>2]|0;
          $158 = ((($157)) + 8|0);
          HEAP32[$158>>2] = $154;
          $159 = HEAP32[$pFree>>2]|0;
          $160 = HEAP32[$pFree>>2]|0;
          $161 = ((($160)) + 16|0);
          $162 = HEAP32[$161>>2]|0;
          $163 = ((($162)) + 8|0);
          HEAP32[$163>>2] = $159;
          $164 = HEAP32[$pFree>>2]|0;
          HEAP32[$p>>2] = $164;
          $165 = HEAP32[$pFree>>2]|0;
          $166 = ((($165)) + 8|0);
          $167 = HEAP32[$166>>2]|0;
          HEAP32[$pFree>>2] = $167;
          $168 = HEAP32[$p>>2]|0;
          $169 = ((($168)) + 8|0);
          HEAP32[$169>>2] = 0;
          break;
         }
        }
       } while(0);
       $170 = HEAP32[$i>>2]|0;
       $171 = (($170) + 1)|0;
       HEAP32[$i>>2] = $171;
      }
      $172 = HEAP32[$p>>2]|0;
      HEAP32[$pRoot>>2] = $172;
     } else {
      HEAP32[$i>>2] = 0;
      while(1) {
       $173 = HEAP32[$i>>2]|0;
       $174 = HEAP32[$1>>2]|0;
       $175 = ($173|0)<($174|0);
       if (!($175)) {
        break;
       }
       $176 = HEAP32[$i>>2]|0;
       $177 = HEAP32[$apLeaf>>2]|0;
       $178 = (($177) + ($176<<2)|0);
       $179 = HEAP32[$178>>2]|0;
       _sqlite3Fts3ExprFree($179);
       $180 = HEAP32[$i>>2]|0;
       $181 = (($180) + 1)|0;
       HEAP32[$i>>2] = $181;
      }
      while(1) {
       $182 = HEAP32[$pFree>>2]|0;
       HEAP32[$pDel>>2] = $182;
       $183 = ($182|0)!=(0|0);
       if (!($183)) {
        break L53;
       }
       $184 = HEAP32[$pDel>>2]|0;
       $185 = ((($184)) + 8|0);
       $186 = HEAP32[$185>>2]|0;
       HEAP32[$pFree>>2] = $186;
       $187 = HEAP32[$pDel>>2]|0;
       _sqlite3_free($187);
      }
     }
    } while(0);
    $188 = HEAP32[$apLeaf>>2]|0;
    _sqlite3_free($188);
   }
  }
 } while(0);
 $229 = HEAP32[$rc>>2]|0;
 $230 = ($229|0)!=(0);
 if (!($230)) {
  $232 = HEAP32[$pRoot>>2]|0;
  $233 = HEAP32[$0>>2]|0;
  HEAP32[$233>>2] = $232;
  $234 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($234|0);
 }
 $231 = HEAP32[$pRoot>>2]|0;
 _sqlite3Fts3ExprFree($231);
 HEAP32[$pRoot>>2] = 0;
 $232 = HEAP32[$pRoot>>2]|0;
 $233 = HEAP32[$0>>2]|0;
 HEAP32[$233>>2] = $232;
 $234 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($234|0);
}
function _fts3ExprCheckDepth($p,$nMaxDepth) {
 $p = $p|0;
 $nMaxDepth = $nMaxDepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nMaxDepth;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = ($4|0)<(0);
   if ($5) {
    HEAP32[$rc>>2] = 18;
    break;
   }
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 12|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = HEAP32[$1>>2]|0;
   $10 = (($9) - 1)|0;
   $11 = (_fts3ExprCheckDepth($8,$10)|0);
   HEAP32[$rc>>2] = $11;
   $12 = HEAP32[$rc>>2]|0;
   $13 = ($12|0)==(0);
   if ($13) {
    $14 = HEAP32[$0>>2]|0;
    $15 = ((($14)) + 16|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = HEAP32[$1>>2]|0;
    $18 = (($17) - 1)|0;
    $19 = (_fts3ExprCheckDepth($16,$18)|0);
    HEAP32[$rc>>2] = $19;
   }
  }
 } while(0);
 $20 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _fts3ExprParse($pParse,$z,$n,$ppExpr,$pnConsumed) {
 $pParse = $pParse|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pnConsumed = $pnConsumed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $eType = 0, $isPhrase = 0, $isRequirePhrase = 0, $nByte = 0, $nIn = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $p = 0, $pAnd = 0, $pNotBranch = 0, $pPrev = 0, $pRet = 0;
 var $rc = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $pRet = sp + 44|0;
 $pPrev = sp + 40|0;
 $pNotBranch = sp + 36|0;
 $nIn = sp + 32|0;
 $zIn = sp + 28|0;
 $rc = sp + 24|0;
 $isRequirePhrase = sp + 20|0;
 $p = sp + 16|0;
 $nByte = sp + 12|0;
 $isPhrase = sp + 8|0;
 $eType = sp + 4|0;
 $pAnd = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $ppExpr;
 HEAP32[$4>>2] = $pnConsumed;
 HEAP32[$pRet>>2] = 0;
 HEAP32[$pPrev>>2] = 0;
 HEAP32[$pNotBranch>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$nIn>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$zIn>>2] = $6;
 HEAP32[$rc>>2] = 0;
 HEAP32[$isRequirePhrase>>2] = 1;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 25;
   break;
  }
  HEAP32[$p>>2] = 0;
  HEAP32[$nByte>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$zIn>>2]|0;
  $11 = HEAP32[$nIn>>2]|0;
  $12 = (_getNextNode($9,$10,$11,$p,$nByte)|0);
  HEAP32[$rc>>2] = $12;
  $13 = HEAP32[$rc>>2]|0;
  $14 = ($13|0)==(0);
  $15 = HEAP32[$p>>2]|0;
  $16 = ($15|0)!=(0|0);
  $or$cond13 = $14 & $16;
  if ($or$cond13) {
   $17 = HEAP32[$p>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$eType>>2] = $18;
   $19 = HEAP32[$eType>>2]|0;
   $20 = ($19|0)==(5);
   if ($20) {
    $26 = 1;
   } else {
    $21 = HEAP32[$p>>2]|0;
    $22 = ((($21)) + 12|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ($23|0)!=(0|0);
    $26 = $24;
   }
   $25 = $26&1;
   HEAP32[$isPhrase>>2] = $25;
   $27 = HEAP32[$isPhrase>>2]|0;
   $28 = ($27|0)==(0);
   $29 = HEAP32[$isRequirePhrase>>2]|0;
   $30 = ($29|0)!=(0);
   $or$cond = $28 & $30;
   if ($or$cond) {
    label = 7;
    break;
   }
   $32 = HEAP32[$isPhrase>>2]|0;
   $33 = ($32|0)==(0);
   $34 = HEAP32[$isRequirePhrase>>2]|0;
   $35 = ($34|0)!=(0);
   $or$cond3 = $33 | $35;
   if (!($or$cond3)) {
    $36 = (_fts3MallocZero(48)|0);
    HEAP32[$pAnd>>2] = $36;
    $37 = HEAP32[$pAnd>>2]|0;
    $38 = ($37|0)!=(0|0);
    if (!($38)) {
     label = 10;
     break;
    }
    $40 = HEAP32[$pAnd>>2]|0;
    HEAP32[$40>>2] = 3;
    $41 = HEAP32[$pPrev>>2]|0;
    $42 = HEAP32[$pAnd>>2]|0;
    _insertBinaryOperator($pRet,$41,$42);
    $43 = HEAP32[$pAnd>>2]|0;
    HEAP32[$pPrev>>2] = $43;
   }
   $44 = HEAP32[$pPrev>>2]|0;
   $45 = ($44|0)!=(0|0);
   if ($45) {
    $46 = HEAP32[$eType>>2]|0;
    $47 = ($46|0)!=(1);
    $48 = HEAP32[$isPhrase>>2]|0;
    $49 = ($48|0)!=(0);
    $or$cond5 = $47 | $49;
    if (!($or$cond5)) {
     $50 = HEAP32[$pPrev>>2]|0;
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)!=(5);
     if ($52) {
      label = 17;
      break;
     }
    }
    $53 = HEAP32[$eType>>2]|0;
    $54 = ($53|0)!=(5);
    $55 = HEAP32[$isPhrase>>2]|0;
    $56 = ($55|0)!=(0);
    $or$cond7 = $54 & $56;
    if ($or$cond7) {
     $57 = HEAP32[$pPrev>>2]|0;
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(1);
     if ($59) {
      label = 17;
      break;
     }
    }
   }
   $61 = HEAP32[$isPhrase>>2]|0;
   $62 = ($61|0)!=(0);
   do {
    if ($62) {
     $63 = HEAP32[$pRet>>2]|0;
     $64 = ($63|0)!=(0|0);
     $65 = HEAP32[$p>>2]|0;
     if ($64) {
      $66 = HEAP32[$pPrev>>2]|0;
      $67 = ((($66)) + 16|0);
      HEAP32[$67>>2] = $65;
      $68 = HEAP32[$pPrev>>2]|0;
      $69 = HEAP32[$p>>2]|0;
      $70 = ((($69)) + 8|0);
      HEAP32[$70>>2] = $68;
      break;
     } else {
      HEAP32[$pRet>>2] = $65;
      break;
     }
    } else {
     $71 = HEAP32[$pPrev>>2]|0;
     $72 = HEAP32[$p>>2]|0;
     _insertBinaryOperator($pRet,$71,$72);
    }
   } while(0);
   $73 = HEAP32[$isPhrase>>2]|0;
   $74 = ($73|0)!=(0);
   $75 = $74 ^ 1;
   $76 = $75&1;
   HEAP32[$isRequirePhrase>>2] = $76;
   $77 = HEAP32[$p>>2]|0;
   HEAP32[$pPrev>>2] = $77;
  }
  $78 = HEAP32[$nByte>>2]|0;
  $79 = HEAP32[$nIn>>2]|0;
  $80 = (($79) - ($78))|0;
  HEAP32[$nIn>>2] = $80;
  $81 = HEAP32[$nByte>>2]|0;
  $82 = HEAP32[$zIn>>2]|0;
  $83 = (($82) + ($81)|0);
  HEAP32[$zIn>>2] = $83;
 }
 if ((label|0) == 7) {
  $31 = HEAP32[$p>>2]|0;
  _sqlite3Fts3ExprFree($31);
  HEAP32[$rc>>2] = 1;
 }
 else if ((label|0) == 10) {
  $39 = HEAP32[$p>>2]|0;
  _sqlite3Fts3ExprFree($39);
  HEAP32[$rc>>2] = 7;
 }
 else if ((label|0) == 17) {
  $60 = HEAP32[$p>>2]|0;
  _sqlite3Fts3ExprFree($60);
  HEAP32[$rc>>2] = 1;
 }
 else if ((label|0) == 25) {
  $84 = HEAP32[$rc>>2]|0;
  $85 = ($84|0)==(101);
  $86 = HEAP32[$pRet>>2]|0;
  $87 = ($86|0)!=(0|0);
  $or$cond9 = $85 & $87;
  $88 = HEAP32[$isRequirePhrase>>2]|0;
  $89 = ($88|0)!=(0);
  $or$cond11 = $or$cond9 & $89;
  if ($or$cond11) {
   HEAP32[$rc>>2] = 1;
  }
  $90 = HEAP32[$rc>>2]|0;
  $91 = ($90|0)==(101);
  if ($91) {
   HEAP32[$rc>>2] = 0;
  }
  $92 = HEAP32[$2>>2]|0;
  $93 = HEAP32[$nIn>>2]|0;
  $94 = (($92) - ($93))|0;
  $95 = HEAP32[$4>>2]|0;
  HEAP32[$95>>2] = $94;
 }
 $96 = HEAP32[$rc>>2]|0;
 $97 = ($96|0)!=(0);
 if (!($97)) {
  $100 = HEAP32[$pRet>>2]|0;
  $101 = HEAP32[$3>>2]|0;
  HEAP32[$101>>2] = $100;
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $98 = HEAP32[$pRet>>2]|0;
 _sqlite3Fts3ExprFree($98);
 $99 = HEAP32[$pNotBranch>>2]|0;
 _sqlite3Fts3ExprFree($99);
 HEAP32[$pRet>>2] = 0;
 $100 = HEAP32[$pRet>>2]|0;
 $101 = HEAP32[$3>>2]|0;
 HEAP32[$101>>2] = $100;
 $102 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _getNextNode($pParse,$z,$n,$ppExpr,$pnConsumed) {
 $pParse = $pParse|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pnConsumed = $pnConsumed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cNext = 0, $iCol = 0, $iColLen = 0, $ii = 0, $nConsumed = 0, $nInput = 0, $nKey = 0;
 var $nNear = 0, $nStr = 0, $pKey = 0, $pRet = 0, $rc = 0, $zInput = 0, $zStr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $5 = sp + 52|0;
 $ii = sp + 48|0;
 $iCol = sp + 44|0;
 $iColLen = sp + 40|0;
 $rc = sp + 36|0;
 $pRet = sp + 32|0;
 $zInput = sp + 28|0;
 $nInput = sp + 24|0;
 $pKey = sp + 20|0;
 $nNear = sp + 16|0;
 $nKey = sp + 12|0;
 $cNext = sp + 76|0;
 $nConsumed = sp + 8|0;
 $zStr = sp + 4|0;
 $nStr = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $ppExpr;
 HEAP32[$5>>2] = $pnConsumed;
 HEAP32[$pRet>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$zInput>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$nInput>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 24|0);
 HEAP32[$9>>2] = 0;
 while(1) {
  $10 = HEAP32[$nInput>>2]|0;
  $11 = ($10|0)>(0);
  if ($11) {
   $12 = HEAP32[$zInput>>2]|0;
   $13 = HEAP8[$12>>0]|0;
   $14 = (_fts3isspace($13)|0);
   $15 = ($14|0)!=(0);
   $272 = $15;
  } else {
   $272 = 0;
  }
  $16 = HEAP32[$nInput>>2]|0;
  if (!($272)) {
   break;
  }
  $17 = (($16) + -1)|0;
  HEAP32[$nInput>>2] = $17;
  $18 = HEAP32[$zInput>>2]|0;
  $19 = ((($18)) + 1|0);
  HEAP32[$zInput>>2] = $19;
 }
 $20 = ($16|0)==(0);
 if ($20) {
  HEAP32[$0>>2] = 101;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $21 = HEAP32[$ii>>2]|0;
  $22 = ($21|0)<(4);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$ii>>2]|0;
  $24 = (2112 + ($23<<3)|0);
  HEAP32[$pKey>>2] = $24;
  $25 = HEAP32[$pKey>>2]|0;
  $26 = ((($25)) + 5|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  $29 = $28 & -2;
  $30 = ($29|0)!=(0);
  if (!($30)) {
   $31 = HEAP32[$nInput>>2]|0;
   $32 = HEAP32[$pKey>>2]|0;
   $33 = ((($32)) + 4|0);
   $34 = HEAP8[$33>>0]|0;
   $35 = $34&255;
   $36 = ($31|0)>=($35|0);
   if ($36) {
    $37 = HEAP32[$zInput>>2]|0;
    $38 = HEAP32[$pKey>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pKey>>2]|0;
    $41 = ((($40)) + 4|0);
    $42 = HEAP8[$41>>0]|0;
    $43 = $42&255;
    $44 = (_memcmp($37,$39,$43)|0);
    $45 = (0)==($44|0);
    if ($45) {
     HEAP32[$nNear>>2] = 10;
     $46 = HEAP32[$pKey>>2]|0;
     $47 = ((($46)) + 4|0);
     $48 = HEAP8[$47>>0]|0;
     $49 = $48&255;
     HEAP32[$nKey>>2] = $49;
     $50 = HEAP32[$pKey>>2]|0;
     $51 = ((($50)) + 6|0);
     $52 = HEAP8[$51>>0]|0;
     $53 = $52&255;
     $54 = ($53|0)==(1);
     L19: do {
      if ($54) {
       $55 = HEAP32[$zInput>>2]|0;
       $56 = ((($55)) + 4|0);
       $57 = HEAP8[$56>>0]|0;
       $58 = $57 << 24 >> 24;
       $59 = ($58|0)==(47);
       if ($59) {
        $60 = HEAP32[$zInput>>2]|0;
        $61 = ((($60)) + 5|0);
        $62 = HEAP8[$61>>0]|0;
        $63 = $62 << 24 >> 24;
        $64 = ($63|0)>=(48);
        if ($64) {
         $65 = HEAP32[$zInput>>2]|0;
         $66 = ((($65)) + 5|0);
         $67 = HEAP8[$66>>0]|0;
         $68 = $67 << 24 >> 24;
         $69 = ($68|0)<=(57);
         if ($69) {
          HEAP32[$nNear>>2] = 0;
          HEAP32[$nKey>>2] = 5;
          while(1) {
           $70 = HEAP32[$nKey>>2]|0;
           $71 = HEAP32[$zInput>>2]|0;
           $72 = (($71) + ($70)|0);
           $73 = HEAP8[$72>>0]|0;
           $74 = $73 << 24 >> 24;
           $75 = ($74|0)>=(48);
           if (!($75)) {
            break L19;
           }
           $76 = HEAP32[$nKey>>2]|0;
           $77 = HEAP32[$zInput>>2]|0;
           $78 = (($77) + ($76)|0);
           $79 = HEAP8[$78>>0]|0;
           $80 = $79 << 24 >> 24;
           $81 = ($80|0)<=(57);
           if (!($81)) {
            break L19;
           }
           $82 = HEAP32[$nNear>>2]|0;
           $83 = ($82*10)|0;
           $84 = HEAP32[$nKey>>2]|0;
           $85 = HEAP32[$zInput>>2]|0;
           $86 = (($85) + ($84)|0);
           $87 = HEAP8[$86>>0]|0;
           $88 = $87 << 24 >> 24;
           $89 = (($88) - 48)|0;
           $90 = (($83) + ($89))|0;
           HEAP32[$nNear>>2] = $90;
           $91 = HEAP32[$nKey>>2]|0;
           $92 = (($91) + 1)|0;
           HEAP32[$nKey>>2] = $92;
          }
         }
        }
       }
      }
     } while(0);
     $93 = HEAP32[$nKey>>2]|0;
     $94 = HEAP32[$zInput>>2]|0;
     $95 = (($94) + ($93)|0);
     $96 = HEAP8[$95>>0]|0;
     HEAP8[$cNext>>0] = $96;
     $97 = HEAP8[$cNext>>0]|0;
     $98 = (_fts3isspace($97)|0);
     $99 = ($98|0)!=(0);
     if ($99) {
      label = 26;
      break;
     }
     $100 = HEAP8[$cNext>>0]|0;
     $101 = $100 << 24 >> 24;
     $102 = ($101|0)==(34);
     if ($102) {
      label = 26;
      break;
     }
     $103 = HEAP8[$cNext>>0]|0;
     $104 = $103 << 24 >> 24;
     $105 = ($104|0)==(40);
     if ($105) {
      label = 26;
      break;
     }
     $106 = HEAP8[$cNext>>0]|0;
     $107 = $106 << 24 >> 24;
     $108 = ($107|0)==(41);
     if ($108) {
      label = 26;
      break;
     }
     $109 = HEAP8[$cNext>>0]|0;
     $110 = $109 << 24 >> 24;
     $111 = ($110|0)==(0);
     if ($111) {
      label = 26;
      break;
     }
    }
   }
  }
  $133 = HEAP32[$ii>>2]|0;
  $134 = (($133) + 1)|0;
  HEAP32[$ii>>2] = $134;
 }
 if ((label|0) == 26) {
  $112 = (_fts3MallocZero(48)|0);
  HEAP32[$pRet>>2] = $112;
  $113 = HEAP32[$pRet>>2]|0;
  $114 = ($113|0)!=(0|0);
  if ($114) {
   $115 = HEAP32[$pKey>>2]|0;
   $116 = ((($115)) + 6|0);
   $117 = HEAP8[$116>>0]|0;
   $118 = $117&255;
   $119 = HEAP32[$pRet>>2]|0;
   HEAP32[$119>>2] = $118;
   $120 = HEAP32[$nNear>>2]|0;
   $121 = HEAP32[$pRet>>2]|0;
   $122 = ((($121)) + 4|0);
   HEAP32[$122>>2] = $120;
   $123 = HEAP32[$pRet>>2]|0;
   $124 = HEAP32[$4>>2]|0;
   HEAP32[$124>>2] = $123;
   $125 = HEAP32[$zInput>>2]|0;
   $126 = HEAP32[$2>>2]|0;
   $127 = $125;
   $128 = $126;
   $129 = (($127) - ($128))|0;
   $130 = HEAP32[$nKey>>2]|0;
   $131 = (($129) + ($130))|0;
   $132 = HEAP32[$5>>2]|0;
   HEAP32[$132>>2] = $131;
   HEAP32[$0>>2] = 0;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  } else {
   HEAP32[$0>>2] = 7;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  }
 }
 $135 = HEAP32[$zInput>>2]|0;
 $136 = HEAP8[$135>>0]|0;
 $137 = $136 << 24 >> 24;
 $138 = ($137|0)==(34);
 if ($138) {
  HEAP32[$ii>>2] = 1;
  while(1) {
   $139 = HEAP32[$ii>>2]|0;
   $140 = HEAP32[$nInput>>2]|0;
   $141 = ($139|0)<($140|0);
   if (!($141)) {
    break;
   }
   $142 = HEAP32[$ii>>2]|0;
   $143 = HEAP32[$zInput>>2]|0;
   $144 = (($143) + ($142)|0);
   $145 = HEAP8[$144>>0]|0;
   $146 = $145 << 24 >> 24;
   $147 = ($146|0)!=(34);
   if (!($147)) {
    break;
   }
   $148 = HEAP32[$ii>>2]|0;
   $149 = (($148) + 1)|0;
   HEAP32[$ii>>2] = $149;
  }
  $150 = HEAP32[$zInput>>2]|0;
  $151 = HEAP32[$2>>2]|0;
  $152 = $150;
  $153 = $151;
  $154 = (($152) - ($153))|0;
  $155 = HEAP32[$ii>>2]|0;
  $156 = (($154) + ($155))|0;
  $157 = (($156) + 1)|0;
  $158 = HEAP32[$5>>2]|0;
  HEAP32[$158>>2] = $157;
  $159 = HEAP32[$ii>>2]|0;
  $160 = HEAP32[$nInput>>2]|0;
  $161 = ($159|0)==($160|0);
  if ($161) {
   HEAP32[$0>>2] = 1;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  } else {
   $162 = HEAP32[$1>>2]|0;
   $163 = HEAP32[$zInput>>2]|0;
   $164 = ((($163)) + 1|0);
   $165 = HEAP32[$ii>>2]|0;
   $166 = (($165) - 1)|0;
   $167 = HEAP32[$4>>2]|0;
   $168 = (_getNextString($162,$164,$166,$167)|0);
   HEAP32[$0>>2] = $168;
   $271 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($271|0);
  }
 }
 $169 = HEAP32[$zInput>>2]|0;
 $170 = HEAP8[$169>>0]|0;
 $171 = $170 << 24 >> 24;
 $172 = ($171|0)==(40);
 if ($172) {
  HEAP32[$nConsumed>>2] = 0;
  $173 = HEAP32[$1>>2]|0;
  $174 = ((($173)) + 32|0);
  $175 = HEAP32[$174>>2]|0;
  $176 = (($175) + 1)|0;
  HEAP32[$174>>2] = $176;
  $177 = HEAP32[$1>>2]|0;
  $178 = HEAP32[$zInput>>2]|0;
  $179 = ((($178)) + 1|0);
  $180 = HEAP32[$nInput>>2]|0;
  $181 = (($180) - 1)|0;
  $182 = HEAP32[$4>>2]|0;
  $183 = (_fts3ExprParse($177,$179,$181,$182,$nConsumed)|0);
  HEAP32[$rc>>2] = $183;
  $184 = HEAP32[$rc>>2]|0;
  $185 = ($184|0)==(0);
  if ($185) {
   $186 = HEAP32[$4>>2]|0;
   $187 = HEAP32[$186>>2]|0;
   $188 = ($187|0)!=(0|0);
   if (!($188)) {
    HEAP32[$rc>>2] = 101;
   }
  }
  $189 = HEAP32[$zInput>>2]|0;
  $190 = HEAP32[$2>>2]|0;
  $191 = $189;
  $192 = $190;
  $193 = (($191) - ($192))|0;
  $194 = (($193) + 1)|0;
  $195 = HEAP32[$nConsumed>>2]|0;
  $196 = (($194) + ($195))|0;
  $197 = HEAP32[$5>>2]|0;
  HEAP32[$197>>2] = $196;
  $198 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $198;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 }
 $199 = HEAP32[$zInput>>2]|0;
 $200 = HEAP8[$199>>0]|0;
 $201 = $200 << 24 >> 24;
 $202 = ($201|0)==(41);
 $203 = HEAP32[$1>>2]|0;
 if ($202) {
  $204 = ((($203)) + 32|0);
  $205 = HEAP32[$204>>2]|0;
  $206 = (($205) + -1)|0;
  HEAP32[$204>>2] = $206;
  $207 = HEAP32[$zInput>>2]|0;
  $208 = HEAP32[$2>>2]|0;
  $209 = $207;
  $210 = $208;
  $211 = (($209) - ($210))|0;
  $212 = (($211) + 1)|0;
  $213 = HEAP32[$5>>2]|0;
  HEAP32[$213>>2] = $212;
  $214 = HEAP32[$4>>2]|0;
  HEAP32[$214>>2] = 0;
  HEAP32[$0>>2] = 101;
  $271 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($271|0);
 }
 $215 = ((($203)) + 20|0);
 $216 = HEAP32[$215>>2]|0;
 HEAP32[$iCol>>2] = $216;
 HEAP32[$iColLen>>2] = 0;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $217 = HEAP32[$ii>>2]|0;
  $218 = HEAP32[$1>>2]|0;
  $219 = ((($218)) + 16|0);
  $220 = HEAP32[$219>>2]|0;
  $221 = ($217|0)<($220|0);
  if (!($221)) {
   break;
  }
  $222 = HEAP32[$ii>>2]|0;
  $223 = HEAP32[$1>>2]|0;
  $224 = ((($223)) + 8|0);
  $225 = HEAP32[$224>>2]|0;
  $226 = (($225) + ($222<<2)|0);
  $227 = HEAP32[$226>>2]|0;
  HEAP32[$zStr>>2] = $227;
  $228 = HEAP32[$zStr>>2]|0;
  $229 = (_strlen($228)|0);
  HEAP32[$nStr>>2] = $229;
  $230 = HEAP32[$nInput>>2]|0;
  $231 = HEAP32[$nStr>>2]|0;
  $232 = ($230|0)>($231|0);
  if ($232) {
   $233 = HEAP32[$nStr>>2]|0;
   $234 = HEAP32[$zInput>>2]|0;
   $235 = (($234) + ($233)|0);
   $236 = HEAP8[$235>>0]|0;
   $237 = $236 << 24 >> 24;
   $238 = ($237|0)==(58);
   if ($238) {
    $239 = HEAP32[$zStr>>2]|0;
    $240 = HEAP32[$zInput>>2]|0;
    $241 = HEAP32[$nStr>>2]|0;
    $242 = (_sqlite3_strnicmp($239,$240,$241)|0);
    $243 = ($242|0)==(0);
    if ($243) {
     label = 50;
     break;
    }
   }
  }
  $253 = HEAP32[$ii>>2]|0;
  $254 = (($253) + 1)|0;
  HEAP32[$ii>>2] = $254;
 }
 if ((label|0) == 50) {
  $244 = HEAP32[$ii>>2]|0;
  HEAP32[$iCol>>2] = $244;
  $245 = HEAP32[$zInput>>2]|0;
  $246 = HEAP32[$2>>2]|0;
  $247 = $245;
  $248 = $246;
  $249 = (($247) - ($248))|0;
  $250 = HEAP32[$nStr>>2]|0;
  $251 = (($249) + ($250))|0;
  $252 = (($251) + 1)|0;
  HEAP32[$iColLen>>2] = $252;
 }
 $255 = HEAP32[$1>>2]|0;
 $256 = HEAP32[$iCol>>2]|0;
 $257 = HEAP32[$iColLen>>2]|0;
 $258 = HEAP32[$2>>2]|0;
 $259 = (($258) + ($257)|0);
 $260 = HEAP32[$3>>2]|0;
 $261 = HEAP32[$iColLen>>2]|0;
 $262 = (($260) - ($261))|0;
 $263 = HEAP32[$4>>2]|0;
 $264 = HEAP32[$5>>2]|0;
 $265 = (_getNextToken($255,$256,$259,$262,$263,$264)|0);
 HEAP32[$rc>>2] = $265;
 $266 = HEAP32[$iColLen>>2]|0;
 $267 = HEAP32[$5>>2]|0;
 $268 = HEAP32[$267>>2]|0;
 $269 = (($268) + ($266))|0;
 HEAP32[$267>>2] = $269;
 $270 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $270;
 $271 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($271|0);
}
function _fts3MallocZero($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $nByte;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_malloc($1)|0);
 HEAP32[$pRet>>2] = $2;
 $3 = HEAP32[$pRet>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $7 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 $5 = HEAP32[$pRet>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 _memset(($5|0),0,($6|0))|0;
 $7 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _insertBinaryOperator($ppHead,$pPrev,$pNew) {
 $ppHead = $ppHead|0;
 $pPrev = $pPrev|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSplit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pSplit = sp;
 HEAP32[$0>>2] = $ppHead;
 HEAP32[$1>>2] = $pPrev;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pSplit>>2] = $3;
 while(1) {
  $4 = HEAP32[$pSplit>>2]|0;
  $5 = ((($4)) + 8|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)!=(0|0);
  if ($7) {
   $8 = HEAP32[$pSplit>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_opPrecedence($10)|0);
   $12 = HEAP32[$2>>2]|0;
   $13 = (_opPrecedence($12)|0);
   $14 = ($11|0)<=($13|0);
   $36 = $14;
  } else {
   $36 = 0;
  }
  $15 = HEAP32[$pSplit>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  if (!($36)) {
   break;
  }
  HEAP32[$pSplit>>2] = $17;
 }
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$2>>2]|0;
 if ($18) {
  $20 = HEAP32[$pSplit>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 16|0);
  HEAP32[$23>>2] = $19;
  $24 = HEAP32[$pSplit>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 8|0);
  HEAP32[$28>>2] = $26;
  $30 = HEAP32[$pSplit>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  $32 = ((($31)) + 12|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$pSplit>>2]|0;
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $33;
  STACKTOP = sp;return;
 } else {
  $29 = HEAP32[$0>>2]|0;
  HEAP32[$29>>2] = $19;
  $30 = HEAP32[$pSplit>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  $32 = ((($31)) + 12|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$pSplit>>2]|0;
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $33;
  STACKTOP = sp;return;
 }
}
function _opPrecedence($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _fts3isspace($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $c;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = ($2|0)==(32);
 if ($3) {
  $20 = 1;
 } else {
  $4 = HEAP8[$0>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = ($5|0)==(9);
  if ($6) {
   $20 = 1;
  } else {
   $7 = HEAP8[$0>>0]|0;
   $8 = $7 << 24 >> 24;
   $9 = ($8|0)==(10);
   if ($9) {
    $20 = 1;
   } else {
    $10 = HEAP8[$0>>0]|0;
    $11 = $10 << 24 >> 24;
    $12 = ($11|0)==(13);
    if ($12) {
     $20 = 1;
    } else {
     $13 = HEAP8[$0>>0]|0;
     $14 = $13 << 24 >> 24;
     $15 = ($14|0)==(11);
     if ($15) {
      $20 = 1;
     } else {
      $16 = HEAP8[$0>>0]|0;
      $17 = $16 << 24 >> 24;
      $18 = ($17|0)==(12);
      $20 = $18;
     }
    }
   }
  }
 }
 $19 = $20&1;
 STACKTOP = sp;return ($19|0);
}
function _getNextString($pParse,$zInput,$nInput,$ppExpr) {
 $pParse = $pParse|0;
 $zInput = $zInput|0;
 $nInput = $nInput|0;
 $ppExpr = $ppExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iBegin = 0;
 var $iEnd = 0, $iPos = 0, $ii = 0, $jj = 0, $nByte = 0, $nSpace = 0, $nTemp = 0, $nToken = 0, $p = 0, $pCursor = 0, $pModule = 0, $pToken = 0, $pTokenizer = 0, $rc = 0, $zBuf = 0, $zByte = 0, $zTemp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $pTokenizer = sp + 68|0;
 $pModule = sp + 64|0;
 $rc = sp + 60|0;
 $p = sp + 56|0;
 $pCursor = sp + 52|0;
 $zTemp = sp + 48|0;
 $nTemp = sp + 44|0;
 $nSpace = sp + 40|0;
 $nToken = sp + 36|0;
 $ii = sp + 32|0;
 $zByte = sp + 28|0;
 $nByte = sp + 24|0;
 $iBegin = sp + 20|0;
 $iEnd = sp + 16|0;
 $iPos = sp + 12|0;
 $pToken = sp + 8|0;
 $jj = sp + 4|0;
 $zBuf = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zInput;
 HEAP32[$3>>2] = $nInput;
 HEAP32[$4>>2] = $ppExpr;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTokenizer>>2] = $6;
 $7 = HEAP32[$pTokenizer>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pModule>>2] = $8;
 HEAP32[$p>>2] = 0;
 HEAP32[$pCursor>>2] = 0;
 HEAP32[$zTemp>>2] = 0;
 HEAP32[$nTemp>>2] = 0;
 HEAP32[$nSpace>>2] = 144;
 HEAP32[$nToken>>2] = 0;
 $9 = HEAP32[$pTokenizer>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = (_sqlite3Fts3OpenTokenizer($9,$12,$13,$14,$pCursor)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 L1: do {
  if ($17) {
   HEAP32[$ii>>2] = 0;
   while(1) {
    $18 = HEAP32[$rc>>2]|0;
    $19 = ($18|0)==(0);
    if (!($19)) {
     break;
    }
    HEAP32[$nByte>>2] = 0;
    HEAP32[$iBegin>>2] = 0;
    HEAP32[$iEnd>>2] = 0;
    HEAP32[$iPos>>2] = 0;
    $20 = HEAP32[$pModule>>2]|0;
    $21 = ((($20)) + 20|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP32[$pCursor>>2]|0;
    $24 = (FUNCTION_TABLE_iiiiiii[$22 & 255]($23,$zByte,$nByte,$iBegin,$iEnd,$iPos)|0);
    HEAP32[$rc>>2] = $24;
    $25 = HEAP32[$rc>>2]|0;
    $26 = ($25|0)==(0);
    if ($26) {
     $27 = HEAP32[$p>>2]|0;
     $28 = HEAP32[$ii>>2]|0;
     $29 = ($28*24)|0;
     $30 = (144 + ($29))|0;
     $31 = (_fts3ReallocOrFree($27,$30)|0);
     HEAP32[$p>>2] = $31;
     $32 = HEAP32[$p>>2]|0;
     $33 = ($32|0)!=(0|0);
     if (!($33)) {
      break L1;
     }
     $34 = HEAP32[$zTemp>>2]|0;
     $35 = HEAP32[$nTemp>>2]|0;
     $36 = HEAP32[$nByte>>2]|0;
     $37 = (($35) + ($36))|0;
     $38 = (_fts3ReallocOrFree($34,$37)|0);
     HEAP32[$zTemp>>2] = $38;
     $39 = HEAP32[$zTemp>>2]|0;
     $40 = ($39|0)!=(0|0);
     if (!($40)) {
      break L1;
     }
     $41 = HEAP32[$ii>>2]|0;
     $42 = HEAP32[$p>>2]|0;
     $43 = ((($42)) + 48|0);
     $44 = ((($43)) + 72|0);
     $45 = (($44) + (($41*24)|0)|0);
     HEAP32[$pToken>>2] = $45;
     $46 = HEAP32[$pToken>>2]|0;
     ;HEAP32[$46>>2]=0|0;HEAP32[$46+4>>2]=0|0;HEAP32[$46+8>>2]=0|0;HEAP32[$46+12>>2]=0|0;HEAP32[$46+16>>2]=0|0;HEAP32[$46+20>>2]=0|0;
     $47 = HEAP32[$nTemp>>2]|0;
     $48 = HEAP32[$zTemp>>2]|0;
     $49 = (($48) + ($47)|0);
     $50 = HEAP32[$zByte>>2]|0;
     $51 = HEAP32[$nByte>>2]|0;
     _memcpy(($49|0),($50|0),($51|0))|0;
     $52 = HEAP32[$nByte>>2]|0;
     $53 = HEAP32[$nTemp>>2]|0;
     $54 = (($53) + ($52))|0;
     HEAP32[$nTemp>>2] = $54;
     $55 = HEAP32[$nByte>>2]|0;
     $56 = HEAP32[$pToken>>2]|0;
     $57 = ((($56)) + 4|0);
     HEAP32[$57>>2] = $55;
     $58 = HEAP32[$iEnd>>2]|0;
     $59 = HEAP32[$3>>2]|0;
     $60 = ($58|0)<($59|0);
     if ($60) {
      $61 = HEAP32[$iEnd>>2]|0;
      $62 = HEAP32[$2>>2]|0;
      $63 = (($62) + ($61)|0);
      $64 = HEAP8[$63>>0]|0;
      $65 = $64 << 24 >> 24;
      $66 = ($65|0)==(42);
      $68 = $66;
     } else {
      $68 = 0;
     }
     $67 = $68&1;
     $69 = HEAP32[$pToken>>2]|0;
     $70 = ((($69)) + 8|0);
     HEAP32[$70>>2] = $67;
     $71 = HEAP32[$iBegin>>2]|0;
     $72 = ($71|0)>(0);
     if ($72) {
      $73 = HEAP32[$iBegin>>2]|0;
      $74 = (($73) - 1)|0;
      $75 = HEAP32[$2>>2]|0;
      $76 = (($75) + ($74)|0);
      $77 = HEAP8[$76>>0]|0;
      $78 = $77 << 24 >> 24;
      $79 = ($78|0)==(94);
      $81 = $79;
     } else {
      $81 = 0;
     }
     $80 = $81&1;
     $82 = HEAP32[$pToken>>2]|0;
     $83 = ((($82)) + 12|0);
     HEAP32[$83>>2] = $80;
     $84 = HEAP32[$ii>>2]|0;
     $85 = (($84) + 1)|0;
     HEAP32[$nToken>>2] = $85;
    }
    $86 = HEAP32[$ii>>2]|0;
    $87 = (($86) + 1)|0;
    HEAP32[$ii>>2] = $87;
   }
   $88 = HEAP32[$pModule>>2]|0;
   $89 = ((($88)) + 16|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = HEAP32[$pCursor>>2]|0;
   (FUNCTION_TABLE_ii[$90 & 255]($91)|0);
   HEAP32[$pCursor>>2] = 0;
   label = 14;
  } else {
   label = 14;
  }
 } while(0);
 do {
  if ((label|0) == 14) {
   $92 = HEAP32[$rc>>2]|0;
   $93 = ($92|0)==(101);
   if ($93) {
    HEAP32[$zBuf>>2] = 0;
    $94 = HEAP32[$p>>2]|0;
    $95 = HEAP32[$nToken>>2]|0;
    $96 = ($95*24)|0;
    $97 = (144 + ($96))|0;
    $98 = HEAP32[$nTemp>>2]|0;
    $99 = (($97) + ($98))|0;
    $100 = (_fts3ReallocOrFree($94,$99)|0);
    HEAP32[$p>>2] = $100;
    $101 = HEAP32[$p>>2]|0;
    $102 = ($101|0)!=(0|0);
    if (!($102)) {
     break;
    }
    $103 = HEAP32[$p>>2]|0;
    $104 = HEAP32[$p>>2]|0;
    $105 = ((($104)) + 48|0);
    $106 = ((($105)) + 72|0);
    $107 = HEAP32[$p>>2]|0;
    $108 = $106;
    $109 = $107;
    $110 = (($108) - ($109))|0;
    _memset(($103|0),0,($110|0))|0;
    $111 = HEAP32[$p>>2]|0;
    HEAP32[$111>>2] = 5;
    $112 = HEAP32[$p>>2]|0;
    $113 = ((($112)) + 48|0);
    $114 = HEAP32[$p>>2]|0;
    $115 = ((($114)) + 20|0);
    HEAP32[$115>>2] = $113;
    $116 = HEAP32[$1>>2]|0;
    $117 = ((($116)) + 20|0);
    $118 = HEAP32[$117>>2]|0;
    $119 = HEAP32[$p>>2]|0;
    $120 = ((($119)) + 20|0);
    $121 = HEAP32[$120>>2]|0;
    $122 = ((($121)) + 68|0);
    HEAP32[$122>>2] = $118;
    $123 = HEAP32[$nToken>>2]|0;
    $124 = HEAP32[$p>>2]|0;
    $125 = ((($124)) + 20|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ((($126)) + 64|0);
    HEAP32[$127>>2] = $123;
    $128 = HEAP32[$nToken>>2]|0;
    $129 = HEAP32[$p>>2]|0;
    $130 = ((($129)) + 20|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = ((($131)) + 72|0);
    $133 = (($132) + (($128*24)|0)|0);
    HEAP32[$zBuf>>2] = $133;
    $134 = HEAP32[$zTemp>>2]|0;
    $135 = ($134|0)!=(0|0);
    if ($135) {
     $136 = HEAP32[$zBuf>>2]|0;
     $137 = HEAP32[$zTemp>>2]|0;
     $138 = HEAP32[$nTemp>>2]|0;
     _memcpy(($136|0),($137|0),($138|0))|0;
     $139 = HEAP32[$zTemp>>2]|0;
     _sqlite3_free($139);
    }
    HEAP32[$jj>>2] = 0;
    while(1) {
     $140 = HEAP32[$jj>>2]|0;
     $141 = HEAP32[$p>>2]|0;
     $142 = ((($141)) + 20|0);
     $143 = HEAP32[$142>>2]|0;
     $144 = ((($143)) + 64|0);
     $145 = HEAP32[$144>>2]|0;
     $146 = ($140|0)<($145|0);
     if (!($146)) {
      break;
     }
     $147 = HEAP32[$zBuf>>2]|0;
     $148 = HEAP32[$jj>>2]|0;
     $149 = HEAP32[$p>>2]|0;
     $150 = ((($149)) + 20|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = ((($151)) + 72|0);
     $153 = (($152) + (($148*24)|0)|0);
     HEAP32[$153>>2] = $147;
     $154 = HEAP32[$jj>>2]|0;
     $155 = HEAP32[$p>>2]|0;
     $156 = ((($155)) + 20|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ((($157)) + 72|0);
     $159 = (($158) + (($154*24)|0)|0);
     $160 = ((($159)) + 4|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$zBuf>>2]|0;
     $163 = (($162) + ($161)|0);
     HEAP32[$zBuf>>2] = $163;
     $164 = HEAP32[$jj>>2]|0;
     $165 = (($164) + 1)|0;
     HEAP32[$jj>>2] = $165;
    }
    HEAP32[$rc>>2] = 0;
   }
   $166 = HEAP32[$p>>2]|0;
   $167 = HEAP32[$4>>2]|0;
   HEAP32[$167>>2] = $166;
   $168 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $168;
   $178 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($178|0);
  }
 } while(0);
 $169 = HEAP32[$pCursor>>2]|0;
 $170 = ($169|0)!=(0|0);
 if ($170) {
  $171 = HEAP32[$pModule>>2]|0;
  $172 = ((($171)) + 16|0);
  $173 = HEAP32[$172>>2]|0;
  $174 = HEAP32[$pCursor>>2]|0;
  (FUNCTION_TABLE_ii[$173 & 255]($174)|0);
 }
 $175 = HEAP32[$zTemp>>2]|0;
 _sqlite3_free($175);
 $176 = HEAP32[$p>>2]|0;
 _sqlite3_free($176);
 $177 = HEAP32[$4>>2]|0;
 HEAP32[$177>>2] = 0;
 HEAP32[$0>>2] = 7;
 $178 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($178|0);
}
function _getNextToken($pParse,$iCol,$z,$n,$ppExpr,$pnConsumed) {
 $pParse = $pParse|0;
 $iCol = $iCol|0;
 $z = $z|0;
 $n = $n|0;
 $ppExpr = $ppExpr|0;
 $pnConsumed = $pnConsumed|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $iEnd = 0, $iPosition = 0, $iStart = 0, $nByte = 0, $nToken = 0, $or$cond = 0, $or$cond3 = 0, $pCursor = 0, $pModule = 0, $pRet = 0, $pTokenizer = 0, $rc = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $pTokenizer = sp + 44|0;
 $pModule = sp + 40|0;
 $rc = sp + 36|0;
 $pCursor = sp + 32|0;
 $pRet = sp + 28|0;
 $i = sp + 24|0;
 $zToken = sp + 20|0;
 $nToken = sp + 16|0;
 $iStart = sp + 12|0;
 $iEnd = sp + 8|0;
 $iPosition = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iCol;
 HEAP32[$2>>2] = $z;
 HEAP32[$3>>2] = $n;
 HEAP32[$4>>2] = $ppExpr;
 HEAP32[$5>>2] = $pnConsumed;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTokenizer>>2] = $7;
 $8 = HEAP32[$pTokenizer>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pModule>>2] = $9;
 HEAP32[$pRet>>2] = 0;
 HEAP32[$i>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (($14) + ($13)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)==(40);
  if ($18) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (($20) + ($19)|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22 << 24 >> 24;
  $24 = ($23|0)==(41);
  if ($24) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (($26) + ($25)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($29|0)==(34);
  if ($30) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$i>>2] = $32;
 }
 $33 = HEAP32[$i>>2]|0;
 $34 = HEAP32[$5>>2]|0;
 HEAP32[$34>>2] = $33;
 $35 = HEAP32[$pTokenizer>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 4|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$i>>2]|0;
 $41 = (_sqlite3Fts3OpenTokenizer($35,$38,$39,$40,$pCursor)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)==(0);
 if (!($43)) {
  $138 = HEAP32[$pRet>>2]|0;
  $139 = HEAP32[$4>>2]|0;
  HEAP32[$139>>2] = $138;
  $140 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($140|0);
 }
 HEAP32[$nToken>>2] = 0;
 HEAP32[$iStart>>2] = 0;
 HEAP32[$iEnd>>2] = 0;
 HEAP32[$iPosition>>2] = 0;
 $44 = HEAP32[$pModule>>2]|0;
 $45 = ((($44)) + 20|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$pCursor>>2]|0;
 $48 = (FUNCTION_TABLE_iiiiiii[$46 & 255]($47,$zToken,$nToken,$iStart,$iEnd,$iPosition)|0);
 HEAP32[$rc>>2] = $48;
 $49 = HEAP32[$rc>>2]|0;
 $50 = ($49|0)==(0);
 if ($50) {
  $51 = HEAP32[$nToken>>2]|0;
  $52 = (144 + ($51))|0;
  HEAP32[$nByte>>2] = $52;
  $53 = HEAP32[$nByte>>2]|0;
  $54 = (_fts3MallocZero($53)|0);
  HEAP32[$pRet>>2] = $54;
  $55 = HEAP32[$pRet>>2]|0;
  $56 = ($55|0)!=(0|0);
  L13: do {
   if ($56) {
    $57 = HEAP32[$pRet>>2]|0;
    HEAP32[$57>>2] = 5;
    $58 = HEAP32[$pRet>>2]|0;
    $59 = ((($58)) + 48|0);
    $60 = HEAP32[$pRet>>2]|0;
    $61 = ((($60)) + 20|0);
    HEAP32[$61>>2] = $59;
    $62 = HEAP32[$pRet>>2]|0;
    $63 = ((($62)) + 20|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = ((($64)) + 64|0);
    HEAP32[$65>>2] = 1;
    $66 = HEAP32[$1>>2]|0;
    $67 = HEAP32[$pRet>>2]|0;
    $68 = ((($67)) + 20|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = ((($69)) + 68|0);
    HEAP32[$70>>2] = $66;
    $71 = HEAP32[$nToken>>2]|0;
    $72 = HEAP32[$pRet>>2]|0;
    $73 = ((($72)) + 20|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ((($74)) + 72|0);
    $76 = ((($75)) + 4|0);
    HEAP32[$76>>2] = $71;
    $77 = HEAP32[$pRet>>2]|0;
    $78 = ((($77)) + 20|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ((($79)) + 96|0);
    $81 = HEAP32[$pRet>>2]|0;
    $82 = ((($81)) + 20|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ((($83)) + 72|0);
    HEAP32[$84>>2] = $80;
    $85 = HEAP32[$pRet>>2]|0;
    $86 = ((($85)) + 20|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ((($87)) + 72|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$zToken>>2]|0;
    $91 = HEAP32[$nToken>>2]|0;
    _memcpy(($89|0),($90|0),($91|0))|0;
    $92 = HEAP32[$iEnd>>2]|0;
    $93 = HEAP32[$3>>2]|0;
    $94 = ($92|0)<($93|0);
    if ($94) {
     $95 = HEAP32[$iEnd>>2]|0;
     $96 = HEAP32[$2>>2]|0;
     $97 = (($96) + ($95)|0);
     $98 = HEAP8[$97>>0]|0;
     $99 = $98 << 24 >> 24;
     $100 = ($99|0)==(42);
     if ($100) {
      $101 = HEAP32[$pRet>>2]|0;
      $102 = ((($101)) + 20|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = ((($103)) + 72|0);
      $105 = ((($104)) + 8|0);
      HEAP32[$105>>2] = 1;
      $106 = HEAP32[$iEnd>>2]|0;
      $107 = (($106) + 1)|0;
      HEAP32[$iEnd>>2] = $107;
     }
    }
    while(1) {
     $108 = HEAP32[$0>>2]|0;
     $109 = ((($108)) + 12|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = ($110|0)!=(0);
     $112 = HEAP32[$iStart>>2]|0;
     $113 = ($112|0)>(0);
     $or$cond = $111 & $113;
     if (!($or$cond)) {
      break L13;
     }
     $114 = HEAP32[$iStart>>2]|0;
     $115 = (($114) - 1)|0;
     $116 = HEAP32[$2>>2]|0;
     $117 = (($116) + ($115)|0);
     $118 = HEAP8[$117>>0]|0;
     $119 = $118 << 24 >> 24;
     $120 = ($119|0)==(94);
     if (!($120)) {
      break L13;
     }
     $121 = HEAP32[$pRet>>2]|0;
     $122 = ((($121)) + 20|0);
     $123 = HEAP32[$122>>2]|0;
     $124 = ((($123)) + 72|0);
     $125 = ((($124)) + 12|0);
     HEAP32[$125>>2] = 1;
     $126 = HEAP32[$iStart>>2]|0;
     $127 = (($126) + -1)|0;
     HEAP32[$iStart>>2] = $127;
    }
   } else {
    HEAP32[$rc>>2] = 7;
   }
  } while(0);
  $128 = HEAP32[$iEnd>>2]|0;
  $129 = HEAP32[$5>>2]|0;
  HEAP32[$129>>2] = $128;
 } else {
  $130 = HEAP32[$i>>2]|0;
  $131 = ($130|0)!=(0);
  $132 = HEAP32[$rc>>2]|0;
  $133 = ($132|0)==(101);
  $or$cond3 = $131 & $133;
  if ($or$cond3) {
   HEAP32[$rc>>2] = 0;
  }
 }
 $134 = HEAP32[$pModule>>2]|0;
 $135 = ((($134)) + 16|0);
 $136 = HEAP32[$135>>2]|0;
 $137 = HEAP32[$pCursor>>2]|0;
 (FUNCTION_TABLE_ii[$136 & 255]($137)|0);
 $138 = HEAP32[$pRet>>2]|0;
 $139 = HEAP32[$4>>2]|0;
 HEAP32[$139>>2] = $138;
 $140 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($140|0);
}
function _fts3ReallocOrFree($pOrig,$nNew) {
 $pOrig = $pOrig|0;
 $nNew = $nNew|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $pOrig;
 HEAP32[$1>>2] = $nNew;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3_realloc($2,$3)|0);
 HEAP32[$pRet>>2] = $4;
 $5 = HEAP32[$pRet>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $8 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $7 = HEAP32[$0>>2]|0;
 _sqlite3_free($7);
 $8 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _fts3FreeExprNode($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3Fts3EvalPhraseCleanup($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3_free($6);
 $7 = HEAP32[$0>>2]|0;
 _sqlite3_free($7);
 STACKTOP = sp;return;
}
function _sqlite3Fts3EvalPhraseCleanup($pPhrase) {
 $pPhrase = $pPhrase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pPhrase;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$0>>2]|0;
 _fts3EvalInvalidatePoslist($5);
 $6 = HEAP32[$0>>2]|0;
 dest=$6; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 72|0);
  $15 = (($14) + (($12*24)|0)|0);
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  _fts3SegReaderCursorFree($17);
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 72|0);
  $21 = (($20) + (($18*24)|0)|0);
  $22 = ((($21)) + 20|0);
  HEAP32[$22>>2] = 0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _sqlite3Fts3FreeDeferredTokens($pCsr) {
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pDef = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pCsr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pDef>>2] = $3;
 while(1) {
  $4 = HEAP32[$pDef>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$pDef>>2]|0;
  $7 = ((($6)) + 8|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$pNext>>2] = $8;
  $9 = HEAP32[$pDef>>2]|0;
  $10 = ((($9)) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  _fts3PendingListDelete($11);
  $12 = HEAP32[$pDef>>2]|0;
  _sqlite3_free($12);
  $13 = HEAP32[$pNext>>2]|0;
  HEAP32[$pDef>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 24|0);
 HEAP32[$15>>2] = 0;
 STACKTOP = sp;return;
}
function _fts3SetEstimatedRows($pIdxInfo,$0,$1) {
 $pIdxInfo = $pIdxInfo|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pIdxInfo;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = (_sqlite3_libversion_number()|0);
 $9 = ($8|0)>=(3008002);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 48|0);
 $18 = $17;
 $19 = $18;
 HEAP32[$19>>2] = $12;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $15;
 STACKTOP = sp;return;
}
function _fts3SetUniqueFlag($pIdxInfo) {
 $pIdxInfo = $pIdxInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pIdxInfo;
 $1 = (_sqlite3_libversion_number()|0);
 $2 = ($1|0)>=(3008012);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 | 1;
 HEAP32[$4>>2] = $6;
 STACKTOP = sp;return;
}
function _fts3InitVtab($isCreate,$db,$pAux,$argc,$argv,$ppVTab,$pzErr) {
 $isCreate = $isCreate|0;
 $db = $db|0;
 $pAux = $pAux|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVTab = $ppVTab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aCol = 0, $aFts4Opt = 0, $aIndex = 0, $azNotindexed = 0, $bDescIdx = 0, $bNoDocsize = 0, $i = 0, $iCol = 0, $iOpt = 0, $isFts4 = 0, $j = 0, $k = 0, $n = 0, $n2 = 0, $nByte = 0, $nCol = 0, $nDb = 0, $nIndex = 0, $nKey = 0, $nName = 0;
 var $nNotindexed = 0, $nString = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $p = 0, $pHash = 0, $pOp = 0, $pTokenizer = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer17 = 0, $vararg_buffer20 = 0, $vararg_buffer8 = 0, $z = 0, $z1 = 0, $zCompress = 0;
 var $zContent = 0, $zCsr = 0, $zLanguageid = 0, $zMiss = 0, $zNot = 0, $zPrefix = 0, $zUncompress = 0, $zVal = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0;
 $vararg_buffer20 = sp + 40|0;
 $vararg_buffer17 = sp + 32|0;
 $vararg_buffer14 = sp + 24|0;
 $vararg_buffer11 = sp + 16|0;
 $vararg_buffer8 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 280|0;
 $1 = sp + 276|0;
 $2 = sp + 272|0;
 $3 = sp + 268|0;
 $4 = sp + 264|0;
 $5 = sp + 260|0;
 $6 = sp + 256|0;
 $pHash = sp + 252|0;
 $p = sp + 248|0;
 $rc = sp + 244|0;
 $i = sp + 240|0;
 $nByte = sp + 236|0;
 $iCol = sp + 232|0;
 $nString = sp + 228|0;
 $nCol = sp + 224|0;
 $zCsr = sp + 220|0;
 $nDb = sp + 216|0;
 $nName = sp + 212|0;
 $isFts4 = sp + 208|0;
 $aCol = sp + 204|0;
 $pTokenizer = sp + 200|0;
 $nIndex = sp + 196|0;
 $aIndex = sp + 192|0;
 $bNoDocsize = sp + 188|0;
 $bDescIdx = sp + 184|0;
 $zPrefix = sp + 180|0;
 $zCompress = sp + 176|0;
 $zUncompress = sp + 172|0;
 $zContent = sp + 168|0;
 $zLanguageid = sp + 164|0;
 $azNotindexed = sp + 160|0;
 $nNotindexed = sp + 156|0;
 $z = sp + 152|0;
 $nKey = sp + 148|0;
 $zVal = sp + 144|0;
 $aFts4Opt = sp + 80|0;
 $iOpt = sp + 76|0;
 $pOp = sp + 72|0;
 $j = sp + 68|0;
 $k = sp + 64|0;
 $z1 = sp + 60|0;
 $n = sp + 56|0;
 $n2 = sp + 52|0;
 $zNot = sp + 48|0;
 $zMiss = sp + 44|0;
 HEAP32[$0>>2] = $isCreate;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pAux;
 HEAP32[$3>>2] = $argc;
 HEAP32[$4>>2] = $argv;
 HEAP32[$5>>2] = $ppVTab;
 HEAP32[$6>>2] = $pzErr;
 $7 = HEAP32[$2>>2]|0;
 HEAP32[$pHash>>2] = $7;
 HEAP32[$p>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nString>>2] = 0;
 HEAP32[$nCol>>2] = 0;
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 3|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 $13 = ($12|0)==(52);
 $14 = $13&1;
 HEAP32[$isFts4>>2] = $14;
 HEAP32[$pTokenizer>>2] = 0;
 HEAP32[$nIndex>>2] = 0;
 HEAP32[$aIndex>>2] = 0;
 HEAP32[$bNoDocsize>>2] = 0;
 HEAP32[$bDescIdx>>2] = 0;
 HEAP32[$zPrefix>>2] = 0;
 HEAP32[$zCompress>>2] = 0;
 HEAP32[$zUncompress>>2] = 0;
 HEAP32[$zContent>>2] = 0;
 HEAP32[$zLanguageid>>2] = 0;
 HEAP32[$azNotindexed>>2] = 0;
 HEAP32[$nNotindexed>>2] = 0;
 $15 = HEAP32[$4>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (_strlen($17)|0);
 $19 = (($18) + 1)|0;
 HEAP32[$nDb>>2] = $19;
 $20 = HEAP32[$4>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_strlen($22)|0);
 $24 = (($23) + 1)|0;
 HEAP32[$nName>>2] = $24;
 $25 = HEAP32[$3>>2]|0;
 $26 = (($25) - 2)|0;
 $27 = $26<<2;
 HEAP32[$nByte>>2] = $27;
 $28 = HEAP32[$nByte>>2]|0;
 $29 = (_sqlite3_malloc($28)|0);
 HEAP32[$aCol>>2] = $29;
 $30 = HEAP32[$aCol>>2]|0;
 $31 = ($30|0)!=(0|0);
 if ($31) {
  $32 = HEAP32[$aCol>>2]|0;
  $33 = HEAP32[$nByte>>2]|0;
  _memset(($32|0),0,($33|0))|0;
  $34 = HEAP32[$nByte>>2]|0;
  $35 = (_sqlite3_malloc($34)|0);
  HEAP32[$azNotindexed>>2] = $35;
 }
 $36 = HEAP32[$azNotindexed>>2]|0;
 $37 = ($36|0)!=(0|0);
 if ($37) {
  $38 = HEAP32[$azNotindexed>>2]|0;
  $39 = HEAP32[$nByte>>2]|0;
  _memset(($38|0),0,($39|0))|0;
 }
 $40 = HEAP32[$aCol>>2]|0;
 $41 = ($40|0)!=(0|0);
 $42 = HEAP32[$azNotindexed>>2]|0;
 $43 = ($42|0)!=(0|0);
 $or$cond = $41 & $43;
 do {
  if ($or$cond) {
   HEAP32[$i>>2] = 3;
   while(1) {
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)==(0);
    if (!($45)) {
     break;
    }
    $46 = HEAP32[$i>>2]|0;
    $47 = HEAP32[$3>>2]|0;
    $48 = ($46|0)<($47|0);
    if (!($48)) {
     break;
    }
    $49 = HEAP32[$i>>2]|0;
    $50 = HEAP32[$4>>2]|0;
    $51 = (($50) + ($49<<2)|0);
    $52 = HEAP32[$51>>2]|0;
    HEAP32[$z>>2] = $52;
    $53 = HEAP32[$pTokenizer>>2]|0;
    $54 = ($53|0)!=(0|0);
    if ($54) {
     label = 15;
    } else {
     $55 = HEAP32[$z>>2]|0;
     $56 = (_strlen($55)|0);
     $57 = ($56>>>0)>(8);
     if ($57) {
      $58 = HEAP32[$z>>2]|0;
      $59 = (_sqlite3_strnicmp($58,24006,8)|0);
      $60 = (0)==($59|0);
      if ($60) {
       $61 = HEAP32[$z>>2]|0;
       $62 = ((($61)) + 8|0);
       $63 = HEAP8[$62>>0]|0;
       $64 = (_sqlite3Fts3IsIdChar($63)|0);
       $65 = (0)==($64|0);
       if ($65) {
        $66 = HEAP32[$pHash>>2]|0;
        $67 = HEAP32[$z>>2]|0;
        $68 = ((($67)) + 9|0);
        $69 = HEAP32[$6>>2]|0;
        $70 = (_sqlite3Fts3InitTokenizer($66,$68,$pTokenizer,$69)|0);
        HEAP32[$rc>>2] = $70;
       } else {
        label = 15;
       }
      } else {
       label = 15;
      }
     } else {
      label = 15;
     }
    }
    do {
     if ((label|0) == 15) {
      label = 0;
      $71 = HEAP32[$isFts4>>2]|0;
      $72 = ($71|0)!=(0);
      if ($72) {
       $73 = HEAP32[$z>>2]|0;
       $74 = (_fts3IsSpecialColumn($73,$nKey,$zVal)|0);
       $75 = ($74|0)!=(0);
       if ($75) {
        dest=$aFts4Opt; src=2144; stop=dest+64|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
        $76 = HEAP32[$zVal>>2]|0;
        $77 = ($76|0)!=(0|0);
        if (!($77)) {
         HEAP32[$rc>>2] = 7;
         break;
        }
        HEAP32[$iOpt>>2] = 0;
        while(1) {
         $78 = HEAP32[$iOpt>>2]|0;
         $79 = ($78|0)<(8);
         if (!($79)) {
          break;
         }
         $80 = HEAP32[$iOpt>>2]|0;
         $81 = (($aFts4Opt) + ($80<<3)|0);
         HEAP32[$pOp>>2] = $81;
         $82 = HEAP32[$nKey>>2]|0;
         $83 = HEAP32[$pOp>>2]|0;
         $84 = ((($83)) + 4|0);
         $85 = HEAP32[$84>>2]|0;
         $86 = ($82|0)==($85|0);
         if ($86) {
          $87 = HEAP32[$z>>2]|0;
          $88 = HEAP32[$pOp>>2]|0;
          $89 = HEAP32[$88>>2]|0;
          $90 = HEAP32[$pOp>>2]|0;
          $91 = ((($90)) + 4|0);
          $92 = HEAP32[$91>>2]|0;
          $93 = (_sqlite3_strnicmp($87,$89,$92)|0);
          $94 = ($93|0)!=(0);
          if (!($94)) {
           break;
          }
         }
         $95 = HEAP32[$iOpt>>2]|0;
         $96 = (($95) + 1)|0;
         HEAP32[$iOpt>>2] = $96;
        }
        $97 = HEAP32[$iOpt>>2]|0;
        $98 = ($97|0)==(8);
        L33: do {
         if ($98) {
          $99 = HEAP32[$6>>2]|0;
          $100 = HEAP32[$z>>2]|0;
          HEAP32[$vararg_buffer>>2] = $100;
          _sqlite3Fts3ErrMsg($99,24015,$vararg_buffer);
          HEAP32[$rc>>2] = 1;
         } else {
          $101 = HEAP32[$iOpt>>2]|0;
          switch ($101|0) {
          case 0:  {
           $102 = HEAP32[$zVal>>2]|0;
           $103 = (_strlen($102)|0);
           $104 = ($103|0)!=(4);
           if ($104) {
            label = 29;
           } else {
            $105 = HEAP32[$zVal>>2]|0;
            $106 = (_sqlite3_strnicmp($105,20521,4)|0);
            $107 = ($106|0)!=(0);
            if ($107) {
             label = 29;
            }
           }
           if ((label|0) == 29) {
            label = 0;
            $108 = HEAP32[$6>>2]|0;
            $109 = HEAP32[$zVal>>2]|0;
            HEAP32[$vararg_buffer8>>2] = $109;
            _sqlite3Fts3ErrMsg($108,24042,$vararg_buffer8);
            HEAP32[$rc>>2] = 1;
           }
           HEAP32[$bNoDocsize>>2] = 1;
           break L33;
           break;
          }
          case 1:  {
           $110 = HEAP32[$zPrefix>>2]|0;
           _sqlite3_free($110);
           $111 = HEAP32[$zVal>>2]|0;
           HEAP32[$zPrefix>>2] = $111;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 2:  {
           $112 = HEAP32[$zCompress>>2]|0;
           _sqlite3_free($112);
           $113 = HEAP32[$zVal>>2]|0;
           HEAP32[$zCompress>>2] = $113;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 3:  {
           $114 = HEAP32[$zUncompress>>2]|0;
           _sqlite3_free($114);
           $115 = HEAP32[$zVal>>2]|0;
           HEAP32[$zUncompress>>2] = $115;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 4:  {
           $116 = HEAP32[$zVal>>2]|0;
           $117 = (_strlen($116)|0);
           $118 = ($117|0)!=(3);
           if ($118) {
            label = 36;
           } else {
            $119 = HEAP32[$zVal>>2]|0;
            $120 = (_sqlite3_strnicmp($119,24069,3)|0);
            $121 = ($120|0)!=(0);
            if ($121) {
             label = 36;
            }
           }
           do {
            if ((label|0) == 36) {
             label = 0;
             $122 = HEAP32[$zVal>>2]|0;
             $123 = (_strlen($122)|0);
             $124 = ($123|0)!=(4);
             if (!($124)) {
              $125 = HEAP32[$zVal>>2]|0;
              $126 = (_sqlite3_strnicmp($125,24073,4)|0);
              $127 = ($126|0)!=(0);
              if (!($127)) {
               break;
              }
             }
             $128 = HEAP32[$6>>2]|0;
             $129 = HEAP32[$zVal>>2]|0;
             HEAP32[$vararg_buffer11>>2] = $129;
             _sqlite3Fts3ErrMsg($128,24078,$vararg_buffer11);
             HEAP32[$rc>>2] = 1;
            }
           } while(0);
           $130 = HEAP32[$zVal>>2]|0;
           $131 = HEAP8[$130>>0]|0;
           $132 = $131 << 24 >> 24;
           $133 = ($132|0)==(100);
           if ($133) {
            $139 = 1;
           } else {
            $134 = HEAP32[$zVal>>2]|0;
            $135 = HEAP8[$134>>0]|0;
            $136 = $135 << 24 >> 24;
            $137 = ($136|0)==(68);
            $139 = $137;
           }
           $138 = $139&1;
           HEAP32[$bDescIdx>>2] = $138;
           break L33;
           break;
          }
          case 5:  {
           $140 = HEAP32[$zContent>>2]|0;
           _sqlite3_free($140);
           $141 = HEAP32[$zVal>>2]|0;
           HEAP32[$zContent>>2] = $141;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 6:  {
           $142 = HEAP32[$zLanguageid>>2]|0;
           _sqlite3_free($142);
           $143 = HEAP32[$zVal>>2]|0;
           HEAP32[$zLanguageid>>2] = $143;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          case 7:  {
           $144 = HEAP32[$zVal>>2]|0;
           $145 = HEAP32[$nNotindexed>>2]|0;
           $146 = (($145) + 1)|0;
           HEAP32[$nNotindexed>>2] = $146;
           $147 = HEAP32[$azNotindexed>>2]|0;
           $148 = (($147) + ($145<<2)|0);
           HEAP32[$148>>2] = $144;
           HEAP32[$zVal>>2] = 0;
           break L33;
           break;
          }
          default: {
           break L33;
          }
          }
         }
        } while(0);
        $149 = HEAP32[$zVal>>2]|0;
        _sqlite3_free($149);
        break;
       }
      }
      $150 = HEAP32[$z>>2]|0;
      $151 = (_strlen($150)|0);
      $152 = (($151) + 1)|0;
      $153 = HEAP32[$nString>>2]|0;
      $154 = (($153) + ($152))|0;
      HEAP32[$nString>>2] = $154;
      $155 = HEAP32[$z>>2]|0;
      $156 = HEAP32[$nCol>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$nCol>>2] = $157;
      $158 = HEAP32[$aCol>>2]|0;
      $159 = (($158) + ($156<<2)|0);
      HEAP32[$159>>2] = $155;
     }
    } while(0);
    $160 = HEAP32[$i>>2]|0;
    $161 = (($160) + 1)|0;
    HEAP32[$i>>2] = $161;
   }
   $162 = HEAP32[$rc>>2]|0;
   $163 = ($162|0)==(0);
   $164 = HEAP32[$zContent>>2]|0;
   $165 = ($164|0)!=(0|0);
   $or$cond3 = $163 & $165;
   L65: do {
    if ($or$cond3) {
     $166 = HEAP32[$zCompress>>2]|0;
     _sqlite3_free($166);
     $167 = HEAP32[$zUncompress>>2]|0;
     _sqlite3_free($167);
     HEAP32[$zCompress>>2] = 0;
     HEAP32[$zUncompress>>2] = 0;
     $168 = HEAP32[$nCol>>2]|0;
     $169 = ($168|0)==(0);
     if ($169) {
      $170 = HEAP32[$aCol>>2]|0;
      _sqlite3_free($170);
      HEAP32[$aCol>>2] = 0;
      $171 = HEAP32[$1>>2]|0;
      $172 = HEAP32[$4>>2]|0;
      $173 = ((($172)) + 4|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = HEAP32[$zContent>>2]|0;
      $176 = HEAP32[$6>>2]|0;
      $177 = (_fts3ContentColumns($171,$174,$175,$aCol,$nCol,$nString,$176)|0);
      HEAP32[$rc>>2] = $177;
      $178 = HEAP32[$rc>>2]|0;
      $179 = ($178|0)==(0);
      $180 = HEAP32[$zLanguageid>>2]|0;
      $181 = ($180|0)!=(0|0);
      $or$cond5 = $179 & $181;
      if ($or$cond5) {
       HEAP32[$j>>2] = 0;
       while(1) {
        $182 = HEAP32[$j>>2]|0;
        $183 = HEAP32[$nCol>>2]|0;
        $184 = ($182|0)<($183|0);
        if (!($184)) {
         break L65;
        }
        $185 = HEAP32[$zLanguageid>>2]|0;
        $186 = HEAP32[$j>>2]|0;
        $187 = HEAP32[$aCol>>2]|0;
        $188 = (($187) + ($186<<2)|0);
        $189 = HEAP32[$188>>2]|0;
        $190 = (_sqlite3_stricmp($185,$189)|0);
        $191 = ($190|0)==(0);
        $192 = HEAP32[$j>>2]|0;
        if ($191) {
         break;
        }
        $208 = (($192) + 1)|0;
        HEAP32[$j>>2] = $208;
       }
       HEAP32[$k>>2] = $192;
       while(1) {
        $193 = HEAP32[$k>>2]|0;
        $194 = HEAP32[$nCol>>2]|0;
        $195 = ($193|0)<($194|0);
        if (!($195)) {
         break;
        }
        $196 = HEAP32[$k>>2]|0;
        $197 = (($196) + 1)|0;
        $198 = HEAP32[$aCol>>2]|0;
        $199 = (($198) + ($197<<2)|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = HEAP32[$k>>2]|0;
        $202 = HEAP32[$aCol>>2]|0;
        $203 = (($202) + ($201<<2)|0);
        HEAP32[$203>>2] = $200;
        $204 = HEAP32[$k>>2]|0;
        $205 = (($204) + 1)|0;
        HEAP32[$k>>2] = $205;
       }
       $206 = HEAP32[$nCol>>2]|0;
       $207 = (($206) + -1)|0;
       HEAP32[$nCol>>2] = $207;
      }
     }
    }
   } while(0);
   $209 = HEAP32[$rc>>2]|0;
   $210 = ($209|0)!=(0);
   if (!($210)) {
    $211 = HEAP32[$nCol>>2]|0;
    $212 = ($211|0)==(0);
    if ($212) {
     $213 = HEAP32[$aCol>>2]|0;
     HEAP32[$213>>2] = 24101;
     HEAP32[$nString>>2] = 8;
     HEAP32[$nCol>>2] = 1;
    }
    $214 = HEAP32[$pTokenizer>>2]|0;
    $215 = ($214|0)==(0|0);
    if ($215) {
     $216 = HEAP32[$pHash>>2]|0;
     $217 = HEAP32[$6>>2]|0;
     $218 = (_sqlite3Fts3InitTokenizer($216,20447,$pTokenizer,$217)|0);
     HEAP32[$rc>>2] = $218;
     $219 = HEAP32[$rc>>2]|0;
     $220 = ($219|0)!=(0);
     if ($220) {
      break;
     }
    }
    $221 = HEAP32[$zPrefix>>2]|0;
    $222 = (_fts3PrefixParameter($221,$nIndex,$aIndex)|0);
    HEAP32[$rc>>2] = $222;
    $223 = HEAP32[$rc>>2]|0;
    $224 = ($223|0)==(1);
    if ($224) {
     $225 = HEAP32[$6>>2]|0;
     $226 = HEAP32[$zPrefix>>2]|0;
     HEAP32[$vararg_buffer14>>2] = $226;
     _sqlite3Fts3ErrMsg($225,24109,$vararg_buffer14);
    }
    $227 = HEAP32[$rc>>2]|0;
    $228 = ($227|0)!=(0);
    if (!($228)) {
     $229 = HEAP32[$nCol>>2]|0;
     $230 = $229<<2;
     $231 = (280 + ($230))|0;
     $232 = HEAP32[$nIndex>>2]|0;
     $233 = ($232*24)|0;
     $234 = (($231) + ($233))|0;
     $235 = HEAP32[$nCol>>2]|0;
     $236 = $235;
     $237 = (($234) + ($236))|0;
     $238 = HEAP32[$nName>>2]|0;
     $239 = (($237) + ($238))|0;
     $240 = HEAP32[$nDb>>2]|0;
     $241 = (($239) + ($240))|0;
     $242 = HEAP32[$nString>>2]|0;
     $243 = (($241) + ($242))|0;
     HEAP32[$nByte>>2] = $243;
     $244 = HEAP32[$nByte>>2]|0;
     $245 = (_sqlite3_malloc($244)|0);
     HEAP32[$p>>2] = $245;
     $246 = HEAP32[$p>>2]|0;
     $247 = ($246|0)==(0|0);
     if ($247) {
      HEAP32[$rc>>2] = 7;
      break;
     }
     $248 = HEAP32[$p>>2]|0;
     $249 = HEAP32[$nByte>>2]|0;
     _memset(($248|0),0,($249|0))|0;
     $250 = HEAP32[$1>>2]|0;
     $251 = HEAP32[$p>>2]|0;
     $252 = ((($251)) + 12|0);
     HEAP32[$252>>2] = $250;
     $253 = HEAP32[$nCol>>2]|0;
     $254 = HEAP32[$p>>2]|0;
     $255 = ((($254)) + 24|0);
     HEAP32[$255>>2] = $253;
     $256 = HEAP32[$p>>2]|0;
     $257 = ((($256)) + 260|0);
     HEAP32[$257>>2] = 0;
     $258 = HEAP32[$p>>2]|0;
     $259 = ((($258)) + 280|0);
     $260 = HEAP32[$p>>2]|0;
     $261 = ((($260)) + 28|0);
     HEAP32[$261>>2] = $259;
     $262 = HEAP32[$pTokenizer>>2]|0;
     $263 = HEAP32[$p>>2]|0;
     $264 = ((($263)) + 36|0);
     HEAP32[$264>>2] = $262;
     $265 = HEAP32[$p>>2]|0;
     $266 = ((($265)) + 256|0);
     HEAP32[$266>>2] = 1048576;
     $267 = HEAP32[$isFts4>>2]|0;
     $268 = ($267|0)!=(0);
     $269 = HEAP32[$bNoDocsize>>2]|0;
     $270 = ($269|0)==(0);
     $271 = $268 ? $270 : 0;
     $272 = $271&1;
     $273 = $272&255;
     $274 = HEAP32[$p>>2]|0;
     $275 = ((($274)) + 230|0);
     HEAP8[$275>>0] = $273;
     $276 = HEAP32[$isFts4>>2]|0;
     $277 = $276&255;
     $278 = HEAP32[$p>>2]|0;
     $279 = ((($278)) + 229|0);
     HEAP8[$279>>0] = $277;
     $280 = HEAP32[$isFts4>>2]|0;
     $281 = $280&255;
     $282 = HEAP32[$p>>2]|0;
     $283 = ((($282)) + 228|0);
     HEAP8[$283>>0] = $281;
     $284 = HEAP32[$bDescIdx>>2]|0;
     $285 = $284&255;
     $286 = HEAP32[$p>>2]|0;
     $287 = ((($286)) + 231|0);
     HEAP8[$287>>0] = $285;
     $288 = HEAP32[$p>>2]|0;
     $289 = ((($288)) + 48|0);
     HEAP32[$289>>2] = 255;
     $290 = HEAP32[$zContent>>2]|0;
     $291 = HEAP32[$p>>2]|0;
     $292 = ((($291)) + 40|0);
     HEAP32[$292>>2] = $290;
     $293 = HEAP32[$zLanguageid>>2]|0;
     $294 = HEAP32[$p>>2]|0;
     $295 = ((($294)) + 44|0);
     HEAP32[$295>>2] = $293;
     HEAP32[$zContent>>2] = 0;
     HEAP32[$zLanguageid>>2] = 0;
     $296 = HEAP32[$nCol>>2]|0;
     $297 = HEAP32[$p>>2]|0;
     $298 = ((($297)) + 28|0);
     $299 = HEAP32[$298>>2]|0;
     $300 = (($299) + ($296<<2)|0);
     $301 = HEAP32[$p>>2]|0;
     $302 = ((($301)) + 252|0);
     HEAP32[$302>>2] = $300;
     $303 = HEAP32[$p>>2]|0;
     $304 = ((($303)) + 252|0);
     $305 = HEAP32[$304>>2]|0;
     $306 = HEAP32[$aIndex>>2]|0;
     $307 = HEAP32[$nIndex>>2]|0;
     $308 = ($307*24)|0;
     _memcpy(($305|0),($306|0),($308|0))|0;
     $309 = HEAP32[$nIndex>>2]|0;
     $310 = HEAP32[$p>>2]|0;
     $311 = ((($310)) + 248|0);
     HEAP32[$311>>2] = $309;
     HEAP32[$i>>2] = 0;
     while(1) {
      $312 = HEAP32[$i>>2]|0;
      $313 = HEAP32[$nIndex>>2]|0;
      $314 = ($312|0)<($313|0);
      if (!($314)) {
       break;
      }
      $315 = HEAP32[$i>>2]|0;
      $316 = HEAP32[$p>>2]|0;
      $317 = ((($316)) + 252|0);
      $318 = HEAP32[$317>>2]|0;
      $319 = (($318) + (($315*24)|0)|0);
      $320 = ((($319)) + 4|0);
      _sqlite3Fts3HashInit($320,1,1);
      $321 = HEAP32[$i>>2]|0;
      $322 = (($321) + 1)|0;
      HEAP32[$i>>2] = $322;
     }
     $323 = HEAP32[$nIndex>>2]|0;
     $324 = HEAP32[$p>>2]|0;
     $325 = ((($324)) + 252|0);
     $326 = HEAP32[$325>>2]|0;
     $327 = (($326) + (($323*24)|0)|0);
     $328 = HEAP32[$p>>2]|0;
     $329 = ((($328)) + 32|0);
     HEAP32[$329>>2] = $327;
     $330 = HEAP32[$nCol>>2]|0;
     $331 = HEAP32[$p>>2]|0;
     $332 = ((($331)) + 32|0);
     $333 = HEAP32[$332>>2]|0;
     $334 = (($333) + ($330)|0);
     HEAP32[$zCsr>>2] = $334;
     $335 = HEAP32[$zCsr>>2]|0;
     $336 = HEAP32[$p>>2]|0;
     $337 = ((($336)) + 20|0);
     HEAP32[$337>>2] = $335;
     $338 = HEAP32[$zCsr>>2]|0;
     $339 = HEAP32[$4>>2]|0;
     $340 = ((($339)) + 8|0);
     $341 = HEAP32[$340>>2]|0;
     $342 = HEAP32[$nName>>2]|0;
     _memcpy(($338|0),($341|0),($342|0))|0;
     $343 = HEAP32[$nName>>2]|0;
     $344 = HEAP32[$zCsr>>2]|0;
     $345 = (($344) + ($343)|0);
     HEAP32[$zCsr>>2] = $345;
     $346 = HEAP32[$zCsr>>2]|0;
     $347 = HEAP32[$p>>2]|0;
     $348 = ((($347)) + 16|0);
     HEAP32[$348>>2] = $346;
     $349 = HEAP32[$zCsr>>2]|0;
     $350 = HEAP32[$4>>2]|0;
     $351 = ((($350)) + 4|0);
     $352 = HEAP32[$351>>2]|0;
     $353 = HEAP32[$nDb>>2]|0;
     _memcpy(($349|0),($352|0),($353|0))|0;
     $354 = HEAP32[$nDb>>2]|0;
     $355 = HEAP32[$zCsr>>2]|0;
     $356 = (($355) + ($354)|0);
     HEAP32[$zCsr>>2] = $356;
     HEAP32[$iCol>>2] = 0;
     while(1) {
      $357 = HEAP32[$iCol>>2]|0;
      $358 = HEAP32[$nCol>>2]|0;
      $359 = ($357|0)<($358|0);
      if (!($359)) {
       break;
      }
      HEAP32[$n>>2] = 0;
      $360 = HEAP32[$iCol>>2]|0;
      $361 = HEAP32[$aCol>>2]|0;
      $362 = (($361) + ($360<<2)|0);
      $363 = HEAP32[$362>>2]|0;
      $364 = (_sqlite3Fts3NextToken($363,$n)|0);
      HEAP32[$z1>>2] = $364;
      $365 = HEAP32[$zCsr>>2]|0;
      $366 = HEAP32[$z1>>2]|0;
      $367 = HEAP32[$n>>2]|0;
      _memcpy(($365|0),($366|0),($367|0))|0;
      $368 = HEAP32[$n>>2]|0;
      $369 = HEAP32[$zCsr>>2]|0;
      $370 = (($369) + ($368)|0);
      HEAP8[$370>>0] = 0;
      $371 = HEAP32[$zCsr>>2]|0;
      _sqlite3Fts3Dequote($371);
      $372 = HEAP32[$zCsr>>2]|0;
      $373 = HEAP32[$iCol>>2]|0;
      $374 = HEAP32[$p>>2]|0;
      $375 = ((($374)) + 28|0);
      $376 = HEAP32[$375>>2]|0;
      $377 = (($376) + ($373<<2)|0);
      HEAP32[$377>>2] = $372;
      $378 = HEAP32[$n>>2]|0;
      $379 = (($378) + 1)|0;
      $380 = HEAP32[$zCsr>>2]|0;
      $381 = (($380) + ($379)|0);
      HEAP32[$zCsr>>2] = $381;
      $382 = HEAP32[$iCol>>2]|0;
      $383 = (($382) + 1)|0;
      HEAP32[$iCol>>2] = $383;
     }
     HEAP32[$iCol>>2] = 0;
     while(1) {
      $384 = HEAP32[$iCol>>2]|0;
      $385 = HEAP32[$nCol>>2]|0;
      $386 = ($384|0)<($385|0);
      if (!($386)) {
       break;
      }
      $387 = HEAP32[$iCol>>2]|0;
      $388 = HEAP32[$p>>2]|0;
      $389 = ((($388)) + 28|0);
      $390 = HEAP32[$389>>2]|0;
      $391 = (($390) + ($387<<2)|0);
      $392 = HEAP32[$391>>2]|0;
      $393 = (_strlen($392)|0);
      HEAP32[$n2>>2] = $393;
      HEAP32[$i>>2] = 0;
      while(1) {
       $394 = HEAP32[$i>>2]|0;
       $395 = HEAP32[$nNotindexed>>2]|0;
       $396 = ($394|0)<($395|0);
       if (!($396)) {
        break;
       }
       $397 = HEAP32[$i>>2]|0;
       $398 = HEAP32[$azNotindexed>>2]|0;
       $399 = (($398) + ($397<<2)|0);
       $400 = HEAP32[$399>>2]|0;
       HEAP32[$zNot>>2] = $400;
       $401 = HEAP32[$zNot>>2]|0;
       $402 = ($401|0)!=(0|0);
       if ($402) {
        $403 = HEAP32[$n2>>2]|0;
        $404 = HEAP32[$zNot>>2]|0;
        $405 = (_strlen($404)|0);
        $406 = ($403|0)==($405|0);
        if ($406) {
         $407 = HEAP32[$iCol>>2]|0;
         $408 = HEAP32[$p>>2]|0;
         $409 = ((($408)) + 28|0);
         $410 = HEAP32[$409>>2]|0;
         $411 = (($410) + ($407<<2)|0);
         $412 = HEAP32[$411>>2]|0;
         $413 = HEAP32[$zNot>>2]|0;
         $414 = HEAP32[$n2>>2]|0;
         $415 = (_sqlite3_strnicmp($412,$413,$414)|0);
         $416 = (0)==($415|0);
         if ($416) {
          $417 = HEAP32[$iCol>>2]|0;
          $418 = HEAP32[$p>>2]|0;
          $419 = ((($418)) + 32|0);
          $420 = HEAP32[$419>>2]|0;
          $421 = (($420) + ($417)|0);
          HEAP8[$421>>0] = 1;
          $422 = HEAP32[$zNot>>2]|0;
          _sqlite3_free($422);
          $423 = HEAP32[$i>>2]|0;
          $424 = HEAP32[$azNotindexed>>2]|0;
          $425 = (($424) + ($423<<2)|0);
          HEAP32[$425>>2] = 0;
         }
        }
       }
       $426 = HEAP32[$i>>2]|0;
       $427 = (($426) + 1)|0;
       HEAP32[$i>>2] = $427;
      }
      $428 = HEAP32[$iCol>>2]|0;
      $429 = (($428) + 1)|0;
      HEAP32[$iCol>>2] = $429;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $430 = HEAP32[$i>>2]|0;
      $431 = HEAP32[$nNotindexed>>2]|0;
      $432 = ($430|0)<($431|0);
      if (!($432)) {
       break;
      }
      $433 = HEAP32[$i>>2]|0;
      $434 = HEAP32[$azNotindexed>>2]|0;
      $435 = (($434) + ($433<<2)|0);
      $436 = HEAP32[$435>>2]|0;
      $437 = ($436|0)!=(0|0);
      if ($437) {
       $438 = HEAP32[$6>>2]|0;
       $439 = HEAP32[$i>>2]|0;
       $440 = HEAP32[$azNotindexed>>2]|0;
       $441 = (($440) + ($439<<2)|0);
       $442 = HEAP32[$441>>2]|0;
       HEAP32[$vararg_buffer17>>2] = $442;
       _sqlite3Fts3ErrMsg($438,24144,$vararg_buffer17);
       HEAP32[$rc>>2] = 1;
      }
      $443 = HEAP32[$i>>2]|0;
      $444 = (($443) + 1)|0;
      HEAP32[$i>>2] = $444;
     }
     $445 = HEAP32[$rc>>2]|0;
     $446 = ($445|0)==(0);
     if ($446) {
      $447 = HEAP32[$zCompress>>2]|0;
      $448 = ($447|0)==(0|0);
      $449 = $448&1;
      $450 = HEAP32[$zUncompress>>2]|0;
      $451 = ($450|0)==(0|0);
      $452 = $451&1;
      $453 = ($449|0)!=($452|0);
      if ($453) {
       $454 = HEAP32[$zCompress>>2]|0;
       $455 = ($454|0)==(0|0);
       $456 = $455 ? 24163 : 24172;
       HEAP32[$zMiss>>2] = $456;
       HEAP32[$rc>>2] = 1;
       $457 = HEAP32[$6>>2]|0;
       $458 = HEAP32[$zMiss>>2]|0;
       HEAP32[$vararg_buffer20>>2] = $458;
       _sqlite3Fts3ErrMsg($457,24183,$vararg_buffer20);
      }
     }
     $459 = HEAP32[$p>>2]|0;
     $460 = HEAP32[$zUncompress>>2]|0;
     $461 = (_fts3ReadExprList($459,$460,$rc)|0);
     $462 = HEAP32[$p>>2]|0;
     $463 = ((($462)) + 216|0);
     HEAP32[$463>>2] = $461;
     $464 = HEAP32[$p>>2]|0;
     $465 = HEAP32[$zCompress>>2]|0;
     $466 = (_fts3WriteExprList($464,$465,$rc)|0);
     $467 = HEAP32[$p>>2]|0;
     $468 = ((($467)) + 220|0);
     HEAP32[$468>>2] = $466;
     $469 = HEAP32[$rc>>2]|0;
     $470 = ($469|0)!=(0);
     if (!($470)) {
      $471 = HEAP32[$0>>2]|0;
      $472 = ($471|0)!=(0);
      if ($472) {
       $473 = HEAP32[$p>>2]|0;
       $474 = (_fts3CreateTables($473)|0);
       HEAP32[$rc>>2] = $474;
      }
      $475 = HEAP32[$isFts4>>2]|0;
      $476 = ($475|0)!=(0);
      $477 = HEAP32[$0>>2]|0;
      $478 = ($477|0)!=(0);
      $or$cond7 = $476 | $478;
      if (!($or$cond7)) {
       $479 = HEAP32[$p>>2]|0;
       $480 = ((($479)) + 229|0);
       HEAP8[$480>>0] = 2;
      }
      $481 = HEAP32[$p>>2]|0;
      _fts3DatabasePageSize($rc,$481);
      $482 = HEAP32[$p>>2]|0;
      $483 = ((($482)) + 236|0);
      $484 = HEAP32[$483>>2]|0;
      $485 = (($484) - 35)|0;
      $486 = HEAP32[$p>>2]|0;
      $487 = ((($486)) + 224|0);
      HEAP32[$487>>2] = $485;
      $488 = HEAP32[$p>>2]|0;
      _fts3DeclareVtab($rc,$488);
     }
    }
   }
  } else {
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $489 = HEAP32[$zPrefix>>2]|0;
 _sqlite3_free($489);
 $490 = HEAP32[$aIndex>>2]|0;
 _sqlite3_free($490);
 $491 = HEAP32[$zCompress>>2]|0;
 _sqlite3_free($491);
 $492 = HEAP32[$zUncompress>>2]|0;
 _sqlite3_free($492);
 $493 = HEAP32[$zContent>>2]|0;
 _sqlite3_free($493);
 $494 = HEAP32[$zLanguageid>>2]|0;
 _sqlite3_free($494);
 HEAP32[$i>>2] = 0;
 while(1) {
  $495 = HEAP32[$i>>2]|0;
  $496 = HEAP32[$nNotindexed>>2]|0;
  $497 = ($495|0)<($496|0);
  if (!($497)) {
   break;
  }
  $498 = HEAP32[$i>>2]|0;
  $499 = HEAP32[$azNotindexed>>2]|0;
  $500 = (($499) + ($498<<2)|0);
  $501 = HEAP32[$500>>2]|0;
  _sqlite3_free($501);
  $502 = HEAP32[$i>>2]|0;
  $503 = (($502) + 1)|0;
  HEAP32[$i>>2] = $503;
 }
 $504 = HEAP32[$aCol>>2]|0;
 _sqlite3_free($504);
 $505 = HEAP32[$azNotindexed>>2]|0;
 _sqlite3_free($505);
 $506 = HEAP32[$rc>>2]|0;
 $507 = ($506|0)!=(0);
 $508 = HEAP32[$p>>2]|0;
 if (!($507)) {
  $518 = HEAP32[$5>>2]|0;
  HEAP32[$518>>2] = $508;
  $519 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($519|0);
 }
 $509 = ($508|0)!=(0|0);
 if ($509) {
  $510 = HEAP32[$p>>2]|0;
  (_fts3DisconnectMethod($510)|0);
  $519 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($519|0);
 }
 $511 = HEAP32[$pTokenizer>>2]|0;
 $512 = ($511|0)!=(0|0);
 if (!($512)) {
  $519 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($519|0);
 }
 $513 = HEAP32[$pTokenizer>>2]|0;
 $514 = HEAP32[$513>>2]|0;
 $515 = ((($514)) + 8|0);
 $516 = HEAP32[$515>>2]|0;
 $517 = HEAP32[$pTokenizer>>2]|0;
 (FUNCTION_TABLE_ii[$516 & 255]($517)|0);
 $519 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($519|0);
}
function _sqlite3Fts3IsIdChar($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $c;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1 << 24 >> 24;
 $3 = $2 & 128;
 $4 = ($3|0)!=(0);
 if ($4) {
  $12 = 1;
  $11 = $12&1;
  STACKTOP = sp;return ($11|0);
 }
 $5 = HEAP8[$0>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = (24865 + ($6)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)!=(0);
 $12 = $10;
 $11 = $12&1;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3Fts3InitTokenizer($pHash,$zArg,$ppTok,$pzErr) {
 $pHash = $pHash|0;
 $zArg = $zArg|0;
 $ppTok = $ppTok|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $aArg = 0, $aNew = 0, $iArg = 0, $m = 0, $n = 0, $nNew = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $z = 0, $zCopy = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $rc = sp + 56|0;
 $z = sp + 52|0;
 $n = sp + 48|0;
 $zCopy = sp + 44|0;
 $zEnd = sp + 40|0;
 $m = sp + 36|0;
 $aArg = sp + 32|0;
 $iArg = sp + 28|0;
 $nNew = sp + 24|0;
 $aNew = sp + 20|0;
 HEAP32[$1>>2] = $pHash;
 HEAP32[$2>>2] = $zArg;
 HEAP32[$3>>2] = $ppTok;
 HEAP32[$4>>2] = $pzErr;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$z>>2] = $5;
 HEAP32[$n>>2] = 0;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $6;
 $7 = (_sqlite3_mprintf(18391,$vararg_buffer)|0);
 HEAP32[$zCopy>>2] = $7;
 $8 = HEAP32[$zCopy>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  HEAP32[$0>>2] = 7;
  $83 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($83|0);
 }
 $10 = HEAP32[$zCopy>>2]|0;
 $11 = (_strlen($10)|0);
 $12 = HEAP32[$zCopy>>2]|0;
 $13 = (($12) + ($11)|0);
 HEAP32[$zEnd>>2] = $13;
 $14 = HEAP32[$zCopy>>2]|0;
 $15 = (_sqlite3Fts3NextToken($14,$n)|0);
 HEAP32[$z>>2] = $15;
 $16 = HEAP32[$z>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$zCopy>>2]|0;
  HEAP32[$z>>2] = $18;
 }
 $19 = HEAP32[$n>>2]|0;
 $20 = HEAP32[$z>>2]|0;
 $21 = (($20) + ($19)|0);
 HEAP8[$21>>0] = 0;
 $22 = HEAP32[$z>>2]|0;
 _sqlite3Fts3Dequote($22);
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$z>>2]|0;
 $25 = HEAP32[$z>>2]|0;
 $26 = (_strlen($25)|0);
 $27 = (($26) + 1)|0;
 $28 = (_sqlite3Fts3HashFind($23,$24,$27)|0);
 HEAP32[$m>>2] = $28;
 $29 = HEAP32[$m>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  HEAP32[$aArg>>2] = 0;
  HEAP32[$iArg>>2] = 0;
  $33 = HEAP32[$n>>2]|0;
  $34 = (($33) + 1)|0;
  $35 = HEAP32[$z>>2]|0;
  $36 = (($35) + ($34)|0);
  HEAP32[$z>>2] = $36;
  while(1) {
   $37 = HEAP32[$z>>2]|0;
   $38 = HEAP32[$zEnd>>2]|0;
   $39 = ($37>>>0)<($38>>>0);
   if (!($39)) {
    break;
   }
   $40 = HEAP32[$z>>2]|0;
   $41 = (_sqlite3Fts3NextToken($40,$n)|0);
   HEAP32[$z>>2] = $41;
   $42 = (0|0)!=($41|0);
   if (!($42)) {
    break;
   }
   $43 = HEAP32[$iArg>>2]|0;
   $44 = (($43) + 1)|0;
   $45 = $44<<2;
   HEAP32[$nNew>>2] = $45;
   $46 = HEAP32[$aArg>>2]|0;
   $47 = HEAP32[$nNew>>2]|0;
   $48 = (_sqlite3_realloc($46,$47)|0);
   HEAP32[$aNew>>2] = $48;
   $49 = HEAP32[$aNew>>2]|0;
   $50 = ($49|0)!=(0|0);
   if (!($50)) {
    label = 11;
    break;
   }
   $53 = HEAP32[$aNew>>2]|0;
   HEAP32[$aArg>>2] = $53;
   $54 = HEAP32[$z>>2]|0;
   $55 = HEAP32[$iArg>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$iArg>>2] = $56;
   $57 = HEAP32[$aArg>>2]|0;
   $58 = (($57) + ($55<<2)|0);
   HEAP32[$58>>2] = $54;
   $59 = HEAP32[$n>>2]|0;
   $60 = HEAP32[$z>>2]|0;
   $61 = (($60) + ($59)|0);
   HEAP8[$61>>0] = 0;
   $62 = HEAP32[$z>>2]|0;
   _sqlite3Fts3Dequote($62);
   $63 = HEAP32[$n>>2]|0;
   $64 = (($63) + 1)|0;
   $65 = HEAP32[$z>>2]|0;
   $66 = (($65) + ($64)|0);
   HEAP32[$z>>2] = $66;
  }
  if ((label|0) == 11) {
   $51 = HEAP32[$zCopy>>2]|0;
   _sqlite3_free($51);
   $52 = HEAP32[$aArg>>2]|0;
   _sqlite3_free($52);
   HEAP32[$0>>2] = 7;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  }
  $67 = HEAP32[$m>>2]|0;
  $68 = ((($67)) + 4|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$iArg>>2]|0;
  $71 = HEAP32[$aArg>>2]|0;
  $72 = HEAP32[$3>>2]|0;
  $73 = (FUNCTION_TABLE_iiii[$69 & 255]($70,$71,$72)|0);
  HEAP32[$rc>>2] = $73;
  $74 = HEAP32[$rc>>2]|0;
  $75 = ($74|0)!=(0);
  if ($75) {
   $76 = HEAP32[$4>>2]|0;
   _sqlite3Fts3ErrMsg($76,24847,$vararg_buffer4);
  } else {
   $77 = HEAP32[$m>>2]|0;
   $78 = HEAP32[$3>>2]|0;
   $79 = HEAP32[$78>>2]|0;
   HEAP32[$79>>2] = $77;
  }
  $80 = HEAP32[$aArg>>2]|0;
  _sqlite3_free($80);
 } else {
  $31 = HEAP32[$4>>2]|0;
  $32 = HEAP32[$z>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $32;
  _sqlite3Fts3ErrMsg($31,20595,$vararg_buffer1);
  HEAP32[$rc>>2] = 1;
 }
 $81 = HEAP32[$zCopy>>2]|0;
 _sqlite3_free($81);
 $82 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $82;
 $83 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($83|0);
}
function _fts3IsSpecialColumn($z,$pnKey,$pzValue) {
 $z = $z|0;
 $pnKey = $pnKey|0;
 $pzValue = $pzValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $zCsr = 0, $zValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $zValue = sp + 8|0;
 $zCsr = sp + 4|0;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pnKey;
 HEAP32[$3>>2] = $pzValue;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$zCsr>>2] = $4;
 while(1) {
  $5 = HEAP32[$zCsr>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)!=(61);
  $9 = HEAP32[$zCsr>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)==(0);
  if ($12) {
   label = 4;
   break;
  }
  $13 = HEAP32[$zCsr>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP32[$zCsr>>2] = $14;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = $9;
 $17 = $15;
 $18 = (($16) - ($17))|0;
 $19 = HEAP32[$2>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$zCsr>>2]|0;
 $21 = ((($20)) + 1|0);
 HEAP32[$vararg_buffer>>2] = $21;
 $22 = (_sqlite3_mprintf(18391,$vararg_buffer)|0);
 HEAP32[$zValue>>2] = $22;
 $23 = HEAP32[$zValue>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$zValue>>2]|0;
  _sqlite3Fts3Dequote($25);
 }
 $26 = HEAP32[$zValue>>2]|0;
 $27 = HEAP32[$3>>2]|0;
 HEAP32[$27>>2] = $26;
 HEAP32[$0>>2] = 1;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _fts3ContentColumns($db,$zDb,$zTbl,$pazCol,$pnCol,$pnStr,$pzErr) {
 $db = $db|0;
 $zDb = $zDb|0;
 $zTbl = $zTbl|0;
 $pazCol = $pazCol|0;
 $pnCol = $pnCol|0;
 $pnStr = $pnStr|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $azCol = 0, $i = 0, $n = 0;
 var $nCol = 0, $nStr = 0, $p = 0, $pStmt = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $zCol = 0, $zCol1 = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $6 = sp + 56|0;
 $rc = sp + 52|0;
 $zSql = sp + 48|0;
 $pStmt = sp + 44|0;
 $azCol = sp + 40|0;
 $nStr = sp + 36|0;
 $nCol = sp + 32|0;
 $i = sp + 28|0;
 $zCol = sp + 24|0;
 $p = sp + 20|0;
 $zCol1 = sp + 16|0;
 $n = sp + 12|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zDb;
 HEAP32[$2>>2] = $zTbl;
 HEAP32[$3>>2] = $pazCol;
 HEAP32[$4>>2] = $pnCol;
 HEAP32[$5>>2] = $pnStr;
 HEAP32[$6>>2] = $pzErr;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pStmt>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $7;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $8;
 $9 = (_sqlite3_mprintf(24792,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $9;
 $10 = HEAP32[$zSql>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$zSql>>2]|0;
  $14 = (_sqlite3_prepare($12,$13,-1,$pStmt,0)|0);
  HEAP32[$rc>>2] = $14;
  $15 = HEAP32[$rc>>2]|0;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = HEAP32[$6>>2]|0;
   $18 = HEAP32[$0>>2]|0;
   $19 = (_sqlite3_errmsg($18)|0);
   HEAP32[$vararg_buffer2>>2] = $19;
   _sqlite3Fts3ErrMsg($17,18391,$vararg_buffer2);
  }
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $20 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($20);
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if (!($22)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 HEAP32[$nStr>>2] = 0;
 $23 = HEAP32[$pStmt>>2]|0;
 $24 = (_sqlite3_column_count($23)|0);
 HEAP32[$nCol>>2] = $24;
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$nCol>>2]|0;
  $27 = ($25|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$pStmt>>2]|0;
  $29 = HEAP32[$i>>2]|0;
  $30 = (_sqlite3_column_name($28,$29)|0);
  HEAP32[$zCol>>2] = $30;
  $31 = HEAP32[$zCol>>2]|0;
  $32 = (_strlen($31)|0);
  $33 = (($32) + 1)|0;
  $34 = HEAP32[$nStr>>2]|0;
  $35 = (($34) + ($33))|0;
  HEAP32[$nStr>>2] = $35;
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 $38 = HEAP32[$nCol>>2]|0;
 $39 = $38<<2;
 $40 = HEAP32[$nStr>>2]|0;
 $41 = (($39) + ($40))|0;
 $42 = (_sqlite3_malloc($41)|0);
 HEAP32[$azCol>>2] = $42;
 $43 = HEAP32[$azCol>>2]|0;
 $44 = ($43|0)==(0|0);
 L13: do {
  if ($44) {
   HEAP32[$rc>>2] = 7;
  } else {
   $45 = HEAP32[$nCol>>2]|0;
   $46 = HEAP32[$azCol>>2]|0;
   $47 = (($46) + ($45<<2)|0);
   HEAP32[$p>>2] = $47;
   HEAP32[$i>>2] = 0;
   while(1) {
    $48 = HEAP32[$i>>2]|0;
    $49 = HEAP32[$nCol>>2]|0;
    $50 = ($48|0)<($49|0);
    if (!($50)) {
     break L13;
    }
    $51 = HEAP32[$pStmt>>2]|0;
    $52 = HEAP32[$i>>2]|0;
    $53 = (_sqlite3_column_name($51,$52)|0);
    HEAP32[$zCol1>>2] = $53;
    $54 = HEAP32[$zCol1>>2]|0;
    $55 = (_strlen($54)|0);
    $56 = (($55) + 1)|0;
    HEAP32[$n>>2] = $56;
    $57 = HEAP32[$p>>2]|0;
    $58 = HEAP32[$zCol1>>2]|0;
    $59 = HEAP32[$n>>2]|0;
    _memcpy(($57|0),($58|0),($59|0))|0;
    $60 = HEAP32[$p>>2]|0;
    $61 = HEAP32[$i>>2]|0;
    $62 = HEAP32[$azCol>>2]|0;
    $63 = (($62) + ($61<<2)|0);
    HEAP32[$63>>2] = $60;
    $64 = HEAP32[$n>>2]|0;
    $65 = HEAP32[$p>>2]|0;
    $66 = (($65) + ($64)|0);
    HEAP32[$p>>2] = $66;
    $67 = HEAP32[$i>>2]|0;
    $68 = (($67) + 1)|0;
    HEAP32[$i>>2] = $68;
   }
  }
 } while(0);
 $69 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_finalize($69)|0);
 $70 = HEAP32[$nCol>>2]|0;
 $71 = HEAP32[$4>>2]|0;
 HEAP32[$71>>2] = $70;
 $72 = HEAP32[$nStr>>2]|0;
 $73 = HEAP32[$5>>2]|0;
 HEAP32[$73>>2] = $72;
 $74 = HEAP32[$azCol>>2]|0;
 $75 = HEAP32[$3>>2]|0;
 HEAP32[$75>>2] = $74;
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _fts3PrefixParameter($zParam,$pnIndex,$apIndex) {
 $zParam = $zParam|0;
 $pnIndex = $pnIndex|0;
 $apIndex = $apIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aIndex = 0;
 var $i = 0, $nIndex = 0, $nPrefix = 0, $p = 0, $p1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aIndex = sp + 20|0;
 $nIndex = sp + 16|0;
 $p = sp + 12|0;
 $p1 = sp + 8|0;
 $i = sp + 4|0;
 $nPrefix = sp;
 HEAP32[$1>>2] = $zParam;
 HEAP32[$2>>2] = $pnIndex;
 HEAP32[$3>>2] = $apIndex;
 HEAP32[$nIndex>>2] = 1;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 L1: do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = HEAP8[$6>>0]|0;
   $8 = $7 << 24 >> 24;
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$nIndex>>2]|0;
    $11 = (($10) + 1)|0;
    HEAP32[$nIndex>>2] = $11;
    $12 = HEAP32[$1>>2]|0;
    HEAP32[$p>>2] = $12;
    while(1) {
     $13 = HEAP32[$p>>2]|0;
     $14 = HEAP8[$13>>0]|0;
     $15 = ($14<<24>>24)!=(0);
     if (!($15)) {
      break L1;
     }
     $16 = HEAP32[$p>>2]|0;
     $17 = HEAP8[$16>>0]|0;
     $18 = $17 << 24 >> 24;
     $19 = ($18|0)==(44);
     if ($19) {
      $20 = HEAP32[$nIndex>>2]|0;
      $21 = (($20) + 1)|0;
      HEAP32[$nIndex>>2] = $21;
     }
     $22 = HEAP32[$p>>2]|0;
     $23 = ((($22)) + 1|0);
     HEAP32[$p>>2] = $23;
    }
   }
  }
 } while(0);
 $24 = HEAP32[$nIndex>>2]|0;
 $25 = ($24*24)|0;
 $26 = (_sqlite3_malloc($25)|0);
 HEAP32[$aIndex>>2] = $26;
 $27 = HEAP32[$aIndex>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 HEAP32[$28>>2] = $27;
 $29 = HEAP32[$aIndex>>2]|0;
 $30 = ($29|0)!=(0|0);
 if (!($30)) {
  HEAP32[$0>>2] = 7;
  $58 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $31 = HEAP32[$aIndex>>2]|0;
 $32 = HEAP32[$nIndex>>2]|0;
 $33 = ($32*24)|0;
 _memset(($31|0),0,($33|0))|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = ($34|0)!=(0|0);
 L15: do {
  if ($35) {
   $36 = HEAP32[$1>>2]|0;
   HEAP32[$p1>>2] = $36;
   HEAP32[$i>>2] = 1;
   while(1) {
    $37 = HEAP32[$i>>2]|0;
    $38 = HEAP32[$nIndex>>2]|0;
    $39 = ($37|0)<($38|0);
    if (!($39)) {
     break L15;
    }
    HEAP32[$nPrefix>>2] = 0;
    $40 = (_fts3GobbleInt($p1,$nPrefix)|0);
    $41 = ($40|0)!=(0);
    if ($41) {
     break;
    }
    $42 = HEAP32[$nPrefix>>2]|0;
    $43 = ($42|0)==(0);
    if ($43) {
     $44 = HEAP32[$nIndex>>2]|0;
     $45 = (($44) + -1)|0;
     HEAP32[$nIndex>>2] = $45;
     $46 = HEAP32[$i>>2]|0;
     $47 = (($46) + -1)|0;
     HEAP32[$i>>2] = $47;
    } else {
     $48 = HEAP32[$nPrefix>>2]|0;
     $49 = HEAP32[$i>>2]|0;
     $50 = HEAP32[$aIndex>>2]|0;
     $51 = (($50) + (($49*24)|0)|0);
     HEAP32[$51>>2] = $48;
    }
    $52 = HEAP32[$p1>>2]|0;
    $53 = ((($52)) + 1|0);
    HEAP32[$p1>>2] = $53;
    $54 = HEAP32[$i>>2]|0;
    $55 = (($54) + 1)|0;
    HEAP32[$i>>2] = $55;
   }
   HEAP32[$0>>2] = 1;
   $58 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($58|0);
  }
 } while(0);
 $56 = HEAP32[$nIndex>>2]|0;
 $57 = HEAP32[$2>>2]|0;
 HEAP32[$57>>2] = $56;
 HEAP32[$0>>2] = 0;
 $58 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _sqlite3Fts3NextToken($zStr,$pn) {
 $zStr = $zStr|0;
 $pn = $pn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $z1 = sp + 4|0;
 $z2 = sp;
 $c = sp + 20|0;
 HEAP32[$1>>2] = $zStr;
 HEAP32[$2>>2] = $pn;
 HEAP32[$z2>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$z1>>2] = $3;
 L1: while(1) {
  $4 = HEAP32[$z2>>2]|0;
  $5 = ($4|0)==(0|0);
  if (!($5)) {
   label = 21;
   break;
  }
  $6 = HEAP32[$z1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  HEAP8[$c>>0] = $7;
  $8 = HEAP8[$c>>0]|0;
  $9 = $8 << 24 >> 24;
  switch ($9|0) {
  case 0:  {
   label = 4;
   break L1;
   break;
  }
  case 96: case 34: case 39:  {
   $10 = HEAP32[$z1>>2]|0;
   HEAP32[$z2>>2] = $10;
   while(1) {
    $11 = HEAP32[$z2>>2]|0;
    $12 = ((($11)) + 1|0);
    HEAP32[$z2>>2] = $12;
    $13 = HEAP8[$12>>0]|0;
    $14 = $13 << 24 >> 24;
    $15 = ($14|0)!=(0);
    if (!($15)) {
     continue L1;
    }
    $16 = HEAP32[$z2>>2]|0;
    $17 = HEAP8[$16>>0]|0;
    $18 = $17 << 24 >> 24;
    $19 = HEAP8[$c>>0]|0;
    $20 = $19 << 24 >> 24;
    $21 = ($18|0)!=($20|0);
    if ($21) {
     continue;
    }
    $22 = HEAP32[$z2>>2]|0;
    $23 = ((($22)) + 1|0);
    HEAP32[$z2>>2] = $23;
    $24 = HEAP8[$23>>0]|0;
    $25 = $24 << 24 >> 24;
    $26 = HEAP8[$c>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($25|0)==($27|0);
    if (!($28)) {
     continue L1;
    }
   }
   break;
  }
  case 91:  {
   $29 = HEAP32[$z1>>2]|0;
   $30 = ((($29)) + 1|0);
   HEAP32[$z2>>2] = $30;
   while(1) {
    $31 = HEAP32[$z2>>2]|0;
    $32 = HEAP8[$31>>0]|0;
    $33 = $32 << 24 >> 24;
    $34 = ($33|0)!=(0);
    if ($34) {
     $35 = HEAP32[$z2>>2]|0;
     $36 = HEAP8[$35>>0]|0;
     $37 = $36 << 24 >> 24;
     $38 = ($37|0)!=(93);
     $65 = $38;
    } else {
     $65 = 0;
    }
    $39 = HEAP32[$z2>>2]|0;
    if (!($65)) {
     break;
    }
    $40 = ((($39)) + 1|0);
    HEAP32[$z2>>2] = $40;
   }
   $41 = HEAP8[$39>>0]|0;
   $42 = ($41<<24>>24)!=(0);
   if (!($42)) {
    continue L1;
   }
   $43 = HEAP32[$z2>>2]|0;
   $44 = ((($43)) + 1|0);
   HEAP32[$z2>>2] = $44;
   continue L1;
   break;
  }
  default: {
   $45 = HEAP32[$z1>>2]|0;
   $46 = HEAP8[$45>>0]|0;
   $47 = (_sqlite3Fts3IsIdChar($46)|0);
   $48 = ($47|0)!=(0);
   $49 = HEAP32[$z1>>2]|0;
   $50 = ((($49)) + 1|0);
   if (!($48)) {
    HEAP32[$z1>>2] = $50;
    continue L1;
   }
   HEAP32[$z2>>2] = $50;
   while(1) {
    $51 = HEAP32[$z2>>2]|0;
    $52 = HEAP8[$51>>0]|0;
    $53 = (_sqlite3Fts3IsIdChar($52)|0);
    $54 = ($53|0)!=(0);
    if (!($54)) {
     continue L1;
    }
    $55 = HEAP32[$z2>>2]|0;
    $56 = ((($55)) + 1|0);
    HEAP32[$z2>>2] = $56;
   }
  }
  }
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 else if ((label|0) == 21) {
  $57 = HEAP32[$z2>>2]|0;
  $58 = HEAP32[$z1>>2]|0;
  $59 = $57;
  $60 = $58;
  $61 = (($59) - ($60))|0;
  $62 = HEAP32[$2>>2]|0;
  HEAP32[$62>>2] = $61;
  $63 = HEAP32[$z1>>2]|0;
  HEAP32[$0>>2] = $63;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _fts3ReadExprList($p,$zFunc,$pRc) {
 $p = $p|0;
 $zFunc = $zFunc|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $zFree = 0, $zFunction = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer16 = sp + 56|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer10 = sp + 40|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer5 = sp + 24|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $zRet = sp + 80|0;
 $zFree = sp + 76|0;
 $zFunction = sp + 72|0;
 $i = sp + 68|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $pRc;
 HEAP32[$zRet>>2] = 0;
 HEAP32[$zFree>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = (_fts3QuoteId($9)|0);
   HEAP32[$zFunction>>2] = $10;
   HEAP32[$zFree>>2] = $10;
  } else {
   HEAP32[$zFunction>>2] = 18402;
  }
  $11 = HEAP32[$2>>2]|0;
  _fts3Appendf($11,$zRet,24710,$vararg_buffer);
  HEAP32[$i>>2] = 0;
  while(1) {
   $12 = HEAP32[$i>>2]|0;
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 24|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($12|0)<($15|0);
   if (!($16)) {
    break;
   }
   $17 = HEAP32[$2>>2]|0;
   $18 = HEAP32[$zFunction>>2]|0;
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$i>>2]|0;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 28|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + ($20<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $18;
   $vararg_ptr3 = ((($vararg_buffer1)) + 4|0);
   HEAP32[$vararg_ptr3>>2] = $19;
   $vararg_ptr4 = ((($vararg_buffer1)) + 8|0);
   HEAP32[$vararg_ptr4>>2] = $25;
   _fts3Appendf($17,$zRet,24716,$vararg_buffer1);
   $26 = HEAP32[$i>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$i>>2] = $27;
  }
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 44|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer5>>2] = 24738;
   _fts3Appendf($32,$zRet,24731,$vararg_buffer5);
  }
  $33 = HEAP32[$zFree>>2]|0;
  _sqlite3_free($33);
 } else {
  $34 = HEAP32[$2>>2]|0;
  _fts3Appendf($34,$zRet,24745,$vararg_buffer8);
  HEAP32[$i>>2] = 0;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$0>>2]|0;
   $37 = ((($36)) + 24|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ($35|0)<($38|0);
   if (!($39)) {
    break;
   }
   $40 = HEAP32[$2>>2]|0;
   $41 = HEAP32[$i>>2]|0;
   $42 = HEAP32[$0>>2]|0;
   $43 = ((($42)) + 28|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + ($41<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$vararg_buffer10>>2] = $46;
   _fts3Appendf($40,$zRet,24751,$vararg_buffer10);
   $47 = HEAP32[$i>>2]|0;
   $48 = (($47) + 1)|0;
   HEAP32[$i>>2] = $48;
  }
  $49 = HEAP32[$0>>2]|0;
  $50 = ((($49)) + 44|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = ($51|0)!=(0|0);
  if ($52) {
   $53 = HEAP32[$2>>2]|0;
   $54 = HEAP32[$0>>2]|0;
   $55 = ((($54)) + 44|0);
   $56 = HEAP32[$55>>2]|0;
   HEAP32[$vararg_buffer13>>2] = $56;
   _fts3Appendf($53,$zRet,24731,$vararg_buffer13);
  }
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 40|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)!=(0|0);
 $65 = HEAP32[$0>>2]|0;
 if ($64) {
  $66 = ((($65)) + 40|0);
  $67 = HEAP32[$66>>2]|0;
  $75 = $67;
  $70 = HEAP32[$0>>2]|0;
  $71 = ((($70)) + 40|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($72|0)!=(0|0);
  $74 = $73 ? 18402 : 24760;
  HEAP32[$vararg_buffer16>>2] = $60;
  $vararg_ptr19 = ((($vararg_buffer16)) + 4|0);
  HEAP32[$vararg_ptr19>>2] = $75;
  $vararg_ptr20 = ((($vararg_buffer16)) + 8|0);
  HEAP32[$vararg_ptr20>>2] = $74;
  _fts3Appendf($57,$zRet,24769,$vararg_buffer16);
  $76 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($76|0);
 } else {
  $68 = ((($65)) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  $75 = $69;
  $70 = HEAP32[$0>>2]|0;
  $71 = ((($70)) + 40|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = ($72|0)!=(0|0);
  $74 = $73 ? 18402 : 24760;
  HEAP32[$vararg_buffer16>>2] = $60;
  $vararg_ptr19 = ((($vararg_buffer16)) + 4|0);
  HEAP32[$vararg_ptr19>>2] = $75;
  $vararg_ptr20 = ((($vararg_buffer16)) + 8|0);
  HEAP32[$vararg_ptr20>>2] = $74;
  _fts3Appendf($57,$zRet,24769,$vararg_buffer16);
  $76 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 return (0)|0;
}
function _fts3WriteExprList($p,$zFunc,$pRc) {
 $p = $p|0;
 $zFunc = $zFunc|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $zFree = 0, $zFunction = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $zRet = sp + 32|0;
 $zFree = sp + 28|0;
 $zFunction = sp + 24|0;
 $i = sp + 20|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $pRc;
 HEAP32[$zRet>>2] = 0;
 HEAP32[$zFree>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (_fts3QuoteId($5)|0);
  HEAP32[$zFunction>>2] = $6;
  HEAP32[$zFree>>2] = $6;
 } else {
  HEAP32[$zFunction>>2] = 18402;
 }
 $7 = HEAP32[$2>>2]|0;
 _fts3Appendf($7,$zRet,20275,$vararg_buffer);
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$zFunction>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $14;
  _fts3Appendf($13,$zRet,24694,$vararg_buffer1);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  $22 = HEAP32[$zFree>>2]|0;
  _sqlite3_free($22);
  $23 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $21 = HEAP32[$2>>2]|0;
 _fts3Appendf($21,$zRet,24701,$vararg_buffer3);
 $22 = HEAP32[$zFree>>2]|0;
 _sqlite3_free($22);
 $23 = HEAP32[$zRet>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _fts3CreateTables($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $db = 0, $i = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer15 = 0, $vararg_buffer19 = 0, $vararg_buffer2 = 0, $vararg_buffer23 = 0, $vararg_buffer6 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr18 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $z = 0;
 var $zContentCols = 0, $zLanguageid = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer23 = sp + 64|0;
 $vararg_buffer19 = sp + 56|0;
 $vararg_buffer15 = sp + 48|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 96|0;
 $rc = sp + 92|0;
 $i = sp + 88|0;
 $db = sp + 84|0;
 $zLanguageid = sp + 80|0;
 $zContentCols = sp + 76|0;
 $z = sp + 72|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$zLanguageid>>2] = $10;
  $11 = (_sqlite3_mprintf(24319,$vararg_buffer)|0);
  HEAP32[$zContentCols>>2] = $11;
  HEAP32[$i>>2] = 0;
  while(1) {
   $12 = HEAP32[$zContentCols>>2]|0;
   $13 = ($12|0)!=(0|0);
   if (!($13)) {
    break;
   }
   $14 = HEAP32[$i>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 24|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ($14|0)<($17|0);
   if (!($18)) {
    break;
   }
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 28|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (($22) + ($19<<2)|0);
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$z>>2] = $24;
   $25 = HEAP32[$zContentCols>>2]|0;
   $26 = HEAP32[$i>>2]|0;
   $27 = HEAP32[$z>>2]|0;
   HEAP32[$vararg_buffer2>>2] = $25;
   $vararg_ptr4 = ((($vararg_buffer2)) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $26;
   $vararg_ptr5 = ((($vararg_buffer2)) + 8|0);
   HEAP32[$vararg_ptr5>>2] = $27;
   $28 = (_sqlite3_mprintf(24345,$vararg_buffer2)|0);
   HEAP32[$zContentCols>>2] = $28;
   $29 = HEAP32[$i>>2]|0;
   $30 = (($29) + 1)|0;
   HEAP32[$i>>2] = $30;
  }
  $31 = HEAP32[$zLanguageid>>2]|0;
  $32 = ($31|0)!=(0|0);
  $33 = HEAP32[$zContentCols>>2]|0;
  $34 = ($33|0)!=(0|0);
  $or$cond = $32 & $34;
  if ($or$cond) {
   $35 = HEAP32[$zContentCols>>2]|0;
   $36 = HEAP32[$zLanguageid>>2]|0;
   HEAP32[$vararg_buffer6>>2] = $35;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $36;
   $37 = (_sqlite3_mprintf(24357,$vararg_buffer6)|0);
   HEAP32[$zContentCols>>2] = $37;
  }
  $38 = HEAP32[$zContentCols>>2]|0;
  $39 = ($38|0)==(0|0);
  if ($39) {
   HEAP32[$rc>>2] = 7;
  }
  $40 = HEAP32[$db>>2]|0;
  $41 = HEAP32[$0>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 20|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$zContentCols>>2]|0;
  HEAP32[$vararg_buffer10>>2] = $43;
  $vararg_ptr13 = ((($vararg_buffer10)) + 4|0);
  HEAP32[$vararg_ptr13>>2] = $46;
  $vararg_ptr14 = ((($vararg_buffer10)) + 8|0);
  HEAP32[$vararg_ptr14>>2] = $47;
  _fts3DbExec($rc,$40,24368,$vararg_buffer10);
  $48 = HEAP32[$zContentCols>>2]|0;
  _sqlite3_free($48);
 }
 $49 = HEAP32[$db>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 16|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 20|0);
 $55 = HEAP32[$54>>2]|0;
 HEAP32[$vararg_buffer15>>2] = $52;
 $vararg_ptr18 = ((($vararg_buffer15)) + 4|0);
 HEAP32[$vararg_ptr18>>2] = $55;
 _fts3DbExec($rc,$49,24401,$vararg_buffer15);
 $56 = HEAP32[$db>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 20|0);
 $62 = HEAP32[$61>>2]|0;
 HEAP32[$vararg_buffer19>>2] = $59;
 $vararg_ptr22 = ((($vararg_buffer19)) + 4|0);
 HEAP32[$vararg_ptr22>>2] = $62;
 _fts3DbExec($rc,$56,24473,$vararg_buffer19);
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 230|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = ($65<<24>>24)!=(0);
 if ($66) {
  $67 = HEAP32[$db>>2]|0;
  $68 = HEAP32[$0>>2]|0;
  $69 = ((($68)) + 16|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = HEAP32[$0>>2]|0;
  $72 = ((($71)) + 20|0);
  $73 = HEAP32[$72>>2]|0;
  HEAP32[$vararg_buffer23>>2] = $70;
  $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
  HEAP32[$vararg_ptr26>>2] = $73;
  _fts3DbExec($rc,$67,24626,$vararg_buffer23);
 }
 $74 = HEAP32[$0>>2]|0;
 $75 = ((($74)) + 229|0);
 $76 = HEAP8[$75>>0]|0;
 $77 = ($76<<24>>24)!=(0);
 if (!($77)) {
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $78 = HEAP32[$0>>2]|0;
 _sqlite3Fts3CreateStatTable($rc,$78);
 $79 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _fts3DatabasePageSize($pRc,$p) {
 $pRc = $pRc|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, $vararg_buffer = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $zSql = sp + 8|0;
 $pStmt = sp + 4|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$vararg_buffer>>2] = $7;
 $8 = (_sqlite3_mprintf(24299,$vararg_buffer)|0);
 HEAP32[$zSql>>2] = $8;
 $9 = HEAP32[$zSql>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$zSql>>2]|0;
   $15 = (_sqlite3_prepare($13,$14,-1,$pStmt,0)|0);
   HEAP32[$rc>>2] = $15;
   $16 = HEAP32[$rc>>2]|0;
   $17 = ($16|0)==(0);
   if ($17) {
    $18 = HEAP32[$pStmt>>2]|0;
    (_sqlite3_step($18)|0);
    $19 = HEAP32[$pStmt>>2]|0;
    $20 = (_sqlite3_column_int($19,0)|0);
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 236|0);
    HEAP32[$22>>2] = $20;
    $23 = HEAP32[$pStmt>>2]|0;
    $24 = (_sqlite3_finalize($23)|0);
    HEAP32[$rc>>2] = $24;
    break;
   }
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(23);
   if ($26) {
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 236|0);
    HEAP32[$28>>2] = 1024;
    HEAP32[$rc>>2] = 0;
   }
  } else {
   HEAP32[$rc>>2] = 7;
  }
 } while(0);
 $29 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($29);
 $30 = HEAP32[$rc>>2]|0;
 $31 = HEAP32[$0>>2]|0;
 HEAP32[$31>>2] = $30;
 STACKTOP = sp;return;
}
function _fts3DeclareVtab($pRc,$p) {
 $pRc = $pRc|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $or$cond = 0, $rc = 0;
 var $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer9 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr8 = 0, $zCols = 0, $zLanguageid = 0, $zSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer9 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $i = sp + 52|0;
 $rc = sp + 48|0;
 $zSql = sp + 44|0;
 $zCols = sp + 40|0;
 $zLanguageid = sp + 36|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = 24224;
 }
 HEAP32[$zLanguageid>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$vararg_buffer>>2] = 1;
 (_sqlite3_vtab_config($15,1,$vararg_buffer)|0);
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 28|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $19;
 $20 = (_sqlite3_mprintf(24233,$vararg_buffer2)|0);
 HEAP32[$zCols>>2] = $20;
 HEAP32[$i>>2] = 1;
 while(1) {
  $21 = HEAP32[$zCols>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$i>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 24|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($23|0)<($26|0);
   $56 = $27;
  } else {
   $56 = 0;
  }
  $28 = HEAP32[$zCols>>2]|0;
  if (!($56)) {
   break;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 28|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29<<2)|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$vararg_buffer5>>2] = $28;
  $vararg_ptr8 = ((($vararg_buffer5)) + 4|0);
  HEAP32[$vararg_ptr8>>2] = $34;
  $35 = (_sqlite3_mprintf(24238,$vararg_buffer5)|0);
  HEAP32[$zCols>>2] = $35;
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 20|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$zLanguageid>>2]|0;
 HEAP32[$vararg_buffer9>>2] = $28;
 $vararg_ptr12 = ((($vararg_buffer9)) + 4|0);
 HEAP32[$vararg_ptr12>>2] = $40;
 $vararg_ptr13 = ((($vararg_buffer9)) + 8|0);
 HEAP32[$vararg_ptr13>>2] = $41;
 $42 = (_sqlite3_mprintf(24245,$vararg_buffer9)|0);
 HEAP32[$zSql>>2] = $42;
 $43 = HEAP32[$zCols>>2]|0;
 $44 = ($43|0)!=(0|0);
 $45 = HEAP32[$zSql>>2]|0;
 $46 = ($45|0)!=(0|0);
 $or$cond = $44 & $46;
 if ($or$cond) {
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$zSql>>2]|0;
  $51 = (_sqlite3_declare_vtab($49,$50)|0);
  HEAP32[$rc>>2] = $51;
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $52 = HEAP32[$zSql>>2]|0;
 _sqlite3_free($52);
 $53 = HEAP32[$zCols>>2]|0;
 _sqlite3_free($53);
 $54 = HEAP32[$rc>>2]|0;
 $55 = HEAP32[$0>>2]|0;
 HEAP32[$55>>2] = $54;
 STACKTOP = sp;return;
}
function _fts3QuoteId($zInput) {
 $zInput = $zInput|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $nRet = 0, $z = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $nRet = sp + 12|0;
 $zRet = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $zInput;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_strlen($1)|0);
 $3 = $2<<1;
 $4 = (2 + ($3))|0;
 $5 = (($4) + 1)|0;
 HEAP32[$nRet>>2] = $5;
 $6 = HEAP32[$nRet>>2]|0;
 $7 = (_sqlite3_malloc($6)|0);
 HEAP32[$zRet>>2] = $7;
 $8 = HEAP32[$zRet>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $38 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $10 = HEAP32[$zRet>>2]|0;
 HEAP32[$z>>2] = $10;
 $11 = HEAP32[$z>>2]|0;
 $12 = ((($11)) + 1|0);
 HEAP32[$z>>2] = $12;
 HEAP8[$11>>0] = 34;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = (($14) + ($13)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = (($19) + ($18)|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21 << 24 >> 24;
  $23 = ($22|0)==(34);
  if ($23) {
   $24 = HEAP32[$z>>2]|0;
   $25 = ((($24)) + 1|0);
   HEAP32[$z>>2] = $25;
   HEAP8[$24>>0] = 34;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = (($27) + ($26)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = HEAP32[$z>>2]|0;
  $31 = ((($30)) + 1|0);
  HEAP32[$z>>2] = $31;
  HEAP8[$30>>0] = $29;
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 $34 = HEAP32[$z>>2]|0;
 $35 = ((($34)) + 1|0);
 HEAP32[$z>>2] = $35;
 HEAP8[$34>>0] = 34;
 $36 = HEAP32[$z>>2]|0;
 $37 = ((($36)) + 1|0);
 HEAP32[$z>>2] = $37;
 HEAP8[$36>>0] = 0;
 $38 = HEAP32[$zRet>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _fts3Appendf($pRc,$pz,$zFormat,$varargs) {
 $pRc = $pRc|0;
 $pz = $pz|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $z = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $ap = sp + 16|0;
 $z = sp + 12|0;
 $z2 = sp + 8|0;
 HEAP32[$0>>2] = $pRc;
 HEAP32[$1>>2] = $pz;
 HEAP32[$2>>2] = $zFormat;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3_vmprintf($6,$ap)|0);
 HEAP32[$z>>2] = $7;
 $8 = HEAP32[$z>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$z>>2]|0;
   HEAP32[$vararg_buffer>>2] = $14;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $15;
   $16 = (_sqlite3_mprintf(24705,$vararg_buffer)|0);
   HEAP32[$z2>>2] = $16;
   $17 = HEAP32[$z>>2]|0;
   _sqlite3_free($17);
   $18 = HEAP32[$z2>>2]|0;
   HEAP32[$z>>2] = $18;
  }
 }
 $19 = HEAP32[$z>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  HEAP32[$21>>2] = 7;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 _sqlite3_free($23);
 $24 = HEAP32[$z>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 HEAP32[$25>>2] = $24;
 STACKTOP = sp;return;
}
function _fts3GobbleInt($pp,$pnOut) {
 $pp = $pp|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $MAX_NPREFIX = 0, $nInt = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $MAX_NPREFIX = sp + 8|0;
 $p = sp + 4|0;
 $nInt = sp;
 HEAP32[$1>>2] = $pp;
 HEAP32[$2>>2] = $pnOut;
 HEAP32[$MAX_NPREFIX>>2] = 10000000;
 HEAP32[$nInt>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)>=(48);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  $12 = ($11|0)<=(57);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$nInt>>2]|0;
  $14 = ($13*10)|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = (($17) - 48)|0;
  $19 = (($14) + ($18))|0;
  HEAP32[$nInt>>2] = $19;
  $20 = HEAP32[$nInt>>2]|0;
  $21 = ($20|0)>(10000000);
  if ($21) {
   label = 5;
   break;
  }
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 1|0);
  HEAP32[$p>>2] = $23;
 }
 if ((label|0) == 5) {
  HEAP32[$nInt>>2] = 0;
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = ($24|0)==($26|0);
 if ($27) {
  HEAP32[$0>>2] = 1;
  $32 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($32|0);
 } else {
  $28 = HEAP32[$nInt>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  HEAP32[$29>>2] = $28;
  $30 = HEAP32[$p>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  HEAP32[$31>>2] = $30;
  HEAP32[$0>>2] = 0;
  $32 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 return (0)|0;
}
function _fts3TokenizerFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $nName = 0, $or$cond = 0, $pHash = 0, $pOld = 0;
 var $pPtr = 0, $vararg_buffer = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pHash = sp + 28|0;
 $pPtr = sp + 24|0;
 $zName = sp + 20|0;
 $nName = sp + 16|0;
 $pOld = sp + 12|0;
 $n = sp + 8|0;
 $zErr = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$pPtr>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_user_data($3)|0);
 HEAP32[$pHash>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zName>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3_value_bytes($9)|0);
 $11 = (($10) + 1)|0;
 HEAP32[$nName>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)==(2);
 do {
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = (_fts3TokenizerEnabled($14)|0);
   $16 = ($15|0)!=(0);
   if (!($16)) {
    $40 = HEAP32[$0>>2]|0;
    _sqlite3_result_error($40,25030,-1);
    STACKTOP = sp;return;
   }
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3_value_bytes($19)|0);
   HEAP32[$n>>2] = $20;
   $21 = HEAP32[$zName>>2]|0;
   $22 = ($21|0)==(0|0);
   $23 = HEAP32[$n>>2]|0;
   $24 = ($23|0)!=(4);
   $or$cond = $22 | $24;
   if ($or$cond) {
    $25 = HEAP32[$0>>2]|0;
    _sqlite3_result_error($25,24993,-1);
    STACKTOP = sp;return;
   } else {
    $26 = HEAP32[$2>>2]|0;
    $27 = ((($26)) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (_sqlite3_value_blob($28)|0);
    $30 = HEAP32[$29>>2]|0;
    HEAP32[$pPtr>>2] = $30;
    $31 = HEAP32[$pHash>>2]|0;
    $32 = HEAP32[$zName>>2]|0;
    $33 = HEAP32[$nName>>2]|0;
    $34 = HEAP32[$pPtr>>2]|0;
    $35 = (_sqlite3Fts3HashInsert($31,$32,$33,$34)|0);
    HEAP32[$pOld>>2] = $35;
    $36 = HEAP32[$pOld>>2]|0;
    $37 = HEAP32[$pPtr>>2]|0;
    $38 = ($36|0)==($37|0);
    if (!($38)) {
     break;
    }
    $39 = HEAP32[$0>>2]|0;
    _sqlite3_result_error($39,25016,-1);
    break;
   }
  } else {
   $41 = HEAP32[$zName>>2]|0;
   $42 = ($41|0)!=(0|0);
   if ($42) {
    $43 = HEAP32[$pHash>>2]|0;
    $44 = HEAP32[$zName>>2]|0;
    $45 = HEAP32[$nName>>2]|0;
    $46 = (_sqlite3Fts3HashFind($43,$44,$45)|0);
    HEAP32[$pPtr>>2] = $46;
   }
   $47 = HEAP32[$pPtr>>2]|0;
   $48 = ($47|0)!=(0|0);
   if (!($48)) {
    $49 = HEAP32[$zName>>2]|0;
    HEAP32[$vararg_buffer>>2] = $49;
    $50 = (_sqlite3_mprintf(20595,$vararg_buffer)|0);
    HEAP32[$zErr>>2] = $50;
    $51 = HEAP32[$0>>2]|0;
    $52 = HEAP32[$zErr>>2]|0;
    _sqlite3_result_error($51,$52,-1);
    $53 = HEAP32[$zErr>>2]|0;
    _sqlite3_free($53);
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $54 = HEAP32[$0>>2]|0;
 _sqlite3_result_blob($54,$pPtr,4,(-1));
 STACKTOP = sp;return;
}
function _fts3TokenizerEnabled($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $db = 0, $isEnabled = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $db = sp + 12|0;
 $isEnabled = sp + 8|0;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_context_db_handle($1)|0);
 HEAP32[$db>>2] = $2;
 HEAP32[$isEnabled>>2] = 0;
 $3 = HEAP32[$db>>2]|0;
 HEAP32[$vararg_buffer>>2] = -1;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $isEnabled;
 (_sqlite3_db_config($3,1004,$vararg_buffer)|0);
 $4 = HEAP32[$isEnabled>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _fts3RemoveElementByHash($pH,$elem,$h) {
 $pH = $pH|0;
 $elem = $elem|0;
 $h = $h|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $pEntry = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pEntry = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $elem;
 HEAP32[$2>>2] = $h;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 if ($6) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$11>>2] = $8;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  HEAP32[$13>>2] = $8;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  HEAP32[$22>>2] = $19;
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 16|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($23<<3)|0);
 HEAP32[$pEntry>>2] = $27;
 $28 = HEAP32[$pEntry>>2]|0;
 $29 = ((($28)) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$1>>2]|0;
 $32 = ($30|0)==($31|0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$pEntry>>2]|0;
  $36 = ((($35)) + 4|0);
  HEAP32[$36>>2] = $34;
 }
 $37 = HEAP32[$pEntry>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + -1)|0;
 HEAP32[$37>>2] = $39;
 $40 = HEAP32[$pEntry>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)<=(0);
 if ($42) {
  $43 = HEAP32[$pEntry>>2]|0;
  $44 = ((($43)) + 4|0);
  HEAP32[$44>>2] = 0;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 1|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47 << 24 >> 24;
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 12|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = ($52|0)!=(0|0);
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   _fts3HashFree($56);
  }
 }
 $57 = HEAP32[$1>>2]|0;
 _fts3HashFree($57);
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 4|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + -1)|0;
 HEAP32[$59>>2] = $61;
 $62 = HEAP32[$0>>2]|0;
 $63 = ((($62)) + 4|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = ($64|0)<=(0);
 if (!($65)) {
  STACKTOP = sp;return;
 }
 $66 = HEAP32[$0>>2]|0;
 _sqlite3Fts3HashClear($66);
 STACKTOP = sp;return;
}
function _fts3Rehash($pH,$new_size) {
 $pH = $pH|0;
 $new_size = $new_size|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $elem = 0, $h = 0, $new_ht = 0, $next_elem = 0, $xHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $new_ht = sp + 16|0;
 $elem = sp + 12|0;
 $next_elem = sp + 8|0;
 $xHash = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pH;
 HEAP32[$2>>2] = $new_size;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<3;
 $5 = (_fts3HashMalloc($4)|0);
 HEAP32[$new_ht>>2] = $5;
 $6 = HEAP32[$new_ht>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 1;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 _fts3HashFree($10);
 $11 = HEAP32[$new_ht>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 16|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18 << 24 >> 24;
 $20 = (_ftsHashFunction($19)|0);
 HEAP32[$xHash>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$elem>>2] = $23;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 8|0);
 HEAP32[$25>>2] = 0;
 while(1) {
  $26 = HEAP32[$elem>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$xHash>>2]|0;
  $29 = HEAP32[$elem>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$elem>>2]|0;
  $33 = ((($32)) + 16|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (FUNCTION_TABLE_iii[$28 & 255]($31,$34)|0);
  $36 = HEAP32[$2>>2]|0;
  $37 = (($36) - 1)|0;
  $38 = $35 & $37;
  HEAP32[$h>>2] = $38;
  $39 = HEAP32[$elem>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$next_elem>>2] = $40;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$h>>2]|0;
  $43 = HEAP32[$new_ht>>2]|0;
  $44 = (($43) + ($42<<3)|0);
  $45 = HEAP32[$elem>>2]|0;
  _fts3HashInsertElement($41,$44,$45);
  $46 = HEAP32[$next_elem>>2]|0;
  HEAP32[$elem>>2] = $46;
 }
 HEAP32[$0>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _fts3HashMalloc($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_malloc($1)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $7 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 _memset(($5|0),0,($6|0))|0;
 $7 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _fts3HashInsertElement($pH,$pEntry,$pNew) {
 $pH = $pH|0;
 $pEntry = $pEntry|0;
 $pNew = $pNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pHead = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pHead = sp;
 HEAP32[$0>>2] = $pH;
 HEAP32[$1>>2] = $pEntry;
 HEAP32[$2>>2] = $pNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pHead>>2] = $5;
 $6 = HEAP32[$pHead>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 8|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  HEAP32[$31>>2] = $30;
  $32 = HEAP32[$0>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)!=(0|0);
  if ($35) {
   $36 = HEAP32[$2>>2]|0;
   $37 = HEAP32[$0>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ((($39)) + 4|0);
   HEAP32[$40>>2] = $36;
  }
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 4|0);
  HEAP32[$42>>2] = 0;
  $43 = HEAP32[$2>>2]|0;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  HEAP32[$45>>2] = $43;
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$46>>2] = $48;
  $49 = HEAP32[$2>>2]|0;
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 4|0);
  HEAP32[$51>>2] = $49;
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$pHead>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$pHead>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$pHead>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$2>>2]|0;
 if ($18) {
  $20 = HEAP32[$pHead>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$22>>2] = $19;
 } else {
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 8|0);
  HEAP32[$24>>2] = $19;
 }
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$pHead>>2]|0;
 $27 = ((($26)) + 4|0);
 HEAP32[$27>>2] = $25;
 $46 = HEAP32[$1>>2]|0;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + 1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$2>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 4|0);
 HEAP32[$51>>2] = $49;
 STACKTOP = sp;return;
}
function _porterCreate($argc,$argv,$ppTokenizer) {
 $argc = $argc|0;
 $argv = $argv|0;
 $ppTokenizer = $ppTokenizer|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $t = sp;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$3>>2] = $ppTokenizer;
 $4 = (_sqlite3_malloc(4)|0);
 HEAP32[$t>>2] = $4;
 $5 = HEAP32[$t>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 7;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $7 = HEAP32[$t>>2]|0;
  ;HEAP32[$7>>2]=0|0;
  $8 = HEAP32[$t>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  HEAP32[$9>>2] = $8;
  HEAP32[$0>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return (0)|0;
}
function _porterDestroy($pTokenizer) {
 $pTokenizer = $pTokenizer|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTokenizer;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return 0;
}
function _porterOpen($pTokenizer,$zInput,$nInput,$ppCursor) {
 $pTokenizer = $pTokenizer|0;
 $zInput = $zInput|0;
 $nInput = $nInput|0;
 $ppCursor = $ppCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $c = sp;
 HEAP32[$1>>2] = $pTokenizer;
 HEAP32[$2>>2] = $zInput;
 HEAP32[$3>>2] = $nInput;
 HEAP32[$4>>2] = $ppCursor;
 $5 = (_sqlite3_malloc(28)|0);
 HEAP32[$c>>2] = $5;
 $6 = HEAP32[$c>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 7;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$c>>2]|0;
 $10 = ((($9)) + 4|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 do {
  if ($12) {
   $13 = HEAP32[$c>>2]|0;
   $14 = ((($13)) + 8|0);
   HEAP32[$14>>2] = 0;
  } else {
   $15 = HEAP32[$3>>2]|0;
   $16 = ($15|0)<(0);
   if ($16) {
    $17 = HEAP32[$2>>2]|0;
    $18 = (_strlen($17)|0);
    $19 = HEAP32[$c>>2]|0;
    $20 = ((($19)) + 8|0);
    HEAP32[$20>>2] = $18;
    break;
   } else {
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$c>>2]|0;
    $23 = ((($22)) + 8|0);
    HEAP32[$23>>2] = $21;
    break;
   }
  }
 } while(0);
 $24 = HEAP32[$c>>2]|0;
 $25 = ((($24)) + 12|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$c>>2]|0;
 $27 = ((($26)) + 16|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$c>>2]|0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$c>>2]|0;
 $31 = ((($30)) + 24|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$c>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 HEAP32[$33>>2] = $32;
 HEAP32[$0>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _porterClose($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$c>>2] = $1;
 $2 = HEAP32[$c>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$c>>2]|0;
 _sqlite3_free($5);
 STACKTOP = sp;return 0;
}
function _porterNext($pCursor,$pzToken,$pnBytes,$piStartOffset,$piEndOffset,$piPosition) {
 $pCursor = $pCursor|0;
 $pzToken = $pzToken|0;
 $pnBytes = $pnBytes|0;
 $piStartOffset = $piStartOffset|0;
 $piEndOffset = $piEndOffset|0;
 $piPosition = $piPosition|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $ch = 0, $iStartOffset = 0, $n = 0, $pNew = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $c = sp + 20|0;
 $z = sp + 16|0;
 $iStartOffset = sp + 12|0;
 $ch = sp + 8|0;
 $n = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $pzToken;
 HEAP32[$3>>2] = $pnBytes;
 HEAP32[$4>>2] = $piStartOffset;
 HEAP32[$5>>2] = $piEndOffset;
 HEAP32[$6>>2] = $piPosition;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$c>>2] = $7;
 $8 = HEAP32[$c>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$z>>2] = $10;
 while(1) {
  $11 = HEAP32[$c>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$c>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   label = 23;
   break;
  }
  while(1) {
   $18 = HEAP32[$c>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$c>>2]|0;
   $22 = ((($21)) + 8|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($20|0)<($23|0);
   if ($24) {
    $25 = HEAP32[$c>>2]|0;
    $26 = ((($25)) + 12|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$z>>2]|0;
    $29 = (($28) + ($27)|0);
    $30 = HEAP8[$29>>0]|0;
    $31 = $30 << 24 >> 24;
    HEAP32[$ch>>2] = $31;
    $32 = $31 & 128;
    $33 = ($32|0)==(0);
    if ($33) {
     $34 = HEAP32[$ch>>2]|0;
     $35 = ($34|0)<(48);
     if ($35) {
      $128 = 1;
     } else {
      $36 = HEAP32[$ch>>2]|0;
      $37 = (($36) - 48)|0;
      $38 = (25052 + ($37)|0);
      $39 = HEAP8[$38>>0]|0;
      $40 = ($39<<24>>24)!=(0);
      $41 = $40 ^ 1;
      $128 = $41;
     }
    } else {
     $128 = 0;
    }
   } else {
    $128 = 0;
   }
   $42 = HEAP32[$c>>2]|0;
   $43 = ((($42)) + 12|0);
   $44 = HEAP32[$43>>2]|0;
   if (!($128)) {
    break;
   }
   $45 = (($44) + 1)|0;
   HEAP32[$43>>2] = $45;
  }
  HEAP32[$iStartOffset>>2] = $44;
  while(1) {
   $46 = HEAP32[$c>>2]|0;
   $47 = ((($46)) + 12|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$c>>2]|0;
   $50 = ((($49)) + 8|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($48|0)<($51|0);
   if ($52) {
    $53 = HEAP32[$c>>2]|0;
    $54 = ((($53)) + 12|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = HEAP32[$z>>2]|0;
    $57 = (($56) + ($55)|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58 << 24 >> 24;
    HEAP32[$ch>>2] = $59;
    $60 = $59 & 128;
    $61 = ($60|0)==(0);
    if ($61) {
     $62 = HEAP32[$ch>>2]|0;
     $63 = ($62|0)<(48);
     if ($63) {
      $71 = 1;
     } else {
      $64 = HEAP32[$ch>>2]|0;
      $65 = (($64) - 48)|0;
      $66 = (25052 + ($65)|0);
      $67 = HEAP8[$66>>0]|0;
      $68 = ($67<<24>>24)!=(0);
      $69 = $68 ^ 1;
      $71 = $69;
     }
    } else {
     $71 = 0;
    }
    $70 = $71 ^ 1;
    $129 = $70;
   } else {
    $129 = 0;
   }
   $72 = HEAP32[$c>>2]|0;
   $73 = ((($72)) + 12|0);
   $74 = HEAP32[$73>>2]|0;
   if (!($129)) {
    break;
   }
   $75 = (($74) + 1)|0;
   HEAP32[$73>>2] = $75;
  }
  $76 = HEAP32[$iStartOffset>>2]|0;
  $77 = ($74|0)>($76|0);
  if ($77) {
   break;
  }
 }
 if ((label|0) == 23) {
  HEAP32[$0>>2] = 101;
  $127 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $78 = HEAP32[$c>>2]|0;
 $79 = ((($78)) + 12|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = HEAP32[$iStartOffset>>2]|0;
 $82 = (($80) - ($81))|0;
 HEAP32[$n>>2] = $82;
 $83 = HEAP32[$n>>2]|0;
 $84 = HEAP32[$c>>2]|0;
 $85 = ((($84)) + 24|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = ($83|0)>($86|0);
 do {
  if ($87) {
   $88 = HEAP32[$n>>2]|0;
   $89 = (($88) + 20)|0;
   $90 = HEAP32[$c>>2]|0;
   $91 = ((($90)) + 24|0);
   HEAP32[$91>>2] = $89;
   $92 = HEAP32[$c>>2]|0;
   $93 = ((($92)) + 20|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = HEAP32[$c>>2]|0;
   $96 = ((($95)) + 24|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = (_sqlite3_realloc($94,$97)|0);
   HEAP32[$pNew>>2] = $98;
   $99 = HEAP32[$pNew>>2]|0;
   $100 = ($99|0)!=(0|0);
   if ($100) {
    $101 = HEAP32[$pNew>>2]|0;
    $102 = HEAP32[$c>>2]|0;
    $103 = ((($102)) + 20|0);
    HEAP32[$103>>2] = $101;
    break;
   }
   HEAP32[$0>>2] = 7;
   $127 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($127|0);
  }
 } while(0);
 $104 = HEAP32[$iStartOffset>>2]|0;
 $105 = HEAP32[$z>>2]|0;
 $106 = (($105) + ($104)|0);
 $107 = HEAP32[$n>>2]|0;
 $108 = HEAP32[$c>>2]|0;
 $109 = ((($108)) + 20|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = HEAP32[$3>>2]|0;
 _porter_stemmer($106,$107,$110,$111);
 $112 = HEAP32[$c>>2]|0;
 $113 = ((($112)) + 20|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = HEAP32[$2>>2]|0;
 HEAP32[$115>>2] = $114;
 $116 = HEAP32[$iStartOffset>>2]|0;
 $117 = HEAP32[$4>>2]|0;
 HEAP32[$117>>2] = $116;
 $118 = HEAP32[$c>>2]|0;
 $119 = ((($118)) + 12|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = HEAP32[$5>>2]|0;
 HEAP32[$121>>2] = $120;
 $122 = HEAP32[$c>>2]|0;
 $123 = ((($122)) + 16|0);
 $124 = HEAP32[$123>>2]|0;
 $125 = (($124) + 1)|0;
 HEAP32[$123>>2] = $125;
 $126 = HEAP32[$6>>2]|0;
 HEAP32[$126>>2] = $124;
 HEAP32[$0>>2] = 0;
 $127 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($127|0);
}
function _porter_stemmer($zIn,$nIn,$zOut,$pnOut) {
 $zIn = $zIn|0;
 $nIn = $nIn|0;
 $zOut = $zOut|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $j = 0, $or$cond = 0, $z = 0;
 var $z2 = 0, $zReverse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $j = sp + 8|0;
 $zReverse = sp + 36|0;
 $z = sp + 4|0;
 $z2 = sp;
 $c = sp + 32|0;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $nIn;
 HEAP32[$2>>2] = $zOut;
 HEAP32[$3>>2] = $pnOut;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(3);
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)>=(21);
 $or$cond = $5 | $7;
 if ($or$cond) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  _copy_stemmer($8,$9,$10,$11);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 HEAP32[$j>>2] = 22;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + ($15)|0);
  $18 = HEAP8[$17>>0]|0;
  HEAP8[$c>>0] = $18;
  $19 = HEAP8[$c>>0]|0;
  $20 = $19 << 24 >> 24;
  $21 = ($20|0)>=(65);
  if ($21) {
   $22 = HEAP8[$c>>0]|0;
   $23 = $22 << 24 >> 24;
   $24 = ($23|0)<=(90);
   if ($24) {
    $25 = HEAP8[$c>>0]|0;
    $26 = $25 << 24 >> 24;
    $27 = (($26) + 97)|0;
    $28 = (($27) - 65)|0;
    $29 = $28&255;
    $30 = HEAP32[$j>>2]|0;
    $31 = (($zReverse) + ($30)|0);
    HEAP8[$31>>0] = $29;
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  if ((label|0) == 8) {
   label = 0;
   $32 = HEAP8[$c>>0]|0;
   $33 = $32 << 24 >> 24;
   $34 = ($33|0)>=(97);
   if (!($34)) {
    label = 11;
    break;
   }
   $35 = HEAP8[$c>>0]|0;
   $36 = $35 << 24 >> 24;
   $37 = ($36|0)<=(122);
   if (!($37)) {
    label = 11;
    break;
   }
   $38 = HEAP8[$c>>0]|0;
   $39 = HEAP32[$j>>2]|0;
   $40 = (($zReverse) + ($39)|0);
   HEAP8[$40>>0] = $38;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
  $47 = HEAP32[$j>>2]|0;
  $48 = (($47) + -1)|0;
  HEAP32[$j>>2] = $48;
 }
 if ((label|0) == 11) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  _copy_stemmer($41,$42,$43,$44);
  STACKTOP = sp;return;
 }
 $49 = ((($zReverse)) + 23|0);
 ;HEAP8[$49>>0]=0|0;HEAP8[$49+1>>0]=0|0;HEAP8[$49+2>>0]=0|0;HEAP8[$49+3>>0]=0|0;HEAP8[$49+4>>0]=0|0;
 $50 = HEAP32[$j>>2]|0;
 $51 = (($50) + 1)|0;
 $52 = (($zReverse) + ($51)|0);
 HEAP32[$z>>2] = $52;
 $53 = HEAP32[$z>>2]|0;
 $54 = HEAP8[$53>>0]|0;
 $55 = $54 << 24 >> 24;
 $56 = ($55|0)==(115);
 if ($56) {
  $57 = (_stem($z,25132,25137,0)|0);
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = (_stem($z,25140,25144,0)|0);
   $60 = ($59|0)!=(0);
   if (!($60)) {
    $61 = (_stem($z,25137,25137,0)|0);
    $62 = ($61|0)!=(0);
    if (!($62)) {
     $63 = HEAP32[$z>>2]|0;
     $64 = ((($63)) + 1|0);
     HEAP32[$z>>2] = $64;
    }
   }
  }
 }
 $65 = HEAP32[$z>>2]|0;
 HEAP32[$z2>>2] = $65;
 $66 = (_stem($z,25146,25150,172)|0);
 $67 = ($66|0)!=(0);
 do {
  if (!($67)) {
   $68 = (_stem($z,25153,18402,173)|0);
   $69 = ($68|0)!=(0);
   if (!($69)) {
    $70 = (_stem($z,25157,18402,173)|0);
    $71 = ($70|0)!=(0);
    if (!($71)) {
     break;
    }
   }
   $72 = HEAP32[$z>>2]|0;
   $73 = HEAP32[$z2>>2]|0;
   $74 = ($72|0)!=($73|0);
   if ($74) {
    $75 = (_stem($z,25160,25163,0)|0);
    $76 = ($75|0)!=(0);
    if (!($76)) {
     $77 = (_stem($z,25167,25170,0)|0);
     $78 = ($77|0)!=(0);
     if (!($78)) {
      $79 = (_stem($z,25174,25177,0)|0);
      $80 = ($79|0)!=(0);
      if (!($80)) {
       $81 = HEAP32[$z>>2]|0;
       $82 = (_doubleConsonant($81)|0);
       $83 = ($82|0)!=(0);
       if ($83) {
        $84 = HEAP32[$z>>2]|0;
        $85 = HEAP8[$84>>0]|0;
        $86 = $85 << 24 >> 24;
        $87 = ($86|0)!=(108);
        if ($87) {
         $88 = HEAP32[$z>>2]|0;
         $89 = HEAP8[$88>>0]|0;
         $90 = $89 << 24 >> 24;
         $91 = ($90|0)!=(115);
         if ($91) {
          $92 = HEAP32[$z>>2]|0;
          $93 = HEAP8[$92>>0]|0;
          $94 = $93 << 24 >> 24;
          $95 = ($94|0)!=(122);
          if ($95) {
           $96 = HEAP32[$z>>2]|0;
           $97 = ((($96)) + 1|0);
           HEAP32[$z>>2] = $97;
           break;
          }
         }
        }
       }
       $98 = HEAP32[$z>>2]|0;
       $99 = (_m_eq_1($98)|0);
       $100 = ($99|0)!=(0);
       if ($100) {
        $101 = HEAP32[$z>>2]|0;
        $102 = (_star_oh($101)|0);
        $103 = ($102|0)!=(0);
        if ($103) {
         $104 = HEAP32[$z>>2]|0;
         $105 = ((($104)) + -1|0);
         HEAP32[$z>>2] = $105;
         HEAP8[$105>>0] = 101;
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $106 = HEAP32[$z>>2]|0;
 $107 = HEAP8[$106>>0]|0;
 $108 = $107 << 24 >> 24;
 $109 = ($108|0)==(121);
 if ($109) {
  $110 = HEAP32[$z>>2]|0;
  $111 = ((($110)) + 1|0);
  $112 = (_hasVowel($111)|0);
  $113 = ($112|0)!=(0);
  if ($113) {
   $114 = HEAP32[$z>>2]|0;
   HEAP8[$114>>0] = 105;
  }
 }
 $115 = HEAP32[$z>>2]|0;
 $116 = ((($115)) + 1|0);
 $117 = HEAP8[$116>>0]|0;
 $118 = $117 << 24 >> 24;
 switch ($118|0) {
 case 97:  {
  $119 = (_stem($z,25181,25163,172)|0);
  $120 = ($119|0)!=(0);
  if (!($120)) {
   (_stem($z,25189,25196,172)|0);
  }
  break;
 }
 case 99:  {
  $121 = (_stem($z,25201,25206,172)|0);
  $122 = ($121|0)!=(0);
  if (!($122)) {
   (_stem($z,25211,25216,172)|0);
  }
  break;
 }
 case 101:  {
  (_stem($z,25221,25177,172)|0);
  break;
 }
 case 103:  {
  (_stem($z,25226,25231,172)|0);
  break;
 }
 case 108:  {
  $123 = (_stem($z,25235,25170,172)|0);
  $124 = ($123|0)!=(0);
  if (!($124)) {
   $125 = (_stem($z,25239,25244,172)|0);
   $126 = ($125|0)!=(0);
   if (!($126)) {
    $127 = (_stem($z,25247,25253,172)|0);
    $128 = ($127|0)!=(0);
    if (!($128)) {
     $129 = (_stem($z,25257,25261,172)|0);
     $130 = ($129|0)!=(0);
     if (!($130)) {
      (_stem($z,25263,25269,172)|0);
     }
    }
   }
  }
  break;
 }
 case 111:  {
  $131 = (_stem($z,25273,25177,172)|0);
  $132 = ($131|0)!=(0);
  if (!($132)) {
   $133 = (_stem($z,25281,25163,172)|0);
   $134 = ($133|0)!=(0);
   if (!($134)) {
    (_stem($z,25287,25163,172)|0);
   }
  }
  break;
 }
 case 115:  {
  $135 = (_stem($z,25292,25244,172)|0);
  $136 = ($135|0)!=(0);
  if (!($136)) {
   $137 = (_stem($z,25298,25306,172)|0);
   $138 = ($137|0)!=(0);
   if (!($138)) {
    $139 = (_stem($z,25310,25318,172)|0);
    $140 = ($139|0)!=(0);
    if (!($140)) {
     (_stem($z,25322,25269,172)|0);
    }
   }
  }
  break;
 }
 case 116:  {
  $141 = (_stem($z,25330,25244,172)|0);
  $142 = ($141|0)!=(0);
  if (!($142)) {
   $143 = (_stem($z,25336,25306,172)|0);
   $144 = ($143|0)!=(0);
   if (!($144)) {
    (_stem($z,25342,25170,172)|0);
   }
  }
  break;
 }
 default: {
 }
 }
 $145 = HEAP32[$z>>2]|0;
 $146 = HEAP8[$145>>0]|0;
 $147 = $146 << 24 >> 24;
 switch ($147|0) {
 case 101:  {
  $148 = (_stem($z,25349,25355,172)|0);
  $149 = ($148|0)!=(0);
  if (!($149)) {
   $150 = (_stem($z,25358,18402,172)|0);
   $151 = ($150|0)!=(0);
   if (!($151)) {
    (_stem($z,25364,25244,172)|0);
   }
  }
  break;
 }
 case 105:  {
  (_stem($z,25370,25355,172)|0);
  break;
 }
 case 108:  {
  $152 = (_stem($z,25376,25355,172)|0);
  $153 = ($152|0)!=(0);
  if (!($153)) {
   (_stem($z,25381,18402,172)|0);
  }
  break;
 }
 case 115:  {
  (_stem($z,25385,18402,172)|0);
  break;
 }
 default: {
 }
 }
 $154 = HEAP32[$z>>2]|0;
 $155 = ((($154)) + 1|0);
 $156 = HEAP8[$155>>0]|0;
 $157 = $156 << 24 >> 24;
 L80: do {
  switch ($157|0) {
  case 97:  {
   $158 = HEAP32[$z>>2]|0;
   $159 = HEAP8[$158>>0]|0;
   $160 = $159 << 24 >> 24;
   $161 = ($160|0)==(108);
   if ($161) {
    $162 = HEAP32[$z>>2]|0;
    $163 = ((($162)) + 2|0);
    $164 = (_m_gt_1($163)|0);
    $165 = ($164|0)!=(0);
    if ($165) {
     $166 = HEAP32[$z>>2]|0;
     $167 = ((($166)) + 2|0);
     HEAP32[$z>>2] = $167;
    }
   }
   break;
  }
  case 99:  {
   $168 = HEAP32[$z>>2]|0;
   $169 = HEAP8[$168>>0]|0;
   $170 = $169 << 24 >> 24;
   $171 = ($170|0)==(101);
   if ($171) {
    $172 = HEAP32[$z>>2]|0;
    $173 = ((($172)) + 2|0);
    $174 = HEAP8[$173>>0]|0;
    $175 = $174 << 24 >> 24;
    $176 = ($175|0)==(110);
    if ($176) {
     $177 = HEAP32[$z>>2]|0;
     $178 = ((($177)) + 3|0);
     $179 = HEAP8[$178>>0]|0;
     $180 = $179 << 24 >> 24;
     $181 = ($180|0)==(97);
     if (!($181)) {
      $182 = HEAP32[$z>>2]|0;
      $183 = ((($182)) + 3|0);
      $184 = HEAP8[$183>>0]|0;
      $185 = $184 << 24 >> 24;
      $186 = ($185|0)==(101);
      if (!($186)) {
       break L80;
      }
     }
     $187 = HEAP32[$z>>2]|0;
     $188 = ((($187)) + 4|0);
     $189 = (_m_gt_1($188)|0);
     $190 = ($189|0)!=(0);
     if ($190) {
      $191 = HEAP32[$z>>2]|0;
      $192 = ((($191)) + 4|0);
      HEAP32[$z>>2] = $192;
     }
    }
   }
   break;
  }
  case 101:  {
   $193 = HEAP32[$z>>2]|0;
   $194 = HEAP8[$193>>0]|0;
   $195 = $194 << 24 >> 24;
   $196 = ($195|0)==(114);
   if ($196) {
    $197 = HEAP32[$z>>2]|0;
    $198 = ((($197)) + 2|0);
    $199 = (_m_gt_1($198)|0);
    $200 = ($199|0)!=(0);
    if ($200) {
     $201 = HEAP32[$z>>2]|0;
     $202 = ((($201)) + 2|0);
     HEAP32[$z>>2] = $202;
    }
   }
   break;
  }
  case 105:  {
   $203 = HEAP32[$z>>2]|0;
   $204 = HEAP8[$203>>0]|0;
   $205 = $204 << 24 >> 24;
   $206 = ($205|0)==(99);
   if ($206) {
    $207 = HEAP32[$z>>2]|0;
    $208 = ((($207)) + 2|0);
    $209 = (_m_gt_1($208)|0);
    $210 = ($209|0)!=(0);
    if ($210) {
     $211 = HEAP32[$z>>2]|0;
     $212 = ((($211)) + 2|0);
     HEAP32[$z>>2] = $212;
    }
   }
   break;
  }
  case 108:  {
   $213 = HEAP32[$z>>2]|0;
   $214 = HEAP8[$213>>0]|0;
   $215 = $214 << 24 >> 24;
   $216 = ($215|0)==(101);
   if ($216) {
    $217 = HEAP32[$z>>2]|0;
    $218 = ((($217)) + 2|0);
    $219 = HEAP8[$218>>0]|0;
    $220 = $219 << 24 >> 24;
    $221 = ($220|0)==(98);
    if ($221) {
     $222 = HEAP32[$z>>2]|0;
     $223 = ((($222)) + 3|0);
     $224 = HEAP8[$223>>0]|0;
     $225 = $224 << 24 >> 24;
     $226 = ($225|0)==(97);
     if (!($226)) {
      $227 = HEAP32[$z>>2]|0;
      $228 = ((($227)) + 3|0);
      $229 = HEAP8[$228>>0]|0;
      $230 = $229 << 24 >> 24;
      $231 = ($230|0)==(105);
      if (!($231)) {
       break L80;
      }
     }
     $232 = HEAP32[$z>>2]|0;
     $233 = ((($232)) + 4|0);
     $234 = (_m_gt_1($233)|0);
     $235 = ($234|0)!=(0);
     if ($235) {
      $236 = HEAP32[$z>>2]|0;
      $237 = ((($236)) + 4|0);
      HEAP32[$z>>2] = $237;
     }
    }
   }
   break;
  }
  case 110:  {
   $238 = HEAP32[$z>>2]|0;
   $239 = HEAP8[$238>>0]|0;
   $240 = $239 << 24 >> 24;
   $241 = ($240|0)==(116);
   if ($241) {
    $242 = HEAP32[$z>>2]|0;
    $243 = ((($242)) + 2|0);
    $244 = HEAP8[$243>>0]|0;
    $245 = $244 << 24 >> 24;
    $246 = ($245|0)==(97);
    $247 = HEAP32[$z>>2]|0;
    if ($246) {
     $248 = ((($247)) + 3|0);
     $249 = (_m_gt_1($248)|0);
     $250 = ($249|0)!=(0);
     if (!($250)) {
      break L80;
     }
     $251 = HEAP32[$z>>2]|0;
     $252 = ((($251)) + 3|0);
     HEAP32[$z>>2] = $252;
     break L80;
    }
    $253 = ((($247)) + 2|0);
    $254 = HEAP8[$253>>0]|0;
    $255 = $254 << 24 >> 24;
    $256 = ($255|0)==(101);
    if ($256) {
     $257 = (_stem($z,25390,18402,174)|0);
     $258 = ($257|0)!=(0);
     if (!($258)) {
      $259 = (_stem($z,25396,18402,174)|0);
      $260 = ($259|0)!=(0);
      if (!($260)) {
       (_stem($z,25401,18402,174)|0);
      }
     }
    }
   }
   break;
  }
  case 111:  {
   $261 = HEAP32[$z>>2]|0;
   $262 = HEAP8[$261>>0]|0;
   $263 = $262 << 24 >> 24;
   $264 = ($263|0)==(117);
   $265 = HEAP32[$z>>2]|0;
   if ($264) {
    $266 = ((($265)) + 2|0);
    $267 = (_m_gt_1($266)|0);
    $268 = ($267|0)!=(0);
    if (!($268)) {
     break L80;
    }
    $269 = HEAP32[$z>>2]|0;
    $270 = ((($269)) + 2|0);
    HEAP32[$z>>2] = $270;
    break L80;
   }
   $271 = ((($265)) + 3|0);
   $272 = HEAP8[$271>>0]|0;
   $273 = $272 << 24 >> 24;
   $274 = ($273|0)==(115);
   if (!($274)) {
    $275 = HEAP32[$z>>2]|0;
    $276 = ((($275)) + 3|0);
    $277 = HEAP8[$276>>0]|0;
    $278 = $277 << 24 >> 24;
    $279 = ($278|0)==(116);
    if (!($279)) {
     break L80;
    }
   }
   (_stem($z,25405,18402,174)|0);
   break;
  }
  case 115:  {
   $280 = HEAP32[$z>>2]|0;
   $281 = HEAP8[$280>>0]|0;
   $282 = $281 << 24 >> 24;
   $283 = ($282|0)==(109);
   if ($283) {
    $284 = HEAP32[$z>>2]|0;
    $285 = ((($284)) + 2|0);
    $286 = HEAP8[$285>>0]|0;
    $287 = $286 << 24 >> 24;
    $288 = ($287|0)==(105);
    if ($288) {
     $289 = HEAP32[$z>>2]|0;
     $290 = ((($289)) + 3|0);
     $291 = (_m_gt_1($290)|0);
     $292 = ($291|0)!=(0);
     if ($292) {
      $293 = HEAP32[$z>>2]|0;
      $294 = ((($293)) + 3|0);
      HEAP32[$z>>2] = $294;
     }
    }
   }
   break;
  }
  case 116:  {
   $295 = (_stem($z,25409,18402,174)|0);
   $296 = ($295|0)!=(0);
   if (!($296)) {
    (_stem($z,25413,18402,174)|0);
   }
   break;
  }
  case 117:  {
   $297 = HEAP32[$z>>2]|0;
   $298 = HEAP8[$297>>0]|0;
   $299 = $298 << 24 >> 24;
   $300 = ($299|0)==(115);
   if ($300) {
    $301 = HEAP32[$z>>2]|0;
    $302 = ((($301)) + 2|0);
    $303 = HEAP8[$302>>0]|0;
    $304 = $303 << 24 >> 24;
    $305 = ($304|0)==(111);
    if ($305) {
     $306 = HEAP32[$z>>2]|0;
     $307 = ((($306)) + 3|0);
     $308 = (_m_gt_1($307)|0);
     $309 = ($308|0)!=(0);
     if ($309) {
      $310 = HEAP32[$z>>2]|0;
      $311 = ((($310)) + 3|0);
      HEAP32[$z>>2] = $311;
     }
    }
   }
   break;
  }
  case 122: case 118:  {
   $312 = HEAP32[$z>>2]|0;
   $313 = HEAP8[$312>>0]|0;
   $314 = $313 << 24 >> 24;
   $315 = ($314|0)==(101);
   if ($315) {
    $316 = HEAP32[$z>>2]|0;
    $317 = ((($316)) + 2|0);
    $318 = HEAP8[$317>>0]|0;
    $319 = $318 << 24 >> 24;
    $320 = ($319|0)==(105);
    if ($320) {
     $321 = HEAP32[$z>>2]|0;
     $322 = ((($321)) + 3|0);
     $323 = (_m_gt_1($322)|0);
     $324 = ($323|0)!=(0);
     if ($324) {
      $325 = HEAP32[$z>>2]|0;
      $326 = ((($325)) + 3|0);
      HEAP32[$z>>2] = $326;
     }
    }
   }
   break;
  }
  default: {
  }
  }
 } while(0);
 $327 = HEAP32[$z>>2]|0;
 $328 = HEAP8[$327>>0]|0;
 $329 = $328 << 24 >> 24;
 $330 = ($329|0)==(101);
 do {
  if ($330) {
   $331 = HEAP32[$z>>2]|0;
   $332 = ((($331)) + 1|0);
   $333 = (_m_gt_1($332)|0);
   $334 = ($333|0)!=(0);
   $335 = HEAP32[$z>>2]|0;
   $336 = ((($335)) + 1|0);
   if ($334) {
    HEAP32[$z>>2] = $336;
    break;
   }
   $337 = (_m_eq_1($336)|0);
   $338 = ($337|0)!=(0);
   if ($338) {
    $339 = HEAP32[$z>>2]|0;
    $340 = ((($339)) + 1|0);
    $341 = (_star_oh($340)|0);
    $342 = ($341|0)!=(0);
    if (!($342)) {
     $343 = HEAP32[$z>>2]|0;
     $344 = ((($343)) + 1|0);
     HEAP32[$z>>2] = $344;
    }
   }
  }
 } while(0);
 $345 = HEAP32[$z>>2]|0;
 $346 = (_m_gt_1($345)|0);
 $347 = ($346|0)!=(0);
 if ($347) {
  $348 = HEAP32[$z>>2]|0;
  $349 = HEAP8[$348>>0]|0;
  $350 = $349 << 24 >> 24;
  $351 = ($350|0)==(108);
  if ($351) {
   $352 = HEAP32[$z>>2]|0;
   $353 = ((($352)) + 1|0);
   $354 = HEAP8[$353>>0]|0;
   $355 = $354 << 24 >> 24;
   $356 = ($355|0)==(108);
   if ($356) {
    $357 = HEAP32[$z>>2]|0;
    $358 = ((($357)) + 1|0);
    HEAP32[$z>>2] = $358;
   }
  }
 }
 $359 = HEAP32[$z>>2]|0;
 $360 = (_strlen($359)|0);
 HEAP32[$i>>2] = $360;
 $361 = HEAP32[$3>>2]|0;
 HEAP32[$361>>2] = $360;
 $362 = HEAP32[$i>>2]|0;
 $363 = HEAP32[$2>>2]|0;
 $364 = (($363) + ($362)|0);
 HEAP8[$364>>0] = 0;
 while(1) {
  $365 = HEAP32[$z>>2]|0;
  $366 = HEAP8[$365>>0]|0;
  $367 = ($366<<24>>24)!=(0);
  if (!($367)) {
   break;
  }
  $368 = HEAP32[$z>>2]|0;
  $369 = ((($368)) + 1|0);
  HEAP32[$z>>2] = $369;
  $370 = HEAP8[$368>>0]|0;
  $371 = HEAP32[$i>>2]|0;
  $372 = (($371) + -1)|0;
  HEAP32[$i>>2] = $372;
  $373 = HEAP32[$2>>2]|0;
  $374 = (($373) + ($372)|0);
  HEAP8[$374>>0] = $370;
 }
 STACKTOP = sp;return;
}
function _copy_stemmer($zIn,$nIn,$zOut,$pnOut) {
 $zIn = $zIn|0;
 $nIn = $nIn|0;
 $zOut = $zOut|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $hasDigit = 0, $i = 0, $j = 0, $mx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $mx = sp + 8|0;
 $j = sp + 4|0;
 $hasDigit = sp;
 $c = sp + 32|0;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $nIn;
 HEAP32[$2>>2] = $zOut;
 HEAP32[$3>>2] = $pnOut;
 HEAP32[$hasDigit>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ($4|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = (($8) + ($7)|0);
  $10 = HEAP8[$9>>0]|0;
  HEAP8[$c>>0] = $10;
  $11 = HEAP8[$c>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)>=(65);
  if ($13) {
   $14 = HEAP8[$c>>0]|0;
   $15 = $14 << 24 >> 24;
   $16 = ($15|0)<=(90);
   if ($16) {
    $17 = HEAP8[$c>>0]|0;
    $18 = $17 << 24 >> 24;
    $19 = (($18) - 65)|0;
    $20 = (($19) + 97)|0;
    $21 = $20&255;
    $22 = HEAP32[$i>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    $24 = (($23) + ($22)|0);
    HEAP8[$24>>0] = $21;
   } else {
    label = 6;
   }
  } else {
   label = 6;
  }
  if ((label|0) == 6) {
   label = 0;
   $25 = HEAP8[$c>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = ($26|0)>=(48);
   if ($27) {
    $28 = HEAP8[$c>>0]|0;
    $29 = $28 << 24 >> 24;
    $30 = ($29|0)<=(57);
    if ($30) {
     HEAP32[$hasDigit>>2] = 1;
    }
   }
   $31 = HEAP8[$c>>0]|0;
   $32 = HEAP32[$i>>2]|0;
   $33 = HEAP32[$2>>2]|0;
   $34 = (($33) + ($32)|0);
   HEAP8[$34>>0] = $31;
  }
  $35 = HEAP32[$i>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$i>>2] = $36;
 }
 $37 = HEAP32[$hasDigit>>2]|0;
 $38 = ($37|0)!=(0);
 $39 = $38 ? 3 : 10;
 HEAP32[$mx>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$mx>>2]|0;
 $42 = $41<<1;
 $43 = ($40|0)>($42|0);
 if (!($43)) {
  $63 = HEAP32[$i>>2]|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = (($64) + ($63)|0);
  HEAP8[$65>>0] = 0;
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$3>>2]|0;
  HEAP32[$67>>2] = $66;
  STACKTOP = sp;return;
 }
 $44 = HEAP32[$mx>>2]|0;
 HEAP32[$j>>2] = $44;
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$mx>>2]|0;
 $47 = (($45) - ($46))|0;
 HEAP32[$i>>2] = $47;
 while(1) {
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = ($48|0)<($49|0);
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$i>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = (($52) + ($51)|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = HEAP32[$j>>2]|0;
  $56 = HEAP32[$2>>2]|0;
  $57 = (($56) + ($55)|0);
  HEAP8[$57>>0] = $54;
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
  $60 = HEAP32[$j>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$j>>2] = $61;
 }
 $62 = HEAP32[$j>>2]|0;
 HEAP32[$i>>2] = $62;
 $63 = HEAP32[$i>>2]|0;
 $64 = HEAP32[$2>>2]|0;
 $65 = (($64) + ($63)|0);
 HEAP8[$65>>0] = 0;
 $66 = HEAP32[$i>>2]|0;
 $67 = HEAP32[$3>>2]|0;
 HEAP32[$67>>2] = $66;
 STACKTOP = sp;return;
}
function _stem($pz,$zFrom,$zTo,$xCond) {
 $pz = $pz|0;
 $zFrom = $zFrom|0;
 $zTo = $zTo|0;
 $xCond = $xCond|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $pz;
 HEAP32[$2>>2] = $zFrom;
 HEAP32[$3>>2] = $zTo;
 HEAP32[$4>>2] = $xCond;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$z>>2] = $6;
 while(1) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = HEAP32[$z>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = ($13|0)==($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$z>>2]|0;
  $19 = ((($18)) + 1|0);
  HEAP32[$z>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 1|0);
  HEAP32[$2>>2] = $21;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP8[$22>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = ($24|0)!=(0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $26 = HEAP32[$4>>2]|0;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = HEAP32[$4>>2]|0;
  $29 = HEAP32[$z>>2]|0;
  $30 = (FUNCTION_TABLE_ii[$28 & 255]($29)|0);
  $31 = ($30|0)!=(0);
  if (!($31)) {
   HEAP32[$0>>2] = 1;
   $42 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($42|0);
  }
 }
 while(1) {
  $32 = HEAP32[$3>>2]|0;
  $33 = HEAP8[$32>>0]|0;
  $34 = ($33<<24>>24)!=(0);
  if (!($34)) {
   break;
  }
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 1|0);
  HEAP32[$3>>2] = $36;
  $37 = HEAP8[$35>>0]|0;
  $38 = HEAP32[$z>>2]|0;
  $39 = ((($38)) + -1|0);
  HEAP32[$z>>2] = $39;
  HEAP8[$39>>0] = $37;
 }
 $40 = HEAP32[$z>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 HEAP32[$41>>2] = $40;
 HEAP32[$0>>2] = 1;
 $42 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _m_gt_0($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = (_isVowel($2)|0);
  $4 = ($3|0)!=(0);
  $5 = HEAP32[$1>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 1|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP8[$5>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_isConsonant($10)|0);
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   break;
  }
  $14 = ((($13)) + 1|0);
  HEAP32[$1>>2] = $14;
 }
 $15 = HEAP8[$13>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)!=(0);
 $18 = $17&1;
 HEAP32[$0>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _hasVowel($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $z;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = (_isConsonant($1)|0);
  $3 = ($2|0)!=(0);
  $4 = HEAP32[$0>>2]|0;
  if (!($3)) {
   break;
  }
  $5 = ((($4)) + 1|0);
  HEAP32[$0>>2] = $5;
 }
 $6 = HEAP8[$4>>0]|0;
 $7 = $6 << 24 >> 24;
 $8 = ($7|0)!=(0);
 $9 = $8&1;
 STACKTOP = sp;return ($9|0);
}
function _doubleConsonant($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_isConsonant($1)|0);
 $3 = ($2|0)!=(0);
 if (!($3)) {
  $13 = 0;
  $12 = $13&1;
  STACKTOP = sp;return ($12|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 1|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = ($6|0)==($10|0);
 $13 = $11;
 $12 = $13&1;
 STACKTOP = sp;return ($12|0);
}
function _m_eq_1($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = (_isVowel($2)|0);
  $4 = ($3|0)!=(0);
  $5 = HEAP32[$1>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 1|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP8[$5>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_isConsonant($10)|0);
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   break;
  }
  $14 = ((($13)) + 1|0);
  HEAP32[$1>>2] = $14;
 }
 $15 = HEAP8[$13>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)==(0);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $18 = HEAP32[$1>>2]|0;
  $19 = (_isVowel($18)|0);
  $20 = ($19|0)!=(0);
  $21 = HEAP32[$1>>2]|0;
  if (!($20)) {
   break;
  }
  $22 = ((($21)) + 1|0);
  HEAP32[$1>>2] = $22;
 }
 $23 = HEAP8[$21>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = ($24|0)==(0);
 if ($25) {
  HEAP32[$0>>2] = 1;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (_isConsonant($26)|0);
  $28 = ($27|0)!=(0);
  $29 = HEAP32[$1>>2]|0;
  if (!($28)) {
   break;
  }
  $30 = ((($29)) + 1|0);
  HEAP32[$1>>2] = $30;
 }
 $31 = HEAP8[$29>>0]|0;
 $32 = $31 << 24 >> 24;
 $33 = ($32|0)==(0);
 $34 = $33&1;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _star_oh($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_isConsonant($1)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)!=(119);
  if ($7) {
   $8 = HEAP32[$0>>2]|0;
   $9 = HEAP8[$8>>0]|0;
   $10 = $9 << 24 >> 24;
   $11 = ($10|0)!=(120);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = HEAP8[$12>>0]|0;
    $14 = $13 << 24 >> 24;
    $15 = ($14|0)!=(121);
    if ($15) {
     $16 = HEAP32[$0>>2]|0;
     $17 = ((($16)) + 1|0);
     $18 = (_isVowel($17)|0);
     $19 = ($18|0)!=(0);
     if ($19) {
      $20 = HEAP32[$0>>2]|0;
      $21 = ((($20)) + 2|0);
      $22 = (_isConsonant($21)|0);
      $23 = ($22|0)!=(0);
      $25 = $23;
     } else {
      $25 = 0;
     }
    } else {
     $25 = 0;
    }
   } else {
    $25 = 0;
   }
  } else {
   $25 = 0;
  }
 } else {
  $25 = 0;
 }
 $24 = $25&1;
 STACKTOP = sp;return ($24|0);
}
function _m_gt_1($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = (_isVowel($2)|0);
  $4 = ($3|0)!=(0);
  $5 = HEAP32[$1>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 1|0);
  HEAP32[$1>>2] = $6;
 }
 $7 = HEAP8[$5>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_isConsonant($10)|0);
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$1>>2]|0;
  if (!($12)) {
   break;
  }
  $14 = ((($13)) + 1|0);
  HEAP32[$1>>2] = $14;
 }
 $15 = HEAP8[$13>>0]|0;
 $16 = $15 << 24 >> 24;
 $17 = ($16|0)==(0);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $18 = HEAP32[$1>>2]|0;
  $19 = (_isVowel($18)|0);
  $20 = ($19|0)!=(0);
  $21 = HEAP32[$1>>2]|0;
  if (!($20)) {
   break;
  }
  $22 = ((($21)) + 1|0);
  HEAP32[$1>>2] = $22;
 }
 $23 = HEAP8[$21>>0]|0;
 $24 = $23 << 24 >> 24;
 $25 = ($24|0)==(0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 while(1) {
  $26 = HEAP32[$1>>2]|0;
  $27 = (_isConsonant($26)|0);
  $28 = ($27|0)!=(0);
  $29 = HEAP32[$1>>2]|0;
  if (!($28)) {
   break;
  }
  $30 = ((($29)) + 1|0);
  HEAP32[$1>>2] = $30;
 }
 $31 = HEAP8[$29>>0]|0;
 $32 = $31 << 24 >> 24;
 $33 = ($32|0)!=(0);
 $34 = $33&1;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _isVowel($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $j = sp;
 $x = sp + 12|0;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 HEAP8[$x>>0] = $3;
 $4 = HEAP8[$x>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $7 = HEAP8[$x>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = (($8) - 97)|0;
 $10 = (25417 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 HEAP32[$j>>2] = $12;
 $13 = HEAP32[$j>>2]|0;
 $14 = ($13|0)<(2);
 if ($14) {
  $15 = HEAP32[$j>>2]|0;
  $16 = (1 - ($15))|0;
  HEAP32[$0>>2] = $16;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 } else {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 1|0);
  $19 = (_isConsonant($18)|0);
  HEAP32[$0>>2] = $19;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _isConsonant($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $j = sp;
 $x = sp + 12|0;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 HEAP8[$x>>0] = $3;
 $4 = HEAP8[$x>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = HEAP8[$x>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = (($8) - 97)|0;
 $10 = (25417 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11 << 24 >> 24;
 HEAP32[$j>>2] = $12;
 $13 = HEAP32[$j>>2]|0;
 $14 = ($13|0)<(2);
 if ($14) {
  $15 = HEAP32[$j>>2]|0;
  HEAP32[$0>>2] = $15;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 1|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18 << 24 >> 24;
 $20 = ($19|0)==(0);
 if ($20) {
  $26 = 1;
 } else {
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 1|0);
  $23 = (_isVowel($22)|0);
  $24 = ($23|0)!=(0);
  $26 = $24;
 }
 $25 = $26&1;
 HEAP32[$0>>2] = $25;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _simpleCreate($argc,$argv,$ppTokenizer) {
 $argc = $argc|0;
 $argv = $argv|0;
 $ppTokenizer = $ppTokenizer|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ch = 0, $i = 0, $i1 = 0, $n = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $t = sp + 12|0;
 $i = sp + 8|0;
 $n = sp + 4|0;
 $ch = sp + 32|0;
 $i1 = sp;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$3>>2] = $ppTokenizer;
 $4 = (_sqlite3_malloc(132)|0);
 HEAP32[$t>>2] = $4;
 $5 = HEAP32[$t>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 7;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 $7 = HEAP32[$t>>2]|0;
 _memset(($7|0),0,132)|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)>(1);
 L5: do {
  if ($9) {
   $10 = HEAP32[$2>>2]|0;
   $11 = ((($10)) + 4|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (_strlen($12)|0);
   HEAP32[$n>>2] = $13;
   HEAP32[$i>>2] = 0;
   while(1) {
    $14 = HEAP32[$i>>2]|0;
    $15 = HEAP32[$n>>2]|0;
    $16 = ($14|0)<($15|0);
    if (!($16)) {
     break L5;
    }
    $17 = HEAP32[$i>>2]|0;
    $18 = HEAP32[$2>>2]|0;
    $19 = ((($18)) + 4|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($20) + ($17)|0);
    $22 = HEAP8[$21>>0]|0;
    HEAP8[$ch>>0] = $22;
    $23 = HEAP8[$ch>>0]|0;
    $24 = $23&255;
    $25 = ($24|0)>=(128);
    if ($25) {
     break;
    }
    $27 = HEAP8[$ch>>0]|0;
    $28 = $27&255;
    $29 = HEAP32[$t>>2]|0;
    $30 = ((($29)) + 4|0);
    $31 = (($30) + ($28)|0);
    HEAP8[$31>>0] = 1;
    $32 = HEAP32[$i>>2]|0;
    $33 = (($32) + 1)|0;
    HEAP32[$i>>2] = $33;
   }
   $26 = HEAP32[$t>>2]|0;
   _sqlite3_free($26);
   HEAP32[$0>>2] = 1;
   $50 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($50|0);
  } else {
   HEAP32[$i1>>2] = 1;
   while(1) {
    $34 = HEAP32[$i1>>2]|0;
    $35 = ($34|0)<(128);
    if (!($35)) {
     break L5;
    }
    $36 = HEAP32[$i1>>2]|0;
    $37 = (_fts3_isalnum($36)|0);
    $38 = ($37|0)!=(0);
    $39 = $38 ^ 1;
    $40 = $39 ? -1 : 0;
    $41 = $40&255;
    $42 = HEAP32[$i1>>2]|0;
    $43 = HEAP32[$t>>2]|0;
    $44 = ((($43)) + 4|0);
    $45 = (($44) + ($42)|0);
    HEAP8[$45>>0] = $41;
    $46 = HEAP32[$i1>>2]|0;
    $47 = (($46) + 1)|0;
    HEAP32[$i1>>2] = $47;
   }
  }
 } while(0);
 $48 = HEAP32[$t>>2]|0;
 $49 = HEAP32[$3>>2]|0;
 HEAP32[$49>>2] = $48;
 HEAP32[$0>>2] = 0;
 $50 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _simpleDestroy($pTokenizer) {
 $pTokenizer = $pTokenizer|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTokenizer;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return 0;
}
function _simpleOpen($pTokenizer,$pInput,$nBytes,$ppCursor) {
 $pTokenizer = $pTokenizer|0;
 $pInput = $pInput|0;
 $nBytes = $nBytes|0;
 $ppCursor = $ppCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $c = sp;
 HEAP32[$1>>2] = $pTokenizer;
 HEAP32[$2>>2] = $pInput;
 HEAP32[$3>>2] = $nBytes;
 HEAP32[$4>>2] = $ppCursor;
 $5 = (_sqlite3_malloc(28)|0);
 HEAP32[$c>>2] = $5;
 $6 = HEAP32[$c>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 7;
  $34 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$c>>2]|0;
 $10 = ((($9)) + 4|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 do {
  if ($12) {
   $13 = HEAP32[$c>>2]|0;
   $14 = ((($13)) + 8|0);
   HEAP32[$14>>2] = 0;
  } else {
   $15 = HEAP32[$3>>2]|0;
   $16 = ($15|0)<(0);
   if ($16) {
    $17 = HEAP32[$2>>2]|0;
    $18 = (_strlen($17)|0);
    $19 = HEAP32[$c>>2]|0;
    $20 = ((($19)) + 8|0);
    HEAP32[$20>>2] = $18;
    break;
   } else {
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$c>>2]|0;
    $23 = ((($22)) + 8|0);
    HEAP32[$23>>2] = $21;
    break;
   }
  }
 } while(0);
 $24 = HEAP32[$c>>2]|0;
 $25 = ((($24)) + 12|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$c>>2]|0;
 $27 = ((($26)) + 16|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$c>>2]|0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$c>>2]|0;
 $31 = ((($30)) + 24|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$c>>2]|0;
 $33 = HEAP32[$4>>2]|0;
 HEAP32[$33>>2] = $32;
 HEAP32[$0>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _simpleClose($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$c>>2] = $1;
 $2 = HEAP32[$c>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$c>>2]|0;
 _sqlite3_free($5);
 STACKTOP = sp;return 0;
}
function _simpleNext($pCursor,$ppToken,$pnBytes,$piStartOffset,$piEndOffset,$piPosition) {
 $pCursor = $pCursor|0;
 $ppToken = $ppToken|0;
 $pnBytes = $pnBytes|0;
 $piStartOffset = $piStartOffset|0;
 $piEndOffset = $piEndOffset|0;
 $piPosition = $piPosition|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0;
 var $ch = 0, $i = 0, $iStartOffset = 0, $n = 0, $p = 0, $pNew = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $c = sp + 24|0;
 $t = sp + 20|0;
 $p = sp + 16|0;
 $iStartOffset = sp + 12|0;
 $i = sp + 8|0;
 $n = sp + 4|0;
 $pNew = sp;
 $ch = sp + 56|0;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $ppToken;
 HEAP32[$3>>2] = $pnBytes;
 HEAP32[$4>>2] = $piStartOffset;
 HEAP32[$5>>2] = $piEndOffset;
 HEAP32[$6>>2] = $piPosition;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$c>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$t>>2] = $9;
 $10 = HEAP32[$c>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p>>2] = $12;
 while(1) {
  $13 = HEAP32[$c>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$c>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)<($18|0);
  if (!($19)) {
   label = 25;
   break;
  }
  while(1) {
   $20 = HEAP32[$c>>2]|0;
   $21 = ((($20)) + 12|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$c>>2]|0;
   $24 = ((($23)) + 8|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($22|0)<($25|0);
   if ($26) {
    $27 = HEAP32[$t>>2]|0;
    $28 = HEAP32[$c>>2]|0;
    $29 = ((($28)) + 12|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$p>>2]|0;
    $32 = (($31) + ($30)|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = (_simpleDelim($27,$33)|0);
    $35 = ($34|0)!=(0);
    $137 = $35;
   } else {
    $137 = 0;
   }
   $36 = HEAP32[$c>>2]|0;
   $37 = ((($36)) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   if (!($137)) {
    break;
   }
   $39 = (($38) + 1)|0;
   HEAP32[$37>>2] = $39;
  }
  HEAP32[$iStartOffset>>2] = $38;
  while(1) {
   $40 = HEAP32[$c>>2]|0;
   $41 = ((($40)) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$c>>2]|0;
   $44 = ((($43)) + 8|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($42|0)<($45|0);
   if ($46) {
    $47 = HEAP32[$t>>2]|0;
    $48 = HEAP32[$c>>2]|0;
    $49 = ((($48)) + 12|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$p>>2]|0;
    $52 = (($51) + ($50)|0);
    $53 = HEAP8[$52>>0]|0;
    $54 = (_simpleDelim($47,$53)|0);
    $55 = ($54|0)!=(0);
    $56 = $55 ^ 1;
    $138 = $56;
   } else {
    $138 = 0;
   }
   $57 = HEAP32[$c>>2]|0;
   $58 = ((($57)) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   if (!($138)) {
    break;
   }
   $60 = (($59) + 1)|0;
   HEAP32[$58>>2] = $60;
  }
  $61 = HEAP32[$iStartOffset>>2]|0;
  $62 = ($59|0)>($61|0);
  if ($62) {
   break;
  }
 }
 if ((label|0) == 25) {
  HEAP32[$0>>2] = 101;
  $136 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($136|0);
 }
 $63 = HEAP32[$c>>2]|0;
 $64 = ((($63)) + 12|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$iStartOffset>>2]|0;
 $67 = (($65) - ($66))|0;
 HEAP32[$n>>2] = $67;
 $68 = HEAP32[$n>>2]|0;
 $69 = HEAP32[$c>>2]|0;
 $70 = ((($69)) + 24|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($68|0)>($71|0);
 do {
  if ($72) {
   $73 = HEAP32[$n>>2]|0;
   $74 = (($73) + 20)|0;
   $75 = HEAP32[$c>>2]|0;
   $76 = ((($75)) + 24|0);
   HEAP32[$76>>2] = $74;
   $77 = HEAP32[$c>>2]|0;
   $78 = ((($77)) + 20|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = HEAP32[$c>>2]|0;
   $81 = ((($80)) + 24|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (_sqlite3_realloc($79,$82)|0);
   HEAP32[$pNew>>2] = $83;
   $84 = HEAP32[$pNew>>2]|0;
   $85 = ($84|0)!=(0|0);
   if ($85) {
    $86 = HEAP32[$pNew>>2]|0;
    $87 = HEAP32[$c>>2]|0;
    $88 = ((($87)) + 20|0);
    HEAP32[$88>>2] = $86;
    break;
   }
   HEAP32[$0>>2] = 7;
   $136 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($136|0);
  }
 } while(0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $89 = HEAP32[$i>>2]|0;
  $90 = HEAP32[$n>>2]|0;
  $91 = ($89|0)<($90|0);
  if (!($91)) {
   break;
  }
  $92 = HEAP32[$iStartOffset>>2]|0;
  $93 = HEAP32[$i>>2]|0;
  $94 = (($92) + ($93))|0;
  $95 = HEAP32[$p>>2]|0;
  $96 = (($95) + ($94)|0);
  $97 = HEAP8[$96>>0]|0;
  HEAP8[$ch>>0] = $97;
  $98 = HEAP8[$ch>>0]|0;
  $99 = $98&255;
  $100 = ($99|0)>=(65);
  if ($100) {
   $101 = HEAP8[$ch>>0]|0;
   $102 = $101&255;
   $103 = ($102|0)<=(90);
   if ($103) {
    $104 = HEAP8[$ch>>0]|0;
    $105 = $104&255;
    $106 = (($105) - 65)|0;
    $107 = (($106) + 97)|0;
    $111 = $107;
   } else {
    label = 22;
   }
  } else {
   label = 22;
  }
  if ((label|0) == 22) {
   label = 0;
   $108 = HEAP8[$ch>>0]|0;
   $109 = $108&255;
   $111 = $109;
  }
  $110 = $111&255;
  $112 = HEAP32[$i>>2]|0;
  $113 = HEAP32[$c>>2]|0;
  $114 = ((($113)) + 20|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($115) + ($112)|0);
  HEAP8[$116>>0] = $110;
  $117 = HEAP32[$i>>2]|0;
  $118 = (($117) + 1)|0;
  HEAP32[$i>>2] = $118;
 }
 $119 = HEAP32[$c>>2]|0;
 $120 = ((($119)) + 20|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$2>>2]|0;
 HEAP32[$122>>2] = $121;
 $123 = HEAP32[$n>>2]|0;
 $124 = HEAP32[$3>>2]|0;
 HEAP32[$124>>2] = $123;
 $125 = HEAP32[$iStartOffset>>2]|0;
 $126 = HEAP32[$4>>2]|0;
 HEAP32[$126>>2] = $125;
 $127 = HEAP32[$c>>2]|0;
 $128 = ((($127)) + 12|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = HEAP32[$5>>2]|0;
 HEAP32[$130>>2] = $129;
 $131 = HEAP32[$c>>2]|0;
 $132 = ((($131)) + 16|0);
 $133 = HEAP32[$132>>2]|0;
 $134 = (($133) + 1)|0;
 HEAP32[$132>>2] = $134;
 $135 = HEAP32[$6>>2]|0;
 HEAP32[$135>>2] = $133;
 HEAP32[$0>>2] = 0;
 $136 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($136|0);
}
function _simpleDelim($t,$c) {
 $t = $t|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $t;
 HEAP8[$1>>0] = $c;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = ($3|0)<(128);
 if (!($4)) {
  $14 = 0;
  $13 = $14&1;
  STACKTOP = sp;return ($13|0);
 }
 $5 = HEAP8[$1>>0]|0;
 $6 = $5&255;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = (($8) + ($6)|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10 << 24 >> 24;
 $12 = ($11|0)!=(0);
 $14 = $12;
 $13 = $14&1;
 STACKTOP = sp;return ($13|0);
}
function _fts3_isalnum($x) {
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $x;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)>=(48);
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)<=(57);
 $or$cond = $2 & $4;
 if ($or$cond) {
  $15 = 1;
 } else {
  $5 = HEAP32[$0>>2]|0;
  $6 = ($5|0)>=(65);
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)<=(90);
  $or$cond3 = $6 & $8;
  if ($or$cond3) {
   $15 = 1;
  } else {
   $9 = HEAP32[$0>>2]|0;
   $10 = ($9|0)>=(97);
   $11 = HEAP32[$0>>2]|0;
   $12 = ($11|0)<=(122);
   $13 = $10 ? $12 : 0;
   $15 = $13;
  }
 }
 $14 = $15&1;
 STACKTOP = sp;return ($14|0);
}
function _fts3auxConnectMethod($db,$pUnused,$argc,$argv,$ppVtab,$pzErr) {
 $db = $db|0;
 $pUnused = $pUnused|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $ppVtab = $ppVtab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nByte = 0, $nDb = 0, $nFts3 = 0, $or$cond = 0, $p = 0, $rc = 0, $vararg_buffer = 0, $zDb = 0, $zFts3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $zDb = sp + 28|0;
 $zFts3 = sp + 24|0;
 $nDb = sp + 20|0;
 $nFts3 = sp + 16|0;
 $nByte = sp + 12|0;
 $rc = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pUnused;
 HEAP32[$3>>2] = $argc;
 HEAP32[$4>>2] = $argv;
 HEAP32[$5>>2] = $ppVtab;
 HEAP32[$6>>2] = $pzErr;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)!=(4);
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(5);
 $or$cond = $8 & $10;
 do {
  if (!($or$cond)) {
   $11 = HEAP32[$4>>2]|0;
   $12 = ((($11)) + 4|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$zDb>>2] = $13;
   $14 = HEAP32[$zDb>>2]|0;
   $15 = (_strlen($14)|0);
   HEAP32[$nDb>>2] = $15;
   $16 = HEAP32[$3>>2]|0;
   $17 = ($16|0)==(5);
   if ($17) {
    $18 = HEAP32[$nDb>>2]|0;
    $19 = ($18|0)==(4);
    if (!($19)) {
     break;
    }
    $20 = HEAP32[$zDb>>2]|0;
    $21 = (_sqlite3_strnicmp(18672,$20,4)|0);
    $22 = (0)==($21|0);
    if (!($22)) {
     break;
    }
    $23 = HEAP32[$4>>2]|0;
    $24 = ((($23)) + 12|0);
    $25 = HEAP32[$24>>2]|0;
    HEAP32[$zDb>>2] = $25;
    $26 = HEAP32[$zDb>>2]|0;
    $27 = (_strlen($26)|0);
    HEAP32[$nDb>>2] = $27;
    $28 = HEAP32[$4>>2]|0;
    $29 = ((($28)) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    HEAP32[$zFts3>>2] = $30;
   } else {
    $31 = HEAP32[$4>>2]|0;
    $32 = ((($31)) + 12|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$zFts3>>2] = $33;
   }
   $34 = HEAP32[$zFts3>>2]|0;
   $35 = (_strlen($34)|0);
   HEAP32[$nFts3>>2] = $35;
   $36 = HEAP32[$1>>2]|0;
   $37 = (_sqlite3_declare_vtab($36,25453)|0);
   HEAP32[$rc>>2] = $37;
   $38 = HEAP32[$rc>>2]|0;
   $39 = ($38|0)!=(0);
   if ($39) {
    $40 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $40;
    $107 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($107|0);
   }
   $41 = HEAP32[$nDb>>2]|0;
   $42 = (296 + ($41))|0;
   $43 = HEAP32[$nFts3>>2]|0;
   $44 = (($42) + ($43))|0;
   $45 = (($44) + 2)|0;
   HEAP32[$nByte>>2] = $45;
   $46 = HEAP32[$nByte>>2]|0;
   $47 = (_sqlite3_malloc($46)|0);
   HEAP32[$p>>2] = $47;
   $48 = HEAP32[$p>>2]|0;
   $49 = ($48|0)!=(0|0);
   if ($49) {
    $50 = HEAP32[$p>>2]|0;
    $51 = HEAP32[$nByte>>2]|0;
    _memset(($50|0),0,($51|0))|0;
    $52 = HEAP32[$p>>2]|0;
    $53 = ((($52)) + 16|0);
    $54 = HEAP32[$p>>2]|0;
    $55 = ((($54)) + 12|0);
    HEAP32[$55>>2] = $53;
    $56 = HEAP32[$p>>2]|0;
    $57 = ((($56)) + 12|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = ((($58)) + 280|0);
    $60 = HEAP32[$p>>2]|0;
    $61 = ((($60)) + 12|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ((($62)) + 16|0);
    HEAP32[$63>>2] = $59;
    $64 = HEAP32[$nDb>>2]|0;
    $65 = (($64) + 1)|0;
    $66 = HEAP32[$p>>2]|0;
    $67 = ((($66)) + 12|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = ((($68)) + 16|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (($70) + ($65)|0);
    $72 = HEAP32[$p>>2]|0;
    $73 = ((($72)) + 12|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ((($74)) + 20|0);
    HEAP32[$75>>2] = $71;
    $76 = HEAP32[$1>>2]|0;
    $77 = HEAP32[$p>>2]|0;
    $78 = ((($77)) + 12|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ((($79)) + 12|0);
    HEAP32[$80>>2] = $76;
    $81 = HEAP32[$p>>2]|0;
    $82 = ((($81)) + 12|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ((($83)) + 248|0);
    HEAP32[$84>>2] = 1;
    $85 = HEAP32[$p>>2]|0;
    $86 = ((($85)) + 12|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ((($87)) + 16|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$zDb>>2]|0;
    $91 = HEAP32[$nDb>>2]|0;
    _memcpy(($89|0),($90|0),($91|0))|0;
    $92 = HEAP32[$p>>2]|0;
    $93 = ((($92)) + 12|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ((($94)) + 20|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$zFts3>>2]|0;
    $98 = HEAP32[$nFts3>>2]|0;
    _memcpy(($96|0),($97|0),($98|0))|0;
    $99 = HEAP32[$p>>2]|0;
    $100 = ((($99)) + 12|0);
    $101 = HEAP32[$100>>2]|0;
    $102 = ((($101)) + 20|0);
    $103 = HEAP32[$102>>2]|0;
    _sqlite3Fts3Dequote($103);
    $104 = HEAP32[$p>>2]|0;
    $105 = HEAP32[$5>>2]|0;
    HEAP32[$105>>2] = $104;
    HEAP32[$0>>2] = 0;
    $107 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($107|0);
   } else {
    HEAP32[$0>>2] = 7;
    $107 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($107|0);
   }
  }
 } while(0);
 $106 = HEAP32[$6>>2]|0;
 _sqlite3Fts3ErrMsg($106,25522,$vararg_buffer);
 HEAP32[$0>>2] = 1;
 $107 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($107|0);
}
function _fts3auxBestIndexMethod($pVTab,$pInfo) {
 $pVTab = $pVTab|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0.0, $101 = 0.0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0.0, $118 = 0.0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0.0, $131 = 0.0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCol = 0, $iEq = 0, $iGe = 0, $iLangid = 0, $iLe = 0, $iNext = 0, $op = 0;
 var $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $i = sp + 28|0;
 $iEq = sp + 24|0;
 $iGe = sp + 20|0;
 $iLe = sp + 16|0;
 $iLangid = sp + 12|0;
 $iNext = sp + 8|0;
 $op = sp + 4|0;
 $iCol = sp;
 HEAP32[$0>>2] = $pVTab;
 HEAP32[$1>>2] = $pInfo;
 HEAP32[$iEq>>2] = -1;
 HEAP32[$iGe>>2] = -1;
 HEAP32[$iLe>>2] = -1;
 HEAP32[$iLangid>>2] = -1;
 HEAP32[$iNext>>2] = 1;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(1);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ((($13)) + 4|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)==(0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 32|0);
    HEAP32[$19>>2] = 1;
   }
  }
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = ($20|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($24*12)|0)|0);
  $29 = ((($28)) + 5|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = ($30<<24>>24)!=(0);
  if ($31) {
   $32 = HEAP32[$i>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + (($32*12)|0)|0);
   $37 = ((($36)) + 4|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   HEAP32[$op>>2] = $39;
   $40 = HEAP32[$i>>2]|0;
   $41 = HEAP32[$1>>2]|0;
   $42 = ((($41)) + 4|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (($43) + (($40*12)|0)|0);
   $45 = HEAP32[$44>>2]|0;
   HEAP32[$iCol>>2] = $45;
   $46 = HEAP32[$iCol>>2]|0;
   $47 = ($46|0)==(0);
   if ($47) {
    $48 = HEAP32[$op>>2]|0;
    $49 = ($48|0)==(2);
    if ($49) {
     $50 = HEAP32[$i>>2]|0;
     HEAP32[$iEq>>2] = $50;
    }
    $51 = HEAP32[$op>>2]|0;
    $52 = ($51|0)==(16);
    if ($52) {
     $53 = HEAP32[$i>>2]|0;
     HEAP32[$iLe>>2] = $53;
    }
    $54 = HEAP32[$op>>2]|0;
    $55 = ($54|0)==(8);
    if ($55) {
     $56 = HEAP32[$i>>2]|0;
     HEAP32[$iLe>>2] = $56;
    }
    $57 = HEAP32[$op>>2]|0;
    $58 = ($57|0)==(4);
    if ($58) {
     $59 = HEAP32[$i>>2]|0;
     HEAP32[$iGe>>2] = $59;
    }
    $60 = HEAP32[$op>>2]|0;
    $61 = ($60|0)==(32);
    if ($61) {
     $62 = HEAP32[$i>>2]|0;
     HEAP32[$iGe>>2] = $62;
    }
   }
   $63 = HEAP32[$iCol>>2]|0;
   $64 = ($63|0)==(4);
   $65 = HEAP32[$op>>2]|0;
   $66 = ($65|0)==(2);
   $or$cond = $64 & $66;
   if ($or$cond) {
    $67 = HEAP32[$i>>2]|0;
    HEAP32[$iLangid>>2] = $67;
   }
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = (($68) + 1)|0;
  HEAP32[$i>>2] = $69;
 }
 $70 = HEAP32[$iEq>>2]|0;
 $71 = ($70|0)>=(0);
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 20|0);
 if ($71) {
  HEAP32[$73>>2] = 1;
  $74 = HEAP32[$iNext>>2]|0;
  $75 = (($74) + 1)|0;
  HEAP32[$iNext>>2] = $75;
  $76 = HEAP32[$iEq>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 16|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (($79) + ($76<<3)|0);
  HEAP32[$80>>2] = $74;
  $81 = HEAP32[$1>>2]|0;
  $82 = ((($81)) + 40|0);
  HEAPF64[$82>>3] = 5.0;
 } else {
  HEAP32[$73>>2] = 0;
  $83 = HEAP32[$1>>2]|0;
  $84 = ((($83)) + 40|0);
  HEAPF64[$84>>3] = 2.0E+4;
  $85 = HEAP32[$iGe>>2]|0;
  $86 = ($85|0)>=(0);
  if ($86) {
   $87 = HEAP32[$1>>2]|0;
   $88 = ((($87)) + 20|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + 2)|0;
   HEAP32[$88>>2] = $90;
   $91 = HEAP32[$iNext>>2]|0;
   $92 = (($91) + 1)|0;
   HEAP32[$iNext>>2] = $92;
   $93 = HEAP32[$iGe>>2]|0;
   $94 = HEAP32[$1>>2]|0;
   $95 = ((($94)) + 16|0);
   $96 = HEAP32[$95>>2]|0;
   $97 = (($96) + ($93<<3)|0);
   HEAP32[$97>>2] = $91;
   $98 = HEAP32[$1>>2]|0;
   $99 = ((($98)) + 40|0);
   $100 = +HEAPF64[$99>>3];
   $101 = $100 / 2.0;
   HEAPF64[$99>>3] = $101;
  }
  $102 = HEAP32[$iLe>>2]|0;
  $103 = ($102|0)>=(0);
  if ($103) {
   $104 = HEAP32[$1>>2]|0;
   $105 = ((($104)) + 20|0);
   $106 = HEAP32[$105>>2]|0;
   $107 = (($106) + 4)|0;
   HEAP32[$105>>2] = $107;
   $108 = HEAP32[$iNext>>2]|0;
   $109 = (($108) + 1)|0;
   HEAP32[$iNext>>2] = $109;
   $110 = HEAP32[$iLe>>2]|0;
   $111 = HEAP32[$1>>2]|0;
   $112 = ((($111)) + 16|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = (($113) + ($110<<3)|0);
   HEAP32[$114>>2] = $108;
   $115 = HEAP32[$1>>2]|0;
   $116 = ((($115)) + 40|0);
   $117 = +HEAPF64[$116>>3];
   $118 = $117 / 2.0;
   HEAPF64[$116>>3] = $118;
  }
 }
 $119 = HEAP32[$iLangid>>2]|0;
 $120 = ($119|0)>=(0);
 if (!($120)) {
  STACKTOP = sp;return 0;
 }
 $121 = HEAP32[$iNext>>2]|0;
 $122 = (($121) + 1)|0;
 HEAP32[$iNext>>2] = $122;
 $123 = HEAP32[$iLangid>>2]|0;
 $124 = HEAP32[$1>>2]|0;
 $125 = ((($124)) + 16|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = (($126) + ($123<<3)|0);
 HEAP32[$127>>2] = $121;
 $128 = HEAP32[$1>>2]|0;
 $129 = ((($128)) + 40|0);
 $130 = +HEAPF64[$129>>3];
 $131 = $130 + -1.0;
 HEAPF64[$129>>3] = $131;
 STACKTOP = sp;return 0;
}
function _fts3auxDisconnectMethod($pVtab) {
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0;
 var $pFts3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $p = sp + 8|0;
 $pFts3 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pVtab;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pFts3>>2] = $4;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = ($5|0)<(40);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$pFts3>>2]|0;
  $9 = ((($8)) + 56|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  (_sqlite3_finalize($11)|0);
  $12 = HEAP32[$i>>2]|0;
  $13 = (($12) + 1)|0;
  HEAP32[$i>>2] = $13;
 }
 $14 = HEAP32[$pFts3>>2]|0;
 $15 = ((($14)) + 240|0);
 $16 = HEAP32[$15>>2]|0;
 _sqlite3_free($16);
 $17 = HEAP32[$p>>2]|0;
 _sqlite3_free($17);
 STACKTOP = sp;return 0;
}
function _fts3auxOpenMethod($pVTab,$ppCsr) {
 $pVTab = $pVTab|0;
 $ppCsr = $ppCsr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $pVTab;
 HEAP32[$2>>2] = $ppCsr;
 $3 = (_sqlite3_malloc(120)|0);
 HEAP32[$pCsr>>2] = $3;
 $4 = HEAP32[$pCsr>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$pCsr>>2]|0;
  dest=$6; stop=dest+120|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $7 = HEAP32[$pCsr>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  HEAP32[$8>>2] = $7;
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  HEAP32[$0>>2] = 7;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _fts3auxCloseMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0;
 var $pFts3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pFts3 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pFts3>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $5;
 $6 = HEAP32[$pFts3>>2]|0;
 _sqlite3Fts3SegmentsClose($6);
 $7 = HEAP32[$pCsr>>2]|0;
 $8 = ((($7)) + 4|0);
 _sqlite3Fts3SegReaderFinish($8);
 $9 = HEAP32[$pCsr>>2]|0;
 $10 = ((($9)) + 60|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3_free($11);
 $12 = HEAP32[$pCsr>>2]|0;
 $13 = ((($12)) + 76|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3_free($14);
 $15 = HEAP32[$pCsr>>2]|0;
 $16 = ((($15)) + 112|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3_free($17);
 $18 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($18);
 STACKTOP = sp;return 0;
}
function _fts3auxFilterMethod($pCursor,$idxNum,$idxStr,$nVal,$apVal) {
 $pCursor = $pCursor|0;
 $idxNum = $idxNum|0;
 $idxStr = $idxStr|0;
 $nVal = $nVal|0;
 $apVal = $apVal|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $iEq = 0, $iGe = 0, $iLangVal = 0, $iLangid = 0, $iLe = 0, $iNext = 0, $isScan = 0, $or$cond = 0, $pCsr = 0, $pFts3 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $zStr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $pCsr = sp + 52|0;
 $pFts3 = sp + 48|0;
 $rc = sp + 44|0;
 $isScan = sp + 40|0;
 $iLangVal = sp + 36|0;
 $iEq = sp + 32|0;
 $iGe = sp + 28|0;
 $iLe = sp + 24|0;
 $iLangid = sp + 20|0;
 $iNext = sp + 16|0;
 $zStr = sp + 12|0;
 HEAP32[$1>>2] = $pCursor;
 HEAP32[$2>>2] = $idxNum;
 HEAP32[$3>>2] = $idxStr;
 HEAP32[$4>>2] = $nVal;
 HEAP32[$5>>2] = $apVal;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pFts3>>2] = $10;
 HEAP32[$isScan>>2] = 0;
 HEAP32[$iLangVal>>2] = 0;
 HEAP32[$iEq>>2] = -1;
 HEAP32[$iGe>>2] = -1;
 HEAP32[$iLe>>2] = -1;
 HEAP32[$iLangid>>2] = -1;
 HEAP32[$iNext>>2] = 0;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(1);
 if ($12) {
  $13 = HEAP32[$iNext>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$iNext>>2] = $14;
  HEAP32[$iEq>>2] = $13;
 } else {
  HEAP32[$isScan>>2] = 1;
  $15 = HEAP32[$2>>2]|0;
  $16 = $15 & 2;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$iNext>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$iNext>>2] = $19;
   HEAP32[$iGe>>2] = $18;
  }
  $20 = HEAP32[$2>>2]|0;
  $21 = $20 & 4;
  $22 = ($21|0)!=(0);
  if ($22) {
   $23 = HEAP32[$iNext>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$iNext>>2] = $24;
   HEAP32[$iLe>>2] = $23;
  }
 }
 $25 = HEAP32[$iNext>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 $27 = ($25|0)<($26|0);
 if ($27) {
  $28 = HEAP32[$iNext>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$iNext>>2] = $29;
  HEAP32[$iLangid>>2] = $28;
 }
 $30 = HEAP32[$pCsr>>2]|0;
 $31 = ((($30)) + 4|0);
 _sqlite3Fts3SegReaderFinish($31);
 $32 = HEAP32[$pCsr>>2]|0;
 $33 = ((($32)) + 60|0);
 $34 = HEAP32[$33>>2]|0;
 _sqlite3_free($34);
 $35 = HEAP32[$pCsr>>2]|0;
 $36 = ((($35)) + 112|0);
 $37 = HEAP32[$36>>2]|0;
 _sqlite3_free($37);
 $38 = HEAP32[$pCsr>>2]|0;
 $39 = ((($38)) + 4|0);
 $40 = HEAP32[$pCsr>>2]|0;
 $41 = ((($40)) + 120|0);
 $42 = HEAP32[$pCsr>>2]|0;
 $43 = ((($42)) + 4|0);
 $44 = $41;
 $45 = $43;
 $46 = (($44) - ($45))|0;
 _memset(($39|0),0,($46|0))|0;
 $47 = HEAP32[$pCsr>>2]|0;
 $48 = ((($47)) + 60|0);
 $49 = ((($48)) + 12|0);
 HEAP32[$49>>2] = 3;
 $50 = HEAP32[$isScan>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = HEAP32[$pCsr>>2]|0;
  $53 = ((($52)) + 60|0);
  $54 = ((($53)) + 12|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = $55 | 16;
  HEAP32[$54>>2] = $56;
 }
 $57 = HEAP32[$iEq>>2]|0;
 $58 = ($57|0)>=(0);
 $59 = HEAP32[$iGe>>2]|0;
 $60 = ($59|0)>=(0);
 $or$cond = $58 | $60;
 if ($or$cond) {
  $61 = HEAP32[$5>>2]|0;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_sqlite3_value_text($62)|0);
  HEAP32[$zStr>>2] = $63;
  $64 = HEAP32[$zStr>>2]|0;
  $65 = ($64|0)!=(0|0);
  if ($65) {
   $66 = HEAP32[$zStr>>2]|0;
   HEAP32[$vararg_buffer>>2] = $66;
   $67 = (_sqlite3_mprintf(18391,$vararg_buffer)|0);
   $68 = HEAP32[$pCsr>>2]|0;
   $69 = ((($68)) + 60|0);
   HEAP32[$69>>2] = $67;
   $70 = HEAP32[$5>>2]|0;
   $71 = HEAP32[$70>>2]|0;
   $72 = (_sqlite3_value_bytes($71)|0);
   $73 = HEAP32[$pCsr>>2]|0;
   $74 = ((($73)) + 60|0);
   $75 = ((($74)) + 4|0);
   HEAP32[$75>>2] = $72;
   $76 = HEAP32[$pCsr>>2]|0;
   $77 = ((($76)) + 60|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ($78|0)==(0|0);
   if ($79) {
    HEAP32[$0>>2] = 7;
    $139 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($139|0);
   }
  }
 }
 $80 = HEAP32[$iLe>>2]|0;
 $81 = ($80|0)>=(0);
 if ($81) {
  $82 = HEAP32[$iLe>>2]|0;
  $83 = HEAP32[$5>>2]|0;
  $84 = (($83) + ($82<<2)|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (_sqlite3_value_text($85)|0);
  HEAP32[$vararg_buffer2>>2] = $86;
  $87 = (_sqlite3_mprintf(18391,$vararg_buffer2)|0);
  $88 = HEAP32[$pCsr>>2]|0;
  $89 = ((($88)) + 76|0);
  HEAP32[$89>>2] = $87;
  $90 = HEAP32[$iLe>>2]|0;
  $91 = HEAP32[$5>>2]|0;
  $92 = (($91) + ($90<<2)|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (_sqlite3_value_bytes($93)|0);
  $95 = HEAP32[$pCsr>>2]|0;
  $96 = ((($95)) + 80|0);
  HEAP32[$96>>2] = $94;
  $97 = HEAP32[$pCsr>>2]|0;
  $98 = ((($97)) + 76|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = ($99|0)==(0|0);
  if ($100) {
   HEAP32[$0>>2] = 7;
   $139 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($139|0);
  }
 }
 $101 = HEAP32[$iLangid>>2]|0;
 $102 = ($101|0)>=(0);
 if ($102) {
  $103 = HEAP32[$iLangid>>2]|0;
  $104 = HEAP32[$5>>2]|0;
  $105 = (($104) + ($103<<2)|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = (_sqlite3_value_int($106)|0);
  HEAP32[$iLangVal>>2] = $107;
  $108 = HEAP32[$iLangVal>>2]|0;
  $109 = ($108|0)<(0);
  $$ = $109 ? 0 : $107;
  HEAP32[$iLangVal>>2] = $$;
 }
 $110 = HEAP32[$iLangVal>>2]|0;
 $111 = HEAP32[$pCsr>>2]|0;
 $112 = ((($111)) + 84|0);
 HEAP32[$112>>2] = $110;
 $113 = HEAP32[$pFts3>>2]|0;
 $114 = HEAP32[$iLangVal>>2]|0;
 $115 = HEAP32[$pCsr>>2]|0;
 $116 = ((($115)) + 60|0);
 $117 = HEAP32[$116>>2]|0;
 $118 = HEAP32[$pCsr>>2]|0;
 $119 = ((($118)) + 60|0);
 $120 = ((($119)) + 4|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$isScan>>2]|0;
 $123 = HEAP32[$pCsr>>2]|0;
 $124 = ((($123)) + 4|0);
 $125 = (_sqlite3Fts3SegReaderCursor($113,$114,0,-2,$117,$121,0,$122,$124)|0);
 HEAP32[$rc>>2] = $125;
 $126 = HEAP32[$rc>>2]|0;
 $127 = ($126|0)==(0);
 if ($127) {
  $128 = HEAP32[$pFts3>>2]|0;
  $129 = HEAP32[$pCsr>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$pCsr>>2]|0;
  $132 = ((($131)) + 60|0);
  $133 = (_sqlite3Fts3SegReaderStart($128,$130,$132)|0);
  HEAP32[$rc>>2] = $133;
 }
 $134 = HEAP32[$rc>>2]|0;
 $135 = ($134|0)==(0);
 if ($135) {
  $136 = HEAP32[$1>>2]|0;
  $137 = (_fts3auxNextMethod($136)|0);
  HEAP32[$rc>>2] = $137;
 }
 $138 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $138;
 $139 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($139|0);
}
function _fts3auxNextMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aDoclist = 0, $eState = 0, $i = 0, $iCol = 0, $mc = 0, $n = 0, $nDoclist = 0, $pCsr = 0, $pFts3 = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $pCsr = sp + 44|0;
 $pFts3 = sp + 40|0;
 $rc = sp + 36|0;
 $i = sp + 32|0;
 $nDoclist = sp + 28|0;
 $aDoclist = sp + 24|0;
 $iCol = sp + 20|0;
 $eState = sp + 16|0;
 $n = sp + 12|0;
 $mc = sp + 8|0;
 $v = sp;
 HEAP32[$1>>2] = $pCursor;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pFts3>>2] = $6;
 $7 = HEAP32[$pCsr>>2]|0;
 $8 = ((($7)) + 96|0);
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_i64Add(($11|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = $8;
 $18 = $17;
 HEAP32[$18>>2] = $15;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $16;
 $21 = HEAP32[$pCsr>>2]|0;
 $22 = ((($21)) + 104|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 1)|0;
 HEAP32[$22>>2] = $24;
 while(1) {
  $25 = HEAP32[$pCsr>>2]|0;
  $26 = ((($25)) + 104|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$pCsr>>2]|0;
  $29 = ((($28)) + 108|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($27|0)<($30|0);
  if (!($31)) {
   break;
  }
  $32 = HEAP32[$pCsr>>2]|0;
  $33 = ((($32)) + 104|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$pCsr>>2]|0;
  $36 = ((($35)) + 112|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<4)|0);
  $39 = $38;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (($39) + 4)|0;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = ($44|0)>(0);
  $46 = ($41>>>0)>(0);
  $47 = ($44|0)==(0);
  $48 = $47 & $46;
  $49 = $45 | $48;
  if ($49) {
   label = 4;
   break;
  }
  $50 = HEAP32[$pCsr>>2]|0;
  $51 = ((($50)) + 104|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$51>>2] = $53;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 $54 = HEAP32[$pFts3>>2]|0;
 $55 = HEAP32[$pCsr>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = (_sqlite3Fts3SegReaderStep($54,$56)|0);
 HEAP32[$rc>>2] = $57;
 $58 = HEAP32[$rc>>2]|0;
 $59 = ($58|0)==(100);
 do {
  if ($59) {
   HEAP32[$i>>2] = 0;
   $60 = HEAP32[$pCsr>>2]|0;
   $61 = ((($60)) + 4|0);
   $62 = ((($61)) + 52|0);
   $63 = HEAP32[$62>>2]|0;
   HEAP32[$nDoclist>>2] = $63;
   $64 = HEAP32[$pCsr>>2]|0;
   $65 = ((($64)) + 4|0);
   $66 = ((($65)) + 48|0);
   $67 = HEAP32[$66>>2]|0;
   HEAP32[$aDoclist>>2] = $67;
   HEAP32[$eState>>2] = 0;
   $68 = HEAP32[$pCsr>>2]|0;
   $69 = ((($68)) + 76|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = ($70|0)!=(0|0);
   do {
    if ($71) {
     $72 = HEAP32[$pCsr>>2]|0;
     $73 = ((($72)) + 80|0);
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$pCsr>>2]|0;
     $76 = ((($75)) + 4|0);
     $77 = ((($76)) + 44|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = ($74|0)<($78|0);
     $80 = HEAP32[$pCsr>>2]|0;
     if ($79) {
      $81 = ((($80)) + 80|0);
      $82 = HEAP32[$81>>2]|0;
      $86 = $82;
     } else {
      $83 = ((($80)) + 4|0);
      $84 = ((($83)) + 44|0);
      $85 = HEAP32[$84>>2]|0;
      $86 = $85;
     }
     HEAP32[$n>>2] = $86;
     $87 = HEAP32[$pCsr>>2]|0;
     $88 = ((($87)) + 76|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = HEAP32[$pCsr>>2]|0;
     $91 = ((($90)) + 4|0);
     $92 = ((($91)) + 40|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = HEAP32[$n>>2]|0;
     $95 = (_memcmp($89,$93,$94)|0);
     HEAP32[$mc>>2] = $95;
     $96 = HEAP32[$mc>>2]|0;
     $97 = ($96|0)<(0);
     if (!($97)) {
      $98 = HEAP32[$mc>>2]|0;
      $99 = ($98|0)==(0);
      if (!($99)) {
       break;
      }
      $100 = HEAP32[$pCsr>>2]|0;
      $101 = ((($100)) + 4|0);
      $102 = ((($101)) + 44|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = HEAP32[$pCsr>>2]|0;
      $105 = ((($104)) + 80|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = ($103|0)>($106|0);
      if (!($107)) {
       break;
      }
     }
     $108 = HEAP32[$pCsr>>2]|0;
     $109 = ((($108)) + 88|0);
     HEAP32[$109>>2] = 1;
     HEAP32[$0>>2] = 0;
     $263 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($263|0);
    }
   } while(0);
   $110 = HEAP32[$pCsr>>2]|0;
   $111 = (_fts3auxGrowStatArray($110,2)|0);
   $112 = ($111|0)!=(0);
   if ($112) {
    HEAP32[$0>>2] = 7;
    $263 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($263|0);
   }
   $113 = HEAP32[$pCsr>>2]|0;
   $114 = ((($113)) + 112|0);
   $115 = HEAP32[$114>>2]|0;
   $116 = HEAP32[$pCsr>>2]|0;
   $117 = ((($116)) + 108|0);
   $118 = HEAP32[$117>>2]|0;
   $119 = $118<<4;
   _memset(($115|0),0,($119|0))|0;
   HEAP32[$iCol>>2] = 0;
   L27: while(1) {
    $120 = HEAP32[$i>>2]|0;
    $121 = HEAP32[$nDoclist>>2]|0;
    $122 = ($120|0)<($121|0);
    if (!($122)) {
     label = 32;
     break;
    }
    $123 = $v;
    $124 = $123;
    HEAP32[$124>>2] = 0;
    $125 = (($123) + 4)|0;
    $126 = $125;
    HEAP32[$126>>2] = 0;
    $127 = HEAP32[$i>>2]|0;
    $128 = HEAP32[$aDoclist>>2]|0;
    $129 = (($128) + ($127)|0);
    $130 = (_sqlite3Fts3GetVarint($129,$v)|0);
    $131 = HEAP32[$i>>2]|0;
    $132 = (($131) + ($130))|0;
    HEAP32[$i>>2] = $132;
    $133 = HEAP32[$eState>>2]|0;
    switch ($133|0) {
    case 0:  {
     $134 = HEAP32[$pCsr>>2]|0;
     $135 = ((($134)) + 112|0);
     $136 = HEAP32[$135>>2]|0;
     $137 = $136;
     $138 = $137;
     $139 = HEAP32[$138>>2]|0;
     $140 = (($137) + 4)|0;
     $141 = $140;
     $142 = HEAP32[$141>>2]|0;
     $143 = (_i64Add(($139|0),($142|0),1,0)|0);
     $144 = tempRet0;
     $145 = $136;
     $146 = $145;
     HEAP32[$146>>2] = $143;
     $147 = (($145) + 4)|0;
     $148 = $147;
     HEAP32[$148>>2] = $144;
     HEAP32[$eState>>2] = 1;
     HEAP32[$iCol>>2] = 0;
     continue L27;
     break;
    }
    case 1:  {
     $149 = $v;
     $150 = $149;
     $151 = HEAP32[$150>>2]|0;
     $152 = (($149) + 4)|0;
     $153 = $152;
     $154 = HEAP32[$153>>2]|0;
     $155 = ($154|0)>(0);
     $156 = ($151>>>0)>(1);
     $157 = ($154|0)==(0);
     $158 = $157 & $156;
     $159 = $155 | $158;
     if ($159) {
      $160 = HEAP32[$pCsr>>2]|0;
      $161 = ((($160)) + 112|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = ((($162)) + 16|0);
      $164 = $163;
      $165 = $164;
      $166 = HEAP32[$165>>2]|0;
      $167 = (($164) + 4)|0;
      $168 = $167;
      $169 = HEAP32[$168>>2]|0;
      $170 = (_i64Add(($166|0),($169|0),1,0)|0);
      $171 = tempRet0;
      $172 = $163;
      $173 = $172;
      HEAP32[$173>>2] = $170;
      $174 = (($172) + 4)|0;
      $175 = $174;
      HEAP32[$175>>2] = $171;
     }
     HEAP32[$eState>>2] = 2;
     break;
    }
    case 2:  {
     break;
    }
    default: {
     $229 = $v;
     $230 = $229;
     $231 = HEAP32[$230>>2]|0;
     $232 = (($229) + 4)|0;
     $233 = $232;
     $234 = HEAP32[$233>>2]|0;
     HEAP32[$iCol>>2] = $231;
     $235 = HEAP32[$pCsr>>2]|0;
     $236 = HEAP32[$iCol>>2]|0;
     $237 = (($236) + 2)|0;
     $238 = (_fts3auxGrowStatArray($235,$237)|0);
     $239 = ($238|0)!=(0);
     if ($239) {
      break L27;
     }
     $240 = HEAP32[$iCol>>2]|0;
     $241 = (($240) + 1)|0;
     $242 = HEAP32[$pCsr>>2]|0;
     $243 = ((($242)) + 112|0);
     $244 = HEAP32[$243>>2]|0;
     $245 = (($244) + ($241<<4)|0);
     $246 = $245;
     $247 = $246;
     $248 = HEAP32[$247>>2]|0;
     $249 = (($246) + 4)|0;
     $250 = $249;
     $251 = HEAP32[$250>>2]|0;
     $252 = (_i64Add(($248|0),($251|0),1,0)|0);
     $253 = tempRet0;
     $254 = $245;
     $255 = $254;
     HEAP32[$255>>2] = $252;
     $256 = (($254) + 4)|0;
     $257 = $256;
     HEAP32[$257>>2] = $253;
     HEAP32[$eState>>2] = 2;
     continue L27;
    }
    }
    $176 = $v;
    $177 = $176;
    $178 = HEAP32[$177>>2]|0;
    $179 = (($176) + 4)|0;
    $180 = $179;
    $181 = HEAP32[$180>>2]|0;
    $182 = ($178|0)==(0);
    $183 = ($181|0)==(0);
    $184 = $182 & $183;
    if ($184) {
     HEAP32[$eState>>2] = 0;
     continue;
    }
    $185 = $v;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = (($185) + 4)|0;
    $189 = $188;
    $190 = HEAP32[$189>>2]|0;
    $191 = ($187|0)==(1);
    $192 = ($190|0)==(0);
    $193 = $191 & $192;
    if ($193) {
     HEAP32[$eState>>2] = 3;
     continue;
    } else {
     $194 = HEAP32[$iCol>>2]|0;
     $195 = (($194) + 1)|0;
     $196 = HEAP32[$pCsr>>2]|0;
     $197 = ((($196)) + 112|0);
     $198 = HEAP32[$197>>2]|0;
     $199 = (($198) + ($195<<4)|0);
     $200 = ((($199)) + 8|0);
     $201 = $200;
     $202 = $201;
     $203 = HEAP32[$202>>2]|0;
     $204 = (($201) + 4)|0;
     $205 = $204;
     $206 = HEAP32[$205>>2]|0;
     $207 = (_i64Add(($203|0),($206|0),1,0)|0);
     $208 = tempRet0;
     $209 = $200;
     $210 = $209;
     HEAP32[$210>>2] = $207;
     $211 = (($209) + 4)|0;
     $212 = $211;
     HEAP32[$212>>2] = $208;
     $213 = HEAP32[$pCsr>>2]|0;
     $214 = ((($213)) + 112|0);
     $215 = HEAP32[$214>>2]|0;
     $216 = ((($215)) + 8|0);
     $217 = $216;
     $218 = $217;
     $219 = HEAP32[$218>>2]|0;
     $220 = (($217) + 4)|0;
     $221 = $220;
     $222 = HEAP32[$221>>2]|0;
     $223 = (_i64Add(($219|0),($222|0),1,0)|0);
     $224 = tempRet0;
     $225 = $216;
     $226 = $225;
     HEAP32[$226>>2] = $223;
     $227 = (($225) + 4)|0;
     $228 = $227;
     HEAP32[$228>>2] = $224;
     continue;
    }
   }
   if ((label|0) == 32) {
    $258 = HEAP32[$pCsr>>2]|0;
    $259 = ((($258)) + 104|0);
    HEAP32[$259>>2] = 0;
    HEAP32[$rc>>2] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $263 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($263|0);
  } else {
   $260 = HEAP32[$pCsr>>2]|0;
   $261 = ((($260)) + 88|0);
   HEAP32[$261>>2] = 1;
  }
 } while(0);
 $262 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $262;
 $263 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($263|0);
}
function _fts3auxEofMethod($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 88|0);
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _fts3auxColumnMethod($pCursor,$pCtx,$iCol) {
 $pCursor = $pCursor|0;
 $pCtx = $pCtx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pCtx;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 switch ($4|0) {
 case 0:  {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = ((($7)) + 40|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = ((($11)) + 44|0);
  $13 = HEAP32[$12>>2]|0;
  _sqlite3_result_text($5,$9,$13,(-1));
  STACKTOP = sp;return 0;
  break;
 }
 case 1:  {
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 104|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0);
  $18 = HEAP32[$1>>2]|0;
  if ($17) {
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 104|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) - 1)|0;
   _sqlite3_result_int($18,$22);
   STACKTOP = sp;return 0;
  } else {
   _sqlite3_result_text($18,25451,-1,0);
   STACKTOP = sp;return 0;
  }
  break;
 }
 case 2:  {
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 104|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 112|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<4)|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  _sqlite3_result_int64($23,$33,$36);
  STACKTOP = sp;return 0;
  break;
 }
 case 3:  {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP32[$p>>2]|0;
  $39 = ((($38)) + 104|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 112|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + ($40<<4)|0);
  $45 = ((($44)) + 8|0);
  $46 = $45;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($46) + 4)|0;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  _sqlite3_result_int64($37,$48,$51);
  STACKTOP = sp;return 0;
  break;
 }
 default: {
  $52 = HEAP32[$1>>2]|0;
  $53 = HEAP32[$p>>2]|0;
  $54 = ((($53)) + 84|0);
  $55 = HEAP32[$54>>2]|0;
  _sqlite3_result_int($52,$55);
  STACKTOP = sp;return 0;
 }
 }
 return (0)|0;
}
function _fts3auxRowidMethod($pCursor,$pRowid) {
 $pCursor = $pCursor|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 HEAP32[$1>>2] = $pRowid;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $2;
 $3 = HEAP32[$pCsr>>2]|0;
 $4 = ((($3)) + 96|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _fts3auxGrowStatArray($pCsr,$nSize) {
 $pCsr = $pCsr|0;
 $nSize = $nSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $aNew = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $nSize;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)>($6|0);
 do {
  if ($7) {
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 112|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$2>>2]|0;
   $12 = $11<<4;
   $13 = (_sqlite3_realloc($10,$12)|0);
   HEAP32[$aNew>>2] = $13;
   $14 = HEAP32[$aNew>>2]|0;
   $15 = ($14|0)==(0|0);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 108|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$aNew>>2]|0;
    $20 = (($19) + ($18<<4)|0);
    $21 = HEAP32[$2>>2]|0;
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 108|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($21) - ($24))|0;
    $26 = $25<<4;
    _memset(($20|0),0,($26|0))|0;
    $27 = HEAP32[$aNew>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 112|0);
    HEAP32[$29>>2] = $27;
    $30 = HEAP32[$2>>2]|0;
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 108|0);
    HEAP32[$32>>2] = $30;
    break;
   }
   HEAP32[$0>>2] = 7;
   $33 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($33|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _unicodeCreate($nArg,$azArg,$pp) {
 $nArg = $nArg|0;
 $azArg = $azArg|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, $pNew = 0, $rc = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pNew = sp + 16|0;
 $i = sp + 12|0;
 $rc = sp + 8|0;
 $z = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $nArg;
 HEAP32[$2>>2] = $azArg;
 HEAP32[$3>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $4 = (_sqlite3_malloc(16)|0);
 HEAP32[$pNew>>2] = $4;
 $5 = HEAP32[$pNew>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 7;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 $7 = HEAP32[$pNew>>2]|0;
 ;HEAP32[$7>>2]=0|0;HEAP32[$7+4>>2]=0|0;HEAP32[$7+8>>2]=0|0;HEAP32[$7+12>>2]=0|0;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ((($8)) + 4|0);
 HEAP32[$9>>2] = 1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$z>>2] = $18;
  $19 = HEAP32[$z>>2]|0;
  $20 = (_strlen($19)|0);
  HEAP32[$n>>2] = $20;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($21|0)==(19);
  if ($22) {
   $23 = HEAP32[$z>>2]|0;
   $24 = (_memcmp(25728,$23,19)|0);
   $25 = ($24|0)==(0);
   if ($25) {
    $26 = HEAP32[$pNew>>2]|0;
    $27 = ((($26)) + 4|0);
    HEAP32[$27>>2] = 1;
   } else {
    label = 9;
   }
  } else {
   label = 9;
  }
  do {
   if ((label|0) == 9) {
    label = 0;
    $28 = HEAP32[$n>>2]|0;
    $29 = ($28|0)==(19);
    if ($29) {
     $30 = HEAP32[$z>>2]|0;
     $31 = (_memcmp(25748,$30,19)|0);
     $32 = ($31|0)==(0);
     if ($32) {
      $33 = HEAP32[$pNew>>2]|0;
      $34 = ((($33)) + 4|0);
      HEAP32[$34>>2] = 0;
      break;
     }
    }
    $35 = HEAP32[$n>>2]|0;
    $36 = ($35|0)>=(11);
    if ($36) {
     $37 = HEAP32[$z>>2]|0;
     $38 = (_memcmp(25768,$37,11)|0);
     $39 = ($38|0)==(0);
     if ($39) {
      $40 = HEAP32[$pNew>>2]|0;
      $41 = HEAP32[$z>>2]|0;
      $42 = ((($41)) + 11|0);
      $43 = HEAP32[$n>>2]|0;
      $44 = (($43) - 11)|0;
      $45 = (_unicodeAddExceptions($40,1,$42,$44)|0);
      HEAP32[$rc>>2] = $45;
      break;
     }
    }
    $46 = HEAP32[$n>>2]|0;
    $47 = ($46|0)>=(11);
    if ($47) {
     $48 = HEAP32[$z>>2]|0;
     $49 = (_memcmp(25780,$48,11)|0);
     $50 = ($49|0)==(0);
     if ($50) {
      $51 = HEAP32[$pNew>>2]|0;
      $52 = HEAP32[$z>>2]|0;
      $53 = ((($52)) + 11|0);
      $54 = HEAP32[$n>>2]|0;
      $55 = (($54) - 11)|0;
      $56 = (_unicodeAddExceptions($51,0,$53,$55)|0);
      HEAP32[$rc>>2] = $56;
      break;
     }
    }
    HEAP32[$rc>>2] = 1;
   }
  } while(0);
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$i>>2] = $58;
 }
 $59 = HEAP32[$rc>>2]|0;
 $60 = ($59|0)!=(0);
 if ($60) {
  $61 = HEAP32[$pNew>>2]|0;
  (_unicodeDestroy($61)|0);
  HEAP32[$pNew>>2] = 0;
 }
 $62 = HEAP32[$pNew>>2]|0;
 $63 = HEAP32[$3>>2]|0;
 HEAP32[$63>>2] = $62;
 $64 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $64;
 $65 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($65|0);
}
function _unicodeDestroy($pTokenizer) {
 $pTokenizer = $pTokenizer|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pTokenizer;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return 0;
 }
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3_free($6);
 $7 = HEAP32[$p>>2]|0;
 _sqlite3_free($7);
 STACKTOP = sp;return 0;
}
function _unicodeOpen($p,$aInput,$nInput,$pp) {
 $p = $p|0;
 $aInput = $aInput|0;
 $nInput = $nInput|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $aInput;
 HEAP32[$3>>2] = $nInput;
 HEAP32[$4>>2] = $pp;
 $5 = (_sqlite3_malloc(28)|0);
 HEAP32[$pCsr>>2] = $5;
 $6 = HEAP32[$pCsr>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 7;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $8 = HEAP32[$pCsr>>2]|0;
 ;HEAP32[$8>>2]=0|0;HEAP32[$8+4>>2]=0|0;HEAP32[$8+8>>2]=0|0;HEAP32[$8+12>>2]=0|0;HEAP32[$8+16>>2]=0|0;HEAP32[$8+20>>2]=0|0;HEAP32[$8+24>>2]=0|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$pCsr>>2]|0;
 $11 = ((($10)) + 4|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(0|0);
 do {
  if ($13) {
   $14 = HEAP32[$pCsr>>2]|0;
   $15 = ((($14)) + 8|0);
   HEAP32[$15>>2] = 0;
  } else {
   $16 = HEAP32[$3>>2]|0;
   $17 = ($16|0)<(0);
   if ($17) {
    $18 = HEAP32[$2>>2]|0;
    $19 = (_strlen($18)|0);
    $20 = HEAP32[$pCsr>>2]|0;
    $21 = ((($20)) + 8|0);
    HEAP32[$21>>2] = $19;
    break;
   } else {
    $22 = HEAP32[$3>>2]|0;
    $23 = HEAP32[$pCsr>>2]|0;
    $24 = ((($23)) + 8|0);
    HEAP32[$24>>2] = $22;
    break;
   }
  }
 } while(0);
 $25 = HEAP32[$pCsr>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 HEAP32[$26>>2] = $25;
 HEAP32[$0>>2] = 0;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _unicodeClose($pCursor) {
 $pCursor = $pCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $pCsr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCsr = sp;
 HEAP32[$0>>2] = $pCursor;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCsr>>2] = $1;
 $2 = HEAP32[$pCsr>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _sqlite3_free($4);
 $5 = HEAP32[$pCsr>>2]|0;
 _sqlite3_free($5);
 STACKTOP = sp;return 0;
}
function _unicodeNext($pC,$paToken,$pnToken,$piStart,$piEnd,$piPos) {
 $pC = $pC|0;
 $paToken = $paToken|0;
 $pnToken = $pnToken|0;
 $piStart = $piStart|0;
 $piEnd = $piEnd|0;
 $piPos = $piPos|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iCode = 0, $iOut = 0, $p = 0, $pCsr = 0, $z = 0, $zEnd = 0;
 var $zNew = 0, $zOut = 0, $zStart = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $6 = sp + 40|0;
 $pCsr = sp + 36|0;
 $p = sp + 32|0;
 $iCode = sp + 28|0;
 $zOut = sp + 24|0;
 $z = sp + 20|0;
 $zStart = sp + 16|0;
 $zEnd = sp + 12|0;
 $zTerm = sp + 8|0;
 $iOut = sp + 4|0;
 $zNew = sp;
 HEAP32[$1>>2] = $pC;
 HEAP32[$2>>2] = $paToken;
 HEAP32[$3>>2] = $pnToken;
 HEAP32[$4>>2] = $piStart;
 HEAP32[$5>>2] = $piEnd;
 HEAP32[$6>>2] = $piPos;
 $7 = HEAP32[$1>>2]|0;
 HEAP32[$pCsr>>2] = $7;
 $8 = HEAP32[$pCsr>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 HEAP32[$iCode>>2] = 0;
 $10 = HEAP32[$pCsr>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pCsr>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12)|0);
 HEAP32[$z>>2] = $16;
 $17 = HEAP32[$z>>2]|0;
 HEAP32[$zStart>>2] = $17;
 $18 = HEAP32[$pCsr>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pCsr>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20)|0);
 HEAP32[$zTerm>>2] = $24;
 while(1) {
  $25 = HEAP32[$z>>2]|0;
  $26 = HEAP32[$zTerm>>2]|0;
  $27 = ($25>>>0)<($26>>>0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$z>>2]|0;
  $29 = ((($28)) + 1|0);
  HEAP32[$z>>2] = $29;
  $30 = HEAP8[$28>>0]|0;
  $31 = $30&255;
  HEAP32[$iCode>>2] = $31;
  $32 = HEAP32[$iCode>>2]|0;
  $33 = ($32|0)>=(192);
  do {
   if ($33) {
    $34 = HEAP32[$iCode>>2]|0;
    $35 = (($34) - 192)|0;
    $36 = (25563 + ($35)|0);
    $37 = HEAP8[$36>>0]|0;
    $38 = $37&255;
    HEAP32[$iCode>>2] = $38;
    while(1) {
     $39 = HEAP32[$z>>2]|0;
     $40 = HEAP32[$zTerm>>2]|0;
     $41 = ($39|0)!=($40|0);
     if ($41) {
      $42 = HEAP32[$z>>2]|0;
      $43 = HEAP8[$42>>0]|0;
      $44 = $43&255;
      $45 = $44 & 192;
      $46 = ($45|0)==(128);
      $292 = $46;
     } else {
      $292 = 0;
     }
     $47 = HEAP32[$iCode>>2]|0;
     if (!($292)) {
      break;
     }
     $48 = $47 << 6;
     $49 = HEAP32[$z>>2]|0;
     $50 = ((($49)) + 1|0);
     HEAP32[$z>>2] = $50;
     $51 = HEAP8[$49>>0]|0;
     $52 = $51&255;
     $53 = 63 & $52;
     $54 = (($48) + ($53))|0;
     HEAP32[$iCode>>2] = $54;
    }
    $55 = ($47|0)<(128);
    if (!($55)) {
     $56 = HEAP32[$iCode>>2]|0;
     $57 = $56 & -2048;
     $58 = ($57|0)==(55296);
     if (!($58)) {
      $59 = HEAP32[$iCode>>2]|0;
      $60 = $59 & -2;
      $61 = ($60|0)==(65534);
      if (!($61)) {
       break;
      }
     }
    }
    HEAP32[$iCode>>2] = 65533;
   }
  } while(0);
  $62 = HEAP32[$p>>2]|0;
  $63 = HEAP32[$iCode>>2]|0;
  $64 = (_unicodeIsAlnum($62,$63)|0);
  $65 = ($64|0)!=(0);
  if ($65) {
   break;
  }
  $66 = HEAP32[$z>>2]|0;
  HEAP32[$zStart>>2] = $66;
 }
 $67 = HEAP32[$zStart>>2]|0;
 $68 = HEAP32[$zTerm>>2]|0;
 $69 = ($67>>>0)>=($68>>>0);
 if ($69) {
  HEAP32[$0>>2] = 101;
  $291 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($291|0);
 }
 $70 = HEAP32[$pCsr>>2]|0;
 $71 = ((($70)) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 HEAP32[$zOut>>2] = $72;
 while(1) {
  $73 = HEAP32[$zOut>>2]|0;
  $74 = HEAP32[$pCsr>>2]|0;
  $75 = ((($74)) + 20|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = $73;
  $78 = $76;
  $79 = (($77) - ($78))|0;
  $80 = HEAP32[$pCsr>>2]|0;
  $81 = ((($80)) + 24|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) - 4)|0;
  $84 = ($79|0)>=($83|0);
  if ($84) {
   $85 = HEAP32[$pCsr>>2]|0;
   $86 = ((($85)) + 20|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$pCsr>>2]|0;
   $89 = ((($88)) + 24|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (($90) + 64)|0;
   $92 = (_sqlite3_realloc($87,$91)|0);
   HEAP32[$zNew>>2] = $92;
   $93 = HEAP32[$zNew>>2]|0;
   $94 = ($93|0)!=(0|0);
   if (!($94)) {
    label = 20;
    break;
   }
   $95 = HEAP32[$zOut>>2]|0;
   $96 = HEAP32[$pCsr>>2]|0;
   $97 = ((($96)) + 20|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = $95;
   $100 = $98;
   $101 = (($99) - ($100))|0;
   $102 = HEAP32[$zNew>>2]|0;
   $103 = (($102) + ($101)|0);
   HEAP32[$zOut>>2] = $103;
   $104 = HEAP32[$zNew>>2]|0;
   $105 = HEAP32[$pCsr>>2]|0;
   $106 = ((($105)) + 20|0);
   HEAP32[$106>>2] = $104;
   $107 = HEAP32[$pCsr>>2]|0;
   $108 = ((($107)) + 24|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + 64)|0;
   HEAP32[$108>>2] = $110;
  }
  $111 = HEAP32[$z>>2]|0;
  HEAP32[$zEnd>>2] = $111;
  $112 = HEAP32[$iCode>>2]|0;
  $113 = HEAP32[$p>>2]|0;
  $114 = ((($113)) + 4|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (_sqlite3FtsUnicodeFold($112,$115)|0);
  HEAP32[$iOut>>2] = $116;
  $117 = HEAP32[$iOut>>2]|0;
  $118 = ($117|0)!=(0);
  do {
   if ($118) {
    $119 = HEAP32[$iOut>>2]|0;
    $120 = ($119|0)<(128);
    $121 = HEAP32[$iOut>>2]|0;
    if ($120) {
     $122 = $121 & 255;
     $123 = $122&255;
     $124 = HEAP32[$zOut>>2]|0;
     $125 = ((($124)) + 1|0);
     HEAP32[$zOut>>2] = $125;
     HEAP8[$124>>0] = $123;
     break;
    }
    $126 = ($121|0)<(2048);
    $127 = HEAP32[$iOut>>2]|0;
    if ($126) {
     $128 = $127 >> 6;
     $129 = $128 & 31;
     $130 = $129&255;
     $131 = $130&255;
     $132 = (192 + ($131))|0;
     $133 = $132&255;
     $134 = HEAP32[$zOut>>2]|0;
     $135 = ((($134)) + 1|0);
     HEAP32[$zOut>>2] = $135;
     HEAP8[$134>>0] = $133;
     $136 = HEAP32[$iOut>>2]|0;
     $137 = $136 & 63;
     $138 = $137&255;
     $139 = $138&255;
     $140 = (128 + ($139))|0;
     $141 = $140&255;
     $142 = HEAP32[$zOut>>2]|0;
     $143 = ((($142)) + 1|0);
     HEAP32[$zOut>>2] = $143;
     HEAP8[$142>>0] = $141;
     break;
    }
    $144 = ($127|0)<(65536);
    $145 = HEAP32[$iOut>>2]|0;
    if ($144) {
     $146 = $145 >> 12;
     $147 = $146 & 15;
     $148 = $147&255;
     $149 = $148&255;
     $150 = (224 + ($149))|0;
     $151 = $150&255;
     $152 = HEAP32[$zOut>>2]|0;
     $153 = ((($152)) + 1|0);
     HEAP32[$zOut>>2] = $153;
     HEAP8[$152>>0] = $151;
     $154 = HEAP32[$iOut>>2]|0;
     $155 = $154 >> 6;
     $156 = $155 & 63;
     $157 = $156&255;
     $158 = $157&255;
     $159 = (128 + ($158))|0;
     $160 = $159&255;
     $161 = HEAP32[$zOut>>2]|0;
     $162 = ((($161)) + 1|0);
     HEAP32[$zOut>>2] = $162;
     HEAP8[$161>>0] = $160;
     $163 = HEAP32[$iOut>>2]|0;
     $164 = $163 & 63;
     $165 = $164&255;
     $166 = $165&255;
     $167 = (128 + ($166))|0;
     $168 = $167&255;
     $169 = HEAP32[$zOut>>2]|0;
     $170 = ((($169)) + 1|0);
     HEAP32[$zOut>>2] = $170;
     HEAP8[$169>>0] = $168;
     break;
    } else {
     $171 = $145 >> 18;
     $172 = $171 & 7;
     $173 = $172&255;
     $174 = $173&255;
     $175 = (240 + ($174))|0;
     $176 = $175&255;
     $177 = HEAP32[$zOut>>2]|0;
     $178 = ((($177)) + 1|0);
     HEAP32[$zOut>>2] = $178;
     HEAP8[$177>>0] = $176;
     $179 = HEAP32[$iOut>>2]|0;
     $180 = $179 >> 12;
     $181 = $180 & 63;
     $182 = $181&255;
     $183 = $182&255;
     $184 = (128 + ($183))|0;
     $185 = $184&255;
     $186 = HEAP32[$zOut>>2]|0;
     $187 = ((($186)) + 1|0);
     HEAP32[$zOut>>2] = $187;
     HEAP8[$186>>0] = $185;
     $188 = HEAP32[$iOut>>2]|0;
     $189 = $188 >> 6;
     $190 = $189 & 63;
     $191 = $190&255;
     $192 = $191&255;
     $193 = (128 + ($192))|0;
     $194 = $193&255;
     $195 = HEAP32[$zOut>>2]|0;
     $196 = ((($195)) + 1|0);
     HEAP32[$zOut>>2] = $196;
     HEAP8[$195>>0] = $194;
     $197 = HEAP32[$iOut>>2]|0;
     $198 = $197 & 63;
     $199 = $198&255;
     $200 = $199&255;
     $201 = (128 + ($200))|0;
     $202 = $201&255;
     $203 = HEAP32[$zOut>>2]|0;
     $204 = ((($203)) + 1|0);
     HEAP32[$zOut>>2] = $204;
     HEAP8[$203>>0] = $202;
     break;
    }
   }
  } while(0);
  $205 = HEAP32[$z>>2]|0;
  $206 = HEAP32[$zTerm>>2]|0;
  $207 = ($205>>>0)>=($206>>>0);
  if ($207) {
   label = 43;
   break;
  }
  $208 = HEAP32[$z>>2]|0;
  $209 = ((($208)) + 1|0);
  HEAP32[$z>>2] = $209;
  $210 = HEAP8[$208>>0]|0;
  $211 = $210&255;
  HEAP32[$iCode>>2] = $211;
  $212 = HEAP32[$iCode>>2]|0;
  $213 = ($212|0)>=(192);
  do {
   if ($213) {
    $214 = HEAP32[$iCode>>2]|0;
    $215 = (($214) - 192)|0;
    $216 = (25563 + ($215)|0);
    $217 = HEAP8[$216>>0]|0;
    $218 = $217&255;
    HEAP32[$iCode>>2] = $218;
    while(1) {
     $219 = HEAP32[$z>>2]|0;
     $220 = HEAP32[$zTerm>>2]|0;
     $221 = ($219|0)!=($220|0);
     if ($221) {
      $222 = HEAP32[$z>>2]|0;
      $223 = HEAP8[$222>>0]|0;
      $224 = $223&255;
      $225 = $224 & 192;
      $226 = ($225|0)==(128);
      $293 = $226;
     } else {
      $293 = 0;
     }
     $227 = HEAP32[$iCode>>2]|0;
     if (!($293)) {
      break;
     }
     $228 = $227 << 6;
     $229 = HEAP32[$z>>2]|0;
     $230 = ((($229)) + 1|0);
     HEAP32[$z>>2] = $230;
     $231 = HEAP8[$229>>0]|0;
     $232 = $231&255;
     $233 = 63 & $232;
     $234 = (($228) + ($233))|0;
     HEAP32[$iCode>>2] = $234;
    }
    $235 = ($227|0)<(128);
    if (!($235)) {
     $236 = HEAP32[$iCode>>2]|0;
     $237 = $236 & -2048;
     $238 = ($237|0)==(55296);
     if (!($238)) {
      $239 = HEAP32[$iCode>>2]|0;
      $240 = $239 & -2;
      $241 = ($240|0)==(65534);
      if (!($241)) {
       break;
      }
     }
    }
    HEAP32[$iCode>>2] = 65533;
   }
  } while(0);
  $242 = HEAP32[$p>>2]|0;
  $243 = HEAP32[$iCode>>2]|0;
  $244 = (_unicodeIsAlnum($242,$243)|0);
  $245 = ($244|0)!=(0);
  if ($245) {
   continue;
  }
  $246 = HEAP32[$iCode>>2]|0;
  $247 = (_sqlite3FtsUnicodeIsdiacritic($246)|0);
  $248 = ($247|0)!=(0);
  if (!($248)) {
   label = 43;
   break;
  }
 }
 if ((label|0) == 20) {
  HEAP32[$0>>2] = 7;
  $291 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($291|0);
 }
 else if ((label|0) == 43) {
  $249 = HEAP32[$z>>2]|0;
  $250 = HEAP32[$pCsr>>2]|0;
  $251 = ((($250)) + 4|0);
  $252 = HEAP32[$251>>2]|0;
  $253 = $249;
  $254 = $252;
  $255 = (($253) - ($254))|0;
  $256 = HEAP32[$pCsr>>2]|0;
  $257 = ((($256)) + 12|0);
  HEAP32[$257>>2] = $255;
  $258 = HEAP32[$pCsr>>2]|0;
  $259 = ((($258)) + 20|0);
  $260 = HEAP32[$259>>2]|0;
  $261 = HEAP32[$2>>2]|0;
  HEAP32[$261>>2] = $260;
  $262 = HEAP32[$zOut>>2]|0;
  $263 = HEAP32[$pCsr>>2]|0;
  $264 = ((($263)) + 20|0);
  $265 = HEAP32[$264>>2]|0;
  $266 = $262;
  $267 = $265;
  $268 = (($266) - ($267))|0;
  $269 = HEAP32[$3>>2]|0;
  HEAP32[$269>>2] = $268;
  $270 = HEAP32[$zStart>>2]|0;
  $271 = HEAP32[$pCsr>>2]|0;
  $272 = ((($271)) + 4|0);
  $273 = HEAP32[$272>>2]|0;
  $274 = $270;
  $275 = $273;
  $276 = (($274) - ($275))|0;
  $277 = HEAP32[$4>>2]|0;
  HEAP32[$277>>2] = $276;
  $278 = HEAP32[$zEnd>>2]|0;
  $279 = HEAP32[$pCsr>>2]|0;
  $280 = ((($279)) + 4|0);
  $281 = HEAP32[$280>>2]|0;
  $282 = $278;
  $283 = $281;
  $284 = (($282) - ($283))|0;
  $285 = HEAP32[$5>>2]|0;
  HEAP32[$285>>2] = $284;
  $286 = HEAP32[$pCsr>>2]|0;
  $287 = ((($286)) + 16|0);
  $288 = HEAP32[$287>>2]|0;
  $289 = (($288) + 1)|0;
  HEAP32[$287>>2] = $289;
  $290 = HEAP32[$6>>2]|0;
  HEAP32[$290>>2] = $288;
  HEAP32[$0>>2] = 0;
  $291 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($291|0);
 }
 return (0)|0;
}
function _unicodeIsAlnum($p,$iCode) {
 $p = $p|0;
 $iCode = $iCode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCode;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3FtsUnicodeIsalnum($2)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_unicodeIsException($4,$5)|0);
 $7 = $3 ^ $6;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3FtsUnicodeFold($c,$bRemoveDiacritic) {
 $c = $c|0;
 $bRemoveDiacritic = $bRemoveDiacritic|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $cmp = 0;
 var $iHi = 0, $iLo = 0, $iRes = 0, $iTest = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $ret = sp + 24|0;
 $iHi = sp + 20|0;
 $iLo = sp + 16|0;
 $iRes = sp + 12|0;
 $iTest = sp + 8|0;
 $cmp = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $c;
 HEAP32[$1>>2] = $bRemoveDiacritic;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$ret>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)<(128);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ($5|0)>=(65);
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)<=(90);
  $or$cond = $6 & $8;
  if (!($or$cond)) {
   $78 = HEAP32[$ret>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = (($9) + 32)|0;
  HEAP32[$ret>>2] = $10;
  $78 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $11 = ($5|0)<(65536);
 if (!($11)) {
  $72 = HEAP32[$0>>2]|0;
  $73 = ($72|0)>=(66560);
  $74 = HEAP32[$0>>2]|0;
  $75 = ($74|0)<(66600);
  $or$cond3 = $73 & $75;
  if (!($or$cond3)) {
   $78 = HEAP32[$ret>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
  $76 = HEAP32[$0>>2]|0;
  $77 = (($76) + 40)|0;
  HEAP32[$ret>>2] = $77;
  $78 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 HEAP32[$iHi>>2] = 162;
 HEAP32[$iLo>>2] = 0;
 HEAP32[$iRes>>2] = -1;
 while(1) {
  $12 = HEAP32[$iHi>>2]|0;
  $13 = HEAP32[$iLo>>2]|0;
  $14 = ($12|0)>=($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$iHi>>2]|0;
  $16 = HEAP32[$iLo>>2]|0;
  $17 = (($15) + ($16))|0;
  $18 = (($17|0) / 2)&-1;
  HEAP32[$iTest>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$iTest>>2]|0;
  $21 = (9798 + ($20<<2)|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22&65535;
  $24 = (($19) - ($23))|0;
  HEAP32[$cmp>>2] = $24;
  $25 = HEAP32[$cmp>>2]|0;
  $26 = ($25|0)>=(0);
  $27 = HEAP32[$iTest>>2]|0;
  if ($26) {
   HEAP32[$iRes>>2] = $27;
   $28 = HEAP32[$iTest>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$iLo>>2] = $29;
   continue;
  } else {
   $30 = (($27) - 1)|0;
   HEAP32[$iHi>>2] = $30;
   continue;
  }
 }
 $31 = HEAP32[$iRes>>2]|0;
 $32 = ($31|0)>=(0);
 if ($32) {
  $33 = HEAP32[$iRes>>2]|0;
  $34 = (9798 + ($33<<2)|0);
  HEAP32[$p>>2] = $34;
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$p>>2]|0;
  $37 = HEAP16[$36>>1]|0;
  $38 = $37&65535;
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 3|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41&255;
  $43 = (($38) + ($42))|0;
  $44 = ($35|0)<($43|0);
  if ($44) {
   $45 = HEAP32[$p>>2]|0;
   $46 = ((($45)) + 2|0);
   $47 = HEAP8[$46>>0]|0;
   $48 = $47&255;
   $49 = 1 & $48;
   $50 = HEAP32[$p>>2]|0;
   $51 = HEAP16[$50>>1]|0;
   $52 = $51&65535;
   $53 = HEAP32[$0>>2]|0;
   $54 = $52 ^ $53;
   $55 = $49 & $54;
   $56 = (0)==($55|0);
   if ($56) {
    $57 = HEAP32[$0>>2]|0;
    $58 = HEAP32[$p>>2]|0;
    $59 = ((($58)) + 2|0);
    $60 = HEAP8[$59>>0]|0;
    $61 = $60&255;
    $62 = $61 >> 1;
    $63 = (10450 + ($62<<1)|0);
    $64 = HEAP16[$63>>1]|0;
    $65 = $64&65535;
    $66 = (($57) + ($65))|0;
    $67 = $66 & 65535;
    HEAP32[$ret>>2] = $67;
   }
  }
 }
 $68 = HEAP32[$1>>2]|0;
 $69 = ($68|0)!=(0);
 if (!($69)) {
  $78 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $70 = HEAP32[$ret>>2]|0;
 $71 = (_remove_diacritic($70)|0);
 HEAP32[$ret>>2] = $71;
 $78 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3FtsUnicodeIsdiacritic($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $mask0 = 0, $mask1 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $mask0 = sp + 4|0;
 $mask1 = sp;
 HEAP32[$1>>2] = $c;
 HEAP32[$mask0>>2] = 134389727;
 HEAP32[$mask1>>2] = 221688;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)<(768);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)>(817);
 $or$cond = $3 | $5;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)<(800);
 if ($7) {
  $8 = HEAP32[$mask0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (($9) - 768)|0;
  $11 = 1 << $10;
  $12 = $8 & $11;
  $19 = $12;
 } else {
  $13 = HEAP32[$mask1>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (($14) - 768)|0;
  $16 = (($15) - 32)|0;
  $17 = 1 << $16;
  $18 = $13 & $17;
  $19 = $18;
 }
 HEAP32[$0>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _remove_diacritic($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $aChar = 0, $aDia = 0, $iHi = 0, $iLo = 0, $iRes = 0, $iTest = 0, $key = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0;
 $0 = sp + 20|0;
 $aDia = sp + 24|0;
 $aChar = sp + 226|0;
 $key = sp + 16|0;
 $iRes = sp + 12|0;
 $iHi = sp + 8|0;
 $iLo = sp + 4|0;
 $iTest = sp;
 HEAP32[$0>>2] = $c;
 _memcpy(($aDia|0),(10604|0),202)|0;
 dest=$aChar; src=25627; stop=dest+101|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 << 3;
 $3 = $2 | 7;
 HEAP32[$key>>2] = $3;
 HEAP32[$iRes>>2] = 0;
 HEAP32[$iHi>>2] = 100;
 HEAP32[$iLo>>2] = 0;
 while(1) {
  $4 = HEAP32[$iHi>>2]|0;
  $5 = HEAP32[$iLo>>2]|0;
  $6 = ($4|0)>=($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$iHi>>2]|0;
  $8 = HEAP32[$iLo>>2]|0;
  $9 = (($7) + ($8))|0;
  $10 = (($9|0) / 2)&-1;
  HEAP32[$iTest>>2] = $10;
  $11 = HEAP32[$key>>2]|0;
  $12 = HEAP32[$iTest>>2]|0;
  $13 = (($aDia) + ($12<<1)|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = ($11>>>0)>=($15>>>0);
  $17 = HEAP32[$iTest>>2]|0;
  if ($16) {
   HEAP32[$iRes>>2] = $17;
   $18 = HEAP32[$iTest>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$iLo>>2] = $19;
   continue;
  } else {
   $20 = (($17) - 1)|0;
   HEAP32[$iHi>>2] = $20;
   continue;
  }
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$iRes>>2]|0;
 $23 = (($aDia) + ($22<<1)|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = $25 >> 3;
 $27 = HEAP32[$iRes>>2]|0;
 $28 = (($aDia) + ($27<<1)|0);
 $29 = HEAP16[$28>>1]|0;
 $30 = $29&65535;
 $31 = $30 & 7;
 $32 = (($26) + ($31))|0;
 $33 = ($21|0)>($32|0);
 if ($33) {
  $34 = HEAP32[$0>>2]|0;
  $39 = $34;
  STACKTOP = sp;return ($39|0);
 } else {
  $35 = HEAP32[$iRes>>2]|0;
  $36 = (($aChar) + ($35)|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = $37 << 24 >> 24;
  $39 = $38;
  STACKTOP = sp;return ($39|0);
 }
 return (0)|0;
}
function _sqlite3FtsUnicodeIsalnum($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iHi = 0, $iLo = 0, $iRes = 0, $iTest = 0, $key = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $key = sp + 16|0;
 $iRes = sp + 12|0;
 $iHi = sp + 8|0;
 $iLo = sp + 4|0;
 $iTest = sp;
 HEAP32[$1>>2] = $c;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)<(128);
 $4 = HEAP32[$1>>2]|0;
 if ($3) {
  $5 = $4 >> 5;
  $6 = (2384 + ($5<<2)|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = $8 & 31;
  $10 = 1 << $9;
  $11 = $7 & $10;
  $12 = ($11|0)==(0);
  $13 = $12&1;
  HEAP32[$0>>2] = $13;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $14 = ($4|0)<(4194304);
 if (!($14)) {
  HEAP32[$0>>2] = 1;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = $15 << 10;
 $17 = $16 | 1023;
 HEAP32[$key>>2] = $17;
 HEAP32[$iRes>>2] = 0;
 HEAP32[$iHi>>2] = 405;
 HEAP32[$iLo>>2] = 0;
 while(1) {
  $18 = HEAP32[$iHi>>2]|0;
  $19 = HEAP32[$iLo>>2]|0;
  $20 = ($18|0)>=($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$iHi>>2]|0;
  $22 = HEAP32[$iLo>>2]|0;
  $23 = (($21) + ($22))|0;
  $24 = (($23|0) / 2)&-1;
  HEAP32[$iTest>>2] = $24;
  $25 = HEAP32[$key>>2]|0;
  $26 = HEAP32[$iTest>>2]|0;
  $27 = (2400 + ($26<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25>>>0)>=($28>>>0);
  $30 = HEAP32[$iTest>>2]|0;
  if ($29) {
   HEAP32[$iRes>>2] = $30;
   $31 = HEAP32[$iTest>>2]|0;
   $32 = (($31) + 1)|0;
   HEAP32[$iLo>>2] = $32;
   continue;
  } else {
   $33 = (($30) - 1)|0;
   HEAP32[$iHi>>2] = $33;
   continue;
  }
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = HEAP32[$iRes>>2]|0;
 $36 = (2400 + ($35<<2)|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $37 >>> 10;
 $39 = HEAP32[$iRes>>2]|0;
 $40 = (2400 + ($39<<2)|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $41 & 1023;
 $43 = (($38) + ($42))|0;
 $44 = ($34>>>0)>=($43>>>0);
 $45 = $44&1;
 HEAP32[$0>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _unicodeIsException($p,$iCode) {
 $p = $p|0;
 $iCode = $iCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $iHi = 0, $iLo = 0;
 var $iTest = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $a = sp + 12|0;
 $iLo = sp + 8|0;
 $iHi = sp + 4|0;
 $iTest = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iCode;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 L1: do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 12|0);
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$a>>2] = $9;
   HEAP32[$iLo>>2] = 0;
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 8|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = (($12) - 1)|0;
   HEAP32[$iHi>>2] = $13;
   while(1) {
    $14 = HEAP32[$iHi>>2]|0;
    $15 = HEAP32[$iLo>>2]|0;
    $16 = ($14|0)>=($15|0);
    if (!($16)) {
     break L1;
    }
    $17 = HEAP32[$iHi>>2]|0;
    $18 = HEAP32[$iLo>>2]|0;
    $19 = (($17) + ($18))|0;
    $20 = (($19|0) / 2)&-1;
    HEAP32[$iTest>>2] = $20;
    $21 = HEAP32[$2>>2]|0;
    $22 = HEAP32[$iTest>>2]|0;
    $23 = HEAP32[$a>>2]|0;
    $24 = (($23) + ($22<<2)|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($21|0)==($25|0);
    if ($26) {
     break;
    }
    $27 = HEAP32[$2>>2]|0;
    $28 = HEAP32[$iTest>>2]|0;
    $29 = HEAP32[$a>>2]|0;
    $30 = (($29) + ($28<<2)|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = ($27|0)>($31|0);
    $33 = HEAP32[$iTest>>2]|0;
    if ($32) {
     $34 = (($33) + 1)|0;
     HEAP32[$iLo>>2] = $34;
     continue;
    } else {
     $35 = (($33) - 1)|0;
     HEAP32[$iHi>>2] = $35;
     continue;
    }
   }
   HEAP32[$0>>2] = 1;
   $36 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _unicodeAddExceptions($p,$bAlnum,$zIn,$nIn) {
 $p = $p|0;
 $bAlnum = $bAlnum|0;
 $zIn = $zIn|0;
 $nIn = $nIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $i = 0, $iCode = 0;
 var $j = 0, $nEntry = 0, $nNew = 0, $z = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $z = sp + 28|0;
 $zTerm = sp + 24|0;
 $iCode = sp + 20|0;
 $nEntry = sp + 16|0;
 $aNew = sp + 12|0;
 $nNew = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $bAlnum;
 HEAP32[$3>>2] = $zIn;
 HEAP32[$4>>2] = $nIn;
 $5 = HEAP32[$3>>2]|0;
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$z>>2]|0;
 $8 = (($7) + ($6)|0);
 HEAP32[$zTerm>>2] = $8;
 HEAP32[$nEntry>>2] = 0;
 while(1) {
  $9 = HEAP32[$z>>2]|0;
  $10 = HEAP32[$zTerm>>2]|0;
  $11 = ($9>>>0)<($10>>>0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$z>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$z>>2] = $13;
  $14 = HEAP8[$12>>0]|0;
  $15 = $14&255;
  HEAP32[$iCode>>2] = $15;
  $16 = HEAP32[$iCode>>2]|0;
  $17 = ($16|0)>=(192);
  do {
   if ($17) {
    $18 = HEAP32[$iCode>>2]|0;
    $19 = (($18) - 192)|0;
    $20 = (25563 + ($19)|0);
    $21 = HEAP8[$20>>0]|0;
    $22 = $21&255;
    HEAP32[$iCode>>2] = $22;
    while(1) {
     $23 = HEAP32[$z>>2]|0;
     $24 = HEAP32[$zTerm>>2]|0;
     $25 = ($23|0)!=($24|0);
     if ($25) {
      $26 = HEAP32[$z>>2]|0;
      $27 = HEAP8[$26>>0]|0;
      $28 = $27&255;
      $29 = $28 & 192;
      $30 = ($29|0)==(128);
      $155 = $30;
     } else {
      $155 = 0;
     }
     $31 = HEAP32[$iCode>>2]|0;
     if (!($155)) {
      break;
     }
     $32 = $31 << 6;
     $33 = HEAP32[$z>>2]|0;
     $34 = ((($33)) + 1|0);
     HEAP32[$z>>2] = $34;
     $35 = HEAP8[$33>>0]|0;
     $36 = $35&255;
     $37 = 63 & $36;
     $38 = (($32) + ($37))|0;
     HEAP32[$iCode>>2] = $38;
    }
    $39 = ($31|0)<(128);
    if (!($39)) {
     $40 = HEAP32[$iCode>>2]|0;
     $41 = $40 & -2048;
     $42 = ($41|0)==(55296);
     if (!($42)) {
      $43 = HEAP32[$iCode>>2]|0;
      $44 = $43 & -2;
      $45 = ($44|0)==(65534);
      if (!($45)) {
       break;
      }
     }
    }
    HEAP32[$iCode>>2] = 65533;
   }
  } while(0);
  $46 = HEAP32[$iCode>>2]|0;
  $47 = (_sqlite3FtsUnicodeIsalnum($46)|0);
  $48 = HEAP32[$2>>2]|0;
  $49 = ($47|0)!=($48|0);
  if (!($49)) {
   continue;
  }
  $50 = HEAP32[$iCode>>2]|0;
  $51 = (_sqlite3FtsUnicodeIsdiacritic($50)|0);
  $52 = ($51|0)==(0);
  if (!($52)) {
   continue;
  }
  $53 = HEAP32[$nEntry>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$nEntry>>2] = $54;
 }
 $55 = HEAP32[$nEntry>>2]|0;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 8|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = HEAP32[$nEntry>>2]|0;
  $64 = (($62) + ($63))|0;
  $65 = $64<<2;
  $66 = (_sqlite3_realloc($59,$65)|0);
  HEAP32[$aNew>>2] = $66;
  $67 = HEAP32[$aNew>>2]|0;
  $68 = ($67|0)==(0|0);
  if ($68) {
   HEAP32[$0>>2] = 7;
   $154 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($154|0);
  }
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 8|0);
  $71 = HEAP32[$70>>2]|0;
  HEAP32[$nNew>>2] = $71;
  $72 = HEAP32[$3>>2]|0;
  HEAP32[$z>>2] = $72;
  while(1) {
   $73 = HEAP32[$z>>2]|0;
   $74 = HEAP32[$zTerm>>2]|0;
   $75 = ($73>>>0)<($74>>>0);
   if (!($75)) {
    break;
   }
   $76 = HEAP32[$z>>2]|0;
   $77 = ((($76)) + 1|0);
   HEAP32[$z>>2] = $77;
   $78 = HEAP8[$76>>0]|0;
   $79 = $78&255;
   HEAP32[$iCode>>2] = $79;
   $80 = HEAP32[$iCode>>2]|0;
   $81 = ($80|0)>=(192);
   do {
    if ($81) {
     $82 = HEAP32[$iCode>>2]|0;
     $83 = (($82) - 192)|0;
     $84 = (25563 + ($83)|0);
     $85 = HEAP8[$84>>0]|0;
     $86 = $85&255;
     HEAP32[$iCode>>2] = $86;
     while(1) {
      $87 = HEAP32[$z>>2]|0;
      $88 = HEAP32[$zTerm>>2]|0;
      $89 = ($87|0)!=($88|0);
      if ($89) {
       $90 = HEAP32[$z>>2]|0;
       $91 = HEAP8[$90>>0]|0;
       $92 = $91&255;
       $93 = $92 & 192;
       $94 = ($93|0)==(128);
       $156 = $94;
      } else {
       $156 = 0;
      }
      $95 = HEAP32[$iCode>>2]|0;
      if (!($156)) {
       break;
      }
      $96 = $95 << 6;
      $97 = HEAP32[$z>>2]|0;
      $98 = ((($97)) + 1|0);
      HEAP32[$z>>2] = $98;
      $99 = HEAP8[$97>>0]|0;
      $100 = $99&255;
      $101 = 63 & $100;
      $102 = (($96) + ($101))|0;
      HEAP32[$iCode>>2] = $102;
     }
     $103 = ($95|0)<(128);
     if (!($103)) {
      $104 = HEAP32[$iCode>>2]|0;
      $105 = $104 & -2048;
      $106 = ($105|0)==(55296);
      if (!($106)) {
       $107 = HEAP32[$iCode>>2]|0;
       $108 = $107 & -2;
       $109 = ($108|0)==(65534);
       if (!($109)) {
        break;
       }
      }
     }
     HEAP32[$iCode>>2] = 65533;
    }
   } while(0);
   $110 = HEAP32[$iCode>>2]|0;
   $111 = (_sqlite3FtsUnicodeIsalnum($110)|0);
   $112 = HEAP32[$2>>2]|0;
   $113 = ($111|0)!=($112|0);
   if (!($113)) {
    continue;
   }
   $114 = HEAP32[$iCode>>2]|0;
   $115 = (_sqlite3FtsUnicodeIsdiacritic($114)|0);
   $116 = ($115|0)==(0);
   if (!($116)) {
    continue;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $117 = HEAP32[$i>>2]|0;
    $118 = HEAP32[$nNew>>2]|0;
    $119 = ($117|0)<($118|0);
    if (!($119)) {
     break;
    }
    $120 = HEAP32[$i>>2]|0;
    $121 = HEAP32[$aNew>>2]|0;
    $122 = (($121) + ($120<<2)|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = HEAP32[$iCode>>2]|0;
    $125 = ($123|0)<($124|0);
    if (!($125)) {
     break;
    }
    $126 = HEAP32[$i>>2]|0;
    $127 = (($126) + 1)|0;
    HEAP32[$i>>2] = $127;
   }
   $128 = HEAP32[$nNew>>2]|0;
   HEAP32[$j>>2] = $128;
   while(1) {
    $129 = HEAP32[$j>>2]|0;
    $130 = HEAP32[$i>>2]|0;
    $131 = ($129|0)>($130|0);
    if (!($131)) {
     break;
    }
    $132 = HEAP32[$j>>2]|0;
    $133 = (($132) - 1)|0;
    $134 = HEAP32[$aNew>>2]|0;
    $135 = (($134) + ($133<<2)|0);
    $136 = HEAP32[$135>>2]|0;
    $137 = HEAP32[$j>>2]|0;
    $138 = HEAP32[$aNew>>2]|0;
    $139 = (($138) + ($137<<2)|0);
    HEAP32[$139>>2] = $136;
    $140 = HEAP32[$j>>2]|0;
    $141 = (($140) + -1)|0;
    HEAP32[$j>>2] = $141;
   }
   $142 = HEAP32[$iCode>>2]|0;
   $143 = HEAP32[$i>>2]|0;
   $144 = HEAP32[$aNew>>2]|0;
   $145 = (($144) + ($143<<2)|0);
   HEAP32[$145>>2] = $142;
   $146 = HEAP32[$nNew>>2]|0;
   $147 = (($146) + 1)|0;
   HEAP32[$nNew>>2] = $147;
  }
  $148 = HEAP32[$aNew>>2]|0;
  $149 = HEAP32[$1>>2]|0;
  $150 = ((($149)) + 12|0);
  HEAP32[$150>>2] = $148;
  $151 = HEAP32[$nNew>>2]|0;
  $152 = HEAP32[$1>>2]|0;
  $153 = ((($152)) + 8|0);
  HEAP32[$153>>2] = $151;
 }
 HEAP32[$0>>2] = 0;
 $154 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($154|0);
}
function _sqlite3BtreeSchema($p,$nBytes,$xFree) {
 $p = $p|0;
 $nBytes = $nBytes|0;
 $xFree = $xFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nBytes;
 HEAP32[$2>>2] = $xFree;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 48|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 $or$cond = $10 & $12;
 if (!($or$cond)) {
  $22 = HEAP32[$pBt>>2]|0;
  $23 = ((($22)) + 48|0);
  $24 = HEAP32[$23>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)<(0);
 $15 = $14 << 31 >> 31;
 $16 = (_sqlite3DbMallocZero(0,$13,$15)|0);
 $17 = HEAP32[$pBt>>2]|0;
 $18 = ((($17)) + 48|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$2>>2]|0;
 $20 = HEAP32[$pBt>>2]|0;
 $21 = ((($20)) + 52|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pBt>>2]|0;
 $23 = ((($22)) + 48|0);
 $24 = HEAP32[$23>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3TempInMemory($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 68|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(2);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3OsFullPathname($pVfs,$zPath,$nPathOut,$zPathOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $nPathOut = $nPathOut|0;
 $zPathOut = $zPathOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $nPathOut;
 HEAP32[$3>>2] = $zPathOut;
 $4 = HEAP32[$3>>2]|0;
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iiiii[$7 & 255]($8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _pageReinit($pData) {
 $pData = $pData|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pData;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3PagerGetExtra($1)|0);
 HEAP32[$pPage>>2] = $2;
 $3 = HEAP32[$pPage>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pPage>>2]|0;
 HEAP8[$6>>0] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3PagerPageRefcount($7)|0);
 $9 = ($8|0)>(1);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pPage>>2]|0;
 (_btreeInitPage($10)|0);
 STACKTOP = sp;return;
}
function _sqlite3PagerOpen($pVfs,$ppPager,$zFilename,$nExtra,$flags,$vfsFlags,$xReinit) {
 $pVfs = $pVfs|0;
 $ppPager = $ppPager|0;
 $zFilename = $zFilename|0;
 $nExtra = $nExtra|0;
 $flags = $flags|0;
 $vfsFlags = $vfsFlags|0;
 $xReinit = $xReinit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $fout = 0;
 var $iDc = 0, $journalFileSize = 0, $memDb = 0, $nPathname = 0, $nUri = 0, $pPager = 0, $pPtr = 0, $pcacheSize = 0, $rc = 0, $readOnly = 0, $szPageDflt = 0, $tempFile = 0, $useJournal = 0, $z = 0, $zPathname = 0, $zUri = 0, dest = 0, label = 0, sp = 0, src = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $7 = sp + 68|0;
 $pPtr = sp + 64|0;
 $pPager = sp + 60|0;
 $rc = sp + 56|0;
 $tempFile = sp + 52|0;
 $memDb = sp + 48|0;
 $readOnly = sp + 44|0;
 $journalFileSize = sp + 40|0;
 $zPathname = sp + 36|0;
 $nPathname = sp + 32|0;
 $useJournal = sp + 28|0;
 $pcacheSize = sp + 24|0;
 $szPageDflt = sp + 20|0;
 $zUri = sp + 16|0;
 $nUri = sp + 12|0;
 $z = sp + 8|0;
 $fout = sp + 4|0;
 $iDc = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $ppPager;
 HEAP32[$3>>2] = $zFilename;
 HEAP32[$4>>2] = $nExtra;
 HEAP32[$5>>2] = $flags;
 HEAP32[$6>>2] = $vfsFlags;
 HEAP32[$7>>2] = $xReinit;
 HEAP32[$pPager>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$tempFile>>2] = 0;
 HEAP32[$memDb>>2] = 0;
 HEAP32[$readOnly>>2] = 0;
 HEAP32[$zPathname>>2] = 0;
 HEAP32[$nPathname>>2] = 0;
 $8 = HEAP32[$5>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)==(0);
 $11 = $10&1;
 HEAP32[$useJournal>>2] = $11;
 $12 = (_sqlite3PcacheSize()|0);
 HEAP32[$pcacheSize>>2] = $12;
 HEAP32[$szPageDflt>>2] = 4096;
 HEAP32[$zUri>>2] = 0;
 HEAP32[$nUri>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_sqlite3JournalSize($13)|0);
 $15 = (($14) + 7)|0;
 $16 = $15 & -8;
 HEAP32[$journalFileSize>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$5>>2]|0;
 $19 = $18 & 2;
 $20 = ($19|0)!=(0);
 do {
  if ($20) {
   HEAP32[$memDb>>2] = 1;
   $21 = HEAP32[$3>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$3>>2]|0;
    $24 = HEAP8[$23>>0]|0;
    $25 = $24 << 24 >> 24;
    $26 = ($25|0)!=(0);
    if ($26) {
     $27 = HEAP32[$3>>2]|0;
     $28 = (_sqlite3DbStrDup(0,$27)|0);
     HEAP32[$zPathname>>2] = $28;
     $29 = HEAP32[$zPathname>>2]|0;
     $30 = ($29|0)==(0|0);
     if (!($30)) {
      $31 = HEAP32[$zPathname>>2]|0;
      $32 = (_sqlite3Strlen30($31)|0);
      HEAP32[$nPathname>>2] = $32;
      HEAP32[$3>>2] = 0;
      break;
     }
     HEAP32[$0>>2] = 7;
     $378 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($378|0);
    }
   }
  }
 } while(0);
 $33 = HEAP32[$3>>2]|0;
 $34 = ($33|0)!=(0|0);
 if ($34) {
  $35 = HEAP32[$3>>2]|0;
  $36 = HEAP8[$35>>0]|0;
  $37 = $36 << 24 >> 24;
  $38 = ($37|0)!=(0);
  if ($38) {
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 8|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (($41) + 1)|0;
   HEAP32[$nPathname>>2] = $42;
   $43 = HEAP32[$nPathname>>2]|0;
   $44 = $43<<1;
   $45 = ($44|0)<(0);
   $46 = $45 << 31 >> 31;
   $47 = (_sqlite3DbMallocRaw(0,$44,$46)|0);
   HEAP32[$zPathname>>2] = $47;
   $48 = HEAP32[$zPathname>>2]|0;
   $49 = ($48|0)==(0|0);
   if ($49) {
    HEAP32[$0>>2] = 7;
    $378 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($378|0);
   }
   $50 = HEAP32[$zPathname>>2]|0;
   HEAP8[$50>>0] = 0;
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$3>>2]|0;
   $53 = HEAP32[$nPathname>>2]|0;
   $54 = HEAP32[$zPathname>>2]|0;
   $55 = (_sqlite3OsFullPathname($51,$52,$53,$54)|0);
   HEAP32[$rc>>2] = $55;
   $56 = HEAP32[$zPathname>>2]|0;
   $57 = (_sqlite3Strlen30($56)|0);
   HEAP32[$nPathname>>2] = $57;
   $58 = HEAP32[$3>>2]|0;
   $59 = (_sqlite3Strlen30($58)|0);
   $60 = (($59) + 1)|0;
   $61 = HEAP32[$3>>2]|0;
   $62 = (($61) + ($60)|0);
   HEAP32[$zUri>>2] = $62;
   HEAP32[$z>>2] = $62;
   while(1) {
    $63 = HEAP32[$z>>2]|0;
    $64 = HEAP8[$63>>0]|0;
    $65 = ($64<<24>>24)!=(0);
    $66 = HEAP32[$z>>2]|0;
    if (!($65)) {
     break;
    }
    $67 = (_sqlite3Strlen30($66)|0);
    $68 = (($67) + 1)|0;
    $69 = HEAP32[$z>>2]|0;
    $70 = (($69) + ($68)|0);
    HEAP32[$z>>2] = $70;
    $71 = HEAP32[$z>>2]|0;
    $72 = (_sqlite3Strlen30($71)|0);
    $73 = (($72) + 1)|0;
    $74 = HEAP32[$z>>2]|0;
    $75 = (($74) + ($73)|0);
    HEAP32[$z>>2] = $75;
   }
   $76 = ((($66)) + 1|0);
   $77 = HEAP32[$zUri>>2]|0;
   $78 = $76;
   $79 = $77;
   $80 = (($78) - ($79))|0;
   HEAP32[$nUri>>2] = $80;
   $81 = HEAP32[$rc>>2]|0;
   $82 = ($81|0)==(0);
   if ($82) {
    $83 = HEAP32[$nPathname>>2]|0;
    $84 = (($83) + 8)|0;
    $85 = HEAP32[$1>>2]|0;
    $86 = ((($85)) + 8|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ($84|0)>($87|0);
    if ($88) {
     $89 = (_sqlite3CantopenError(51006)|0);
     HEAP32[$rc>>2] = $89;
    }
   }
   $90 = HEAP32[$rc>>2]|0;
   $91 = ($90|0)!=(0);
   if ($91) {
    $92 = HEAP32[$zPathname>>2]|0;
    _sqlite3DbFree(0,$92);
    $93 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $93;
    $378 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($378|0);
   }
  }
 }
 $94 = HEAP32[$pcacheSize>>2]|0;
 $95 = (($94) + 7)|0;
 $96 = $95 & -8;
 $97 = (224 + ($96))|0;
 $98 = HEAP32[$1>>2]|0;
 $99 = ((($98)) + 4|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = (($100) + 7)|0;
 $102 = $101 & -8;
 $103 = (($97) + ($102))|0;
 $104 = HEAP32[$journalFileSize>>2]|0;
 $105 = $104<<1;
 $106 = (($103) + ($105))|0;
 $107 = HEAP32[$nPathname>>2]|0;
 $108 = (($106) + ($107))|0;
 $109 = (($108) + 1)|0;
 $110 = HEAP32[$nUri>>2]|0;
 $111 = (($109) + ($110))|0;
 $112 = HEAP32[$nPathname>>2]|0;
 $113 = (($111) + ($112))|0;
 $114 = (($113) + 8)|0;
 $115 = (($114) + 2)|0;
 $116 = HEAP32[$nPathname>>2]|0;
 $117 = (($115) + ($116))|0;
 $118 = (($117) + 4)|0;
 $119 = (($118) + 2)|0;
 $120 = (_sqlite3MallocZero($119,0)|0);
 HEAP32[$pPtr>>2] = $120;
 $121 = HEAP32[$pPtr>>2]|0;
 $122 = ($121|0)!=(0|0);
 if (!($122)) {
  $123 = HEAP32[$zPathname>>2]|0;
  _sqlite3DbFree(0,$123);
  HEAP32[$0>>2] = 7;
  $378 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($378|0);
 }
 $124 = HEAP32[$pPtr>>2]|0;
 HEAP32[$pPager>>2] = $124;
 $125 = HEAP32[$pPtr>>2]|0;
 $126 = ((($125)) + 224|0);
 HEAP32[$pPtr>>2] = $126;
 $127 = HEAP32[$pPager>>2]|0;
 $128 = ((($127)) + 212|0);
 HEAP32[$128>>2] = $126;
 $129 = HEAP32[$pcacheSize>>2]|0;
 $130 = (($129) + 7)|0;
 $131 = $130 & -8;
 $132 = HEAP32[$pPtr>>2]|0;
 $133 = (($132) + ($131)|0);
 HEAP32[$pPtr>>2] = $133;
 $134 = HEAP32[$pPager>>2]|0;
 $135 = ((($134)) + 64|0);
 HEAP32[$135>>2] = $133;
 $136 = HEAP32[$1>>2]|0;
 $137 = ((($136)) + 4|0);
 $138 = HEAP32[$137>>2]|0;
 $139 = (($138) + 7)|0;
 $140 = $139 & -8;
 $141 = HEAP32[$pPtr>>2]|0;
 $142 = (($141) + ($140)|0);
 HEAP32[$pPtr>>2] = $142;
 $143 = HEAP32[$pPager>>2]|0;
 $144 = ((($143)) + 72|0);
 HEAP32[$144>>2] = $142;
 $145 = HEAP32[$journalFileSize>>2]|0;
 $146 = HEAP32[$pPtr>>2]|0;
 $147 = (($146) + ($145)|0);
 HEAP32[$pPtr>>2] = $147;
 $148 = HEAP32[$pPager>>2]|0;
 $149 = ((($148)) + 68|0);
 HEAP32[$149>>2] = $147;
 $150 = HEAP32[$journalFileSize>>2]|0;
 $151 = HEAP32[$pPtr>>2]|0;
 $152 = (($151) + ($150)|0);
 HEAP32[$pPtr>>2] = $152;
 $153 = HEAP32[$pPager>>2]|0;
 $154 = ((($153)) + 176|0);
 HEAP32[$154>>2] = $152;
 $155 = HEAP32[$zPathname>>2]|0;
 $156 = ($155|0)!=(0|0);
 if ($156) {
  $157 = HEAP32[$nPathname>>2]|0;
  $158 = (($157) + 1)|0;
  $159 = HEAP32[$nUri>>2]|0;
  $160 = (($158) + ($159))|0;
  $161 = HEAP32[$pPtr>>2]|0;
  $162 = (($161) + ($160)|0);
  HEAP32[$pPtr>>2] = $162;
  $163 = HEAP32[$pPager>>2]|0;
  $164 = ((($163)) + 180|0);
  HEAP32[$164>>2] = $162;
  $165 = HEAP32[$pPager>>2]|0;
  $166 = ((($165)) + 176|0);
  $167 = HEAP32[$166>>2]|0;
  $168 = HEAP32[$zPathname>>2]|0;
  $169 = HEAP32[$nPathname>>2]|0;
  _memcpy(($167|0),($168|0),($169|0))|0;
  $170 = HEAP32[$nUri>>2]|0;
  $171 = ($170|0)!=(0);
  if ($171) {
   $172 = HEAP32[$nPathname>>2]|0;
   $173 = (($172) + 1)|0;
   $174 = HEAP32[$pPager>>2]|0;
   $175 = ((($174)) + 176|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = (($176) + ($173)|0);
   $178 = HEAP32[$zUri>>2]|0;
   $179 = HEAP32[$nUri>>2]|0;
   _memcpy(($177|0),($178|0),($179|0))|0;
  }
  $180 = HEAP32[$pPager>>2]|0;
  $181 = ((($180)) + 180|0);
  $182 = HEAP32[$181>>2]|0;
  $183 = HEAP32[$zPathname>>2]|0;
  $184 = HEAP32[$nPathname>>2]|0;
  _memcpy(($182|0),($183|0),($184|0))|0;
  $185 = HEAP32[$nPathname>>2]|0;
  $186 = HEAP32[$pPager>>2]|0;
  $187 = ((($186)) + 180|0);
  $188 = HEAP32[$187>>2]|0;
  $189 = (($188) + ($185)|0);
  dest=$189; src=25989; stop=dest+10|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $190 = HEAP32[$nPathname>>2]|0;
  $191 = (($190) + 8)|0;
  $192 = (($191) + 1)|0;
  $193 = HEAP32[$pPager>>2]|0;
  $194 = ((($193)) + 180|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = (($195) + ($192)|0);
  $197 = HEAP32[$pPager>>2]|0;
  $198 = ((($197)) + 220|0);
  HEAP32[$198>>2] = $196;
  $199 = HEAP32[$pPager>>2]|0;
  $200 = ((($199)) + 220|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = HEAP32[$zPathname>>2]|0;
  $203 = HEAP32[$nPathname>>2]|0;
  _memcpy(($201|0),($202|0),($203|0))|0;
  $204 = HEAP32[$nPathname>>2]|0;
  $205 = HEAP32[$pPager>>2]|0;
  $206 = ((($205)) + 220|0);
  $207 = HEAP32[$206>>2]|0;
  $208 = (($207) + ($204)|0);
  ;HEAP8[$208>>0]=HEAP8[25999>>0]|0;HEAP8[$208+1>>0]=HEAP8[25999+1>>0]|0;HEAP8[$208+2>>0]=HEAP8[25999+2>>0]|0;HEAP8[$208+3>>0]=HEAP8[25999+3>>0]|0;HEAP8[$208+4>>0]=HEAP8[25999+4>>0]|0;
  $209 = HEAP32[$zPathname>>2]|0;
  _sqlite3DbFree(0,$209);
 }
 $210 = HEAP32[$1>>2]|0;
 $211 = HEAP32[$pPager>>2]|0;
 HEAP32[$211>>2] = $210;
 $212 = HEAP32[$6>>2]|0;
 $213 = HEAP32[$pPager>>2]|0;
 $214 = ((($213)) + 152|0);
 HEAP32[$214>>2] = $212;
 $215 = HEAP32[$3>>2]|0;
 $216 = ($215|0)!=(0|0);
 do {
  if ($216) {
   $217 = HEAP32[$3>>2]|0;
   $218 = HEAP8[$217>>0]|0;
   $219 = $218 << 24 >> 24;
   $220 = ($219|0)!=(0);
   if ($220) {
    HEAP32[$fout>>2] = 0;
    $221 = HEAP32[$1>>2]|0;
    $222 = HEAP32[$pPager>>2]|0;
    $223 = ((($222)) + 176|0);
    $224 = HEAP32[$223>>2]|0;
    $225 = HEAP32[$pPager>>2]|0;
    $226 = ((($225)) + 64|0);
    $227 = HEAP32[$226>>2]|0;
    $228 = HEAP32[$6>>2]|0;
    $229 = (_sqlite3OsOpen($221,$224,$227,$228,$fout)|0);
    HEAP32[$rc>>2] = $229;
    $230 = HEAP32[$fout>>2]|0;
    $231 = $230 & 1;
    HEAP32[$readOnly>>2] = $231;
    $232 = HEAP32[$rc>>2]|0;
    $233 = ($232|0)==(0);
    if ($233) {
     $234 = HEAP32[$pPager>>2]|0;
     $235 = ((($234)) + 64|0);
     $236 = HEAP32[$235>>2]|0;
     $237 = (_sqlite3OsDeviceCharacteristics($236)|0);
     HEAP32[$iDc>>2] = $237;
     $238 = HEAP32[$readOnly>>2]|0;
     $239 = ($238|0)!=(0);
     do {
      if (!($239)) {
       $240 = HEAP32[$pPager>>2]|0;
       _setSectorSize($240);
       $241 = HEAP32[$szPageDflt>>2]|0;
       $242 = HEAP32[$pPager>>2]|0;
       $243 = ((($242)) + 156|0);
       $244 = HEAP32[$243>>2]|0;
       $245 = ($241>>>0)<($244>>>0);
       if ($245) {
        $246 = HEAP32[$pPager>>2]|0;
        $247 = ((($246)) + 156|0);
        $248 = HEAP32[$247>>2]|0;
        $249 = ($248>>>0)>(8192);
        if ($249) {
         HEAP32[$szPageDflt>>2] = 8192;
         break;
        } else {
         $250 = HEAP32[$pPager>>2]|0;
         $251 = ((($250)) + 156|0);
         $252 = HEAP32[$251>>2]|0;
         HEAP32[$szPageDflt>>2] = $252;
         break;
        }
       }
      }
     } while(0);
     $253 = HEAP32[$3>>2]|0;
     $254 = (_sqlite3_uri_boolean($253,26005,0)|0);
     $255 = $254&255;
     $256 = HEAP32[$pPager>>2]|0;
     $257 = ((($256)) + 14|0);
     HEAP8[$257>>0] = $255;
     $258 = HEAP32[$iDc>>2]|0;
     $259 = $258 & 8192;
     $260 = ($259|0)!=(0);
     if (!($260)) {
      $261 = HEAP32[$3>>2]|0;
      $262 = (_sqlite3_uri_boolean($261,26012,0)|0);
      $263 = ($262|0)!=(0);
      if (!($263)) {
       break;
      }
     }
     $264 = HEAP32[$6>>2]|0;
     $265 = $264 | 1;
     HEAP32[$6>>2] = $265;
     label = 36;
    }
   } else {
    label = 36;
   }
  } else {
   label = 36;
  }
 } while(0);
 if ((label|0) == 36) {
  HEAP32[$tempFile>>2] = 1;
  $266 = HEAP32[$pPager>>2]|0;
  $267 = ((($266)) + 17|0);
  HEAP8[$267>>0] = 1;
  $268 = HEAP32[$pPager>>2]|0;
  $269 = ((($268)) + 18|0);
  HEAP8[$269>>0] = 4;
  $270 = HEAP32[$pPager>>2]|0;
  $271 = ((($270)) + 14|0);
  HEAP8[$271>>0] = 1;
  $272 = HEAP32[$6>>2]|0;
  $273 = $272 & 1;
  HEAP32[$readOnly>>2] = $273;
 }
 $274 = HEAP32[$rc>>2]|0;
 $275 = ($274|0)==(0);
 if ($275) {
  $276 = HEAP32[$pPager>>2]|0;
  $277 = (_sqlite3PagerSetPagesize($276,$szPageDflt,-1)|0);
  HEAP32[$rc>>2] = $277;
 }
 $278 = HEAP32[$rc>>2]|0;
 $279 = ($278|0)==(0);
 if ($279) {
  $280 = HEAP32[$4>>2]|0;
  $281 = (($280) + 7)|0;
  $282 = $281 & -8;
  HEAP32[$4>>2] = $282;
  $283 = HEAP32[$szPageDflt>>2]|0;
  $284 = HEAP32[$4>>2]|0;
  $285 = HEAP32[$memDb>>2]|0;
  $286 = ($285|0)!=(0);
  $287 = $286 ^ 1;
  $288 = $287&1;
  $289 = HEAP32[$memDb>>2]|0;
  $290 = ($289|0)!=(0);
  $291 = $290 ^ 1;
  $292 = $291 ? 186 : 0;
  $293 = HEAP32[$pPager>>2]|0;
  $294 = HEAP32[$pPager>>2]|0;
  $295 = ((($294)) + 212|0);
  $296 = HEAP32[$295>>2]|0;
  $297 = (_sqlite3PcacheOpen($283,$284,$288,$292,$293,$296)|0);
  HEAP32[$rc>>2] = $297;
 }
 $298 = HEAP32[$rc>>2]|0;
 $299 = ($298|0)!=(0);
 if ($299) {
  $300 = HEAP32[$pPager>>2]|0;
  $301 = ((($300)) + 64|0);
  $302 = HEAP32[$301>>2]|0;
  _sqlite3OsClose($302);
  $303 = HEAP32[$pPager>>2]|0;
  $304 = ((($303)) + 208|0);
  $305 = HEAP32[$304>>2]|0;
  _sqlite3PageFree($305);
  $306 = HEAP32[$pPager>>2]|0;
  _sqlite3_free($306);
  $307 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $307;
  $378 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($378|0);
 }
 $308 = HEAP32[$useJournal>>2]|0;
 $309 = $308&255;
 $310 = HEAP32[$pPager>>2]|0;
 $311 = ((($310)) + 6|0);
 HEAP8[$311>>0] = $309;
 $312 = HEAP32[$pPager>>2]|0;
 $313 = ((($312)) + 164|0);
 HEAP32[$313>>2] = 1073741823;
 $314 = HEAP32[$tempFile>>2]|0;
 $315 = $314&255;
 $316 = HEAP32[$pPager>>2]|0;
 $317 = ((($316)) + 13|0);
 HEAP8[$317>>0] = $315;
 $318 = HEAP32[$tempFile>>2]|0;
 $319 = $318&255;
 $320 = HEAP32[$pPager>>2]|0;
 $321 = ((($320)) + 4|0);
 HEAP8[$321>>0] = $319;
 $322 = HEAP32[$pPager>>2]|0;
 $323 = ((($322)) + 13|0);
 $324 = HEAP8[$323>>0]|0;
 $325 = HEAP32[$pPager>>2]|0;
 $326 = ((($325)) + 19|0);
 HEAP8[$326>>0] = $324;
 $327 = HEAP32[$memDb>>2]|0;
 $328 = $327&255;
 $329 = HEAP32[$pPager>>2]|0;
 $330 = ((($329)) + 16|0);
 HEAP8[$330>>0] = $328;
 $331 = HEAP32[$readOnly>>2]|0;
 $332 = $331&255;
 $333 = HEAP32[$pPager>>2]|0;
 $334 = ((($333)) + 15|0);
 HEAP8[$334>>0] = $332;
 $335 = HEAP32[$pPager>>2]|0;
 $336 = ((($335)) + 13|0);
 $337 = HEAP8[$336>>0]|0;
 $338 = HEAP32[$pPager>>2]|0;
 $339 = ((($338)) + 7|0);
 HEAP8[$339>>0] = $337;
 $340 = HEAP32[$pPager>>2]|0;
 $341 = ((($340)) + 7|0);
 $342 = HEAP8[$341>>0]|0;
 $343 = ($342<<24>>24)!=(0);
 if (!($343)) {
  $344 = HEAP32[$pPager>>2]|0;
  $345 = ((($344)) + 8|0);
  HEAP8[$345>>0] = 1;
  $346 = HEAP32[$pPager>>2]|0;
  $347 = ((($346)) + 9|0);
  HEAP8[$347>>0] = 0;
  $348 = HEAP32[$pPager>>2]|0;
  $349 = ((($348)) + 12|0);
  HEAP8[$349>>0] = 2;
  $350 = HEAP32[$pPager>>2]|0;
  $351 = ((($350)) + 11|0);
  HEAP8[$351>>0] = 34;
  $352 = HEAP32[$pPager>>2]|0;
  $353 = ((($352)) + 10|0);
  HEAP8[$353>>0] = 2;
 }
 $354 = HEAP32[$4>>2]|0;
 $355 = $354&65535;
 $356 = HEAP32[$pPager>>2]|0;
 $357 = ((($356)) + 148|0);
 HEAP16[$357>>1] = $355;
 $358 = HEAP32[$pPager>>2]|0;
 $359 = ((($358)) + 168|0);
 $360 = $359;
 $361 = $360;
 HEAP32[$361>>2] = -1;
 $362 = (($360) + 4)|0;
 $363 = $362;
 HEAP32[$363>>2] = -1;
 $364 = HEAP32[$pPager>>2]|0;
 _setSectorSize($364);
 $365 = HEAP32[$useJournal>>2]|0;
 $366 = ($365|0)!=(0);
 if ($366) {
  $369 = HEAP32[$memDb>>2]|0;
  $370 = ($369|0)!=(0);
  if ($370) {
   $371 = HEAP32[$pPager>>2]|0;
   $372 = ((($371)) + 5|0);
   HEAP8[$372>>0] = 4;
  }
 } else {
  $367 = HEAP32[$pPager>>2]|0;
  $368 = ((($367)) + 5|0);
  HEAP8[$368>>0] = 2;
 }
 $373 = HEAP32[$7>>2]|0;
 $374 = HEAP32[$pPager>>2]|0;
 $375 = ((($374)) + 204|0);
 HEAP32[$375>>2] = $373;
 $376 = HEAP32[$pPager>>2]|0;
 $377 = HEAP32[$2>>2]|0;
 HEAP32[$377>>2] = $376;
 HEAP32[$0>>2] = 0;
 $378 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($378|0);
}
function _sqlite3PagerSetMmapLimit($pPager,$0,$1) {
 $pPager = $pPager|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pPager;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 136|0);
 $16 = $15;
 $17 = $16;
 HEAP32[$17>>2] = $10;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $13;
 $20 = HEAP32[$2>>2]|0;
 _pagerFixMaplimit($20);
 STACKTOP = sp;return;
}
function _sqlite3PagerReadFileheader($pPager,$N,$pDest) {
 $pPager = $pPager|0;
 $N = $N|0;
 $pDest = $pDest|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memset(($3|0),0,($4|0))|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 64|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3OsRead($12,$13,$14,0,0)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(522);
 $$ = $17 ? 0 : $15;
 HEAP32[$rc>>2] = $$;
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _btreeInvokeBusyHandler($pArg) {
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $pArg;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pBt>>2] = $1;
 $2 = HEAP32[$pBt>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 380|0);
 $6 = (_sqlite3InvokeBusyHandler($5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3PagerSetBusyhandler($pPager,$xBusyHandler,$pBusyHandlerArg) {
 $pPager = $pPager|0;
 $xBusyHandler = $xBusyHandler|0;
 $pBusyHandlerArg = $pBusyHandlerArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $ap = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $xBusyHandler;
 HEAP32[$2>>2] = $pBusyHandlerArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 184|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 188|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 64|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 184|0);
 HEAP32[$ap>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$ap>>2]|0;
 _sqlite3OsFileControlHint($18,15,$19);
 STACKTOP = sp;return;
}
function _sqlite3PagerIsreadonly($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 15|0);
 $3 = HEAP8[$2>>0]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3Get4byte($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = $3 << 24;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 << 16;
 $10 = $4 | $9;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 2|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 << 8;
 $16 = $10 | $15;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 3|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = $16 | $20;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3PagerClose($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pTmp>>2] = $3;
 _sqlite3BeginBenignMalloc();
 $4 = HEAP32[$0>>2]|0;
 _pagerFreeMapHdrs($4);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP8[$6>>0] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 216|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 10|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 160|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$pTmp>>2]|0;
 (_sqlite3WalClose($9,$13,$16,$17)|0);
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 216|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 _pager_reset($20);
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = ($23<<24>>24)!=(0);
 $25 = HEAP32[$0>>2]|0;
 if ($24) {
  _pager_unlock($25);
 } else {
  $26 = ((($25)) + 68|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = HEAP32[$0>>2]|0;
   $32 = (_pagerSyncHotJournal($31)|0);
   (_pager_error($30,$32)|0);
  }
  $33 = HEAP32[$0>>2]|0;
  _pagerUnlockAndRollback($33);
 }
 _sqlite3EndBenignMalloc();
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 68|0);
 $36 = HEAP32[$35>>2]|0;
 _sqlite3OsClose($36);
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 64|0);
 $39 = HEAP32[$38>>2]|0;
 _sqlite3OsClose($39);
 $40 = HEAP32[$pTmp>>2]|0;
 _sqlite3PageFree($40);
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 212|0);
 $43 = HEAP32[$42>>2]|0;
 _sqlite3PcacheClose($43);
 $44 = HEAP32[$0>>2]|0;
 _sqlite3_free($44);
 STACKTOP = sp;return 0;
}
function _pagerFreeMapHdrs($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 144|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$pNext>>2] = $8;
  $9 = HEAP32[$p>>2]|0;
  _sqlite3_free($9);
  $10 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $10;
 }
 STACKTOP = sp;return;
}
function _sqlite3WalClose($pWal,$sync_flags,$nBuf,$zBuf) {
 $pWal = $pWal|0;
 $sync_flags = $sync_flags|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bPersist = 0, $isDelete = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $isDelete = sp + 4|0;
 $bPersist = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $sync_flags;
 HEAP32[$2>>2] = $nBuf;
 HEAP32[$3>>2] = $zBuf;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $61 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 HEAP32[$isDelete>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3OsLock($8,4)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 43|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)==(0);
   if ($16) {
    $17 = HEAP32[$0>>2]|0;
    $18 = ((($17)) + 43|0);
    HEAP8[$18>>0] = 1;
   }
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   $23 = (_sqlite3WalCheckpoint($19,0,0,0,$20,$21,$22,0,0)|0);
   HEAP32[$rc>>2] = $23;
   $24 = HEAP32[$rc>>2]|0;
   $25 = ($24|0)==(0);
   if ($25) {
    HEAP32[$bPersist>>2] = -1;
    $26 = HEAP32[$0>>2]|0;
    $27 = ((($26)) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    _sqlite3OsFileControlHint($28,10,$bPersist);
    $29 = HEAP32[$bPersist>>2]|0;
    $30 = ($29|0)!=(1);
    if ($30) {
     HEAP32[$isDelete>>2] = 1;
     break;
    }
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 16|0);
    $33 = $32;
    $34 = $33;
    $35 = HEAP32[$34>>2]|0;
    $36 = (($33) + 4)|0;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = ($38|0)>(0);
    $40 = ($35>>>0)>=(0);
    $41 = ($38|0)==(0);
    $42 = $41 & $40;
    $43 = $39 | $42;
    if ($43) {
     $44 = HEAP32[$0>>2]|0;
     _walLimitSize($44,0,0);
    }
   }
  }
 } while(0);
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$isDelete>>2]|0;
 _walIndexClose($45,$46);
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 _sqlite3OsClose($49);
 $50 = HEAP32[$isDelete>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  _sqlite3BeginBenignMalloc();
  $52 = HEAP32[$0>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$0>>2]|0;
  $55 = ((($54)) + 108|0);
  $56 = HEAP32[$55>>2]|0;
  (_sqlite3OsDelete($53,$56,0)|0);
  _sqlite3EndBenignMalloc();
 }
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 32|0);
 $59 = HEAP32[$58>>2]|0;
 _sqlite3_free($59);
 $60 = HEAP32[$0>>2]|0;
 _sqlite3_free($60);
 $61 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _pager_unlock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $iDc = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $iDc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3BitvecDestroy($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 60|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 _releaseAllSavepoints($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3PagerUseWal($7)|0);
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = ((($10)) + 216|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3WalEndReadTransaction($12);
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 17|0);
  HEAP8[$14>>0] = 0;
 } else {
  $15 = ((($10)) + 4|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 64|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$0>>2]|0;
    $24 = ((($23)) + 64|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (_sqlite3OsDeviceCharacteristics($25)|0);
    $27 = $26;
   } else {
    $27 = 0;
   }
   HEAP32[$iDc>>2] = $27;
   $28 = HEAP32[$iDc>>2]|0;
   $29 = $28 & 2048;
   $30 = (0)==($29|0);
   if ($30) {
    label = 8;
   } else {
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 5|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = $33&255;
    $35 = $34 & 5;
    $36 = (1)!=($35|0);
    if ($36) {
     label = 8;
    }
   }
   if ((label|0) == 8) {
    $37 = HEAP32[$0>>2]|0;
    $38 = ((($37)) + 68|0);
    $39 = HEAP32[$38>>2]|0;
    _sqlite3OsClose($39);
   }
   $40 = HEAP32[$0>>2]|0;
   $41 = (_pagerUnlockDb($40,0)|0);
   HEAP32[$rc>>2] = $41;
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)!=(0);
   if ($43) {
    $44 = HEAP32[$0>>2]|0;
    $45 = ((($44)) + 17|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $48 = ($47|0)==(6);
    if ($48) {
     $49 = HEAP32[$0>>2]|0;
     $50 = ((($49)) + 18|0);
     HEAP8[$50>>0] = 5;
    }
   }
   $51 = HEAP32[$0>>2]|0;
   $52 = ((($51)) + 19|0);
   HEAP8[$52>>0] = 0;
   $53 = HEAP32[$0>>2]|0;
   $54 = ((($53)) + 17|0);
   HEAP8[$54>>0] = 0;
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 44|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)!=(0);
 if (!($58)) {
  $79 = HEAP32[$0>>2]|0;
  $80 = ((($79)) + 80|0);
  $81 = $80;
  $82 = $81;
  HEAP32[$82>>2] = 0;
  $83 = (($81) + 4)|0;
  $84 = $83;
  HEAP32[$84>>2] = 0;
  $85 = HEAP32[$0>>2]|0;
  $86 = ((($85)) + 88|0);
  $87 = $86;
  $88 = $87;
  HEAP32[$88>>2] = 0;
  $89 = (($87) + 4)|0;
  $90 = $89;
  HEAP32[$90>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  $92 = ((($91)) + 20|0);
  HEAP8[$92>>0] = 0;
  STACKTOP = sp;return;
 }
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 13|0);
 $61 = HEAP8[$60>>0]|0;
 $62 = $61&255;
 $63 = ($62|0)==(0);
 $64 = HEAP32[$0>>2]|0;
 if ($63) {
  _pager_reset($64);
  $65 = HEAP32[$0>>2]|0;
  $66 = ((($65)) + 19|0);
  HEAP8[$66>>0] = 0;
  $67 = HEAP32[$0>>2]|0;
  $68 = ((($67)) + 17|0);
  HEAP8[$68>>0] = 0;
 } else {
  $69 = ((($64)) + 68|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = HEAP32[$70>>2]|0;
  $72 = ($71|0)!=(0|0);
  $73 = $72 ? 0 : 1;
  $74 = $73&255;
  $75 = HEAP32[$0>>2]|0;
  $76 = ((($75)) + 17|0);
  HEAP8[$76>>0] = $74;
 }
 $77 = HEAP32[$0>>2]|0;
 $78 = ((($77)) + 44|0);
 HEAP32[$78>>2] = 0;
 $79 = HEAP32[$0>>2]|0;
 $80 = ((($79)) + 80|0);
 $81 = $80;
 $82 = $81;
 HEAP32[$82>>2] = 0;
 $83 = (($81) + 4)|0;
 $84 = $83;
 HEAP32[$84>>2] = 0;
 $85 = HEAP32[$0>>2]|0;
 $86 = ((($85)) + 88|0);
 $87 = $86;
 $88 = $87;
 HEAP32[$88>>2] = 0;
 $89 = (($87) + 4)|0;
 $90 = $89;
 HEAP32[$90>>2] = 0;
 $91 = HEAP32[$0>>2]|0;
 $92 = ((($91)) + 20|0);
 HEAP8[$92>>0] = 0;
 STACKTOP = sp;return;
}
function _pagerSyncHotJournal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 7|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 68|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3OsSync($7,2)|0);
  HEAP32[$rc>>2] = $8;
 }
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)==(0);
 if (!($10)) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 68|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 88|0);
 $16 = (_sqlite3OsFileSize($13,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _pager_error($pPager,$rc) {
 $pPager = $pPager|0;
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc2 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $rc;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 255;
 HEAP32[$rc2>>2] = $3;
 $4 = HEAP32[$rc2>>2]|0;
 $5 = ($4|0)==(13);
 $6 = HEAP32[$rc2>>2]|0;
 $7 = ($6|0)==(10);
 $or$cond = $5 | $7;
 if (!($or$cond)) {
  $13 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 44|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 17|0);
 HEAP8[$12>>0] = 6;
 $13 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _pagerUnlockAndRollback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 17|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(6);
 do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 17|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = ($9|0)!=(0);
   if ($10) {
    $11 = HEAP32[$0>>2]|0;
    $12 = ((($11)) + 17|0);
    $13 = HEAP8[$12>>0]|0;
    $14 = $13&255;
    $15 = ($14|0)>=(2);
    if ($15) {
     _sqlite3BeginBenignMalloc();
     $16 = HEAP32[$0>>2]|0;
     (_sqlite3PagerRollback($16)|0);
     _sqlite3EndBenignMalloc();
     break;
    }
    $17 = HEAP32[$0>>2]|0;
    $18 = ((($17)) + 4|0);
    $19 = HEAP8[$18>>0]|0;
    $20 = ($19<<24>>24)!=(0);
    if (!($20)) {
     $21 = HEAP32[$0>>2]|0;
     (_pager_end_transaction($21,0,0)|0);
    }
   }
  }
 } while(0);
 $22 = HEAP32[$0>>2]|0;
 _pager_unlock($22);
 STACKTOP = sp;return;
}
function _sqlite3OsClose($pId) {
 $pId = $pId|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pId;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 (FUNCTION_TABLE_ii[$7 & 255]($8)|0);
 $9 = HEAP32[$0>>2]|0;
 HEAP32[$9>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3PcacheClose($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[(160)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$1 & 255]($4);
 STACKTOP = sp;return;
}
function _sqlite3PagerRollback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $eState = 0, $or$cond = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $rc2 = sp + 4|0;
 $eState = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 17|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(6);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 44|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$0>>2] = $9;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $10 = ((($7)) + 17|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)<=(1);
 if ($13) {
  HEAP32[$0>>2] = 0;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3PagerUseWal($14)|0);
 $16 = ($15|0)!=(0);
 $17 = HEAP32[$1>>2]|0;
 do {
  if ($16) {
   $18 = (_sqlite3PagerSavepoint($17,2,-1)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = (_pager_end_transaction($19,$23,0)|0);
   HEAP32[$rc2>>2] = $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(0);
   if ($26) {
    $27 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $27;
   }
  } else {
   $28 = ((($17)) + 68|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 17|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = ($35|0)==(2);
    if (!($36)) {
     $54 = HEAP32[$1>>2]|0;
     $55 = (_pager_playback($54,0)|0);
     HEAP32[$rc>>2] = $55;
     break;
    }
   }
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 17|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   HEAP32[$eState>>2] = $40;
   $41 = HEAP32[$1>>2]|0;
   $42 = (_pager_end_transaction($41,0,0)|0);
   HEAP32[$rc>>2] = $42;
   $43 = HEAP32[$1>>2]|0;
   $44 = ((($43)) + 16|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = ($45<<24>>24)==(0);
   $47 = HEAP32[$eState>>2]|0;
   $48 = ($47|0)>(2);
   $or$cond = $46 & $48;
   if ($or$cond) {
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 44|0);
    HEAP32[$50>>2] = 4;
    $51 = HEAP32[$1>>2]|0;
    $52 = ((($51)) + 17|0);
    HEAP8[$52>>0] = 6;
    $53 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $53;
    $59 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($59|0);
   }
  }
 } while(0);
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$rc>>2]|0;
 $58 = (_pager_error($56,$57)|0);
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _pager_end_transaction($pPager,$hasMaster,$bCommit) {
 $pPager = $pPager|0;
 $hasMaster = $hasMaster|0;
 $bCommit = $bCommit|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bDelete = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $rc2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $rc2 = sp + 4|0;
 $bDelete = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $hasMaster;
 HEAP32[$3>>2] = $bCommit;
 HEAP32[$rc>>2] = 0;
 HEAP32[$rc2>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)<(2);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 18|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)<(2);
  if ($13) {
   HEAP32[$0>>2] = 0;
   $205 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($205|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 _releaseAllSavepoints($14);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 68|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 L6: do {
  if ($19) {
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 68|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3JournalIsInMemory($22)|0);
   $24 = ($23|0)!=(0);
   $25 = HEAP32[$1>>2]|0;
   if ($24) {
    $26 = ((($25)) + 68|0);
    $27 = HEAP32[$26>>2]|0;
    _sqlite3OsClose($27);
    break;
   }
   $28 = ((($25)) + 5|0);
   $29 = HEAP8[$28>>0]|0;
   $30 = $29&255;
   $31 = ($30|0)==(3);
   $32 = HEAP32[$1>>2]|0;
   if ($31) {
    $33 = ((($32)) + 80|0);
    $34 = $33;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = ($36|0)==(0);
    $41 = ($39|0)==(0);
    $42 = $40 & $41;
    if ($42) {
     HEAP32[$rc>>2] = 0;
    } else {
     $43 = HEAP32[$1>>2]|0;
     $44 = ((($43)) + 68|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (_sqlite3OsTruncate($45,0,0)|0);
     HEAP32[$rc>>2] = $46;
     $47 = HEAP32[$rc>>2]|0;
     $48 = ($47|0)==(0);
     if ($48) {
      $49 = HEAP32[$1>>2]|0;
      $50 = ((($49)) + 8|0);
      $51 = HEAP8[$50>>0]|0;
      $52 = $51&255;
      $53 = ($52|0)!=(0);
      if ($53) {
       $54 = HEAP32[$1>>2]|0;
       $55 = ((($54)) + 68|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = HEAP32[$1>>2]|0;
       $58 = ((($57)) + 12|0);
       $59 = HEAP8[$58>>0]|0;
       $60 = $59&255;
       $61 = (_sqlite3OsSync($56,$60)|0);
       HEAP32[$rc>>2] = $61;
      }
     }
    }
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 80|0);
    $64 = $63;
    $65 = $64;
    HEAP32[$65>>2] = 0;
    $66 = (($64) + 4)|0;
    $67 = $66;
    HEAP32[$67>>2] = 0;
    break;
   }
   $68 = ((($32)) + 5|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = $69&255;
   $71 = ($70|0)==(1);
   do {
    if (!($71)) {
     $72 = HEAP32[$1>>2]|0;
     $73 = ((($72)) + 4|0);
     $74 = HEAP8[$73>>0]|0;
     $75 = $74&255;
     $76 = ($75|0)!=(0);
     if ($76) {
      $77 = HEAP32[$1>>2]|0;
      $78 = ((($77)) + 5|0);
      $79 = HEAP8[$78>>0]|0;
      $80 = $79&255;
      $81 = ($80|0)!=(5);
      if ($81) {
       break;
      }
     }
     $99 = HEAP32[$1>>2]|0;
     $100 = ((($99)) + 13|0);
     $101 = HEAP8[$100>>0]|0;
     $102 = ($101<<24>>24)!=(0);
     $103 = $102 ^ 1;
     $104 = $103&1;
     HEAP32[$bDelete>>2] = $104;
     $105 = HEAP32[$1>>2]|0;
     $106 = ((($105)) + 68|0);
     $107 = HEAP32[$106>>2]|0;
     _sqlite3OsClose($107);
     $108 = HEAP32[$bDelete>>2]|0;
     $109 = ($108|0)!=(0);
     if (!($109)) {
      break L6;
     }
     $110 = HEAP32[$1>>2]|0;
     $111 = HEAP32[$110>>2]|0;
     $112 = HEAP32[$1>>2]|0;
     $113 = ((($112)) + 180|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = HEAP32[$1>>2]|0;
     $116 = ((($115)) + 9|0);
     $117 = HEAP8[$116>>0]|0;
     $118 = $117&255;
     $119 = (_sqlite3OsDelete($111,$114,$118)|0);
     HEAP32[$rc>>2] = $119;
     break L6;
    }
   } while(0);
   $82 = HEAP32[$1>>2]|0;
   $83 = HEAP32[$2>>2]|0;
   $84 = ($83|0)!=(0);
   if ($84) {
    $91 = 1;
   } else {
    $85 = HEAP32[$1>>2]|0;
    $86 = ((($85)) + 13|0);
    $87 = HEAP8[$86>>0]|0;
    $88 = $87&255;
    $89 = ($88|0)!=(0);
    $91 = $89;
   }
   $90 = $91&1;
   $92 = (_zeroJournalHdr($82,$90)|0);
   HEAP32[$rc>>2] = $92;
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 80|0);
   $95 = $94;
   $96 = $95;
   HEAP32[$96>>2] = 0;
   $97 = (($95) + 4)|0;
   $98 = $97;
   HEAP32[$98>>2] = 0;
  }
 } while(0);
 $120 = HEAP32[$1>>2]|0;
 $121 = ((($120)) + 60|0);
 $122 = HEAP32[$121>>2]|0;
 _sqlite3BitvecDestroy($122);
 $123 = HEAP32[$1>>2]|0;
 $124 = ((($123)) + 60|0);
 HEAP32[$124>>2] = 0;
 $125 = HEAP32[$1>>2]|0;
 $126 = ((($125)) + 48|0);
 HEAP32[$126>>2] = 0;
 $127 = HEAP32[$rc>>2]|0;
 $128 = ($127|0)==(0);
 if ($128) {
  $129 = HEAP32[$1>>2]|0;
  $130 = HEAP32[$3>>2]|0;
  $131 = (_pagerFlushOnCommit($129,$130)|0);
  $132 = ($131|0)!=(0);
  $133 = HEAP32[$1>>2]|0;
  $134 = ((($133)) + 212|0);
  $135 = HEAP32[$134>>2]|0;
  if ($132) {
   _sqlite3PcacheCleanAll($135);
  } else {
   _sqlite3PcacheClearWritable($135);
  }
  $136 = HEAP32[$1>>2]|0;
  $137 = ((($136)) + 212|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  $140 = ((($139)) + 28|0);
  $141 = HEAP32[$140>>2]|0;
  _sqlite3PcacheTruncate($138,$141);
 }
 $142 = HEAP32[$1>>2]|0;
 $143 = (_sqlite3PagerUseWal($142)|0);
 $144 = ($143|0)!=(0);
 if ($144) {
  $145 = HEAP32[$1>>2]|0;
  $146 = ((($145)) + 216|0);
  $147 = HEAP32[$146>>2]|0;
  $148 = (_sqlite3WalEndWriteTransaction($147)|0);
  HEAP32[$rc2>>2] = $148;
 } else {
  $149 = HEAP32[$rc>>2]|0;
  $150 = ($149|0)==(0);
  $151 = HEAP32[$3>>2]|0;
  $152 = ($151|0)!=(0);
  $or$cond = $150 & $152;
  if ($or$cond) {
   $153 = HEAP32[$1>>2]|0;
   $154 = ((($153)) + 36|0);
   $155 = HEAP32[$154>>2]|0;
   $156 = HEAP32[$1>>2]|0;
   $157 = ((($156)) + 28|0);
   $158 = HEAP32[$157>>2]|0;
   $159 = ($155>>>0)>($158>>>0);
   if ($159) {
    $160 = HEAP32[$1>>2]|0;
    $161 = HEAP32[$1>>2]|0;
    $162 = ((($161)) + 28|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = (_pager_truncate($160,$163)|0);
    HEAP32[$rc>>2] = $164;
   }
  }
 }
 $165 = HEAP32[$rc>>2]|0;
 $166 = ($165|0)==(0);
 $167 = HEAP32[$3>>2]|0;
 $168 = ($167|0)!=(0);
 $or$cond3 = $166 & $168;
 if ($or$cond3) {
  $169 = HEAP32[$1>>2]|0;
  $170 = ((($169)) + 64|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = HEAP32[$171>>2]|0;
  $173 = ($172|0)!=(0|0);
  if ($173) {
   $174 = HEAP32[$1>>2]|0;
   $175 = ((($174)) + 64|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = (_sqlite3OsFileControl($176,22,0)|0);
   HEAP32[$rc>>2] = $177;
   $178 = HEAP32[$rc>>2]|0;
   $179 = ($178|0)==(12);
   $$ = $179 ? 0 : $177;
   HEAP32[$rc>>2] = $$;
  }
 }
 $180 = HEAP32[$1>>2]|0;
 $181 = ((($180)) + 4|0);
 $182 = HEAP8[$181>>0]|0;
 $183 = ($182<<24>>24)!=(0);
 do {
  if (!($183)) {
   $184 = HEAP32[$1>>2]|0;
   $185 = (_sqlite3PagerUseWal($184)|0);
   $186 = ($185|0)!=(0);
   if ($186) {
    $187 = HEAP32[$1>>2]|0;
    $188 = ((($187)) + 216|0);
    $189 = HEAP32[$188>>2]|0;
    $190 = (_sqlite3WalExclusiveMode($189,0)|0);
    $191 = ($190|0)!=(0);
    if (!($191)) {
     break;
    }
   }
   $192 = HEAP32[$1>>2]|0;
   $193 = (_pagerUnlockDb($192,1)|0);
   HEAP32[$rc2>>2] = $193;
   $194 = HEAP32[$1>>2]|0;
   $195 = ((($194)) + 19|0);
   HEAP8[$195>>0] = 0;
  }
 } while(0);
 $196 = HEAP32[$1>>2]|0;
 $197 = ((($196)) + 17|0);
 HEAP8[$197>>0] = 1;
 $198 = HEAP32[$1>>2]|0;
 $199 = ((($198)) + 20|0);
 HEAP8[$199>>0] = 0;
 $200 = HEAP32[$rc>>2]|0;
 $201 = ($200|0)==(0);
 $202 = HEAP32[$rc2>>2]|0;
 $203 = HEAP32[$rc>>2]|0;
 $204 = $201 ? $202 : $203;
 HEAP32[$0>>2] = $204;
 $205 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($205|0);
}
function _releaseAllSavepoints($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ii = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $ii = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $1 = HEAP32[$ii>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 104|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$ii>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 100|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + (($6*48)|0)|0);
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  _sqlite3BitvecDestroy($12);
  $13 = HEAP32[$ii>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$ii>>2] = $14;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 72|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3JournalIsInMemory($21)|0);
  $23 = ($22|0)!=(0);
  if ($23) {
   label = 6;
  }
 } else {
  label = 6;
 }
 if ((label|0) == 6) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 72|0);
  $26 = HEAP32[$25>>2]|0;
  _sqlite3OsClose($26);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 100|0);
 $29 = HEAP32[$28>>2]|0;
 _sqlite3_free($29);
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 100|0);
 HEAP32[$31>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 104|0);
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 56|0);
 HEAP32[$35>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3JournalIsInMemory($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(4028|0);
 $4 = $3&1;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3OsTruncate($id,$0,$1) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $id;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = $3;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (FUNCTION_TABLE_iiii[$11 & 255]($12,$15,$18)|0);
 STACKTOP = sp;return ($19|0);
}
function _sqlite3OsSync($id,$flags) {
 $id = $id|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _zeroJournalHdr($pPager,$doTruncate) {
 $pPager = $pPager|0;
 $doTruncate = $doTruncate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iLimit = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $iLimit = sp + 8|0;
 $sz = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $doTruncate;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 80|0);
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)!=(0);
 $11 = ($9|0)!=(0);
 $12 = $10 | $11;
 if (!($12)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 168|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $iLimit;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = HEAP32[$1>>2]|0;
 $26 = ($25|0)!=(0);
 $27 = $iLimit;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = ($29|0)==(0);
 $34 = ($32|0)==(0);
 $35 = $33 & $34;
 $or$cond = $26 | $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 68|0);
 $38 = HEAP32[$37>>2]|0;
 if ($or$cond) {
  $39 = (_sqlite3OsTruncate($38,0,0)|0);
  HEAP32[$rc>>2] = $39;
 } else {
  $40 = (_sqlite3OsWrite($38,25846,28,0,0)|0);
  HEAP32[$rc>>2] = $40;
 }
 $41 = HEAP32[$rc>>2]|0;
 $42 = ($41|0)==(0);
 if ($42) {
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 7|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = ($45<<24>>24)!=(0);
  if (!($46)) {
   $47 = HEAP32[$0>>2]|0;
   $48 = ((($47)) + 68|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$0>>2]|0;
   $51 = ((($50)) + 12|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = 16 | $53;
   $55 = (_sqlite3OsSync($49,$54)|0);
   HEAP32[$rc>>2] = $55;
  }
 }
 $56 = HEAP32[$rc>>2]|0;
 $57 = ($56|0)==(0);
 $58 = $iLimit;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = ($63|0)>(0);
 $65 = ($60>>>0)>(0);
 $66 = ($63|0)==(0);
 $67 = $66 & $65;
 $68 = $64 | $67;
 $or$cond3 = $57 & $68;
 if (!($or$cond3)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 68|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (_sqlite3OsFileSize($71,$sz)|0);
 HEAP32[$rc>>2] = $72;
 $73 = HEAP32[$rc>>2]|0;
 $74 = ($73|0)==(0);
 if (!($74)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $75 = $sz;
 $76 = $75;
 $77 = HEAP32[$76>>2]|0;
 $78 = (($75) + 4)|0;
 $79 = $78;
 $80 = HEAP32[$79>>2]|0;
 $81 = $iLimit;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = ($80|0)>($86|0);
 $88 = ($77>>>0)>($83>>>0);
 $89 = ($80|0)==($86|0);
 $90 = $89 & $88;
 $91 = $87 | $90;
 if (!($91)) {
  $102 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $92 = HEAP32[$0>>2]|0;
 $93 = ((($92)) + 68|0);
 $94 = HEAP32[$93>>2]|0;
 $95 = $iLimit;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (($95) + 4)|0;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = (_sqlite3OsTruncate($94,$97,$100)|0);
 HEAP32[$rc>>2] = $101;
 $102 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($102|0);
}
function _sqlite3OsDelete($pVfs,$zPath,$dirSync) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $dirSync = $dirSync|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $dirSync;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (FUNCTION_TABLE_iiii[$5 & 255]($6,$7,$8)|0);
 STACKTOP = sp;return ($9|0);
}
function _pagerFlushOnCommit($pPager,$bCommit) {
 $pPager = $pPager|0;
 $bCommit = $bCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $bCommit;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 13|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(0);
 do {
  if ($7) {
   HEAP32[$0>>2] = 1;
  } else {
   $8 = HEAP32[$2>>2]|0;
   $9 = ($8|0)!=(0);
   if (!($9)) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 64|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)!=(0|0);
   if ($14) {
    $15 = HEAP32[$1>>2]|0;
    $16 = ((($15)) + 212|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (_sqlite3PCachePercentDirty($17)|0);
    $19 = ($18|0)>=(25);
    $20 = $19&1;
    HEAP32[$0>>2] = $20;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3PcacheCleanAll($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = HEAP32[$1>>2]|0;
  HEAP32[$p>>2] = $2;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$p>>2]|0;
  _sqlite3PcacheMakeClean($4);
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheClearWritable($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 24|0);
  $7 = HEAP16[$6>>1]|0;
  $8 = $7&65535;
  $9 = $8 & -13;
  $10 = $9&65535;
  HEAP16[$6>>1] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 32|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _sqlite3PagerUseWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = $4&1;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3WalEndWriteTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  STACKTOP = sp;return 0;
 }
 $5 = HEAP32[$0>>2]|0;
 _walUnlockExclusive($5,0,1);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 44|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 104|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 47|0);
 HEAP8[$11>>0] = 0;
 STACKTOP = sp;return 0;
}
function _pager_truncate($pPager,$nPage) {
 $pPager = $pPager|0;
 $nPage = $nPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $currentSize = 0, $newSize = 0, $pTmp = 0, $rc = 0, $szPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $currentSize = sp + 8|0;
 $newSize = sp;
 $szPage = sp + 20|0;
 $pTmp = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $nPage;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 17|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)>=(4);
 if (!($11)) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 17|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)==(0);
  if (!($16)) {
   $127 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($127|0);
  }
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 160|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$szPage>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 64|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3OsFileSize($22,$currentSize)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$szPage>>2]|0;
 $25 = ($24|0)<(0);
 $26 = $25 << 31 >> 31;
 $27 = HEAP32[$1>>2]|0;
 $28 = (___muldi3(($24|0),($26|0),($27|0),0)|0);
 $29 = tempRet0;
 $30 = $newSize;
 $31 = $30;
 HEAP32[$31>>2] = $28;
 $32 = (($30) + 4)|0;
 $33 = $32;
 HEAP32[$33>>2] = $29;
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)==(0);
 if (!($35)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $36 = $currentSize;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = $newSize;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = ($38|0)!=($44|0);
 $49 = ($41|0)!=($47|0);
 $50 = $48 | $49;
 if (!($50)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $51 = $currentSize;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (($51) + 4)|0;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = $newSize;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = ($56|0)>($62|0);
 $64 = ($53>>>0)>($59>>>0);
 $65 = ($56|0)==($62|0);
 $66 = $65 & $64;
 $67 = $63 | $66;
 if ($67) {
  $68 = HEAP32[$0>>2]|0;
  $69 = ((($68)) + 64|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = $newSize;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = (($71) + 4)|0;
  $75 = $74;
  $76 = HEAP32[$75>>2]|0;
  $77 = (_sqlite3OsTruncate($70,$73,$76)|0);
  HEAP32[$rc>>2] = $77;
 } else {
  $78 = $currentSize;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = HEAP32[$szPage>>2]|0;
  $85 = ($84|0)<(0);
  $86 = $85 << 31 >> 31;
  $87 = (_i64Add(($80|0),($83|0),($84|0),($86|0))|0);
  $88 = tempRet0;
  $89 = $newSize;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = ($88|0)<($94|0);
  $96 = ($87>>>0)<=($91>>>0);
  $97 = ($88|0)==($94|0);
  $98 = $97 & $96;
  $99 = $95 | $98;
  if ($99) {
   $100 = HEAP32[$0>>2]|0;
   $101 = ((($100)) + 208|0);
   $102 = HEAP32[$101>>2]|0;
   HEAP32[$pTmp>>2] = $102;
   $103 = HEAP32[$pTmp>>2]|0;
   $104 = HEAP32[$szPage>>2]|0;
   _memset(($103|0),0,($104|0))|0;
   $105 = HEAP32[$0>>2]|0;
   $106 = ((($105)) + 64|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = HEAP32[$pTmp>>2]|0;
   $109 = HEAP32[$szPage>>2]|0;
   $110 = $newSize;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $113 = (($110) + 4)|0;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = HEAP32[$szPage>>2]|0;
   $117 = ($116|0)<(0);
   $118 = $117 << 31 >> 31;
   $119 = (_i64Subtract(($112|0),($115|0),($116|0),($118|0))|0);
   $120 = tempRet0;
   $121 = (_sqlite3OsWrite($107,$108,$109,$119,$120)|0);
   HEAP32[$rc>>2] = $121;
  }
 }
 $122 = HEAP32[$rc>>2]|0;
 $123 = ($122|0)==(0);
 if (!($123)) {
  $127 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($127|0);
 }
 $124 = HEAP32[$1>>2]|0;
 $125 = HEAP32[$0>>2]|0;
 $126 = ((($125)) + 36|0);
 HEAP32[$126>>2] = $124;
 $127 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($127|0);
}
function _sqlite3WalExclusiveMode($pWal,$op) {
 $pWal = $pWal|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $op;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0);
 if (!($3)) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ($26|0)>(0);
  $28 = HEAP32[$0>>2]|0;
  if ($27) {
   $29 = HEAP32[$0>>2]|0;
   $30 = ((($29)) + 40|0);
   $31 = HEAP16[$30>>1]|0;
   $32 = $31 << 16 >> 16;
   $33 = (3 + ($32))|0;
   _walUnlockShared($28,$33);
   $34 = HEAP32[$0>>2]|0;
   $35 = ((($34)) + 43|0);
   HEAP8[$35>>0] = 1;
   HEAP32[$rc>>2] = 1;
   $41 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($41|0);
  } else {
   $36 = ((($28)) + 43|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)==(0);
   $40 = $39&1;
   HEAP32[$rc>>2] = $40;
   $41 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 43|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  HEAP32[$rc>>2] = 0;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 43|0);
 HEAP8[$9>>0] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 40|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13 << 16 >> 16;
 $15 = (3 + ($14))|0;
 $16 = (_walLockShared($10,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 43|0);
  HEAP8[$19>>0] = 1;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 43|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)==(0);
 $25 = $24&1;
 HEAP32[$rc>>2] = $25;
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _pagerUnlockDb($pPager,$eLock) {
 $pPager = $pPager|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eLock;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 14|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if ($11) {
  $17 = 0;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 64|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3OsUnlock($14,$15)|0);
  $17 = $16;
 }
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 18|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = ($21|0)!=(5);
 if (!($22)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = $23&255;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 18|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3OsUnlock($id,$lockType) {
 $id = $id|0;
 $lockType = $lockType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $lockType;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _walLockShared($pWal,$lockIdx) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $lockIdx;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_sqlite3OsShmLock($9,$10,1,6)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _walUnlockShared($pWal,$lockIdx) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $lockIdx;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 43|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 (_sqlite3OsShmLock($8,$9,1,5)|0);
 STACKTOP = sp;return;
}
function _sqlite3OsShmLock($id,$offset,$n,$flags) {
 $id = $id|0;
 $offset = $offset|0;
 $n = $n|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (FUNCTION_TABLE_iiiii[$7 & 255]($8,$9,$10,$11)|0);
 STACKTOP = sp;return ($12|0);
}
function _walUnlockExclusive($pWal,$lockIdx,$n) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $lockIdx;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 (_sqlite3OsShmLock($9,$10,$11,9)|0);
 STACKTOP = sp;return;
}
function _sqlite3PCachePercentDirty($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nCache = 0, $nDirty = 0, $pDirty = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pDirty = sp + 8|0;
 $nDirty = sp + 4|0;
 $nCache = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$nDirty>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_numberOfCachePages($1)|0);
 HEAP32[$nCache>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pDirty>>2] = $4;
 while(1) {
  $5 = HEAP32[$pDirty>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$nDirty>>2]|0;
  $8 = (($7) + 1)|0;
  HEAP32[$nDirty>>2] = $8;
  $9 = HEAP32[$pDirty>>2]|0;
  $10 = ((($9)) + 32|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pDirty>>2] = $11;
 }
 $12 = HEAP32[$nCache>>2]|0;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $24 = 0;
  STACKTOP = sp;return ($24|0);
 }
 $14 = HEAP32[$nDirty>>2]|0;
 $15 = ($14|0)<(0);
 $16 = $15 << 31 >> 31;
 $17 = (___muldi3(($14|0),($16|0),100,0)|0);
 $18 = tempRet0;
 $19 = HEAP32[$nCache>>2]|0;
 $20 = ($19|0)<(0);
 $21 = $20 << 31 >> 31;
 $22 = (___divdi3(($17|0),($18|0),($19|0),($21|0))|0);
 $23 = tempRet0;
 $24 = $22;
 STACKTOP = sp;return ($24|0);
}
function _memjrnlClose($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, $2 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 _memjrnlFreeChunks($2);
 STACKTOP = sp;return 0;
}
function _memjrnlRead($pJfd,$zBuf,$iAmt,$0,$1) {
 $pJfd = $pJfd|0;
 $zBuf = $zBuf|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iChunkOffset = 0, $iOff = 0, $iSpace = 0, $nCopy = 0, $nRead = 0, $or$cond = 0, $or$cond5 = 0, $p = 0, $pChunk = 0, $zOut = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 8|0;
 $p = sp + 40|0;
 $zOut = sp + 36|0;
 $nRead = sp + 32|0;
 $iChunkOffset = sp + 28|0;
 $pChunk = sp + 24|0;
 $iOff = sp;
 $iSpace = sp + 20|0;
 $nCopy = sp + 16|0;
 HEAP32[$2>>2] = $pJfd;
 HEAP32[$3>>2] = $zBuf;
 HEAP32[$4>>2] = $iAmt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$zOut>>2] = $11;
 $12 = HEAP32[$4>>2]|0;
 HEAP32[$nRead>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ((($13)) + 40|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $5;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = ($17|0)!=($23|0);
 $28 = ($20|0)!=($26|0);
 $29 = $27 | $28;
 $30 = $5;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($32|0)==(0);
 $37 = ($35|0)==(0);
 $38 = $36 & $37;
 $or$cond = $29 | $38;
 L1: do {
  if ($or$cond) {
   $39 = $iOff;
   $40 = $39;
   HEAP32[$40>>2] = 0;
   $41 = (($39) + 4)|0;
   $42 = $41;
   HEAP32[$42>>2] = 0;
   $43 = HEAP32[$p>>2]|0;
   $44 = ((($43)) + 16|0);
   $45 = HEAP32[$44>>2]|0;
   HEAP32[$pChunk>>2] = $45;
   while(1) {
    $46 = HEAP32[$pChunk>>2]|0;
    $47 = ($46|0)!=(0|0);
    if (!($47)) {
     break L1;
    }
    $48 = $iOff;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($48) + 4)|0;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = HEAP32[$p>>2]|0;
    $55 = ((($54)) + 4|0);
    $56 = HEAP32[$55>>2]|0;
    $57 = ($56|0)<(0);
    $58 = $57 << 31 >> 31;
    $59 = (_i64Add(($50|0),($53|0),($56|0),($58|0))|0);
    $60 = tempRet0;
    $61 = $5;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (($61) + 4)|0;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = ($60|0)<($66|0);
    $68 = ($59>>>0)<=($63>>>0);
    $69 = ($60|0)==($66|0);
    $70 = $69 & $68;
    $71 = $67 | $70;
    if (!($71)) {
     break L1;
    }
    $72 = HEAP32[$p>>2]|0;
    $73 = ((($72)) + 4|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($74|0)<(0);
    $76 = $75 << 31 >> 31;
    $77 = $iOff;
    $78 = $77;
    $79 = HEAP32[$78>>2]|0;
    $80 = (($77) + 4)|0;
    $81 = $80;
    $82 = HEAP32[$81>>2]|0;
    $83 = (_i64Add(($79|0),($82|0),($74|0),($76|0))|0);
    $84 = tempRet0;
    $85 = $iOff;
    $86 = $85;
    HEAP32[$86>>2] = $83;
    $87 = (($85) + 4)|0;
    $88 = $87;
    HEAP32[$88>>2] = $84;
    $89 = HEAP32[$pChunk>>2]|0;
    $90 = HEAP32[$89>>2]|0;
    HEAP32[$pChunk>>2] = $90;
   }
  } else {
   $91 = HEAP32[$p>>2]|0;
   $92 = ((($91)) + 40|0);
   $93 = ((($92)) + 8|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$pChunk>>2] = $94;
  }
 } while(0);
 $95 = $5;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (($95) + 4)|0;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = HEAP32[$p>>2]|0;
 $102 = ((($101)) + 4|0);
 $103 = HEAP32[$102>>2]|0;
 $104 = ($103|0)<(0);
 $105 = $104 << 31 >> 31;
 $106 = (___remdi3(($97|0),($100|0),($103|0),($105|0))|0);
 $107 = tempRet0;
 HEAP32[$iChunkOffset>>2] = $106;
 while(1) {
  $108 = HEAP32[$p>>2]|0;
  $109 = ((($108)) + 4|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = HEAP32[$iChunkOffset>>2]|0;
  $112 = (($110) - ($111))|0;
  HEAP32[$iSpace>>2] = $112;
  $113 = HEAP32[$nRead>>2]|0;
  $114 = HEAP32[$p>>2]|0;
  $115 = ((($114)) + 4|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = HEAP32[$iChunkOffset>>2]|0;
  $118 = (($116) - ($117))|0;
  $119 = ($113|0)<($118|0);
  if ($119) {
   $120 = HEAP32[$nRead>>2]|0;
   $126 = $120;
  } else {
   $121 = HEAP32[$p>>2]|0;
   $122 = ((($121)) + 4|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = HEAP32[$iChunkOffset>>2]|0;
   $125 = (($123) - ($124))|0;
   $126 = $125;
  }
  HEAP32[$nCopy>>2] = $126;
  $127 = HEAP32[$zOut>>2]|0;
  $128 = HEAP32[$pChunk>>2]|0;
  $129 = ((($128)) + 4|0);
  $130 = HEAP32[$iChunkOffset>>2]|0;
  $131 = (($129) + ($130)|0);
  $132 = HEAP32[$nCopy>>2]|0;
  _memcpy(($127|0),($131|0),($132|0))|0;
  $133 = HEAP32[$nCopy>>2]|0;
  $134 = HEAP32[$zOut>>2]|0;
  $135 = (($134) + ($133)|0);
  HEAP32[$zOut>>2] = $135;
  $136 = HEAP32[$iSpace>>2]|0;
  $137 = HEAP32[$nRead>>2]|0;
  $138 = (($137) - ($136))|0;
  HEAP32[$nRead>>2] = $138;
  HEAP32[$iChunkOffset>>2] = 0;
  $139 = HEAP32[$nRead>>2]|0;
  $140 = ($139|0)>=(0);
  if (!($140)) {
   break;
  }
  $141 = HEAP32[$pChunk>>2]|0;
  $142 = HEAP32[$141>>2]|0;
  HEAP32[$pChunk>>2] = $142;
  $143 = ($142|0)!=(0|0);
  $144 = HEAP32[$nRead>>2]|0;
  $145 = ($144|0)>(0);
  $or$cond5 = $143 & $145;
  if (!($or$cond5)) {
   break;
  }
 }
 $146 = HEAP32[$pChunk>>2]|0;
 $147 = ($146|0)!=(0|0);
 if (!($147)) {
  $163 = 0;$166 = 0;
  $159 = HEAP32[$p>>2]|0;
  $160 = ((($159)) + 40|0);
  $161 = $160;
  $162 = $161;
  HEAP32[$162>>2] = $163;
  $164 = (($161) + 4)|0;
  $165 = $164;
  HEAP32[$165>>2] = $166;
  $167 = HEAP32[$pChunk>>2]|0;
  $168 = HEAP32[$p>>2]|0;
  $169 = ((($168)) + 40|0);
  $170 = ((($169)) + 8|0);
  HEAP32[$170>>2] = $167;
  STACKTOP = sp;return 0;
 }
 $148 = $5;
 $149 = $148;
 $150 = HEAP32[$149>>2]|0;
 $151 = (($148) + 4)|0;
 $152 = $151;
 $153 = HEAP32[$152>>2]|0;
 $154 = HEAP32[$4>>2]|0;
 $155 = ($154|0)<(0);
 $156 = $155 << 31 >> 31;
 $157 = (_i64Add(($150|0),($153|0),($154|0),($156|0))|0);
 $158 = tempRet0;
 $163 = $157;$166 = $158;
 $159 = HEAP32[$p>>2]|0;
 $160 = ((($159)) + 40|0);
 $161 = $160;
 $162 = $161;
 HEAP32[$162>>2] = $163;
 $164 = (($161) + 4)|0;
 $165 = $164;
 HEAP32[$165>>2] = $166;
 $167 = HEAP32[$pChunk>>2]|0;
 $168 = HEAP32[$p>>2]|0;
 $169 = ((($168)) + 40|0);
 $170 = ((($169)) + 8|0);
 HEAP32[$170>>2] = $167;
 STACKTOP = sp;return 0;
}
function _memjrnlWrite($pJfd,$zBuf,$iAmt,$0,$1) {
 $pJfd = $pJfd|0;
 $zBuf = $zBuf|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iChunkOffset = 0, $iSpace = 0, $nWrite = 0, $p = 0, $pChunk = 0, $pNew = 0;
 var $rc = 0, $zWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 40|0;
 $6 = sp;
 $p = sp + 36|0;
 $nWrite = sp + 32|0;
 $zWrite = sp + 28|0;
 $rc = sp + 24|0;
 $pChunk = sp + 20|0;
 $iChunkOffset = sp + 16|0;
 $iSpace = sp + 12|0;
 $pNew = sp + 8|0;
 HEAP32[$3>>2] = $pJfd;
 HEAP32[$4>>2] = $zBuf;
 HEAP32[$5>>2] = $iAmt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $11;
 $12 = HEAP32[$5>>2]|0;
 HEAP32[$nWrite>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 HEAP32[$zWrite>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)>(0);
 if ($17) {
  $18 = HEAP32[$5>>2]|0;
  $19 = ($18|0)<(0);
  $20 = $19 << 31 >> 31;
  $21 = $6;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = (_i64Add(($18|0),($20|0),($23|0),($26|0))|0);
  $28 = tempRet0;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)<(0);
  $33 = $32 << 31 >> 31;
  $34 = ($28|0)>($33|0);
  $35 = ($27>>>0)>($31>>>0);
  $36 = ($28|0)==($33|0);
  $37 = $36 & $35;
  $38 = $34 | $37;
  if ($38) {
   $39 = HEAP32[$p>>2]|0;
   $40 = (_memjrnlCreateFile($39)|0);
   HEAP32[$rc>>2] = $40;
   $41 = HEAP32[$rc>>2]|0;
   $42 = ($41|0)==(0);
   if ($42) {
    $43 = HEAP32[$3>>2]|0;
    $44 = HEAP32[$4>>2]|0;
    $45 = HEAP32[$5>>2]|0;
    $46 = $6;
    $47 = $46;
    $48 = HEAP32[$47>>2]|0;
    $49 = (($46) + 4)|0;
    $50 = $49;
    $51 = HEAP32[$50>>2]|0;
    $52 = (_sqlite3OsWrite($43,$44,$45,$48,$51)|0);
    HEAP32[$rc>>2] = $52;
   }
   $53 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $53;
   $155 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($155|0);
  }
 }
 while(1) {
  $54 = HEAP32[$nWrite>>2]|0;
  $55 = ($54|0)>(0);
  if (!($55)) {
   label = 18;
   break;
  }
  $56 = HEAP32[$p>>2]|0;
  $57 = ((($56)) + 24|0);
  $58 = ((($57)) + 8|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$pChunk>>2] = $59;
  $60 = HEAP32[$p>>2]|0;
  $61 = ((($60)) + 24|0);
  $62 = $61;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = HEAP32[$p>>2]|0;
  $69 = ((($68)) + 4|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = ($70|0)<(0);
  $72 = $71 << 31 >> 31;
  $73 = (___remdi3(($64|0),($67|0),($70|0),($72|0))|0);
  $74 = tempRet0;
  HEAP32[$iChunkOffset>>2] = $73;
  $75 = HEAP32[$nWrite>>2]|0;
  $76 = HEAP32[$p>>2]|0;
  $77 = ((($76)) + 4|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = HEAP32[$iChunkOffset>>2]|0;
  $80 = (($78) - ($79))|0;
  $81 = ($75|0)<($80|0);
  if ($81) {
   $82 = HEAP32[$nWrite>>2]|0;
   $88 = $82;
  } else {
   $83 = HEAP32[$p>>2]|0;
   $84 = ((($83)) + 4|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = HEAP32[$iChunkOffset>>2]|0;
   $87 = (($85) - ($86))|0;
   $88 = $87;
  }
  HEAP32[$iSpace>>2] = $88;
  $89 = HEAP32[$iChunkOffset>>2]|0;
  $90 = ($89|0)==(0);
  if ($90) {
   $91 = HEAP32[$p>>2]|0;
   $92 = ((($91)) + 4|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = (($93) - 8)|0;
   $95 = (12 + ($94))|0;
   $96 = (_sqlite3_malloc($95)|0);
   HEAP32[$pNew>>2] = $96;
   $97 = HEAP32[$pNew>>2]|0;
   $98 = ($97|0)!=(0|0);
   if (!($98)) {
    label = 12;
    break;
   }
   $99 = HEAP32[$pNew>>2]|0;
   HEAP32[$99>>2] = 0;
   $100 = HEAP32[$pChunk>>2]|0;
   $101 = ($100|0)!=(0|0);
   $102 = HEAP32[$pNew>>2]|0;
   if ($101) {
    $103 = HEAP32[$pChunk>>2]|0;
    HEAP32[$103>>2] = $102;
   } else {
    $104 = HEAP32[$p>>2]|0;
    $105 = ((($104)) + 16|0);
    HEAP32[$105>>2] = $102;
   }
   $106 = HEAP32[$pNew>>2]|0;
   $107 = HEAP32[$p>>2]|0;
   $108 = ((($107)) + 24|0);
   $109 = ((($108)) + 8|0);
   HEAP32[$109>>2] = $106;
  }
  $110 = HEAP32[$p>>2]|0;
  $111 = ((($110)) + 24|0);
  $112 = ((($111)) + 8|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ((($113)) + 4|0);
  $115 = HEAP32[$iChunkOffset>>2]|0;
  $116 = (($114) + ($115)|0);
  $117 = HEAP32[$zWrite>>2]|0;
  $118 = HEAP32[$iSpace>>2]|0;
  _memcpy(($116|0),($117|0),($118|0))|0;
  $119 = HEAP32[$iSpace>>2]|0;
  $120 = HEAP32[$zWrite>>2]|0;
  $121 = (($120) + ($119)|0);
  HEAP32[$zWrite>>2] = $121;
  $122 = HEAP32[$iSpace>>2]|0;
  $123 = HEAP32[$nWrite>>2]|0;
  $124 = (($123) - ($122))|0;
  HEAP32[$nWrite>>2] = $124;
  $125 = HEAP32[$iSpace>>2]|0;
  $126 = ($125|0)<(0);
  $127 = $126 << 31 >> 31;
  $128 = HEAP32[$p>>2]|0;
  $129 = ((($128)) + 24|0);
  $130 = $129;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  $136 = (_i64Add(($132|0),($135|0),($125|0),($127|0))|0);
  $137 = tempRet0;
  $138 = $129;
  $139 = $138;
  HEAP32[$139>>2] = $136;
  $140 = (($138) + 4)|0;
  $141 = $140;
  HEAP32[$141>>2] = $137;
 }
 if ((label|0) == 12) {
  HEAP32[$2>>2] = 3082;
  $155 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 else if ((label|0) == 18) {
  $142 = HEAP32[$5>>2]|0;
  $143 = ($142|0)<(0);
  $144 = $143 << 31 >> 31;
  $145 = $6;
  $146 = $145;
  $147 = HEAP32[$146>>2]|0;
  $148 = (($145) + 4)|0;
  $149 = $148;
  $150 = HEAP32[$149>>2]|0;
  $151 = (_i64Add(($142|0),($144|0),($147|0),($150|0))|0);
  $152 = tempRet0;
  $153 = HEAP32[$p>>2]|0;
  $154 = ((($153)) + 12|0);
  HEAP32[$154>>2] = $151;
  HEAP32[$2>>2] = 0;
  $155 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 return (0)|0;
}
function _memjrnlTruncate($pJfd,$0,$1) {
 $pJfd = $pJfd|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $pJfd;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $8;
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)==(0);
 $16 = ($14|0)==(0);
 $17 = $15 & $16;
 if (!($17)) {
  STACKTOP = sp;return 0;
 }
 $18 = HEAP32[$p>>2]|0;
 _memjrnlFreeChunks($18);
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 12|0);
 HEAP32[$20>>2] = 0;
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 24|0);
 $23 = ((($22)) + 8|0);
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = $25;
 $27 = $26;
 HEAP32[$27>>2] = 0;
 $28 = (($26) + 4)|0;
 $29 = $28;
 HEAP32[$29>>2] = 0;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 40|0);
 $32 = ((($31)) + 8|0);
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$p>>2]|0;
 $34 = ((($33)) + 40|0);
 $35 = $34;
 $36 = $35;
 HEAP32[$36>>2] = 0;
 $37 = (($35) + 4)|0;
 $38 = $37;
 HEAP32[$38>>2] = 0;
 STACKTOP = sp;return 0;
}
function _memjrnlSync($pJfd,$flags) {
 $pJfd = $pJfd|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pJfd;
 HEAP32[$1>>2] = $flags;
 STACKTOP = sp;return 0;
}
function _memjrnlFileSize($pJfd,$pSize) {
 $pJfd = $pJfd|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pJfd;
 HEAP32[$1>>2] = $pSize;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11;
 $13 = $12;
 HEAP32[$13>>2] = $7;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $10;
 STACKTOP = sp;return 0;
}
function _memjrnlFreeChunks($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pIter = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pIter = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pIter>>2] = $3;
 while(1) {
  $4 = HEAP32[$pIter>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$pIter>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pNext>>2] = $7;
  $8 = HEAP32[$pIter>>2]|0;
  _sqlite3_free($8);
  $9 = HEAP32[$pNext>>2]|0;
  HEAP32[$pIter>>2] = $9;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 16|0);
 HEAP32[$11>>2] = 0;
 STACKTOP = sp;return;
}
function _memjrnlCreateFile($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $copy = 0;
 var $iOff = 0, $nChunk = 0, $pIter = 0, $pReal = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $rc = sp + 92|0;
 $pReal = sp + 88|0;
 $copy = sp + 8|0;
 $nChunk = sp + 84|0;
 $iOff = sp;
 $pIter = sp + 80|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pReal>>2] = $1;
 $2 = HEAP32[$0>>2]|0;
 dest=$copy; src=$2; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $3 = HEAP32[$0>>2]|0;
 dest=$3; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $4 = ((($copy)) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($copy)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$pReal>>2]|0;
 $9 = ((($copy)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3OsOpen($5,$7,$8,$10,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = ((($copy)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$nChunk>>2] = $15;
  $16 = $iOff;
  $17 = $16;
  HEAP32[$17>>2] = 0;
  $18 = (($16) + 4)|0;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = ((($copy)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pIter>>2] = $21;
  while(1) {
   $22 = HEAP32[$pIter>>2]|0;
   $23 = ($22|0)!=(0|0);
   if (!($23)) {
    break;
   }
   $24 = $iOff;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + 4)|0;
   $28 = $27;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$nChunk>>2]|0;
   $31 = ($30|0)<(0);
   $32 = $31 << 31 >> 31;
   $33 = (_i64Add(($26|0),($29|0),($30|0),($32|0))|0);
   $34 = tempRet0;
   $35 = ((($copy)) + 24|0);
   $36 = $35;
   $37 = $36;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($36) + 4)|0;
   $40 = $39;
   $41 = HEAP32[$40>>2]|0;
   $42 = ($34|0)>($41|0);
   $43 = ($33>>>0)>($38>>>0);
   $44 = ($34|0)==($41|0);
   $45 = $44 & $43;
   $46 = $42 | $45;
   if ($46) {
    $47 = ((($copy)) + 24|0);
    $48 = $47;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($48) + 4)|0;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = $iOff;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = (($54) + 4)|0;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = (_i64Subtract(($50|0),($53|0),($56|0),($59|0))|0);
    $61 = tempRet0;
    HEAP32[$nChunk>>2] = $60;
   }
   $62 = HEAP32[$pReal>>2]|0;
   $63 = HEAP32[$pIter>>2]|0;
   $64 = ((($63)) + 4|0);
   $65 = HEAP32[$nChunk>>2]|0;
   $66 = $iOff;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = (($66) + 4)|0;
   $70 = $69;
   $71 = HEAP32[$70>>2]|0;
   $72 = (_sqlite3OsWrite($62,$64,$65,$68,$71)|0);
   HEAP32[$rc>>2] = $72;
   $73 = HEAP32[$rc>>2]|0;
   $74 = ($73|0)!=(0);
   if ($74) {
    break;
   }
   $75 = HEAP32[$nChunk>>2]|0;
   $76 = ($75|0)<(0);
   $77 = $76 << 31 >> 31;
   $78 = $iOff;
   $79 = $78;
   $80 = HEAP32[$79>>2]|0;
   $81 = (($78) + 4)|0;
   $82 = $81;
   $83 = HEAP32[$82>>2]|0;
   $84 = (_i64Add(($80|0),($83|0),($75|0),($77|0))|0);
   $85 = tempRet0;
   $86 = $iOff;
   $87 = $86;
   HEAP32[$87>>2] = $84;
   $88 = (($86) + 4)|0;
   $89 = $88;
   HEAP32[$89>>2] = $85;
   $90 = HEAP32[$pIter>>2]|0;
   $91 = HEAP32[$90>>2]|0;
   HEAP32[$pIter>>2] = $91;
  }
  $92 = HEAP32[$rc>>2]|0;
  $93 = ($92|0)==(0);
  if ($93) {
   _memjrnlFreeChunks($copy);
  }
 }
 $94 = HEAP32[$rc>>2]|0;
 $95 = ($94|0)!=(0);
 if (!($95)) {
  $98 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 $96 = HEAP32[$pReal>>2]|0;
 _sqlite3OsClose($96);
 $97 = HEAP32[$0>>2]|0;
 dest=$97; src=$copy; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $98 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($98|0);
}
function _sqlite3OsOpen($pVfs,$zPath,$pFile,$flags,$pFlagsOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $pFile = $pFile|0;
 $flags = $flags|0;
 $pFlagsOut = $pFlagsOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $pFile;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $pFlagsOut;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = $11 & 556927;
 $13 = HEAP32[$4>>2]|0;
 $14 = (FUNCTION_TABLE_iiiiii[$7 & 255]($8,$9,$10,$12,$13)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3PagerSavepoint($pPager,$op,$iSavepoint) {
 $pPager = $pPager|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $ii = 0, $nNew = 0, $pSavepoint = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $ii = sp + 8|0;
 $nNew = sp + 4|0;
 $pSavepoint = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$rc>>2]|0;
 $7 = ($6|0)==(0);
 if (!($7)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 104|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)<($11|0);
 if (!($12)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ($14|0)==(1);
 $16 = $15 ? 0 : 1;
 $17 = (($13) + ($16))|0;
 HEAP32[$nNew>>2] = $17;
 $18 = HEAP32[$nNew>>2]|0;
 HEAP32[$ii>>2] = $18;
 while(1) {
  $19 = HEAP32[$ii>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 104|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($19|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$ii>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 100|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + (($24*48)|0)|0);
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3BitvecDestroy($30);
  $31 = HEAP32[$ii>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$ii>>2] = $32;
 }
 $33 = HEAP32[$nNew>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 104|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$1>>2]|0;
 $37 = ($36|0)==(1);
 if (!($37)) {
  $56 = HEAP32[$0>>2]|0;
  $57 = (_sqlite3PagerUseWal($56)|0);
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = HEAP32[$0>>2]|0;
   $60 = ((($59)) + 68|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$61>>2]|0;
   $63 = ($62|0)!=(0|0);
   if (!($63)) {
    $76 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($76|0);
   }
  }
  $64 = HEAP32[$nNew>>2]|0;
  $65 = ($64|0)==(0);
  if ($65) {
   $72 = 0;
  } else {
   $66 = HEAP32[$nNew>>2]|0;
   $67 = (($66) - 1)|0;
   $68 = HEAP32[$0>>2]|0;
   $69 = ((($68)) + 100|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + (($67*48)|0)|0);
   $72 = $71;
  }
  HEAP32[$pSavepoint>>2] = $72;
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$pSavepoint>>2]|0;
  $75 = (_pagerPlaybackSavepoint($73,$74)|0);
  HEAP32[$rc>>2] = $75;
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $38 = HEAP32[$nNew>>2]|0;
 $39 = ($38|0)==(0);
 if (!($39)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 72|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0|0);
 if (!($44)) {
  $76 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 72|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (_sqlite3JournalIsInMemory($47)|0);
 $49 = ($48|0)!=(0);
 if ($49) {
  $50 = HEAP32[$0>>2]|0;
  $51 = ((($50)) + 72|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (_sqlite3OsTruncate($52,0,0)|0);
  HEAP32[$rc>>2] = $53;
 }
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 56|0);
 HEAP32[$55>>2] = 0;
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _pager_playback($pPager,$isHot) {
 $pPager = $pPager|0;
 $isHot = $isHot|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $mxPg = 0, $nPlayback = 0, $nRec = 0, $needPagerReset = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pVfs = 0, $rc = 0, $res = 0, $szJ = 0, $u = 0;
 var $vararg_buffer = 0, $vararg_ptr8 = 0, $zMaster = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $pVfs = sp + 48|0;
 $szJ = sp;
 $nRec = sp + 44|0;
 $u = sp + 40|0;
 $mxPg = sp + 36|0;
 $rc = sp + 32|0;
 $res = sp + 28|0;
 $zMaster = sp + 24|0;
 $needPagerReset = sp + 20|0;
 $nPlayback = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $isHot;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$mxPg>>2] = 0;
 HEAP32[$res>>2] = 1;
 HEAP32[$zMaster>>2] = 0;
 HEAP32[$nPlayback>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3OsFileSize($6,$szJ)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 L1: do {
  if (!($9)) {
   $10 = HEAP32[$0>>2]|0;
   $11 = ((($10)) + 208|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$zMaster>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 68|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$zMaster>>2]|0;
   $17 = HEAP32[$0>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = ((($18)) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + 1)|0;
   $22 = (_readMasterJournal($15,$16,$21)|0);
   HEAP32[$rc>>2] = $22;
   $23 = HEAP32[$rc>>2]|0;
   $24 = ($23|0)==(0);
   if ($24) {
    $25 = HEAP32[$zMaster>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)!=(0);
    if ($28) {
     $29 = HEAP32[$pVfs>>2]|0;
     $30 = HEAP32[$zMaster>>2]|0;
     $31 = (_sqlite3OsAccess($29,$30,0,$res)|0);
     HEAP32[$rc>>2] = $31;
    }
   }
   HEAP32[$zMaster>>2] = 0;
   $32 = HEAP32[$rc>>2]|0;
   $33 = ($32|0)==(0);
   $34 = HEAP32[$res>>2]|0;
   $35 = ($34|0)!=(0);
   $or$cond = $33 & $35;
   if ($or$cond) {
    $36 = HEAP32[$0>>2]|0;
    $37 = ((($36)) + 80|0);
    $38 = $37;
    $39 = $38;
    HEAP32[$39>>2] = 0;
    $40 = (($38) + 4)|0;
    $41 = $40;
    HEAP32[$41>>2] = 0;
    $42 = HEAP32[$1>>2]|0;
    HEAP32[$needPagerReset>>2] = $42;
    L8: while(1) {
     $43 = HEAP32[$0>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = $szJ;
     $46 = $45;
     $47 = HEAP32[$46>>2]|0;
     $48 = (($45) + 4)|0;
     $49 = $48;
     $50 = HEAP32[$49>>2]|0;
     $51 = (_readJournalHdr($43,$44,$47,$50,$nRec,$mxPg)|0);
     HEAP32[$rc>>2] = $51;
     $52 = HEAP32[$rc>>2]|0;
     $53 = ($52|0)!=(0);
     if ($53) {
      label = 8;
      break;
     }
     $56 = HEAP32[$nRec>>2]|0;
     $57 = ($56|0)==(-1);
     if ($57) {
      $58 = $szJ;
      $59 = $58;
      $60 = HEAP32[$59>>2]|0;
      $61 = (($58) + 4)|0;
      $62 = $61;
      $63 = HEAP32[$62>>2]|0;
      $64 = HEAP32[$0>>2]|0;
      $65 = ((($64)) + 156|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = (_i64Subtract(($60|0),($63|0),($66|0),0)|0);
      $68 = tempRet0;
      $69 = HEAP32[$0>>2]|0;
      $70 = ((($69)) + 160|0);
      $71 = HEAP32[$70>>2]|0;
      $72 = (($71) + 8)|0;
      $73 = ($72|0)<(0);
      $74 = $73 << 31 >> 31;
      $75 = (___divdi3(($67|0),($68|0),($72|0),($74|0))|0);
      $76 = tempRet0;
      HEAP32[$nRec>>2] = $75;
     }
     $77 = HEAP32[$nRec>>2]|0;
     $78 = ($77|0)!=(0);
     $79 = HEAP32[$1>>2]|0;
     $80 = ($79|0)!=(0);
     $or$cond3 = $78 | $80;
     if (!($or$cond3)) {
      $81 = HEAP32[$0>>2]|0;
      $82 = ((($81)) + 88|0);
      $83 = $82;
      $84 = $83;
      $85 = HEAP32[$84>>2]|0;
      $86 = (($83) + 4)|0;
      $87 = $86;
      $88 = HEAP32[$87>>2]|0;
      $89 = HEAP32[$0>>2]|0;
      $90 = ((($89)) + 156|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = (_i64Add(($85|0),($88|0),($91|0),0)|0);
      $93 = tempRet0;
      $94 = HEAP32[$0>>2]|0;
      $95 = ((($94)) + 80|0);
      $96 = $95;
      $97 = $96;
      $98 = HEAP32[$97>>2]|0;
      $99 = (($96) + 4)|0;
      $100 = $99;
      $101 = HEAP32[$100>>2]|0;
      $102 = ($92|0)==($98|0);
      $103 = ($93|0)==($101|0);
      $104 = $102 & $103;
      if ($104) {
       $105 = $szJ;
       $106 = $105;
       $107 = HEAP32[$106>>2]|0;
       $108 = (($105) + 4)|0;
       $109 = $108;
       $110 = HEAP32[$109>>2]|0;
       $111 = HEAP32[$0>>2]|0;
       $112 = ((($111)) + 80|0);
       $113 = $112;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (($113) + 4)|0;
       $117 = $116;
       $118 = HEAP32[$117>>2]|0;
       $119 = (_i64Subtract(($107|0),($110|0),($115|0),($118|0))|0);
       $120 = tempRet0;
       $121 = HEAP32[$0>>2]|0;
       $122 = ((($121)) + 160|0);
       $123 = HEAP32[$122>>2]|0;
       $124 = (($123) + 8)|0;
       $125 = ($124|0)<(0);
       $126 = $125 << 31 >> 31;
       $127 = (___divdi3(($119|0),($120|0),($124|0),($126|0))|0);
       $128 = tempRet0;
       HEAP32[$nRec>>2] = $127;
      }
     }
     $129 = HEAP32[$0>>2]|0;
     $130 = ((($129)) + 80|0);
     $131 = $130;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = (($131) + 4)|0;
     $135 = $134;
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$0>>2]|0;
     $138 = ((($137)) + 156|0);
     $139 = HEAP32[$138>>2]|0;
     $140 = ($133|0)==($139|0);
     $141 = ($136|0)==(0);
     $142 = $140 & $141;
     if ($142) {
      $143 = HEAP32[$0>>2]|0;
      $144 = HEAP32[$mxPg>>2]|0;
      $145 = (_pager_truncate($143,$144)|0);
      HEAP32[$rc>>2] = $145;
      $146 = HEAP32[$rc>>2]|0;
      $147 = ($146|0)!=(0);
      if ($147) {
       break L1;
      }
      $148 = HEAP32[$mxPg>>2]|0;
      $149 = HEAP32[$0>>2]|0;
      $150 = ((($149)) + 28|0);
      HEAP32[$150>>2] = $148;
     }
     HEAP32[$u>>2] = 0;
     while(1) {
      $151 = HEAP32[$u>>2]|0;
      $152 = HEAP32[$nRec>>2]|0;
      $153 = ($151>>>0)<($152>>>0);
      if (!($153)) {
       continue L8;
      }
      $154 = HEAP32[$needPagerReset>>2]|0;
      $155 = ($154|0)!=(0);
      if ($155) {
       $156 = HEAP32[$0>>2]|0;
       _pager_reset($156);
       HEAP32[$needPagerReset>>2] = 0;
      }
      $157 = HEAP32[$0>>2]|0;
      $158 = HEAP32[$0>>2]|0;
      $159 = ((($158)) + 80|0);
      $160 = (_pager_playback_one_page($157,$159,0,1,0)|0);
      HEAP32[$rc>>2] = $160;
      $161 = HEAP32[$rc>>2]|0;
      $162 = ($161|0)==(0);
      if (!($162)) {
       break;
      }
      $163 = HEAP32[$nPlayback>>2]|0;
      $164 = (($163) + 1)|0;
      HEAP32[$nPlayback>>2] = $164;
      $165 = HEAP32[$u>>2]|0;
      $166 = (($165) + 1)|0;
      HEAP32[$u>>2] = $166;
     }
     $167 = HEAP32[$rc>>2]|0;
     $168 = ($167|0)==(101);
     if (!($168)) {
      label = 26;
      break;
     }
     $169 = $szJ;
     $170 = $169;
     $171 = HEAP32[$170>>2]|0;
     $172 = (($169) + 4)|0;
     $173 = $172;
     $174 = HEAP32[$173>>2]|0;
     $175 = HEAP32[$0>>2]|0;
     $176 = ((($175)) + 80|0);
     $177 = $176;
     $178 = $177;
     HEAP32[$178>>2] = $171;
     $179 = (($177) + 4)|0;
     $180 = $179;
     HEAP32[$180>>2] = $174;
    }
    if ((label|0) == 8) {
     $54 = HEAP32[$rc>>2]|0;
     $55 = ($54|0)==(101);
     if (!($55)) {
      break;
     }
     HEAP32[$rc>>2] = 0;
     break;
    }
    else if ((label|0) == 26) {
     $181 = HEAP32[$rc>>2]|0;
     $182 = ($181|0)==(522);
     if (!($182)) {
      break;
     }
     HEAP32[$rc>>2] = 0;
     break;
    }
   }
  }
 } while(0);
 $183 = HEAP32[$0>>2]|0;
 $184 = ((($183)) + 13|0);
 $185 = HEAP8[$184>>0]|0;
 $186 = HEAP32[$0>>2]|0;
 $187 = ((($186)) + 19|0);
 HEAP8[$187>>0] = $185;
 $188 = HEAP32[$rc>>2]|0;
 $189 = ($188|0)==(0);
 if ($189) {
  $190 = HEAP32[$0>>2]|0;
  $191 = ((($190)) + 208|0);
  $192 = HEAP32[$191>>2]|0;
  HEAP32[$zMaster>>2] = $192;
  $193 = HEAP32[$0>>2]|0;
  $194 = ((($193)) + 68|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = HEAP32[$zMaster>>2]|0;
  $197 = HEAP32[$0>>2]|0;
  $198 = HEAP32[$197>>2]|0;
  $199 = ((($198)) + 8|0);
  $200 = HEAP32[$199>>2]|0;
  $201 = (($200) + 1)|0;
  $202 = (_readMasterJournal($195,$196,$201)|0);
  HEAP32[$rc>>2] = $202;
 }
 $203 = HEAP32[$rc>>2]|0;
 $204 = ($203|0)==(0);
 do {
  if ($204) {
   $205 = HEAP32[$0>>2]|0;
   $206 = ((($205)) + 17|0);
   $207 = HEAP8[$206>>0]|0;
   $208 = $207&255;
   $209 = ($208|0)>=(4);
   if (!($209)) {
    $210 = HEAP32[$0>>2]|0;
    $211 = ((($210)) + 17|0);
    $212 = HEAP8[$211>>0]|0;
    $213 = $212&255;
    $214 = ($213|0)==(0);
    if (!($214)) {
     break;
    }
   }
   $215 = HEAP32[$0>>2]|0;
   $216 = (_sqlite3PagerSync($215,0)|0);
   HEAP32[$rc>>2] = $216;
  }
 } while(0);
 $217 = HEAP32[$rc>>2]|0;
 $218 = ($217|0)==(0);
 if ($218) {
  $219 = HEAP32[$0>>2]|0;
  $220 = HEAP32[$zMaster>>2]|0;
  $221 = HEAP8[$220>>0]|0;
  $222 = $221 << 24 >> 24;
  $223 = ($222|0)!=(0);
  $224 = $223&1;
  $225 = (_pager_end_transaction($219,$224,0)|0);
  HEAP32[$rc>>2] = $225;
 }
 $226 = HEAP32[$rc>>2]|0;
 $227 = ($226|0)==(0);
 if ($227) {
  $228 = HEAP32[$zMaster>>2]|0;
  $229 = HEAP8[$228>>0]|0;
  $230 = $229 << 24 >> 24;
  $231 = ($230|0)!=(0);
  $232 = HEAP32[$res>>2]|0;
  $233 = ($232|0)!=(0);
  $or$cond5 = $231 & $233;
  if ($or$cond5) {
   $234 = HEAP32[$0>>2]|0;
   $235 = HEAP32[$zMaster>>2]|0;
   $236 = (_pager_delmaster($234,$235)|0);
   HEAP32[$rc>>2] = $236;
  }
 }
 $237 = HEAP32[$1>>2]|0;
 $238 = ($237|0)!=(0);
 $239 = HEAP32[$nPlayback>>2]|0;
 $240 = ($239|0)!=(0);
 $or$cond7 = $238 & $240;
 if (!($or$cond7)) {
  $245 = HEAP32[$0>>2]|0;
  _setSectorSize($245);
  $246 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($246|0);
 }
 $241 = HEAP32[$nPlayback>>2]|0;
 $242 = HEAP32[$0>>2]|0;
 $243 = ((($242)) + 180|0);
 $244 = HEAP32[$243>>2]|0;
 HEAP32[$vararg_buffer>>2] = $241;
 $vararg_ptr8 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $244;
 _sqlite3_log(539,25874,$vararg_buffer);
 $245 = HEAP32[$0>>2]|0;
 _setSectorSize($245);
 $246 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($246|0);
}
function _readMasterJournal($pJrnl,$zMaster,$nMaster) {
 $pJrnl = $pJrnl|0;
 $zMaster = $zMaster|0;
 $nMaster = $nMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aMagic = 0, $cksum = 0, $len = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0;
 var $szJ = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $rc = sp + 20|0;
 $len = sp + 16|0;
 $szJ = sp;
 $cksum = sp + 12|0;
 $u = sp + 8|0;
 $aMagic = sp + 40|0;
 HEAP32[$1>>2] = $pJrnl;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$3>>2] = $nMaster;
 $4 = HEAP32[$2>>2]|0;
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3OsFileSize($5,$szJ)|0);
 HEAP32[$rc>>2] = $6;
 $7 = (0)!=($6|0);
 $8 = $szJ;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)<(0);
 $15 = ($10>>>0)<(16);
 $16 = ($13|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 $or$cond = $7 | $18;
 if (!($or$cond)) {
  $19 = HEAP32[$1>>2]|0;
  $20 = $szJ;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (_i64Subtract(($22|0),($25|0),16,0)|0);
  $27 = tempRet0;
  $28 = (_read32bits($19,$26,$27,$len)|0);
  HEAP32[$rc>>2] = $28;
  $29 = (0)!=($28|0);
  if (!($29)) {
   $30 = HEAP32[$len>>2]|0;
   $31 = HEAP32[$3>>2]|0;
   $32 = ($30>>>0)>=($31>>>0);
   $33 = HEAP32[$len>>2]|0;
   $34 = ($33|0)==(0);
   $or$cond3 = $32 | $34;
   if (!($or$cond3)) {
    $35 = HEAP32[$1>>2]|0;
    $36 = $szJ;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = (($36) + 4)|0;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = (_i64Subtract(($38|0),($41|0),12,0)|0);
    $43 = tempRet0;
    $44 = (_read32bits($35,$42,$43,$cksum)|0);
    HEAP32[$rc>>2] = $44;
    $45 = (0)!=($44|0);
    if (!($45)) {
     $46 = HEAP32[$1>>2]|0;
     $47 = $szJ;
     $48 = $47;
     $49 = HEAP32[$48>>2]|0;
     $50 = (($47) + 4)|0;
     $51 = $50;
     $52 = HEAP32[$51>>2]|0;
     $53 = (_i64Subtract(($49|0),($52|0),8,0)|0);
     $54 = tempRet0;
     $55 = (_sqlite3OsRead($46,$aMagic,8,$53,$54)|0);
     HEAP32[$rc>>2] = $55;
     $56 = (0)!=($55|0);
     if (!($56)) {
      $57 = (_memcmp($aMagic,25901,8)|0);
      $58 = ($57|0)!=(0);
      if (!($58)) {
       $59 = HEAP32[$1>>2]|0;
       $60 = HEAP32[$2>>2]|0;
       $61 = HEAP32[$len>>2]|0;
       $62 = $szJ;
       $63 = $62;
       $64 = HEAP32[$63>>2]|0;
       $65 = (($62) + 4)|0;
       $66 = $65;
       $67 = HEAP32[$66>>2]|0;
       $68 = (_i64Subtract(($64|0),($67|0),16,0)|0);
       $69 = tempRet0;
       $70 = HEAP32[$len>>2]|0;
       $71 = (_i64Subtract(($68|0),($69|0),($70|0),0)|0);
       $72 = tempRet0;
       $73 = (_sqlite3OsRead($59,$60,$61,$71,$72)|0);
       HEAP32[$rc>>2] = $73;
       $74 = (0)!=($73|0);
       if (!($74)) {
        HEAP32[$u>>2] = 0;
        while(1) {
         $76 = HEAP32[$u>>2]|0;
         $77 = HEAP32[$len>>2]|0;
         $78 = ($76>>>0)<($77>>>0);
         if (!($78)) {
          break;
         }
         $79 = HEAP32[$u>>2]|0;
         $80 = HEAP32[$2>>2]|0;
         $81 = (($80) + ($79)|0);
         $82 = HEAP8[$81>>0]|0;
         $83 = $82 << 24 >> 24;
         $84 = HEAP32[$cksum>>2]|0;
         $85 = (($84) - ($83))|0;
         HEAP32[$cksum>>2] = $85;
         $86 = HEAP32[$u>>2]|0;
         $87 = (($86) + 1)|0;
         HEAP32[$u>>2] = $87;
        }
        $88 = HEAP32[$cksum>>2]|0;
        $89 = ($88|0)!=(0);
        if ($89) {
         HEAP32[$len>>2] = 0;
        }
        $90 = HEAP32[$len>>2]|0;
        $91 = HEAP32[$2>>2]|0;
        $92 = (($91) + ($90)|0);
        HEAP8[$92>>0] = 0;
        HEAP32[$0>>2] = 0;
        $93 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($93|0);
       }
      }
     }
    }
   }
  }
 }
 $75 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $75;
 $93 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3OsAccess($pVfs,$zPath,$flags,$pResOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pResOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = (FUNCTION_TABLE_iiiii[$6 & 255]($7,$8,$9,$10)|0);
 STACKTOP = sp;return ($11|0);
}
function _readJournalHdr($pPager,$isHot,$0,$1,$pNRec,$pDbSize) {
 $pPager = $pPager|0;
 $isHot = $isHot|0;
 $0 = $0|0;
 $1 = $1|0;
 $pNRec = $pNRec|0;
 $pDbSize = $pDbSize|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMagic = 0, $iHdrOff = 0;
 var $iPageSize = 0, $iSectorSize = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 8|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $rc = sp + 24|0;
 $aMagic = sp + 48|0;
 $iHdrOff = sp;
 $iPageSize = sp + 20|0;
 $iSectorSize = sp + 16|0;
 HEAP32[$3>>2] = $pPager;
 HEAP32[$4>>2] = $isHot;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $pNRec;
 HEAP32[$7>>2] = $pDbSize;
 $12 = HEAP32[$3>>2]|0;
 $13 = (_journalHdrOffset($12)|0);
 $14 = tempRet0;
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 80|0);
 $17 = $16;
 $18 = $17;
 HEAP32[$18>>2] = $13;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $14;
 $21 = HEAP32[$3>>2]|0;
 $22 = ((($21)) + 80|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = ((($29)) + 156|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_i64Add(($25|0),($28|0),($31|0),0)|0);
 $33 = tempRet0;
 $34 = $5;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = ($33|0)>($39|0);
 $41 = ($32>>>0)>($36>>>0);
 $42 = ($33|0)==($39|0);
 $43 = $42 & $41;
 $44 = $40 | $43;
 if ($44) {
  HEAP32[$2>>2] = 101;
  $219 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($219|0);
 }
 $45 = HEAP32[$3>>2]|0;
 $46 = ((($45)) + 80|0);
 $47 = $46;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($47) + 4)|0;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = $iHdrOff;
 $54 = $53;
 HEAP32[$54>>2] = $49;
 $55 = (($53) + 4)|0;
 $56 = $55;
 HEAP32[$56>>2] = $52;
 $57 = HEAP32[$4>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  label = 5;
 } else {
  $59 = $iHdrOff;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($59) + 4)|0;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = HEAP32[$3>>2]|0;
  $66 = ((($65)) + 88|0);
  $67 = $66;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($61|0)!=($69|0);
  $74 = ($64|0)!=($72|0);
  $75 = $73 | $74;
  if ($75) {
   label = 5;
  }
 }
 if ((label|0) == 5) {
  $76 = HEAP32[$3>>2]|0;
  $77 = ((($76)) + 68|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = $iHdrOff;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (($79) + 4)|0;
  $83 = $82;
  $84 = HEAP32[$83>>2]|0;
  $85 = (_sqlite3OsRead($78,$aMagic,8,$81,$84)|0);
  HEAP32[$rc>>2] = $85;
  $86 = HEAP32[$rc>>2]|0;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$rc>>2]|0;
   HEAP32[$2>>2] = $88;
   $219 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($219|0);
  }
  $89 = (_memcmp($aMagic,25901,8)|0);
  $90 = ($89|0)!=(0);
  if ($90) {
   HEAP32[$2>>2] = 101;
   $219 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($219|0);
  }
 }
 $91 = HEAP32[$3>>2]|0;
 $92 = ((($91)) + 68|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = $iHdrOff;
 $95 = $94;
 $96 = HEAP32[$95>>2]|0;
 $97 = (($94) + 4)|0;
 $98 = $97;
 $99 = HEAP32[$98>>2]|0;
 $100 = (_i64Add(($96|0),($99|0),8,0)|0);
 $101 = tempRet0;
 $102 = HEAP32[$6>>2]|0;
 $103 = (_read32bits($93,$100,$101,$102)|0);
 HEAP32[$rc>>2] = $103;
 $104 = (0)!=($103|0);
 if (!($104)) {
  $105 = HEAP32[$3>>2]|0;
  $106 = ((($105)) + 68|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = $iHdrOff;
  $109 = $108;
  $110 = HEAP32[$109>>2]|0;
  $111 = (($108) + 4)|0;
  $112 = $111;
  $113 = HEAP32[$112>>2]|0;
  $114 = (_i64Add(($110|0),($113|0),12,0)|0);
  $115 = tempRet0;
  $116 = HEAP32[$3>>2]|0;
  $117 = ((($116)) + 52|0);
  $118 = (_read32bits($107,$114,$115,$117)|0);
  HEAP32[$rc>>2] = $118;
  $119 = (0)!=($118|0);
  if (!($119)) {
   $120 = HEAP32[$3>>2]|0;
   $121 = ((($120)) + 68|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = $iHdrOff;
   $124 = $123;
   $125 = HEAP32[$124>>2]|0;
   $126 = (($123) + 4)|0;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = (_i64Add(($125|0),($128|0),16,0)|0);
   $130 = tempRet0;
   $131 = HEAP32[$7>>2]|0;
   $132 = (_read32bits($122,$129,$130,$131)|0);
   HEAP32[$rc>>2] = $132;
   $133 = (0)!=($132|0);
   if (!($133)) {
    $135 = HEAP32[$3>>2]|0;
    $136 = ((($135)) + 80|0);
    $137 = $136;
    $138 = $137;
    $139 = HEAP32[$138>>2]|0;
    $140 = (($137) + 4)|0;
    $141 = $140;
    $142 = HEAP32[$141>>2]|0;
    $143 = ($139|0)==(0);
    $144 = ($142|0)==(0);
    $145 = $143 & $144;
    do {
     if ($145) {
      $146 = HEAP32[$3>>2]|0;
      $147 = ((($146)) + 68|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = $iHdrOff;
      $150 = $149;
      $151 = HEAP32[$150>>2]|0;
      $152 = (($149) + 4)|0;
      $153 = $152;
      $154 = HEAP32[$153>>2]|0;
      $155 = (_i64Add(($151|0),($154|0),20,0)|0);
      $156 = tempRet0;
      $157 = (_read32bits($148,$155,$156,$iSectorSize)|0);
      HEAP32[$rc>>2] = $157;
      $158 = (0)!=($157|0);
      if (!($158)) {
       $159 = HEAP32[$3>>2]|0;
       $160 = ((($159)) + 68|0);
       $161 = HEAP32[$160>>2]|0;
       $162 = $iHdrOff;
       $163 = $162;
       $164 = HEAP32[$163>>2]|0;
       $165 = (($162) + 4)|0;
       $166 = $165;
       $167 = HEAP32[$166>>2]|0;
       $168 = (_i64Add(($164|0),($167|0),24,0)|0);
       $169 = tempRet0;
       $170 = (_read32bits($161,$168,$169,$iPageSize)|0);
       HEAP32[$rc>>2] = $170;
       $171 = (0)!=($170|0);
       if (!($171)) {
        $173 = HEAP32[$iPageSize>>2]|0;
        $174 = ($173|0)==(0);
        if ($174) {
         $175 = HEAP32[$3>>2]|0;
         $176 = ((($175)) + 160|0);
         $177 = HEAP32[$176>>2]|0;
         HEAP32[$iPageSize>>2] = $177;
        }
        $178 = HEAP32[$iPageSize>>2]|0;
        $179 = ($178>>>0)<(512);
        $180 = HEAP32[$iSectorSize>>2]|0;
        $181 = ($180>>>0)<(32);
        $or$cond = $179 | $181;
        $182 = HEAP32[$iPageSize>>2]|0;
        $183 = ($182>>>0)>(65536);
        $or$cond3 = $or$cond | $183;
        $184 = HEAP32[$iSectorSize>>2]|0;
        $185 = ($184>>>0)>(65536);
        $or$cond5 = $or$cond3 | $185;
        if (!($or$cond5)) {
         $186 = HEAP32[$iPageSize>>2]|0;
         $187 = (($186) - 1)|0;
         $188 = HEAP32[$iPageSize>>2]|0;
         $189 = $187 & $188;
         $190 = ($189|0)!=(0);
         if (!($190)) {
          $191 = HEAP32[$iSectorSize>>2]|0;
          $192 = (($191) - 1)|0;
          $193 = HEAP32[$iSectorSize>>2]|0;
          $194 = $192 & $193;
          $195 = ($194|0)!=(0);
          if (!($195)) {
           $196 = HEAP32[$3>>2]|0;
           $197 = (_sqlite3PagerSetPagesize($196,$iPageSize,-1)|0);
           HEAP32[$rc>>2] = $197;
           $198 = HEAP32[$iSectorSize>>2]|0;
           $199 = HEAP32[$3>>2]|0;
           $200 = ((($199)) + 156|0);
           HEAP32[$200>>2] = $198;
           break;
          }
         }
        }
        HEAP32[$2>>2] = 101;
        $219 = HEAP32[$2>>2]|0;
        STACKTOP = sp;return ($219|0);
       }
      }
      $172 = HEAP32[$rc>>2]|0;
      HEAP32[$2>>2] = $172;
      $219 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($219|0);
     }
    } while(0);
    $201 = HEAP32[$3>>2]|0;
    $202 = ((($201)) + 156|0);
    $203 = HEAP32[$202>>2]|0;
    $204 = HEAP32[$3>>2]|0;
    $205 = ((($204)) + 80|0);
    $206 = $205;
    $207 = $206;
    $208 = HEAP32[$207>>2]|0;
    $209 = (($206) + 4)|0;
    $210 = $209;
    $211 = HEAP32[$210>>2]|0;
    $212 = (_i64Add(($208|0),($211|0),($203|0),0)|0);
    $213 = tempRet0;
    $214 = $205;
    $215 = $214;
    HEAP32[$215>>2] = $212;
    $216 = (($214) + 4)|0;
    $217 = $216;
    HEAP32[$217>>2] = $213;
    $218 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $218;
    $219 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($219|0);
   }
  }
 }
 $134 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $134;
 $219 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($219|0);
}
function _pager_playback_one_page($pPager,$pOffset,$pDone,$isMainJrnl,$isSavepnt) {
 $pPager = $pPager|0;
 $pOffset = $pOffset|0;
 $pDone = $pDone|0;
 $isMainJrnl = $isMainJrnl|0;
 $isSavepnt = $isSavepnt|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0, $cksum = 0, $isSynced = 0, $jfd = 0, $ofst = 0, $or$cond = 0, $or$cond4 = 0, $pData = 0, $pPg = 0, $pgno = 0;
 var $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $5 = sp + 40|0;
 $rc = sp + 36|0;
 $pPg = sp + 32|0;
 $pgno = sp + 28|0;
 $cksum = sp + 24|0;
 $aData = sp + 20|0;
 $jfd = sp + 16|0;
 $isSynced = sp + 12|0;
 $ofst = sp;
 $pData = sp + 8|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pOffset;
 HEAP32[$3>>2] = $pDone;
 HEAP32[$4>>2] = $isMainJrnl;
 HEAP32[$5>>2] = $isSavepnt;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 208|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$aData>>2] = $8;
 $9 = HEAP32[$4>>2]|0;
 $10 = ($9|0)!=(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = ((($11)) + 68|0);
  $13 = HEAP32[$12>>2]|0;
  $16 = $13;
 } else {
  $14 = ((($11)) + 72|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $15;
 }
 HEAP32[$jfd>>2] = $16;
 $17 = HEAP32[$jfd>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (_read32bits($17,$21,$24,$pgno)|0);
 HEAP32[$rc>>2] = $25;
 $26 = HEAP32[$rc>>2]|0;
 $27 = ($26|0)!=(0);
 if ($27) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $287 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($287|0);
 }
 $29 = HEAP32[$jfd>>2]|0;
 $30 = HEAP32[$aData>>2]|0;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 160|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$2>>2]|0;
 $35 = $34;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (_i64Add(($37|0),($40|0),4,0)|0);
 $42 = tempRet0;
 $43 = (_sqlite3OsRead($29,$30,$33,$41,$42)|0);
 HEAP32[$rc>>2] = $43;
 $44 = HEAP32[$rc>>2]|0;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $46;
  $287 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($287|0);
 }
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 160|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 4)|0;
 $51 = HEAP32[$4>>2]|0;
 $52 = $51<<2;
 $53 = (($50) + ($52))|0;
 $54 = ($53|0)<(0);
 $55 = $54 << 31 >> 31;
 $56 = HEAP32[$2>>2]|0;
 $57 = $56;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = (($57) + 4)|0;
 $61 = $60;
 $62 = HEAP32[$61>>2]|0;
 $63 = (_i64Add(($59|0),($62|0),($53|0),($55|0))|0);
 $64 = tempRet0;
 $65 = $56;
 $66 = $65;
 HEAP32[$66>>2] = $63;
 $67 = (($65) + 4)|0;
 $68 = $67;
 HEAP32[$68>>2] = $64;
 $69 = HEAP32[$pgno>>2]|0;
 $70 = ($69|0)==(0);
 if (!($70)) {
  $71 = HEAP32[$pgno>>2]|0;
  $72 = HEAP32[1536>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 160|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($72|0) / ($75|0))&-1;
  $77 = (($76) + 1)|0;
  $78 = ($71|0)==($77|0);
  if (!($78)) {
   $79 = HEAP32[$pgno>>2]|0;
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 28|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = ($79>>>0)>($82>>>0);
   if (!($83)) {
    $84 = HEAP32[$3>>2]|0;
    $85 = HEAP32[$pgno>>2]|0;
    $86 = (_sqlite3BitvecTest($84,$85)|0);
    $87 = ($86|0)!=(0);
    if (!($87)) {
     $88 = HEAP32[$4>>2]|0;
     $89 = ($88|0)!=(0);
     if ($89) {
      $90 = HEAP32[$jfd>>2]|0;
      $91 = HEAP32[$2>>2]|0;
      $92 = $91;
      $93 = $92;
      $94 = HEAP32[$93>>2]|0;
      $95 = (($92) + 4)|0;
      $96 = $95;
      $97 = HEAP32[$96>>2]|0;
      $98 = (_i64Subtract(($94|0),($97|0),4,0)|0);
      $99 = tempRet0;
      $100 = (_read32bits($90,$98,$99,$cksum)|0);
      HEAP32[$rc>>2] = $100;
      $101 = HEAP32[$rc>>2]|0;
      $102 = ($101|0)!=(0);
      if ($102) {
       $103 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $103;
       $287 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($287|0);
      }
      $104 = HEAP32[$5>>2]|0;
      $105 = ($104|0)!=(0);
      if (!($105)) {
       $106 = HEAP32[$1>>2]|0;
       $107 = HEAP32[$aData>>2]|0;
       $108 = (_pager_cksum($106,$107)|0);
       $109 = HEAP32[$cksum>>2]|0;
       $110 = ($108|0)!=($109|0);
       if ($110) {
        HEAP32[$0>>2] = 101;
        $287 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($287|0);
       }
      }
     }
     $111 = HEAP32[$3>>2]|0;
     $112 = ($111|0)!=(0|0);
     if ($112) {
      $113 = HEAP32[$3>>2]|0;
      $114 = HEAP32[$pgno>>2]|0;
      $115 = (_sqlite3BitvecSet($113,$114)|0);
      HEAP32[$rc>>2] = $115;
      $116 = ($115|0)!=(0);
      if ($116) {
       $117 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $117;
       $287 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($287|0);
      }
     }
     $118 = HEAP32[$pgno>>2]|0;
     $119 = ($118|0)==(1);
     if ($119) {
      $120 = HEAP32[$1>>2]|0;
      $121 = ((($120)) + 150|0);
      $122 = HEAP16[$121>>1]|0;
      $123 = $122 << 16 >> 16;
      $124 = HEAP32[$aData>>2]|0;
      $125 = ((($124)) + 20|0);
      $126 = HEAP8[$125>>0]|0;
      $127 = $126&255;
      $128 = ($123|0)!=($127|0);
      if ($128) {
       $129 = HEAP32[$aData>>2]|0;
       $130 = ((($129)) + 20|0);
       $131 = HEAP8[$130>>0]|0;
       $132 = $131&255;
       $133 = HEAP32[$1>>2]|0;
       $134 = ((($133)) + 150|0);
       HEAP16[$134>>1] = $132;
      }
     }
     $135 = HEAP32[$1>>2]|0;
     $136 = (_sqlite3PagerUseWal($135)|0);
     $137 = ($136|0)!=(0);
     if ($137) {
      HEAP32[$pPg>>2] = 0;
     } else {
      $138 = HEAP32[$1>>2]|0;
      $139 = HEAP32[$pgno>>2]|0;
      $140 = (_sqlite3PagerLookup($138,$139)|0);
      HEAP32[$pPg>>2] = $140;
     }
     $141 = HEAP32[$4>>2]|0;
     $142 = ($141|0)!=(0);
     if ($142) {
      $143 = HEAP32[$1>>2]|0;
      $144 = ((($143)) + 7|0);
      $145 = HEAP8[$144>>0]|0;
      $146 = $145&255;
      $147 = ($146|0)!=(0);
      if ($147) {
       $169 = 1;
      } else {
       $148 = HEAP32[$2>>2]|0;
       $149 = $148;
       $150 = $149;
       $151 = HEAP32[$150>>2]|0;
       $152 = (($149) + 4)|0;
       $153 = $152;
       $154 = HEAP32[$153>>2]|0;
       $155 = HEAP32[$1>>2]|0;
       $156 = ((($155)) + 88|0);
       $157 = $156;
       $158 = $157;
       $159 = HEAP32[$158>>2]|0;
       $160 = (($157) + 4)|0;
       $161 = $160;
       $162 = HEAP32[$161>>2]|0;
       $163 = ($154|0)<($162|0);
       $164 = ($151>>>0)<=($159>>>0);
       $165 = ($154|0)==($162|0);
       $166 = $165 & $164;
       $167 = $163 | $166;
       $169 = $167;
      }
      $168 = $169&1;
      HEAP32[$isSynced>>2] = $168;
     } else {
      $170 = HEAP32[$pPg>>2]|0;
      $171 = ($170|0)==(0|0);
      if ($171) {
       $179 = 1;
      } else {
       $172 = HEAP32[$pPg>>2]|0;
       $173 = ((($172)) + 24|0);
       $174 = HEAP16[$173>>1]|0;
       $175 = $174&65535;
       $176 = $175 & 8;
       $177 = (0)==($176|0);
       $179 = $177;
      }
      $178 = $179&1;
      HEAP32[$isSynced>>2] = $178;
     }
     $180 = HEAP32[$1>>2]|0;
     $181 = ((($180)) + 64|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = HEAP32[$182>>2]|0;
     $184 = ($183|0)!=(0|0);
     do {
      if ($184) {
       $185 = HEAP32[$1>>2]|0;
       $186 = ((($185)) + 17|0);
       $187 = HEAP8[$186>>0]|0;
       $188 = $187&255;
       $189 = ($188|0)>=(4);
       if ($189) {
        $$old = HEAP32[$isSynced>>2]|0;
        $$old1 = ($$old|0)!=(0);
        if (!($$old1)) {
         label = 44;
         break;
        }
       } else {
        $190 = HEAP32[$1>>2]|0;
        $191 = ((($190)) + 17|0);
        $192 = HEAP8[$191>>0]|0;
        $193 = $192&255;
        $194 = ($193|0)==(0);
        $195 = HEAP32[$isSynced>>2]|0;
        $196 = ($195|0)!=(0);
        $or$cond = $194 & $196;
        if (!($or$cond)) {
         label = 44;
         break;
        }
       }
       $197 = HEAP32[$pgno>>2]|0;
       $198 = (($197) - 1)|0;
       $199 = HEAP32[$1>>2]|0;
       $200 = ((($199)) + 160|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)<(0);
       $203 = $202 << 31 >> 31;
       $204 = (___muldi3(($198|0),0,($201|0),($203|0))|0);
       $205 = tempRet0;
       $206 = $ofst;
       $207 = $206;
       HEAP32[$207>>2] = $204;
       $208 = (($206) + 4)|0;
       $209 = $208;
       HEAP32[$209>>2] = $205;
       $210 = HEAP32[$1>>2]|0;
       $211 = ((($210)) + 64|0);
       $212 = HEAP32[$211>>2]|0;
       $213 = HEAP32[$aData>>2]|0;
       $214 = HEAP32[$1>>2]|0;
       $215 = ((($214)) + 160|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = $ofst;
       $218 = $217;
       $219 = HEAP32[$218>>2]|0;
       $220 = (($217) + 4)|0;
       $221 = $220;
       $222 = HEAP32[$221>>2]|0;
       $223 = (_sqlite3OsWrite($212,$213,$216,$219,$222)|0);
       HEAP32[$rc>>2] = $223;
       $224 = HEAP32[$pgno>>2]|0;
       $225 = HEAP32[$1>>2]|0;
       $226 = ((($225)) + 36|0);
       $227 = HEAP32[$226>>2]|0;
       $228 = ($224>>>0)>($227>>>0);
       if ($228) {
        $229 = HEAP32[$pgno>>2]|0;
        $230 = HEAP32[$1>>2]|0;
        $231 = ((($230)) + 36|0);
        HEAP32[$231>>2] = $229;
       }
       $232 = HEAP32[$1>>2]|0;
       $233 = ((($232)) + 96|0);
       $234 = HEAP32[$233>>2]|0;
       $235 = ($234|0)!=(0|0);
       if ($235) {
        $236 = HEAP32[$1>>2]|0;
        $237 = ((($236)) + 96|0);
        $238 = HEAP32[$237>>2]|0;
        $239 = HEAP32[$pgno>>2]|0;
        $240 = HEAP32[$aData>>2]|0;
        _sqlite3BackupUpdate($238,$239,$240);
        $241 = HEAP32[$aData>>2]|0;
        HEAP32[$aData>>2] = $241;
       }
      } else {
       label = 44;
      }
     } while(0);
     do {
      if ((label|0) == 44) {
       $242 = HEAP32[$4>>2]|0;
       $243 = ($242|0)==(0);
       $244 = HEAP32[$pPg>>2]|0;
       $245 = ($244|0)==(0|0);
       $or$cond4 = $243 & $245;
       if ($or$cond4) {
        $246 = HEAP32[$1>>2]|0;
        $247 = ((($246)) + 21|0);
        $248 = HEAP8[$247>>0]|0;
        $249 = $248&255;
        $250 = $249 | 2;
        $251 = $250&255;
        HEAP8[$247>>0] = $251;
        $252 = HEAP32[$1>>2]|0;
        $253 = HEAP32[$pgno>>2]|0;
        $254 = (_sqlite3PagerGet($252,$253,$pPg,1)|0);
        HEAP32[$rc>>2] = $254;
        $255 = HEAP32[$1>>2]|0;
        $256 = ((($255)) + 21|0);
        $257 = HEAP8[$256>>0]|0;
        $258 = $257&255;
        $259 = $258 & -3;
        $260 = $259&255;
        HEAP8[$256>>0] = $260;
        $261 = HEAP32[$rc>>2]|0;
        $262 = ($261|0)!=(0);
        if (!($262)) {
         $264 = HEAP32[$pPg>>2]|0;
         _sqlite3PcacheMakeDirty($264);
         break;
        }
        $263 = HEAP32[$rc>>2]|0;
        HEAP32[$0>>2] = $263;
        $287 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($287|0);
       }
      }
     } while(0);
     $265 = HEAP32[$pPg>>2]|0;
     $266 = ($265|0)!=(0|0);
     if ($266) {
      $267 = HEAP32[$pPg>>2]|0;
      $268 = ((($267)) + 4|0);
      $269 = HEAP32[$268>>2]|0;
      HEAP32[$pData>>2] = $269;
      $270 = HEAP32[$pData>>2]|0;
      $271 = HEAP32[$aData>>2]|0;
      $272 = HEAP32[$1>>2]|0;
      $273 = ((($272)) + 160|0);
      $274 = HEAP32[$273>>2]|0;
      _memcpy(($270|0),($271|0),($274|0))|0;
      $275 = HEAP32[$1>>2]|0;
      $276 = ((($275)) + 204|0);
      $277 = HEAP32[$276>>2]|0;
      $278 = HEAP32[$pPg>>2]|0;
      FUNCTION_TABLE_vi[$277 & 255]($278);
      $279 = HEAP32[$pgno>>2]|0;
      $280 = ($279|0)==(1);
      if ($280) {
       $281 = HEAP32[$1>>2]|0;
       $282 = ((($281)) + 112|0);
       $283 = HEAP32[$pData>>2]|0;
       $284 = ((($283)) + 24|0);
       dest=$282; src=$284; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
      }
      $285 = HEAP32[$pPg>>2]|0;
      _sqlite3PcacheRelease($285);
     }
     $286 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $286;
     $287 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($287|0);
    }
   }
   HEAP32[$0>>2] = 0;
   $287 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($287|0);
  }
 }
 HEAP32[$0>>2] = 101;
 $287 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($287|0);
}
function _pager_delmaster($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $exists = 0, $flags = 0, $flags1 = 0, $nMasterJournal = 0, $nMasterPtr = 0, $pJournal = 0, $pMaster = 0, $pVfs = 0, $rc = 0, $zJournal = 0, $zMasterJournal = 0, $zMasterPtr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $pVfs = sp + 52|0;
 $rc = sp + 48|0;
 $pMaster = sp + 44|0;
 $pJournal = sp + 40|0;
 $zMasterJournal = sp + 36|0;
 $nMasterJournal = sp;
 $zJournal = sp + 32|0;
 $zMasterPtr = sp + 28|0;
 $nMasterPtr = sp + 24|0;
 $flags = sp + 20|0;
 $exists = sp + 16|0;
 $c = sp + 12|0;
 $flags1 = sp + 8|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $zMaster;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$zMasterJournal>>2] = 0;
 $4 = HEAP32[$pVfs>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6<<1;
 $8 = ($7|0)<(0);
 $9 = $8 << 31 >> 31;
 $10 = (_sqlite3MallocZero($7,$9)|0);
 HEAP32[$pMaster>>2] = $10;
 $11 = HEAP32[$pMaster>>2]|0;
 $12 = HEAP32[$pVfs>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($11) + ($14)|0);
 HEAP32[$pJournal>>2] = $15;
 $16 = HEAP32[$pMaster>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  HEAP32[$flags>>2] = 16385;
  $18 = HEAP32[$pVfs>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$pMaster>>2]|0;
  $21 = (_sqlite3OsOpen($18,$19,$20,16385,0)|0);
  HEAP32[$rc>>2] = $21;
 } else {
  HEAP32[$rc>>2] = 7;
 }
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)!=(0);
 L5: do {
  if (!($23)) {
   $24 = HEAP32[$pMaster>>2]|0;
   $25 = (_sqlite3OsFileSize($24,$nMasterJournal)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)!=(0);
   if (!($27)) {
    $28 = HEAP32[$pVfs>>2]|0;
    $29 = ((($28)) + 8|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$nMasterPtr>>2] = $31;
    $32 = $nMasterJournal;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$nMasterPtr>>2]|0;
    $39 = ($38|0)<(0);
    $40 = $39 << 31 >> 31;
    $41 = (_i64Add(($34|0),($37|0),($38|0),($40|0))|0);
    $42 = tempRet0;
    $43 = (_i64Add(($41|0),($42|0),1,0)|0);
    $44 = tempRet0;
    $45 = (_sqlite3Malloc($43,$44)|0);
    HEAP32[$zMasterJournal>>2] = $45;
    $46 = HEAP32[$zMasterJournal>>2]|0;
    $47 = ($46|0)!=(0|0);
    if (!($47)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    $48 = $nMasterJournal;
    $49 = $48;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($48) + 4)|0;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (_i64Add(($50|0),($53|0),1,0)|0);
    $55 = tempRet0;
    $56 = HEAP32[$zMasterJournal>>2]|0;
    $57 = (($56) + ($54)|0);
    HEAP32[$zMasterPtr>>2] = $57;
    $58 = HEAP32[$pMaster>>2]|0;
    $59 = HEAP32[$zMasterJournal>>2]|0;
    $60 = $nMasterJournal;
    $61 = $60;
    $62 = HEAP32[$61>>2]|0;
    $63 = (($60) + 4)|0;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (_sqlite3OsRead($58,$59,$62,0,0)|0);
    HEAP32[$rc>>2] = $66;
    $67 = HEAP32[$rc>>2]|0;
    $68 = ($67|0)!=(0);
    if (!($68)) {
     $69 = $nMasterJournal;
     $70 = $69;
     $71 = HEAP32[$70>>2]|0;
     $72 = (($69) + 4)|0;
     $73 = $72;
     $74 = HEAP32[$73>>2]|0;
     $75 = HEAP32[$zMasterJournal>>2]|0;
     $76 = (($75) + ($71)|0);
     HEAP8[$76>>0] = 0;
     $77 = HEAP32[$zMasterJournal>>2]|0;
     HEAP32[$zJournal>>2] = $77;
     while(1) {
      $78 = HEAP32[$zJournal>>2]|0;
      $79 = HEAP32[$zMasterJournal>>2]|0;
      $80 = $78;
      $81 = $79;
      $82 = (($80) - ($81))|0;
      $83 = ($82|0)<(0);
      $84 = $83 << 31 >> 31;
      $85 = $nMasterJournal;
      $86 = $85;
      $87 = HEAP32[$86>>2]|0;
      $88 = (($85) + 4)|0;
      $89 = $88;
      $90 = HEAP32[$89>>2]|0;
      $91 = ($84|0)<($90|0);
      $92 = ($82>>>0)<($87>>>0);
      $93 = ($84|0)==($90|0);
      $94 = $93 & $92;
      $95 = $91 | $94;
      if (!($95)) {
       break;
      }
      $96 = HEAP32[$pVfs>>2]|0;
      $97 = HEAP32[$zJournal>>2]|0;
      $98 = (_sqlite3OsAccess($96,$97,0,$exists)|0);
      HEAP32[$rc>>2] = $98;
      $99 = HEAP32[$rc>>2]|0;
      $100 = ($99|0)!=(0);
      if ($100) {
       break L5;
      }
      $101 = HEAP32[$exists>>2]|0;
      $102 = ($101|0)!=(0);
      if ($102) {
       HEAP32[$flags1>>2] = 2049;
       $103 = HEAP32[$pVfs>>2]|0;
       $104 = HEAP32[$zJournal>>2]|0;
       $105 = HEAP32[$pJournal>>2]|0;
       $106 = HEAP32[$flags1>>2]|0;
       $107 = (_sqlite3OsOpen($103,$104,$105,$106,0)|0);
       HEAP32[$rc>>2] = $107;
       $108 = HEAP32[$rc>>2]|0;
       $109 = ($108|0)!=(0);
       if ($109) {
        break L5;
       }
       $110 = HEAP32[$pJournal>>2]|0;
       $111 = HEAP32[$zMasterPtr>>2]|0;
       $112 = HEAP32[$nMasterPtr>>2]|0;
       $113 = (_readMasterJournal($110,$111,$112)|0);
       HEAP32[$rc>>2] = $113;
       $114 = HEAP32[$pJournal>>2]|0;
       _sqlite3OsClose($114);
       $115 = HEAP32[$rc>>2]|0;
       $116 = ($115|0)!=(0);
       if ($116) {
        break L5;
       }
       $117 = HEAP32[$zMasterPtr>>2]|0;
       $118 = HEAP8[$117>>0]|0;
       $119 = $118 << 24 >> 24;
       $120 = ($119|0)!=(0);
       if ($120) {
        $121 = HEAP32[$zMasterPtr>>2]|0;
        $122 = HEAP32[$1>>2]|0;
        $123 = (_strcmp($121,$122)|0);
        $124 = ($123|0)==(0);
        $126 = $124;
       } else {
        $126 = 0;
       }
       $125 = $126&1;
       HEAP32[$c>>2] = $125;
       $127 = HEAP32[$c>>2]|0;
       $128 = ($127|0)!=(0);
       if ($128) {
        break L5;
       }
      }
      $129 = HEAP32[$zJournal>>2]|0;
      $130 = (_sqlite3Strlen30($129)|0);
      $131 = (($130) + 1)|0;
      $132 = HEAP32[$zJournal>>2]|0;
      $133 = (($132) + ($131)|0);
      HEAP32[$zJournal>>2] = $133;
     }
     $134 = HEAP32[$pMaster>>2]|0;
     _sqlite3OsClose($134);
     $135 = HEAP32[$pVfs>>2]|0;
     $136 = HEAP32[$1>>2]|0;
     $137 = (_sqlite3OsDelete($135,$136,0)|0);
     HEAP32[$rc>>2] = $137;
    }
   }
  }
 } while(0);
 $138 = HEAP32[$zMasterJournal>>2]|0;
 _sqlite3_free($138);
 $139 = HEAP32[$pMaster>>2]|0;
 $140 = ($139|0)!=(0|0);
 if (!($140)) {
  $143 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($143|0);
 }
 $141 = HEAP32[$pMaster>>2]|0;
 _sqlite3OsClose($141);
 $142 = HEAP32[$pMaster>>2]|0;
 _sqlite3_free($142);
 $143 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($143|0);
}
function _setSectorSize($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 13|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 64|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3OsDeviceCharacteristics($8)|0);
  $10 = $9 & 4096;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 64|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (_sqlite3SectorSize($16)|0);
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 156|0);
   HEAP32[$19>>2] = $17;
   STACKTOP = sp;return;
  }
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 156|0);
 HEAP32[$13>>2] = 512;
 STACKTOP = sp;return;
}
function _sqlite3OsDeviceCharacteristics($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 255]($5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3SectorSize($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $iRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iRet = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3OsSectorSize($1)|0);
 HEAP32[$iRet>>2] = $2;
 $3 = HEAP32[$iRet>>2]|0;
 $4 = ($3|0)<(32);
 if ($4) {
  HEAP32[$iRet>>2] = 512;
 } else {
  $5 = HEAP32[$iRet>>2]|0;
  $6 = ($5|0)>(65536);
  if ($6) {
   HEAP32[$iRet>>2] = 65536;
  }
 }
 $7 = HEAP32[$iRet>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3OsSectorSize($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $xSectorSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $xSectorSize = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$xSectorSize>>2] = $4;
 $5 = HEAP32[$xSectorSize>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = 4096;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$xSectorSize>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (FUNCTION_TABLE_ii[$7 & 255]($8)|0);
 $10 = $9;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3OsRead($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $id;
 HEAP32[$3>>2] = $pBuf;
 HEAP32[$4>>2] = $amt;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = $5;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (FUNCTION_TABLE_iiiiii[$13 & 255]($14,$15,$16,$19,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _read32bits($fd,$0,$1,$pRes) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $pRes = $pRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $ac = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $ac = sp + 20|0;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $pRes;
 $9 = HEAP32[$2>>2]|0;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3OsRead($9,$ac,4,$12,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)==(0);
 if (!($18)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $19 = (_sqlite3Get4byte($ac)|0);
 $20 = HEAP32[$4>>2]|0;
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _pager_cksum($pPager,$aData) {
 $pPager = $pPager|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $cksum = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $cksum = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $aData;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 52|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$cksum>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 160|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 200)|0;
 HEAP32[$i>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)>(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = HEAP32[$cksum>>2]|0;
  $17 = (($16) + ($15))|0;
  HEAP32[$cksum>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) - 200)|0;
  HEAP32[$i>>2] = $19;
 }
 $20 = HEAP32[$cksum>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3PagerLookup($pPager,$pgno) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 212|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3PcacheFetch($5,$6,0)|0);
 HEAP32[$pPage>>2] = $7;
 $8 = HEAP32[$pPage>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 } else {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 212|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$pPage>>2]|0;
  $15 = (_sqlite3PcacheFetchFinish($12,$13,$14)|0);
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _sqlite3BackupUpdate($pBackup,$iPage,$aData) {
 $pBackup = $pBackup|0;
 $iPage = $iPage|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pBackup;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $aData;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 _backupUpdate($5,$6,$7);
 STACKTOP = sp;return;
}
function _sqlite3PcacheMakeDirty($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 17;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = $10 & -17;
 $12 = $11&65535;
 HEAP16[$8>>1] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 & 1;
 $18 = ($17|0)!=(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 24|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 $23 = $22 ^ 3;
 $24 = $23&65535;
 HEAP16[$20>>1] = $24;
 $25 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($25,2);
 STACKTOP = sp;return;
}
function _sqlite3PcacheRelease($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 26|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = (($9) + -1)<<16>>16;
 HEAP16[$8>>1] = $10;
 $11 = $10 << 16 >> 16;
 $12 = ($11|0)==(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 & 1;
 $18 = ($17|0)!=(0);
 $19 = HEAP32[$0>>2]|0;
 if ($18) {
  _pcacheUnpin($19);
  STACKTOP = sp;return;
 }
 $20 = ((($19)) + 36|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($23,3);
 STACKTOP = sp;return;
}
function _backupUpdate($p,$iPage,$aData) {
 $p = $p|0;
 $iPage = $iPage|0;
 $aData = $aData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $aData;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 28|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = (_isFatalError($5)|0);
  $7 = ($6|0)!=(0);
  if (!($7)) {
   $8 = HEAP32[$1>>2]|0;
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 16|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = ($8>>>0)<($11>>>0);
   if ($12) {
    $13 = HEAP32[$0>>2]|0;
    $14 = HEAP32[$1>>2]|0;
    $15 = HEAP32[$2>>2]|0;
    $16 = (_backupOnePage($13,$14,$15,1)|0);
    HEAP32[$rc>>2] = $16;
    $17 = HEAP32[$rc>>2]|0;
    $18 = ($17|0)!=(0);
    if ($18) {
     $19 = HEAP32[$rc>>2]|0;
     $20 = HEAP32[$0>>2]|0;
     $21 = ((($20)) + 28|0);
     HEAP32[$21>>2] = $19;
    }
   }
  }
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 44|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$0>>2] = $24;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheFetch($pCache,$pgno,$createFlag) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eCreate = 0, $pRes = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $eCreate = sp + 4|0;
 $pRes = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $createFlag;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 33|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $3 & $7;
 HEAP32[$eCreate>>2] = $8;
 $9 = HEAP32[(144)>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 44|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$eCreate>>2]|0;
 $15 = (FUNCTION_TABLE_iiii[$9 & 255]($12,$13,$14)|0);
 HEAP32[$pRes>>2] = $15;
 $16 = HEAP32[$pRes>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3PcacheFetchFinish($pCache,$pgno,$pPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $pPage;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pPgHdr>>2] = $6;
 $7 = HEAP32[$pPgHdr>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$1>>2]|0;
 if ($9) {
  $14 = ((($10)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$14>>2] = $16;
  $17 = HEAP32[$pPgHdr>>2]|0;
  $18 = ((($17)) + 26|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = (($19) + 1)<<16>>16;
  HEAP16[$18>>1] = $20;
  $21 = HEAP32[$pPgHdr>>2]|0;
  HEAP32[$0>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 } else {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = (_pcacheFetchFinishWithInit($10,$11,$12)|0);
  HEAP32[$0>>2] = $13;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _pcacheFetchFinishWithInit($pCache,$pgno,$pPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $pPage;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pPgHdr>>2] = $5;
 $6 = HEAP32[$pPgHdr>>2]|0;
 $7 = ((($6)) + 12|0);
 ;HEAP32[$7>>2]=0|0;HEAP32[$7+4>>2]=0|0;HEAP32[$7+8>>2]=0|0;HEAP32[$7+12>>2]=0|0;HEAP32[$7+16>>2]=0|0;HEAP32[$7+20>>2]=0|0;HEAP32[$7+24>>2]=0|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$pPgHdr>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pPgHdr>>2]|0;
 $13 = ((($12)) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$pPgHdr>>2]|0;
 $15 = ((($14)) + 40|0);
 $16 = HEAP32[$pPgHdr>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$pPgHdr>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 28|0);
 $23 = HEAP32[$22>>2]|0;
 _memset(($20|0),0,($23|0))|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = HEAP32[$pPgHdr>>2]|0;
 $26 = ((($25)) + 28|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$pPgHdr>>2]|0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$pPgHdr>>2]|0;
 $31 = ((($30)) + 24|0);
 HEAP16[$31>>1] = 1;
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 $34 = HEAP32[$2>>2]|0;
 $35 = (_sqlite3PcacheFetchFinish($32,$33,$34)|0);
 STACKTOP = sp;return ($35|0);
}
function _journalHdrOffset($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $offset = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $offset = sp + 8|0;
 $c = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = $offset;
 $2 = $1;
 HEAP32[$2>>2] = 0;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $c;
 $14 = $13;
 HEAP32[$14>>2] = $9;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = $12;
 $17 = $c;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($19|0)!=(0);
 $24 = ($22|0)!=(0);
 $25 = $23 | $24;
 if (!($25)) {
  $50 = $offset;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  tempRet0 = ($55);
  STACKTOP = sp;return ($52|0);
 }
 $26 = $c;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (_i64Subtract(($28|0),($31|0),1,0)|0);
 $33 = tempRet0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 156|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (___divdi3(($32|0),($33|0),($36|0),0)|0);
 $38 = tempRet0;
 $39 = (_i64Add(($37|0),($38|0),1,0)|0);
 $40 = tempRet0;
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 156|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (___muldi3(($39|0),($40|0),($43|0),0)|0);
 $45 = tempRet0;
 $46 = $offset;
 $47 = $46;
 HEAP32[$47>>2] = $44;
 $48 = (($46) + 4)|0;
 $49 = $48;
 HEAP32[$49>>2] = $45;
 $50 = $offset;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + 4)|0;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 tempRet0 = ($55);
 STACKTOP = sp;return ($52|0);
}
function _pagerPlaybackSavepoint($pPager,$pSavepoint) {
 $pPager = $pPager|0;
 $pSavepoint = $pSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dummy = 0, $iHdrOff = 0, $ii = 0, $ii1 = 0, $nJRec = 0;
 var $offset = 0, $pDone = 0, $rc = 0, $szJ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $szJ = sp + 16|0;
 $iHdrOff = sp + 8|0;
 $rc = sp + 44|0;
 $pDone = sp + 40|0;
 $ii = sp + 36|0;
 $nJRec = sp + 32|0;
 $dummy = sp + 28|0;
 $ii1 = sp + 24|0;
 $offset = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pSavepoint;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDone>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3BitvecCreate($7)|0);
  HEAP32[$pDone>>2] = $8;
  $9 = HEAP32[$pDone>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   HEAP32[$0>>2] = 7;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $21 = $15;
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  $21 = $18;
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 28|0);
 HEAP32[$20>>2] = $21;
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 13|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 19|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$2>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $29 = HEAP32[$1>>2]|0;
  $30 = (_sqlite3PagerUseWal($29)|0);
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$1>>2]|0;
   $33 = (_pagerRollbackWal($32)|0);
   HEAP32[$0>>2] = $33;
   $294 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($294|0);
  }
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($34)) + 80|0);
 $36 = $35;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = $szJ;
 $43 = $42;
 HEAP32[$43>>2] = $38;
 $44 = (($42) + 4)|0;
 $45 = $44;
 HEAP32[$45>>2] = $41;
 $46 = HEAP32[$2>>2]|0;
 $47 = ($46|0)!=(0|0);
 L15: do {
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = (_sqlite3PagerUseWal($48)|0);
   $50 = ($49|0)!=(0);
   if ($50) {
    label = 19;
   } else {
    $51 = HEAP32[$2>>2]|0;
    $52 = ((($51)) + 8|0);
    $53 = $52;
    $54 = $53;
    $55 = HEAP32[$54>>2]|0;
    $56 = (($53) + 4)|0;
    $57 = $56;
    $58 = HEAP32[$57>>2]|0;
    $59 = ($55|0)!=(0);
    $60 = ($58|0)!=(0);
    $61 = $59 | $60;
    if ($61) {
     $62 = HEAP32[$2>>2]|0;
     $63 = ((($62)) + 8|0);
     $64 = $63;
     $65 = $64;
     $66 = HEAP32[$65>>2]|0;
     $67 = (($64) + 4)|0;
     $68 = $67;
     $69 = HEAP32[$68>>2]|0;
     $78 = $66;$81 = $69;
    } else {
     $70 = $szJ;
     $71 = $70;
     $72 = HEAP32[$71>>2]|0;
     $73 = (($70) + 4)|0;
     $74 = $73;
     $75 = HEAP32[$74>>2]|0;
     $78 = $72;$81 = $75;
    }
    $76 = $iHdrOff;
    $77 = $76;
    HEAP32[$77>>2] = $78;
    $79 = (($76) + 4)|0;
    $80 = $79;
    HEAP32[$80>>2] = $81;
    $82 = HEAP32[$2>>2]|0;
    $83 = $82;
    $84 = $83;
    $85 = HEAP32[$84>>2]|0;
    $86 = (($83) + 4)|0;
    $87 = $86;
    $88 = HEAP32[$87>>2]|0;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 80|0);
    $91 = $90;
    $92 = $91;
    HEAP32[$92>>2] = $85;
    $93 = (($91) + 4)|0;
    $94 = $93;
    HEAP32[$94>>2] = $88;
    while(1) {
     $95 = HEAP32[$rc>>2]|0;
     $96 = ($95|0)==(0);
     if (!($96)) {
      break L15;
     }
     $97 = HEAP32[$1>>2]|0;
     $98 = ((($97)) + 80|0);
     $99 = $98;
     $100 = $99;
     $101 = HEAP32[$100>>2]|0;
     $102 = (($99) + 4)|0;
     $103 = $102;
     $104 = HEAP32[$103>>2]|0;
     $105 = $iHdrOff;
     $106 = $105;
     $107 = HEAP32[$106>>2]|0;
     $108 = (($105) + 4)|0;
     $109 = $108;
     $110 = HEAP32[$109>>2]|0;
     $111 = ($104|0)<($110|0);
     $112 = ($101>>>0)<($107>>>0);
     $113 = ($104|0)==($110|0);
     $114 = $113 & $112;
     $115 = $111 | $114;
     if (!($115)) {
      break L15;
     }
     $116 = HEAP32[$1>>2]|0;
     $117 = HEAP32[$1>>2]|0;
     $118 = ((($117)) + 80|0);
     $119 = HEAP32[$pDone>>2]|0;
     $120 = (_pager_playback_one_page($116,$118,$119,1,1)|0);
     HEAP32[$rc>>2] = $120;
    }
   }
  } else {
   label = 19;
  }
 } while(0);
 if ((label|0) == 19) {
  $121 = HEAP32[$1>>2]|0;
  $122 = ((($121)) + 80|0);
  $123 = $122;
  $124 = $123;
  HEAP32[$124>>2] = 0;
  $125 = (($123) + 4)|0;
  $126 = $125;
  HEAP32[$126>>2] = 0;
 }
 L28: while(1) {
  $127 = HEAP32[$rc>>2]|0;
  $128 = ($127|0)==(0);
  if (!($128)) {
   break;
  }
  $129 = HEAP32[$1>>2]|0;
  $130 = ((($129)) + 80|0);
  $131 = $130;
  $132 = $131;
  $133 = HEAP32[$132>>2]|0;
  $134 = (($131) + 4)|0;
  $135 = $134;
  $136 = HEAP32[$135>>2]|0;
  $137 = $szJ;
  $138 = $137;
  $139 = HEAP32[$138>>2]|0;
  $140 = (($137) + 4)|0;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = ($136|0)<($142|0);
  $144 = ($133>>>0)<($139>>>0);
  $145 = ($136|0)==($142|0);
  $146 = $145 & $144;
  $147 = $143 | $146;
  if (!($147)) {
   break;
  }
  HEAP32[$nJRec>>2] = 0;
  $148 = HEAP32[$1>>2]|0;
  $149 = $szJ;
  $150 = $149;
  $151 = HEAP32[$150>>2]|0;
  $152 = (($149) + 4)|0;
  $153 = $152;
  $154 = HEAP32[$153>>2]|0;
  $155 = (_readJournalHdr($148,0,$151,$154,$nJRec,$dummy)|0);
  HEAP32[$rc>>2] = $155;
  $156 = HEAP32[$nJRec>>2]|0;
  $157 = ($156|0)==(0);
  if ($157) {
   $158 = HEAP32[$1>>2]|0;
   $159 = ((($158)) + 88|0);
   $160 = $159;
   $161 = $160;
   $162 = HEAP32[$161>>2]|0;
   $163 = (($160) + 4)|0;
   $164 = $163;
   $165 = HEAP32[$164>>2]|0;
   $166 = HEAP32[$1>>2]|0;
   $167 = ((($166)) + 156|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (_i64Add(($162|0),($165|0),($168|0),0)|0);
   $170 = tempRet0;
   $171 = HEAP32[$1>>2]|0;
   $172 = ((($171)) + 80|0);
   $173 = $172;
   $174 = $173;
   $175 = HEAP32[$174>>2]|0;
   $176 = (($173) + 4)|0;
   $177 = $176;
   $178 = HEAP32[$177>>2]|0;
   $179 = ($169|0)==($175|0);
   $180 = ($170|0)==($178|0);
   $181 = $179 & $180;
   if ($181) {
    $182 = $szJ;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = (($182) + 4)|0;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = HEAP32[$1>>2]|0;
    $189 = ((($188)) + 80|0);
    $190 = $189;
    $191 = $190;
    $192 = HEAP32[$191>>2]|0;
    $193 = (($190) + 4)|0;
    $194 = $193;
    $195 = HEAP32[$194>>2]|0;
    $196 = (_i64Subtract(($184|0),($187|0),($192|0),($195|0))|0);
    $197 = tempRet0;
    $198 = HEAP32[$1>>2]|0;
    $199 = ((($198)) + 160|0);
    $200 = HEAP32[$199>>2]|0;
    $201 = (($200) + 8)|0;
    $202 = ($201|0)<(0);
    $203 = $202 << 31 >> 31;
    $204 = (___divdi3(($196|0),($197|0),($201|0),($203|0))|0);
    $205 = tempRet0;
    HEAP32[$nJRec>>2] = $204;
   }
  }
  HEAP32[$ii>>2] = 0;
  while(1) {
   $206 = HEAP32[$rc>>2]|0;
   $207 = ($206|0)==(0);
   if (!($207)) {
    continue L28;
   }
   $208 = HEAP32[$ii>>2]|0;
   $209 = HEAP32[$nJRec>>2]|0;
   $210 = ($208>>>0)<($209>>>0);
   if (!($210)) {
    continue L28;
   }
   $211 = HEAP32[$1>>2]|0;
   $212 = ((($211)) + 80|0);
   $213 = $212;
   $214 = $213;
   $215 = HEAP32[$214>>2]|0;
   $216 = (($213) + 4)|0;
   $217 = $216;
   $218 = HEAP32[$217>>2]|0;
   $219 = $szJ;
   $220 = $219;
   $221 = HEAP32[$220>>2]|0;
   $222 = (($219) + 4)|0;
   $223 = $222;
   $224 = HEAP32[$223>>2]|0;
   $225 = ($218|0)<($224|0);
   $226 = ($215>>>0)<($221>>>0);
   $227 = ($218|0)==($224|0);
   $228 = $227 & $226;
   $229 = $225 | $228;
   if (!($229)) {
    continue L28;
   }
   $230 = HEAP32[$1>>2]|0;
   $231 = HEAP32[$1>>2]|0;
   $232 = ((($231)) + 80|0);
   $233 = HEAP32[$pDone>>2]|0;
   $234 = (_pager_playback_one_page($230,$232,$233,1,1)|0);
   HEAP32[$rc>>2] = $234;
   $235 = HEAP32[$ii>>2]|0;
   $236 = (($235) + 1)|0;
   HEAP32[$ii>>2] = $236;
  }
 }
 $237 = HEAP32[$2>>2]|0;
 $238 = ($237|0)!=(0|0);
 L42: do {
  if ($238) {
   $239 = HEAP32[$2>>2]|0;
   $240 = ((($239)) + 24|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[$1>>2]|0;
   $243 = ((($242)) + 160|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (4 + ($244))|0;
   $246 = ($245|0)<(0);
   $247 = $246 << 31 >> 31;
   $248 = (___muldi3(($241|0),0,($245|0),($247|0))|0);
   $249 = tempRet0;
   $250 = $offset;
   $251 = $250;
   HEAP32[$251>>2] = $248;
   $252 = (($250) + 4)|0;
   $253 = $252;
   HEAP32[$253>>2] = $249;
   $254 = HEAP32[$1>>2]|0;
   $255 = (_sqlite3PagerUseWal($254)|0);
   $256 = ($255|0)!=(0);
   if ($256) {
    $257 = HEAP32[$1>>2]|0;
    $258 = ((($257)) + 216|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = HEAP32[$2>>2]|0;
    $261 = ((($260)) + 28|0);
    $262 = (_sqlite3WalSavepointUndo($259,$261)|0);
    HEAP32[$rc>>2] = $262;
   }
   $263 = HEAP32[$2>>2]|0;
   $264 = ((($263)) + 24|0);
   $265 = HEAP32[$264>>2]|0;
   HEAP32[$ii1>>2] = $265;
   while(1) {
    $266 = HEAP32[$rc>>2]|0;
    $267 = ($266|0)==(0);
    if (!($267)) {
     break L42;
    }
    $268 = HEAP32[$ii1>>2]|0;
    $269 = HEAP32[$1>>2]|0;
    $270 = ((($269)) + 56|0);
    $271 = HEAP32[$270>>2]|0;
    $272 = ($268>>>0)<($271>>>0);
    if (!($272)) {
     break L42;
    }
    $273 = HEAP32[$1>>2]|0;
    $274 = HEAP32[$pDone>>2]|0;
    $275 = (_pager_playback_one_page($273,$offset,$274,0,1)|0);
    HEAP32[$rc>>2] = $275;
    $276 = HEAP32[$ii1>>2]|0;
    $277 = (($276) + 1)|0;
    HEAP32[$ii1>>2] = $277;
   }
  }
 } while(0);
 $278 = HEAP32[$pDone>>2]|0;
 _sqlite3BitvecDestroy($278);
 $279 = HEAP32[$rc>>2]|0;
 $280 = ($279|0)==(0);
 if ($280) {
  $281 = $szJ;
  $282 = $281;
  $283 = HEAP32[$282>>2]|0;
  $284 = (($281) + 4)|0;
  $285 = $284;
  $286 = HEAP32[$285>>2]|0;
  $287 = HEAP32[$1>>2]|0;
  $288 = ((($287)) + 80|0);
  $289 = $288;
  $290 = $289;
  HEAP32[$290>>2] = $283;
  $291 = (($289) + 4)|0;
  $292 = $291;
  HEAP32[$292>>2] = $286;
 }
 $293 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $293;
 $294 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($294|0);
}
function _pagerRollbackWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, $pNext = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $pList = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 28|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 216|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3WalUndo($8,187,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 212|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3PcacheDirtyList($13)|0);
 HEAP32[$pList>>2] = $14;
 while(1) {
  $15 = HEAP32[$pList>>2]|0;
  $16 = ($15|0)!=(0|0);
  $17 = HEAP32[$rc>>2]|0;
  $18 = ($17|0)==(0);
  $19 = $16 ? $18 : 0;
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$pList>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$pNext>>2] = $22;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$pList>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_pagerUndoCallback($23,$26)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$pNext>>2]|0;
  HEAP32[$pList>>2] = $28;
 }
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3WalSavepointUndo($pWal,$aWalData) {
 $pWal = $pWal|0;
 $aWalData = $aWalData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $aWalData;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 112|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($4|0)!=($7|0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  HEAP32[$9>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 112|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 12|0);
  HEAP32[$14>>2] = $12;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($16>>>0)<($20>>>0);
 if (!($21)) {
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 52|0);
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $23;
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 52|0);
 $32 = ((($31)) + 24|0);
 HEAP32[$32>>2] = $29;
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 52|0);
 $38 = ((($37)) + 24|0);
 $39 = ((($38)) + 4|0);
 HEAP32[$39>>2] = $35;
 $40 = HEAP32[$0>>2]|0;
 _walCleanupHash($40);
 $41 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _walCleanupHash($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $i = 0, $iLimit = 0, $iZero = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $aHash = sp + 20|0;
 $aPgno = sp + 16|0;
 $iZero = sp + 12|0;
 $iLimit = sp + 8|0;
 $nByte = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$aHash>>2] = 0;
 HEAP32[$aPgno>>2] = 0;
 HEAP32[$iZero>>2] = 0;
 HEAP32[$iLimit>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 52|0);
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_walFramePage($10)|0);
 (_walHashGet($6,$11,$aHash,$aPgno,$iZero)|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 52|0);
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$iZero>>2]|0;
 $17 = (($15) - ($16))|0;
 HEAP32[$iLimit>>2] = $17;
 HEAP32[$i>>2] = 0;
 while(1) {
  $18 = HEAP32[$i>>2]|0;
  $19 = ($18|0)<(8192);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$aHash>>2]|0;
  $22 = (($21) + ($20<<1)|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23&65535;
  $25 = HEAP32[$iLimit>>2]|0;
  $26 = ($24|0)>($25|0);
  if ($26) {
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$aHash>>2]|0;
   $29 = (($28) + ($27<<1)|0);
   HEAP16[$29>>1] = 0;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$aHash>>2]|0;
 $33 = HEAP32[$iLimit>>2]|0;
 $34 = (($33) + 1)|0;
 $35 = HEAP32[$aPgno>>2]|0;
 $36 = (($35) + ($34<<2)|0);
 $37 = $32;
 $38 = $36;
 $39 = (($37) - ($38))|0;
 HEAP32[$nByte>>2] = $39;
 $40 = HEAP32[$iLimit>>2]|0;
 $41 = (($40) + 1)|0;
 $42 = HEAP32[$aPgno>>2]|0;
 $43 = (($42) + ($41<<2)|0);
 $44 = HEAP32[$nByte>>2]|0;
 _memset(($43|0),0,($44|0))|0;
 STACKTOP = sp;return;
}
function _walFramePage($iFrame) {
 $iFrame = $iFrame|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $iHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iHash = sp;
 HEAP32[$0>>2] = $iFrame;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 4096)|0;
 $3 = (($2) - 4062)|0;
 $4 = (($3) - 1)|0;
 $5 = (($4>>>0) / 4096)&-1;
 HEAP32[$iHash>>2] = $5;
 $6 = HEAP32[$iHash>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _walHashGet($pWal,$iHash,$paHash,$paPgno,$piZero) {
 $pWal = $pWal|0;
 $iHash = $iHash|0;
 $paHash = $paHash|0;
 $paPgno = $paPgno|0;
 $piZero = $piZero|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $iZero = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $rc = sp + 12|0;
 $aPgno = sp + 8|0;
 $iZero = sp + 4|0;
 $aHash = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iHash;
 HEAP32[$2>>2] = $paHash;
 HEAP32[$3>>2] = $paPgno;
 HEAP32[$4>>2] = $piZero;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_walIndexPage($5,$6,$aPgno)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  $27 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $10 = HEAP32[$aPgno>>2]|0;
 $11 = ((($10)) + 16384|0);
 HEAP32[$aHash>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$aPgno>>2]|0;
  $15 = ((($14)) + 136|0);
  HEAP32[$aPgno>>2] = $15;
  HEAP32[$iZero>>2] = 0;
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = (($16) - 1)|0;
  $18 = $17<<12;
  $19 = (4062 + ($18))|0;
  HEAP32[$iZero>>2] = $19;
 }
 $20 = HEAP32[$aPgno>>2]|0;
 $21 = ((($20)) + -4|0);
 $22 = HEAP32[$3>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$aHash>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$iZero>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _walIndexPage($pWal,$iPage,$ppPage) {
 $pWal = $pWal|0;
 $iPage = $iPage|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $apNew = 0, $nByte = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $nByte = sp + 4|0;
 $apNew = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($6|0)<=($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$2>>2]|0;
   $10 = (($9) + 1)|0;
   $11 = $10<<2;
   HEAP32[$nByte>>2] = $11;
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 32|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$nByte>>2]|0;
   $16 = ($15|0)<(0);
   $17 = $16 << 31 >> 31;
   $18 = (_sqlite3_realloc64($14,$15,$17)|0);
   HEAP32[$apNew>>2] = $18;
   $19 = HEAP32[$apNew>>2]|0;
   $20 = ($19|0)!=(0|0);
   if ($20) {
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 24|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$apNew>>2]|0;
    $26 = (($25) + ($24<<2)|0);
    $27 = HEAP32[$2>>2]|0;
    $28 = (($27) + 1)|0;
    $29 = HEAP32[$1>>2]|0;
    $30 = ((($29)) + 24|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (($28) - ($31))|0;
    $33 = $32<<2;
    _memset(($26|0),0,($33|0))|0;
    $34 = HEAP32[$apNew>>2]|0;
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 32|0);
    HEAP32[$36>>2] = $34;
    $37 = HEAP32[$2>>2]|0;
    $38 = (($37) + 1)|0;
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 24|0);
    HEAP32[$40>>2] = $38;
    break;
   }
   $21 = HEAP32[$3>>2]|0;
   HEAP32[$21>>2] = 0;
   HEAP32[$0>>2] = 7;
   $96 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($96|0);
  }
 } while(0);
 $41 = HEAP32[$2>>2]|0;
 $42 = HEAP32[$1>>2]|0;
 $43 = ((($42)) + 32|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<2)|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)==(0|0);
 do {
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 43|0);
   $50 = HEAP8[$49>>0]|0;
   $51 = $50&255;
   $52 = ($51|0)==(2);
   if ($52) {
    $53 = (_sqlite3MallocZero(32768,0)|0);
    $54 = HEAP32[$2>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 32|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + ($54<<2)|0);
    HEAP32[$58>>2] = $53;
    $59 = HEAP32[$2>>2]|0;
    $60 = HEAP32[$1>>2]|0;
    $61 = ((($60)) + 32|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + ($59<<2)|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = ($64|0)!=(0|0);
    if ($65) {
     break;
    }
    HEAP32[$rc>>2] = 7;
    break;
   } else {
    $66 = HEAP32[$1>>2]|0;
    $67 = ((($66)) + 4|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP32[$2>>2]|0;
    $70 = HEAP32[$1>>2]|0;
    $71 = ((($70)) + 44|0);
    $72 = HEAP8[$71>>0]|0;
    $73 = $72&255;
    $74 = HEAP32[$2>>2]|0;
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 32|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = (($77) + ($74<<2)|0);
    $79 = (_sqlite3OsShmMap($68,$69,32768,$73,$78)|0);
    HEAP32[$rc>>2] = $79;
    $80 = HEAP32[$rc>>2]|0;
    $81 = ($80|0)==(8);
    if (!($81)) {
     break;
    }
    $82 = HEAP32[$1>>2]|0;
    $83 = ((($82)) + 46|0);
    $84 = HEAP8[$83>>0]|0;
    $85 = $84&255;
    $86 = $85 | 2;
    $87 = $86&255;
    HEAP8[$83>>0] = $87;
    HEAP32[$rc>>2] = 0;
    break;
   }
  }
 } while(0);
 $88 = HEAP32[$2>>2]|0;
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 32|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = (($91) + ($88<<2)|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = HEAP32[$3>>2]|0;
 HEAP32[$94>>2] = $93;
 $95 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $95;
 $96 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($96|0);
}
function _sqlite3OsShmMap($id,$iPage,$pgsz,$bExtend,$pp) {
 $id = $id|0;
 $iPage = $iPage|0;
 $pgsz = $pgsz|0;
 $bExtend = $bExtend|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $pgsz;
 HEAP32[$3>>2] = $bExtend;
 HEAP32[$4>>2] = $pp;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = (FUNCTION_TABLE_iiiiii[$8 & 255]($9,$10,$11,$12,$13)|0);
 STACKTOP = sp;return ($14|0);
}
function _pagerUndoCallback($pCtx,$iPg) {
 $pCtx = $pCtx|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iFrame = 0, $pPager = 0, $pPg = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $pPager = sp + 8|0;
 $pPg = sp + 4|0;
 $iFrame = sp;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $iPg;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pPager>>2] = $2;
 $3 = HEAP32[$pPager>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3PagerLookup($3,$4)|0);
 HEAP32[$pPg>>2] = $5;
 $6 = HEAP32[$pPg>>2]|0;
 $7 = ($6|0)!=(0|0);
 do {
  if ($7) {
   $8 = HEAP32[$pPg>>2]|0;
   $9 = (_sqlite3PcachePageRefcount($8)|0);
   $10 = ($9|0)==(1);
   if ($10) {
    $11 = HEAP32[$pPg>>2]|0;
    _sqlite3PcacheDrop($11);
    break;
   }
   HEAP32[$iFrame>>2] = 0;
   $12 = HEAP32[$pPager>>2]|0;
   $13 = ((($12)) + 216|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$pPg>>2]|0;
   $16 = ((($15)) + 20|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3WalFindFrame($14,$17,$iFrame)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)==(0);
   if ($20) {
    $21 = HEAP32[$pPg>>2]|0;
    $22 = HEAP32[$iFrame>>2]|0;
    $23 = (_readDbPage($21,$22)|0);
    HEAP32[$rc>>2] = $23;
   }
   $24 = HEAP32[$rc>>2]|0;
   $25 = ($24|0)==(0);
   if ($25) {
    $26 = HEAP32[$pPager>>2]|0;
    $27 = ((($26)) + 204|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$pPg>>2]|0;
    FUNCTION_TABLE_vi[$28 & 255]($29);
   }
   $30 = HEAP32[$pPg>>2]|0;
   _sqlite3PagerUnrefNotNull($30);
  }
 } while(0);
 $31 = HEAP32[$pPager>>2]|0;
 $32 = ((($31)) + 96|0);
 $33 = HEAP32[$32>>2]|0;
 _sqlite3BackupRestart($33);
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _sqlite3WalUndo($pWal,$xUndo,$pUndoCtx) {
 $pWal = $pWal|0;
 $xUndo = $xUndo|0;
 $pUndoCtx = $pUndoCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iFrame = 0, $iMax = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $iMax = sp + 4|0;
 $iFrame = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $xUndo;
 HEAP32[$2>>2] = $pUndoCtx;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $40 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$iMax>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 52|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = (_walIndexHdr($13)|0);
 dest=$12; src=$14; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 52|0);
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 1)|0;
 HEAP32[$iFrame>>2] = $19;
 while(1) {
  $20 = HEAP32[$rc>>2]|0;
  $21 = ($20|0)==(0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$iFrame>>2]|0;
  $23 = HEAP32[$iMax>>2]|0;
  $24 = ($22>>>0)<=($23>>>0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$iFrame>>2]|0;
  $29 = (_walFramePgno($27,$28)|0);
  $30 = (FUNCTION_TABLE_iii[$25 & 255]($26,$29)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$iFrame>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$iFrame>>2] = $32;
 }
 $33 = HEAP32[$iMax>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 52|0);
 $36 = ((($35)) + 16|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ($33|0)!=($37|0);
 if (!($38)) {
  $40 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 $39 = HEAP32[$0>>2]|0;
 _walCleanupHash($39);
 $40 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($40|0);
}
function _sqlite3PcacheDirtyList($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 32|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$p>>2]|0;
  $9 = ((($8)) + 12|0);
  HEAP32[$9>>2] = $7;
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$p>>2] = $12;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (_pcacheSortDirtyList($14)|0);
 STACKTOP = sp;return ($15|0);
}
function _pcacheSortDirtyList($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $i = 0, $p = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 136|0;
 $a = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pIn;
 dest=$a; stop=dest+128|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if (!($2)) {
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  HEAP32[$p>>2] = $3;
  $4 = HEAP32[$p>>2]|0;
  $5 = ((($4)) + 12|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$p>>2]|0;
  $8 = ((($7)) + 12|0);
  HEAP32[$8>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $9 = HEAP32[$i>>2]|0;
   $10 = ($9|0)<(31);
   if (!($10)) {
    break;
   }
   $11 = HEAP32[$i>>2]|0;
   $12 = (($a) + ($11<<2)|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)==(0|0);
   if ($14) {
    label = 6;
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = (($a) + ($18<<2)|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$p>>2]|0;
   $22 = (_pcacheMergeDirtyList($20,$21)|0);
   HEAP32[$p>>2] = $22;
   $23 = HEAP32[$i>>2]|0;
   $24 = (($a) + ($23<<2)|0);
   HEAP32[$24>>2] = 0;
   $25 = HEAP32[$i>>2]|0;
   $26 = (($25) + 1)|0;
   HEAP32[$i>>2] = $26;
  }
  if ((label|0) == 6) {
   label = 0;
   $15 = HEAP32[$p>>2]|0;
   $16 = HEAP32[$i>>2]|0;
   $17 = (($a) + ($16<<2)|0);
   HEAP32[$17>>2] = $15;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = ($27|0)==(31);
  if (!($28)) {
   continue;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($a) + ($29<<2)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$p>>2]|0;
  $33 = (_pcacheMergeDirtyList($31,$32)|0);
  $34 = HEAP32[$i>>2]|0;
  $35 = (($a) + ($34<<2)|0);
  HEAP32[$35>>2] = $33;
 }
 $36 = HEAP32[$a>>2]|0;
 HEAP32[$p>>2] = $36;
 HEAP32[$i>>2] = 1;
 while(1) {
  $37 = HEAP32[$i>>2]|0;
  $38 = ($37|0)<(32);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = (($a) + ($39<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ($41|0)==(0|0);
  if (!($42)) {
   $43 = HEAP32[$p>>2]|0;
   $44 = ($43|0)!=(0|0);
   if ($44) {
    $45 = HEAP32[$p>>2]|0;
    $46 = HEAP32[$i>>2]|0;
    $47 = (($a) + ($46<<2)|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (_pcacheMergeDirtyList($45,$48)|0);
    $53 = $49;
   } else {
    $50 = HEAP32[$i>>2]|0;
    $51 = (($a) + ($50<<2)|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = $52;
   }
   HEAP32[$p>>2] = $53;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 $56 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _pcacheMergeDirtyList($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTail = 0, $result = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $result = sp + 8|0;
 $pTail = sp;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 HEAP32[$pTail>>2] = $result;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4>>>0)<($7>>>0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$pTail>>2]|0;
   $11 = ((($10)) + 12|0);
   HEAP32[$11>>2] = $9;
   $12 = HEAP32[$0>>2]|0;
   HEAP32[$pTail>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   HEAP32[$0>>2] = $15;
   $16 = HEAP32[$0>>2]|0;
   $17 = ($16|0)==(0|0);
   if ($17) {
    label = 4;
    break;
   } else {
    continue;
   }
  } else {
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$pTail>>2]|0;
   $23 = ((($22)) + 12|0);
   HEAP32[$23>>2] = $21;
   $24 = HEAP32[$1>>2]|0;
   HEAP32[$pTail>>2] = $24;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 12|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$1>>2] = $27;
   $28 = HEAP32[$1>>2]|0;
   $29 = ($28|0)==(0|0);
   if ($29) {
    label = 6;
    break;
   } else {
    continue;
   }
  }
 }
 if ((label|0) == 4) {
  $18 = HEAP32[$1>>2]|0;
  $19 = HEAP32[$pTail>>2]|0;
  $20 = ((($19)) + 12|0);
  HEAP32[$20>>2] = $18;
  $33 = ((($result)) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 else if ((label|0) == 6) {
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$pTail>>2]|0;
  $32 = ((($31)) + 12|0);
  HEAP32[$32>>2] = $30;
  $33 = ((($result)) + 12|0);
  $34 = HEAP32[$33>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return (0)|0;
}
function _walIndexHdr($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _walFramePgno($pWal,$iFrame) {
 $pWal = $pWal|0;
 $iFrame = $iFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iHash = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $iHash = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iFrame;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_walFramePage($3)|0);
 HEAP32[$iHash>>2] = $4;
 $5 = HEAP32[$iHash>>2]|0;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$2>>2]|0;
 if ($6) {
  $8 = (34 + ($7))|0;
  $9 = (($8) - 1)|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($9<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$0>>2] = $15;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $16 = (($7) - 1)|0;
  $17 = (($16) - 4062)|0;
  $18 = (($17>>>0) % 4096)&-1;
  $19 = HEAP32[$iHash>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19<<2)|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($18<<2)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$0>>2] = $26;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3PcachePageRefcount($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 << 16 >> 16;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3PcacheDrop($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 2;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  _pcacheManageDirtyList($7,1);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + -1)|0;
 HEAP32[$11>>2] = $13;
 $14 = HEAP32[(148)>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 28|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 FUNCTION_TABLE_viii[$14 & 255]($19,$21,1);
 STACKTOP = sp;return;
}
function _sqlite3WalFindFrame($pWal,$pgno,$piRead) {
 $pWal = $pWal|0;
 $pgno = $pgno|0;
 $piRead = $piRead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0;
 var $iFrame = 0, $iHash = 0, $iKey = 0, $iLast = 0, $iMinHash = 0, $iRead = 0, $iZero = 0, $nCollide = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $iRead = sp + 40|0;
 $iLast = sp + 36|0;
 $iHash = sp + 32|0;
 $iMinHash = sp + 28|0;
 $aHash = sp + 24|0;
 $aPgno = sp + 20|0;
 $iZero = sp + 16|0;
 $iKey = sp + 12|0;
 $nCollide = sp + 8|0;
 $rc = sp + 4|0;
 $iFrame = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $piRead;
 HEAP32[$iRead>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$iLast>>2] = $7;
 $8 = HEAP32[$iLast>>2]|0;
 $9 = ($8|0)==(0);
 if (!($9)) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 40|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = ($13|0)==(0);
  if (!($14)) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 100|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_walFramePage($18)|0);
   HEAP32[$iMinHash>>2] = $19;
   $20 = HEAP32[$iLast>>2]|0;
   $21 = (_walFramePage($20)|0);
   HEAP32[$iHash>>2] = $21;
   L4: while(1) {
    $22 = HEAP32[$iHash>>2]|0;
    $23 = HEAP32[$iMinHash>>2]|0;
    $24 = ($22|0)>=($23|0);
    $25 = HEAP32[$iRead>>2]|0;
    $26 = ($25|0)==(0);
    $27 = $24 ? $26 : 0;
    if (!($27)) {
     label = 18;
     break;
    }
    $28 = HEAP32[$1>>2]|0;
    $29 = HEAP32[$iHash>>2]|0;
    $30 = (_walHashGet($28,$29,$aHash,$aPgno,$iZero)|0);
    HEAP32[$rc>>2] = $30;
    $31 = HEAP32[$rc>>2]|0;
    $32 = ($31|0)!=(0);
    if ($32) {
     label = 7;
     break;
    }
    HEAP32[$nCollide>>2] = 8192;
    $34 = HEAP32[$2>>2]|0;
    $35 = (_walHash($34)|0);
    HEAP32[$iKey>>2] = $35;
    while(1) {
     $36 = HEAP32[$iKey>>2]|0;
     $37 = HEAP32[$aHash>>2]|0;
     $38 = (($37) + ($36<<1)|0);
     $39 = HEAP16[$38>>1]|0;
     $40 = ($39<<16>>16)!=(0);
     if (!($40)) {
      break;
     }
     $41 = HEAP32[$iKey>>2]|0;
     $42 = HEAP32[$aHash>>2]|0;
     $43 = (($42) + ($41<<1)|0);
     $44 = HEAP16[$43>>1]|0;
     $45 = $44&65535;
     $46 = HEAP32[$iZero>>2]|0;
     $47 = (($45) + ($46))|0;
     HEAP32[$iFrame>>2] = $47;
     $48 = HEAP32[$iFrame>>2]|0;
     $49 = HEAP32[$iLast>>2]|0;
     $50 = ($48>>>0)<=($49>>>0);
     if ($50) {
      $51 = HEAP32[$iFrame>>2]|0;
      $52 = HEAP32[$1>>2]|0;
      $53 = ((($52)) + 100|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = ($51>>>0)>=($54>>>0);
      if ($55) {
       $56 = HEAP32[$iKey>>2]|0;
       $57 = HEAP32[$aHash>>2]|0;
       $58 = (($57) + ($56<<1)|0);
       $59 = HEAP16[$58>>1]|0;
       $60 = $59&65535;
       $61 = HEAP32[$aPgno>>2]|0;
       $62 = (($61) + ($60<<2)|0);
       $63 = HEAP32[$62>>2]|0;
       $64 = HEAP32[$2>>2]|0;
       $65 = ($63|0)==($64|0);
       if ($65) {
        $66 = HEAP32[$iFrame>>2]|0;
        HEAP32[$iRead>>2] = $66;
       }
      }
     }
     $67 = HEAP32[$nCollide>>2]|0;
     $68 = (($67) + -1)|0;
     HEAP32[$nCollide>>2] = $68;
     $69 = ($67|0)==(0);
     if ($69) {
      label = 15;
      break L4;
     }
     $71 = HEAP32[$iKey>>2]|0;
     $72 = (_walNextHash($71)|0);
     HEAP32[$iKey>>2] = $72;
    }
    $73 = HEAP32[$iHash>>2]|0;
    $74 = (($73) + -1)|0;
    HEAP32[$iHash>>2] = $74;
   }
   if ((label|0) == 7) {
    $33 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $33;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
   else if ((label|0) == 15) {
    $70 = (_sqlite3CorruptError(56344)|0);
    HEAP32[$0>>2] = $70;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
   else if ((label|0) == 18) {
    $75 = HEAP32[$iRead>>2]|0;
    $76 = HEAP32[$3>>2]|0;
    HEAP32[$76>>2] = $75;
    HEAP32[$0>>2] = 0;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
  }
 }
 $15 = HEAP32[$3>>2]|0;
 HEAP32[$15>>2] = 0;
 HEAP32[$0>>2] = 0;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _readDbPage($pPg,$iFrame) {
 $pPg = $pPg|0;
 $iFrame = $iFrame|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $dbFileVers = 0, $iOffset = 0, $pPager = 0, $pgno = 0, $pgsz = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pPager = sp + 24|0;
 $pgno = sp + 20|0;
 $rc = sp + 16|0;
 $pgsz = sp + 12|0;
 $iOffset = sp;
 $dbFileVers = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$1>>2] = $iFrame;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgno>>2] = $7;
 HEAP32[$rc>>2] = 0;
 $8 = HEAP32[$pPager>>2]|0;
 $9 = ((($8)) + 160|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pgsz>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$pPager>>2]|0;
  $14 = ((($13)) + 216|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pgsz>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3WalReadFrame($15,$16,$17,$20)|0);
  HEAP32[$rc>>2] = $21;
 } else {
  $22 = HEAP32[$pgno>>2]|0;
  $23 = (($22) - 1)|0;
  $24 = HEAP32[$pPager>>2]|0;
  $25 = ((($24)) + 160|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)<(0);
  $28 = $27 << 31 >> 31;
  $29 = (___muldi3(($23|0),0,($26|0),($28|0))|0);
  $30 = tempRet0;
  $31 = $iOffset;
  $32 = $31;
  HEAP32[$32>>2] = $29;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $30;
  $35 = HEAP32[$pPager>>2]|0;
  $36 = ((($35)) + 64|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = ((($38)) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pgsz>>2]|0;
  $42 = $iOffset;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (_sqlite3OsRead($37,$40,$41,$44,$47)|0);
  HEAP32[$rc>>2] = $48;
  $49 = HEAP32[$rc>>2]|0;
  $50 = ($49|0)==(522);
  $$ = $50 ? 0 : $48;
  HEAP32[$rc>>2] = $$;
 }
 $51 = HEAP32[$pgno>>2]|0;
 $52 = ($51|0)==(1);
 if (!($52)) {
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $53 = HEAP32[$rc>>2]|0;
 $54 = ($53|0)!=(0);
 if ($54) {
  $55 = HEAP32[$pPager>>2]|0;
  $56 = ((($55)) + 112|0);
  dest=$56; stop=dest+16|0; do { HEAP8[dest>>0]=255|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  $57 = HEAP32[$0>>2]|0;
  $58 = ((($57)) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = ((($59)) + 24|0);
  HEAP32[$dbFileVers>>2] = $60;
  $61 = HEAP32[$pPager>>2]|0;
  $62 = ((($61)) + 112|0);
  $63 = HEAP32[$dbFileVers>>2]|0;
  dest=$62; src=$63; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _sqlite3PagerUnrefNotNull($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 32;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  _pagerReleaseMapPage($10);
  $11 = HEAP32[$pPager>>2]|0;
  _pagerUnlockIfUnused($11);
  STACKTOP = sp;return;
 } else {
  _sqlite3PcacheRelease($10);
  $11 = HEAP32[$pPager>>2]|0;
  _pagerUnlockIfUnused($11);
  STACKTOP = sp;return;
 }
}
function _pagerReleaseMapPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 128|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + -1)|0;
 HEAP32[$5>>2] = $7;
 $8 = HEAP32[$pPager>>2]|0;
 $9 = ((($8)) + 144|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pPager>>2]|0;
 $15 = ((($14)) + 144|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$pPager>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = HEAP32[$pPager>>2]|0;
 $24 = ((($23)) + 160|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)<(0);
 $27 = $26 << 31 >> 31;
 $28 = (___muldi3(($22|0),0,($25|0),($27|0))|0);
 $29 = tempRet0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 (_sqlite3OsUnfetch($18,$28,$29,$32)|0);
 STACKTOP = sp;return;
}
function _pagerUnlockIfUnused($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 128|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 212|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PcacheRefCount($7)|0);
 $9 = ($8|0)==(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 _pagerUnlockAndRollback($10);
 STACKTOP = sp;return;
}
function _sqlite3OsUnfetch($id,$0,$1,$p) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $p = $p|0;
 var $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $p;
 STACKTOP = sp;return 0;
}
function _sqlite3WalReadFrame($pWal,$iRead,$nOut,$pOut) {
 $pWal = $pWal|0;
 $iRead = $iRead|0;
 $nOut = $nOut|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iOffset = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $sz = sp + 8|0;
 $iOffset = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iRead;
 HEAP32[$2>>2] = $nOut;
 HEAP32[$3>>2] = $pOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = ((($5)) + 14|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 HEAP32[$sz>>2] = $8;
 $9 = HEAP32[$sz>>2]|0;
 $10 = $9 & 65024;
 $11 = HEAP32[$sz>>2]|0;
 $12 = $11 & 1;
 $13 = $12 << 16;
 $14 = (($10) + ($13))|0;
 HEAP32[$sz>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = (($15) - 1)|0;
 $17 = HEAP32[$sz>>2]|0;
 $18 = (($17) + 24)|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = (___muldi3(($16|0),0,($18|0),($20|0))|0);
 $22 = tempRet0;
 $23 = (_i64Add(32,0,($21|0),($22|0))|0);
 $24 = tempRet0;
 $25 = (_i64Add(($23|0),($24|0),24,0)|0);
 $26 = tempRet0;
 $27 = $iOffset;
 $28 = $27;
 HEAP32[$28>>2] = $25;
 $29 = (($27) + 4)|0;
 $30 = $29;
 HEAP32[$30>>2] = $26;
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$sz>>2]|0;
 $37 = ($35|0)>($36|0);
 $38 = HEAP32[$sz>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = $37 ? $38 : $39;
 $41 = $iOffset;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_sqlite3OsRead($33,$34,$40,$43,$46)|0);
 STACKTOP = sp;return ($47|0);
}
function _walHash($iPage) {
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $iPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1*383)|0;
 $3 = $2 & 8191;
 STACKTOP = sp;return ($3|0);
}
function _walNextHash($iPriorHash) {
 $iPriorHash = $iPriorHash|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $iPriorHash;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 1)|0;
 $3 = $2 & 8191;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3WalEndReadTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 (_sqlite3WalEndWriteTransaction($1)|0);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)>=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (3 + ($11))|0;
 _walUnlockShared($7,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 40|0);
 HEAP16[$14>>1] = -1;
 STACKTOP = sp;return;
}
function _sqlite3OsLock($id,$lockType) {
 $id = $id|0;
 $lockType = $lockType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $lockType;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3WalCheckpoint($pWal,$eMode,$xBusy,$pBusyArg,$sync_flags,$nBuf,$zBuf,$pnLog,$pnCkpt) {
 $pWal = $pWal|0;
 $eMode = $eMode|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $sync_flags = $sync_flags|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $eMode2 = 0, $isChanged = 0, $or$cond = 0, $rc = 0, $xBusy2 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $9 = sp + 16|0;
 $rc = sp + 12|0;
 $isChanged = sp + 8|0;
 $eMode2 = sp + 4|0;
 $xBusy2 = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $xBusy;
 HEAP32[$4>>2] = $pBusyArg;
 HEAP32[$5>>2] = $sync_flags;
 HEAP32[$6>>2] = $nBuf;
 HEAP32[$7>>2] = $zBuf;
 HEAP32[$8>>2] = $pnLog;
 HEAP32[$9>>2] = $pnCkpt;
 HEAP32[$isChanged>>2] = 0;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$eMode2>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$xBusy2>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 46|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = ($14<<24>>24)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 8;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (_walLockExclusive($16,1,1)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $101 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($101|0);
 }
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 45|0);
 HEAP8[$22>>0] = 1;
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)!=(0);
 do {
  if ($24) {
   $25 = HEAP32[$1>>2]|0;
   $26 = HEAP32[$3>>2]|0;
   $27 = HEAP32[$4>>2]|0;
   $28 = (_walBusyLock($25,$26,$27,0,1)|0);
   HEAP32[$rc>>2] = $28;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(0);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 44|0);
    HEAP8[$32>>0] = 1;
    break;
   }
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(5);
   if ($34) {
    HEAP32[$eMode2>>2] = 0;
    HEAP32[$xBusy2>>2] = 0;
    HEAP32[$rc>>2] = 0;
   }
  }
 } while(0);
 $35 = HEAP32[$rc>>2]|0;
 $36 = ($35|0)==(0);
 if ($36) {
  $37 = HEAP32[$1>>2]|0;
  $38 = (_walIndexReadHdr($37,$isChanged)|0);
  HEAP32[$rc>>2] = $38;
  $39 = HEAP32[$isChanged>>2]|0;
  $40 = ($39|0)!=(0);
  if ($40) {
   $41 = HEAP32[$1>>2]|0;
   $42 = ((($41)) + 4|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)>=(3);
   if ($46) {
    $47 = HEAP32[$1>>2]|0;
    $48 = ((($47)) + 4|0);
    $49 = HEAP32[$48>>2]|0;
    (_sqlite3OsUnfetch($49,0,0,0)|0);
   }
  }
 }
 $50 = HEAP32[$rc>>2]|0;
 $51 = ($50|0)==(0);
 if ($51) {
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 52|0);
  $54 = ((($53)) + 16|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(0);
  if ($56) {
   $57 = HEAP32[$1>>2]|0;
   $58 = (_walPagesize($57)|0);
   $59 = HEAP32[$6>>2]|0;
   $60 = ($58|0)!=($59|0);
   if ($60) {
    $61 = (_sqlite3CorruptError(57049)|0);
    HEAP32[$rc>>2] = $61;
   } else {
    label = 18;
   }
  } else {
   label = 18;
  }
  if ((label|0) == 18) {
   $62 = HEAP32[$1>>2]|0;
   $63 = HEAP32[$eMode2>>2]|0;
   $64 = HEAP32[$xBusy2>>2]|0;
   $65 = HEAP32[$4>>2]|0;
   $66 = HEAP32[$5>>2]|0;
   $67 = HEAP32[$7>>2]|0;
   $68 = (_walCheckpoint($62,$63,$64,$65,$66,$67)|0);
   HEAP32[$rc>>2] = $68;
  }
  $69 = HEAP32[$rc>>2]|0;
  $70 = ($69|0)==(0);
  $71 = HEAP32[$rc>>2]|0;
  $72 = ($71|0)==(5);
  $or$cond = $70 | $72;
  if ($or$cond) {
   $73 = HEAP32[$8>>2]|0;
   $74 = ($73|0)!=(0|0);
   if ($74) {
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 52|0);
    $77 = ((($76)) + 16|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = HEAP32[$8>>2]|0;
    HEAP32[$79>>2] = $78;
   }
   $80 = HEAP32[$9>>2]|0;
   $81 = ($80|0)!=(0|0);
   if ($81) {
    $82 = HEAP32[$1>>2]|0;
    $83 = (_walCkptInfo($82)|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = HEAP32[$9>>2]|0;
    HEAP32[$85>>2] = $84;
   }
  }
 }
 $86 = HEAP32[$isChanged>>2]|0;
 $87 = ($86|0)!=(0);
 if ($87) {
  $88 = HEAP32[$1>>2]|0;
  $89 = ((($88)) + 52|0);
  dest=$89; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 }
 $90 = HEAP32[$1>>2]|0;
 (_sqlite3WalEndWriteTransaction($90)|0);
 $91 = HEAP32[$1>>2]|0;
 _walUnlockExclusive($91,1,1);
 $92 = HEAP32[$1>>2]|0;
 $93 = ((($92)) + 45|0);
 HEAP8[$93>>0] = 0;
 $94 = HEAP32[$rc>>2]|0;
 $95 = ($94|0)==(0);
 if ($95) {
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$eMode2>>2]|0;
  $98 = ($96|0)!=($97|0);
  if ($98) {
   $100 = 5;
  } else {
   label = 28;
  }
 } else {
  label = 28;
 }
 if ((label|0) == 28) {
  $99 = HEAP32[$rc>>2]|0;
  $100 = $99;
 }
 HEAP32[$0>>2] = $100;
 $101 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($101|0);
}
function _sqlite3OsFileControlHint($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (FUNCTION_TABLE_iiii[$6 & 255]($7,$8,$9)|0);
 STACKTOP = sp;return;
}
function _walLimitSize($pWal,$0,$1) {
 $pWal = $pWal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rx = 0, $sz = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp + 16|0;
 $2 = sp + 24|0;
 $3 = sp + 8|0;
 $sz = sp;
 $rx = sp + 20|0;
 HEAP32[$2>>2] = $pWal;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3OsFileSize($10,$sz)|0);
 HEAP32[$rx>>2] = $11;
 $12 = HEAP32[$rx>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = $sz;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = $3;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($19|0)>($25|0);
  $27 = ($16>>>0)>($22>>>0);
  $28 = ($19|0)==($25|0);
  $29 = $28 & $27;
  $30 = $26 | $29;
  if ($30) {
   $31 = HEAP32[$2>>2]|0;
   $32 = ((($31)) + 8|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $3;
   $35 = $34;
   $36 = HEAP32[$35>>2]|0;
   $37 = (($34) + 4)|0;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3OsTruncate($33,$36,$39)|0);
   HEAP32[$rx>>2] = $40;
  }
 }
 _sqlite3EndBenignMalloc();
 $41 = HEAP32[$rx>>2]|0;
 $42 = ($41|0)!=(0);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$rx>>2]|0;
 $44 = HEAP32[$2>>2]|0;
 $45 = ((($44)) + 108|0);
 $46 = HEAP32[$45>>2]|0;
 HEAP32[$vararg_buffer>>2] = $46;
 _sqlite3_log($43,25909,$vararg_buffer);
 STACKTOP = sp;return;
}
function _walIndexClose($pWal,$isDelete) {
 $pWal = $pWal|0;
 $isDelete = $isDelete|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $isDelete;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 43|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(2);
 if (!($6)) {
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  (_sqlite3OsShmUnmap($27,$28)|0);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 32|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_free($17);
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18<<2)|0);
  HEAP32[$22>>2] = 0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _sqlite3OsShmUnmap($id,$deleteFlag) {
 $id = $id|0;
 $deleteFlag = $deleteFlag|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $deleteFlag;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 64|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _walLockExclusive($pWal,$lockIdx,$n) {
 $pWal = $pWal|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $lockIdx;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 43|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = (_sqlite3OsShmLock($10,$11,$12,10)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _walBusyLock($pWal,$xBusy,$pBusyArg,$lockIdx,$n) {
 $pWal = $pWal|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $lockIdx = $lockIdx|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $xBusy;
 HEAP32[$2>>2] = $pBusyArg;
 HEAP32[$3>>2] = $lockIdx;
 HEAP32[$4>>2] = $n;
 while(1) {
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$3>>2]|0;
  $7 = HEAP32[$4>>2]|0;
  $8 = (_walLockExclusive($5,$6,$7)|0);
  HEAP32[$rc>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($9|0)!=(0|0);
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)==(5);
  $or$cond = $10 & $12;
  if (!($or$cond)) {
   label = 4;
   break;
  }
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (FUNCTION_TABLE_ii[$13 & 255]($14)|0);
  $16 = ($15|0)!=(0);
  if (!($16)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _walIndexReadHdr($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $badHdr = 0, $page0 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $badHdr = sp + 4|0;
 $page0 = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_walIndexPage($3,0,$page0)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $8 = HEAP32[$page0>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_walIndexTryHdr($10,$11)|0);
  $13 = $12;
 } else {
  $13 = 1;
 }
 HEAP32[$badHdr>>2] = $13;
 $14 = HEAP32[$badHdr>>2]|0;
 $15 = ($14|0)!=(0);
 do {
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 46|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $19 & 2;
   $21 = ($20|0)!=(0);
   $22 = HEAP32[$1>>2]|0;
   if ($21) {
    $23 = (_walLockShared($22,0)|0);
    HEAP32[$rc>>2] = $23;
    $24 = (0)==($23|0);
    if (!($24)) {
     break;
    }
    $25 = HEAP32[$1>>2]|0;
    _walUnlockShared($25,0);
    HEAP32[$rc>>2] = 264;
    break;
   }
   $26 = (_walLockExclusive($22,0,1)|0);
   HEAP32[$rc>>2] = $26;
   $27 = (0)==($26|0);
   if ($27) {
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 44|0);
    HEAP8[$29>>0] = 1;
    $30 = HEAP32[$1>>2]|0;
    $31 = (_walIndexPage($30,0,$page0)|0);
    HEAP32[$rc>>2] = $31;
    $32 = (0)==($31|0);
    if ($32) {
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     $35 = (_walIndexTryHdr($33,$34)|0);
     HEAP32[$badHdr>>2] = $35;
     $36 = HEAP32[$badHdr>>2]|0;
     $37 = ($36|0)!=(0);
     if ($37) {
      $38 = HEAP32[$1>>2]|0;
      $39 = (_walIndexRecover($38)|0);
      HEAP32[$rc>>2] = $39;
      $40 = HEAP32[$2>>2]|0;
      HEAP32[$40>>2] = 1;
     }
    }
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 44|0);
    HEAP8[$42>>0] = 0;
    $43 = HEAP32[$1>>2]|0;
    _walUnlockExclusive($43,0,1);
   }
  }
 } while(0);
 $44 = HEAP32[$badHdr>>2]|0;
 $45 = ($44|0)==(0);
 if ($45) {
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 52|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = ($48|0)!=(3007000);
  if ($49) {
   $50 = (_sqlite3CantopenError(55897)|0);
   HEAP32[$rc>>2] = $50;
  }
 }
 $51 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _walPagesize($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 52|0);
 $3 = ((($2)) + 14|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 65024;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 14|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 1;
 $13 = $12 << 16;
 $14 = (($6) + ($13))|0;
 STACKTOP = sp;return ($14|0);
}
function _walCheckpoint($pWal,$eMode,$xBusy,$pBusyArg,$sync_flags,$zBuf) {
 $pWal = $pWal|0;
 $eMode = $eMode|0;
 $xBusy = $xBusy|0;
 $pBusyArg = $pBusyArg|0;
 $sync_flags = $sync_flags|0;
 $zBuf = $zBuf|0;
 var $$old = 0, $$old4 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDbpage = 0, $iFrame = 0, $iOffset = 0, $mxPage = 0, $mxSafeFrame = 0, $nBackfill = 0;
 var $nReq = 0, $nSize = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pInfo = 0, $pIter = 0, $rc = 0, $salt1 = 0, $szDb = 0, $szPage = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp + 80|0;
 $rc = sp + 76|0;
 $szPage = sp + 72|0;
 $pIter = sp + 68|0;
 $iDbpage = sp + 64|0;
 $iFrame = sp + 60|0;
 $mxSafeFrame = sp + 56|0;
 $mxPage = sp + 52|0;
 $i = sp + 48|0;
 $pInfo = sp + 44|0;
 $y = sp + 40|0;
 $nSize = sp + 24|0;
 $nBackfill = sp + 36|0;
 $nReq = sp + 16|0;
 $iOffset = sp + 8|0;
 $szDb = sp;
 $salt1 = sp + 32|0;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $eMode;
 HEAP32[$3>>2] = $xBusy;
 HEAP32[$4>>2] = $pBusyArg;
 HEAP32[$5>>2] = $sync_flags;
 HEAP32[$6>>2] = $zBuf;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pIter>>2] = 0;
 HEAP32[$iDbpage>>2] = 0;
 HEAP32[$iFrame>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_walPagesize($7)|0);
 HEAP32[$szPage>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_walCkptInfo($9)|0);
 HEAP32[$pInfo>>2] = $10;
 $11 = HEAP32[$pInfo>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($12>>>0)<($16>>>0);
 L1: do {
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = (_walIteratorInit($18,$pIter)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)!=(0);
   if ($21) {
    $22 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $22;
    $270 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($270|0);
   }
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 52|0);
   $25 = ((($24)) + 16|0);
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$mxSafeFrame>>2] = $26;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 52|0);
   $29 = ((($28)) + 20|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$mxPage>>2] = $30;
   HEAP32[$i>>2] = 1;
   while(1) {
    $31 = HEAP32[$i>>2]|0;
    $32 = ($31|0)<(5);
    if (!($32)) {
     break;
    }
    $33 = HEAP32[$i>>2]|0;
    $34 = HEAP32[$pInfo>>2]|0;
    $35 = ((($34)) + 4|0);
    $36 = (($35) + ($33<<2)|0);
    $37 = HEAP32[$36>>2]|0;
    HEAP32[$y>>2] = $37;
    $38 = HEAP32[$mxSafeFrame>>2]|0;
    $39 = HEAP32[$y>>2]|0;
    $40 = ($38>>>0)>($39>>>0);
    do {
     if ($40) {
      $41 = HEAP32[$1>>2]|0;
      $42 = HEAP32[$3>>2]|0;
      $43 = HEAP32[$4>>2]|0;
      $44 = HEAP32[$i>>2]|0;
      $45 = (3 + ($44))|0;
      $46 = (_walBusyLock($41,$42,$43,$45,1)|0);
      HEAP32[$rc>>2] = $46;
      $47 = HEAP32[$rc>>2]|0;
      $48 = ($47|0)==(0);
      if ($48) {
       $49 = HEAP32[$i>>2]|0;
       $50 = ($49|0)==(1);
       $51 = HEAP32[$mxSafeFrame>>2]|0;
       $52 = $50 ? $51 : -1;
       $53 = HEAP32[$i>>2]|0;
       $54 = HEAP32[$pInfo>>2]|0;
       $55 = ((($54)) + 4|0);
       $56 = (($55) + ($53<<2)|0);
       HEAP32[$56>>2] = $52;
       $57 = HEAP32[$1>>2]|0;
       $58 = HEAP32[$i>>2]|0;
       $59 = (3 + ($58))|0;
       _walUnlockExclusive($57,$59,1);
       break;
      }
      $60 = HEAP32[$rc>>2]|0;
      $61 = ($60|0)==(5);
      if (!($61)) {
       break L1;
      }
      $62 = HEAP32[$y>>2]|0;
      HEAP32[$mxSafeFrame>>2] = $62;
      HEAP32[$3>>2] = 0;
     }
    } while(0);
    $63 = HEAP32[$i>>2]|0;
    $64 = (($63) + 1)|0;
    HEAP32[$i>>2] = $64;
   }
   $65 = HEAP32[$pInfo>>2]|0;
   $66 = HEAP32[$65>>2]|0;
   $67 = HEAP32[$mxSafeFrame>>2]|0;
   $68 = ($66>>>0)<($67>>>0);
   if ($68) {
    $69 = HEAP32[$1>>2]|0;
    $70 = HEAP32[$3>>2]|0;
    $71 = HEAP32[$4>>2]|0;
    $72 = (_walBusyLock($69,$70,$71,3,1)|0);
    HEAP32[$rc>>2] = $72;
    $73 = ($72|0)==(0);
    if ($73) {
     $74 = HEAP32[$pInfo>>2]|0;
     $75 = HEAP32[$74>>2]|0;
     HEAP32[$nBackfill>>2] = $75;
     $76 = HEAP32[$mxSafeFrame>>2]|0;
     $77 = HEAP32[$pInfo>>2]|0;
     $78 = ((($77)) + 32|0);
     HEAP32[$78>>2] = $76;
     $79 = HEAP32[$5>>2]|0;
     $80 = ($79|0)!=(0);
     if ($80) {
      $81 = HEAP32[$1>>2]|0;
      $82 = ((($81)) + 8|0);
      $83 = HEAP32[$82>>2]|0;
      $84 = HEAP32[$5>>2]|0;
      $85 = (_sqlite3OsSync($83,$84)|0);
      HEAP32[$rc>>2] = $85;
     }
     $86 = HEAP32[$rc>>2]|0;
     $87 = ($86|0)==(0);
     if ($87) {
      $88 = HEAP32[$mxPage>>2]|0;
      $89 = HEAP32[$szPage>>2]|0;
      $90 = ($89|0)<(0);
      $91 = $90 << 31 >> 31;
      $92 = (___muldi3(($88|0),0,($89|0),($91|0))|0);
      $93 = tempRet0;
      $94 = $nReq;
      $95 = $94;
      HEAP32[$95>>2] = $92;
      $96 = (($94) + 4)|0;
      $97 = $96;
      HEAP32[$97>>2] = $93;
      $98 = HEAP32[$1>>2]|0;
      $99 = ((($98)) + 4|0);
      $100 = HEAP32[$99>>2]|0;
      $101 = (_sqlite3OsFileSize($100,$nSize)|0);
      HEAP32[$rc>>2] = $101;
      $102 = HEAP32[$rc>>2]|0;
      $103 = ($102|0)==(0);
      if ($103) {
       $104 = $nSize;
       $105 = $104;
       $106 = HEAP32[$105>>2]|0;
       $107 = (($104) + 4)|0;
       $108 = $107;
       $109 = HEAP32[$108>>2]|0;
       $110 = $nReq;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = (($110) + 4)|0;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = ($109|0)<($115|0);
       $117 = ($106>>>0)<($112>>>0);
       $118 = ($109|0)==($115|0);
       $119 = $118 & $117;
       $120 = $116 | $119;
       if ($120) {
        $121 = HEAP32[$1>>2]|0;
        $122 = ((($121)) + 4|0);
        $123 = HEAP32[$122>>2]|0;
        _sqlite3OsFileControlHint($123,5,$nReq);
       }
      }
     }
     L28: while(1) {
      $$old = HEAP32[$rc>>2]|0;
      $$old4 = ($$old|0)==(0);
      if (!($$old4)) {
       break;
      }
      while(1) {
       $124 = HEAP32[$pIter>>2]|0;
       $125 = (_walIteratorNext($124,$iDbpage,$iFrame)|0);
       $126 = (0)==($125|0);
       if (!($126)) {
        break L28;
       }
       $127 = HEAP32[$iFrame>>2]|0;
       $128 = HEAP32[$nBackfill>>2]|0;
       $129 = ($127>>>0)<=($128>>>0);
       if ($129) {
        continue L28;
       }
       $130 = HEAP32[$iFrame>>2]|0;
       $131 = HEAP32[$mxSafeFrame>>2]|0;
       $132 = ($130>>>0)>($131>>>0);
       if ($132) {
        continue L28;
       }
       $133 = HEAP32[$iDbpage>>2]|0;
       $134 = HEAP32[$mxPage>>2]|0;
       $135 = ($133>>>0)>($134>>>0);
       if ($135) {
        continue L28;
       }
       $136 = HEAP32[$iFrame>>2]|0;
       $137 = (($136) - 1)|0;
       $138 = HEAP32[$szPage>>2]|0;
       $139 = (($138) + 24)|0;
       $140 = ($139|0)<(0);
       $141 = $140 << 31 >> 31;
       $142 = (___muldi3(($137|0),0,($139|0),($141|0))|0);
       $143 = tempRet0;
       $144 = (_i64Add(32,0,($142|0),($143|0))|0);
       $145 = tempRet0;
       $146 = (_i64Add(($144|0),($145|0),24,0)|0);
       $147 = tempRet0;
       $148 = $iOffset;
       $149 = $148;
       HEAP32[$149>>2] = $146;
       $150 = (($148) + 4)|0;
       $151 = $150;
       HEAP32[$151>>2] = $147;
       $152 = HEAP32[$1>>2]|0;
       $153 = ((($152)) + 8|0);
       $154 = HEAP32[$153>>2]|0;
       $155 = HEAP32[$6>>2]|0;
       $156 = HEAP32[$szPage>>2]|0;
       $157 = $iOffset;
       $158 = $157;
       $159 = HEAP32[$158>>2]|0;
       $160 = (($157) + 4)|0;
       $161 = $160;
       $162 = HEAP32[$161>>2]|0;
       $163 = (_sqlite3OsRead($154,$155,$156,$159,$162)|0);
       HEAP32[$rc>>2] = $163;
       $164 = HEAP32[$rc>>2]|0;
       $165 = ($164|0)!=(0);
       if ($165) {
        break L28;
       }
       $166 = HEAP32[$iDbpage>>2]|0;
       $167 = (($166) - 1)|0;
       $168 = HEAP32[$szPage>>2]|0;
       $169 = ($168|0)<(0);
       $170 = $169 << 31 >> 31;
       $171 = (___muldi3(($167|0),0,($168|0),($170|0))|0);
       $172 = tempRet0;
       $173 = $iOffset;
       $174 = $173;
       HEAP32[$174>>2] = $171;
       $175 = (($173) + 4)|0;
       $176 = $175;
       HEAP32[$176>>2] = $172;
       $177 = HEAP32[$1>>2]|0;
       $178 = ((($177)) + 4|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = HEAP32[$6>>2]|0;
       $181 = HEAP32[$szPage>>2]|0;
       $182 = $iOffset;
       $183 = $182;
       $184 = HEAP32[$183>>2]|0;
       $185 = (($182) + 4)|0;
       $186 = $185;
       $187 = HEAP32[$186>>2]|0;
       $188 = (_sqlite3OsWrite($179,$180,$181,$184,$187)|0);
       HEAP32[$rc>>2] = $188;
       $189 = HEAP32[$rc>>2]|0;
       $190 = ($189|0)==(0);
       $191 = HEAP32[$rc>>2]|0;
       $192 = ($191|0)==(0);
       $or$cond5 = $190 & $192;
       if (!($or$cond5)) {
        break L28;
       }
      }
     }
     $193 = HEAP32[$rc>>2]|0;
     $194 = ($193|0)==(0);
     if ($194) {
      $195 = HEAP32[$mxSafeFrame>>2]|0;
      $196 = HEAP32[$1>>2]|0;
      $197 = (_walIndexHdr($196)|0);
      $198 = ((($197)) + 16|0);
      $199 = HEAP32[$198>>2]|0;
      $200 = ($195|0)==($199|0);
      if ($200) {
       $201 = HEAP32[$1>>2]|0;
       $202 = ((($201)) + 52|0);
       $203 = ((($202)) + 20|0);
       $204 = HEAP32[$203>>2]|0;
       $205 = HEAP32[$szPage>>2]|0;
       $206 = ($205|0)<(0);
       $207 = $206 << 31 >> 31;
       $208 = (___muldi3(($204|0),0,($205|0),($207|0))|0);
       $209 = tempRet0;
       $210 = $szDb;
       $211 = $210;
       HEAP32[$211>>2] = $208;
       $212 = (($210) + 4)|0;
       $213 = $212;
       HEAP32[$213>>2] = $209;
       $214 = HEAP32[$1>>2]|0;
       $215 = ((($214)) + 4|0);
       $216 = HEAP32[$215>>2]|0;
       $217 = $szDb;
       $218 = $217;
       $219 = HEAP32[$218>>2]|0;
       $220 = (($217) + 4)|0;
       $221 = $220;
       $222 = HEAP32[$221>>2]|0;
       $223 = (_sqlite3OsTruncate($216,$219,$222)|0);
       HEAP32[$rc>>2] = $223;
       $224 = HEAP32[$rc>>2]|0;
       $225 = ($224|0)==(0);
       $226 = HEAP32[$5>>2]|0;
       $227 = ($226|0)!=(0);
       $or$cond = $225 & $227;
       if ($or$cond) {
        $228 = HEAP32[$1>>2]|0;
        $229 = ((($228)) + 4|0);
        $230 = HEAP32[$229>>2]|0;
        $231 = HEAP32[$5>>2]|0;
        $232 = (_sqlite3OsSync($230,$231)|0);
        HEAP32[$rc>>2] = $232;
       }
      }
      $233 = HEAP32[$rc>>2]|0;
      $234 = ($233|0)==(0);
      if ($234) {
       $235 = HEAP32[$mxSafeFrame>>2]|0;
       $236 = HEAP32[$pInfo>>2]|0;
       HEAP32[$236>>2] = $235;
      }
     }
     $237 = HEAP32[$1>>2]|0;
     _walUnlockExclusive($237,3,1);
    }
   }
   $238 = HEAP32[$rc>>2]|0;
   $239 = ($238|0)==(5);
   if ($239) {
    HEAP32[$rc>>2] = 0;
    label = 36;
   } else {
    label = 36;
   }
  } else {
   label = 36;
  }
 } while(0);
 do {
  if ((label|0) == 36) {
   $240 = HEAP32[$rc>>2]|0;
   $241 = ($240|0)==(0);
   $242 = HEAP32[$2>>2]|0;
   $243 = ($242|0)!=(0);
   $or$cond3 = $241 & $243;
   if ($or$cond3) {
    $244 = HEAP32[$pInfo>>2]|0;
    $245 = HEAP32[$244>>2]|0;
    $246 = HEAP32[$1>>2]|0;
    $247 = ((($246)) + 52|0);
    $248 = ((($247)) + 16|0);
    $249 = HEAP32[$248>>2]|0;
    $250 = ($245>>>0)<($249>>>0);
    if ($250) {
     HEAP32[$rc>>2] = 5;
     break;
    }
    $251 = HEAP32[$2>>2]|0;
    $252 = ($251|0)>=(2);
    if ($252) {
     _sqlite3_randomness(4,$salt1);
     $253 = HEAP32[$1>>2]|0;
     $254 = HEAP32[$3>>2]|0;
     $255 = HEAP32[$4>>2]|0;
     $256 = (_walBusyLock($253,$254,$255,4,4)|0);
     HEAP32[$rc>>2] = $256;
     $257 = HEAP32[$rc>>2]|0;
     $258 = ($257|0)==(0);
     if ($258) {
      $259 = HEAP32[$2>>2]|0;
      $260 = ($259|0)==(3);
      if ($260) {
       $261 = HEAP32[$1>>2]|0;
       $262 = HEAP32[$salt1>>2]|0;
       _walRestartHdr($261,$262);
       $263 = HEAP32[$1>>2]|0;
       $264 = ((($263)) + 8|0);
       $265 = HEAP32[$264>>2]|0;
       $266 = (_sqlite3OsTruncate($265,0,0)|0);
       HEAP32[$rc>>2] = $266;
      }
      $267 = HEAP32[$1>>2]|0;
      _walUnlockExclusive($267,4,4);
     }
    }
   }
  }
 } while(0);
 $268 = HEAP32[$pIter>>2]|0;
 _walIteratorFree($268);
 $269 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $269;
 $270 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($270|0);
}
function _walCkptInfo($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 96|0);
 STACKTOP = sp;return ($5|0);
}
function _walIteratorInit($pWal,$pp) {
 $pWal = $pWal|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aHash = 0, $aIndex = 0;
 var $aPgno = 0, $aTmp = 0, $i = 0, $iLast = 0, $iZero = 0, $j = 0, $nByte = 0, $nEntry = 0, $nSegment = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $p = sp + 48|0;
 $nSegment = sp + 44|0;
 $iLast = sp + 40|0;
 $nByte = sp + 36|0;
 $i = sp + 32|0;
 $aTmp = sp + 28|0;
 $rc = sp + 24|0;
 $aHash = sp + 20|0;
 $iZero = sp + 16|0;
 $aPgno = sp + 12|0;
 $j = sp + 8|0;
 $nEntry = sp + 4|0;
 $aIndex = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iLast>>2] = $6;
 $7 = HEAP32[$iLast>>2]|0;
 $8 = (_walFramePage($7)|0);
 $9 = (($8) + 1)|0;
 HEAP32[$nSegment>>2] = $9;
 $10 = HEAP32[$nSegment>>2]|0;
 $11 = (($10) - 1)|0;
 $12 = ($11*20)|0;
 $13 = (28 + ($12))|0;
 $14 = HEAP32[$iLast>>2]|0;
 $15 = $14<<1;
 $16 = (($13) + ($15))|0;
 HEAP32[$nByte>>2] = $16;
 $17 = HEAP32[$nByte>>2]|0;
 $18 = ($17|0)<(0);
 $19 = $18 << 31 >> 31;
 $20 = (_sqlite3_malloc64($17,$19)|0);
 HEAP32[$p>>2] = $20;
 $21 = HEAP32[$p>>2]|0;
 $22 = ($21|0)!=(0|0);
 if (!($22)) {
  HEAP32[$0>>2] = 7;
  $117 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($117|0);
 }
 $23 = HEAP32[$p>>2]|0;
 $24 = HEAP32[$nByte>>2]|0;
 _memset(($23|0),0,($24|0))|0;
 $25 = HEAP32[$nSegment>>2]|0;
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 4|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$iLast>>2]|0;
 $29 = ($28>>>0)>(4096);
 $30 = HEAP32[$iLast>>2]|0;
 $31 = $29 ? 4096 : $30;
 $32 = $31<<1;
 $33 = (_sqlite3_malloc64($32,0)|0);
 HEAP32[$aTmp>>2] = $33;
 $34 = HEAP32[$aTmp>>2]|0;
 $35 = ($34|0)!=(0|0);
 if (!($35)) {
  HEAP32[$rc>>2] = 7;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $36 = HEAP32[$rc>>2]|0;
  $37 = ($36|0)==(0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$nSegment>>2]|0;
  $40 = ($38|0)<($39|0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$i>>2]|0;
  $43 = (_walHashGet($41,$42,$aHash,$aPgno,$iZero)|0);
  HEAP32[$rc>>2] = $43;
  $44 = HEAP32[$rc>>2]|0;
  $45 = ($44|0)==(0);
  if ($45) {
   $46 = HEAP32[$aPgno>>2]|0;
   $47 = ((($46)) + 4|0);
   HEAP32[$aPgno>>2] = $47;
   $48 = HEAP32[$i>>2]|0;
   $49 = (($48) + 1)|0;
   $50 = HEAP32[$nSegment>>2]|0;
   $51 = ($49|0)==($50|0);
   if ($51) {
    $52 = HEAP32[$iLast>>2]|0;
    $53 = HEAP32[$iZero>>2]|0;
    $54 = (($52) - ($53))|0;
    HEAP32[$nEntry>>2] = $54;
   } else {
    $55 = HEAP32[$aHash>>2]|0;
    $56 = HEAP32[$aPgno>>2]|0;
    $57 = $55;
    $58 = $56;
    $59 = (($57) - ($58))|0;
    $60 = (($59|0) / 4)&-1;
    HEAP32[$nEntry>>2] = $60;
   }
   $61 = HEAP32[$iZero>>2]|0;
   $62 = HEAP32[$p>>2]|0;
   $63 = ((($62)) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = HEAP32[$p>>2]|0;
   $66 = ((($65)) + 8|0);
   $67 = (($66) + (($64*20)|0)|0);
   $68 = (($67) + ($61<<1)|0);
   HEAP32[$aIndex>>2] = $68;
   $69 = HEAP32[$iZero>>2]|0;
   $70 = (($69) + 1)|0;
   HEAP32[$iZero>>2] = $70;
   HEAP32[$j>>2] = 0;
   while(1) {
    $71 = HEAP32[$j>>2]|0;
    $72 = HEAP32[$nEntry>>2]|0;
    $73 = ($71|0)<($72|0);
    if (!($73)) {
     break;
    }
    $74 = HEAP32[$j>>2]|0;
    $75 = $74&65535;
    $76 = HEAP32[$j>>2]|0;
    $77 = HEAP32[$aIndex>>2]|0;
    $78 = (($77) + ($76<<1)|0);
    HEAP16[$78>>1] = $75;
    $79 = HEAP32[$j>>2]|0;
    $80 = (($79) + 1)|0;
    HEAP32[$j>>2] = $80;
   }
   $81 = HEAP32[$aPgno>>2]|0;
   $82 = HEAP32[$aTmp>>2]|0;
   $83 = HEAP32[$aIndex>>2]|0;
   _walMergesort($81,$82,$83,$nEntry);
   $84 = HEAP32[$iZero>>2]|0;
   $85 = HEAP32[$i>>2]|0;
   $86 = HEAP32[$p>>2]|0;
   $87 = ((($86)) + 8|0);
   $88 = (($87) + (($85*20)|0)|0);
   $89 = ((($88)) + 16|0);
   HEAP32[$89>>2] = $84;
   $90 = HEAP32[$nEntry>>2]|0;
   $91 = HEAP32[$i>>2]|0;
   $92 = HEAP32[$p>>2]|0;
   $93 = ((($92)) + 8|0);
   $94 = (($93) + (($91*20)|0)|0);
   $95 = ((($94)) + 12|0);
   HEAP32[$95>>2] = $90;
   $96 = HEAP32[$aIndex>>2]|0;
   $97 = HEAP32[$i>>2]|0;
   $98 = HEAP32[$p>>2]|0;
   $99 = ((($98)) + 8|0);
   $100 = (($99) + (($97*20)|0)|0);
   $101 = ((($100)) + 4|0);
   HEAP32[$101>>2] = $96;
   $102 = HEAP32[$aPgno>>2]|0;
   $103 = HEAP32[$i>>2]|0;
   $104 = HEAP32[$p>>2]|0;
   $105 = ((($104)) + 8|0);
   $106 = (($105) + (($103*20)|0)|0);
   $107 = ((($106)) + 8|0);
   HEAP32[$107>>2] = $102;
  }
  $108 = HEAP32[$i>>2]|0;
  $109 = (($108) + 1)|0;
  HEAP32[$i>>2] = $109;
 }
 $110 = HEAP32[$aTmp>>2]|0;
 _sqlite3_free($110);
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)!=(0);
 if ($112) {
  $113 = HEAP32[$p>>2]|0;
  _walIteratorFree($113);
 }
 $114 = HEAP32[$p>>2]|0;
 $115 = HEAP32[$2>>2]|0;
 HEAP32[$115>>2] = $114;
 $116 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $116;
 $117 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($117|0);
}
function _walIteratorNext($p,$piPage,$piFrame) {
 $p = $p|0;
 $piPage = $piPage|0;
 $piFrame = $piFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iMin = 0, $iPg = 0, $iRet = 0, $pSegment = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $iMin = sp + 16|0;
 $iRet = sp + 12|0;
 $i = sp + 8|0;
 $pSegment = sp + 4|0;
 $iPg = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piPage;
 HEAP32[$2>>2] = $piFrame;
 HEAP32[$iRet>>2] = -1;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$iMin>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 HEAP32[$i>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)>=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = (($13) + (($11*20)|0)|0);
  HEAP32[$pSegment>>2] = $14;
  while(1) {
   $15 = HEAP32[$pSegment>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pSegment>>2]|0;
   $18 = ((($17)) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($16|0)<($19|0);
   if (!($20)) {
    break;
   }
   $21 = HEAP32[$pSegment>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$pSegment>>2]|0;
   $24 = ((($23)) + 4|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + ($22<<1)|0);
   $27 = HEAP16[$26>>1]|0;
   $28 = $27&65535;
   $29 = HEAP32[$pSegment>>2]|0;
   $30 = ((($29)) + 8|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + ($28<<2)|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$iPg>>2] = $33;
   $34 = HEAP32[$iPg>>2]|0;
   $35 = HEAP32[$iMin>>2]|0;
   $36 = ($34>>>0)>($35>>>0);
   if ($36) {
    label = 6;
    break;
   }
   $54 = HEAP32[$pSegment>>2]|0;
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + 1)|0;
   HEAP32[$54>>2] = $56;
  }
  if ((label|0) == 6) {
   label = 0;
   $37 = HEAP32[$iPg>>2]|0;
   $38 = HEAP32[$iRet>>2]|0;
   $39 = ($37>>>0)<($38>>>0);
   if ($39) {
    $40 = HEAP32[$iPg>>2]|0;
    HEAP32[$iRet>>2] = $40;
    $41 = HEAP32[$pSegment>>2]|0;
    $42 = ((($41)) + 16|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$pSegment>>2]|0;
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$pSegment>>2]|0;
    $47 = ((($46)) + 4|0);
    $48 = HEAP32[$47>>2]|0;
    $49 = (($48) + ($45<<1)|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = $50&65535;
    $52 = (($43) + ($51))|0;
    $53 = HEAP32[$2>>2]|0;
    HEAP32[$53>>2] = $52;
   }
  }
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + -1)|0;
  HEAP32[$i>>2] = $58;
 }
 $59 = HEAP32[$iRet>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 HEAP32[$60>>2] = $59;
 $61 = HEAP32[$1>>2]|0;
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$iRet>>2]|0;
 $63 = ($62|0)==(-1);
 $64 = $63&1;
 STACKTOP = sp;return ($64|0);
}
function _walRestartHdr($pWal,$salt1) {
 $pWal = $pWal|0;
 $salt1 = $salt1|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aSalt = 0, $i = 0, $pInfo = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pInfo = sp + 8|0;
 $i = sp + 4|0;
 $aSalt = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $salt1;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_walCkptInfo($2)|0);
 HEAP32[$pInfo>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = ((($5)) + 32|0);
 HEAP32[$aSalt>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 112|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 52|0);
 $13 = ((($12)) + 16|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$aSalt>>2]|0;
 $15 = HEAP32[$aSalt>>2]|0;
 $16 = (_sqlite3Get4byte($15)|0);
 $17 = (1 + ($16))|0;
 _sqlite3Put4byte($14,$17);
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 52|0);
 $20 = ((($19)) + 32|0);
 $21 = ((($20)) + 4|0);
 ;HEAP32[$21>>2]=HEAP32[$1>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 _walIndexWriteHdr($22);
 $23 = HEAP32[$pInfo>>2]|0;
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$pInfo>>2]|0;
 $25 = ((($24)) + 32|0);
 HEAP32[$25>>2] = 0;
 $26 = HEAP32[$pInfo>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = ((($27)) + 4|0);
 HEAP32[$28>>2] = 0;
 HEAP32[$i>>2] = 2;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = ($29|0)<(5);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pInfo>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = (($33) + ($31<<2)|0);
  HEAP32[$34>>2] = -1;
  $35 = HEAP32[$i>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$i>>2] = $36;
 }
 STACKTOP = sp;return;
}
function _walIteratorFree($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3_free($1);
 STACKTOP = sp;return;
}
function _sqlite3Put4byte($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $v;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 >>> 24;
 $4 = $3&255;
 $5 = HEAP32[$0>>2]|0;
 HEAP8[$5>>0] = $4;
 $6 = HEAP32[$1>>2]|0;
 $7 = $6 >>> 16;
 $8 = $7&255;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 1|0);
 HEAP8[$10>>0] = $8;
 $11 = HEAP32[$1>>2]|0;
 $12 = $11 >>> 8;
 $13 = $12&255;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 2|0);
 HEAP8[$15>>0] = $13;
 $16 = HEAP32[$1>>2]|0;
 $17 = $16&255;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 3|0);
 HEAP8[$19>>0] = $17;
 STACKTOP = sp;return;
}
function _walIndexWriteHdr($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aHdr = 0, $nCksum = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $aHdr = sp + 4|0;
 $nCksum = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_walIndexHdr($1)|0);
 HEAP32[$aHdr>>2] = $2;
 HEAP32[$nCksum>>2] = 40;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = ((($4)) + 12|0);
 HEAP8[$5>>0] = 1;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 52|0);
 HEAP32[$7>>2] = 3007000;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 52|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 52|0);
 $12 = ((($11)) + 40|0);
 _walChecksumBytes(1,$9,40,0,$12);
 $13 = HEAP32[$aHdr>>2]|0;
 $14 = ((($13)) + 48|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 52|0);
 dest=$14; src=$16; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $17 = HEAP32[$0>>2]|0;
 _walShmBarrier($17);
 $18 = HEAP32[$aHdr>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 52|0);
 dest=$18; src=$20; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _walChecksumBytes($nativeCksum,$a,$nByte,$aIn,$aOut) {
 $nativeCksum = $nativeCksum|0;
 $a = $a|0;
 $nByte = $nByte|0;
 $aIn = $aIn|0;
 $aOut = $aOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $aData = 0, $aEnd = 0, $s1 = 0, $s2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $s1 = sp + 12|0;
 $s2 = sp + 8|0;
 $aData = sp + 4|0;
 $aEnd = sp;
 HEAP32[$0>>2] = $nativeCksum;
 HEAP32[$1>>2] = $a;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $aIn;
 HEAP32[$4>>2] = $aOut;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$aData>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (($7) + ($6)|0);
 HEAP32[$aEnd>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$s1>>2] = $12;
  $13 = HEAP32[$3>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$s2>>2] = $15;
 } else {
  HEAP32[$s2>>2] = 0;
  HEAP32[$s1>>2] = 0;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  while(1) {
   $18 = HEAP32[$aData>>2]|0;
   $19 = ((($18)) + 4|0);
   HEAP32[$aData>>2] = $19;
   $20 = HEAP32[$18>>2]|0;
   $21 = HEAP32[$s2>>2]|0;
   $22 = (($20) + ($21))|0;
   $23 = HEAP32[$s1>>2]|0;
   $24 = (($23) + ($22))|0;
   HEAP32[$s1>>2] = $24;
   $25 = HEAP32[$aData>>2]|0;
   $26 = ((($25)) + 4|0);
   HEAP32[$aData>>2] = $26;
   $27 = HEAP32[$25>>2]|0;
   $28 = HEAP32[$s1>>2]|0;
   $29 = (($27) + ($28))|0;
   $30 = HEAP32[$s2>>2]|0;
   $31 = (($30) + ($29))|0;
   HEAP32[$s2>>2] = $31;
   $32 = HEAP32[$aData>>2]|0;
   $33 = HEAP32[$aEnd>>2]|0;
   $34 = ($32>>>0)<($33>>>0);
   if (!($34)) {
    break;
   }
  }
  $90 = HEAP32[$s1>>2]|0;
  $91 = HEAP32[$4>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$s2>>2]|0;
  $93 = HEAP32[$4>>2]|0;
  $94 = ((($93)) + 4|0);
  HEAP32[$94>>2] = $92;
  STACKTOP = sp;return;
 } else {
  while(1) {
   $35 = HEAP32[$aData>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = $36 & 255;
   $38 = $37 << 24;
   $39 = HEAP32[$aData>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = $40 & 65280;
   $42 = $41 << 8;
   $43 = (($38) + ($42))|0;
   $44 = HEAP32[$aData>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = $45 & 16711680;
   $47 = $46 >>> 8;
   $48 = (($43) + ($47))|0;
   $49 = HEAP32[$aData>>2]|0;
   $50 = HEAP32[$49>>2]|0;
   $51 = $50 & -16777216;
   $52 = $51 >>> 24;
   $53 = (($48) + ($52))|0;
   $54 = HEAP32[$s2>>2]|0;
   $55 = (($53) + ($54))|0;
   $56 = HEAP32[$s1>>2]|0;
   $57 = (($56) + ($55))|0;
   HEAP32[$s1>>2] = $57;
   $58 = HEAP32[$aData>>2]|0;
   $59 = ((($58)) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = $60 & 255;
   $62 = $61 << 24;
   $63 = HEAP32[$aData>>2]|0;
   $64 = ((($63)) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 & 65280;
   $67 = $66 << 8;
   $68 = (($62) + ($67))|0;
   $69 = HEAP32[$aData>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = $71 & 16711680;
   $73 = $72 >>> 8;
   $74 = (($68) + ($73))|0;
   $75 = HEAP32[$aData>>2]|0;
   $76 = ((($75)) + 4|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = $77 & -16777216;
   $79 = $78 >>> 24;
   $80 = (($74) + ($79))|0;
   $81 = HEAP32[$s1>>2]|0;
   $82 = (($80) + ($81))|0;
   $83 = HEAP32[$s2>>2]|0;
   $84 = (($83) + ($82))|0;
   HEAP32[$s2>>2] = $84;
   $85 = HEAP32[$aData>>2]|0;
   $86 = ((($85)) + 8|0);
   HEAP32[$aData>>2] = $86;
   $87 = HEAP32[$aData>>2]|0;
   $88 = HEAP32[$aEnd>>2]|0;
   $89 = ($87>>>0)<($88>>>0);
   if (!($89)) {
    break;
   }
  }
  $90 = HEAP32[$s1>>2]|0;
  $91 = HEAP32[$4>>2]|0;
  HEAP32[$91>>2] = $90;
  $92 = HEAP32[$s2>>2]|0;
  $93 = HEAP32[$4>>2]|0;
  $94 = ((($93)) + 4|0);
  HEAP32[$94>>2] = $92;
  STACKTOP = sp;return;
 }
}
function _walShmBarrier($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 43|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(2);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 _sqlite3OsShmBarrier($8);
 STACKTOP = sp;return;
}
function _sqlite3OsShmBarrier($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 FUNCTION_TABLE_vi[$4 & 255]($5);
 STACKTOP = sp;return;
}
function _walMergesort($aContent,$aBuffer,$aList,$pnList) {
 $aContent = $aContent|0;
 $aBuffer = $aBuffer|0;
 $aList = $aList|0;
 $pnList = $pnList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aMerge = 0;
 var $aSub = 0, $iList = 0, $iSub = 0, $nList = 0, $nMerge = 0, $p = 0, $p1 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 144|0;
 $1 = sp + 140|0;
 $2 = sp + 136|0;
 $3 = sp + 132|0;
 $nList = sp + 128|0;
 $nMerge = sp + 124|0;
 $aMerge = sp + 120|0;
 $iList = sp + 116|0;
 $iSub = sp + 112|0;
 $aSub = sp + 8|0;
 $p = sp + 4|0;
 $p1 = sp;
 HEAP32[$0>>2] = $aContent;
 HEAP32[$1>>2] = $aBuffer;
 HEAP32[$2>>2] = $aList;
 HEAP32[$3>>2] = $pnList;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nList>>2] = $5;
 HEAP32[$nMerge>>2] = 0;
 HEAP32[$aMerge>>2] = 0;
 HEAP32[$iSub>>2] = 0;
 dest=$aSub; stop=dest+104|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$iList>>2] = 0;
 while(1) {
  $6 = HEAP32[$iList>>2]|0;
  $7 = HEAP32[$nList>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   break;
  }
  HEAP32[$nMerge>>2] = 1;
  $9 = HEAP32[$iList>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (($10) + ($9<<1)|0);
  HEAP32[$aMerge>>2] = $11;
  HEAP32[$iSub>>2] = 0;
  while(1) {
   $12 = HEAP32[$iList>>2]|0;
   $13 = HEAP32[$iSub>>2]|0;
   $14 = 1 << $13;
   $15 = $12 & $14;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    break;
   }
   $17 = HEAP32[$iSub>>2]|0;
   $18 = (($aSub) + ($17<<3)|0);
   HEAP32[$p>>2] = $18;
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$p>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   _walMerge($19,$22,$24,$aMerge,$nMerge,$25);
   $26 = HEAP32[$iSub>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$iSub>>2] = $27;
  }
  $28 = HEAP32[$aMerge>>2]|0;
  $29 = HEAP32[$iSub>>2]|0;
  $30 = (($aSub) + ($29<<3)|0);
  $31 = ((($30)) + 4|0);
  HEAP32[$31>>2] = $28;
  $32 = HEAP32[$nMerge>>2]|0;
  $33 = HEAP32[$iSub>>2]|0;
  $34 = (($aSub) + ($33<<3)|0);
  HEAP32[$34>>2] = $32;
  $35 = HEAP32[$iList>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$iList>>2] = $36;
 }
 $37 = HEAP32[$iSub>>2]|0;
 $38 = (($37) + 1)|0;
 HEAP32[$iSub>>2] = $38;
 while(1) {
  $39 = HEAP32[$iSub>>2]|0;
  $40 = ($39>>>0)<(13);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$nList>>2]|0;
  $42 = HEAP32[$iSub>>2]|0;
  $43 = 1 << $42;
  $44 = $41 & $43;
  $45 = ($44|0)!=(0);
  if ($45) {
   $46 = HEAP32[$iSub>>2]|0;
   $47 = (($aSub) + ($46<<3)|0);
   HEAP32[$p1>>2] = $47;
   $48 = HEAP32[$0>>2]|0;
   $49 = HEAP32[$p1>>2]|0;
   $50 = ((($49)) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$p1>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   _walMerge($48,$51,$53,$aMerge,$nMerge,$54);
  }
  $55 = HEAP32[$iSub>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$iSub>>2] = $56;
 }
 $57 = HEAP32[$nMerge>>2]|0;
 $58 = HEAP32[$3>>2]|0;
 HEAP32[$58>>2] = $57;
 STACKTOP = sp;return;
}
function _walMerge($aContent,$aLeft,$nLeft,$paRight,$pnRight,$aTmp) {
 $aContent = $aContent|0;
 $aLeft = $aLeft|0;
 $nLeft = $nLeft|0;
 $paRight = $paRight|0;
 $pnRight = $pnRight|0;
 $aTmp = $aTmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $aRight = 0, $dbpage = 0, $iLeft = 0, $iOut = 0, $iRight = 0, $logpage = 0, $nRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $iLeft = sp + 20|0;
 $iRight = sp + 16|0;
 $iOut = sp + 12|0;
 $nRight = sp + 8|0;
 $aRight = sp + 4|0;
 $logpage = sp + 48|0;
 $dbpage = sp;
 HEAP32[$0>>2] = $aContent;
 HEAP32[$1>>2] = $aLeft;
 HEAP32[$2>>2] = $nLeft;
 HEAP32[$3>>2] = $paRight;
 HEAP32[$4>>2] = $pnRight;
 HEAP32[$5>>2] = $aTmp;
 HEAP32[$iLeft>>2] = 0;
 HEAP32[$iRight>>2] = 0;
 HEAP32[$iOut>>2] = 0;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nRight>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aRight>>2] = $9;
 while(1) {
  $10 = HEAP32[$iRight>>2]|0;
  $11 = HEAP32[$nRight>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   $13 = HEAP32[$iLeft>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ($13|0)<($14|0);
   if (!($15)) {
    break;
   }
  }
  $16 = HEAP32[$iLeft>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ($16|0)<($17|0);
  do {
   if ($18) {
    $19 = HEAP32[$iRight>>2]|0;
    $20 = HEAP32[$nRight>>2]|0;
    $21 = ($19|0)>=($20|0);
    if (!($21)) {
     $22 = HEAP32[$iLeft>>2]|0;
     $23 = HEAP32[$1>>2]|0;
     $24 = (($23) + ($22<<1)|0);
     $25 = HEAP16[$24>>1]|0;
     $26 = $25&65535;
     $27 = HEAP32[$0>>2]|0;
     $28 = (($27) + ($26<<2)|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = HEAP32[$iRight>>2]|0;
     $31 = HEAP32[$aRight>>2]|0;
     $32 = (($31) + ($30<<1)|0);
     $33 = HEAP16[$32>>1]|0;
     $34 = $33&65535;
     $35 = HEAP32[$0>>2]|0;
     $36 = (($35) + ($34<<2)|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = ($29>>>0)<($37>>>0);
     if (!($38)) {
      label = 8;
      break;
     }
    }
    $39 = HEAP32[$iLeft>>2]|0;
    $40 = (($39) + 1)|0;
    HEAP32[$iLeft>>2] = $40;
    $41 = HEAP32[$1>>2]|0;
    $42 = (($41) + ($39<<1)|0);
    $43 = HEAP16[$42>>1]|0;
    HEAP16[$logpage>>1] = $43;
   } else {
    label = 8;
   }
  } while(0);
  if ((label|0) == 8) {
   label = 0;
   $44 = HEAP32[$iRight>>2]|0;
   $45 = (($44) + 1)|0;
   HEAP32[$iRight>>2] = $45;
   $46 = HEAP32[$aRight>>2]|0;
   $47 = (($46) + ($44<<1)|0);
   $48 = HEAP16[$47>>1]|0;
   HEAP16[$logpage>>1] = $48;
  }
  $49 = HEAP16[$logpage>>1]|0;
  $50 = $49&65535;
  $51 = HEAP32[$0>>2]|0;
  $52 = (($51) + ($50<<2)|0);
  $53 = HEAP32[$52>>2]|0;
  HEAP32[$dbpage>>2] = $53;
  $54 = HEAP16[$logpage>>1]|0;
  $55 = HEAP32[$iOut>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$iOut>>2] = $56;
  $57 = HEAP32[$5>>2]|0;
  $58 = (($57) + ($55<<1)|0);
  HEAP16[$58>>1] = $54;
  $59 = HEAP32[$iLeft>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   continue;
  }
  $62 = HEAP32[$iLeft>>2]|0;
  $63 = HEAP32[$1>>2]|0;
  $64 = (($63) + ($62<<1)|0);
  $65 = HEAP16[$64>>1]|0;
  $66 = $65&65535;
  $67 = HEAP32[$0>>2]|0;
  $68 = (($67) + ($66<<2)|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$dbpage>>2]|0;
  $71 = ($69|0)==($70|0);
  if (!($71)) {
   continue;
  }
  $72 = HEAP32[$iLeft>>2]|0;
  $73 = (($72) + 1)|0;
  HEAP32[$iLeft>>2] = $73;
 }
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$3>>2]|0;
 HEAP32[$75>>2] = $74;
 $76 = HEAP32[$iOut>>2]|0;
 $77 = HEAP32[$4>>2]|0;
 HEAP32[$77>>2] = $76;
 $78 = HEAP32[$1>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = HEAP32[$iOut>>2]|0;
 $81 = $80<<1;
 _memcpy(($78|0),($79|0),($81|0))|0;
 STACKTOP = sp;return;
}
function _walIndexTryHdr($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aCksum = 0, $aHdr = 0, $h1 = 0, $h2 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $aCksum = sp + 104|0;
 $h1 = sp + 56|0;
 $h2 = sp + 8|0;
 $aHdr = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_walIndexHdr($3)|0);
 HEAP32[$aHdr>>2] = $4;
 $5 = HEAP32[$aHdr>>2]|0;
 dest=$h1; src=$5; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $6 = HEAP32[$1>>2]|0;
 _walShmBarrier($6);
 $7 = HEAP32[$aHdr>>2]|0;
 $8 = ((($7)) + 48|0);
 dest=$h2; src=$8; stop=dest+48|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $9 = (_memcmp($h1,$h2,48)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $11 = ((($h1)) + 12|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(0);
 if ($14) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 _walChecksumBytes(1,$h1,40,0,$aCksum);
 $15 = HEAP32[$aCksum>>2]|0;
 $16 = ((($h1)) + 40|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($15|0)!=($17|0);
 if (!($18)) {
  $19 = ((($aCksum)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ((($h1)) + 40|0);
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)!=($23|0);
  if (!($24)) {
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 52|0);
   $27 = (_memcmp($26,$h1,48)|0);
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$2>>2]|0;
    HEAP32[$29>>2] = 1;
    $30 = HEAP32[$1>>2]|0;
    $31 = ((($30)) + 52|0);
    dest=$31; src=$h1; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 52|0);
    $34 = ((($33)) + 14|0);
    $35 = HEAP16[$34>>1]|0;
    $36 = $35&65535;
    $37 = $36 & 65024;
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 52|0);
    $40 = ((($39)) + 14|0);
    $41 = HEAP16[$40>>1]|0;
    $42 = $41&65535;
    $43 = $42 & 1;
    $44 = $43 << 16;
    $45 = (($37) + ($44))|0;
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 36|0);
    HEAP32[$47>>2] = $45;
   }
   HEAP32[$0>>2] = 0;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _walIndexRecover($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBuf = 0, $aData = 0, $aFrame = 0;
 var $aFrameCksum = 0, $i = 0, $iFrame = 0, $iLock = 0, $iOffset = 0, $isValid = 0, $magic = 0, $nLock = 0, $nSize = 0, $nTruncate = 0, $or$cond = 0, $or$cond3 = 0, $pInfo = 0, $pgno = 0, $rc = 0, $szFrame = 0, $szPage = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, $version = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $rc = sp + 88|0;
 $nSize = sp + 8|0;
 $aFrameCksum = sp + 80|0;
 $iLock = sp + 76|0;
 $nLock = sp + 72|0;
 $aBuf = sp + 104|0;
 $aFrame = sp + 68|0;
 $szFrame = sp + 64|0;
 $aData = sp + 60|0;
 $iFrame = sp + 56|0;
 $iOffset = sp;
 $szPage = sp + 52|0;
 $magic = sp + 48|0;
 $version = sp + 44|0;
 $isValid = sp + 40|0;
 $pgno = sp + 36|0;
 $nTruncate = sp + 32|0;
 $pInfo = sp + 28|0;
 $i = sp + 24|0;
 HEAP32[$1>>2] = $pWal;
 ;HEAP32[$aFrameCksum>>2]=0|0;HEAP32[$aFrameCksum+4>>2]=0|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 45|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = (1 + ($5))|0;
 HEAP32[$iLock>>2] = $6;
 $7 = HEAP32[$iLock>>2]|0;
 $8 = (8 - ($7))|0;
 HEAP32[$nLock>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$iLock>>2]|0;
 $11 = HEAP32[$nLock>>2]|0;
 $12 = (_walLockExclusive($9,$10,$11)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $276 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($276|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 52|0);
 dest=$17; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3OsFileSize($20,$nSize)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)!=(0);
 L5: do {
  if (!($23)) {
   $24 = $nSize;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (($24) + 4)|0;
   $28 = $27;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)>(0);
   $31 = ($26>>>0)>(32);
   $32 = ($29|0)==(0);
   $33 = $32 & $31;
   $34 = $30 | $33;
   do {
    if ($34) {
     HEAP32[$aFrame>>2] = 0;
     $35 = HEAP32[$1>>2]|0;
     $36 = ((($35)) + 8|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = (_sqlite3OsRead($37,$aBuf,32,0,0)|0);
     HEAP32[$rc>>2] = $38;
     $39 = HEAP32[$rc>>2]|0;
     $40 = ($39|0)!=(0);
     if ($40) {
      break L5;
     }
     $41 = (_sqlite3Get4byte($aBuf)|0);
     HEAP32[$magic>>2] = $41;
     $42 = ((($aBuf)) + 8|0);
     $43 = (_sqlite3Get4byte($42)|0);
     HEAP32[$szPage>>2] = $43;
     $44 = HEAP32[$magic>>2]|0;
     $45 = $44 & -2;
     $46 = ($45|0)!=(931071618);
     if (!($46)) {
      $47 = HEAP32[$szPage>>2]|0;
      $48 = HEAP32[$szPage>>2]|0;
      $49 = (($48) - 1)|0;
      $50 = $47 & $49;
      $51 = ($50|0)!=(0);
      $52 = HEAP32[$szPage>>2]|0;
      $53 = ($52|0)>(65536);
      $or$cond = $51 | $53;
      $54 = HEAP32[$szPage>>2]|0;
      $55 = ($54|0)<(512);
      $or$cond3 = $or$cond | $55;
      if (!($or$cond3)) {
       $56 = HEAP32[$magic>>2]|0;
       $57 = $56 & 1;
       $58 = $57&255;
       $59 = HEAP32[$1>>2]|0;
       $60 = ((($59)) + 52|0);
       $61 = ((($60)) + 13|0);
       HEAP8[$61>>0] = $58;
       $62 = HEAP32[$szPage>>2]|0;
       $63 = HEAP32[$1>>2]|0;
       $64 = ((($63)) + 36|0);
       HEAP32[$64>>2] = $62;
       $65 = ((($aBuf)) + 12|0);
       $66 = (_sqlite3Get4byte($65)|0);
       $67 = HEAP32[$1>>2]|0;
       $68 = ((($67)) + 112|0);
       HEAP32[$68>>2] = $66;
       $69 = HEAP32[$1>>2]|0;
       $70 = ((($69)) + 52|0);
       $71 = ((($70)) + 32|0);
       $72 = ((($aBuf)) + 16|0);
       ;HEAP8[$71>>0]=HEAP8[$72>>0]|0;HEAP8[$71+1>>0]=HEAP8[$72+1>>0]|0;HEAP8[$71+2>>0]=HEAP8[$72+2>>0]|0;HEAP8[$71+3>>0]=HEAP8[$72+3>>0]|0;HEAP8[$71+4>>0]=HEAP8[$72+4>>0]|0;HEAP8[$71+5>>0]=HEAP8[$72+5>>0]|0;HEAP8[$71+6>>0]=HEAP8[$72+6>>0]|0;HEAP8[$71+7>>0]=HEAP8[$72+7>>0]|0;
       $73 = HEAP32[$1>>2]|0;
       $74 = ((($73)) + 52|0);
       $75 = ((($74)) + 13|0);
       $76 = HEAP8[$75>>0]|0;
       $77 = $76&255;
       $78 = HEAP8[968>>0]|0;
       $79 = $78 << 24 >> 24;
       $80 = ($79|0)==(0);
       $81 = $80&1;
       $82 = ($77|0)==($81|0);
       $83 = $82&1;
       $84 = HEAP32[$1>>2]|0;
       $85 = ((($84)) + 52|0);
       $86 = ((($85)) + 24|0);
       _walChecksumBytes($83,$aBuf,24,0,$86);
       $87 = HEAP32[$1>>2]|0;
       $88 = ((($87)) + 52|0);
       $89 = ((($88)) + 24|0);
       $90 = HEAP32[$89>>2]|0;
       $91 = ((($aBuf)) + 24|0);
       $92 = (_sqlite3Get4byte($91)|0);
       $93 = ($90|0)!=($92|0);
       if (!($93)) {
        $94 = HEAP32[$1>>2]|0;
        $95 = ((($94)) + 52|0);
        $96 = ((($95)) + 24|0);
        $97 = ((($96)) + 4|0);
        $98 = HEAP32[$97>>2]|0;
        $99 = ((($aBuf)) + 28|0);
        $100 = (_sqlite3Get4byte($99)|0);
        $101 = ($98|0)!=($100|0);
        if (!($101)) {
         $102 = ((($aBuf)) + 4|0);
         $103 = (_sqlite3Get4byte($102)|0);
         HEAP32[$version>>2] = $103;
         $104 = HEAP32[$version>>2]|0;
         $105 = ($104|0)!=(3007000);
         if ($105) {
          $106 = (_sqlite3CantopenError(54951)|0);
          HEAP32[$rc>>2] = $106;
          break;
         }
         $107 = HEAP32[$szPage>>2]|0;
         $108 = (($107) + 24)|0;
         HEAP32[$szFrame>>2] = $108;
         $109 = HEAP32[$szFrame>>2]|0;
         $110 = ($109|0)<(0);
         $111 = $110 << 31 >> 31;
         $112 = (_sqlite3_malloc64($109,$111)|0);
         HEAP32[$aFrame>>2] = $112;
         $113 = HEAP32[$aFrame>>2]|0;
         $114 = ($113|0)!=(0|0);
         if (!($114)) {
          HEAP32[$rc>>2] = 7;
          break L5;
         }
         $115 = HEAP32[$aFrame>>2]|0;
         $116 = ((($115)) + 24|0);
         HEAP32[$aData>>2] = $116;
         HEAP32[$iFrame>>2] = 0;
         $117 = $iOffset;
         $118 = $117;
         HEAP32[$118>>2] = 32;
         $119 = (($117) + 4)|0;
         $120 = $119;
         HEAP32[$120>>2] = 0;
         while(1) {
          $121 = $iOffset;
          $122 = $121;
          $123 = HEAP32[$122>>2]|0;
          $124 = (($121) + 4)|0;
          $125 = $124;
          $126 = HEAP32[$125>>2]|0;
          $127 = HEAP32[$szFrame>>2]|0;
          $128 = ($127|0)<(0);
          $129 = $128 << 31 >> 31;
          $130 = (_i64Add(($123|0),($126|0),($127|0),($129|0))|0);
          $131 = tempRet0;
          $132 = $nSize;
          $133 = $132;
          $134 = HEAP32[$133>>2]|0;
          $135 = (($132) + 4)|0;
          $136 = $135;
          $137 = HEAP32[$136>>2]|0;
          $138 = ($131|0)<($137|0);
          $139 = ($130>>>0)<=($134>>>0);
          $140 = ($131|0)==($137|0);
          $141 = $140 & $139;
          $142 = $138 | $141;
          if (!($142)) {
           break;
          }
          $143 = HEAP32[$iFrame>>2]|0;
          $144 = (($143) + 1)|0;
          HEAP32[$iFrame>>2] = $144;
          $145 = HEAP32[$1>>2]|0;
          $146 = ((($145)) + 8|0);
          $147 = HEAP32[$146>>2]|0;
          $148 = HEAP32[$aFrame>>2]|0;
          $149 = HEAP32[$szFrame>>2]|0;
          $150 = $iOffset;
          $151 = $150;
          $152 = HEAP32[$151>>2]|0;
          $153 = (($150) + 4)|0;
          $154 = $153;
          $155 = HEAP32[$154>>2]|0;
          $156 = (_sqlite3OsRead($147,$148,$149,$152,$155)|0);
          HEAP32[$rc>>2] = $156;
          $157 = HEAP32[$rc>>2]|0;
          $158 = ($157|0)!=(0);
          if ($158) {
           break;
          }
          $159 = HEAP32[$1>>2]|0;
          $160 = HEAP32[$aData>>2]|0;
          $161 = HEAP32[$aFrame>>2]|0;
          $162 = (_walDecodeFrame($159,$pgno,$nTruncate,$160,$161)|0);
          HEAP32[$isValid>>2] = $162;
          $163 = HEAP32[$isValid>>2]|0;
          $164 = ($163|0)!=(0);
          if (!($164)) {
           break;
          }
          $165 = HEAP32[$1>>2]|0;
          $166 = HEAP32[$iFrame>>2]|0;
          $167 = HEAP32[$pgno>>2]|0;
          $168 = (_walIndexAppend($165,$166,$167)|0);
          HEAP32[$rc>>2] = $168;
          $169 = HEAP32[$rc>>2]|0;
          $170 = ($169|0)!=(0);
          if ($170) {
           break;
          }
          $171 = HEAP32[$nTruncate>>2]|0;
          $172 = ($171|0)!=(0);
          if ($172) {
           $173 = HEAP32[$iFrame>>2]|0;
           $174 = HEAP32[$1>>2]|0;
           $175 = ((($174)) + 52|0);
           $176 = ((($175)) + 16|0);
           HEAP32[$176>>2] = $173;
           $177 = HEAP32[$nTruncate>>2]|0;
           $178 = HEAP32[$1>>2]|0;
           $179 = ((($178)) + 52|0);
           $180 = ((($179)) + 20|0);
           HEAP32[$180>>2] = $177;
           $181 = HEAP32[$szPage>>2]|0;
           $182 = $181 & 65280;
           $183 = HEAP32[$szPage>>2]|0;
           $184 = $183 >> 16;
           $185 = $182 | $184;
           $186 = $185&65535;
           $187 = HEAP32[$1>>2]|0;
           $188 = ((($187)) + 52|0);
           $189 = ((($188)) + 14|0);
           HEAP16[$189>>1] = $186;
           $190 = HEAP32[$1>>2]|0;
           $191 = ((($190)) + 52|0);
           $192 = ((($191)) + 24|0);
           $193 = HEAP32[$192>>2]|0;
           HEAP32[$aFrameCksum>>2] = $193;
           $194 = HEAP32[$1>>2]|0;
           $195 = ((($194)) + 52|0);
           $196 = ((($195)) + 24|0);
           $197 = ((($196)) + 4|0);
           $198 = HEAP32[$197>>2]|0;
           $199 = ((($aFrameCksum)) + 4|0);
           HEAP32[$199>>2] = $198;
          }
          $200 = HEAP32[$szFrame>>2]|0;
          $201 = ($200|0)<(0);
          $202 = $201 << 31 >> 31;
          $203 = $iOffset;
          $204 = $203;
          $205 = HEAP32[$204>>2]|0;
          $206 = (($203) + 4)|0;
          $207 = $206;
          $208 = HEAP32[$207>>2]|0;
          $209 = (_i64Add(($205|0),($208|0),($200|0),($202|0))|0);
          $210 = tempRet0;
          $211 = $iOffset;
          $212 = $211;
          HEAP32[$212>>2] = $209;
          $213 = (($211) + 4)|0;
          $214 = $213;
          HEAP32[$214>>2] = $210;
         }
         $215 = HEAP32[$aFrame>>2]|0;
         _sqlite3_free($215);
        }
       }
      }
     }
    }
   } while(0);
   $216 = HEAP32[$rc>>2]|0;
   $217 = ($216|0)==(0);
   if ($217) {
    $218 = HEAP32[$aFrameCksum>>2]|0;
    $219 = HEAP32[$1>>2]|0;
    $220 = ((($219)) + 52|0);
    $221 = ((($220)) + 24|0);
    HEAP32[$221>>2] = $218;
    $222 = ((($aFrameCksum)) + 4|0);
    $223 = HEAP32[$222>>2]|0;
    $224 = HEAP32[$1>>2]|0;
    $225 = ((($224)) + 52|0);
    $226 = ((($225)) + 24|0);
    $227 = ((($226)) + 4|0);
    HEAP32[$227>>2] = $223;
    $228 = HEAP32[$1>>2]|0;
    _walIndexWriteHdr($228);
    $229 = HEAP32[$1>>2]|0;
    $230 = (_walCkptInfo($229)|0);
    HEAP32[$pInfo>>2] = $230;
    $231 = HEAP32[$pInfo>>2]|0;
    HEAP32[$231>>2] = 0;
    $232 = HEAP32[$1>>2]|0;
    $233 = ((($232)) + 52|0);
    $234 = ((($233)) + 16|0);
    $235 = HEAP32[$234>>2]|0;
    $236 = HEAP32[$pInfo>>2]|0;
    $237 = ((($236)) + 32|0);
    HEAP32[$237>>2] = $235;
    $238 = HEAP32[$pInfo>>2]|0;
    $239 = ((($238)) + 4|0);
    HEAP32[$239>>2] = 0;
    HEAP32[$i>>2] = 1;
    while(1) {
     $240 = HEAP32[$i>>2]|0;
     $241 = ($240|0)<(5);
     if (!($241)) {
      break;
     }
     $242 = HEAP32[$i>>2]|0;
     $243 = HEAP32[$pInfo>>2]|0;
     $244 = ((($243)) + 4|0);
     $245 = (($244) + ($242<<2)|0);
     HEAP32[$245>>2] = -1;
     $246 = HEAP32[$i>>2]|0;
     $247 = (($246) + 1)|0;
     HEAP32[$i>>2] = $247;
    }
    $248 = HEAP32[$1>>2]|0;
    $249 = ((($248)) + 52|0);
    $250 = ((($249)) + 16|0);
    $251 = HEAP32[$250>>2]|0;
    $252 = ($251|0)!=(0);
    if ($252) {
     $253 = HEAP32[$1>>2]|0;
     $254 = ((($253)) + 52|0);
     $255 = ((($254)) + 16|0);
     $256 = HEAP32[$255>>2]|0;
     $257 = HEAP32[$pInfo>>2]|0;
     $258 = ((($257)) + 4|0);
     $259 = ((($258)) + 4|0);
     HEAP32[$259>>2] = $256;
    }
    $260 = HEAP32[$1>>2]|0;
    $261 = ((($260)) + 52|0);
    $262 = ((($261)) + 20|0);
    $263 = HEAP32[$262>>2]|0;
    $264 = ($263|0)!=(0);
    if ($264) {
     $265 = HEAP32[$1>>2]|0;
     $266 = ((($265)) + 52|0);
     $267 = ((($266)) + 16|0);
     $268 = HEAP32[$267>>2]|0;
     $269 = HEAP32[$1>>2]|0;
     $270 = ((($269)) + 108|0);
     $271 = HEAP32[$270>>2]|0;
     HEAP32[$vararg_buffer>>2] = $268;
     $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr4>>2] = $271;
     _sqlite3_log(283,25952,$vararg_buffer);
    }
   }
  }
 } while(0);
 $272 = HEAP32[$1>>2]|0;
 $273 = HEAP32[$iLock>>2]|0;
 $274 = HEAP32[$nLock>>2]|0;
 _walUnlockExclusive($272,$273,$274);
 $275 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $275;
 $276 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($276|0);
}
function _sqlite3CantopenError($lineno) {
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $lineno;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_reportError(14,$1,25935)|0);
 STACKTOP = sp;return ($2|0);
}
function _walDecodeFrame($pWal,$piPage,$pnTruncate,$aData,$aFrame) {
 $pWal = $pWal|0;
 $piPage = $piPage|0;
 $pnTruncate = $pnTruncate|0;
 $aData = $aData|0;
 $aFrame = $aFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aCksum = 0, $nativeCksum = 0, $pgno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $nativeCksum = sp + 8|0;
 $aCksum = sp + 4|0;
 $pgno = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $piPage;
 HEAP32[$3>>2] = $pnTruncate;
 HEAP32[$4>>2] = $aData;
 HEAP32[$5>>2] = $aFrame;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = ((($7)) + 24|0);
 HEAP32[$aCksum>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = ((($10)) + 32|0);
 $12 = HEAP32[$5>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = (_memcmp($11,$13,8)|0);
 $15 = ($14|0)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $16 = HEAP32[$5>>2]|0;
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$pgno>>2] = $17;
 $18 = HEAP32[$pgno>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 52|0);
 $22 = ((($21)) + 13|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = HEAP8[968>>0]|0;
 $26 = $25 << 24 >> 24;
 $27 = ($26|0)==(0);
 $28 = $27&1;
 $29 = ($24|0)==($28|0);
 $30 = $29&1;
 HEAP32[$nativeCksum>>2] = $30;
 $31 = HEAP32[$nativeCksum>>2]|0;
 $32 = HEAP32[$5>>2]|0;
 $33 = HEAP32[$aCksum>>2]|0;
 $34 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($31,$32,8,$33,$34);
 $35 = HEAP32[$nativeCksum>>2]|0;
 $36 = HEAP32[$4>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 36|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$aCksum>>2]|0;
 $41 = HEAP32[$aCksum>>2]|0;
 _walChecksumBytes($35,$36,$39,$40,$41);
 $42 = HEAP32[$aCksum>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$5>>2]|0;
 $45 = ((($44)) + 16|0);
 $46 = (_sqlite3Get4byte($45)|0);
 $47 = ($43|0)!=($46|0);
 if (!($47)) {
  $48 = HEAP32[$aCksum>>2]|0;
  $49 = ((($48)) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$5>>2]|0;
  $52 = ((($51)) + 20|0);
  $53 = (_sqlite3Get4byte($52)|0);
  $54 = ($50|0)!=($53|0);
  if (!($54)) {
   $55 = HEAP32[$pgno>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   HEAP32[$56>>2] = $55;
   $57 = HEAP32[$5>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = (_sqlite3Get4byte($58)|0);
   $60 = HEAP32[$3>>2]|0;
   HEAP32[$60>>2] = $59;
   HEAP32[$0>>2] = 1;
   $61 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($61|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _walIndexAppend($pWal,$iFrame,$iPage) {
 $pWal = $pWal|0;
 $iFrame = $iFrame|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aHash = 0, $aPgno = 0, $iKey = 0, $iZero = 0;
 var $idx = 0, $nByte = 0, $nCollide = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $rc = sp + 28|0;
 $iZero = sp + 24|0;
 $aPgno = sp + 20|0;
 $aHash = sp + 16|0;
 $iKey = sp + 12|0;
 $idx = sp + 8|0;
 $nCollide = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iFrame;
 HEAP32[$3>>2] = $iPage;
 HEAP32[$iZero>>2] = 0;
 HEAP32[$aPgno>>2] = 0;
 HEAP32[$aHash>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_walFramePage($5)|0);
 $7 = (_walHashGet($4,$6,$aHash,$aPgno,$iZero)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)==(0);
 do {
  if ($9) {
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$iZero>>2]|0;
   $12 = (($10) - ($11))|0;
   HEAP32[$idx>>2] = $12;
   $13 = HEAP32[$idx>>2]|0;
   $14 = ($13|0)==(1);
   if ($14) {
    $15 = HEAP32[$aHash>>2]|0;
    $16 = ((($15)) + 16384|0);
    $17 = HEAP32[$aPgno>>2]|0;
    $18 = ((($17)) + 4|0);
    $19 = $16;
    $20 = $18;
    $21 = (($19) - ($20))|0;
    HEAP32[$nByte>>2] = $21;
    $22 = HEAP32[$aPgno>>2]|0;
    $23 = ((($22)) + 4|0);
    $24 = HEAP32[$nByte>>2]|0;
    _memset(($23|0),0,($24|0))|0;
   }
   $25 = HEAP32[$idx>>2]|0;
   $26 = HEAP32[$aPgno>>2]|0;
   $27 = (($26) + ($25<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)!=(0);
   if ($29) {
    $30 = HEAP32[$1>>2]|0;
    _walCleanupHash($30);
   }
   $31 = HEAP32[$idx>>2]|0;
   HEAP32[$nCollide>>2] = $31;
   $32 = HEAP32[$3>>2]|0;
   $33 = (_walHash($32)|0);
   HEAP32[$iKey>>2] = $33;
   while(1) {
    $34 = HEAP32[$iKey>>2]|0;
    $35 = HEAP32[$aHash>>2]|0;
    $36 = (($35) + ($34<<1)|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = ($37<<16>>16)!=(0);
    if (!($38)) {
     label = 11;
     break;
    }
    $39 = HEAP32[$nCollide>>2]|0;
    $40 = (($39) + -1)|0;
    HEAP32[$nCollide>>2] = $40;
    $41 = ($39|0)==(0);
    if ($41) {
     break;
    }
    $43 = HEAP32[$iKey>>2]|0;
    $44 = (_walNextHash($43)|0);
    HEAP32[$iKey>>2] = $44;
   }
   if ((label|0) == 11) {
    $45 = HEAP32[$3>>2]|0;
    $46 = HEAP32[$idx>>2]|0;
    $47 = HEAP32[$aPgno>>2]|0;
    $48 = (($47) + ($46<<2)|0);
    HEAP32[$48>>2] = $45;
    $49 = HEAP32[$idx>>2]|0;
    $50 = $49&65535;
    $51 = HEAP32[$iKey>>2]|0;
    $52 = HEAP32[$aHash>>2]|0;
    $53 = (($52) + ($51<<1)|0);
    HEAP16[$53>>1] = $50;
    break;
   }
   $42 = (_sqlite3CorruptError(54820)|0);
   HEAP32[$0>>2] = $42;
   $55 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
 } while(0);
 $54 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3InvokeBusyHandler($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if (!($3)) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)==(0|0);
  if (!($6)) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 8|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)<(0);
   if (!($10)) {
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP32[$11>>2]|0;
    $13 = HEAP32[$1>>2]|0;
    $14 = ((($13)) + 4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (FUNCTION_TABLE_iii[$12 & 255]($15,$18)|0);
    HEAP32[$rc>>2] = $19;
    $20 = HEAP32[$rc>>2]|0;
    $21 = ($20|0)==(0);
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 8|0);
    if ($21) {
     HEAP32[$23>>2] = -1;
    } else {
     $24 = HEAP32[$23>>2]|0;
     $25 = (($24) + 1)|0;
     HEAP32[$23>>2] = $25;
    }
    $26 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $26;
    $27 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($27|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3PcacheSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 48;
}
function _sqlite3JournalSize($pVfs) {
 $pVfs = $pVfs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVfs;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>(72);
 if (!($4)) {
  $8 = 72;
  STACKTOP = sp;return ($8|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7;
 STACKTOP = sp;return ($8|0);
}
function _pagerStress($p,$pPg) {
 $p = $p|0;
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pPager = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pPg;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pPager>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $8 = HEAP32[$pPager>>2]|0;
 $9 = ((($8)) + 21|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)!=(0);
 do {
  if ($12) {
   $13 = HEAP32[$pPager>>2]|0;
   $14 = ((($13)) + 21|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = $16 & 3;
   $18 = ($17|0)!=(0);
   if (!($18)) {
    $19 = HEAP32[$2>>2]|0;
    $20 = ((($19)) + 24|0);
    $21 = HEAP16[$20>>1]|0;
    $22 = $21&65535;
    $23 = $22 & 8;
    $24 = ($23|0)!=(0);
    if (!($24)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $60 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($60|0);
  }
 } while(0);
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 12|0);
 HEAP32[$26>>2] = 0;
 $27 = HEAP32[$pPager>>2]|0;
 $28 = (_sqlite3PagerUseWal($27)|0);
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$2>>2]|0;
 if ($29) {
  $31 = (_subjournalPageIfRequired($30)|0);
  HEAP32[$rc>>2] = $31;
  $32 = HEAP32[$rc>>2]|0;
  $33 = ($32|0)==(0);
  if ($33) {
   $34 = HEAP32[$pPager>>2]|0;
   $35 = HEAP32[$2>>2]|0;
   $36 = (_pagerWalFrames($34,$35,0,0)|0);
   HEAP32[$rc>>2] = $36;
  }
 } else {
  $37 = ((($30)) + 24|0);
  $38 = HEAP16[$37>>1]|0;
  $39 = $38&65535;
  $40 = $39 & 8;
  $41 = ($40|0)!=(0);
  if ($41) {
   label = 12;
  } else {
   $42 = HEAP32[$pPager>>2]|0;
   $43 = ((($42)) + 17|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = ($45|0)==(3);
   if ($46) {
    label = 12;
   }
  }
  if ((label|0) == 12) {
   $47 = HEAP32[$pPager>>2]|0;
   $48 = (_syncJournal($47,1)|0);
   HEAP32[$rc>>2] = $48;
  }
  $49 = HEAP32[$rc>>2]|0;
  $50 = ($49|0)==(0);
  if ($50) {
   $51 = HEAP32[$pPager>>2]|0;
   $52 = HEAP32[$2>>2]|0;
   $53 = (_pager_write_pagelist($51,$52)|0);
   HEAP32[$rc>>2] = $53;
  }
 }
 $54 = HEAP32[$rc>>2]|0;
 $55 = ($54|0)==(0);
 if ($55) {
  $56 = HEAP32[$2>>2]|0;
  _sqlite3PcacheMakeClean($56);
 }
 $57 = HEAP32[$pPager>>2]|0;
 $58 = HEAP32[$rc>>2]|0;
 $59 = (_pager_error($57,$58)|0);
 HEAP32[$0>>2] = $59;
 $60 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _sqlite3PcacheOpen($szPage,$szExtra,$bPurgeable,$xStress,$pStress,$p) {
 $szPage = $szPage|0;
 $szExtra = $szExtra|0;
 $bPurgeable = $bPurgeable|0;
 $xStress = $xStress|0;
 $pStress = $pStress|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$0>>2] = $szPage;
 HEAP32[$1>>2] = $szExtra;
 HEAP32[$2>>2] = $bPurgeable;
 HEAP32[$3>>2] = $xStress;
 HEAP32[$4>>2] = $pStress;
 HEAP32[$5>>2] = $p;
 $6 = HEAP32[$5>>2]|0;
 dest=$6; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$5>>2]|0;
 $8 = ((($7)) + 24|0);
 HEAP32[$8>>2] = 1;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$5>>2]|0;
 $11 = ((($10)) + 28|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = $12&255;
 $14 = HEAP32[$5>>2]|0;
 $15 = ((($14)) + 32|0);
 HEAP8[$15>>0] = $13;
 $16 = HEAP32[$5>>2]|0;
 $17 = ((($16)) + 33|0);
 HEAP8[$17>>0] = 2;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = ((($19)) + 36|0);
 HEAP32[$20>>2] = $18;
 $21 = HEAP32[$4>>2]|0;
 $22 = HEAP32[$5>>2]|0;
 $23 = ((($22)) + 40|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$5>>2]|0;
 $25 = ((($24)) + 16|0);
 HEAP32[$25>>2] = 100;
 $26 = HEAP32[$5>>2]|0;
 $27 = ((($26)) + 20|0);
 HEAP32[$27>>2] = 1;
 $28 = HEAP32[$5>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = (_sqlite3PcacheSetPageSize($28,$29)|0);
 STACKTOP = sp;return ($30|0);
}
function _subjournalPageIfRequired($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_subjRequiresPage($2)|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = (_subjournalPage($5)|0);
  HEAP32[$0>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 } else {
  HEAP32[$0>>2] = 0;
  $7 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($7|0);
 }
 return (0)|0;
}
function _pagerWalFrames($pPager,$pList,$nTruncate,$isCommit) {
 $pPager = $pPager|0;
 $pList = $pList|0;
 $nTruncate = $nTruncate|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $nList = 0, $p = 0, $ppNext = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $nList = sp + 8|0;
 $p = sp + 4|0;
 $ppNext = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $nTruncate;
 HEAP32[$3>>2] = $isCommit;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 L1: do {
  if ($5) {
   HEAP32[$ppNext>>2] = $1;
   HEAP32[$nList>>2] = 0;
   $6 = HEAP32[$1>>2]|0;
   HEAP32[$p>>2] = $6;
   while(1) {
    $7 = HEAP32[$p>>2]|0;
    $8 = HEAP32[$ppNext>>2]|0;
    HEAP32[$8>>2] = $7;
    $9 = ($7|0)!=(0|0);
    if (!($9)) {
     break L1;
    }
    $10 = HEAP32[$p>>2]|0;
    $11 = ((($10)) + 20|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = HEAP32[$2>>2]|0;
    $14 = ($12>>>0)<=($13>>>0);
    if ($14) {
     $15 = HEAP32[$p>>2]|0;
     $16 = ((($15)) + 12|0);
     HEAP32[$ppNext>>2] = $16;
     $17 = HEAP32[$nList>>2]|0;
     $18 = (($17) + 1)|0;
     HEAP32[$nList>>2] = $18;
    }
    $19 = HEAP32[$p>>2]|0;
    $20 = ((($19)) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$p>>2] = $21;
   }
  } else {
   HEAP32[$nList>>2] = 1;
  }
 } while(0);
 $22 = HEAP32[$nList>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 192|0);
 $25 = ((($24)) + 8|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($22))|0;
 HEAP32[$25>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 20|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(1);
 if ($31) {
  $32 = HEAP32[$1>>2]|0;
  _pager_write_changecounter($32);
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 216|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 160|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$3>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 11|0);
 $44 = HEAP8[$43>>0]|0;
 $45 = $44&255;
 $46 = (_sqlite3WalFrames($35,$38,$39,$40,$41,$45)|0);
 HEAP32[$rc>>2] = $46;
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)==(0);
 if (!($48)) {
  $68 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 96|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0|0);
 if (!($52)) {
  $68 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($68|0);
 }
 $53 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $53;
 while(1) {
  $54 = HEAP32[$p>>2]|0;
  $55 = ($54|0)!=(0|0);
  if (!($55)) {
   break;
  }
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 96|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$p>>2]|0;
  $60 = ((($59)) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$p>>2]|0;
  $63 = ((($62)) + 4|0);
  $64 = HEAP32[$63>>2]|0;
  _sqlite3BackupUpdate($58,$61,$64);
  $65 = HEAP32[$p>>2]|0;
  $66 = ((($65)) + 12|0);
  $67 = HEAP32[$66>>2]|0;
  HEAP32[$p>>2] = $67;
 }
 $68 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($68|0);
}
function _syncJournal($pPager,$newHdr) {
 $pPager = $pPager|0;
 $newHdr = $newHdr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aMagic = 0, $iDc = 0, $iNextHdrOffset = 0, $or$cond = 0, $rc = 0, $zHeader = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $iDc = sp + 8|0;
 $iNextHdrOffset = sp;
 $aMagic = sp + 40|0;
 $zHeader = sp + 28|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $newHdr;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3PagerExclusiveLock($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 7|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 68|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    $17 = HEAP32[$1>>2]|0;
    $18 = ((($17)) + 5|0);
    $19 = HEAP8[$18>>0]|0;
    $20 = $19&255;
    $21 = ($20|0)!=(4);
    if ($21) {
     $22 = HEAP32[$1>>2]|0;
     $23 = ((($22)) + 64|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = (_sqlite3OsDeviceCharacteristics($24)|0);
     HEAP32[$iDc>>2] = $25;
     $26 = HEAP32[$iDc>>2]|0;
     $27 = $26 & 512;
     $28 = (0)==($27|0);
     if ($28) {
      ;HEAP8[$zHeader>>0]=HEAP8[25901>>0]|0;HEAP8[$zHeader+1>>0]=HEAP8[25901+1>>0]|0;HEAP8[$zHeader+2>>0]=HEAP8[25901+2>>0]|0;HEAP8[$zHeader+3>>0]=HEAP8[25901+3>>0]|0;HEAP8[$zHeader+4>>0]=HEAP8[25901+4>>0]|0;HEAP8[$zHeader+5>>0]=HEAP8[25901+5>>0]|0;HEAP8[$zHeader+6>>0]=HEAP8[25901+6>>0]|0;HEAP8[$zHeader+7>>0]=HEAP8[25901+7>>0]|0;
      $29 = ((($zHeader)) + 8|0);
      $30 = HEAP32[$1>>2]|0;
      $31 = ((($30)) + 48|0);
      $32 = HEAP32[$31>>2]|0;
      _sqlite3Put4byte($29,$32);
      $33 = HEAP32[$1>>2]|0;
      $34 = (_journalHdrOffset($33)|0);
      $35 = tempRet0;
      $36 = $iNextHdrOffset;
      $37 = $36;
      HEAP32[$37>>2] = $34;
      $38 = (($36) + 4)|0;
      $39 = $38;
      HEAP32[$39>>2] = $35;
      $40 = HEAP32[$1>>2]|0;
      $41 = ((($40)) + 68|0);
      $42 = HEAP32[$41>>2]|0;
      $43 = $iNextHdrOffset;
      $44 = $43;
      $45 = HEAP32[$44>>2]|0;
      $46 = (($43) + 4)|0;
      $47 = $46;
      $48 = HEAP32[$47>>2]|0;
      $49 = (_sqlite3OsRead($42,$aMagic,8,$45,$48)|0);
      HEAP32[$rc>>2] = $49;
      $50 = HEAP32[$rc>>2]|0;
      $51 = ($50|0)==(0);
      if ($51) {
       $52 = (_memcmp($aMagic,25901,8)|0);
       $53 = (0)==($52|0);
       if ($53) {
        $54 = HEAP32[$1>>2]|0;
        $55 = ((($54)) + 68|0);
        $56 = HEAP32[$55>>2]|0;
        $57 = $iNextHdrOffset;
        $58 = $57;
        $59 = HEAP32[$58>>2]|0;
        $60 = (($57) + 4)|0;
        $61 = $60;
        $62 = HEAP32[$61>>2]|0;
        $63 = (_sqlite3OsWrite($56,26022,1,$59,$62)|0);
        HEAP32[$rc>>2] = $63;
       }
      }
      $64 = HEAP32[$rc>>2]|0;
      $65 = ($64|0)!=(0);
      $66 = HEAP32[$rc>>2]|0;
      $67 = ($66|0)!=(522);
      $or$cond = $65 & $67;
      if ($or$cond) {
       $68 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $68;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
      $69 = HEAP32[$1>>2]|0;
      $70 = ((($69)) + 8|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = $71&255;
      $73 = ($72|0)!=(0);
      if ($73) {
       $74 = HEAP32[$iDc>>2]|0;
       $75 = $74 & 1024;
       $76 = (0)==($75|0);
       if ($76) {
        $77 = HEAP32[$1>>2]|0;
        $78 = ((($77)) + 68|0);
        $79 = HEAP32[$78>>2]|0;
        $80 = HEAP32[$1>>2]|0;
        $81 = ((($80)) + 12|0);
        $82 = HEAP8[$81>>0]|0;
        $83 = $82&255;
        $84 = (_sqlite3OsSync($79,$83)|0);
        HEAP32[$rc>>2] = $84;
        $85 = HEAP32[$rc>>2]|0;
        $86 = ($85|0)!=(0);
        if ($86) {
         $87 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $87;
         $169 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($169|0);
        }
       }
      }
      $88 = HEAP32[$1>>2]|0;
      $89 = ((($88)) + 68|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = HEAP32[$1>>2]|0;
      $92 = ((($91)) + 88|0);
      $93 = $92;
      $94 = $93;
      $95 = HEAP32[$94>>2]|0;
      $96 = (($93) + 4)|0;
      $97 = $96;
      $98 = HEAP32[$97>>2]|0;
      $99 = (_sqlite3OsWrite($90,$zHeader,12,$95,$98)|0);
      HEAP32[$rc>>2] = $99;
      $100 = HEAP32[$rc>>2]|0;
      $101 = ($100|0)!=(0);
      if ($101) {
       $102 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $102;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
     }
     $103 = HEAP32[$iDc>>2]|0;
     $104 = $103 & 1024;
     $105 = (0)==($104|0);
     if ($105) {
      $106 = HEAP32[$1>>2]|0;
      $107 = ((($106)) + 68|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = HEAP32[$1>>2]|0;
      $110 = ((($109)) + 12|0);
      $111 = HEAP8[$110>>0]|0;
      $112 = $111&255;
      $113 = HEAP32[$1>>2]|0;
      $114 = ((($113)) + 12|0);
      $115 = HEAP8[$114>>0]|0;
      $116 = $115&255;
      $117 = ($116|0)==(3);
      $118 = $117 ? 16 : 0;
      $119 = $112 | $118;
      $120 = (_sqlite3OsSync($108,$119)|0);
      HEAP32[$rc>>2] = $120;
      $121 = HEAP32[$rc>>2]|0;
      $122 = ($121|0)!=(0);
      if ($122) {
       $123 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $123;
       $169 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($169|0);
      }
     }
     $124 = HEAP32[$1>>2]|0;
     $125 = ((($124)) + 80|0);
     $126 = $125;
     $127 = $126;
     $128 = HEAP32[$127>>2]|0;
     $129 = (($126) + 4)|0;
     $130 = $129;
     $131 = HEAP32[$130>>2]|0;
     $132 = HEAP32[$1>>2]|0;
     $133 = ((($132)) + 88|0);
     $134 = $133;
     $135 = $134;
     HEAP32[$135>>2] = $128;
     $136 = (($134) + 4)|0;
     $137 = $136;
     HEAP32[$137>>2] = $131;
     $138 = HEAP32[$2>>2]|0;
     $139 = ($138|0)!=(0);
     if (!($139)) {
      break;
     }
     $140 = HEAP32[$iDc>>2]|0;
     $141 = $140 & 512;
     $142 = (0)==($141|0);
     if (!($142)) {
      break;
     }
     $143 = HEAP32[$1>>2]|0;
     $144 = ((($143)) + 48|0);
     HEAP32[$144>>2] = 0;
     $145 = HEAP32[$1>>2]|0;
     $146 = (_writeJournalHdr($145)|0);
     HEAP32[$rc>>2] = $146;
     $147 = HEAP32[$rc>>2]|0;
     $148 = ($147|0)!=(0);
     if (!($148)) {
      break;
     }
     $149 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $149;
     $169 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($169|0);
    }
   }
   $150 = HEAP32[$1>>2]|0;
   $151 = ((($150)) + 80|0);
   $152 = $151;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = (($152) + 4)|0;
   $156 = $155;
   $157 = HEAP32[$156>>2]|0;
   $158 = HEAP32[$1>>2]|0;
   $159 = ((($158)) + 88|0);
   $160 = $159;
   $161 = $160;
   HEAP32[$161>>2] = $154;
   $162 = (($160) + 4)|0;
   $163 = $162;
   HEAP32[$163>>2] = $157;
  }
 } while(0);
 $164 = HEAP32[$1>>2]|0;
 $165 = ((($164)) + 212|0);
 $166 = HEAP32[$165>>2]|0;
 _sqlite3PcacheClearSyncFlags($166);
 $167 = HEAP32[$1>>2]|0;
 $168 = ((($167)) + 17|0);
 HEAP8[$168>>0] = 4;
 HEAP32[$0>>2] = 0;
 $169 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _pager_write_pagelist($pPager,$pList) {
 $pPager = $pPager|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $offset = 0, $pData = 0, $pgno = 0, $rc = 0, $szFile = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $rc = sp + 24|0;
 $szFile = sp + 8|0;
 $pgno = sp + 20|0;
 $offset = sp;
 $pData = sp + 16|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pList;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 152|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_pagerOpentemp($7,$10,$13)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 do {
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   $18 = ((($17)) + 40|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 28|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19>>>0)<($22>>>0);
   if ($23) {
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 12|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ($26|0)!=(0|0);
    if (!($27)) {
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 20|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$0>>2]|0;
     $32 = ((($31)) + 40|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($30>>>0)>($33>>>0);
     if (!($34)) {
      break;
     }
    }
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 160|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)<(0);
    $39 = $38 << 31 >> 31;
    $40 = HEAP32[$0>>2]|0;
    $41 = ((($40)) + 28|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = (___muldi3(($37|0),($39|0),($42|0),0)|0);
    $44 = tempRet0;
    $45 = $szFile;
    $46 = $45;
    HEAP32[$46>>2] = $43;
    $47 = (($45) + 4)|0;
    $48 = $47;
    HEAP32[$48>>2] = $44;
    $49 = HEAP32[$0>>2]|0;
    $50 = ((($49)) + 64|0);
    $51 = HEAP32[$50>>2]|0;
    _sqlite3OsFileControlHint($51,5,$szFile);
    $52 = HEAP32[$0>>2]|0;
    $53 = ((($52)) + 28|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = HEAP32[$0>>2]|0;
    $56 = ((($55)) + 40|0);
    HEAP32[$56>>2] = $54;
   }
  }
 } while(0);
 while(1) {
  $57 = HEAP32[$rc>>2]|0;
  $58 = ($57|0)==(0);
  $59 = HEAP32[$1>>2]|0;
  $60 = ($59|0)!=(0|0);
  $61 = $58 ? $60 : 0;
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 20|0);
  $64 = HEAP32[$63>>2]|0;
  HEAP32[$pgno>>2] = $64;
  $65 = HEAP32[$pgno>>2]|0;
  $66 = HEAP32[$0>>2]|0;
  $67 = ((($66)) + 28|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = ($65>>>0)<=($68>>>0);
  if ($69) {
   $70 = HEAP32[$1>>2]|0;
   $71 = ((($70)) + 24|0);
   $72 = HEAP16[$71>>1]|0;
   $73 = $72&65535;
   $74 = $73 & 16;
   $75 = (0)==($74|0);
   if ($75) {
    $76 = HEAP32[$pgno>>2]|0;
    $77 = (($76) - 1)|0;
    $78 = HEAP32[$0>>2]|0;
    $79 = ((($78)) + 160|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ($80|0)<(0);
    $82 = $81 << 31 >> 31;
    $83 = (___muldi3(($77|0),0,($80|0),($82|0))|0);
    $84 = tempRet0;
    $85 = $offset;
    $86 = $85;
    HEAP32[$86>>2] = $83;
    $87 = (($85) + 4)|0;
    $88 = $87;
    HEAP32[$88>>2] = $84;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 20|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(1);
    if ($92) {
     $93 = HEAP32[$1>>2]|0;
     _pager_write_changecounter($93);
    }
    $94 = HEAP32[$1>>2]|0;
    $95 = ((($94)) + 4|0);
    $96 = HEAP32[$95>>2]|0;
    HEAP32[$pData>>2] = $96;
    $97 = HEAP32[$0>>2]|0;
    $98 = ((($97)) + 64|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = HEAP32[$pData>>2]|0;
    $101 = HEAP32[$0>>2]|0;
    $102 = ((($101)) + 160|0);
    $103 = HEAP32[$102>>2]|0;
    $104 = $offset;
    $105 = $104;
    $106 = HEAP32[$105>>2]|0;
    $107 = (($104) + 4)|0;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (_sqlite3OsWrite($99,$100,$103,$106,$109)|0);
    HEAP32[$rc>>2] = $110;
    $111 = HEAP32[$pgno>>2]|0;
    $112 = ($111|0)==(1);
    if ($112) {
     $113 = HEAP32[$0>>2]|0;
     $114 = ((($113)) + 112|0);
     $115 = HEAP32[$pData>>2]|0;
     $116 = ((($115)) + 24|0);
     dest=$114; src=$116; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
    }
    $117 = HEAP32[$pgno>>2]|0;
    $118 = HEAP32[$0>>2]|0;
    $119 = ((($118)) + 36|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = ($117>>>0)>($120>>>0);
    if ($121) {
     $122 = HEAP32[$pgno>>2]|0;
     $123 = HEAP32[$0>>2]|0;
     $124 = ((($123)) + 36|0);
     HEAP32[$124>>2] = $122;
    }
    $125 = HEAP32[$0>>2]|0;
    $126 = ((($125)) + 192|0);
    $127 = ((($126)) + 8|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = (($128) + 1)|0;
    HEAP32[$127>>2] = $129;
    $130 = HEAP32[$0>>2]|0;
    $131 = ((($130)) + 96|0);
    $132 = HEAP32[$131>>2]|0;
    $133 = HEAP32[$pgno>>2]|0;
    $134 = HEAP32[$1>>2]|0;
    $135 = ((($134)) + 4|0);
    $136 = HEAP32[$135>>2]|0;
    _sqlite3BackupUpdate($132,$133,$136);
   }
  }
  $137 = HEAP32[$1>>2]|0;
  $138 = ((($137)) + 12|0);
  $139 = HEAP32[$138>>2]|0;
  HEAP32[$1>>2] = $139;
 }
 $140 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($140|0);
}
function _pagerOpentemp($pPager,$pFile,$vfsFlags) {
 $pPager = $pPager|0;
 $pFile = $pFile|0;
 $vfsFlags = $vfsFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $vfsFlags;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 | 30;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3OsOpen($6,0,$7,$8,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _pager_write_changecounter($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $change_counter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $change_counter = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 112|0);
 $5 = (_sqlite3Get4byte($4)|0);
 $6 = (($5) + 1)|0;
 HEAP32[$change_counter>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = HEAP32[$change_counter>>2]|0;
 _sqlite3Put4byte($10,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 92|0);
 $16 = HEAP32[$change_counter>>2]|0;
 _sqlite3Put4byte($15,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 96|0);
 _sqlite3Put4byte($20,3015001);
 STACKTOP = sp;return;
}
function _sqlite3PagerExclusiveLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = (_sqlite3PagerUseWal($6)|0);
  $8 = (0)==($7|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = (_pager_wait_on_lock($9,4)|0);
   HEAP32[$rc>>2] = $10;
  }
 }
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _writeJournalHdr($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ii = 0, $nHeader = 0, $nWrite = 0, $rc = 0;
 var $zHeader = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $rc = sp + 16|0;
 $zHeader = sp + 12|0;
 $nHeader = sp + 8|0;
 $nWrite = sp + 4|0;
 $ii = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$zHeader>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 160|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$nHeader>>2] = $6;
 $7 = HEAP32[$nHeader>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 156|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7>>>0)>($10>>>0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 156|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$nHeader>>2] = $14;
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $15 = HEAP32[$ii>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 104|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($15|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$ii>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = ((($21)) + 100|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*48)|0)|0);
  $25 = ((($24)) + 8|0);
  $26 = $25;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)==(0);
  $33 = ($31|0)==(0);
  $34 = $32 & $33;
  if ($34) {
   $35 = HEAP32[$0>>2]|0;
   $36 = ((($35)) + 80|0);
   $37 = $36;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($37) + 4)|0;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$ii>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = ((($44)) + 100|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (($46) + (($43*48)|0)|0);
   $48 = ((($47)) + 8|0);
   $49 = $48;
   $50 = $49;
   HEAP32[$50>>2] = $39;
   $51 = (($49) + 4)|0;
   $52 = $51;
   HEAP32[$52>>2] = $42;
  }
  $53 = HEAP32[$ii>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$ii>>2] = $54;
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = (_journalHdrOffset($55)|0);
 $57 = tempRet0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 80|0);
 $60 = $59;
 $61 = $60;
 HEAP32[$61>>2] = $56;
 $62 = (($60) + 4)|0;
 $63 = $62;
 HEAP32[$63>>2] = $57;
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 88|0);
 $66 = $65;
 $67 = $66;
 HEAP32[$67>>2] = $56;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = $57;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 7|0);
 $72 = HEAP8[$71>>0]|0;
 $73 = $72&255;
 $74 = ($73|0)!=(0);
 if ($74) {
  label = 11;
 } else {
  $75 = HEAP32[$0>>2]|0;
  $76 = ((($75)) + 5|0);
  $77 = HEAP8[$76>>0]|0;
  $78 = $77&255;
  $79 = ($78|0)==(4);
  if ($79) {
   label = 11;
  } else {
   $80 = HEAP32[$0>>2]|0;
   $81 = ((($80)) + 64|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = (_sqlite3OsDeviceCharacteristics($82)|0);
   $84 = $83 & 512;
   $85 = ($84|0)!=(0);
   if ($85) {
    label = 11;
   } else {
    $89 = HEAP32[$zHeader>>2]|0;
    dest=$89; stop=dest+12|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
   }
  }
 }
 if ((label|0) == 11) {
  $86 = HEAP32[$zHeader>>2]|0;
  ;HEAP8[$86>>0]=HEAP8[25901>>0]|0;HEAP8[$86+1>>0]=HEAP8[25901+1>>0]|0;HEAP8[$86+2>>0]=HEAP8[25901+2>>0]|0;HEAP8[$86+3>>0]=HEAP8[25901+3>>0]|0;HEAP8[$86+4>>0]=HEAP8[25901+4>>0]|0;HEAP8[$86+5>>0]=HEAP8[25901+5>>0]|0;HEAP8[$86+6>>0]=HEAP8[25901+6>>0]|0;HEAP8[$86+7>>0]=HEAP8[25901+7>>0]|0;
  $87 = HEAP32[$zHeader>>2]|0;
  $88 = ((($87)) + 8|0);
  _sqlite3Put4byte($88,-1);
 }
 $90 = HEAP32[$0>>2]|0;
 $91 = ((($90)) + 52|0);
 _sqlite3_randomness(4,$91);
 $92 = HEAP32[$zHeader>>2]|0;
 $93 = ((($92)) + 12|0);
 $94 = HEAP32[$0>>2]|0;
 $95 = ((($94)) + 52|0);
 $96 = HEAP32[$95>>2]|0;
 _sqlite3Put4byte($93,$96);
 $97 = HEAP32[$zHeader>>2]|0;
 $98 = ((($97)) + 16|0);
 $99 = HEAP32[$0>>2]|0;
 $100 = ((($99)) + 32|0);
 $101 = HEAP32[$100>>2]|0;
 _sqlite3Put4byte($98,$101);
 $102 = HEAP32[$zHeader>>2]|0;
 $103 = ((($102)) + 20|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = ((($104)) + 156|0);
 $106 = HEAP32[$105>>2]|0;
 _sqlite3Put4byte($103,$106);
 $107 = HEAP32[$zHeader>>2]|0;
 $108 = ((($107)) + 24|0);
 $109 = HEAP32[$0>>2]|0;
 $110 = ((($109)) + 160|0);
 $111 = HEAP32[$110>>2]|0;
 _sqlite3Put4byte($108,$111);
 $112 = HEAP32[$zHeader>>2]|0;
 $113 = ((($112)) + 28|0);
 $114 = HEAP32[$nHeader>>2]|0;
 $115 = (($114) - 28)|0;
 _memset(($113|0),0,($115|0))|0;
 HEAP32[$nWrite>>2] = 0;
 while(1) {
  $116 = HEAP32[$rc>>2]|0;
  $117 = ($116|0)==(0);
  if (!($117)) {
   label = 17;
   break;
  }
  $118 = HEAP32[$nWrite>>2]|0;
  $119 = HEAP32[$0>>2]|0;
  $120 = ((($119)) + 156|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($118>>>0)<($121>>>0);
  if (!($122)) {
   label = 17;
   break;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = ((($123)) + 68|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = HEAP32[$zHeader>>2]|0;
  $127 = HEAP32[$nHeader>>2]|0;
  $128 = HEAP32[$0>>2]|0;
  $129 = ((($128)) + 80|0);
  $130 = $129;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = (($130) + 4)|0;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  $136 = (_sqlite3OsWrite($125,$126,$127,$132,$135)|0);
  HEAP32[$rc>>2] = $136;
  $137 = HEAP32[$nHeader>>2]|0;
  $138 = HEAP32[$0>>2]|0;
  $139 = ((($138)) + 80|0);
  $140 = $139;
  $141 = $140;
  $142 = HEAP32[$141>>2]|0;
  $143 = (($140) + 4)|0;
  $144 = $143;
  $145 = HEAP32[$144>>2]|0;
  $146 = (_i64Add(($142|0),($145|0),($137|0),0)|0);
  $147 = tempRet0;
  $148 = $139;
  $149 = $148;
  HEAP32[$149>>2] = $146;
  $150 = (($148) + 4)|0;
  $151 = $150;
  HEAP32[$151>>2] = $147;
  $152 = HEAP32[$nHeader>>2]|0;
  $153 = HEAP32[$nWrite>>2]|0;
  $154 = (($153) + ($152))|0;
  HEAP32[$nWrite>>2] = $154;
 }
 if ((label|0) == 17) {
  $155 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 return (0)|0;
}
function _sqlite3PcacheClearSyncFlags($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 24|0);
  $7 = HEAP16[$6>>1]|0;
  $8 = $7&65535;
  $9 = $8 & -9;
  $10 = $9&65535;
  HEAP16[$6>>1] = $10;
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 32|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _pager_wait_on_lock($pPager,$locktype) {
 $pPager = $pPager|0;
 $locktype = $locktype|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $locktype;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = (_pagerLockDb($2,$3)|0);
  HEAP32[$rc>>2] = $4;
  $5 = HEAP32[$rc>>2]|0;
  $6 = ($5|0)==(5);
  if (!($6)) {
   label = 4;
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 184|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 188|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (FUNCTION_TABLE_ii[$9 & 255]($12)|0);
  $14 = ($13|0)!=(0);
  if (!($14)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _pagerLockDb($pPager,$eLock) {
 $pPager = $pPager|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eLock;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 18|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($5|0)<($6|0);
 if (!($7)) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 18|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)==(5);
  if (!($12)) {
   $37 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 14|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(0);
 if ($17) {
  $23 = 0;
 } else {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 64|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = (_sqlite3OsLock($20,$21)|0);
  $23 = $22;
 }
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(0);
 if (!($25)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 18|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)!=(5);
 $31 = HEAP32[$1>>2]|0;
 $32 = ($31|0)==(4);
 $or$cond = $30 | $32;
 if (!($or$cond)) {
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = $33&255;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 18|0);
 HEAP8[$36>>0] = $34;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3WalFrames($pWal,$szPage,$pList,$nTruncate,$isCommit,$sync_flags) {
 $pWal = $pWal|0;
 $szPage = $szPage|0;
 $pList = $pList|0;
 $nTruncate = $nTruncate|0;
 $isCommit = $isCommit|0;
 $sync_flags = $sync_flags|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCksum = 0, $aWalHdr = 0, $bSync = 0, $iFirst = 0, $iFrame = 0;
 var $iOff = 0, $iOffset = 0, $iWrite = 0, $nDbSize = 0, $nExtra = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pData = 0, $pLast = 0, $pLive = 0, $rc = 0, $sectorSize = 0, $sz = 0, $szFrame = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 136|0;
 $1 = sp + 132|0;
 $2 = sp + 128|0;
 $3 = sp + 124|0;
 $4 = sp + 120|0;
 $5 = sp + 116|0;
 $6 = sp + 112|0;
 $rc = sp + 108|0;
 $iFrame = sp + 104|0;
 $p = sp + 100|0;
 $pLast = sp + 96|0;
 $nExtra = sp + 92|0;
 $szFrame = sp + 88|0;
 $iOffset = sp + 40|0;
 $w = sp + 16|0;
 $iFirst = sp + 84|0;
 $pLive = sp + 80|0;
 $aWalHdr = sp + 144|0;
 $aCksum = sp + 72|0;
 $nDbSize = sp + 64|0;
 $iWrite = sp + 60|0;
 $iOff = sp + 8|0;
 $pData = sp + 56|0;
 $bSync = sp + 52|0;
 $sectorSize = sp + 48|0;
 $sz = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $szPage;
 HEAP32[$3>>2] = $pList;
 HEAP32[$4>>2] = $nTruncate;
 HEAP32[$5>>2] = $isCommit;
 HEAP32[$6>>2] = $sync_flags;
 HEAP32[$pLast>>2] = 0;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$iFirst>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_walIndexHdr($7)|0);
 HEAP32[$pLive>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$pLive>>2]|0;
 $12 = (_memcmp($10,$11,48)|0);
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$pLive>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$iFirst>>2] = $17;
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = (_walRestartLog($18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = (0)!=($19|0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $21;
  $533 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($533|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 52|0);
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$iFrame>>2] = $25;
 $26 = HEAP32[$iFrame>>2]|0;
 $27 = ($26|0)==(0);
 if ($27) {
  $28 = HEAP8[968>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($29|0)==(0);
  $31 = $30&1;
  $32 = 931071618 | $31;
  _sqlite3Put4byte($aWalHdr,$32);
  $33 = ((($aWalHdr)) + 4|0);
  _sqlite3Put4byte($33,3007000);
  $34 = ((($aWalHdr)) + 8|0);
  $35 = HEAP32[$2>>2]|0;
  _sqlite3Put4byte($34,$35);
  $36 = ((($aWalHdr)) + 12|0);
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 112|0);
  $39 = HEAP32[$38>>2]|0;
  _sqlite3Put4byte($36,$39);
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 112|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)==(0);
  if ($43) {
   $44 = HEAP32[$1>>2]|0;
   $45 = ((($44)) + 52|0);
   $46 = ((($45)) + 32|0);
   _sqlite3_randomness(8,$46);
  }
  $47 = ((($aWalHdr)) + 16|0);
  $48 = HEAP32[$1>>2]|0;
  $49 = ((($48)) + 52|0);
  $50 = ((($49)) + 32|0);
  ;HEAP8[$47>>0]=HEAP8[$50>>0]|0;HEAP8[$47+1>>0]=HEAP8[$50+1>>0]|0;HEAP8[$47+2>>0]=HEAP8[$50+2>>0]|0;HEAP8[$47+3>>0]=HEAP8[$50+3>>0]|0;HEAP8[$47+4>>0]=HEAP8[$50+4>>0]|0;HEAP8[$47+5>>0]=HEAP8[$50+5>>0]|0;HEAP8[$47+6>>0]=HEAP8[$50+6>>0]|0;HEAP8[$47+7>>0]=HEAP8[$50+7>>0]|0;
  _walChecksumBytes(1,$aWalHdr,24,0,$aCksum);
  $51 = ((($aWalHdr)) + 24|0);
  $52 = HEAP32[$aCksum>>2]|0;
  _sqlite3Put4byte($51,$52);
  $53 = ((($aWalHdr)) + 28|0);
  $54 = ((($aCksum)) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  _sqlite3Put4byte($53,$55);
  $56 = HEAP32[$2>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 36|0);
  HEAP32[$58>>2] = $56;
  $59 = HEAP8[968>>0]|0;
  $60 = $59 << 24 >> 24;
  $61 = ($60|0)==(0);
  $62 = $61&1;
  $63 = $62&255;
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 52|0);
  $66 = ((($65)) + 13|0);
  HEAP8[$66>>0] = $63;
  $67 = HEAP32[$aCksum>>2]|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = ((($68)) + 52|0);
  $70 = ((($69)) + 24|0);
  HEAP32[$70>>2] = $67;
  $71 = ((($aCksum)) + 4|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 52|0);
  $75 = ((($74)) + 24|0);
  $76 = ((($75)) + 4|0);
  HEAP32[$76>>2] = $72;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 47|0);
  HEAP8[$78>>0] = 1;
  $79 = HEAP32[$1>>2]|0;
  $80 = ((($79)) + 8|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (_sqlite3OsWrite($81,$aWalHdr,32,0,0)|0);
  HEAP32[$rc>>2] = $82;
  $83 = HEAP32[$rc>>2]|0;
  $84 = ($83|0)!=(0);
  if ($84) {
   $85 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $85;
   $533 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($533|0);
  }
  $86 = HEAP32[$1>>2]|0;
  $87 = ((($86)) + 48|0);
  $88 = HEAP8[$87>>0]|0;
  $89 = $88&255;
  $90 = ($89|0)!=(0);
  $91 = HEAP32[$6>>2]|0;
  $92 = ($91|0)!=(0);
  $or$cond = $90 & $92;
  if ($or$cond) {
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 8|0);
   $95 = HEAP32[$94>>2]|0;
   $96 = HEAP32[$6>>2]|0;
   $97 = $96 & 19;
   $98 = (_sqlite3OsSync($95,$97)|0);
   HEAP32[$rc>>2] = $98;
   $99 = HEAP32[$rc>>2]|0;
   $100 = ($99|0)!=(0);
   if ($100) {
    $101 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $101;
    $533 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($533|0);
   }
  }
 }
 $102 = HEAP32[$1>>2]|0;
 HEAP32[$w>>2] = $102;
 $103 = HEAP32[$1>>2]|0;
 $104 = ((($103)) + 8|0);
 $105 = HEAP32[$104>>2]|0;
 $106 = ((($w)) + 4|0);
 HEAP32[$106>>2] = $105;
 $107 = ((($w)) + 8|0);
 $108 = $107;
 $109 = $108;
 HEAP32[$109>>2] = 0;
 $110 = (($108) + 4)|0;
 $111 = $110;
 HEAP32[$111>>2] = 0;
 $112 = HEAP32[$6>>2]|0;
 $113 = ((($w)) + 16|0);
 HEAP32[$113>>2] = $112;
 $114 = HEAP32[$2>>2]|0;
 $115 = ((($w)) + 20|0);
 HEAP32[$115>>2] = $114;
 $116 = HEAP32[$iFrame>>2]|0;
 $117 = (($116) + 1)|0;
 $118 = (($117) - 1)|0;
 $119 = HEAP32[$2>>2]|0;
 $120 = (($119) + 24)|0;
 $121 = ($120|0)<(0);
 $122 = $121 << 31 >> 31;
 $123 = (___muldi3(($118|0),0,($120|0),($122|0))|0);
 $124 = tempRet0;
 $125 = (_i64Add(32,0,($123|0),($124|0))|0);
 $126 = tempRet0;
 $127 = $iOffset;
 $128 = $127;
 HEAP32[$128>>2] = $125;
 $129 = (($127) + 4)|0;
 $130 = $129;
 HEAP32[$130>>2] = $126;
 $131 = HEAP32[$2>>2]|0;
 $132 = (($131) + 24)|0;
 HEAP32[$szFrame>>2] = $132;
 $133 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $133;
 while(1) {
  $134 = HEAP32[$p>>2]|0;
  $135 = ($134|0)!=(0|0);
  if (!($135)) {
   label = 30;
   break;
  }
  $136 = HEAP32[$iFirst>>2]|0;
  $137 = ($136|0)!=(0);
  if ($137) {
   $138 = HEAP32[$p>>2]|0;
   $139 = ((($138)) + 12|0);
   $140 = HEAP32[$139>>2]|0;
   $141 = ($140|0)!=(0|0);
   $142 = HEAP32[$5>>2]|0;
   $143 = ($142|0)==(0);
   $or$cond3 = $141 | $143;
   if ($or$cond3) {
    HEAP32[$iWrite>>2] = 0;
    $144 = HEAP32[$1>>2]|0;
    $145 = HEAP32[$p>>2]|0;
    $146 = ((($145)) + 20|0);
    $147 = HEAP32[$146>>2]|0;
    (_sqlite3WalFindFrame($144,$147,$iWrite)|0);
    $148 = HEAP32[$iWrite>>2]|0;
    $149 = HEAP32[$iFirst>>2]|0;
    $150 = ($148>>>0)>=($149>>>0);
    if ($150) {
     $151 = HEAP32[$iWrite>>2]|0;
     $152 = (($151) - 1)|0;
     $153 = HEAP32[$2>>2]|0;
     $154 = (($153) + 24)|0;
     $155 = ($154|0)<(0);
     $156 = $155 << 31 >> 31;
     $157 = (___muldi3(($152|0),0,($154|0),($156|0))|0);
     $158 = tempRet0;
     $159 = (_i64Add(32,0,($157|0),($158|0))|0);
     $160 = tempRet0;
     $161 = (_i64Add(($159|0),($160|0),24,0)|0);
     $162 = tempRet0;
     $163 = $iOff;
     $164 = $163;
     HEAP32[$164>>2] = $161;
     $165 = (($163) + 4)|0;
     $166 = $165;
     HEAP32[$166>>2] = $162;
     $167 = HEAP32[$1>>2]|0;
     $168 = ((($167)) + 104|0);
     $169 = HEAP32[$168>>2]|0;
     $170 = ($169|0)==(0);
     if ($170) {
      label = 20;
     } else {
      $171 = HEAP32[$iWrite>>2]|0;
      $172 = HEAP32[$1>>2]|0;
      $173 = ((($172)) + 104|0);
      $174 = HEAP32[$173>>2]|0;
      $175 = ($171>>>0)<($174>>>0);
      if ($175) {
       label = 20;
      }
     }
     if ((label|0) == 20) {
      label = 0;
      $176 = HEAP32[$iWrite>>2]|0;
      $177 = HEAP32[$1>>2]|0;
      $178 = ((($177)) + 104|0);
      HEAP32[$178>>2] = $176;
     }
     $179 = HEAP32[$p>>2]|0;
     $180 = ((($179)) + 4|0);
     $181 = HEAP32[$180>>2]|0;
     HEAP32[$pData>>2] = $181;
     $182 = HEAP32[$1>>2]|0;
     $183 = ((($182)) + 8|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = HEAP32[$pData>>2]|0;
     $186 = HEAP32[$2>>2]|0;
     $187 = $iOff;
     $188 = $187;
     $189 = HEAP32[$188>>2]|0;
     $190 = (($187) + 4)|0;
     $191 = $190;
     $192 = HEAP32[$191>>2]|0;
     $193 = (_sqlite3OsWrite($184,$185,$186,$189,$192)|0);
     HEAP32[$rc>>2] = $193;
     $194 = HEAP32[$rc>>2]|0;
     $195 = ($194|0)!=(0);
     if ($195) {
      label = 22;
      break;
     }
     $197 = HEAP32[$p>>2]|0;
     $198 = ((($197)) + 24|0);
     $199 = HEAP16[$198>>1]|0;
     $200 = $199&65535;
     $201 = $200 & -65;
     $202 = $201&65535;
     HEAP16[$198>>1] = $202;
    } else {
     label = 24;
    }
   } else {
    label = 24;
   }
  } else {
   label = 24;
  }
  if ((label|0) == 24) {
   label = 0;
   $203 = HEAP32[$iFrame>>2]|0;
   $204 = (($203) + 1)|0;
   HEAP32[$iFrame>>2] = $204;
   $205 = HEAP32[$5>>2]|0;
   $206 = ($205|0)!=(0);
   if ($206) {
    $207 = HEAP32[$p>>2]|0;
    $208 = ((($207)) + 12|0);
    $209 = HEAP32[$208>>2]|0;
    $210 = ($209|0)==(0|0);
    $211 = HEAP32[$4>>2]|0;
    $$ = $210 ? $211 : 0;
    $212 = $$;
   } else {
    $212 = 0;
   }
   HEAP32[$nDbSize>>2] = $212;
   $213 = HEAP32[$p>>2]|0;
   $214 = HEAP32[$nDbSize>>2]|0;
   $215 = $iOffset;
   $216 = $215;
   $217 = HEAP32[$216>>2]|0;
   $218 = (($215) + 4)|0;
   $219 = $218;
   $220 = HEAP32[$219>>2]|0;
   $221 = (_walWriteOneFrame($w,$213,$214,$217,$220)|0);
   HEAP32[$rc>>2] = $221;
   $222 = HEAP32[$rc>>2]|0;
   $223 = ($222|0)!=(0);
   if ($223) {
    label = 27;
    break;
   }
   $225 = HEAP32[$p>>2]|0;
   HEAP32[$pLast>>2] = $225;
   $226 = HEAP32[$szFrame>>2]|0;
   $227 = ($226|0)<(0);
   $228 = $227 << 31 >> 31;
   $229 = $iOffset;
   $230 = $229;
   $231 = HEAP32[$230>>2]|0;
   $232 = (($229) + 4)|0;
   $233 = $232;
   $234 = HEAP32[$233>>2]|0;
   $235 = (_i64Add(($231|0),($234|0),($226|0),($228|0))|0);
   $236 = tempRet0;
   $237 = $iOffset;
   $238 = $237;
   HEAP32[$238>>2] = $235;
   $239 = (($237) + 4)|0;
   $240 = $239;
   HEAP32[$240>>2] = $236;
   $241 = HEAP32[$p>>2]|0;
   $242 = ((($241)) + 24|0);
   $243 = HEAP16[$242>>1]|0;
   $244 = $243&65535;
   $245 = $244 | 64;
   $246 = $245&65535;
   HEAP16[$242>>1] = $246;
  }
  $247 = HEAP32[$p>>2]|0;
  $248 = ((($247)) + 12|0);
  $249 = HEAP32[$248>>2]|0;
  HEAP32[$p>>2] = $249;
 }
 if ((label|0) == 22) {
  $196 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $196;
  $533 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($533|0);
 }
 else if ((label|0) == 27) {
  $224 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $224;
  $533 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($533|0);
 }
 else if ((label|0) == 30) {
  $250 = HEAP32[$5>>2]|0;
  $251 = ($250|0)!=(0);
  if ($251) {
   $252 = HEAP32[$1>>2]|0;
   $253 = ((($252)) + 104|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = ($254|0)!=(0);
   if ($255) {
    $256 = HEAP32[$1>>2]|0;
    $257 = HEAP32[$iFrame>>2]|0;
    $258 = (_walRewriteChecksums($256,$257)|0);
    HEAP32[$rc>>2] = $258;
    $259 = HEAP32[$rc>>2]|0;
    $260 = ($259|0)!=(0);
    if ($260) {
     $261 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $261;
     $533 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($533|0);
    }
   }
  }
  $262 = HEAP32[$5>>2]|0;
  $263 = ($262|0)!=(0);
  if ($263) {
   $264 = HEAP32[$6>>2]|0;
   $265 = $264 & 32;
   $266 = ($265|0)!=(0);
   if ($266) {
    HEAP32[$bSync>>2] = 1;
    $267 = HEAP32[$1>>2]|0;
    $268 = ((($267)) + 49|0);
    $269 = HEAP8[$268>>0]|0;
    $270 = ($269<<24>>24)!=(0);
    L52: do {
     if ($270) {
      $271 = HEAP32[$1>>2]|0;
      $272 = ((($271)) + 8|0);
      $273 = HEAP32[$272>>2]|0;
      $274 = (_sqlite3SectorSize($273)|0);
      HEAP32[$sectorSize>>2] = $274;
      $275 = $iOffset;
      $276 = $275;
      $277 = HEAP32[$276>>2]|0;
      $278 = (($275) + 4)|0;
      $279 = $278;
      $280 = HEAP32[$279>>2]|0;
      $281 = HEAP32[$sectorSize>>2]|0;
      $282 = ($281|0)<(0);
      $283 = $282 << 31 >> 31;
      $284 = (_i64Add(($277|0),($280|0),($281|0),($283|0))|0);
      $285 = tempRet0;
      $286 = (_i64Subtract(($284|0),($285|0),1,0)|0);
      $287 = tempRet0;
      $288 = HEAP32[$sectorSize>>2]|0;
      $289 = ($288|0)<(0);
      $290 = $289 << 31 >> 31;
      $291 = (___divdi3(($286|0),($287|0),($288|0),($290|0))|0);
      $292 = tempRet0;
      $293 = HEAP32[$sectorSize>>2]|0;
      $294 = ($293|0)<(0);
      $295 = $294 << 31 >> 31;
      $296 = (___muldi3(($291|0),($292|0),($293|0),($295|0))|0);
      $297 = tempRet0;
      $298 = ((($w)) + 8|0);
      $299 = $298;
      $300 = $299;
      HEAP32[$300>>2] = $296;
      $301 = (($299) + 4)|0;
      $302 = $301;
      HEAP32[$302>>2] = $297;
      $303 = ((($w)) + 8|0);
      $304 = $303;
      $305 = $304;
      $306 = HEAP32[$305>>2]|0;
      $307 = (($304) + 4)|0;
      $308 = $307;
      $309 = HEAP32[$308>>2]|0;
      $310 = $iOffset;
      $311 = $310;
      $312 = HEAP32[$311>>2]|0;
      $313 = (($310) + 4)|0;
      $314 = $313;
      $315 = HEAP32[$314>>2]|0;
      $316 = ($306|0)==($312|0);
      $317 = ($309|0)==($315|0);
      $318 = $316 & $317;
      $319 = $318&1;
      HEAP32[$bSync>>2] = $319;
      while(1) {
       $320 = $iOffset;
       $321 = $320;
       $322 = HEAP32[$321>>2]|0;
       $323 = (($320) + 4)|0;
       $324 = $323;
       $325 = HEAP32[$324>>2]|0;
       $326 = ((($w)) + 8|0);
       $327 = $326;
       $328 = $327;
       $329 = HEAP32[$328>>2]|0;
       $330 = (($327) + 4)|0;
       $331 = $330;
       $332 = HEAP32[$331>>2]|0;
       $333 = ($325|0)<($332|0);
       $334 = ($322>>>0)<($329>>>0);
       $335 = ($325|0)==($332|0);
       $336 = $335 & $334;
       $337 = $333 | $336;
       if (!($337)) {
        break L52;
       }
       $338 = HEAP32[$pLast>>2]|0;
       $339 = HEAP32[$4>>2]|0;
       $340 = $iOffset;
       $341 = $340;
       $342 = HEAP32[$341>>2]|0;
       $343 = (($340) + 4)|0;
       $344 = $343;
       $345 = HEAP32[$344>>2]|0;
       $346 = (_walWriteOneFrame($w,$338,$339,$342,$345)|0);
       HEAP32[$rc>>2] = $346;
       $347 = HEAP32[$rc>>2]|0;
       $348 = ($347|0)!=(0);
       if ($348) {
        break;
       }
       $350 = HEAP32[$szFrame>>2]|0;
       $351 = ($350|0)<(0);
       $352 = $351 << 31 >> 31;
       $353 = $iOffset;
       $354 = $353;
       $355 = HEAP32[$354>>2]|0;
       $356 = (($353) + 4)|0;
       $357 = $356;
       $358 = HEAP32[$357>>2]|0;
       $359 = (_i64Add(($355|0),($358|0),($350|0),($352|0))|0);
       $360 = tempRet0;
       $361 = $iOffset;
       $362 = $361;
       HEAP32[$362>>2] = $359;
       $363 = (($361) + 4)|0;
       $364 = $363;
       HEAP32[$364>>2] = $360;
       $365 = HEAP32[$nExtra>>2]|0;
       $366 = (($365) + 1)|0;
       HEAP32[$nExtra>>2] = $366;
      }
      $349 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $349;
      $533 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($533|0);
     }
    } while(0);
    $367 = HEAP32[$bSync>>2]|0;
    $368 = ($367|0)!=(0);
    if ($368) {
     $369 = ((($w)) + 4|0);
     $370 = HEAP32[$369>>2]|0;
     $371 = HEAP32[$6>>2]|0;
     $372 = $371 & 19;
     $373 = (_sqlite3OsSync($370,$372)|0);
     HEAP32[$rc>>2] = $373;
    }
   }
  }
  $374 = HEAP32[$5>>2]|0;
  $375 = ($374|0)!=(0);
  if ($375) {
   $376 = HEAP32[$1>>2]|0;
   $377 = ((($376)) + 47|0);
   $378 = HEAP8[$377>>0]|0;
   $379 = $378&255;
   $380 = ($379|0)!=(0);
   if ($380) {
    $381 = HEAP32[$1>>2]|0;
    $382 = ((($381)) + 16|0);
    $383 = $382;
    $384 = $383;
    $385 = HEAP32[$384>>2]|0;
    $386 = (($383) + 4)|0;
    $387 = $386;
    $388 = HEAP32[$387>>2]|0;
    $389 = ($388|0)>(0);
    $390 = ($385>>>0)>=(0);
    $391 = ($388|0)==(0);
    $392 = $391 & $390;
    $393 = $389 | $392;
    if ($393) {
     $394 = HEAP32[$1>>2]|0;
     $395 = ((($394)) + 16|0);
     $396 = $395;
     $397 = $396;
     $398 = HEAP32[$397>>2]|0;
     $399 = (($396) + 4)|0;
     $400 = $399;
     $401 = HEAP32[$400>>2]|0;
     $402 = $sz;
     $403 = $402;
     HEAP32[$403>>2] = $398;
     $404 = (($402) + 4)|0;
     $405 = $404;
     HEAP32[$405>>2] = $401;
     $406 = HEAP32[$iFrame>>2]|0;
     $407 = HEAP32[$nExtra>>2]|0;
     $408 = (($406) + ($407))|0;
     $409 = (($408) + 1)|0;
     $410 = (($409) - 1)|0;
     $411 = HEAP32[$2>>2]|0;
     $412 = (($411) + 24)|0;
     $413 = ($412|0)<(0);
     $414 = $413 << 31 >> 31;
     $415 = (___muldi3(($410|0),0,($412|0),($414|0))|0);
     $416 = tempRet0;
     $417 = (_i64Add(32,0,($415|0),($416|0))|0);
     $418 = tempRet0;
     $419 = HEAP32[$1>>2]|0;
     $420 = ((($419)) + 16|0);
     $421 = $420;
     $422 = $421;
     $423 = HEAP32[$422>>2]|0;
     $424 = (($421) + 4)|0;
     $425 = $424;
     $426 = HEAP32[$425>>2]|0;
     $427 = ($418|0)>($426|0);
     $428 = ($417>>>0)>($423>>>0);
     $429 = ($418|0)==($426|0);
     $430 = $429 & $428;
     $431 = $427 | $430;
     if ($431) {
      $432 = HEAP32[$iFrame>>2]|0;
      $433 = HEAP32[$nExtra>>2]|0;
      $434 = (($432) + ($433))|0;
      $435 = (($434) + 1)|0;
      $436 = (($435) - 1)|0;
      $437 = HEAP32[$2>>2]|0;
      $438 = (($437) + 24)|0;
      $439 = ($438|0)<(0);
      $440 = $439 << 31 >> 31;
      $441 = (___muldi3(($436|0),0,($438|0),($440|0))|0);
      $442 = tempRet0;
      $443 = (_i64Add(32,0,($441|0),($442|0))|0);
      $444 = tempRet0;
      $445 = $sz;
      $446 = $445;
      HEAP32[$446>>2] = $443;
      $447 = (($445) + 4)|0;
      $448 = $447;
      HEAP32[$448>>2] = $444;
     }
     $449 = HEAP32[$1>>2]|0;
     $450 = $sz;
     $451 = $450;
     $452 = HEAP32[$451>>2]|0;
     $453 = (($450) + 4)|0;
     $454 = $453;
     $455 = HEAP32[$454>>2]|0;
     _walLimitSize($449,$452,$455);
     $456 = HEAP32[$1>>2]|0;
     $457 = ((($456)) + 47|0);
     HEAP8[$457>>0] = 0;
    }
   }
  }
  $458 = HEAP32[$1>>2]|0;
  $459 = ((($458)) + 52|0);
  $460 = ((($459)) + 16|0);
  $461 = HEAP32[$460>>2]|0;
  HEAP32[$iFrame>>2] = $461;
  $462 = HEAP32[$3>>2]|0;
  HEAP32[$p>>2] = $462;
  while(1) {
   $463 = HEAP32[$p>>2]|0;
   $464 = ($463|0)!=(0|0);
   $465 = HEAP32[$rc>>2]|0;
   $466 = ($465|0)==(0);
   $467 = $464 ? $466 : 0;
   if (!($467)) {
    break;
   }
   $468 = HEAP32[$p>>2]|0;
   $469 = ((($468)) + 24|0);
   $470 = HEAP16[$469>>1]|0;
   $471 = $470&65535;
   $472 = $471 & 64;
   $473 = ($472|0)==(0);
   if (!($473)) {
    $474 = HEAP32[$iFrame>>2]|0;
    $475 = (($474) + 1)|0;
    HEAP32[$iFrame>>2] = $475;
    $476 = HEAP32[$1>>2]|0;
    $477 = HEAP32[$iFrame>>2]|0;
    $478 = HEAP32[$p>>2]|0;
    $479 = ((($478)) + 20|0);
    $480 = HEAP32[$479>>2]|0;
    $481 = (_walIndexAppend($476,$477,$480)|0);
    HEAP32[$rc>>2] = $481;
   }
   $482 = HEAP32[$p>>2]|0;
   $483 = ((($482)) + 12|0);
   $484 = HEAP32[$483>>2]|0;
   HEAP32[$p>>2] = $484;
  }
  while(1) {
   $485 = HEAP32[$rc>>2]|0;
   $486 = ($485|0)==(0);
   $487 = HEAP32[$nExtra>>2]|0;
   $488 = ($487|0)>(0);
   $489 = $486 ? $488 : 0;
   if (!($489)) {
    break;
   }
   $490 = HEAP32[$iFrame>>2]|0;
   $491 = (($490) + 1)|0;
   HEAP32[$iFrame>>2] = $491;
   $492 = HEAP32[$nExtra>>2]|0;
   $493 = (($492) + -1)|0;
   HEAP32[$nExtra>>2] = $493;
   $494 = HEAP32[$1>>2]|0;
   $495 = HEAP32[$iFrame>>2]|0;
   $496 = HEAP32[$pLast>>2]|0;
   $497 = ((($496)) + 20|0);
   $498 = HEAP32[$497>>2]|0;
   $499 = (_walIndexAppend($494,$495,$498)|0);
   HEAP32[$rc>>2] = $499;
  }
  $500 = HEAP32[$rc>>2]|0;
  $501 = ($500|0)==(0);
  if ($501) {
   $502 = HEAP32[$2>>2]|0;
   $503 = $502 & 65280;
   $504 = HEAP32[$2>>2]|0;
   $505 = $504 >> 16;
   $506 = $503 | $505;
   $507 = $506&65535;
   $508 = HEAP32[$1>>2]|0;
   $509 = ((($508)) + 52|0);
   $510 = ((($509)) + 14|0);
   HEAP16[$510>>1] = $507;
   $511 = HEAP32[$iFrame>>2]|0;
   $512 = HEAP32[$1>>2]|0;
   $513 = ((($512)) + 52|0);
   $514 = ((($513)) + 16|0);
   HEAP32[$514>>2] = $511;
   $515 = HEAP32[$5>>2]|0;
   $516 = ($515|0)!=(0);
   if ($516) {
    $517 = HEAP32[$1>>2]|0;
    $518 = ((($517)) + 52|0);
    $519 = ((($518)) + 8|0);
    $520 = HEAP32[$519>>2]|0;
    $521 = (($520) + 1)|0;
    HEAP32[$519>>2] = $521;
    $522 = HEAP32[$4>>2]|0;
    $523 = HEAP32[$1>>2]|0;
    $524 = ((($523)) + 52|0);
    $525 = ((($524)) + 20|0);
    HEAP32[$525>>2] = $522;
   }
   $526 = HEAP32[$5>>2]|0;
   $527 = ($526|0)!=(0);
   if ($527) {
    $528 = HEAP32[$1>>2]|0;
    _walIndexWriteHdr($528);
    $529 = HEAP32[$iFrame>>2]|0;
    $530 = HEAP32[$1>>2]|0;
    $531 = ((($530)) + 12|0);
    HEAP32[$531>>2] = $529;
   }
  }
  $532 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $532;
  $533 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($533|0);
 }
 return (0)|0;
}
function _walRestartLog($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $notUsed = 0, $pInfo = 0, $rc = 0, $salt1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $cnt = sp + 12|0;
 $pInfo = sp + 8|0;
 $salt1 = sp + 4|0;
 $notUsed = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (_walCkptInfo($7)|0);
  HEAP32[$pInfo>>2] = $8;
  $9 = HEAP32[$pInfo>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($10>>>0)>(0);
  do {
   if ($11) {
    _sqlite3_randomness(4,$salt1);
    $12 = HEAP32[$1>>2]|0;
    $13 = (_walLockExclusive($12,4,4)|0);
    HEAP32[$rc>>2] = $13;
    $14 = HEAP32[$rc>>2]|0;
    $15 = ($14|0)==(0);
    if ($15) {
     $16 = HEAP32[$1>>2]|0;
     $17 = HEAP32[$salt1>>2]|0;
     _walRestartHdr($16,$17);
     $18 = HEAP32[$1>>2]|0;
     _walUnlockExclusive($18,4,4);
     break;
    }
    $19 = HEAP32[$rc>>2]|0;
    $20 = ($19|0)!=(5);
    if ($20) {
     $21 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $21;
     $32 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($32|0);
    }
   }
  } while(0);
  $22 = HEAP32[$1>>2]|0;
  _walUnlockShared($22,3);
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 40|0);
  HEAP16[$24>>1] = -1;
  HEAP32[$cnt>>2] = 0;
  while(1) {
   $25 = HEAP32[$1>>2]|0;
   $26 = HEAP32[$cnt>>2]|0;
   $27 = (($26) + 1)|0;
   HEAP32[$cnt>>2] = $27;
   $28 = (_walTryBeginRead($25,$notUsed,1,$27)|0);
   HEAP32[$rc>>2] = $28;
   $29 = HEAP32[$rc>>2]|0;
   $30 = ($29|0)==(-1);
   if (!($30)) {
    break;
   }
  }
 }
 $31 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _walWriteOneFrame($p,$pPage,$nTruncate,$0,$1) {
 $p = $p|0;
 $pPage = $pPage|0;
 $nTruncate = $nTruncate|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aFrame = 0, $pData = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $rc = sp + 12|0;
 $pData = sp + 8|0;
 $aFrame = sp + 32|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pPage;
 HEAP32[$5>>2] = $nTruncate;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$4>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pData>>2] = $13;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$5>>2]|0;
 $20 = HEAP32[$pData>>2]|0;
 _walEncodeFrame($15,$18,$19,$20,$aFrame);
 $21 = HEAP32[$3>>2]|0;
 $22 = $6;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($22) + 4)|0;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (_walWriteToLog($21,$aFrame,24,$24,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)!=(0);
 if ($30) {
  $31 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $31;
  $47 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($47|0);
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = HEAP32[$pData>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = ((($34)) + 20|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $6;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Add(($39|0),($42|0),24,0)|0);
  $44 = tempRet0;
  $45 = (_walWriteToLog($32,$33,$36,$43,$44)|0);
  HEAP32[$rc>>2] = $45;
  $46 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $46;
  $47 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 return (0)|0;
}
function _walRewriteChecksums($pWal,$iLast) {
 $pWal = $pWal|0;
 $iLast = $iLast|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBuf = 0, $aFrame = 0, $iCksumOff = 0, $iOff = 0, $iPgno = 0, $iRead = 0, $nDbSize = 0, $rc = 0, $szPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $szPage = sp + 36|0;
 $rc = sp + 32|0;
 $aBuf = sp + 28|0;
 $aFrame = sp + 56|0;
 $iRead = sp + 24|0;
 $iCksumOff = sp + 8|0;
 $iOff = sp;
 $iPgno = sp + 20|0;
 $nDbSize = sp + 16|0;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $iLast;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$szPage>>2] = $5;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$szPage>>2]|0;
 $7 = (($6) + 24)|0;
 $8 = (_sqlite3_malloc($7)|0);
 HEAP32[$aBuf>>2] = $8;
 $9 = HEAP32[$aBuf>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  HEAP32[$0>>2] = 7;
  $124 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($124|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 104|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(1);
 if ($14) {
  $15 = $iCksumOff;
  $16 = $15;
  HEAP32[$16>>2] = 24;
  $17 = (($15) + 4)|0;
  $18 = $17;
  HEAP32[$18>>2] = 0;
 } else {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 104|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) - 1)|0;
  $23 = (($22) - 1)|0;
  $24 = HEAP32[$szPage>>2]|0;
  $25 = (($24) + 24)|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (___muldi3(($23|0),0,($25|0),($27|0))|0);
  $29 = tempRet0;
  $30 = (_i64Add(32,0,($28|0),($29|0))|0);
  $31 = tempRet0;
  $32 = (_i64Add(($30|0),($31|0),16,0)|0);
  $33 = tempRet0;
  $34 = $iCksumOff;
  $35 = $34;
  HEAP32[$35>>2] = $32;
  $36 = (($34) + 4)|0;
  $37 = $36;
  HEAP32[$37>>2] = $33;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 8|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$aBuf>>2]|0;
 $42 = $iCksumOff;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (_sqlite3OsRead($40,$41,8,$44,$47)|0);
 HEAP32[$rc>>2] = $48;
 $49 = HEAP32[$aBuf>>2]|0;
 $50 = (_sqlite3Get4byte($49)|0);
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 52|0);
 $53 = ((($52)) + 24|0);
 HEAP32[$53>>2] = $50;
 $54 = HEAP32[$aBuf>>2]|0;
 $55 = ((($54)) + 4|0);
 $56 = (_sqlite3Get4byte($55)|0);
 $57 = HEAP32[$1>>2]|0;
 $58 = ((($57)) + 52|0);
 $59 = ((($58)) + 24|0);
 $60 = ((($59)) + 4|0);
 HEAP32[$60>>2] = $56;
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 104|0);
 $63 = HEAP32[$62>>2]|0;
 HEAP32[$iRead>>2] = $63;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 104|0);
 HEAP32[$65>>2] = 0;
 while(1) {
  $66 = HEAP32[$rc>>2]|0;
  $67 = ($66|0)==(0);
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$iRead>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = ($68>>>0)<=($69>>>0);
  if (!($70)) {
   break;
  }
  $71 = HEAP32[$iRead>>2]|0;
  $72 = (($71) - 1)|0;
  $73 = HEAP32[$szPage>>2]|0;
  $74 = (($73) + 24)|0;
  $75 = ($74|0)<(0);
  $76 = $75 << 31 >> 31;
  $77 = (___muldi3(($72|0),0,($74|0),($76|0))|0);
  $78 = tempRet0;
  $79 = (_i64Add(32,0,($77|0),($78|0))|0);
  $80 = tempRet0;
  $81 = $iOff;
  $82 = $81;
  HEAP32[$82>>2] = $79;
  $83 = (($81) + 4)|0;
  $84 = $83;
  HEAP32[$84>>2] = $80;
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = HEAP32[$aBuf>>2]|0;
  $89 = HEAP32[$szPage>>2]|0;
  $90 = (($89) + 24)|0;
  $91 = $iOff;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = (($91) + 4)|0;
  $95 = $94;
  $96 = HEAP32[$95>>2]|0;
  $97 = (_sqlite3OsRead($87,$88,$90,$93,$96)|0);
  HEAP32[$rc>>2] = $97;
  $98 = HEAP32[$rc>>2]|0;
  $99 = ($98|0)==(0);
  if ($99) {
   $100 = HEAP32[$aBuf>>2]|0;
   $101 = (_sqlite3Get4byte($100)|0);
   HEAP32[$iPgno>>2] = $101;
   $102 = HEAP32[$aBuf>>2]|0;
   $103 = ((($102)) + 4|0);
   $104 = (_sqlite3Get4byte($103)|0);
   HEAP32[$nDbSize>>2] = $104;
   $105 = HEAP32[$1>>2]|0;
   $106 = HEAP32[$iPgno>>2]|0;
   $107 = HEAP32[$nDbSize>>2]|0;
   $108 = HEAP32[$aBuf>>2]|0;
   $109 = ((($108)) + 24|0);
   _walEncodeFrame($105,$106,$107,$109,$aFrame);
   $110 = HEAP32[$1>>2]|0;
   $111 = ((($110)) + 8|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = $iOff;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = (($113) + 4)|0;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (_sqlite3OsWrite($112,$aFrame,24,$115,$118)|0);
   HEAP32[$rc>>2] = $119;
  }
  $120 = HEAP32[$iRead>>2]|0;
  $121 = (($120) + 1)|0;
  HEAP32[$iRead>>2] = $121;
 }
 $122 = HEAP32[$aBuf>>2]|0;
 _sqlite3_free($122);
 $123 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $123;
 $124 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($124|0);
}
function _walEncodeFrame($pWal,$iPage,$nTruncate,$aData,$aFrame) {
 $pWal = $pWal|0;
 $iPage = $iPage|0;
 $nTruncate = $nTruncate|0;
 $aData = $aData|0;
 $aFrame = $aFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aCksum = 0, $nativeCksum = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $nativeCksum = sp + 4|0;
 $aCksum = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $iPage;
 HEAP32[$2>>2] = $nTruncate;
 HEAP32[$3>>2] = $aData;
 HEAP32[$4>>2] = $aFrame;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = ((($6)) + 24|0);
 HEAP32[$aCksum>>2] = $7;
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 _sqlite3Put4byte($8,$9);
 $10 = HEAP32[$4>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$2>>2]|0;
 _sqlite3Put4byte($11,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 104|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0);
 $17 = HEAP32[$4>>2]|0;
 $18 = ((($17)) + 8|0);
 if ($16) {
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 52|0);
  $21 = ((($20)) + 32|0);
  ;HEAP8[$18>>0]=HEAP8[$21>>0]|0;HEAP8[$18+1>>0]=HEAP8[$21+1>>0]|0;HEAP8[$18+2>>0]=HEAP8[$21+2>>0]|0;HEAP8[$18+3>>0]=HEAP8[$21+3>>0]|0;HEAP8[$18+4>>0]=HEAP8[$21+4>>0]|0;HEAP8[$18+5>>0]=HEAP8[$21+5>>0]|0;HEAP8[$18+6>>0]=HEAP8[$21+6>>0]|0;HEAP8[$18+7>>0]=HEAP8[$21+7>>0]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 52|0);
  $24 = ((($23)) + 13|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = HEAP8[968>>0]|0;
  $28 = $27 << 24 >> 24;
  $29 = ($28|0)==(0);
  $30 = $29&1;
  $31 = ($26|0)==($30|0);
  $32 = $31&1;
  HEAP32[$nativeCksum>>2] = $32;
  $33 = HEAP32[$nativeCksum>>2]|0;
  $34 = HEAP32[$4>>2]|0;
  $35 = HEAP32[$aCksum>>2]|0;
  $36 = HEAP32[$aCksum>>2]|0;
  _walChecksumBytes($33,$34,8,$35,$36);
  $37 = HEAP32[$nativeCksum>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 36|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$aCksum>>2]|0;
  $43 = HEAP32[$aCksum>>2]|0;
  _walChecksumBytes($37,$38,$41,$42,$43);
  $44 = HEAP32[$4>>2]|0;
  $45 = ((($44)) + 16|0);
  $46 = HEAP32[$aCksum>>2]|0;
  $47 = HEAP32[$46>>2]|0;
  _sqlite3Put4byte($45,$47);
  $48 = HEAP32[$4>>2]|0;
  $49 = ((($48)) + 20|0);
  $50 = HEAP32[$aCksum>>2]|0;
  $51 = ((($50)) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  _sqlite3Put4byte($49,$52);
  STACKTOP = sp;return;
 } else {
  dest=$18; stop=dest+16|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  STACKTOP = sp;return;
 }
}
function _walWriteToLog($p,$pContent,$iAmt,$0,$1) {
 $p = $p|0;
 $pContent = $pContent|0;
 $iAmt = $iAmt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iFirstAmt = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $rc = sp + 12|0;
 $iFirstAmt = sp + 8|0;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $pContent;
 HEAP32[$5>>2] = $iAmt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($16|0)<($24|0);
 $26 = ($13>>>0)<($21>>>0);
 $27 = ($16|0)==($24|0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  $30 = $6;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$5>>2]|0;
  $37 = ($36|0)<(0);
  $38 = $37 << 31 >> 31;
  $39 = (_i64Add(($32|0),($35|0),($36|0),($38|0))|0);
  $40 = tempRet0;
  $41 = HEAP32[$3>>2]|0;
  $42 = ((($41)) + 8|0);
  $43 = $42;
  $44 = $43;
  $45 = HEAP32[$44>>2]|0;
  $46 = (($43) + 4)|0;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = ($40|0)>($48|0);
  $50 = ($39>>>0)>=($45>>>0);
  $51 = ($40|0)==($48|0);
  $52 = $51 & $50;
  $53 = $49 | $52;
  if ($53) {
   $54 = HEAP32[$3>>2]|0;
   $55 = ((($54)) + 8|0);
   $56 = $55;
   $57 = $56;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($56) + 4)|0;
   $60 = $59;
   $61 = HEAP32[$60>>2]|0;
   $62 = $6;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Subtract(($58|0),($61|0),($64|0),($67|0))|0);
   $69 = tempRet0;
   HEAP32[$iFirstAmt>>2] = $68;
   $70 = HEAP32[$3>>2]|0;
   $71 = ((($70)) + 4|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = HEAP32[$4>>2]|0;
   $74 = HEAP32[$iFirstAmt>>2]|0;
   $75 = $6;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (($75) + 4)|0;
   $79 = $78;
   $80 = HEAP32[$79>>2]|0;
   $81 = (_sqlite3OsWrite($72,$73,$74,$77,$80)|0);
   HEAP32[$rc>>2] = $81;
   $82 = HEAP32[$rc>>2]|0;
   $83 = ($82|0)!=(0);
   if ($83) {
    $84 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $84;
    $132 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($132|0);
   }
   $85 = HEAP32[$iFirstAmt>>2]|0;
   $86 = ($85|0)<(0);
   $87 = $86 << 31 >> 31;
   $88 = $6;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (($88) + 4)|0;
   $92 = $91;
   $93 = HEAP32[$92>>2]|0;
   $94 = (_i64Add(($90|0),($93|0),($85|0),($87|0))|0);
   $95 = tempRet0;
   $96 = $6;
   $97 = $96;
   HEAP32[$97>>2] = $94;
   $98 = (($96) + 4)|0;
   $99 = $98;
   HEAP32[$99>>2] = $95;
   $100 = HEAP32[$iFirstAmt>>2]|0;
   $101 = HEAP32[$5>>2]|0;
   $102 = (($101) - ($100))|0;
   HEAP32[$5>>2] = $102;
   $103 = HEAP32[$iFirstAmt>>2]|0;
   $104 = HEAP32[$4>>2]|0;
   $105 = (($104) + ($103)|0);
   HEAP32[$4>>2] = $105;
   $106 = HEAP32[$3>>2]|0;
   $107 = ((($106)) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = HEAP32[$3>>2]|0;
   $110 = ((($109)) + 16|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = $111 & 19;
   $113 = (_sqlite3OsSync($108,$112)|0);
   HEAP32[$rc>>2] = $113;
   $114 = HEAP32[$5>>2]|0;
   $115 = ($114|0)==(0);
   $116 = HEAP32[$rc>>2]|0;
   $117 = ($116|0)!=(0);
   $or$cond = $115 | $117;
   if ($or$cond) {
    $118 = HEAP32[$rc>>2]|0;
    HEAP32[$2>>2] = $118;
    $132 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($132|0);
   }
  }
 }
 $119 = HEAP32[$3>>2]|0;
 $120 = ((($119)) + 4|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$4>>2]|0;
 $123 = HEAP32[$5>>2]|0;
 $124 = $6;
 $125 = $124;
 $126 = HEAP32[$125>>2]|0;
 $127 = (($124) + 4)|0;
 $128 = $127;
 $129 = HEAP32[$128>>2]|0;
 $130 = (_sqlite3OsWrite($121,$122,$123,$126,$129)|0);
 HEAP32[$rc>>2] = $130;
 $131 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $131;
 $132 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($132|0);
}
function _walTryBeginRead($pWal,$pChanged,$useWal,$cnt) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 $useWal = $useWal|0;
 $cnt = $cnt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $mxFrame = 0, $mxI = 0, $mxReadMark = 0, $nDelay = 0, $or$cond = 0, $pInfo = 0, $rc = 0, $thisMark = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $pInfo = sp + 28|0;
 $mxReadMark = sp + 24|0;
 $mxI = sp + 20|0;
 $i = sp + 16|0;
 $rc = sp + 12|0;
 $mxFrame = sp + 8|0;
 $nDelay = sp + 4|0;
 $thisMark = sp;
 HEAP32[$1>>2] = $pWal;
 HEAP32[$2>>2] = $pChanged;
 HEAP32[$3>>2] = $useWal;
 HEAP32[$4>>2] = $cnt;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(5);
 if ($6) {
  HEAP32[$nDelay>>2] = 1;
  $7 = HEAP32[$4>>2]|0;
  $8 = ($7|0)>(100);
  if ($8) {
   HEAP32[$0>>2] = 15;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
  $9 = HEAP32[$4>>2]|0;
  $10 = ($9|0)>=(10);
  if ($10) {
   $11 = HEAP32[$4>>2]|0;
   $12 = (($11) - 9)|0;
   $13 = HEAP32[$4>>2]|0;
   $14 = (($13) - 9)|0;
   $15 = Math_imul($12, $14)|0;
   $16 = ($15*39)|0;
   HEAP32[$nDelay>>2] = $16;
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$nDelay>>2]|0;
  (_sqlite3OsSleep($18,$19)|0);
 }
 $20 = HEAP32[$3>>2]|0;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (_walIndexReadHdr($22,$23)|0);
  HEAP32[$rc>>2] = $24;
  $25 = HEAP32[$rc>>2]|0;
  $26 = ($25|0)==(5);
  do {
   if ($26) {
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 32|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = (_walLockShared($32,2)|0);
    HEAP32[$rc>>2] = $33;
    $34 = (0)==($33|0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     _walUnlockShared($35,2);
     HEAP32[$rc>>2] = -1;
     break;
    }
    $36 = HEAP32[$rc>>2]|0;
    $37 = ($36|0)==(5);
    if ($37) {
     HEAP32[$rc>>2] = 261;
    }
   }
  } while(0);
  $38 = HEAP32[$rc>>2]|0;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $40;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = (_walCkptInfo($41)|0);
 HEAP32[$pInfo>>2] = $42;
 $43 = HEAP32[$3>>2]|0;
 $44 = ($43|0)!=(0);
 do {
  if (!($44)) {
   $45 = HEAP32[$pInfo>>2]|0;
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 52|0);
   $49 = ((($48)) + 16|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($46|0)==($50|0);
   if ($51) {
    $52 = HEAP32[$1>>2]|0;
    $53 = (_walLockShared($52,3)|0);
    HEAP32[$rc>>2] = $53;
    $54 = HEAP32[$1>>2]|0;
    _walShmBarrier($54);
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)==(0);
    if (!($56)) {
     $65 = HEAP32[$rc>>2]|0;
     $66 = ($65|0)!=(5);
     if (!($66)) {
      break;
     }
     $67 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $67;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    }
    $57 = HEAP32[$1>>2]|0;
    $58 = (_walIndexHdr($57)|0);
    $59 = HEAP32[$1>>2]|0;
    $60 = ((($59)) + 52|0);
    $61 = (_memcmp($58,$60,48)|0);
    $62 = ($61|0)!=(0);
    $63 = HEAP32[$1>>2]|0;
    if ($62) {
     _walUnlockShared($63,3);
     HEAP32[$0>>2] = -1;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    } else {
     $64 = ((($63)) + 40|0);
     HEAP16[$64>>1] = 0;
     HEAP32[$0>>2] = 0;
     $164 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($164|0);
    }
   }
  }
 } while(0);
 HEAP32[$mxReadMark>>2] = 0;
 HEAP32[$mxI>>2] = 0;
 $68 = HEAP32[$1>>2]|0;
 $69 = ((($68)) + 52|0);
 $70 = ((($69)) + 16|0);
 $71 = HEAP32[$70>>2]|0;
 HEAP32[$mxFrame>>2] = $71;
 HEAP32[$i>>2] = 1;
 while(1) {
  $72 = HEAP32[$i>>2]|0;
  $73 = ($72|0)<(5);
  if (!($73)) {
   break;
  }
  $74 = HEAP32[$i>>2]|0;
  $75 = HEAP32[$pInfo>>2]|0;
  $76 = ((($75)) + 4|0);
  $77 = (($76) + ($74<<2)|0);
  $78 = HEAP32[$77>>2]|0;
  HEAP32[$thisMark>>2] = $78;
  $79 = HEAP32[$mxReadMark>>2]|0;
  $80 = HEAP32[$thisMark>>2]|0;
  $81 = ($79>>>0)<=($80>>>0);
  if ($81) {
   $82 = HEAP32[$thisMark>>2]|0;
   $83 = HEAP32[$mxFrame>>2]|0;
   $84 = ($82>>>0)<=($83>>>0);
   if ($84) {
    $85 = HEAP32[$thisMark>>2]|0;
    HEAP32[$mxReadMark>>2] = $85;
    $86 = HEAP32[$i>>2]|0;
    HEAP32[$mxI>>2] = $86;
   }
  }
  $87 = HEAP32[$i>>2]|0;
  $88 = (($87) + 1)|0;
  HEAP32[$i>>2] = $88;
 }
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 46|0);
 $91 = HEAP8[$90>>0]|0;
 $92 = $91&255;
 $93 = $92 & 2;
 $94 = ($93|0)==(0);
 L48: do {
  if ($94) {
   $95 = HEAP32[$mxReadMark>>2]|0;
   $96 = HEAP32[$mxFrame>>2]|0;
   $97 = ($95>>>0)<($96>>>0);
   $98 = HEAP32[$mxI>>2]|0;
   $99 = ($98|0)==(0);
   $or$cond = $97 | $99;
   if ($or$cond) {
    HEAP32[$i>>2] = 1;
    while(1) {
     $100 = HEAP32[$i>>2]|0;
     $101 = ($100|0)<(5);
     if (!($101)) {
      break L48;
     }
     $102 = HEAP32[$1>>2]|0;
     $103 = HEAP32[$i>>2]|0;
     $104 = (3 + ($103))|0;
     $105 = (_walLockExclusive($102,$104,1)|0);
     HEAP32[$rc>>2] = $105;
     $106 = HEAP32[$rc>>2]|0;
     $107 = ($106|0)==(0);
     if ($107) {
      label = 36;
      break;
     }
     $117 = HEAP32[$rc>>2]|0;
     $118 = ($117|0)!=(5);
     if ($118) {
      break;
     }
     $120 = HEAP32[$i>>2]|0;
     $121 = (($120) + 1)|0;
     HEAP32[$i>>2] = $121;
    }
    if ((label|0) == 36) {
     $108 = HEAP32[$mxFrame>>2]|0;
     $109 = HEAP32[$i>>2]|0;
     $110 = HEAP32[$pInfo>>2]|0;
     $111 = ((($110)) + 4|0);
     $112 = (($111) + ($109<<2)|0);
     HEAP32[$112>>2] = $108;
     HEAP32[$mxReadMark>>2] = $108;
     $113 = HEAP32[$i>>2]|0;
     HEAP32[$mxI>>2] = $113;
     $114 = HEAP32[$1>>2]|0;
     $115 = HEAP32[$i>>2]|0;
     $116 = (3 + ($115))|0;
     _walUnlockExclusive($114,$116,1);
     break;
    }
    $119 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $119;
    $164 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($164|0);
   }
  }
 } while(0);
 $122 = HEAP32[$mxI>>2]|0;
 $123 = ($122|0)==(0);
 if ($123) {
  $124 = HEAP32[$rc>>2]|0;
  $125 = ($124|0)==(5);
  $126 = $125 ? -1 : 520;
  HEAP32[$0>>2] = $126;
  $164 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($164|0);
 }
 $127 = HEAP32[$1>>2]|0;
 $128 = HEAP32[$mxI>>2]|0;
 $129 = (3 + ($128))|0;
 $130 = (_walLockShared($127,$129)|0);
 HEAP32[$rc>>2] = $130;
 $131 = HEAP32[$rc>>2]|0;
 $132 = ($131|0)!=(0);
 if ($132) {
  $133 = HEAP32[$rc>>2]|0;
  $134 = ($133|0)==(5);
  $135 = HEAP32[$rc>>2]|0;
  $136 = $134 ? -1 : $135;
  HEAP32[$0>>2] = $136;
  $164 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($164|0);
 }
 $137 = HEAP32[$pInfo>>2]|0;
 $138 = HEAP32[$137>>2]|0;
 $139 = (($138) + 1)|0;
 $140 = HEAP32[$1>>2]|0;
 $141 = ((($140)) + 100|0);
 HEAP32[$141>>2] = $139;
 $142 = HEAP32[$1>>2]|0;
 _walShmBarrier($142);
 $143 = HEAP32[$mxI>>2]|0;
 $144 = HEAP32[$pInfo>>2]|0;
 $145 = ((($144)) + 4|0);
 $146 = (($145) + ($143<<2)|0);
 $147 = HEAP32[$146>>2]|0;
 $148 = HEAP32[$mxReadMark>>2]|0;
 $149 = ($147|0)!=($148|0);
 if (!($149)) {
  $150 = HEAP32[$1>>2]|0;
  $151 = (_walIndexHdr($150)|0);
  $152 = HEAP32[$1>>2]|0;
  $153 = ((($152)) + 52|0);
  $154 = (_memcmp($151,$153,48)|0);
  $155 = ($154|0)!=(0);
  if (!($155)) {
   $159 = HEAP32[$mxI>>2]|0;
   $160 = $159&65535;
   $161 = HEAP32[$1>>2]|0;
   $162 = ((($161)) + 40|0);
   HEAP16[$162>>1] = $160;
   $163 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $163;
   $164 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($164|0);
  }
 }
 $156 = HEAP32[$1>>2]|0;
 $157 = HEAP32[$mxI>>2]|0;
 $158 = (3 + ($157))|0;
 _walUnlockShared($156,$158);
 HEAP32[$0>>2] = -1;
 $164 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($164|0);
}
function _subjRequiresPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pPager = 0, $pgno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pPager = sp + 12|0;
 $p = sp + 8|0;
 $pgno = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pgno>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$pPager>>2]|0;
  $10 = ((($9)) + 104|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   label = 7;
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pPager>>2]|0;
  $15 = ((($14)) + 100|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*48)|0)|0);
  HEAP32[$p>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$pgno>>2]|0;
  $22 = ($20>>>0)>=($21>>>0);
  if ($22) {
   $23 = HEAP32[$p>>2]|0;
   $24 = ((($23)) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$pgno>>2]|0;
   $27 = (_sqlite3BitvecTestNotNull($25,$26)|0);
   $28 = (0)==($27|0);
   if ($28) {
    label = 5;
    break;
   }
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 1;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 return (0)|0;
}
function _subjournalPage($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $offset = 0, $pData = 0, $pData2 = 0, $pPager = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $rc = sp + 20|0;
 $pPager = sp + 16|0;
 $pData = sp + 12|0;
 $offset = sp;
 $pData2 = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)!=(2);
 if ($8) {
  $9 = HEAP32[$pPager>>2]|0;
  $10 = (_openSubJournal($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)==(0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 4|0);
   $15 = HEAP32[$14>>2]|0;
   HEAP32[$pData>>2] = $15;
   $16 = HEAP32[$pPager>>2]|0;
   $17 = ((($16)) + 56|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$pPager>>2]|0;
   $20 = ((($19)) + 160|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (4 + ($21))|0;
   $23 = ($22|0)<(0);
   $24 = $23 << 31 >> 31;
   $25 = (___muldi3(($18|0),0,($22|0),($24|0))|0);
   $26 = tempRet0;
   $27 = $offset;
   $28 = $27;
   HEAP32[$28>>2] = $25;
   $29 = (($27) + 4)|0;
   $30 = $29;
   HEAP32[$30>>2] = $26;
   $31 = HEAP32[$pData>>2]|0;
   HEAP32[$pData2>>2] = $31;
   $32 = HEAP32[$pPager>>2]|0;
   $33 = ((($32)) + 72|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = $offset;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = (($35) + 4)|0;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$0>>2]|0;
   $42 = ((($41)) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_write32bits($34,$37,$40,$43)|0);
   HEAP32[$rc>>2] = $44;
   $45 = HEAP32[$rc>>2]|0;
   $46 = ($45|0)==(0);
   if ($46) {
    $47 = HEAP32[$pPager>>2]|0;
    $48 = ((($47)) + 72|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = HEAP32[$pData2>>2]|0;
    $51 = HEAP32[$pPager>>2]|0;
    $52 = ((($51)) + 160|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = $offset;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = (($54) + 4)|0;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = (_i64Add(($56|0),($59|0),4,0)|0);
    $61 = tempRet0;
    $62 = (_sqlite3OsWrite($49,$50,$53,$60,$61)|0);
    HEAP32[$rc>>2] = $62;
   }
  }
 }
 $63 = HEAP32[$rc>>2]|0;
 $64 = ($63|0)==(0);
 if (!($64)) {
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 $65 = HEAP32[$pPager>>2]|0;
 $66 = ((($65)) + 56|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 HEAP32[$66>>2] = $68;
 $69 = HEAP32[$pPager>>2]|0;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = (_addToSavepointBitvecs($69,$72)|0);
 HEAP32[$rc>>2] = $73;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _openSubJournal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flags = 0, $nStmtSpill = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $flags = sp + 4|0;
 $nStmtSpill = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $24 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 HEAP32[$flags>>2] = 8222;
 $6 = HEAP32[(44)>>2]|0;
 HEAP32[$nStmtSpill>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 5|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)==(4);
 if ($11) {
  label = 4;
 } else {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 22|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 4;
  }
 }
 if ((label|0) == 4) {
  HEAP32[$nStmtSpill>>2] = -1;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 72|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$nStmtSpill>>2]|0;
 $23 = (_sqlite3JournalOpen($18,0,$21,8222,$22)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _write32bits($fd,$0,$1,$val) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $val = $val|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ac = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 $ac = sp + 16|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $val;
 $9 = HEAP32[$4>>2]|0;
 _sqlite3Put4byte($ac,$9);
 $10 = HEAP32[$2>>2]|0;
 $11 = $3;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3OsWrite($10,$ac,4,$13,$16)|0);
 STACKTOP = sp;return ($17|0);
}
function _addToSavepointBitvecs($pPager,$pgno) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ii = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $ii = sp + 8|0;
 $rc = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ii>>2] = 0;
 while(1) {
  $2 = HEAP32[$ii>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 104|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$ii>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 100|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*48)|0)|0);
  HEAP32[$p>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($12>>>0)<=($15>>>0);
  if ($16) {
   $17 = HEAP32[$p>>2]|0;
   $18 = ((($17)) + 16|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = (_sqlite3BitvecSet($19,$20)|0);
   $22 = HEAP32[$rc>>2]|0;
   $23 = $22 | $21;
   HEAP32[$rc>>2] = $23;
  }
  $24 = HEAP32[$ii>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$ii>>2] = $25;
 }
 $26 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3JournalOpen($pVfs,$zName,$pJfd,$flags,$nSpill) {
 $pVfs = $pVfs|0;
 $zName = $zName|0;
 $pJfd = $pJfd|0;
 $flags = $flags|0;
 $nSpill = $nSpill|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zName;
 HEAP32[$3>>2] = $pJfd;
 HEAP32[$4>>2] = $flags;
 HEAP32[$5>>2] = $nSpill;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 dest=$7; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = HEAP32[$5>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$4>>2]|0;
  $14 = (_sqlite3OsOpen($10,$11,$12,$13,0)|0);
  HEAP32[$0>>2] = $14;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $15 = HEAP32[$5>>2]|0;
 $16 = ($15|0)>(0);
 if ($16) {
  $17 = HEAP32[$5>>2]|0;
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 4|0);
  HEAP32[$19>>2] = $17;
 } else {
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 4|0);
  HEAP32[$21>>2] = 1020;
 }
 $22 = HEAP32[$p>>2]|0;
 HEAP32[$22>>2] = 4028;
 $23 = HEAP32[$5>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 8|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$4>>2]|0;
 $27 = HEAP32[$p>>2]|0;
 $28 = ((($27)) + 56|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 64|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$p>>2]|0;
 $34 = ((($33)) + 60|0);
 HEAP32[$34>>2] = $32;
 HEAP32[$0>>2] = 0;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3PagerGetExtra($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3PagerPageRefcount($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3PcachePageRefcount($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _btreeInitPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cellOffset = 0, $data = 0, $hdr = 0, $i = 0, $iCellFirst = 0, $iCellLast = 0, $nFree = 0, $next = 0;
 var $pBt = 0, $pc = 0, $size = 0, $sz = 0, $top = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $pc = sp + 50|0;
 $hdr = sp + 52|0;
 $data = sp + 32|0;
 $pBt = sp + 28|0;
 $usableSize = sp + 24|0;
 $cellOffset = sp + 48|0;
 $nFree = sp + 20|0;
 $top = sp + 16|0;
 $iCellFirst = sp + 12|0;
 $iCellLast = sp + 8|0;
 $i = sp + 4|0;
 $sz = sp;
 $next = sp + 46|0;
 $size = sp + 44|0;
 HEAP32[$1>>2] = $pPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 do {
  if (!($4)) {
   $5 = HEAP32[$1>>2]|0;
   $6 = ((($5)) + 52|0);
   $7 = HEAP32[$6>>2]|0;
   HEAP32[$pBt>>2] = $7;
   $8 = HEAP32[$1>>2]|0;
   $9 = ((($8)) + 5|0);
   $10 = HEAP8[$9>>0]|0;
   HEAP8[$hdr>>0] = $10;
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 56|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$data>>2] = $13;
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP8[$hdr>>0]|0;
   $16 = $15&255;
   $17 = HEAP32[$data>>2]|0;
   $18 = (($17) + ($16)|0);
   $19 = HEAP8[$18>>0]|0;
   $20 = $19&255;
   $21 = (_decodeFlags($14,$20)|0);
   $22 = ($21|0)!=(0);
   if ($22) {
    $23 = (_sqlite3CorruptError(60006)|0);
    HEAP32[$0>>2] = $23;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   $24 = HEAP32[$pBt>>2]|0;
   $25 = ((($24)) + 32|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (($26) - 1)|0;
   $28 = $27&65535;
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 20|0);
   HEAP16[$30>>1] = $28;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 1|0);
   HEAP8[$32>>0] = 0;
   $33 = HEAP32[$pBt>>2]|0;
   $34 = ((($33)) + 36|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$usableSize>>2] = $35;
   $36 = HEAP8[$hdr>>0]|0;
   $37 = $36&255;
   $38 = (($37) + 8)|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 6|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = (($38) + ($42))|0;
   $44 = $43&65535;
   HEAP16[$cellOffset>>1] = $44;
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 14|0);
   HEAP16[$46>>1] = $44;
   $47 = HEAP32[$usableSize>>2]|0;
   $48 = HEAP32[$data>>2]|0;
   $49 = (($48) + ($47)|0);
   $50 = HEAP32[$1>>2]|0;
   $51 = ((($50)) + 60|0);
   HEAP32[$51>>2] = $49;
   $52 = HEAP16[$cellOffset>>1]|0;
   $53 = $52&65535;
   $54 = HEAP32[$data>>2]|0;
   $55 = (($54) + ($53)|0);
   $56 = HEAP32[$1>>2]|0;
   $57 = ((($56)) + 64|0);
   HEAP32[$57>>2] = $55;
   $58 = HEAP32[$1>>2]|0;
   $59 = ((($58)) + 6|0);
   $60 = HEAP8[$59>>0]|0;
   $61 = $60&255;
   $62 = HEAP32[$data>>2]|0;
   $63 = (($62) + ($61)|0);
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 68|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP8[$hdr>>0]|0;
   $67 = $66&255;
   $68 = (($67) + 5)|0;
   $69 = HEAP32[$data>>2]|0;
   $70 = (($69) + ($68)|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = $72 << 8;
   $74 = HEAP8[$hdr>>0]|0;
   $75 = $74&255;
   $76 = (($75) + 5)|0;
   $77 = HEAP32[$data>>2]|0;
   $78 = (($77) + ($76)|0);
   $79 = ((($78)) + 1|0);
   $80 = HEAP8[$79>>0]|0;
   $81 = $80&255;
   $82 = $73 | $81;
   $83 = (($82) - 1)|0;
   $84 = $83 & 65535;
   $85 = (($84) + 1)|0;
   HEAP32[$top>>2] = $85;
   $86 = HEAP8[$hdr>>0]|0;
   $87 = $86&255;
   $88 = (($87) + 3)|0;
   $89 = HEAP32[$data>>2]|0;
   $90 = (($89) + ($88)|0);
   $91 = HEAP8[$90>>0]|0;
   $92 = $91&255;
   $93 = $92 << 8;
   $94 = HEAP8[$hdr>>0]|0;
   $95 = $94&255;
   $96 = (($95) + 3)|0;
   $97 = HEAP32[$data>>2]|0;
   $98 = (($97) + ($96)|0);
   $99 = ((($98)) + 1|0);
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = $93 | $101;
   $103 = $102&65535;
   $104 = HEAP32[$1>>2]|0;
   $105 = ((($104)) + 18|0);
   HEAP16[$105>>1] = $103;
   $106 = HEAP32[$1>>2]|0;
   $107 = ((($106)) + 18|0);
   $108 = HEAP16[$107>>1]|0;
   $109 = $108&65535;
   $110 = HEAP32[$pBt>>2]|0;
   $111 = ((($110)) + 32|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = (($112) - 8)|0;
   $114 = (($113>>>0) / 6)&-1;
   $115 = ($109>>>0)>($114>>>0);
   if ($115) {
    $116 = (_sqlite3CorruptError(60024)|0);
    HEAP32[$0>>2] = $116;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   $117 = HEAP16[$cellOffset>>1]|0;
   $118 = $117&65535;
   $119 = HEAP32[$1>>2]|0;
   $120 = ((($119)) + 18|0);
   $121 = HEAP16[$120>>1]|0;
   $122 = $121&65535;
   $123 = $122<<1;
   $124 = (($118) + ($123))|0;
   HEAP32[$iCellFirst>>2] = $124;
   $125 = HEAP32[$usableSize>>2]|0;
   $126 = (($125) - 4)|0;
   HEAP32[$iCellLast>>2] = $126;
   $127 = HEAP32[$pBt>>2]|0;
   $128 = ((($127)) + 4|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = ((($129)) + 24|0);
   $131 = HEAP32[$130>>2]|0;
   $132 = $131 & 536870912;
   $133 = ($132|0)!=(0);
   do {
    if ($133) {
     $134 = HEAP32[$1>>2]|0;
     $135 = ((($134)) + 4|0);
     $136 = HEAP8[$135>>0]|0;
     $137 = ($136<<24>>24)!=(0);
     if (!($137)) {
      $138 = HEAP32[$iCellLast>>2]|0;
      $139 = (($138) + -1)|0;
      HEAP32[$iCellLast>>2] = $139;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $140 = HEAP32[$i>>2]|0;
      $141 = HEAP32[$1>>2]|0;
      $142 = ((($141)) + 18|0);
      $143 = HEAP16[$142>>1]|0;
      $144 = $143&65535;
      $145 = ($140|0)<($144|0);
      if (!($145)) {
       label = 17;
       break;
      }
      $146 = HEAP16[$cellOffset>>1]|0;
      $147 = $146&65535;
      $148 = HEAP32[$i>>2]|0;
      $149 = $148<<1;
      $150 = (($147) + ($149))|0;
      $151 = HEAP32[$data>>2]|0;
      $152 = (($151) + ($150)|0);
      $153 = HEAP8[$152>>0]|0;
      $154 = $153&255;
      $155 = $154 << 8;
      $156 = HEAP16[$cellOffset>>1]|0;
      $157 = $156&65535;
      $158 = HEAP32[$i>>2]|0;
      $159 = $158<<1;
      $160 = (($157) + ($159))|0;
      $161 = HEAP32[$data>>2]|0;
      $162 = (($161) + ($160)|0);
      $163 = ((($162)) + 1|0);
      $164 = HEAP8[$163>>0]|0;
      $165 = $164&255;
      $166 = $155 | $165;
      $167 = $166&65535;
      HEAP16[$pc>>1] = $167;
      $168 = HEAP16[$pc>>1]|0;
      $169 = $168&65535;
      $170 = HEAP32[$iCellFirst>>2]|0;
      $171 = ($169|0)<($170|0);
      if ($171) {
       label = 13;
       break;
      }
      $172 = HEAP16[$pc>>1]|0;
      $173 = $172&65535;
      $174 = HEAP32[$iCellLast>>2]|0;
      $175 = ($173|0)>($174|0);
      if ($175) {
       label = 13;
       break;
      }
      $177 = HEAP32[$1>>2]|0;
      $178 = ((($177)) + 76|0);
      $179 = HEAP32[$178>>2]|0;
      $180 = HEAP32[$1>>2]|0;
      $181 = HEAP16[$pc>>1]|0;
      $182 = $181&65535;
      $183 = HEAP32[$data>>2]|0;
      $184 = (($183) + ($182)|0);
      $185 = (FUNCTION_TABLE_iii[$179 & 255]($180,$184)|0);
      $186 = $185&65535;
      HEAP32[$sz>>2] = $186;
      $187 = HEAP16[$pc>>1]|0;
      $188 = $187&65535;
      $189 = HEAP32[$sz>>2]|0;
      $190 = (($188) + ($189))|0;
      $191 = HEAP32[$usableSize>>2]|0;
      $192 = ($190|0)>($191|0);
      if ($192) {
       label = 15;
       break;
      }
      $194 = HEAP32[$i>>2]|0;
      $195 = (($194) + 1)|0;
      HEAP32[$i>>2] = $195;
     }
     if ((label|0) == 13) {
      $176 = (_sqlite3CorruptError(60052)|0);
      HEAP32[$0>>2] = $176;
      $311 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($311|0);
     }
     else if ((label|0) == 15) {
      $193 = (_sqlite3CorruptError(60057)|0);
      HEAP32[$0>>2] = $193;
      $311 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($311|0);
     }
     else if ((label|0) == 17) {
      $196 = HEAP32[$1>>2]|0;
      $197 = ((($196)) + 4|0);
      $198 = HEAP8[$197>>0]|0;
      $199 = ($198<<24>>24)!=(0);
      if ($199) {
       break;
      }
      $200 = HEAP32[$iCellLast>>2]|0;
      $201 = (($200) + 1)|0;
      HEAP32[$iCellLast>>2] = $201;
      break;
     }
    }
   } while(0);
   $202 = HEAP8[$hdr>>0]|0;
   $203 = $202&255;
   $204 = (($203) + 1)|0;
   $205 = HEAP32[$data>>2]|0;
   $206 = (($205) + ($204)|0);
   $207 = HEAP8[$206>>0]|0;
   $208 = $207&255;
   $209 = $208 << 8;
   $210 = HEAP8[$hdr>>0]|0;
   $211 = $210&255;
   $212 = (($211) + 1)|0;
   $213 = HEAP32[$data>>2]|0;
   $214 = (($213) + ($212)|0);
   $215 = ((($214)) + 1|0);
   $216 = HEAP8[$215>>0]|0;
   $217 = $216&255;
   $218 = $209 | $217;
   $219 = $218&65535;
   HEAP16[$pc>>1] = $219;
   $220 = HEAP8[$hdr>>0]|0;
   $221 = $220&255;
   $222 = (($221) + 7)|0;
   $223 = HEAP32[$data>>2]|0;
   $224 = (($223) + ($222)|0);
   $225 = HEAP8[$224>>0]|0;
   $226 = $225&255;
   $227 = HEAP32[$top>>2]|0;
   $228 = (($226) + ($227))|0;
   HEAP32[$nFree>>2] = $228;
   while(1) {
    $229 = HEAP16[$pc>>1]|0;
    $230 = $229&65535;
    $231 = ($230|0)>(0);
    if (!($231)) {
     label = 29;
     break;
    }
    $232 = HEAP16[$pc>>1]|0;
    $233 = $232&65535;
    $234 = HEAP32[$iCellFirst>>2]|0;
    $235 = ($233|0)<($234|0);
    if ($235) {
     label = 23;
     break;
    }
    $236 = HEAP16[$pc>>1]|0;
    $237 = $236&65535;
    $238 = HEAP32[$iCellLast>>2]|0;
    $239 = ($237|0)>($238|0);
    if ($239) {
     label = 23;
     break;
    }
    $241 = HEAP16[$pc>>1]|0;
    $242 = $241&65535;
    $243 = HEAP32[$data>>2]|0;
    $244 = (($243) + ($242)|0);
    $245 = HEAP8[$244>>0]|0;
    $246 = $245&255;
    $247 = $246 << 8;
    $248 = HEAP16[$pc>>1]|0;
    $249 = $248&65535;
    $250 = HEAP32[$data>>2]|0;
    $251 = (($250) + ($249)|0);
    $252 = ((($251)) + 1|0);
    $253 = HEAP8[$252>>0]|0;
    $254 = $253&255;
    $255 = $247 | $254;
    $256 = $255&65535;
    HEAP16[$next>>1] = $256;
    $257 = HEAP16[$pc>>1]|0;
    $258 = $257&65535;
    $259 = (($258) + 2)|0;
    $260 = HEAP32[$data>>2]|0;
    $261 = (($260) + ($259)|0);
    $262 = HEAP8[$261>>0]|0;
    $263 = $262&255;
    $264 = $263 << 8;
    $265 = HEAP16[$pc>>1]|0;
    $266 = $265&65535;
    $267 = (($266) + 2)|0;
    $268 = HEAP32[$data>>2]|0;
    $269 = (($268) + ($267)|0);
    $270 = ((($269)) + 1|0);
    $271 = HEAP8[$270>>0]|0;
    $272 = $271&255;
    $273 = $264 | $272;
    $274 = $273&65535;
    HEAP16[$size>>1] = $274;
    $275 = HEAP16[$next>>1]|0;
    $276 = $275&65535;
    $277 = ($276|0)>(0);
    if ($277) {
     $278 = HEAP16[$next>>1]|0;
     $279 = $278&65535;
     $280 = HEAP16[$pc>>1]|0;
     $281 = $280&65535;
     $282 = HEAP16[$size>>1]|0;
     $283 = $282&65535;
     $284 = (($281) + ($283))|0;
     $285 = (($284) + 3)|0;
     $286 = ($279|0)<=($285|0);
     if ($286) {
      label = 27;
      break;
     }
    }
    $287 = HEAP16[$pc>>1]|0;
    $288 = $287&65535;
    $289 = HEAP16[$size>>1]|0;
    $290 = $289&65535;
    $291 = (($288) + ($290))|0;
    $292 = HEAP32[$usableSize>>2]|0;
    $293 = ($291|0)>($292|0);
    if ($293) {
     label = 27;
     break;
    }
    $295 = HEAP32[$nFree>>2]|0;
    $296 = HEAP16[$size>>1]|0;
    $297 = $296&65535;
    $298 = (($295) + ($297))|0;
    HEAP32[$nFree>>2] = $298;
    $299 = HEAP16[$next>>1]|0;
    HEAP16[$pc>>1] = $299;
   }
   if ((label|0) == 23) {
    $240 = (_sqlite3CorruptError(60077)|0);
    HEAP32[$0>>2] = $240;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   else if ((label|0) == 27) {
    $294 = (_sqlite3CorruptError(60084)|0);
    HEAP32[$0>>2] = $294;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
   else if ((label|0) == 29) {
    $300 = HEAP32[$nFree>>2]|0;
    $301 = HEAP32[$usableSize>>2]|0;
    $302 = ($300|0)>($301|0);
    if (!($302)) {
     $304 = HEAP32[$nFree>>2]|0;
     $305 = HEAP32[$iCellFirst>>2]|0;
     $306 = (($304) - ($305))|0;
     $307 = $306&65535;
     $308 = HEAP32[$1>>2]|0;
     $309 = ((($308)) + 16|0);
     HEAP16[$309>>1] = $307;
     $310 = HEAP32[$1>>2]|0;
     HEAP8[$310>>0] = 1;
     break;
    }
    $303 = (_sqlite3CorruptError(60098)|0);
    HEAP32[$0>>2] = $303;
    $311 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($311|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $311 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($311|0);
}
function _decodeFlags($pPage,$flagByte) {
 $pPage = $pPage|0;
 $flagByte = $flagByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $flagByte;
 $3 = HEAP32[$2>>2]|0;
 $4 = $3 >> 3;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP8[$7>>0] = $5;
 $8 = HEAP32[$2>>2]|0;
 $9 = $8 & -9;
 HEAP32[$2>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13<<2;
 $15 = (4 - ($14))|0;
 $16 = $15&255;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 6|0);
 HEAP8[$18>>0] = $16;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 76|0);
 HEAP32[$20>>2] = 188;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 52|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pBt>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = ($24|0)==(5);
 do {
  if ($25) {
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 2|0);
   HEAP8[$27>>0] = 1;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = ($30<<24>>24)!=(0);
   $32 = HEAP32[$1>>2]|0;
   $33 = ((($32)) + 3|0);
   if ($31) {
    HEAP8[$33>>0] = 1;
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 80|0);
    HEAP32[$35>>2] = 226;
   } else {
    HEAP8[$33>>0] = 0;
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 76|0);
    HEAP32[$37>>2] = 189;
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 80|0);
    HEAP32[$39>>2] = 227;
   }
   $40 = HEAP32[$pBt>>2]|0;
   $41 = ((($40)) + 28|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = HEAP32[$1>>2]|0;
   $44 = ((($43)) + 10|0);
   HEAP16[$44>>1] = $42;
   $45 = HEAP32[$pBt>>2]|0;
   $46 = ((($45)) + 30|0);
   $47 = HEAP16[$46>>1]|0;
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 12|0);
   HEAP16[$49>>1] = $47;
  } else {
   $50 = HEAP32[$2>>2]|0;
   $51 = ($50|0)==(2);
   if ($51) {
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 2|0);
    HEAP8[$53>>0] = 0;
    $54 = HEAP32[$1>>2]|0;
    $55 = ((($54)) + 3|0);
    HEAP8[$55>>0] = 0;
    $56 = HEAP32[$1>>2]|0;
    $57 = ((($56)) + 80|0);
    HEAP32[$57>>2] = 228;
    $58 = HEAP32[$pBt>>2]|0;
    $59 = ((($58)) + 24|0);
    $60 = HEAP16[$59>>1]|0;
    $61 = HEAP32[$1>>2]|0;
    $62 = ((($61)) + 10|0);
    HEAP16[$62>>1] = $60;
    $63 = HEAP32[$pBt>>2]|0;
    $64 = ((($63)) + 26|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = ((($66)) + 12|0);
    HEAP16[$67>>1] = $65;
    break;
   }
   $68 = (_sqlite3CorruptError(59964)|0);
   HEAP32[$0>>2] = $68;
   $74 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($74|0);
  }
 } while(0);
 $69 = HEAP32[$pBt>>2]|0;
 $70 = ((($69)) + 21|0);
 $71 = HEAP8[$70>>0]|0;
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 7|0);
 HEAP8[$73>>0] = $71;
 HEAP32[$0>>2] = 0;
 $74 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _cellSizePtr($pPage,$pCell) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $minLocal = 0, $nSize = 0, $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pIter = sp + 12|0;
 $pEnd = sp + 8|0;
 $nSize = sp + 4|0;
 $minLocal = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 6|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (($2) + ($6)|0);
 HEAP32[$pIter>>2] = $7;
 $8 = HEAP32[$pIter>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$nSize>>2] = $10;
 $11 = HEAP32[$nSize>>2]|0;
 $12 = ($11>>>0)>=(128);
 L1: do {
  if ($12) {
   $13 = HEAP32[$pIter>>2]|0;
   $14 = ((($13)) + 8|0);
   HEAP32[$pEnd>>2] = $14;
   $15 = HEAP32[$nSize>>2]|0;
   $16 = $15 & 127;
   HEAP32[$nSize>>2] = $16;
   while(1) {
    $17 = HEAP32[$nSize>>2]|0;
    $18 = $17 << 7;
    $19 = HEAP32[$pIter>>2]|0;
    $20 = ((($19)) + 1|0);
    HEAP32[$pIter>>2] = $20;
    $21 = HEAP8[$20>>0]|0;
    $22 = $21&255;
    $23 = $22 & 127;
    $24 = $18 | $23;
    HEAP32[$nSize>>2] = $24;
    $25 = HEAP32[$pIter>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = ($27|0)>=(128);
    if (!($28)) {
     break L1;
    }
    $29 = HEAP32[$pIter>>2]|0;
    $30 = HEAP32[$pEnd>>2]|0;
    $31 = ($29>>>0)<($30>>>0);
    if (!($31)) {
     break;
    }
   }
  }
 } while(0);
 $32 = HEAP32[$pIter>>2]|0;
 $33 = ((($32)) + 1|0);
 HEAP32[$pIter>>2] = $33;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 2|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = ($36<<24>>24)!=(0);
 L7: do {
  if ($37) {
   $38 = HEAP32[$pIter>>2]|0;
   $39 = ((($38)) + 9|0);
   HEAP32[$pEnd>>2] = $39;
   while(1) {
    $40 = HEAP32[$pIter>>2]|0;
    $41 = ((($40)) + 1|0);
    HEAP32[$pIter>>2] = $41;
    $42 = HEAP8[$40>>0]|0;
    $43 = $42&255;
    $44 = $43 & 128;
    $45 = ($44|0)!=(0);
    if (!($45)) {
     break L7;
    }
    $46 = HEAP32[$pIter>>2]|0;
    $47 = HEAP32[$pEnd>>2]|0;
    $48 = ($46>>>0)<($47>>>0);
    if (!($48)) {
     break;
    }
   }
  }
 } while(0);
 $49 = HEAP32[$nSize>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 10|0);
 $52 = HEAP16[$51>>1]|0;
 $53 = $52&65535;
 $54 = ($49>>>0)<=($53>>>0);
 if ($54) {
  $55 = HEAP32[$pIter>>2]|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = $55;
  $58 = $56;
  $59 = (($57) - ($58))|0;
  $60 = HEAP32[$nSize>>2]|0;
  $61 = (($60) + ($59))|0;
  HEAP32[$nSize>>2] = $61;
  $62 = HEAP32[$nSize>>2]|0;
  $63 = ($62>>>0)<(4);
  $$ = $63 ? 4 : $61;
  HEAP32[$nSize>>2] = $$;
  $97 = HEAP32[$nSize>>2]|0;
  $98 = $97&65535;
  STACKTOP = sp;return ($98|0);
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 12|0);
 $66 = HEAP16[$65>>1]|0;
 $67 = $66&65535;
 HEAP32[$minLocal>>2] = $67;
 $68 = HEAP32[$minLocal>>2]|0;
 $69 = HEAP32[$nSize>>2]|0;
 $70 = HEAP32[$minLocal>>2]|0;
 $71 = (($69) - ($70))|0;
 $72 = HEAP32[$0>>2]|0;
 $73 = ((($72)) + 52|0);
 $74 = HEAP32[$73>>2]|0;
 $75 = ((($74)) + 36|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = (($76) - 4)|0;
 $78 = (($71>>>0) % ($77>>>0))&-1;
 $79 = (($68) + ($78))|0;
 HEAP32[$nSize>>2] = $79;
 $80 = HEAP32[$nSize>>2]|0;
 $81 = HEAP32[$0>>2]|0;
 $82 = ((($81)) + 10|0);
 $83 = HEAP16[$82>>1]|0;
 $84 = $83&65535;
 $85 = ($80>>>0)>($84>>>0);
 if ($85) {
  $86 = HEAP32[$minLocal>>2]|0;
  HEAP32[$nSize>>2] = $86;
 }
 $87 = HEAP32[$pIter>>2]|0;
 $88 = HEAP32[$1>>2]|0;
 $89 = $87;
 $90 = $88;
 $91 = (($89) - ($90))|0;
 $92 = $91&65535;
 $93 = $92&65535;
 $94 = (4 + ($93))|0;
 $95 = HEAP32[$nSize>>2]|0;
 $96 = (($95) + ($94))|0;
 HEAP32[$nSize>>2] = $96;
 $97 = HEAP32[$nSize>>2]|0;
 $98 = $97&65535;
 STACKTOP = sp;return ($98|0);
}
function _btreeParseCellPtr($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iKey = 0, $nPayload = 0, $pEnd = 0;
 var $pEnd1 = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $pIter = sp + 20|0;
 $nPayload = sp + 16|0;
 $iKey = sp;
 $pEnd = sp + 12|0;
 $pEnd1 = sp + 8|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pIter>>2] = $3;
 $4 = HEAP32[$pIter>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 HEAP32[$nPayload>>2] = $6;
 $7 = HEAP32[$nPayload>>2]|0;
 $8 = ($7>>>0)>=(128);
 L1: do {
  if ($8) {
   $9 = HEAP32[$pIter>>2]|0;
   $10 = ((($9)) + 8|0);
   HEAP32[$pEnd>>2] = $10;
   $11 = HEAP32[$nPayload>>2]|0;
   $12 = $11 & 127;
   HEAP32[$nPayload>>2] = $12;
   while(1) {
    $13 = HEAP32[$nPayload>>2]|0;
    $14 = $13 << 7;
    $15 = HEAP32[$pIter>>2]|0;
    $16 = ((($15)) + 1|0);
    HEAP32[$pIter>>2] = $16;
    $17 = HEAP8[$16>>0]|0;
    $18 = $17&255;
    $19 = $18 & 127;
    $20 = $14 | $19;
    HEAP32[$nPayload>>2] = $20;
    $21 = HEAP32[$pIter>>2]|0;
    $22 = HEAP8[$21>>0]|0;
    $23 = $22&255;
    $24 = ($23|0)>=(128);
    if (!($24)) {
     break L1;
    }
    $25 = HEAP32[$pIter>>2]|0;
    $26 = HEAP32[$pEnd>>2]|0;
    $27 = ($25>>>0)<($26>>>0);
    if (!($27)) {
     break;
    }
   }
  }
 } while(0);
 $28 = HEAP32[$pIter>>2]|0;
 $29 = ((($28)) + 1|0);
 HEAP32[$pIter>>2] = $29;
 $30 = HEAP32[$pIter>>2]|0;
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 $33 = $iKey;
 $34 = $33;
 HEAP32[$34>>2] = $32;
 $35 = (($33) + 4)|0;
 $36 = $35;
 HEAP32[$36>>2] = 0;
 $37 = $iKey;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) + 4)|0;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42>>>0)>(0);
 $44 = ($39>>>0)>=(128);
 $45 = ($42|0)==(0);
 $46 = $45 & $44;
 $47 = $43 | $46;
 L7: do {
  if ($47) {
   $48 = HEAP32[$pIter>>2]|0;
   $49 = ((($48)) + 7|0);
   HEAP32[$pEnd1>>2] = $49;
   $50 = $iKey;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = $52 & 127;
   $57 = $iKey;
   $58 = $57;
   HEAP32[$58>>2] = $56;
   $59 = (($57) + 4)|0;
   $60 = $59;
   HEAP32[$60>>2] = 0;
   while(1) {
    $61 = $iKey;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (($61) + 4)|0;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = (_bitshift64Shl(($63|0),($66|0),7)|0);
    $68 = tempRet0;
    $69 = HEAP32[$pIter>>2]|0;
    $70 = ((($69)) + 1|0);
    HEAP32[$pIter>>2] = $70;
    $71 = HEAP8[$70>>0]|0;
    $72 = $71&255;
    $73 = $72 & 127;
    $74 = ($73|0)<(0);
    $75 = $74 << 31 >> 31;
    $76 = $67 | $73;
    $77 = $68 | $75;
    $78 = $iKey;
    $79 = $78;
    HEAP32[$79>>2] = $76;
    $80 = (($78) + 4)|0;
    $81 = $80;
    HEAP32[$81>>2] = $77;
    $82 = HEAP32[$pIter>>2]|0;
    $83 = HEAP8[$82>>0]|0;
    $84 = $83&255;
    $85 = ($84|0)<(128);
    if ($85) {
     break L7;
    }
    $86 = HEAP32[$pIter>>2]|0;
    $87 = HEAP32[$pEnd1>>2]|0;
    $88 = ($86>>>0)>=($87>>>0);
    if ($88) {
     break;
    }
   }
   $89 = $iKey;
   $90 = $89;
   $91 = HEAP32[$90>>2]|0;
   $92 = (($89) + 4)|0;
   $93 = $92;
   $94 = HEAP32[$93>>2]|0;
   $95 = (_bitshift64Shl(($91|0),($94|0),8)|0);
   $96 = tempRet0;
   $97 = HEAP32[$pIter>>2]|0;
   $98 = ((($97)) + 1|0);
   HEAP32[$pIter>>2] = $98;
   $99 = HEAP8[$98>>0]|0;
   $100 = $99&255;
   $101 = $95 | $100;
   $102 = $iKey;
   $103 = $102;
   HEAP32[$103>>2] = $101;
   $104 = (($102) + 4)|0;
   $105 = $104;
   HEAP32[$105>>2] = $96;
  }
 } while(0);
 $106 = HEAP32[$pIter>>2]|0;
 $107 = ((($106)) + 1|0);
 HEAP32[$pIter>>2] = $107;
 $108 = $iKey;
 $109 = $108;
 $110 = HEAP32[$109>>2]|0;
 $111 = (($108) + 4)|0;
 $112 = $111;
 $113 = HEAP32[$112>>2]|0;
 $114 = HEAP32[$2>>2]|0;
 $115 = $114;
 $116 = $115;
 HEAP32[$116>>2] = $110;
 $117 = (($115) + 4)|0;
 $118 = $117;
 HEAP32[$118>>2] = $113;
 $119 = HEAP32[$nPayload>>2]|0;
 $120 = HEAP32[$2>>2]|0;
 $121 = ((($120)) + 12|0);
 HEAP32[$121>>2] = $119;
 $122 = HEAP32[$pIter>>2]|0;
 $123 = HEAP32[$2>>2]|0;
 $124 = ((($123)) + 8|0);
 HEAP32[$124>>2] = $122;
 $125 = HEAP32[$nPayload>>2]|0;
 $126 = HEAP32[$0>>2]|0;
 $127 = ((($126)) + 10|0);
 $128 = HEAP16[$127>>1]|0;
 $129 = $128&65535;
 $130 = ($125>>>0)<=($129>>>0);
 if (!($130)) {
  $154 = HEAP32[$0>>2]|0;
  $155 = HEAP32[$1>>2]|0;
  $156 = HEAP32[$2>>2]|0;
  _btreeParseCellAdjustSizeForOverflow($154,$155,$156);
  STACKTOP = sp;return;
 }
 $131 = HEAP32[$nPayload>>2]|0;
 $132 = HEAP32[$pIter>>2]|0;
 $133 = HEAP32[$1>>2]|0;
 $134 = $132;
 $135 = $133;
 $136 = (($134) - ($135))|0;
 $137 = $136&65535;
 $138 = $137&65535;
 $139 = (($131) + ($138))|0;
 $140 = $139&65535;
 $141 = HEAP32[$2>>2]|0;
 $142 = ((($141)) + 18|0);
 HEAP16[$142>>1] = $140;
 $143 = HEAP32[$2>>2]|0;
 $144 = ((($143)) + 18|0);
 $145 = HEAP16[$144>>1]|0;
 $146 = $145&65535;
 $147 = ($146|0)<(4);
 if ($147) {
  $148 = HEAP32[$2>>2]|0;
  $149 = ((($148)) + 18|0);
  HEAP16[$149>>1] = 4;
 }
 $150 = HEAP32[$nPayload>>2]|0;
 $151 = $150&65535;
 $152 = HEAP32[$2>>2]|0;
 $153 = ((($152)) + 16|0);
 HEAP16[$153>>1] = $151;
 STACKTOP = sp;return;
}
function _cellSizePtrNoPayload($pPage,$pCell) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pIter = sp + 4|0;
 $pEnd = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 HEAP32[$pIter>>2] = $3;
 $4 = HEAP32[$pIter>>2]|0;
 $5 = ((($4)) + 9|0);
 HEAP32[$pEnd>>2] = $5;
 while(1) {
  $6 = HEAP32[$pIter>>2]|0;
  $7 = ((($6)) + 1|0);
  HEAP32[$pIter>>2] = $7;
  $8 = HEAP8[$6>>0]|0;
  $9 = $8&255;
  $10 = $9 & 128;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pIter>>2]|0;
  $13 = HEAP32[$pEnd>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if (!($14)) {
   break;
  }
 }
 $15 = HEAP32[$pIter>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = $15;
 $18 = $16;
 $19 = (($17) - ($18))|0;
 $20 = $19&65535;
 STACKTOP = sp;return ($20|0);
}
function _btreeParseCellPtrNoPayload($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3GetVarint($4,$5)|0);
 $7 = $6&255;
 $8 = (4 + ($7))|0;
 $9 = $8&65535;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 18|0);
 HEAP16[$11>>1] = $9;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 12|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 16|0);
 HEAP16[$15>>1] = 0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = 0;
 STACKTOP = sp;return;
}
function _btreeParseCellPtrIndex($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $nPayload = 0;
 var $pEnd = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pIter = sp + 8|0;
 $nPayload = sp + 4|0;
 $pEnd = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 6|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = (($3) + ($7)|0);
 HEAP32[$pIter>>2] = $8;
 $9 = HEAP32[$pIter>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$nPayload>>2] = $11;
 $12 = HEAP32[$nPayload>>2]|0;
 $13 = ($12>>>0)>=(128);
 L1: do {
  if ($13) {
   $14 = HEAP32[$pIter>>2]|0;
   $15 = ((($14)) + 8|0);
   HEAP32[$pEnd>>2] = $15;
   $16 = HEAP32[$nPayload>>2]|0;
   $17 = $16 & 127;
   HEAP32[$nPayload>>2] = $17;
   while(1) {
    $18 = HEAP32[$nPayload>>2]|0;
    $19 = $18 << 7;
    $20 = HEAP32[$pIter>>2]|0;
    $21 = ((($20)) + 1|0);
    HEAP32[$pIter>>2] = $21;
    $22 = HEAP8[$21>>0]|0;
    $23 = $22&255;
    $24 = $23 & 127;
    $25 = $19 | $24;
    HEAP32[$nPayload>>2] = $25;
    $26 = HEAP32[$pIter>>2]|0;
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = ($28|0)>=(128);
    if (!($29)) {
     break L1;
    }
    $30 = HEAP32[$pIter>>2]|0;
    $31 = HEAP32[$pEnd>>2]|0;
    $32 = ($30>>>0)<($31>>>0);
    if (!($32)) {
     break;
    }
   }
  }
 } while(0);
 $33 = HEAP32[$pIter>>2]|0;
 $34 = ((($33)) + 1|0);
 HEAP32[$pIter>>2] = $34;
 $35 = HEAP32[$nPayload>>2]|0;
 $36 = HEAP32[$2>>2]|0;
 $37 = $36;
 $38 = $37;
 HEAP32[$38>>2] = $35;
 $39 = (($37) + 4)|0;
 $40 = $39;
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$nPayload>>2]|0;
 $42 = HEAP32[$2>>2]|0;
 $43 = ((($42)) + 12|0);
 HEAP32[$43>>2] = $41;
 $44 = HEAP32[$pIter>>2]|0;
 $45 = HEAP32[$2>>2]|0;
 $46 = ((($45)) + 8|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$nPayload>>2]|0;
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 10|0);
 $50 = HEAP16[$49>>1]|0;
 $51 = $50&65535;
 $52 = ($47>>>0)<=($51>>>0);
 if (!($52)) {
  $76 = HEAP32[$0>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  _btreeParseCellAdjustSizeForOverflow($76,$77,$78);
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$nPayload>>2]|0;
 $54 = HEAP32[$pIter>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 $56 = $54;
 $57 = $55;
 $58 = (($56) - ($57))|0;
 $59 = $58&65535;
 $60 = $59&65535;
 $61 = (($53) + ($60))|0;
 $62 = $61&65535;
 $63 = HEAP32[$2>>2]|0;
 $64 = ((($63)) + 18|0);
 HEAP16[$64>>1] = $62;
 $65 = HEAP32[$2>>2]|0;
 $66 = ((($65)) + 18|0);
 $67 = HEAP16[$66>>1]|0;
 $68 = $67&65535;
 $69 = ($68|0)<(4);
 if ($69) {
  $70 = HEAP32[$2>>2]|0;
  $71 = ((($70)) + 18|0);
  HEAP16[$71>>1] = 4;
 }
 $72 = HEAP32[$nPayload>>2]|0;
 $73 = $72&65535;
 $74 = HEAP32[$2>>2]|0;
 $75 = ((($74)) + 16|0);
 HEAP16[$75>>1] = $73;
 STACKTOP = sp;return;
}
function _btreeParseCellAdjustSizeForOverflow($pPage,$pCell,$pInfo) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $maxLocal = 0, $minLocal = 0, $surplus = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $minLocal = sp + 8|0;
 $maxLocal = sp + 4|0;
 $surplus = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 HEAP32[$minLocal>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 10|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 HEAP32[$maxLocal>>2] = $10;
 $11 = HEAP32[$minLocal>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 12|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$minLocal>>2]|0;
 $16 = (($14) - ($15))|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 36|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - 4)|0;
 $23 = (($16>>>0) % ($22>>>0))&-1;
 $24 = (($11) + ($23))|0;
 HEAP32[$surplus>>2] = $24;
 $25 = HEAP32[$surplus>>2]|0;
 $26 = HEAP32[$maxLocal>>2]|0;
 $27 = ($25|0)<=($26|0);
 if ($27) {
  $28 = HEAP32[$surplus>>2]|0;
  $29 = $28&65535;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 16|0);
  HEAP16[$31>>1] = $29;
 } else {
  $32 = HEAP32[$minLocal>>2]|0;
  $33 = $32&65535;
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 16|0);
  HEAP16[$35>>1] = $33;
 }
 $36 = HEAP32[$2>>2]|0;
 $37 = ((($36)) + 16|0);
 $38 = HEAP16[$37>>1]|0;
 $39 = $38&65535;
 $40 = HEAP32[$2>>2]|0;
 $41 = ((($40)) + 8|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = (($42) + ($39)|0);
 $44 = HEAP32[$1>>2]|0;
 $45 = $43;
 $46 = $44;
 $47 = (($45) - ($46))|0;
 $48 = $47&65535;
 $49 = $48&65535;
 $50 = (($49) + 4)|0;
 $51 = $50&65535;
 $52 = HEAP32[$2>>2]|0;
 $53 = ((($52)) + 18|0);
 HEAP16[$53>>1] = $51;
 STACKTOP = sp;return;
}
function _sqlite3GetVarint($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $a = 0, $b = 0, $s = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $a = sp + 8|0;
 $b = sp + 4|0;
 $s = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 HEAP32[$a>>2] = $5;
 $6 = HEAP32[$a>>2]|0;
 $7 = $6 & 128;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = HEAP32[$a>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = $10;
  $12 = $11;
  HEAP32[$12>>2] = $9;
  $13 = (($11) + 4)|0;
  $14 = $13;
  HEAP32[$14>>2] = 0;
  HEAP8[$0>>0] = 1;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 1|0);
 HEAP32[$1>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 HEAP32[$b>>2] = $19;
 $20 = HEAP32[$b>>2]|0;
 $21 = $20 & 128;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $23 = HEAP32[$a>>2]|0;
  $24 = $23 & 127;
  HEAP32[$a>>2] = $24;
  $25 = HEAP32[$a>>2]|0;
  $26 = $25 << 7;
  HEAP32[$a>>2] = $26;
  $27 = HEAP32[$b>>2]|0;
  $28 = HEAP32[$a>>2]|0;
  $29 = $28 | $27;
  HEAP32[$a>>2] = $29;
  $30 = HEAP32[$a>>2]|0;
  $31 = HEAP32[$2>>2]|0;
  $32 = $31;
  $33 = $32;
  HEAP32[$33>>2] = $30;
  $34 = (($32) + 4)|0;
  $35 = $34;
  HEAP32[$35>>2] = 0;
  HEAP8[$0>>0] = 2;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 1|0);
 HEAP32[$1>>2] = $37;
 $38 = HEAP32[$a>>2]|0;
 $39 = $38 << 14;
 HEAP32[$a>>2] = $39;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = HEAP32[$a>>2]|0;
 $44 = $43 | $42;
 HEAP32[$a>>2] = $44;
 $45 = HEAP32[$a>>2]|0;
 $46 = $45 & 128;
 $47 = ($46|0)!=(0);
 $48 = HEAP32[$a>>2]|0;
 $49 = $48 & 2080895;
 HEAP32[$a>>2] = $49;
 if (!($47)) {
  $50 = HEAP32[$b>>2]|0;
  $51 = $50 & 127;
  HEAP32[$b>>2] = $51;
  $52 = HEAP32[$b>>2]|0;
  $53 = $52 << 7;
  HEAP32[$b>>2] = $53;
  $54 = HEAP32[$b>>2]|0;
  $55 = HEAP32[$a>>2]|0;
  $56 = $55 | $54;
  HEAP32[$a>>2] = $56;
  $57 = HEAP32[$a>>2]|0;
  $58 = HEAP32[$2>>2]|0;
  $59 = $58;
  $60 = $59;
  HEAP32[$60>>2] = $57;
  $61 = (($59) + 4)|0;
  $62 = $61;
  HEAP32[$62>>2] = 0;
  HEAP8[$0>>0] = 3;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = ((($63)) + 1|0);
 HEAP32[$1>>2] = $64;
 $65 = HEAP32[$b>>2]|0;
 $66 = $65 << 14;
 HEAP32[$b>>2] = $66;
 $67 = HEAP32[$1>>2]|0;
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = HEAP32[$b>>2]|0;
 $71 = $70 | $69;
 HEAP32[$b>>2] = $71;
 $72 = HEAP32[$b>>2]|0;
 $73 = $72 & 128;
 $74 = ($73|0)!=(0);
 $75 = HEAP32[$b>>2]|0;
 $76 = $75 & 2080895;
 HEAP32[$b>>2] = $76;
 $77 = HEAP32[$a>>2]|0;
 if (!($74)) {
  $78 = $77 << 7;
  HEAP32[$a>>2] = $78;
  $79 = HEAP32[$b>>2]|0;
  $80 = HEAP32[$a>>2]|0;
  $81 = $80 | $79;
  HEAP32[$a>>2] = $81;
  $82 = HEAP32[$a>>2]|0;
  $83 = HEAP32[$2>>2]|0;
  $84 = $83;
  $85 = $84;
  HEAP32[$85>>2] = $82;
  $86 = (($84) + 4)|0;
  $87 = $86;
  HEAP32[$87>>2] = 0;
  HEAP8[$0>>0] = 4;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 HEAP32[$s>>2] = $77;
 $88 = HEAP32[$1>>2]|0;
 $89 = ((($88)) + 1|0);
 HEAP32[$1>>2] = $89;
 $90 = HEAP32[$a>>2]|0;
 $91 = $90 << 14;
 HEAP32[$a>>2] = $91;
 $92 = HEAP32[$1>>2]|0;
 $93 = HEAP8[$92>>0]|0;
 $94 = $93&255;
 $95 = HEAP32[$a>>2]|0;
 $96 = $95 | $94;
 HEAP32[$a>>2] = $96;
 $97 = HEAP32[$a>>2]|0;
 $98 = $97 & 128;
 $99 = ($98|0)!=(0);
 if (!($99)) {
  $100 = HEAP32[$b>>2]|0;
  $101 = $100 << 7;
  HEAP32[$b>>2] = $101;
  $102 = HEAP32[$b>>2]|0;
  $103 = HEAP32[$a>>2]|0;
  $104 = $103 | $102;
  HEAP32[$a>>2] = $104;
  $105 = HEAP32[$s>>2]|0;
  $106 = $105 >>> 18;
  HEAP32[$s>>2] = $106;
  $107 = HEAP32[$s>>2]|0;
  $108 = HEAP32[$a>>2]|0;
  $109 = HEAP32[$2>>2]|0;
  $110 = $109;
  $111 = $110;
  HEAP32[$111>>2] = $108;
  $112 = (($110) + 4)|0;
  $113 = $112;
  HEAP32[$113>>2] = $107;
  HEAP8[$0>>0] = 5;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $114 = HEAP32[$s>>2]|0;
 $115 = $114 << 7;
 HEAP32[$s>>2] = $115;
 $116 = HEAP32[$b>>2]|0;
 $117 = HEAP32[$s>>2]|0;
 $118 = $117 | $116;
 HEAP32[$s>>2] = $118;
 $119 = HEAP32[$1>>2]|0;
 $120 = ((($119)) + 1|0);
 HEAP32[$1>>2] = $120;
 $121 = HEAP32[$b>>2]|0;
 $122 = $121 << 14;
 HEAP32[$b>>2] = $122;
 $123 = HEAP32[$1>>2]|0;
 $124 = HEAP8[$123>>0]|0;
 $125 = $124&255;
 $126 = HEAP32[$b>>2]|0;
 $127 = $126 | $125;
 HEAP32[$b>>2] = $127;
 $128 = HEAP32[$b>>2]|0;
 $129 = $128 & 128;
 $130 = ($129|0)!=(0);
 if (!($130)) {
  $131 = HEAP32[$a>>2]|0;
  $132 = $131 & 2080895;
  HEAP32[$a>>2] = $132;
  $133 = HEAP32[$a>>2]|0;
  $134 = $133 << 7;
  HEAP32[$a>>2] = $134;
  $135 = HEAP32[$b>>2]|0;
  $136 = HEAP32[$a>>2]|0;
  $137 = $136 | $135;
  HEAP32[$a>>2] = $137;
  $138 = HEAP32[$s>>2]|0;
  $139 = $138 >>> 18;
  HEAP32[$s>>2] = $139;
  $140 = HEAP32[$s>>2]|0;
  $141 = HEAP32[$a>>2]|0;
  $142 = HEAP32[$2>>2]|0;
  $143 = $142;
  $144 = $143;
  HEAP32[$144>>2] = $141;
  $145 = (($143) + 4)|0;
  $146 = $145;
  HEAP32[$146>>2] = $140;
  HEAP8[$0>>0] = 6;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $147 = HEAP32[$1>>2]|0;
 $148 = ((($147)) + 1|0);
 HEAP32[$1>>2] = $148;
 $149 = HEAP32[$a>>2]|0;
 $150 = $149 << 14;
 HEAP32[$a>>2] = $150;
 $151 = HEAP32[$1>>2]|0;
 $152 = HEAP8[$151>>0]|0;
 $153 = $152&255;
 $154 = HEAP32[$a>>2]|0;
 $155 = $154 | $153;
 HEAP32[$a>>2] = $155;
 $156 = HEAP32[$a>>2]|0;
 $157 = $156 & 128;
 $158 = ($157|0)!=(0);
 $159 = HEAP32[$a>>2]|0;
 if (!($158)) {
  $160 = $159 & -266354561;
  HEAP32[$a>>2] = $160;
  $161 = HEAP32[$b>>2]|0;
  $162 = $161 & 2080895;
  HEAP32[$b>>2] = $162;
  $163 = HEAP32[$b>>2]|0;
  $164 = $163 << 7;
  HEAP32[$b>>2] = $164;
  $165 = HEAP32[$b>>2]|0;
  $166 = HEAP32[$a>>2]|0;
  $167 = $166 | $165;
  HEAP32[$a>>2] = $167;
  $168 = HEAP32[$s>>2]|0;
  $169 = $168 >>> 11;
  HEAP32[$s>>2] = $169;
  $170 = HEAP32[$s>>2]|0;
  $171 = HEAP32[$a>>2]|0;
  $172 = HEAP32[$2>>2]|0;
  $173 = $172;
  $174 = $173;
  HEAP32[$174>>2] = $171;
  $175 = (($173) + 4)|0;
  $176 = $175;
  HEAP32[$176>>2] = $170;
  HEAP8[$0>>0] = 7;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 $177 = $159 & 2080895;
 HEAP32[$a>>2] = $177;
 $178 = HEAP32[$1>>2]|0;
 $179 = ((($178)) + 1|0);
 HEAP32[$1>>2] = $179;
 $180 = HEAP32[$b>>2]|0;
 $181 = $180 << 14;
 HEAP32[$b>>2] = $181;
 $182 = HEAP32[$1>>2]|0;
 $183 = HEAP8[$182>>0]|0;
 $184 = $183&255;
 $185 = HEAP32[$b>>2]|0;
 $186 = $185 | $184;
 HEAP32[$b>>2] = $186;
 $187 = HEAP32[$b>>2]|0;
 $188 = $187 & 128;
 $189 = ($188|0)!=(0);
 if ($189) {
  $206 = HEAP32[$1>>2]|0;
  $207 = ((($206)) + 1|0);
  HEAP32[$1>>2] = $207;
  $208 = HEAP32[$a>>2]|0;
  $209 = $208 << 15;
  HEAP32[$a>>2] = $209;
  $210 = HEAP32[$1>>2]|0;
  $211 = HEAP8[$210>>0]|0;
  $212 = $211&255;
  $213 = HEAP32[$a>>2]|0;
  $214 = $213 | $212;
  HEAP32[$a>>2] = $214;
  $215 = HEAP32[$b>>2]|0;
  $216 = $215 & 2080895;
  HEAP32[$b>>2] = $216;
  $217 = HEAP32[$b>>2]|0;
  $218 = $217 << 8;
  HEAP32[$b>>2] = $218;
  $219 = HEAP32[$b>>2]|0;
  $220 = HEAP32[$a>>2]|0;
  $221 = $220 | $219;
  HEAP32[$a>>2] = $221;
  $222 = HEAP32[$s>>2]|0;
  $223 = $222 << 4;
  HEAP32[$s>>2] = $223;
  $224 = HEAP32[$1>>2]|0;
  $225 = ((($224)) + -4|0);
  $226 = HEAP8[$225>>0]|0;
  $227 = $226&255;
  HEAP32[$b>>2] = $227;
  $228 = HEAP32[$b>>2]|0;
  $229 = $228 & 127;
  HEAP32[$b>>2] = $229;
  $230 = HEAP32[$b>>2]|0;
  $231 = $230 >>> 3;
  HEAP32[$b>>2] = $231;
  $232 = HEAP32[$b>>2]|0;
  $233 = HEAP32[$s>>2]|0;
  $234 = $233 | $232;
  HEAP32[$s>>2] = $234;
  $235 = HEAP32[$s>>2]|0;
  $236 = HEAP32[$a>>2]|0;
  $237 = HEAP32[$2>>2]|0;
  $238 = $237;
  $239 = $238;
  HEAP32[$239>>2] = $236;
  $240 = (($238) + 4)|0;
  $241 = $240;
  HEAP32[$241>>2] = $235;
  HEAP8[$0>>0] = 9;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 } else {
  $190 = HEAP32[$b>>2]|0;
  $191 = $190 & -266354561;
  HEAP32[$b>>2] = $191;
  $192 = HEAP32[$a>>2]|0;
  $193 = $192 << 7;
  HEAP32[$a>>2] = $193;
  $194 = HEAP32[$b>>2]|0;
  $195 = HEAP32[$a>>2]|0;
  $196 = $195 | $194;
  HEAP32[$a>>2] = $196;
  $197 = HEAP32[$s>>2]|0;
  $198 = $197 >>> 4;
  HEAP32[$s>>2] = $198;
  $199 = HEAP32[$s>>2]|0;
  $200 = HEAP32[$a>>2]|0;
  $201 = HEAP32[$2>>2]|0;
  $202 = $201;
  $203 = $202;
  HEAP32[$203>>2] = $200;
  $204 = (($202) + 4)|0;
  $205 = $204;
  HEAP32[$205>>2] = $199;
  HEAP8[$0>>0] = 8;
  $242 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($242|0);
 }
 return (0)|0;
}
function _allSpaces($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $n;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)>(0);
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = (($4) - 1)|0;
   $6 = HEAP32[$0>>2]|0;
   $7 = (($6) + ($5)|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8 << 24 >> 24;
   $10 = ($9|0)==(32);
   $15 = $10;
  } else {
   $15 = 0;
  }
  $11 = HEAP32[$1>>2]|0;
  if (!($15)) {
   break;
  }
  $12 = (($11) + -1)|0;
  HEAP32[$1>>2] = $12;
 }
 $13 = ($11|0)==(0);
 $14 = $13&1;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3BtreeCheckpoint($p,$eMode,$pnLog,$pnCkpt) {
 $p = $p|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$2>>2] = $pnLog;
 HEAP32[$3>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($9);
 $10 = HEAP32[$pBt>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$rc>>2] = 6;
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $15 = HEAP32[$pBt>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$3>>2]|0;
  $20 = (_sqlite3PagerCheckpoint($16,$17,$18,$19)|0);
  HEAP32[$rc>>2] = $20;
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3PagerCheckpoint($pPager,$eMode,$pnLog,$pnCkpt) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 $pnLog = $pnLog|0;
 $pnCkpt = $pnCkpt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$2>>2] = $pnLog;
 HEAP32[$3>>2] = $pnCkpt;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $34 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 216|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $32 = 0;
 } else {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 184|0);
  $16 = HEAP32[$15>>2]|0;
  $32 = $16;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 188|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 10|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 160|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 208|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$3>>2]|0;
 $33 = (_sqlite3WalCheckpoint($10,$11,$32,$19,$23,$26,$29,$30,$31)|0);
 HEAP32[$rc>>2] = $33;
 $34 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($34|0);
}
function _matchQuality($p,$nArg,$enc) {
 $p = $p|0;
 $nArg = $nArg|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $match = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $match = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nArg;
 HEAP8[$3>>0] = $enc;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(-2);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($8|0)==(0|0);
  $10 = $9 ? 0 : 6;
  HEAP32[$0>>2] = $10;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $11 = HEAP8[$6>>0]|0;
 $12 = $11 << 24 >> 24;
 $13 = HEAP32[$2>>2]|0;
 $14 = ($12|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = $16 << 24 >> 24;
  $18 = ($17|0)>=(0);
  if ($18) {
   HEAP32[$0>>2] = 0;
   $46 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP8[$19>>0]|0;
 $21 = $20 << 24 >> 24;
 $22 = HEAP32[$2>>2]|0;
 $23 = ($21|0)==($22|0);
 if ($23) {
  HEAP32[$match>>2] = 4;
 } else {
  HEAP32[$match>>2] = 1;
 }
 $24 = HEAP8[$3>>0]|0;
 $25 = $24&255;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 2|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = $29 & 3;
 $31 = ($25|0)==($30|0);
 if ($31) {
  $32 = HEAP32[$match>>2]|0;
  $33 = (($32) + 2)|0;
  HEAP32[$match>>2] = $33;
 } else {
  $34 = HEAP8[$3>>0]|0;
  $35 = $34&255;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 2|0);
  $38 = HEAP16[$37>>1]|0;
  $39 = $38&65535;
  $40 = $35 & $39;
  $41 = $40 & 2;
  $42 = ($41|0)!=(0);
  if ($42) {
   $43 = HEAP32[$match>>2]|0;
   $44 = (($43) + 1)|0;
   HEAP32[$match>>2] = $44;
  }
 }
 $45 = HEAP32[$match>>2]|0;
 HEAP32[$0>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _functionSearch($h,$zFunc) {
 $h = $h|0;
 $zFunc = $zFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $h;
 HEAP32[$2>>2] = $zFunc;
 $3 = HEAP32[$1>>2]|0;
 $4 = (1092 + ($3<<2)|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 while(1) {
  $6 = HEAP32[$p>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   label = 6;
   break;
  }
  $8 = HEAP32[$p>>2]|0;
  $9 = ((($8)) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_sqlite3StrICmp($10,$11)|0);
  $13 = ($12|0)==(0);
  $14 = HEAP32[$p>>2]|0;
  if ($13) {
   label = 4;
   break;
  }
  $15 = ((($14)) + 24|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$p>>2] = $16;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = $14;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $17 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 return (0)|0;
}
function _sqlite3VdbeChangeEncoding($pMem,$desiredEnc) {
 $pMem = $pMem|0;
 $desiredEnc = $desiredEnc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $desiredEnc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 2;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 10|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($12|0)==($13|0);
  if (!($14)) {
   $15 = HEAP32[$1>>2]|0;
   $16 = HEAP32[$2>>2]|0;
   $17 = $16&255;
   $18 = (_sqlite3VdbeMemTranslate($15,$17)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $19;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3VdbeMemTranslate($pMem,$desiredEnc) {
 $pMem = $pMem|0;
 $desiredEnc = $desiredEnc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c2 = 0, $c21 = 0, $len = 0, $or$cond = 0;
 var $or$cond3 = 0, $rc = 0, $temp = 0, $z = 0, $zIn = 0, $zOut = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 45|0;
 $len = sp + 32|0;
 $zOut = sp + 28|0;
 $zIn = sp + 24|0;
 $zTerm = sp + 20|0;
 $z = sp + 16|0;
 $c = sp + 12|0;
 $temp = sp + 44|0;
 $rc = sp + 8|0;
 $c2 = sp + 4|0;
 $c21 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP8[$2>>0] = $desiredEnc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 10|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP8[$2>>0]|0;
  $9 = $8&255;
  $10 = ($9|0)!=(1);
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = (_sqlite3VdbeMemMakeWriteable($11)|0);
   HEAP32[$rc>>2] = $12;
   $13 = HEAP32[$rc>>2]|0;
   $14 = ($13|0)!=(0);
   if ($14) {
    HEAP32[$0>>2] = 7;
    $552 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($552|0);
   }
   $15 = HEAP32[$1>>2]|0;
   $16 = ((($15)) + 16|0);
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$zIn>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = $20 & -2;
   $22 = HEAP32[$zIn>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zTerm>>2] = $23;
   while(1) {
    $24 = HEAP32[$zIn>>2]|0;
    $25 = HEAP32[$zTerm>>2]|0;
    $26 = ($24>>>0)<($25>>>0);
    if (!($26)) {
     break;
    }
    $27 = HEAP32[$zIn>>2]|0;
    $28 = HEAP8[$27>>0]|0;
    HEAP8[$temp>>0] = $28;
    $29 = HEAP32[$zIn>>2]|0;
    $30 = ((($29)) + 1|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = HEAP32[$zIn>>2]|0;
    HEAP8[$32>>0] = $31;
    $33 = HEAP32[$zIn>>2]|0;
    $34 = ((($33)) + 1|0);
    HEAP32[$zIn>>2] = $34;
    $35 = HEAP8[$temp>>0]|0;
    $36 = HEAP32[$zIn>>2]|0;
    $37 = ((($36)) + 1|0);
    HEAP32[$zIn>>2] = $37;
    HEAP8[$36>>0] = $35;
   }
   $38 = HEAP8[$2>>0]|0;
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 10|0);
   HEAP8[$40>>0] = $38;
  } else {
   label = 9;
  }
 } else {
  label = 9;
 }
 if ((label|0) == 9) {
  $41 = HEAP8[$2>>0]|0;
  $42 = $41&255;
  $43 = ($42|0)==(1);
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 12|0);
  $46 = HEAP32[$45>>2]|0;
  if ($43) {
   $47 = $46 & -2;
   HEAP32[$45>>2] = $47;
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $50<<1;
   $52 = (($51) + 1)|0;
   HEAP32[$len>>2] = $52;
  } else {
   $53 = $46<<1;
   $54 = (($53) + 2)|0;
   HEAP32[$len>>2] = $54;
  }
  $55 = HEAP32[$1>>2]|0;
  $56 = ((($55)) + 16|0);
  $57 = HEAP32[$56>>2]|0;
  HEAP32[$zIn>>2] = $57;
  $58 = HEAP32[$1>>2]|0;
  $59 = ((($58)) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$zIn>>2]|0;
  $62 = (($61) + ($60)|0);
  HEAP32[$zTerm>>2] = $62;
  $63 = HEAP32[$1>>2]|0;
  $64 = ((($63)) + 32|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$len>>2]|0;
  $67 = ($66|0)<(0);
  $68 = $67 << 31 >> 31;
  $69 = (_sqlite3DbMallocRaw($65,$66,$68)|0);
  HEAP32[$zOut>>2] = $69;
  $70 = HEAP32[$zOut>>2]|0;
  $71 = ($70|0)!=(0|0);
  if (!($71)) {
   HEAP32[$0>>2] = 7;
   $552 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($552|0);
  }
  $72 = HEAP32[$zOut>>2]|0;
  HEAP32[$z>>2] = $72;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 10|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  $77 = ($76|0)==(1);
  if ($77) {
   $78 = HEAP8[$2>>0]|0;
   $79 = $78&255;
   $80 = ($79|0)==(2);
   L24: do {
    if ($80) {
     while(1) {
      $81 = HEAP32[$zIn>>2]|0;
      $82 = HEAP32[$zTerm>>2]|0;
      $83 = ($81>>>0)<($82>>>0);
      if (!($83)) {
       break L24;
      }
      $84 = HEAP32[$zIn>>2]|0;
      $85 = ((($84)) + 1|0);
      HEAP32[$zIn>>2] = $85;
      $86 = HEAP8[$84>>0]|0;
      $87 = $86&255;
      HEAP32[$c>>2] = $87;
      $88 = HEAP32[$c>>2]|0;
      $89 = ($88>>>0)>=(192);
      do {
       if ($89) {
        $90 = HEAP32[$c>>2]|0;
        $91 = (($90) - 192)|0;
        $92 = (25563 + ($91)|0);
        $93 = HEAP8[$92>>0]|0;
        $94 = $93&255;
        HEAP32[$c>>2] = $94;
        while(1) {
         $95 = HEAP32[$zIn>>2]|0;
         $96 = HEAP32[$zTerm>>2]|0;
         $97 = ($95|0)!=($96|0);
         if ($97) {
          $98 = HEAP32[$zIn>>2]|0;
          $99 = HEAP8[$98>>0]|0;
          $100 = $99&255;
          $101 = $100 & 192;
          $102 = ($101|0)==(128);
          $553 = $102;
         } else {
          $553 = 0;
         }
         $103 = HEAP32[$c>>2]|0;
         if (!($553)) {
          break;
         }
         $104 = $103 << 6;
         $105 = HEAP32[$zIn>>2]|0;
         $106 = ((($105)) + 1|0);
         HEAP32[$zIn>>2] = $106;
         $107 = HEAP8[$105>>0]|0;
         $108 = $107&255;
         $109 = 63 & $108;
         $110 = (($104) + ($109))|0;
         HEAP32[$c>>2] = $110;
        }
        $111 = ($103>>>0)<(128);
        if (!($111)) {
         $112 = HEAP32[$c>>2]|0;
         $113 = $112 & -2048;
         $114 = ($113|0)==(55296);
         if (!($114)) {
          $115 = HEAP32[$c>>2]|0;
          $116 = $115 & -2;
          $117 = ($116|0)==(65534);
          if (!($117)) {
           break;
          }
         }
        }
        HEAP32[$c>>2] = 65533;
       }
      } while(0);
      $118 = HEAP32[$c>>2]|0;
      $119 = ($118>>>0)<=(65535);
      $120 = HEAP32[$c>>2]|0;
      if ($119) {
       $121 = $120 & 255;
       $122 = $121&255;
       $123 = HEAP32[$z>>2]|0;
       $124 = ((($123)) + 1|0);
       HEAP32[$z>>2] = $124;
       HEAP8[$123>>0] = $122;
       $125 = HEAP32[$c>>2]|0;
       $126 = $125 >>> 8;
       $127 = $126 & 255;
       $128 = $127&255;
       $129 = HEAP32[$z>>2]|0;
       $130 = ((($129)) + 1|0);
       HEAP32[$z>>2] = $130;
       HEAP8[$129>>0] = $128;
       continue;
      } else {
       $131 = $120 >>> 10;
       $132 = $131 & 63;
       $133 = HEAP32[$c>>2]|0;
       $134 = (($133) - 65536)|0;
       $135 = $134 >>> 10;
       $136 = $135 & 192;
       $137 = (($132) + ($136))|0;
       $138 = $137&255;
       $139 = HEAP32[$z>>2]|0;
       $140 = ((($139)) + 1|0);
       HEAP32[$z>>2] = $140;
       HEAP8[$139>>0] = $138;
       $141 = HEAP32[$c>>2]|0;
       $142 = (($141) - 65536)|0;
       $143 = $142 >>> 18;
       $144 = $143 & 3;
       $145 = (216 + ($144))|0;
       $146 = $145&255;
       $147 = HEAP32[$z>>2]|0;
       $148 = ((($147)) + 1|0);
       HEAP32[$z>>2] = $148;
       HEAP8[$147>>0] = $146;
       $149 = HEAP32[$c>>2]|0;
       $150 = $149 & 255;
       $151 = $150&255;
       $152 = HEAP32[$z>>2]|0;
       $153 = ((($152)) + 1|0);
       HEAP32[$z>>2] = $153;
       HEAP8[$152>>0] = $151;
       $154 = HEAP32[$c>>2]|0;
       $155 = $154 >>> 8;
       $156 = $155 & 3;
       $157 = (220 + ($156))|0;
       $158 = $157&255;
       $159 = HEAP32[$z>>2]|0;
       $160 = ((($159)) + 1|0);
       HEAP32[$z>>2] = $160;
       HEAP8[$159>>0] = $158;
       continue;
      }
     }
    } else {
     while(1) {
      $161 = HEAP32[$zIn>>2]|0;
      $162 = HEAP32[$zTerm>>2]|0;
      $163 = ($161>>>0)<($162>>>0);
      if (!($163)) {
       break L24;
      }
      $164 = HEAP32[$zIn>>2]|0;
      $165 = ((($164)) + 1|0);
      HEAP32[$zIn>>2] = $165;
      $166 = HEAP8[$164>>0]|0;
      $167 = $166&255;
      HEAP32[$c>>2] = $167;
      $168 = HEAP32[$c>>2]|0;
      $169 = ($168>>>0)>=(192);
      do {
       if ($169) {
        $170 = HEAP32[$c>>2]|0;
        $171 = (($170) - 192)|0;
        $172 = (25563 + ($171)|0);
        $173 = HEAP8[$172>>0]|0;
        $174 = $173&255;
        HEAP32[$c>>2] = $174;
        while(1) {
         $175 = HEAP32[$zIn>>2]|0;
         $176 = HEAP32[$zTerm>>2]|0;
         $177 = ($175|0)!=($176|0);
         if ($177) {
          $178 = HEAP32[$zIn>>2]|0;
          $179 = HEAP8[$178>>0]|0;
          $180 = $179&255;
          $181 = $180 & 192;
          $182 = ($181|0)==(128);
          $554 = $182;
         } else {
          $554 = 0;
         }
         $183 = HEAP32[$c>>2]|0;
         if (!($554)) {
          break;
         }
         $184 = $183 << 6;
         $185 = HEAP32[$zIn>>2]|0;
         $186 = ((($185)) + 1|0);
         HEAP32[$zIn>>2] = $186;
         $187 = HEAP8[$185>>0]|0;
         $188 = $187&255;
         $189 = 63 & $188;
         $190 = (($184) + ($189))|0;
         HEAP32[$c>>2] = $190;
        }
        $191 = ($183>>>0)<(128);
        if (!($191)) {
         $192 = HEAP32[$c>>2]|0;
         $193 = $192 & -2048;
         $194 = ($193|0)==(55296);
         if (!($194)) {
          $195 = HEAP32[$c>>2]|0;
          $196 = $195 & -2;
          $197 = ($196|0)==(65534);
          if (!($197)) {
           break;
          }
         }
        }
        HEAP32[$c>>2] = 65533;
       }
      } while(0);
      $198 = HEAP32[$c>>2]|0;
      $199 = ($198>>>0)<=(65535);
      $200 = HEAP32[$c>>2]|0;
      if ($199) {
       $201 = $200 >>> 8;
       $202 = $201 & 255;
       $203 = $202&255;
       $204 = HEAP32[$z>>2]|0;
       $205 = ((($204)) + 1|0);
       HEAP32[$z>>2] = $205;
       HEAP8[$204>>0] = $203;
       $206 = HEAP32[$c>>2]|0;
       $207 = $206 & 255;
       $208 = $207&255;
       $209 = HEAP32[$z>>2]|0;
       $210 = ((($209)) + 1|0);
       HEAP32[$z>>2] = $210;
       HEAP8[$209>>0] = $208;
       continue;
      } else {
       $211 = (($200) - 65536)|0;
       $212 = $211 >>> 18;
       $213 = $212 & 3;
       $214 = (216 + ($213))|0;
       $215 = $214&255;
       $216 = HEAP32[$z>>2]|0;
       $217 = ((($216)) + 1|0);
       HEAP32[$z>>2] = $217;
       HEAP8[$216>>0] = $215;
       $218 = HEAP32[$c>>2]|0;
       $219 = $218 >>> 10;
       $220 = $219 & 63;
       $221 = HEAP32[$c>>2]|0;
       $222 = (($221) - 65536)|0;
       $223 = $222 >>> 10;
       $224 = $223 & 192;
       $225 = (($220) + ($224))|0;
       $226 = $225&255;
       $227 = HEAP32[$z>>2]|0;
       $228 = ((($227)) + 1|0);
       HEAP32[$z>>2] = $228;
       HEAP8[$227>>0] = $226;
       $229 = HEAP32[$c>>2]|0;
       $230 = $229 >>> 8;
       $231 = $230 & 3;
       $232 = (220 + ($231))|0;
       $233 = $232&255;
       $234 = HEAP32[$z>>2]|0;
       $235 = ((($234)) + 1|0);
       HEAP32[$z>>2] = $235;
       HEAP8[$234>>0] = $233;
       $236 = HEAP32[$c>>2]|0;
       $237 = $236 & 255;
       $238 = $237&255;
       $239 = HEAP32[$z>>2]|0;
       $240 = ((($239)) + 1|0);
       HEAP32[$z>>2] = $240;
       HEAP8[$239>>0] = $238;
       continue;
      }
     }
    }
   } while(0);
   $241 = HEAP32[$z>>2]|0;
   $242 = HEAP32[$zOut>>2]|0;
   $243 = $241;
   $244 = $242;
   $245 = (($243) - ($244))|0;
   $246 = HEAP32[$1>>2]|0;
   $247 = ((($246)) + 12|0);
   HEAP32[$247>>2] = $245;
   $248 = HEAP32[$z>>2]|0;
   $249 = ((($248)) + 1|0);
   HEAP32[$z>>2] = $249;
   HEAP8[$248>>0] = 0;
  } else {
   $250 = HEAP32[$1>>2]|0;
   $251 = ((($250)) + 10|0);
   $252 = HEAP8[$251>>0]|0;
   $253 = $252&255;
   $254 = ($253|0)==(2);
   L67: do {
    if ($254) {
     while(1) {
      $255 = HEAP32[$zIn>>2]|0;
      $256 = HEAP32[$zTerm>>2]|0;
      $257 = ($255>>>0)<($256>>>0);
      if (!($257)) {
       break L67;
      }
      $258 = HEAP32[$zIn>>2]|0;
      $259 = ((($258)) + 1|0);
      HEAP32[$zIn>>2] = $259;
      $260 = HEAP8[$258>>0]|0;
      $261 = $260&255;
      HEAP32[$c>>2] = $261;
      $262 = HEAP32[$zIn>>2]|0;
      $263 = ((($262)) + 1|0);
      HEAP32[$zIn>>2] = $263;
      $264 = HEAP8[$262>>0]|0;
      $265 = $264&255;
      $266 = $265 << 8;
      $267 = HEAP32[$c>>2]|0;
      $268 = (($267) + ($266))|0;
      HEAP32[$c>>2] = $268;
      $269 = HEAP32[$c>>2]|0;
      $270 = ($269>>>0)>=(55296);
      $271 = HEAP32[$c>>2]|0;
      $272 = ($271>>>0)<(57344);
      $or$cond = $270 & $272;
      if ($or$cond) {
       $273 = HEAP32[$zIn>>2]|0;
       $274 = HEAP32[$zTerm>>2]|0;
       $275 = ($273>>>0)<($274>>>0);
       if ($275) {
        $276 = HEAP32[$zIn>>2]|0;
        $277 = ((($276)) + 1|0);
        HEAP32[$zIn>>2] = $277;
        $278 = HEAP8[$276>>0]|0;
        $279 = $278&255;
        HEAP32[$c2>>2] = $279;
        $280 = HEAP32[$zIn>>2]|0;
        $281 = ((($280)) + 1|0);
        HEAP32[$zIn>>2] = $281;
        $282 = HEAP8[$280>>0]|0;
        $283 = $282&255;
        $284 = $283 << 8;
        $285 = HEAP32[$c2>>2]|0;
        $286 = (($285) + ($284))|0;
        HEAP32[$c2>>2] = $286;
        $287 = HEAP32[$c2>>2]|0;
        $288 = $287 & 1023;
        $289 = HEAP32[$c>>2]|0;
        $290 = $289 & 63;
        $291 = $290 << 10;
        $292 = (($288) + ($291))|0;
        $293 = HEAP32[$c>>2]|0;
        $294 = $293 & 960;
        $295 = (($294) + 64)|0;
        $296 = $295 << 10;
        $297 = (($292) + ($296))|0;
        HEAP32[$c>>2] = $297;
       }
      }
      $298 = HEAP32[$c>>2]|0;
      $299 = ($298>>>0)<(128);
      $300 = HEAP32[$c>>2]|0;
      if ($299) {
       $301 = $300 & 255;
       $302 = $301&255;
       $303 = HEAP32[$z>>2]|0;
       $304 = ((($303)) + 1|0);
       HEAP32[$z>>2] = $304;
       HEAP8[$303>>0] = $302;
       continue;
      }
      $305 = ($300>>>0)<(2048);
      $306 = HEAP32[$c>>2]|0;
      if ($305) {
       $307 = $306 >>> 6;
       $308 = $307 & 31;
       $309 = $308&255;
       $310 = $309&255;
       $311 = (192 + ($310))|0;
       $312 = $311&255;
       $313 = HEAP32[$z>>2]|0;
       $314 = ((($313)) + 1|0);
       HEAP32[$z>>2] = $314;
       HEAP8[$313>>0] = $312;
       $315 = HEAP32[$c>>2]|0;
       $316 = $315 & 63;
       $317 = $316&255;
       $318 = $317&255;
       $319 = (128 + ($318))|0;
       $320 = $319&255;
       $321 = HEAP32[$z>>2]|0;
       $322 = ((($321)) + 1|0);
       HEAP32[$z>>2] = $322;
       HEAP8[$321>>0] = $320;
       continue;
      }
      $323 = ($306>>>0)<(65536);
      $324 = HEAP32[$c>>2]|0;
      if ($323) {
       $325 = $324 >>> 12;
       $326 = $325 & 15;
       $327 = $326&255;
       $328 = $327&255;
       $329 = (224 + ($328))|0;
       $330 = $329&255;
       $331 = HEAP32[$z>>2]|0;
       $332 = ((($331)) + 1|0);
       HEAP32[$z>>2] = $332;
       HEAP8[$331>>0] = $330;
       $333 = HEAP32[$c>>2]|0;
       $334 = $333 >>> 6;
       $335 = $334 & 63;
       $336 = $335&255;
       $337 = $336&255;
       $338 = (128 + ($337))|0;
       $339 = $338&255;
       $340 = HEAP32[$z>>2]|0;
       $341 = ((($340)) + 1|0);
       HEAP32[$z>>2] = $341;
       HEAP8[$340>>0] = $339;
       $342 = HEAP32[$c>>2]|0;
       $343 = $342 & 63;
       $344 = $343&255;
       $345 = $344&255;
       $346 = (128 + ($345))|0;
       $347 = $346&255;
       $348 = HEAP32[$z>>2]|0;
       $349 = ((($348)) + 1|0);
       HEAP32[$z>>2] = $349;
       HEAP8[$348>>0] = $347;
       continue;
      } else {
       $350 = $324 >>> 18;
       $351 = $350 & 7;
       $352 = $351&255;
       $353 = $352&255;
       $354 = (240 + ($353))|0;
       $355 = $354&255;
       $356 = HEAP32[$z>>2]|0;
       $357 = ((($356)) + 1|0);
       HEAP32[$z>>2] = $357;
       HEAP8[$356>>0] = $355;
       $358 = HEAP32[$c>>2]|0;
       $359 = $358 >>> 12;
       $360 = $359 & 63;
       $361 = $360&255;
       $362 = $361&255;
       $363 = (128 + ($362))|0;
       $364 = $363&255;
       $365 = HEAP32[$z>>2]|0;
       $366 = ((($365)) + 1|0);
       HEAP32[$z>>2] = $366;
       HEAP8[$365>>0] = $364;
       $367 = HEAP32[$c>>2]|0;
       $368 = $367 >>> 6;
       $369 = $368 & 63;
       $370 = $369&255;
       $371 = $370&255;
       $372 = (128 + ($371))|0;
       $373 = $372&255;
       $374 = HEAP32[$z>>2]|0;
       $375 = ((($374)) + 1|0);
       HEAP32[$z>>2] = $375;
       HEAP8[$374>>0] = $373;
       $376 = HEAP32[$c>>2]|0;
       $377 = $376 & 63;
       $378 = $377&255;
       $379 = $378&255;
       $380 = (128 + ($379))|0;
       $381 = $380&255;
       $382 = HEAP32[$z>>2]|0;
       $383 = ((($382)) + 1|0);
       HEAP32[$z>>2] = $383;
       HEAP8[$382>>0] = $381;
       continue;
      }
     }
    } else {
     while(1) {
      $384 = HEAP32[$zIn>>2]|0;
      $385 = HEAP32[$zTerm>>2]|0;
      $386 = ($384>>>0)<($385>>>0);
      if (!($386)) {
       break L67;
      }
      $387 = HEAP32[$zIn>>2]|0;
      $388 = ((($387)) + 1|0);
      HEAP32[$zIn>>2] = $388;
      $389 = HEAP8[$387>>0]|0;
      $390 = $389&255;
      $391 = $390 << 8;
      HEAP32[$c>>2] = $391;
      $392 = HEAP32[$zIn>>2]|0;
      $393 = ((($392)) + 1|0);
      HEAP32[$zIn>>2] = $393;
      $394 = HEAP8[$392>>0]|0;
      $395 = $394&255;
      $396 = HEAP32[$c>>2]|0;
      $397 = (($396) + ($395))|0;
      HEAP32[$c>>2] = $397;
      $398 = HEAP32[$c>>2]|0;
      $399 = ($398>>>0)>=(55296);
      $400 = HEAP32[$c>>2]|0;
      $401 = ($400>>>0)<(57344);
      $or$cond3 = $399 & $401;
      if ($or$cond3) {
       $402 = HEAP32[$zIn>>2]|0;
       $403 = HEAP32[$zTerm>>2]|0;
       $404 = ($402>>>0)<($403>>>0);
       if ($404) {
        $405 = HEAP32[$zIn>>2]|0;
        $406 = ((($405)) + 1|0);
        HEAP32[$zIn>>2] = $406;
        $407 = HEAP8[$405>>0]|0;
        $408 = $407&255;
        $409 = $408 << 8;
        HEAP32[$c21>>2] = $409;
        $410 = HEAP32[$zIn>>2]|0;
        $411 = ((($410)) + 1|0);
        HEAP32[$zIn>>2] = $411;
        $412 = HEAP8[$410>>0]|0;
        $413 = $412&255;
        $414 = HEAP32[$c21>>2]|0;
        $415 = (($414) + ($413))|0;
        HEAP32[$c21>>2] = $415;
        $416 = HEAP32[$c21>>2]|0;
        $417 = $416 & 1023;
        $418 = HEAP32[$c>>2]|0;
        $419 = $418 & 63;
        $420 = $419 << 10;
        $421 = (($417) + ($420))|0;
        $422 = HEAP32[$c>>2]|0;
        $423 = $422 & 960;
        $424 = (($423) + 64)|0;
        $425 = $424 << 10;
        $426 = (($421) + ($425))|0;
        HEAP32[$c>>2] = $426;
       }
      }
      $427 = HEAP32[$c>>2]|0;
      $428 = ($427>>>0)<(128);
      $429 = HEAP32[$c>>2]|0;
      if ($428) {
       $430 = $429 & 255;
       $431 = $430&255;
       $432 = HEAP32[$z>>2]|0;
       $433 = ((($432)) + 1|0);
       HEAP32[$z>>2] = $433;
       HEAP8[$432>>0] = $431;
       continue;
      }
      $434 = ($429>>>0)<(2048);
      $435 = HEAP32[$c>>2]|0;
      if ($434) {
       $436 = $435 >>> 6;
       $437 = $436 & 31;
       $438 = $437&255;
       $439 = $438&255;
       $440 = (192 + ($439))|0;
       $441 = $440&255;
       $442 = HEAP32[$z>>2]|0;
       $443 = ((($442)) + 1|0);
       HEAP32[$z>>2] = $443;
       HEAP8[$442>>0] = $441;
       $444 = HEAP32[$c>>2]|0;
       $445 = $444 & 63;
       $446 = $445&255;
       $447 = $446&255;
       $448 = (128 + ($447))|0;
       $449 = $448&255;
       $450 = HEAP32[$z>>2]|0;
       $451 = ((($450)) + 1|0);
       HEAP32[$z>>2] = $451;
       HEAP8[$450>>0] = $449;
       continue;
      }
      $452 = ($435>>>0)<(65536);
      $453 = HEAP32[$c>>2]|0;
      if ($452) {
       $454 = $453 >>> 12;
       $455 = $454 & 15;
       $456 = $455&255;
       $457 = $456&255;
       $458 = (224 + ($457))|0;
       $459 = $458&255;
       $460 = HEAP32[$z>>2]|0;
       $461 = ((($460)) + 1|0);
       HEAP32[$z>>2] = $461;
       HEAP8[$460>>0] = $459;
       $462 = HEAP32[$c>>2]|0;
       $463 = $462 >>> 6;
       $464 = $463 & 63;
       $465 = $464&255;
       $466 = $465&255;
       $467 = (128 + ($466))|0;
       $468 = $467&255;
       $469 = HEAP32[$z>>2]|0;
       $470 = ((($469)) + 1|0);
       HEAP32[$z>>2] = $470;
       HEAP8[$469>>0] = $468;
       $471 = HEAP32[$c>>2]|0;
       $472 = $471 & 63;
       $473 = $472&255;
       $474 = $473&255;
       $475 = (128 + ($474))|0;
       $476 = $475&255;
       $477 = HEAP32[$z>>2]|0;
       $478 = ((($477)) + 1|0);
       HEAP32[$z>>2] = $478;
       HEAP8[$477>>0] = $476;
       continue;
      } else {
       $479 = $453 >>> 18;
       $480 = $479 & 7;
       $481 = $480&255;
       $482 = $481&255;
       $483 = (240 + ($482))|0;
       $484 = $483&255;
       $485 = HEAP32[$z>>2]|0;
       $486 = ((($485)) + 1|0);
       HEAP32[$z>>2] = $486;
       HEAP8[$485>>0] = $484;
       $487 = HEAP32[$c>>2]|0;
       $488 = $487 >>> 12;
       $489 = $488 & 63;
       $490 = $489&255;
       $491 = $490&255;
       $492 = (128 + ($491))|0;
       $493 = $492&255;
       $494 = HEAP32[$z>>2]|0;
       $495 = ((($494)) + 1|0);
       HEAP32[$z>>2] = $495;
       HEAP8[$494>>0] = $493;
       $496 = HEAP32[$c>>2]|0;
       $497 = $496 >>> 6;
       $498 = $497 & 63;
       $499 = $498&255;
       $500 = $499&255;
       $501 = (128 + ($500))|0;
       $502 = $501&255;
       $503 = HEAP32[$z>>2]|0;
       $504 = ((($503)) + 1|0);
       HEAP32[$z>>2] = $504;
       HEAP8[$503>>0] = $502;
       $505 = HEAP32[$c>>2]|0;
       $506 = $505 & 63;
       $507 = $506&255;
       $508 = $507&255;
       $509 = (128 + ($508))|0;
       $510 = $509&255;
       $511 = HEAP32[$z>>2]|0;
       $512 = ((($511)) + 1|0);
       HEAP32[$z>>2] = $512;
       HEAP8[$511>>0] = $510;
       continue;
      }
     }
    }
   } while(0);
   $513 = HEAP32[$z>>2]|0;
   $514 = HEAP32[$zOut>>2]|0;
   $515 = $513;
   $516 = $514;
   $517 = (($515) - ($516))|0;
   $518 = HEAP32[$1>>2]|0;
   $519 = ((($518)) + 12|0);
   HEAP32[$519>>2] = $517;
  }
  $520 = HEAP32[$z>>2]|0;
  HEAP8[$520>>0] = 0;
  $521 = HEAP32[$1>>2]|0;
  $522 = ((($521)) + 8|0);
  $523 = HEAP16[$522>>1]|0;
  $524 = $523&65535;
  HEAP32[$c>>2] = $524;
  $525 = HEAP32[$1>>2]|0;
  _sqlite3VdbeMemRelease($525);
  $526 = HEAP32[$c>>2]|0;
  $527 = $526 & 32799;
  $528 = 514 | $527;
  $529 = $528&65535;
  $530 = HEAP32[$1>>2]|0;
  $531 = ((($530)) + 8|0);
  HEAP16[$531>>1] = $529;
  $532 = HEAP8[$2>>0]|0;
  $533 = HEAP32[$1>>2]|0;
  $534 = ((($533)) + 10|0);
  HEAP8[$534>>0] = $532;
  $535 = HEAP32[$zOut>>2]|0;
  $536 = HEAP32[$1>>2]|0;
  $537 = ((($536)) + 16|0);
  HEAP32[$537>>2] = $535;
  $538 = HEAP32[$1>>2]|0;
  $539 = ((($538)) + 16|0);
  $540 = HEAP32[$539>>2]|0;
  $541 = HEAP32[$1>>2]|0;
  $542 = ((($541)) + 20|0);
  HEAP32[$542>>2] = $540;
  $543 = HEAP32[$1>>2]|0;
  $544 = ((($543)) + 32|0);
  $545 = HEAP32[$544>>2]|0;
  $546 = HEAP32[$1>>2]|0;
  $547 = ((($546)) + 16|0);
  $548 = HEAP32[$547>>2]|0;
  $549 = (_sqlite3DbMallocSize($545,$548)|0);
  $550 = HEAP32[$1>>2]|0;
  $551 = ((($550)) + 24|0);
  HEAP32[$551>>2] = $549;
 }
 HEAP32[$0>>2] = 0;
 $552 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($552|0);
}
function _sqlite3DbMallocSize($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = (_isLookaside($5,$6)|0);
  $8 = ($7|0)!=(0);
  if ($8) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 256|0);
   $14 = ((($13)) + 4|0);
   $15 = HEAP16[$14>>1]|0;
   $16 = $15&65535;
   HEAP32[$0>>2] = $16;
   $17 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($17|0);
  }
 }
 $9 = HEAP32[(60)>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (FUNCTION_TABLE_ii[$9 & 255]($10)|0);
 HEAP32[$0>>2] = $11;
 $17 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _isLookaside($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 256|0);
 $6 = ((($5)) + 32|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7;
 $9 = ($3>>>0)>=($8>>>0);
 if (!($9)) {
  $19 = 0;
  $18 = $19&1;
  STACKTOP = sp;return ($18|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 256|0);
 $14 = ((($13)) + 36|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15;
 $17 = ($11>>>0)<($16>>>0);
 $19 = $17;
 $18 = $19&1;
 STACKTOP = sp;return ($18|0);
}
function _functionDestroy($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pDestructor = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pDestructor = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pDestructor>>2] = $4;
 $5 = HEAP32[$pDestructor>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$pDestructor>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$pDestructor>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$pDestructor>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$pDestructor>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 FUNCTION_TABLE_vi[$15 & 255]($18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$pDestructor>>2]|0;
 _sqlite3DbFree($19,$20);
 STACKTOP = sp;return;
}
function _disconnectAllVtab($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pMod = 0, $pSchema = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $i = sp + 16|0;
 $p = sp + 12|0;
 $pSchema = sp + 8|0;
 $pTab = sp + 4|0;
 $pMod = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($1);
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 20|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($2|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 16|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + ($7<<4)|0);
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pSchema>>2] = $13;
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<4)|0);
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)!=(0|0);
  L4: do {
   if ($21) {
    $22 = HEAP32[$pSchema>>2]|0;
    $23 = ((($22)) + 8|0);
    $24 = ((($23)) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    HEAP32[$p>>2] = $25;
    while(1) {
     $26 = HEAP32[$p>>2]|0;
     $27 = ($26|0)!=(0|0);
     if (!($27)) {
      break L4;
     }
     $28 = HEAP32[$p>>2]|0;
     $29 = ((($28)) + 8|0);
     $30 = HEAP32[$29>>2]|0;
     HEAP32[$pTab>>2] = $30;
     $31 = HEAP32[$pTab>>2]|0;
     $32 = ((($31)) + 42|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33&255;
     $35 = $34 & 16;
     $36 = ($35|0)!=(0);
     if ($36) {
      $37 = HEAP32[$0>>2]|0;
      $38 = HEAP32[$pTab>>2]|0;
      _sqlite3VtabDisconnect($37,$38);
     }
     $39 = HEAP32[$p>>2]|0;
     $40 = HEAP32[$39>>2]|0;
     HEAP32[$p>>2] = $40;
    }
   }
  } while(0);
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = ((($43)) + 320|0);
 $45 = ((($44)) + 8|0);
 $46 = HEAP32[$45>>2]|0;
 HEAP32[$p>>2] = $46;
 while(1) {
  $47 = HEAP32[$p>>2]|0;
  $48 = ($47|0)!=(0|0);
  if (!($48)) {
   break;
  }
  $49 = HEAP32[$p>>2]|0;
  $50 = ((($49)) + 8|0);
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$pMod>>2] = $51;
  $52 = HEAP32[$pMod>>2]|0;
  $53 = ((($52)) + 16|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0|0);
  if ($55) {
   $56 = HEAP32[$0>>2]|0;
   $57 = HEAP32[$pMod>>2]|0;
   $58 = ((($57)) + 16|0);
   $59 = HEAP32[$58>>2]|0;
   _sqlite3VtabDisconnect($56,$59);
  }
  $60 = HEAP32[$p>>2]|0;
  $61 = HEAP32[$60>>2]|0;
  HEAP32[$p>>2] = $61;
 }
 $62 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($62);
 STACKTOP = sp;return;
}
function _sqlite3VtabRollback($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _callFinaliser($1,68);
 STACKTOP = sp;return 0;
}
function _connectionIsBusy($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $j = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $db;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($6|0)<($9|0);
  if (!($10)) {
   label = 9;
   break;
  }
  $11 = HEAP32[$j>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<4)|0);
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pBt>>2] = $17;
  $18 = HEAP32[$pBt>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pBt>>2]|0;
   $21 = (_sqlite3BtreeIsInBackup($20)|0);
   $22 = ($21|0)!=(0);
   if ($22) {
    label = 7;
    break;
   }
  }
  $23 = HEAP32[$j>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$j>>2] = $24;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 else if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _sqlite3BtreeIsInBackup($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 $5 = $4&1;
 STACKTOP = sp;return ($5|0);
}
function _callFinaliser($db,$offset) {
 $db = $db|0;
 $offset = $offset|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $aVTrans = 0, $i = 0, $p = 0, $pVTab = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $i = sp + 16|0;
 $aVTrans = sp + 12|0;
 $pVTab = sp + 8|0;
 $p = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $offset;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 340|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 340|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$aVTrans>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 340|0);
 HEAP32[$10>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 316|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$aVTrans>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVTab>>2] = $19;
  $20 = HEAP32[$pVTab>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$p>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = (($26) + ($27)|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$x>>2] = $29;
   $30 = HEAP32[$x>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$x>>2]|0;
    $33 = HEAP32[$p>>2]|0;
    (FUNCTION_TABLE_ii[$32 & 255]($33)|0);
   }
  }
  $34 = HEAP32[$pVTab>>2]|0;
  $35 = ((($34)) + 20|0);
  HEAP32[$35>>2] = 0;
  $36 = HEAP32[$pVTab>>2]|0;
  _sqlite3VtabUnlock($36);
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$aVTrans>>2]|0;
 _sqlite3DbFree($39,$40);
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 316|0);
 HEAP32[$42>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VtabUnlock($pVTab) {
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pVTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + -1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)!=(0|0);
 if ($15) {
  $16 = HEAP32[$p>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  (FUNCTION_TABLE_ii[$19 & 255]($20)|0);
 }
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($21,$22);
 STACKTOP = sp;return;
}
function _sqlite3VtabDisconnect($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pVTab = 0, $ppVTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $ppVTab = sp + 4|0;
 $pVTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 HEAP32[$ppVTab>>2] = $3;
 while(1) {
  $4 = HEAP32[$ppVTab>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$ppVTab>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ($9|0)==($10|0);
  $12 = HEAP32[$ppVTab>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  if ($11) {
   break;
  }
  $19 = ((($13)) + 24|0);
  HEAP32[$ppVTab>>2] = $19;
 }
 if ((label|0) == 6) {
  STACKTOP = sp;return;
 }
 HEAP32[$pVTab>>2] = $13;
 $14 = HEAP32[$pVTab>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$ppVTab>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$pVTab>>2]|0;
 _sqlite3VtabUnlock($18);
 STACKTOP = sp;return;
}
function _sqlite3VtabUnlockList($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 344|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 344|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 _sqlite3ExpirePreparedStatements($8);
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 24|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pNext>>2] = $11;
  $12 = HEAP32[$p>>2]|0;
  _sqlite3VtabUnlock($12);
  $13 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $13;
  $14 = HEAP32[$p>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _sqlite3PcacheShrink($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[(164)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 FUNCTION_TABLE_vi[$1 & 255]($4);
 STACKTOP = sp;return;
}
function _pcache1Init($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 dest=1628; stop=dest+100|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $1 = HEAP32[(212)>>2]|0;
 $2 = ($1|0)==(0|0);
 $3 = $2&1;
 HEAP32[(1684)>>2] = $3;
 $4 = HEAP32[(1684)>>2]|0;
 $5 = ($4|0)!=(0);
 $6 = HEAP32[(220)>>2]|0;
 $7 = ($6|0)!=(0);
 $or$cond = $5 & $7;
 $8 = HEAP32[(212)>>2]|0;
 $9 = ($8|0)==(0|0);
 $or$cond3 = $or$cond & $9;
 if ($or$cond3) {
  $10 = HEAP32[(220)>>2]|0;
  HEAP32[(1688)>>2] = $10;
  HEAP32[(1640)>>2] = 10;
  HEAP32[(1680)>>2] = 1;
  STACKTOP = sp;return 0;
 } else {
  HEAP32[(1688)>>2] = 0;
  HEAP32[(1640)>>2] = 10;
  HEAP32[(1680)>>2] = 1;
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _pcache1Shutdown($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 dest=1628; stop=dest+100|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _pcache1Create($szPage,$szExtra,$bPurgeable) {
 $szPage = $szPage|0;
 $szExtra = $szExtra|0;
 $bPurgeable = $bPurgeable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $pCache = 0, $pGroup = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pCache = sp + 8|0;
 $pGroup = sp + 4|0;
 $sz = sp;
 HEAP32[$0>>2] = $szPage;
 HEAP32[$1>>2] = $szExtra;
 HEAP32[$2>>2] = $bPurgeable;
 $3 = HEAP32[(1684)>>2]|0;
 $4 = ($3*52)|0;
 $5 = (60 + ($4))|0;
 HEAP32[$sz>>2] = $5;
 $6 = HEAP32[$sz>>2]|0;
 $7 = ($6|0)<(0);
 $8 = $7 << 31 >> 31;
 $9 = (_sqlite3MallocZero($6,$8)|0);
 HEAP32[$pCache>>2] = $9;
 $10 = HEAP32[$pCache>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $81 = HEAP32[$pCache>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $12 = HEAP32[(1684)>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$pCache>>2]|0;
  $15 = ((($14)) + 60|0);
  HEAP32[$pGroup>>2] = $15;
  $16 = HEAP32[$pGroup>>2]|0;
  $17 = ((($16)) + 12|0);
  HEAP32[$17>>2] = 10;
 } else {
  HEAP32[$pGroup>>2] = 1628;
 }
 $18 = HEAP32[$pGroup>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = ((($19)) + 14|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = HEAP32[$pGroup>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = ((($25)) + 14|0);
  HEAP8[$26>>0] = 1;
  $27 = HEAP32[$pGroup>>2]|0;
  $28 = ((($27)) + 20|0);
  $29 = HEAP32[$pGroup>>2]|0;
  $30 = ((($29)) + 20|0);
  $31 = ((($30)) + 24|0);
  HEAP32[$31>>2] = $28;
  $32 = HEAP32[$pGroup>>2]|0;
  $33 = ((($32)) + 20|0);
  $34 = ((($33)) + 28|0);
  HEAP32[$34>>2] = $28;
 }
 $35 = HEAP32[$pGroup>>2]|0;
 $36 = HEAP32[$pCache>>2]|0;
 HEAP32[$36>>2] = $35;
 $37 = HEAP32[$0>>2]|0;
 $38 = HEAP32[$pCache>>2]|0;
 $39 = ((($38)) + 4|0);
 HEAP32[$39>>2] = $37;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$pCache>>2]|0;
 $42 = ((($41)) + 8|0);
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = (($43) + ($44))|0;
 $46 = (($45) + 32)|0;
 $47 = HEAP32[$pCache>>2]|0;
 $48 = ((($47)) + 12|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP32[$2>>2]|0;
 $50 = ($49|0)!=(0);
 $51 = $50 ? 1 : 0;
 $52 = HEAP32[$pCache>>2]|0;
 $53 = ((($52)) + 16|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$pCache>>2]|0;
 _pcache1ResizeHash($54);
 $55 = HEAP32[$2>>2]|0;
 $56 = ($55|0)!=(0);
 if ($56) {
  $57 = HEAP32[$pCache>>2]|0;
  $58 = ((($57)) + 20|0);
  HEAP32[$58>>2] = 10;
  $59 = HEAP32[$pCache>>2]|0;
  $60 = ((($59)) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$pGroup>>2]|0;
  $63 = ((($62)) + 8|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (($64) + ($61))|0;
  HEAP32[$63>>2] = $65;
  $66 = HEAP32[$pGroup>>2]|0;
  $67 = ((($66)) + 4|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + 10)|0;
  $70 = HEAP32[$pGroup>>2]|0;
  $71 = ((($70)) + 8|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($69) - ($72))|0;
  $74 = HEAP32[$pGroup>>2]|0;
  $75 = ((($74)) + 12|0);
  HEAP32[$75>>2] = $73;
 }
 $76 = HEAP32[$pCache>>2]|0;
 $77 = ((($76)) + 44|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = ($78|0)==(0);
 if (!($79)) {
  $81 = HEAP32[$pCache>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $80 = HEAP32[$pCache>>2]|0;
 _pcache1Destroy($80);
 HEAP32[$pCache>>2] = 0;
 $81 = HEAP32[$pCache>>2]|0;
 STACKTOP = sp;return ($81|0);
}
function _pcache1Cachesize($p,$nMax) {
 $p = $p|0;
 $nMax = $nMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0;
 var $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pCache = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nMax;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $2;
 $3 = HEAP32[$pCache>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$pCache>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pGroup>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$pCache>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($9) - ($12))|0;
 $14 = HEAP32[$pGroup>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13))|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$pGroup>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 10)|0;
 $22 = HEAP32[$pGroup>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($21) - ($24))|0;
 $26 = HEAP32[$pGroup>>2]|0;
 $27 = ((($26)) + 12|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$pCache>>2]|0;
 $30 = ((($29)) + 24|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pCache>>2]|0;
 $32 = ((($31)) + 24|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33*9)|0;
 $35 = (($34>>>0) / 10)&-1;
 $36 = HEAP32[$pCache>>2]|0;
 $37 = ((($36)) + 28|0);
 HEAP32[$37>>2] = $35;
 $38 = HEAP32[$pCache>>2]|0;
 _pcache1EnforceMaxPage($38);
 STACKTOP = sp;return;
}
function _pcache1Pagecount($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $n = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $n = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = ((($2)) + 40|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$n>>2] = $4;
 $5 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _pcache1Fetch($p,$iKey,$createFlag) {
 $p = $p|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iKey;
 HEAP32[$2>>2] = $createFlag;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_pcache1FetchNoMutex($3,$4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _pcache1Unpin($p,$pPg,$reuseUnlikely) {
 $p = $p|0;
 $pPg = $pPg|0;
 $reuseUnlikely = $reuseUnlikely|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0;
 var $pPage = 0, $ppFirst = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pCache = sp + 12|0;
 $pPage = sp + 8|0;
 $pGroup = sp + 4|0;
 $ppFirst = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $reuseUnlikely;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pPage>>2] = $4;
 $5 = HEAP32[$pCache>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pGroup>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $9 = HEAP32[$pGroup>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$pGroup>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($11>>>0)>($14>>>0);
  if (!($15)) {
   $17 = HEAP32[$pGroup>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = ((($18)) + 24|0);
   HEAP32[$ppFirst>>2] = $19;
   $20 = HEAP32[$pGroup>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP32[$pPage>>2]|0;
   $23 = ((($22)) + 28|0);
   HEAP32[$23>>2] = $21;
   $24 = HEAP32[$pPage>>2]|0;
   $25 = HEAP32[$ppFirst>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$pPage>>2]|0;
   $28 = ((($27)) + 24|0);
   HEAP32[$28>>2] = $26;
   $29 = ((($26)) + 28|0);
   HEAP32[$29>>2] = $24;
   $30 = HEAP32[$pPage>>2]|0;
   $31 = HEAP32[$ppFirst>>2]|0;
   HEAP32[$31>>2] = $30;
   $32 = HEAP32[$pCache>>2]|0;
   $33 = ((($32)) + 36|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$33>>2] = $35;
   $36 = HEAP32[$pPage>>2]|0;
   $37 = ((($36)) + 12|0);
   HEAP8[$37>>0] = 0;
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$pPage>>2]|0;
 _pcache1RemoveFromHash($16,1);
 STACKTOP = sp;return;
}
function _pcache1Rekey($p,$pPg,$iOld,$iNew) {
 $p = $p|0;
 $pPg = $pPg|0;
 $iOld = $iOld|0;
 $iNew = $iNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pCache = 0, $pPage = 0;
 var $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pCache = sp + 12|0;
 $pPage = sp + 8|0;
 $pp = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $iOld;
 HEAP32[$3>>2] = $iNew;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pPage>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$pCache>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6>>>0) % ($9>>>0))&-1;
 HEAP32[$h>>2] = $10;
 $11 = HEAP32[$h>>2]|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<2)|0);
 HEAP32[$pp>>2] = $15;
 while(1) {
  $16 = HEAP32[$pp>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pPage>>2]|0;
  $19 = ($17|0)!=($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$pp>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 16|0);
  HEAP32[$pp>>2] = $22;
 }
 $23 = HEAP32[$pPage>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$pp>>2]|0;
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$pCache>>2]|0;
 $29 = ((($28)) + 44|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (($27>>>0) % ($30>>>0))&-1;
 HEAP32[$h>>2] = $31;
 $32 = HEAP32[$3>>2]|0;
 $33 = HEAP32[$pPage>>2]|0;
 $34 = ((($33)) + 8|0);
 HEAP32[$34>>2] = $32;
 $35 = HEAP32[$h>>2]|0;
 $36 = HEAP32[$pCache>>2]|0;
 $37 = ((($36)) + 48|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + ($35<<2)|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$pPage>>2]|0;
 $42 = ((($41)) + 16|0);
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$pPage>>2]|0;
 $44 = HEAP32[$h>>2]|0;
 $45 = HEAP32[$pCache>>2]|0;
 $46 = ((($45)) + 48|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44<<2)|0);
 HEAP32[$48>>2] = $43;
 $49 = HEAP32[$3>>2]|0;
 $50 = HEAP32[$pCache>>2]|0;
 $51 = ((($50)) + 32|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = ($49>>>0)>($52>>>0);
 if (!($53)) {
  STACKTOP = sp;return;
 }
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$pCache>>2]|0;
 $56 = ((($55)) + 32|0);
 HEAP32[$56>>2] = $54;
 STACKTOP = sp;return;
}
function _pcache1Truncate($p,$iLimit) {
 $p = $p|0;
 $iLimit = $iLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iLimit;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$pCache>>2]|0;
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3>>>0)<=($6>>>0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$pCache>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 _pcache1TruncateUnsafe($8,$9);
 $10 = HEAP32[$1>>2]|0;
 $11 = (($10) - 1)|0;
 $12 = HEAP32[$pCache>>2]|0;
 $13 = ((($12)) + 32|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _pcache1Destroy($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pCache = 0, $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pCache = sp + 4|0;
 $pGroup = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pGroup>>2] = $3;
 $4 = HEAP32[$pCache>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$pCache>>2]|0;
  _pcache1TruncateUnsafe($8,0);
 }
 $9 = HEAP32[$pCache>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pGroup>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) - ($11))|0;
 HEAP32[$13>>2] = $15;
 $16 = HEAP32[$pCache>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$pGroup>>2]|0;
 $20 = ((($19)) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - ($18))|0;
 HEAP32[$20>>2] = $22;
 $23 = HEAP32[$pGroup>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 10)|0;
 $27 = HEAP32[$pGroup>>2]|0;
 $28 = ((($27)) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($26) - ($29))|0;
 $31 = HEAP32[$pGroup>>2]|0;
 $32 = ((($31)) + 12|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$pCache>>2]|0;
 _pcache1EnforceMaxPage($33);
 $34 = HEAP32[$pCache>>2]|0;
 $35 = ((($34)) + 56|0);
 $36 = HEAP32[$35>>2]|0;
 _sqlite3_free($36);
 $37 = HEAP32[$pCache>>2]|0;
 $38 = ((($37)) + 48|0);
 $39 = HEAP32[$38>>2]|0;
 _sqlite3_free($39);
 $40 = HEAP32[$pCache>>2]|0;
 _sqlite3_free($40);
 STACKTOP = sp;return;
}
function _pcache1Shrink($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, $pGroup = 0, $savedMaxPage = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pCache = sp + 8|0;
 $pGroup = sp + 4|0;
 $savedMaxPage = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pCache>>2] = $1;
 $2 = HEAP32[$pCache>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pCache>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pGroup>>2] = $7;
 $8 = HEAP32[$pGroup>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$savedMaxPage>>2] = $10;
 $11 = HEAP32[$pGroup>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$pCache>>2]|0;
 _pcache1EnforceMaxPage($13);
 $14 = HEAP32[$savedMaxPage>>2]|0;
 $15 = HEAP32[$pGroup>>2]|0;
 $16 = ((($15)) + 4|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _pcache1EnforceMaxPage($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pGroup = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pGroup = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pGroup>>2] = $2;
 while(1) {
  $3 = HEAP32[$pGroup>>2]|0;
  $4 = ((($3)) + 16|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = HEAP32[$pGroup>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5>>>0)>($8>>>0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pGroup>>2]|0;
  $11 = ((($10)) + 20|0);
  $12 = ((($11)) + 28|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$p>>2] = $13;
  $14 = ((($13)) + 14|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)==(0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$p>>2]|0;
  (_pcache1PinPage($18)|0);
  $19 = HEAP32[$p>>2]|0;
  _pcache1RemoveFromHash($19,1);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 40|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)==(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 56|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($26|0)!=(0|0);
 if (!($27)) {
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 56|0);
 $30 = HEAP32[$29>>2]|0;
 _sqlite3_free($30);
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 52|0);
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 56|0);
 HEAP32[$34>>2] = 0;
 STACKTOP = sp;return;
}
function _pcache1PinPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 28|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($16)) + 28|0);
 HEAP32[$17>>2] = $13;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 24|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 28|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 12|0);
 HEAP8[$23>>0] = 1;
 $24 = HEAP32[$pCache>>2]|0;
 $25 = ((($24)) + 36|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + -1)|0;
 HEAP32[$25>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _pcache1RemoveFromHash($pPage,$freeFlag) {
 $pPage = $pPage|0;
 $freeFlag = $freeFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $pCache = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $h = sp + 8|0;
 $pCache = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $freeFlag;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$pCache>>2]|0;
 $9 = ((($8)) + 44|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($7>>>0) % ($10>>>0))&-1;
 HEAP32[$h>>2] = $11;
 $12 = HEAP32[$h>>2]|0;
 $13 = HEAP32[$pCache>>2]|0;
 $14 = ((($13)) + 48|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<2)|0);
 HEAP32[$pp>>2] = $16;
 while(1) {
  $17 = HEAP32[$pp>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ($18|0)!=($19|0);
  $21 = HEAP32[$pp>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 16|0);
  if (!($20)) {
   break;
  }
  HEAP32[$pp>>2] = $23;
 }
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$pp>>2]|0;
 HEAP32[$25>>2] = $24;
 $26 = HEAP32[$pCache>>2]|0;
 $27 = ((($26)) + 40|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($28) + -1)|0;
 HEAP32[$27>>2] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ($30|0)!=(0);
 if (!($31)) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 _pcache1FreePage($32);
 STACKTOP = sp;return;
}
function _pcache1FreePage($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pCache>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 13|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$pCache>>2]|0;
  $9 = ((($8)) + 52|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 16|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$pCache>>2]|0;
  $15 = ((($14)) + 52|0);
  HEAP32[$15>>2] = $13;
 } else {
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  _pcache1Free($17);
 }
 $18 = HEAP32[$pCache>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0);
 if (!($21)) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$pCache>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + -1)|0;
 HEAP32[$24>>2] = $26;
 STACKTOP = sp;return;
}
function _pcache1TruncateUnsafe($pCache,$iLimit) {
 $pCache = $pCache|0;
 $iLimit = $iLimit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $iStop = 0, $pPage = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $h = sp + 12|0;
 $iStop = sp + 8|0;
 $pp = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pCache;
 HEAP32[$1>>2] = $iLimit;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 32|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($4) - ($5))|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6>>>0)<($9>>>0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 44|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($11>>>0) % ($14>>>0))&-1;
  HEAP32[$h>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 44|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($18>>>0) % ($21>>>0))&-1;
  HEAP32[$iStop>>2] = $22;
 } else {
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 44|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25>>>0) / 2)&-1;
  HEAP32[$h>>2] = $26;
  $27 = HEAP32[$h>>2]|0;
  $28 = (($27) - 1)|0;
  HEAP32[$iStop>>2] = $28;
 }
 while(1) {
  $29 = HEAP32[$h>>2]|0;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 48|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29<<2)|0);
  HEAP32[$pp>>2] = $33;
  while(1) {
   $34 = HEAP32[$pp>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$pPage>>2] = $35;
   $36 = ($35|0)!=(0|0);
   if (!($36)) {
    break;
   }
   $37 = HEAP32[$pPage>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$1>>2]|0;
   $41 = ($39>>>0)>=($40>>>0);
   if (!($41)) {
    $56 = HEAP32[$pPage>>2]|0;
    $57 = ((($56)) + 16|0);
    HEAP32[$pp>>2] = $57;
    continue;
   }
   $42 = HEAP32[$0>>2]|0;
   $43 = ((($42)) + 40|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + -1)|0;
   HEAP32[$43>>2] = $45;
   $46 = HEAP32[$pPage>>2]|0;
   $47 = ((($46)) + 16|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$pp>>2]|0;
   HEAP32[$49>>2] = $48;
   $50 = HEAP32[$pPage>>2]|0;
   $51 = ((($50)) + 12|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = ($52<<24>>24)!=(0);
   if (!($53)) {
    $54 = HEAP32[$pPage>>2]|0;
    (_pcache1PinPage($54)|0);
   }
   $55 = HEAP32[$pPage>>2]|0;
   _pcache1FreePage($55);
  }
  $58 = HEAP32[$h>>2]|0;
  $59 = HEAP32[$iStop>>2]|0;
  $60 = ($58|0)==($59|0);
  if ($60) {
   break;
  }
  $61 = HEAP32[$h>>2]|0;
  $62 = (($61) + 1)|0;
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 44|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($62>>>0) % ($65>>>0))&-1;
  HEAP32[$h>>2] = $66;
 }
 STACKTOP = sp;return;
}
function _pcache1FetchNoMutex($p,$iKey,$createFlag) {
 $p = $p|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pCache = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pCache = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iKey;
 HEAP32[$3>>2] = $createFlag;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pCache>>2] = $4;
 HEAP32[$pPage>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$pCache>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($5>>>0) % ($8>>>0))&-1;
 $10 = HEAP32[$pCache>>2]|0;
 $11 = ((($10)) + 48|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($9<<2)|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pPage>>2] = $14;
 while(1) {
  $15 = HEAP32[$pPage>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   $17 = HEAP32[$pPage>>2]|0;
   $18 = ((($17)) + 8|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ($19|0)!=($20|0);
   $39 = $21;
  } else {
   $39 = 0;
  }
  $22 = HEAP32[$pPage>>2]|0;
  if (!($39)) {
   break;
  }
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pPage>>2] = $24;
 }
 $25 = ($22|0)!=(0|0);
 if ($25) {
  $26 = HEAP32[$pPage>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = ($28<<24>>24)!=(0);
  $30 = HEAP32[$pPage>>2]|0;
  if ($29) {
   HEAP32[$0>>2] = $30;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  } else {
   $31 = (_pcache1PinPage($30)|0);
   HEAP32[$0>>2] = $31;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = ($32|0)!=(0);
  if ($33) {
   $34 = HEAP32[$pCache>>2]|0;
   $35 = HEAP32[$2>>2]|0;
   $36 = HEAP32[$3>>2]|0;
   $37 = (_pcache1FetchStage2($34,$35,$36)|0);
   HEAP32[$0>>2] = $37;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  } else {
   HEAP32[$0>>2] = 0;
   $38 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($38|0);
  }
 }
 return (0)|0;
}
function _pcache1FetchStage2($pCache,$iKey,$createFlag) {
 $pCache = $pCache|0;
 $iKey = $iKey|0;
 $createFlag = $createFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $h = 0, $nPinned = 0, $pGroup = 0, $pOther = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $nPinned = sp + 16|0;
 $pGroup = sp + 12|0;
 $pPage = sp + 8|0;
 $pOther = sp + 4|0;
 $h = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $iKey;
 HEAP32[$3>>2] = $createFlag;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pGroup>>2] = $5;
 HEAP32[$pPage>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 36|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($8) - ($11))|0;
 HEAP32[$nPinned>>2] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ($13|0)==(1);
 do {
  if ($14) {
   $15 = HEAP32[$nPinned>>2]|0;
   $16 = HEAP32[$pGroup>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($15>>>0)>=($18>>>0);
   if (!($19)) {
    $20 = HEAP32[$nPinned>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 28|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ($20>>>0)>=($23>>>0);
    if (!($24)) {
     $25 = HEAP32[$1>>2]|0;
     $26 = (_pcache1UnderMemoryPressure($25)|0);
     $27 = ($26|0)!=(0);
     if (!($27)) {
      break;
     }
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 36|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$nPinned>>2]|0;
     $32 = ($30>>>0)<($31>>>0);
     if (!($32)) {
      break;
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $147 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($147|0);
  }
 } while(0);
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 40|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 44|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($35>>>0)>=($38>>>0);
 if ($39) {
  $40 = HEAP32[$1>>2]|0;
  _pcache1ResizeHash($40);
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 16|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)!=(0);
 do {
  if ($44) {
   $45 = HEAP32[$pGroup>>2]|0;
   $46 = ((($45)) + 20|0);
   $47 = ((($46)) + 28|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ((($48)) + 14|0);
   $50 = HEAP8[$49>>0]|0;
   $51 = ($50<<24>>24)!=(0);
   if (!($51)) {
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 40|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + 1)|0;
    $56 = HEAP32[$1>>2]|0;
    $57 = ((($56)) + 24|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = ($55>>>0)>=($58>>>0);
    if (!($59)) {
     $60 = HEAP32[$1>>2]|0;
     $61 = (_pcache1UnderMemoryPressure($60)|0);
     $62 = ($61|0)!=(0);
     if (!($62)) {
      break;
     }
    }
    $63 = HEAP32[$pGroup>>2]|0;
    $64 = ((($63)) + 20|0);
    $65 = ((($64)) + 28|0);
    $66 = HEAP32[$65>>2]|0;
    HEAP32[$pPage>>2] = $66;
    $67 = HEAP32[$pPage>>2]|0;
    _pcache1RemoveFromHash($67,0);
    $68 = HEAP32[$pPage>>2]|0;
    (_pcache1PinPage($68)|0);
    $69 = HEAP32[$pPage>>2]|0;
    $70 = ((($69)) + 20|0);
    $71 = HEAP32[$70>>2]|0;
    HEAP32[$pOther>>2] = $71;
    $72 = HEAP32[$pOther>>2]|0;
    $73 = ((($72)) + 12|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 12|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = ($74|0)!=($77|0);
    if ($78) {
     $79 = HEAP32[$pPage>>2]|0;
     _pcache1FreePage($79);
     HEAP32[$pPage>>2] = 0;
     break;
    } else {
     $80 = HEAP32[$pOther>>2]|0;
     $81 = ((($80)) + 16|0);
     $82 = HEAP32[$81>>2]|0;
     $83 = HEAP32[$1>>2]|0;
     $84 = ((($83)) + 16|0);
     $85 = HEAP32[$84>>2]|0;
     $86 = (($82) - ($85))|0;
     $87 = HEAP32[$pGroup>>2]|0;
     $88 = ((($87)) + 16|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = (($89) - ($86))|0;
     HEAP32[$88>>2] = $90;
     break;
    }
   }
  }
 } while(0);
 $91 = HEAP32[$pPage>>2]|0;
 $92 = ($91|0)!=(0|0);
 if (!($92)) {
  $93 = HEAP32[$1>>2]|0;
  $94 = HEAP32[$3>>2]|0;
  $95 = ($94|0)==(1);
  $96 = $95&1;
  $97 = (_pcache1AllocPage($93,$96)|0);
  HEAP32[$pPage>>2] = $97;
 }
 $98 = HEAP32[$pPage>>2]|0;
 $99 = ($98|0)!=(0|0);
 if ($99) {
  $100 = HEAP32[$2>>2]|0;
  $101 = HEAP32[$1>>2]|0;
  $102 = ((($101)) + 44|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = (($100>>>0) % ($103>>>0))&-1;
  HEAP32[$h>>2] = $104;
  $105 = HEAP32[$1>>2]|0;
  $106 = ((($105)) + 40|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = (($107) + 1)|0;
  HEAP32[$106>>2] = $108;
  $109 = HEAP32[$2>>2]|0;
  $110 = HEAP32[$pPage>>2]|0;
  $111 = ((($110)) + 8|0);
  HEAP32[$111>>2] = $109;
  $112 = HEAP32[$h>>2]|0;
  $113 = HEAP32[$1>>2]|0;
  $114 = ((($113)) + 48|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (($115) + ($112<<2)|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = HEAP32[$pPage>>2]|0;
  $119 = ((($118)) + 16|0);
  HEAP32[$119>>2] = $117;
  $120 = HEAP32[$1>>2]|0;
  $121 = HEAP32[$pPage>>2]|0;
  $122 = ((($121)) + 20|0);
  HEAP32[$122>>2] = $120;
  $123 = HEAP32[$pPage>>2]|0;
  $124 = ((($123)) + 28|0);
  HEAP32[$124>>2] = 0;
  $125 = HEAP32[$pPage>>2]|0;
  $126 = ((($125)) + 24|0);
  HEAP32[$126>>2] = 0;
  $127 = HEAP32[$pPage>>2]|0;
  $128 = ((($127)) + 12|0);
  HEAP8[$128>>0] = 1;
  $129 = HEAP32[$pPage>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  HEAP32[$131>>2] = 0;
  $132 = HEAP32[$pPage>>2]|0;
  $133 = HEAP32[$h>>2]|0;
  $134 = HEAP32[$1>>2]|0;
  $135 = ((($134)) + 48|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = (($136) + ($133<<2)|0);
  HEAP32[$137>>2] = $132;
  $138 = HEAP32[$2>>2]|0;
  $139 = HEAP32[$1>>2]|0;
  $140 = ((($139)) + 32|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ($138>>>0)>($141>>>0);
  if ($142) {
   $143 = HEAP32[$2>>2]|0;
   $144 = HEAP32[$1>>2]|0;
   $145 = ((($144)) + 32|0);
   HEAP32[$145>>2] = $143;
  }
 }
 $146 = HEAP32[$pPage>>2]|0;
 HEAP32[$0>>2] = $146;
 $147 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($147|0);
}
function _pcache1UnderMemoryPressure($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pCache;
 $2 = HEAP32[(1696)>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($6) + ($9))|0;
  $11 = HEAP32[(1692)>>2]|0;
  $12 = ($10|0)<=($11|0);
  if ($12) {
   $13 = HEAP32[(1724)>>2]|0;
   HEAP32[$0>>2] = $13;
   $15 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($15|0);
  }
 }
 $14 = (_sqlite3HeapNearlyFull()|0);
 HEAP32[$0>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _pcache1ResizeHash($p) {
 $p = $p|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $apNew = 0, $h = 0, $i = 0, $nNew = 0, $pNext = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $apNew = sp + 20|0;
 $nNew = sp + 16|0;
 $i = sp + 12|0;
 $pPage = sp + 8|0;
 $pNext = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = $3<<1;
 HEAP32[$nNew>>2] = $4;
 $5 = HEAP32[$nNew>>2]|0;
 $6 = ($5>>>0)<(256);
 $$ = $6 ? 256 : $4;
 HEAP32[$nNew>>2] = $$;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  _sqlite3BeginBenignMalloc();
 }
 $11 = HEAP32[$nNew>>2]|0;
 $12 = $11<<2;
 $13 = (_sqlite3MallocZero($12,0)|0);
 HEAP32[$apNew>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 44|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  _sqlite3EndBenignMalloc();
 }
 $18 = HEAP32[$apNew>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = ((($21)) + 44|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20>>>0)<($23>>>0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + ($25<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pNext>>2] = $30;
  while(1) {
   $31 = HEAP32[$pNext>>2]|0;
   HEAP32[$pPage>>2] = $31;
   $32 = ($31|0)!=(0|0);
   if (!($32)) {
    break;
   }
   $33 = HEAP32[$pPage>>2]|0;
   $34 = ((($33)) + 8|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = HEAP32[$nNew>>2]|0;
   $37 = (($35>>>0) % ($36>>>0))&-1;
   HEAP32[$h>>2] = $37;
   $38 = HEAP32[$pPage>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$pNext>>2] = $40;
   $41 = HEAP32[$h>>2]|0;
   $42 = HEAP32[$apNew>>2]|0;
   $43 = (($42) + ($41<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$pPage>>2]|0;
   $46 = ((($45)) + 16|0);
   HEAP32[$46>>2] = $44;
   $47 = HEAP32[$pPage>>2]|0;
   $48 = HEAP32[$h>>2]|0;
   $49 = HEAP32[$apNew>>2]|0;
   $50 = (($49) + ($48<<2)|0);
   HEAP32[$50>>2] = $47;
  }
  $51 = HEAP32[$i>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$i>>2] = $52;
 }
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 48|0);
 $55 = HEAP32[$54>>2]|0;
 _sqlite3_free($55);
 $56 = HEAP32[$apNew>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 48|0);
 HEAP32[$58>>2] = $56;
 $59 = HEAP32[$nNew>>2]|0;
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 44|0);
 HEAP32[$61>>2] = $59;
 STACKTOP = sp;return;
}
function _pcache1AllocPage($pCache,$benignMalloc) {
 $pCache = $pCache|0;
 $benignMalloc = $benignMalloc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pPg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pPg = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $benignMalloc;
 HEAP32[$p>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   label = 4;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 40|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)==(0);
   if ($10) {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_pcache1InitBulk($11)|0);
    $13 = ($12|0)!=(0);
    if ($13) {
     label = 4;
     break;
    }
   }
   $24 = HEAP32[$2>>2]|0;
   $25 = ($24|0)!=(0);
   if ($25) {
    _sqlite3BeginBenignMalloc();
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 12|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (_pcache1Alloc($28)|0);
   HEAP32[$pPg>>2] = $29;
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 4|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$pPg>>2]|0;
   $34 = (($33) + ($32)|0);
   HEAP32[$p>>2] = $34;
   $35 = HEAP32[$2>>2]|0;
   $36 = ($35|0)!=(0);
   if ($36) {
    _sqlite3EndBenignMalloc();
   }
   $37 = HEAP32[$pPg>>2]|0;
   $38 = ($37|0)==(0|0);
   if (!($38)) {
    $39 = HEAP32[$pPg>>2]|0;
    $40 = HEAP32[$p>>2]|0;
    HEAP32[$40>>2] = $39;
    $41 = HEAP32[$p>>2]|0;
    $42 = ((($41)) + 32|0);
    $43 = HEAP32[$p>>2]|0;
    $44 = ((($43)) + 4|0);
    HEAP32[$44>>2] = $42;
    $45 = HEAP32[$p>>2]|0;
    $46 = ((($45)) + 13|0);
    HEAP8[$46>>0] = 0;
    $47 = HEAP32[$p>>2]|0;
    $48 = ((($47)) + 14|0);
    HEAP8[$48>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 0;
   $59 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($59|0);
  }
 } while(0);
 if ((label|0) == 4) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 52|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$p>>2] = $16;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 52|0);
  HEAP32[$21>>2] = $19;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 16|0);
  HEAP32[$23>>2] = 0;
 }
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = HEAP32[$53>>2]|0;
  $55 = ((($54)) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + 1)|0;
  HEAP32[$55>>2] = $57;
 }
 $58 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _pcache1InitBulk($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $nBulk = 0, $pX = 0, $szBulk = 0, $zBulk = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $szBulk = sp;
 $zBulk = sp + 20|0;
 $nBulk = sp + 16|0;
 $i = sp + 12|0;
 $pX = sp + 8|0;
 HEAP32[$1>>2] = $pCache;
 $2 = HEAP32[(1688)>>2]|0;
 $3 = ($2|0)==(0);
 if ($3) {
  HEAP32[$0>>2] = 0;
  $123 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6>>>0)<(3);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $123 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 _sqlite3BeginBenignMalloc();
 $8 = HEAP32[(1688)>>2]|0;
 $9 = ($8|0)>(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)<(0);
  $14 = $13 << 31 >> 31;
  $15 = HEAP32[(1688)>>2]|0;
  $16 = ($15|0)<(0);
  $17 = $16 << 31 >> 31;
  $18 = (___muldi3(($12|0),($14|0),($15|0),($17|0))|0);
  $19 = tempRet0;
  $20 = $szBulk;
  $21 = $20;
  HEAP32[$21>>2] = $18;
  $22 = (($20) + 4)|0;
  $23 = $22;
  HEAP32[$23>>2] = $19;
 } else {
  $24 = HEAP32[(1688)>>2]|0;
  $25 = ($24|0)<(0);
  $26 = $25 << 31 >> 31;
  $27 = (___muldi3(-1024,-1,($24|0),($26|0))|0);
  $28 = tempRet0;
  $29 = $szBulk;
  $30 = $29;
  HEAP32[$30>>2] = $27;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
 }
 $33 = $szBulk;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)<(0);
 $43 = $42 << 31 >> 31;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 24|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (___muldi3(($41|0),($43|0),($46|0),0)|0);
 $48 = tempRet0;
 $49 = ($38|0)>($48|0);
 $50 = ($35>>>0)>($47>>>0);
 $51 = ($38|0)==($48|0);
 $52 = $51 & $50;
 $53 = $49 | $52;
 if ($53) {
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)<(0);
  $58 = $57 << 31 >> 31;
  $59 = HEAP32[$1>>2]|0;
  $60 = ((($59)) + 24|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (___muldi3(($56|0),($58|0),($61|0),0)|0);
  $63 = tempRet0;
  $64 = $szBulk;
  $65 = $64;
  HEAP32[$65>>2] = $62;
  $66 = (($64) + 4)|0;
  $67 = $66;
  HEAP32[$67>>2] = $63;
 }
 $68 = $szBulk;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (($68) + 4)|0;
 $72 = $71;
 $73 = HEAP32[$72>>2]|0;
 $74 = (_sqlite3Malloc($70,$73)|0);
 $75 = HEAP32[$1>>2]|0;
 $76 = ((($75)) + 56|0);
 HEAP32[$76>>2] = $74;
 HEAP32[$zBulk>>2] = $74;
 _sqlite3EndBenignMalloc();
 $77 = HEAP32[$zBulk>>2]|0;
 $78 = ($77|0)!=(0|0);
 L16: do {
  if ($78) {
   $79 = HEAP32[$zBulk>>2]|0;
   $80 = (_sqlite3MallocSize($79)|0);
   $81 = HEAP32[$1>>2]|0;
   $82 = ((($81)) + 12|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = (($80|0) / ($83|0))&-1;
   HEAP32[$nBulk>>2] = $84;
   HEAP32[$i>>2] = 0;
   while(1) {
    $85 = HEAP32[$i>>2]|0;
    $86 = HEAP32[$nBulk>>2]|0;
    $87 = ($85|0)<($86|0);
    if (!($87)) {
     break L16;
    }
    $88 = HEAP32[$1>>2]|0;
    $89 = ((($88)) + 4|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$zBulk>>2]|0;
    $92 = (($91) + ($90)|0);
    HEAP32[$pX>>2] = $92;
    $93 = HEAP32[$zBulk>>2]|0;
    $94 = HEAP32[$pX>>2]|0;
    HEAP32[$94>>2] = $93;
    $95 = HEAP32[$pX>>2]|0;
    $96 = ((($95)) + 32|0);
    $97 = HEAP32[$pX>>2]|0;
    $98 = ((($97)) + 4|0);
    HEAP32[$98>>2] = $96;
    $99 = HEAP32[$pX>>2]|0;
    $100 = ((($99)) + 13|0);
    HEAP8[$100>>0] = 1;
    $101 = HEAP32[$pX>>2]|0;
    $102 = ((($101)) + 14|0);
    HEAP8[$102>>0] = 0;
    $103 = HEAP32[$1>>2]|0;
    $104 = ((($103)) + 52|0);
    $105 = HEAP32[$104>>2]|0;
    $106 = HEAP32[$pX>>2]|0;
    $107 = ((($106)) + 16|0);
    HEAP32[$107>>2] = $105;
    $108 = HEAP32[$pX>>2]|0;
    $109 = HEAP32[$1>>2]|0;
    $110 = ((($109)) + 52|0);
    HEAP32[$110>>2] = $108;
    $111 = HEAP32[$1>>2]|0;
    $112 = ((($111)) + 12|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = HEAP32[$zBulk>>2]|0;
    $115 = (($114) + ($113)|0);
    HEAP32[$zBulk>>2] = $115;
    $116 = HEAP32[$i>>2]|0;
    $117 = (($116) + 1)|0;
    HEAP32[$i>>2] = $117;
   }
  }
 } while(0);
 $118 = HEAP32[$1>>2]|0;
 $119 = ((($118)) + 52|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = ($120|0)!=(0|0);
 $122 = $121&1;
 HEAP32[$0>>2] = $122;
 $123 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _sqlite3HeapNearlyFull() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[(316)>>2]|0;
 return ($0|0);
}
function _sqlite3MemMalloc($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 HEAP32[$0>>2] = $nByte;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7)|0;
 $3 = $2 & -8;
 HEAP32[$0>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (($4) + 8)|0;
 $6 = (_malloc($5)|0);
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = ($9|0)<(0);
  $11 = $10 << 31 >> 31;
  $12 = HEAP32[$p>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $9;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 8|0);
  HEAP32[$p>>2] = $18;
  $19 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  HEAP32[$vararg_buffer>>2] = $9;
  _sqlite3_log(7,26399,$vararg_buffer);
  $19 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3MemFree($pPrior) {
 $pPrior = $pPrior|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPrior;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + -8|0);
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 _free($4);
 STACKTOP = sp;return;
}
function _sqlite3MemRealloc($pPrior,$nByte) {
 $pPrior = $pPrior|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pPrior;
 HEAP32[$1>>2] = $nByte;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ((($3)) + -8|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (($6) + 8)|0;
 $8 = (_realloc($5,$7)|0);
 HEAP32[$p>>2] = $8;
 $9 = HEAP32[$p>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = HEAP32[$p>>2]|0;
  $15 = $14;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = (($15) + 4)|0;
  $18 = $17;
  HEAP32[$18>>2] = $13;
  $19 = HEAP32[$p>>2]|0;
  $20 = ((($19)) + 8|0);
  HEAP32[$p>>2] = $20;
  $24 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $21 = HEAP32[$0>>2]|0;
  $22 = (_sqlite3MemSize($21)|0);
  $23 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $22;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $23;
  _sqlite3_log(7,26363,$vararg_buffer);
  $24 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3MemSize($pPrior) {
 $pPrior = $pPrior|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pPrior;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + -8|0);
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3MemRoundup($n) {
 $n = $n|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $n;
 $1 = HEAP32[$0>>2]|0;
 $2 = (($1) + 7)|0;
 $3 = $2 & -8;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3MemInit($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return 0;
}
function _sqlite3MemShutdown($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return;
}
function _sqlite3ParserAlloc($mallocProc) {
 $mallocProc = $mallocProc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pParser = sp;
 HEAP32[$0>>2] = $mallocProc;
 $1 = HEAP32[$0>>2]|0;
 $2 = (FUNCTION_TABLE_iii[$1 & 255](1608,0)|0);
 HEAP32[$pParser>>2] = $2;
 $3 = HEAP32[$pParser>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $13 = HEAP32[$pParser>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 $5 = HEAP32[$pParser>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$pParser>>2]|0;
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$pParser>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP16[$9>>1] = 0;
 $10 = HEAP32[$pParser>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = ((($11)) + 2|0);
 HEAP8[$12>>0] = 0;
 $13 = HEAP32[$pParser>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3GetToken($z,$tokenType) {
 $z = $z|0;
 $tokenType = $tokenType|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $c = 0, $delim = 0, $i = 0, $n = 0, $or$cond = 0, $or$cond7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $c = sp + 8|0;
 $delim = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $tokenType;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = (39282 + ($5)|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 L1: do {
  switch ($8|0) {
  case 7:  {
   HEAP32[$i>>2] = 1;
   while(1) {
    $9 = HEAP32[$i>>2]|0;
    $10 = HEAP32[$1>>2]|0;
    $11 = (($10) + ($9)|0);
    $12 = HEAP8[$11>>0]|0;
    $13 = $12&255;
    $14 = (18403 + ($13)|0);
    $15 = HEAP8[$14>>0]|0;
    $16 = $15&255;
    $17 = $16 & 1;
    $18 = ($17|0)!=(0);
    if (!($18)) {
     break;
    }
    $19 = HEAP32[$i>>2]|0;
    $20 = (($19) + 1)|0;
    HEAP32[$i>>2] = $20;
   }
   $21 = HEAP32[$2>>2]|0;
   HEAP32[$21>>2] = 162;
   $22 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $22;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 11:  {
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 1|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($26|0)==(45);
   if (!($27)) {
    $41 = HEAP32[$2>>2]|0;
    HEAP32[$41>>2] = 48;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   HEAP32[$i>>2] = 2;
   while(1) {
    $28 = HEAP32[$i>>2]|0;
    $29 = HEAP32[$1>>2]|0;
    $30 = (($29) + ($28)|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = $31&255;
    HEAP32[$c>>2] = $32;
    $33 = ($32|0)!=(0);
    $34 = HEAP32[$c>>2]|0;
    $35 = ($34|0)!=(10);
    $36 = $33 ? $35 : 0;
    if (!($36)) {
     break;
    }
    $37 = HEAP32[$i>>2]|0;
    $38 = (($37) + 1)|0;
    HEAP32[$i>>2] = $38;
   }
   $39 = HEAP32[$2>>2]|0;
   HEAP32[$39>>2] = 162;
   $40 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $40;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 17:  {
   $42 = HEAP32[$2>>2]|0;
   HEAP32[$42>>2] = 22;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 18:  {
   $43 = HEAP32[$2>>2]|0;
   HEAP32[$43>>2] = 23;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 19:  {
   $44 = HEAP32[$2>>2]|0;
   HEAP32[$44>>2] = 1;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 20:  {
   $45 = HEAP32[$2>>2]|0;
   HEAP32[$45>>2] = 47;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 21:  {
   $46 = HEAP32[$2>>2]|0;
   HEAP32[$46>>2] = 49;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 16:  {
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 1|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = ($50|0)!=(42);
   if (!($51)) {
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 2|0);
    $54 = HEAP8[$53>>0]|0;
    $55 = $54&255;
    $56 = ($55|0)==(0);
    if (!($56)) {
     HEAP32[$i>>2] = 3;
     $58 = HEAP32[$1>>2]|0;
     $59 = ((($58)) + 2|0);
     $60 = HEAP8[$59>>0]|0;
     $61 = $60&255;
     HEAP32[$c>>2] = $61;
     while(1) {
      $62 = HEAP32[$c>>2]|0;
      $63 = ($62|0)!=(42);
      if (!($63)) {
       $64 = HEAP32[$i>>2]|0;
       $65 = HEAP32[$1>>2]|0;
       $66 = (($65) + ($64)|0);
       $67 = HEAP8[$66>>0]|0;
       $68 = $67&255;
       $69 = ($68|0)!=(47);
       if (!($69)) {
        break;
       }
      }
      $70 = HEAP32[$i>>2]|0;
      $71 = HEAP32[$1>>2]|0;
      $72 = (($71) + ($70)|0);
      $73 = HEAP8[$72>>0]|0;
      $74 = $73&255;
      HEAP32[$c>>2] = $74;
      $75 = ($74|0)!=(0);
      if (!($75)) {
       break;
      }
      $76 = HEAP32[$i>>2]|0;
      $77 = (($76) + 1)|0;
      HEAP32[$i>>2] = $77;
     }
     $78 = HEAP32[$c>>2]|0;
     $79 = ($78|0)!=(0);
     if ($79) {
      $80 = HEAP32[$i>>2]|0;
      $81 = (($80) + 1)|0;
      HEAP32[$i>>2] = $81;
     }
     $82 = HEAP32[$2>>2]|0;
     HEAP32[$82>>2] = 162;
     $83 = HEAP32[$i>>2]|0;
     HEAP32[$0>>2] = $83;
     $502 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($502|0);
    }
   }
   $57 = HEAP32[$2>>2]|0;
   HEAP32[$57>>2] = 50;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 22:  {
   $84 = HEAP32[$2>>2]|0;
   HEAP32[$84>>2] = 51;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 14:  {
   $85 = HEAP32[$2>>2]|0;
   HEAP32[$85>>2] = 37;
   $86 = HEAP32[$1>>2]|0;
   $87 = ((($86)) + 1|0);
   $88 = HEAP8[$87>>0]|0;
   $89 = $88&255;
   $90 = ($89|0)==(61);
   $91 = $90&1;
   $92 = (1 + ($91))|0;
   HEAP32[$0>>2] = $92;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 12:  {
   $93 = HEAP32[$1>>2]|0;
   $94 = ((($93)) + 1|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = $95&255;
   HEAP32[$c>>2] = $96;
   $97 = ($96|0)==(61);
   if ($97) {
    $98 = HEAP32[$2>>2]|0;
    HEAP32[$98>>2] = 39;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $99 = HEAP32[$c>>2]|0;
   $100 = ($99|0)==(62);
   if ($100) {
    $101 = HEAP32[$2>>2]|0;
    HEAP32[$101>>2] = 36;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $102 = HEAP32[$c>>2]|0;
   $103 = ($102|0)==(60);
   $104 = HEAP32[$2>>2]|0;
   if ($103) {
    HEAP32[$104>>2] = 45;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$104>>2] = 40;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 13:  {
   $105 = HEAP32[$1>>2]|0;
   $106 = ((($105)) + 1|0);
   $107 = HEAP8[$106>>0]|0;
   $108 = $107&255;
   HEAP32[$c>>2] = $108;
   $109 = ($108|0)==(61);
   if ($109) {
    $110 = HEAP32[$2>>2]|0;
    HEAP32[$110>>2] = 41;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $111 = HEAP32[$c>>2]|0;
   $112 = ($111|0)==(62);
   $113 = HEAP32[$2>>2]|0;
   if ($112) {
    HEAP32[$113>>2] = 46;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$113>>2] = 38;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 15:  {
   $114 = HEAP32[$1>>2]|0;
   $115 = ((($114)) + 1|0);
   $116 = HEAP8[$115>>0]|0;
   $117 = $116&255;
   $118 = ($117|0)!=(61);
   $119 = HEAP32[$2>>2]|0;
   if ($118) {
    HEAP32[$119>>2] = 163;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$119>>2] = 36;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 10:  {
   $120 = HEAP32[$1>>2]|0;
   $121 = ((($120)) + 1|0);
   $122 = HEAP8[$121>>0]|0;
   $123 = $122&255;
   $124 = ($123|0)!=(124);
   $125 = HEAP32[$2>>2]|0;
   if ($124) {
    HEAP32[$125>>2] = 44;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$125>>2] = 52;
    HEAP32[$0>>2] = 2;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 23:  {
   $126 = HEAP32[$2>>2]|0;
   HEAP32[$126>>2] = 26;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 24:  {
   $127 = HEAP32[$2>>2]|0;
   HEAP32[$127>>2] = 43;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 25:  {
   $128 = HEAP32[$2>>2]|0;
   HEAP32[$128>>2] = 54;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 8:  {
   $129 = HEAP32[$1>>2]|0;
   $130 = HEAP8[$129>>0]|0;
   $131 = $130&255;
   HEAP32[$delim>>2] = $131;
   HEAP32[$i>>2] = 1;
   while(1) {
    $132 = HEAP32[$i>>2]|0;
    $133 = HEAP32[$1>>2]|0;
    $134 = (($133) + ($132)|0);
    $135 = HEAP8[$134>>0]|0;
    $136 = $135&255;
    HEAP32[$c>>2] = $136;
    $137 = ($136|0)!=(0);
    if (!($137)) {
     break;
    }
    $138 = HEAP32[$c>>2]|0;
    $139 = HEAP32[$delim>>2]|0;
    $140 = ($138|0)==($139|0);
    if ($140) {
     $141 = HEAP32[$i>>2]|0;
     $142 = (($141) + 1)|0;
     $143 = HEAP32[$1>>2]|0;
     $144 = (($143) + ($142)|0);
     $145 = HEAP8[$144>>0]|0;
     $146 = $145&255;
     $147 = HEAP32[$delim>>2]|0;
     $148 = ($146|0)==($147|0);
     if (!($148)) {
      break;
     }
     $149 = HEAP32[$i>>2]|0;
     $150 = (($149) + 1)|0;
     HEAP32[$i>>2] = $150;
    }
    $151 = HEAP32[$i>>2]|0;
    $152 = (($151) + 1)|0;
    HEAP32[$i>>2] = $152;
   }
   $153 = HEAP32[$c>>2]|0;
   $154 = ($153|0)==(39);
   if ($154) {
    $155 = HEAP32[$2>>2]|0;
    HEAP32[$155>>2] = 97;
    $156 = HEAP32[$i>>2]|0;
    $157 = (($156) + 1)|0;
    HEAP32[$0>>2] = $157;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   $158 = HEAP32[$c>>2]|0;
   $159 = ($158|0)!=(0);
   $160 = HEAP32[$2>>2]|0;
   if ($159) {
    HEAP32[$160>>2] = 55;
    $161 = HEAP32[$i>>2]|0;
    $162 = (($161) + 1)|0;
    HEAP32[$0>>2] = $162;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    HEAP32[$160>>2] = 163;
    $163 = HEAP32[$i>>2]|0;
    HEAP32[$0>>2] = $163;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 26:  {
   $164 = HEAP32[$1>>2]|0;
   $165 = ((($164)) + 1|0);
   $166 = HEAP8[$165>>0]|0;
   $167 = $166&255;
   $168 = (18403 + ($167)|0);
   $169 = HEAP8[$168>>0]|0;
   $170 = $169&255;
   $171 = $170 & 4;
   $172 = ($171|0)!=(0);
   if ($172) {
    label = 64;
   } else {
    $173 = HEAP32[$2>>2]|0;
    HEAP32[$173>>2] = 122;
    HEAP32[$0>>2] = 1;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   }
   break;
  }
  case 3:  {
   label = 64;
   break;
  }
  case 9:  {
   HEAP32[$i>>2] = 1;
   $319 = HEAP32[$1>>2]|0;
   $320 = HEAP8[$319>>0]|0;
   $321 = $320&255;
   HEAP32[$c>>2] = $321;
   while(1) {
    $322 = HEAP32[$c>>2]|0;
    $323 = ($322|0)!=(93);
    if (!($323)) {
     break;
    }
    $324 = HEAP32[$i>>2]|0;
    $325 = HEAP32[$1>>2]|0;
    $326 = (($325) + ($324)|0);
    $327 = HEAP8[$326>>0]|0;
    $328 = $327&255;
    HEAP32[$c>>2] = $328;
    $329 = ($328|0)!=(0);
    if (!($329)) {
     break;
    }
    $330 = HEAP32[$i>>2]|0;
    $331 = (($330) + 1)|0;
    HEAP32[$i>>2] = $331;
   }
   $332 = HEAP32[$c>>2]|0;
   $333 = ($332|0)==(93);
   $334 = $333 ? 55 : 163;
   $335 = HEAP32[$2>>2]|0;
   HEAP32[$335>>2] = $334;
   $336 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $336;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 6:  {
   $337 = HEAP32[$2>>2]|0;
   HEAP32[$337>>2] = 135;
   HEAP32[$i>>2] = 1;
   while(1) {
    $338 = HEAP32[$i>>2]|0;
    $339 = HEAP32[$1>>2]|0;
    $340 = (($339) + ($338)|0);
    $341 = HEAP8[$340>>0]|0;
    $342 = $341&255;
    $343 = (18403 + ($342)|0);
    $344 = HEAP8[$343>>0]|0;
    $345 = $344&255;
    $346 = $345 & 4;
    $347 = ($346|0)!=(0);
    $348 = HEAP32[$i>>2]|0;
    if (!($347)) {
     break;
    }
    $349 = (($348) + 1)|0;
    HEAP32[$i>>2] = $349;
   }
   HEAP32[$0>>2] = $348;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 5: case 4:  {
   HEAP32[$n>>2] = 0;
   $350 = HEAP32[$2>>2]|0;
   HEAP32[$350>>2] = 135;
   HEAP32[$i>>2] = 1;
   while(1) {
    $351 = HEAP32[$i>>2]|0;
    $352 = HEAP32[$1>>2]|0;
    $353 = (($352) + ($351)|0);
    $354 = HEAP8[$353>>0]|0;
    $355 = $354&255;
    HEAP32[$c>>2] = $355;
    $356 = ($355|0)!=(0);
    if (!($356)) {
     break;
    }
    $357 = HEAP32[$c>>2]|0;
    $358 = $357&255;
    $359 = $358&255;
    $360 = (18403 + ($359)|0);
    $361 = HEAP8[$360>>0]|0;
    $362 = $361&255;
    $363 = $362 & 70;
    $364 = ($363|0)!=(0);
    if ($364) {
     $365 = HEAP32[$n>>2]|0;
     $366 = (($365) + 1)|0;
     HEAP32[$n>>2] = $366;
    } else {
     $367 = HEAP32[$c>>2]|0;
     $368 = ($367|0)==(40);
     $369 = HEAP32[$n>>2]|0;
     $370 = ($369|0)>(0);
     $or$cond = $368 & $370;
     if ($or$cond) {
      label = 107;
      break;
     }
     $394 = HEAP32[$c>>2]|0;
     $395 = ($394|0)==(58);
     if (!($395)) {
      break;
     }
     $396 = HEAP32[$i>>2]|0;
     $397 = (($396) + 1)|0;
     $398 = HEAP32[$1>>2]|0;
     $399 = (($398) + ($397)|0);
     $400 = HEAP8[$399>>0]|0;
     $401 = $400&255;
     $402 = ($401|0)==(58);
     if (!($402)) {
      break;
     }
     $403 = HEAP32[$i>>2]|0;
     $404 = (($403) + 1)|0;
     HEAP32[$i>>2] = $404;
    }
    $405 = HEAP32[$i>>2]|0;
    $406 = (($405) + 1)|0;
    HEAP32[$i>>2] = $406;
   }
   do {
    if ((label|0) == 107) {
     while(1) {
      label = 0;
      $371 = HEAP32[$i>>2]|0;
      $372 = (($371) + 1)|0;
      HEAP32[$i>>2] = $372;
      $373 = HEAP32[$i>>2]|0;
      $374 = HEAP32[$1>>2]|0;
      $375 = (($374) + ($373)|0);
      $376 = HEAP8[$375>>0]|0;
      $377 = $376&255;
      HEAP32[$c>>2] = $377;
      $378 = ($377|0)!=(0);
      if (!($378)) {
       break;
      }
      $379 = HEAP32[$c>>2]|0;
      $380 = $379&255;
      $381 = $380&255;
      $382 = (18403 + ($381)|0);
      $383 = HEAP8[$382>>0]|0;
      $384 = $383&255;
      $385 = $384 & 1;
      $386 = ($385|0)==(0);
      $387 = HEAP32[$c>>2]|0;
      $388 = ($387|0)!=(41);
      $or$cond7 = $386 & $388;
      if ($or$cond7) {
       label = 107;
      } else {
       break;
      }
     }
     $389 = HEAP32[$c>>2]|0;
     $390 = ($389|0)==(41);
     if ($390) {
      $391 = HEAP32[$i>>2]|0;
      $392 = (($391) + 1)|0;
      HEAP32[$i>>2] = $392;
      break;
     } else {
      $393 = HEAP32[$2>>2]|0;
      HEAP32[$393>>2] = 163;
      break;
     }
    }
   } while(0);
   $407 = HEAP32[$n>>2]|0;
   $408 = ($407|0)==(0);
   if ($408) {
    $409 = HEAP32[$2>>2]|0;
    HEAP32[$409>>2] = 163;
   }
   $410 = HEAP32[$i>>2]|0;
   HEAP32[$0>>2] = $410;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 1:  {
   HEAP32[$i>>2] = 1;
   while(1) {
    $411 = HEAP32[$i>>2]|0;
    $412 = HEAP32[$1>>2]|0;
    $413 = (($412) + ($411)|0);
    $414 = HEAP8[$413>>0]|0;
    $415 = $414&255;
    $416 = (39282 + ($415)|0);
    $417 = HEAP8[$416>>0]|0;
    $418 = $417&255;
    $419 = ($418|0)<=(1);
    $420 = HEAP32[$i>>2]|0;
    if (!($419)) {
     break;
    }
    $421 = (($420) + 1)|0;
    HEAP32[$i>>2] = $421;
   }
   $422 = HEAP32[$1>>2]|0;
   $423 = (($422) + ($420)|0);
   $424 = HEAP8[$423>>0]|0;
   $425 = $424&255;
   $426 = (18403 + ($425)|0);
   $427 = HEAP8[$426>>0]|0;
   $428 = $427&255;
   $429 = $428 & 70;
   $430 = ($429|0)!=(0);
   if ($430) {
    $431 = HEAP32[$i>>2]|0;
    $432 = (($431) + 1)|0;
    HEAP32[$i>>2] = $432;
    break L1;
   }
   $433 = HEAP32[$2>>2]|0;
   HEAP32[$433>>2] = 55;
   $434 = HEAP32[$1>>2]|0;
   $435 = HEAP32[$i>>2]|0;
   $436 = HEAP32[$2>>2]|0;
   $437 = (_keywordCode($434,$435,$436)|0);
   HEAP32[$0>>2] = $437;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
   break;
  }
  case 0:  {
   $438 = HEAP32[$1>>2]|0;
   $439 = ((($438)) + 1|0);
   $440 = HEAP8[$439>>0]|0;
   $441 = $440&255;
   $442 = ($441|0)==(39);
   if ($442) {
    $443 = HEAP32[$2>>2]|0;
    HEAP32[$443>>2] = 133;
    HEAP32[$i>>2] = 2;
    while(1) {
     $444 = HEAP32[$i>>2]|0;
     $445 = HEAP32[$1>>2]|0;
     $446 = (($445) + ($444)|0);
     $447 = HEAP8[$446>>0]|0;
     $448 = $447&255;
     $449 = (18403 + ($448)|0);
     $450 = HEAP8[$449>>0]|0;
     $451 = $450&255;
     $452 = $451 & 8;
     $453 = ($452|0)!=(0);
     $454 = HEAP32[$i>>2]|0;
     if (!($453)) {
      break;
     }
     $455 = (($454) + 1)|0;
     HEAP32[$i>>2] = $455;
    }
    $456 = HEAP32[$1>>2]|0;
    $457 = (($456) + ($454)|0);
    $458 = HEAP8[$457>>0]|0;
    $459 = $458&255;
    $460 = ($459|0)!=(39);
    if ($460) {
     label = 131;
    } else {
     $461 = HEAP32[$i>>2]|0;
     $462 = (($461|0) % 2)&-1;
     $463 = ($462|0)!=(0);
     if ($463) {
      label = 131;
     }
    }
    L168: do {
     if ((label|0) == 131) {
      $464 = HEAP32[$2>>2]|0;
      HEAP32[$464>>2] = 163;
      while(1) {
       $465 = HEAP32[$i>>2]|0;
       $466 = HEAP32[$1>>2]|0;
       $467 = (($466) + ($465)|0);
       $468 = HEAP8[$467>>0]|0;
       $469 = $468&255;
       $470 = ($469|0)!=(0);
       if (!($470)) {
        break L168;
       }
       $471 = HEAP32[$i>>2]|0;
       $472 = HEAP32[$1>>2]|0;
       $473 = (($472) + ($471)|0);
       $474 = HEAP8[$473>>0]|0;
       $475 = $474&255;
       $476 = ($475|0)!=(39);
       if (!($476)) {
        break L168;
       }
       $477 = HEAP32[$i>>2]|0;
       $478 = (($477) + 1)|0;
       HEAP32[$i>>2] = $478;
      }
     }
    } while(0);
    $479 = HEAP32[$i>>2]|0;
    $480 = HEAP32[$1>>2]|0;
    $481 = (($480) + ($479)|0);
    $482 = HEAP8[$481>>0]|0;
    $483 = ($482<<24>>24)!=(0);
    if ($483) {
     $484 = HEAP32[$i>>2]|0;
     $485 = (($484) + 1)|0;
     HEAP32[$i>>2] = $485;
    }
    $486 = HEAP32[$i>>2]|0;
    HEAP32[$0>>2] = $486;
    $502 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($502|0);
   } else {
    label = 138;
   }
   break;
  }
  case 2:  {
   label = 138;
   break;
  }
  default: {
   $487 = HEAP32[$2>>2]|0;
   HEAP32[$487>>2] = 163;
   HEAP32[$0>>2] = 1;
   $502 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($502|0);
  }
  }
 } while(0);
 if ((label|0) == 64) {
  $174 = HEAP32[$2>>2]|0;
  HEAP32[$174>>2] = 134;
  $175 = HEAP32[$1>>2]|0;
  $176 = HEAP8[$175>>0]|0;
  $177 = $176&255;
  $178 = ($177|0)==(48);
  do {
   if ($178) {
    $179 = HEAP32[$1>>2]|0;
    $180 = ((($179)) + 1|0);
    $181 = HEAP8[$180>>0]|0;
    $182 = $181&255;
    $183 = ($182|0)==(120);
    if (!($183)) {
     $184 = HEAP32[$1>>2]|0;
     $185 = ((($184)) + 1|0);
     $186 = HEAP8[$185>>0]|0;
     $187 = $186&255;
     $188 = ($187|0)==(88);
     if (!($188)) {
      break;
     }
    }
    $189 = HEAP32[$1>>2]|0;
    $190 = ((($189)) + 2|0);
    $191 = HEAP8[$190>>0]|0;
    $192 = $191&255;
    $193 = (18403 + ($192)|0);
    $194 = HEAP8[$193>>0]|0;
    $195 = $194&255;
    $196 = $195 & 8;
    $197 = ($196|0)!=(0);
    if ($197) {
     HEAP32[$i>>2] = 3;
     while(1) {
      $198 = HEAP32[$i>>2]|0;
      $199 = HEAP32[$1>>2]|0;
      $200 = (($199) + ($198)|0);
      $201 = HEAP8[$200>>0]|0;
      $202 = $201&255;
      $203 = (18403 + ($202)|0);
      $204 = HEAP8[$203>>0]|0;
      $205 = $204&255;
      $206 = $205 & 8;
      $207 = ($206|0)!=(0);
      $208 = HEAP32[$i>>2]|0;
      if (!($207)) {
       break;
      }
      $209 = (($208) + 1)|0;
      HEAP32[$i>>2] = $209;
     }
     HEAP32[$0>>2] = $208;
     $502 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($502|0);
    }
   }
  } while(0);
  HEAP32[$i>>2] = 0;
  while(1) {
   $210 = HEAP32[$i>>2]|0;
   $211 = HEAP32[$1>>2]|0;
   $212 = (($211) + ($210)|0);
   $213 = HEAP8[$212>>0]|0;
   $214 = $213&255;
   $215 = (18403 + ($214)|0);
   $216 = HEAP8[$215>>0]|0;
   $217 = $216&255;
   $218 = $217 & 4;
   $219 = ($218|0)!=(0);
   $220 = HEAP32[$i>>2]|0;
   if (!($219)) {
    break;
   }
   $221 = (($220) + 1)|0;
   HEAP32[$i>>2] = $221;
  }
  $222 = HEAP32[$1>>2]|0;
  $223 = (($222) + ($220)|0);
  $224 = HEAP8[$223>>0]|0;
  $225 = $224&255;
  $226 = ($225|0)==(46);
  if ($226) {
   $227 = HEAP32[$i>>2]|0;
   $228 = (($227) + 1)|0;
   HEAP32[$i>>2] = $228;
   while(1) {
    $229 = HEAP32[$i>>2]|0;
    $230 = HEAP32[$1>>2]|0;
    $231 = (($230) + ($229)|0);
    $232 = HEAP8[$231>>0]|0;
    $233 = $232&255;
    $234 = (18403 + ($233)|0);
    $235 = HEAP8[$234>>0]|0;
    $236 = $235&255;
    $237 = $236 & 4;
    $238 = ($237|0)!=(0);
    if (!($238)) {
     break;
    }
    $239 = HEAP32[$i>>2]|0;
    $240 = (($239) + 1)|0;
    HEAP32[$i>>2] = $240;
   }
   $241 = HEAP32[$2>>2]|0;
   HEAP32[$241>>2] = 132;
  }
  $242 = HEAP32[$i>>2]|0;
  $243 = HEAP32[$1>>2]|0;
  $244 = (($243) + ($242)|0);
  $245 = HEAP8[$244>>0]|0;
  $246 = $245&255;
  $247 = ($246|0)==(101);
  if ($247) {
   label = 82;
  } else {
   $248 = HEAP32[$i>>2]|0;
   $249 = HEAP32[$1>>2]|0;
   $250 = (($249) + ($248)|0);
   $251 = HEAP8[$250>>0]|0;
   $252 = $251&255;
   $253 = ($252|0)==(69);
   if ($253) {
    label = 82;
   }
  }
  do {
   if ((label|0) == 82) {
    $254 = HEAP32[$i>>2]|0;
    $255 = (($254) + 1)|0;
    $256 = HEAP32[$1>>2]|0;
    $257 = (($256) + ($255)|0);
    $258 = HEAP8[$257>>0]|0;
    $259 = $258&255;
    $260 = (18403 + ($259)|0);
    $261 = HEAP8[$260>>0]|0;
    $262 = $261&255;
    $263 = $262 & 4;
    $264 = ($263|0)!=(0);
    if (!($264)) {
     $265 = HEAP32[$i>>2]|0;
     $266 = (($265) + 1)|0;
     $267 = HEAP32[$1>>2]|0;
     $268 = (($267) + ($266)|0);
     $269 = HEAP8[$268>>0]|0;
     $270 = $269&255;
     $271 = ($270|0)==(43);
     if (!($271)) {
      $272 = HEAP32[$i>>2]|0;
      $273 = (($272) + 1)|0;
      $274 = HEAP32[$1>>2]|0;
      $275 = (($274) + ($273)|0);
      $276 = HEAP8[$275>>0]|0;
      $277 = $276&255;
      $278 = ($277|0)==(45);
      if (!($278)) {
       break;
      }
     }
     $279 = HEAP32[$i>>2]|0;
     $280 = (($279) + 2)|0;
     $281 = HEAP32[$1>>2]|0;
     $282 = (($281) + ($280)|0);
     $283 = HEAP8[$282>>0]|0;
     $284 = $283&255;
     $285 = (18403 + ($284)|0);
     $286 = HEAP8[$285>>0]|0;
     $287 = $286&255;
     $288 = $287 & 4;
     $289 = ($288|0)!=(0);
     if (!($289)) {
      break;
     }
    }
    $290 = HEAP32[$i>>2]|0;
    $291 = (($290) + 2)|0;
    HEAP32[$i>>2] = $291;
    while(1) {
     $292 = HEAP32[$i>>2]|0;
     $293 = HEAP32[$1>>2]|0;
     $294 = (($293) + ($292)|0);
     $295 = HEAP8[$294>>0]|0;
     $296 = $295&255;
     $297 = (18403 + ($296)|0);
     $298 = HEAP8[$297>>0]|0;
     $299 = $298&255;
     $300 = $299 & 4;
     $301 = ($300|0)!=(0);
     if (!($301)) {
      break;
     }
     $302 = HEAP32[$i>>2]|0;
     $303 = (($302) + 1)|0;
     HEAP32[$i>>2] = $303;
    }
    $304 = HEAP32[$2>>2]|0;
    HEAP32[$304>>2] = 132;
   }
  } while(0);
  while(1) {
   $305 = HEAP32[$i>>2]|0;
   $306 = HEAP32[$1>>2]|0;
   $307 = (($306) + ($305)|0);
   $308 = HEAP8[$307>>0]|0;
   $309 = $308&255;
   $310 = (18403 + ($309)|0);
   $311 = HEAP8[$310>>0]|0;
   $312 = $311&255;
   $313 = $312 & 70;
   $314 = ($313|0)!=(0);
   if (!($314)) {
    break;
   }
   $315 = HEAP32[$2>>2]|0;
   HEAP32[$315>>2] = 163;
   $316 = HEAP32[$i>>2]|0;
   $317 = (($316) + 1)|0;
   HEAP32[$i>>2] = $317;
  }
  $318 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $318;
  $502 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($502|0);
 }
 else if ((label|0) == 138) {
  HEAP32[$i>>2] = 1;
 }
 while(1) {
  $488 = HEAP32[$i>>2]|0;
  $489 = HEAP32[$1>>2]|0;
  $490 = (($489) + ($488)|0);
  $491 = HEAP8[$490>>0]|0;
  $492 = $491&255;
  $493 = (18403 + ($492)|0);
  $494 = HEAP8[$493>>0]|0;
  $495 = $494&255;
  $496 = $495 & 70;
  $497 = ($496|0)!=(0);
  if (!($497)) {
   break;
  }
  $498 = HEAP32[$i>>2]|0;
  $499 = (($498) + 1)|0;
  HEAP32[$i>>2] = $499;
 }
 $500 = HEAP32[$2>>2]|0;
 HEAP32[$500>>2] = 55;
 $501 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $501;
 $502 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($502|0);
}
function _sqlite3Parser($yyp,$yymajor,$yyminor,$pParse) {
 $yyp = $yyp|0;
 $yymajor = $yymajor|0;
 $yyminor = $yyminor|0;
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yyact = 0, $yyminor$byval_copy1 = 0, $yyminorunion = 0, $yypParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $yyminor$byval_copy1 = sp + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $yyminorunion = sp + 8|0;
 $yyact = sp + 4|0;
 $yypParser = sp;
 HEAP32[$0>>2] = $yyp;
 HEAP32[$1>>2] = $yymajor;
 HEAP32[$2>>2] = $pParse;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$yypParser>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$yypParser>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = $4;
 while(1) {
  $7 = HEAP32[$yypParser>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = $8&255;
  $10 = (_yy_find_shift_action($7,$9)|0);
  HEAP32[$yyact>>2] = $10;
  $11 = HEAP32[$yyact>>2]|0;
  $12 = ($11>>>0)<=(999);
  do {
   if ($12) {
    $13 = HEAP32[$yypParser>>2]|0;
    $14 = HEAP32[$yyact>>2]|0;
    $15 = HEAP32[$1>>2]|0;
    ;HEAP32[$yyminor$byval_copy1>>2]=HEAP32[$yyminor>>2]|0;HEAP32[$yyminor$byval_copy1+4>>2]=HEAP32[$yyminor+4>>2]|0;
    _yy_shift($13,$14,$15,$yyminor$byval_copy1);
    HEAP32[$1>>2] = 252;
   } else {
    $16 = HEAP32[$yyact>>2]|0;
    $17 = ($16>>>0)<=(1331);
    if ($17) {
     $18 = HEAP32[$yypParser>>2]|0;
     $19 = HEAP32[$yyact>>2]|0;
     $20 = (($19) - 1000)|0;
     _yy_reduce($18,$20);
     break;
    } else {
     ;HEAP32[$yyminorunion>>2]=HEAP32[$yyminor>>2]|0;HEAP32[$yyminorunion+4>>2]=HEAP32[$yyminor+4>>2]|0;
     $21 = HEAP32[$yypParser>>2]|0;
     $22 = HEAP32[$1>>2]|0;
     ;HEAP32[$yyminor$byval_copy1>>2]=HEAP32[$yyminor>>2]|0;HEAP32[$yyminor$byval_copy1+4>>2]=HEAP32[$yyminor+4>>2]|0;
     _yy_syntax_error($21,$22,$yyminor$byval_copy1);
     $23 = HEAP32[$yypParser>>2]|0;
     $24 = HEAP32[$1>>2]|0;
     $25 = $24&255;
     _yy_destructor($23,$25,$yyminorunion);
     HEAP32[$1>>2] = 252;
     break;
    }
   }
  } while(0);
  $26 = HEAP32[$1>>2]|0;
  $27 = ($26|0)!=(252);
  if (!($27)) {
   label = 9;
   break;
  }
  $28 = HEAP32[$yypParser>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$yypParser>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = ($29>>>0)>($31>>>0);
  if (!($32)) {
   label = 9;
   break;
  }
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
}
function _sqlite3ParserFree($p,$freeProc) {
 $p = $p|0;
 $freeProc = $freeProc|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParser = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pParser = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $freeProc;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pParser>>2] = $2;
 while(1) {
  $3 = HEAP32[$pParser>>2]|0;
  $4 = HEAP32[$3>>2]|0;
  $5 = HEAP32[$pParser>>2]|0;
  $6 = ((($5)) + 8|0);
  $7 = ($4>>>0)>($6>>>0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$pParser>>2]|0;
  _yy_pop_parser_stack($8);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$pParser>>2]|0;
 FUNCTION_TABLE_vi[$9 & 255]($10);
 STACKTOP = sp;return;
}
function _sqlite3VdbeDelete($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3VdbeClearObject($5,$6);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 if ($10) {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 8|0);
  HEAP32[$17>>2] = $13;
 } else {
  $18 = HEAP32[$db>>2]|0;
  $19 = ((($18)) + 4|0);
  HEAP32[$19>>2] = $13;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 4|0);
  HEAP32[$30>>2] = $26;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 20|0);
 HEAP32[$32>>2] = 1443283912;
 $33 = HEAP32[$0>>2]|0;
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$db>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($34,$35);
 STACKTOP = sp;return;
}
function _sqlite3WithDelete($db,$pWith) {
 $db = $db|0;
 $pWith = $pWith|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pCte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pCte = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pWith;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = (($10) + ($8<<4)|0);
  HEAP32[$pCte>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$pCte>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3ExprListDelete($12,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pCte>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3SelectDelete($16,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$pCte>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  _sqlite3DbFree($20,$22);
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($25,$26);
 STACKTOP = sp;return;
}
function _sqlite3ExprListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _exprListDeleteNN($4,$5);
 STACKTOP = sp;return;
}
function _sqlite3SelectDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _clearSelect($4,$5,1);
 STACKTOP = sp;return;
}
function _clearSelect($db,$p,$bFree) {
 $db = $db|0;
 $p = $p|0;
 $bFree = $bFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPrior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPrior = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $bFree;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 48|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pPrior>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  _sqlite3ExprListDelete($8,$10);
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 28|0);
  $14 = HEAP32[$13>>2]|0;
  _sqlite3SrcListDelete($11,$14);
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3ExprDelete($15,$18);
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 36|0);
  $22 = HEAP32[$21>>2]|0;
  _sqlite3ExprListDelete($19,$22);
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 40|0);
  $26 = HEAP32[$25>>2]|0;
  _sqlite3ExprDelete($23,$26);
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 44|0);
  $30 = HEAP32[$29>>2]|0;
  _sqlite3ExprListDelete($27,$30);
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 56|0);
  $34 = HEAP32[$33>>2]|0;
  _sqlite3ExprDelete($31,$34);
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 60|0);
  $38 = HEAP32[$37>>2]|0;
  _sqlite3ExprDelete($35,$38);
  $39 = HEAP32[$1>>2]|0;
  $40 = ((($39)) + 64|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ($41|0)!=(0|0);
  if ($42) {
   $43 = HEAP32[$0>>2]|0;
   $44 = HEAP32[$1>>2]|0;
   $45 = ((($44)) + 64|0);
   $46 = HEAP32[$45>>2]|0;
   _sqlite3WithDelete($43,$46);
  }
  $47 = HEAP32[$2>>2]|0;
  $48 = ($47|0)!=(0);
  if ($48) {
   $49 = HEAP32[$0>>2]|0;
   $50 = HEAP32[$1>>2]|0;
   _sqlite3DbFree($49,$50);
  }
  $51 = HEAP32[$pPrior>>2]|0;
  HEAP32[$1>>2] = $51;
  HEAP32[$2>>2] = 1;
 }
 STACKTOP = sp;return;
}
function _sqlite3SrcListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$pItem>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  $10 = HEAP32[$0>>2]|0;
  if (!($9)) {
   break;
  }
  $11 = HEAP32[$pItem>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  _sqlite3DbFree($10,$13);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$pItem>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  _sqlite3DbFree($14,$17);
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  _sqlite3DbFree($18,$21);
  $22 = HEAP32[$pItem>>2]|0;
  $23 = ((($22)) + 36|0);
  $24 = ((($23)) + 1|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = ($25&255) >>> 1;
  $27 = $26 & 1;
  $28 = $27&255;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   $31 = HEAP32[$pItem>>2]|0;
   $32 = ((($31)) + 64|0);
   $33 = HEAP32[$32>>2]|0;
   _sqlite3DbFree($30,$33);
  }
  $34 = HEAP32[$pItem>>2]|0;
  $35 = ((($34)) + 36|0);
  $36 = ((($35)) + 1|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = ($37&255) >>> 2;
  $39 = $38 & 1;
  $40 = $39&255;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = HEAP32[$0>>2]|0;
   $43 = HEAP32[$pItem>>2]|0;
   $44 = ((($43)) + 64|0);
   $45 = HEAP32[$44>>2]|0;
   _sqlite3ExprListDelete($42,$45);
  }
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$pItem>>2]|0;
  $48 = ((($47)) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  _sqlite3DeleteTable($46,$49);
  $50 = HEAP32[$0>>2]|0;
  $51 = HEAP32[$pItem>>2]|0;
  $52 = ((($51)) + 20|0);
  $53 = HEAP32[$52>>2]|0;
  _sqlite3SelectDelete($50,$53);
  $54 = HEAP32[$0>>2]|0;
  $55 = HEAP32[$pItem>>2]|0;
  $56 = ((($55)) + 48|0);
  $57 = HEAP32[$56>>2]|0;
  _sqlite3ExprDelete($54,$57);
  $58 = HEAP32[$0>>2]|0;
  $59 = HEAP32[$pItem>>2]|0;
  $60 = ((($59)) + 52|0);
  $61 = HEAP32[$60>>2]|0;
  _sqlite3IdListDelete($58,$61);
  $62 = HEAP32[$i>>2]|0;
  $63 = (($62) + 1)|0;
  HEAP32[$i>>2] = $63;
  $64 = HEAP32[$pItem>>2]|0;
  $65 = ((($64)) + 72|0);
  HEAP32[$pItem>>2] = $65;
 }
 $66 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($10,$66);
 STACKTOP = sp;return;
}
function _sqlite3ExprDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _sqlite3ExprDeleteNN($4,$5);
 STACKTOP = sp;return;
}
function _sqlite3ExprDeleteNN($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 8404992;
 $6 = ($5|0)!=(0);
 do {
  if (!($6)) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 12|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0|0);
   if ($10) {
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP8[$11>>0]|0;
    $13 = $12&255;
    $14 = ($13|0)!=(159);
    if ($14) {
     $15 = HEAP32[$0>>2]|0;
     $16 = HEAP32[$1>>2]|0;
     $17 = ((($16)) + 12|0);
     $18 = HEAP32[$17>>2]|0;
     _sqlite3ExprDeleteNN($15,$18);
    }
   }
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$21>>2]|0;
   _sqlite3ExprDelete($19,$22);
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 4|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = $25 & 2048;
   $27 = ($26|0)!=(0);
   $28 = HEAP32[$0>>2]|0;
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 20|0);
   if ($27) {
    $31 = HEAP32[$30>>2]|0;
    _sqlite3SelectDelete($28,$31);
    break;
   } else {
    $32 = HEAP32[$30>>2]|0;
    _sqlite3ExprListDelete($28,$32);
    break;
   }
  }
 } while(0);
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 4|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = $35 & 65536;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = ((($39)) + 8|0);
  $41 = HEAP32[$40>>2]|0;
  _sqlite3DbFree($38,$41);
 }
 $42 = HEAP32[$1>>2]|0;
 $43 = ((($42)) + 4|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = $44 & 32768;
 $46 = ($45|0)!=(0);
 if ($46) {
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$0>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($47,$48);
 STACKTOP = sp;return;
}
function _sqlite3IdListDelete($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  $9 = HEAP32[$0>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  _sqlite3DbFree($9,$14);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 _sqlite3DbFree($9,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($19,$20);
 STACKTOP = sp;return;
}
function _exprListDeleteNN($db,$pList) {
 $db = $db|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pItem>>2] = $4;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = ($5|0)<($7|0);
  $9 = HEAP32[$0>>2]|0;
  if (!($8)) {
   break;
  }
  $10 = HEAP32[$pItem>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  _sqlite3ExprDelete($9,$11);
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$pItem>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3DbFree($12,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pItem>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3DbFree($16,$19);
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$i>>2] = $21;
  $22 = HEAP32[$pItem>>2]|0;
  $23 = ((($22)) + 20|0);
  HEAP32[$pItem>>2] = $23;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 _sqlite3DbFree($9,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($27,$28);
 STACKTOP = sp;return;
}
function _yy_pop_parser_stack($pParser) {
 $pParser = $pParser|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yytos = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $yytos = sp;
 HEAP32[$0>>2] = $pParser;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + -16|0);
 HEAP32[$1>>2] = $3;
 HEAP32[$yytos>>2] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$yytos>>2]|0;
 $6 = ((($5)) + 2|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = HEAP32[$yytos>>2]|0;
 $9 = ((($8)) + 4|0);
 _yy_destructor($4,$7,$9);
 STACKTOP = sp;return;
}
function _yy_destructor($yypParser,$yymajor,$yypminor) {
 $yypParser = $yypParser|0;
 $yymajor = $yymajor|0;
 $yypminor = $yypminor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 12|0;
 $2 = sp + 4|0;
 $pParse = sp;
 HEAP32[$0>>2] = $yypParser;
 HEAP8[$1>>0] = $yymajor;
 HEAP32[$2>>2] = $yypminor;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pParse>>2] = $5;
 $6 = HEAP8[$1>>0]|0;
 $7 = $6&255;
 do {
  switch ($7|0) {
  case 206: case 195: case 194: case 163:  {
   $8 = HEAP32[$pParse>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$10>>2]|0;
   _sqlite3SelectDelete($9,$11);
   STACKTOP = sp;return;
   break;
  }
  case 173: case 172:  {
   $12 = HEAP32[$pParse>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   _sqlite3ExprDelete($13,$15);
   STACKTOP = sp;return;
   break;
  }
  case 226: case 224: case 218: case 209: case 208: case 207: case 204: case 202: case 199: case 187: case 186: case 177:  {
   $16 = HEAP32[$pParse>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   _sqlite3ExprListDelete($17,$19);
   STACKTOP = sp;return;
   break;
  }
  case 212: case 211: case 200: case 193:  {
   $20 = HEAP32[$pParse>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$2>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   _sqlite3SrcListDelete($21,$23);
   STACKTOP = sp;return;
   break;
  }
  case 250: case 196:  {
   $24 = HEAP32[$pParse>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   _sqlite3WithDelete($25,$27);
   STACKTOP = sp;return;
   break;
  }
  case 241: case 236: case 227: case 225: case 215: case 203: case 201:  {
   $28 = HEAP32[$pParse>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$2>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   _sqlite3ExprDelete($29,$31);
   STACKTOP = sp;return;
   break;
  }
  case 220: case 217: case 216:  {
   $32 = HEAP32[$pParse>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   _sqlite3IdListDelete($33,$35);
   STACKTOP = sp;return;
   break;
  }
  case 237: case 232:  {
   $36 = HEAP32[$pParse>>2]|0;
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   _sqlite3DeleteTriggerStep($37,$39);
   STACKTOP = sp;return;
   break;
  }
  case 234:  {
   $40 = HEAP32[$pParse>>2]|0;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$2>>2]|0;
   $43 = ((($42)) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   _sqlite3IdListDelete($41,$44);
   STACKTOP = sp;return;
   break;
  }
  default: {
   STACKTOP = sp;return;
  }
  }
 } while(0);
}
function _sqlite3DeleteTriggerStep($db,$pTriggerStep) {
 $db = $db|0;
 $pTriggerStep = $pTriggerStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTriggerStep;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$1>>2]|0;
  HEAP32[$pTmp>>2] = $4;
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 28|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$1>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$pTmp>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  _sqlite3ExprDelete($8,$11);
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$pTmp>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3ExprListDelete($12,$15);
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pTmp>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  _sqlite3SelectDelete($16,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$pTmp>>2]|0;
  $22 = ((($21)) + 24|0);
  $23 = HEAP32[$22>>2]|0;
  _sqlite3IdListDelete($20,$23);
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$pTmp>>2]|0;
  _sqlite3DbFree($24,$25);
 }
 STACKTOP = sp;return;
}
function _yy_find_shift_action($pParser,$iLookAhead) {
 $pParser = $pParser|0;
 $iLookAhead = $iLookAhead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iFallback = 0, $j = 0, $or$cond = 0, $stateno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 21|0;
 $i = sp + 8|0;
 $stateno = sp + 4|0;
 $iFallback = sp + 20|0;
 $j = sp;
 HEAP32[$1>>2] = $pParser;
 HEAP8[$2>>0] = $iLookAhead;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 HEAP32[$stateno>>2] = $6;
 $7 = HEAP32[$stateno>>2]|0;
 $8 = ($7|0)>=(1000);
 if ($8) {
  $9 = HEAP32[$stateno>>2]|0;
  HEAP32[$0>>2] = $9;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 while(1) {
  $10 = HEAP32[$stateno>>2]|0;
  $11 = (14594 + ($10<<1)|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  HEAP32[$i>>2] = $13;
  $14 = HEAP8[$2>>0]|0;
  $15 = $14&255;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + ($15))|0;
  HEAP32[$i>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = ($18|0)<(0);
  $20 = HEAP32[$i>>2]|0;
  $21 = ($20|0)>=(1567);
  $or$cond = $19 | $21;
  if (!($or$cond)) {
   $22 = HEAP32[$i>>2]|0;
   $23 = (37619 + ($22)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = HEAP8[$2>>0]|0;
   $27 = $26&255;
   $28 = ($25|0)!=($27|0);
   if (!($28)) {
    label = 13;
    break;
   }
  }
  $29 = HEAP8[$2>>0]|0;
  $30 = $29&255;
  $31 = ($30>>>0)<(96);
  if (!($31)) {
   break;
  }
  $32 = HEAP8[$2>>0]|0;
  $33 = $32&255;
  $34 = (39186 + ($33)|0);
  $35 = HEAP8[$34>>0]|0;
  HEAP8[$iFallback>>0] = $35;
  $36 = $35&255;
  $37 = ($36|0)!=(0);
  if (!($37)) {
   break;
  }
  $38 = HEAP8[$iFallback>>0]|0;
  HEAP8[$2>>0] = $38;
 }
 if ((label|0) == 13) {
  $62 = HEAP32[$i>>2]|0;
  $63 = (11456 + ($62<<1)|0);
  $64 = HEAP16[$63>>1]|0;
  $65 = $64&65535;
  HEAP32[$0>>2] = $65;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 $39 = HEAP32[$i>>2]|0;
 $40 = HEAP8[$2>>0]|0;
 $41 = $40&255;
 $42 = (($39) - ($41))|0;
 $43 = (($42) + 96)|0;
 HEAP32[$j>>2] = $43;
 $44 = HEAP32[$j>>2]|0;
 $45 = ($44|0)<(1567);
 if ($45) {
  $46 = HEAP32[$j>>2]|0;
  $47 = (37619 + ($46)|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = $48&255;
  $50 = ($49|0)==(96);
  if ($50) {
   $51 = HEAP8[$2>>0]|0;
   $52 = $51&255;
   $53 = ($52|0)>(0);
   if ($53) {
    $54 = HEAP32[$j>>2]|0;
    $55 = (11456 + ($54<<1)|0);
    $56 = HEAP16[$55>>1]|0;
    $57 = $56&65535;
    HEAP32[$0>>2] = $57;
    $66 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($66|0);
   }
  }
 }
 $58 = HEAP32[$stateno>>2]|0;
 $59 = (15506 + ($58<<1)|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60&65535;
 HEAP32[$0>>2] = $61;
 $66 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($66|0);
}
function _yy_shift($yypParser,$yyNewState,$yyMajor,$yyMinor) {
 $yypParser = $yypParser|0;
 $yyNewState = $yyNewState|0;
 $yyMajor = $yyMajor|0;
 $yyMinor = $yyMinor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $yytos = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $yytos = sp;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yyNewState;
 HEAP32[$2>>2] = $yyMajor;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 16|0);
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = ((($9)) + 1600|0);
 $11 = ($7>>>0)>=($10>>>0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  _yyStackOverflow($12);
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)>(455);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = (($15) + 332)|0;
  HEAP32[$1>>2] = $16;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$yytos>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = $19&65535;
 $21 = HEAP32[$yytos>>2]|0;
 HEAP16[$21>>1] = $20;
 $22 = HEAP32[$2>>2]|0;
 $23 = $22&255;
 $24 = HEAP32[$yytos>>2]|0;
 $25 = ((($24)) + 2|0);
 HEAP8[$25>>0] = $23;
 $26 = HEAP32[$yytos>>2]|0;
 $27 = ((($26)) + 4|0);
 ;HEAP32[$27>>2]=HEAP32[$yyMinor>>2]|0;HEAP32[$27+4>>2]=HEAP32[$yyMinor+4>>2]|0;
 STACKTOP = sp;return;
}
function _yy_reduce($yypParser,$yyruleno) {
 $yypParser = $yypParser|0;
 $yyruleno = $yyruleno|0;
 var $$byval_copy16 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0;
 var $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0;
 var $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0;
 var $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0;
 var $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0;
 var $1087 = 0, $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0;
 var $1104 = 0, $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0;
 var $1122 = 0, $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0;
 var $1140 = 0, $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0;
 var $1159 = 0, $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0;
 var $1177 = 0, $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0;
 var $1195 = 0, $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0;
 var $1212 = 0, $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0;
 var $1230 = 0, $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0;
 var $1249 = 0, $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0;
 var $1267 = 0, $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0;
 var $1285 = 0, $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0;
 var $1302 = 0, $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0;
 var $1320 = 0, $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0;
 var $1339 = 0, $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0;
 var $1357 = 0, $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0;
 var $1375 = 0, $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0;
 var $1393 = 0, $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0;
 var $1410 = 0, $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0;
 var $1429 = 0, $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0;
 var $1447 = 0, $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0;
 var $1465 = 0, $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0;
 var $1483 = 0, $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0;
 var $1500 = 0, $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0;
 var $1519 = 0, $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0;
 var $1537 = 0, $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0;
 var $1555 = 0, $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0;
 var $1573 = 0, $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0;
 var $1591 = 0, $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0;
 var $1609 = 0, $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0;
 var $1627 = 0, $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0;
 var $1645 = 0, $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0;
 var $1663 = 0, $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0;
 var $1681 = 0, $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0;
 var $17 = 0, $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0;
 var $1717 = 0, $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0;
 var $1735 = 0, $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0;
 var $1753 = 0, $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0;
 var $1771 = 0, $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0;
 var $179 = 0, $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0;
 var $1807 = 0, $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0;
 var $1825 = 0, $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0;
 var $1843 = 0, $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0;
 var $1861 = 0, $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0;
 var $188 = 0, $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0;
 var $1898 = 0, $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0;
 var $1915 = 0, $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0;
 var $1933 = 0, $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0;
 var $1951 = 0, $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0;
 var $197 = 0, $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0;
 var $1988 = 0, $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0;
 var $2004 = 0, $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0, $2021 = 0;
 var $2022 = 0, $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0, $204 = 0;
 var $2040 = 0, $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $2048 = 0, $2049 = 0, $205 = 0, $2050 = 0, $2051 = 0, $2052 = 0, $2053 = 0, $2054 = 0, $2055 = 0, $2056 = 0, $2057 = 0, $2058 = 0;
 var $2059 = 0, $206 = 0, $2060 = 0, $2061 = 0, $2062 = 0, $2063 = 0, $2064 = 0, $2065 = 0, $2066 = 0, $2067 = 0, $2068 = 0, $2069 = 0, $207 = 0, $2070 = 0, $2071 = 0, $2072 = 0, $2073 = 0, $2074 = 0, $2075 = 0, $2076 = 0;
 var $2077 = 0, $2078 = 0, $2079 = 0, $208 = 0, $2080 = 0, $2081 = 0, $2082 = 0, $2083 = 0, $2084 = 0, $2085 = 0, $2086 = 0, $2087 = 0, $2088 = 0, $2089 = 0, $209 = 0, $2090 = 0, $2091 = 0, $2092 = 0, $2093 = 0, $2094 = 0;
 var $2095 = 0, $2096 = 0, $2097 = 0, $2098 = 0, $2099 = 0, $21 = 0, $210 = 0, $2100 = 0, $2101 = 0, $2102 = 0, $2103 = 0, $2104 = 0, $2105 = 0, $2106 = 0, $2107 = 0, $2108 = 0, $2109 = 0, $211 = 0, $2110 = 0, $2111 = 0;
 var $2112 = 0, $2113 = 0, $2114 = 0, $2115 = 0, $2116 = 0, $2117 = 0, $2118 = 0, $2119 = 0, $212 = 0, $2120 = 0, $2121 = 0, $2122 = 0, $2123 = 0, $2124 = 0, $2125 = 0, $2126 = 0, $2127 = 0, $2128 = 0, $2129 = 0, $213 = 0;
 var $2130 = 0, $2131 = 0, $2132 = 0, $2133 = 0, $2134 = 0, $2135 = 0, $2136 = 0, $2137 = 0, $2138 = 0, $2139 = 0, $214 = 0, $2140 = 0, $2141 = 0, $2142 = 0, $2143 = 0, $2144 = 0, $2145 = 0, $2146 = 0, $2147 = 0, $2148 = 0;
 var $2149 = 0, $215 = 0, $2150 = 0, $2151 = 0, $2152 = 0, $2153 = 0, $2154 = 0, $2155 = 0, $2156 = 0, $2157 = 0, $2158 = 0, $2159 = 0, $216 = 0, $2160 = 0, $2161 = 0, $2162 = 0, $2163 = 0, $2164 = 0, $2165 = 0, $2166 = 0;
 var $2167 = 0, $2168 = 0, $2169 = 0, $217 = 0, $2170 = 0, $2171 = 0, $2172 = 0, $2173 = 0, $2174 = 0, $2175 = 0, $2176 = 0, $2177 = 0, $2178 = 0, $2179 = 0, $218 = 0, $2180 = 0, $2181 = 0, $2182 = 0, $2183 = 0, $2184 = 0;
 var $2185 = 0, $2186 = 0, $2187 = 0, $2188 = 0, $2189 = 0, $219 = 0, $2190 = 0, $2191 = 0, $2192 = 0, $2193 = 0, $2194 = 0, $2195 = 0, $2196 = 0, $2197 = 0, $2198 = 0, $2199 = 0, $22 = 0, $220 = 0, $2200 = 0, $2201 = 0;
 var $2202 = 0, $2203 = 0, $2204 = 0, $2205 = 0, $2206 = 0, $2207 = 0, $2208 = 0, $2209 = 0, $221 = 0, $2210 = 0, $2211 = 0, $2212 = 0, $2213 = 0, $2214 = 0, $2215 = 0, $2216 = 0, $2217 = 0, $2218 = 0, $2219 = 0, $222 = 0;
 var $2220 = 0, $2221 = 0, $2222 = 0, $2223 = 0, $2224 = 0, $2225 = 0, $2226 = 0, $2227 = 0, $2228 = 0, $2229 = 0, $223 = 0, $2230 = 0, $2231 = 0, $2232 = 0, $2233 = 0, $2234 = 0, $2235 = 0, $2236 = 0, $2237 = 0, $2238 = 0;
 var $2239 = 0, $224 = 0, $2240 = 0, $2241 = 0, $2242 = 0, $2243 = 0, $2244 = 0, $2245 = 0, $2246 = 0, $2247 = 0, $2248 = 0, $2249 = 0, $225 = 0, $2250 = 0, $2251 = 0, $2252 = 0, $2253 = 0, $2254 = 0, $2255 = 0, $2256 = 0;
 var $2257 = 0, $2258 = 0, $2259 = 0, $226 = 0, $2260 = 0, $2261 = 0, $2262 = 0, $2263 = 0, $2264 = 0, $2265 = 0, $2266 = 0, $2267 = 0, $2268 = 0, $2269 = 0, $227 = 0, $2270 = 0, $2271 = 0, $2272 = 0, $2273 = 0, $2274 = 0;
 var $2275 = 0, $2276 = 0, $2277 = 0, $2278 = 0, $2279 = 0, $228 = 0, $2280 = 0, $2281 = 0, $2282 = 0, $2283 = 0, $2284 = 0, $2285 = 0, $2286 = 0, $2287 = 0, $2288 = 0, $2289 = 0, $229 = 0, $2290 = 0, $2291 = 0, $2292 = 0;
 var $2293 = 0, $2294 = 0, $2295 = 0, $2296 = 0, $2297 = 0, $2298 = 0, $2299 = 0, $23 = 0, $230 = 0, $2300 = 0, $2301 = 0, $2302 = 0, $2303 = 0, $2304 = 0, $2305 = 0, $2306 = 0, $2307 = 0, $2308 = 0, $2309 = 0, $231 = 0;
 var $2310 = 0, $2311 = 0, $2312 = 0, $2313 = 0, $2314 = 0, $2315 = 0, $2316 = 0, $2317 = 0, $2318 = 0, $2319 = 0, $232 = 0, $2320 = 0, $2321 = 0, $2322 = 0, $2323 = 0, $2324 = 0, $2325 = 0, $2326 = 0, $2327 = 0, $2328 = 0;
 var $2329 = 0, $233 = 0, $2330 = 0, $2331 = 0, $2332 = 0, $2333 = 0, $2334 = 0, $2335 = 0, $2336 = 0, $2337 = 0, $2338 = 0, $2339 = 0, $234 = 0, $2340 = 0, $2341 = 0, $2342 = 0, $2343 = 0, $2344 = 0, $2345 = 0, $2346 = 0;
 var $2347 = 0, $2348 = 0, $2349 = 0, $235 = 0, $2350 = 0, $2351 = 0, $2352 = 0, $2353 = 0, $2354 = 0, $2355 = 0, $2356 = 0, $2357 = 0, $2358 = 0, $2359 = 0, $236 = 0, $2360 = 0, $2361 = 0, $2362 = 0, $2363 = 0, $2364 = 0;
 var $2365 = 0, $2366 = 0, $2367 = 0, $2368 = 0, $2369 = 0, $237 = 0, $2370 = 0, $2371 = 0, $2372 = 0, $2373 = 0, $2374 = 0, $2375 = 0, $2376 = 0, $2377 = 0, $2378 = 0, $2379 = 0, $238 = 0, $2380 = 0, $2381 = 0, $2382 = 0;
 var $2383 = 0, $2384 = 0, $2385 = 0, $2386 = 0, $2387 = 0, $2388 = 0, $2389 = 0, $239 = 0, $2390 = 0, $2391 = 0, $2392 = 0, $2393 = 0, $2394 = 0, $2395 = 0, $2396 = 0, $2397 = 0, $2398 = 0, $2399 = 0, $24 = 0, $240 = 0;
 var $2400 = 0, $2401 = 0, $2402 = 0, $2403 = 0, $2404 = 0, $2405 = 0, $2406 = 0, $2407 = 0, $2408 = 0, $2409 = 0, $241 = 0, $2410 = 0, $2411 = 0, $2412 = 0, $2413 = 0, $2414 = 0, $2415 = 0, $2416 = 0, $2417 = 0, $2418 = 0;
 var $2419 = 0, $242 = 0, $2420 = 0, $2421 = 0, $2422 = 0, $2423 = 0, $2424 = 0, $2425 = 0, $2426 = 0, $2427 = 0, $2428 = 0, $2429 = 0, $243 = 0, $2430 = 0, $2431 = 0, $2432 = 0, $2433 = 0, $2434 = 0, $2435 = 0, $2436 = 0;
 var $2437 = 0, $2438 = 0, $2439 = 0, $244 = 0, $2440 = 0, $2441 = 0, $2442 = 0, $2443 = 0, $2444 = 0, $2445 = 0, $2446 = 0, $2447 = 0, $2448 = 0, $2449 = 0, $245 = 0, $2450 = 0, $2451 = 0, $2452 = 0, $2453 = 0, $2454 = 0;
 var $2455 = 0, $2456 = 0, $2457 = 0, $2458 = 0, $2459 = 0, $246 = 0, $2460 = 0, $2461 = 0, $2462 = 0, $2463 = 0, $2464 = 0, $2465 = 0, $2466 = 0, $2467 = 0, $2468 = 0, $2469 = 0, $247 = 0, $2470 = 0, $2471 = 0, $2472 = 0;
 var $2473 = 0, $2474 = 0, $2475 = 0, $2476 = 0, $2477 = 0, $2478 = 0, $2479 = 0, $248 = 0, $2480 = 0, $2481 = 0, $2482 = 0, $2483 = 0, $2484 = 0, $2485 = 0, $2486 = 0, $2487 = 0, $2488 = 0, $2489 = 0, $249 = 0, $2490 = 0;
 var $2491 = 0, $2492 = 0, $2493 = 0, $2494 = 0, $2495 = 0, $2496 = 0, $2497 = 0, $2498 = 0, $2499 = 0, $25 = 0, $250 = 0, $2500 = 0, $2501 = 0, $2502 = 0, $2503 = 0, $2504 = 0, $2505 = 0, $2506 = 0, $2507 = 0, $2508 = 0;
 var $2509 = 0, $251 = 0, $2510 = 0, $2511 = 0, $2512 = 0, $2513 = 0, $2514 = 0, $2515 = 0, $2516 = 0, $2517 = 0, $2518 = 0, $2519 = 0, $252 = 0, $2520 = 0, $2521 = 0, $2522 = 0, $2523 = 0, $2524 = 0, $2525 = 0, $2526 = 0;
 var $2527 = 0, $2528 = 0, $2529 = 0, $253 = 0, $2530 = 0, $2531 = 0, $2532 = 0, $2533 = 0, $2534 = 0, $2535 = 0, $2536 = 0, $2537 = 0, $2538 = 0, $2539 = 0, $254 = 0, $2540 = 0, $2541 = 0, $2542 = 0, $2543 = 0, $2544 = 0;
 var $2545 = 0, $2546 = 0, $2547 = 0, $2548 = 0, $2549 = 0, $255 = 0, $2550 = 0, $2551 = 0, $2552 = 0, $2553 = 0, $2554 = 0, $2555 = 0, $2556 = 0, $2557 = 0, $2558 = 0, $2559 = 0, $256 = 0, $2560 = 0, $2561 = 0, $2562 = 0;
 var $2563 = 0, $2564 = 0, $2565 = 0, $2566 = 0, $2567 = 0, $2568 = 0, $2569 = 0, $257 = 0, $2570 = 0, $2571 = 0, $2572 = 0, $2573 = 0, $2574 = 0, $2575 = 0, $2576 = 0, $2577 = 0, $2578 = 0, $2579 = 0, $258 = 0, $2580 = 0;
 var $2581 = 0, $2582 = 0, $2583 = 0, $2584 = 0, $2585 = 0, $2586 = 0, $2587 = 0, $2588 = 0, $2589 = 0, $259 = 0, $2590 = 0, $2591 = 0, $2592 = 0, $2593 = 0, $2594 = 0, $2595 = 0, $2596 = 0, $2597 = 0, $2598 = 0, $2599 = 0;
 var $26 = 0, $260 = 0, $2600 = 0, $2601 = 0, $2602 = 0, $2603 = 0, $2604 = 0, $2605 = 0, $2606 = 0, $2607 = 0, $2608 = 0, $2609 = 0, $261 = 0, $2610 = 0, $2611 = 0, $2612 = 0, $2613 = 0, $2614 = 0, $2615 = 0, $2616 = 0;
 var $2617 = 0, $2618 = 0, $2619 = 0, $262 = 0, $2620 = 0, $2621 = 0, $2622 = 0, $2623 = 0, $2624 = 0, $2625 = 0, $2626 = 0, $2627 = 0, $2628 = 0, $2629 = 0, $263 = 0, $2630 = 0, $2631 = 0, $2632 = 0, $2633 = 0, $2634 = 0;
 var $2635 = 0, $2636 = 0, $2637 = 0, $2638 = 0, $2639 = 0, $264 = 0, $2640 = 0, $2641 = 0, $2642 = 0, $2643 = 0, $2644 = 0, $2645 = 0, $2646 = 0, $2647 = 0, $2648 = 0, $2649 = 0, $265 = 0, $2650 = 0, $2651 = 0, $2652 = 0;
 var $2653 = 0, $2654 = 0, $2655 = 0, $2656 = 0, $2657 = 0, $2658 = 0, $2659 = 0, $266 = 0, $2660 = 0, $2661 = 0, $2662 = 0, $2663 = 0, $2664 = 0, $2665 = 0, $2666 = 0, $2667 = 0, $2668 = 0, $2669 = 0, $267 = 0, $2670 = 0;
 var $2671 = 0, $2672 = 0, $2673 = 0, $2674 = 0, $2675 = 0, $2676 = 0, $2677 = 0, $2678 = 0, $2679 = 0, $268 = 0, $2680 = 0, $2681 = 0, $2682 = 0, $2683 = 0, $2684 = 0, $2685 = 0, $2686 = 0, $2687 = 0, $2688 = 0, $2689 = 0;
 var $269 = 0, $2690 = 0, $2691 = 0, $2692 = 0, $2693 = 0, $2694 = 0, $2695 = 0, $2696 = 0, $2697 = 0, $2698 = 0, $2699 = 0, $27 = 0, $270 = 0, $2700 = 0, $2701 = 0, $2702 = 0, $2703 = 0, $2704 = 0, $2705 = 0, $2706 = 0;
 var $2707 = 0, $2708 = 0, $2709 = 0, $271 = 0, $2710 = 0, $2711 = 0, $2712 = 0, $2713 = 0, $2714 = 0, $2715 = 0, $2716 = 0, $2717 = 0, $2718 = 0, $2719 = 0, $272 = 0, $2720 = 0, $2721 = 0, $2722 = 0, $2723 = 0, $2724 = 0;
 var $2725 = 0, $2726 = 0, $2727 = 0, $2728 = 0, $2729 = 0, $273 = 0, $2730 = 0, $2731 = 0, $2732 = 0, $2733 = 0, $2734 = 0, $2735 = 0, $2736 = 0, $2737 = 0, $2738 = 0, $2739 = 0, $274 = 0, $2740 = 0, $2741 = 0, $2742 = 0;
 var $2743 = 0, $2744 = 0, $2745 = 0, $2746 = 0, $2747 = 0, $2748 = 0, $2749 = 0, $275 = 0, $2750 = 0, $2751 = 0, $2752 = 0, $2753 = 0, $2754 = 0, $2755 = 0, $2756 = 0, $2757 = 0, $2758 = 0, $2759 = 0, $276 = 0, $2760 = 0;
 var $2761 = 0, $2762 = 0, $2763 = 0, $2764 = 0, $2765 = 0, $2766 = 0, $2767 = 0, $2768 = 0, $2769 = 0, $277 = 0, $2770 = 0, $2771 = 0, $2772 = 0, $2773 = 0, $2774 = 0, $2775 = 0, $2776 = 0, $2777 = 0, $2778 = 0, $2779 = 0;
 var $278 = 0, $2780 = 0, $2781 = 0, $2782 = 0, $2783 = 0, $2784 = 0, $2785 = 0, $2786 = 0, $2787 = 0, $2788 = 0, $2789 = 0, $279 = 0, $2790 = 0, $2791 = 0, $2792 = 0, $2793 = 0, $2794 = 0, $2795 = 0, $2796 = 0, $2797 = 0;
 var $2798 = 0, $2799 = 0, $28 = 0, $280 = 0, $2800 = 0, $2801 = 0, $2802 = 0, $2803 = 0, $2804 = 0, $2805 = 0, $2806 = 0, $2807 = 0, $2808 = 0, $2809 = 0, $281 = 0, $2810 = 0, $2811 = 0, $2812 = 0, $2813 = 0, $2814 = 0;
 var $2815 = 0, $2816 = 0, $2817 = 0, $2818 = 0, $2819 = 0, $282 = 0, $2820 = 0, $2821 = 0, $2822 = 0, $2823 = 0, $2824 = 0, $2825 = 0, $2826 = 0, $2827 = 0, $2828 = 0, $2829 = 0, $283 = 0, $2830 = 0, $2831 = 0, $2832 = 0;
 var $2833 = 0, $2834 = 0, $2835 = 0, $2836 = 0, $2837 = 0, $2838 = 0, $2839 = 0, $284 = 0, $2840 = 0, $2841 = 0, $2842 = 0, $2843 = 0, $2844 = 0, $2845 = 0, $2846 = 0, $2847 = 0, $2848 = 0, $2849 = 0, $285 = 0, $2850 = 0;
 var $2851 = 0, $2852 = 0, $2853 = 0, $2854 = 0, $2855 = 0, $2856 = 0, $2857 = 0, $2858 = 0, $2859 = 0, $286 = 0, $2860 = 0, $2861 = 0, $2862 = 0, $2863 = 0, $2864 = 0, $2865 = 0, $2866 = 0, $2867 = 0, $2868 = 0, $2869 = 0;
 var $287 = 0, $2870 = 0, $2871 = 0, $2872 = 0, $2873 = 0, $2874 = 0, $2875 = 0, $2876 = 0, $2877 = 0, $2878 = 0, $2879 = 0, $288 = 0, $2880 = 0, $2881 = 0, $2882 = 0, $2883 = 0, $2884 = 0, $2885 = 0, $2886 = 0, $2887 = 0;
 var $2888 = 0, $2889 = 0, $289 = 0, $2890 = 0, $2891 = 0, $2892 = 0, $2893 = 0, $2894 = 0, $2895 = 0, $2896 = 0, $2897 = 0, $2898 = 0, $2899 = 0, $29 = 0, $290 = 0, $2900 = 0, $2901 = 0, $2902 = 0, $2903 = 0, $2904 = 0;
 var $2905 = 0, $2906 = 0, $2907 = 0, $2908 = 0, $2909 = 0, $291 = 0, $2910 = 0, $2911 = 0, $2912 = 0, $2913 = 0, $2914 = 0, $2915 = 0, $2916 = 0, $2917 = 0, $2918 = 0, $2919 = 0, $292 = 0, $2920 = 0, $2921 = 0, $2922 = 0;
 var $2923 = 0, $2924 = 0, $2925 = 0, $2926 = 0, $2927 = 0, $2928 = 0, $2929 = 0, $293 = 0, $2930 = 0, $2931 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0;
 var $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0;
 var $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0;
 var $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0;
 var $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0;
 var $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0;
 var $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0;
 var $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0;
 var $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0;
 var $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0;
 var $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0;
 var $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0;
 var $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0;
 var $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0;
 var $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0;
 var $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0;
 var $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0;
 var $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0;
 var $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0;
 var $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0;
 var $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0;
 var $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0;
 var $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0;
 var $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0;
 var $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0;
 var $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0;
 var $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0;
 var $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0;
 var $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0;
 var $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0;
 var $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0;
 var $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0;
 var $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0;
 var $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0;
 var $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0;
 var $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0;
 var $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0;
 var $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0;
 var $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0;
 var $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $all = 0, $bNot = 0, $bNot9 = 0, $dest = 0, $n = 0, $p = 0, $p11 = 0;
 var $p2 = 0, $pDot = 0, $pFrom = 0, $pLeft = 0, $pLeft4 = 0, $pLhs = 0, $pList = 0, $pList10 = 0, $pList7 = 0, $pList8 = 0, $pNew = 0, $pOld = 0, $pParse = 0, $pRHS = 0, $pRhs = 0, $pRight = 0, $pRight3 = 0, $pSelect = 0, $pSrc = 0, $pSubquery = 0;
 var $t = 0, $temp1 = 0, $temp15 = 0, $temp2 = 0, $temp26 = 0, $temp3 = 0, $temp4 = 0, $v = 0, $v1 = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer12 = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $x = 0, $yyact = 0, $yygoto = 0, $yylhsminor = 0;
 var $yymsp = 0, $yysize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 288|0;
 $$byval_copy16 = sp + 280|0;
 $vararg_buffer12 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 276|0;
 $1 = sp + 272|0;
 $yygoto = sp + 268|0;
 $yyact = sp + 264|0;
 $yymsp = sp + 260|0;
 $yysize = sp + 256|0;
 $pParse = sp + 252|0;
 $yylhsminor = sp + 240|0;
 $v = sp + 228|0;
 $v1 = sp + 216|0;
 $dest = sp + 192|0;
 $p = sp + 188|0;
 $pRhs = sp + 184|0;
 $pLhs = sp + 180|0;
 $pFrom = sp + 176|0;
 $x = sp + 168|0;
 $pRight = sp + 164|0;
 $pLeft = sp + 160|0;
 $p2 = sp + 156|0;
 $pRight3 = sp + 152|0;
 $pLeft4 = sp + 148|0;
 $pDot = sp + 144|0;
 $pNew = sp + 140|0;
 $pOld = sp + 136|0;
 $pSubquery = sp + 132|0;
 $temp1 = sp + 128|0;
 $temp2 = sp + 124|0;
 $temp15 = sp + 120|0;
 $temp26 = sp + 116|0;
 $temp3 = sp + 112|0;
 $temp4 = sp + 108|0;
 $n = sp + 104|0;
 $t = sp + 96|0;
 $pList = sp + 92|0;
 $pList7 = sp + 88|0;
 $bNot = sp + 84|0;
 $pList8 = sp + 80|0;
 $bNot9 = sp + 76|0;
 $pList10 = sp + 72|0;
 $pRHS = sp + 68|0;
 $pSrc = sp + 64|0;
 $pSelect = sp + 60|0;
 $p11 = sp + 56|0;
 $all = sp + 48|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yyruleno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$yymsp>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = (26486 + ($7<<1)|0);
 $9 = ((($8)) + 1|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = ((($16)) + 1584|0);
  $18 = ($14>>>0)>=($17>>>0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   _yyStackOverflow($19);
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$1>>2]|0;
 L6: do {
  switch ($20|0) {
  case 0:  {
   $21 = HEAP32[$pParse>>2]|0;
   $22 = ((($21)) + 409|0);
   HEAP8[$22>>0] = 1;
   break;
  }
  case 1:  {
   $23 = HEAP32[$pParse>>2]|0;
   $24 = ((($23)) + 409|0);
   HEAP8[$24>>0] = 2;
   break;
  }
  case 2:  {
   $25 = HEAP32[$pParse>>2]|0;
   _sqlite3FinishCoding($25);
   break;
  }
  case 3:  {
   $26 = HEAP32[$pParse>>2]|0;
   $27 = HEAP32[$yymsp>>2]|0;
   $28 = ((($27)) + -16|0);
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   _sqlite3BeginTransaction($26,$30);
   break;
  }
  case 4:  {
   $31 = HEAP32[$yymsp>>2]|0;
   $32 = ((($31)) + 16|0);
   $33 = ((($32)) + 4|0);
   HEAP32[$33>>2] = 7;
   break;
  }
  case 7: case 6: case 5:  {
   $34 = HEAP32[$yymsp>>2]|0;
   $35 = ((($34)) + 2|0);
   $36 = HEAP8[$35>>0]|0;
   $37 = $36&255;
   $38 = HEAP32[$yymsp>>2]|0;
   $39 = ((($38)) + 4|0);
   HEAP32[$39>>2] = $37;
   break;
  }
  case 9: case 8:  {
   $40 = HEAP32[$pParse>>2]|0;
   _sqlite3CommitTransaction($40);
   break;
  }
  case 10:  {
   $41 = HEAP32[$pParse>>2]|0;
   _sqlite3RollbackTransaction($41);
   break;
  }
  case 11:  {
   $42 = HEAP32[$pParse>>2]|0;
   $43 = HEAP32[$yymsp>>2]|0;
   $44 = ((($43)) + 4|0);
   _sqlite3Savepoint($42,0,$44);
   break;
  }
  case 12:  {
   $45 = HEAP32[$pParse>>2]|0;
   $46 = HEAP32[$yymsp>>2]|0;
   $47 = ((($46)) + 4|0);
   _sqlite3Savepoint($45,1,$47);
   break;
  }
  case 13:  {
   $48 = HEAP32[$pParse>>2]|0;
   $49 = HEAP32[$yymsp>>2]|0;
   $50 = ((($49)) + 4|0);
   _sqlite3Savepoint($48,2,$50);
   break;
  }
  case 14:  {
   $51 = HEAP32[$pParse>>2]|0;
   $52 = HEAP32[$yymsp>>2]|0;
   $53 = ((($52)) + -16|0);
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$yymsp>>2]|0;
   $56 = ((($55)) + 4|0);
   $57 = HEAP32[$yymsp>>2]|0;
   $58 = ((($57)) + -64|0);
   $59 = ((($58)) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = HEAP32[$yymsp>>2]|0;
   $62 = ((($61)) + -32|0);
   $63 = ((($62)) + 4|0);
   $64 = HEAP32[$63>>2]|0;
   _sqlite3StartTable($51,$54,$56,$60,0,0,$64);
   break;
  }
  case 15:  {
   $65 = HEAP32[$pParse>>2]|0;
   _disableLookaside($65);
   break;
  }
  case 215: case 90: case 76: case 67: case 57: case 42: case 22: case 19: case 16:  {
   $66 = HEAP32[$yymsp>>2]|0;
   $67 = ((($66)) + 16|0);
   $68 = ((($67)) + 4|0);
   HEAP32[$68>>2] = 0;
   break;
  }
  case 17:  {
   $69 = HEAP32[$yymsp>>2]|0;
   $70 = ((($69)) + -32|0);
   $71 = ((($70)) + 4|0);
   HEAP32[$71>>2] = 1;
   break;
  }
  case 43: case 18:  {
   $72 = HEAP32[$yymsp>>2]|0;
   $73 = ((($72)) + 4|0);
   HEAP32[$73>>2] = 1;
   break;
  }
  case 20:  {
   $74 = HEAP32[$pParse>>2]|0;
   $75 = HEAP32[$yymsp>>2]|0;
   $76 = ((($75)) + -32|0);
   $77 = ((($76)) + 4|0);
   $78 = HEAP32[$yymsp>>2]|0;
   $79 = ((($78)) + -16|0);
   $80 = ((($79)) + 4|0);
   $81 = HEAP32[$yymsp>>2]|0;
   $82 = ((($81)) + 4|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = $83&255;
   _sqlite3EndTable($74,$77,$80,$84,0);
   break;
  }
  case 21:  {
   $85 = HEAP32[$pParse>>2]|0;
   $86 = HEAP32[$yymsp>>2]|0;
   $87 = ((($86)) + 4|0);
   $88 = HEAP32[$87>>2]|0;
   _sqlite3EndTable($85,0,0,0,$88);
   $89 = HEAP32[$pParse>>2]|0;
   $90 = HEAP32[$89>>2]|0;
   $91 = HEAP32[$yymsp>>2]|0;
   $92 = ((($91)) + 4|0);
   $93 = HEAP32[$92>>2]|0;
   _sqlite3SelectDelete($90,$93);
   break;
  }
  case 23:  {
   $94 = HEAP32[$yymsp>>2]|0;
   $95 = ((($94)) + 4|0);
   $96 = ((($95)) + 4|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = ($97|0)==(5);
   if ($98) {
    $99 = HEAP32[$yymsp>>2]|0;
    $100 = ((($99)) + 4|0);
    $101 = HEAP32[$100>>2]|0;
    $102 = (_sqlite3_strnicmp($101,24745,5)|0);
    $103 = ($102|0)==(0);
    if ($103) {
     $104 = HEAP32[$yymsp>>2]|0;
     $105 = ((($104)) + -16|0);
     $106 = ((($105)) + 4|0);
     HEAP32[$106>>2] = 96;
     break L6;
    }
   }
   $107 = HEAP32[$yymsp>>2]|0;
   $108 = ((($107)) + -16|0);
   $109 = ((($108)) + 4|0);
   HEAP32[$109>>2] = 0;
   $110 = HEAP32[$pParse>>2]|0;
   $111 = HEAP32[$yymsp>>2]|0;
   $112 = ((($111)) + 4|0);
   $113 = ((($112)) + 4|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = HEAP32[$yymsp>>2]|0;
   $116 = ((($115)) + 4|0);
   $117 = HEAP32[$116>>2]|0;
   HEAP32[$vararg_buffer>>2] = $114;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $117;
   _sqlite3ErrorMsg($110,27150,$vararg_buffer);
   break;
  }
  case 24:  {
   $118 = HEAP32[$pParse>>2]|0;
   $119 = HEAP32[$yymsp>>2]|0;
   $120 = ((($119)) + -16|0);
   $121 = ((($120)) + 4|0);
   $122 = HEAP32[$yymsp>>2]|0;
   $123 = ((($122)) + 4|0);
   _sqlite3AddColumn($118,$121,$123);
   break;
  }
  case 96: case 60: case 25:  {
   $124 = HEAP32[$yymsp>>2]|0;
   $125 = ((($124)) + 16|0);
   $126 = ((($125)) + 4|0);
   $127 = ((($126)) + 4|0);
   HEAP32[$127>>2] = 0;
   $128 = HEAP32[$yymsp>>2]|0;
   $129 = ((($128)) + 16|0);
   $130 = ((($129)) + 4|0);
   HEAP32[$130>>2] = 0;
   break;
  }
  case 26:  {
   $131 = HEAP32[$yymsp>>2]|0;
   $132 = ((($131)) + 4|0);
   $133 = ((($132)) + 4|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = HEAP32[$yymsp>>2]|0;
   $136 = ((($135)) + 4|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = (($137) + ($134)|0);
   $139 = HEAP32[$yymsp>>2]|0;
   $140 = ((($139)) + -48|0);
   $141 = ((($140)) + 4|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = $138;
   $144 = $142;
   $145 = (($143) - ($144))|0;
   $146 = HEAP32[$yymsp>>2]|0;
   $147 = ((($146)) + -48|0);
   $148 = ((($147)) + 4|0);
   $149 = ((($148)) + 4|0);
   HEAP32[$149>>2] = $145;
   break;
  }
  case 27:  {
   $150 = HEAP32[$yymsp>>2]|0;
   $151 = ((($150)) + 4|0);
   $152 = ((($151)) + 4|0);
   $153 = HEAP32[$152>>2]|0;
   $154 = HEAP32[$yymsp>>2]|0;
   $155 = ((($154)) + 4|0);
   $156 = HEAP32[$155>>2]|0;
   $157 = (($156) + ($153)|0);
   $158 = HEAP32[$yymsp>>2]|0;
   $159 = ((($158)) + -80|0);
   $160 = ((($159)) + 4|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = $157;
   $163 = $161;
   $164 = (($162) - ($163))|0;
   $165 = HEAP32[$yymsp>>2]|0;
   $166 = ((($165)) + -80|0);
   $167 = ((($166)) + 4|0);
   $168 = ((($167)) + 4|0);
   HEAP32[$168>>2] = $164;
   break;
  }
  case 28:  {
   $169 = HEAP32[$yymsp>>2]|0;
   $170 = ((($169)) + 4|0);
   $171 = ((($170)) + 4|0);
   $172 = HEAP32[$171>>2]|0;
   $173 = HEAP32[$yymsp>>2]|0;
   $174 = ((($173)) + 4|0);
   $175 = HEAP32[$174>>2]|0;
   $176 = HEAP32[$yymsp>>2]|0;
   $177 = ((($176)) + -16|0);
   $178 = ((($177)) + 4|0);
   $179 = HEAP32[$178>>2]|0;
   $180 = $175;
   $181 = $179;
   $182 = (($180) - ($181))|0;
   $183 = (($172) + ($182))|0;
   $184 = HEAP32[$yymsp>>2]|0;
   $185 = ((($184)) + -16|0);
   $186 = ((($185)) + 4|0);
   $187 = ((($186)) + 4|0);
   HEAP32[$187>>2] = $183;
   break;
  }
  case 62: case 29:  {
   $188 = HEAP32[$pParse>>2]|0;
   $189 = ((($188)) + 84|0);
   $190 = HEAP32[$yymsp>>2]|0;
   $191 = ((($190)) + 4|0);
   ;HEAP32[$189>>2]=HEAP32[$191>>2]|0;HEAP32[$189+4>>2]=HEAP32[$191+4>>2]|0;
   break;
  }
  case 32: case 30:  {
   $192 = HEAP32[$pParse>>2]|0;
   $193 = HEAP32[$yymsp>>2]|0;
   $194 = ((($193)) + 4|0);
   _sqlite3AddDefaultValue($192,$194);
   break;
  }
  case 31:  {
   $195 = HEAP32[$pParse>>2]|0;
   $196 = HEAP32[$yymsp>>2]|0;
   $197 = ((($196)) + -16|0);
   $198 = ((($197)) + 4|0);
   _sqlite3AddDefaultValue($195,$198);
   break;
  }
  case 33:  {
   $199 = HEAP32[$pParse>>2]|0;
   $200 = HEAP32[$yymsp>>2]|0;
   $201 = ((($200)) + 4|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = (_sqlite3PExpr($199,155,$202,0,0)|0);
   HEAP32[$v>>2] = $203;
   $204 = HEAP32[$yymsp>>2]|0;
   $205 = ((($204)) + -16|0);
   $206 = ((($205)) + 4|0);
   $207 = HEAP32[$206>>2]|0;
   $208 = ((($v)) + 4|0);
   HEAP32[$208>>2] = $207;
   $209 = HEAP32[$yymsp>>2]|0;
   $210 = ((($209)) + 4|0);
   $211 = ((($210)) + 8|0);
   $212 = HEAP32[$211>>2]|0;
   $213 = ((($v)) + 8|0);
   HEAP32[$213>>2] = $212;
   $214 = HEAP32[$pParse>>2]|0;
   _sqlite3AddDefaultValue($214,$v);
   break;
  }
  case 34:  {
   $215 = HEAP32[$pParse>>2]|0;
   $216 = HEAP32[$yymsp>>2]|0;
   $217 = ((($216)) + 4|0);
   ;HEAP32[$$byval_copy16>>2]=HEAP32[$217>>2]|0;HEAP32[$$byval_copy16+4>>2]=HEAP32[$217+4>>2]|0;
   _spanExpr($v1,$215,97,$$byval_copy16);
   $218 = HEAP32[$pParse>>2]|0;
   _sqlite3AddDefaultValue($218,$v1);
   break;
  }
  case 35:  {
   $219 = HEAP32[$pParse>>2]|0;
   $220 = HEAP32[$yymsp>>2]|0;
   $221 = ((($220)) + 4|0);
   $222 = HEAP32[$221>>2]|0;
   _sqlite3AddNotNull($219,$222);
   break;
  }
  case 36:  {
   $223 = HEAP32[$pParse>>2]|0;
   $224 = HEAP32[$yymsp>>2]|0;
   $225 = ((($224)) + -16|0);
   $226 = ((($225)) + 4|0);
   $227 = HEAP32[$226>>2]|0;
   $228 = HEAP32[$yymsp>>2]|0;
   $229 = ((($228)) + 4|0);
   $230 = HEAP32[$229>>2]|0;
   $231 = HEAP32[$yymsp>>2]|0;
   $232 = ((($231)) + -32|0);
   $233 = ((($232)) + 4|0);
   $234 = HEAP32[$233>>2]|0;
   _sqlite3AddPrimaryKey($223,0,$227,$230,$234);
   break;
  }
  case 37:  {
   $235 = HEAP32[$pParse>>2]|0;
   $236 = HEAP32[$yymsp>>2]|0;
   $237 = ((($236)) + 4|0);
   $238 = HEAP32[$237>>2]|0;
   _sqlite3CreateIndex($235,0,0,0,0,$238,0,0,0,0,1);
   break;
  }
  case 38:  {
   $239 = HEAP32[$pParse>>2]|0;
   $240 = HEAP32[$yymsp>>2]|0;
   $241 = ((($240)) + -16|0);
   $242 = ((($241)) + 4|0);
   $243 = HEAP32[$242>>2]|0;
   _sqlite3AddCheckConstraint($239,$243);
   break;
  }
  case 39:  {
   $244 = HEAP32[$pParse>>2]|0;
   $245 = HEAP32[$yymsp>>2]|0;
   $246 = ((($245)) + -32|0);
   $247 = ((($246)) + 4|0);
   $248 = HEAP32[$yymsp>>2]|0;
   $249 = ((($248)) + -16|0);
   $250 = ((($249)) + 4|0);
   $251 = HEAP32[$250>>2]|0;
   $252 = HEAP32[$yymsp>>2]|0;
   $253 = ((($252)) + 4|0);
   $254 = HEAP32[$253>>2]|0;
   _sqlite3CreateForeignKey($244,0,$247,$251,$254);
   break;
  }
  case 40:  {
   $255 = HEAP32[$pParse>>2]|0;
   $256 = HEAP32[$yymsp>>2]|0;
   $257 = ((($256)) + 4|0);
   $258 = HEAP32[$257>>2]|0;
   _sqlite3DeferForeignKey($255,$258);
   break;
  }
  case 41:  {
   $259 = HEAP32[$pParse>>2]|0;
   $260 = HEAP32[$yymsp>>2]|0;
   $261 = ((($260)) + 4|0);
   _sqlite3AddCollateType($259,$261);
   break;
  }
  case 44:  {
   $262 = HEAP32[$yymsp>>2]|0;
   $263 = ((($262)) + 16|0);
   $264 = ((($263)) + 4|0);
   HEAP32[$264>>2] = 0;
   break;
  }
  case 45:  {
   $265 = HEAP32[$yymsp>>2]|0;
   $266 = ((($265)) + -16|0);
   $267 = ((($266)) + 4|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = HEAP32[$yymsp>>2]|0;
   $270 = ((($269)) + 4|0);
   $271 = ((($270)) + 4|0);
   $272 = HEAP32[$271>>2]|0;
   $273 = $272 ^ -1;
   $274 = $268 & $273;
   $275 = HEAP32[$yymsp>>2]|0;
   $276 = ((($275)) + 4|0);
   $277 = HEAP32[$276>>2]|0;
   $278 = $274 | $277;
   $279 = HEAP32[$yymsp>>2]|0;
   $280 = ((($279)) + -16|0);
   $281 = ((($280)) + 4|0);
   HEAP32[$281>>2] = $278;
   break;
  }
  case 46:  {
   $282 = HEAP32[$yymsp>>2]|0;
   $283 = ((($282)) + -16|0);
   $284 = ((($283)) + 4|0);
   HEAP32[$284>>2] = 0;
   $285 = HEAP32[$yymsp>>2]|0;
   $286 = ((($285)) + -16|0);
   $287 = ((($286)) + 4|0);
   $288 = ((($287)) + 4|0);
   HEAP32[$288>>2] = 0;
   break;
  }
  case 47:  {
   $289 = HEAP32[$yymsp>>2]|0;
   $290 = ((($289)) + -32|0);
   $291 = ((($290)) + 4|0);
   HEAP32[$291>>2] = 0;
   $292 = HEAP32[$yymsp>>2]|0;
   $293 = ((($292)) + -32|0);
   $294 = ((($293)) + 4|0);
   $295 = ((($294)) + 4|0);
   HEAP32[$295>>2] = 0;
   break;
  }
  case 48:  {
   $296 = HEAP32[$yymsp>>2]|0;
   $297 = ((($296)) + 4|0);
   $298 = HEAP32[$297>>2]|0;
   $299 = HEAP32[$yymsp>>2]|0;
   $300 = ((($299)) + -32|0);
   $301 = ((($300)) + 4|0);
   HEAP32[$301>>2] = $298;
   $302 = HEAP32[$yymsp>>2]|0;
   $303 = ((($302)) + -32|0);
   $304 = ((($303)) + 4|0);
   $305 = ((($304)) + 4|0);
   HEAP32[$305>>2] = 255;
   break;
  }
  case 49:  {
   $306 = HEAP32[$yymsp>>2]|0;
   $307 = ((($306)) + 4|0);
   $308 = HEAP32[$307>>2]|0;
   $309 = $308 << 8;
   $310 = HEAP32[$yymsp>>2]|0;
   $311 = ((($310)) + -32|0);
   $312 = ((($311)) + 4|0);
   HEAP32[$312>>2] = $309;
   $313 = HEAP32[$yymsp>>2]|0;
   $314 = ((($313)) + -32|0);
   $315 = ((($314)) + 4|0);
   $316 = ((($315)) + 4|0);
   HEAP32[$316>>2] = 65280;
   break;
  }
  case 50:  {
   $317 = HEAP32[$yymsp>>2]|0;
   $318 = ((($317)) + -16|0);
   $319 = ((($318)) + 4|0);
   HEAP32[$319>>2] = 7;
   break;
  }
  case 51:  {
   $320 = HEAP32[$yymsp>>2]|0;
   $321 = ((($320)) + -16|0);
   $322 = ((($321)) + 4|0);
   HEAP32[$322>>2] = 8;
   break;
  }
  case 52:  {
   $323 = HEAP32[$yymsp>>2]|0;
   $324 = ((($323)) + 4|0);
   HEAP32[$324>>2] = 9;
   break;
  }
  case 53:  {
   $325 = HEAP32[$yymsp>>2]|0;
   $326 = ((($325)) + 4|0);
   HEAP32[$326>>2] = 6;
   break;
  }
  case 54:  {
   $327 = HEAP32[$yymsp>>2]|0;
   $328 = ((($327)) + -16|0);
   $329 = ((($328)) + 4|0);
   HEAP32[$329>>2] = 0;
   break;
  }
  case 55:  {
   $330 = HEAP32[$yymsp>>2]|0;
   $331 = ((($330)) + -32|0);
   $332 = ((($331)) + 4|0);
   HEAP32[$332>>2] = 0;
   break;
  }
  case 144: case 71: case 56:  {
   $333 = HEAP32[$yymsp>>2]|0;
   $334 = ((($333)) + 4|0);
   $335 = HEAP32[$334>>2]|0;
   $336 = HEAP32[$yymsp>>2]|0;
   $337 = ((($336)) + -16|0);
   $338 = ((($337)) + 4|0);
   HEAP32[$338>>2] = $335;
   break;
  }
  case 216: case 190: case 187: case 75: case 58:  {
   $339 = HEAP32[$yymsp>>2]|0;
   $340 = ((($339)) + -16|0);
   $341 = ((($340)) + 4|0);
   HEAP32[$341>>2] = 1;
   break;
  }
  case 59:  {
   $342 = HEAP32[$yymsp>>2]|0;
   $343 = ((($342)) + -16|0);
   $344 = ((($343)) + 4|0);
   HEAP32[$344>>2] = 0;
   break;
  }
  case 61:  {
   $345 = HEAP32[$pParse>>2]|0;
   $346 = ((($345)) + 84|0);
   $347 = ((($346)) + 4|0);
   HEAP32[$347>>2] = 0;
   break;
  }
  case 63:  {
   $348 = HEAP32[$pParse>>2]|0;
   $349 = HEAP32[$yymsp>>2]|0;
   $350 = ((($349)) + -48|0);
   $351 = ((($350)) + 4|0);
   $352 = HEAP32[$351>>2]|0;
   $353 = HEAP32[$yymsp>>2]|0;
   $354 = ((($353)) + 4|0);
   $355 = HEAP32[$354>>2]|0;
   $356 = HEAP32[$yymsp>>2]|0;
   $357 = ((($356)) + -32|0);
   $358 = ((($357)) + 4|0);
   $359 = HEAP32[$358>>2]|0;
   _sqlite3AddPrimaryKey($348,$352,$355,$359,0);
   break;
  }
  case 64:  {
   $360 = HEAP32[$pParse>>2]|0;
   $361 = HEAP32[$yymsp>>2]|0;
   $362 = ((($361)) + -32|0);
   $363 = ((($362)) + 4|0);
   $364 = HEAP32[$363>>2]|0;
   $365 = HEAP32[$yymsp>>2]|0;
   $366 = ((($365)) + 4|0);
   $367 = HEAP32[$366>>2]|0;
   _sqlite3CreateIndex($360,0,0,0,$364,$367,0,0,0,0,1);
   break;
  }
  case 65:  {
   $368 = HEAP32[$pParse>>2]|0;
   $369 = HEAP32[$yymsp>>2]|0;
   $370 = ((($369)) + -32|0);
   $371 = ((($370)) + 4|0);
   $372 = HEAP32[$371>>2]|0;
   _sqlite3AddCheckConstraint($368,$372);
   break;
  }
  case 66:  {
   $373 = HEAP32[$pParse>>2]|0;
   $374 = HEAP32[$yymsp>>2]|0;
   $375 = ((($374)) + -96|0);
   $376 = ((($375)) + 4|0);
   $377 = HEAP32[$376>>2]|0;
   $378 = HEAP32[$yymsp>>2]|0;
   $379 = ((($378)) + -48|0);
   $380 = ((($379)) + 4|0);
   $381 = HEAP32[$yymsp>>2]|0;
   $382 = ((($381)) + -32|0);
   $383 = ((($382)) + 4|0);
   $384 = HEAP32[$383>>2]|0;
   $385 = HEAP32[$yymsp>>2]|0;
   $386 = ((($385)) + -16|0);
   $387 = ((($386)) + 4|0);
   $388 = HEAP32[$387>>2]|0;
   _sqlite3CreateForeignKey($373,$377,$380,$384,$388);
   $389 = HEAP32[$pParse>>2]|0;
   $390 = HEAP32[$yymsp>>2]|0;
   $391 = ((($390)) + 4|0);
   $392 = HEAP32[$391>>2]|0;
   _sqlite3DeferForeignKey($389,$392);
   break;
  }
  case 70: case 68:  {
   $393 = HEAP32[$yymsp>>2]|0;
   $394 = ((($393)) + 16|0);
   $395 = ((($394)) + 4|0);
   HEAP32[$395>>2] = 10;
   break;
  }
  case 69:  {
   $396 = HEAP32[$yymsp>>2]|0;
   $397 = ((($396)) + 4|0);
   $398 = HEAP32[$397>>2]|0;
   $399 = HEAP32[$yymsp>>2]|0;
   $400 = ((($399)) + -32|0);
   $401 = ((($400)) + 4|0);
   HEAP32[$401>>2] = $398;
   break;
  }
  case 72:  {
   $402 = HEAP32[$yymsp>>2]|0;
   $403 = ((($402)) + 4|0);
   HEAP32[$403>>2] = 4;
   break;
  }
  case 145: case 73:  {
   $404 = HEAP32[$yymsp>>2]|0;
   $405 = ((($404)) + 4|0);
   HEAP32[$405>>2] = 5;
   break;
  }
  case 74:  {
   $406 = HEAP32[$pParse>>2]|0;
   $407 = HEAP32[$yymsp>>2]|0;
   $408 = ((($407)) + 4|0);
   $409 = HEAP32[$408>>2]|0;
   $410 = HEAP32[$yymsp>>2]|0;
   $411 = ((($410)) + -16|0);
   $412 = ((($411)) + 4|0);
   $413 = HEAP32[$412>>2]|0;
   _sqlite3DropTable($406,$409,0,$413);
   break;
  }
  case 77:  {
   $414 = HEAP32[$pParse>>2]|0;
   $415 = HEAP32[$yymsp>>2]|0;
   $416 = ((($415)) + -128|0);
   $417 = ((($416)) + 4|0);
   $418 = HEAP32[$yymsp>>2]|0;
   $419 = ((($418)) + -64|0);
   $420 = ((($419)) + 4|0);
   $421 = HEAP32[$yymsp>>2]|0;
   $422 = ((($421)) + -48|0);
   $423 = ((($422)) + 4|0);
   $424 = HEAP32[$yymsp>>2]|0;
   $425 = ((($424)) + -32|0);
   $426 = ((($425)) + 4|0);
   $427 = HEAP32[$426>>2]|0;
   $428 = HEAP32[$yymsp>>2]|0;
   $429 = ((($428)) + 4|0);
   $430 = HEAP32[$429>>2]|0;
   $431 = HEAP32[$yymsp>>2]|0;
   $432 = ((($431)) + -112|0);
   $433 = ((($432)) + 4|0);
   $434 = HEAP32[$433>>2]|0;
   $435 = HEAP32[$yymsp>>2]|0;
   $436 = ((($435)) + -80|0);
   $437 = ((($436)) + 4|0);
   $438 = HEAP32[$437>>2]|0;
   _sqlite3CreateView($414,$417,$420,$423,$427,$430,$434,$438);
   break;
  }
  case 78:  {
   $439 = HEAP32[$pParse>>2]|0;
   $440 = HEAP32[$yymsp>>2]|0;
   $441 = ((($440)) + 4|0);
   $442 = HEAP32[$441>>2]|0;
   $443 = HEAP32[$yymsp>>2]|0;
   $444 = ((($443)) + -16|0);
   $445 = ((($444)) + 4|0);
   $446 = HEAP32[$445>>2]|0;
   _sqlite3DropTable($439,$442,1,$446);
   break;
  }
  case 79:  {
   ;HEAP32[$dest>>2]=HEAP32[4252>>2]|0;HEAP32[$dest+4>>2]=HEAP32[4252+4>>2]|0;HEAP32[$dest+8>>2]=HEAP32[4252+8>>2]|0;HEAP32[$dest+12>>2]=HEAP32[4252+12>>2]|0;HEAP32[$dest+16>>2]=HEAP32[4252+16>>2]|0;HEAP32[$dest+20>>2]=HEAP32[4252+20>>2]|0;
   $447 = HEAP32[$pParse>>2]|0;
   $448 = HEAP32[$yymsp>>2]|0;
   $449 = ((($448)) + 4|0);
   $450 = HEAP32[$449>>2]|0;
   (_sqlite3Select($447,$450,$dest)|0);
   $451 = HEAP32[$pParse>>2]|0;
   $452 = HEAP32[$451>>2]|0;
   $453 = HEAP32[$yymsp>>2]|0;
   $454 = ((($453)) + 4|0);
   $455 = HEAP32[$454>>2]|0;
   _sqlite3SelectDelete($452,$455);
   break;
  }
  case 80:  {
   $456 = HEAP32[$yymsp>>2]|0;
   $457 = ((($456)) + 4|0);
   $458 = HEAP32[$457>>2]|0;
   HEAP32[$p>>2] = $458;
   $459 = HEAP32[$p>>2]|0;
   $460 = ($459|0)!=(0|0);
   if ($460) {
    $461 = HEAP32[$yymsp>>2]|0;
    $462 = ((($461)) + -16|0);
    $463 = ((($462)) + 4|0);
    $464 = HEAP32[$463>>2]|0;
    $465 = HEAP32[$p>>2]|0;
    $466 = ((($465)) + 64|0);
    HEAP32[$466>>2] = $464;
    $467 = HEAP32[$pParse>>2]|0;
    $468 = HEAP32[$p>>2]|0;
    _parserDoubleLinkSelect($467,$468);
   } else {
    $469 = HEAP32[$pParse>>2]|0;
    $470 = HEAP32[$469>>2]|0;
    $471 = HEAP32[$yymsp>>2]|0;
    $472 = ((($471)) + -16|0);
    $473 = ((($472)) + 4|0);
    $474 = HEAP32[$473>>2]|0;
    _sqlite3WithDelete($470,$474);
   }
   $475 = HEAP32[$p>>2]|0;
   $476 = HEAP32[$yymsp>>2]|0;
   $477 = ((($476)) + -16|0);
   $478 = ((($477)) + 4|0);
   HEAP32[$478>>2] = $475;
   break;
  }
  case 81:  {
   $479 = HEAP32[$yymsp>>2]|0;
   $480 = ((($479)) + 4|0);
   $481 = HEAP32[$480>>2]|0;
   HEAP32[$pRhs>>2] = $481;
   $482 = HEAP32[$yymsp>>2]|0;
   $483 = ((($482)) + -32|0);
   $484 = ((($483)) + 4|0);
   $485 = HEAP32[$484>>2]|0;
   HEAP32[$pLhs>>2] = $485;
   $486 = HEAP32[$pRhs>>2]|0;
   $487 = ($486|0)!=(0|0);
   if ($487) {
    $488 = HEAP32[$pRhs>>2]|0;
    $489 = ((($488)) + 48|0);
    $490 = HEAP32[$489>>2]|0;
    $491 = ($490|0)!=(0|0);
    if ($491) {
     $492 = ((($x)) + 4|0);
     HEAP32[$492>>2] = 0;
     $493 = HEAP32[$pParse>>2]|0;
     $494 = HEAP32[$pRhs>>2]|0;
     _parserDoubleLinkSelect($493,$494);
     $495 = HEAP32[$pParse>>2]|0;
     $496 = HEAP32[$pRhs>>2]|0;
     $497 = (_sqlite3SrcListAppendFromTerm($495,0,0,0,$x,$496,0,0)|0);
     HEAP32[$pFrom>>2] = $497;
     $498 = HEAP32[$pParse>>2]|0;
     $499 = HEAP32[$pFrom>>2]|0;
     $500 = (_sqlite3SelectNew($498,0,$499,0,0,0,0,0,0,0)|0);
     HEAP32[$pRhs>>2] = $500;
    }
   }
   $501 = HEAP32[$pRhs>>2]|0;
   $502 = ($501|0)!=(0|0);
   if ($502) {
    $503 = HEAP32[$yymsp>>2]|0;
    $504 = ((($503)) + -16|0);
    $505 = ((($504)) + 4|0);
    $506 = HEAP32[$505>>2]|0;
    $507 = $506&255;
    $508 = HEAP32[$pRhs>>2]|0;
    $509 = ((($508)) + 4|0);
    HEAP8[$509>>0] = $507;
    $510 = HEAP32[$pLhs>>2]|0;
    $511 = HEAP32[$pRhs>>2]|0;
    $512 = ((($511)) + 48|0);
    HEAP32[$512>>2] = $510;
    $513 = HEAP32[$pLhs>>2]|0;
    $514 = ($513|0)!=(0|0);
    if ($514) {
     $515 = HEAP32[$pLhs>>2]|0;
     $516 = ((($515)) + 8|0);
     $517 = HEAP32[$516>>2]|0;
     $518 = $517 & -1025;
     HEAP32[$516>>2] = $518;
    }
    $519 = HEAP32[$pRhs>>2]|0;
    $520 = ((($519)) + 8|0);
    $521 = HEAP32[$520>>2]|0;
    $522 = $521 & -1025;
    HEAP32[$520>>2] = $522;
    $523 = HEAP32[$yymsp>>2]|0;
    $524 = ((($523)) + -16|0);
    $525 = ((($524)) + 4|0);
    $526 = HEAP32[$525>>2]|0;
    $527 = ($526|0)!=(116);
    if ($527) {
     $528 = HEAP32[$pParse>>2]|0;
     $529 = ((($528)) + 22|0);
     HEAP8[$529>>0] = 1;
    }
   } else {
    $530 = HEAP32[$pParse>>2]|0;
    $531 = HEAP32[$530>>2]|0;
    $532 = HEAP32[$pLhs>>2]|0;
    _sqlite3SelectDelete($531,$532);
   }
   $533 = HEAP32[$pRhs>>2]|0;
   $534 = HEAP32[$yymsp>>2]|0;
   $535 = ((($534)) + -32|0);
   $536 = ((($535)) + 4|0);
   HEAP32[$536>>2] = $533;
   break;
  }
  case 84: case 82:  {
   $537 = HEAP32[$yymsp>>2]|0;
   $538 = ((($537)) + 2|0);
   $539 = HEAP8[$538>>0]|0;
   $540 = $539&255;
   $541 = HEAP32[$yymsp>>2]|0;
   $542 = ((($541)) + 4|0);
   HEAP32[$542>>2] = $540;
   break;
  }
  case 83:  {
   $543 = HEAP32[$yymsp>>2]|0;
   $544 = ((($543)) + -16|0);
   $545 = ((($544)) + 4|0);
   HEAP32[$545>>2] = 116;
   break;
  }
  case 85:  {
   $546 = HEAP32[$pParse>>2]|0;
   $547 = HEAP32[$yymsp>>2]|0;
   $548 = ((($547)) + -96|0);
   $549 = ((($548)) + 4|0);
   $550 = HEAP32[$549>>2]|0;
   $551 = HEAP32[$yymsp>>2]|0;
   $552 = ((($551)) + -80|0);
   $553 = ((($552)) + 4|0);
   $554 = HEAP32[$553>>2]|0;
   $555 = HEAP32[$yymsp>>2]|0;
   $556 = ((($555)) + -64|0);
   $557 = ((($556)) + 4|0);
   $558 = HEAP32[$557>>2]|0;
   $559 = HEAP32[$yymsp>>2]|0;
   $560 = ((($559)) + -48|0);
   $561 = ((($560)) + 4|0);
   $562 = HEAP32[$561>>2]|0;
   $563 = HEAP32[$yymsp>>2]|0;
   $564 = ((($563)) + -32|0);
   $565 = ((($564)) + 4|0);
   $566 = HEAP32[$565>>2]|0;
   $567 = HEAP32[$yymsp>>2]|0;
   $568 = ((($567)) + -16|0);
   $569 = ((($568)) + 4|0);
   $570 = HEAP32[$569>>2]|0;
   $571 = HEAP32[$yymsp>>2]|0;
   $572 = ((($571)) + -112|0);
   $573 = ((($572)) + 4|0);
   $574 = HEAP32[$573>>2]|0;
   $575 = HEAP32[$yymsp>>2]|0;
   $576 = ((($575)) + 4|0);
   $577 = HEAP32[$576>>2]|0;
   $578 = HEAP32[$yymsp>>2]|0;
   $579 = ((($578)) + 4|0);
   $580 = ((($579)) + 4|0);
   $581 = HEAP32[$580>>2]|0;
   $582 = (_sqlite3SelectNew($546,$550,$554,$558,$562,$566,$570,$574,$577,$581)|0);
   $583 = HEAP32[$yymsp>>2]|0;
   $584 = ((($583)) + -128|0);
   $585 = ((($584)) + 4|0);
   HEAP32[$585>>2] = $582;
   break;
  }
  case 86:  {
   $586 = HEAP32[$pParse>>2]|0;
   $587 = HEAP32[$yymsp>>2]|0;
   $588 = ((($587)) + -16|0);
   $589 = ((($588)) + 4|0);
   $590 = HEAP32[$589>>2]|0;
   $591 = (_sqlite3SelectNew($586,$590,0,0,0,0,0,512,0,0)|0);
   $592 = HEAP32[$yymsp>>2]|0;
   $593 = ((($592)) + -48|0);
   $594 = ((($593)) + 4|0);
   HEAP32[$594>>2] = $591;
   break;
  }
  case 87:  {
   $595 = HEAP32[$yymsp>>2]|0;
   $596 = ((($595)) + -64|0);
   $597 = ((($596)) + 4|0);
   $598 = HEAP32[$597>>2]|0;
   HEAP32[$pLeft>>2] = $598;
   $599 = HEAP32[$pParse>>2]|0;
   $600 = HEAP32[$yymsp>>2]|0;
   $601 = ((($600)) + -16|0);
   $602 = ((($601)) + 4|0);
   $603 = HEAP32[$602>>2]|0;
   $604 = (_sqlite3SelectNew($599,$603,0,0,0,0,0,1536,0,0)|0);
   HEAP32[$pRight>>2] = $604;
   $605 = HEAP32[$pLeft>>2]|0;
   $606 = ($605|0)!=(0|0);
   if ($606) {
    $607 = HEAP32[$pLeft>>2]|0;
    $608 = ((($607)) + 8|0);
    $609 = HEAP32[$608>>2]|0;
    $610 = $609 & -1025;
    HEAP32[$608>>2] = $610;
   }
   $611 = HEAP32[$pRight>>2]|0;
   $612 = ($611|0)!=(0|0);
   if ($612) {
    $613 = HEAP32[$pRight>>2]|0;
    $614 = ((($613)) + 4|0);
    HEAP8[$614>>0] = 116;
    $615 = HEAP32[$pLeft>>2]|0;
    $616 = HEAP32[$pRight>>2]|0;
    $617 = ((($616)) + 48|0);
    HEAP32[$617>>2] = $615;
    $618 = HEAP32[$pRight>>2]|0;
    $619 = HEAP32[$yymsp>>2]|0;
    $620 = ((($619)) + -64|0);
    $621 = ((($620)) + 4|0);
    HEAP32[$621>>2] = $618;
    break L6;
   } else {
    $622 = HEAP32[$pLeft>>2]|0;
    $623 = HEAP32[$yymsp>>2]|0;
    $624 = ((($623)) + -64|0);
    $625 = ((($624)) + 4|0);
    HEAP32[$625>>2] = $622;
    break L6;
   }
   break;
  }
  case 88:  {
   $626 = HEAP32[$yymsp>>2]|0;
   $627 = ((($626)) + 4|0);
   HEAP32[$627>>2] = 1;
   break;
  }
  case 89:  {
   $628 = HEAP32[$yymsp>>2]|0;
   $629 = ((($628)) + 4|0);
   HEAP32[$629>>2] = 2;
   break;
  }
  case 211: case 206: case 203: case 126: case 119: case 91:  {
   $630 = HEAP32[$yymsp>>2]|0;
   $631 = ((($630)) + 16|0);
   $632 = ((($631)) + 4|0);
   HEAP32[$632>>2] = 0;
   break;
  }
  case 92:  {
   $633 = HEAP32[$pParse>>2]|0;
   $634 = HEAP32[$yymsp>>2]|0;
   $635 = ((($634)) + -32|0);
   $636 = ((($635)) + 4|0);
   $637 = HEAP32[$636>>2]|0;
   $638 = HEAP32[$yymsp>>2]|0;
   $639 = ((($638)) + -16|0);
   $640 = ((($639)) + 4|0);
   $641 = HEAP32[$640>>2]|0;
   $642 = (_sqlite3ExprListAppend($633,$637,$641)|0);
   $643 = HEAP32[$yymsp>>2]|0;
   $644 = ((($643)) + -32|0);
   $645 = ((($644)) + 4|0);
   HEAP32[$645>>2] = $642;
   $646 = HEAP32[$yymsp>>2]|0;
   $647 = ((($646)) + 4|0);
   $648 = ((($647)) + 4|0);
   $649 = HEAP32[$648>>2]|0;
   $650 = ($649>>>0)>(0);
   if ($650) {
    $651 = HEAP32[$pParse>>2]|0;
    $652 = HEAP32[$yymsp>>2]|0;
    $653 = ((($652)) + -32|0);
    $654 = ((($653)) + 4|0);
    $655 = HEAP32[$654>>2]|0;
    $656 = HEAP32[$yymsp>>2]|0;
    $657 = ((($656)) + 4|0);
    _sqlite3ExprListSetName($651,$655,$657,1);
   }
   $658 = HEAP32[$pParse>>2]|0;
   $659 = HEAP32[$yymsp>>2]|0;
   $660 = ((($659)) + -32|0);
   $661 = ((($660)) + 4|0);
   $662 = HEAP32[$661>>2]|0;
   $663 = HEAP32[$yymsp>>2]|0;
   $664 = ((($663)) + -16|0);
   $665 = ((($664)) + 4|0);
   _sqlite3ExprListSetSpan($658,$662,$665);
   break;
  }
  case 93:  {
   $666 = HEAP32[$pParse>>2]|0;
   $667 = HEAP32[$666>>2]|0;
   $668 = (_sqlite3Expr($667,160,0)|0);
   HEAP32[$p2>>2] = $668;
   $669 = HEAP32[$pParse>>2]|0;
   $670 = HEAP32[$yymsp>>2]|0;
   $671 = ((($670)) + -16|0);
   $672 = ((($671)) + 4|0);
   $673 = HEAP32[$672>>2]|0;
   $674 = HEAP32[$p2>>2]|0;
   $675 = (_sqlite3ExprListAppend($669,$673,$674)|0);
   $676 = HEAP32[$yymsp>>2]|0;
   $677 = ((($676)) + -16|0);
   $678 = ((($677)) + 4|0);
   HEAP32[$678>>2] = $675;
   break;
  }
  case 94:  {
   $679 = HEAP32[$pParse>>2]|0;
   $680 = (_sqlite3PExpr($679,160,0,0,0)|0);
   HEAP32[$pRight3>>2] = $680;
   $681 = HEAP32[$pParse>>2]|0;
   $682 = HEAP32[$yymsp>>2]|0;
   $683 = ((($682)) + -32|0);
   $684 = ((($683)) + 4|0);
   $685 = (_sqlite3PExpr($681,55,0,0,$684)|0);
   HEAP32[$pLeft4>>2] = $685;
   $686 = HEAP32[$pParse>>2]|0;
   $687 = HEAP32[$pLeft4>>2]|0;
   $688 = HEAP32[$pRight3>>2]|0;
   $689 = (_sqlite3PExpr($686,122,$687,$688,0)|0);
   HEAP32[$pDot>>2] = $689;
   $690 = HEAP32[$pParse>>2]|0;
   $691 = HEAP32[$yymsp>>2]|0;
   $692 = ((($691)) + -48|0);
   $693 = ((($692)) + 4|0);
   $694 = HEAP32[$693>>2]|0;
   $695 = HEAP32[$pDot>>2]|0;
   $696 = (_sqlite3ExprListAppend($690,$694,$695)|0);
   $697 = HEAP32[$yymsp>>2]|0;
   $698 = ((($697)) + -48|0);
   $699 = ((($698)) + 4|0);
   HEAP32[$699>>2] = $696;
   break;
  }
  case 226: case 225: case 106: case 95:  {
   $700 = HEAP32[$yymsp>>2]|0;
   $701 = ((($700)) + -16|0);
   $702 = ((($701)) + 4|0);
   $703 = HEAP32[$yymsp>>2]|0;
   $704 = ((($703)) + 4|0);
   ;HEAP32[$702>>2]=HEAP32[$704>>2]|0;HEAP32[$702+4>>2]=HEAP32[$704+4>>2]|0;
   break;
  }
  case 97:  {
   $705 = HEAP32[$pParse>>2]|0;
   $706 = HEAP32[$705>>2]|0;
   $707 = (_sqlite3DbMallocZero($706,80,0)|0);
   $708 = HEAP32[$yymsp>>2]|0;
   $709 = ((($708)) + 16|0);
   $710 = ((($709)) + 4|0);
   HEAP32[$710>>2] = $707;
   break;
  }
  case 98:  {
   $711 = HEAP32[$yymsp>>2]|0;
   $712 = ((($711)) + 4|0);
   $713 = HEAP32[$712>>2]|0;
   $714 = HEAP32[$yymsp>>2]|0;
   $715 = ((($714)) + -16|0);
   $716 = ((($715)) + 4|0);
   HEAP32[$716>>2] = $713;
   $717 = HEAP32[$yymsp>>2]|0;
   $718 = ((($717)) + -16|0);
   $719 = ((($718)) + 4|0);
   $720 = HEAP32[$719>>2]|0;
   _sqlite3SrcListShiftJoinType($720);
   break;
  }
  case 99:  {
   $721 = HEAP32[$yymsp>>2]|0;
   $722 = ((($721)) + -16|0);
   $723 = ((($722)) + 4|0);
   $724 = HEAP32[$723>>2]|0;
   $725 = ($724|0)!=(0|0);
   if ($725) {
    $726 = HEAP32[$yymsp>>2]|0;
    $727 = ((($726)) + -16|0);
    $728 = ((($727)) + 4|0);
    $729 = HEAP32[$728>>2]|0;
    $730 = HEAP32[$729>>2]|0;
    $731 = ($730|0)>(0);
    if ($731) {
     $732 = HEAP32[$yymsp>>2]|0;
     $733 = ((($732)) + 4|0);
     $734 = HEAP32[$733>>2]|0;
     $735 = $734&255;
     $736 = HEAP32[$yymsp>>2]|0;
     $737 = ((($736)) + -16|0);
     $738 = ((($737)) + 4|0);
     $739 = HEAP32[$738>>2]|0;
     $740 = HEAP32[$739>>2]|0;
     $741 = (($740) - 1)|0;
     $742 = HEAP32[$yymsp>>2]|0;
     $743 = ((($742)) + -16|0);
     $744 = ((($743)) + 4|0);
     $745 = HEAP32[$744>>2]|0;
     $746 = ((($745)) + 8|0);
     $747 = (($746) + (($741*72)|0)|0);
     $748 = ((($747)) + 36|0);
     HEAP8[$748>>0] = $735;
    }
   }
   break;
  }
  case 100:  {
   $749 = HEAP32[$yymsp>>2]|0;
   $750 = ((($749)) + 16|0);
   $751 = ((($750)) + 4|0);
   HEAP32[$751>>2] = 0;
   break;
  }
  case 101:  {
   $752 = HEAP32[$pParse>>2]|0;
   $753 = HEAP32[$yymsp>>2]|0;
   $754 = ((($753)) + -96|0);
   $755 = ((($754)) + 4|0);
   $756 = HEAP32[$755>>2]|0;
   $757 = HEAP32[$yymsp>>2]|0;
   $758 = ((($757)) + -80|0);
   $759 = ((($758)) + 4|0);
   $760 = HEAP32[$yymsp>>2]|0;
   $761 = ((($760)) + -64|0);
   $762 = ((($761)) + 4|0);
   $763 = HEAP32[$yymsp>>2]|0;
   $764 = ((($763)) + -48|0);
   $765 = ((($764)) + 4|0);
   $766 = HEAP32[$yymsp>>2]|0;
   $767 = ((($766)) + -16|0);
   $768 = ((($767)) + 4|0);
   $769 = HEAP32[$768>>2]|0;
   $770 = HEAP32[$yymsp>>2]|0;
   $771 = ((($770)) + 4|0);
   $772 = HEAP32[$771>>2]|0;
   $773 = (_sqlite3SrcListAppendFromTerm($752,$756,$759,$762,$765,0,$769,$772)|0);
   $774 = HEAP32[$yymsp>>2]|0;
   $775 = ((($774)) + -96|0);
   $776 = ((($775)) + 4|0);
   HEAP32[$776>>2] = $773;
   $777 = HEAP32[$pParse>>2]|0;
   $778 = HEAP32[$yymsp>>2]|0;
   $779 = ((($778)) + -96|0);
   $780 = ((($779)) + 4|0);
   $781 = HEAP32[$780>>2]|0;
   $782 = HEAP32[$yymsp>>2]|0;
   $783 = ((($782)) + -32|0);
   $784 = ((($783)) + 4|0);
   _sqlite3SrcListIndexedBy($777,$781,$784);
   break;
  }
  case 102:  {
   $785 = HEAP32[$pParse>>2]|0;
   $786 = HEAP32[$yymsp>>2]|0;
   $787 = ((($786)) + -128|0);
   $788 = ((($787)) + 4|0);
   $789 = HEAP32[$788>>2]|0;
   $790 = HEAP32[$yymsp>>2]|0;
   $791 = ((($790)) + -112|0);
   $792 = ((($791)) + 4|0);
   $793 = HEAP32[$yymsp>>2]|0;
   $794 = ((($793)) + -96|0);
   $795 = ((($794)) + 4|0);
   $796 = HEAP32[$yymsp>>2]|0;
   $797 = ((($796)) + -32|0);
   $798 = ((($797)) + 4|0);
   $799 = HEAP32[$yymsp>>2]|0;
   $800 = ((($799)) + -16|0);
   $801 = ((($800)) + 4|0);
   $802 = HEAP32[$801>>2]|0;
   $803 = HEAP32[$yymsp>>2]|0;
   $804 = ((($803)) + 4|0);
   $805 = HEAP32[$804>>2]|0;
   $806 = (_sqlite3SrcListAppendFromTerm($785,$789,$792,$795,$798,0,$802,$805)|0);
   $807 = HEAP32[$yymsp>>2]|0;
   $808 = ((($807)) + -128|0);
   $809 = ((($808)) + 4|0);
   HEAP32[$809>>2] = $806;
   $810 = HEAP32[$pParse>>2]|0;
   $811 = HEAP32[$yymsp>>2]|0;
   $812 = ((($811)) + -128|0);
   $813 = ((($812)) + 4|0);
   $814 = HEAP32[$813>>2]|0;
   $815 = HEAP32[$yymsp>>2]|0;
   $816 = ((($815)) + -64|0);
   $817 = ((($816)) + 4|0);
   $818 = HEAP32[$817>>2]|0;
   _sqlite3SrcListFuncArgs($810,$814,$818);
   break;
  }
  case 103:  {
   $819 = HEAP32[$pParse>>2]|0;
   $820 = HEAP32[$yymsp>>2]|0;
   $821 = ((($820)) + -96|0);
   $822 = ((($821)) + 4|0);
   $823 = HEAP32[$822>>2]|0;
   $824 = HEAP32[$yymsp>>2]|0;
   $825 = ((($824)) + -32|0);
   $826 = ((($825)) + 4|0);
   $827 = HEAP32[$yymsp>>2]|0;
   $828 = ((($827)) + -64|0);
   $829 = ((($828)) + 4|0);
   $830 = HEAP32[$829>>2]|0;
   $831 = HEAP32[$yymsp>>2]|0;
   $832 = ((($831)) + -16|0);
   $833 = ((($832)) + 4|0);
   $834 = HEAP32[$833>>2]|0;
   $835 = HEAP32[$yymsp>>2]|0;
   $836 = ((($835)) + 4|0);
   $837 = HEAP32[$836>>2]|0;
   $838 = (_sqlite3SrcListAppendFromTerm($819,$823,0,0,$826,$830,$834,$837)|0);
   $839 = HEAP32[$yymsp>>2]|0;
   $840 = ((($839)) + -96|0);
   $841 = ((($840)) + 4|0);
   HEAP32[$841>>2] = $838;
   break;
  }
  case 104:  {
   $842 = HEAP32[$yymsp>>2]|0;
   $843 = ((($842)) + -96|0);
   $844 = ((($843)) + 4|0);
   $845 = HEAP32[$844>>2]|0;
   $846 = ($845|0)==(0|0);
   if ($846) {
    $847 = HEAP32[$yymsp>>2]|0;
    $848 = ((($847)) + -32|0);
    $849 = ((($848)) + 4|0);
    $850 = ((($849)) + 4|0);
    $851 = HEAP32[$850>>2]|0;
    $852 = ($851|0)==(0);
    if ($852) {
     $853 = HEAP32[$yymsp>>2]|0;
     $854 = ((($853)) + -16|0);
     $855 = ((($854)) + 4|0);
     $856 = HEAP32[$855>>2]|0;
     $857 = ($856|0)==(0|0);
     if ($857) {
      $858 = HEAP32[$yymsp>>2]|0;
      $859 = ((($858)) + 4|0);
      $860 = HEAP32[$859>>2]|0;
      $861 = ($860|0)==(0|0);
      if ($861) {
       $862 = HEAP32[$yymsp>>2]|0;
       $863 = ((($862)) + -64|0);
       $864 = ((($863)) + 4|0);
       $865 = HEAP32[$864>>2]|0;
       $866 = HEAP32[$yymsp>>2]|0;
       $867 = ((($866)) + -96|0);
       $868 = ((($867)) + 4|0);
       HEAP32[$868>>2] = $865;
       break L6;
      }
     }
    }
   }
   $869 = HEAP32[$yymsp>>2]|0;
   $870 = ((($869)) + -64|0);
   $871 = ((($870)) + 4|0);
   $872 = HEAP32[$871>>2]|0;
   $873 = HEAP32[$872>>2]|0;
   $874 = ($873|0)==(1);
   if (!($874)) {
    $943 = HEAP32[$yymsp>>2]|0;
    $944 = ((($943)) + -64|0);
    $945 = ((($944)) + 4|0);
    $946 = HEAP32[$945>>2]|0;
    _sqlite3SrcListShiftJoinType($946);
    $947 = HEAP32[$pParse>>2]|0;
    $948 = HEAP32[$yymsp>>2]|0;
    $949 = ((($948)) + -64|0);
    $950 = ((($949)) + 4|0);
    $951 = HEAP32[$950>>2]|0;
    $952 = (_sqlite3SelectNew($947,0,$951,0,0,0,0,2048,0,0)|0);
    HEAP32[$pSubquery>>2] = $952;
    $953 = HEAP32[$pParse>>2]|0;
    $954 = HEAP32[$yymsp>>2]|0;
    $955 = ((($954)) + -96|0);
    $956 = ((($955)) + 4|0);
    $957 = HEAP32[$956>>2]|0;
    $958 = HEAP32[$yymsp>>2]|0;
    $959 = ((($958)) + -32|0);
    $960 = ((($959)) + 4|0);
    $961 = HEAP32[$pSubquery>>2]|0;
    $962 = HEAP32[$yymsp>>2]|0;
    $963 = ((($962)) + -16|0);
    $964 = ((($963)) + 4|0);
    $965 = HEAP32[$964>>2]|0;
    $966 = HEAP32[$yymsp>>2]|0;
    $967 = ((($966)) + 4|0);
    $968 = HEAP32[$967>>2]|0;
    $969 = (_sqlite3SrcListAppendFromTerm($953,$957,0,0,$960,$961,$965,$968)|0);
    $970 = HEAP32[$yymsp>>2]|0;
    $971 = ((($970)) + -96|0);
    $972 = ((($971)) + 4|0);
    HEAP32[$972>>2] = $969;
    break L6;
   }
   $875 = HEAP32[$pParse>>2]|0;
   $876 = HEAP32[$yymsp>>2]|0;
   $877 = ((($876)) + -96|0);
   $878 = ((($877)) + 4|0);
   $879 = HEAP32[$878>>2]|0;
   $880 = HEAP32[$yymsp>>2]|0;
   $881 = ((($880)) + -32|0);
   $882 = ((($881)) + 4|0);
   $883 = HEAP32[$yymsp>>2]|0;
   $884 = ((($883)) + -16|0);
   $885 = ((($884)) + 4|0);
   $886 = HEAP32[$885>>2]|0;
   $887 = HEAP32[$yymsp>>2]|0;
   $888 = ((($887)) + 4|0);
   $889 = HEAP32[$888>>2]|0;
   $890 = (_sqlite3SrcListAppendFromTerm($875,$879,0,0,$882,0,$886,$889)|0);
   $891 = HEAP32[$yymsp>>2]|0;
   $892 = ((($891)) + -96|0);
   $893 = ((($892)) + 4|0);
   HEAP32[$893>>2] = $890;
   $894 = HEAP32[$yymsp>>2]|0;
   $895 = ((($894)) + -96|0);
   $896 = ((($895)) + 4|0);
   $897 = HEAP32[$896>>2]|0;
   $898 = ($897|0)!=(0|0);
   if ($898) {
    $899 = HEAP32[$yymsp>>2]|0;
    $900 = ((($899)) + -96|0);
    $901 = ((($900)) + 4|0);
    $902 = HEAP32[$901>>2]|0;
    $903 = HEAP32[$902>>2]|0;
    $904 = (($903) - 1)|0;
    $905 = HEAP32[$yymsp>>2]|0;
    $906 = ((($905)) + -96|0);
    $907 = ((($906)) + 4|0);
    $908 = HEAP32[$907>>2]|0;
    $909 = ((($908)) + 8|0);
    $910 = (($909) + (($904*72)|0)|0);
    HEAP32[$pNew>>2] = $910;
    $911 = HEAP32[$yymsp>>2]|0;
    $912 = ((($911)) + -64|0);
    $913 = ((($912)) + 4|0);
    $914 = HEAP32[$913>>2]|0;
    $915 = ((($914)) + 8|0);
    HEAP32[$pOld>>2] = $915;
    $916 = HEAP32[$pOld>>2]|0;
    $917 = ((($916)) + 8|0);
    $918 = HEAP32[$917>>2]|0;
    $919 = HEAP32[$pNew>>2]|0;
    $920 = ((($919)) + 8|0);
    HEAP32[$920>>2] = $918;
    $921 = HEAP32[$pOld>>2]|0;
    $922 = ((($921)) + 4|0);
    $923 = HEAP32[$922>>2]|0;
    $924 = HEAP32[$pNew>>2]|0;
    $925 = ((($924)) + 4|0);
    HEAP32[$925>>2] = $923;
    $926 = HEAP32[$pOld>>2]|0;
    $927 = ((($926)) + 20|0);
    $928 = HEAP32[$927>>2]|0;
    $929 = HEAP32[$pNew>>2]|0;
    $930 = ((($929)) + 20|0);
    HEAP32[$930>>2] = $928;
    $931 = HEAP32[$pOld>>2]|0;
    $932 = ((($931)) + 4|0);
    HEAP32[$932>>2] = 0;
    $933 = HEAP32[$pOld>>2]|0;
    $934 = ((($933)) + 8|0);
    HEAP32[$934>>2] = 0;
    $935 = HEAP32[$pOld>>2]|0;
    $936 = ((($935)) + 20|0);
    HEAP32[$936>>2] = 0;
   }
   $937 = HEAP32[$pParse>>2]|0;
   $938 = HEAP32[$937>>2]|0;
   $939 = HEAP32[$yymsp>>2]|0;
   $940 = ((($939)) + -64|0);
   $941 = ((($940)) + 4|0);
   $942 = HEAP32[$941>>2]|0;
   _sqlite3SrcListDelete($938,$942);
   break;
  }
  case 114: case 105:  {
   $973 = HEAP32[$yymsp>>2]|0;
   $974 = ((($973)) + 16|0);
   $975 = ((($974)) + 4|0);
   HEAP32[$975>>2] = 0;
   $976 = HEAP32[$yymsp>>2]|0;
   $977 = ((($976)) + 16|0);
   $978 = ((($977)) + 4|0);
   $979 = ((($978)) + 4|0);
   HEAP32[$979>>2] = 0;
   break;
  }
  case 107:  {
   $980 = HEAP32[$pParse>>2]|0;
   $981 = HEAP32[$980>>2]|0;
   $982 = HEAP32[$yymsp>>2]|0;
   $983 = ((($982)) + -16|0);
   $984 = ((($983)) + 4|0);
   $985 = HEAP32[$yymsp>>2]|0;
   $986 = ((($985)) + 4|0);
   $987 = (_sqlite3SrcListAppend($981,0,$984,$986)|0);
   $988 = HEAP32[$yymsp>>2]|0;
   $989 = ((($988)) + -16|0);
   $990 = ((($989)) + 4|0);
   HEAP32[$990>>2] = $987;
   break;
  }
  case 108:  {
   $991 = HEAP32[$yymsp>>2]|0;
   $992 = ((($991)) + 4|0);
   HEAP32[$992>>2] = 1;
   break;
  }
  case 109:  {
   $993 = HEAP32[$pParse>>2]|0;
   $994 = HEAP32[$yymsp>>2]|0;
   $995 = ((($994)) + -16|0);
   $996 = ((($995)) + 4|0);
   $997 = (_sqlite3JoinType($993,$996,0,0)|0);
   $998 = HEAP32[$yymsp>>2]|0;
   $999 = ((($998)) + -16|0);
   $1000 = ((($999)) + 4|0);
   HEAP32[$1000>>2] = $997;
   break;
  }
  case 110:  {
   $1001 = HEAP32[$pParse>>2]|0;
   $1002 = HEAP32[$yymsp>>2]|0;
   $1003 = ((($1002)) + -32|0);
   $1004 = ((($1003)) + 4|0);
   $1005 = HEAP32[$yymsp>>2]|0;
   $1006 = ((($1005)) + -16|0);
   $1007 = ((($1006)) + 4|0);
   $1008 = (_sqlite3JoinType($1001,$1004,$1007,0)|0);
   $1009 = HEAP32[$yymsp>>2]|0;
   $1010 = ((($1009)) + -32|0);
   $1011 = ((($1010)) + 4|0);
   HEAP32[$1011>>2] = $1008;
   break;
  }
  case 111:  {
   $1012 = HEAP32[$pParse>>2]|0;
   $1013 = HEAP32[$yymsp>>2]|0;
   $1014 = ((($1013)) + -48|0);
   $1015 = ((($1014)) + 4|0);
   $1016 = HEAP32[$yymsp>>2]|0;
   $1017 = ((($1016)) + -32|0);
   $1018 = ((($1017)) + 4|0);
   $1019 = HEAP32[$yymsp>>2]|0;
   $1020 = ((($1019)) + -16|0);
   $1021 = ((($1020)) + 4|0);
   $1022 = (_sqlite3JoinType($1012,$1015,$1018,$1021)|0);
   $1023 = HEAP32[$yymsp>>2]|0;
   $1024 = ((($1023)) + -48|0);
   $1025 = ((($1024)) + 4|0);
   HEAP32[$1025>>2] = $1022;
   break;
  }
  case 199: case 136: case 129: case 112:  {
   $1026 = HEAP32[$yymsp>>2]|0;
   $1027 = ((($1026)) + 4|0);
   $1028 = HEAP32[$1027>>2]|0;
   $1029 = HEAP32[$yymsp>>2]|0;
   $1030 = ((($1029)) + -16|0);
   $1031 = ((($1030)) + 4|0);
   HEAP32[$1031>>2] = $1028;
   break;
  }
  case 202: case 200: case 135: case 128: case 113:  {
   $1032 = HEAP32[$yymsp>>2]|0;
   $1033 = ((($1032)) + 16|0);
   $1034 = ((($1033)) + 4|0);
   HEAP32[$1034>>2] = 0;
   break;
  }
  case 115:  {
   $1035 = HEAP32[$yymsp>>2]|0;
   $1036 = ((($1035)) + -32|0);
   $1037 = ((($1036)) + 4|0);
   $1038 = HEAP32[$yymsp>>2]|0;
   $1039 = ((($1038)) + 4|0);
   ;HEAP32[$1037>>2]=HEAP32[$1039>>2]|0;HEAP32[$1037+4>>2]=HEAP32[$1039+4>>2]|0;
   break;
  }
  case 116:  {
   $1040 = HEAP32[$yymsp>>2]|0;
   $1041 = ((($1040)) + -16|0);
   $1042 = ((($1041)) + 4|0);
   HEAP32[$1042>>2] = 0;
   $1043 = HEAP32[$yymsp>>2]|0;
   $1044 = ((($1043)) + -16|0);
   $1045 = ((($1044)) + 4|0);
   $1046 = ((($1045)) + 4|0);
   HEAP32[$1046>>2] = 1;
   break;
  }
  case 117:  {
   $1047 = HEAP32[$yymsp>>2]|0;
   $1048 = ((($1047)) + -16|0);
   $1049 = ((($1048)) + 4|0);
   $1050 = HEAP32[$1049>>2]|0;
   $1051 = HEAP32[$yymsp>>2]|0;
   $1052 = ((($1051)) + -48|0);
   $1053 = ((($1052)) + 4|0);
   HEAP32[$1053>>2] = $1050;
   break;
  }
  case 146: case 118:  {
   $1054 = HEAP32[$yymsp>>2]|0;
   $1055 = ((($1054)) + 16|0);
   $1056 = ((($1055)) + 4|0);
   HEAP32[$1056>>2] = 0;
   break;
  }
  case 127: case 120:  {
   $1057 = HEAP32[$yymsp>>2]|0;
   $1058 = ((($1057)) + 4|0);
   $1059 = HEAP32[$1058>>2]|0;
   $1060 = HEAP32[$yymsp>>2]|0;
   $1061 = ((($1060)) + -32|0);
   $1062 = ((($1061)) + 4|0);
   HEAP32[$1062>>2] = $1059;
   break;
  }
  case 121:  {
   $1063 = HEAP32[$pParse>>2]|0;
   $1064 = HEAP32[$yymsp>>2]|0;
   $1065 = ((($1064)) + -48|0);
   $1066 = ((($1065)) + 4|0);
   $1067 = HEAP32[$1066>>2]|0;
   $1068 = HEAP32[$yymsp>>2]|0;
   $1069 = ((($1068)) + -16|0);
   $1070 = ((($1069)) + 4|0);
   $1071 = HEAP32[$1070>>2]|0;
   $1072 = (_sqlite3ExprListAppend($1063,$1067,$1071)|0);
   $1073 = HEAP32[$yymsp>>2]|0;
   $1074 = ((($1073)) + -48|0);
   $1075 = ((($1074)) + 4|0);
   HEAP32[$1075>>2] = $1072;
   $1076 = HEAP32[$yymsp>>2]|0;
   $1077 = ((($1076)) + -48|0);
   $1078 = ((($1077)) + 4|0);
   $1079 = HEAP32[$1078>>2]|0;
   $1080 = HEAP32[$yymsp>>2]|0;
   $1081 = ((($1080)) + 4|0);
   $1082 = HEAP32[$1081>>2]|0;
   _sqlite3ExprListSetSortOrder($1079,$1082);
   break;
  }
  case 122:  {
   $1083 = HEAP32[$pParse>>2]|0;
   $1084 = HEAP32[$yymsp>>2]|0;
   $1085 = ((($1084)) + -16|0);
   $1086 = ((($1085)) + 4|0);
   $1087 = HEAP32[$1086>>2]|0;
   $1088 = (_sqlite3ExprListAppend($1083,0,$1087)|0);
   $1089 = HEAP32[$yymsp>>2]|0;
   $1090 = ((($1089)) + -16|0);
   $1091 = ((($1090)) + 4|0);
   HEAP32[$1091>>2] = $1088;
   $1092 = HEAP32[$yymsp>>2]|0;
   $1093 = ((($1092)) + -16|0);
   $1094 = ((($1093)) + 4|0);
   $1095 = HEAP32[$1094>>2]|0;
   $1096 = HEAP32[$yymsp>>2]|0;
   $1097 = ((($1096)) + 4|0);
   $1098 = HEAP32[$1097>>2]|0;
   _sqlite3ExprListSetSortOrder($1095,$1098);
   break;
  }
  case 123:  {
   $1099 = HEAP32[$yymsp>>2]|0;
   $1100 = ((($1099)) + 4|0);
   HEAP32[$1100>>2] = 0;
   break;
  }
  case 124:  {
   $1101 = HEAP32[$yymsp>>2]|0;
   $1102 = ((($1101)) + 4|0);
   HEAP32[$1102>>2] = 1;
   break;
  }
  case 125:  {
   $1103 = HEAP32[$yymsp>>2]|0;
   $1104 = ((($1103)) + 16|0);
   $1105 = ((($1104)) + 4|0);
   HEAP32[$1105>>2] = -1;
   break;
  }
  case 130:  {
   $1106 = HEAP32[$yymsp>>2]|0;
   $1107 = ((($1106)) + 16|0);
   $1108 = ((($1107)) + 4|0);
   HEAP32[$1108>>2] = 0;
   $1109 = HEAP32[$yymsp>>2]|0;
   $1110 = ((($1109)) + 16|0);
   $1111 = ((($1110)) + 4|0);
   $1112 = ((($1111)) + 4|0);
   HEAP32[$1112>>2] = 0;
   break;
  }
  case 131:  {
   $1113 = HEAP32[$yymsp>>2]|0;
   $1114 = ((($1113)) + 4|0);
   $1115 = HEAP32[$1114>>2]|0;
   $1116 = HEAP32[$yymsp>>2]|0;
   $1117 = ((($1116)) + -16|0);
   $1118 = ((($1117)) + 4|0);
   HEAP32[$1118>>2] = $1115;
   $1119 = HEAP32[$yymsp>>2]|0;
   $1120 = ((($1119)) + -16|0);
   $1121 = ((($1120)) + 4|0);
   $1122 = ((($1121)) + 4|0);
   HEAP32[$1122>>2] = 0;
   break;
  }
  case 132:  {
   $1123 = HEAP32[$yymsp>>2]|0;
   $1124 = ((($1123)) + -32|0);
   $1125 = ((($1124)) + 4|0);
   $1126 = HEAP32[$1125>>2]|0;
   $1127 = HEAP32[$yymsp>>2]|0;
   $1128 = ((($1127)) + -48|0);
   $1129 = ((($1128)) + 4|0);
   HEAP32[$1129>>2] = $1126;
   $1130 = HEAP32[$yymsp>>2]|0;
   $1131 = ((($1130)) + 4|0);
   $1132 = HEAP32[$1131>>2]|0;
   $1133 = HEAP32[$yymsp>>2]|0;
   $1134 = ((($1133)) + -48|0);
   $1135 = ((($1134)) + 4|0);
   $1136 = ((($1135)) + 4|0);
   HEAP32[$1136>>2] = $1132;
   break;
  }
  case 133:  {
   $1137 = HEAP32[$yymsp>>2]|0;
   $1138 = ((($1137)) + -32|0);
   $1139 = ((($1138)) + 4|0);
   $1140 = HEAP32[$1139>>2]|0;
   $1141 = HEAP32[$yymsp>>2]|0;
   $1142 = ((($1141)) + -48|0);
   $1143 = ((($1142)) + 4|0);
   $1144 = ((($1143)) + 4|0);
   HEAP32[$1144>>2] = $1140;
   $1145 = HEAP32[$yymsp>>2]|0;
   $1146 = ((($1145)) + 4|0);
   $1147 = HEAP32[$1146>>2]|0;
   $1148 = HEAP32[$yymsp>>2]|0;
   $1149 = ((($1148)) + -48|0);
   $1150 = ((($1149)) + 4|0);
   HEAP32[$1150>>2] = $1147;
   break;
  }
  case 134:  {
   $1151 = HEAP32[$pParse>>2]|0;
   $1152 = HEAP32[$yymsp>>2]|0;
   $1153 = ((($1152)) + -80|0);
   $1154 = ((($1153)) + 4|0);
   $1155 = HEAP32[$1154>>2]|0;
   _sqlite3WithPush($1151,$1155,1);
   $1156 = HEAP32[$pParse>>2]|0;
   $1157 = HEAP32[$yymsp>>2]|0;
   $1158 = ((($1157)) + -32|0);
   $1159 = ((($1158)) + 4|0);
   $1160 = HEAP32[$1159>>2]|0;
   $1161 = HEAP32[$yymsp>>2]|0;
   $1162 = ((($1161)) + -16|0);
   $1163 = ((($1162)) + 4|0);
   _sqlite3SrcListIndexedBy($1156,$1160,$1163);
   $1164 = HEAP32[$pParse>>2]|0;
   $1165 = HEAP32[$yymsp>>2]|0;
   $1166 = ((($1165)) + -32|0);
   $1167 = ((($1166)) + 4|0);
   $1168 = HEAP32[$1167>>2]|0;
   $1169 = HEAP32[$yymsp>>2]|0;
   $1170 = ((($1169)) + 4|0);
   $1171 = HEAP32[$1170>>2]|0;
   _sqlite3DeleteFrom($1164,$1168,$1171);
   break;
  }
  case 137:  {
   $1172 = HEAP32[$pParse>>2]|0;
   $1173 = HEAP32[$yymsp>>2]|0;
   $1174 = ((($1173)) + -112|0);
   $1175 = ((($1174)) + 4|0);
   $1176 = HEAP32[$1175>>2]|0;
   _sqlite3WithPush($1172,$1176,1);
   $1177 = HEAP32[$pParse>>2]|0;
   $1178 = HEAP32[$yymsp>>2]|0;
   $1179 = ((($1178)) + -64|0);
   $1180 = ((($1179)) + 4|0);
   $1181 = HEAP32[$1180>>2]|0;
   $1182 = HEAP32[$yymsp>>2]|0;
   $1183 = ((($1182)) + -48|0);
   $1184 = ((($1183)) + 4|0);
   _sqlite3SrcListIndexedBy($1177,$1181,$1184);
   $1185 = HEAP32[$pParse>>2]|0;
   $1186 = HEAP32[$yymsp>>2]|0;
   $1187 = ((($1186)) + -16|0);
   $1188 = ((($1187)) + 4|0);
   $1189 = HEAP32[$1188>>2]|0;
   _sqlite3ExprListCheckLength($1185,$1189,27177);
   $1190 = HEAP32[$pParse>>2]|0;
   $1191 = HEAP32[$yymsp>>2]|0;
   $1192 = ((($1191)) + -64|0);
   $1193 = ((($1192)) + 4|0);
   $1194 = HEAP32[$1193>>2]|0;
   $1195 = HEAP32[$yymsp>>2]|0;
   $1196 = ((($1195)) + -16|0);
   $1197 = ((($1196)) + 4|0);
   $1198 = HEAP32[$1197>>2]|0;
   $1199 = HEAP32[$yymsp>>2]|0;
   $1200 = ((($1199)) + 4|0);
   $1201 = HEAP32[$1200>>2]|0;
   $1202 = HEAP32[$yymsp>>2]|0;
   $1203 = ((($1202)) + -80|0);
   $1204 = ((($1203)) + 4|0);
   $1205 = HEAP32[$1204>>2]|0;
   _sqlite3Update($1190,$1194,$1198,$1201,$1205);
   break;
  }
  case 138:  {
   $1206 = HEAP32[$pParse>>2]|0;
   $1207 = HEAP32[$yymsp>>2]|0;
   $1208 = ((($1207)) + -64|0);
   $1209 = ((($1208)) + 4|0);
   $1210 = HEAP32[$1209>>2]|0;
   $1211 = HEAP32[$yymsp>>2]|0;
   $1212 = ((($1211)) + 4|0);
   $1213 = HEAP32[$1212>>2]|0;
   $1214 = (_sqlite3ExprListAppend($1206,$1210,$1213)|0);
   $1215 = HEAP32[$yymsp>>2]|0;
   $1216 = ((($1215)) + -64|0);
   $1217 = ((($1216)) + 4|0);
   HEAP32[$1217>>2] = $1214;
   $1218 = HEAP32[$pParse>>2]|0;
   $1219 = HEAP32[$yymsp>>2]|0;
   $1220 = ((($1219)) + -64|0);
   $1221 = ((($1220)) + 4|0);
   $1222 = HEAP32[$1221>>2]|0;
   $1223 = HEAP32[$yymsp>>2]|0;
   $1224 = ((($1223)) + -32|0);
   $1225 = ((($1224)) + 4|0);
   _sqlite3ExprListSetName($1218,$1222,$1225,1);
   break;
  }
  case 139:  {
   $1226 = HEAP32[$pParse>>2]|0;
   $1227 = HEAP32[$yymsp>>2]|0;
   $1228 = ((($1227)) + -96|0);
   $1229 = ((($1228)) + 4|0);
   $1230 = HEAP32[$1229>>2]|0;
   $1231 = HEAP32[$yymsp>>2]|0;
   $1232 = ((($1231)) + -48|0);
   $1233 = ((($1232)) + 4|0);
   $1234 = HEAP32[$1233>>2]|0;
   $1235 = HEAP32[$yymsp>>2]|0;
   $1236 = ((($1235)) + 4|0);
   $1237 = HEAP32[$1236>>2]|0;
   $1238 = (_sqlite3ExprListAppendVector($1226,$1230,$1234,$1237)|0);
   $1239 = HEAP32[$yymsp>>2]|0;
   $1240 = ((($1239)) + -96|0);
   $1241 = ((($1240)) + 4|0);
   HEAP32[$1241>>2] = $1238;
   break;
  }
  case 140:  {
   $1242 = HEAP32[$pParse>>2]|0;
   $1243 = HEAP32[$yymsp>>2]|0;
   $1244 = ((($1243)) + 4|0);
   $1245 = HEAP32[$1244>>2]|0;
   $1246 = (_sqlite3ExprListAppend($1242,0,$1245)|0);
   HEAP32[$yylhsminor>>2] = $1246;
   $1247 = HEAP32[$pParse>>2]|0;
   $1248 = HEAP32[$yylhsminor>>2]|0;
   $1249 = HEAP32[$yymsp>>2]|0;
   $1250 = ((($1249)) + -32|0);
   $1251 = ((($1250)) + 4|0);
   _sqlite3ExprListSetName($1247,$1248,$1251,1);
   $1252 = HEAP32[$yylhsminor>>2]|0;
   $1253 = HEAP32[$yymsp>>2]|0;
   $1254 = ((($1253)) + -32|0);
   $1255 = ((($1254)) + 4|0);
   HEAP32[$1255>>2] = $1252;
   break;
  }
  case 141:  {
   $1256 = HEAP32[$pParse>>2]|0;
   $1257 = HEAP32[$yymsp>>2]|0;
   $1258 = ((($1257)) + -48|0);
   $1259 = ((($1258)) + 4|0);
   $1260 = HEAP32[$1259>>2]|0;
   $1261 = HEAP32[$yymsp>>2]|0;
   $1262 = ((($1261)) + 4|0);
   $1263 = HEAP32[$1262>>2]|0;
   $1264 = (_sqlite3ExprListAppendVector($1256,0,$1260,$1263)|0);
   $1265 = HEAP32[$yymsp>>2]|0;
   $1266 = ((($1265)) + -64|0);
   $1267 = ((($1266)) + 4|0);
   HEAP32[$1267>>2] = $1264;
   break;
  }
  case 142:  {
   $1268 = HEAP32[$pParse>>2]|0;
   $1269 = HEAP32[$yymsp>>2]|0;
   $1270 = ((($1269)) + -80|0);
   $1271 = ((($1270)) + 4|0);
   $1272 = HEAP32[$1271>>2]|0;
   _sqlite3WithPush($1268,$1272,1);
   $1273 = HEAP32[$pParse>>2]|0;
   $1274 = HEAP32[$yymsp>>2]|0;
   $1275 = ((($1274)) + -32|0);
   $1276 = ((($1275)) + 4|0);
   $1277 = HEAP32[$1276>>2]|0;
   $1278 = HEAP32[$yymsp>>2]|0;
   $1279 = ((($1278)) + 4|0);
   $1280 = HEAP32[$1279>>2]|0;
   $1281 = HEAP32[$yymsp>>2]|0;
   $1282 = ((($1281)) + -16|0);
   $1283 = ((($1282)) + 4|0);
   $1284 = HEAP32[$1283>>2]|0;
   $1285 = HEAP32[$yymsp>>2]|0;
   $1286 = ((($1285)) + -64|0);
   $1287 = ((($1286)) + 4|0);
   $1288 = HEAP32[$1287>>2]|0;
   _sqlite3Insert($1273,$1277,$1280,$1284,$1288);
   break;
  }
  case 143:  {
   $1289 = HEAP32[$pParse>>2]|0;
   $1290 = HEAP32[$yymsp>>2]|0;
   $1291 = ((($1290)) + -96|0);
   $1292 = ((($1291)) + 4|0);
   $1293 = HEAP32[$1292>>2]|0;
   _sqlite3WithPush($1289,$1293,1);
   $1294 = HEAP32[$pParse>>2]|0;
   $1295 = HEAP32[$yymsp>>2]|0;
   $1296 = ((($1295)) + -48|0);
   $1297 = ((($1296)) + 4|0);
   $1298 = HEAP32[$1297>>2]|0;
   $1299 = HEAP32[$yymsp>>2]|0;
   $1300 = ((($1299)) + -32|0);
   $1301 = ((($1300)) + 4|0);
   $1302 = HEAP32[$1301>>2]|0;
   $1303 = HEAP32[$yymsp>>2]|0;
   $1304 = ((($1303)) + -80|0);
   $1305 = ((($1304)) + 4|0);
   $1306 = HEAP32[$1305>>2]|0;
   _sqlite3Insert($1294,$1298,0,$1302,$1306);
   break;
  }
  case 147:  {
   $1307 = HEAP32[$yymsp>>2]|0;
   $1308 = ((($1307)) + -16|0);
   $1309 = ((($1308)) + 4|0);
   $1310 = HEAP32[$1309>>2]|0;
   $1311 = HEAP32[$yymsp>>2]|0;
   $1312 = ((($1311)) + -32|0);
   $1313 = ((($1312)) + 4|0);
   HEAP32[$1313>>2] = $1310;
   break;
  }
  case 148:  {
   $1314 = HEAP32[$pParse>>2]|0;
   $1315 = HEAP32[$1314>>2]|0;
   $1316 = HEAP32[$yymsp>>2]|0;
   $1317 = ((($1316)) + -32|0);
   $1318 = ((($1317)) + 4|0);
   $1319 = HEAP32[$1318>>2]|0;
   $1320 = HEAP32[$yymsp>>2]|0;
   $1321 = ((($1320)) + 4|0);
   $1322 = (_sqlite3IdListAppend($1315,$1319,$1321)|0);
   $1323 = HEAP32[$yymsp>>2]|0;
   $1324 = ((($1323)) + -32|0);
   $1325 = ((($1324)) + 4|0);
   HEAP32[$1325>>2] = $1322;
   break;
  }
  case 149:  {
   $1326 = HEAP32[$pParse>>2]|0;
   $1327 = HEAP32[$1326>>2]|0;
   $1328 = HEAP32[$yymsp>>2]|0;
   $1329 = ((($1328)) + 4|0);
   $1330 = (_sqlite3IdListAppend($1327,0,$1329)|0);
   $1331 = HEAP32[$yymsp>>2]|0;
   $1332 = ((($1331)) + 4|0);
   HEAP32[$1332>>2] = $1330;
   break;
  }
  case 150:  {
   $1333 = HEAP32[$yymsp>>2]|0;
   $1334 = ((($1333)) + -32|0);
   $1335 = ((($1334)) + 4|0);
   $1336 = HEAP32[$yymsp>>2]|0;
   $1337 = ((($1336)) + -32|0);
   $1338 = ((($1337)) + 4|0);
   $1339 = HEAP32[$yymsp>>2]|0;
   $1340 = ((($1339)) + 4|0);
   _spanSet($1335,$1338,$1340);
   $1341 = HEAP32[$yymsp>>2]|0;
   $1342 = ((($1341)) + -16|0);
   $1343 = ((($1342)) + 4|0);
   $1344 = HEAP32[$1343>>2]|0;
   $1345 = HEAP32[$yymsp>>2]|0;
   $1346 = ((($1345)) + -32|0);
   $1347 = ((($1346)) + 4|0);
   HEAP32[$1347>>2] = $1344;
   break;
  }
  case 157: case 156: case 151:  {
   $1348 = HEAP32[$yymsp>>2]|0;
   $1349 = ((($1348)) + 4|0);
   $1350 = HEAP32[$pParse>>2]|0;
   $1351 = HEAP32[$yymsp>>2]|0;
   $1352 = ((($1351)) + 2|0);
   $1353 = HEAP8[$1352>>0]|0;
   $1354 = $1353&255;
   $1355 = HEAP32[$yymsp>>2]|0;
   $1356 = ((($1355)) + 4|0);
   ;HEAP32[$$byval_copy16>>2]=HEAP32[$1356>>2]|0;HEAP32[$$byval_copy16+4>>2]=HEAP32[$1356+4>>2]|0;
   _spanExpr($1349,$1350,$1354,$$byval_copy16);
   break;
  }
  case 153: case 152:  {
   $1357 = HEAP32[$yymsp>>2]|0;
   $1358 = ((($1357)) + 4|0);
   $1359 = HEAP32[$pParse>>2]|0;
   $1360 = HEAP32[$yymsp>>2]|0;
   $1361 = ((($1360)) + 4|0);
   ;HEAP32[$$byval_copy16>>2]=HEAP32[$1361>>2]|0;HEAP32[$$byval_copy16+4>>2]=HEAP32[$1361+4>>2]|0;
   _spanExpr($1358,$1359,55,$$byval_copy16);
   break;
  }
  case 154:  {
   $1362 = HEAP32[$pParse>>2]|0;
   $1363 = HEAP32[$1362>>2]|0;
   $1364 = HEAP32[$yymsp>>2]|0;
   $1365 = ((($1364)) + -32|0);
   $1366 = ((($1365)) + 4|0);
   $1367 = (_sqlite3ExprAlloc($1363,55,$1366,1)|0);
   HEAP32[$temp1>>2] = $1367;
   $1368 = HEAP32[$pParse>>2]|0;
   $1369 = HEAP32[$1368>>2]|0;
   $1370 = HEAP32[$yymsp>>2]|0;
   $1371 = ((($1370)) + 4|0);
   $1372 = (_sqlite3ExprAlloc($1369,55,$1371,1)|0);
   HEAP32[$temp2>>2] = $1372;
   $1373 = HEAP32[$yymsp>>2]|0;
   $1374 = ((($1373)) + -32|0);
   $1375 = ((($1374)) + 4|0);
   $1376 = HEAP32[$yymsp>>2]|0;
   $1377 = ((($1376)) + -32|0);
   $1378 = ((($1377)) + 4|0);
   $1379 = HEAP32[$yymsp>>2]|0;
   $1380 = ((($1379)) + 4|0);
   _spanSet($1375,$1378,$1380);
   $1381 = HEAP32[$pParse>>2]|0;
   $1382 = HEAP32[$temp1>>2]|0;
   $1383 = HEAP32[$temp2>>2]|0;
   $1384 = (_sqlite3PExpr($1381,122,$1382,$1383,0)|0);
   $1385 = HEAP32[$yymsp>>2]|0;
   $1386 = ((($1385)) + -32|0);
   $1387 = ((($1386)) + 4|0);
   HEAP32[$1387>>2] = $1384;
   break;
  }
  case 155:  {
   $1388 = HEAP32[$pParse>>2]|0;
   $1389 = HEAP32[$1388>>2]|0;
   $1390 = HEAP32[$yymsp>>2]|0;
   $1391 = ((($1390)) + -64|0);
   $1392 = ((($1391)) + 4|0);
   $1393 = (_sqlite3ExprAlloc($1389,55,$1392,1)|0);
   HEAP32[$temp15>>2] = $1393;
   $1394 = HEAP32[$pParse>>2]|0;
   $1395 = HEAP32[$1394>>2]|0;
   $1396 = HEAP32[$yymsp>>2]|0;
   $1397 = ((($1396)) + -32|0);
   $1398 = ((($1397)) + 4|0);
   $1399 = (_sqlite3ExprAlloc($1395,55,$1398,1)|0);
   HEAP32[$temp26>>2] = $1399;
   $1400 = HEAP32[$pParse>>2]|0;
   $1401 = HEAP32[$1400>>2]|0;
   $1402 = HEAP32[$yymsp>>2]|0;
   $1403 = ((($1402)) + 4|0);
   $1404 = (_sqlite3ExprAlloc($1401,55,$1403,1)|0);
   HEAP32[$temp3>>2] = $1404;
   $1405 = HEAP32[$pParse>>2]|0;
   $1406 = HEAP32[$temp26>>2]|0;
   $1407 = HEAP32[$temp3>>2]|0;
   $1408 = (_sqlite3PExpr($1405,122,$1406,$1407,0)|0);
   HEAP32[$temp4>>2] = $1408;
   $1409 = HEAP32[$yymsp>>2]|0;
   $1410 = ((($1409)) + -64|0);
   $1411 = ((($1410)) + 4|0);
   $1412 = HEAP32[$yymsp>>2]|0;
   $1413 = ((($1412)) + -64|0);
   $1414 = ((($1413)) + 4|0);
   $1415 = HEAP32[$yymsp>>2]|0;
   $1416 = ((($1415)) + 4|0);
   _spanSet($1411,$1414,$1416);
   $1417 = HEAP32[$pParse>>2]|0;
   $1418 = HEAP32[$temp15>>2]|0;
   $1419 = HEAP32[$temp4>>2]|0;
   $1420 = (_sqlite3PExpr($1417,122,$1418,$1419,0)|0);
   $1421 = HEAP32[$yymsp>>2]|0;
   $1422 = ((($1421)) + -64|0);
   $1423 = ((($1422)) + 4|0);
   HEAP32[$1423>>2] = $1420;
   break;
  }
  case 158:  {
   $1424 = HEAP32[$pParse>>2]|0;
   $1425 = HEAP32[$1424>>2]|0;
   $1426 = HEAP32[$yymsp>>2]|0;
   $1427 = ((($1426)) + 4|0);
   $1428 = (_sqlite3ExprAlloc($1425,134,$1427,1)|0);
   HEAP32[$yylhsminor>>2] = $1428;
   $1429 = HEAP32[$yymsp>>2]|0;
   $1430 = ((($1429)) + 4|0);
   $1431 = HEAP32[$1430>>2]|0;
   $1432 = ((($yylhsminor)) + 4|0);
   HEAP32[$1432>>2] = $1431;
   $1433 = HEAP32[$yymsp>>2]|0;
   $1434 = ((($1433)) + 4|0);
   $1435 = HEAP32[$1434>>2]|0;
   $1436 = HEAP32[$yymsp>>2]|0;
   $1437 = ((($1436)) + 4|0);
   $1438 = ((($1437)) + 4|0);
   $1439 = HEAP32[$1438>>2]|0;
   $1440 = (($1435) + ($1439)|0);
   $1441 = ((($yylhsminor)) + 8|0);
   HEAP32[$1441>>2] = $1440;
   $1442 = HEAP32[$yylhsminor>>2]|0;
   $1443 = ($1442|0)!=(0|0);
   if ($1443) {
    $1444 = HEAP32[$yylhsminor>>2]|0;
    $1445 = ((($1444)) + 4|0);
    $1446 = HEAP32[$1445>>2]|0;
    $1447 = $1446 | 8388608;
    HEAP32[$1445>>2] = $1447;
   }
   $1448 = HEAP32[$yymsp>>2]|0;
   $1449 = ((($1448)) + 4|0);
   ;HEAP32[$1449>>2]=HEAP32[$yylhsminor>>2]|0;HEAP32[$1449+4>>2]=HEAP32[$yylhsminor+4>>2]|0;HEAP32[$1449+8>>2]=HEAP32[$yylhsminor+8>>2]|0;
   break;
  }
  case 159:  {
   $1450 = HEAP32[$yymsp>>2]|0;
   $1451 = ((($1450)) + 4|0);
   $1452 = HEAP32[$1451>>2]|0;
   $1453 = HEAP8[$1452>>0]|0;
   $1454 = $1453 << 24 >> 24;
   $1455 = ($1454|0)==(35);
   if ($1455) {
    $1456 = HEAP32[$yymsp>>2]|0;
    $1457 = ((($1456)) + 4|0);
    $1458 = HEAP32[$1457>>2]|0;
    $1459 = ((($1458)) + 1|0);
    $1460 = HEAP8[$1459>>0]|0;
    $1461 = $1460&255;
    $1462 = (18403 + ($1461)|0);
    $1463 = HEAP8[$1462>>0]|0;
    $1464 = $1463&255;
    $1465 = $1464 & 4;
    $1466 = ($1465|0)!=(0);
    if ($1466) {
     $1481 = HEAP32[$yymsp>>2]|0;
     $1482 = ((($1481)) + 4|0);
     ;HEAP32[$t>>2]=HEAP32[$1482>>2]|0;HEAP32[$t+4>>2]=HEAP32[$1482+4>>2]|0;
     $1483 = HEAP32[$yymsp>>2]|0;
     $1484 = ((($1483)) + 4|0);
     _spanSet($1484,$t,$t);
     $1485 = HEAP32[$pParse>>2]|0;
     $1486 = ((($1485)) + 18|0);
     $1487 = HEAP8[$1486>>0]|0;
     $1488 = $1487&255;
     $1489 = ($1488|0)==(0);
     $1490 = HEAP32[$pParse>>2]|0;
     if ($1489) {
      HEAP32[$vararg_buffer2>>2] = $t;
      _sqlite3ErrorMsg($1490,26462,$vararg_buffer2);
      $1491 = HEAP32[$yymsp>>2]|0;
      $1492 = ((($1491)) + 4|0);
      HEAP32[$1492>>2] = 0;
      break L6;
     }
     $1493 = (_sqlite3PExpr($1490,157,0,0,0)|0);
     $1494 = HEAP32[$yymsp>>2]|0;
     $1495 = ((($1494)) + 4|0);
     HEAP32[$1495>>2] = $1493;
     $1496 = HEAP32[$yymsp>>2]|0;
     $1497 = ((($1496)) + 4|0);
     $1498 = HEAP32[$1497>>2]|0;
     $1499 = ($1498|0)!=(0|0);
     if (!($1499)) {
      break L6;
     }
     $1500 = HEAP32[$t>>2]|0;
     $1501 = ((($1500)) + 1|0);
     $1502 = HEAP32[$yymsp>>2]|0;
     $1503 = ((($1502)) + 4|0);
     $1504 = HEAP32[$1503>>2]|0;
     $1505 = ((($1504)) + 28|0);
     (_sqlite3GetInt32($1501,$1505)|0);
     break L6;
    }
   }
   $1467 = HEAP32[$yymsp>>2]|0;
   $1468 = ((($1467)) + 4|0);
   $1469 = ((($1468)) + 4|0);
   $1470 = HEAP32[$1469>>2]|0;
   HEAP32[$n>>2] = $1470;
   $1471 = HEAP32[$yymsp>>2]|0;
   $1472 = ((($1471)) + 4|0);
   $1473 = HEAP32[$pParse>>2]|0;
   $1474 = HEAP32[$yymsp>>2]|0;
   $1475 = ((($1474)) + 4|0);
   ;HEAP32[$$byval_copy16>>2]=HEAP32[$1475>>2]|0;HEAP32[$$byval_copy16+4>>2]=HEAP32[$1475+4>>2]|0;
   _spanExpr($1472,$1473,135,$$byval_copy16);
   $1476 = HEAP32[$pParse>>2]|0;
   $1477 = HEAP32[$yymsp>>2]|0;
   $1478 = ((($1477)) + 4|0);
   $1479 = HEAP32[$1478>>2]|0;
   $1480 = HEAP32[$n>>2]|0;
   _sqlite3ExprAssignVarNumber($1476,$1479,$1480);
   break;
  }
  case 160:  {
   $1506 = HEAP32[$pParse>>2]|0;
   $1507 = HEAP32[$yymsp>>2]|0;
   $1508 = ((($1507)) + -32|0);
   $1509 = ((($1508)) + 4|0);
   $1510 = HEAP32[$1509>>2]|0;
   $1511 = HEAP32[$yymsp>>2]|0;
   $1512 = ((($1511)) + 4|0);
   $1513 = (_sqlite3ExprAddCollateToken($1506,$1510,$1512,1)|0);
   $1514 = HEAP32[$yymsp>>2]|0;
   $1515 = ((($1514)) + -32|0);
   $1516 = ((($1515)) + 4|0);
   HEAP32[$1516>>2] = $1513;
   $1517 = HEAP32[$yymsp>>2]|0;
   $1518 = ((($1517)) + 4|0);
   $1519 = ((($1518)) + 4|0);
   $1520 = HEAP32[$1519>>2]|0;
   $1521 = HEAP32[$yymsp>>2]|0;
   $1522 = ((($1521)) + 4|0);
   $1523 = HEAP32[$1522>>2]|0;
   $1524 = (($1523) + ($1520)|0);
   $1525 = HEAP32[$yymsp>>2]|0;
   $1526 = ((($1525)) + -32|0);
   $1527 = ((($1526)) + 4|0);
   $1528 = ((($1527)) + 8|0);
   HEAP32[$1528>>2] = $1524;
   break;
  }
  case 161:  {
   $1529 = HEAP32[$yymsp>>2]|0;
   $1530 = ((($1529)) + -80|0);
   $1531 = ((($1530)) + 4|0);
   $1532 = HEAP32[$yymsp>>2]|0;
   $1533 = ((($1532)) + -80|0);
   $1534 = ((($1533)) + 4|0);
   $1535 = HEAP32[$yymsp>>2]|0;
   $1536 = ((($1535)) + 4|0);
   _spanSet($1531,$1534,$1536);
   $1537 = HEAP32[$pParse>>2]|0;
   $1538 = HEAP32[$yymsp>>2]|0;
   $1539 = ((($1538)) + -48|0);
   $1540 = ((($1539)) + 4|0);
   $1541 = HEAP32[$1540>>2]|0;
   $1542 = HEAP32[$yymsp>>2]|0;
   $1543 = ((($1542)) + -16|0);
   $1544 = ((($1543)) + 4|0);
   $1545 = (_sqlite3PExpr($1537,66,$1541,0,$1544)|0);
   $1546 = HEAP32[$yymsp>>2]|0;
   $1547 = ((($1546)) + -80|0);
   $1548 = ((($1547)) + 4|0);
   HEAP32[$1548>>2] = $1545;
   break;
  }
  case 162:  {
   $1549 = HEAP32[$yymsp>>2]|0;
   $1550 = ((($1549)) + -16|0);
   $1551 = ((($1550)) + 4|0);
   $1552 = HEAP32[$1551>>2]|0;
   $1553 = ($1552|0)!=(0|0);
   if ($1553) {
    $1554 = HEAP32[$yymsp>>2]|0;
    $1555 = ((($1554)) + -16|0);
    $1556 = ((($1555)) + 4|0);
    $1557 = HEAP32[$1556>>2]|0;
    $1558 = HEAP32[$1557>>2]|0;
    $1559 = HEAP32[$pParse>>2]|0;
    $1560 = HEAP32[$1559>>2]|0;
    $1561 = ((($1560)) + 96|0);
    $1562 = ((($1561)) + 24|0);
    $1563 = HEAP32[$1562>>2]|0;
    $1564 = ($1558|0)>($1563|0);
    if ($1564) {
     $1565 = HEAP32[$pParse>>2]|0;
     $1566 = HEAP32[$yymsp>>2]|0;
     $1567 = ((($1566)) + -64|0);
     $1568 = ((($1567)) + 4|0);
     HEAP32[$vararg_buffer5>>2] = $1568;
     _sqlite3ErrorMsg($1565,27186,$vararg_buffer5);
    }
   }
   $1569 = HEAP32[$pParse>>2]|0;
   $1570 = HEAP32[$yymsp>>2]|0;
   $1571 = ((($1570)) + -16|0);
   $1572 = ((($1571)) + 4|0);
   $1573 = HEAP32[$1572>>2]|0;
   $1574 = HEAP32[$yymsp>>2]|0;
   $1575 = ((($1574)) + -64|0);
   $1576 = ((($1575)) + 4|0);
   $1577 = (_sqlite3ExprFunction($1569,$1573,$1576)|0);
   HEAP32[$yylhsminor>>2] = $1577;
   $1578 = HEAP32[$yymsp>>2]|0;
   $1579 = ((($1578)) + -64|0);
   $1580 = ((($1579)) + 4|0);
   $1581 = HEAP32[$yymsp>>2]|0;
   $1582 = ((($1581)) + 4|0);
   _spanSet($yylhsminor,$1580,$1582);
   $1583 = HEAP32[$yymsp>>2]|0;
   $1584 = ((($1583)) + -32|0);
   $1585 = ((($1584)) + 4|0);
   $1586 = HEAP32[$1585>>2]|0;
   $1587 = ($1586|0)==(1);
   if ($1587) {
    $1588 = HEAP32[$yylhsminor>>2]|0;
    $1589 = ($1588|0)!=(0|0);
    if ($1589) {
     $1590 = HEAP32[$yylhsminor>>2]|0;
     $1591 = ((($1590)) + 4|0);
     $1592 = HEAP32[$1591>>2]|0;
     $1593 = $1592 | 16;
     HEAP32[$1591>>2] = $1593;
    }
   }
   $1594 = HEAP32[$yymsp>>2]|0;
   $1595 = ((($1594)) + -64|0);
   $1596 = ((($1595)) + 4|0);
   ;HEAP32[$1596>>2]=HEAP32[$yylhsminor>>2]|0;HEAP32[$1596+4>>2]=HEAP32[$yylhsminor+4>>2]|0;HEAP32[$1596+8>>2]=HEAP32[$yylhsminor+8>>2]|0;
   break;
  }
  case 163:  {
   $1597 = HEAP32[$pParse>>2]|0;
   $1598 = HEAP32[$yymsp>>2]|0;
   $1599 = ((($1598)) + -48|0);
   $1600 = ((($1599)) + 4|0);
   $1601 = (_sqlite3ExprFunction($1597,0,$1600)|0);
   HEAP32[$yylhsminor>>2] = $1601;
   $1602 = HEAP32[$yymsp>>2]|0;
   $1603 = ((($1602)) + -48|0);
   $1604 = ((($1603)) + 4|0);
   $1605 = HEAP32[$yymsp>>2]|0;
   $1606 = ((($1605)) + 4|0);
   _spanSet($yylhsminor,$1604,$1606);
   $1607 = HEAP32[$yymsp>>2]|0;
   $1608 = ((($1607)) + -48|0);
   $1609 = ((($1608)) + 4|0);
   ;HEAP32[$1609>>2]=HEAP32[$yylhsminor>>2]|0;HEAP32[$1609+4>>2]=HEAP32[$yylhsminor+4>>2]|0;HEAP32[$1609+8>>2]=HEAP32[$yylhsminor+8>>2]|0;
   break;
  }
  case 164:  {
   $1610 = HEAP32[$pParse>>2]|0;
   $1611 = HEAP32[$yymsp>>2]|0;
   $1612 = ((($1611)) + 4|0);
   $1613 = (_sqlite3ExprFunction($1610,0,$1612)|0);
   HEAP32[$yylhsminor>>2] = $1613;
   $1614 = HEAP32[$yymsp>>2]|0;
   $1615 = ((($1614)) + 4|0);
   $1616 = HEAP32[$yymsp>>2]|0;
   $1617 = ((($1616)) + 4|0);
   _spanSet($yylhsminor,$1615,$1617);
   $1618 = HEAP32[$yymsp>>2]|0;
   $1619 = ((($1618)) + 4|0);
   ;HEAP32[$1619>>2]=HEAP32[$yylhsminor>>2]|0;HEAP32[$1619+4>>2]=HEAP32[$yylhsminor+4>>2]|0;HEAP32[$1619+8>>2]=HEAP32[$yylhsminor+8>>2]|0;
   break;
  }
  case 165:  {
   $1620 = HEAP32[$pParse>>2]|0;
   $1621 = HEAP32[$yymsp>>2]|0;
   $1622 = ((($1621)) + -48|0);
   $1623 = ((($1622)) + 4|0);
   $1624 = HEAP32[$1623>>2]|0;
   $1625 = HEAP32[$yymsp>>2]|0;
   $1626 = ((($1625)) + -16|0);
   $1627 = ((($1626)) + 4|0);
   $1628 = HEAP32[$1627>>2]|0;
   $1629 = (_sqlite3ExprListAppend($1620,$1624,$1628)|0);
   HEAP32[$pList>>2] = $1629;
   $1630 = HEAP32[$pParse>>2]|0;
   $1631 = (_sqlite3PExpr($1630,158,0,0,0)|0);
   HEAP32[$yylhsminor>>2] = $1631;
   $1632 = HEAP32[$yylhsminor>>2]|0;
   $1633 = ($1632|0)!=(0|0);
   if ($1633) {
    $1634 = HEAP32[$pList>>2]|0;
    $1635 = HEAP32[$yylhsminor>>2]|0;
    $1636 = ((($1635)) + 20|0);
    HEAP32[$1636>>2] = $1634;
    $1637 = HEAP32[$yymsp>>2]|0;
    $1638 = ((($1637)) + -64|0);
    $1639 = ((($1638)) + 4|0);
    $1640 = HEAP32[$yymsp>>2]|0;
    $1641 = ((($1640)) + 4|0);
    _spanSet($yylhsminor,$1639,$1641);
   } else {
    $1642 = HEAP32[$pParse>>2]|0;
    $1643 = HEAP32[$1642>>2]|0;
    $1644 = HEAP32[$pList>>2]|0;
    _sqlite3ExprListDelete($1643,$1644);
   }
   $1645 = HEAP32[$yymsp>>2]|0;
   $1646 = ((($1645)) + -64|0);
   $1647 = ((($1646)) + 4|0);
   ;HEAP32[$1647>>2]=HEAP32[$yylhsminor>>2]|0;HEAP32[$1647+4>>2]=HEAP32[$yylhsminor+4>>2]|0;HEAP32[$1647+8>>2]=HEAP32[$yylhsminor+8>>2]|0;
   break;
  }
  case 173: case 172: case 171: case 170: case 169: case 168: case 167: case 166:  {
   $1648 = HEAP32[$pParse>>2]|0;
   $1649 = HEAP32[$yymsp>>2]|0;
   $1650 = ((($1649)) + -16|0);
   $1651 = ((($1650)) + 2|0);
   $1652 = HEAP8[$1651>>0]|0;
   $1653 = $1652&255;
   $1654 = HEAP32[$yymsp>>2]|0;
   $1655 = ((($1654)) + -32|0);
   $1656 = ((($1655)) + 4|0);
   $1657 = HEAP32[$yymsp>>2]|0;
   $1658 = ((($1657)) + 4|0);
   _spanBinaryExpr($1648,$1653,$1656,$1658);
   break;
  }
  case 174:  {
   $1659 = HEAP32[$yymsp>>2]|0;
   $1660 = ((($1659)) + 4|0);
   $1661 = HEAP32[$yymsp>>2]|0;
   $1662 = ((($1661)) + 4|0);
   ;HEAP32[$1660>>2]=HEAP32[$1662>>2]|0;HEAP32[$1660+4>>2]=HEAP32[$1662+4>>2]|0;
   break;
  }
  case 175:  {
   $1663 = HEAP32[$yymsp>>2]|0;
   $1664 = ((($1663)) + -16|0);
   $1665 = ((($1664)) + 4|0);
   $1666 = HEAP32[$yymsp>>2]|0;
   $1667 = ((($1666)) + 4|0);
   ;HEAP32[$1665>>2]=HEAP32[$1667>>2]|0;HEAP32[$1665+4>>2]=HEAP32[$1667+4>>2]|0;
   $1668 = HEAP32[$yymsp>>2]|0;
   $1669 = ((($1668)) + -16|0);
   $1670 = ((($1669)) + 4|0);
   $1671 = ((($1670)) + 4|0);
   $1672 = HEAP32[$1671>>2]|0;
   $1673 = $1672 | -2147483648;
   HEAP32[$1671>>2] = $1673;
   break;
  }
  case 176:  {
   $1674 = HEAP32[$yymsp>>2]|0;
   $1675 = ((($1674)) + -16|0);
   $1676 = ((($1675)) + 4|0);
   $1677 = ((($1676)) + 4|0);
   $1678 = HEAP32[$1677>>2]|0;
   $1679 = $1678 & -2147483648;
   HEAP32[$bNot>>2] = $1679;
   $1680 = HEAP32[$yymsp>>2]|0;
   $1681 = ((($1680)) + -16|0);
   $1682 = ((($1681)) + 4|0);
   $1683 = ((($1682)) + 4|0);
   $1684 = HEAP32[$1683>>2]|0;
   $1685 = $1684 & 2147483647;
   HEAP32[$1683>>2] = $1685;
   $1686 = HEAP32[$pParse>>2]|0;
   $1687 = HEAP32[$yymsp>>2]|0;
   $1688 = ((($1687)) + 4|0);
   $1689 = HEAP32[$1688>>2]|0;
   $1690 = (_sqlite3ExprListAppend($1686,0,$1689)|0);
   HEAP32[$pList7>>2] = $1690;
   $1691 = HEAP32[$pParse>>2]|0;
   $1692 = HEAP32[$pList7>>2]|0;
   $1693 = HEAP32[$yymsp>>2]|0;
   $1694 = ((($1693)) + -32|0);
   $1695 = ((($1694)) + 4|0);
   $1696 = HEAP32[$1695>>2]|0;
   $1697 = (_sqlite3ExprListAppend($1691,$1692,$1696)|0);
   HEAP32[$pList7>>2] = $1697;
   $1698 = HEAP32[$pParse>>2]|0;
   $1699 = HEAP32[$pList7>>2]|0;
   $1700 = HEAP32[$yymsp>>2]|0;
   $1701 = ((($1700)) + -16|0);
   $1702 = ((($1701)) + 4|0);
   $1703 = (_sqlite3ExprFunction($1698,$1699,$1702)|0);
   $1704 = HEAP32[$yymsp>>2]|0;
   $1705 = ((($1704)) + -32|0);
   $1706 = ((($1705)) + 4|0);
   HEAP32[$1706>>2] = $1703;
   $1707 = HEAP32[$pParse>>2]|0;
   $1708 = HEAP32[$bNot>>2]|0;
   $1709 = HEAP32[$yymsp>>2]|0;
   $1710 = ((($1709)) + -32|0);
   $1711 = ((($1710)) + 4|0);
   _exprNot($1707,$1708,$1711);
   $1712 = HEAP32[$yymsp>>2]|0;
   $1713 = ((($1712)) + 4|0);
   $1714 = ((($1713)) + 8|0);
   $1715 = HEAP32[$1714>>2]|0;
   $1716 = HEAP32[$yymsp>>2]|0;
   $1717 = ((($1716)) + -32|0);
   $1718 = ((($1717)) + 4|0);
   $1719 = ((($1718)) + 8|0);
   HEAP32[$1719>>2] = $1715;
   $1720 = HEAP32[$yymsp>>2]|0;
   $1721 = ((($1720)) + -32|0);
   $1722 = ((($1721)) + 4|0);
   $1723 = HEAP32[$1722>>2]|0;
   $1724 = ($1723|0)!=(0|0);
   if ($1724) {
    $1725 = HEAP32[$yymsp>>2]|0;
    $1726 = ((($1725)) + -32|0);
    $1727 = ((($1726)) + 4|0);
    $1728 = HEAP32[$1727>>2]|0;
    $1729 = ((($1728)) + 4|0);
    $1730 = HEAP32[$1729>>2]|0;
    $1731 = $1730 | 128;
    HEAP32[$1729>>2] = $1731;
   }
   break;
  }
  case 177:  {
   $1732 = HEAP32[$yymsp>>2]|0;
   $1733 = ((($1732)) + -48|0);
   $1734 = ((($1733)) + 4|0);
   $1735 = ((($1734)) + 4|0);
   $1736 = HEAP32[$1735>>2]|0;
   $1737 = $1736 & -2147483648;
   HEAP32[$bNot9>>2] = $1737;
   $1738 = HEAP32[$yymsp>>2]|0;
   $1739 = ((($1738)) + -48|0);
   $1740 = ((($1739)) + 4|0);
   $1741 = ((($1740)) + 4|0);
   $1742 = HEAP32[$1741>>2]|0;
   $1743 = $1742 & 2147483647;
   HEAP32[$1741>>2] = $1743;
   $1744 = HEAP32[$pParse>>2]|0;
   $1745 = HEAP32[$yymsp>>2]|0;
   $1746 = ((($1745)) + -32|0);
   $1747 = ((($1746)) + 4|0);
   $1748 = HEAP32[$1747>>2]|0;
   $1749 = (_sqlite3ExprListAppend($1744,0,$1748)|0);
   HEAP32[$pList8>>2] = $1749;
   $1750 = HEAP32[$pParse>>2]|0;
   $1751 = HEAP32[$pList8>>2]|0;
   $1752 = HEAP32[$yymsp>>2]|0;
   $1753 = ((($1752)) + -64|0);
   $1754 = ((($1753)) + 4|0);
   $1755 = HEAP32[$1754>>2]|0;
   $1756 = (_sqlite3ExprListAppend($1750,$1751,$1755)|0);
   HEAP32[$pList8>>2] = $1756;
   $1757 = HEAP32[$pParse>>2]|0;
   $1758 = HEAP32[$pList8>>2]|0;
   $1759 = HEAP32[$yymsp>>2]|0;
   $1760 = ((($1759)) + 4|0);
   $1761 = HEAP32[$1760>>2]|0;
   $1762 = (_sqlite3ExprListAppend($1757,$1758,$1761)|0);
   HEAP32[$pList8>>2] = $1762;
   $1763 = HEAP32[$pParse>>2]|0;
   $1764 = HEAP32[$pList8>>2]|0;
   $1765 = HEAP32[$yymsp>>2]|0;
   $1766 = ((($1765)) + -48|0);
   $1767 = ((($1766)) + 4|0);
   $1768 = (_sqlite3ExprFunction($1763,$1764,$1767)|0);
   $1769 = HEAP32[$yymsp>>2]|0;
   $1770 = ((($1769)) + -64|0);
   $1771 = ((($1770)) + 4|0);
   HEAP32[$1771>>2] = $1768;
   $1772 = HEAP32[$pParse>>2]|0;
   $1773 = HEAP32[$bNot9>>2]|0;
   $1774 = HEAP32[$yymsp>>2]|0;
   $1775 = ((($1774)) + -64|0);
   $1776 = ((($1775)) + 4|0);
   _exprNot($1772,$1773,$1776);
   $1777 = HEAP32[$yymsp>>2]|0;
   $1778 = ((($1777)) + 4|0);
   $1779 = ((($1778)) + 8|0);
   $1780 = HEAP32[$1779>>2]|0;
   $1781 = HEAP32[$yymsp>>2]|0;
   $1782 = ((($1781)) + -64|0);
   $1783 = ((($1782)) + 4|0);
   $1784 = ((($1783)) + 8|0);
   HEAP32[$1784>>2] = $1780;
   $1785 = HEAP32[$yymsp>>2]|0;
   $1786 = ((($1785)) + -64|0);
   $1787 = ((($1786)) + 4|0);
   $1788 = HEAP32[$1787>>2]|0;
   $1789 = ($1788|0)!=(0|0);
   if ($1789) {
    $1790 = HEAP32[$yymsp>>2]|0;
    $1791 = ((($1790)) + -64|0);
    $1792 = ((($1791)) + 4|0);
    $1793 = HEAP32[$1792>>2]|0;
    $1794 = ((($1793)) + 4|0);
    $1795 = HEAP32[$1794>>2]|0;
    $1796 = $1795 | 128;
    HEAP32[$1794>>2] = $1796;
   }
   break;
  }
  case 178:  {
   $1797 = HEAP32[$pParse>>2]|0;
   $1798 = HEAP32[$yymsp>>2]|0;
   $1799 = ((($1798)) + 2|0);
   $1800 = HEAP8[$1799>>0]|0;
   $1801 = $1800&255;
   $1802 = HEAP32[$yymsp>>2]|0;
   $1803 = ((($1802)) + -16|0);
   $1804 = ((($1803)) + 4|0);
   $1805 = HEAP32[$yymsp>>2]|0;
   $1806 = ((($1805)) + 4|0);
   _spanUnaryPostfix($1797,$1801,$1804,$1806);
   break;
  }
  case 179:  {
   $1807 = HEAP32[$pParse>>2]|0;
   $1808 = HEAP32[$yymsp>>2]|0;
   $1809 = ((($1808)) + -32|0);
   $1810 = ((($1809)) + 4|0);
   $1811 = HEAP32[$yymsp>>2]|0;
   $1812 = ((($1811)) + 4|0);
   _spanUnaryPostfix($1807,35,$1810,$1812);
   break;
  }
  case 180:  {
   $1813 = HEAP32[$pParse>>2]|0;
   $1814 = HEAP32[$yymsp>>2]|0;
   $1815 = ((($1814)) + -32|0);
   $1816 = ((($1815)) + 4|0);
   $1817 = HEAP32[$yymsp>>2]|0;
   $1818 = ((($1817)) + 4|0);
   _spanBinaryExpr($1813,29,$1816,$1818);
   $1819 = HEAP32[$pParse>>2]|0;
   $1820 = HEAP32[$yymsp>>2]|0;
   $1821 = ((($1820)) + 4|0);
   $1822 = HEAP32[$1821>>2]|0;
   $1823 = HEAP32[$yymsp>>2]|0;
   $1824 = ((($1823)) + -32|0);
   $1825 = ((($1824)) + 4|0);
   $1826 = HEAP32[$1825>>2]|0;
   _binaryToUnaryIfNull($1819,$1822,$1826,34);
   break;
  }
  case 181:  {
   $1827 = HEAP32[$pParse>>2]|0;
   $1828 = HEAP32[$yymsp>>2]|0;
   $1829 = ((($1828)) + -48|0);
   $1830 = ((($1829)) + 4|0);
   $1831 = HEAP32[$yymsp>>2]|0;
   $1832 = ((($1831)) + 4|0);
   _spanBinaryExpr($1827,148,$1830,$1832);
   $1833 = HEAP32[$pParse>>2]|0;
   $1834 = HEAP32[$yymsp>>2]|0;
   $1835 = ((($1834)) + 4|0);
   $1836 = HEAP32[$1835>>2]|0;
   $1837 = HEAP32[$yymsp>>2]|0;
   $1838 = ((($1837)) + -48|0);
   $1839 = ((($1838)) + 4|0);
   $1840 = HEAP32[$1839>>2]|0;
   _binaryToUnaryIfNull($1833,$1836,$1840,35);
   break;
  }
  case 183: case 182:  {
   $1841 = HEAP32[$yymsp>>2]|0;
   $1842 = ((($1841)) + -16|0);
   $1843 = ((($1842)) + 4|0);
   $1844 = HEAP32[$pParse>>2]|0;
   $1845 = HEAP32[$yymsp>>2]|0;
   $1846 = ((($1845)) + -16|0);
   $1847 = ((($1846)) + 2|0);
   $1848 = HEAP8[$1847>>0]|0;
   $1849 = $1848&255;
   $1850 = HEAP32[$yymsp>>2]|0;
   $1851 = ((($1850)) + 4|0);
   $1852 = HEAP32[$yymsp>>2]|0;
   $1853 = ((($1852)) + -16|0);
   $1854 = ((($1853)) + 4|0);
   _spanUnaryPrefix($1843,$1844,$1849,$1851,$1854);
   break;
  }
  case 184:  {
   $1855 = HEAP32[$yymsp>>2]|0;
   $1856 = ((($1855)) + -16|0);
   $1857 = ((($1856)) + 4|0);
   $1858 = HEAP32[$pParse>>2]|0;
   $1859 = HEAP32[$yymsp>>2]|0;
   $1860 = ((($1859)) + 4|0);
   $1861 = HEAP32[$yymsp>>2]|0;
   $1862 = ((($1861)) + -16|0);
   $1863 = ((($1862)) + 4|0);
   _spanUnaryPrefix($1857,$1858,155,$1860,$1863);
   break;
  }
  case 185:  {
   $1864 = HEAP32[$yymsp>>2]|0;
   $1865 = ((($1864)) + -16|0);
   $1866 = ((($1865)) + 4|0);
   $1867 = HEAP32[$pParse>>2]|0;
   $1868 = HEAP32[$yymsp>>2]|0;
   $1869 = ((($1868)) + 4|0);
   $1870 = HEAP32[$yymsp>>2]|0;
   $1871 = ((($1870)) + -16|0);
   $1872 = ((($1871)) + 4|0);
   _spanUnaryPrefix($1866,$1867,156,$1869,$1872);
   break;
  }
  case 189: case 186:  {
   $1873 = HEAP32[$yymsp>>2]|0;
   $1874 = ((($1873)) + 4|0);
   HEAP32[$1874>>2] = 0;
   break;
  }
  case 188:  {
   $1875 = HEAP32[$pParse>>2]|0;
   $1876 = HEAP32[$yymsp>>2]|0;
   $1877 = ((($1876)) + -32|0);
   $1878 = ((($1877)) + 4|0);
   $1879 = HEAP32[$1878>>2]|0;
   $1880 = (_sqlite3ExprListAppend($1875,0,$1879)|0);
   HEAP32[$pList10>>2] = $1880;
   $1881 = HEAP32[$pParse>>2]|0;
   $1882 = HEAP32[$pList10>>2]|0;
   $1883 = HEAP32[$yymsp>>2]|0;
   $1884 = ((($1883)) + 4|0);
   $1885 = HEAP32[$1884>>2]|0;
   $1886 = (_sqlite3ExprListAppend($1881,$1882,$1885)|0);
   HEAP32[$pList10>>2] = $1886;
   $1887 = HEAP32[$pParse>>2]|0;
   $1888 = HEAP32[$yymsp>>2]|0;
   $1889 = ((($1888)) + -64|0);
   $1890 = ((($1889)) + 4|0);
   $1891 = HEAP32[$1890>>2]|0;
   $1892 = (_sqlite3PExpr($1887,32,$1891,0,0)|0);
   $1893 = HEAP32[$yymsp>>2]|0;
   $1894 = ((($1893)) + -64|0);
   $1895 = ((($1894)) + 4|0);
   HEAP32[$1895>>2] = $1892;
   $1896 = HEAP32[$yymsp>>2]|0;
   $1897 = ((($1896)) + -64|0);
   $1898 = ((($1897)) + 4|0);
   $1899 = HEAP32[$1898>>2]|0;
   $1900 = ($1899|0)!=(0|0);
   if ($1900) {
    $1901 = HEAP32[$pList10>>2]|0;
    $1902 = HEAP32[$yymsp>>2]|0;
    $1903 = ((($1902)) + -64|0);
    $1904 = ((($1903)) + 4|0);
    $1905 = HEAP32[$1904>>2]|0;
    $1906 = ((($1905)) + 20|0);
    HEAP32[$1906>>2] = $1901;
   } else {
    $1907 = HEAP32[$pParse>>2]|0;
    $1908 = HEAP32[$1907>>2]|0;
    $1909 = HEAP32[$pList10>>2]|0;
    _sqlite3ExprListDelete($1908,$1909);
   }
   $1910 = HEAP32[$pParse>>2]|0;
   $1911 = HEAP32[$yymsp>>2]|0;
   $1912 = ((($1911)) + -48|0);
   $1913 = ((($1912)) + 4|0);
   $1914 = HEAP32[$1913>>2]|0;
   $1915 = HEAP32[$yymsp>>2]|0;
   $1916 = ((($1915)) + -64|0);
   $1917 = ((($1916)) + 4|0);
   _exprNot($1910,$1914,$1917);
   $1918 = HEAP32[$yymsp>>2]|0;
   $1919 = ((($1918)) + 4|0);
   $1920 = ((($1919)) + 8|0);
   $1921 = HEAP32[$1920>>2]|0;
   $1922 = HEAP32[$yymsp>>2]|0;
   $1923 = ((($1922)) + -64|0);
   $1924 = ((($1923)) + 4|0);
   $1925 = ((($1924)) + 8|0);
   HEAP32[$1925>>2] = $1921;
   break;
  }
  case 191:  {
   $1926 = HEAP32[$yymsp>>2]|0;
   $1927 = ((($1926)) + -16|0);
   $1928 = ((($1927)) + 4|0);
   $1929 = HEAP32[$1928>>2]|0;
   $1930 = ($1929|0)==(0|0);
   do {
    if ($1930) {
     $1931 = HEAP32[$pParse>>2]|0;
     $1932 = HEAP32[$1931>>2]|0;
     $1933 = HEAP32[$yymsp>>2]|0;
     $1934 = ((($1933)) + -64|0);
     $1935 = ((($1934)) + 4|0);
     $1936 = HEAP32[$1935>>2]|0;
     _sqlite3ExprDelete($1932,$1936);
     $1937 = HEAP32[$pParse>>2]|0;
     $1938 = HEAP32[$yymsp>>2]|0;
     $1939 = ((($1938)) + -48|0);
     $1940 = ((($1939)) + 4|0);
     $1941 = HEAP32[$1940>>2]|0;
     $1942 = (4276 + ($1941<<3)|0);
     $1943 = (_sqlite3PExpr($1937,134,0,0,$1942)|0);
     $1944 = HEAP32[$yymsp>>2]|0;
     $1945 = ((($1944)) + -64|0);
     $1946 = ((($1945)) + 4|0);
     HEAP32[$1946>>2] = $1943;
    } else {
     $1947 = HEAP32[$yymsp>>2]|0;
     $1948 = ((($1947)) + -16|0);
     $1949 = ((($1948)) + 4|0);
     $1950 = HEAP32[$1949>>2]|0;
     $1951 = HEAP32[$1950>>2]|0;
     $1952 = ($1951|0)==(1);
     if ($1952) {
      $1953 = HEAP32[$yymsp>>2]|0;
      $1954 = ((($1953)) + -16|0);
      $1955 = ((($1954)) + 4|0);
      $1956 = HEAP32[$1955>>2]|0;
      $1957 = ((($1956)) + 4|0);
      $1958 = HEAP32[$1957>>2]|0;
      $1959 = HEAP32[$1958>>2]|0;
      HEAP32[$pRHS>>2] = $1959;
      $1960 = HEAP32[$yymsp>>2]|0;
      $1961 = ((($1960)) + -16|0);
      $1962 = ((($1961)) + 4|0);
      $1963 = HEAP32[$1962>>2]|0;
      $1964 = ((($1963)) + 4|0);
      $1965 = HEAP32[$1964>>2]|0;
      HEAP32[$1965>>2] = 0;
      $1966 = HEAP32[$pParse>>2]|0;
      $1967 = HEAP32[$1966>>2]|0;
      $1968 = HEAP32[$yymsp>>2]|0;
      $1969 = ((($1968)) + -16|0);
      $1970 = ((($1969)) + 4|0);
      $1971 = HEAP32[$1970>>2]|0;
      _sqlite3ExprListDelete($1967,$1971);
      $1972 = HEAP32[$pRHS>>2]|0;
      $1973 = ($1972|0)!=(0|0);
      if ($1973) {
       $1974 = HEAP32[$pRHS>>2]|0;
       $1975 = ((($1974)) + 4|0);
       $1976 = HEAP32[$1975>>2]|0;
       $1977 = $1976 & -257;
       HEAP32[$1975>>2] = $1977;
       $1978 = HEAP32[$pRHS>>2]|0;
       $1979 = ((($1978)) + 4|0);
       $1980 = HEAP32[$1979>>2]|0;
       $1981 = $1980 | 512;
       HEAP32[$1979>>2] = $1981;
      }
      $1982 = HEAP32[$pParse>>2]|0;
      $1983 = HEAP32[$yymsp>>2]|0;
      $1984 = ((($1983)) + -48|0);
      $1985 = ((($1984)) + 4|0);
      $1986 = HEAP32[$1985>>2]|0;
      $1987 = ($1986|0)!=(0);
      $1988 = $1987 ? 36 : 37;
      $1989 = HEAP32[$yymsp>>2]|0;
      $1990 = ((($1989)) + -64|0);
      $1991 = ((($1990)) + 4|0);
      $1992 = HEAP32[$1991>>2]|0;
      $1993 = HEAP32[$pRHS>>2]|0;
      $1994 = (_sqlite3PExpr($1982,$1988,$1992,$1993,0)|0);
      $1995 = HEAP32[$yymsp>>2]|0;
      $1996 = ((($1995)) + -64|0);
      $1997 = ((($1996)) + 4|0);
      HEAP32[$1997>>2] = $1994;
      break;
     }
     $1998 = HEAP32[$pParse>>2]|0;
     $1999 = HEAP32[$yymsp>>2]|0;
     $2000 = ((($1999)) + -64|0);
     $2001 = ((($2000)) + 4|0);
     $2002 = HEAP32[$2001>>2]|0;
     $2003 = (_sqlite3PExpr($1998,33,$2002,0,0)|0);
     $2004 = HEAP32[$yymsp>>2]|0;
     $2005 = ((($2004)) + -64|0);
     $2006 = ((($2005)) + 4|0);
     HEAP32[$2006>>2] = $2003;
     $2007 = HEAP32[$yymsp>>2]|0;
     $2008 = ((($2007)) + -64|0);
     $2009 = ((($2008)) + 4|0);
     $2010 = HEAP32[$2009>>2]|0;
     $2011 = ($2010|0)!=(0|0);
     if ($2011) {
      $2012 = HEAP32[$yymsp>>2]|0;
      $2013 = ((($2012)) + -16|0);
      $2014 = ((($2013)) + 4|0);
      $2015 = HEAP32[$2014>>2]|0;
      $2016 = HEAP32[$yymsp>>2]|0;
      $2017 = ((($2016)) + -64|0);
      $2018 = ((($2017)) + 4|0);
      $2019 = HEAP32[$2018>>2]|0;
      $2020 = ((($2019)) + 20|0);
      HEAP32[$2020>>2] = $2015;
      $2021 = HEAP32[$pParse>>2]|0;
      $2022 = HEAP32[$yymsp>>2]|0;
      $2023 = ((($2022)) + -64|0);
      $2024 = ((($2023)) + 4|0);
      $2025 = HEAP32[$2024>>2]|0;
      _sqlite3ExprSetHeightAndFlags($2021,$2025);
     } else {
      $2026 = HEAP32[$pParse>>2]|0;
      $2027 = HEAP32[$2026>>2]|0;
      $2028 = HEAP32[$yymsp>>2]|0;
      $2029 = ((($2028)) + -16|0);
      $2030 = ((($2029)) + 4|0);
      $2031 = HEAP32[$2030>>2]|0;
      _sqlite3ExprListDelete($2027,$2031);
     }
     $2032 = HEAP32[$pParse>>2]|0;
     $2033 = HEAP32[$yymsp>>2]|0;
     $2034 = ((($2033)) + -48|0);
     $2035 = ((($2034)) + 4|0);
     $2036 = HEAP32[$2035>>2]|0;
     $2037 = HEAP32[$yymsp>>2]|0;
     $2038 = ((($2037)) + -64|0);
     $2039 = ((($2038)) + 4|0);
     _exprNot($2032,$2036,$2039);
    }
   } while(0);
   $2040 = HEAP32[$yymsp>>2]|0;
   $2041 = ((($2040)) + 4|0);
   $2042 = ((($2041)) + 4|0);
   $2043 = HEAP32[$2042>>2]|0;
   $2044 = HEAP32[$yymsp>>2]|0;
   $2045 = ((($2044)) + 4|0);
   $2046 = HEAP32[$2045>>2]|0;
   $2047 = (($2046) + ($2043)|0);
   $2048 = HEAP32[$yymsp>>2]|0;
   $2049 = ((($2048)) + -64|0);
   $2050 = ((($2049)) + 4|0);
   $2051 = ((($2050)) + 8|0);
   HEAP32[$2051>>2] = $2047;
   break;
  }
  case 192:  {
   $2052 = HEAP32[$yymsp>>2]|0;
   $2053 = ((($2052)) + -32|0);
   $2054 = ((($2053)) + 4|0);
   $2055 = HEAP32[$yymsp>>2]|0;
   $2056 = ((($2055)) + -32|0);
   $2057 = ((($2056)) + 4|0);
   $2058 = HEAP32[$yymsp>>2]|0;
   $2059 = ((($2058)) + 4|0);
   _spanSet($2054,$2057,$2059);
   $2060 = HEAP32[$pParse>>2]|0;
   $2061 = (_sqlite3PExpr($2060,119,0,0,0)|0);
   $2062 = HEAP32[$yymsp>>2]|0;
   $2063 = ((($2062)) + -32|0);
   $2064 = ((($2063)) + 4|0);
   HEAP32[$2064>>2] = $2061;
   $2065 = HEAP32[$pParse>>2]|0;
   $2066 = HEAP32[$yymsp>>2]|0;
   $2067 = ((($2066)) + -32|0);
   $2068 = ((($2067)) + 4|0);
   $2069 = HEAP32[$2068>>2]|0;
   $2070 = HEAP32[$yymsp>>2]|0;
   $2071 = ((($2070)) + -16|0);
   $2072 = ((($2071)) + 4|0);
   $2073 = HEAP32[$2072>>2]|0;
   _sqlite3PExprAddSelect($2065,$2069,$2073);
   break;
  }
  case 193:  {
   $2074 = HEAP32[$pParse>>2]|0;
   $2075 = HEAP32[$yymsp>>2]|0;
   $2076 = ((($2075)) + -64|0);
   $2077 = ((($2076)) + 4|0);
   $2078 = HEAP32[$2077>>2]|0;
   $2079 = (_sqlite3PExpr($2074,33,$2078,0,0)|0);
   $2080 = HEAP32[$yymsp>>2]|0;
   $2081 = ((($2080)) + -64|0);
   $2082 = ((($2081)) + 4|0);
   HEAP32[$2082>>2] = $2079;
   $2083 = HEAP32[$pParse>>2]|0;
   $2084 = HEAP32[$yymsp>>2]|0;
   $2085 = ((($2084)) + -64|0);
   $2086 = ((($2085)) + 4|0);
   $2087 = HEAP32[$2086>>2]|0;
   $2088 = HEAP32[$yymsp>>2]|0;
   $2089 = ((($2088)) + -16|0);
   $2090 = ((($2089)) + 4|0);
   $2091 = HEAP32[$2090>>2]|0;
   _sqlite3PExprAddSelect($2083,$2087,$2091);
   $2092 = HEAP32[$pParse>>2]|0;
   $2093 = HEAP32[$yymsp>>2]|0;
   $2094 = ((($2093)) + -48|0);
   $2095 = ((($2094)) + 4|0);
   $2096 = HEAP32[$2095>>2]|0;
   $2097 = HEAP32[$yymsp>>2]|0;
   $2098 = ((($2097)) + -64|0);
   $2099 = ((($2098)) + 4|0);
   _exprNot($2092,$2096,$2099);
   $2100 = HEAP32[$yymsp>>2]|0;
   $2101 = ((($2100)) + 4|0);
   $2102 = ((($2101)) + 4|0);
   $2103 = HEAP32[$2102>>2]|0;
   $2104 = HEAP32[$yymsp>>2]|0;
   $2105 = ((($2104)) + 4|0);
   $2106 = HEAP32[$2105>>2]|0;
   $2107 = (($2106) + ($2103)|0);
   $2108 = HEAP32[$yymsp>>2]|0;
   $2109 = ((($2108)) + -64|0);
   $2110 = ((($2109)) + 4|0);
   $2111 = ((($2110)) + 8|0);
   HEAP32[$2111>>2] = $2107;
   break;
  }
  case 194:  {
   $2112 = HEAP32[$pParse>>2]|0;
   $2113 = HEAP32[$2112>>2]|0;
   $2114 = HEAP32[$yymsp>>2]|0;
   $2115 = ((($2114)) + -32|0);
   $2116 = ((($2115)) + 4|0);
   $2117 = HEAP32[$yymsp>>2]|0;
   $2118 = ((($2117)) + -16|0);
   $2119 = ((($2118)) + 4|0);
   $2120 = (_sqlite3SrcListAppend($2113,0,$2116,$2119)|0);
   HEAP32[$pSrc>>2] = $2120;
   $2121 = HEAP32[$pParse>>2]|0;
   $2122 = HEAP32[$pSrc>>2]|0;
   $2123 = (_sqlite3SelectNew($2121,0,$2122,0,0,0,0,0,0,0)|0);
   HEAP32[$pSelect>>2] = $2123;
   $2124 = HEAP32[$yymsp>>2]|0;
   $2125 = ((($2124)) + 4|0);
   $2126 = HEAP32[$2125>>2]|0;
   $2127 = ($2126|0)!=(0|0);
   if ($2127) {
    $2128 = HEAP32[$pParse>>2]|0;
    $2129 = HEAP32[$pSelect>>2]|0;
    $2130 = ($2129|0)!=(0|0);
    $2131 = HEAP32[$pSrc>>2]|0;
    $2132 = $2130 ? $2131 : 0;
    $2133 = HEAP32[$yymsp>>2]|0;
    $2134 = ((($2133)) + 4|0);
    $2135 = HEAP32[$2134>>2]|0;
    _sqlite3SrcListFuncArgs($2128,$2132,$2135);
   }
   $2136 = HEAP32[$pParse>>2]|0;
   $2137 = HEAP32[$yymsp>>2]|0;
   $2138 = ((($2137)) + -64|0);
   $2139 = ((($2138)) + 4|0);
   $2140 = HEAP32[$2139>>2]|0;
   $2141 = (_sqlite3PExpr($2136,33,$2140,0,0)|0);
   $2142 = HEAP32[$yymsp>>2]|0;
   $2143 = ((($2142)) + -64|0);
   $2144 = ((($2143)) + 4|0);
   HEAP32[$2144>>2] = $2141;
   $2145 = HEAP32[$pParse>>2]|0;
   $2146 = HEAP32[$yymsp>>2]|0;
   $2147 = ((($2146)) + -64|0);
   $2148 = ((($2147)) + 4|0);
   $2149 = HEAP32[$2148>>2]|0;
   $2150 = HEAP32[$pSelect>>2]|0;
   _sqlite3PExprAddSelect($2145,$2149,$2150);
   $2151 = HEAP32[$pParse>>2]|0;
   $2152 = HEAP32[$yymsp>>2]|0;
   $2153 = ((($2152)) + -48|0);
   $2154 = ((($2153)) + 4|0);
   $2155 = HEAP32[$2154>>2]|0;
   $2156 = HEAP32[$yymsp>>2]|0;
   $2157 = ((($2156)) + -64|0);
   $2158 = ((($2157)) + 4|0);
   _exprNot($2151,$2155,$2158);
   $2159 = HEAP32[$yymsp>>2]|0;
   $2160 = ((($2159)) + -16|0);
   $2161 = ((($2160)) + 4|0);
   $2162 = HEAP32[$2161>>2]|0;
   $2163 = ($2162|0)!=(0|0);
   $2164 = HEAP32[$yymsp>>2]|0;
   if ($2163) {
    $2165 = ((($2164)) + -16|0);
    $2166 = ((($2165)) + 4|0);
    $2167 = ((($2166)) + 4|0);
    $2168 = HEAP32[$2167>>2]|0;
    $2169 = HEAP32[$yymsp>>2]|0;
    $2170 = ((($2169)) + -16|0);
    $2171 = ((($2170)) + 4|0);
    $2172 = HEAP32[$2171>>2]|0;
    $2173 = (($2172) + ($2168)|0);
    $2187 = $2173;
   } else {
    $2174 = ((($2164)) + -32|0);
    $2175 = ((($2174)) + 4|0);
    $2176 = ((($2175)) + 4|0);
    $2177 = HEAP32[$2176>>2]|0;
    $2178 = HEAP32[$yymsp>>2]|0;
    $2179 = ((($2178)) + -32|0);
    $2180 = ((($2179)) + 4|0);
    $2181 = HEAP32[$2180>>2]|0;
    $2182 = (($2181) + ($2177)|0);
    $2187 = $2182;
   }
   $2183 = HEAP32[$yymsp>>2]|0;
   $2184 = ((($2183)) + -64|0);
   $2185 = ((($2184)) + 4|0);
   $2186 = ((($2185)) + 8|0);
   HEAP32[$2186>>2] = $2187;
   break;
  }
  case 195:  {
   $2188 = HEAP32[$yymsp>>2]|0;
   $2189 = ((($2188)) + -48|0);
   $2190 = ((($2189)) + 4|0);
   $2191 = HEAP32[$yymsp>>2]|0;
   $2192 = ((($2191)) + -48|0);
   $2193 = ((($2192)) + 4|0);
   $2194 = HEAP32[$yymsp>>2]|0;
   $2195 = ((($2194)) + 4|0);
   _spanSet($2190,$2193,$2195);
   $2196 = HEAP32[$pParse>>2]|0;
   $2197 = (_sqlite3PExpr($2196,20,0,0,0)|0);
   $2198 = HEAP32[$yymsp>>2]|0;
   $2199 = ((($2198)) + -48|0);
   $2200 = ((($2199)) + 4|0);
   HEAP32[$2200>>2] = $2197;
   HEAP32[$p11>>2] = $2197;
   $2201 = HEAP32[$pParse>>2]|0;
   $2202 = HEAP32[$p11>>2]|0;
   $2203 = HEAP32[$yymsp>>2]|0;
   $2204 = ((($2203)) + -16|0);
   $2205 = ((($2204)) + 4|0);
   $2206 = HEAP32[$2205>>2]|0;
   _sqlite3PExprAddSelect($2201,$2202,$2206);
   break;
  }
  case 196:  {
   $2207 = HEAP32[$yymsp>>2]|0;
   $2208 = ((($2207)) + -64|0);
   $2209 = ((($2208)) + 4|0);
   $2210 = HEAP32[$yymsp>>2]|0;
   $2211 = ((($2210)) + -64|0);
   $2212 = ((($2211)) + 4|0);
   $2213 = HEAP32[$yymsp>>2]|0;
   $2214 = ((($2213)) + 4|0);
   _spanSet($2209,$2212,$2214);
   $2215 = HEAP32[$pParse>>2]|0;
   $2216 = HEAP32[$yymsp>>2]|0;
   $2217 = ((($2216)) + -48|0);
   $2218 = ((($2217)) + 4|0);
   $2219 = HEAP32[$2218>>2]|0;
   $2220 = (_sqlite3PExpr($2215,136,$2219,0,0)|0);
   $2221 = HEAP32[$yymsp>>2]|0;
   $2222 = ((($2221)) + -64|0);
   $2223 = ((($2222)) + 4|0);
   HEAP32[$2223>>2] = $2220;
   $2224 = HEAP32[$yymsp>>2]|0;
   $2225 = ((($2224)) + -64|0);
   $2226 = ((($2225)) + 4|0);
   $2227 = HEAP32[$2226>>2]|0;
   $2228 = ($2227|0)!=(0|0);
   if (!($2228)) {
    $2259 = HEAP32[$pParse>>2]|0;
    $2260 = HEAP32[$2259>>2]|0;
    $2261 = HEAP32[$yymsp>>2]|0;
    $2262 = ((($2261)) + -32|0);
    $2263 = ((($2262)) + 4|0);
    $2264 = HEAP32[$2263>>2]|0;
    _sqlite3ExprListDelete($2260,$2264);
    $2265 = HEAP32[$pParse>>2]|0;
    $2266 = HEAP32[$2265>>2]|0;
    $2267 = HEAP32[$yymsp>>2]|0;
    $2268 = ((($2267)) + -16|0);
    $2269 = ((($2268)) + 4|0);
    $2270 = HEAP32[$2269>>2]|0;
    _sqlite3ExprDelete($2266,$2270);
    break L6;
   }
   $2229 = HEAP32[$yymsp>>2]|0;
   $2230 = ((($2229)) + -16|0);
   $2231 = ((($2230)) + 4|0);
   $2232 = HEAP32[$2231>>2]|0;
   $2233 = ($2232|0)!=(0|0);
   if ($2233) {
    $2234 = HEAP32[$pParse>>2]|0;
    $2235 = HEAP32[$yymsp>>2]|0;
    $2236 = ((($2235)) + -32|0);
    $2237 = ((($2236)) + 4|0);
    $2238 = HEAP32[$2237>>2]|0;
    $2239 = HEAP32[$yymsp>>2]|0;
    $2240 = ((($2239)) + -16|0);
    $2241 = ((($2240)) + 4|0);
    $2242 = HEAP32[$2241>>2]|0;
    $2243 = (_sqlite3ExprListAppend($2234,$2238,$2242)|0);
    $2253 = $2243;
   } else {
    $2244 = HEAP32[$yymsp>>2]|0;
    $2245 = ((($2244)) + -32|0);
    $2246 = ((($2245)) + 4|0);
    $2247 = HEAP32[$2246>>2]|0;
    $2253 = $2247;
   }
   $2248 = HEAP32[$yymsp>>2]|0;
   $2249 = ((($2248)) + -64|0);
   $2250 = ((($2249)) + 4|0);
   $2251 = HEAP32[$2250>>2]|0;
   $2252 = ((($2251)) + 20|0);
   HEAP32[$2252>>2] = $2253;
   $2254 = HEAP32[$pParse>>2]|0;
   $2255 = HEAP32[$yymsp>>2]|0;
   $2256 = ((($2255)) + -64|0);
   $2257 = ((($2256)) + 4|0);
   $2258 = HEAP32[$2257>>2]|0;
   _sqlite3ExprSetHeightAndFlags($2254,$2258);
   break;
  }
  case 197:  {
   $2271 = HEAP32[$pParse>>2]|0;
   $2272 = HEAP32[$yymsp>>2]|0;
   $2273 = ((($2272)) + -64|0);
   $2274 = ((($2273)) + 4|0);
   $2275 = HEAP32[$2274>>2]|0;
   $2276 = HEAP32[$yymsp>>2]|0;
   $2277 = ((($2276)) + -32|0);
   $2278 = ((($2277)) + 4|0);
   $2279 = HEAP32[$2278>>2]|0;
   $2280 = (_sqlite3ExprListAppend($2271,$2275,$2279)|0);
   $2281 = HEAP32[$yymsp>>2]|0;
   $2282 = ((($2281)) + -64|0);
   $2283 = ((($2282)) + 4|0);
   HEAP32[$2283>>2] = $2280;
   $2284 = HEAP32[$pParse>>2]|0;
   $2285 = HEAP32[$yymsp>>2]|0;
   $2286 = ((($2285)) + -64|0);
   $2287 = ((($2286)) + 4|0);
   $2288 = HEAP32[$2287>>2]|0;
   $2289 = HEAP32[$yymsp>>2]|0;
   $2290 = ((($2289)) + 4|0);
   $2291 = HEAP32[$2290>>2]|0;
   $2292 = (_sqlite3ExprListAppend($2284,$2288,$2291)|0);
   $2293 = HEAP32[$yymsp>>2]|0;
   $2294 = ((($2293)) + -64|0);
   $2295 = ((($2294)) + 4|0);
   HEAP32[$2295>>2] = $2292;
   break;
  }
  case 198:  {
   $2296 = HEAP32[$pParse>>2]|0;
   $2297 = HEAP32[$yymsp>>2]|0;
   $2298 = ((($2297)) + -32|0);
   $2299 = ((($2298)) + 4|0);
   $2300 = HEAP32[$2299>>2]|0;
   $2301 = (_sqlite3ExprListAppend($2296,0,$2300)|0);
   $2302 = HEAP32[$yymsp>>2]|0;
   $2303 = ((($2302)) + -48|0);
   $2304 = ((($2303)) + 4|0);
   HEAP32[$2304>>2] = $2301;
   $2305 = HEAP32[$pParse>>2]|0;
   $2306 = HEAP32[$yymsp>>2]|0;
   $2307 = ((($2306)) + -48|0);
   $2308 = ((($2307)) + 4|0);
   $2309 = HEAP32[$2308>>2]|0;
   $2310 = HEAP32[$yymsp>>2]|0;
   $2311 = ((($2310)) + 4|0);
   $2312 = HEAP32[$2311>>2]|0;
   $2313 = (_sqlite3ExprListAppend($2305,$2309,$2312)|0);
   $2314 = HEAP32[$yymsp>>2]|0;
   $2315 = ((($2314)) + -48|0);
   $2316 = ((($2315)) + 4|0);
   HEAP32[$2316>>2] = $2313;
   break;
  }
  case 201:  {
   $2317 = HEAP32[$yymsp>>2]|0;
   $2318 = ((($2317)) + 4|0);
   $2319 = HEAP32[$2318>>2]|0;
   $2320 = HEAP32[$yymsp>>2]|0;
   $2321 = ((($2320)) + 4|0);
   HEAP32[$2321>>2] = $2319;
   break;
  }
  case 204:  {
   $2322 = HEAP32[$pParse>>2]|0;
   $2323 = HEAP32[$yymsp>>2]|0;
   $2324 = ((($2323)) + -32|0);
   $2325 = ((($2324)) + 4|0);
   $2326 = HEAP32[$2325>>2]|0;
   $2327 = HEAP32[$yymsp>>2]|0;
   $2328 = ((($2327)) + 4|0);
   $2329 = HEAP32[$2328>>2]|0;
   $2330 = (_sqlite3ExprListAppend($2322,$2326,$2329)|0);
   $2331 = HEAP32[$yymsp>>2]|0;
   $2332 = ((($2331)) + -32|0);
   $2333 = ((($2332)) + 4|0);
   HEAP32[$2333>>2] = $2330;
   break;
  }
  case 205:  {
   $2334 = HEAP32[$pParse>>2]|0;
   $2335 = HEAP32[$yymsp>>2]|0;
   $2336 = ((($2335)) + 4|0);
   $2337 = HEAP32[$2336>>2]|0;
   $2338 = (_sqlite3ExprListAppend($2334,0,$2337)|0);
   $2339 = HEAP32[$yymsp>>2]|0;
   $2340 = ((($2339)) + 4|0);
   HEAP32[$2340>>2] = $2338;
   break;
  }
  case 212: case 207:  {
   $2341 = HEAP32[$yymsp>>2]|0;
   $2342 = ((($2341)) + -16|0);
   $2343 = ((($2342)) + 4|0);
   $2344 = HEAP32[$2343>>2]|0;
   $2345 = HEAP32[$yymsp>>2]|0;
   $2346 = ((($2345)) + -32|0);
   $2347 = ((($2346)) + 4|0);
   HEAP32[$2347>>2] = $2344;
   break;
  }
  case 208:  {
   $2348 = HEAP32[$pParse>>2]|0;
   $2349 = HEAP32[$yymsp>>2]|0;
   $2350 = ((($2349)) + -112|0);
   $2351 = ((($2350)) + 4|0);
   $2352 = HEAP32[$yymsp>>2]|0;
   $2353 = ((($2352)) + -96|0);
   $2354 = ((($2353)) + 4|0);
   $2355 = HEAP32[$pParse>>2]|0;
   $2356 = HEAP32[$2355>>2]|0;
   $2357 = HEAP32[$yymsp>>2]|0;
   $2358 = ((($2357)) + -64|0);
   $2359 = ((($2358)) + 4|0);
   $2360 = (_sqlite3SrcListAppend($2356,0,$2359,0)|0);
   $2361 = HEAP32[$yymsp>>2]|0;
   $2362 = ((($2361)) + -32|0);
   $2363 = ((($2362)) + 4|0);
   $2364 = HEAP32[$2363>>2]|0;
   $2365 = HEAP32[$yymsp>>2]|0;
   $2366 = ((($2365)) + -160|0);
   $2367 = ((($2366)) + 4|0);
   $2368 = HEAP32[$2367>>2]|0;
   $2369 = HEAP32[$yymsp>>2]|0;
   $2370 = ((($2369)) + -176|0);
   $2371 = ((($2370)) + 4|0);
   $2372 = HEAP32[$yymsp>>2]|0;
   $2373 = ((($2372)) + 4|0);
   $2374 = HEAP32[$2373>>2]|0;
   $2375 = HEAP32[$yymsp>>2]|0;
   $2376 = ((($2375)) + -128|0);
   $2377 = ((($2376)) + 4|0);
   $2378 = HEAP32[$2377>>2]|0;
   _sqlite3CreateIndex($2348,$2351,$2354,$2360,$2364,$2368,$2371,$2374,0,$2378,0);
   break;
  }
  case 250: case 209:  {
   $2379 = HEAP32[$yymsp>>2]|0;
   $2380 = ((($2379)) + 4|0);
   HEAP32[$2380>>2] = 2;
   break;
  }
  case 210:  {
   $2381 = HEAP32[$yymsp>>2]|0;
   $2382 = ((($2381)) + 16|0);
   $2383 = ((($2382)) + 4|0);
   HEAP32[$2383>>2] = 0;
   break;
  }
  case 213:  {
   $2384 = HEAP32[$pParse>>2]|0;
   $2385 = HEAP32[$yymsp>>2]|0;
   $2386 = ((($2385)) + -64|0);
   $2387 = ((($2386)) + 4|0);
   $2388 = HEAP32[$2387>>2]|0;
   $2389 = HEAP32[$yymsp>>2]|0;
   $2390 = ((($2389)) + -32|0);
   $2391 = ((($2390)) + 4|0);
   $2392 = HEAP32[$yymsp>>2]|0;
   $2393 = ((($2392)) + -16|0);
   $2394 = ((($2393)) + 4|0);
   $2395 = HEAP32[$2394>>2]|0;
   $2396 = HEAP32[$yymsp>>2]|0;
   $2397 = ((($2396)) + 4|0);
   $2398 = HEAP32[$2397>>2]|0;
   $2399 = (_parserAddExprIdListTerm($2384,$2388,$2391,$2395,$2398)|0);
   $2400 = HEAP32[$yymsp>>2]|0;
   $2401 = ((($2400)) + -64|0);
   $2402 = ((($2401)) + 4|0);
   HEAP32[$2402>>2] = $2399;
   break;
  }
  case 214:  {
   $2403 = HEAP32[$pParse>>2]|0;
   $2404 = HEAP32[$yymsp>>2]|0;
   $2405 = ((($2404)) + -32|0);
   $2406 = ((($2405)) + 4|0);
   $2407 = HEAP32[$yymsp>>2]|0;
   $2408 = ((($2407)) + -16|0);
   $2409 = ((($2408)) + 4|0);
   $2410 = HEAP32[$2409>>2]|0;
   $2411 = HEAP32[$yymsp>>2]|0;
   $2412 = ((($2411)) + 4|0);
   $2413 = HEAP32[$2412>>2]|0;
   $2414 = (_parserAddExprIdListTerm($2403,0,$2406,$2410,$2413)|0);
   $2415 = HEAP32[$yymsp>>2]|0;
   $2416 = ((($2415)) + -32|0);
   $2417 = ((($2416)) + 4|0);
   HEAP32[$2417>>2] = $2414;
   break;
  }
  case 217:  {
   $2418 = HEAP32[$pParse>>2]|0;
   $2419 = HEAP32[$yymsp>>2]|0;
   $2420 = ((($2419)) + 4|0);
   $2421 = HEAP32[$2420>>2]|0;
   $2422 = HEAP32[$yymsp>>2]|0;
   $2423 = ((($2422)) + -16|0);
   $2424 = ((($2423)) + 4|0);
   $2425 = HEAP32[$2424>>2]|0;
   _sqlite3DropIndex($2418,$2421,$2425);
   break;
  }
  case 218:  {
   $2426 = HEAP32[$pParse>>2]|0;
   _sqlite3Vacuum($2426,0);
   break;
  }
  case 219:  {
   $2427 = HEAP32[$pParse>>2]|0;
   $2428 = HEAP32[$yymsp>>2]|0;
   $2429 = ((($2428)) + 4|0);
   _sqlite3Vacuum($2427,$2429);
   break;
  }
  case 220:  {
   $2430 = HEAP32[$pParse>>2]|0;
   $2431 = HEAP32[$yymsp>>2]|0;
   $2432 = ((($2431)) + -16|0);
   $2433 = ((($2432)) + 4|0);
   $2434 = HEAP32[$yymsp>>2]|0;
   $2435 = ((($2434)) + 4|0);
   _sqlite3Pragma($2430,$2433,$2435,0,0);
   break;
  }
  case 221:  {
   $2436 = HEAP32[$pParse>>2]|0;
   $2437 = HEAP32[$yymsp>>2]|0;
   $2438 = ((($2437)) + -48|0);
   $2439 = ((($2438)) + 4|0);
   $2440 = HEAP32[$yymsp>>2]|0;
   $2441 = ((($2440)) + -32|0);
   $2442 = ((($2441)) + 4|0);
   $2443 = HEAP32[$yymsp>>2]|0;
   $2444 = ((($2443)) + 4|0);
   _sqlite3Pragma($2436,$2439,$2442,$2444,0);
   break;
  }
  case 222:  {
   $2445 = HEAP32[$pParse>>2]|0;
   $2446 = HEAP32[$yymsp>>2]|0;
   $2447 = ((($2446)) + -64|0);
   $2448 = ((($2447)) + 4|0);
   $2449 = HEAP32[$yymsp>>2]|0;
   $2450 = ((($2449)) + -48|0);
   $2451 = ((($2450)) + 4|0);
   $2452 = HEAP32[$yymsp>>2]|0;
   $2453 = ((($2452)) + -16|0);
   $2454 = ((($2453)) + 4|0);
   _sqlite3Pragma($2445,$2448,$2451,$2454,0);
   break;
  }
  case 223:  {
   $2455 = HEAP32[$pParse>>2]|0;
   $2456 = HEAP32[$yymsp>>2]|0;
   $2457 = ((($2456)) + -48|0);
   $2458 = ((($2457)) + 4|0);
   $2459 = HEAP32[$yymsp>>2]|0;
   $2460 = ((($2459)) + -32|0);
   $2461 = ((($2460)) + 4|0);
   $2462 = HEAP32[$yymsp>>2]|0;
   $2463 = ((($2462)) + 4|0);
   _sqlite3Pragma($2455,$2458,$2461,$2463,1);
   break;
  }
  case 224:  {
   $2464 = HEAP32[$pParse>>2]|0;
   $2465 = HEAP32[$yymsp>>2]|0;
   $2466 = ((($2465)) + -64|0);
   $2467 = ((($2466)) + 4|0);
   $2468 = HEAP32[$yymsp>>2]|0;
   $2469 = ((($2468)) + -48|0);
   $2470 = ((($2469)) + 4|0);
   $2471 = HEAP32[$yymsp>>2]|0;
   $2472 = ((($2471)) + -16|0);
   $2473 = ((($2472)) + 4|0);
   _sqlite3Pragma($2464,$2467,$2470,$2473,1);
   break;
  }
  case 227:  {
   $2474 = HEAP32[$yymsp>>2]|0;
   $2475 = ((($2474)) + -48|0);
   $2476 = ((($2475)) + 4|0);
   $2477 = HEAP32[$2476>>2]|0;
   HEAP32[$all>>2] = $2477;
   $2478 = HEAP32[$yymsp>>2]|0;
   $2479 = ((($2478)) + 4|0);
   $2480 = HEAP32[$2479>>2]|0;
   $2481 = HEAP32[$yymsp>>2]|0;
   $2482 = ((($2481)) + -48|0);
   $2483 = ((($2482)) + 4|0);
   $2484 = HEAP32[$2483>>2]|0;
   $2485 = $2480;
   $2486 = $2484;
   $2487 = (($2485) - ($2486))|0;
   $2488 = HEAP32[$yymsp>>2]|0;
   $2489 = ((($2488)) + 4|0);
   $2490 = ((($2489)) + 4|0);
   $2491 = HEAP32[$2490>>2]|0;
   $2492 = (($2487) + ($2491))|0;
   $2493 = ((($all)) + 4|0);
   HEAP32[$2493>>2] = $2492;
   $2494 = HEAP32[$pParse>>2]|0;
   $2495 = HEAP32[$yymsp>>2]|0;
   $2496 = ((($2495)) + -16|0);
   $2497 = ((($2496)) + 4|0);
   $2498 = HEAP32[$2497>>2]|0;
   _sqlite3FinishTrigger($2494,$2498,$all);
   break;
  }
  case 228:  {
   $2499 = HEAP32[$pParse>>2]|0;
   $2500 = HEAP32[$yymsp>>2]|0;
   $2501 = ((($2500)) + -112|0);
   $2502 = ((($2501)) + 4|0);
   $2503 = HEAP32[$yymsp>>2]|0;
   $2504 = ((($2503)) + -96|0);
   $2505 = ((($2504)) + 4|0);
   $2506 = HEAP32[$yymsp>>2]|0;
   $2507 = ((($2506)) + -80|0);
   $2508 = ((($2507)) + 4|0);
   $2509 = HEAP32[$2508>>2]|0;
   $2510 = HEAP32[$yymsp>>2]|0;
   $2511 = ((($2510)) + -64|0);
   $2512 = ((($2511)) + 4|0);
   $2513 = HEAP32[$2512>>2]|0;
   $2514 = HEAP32[$yymsp>>2]|0;
   $2515 = ((($2514)) + -64|0);
   $2516 = ((($2515)) + 4|0);
   $2517 = ((($2516)) + 4|0);
   $2518 = HEAP32[$2517>>2]|0;
   $2519 = HEAP32[$yymsp>>2]|0;
   $2520 = ((($2519)) + -32|0);
   $2521 = ((($2520)) + 4|0);
   $2522 = HEAP32[$2521>>2]|0;
   $2523 = HEAP32[$yymsp>>2]|0;
   $2524 = ((($2523)) + 4|0);
   $2525 = HEAP32[$2524>>2]|0;
   $2526 = HEAP32[$yymsp>>2]|0;
   $2527 = ((($2526)) + -160|0);
   $2528 = ((($2527)) + 4|0);
   $2529 = HEAP32[$2528>>2]|0;
   $2530 = HEAP32[$yymsp>>2]|0;
   $2531 = ((($2530)) + -128|0);
   $2532 = ((($2531)) + 4|0);
   $2533 = HEAP32[$2532>>2]|0;
   _sqlite3BeginTrigger($2499,$2502,$2505,$2509,$2513,$2518,$2522,$2525,$2529,$2533);
   $2534 = HEAP32[$yymsp>>2]|0;
   $2535 = ((($2534)) + -160|0);
   $2536 = ((($2535)) + 4|0);
   $2537 = HEAP32[$yymsp>>2]|0;
   $2538 = ((($2537)) + -96|0);
   $2539 = ((($2538)) + 4|0);
   $2540 = ((($2539)) + 4|0);
   $2541 = HEAP32[$2540>>2]|0;
   $2542 = ($2541|0)==(0);
   $2543 = HEAP32[$yymsp>>2]|0;
   if ($2542) {
    $2544 = ((($2543)) + -112|0);
    $2545 = ((($2544)) + 4|0);
    ;HEAP32[$2536>>2]=HEAP32[$2545>>2]|0;HEAP32[$2536+4>>2]=HEAP32[$2545+4>>2]|0;
    break L6;
   } else {
    $2546 = ((($2543)) + -96|0);
    $2547 = ((($2546)) + 4|0);
    ;HEAP32[$2536>>2]=HEAP32[$2547>>2]|0;HEAP32[$2536+4>>2]=HEAP32[$2547+4>>2]|0;
    break L6;
   }
   break;
  }
  case 229:  {
   $2548 = HEAP32[$yymsp>>2]|0;
   $2549 = ((($2548)) + 4|0);
   HEAP32[$2549>>2] = 63;
   break;
  }
  case 230:  {
   $2550 = HEAP32[$yymsp>>2]|0;
   $2551 = ((($2550)) + 4|0);
   HEAP32[$2551>>2] = 59;
   break;
  }
  case 231:  {
   $2552 = HEAP32[$yymsp>>2]|0;
   $2553 = ((($2552)) + -16|0);
   $2554 = ((($2553)) + 4|0);
   HEAP32[$2554>>2] = 77;
   break;
  }
  case 232:  {
   $2555 = HEAP32[$yymsp>>2]|0;
   $2556 = ((($2555)) + 16|0);
   $2557 = ((($2556)) + 4|0);
   HEAP32[$2557>>2] = 63;
   break;
  }
  case 234: case 233:  {
   $2558 = HEAP32[$yymsp>>2]|0;
   $2559 = ((($2558)) + 2|0);
   $2560 = HEAP8[$2559>>0]|0;
   $2561 = $2560&255;
   $2562 = HEAP32[$yymsp>>2]|0;
   $2563 = ((($2562)) + 4|0);
   HEAP32[$2563>>2] = $2561;
   $2564 = HEAP32[$yymsp>>2]|0;
   $2565 = ((($2564)) + 4|0);
   $2566 = ((($2565)) + 4|0);
   HEAP32[$2566>>2] = 0;
   break;
  }
  case 235:  {
   $2567 = HEAP32[$yymsp>>2]|0;
   $2568 = ((($2567)) + -32|0);
   $2569 = ((($2568)) + 4|0);
   HEAP32[$2569>>2] = 110;
   $2570 = HEAP32[$yymsp>>2]|0;
   $2571 = ((($2570)) + 4|0);
   $2572 = HEAP32[$2571>>2]|0;
   $2573 = HEAP32[$yymsp>>2]|0;
   $2574 = ((($2573)) + -32|0);
   $2575 = ((($2574)) + 4|0);
   $2576 = ((($2575)) + 4|0);
   HEAP32[$2576>>2] = $2572;
   break;
  }
  case 255: case 236:  {
   $2577 = HEAP32[$yymsp>>2]|0;
   $2578 = ((($2577)) + 16|0);
   $2579 = ((($2578)) + 4|0);
   HEAP32[$2579>>2] = 0;
   break;
  }
  case 256: case 237:  {
   $2580 = HEAP32[$yymsp>>2]|0;
   $2581 = ((($2580)) + 4|0);
   $2582 = HEAP32[$2581>>2]|0;
   $2583 = HEAP32[$yymsp>>2]|0;
   $2584 = ((($2583)) + -16|0);
   $2585 = ((($2584)) + 4|0);
   HEAP32[$2585>>2] = $2582;
   break;
  }
  case 238:  {
   $2586 = HEAP32[$yymsp>>2]|0;
   $2587 = ((($2586)) + -16|0);
   $2588 = ((($2587)) + 4|0);
   $2589 = HEAP32[$2588>>2]|0;
   $2590 = HEAP32[$yymsp>>2]|0;
   $2591 = ((($2590)) + -32|0);
   $2592 = ((($2591)) + 4|0);
   $2593 = HEAP32[$2592>>2]|0;
   $2594 = ((($2593)) + 32|0);
   $2595 = HEAP32[$2594>>2]|0;
   $2596 = ((($2595)) + 28|0);
   HEAP32[$2596>>2] = $2589;
   $2597 = HEAP32[$yymsp>>2]|0;
   $2598 = ((($2597)) + -16|0);
   $2599 = ((($2598)) + 4|0);
   $2600 = HEAP32[$2599>>2]|0;
   $2601 = HEAP32[$yymsp>>2]|0;
   $2602 = ((($2601)) + -32|0);
   $2603 = ((($2602)) + 4|0);
   $2604 = HEAP32[$2603>>2]|0;
   $2605 = ((($2604)) + 32|0);
   HEAP32[$2605>>2] = $2600;
   break;
  }
  case 239:  {
   $2606 = HEAP32[$yymsp>>2]|0;
   $2607 = ((($2606)) + -16|0);
   $2608 = ((($2607)) + 4|0);
   $2609 = HEAP32[$2608>>2]|0;
   $2610 = HEAP32[$yymsp>>2]|0;
   $2611 = ((($2610)) + -16|0);
   $2612 = ((($2611)) + 4|0);
   $2613 = HEAP32[$2612>>2]|0;
   $2614 = ((($2613)) + 32|0);
   HEAP32[$2614>>2] = $2609;
   break;
  }
  case 240:  {
   $2615 = HEAP32[$yymsp>>2]|0;
   $2616 = ((($2615)) + -32|0);
   $2617 = ((($2616)) + 4|0);
   $2618 = HEAP32[$yymsp>>2]|0;
   $2619 = ((($2618)) + 4|0);
   ;HEAP32[$2617>>2]=HEAP32[$2619>>2]|0;HEAP32[$2617+4>>2]=HEAP32[$2619+4>>2]|0;
   $2620 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2620,27220,$vararg_buffer8);
   break;
  }
  case 241:  {
   $2621 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2621,27315,$vararg_buffer10);
   break;
  }
  case 242:  {
   $2622 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($2622,27399,$vararg_buffer12);
   break;
  }
  case 243:  {
   $2623 = HEAP32[$pParse>>2]|0;
   $2624 = HEAP32[$2623>>2]|0;
   $2625 = HEAP32[$yymsp>>2]|0;
   $2626 = ((($2625)) + -64|0);
   $2627 = ((($2626)) + 4|0);
   $2628 = HEAP32[$yymsp>>2]|0;
   $2629 = ((($2628)) + -16|0);
   $2630 = ((($2629)) + 4|0);
   $2631 = HEAP32[$2630>>2]|0;
   $2632 = HEAP32[$yymsp>>2]|0;
   $2633 = ((($2632)) + 4|0);
   $2634 = HEAP32[$2633>>2]|0;
   $2635 = HEAP32[$yymsp>>2]|0;
   $2636 = ((($2635)) + -80|0);
   $2637 = ((($2636)) + 4|0);
   $2638 = HEAP32[$2637>>2]|0;
   $2639 = $2638&255;
   $2640 = (_sqlite3TriggerUpdateStep($2624,$2627,$2631,$2634,$2639)|0);
   $2641 = HEAP32[$yymsp>>2]|0;
   $2642 = ((($2641)) + -96|0);
   $2643 = ((($2642)) + 4|0);
   HEAP32[$2643>>2] = $2640;
   break;
  }
  case 244:  {
   $2644 = HEAP32[$pParse>>2]|0;
   $2645 = HEAP32[$2644>>2]|0;
   $2646 = HEAP32[$yymsp>>2]|0;
   $2647 = ((($2646)) + -32|0);
   $2648 = ((($2647)) + 4|0);
   $2649 = HEAP32[$yymsp>>2]|0;
   $2650 = ((($2649)) + -16|0);
   $2651 = ((($2650)) + 4|0);
   $2652 = HEAP32[$2651>>2]|0;
   $2653 = HEAP32[$yymsp>>2]|0;
   $2654 = ((($2653)) + 4|0);
   $2655 = HEAP32[$2654>>2]|0;
   $2656 = HEAP32[$yymsp>>2]|0;
   $2657 = ((($2656)) + -64|0);
   $2658 = ((($2657)) + 4|0);
   $2659 = HEAP32[$2658>>2]|0;
   $2660 = $2659&255;
   $2661 = (_sqlite3TriggerInsertStep($2645,$2648,$2652,$2655,$2660)|0);
   $2662 = HEAP32[$yymsp>>2]|0;
   $2663 = ((($2662)) + -64|0);
   $2664 = ((($2663)) + 4|0);
   HEAP32[$2664>>2] = $2661;
   break;
  }
  case 245:  {
   $2665 = HEAP32[$pParse>>2]|0;
   $2666 = HEAP32[$2665>>2]|0;
   $2667 = HEAP32[$yymsp>>2]|0;
   $2668 = ((($2667)) + -32|0);
   $2669 = ((($2668)) + 4|0);
   $2670 = HEAP32[$yymsp>>2]|0;
   $2671 = ((($2670)) + 4|0);
   $2672 = HEAP32[$2671>>2]|0;
   $2673 = (_sqlite3TriggerDeleteStep($2666,$2669,$2672)|0);
   $2674 = HEAP32[$yymsp>>2]|0;
   $2675 = ((($2674)) + -64|0);
   $2676 = ((($2675)) + 4|0);
   HEAP32[$2676>>2] = $2673;
   break;
  }
  case 246:  {
   $2677 = HEAP32[$pParse>>2]|0;
   $2678 = HEAP32[$2677>>2]|0;
   $2679 = HEAP32[$yymsp>>2]|0;
   $2680 = ((($2679)) + 4|0);
   $2681 = HEAP32[$2680>>2]|0;
   $2682 = (_sqlite3TriggerSelectStep($2678,$2681)|0);
   $2683 = HEAP32[$yymsp>>2]|0;
   $2684 = ((($2683)) + 4|0);
   HEAP32[$2684>>2] = $2682;
   break;
  }
  case 247:  {
   $2685 = HEAP32[$yymsp>>2]|0;
   $2686 = ((($2685)) + -48|0);
   $2687 = ((($2686)) + 4|0);
   $2688 = HEAP32[$yymsp>>2]|0;
   $2689 = ((($2688)) + -48|0);
   $2690 = ((($2689)) + 4|0);
   $2691 = HEAP32[$yymsp>>2]|0;
   $2692 = ((($2691)) + 4|0);
   _spanSet($2687,$2690,$2692);
   $2693 = HEAP32[$pParse>>2]|0;
   $2694 = (_sqlite3PExpr($2693,83,0,0,0)|0);
   $2695 = HEAP32[$yymsp>>2]|0;
   $2696 = ((($2695)) + -48|0);
   $2697 = ((($2696)) + 4|0);
   HEAP32[$2697>>2] = $2694;
   $2698 = HEAP32[$yymsp>>2]|0;
   $2699 = ((($2698)) + -48|0);
   $2700 = ((($2699)) + 4|0);
   $2701 = HEAP32[$2700>>2]|0;
   $2702 = ($2701|0)!=(0|0);
   if ($2702) {
    $2703 = HEAP32[$yymsp>>2]|0;
    $2704 = ((($2703)) + -48|0);
    $2705 = ((($2704)) + 4|0);
    $2706 = HEAP32[$2705>>2]|0;
    $2707 = ((($2706)) + 1|0);
    HEAP8[$2707>>0] = 4;
   }
   break;
  }
  case 248:  {
   $2708 = HEAP32[$yymsp>>2]|0;
   $2709 = ((($2708)) + -80|0);
   $2710 = ((($2709)) + 4|0);
   $2711 = HEAP32[$yymsp>>2]|0;
   $2712 = ((($2711)) + -80|0);
   $2713 = ((($2712)) + 4|0);
   $2714 = HEAP32[$yymsp>>2]|0;
   $2715 = ((($2714)) + 4|0);
   _spanSet($2710,$2713,$2715);
   $2716 = HEAP32[$pParse>>2]|0;
   $2717 = HEAP32[$yymsp>>2]|0;
   $2718 = ((($2717)) + -16|0);
   $2719 = ((($2718)) + 4|0);
   $2720 = (_sqlite3PExpr($2716,83,0,0,$2719)|0);
   $2721 = HEAP32[$yymsp>>2]|0;
   $2722 = ((($2721)) + -80|0);
   $2723 = ((($2722)) + 4|0);
   HEAP32[$2723>>2] = $2720;
   $2724 = HEAP32[$yymsp>>2]|0;
   $2725 = ((($2724)) + -80|0);
   $2726 = ((($2725)) + 4|0);
   $2727 = HEAP32[$2726>>2]|0;
   $2728 = ($2727|0)!=(0|0);
   if ($2728) {
    $2729 = HEAP32[$yymsp>>2]|0;
    $2730 = ((($2729)) + -48|0);
    $2731 = ((($2730)) + 4|0);
    $2732 = HEAP32[$2731>>2]|0;
    $2733 = $2732&255;
    $2734 = HEAP32[$yymsp>>2]|0;
    $2735 = ((($2734)) + -80|0);
    $2736 = ((($2735)) + 4|0);
    $2737 = HEAP32[$2736>>2]|0;
    $2738 = ((($2737)) + 1|0);
    HEAP8[$2738>>0] = $2733;
   }
   break;
  }
  case 249:  {
   $2739 = HEAP32[$yymsp>>2]|0;
   $2740 = ((($2739)) + 4|0);
   HEAP32[$2740>>2] = 1;
   break;
  }
  case 251:  {
   $2741 = HEAP32[$yymsp>>2]|0;
   $2742 = ((($2741)) + 4|0);
   HEAP32[$2742>>2] = 3;
   break;
  }
  case 252:  {
   $2743 = HEAP32[$pParse>>2]|0;
   $2744 = HEAP32[$yymsp>>2]|0;
   $2745 = ((($2744)) + 4|0);
   $2746 = HEAP32[$2745>>2]|0;
   $2747 = HEAP32[$yymsp>>2]|0;
   $2748 = ((($2747)) + -16|0);
   $2749 = ((($2748)) + 4|0);
   $2750 = HEAP32[$2749>>2]|0;
   _sqlite3DropTrigger($2743,$2746,$2750);
   break;
  }
  case 253:  {
   $2751 = HEAP32[$pParse>>2]|0;
   $2752 = HEAP32[$yymsp>>2]|0;
   $2753 = ((($2752)) + -48|0);
   $2754 = ((($2753)) + 4|0);
   $2755 = HEAP32[$2754>>2]|0;
   $2756 = HEAP32[$yymsp>>2]|0;
   $2757 = ((($2756)) + -16|0);
   $2758 = ((($2757)) + 4|0);
   $2759 = HEAP32[$2758>>2]|0;
   $2760 = HEAP32[$yymsp>>2]|0;
   $2761 = ((($2760)) + 4|0);
   $2762 = HEAP32[$2761>>2]|0;
   _sqlite3Attach($2751,$2755,$2759,$2762);
   break;
  }
  case 254:  {
   $2763 = HEAP32[$pParse>>2]|0;
   $2764 = HEAP32[$yymsp>>2]|0;
   $2765 = ((($2764)) + 4|0);
   $2766 = HEAP32[$2765>>2]|0;
   _sqlite3Detach($2763,$2766);
   break;
  }
  case 257:  {
   $2767 = HEAP32[$pParse>>2]|0;
   _sqlite3Reindex($2767,0,0);
   break;
  }
  case 258:  {
   $2768 = HEAP32[$pParse>>2]|0;
   $2769 = HEAP32[$yymsp>>2]|0;
   $2770 = ((($2769)) + -16|0);
   $2771 = ((($2770)) + 4|0);
   $2772 = HEAP32[$yymsp>>2]|0;
   $2773 = ((($2772)) + 4|0);
   _sqlite3Reindex($2768,$2771,$2773);
   break;
  }
  case 259:  {
   $2774 = HEAP32[$pParse>>2]|0;
   _sqlite3Analyze($2774,0,0);
   break;
  }
  case 260:  {
   $2775 = HEAP32[$pParse>>2]|0;
   $2776 = HEAP32[$yymsp>>2]|0;
   $2777 = ((($2776)) + -16|0);
   $2778 = ((($2777)) + 4|0);
   $2779 = HEAP32[$yymsp>>2]|0;
   $2780 = ((($2779)) + 4|0);
   _sqlite3Analyze($2775,$2778,$2780);
   break;
  }
  case 261:  {
   $2781 = HEAP32[$pParse>>2]|0;
   $2782 = HEAP32[$yymsp>>2]|0;
   $2783 = ((($2782)) + -48|0);
   $2784 = ((($2783)) + 4|0);
   $2785 = HEAP32[$2784>>2]|0;
   $2786 = HEAP32[$yymsp>>2]|0;
   $2787 = ((($2786)) + 4|0);
   _sqlite3AlterRenameTable($2781,$2785,$2787);
   break;
  }
  case 262:  {
   $2788 = HEAP32[$pParse>>2]|0;
   $2789 = ((($2788)) + 392|0);
   $2790 = HEAP32[$2789>>2]|0;
   $2791 = HEAP32[$yymsp>>2]|0;
   $2792 = ((($2791)) + -16|0);
   $2793 = ((($2792)) + 4|0);
   $2794 = HEAP32[$2793>>2]|0;
   $2795 = $2790;
   $2796 = $2794;
   $2797 = (($2795) - ($2796))|0;
   $2798 = HEAP32[$pParse>>2]|0;
   $2799 = ((($2798)) + 392|0);
   $2800 = ((($2799)) + 4|0);
   $2801 = HEAP32[$2800>>2]|0;
   $2802 = (($2797) + ($2801))|0;
   $2803 = HEAP32[$yymsp>>2]|0;
   $2804 = ((($2803)) + -16|0);
   $2805 = ((($2804)) + 4|0);
   $2806 = ((($2805)) + 4|0);
   HEAP32[$2806>>2] = $2802;
   $2807 = HEAP32[$pParse>>2]|0;
   $2808 = HEAP32[$yymsp>>2]|0;
   $2809 = ((($2808)) + -16|0);
   $2810 = ((($2809)) + 4|0);
   _sqlite3AlterFinishAddColumn($2807,$2810);
   break;
  }
  case 263:  {
   $2811 = HEAP32[$pParse>>2]|0;
   _disableLookaside($2811);
   $2812 = HEAP32[$pParse>>2]|0;
   $2813 = HEAP32[$yymsp>>2]|0;
   $2814 = ((($2813)) + 4|0);
   $2815 = HEAP32[$2814>>2]|0;
   _sqlite3AlterBeginAddColumn($2812,$2815);
   break;
  }
  case 264:  {
   $2816 = HEAP32[$pParse>>2]|0;
   _sqlite3VtabFinishParse($2816,0);
   break;
  }
  case 265:  {
   $2817 = HEAP32[$pParse>>2]|0;
   $2818 = HEAP32[$yymsp>>2]|0;
   $2819 = ((($2818)) + 4|0);
   _sqlite3VtabFinishParse($2817,$2819);
   break;
  }
  case 266:  {
   $2820 = HEAP32[$pParse>>2]|0;
   $2821 = HEAP32[$yymsp>>2]|0;
   $2822 = ((($2821)) + -48|0);
   $2823 = ((($2822)) + 4|0);
   $2824 = HEAP32[$yymsp>>2]|0;
   $2825 = ((($2824)) + -32|0);
   $2826 = ((($2825)) + 4|0);
   $2827 = HEAP32[$yymsp>>2]|0;
   $2828 = ((($2827)) + 4|0);
   $2829 = HEAP32[$yymsp>>2]|0;
   $2830 = ((($2829)) + -64|0);
   $2831 = ((($2830)) + 4|0);
   $2832 = HEAP32[$2831>>2]|0;
   _sqlite3VtabBeginParse($2820,$2823,$2826,$2828,$2832);
   break;
  }
  case 267:  {
   $2833 = HEAP32[$pParse>>2]|0;
   _sqlite3VtabArgInit($2833);
   break;
  }
  case 270: case 269: case 268:  {
   $2834 = HEAP32[$pParse>>2]|0;
   $2835 = HEAP32[$yymsp>>2]|0;
   $2836 = ((($2835)) + 4|0);
   _sqlite3VtabArgExtend($2834,$2836);
   break;
  }
  case 271:  {
   $2837 = HEAP32[$yymsp>>2]|0;
   $2838 = ((($2837)) + 16|0);
   $2839 = ((($2838)) + 4|0);
   HEAP32[$2839>>2] = 0;
   break;
  }
  case 272:  {
   $2840 = HEAP32[$yymsp>>2]|0;
   $2841 = ((($2840)) + 4|0);
   $2842 = HEAP32[$2841>>2]|0;
   $2843 = HEAP32[$yymsp>>2]|0;
   $2844 = ((($2843)) + -16|0);
   $2845 = ((($2844)) + 4|0);
   HEAP32[$2845>>2] = $2842;
   break;
  }
  case 273:  {
   $2846 = HEAP32[$yymsp>>2]|0;
   $2847 = ((($2846)) + 4|0);
   $2848 = HEAP32[$2847>>2]|0;
   $2849 = HEAP32[$yymsp>>2]|0;
   $2850 = ((($2849)) + -32|0);
   $2851 = ((($2850)) + 4|0);
   HEAP32[$2851>>2] = $2848;
   break;
  }
  case 274:  {
   $2852 = HEAP32[$pParse>>2]|0;
   $2853 = HEAP32[$yymsp>>2]|0;
   $2854 = ((($2853)) + -80|0);
   $2855 = ((($2854)) + 4|0);
   $2856 = HEAP32[$yymsp>>2]|0;
   $2857 = ((($2856)) + -64|0);
   $2858 = ((($2857)) + 4|0);
   $2859 = HEAP32[$2858>>2]|0;
   $2860 = HEAP32[$yymsp>>2]|0;
   $2861 = ((($2860)) + -16|0);
   $2862 = ((($2861)) + 4|0);
   $2863 = HEAP32[$2862>>2]|0;
   $2864 = (_sqlite3WithAdd($2852,0,$2855,$2859,$2863)|0);
   $2865 = HEAP32[$yymsp>>2]|0;
   $2866 = ((($2865)) + -80|0);
   $2867 = ((($2866)) + 4|0);
   HEAP32[$2867>>2] = $2864;
   break;
  }
  case 275:  {
   $2868 = HEAP32[$pParse>>2]|0;
   $2869 = HEAP32[$yymsp>>2]|0;
   $2870 = ((($2869)) + -112|0);
   $2871 = ((($2870)) + 4|0);
   $2872 = HEAP32[$2871>>2]|0;
   $2873 = HEAP32[$yymsp>>2]|0;
   $2874 = ((($2873)) + -80|0);
   $2875 = ((($2874)) + 4|0);
   $2876 = HEAP32[$yymsp>>2]|0;
   $2877 = ((($2876)) + -64|0);
   $2878 = ((($2877)) + 4|0);
   $2879 = HEAP32[$2878>>2]|0;
   $2880 = HEAP32[$yymsp>>2]|0;
   $2881 = ((($2880)) + -16|0);
   $2882 = ((($2881)) + 4|0);
   $2883 = HEAP32[$2882>>2]|0;
   $2884 = (_sqlite3WithAdd($2868,$2872,$2875,$2879,$2883)|0);
   $2885 = HEAP32[$yymsp>>2]|0;
   $2886 = ((($2885)) + -112|0);
   $2887 = ((($2886)) + 4|0);
   HEAP32[$2887>>2] = $2884;
   break;
  }
  default: {
  }
  }
 } while(0);
 $2888 = HEAP32[$1>>2]|0;
 $2889 = (26486 + ($2888<<1)|0);
 $2890 = HEAP8[$2889>>0]|0;
 $2891 = $2890&255;
 HEAP32[$yygoto>>2] = $2891;
 $2892 = HEAP32[$1>>2]|0;
 $2893 = (26486 + ($2892<<1)|0);
 $2894 = ((($2893)) + 1|0);
 $2895 = HEAP8[$2894>>0]|0;
 $2896 = $2895&255;
 HEAP32[$yysize>>2] = $2896;
 $2897 = HEAP32[$yysize>>2]|0;
 $2898 = (0 - ($2897))|0;
 $2899 = HEAP32[$yymsp>>2]|0;
 $2900 = (($2899) + ($2898<<4)|0);
 $2901 = HEAP16[$2900>>1]|0;
 $2902 = $2901&65535;
 $2903 = HEAP32[$yygoto>>2]|0;
 $2904 = $2903&255;
 $2905 = (_yy_find_reduce_action($2902,$2904)|0);
 HEAP32[$yyact>>2] = $2905;
 $2906 = HEAP32[$yyact>>2]|0;
 $2907 = ($2906|0)<=(999);
 if (!($2907)) {
  $2926 = HEAP32[$yysize>>2]|0;
  $2927 = HEAP32[$0>>2]|0;
  $2928 = HEAP32[$2927>>2]|0;
  $2929 = (0 - ($2926))|0;
  $2930 = (($2928) + ($2929<<4)|0);
  HEAP32[$2927>>2] = $2930;
  $2931 = HEAP32[$0>>2]|0;
  _yy_accept($2931);
  STACKTOP = sp;return;
 }
 $2908 = HEAP32[$yyact>>2]|0;
 $2909 = ($2908|0)>(455);
 if ($2909) {
  $2910 = HEAP32[$yyact>>2]|0;
  $2911 = (($2910) + 332)|0;
  HEAP32[$yyact>>2] = $2911;
 }
 $2912 = HEAP32[$yysize>>2]|0;
 $2913 = (($2912) - 1)|0;
 $2914 = HEAP32[$yymsp>>2]|0;
 $2915 = (0 - ($2913))|0;
 $2916 = (($2914) + ($2915<<4)|0);
 HEAP32[$yymsp>>2] = $2916;
 $2917 = HEAP32[$yymsp>>2]|0;
 $2918 = HEAP32[$0>>2]|0;
 HEAP32[$2918>>2] = $2917;
 $2919 = HEAP32[$yyact>>2]|0;
 $2920 = $2919&65535;
 $2921 = HEAP32[$yymsp>>2]|0;
 HEAP16[$2921>>1] = $2920;
 $2922 = HEAP32[$yygoto>>2]|0;
 $2923 = $2922&255;
 $2924 = HEAP32[$yymsp>>2]|0;
 $2925 = ((($2924)) + 2|0);
 HEAP8[$2925>>0] = $2923;
 STACKTOP = sp;return;
}
function _yy_syntax_error($yypParser,$yymajor,$yyminor) {
 $yypParser = $yypParser|0;
 $yymajor = $yymajor|0;
 $yyminor = $yyminor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pParse = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 HEAP32[$0>>2] = $yypParser;
 HEAP32[$1>>2] = $yymajor;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 HEAP32[$vararg_buffer>>2] = $yyminor;
 _sqlite3ErrorMsg($5,26462,$vararg_buffer);
 $6 = HEAP32[$pParse>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 HEAP32[$8>>2] = $6;
 STACKTOP = sp;return;
}
function _yyStackOverflow($yypParser) {
 $yypParser = $yypParser|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, $vararg_buffer = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 8|0;
 $pParse = sp + 4|0;
 HEAP32[$0>>2] = $yypParser;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pParse>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + -16|0);
 HEAP32[$4>>2] = $6;
 while(1) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = ($8>>>0)>($10>>>0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$0>>2]|0;
  _yy_pop_parser_stack($12);
 }
 $13 = HEAP32[$pParse>>2]|0;
 _sqlite3ErrorMsg($13,37597,$vararg_buffer);
 $14 = HEAP32[$pParse>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 4|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _sqlite3FinishCoding($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $pEL = 0, $pSchema = 0, $v = 0, $vtab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $db = sp + 24|0;
 $v = sp + 20|0;
 $iDb = sp + 16|0;
 $i = sp + 12|0;
 $pSchema = sp + 8|0;
 $vtab = sp + 4|0;
 $pEL = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 18|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 69|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 36|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0);
  if (!($15)) {
   $22 = HEAP32[$0>>2]|0;
   $23 = (_sqlite3GetVdbe($22)|0);
   HEAP32[$v>>2] = $23;
   $24 = HEAP32[$v>>2]|0;
   $25 = ($24|0)!=(0|0);
   do {
    if ($25) {
     $26 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($26,75)|0);
     $27 = HEAP32[$db>>2]|0;
     $28 = ((($27)) + 69|0);
     $29 = HEAP8[$28>>0]|0;
     $30 = $29&255;
     $31 = ($30|0)==(0);
     if ($31) {
      $32 = HEAP32[$0>>2]|0;
      $33 = ((($32)) + 96|0);
      $34 = HEAP32[$33>>2]|0;
      $35 = ($34|0)!=(0);
      if (!($35)) {
       $36 = HEAP32[$0>>2]|0;
       $37 = ((($36)) + 80|0);
       $38 = HEAP32[$37>>2]|0;
       $39 = ($38|0)!=(0|0);
       if (!($39)) {
        break;
       }
      }
      $40 = HEAP32[$v>>2]|0;
      _sqlite3VdbeJumpHere($40,0);
      HEAP32[$iDb>>2] = 0;
      while(1) {
       $41 = HEAP32[$iDb>>2]|0;
       $42 = HEAP32[$db>>2]|0;
       $43 = ((($42)) + 20|0);
       $44 = HEAP32[$43>>2]|0;
       $45 = ($41|0)<($44|0);
       if (!($45)) {
        break;
       }
       $46 = HEAP32[$0>>2]|0;
       $47 = ((($46)) + 96|0);
       $48 = HEAP32[$47>>2]|0;
       $49 = HEAP32[$iDb>>2]|0;
       $50 = 1 << $49;
       $51 = $48 & $50;
       $52 = ($51|0)!=(0);
       $53 = $52&1;
       $54 = ($53|0)==(0);
       if (!($54)) {
        $55 = HEAP32[$v>>2]|0;
        $56 = HEAP32[$iDb>>2]|0;
        _sqlite3VdbeUsesBtree($55,$56);
        $57 = HEAP32[$iDb>>2]|0;
        $58 = HEAP32[$db>>2]|0;
        $59 = ((($58)) + 16|0);
        $60 = HEAP32[$59>>2]|0;
        $61 = (($60) + ($57<<4)|0);
        $62 = ((($61)) + 12|0);
        $63 = HEAP32[$62>>2]|0;
        HEAP32[$pSchema>>2] = $63;
        $64 = HEAP32[$v>>2]|0;
        $65 = HEAP32[$iDb>>2]|0;
        $66 = HEAP32[$0>>2]|0;
        $67 = ((($66)) + 92|0);
        $68 = HEAP32[$67>>2]|0;
        $69 = HEAP32[$iDb>>2]|0;
        $70 = 1 << $69;
        $71 = $68 & $70;
        $72 = ($71|0)!=(0);
        $73 = $72&1;
        $74 = HEAP32[$pSchema>>2]|0;
        $75 = HEAP32[$74>>2]|0;
        $76 = HEAP32[$pSchema>>2]|0;
        $77 = ((($76)) + 4|0);
        $78 = HEAP32[$77>>2]|0;
        (_sqlite3VdbeAddOp4Int($64,2,$65,$73,$75,$78)|0);
        $79 = HEAP32[$db>>2]|0;
        $80 = ((($79)) + 148|0);
        $81 = ((($80)) + 5|0);
        $82 = HEAP8[$81>>0]|0;
        $83 = $82&255;
        $84 = ($83|0)==(0);
        if ($84) {
         $85 = HEAP32[$v>>2]|0;
         _sqlite3VdbeChangeP5($85,1);
        }
       }
       $86 = HEAP32[$iDb>>2]|0;
       $87 = (($86) + 1)|0;
       HEAP32[$iDb>>2] = $87;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $88 = HEAP32[$i>>2]|0;
       $89 = HEAP32[$0>>2]|0;
       $90 = ((($89)) + 412|0);
       $91 = HEAP32[$90>>2]|0;
       $92 = ($88|0)<($91|0);
       if (!($92)) {
        break;
       }
       $93 = HEAP32[$db>>2]|0;
       $94 = HEAP32[$i>>2]|0;
       $95 = HEAP32[$0>>2]|0;
       $96 = ((($95)) + 460|0);
       $97 = HEAP32[$96>>2]|0;
       $98 = (($97) + ($94<<2)|0);
       $99 = HEAP32[$98>>2]|0;
       $100 = (_sqlite3GetVTable($93,$99)|0);
       HEAP32[$vtab>>2] = $100;
       $101 = HEAP32[$v>>2]|0;
       $102 = HEAP32[$vtab>>2]|0;
       (_sqlite3VdbeAddOp4($101,152,0,0,0,$102,-10)|0);
       $103 = HEAP32[$i>>2]|0;
       $104 = (($103) + 1)|0;
       HEAP32[$i>>2] = $104;
      }
      $105 = HEAP32[$0>>2]|0;
      $106 = ((($105)) + 412|0);
      HEAP32[$106>>2] = 0;
      $107 = HEAP32[$0>>2]|0;
      _codeTableLocks($107);
      $108 = HEAP32[$0>>2]|0;
      _sqlite3AutoincrementBegin($108);
      $109 = HEAP32[$0>>2]|0;
      $110 = ((($109)) + 80|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = ($111|0)!=(0|0);
      L25: do {
       if ($112) {
        $113 = HEAP32[$0>>2]|0;
        $114 = ((($113)) + 80|0);
        $115 = HEAP32[$114>>2]|0;
        HEAP32[$pEL>>2] = $115;
        $116 = HEAP32[$0>>2]|0;
        $117 = ((($116)) + 23|0);
        HEAP8[$117>>0] = 0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $118 = HEAP32[$i>>2]|0;
         $119 = HEAP32[$pEL>>2]|0;
         $120 = HEAP32[$119>>2]|0;
         $121 = ($118|0)<($120|0);
         if (!($121)) {
          break L25;
         }
         $122 = HEAP32[$0>>2]|0;
         $123 = HEAP32[$i>>2]|0;
         $124 = HEAP32[$pEL>>2]|0;
         $125 = ((($124)) + 4|0);
         $126 = HEAP32[$125>>2]|0;
         $127 = (($126) + (($123*20)|0)|0);
         $128 = HEAP32[$127>>2]|0;
         $129 = HEAP32[$i>>2]|0;
         $130 = HEAP32[$pEL>>2]|0;
         $131 = ((($130)) + 4|0);
         $132 = HEAP32[$131>>2]|0;
         $133 = (($132) + (($129*20)|0)|0);
         $134 = ((($133)) + 16|0);
         $135 = HEAP32[$134>>2]|0;
         _sqlite3ExprCode($122,$128,$135);
         $136 = HEAP32[$i>>2]|0;
         $137 = (($136) + 1)|0;
         HEAP32[$i>>2] = $137;
        }
       }
      } while(0);
      $138 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeGoto($138,1)|0);
     }
    }
   } while(0);
   $139 = HEAP32[$v>>2]|0;
   $140 = ($139|0)!=(0|0);
   if ($140) {
    $141 = HEAP32[$0>>2]|0;
    $142 = ((($141)) + 36|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = ($143|0)==(0);
    if ($144) {
     $145 = HEAP32[$db>>2]|0;
     $146 = ((($145)) + 69|0);
     $147 = HEAP8[$146>>0]|0;
     $148 = ($147<<24>>24)!=(0);
     if (!($148)) {
      $149 = HEAP32[$0>>2]|0;
      $150 = ((($149)) + 120|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = ($151|0)!=(0|0);
      if ($152) {
       $153 = HEAP32[$0>>2]|0;
       $154 = ((($153)) + 40|0);
       $155 = HEAP32[$154>>2]|0;
       $156 = ($155|0)==(0);
       if ($156) {
        $157 = HEAP32[$0>>2]|0;
        $158 = ((($157)) + 40|0);
        HEAP32[$158>>2] = 1;
       }
      }
      $159 = HEAP32[$v>>2]|0;
      $160 = HEAP32[$0>>2]|0;
      _sqlite3VdbeMakeReady($159,$160);
      $161 = HEAP32[$0>>2]|0;
      $162 = ((($161)) + 12|0);
      HEAP32[$162>>2] = 101;
      STACKTOP = sp;return;
     }
    }
   }
   $163 = HEAP32[$0>>2]|0;
   $164 = ((($163)) + 12|0);
   HEAP32[$164>>2] = 1;
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)==(0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 12|0);
 HEAP32[$21>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3BeginTransaction($pParse,$type) {
 $pParse = $pParse|0;
 $type = $type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $db = sp + 8|0;
 $v = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $type;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3AuthCheck($4,22,37527,0,0)|0);
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(7);
 L7: do {
  if ($12) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $13 = HEAP32[$i>>2]|0;
    $14 = HEAP32[$db>>2]|0;
    $15 = ((($14)) + 20|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ($13|0)<($16|0);
    if (!($17)) {
     break L7;
    }
    $18 = HEAP32[$v>>2]|0;
    $19 = HEAP32[$i>>2]|0;
    $20 = HEAP32[$1>>2]|0;
    $21 = ($20|0)==(9);
    $22 = $21&1;
    $23 = (($22) + 1)|0;
    (_sqlite3VdbeAddOp2($18,2,$19,$23)|0);
    $24 = HEAP32[$v>>2]|0;
    $25 = HEAP32[$i>>2]|0;
    _sqlite3VdbeUsesBtree($24,$25);
    $26 = HEAP32[$i>>2]|0;
    $27 = (($26) + 1)|0;
    HEAP32[$i>>2] = $27;
   }
  }
 } while(0);
 $28 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp0($28,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3CommitTransaction($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3AuthCheck($1,22,37550,0,0)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp1($8,1,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3RollbackTransaction($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3AuthCheck($1,22,37541,0,0)|0);
 $3 = ($2|0)!=(0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp2($8,1,1,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3Savepoint($pParse,$op,$pName) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zName = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pName;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3NameFromToken($4,$5)|0);
 HEAP32[$zName>>2] = $6;
 $7 = HEAP32[$zName>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = (5512 + ($14<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$zName>>2]|0;
  $18 = (_sqlite3AuthCheck($13,32,$16,$17,0)|0);
  $19 = ($18|0)!=(0);
  if (!($19)) {
   $23 = HEAP32[$v>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$zName>>2]|0;
   (_sqlite3VdbeAddOp4($23,0,$24,0,0,$25,-1)|0);
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($21,$22);
 STACKTOP = sp;return;
}
function _sqlite3StartTable($pParse,$pName1,$pName2,$isTemp,$isView,$isVirtual,$noErr) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $isTemp = $isTemp|0;
 $isView = $isView|0;
 $isVirtual = $isVirtual|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $db = 0, $fileFormat = 0, $iDb = 0, $or$cond = 0, $or$cond3 = 0, $pName = 0, $pTable = 0, $reg1 = 0, $reg2 = 0, $reg3 = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $vararg_buffer6 = 0, $zDb = 0, $zDb1 = 0;
 var $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $pTable = sp + 68|0;
 $zName = sp + 64|0;
 $db = sp + 60|0;
 $v = sp + 56|0;
 $iDb = sp + 52|0;
 $pName = sp + 48|0;
 $zDb = sp + 44|0;
 $zDb1 = sp + 40|0;
 $addr1 = sp + 36|0;
 $fileFormat = sp + 32|0;
 $reg1 = sp + 28|0;
 $reg2 = sp + 24|0;
 $reg3 = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $isTemp;
 HEAP32[$4>>2] = $isView;
 HEAP32[$5>>2] = $isVirtual;
 HEAP32[$6>>2] = $noErr;
 HEAP32[$zName>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 148|0);
 $11 = ((($10)) + 5|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$db>>2]|0;
  $16 = ((($15)) + 148|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)==(1);
  if ($18) {
   $19 = HEAP32[$db>>2]|0;
   $20 = ((($19)) + 148|0);
   $21 = ((($20)) + 4|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$db>>2]|0;
   $25 = HEAP32[$iDb>>2]|0;
   $26 = ($25|0)==(1);
   $27 = $26 ? 19905 : 19924;
   $28 = (_sqlite3DbStrDup($24,$27)|0);
   HEAP32[$zName>>2] = $28;
   $29 = HEAP32[$1>>2]|0;
   HEAP32[$pName>>2] = $29;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$1>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (_sqlite3TwoPartName($30,$31,$32,$pName)|0);
  HEAP32[$iDb>>2] = $33;
  $34 = HEAP32[$iDb>>2]|0;
  $35 = ($34|0)<(0);
  if ($35) {
   STACKTOP = sp;return;
  }
  $36 = HEAP32[$3>>2]|0;
  $37 = ($36|0)!=(0);
  if ($37) {
   $38 = HEAP32[$2>>2]|0;
   $39 = ((($38)) + 4|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($40>>>0)>(0);
   $42 = HEAP32[$iDb>>2]|0;
   $43 = ($42|0)!=(1);
   $or$cond = $41 & $43;
   if ($or$cond) {
    $44 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($44,37417,$vararg_buffer);
    STACKTOP = sp;return;
   }
  }
  $45 = HEAP32[$3>>2]|0;
  $46 = ($45|0)!=(0);
  if ($46) {
   HEAP32[$iDb>>2] = 1;
  }
  $47 = HEAP32[$db>>2]|0;
  $48 = HEAP32[$pName>>2]|0;
  $49 = (_sqlite3NameFromToken($47,$48)|0);
  HEAP32[$zName>>2] = $49;
 }
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 384|0);
 $52 = HEAP32[$pName>>2]|0;
 ;HEAP32[$51>>2]=HEAP32[$52>>2]|0;HEAP32[$51+4>>2]=HEAP32[$52+4>>2]|0;
 $53 = HEAP32[$zName>>2]|0;
 $54 = ($53|0)==(0|0);
 if ($54) {
  STACKTOP = sp;return;
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$zName>>2]|0;
 $57 = (_sqlite3CheckObjectName($55,$56)|0);
 $58 = (0)!=($57|0);
 L21: do {
  if (!($58)) {
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 148|0);
   $61 = ((($60)) + 4|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62&255;
   $64 = ($63|0)==(1);
   if ($64) {
    HEAP32[$3>>2] = 1;
   }
   $65 = HEAP32[$iDb>>2]|0;
   $66 = HEAP32[$db>>2]|0;
   $67 = ((($66)) + 16|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (($68) + ($65<<4)|0);
   $70 = HEAP32[$69>>2]|0;
   HEAP32[$zDb>>2] = $70;
   $71 = HEAP32[$0>>2]|0;
   $72 = HEAP32[$3>>2]|0;
   $73 = ($72|0)==(1);
   $74 = $73 ? 19905 : 19924;
   $75 = HEAP32[$zDb>>2]|0;
   $76 = (_sqlite3AuthCheck($71,18,$74,0,$75)|0);
   $77 = ($76|0)!=(0);
   if (!($77)) {
    $78 = HEAP32[$5>>2]|0;
    $79 = ($78|0)!=(0);
    if (!($79)) {
     $80 = HEAP32[$0>>2]|0;
     $81 = HEAP32[$3>>2]|0;
     $82 = HEAP32[$4>>2]|0;
     $83 = $82<<1;
     $84 = (($81) + ($83))|0;
     $85 = (37458 + ($84)|0);
     $86 = HEAP8[$85>>0]|0;
     $87 = $86&255;
     $88 = HEAP32[$zName>>2]|0;
     $89 = HEAP32[$zDb>>2]|0;
     $90 = (_sqlite3AuthCheck($80,$87,$88,0,$89)|0);
     $91 = ($90|0)!=(0);
     if ($91) {
      break;
     }
    }
    $92 = HEAP32[$0>>2]|0;
    $93 = ((($92)) + 410|0);
    $94 = HEAP8[$93>>0]|0;
    $95 = ($94<<24>>24)!=(0);
    do {
     if (!($95)) {
      $96 = HEAP32[$iDb>>2]|0;
      $97 = HEAP32[$db>>2]|0;
      $98 = ((($97)) + 16|0);
      $99 = HEAP32[$98>>2]|0;
      $100 = (($99) + ($96<<4)|0);
      $101 = HEAP32[$100>>2]|0;
      HEAP32[$zDb1>>2] = $101;
      $102 = HEAP32[$0>>2]|0;
      $103 = (_sqlite3ReadSchema($102)|0);
      $104 = (0)!=($103|0);
      if ($104) {
       break L21;
      }
      $105 = HEAP32[$db>>2]|0;
      $106 = HEAP32[$zName>>2]|0;
      $107 = HEAP32[$zDb1>>2]|0;
      $108 = (_sqlite3FindTable($105,$106,$107)|0);
      HEAP32[$pTable>>2] = $108;
      $109 = HEAP32[$pTable>>2]|0;
      $110 = ($109|0)!=(0|0);
      if (!($110)) {
       $116 = HEAP32[$db>>2]|0;
       $117 = HEAP32[$zName>>2]|0;
       $118 = HEAP32[$zDb1>>2]|0;
       $119 = (_sqlite3FindIndex($116,$117,$118)|0);
       $120 = ($119|0)!=(0|0);
       if (!($120)) {
        break;
       }
       $121 = HEAP32[$0>>2]|0;
       $122 = HEAP32[$zName>>2]|0;
       HEAP32[$vararg_buffer6>>2] = $122;
       _sqlite3ErrorMsg($121,37486,$vararg_buffer6);
       break L21;
      }
      $111 = HEAP32[$6>>2]|0;
      $112 = ($111|0)!=(0);
      $113 = HEAP32[$0>>2]|0;
      if ($112) {
       $115 = HEAP32[$iDb>>2]|0;
       _sqlite3CodeVerifySchema($113,$115);
       break L21;
      } else {
       $114 = HEAP32[$pName>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $114;
       _sqlite3ErrorMsg($113,37462,$vararg_buffer4);
       break L21;
      }
     }
    } while(0);
    $123 = HEAP32[$db>>2]|0;
    $124 = (_sqlite3DbMallocZero($123,72,0)|0);
    HEAP32[$pTable>>2] = $124;
    $125 = HEAP32[$pTable>>2]|0;
    $126 = ($125|0)==(0|0);
    if ($126) {
     $127 = HEAP32[$0>>2]|0;
     $128 = ((($127)) + 12|0);
     HEAP32[$128>>2] = 7;
     $129 = HEAP32[$0>>2]|0;
     $130 = ((($129)) + 36|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = (($131) + 1)|0;
     HEAP32[$130>>2] = $132;
     break;
    }
    $133 = HEAP32[$zName>>2]|0;
    $134 = HEAP32[$pTable>>2]|0;
    HEAP32[$134>>2] = $133;
    $135 = HEAP32[$pTable>>2]|0;
    $136 = ((($135)) + 32|0);
    HEAP16[$136>>1] = -1;
    $137 = HEAP32[$iDb>>2]|0;
    $138 = HEAP32[$db>>2]|0;
    $139 = ((($138)) + 16|0);
    $140 = HEAP32[$139>>2]|0;
    $141 = (($140) + ($137<<4)|0);
    $142 = ((($141)) + 12|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = HEAP32[$pTable>>2]|0;
    $145 = ((($144)) + 64|0);
    HEAP32[$145>>2] = $143;
    $146 = HEAP32[$pTable>>2]|0;
    $147 = ((($146)) + 36|0);
    HEAP16[$147>>1] = 1;
    $148 = HEAP32[$pTable>>2]|0;
    $149 = ((($148)) + 38|0);
    HEAP16[$149>>1] = 200;
    $150 = HEAP32[$pTable>>2]|0;
    $151 = HEAP32[$0>>2]|0;
    $152 = ((($151)) + 440|0);
    HEAP32[$152>>2] = $150;
    $153 = HEAP32[$0>>2]|0;
    $154 = ((($153)) + 18|0);
    $155 = HEAP8[$154>>0]|0;
    $156 = ($155<<24>>24)!=(0);
    if (!($156)) {
     $157 = HEAP32[$zName>>2]|0;
     $158 = (_strcmp($157,28817)|0);
     $159 = ($158|0)==(0);
     if ($159) {
      $160 = HEAP32[$pTable>>2]|0;
      $161 = HEAP32[$pTable>>2]|0;
      $162 = ((($161)) + 64|0);
      $163 = HEAP32[$162>>2]|0;
      $164 = ((($163)) + 72|0);
      HEAP32[$164>>2] = $160;
     }
    }
    $165 = HEAP32[$db>>2]|0;
    $166 = ((($165)) + 148|0);
    $167 = ((($166)) + 5|0);
    $168 = HEAP8[$167>>0]|0;
    $169 = ($168<<24>>24)!=(0);
    if ($169) {
     STACKTOP = sp;return;
    }
    $170 = HEAP32[$0>>2]|0;
    $171 = (_sqlite3GetVdbe($170)|0);
    HEAP32[$v>>2] = $171;
    $172 = ($171|0)!=(0|0);
    if (!($172)) {
     STACKTOP = sp;return;
    }
    $173 = HEAP32[$0>>2]|0;
    $174 = HEAP32[$iDb>>2]|0;
    _sqlite3BeginWriteOperation($173,1,$174);
    $175 = HEAP32[$5>>2]|0;
    $176 = ($175|0)!=(0);
    if ($176) {
     $177 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($177,152)|0);
    }
    $178 = HEAP32[$0>>2]|0;
    $179 = ((($178)) + 44|0);
    $180 = HEAP32[$179>>2]|0;
    $181 = (($180) + 1)|0;
    HEAP32[$179>>2] = $181;
    $182 = HEAP32[$0>>2]|0;
    $183 = ((($182)) + 100|0);
    HEAP32[$183>>2] = $181;
    HEAP32[$reg1>>2] = $181;
    $184 = HEAP32[$0>>2]|0;
    $185 = ((($184)) + 44|0);
    $186 = HEAP32[$185>>2]|0;
    $187 = (($186) + 1)|0;
    HEAP32[$185>>2] = $187;
    $188 = HEAP32[$0>>2]|0;
    $189 = ((($188)) + 104|0);
    HEAP32[$189>>2] = $187;
    HEAP32[$reg2>>2] = $187;
    $190 = HEAP32[$0>>2]|0;
    $191 = ((($190)) + 44|0);
    $192 = HEAP32[$191>>2]|0;
    $193 = (($192) + 1)|0;
    HEAP32[$191>>2] = $193;
    HEAP32[$reg3>>2] = $193;
    $194 = HEAP32[$v>>2]|0;
    $195 = HEAP32[$iDb>>2]|0;
    $196 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp3($194,101,$195,$196,2)|0);
    $197 = HEAP32[$v>>2]|0;
    $198 = HEAP32[$iDb>>2]|0;
    _sqlite3VdbeUsesBtree($197,$198);
    $199 = HEAP32[$v>>2]|0;
    $200 = HEAP32[$reg3>>2]|0;
    $201 = (_sqlite3VdbeAddOp1($199,21,$200)|0);
    HEAP32[$addr1>>2] = $201;
    $202 = HEAP32[$db>>2]|0;
    $203 = ((($202)) + 24|0);
    $204 = HEAP32[$203>>2]|0;
    $205 = $204 & 32768;
    $206 = ($205|0)!=(0);
    $207 = $206 ? 1 : 4;
    HEAP32[$fileFormat>>2] = $207;
    $208 = HEAP32[$v>>2]|0;
    $209 = HEAP32[$iDb>>2]|0;
    $210 = HEAP32[$fileFormat>>2]|0;
    (_sqlite3VdbeAddOp3($208,102,$209,2,$210)|0);
    $211 = HEAP32[$v>>2]|0;
    $212 = HEAP32[$iDb>>2]|0;
    $213 = HEAP32[$db>>2]|0;
    $214 = ((($213)) + 66|0);
    $215 = HEAP8[$214>>0]|0;
    $216 = $215&255;
    (_sqlite3VdbeAddOp3($211,102,$212,5,$216)|0);
    $217 = HEAP32[$v>>2]|0;
    $218 = HEAP32[$addr1>>2]|0;
    _sqlite3VdbeJumpHere($217,$218);
    $219 = HEAP32[$4>>2]|0;
    $220 = ($219|0)!=(0);
    $221 = HEAP32[$5>>2]|0;
    $222 = ($221|0)!=(0);
    $or$cond3 = $220 | $222;
    $223 = HEAP32[$v>>2]|0;
    if ($or$cond3) {
     $224 = HEAP32[$reg2>>2]|0;
     (_sqlite3VdbeAddOp2($223,76,0,$224)|0);
    } else {
     $225 = HEAP32[$iDb>>2]|0;
     $226 = HEAP32[$reg2>>2]|0;
     $227 = (_sqlite3VdbeAddOp2($223,135,$225,$226)|0);
     $228 = HEAP32[$0>>2]|0;
     $229 = ((($228)) + 132|0);
     HEAP32[$229>>2] = $227;
    }
    $230 = HEAP32[$0>>2]|0;
    $231 = HEAP32[$iDb>>2]|0;
    _sqlite3OpenMasterTable($230,$231);
    $232 = HEAP32[$v>>2]|0;
    $233 = HEAP32[$reg1>>2]|0;
    (_sqlite3VdbeAddOp2($232,114,0,$233)|0);
    $234 = HEAP32[$v>>2]|0;
    $235 = HEAP32[$reg3>>2]|0;
    (_sqlite3VdbeAddOp4($234,81,6,$235,0,37521,-2)|0);
    $236 = HEAP32[$v>>2]|0;
    $237 = HEAP32[$reg3>>2]|0;
    $238 = HEAP32[$reg1>>2]|0;
    (_sqlite3VdbeAddOp3($236,115,0,$237,$238)|0);
    $239 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($239,8);
    $240 = HEAP32[$v>>2]|0;
    (_sqlite3VdbeAddOp0($240,111)|0);
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $241 = HEAP32[$db>>2]|0;
 $242 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($241,$242);
 STACKTOP = sp;return;
}
function _disableLookaside($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = (($3) + 1)<<24>>24;
 HEAP8[$2>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 256|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3EndTable($pParse,$pCons,$pEnd,$tabOpts,$pSelect) {
 $pParse = $pParse|0;
 $pCons = $pCons|0;
 $pEnd = $pEnd|0;
 $tabOpts = $tabOpts|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addrInsLoop = 0, $addrTop = 0, $db = 0, $dest = 0, $iDb = 0, $n = 0, $nName = 0, $or$cond = 0, $p = 0, $pDb = 0, $pEnd2 = 0, $pIdx = 0, $pOld = 0, $pSchema = 0, $pSelTab = 0, $regRec = 0, $regRowid = 0, $regYield = 0, $v = 0;
 var $vararg_buffer = 0, $vararg_buffer19 = 0, $vararg_buffer2 = 0, $vararg_buffer22 = 0, $vararg_buffer4 = 0, $vararg_buffer9 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $zName = 0, $zStmt = 0, $zType = 0, $zType2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer22 = sp + 72|0;
 $vararg_buffer19 = sp + 64|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 200|0;
 $1 = sp + 196|0;
 $2 = sp + 192|0;
 $3 = sp + 204|0;
 $4 = sp + 188|0;
 $p = sp + 184|0;
 $db = sp + 180|0;
 $iDb = sp + 176|0;
 $pIdx = sp + 172|0;
 $n = sp + 168|0;
 $v = sp + 164|0;
 $zType = sp + 160|0;
 $zType2 = sp + 156|0;
 $zStmt = sp + 152|0;
 $dest = sp + 128|0;
 $regYield = sp + 120|0;
 $addrTop = sp + 116|0;
 $regRec = sp + 112|0;
 $regRowid = sp + 108|0;
 $addrInsLoop = sp + 104|0;
 $pSelTab = sp + 100|0;
 $pEnd2 = sp + 96|0;
 $pDb = sp + 92|0;
 $pOld = sp + 88|0;
 $pSchema = sp + 84|0;
 $zName = sp + 80|0;
 $nName = sp + 76|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pCons;
 HEAP32[$2>>2] = $pEnd;
 HEAP8[$3>>0] = $tabOpts;
 HEAP32[$4>>2] = $pSelect;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$4>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 & $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 440|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 148|0);
 $18 = ((($17)) + 5|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if ($20) {
  $21 = HEAP32[$db>>2]|0;
  $22 = ((($21)) + 148|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 28|0);
  HEAP32[$25>>2] = $23;
  $26 = HEAP32[$p>>2]|0;
  $27 = ((($26)) + 28|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(1);
  if ($29) {
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 42|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = $32&255;
   $34 = $33 | 1;
   $35 = $34&255;
   HEAP8[$31>>0] = $35;
  }
 }
 $36 = HEAP8[$3>>0]|0;
 $37 = $36&255;
 $38 = $37 & 32;
 $39 = ($38|0)!=(0);
 do {
  if ($39) {
   $40 = HEAP32[$p>>2]|0;
   $41 = ((($40)) + 42|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = $42&255;
   $44 = $43 & 8;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($46,37098,$vararg_buffer);
    STACKTOP = sp;return;
   }
   $47 = HEAP32[$p>>2]|0;
   $48 = ((($47)) + 42|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = $50 & 4;
   $52 = ($51|0)==(0);
   if ($52) {
    $53 = HEAP32[$0>>2]|0;
    $54 = HEAP32[$p>>2]|0;
    $55 = HEAP32[$54>>2]|0;
    HEAP32[$vararg_buffer2>>2] = $55;
    _sqlite3ErrorMsg($53,37148,$vararg_buffer2);
    break;
   } else {
    $56 = HEAP32[$p>>2]|0;
    $57 = ((($56)) + 42|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58&255;
    $60 = $59 | 96;
    $61 = $60&255;
    HEAP8[$57>>0] = $61;
    $62 = HEAP32[$0>>2]|0;
    $63 = HEAP32[$p>>2]|0;
    _convertToWithoutRowidTable($62,$63);
    break;
   }
  }
 } while(0);
 $64 = HEAP32[$db>>2]|0;
 $65 = HEAP32[$p>>2]|0;
 $66 = ((($65)) + 64|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (_sqlite3SchemaToIndex($64,$67)|0);
 HEAP32[$iDb>>2] = $68;
 $69 = HEAP32[$p>>2]|0;
 $70 = ((($69)) + 24|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = ($71|0)!=(0|0);
 if ($72) {
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$p>>2]|0;
  $75 = HEAP32[$p>>2]|0;
  $76 = ((($75)) + 24|0);
  $77 = HEAP32[$76>>2]|0;
  _sqlite3ResolveSelfReference($73,$74,4,0,$77);
 }
 $78 = HEAP32[$p>>2]|0;
 _estimateTableWidth($78);
 $79 = HEAP32[$p>>2]|0;
 $80 = ((($79)) + 8|0);
 $81 = HEAP32[$80>>2]|0;
 HEAP32[$pIdx>>2] = $81;
 while(1) {
  $82 = HEAP32[$pIdx>>2]|0;
  $83 = ($82|0)!=(0|0);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$pIdx>>2]|0;
  _estimateIndexWidth($84);
  $85 = HEAP32[$pIdx>>2]|0;
  $86 = ((($85)) + 20|0);
  $87 = HEAP32[$86>>2]|0;
  HEAP32[$pIdx>>2] = $87;
 }
 $88 = HEAP32[$db>>2]|0;
 $89 = ((($88)) + 148|0);
 $90 = ((($89)) + 5|0);
 $91 = HEAP8[$90>>0]|0;
 $92 = ($91<<24>>24)!=(0);
 if (!($92)) {
  $93 = HEAP32[$0>>2]|0;
  $94 = (_sqlite3GetVdbe($93)|0);
  HEAP32[$v>>2] = $94;
  $95 = HEAP32[$v>>2]|0;
  $96 = ($95|0)==(0|0);
  if ($96) {
   STACKTOP = sp;return;
  }
  $97 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp1($97,111,0)|0);
  $98 = HEAP32[$p>>2]|0;
  $99 = ((($98)) + 12|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = ($100|0)==(0|0);
  if ($101) {
   HEAP32[$zType>>2] = 32993;
   HEAP32[$zType2>>2] = 37180;
  } else {
   HEAP32[$zType>>2] = 36058;
   HEAP32[$zType2>>2] = 37186;
  }
  $102 = HEAP32[$4>>2]|0;
  $103 = ($102|0)!=(0|0);
  do {
   if ($103) {
    $104 = HEAP32[$0>>2]|0;
    $105 = ((($104)) + 44|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = (($106) + 1)|0;
    HEAP32[$105>>2] = $107;
    HEAP32[$regYield>>2] = $107;
    $108 = HEAP32[$0>>2]|0;
    $109 = ((($108)) + 44|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = (($110) + 1)|0;
    HEAP32[$109>>2] = $111;
    HEAP32[$regRec>>2] = $111;
    $112 = HEAP32[$0>>2]|0;
    $113 = ((($112)) + 44|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = (($114) + 1)|0;
    HEAP32[$113>>2] = $115;
    HEAP32[$regRowid>>2] = $115;
    $116 = HEAP32[$0>>2]|0;
    _sqlite3MayAbort($116);
    $117 = HEAP32[$v>>2]|0;
    $118 = HEAP32[$0>>2]|0;
    $119 = ((($118)) + 104|0);
    $120 = HEAP32[$119>>2]|0;
    $121 = HEAP32[$iDb>>2]|0;
    (_sqlite3VdbeAddOp3($117,105,1,$120,$121)|0);
    $122 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($122,16);
    $123 = HEAP32[$0>>2]|0;
    $124 = ((($123)) + 40|0);
    HEAP32[$124>>2] = 2;
    $125 = HEAP32[$v>>2]|0;
    $126 = (_sqlite3VdbeCurrentAddr($125)|0);
    $127 = (($126) + 1)|0;
    HEAP32[$addrTop>>2] = $127;
    $128 = HEAP32[$v>>2]|0;
    $129 = HEAP32[$regYield>>2]|0;
    $130 = HEAP32[$addrTop>>2]|0;
    (_sqlite3VdbeAddOp3($128,15,$129,0,$130)|0);
    $131 = HEAP32[$regYield>>2]|0;
    _sqlite3SelectDestInit($dest,13,$131);
    $132 = HEAP32[$0>>2]|0;
    $133 = HEAP32[$4>>2]|0;
    (_sqlite3Select($132,$133,$dest)|0);
    $134 = HEAP32[$v>>2]|0;
    $135 = HEAP32[$regYield>>2]|0;
    _sqlite3VdbeEndCoroutine($134,$135);
    $136 = HEAP32[$v>>2]|0;
    $137 = HEAP32[$addrTop>>2]|0;
    $138 = (($137) - 1)|0;
    _sqlite3VdbeJumpHere($136,$138);
    $139 = HEAP32[$0>>2]|0;
    $140 = ((($139)) + 36|0);
    $141 = HEAP32[$140>>2]|0;
    $142 = ($141|0)!=(0);
    if ($142) {
     STACKTOP = sp;return;
    }
    $143 = HEAP32[$0>>2]|0;
    $144 = HEAP32[$4>>2]|0;
    $145 = (_sqlite3ResultSetOfSelect($143,$144)|0);
    HEAP32[$pSelTab>>2] = $145;
    $146 = HEAP32[$pSelTab>>2]|0;
    $147 = ($146|0)==(0|0);
    if ($147) {
     STACKTOP = sp;return;
    } else {
     $148 = HEAP32[$pSelTab>>2]|0;
     $149 = ((($148)) + 34|0);
     $150 = HEAP16[$149>>1]|0;
     $151 = HEAP32[$p>>2]|0;
     $152 = ((($151)) + 34|0);
     HEAP16[$152>>1] = $150;
     $153 = HEAP32[$pSelTab>>2]|0;
     $154 = ((($153)) + 4|0);
     $155 = HEAP32[$154>>2]|0;
     $156 = HEAP32[$p>>2]|0;
     $157 = ((($156)) + 4|0);
     HEAP32[$157>>2] = $155;
     $158 = HEAP32[$pSelTab>>2]|0;
     $159 = ((($158)) + 34|0);
     HEAP16[$159>>1] = 0;
     $160 = HEAP32[$pSelTab>>2]|0;
     $161 = ((($160)) + 4|0);
     HEAP32[$161>>2] = 0;
     $162 = HEAP32[$db>>2]|0;
     $163 = HEAP32[$pSelTab>>2]|0;
     _sqlite3DeleteTable($162,$163);
     $164 = HEAP32[$v>>2]|0;
     $165 = ((($dest)) + 8|0);
     $166 = HEAP32[$165>>2]|0;
     $167 = (_sqlite3VdbeAddOp1($164,16,$166)|0);
     HEAP32[$addrInsLoop>>2] = $167;
     $168 = HEAP32[$v>>2]|0;
     $169 = ((($dest)) + 12|0);
     $170 = HEAP32[$169>>2]|0;
     $171 = ((($dest)) + 16|0);
     $172 = HEAP32[$171>>2]|0;
     $173 = HEAP32[$regRec>>2]|0;
     (_sqlite3VdbeAddOp3($168,99,$170,$172,$173)|0);
     $174 = HEAP32[$v>>2]|0;
     $175 = HEAP32[$p>>2]|0;
     _sqlite3TableAffinity($174,$175,0);
     $176 = HEAP32[$v>>2]|0;
     $177 = HEAP32[$regRowid>>2]|0;
     (_sqlite3VdbeAddOp2($176,114,1,$177)|0);
     $178 = HEAP32[$v>>2]|0;
     $179 = HEAP32[$regRec>>2]|0;
     $180 = HEAP32[$regRowid>>2]|0;
     (_sqlite3VdbeAddOp3($178,115,1,$179,$180)|0);
     $181 = HEAP32[$v>>2]|0;
     $182 = HEAP32[$addrInsLoop>>2]|0;
     (_sqlite3VdbeGoto($181,$182)|0);
     $183 = HEAP32[$v>>2]|0;
     $184 = HEAP32[$addrInsLoop>>2]|0;
     _sqlite3VdbeJumpHere($183,$184);
     $185 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp1($185,111,1)|0);
     break;
    }
   }
  } while(0);
  $186 = HEAP32[$4>>2]|0;
  $187 = ($186|0)!=(0|0);
  if ($187) {
   $188 = HEAP32[$db>>2]|0;
   $189 = HEAP32[$p>>2]|0;
   $190 = (_createTableStmt($188,$189)|0);
   HEAP32[$zStmt>>2] = $190;
  } else {
   $191 = HEAP8[$3>>0]|0;
   $192 = $191&255;
   $193 = ($192|0)!=(0);
   $194 = HEAP32[$0>>2]|0;
   $195 = ((($194)) + 392|0);
   $196 = HEAP32[$2>>2]|0;
   $197 = $193 ? $195 : $196;
   HEAP32[$pEnd2>>2] = $197;
   $198 = HEAP32[$pEnd2>>2]|0;
   $199 = HEAP32[$198>>2]|0;
   $200 = HEAP32[$0>>2]|0;
   $201 = ((($200)) + 384|0);
   $202 = HEAP32[$201>>2]|0;
   $203 = $199;
   $204 = $202;
   $205 = (($203) - ($204))|0;
   HEAP32[$n>>2] = $205;
   $206 = HEAP32[$pEnd2>>2]|0;
   $207 = HEAP32[$206>>2]|0;
   $208 = HEAP8[$207>>0]|0;
   $209 = $208 << 24 >> 24;
   $210 = ($209|0)!=(59);
   if ($210) {
    $211 = HEAP32[$pEnd2>>2]|0;
    $212 = ((($211)) + 4|0);
    $213 = HEAP32[$212>>2]|0;
    $214 = HEAP32[$n>>2]|0;
    $215 = (($214) + ($213))|0;
    HEAP32[$n>>2] = $215;
   }
   $216 = HEAP32[$db>>2]|0;
   $217 = HEAP32[$zType2>>2]|0;
   $218 = HEAP32[$n>>2]|0;
   $219 = HEAP32[$0>>2]|0;
   $220 = ((($219)) + 384|0);
   $221 = HEAP32[$220>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $217;
   $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
   HEAP32[$vararg_ptr7>>2] = $218;
   $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
   HEAP32[$vararg_ptr8>>2] = $221;
   $222 = (_sqlite3MPrintf($216,37191,$vararg_buffer4)|0);
   HEAP32[$zStmt>>2] = $222;
  }
  $223 = HEAP32[$0>>2]|0;
  $224 = HEAP32[$iDb>>2]|0;
  $225 = HEAP32[$db>>2]|0;
  $226 = ((($225)) + 16|0);
  $227 = HEAP32[$226>>2]|0;
  $228 = (($227) + ($224<<4)|0);
  $229 = HEAP32[$228>>2]|0;
  $230 = HEAP32[$iDb>>2]|0;
  $231 = ($230|0)==(1);
  $232 = $231 ? 19905 : 19924;
  $233 = HEAP32[$zType>>2]|0;
  $234 = HEAP32[$p>>2]|0;
  $235 = HEAP32[$234>>2]|0;
  $236 = HEAP32[$p>>2]|0;
  $237 = HEAP32[$236>>2]|0;
  $238 = HEAP32[$0>>2]|0;
  $239 = ((($238)) + 104|0);
  $240 = HEAP32[$239>>2]|0;
  $241 = HEAP32[$zStmt>>2]|0;
  $242 = HEAP32[$0>>2]|0;
  $243 = ((($242)) + 100|0);
  $244 = HEAP32[$243>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $229;
  $vararg_ptr12 = ((($vararg_buffer9)) + 4|0);
  HEAP32[$vararg_ptr12>>2] = $232;
  $vararg_ptr13 = ((($vararg_buffer9)) + 8|0);
  HEAP32[$vararg_ptr13>>2] = $233;
  $vararg_ptr14 = ((($vararg_buffer9)) + 12|0);
  HEAP32[$vararg_ptr14>>2] = $235;
  $vararg_ptr15 = ((($vararg_buffer9)) + 16|0);
  HEAP32[$vararg_ptr15>>2] = $237;
  $vararg_ptr16 = ((($vararg_buffer9)) + 20|0);
  HEAP32[$vararg_ptr16>>2] = $240;
  $vararg_ptr17 = ((($vararg_buffer9)) + 24|0);
  HEAP32[$vararg_ptr17>>2] = $241;
  $vararg_ptr18 = ((($vararg_buffer9)) + 28|0);
  HEAP32[$vararg_ptr18>>2] = $244;
  _sqlite3NestedParse($223,37206,$vararg_buffer9);
  $245 = HEAP32[$db>>2]|0;
  $246 = HEAP32[$zStmt>>2]|0;
  _sqlite3DbFree($245,$246);
  $247 = HEAP32[$0>>2]|0;
  $248 = HEAP32[$iDb>>2]|0;
  _sqlite3ChangeCookie($247,$248);
  $249 = HEAP32[$p>>2]|0;
  $250 = ((($249)) + 42|0);
  $251 = HEAP8[$250>>0]|0;
  $252 = $251&255;
  $253 = $252 & 8;
  $254 = ($253|0)!=(0);
  if ($254) {
   $255 = HEAP32[$iDb>>2]|0;
   $256 = HEAP32[$db>>2]|0;
   $257 = ((($256)) + 16|0);
   $258 = HEAP32[$257>>2]|0;
   $259 = (($258) + ($255<<4)|0);
   HEAP32[$pDb>>2] = $259;
   $260 = HEAP32[$pDb>>2]|0;
   $261 = ((($260)) + 12|0);
   $262 = HEAP32[$261>>2]|0;
   $263 = ((($262)) + 72|0);
   $264 = HEAP32[$263>>2]|0;
   $265 = ($264|0)==(0|0);
   if ($265) {
    $266 = HEAP32[$0>>2]|0;
    $267 = HEAP32[$pDb>>2]|0;
    $268 = HEAP32[$267>>2]|0;
    HEAP32[$vararg_buffer19>>2] = $268;
    _sqlite3NestedParse($266,37293,$vararg_buffer19);
   }
  }
  $269 = HEAP32[$v>>2]|0;
  $270 = HEAP32[$iDb>>2]|0;
  $271 = HEAP32[$db>>2]|0;
  $272 = HEAP32[$p>>2]|0;
  $273 = HEAP32[$272>>2]|0;
  HEAP32[$vararg_buffer22>>2] = $273;
  $274 = (_sqlite3MPrintf($271,37335,$vararg_buffer22)|0);
  _sqlite3VdbeAddParseSchemaOp($269,$270,$274);
 }
 $275 = HEAP32[$db>>2]|0;
 $276 = ((($275)) + 148|0);
 $277 = ((($276)) + 5|0);
 $278 = HEAP8[$277>>0]|0;
 $279 = ($278<<24>>24)!=(0);
 if (!($279)) {
  STACKTOP = sp;return;
 }
 $280 = HEAP32[$p>>2]|0;
 $281 = ((($280)) + 64|0);
 $282 = HEAP32[$281>>2]|0;
 HEAP32[$pSchema>>2] = $282;
 $283 = HEAP32[$pSchema>>2]|0;
 $284 = ((($283)) + 8|0);
 $285 = HEAP32[$p>>2]|0;
 $286 = HEAP32[$285>>2]|0;
 $287 = HEAP32[$p>>2]|0;
 $288 = (_sqlite3HashInsert($284,$286,$287)|0);
 HEAP32[$pOld>>2] = $288;
 $289 = HEAP32[$pOld>>2]|0;
 $290 = ($289|0)!=(0|0);
 if ($290) {
  $291 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($291);
  STACKTOP = sp;return;
 }
 $292 = HEAP32[$0>>2]|0;
 $293 = ((($292)) + 440|0);
 HEAP32[$293>>2] = 0;
 $294 = HEAP32[$db>>2]|0;
 $295 = ((($294)) + 24|0);
 $296 = HEAP32[$295>>2]|0;
 $297 = $296 | 2;
 HEAP32[$295>>2] = $297;
 $298 = HEAP32[$p>>2]|0;
 $299 = ((($298)) + 12|0);
 $300 = HEAP32[$299>>2]|0;
 $301 = ($300|0)!=(0|0);
 if ($301) {
  STACKTOP = sp;return;
 }
 $302 = HEAP32[$0>>2]|0;
 $303 = ((($302)) + 384|0);
 $304 = HEAP32[$303>>2]|0;
 HEAP32[$zName>>2] = $304;
 $305 = HEAP32[$1>>2]|0;
 $306 = HEAP32[$305>>2]|0;
 $307 = ($306|0)==(0|0);
 if ($307) {
  $308 = HEAP32[$2>>2]|0;
  HEAP32[$1>>2] = $308;
 }
 $309 = HEAP32[$1>>2]|0;
 $310 = HEAP32[$309>>2]|0;
 $311 = HEAP32[$zName>>2]|0;
 $312 = $310;
 $313 = $311;
 $314 = (($312) - ($313))|0;
 HEAP32[$nName>>2] = $314;
 $315 = HEAP32[$zName>>2]|0;
 $316 = HEAP32[$nName>>2]|0;
 $317 = (_sqlite3Utf8CharLen81($315,$316)|0);
 $318 = (13 + ($317))|0;
 $319 = HEAP32[$p>>2]|0;
 $320 = ((($319)) + 44|0);
 HEAP32[$320>>2] = $318;
 STACKTOP = sp;return;
}
function _sqlite3AddColumn($pParse,$pName,$pType) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $pType = $pType|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $db = 0, $i = 0, $p = 0, $pCol = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $z = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $p = sp + 36|0;
 $i = sp + 32|0;
 $z = sp + 28|0;
 $zType = sp + 24|0;
 $pCol = sp + 20|0;
 $db = sp + 16|0;
 $aNew = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $pType;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 440|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = ($7|0)==(0|0);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 34|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11 << 16 >> 16;
 $13 = (($12) + 1)|0;
 $14 = HEAP32[$db>>2]|0;
 $15 = ((($14)) + 96|0);
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($13|0)>($17|0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$vararg_buffer>>2] = $21;
  _sqlite3ErrorMsg($19,37049,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$2>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = (($25) + ($28))|0;
 $30 = (($29) + 2)|0;
 $31 = (_sqlite3DbMallocRaw($22,$30,0)|0);
 HEAP32[$z>>2] = $31;
 $32 = HEAP32[$z>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$z>>2]|0;
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 _memcpy(($34|0),($36|0),($39|0))|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 4|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$z>>2]|0;
 $44 = (($43) + ($42)|0);
 HEAP8[$44>>0] = 0;
 $45 = HEAP32[$z>>2]|0;
 _sqlite3Dequote($45);
 HEAP32[$i>>2] = 0;
 while(1) {
  $46 = HEAP32[$i>>2]|0;
  $47 = HEAP32[$p>>2]|0;
  $48 = ((($47)) + 34|0);
  $49 = HEAP16[$48>>1]|0;
  $50 = $49 << 16 >> 16;
  $51 = ($46|0)<($50|0);
  if (!($51)) {
   break;
  }
  $52 = HEAP32[$z>>2]|0;
  $53 = HEAP32[$i>>2]|0;
  $54 = HEAP32[$p>>2]|0;
  $55 = ((($54)) + 4|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + ($53<<4)|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (_sqlite3_stricmp($52,$58)|0);
  $60 = ($59|0)==(0);
  if ($60) {
   label = 8;
   break;
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = (($65) + 1)|0;
  HEAP32[$i>>2] = $66;
 }
 if ((label|0) == 8) {
  $61 = HEAP32[$0>>2]|0;
  $62 = HEAP32[$z>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $62;
  _sqlite3ErrorMsg($61,37072,$vararg_buffer1);
  $63 = HEAP32[$db>>2]|0;
  $64 = HEAP32[$z>>2]|0;
  _sqlite3DbFree($63,$64);
  STACKTOP = sp;return;
 }
 $67 = HEAP32[$p>>2]|0;
 $68 = ((($67)) + 34|0);
 $69 = HEAP16[$68>>1]|0;
 $70 = $69 << 16 >> 16;
 $71 = $70 & 7;
 $72 = ($71|0)==(0);
 do {
  if ($72) {
   $73 = HEAP32[$db>>2]|0;
   $74 = HEAP32[$p>>2]|0;
   $75 = ((($74)) + 4|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = HEAP32[$p>>2]|0;
   $78 = ((($77)) + 34|0);
   $79 = HEAP16[$78>>1]|0;
   $80 = $79 << 16 >> 16;
   $81 = (($80) + 8)|0;
   $82 = $81<<4;
   $83 = (_sqlite3DbRealloc($73,$76,$82,0)|0);
   HEAP32[$aNew>>2] = $83;
   $84 = HEAP32[$aNew>>2]|0;
   $85 = ($84|0)==(0|0);
   if (!($85)) {
    $88 = HEAP32[$aNew>>2]|0;
    $89 = HEAP32[$p>>2]|0;
    $90 = ((($89)) + 4|0);
    HEAP32[$90>>2] = $88;
    break;
   }
   $86 = HEAP32[$db>>2]|0;
   $87 = HEAP32[$z>>2]|0;
   _sqlite3DbFree($86,$87);
   STACKTOP = sp;return;
  }
 } while(0);
 $91 = HEAP32[$p>>2]|0;
 $92 = ((($91)) + 34|0);
 $93 = HEAP16[$92>>1]|0;
 $94 = $93 << 16 >> 16;
 $95 = HEAP32[$p>>2]|0;
 $96 = ((($95)) + 4|0);
 $97 = HEAP32[$96>>2]|0;
 $98 = (($97) + ($94<<4)|0);
 HEAP32[$pCol>>2] = $98;
 $99 = HEAP32[$pCol>>2]|0;
 ;HEAP32[$99>>2]=0|0;HEAP32[$99+4>>2]=0|0;HEAP32[$99+8>>2]=0|0;HEAP32[$99+12>>2]=0|0;
 $100 = HEAP32[$z>>2]|0;
 $101 = HEAP32[$pCol>>2]|0;
 HEAP32[$101>>2] = $100;
 $102 = HEAP32[$2>>2]|0;
 $103 = ((($102)) + 4|0);
 $104 = HEAP32[$103>>2]|0;
 $105 = ($104|0)==(0);
 if ($105) {
  $106 = HEAP32[$pCol>>2]|0;
  $107 = ((($106)) + 13|0);
  HEAP8[$107>>0] = 65;
  $108 = HEAP32[$pCol>>2]|0;
  $109 = ((($108)) + 14|0);
  HEAP8[$109>>0] = 1;
 } else {
  $110 = HEAP32[$z>>2]|0;
  $111 = HEAP32[$z>>2]|0;
  $112 = (_sqlite3Strlen30($111)|0);
  $113 = (($110) + ($112)|0);
  $114 = ((($113)) + 1|0);
  HEAP32[$zType>>2] = $114;
  $115 = HEAP32[$zType>>2]|0;
  $116 = HEAP32[$2>>2]|0;
  $117 = HEAP32[$116>>2]|0;
  $118 = HEAP32[$2>>2]|0;
  $119 = ((($118)) + 4|0);
  $120 = HEAP32[$119>>2]|0;
  _memcpy(($115|0),($117|0),($120|0))|0;
  $121 = HEAP32[$2>>2]|0;
  $122 = ((($121)) + 4|0);
  $123 = HEAP32[$122>>2]|0;
  $124 = HEAP32[$zType>>2]|0;
  $125 = (($124) + ($123)|0);
  HEAP8[$125>>0] = 0;
  $126 = HEAP32[$zType>>2]|0;
  _sqlite3Dequote($126);
  $127 = HEAP32[$zType>>2]|0;
  $128 = HEAP32[$pCol>>2]|0;
  $129 = ((($128)) + 14|0);
  $130 = (_sqlite3AffinityType($127,$129)|0);
  $131 = HEAP32[$pCol>>2]|0;
  $132 = ((($131)) + 13|0);
  HEAP8[$132>>0] = $130;
  $133 = HEAP32[$pCol>>2]|0;
  $134 = ((($133)) + 15|0);
  $135 = HEAP8[$134>>0]|0;
  $136 = $135&255;
  $137 = $136 | 4;
  $138 = $137&255;
  HEAP8[$134>>0] = $138;
 }
 $139 = HEAP32[$p>>2]|0;
 $140 = ((($139)) + 34|0);
 $141 = HEAP16[$140>>1]|0;
 $142 = (($141) + 1)<<16>>16;
 HEAP16[$140>>1] = $142;
 $143 = HEAP32[$0>>2]|0;
 $144 = ((($143)) + 84|0);
 $145 = ((($144)) + 4|0);
 HEAP32[$145>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3AddDefaultValue($pParse,$pSpan) {
 $pParse = $pParse|0;
 $pSpan = $pSpan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $pCol = 0, $vararg_buffer = 0, $x = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $p = sp + 64|0;
 $pCol = sp + 60|0;
 $db = sp + 56|0;
 $x = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSpan;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 440|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = ((($9)) + 34|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11 << 16 >> 16;
 $13 = (($12) - 1)|0;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + ($13<<4)|0);
 HEAP32[$pCol>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$db>>2]|0;
 $21 = ((($20)) + 148|0);
 $22 = ((($21)) + 5|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = (_sqlite3ExprIsConstantOrFunction($19,$23)|0);
 $25 = ($24|0)!=(0);
 if ($25) {
  $29 = HEAP32[$db>>2]|0;
  $30 = HEAP32[$pCol>>2]|0;
  $31 = ((($30)) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  _sqlite3ExprDelete($29,$32);
  dest=$x; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  HEAP8[$x>>0] = -95;
  $33 = HEAP32[$db>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 8|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = $39;
  $44 = $42;
  $45 = (($43) - ($44))|0;
  $46 = ($45|0)<(0);
  $47 = $46 << 31 >> 31;
  $48 = (_sqlite3DbStrNDup($33,$36,$45,$47)|0);
  $49 = ((($x)) + 8|0);
  HEAP32[$49>>2] = $48;
  $50 = HEAP32[$1>>2]|0;
  $51 = HEAP32[$50>>2]|0;
  $52 = ((($x)) + 12|0);
  HEAP32[$52>>2] = $51;
  $53 = ((($x)) + 4|0);
  HEAP32[$53>>2] = 4096;
  $54 = HEAP32[$db>>2]|0;
  $55 = (_sqlite3ExprDup($54,$x,1)|0);
  $56 = HEAP32[$pCol>>2]|0;
  $57 = ((($56)) + 4|0);
  HEAP32[$57>>2] = $55;
  $58 = HEAP32[$db>>2]|0;
  $59 = ((($x)) + 8|0);
  $60 = HEAP32[$59>>2]|0;
  _sqlite3DbFree($58,$60);
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 } else {
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$pCol>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$vararg_buffer>>2] = $28;
  _sqlite3ErrorMsg($26,37004,$vararg_buffer);
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = HEAP32[$62>>2]|0;
  _sqlite3ExprDelete($61,$63);
  STACKTOP = sp;return;
 }
}
function _sqlite3PExpr($pParse,$op,$pLeft,$pRight,$pToken) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 HEAP32[$4>>2] = $pToken;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(28);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 36|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0);
  if ($10) {
   $11 = HEAP32[$0>>2]|0;
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$2>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = (_sqlite3ExprAnd($12,$13,$14)|0);
   HEAP32[$p>>2] = $15;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 if ((label|0) == 4) {
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = $18 & 255;
  $20 = HEAP32[$4>>2]|0;
  $21 = (_sqlite3ExprAlloc($17,$19,$20,1)|0);
  HEAP32[$p>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$3>>2]|0;
  _sqlite3ExprAttachSubtrees($23,$24,$25,$26);
 }
 $27 = HEAP32[$p>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $33 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 24|0);
 $32 = HEAP32[$31>>2]|0;
 (_sqlite3ExprCheckHeight($29,$32)|0);
 $33 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _spanExpr($pOut,$pParse,$op,$t) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $t = $t|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $p = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($t)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (48 + ($6))|0;
 $8 = (($7) + 1)|0;
 $9 = (_sqlite3DbMallocRawNN($4,$8,0)|0);
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $61 = HEAP32[$p>>2]|0;
  $62 = HEAP32[$0>>2]|0;
  HEAP32[$62>>2] = $61;
  $63 = HEAP32[$t>>2]|0;
  $64 = HEAP32[$0>>2]|0;
  $65 = ((($64)) + 4|0);
  HEAP32[$65>>2] = $63;
  $66 = ((($t)) + 4|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = HEAP32[$t>>2]|0;
  $69 = (($68) + ($67)|0);
  $70 = HEAP32[$0>>2]|0;
  $71 = ((($70)) + 8|0);
  HEAP32[$71>>2] = $69;
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$p>>2]|0;
 dest=$12; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $13 = HEAP32[$2>>2]|0;
 $14 = $13&255;
 $15 = HEAP32[$p>>2]|0;
 HEAP8[$15>>0] = $14;
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 4|0);
 HEAP32[$17>>2] = 8388608;
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 34|0);
 HEAP16[$19>>1] = -1;
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 8|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 8|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$t>>2]|0;
 $28 = ((($t)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 _memcpy(($26|0),($27|0),($29|0))|0;
 $30 = ((($t)) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31)|0);
 HEAP8[$35>>0] = 0;
 $36 = HEAP32[$p>>2]|0;
 $37 = ((($36)) + 8|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP8[$38>>0]|0;
 $40 = $39&255;
 $41 = (18403 + ($40)|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = $43 & 128;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[$p>>2]|0;
  $47 = ((($46)) + 8|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP8[$48>>0]|0;
  $50 = $49 << 24 >> 24;
  $51 = ($50|0)==(34);
  if ($51) {
   $52 = HEAP32[$p>>2]|0;
   $53 = ((($52)) + 4|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = $54 | 64;
   HEAP32[$53>>2] = $55;
  }
  $56 = HEAP32[$p>>2]|0;
  $57 = ((($56)) + 8|0);
  $58 = HEAP32[$57>>2]|0;
  _sqlite3Dequote($58);
 }
 $59 = HEAP32[$p>>2]|0;
 $60 = ((($59)) + 24|0);
 HEAP32[$60>>2] = 1;
 $61 = HEAP32[$p>>2]|0;
 $62 = HEAP32[$0>>2]|0;
 HEAP32[$62>>2] = $61;
 $63 = HEAP32[$t>>2]|0;
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 4|0);
 HEAP32[$65>>2] = $63;
 $66 = ((($t)) + 4|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$t>>2]|0;
 $69 = (($68) + ($67)|0);
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 8|0);
 HEAP32[$71>>2] = $69;
 STACKTOP = sp;return;
}
function _sqlite3AddNotNull($pParse,$onError) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 440|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 34|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9 << 16 >> 16;
 $11 = ($10|0)<(1);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = $12&255;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 34|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16 << 16 >> 16;
 $18 = (($17) - 1)|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + ($18<<4)|0);
 $23 = ((($22)) + 12|0);
 HEAP8[$23>>0] = $13;
 STACKTOP = sp;return;
}
function _sqlite3AddPrimaryKey($pParse,$pList,$onError,$autoInc,$sortOrder) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $onError = $onError|0;
 $autoInc = $autoInc|0;
 $sortOrder = $sortOrder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCol = 0, $nTerm = 0, $or$cond = 0;
 var $or$cond3 = 0, $pCExpr = 0, $pCol = 0, $pTab = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $zCName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $pTab = sp + 36|0;
 $pCol = sp + 32|0;
 $iCol = sp + 28|0;
 $i = sp + 24|0;
 $nTerm = sp + 20|0;
 $pCExpr = sp + 16|0;
 $zCName = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $onError;
 HEAP32[$3>>2] = $autoInc;
 HEAP32[$4>>2] = $sortOrder;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 440|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTab>>2] = $7;
 HEAP32[$pCol>>2] = 0;
 HEAP32[$iCol>>2] = -1;
 $8 = HEAP32[$pTab>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $133 = HEAP32[$0>>2]|0;
  $134 = HEAP32[$133>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($134,$135);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pTab>>2]|0;
 $11 = ((($10)) + 42|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13 & 4;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$pTab>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$vararg_buffer>>2] = $18;
  _sqlite3ErrorMsg($16,36907,$vararg_buffer);
  $133 = HEAP32[$0>>2]|0;
  $134 = HEAP32[$133>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($134,$135);
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$pTab>>2]|0;
 $20 = ((($19)) + 42|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = $22 | 4;
 $24 = $23&255;
 HEAP8[$20>>0] = $24;
 $25 = HEAP32[$1>>2]|0;
 $26 = ($25|0)==(0|0);
 L8: do {
  if ($26) {
   $27 = HEAP32[$pTab>>2]|0;
   $28 = ((($27)) + 34|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = (($30) - 1)|0;
   HEAP32[$iCol>>2] = $31;
   $32 = HEAP32[$iCol>>2]|0;
   $33 = HEAP32[$pTab>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<4)|0);
   HEAP32[$pCol>>2] = $36;
   $37 = HEAP32[$pCol>>2]|0;
   $38 = ((($37)) + 15|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = $40 | 1;
   $42 = $41&255;
   HEAP8[$38>>0] = $42;
   HEAP32[$nTerm>>2] = 1;
  } else {
   $43 = HEAP32[$1>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   HEAP32[$nTerm>>2] = $44;
   HEAP32[$i>>2] = 0;
   while(1) {
    $45 = HEAP32[$i>>2]|0;
    $46 = HEAP32[$nTerm>>2]|0;
    $47 = ($45|0)<($46|0);
    if (!($47)) {
     break L8;
    }
    $48 = HEAP32[$i>>2]|0;
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (($51) + (($48*20)|0)|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (_sqlite3ExprSkipCollate($53)|0);
    HEAP32[$pCExpr>>2] = $54;
    $55 = HEAP32[$pCExpr>>2]|0;
    _sqlite3StringToId($55);
    $56 = HEAP32[$pCExpr>>2]|0;
    $57 = HEAP8[$56>>0]|0;
    $58 = $57&255;
    $59 = ($58|0)==(55);
    L14: do {
     if ($59) {
      $60 = HEAP32[$pCExpr>>2]|0;
      $61 = ((($60)) + 8|0);
      $62 = HEAP32[$61>>2]|0;
      HEAP32[$zCName>>2] = $62;
      HEAP32[$iCol>>2] = 0;
      while(1) {
       $63 = HEAP32[$iCol>>2]|0;
       $64 = HEAP32[$pTab>>2]|0;
       $65 = ((($64)) + 34|0);
       $66 = HEAP16[$65>>1]|0;
       $67 = $66 << 16 >> 16;
       $68 = ($63|0)<($67|0);
       if (!($68)) {
        break L14;
       }
       $69 = HEAP32[$zCName>>2]|0;
       $70 = HEAP32[$iCol>>2]|0;
       $71 = HEAP32[$pTab>>2]|0;
       $72 = ((($71)) + 4|0);
       $73 = HEAP32[$72>>2]|0;
       $74 = (($73) + ($70<<4)|0);
       $75 = HEAP32[$74>>2]|0;
       $76 = (_sqlite3StrICmp($69,$75)|0);
       $77 = ($76|0)==(0);
       $78 = HEAP32[$iCol>>2]|0;
       if ($77) {
        break;
       }
       $89 = (($78) + 1)|0;
       HEAP32[$iCol>>2] = $89;
      }
      $79 = HEAP32[$pTab>>2]|0;
      $80 = ((($79)) + 4|0);
      $81 = HEAP32[$80>>2]|0;
      $82 = (($81) + ($78<<4)|0);
      HEAP32[$pCol>>2] = $82;
      $83 = HEAP32[$pCol>>2]|0;
      $84 = ((($83)) + 15|0);
      $85 = HEAP8[$84>>0]|0;
      $86 = $85&255;
      $87 = $86 | 1;
      $88 = $87&255;
      HEAP8[$84>>0] = $88;
     }
    } while(0);
    $90 = HEAP32[$i>>2]|0;
    $91 = (($90) + 1)|0;
    HEAP32[$i>>2] = $91;
   }
  }
 } while(0);
 $92 = HEAP32[$nTerm>>2]|0;
 $93 = ($92|0)==(1);
 $94 = HEAP32[$pCol>>2]|0;
 $95 = ($94|0)!=(0|0);
 $or$cond = $93 & $95;
 if ($or$cond) {
  $96 = HEAP32[$pCol>>2]|0;
  $97 = (_sqlite3ColumnType($96,18402)|0);
  $98 = (_sqlite3StrICmp($97,18759)|0);
  $99 = ($98|0)==(0);
  $100 = HEAP32[$4>>2]|0;
  $101 = ($100|0)!=(1);
  $or$cond3 = $99 & $101;
  if ($or$cond3) {
   $102 = HEAP32[$iCol>>2]|0;
   $103 = $102&65535;
   $104 = HEAP32[$pTab>>2]|0;
   $105 = ((($104)) + 32|0);
   HEAP16[$105>>1] = $103;
   $106 = HEAP32[$2>>2]|0;
   $107 = $106&255;
   $108 = HEAP32[$pTab>>2]|0;
   $109 = ((($108)) + 43|0);
   HEAP8[$109>>0] = $107;
   $110 = HEAP32[$3>>2]|0;
   $111 = $110<<3;
   $112 = HEAP32[$pTab>>2]|0;
   $113 = ((($112)) + 42|0);
   $114 = HEAP8[$113>>0]|0;
   $115 = $114&255;
   $116 = $115 | $111;
   $117 = $116&255;
   HEAP8[$113>>0] = $117;
   $118 = HEAP32[$1>>2]|0;
   $119 = ($118|0)!=(0|0);
   if (!($119)) {
    $133 = HEAP32[$0>>2]|0;
    $134 = HEAP32[$133>>2]|0;
    $135 = HEAP32[$1>>2]|0;
    _sqlite3ExprListDelete($134,$135);
    STACKTOP = sp;return;
   }
   $120 = HEAP32[$1>>2]|0;
   $121 = ((($120)) + 4|0);
   $122 = HEAP32[$121>>2]|0;
   $123 = ((($122)) + 12|0);
   $124 = HEAP8[$123>>0]|0;
   $125 = HEAP32[$0>>2]|0;
   $126 = ((($125)) + 408|0);
   HEAP8[$126>>0] = $124;
   $133 = HEAP32[$0>>2]|0;
   $134 = HEAP32[$133>>2]|0;
   $135 = HEAP32[$1>>2]|0;
   _sqlite3ExprListDelete($134,$135);
   STACKTOP = sp;return;
  }
 }
 $127 = HEAP32[$3>>2]|0;
 $128 = ($127|0)!=(0);
 $129 = HEAP32[$0>>2]|0;
 if ($128) {
  _sqlite3ErrorMsg($129,36948,$vararg_buffer4);
  $133 = HEAP32[$0>>2]|0;
  $134 = HEAP32[$133>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($134,$135);
  STACKTOP = sp;return;
 } else {
  $130 = HEAP32[$1>>2]|0;
  $131 = HEAP32[$2>>2]|0;
  $132 = HEAP32[$4>>2]|0;
  _sqlite3CreateIndex($129,0,0,0,$130,$131,0,0,$132,0,2);
  HEAP32[$1>>2] = 0;
  $133 = HEAP32[$0>>2]|0;
  $134 = HEAP32[$133>>2]|0;
  $135 = HEAP32[$1>>2]|0;
  _sqlite3ExprListDelete($134,$135);
  STACKTOP = sp;return;
 }
}
function _sqlite3CreateIndex($pParse,$pName1,$pName2,$pTblName,$pList,$onError,$pStart,$pPIWhere,$sortOrder,$ifNotExist,$idxType) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pTblName = $pTblName|0;
 $pList = $pList|0;
 $onError = $onError|0;
 $pStart = $pStart|0;
 $pPIWhere = $pPIWhere|0;
 $sortOrder = $sortOrder|0;
 $ifNotExist = $ifNotExist|0;
 $idxType = $idxType|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0;
 var $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0;
 var $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0;
 var $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0;
 var $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0;
 var $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0;
 var $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0;
 var $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0;
 var $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0;
 var $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0;
 var $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0;
 var $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0;
 var $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0;
 var $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0;
 var $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0;
 var $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0;
 var $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0;
 var $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $db = 0, $i = 0, $iDb = 0, $iMem = 0, $j = 0, $k = 0, $n = 0, $n1 = 0, $nColl = 0, $nExtra = 0, $nExtraCol = 0, $nName = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $p = 0, $pCExpr = 0, $pCopy = 0, $pDb = 0;
 var $pExpr = 0, $pIdx = 0, $pIndex = 0, $pListItem = 0, $pLoop = 0, $pName = 0, $pOther = 0, $pPk = 0, $pTab = 0, $prevCol = 0, $requestedSortOrder = 0, $sFix = 0, $sortOrderMask = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer23 = 0;
 var $vararg_buffer25 = 0, $vararg_buffer28 = 0, $vararg_buffer33 = 0, $vararg_buffer41 = 0, $vararg_buffer6 = 0, $vararg_buffer9 = 0, $vararg_ptr22 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr36 = 0, $vararg_ptr37 = 0, $vararg_ptr38 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0, $x = 0, $z1 = 0, $z2 = 0, $zColl = 0, $zDb = 0, $zExtra = 0;
 var $zName = 0, $zStmt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 352|0;
 $vararg_buffer41 = sp + 112|0;
 $vararg_buffer33 = sp + 88|0;
 $vararg_buffer28 = sp + 72|0;
 $vararg_buffer25 = sp + 64|0;
 $vararg_buffer23 = sp + 56|0;
 $vararg_buffer19 = sp + 48|0;
 $vararg_buffer16 = sp + 40|0;
 $vararg_buffer13 = sp + 32|0;
 $vararg_buffer11 = sp + 24|0;
 $vararg_buffer9 = sp + 16|0;
 $vararg_buffer6 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 332|0;
 $1 = sp + 328|0;
 $2 = sp + 324|0;
 $3 = sp + 320|0;
 $4 = sp + 316|0;
 $5 = sp + 312|0;
 $6 = sp + 308|0;
 $7 = sp + 304|0;
 $8 = sp + 300|0;
 $9 = sp + 296|0;
 $10 = sp + 336|0;
 $pTab = sp + 292|0;
 $pIndex = sp + 288|0;
 $zName = sp + 284|0;
 $nName = sp + 280|0;
 $i = sp + 276|0;
 $j = sp + 272|0;
 $sFix = sp + 248|0;
 $sortOrderMask = sp + 240|0;
 $db = sp + 236|0;
 $pDb = sp + 232|0;
 $iDb = sp + 228|0;
 $pName = sp + 224|0;
 $pListItem = sp + 220|0;
 $nExtra = sp + 216|0;
 $nExtraCol = sp + 212|0;
 $zExtra = sp + 208|0;
 $pPk = sp + 204|0;
 $n = sp + 200|0;
 $pLoop = sp + 196|0;
 $zDb = sp + 192|0;
 $prevCol = sp + 184|0;
 $pExpr = sp + 180|0;
 $pCExpr = sp + 176|0;
 $requestedSortOrder = sp + 172|0;
 $zColl = sp + 168|0;
 $pCopy = sp + 164|0;
 $nColl = sp + 160|0;
 $x = sp + 156|0;
 $pIdx = sp + 152|0;
 $k = sp + 148|0;
 $z1 = sp + 144|0;
 $z2 = sp + 140|0;
 $p = sp + 136|0;
 $v = sp + 132|0;
 $zStmt = sp + 128|0;
 $iMem = sp + 124|0;
 $n1 = sp + 120|0;
 $pOther = sp + 116|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $pTblName;
 HEAP32[$4>>2] = $pList;
 HEAP32[$5>>2] = $onError;
 HEAP32[$6>>2] = $pStart;
 HEAP32[$7>>2] = $pPIWhere;
 HEAP32[$8>>2] = $sortOrder;
 HEAP32[$9>>2] = $ifNotExist;
 HEAP8[$10>>0] = $idxType;
 HEAP32[$pTab>>2] = 0;
 HEAP32[$pIndex>>2] = 0;
 HEAP32[$zName>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 HEAP32[$pName>>2] = 0;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$zExtra>>2] = 0;
 HEAP32[$pPk>>2] = 0;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 69|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(0);
 L1: do {
  if (!($17)) {
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 36|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)>(0);
   if (!($21)) {
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 410|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24&255;
    $26 = ($25|0)!=(0);
    if ($26) {
     $27 = HEAP8[$10>>0]|0;
     $28 = $27&255;
     $29 = ($28|0)!=(2);
     if ($29) {
      break;
     }
    }
    $30 = HEAP32[$0>>2]|0;
    $31 = (_sqlite3ReadSchema($30)|0);
    $32 = (0)!=($31|0);
    if (!($32)) {
     $33 = HEAP32[$3>>2]|0;
     $34 = ($33|0)!=(0|0);
     $35 = HEAP32[$0>>2]|0;
     if ($34) {
      $36 = HEAP32[$1>>2]|0;
      $37 = HEAP32[$2>>2]|0;
      $38 = (_sqlite3TwoPartName($35,$36,$37,$pName)|0);
      HEAP32[$iDb>>2] = $38;
      $39 = HEAP32[$iDb>>2]|0;
      $40 = ($39|0)<(0);
      if ($40) {
       break;
      }
      $41 = HEAP32[$db>>2]|0;
      $42 = ((($41)) + 148|0);
      $43 = ((($42)) + 5|0);
      $44 = HEAP8[$43>>0]|0;
      $45 = ($44<<24>>24)!=(0);
      if (!($45)) {
       $46 = HEAP32[$0>>2]|0;
       $47 = HEAP32[$3>>2]|0;
       $48 = (_sqlite3SrcListLookup($46,$47)|0);
       HEAP32[$pTab>>2] = $48;
       $49 = HEAP32[$2>>2]|0;
       $50 = ((($49)) + 4|0);
       $51 = HEAP32[$50>>2]|0;
       $52 = ($51|0)==(0);
       $53 = HEAP32[$pTab>>2]|0;
       $54 = ($53|0)!=(0|0);
       $or$cond = $52 & $54;
       if ($or$cond) {
        $55 = HEAP32[$pTab>>2]|0;
        $56 = ((($55)) + 64|0);
        $57 = HEAP32[$56>>2]|0;
        $58 = HEAP32[$db>>2]|0;
        $59 = ((($58)) + 16|0);
        $60 = HEAP32[$59>>2]|0;
        $61 = ((($60)) + 16|0);
        $62 = ((($61)) + 12|0);
        $63 = HEAP32[$62>>2]|0;
        $64 = ($57|0)==($63|0);
        if ($64) {
         HEAP32[$iDb>>2] = 1;
        }
       }
      }
      $65 = HEAP32[$0>>2]|0;
      $66 = HEAP32[$iDb>>2]|0;
      $67 = HEAP32[$pName>>2]|0;
      _sqlite3FixInit($sFix,$65,$66,33158,$67);
      $68 = HEAP32[$3>>2]|0;
      (_sqlite3FixSrcList($sFix,$68)|0);
      $69 = HEAP32[$0>>2]|0;
      $70 = HEAP32[$3>>2]|0;
      $71 = ((($70)) + 8|0);
      $72 = (_sqlite3LocateTableItem($69,0,$71)|0);
      HEAP32[$pTab>>2] = $72;
      $73 = HEAP32[$pTab>>2]|0;
      $74 = ($73|0)==(0|0);
      if ($74) {
       break;
      }
      $75 = HEAP32[$iDb>>2]|0;
      $76 = ($75|0)==(1);
      if ($76) {
       $77 = HEAP32[$iDb>>2]|0;
       $78 = HEAP32[$db>>2]|0;
       $79 = ((($78)) + 16|0);
       $80 = HEAP32[$79>>2]|0;
       $81 = (($80) + ($77<<4)|0);
       $82 = ((($81)) + 12|0);
       $83 = HEAP32[$82>>2]|0;
       $84 = HEAP32[$pTab>>2]|0;
       $85 = ((($84)) + 64|0);
       $86 = HEAP32[$85>>2]|0;
       $87 = ($83|0)!=($86|0);
       if ($87) {
        $88 = HEAP32[$0>>2]|0;
        $89 = HEAP32[$pTab>>2]|0;
        $90 = HEAP32[$89>>2]|0;
        HEAP32[$vararg_buffer>>2] = $90;
        _sqlite3ErrorMsg($88,36473,$vararg_buffer);
        break;
       }
      }
      $91 = HEAP32[$pTab>>2]|0;
      $92 = ((($91)) + 42|0);
      $93 = HEAP8[$92>>0]|0;
      $94 = $93&255;
      $95 = $94 & 32;
      $96 = ($95|0)==(0);
      if (!($96)) {
       $97 = HEAP32[$pTab>>2]|0;
       $98 = (_sqlite3PrimaryKeyIndex($97)|0);
       HEAP32[$pPk>>2] = $98;
      }
     } else {
      $99 = ((($35)) + 440|0);
      $100 = HEAP32[$99>>2]|0;
      HEAP32[$pTab>>2] = $100;
      $101 = HEAP32[$pTab>>2]|0;
      $102 = ($101|0)!=(0|0);
      if (!($102)) {
       break;
      }
      $103 = HEAP32[$db>>2]|0;
      $104 = HEAP32[$pTab>>2]|0;
      $105 = ((($104)) + 64|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = (_sqlite3SchemaToIndex($103,$106)|0);
      HEAP32[$iDb>>2] = $107;
     }
     $108 = HEAP32[$iDb>>2]|0;
     $109 = HEAP32[$db>>2]|0;
     $110 = ((($109)) + 16|0);
     $111 = HEAP32[$110>>2]|0;
     $112 = (($111) + ($108<<4)|0);
     HEAP32[$pDb>>2] = $112;
     $113 = HEAP32[$pTab>>2]|0;
     $114 = HEAP32[$113>>2]|0;
     $115 = (_sqlite3_strnicmp($114,27844,7)|0);
     $116 = ($115|0)==(0);
     if ($116) {
      $117 = HEAP32[$db>>2]|0;
      $118 = ((($117)) + 148|0);
      $119 = ((($118)) + 5|0);
      $120 = HEAP8[$119>>0]|0;
      $121 = $120&255;
      $122 = ($121|0)==(0);
      if ($122) {
       $123 = HEAP32[$pTab>>2]|0;
       $124 = HEAP32[$123>>2]|0;
       $125 = ((($124)) + 7|0);
       $126 = (_sqlite3_strnicmp($125,36523,9)|0);
       $127 = ($126|0)!=(0);
       if ($127) {
        $128 = HEAP32[$0>>2]|0;
        $129 = HEAP32[$pTab>>2]|0;
        $130 = HEAP32[$129>>2]|0;
        HEAP32[$vararg_buffer6>>2] = $130;
        _sqlite3ErrorMsg($128,36533,$vararg_buffer6);
        break;
       }
      }
     }
     $131 = HEAP32[$pTab>>2]|0;
     $132 = ((($131)) + 12|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = ($133|0)!=(0|0);
     if ($134) {
      $135 = HEAP32[$0>>2]|0;
      _sqlite3ErrorMsg($135,36561,$vararg_buffer9);
      break;
     }
     $136 = HEAP32[$pTab>>2]|0;
     $137 = ((($136)) + 42|0);
     $138 = HEAP8[$137>>0]|0;
     $139 = $138&255;
     $140 = $139 & 16;
     $141 = ($140|0)!=(0);
     if ($141) {
      $142 = HEAP32[$0>>2]|0;
      _sqlite3ErrorMsg($142,36586,$vararg_buffer11);
      break;
     }
     $143 = HEAP32[$pName>>2]|0;
     $144 = ($143|0)!=(0|0);
     if ($144) {
      $145 = HEAP32[$db>>2]|0;
      $146 = HEAP32[$pName>>2]|0;
      $147 = (_sqlite3NameFromToken($145,$146)|0);
      HEAP32[$zName>>2] = $147;
      $148 = HEAP32[$zName>>2]|0;
      $149 = ($148|0)==(0|0);
      if ($149) {
       break;
      }
      $150 = HEAP32[$0>>2]|0;
      $151 = HEAP32[$zName>>2]|0;
      $152 = (_sqlite3CheckObjectName($150,$151)|0);
      $153 = (0)!=($152|0);
      if ($153) {
       break;
      }
      $154 = HEAP32[$db>>2]|0;
      $155 = ((($154)) + 148|0);
      $156 = ((($155)) + 5|0);
      $157 = HEAP8[$156>>0]|0;
      $158 = ($157<<24>>24)!=(0);
      if (!($158)) {
       $159 = HEAP32[$db>>2]|0;
       $160 = HEAP32[$zName>>2]|0;
       $161 = (_sqlite3FindTable($159,$160,0)|0);
       $162 = ($161|0)!=(0|0);
       if ($162) {
        $163 = HEAP32[$0>>2]|0;
        $164 = HEAP32[$zName>>2]|0;
        HEAP32[$vararg_buffer13>>2] = $164;
        _sqlite3ErrorMsg($163,36620,$vararg_buffer13);
        break;
       }
      }
      $165 = HEAP32[$db>>2]|0;
      $166 = HEAP32[$zName>>2]|0;
      $167 = HEAP32[$pDb>>2]|0;
      $168 = HEAP32[$167>>2]|0;
      $169 = (_sqlite3FindIndex($165,$166,$168)|0);
      $170 = ($169|0)!=(0|0);
      if ($170) {
       $171 = HEAP32[$9>>2]|0;
       $172 = ($171|0)!=(0);
       $173 = HEAP32[$0>>2]|0;
       if ($172) {
        $175 = HEAP32[$iDb>>2]|0;
        _sqlite3CodeVerifySchema($173,$175);
        break;
       } else {
        $174 = HEAP32[$zName>>2]|0;
        HEAP32[$vararg_buffer16>>2] = $174;
        _sqlite3ErrorMsg($173,36654,$vararg_buffer16);
        break;
       }
      }
     } else {
      $176 = HEAP32[$pTab>>2]|0;
      $177 = ((($176)) + 8|0);
      $178 = HEAP32[$177>>2]|0;
      HEAP32[$pLoop>>2] = $178;
      HEAP32[$n>>2] = 1;
      while(1) {
       $179 = HEAP32[$pLoop>>2]|0;
       $180 = ($179|0)!=(0|0);
       if (!($180)) {
        break;
       }
       $181 = HEAP32[$pLoop>>2]|0;
       $182 = ((($181)) + 20|0);
       $183 = HEAP32[$182>>2]|0;
       HEAP32[$pLoop>>2] = $183;
       $184 = HEAP32[$n>>2]|0;
       $185 = (($184) + 1)|0;
       HEAP32[$n>>2] = $185;
      }
      $186 = HEAP32[$db>>2]|0;
      $187 = HEAP32[$pTab>>2]|0;
      $188 = HEAP32[$187>>2]|0;
      $189 = HEAP32[$n>>2]|0;
      HEAP32[$vararg_buffer19>>2] = $188;
      $vararg_ptr22 = ((($vararg_buffer19)) + 4|0);
      HEAP32[$vararg_ptr22>>2] = $189;
      $190 = (_sqlite3MPrintf($186,36678,$vararg_buffer19)|0);
      HEAP32[$zName>>2] = $190;
      $191 = HEAP32[$zName>>2]|0;
      $192 = ($191|0)==(0|0);
      if ($192) {
       break;
      }
      $193 = HEAP32[$0>>2]|0;
      $194 = ((($193)) + 410|0);
      $195 = HEAP8[$194>>0]|0;
      $196 = ($195<<24>>24)!=(0);
      if ($196) {
       $197 = HEAP32[$zName>>2]|0;
       $198 = ((($197)) + 7|0);
       $199 = HEAP8[$198>>0]|0;
       $200 = (($199) + 1)<<24>>24;
       HEAP8[$198>>0] = $200;
      }
     }
     $201 = HEAP32[$pDb>>2]|0;
     $202 = HEAP32[$201>>2]|0;
     HEAP32[$zDb>>2] = $202;
     $203 = HEAP32[$0>>2]|0;
     $204 = HEAP32[$iDb>>2]|0;
     $205 = ($204|0)==(1);
     $206 = $205 ? 19905 : 19924;
     $207 = HEAP32[$zDb>>2]|0;
     $208 = (_sqlite3AuthCheck($203,18,$206,0,$207)|0);
     $209 = ($208|0)!=(0);
     if (!($209)) {
      HEAP32[$i>>2] = 1;
      $210 = HEAP32[$iDb>>2]|0;
      $211 = ($210|0)==(1);
      $$ = $211 ? 3 : 1;
      HEAP32[$i>>2] = $$;
      $212 = HEAP32[$0>>2]|0;
      $213 = HEAP32[$i>>2]|0;
      $214 = HEAP32[$zName>>2]|0;
      $215 = HEAP32[$pTab>>2]|0;
      $216 = HEAP32[$215>>2]|0;
      $217 = HEAP32[$zDb>>2]|0;
      $218 = (_sqlite3AuthCheck($212,$213,$214,$216,$217)|0);
      $219 = ($218|0)!=(0);
      if (!($219)) {
       $220 = HEAP32[$4>>2]|0;
       $221 = ($220|0)==(0|0);
       if ($221) {
        $222 = HEAP32[$pTab>>2]|0;
        $223 = ((($222)) + 34|0);
        $224 = HEAP16[$223>>1]|0;
        $225 = $224 << 16 >> 16;
        $226 = (($225) - 1)|0;
        $227 = HEAP32[$pTab>>2]|0;
        $228 = ((($227)) + 4|0);
        $229 = HEAP32[$228>>2]|0;
        $230 = (($229) + ($226<<4)|0);
        $231 = HEAP32[$230>>2]|0;
        _sqlite3TokenInit($prevCol,$231);
        $232 = HEAP32[$0>>2]|0;
        $233 = HEAP32[$db>>2]|0;
        $234 = (_sqlite3ExprAlloc($233,55,$prevCol,0)|0);
        $235 = (_sqlite3ExprListAppend($232,0,$234)|0);
        HEAP32[$4>>2] = $235;
        $236 = HEAP32[$4>>2]|0;
        $237 = ($236|0)==(0|0);
        if ($237) {
         break;
        }
        $238 = HEAP32[$4>>2]|0;
        $239 = HEAP32[$8>>2]|0;
        _sqlite3ExprListSetSortOrder($238,$239);
       } else {
        $240 = HEAP32[$0>>2]|0;
        $241 = HEAP32[$4>>2]|0;
        _sqlite3ExprListCheckLength($240,$241,33158);
       }
       HEAP32[$i>>2] = 0;
       while(1) {
        $242 = HEAP32[$i>>2]|0;
        $243 = HEAP32[$4>>2]|0;
        $244 = HEAP32[$243>>2]|0;
        $245 = ($242|0)<($244|0);
        if (!($245)) {
         break;
        }
        $246 = HEAP32[$i>>2]|0;
        $247 = HEAP32[$4>>2]|0;
        $248 = ((($247)) + 4|0);
        $249 = HEAP32[$248>>2]|0;
        $250 = (($249) + (($246*20)|0)|0);
        $251 = HEAP32[$250>>2]|0;
        HEAP32[$pExpr>>2] = $251;
        $252 = HEAP32[$pExpr>>2]|0;
        $253 = HEAP8[$252>>0]|0;
        $254 = $253&255;
        $255 = ($254|0)==(53);
        if ($255) {
         $256 = HEAP32[$pExpr>>2]|0;
         $257 = ((($256)) + 8|0);
         $258 = HEAP32[$257>>2]|0;
         $259 = (_sqlite3Strlen30($258)|0);
         $260 = (1 + ($259))|0;
         $261 = HEAP32[$nExtra>>2]|0;
         $262 = (($261) + ($260))|0;
         HEAP32[$nExtra>>2] = $262;
        }
        $263 = HEAP32[$i>>2]|0;
        $264 = (($263) + 1)|0;
        HEAP32[$i>>2] = $264;
       }
       $265 = HEAP32[$zName>>2]|0;
       $266 = (_sqlite3Strlen30($265)|0);
       HEAP32[$nName>>2] = $266;
       $267 = HEAP32[$pPk>>2]|0;
       $268 = ($267|0)!=(0|0);
       if ($268) {
        $269 = HEAP32[$pPk>>2]|0;
        $270 = ((($269)) + 50|0);
        $271 = HEAP16[$270>>1]|0;
        $272 = $271&65535;
        $273 = $272;
       } else {
        $273 = 1;
       }
       HEAP32[$nExtraCol>>2] = $273;
       $274 = HEAP32[$db>>2]|0;
       $275 = HEAP32[$4>>2]|0;
       $276 = HEAP32[$275>>2]|0;
       $277 = HEAP32[$nExtraCol>>2]|0;
       $278 = (($276) + ($277))|0;
       $279 = $278&65535;
       $280 = HEAP32[$nName>>2]|0;
       $281 = HEAP32[$nExtra>>2]|0;
       $282 = (($280) + ($281))|0;
       $283 = (($282) + 1)|0;
       $284 = (_sqlite3AllocateIndexObject($274,$279,$283,$zExtra)|0);
       HEAP32[$pIndex>>2] = $284;
       $285 = HEAP32[$db>>2]|0;
       $286 = ((($285)) + 69|0);
       $287 = HEAP8[$286>>0]|0;
       $288 = ($287<<24>>24)!=(0);
       if (!($288)) {
        $289 = HEAP32[$zExtra>>2]|0;
        $290 = HEAP32[$pIndex>>2]|0;
        HEAP32[$290>>2] = $289;
        $291 = HEAP32[$nName>>2]|0;
        $292 = (($291) + 1)|0;
        $293 = HEAP32[$zExtra>>2]|0;
        $294 = (($293) + ($292)|0);
        HEAP32[$zExtra>>2] = $294;
        $295 = HEAP32[$pIndex>>2]|0;
        $296 = HEAP32[$295>>2]|0;
        $297 = HEAP32[$zName>>2]|0;
        $298 = HEAP32[$nName>>2]|0;
        $299 = (($298) + 1)|0;
        _memcpy(($296|0),($297|0),($299|0))|0;
        $300 = HEAP32[$pTab>>2]|0;
        $301 = HEAP32[$pIndex>>2]|0;
        $302 = ((($301)) + 12|0);
        HEAP32[$302>>2] = $300;
        $303 = HEAP32[$5>>2]|0;
        $304 = $303&255;
        $305 = HEAP32[$pIndex>>2]|0;
        $306 = ((($305)) + 54|0);
        HEAP8[$306>>0] = $304;
        $307 = HEAP32[$5>>2]|0;
        $308 = ($307|0)!=(0);
        $309 = $308&1;
        $310 = HEAP32[$pIndex>>2]|0;
        $311 = ((($310)) + 55|0);
        $312 = $309&255;
        $313 = HEAP8[$311>>0]|0;
        $314 = $312 & 1;
        $315 = ($314 << 3)&255;
        $316 = $313 & -9;
        $317 = $316 | $315;
        HEAP8[$311>>0] = $317;
        $318 = HEAP8[$10>>0]|0;
        $319 = $318&255;
        $320 = HEAP32[$pIndex>>2]|0;
        $321 = ((($320)) + 55|0);
        $322 = $319&255;
        $323 = HEAP8[$321>>0]|0;
        $324 = $322 & 3;
        $325 = $323 & -4;
        $326 = $325 | $324;
        HEAP8[$321>>0] = $326;
        $327 = HEAP32[$iDb>>2]|0;
        $328 = HEAP32[$db>>2]|0;
        $329 = ((($328)) + 16|0);
        $330 = HEAP32[$329>>2]|0;
        $331 = (($330) + ($327<<4)|0);
        $332 = ((($331)) + 12|0);
        $333 = HEAP32[$332>>2]|0;
        $334 = HEAP32[$pIndex>>2]|0;
        $335 = ((($334)) + 24|0);
        HEAP32[$335>>2] = $333;
        $336 = HEAP32[$4>>2]|0;
        $337 = HEAP32[$336>>2]|0;
        $338 = $337&65535;
        $339 = HEAP32[$pIndex>>2]|0;
        $340 = ((($339)) + 50|0);
        HEAP16[$340>>1] = $338;
        $341 = HEAP32[$7>>2]|0;
        $342 = ($341|0)!=(0|0);
        if ($342) {
         $343 = HEAP32[$0>>2]|0;
         $344 = HEAP32[$pTab>>2]|0;
         $345 = HEAP32[$7>>2]|0;
         _sqlite3ResolveSelfReference($343,$344,2,$345,0);
         $346 = HEAP32[$7>>2]|0;
         $347 = HEAP32[$pIndex>>2]|0;
         $348 = ((($347)) + 36|0);
         HEAP32[$348>>2] = $346;
         HEAP32[$7>>2] = 0;
        }
        $349 = HEAP32[$pDb>>2]|0;
        $350 = ((($349)) + 12|0);
        $351 = HEAP32[$350>>2]|0;
        $352 = ((($351)) + 76|0);
        $353 = HEAP8[$352>>0]|0;
        $354 = $353&255;
        $355 = ($354|0)>=(4);
        if ($355) {
         HEAP32[$sortOrderMask>>2] = -1;
        } else {
         HEAP32[$sortOrderMask>>2] = 0;
        }
        HEAP32[$i>>2] = 0;
        $356 = HEAP32[$4>>2]|0;
        $357 = ((($356)) + 4|0);
        $358 = HEAP32[$357>>2]|0;
        HEAP32[$pListItem>>2] = $358;
        while(1) {
         $359 = HEAP32[$i>>2]|0;
         $360 = HEAP32[$4>>2]|0;
         $361 = HEAP32[$360>>2]|0;
         $362 = ($359|0)<($361|0);
         if (!($362)) {
          break;
         }
         $363 = HEAP32[$pListItem>>2]|0;
         $364 = HEAP32[$363>>2]|0;
         _sqlite3StringToId($364);
         $365 = HEAP32[$0>>2]|0;
         $366 = HEAP32[$pTab>>2]|0;
         $367 = HEAP32[$pListItem>>2]|0;
         $368 = HEAP32[$367>>2]|0;
         _sqlite3ResolveSelfReference($365,$366,32,$368,0);
         $369 = HEAP32[$0>>2]|0;
         $370 = ((($369)) + 36|0);
         $371 = HEAP32[$370>>2]|0;
         $372 = ($371|0)!=(0);
         if ($372) {
          break L1;
         }
         $373 = HEAP32[$pListItem>>2]|0;
         $374 = HEAP32[$373>>2]|0;
         $375 = (_sqlite3ExprSkipCollate($374)|0);
         HEAP32[$pCExpr>>2] = $375;
         $376 = HEAP32[$pCExpr>>2]|0;
         $377 = HEAP8[$376>>0]|0;
         $378 = $377&255;
         $379 = ($378|0)!=(152);
         if ($379) {
          $380 = HEAP32[$pTab>>2]|0;
          $381 = HEAP32[$0>>2]|0;
          $382 = ((($381)) + 440|0);
          $383 = HEAP32[$382>>2]|0;
          $384 = ($380|0)==($383|0);
          if ($384) {
           label = 68;
           break;
          }
          $386 = HEAP32[$pIndex>>2]|0;
          $387 = ((($386)) + 40|0);
          $388 = HEAP32[$387>>2]|0;
          $389 = ($388|0)==(0|0);
          do {
           if ($389) {
            $390 = HEAP32[$db>>2]|0;
            $391 = HEAP32[$4>>2]|0;
            $392 = (_sqlite3ExprListDup($390,$391,0)|0);
            HEAP32[$pCopy>>2] = $392;
            $393 = HEAP32[$pCopy>>2]|0;
            $394 = HEAP32[$pIndex>>2]|0;
            $395 = ((($394)) + 40|0);
            HEAP32[$395>>2] = $393;
            $396 = HEAP32[$db>>2]|0;
            $397 = ((($396)) + 69|0);
            $398 = HEAP8[$397>>0]|0;
            $399 = ($398<<24>>24)!=(0);
            if ($399) {
             break;
            }
            $400 = HEAP32[$i>>2]|0;
            $401 = HEAP32[$pCopy>>2]|0;
            $402 = ((($401)) + 4|0);
            $403 = HEAP32[$402>>2]|0;
            $404 = (($403) + (($400*20)|0)|0);
            HEAP32[$pListItem>>2] = $404;
           }
          } while(0);
          HEAP32[$j>>2] = -2;
          $405 = HEAP32[$i>>2]|0;
          $406 = HEAP32[$pIndex>>2]|0;
          $407 = ((($406)) + 4|0);
          $408 = HEAP32[$407>>2]|0;
          $409 = (($408) + ($405<<1)|0);
          HEAP16[$409>>1] = -2;
          $410 = HEAP32[$pIndex>>2]|0;
          $411 = ((($410)) + 55|0);
          $412 = HEAP8[$411>>0]|0;
          $413 = $412 & -9;
          HEAP8[$411>>0] = $413;
         } else {
          $414 = HEAP32[$pCExpr>>2]|0;
          $415 = ((($414)) + 32|0);
          $416 = HEAP16[$415>>1]|0;
          $417 = $416 << 16 >> 16;
          HEAP32[$j>>2] = $417;
          $418 = HEAP32[$j>>2]|0;
          $419 = ($418|0)<(0);
          do {
           if ($419) {
            $420 = HEAP32[$pTab>>2]|0;
            $421 = ((($420)) + 32|0);
            $422 = HEAP16[$421>>1]|0;
            $423 = $422 << 16 >> 16;
            HEAP32[$j>>2] = $423;
           } else {
            $424 = HEAP32[$j>>2]|0;
            $425 = HEAP32[$pTab>>2]|0;
            $426 = ((($425)) + 4|0);
            $427 = HEAP32[$426>>2]|0;
            $428 = (($427) + ($424<<4)|0);
            $429 = ((($428)) + 12|0);
            $430 = HEAP8[$429>>0]|0;
            $431 = $430&255;
            $432 = ($431|0)==(0);
            if (!($432)) {
             break;
            }
            $433 = HEAP32[$pIndex>>2]|0;
            $434 = ((($433)) + 55|0);
            $435 = HEAP8[$434>>0]|0;
            $436 = $435 & -9;
            HEAP8[$434>>0] = $436;
           }
          } while(0);
          $437 = HEAP32[$j>>2]|0;
          $438 = $437&65535;
          $439 = HEAP32[$i>>2]|0;
          $440 = HEAP32[$pIndex>>2]|0;
          $441 = ((($440)) + 4|0);
          $442 = HEAP32[$441>>2]|0;
          $443 = (($442) + ($439<<1)|0);
          HEAP16[$443>>1] = $438;
         }
         HEAP32[$zColl>>2] = 0;
         $444 = HEAP32[$pListItem>>2]|0;
         $445 = HEAP32[$444>>2]|0;
         $446 = HEAP8[$445>>0]|0;
         $447 = $446&255;
         $448 = ($447|0)==(53);
         do {
          if ($448) {
           $449 = HEAP32[$pListItem>>2]|0;
           $450 = HEAP32[$449>>2]|0;
           $451 = ((($450)) + 8|0);
           $452 = HEAP32[$451>>2]|0;
           HEAP32[$zColl>>2] = $452;
           $453 = HEAP32[$zColl>>2]|0;
           $454 = (_sqlite3Strlen30($453)|0);
           $455 = (($454) + 1)|0;
           HEAP32[$nColl>>2] = $455;
           $456 = HEAP32[$zExtra>>2]|0;
           $457 = HEAP32[$zColl>>2]|0;
           $458 = HEAP32[$nColl>>2]|0;
           _memcpy(($456|0),($457|0),($458|0))|0;
           $459 = HEAP32[$zExtra>>2]|0;
           HEAP32[$zColl>>2] = $459;
           $460 = HEAP32[$nColl>>2]|0;
           $461 = HEAP32[$zExtra>>2]|0;
           $462 = (($461) + ($460)|0);
           HEAP32[$zExtra>>2] = $462;
           $463 = HEAP32[$nColl>>2]|0;
           $464 = HEAP32[$nExtra>>2]|0;
           $465 = (($464) - ($463))|0;
           HEAP32[$nExtra>>2] = $465;
          } else {
           $466 = HEAP32[$j>>2]|0;
           $467 = ($466|0)>=(0);
           if (!($467)) {
            break;
           }
           $468 = HEAP32[$j>>2]|0;
           $469 = HEAP32[$pTab>>2]|0;
           $470 = ((($469)) + 4|0);
           $471 = HEAP32[$470>>2]|0;
           $472 = (($471) + ($468<<4)|0);
           $473 = ((($472)) + 8|0);
           $474 = HEAP32[$473>>2]|0;
           HEAP32[$zColl>>2] = $474;
          }
         } while(0);
         $475 = HEAP32[$zColl>>2]|0;
         $476 = ($475|0)!=(0|0);
         if (!($476)) {
          HEAP32[$zColl>>2] = 34834;
         }
         $477 = HEAP32[$db>>2]|0;
         $478 = ((($477)) + 148|0);
         $479 = ((($478)) + 5|0);
         $480 = HEAP8[$479>>0]|0;
         $481 = ($480<<24>>24)!=(0);
         if (!($481)) {
          $482 = HEAP32[$0>>2]|0;
          $483 = HEAP32[$zColl>>2]|0;
          $484 = (_sqlite3LocateCollSeq($482,$483)|0);
          $485 = ($484|0)!=(0|0);
          if (!($485)) {
           break L1;
          }
         }
         $486 = HEAP32[$zColl>>2]|0;
         $487 = HEAP32[$i>>2]|0;
         $488 = HEAP32[$pIndex>>2]|0;
         $489 = ((($488)) + 32|0);
         $490 = HEAP32[$489>>2]|0;
         $491 = (($490) + ($487<<2)|0);
         HEAP32[$491>>2] = $486;
         $492 = HEAP32[$pListItem>>2]|0;
         $493 = ((($492)) + 12|0);
         $494 = HEAP8[$493>>0]|0;
         $495 = $494&255;
         $496 = HEAP32[$sortOrderMask>>2]|0;
         $497 = $495 & $496;
         HEAP32[$requestedSortOrder>>2] = $497;
         $498 = HEAP32[$requestedSortOrder>>2]|0;
         $499 = $498&255;
         $500 = HEAP32[$i>>2]|0;
         $501 = HEAP32[$pIndex>>2]|0;
         $502 = ((($501)) + 28|0);
         $503 = HEAP32[$502>>2]|0;
         $504 = (($503) + ($500)|0);
         HEAP8[$504>>0] = $499;
         $505 = HEAP32[$i>>2]|0;
         $506 = (($505) + 1)|0;
         HEAP32[$i>>2] = $506;
         $507 = HEAP32[$pListItem>>2]|0;
         $508 = ((($507)) + 20|0);
         HEAP32[$pListItem>>2] = $508;
        }
        if ((label|0) == 68) {
         $385 = HEAP32[$0>>2]|0;
         _sqlite3ErrorMsg($385,36701,$vararg_buffer23);
         break;
        }
        $509 = HEAP32[$pPk>>2]|0;
        $510 = ($509|0)!=(0|0);
        L113: do {
         if ($510) {
          HEAP32[$j>>2] = 0;
          while(1) {
           $511 = HEAP32[$j>>2]|0;
           $512 = HEAP32[$pPk>>2]|0;
           $513 = ((($512)) + 50|0);
           $514 = HEAP16[$513>>1]|0;
           $515 = $514&65535;
           $516 = ($511|0)<($515|0);
           if (!($516)) {
            break L113;
           }
           $517 = HEAP32[$j>>2]|0;
           $518 = HEAP32[$pPk>>2]|0;
           $519 = ((($518)) + 4|0);
           $520 = HEAP32[$519>>2]|0;
           $521 = (($520) + ($517<<1)|0);
           $522 = HEAP16[$521>>1]|0;
           $523 = $522 << 16 >> 16;
           HEAP32[$x>>2] = $523;
           $524 = HEAP32[$pIndex>>2]|0;
           $525 = ((($524)) + 4|0);
           $526 = HEAP32[$525>>2]|0;
           $527 = HEAP32[$pIndex>>2]|0;
           $528 = ((($527)) + 50|0);
           $529 = HEAP16[$528>>1]|0;
           $530 = $529&65535;
           $531 = HEAP32[$x>>2]|0;
           $532 = (_hasColumn($526,$530,$531)|0);
           $533 = ($532|0)!=(0);
           if ($533) {
            $534 = HEAP32[$pIndex>>2]|0;
            $535 = ((($534)) + 52|0);
            $536 = HEAP16[$535>>1]|0;
            $537 = (($536) + -1)<<16>>16;
            HEAP16[$535>>1] = $537;
           } else {
            $538 = HEAP32[$x>>2]|0;
            $539 = $538&65535;
            $540 = HEAP32[$i>>2]|0;
            $541 = HEAP32[$pIndex>>2]|0;
            $542 = ((($541)) + 4|0);
            $543 = HEAP32[$542>>2]|0;
            $544 = (($543) + ($540<<1)|0);
            HEAP16[$544>>1] = $539;
            $545 = HEAP32[$j>>2]|0;
            $546 = HEAP32[$pPk>>2]|0;
            $547 = ((($546)) + 32|0);
            $548 = HEAP32[$547>>2]|0;
            $549 = (($548) + ($545<<2)|0);
            $550 = HEAP32[$549>>2]|0;
            $551 = HEAP32[$i>>2]|0;
            $552 = HEAP32[$pIndex>>2]|0;
            $553 = ((($552)) + 32|0);
            $554 = HEAP32[$553>>2]|0;
            $555 = (($554) + ($551<<2)|0);
            HEAP32[$555>>2] = $550;
            $556 = HEAP32[$j>>2]|0;
            $557 = HEAP32[$pPk>>2]|0;
            $558 = ((($557)) + 28|0);
            $559 = HEAP32[$558>>2]|0;
            $560 = (($559) + ($556)|0);
            $561 = HEAP8[$560>>0]|0;
            $562 = HEAP32[$i>>2]|0;
            $563 = HEAP32[$pIndex>>2]|0;
            $564 = ((($563)) + 28|0);
            $565 = HEAP32[$564>>2]|0;
            $566 = (($565) + ($562)|0);
            HEAP8[$566>>0] = $561;
            $567 = HEAP32[$i>>2]|0;
            $568 = (($567) + 1)|0;
            HEAP32[$i>>2] = $568;
           }
           $569 = HEAP32[$j>>2]|0;
           $570 = (($569) + 1)|0;
           HEAP32[$j>>2] = $570;
          }
         } else {
          $571 = HEAP32[$i>>2]|0;
          $572 = HEAP32[$pIndex>>2]|0;
          $573 = ((($572)) + 4|0);
          $574 = HEAP32[$573>>2]|0;
          $575 = (($574) + ($571<<1)|0);
          HEAP16[$575>>1] = -1;
          $576 = HEAP32[$i>>2]|0;
          $577 = HEAP32[$pIndex>>2]|0;
          $578 = ((($577)) + 32|0);
          $579 = HEAP32[$578>>2]|0;
          $580 = (($579) + ($576<<2)|0);
          HEAP32[$580>>2] = 34834;
         }
        } while(0);
        $581 = HEAP32[$pIndex>>2]|0;
        _sqlite3DefaultRowEst($581);
        $582 = HEAP32[$0>>2]|0;
        $583 = ((($582)) + 440|0);
        $584 = HEAP32[$583>>2]|0;
        $585 = ($584|0)==(0|0);
        if ($585) {
         $586 = HEAP32[$pIndex>>2]|0;
         _estimateIndexWidth($586);
        }
        $587 = HEAP32[$3>>2]|0;
        $588 = ($587|0)!=(0|0);
        L127: do {
         if ($588) {
          $589 = HEAP32[$pIndex>>2]|0;
          $590 = ((($589)) + 52|0);
          $591 = HEAP16[$590>>1]|0;
          $592 = $591&65535;
          $593 = HEAP32[$pTab>>2]|0;
          $594 = ((($593)) + 34|0);
          $595 = HEAP16[$594>>1]|0;
          $596 = $595 << 16 >> 16;
          $597 = ($592|0)>=($596|0);
          if (!($597)) {
           break;
          }
          $598 = HEAP32[$pIndex>>2]|0;
          $599 = ((($598)) + 55|0);
          $600 = HEAP8[$599>>0]|0;
          $601 = $600 & -33;
          $602 = $601 | 32;
          HEAP8[$599>>0] = $602;
          HEAP32[$j>>2] = 0;
          while(1) {
           $603 = HEAP32[$j>>2]|0;
           $604 = HEAP32[$pTab>>2]|0;
           $605 = ((($604)) + 34|0);
           $606 = HEAP16[$605>>1]|0;
           $607 = $606 << 16 >> 16;
           $608 = ($603|0)<($607|0);
           if (!($608)) {
            break L127;
           }
           $609 = HEAP32[$j>>2]|0;
           $610 = HEAP32[$pTab>>2]|0;
           $611 = ((($610)) + 32|0);
           $612 = HEAP16[$611>>1]|0;
           $613 = $612 << 16 >> 16;
           $614 = ($609|0)==($613|0);
           if (!($614)) {
            $615 = HEAP32[$pIndex>>2]|0;
            $616 = HEAP32[$j>>2]|0;
            $617 = $616&65535;
            $618 = (_sqlite3ColumnOfIndex($615,$617)|0);
            $619 = $618 << 16 >> 16;
            $620 = ($619|0)>=(0);
            if (!($620)) {
             break;
            }
           }
           $625 = HEAP32[$j>>2]|0;
           $626 = (($625) + 1)|0;
           HEAP32[$j>>2] = $626;
          }
          $621 = HEAP32[$pIndex>>2]|0;
          $622 = ((($621)) + 55|0);
          $623 = HEAP8[$622>>0]|0;
          $624 = $623 & -33;
          HEAP8[$622>>0] = $624;
         }
        } while(0);
        $627 = HEAP32[$pTab>>2]|0;
        $628 = HEAP32[$0>>2]|0;
        $629 = ((($628)) + 440|0);
        $630 = HEAP32[$629>>2]|0;
        $631 = ($627|0)==($630|0);
        L138: do {
         if ($631) {
          $632 = HEAP32[$pTab>>2]|0;
          $633 = ((($632)) + 8|0);
          $634 = HEAP32[$633>>2]|0;
          HEAP32[$pIdx>>2] = $634;
          while(1) {
           $635 = HEAP32[$pIdx>>2]|0;
           $636 = ($635|0)!=(0|0);
           if (!($636)) {
            break L138;
           }
           $637 = HEAP32[$pIdx>>2]|0;
           $638 = ((($637)) + 50|0);
           $639 = HEAP16[$638>>1]|0;
           $640 = $639&65535;
           $641 = HEAP32[$pIndex>>2]|0;
           $642 = ((($641)) + 50|0);
           $643 = HEAP16[$642>>1]|0;
           $644 = $643&65535;
           $645 = ($640|0)!=($644|0);
           if (!($645)) {
            HEAP32[$k>>2] = 0;
            while(1) {
             $646 = HEAP32[$k>>2]|0;
             $647 = HEAP32[$pIdx>>2]|0;
             $648 = ((($647)) + 50|0);
             $649 = HEAP16[$648>>1]|0;
             $650 = $649&65535;
             $651 = ($646|0)<($650|0);
             if (!($651)) {
              break;
             }
             $652 = HEAP32[$k>>2]|0;
             $653 = HEAP32[$pIdx>>2]|0;
             $654 = ((($653)) + 4|0);
             $655 = HEAP32[$654>>2]|0;
             $656 = (($655) + ($652<<1)|0);
             $657 = HEAP16[$656>>1]|0;
             $658 = $657 << 16 >> 16;
             $659 = HEAP32[$k>>2]|0;
             $660 = HEAP32[$pIndex>>2]|0;
             $661 = ((($660)) + 4|0);
             $662 = HEAP32[$661>>2]|0;
             $663 = (($662) + ($659<<1)|0);
             $664 = HEAP16[$663>>1]|0;
             $665 = $664 << 16 >> 16;
             $666 = ($658|0)!=($665|0);
             if ($666) {
              break;
             }
             $667 = HEAP32[$k>>2]|0;
             $668 = HEAP32[$pIdx>>2]|0;
             $669 = ((($668)) + 32|0);
             $670 = HEAP32[$669>>2]|0;
             $671 = (($670) + ($667<<2)|0);
             $672 = HEAP32[$671>>2]|0;
             HEAP32[$z1>>2] = $672;
             $673 = HEAP32[$k>>2]|0;
             $674 = HEAP32[$pIndex>>2]|0;
             $675 = ((($674)) + 32|0);
             $676 = HEAP32[$675>>2]|0;
             $677 = (($676) + ($673<<2)|0);
             $678 = HEAP32[$677>>2]|0;
             HEAP32[$z2>>2] = $678;
             $679 = HEAP32[$z1>>2]|0;
             $680 = HEAP32[$z2>>2]|0;
             $681 = (_sqlite3StrICmp($679,$680)|0);
             $682 = ($681|0)!=(0);
             if ($682) {
              break;
             }
             $683 = HEAP32[$k>>2]|0;
             $684 = (($683) + 1)|0;
             HEAP32[$k>>2] = $684;
            }
            $685 = HEAP32[$k>>2]|0;
            $686 = HEAP32[$pIdx>>2]|0;
            $687 = ((($686)) + 50|0);
            $688 = HEAP16[$687>>1]|0;
            $689 = $688&65535;
            $690 = ($685|0)==($689|0);
            if ($690) {
             break;
            }
           }
           $733 = HEAP32[$pIdx>>2]|0;
           $734 = ((($733)) + 20|0);
           $735 = HEAP32[$734>>2]|0;
           HEAP32[$pIdx>>2] = $735;
          }
          $691 = HEAP32[$pIdx>>2]|0;
          $692 = ((($691)) + 54|0);
          $693 = HEAP8[$692>>0]|0;
          $694 = $693&255;
          $695 = HEAP32[$pIndex>>2]|0;
          $696 = ((($695)) + 54|0);
          $697 = HEAP8[$696>>0]|0;
          $698 = $697&255;
          $699 = ($694|0)!=($698|0);
          do {
           if ($699) {
            $700 = HEAP32[$pIdx>>2]|0;
            $701 = ((($700)) + 54|0);
            $702 = HEAP8[$701>>0]|0;
            $703 = $702&255;
            $704 = ($703|0)==(10);
            do {
             if (!($704)) {
              $705 = HEAP32[$pIndex>>2]|0;
              $706 = ((($705)) + 54|0);
              $707 = HEAP8[$706>>0]|0;
              $708 = $707&255;
              $709 = ($708|0)==(10);
              if ($709) {
               break;
              }
              $710 = HEAP32[$0>>2]|0;
              HEAP32[$vararg_buffer25>>2] = 0;
              _sqlite3ErrorMsg($710,36762,$vararg_buffer25);
             }
            } while(0);
            $711 = HEAP32[$pIdx>>2]|0;
            $712 = ((($711)) + 54|0);
            $713 = HEAP8[$712>>0]|0;
            $714 = $713&255;
            $715 = ($714|0)==(10);
            if (!($715)) {
             break;
            }
            $716 = HEAP32[$pIndex>>2]|0;
            $717 = ((($716)) + 54|0);
            $718 = HEAP8[$717>>0]|0;
            $719 = HEAP32[$pIdx>>2]|0;
            $720 = ((($719)) + 54|0);
            HEAP8[$720>>0] = $718;
           }
          } while(0);
          $721 = HEAP8[$10>>0]|0;
          $722 = $721&255;
          $723 = ($722|0)==(2);
          if (!($723)) {
           break L1;
          }
          $724 = HEAP8[$10>>0]|0;
          $725 = $724&255;
          $726 = HEAP32[$pIdx>>2]|0;
          $727 = ((($726)) + 55|0);
          $728 = $725&255;
          $729 = HEAP8[$727>>0]|0;
          $730 = $728 & 3;
          $731 = $729 & -4;
          $732 = $731 | $730;
          HEAP8[$727>>0] = $732;
          break L1;
         }
        } while(0);
        $736 = HEAP32[$db>>2]|0;
        $737 = ((($736)) + 148|0);
        $738 = ((($737)) + 5|0);
        $739 = HEAP8[$738>>0]|0;
        $740 = ($739<<24>>24)!=(0);
        do {
         if ($740) {
          $741 = HEAP32[$pIndex>>2]|0;
          $742 = ((($741)) + 24|0);
          $743 = HEAP32[$742>>2]|0;
          $744 = ((($743)) + 24|0);
          $745 = HEAP32[$pIndex>>2]|0;
          $746 = HEAP32[$745>>2]|0;
          $747 = HEAP32[$pIndex>>2]|0;
          $748 = (_sqlite3HashInsert($744,$746,$747)|0);
          HEAP32[$p>>2] = $748;
          $749 = HEAP32[$p>>2]|0;
          $750 = ($749|0)!=(0|0);
          $751 = HEAP32[$db>>2]|0;
          if ($750) {
           _sqlite3OomFault($751);
           break L1;
          }
          $752 = ((($751)) + 24|0);
          $753 = HEAP32[$752>>2]|0;
          $754 = $753 | 2;
          HEAP32[$752>>2] = $754;
          $755 = HEAP32[$3>>2]|0;
          $756 = ($755|0)!=(0|0);
          if (!($756)) {
           break;
          }
          $757 = HEAP32[$db>>2]|0;
          $758 = ((($757)) + 148|0);
          $759 = HEAP32[$758>>2]|0;
          $760 = HEAP32[$pIndex>>2]|0;
          $761 = ((($760)) + 44|0);
          HEAP32[$761>>2] = $759;
         } else {
          $762 = HEAP32[$pTab>>2]|0;
          $763 = ((($762)) + 42|0);
          $764 = HEAP8[$763>>0]|0;
          $765 = $764&255;
          $766 = $765 & 32;
          $767 = ($766|0)==(0);
          $768 = HEAP32[$3>>2]|0;
          $769 = ($768|0)!=(0|0);
          $or$cond3 = $767 | $769;
          if (!($or$cond3)) {
           break;
          }
          $770 = HEAP32[$0>>2]|0;
          $771 = ((($770)) + 44|0);
          $772 = HEAP32[$771>>2]|0;
          $773 = (($772) + 1)|0;
          HEAP32[$771>>2] = $773;
          HEAP32[$iMem>>2] = $773;
          $774 = HEAP32[$0>>2]|0;
          $775 = (_sqlite3GetVdbe($774)|0);
          HEAP32[$v>>2] = $775;
          $776 = HEAP32[$v>>2]|0;
          $777 = ($776|0)==(0|0);
          if ($777) {
           break L1;
          }
          $778 = HEAP32[$0>>2]|0;
          $779 = HEAP32[$iDb>>2]|0;
          _sqlite3BeginWriteOperation($778,1,$779);
          $780 = HEAP32[$v>>2]|0;
          $781 = (_sqlite3VdbeAddOp0($780,161)|0);
          $782 = HEAP32[$pIndex>>2]|0;
          $783 = ((($782)) + 44|0);
          HEAP32[$783>>2] = $781;
          $784 = HEAP32[$v>>2]|0;
          $785 = HEAP32[$iDb>>2]|0;
          $786 = HEAP32[$iMem>>2]|0;
          (_sqlite3VdbeAddOp2($784,134,$785,$786)|0);
          $787 = HEAP32[$6>>2]|0;
          $788 = ($787|0)!=(0|0);
          if ($788) {
           $789 = HEAP32[$0>>2]|0;
           $790 = ((($789)) + 392|0);
           $791 = HEAP32[$790>>2]|0;
           $792 = HEAP32[$pName>>2]|0;
           $793 = HEAP32[$792>>2]|0;
           $794 = $791;
           $795 = $793;
           $796 = (($794) - ($795))|0;
           $797 = HEAP32[$0>>2]|0;
           $798 = ((($797)) + 392|0);
           $799 = ((($798)) + 4|0);
           $800 = HEAP32[$799>>2]|0;
           $801 = (($796) + ($800))|0;
           HEAP32[$n1>>2] = $801;
           $802 = HEAP32[$n1>>2]|0;
           $803 = (($802) - 1)|0;
           $804 = HEAP32[$pName>>2]|0;
           $805 = HEAP32[$804>>2]|0;
           $806 = (($805) + ($803)|0);
           $807 = HEAP8[$806>>0]|0;
           $808 = $807 << 24 >> 24;
           $809 = ($808|0)==(59);
           if ($809) {
            $810 = HEAP32[$n1>>2]|0;
            $811 = (($810) + -1)|0;
            HEAP32[$n1>>2] = $811;
           }
           $812 = HEAP32[$db>>2]|0;
           $813 = HEAP32[$5>>2]|0;
           $814 = ($813|0)==(0);
           $815 = $814 ? 18402 : 36804;
           $816 = HEAP32[$n1>>2]|0;
           $817 = HEAP32[$pName>>2]|0;
           $818 = HEAP32[$817>>2]|0;
           HEAP32[$vararg_buffer28>>2] = $815;
           $vararg_ptr31 = ((($vararg_buffer28)) + 4|0);
           HEAP32[$vararg_ptr31>>2] = $816;
           $vararg_ptr32 = ((($vararg_buffer28)) + 8|0);
           HEAP32[$vararg_ptr32>>2] = $818;
           $819 = (_sqlite3MPrintf($812,36812,$vararg_buffer28)|0);
           HEAP32[$zStmt>>2] = $819;
          } else {
           HEAP32[$zStmt>>2] = 0;
          }
          $820 = HEAP32[$0>>2]|0;
          $821 = HEAP32[$iDb>>2]|0;
          $822 = HEAP32[$db>>2]|0;
          $823 = ((($822)) + 16|0);
          $824 = HEAP32[$823>>2]|0;
          $825 = (($824) + ($821<<4)|0);
          $826 = HEAP32[$825>>2]|0;
          $827 = HEAP32[$iDb>>2]|0;
          $828 = ($827|0)==(1);
          $829 = $828 ? 19905 : 19924;
          $830 = HEAP32[$pIndex>>2]|0;
          $831 = HEAP32[$830>>2]|0;
          $832 = HEAP32[$pTab>>2]|0;
          $833 = HEAP32[$832>>2]|0;
          $834 = HEAP32[$iMem>>2]|0;
          $835 = HEAP32[$zStmt>>2]|0;
          HEAP32[$vararg_buffer33>>2] = $826;
          $vararg_ptr36 = ((($vararg_buffer33)) + 4|0);
          HEAP32[$vararg_ptr36>>2] = $829;
          $vararg_ptr37 = ((($vararg_buffer33)) + 8|0);
          HEAP32[$vararg_ptr37>>2] = $831;
          $vararg_ptr38 = ((($vararg_buffer33)) + 12|0);
          HEAP32[$vararg_ptr38>>2] = $833;
          $vararg_ptr39 = ((($vararg_buffer33)) + 16|0);
          HEAP32[$vararg_ptr39>>2] = $834;
          $vararg_ptr40 = ((($vararg_buffer33)) + 20|0);
          HEAP32[$vararg_ptr40>>2] = $835;
          _sqlite3NestedParse($820,36832,$vararg_buffer33);
          $836 = HEAP32[$db>>2]|0;
          $837 = HEAP32[$zStmt>>2]|0;
          _sqlite3DbFree($836,$837);
          $838 = HEAP32[$3>>2]|0;
          $839 = ($838|0)!=(0|0);
          if ($839) {
           $840 = HEAP32[$0>>2]|0;
           $841 = HEAP32[$pIndex>>2]|0;
           $842 = HEAP32[$iMem>>2]|0;
           _sqlite3RefillIndex($840,$841,$842);
           $843 = HEAP32[$0>>2]|0;
           $844 = HEAP32[$iDb>>2]|0;
           _sqlite3ChangeCookie($843,$844);
           $845 = HEAP32[$v>>2]|0;
           $846 = HEAP32[$iDb>>2]|0;
           $847 = HEAP32[$db>>2]|0;
           $848 = HEAP32[$pIndex>>2]|0;
           $849 = HEAP32[$848>>2]|0;
           HEAP32[$vararg_buffer41>>2] = $849;
           $850 = (_sqlite3MPrintf($847,36880,$vararg_buffer41)|0);
           _sqlite3VdbeAddParseSchemaOp($845,$846,$850);
           $851 = HEAP32[$v>>2]|0;
           (_sqlite3VdbeAddOp0($851,150)|0);
          }
          $852 = HEAP32[$v>>2]|0;
          $853 = HEAP32[$pIndex>>2]|0;
          $854 = ((($853)) + 44|0);
          $855 = HEAP32[$854>>2]|0;
          _sqlite3VdbeJumpHere($852,$855);
         }
        } while(0);
        $856 = HEAP32[$db>>2]|0;
        $857 = ((($856)) + 148|0);
        $858 = ((($857)) + 5|0);
        $859 = HEAP8[$858>>0]|0;
        $860 = $859&255;
        $861 = ($860|0)!=(0);
        $862 = HEAP32[$3>>2]|0;
        $863 = ($862|0)==(0|0);
        $or$cond5 = $861 | $863;
        if (!($or$cond5)) {
         break;
        }
        $864 = HEAP32[$5>>2]|0;
        $865 = ($864|0)!=(5);
        do {
         if ($865) {
          label = 143;
         } else {
          $866 = HEAP32[$pTab>>2]|0;
          $867 = ((($866)) + 8|0);
          $868 = HEAP32[$867>>2]|0;
          $869 = ($868|0)==(0|0);
          if ($869) {
           label = 143;
           break;
          }
          $870 = HEAP32[$pTab>>2]|0;
          $871 = ((($870)) + 8|0);
          $872 = HEAP32[$871>>2]|0;
          $873 = ((($872)) + 54|0);
          $874 = HEAP8[$873>>0]|0;
          $875 = $874&255;
          $876 = ($875|0)==(5);
          if ($876) {
           label = 143;
           break;
          }
          $885 = HEAP32[$pTab>>2]|0;
          $886 = ((($885)) + 8|0);
          $887 = HEAP32[$886>>2]|0;
          HEAP32[$pOther>>2] = $887;
          while(1) {
           $888 = HEAP32[$pOther>>2]|0;
           $889 = ((($888)) + 20|0);
           $890 = HEAP32[$889>>2]|0;
           $891 = ($890|0)!=(0|0);
           if ($891) {
            $892 = HEAP32[$pOther>>2]|0;
            $893 = ((($892)) + 20|0);
            $894 = HEAP32[$893>>2]|0;
            $895 = ((($894)) + 54|0);
            $896 = HEAP8[$895>>0]|0;
            $897 = $896&255;
            $898 = ($897|0)!=(5);
            $919 = $898;
           } else {
            $919 = 0;
           }
           $899 = HEAP32[$pOther>>2]|0;
           $900 = ((($899)) + 20|0);
           $901 = HEAP32[$900>>2]|0;
           if (!($919)) {
            break;
           }
           HEAP32[$pOther>>2] = $901;
          }
          $902 = HEAP32[$pIndex>>2]|0;
          $903 = ((($902)) + 20|0);
          HEAP32[$903>>2] = $901;
          $904 = HEAP32[$pIndex>>2]|0;
          $905 = HEAP32[$pOther>>2]|0;
          $906 = ((($905)) + 20|0);
          HEAP32[$906>>2] = $904;
         }
        } while(0);
        if ((label|0) == 143) {
         $877 = HEAP32[$pTab>>2]|0;
         $878 = ((($877)) + 8|0);
         $879 = HEAP32[$878>>2]|0;
         $880 = HEAP32[$pIndex>>2]|0;
         $881 = ((($880)) + 20|0);
         HEAP32[$881>>2] = $879;
         $882 = HEAP32[$pIndex>>2]|0;
         $883 = HEAP32[$pTab>>2]|0;
         $884 = ((($883)) + 8|0);
         HEAP32[$884>>2] = $882;
        }
        HEAP32[$pIndex>>2] = 0;
       }
      }
     }
    }
   }
  }
 } while(0);
 $907 = HEAP32[$pIndex>>2]|0;
 $908 = ($907|0)!=(0|0);
 if (!($908)) {
  $911 = HEAP32[$db>>2]|0;
  $912 = HEAP32[$7>>2]|0;
  _sqlite3ExprDelete($911,$912);
  $913 = HEAP32[$db>>2]|0;
  $914 = HEAP32[$4>>2]|0;
  _sqlite3ExprListDelete($913,$914);
  $915 = HEAP32[$db>>2]|0;
  $916 = HEAP32[$3>>2]|0;
  _sqlite3SrcListDelete($915,$916);
  $917 = HEAP32[$db>>2]|0;
  $918 = HEAP32[$zName>>2]|0;
  _sqlite3DbFree($917,$918);
  STACKTOP = sp;return;
 }
 $909 = HEAP32[$db>>2]|0;
 $910 = HEAP32[$pIndex>>2]|0;
 _freeIndex($909,$910);
 $911 = HEAP32[$db>>2]|0;
 $912 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($911,$912);
 $913 = HEAP32[$db>>2]|0;
 $914 = HEAP32[$4>>2]|0;
 _sqlite3ExprListDelete($913,$914);
 $915 = HEAP32[$db>>2]|0;
 $916 = HEAP32[$3>>2]|0;
 _sqlite3SrcListDelete($915,$916);
 $917 = HEAP32[$db>>2]|0;
 $918 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($917,$918);
 STACKTOP = sp;return;
}
function _sqlite3AddCheckConstraint($pParse,$pCheckExpr) {
 $pParse = $pParse|0;
 $pCheckExpr = $pCheckExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pTab = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pCheckExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 440|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 410|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  if (!($12)) {
   $13 = HEAP32[$db>>2]|0;
   $14 = ((($13)) + 148|0);
   $15 = ((($14)) + 4|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = $16&255;
   $18 = HEAP32[$db>>2]|0;
   $19 = ((($18)) + 16|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (($20) + ($17<<4)|0);
   $22 = ((($21)) + 4|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3BtreeIsReadonly($23)|0);
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$0>>2]|0;
    $27 = HEAP32[$pTab>>2]|0;
    $28 = ((($27)) + 24|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$1>>2]|0;
    $31 = (_sqlite3ExprListAppend($26,$29,$30)|0);
    $32 = HEAP32[$pTab>>2]|0;
    $33 = ((($32)) + 24|0);
    HEAP32[$33>>2] = $31;
    $34 = HEAP32[$0>>2]|0;
    $35 = ((($34)) + 84|0);
    $36 = ((($35)) + 4|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)!=(0);
    if (!($38)) {
     STACKTOP = sp;return;
    }
    $39 = HEAP32[$0>>2]|0;
    $40 = HEAP32[$pTab>>2]|0;
    $41 = ((($40)) + 24|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$0>>2]|0;
    $44 = ((($43)) + 84|0);
    _sqlite3ExprListSetName($39,$42,$44,1);
    STACKTOP = sp;return;
   }
  }
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 _sqlite3ExprDelete($46,$47);
 STACKTOP = sp;return;
}
function _sqlite3CreateForeignKey($pParse,$pFromCol,$pTo,$pToCol,$flags) {
 $pParse = $pParse|0;
 $pFromCol = $pFromCol|0;
 $pTo = $pTo|0;
 $pToCol = $pToCol|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $j = 0, $n = 0;
 var $nByte = 0, $nCol = 0, $p = 0, $pFKey = 0, $pNextTo = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer4 = 0, $vararg_ptr1 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $db = sp + 60|0;
 $pFKey = sp + 56|0;
 $pNextTo = sp + 52|0;
 $p = sp + 48|0;
 $nByte = sp + 44|0;
 $i = sp + 40|0;
 $nCol = sp + 36|0;
 $z = sp + 32|0;
 $iCol = sp + 28|0;
 $j = sp + 24|0;
 $n = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pFromCol;
 HEAP32[$2>>2] = $pTo;
 HEAP32[$3>>2] = $pToCol;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP32[$pFKey>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 440|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = ($10|0)==(0|0);
 L1: do {
  if (!($11)) {
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 410|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)!=(0);
   if (!($16)) {
    $17 = HEAP32[$1>>2]|0;
    $18 = ($17|0)==(0|0);
    if ($18) {
     $19 = HEAP32[$p>>2]|0;
     $20 = ((($19)) + 34|0);
     $21 = HEAP16[$20>>1]|0;
     $22 = $21 << 16 >> 16;
     $23 = (($22) - 1)|0;
     HEAP32[$iCol>>2] = $23;
     $24 = HEAP32[$iCol>>2]|0;
     $25 = ($24|0)<(0);
     if ($25) {
      break;
     }
     $26 = HEAP32[$3>>2]|0;
     $27 = ($26|0)!=(0|0);
     if ($27) {
      $28 = HEAP32[$3>>2]|0;
      $29 = HEAP32[$28>>2]|0;
      $30 = ($29|0)!=(1);
      if ($30) {
       $31 = HEAP32[$0>>2]|0;
       $32 = HEAP32[$iCol>>2]|0;
       $33 = HEAP32[$p>>2]|0;
       $34 = ((($33)) + 4|0);
       $35 = HEAP32[$34>>2]|0;
       $36 = (($35) + ($32<<4)|0);
       $37 = HEAP32[$36>>2]|0;
       $38 = HEAP32[$2>>2]|0;
       HEAP32[$vararg_buffer>>2] = $37;
       $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
       HEAP32[$vararg_ptr1>>2] = $38;
       _sqlite3ErrorMsg($31,36270,$vararg_buffer);
       break;
      }
     }
     HEAP32[$nCol>>2] = 1;
    } else {
     $39 = HEAP32[$3>>2]|0;
     $40 = ($39|0)!=(0|0);
     if ($40) {
      $41 = HEAP32[$3>>2]|0;
      $42 = HEAP32[$41>>2]|0;
      $43 = HEAP32[$1>>2]|0;
      $44 = HEAP32[$43>>2]|0;
      $45 = ($42|0)!=($44|0);
      if ($45) {
       $46 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($46,36333,$vararg_buffer2);
       break;
      }
     }
     $47 = HEAP32[$1>>2]|0;
     $48 = HEAP32[$47>>2]|0;
     HEAP32[$nCol>>2] = $48;
    }
    $49 = HEAP32[$nCol>>2]|0;
    $50 = (($49) - 1)|0;
    $51 = $50<<3;
    $52 = (44 + ($51))|0;
    $53 = HEAP32[$2>>2]|0;
    $54 = ((($53)) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($52) + ($55))|0;
    $57 = (($56) + 1)|0;
    HEAP32[$nByte>>2] = $57;
    $58 = HEAP32[$3>>2]|0;
    $59 = ($58|0)!=(0|0);
    L17: do {
     if ($59) {
      HEAP32[$i>>2] = 0;
      while(1) {
       $60 = HEAP32[$i>>2]|0;
       $61 = HEAP32[$3>>2]|0;
       $62 = HEAP32[$61>>2]|0;
       $63 = ($60|0)<($62|0);
       if (!($63)) {
        break L17;
       }
       $64 = HEAP32[$i>>2]|0;
       $65 = HEAP32[$3>>2]|0;
       $66 = ((($65)) + 4|0);
       $67 = HEAP32[$66>>2]|0;
       $68 = (($67) + (($64*20)|0)|0);
       $69 = ((($68)) + 4|0);
       $70 = HEAP32[$69>>2]|0;
       $71 = (_sqlite3Strlen30($70)|0);
       $72 = (($71) + 1)|0;
       $73 = HEAP32[$nByte>>2]|0;
       $74 = (($73) + ($72))|0;
       HEAP32[$nByte>>2] = $74;
       $75 = HEAP32[$i>>2]|0;
       $76 = (($75) + 1)|0;
       HEAP32[$i>>2] = $76;
      }
     }
    } while(0);
    $77 = HEAP32[$db>>2]|0;
    $78 = HEAP32[$nByte>>2]|0;
    $79 = ($78|0)<(0);
    $80 = $79 << 31 >> 31;
    $81 = (_sqlite3DbMallocZero($77,$78,$80)|0);
    HEAP32[$pFKey>>2] = $81;
    $82 = HEAP32[$pFKey>>2]|0;
    $83 = ($82|0)==(0|0);
    if (!($83)) {
     $84 = HEAP32[$p>>2]|0;
     $85 = HEAP32[$pFKey>>2]|0;
     HEAP32[$85>>2] = $84;
     $86 = HEAP32[$p>>2]|0;
     $87 = ((($86)) + 16|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = HEAP32[$pFKey>>2]|0;
     $90 = ((($89)) + 4|0);
     HEAP32[$90>>2] = $88;
     $91 = HEAP32[$nCol>>2]|0;
     $92 = HEAP32[$pFKey>>2]|0;
     $93 = ((($92)) + 36|0);
     $94 = (($93) + ($91<<3)|0);
     HEAP32[$z>>2] = $94;
     $95 = HEAP32[$z>>2]|0;
     $96 = HEAP32[$pFKey>>2]|0;
     $97 = ((($96)) + 8|0);
     HEAP32[$97>>2] = $95;
     $98 = HEAP32[$z>>2]|0;
     $99 = HEAP32[$2>>2]|0;
     $100 = HEAP32[$99>>2]|0;
     $101 = HEAP32[$2>>2]|0;
     $102 = ((($101)) + 4|0);
     $103 = HEAP32[$102>>2]|0;
     _memcpy(($98|0),($100|0),($103|0))|0;
     $104 = HEAP32[$2>>2]|0;
     $105 = ((($104)) + 4|0);
     $106 = HEAP32[$105>>2]|0;
     $107 = HEAP32[$z>>2]|0;
     $108 = (($107) + ($106)|0);
     HEAP8[$108>>0] = 0;
     $109 = HEAP32[$z>>2]|0;
     _sqlite3Dequote($109);
     $110 = HEAP32[$2>>2]|0;
     $111 = ((($110)) + 4|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (($112) + 1)|0;
     $114 = HEAP32[$z>>2]|0;
     $115 = (($114) + ($113)|0);
     HEAP32[$z>>2] = $115;
     $116 = HEAP32[$nCol>>2]|0;
     $117 = HEAP32[$pFKey>>2]|0;
     $118 = ((($117)) + 20|0);
     HEAP32[$118>>2] = $116;
     $119 = HEAP32[$1>>2]|0;
     $120 = ($119|0)==(0|0);
     L24: do {
      if ($120) {
       $121 = HEAP32[$p>>2]|0;
       $122 = ((($121)) + 34|0);
       $123 = HEAP16[$122>>1]|0;
       $124 = $123 << 16 >> 16;
       $125 = (($124) - 1)|0;
       $126 = HEAP32[$pFKey>>2]|0;
       $127 = ((($126)) + 36|0);
       HEAP32[$127>>2] = $125;
      } else {
       HEAP32[$i>>2] = 0;
       while(1) {
        $128 = HEAP32[$i>>2]|0;
        $129 = HEAP32[$nCol>>2]|0;
        $130 = ($128|0)<($129|0);
        if (!($130)) {
         break L24;
        }
        HEAP32[$j>>2] = 0;
        while(1) {
         $131 = HEAP32[$j>>2]|0;
         $132 = HEAP32[$p>>2]|0;
         $133 = ((($132)) + 34|0);
         $134 = HEAP16[$133>>1]|0;
         $135 = $134 << 16 >> 16;
         $136 = ($131|0)<($135|0);
         if (!($136)) {
          break;
         }
         $137 = HEAP32[$j>>2]|0;
         $138 = HEAP32[$p>>2]|0;
         $139 = ((($138)) + 4|0);
         $140 = HEAP32[$139>>2]|0;
         $141 = (($140) + ($137<<4)|0);
         $142 = HEAP32[$141>>2]|0;
         $143 = HEAP32[$i>>2]|0;
         $144 = HEAP32[$1>>2]|0;
         $145 = ((($144)) + 4|0);
         $146 = HEAP32[$145>>2]|0;
         $147 = (($146) + (($143*20)|0)|0);
         $148 = ((($147)) + 4|0);
         $149 = HEAP32[$148>>2]|0;
         $150 = (_sqlite3StrICmp($142,$149)|0);
         $151 = ($150|0)==(0);
         $152 = HEAP32[$j>>2]|0;
         if ($151) {
          label = 25;
          break;
         }
         $157 = (($152) + 1)|0;
         HEAP32[$j>>2] = $157;
        }
        if ((label|0) == 25) {
         label = 0;
         $153 = HEAP32[$i>>2]|0;
         $154 = HEAP32[$pFKey>>2]|0;
         $155 = ((($154)) + 36|0);
         $156 = (($155) + ($153<<3)|0);
         HEAP32[$156>>2] = $152;
        }
        $158 = HEAP32[$j>>2]|0;
        $159 = HEAP32[$p>>2]|0;
        $160 = ((($159)) + 34|0);
        $161 = HEAP16[$160>>1]|0;
        $162 = $161 << 16 >> 16;
        $163 = ($158|0)>=($162|0);
        if ($163) {
         break;
        }
        $172 = HEAP32[$i>>2]|0;
        $173 = (($172) + 1)|0;
        HEAP32[$i>>2] = $173;
       }
       $164 = HEAP32[$0>>2]|0;
       $165 = HEAP32[$i>>2]|0;
       $166 = HEAP32[$1>>2]|0;
       $167 = ((($166)) + 4|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = (($168) + (($165*20)|0)|0);
       $170 = ((($169)) + 4|0);
       $171 = HEAP32[$170>>2]|0;
       HEAP32[$vararg_buffer4>>2] = $171;
       _sqlite3ErrorMsg($164,36427,$vararg_buffer4);
       break L1;
      }
     } while(0);
     $174 = HEAP32[$3>>2]|0;
     $175 = ($174|0)!=(0|0);
     L40: do {
      if ($175) {
       HEAP32[$i>>2] = 0;
       while(1) {
        $176 = HEAP32[$i>>2]|0;
        $177 = HEAP32[$nCol>>2]|0;
        $178 = ($176|0)<($177|0);
        if (!($178)) {
         break L40;
        }
        $179 = HEAP32[$i>>2]|0;
        $180 = HEAP32[$3>>2]|0;
        $181 = ((($180)) + 4|0);
        $182 = HEAP32[$181>>2]|0;
        $183 = (($182) + (($179*20)|0)|0);
        $184 = ((($183)) + 4|0);
        $185 = HEAP32[$184>>2]|0;
        $186 = (_sqlite3Strlen30($185)|0);
        HEAP32[$n>>2] = $186;
        $187 = HEAP32[$z>>2]|0;
        $188 = HEAP32[$i>>2]|0;
        $189 = HEAP32[$pFKey>>2]|0;
        $190 = ((($189)) + 36|0);
        $191 = (($190) + ($188<<3)|0);
        $192 = ((($191)) + 4|0);
        HEAP32[$192>>2] = $187;
        $193 = HEAP32[$z>>2]|0;
        $194 = HEAP32[$i>>2]|0;
        $195 = HEAP32[$3>>2]|0;
        $196 = ((($195)) + 4|0);
        $197 = HEAP32[$196>>2]|0;
        $198 = (($197) + (($194*20)|0)|0);
        $199 = ((($198)) + 4|0);
        $200 = HEAP32[$199>>2]|0;
        $201 = HEAP32[$n>>2]|0;
        _memcpy(($193|0),($200|0),($201|0))|0;
        $202 = HEAP32[$n>>2]|0;
        $203 = HEAP32[$z>>2]|0;
        $204 = (($203) + ($202)|0);
        HEAP8[$204>>0] = 0;
        $205 = HEAP32[$n>>2]|0;
        $206 = (($205) + 1)|0;
        $207 = HEAP32[$z>>2]|0;
        $208 = (($207) + ($206)|0);
        HEAP32[$z>>2] = $208;
        $209 = HEAP32[$i>>2]|0;
        $210 = (($209) + 1)|0;
        HEAP32[$i>>2] = $210;
       }
      }
     } while(0);
     $211 = HEAP32[$pFKey>>2]|0;
     $212 = ((($211)) + 24|0);
     HEAP8[$212>>0] = 0;
     $213 = HEAP32[$4>>2]|0;
     $214 = $213 & 255;
     $215 = $214&255;
     $216 = HEAP32[$pFKey>>2]|0;
     $217 = ((($216)) + 25|0);
     HEAP8[$217>>0] = $215;
     $218 = HEAP32[$4>>2]|0;
     $219 = $218 >> 8;
     $220 = $219 & 255;
     $221 = $220&255;
     $222 = HEAP32[$pFKey>>2]|0;
     $223 = ((($222)) + 25|0);
     $224 = ((($223)) + 1|0);
     HEAP8[$224>>0] = $221;
     $225 = HEAP32[$p>>2]|0;
     $226 = ((($225)) + 64|0);
     $227 = HEAP32[$226>>2]|0;
     $228 = ((($227)) + 56|0);
     $229 = HEAP32[$pFKey>>2]|0;
     $230 = ((($229)) + 8|0);
     $231 = HEAP32[$230>>2]|0;
     $232 = HEAP32[$pFKey>>2]|0;
     $233 = (_sqlite3HashInsert($228,$231,$232)|0);
     HEAP32[$pNextTo>>2] = $233;
     $234 = HEAP32[$pNextTo>>2]|0;
     $235 = HEAP32[$pFKey>>2]|0;
     $236 = ($234|0)==($235|0);
     if ($236) {
      $237 = HEAP32[$db>>2]|0;
      _sqlite3OomFault($237);
      break;
     }
     $238 = HEAP32[$pNextTo>>2]|0;
     $239 = ($238|0)!=(0|0);
     if ($239) {
      $240 = HEAP32[$pNextTo>>2]|0;
      $241 = HEAP32[$pFKey>>2]|0;
      $242 = ((($241)) + 12|0);
      HEAP32[$242>>2] = $240;
      $243 = HEAP32[$pFKey>>2]|0;
      $244 = HEAP32[$pNextTo>>2]|0;
      $245 = ((($244)) + 16|0);
      HEAP32[$245>>2] = $243;
     }
     $246 = HEAP32[$pFKey>>2]|0;
     $247 = HEAP32[$p>>2]|0;
     $248 = ((($247)) + 16|0);
     HEAP32[$248>>2] = $246;
     HEAP32[$pFKey>>2] = 0;
    }
   }
  }
 } while(0);
 $249 = HEAP32[$db>>2]|0;
 $250 = HEAP32[$pFKey>>2]|0;
 _sqlite3DbFree($249,$250);
 $251 = HEAP32[$db>>2]|0;
 $252 = HEAP32[$1>>2]|0;
 _sqlite3ExprListDelete($251,$252);
 $253 = HEAP32[$db>>2]|0;
 $254 = HEAP32[$3>>2]|0;
 _sqlite3ExprListDelete($253,$254);
 STACKTOP = sp;return;
}
function _sqlite3DeferForeignKey($pParse,$isDeferred) {
 $pParse = $pParse|0;
 $isDeferred = $isDeferred|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFKey = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pTab = sp + 4|0;
 $pFKey = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $isDeferred;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 440|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pTab>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pFKey>>2] = $8;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = $10&255;
 $12 = HEAP32[$pFKey>>2]|0;
 $13 = ((($12)) + 24|0);
 HEAP8[$13>>0] = $11;
 STACKTOP = sp;return;
}
function _sqlite3AddCollateType($pParse,$pToken) {
 $pParse = $pParse|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $p = 0, $pIdx = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $p = sp + 16|0;
 $i = sp + 12|0;
 $zColl = sp + 8|0;
 $db = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pToken;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 440|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 34|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 $10 = (($9) - 1)|0;
 HEAP32[$i>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3NameFromToken($13,$14)|0);
 HEAP32[$zColl>>2] = $15;
 $16 = HEAP32[$zColl>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$zColl>>2]|0;
 $20 = (_sqlite3LocateCollSeq($18,$19)|0);
 $21 = ($20|0)!=(0|0);
 $22 = HEAP32[$db>>2]|0;
 if (!($21)) {
  $62 = HEAP32[$zColl>>2]|0;
  _sqlite3DbFree($22,$62);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$i>>2]|0;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + ($23<<4)|0);
 $28 = ((($27)) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 _sqlite3DbFree($22,$29);
 $30 = HEAP32[$zColl>>2]|0;
 $31 = HEAP32[$i>>2]|0;
 $32 = HEAP32[$p>>2]|0;
 $33 = ((($32)) + 4|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31<<4)|0);
 $36 = ((($35)) + 8|0);
 HEAP32[$36>>2] = $30;
 $37 = HEAP32[$p>>2]|0;
 $38 = ((($37)) + 8|0);
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$pIdx>>2] = $39;
 while(1) {
  $40 = HEAP32[$pIdx>>2]|0;
  $41 = ($40|0)!=(0|0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$pIdx>>2]|0;
  $43 = ((($42)) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP16[$44>>1]|0;
  $46 = $45 << 16 >> 16;
  $47 = HEAP32[$i>>2]|0;
  $48 = ($46|0)==($47|0);
  if ($48) {
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$p>>2]|0;
   $51 = ((($50)) + 4|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + ($49<<4)|0);
   $54 = ((($53)) + 8|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = HEAP32[$pIdx>>2]|0;
   $57 = ((($56)) + 32|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$58>>2] = $55;
  }
  $59 = HEAP32[$pIdx>>2]|0;
  $60 = ((($59)) + 20|0);
  $61 = HEAP32[$60>>2]|0;
  HEAP32[$pIdx>>2] = $61;
 }
 STACKTOP = sp;return;
}
function _sqlite3DropTable($pParse,$pName,$isView,$noErr) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $isView = $isView|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $zArg2 = 0, $zDb = 0, $zTab = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $pTab = sp + 48|0;
 $v = sp + 44|0;
 $db = sp + 40|0;
 $iDb = sp + 36|0;
 $code = sp + 32|0;
 $zTab = sp + 28|0;
 $zDb = sp + 24|0;
 $zArg2 = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $isView;
 HEAP32[$3>>2] = $noErr;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 69|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 do {
  if (!($9)) {
   $10 = HEAP32[$0>>2]|0;
   $11 = (_sqlite3ReadSchema($10)|0);
   $12 = ($11|0)!=(0);
   if (!($12)) {
    $13 = HEAP32[$3>>2]|0;
    $14 = ($13|0)!=(0);
    if ($14) {
     $15 = HEAP32[$db>>2]|0;
     $16 = ((($15)) + 73|0);
     $17 = HEAP8[$16>>0]|0;
     $18 = (($17) + 1)<<24>>24;
     HEAP8[$16>>0] = $18;
    }
    $19 = HEAP32[$0>>2]|0;
    $20 = HEAP32[$2>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 8|0);
    $23 = (_sqlite3LocateTableItem($19,$20,$22)|0);
    HEAP32[$pTab>>2] = $23;
    $24 = HEAP32[$3>>2]|0;
    $25 = ($24|0)!=(0);
    if ($25) {
     $26 = HEAP32[$db>>2]|0;
     $27 = ((($26)) + 73|0);
     $28 = HEAP8[$27>>0]|0;
     $29 = (($28) + -1)<<24>>24;
     HEAP8[$27>>0] = $29;
    }
    $30 = HEAP32[$pTab>>2]|0;
    $31 = ($30|0)==(0|0);
    if ($31) {
     $32 = HEAP32[$3>>2]|0;
     $33 = ($32|0)!=(0);
     if (!($33)) {
      break;
     }
     $34 = HEAP32[$0>>2]|0;
     $35 = HEAP32[$1>>2]|0;
     $36 = ((($35)) + 8|0);
     $37 = ((($36)) + 4|0);
     $38 = HEAP32[$37>>2]|0;
     _sqlite3CodeVerifyNamedSchema($34,$38);
     break;
    }
    $39 = HEAP32[$db>>2]|0;
    $40 = HEAP32[$pTab>>2]|0;
    $41 = ((($40)) + 64|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = (_sqlite3SchemaToIndex($39,$42)|0);
    HEAP32[$iDb>>2] = $43;
    $44 = HEAP32[$pTab>>2]|0;
    $45 = ((($44)) + 42|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $48 = $47 & 16;
    $49 = ($48|0)!=(0);
    if ($49) {
     $50 = HEAP32[$0>>2]|0;
     $51 = HEAP32[$pTab>>2]|0;
     $52 = (_sqlite3ViewGetColumnNames($50,$51)|0);
     $53 = ($52|0)!=(0);
     if ($53) {
      break;
     }
    }
    $54 = HEAP32[$iDb>>2]|0;
    $55 = ($54|0)==(1);
    $56 = $55 ? 19905 : 19924;
    HEAP32[$zTab>>2] = $56;
    $57 = HEAP32[$iDb>>2]|0;
    $58 = HEAP32[$db>>2]|0;
    $59 = ((($58)) + 16|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = (($60) + ($57<<4)|0);
    $62 = HEAP32[$61>>2]|0;
    HEAP32[$zDb>>2] = $62;
    HEAP32[$zArg2>>2] = 0;
    $63 = HEAP32[$0>>2]|0;
    $64 = HEAP32[$zTab>>2]|0;
    $65 = HEAP32[$zDb>>2]|0;
    $66 = (_sqlite3AuthCheck($63,9,$64,0,$65)|0);
    $67 = ($66|0)!=(0);
    if (!($67)) {
     $68 = HEAP32[$2>>2]|0;
     $69 = ($68|0)!=(0);
     do {
      if ($69) {
       $70 = HEAP32[$iDb>>2]|0;
       $71 = ($70|0)==(1);
       if ($71) {
        HEAP32[$code>>2] = 15;
        break;
       } else {
        HEAP32[$code>>2] = 17;
        break;
       }
      } else {
       $72 = HEAP32[$pTab>>2]|0;
       $73 = ((($72)) + 42|0);
       $74 = HEAP8[$73>>0]|0;
       $75 = $74&255;
       $76 = $75 & 16;
       $77 = ($76|0)!=(0);
       if ($77) {
        HEAP32[$code>>2] = 30;
        $78 = HEAP32[$db>>2]|0;
        $79 = HEAP32[$pTab>>2]|0;
        $80 = (_sqlite3GetVTable($78,$79)|0);
        $81 = ((($80)) + 4|0);
        $82 = HEAP32[$81>>2]|0;
        $83 = ((($82)) + 4|0);
        $84 = HEAP32[$83>>2]|0;
        HEAP32[$zArg2>>2] = $84;
        break;
       }
       $85 = HEAP32[$iDb>>2]|0;
       $86 = ($85|0)==(1);
       if ($86) {
        HEAP32[$code>>2] = 13;
        break;
       } else {
        HEAP32[$code>>2] = 11;
        break;
       }
      }
     } while(0);
     $87 = HEAP32[$0>>2]|0;
     $88 = HEAP32[$code>>2]|0;
     $89 = HEAP32[$pTab>>2]|0;
     $90 = HEAP32[$89>>2]|0;
     $91 = HEAP32[$zArg2>>2]|0;
     $92 = HEAP32[$zDb>>2]|0;
     $93 = (_sqlite3AuthCheck($87,$88,$90,$91,$92)|0);
     $94 = ($93|0)!=(0);
     if (!($94)) {
      $95 = HEAP32[$0>>2]|0;
      $96 = HEAP32[$pTab>>2]|0;
      $97 = HEAP32[$96>>2]|0;
      $98 = HEAP32[$zDb>>2]|0;
      $99 = (_sqlite3AuthCheck($95,9,$97,0,$98)|0);
      $100 = ($99|0)!=(0);
      if (!($100)) {
       $101 = HEAP32[$pTab>>2]|0;
       $102 = HEAP32[$101>>2]|0;
       $103 = (_sqlite3_strnicmp($102,27844,7)|0);
       $104 = ($103|0)==(0);
       if ($104) {
        $105 = HEAP32[$pTab>>2]|0;
        $106 = HEAP32[$105>>2]|0;
        $107 = (_sqlite3_strnicmp($106,36063,11)|0);
        $108 = ($107|0)!=(0);
        if ($108) {
         $109 = HEAP32[$0>>2]|0;
         $110 = HEAP32[$pTab>>2]|0;
         $111 = HEAP32[$110>>2]|0;
         HEAP32[$vararg_buffer>>2] = $111;
         _sqlite3ErrorMsg($109,36075,$vararg_buffer);
         break;
        }
       }
       $112 = HEAP32[$2>>2]|0;
       $113 = ($112|0)!=(0);
       if ($113) {
        $114 = HEAP32[$pTab>>2]|0;
        $115 = ((($114)) + 12|0);
        $116 = HEAP32[$115>>2]|0;
        $117 = ($116|0)==(0|0);
        if ($117) {
         $118 = HEAP32[$0>>2]|0;
         $119 = HEAP32[$pTab>>2]|0;
         $120 = HEAP32[$119>>2]|0;
         HEAP32[$vararg_buffer1>>2] = $120;
         _sqlite3ErrorMsg($118,36103,$vararg_buffer1);
         break;
        }
       }
       $121 = HEAP32[$2>>2]|0;
       $122 = ($121|0)!=(0);
       if (!($122)) {
        $123 = HEAP32[$pTab>>2]|0;
        $124 = ((($123)) + 12|0);
        $125 = HEAP32[$124>>2]|0;
        $126 = ($125|0)!=(0|0);
        if ($126) {
         $127 = HEAP32[$0>>2]|0;
         $128 = HEAP32[$pTab>>2]|0;
         $129 = HEAP32[$128>>2]|0;
         HEAP32[$vararg_buffer4>>2] = $129;
         _sqlite3ErrorMsg($127,36137,$vararg_buffer4);
         break;
        }
       }
       $130 = HEAP32[$0>>2]|0;
       $131 = (_sqlite3GetVdbe($130)|0);
       HEAP32[$v>>2] = $131;
       $132 = HEAP32[$v>>2]|0;
       $133 = ($132|0)!=(0|0);
       if ($133) {
        $134 = HEAP32[$0>>2]|0;
        $135 = HEAP32[$iDb>>2]|0;
        _sqlite3BeginWriteOperation($134,1,$135);
        $136 = HEAP32[$0>>2]|0;
        $137 = HEAP32[$iDb>>2]|0;
        $138 = HEAP32[$pTab>>2]|0;
        $139 = HEAP32[$138>>2]|0;
        _sqlite3ClearStatTables($136,$137,30858,$139);
        $140 = HEAP32[$0>>2]|0;
        $141 = HEAP32[$1>>2]|0;
        $142 = HEAP32[$pTab>>2]|0;
        _sqlite3FkDropTable($140,$141,$142);
        $143 = HEAP32[$0>>2]|0;
        $144 = HEAP32[$pTab>>2]|0;
        $145 = HEAP32[$iDb>>2]|0;
        $146 = HEAP32[$2>>2]|0;
        _sqlite3CodeDropTable($143,$144,$145,$146);
       }
      }
     }
    }
   }
  }
 } while(0);
 $147 = HEAP32[$db>>2]|0;
 $148 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($147,$148);
 STACKTOP = sp;return;
}
function _sqlite3CreateView($pParse,$pBegin,$pName1,$pName2,$pCNames,$pSelect,$isTemp,$noErr) {
 $pParse = $pParse|0;
 $pBegin = $pBegin|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pCNames = $pCNames|0;
 $pSelect = $pSelect|0;
 $isTemp = $isTemp|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $db = 0, $iDb = 0;
 var $n = 0, $p = 0, $pName = 0, $sEnd = 0, $sFix = 0, $vararg_buffer = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $p = sp + 56|0;
 $n = sp + 52|0;
 $z = sp + 48|0;
 $sEnd = sp + 40|0;
 $sFix = sp + 16|0;
 $pName = sp + 12|0;
 $iDb = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pBegin;
 HEAP32[$2>>2] = $pName1;
 HEAP32[$3>>2] = $pName2;
 HEAP32[$4>>2] = $pCNames;
 HEAP32[$5>>2] = $pSelect;
 HEAP32[$6>>2] = $isTemp;
 HEAP32[$7>>2] = $noErr;
 HEAP32[$pName>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 400|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12 << 16 >> 16;
 $14 = ($13|0)>(0);
 $15 = HEAP32[$0>>2]|0;
 if ($14) {
  _sqlite3ErrorMsg($15,36022,$vararg_buffer);
 } else {
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  $18 = HEAP32[$6>>2]|0;
  $19 = HEAP32[$7>>2]|0;
  _sqlite3StartTable($15,$16,$17,$18,1,0,$19);
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 440|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = ($23|0)==(0|0);
  if (!($24)) {
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 36|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)!=(0);
   if (!($28)) {
    $29 = HEAP32[$0>>2]|0;
    $30 = HEAP32[$2>>2]|0;
    $31 = HEAP32[$3>>2]|0;
    (_sqlite3TwoPartName($29,$30,$31,$pName)|0);
    $32 = HEAP32[$db>>2]|0;
    $33 = HEAP32[$p>>2]|0;
    $34 = ((($33)) + 64|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = (_sqlite3SchemaToIndex($32,$35)|0);
    HEAP32[$iDb>>2] = $36;
    $37 = HEAP32[$0>>2]|0;
    $38 = HEAP32[$iDb>>2]|0;
    $39 = HEAP32[$pName>>2]|0;
    _sqlite3FixInit($sFix,$37,$38,36058,$39);
    $40 = HEAP32[$5>>2]|0;
    $41 = (_sqlite3FixSelect($sFix,$40)|0);
    $42 = ($41|0)!=(0);
    if (!($42)) {
     $43 = HEAP32[$db>>2]|0;
     $44 = HEAP32[$5>>2]|0;
     $45 = (_sqlite3SelectDup($43,$44,1)|0);
     $46 = HEAP32[$p>>2]|0;
     $47 = ((($46)) + 12|0);
     HEAP32[$47>>2] = $45;
     $48 = HEAP32[$db>>2]|0;
     $49 = HEAP32[$4>>2]|0;
     $50 = (_sqlite3ExprListDup($48,$49,1)|0);
     $51 = HEAP32[$p>>2]|0;
     $52 = ((($51)) + 24|0);
     HEAP32[$52>>2] = $50;
     $53 = HEAP32[$db>>2]|0;
     $54 = ((($53)) + 69|0);
     $55 = HEAP8[$54>>0]|0;
     $56 = ($55<<24>>24)!=(0);
     if (!($56)) {
      $57 = HEAP32[$0>>2]|0;
      $58 = ((($57)) + 392|0);
      ;HEAP32[$sEnd>>2]=HEAP32[$58>>2]|0;HEAP32[$sEnd+4>>2]=HEAP32[$58+4>>2]|0;
      $59 = HEAP32[$sEnd>>2]|0;
      $60 = HEAP8[$59>>0]|0;
      $61 = $60 << 24 >> 24;
      $62 = ($61|0)!=(59);
      if ($62) {
       $63 = ((($sEnd)) + 4|0);
       $64 = HEAP32[$63>>2]|0;
       $65 = HEAP32[$sEnd>>2]|0;
       $66 = (($65) + ($64)|0);
       HEAP32[$sEnd>>2] = $66;
      }
      $67 = ((($sEnd)) + 4|0);
      HEAP32[$67>>2] = 0;
      $68 = HEAP32[$sEnd>>2]|0;
      $69 = HEAP32[$1>>2]|0;
      $70 = HEAP32[$69>>2]|0;
      $71 = $68;
      $72 = $70;
      $73 = (($71) - ($72))|0;
      HEAP32[$n>>2] = $73;
      $74 = HEAP32[$1>>2]|0;
      $75 = HEAP32[$74>>2]|0;
      HEAP32[$z>>2] = $75;
      while(1) {
       $76 = HEAP32[$n>>2]|0;
       $77 = (($76) - 1)|0;
       $78 = HEAP32[$z>>2]|0;
       $79 = (($78) + ($77)|0);
       $80 = HEAP8[$79>>0]|0;
       $81 = $80&255;
       $82 = (18403 + ($81)|0);
       $83 = HEAP8[$82>>0]|0;
       $84 = $83&255;
       $85 = $84 & 1;
       $86 = ($85|0)!=(0);
       $87 = HEAP32[$n>>2]|0;
       if (!($86)) {
        break;
       }
       $88 = (($87) + -1)|0;
       HEAP32[$n>>2] = $88;
      }
      $89 = (($87) - 1)|0;
      $90 = HEAP32[$z>>2]|0;
      $91 = (($90) + ($89)|0);
      HEAP32[$sEnd>>2] = $91;
      $92 = ((($sEnd)) + 4|0);
      HEAP32[$92>>2] = 1;
      $93 = HEAP32[$0>>2]|0;
      _sqlite3EndTable($93,0,$sEnd,0,0);
     }
    }
   }
  }
 }
 $94 = HEAP32[$db>>2]|0;
 $95 = HEAP32[$5>>2]|0;
 _sqlite3SelectDelete($94,$95);
 $96 = HEAP32[$db>>2]|0;
 $97 = HEAP32[$4>>2]|0;
 _sqlite3ExprListDelete($96,$97);
 STACKTOP = sp;return;
}
function _sqlite3Select($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0;
 var $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0;
 var $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0;
 var $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0;
 var $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0;
 var $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0;
 var $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0;
 var $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0;
 var $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0;
 var $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0;
 var $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0;
 var $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0;
 var $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0;
 var $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0;
 var $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0;
 var $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0;
 var $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0;
 var $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0;
 var $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0;
 var $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0;
 var $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0;
 var $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0;
 var $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0;
 var $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0;
 var $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0;
 var $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0;
 var $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0;
 var $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0;
 var $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0;
 var $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0;
 var $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0;
 var $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0;
 var $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0;
 var $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0;
 var $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0;
 var $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0;
 var $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0;
 var $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0;
 var $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0;
 var $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0;
 var $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0;
 var $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0;
 var $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0;
 var $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0;
 var $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0;
 var $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0;
 var $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0;
 var $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0;
 var $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0;
 var $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $addr1 = 0, $addrEnd = 0, $addrOutputRow = 0, $addrReset = 0, $addrSetAbort = 0, $addrSortingIdx = 0, $addrTop = 0, $addrTopOfLoop = 0, $db = 0, $dest = 0, $flag = 0, $groupBySort = 0, $i = 0;
 var $iAMem = 0, $iAbortFlag = 0, $iBMem = 0, $iCsr = 0, $iDb = 0, $iEnd = 0, $iRestoreSelectId = 0, $iRoot = 0, $iUseFlag = 0, $isAgg = 0, $isAggSub = 0, $j = 0, $k = 0, $nCol = 0, $nGroupBy = 0, $onceAddr = 0, $or$cond = 0, $orderByGrp = 0, $pBest = 0, $pCol = 0;
 var $pDel = 0, $pEList = 0, $pGroupBy = 0, $pHaving = 0, $pIdx = 0, $pItem = 0, $pItem1 = 0, $pItem3 = 0, $pKeyInfo = 0, $pKeyInfo4 = 0, $pKeyInfo6 = 0, $pMinMax = 0, $pSub = 0, $pSub2 = 0, $pTab = 0, $pTab5 = 0, $pTabList = 0, $pWInfo = 0, $pWhere = 0, $r1 = 0;
 var $rc = 0, $regBase = 0, $regOutputRow = 0, $regRecord = 0, $regReset = 0, $retAddr = 0, $sAggInfo = 0, $sDistinct = 0, $sNC = 0, $sSort = 0, $sortOut = 0, $sortPTab = 0, $topAddr = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $wctrlFlags = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 432|0;
 $vararg_buffer = sp;
 $0 = sp + 412|0;
 $1 = sp + 408|0;
 $2 = sp + 404|0;
 $3 = sp + 400|0;
 $i = sp + 396|0;
 $j = sp + 392|0;
 $pWInfo = sp + 388|0;
 $v = sp + 384|0;
 $isAgg = sp + 380|0;
 $pEList = sp + 376|0;
 $pTabList = sp + 372|0;
 $pWhere = sp + 368|0;
 $pGroupBy = sp + 364|0;
 $pHaving = sp + 360|0;
 $rc = sp + 356|0;
 $sDistinct = sp + 344|0;
 $sSort = sp + 312|0;
 $sAggInfo = sp + 264|0;
 $iEnd = sp + 260|0;
 $db = sp + 256|0;
 $iRestoreSelectId = sp + 252|0;
 $pItem = sp + 248|0;
 $pSub = sp + 244|0;
 $isAggSub = sp + 240|0;
 $pTab = sp + 236|0;
 $pItem1 = sp + 232|0;
 $dest = sp + 208|0;
 $pSub2 = sp + 204|0;
 $addrTop = sp + 200|0;
 $topAddr = sp + 196|0;
 $onceAddr = sp + 192|0;
 $retAddr = sp + 188|0;
 $pKeyInfo = sp + 184|0;
 $wctrlFlags = sp + 416|0;
 $sNC = sp + 152|0;
 $iAMem = sp + 148|0;
 $iBMem = sp + 144|0;
 $iUseFlag = sp + 140|0;
 $iAbortFlag = sp + 136|0;
 $groupBySort = sp + 132|0;
 $addrEnd = sp + 128|0;
 $sortPTab = sp + 124|0;
 $sortOut = sp + 120|0;
 $orderByGrp = sp + 116|0;
 $k = sp + 112|0;
 $pItem3 = sp + 108|0;
 $pKeyInfo4 = sp + 104|0;
 $addr1 = sp + 100|0;
 $addrOutputRow = sp + 96|0;
 $regOutputRow = sp + 92|0;
 $addrSetAbort = sp + 88|0;
 $addrTopOfLoop = sp + 84|0;
 $addrSortingIdx = sp + 80|0;
 $addrReset = sp + 76|0;
 $regReset = sp + 72|0;
 $regBase = sp + 68|0;
 $regRecord = sp + 64|0;
 $nCol = sp + 60|0;
 $nGroupBy = sp + 56|0;
 $pCol = sp + 52|0;
 $r1 = sp + 48|0;
 $pDel = sp + 44|0;
 $pTab5 = sp + 40|0;
 $iDb = sp + 36|0;
 $iCsr = sp + 32|0;
 $pIdx = sp + 28|0;
 $pKeyInfo6 = sp + 24|0;
 $pBest = sp + 20|0;
 $iRoot = sp + 16|0;
 $pMinMax = sp + 12|0;
 $flag = sp + 418|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$pEList>>2] = 0;
 HEAP32[$rc>>2] = 1;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 420|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iRestoreSelectId>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 424|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 420|0);
 HEAP32[$12>>2] = $9;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ($15|0)==(0|0);
 if (!($16)) {
  $17 = HEAP32[$db>>2]|0;
  $18 = ((($17)) + 69|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)!=(0);
  if (!($21)) {
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 36|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$1>>2]|0;
    $27 = (_sqlite3AuthCheck($26,21,0,0,0)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$0>>2] = 1;
     $1186 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($1186|0);
    }
    dest=$sAggInfo; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
    $29 = HEAP32[$3>>2]|0;
    $30 = HEAP8[$29>>0]|0;
    $31 = $30&255;
    $32 = ($31|0)<=(8);
    if ($32) {
     $33 = HEAP32[$db>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     $35 = ((($34)) + 44|0);
     $36 = HEAP32[$35>>2]|0;
     _sqlite3ExprListDelete($33,$36);
     $37 = HEAP32[$2>>2]|0;
     $38 = ((($37)) + 44|0);
     HEAP32[$38>>2] = 0;
     $39 = HEAP32[$2>>2]|0;
     $40 = ((($39)) + 8|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = $41 & -2;
     HEAP32[$40>>2] = $42;
    }
    $43 = HEAP32[$1>>2]|0;
    $44 = HEAP32[$2>>2]|0;
    _sqlite3SelectPrep($43,$44,0);
    ;HEAP32[$sSort>>2]=0|0;HEAP32[$sSort+4>>2]=0|0;HEAP32[$sSort+8>>2]=0|0;HEAP32[$sSort+12>>2]=0|0;HEAP32[$sSort+16>>2]=0|0;HEAP32[$sSort+20>>2]=0|0;HEAP32[$sSort+24>>2]=0|0;HEAP32[$sSort+28>>2]=0|0;
    $45 = HEAP32[$2>>2]|0;
    $46 = ((($45)) + 44|0);
    $47 = HEAP32[$46>>2]|0;
    HEAP32[$sSort>>2] = $47;
    $48 = HEAP32[$2>>2]|0;
    $49 = ((($48)) + 28|0);
    $50 = HEAP32[$49>>2]|0;
    HEAP32[$pTabList>>2] = $50;
    $51 = HEAP32[$1>>2]|0;
    $52 = ((($51)) + 36|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = ($53|0)!=(0);
    L12: do {
     if (!($54)) {
      $55 = HEAP32[$db>>2]|0;
      $56 = ((($55)) + 69|0);
      $57 = HEAP8[$56>>0]|0;
      $58 = $57&255;
      $59 = ($58|0)!=(0);
      if (!($59)) {
       $60 = HEAP32[$2>>2]|0;
       $61 = ((($60)) + 8|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = $62 & 8;
       $64 = ($63|0)!=(0);
       $65 = $64&1;
       HEAP32[$isAgg>>2] = $65;
       HEAP32[$i>>2] = 0;
       while(1) {
        $66 = HEAP32[$2>>2]|0;
        $67 = ((($66)) + 48|0);
        $68 = HEAP32[$67>>2]|0;
        $69 = ($68|0)!=(0|0);
        if ($69) {
         break;
        }
        $70 = HEAP32[$i>>2]|0;
        $71 = HEAP32[$pTabList>>2]|0;
        $72 = HEAP32[$71>>2]|0;
        $73 = ($70|0)<($72|0);
        if (!($73)) {
         break;
        }
        $74 = HEAP32[$i>>2]|0;
        $75 = HEAP32[$pTabList>>2]|0;
        $76 = ((($75)) + 8|0);
        $77 = (($76) + (($74*72)|0)|0);
        HEAP32[$pItem>>2] = $77;
        $78 = HEAP32[$pItem>>2]|0;
        $79 = ((($78)) + 20|0);
        $80 = HEAP32[$79>>2]|0;
        HEAP32[$pSub>>2] = $80;
        $81 = HEAP32[$pItem>>2]|0;
        $82 = ((($81)) + 16|0);
        $83 = HEAP32[$82>>2]|0;
        HEAP32[$pTab>>2] = $83;
        $84 = HEAP32[$pSub>>2]|0;
        $85 = ($84|0)==(0|0);
        if (!($85)) {
         $86 = HEAP32[$pTab>>2]|0;
         $87 = ((($86)) + 34|0);
         $88 = HEAP16[$87>>1]|0;
         $89 = $88 << 16 >> 16;
         $90 = HEAP32[$pSub>>2]|0;
         $91 = HEAP32[$90>>2]|0;
         $92 = HEAP32[$91>>2]|0;
         $93 = ($89|0)!=($92|0);
         if ($93) {
          label = 16;
          break;
         }
         $104 = HEAP32[$pSub>>2]|0;
         $105 = ((($104)) + 8|0);
         $106 = HEAP32[$105>>2]|0;
         $107 = $106 & 8;
         $108 = ($107|0)!=(0);
         $109 = $108&1;
         HEAP32[$isAggSub>>2] = $109;
         $110 = HEAP32[$1>>2]|0;
         $111 = HEAP32[$2>>2]|0;
         $112 = HEAP32[$i>>2]|0;
         $113 = HEAP32[$isAgg>>2]|0;
         $114 = HEAP32[$isAggSub>>2]|0;
         $115 = (_flattenSubquery($110,$111,$112,$113,$114)|0);
         $116 = ($115|0)!=(0);
         if ($116) {
          $117 = HEAP32[$isAggSub>>2]|0;
          $118 = ($117|0)!=(0);
          if ($118) {
           HEAP32[$isAgg>>2] = 1;
           $119 = HEAP32[$2>>2]|0;
           $120 = ((($119)) + 8|0);
           $121 = HEAP32[$120>>2]|0;
           $122 = $121 | 8;
           HEAP32[$120>>2] = $122;
          }
          HEAP32[$i>>2] = -1;
         }
         $123 = HEAP32[$2>>2]|0;
         $124 = ((($123)) + 28|0);
         $125 = HEAP32[$124>>2]|0;
         HEAP32[$pTabList>>2] = $125;
         $126 = HEAP32[$db>>2]|0;
         $127 = ((($126)) + 69|0);
         $128 = HEAP8[$127>>0]|0;
         $129 = ($128<<24>>24)!=(0);
         if ($129) {
          break L12;
         }
         $130 = HEAP32[$3>>2]|0;
         $131 = HEAP8[$130>>0]|0;
         $132 = $131&255;
         $133 = ($132|0)<=(8);
         if (!($133)) {
          $134 = HEAP32[$2>>2]|0;
          $135 = ((($134)) + 44|0);
          $136 = HEAP32[$135>>2]|0;
          HEAP32[$sSort>>2] = $136;
         }
        }
        $137 = HEAP32[$i>>2]|0;
        $138 = (($137) + 1)|0;
        HEAP32[$i>>2] = $138;
       }
       if ((label|0) == 16) {
        $94 = HEAP32[$1>>2]|0;
        $95 = HEAP32[$pTab>>2]|0;
        $96 = ((($95)) + 34|0);
        $97 = HEAP16[$96>>1]|0;
        $98 = $97 << 16 >> 16;
        $99 = HEAP32[$pTab>>2]|0;
        $100 = HEAP32[$99>>2]|0;
        $101 = HEAP32[$pSub>>2]|0;
        $102 = HEAP32[$101>>2]|0;
        $103 = HEAP32[$102>>2]|0;
        HEAP32[$vararg_buffer>>2] = $98;
        $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
        HEAP32[$vararg_ptr2>>2] = $100;
        $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
        HEAP32[$vararg_ptr3>>2] = $103;
        _sqlite3ErrorMsg($94,35609,$vararg_buffer);
        break;
       }
       $139 = HEAP32[$1>>2]|0;
       $140 = (_sqlite3GetVdbe($139)|0);
       HEAP32[$v>>2] = $140;
       $141 = HEAP32[$v>>2]|0;
       $142 = ($141|0)==(0|0);
       if (!($142)) {
        $143 = HEAP32[$2>>2]|0;
        $144 = ((($143)) + 48|0);
        $145 = HEAP32[$144>>2]|0;
        $146 = ($145|0)!=(0|0);
        if ($146) {
         $147 = HEAP32[$1>>2]|0;
         $148 = HEAP32[$2>>2]|0;
         $149 = HEAP32[$3>>2]|0;
         $150 = (_multiSelect($147,$148,$149)|0);
         HEAP32[$rc>>2] = $150;
         $151 = HEAP32[$iRestoreSelectId>>2]|0;
         $152 = HEAP32[$1>>2]|0;
         $153 = ((($152)) + 420|0);
         HEAP32[$153>>2] = $151;
         $154 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $154;
         $1186 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($1186|0);
        }
        HEAP32[$i>>2] = 0;
        while(1) {
         $155 = HEAP32[$i>>2]|0;
         $156 = HEAP32[$pTabList>>2]|0;
         $157 = HEAP32[$156>>2]|0;
         $158 = ($155|0)<($157|0);
         if (!($158)) {
          break;
         }
         $159 = HEAP32[$i>>2]|0;
         $160 = HEAP32[$pTabList>>2]|0;
         $161 = ((($160)) + 8|0);
         $162 = (($161) + (($159*72)|0)|0);
         HEAP32[$pItem1>>2] = $162;
         $163 = HEAP32[$pItem1>>2]|0;
         $164 = ((($163)) + 20|0);
         $165 = HEAP32[$164>>2]|0;
         HEAP32[$pSub2>>2] = $165;
         $166 = HEAP32[$pSub2>>2]|0;
         $167 = ($166|0)==(0|0);
         do {
          if (!($167)) {
           $168 = HEAP32[$pItem1>>2]|0;
           $169 = ((($168)) + 24|0);
           $170 = HEAP32[$169>>2]|0;
           $171 = ($170|0)!=(0);
           if ($171) {
            $172 = HEAP32[$pItem1>>2]|0;
            $173 = ((($172)) + 36|0);
            $174 = ((($173)) + 1|0);
            $175 = HEAP8[$174>>0]|0;
            $176 = ($175&255) >>> 4;
            $177 = $176 & 1;
            $178 = $177&255;
            $179 = ($178|0)==(0);
            if (!($179)) {
             break;
            }
            $180 = HEAP32[$v>>2]|0;
            $181 = HEAP32[$pItem1>>2]|0;
            $182 = ((($181)) + 28|0);
            $183 = HEAP32[$182>>2]|0;
            $184 = HEAP32[$pItem1>>2]|0;
            $185 = ((($184)) + 24|0);
            $186 = HEAP32[$185>>2]|0;
            (_sqlite3VdbeAddOp2($180,14,$183,$186)|0);
            break;
           }
           $187 = HEAP32[$2>>2]|0;
           $188 = (_sqlite3SelectExprHeight($187)|0);
           $189 = HEAP32[$1>>2]|0;
           $190 = ((($189)) + 416|0);
           $191 = HEAP32[$190>>2]|0;
           $192 = (($191) + ($188))|0;
           HEAP32[$190>>2] = $192;
           $193 = HEAP32[$pItem1>>2]|0;
           $194 = ((($193)) + 36|0);
           $195 = HEAP8[$194>>0]|0;
           $196 = $195&255;
           $197 = $196 & 32;
           $198 = ($197|0)==(0);
           if ($198) {
            $199 = HEAP32[$db>>2]|0;
            $200 = HEAP32[$pSub2>>2]|0;
            $201 = HEAP32[$2>>2]|0;
            $202 = ((($201)) + 32|0);
            $203 = HEAP32[$202>>2]|0;
            $204 = HEAP32[$pItem1>>2]|0;
            $205 = ((($204)) + 44|0);
            $206 = HEAP32[$205>>2]|0;
            (_pushDownWhereTerms($199,$200,$203,$206)|0);
           }
           $207 = HEAP32[$i>>2]|0;
           $208 = ($207|0)==(0);
           do {
            if ($208) {
             $209 = HEAP32[$pTabList>>2]|0;
             $210 = HEAP32[$209>>2]|0;
             $211 = ($210|0)==(1);
             if (!($211)) {
              $212 = HEAP32[$pTabList>>2]|0;
              $213 = ((($212)) + 8|0);
              $214 = ((($213)) + 72|0);
              $215 = ((($214)) + 36|0);
              $216 = HEAP8[$215>>0]|0;
              $217 = $216&255;
              $218 = $217 & 10;
              $219 = ($218|0)!=(0);
              if (!($219)) {
               label = 42;
               break;
              }
             }
             $220 = HEAP32[$2>>2]|0;
             $221 = ((($220)) + 8|0);
             $222 = HEAP32[$221>>2]|0;
             $223 = $222 & 2;
             $224 = ($223|0)==(0);
             if (!($224)) {
              label = 42;
              break;
             }
             $225 = HEAP32[$db>>2]|0;
             $226 = ((($225)) + 64|0);
             $227 = HEAP16[$226>>1]|0;
             $228 = $227&65535;
             $229 = $228 & 256;
             $230 = ($229|0)==(0);
             if (!($230)) {
              label = 42;
              break;
             }
             $231 = HEAP32[$v>>2]|0;
             $232 = (_sqlite3VdbeCurrentAddr($231)|0);
             $233 = (($232) + 1)|0;
             HEAP32[$addrTop>>2] = $233;
             $234 = HEAP32[$1>>2]|0;
             $235 = ((($234)) + 44|0);
             $236 = HEAP32[$235>>2]|0;
             $237 = (($236) + 1)|0;
             HEAP32[$235>>2] = $237;
             $238 = HEAP32[$pItem1>>2]|0;
             $239 = ((($238)) + 28|0);
             HEAP32[$239>>2] = $237;
             $240 = HEAP32[$v>>2]|0;
             $241 = HEAP32[$pItem1>>2]|0;
             $242 = ((($241)) + 28|0);
             $243 = HEAP32[$242>>2]|0;
             $244 = HEAP32[$addrTop>>2]|0;
             (_sqlite3VdbeAddOp3($240,15,$243,0,$244)|0);
             $245 = HEAP32[$addrTop>>2]|0;
             $246 = HEAP32[$pItem1>>2]|0;
             $247 = ((($246)) + 24|0);
             HEAP32[$247>>2] = $245;
             $248 = HEAP32[$pItem1>>2]|0;
             $249 = ((($248)) + 28|0);
             $250 = HEAP32[$249>>2]|0;
             _sqlite3SelectDestInit($dest,13,$250);
             $251 = HEAP32[$1>>2]|0;
             $252 = ((($251)) + 424|0);
             $253 = HEAP32[$252>>2]|0;
             $254 = $253&255;
             $255 = HEAP32[$pItem1>>2]|0;
             $256 = ((($255)) + 40|0);
             HEAP8[$256>>0] = $254;
             $257 = HEAP32[$1>>2]|0;
             $258 = HEAP32[$pSub2>>2]|0;
             (_sqlite3Select($257,$258,$dest)|0);
             $259 = HEAP32[$pSub2>>2]|0;
             $260 = ((($259)) + 6|0);
             $261 = HEAP16[$260>>1]|0;
             $262 = HEAP32[$pItem1>>2]|0;
             $263 = ((($262)) + 16|0);
             $264 = HEAP32[$263>>2]|0;
             $265 = ((($264)) + 38|0);
             HEAP16[$265>>1] = $261;
             $266 = HEAP32[$pItem1>>2]|0;
             $267 = ((($266)) + 36|0);
             $268 = ((($267)) + 1|0);
             $269 = HEAP8[$268>>0]|0;
             $270 = $269 & -17;
             $271 = $270 | 16;
             HEAP8[$268>>0] = $271;
             $272 = ((($dest)) + 12|0);
             $273 = HEAP32[$272>>2]|0;
             $274 = HEAP32[$pItem1>>2]|0;
             $275 = ((($274)) + 32|0);
             HEAP32[$275>>2] = $273;
             $276 = HEAP32[$v>>2]|0;
             $277 = HEAP32[$pItem1>>2]|0;
             $278 = ((($277)) + 28|0);
             $279 = HEAP32[$278>>2]|0;
             _sqlite3VdbeEndCoroutine($276,$279);
             $280 = HEAP32[$v>>2]|0;
             $281 = HEAP32[$addrTop>>2]|0;
             $282 = (($281) - 1)|0;
             _sqlite3VdbeJumpHere($280,$282);
             $283 = HEAP32[$1>>2]|0;
             _sqlite3ClearTempRegCache($283);
            } else {
             label = 42;
            }
           } while(0);
           if ((label|0) == 42) {
            label = 0;
            HEAP32[$onceAddr>>2] = 0;
            $284 = HEAP32[$1>>2]|0;
            $285 = ((($284)) + 44|0);
            $286 = HEAP32[$285>>2]|0;
            $287 = (($286) + 1)|0;
            HEAP32[$285>>2] = $287;
            $288 = HEAP32[$pItem1>>2]|0;
            $289 = ((($288)) + 28|0);
            HEAP32[$289>>2] = $287;
            $290 = HEAP32[$v>>2]|0;
            $291 = HEAP32[$pItem1>>2]|0;
            $292 = ((($291)) + 28|0);
            $293 = HEAP32[$292>>2]|0;
            $294 = (_sqlite3VdbeAddOp2($290,76,0,$293)|0);
            HEAP32[$topAddr>>2] = $294;
            $295 = HEAP32[$topAddr>>2]|0;
            $296 = (($295) + 1)|0;
            $297 = HEAP32[$pItem1>>2]|0;
            $298 = ((($297)) + 24|0);
            HEAP32[$298>>2] = $296;
            $299 = HEAP32[$pItem1>>2]|0;
            $300 = ((($299)) + 36|0);
            $301 = ((($300)) + 1|0);
            $302 = HEAP8[$301>>0]|0;
            $303 = ($302&255) >>> 3;
            $304 = $303 & 1;
            $305 = $304&255;
            $306 = ($305|0)==(0);
            if ($306) {
             $307 = HEAP32[$v>>2]|0;
             $308 = (_sqlite3VdbeAddOp0($307,20)|0);
             HEAP32[$onceAddr>>2] = $308;
            }
            $309 = HEAP32[$pItem1>>2]|0;
            $310 = ((($309)) + 44|0);
            $311 = HEAP32[$310>>2]|0;
            _sqlite3SelectDestInit($dest,12,$311);
            $312 = HEAP32[$1>>2]|0;
            $313 = ((($312)) + 424|0);
            $314 = HEAP32[$313>>2]|0;
            $315 = $314&255;
            $316 = HEAP32[$pItem1>>2]|0;
            $317 = ((($316)) + 40|0);
            HEAP8[$317>>0] = $315;
            $318 = HEAP32[$1>>2]|0;
            $319 = HEAP32[$pSub2>>2]|0;
            (_sqlite3Select($318,$319,$dest)|0);
            $320 = HEAP32[$pSub2>>2]|0;
            $321 = ((($320)) + 6|0);
            $322 = HEAP16[$321>>1]|0;
            $323 = HEAP32[$pItem1>>2]|0;
            $324 = ((($323)) + 16|0);
            $325 = HEAP32[$324>>2]|0;
            $326 = ((($325)) + 38|0);
            HEAP16[$326>>1] = $322;
            $327 = HEAP32[$onceAddr>>2]|0;
            $328 = ($327|0)!=(0);
            if ($328) {
             $329 = HEAP32[$v>>2]|0;
             $330 = HEAP32[$onceAddr>>2]|0;
             _sqlite3VdbeJumpHere($329,$330);
            }
            $331 = HEAP32[$v>>2]|0;
            $332 = HEAP32[$pItem1>>2]|0;
            $333 = ((($332)) + 28|0);
            $334 = HEAP32[$333>>2]|0;
            $335 = (_sqlite3VdbeAddOp1($331,72,$334)|0);
            HEAP32[$retAddr>>2] = $335;
            $336 = HEAP32[$v>>2]|0;
            $337 = HEAP32[$topAddr>>2]|0;
            $338 = HEAP32[$retAddr>>2]|0;
            _sqlite3VdbeChangeP1($336,$337,$338);
            $339 = HEAP32[$1>>2]|0;
            _sqlite3ClearTempRegCache($339);
           }
           $340 = HEAP32[$db>>2]|0;
           $341 = ((($340)) + 69|0);
           $342 = HEAP8[$341>>0]|0;
           $343 = ($342<<24>>24)!=(0);
           if ($343) {
            break L12;
           }
           $344 = HEAP32[$2>>2]|0;
           $345 = (_sqlite3SelectExprHeight($344)|0);
           $346 = HEAP32[$1>>2]|0;
           $347 = ((($346)) + 416|0);
           $348 = HEAP32[$347>>2]|0;
           $349 = (($348) - ($345))|0;
           HEAP32[$347>>2] = $349;
          }
         } while(0);
         $350 = HEAP32[$i>>2]|0;
         $351 = (($350) + 1)|0;
         HEAP32[$i>>2] = $351;
        }
        $352 = HEAP32[$2>>2]|0;
        $353 = HEAP32[$352>>2]|0;
        HEAP32[$pEList>>2] = $353;
        $354 = HEAP32[$2>>2]|0;
        $355 = ((($354)) + 32|0);
        $356 = HEAP32[$355>>2]|0;
        HEAP32[$pWhere>>2] = $356;
        $357 = HEAP32[$2>>2]|0;
        $358 = ((($357)) + 36|0);
        $359 = HEAP32[$358>>2]|0;
        HEAP32[$pGroupBy>>2] = $359;
        $360 = HEAP32[$2>>2]|0;
        $361 = ((($360)) + 40|0);
        $362 = HEAP32[$361>>2]|0;
        HEAP32[$pHaving>>2] = $362;
        $363 = HEAP32[$2>>2]|0;
        $364 = ((($363)) + 8|0);
        $365 = HEAP32[$364>>2]|0;
        $366 = $365 & 1;
        $367 = ($366|0)!=(0);
        $368 = $367&1;
        $369 = $368&255;
        HEAP8[$sDistinct>>0] = $369;
        $370 = HEAP32[$2>>2]|0;
        $371 = ((($370)) + 8|0);
        $372 = HEAP32[$371>>2]|0;
        $373 = $372 & 9;
        $374 = ($373|0)==(1);
        if ($374) {
         $375 = HEAP32[$sSort>>2]|0;
         $376 = HEAP32[$pEList>>2]|0;
         $377 = (_sqlite3ExprListCompare($375,$376,-1)|0);
         $378 = ($377|0)==(0);
         if ($378) {
          $379 = HEAP32[$2>>2]|0;
          $380 = ((($379)) + 8|0);
          $381 = HEAP32[$380>>2]|0;
          $382 = $381 & -2;
          HEAP32[$380>>2] = $382;
          $383 = HEAP32[$db>>2]|0;
          $384 = HEAP32[$pEList>>2]|0;
          $385 = (_sqlite3ExprListDup($383,$384,0)|0);
          $386 = HEAP32[$2>>2]|0;
          $387 = ((($386)) + 36|0);
          HEAP32[$387>>2] = $385;
          HEAP32[$pGroupBy>>2] = $385;
         }
        }
        $388 = HEAP32[$sSort>>2]|0;
        $389 = ($388|0)!=(0|0);
        if ($389) {
         $390 = HEAP32[$1>>2]|0;
         $391 = HEAP32[$sSort>>2]|0;
         $392 = HEAP32[$pEList>>2]|0;
         $393 = HEAP32[$392>>2]|0;
         $394 = (_keyInfoFromExprList($390,$391,0,$393)|0);
         HEAP32[$pKeyInfo>>2] = $394;
         $395 = HEAP32[$1>>2]|0;
         $396 = ((($395)) + 40|0);
         $397 = HEAP32[$396>>2]|0;
         $398 = (($397) + 1)|0;
         HEAP32[$396>>2] = $398;
         $399 = ((($sSort)) + 8|0);
         HEAP32[$399>>2] = $397;
         $400 = HEAP32[$v>>2]|0;
         $401 = ((($sSort)) + 8|0);
         $402 = HEAP32[$401>>2]|0;
         $403 = HEAP32[$sSort>>2]|0;
         $404 = HEAP32[$403>>2]|0;
         $405 = (($404) + 1)|0;
         $406 = HEAP32[$pEList>>2]|0;
         $407 = HEAP32[$406>>2]|0;
         $408 = (($405) + ($407))|0;
         $409 = HEAP32[$pKeyInfo>>2]|0;
         $410 = (_sqlite3VdbeAddOp4($400,107,$402,$408,0,$409,-6)|0);
         $411 = ((($sSort)) + 20|0);
         HEAP32[$411>>2] = $410;
        } else {
         $412 = ((($sSort)) + 20|0);
         HEAP32[$412>>2] = -1;
        }
        $413 = HEAP32[$3>>2]|0;
        $414 = HEAP8[$413>>0]|0;
        $415 = $414&255;
        $416 = ($415|0)==(12);
        if ($416) {
         $417 = HEAP32[$v>>2]|0;
         $418 = HEAP32[$3>>2]|0;
         $419 = ((($418)) + 8|0);
         $420 = HEAP32[$419>>2]|0;
         $421 = HEAP32[$pEList>>2]|0;
         $422 = HEAP32[$421>>2]|0;
         (_sqlite3VdbeAddOp2($417,107,$420,$422)|0);
        }
        $423 = HEAP32[$v>>2]|0;
        $424 = (_sqlite3VdbeMakeLabel($423)|0);
        HEAP32[$iEnd>>2] = $424;
        $425 = HEAP32[$2>>2]|0;
        $426 = ((($425)) + 6|0);
        HEAP16[$426>>1] = 320;
        $427 = HEAP32[$1>>2]|0;
        $428 = HEAP32[$2>>2]|0;
        $429 = HEAP32[$iEnd>>2]|0;
        _computeLimitRegisters($427,$428,$429);
        $430 = HEAP32[$2>>2]|0;
        $431 = ((($430)) + 12|0);
        $432 = HEAP32[$431>>2]|0;
        $433 = ($432|0)==(0);
        if ($433) {
         $434 = ((($sSort)) + 20|0);
         $435 = HEAP32[$434>>2]|0;
         $436 = ($435|0)>=(0);
         if ($436) {
          $437 = HEAP32[$v>>2]|0;
          $438 = ((($sSort)) + 20|0);
          $439 = HEAP32[$438>>2]|0;
          _sqlite3VdbeChangeOpcode($437,$439,108);
          $440 = ((($sSort)) + 28|0);
          $441 = HEAP8[$440>>0]|0;
          $442 = $441&255;
          $443 = $442 | 1;
          $444 = $443&255;
          HEAP8[$440>>0] = $444;
         }
        }
        $445 = HEAP32[$2>>2]|0;
        $446 = ((($445)) + 8|0);
        $447 = HEAP32[$446>>2]|0;
        $448 = $447 & 1;
        $449 = ($448|0)!=(0);
        if ($449) {
         $450 = HEAP32[$1>>2]|0;
         $451 = ((($450)) + 40|0);
         $452 = HEAP32[$451>>2]|0;
         $453 = (($452) + 1)|0;
         HEAP32[$451>>2] = $453;
         $454 = ((($sDistinct)) + 4|0);
         HEAP32[$454>>2] = $452;
         $455 = HEAP32[$v>>2]|0;
         $456 = ((($sDistinct)) + 4|0);
         $457 = HEAP32[$456>>2]|0;
         $458 = HEAP32[$1>>2]|0;
         $459 = HEAP32[$2>>2]|0;
         $460 = HEAP32[$459>>2]|0;
         $461 = (_keyInfoFromExprList($458,$460,0,0)|0);
         $462 = (_sqlite3VdbeAddOp4($455,107,$457,0,0,$461,-6)|0);
         $463 = ((($sDistinct)) + 8|0);
         HEAP32[$463>>2] = $462;
         $464 = HEAP32[$v>>2]|0;
         _sqlite3VdbeChangeP5($464,8);
         $465 = ((($sDistinct)) + 1|0);
         HEAP8[$465>>0] = 3;
        } else {
         $466 = ((($sDistinct)) + 1|0);
         HEAP8[$466>>0] = 0;
        }
        $467 = HEAP32[$isAgg>>2]|0;
        $468 = ($467|0)==(0);
        $469 = HEAP32[$pGroupBy>>2]|0;
        $470 = ($469|0)==(0|0);
        $or$cond = $468 & $470;
        if ($or$cond) {
         $471 = HEAP8[$sDistinct>>0]|0;
         $472 = $471&255;
         $473 = ($472|0)!=(0);
         $474 = $473 ? 256 : 0;
         $475 = $474&65535;
         HEAP16[$wctrlFlags>>1] = $475;
         $476 = HEAP32[$2>>2]|0;
         $477 = ((($476)) + 8|0);
         $478 = HEAP32[$477>>2]|0;
         $479 = $478 & 16384;
         $480 = HEAP16[$wctrlFlags>>1]|0;
         $481 = $480&65535;
         $482 = $481 | $479;
         $483 = $482&65535;
         HEAP16[$wctrlFlags>>1] = $483;
         $484 = HEAP32[$1>>2]|0;
         $485 = HEAP32[$pTabList>>2]|0;
         $486 = HEAP32[$pWhere>>2]|0;
         $487 = HEAP32[$sSort>>2]|0;
         $488 = HEAP32[$2>>2]|0;
         $489 = HEAP32[$488>>2]|0;
         $490 = HEAP16[$wctrlFlags>>1]|0;
         $491 = HEAP32[$2>>2]|0;
         $492 = ((($491)) + 6|0);
         $493 = HEAP16[$492>>1]|0;
         $494 = $493 << 16 >> 16;
         $495 = (_sqlite3WhereBegin($484,$485,$486,$487,$489,$490,$494)|0);
         HEAP32[$pWInfo>>2] = $495;
         $496 = HEAP32[$pWInfo>>2]|0;
         $497 = ($496|0)==(0|0);
         if ($497) {
          break;
         }
         $498 = HEAP32[$pWInfo>>2]|0;
         $499 = (_sqlite3WhereOutputRowCount($498)|0);
         $500 = $499 << 16 >> 16;
         $501 = HEAP32[$2>>2]|0;
         $502 = ((($501)) + 6|0);
         $503 = HEAP16[$502>>1]|0;
         $504 = $503 << 16 >> 16;
         $505 = ($500|0)<($504|0);
         if ($505) {
          $506 = HEAP32[$pWInfo>>2]|0;
          $507 = (_sqlite3WhereOutputRowCount($506)|0);
          $508 = HEAP32[$2>>2]|0;
          $509 = ((($508)) + 6|0);
          HEAP16[$509>>1] = $507;
         }
         $510 = HEAP8[$sDistinct>>0]|0;
         $511 = $510&255;
         $512 = ($511|0)!=(0);
         do {
          if ($512) {
           $513 = HEAP32[$pWInfo>>2]|0;
           $514 = (_sqlite3WhereIsDistinct($513)|0);
           $515 = ($514|0)!=(0);
           if (!($515)) {
            break;
           }
           $516 = HEAP32[$pWInfo>>2]|0;
           $517 = (_sqlite3WhereIsDistinct($516)|0);
           $518 = $517&255;
           $519 = ((($sDistinct)) + 1|0);
           HEAP8[$519>>0] = $518;
          }
         } while(0);
         $520 = HEAP32[$sSort>>2]|0;
         $521 = ($520|0)!=(0|0);
         do {
          if ($521) {
           $522 = HEAP32[$pWInfo>>2]|0;
           $523 = (_sqlite3WhereIsOrdered($522)|0);
           $524 = ((($sSort)) + 4|0);
           HEAP32[$524>>2] = $523;
           $525 = HEAP32[$pWInfo>>2]|0;
           $526 = (_sqlite3WhereOrderedInnerLoop($525)|0);
           $527 = $526&255;
           $528 = ((($sSort)) + 29|0);
           HEAP8[$528>>0] = $527;
           $529 = ((($sSort)) + 4|0);
           $530 = HEAP32[$529>>2]|0;
           $531 = HEAP32[$sSort>>2]|0;
           $532 = HEAP32[$531>>2]|0;
           $533 = ($530|0)==($532|0);
           if (!($533)) {
            break;
           }
           HEAP32[$sSort>>2] = 0;
          }
         } while(0);
         $534 = ((($sSort)) + 20|0);
         $535 = HEAP32[$534>>2]|0;
         $536 = ($535|0)>=(0);
         do {
          if ($536) {
           $537 = HEAP32[$sSort>>2]|0;
           $538 = ($537|0)==(0|0);
           if (!($538)) {
            break;
           }
           $539 = HEAP32[$v>>2]|0;
           $540 = ((($sSort)) + 20|0);
           $541 = HEAP32[$540>>2]|0;
           (_sqlite3VdbeChangeToNoop($539,$541)|0);
          }
         } while(0);
         $542 = HEAP32[$1>>2]|0;
         $543 = HEAP32[$2>>2]|0;
         $544 = HEAP32[$pEList>>2]|0;
         $545 = HEAP32[$3>>2]|0;
         $546 = HEAP32[$pWInfo>>2]|0;
         $547 = (_sqlite3WhereContinueLabel($546)|0);
         $548 = HEAP32[$pWInfo>>2]|0;
         $549 = (_sqlite3WhereBreakLabel($548)|0);
         _selectInnerLoop($542,$543,$544,-1,$sSort,$sDistinct,$545,$547,$549);
         $550 = HEAP32[$pWInfo>>2]|0;
         _sqlite3WhereEnd($550);
        } else {
         HEAP32[$sortPTab>>2] = 0;
         HEAP32[$sortOut>>2] = 0;
         HEAP32[$orderByGrp>>2] = 0;
         $551 = HEAP32[$pGroupBy>>2]|0;
         $552 = ($551|0)!=(0|0);
         $553 = HEAP32[$2>>2]|0;
         do {
          if ($552) {
           $554 = HEAP32[$553>>2]|0;
           $555 = HEAP32[$554>>2]|0;
           HEAP32[$k>>2] = $555;
           $556 = HEAP32[$2>>2]|0;
           $557 = HEAP32[$556>>2]|0;
           $558 = ((($557)) + 4|0);
           $559 = HEAP32[$558>>2]|0;
           HEAP32[$pItem3>>2] = $559;
           while(1) {
            $560 = HEAP32[$k>>2]|0;
            $561 = ($560|0)>(0);
            if (!($561)) {
             break;
            }
            $562 = HEAP32[$pItem3>>2]|0;
            $563 = ((($562)) + 16|0);
            $564 = ((($563)) + 2|0);
            HEAP16[$564>>1] = 0;
            $565 = HEAP32[$k>>2]|0;
            $566 = (($565) + -1)|0;
            HEAP32[$k>>2] = $566;
            $567 = HEAP32[$pItem3>>2]|0;
            $568 = ((($567)) + 20|0);
            HEAP32[$pItem3>>2] = $568;
           }
           $569 = HEAP32[$pGroupBy>>2]|0;
           $570 = HEAP32[$569>>2]|0;
           HEAP32[$k>>2] = $570;
           $571 = HEAP32[$pGroupBy>>2]|0;
           $572 = ((($571)) + 4|0);
           $573 = HEAP32[$572>>2]|0;
           HEAP32[$pItem3>>2] = $573;
           while(1) {
            $574 = HEAP32[$k>>2]|0;
            $575 = ($574|0)>(0);
            if (!($575)) {
             break;
            }
            $576 = HEAP32[$pItem3>>2]|0;
            $577 = ((($576)) + 16|0);
            $578 = ((($577)) + 2|0);
            HEAP16[$578>>1] = 0;
            $579 = HEAP32[$k>>2]|0;
            $580 = (($579) + -1)|0;
            HEAP32[$k>>2] = $580;
            $581 = HEAP32[$pItem3>>2]|0;
            $582 = ((($581)) + 20|0);
            HEAP32[$pItem3>>2] = $582;
           }
           $583 = HEAP32[$2>>2]|0;
           $584 = ((($583)) + 6|0);
           $585 = HEAP16[$584>>1]|0;
           $586 = $585 << 16 >> 16;
           $587 = ($586|0)>(66);
           if (!($587)) {
            break;
           }
           $588 = HEAP32[$2>>2]|0;
           $589 = ((($588)) + 6|0);
           HEAP16[$589>>1] = 66;
          } else {
           $590 = ((($553)) + 6|0);
           HEAP16[$590>>1] = 0;
          }
         } while(0);
         $591 = HEAP32[$pGroupBy>>2]|0;
         $592 = HEAP32[$sSort>>2]|0;
         $593 = (_sqlite3ExprListCompare($591,$592,-1)|0);
         $594 = ($593|0)==(0);
         if ($594) {
          HEAP32[$orderByGrp>>2] = 1;
         }
         $595 = HEAP32[$v>>2]|0;
         $596 = (_sqlite3VdbeMakeLabel($595)|0);
         HEAP32[$addrEnd>>2] = $596;
         ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
         $597 = HEAP32[$1>>2]|0;
         HEAP32[$sNC>>2] = $597;
         $598 = HEAP32[$pTabList>>2]|0;
         $599 = ((($sNC)) + 4|0);
         HEAP32[$599>>2] = $598;
         $600 = ((($sNC)) + 12|0);
         HEAP32[$600>>2] = $sAggInfo;
         $601 = HEAP32[$1>>2]|0;
         $602 = ((($601)) + 44|0);
         $603 = HEAP32[$602>>2]|0;
         $604 = (($603) + 1)|0;
         $605 = ((($sAggInfo)) + 16|0);
         HEAP32[$605>>2] = $604;
         $606 = HEAP32[$pGroupBy>>2]|0;
         $607 = ($606|0)!=(0|0);
         if ($607) {
          $608 = HEAP32[$pGroupBy>>2]|0;
          $609 = HEAP32[$608>>2]|0;
          $611 = $609;
         } else {
          $611 = 0;
         }
         $610 = ((($sAggInfo)) + 12|0);
         HEAP32[$610>>2] = $611;
         $612 = HEAP32[$pGroupBy>>2]|0;
         $613 = ((($sAggInfo)) + 24|0);
         HEAP32[$613>>2] = $612;
         $614 = HEAP32[$pEList>>2]|0;
         _sqlite3ExprAnalyzeAggList($sNC,$614);
         $615 = HEAP32[$sSort>>2]|0;
         _sqlite3ExprAnalyzeAggList($sNC,$615);
         $616 = HEAP32[$pHaving>>2]|0;
         $617 = ($616|0)!=(0|0);
         if ($617) {
          $618 = HEAP32[$pHaving>>2]|0;
          _sqlite3ExprAnalyzeAggregates($sNC,$618);
         }
         $619 = ((($sAggInfo)) + 32|0);
         $620 = HEAP32[$619>>2]|0;
         $621 = ((($sAggInfo)) + 36|0);
         HEAP32[$621>>2] = $620;
         HEAP32[$i>>2] = 0;
         while(1) {
          $622 = HEAP32[$i>>2]|0;
          $623 = ((($sAggInfo)) + 44|0);
          $624 = HEAP32[$623>>2]|0;
          $625 = ($622|0)<($624|0);
          if (!($625)) {
           break;
          }
          $626 = ((($sNC)) + 28|0);
          $627 = HEAP16[$626>>1]|0;
          $628 = $627&65535;
          $629 = $628 | 8;
          $630 = $629&65535;
          HEAP16[$626>>1] = $630;
          $631 = HEAP32[$i>>2]|0;
          $632 = ((($sAggInfo)) + 40|0);
          $633 = HEAP32[$632>>2]|0;
          $634 = (($633) + ($631<<4)|0);
          $635 = HEAP32[$634>>2]|0;
          $636 = ((($635)) + 20|0);
          $637 = HEAP32[$636>>2]|0;
          _sqlite3ExprAnalyzeAggList($sNC,$637);
          $638 = ((($sNC)) + 28|0);
          $639 = HEAP16[$638>>1]|0;
          $640 = $639&65535;
          $641 = $640 & -9;
          $642 = $641&65535;
          HEAP16[$638>>1] = $642;
          $643 = HEAP32[$i>>2]|0;
          $644 = (($643) + 1)|0;
          HEAP32[$i>>2] = $644;
         }
         $645 = HEAP32[$1>>2]|0;
         $646 = ((($645)) + 44|0);
         $647 = HEAP32[$646>>2]|0;
         $648 = ((($sAggInfo)) + 20|0);
         HEAP32[$648>>2] = $647;
         $649 = HEAP32[$db>>2]|0;
         $650 = ((($649)) + 69|0);
         $651 = HEAP8[$650>>0]|0;
         $652 = ($651<<24>>24)!=(0);
         if ($652) {
          break;
         }
         $653 = HEAP32[$pGroupBy>>2]|0;
         $654 = ($653|0)!=(0|0);
         if ($654) {
          $655 = HEAP32[$1>>2]|0;
          $656 = ((($655)) + 40|0);
          $657 = HEAP32[$656>>2]|0;
          $658 = (($657) + 1)|0;
          HEAP32[$656>>2] = $658;
          $659 = ((($sAggInfo)) + 4|0);
          HEAP32[$659>>2] = $657;
          $660 = HEAP32[$1>>2]|0;
          $661 = HEAP32[$pGroupBy>>2]|0;
          $662 = ((($sAggInfo)) + 32|0);
          $663 = HEAP32[$662>>2]|0;
          $664 = (_keyInfoFromExprList($660,$661,0,$663)|0);
          HEAP32[$pKeyInfo4>>2] = $664;
          $665 = HEAP32[$v>>2]|0;
          $666 = ((($sAggInfo)) + 4|0);
          $667 = HEAP32[$666>>2]|0;
          $668 = ((($sAggInfo)) + 12|0);
          $669 = HEAP32[$668>>2]|0;
          $670 = HEAP32[$pKeyInfo4>>2]|0;
          $671 = (_sqlite3VdbeAddOp4($665,108,$667,$669,0,$670,-6)|0);
          HEAP32[$addrSortingIdx>>2] = $671;
          $672 = HEAP32[$1>>2]|0;
          $673 = ((($672)) + 44|0);
          $674 = HEAP32[$673>>2]|0;
          $675 = (($674) + 1)|0;
          HEAP32[$673>>2] = $675;
          HEAP32[$iUseFlag>>2] = $675;
          $676 = HEAP32[$1>>2]|0;
          $677 = ((($676)) + 44|0);
          $678 = HEAP32[$677>>2]|0;
          $679 = (($678) + 1)|0;
          HEAP32[$677>>2] = $679;
          HEAP32[$iAbortFlag>>2] = $679;
          $680 = HEAP32[$1>>2]|0;
          $681 = ((($680)) + 44|0);
          $682 = HEAP32[$681>>2]|0;
          $683 = (($682) + 1)|0;
          HEAP32[$681>>2] = $683;
          HEAP32[$regOutputRow>>2] = $683;
          $684 = HEAP32[$v>>2]|0;
          $685 = (_sqlite3VdbeMakeLabel($684)|0);
          HEAP32[$addrOutputRow>>2] = $685;
          $686 = HEAP32[$1>>2]|0;
          $687 = ((($686)) + 44|0);
          $688 = HEAP32[$687>>2]|0;
          $689 = (($688) + 1)|0;
          HEAP32[$687>>2] = $689;
          HEAP32[$regReset>>2] = $689;
          $690 = HEAP32[$v>>2]|0;
          $691 = (_sqlite3VdbeMakeLabel($690)|0);
          HEAP32[$addrReset>>2] = $691;
          $692 = HEAP32[$1>>2]|0;
          $693 = ((($692)) + 44|0);
          $694 = HEAP32[$693>>2]|0;
          $695 = (($694) + 1)|0;
          HEAP32[$iAMem>>2] = $695;
          $696 = HEAP32[$pGroupBy>>2]|0;
          $697 = HEAP32[$696>>2]|0;
          $698 = HEAP32[$1>>2]|0;
          $699 = ((($698)) + 44|0);
          $700 = HEAP32[$699>>2]|0;
          $701 = (($700) + ($697))|0;
          HEAP32[$699>>2] = $701;
          $702 = HEAP32[$1>>2]|0;
          $703 = ((($702)) + 44|0);
          $704 = HEAP32[$703>>2]|0;
          $705 = (($704) + 1)|0;
          HEAP32[$iBMem>>2] = $705;
          $706 = HEAP32[$pGroupBy>>2]|0;
          $707 = HEAP32[$706>>2]|0;
          $708 = HEAP32[$1>>2]|0;
          $709 = ((($708)) + 44|0);
          $710 = HEAP32[$709>>2]|0;
          $711 = (($710) + ($707))|0;
          HEAP32[$709>>2] = $711;
          $712 = HEAP32[$v>>2]|0;
          $713 = HEAP32[$iAbortFlag>>2]|0;
          (_sqlite3VdbeAddOp2($712,76,0,$713)|0);
          $714 = HEAP32[$v>>2]|0;
          $715 = HEAP32[$iUseFlag>>2]|0;
          (_sqlite3VdbeAddOp2($714,76,0,$715)|0);
          $716 = HEAP32[$v>>2]|0;
          $717 = HEAP32[$iAMem>>2]|0;
          $718 = HEAP32[$iAMem>>2]|0;
          $719 = HEAP32[$pGroupBy>>2]|0;
          $720 = HEAP32[$719>>2]|0;
          $721 = (($718) + ($720))|0;
          $722 = (($721) - 1)|0;
          (_sqlite3VdbeAddOp3($716,79,0,$717,$722)|0);
          $723 = HEAP32[$v>>2]|0;
          $724 = HEAP32[$regReset>>2]|0;
          $725 = HEAP32[$addrReset>>2]|0;
          (_sqlite3VdbeAddOp2($723,14,$724,$725)|0);
          $726 = HEAP32[$1>>2]|0;
          $727 = HEAP32[$pTabList>>2]|0;
          $728 = HEAP32[$pWhere>>2]|0;
          $729 = HEAP32[$pGroupBy>>2]|0;
          $730 = HEAP32[$orderByGrp>>2]|0;
          $731 = ($730|0)!=(0);
          $732 = $731 ? 512 : 0;
          $733 = 64 | $732;
          $734 = $733&65535;
          $735 = (_sqlite3WhereBegin($726,$727,$728,$729,0,$734,0)|0);
          HEAP32[$pWInfo>>2] = $735;
          $736 = HEAP32[$pWInfo>>2]|0;
          $737 = ($736|0)==(0|0);
          if ($737) {
           break;
          }
          $738 = HEAP32[$pWInfo>>2]|0;
          $739 = (_sqlite3WhereIsOrdered($738)|0);
          $740 = HEAP32[$pGroupBy>>2]|0;
          $741 = HEAP32[$740>>2]|0;
          $742 = ($739|0)==($741|0);
          if ($742) {
           HEAP32[$groupBySort>>2] = 0;
          } else {
           $743 = HEAP32[$1>>2]|0;
           $744 = HEAP8[$sDistinct>>0]|0;
           $745 = $744&255;
           $746 = ($745|0)!=(0);
           if ($746) {
            $747 = HEAP32[$2>>2]|0;
            $748 = ((($747)) + 8|0);
            $749 = HEAP32[$748>>2]|0;
            $750 = $749 & 1;
            $751 = ($750|0)==(0);
            $753 = $751;
           } else {
            $753 = 0;
           }
           $752 = $753 ? 35649 : 35658;
           _explainTempTable($743,$752);
           HEAP32[$groupBySort>>2] = 1;
           $754 = HEAP32[$pGroupBy>>2]|0;
           $755 = HEAP32[$754>>2]|0;
           HEAP32[$nGroupBy>>2] = $755;
           $756 = HEAP32[$nGroupBy>>2]|0;
           HEAP32[$nCol>>2] = $756;
           $757 = HEAP32[$nGroupBy>>2]|0;
           HEAP32[$j>>2] = $757;
           HEAP32[$i>>2] = 0;
           while(1) {
            $758 = HEAP32[$i>>2]|0;
            $759 = ((($sAggInfo)) + 32|0);
            $760 = HEAP32[$759>>2]|0;
            $761 = ($758|0)<($760|0);
            if (!($761)) {
             break;
            }
            $762 = HEAP32[$i>>2]|0;
            $763 = ((($sAggInfo)) + 28|0);
            $764 = HEAP32[$763>>2]|0;
            $765 = (($764) + (($762*24)|0)|0);
            $766 = ((($765)) + 12|0);
            $767 = HEAP32[$766>>2]|0;
            $768 = HEAP32[$j>>2]|0;
            $769 = ($767|0)>=($768|0);
            if ($769) {
             $770 = HEAP32[$nCol>>2]|0;
             $771 = (($770) + 1)|0;
             HEAP32[$nCol>>2] = $771;
             $772 = HEAP32[$j>>2]|0;
             $773 = (($772) + 1)|0;
             HEAP32[$j>>2] = $773;
            }
            $774 = HEAP32[$i>>2]|0;
            $775 = (($774) + 1)|0;
            HEAP32[$i>>2] = $775;
           }
           $776 = HEAP32[$1>>2]|0;
           $777 = HEAP32[$nCol>>2]|0;
           $778 = (_sqlite3GetTempRange($776,$777)|0);
           HEAP32[$regBase>>2] = $778;
           $779 = HEAP32[$1>>2]|0;
           _sqlite3ExprCacheClear($779);
           $780 = HEAP32[$1>>2]|0;
           $781 = HEAP32[$pGroupBy>>2]|0;
           $782 = HEAP32[$regBase>>2]|0;
           (_sqlite3ExprCodeExprList($780,$781,$782,0,0)|0);
           $783 = HEAP32[$nGroupBy>>2]|0;
           HEAP32[$j>>2] = $783;
           HEAP32[$i>>2] = 0;
           while(1) {
            $784 = HEAP32[$i>>2]|0;
            $785 = ((($sAggInfo)) + 32|0);
            $786 = HEAP32[$785>>2]|0;
            $787 = ($784|0)<($786|0);
            if (!($787)) {
             break;
            }
            $788 = HEAP32[$i>>2]|0;
            $789 = ((($sAggInfo)) + 28|0);
            $790 = HEAP32[$789>>2]|0;
            $791 = (($790) + (($788*24)|0)|0);
            HEAP32[$pCol>>2] = $791;
            $792 = HEAP32[$pCol>>2]|0;
            $793 = ((($792)) + 12|0);
            $794 = HEAP32[$793>>2]|0;
            $795 = HEAP32[$j>>2]|0;
            $796 = ($794|0)>=($795|0);
            if ($796) {
             $797 = HEAP32[$j>>2]|0;
             $798 = HEAP32[$regBase>>2]|0;
             $799 = (($797) + ($798))|0;
             HEAP32[$r1>>2] = $799;
             $800 = HEAP32[$1>>2]|0;
             $801 = HEAP32[$pCol>>2]|0;
             $802 = HEAP32[$801>>2]|0;
             $803 = HEAP32[$pCol>>2]|0;
             $804 = ((($803)) + 8|0);
             $805 = HEAP32[$804>>2]|0;
             $806 = HEAP32[$pCol>>2]|0;
             $807 = ((($806)) + 4|0);
             $808 = HEAP32[$807>>2]|0;
             $809 = HEAP32[$r1>>2]|0;
             _sqlite3ExprCodeGetColumnToReg($800,$802,$805,$808,$809);
             $810 = HEAP32[$j>>2]|0;
             $811 = (($810) + 1)|0;
             HEAP32[$j>>2] = $811;
            }
            $812 = HEAP32[$i>>2]|0;
            $813 = (($812) + 1)|0;
            HEAP32[$i>>2] = $813;
           }
           $814 = HEAP32[$1>>2]|0;
           $815 = (_sqlite3GetTempReg($814)|0);
           HEAP32[$regRecord>>2] = $815;
           $816 = HEAP32[$v>>2]|0;
           $817 = HEAP32[$regBase>>2]|0;
           $818 = HEAP32[$nCol>>2]|0;
           $819 = HEAP32[$regRecord>>2]|0;
           (_sqlite3VdbeAddOp3($816,99,$817,$818,$819)|0);
           $820 = HEAP32[$v>>2]|0;
           $821 = ((($sAggInfo)) + 4|0);
           $822 = HEAP32[$821>>2]|0;
           $823 = HEAP32[$regRecord>>2]|0;
           (_sqlite3VdbeAddOp2($820,125,$822,$823)|0);
           $824 = HEAP32[$1>>2]|0;
           $825 = HEAP32[$regRecord>>2]|0;
           _sqlite3ReleaseTempReg($824,$825);
           $826 = HEAP32[$1>>2]|0;
           $827 = HEAP32[$regBase>>2]|0;
           $828 = HEAP32[$nCol>>2]|0;
           _sqlite3ReleaseTempRange($826,$827,$828);
           $829 = HEAP32[$pWInfo>>2]|0;
           _sqlite3WhereEnd($829);
           $830 = HEAP32[$1>>2]|0;
           $831 = ((($830)) + 40|0);
           $832 = HEAP32[$831>>2]|0;
           $833 = (($832) + 1)|0;
           HEAP32[$831>>2] = $833;
           HEAP32[$sortPTab>>2] = $832;
           $834 = ((($sAggInfo)) + 8|0);
           HEAP32[$834>>2] = $832;
           $835 = HEAP32[$1>>2]|0;
           $836 = (_sqlite3GetTempReg($835)|0);
           HEAP32[$sortOut>>2] = $836;
           $837 = HEAP32[$v>>2]|0;
           $838 = HEAP32[$sortPTab>>2]|0;
           $839 = HEAP32[$sortOut>>2]|0;
           $840 = HEAP32[$nCol>>2]|0;
           (_sqlite3VdbeAddOp3($837,110,$838,$839,$840)|0);
           $841 = HEAP32[$v>>2]|0;
           $842 = ((($sAggInfo)) + 4|0);
           $843 = HEAP32[$842>>2]|0;
           $844 = HEAP32[$addrEnd>>2]|0;
           (_sqlite3VdbeAddOp2($841,55,$843,$844)|0);
           $845 = ((($sAggInfo)) + 1|0);
           HEAP8[$845>>0] = 1;
           $846 = HEAP32[$1>>2]|0;
           _sqlite3ExprCacheClear($846);
          }
          $847 = HEAP32[$orderByGrp>>2]|0;
          $848 = ($847|0)!=(0);
          do {
           if ($848) {
            $849 = HEAP32[$db>>2]|0;
            $850 = ((($849)) + 64|0);
            $851 = HEAP16[$850>>1]|0;
            $852 = $851&65535;
            $853 = $852 & 4;
            $854 = ($853|0)==(0);
            if (!($854)) {
             break;
            }
            $855 = HEAP32[$groupBySort>>2]|0;
            $856 = ($855|0)!=(0);
            if (!($856)) {
             $857 = HEAP32[$pWInfo>>2]|0;
             $858 = (_sqlite3WhereIsSorted($857)|0);
             $859 = ($858|0)!=(0);
             if (!($859)) {
              break;
             }
            }
            HEAP32[$sSort>>2] = 0;
            $860 = HEAP32[$v>>2]|0;
            $861 = ((($sSort)) + 20|0);
            $862 = HEAP32[$861>>2]|0;
            (_sqlite3VdbeChangeToNoop($860,$862)|0);
           }
          } while(0);
          $863 = HEAP32[$v>>2]|0;
          $864 = (_sqlite3VdbeCurrentAddr($863)|0);
          HEAP32[$addrTopOfLoop>>2] = $864;
          $865 = HEAP32[$1>>2]|0;
          _sqlite3ExprCacheClear($865);
          $866 = HEAP32[$groupBySort>>2]|0;
          $867 = ($866|0)!=(0);
          if ($867) {
           $868 = HEAP32[$v>>2]|0;
           $869 = ((($sAggInfo)) + 4|0);
           $870 = HEAP32[$869>>2]|0;
           $871 = HEAP32[$sortOut>>2]|0;
           $872 = HEAP32[$sortPTab>>2]|0;
           (_sqlite3VdbeAddOp3($868,120,$870,$871,$872)|0);
          }
          HEAP32[$j>>2] = 0;
          while(1) {
           $873 = HEAP32[$j>>2]|0;
           $874 = HEAP32[$pGroupBy>>2]|0;
           $875 = HEAP32[$874>>2]|0;
           $876 = ($873|0)<($875|0);
           if (!($876)) {
            break;
           }
           $877 = HEAP32[$groupBySort>>2]|0;
           $878 = ($877|0)!=(0);
           if ($878) {
            $879 = HEAP32[$v>>2]|0;
            $880 = HEAP32[$sortPTab>>2]|0;
            $881 = HEAP32[$j>>2]|0;
            $882 = HEAP32[$iBMem>>2]|0;
            $883 = HEAP32[$j>>2]|0;
            $884 = (($882) + ($883))|0;
            (_sqlite3VdbeAddOp3($879,96,$880,$881,$884)|0);
           } else {
            HEAP8[$sAggInfo>>0] = 1;
            $885 = HEAP32[$1>>2]|0;
            $886 = HEAP32[$j>>2]|0;
            $887 = HEAP32[$pGroupBy>>2]|0;
            $888 = ((($887)) + 4|0);
            $889 = HEAP32[$888>>2]|0;
            $890 = (($889) + (($886*20)|0)|0);
            $891 = HEAP32[$890>>2]|0;
            $892 = HEAP32[$iBMem>>2]|0;
            $893 = HEAP32[$j>>2]|0;
            $894 = (($892) + ($893))|0;
            _sqlite3ExprCode($885,$891,$894);
           }
           $895 = HEAP32[$j>>2]|0;
           $896 = (($895) + 1)|0;
           HEAP32[$j>>2] = $896;
          }
          $897 = HEAP32[$v>>2]|0;
          $898 = HEAP32[$iAMem>>2]|0;
          $899 = HEAP32[$iBMem>>2]|0;
          $900 = HEAP32[$pGroupBy>>2]|0;
          $901 = HEAP32[$900>>2]|0;
          $902 = HEAP32[$pKeyInfo4>>2]|0;
          $903 = (_sqlite3KeyInfoRef($902)|0);
          (_sqlite3VdbeAddOp4($897,95,$898,$899,$901,$903,-6)|0);
          $904 = HEAP32[$v>>2]|0;
          $905 = (_sqlite3VdbeCurrentAddr($904)|0);
          HEAP32[$addr1>>2] = $905;
          $906 = HEAP32[$v>>2]|0;
          $907 = HEAP32[$addr1>>2]|0;
          $908 = (($907) + 1)|0;
          $909 = HEAP32[$addr1>>2]|0;
          $910 = (($909) + 1)|0;
          (_sqlite3VdbeAddOp3($906,18,$908,0,$910)|0);
          $911 = HEAP32[$1>>2]|0;
          $912 = HEAP32[$iBMem>>2]|0;
          $913 = HEAP32[$iAMem>>2]|0;
          $914 = HEAP32[$pGroupBy>>2]|0;
          $915 = HEAP32[$914>>2]|0;
          _sqlite3ExprCodeMove($911,$912,$913,$915);
          $916 = HEAP32[$v>>2]|0;
          $917 = HEAP32[$regOutputRow>>2]|0;
          $918 = HEAP32[$addrOutputRow>>2]|0;
          (_sqlite3VdbeAddOp2($916,14,$917,$918)|0);
          $919 = HEAP32[$v>>2]|0;
          $920 = HEAP32[$iAbortFlag>>2]|0;
          $921 = HEAP32[$addrEnd>>2]|0;
          (_sqlite3VdbeAddOp2($919,66,$920,$921)|0);
          $922 = HEAP32[$v>>2]|0;
          $923 = HEAP32[$regReset>>2]|0;
          $924 = HEAP32[$addrReset>>2]|0;
          (_sqlite3VdbeAddOp2($922,14,$923,$924)|0);
          $925 = HEAP32[$v>>2]|0;
          $926 = HEAP32[$addr1>>2]|0;
          _sqlite3VdbeJumpHere($925,$926);
          $927 = HEAP32[$1>>2]|0;
          _updateAccumulator($927,$sAggInfo);
          $928 = HEAP32[$v>>2]|0;
          $929 = HEAP32[$iUseFlag>>2]|0;
          (_sqlite3VdbeAddOp2($928,76,1,$929)|0);
          $930 = HEAP32[$groupBySort>>2]|0;
          $931 = ($930|0)!=(0);
          if ($931) {
           $932 = HEAP32[$v>>2]|0;
           $933 = ((($sAggInfo)) + 4|0);
           $934 = HEAP32[$933>>2]|0;
           $935 = HEAP32[$addrTopOfLoop>>2]|0;
           (_sqlite3VdbeAddOp2($932,3,$934,$935)|0);
          } else {
           $936 = HEAP32[$pWInfo>>2]|0;
           _sqlite3WhereEnd($936);
           $937 = HEAP32[$v>>2]|0;
           $938 = HEAP32[$addrSortingIdx>>2]|0;
           (_sqlite3VdbeChangeToNoop($937,$938)|0);
          }
          $939 = HEAP32[$v>>2]|0;
          $940 = HEAP32[$regOutputRow>>2]|0;
          $941 = HEAP32[$addrOutputRow>>2]|0;
          (_sqlite3VdbeAddOp2($939,14,$940,$941)|0);
          $942 = HEAP32[$v>>2]|0;
          $943 = HEAP32[$addrEnd>>2]|0;
          (_sqlite3VdbeGoto($942,$943)|0);
          $944 = HEAP32[$v>>2]|0;
          $945 = (_sqlite3VdbeCurrentAddr($944)|0);
          HEAP32[$addrSetAbort>>2] = $945;
          $946 = HEAP32[$v>>2]|0;
          $947 = HEAP32[$iAbortFlag>>2]|0;
          (_sqlite3VdbeAddOp2($946,76,1,$947)|0);
          $948 = HEAP32[$v>>2]|0;
          $949 = HEAP32[$regOutputRow>>2]|0;
          (_sqlite3VdbeAddOp1($948,72,$949)|0);
          $950 = HEAP32[$v>>2]|0;
          $951 = HEAP32[$addrOutputRow>>2]|0;
          _sqlite3VdbeResolveLabel($950,$951);
          $952 = HEAP32[$v>>2]|0;
          $953 = (_sqlite3VdbeCurrentAddr($952)|0);
          HEAP32[$addrOutputRow>>2] = $953;
          $954 = HEAP32[$v>>2]|0;
          $955 = HEAP32[$iUseFlag>>2]|0;
          $956 = HEAP32[$addrOutputRow>>2]|0;
          $957 = (($956) + 2)|0;
          (_sqlite3VdbeAddOp2($954,66,$955,$957)|0);
          $958 = HEAP32[$v>>2]|0;
          $959 = HEAP32[$regOutputRow>>2]|0;
          (_sqlite3VdbeAddOp1($958,72,$959)|0);
          $960 = HEAP32[$1>>2]|0;
          _finalizeAggFunctions($960,$sAggInfo);
          $961 = HEAP32[$1>>2]|0;
          $962 = HEAP32[$pHaving>>2]|0;
          $963 = HEAP32[$addrOutputRow>>2]|0;
          $964 = (($963) + 1)|0;
          _sqlite3ExprIfFalse($961,$962,$964,16);
          $965 = HEAP32[$1>>2]|0;
          $966 = HEAP32[$2>>2]|0;
          $967 = HEAP32[$2>>2]|0;
          $968 = HEAP32[$967>>2]|0;
          $969 = HEAP32[$3>>2]|0;
          $970 = HEAP32[$addrOutputRow>>2]|0;
          $971 = (($970) + 1)|0;
          $972 = HEAP32[$addrSetAbort>>2]|0;
          _selectInnerLoop($965,$966,$968,-1,$sSort,$sDistinct,$969,$971,$972);
          $973 = HEAP32[$v>>2]|0;
          $974 = HEAP32[$regOutputRow>>2]|0;
          (_sqlite3VdbeAddOp1($973,72,$974)|0);
          $975 = HEAP32[$v>>2]|0;
          $976 = HEAP32[$addrReset>>2]|0;
          _sqlite3VdbeResolveLabel($975,$976);
          $977 = HEAP32[$1>>2]|0;
          _resetAccumulator($977,$sAggInfo);
          $978 = HEAP32[$v>>2]|0;
          $979 = HEAP32[$regReset>>2]|0;
          (_sqlite3VdbeAddOp1($978,72,$979)|0);
         } else {
          HEAP32[$pDel>>2] = 0;
          $980 = HEAP32[$2>>2]|0;
          $981 = (_isSimpleCount($980,$sAggInfo)|0);
          HEAP32[$pTab5>>2] = $981;
          $982 = ($981|0)!=(0|0);
          if ($982) {
           $983 = HEAP32[$1>>2]|0;
           $984 = HEAP32[$983>>2]|0;
           $985 = HEAP32[$pTab5>>2]|0;
           $986 = ((($985)) + 64|0);
           $987 = HEAP32[$986>>2]|0;
           $988 = (_sqlite3SchemaToIndex($984,$987)|0);
           HEAP32[$iDb>>2] = $988;
           $989 = HEAP32[$1>>2]|0;
           $990 = ((($989)) + 40|0);
           $991 = HEAP32[$990>>2]|0;
           $992 = (($991) + 1)|0;
           HEAP32[$990>>2] = $992;
           HEAP32[$iCsr>>2] = $991;
           HEAP32[$pKeyInfo6>>2] = 0;
           HEAP32[$pBest>>2] = 0;
           $993 = HEAP32[$pTab5>>2]|0;
           $994 = ((($993)) + 28|0);
           $995 = HEAP32[$994>>2]|0;
           HEAP32[$iRoot>>2] = $995;
           $996 = HEAP32[$1>>2]|0;
           $997 = HEAP32[$iDb>>2]|0;
           _sqlite3CodeVerifySchema($996,$997);
           $998 = HEAP32[$1>>2]|0;
           $999 = HEAP32[$iDb>>2]|0;
           $1000 = HEAP32[$pTab5>>2]|0;
           $1001 = ((($1000)) + 28|0);
           $1002 = HEAP32[$1001>>2]|0;
           $1003 = HEAP32[$pTab5>>2]|0;
           $1004 = HEAP32[$1003>>2]|0;
           _sqlite3TableLock($998,$999,$1002,0,$1004);
           $1005 = HEAP32[$pTab5>>2]|0;
           $1006 = ((($1005)) + 42|0);
           $1007 = HEAP8[$1006>>0]|0;
           $1008 = $1007&255;
           $1009 = $1008 & 32;
           $1010 = ($1009|0)==(0);
           if (!($1010)) {
            $1011 = HEAP32[$pTab5>>2]|0;
            $1012 = (_sqlite3PrimaryKeyIndex($1011)|0);
            HEAP32[$pBest>>2] = $1012;
           }
           $1013 = HEAP32[$pTab5>>2]|0;
           $1014 = ((($1013)) + 8|0);
           $1015 = HEAP32[$1014>>2]|0;
           HEAP32[$pIdx>>2] = $1015;
           while(1) {
            $1016 = HEAP32[$pIdx>>2]|0;
            $1017 = ($1016|0)!=(0|0);
            if (!($1017)) {
             break;
            }
            $1018 = HEAP32[$pIdx>>2]|0;
            $1019 = ((($1018)) + 55|0);
            $1020 = HEAP8[$1019>>0]|0;
            $1021 = ($1020&255) >>> 2;
            $1022 = $1021 & 1;
            $1023 = $1022&255;
            $1024 = ($1023|0)==(0);
            do {
             if ($1024) {
              $1025 = HEAP32[$pIdx>>2]|0;
              $1026 = ((($1025)) + 48|0);
              $1027 = HEAP16[$1026>>1]|0;
              $1028 = $1027 << 16 >> 16;
              $1029 = HEAP32[$pTab5>>2]|0;
              $1030 = ((($1029)) + 40|0);
              $1031 = HEAP16[$1030>>1]|0;
              $1032 = $1031 << 16 >> 16;
              $1033 = ($1028|0)<($1032|0);
              if (!($1033)) {
               break;
              }
              $1034 = HEAP32[$pIdx>>2]|0;
              $1035 = ((($1034)) + 36|0);
              $1036 = HEAP32[$1035>>2]|0;
              $1037 = ($1036|0)==(0|0);
              if (!($1037)) {
               break;
              }
              $1038 = HEAP32[$pBest>>2]|0;
              $1039 = ($1038|0)!=(0|0);
              if ($1039) {
               $1040 = HEAP32[$pIdx>>2]|0;
               $1041 = ((($1040)) + 48|0);
               $1042 = HEAP16[$1041>>1]|0;
               $1043 = $1042 << 16 >> 16;
               $1044 = HEAP32[$pBest>>2]|0;
               $1045 = ((($1044)) + 48|0);
               $1046 = HEAP16[$1045>>1]|0;
               $1047 = $1046 << 16 >> 16;
               $1048 = ($1043|0)<($1047|0);
               if (!($1048)) {
                break;
               }
              }
              $1049 = HEAP32[$pIdx>>2]|0;
              HEAP32[$pBest>>2] = $1049;
             }
            } while(0);
            $1050 = HEAP32[$pIdx>>2]|0;
            $1051 = ((($1050)) + 20|0);
            $1052 = HEAP32[$1051>>2]|0;
            HEAP32[$pIdx>>2] = $1052;
           }
           $1053 = HEAP32[$pBest>>2]|0;
           $1054 = ($1053|0)!=(0|0);
           if ($1054) {
            $1055 = HEAP32[$pBest>>2]|0;
            $1056 = ((($1055)) + 44|0);
            $1057 = HEAP32[$1056>>2]|0;
            HEAP32[$iRoot>>2] = $1057;
            $1058 = HEAP32[$1>>2]|0;
            $1059 = HEAP32[$pBest>>2]|0;
            $1060 = (_sqlite3KeyInfoOfIndex($1058,$1059)|0);
            HEAP32[$pKeyInfo6>>2] = $1060;
           }
           $1061 = HEAP32[$v>>2]|0;
           $1062 = HEAP32[$iCsr>>2]|0;
           $1063 = HEAP32[$iRoot>>2]|0;
           $1064 = HEAP32[$iDb>>2]|0;
           (_sqlite3VdbeAddOp4Int($1061,104,$1062,$1063,$1064,1)|0);
           $1065 = HEAP32[$pKeyInfo6>>2]|0;
           $1066 = ($1065|0)!=(0|0);
           if ($1066) {
            $1067 = HEAP32[$v>>2]|0;
            $1068 = HEAP32[$pKeyInfo6>>2]|0;
            _sqlite3VdbeChangeP4($1067,-1,$1068,-6);
           }
           $1069 = HEAP32[$v>>2]|0;
           $1070 = HEAP32[$iCsr>>2]|0;
           $1071 = ((($sAggInfo)) + 40|0);
           $1072 = HEAP32[$1071>>2]|0;
           $1073 = ((($1072)) + 8|0);
           $1074 = HEAP32[$1073>>2]|0;
           (_sqlite3VdbeAddOp2($1069,100,$1070,$1074)|0);
           $1075 = HEAP32[$v>>2]|0;
           $1076 = HEAP32[$iCsr>>2]|0;
           (_sqlite3VdbeAddOp1($1075,111,$1076)|0);
           $1077 = HEAP32[$1>>2]|0;
           $1078 = HEAP32[$pTab5>>2]|0;
           $1079 = HEAP32[$pBest>>2]|0;
           _explainSimpleCount($1077,$1078,$1079);
          } else {
           HEAP32[$pMinMax>>2] = 0;
           HEAP8[$flag>>0] = 0;
           $1080 = HEAP32[$2>>2]|0;
           $1081 = ((($1080)) + 40|0);
           $1082 = HEAP32[$1081>>2]|0;
           $1083 = ($1082|0)==(0|0);
           if ($1083) {
            $1084 = (_minMaxQuery($sAggInfo,$pMinMax)|0);
            HEAP8[$flag>>0] = $1084;
           }
           $1085 = HEAP8[$flag>>0]|0;
           $1086 = ($1085<<24>>24)!=(0);
           do {
            if ($1086) {
             $1087 = HEAP32[$db>>2]|0;
             $1088 = HEAP32[$pMinMax>>2]|0;
             $1089 = (_sqlite3ExprListDup($1087,$1088,0)|0);
             HEAP32[$pMinMax>>2] = $1089;
             $1090 = HEAP32[$pMinMax>>2]|0;
             HEAP32[$pDel>>2] = $1090;
             $1091 = HEAP32[$db>>2]|0;
             $1092 = ((($1091)) + 69|0);
             $1093 = HEAP8[$1092>>0]|0;
             $1094 = ($1093<<24>>24)!=(0);
             if ($1094) {
              break;
             }
             $1095 = HEAP8[$flag>>0]|0;
             $1096 = $1095&255;
             $1097 = ($1096|0)!=(1);
             $1098 = $1097 ? 1 : 0;
             $1099 = $1098&255;
             $1100 = HEAP32[$pMinMax>>2]|0;
             $1101 = ((($1100)) + 4|0);
             $1102 = HEAP32[$1101>>2]|0;
             $1103 = ((($1102)) + 12|0);
             HEAP8[$1103>>0] = $1099;
             $1104 = HEAP32[$pMinMax>>2]|0;
             $1105 = ((($1104)) + 4|0);
             $1106 = HEAP32[$1105>>2]|0;
             $1107 = HEAP32[$1106>>2]|0;
             HEAP8[$1107>>0] = -104;
            }
           } while(0);
           $1108 = HEAP32[$1>>2]|0;
           _resetAccumulator($1108,$sAggInfo);
           $1109 = HEAP32[$1>>2]|0;
           $1110 = HEAP32[$pTabList>>2]|0;
           $1111 = HEAP32[$pWhere>>2]|0;
           $1112 = HEAP32[$pMinMax>>2]|0;
           $1113 = HEAP8[$flag>>0]|0;
           $1114 = $1113&255;
           $1115 = (_sqlite3WhereBegin($1109,$1110,$1111,$1112,0,$1114,0)|0);
           HEAP32[$pWInfo>>2] = $1115;
           $1116 = HEAP32[$pWInfo>>2]|0;
           $1117 = ($1116|0)==(0|0);
           if ($1117) {
            $1118 = HEAP32[$db>>2]|0;
            $1119 = HEAP32[$pDel>>2]|0;
            _sqlite3ExprListDelete($1118,$1119);
            break;
           }
           $1120 = HEAP32[$1>>2]|0;
           _updateAccumulator($1120,$sAggInfo);
           $1121 = HEAP32[$pWInfo>>2]|0;
           $1122 = (_sqlite3WhereIsOrdered($1121)|0);
           $1123 = ($1122|0)>(0);
           if ($1123) {
            $1124 = HEAP32[$v>>2]|0;
            $1125 = HEAP32[$pWInfo>>2]|0;
            $1126 = (_sqlite3WhereBreakLabel($1125)|0);
            (_sqlite3VdbeGoto($1124,$1126)|0);
           }
           $1127 = HEAP32[$pWInfo>>2]|0;
           _sqlite3WhereEnd($1127);
           $1128 = HEAP32[$1>>2]|0;
           _finalizeAggFunctions($1128,$sAggInfo);
          }
          HEAP32[$sSort>>2] = 0;
          $1129 = HEAP32[$1>>2]|0;
          $1130 = HEAP32[$pHaving>>2]|0;
          $1131 = HEAP32[$addrEnd>>2]|0;
          _sqlite3ExprIfFalse($1129,$1130,$1131,16);
          $1132 = HEAP32[$1>>2]|0;
          $1133 = HEAP32[$2>>2]|0;
          $1134 = HEAP32[$2>>2]|0;
          $1135 = HEAP32[$1134>>2]|0;
          $1136 = HEAP32[$3>>2]|0;
          $1137 = HEAP32[$addrEnd>>2]|0;
          $1138 = HEAP32[$addrEnd>>2]|0;
          _selectInnerLoop($1132,$1133,$1135,-1,0,0,$1136,$1137,$1138);
          $1139 = HEAP32[$db>>2]|0;
          $1140 = HEAP32[$pDel>>2]|0;
          _sqlite3ExprListDelete($1139,$1140);
         }
         $1141 = HEAP32[$v>>2]|0;
         $1142 = HEAP32[$addrEnd>>2]|0;
         _sqlite3VdbeResolveLabel($1141,$1142);
        }
        $1143 = ((($sDistinct)) + 1|0);
        $1144 = HEAP8[$1143>>0]|0;
        $1145 = $1144&255;
        $1146 = ($1145|0)==(3);
        if ($1146) {
         $1147 = HEAP32[$1>>2]|0;
         _explainTempTable($1147,35649);
        }
        $1148 = HEAP32[$sSort>>2]|0;
        $1149 = ($1148|0)!=(0|0);
        if ($1149) {
         $1150 = HEAP32[$1>>2]|0;
         $1151 = ((($sSort)) + 4|0);
         $1152 = HEAP32[$1151>>2]|0;
         $1153 = ($1152|0)>(0);
         $1154 = $1153 ? 35667 : 35690;
         _explainTempTable($1150,$1154);
         $1155 = HEAP32[$1>>2]|0;
         $1156 = HEAP32[$2>>2]|0;
         $1157 = HEAP32[$pEList>>2]|0;
         $1158 = HEAP32[$1157>>2]|0;
         $1159 = HEAP32[$3>>2]|0;
         _generateSortTail($1155,$1156,$sSort,$1158,$1159);
        }
        $1160 = HEAP32[$v>>2]|0;
        $1161 = HEAP32[$iEnd>>2]|0;
        _sqlite3VdbeResolveLabel($1160,$1161);
        $1162 = HEAP32[$1>>2]|0;
        $1163 = ((($1162)) + 36|0);
        $1164 = HEAP32[$1163>>2]|0;
        $1165 = ($1164|0)>(0);
        $1166 = $1165&1;
        HEAP32[$rc>>2] = $1166;
       }
      }
     }
    } while(0);
    $1167 = HEAP32[$iRestoreSelectId>>2]|0;
    $1168 = HEAP32[$1>>2]|0;
    $1169 = ((($1168)) + 420|0);
    HEAP32[$1169>>2] = $1167;
    $1170 = HEAP32[$rc>>2]|0;
    $1171 = ($1170|0)==(0);
    if ($1171) {
     $1172 = HEAP32[$3>>2]|0;
     $1173 = HEAP8[$1172>>0]|0;
     $1174 = $1173&255;
     $1175 = ($1174|0)==(9);
     if ($1175) {
      $1176 = HEAP32[$1>>2]|0;
      $1177 = HEAP32[$pTabList>>2]|0;
      $1178 = HEAP32[$pEList>>2]|0;
      _generateColumnNames($1176,$1177,$1178);
     }
    }
    $1179 = HEAP32[$db>>2]|0;
    $1180 = ((($sAggInfo)) + 28|0);
    $1181 = HEAP32[$1180>>2]|0;
    _sqlite3DbFree($1179,$1181);
    $1182 = HEAP32[$db>>2]|0;
    $1183 = ((($sAggInfo)) + 40|0);
    $1184 = HEAP32[$1183>>2]|0;
    _sqlite3DbFree($1182,$1184);
    $1185 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $1185;
    $1186 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($1186|0);
   }
  }
 }
 HEAP32[$0>>2] = 1;
 $1186 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($1186|0);
}
function _parserDoubleLinkSelect($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $mxSelect = 0, $pLoop = 0;
 var $pNext = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pNext = sp + 16|0;
 $pLoop = sp + 12|0;
 $mxSelect = sp + 8|0;
 $cnt = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$pNext>>2] = 0;
 HEAP32[$cnt>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pLoop>>2] = $6;
 while(1) {
  $7 = HEAP32[$pLoop>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$pNext>>2]|0;
  $10 = HEAP32[$pLoop>>2]|0;
  $11 = ((($10)) + 52|0);
  HEAP32[$11>>2] = $9;
  $12 = HEAP32[$pLoop>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $14 | 256;
  HEAP32[$13>>2] = $15;
  $16 = HEAP32[$pLoop>>2]|0;
  HEAP32[$pNext>>2] = $16;
  $17 = HEAP32[$pLoop>>2]|0;
  $18 = ((($17)) + 48|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pLoop>>2] = $19;
  $20 = HEAP32[$cnt>>2]|0;
  $21 = (($20) + 1)|0;
  HEAP32[$cnt>>2] = $21;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = $24 & 1024;
 $26 = ($25|0)==(0);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = ((($28)) + 96|0);
 $30 = ((($29)) + 16|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$mxSelect>>2] = $31;
 $32 = ($31|0)>(0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$cnt>>2]|0;
 $34 = HEAP32[$mxSelect>>2]|0;
 $35 = ($33|0)>($34|0);
 if (!($35)) {
  STACKTOP = sp;return;
 }
 $36 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($36,35575,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3SrcListAppendFromTerm($pParse,$p,$pTable,$pDatabase,$pAlias,$pSubquery,$pOn,$pUsing) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pTable = $pTable|0;
 $pDatabase = $pDatabase|0;
 $pAlias = $pAlias|0;
 $pSubquery = $pSubquery|0;
 $pOn = $pOn|0;
 $pUsing = $pUsing|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pItem = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $8 = sp + 12|0;
 $pItem = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pTable;
 HEAP32[$4>>2] = $pDatabase;
 HEAP32[$5>>2] = $pAlias;
 HEAP32[$6>>2] = $pSubquery;
 HEAP32[$7>>2] = $pOn;
 HEAP32[$8>>2] = $pUsing;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  label = 4;
 } else {
  $13 = HEAP32[$7>>2]|0;
  $14 = ($13|0)!=(0|0);
  $15 = HEAP32[$8>>2]|0;
  $16 = ($15|0)!=(0|0);
  $or$cond = $14 | $16;
  if ($or$cond) {
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$7>>2]|0;
   $19 = ($18|0)!=(0|0);
   $20 = $19 ? 35530 : 35533;
   HEAP32[$vararg_buffer>>2] = $20;
   _sqlite3ErrorMsg($17,35539,$vararg_buffer);
  } else {
   label = 4;
  }
 }
 if ((label|0) == 4) {
  $21 = HEAP32[$db>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  $24 = HEAP32[$4>>2]|0;
  $25 = (_sqlite3SrcListAppend($21,$22,$23,$24)|0);
  HEAP32[$2>>2] = $25;
  $26 = HEAP32[$2>>2]|0;
  $27 = ($26|0)==(0|0);
  if (!($27)) {
   $28 = HEAP32[$2>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0);
   if (!($30)) {
    $31 = HEAP32[$2>>2]|0;
    $32 = HEAP32[$31>>2]|0;
    $33 = (($32) - 1)|0;
    $34 = HEAP32[$2>>2]|0;
    $35 = ((($34)) + 8|0);
    $36 = (($35) + (($33*72)|0)|0);
    HEAP32[$pItem>>2] = $36;
    $37 = HEAP32[$5>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$db>>2]|0;
     $42 = HEAP32[$5>>2]|0;
     $43 = (_sqlite3NameFromToken($41,$42)|0);
     $44 = HEAP32[$pItem>>2]|0;
     $45 = ((($44)) + 12|0);
     HEAP32[$45>>2] = $43;
    }
    $46 = HEAP32[$6>>2]|0;
    $47 = HEAP32[$pItem>>2]|0;
    $48 = ((($47)) + 20|0);
    HEAP32[$48>>2] = $46;
    $49 = HEAP32[$7>>2]|0;
    $50 = HEAP32[$pItem>>2]|0;
    $51 = ((($50)) + 48|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$8>>2]|0;
    $53 = HEAP32[$pItem>>2]|0;
    $54 = ((($53)) + 52|0);
    HEAP32[$54>>2] = $52;
    $55 = HEAP32[$2>>2]|0;
    HEAP32[$0>>2] = $55;
    $62 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($62|0);
   }
  }
 }
 $56 = HEAP32[$db>>2]|0;
 $57 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($56,$57);
 $58 = HEAP32[$db>>2]|0;
 $59 = HEAP32[$8>>2]|0;
 _sqlite3IdListDelete($58,$59);
 $60 = HEAP32[$db>>2]|0;
 $61 = HEAP32[$6>>2]|0;
 _sqlite3SelectDelete($60,$61);
 HEAP32[$0>>2] = 0;
 $62 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3SelectNew($pParse,$pEList,$pSrc,$pWhere,$pGroupBy,$pHaving,$pOrderBy,$selFlags,$pLimit,$pOffset) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pSrc = $pSrc|0;
 $pWhere = $pWhere|0;
 $pGroupBy = $pGroupBy|0;
 $pHaving = $pHaving|0;
 $pOrderBy = $pOrderBy|0;
 $selFlags = $selFlags|0;
 $pLimit = $pLimit|0;
 $pOffset = $pOffset|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $db = 0, $pNew = 0, $standin = 0, $standin$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $6 = sp + 88|0;
 $7 = sp + 84|0;
 $8 = sp + 80|0;
 $9 = sp + 76|0;
 $pNew = sp + 72|0;
 $standin = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $pGroupBy;
 HEAP32[$5>>2] = $pHaving;
 HEAP32[$6>>2] = $pOrderBy;
 HEAP32[$7>>2] = $selFlags;
 HEAP32[$8>>2] = $pLimit;
 HEAP32[$9>>2] = $pOffset;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = (_sqlite3DbMallocRawNN($12,68,0)|0);
 HEAP32[$pNew>>2] = $13;
 $14 = HEAP32[$pNew>>2]|0;
 $15 = ($14|0)==(0|0);
 $standin$ = $15 ? $standin : $13;
 HEAP32[$pNew>>2] = $standin$;
 $16 = HEAP32[$1>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$db>>2]|0;
  $20 = (_sqlite3Expr($19,160,0)|0);
  $21 = (_sqlite3ExprListAppend($18,0,$20)|0);
  HEAP32[$1>>2] = $21;
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$pNew>>2]|0;
 HEAP32[$23>>2] = $22;
 $24 = HEAP32[$pNew>>2]|0;
 $25 = ((($24)) + 4|0);
 HEAP8[$25>>0] = 119;
 $26 = HEAP32[$7>>2]|0;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = ((($27)) + 8|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pNew>>2]|0;
 $30 = ((($29)) + 12|0);
 HEAP32[$30>>2] = 0;
 $31 = HEAP32[$pNew>>2]|0;
 $32 = ((($31)) + 16|0);
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$pNew>>2]|0;
 $34 = ((($33)) + 20|0);
 HEAP32[$34>>2] = -1;
 $35 = HEAP32[$pNew>>2]|0;
 $36 = ((($35)) + 20|0);
 $37 = ((($36)) + 4|0);
 HEAP32[$37>>2] = -1;
 $38 = HEAP32[$pNew>>2]|0;
 $39 = ((($38)) + 6|0);
 HEAP16[$39>>1] = 0;
 $40 = HEAP32[$2>>2]|0;
 $41 = ($40|0)==(0|0);
 if ($41) {
  $42 = HEAP32[$db>>2]|0;
  $43 = (_sqlite3DbMallocZero($42,80,0)|0);
  HEAP32[$2>>2] = $43;
 }
 $44 = HEAP32[$2>>2]|0;
 $45 = HEAP32[$pNew>>2]|0;
 $46 = ((($45)) + 28|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$3>>2]|0;
 $48 = HEAP32[$pNew>>2]|0;
 $49 = ((($48)) + 32|0);
 HEAP32[$49>>2] = $47;
 $50 = HEAP32[$4>>2]|0;
 $51 = HEAP32[$pNew>>2]|0;
 $52 = ((($51)) + 36|0);
 HEAP32[$52>>2] = $50;
 $53 = HEAP32[$5>>2]|0;
 $54 = HEAP32[$pNew>>2]|0;
 $55 = ((($54)) + 40|0);
 HEAP32[$55>>2] = $53;
 $56 = HEAP32[$6>>2]|0;
 $57 = HEAP32[$pNew>>2]|0;
 $58 = ((($57)) + 44|0);
 HEAP32[$58>>2] = $56;
 $59 = HEAP32[$pNew>>2]|0;
 $60 = ((($59)) + 48|0);
 HEAP32[$60>>2] = 0;
 $61 = HEAP32[$pNew>>2]|0;
 $62 = ((($61)) + 52|0);
 HEAP32[$62>>2] = 0;
 $63 = HEAP32[$8>>2]|0;
 $64 = HEAP32[$pNew>>2]|0;
 $65 = ((($64)) + 56|0);
 HEAP32[$65>>2] = $63;
 $66 = HEAP32[$9>>2]|0;
 $67 = HEAP32[$pNew>>2]|0;
 $68 = ((($67)) + 60|0);
 HEAP32[$68>>2] = $66;
 $69 = HEAP32[$pNew>>2]|0;
 $70 = ((($69)) + 64|0);
 HEAP32[$70>>2] = 0;
 $71 = HEAP32[$db>>2]|0;
 $72 = ((($71)) + 69|0);
 $73 = HEAP8[$72>>0]|0;
 $74 = ($73<<24>>24)!=(0);
 if (!($74)) {
  $80 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($80|0);
 }
 $75 = HEAP32[$db>>2]|0;
 $76 = HEAP32[$pNew>>2]|0;
 $77 = HEAP32[$pNew>>2]|0;
 $78 = ($77|0)!=($standin|0);
 $79 = $78&1;
 _clearSelect($75,$76,$79);
 HEAP32[$pNew>>2] = 0;
 $80 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($80|0);
}
function _sqlite3ExprListAppend($pParse,$pList,$pExpr) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $db = 0;
 var $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $db = sp + 8|0;
 $a = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pExpr;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$db>>2]|0;
  $9 = (_sqlite3DbMallocRawNN($8,8,0)|0);
  HEAP32[$2>>2] = $9;
  $10 = HEAP32[$2>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   label = 8;
  } else {
   $12 = HEAP32[$2>>2]|0;
   HEAP32[$12>>2] = 0;
   $13 = HEAP32[$db>>2]|0;
   $14 = (_sqlite3DbMallocRawNN($13,20,0)|0);
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 4|0);
   HEAP32[$16>>2] = $14;
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = ($19|0)==(0|0);
   if ($20) {
    label = 8;
   } else {
    label = 7;
   }
  }
 } else {
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) - 1)|0;
  $26 = $22 & $25;
  $27 = ($26|0)==(0);
  if ($27) {
   $28 = HEAP32[$db>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = ((($29)) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   $33 = HEAP32[$32>>2]|0;
   $34 = $33<<1;
   $35 = ($34*20)|0;
   $36 = (_sqlite3DbRealloc($28,$31,$35,0)|0);
   HEAP32[$a>>2] = $36;
   $37 = HEAP32[$a>>2]|0;
   $38 = ($37|0)==(0|0);
   if ($38) {
    label = 8;
   } else {
    $39 = HEAP32[$a>>2]|0;
    $40 = HEAP32[$2>>2]|0;
    $41 = ((($40)) + 4|0);
    HEAP32[$41>>2] = $39;
    label = 7;
   }
  } else {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $42 = HEAP32[$2>>2]|0;
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$42>>2] = $44;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 4|0);
  $47 = HEAP32[$46>>2]|0;
  $48 = (($47) + (($43*20)|0)|0);
  HEAP32[$pItem>>2] = $48;
  $49 = HEAP32[$pItem>>2]|0;
  ;HEAP32[$49>>2]=0|0;HEAP32[$49+4>>2]=0|0;HEAP32[$49+8>>2]=0|0;HEAP32[$49+12>>2]=0|0;HEAP32[$49+16>>2]=0|0;
  $50 = HEAP32[$3>>2]|0;
  $51 = HEAP32[$pItem>>2]|0;
  HEAP32[$51>>2] = $50;
  $52 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $52;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 8) {
  $53 = HEAP32[$db>>2]|0;
  $54 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($53,$54);
  $55 = HEAP32[$db>>2]|0;
  $56 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($55,$56);
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return (0)|0;
}
function _sqlite3ExprListSetName($pParse,$pList,$pName,$dequote) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pName = $pName|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $pName;
 HEAP32[$3>>2] = $dequote;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + (($8*20)|0)|0);
 HEAP32[$pItem>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3DbStrNDup($14,$16,$19,0)|0);
 $21 = HEAP32[$pItem>>2]|0;
 $22 = ((($21)) + 4|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$3>>2]|0;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$pItem>>2]|0;
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 _sqlite3Dequote($27);
 STACKTOP = sp;return;
}
function _sqlite3ExprListSetSpan($pParse,$pList,$pSpan) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pSpan = $pSpan|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $pSpan;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) - 1)|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*20)|0)|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = HEAP32[$pItem>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3DbFree($14,$17);
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $24;
 $29 = $27;
 $30 = (($28) - ($29))|0;
 $31 = ($30|0)<(0);
 $32 = $31 << 31 >> 31;
 $33 = (_sqlite3DbStrNDup($18,$21,$30,$32)|0);
 $34 = HEAP32[$pItem>>2]|0;
 $35 = ((($34)) + 8|0);
 HEAP32[$35>>2] = $33;
 STACKTOP = sp;return;
}
function _sqlite3Expr($db,$op,$zToken) {
 $db = $db|0;
 $op = $op|0;
 $zToken = $zToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $x = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $zToken;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$x>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_sqlite3Strlen30($6)|0);
  $9 = $7;
 } else {
  $9 = 0;
 }
 $8 = ((($x)) + 4|0);
 HEAP32[$8>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (_sqlite3ExprAlloc($10,$11,$x,0)|0);
 STACKTOP = sp;return ($12|0);
}
function _sqlite3SrcListShiftJoinType($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 HEAP32[$i>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = ($6|0)>(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = (($8) - 1)|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = (($11) + (($9*72)|0)|0);
  $13 = ((($12)) + 36|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = (($17) + (($15*72)|0)|0);
  $19 = ((($18)) + 36|0);
  HEAP8[$19>>0] = $14;
  $20 = HEAP32[$i>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$i>>2] = $21;
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 8|0);
 $24 = ((($23)) + 36|0);
 HEAP8[$24>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3SrcListIndexedBy($pParse,$p,$pIndexedBy) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pIndexedBy = $pIndexedBy|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pIndexedBy;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)>(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = (($12) + (($10*72)|0)|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   $21 = HEAP32[$pItem>>2]|0;
   $22 = ((($21)) + 36|0);
   $23 = ((($22)) + 1|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24 & -2;
   $26 = $25 | 1;
   HEAP8[$23>>0] = $26;
   STACKTOP = sp;return;
  }
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = (_sqlite3NameFromToken($28,$29)|0);
 $31 = HEAP32[$pItem>>2]|0;
 $32 = ((($31)) + 64|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$pItem>>2]|0;
 $34 = ((($33)) + 64|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)!=(0|0);
 $37 = $36&1;
 $38 = HEAP32[$pItem>>2]|0;
 $39 = ((($38)) + 36|0);
 $40 = ((($39)) + 1|0);
 $41 = $37&255;
 $42 = HEAP8[$40>>0]|0;
 $43 = $41 & 1;
 $44 = ($43 << 1)&255;
 $45 = $42 & -3;
 $46 = $45 | $44;
 HEAP8[$40>>0] = $46;
 STACKTOP = sp;return;
}
function _sqlite3SrcListFuncArgs($pParse,$p,$pList) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 8|0);
  $10 = (($9) + (($7*72)|0)|0);
  HEAP32[$pItem>>2] = $10;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$pItem>>2]|0;
  $13 = ((($12)) + 64|0);
  HEAP32[$13>>2] = $11;
  $14 = HEAP32[$pItem>>2]|0;
  $15 = ((($14)) + 36|0);
  $16 = ((($15)) + 1|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 & -5;
  $19 = $18 | 4;
  HEAP8[$16>>0] = $19;
  STACKTOP = sp;return;
 } else {
  $20 = HEAP32[$0>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($21,$22);
  STACKTOP = sp;return;
 }
}
function _sqlite3SrcListAppend($db,$pList,$pTable,$pDatabase) {
 $db = $db|0;
 $pList = $pList|0;
 $pTable = $pTable|0;
 $pDatabase = $pDatabase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, $pTemp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pItem = sp + 4|0;
 $pTemp = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pTable;
 HEAP32[$4>>2] = $pDatabase;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3DbMallocRawNN($7,80,0)|0);
   HEAP32[$2>>2] = $8;
   $9 = HEAP32[$2>>2]|0;
   $10 = ($9|0)==(0|0);
   if (!($10)) {
    $11 = HEAP32[$2>>2]|0;
    $12 = ((($11)) + 4|0);
    HEAP32[$12>>2] = 1;
    $13 = HEAP32[$2>>2]|0;
    HEAP32[$13>>2] = 0;
    break;
   }
   HEAP32[$0>>2] = 0;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
 } while(0);
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = (_sqlite3SrcListEnlarge($14,$15,1,$17)|0);
 HEAP32[$2>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 69|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = ($21<<24>>24)!=(0);
 if ($22) {
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  _sqlite3SrcListDelete($23,$24);
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) - 1)|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = (($29) + (($27*72)|0)|0);
 HEAP32[$pItem>>2] = $30;
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$4>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = ($34|0)==(0|0);
  if ($35) {
   HEAP32[$4>>2] = 0;
  }
 }
 $36 = HEAP32[$4>>2]|0;
 $37 = ($36|0)!=(0|0);
 if ($37) {
  $38 = HEAP32[$4>>2]|0;
  HEAP32[$pTemp>>2] = $38;
  $39 = HEAP32[$3>>2]|0;
  HEAP32[$4>>2] = $39;
  $40 = HEAP32[$pTemp>>2]|0;
  HEAP32[$3>>2] = $40;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 $43 = (_sqlite3NameFromToken($41,$42)|0);
 $44 = HEAP32[$pItem>>2]|0;
 $45 = ((($44)) + 8|0);
 HEAP32[$45>>2] = $43;
 $46 = HEAP32[$1>>2]|0;
 $47 = HEAP32[$4>>2]|0;
 $48 = (_sqlite3NameFromToken($46,$47)|0);
 $49 = HEAP32[$pItem>>2]|0;
 $50 = ((($49)) + 4|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3JoinType($pParse,$pA,$pB,$pC) {
 $pParse = $pParse|0;
 $pA = $pA|0;
 $pB = $pB|0;
 $pC = $pC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $apAll = 0;
 var $i = 0, $j = 0, $jointype = 0, $p = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zSp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $jointype = sp + 48|0;
 $apAll = sp + 36|0;
 $p = sp + 32|0;
 $i = sp + 28|0;
 $j = sp + 24|0;
 $zSp = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $pC;
 HEAP32[$jointype>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$apAll>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($apAll)) + 4|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$3>>2]|0;
 $8 = ((($apAll)) + 8|0);
 HEAP32[$8>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = ($9|0)<(3);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = (($apAll) + ($11<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = (($apAll) + ($15<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$p>>2] = $17;
  HEAP32[$j>>2] = 0;
  while(1) {
   $18 = HEAP32[$j>>2]|0;
   $19 = ($18|0)<(7);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$j>>2]|0;
   $24 = (35374 + (($23*3)|0)|0);
   $25 = ((($24)) + 1|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = ($22|0)==($27|0);
   if ($28) {
    $29 = HEAP32[$p>>2]|0;
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$j>>2]|0;
    $32 = (35374 + (($31*3)|0)|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = $33&255;
    $35 = (35395 + ($34)|0);
    $36 = HEAP32[$p>>2]|0;
    $37 = ((($36)) + 4|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (_sqlite3_strnicmp($30,$35,$38)|0);
    $40 = ($39|0)==(0);
    if ($40) {
     label = 8;
     break;
    }
   }
   $48 = HEAP32[$j>>2]|0;
   $49 = (($48) + 1)|0;
   HEAP32[$j>>2] = $49;
  }
  if ((label|0) == 8) {
   label = 0;
   $41 = HEAP32[$j>>2]|0;
   $42 = (35374 + (($41*3)|0)|0);
   $43 = ((($42)) + 2|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = HEAP32[$jointype>>2]|0;
   $47 = $46 | $45;
   HEAP32[$jointype>>2] = $47;
  }
  $50 = HEAP32[$j>>2]|0;
  $51 = ($50|0)>=(7);
  if ($51) {
   label = 11;
   break;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 11) {
  $52 = HEAP32[$jointype>>2]|0;
  $53 = $52 | 64;
  HEAP32[$jointype>>2] = $53;
 }
 $56 = HEAP32[$jointype>>2]|0;
 $57 = $56 & 33;
 $58 = ($57|0)==(33);
 if (!($58)) {
  $59 = HEAP32[$jointype>>2]|0;
  $60 = $59 & 64;
  $61 = ($60|0)!=(0);
  if (!($61)) {
   $71 = HEAP32[$jointype>>2]|0;
   $72 = $71 & 32;
   $73 = ($72|0)!=(0);
   if (!($73)) {
    $78 = HEAP32[$jointype>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
   $74 = HEAP32[$jointype>>2]|0;
   $75 = $74 & 24;
   $76 = ($75|0)!=(8);
   if (!($76)) {
    $78 = HEAP32[$jointype>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
   $77 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($77,35475,$vararg_buffer4);
   HEAP32[$jointype>>2] = 1;
   $78 = HEAP32[$jointype>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
 }
 HEAP32[$zSp>>2] = 35429;
 $62 = HEAP32[$3>>2]|0;
 $63 = ($62|0)==(0|0);
 if ($63) {
  $64 = HEAP32[$zSp>>2]|0;
  $65 = ((($64)) + 1|0);
  HEAP32[$zSp>>2] = $65;
 }
 $66 = HEAP32[$0>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 $68 = HEAP32[$2>>2]|0;
 $69 = HEAP32[$zSp>>2]|0;
 $70 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $67;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $68;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $69;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $70;
 _sqlite3ErrorMsg($66,35431,$vararg_buffer);
 HEAP32[$jointype>>2] = 1;
 $78 = HEAP32[$jointype>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3ExprListSetSortOrder($p,$iSortOrder) {
 $p = $p|0;
 $iSortOrder = $iSortOrder|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iSortOrder;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)==(0|0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)<(0);
 $or$cond = $3 | $5;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = $6&255;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*20)|0)|0);
 $15 = ((($14)) + 12|0);
 HEAP8[$15>>0] = $7;
 STACKTOP = sp;return;
}
function _sqlite3WithPush($pParse,$pWith,$bFree) {
 $pParse = $pParse|0;
 $pWith = $pWith|0;
 $bFree = $bFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pWith;
 HEAP8[$2>>0] = $bFree;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 472|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 4|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 472|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP8[$2>>0]|0;
 $14 = ($13<<24>>24)!=(0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 476|0);
 HEAP32[$17>>2] = $15;
 STACKTOP = sp;return;
}
function _sqlite3DeleteFrom($pParse,$pTabList,$pWhere) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aToOpen = 0, $addrBypass = 0, $addrEphOpen = 0;
 var $addrLoop = 0, $aiCurOnePass = 0, $bComplex = 0, $count = 0, $db = 0, $eOnePass = 0, $i = 0, $iAddrOnce = 0, $iDataCur = 0, $iDb = 0, $iEphCur = 0, $iIdxCur = 0, $iIdxNoSeek = 0, $iKey = 0, $iPk = 0, $iRowSet = 0, $iTabCur = 0, $isView = 0, $memCnt = 0, $nIdx = 0;
 var $nKey = 0, $nPk = 0, $or$cond = 0, $or$cond$not = 0, $or$cond3 = 0, $pIdx = 0, $pPk = 0, $pTab = 0, $pTrigger = 0, $pVTab = 0, $pWInfo = 0, $rcauth = 0, $sContext = 0, $sNC = 0, $v = 0, $wcf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $v = sp + 164|0;
 $pTab = sp + 160|0;
 $i = sp + 156|0;
 $pWInfo = sp + 152|0;
 $pIdx = sp + 148|0;
 $iTabCur = sp + 144|0;
 $iDataCur = sp + 140|0;
 $iIdxCur = sp + 136|0;
 $nIdx = sp + 132|0;
 $db = sp + 128|0;
 $sContext = sp + 120|0;
 $sNC = sp + 88|0;
 $iDb = sp + 84|0;
 $memCnt = sp + 80|0;
 $rcauth = sp + 76|0;
 $eOnePass = sp + 72|0;
 $aiCurOnePass = sp + 64|0;
 $aToOpen = sp + 60|0;
 $pPk = sp + 56|0;
 $iPk = sp + 52|0;
 $nPk = sp + 184|0;
 $iKey = sp + 48|0;
 $nKey = sp + 182|0;
 $iEphCur = sp + 44|0;
 $iRowSet = sp + 40|0;
 $addrBypass = sp + 36|0;
 $addrLoop = sp + 32|0;
 $addrEphOpen = sp + 28|0;
 $bComplex = sp + 24|0;
 $isView = sp + 20|0;
 $pTrigger = sp + 16|0;
 $wcf = sp + 180|0;
 $iAddrOnce = sp + 12|0;
 $pVTab = sp + 8|0;
 $count = sp + 4|0;
 $iIdxNoSeek = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pWhere;
 HEAP32[$iDataCur>>2] = 0;
 HEAP32[$iIdxCur>>2] = 0;
 HEAP32[$memCnt>>2] = -1;
 HEAP32[$aToOpen>>2] = 0;
 HEAP32[$iPk>>2] = 0;
 HEAP16[$nPk>>1] = 1;
 HEAP32[$iEphCur>>2] = 0;
 HEAP32[$iRowSet>>2] = 0;
 HEAP32[$addrBypass>>2] = 0;
 HEAP32[$addrLoop>>2] = 0;
 HEAP32[$addrEphOpen>>2] = 0;
 ;HEAP32[$sContext>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 L1: do {
  if (!($8)) {
   $9 = HEAP32[$db>>2]|0;
   $10 = ((($9)) + 69|0);
   $11 = HEAP8[$10>>0]|0;
   $12 = $11&255;
   $13 = ($12|0)!=(0);
   if (!($13)) {
    $14 = HEAP32[$0>>2]|0;
    $15 = HEAP32[$1>>2]|0;
    $16 = (_sqlite3SrcListLookup($14,$15)|0);
    HEAP32[$pTab>>2] = $16;
    $17 = HEAP32[$pTab>>2]|0;
    $18 = ($17|0)==(0|0);
    if (!($18)) {
     $19 = HEAP32[$0>>2]|0;
     $20 = HEAP32[$pTab>>2]|0;
     $21 = (_sqlite3TriggersExist($19,$20,109,0,0)|0);
     HEAP32[$pTrigger>>2] = $21;
     $22 = HEAP32[$pTab>>2]|0;
     $23 = ((($22)) + 12|0);
     $24 = HEAP32[$23>>2]|0;
     $25 = ($24|0)!=(0|0);
     $26 = $25&1;
     HEAP32[$isView>>2] = $26;
     $27 = HEAP32[$pTrigger>>2]|0;
     $28 = ($27|0)!=(0|0);
     if ($28) {
      $34 = 1;
     } else {
      $29 = HEAP32[$0>>2]|0;
      $30 = HEAP32[$pTab>>2]|0;
      $31 = (_sqlite3FkRequired($29,$30,0,0)|0);
      $32 = ($31|0)!=(0);
      $34 = $32;
     }
     $33 = $34&1;
     HEAP32[$bComplex>>2] = $33;
     $35 = HEAP32[$0>>2]|0;
     $36 = HEAP32[$pTab>>2]|0;
     $37 = (_sqlite3ViewGetColumnNames($35,$36)|0);
     $38 = ($37|0)!=(0);
     if (!($38)) {
      $39 = HEAP32[$0>>2]|0;
      $40 = HEAP32[$pTab>>2]|0;
      $41 = HEAP32[$pTrigger>>2]|0;
      $42 = ($41|0)!=(0|0);
      $43 = $42 ? 1 : 0;
      $44 = (_sqlite3IsReadOnly($39,$40,$43)|0);
      $45 = ($44|0)!=(0);
      if (!($45)) {
       $46 = HEAP32[$db>>2]|0;
       $47 = HEAP32[$pTab>>2]|0;
       $48 = ((($47)) + 64|0);
       $49 = HEAP32[$48>>2]|0;
       $50 = (_sqlite3SchemaToIndex($46,$49)|0);
       HEAP32[$iDb>>2] = $50;
       $51 = HEAP32[$0>>2]|0;
       $52 = HEAP32[$pTab>>2]|0;
       $53 = HEAP32[$52>>2]|0;
       $54 = HEAP32[$iDb>>2]|0;
       $55 = HEAP32[$db>>2]|0;
       $56 = ((($55)) + 16|0);
       $57 = HEAP32[$56>>2]|0;
       $58 = (($57) + ($54<<4)|0);
       $59 = HEAP32[$58>>2]|0;
       $60 = (_sqlite3AuthCheck($51,9,$53,0,$59)|0);
       HEAP32[$rcauth>>2] = $60;
       $61 = HEAP32[$rcauth>>2]|0;
       $62 = ($61|0)==(1);
       if (!($62)) {
        $63 = HEAP32[$0>>2]|0;
        $64 = ((($63)) + 40|0);
        $65 = HEAP32[$64>>2]|0;
        $66 = (($65) + 1)|0;
        HEAP32[$64>>2] = $66;
        $67 = HEAP32[$1>>2]|0;
        $68 = ((($67)) + 8|0);
        $69 = ((($68)) + 44|0);
        HEAP32[$69>>2] = $65;
        HEAP32[$iTabCur>>2] = $65;
        HEAP32[$nIdx>>2] = 0;
        $70 = HEAP32[$pTab>>2]|0;
        $71 = ((($70)) + 8|0);
        $72 = HEAP32[$71>>2]|0;
        HEAP32[$pIdx>>2] = $72;
        while(1) {
         $73 = HEAP32[$pIdx>>2]|0;
         $74 = ($73|0)!=(0|0);
         if (!($74)) {
          break;
         }
         $75 = HEAP32[$0>>2]|0;
         $76 = ((($75)) + 40|0);
         $77 = HEAP32[$76>>2]|0;
         $78 = (($77) + 1)|0;
         HEAP32[$76>>2] = $78;
         $79 = HEAP32[$pIdx>>2]|0;
         $80 = ((($79)) + 20|0);
         $81 = HEAP32[$80>>2]|0;
         HEAP32[$pIdx>>2] = $81;
         $82 = HEAP32[$nIdx>>2]|0;
         $83 = (($82) + 1)|0;
         HEAP32[$nIdx>>2] = $83;
        }
        $84 = HEAP32[$isView>>2]|0;
        $85 = ($84|0)!=(0);
        if ($85) {
         $86 = HEAP32[$0>>2]|0;
         $87 = HEAP32[$pTab>>2]|0;
         $88 = HEAP32[$87>>2]|0;
         _sqlite3AuthContextPush($86,$sContext,$88);
        }
        $89 = HEAP32[$0>>2]|0;
        $90 = (_sqlite3GetVdbe($89)|0);
        HEAP32[$v>>2] = $90;
        $91 = HEAP32[$v>>2]|0;
        $92 = ($91|0)==(0|0);
        if (!($92)) {
         $93 = HEAP32[$0>>2]|0;
         $94 = ((($93)) + 18|0);
         $95 = HEAP8[$94>>0]|0;
         $96 = $95&255;
         $97 = ($96|0)==(0);
         if ($97) {
          $98 = HEAP32[$v>>2]|0;
          _sqlite3VdbeCountChanges($98);
         }
         $99 = HEAP32[$0>>2]|0;
         $100 = HEAP32[$iDb>>2]|0;
         _sqlite3BeginWriteOperation($99,1,$100);
         $101 = HEAP32[$isView>>2]|0;
         $102 = ($101|0)!=(0);
         if ($102) {
          $103 = HEAP32[$0>>2]|0;
          $104 = HEAP32[$pTab>>2]|0;
          $105 = HEAP32[$2>>2]|0;
          $106 = HEAP32[$iTabCur>>2]|0;
          _sqlite3MaterializeView($103,$104,$105,$106);
          $107 = HEAP32[$iTabCur>>2]|0;
          HEAP32[$iIdxCur>>2] = $107;
          HEAP32[$iDataCur>>2] = $107;
         }
         ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
         $108 = HEAP32[$0>>2]|0;
         HEAP32[$sNC>>2] = $108;
         $109 = HEAP32[$1>>2]|0;
         $110 = ((($sNC)) + 4|0);
         HEAP32[$110>>2] = $109;
         $111 = HEAP32[$2>>2]|0;
         $112 = (_sqlite3ResolveExprNames($sNC,$111)|0);
         $113 = ($112|0)!=(0);
         if (!($113)) {
          $114 = HEAP32[$db>>2]|0;
          $115 = ((($114)) + 24|0);
          $116 = HEAP32[$115>>2]|0;
          $117 = $116 & 128;
          $118 = ($117|0)!=(0);
          if ($118) {
           $119 = HEAP32[$0>>2]|0;
           $120 = ((($119)) + 44|0);
           $121 = HEAP32[$120>>2]|0;
           $122 = (($121) + 1)|0;
           HEAP32[$120>>2] = $122;
           HEAP32[$memCnt>>2] = $122;
           $123 = HEAP32[$v>>2]|0;
           $124 = HEAP32[$memCnt>>2]|0;
           (_sqlite3VdbeAddOp2($123,76,0,$124)|0);
          }
          $125 = HEAP32[$rcauth>>2]|0;
          $126 = ($125|0)==(0);
          $127 = HEAP32[$2>>2]|0;
          $128 = ($127|0)==(0|0);
          $or$cond = $126 & $128;
          $or$cond$not = $or$cond ^ 1;
          $129 = HEAP32[$bComplex>>2]|0;
          $130 = ($129|0)!=(0);
          $or$cond3 = $or$cond$not | $130;
          L29: do {
           if ($or$cond3) {
            label = 29;
           } else {
            $131 = HEAP32[$pTab>>2]|0;
            $132 = ((($131)) + 42|0);
            $133 = HEAP8[$132>>0]|0;
            $134 = $133&255;
            $135 = $134 & 16;
            $136 = ($135|0)!=(0);
            if ($136) {
             label = 29;
            } else {
             $137 = HEAP32[$0>>2]|0;
             $138 = HEAP32[$iDb>>2]|0;
             $139 = HEAP32[$pTab>>2]|0;
             $140 = ((($139)) + 28|0);
             $141 = HEAP32[$140>>2]|0;
             $142 = HEAP32[$pTab>>2]|0;
             $143 = HEAP32[$142>>2]|0;
             _sqlite3TableLock($137,$138,$141,1,$143);
             $144 = HEAP32[$pTab>>2]|0;
             $145 = ((($144)) + 42|0);
             $146 = HEAP8[$145>>0]|0;
             $147 = $146&255;
             $148 = $147 & 32;
             $149 = ($148|0)==(0);
             if ($149) {
              $150 = HEAP32[$v>>2]|0;
              $151 = HEAP32[$pTab>>2]|0;
              $152 = ((($151)) + 28|0);
              $153 = HEAP32[$152>>2]|0;
              $154 = HEAP32[$iDb>>2]|0;
              $155 = HEAP32[$memCnt>>2]|0;
              $156 = HEAP32[$pTab>>2]|0;
              $157 = HEAP32[$156>>2]|0;
              (_sqlite3VdbeAddOp4($150,131,$153,$154,$155,$157,-2)|0);
             }
             $158 = HEAP32[$pTab>>2]|0;
             $159 = ((($158)) + 8|0);
             $160 = HEAP32[$159>>2]|0;
             HEAP32[$pIdx>>2] = $160;
             while(1) {
              $161 = HEAP32[$pIdx>>2]|0;
              $162 = ($161|0)!=(0|0);
              if (!($162)) {
               break L29;
              }
              $163 = HEAP32[$v>>2]|0;
              $164 = HEAP32[$pIdx>>2]|0;
              $165 = ((($164)) + 44|0);
              $166 = HEAP32[$165>>2]|0;
              $167 = HEAP32[$iDb>>2]|0;
              (_sqlite3VdbeAddOp2($163,131,$166,$167)|0);
              $168 = HEAP32[$pIdx>>2]|0;
              $169 = ((($168)) + 20|0);
              $170 = HEAP32[$169>>2]|0;
              HEAP32[$pIdx>>2] = $170;
             }
            }
           }
          } while(0);
          L38: do {
           if ((label|0) == 29) {
            HEAP16[$wcf>>1] = 1044;
            $171 = ((($sNC)) + 28|0);
            $172 = HEAP16[$171>>1]|0;
            $173 = $172&65535;
            $174 = $173 & 64;
            $175 = ($174|0)!=(0);
            if ($175) {
             HEAP32[$bComplex>>2] = 1;
            }
            $176 = HEAP32[$bComplex>>2]|0;
            $177 = ($176|0)!=(0);
            $178 = $177 ? 0 : 8;
            $179 = HEAP16[$wcf>>1]|0;
            $180 = $179&65535;
            $181 = $180 | $178;
            $182 = $181&65535;
            HEAP16[$wcf>>1] = $182;
            $183 = HEAP32[$pTab>>2]|0;
            $184 = ((($183)) + 42|0);
            $185 = HEAP8[$184>>0]|0;
            $186 = $185&255;
            $187 = $186 & 32;
            $188 = ($187|0)==(0);
            if ($188) {
             HEAP32[$pPk>>2] = 0;
             HEAP16[$nPk>>1] = 1;
             $189 = HEAP32[$0>>2]|0;
             $190 = ((($189)) + 44|0);
             $191 = HEAP32[$190>>2]|0;
             $192 = (($191) + 1)|0;
             HEAP32[$190>>2] = $192;
             HEAP32[$iRowSet>>2] = $192;
             $193 = HEAP32[$v>>2]|0;
             $194 = HEAP32[$iRowSet>>2]|0;
             (_sqlite3VdbeAddOp2($193,79,0,$194)|0);
            } else {
             $195 = HEAP32[$pTab>>2]|0;
             $196 = (_sqlite3PrimaryKeyIndex($195)|0);
             HEAP32[$pPk>>2] = $196;
             $197 = HEAP32[$pPk>>2]|0;
             $198 = ((($197)) + 50|0);
             $199 = HEAP16[$198>>1]|0;
             HEAP16[$nPk>>1] = $199;
             $200 = HEAP32[$0>>2]|0;
             $201 = ((($200)) + 44|0);
             $202 = HEAP32[$201>>2]|0;
             $203 = (($202) + 1)|0;
             HEAP32[$iPk>>2] = $203;
             $204 = HEAP16[$nPk>>1]|0;
             $205 = $204 << 16 >> 16;
             $206 = HEAP32[$0>>2]|0;
             $207 = ((($206)) + 44|0);
             $208 = HEAP32[$207>>2]|0;
             $209 = (($208) + ($205))|0;
             HEAP32[$207>>2] = $209;
             $210 = HEAP32[$0>>2]|0;
             $211 = ((($210)) + 40|0);
             $212 = HEAP32[$211>>2]|0;
             $213 = (($212) + 1)|0;
             HEAP32[$211>>2] = $213;
             HEAP32[$iEphCur>>2] = $212;
             $214 = HEAP32[$v>>2]|0;
             $215 = HEAP32[$iEphCur>>2]|0;
             $216 = HEAP16[$nPk>>1]|0;
             $217 = $216 << 16 >> 16;
             $218 = (_sqlite3VdbeAddOp2($214,107,$215,$217)|0);
             HEAP32[$addrEphOpen>>2] = $218;
             $219 = HEAP32[$0>>2]|0;
             $220 = HEAP32[$pPk>>2]|0;
             _sqlite3VdbeSetP4KeyInfo($219,$220);
            }
            $221 = HEAP32[$0>>2]|0;
            $222 = HEAP32[$1>>2]|0;
            $223 = HEAP32[$2>>2]|0;
            $224 = HEAP16[$wcf>>1]|0;
            $225 = HEAP32[$iTabCur>>2]|0;
            $226 = (($225) + 1)|0;
            $227 = (_sqlite3WhereBegin($221,$222,$223,0,0,$224,$226)|0);
            HEAP32[$pWInfo>>2] = $227;
            $228 = HEAP32[$pWInfo>>2]|0;
            $229 = ($228|0)==(0|0);
            if ($229) {
             break L1;
            }
            $230 = HEAP32[$pWInfo>>2]|0;
            $231 = (_sqlite3WhereOkOnePass($230,$aiCurOnePass)|0);
            HEAP32[$eOnePass>>2] = $231;
            $232 = HEAP32[$db>>2]|0;
            $233 = ((($232)) + 24|0);
            $234 = HEAP32[$233>>2]|0;
            $235 = $234 & 128;
            $236 = ($235|0)!=(0);
            if ($236) {
             $237 = HEAP32[$v>>2]|0;
             $238 = HEAP32[$memCnt>>2]|0;
             (_sqlite3VdbeAddOp2($237,91,$238,1)|0);
            }
            $239 = HEAP32[$pPk>>2]|0;
            $240 = ($239|0)!=(0|0);
            do {
             if ($240) {
              HEAP32[$i>>2] = 0;
              while(1) {
               $241 = HEAP32[$i>>2]|0;
               $242 = HEAP16[$nPk>>1]|0;
               $243 = $242 << 16 >> 16;
               $244 = ($241|0)<($243|0);
               if (!($244)) {
                break;
               }
               $245 = HEAP32[$v>>2]|0;
               $246 = HEAP32[$pTab>>2]|0;
               $247 = HEAP32[$iTabCur>>2]|0;
               $248 = HEAP32[$i>>2]|0;
               $249 = HEAP32[$pPk>>2]|0;
               $250 = ((($249)) + 4|0);
               $251 = HEAP32[$250>>2]|0;
               $252 = (($251) + ($248<<1)|0);
               $253 = HEAP16[$252>>1]|0;
               $254 = $253 << 16 >> 16;
               $255 = HEAP32[$iPk>>2]|0;
               $256 = HEAP32[$i>>2]|0;
               $257 = (($255) + ($256))|0;
               _sqlite3ExprCodeGetColumnOfTable($245,$246,$247,$254,$257);
               $258 = HEAP32[$i>>2]|0;
               $259 = (($258) + 1)|0;
               HEAP32[$i>>2] = $259;
              }
              $260 = HEAP32[$iPk>>2]|0;
              HEAP32[$iKey>>2] = $260;
             } else {
              $261 = HEAP32[$0>>2]|0;
              $262 = ((($261)) + 44|0);
              $263 = HEAP32[$262>>2]|0;
              $264 = (($263) + 1)|0;
              HEAP32[$iKey>>2] = $264;
              $265 = HEAP32[$0>>2]|0;
              $266 = HEAP32[$pTab>>2]|0;
              $267 = HEAP32[$iTabCur>>2]|0;
              $268 = HEAP32[$iKey>>2]|0;
              $269 = (_sqlite3ExprCodeGetColumn($265,$266,-1,$267,$268,0)|0);
              HEAP32[$iKey>>2] = $269;
              $270 = HEAP32[$iKey>>2]|0;
              $271 = HEAP32[$0>>2]|0;
              $272 = ((($271)) + 44|0);
              $273 = HEAP32[$272>>2]|0;
              $274 = ($270|0)>($273|0);
              if (!($274)) {
               break;
              }
              $275 = HEAP32[$iKey>>2]|0;
              $276 = HEAP32[$0>>2]|0;
              $277 = ((($276)) + 44|0);
              HEAP32[$277>>2] = $275;
             }
            } while(0);
            $278 = HEAP32[$eOnePass>>2]|0;
            $279 = ($278|0)!=(0);
            do {
             if ($279) {
              $280 = HEAP16[$nPk>>1]|0;
              HEAP16[$nKey>>1] = $280;
              $281 = HEAP32[$db>>2]|0;
              $282 = HEAP32[$nIdx>>2]|0;
              $283 = (($282) + 2)|0;
              $284 = ($283|0)<(0);
              $285 = $284 << 31 >> 31;
              $286 = (_sqlite3DbMallocRawNN($281,$283,$285)|0);
              HEAP32[$aToOpen>>2] = $286;
              $287 = HEAP32[$aToOpen>>2]|0;
              $288 = ($287|0)==(0|0);
              if ($288) {
               $289 = HEAP32[$pWInfo>>2]|0;
               _sqlite3WhereEnd($289);
               break L1;
              }
              $290 = HEAP32[$aToOpen>>2]|0;
              $291 = HEAP32[$nIdx>>2]|0;
              $292 = (($291) + 1)|0;
              _memset(($290|0),1,($292|0))|0;
              $293 = HEAP32[$nIdx>>2]|0;
              $294 = (($293) + 1)|0;
              $295 = HEAP32[$aToOpen>>2]|0;
              $296 = (($295) + ($294)|0);
              HEAP8[$296>>0] = 0;
              $297 = HEAP32[$aiCurOnePass>>2]|0;
              $298 = ($297|0)>=(0);
              if ($298) {
               $299 = HEAP32[$aiCurOnePass>>2]|0;
               $300 = HEAP32[$iTabCur>>2]|0;
               $301 = (($299) - ($300))|0;
               $302 = HEAP32[$aToOpen>>2]|0;
               $303 = (($302) + ($301)|0);
               HEAP8[$303>>0] = 0;
              }
              $304 = ((($aiCurOnePass)) + 4|0);
              $305 = HEAP32[$304>>2]|0;
              $306 = ($305|0)>=(0);
              if ($306) {
               $307 = ((($aiCurOnePass)) + 4|0);
               $308 = HEAP32[$307>>2]|0;
               $309 = HEAP32[$iTabCur>>2]|0;
               $310 = (($308) - ($309))|0;
               $311 = HEAP32[$aToOpen>>2]|0;
               $312 = (($311) + ($310)|0);
               HEAP8[$312>>0] = 0;
              }
              $313 = HEAP32[$addrEphOpen>>2]|0;
              $314 = ($313|0)!=(0);
              if (!($314)) {
               break;
              }
              $315 = HEAP32[$v>>2]|0;
              $316 = HEAP32[$addrEphOpen>>2]|0;
              (_sqlite3VdbeChangeToNoop($315,$316)|0);
             } else {
              $317 = HEAP32[$pPk>>2]|0;
              $318 = ($317|0)!=(0|0);
              if ($318) {
               $319 = HEAP32[$0>>2]|0;
               $320 = ((($319)) + 44|0);
               $321 = HEAP32[$320>>2]|0;
               $322 = (($321) + 1)|0;
               HEAP32[$320>>2] = $322;
               HEAP32[$iKey>>2] = $322;
               HEAP16[$nKey>>1] = 0;
               $323 = HEAP32[$v>>2]|0;
               $324 = HEAP32[$iPk>>2]|0;
               $325 = HEAP16[$nPk>>1]|0;
               $326 = $325 << 16 >> 16;
               $327 = HEAP32[$iKey>>2]|0;
               $328 = HEAP32[$0>>2]|0;
               $329 = HEAP32[$328>>2]|0;
               $330 = HEAP32[$pPk>>2]|0;
               $331 = (_sqlite3IndexAffinityStr($329,$330)|0);
               $332 = HEAP16[$nPk>>1]|0;
               $333 = $332 << 16 >> 16;
               (_sqlite3VdbeAddOp4($323,99,$324,$326,$327,$331,$333)|0);
               $334 = HEAP32[$v>>2]|0;
               $335 = HEAP32[$iEphCur>>2]|0;
               $336 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($334,126,$335,$336)|0);
               break;
              } else {
               HEAP16[$nKey>>1] = 1;
               $337 = HEAP32[$v>>2]|0;
               $338 = HEAP32[$iRowSet>>2]|0;
               $339 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($337,142,$338,$339)|0);
               break;
              }
             }
            } while(0);
            $340 = HEAP32[$eOnePass>>2]|0;
            $341 = ($340|0)!=(0);
            if ($341) {
             $342 = HEAP32[$v>>2]|0;
             $343 = (_sqlite3VdbeMakeLabel($342)|0);
             HEAP32[$addrBypass>>2] = $343;
            } else {
             $344 = HEAP32[$pWInfo>>2]|0;
             _sqlite3WhereEnd($344);
            }
            $345 = HEAP32[$isView>>2]|0;
            $346 = ($345|0)!=(0);
            do {
             if (!($346)) {
              HEAP32[$iAddrOnce>>2] = 0;
              $347 = HEAP32[$eOnePass>>2]|0;
              $348 = ($347|0)==(2);
              if ($348) {
               $349 = HEAP32[$v>>2]|0;
               $350 = (_sqlite3VdbeAddOp0($349,20)|0);
               HEAP32[$iAddrOnce>>2] = $350;
              }
              $351 = HEAP32[$0>>2]|0;
              $352 = HEAP32[$pTab>>2]|0;
              $353 = HEAP32[$iTabCur>>2]|0;
              $354 = HEAP32[$aToOpen>>2]|0;
              (_sqlite3OpenTableAndIndices($351,$352,105,8,$353,$354,$iDataCur,$iIdxCur)|0);
              $355 = HEAP32[$eOnePass>>2]|0;
              $356 = ($355|0)==(2);
              if (!($356)) {
               break;
              }
              $357 = HEAP32[$v>>2]|0;
              $358 = HEAP32[$iAddrOnce>>2]|0;
              _sqlite3VdbeJumpHere($357,$358);
             }
            } while(0);
            $359 = HEAP32[$eOnePass>>2]|0;
            $360 = ($359|0)!=(0);
            do {
             if ($360) {
              $361 = HEAP32[$pTab>>2]|0;
              $362 = ((($361)) + 42|0);
              $363 = HEAP8[$362>>0]|0;
              $364 = $363&255;
              $365 = $364 & 16;
              $366 = ($365|0)!=(0);
              if ($366) {
               break;
              }
              $367 = HEAP32[$iDataCur>>2]|0;
              $368 = HEAP32[$iTabCur>>2]|0;
              $369 = (($367) - ($368))|0;
              $370 = HEAP32[$aToOpen>>2]|0;
              $371 = (($370) + ($369)|0);
              $372 = HEAP8[$371>>0]|0;
              $373 = $372&255;
              $374 = ($373|0)!=(0);
              if (!($374)) {
               break;
              }
              $375 = HEAP32[$v>>2]|0;
              $376 = HEAP32[$iDataCur>>2]|0;
              $377 = HEAP32[$addrBypass>>2]|0;
              $378 = HEAP32[$iKey>>2]|0;
              $379 = HEAP16[$nKey>>1]|0;
              $380 = $379 << 16 >> 16;
              (_sqlite3VdbeAddOp4Int($375,30,$376,$377,$378,$380)|0);
             } else {
              $381 = HEAP32[$pPk>>2]|0;
              $382 = ($381|0)!=(0|0);
              $383 = HEAP32[$v>>2]|0;
              if ($382) {
               $384 = HEAP32[$iEphCur>>2]|0;
               $385 = (_sqlite3VdbeAddOp1($383,57,$384)|0);
               HEAP32[$addrLoop>>2] = $385;
               $386 = HEAP32[$v>>2]|0;
               $387 = HEAP32[$iEphCur>>2]|0;
               $388 = HEAP32[$iKey>>2]|0;
               (_sqlite3VdbeAddOp2($386,121,$387,$388)|0);
               break;
              } else {
               $389 = HEAP32[$iRowSet>>2]|0;
               $390 = HEAP32[$iKey>>2]|0;
               $391 = (_sqlite3VdbeAddOp3($383,62,$389,0,$390)|0);
               HEAP32[$addrLoop>>2] = $391;
               break;
              }
             }
            } while(0);
            $392 = HEAP32[$pTab>>2]|0;
            $393 = ((($392)) + 42|0);
            $394 = HEAP8[$393>>0]|0;
            $395 = $394&255;
            $396 = $395 & 16;
            $397 = ($396|0)!=(0);
            do {
             if ($397) {
              $398 = HEAP32[$db>>2]|0;
              $399 = HEAP32[$pTab>>2]|0;
              $400 = (_sqlite3GetVTable($398,$399)|0);
              HEAP32[$pVTab>>2] = $400;
              $401 = HEAP32[$0>>2]|0;
              $402 = HEAP32[$pTab>>2]|0;
              _sqlite3VtabMakeWritable($401,$402);
              $403 = HEAP32[$v>>2]|0;
              $404 = HEAP32[$iKey>>2]|0;
              $405 = HEAP32[$pVTab>>2]|0;
              (_sqlite3VdbeAddOp4($403,12,0,1,$404,$405,-10)|0);
              $406 = HEAP32[$v>>2]|0;
              _sqlite3VdbeChangeP5($406,2);
              $407 = HEAP32[$0>>2]|0;
              _sqlite3MayAbort($407);
              $408 = HEAP32[$eOnePass>>2]|0;
              $409 = ($408|0)==(1);
              if (!($409)) {
               break;
              }
              $410 = HEAP32[$0>>2]|0;
              $411 = ((($410)) + 124|0);
              $412 = HEAP32[$411>>2]|0;
              $413 = ($412|0)==(0|0);
              if (!($413)) {
               break;
              }
              $414 = HEAP32[$0>>2]|0;
              $415 = ((($414)) + 20|0);
              HEAP8[$415>>0] = 0;
             } else {
              $416 = HEAP32[$0>>2]|0;
              $417 = ((($416)) + 18|0);
              $418 = HEAP8[$417>>0]|0;
              $419 = $418&255;
              $420 = ($419|0)==(0);
              $421 = $420&1;
              HEAP32[$count>>2] = $421;
              HEAP32[$iIdxNoSeek>>2] = -1;
              $422 = HEAP32[$bComplex>>2]|0;
              $423 = ($422|0)==(0);
              do {
               if ($423) {
                $424 = ((($aiCurOnePass)) + 4|0);
                $425 = HEAP32[$424>>2]|0;
                $426 = HEAP32[$iDataCur>>2]|0;
                $427 = ($425|0)!=($426|0);
                if (!($427)) {
                 break;
                }
                $428 = ((($aiCurOnePass)) + 4|0);
                $429 = HEAP32[$428>>2]|0;
                HEAP32[$iIdxNoSeek>>2] = $429;
               }
              } while(0);
              $430 = HEAP32[$0>>2]|0;
              $431 = HEAP32[$pTab>>2]|0;
              $432 = HEAP32[$pTrigger>>2]|0;
              $433 = HEAP32[$iDataCur>>2]|0;
              $434 = HEAP32[$iIdxCur>>2]|0;
              $435 = HEAP32[$iKey>>2]|0;
              $436 = HEAP16[$nKey>>1]|0;
              $437 = HEAP32[$count>>2]|0;
              $438 = $437&255;
              $439 = HEAP32[$eOnePass>>2]|0;
              $440 = $439&255;
              $441 = HEAP32[$iIdxNoSeek>>2]|0;
              _sqlite3GenerateRowDelete($430,$431,$432,$433,$434,$435,$436,$438,10,$440,$441);
             }
            } while(0);
            $442 = HEAP32[$eOnePass>>2]|0;
            $443 = ($442|0)!=(0);
            do {
             if ($443) {
              $444 = HEAP32[$v>>2]|0;
              $445 = HEAP32[$addrBypass>>2]|0;
              _sqlite3VdbeResolveLabel($444,$445);
              $446 = HEAP32[$pWInfo>>2]|0;
              _sqlite3WhereEnd($446);
             } else {
              $447 = HEAP32[$pPk>>2]|0;
              $448 = ($447|0)!=(0|0);
              $449 = HEAP32[$v>>2]|0;
              if ($448) {
               $450 = HEAP32[$iEphCur>>2]|0;
               $451 = HEAP32[$addrLoop>>2]|0;
               $452 = (($451) + 1)|0;
               (_sqlite3VdbeAddOp2($449,7,$450,$452)|0);
               $453 = HEAP32[$v>>2]|0;
               $454 = HEAP32[$addrLoop>>2]|0;
               _sqlite3VdbeJumpHere($453,$454);
               break;
              } else {
               $455 = HEAP32[$addrLoop>>2]|0;
               (_sqlite3VdbeGoto($449,$455)|0);
               $456 = HEAP32[$v>>2]|0;
               $457 = HEAP32[$addrLoop>>2]|0;
               _sqlite3VdbeJumpHere($456,$457);
               break;
              }
             }
            } while(0);
            $458 = HEAP32[$isView>>2]|0;
            $459 = ($458|0)!=(0);
            if ($459) {
             break;
            }
            $460 = HEAP32[$pTab>>2]|0;
            $461 = ((($460)) + 42|0);
            $462 = HEAP8[$461>>0]|0;
            $463 = $462&255;
            $464 = $463 & 16;
            $465 = ($464|0)!=(0);
            if ($465) {
             break;
            }
            $466 = HEAP32[$pPk>>2]|0;
            $467 = ($466|0)!=(0|0);
            if (!($467)) {
             $468 = HEAP32[$v>>2]|0;
             $469 = HEAP32[$iDataCur>>2]|0;
             (_sqlite3VdbeAddOp1($468,111,$469)|0);
            }
            HEAP32[$i>>2] = 0;
            $470 = HEAP32[$pTab>>2]|0;
            $471 = ((($470)) + 8|0);
            $472 = HEAP32[$471>>2]|0;
            HEAP32[$pIdx>>2] = $472;
            while(1) {
             $473 = HEAP32[$pIdx>>2]|0;
             $474 = ($473|0)!=(0|0);
             if (!($474)) {
              break L38;
             }
             $475 = HEAP32[$v>>2]|0;
             $476 = HEAP32[$iIdxCur>>2]|0;
             $477 = HEAP32[$i>>2]|0;
             $478 = (($476) + ($477))|0;
             (_sqlite3VdbeAddOp1($475,111,$478)|0);
             $479 = HEAP32[$i>>2]|0;
             $480 = (($479) + 1)|0;
             HEAP32[$i>>2] = $480;
             $481 = HEAP32[$pIdx>>2]|0;
             $482 = ((($481)) + 20|0);
             $483 = HEAP32[$482>>2]|0;
             HEAP32[$pIdx>>2] = $483;
            }
           }
          } while(0);
          $484 = HEAP32[$0>>2]|0;
          $485 = ((($484)) + 18|0);
          $486 = HEAP8[$485>>0]|0;
          $487 = $486&255;
          $488 = ($487|0)==(0);
          if ($488) {
           $489 = HEAP32[$0>>2]|0;
           $490 = ((($489)) + 128|0);
           $491 = HEAP32[$490>>2]|0;
           $492 = ($491|0)==(0|0);
           if ($492) {
            $493 = HEAP32[$0>>2]|0;
            _sqlite3AutoincrementEnd($493);
           }
          }
          $494 = HEAP32[$db>>2]|0;
          $495 = ((($494)) + 24|0);
          $496 = HEAP32[$495>>2]|0;
          $497 = $496 & 128;
          $498 = ($497|0)!=(0);
          if ($498) {
           $499 = HEAP32[$0>>2]|0;
           $500 = ((($499)) + 18|0);
           $501 = HEAP8[$500>>0]|0;
           $502 = ($501<<24>>24)!=(0);
           if (!($502)) {
            $503 = HEAP32[$0>>2]|0;
            $504 = ((($503)) + 128|0);
            $505 = HEAP32[$504>>2]|0;
            $506 = ($505|0)!=(0|0);
            if (!($506)) {
             $507 = HEAP32[$v>>2]|0;
             $508 = HEAP32[$memCnt>>2]|0;
             (_sqlite3VdbeAddOp2($507,87,$508,1)|0);
             $509 = HEAP32[$v>>2]|0;
             _sqlite3VdbeSetNumCols($509,1);
             $510 = HEAP32[$v>>2]|0;
             (_sqlite3VdbeSetColName($510,0,0,35361,0)|0);
            }
           }
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 _sqlite3AuthContextPop($sContext);
 $511 = HEAP32[$db>>2]|0;
 $512 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($511,$512);
 $513 = HEAP32[$db>>2]|0;
 $514 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($513,$514);
 $515 = HEAP32[$db>>2]|0;
 $516 = HEAP32[$aToOpen>>2]|0;
 _sqlite3DbFree($515,$516);
 STACKTOP = sp;return;
}
function _sqlite3ExprListCheckLength($pParse,$pEList,$zObject) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $zObject = $zObject|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mx = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $mx = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $zObject;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 96|0);
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$mx>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$mx>>2]|0;
 $13 = ($11|0)>($12|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $15;
 _sqlite3ErrorMsg($14,35338,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3Update($pParse,$pTabList,$pChanges,$pWhere,$onError) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pChanges = $pChanges|0;
 $pWhere = $pWhere|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0;
 var $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0;
 var $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0;
 var $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0;
 var $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0;
 var $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0;
 var $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0;
 var $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0;
 var $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0;
 var $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0;
 var $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0;
 var $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0;
 var $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0;
 var $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0;
 var $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0;
 var $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0;
 var $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0;
 var $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0;
 var $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0;
 var $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0;
 var $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0;
 var $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0;
 var $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0;
 var $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0;
 var $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0;
 var $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0;
 var $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0;
 var $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0;
 var $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0;
 var $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0;
 var $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0;
 var $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0;
 var $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0;
 var $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0;
 var $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0;
 var $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0;
 var $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0;
 var $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0;
 var $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0;
 var $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0;
 var $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0;
 var $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0;
 var $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0;
 var $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0;
 var $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0;
 var $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0;
 var $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0;
 var $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0;
 var $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0;
 var $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0;
 var $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $aRegIdx = 0, $aToOpen = 0, $aXRef = 0, $addr1 = 0, $addrOpen = 0, $addrTop = 0, $aiCurOnePass = 0, $bReplace = 0, $chngKey = 0;
 var $chngPk = 0, $chngRowid = 0, $db = 0, $hasFK = 0, $i = 0, $iBaseCur = 0, $iDataCur = 0, $iDb = 0, $iEph = 0, $iIdxCol = 0, $iIdxCur = 0, $iPk = 0, $isView = 0, $j = 0, $labelBreak = 0, $labelContinue = 0, $nIdx = 0, $nKey = 0, $nPk = 0, $newmask = 0;
 var $okOnePass = 0, $oldmask = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond19 = 0, $or$cond21 = 0, $or$cond23 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pIdx = 0, $pPk = 0, $pRowidExpr = 0, $pTab = 0, $pTrigger = 0, $pWInfo = 0;
 var $rc = 0, $reg = 0, $regKey = 0, $regNew = 0, $regNewRowid = 0, $regOld = 0, $regOldRowid = 0, $regRowCount = 0, $regRowSet = 0, $sContext = 0, $sNC = 0, $tmask = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer = sp;
 $0 = sp + 244|0;
 $1 = sp + 240|0;
 $2 = sp + 236|0;
 $3 = sp + 232|0;
 $4 = sp + 228|0;
 $i = sp + 224|0;
 $j = sp + 220|0;
 $pTab = sp + 216|0;
 $addrTop = sp + 212|0;
 $pWInfo = sp + 208|0;
 $v = sp + 204|0;
 $pIdx = sp + 200|0;
 $pPk = sp + 196|0;
 $nIdx = sp + 192|0;
 $iBaseCur = sp + 188|0;
 $iDataCur = sp + 184|0;
 $iIdxCur = sp + 180|0;
 $db = sp + 176|0;
 $aRegIdx = sp + 172|0;
 $aXRef = sp + 168|0;
 $aToOpen = sp + 164|0;
 $chngPk = sp + 254|0;
 $chngRowid = sp + 253|0;
 $chngKey = sp + 252|0;
 $pRowidExpr = sp + 160|0;
 $sContext = sp + 152|0;
 $sNC = sp + 120|0;
 $iDb = sp + 112|0;
 $okOnePass = sp + 108|0;
 $hasFK = sp + 104|0;
 $labelBreak = sp + 100|0;
 $labelContinue = sp + 96|0;
 $isView = sp + 92|0;
 $pTrigger = sp + 88|0;
 $tmask = sp + 84|0;
 $newmask = sp + 80|0;
 $iEph = sp + 76|0;
 $nKey = sp + 72|0;
 $aiCurOnePass = sp + 64|0;
 $regRowCount = sp + 56|0;
 $regOldRowid = sp + 52|0;
 $regNewRowid = sp + 48|0;
 $regNew = sp + 44|0;
 $regOld = sp + 40|0;
 $regRowSet = sp + 36|0;
 $regKey = sp + 32|0;
 $rc = sp + 28|0;
 $reg = sp + 24|0;
 $iIdxCol = sp + 250|0;
 $iPk = sp + 20|0;
 $nPk = sp + 248|0;
 $addrOpen = sp + 16|0;
 $oldmask = sp + 12|0;
 $addr1 = sp + 8|0;
 $bReplace = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $onError;
 HEAP32[$addrTop>>2] = 0;
 HEAP32[$aRegIdx>>2] = 0;
 HEAP32[$aXRef>>2] = 0;
 HEAP32[$pRowidExpr>>2] = 0;
 HEAP32[$iEph>>2] = 0;
 HEAP32[$nKey>>2] = 0;
 HEAP32[$regRowCount>>2] = 0;
 HEAP32[$regOldRowid>>2] = 0;
 HEAP32[$regNewRowid>>2] = 0;
 HEAP32[$regNew>>2] = 0;
 HEAP32[$regOld>>2] = 0;
 HEAP32[$regRowSet>>2] = 0;
 HEAP32[$regKey>>2] = 0;
 ;HEAP32[$sContext>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 69|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 if ($15) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (_sqlite3SrcListLookup($16,$17)|0);
 HEAP32[$pTab>>2] = $18;
 $19 = HEAP32[$pTab>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pTab>>2]|0;
 $24 = ((($23)) + 64|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (_sqlite3SchemaToIndex($22,$25)|0);
 HEAP32[$iDb>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$pTab>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = (_sqlite3TriggersExist($27,$28,110,$29,$tmask)|0);
 HEAP32[$pTrigger>>2] = $30;
 $31 = HEAP32[$pTab>>2]|0;
 $32 = ((($31)) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0|0);
 $35 = $34&1;
 HEAP32[$isView>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pTab>>2]|0;
 $38 = (_sqlite3ViewGetColumnNames($36,$37)|0);
 $39 = ($38|0)!=(0);
 if ($39) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$pTab>>2]|0;
 $42 = HEAP32[$tmask>>2]|0;
 $43 = (_sqlite3IsReadOnly($40,$41,$42)|0);
 $44 = ($43|0)!=(0);
 if ($44) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 40|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + 1)|0;
 HEAP32[$46>>2] = $48;
 HEAP32[$iDataCur>>2] = $47;
 HEAP32[$iBaseCur>>2] = $47;
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 8|0);
 $51 = ((($50)) + 44|0);
 HEAP32[$51>>2] = $47;
 $52 = HEAP32[$iDataCur>>2]|0;
 $53 = (($52) + 1)|0;
 HEAP32[$iIdxCur>>2] = $53;
 $54 = HEAP32[$pTab>>2]|0;
 $55 = ((($54)) + 42|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = $56&255;
 $58 = $57 & 32;
 $59 = ($58|0)==(0);
 if ($59) {
  $62 = 0;
 } else {
  $60 = HEAP32[$pTab>>2]|0;
  $61 = (_sqlite3PrimaryKeyIndex($60)|0);
  $62 = $61;
 }
 HEAP32[$pPk>>2] = $62;
 HEAP32[$nIdx>>2] = 0;
 $63 = HEAP32[$pTab>>2]|0;
 $64 = ((($63)) + 8|0);
 $65 = HEAP32[$64>>2]|0;
 HEAP32[$pIdx>>2] = $65;
 while(1) {
  $66 = HEAP32[$pIdx>>2]|0;
  $67 = ($66|0)!=(0|0);
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$pIdx>>2]|0;
  $69 = ((($68)) + 55|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = $70 & 3;
  $72 = $71&255;
  $73 = ($72|0)==(2);
  $74 = HEAP32[$pPk>>2]|0;
  $75 = ($74|0)!=(0|0);
  $or$cond = $73 & $75;
  if ($or$cond) {
   $76 = HEAP32[$0>>2]|0;
   $77 = ((($76)) + 40|0);
   $78 = HEAP32[$77>>2]|0;
   HEAP32[$iDataCur>>2] = $78;
   $79 = HEAP32[$iDataCur>>2]|0;
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 8|0);
   $82 = ((($81)) + 44|0);
   HEAP32[$82>>2] = $79;
  }
  $83 = HEAP32[$0>>2]|0;
  $84 = ((($83)) + 40|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (($85) + 1)|0;
  HEAP32[$84>>2] = $86;
  $87 = HEAP32[$pIdx>>2]|0;
  $88 = ((($87)) + 20|0);
  $89 = HEAP32[$88>>2]|0;
  HEAP32[$pIdx>>2] = $89;
  $90 = HEAP32[$nIdx>>2]|0;
  $91 = (($90) + 1)|0;
  HEAP32[$nIdx>>2] = $91;
 }
 $92 = HEAP32[$db>>2]|0;
 $93 = HEAP32[$pTab>>2]|0;
 $94 = ((($93)) + 34|0);
 $95 = HEAP16[$94>>1]|0;
 $96 = $95 << 16 >> 16;
 $97 = HEAP32[$nIdx>>2]|0;
 $98 = (($96) + ($97))|0;
 $99 = $98<<2;
 $100 = HEAP32[$nIdx>>2]|0;
 $101 = (($99) + ($100))|0;
 $102 = (($101) + 2)|0;
 $103 = (_sqlite3DbMallocRawNN($92,$102,0)|0);
 HEAP32[$aXRef>>2] = $103;
 $104 = HEAP32[$aXRef>>2]|0;
 $105 = ($104|0)==(0|0);
 if ($105) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 $106 = HEAP32[$aXRef>>2]|0;
 $107 = HEAP32[$pTab>>2]|0;
 $108 = ((($107)) + 34|0);
 $109 = HEAP16[$108>>1]|0;
 $110 = $109 << 16 >> 16;
 $111 = (($106) + ($110<<2)|0);
 HEAP32[$aRegIdx>>2] = $111;
 $112 = HEAP32[$aRegIdx>>2]|0;
 $113 = HEAP32[$nIdx>>2]|0;
 $114 = (($112) + ($113<<2)|0);
 HEAP32[$aToOpen>>2] = $114;
 $115 = HEAP32[$aToOpen>>2]|0;
 $116 = HEAP32[$nIdx>>2]|0;
 $117 = (($116) + 1)|0;
 _memset(($115|0),1,($117|0))|0;
 $118 = HEAP32[$nIdx>>2]|0;
 $119 = (($118) + 1)|0;
 $120 = HEAP32[$aToOpen>>2]|0;
 $121 = (($120) + ($119)|0);
 HEAP8[$121>>0] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $122 = HEAP32[$i>>2]|0;
  $123 = HEAP32[$pTab>>2]|0;
  $124 = ((($123)) + 34|0);
  $125 = HEAP16[$124>>1]|0;
  $126 = $125 << 16 >> 16;
  $127 = ($122|0)<($126|0);
  if (!($127)) {
   break;
  }
  $128 = HEAP32[$i>>2]|0;
  $129 = HEAP32[$aXRef>>2]|0;
  $130 = (($129) + ($128<<2)|0);
  HEAP32[$130>>2] = -1;
  $131 = HEAP32[$i>>2]|0;
  $132 = (($131) + 1)|0;
  HEAP32[$i>>2] = $132;
 }
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $133 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $133;
 $134 = HEAP32[$1>>2]|0;
 $135 = ((($sNC)) + 4|0);
 HEAP32[$135>>2] = $134;
 HEAP8[$chngPk>>0] = 0;
 HEAP8[$chngRowid>>0] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $136 = HEAP32[$i>>2]|0;
  $137 = HEAP32[$2>>2]|0;
  $138 = HEAP32[$137>>2]|0;
  $139 = ($136|0)<($138|0);
  if (!($139)) {
   label = 41;
   break;
  }
  $140 = HEAP32[$i>>2]|0;
  $141 = HEAP32[$2>>2]|0;
  $142 = ((($141)) + 4|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = (($143) + (($140*20)|0)|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = (_sqlite3ResolveExprNames($sNC,$145)|0);
  $147 = ($146|0)!=(0);
  if ($147) {
   label = 185;
   break;
  }
  HEAP32[$j>>2] = 0;
  while(1) {
   $148 = HEAP32[$j>>2]|0;
   $149 = HEAP32[$pTab>>2]|0;
   $150 = ((($149)) + 34|0);
   $151 = HEAP16[$150>>1]|0;
   $152 = $151 << 16 >> 16;
   $153 = ($148|0)<($152|0);
   if (!($153)) {
    break;
   }
   $154 = HEAP32[$j>>2]|0;
   $155 = HEAP32[$pTab>>2]|0;
   $156 = ((($155)) + 4|0);
   $157 = HEAP32[$156>>2]|0;
   $158 = (($157) + ($154<<4)|0);
   $159 = HEAP32[$158>>2]|0;
   $160 = HEAP32[$i>>2]|0;
   $161 = HEAP32[$2>>2]|0;
   $162 = ((($161)) + 4|0);
   $163 = HEAP32[$162>>2]|0;
   $164 = (($163) + (($160*20)|0)|0);
   $165 = ((($164)) + 4|0);
   $166 = HEAP32[$165>>2]|0;
   $167 = (_sqlite3StrICmp($159,$166)|0);
   $168 = ($167|0)==(0);
   $169 = HEAP32[$j>>2]|0;
   if ($168) {
    label = 23;
    break;
   }
   $197 = (($169) + 1)|0;
   HEAP32[$j>>2] = $197;
  }
  if ((label|0) == 23) {
   label = 0;
   $170 = HEAP32[$pTab>>2]|0;
   $171 = ((($170)) + 32|0);
   $172 = HEAP16[$171>>1]|0;
   $173 = $172 << 16 >> 16;
   $174 = ($169|0)==($173|0);
   if ($174) {
    HEAP8[$chngRowid>>0] = 1;
    $175 = HEAP32[$i>>2]|0;
    $176 = HEAP32[$2>>2]|0;
    $177 = ((($176)) + 4|0);
    $178 = HEAP32[$177>>2]|0;
    $179 = (($178) + (($175*20)|0)|0);
    $180 = HEAP32[$179>>2]|0;
    HEAP32[$pRowidExpr>>2] = $180;
   } else {
    $181 = HEAP32[$pPk>>2]|0;
    $182 = ($181|0)!=(0|0);
    if ($182) {
     $183 = HEAP32[$j>>2]|0;
     $184 = HEAP32[$pTab>>2]|0;
     $185 = ((($184)) + 4|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = (($186) + ($183<<4)|0);
     $188 = ((($187)) + 15|0);
     $189 = HEAP8[$188>>0]|0;
     $190 = $189&255;
     $191 = $190 & 1;
     $192 = ($191|0)!=(0);
     if ($192) {
      HEAP8[$chngPk>>0] = 1;
     }
    }
   }
   $193 = HEAP32[$i>>2]|0;
   $194 = HEAP32[$j>>2]|0;
   $195 = HEAP32[$aXRef>>2]|0;
   $196 = (($195) + ($194<<2)|0);
   HEAP32[$196>>2] = $193;
  }
  $198 = HEAP32[$j>>2]|0;
  $199 = HEAP32[$pTab>>2]|0;
  $200 = ((($199)) + 34|0);
  $201 = HEAP16[$200>>1]|0;
  $202 = $201 << 16 >> 16;
  $203 = ($198|0)>=($202|0);
  if ($203) {
   $204 = HEAP32[$pPk>>2]|0;
   $205 = ($204|0)==(0|0);
   if (!($205)) {
    label = 34;
    break;
   }
   $206 = HEAP32[$i>>2]|0;
   $207 = HEAP32[$2>>2]|0;
   $208 = ((($207)) + 4|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = (($209) + (($206*20)|0)|0);
   $211 = ((($210)) + 4|0);
   $212 = HEAP32[$211>>2]|0;
   $213 = (_sqlite3IsRowid($212)|0);
   $214 = ($213|0)!=(0);
   if (!($214)) {
    label = 34;
    break;
   }
   HEAP32[$j>>2] = -1;
   HEAP8[$chngRowid>>0] = 1;
   $215 = HEAP32[$i>>2]|0;
   $216 = HEAP32[$2>>2]|0;
   $217 = ((($216)) + 4|0);
   $218 = HEAP32[$217>>2]|0;
   $219 = (($218) + (($215*20)|0)|0);
   $220 = HEAP32[$219>>2]|0;
   HEAP32[$pRowidExpr>>2] = $220;
  }
  $231 = HEAP32[$0>>2]|0;
  $232 = HEAP32[$pTab>>2]|0;
  $233 = HEAP32[$232>>2]|0;
  $234 = HEAP32[$j>>2]|0;
  $235 = ($234|0)<(0);
  if ($235) {
   $248 = 19895;
  } else {
   $236 = HEAP32[$j>>2]|0;
   $237 = HEAP32[$pTab>>2]|0;
   $238 = ((($237)) + 4|0);
   $239 = HEAP32[$238>>2]|0;
   $240 = (($239) + ($236<<4)|0);
   $241 = HEAP32[$240>>2]|0;
   $248 = $241;
  }
  $242 = HEAP32[$iDb>>2]|0;
  $243 = HEAP32[$db>>2]|0;
  $244 = ((($243)) + 16|0);
  $245 = HEAP32[$244>>2]|0;
  $246 = (($245) + ($242<<4)|0);
  $247 = HEAP32[$246>>2]|0;
  $249 = (_sqlite3AuthCheck($231,23,$233,$248,$247)|0);
  HEAP32[$rc>>2] = $249;
  $250 = HEAP32[$rc>>2]|0;
  $251 = ($250|0)==(1);
  if ($251) {
   label = 185;
   break;
  }
  $252 = HEAP32[$rc>>2]|0;
  $253 = ($252|0)==(2);
  if ($253) {
   $254 = HEAP32[$j>>2]|0;
   $255 = HEAP32[$aXRef>>2]|0;
   $256 = (($255) + ($254<<2)|0);
   HEAP32[$256>>2] = -1;
  }
  $257 = HEAP32[$i>>2]|0;
  $258 = (($257) + 1)|0;
  HEAP32[$i>>2] = $258;
 }
 if ((label|0) == 34) {
  $221 = HEAP32[$0>>2]|0;
  $222 = HEAP32[$i>>2]|0;
  $223 = HEAP32[$2>>2]|0;
  $224 = ((($223)) + 4|0);
  $225 = HEAP32[$224>>2]|0;
  $226 = (($225) + (($222*20)|0)|0);
  $227 = ((($226)) + 4|0);
  $228 = HEAP32[$227>>2]|0;
  HEAP32[$vararg_buffer>>2] = $228;
  _sqlite3ErrorMsg($221,24144,$vararg_buffer);
  $229 = HEAP32[$0>>2]|0;
  $230 = ((($229)) + 17|0);
  HEAP8[$230>>0] = 1;
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 41) {
  $259 = HEAP8[$chngRowid>>0]|0;
  $260 = $259&255;
  $261 = HEAP8[$chngPk>>0]|0;
  $262 = $261&255;
  $263 = (($260) + ($262))|0;
  $264 = $263&255;
  HEAP8[$chngKey>>0] = $264;
  $265 = HEAP32[$pTab>>2]|0;
  $266 = ((($265)) + 42|0);
  $267 = HEAP8[$266>>0]|0;
  $268 = $267&255;
  $269 = $268 & 16;
  $270 = ($269|0)!=(0);
  $271 = $270 ? -1 : 0;
  $272 = $270 ? -1 : 0;
  $273 = HEAP32[$1>>2]|0;
  $274 = ((($273)) + 8|0);
  $275 = ((($274)) + 56|0);
  $276 = $275;
  $277 = $276;
  HEAP32[$277>>2] = $271;
  $278 = (($276) + 4)|0;
  $279 = $278;
  HEAP32[$279>>2] = $272;
  $280 = HEAP32[$0>>2]|0;
  $281 = HEAP32[$pTab>>2]|0;
  $282 = HEAP32[$aXRef>>2]|0;
  $283 = HEAP8[$chngKey>>0]|0;
  $284 = $283&255;
  $285 = (_sqlite3FkRequired($280,$281,$282,$284)|0);
  HEAP32[$hasFK>>2] = $285;
  HEAP32[$j>>2] = 0;
  $286 = HEAP32[$pTab>>2]|0;
  $287 = ((($286)) + 8|0);
  $288 = HEAP32[$287>>2]|0;
  HEAP32[$pIdx>>2] = $288;
  while(1) {
   $289 = HEAP32[$pIdx>>2]|0;
   $290 = ($289|0)!=(0|0);
   if (!($290)) {
    break;
   }
   $291 = HEAP8[$chngKey>>0]|0;
   $292 = $291&255;
   $293 = ($292|0)!=(0);
   $294 = HEAP32[$hasFK>>2]|0;
   $295 = ($294|0)!=(0);
   $or$cond3 = $293 | $295;
   L67: do {
    if ($or$cond3) {
     label = 46;
    } else {
     $296 = HEAP32[$pIdx>>2]|0;
     $297 = ((($296)) + 36|0);
     $298 = HEAP32[$297>>2]|0;
     $299 = ($298|0)!=(0|0);
     if ($299) {
      label = 46;
     } else {
      $300 = HEAP32[$pIdx>>2]|0;
      $301 = HEAP32[$pPk>>2]|0;
      $302 = ($300|0)==($301|0);
      if ($302) {
       label = 46;
      } else {
       HEAP32[$reg>>2] = 0;
       HEAP32[$i>>2] = 0;
       while(1) {
        $307 = HEAP32[$i>>2]|0;
        $308 = HEAP32[$pIdx>>2]|0;
        $309 = ((($308)) + 50|0);
        $310 = HEAP16[$309>>1]|0;
        $311 = $310&65535;
        $312 = ($307|0)<($311|0);
        if (!($312)) {
         break L67;
        }
        $313 = HEAP32[$i>>2]|0;
        $314 = HEAP32[$pIdx>>2]|0;
        $315 = ((($314)) + 4|0);
        $316 = HEAP32[$315>>2]|0;
        $317 = (($316) + ($313<<1)|0);
        $318 = HEAP16[$317>>1]|0;
        HEAP16[$iIdxCol>>1] = $318;
        $319 = HEAP16[$iIdxCol>>1]|0;
        $320 = $319 << 16 >> 16;
        $321 = ($320|0)<(0);
        if ($321) {
         break;
        }
        $322 = HEAP16[$iIdxCol>>1]|0;
        $323 = $322 << 16 >> 16;
        $324 = HEAP32[$aXRef>>2]|0;
        $325 = (($324) + ($323<<2)|0);
        $326 = HEAP32[$325>>2]|0;
        $327 = ($326|0)>=(0);
        if ($327) {
         break;
        }
        $332 = HEAP32[$i>>2]|0;
        $333 = (($332) + 1)|0;
        HEAP32[$i>>2] = $333;
       }
       $328 = HEAP32[$0>>2]|0;
       $329 = ((($328)) + 44|0);
       $330 = HEAP32[$329>>2]|0;
       $331 = (($330) + 1)|0;
       HEAP32[$329>>2] = $331;
       HEAP32[$reg>>2] = $331;
      }
     }
    }
   } while(0);
   if ((label|0) == 46) {
    label = 0;
    $303 = HEAP32[$0>>2]|0;
    $304 = ((($303)) + 44|0);
    $305 = HEAP32[$304>>2]|0;
    $306 = (($305) + 1)|0;
    HEAP32[$304>>2] = $306;
    HEAP32[$reg>>2] = $306;
   }
   $334 = HEAP32[$reg>>2]|0;
   $335 = ($334|0)==(0);
   if ($335) {
    $336 = HEAP32[$j>>2]|0;
    $337 = (($336) + 1)|0;
    $338 = HEAP32[$aToOpen>>2]|0;
    $339 = (($338) + ($337)|0);
    HEAP8[$339>>0] = 0;
   }
   $340 = HEAP32[$reg>>2]|0;
   $341 = HEAP32[$j>>2]|0;
   $342 = HEAP32[$aRegIdx>>2]|0;
   $343 = (($342) + ($341<<2)|0);
   HEAP32[$343>>2] = $340;
   $344 = HEAP32[$pIdx>>2]|0;
   $345 = ((($344)) + 20|0);
   $346 = HEAP32[$345>>2]|0;
   HEAP32[$pIdx>>2] = $346;
   $347 = HEAP32[$j>>2]|0;
   $348 = (($347) + 1)|0;
   HEAP32[$j>>2] = $348;
  }
  $349 = HEAP32[$0>>2]|0;
  $350 = (_sqlite3GetVdbe($349)|0);
  HEAP32[$v>>2] = $350;
  $351 = HEAP32[$v>>2]|0;
  $352 = ($351|0)==(0|0);
  if ($352) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $353 = HEAP32[$0>>2]|0;
  $354 = ((($353)) + 18|0);
  $355 = HEAP8[$354>>0]|0;
  $356 = $355&255;
  $357 = ($356|0)==(0);
  if ($357) {
   $358 = HEAP32[$v>>2]|0;
   _sqlite3VdbeCountChanges($358);
  }
  $359 = HEAP32[$0>>2]|0;
  $360 = HEAP32[$iDb>>2]|0;
  _sqlite3BeginWriteOperation($359,1,$360);
  $361 = HEAP32[$pTab>>2]|0;
  $362 = ((($361)) + 42|0);
  $363 = HEAP8[$362>>0]|0;
  $364 = $363&255;
  $365 = $364 & 16;
  $366 = ($365|0)!=(0);
  if (!($366)) {
   $367 = HEAP32[$0>>2]|0;
   $368 = ((($367)) + 44|0);
   $369 = HEAP32[$368>>2]|0;
   $370 = (($369) + 1)|0;
   HEAP32[$368>>2] = $370;
   HEAP32[$regRowSet>>2] = $370;
   $371 = HEAP32[$0>>2]|0;
   $372 = ((($371)) + 44|0);
   $373 = HEAP32[$372>>2]|0;
   $374 = (($373) + 1)|0;
   HEAP32[$372>>2] = $374;
   HEAP32[$regNewRowid>>2] = $374;
   HEAP32[$regOldRowid>>2] = $374;
   $375 = HEAP8[$chngPk>>0]|0;
   $376 = $375&255;
   $377 = ($376|0)!=(0);
   $378 = HEAP32[$pTrigger>>2]|0;
   $379 = ($378|0)!=(0|0);
   $or$cond5 = $377 | $379;
   $380 = HEAP32[$hasFK>>2]|0;
   $381 = ($380|0)!=(0);
   $or$cond7 = $or$cond5 | $381;
   if ($or$cond7) {
    $382 = HEAP32[$0>>2]|0;
    $383 = ((($382)) + 44|0);
    $384 = HEAP32[$383>>2]|0;
    $385 = (($384) + 1)|0;
    HEAP32[$regOld>>2] = $385;
    $386 = HEAP32[$pTab>>2]|0;
    $387 = ((($386)) + 34|0);
    $388 = HEAP16[$387>>1]|0;
    $389 = $388 << 16 >> 16;
    $390 = HEAP32[$0>>2]|0;
    $391 = ((($390)) + 44|0);
    $392 = HEAP32[$391>>2]|0;
    $393 = (($392) + ($389))|0;
    HEAP32[$391>>2] = $393;
   }
   $394 = HEAP8[$chngKey>>0]|0;
   $395 = $394&255;
   $396 = ($395|0)!=(0);
   $397 = HEAP32[$pTrigger>>2]|0;
   $398 = ($397|0)!=(0|0);
   $or$cond9 = $396 | $398;
   $399 = HEAP32[$hasFK>>2]|0;
   $400 = ($399|0)!=(0);
   $or$cond11 = $or$cond9 | $400;
   if ($or$cond11) {
    $401 = HEAP32[$0>>2]|0;
    $402 = ((($401)) + 44|0);
    $403 = HEAP32[$402>>2]|0;
    $404 = (($403) + 1)|0;
    HEAP32[$402>>2] = $404;
    HEAP32[$regNewRowid>>2] = $404;
   }
   $405 = HEAP32[$0>>2]|0;
   $406 = ((($405)) + 44|0);
   $407 = HEAP32[$406>>2]|0;
   $408 = (($407) + 1)|0;
   HEAP32[$regNew>>2] = $408;
   $409 = HEAP32[$pTab>>2]|0;
   $410 = ((($409)) + 34|0);
   $411 = HEAP16[$410>>1]|0;
   $412 = $411 << 16 >> 16;
   $413 = HEAP32[$0>>2]|0;
   $414 = ((($413)) + 44|0);
   $415 = HEAP32[$414>>2]|0;
   $416 = (($415) + ($412))|0;
   HEAP32[$414>>2] = $416;
  }
  $417 = HEAP32[$isView>>2]|0;
  $418 = ($417|0)!=(0);
  if ($418) {
   $419 = HEAP32[$0>>2]|0;
   $420 = HEAP32[$pTab>>2]|0;
   $421 = HEAP32[$420>>2]|0;
   _sqlite3AuthContextPush($419,$sContext,$421);
  }
  $422 = HEAP32[$isView>>2]|0;
  $423 = ($422|0)!=(0);
  if ($423) {
   $424 = HEAP32[$0>>2]|0;
   $425 = HEAP32[$pTab>>2]|0;
   $426 = HEAP32[$3>>2]|0;
   $427 = HEAP32[$iDataCur>>2]|0;
   _sqlite3MaterializeView($424,$425,$426,$427);
  }
  $428 = HEAP32[$3>>2]|0;
  $429 = (_sqlite3ResolveExprNames($sNC,$428)|0);
  $430 = ($429|0)!=(0);
  if ($430) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $431 = HEAP32[$pTab>>2]|0;
  $432 = ((($431)) + 42|0);
  $433 = HEAP8[$432>>0]|0;
  $434 = $433&255;
  $435 = $434 & 16;
  $436 = ($435|0)!=(0);
  if ($436) {
   $437 = HEAP32[$0>>2]|0;
   $438 = HEAP32[$1>>2]|0;
   $439 = HEAP32[$pTab>>2]|0;
   $440 = HEAP32[$2>>2]|0;
   $441 = HEAP32[$pRowidExpr>>2]|0;
   $442 = HEAP32[$aXRef>>2]|0;
   $443 = HEAP32[$3>>2]|0;
   $444 = HEAP32[$4>>2]|0;
   _updateVirtualTable($437,$438,$439,$440,$441,$442,$443,$444);
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $445 = HEAP32[$pTab>>2]|0;
  $446 = ((($445)) + 42|0);
  $447 = HEAP8[$446>>0]|0;
  $448 = $447&255;
  $449 = $448 & 32;
  $450 = ($449|0)==(0);
  if ($450) {
   $451 = HEAP32[$v>>2]|0;
   $452 = HEAP32[$regRowSet>>2]|0;
   $453 = HEAP32[$regOldRowid>>2]|0;
   (_sqlite3VdbeAddOp3($451,79,0,$452,$453)|0);
   $454 = HEAP32[$0>>2]|0;
   $455 = HEAP32[$1>>2]|0;
   $456 = HEAP32[$3>>2]|0;
   $457 = HEAP32[$iIdxCur>>2]|0;
   $458 = (_sqlite3WhereBegin($454,$455,$456,0,0,1028,$457)|0);
   HEAP32[$pWInfo>>2] = $458;
   $459 = HEAP32[$pWInfo>>2]|0;
   $460 = ($459|0)==(0|0);
   if ($460) {
    _sqlite3AuthContextPop($sContext);
    $1003 = HEAP32[$db>>2]|0;
    $1004 = HEAP32[$aXRef>>2]|0;
    _sqlite3DbFree($1003,$1004);
    $1005 = HEAP32[$db>>2]|0;
    $1006 = HEAP32[$1>>2]|0;
    _sqlite3SrcListDelete($1005,$1006);
    $1007 = HEAP32[$db>>2]|0;
    $1008 = HEAP32[$2>>2]|0;
    _sqlite3ExprListDelete($1007,$1008);
    $1009 = HEAP32[$db>>2]|0;
    $1010 = HEAP32[$3>>2]|0;
    _sqlite3ExprDelete($1009,$1010);
    STACKTOP = sp;return;
   }
   $461 = HEAP32[$pWInfo>>2]|0;
   $462 = (_sqlite3WhereOkOnePass($461,$aiCurOnePass)|0);
   HEAP32[$okOnePass>>2] = $462;
   $463 = HEAP32[$v>>2]|0;
   $464 = HEAP32[$iDataCur>>2]|0;
   $465 = HEAP32[$regOldRowid>>2]|0;
   (_sqlite3VdbeAddOp2($463,123,$464,$465)|0);
   $466 = HEAP32[$okOnePass>>2]|0;
   $467 = ($466|0)!=(0);
   if (!($467)) {
    $468 = HEAP32[$v>>2]|0;
    $469 = HEAP32[$regRowSet>>2]|0;
    $470 = HEAP32[$regOldRowid>>2]|0;
    (_sqlite3VdbeAddOp2($468,142,$469,$470)|0);
   }
   $471 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($471);
  } else {
   $472 = HEAP32[$pPk>>2]|0;
   $473 = ((($472)) + 50|0);
   $474 = HEAP16[$473>>1]|0;
   HEAP16[$nPk>>1] = $474;
   $475 = HEAP32[$0>>2]|0;
   $476 = ((($475)) + 44|0);
   $477 = HEAP32[$476>>2]|0;
   $478 = (($477) + 1)|0;
   HEAP32[$iPk>>2] = $478;
   $479 = HEAP16[$nPk>>1]|0;
   $480 = $479 << 16 >> 16;
   $481 = HEAP32[$0>>2]|0;
   $482 = ((($481)) + 44|0);
   $483 = HEAP32[$482>>2]|0;
   $484 = (($483) + ($480))|0;
   HEAP32[$482>>2] = $484;
   $485 = HEAP32[$0>>2]|0;
   $486 = ((($485)) + 44|0);
   $487 = HEAP32[$486>>2]|0;
   $488 = (($487) + 1)|0;
   HEAP32[$486>>2] = $488;
   HEAP32[$regKey>>2] = $488;
   $489 = HEAP32[$0>>2]|0;
   $490 = ((($489)) + 40|0);
   $491 = HEAP32[$490>>2]|0;
   $492 = (($491) + 1)|0;
   HEAP32[$490>>2] = $492;
   HEAP32[$iEph>>2] = $491;
   $493 = HEAP32[$v>>2]|0;
   $494 = HEAP32[$iPk>>2]|0;
   (_sqlite3VdbeAddOp2($493,79,0,$494)|0);
   $495 = HEAP32[$v>>2]|0;
   $496 = HEAP32[$iEph>>2]|0;
   $497 = HEAP16[$nPk>>1]|0;
   $498 = $497 << 16 >> 16;
   $499 = (_sqlite3VdbeAddOp2($495,107,$496,$498)|0);
   HEAP32[$addrOpen>>2] = $499;
   $500 = HEAP32[$0>>2]|0;
   $501 = HEAP32[$pPk>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($500,$501);
   $502 = HEAP32[$0>>2]|0;
   $503 = HEAP32[$1>>2]|0;
   $504 = HEAP32[$3>>2]|0;
   $505 = HEAP32[$iIdxCur>>2]|0;
   $506 = (_sqlite3WhereBegin($502,$503,$504,0,0,4,$505)|0);
   HEAP32[$pWInfo>>2] = $506;
   $507 = HEAP32[$pWInfo>>2]|0;
   $508 = ($507|0)==(0|0);
   if ($508) {
    _sqlite3AuthContextPop($sContext);
    $1003 = HEAP32[$db>>2]|0;
    $1004 = HEAP32[$aXRef>>2]|0;
    _sqlite3DbFree($1003,$1004);
    $1005 = HEAP32[$db>>2]|0;
    $1006 = HEAP32[$1>>2]|0;
    _sqlite3SrcListDelete($1005,$1006);
    $1007 = HEAP32[$db>>2]|0;
    $1008 = HEAP32[$2>>2]|0;
    _sqlite3ExprListDelete($1007,$1008);
    $1009 = HEAP32[$db>>2]|0;
    $1010 = HEAP32[$3>>2]|0;
    _sqlite3ExprDelete($1009,$1010);
    STACKTOP = sp;return;
   }
   $509 = HEAP32[$pWInfo>>2]|0;
   $510 = (_sqlite3WhereOkOnePass($509,$aiCurOnePass)|0);
   HEAP32[$okOnePass>>2] = $510;
   HEAP32[$i>>2] = 0;
   while(1) {
    $511 = HEAP32[$i>>2]|0;
    $512 = HEAP16[$nPk>>1]|0;
    $513 = $512 << 16 >> 16;
    $514 = ($511|0)<($513|0);
    if (!($514)) {
     break;
    }
    $515 = HEAP32[$v>>2]|0;
    $516 = HEAP32[$pTab>>2]|0;
    $517 = HEAP32[$iDataCur>>2]|0;
    $518 = HEAP32[$i>>2]|0;
    $519 = HEAP32[$pPk>>2]|0;
    $520 = ((($519)) + 4|0);
    $521 = HEAP32[$520>>2]|0;
    $522 = (($521) + ($518<<1)|0);
    $523 = HEAP16[$522>>1]|0;
    $524 = $523 << 16 >> 16;
    $525 = HEAP32[$iPk>>2]|0;
    $526 = HEAP32[$i>>2]|0;
    $527 = (($525) + ($526))|0;
    _sqlite3ExprCodeGetColumnOfTable($515,$516,$517,$524,$527);
    $528 = HEAP32[$i>>2]|0;
    $529 = (($528) + 1)|0;
    HEAP32[$i>>2] = $529;
   }
   $530 = HEAP32[$okOnePass>>2]|0;
   $531 = ($530|0)!=(0);
   $532 = HEAP32[$v>>2]|0;
   if ($531) {
    $533 = HEAP32[$addrOpen>>2]|0;
    (_sqlite3VdbeChangeToNoop($532,$533)|0);
    $534 = HEAP16[$nPk>>1]|0;
    $535 = $534 << 16 >> 16;
    HEAP32[$nKey>>2] = $535;
    $536 = HEAP32[$iPk>>2]|0;
    HEAP32[$regKey>>2] = $536;
   } else {
    $537 = HEAP32[$iPk>>2]|0;
    $538 = HEAP16[$nPk>>1]|0;
    $539 = $538 << 16 >> 16;
    $540 = HEAP32[$regKey>>2]|0;
    $541 = HEAP32[$db>>2]|0;
    $542 = HEAP32[$pPk>>2]|0;
    $543 = (_sqlite3IndexAffinityStr($541,$542)|0);
    $544 = HEAP16[$nPk>>1]|0;
    $545 = $544 << 16 >> 16;
    (_sqlite3VdbeAddOp4($532,99,$537,$539,$540,$543,$545)|0);
    $546 = HEAP32[$v>>2]|0;
    $547 = HEAP32[$iEph>>2]|0;
    $548 = HEAP32[$regKey>>2]|0;
    (_sqlite3VdbeAddOp2($546,126,$547,$548)|0);
   }
   $549 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($549);
  }
  $550 = HEAP32[$db>>2]|0;
  $551 = ((($550)) + 24|0);
  $552 = HEAP32[$551>>2]|0;
  $553 = $552 & 128;
  $554 = ($553|0)!=(0);
  if ($554) {
   $555 = HEAP32[$0>>2]|0;
   $556 = ((($555)) + 128|0);
   $557 = HEAP32[$556>>2]|0;
   $558 = ($557|0)!=(0|0);
   if (!($558)) {
    $559 = HEAP32[$0>>2]|0;
    $560 = ((($559)) + 44|0);
    $561 = HEAP32[$560>>2]|0;
    $562 = (($561) + 1)|0;
    HEAP32[$560>>2] = $562;
    HEAP32[$regRowCount>>2] = $562;
    $563 = HEAP32[$v>>2]|0;
    $564 = HEAP32[$regRowCount>>2]|0;
    (_sqlite3VdbeAddOp2($563,76,0,$564)|0);
   }
  }
  $565 = HEAP32[$v>>2]|0;
  $566 = (_sqlite3VdbeMakeLabel($565)|0);
  HEAP32[$labelBreak>>2] = $566;
  $567 = HEAP32[$isView>>2]|0;
  $568 = ($567|0)!=(0);
  if (!($568)) {
   $569 = HEAP32[$4>>2]|0;
   $570 = ($569|0)==(5);
   L139: do {
    if ($570) {
     $571 = HEAP32[$aToOpen>>2]|0;
     $572 = HEAP32[$nIdx>>2]|0;
     $573 = (($572) + 1)|0;
     _memset(($571|0),1,($573|0))|0;
    } else {
     $574 = HEAP32[$pTab>>2]|0;
     $575 = ((($574)) + 8|0);
     $576 = HEAP32[$575>>2]|0;
     HEAP32[$pIdx>>2] = $576;
     while(1) {
      $577 = HEAP32[$pIdx>>2]|0;
      $578 = ($577|0)!=(0|0);
      if (!($578)) {
       break L139;
      }
      $579 = HEAP32[$pIdx>>2]|0;
      $580 = ((($579)) + 54|0);
      $581 = HEAP8[$580>>0]|0;
      $582 = $581&255;
      $583 = ($582|0)==(5);
      if ($583) {
       break;
      }
      $587 = HEAP32[$pIdx>>2]|0;
      $588 = ((($587)) + 20|0);
      $589 = HEAP32[$588>>2]|0;
      HEAP32[$pIdx>>2] = $589;
     }
     $584 = HEAP32[$aToOpen>>2]|0;
     $585 = HEAP32[$nIdx>>2]|0;
     $586 = (($585) + 1)|0;
     _memset(($584|0),1,($586|0))|0;
    }
   } while(0);
   $590 = HEAP32[$okOnePass>>2]|0;
   $591 = ($590|0)!=(0);
   do {
    if ($591) {
     $592 = HEAP32[$aiCurOnePass>>2]|0;
     $593 = ($592|0)>=(0);
     if ($593) {
      $594 = HEAP32[$aiCurOnePass>>2]|0;
      $595 = HEAP32[$iBaseCur>>2]|0;
      $596 = (($594) - ($595))|0;
      $597 = HEAP32[$aToOpen>>2]|0;
      $598 = (($597) + ($596)|0);
      HEAP8[$598>>0] = 0;
     }
     $599 = ((($aiCurOnePass)) + 4|0);
     $600 = HEAP32[$599>>2]|0;
     $601 = ($600|0)>=(0);
     if (!($601)) {
      break;
     }
     $602 = ((($aiCurOnePass)) + 4|0);
     $603 = HEAP32[$602>>2]|0;
     $604 = HEAP32[$iBaseCur>>2]|0;
     $605 = (($603) - ($604))|0;
     $606 = HEAP32[$aToOpen>>2]|0;
     $607 = (($606) + ($605)|0);
     HEAP8[$607>>0] = 0;
    }
   } while(0);
   $608 = HEAP32[$0>>2]|0;
   $609 = HEAP32[$pTab>>2]|0;
   $610 = HEAP32[$iBaseCur>>2]|0;
   $611 = HEAP32[$aToOpen>>2]|0;
   (_sqlite3OpenTableAndIndices($608,$609,105,0,$610,$611,0,0)|0);
  }
  $612 = HEAP32[$okOnePass>>2]|0;
  $613 = ($612|0)!=(0);
  do {
   if ($613) {
    $614 = HEAP32[$iDataCur>>2]|0;
    $615 = HEAP32[$iBaseCur>>2]|0;
    $616 = (($614) - ($615))|0;
    $617 = HEAP32[$aToOpen>>2]|0;
    $618 = (($617) + ($616)|0);
    $619 = HEAP8[$618>>0]|0;
    $620 = $619&255;
    $621 = ($620|0)==(0);
    $622 = HEAP32[$isView>>2]|0;
    $623 = ($622|0)!=(0);
    $or$cond13 = $621 | $623;
    if (!($or$cond13)) {
     $624 = HEAP32[$v>>2]|0;
     $625 = HEAP32[$iDataCur>>2]|0;
     $626 = HEAP32[$labelBreak>>2]|0;
     $627 = HEAP32[$regKey>>2]|0;
     $628 = HEAP32[$nKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($624,30,$625,$626,$627,$628)|0);
    }
    $629 = HEAP32[$labelBreak>>2]|0;
    HEAP32[$labelContinue>>2] = $629;
    $630 = HEAP32[$v>>2]|0;
    $631 = HEAP32[$pPk>>2]|0;
    $632 = ($631|0)!=(0|0);
    $633 = HEAP32[$regKey>>2]|0;
    $634 = HEAP32[$regOldRowid>>2]|0;
    $635 = $632 ? $633 : $634;
    $636 = HEAP32[$labelBreak>>2]|0;
    (_sqlite3VdbeAddOp2($630,34,$635,$636)|0);
   } else {
    $637 = HEAP32[$pPk>>2]|0;
    $638 = ($637|0)!=(0|0);
    $639 = HEAP32[$v>>2]|0;
    if ($638) {
     $640 = (_sqlite3VdbeMakeLabel($639)|0);
     HEAP32[$labelContinue>>2] = $640;
     $641 = HEAP32[$v>>2]|0;
     $642 = HEAP32[$iEph>>2]|0;
     $643 = HEAP32[$labelBreak>>2]|0;
     (_sqlite3VdbeAddOp2($641,57,$642,$643)|0);
     $644 = HEAP32[$v>>2]|0;
     $645 = HEAP32[$iEph>>2]|0;
     $646 = HEAP32[$regKey>>2]|0;
     $647 = (_sqlite3VdbeAddOp2($644,121,$645,$646)|0);
     HEAP32[$addrTop>>2] = $647;
     $648 = HEAP32[$v>>2]|0;
     $649 = HEAP32[$iDataCur>>2]|0;
     $650 = HEAP32[$labelContinue>>2]|0;
     $651 = HEAP32[$regKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($648,30,$649,$650,$651,0)|0);
     break;
    } else {
     $652 = HEAP32[$regRowSet>>2]|0;
     $653 = HEAP32[$labelBreak>>2]|0;
     $654 = HEAP32[$regOldRowid>>2]|0;
     $655 = (_sqlite3VdbeAddOp3($639,62,$652,$653,$654)|0);
     HEAP32[$labelContinue>>2] = $655;
     $656 = HEAP32[$v>>2]|0;
     $657 = HEAP32[$iDataCur>>2]|0;
     $658 = HEAP32[$labelContinue>>2]|0;
     $659 = HEAP32[$regOldRowid>>2]|0;
     (_sqlite3VdbeAddOp3($656,33,$657,$658,$659)|0);
     break;
    }
   }
  } while(0);
  $660 = HEAP8[$chngRowid>>0]|0;
  $661 = ($660<<24>>24)!=(0);
  if ($661) {
   $662 = HEAP32[$0>>2]|0;
   $663 = HEAP32[$pRowidExpr>>2]|0;
   $664 = HEAP32[$regNewRowid>>2]|0;
   _sqlite3ExprCode($662,$663,$664);
   $665 = HEAP32[$v>>2]|0;
   $666 = HEAP32[$regNewRowid>>2]|0;
   (_sqlite3VdbeAddOp1($665,17,$666)|0);
  }
  $667 = HEAP8[$chngPk>>0]|0;
  $668 = $667&255;
  $669 = ($668|0)!=(0);
  $670 = HEAP32[$hasFK>>2]|0;
  $671 = ($670|0)!=(0);
  $or$cond15 = $669 | $671;
  $672 = HEAP32[$pTrigger>>2]|0;
  $673 = ($672|0)!=(0|0);
  $or$cond17 = $or$cond15 | $673;
  do {
   if ($or$cond17) {
    $674 = HEAP32[$hasFK>>2]|0;
    $675 = ($674|0)!=(0);
    if ($675) {
     $676 = HEAP32[$0>>2]|0;
     $677 = HEAP32[$pTab>>2]|0;
     $678 = (_sqlite3FkOldmask($676,$677)|0);
     $679 = $678;
    } else {
     $679 = 0;
    }
    HEAP32[$oldmask>>2] = $679;
    $680 = HEAP32[$0>>2]|0;
    $681 = HEAP32[$pTrigger>>2]|0;
    $682 = HEAP32[$2>>2]|0;
    $683 = HEAP32[$pTab>>2]|0;
    $684 = HEAP32[$4>>2]|0;
    $685 = (_sqlite3TriggerColmask($680,$681,$682,0,3,$683,$684)|0);
    $686 = HEAP32[$oldmask>>2]|0;
    $687 = $686 | $685;
    HEAP32[$oldmask>>2] = $687;
    HEAP32[$i>>2] = 0;
    while(1) {
     $688 = HEAP32[$i>>2]|0;
     $689 = HEAP32[$pTab>>2]|0;
     $690 = ((($689)) + 34|0);
     $691 = HEAP16[$690>>1]|0;
     $692 = $691 << 16 >> 16;
     $693 = ($688|0)<($692|0);
     if (!($693)) {
      break;
     }
     $694 = HEAP32[$oldmask>>2]|0;
     $695 = ($694|0)==(-1);
     do {
      if ($695) {
       label = 120;
      } else {
       $696 = HEAP32[$i>>2]|0;
       $697 = ($696|0)<(32);
       if ($697) {
        $698 = HEAP32[$oldmask>>2]|0;
        $699 = HEAP32[$i>>2]|0;
        $700 = 1 << $699;
        $701 = $698 & $700;
        $702 = ($701|0)!=(0);
        if ($702) {
         label = 120;
         break;
        }
       }
       $703 = HEAP32[$i>>2]|0;
       $704 = HEAP32[$pTab>>2]|0;
       $705 = ((($704)) + 4|0);
       $706 = HEAP32[$705>>2]|0;
       $707 = (($706) + ($703<<4)|0);
       $708 = ((($707)) + 15|0);
       $709 = HEAP8[$708>>0]|0;
       $710 = $709&255;
       $711 = $710 & 1;
       $712 = ($711|0)!=(0);
       if ($712) {
        label = 120;
        break;
       }
       $720 = HEAP32[$v>>2]|0;
       $721 = HEAP32[$regOld>>2]|0;
       $722 = HEAP32[$i>>2]|0;
       $723 = (($721) + ($722))|0;
       (_sqlite3VdbeAddOp2($720,79,0,$723)|0);
      }
     } while(0);
     if ((label|0) == 120) {
      label = 0;
      $713 = HEAP32[$v>>2]|0;
      $714 = HEAP32[$pTab>>2]|0;
      $715 = HEAP32[$iDataCur>>2]|0;
      $716 = HEAP32[$i>>2]|0;
      $717 = HEAP32[$regOld>>2]|0;
      $718 = HEAP32[$i>>2]|0;
      $719 = (($717) + ($718))|0;
      _sqlite3ExprCodeGetColumnOfTable($713,$714,$715,$716,$719);
     }
     $724 = HEAP32[$i>>2]|0;
     $725 = (($724) + 1)|0;
     HEAP32[$i>>2] = $725;
    }
    $726 = HEAP8[$chngRowid>>0]|0;
    $727 = $726&255;
    $728 = ($727|0)==(0);
    $729 = HEAP32[$pPk>>2]|0;
    $730 = ($729|0)==(0|0);
    $or$cond19 = $728 & $730;
    if (!($or$cond19)) {
     break;
    }
    $731 = HEAP32[$v>>2]|0;
    $732 = HEAP32[$regOldRowid>>2]|0;
    $733 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp2($731,84,$732,$733)|0);
   }
  } while(0);
  $734 = HEAP32[$0>>2]|0;
  $735 = HEAP32[$pTrigger>>2]|0;
  $736 = HEAP32[$2>>2]|0;
  $737 = HEAP32[$pTab>>2]|0;
  $738 = HEAP32[$4>>2]|0;
  $739 = (_sqlite3TriggerColmask($734,$735,$736,1,1,$737,$738)|0);
  HEAP32[$newmask>>2] = $739;
  HEAP32[$i>>2] = 0;
  while(1) {
   $740 = HEAP32[$i>>2]|0;
   $741 = HEAP32[$pTab>>2]|0;
   $742 = ((($741)) + 34|0);
   $743 = HEAP16[$742>>1]|0;
   $744 = $743 << 16 >> 16;
   $745 = ($740|0)<($744|0);
   if (!($745)) {
    break;
   }
   $746 = HEAP32[$i>>2]|0;
   $747 = HEAP32[$pTab>>2]|0;
   $748 = ((($747)) + 32|0);
   $749 = HEAP16[$748>>1]|0;
   $750 = $749 << 16 >> 16;
   $751 = ($746|0)==($750|0);
   L192: do {
    if ($751) {
     $752 = HEAP32[$v>>2]|0;
     $753 = HEAP32[$regNew>>2]|0;
     $754 = HEAP32[$i>>2]|0;
     $755 = (($753) + ($754))|0;
     (_sqlite3VdbeAddOp2($752,79,0,$755)|0);
    } else {
     $756 = HEAP32[$i>>2]|0;
     $757 = HEAP32[$aXRef>>2]|0;
     $758 = (($757) + ($756<<2)|0);
     $759 = HEAP32[$758>>2]|0;
     HEAP32[$j>>2] = $759;
     $760 = HEAP32[$j>>2]|0;
     $761 = ($760|0)>=(0);
     if ($761) {
      $762 = HEAP32[$0>>2]|0;
      $763 = HEAP32[$j>>2]|0;
      $764 = HEAP32[$2>>2]|0;
      $765 = ((($764)) + 4|0);
      $766 = HEAP32[$765>>2]|0;
      $767 = (($766) + (($763*20)|0)|0);
      $768 = HEAP32[$767>>2]|0;
      $769 = HEAP32[$regNew>>2]|0;
      $770 = HEAP32[$i>>2]|0;
      $771 = (($769) + ($770))|0;
      _sqlite3ExprCode($762,$768,$771);
      break;
     }
     $772 = HEAP32[$tmask>>2]|0;
     $773 = $772 & 1;
     $774 = (0)==($773|0);
     $775 = HEAP32[$i>>2]|0;
     $776 = ($775|0)>(31);
     $or$cond21 = $774 | $776;
     do {
      if (!($or$cond21)) {
       $777 = HEAP32[$newmask>>2]|0;
       $778 = HEAP32[$i>>2]|0;
       $779 = 1 << $778;
       $780 = $777 & $779;
       $781 = ($780|0)!=(0);
       if ($781) {
        break;
       }
       $789 = HEAP32[$v>>2]|0;
       $790 = HEAP32[$regNew>>2]|0;
       $791 = HEAP32[$i>>2]|0;
       $792 = (($790) + ($791))|0;
       (_sqlite3VdbeAddOp2($789,79,0,$792)|0);
       break L192;
      }
     } while(0);
     $782 = HEAP32[$0>>2]|0;
     $783 = HEAP32[$pTab>>2]|0;
     $784 = HEAP32[$i>>2]|0;
     $785 = HEAP32[$iDataCur>>2]|0;
     $786 = HEAP32[$regNew>>2]|0;
     $787 = HEAP32[$i>>2]|0;
     $788 = (($786) + ($787))|0;
     _sqlite3ExprCodeGetColumnToReg($782,$783,$784,$785,$788);
    }
   } while(0);
   $793 = HEAP32[$i>>2]|0;
   $794 = (($793) + 1)|0;
   HEAP32[$i>>2] = $794;
  }
  $795 = HEAP32[$tmask>>2]|0;
  $796 = $795 & 1;
  $797 = ($796|0)!=(0);
  L204: do {
   if ($797) {
    $798 = HEAP32[$v>>2]|0;
    $799 = HEAP32[$pTab>>2]|0;
    $800 = HEAP32[$regNew>>2]|0;
    _sqlite3TableAffinity($798,$799,$800);
    $801 = HEAP32[$0>>2]|0;
    $802 = HEAP32[$pTrigger>>2]|0;
    $803 = HEAP32[$2>>2]|0;
    $804 = HEAP32[$pTab>>2]|0;
    $805 = HEAP32[$regOldRowid>>2]|0;
    $806 = HEAP32[$4>>2]|0;
    $807 = HEAP32[$labelContinue>>2]|0;
    _sqlite3CodeRowTrigger($801,$802,110,$803,1,$804,$805,$806,$807);
    $808 = HEAP32[$pPk>>2]|0;
    $809 = ($808|0)!=(0|0);
    $810 = HEAP32[$v>>2]|0;
    $811 = HEAP32[$iDataCur>>2]|0;
    $812 = HEAP32[$labelContinue>>2]|0;
    if ($809) {
     $813 = HEAP32[$regKey>>2]|0;
     $814 = HEAP32[$nKey>>2]|0;
     (_sqlite3VdbeAddOp4Int($810,30,$811,$812,$813,$814)|0);
    } else {
     $815 = HEAP32[$regOldRowid>>2]|0;
     (_sqlite3VdbeAddOp3($810,33,$811,$812,$815)|0);
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $816 = HEAP32[$i>>2]|0;
     $817 = HEAP32[$pTab>>2]|0;
     $818 = ((($817)) + 34|0);
     $819 = HEAP16[$818>>1]|0;
     $820 = $819 << 16 >> 16;
     $821 = ($816|0)<($820|0);
     if (!($821)) {
      break L204;
     }
     $822 = HEAP32[$i>>2]|0;
     $823 = HEAP32[$aXRef>>2]|0;
     $824 = (($823) + ($822<<2)|0);
     $825 = HEAP32[$824>>2]|0;
     $826 = ($825|0)<(0);
     do {
      if ($826) {
       $827 = HEAP32[$i>>2]|0;
       $828 = HEAP32[$pTab>>2]|0;
       $829 = ((($828)) + 32|0);
       $830 = HEAP16[$829>>1]|0;
       $831 = $830 << 16 >> 16;
       $832 = ($827|0)!=($831|0);
       if (!($832)) {
        break;
       }
       $833 = HEAP32[$v>>2]|0;
       $834 = HEAP32[$pTab>>2]|0;
       $835 = HEAP32[$iDataCur>>2]|0;
       $836 = HEAP32[$i>>2]|0;
       $837 = HEAP32[$regNew>>2]|0;
       $838 = HEAP32[$i>>2]|0;
       $839 = (($837) + ($838))|0;
       _sqlite3ExprCodeGetColumnOfTable($833,$834,$835,$836,$839);
      }
     } while(0);
     $840 = HEAP32[$i>>2]|0;
     $841 = (($840) + 1)|0;
     HEAP32[$i>>2] = $841;
    }
   }
  } while(0);
  $842 = HEAP32[$isView>>2]|0;
  $843 = ($842|0)!=(0);
  do {
   if (!($843)) {
    HEAP32[$addr1>>2] = 0;
    HEAP32[$bReplace>>2] = 0;
    $844 = HEAP32[$0>>2]|0;
    $845 = HEAP32[$pTab>>2]|0;
    $846 = HEAP32[$aRegIdx>>2]|0;
    $847 = HEAP32[$iDataCur>>2]|0;
    $848 = HEAP32[$iIdxCur>>2]|0;
    $849 = HEAP32[$regNewRowid>>2]|0;
    $850 = HEAP32[$regOldRowid>>2]|0;
    $851 = HEAP8[$chngKey>>0]|0;
    $852 = HEAP32[$4>>2]|0;
    $853 = $852&255;
    $854 = HEAP32[$labelContinue>>2]|0;
    $855 = HEAP32[$aXRef>>2]|0;
    _sqlite3GenerateConstraintChecks($844,$845,$846,$847,$848,$849,$850,$851,$853,$854,$bReplace,$855);
    $856 = HEAP32[$hasFK>>2]|0;
    $857 = ($856|0)!=(0);
    if ($857) {
     $858 = HEAP32[$0>>2]|0;
     $859 = HEAP32[$pTab>>2]|0;
     $860 = HEAP32[$regOldRowid>>2]|0;
     $861 = HEAP32[$aXRef>>2]|0;
     $862 = HEAP8[$chngKey>>0]|0;
     $863 = $862&255;
     _sqlite3FkCheck($858,$859,$860,0,$861,$863);
    }
    $864 = HEAP32[$bReplace>>2]|0;
    $865 = ($864|0)!=(0);
    if ($865) {
     label = 151;
    } else {
     $866 = HEAP8[$chngKey>>0]|0;
     $867 = $866&255;
     $868 = ($867|0)!=(0);
     if ($868) {
      label = 151;
     }
    }
    do {
     if ((label|0) == 151) {
      $869 = HEAP32[$pPk>>2]|0;
      $870 = ($869|0)!=(0|0);
      $871 = HEAP32[$v>>2]|0;
      $872 = HEAP32[$iDataCur>>2]|0;
      if ($870) {
       $873 = HEAP32[$regKey>>2]|0;
       $874 = HEAP32[$nKey>>2]|0;
       $875 = (_sqlite3VdbeAddOp4Int($871,30,$872,0,$873,$874)|0);
       HEAP32[$addr1>>2] = $875;
       break;
      } else {
       $876 = HEAP32[$regOldRowid>>2]|0;
       $877 = (_sqlite3VdbeAddOp3($871,33,$872,0,$876)|0);
       HEAP32[$addr1>>2] = $877;
       break;
      }
     }
    } while(0);
    $878 = HEAP32[$0>>2]|0;
    $879 = HEAP32[$pTab>>2]|0;
    $880 = HEAP32[$iDataCur>>2]|0;
    $881 = HEAP32[$iIdxCur>>2]|0;
    $882 = HEAP32[$aRegIdx>>2]|0;
    _sqlite3GenerateRowIndexDelete($878,$879,$880,$881,$882,-1);
    $883 = HEAP32[$hasFK>>2]|0;
    $884 = ($883|0)!=(0);
    if ($884) {
     label = 156;
    } else {
     $885 = HEAP8[$chngKey>>0]|0;
     $886 = $885&255;
     $887 = ($886|0)!=(0);
     $888 = HEAP32[$pPk>>2]|0;
     $889 = ($888|0)!=(0|0);
     $or$cond23 = $887 | $889;
     if ($or$cond23) {
      label = 156;
     }
    }
    if ((label|0) == 156) {
     $890 = HEAP32[$v>>2]|0;
     $891 = HEAP32[$iDataCur>>2]|0;
     (_sqlite3VdbeAddOp2($890,117,$891,0)|0);
    }
    $892 = HEAP32[$bReplace>>2]|0;
    $893 = ($892|0)!=(0);
    if ($893) {
     label = 159;
    } else {
     $894 = HEAP8[$chngKey>>0]|0;
     $895 = $894&255;
     $896 = ($895|0)!=(0);
     if ($896) {
      label = 159;
     }
    }
    if ((label|0) == 159) {
     $897 = HEAP32[$v>>2]|0;
     $898 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($897,$898);
    }
    $899 = HEAP32[$hasFK>>2]|0;
    $900 = ($899|0)!=(0);
    if ($900) {
     $901 = HEAP32[$0>>2]|0;
     $902 = HEAP32[$pTab>>2]|0;
     $903 = HEAP32[$regNewRowid>>2]|0;
     $904 = HEAP32[$aXRef>>2]|0;
     $905 = HEAP8[$chngKey>>0]|0;
     $906 = $905&255;
     _sqlite3FkCheck($901,$902,0,$903,$904,$906);
    }
    $907 = HEAP32[$0>>2]|0;
    $908 = HEAP32[$pTab>>2]|0;
    $909 = HEAP32[$iDataCur>>2]|0;
    $910 = HEAP32[$iIdxCur>>2]|0;
    $911 = HEAP32[$regNewRowid>>2]|0;
    $912 = HEAP32[$aRegIdx>>2]|0;
    _sqlite3CompleteInsertion($907,$908,$909,$910,$911,$912,1,0,0);
    $913 = HEAP32[$hasFK>>2]|0;
    $914 = ($913|0)!=(0);
    if (!($914)) {
     break;
    }
    $915 = HEAP32[$0>>2]|0;
    $916 = HEAP32[$pTab>>2]|0;
    $917 = HEAP32[$2>>2]|0;
    $918 = HEAP32[$regOldRowid>>2]|0;
    $919 = HEAP32[$aXRef>>2]|0;
    $920 = HEAP8[$chngKey>>0]|0;
    $921 = $920&255;
    _sqlite3FkActions($915,$916,$917,$918,$919,$921);
   }
  } while(0);
  $922 = HEAP32[$db>>2]|0;
  $923 = ((($922)) + 24|0);
  $924 = HEAP32[$923>>2]|0;
  $925 = $924 & 128;
  $926 = ($925|0)!=(0);
  do {
   if ($926) {
    $927 = HEAP32[$0>>2]|0;
    $928 = ((($927)) + 128|0);
    $929 = HEAP32[$928>>2]|0;
    $930 = ($929|0)!=(0|0);
    if ($930) {
     break;
    }
    $931 = HEAP32[$v>>2]|0;
    $932 = HEAP32[$regRowCount>>2]|0;
    (_sqlite3VdbeAddOp2($931,91,$932,1)|0);
   }
  } while(0);
  $933 = HEAP32[$0>>2]|0;
  $934 = HEAP32[$pTrigger>>2]|0;
  $935 = HEAP32[$2>>2]|0;
  $936 = HEAP32[$pTab>>2]|0;
  $937 = HEAP32[$regOldRowid>>2]|0;
  $938 = HEAP32[$4>>2]|0;
  $939 = HEAP32[$labelContinue>>2]|0;
  _sqlite3CodeRowTrigger($933,$934,110,$935,2,$936,$937,$938,$939);
  $940 = HEAP32[$okOnePass>>2]|0;
  $941 = ($940|0)!=(0);
  do {
   if (!($941)) {
    $942 = HEAP32[$pPk>>2]|0;
    $943 = ($942|0)!=(0|0);
    $944 = HEAP32[$v>>2]|0;
    $945 = HEAP32[$labelContinue>>2]|0;
    if ($943) {
     _sqlite3VdbeResolveLabel($944,$945);
     $946 = HEAP32[$v>>2]|0;
     $947 = HEAP32[$iEph>>2]|0;
     $948 = HEAP32[$addrTop>>2]|0;
     (_sqlite3VdbeAddOp2($946,7,$947,$948)|0);
     break;
    } else {
     (_sqlite3VdbeGoto($944,$945)|0);
     break;
    }
   }
  } while(0);
  $949 = HEAP32[$v>>2]|0;
  $950 = HEAP32[$labelBreak>>2]|0;
  _sqlite3VdbeResolveLabel($949,$950);
  HEAP32[$i>>2] = 0;
  $951 = HEAP32[$pTab>>2]|0;
  $952 = ((($951)) + 8|0);
  $953 = HEAP32[$952>>2]|0;
  HEAP32[$pIdx>>2] = $953;
  while(1) {
   $954 = HEAP32[$pIdx>>2]|0;
   $955 = ($954|0)!=(0|0);
   if (!($955)) {
    break;
   }
   $956 = HEAP32[$i>>2]|0;
   $957 = (($956) + 1)|0;
   $958 = HEAP32[$aToOpen>>2]|0;
   $959 = (($958) + ($957)|0);
   $960 = HEAP8[$959>>0]|0;
   $961 = ($960<<24>>24)!=(0);
   if ($961) {
    $962 = HEAP32[$v>>2]|0;
    $963 = HEAP32[$iIdxCur>>2]|0;
    $964 = HEAP32[$i>>2]|0;
    $965 = (($963) + ($964))|0;
    (_sqlite3VdbeAddOp2($962,111,$965,0)|0);
   }
   $966 = HEAP32[$pIdx>>2]|0;
   $967 = ((($966)) + 20|0);
   $968 = HEAP32[$967>>2]|0;
   HEAP32[$pIdx>>2] = $968;
   $969 = HEAP32[$i>>2]|0;
   $970 = (($969) + 1)|0;
   HEAP32[$i>>2] = $970;
  }
  $971 = HEAP32[$iDataCur>>2]|0;
  $972 = HEAP32[$iIdxCur>>2]|0;
  $973 = ($971|0)<($972|0);
  if ($973) {
   $974 = HEAP32[$v>>2]|0;
   $975 = HEAP32[$iDataCur>>2]|0;
   (_sqlite3VdbeAddOp2($974,111,$975,0)|0);
  }
  $976 = HEAP32[$0>>2]|0;
  $977 = ((($976)) + 18|0);
  $978 = HEAP8[$977>>0]|0;
  $979 = $978&255;
  $980 = ($979|0)==(0);
  do {
   if ($980) {
    $981 = HEAP32[$0>>2]|0;
    $982 = ((($981)) + 128|0);
    $983 = HEAP32[$982>>2]|0;
    $984 = ($983|0)==(0|0);
    if (!($984)) {
     break;
    }
    $985 = HEAP32[$0>>2]|0;
    _sqlite3AutoincrementEnd($985);
   }
  } while(0);
  $986 = HEAP32[$db>>2]|0;
  $987 = ((($986)) + 24|0);
  $988 = HEAP32[$987>>2]|0;
  $989 = $988 & 128;
  $990 = ($989|0)!=(0);
  if (!($990)) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $991 = HEAP32[$0>>2]|0;
  $992 = ((($991)) + 128|0);
  $993 = HEAP32[$992>>2]|0;
  $994 = ($993|0)!=(0|0);
  if ($994) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $995 = HEAP32[$0>>2]|0;
  $996 = ((($995)) + 18|0);
  $997 = HEAP8[$996>>0]|0;
  $998 = ($997<<24>>24)!=(0);
  if ($998) {
   _sqlite3AuthContextPop($sContext);
   $1003 = HEAP32[$db>>2]|0;
   $1004 = HEAP32[$aXRef>>2]|0;
   _sqlite3DbFree($1003,$1004);
   $1005 = HEAP32[$db>>2]|0;
   $1006 = HEAP32[$1>>2]|0;
   _sqlite3SrcListDelete($1005,$1006);
   $1007 = HEAP32[$db>>2]|0;
   $1008 = HEAP32[$2>>2]|0;
   _sqlite3ExprListDelete($1007,$1008);
   $1009 = HEAP32[$db>>2]|0;
   $1010 = HEAP32[$3>>2]|0;
   _sqlite3ExprDelete($1009,$1010);
   STACKTOP = sp;return;
  }
  $999 = HEAP32[$v>>2]|0;
  $1000 = HEAP32[$regRowCount>>2]|0;
  (_sqlite3VdbeAddOp2($999,87,$1000,1)|0);
  $1001 = HEAP32[$v>>2]|0;
  _sqlite3VdbeSetNumCols($1001,1);
  $1002 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeSetColName($1002,0,0,35325,0)|0);
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 185) {
  _sqlite3AuthContextPop($sContext);
  $1003 = HEAP32[$db>>2]|0;
  $1004 = HEAP32[$aXRef>>2]|0;
  _sqlite3DbFree($1003,$1004);
  $1005 = HEAP32[$db>>2]|0;
  $1006 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($1005,$1006);
  $1007 = HEAP32[$db>>2]|0;
  $1008 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($1007,$1008);
  $1009 = HEAP32[$db>>2]|0;
  $1010 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($1009,$1010);
  STACKTOP = sp;return;
 }
}
function _sqlite3ExprListAppendVector($pParse,$pList,$pColumns,$pExpr) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pColumns = $pColumns|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $db = 0, $i = 0, $iFirst = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $pSubExpr = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $db = sp + 24|0;
 $n = sp + 20|0;
 $i = sp + 16|0;
 $iFirst = sp + 12|0;
 $pSubExpr = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $pColumns;
 HEAP32[$3>>2] = $pExpr;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = $9;
 } else {
  $10 = 0;
 }
 HEAP32[$iFirst>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 $13 = HEAP32[$3>>2]|0;
 $14 = ($13|0)==(0|0);
 $or$cond = $12 | $14;
 do {
  if (!($or$cond)) {
   $15 = HEAP32[$3>>2]|0;
   $16 = (_sqlite3ExprVectorSize($15)|0);
   HEAP32[$n>>2] = $16;
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$n>>2]|0;
   $21 = ($19|0)!=($20|0);
   if ($21) {
    $22 = HEAP32[$0>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    $24 = ((($23)) + 4|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$n>>2]|0;
    HEAP32[$vararg_buffer>>2] = $25;
    $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $26;
    _sqlite3ErrorMsg($22,35295,$vararg_buffer);
    break;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$n>>2]|0;
    $29 = ($27|0)<($28|0);
    if (!($29)) {
     break;
    }
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$3>>2]|0;
    $32 = HEAP32[$i>>2]|0;
    $33 = (_sqlite3ExprForVectorField($30,$31,$32)|0);
    HEAP32[$pSubExpr>>2] = $33;
    $34 = HEAP32[$0>>2]|0;
    $35 = HEAP32[$1>>2]|0;
    $36 = HEAP32[$pSubExpr>>2]|0;
    $37 = (_sqlite3ExprListAppend($34,$35,$36)|0);
    HEAP32[$1>>2] = $37;
    $38 = HEAP32[$1>>2]|0;
    $39 = ($38|0)!=(0|0);
    if ($39) {
     $40 = HEAP32[$i>>2]|0;
     $41 = HEAP32[$2>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     $43 = (($42) + ($40<<3)|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$1>>2]|0;
     $46 = HEAP32[$45>>2]|0;
     $47 = (($46) - 1)|0;
     $48 = HEAP32[$1>>2]|0;
     $49 = ((($48)) + 4|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = (($50) + (($47*20)|0)|0);
     $52 = ((($51)) + 4|0);
     HEAP32[$52>>2] = $44;
     $53 = HEAP32[$i>>2]|0;
     $54 = HEAP32[$2>>2]|0;
     $55 = HEAP32[$54>>2]|0;
     $56 = (($55) + ($53<<3)|0);
     HEAP32[$56>>2] = 0;
    }
    $57 = HEAP32[$i>>2]|0;
    $58 = (($57) + 1)|0;
    HEAP32[$i>>2] = $58;
   }
   $59 = HEAP32[$3>>2]|0;
   $60 = HEAP8[$59>>0]|0;
   $61 = $60&255;
   $62 = ($61|0)==(119);
   $63 = HEAP32[$1>>2]|0;
   $64 = ($63|0)!=(0|0);
   $or$cond3 = $62 & $64;
   if ($or$cond3) {
    $65 = HEAP32[$iFirst>>2]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = ((($66)) + 4|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($68) + (($65*20)|0)|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ($70|0)!=(0|0);
    if ($71) {
     $72 = HEAP32[$3>>2]|0;
     $73 = HEAP32[$iFirst>>2]|0;
     $74 = HEAP32[$1>>2]|0;
     $75 = ((($74)) + 4|0);
     $76 = HEAP32[$75>>2]|0;
     $77 = (($76) + (($73*20)|0)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = ((($78)) + 16|0);
     HEAP32[$79>>2] = $72;
     HEAP32[$3>>2] = 0;
    }
   }
  }
 } while(0);
 $80 = HEAP32[$db>>2]|0;
 $81 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($80,$81);
 $82 = HEAP32[$db>>2]|0;
 $83 = HEAP32[$2>>2]|0;
 _sqlite3IdListDelete($82,$83);
 $84 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($84|0);
}
function _sqlite3Insert($pParse,$pTabList,$pSelect,$pColumn,$onError) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pSelect = $pSelect|0;
 $pColumn = $pColumn|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0;
 var $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0;
 var $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0;
 var $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0;
 var $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0;
 var $927 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aRegIdx = 0, $addr1 = 0, $addr11 = 0, $addrCont = 0, $addrInsTop = 0, $addrL = 0, $addrTop = 0, $appendFlag = 0, $bIdListInOrder = 0, $db = 0, $dest = 0, $endOfLoop = 0;
 var $i = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $iRegStore = 0, $idx = 0, $ipkColumn = 0, $isReplace = 0, $isView = 0, $j = 0, $nColumn = 0, $nHidden = 0, $nIdx = 0, $or$cond = 0, $or$cond11 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pIdx = 0;
 var $pList = 0, $pOp = 0, $pTab = 0, $pTrigger = 0, $pVTab = 0, $rc = 0, $regAutoinc = 0, $regCols = 0, $regData = 0, $regFromSelect = 0, $regIns = 0, $regRec = 0, $regRowCount = 0, $regRowid = 0, $regTempRowid = 0, $regYield = 0, $sNC = 0, $srcTab = 0, $tmask = 0, $useTempTable = 0;
 var $v = 0, $vararg_buffer = 0, $vararg_buffer14 = 0, $vararg_buffer20 = 0, $vararg_ptr12 = 0, $vararg_ptr13 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr23 = 0, $withoutRowid = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 304|0;
 $vararg_buffer20 = sp + 32|0;
 $vararg_buffer14 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 284|0;
 $1 = sp + 280|0;
 $2 = sp + 276|0;
 $3 = sp + 272|0;
 $4 = sp + 268|0;
 $db = sp + 264|0;
 $pTab = sp + 260|0;
 $zTab = sp + 256|0;
 $i = sp + 252|0;
 $j = sp + 248|0;
 $idx = sp + 244|0;
 $v = sp + 240|0;
 $pIdx = sp + 236|0;
 $nColumn = sp + 232|0;
 $nHidden = sp + 228|0;
 $iDataCur = sp + 224|0;
 $iIdxCur = sp + 220|0;
 $ipkColumn = sp + 216|0;
 $endOfLoop = sp + 212|0;
 $srcTab = sp + 208|0;
 $addrInsTop = sp + 204|0;
 $addrCont = sp + 200|0;
 $dest = sp + 176|0;
 $iDb = sp + 172|0;
 $useTempTable = sp + 291|0;
 $appendFlag = sp + 290|0;
 $withoutRowid = sp + 289|0;
 $bIdListInOrder = sp + 288|0;
 $pList = sp + 168|0;
 $regFromSelect = sp + 164|0;
 $regAutoinc = sp + 160|0;
 $regRowCount = sp + 156|0;
 $regIns = sp + 152|0;
 $regRowid = sp + 148|0;
 $regData = sp + 144|0;
 $aRegIdx = sp + 140|0;
 $isView = sp + 136|0;
 $pTrigger = sp + 132|0;
 $tmask = sp + 128|0;
 $regYield = sp + 124|0;
 $addrTop = sp + 120|0;
 $rc = sp + 116|0;
 $regRec = sp + 112|0;
 $regTempRowid = sp + 108|0;
 $addrL = sp + 104|0;
 $sNC = sp + 72|0;
 $nIdx = sp + 68|0;
 $regCols = sp + 64|0;
 $addr1 = sp + 60|0;
 $pOp = sp + 56|0;
 $addr11 = sp + 52|0;
 $iRegStore = sp + 48|0;
 $pVTab = sp + 44|0;
 $isReplace = sp + 40|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pColumn;
 HEAP32[$4>>2] = $onError;
 HEAP32[$nHidden>>2] = 0;
 HEAP32[$iDataCur>>2] = 0;
 HEAP32[$iIdxCur>>2] = 0;
 HEAP32[$ipkColumn>>2] = -1;
 HEAP32[$srcTab>>2] = 0;
 HEAP32[$addrInsTop>>2] = 0;
 HEAP32[$addrCont>>2] = 0;
 HEAP8[$useTempTable>>0] = 0;
 HEAP8[$appendFlag>>0] = 0;
 HEAP32[$pList>>2] = 0;
 HEAP32[$regFromSelect>>2] = 0;
 HEAP32[$regAutoinc>>2] = 0;
 HEAP32[$regRowCount>>2] = 0;
 HEAP32[$aRegIdx>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 ;HEAP32[$dest>>2]=0|0;HEAP32[$dest+4>>2]=0|0;HEAP32[$dest+8>>2]=0|0;HEAP32[$dest+12>>2]=0|0;HEAP32[$dest+16>>2]=0|0;HEAP32[$dest+20>>2]=0|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 L1: do {
  if (!($10)) {
   $11 = HEAP32[$db>>2]|0;
   $12 = ((($11)) + 69|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = $13&255;
   $15 = ($14|0)!=(0);
   if (!($15)) {
    $16 = HEAP32[$2>>2]|0;
    $17 = ($16|0)!=(0|0);
    if ($17) {
     $18 = HEAP32[$2>>2]|0;
     $19 = ((($18)) + 8|0);
     $20 = HEAP32[$19>>2]|0;
     $21 = $20 & 512;
     $22 = ($21|0)!=(0);
     if ($22) {
      $23 = HEAP32[$2>>2]|0;
      $24 = ((($23)) + 48|0);
      $25 = HEAP32[$24>>2]|0;
      $26 = ($25|0)==(0|0);
      if ($26) {
       $27 = HEAP32[$2>>2]|0;
       $28 = HEAP32[$27>>2]|0;
       HEAP32[$pList>>2] = $28;
       $29 = HEAP32[$2>>2]|0;
       HEAP32[$29>>2] = 0;
       $30 = HEAP32[$db>>2]|0;
       $31 = HEAP32[$2>>2]|0;
       _sqlite3SelectDelete($30,$31);
       HEAP32[$2>>2] = 0;
      }
     }
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 8|0);
    $34 = ((($33)) + 8|0);
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$zTab>>2] = $35;
    $36 = HEAP32[$zTab>>2]|0;
    $37 = ($36|0)==(0|0);
    if (!($37)) {
     $38 = HEAP32[$0>>2]|0;
     $39 = HEAP32[$1>>2]|0;
     $40 = (_sqlite3SrcListLookup($38,$39)|0);
     HEAP32[$pTab>>2] = $40;
     $41 = HEAP32[$pTab>>2]|0;
     $42 = ($41|0)==(0|0);
     if (!($42)) {
      $43 = HEAP32[$db>>2]|0;
      $44 = HEAP32[$pTab>>2]|0;
      $45 = ((($44)) + 64|0);
      $46 = HEAP32[$45>>2]|0;
      $47 = (_sqlite3SchemaToIndex($43,$46)|0);
      HEAP32[$iDb>>2] = $47;
      $48 = HEAP32[$0>>2]|0;
      $49 = HEAP32[$pTab>>2]|0;
      $50 = HEAP32[$49>>2]|0;
      $51 = HEAP32[$iDb>>2]|0;
      $52 = HEAP32[$db>>2]|0;
      $53 = ((($52)) + 16|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = (($54) + ($51<<4)|0);
      $56 = HEAP32[$55>>2]|0;
      $57 = (_sqlite3AuthCheck($48,18,$50,0,$56)|0);
      $58 = ($57|0)!=(0);
      if (!($58)) {
       $59 = HEAP32[$pTab>>2]|0;
       $60 = ((($59)) + 42|0);
       $61 = HEAP8[$60>>0]|0;
       $62 = $61&255;
       $63 = $62 & 32;
       $64 = ($63|0)==(0);
       $65 = $64 ^ 1;
       $66 = $65&1;
       $67 = $66&255;
       HEAP8[$withoutRowid>>0] = $67;
       $68 = HEAP32[$0>>2]|0;
       $69 = HEAP32[$pTab>>2]|0;
       $70 = (_sqlite3TriggersExist($68,$69,108,0,$tmask)|0);
       HEAP32[$pTrigger>>2] = $70;
       $71 = HEAP32[$pTab>>2]|0;
       $72 = ((($71)) + 12|0);
       $73 = HEAP32[$72>>2]|0;
       $74 = ($73|0)!=(0|0);
       $75 = $74&1;
       HEAP32[$isView>>2] = $75;
       $76 = HEAP32[$0>>2]|0;
       $77 = HEAP32[$pTab>>2]|0;
       $78 = (_sqlite3ViewGetColumnNames($76,$77)|0);
       $79 = ($78|0)!=(0);
       if (!($79)) {
        $80 = HEAP32[$0>>2]|0;
        $81 = HEAP32[$pTab>>2]|0;
        $82 = HEAP32[$tmask>>2]|0;
        $83 = (_sqlite3IsReadOnly($80,$81,$82)|0);
        $84 = ($83|0)!=(0);
        if (!($84)) {
         $85 = HEAP32[$0>>2]|0;
         $86 = (_sqlite3GetVdbe($85)|0);
         HEAP32[$v>>2] = $86;
         $87 = HEAP32[$v>>2]|0;
         $88 = ($87|0)==(0|0);
         if (!($88)) {
          $89 = HEAP32[$0>>2]|0;
          $90 = ((($89)) + 18|0);
          $91 = HEAP8[$90>>0]|0;
          $92 = $91&255;
          $93 = ($92|0)==(0);
          if ($93) {
           $94 = HEAP32[$v>>2]|0;
           _sqlite3VdbeCountChanges($94);
          }
          $95 = HEAP32[$0>>2]|0;
          $96 = HEAP32[$2>>2]|0;
          $97 = ($96|0)!=(0|0);
          $98 = HEAP32[$pTrigger>>2]|0;
          $99 = ($98|0)!=(0|0);
          $100 = $97 ? 1 : $99;
          $101 = $100&1;
          $102 = HEAP32[$iDb>>2]|0;
          _sqlite3BeginWriteOperation($95,$101,$102);
          $103 = HEAP32[$3>>2]|0;
          $104 = ($103|0)==(0|0);
          if ($104) {
           $105 = HEAP32[$0>>2]|0;
           $106 = HEAP32[$pTab>>2]|0;
           $107 = HEAP32[$2>>2]|0;
           $108 = HEAP32[$4>>2]|0;
           $109 = HEAP32[$iDb>>2]|0;
           $110 = (_xferOptimization($105,$106,$107,$108,$109)|0);
           $111 = ($110|0)!=(0);
           if (!($111)) {
            label = 17;
           }
          } else {
           label = 17;
          }
          L20: do {
           if ((label|0) == 17) {
            $112 = HEAP32[$0>>2]|0;
            $113 = HEAP32[$iDb>>2]|0;
            $114 = HEAP32[$pTab>>2]|0;
            $115 = (_autoIncBegin($112,$113,$114)|0);
            HEAP32[$regAutoinc>>2] = $115;
            $116 = HEAP32[$0>>2]|0;
            $117 = ((($116)) + 44|0);
            $118 = HEAP32[$117>>2]|0;
            $119 = (($118) + 1)|0;
            HEAP32[$regIns>>2] = $119;
            HEAP32[$regRowid>>2] = $119;
            $120 = HEAP32[$pTab>>2]|0;
            $121 = ((($120)) + 34|0);
            $122 = HEAP16[$121>>1]|0;
            $123 = $122 << 16 >> 16;
            $124 = (($123) + 1)|0;
            $125 = HEAP32[$0>>2]|0;
            $126 = ((($125)) + 44|0);
            $127 = HEAP32[$126>>2]|0;
            $128 = (($127) + ($124))|0;
            HEAP32[$126>>2] = $128;
            $129 = HEAP32[$pTab>>2]|0;
            $130 = ((($129)) + 42|0);
            $131 = HEAP8[$130>>0]|0;
            $132 = $131&255;
            $133 = $132 & 16;
            $134 = ($133|0)!=(0);
            if ($134) {
             $135 = HEAP32[$regRowid>>2]|0;
             $136 = (($135) + 1)|0;
             HEAP32[$regRowid>>2] = $136;
             $137 = HEAP32[$0>>2]|0;
             $138 = ((($137)) + 44|0);
             $139 = HEAP32[$138>>2]|0;
             $140 = (($139) + 1)|0;
             HEAP32[$138>>2] = $140;
            }
            $141 = HEAP32[$regRowid>>2]|0;
            $142 = (($141) + 1)|0;
            HEAP32[$regData>>2] = $142;
            $143 = HEAP32[$pTab>>2]|0;
            $144 = ((($143)) + 42|0);
            $145 = HEAP8[$144>>0]|0;
            $146 = $145&255;
            $147 = $146 & 128;
            $148 = ($147|0)==(0);
            $149 = $148&1;
            $150 = $149&255;
            HEAP8[$bIdListInOrder>>0] = $150;
            $151 = HEAP32[$3>>2]|0;
            $152 = ($151|0)!=(0|0);
            L25: do {
             if ($152) {
              HEAP32[$i>>2] = 0;
              while(1) {
               $153 = HEAP32[$i>>2]|0;
               $154 = HEAP32[$3>>2]|0;
               $155 = ((($154)) + 4|0);
               $156 = HEAP32[$155>>2]|0;
               $157 = ($153|0)<($156|0);
               if (!($157)) {
                break;
               }
               $158 = HEAP32[$i>>2]|0;
               $159 = HEAP32[$3>>2]|0;
               $160 = HEAP32[$159>>2]|0;
               $161 = (($160) + ($158<<3)|0);
               $162 = ((($161)) + 4|0);
               HEAP32[$162>>2] = -1;
               $163 = HEAP32[$i>>2]|0;
               $164 = (($163) + 1)|0;
               HEAP32[$i>>2] = $164;
              }
              HEAP32[$i>>2] = 0;
              while(1) {
               $165 = HEAP32[$i>>2]|0;
               $166 = HEAP32[$3>>2]|0;
               $167 = ((($166)) + 4|0);
               $168 = HEAP32[$167>>2]|0;
               $169 = ($165|0)<($168|0);
               if (!($169)) {
                break L25;
               }
               HEAP32[$j>>2] = 0;
               while(1) {
                $170 = HEAP32[$j>>2]|0;
                $171 = HEAP32[$pTab>>2]|0;
                $172 = ((($171)) + 34|0);
                $173 = HEAP16[$172>>1]|0;
                $174 = $173 << 16 >> 16;
                $175 = ($170|0)<($174|0);
                if (!($175)) {
                 break;
                }
                $176 = HEAP32[$i>>2]|0;
                $177 = HEAP32[$3>>2]|0;
                $178 = HEAP32[$177>>2]|0;
                $179 = (($178) + ($176<<3)|0);
                $180 = HEAP32[$179>>2]|0;
                $181 = HEAP32[$j>>2]|0;
                $182 = HEAP32[$pTab>>2]|0;
                $183 = ((($182)) + 4|0);
                $184 = HEAP32[$183>>2]|0;
                $185 = (($184) + ($181<<4)|0);
                $186 = HEAP32[$185>>2]|0;
                $187 = (_sqlite3StrICmp($180,$186)|0);
                $188 = ($187|0)==(0);
                $189 = HEAP32[$j>>2]|0;
                if ($188) {
                 label = 28;
                 break;
                }
                $205 = (($189) + 1)|0;
                HEAP32[$j>>2] = $205;
               }
               do {
                if ((label|0) == 28) {
                 label = 0;
                 $190 = HEAP32[$i>>2]|0;
                 $191 = HEAP32[$3>>2]|0;
                 $192 = HEAP32[$191>>2]|0;
                 $193 = (($192) + ($190<<3)|0);
                 $194 = ((($193)) + 4|0);
                 HEAP32[$194>>2] = $189;
                 $195 = HEAP32[$i>>2]|0;
                 $196 = HEAP32[$j>>2]|0;
                 $197 = ($195|0)!=($196|0);
                 if ($197) {
                  HEAP8[$bIdListInOrder>>0] = 0;
                 }
                 $198 = HEAP32[$j>>2]|0;
                 $199 = HEAP32[$pTab>>2]|0;
                 $200 = ((($199)) + 32|0);
                 $201 = HEAP16[$200>>1]|0;
                 $202 = $201 << 16 >> 16;
                 $203 = ($198|0)==($202|0);
                 if (!($203)) {
                  break;
                 }
                 $204 = HEAP32[$i>>2]|0;
                 HEAP32[$ipkColumn>>2] = $204;
                }
               } while(0);
               $206 = HEAP32[$j>>2]|0;
               $207 = HEAP32[$pTab>>2]|0;
               $208 = ((($207)) + 34|0);
               $209 = HEAP16[$208>>1]|0;
               $210 = $209 << 16 >> 16;
               $211 = ($206|0)>=($210|0);
               if ($211) {
                $212 = HEAP32[$i>>2]|0;
                $213 = HEAP32[$3>>2]|0;
                $214 = HEAP32[$213>>2]|0;
                $215 = (($214) + ($212<<3)|0);
                $216 = HEAP32[$215>>2]|0;
                $217 = (_sqlite3IsRowid($216)|0);
                $218 = ($217|0)==(0);
                $219 = HEAP8[$withoutRowid>>0]|0;
                $220 = ($219<<24>>24)!=(0);
                $or$cond = $218 | $220;
                if ($or$cond) {
                 break;
                }
                $221 = HEAP32[$i>>2]|0;
                HEAP32[$ipkColumn>>2] = $221;
                HEAP8[$bIdListInOrder>>0] = 0;
               }
               $231 = HEAP32[$i>>2]|0;
               $232 = (($231) + 1)|0;
               HEAP32[$i>>2] = $232;
              }
              $222 = HEAP32[$0>>2]|0;
              $223 = HEAP32[$1>>2]|0;
              $224 = HEAP32[$i>>2]|0;
              $225 = HEAP32[$3>>2]|0;
              $226 = HEAP32[$225>>2]|0;
              $227 = (($226) + ($224<<3)|0);
              $228 = HEAP32[$227>>2]|0;
              HEAP32[$vararg_buffer>>2] = $223;
              $vararg_ptr12 = ((($vararg_buffer)) + 4|0);
              HEAP32[$vararg_ptr12>>2] = 0;
              $vararg_ptr13 = ((($vararg_buffer)) + 8|0);
              HEAP32[$vararg_ptr13>>2] = $228;
              _sqlite3ErrorMsg($222,34261,$vararg_buffer);
              $229 = HEAP32[$0>>2]|0;
              $230 = ((($229)) + 17|0);
              HEAP8[$230>>0] = 1;
              break L1;
             }
            } while(0);
            $233 = HEAP32[$2>>2]|0;
            $234 = ($233|0)!=(0|0);
            do {
             if ($234) {
              $235 = HEAP32[$0>>2]|0;
              $236 = ((($235)) + 44|0);
              $237 = HEAP32[$236>>2]|0;
              $238 = (($237) + 1)|0;
              HEAP32[$236>>2] = $238;
              HEAP32[$regYield>>2] = $238;
              $239 = HEAP32[$v>>2]|0;
              $240 = (_sqlite3VdbeCurrentAddr($239)|0);
              $241 = (($240) + 1)|0;
              HEAP32[$addrTop>>2] = $241;
              $242 = HEAP32[$v>>2]|0;
              $243 = HEAP32[$regYield>>2]|0;
              $244 = HEAP32[$addrTop>>2]|0;
              (_sqlite3VdbeAddOp3($242,15,$243,0,$244)|0);
              $245 = HEAP32[$regYield>>2]|0;
              _sqlite3SelectDestInit($dest,13,$245);
              $246 = HEAP8[$bIdListInOrder>>0]|0;
              $247 = $246&255;
              $248 = ($247|0)!=(0);
              $249 = HEAP32[$regData>>2]|0;
              $250 = $248 ? $249 : 0;
              $251 = ((($dest)) + 12|0);
              HEAP32[$251>>2] = $250;
              $252 = HEAP32[$pTab>>2]|0;
              $253 = ((($252)) + 34|0);
              $254 = HEAP16[$253>>1]|0;
              $255 = $254 << 16 >> 16;
              $256 = ((($dest)) + 16|0);
              HEAP32[$256>>2] = $255;
              $257 = HEAP32[$0>>2]|0;
              $258 = HEAP32[$2>>2]|0;
              $259 = (_sqlite3Select($257,$258,$dest)|0);
              HEAP32[$rc>>2] = $259;
              $260 = ((($dest)) + 12|0);
              $261 = HEAP32[$260>>2]|0;
              HEAP32[$regFromSelect>>2] = $261;
              $262 = HEAP32[$rc>>2]|0;
              $263 = ($262|0)!=(0);
              if ($263) {
               break L1;
              }
              $264 = HEAP32[$db>>2]|0;
              $265 = ((($264)) + 69|0);
              $266 = HEAP8[$265>>0]|0;
              $267 = $266&255;
              $268 = ($267|0)!=(0);
              if ($268) {
               break L1;
              }
              $269 = HEAP32[$0>>2]|0;
              $270 = ((($269)) + 36|0);
              $271 = HEAP32[$270>>2]|0;
              $272 = ($271|0)!=(0);
              if ($272) {
               break L1;
              }
              $273 = HEAP32[$v>>2]|0;
              $274 = HEAP32[$regYield>>2]|0;
              _sqlite3VdbeEndCoroutine($273,$274);
              $275 = HEAP32[$v>>2]|0;
              $276 = HEAP32[$addrTop>>2]|0;
              $277 = (($276) - 1)|0;
              _sqlite3VdbeJumpHere($275,$277);
              $278 = HEAP32[$2>>2]|0;
              $279 = HEAP32[$278>>2]|0;
              $280 = HEAP32[$279>>2]|0;
              HEAP32[$nColumn>>2] = $280;
              $281 = HEAP32[$pTrigger>>2]|0;
              $282 = ($281|0)!=(0|0);
              if ($282) {
               label = 44;
              } else {
               $283 = HEAP32[$0>>2]|0;
               $284 = HEAP32[$iDb>>2]|0;
               $285 = HEAP32[$pTab>>2]|0;
               $286 = (_readsTable($283,$284,$285)|0);
               $287 = ($286|0)!=(0);
               if ($287) {
                label = 44;
               }
              }
              if ((label|0) == 44) {
               HEAP8[$useTempTable>>0] = 1;
              }
              $288 = HEAP8[$useTempTable>>0]|0;
              $289 = ($288<<24>>24)!=(0);
              if (!($289)) {
               break;
              }
              $290 = HEAP32[$0>>2]|0;
              $291 = ((($290)) + 40|0);
              $292 = HEAP32[$291>>2]|0;
              $293 = (($292) + 1)|0;
              HEAP32[$291>>2] = $293;
              HEAP32[$srcTab>>2] = $292;
              $294 = HEAP32[$0>>2]|0;
              $295 = (_sqlite3GetTempReg($294)|0);
              HEAP32[$regRec>>2] = $295;
              $296 = HEAP32[$0>>2]|0;
              $297 = (_sqlite3GetTempReg($296)|0);
              HEAP32[$regTempRowid>>2] = $297;
              $298 = HEAP32[$v>>2]|0;
              $299 = HEAP32[$srcTab>>2]|0;
              $300 = HEAP32[$nColumn>>2]|0;
              (_sqlite3VdbeAddOp2($298,107,$299,$300)|0);
              $301 = HEAP32[$v>>2]|0;
              $302 = ((($dest)) + 8|0);
              $303 = HEAP32[$302>>2]|0;
              $304 = (_sqlite3VdbeAddOp1($301,16,$303)|0);
              HEAP32[$addrL>>2] = $304;
              $305 = HEAP32[$v>>2]|0;
              $306 = HEAP32[$regFromSelect>>2]|0;
              $307 = HEAP32[$nColumn>>2]|0;
              $308 = HEAP32[$regRec>>2]|0;
              (_sqlite3VdbeAddOp3($305,99,$306,$307,$308)|0);
              $309 = HEAP32[$v>>2]|0;
              $310 = HEAP32[$srcTab>>2]|0;
              $311 = HEAP32[$regTempRowid>>2]|0;
              (_sqlite3VdbeAddOp2($309,114,$310,$311)|0);
              $312 = HEAP32[$v>>2]|0;
              $313 = HEAP32[$srcTab>>2]|0;
              $314 = HEAP32[$regRec>>2]|0;
              $315 = HEAP32[$regTempRowid>>2]|0;
              (_sqlite3VdbeAddOp3($312,115,$313,$314,$315)|0);
              $316 = HEAP32[$v>>2]|0;
              $317 = HEAP32[$addrL>>2]|0;
              (_sqlite3VdbeGoto($316,$317)|0);
              $318 = HEAP32[$v>>2]|0;
              $319 = HEAP32[$addrL>>2]|0;
              _sqlite3VdbeJumpHere($318,$319);
              $320 = HEAP32[$0>>2]|0;
              $321 = HEAP32[$regRec>>2]|0;
              _sqlite3ReleaseTempReg($320,$321);
              $322 = HEAP32[$0>>2]|0;
              $323 = HEAP32[$regTempRowid>>2]|0;
              _sqlite3ReleaseTempReg($322,$323);
             } else {
              ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
              $324 = HEAP32[$0>>2]|0;
              HEAP32[$sNC>>2] = $324;
              HEAP32[$srcTab>>2] = -1;
              $325 = HEAP32[$pList>>2]|0;
              $326 = ($325|0)!=(0|0);
              if ($326) {
               $327 = HEAP32[$pList>>2]|0;
               $328 = HEAP32[$327>>2]|0;
               HEAP32[$nColumn>>2] = $328;
               $329 = HEAP32[$pList>>2]|0;
               $330 = (_sqlite3ResolveExprListNames($sNC,$329)|0);
               $331 = ($330|0)!=(0);
               if ($331) {
                break L1;
               } else {
                break;
               }
              } else {
               HEAP32[$nColumn>>2] = 0;
               break;
              }
             }
            } while(0);
            $332 = HEAP32[$3>>2]|0;
            $333 = ($332|0)==(0|0);
            $334 = HEAP32[$nColumn>>2]|0;
            $335 = ($334|0)>(0);
            $or$cond3 = $333 & $335;
            if ($or$cond3) {
             $336 = HEAP32[$pTab>>2]|0;
             $337 = ((($336)) + 32|0);
             $338 = HEAP16[$337>>1]|0;
             $339 = $338 << 16 >> 16;
             HEAP32[$ipkColumn>>2] = $339;
            }
            HEAP32[$i>>2] = 0;
            while(1) {
             $340 = HEAP32[$i>>2]|0;
             $341 = HEAP32[$pTab>>2]|0;
             $342 = ((($341)) + 34|0);
             $343 = HEAP16[$342>>1]|0;
             $344 = $343 << 16 >> 16;
             $345 = ($340|0)<($344|0);
             if (!($345)) {
              break;
             }
             $346 = HEAP32[$i>>2]|0;
             $347 = HEAP32[$pTab>>2]|0;
             $348 = ((($347)) + 4|0);
             $349 = HEAP32[$348>>2]|0;
             $350 = (($349) + ($346<<4)|0);
             $351 = ((($350)) + 15|0);
             $352 = HEAP8[$351>>0]|0;
             $353 = $352&255;
             $354 = $353 & 2;
             $355 = ($354|0)!=(0);
             $356 = $355 ? 1 : 0;
             $357 = HEAP32[$nHidden>>2]|0;
             $358 = (($357) + ($356))|0;
             HEAP32[$nHidden>>2] = $358;
             $359 = HEAP32[$i>>2]|0;
             $360 = (($359) + 1)|0;
             HEAP32[$i>>2] = $360;
            }
            $361 = HEAP32[$3>>2]|0;
            $362 = ($361|0)==(0|0);
            $363 = HEAP32[$nColumn>>2]|0;
            $364 = ($363|0)!=(0);
            $or$cond5 = $362 & $364;
            if ($or$cond5) {
             $365 = HEAP32[$nColumn>>2]|0;
             $366 = HEAP32[$pTab>>2]|0;
             $367 = ((($366)) + 34|0);
             $368 = HEAP16[$367>>1]|0;
             $369 = $368 << 16 >> 16;
             $370 = HEAP32[$nHidden>>2]|0;
             $371 = (($369) - ($370))|0;
             $372 = ($365|0)!=($371|0);
             if ($372) {
              $373 = HEAP32[$0>>2]|0;
              $374 = HEAP32[$1>>2]|0;
              $375 = HEAP32[$pTab>>2]|0;
              $376 = ((($375)) + 34|0);
              $377 = HEAP16[$376>>1]|0;
              $378 = $377 << 16 >> 16;
              $379 = HEAP32[$nHidden>>2]|0;
              $380 = (($378) - ($379))|0;
              $381 = HEAP32[$nColumn>>2]|0;
              HEAP32[$vararg_buffer14>>2] = $374;
              $vararg_ptr17 = ((($vararg_buffer14)) + 4|0);
              HEAP32[$vararg_ptr17>>2] = 0;
              $vararg_ptr18 = ((($vararg_buffer14)) + 8|0);
              HEAP32[$vararg_ptr18>>2] = $380;
              $vararg_ptr19 = ((($vararg_buffer14)) + 12|0);
              HEAP32[$vararg_ptr19>>2] = $381;
              _sqlite3ErrorMsg($373,34293,$vararg_buffer14);
              break L1;
             }
            }
            $382 = HEAP32[$3>>2]|0;
            $383 = ($382|0)!=(0|0);
            do {
             if ($383) {
              $384 = HEAP32[$nColumn>>2]|0;
              $385 = HEAP32[$3>>2]|0;
              $386 = ((($385)) + 4|0);
              $387 = HEAP32[$386>>2]|0;
              $388 = ($384|0)!=($387|0);
              if (!($388)) {
               break;
              }
              $389 = HEAP32[$0>>2]|0;
              $390 = HEAP32[$nColumn>>2]|0;
              $391 = HEAP32[$3>>2]|0;
              $392 = ((($391)) + 4|0);
              $393 = HEAP32[$392>>2]|0;
              HEAP32[$vararg_buffer20>>2] = $390;
              $vararg_ptr23 = ((($vararg_buffer20)) + 4|0);
              HEAP32[$vararg_ptr23>>2] = $393;
              _sqlite3ErrorMsg($389,34345,$vararg_buffer20);
              break L1;
             }
            } while(0);
            $394 = HEAP32[$db>>2]|0;
            $395 = ((($394)) + 24|0);
            $396 = HEAP32[$395>>2]|0;
            $397 = $396 & 128;
            $398 = ($397|0)!=(0);
            if ($398) {
             $399 = HEAP32[$0>>2]|0;
             $400 = ((($399)) + 44|0);
             $401 = HEAP32[$400>>2]|0;
             $402 = (($401) + 1)|0;
             HEAP32[$400>>2] = $402;
             HEAP32[$regRowCount>>2] = $402;
             $403 = HEAP32[$v>>2]|0;
             $404 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($403,76,0,$404)|0);
            }
            $405 = HEAP32[$isView>>2]|0;
            $406 = ($405|0)!=(0);
            L85: do {
             if (!($406)) {
              $407 = HEAP32[$0>>2]|0;
              $408 = HEAP32[$pTab>>2]|0;
              $409 = (_sqlite3OpenTableAndIndices($407,$408,105,0,-1,0,$iDataCur,$iIdxCur)|0);
              HEAP32[$nIdx>>2] = $409;
              $410 = HEAP32[$db>>2]|0;
              $411 = HEAP32[$nIdx>>2]|0;
              $412 = (($411) + 1)|0;
              $413 = $412<<2;
              $414 = (_sqlite3DbMallocRawNN($410,$413,0)|0);
              HEAP32[$aRegIdx>>2] = $414;
              $415 = HEAP32[$aRegIdx>>2]|0;
              $416 = ($415|0)==(0|0);
              if ($416) {
               break L1;
              }
              HEAP32[$i>>2] = 0;
              while(1) {
               $417 = HEAP32[$i>>2]|0;
               $418 = HEAP32[$nIdx>>2]|0;
               $419 = ($417|0)<($418|0);
               if (!($419)) {
                break L85;
               }
               $420 = HEAP32[$0>>2]|0;
               $421 = ((($420)) + 44|0);
               $422 = HEAP32[$421>>2]|0;
               $423 = (($422) + 1)|0;
               HEAP32[$421>>2] = $423;
               $424 = HEAP32[$i>>2]|0;
               $425 = HEAP32[$aRegIdx>>2]|0;
               $426 = (($425) + ($424<<2)|0);
               HEAP32[$426>>2] = $423;
               $427 = HEAP32[$i>>2]|0;
               $428 = (($427) + 1)|0;
               HEAP32[$i>>2] = $428;
              }
             }
            } while(0);
            $429 = HEAP8[$useTempTable>>0]|0;
            $430 = ($429<<24>>24)!=(0);
            do {
             if ($430) {
              $431 = HEAP32[$v>>2]|0;
              $432 = HEAP32[$srcTab>>2]|0;
              $433 = (_sqlite3VdbeAddOp1($431,57,$432)|0);
              HEAP32[$addrInsTop>>2] = $433;
              $434 = HEAP32[$v>>2]|0;
              $435 = (_sqlite3VdbeCurrentAddr($434)|0);
              HEAP32[$addrCont>>2] = $435;
             } else {
              $436 = HEAP32[$2>>2]|0;
              $437 = ($436|0)!=(0|0);
              if (!($437)) {
               break;
              }
              $438 = HEAP32[$v>>2]|0;
              $439 = ((($dest)) + 8|0);
              $440 = HEAP32[$439>>2]|0;
              $441 = (_sqlite3VdbeAddOp1($438,16,$440)|0);
              HEAP32[$addrCont>>2] = $441;
              HEAP32[$addrInsTop>>2] = $441;
             }
            } while(0);
            $442 = HEAP32[$v>>2]|0;
            $443 = (_sqlite3VdbeMakeLabel($442)|0);
            HEAP32[$endOfLoop>>2] = $443;
            $444 = HEAP32[$tmask>>2]|0;
            $445 = $444 & 1;
            $446 = ($445|0)!=(0);
            if ($446) {
             $447 = HEAP32[$0>>2]|0;
             $448 = HEAP32[$pTab>>2]|0;
             $449 = ((($448)) + 34|0);
             $450 = HEAP16[$449>>1]|0;
             $451 = $450 << 16 >> 16;
             $452 = (($451) + 1)|0;
             $453 = (_sqlite3GetTempRange($447,$452)|0);
             HEAP32[$regCols>>2] = $453;
             $454 = HEAP32[$ipkColumn>>2]|0;
             $455 = ($454|0)<(0);
             if ($455) {
              $456 = HEAP32[$v>>2]|0;
              $457 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp2($456,76,-1,$457)|0);
             } else {
              $458 = HEAP8[$useTempTable>>0]|0;
              $459 = ($458<<24>>24)!=(0);
              if ($459) {
               $460 = HEAP32[$v>>2]|0;
               $461 = HEAP32[$srcTab>>2]|0;
               $462 = HEAP32[$ipkColumn>>2]|0;
               $463 = HEAP32[$regCols>>2]|0;
               (_sqlite3VdbeAddOp3($460,96,$461,$462,$463)|0);
              } else {
               $464 = HEAP32[$0>>2]|0;
               $465 = HEAP32[$ipkColumn>>2]|0;
               $466 = HEAP32[$pList>>2]|0;
               $467 = ((($466)) + 4|0);
               $468 = HEAP32[$467>>2]|0;
               $469 = (($468) + (($465*20)|0)|0);
               $470 = HEAP32[$469>>2]|0;
               $471 = HEAP32[$regCols>>2]|0;
               _sqlite3ExprCode($464,$470,$471);
              }
              $472 = HEAP32[$v>>2]|0;
              $473 = HEAP32[$regCols>>2]|0;
              $474 = (_sqlite3VdbeAddOp1($472,35,$473)|0);
              HEAP32[$addr1>>2] = $474;
              $475 = HEAP32[$v>>2]|0;
              $476 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp2($475,76,-1,$476)|0);
              $477 = HEAP32[$v>>2]|0;
              $478 = HEAP32[$addr1>>2]|0;
              _sqlite3VdbeJumpHere($477,$478);
              $479 = HEAP32[$v>>2]|0;
              $480 = HEAP32[$regCols>>2]|0;
              (_sqlite3VdbeAddOp1($479,17,$480)|0);
             }
             HEAP32[$j>>2] = 0;
             HEAP32[$i>>2] = 0;
             while(1) {
              $481 = HEAP32[$i>>2]|0;
              $482 = HEAP32[$pTab>>2]|0;
              $483 = ((($482)) + 34|0);
              $484 = HEAP16[$483>>1]|0;
              $485 = $484 << 16 >> 16;
              $486 = ($481|0)<($485|0);
              if (!($486)) {
               break;
              }
              $487 = HEAP32[$3>>2]|0;
              $488 = ($487|0)!=(0|0);
              L110: do {
               if ($488) {
                HEAP32[$j>>2] = 0;
                while(1) {
                 $489 = HEAP32[$j>>2]|0;
                 $490 = HEAP32[$3>>2]|0;
                 $491 = ((($490)) + 4|0);
                 $492 = HEAP32[$491>>2]|0;
                 $493 = ($489|0)<($492|0);
                 if (!($493)) {
                  break L110;
                 }
                 $494 = HEAP32[$j>>2]|0;
                 $495 = HEAP32[$3>>2]|0;
                 $496 = HEAP32[$495>>2]|0;
                 $497 = (($496) + ($494<<3)|0);
                 $498 = ((($497)) + 4|0);
                 $499 = HEAP32[$498>>2]|0;
                 $500 = HEAP32[$i>>2]|0;
                 $501 = ($499|0)==($500|0);
                 if ($501) {
                  break L110;
                 }
                 $502 = HEAP32[$j>>2]|0;
                 $503 = (($502) + 1)|0;
                 HEAP32[$j>>2] = $503;
                }
               }
              } while(0);
              $504 = HEAP8[$useTempTable>>0]|0;
              $505 = ($504<<24>>24)!=(0);
              $506 = HEAP32[$pList>>2]|0;
              $507 = ($506|0)!=(0|0);
              $or$cond7 = $505 | $507;
              do {
               if ($or$cond7) {
                $508 = HEAP32[$3>>2]|0;
                $509 = ($508|0)!=(0|0);
                if ($509) {
                 $510 = HEAP32[$j>>2]|0;
                 $511 = HEAP32[$3>>2]|0;
                 $512 = ((($511)) + 4|0);
                 $513 = HEAP32[$512>>2]|0;
                 $514 = ($510|0)>=($513|0);
                 if ($514) {
                  label = 90;
                  break;
                 }
                }
                $515 = HEAP8[$useTempTable>>0]|0;
                $516 = ($515<<24>>24)!=(0);
                if ($516) {
                 $529 = HEAP32[$v>>2]|0;
                 $530 = HEAP32[$srcTab>>2]|0;
                 $531 = HEAP32[$j>>2]|0;
                 $532 = HEAP32[$regCols>>2]|0;
                 $533 = HEAP32[$i>>2]|0;
                 $534 = (($532) + ($533))|0;
                 $535 = (($534) + 1)|0;
                 (_sqlite3VdbeAddOp3($529,96,$530,$531,$535)|0);
                 break;
                } else {
                 $536 = HEAP32[$0>>2]|0;
                 $537 = HEAP32[$j>>2]|0;
                 $538 = HEAP32[$pList>>2]|0;
                 $539 = ((($538)) + 4|0);
                 $540 = HEAP32[$539>>2]|0;
                 $541 = (($540) + (($537*20)|0)|0);
                 $542 = HEAP32[$541>>2]|0;
                 $543 = HEAP32[$regCols>>2]|0;
                 $544 = HEAP32[$i>>2]|0;
                 $545 = (($543) + ($544))|0;
                 $546 = (($545) + 1)|0;
                 _sqlite3ExprCodeAndCache($536,$542,$546);
                 break;
                }
               } else {
                label = 90;
               }
              } while(0);
              if ((label|0) == 90) {
               label = 0;
               $517 = HEAP32[$0>>2]|0;
               $518 = HEAP32[$i>>2]|0;
               $519 = HEAP32[$pTab>>2]|0;
               $520 = ((($519)) + 4|0);
               $521 = HEAP32[$520>>2]|0;
               $522 = (($521) + ($518<<4)|0);
               $523 = ((($522)) + 4|0);
               $524 = HEAP32[$523>>2]|0;
               $525 = HEAP32[$regCols>>2]|0;
               $526 = HEAP32[$i>>2]|0;
               $527 = (($525) + ($526))|0;
               $528 = (($527) + 1)|0;
               _sqlite3ExprCode($517,$524,$528);
              }
              $547 = HEAP32[$3>>2]|0;
              $548 = ($547|0)==(0|0);
              if ($548) {
               $549 = HEAP32[$j>>2]|0;
               $550 = (($549) + 1)|0;
               HEAP32[$j>>2] = $550;
              }
              $551 = HEAP32[$i>>2]|0;
              $552 = (($551) + 1)|0;
              HEAP32[$i>>2] = $552;
             }
             $553 = HEAP32[$isView>>2]|0;
             $554 = ($553|0)!=(0);
             if (!($554)) {
              $555 = HEAP32[$v>>2]|0;
              $556 = HEAP32[$pTab>>2]|0;
              $557 = HEAP32[$regCols>>2]|0;
              $558 = (($557) + 1)|0;
              _sqlite3TableAffinity($555,$556,$558);
             }
             $559 = HEAP32[$0>>2]|0;
             $560 = HEAP32[$pTrigger>>2]|0;
             $561 = HEAP32[$pTab>>2]|0;
             $562 = HEAP32[$regCols>>2]|0;
             $563 = HEAP32[$pTab>>2]|0;
             $564 = ((($563)) + 34|0);
             $565 = HEAP16[$564>>1]|0;
             $566 = $565 << 16 >> 16;
             $567 = (($562) - ($566))|0;
             $568 = (($567) - 1)|0;
             $569 = HEAP32[$4>>2]|0;
             $570 = HEAP32[$endOfLoop>>2]|0;
             _sqlite3CodeRowTrigger($559,$560,108,0,1,$561,$568,$569,$570);
             $571 = HEAP32[$0>>2]|0;
             $572 = HEAP32[$regCols>>2]|0;
             $573 = HEAP32[$pTab>>2]|0;
             $574 = ((($573)) + 34|0);
             $575 = HEAP16[$574>>1]|0;
             $576 = $575 << 16 >> 16;
             $577 = (($576) + 1)|0;
             _sqlite3ReleaseTempRange($571,$572,$577);
            }
            $578 = HEAP32[$isView>>2]|0;
            $579 = ($578|0)!=(0);
            do {
             if (!($579)) {
              $580 = HEAP32[$pTab>>2]|0;
              $581 = ((($580)) + 42|0);
              $582 = HEAP8[$581>>0]|0;
              $583 = $582&255;
              $584 = $583 & 16;
              $585 = ($584|0)!=(0);
              if ($585) {
               $586 = HEAP32[$v>>2]|0;
               $587 = HEAP32[$regIns>>2]|0;
               (_sqlite3VdbeAddOp2($586,79,0,$587)|0);
              }
              $588 = HEAP32[$ipkColumn>>2]|0;
              $589 = ($588|0)>=(0);
              L141: do {
               if ($589) {
                $590 = HEAP8[$useTempTable>>0]|0;
                $591 = ($590<<24>>24)!=(0);
                do {
                 if ($591) {
                  $592 = HEAP32[$v>>2]|0;
                  $593 = HEAP32[$srcTab>>2]|0;
                  $594 = HEAP32[$ipkColumn>>2]|0;
                  $595 = HEAP32[$regRowid>>2]|0;
                  (_sqlite3VdbeAddOp3($592,96,$593,$594,$595)|0);
                 } else {
                  $596 = HEAP32[$2>>2]|0;
                  $597 = ($596|0)!=(0|0);
                  if ($597) {
                   $598 = HEAP32[$v>>2]|0;
                   $599 = HEAP32[$regFromSelect>>2]|0;
                   $600 = HEAP32[$ipkColumn>>2]|0;
                   $601 = (($599) + ($600))|0;
                   $602 = HEAP32[$regRowid>>2]|0;
                   (_sqlite3VdbeAddOp2($598,84,$601,$602)|0);
                   break;
                  }
                  $603 = HEAP32[$0>>2]|0;
                  $604 = HEAP32[$ipkColumn>>2]|0;
                  $605 = HEAP32[$pList>>2]|0;
                  $606 = ((($605)) + 4|0);
                  $607 = HEAP32[$606>>2]|0;
                  $608 = (($607) + (($604*20)|0)|0);
                  $609 = HEAP32[$608>>2]|0;
                  $610 = HEAP32[$regRowid>>2]|0;
                  _sqlite3ExprCode($603,$609,$610);
                  $611 = HEAP32[$v>>2]|0;
                  $612 = (_sqlite3VdbeGetOp($611,-1)|0);
                  HEAP32[$pOp>>2] = $612;
                  $613 = HEAP32[$pOp>>2]|0;
                  $614 = ($613|0)!=(0|0);
                  if (!($614)) {
                   break;
                  }
                  $615 = HEAP32[$pOp>>2]|0;
                  $616 = HEAP8[$615>>0]|0;
                  $617 = $616&255;
                  $618 = ($617|0)==(79);
                  if (!($618)) {
                   break;
                  }
                  $619 = HEAP32[$pTab>>2]|0;
                  $620 = ((($619)) + 42|0);
                  $621 = HEAP8[$620>>0]|0;
                  $622 = $621&255;
                  $623 = $622 & 16;
                  $624 = ($623|0)!=(0);
                  if ($624) {
                   break;
                  }
                  HEAP8[$appendFlag>>0] = 1;
                  $625 = HEAP32[$pOp>>2]|0;
                  HEAP8[$625>>0] = 114;
                  $626 = HEAP32[$iDataCur>>2]|0;
                  $627 = HEAP32[$pOp>>2]|0;
                  $628 = ((($627)) + 4|0);
                  HEAP32[$628>>2] = $626;
                  $629 = HEAP32[$regRowid>>2]|0;
                  $630 = HEAP32[$pOp>>2]|0;
                  $631 = ((($630)) + 8|0);
                  HEAP32[$631>>2] = $629;
                  $632 = HEAP32[$regAutoinc>>2]|0;
                  $633 = HEAP32[$pOp>>2]|0;
                  $634 = ((($633)) + 12|0);
                  HEAP32[$634>>2] = $632;
                 }
                } while(0);
                $635 = HEAP8[$appendFlag>>0]|0;
                $636 = ($635<<24>>24)!=(0);
                if ($636) {
                 break;
                }
                $637 = HEAP32[$pTab>>2]|0;
                $638 = ((($637)) + 42|0);
                $639 = HEAP8[$638>>0]|0;
                $640 = $639&255;
                $641 = $640 & 16;
                $642 = ($641|0)!=(0);
                $643 = HEAP32[$v>>2]|0;
                if ($642) {
                 $652 = (_sqlite3VdbeCurrentAddr($643)|0);
                 HEAP32[$addr11>>2] = $652;
                 $653 = HEAP32[$v>>2]|0;
                 $654 = HEAP32[$regRowid>>2]|0;
                 $655 = HEAP32[$addr11>>2]|0;
                 $656 = (($655) + 2)|0;
                 (_sqlite3VdbeAddOp2($653,34,$654,$656)|0);
                } else {
                 $644 = HEAP32[$regRowid>>2]|0;
                 $645 = (_sqlite3VdbeAddOp1($643,35,$644)|0);
                 HEAP32[$addr11>>2] = $645;
                 $646 = HEAP32[$v>>2]|0;
                 $647 = HEAP32[$iDataCur>>2]|0;
                 $648 = HEAP32[$regRowid>>2]|0;
                 $649 = HEAP32[$regAutoinc>>2]|0;
                 (_sqlite3VdbeAddOp3($646,114,$647,$648,$649)|0);
                 $650 = HEAP32[$v>>2]|0;
                 $651 = HEAP32[$addr11>>2]|0;
                 _sqlite3VdbeJumpHere($650,$651);
                }
                $657 = HEAP32[$v>>2]|0;
                $658 = HEAP32[$regRowid>>2]|0;
                (_sqlite3VdbeAddOp1($657,17,$658)|0);
               } else {
                $659 = HEAP32[$pTab>>2]|0;
                $660 = ((($659)) + 42|0);
                $661 = HEAP8[$660>>0]|0;
                $662 = $661&255;
                $663 = $662 & 16;
                $664 = ($663|0)!=(0);
                do {
                 if (!($664)) {
                  $665 = HEAP8[$withoutRowid>>0]|0;
                  $666 = $665&255;
                  $667 = ($666|0)!=(0);
                  if ($667) {
                   break;
                  }
                  $670 = HEAP32[$v>>2]|0;
                  $671 = HEAP32[$iDataCur>>2]|0;
                  $672 = HEAP32[$regRowid>>2]|0;
                  $673 = HEAP32[$regAutoinc>>2]|0;
                  (_sqlite3VdbeAddOp3($670,114,$671,$672,$673)|0);
                  HEAP8[$appendFlag>>0] = 1;
                  break L141;
                 }
                } while(0);
                $668 = HEAP32[$v>>2]|0;
                $669 = HEAP32[$regRowid>>2]|0;
                (_sqlite3VdbeAddOp2($668,79,0,$669)|0);
               }
              } while(0);
              $674 = HEAP32[$0>>2]|0;
              $675 = HEAP32[$regAutoinc>>2]|0;
              $676 = HEAP32[$regRowid>>2]|0;
              _autoIncStep($674,$675,$676);
              HEAP32[$nHidden>>2] = 0;
              HEAP32[$i>>2] = 0;
              while(1) {
               $677 = HEAP32[$i>>2]|0;
               $678 = HEAP32[$pTab>>2]|0;
               $679 = ((($678)) + 34|0);
               $680 = HEAP16[$679>>1]|0;
               $681 = $680 << 16 >> 16;
               $682 = ($677|0)<($681|0);
               if (!($682)) {
                break;
               }
               $683 = HEAP32[$regRowid>>2]|0;
               $684 = (($683) + 1)|0;
               $685 = HEAP32[$i>>2]|0;
               $686 = (($684) + ($685))|0;
               HEAP32[$iRegStore>>2] = $686;
               $687 = HEAP32[$i>>2]|0;
               $688 = HEAP32[$pTab>>2]|0;
               $689 = ((($688)) + 32|0);
               $690 = HEAP16[$689>>1]|0;
               $691 = $690 << 16 >> 16;
               $692 = ($687|0)==($691|0);
               L167: do {
                if ($692) {
                 $693 = HEAP32[$v>>2]|0;
                 $694 = HEAP32[$iRegStore>>2]|0;
                 (_sqlite3VdbeAddOp1($693,80,$694)|0);
                } else {
                 $695 = HEAP32[$3>>2]|0;
                 $696 = ($695|0)==(0|0);
                 L170: do {
                  if ($696) {
                   $697 = HEAP32[$i>>2]|0;
                   $698 = HEAP32[$pTab>>2]|0;
                   $699 = ((($698)) + 4|0);
                   $700 = HEAP32[$699>>2]|0;
                   $701 = (($700) + ($697<<4)|0);
                   $702 = ((($701)) + 15|0);
                   $703 = HEAP8[$702>>0]|0;
                   $704 = $703&255;
                   $705 = $704 & 2;
                   $706 = ($705|0)!=(0);
                   if ($706) {
                    HEAP32[$j>>2] = -1;
                    $707 = HEAP32[$nHidden>>2]|0;
                    $708 = (($707) + 1)|0;
                    HEAP32[$nHidden>>2] = $708;
                    break;
                   } else {
                    $709 = HEAP32[$i>>2]|0;
                    $710 = HEAP32[$nHidden>>2]|0;
                    $711 = (($709) - ($710))|0;
                    HEAP32[$j>>2] = $711;
                    break;
                   }
                  } else {
                   HEAP32[$j>>2] = 0;
                   while(1) {
                    $712 = HEAP32[$j>>2]|0;
                    $713 = HEAP32[$3>>2]|0;
                    $714 = ((($713)) + 4|0);
                    $715 = HEAP32[$714>>2]|0;
                    $716 = ($712|0)<($715|0);
                    if (!($716)) {
                     break L170;
                    }
                    $717 = HEAP32[$j>>2]|0;
                    $718 = HEAP32[$3>>2]|0;
                    $719 = HEAP32[$718>>2]|0;
                    $720 = (($719) + ($717<<3)|0);
                    $721 = ((($720)) + 4|0);
                    $722 = HEAP32[$721>>2]|0;
                    $723 = HEAP32[$i>>2]|0;
                    $724 = ($722|0)==($723|0);
                    if ($724) {
                     break L170;
                    }
                    $725 = HEAP32[$j>>2]|0;
                    $726 = (($725) + 1)|0;
                    HEAP32[$j>>2] = $726;
                   }
                  }
                 } while(0);
                 $727 = HEAP32[$j>>2]|0;
                 $728 = ($727|0)<(0);
                 $729 = HEAP32[$nColumn>>2]|0;
                 $730 = ($729|0)==(0);
                 $or$cond9 = $728 | $730;
                 do {
                  if (!($or$cond9)) {
                   $731 = HEAP32[$3>>2]|0;
                   $732 = ($731|0)!=(0|0);
                   if ($732) {
                    $733 = HEAP32[$j>>2]|0;
                    $734 = HEAP32[$3>>2]|0;
                    $735 = ((($734)) + 4|0);
                    $736 = HEAP32[$735>>2]|0;
                    $737 = ($733|0)>=($736|0);
                    if ($737) {
                     break;
                    }
                   }
                   $747 = HEAP8[$useTempTable>>0]|0;
                   $748 = ($747<<24>>24)!=(0);
                   if ($748) {
                    $749 = HEAP32[$v>>2]|0;
                    $750 = HEAP32[$srcTab>>2]|0;
                    $751 = HEAP32[$j>>2]|0;
                    $752 = HEAP32[$iRegStore>>2]|0;
                    (_sqlite3VdbeAddOp3($749,96,$750,$751,$752)|0);
                    break L167;
                   }
                   $753 = HEAP32[$2>>2]|0;
                   $754 = ($753|0)!=(0|0);
                   if (!($754)) {
                    $763 = HEAP32[$0>>2]|0;
                    $764 = HEAP32[$j>>2]|0;
                    $765 = HEAP32[$pList>>2]|0;
                    $766 = ((($765)) + 4|0);
                    $767 = HEAP32[$766>>2]|0;
                    $768 = (($767) + (($764*20)|0)|0);
                    $769 = HEAP32[$768>>2]|0;
                    $770 = HEAP32[$iRegStore>>2]|0;
                    _sqlite3ExprCode($763,$769,$770);
                    break L167;
                   }
                   $755 = HEAP32[$regFromSelect>>2]|0;
                   $756 = HEAP32[$regData>>2]|0;
                   $757 = ($755|0)!=($756|0);
                   if (!($757)) {
                    break L167;
                   }
                   $758 = HEAP32[$v>>2]|0;
                   $759 = HEAP32[$regFromSelect>>2]|0;
                   $760 = HEAP32[$j>>2]|0;
                   $761 = (($759) + ($760))|0;
                   $762 = HEAP32[$iRegStore>>2]|0;
                   (_sqlite3VdbeAddOp2($758,85,$761,$762)|0);
                   break L167;
                  }
                 } while(0);
                 $738 = HEAP32[$0>>2]|0;
                 $739 = HEAP32[$i>>2]|0;
                 $740 = HEAP32[$pTab>>2]|0;
                 $741 = ((($740)) + 4|0);
                 $742 = HEAP32[$741>>2]|0;
                 $743 = (($742) + ($739<<4)|0);
                 $744 = ((($743)) + 4|0);
                 $745 = HEAP32[$744>>2]|0;
                 $746 = HEAP32[$iRegStore>>2]|0;
                 _sqlite3ExprCodeFactorable($738,$745,$746);
                }
               } while(0);
               $771 = HEAP32[$i>>2]|0;
               $772 = (($771) + 1)|0;
               HEAP32[$i>>2] = $772;
              }
              $773 = HEAP32[$pTab>>2]|0;
              $774 = ((($773)) + 42|0);
              $775 = HEAP8[$774>>0]|0;
              $776 = $775&255;
              $777 = $776 & 16;
              $778 = ($777|0)!=(0);
              if ($778) {
               $779 = HEAP32[$db>>2]|0;
               $780 = HEAP32[$pTab>>2]|0;
               $781 = (_sqlite3GetVTable($779,$780)|0);
               HEAP32[$pVTab>>2] = $781;
               $782 = HEAP32[$0>>2]|0;
               $783 = HEAP32[$pTab>>2]|0;
               _sqlite3VtabMakeWritable($782,$783);
               $784 = HEAP32[$v>>2]|0;
               $785 = HEAP32[$pTab>>2]|0;
               $786 = ((($785)) + 34|0);
               $787 = HEAP16[$786>>1]|0;
               $788 = $787 << 16 >> 16;
               $789 = (($788) + 2)|0;
               $790 = HEAP32[$regIns>>2]|0;
               $791 = HEAP32[$pVTab>>2]|0;
               (_sqlite3VdbeAddOp4($784,12,1,$789,$790,$791,-10)|0);
               $792 = HEAP32[$v>>2]|0;
               $793 = HEAP32[$4>>2]|0;
               $794 = ($793|0)==(10);
               $795 = HEAP32[$4>>2]|0;
               $796 = $794 ? 2 : $795;
               $797 = $796&255;
               _sqlite3VdbeChangeP5($792,$797);
               $798 = HEAP32[$0>>2]|0;
               _sqlite3MayAbort($798);
               break;
              } else {
               $799 = HEAP32[$0>>2]|0;
               $800 = HEAP32[$pTab>>2]|0;
               $801 = HEAP32[$aRegIdx>>2]|0;
               $802 = HEAP32[$iDataCur>>2]|0;
               $803 = HEAP32[$iIdxCur>>2]|0;
               $804 = HEAP32[$regIns>>2]|0;
               $805 = HEAP32[$ipkColumn>>2]|0;
               $806 = ($805|0)>=(0);
               $807 = $806&1;
               $808 = $807&255;
               $809 = HEAP32[$4>>2]|0;
               $810 = $809&255;
               $811 = HEAP32[$endOfLoop>>2]|0;
               _sqlite3GenerateConstraintChecks($799,$800,$801,$802,$803,$804,0,$808,$810,$811,$isReplace,0);
               $812 = HEAP32[$0>>2]|0;
               $813 = HEAP32[$pTab>>2]|0;
               $814 = HEAP32[$regIns>>2]|0;
               _sqlite3FkCheck($812,$813,0,$814,0,0);
               $815 = HEAP32[$0>>2]|0;
               $816 = HEAP32[$pTab>>2]|0;
               $817 = HEAP32[$iDataCur>>2]|0;
               $818 = HEAP32[$iIdxCur>>2]|0;
               $819 = HEAP32[$regIns>>2]|0;
               $820 = HEAP32[$aRegIdx>>2]|0;
               $821 = HEAP8[$appendFlag>>0]|0;
               $822 = $821&255;
               $823 = HEAP32[$isReplace>>2]|0;
               $824 = ($823|0)==(0);
               $825 = $824&1;
               _sqlite3CompleteInsertion($815,$816,$817,$818,$819,$820,0,$822,$825);
               break;
              }
             }
            } while(0);
            $826 = HEAP32[$db>>2]|0;
            $827 = ((($826)) + 24|0);
            $828 = HEAP32[$827>>2]|0;
            $829 = $828 & 128;
            $830 = ($829|0)!=(0);
            if ($830) {
             $831 = HEAP32[$v>>2]|0;
             $832 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($831,91,$832,1)|0);
            }
            $833 = HEAP32[$pTrigger>>2]|0;
            $834 = ($833|0)!=(0|0);
            if ($834) {
             $835 = HEAP32[$0>>2]|0;
             $836 = HEAP32[$pTrigger>>2]|0;
             $837 = HEAP32[$pTab>>2]|0;
             $838 = HEAP32[$regData>>2]|0;
             $839 = (($838) - 2)|0;
             $840 = HEAP32[$pTab>>2]|0;
             $841 = ((($840)) + 34|0);
             $842 = HEAP16[$841>>1]|0;
             $843 = $842 << 16 >> 16;
             $844 = (($839) - ($843))|0;
             $845 = HEAP32[$4>>2]|0;
             $846 = HEAP32[$endOfLoop>>2]|0;
             _sqlite3CodeRowTrigger($835,$836,108,0,2,$837,$844,$845,$846);
            }
            $847 = HEAP32[$v>>2]|0;
            $848 = HEAP32[$endOfLoop>>2]|0;
            _sqlite3VdbeResolveLabel($847,$848);
            $849 = HEAP8[$useTempTable>>0]|0;
            $850 = ($849<<24>>24)!=(0);
            do {
             if ($850) {
              $851 = HEAP32[$v>>2]|0;
              $852 = HEAP32[$srcTab>>2]|0;
              $853 = HEAP32[$addrCont>>2]|0;
              (_sqlite3VdbeAddOp2($851,7,$852,$853)|0);
              $854 = HEAP32[$v>>2]|0;
              $855 = HEAP32[$addrInsTop>>2]|0;
              _sqlite3VdbeJumpHere($854,$855);
              $856 = HEAP32[$v>>2]|0;
              $857 = HEAP32[$srcTab>>2]|0;
              (_sqlite3VdbeAddOp1($856,111,$857)|0);
             } else {
              $858 = HEAP32[$2>>2]|0;
              $859 = ($858|0)!=(0|0);
              if (!($859)) {
               break;
              }
              $860 = HEAP32[$v>>2]|0;
              $861 = HEAP32[$addrCont>>2]|0;
              (_sqlite3VdbeGoto($860,$861)|0);
              $862 = HEAP32[$v>>2]|0;
              $863 = HEAP32[$addrInsTop>>2]|0;
              _sqlite3VdbeJumpHere($862,$863);
             }
            } while(0);
            $864 = HEAP32[$pTab>>2]|0;
            $865 = ((($864)) + 42|0);
            $866 = HEAP8[$865>>0]|0;
            $867 = $866&255;
            $868 = $867 & 16;
            $869 = ($868|0)!=(0);
            $870 = HEAP32[$isView>>2]|0;
            $871 = ($870|0)!=(0);
            $or$cond11 = $869 | $871;
            if ($or$cond11) {
             break;
            }
            $872 = HEAP32[$iDataCur>>2]|0;
            $873 = HEAP32[$iIdxCur>>2]|0;
            $874 = ($872|0)<($873|0);
            if ($874) {
             $875 = HEAP32[$v>>2]|0;
             $876 = HEAP32[$iDataCur>>2]|0;
             (_sqlite3VdbeAddOp1($875,111,$876)|0);
            }
            HEAP32[$idx>>2] = 0;
            $877 = HEAP32[$pTab>>2]|0;
            $878 = ((($877)) + 8|0);
            $879 = HEAP32[$878>>2]|0;
            HEAP32[$pIdx>>2] = $879;
            while(1) {
             $880 = HEAP32[$pIdx>>2]|0;
             $881 = ($880|0)!=(0|0);
             if (!($881)) {
              break L20;
             }
             $882 = HEAP32[$v>>2]|0;
             $883 = HEAP32[$idx>>2]|0;
             $884 = HEAP32[$iIdxCur>>2]|0;
             $885 = (($883) + ($884))|0;
             (_sqlite3VdbeAddOp1($882,111,$885)|0);
             $886 = HEAP32[$pIdx>>2]|0;
             $887 = ((($886)) + 20|0);
             $888 = HEAP32[$887>>2]|0;
             HEAP32[$pIdx>>2] = $888;
             $889 = HEAP32[$idx>>2]|0;
             $890 = (($889) + 1)|0;
             HEAP32[$idx>>2] = $890;
            }
           }
          } while(0);
          $891 = HEAP32[$0>>2]|0;
          $892 = ((($891)) + 18|0);
          $893 = HEAP8[$892>>0]|0;
          $894 = $893&255;
          $895 = ($894|0)==(0);
          if ($895) {
           $896 = HEAP32[$0>>2]|0;
           $897 = ((($896)) + 128|0);
           $898 = HEAP32[$897>>2]|0;
           $899 = ($898|0)==(0|0);
           if ($899) {
            $900 = HEAP32[$0>>2]|0;
            _sqlite3AutoincrementEnd($900);
           }
          }
          $901 = HEAP32[$db>>2]|0;
          $902 = ((($901)) + 24|0);
          $903 = HEAP32[$902>>2]|0;
          $904 = $903 & 128;
          $905 = ($904|0)!=(0);
          if ($905) {
           $906 = HEAP32[$0>>2]|0;
           $907 = ((($906)) + 18|0);
           $908 = HEAP8[$907>>0]|0;
           $909 = ($908<<24>>24)!=(0);
           if (!($909)) {
            $910 = HEAP32[$0>>2]|0;
            $911 = ((($910)) + 128|0);
            $912 = HEAP32[$911>>2]|0;
            $913 = ($912|0)!=(0|0);
            if (!($913)) {
             $914 = HEAP32[$v>>2]|0;
             $915 = HEAP32[$regRowCount>>2]|0;
             (_sqlite3VdbeAddOp2($914,87,$915,1)|0);
             $916 = HEAP32[$v>>2]|0;
             _sqlite3VdbeSetNumCols($916,1);
             $917 = HEAP32[$v>>2]|0;
             (_sqlite3VdbeSetColName($917,0,0,34370,0)|0);
            }
           }
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $918 = HEAP32[$db>>2]|0;
 $919 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($918,$919);
 $920 = HEAP32[$db>>2]|0;
 $921 = HEAP32[$pList>>2]|0;
 _sqlite3ExprListDelete($920,$921);
 $922 = HEAP32[$db>>2]|0;
 $923 = HEAP32[$2>>2]|0;
 _sqlite3SelectDelete($922,$923);
 $924 = HEAP32[$db>>2]|0;
 $925 = HEAP32[$3>>2]|0;
 _sqlite3IdListDelete($924,$925);
 $926 = HEAP32[$db>>2]|0;
 $927 = HEAP32[$aRegIdx>>2]|0;
 _sqlite3DbFree($926,$927);
 STACKTOP = sp;return;
}
function _sqlite3IdListAppend($db,$pList,$pToken) {
 $db = $db|0;
 $pList = $pList|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pToken;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = (_sqlite3DbMallocZero($6,8,0)|0);
  HEAP32[$2>>2] = $7;
  $8 = HEAP32[$2>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   HEAP32[$0>>2] = 0;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = (_sqlite3ArrayAllocate($10,$12,8,$14,$i)|0);
 $16 = HEAP32[$2>>2]|0;
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$i>>2]|0;
 $18 = ($17|0)<(0);
 $19 = HEAP32[$1>>2]|0;
 if ($18) {
  $20 = HEAP32[$2>>2]|0;
  _sqlite3IdListDelete($19,$20);
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 } else {
  $21 = HEAP32[$3>>2]|0;
  $22 = (_sqlite3NameFromToken($19,$21)|0);
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + ($23<<3)|0);
  HEAP32[$26>>2] = $22;
  $27 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $27;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _spanSet($pOut,$pStart,$pEnd) {
 $pOut = $pOut|0;
 $pStart = $pStart|0;
 $pEnd = $pEnd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pStart;
 HEAP32[$2>>2] = $pEnd;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = $4;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + ($9)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 8|0);
 HEAP32[$14>>2] = $12;
 STACKTOP = sp;return;
}
function _sqlite3ExprAlloc($db,$op,$pToken,$dequote) {
 $db = $db|0;
 $op = $op|0;
 $pToken = $pToken|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $iValue = 0, $nExtra = 0, $pNew = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $pNew = sp + 8|0;
 $nExtra = sp + 4|0;
 $iValue = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pToken;
 HEAP32[$3>>2] = $dequote;
 HEAP32[$nExtra>>2] = 0;
 HEAP32[$iValue>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)!=(0|0);
 do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ($6|0)!=(134);
   if (!($7)) {
    $8 = HEAP32[$2>>2]|0;
    $9 = HEAP32[$8>>2]|0;
    $10 = ($9|0)==(0|0);
    if (!($10)) {
     $11 = HEAP32[$2>>2]|0;
     $12 = HEAP32[$11>>2]|0;
     $13 = (_sqlite3GetInt32($12,$iValue)|0);
     $14 = ($13|0)==(0);
     if (!($14)) {
      break;
     }
    }
   }
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (($17) + 1)|0;
   HEAP32[$nExtra>>2] = $18;
  }
 } while(0);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$nExtra>>2]|0;
 $21 = (48 + ($20))|0;
 $22 = (_sqlite3DbMallocRawNN($19,$21,0)|0);
 HEAP32[$pNew>>2] = $22;
 $23 = HEAP32[$pNew>>2]|0;
 $24 = ($23|0)!=(0|0);
 if (!($24)) {
  $91 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $25 = HEAP32[$pNew>>2]|0;
 dest=$25; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $26 = HEAP32[$1>>2]|0;
 $27 = $26&255;
 $28 = HEAP32[$pNew>>2]|0;
 HEAP8[$28>>0] = $27;
 $29 = HEAP32[$pNew>>2]|0;
 $30 = ((($29)) + 34|0);
 HEAP16[$30>>1] = -1;
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0|0);
 do {
  if ($32) {
   $33 = HEAP32[$nExtra>>2]|0;
   $34 = ($33|0)==(0);
   $35 = HEAP32[$pNew>>2]|0;
   if ($34) {
    $36 = ((($35)) + 4|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = $37 | 1024;
    HEAP32[$36>>2] = $38;
    $39 = HEAP32[$iValue>>2]|0;
    $40 = HEAP32[$pNew>>2]|0;
    $41 = ((($40)) + 8|0);
    HEAP32[$41>>2] = $39;
    break;
   }
   $42 = ((($35)) + 48|0);
   $43 = HEAP32[$pNew>>2]|0;
   $44 = ((($43)) + 8|0);
   HEAP32[$44>>2] = $42;
   $45 = HEAP32[$2>>2]|0;
   $46 = ((($45)) + 4|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ($47|0)!=(0);
   if ($48) {
    $49 = HEAP32[$pNew>>2]|0;
    $50 = ((($49)) + 8|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    $54 = HEAP32[$2>>2]|0;
    $55 = ((($54)) + 4|0);
    $56 = HEAP32[$55>>2]|0;
    _memcpy(($51|0),($53|0),($56|0))|0;
   }
   $57 = HEAP32[$2>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$pNew>>2]|0;
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + ($59)|0);
   HEAP8[$63>>0] = 0;
   $64 = HEAP32[$3>>2]|0;
   $65 = ($64|0)!=(0);
   if ($65) {
    $66 = HEAP32[$pNew>>2]|0;
    $67 = ((($66)) + 8|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP8[$68>>0]|0;
    $70 = $69&255;
    $71 = (18403 + ($70)|0);
    $72 = HEAP8[$71>>0]|0;
    $73 = $72&255;
    $74 = $73 & 128;
    $75 = ($74|0)!=(0);
    if ($75) {
     $76 = HEAP32[$pNew>>2]|0;
     $77 = ((($76)) + 8|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP8[$78>>0]|0;
     $80 = $79 << 24 >> 24;
     $81 = ($80|0)==(34);
     if ($81) {
      $82 = HEAP32[$pNew>>2]|0;
      $83 = ((($82)) + 4|0);
      $84 = HEAP32[$83>>2]|0;
      $85 = $84 | 64;
      HEAP32[$83>>2] = $85;
     }
     $86 = HEAP32[$pNew>>2]|0;
     $87 = ((($86)) + 8|0);
     $88 = HEAP32[$87>>2]|0;
     _sqlite3Dequote($88);
    }
   }
  }
 } while(0);
 $89 = HEAP32[$pNew>>2]|0;
 $90 = ((($89)) + 24|0);
 HEAP32[$90>>2] = 1;
 $91 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($91|0);
}
function _sqlite3ExprAssignVarNumber($pParse,$pExpr,$n) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $bOk = 0, $db = 0, $i = 0, $i1 = 0, $or$cond = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $x = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $db = sp + 32|0;
 $z = sp + 28|0;
 $x = sp + 50|0;
 $i = sp;
 $bOk = sp + 24|0;
 $i1 = sp + 48|0;
 $a = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$z>>2] = $9;
 $10 = HEAP32[$z>>2]|0;
 $11 = ((($10)) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12 << 24 >> 24;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 400|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = (($17) + 1)<<16>>16;
  HEAP16[$16>>1] = $18;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 32|0);
  HEAP16[$20>>1] = $18;
 } else {
  $21 = HEAP32[$z>>2]|0;
  $22 = HEAP8[$21>>0]|0;
  $23 = $22 << 24 >> 24;
  $24 = ($23|0)==(63);
  do {
   if ($24) {
    $25 = HEAP32[$z>>2]|0;
    $26 = ((($25)) + 1|0);
    $27 = HEAP32[$2>>2]|0;
    $28 = (($27) - 1)|0;
    $29 = (_sqlite3Atoi64($26,$i,$28,1)|0);
    $30 = (0)==($29|0);
    $31 = $30&1;
    HEAP32[$bOk>>2] = $31;
    $32 = $i;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = $34&65535;
    HEAP16[$x>>1] = $38;
    $39 = HEAP32[$bOk>>2]|0;
    $40 = ($39|0)==(0);
    $41 = $i;
    $42 = $41;
    $43 = HEAP32[$42>>2]|0;
    $44 = (($41) + 4)|0;
    $45 = $44;
    $46 = HEAP32[$45>>2]|0;
    $47 = ($46|0)<(0);
    $48 = ($43>>>0)<(1);
    $49 = ($46|0)==(0);
    $50 = $49 & $48;
    $51 = $47 | $50;
    $or$cond = $40 | $51;
    if (!($or$cond)) {
     $52 = $i;
     $53 = $52;
     $54 = HEAP32[$53>>2]|0;
     $55 = (($52) + 4)|0;
     $56 = $55;
     $57 = HEAP32[$56>>2]|0;
     $58 = HEAP32[$db>>2]|0;
     $59 = ((($58)) + 96|0);
     $60 = ((($59)) + 36|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = ($61|0)<(0);
     $63 = $62 << 31 >> 31;
     $64 = ($57|0)>($63|0);
     $65 = ($54>>>0)>($61>>>0);
     $66 = ($57|0)==($63|0);
     $67 = $66 & $65;
     $68 = $64 | $67;
     if (!($68)) {
      $74 = $i;
      $75 = $74;
      $76 = HEAP32[$75>>2]|0;
      $77 = (($74) + 4)|0;
      $78 = $77;
      $79 = HEAP32[$78>>2]|0;
      $80 = HEAP32[$0>>2]|0;
      $81 = ((($80)) + 400|0);
      $82 = HEAP16[$81>>1]|0;
      $83 = $82 << 16 >> 16;
      $84 = ($83|0)<(0);
      $85 = $84 << 31 >> 31;
      $86 = ($79|0)>($85|0);
      $87 = ($76>>>0)>($83>>>0);
      $88 = ($79|0)==($85|0);
      $89 = $88 & $87;
      $90 = $86 | $89;
      if (!($90)) {
       break;
      }
      $91 = $i;
      $92 = $91;
      $93 = HEAP32[$92>>2]|0;
      $94 = (($91) + 4)|0;
      $95 = $94;
      $96 = HEAP32[$95>>2]|0;
      $97 = $93&65535;
      $98 = HEAP32[$0>>2]|0;
      $99 = ((($98)) + 400|0);
      HEAP16[$99>>1] = $97;
      break;
     }
    }
    $69 = HEAP32[$0>>2]|0;
    $70 = HEAP32[$db>>2]|0;
    $71 = ((($70)) + 96|0);
    $72 = ((($71)) + 36|0);
    $73 = HEAP32[$72>>2]|0;
    HEAP32[$vararg_buffer>>2] = $73;
    _sqlite3ErrorMsg($69,34195,$vararg_buffer);
    STACKTOP = sp;return;
   } else {
    HEAP16[$x>>1] = 0;
    HEAP16[$i1>>1] = 0;
    while(1) {
     $100 = HEAP16[$i1>>1]|0;
     $101 = $100 << 16 >> 16;
     $102 = HEAP32[$0>>2]|0;
     $103 = ((($102)) + 404|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = ($101|0)<($104|0);
     if (!($105)) {
      break;
     }
     $106 = HEAP16[$i1>>1]|0;
     $107 = $106 << 16 >> 16;
     $108 = HEAP32[$0>>2]|0;
     $109 = ((($108)) + 428|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = (($110) + ($107<<2)|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = ($112|0)!=(0|0);
     if ($113) {
      $114 = HEAP16[$i1>>1]|0;
      $115 = $114 << 16 >> 16;
      $116 = HEAP32[$0>>2]|0;
      $117 = ((($116)) + 428|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = (($118) + ($115<<2)|0);
      $120 = HEAP32[$119>>2]|0;
      $121 = HEAP32[$z>>2]|0;
      $122 = (_strcmp($120,$121)|0);
      $123 = ($122|0)==(0);
      if ($123) {
       label = 14;
       break;
      }
     }
     $128 = HEAP16[$i1>>1]|0;
     $129 = (($128) + 1)<<16>>16;
     HEAP16[$i1>>1] = $129;
    }
    if ((label|0) == 14) {
     $124 = HEAP16[$i1>>1]|0;
     $125 = $124 << 16 >> 16;
     $126 = (($125) + 1)|0;
     $127 = $126&65535;
     HEAP16[$x>>1] = $127;
    }
    $130 = HEAP16[$x>>1]|0;
    $131 = $130 << 16 >> 16;
    $132 = ($131|0)==(0);
    if ($132) {
     $133 = HEAP32[$0>>2]|0;
     $134 = ((($133)) + 400|0);
     $135 = HEAP16[$134>>1]|0;
     $136 = (($135) + 1)<<16>>16;
     HEAP16[$134>>1] = $136;
     HEAP16[$x>>1] = $136;
    }
   }
  } while(0);
  $137 = HEAP16[$x>>1]|0;
  $138 = HEAP32[$1>>2]|0;
  $139 = ((($138)) + 32|0);
  HEAP16[$139>>1] = $137;
  $140 = HEAP16[$x>>1]|0;
  $141 = $140 << 16 >> 16;
  $142 = HEAP32[$0>>2]|0;
  $143 = ((($142)) + 404|0);
  $144 = HEAP32[$143>>2]|0;
  $145 = ($141|0)>($144|0);
  do {
   if ($145) {
    $146 = HEAP32[$db>>2]|0;
    $147 = HEAP32[$0>>2]|0;
    $148 = ((($147)) + 428|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = HEAP16[$x>>1]|0;
    $151 = $150 << 16 >> 16;
    $152 = $151<<2;
    $153 = (_sqlite3DbRealloc($146,$149,$152,0)|0);
    HEAP32[$a>>2] = $153;
    $154 = HEAP32[$a>>2]|0;
    $155 = ($154|0)==(0|0);
    if ($155) {
     STACKTOP = sp;return;
    } else {
     $156 = HEAP32[$a>>2]|0;
     $157 = HEAP32[$0>>2]|0;
     $158 = ((($157)) + 428|0);
     HEAP32[$158>>2] = $156;
     $159 = HEAP32[$0>>2]|0;
     $160 = ((($159)) + 404|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$a>>2]|0;
     $163 = (($162) + ($161<<2)|0);
     $164 = HEAP16[$x>>1]|0;
     $165 = $164 << 16 >> 16;
     $166 = HEAP32[$0>>2]|0;
     $167 = ((($166)) + 404|0);
     $168 = HEAP32[$167>>2]|0;
     $169 = (($165) - ($168))|0;
     $170 = $169<<2;
     _memset(($163|0),0,($170|0))|0;
     $171 = HEAP16[$x>>1]|0;
     $172 = $171 << 16 >> 16;
     $173 = HEAP32[$0>>2]|0;
     $174 = ((($173)) + 404|0);
     HEAP32[$174>>2] = $172;
     break;
    }
   }
  } while(0);
  $175 = HEAP16[$x>>1]|0;
  $176 = $175 << 16 >> 16;
  $177 = (($176) - 1)|0;
  $178 = HEAP32[$0>>2]|0;
  $179 = ((($178)) + 428|0);
  $180 = HEAP32[$179>>2]|0;
  $181 = (($180) + ($177<<2)|0);
  $182 = HEAP32[$181>>2]|0;
  $183 = ($182|0)==(0|0);
  if ($183) {
   $184 = HEAP32[$db>>2]|0;
   $185 = HEAP32[$z>>2]|0;
   $186 = HEAP32[$2>>2]|0;
   $187 = (_sqlite3DbStrNDup($184,$185,$186,0)|0);
   $188 = HEAP16[$x>>1]|0;
   $189 = $188 << 16 >> 16;
   $190 = (($189) - 1)|0;
   $191 = HEAP32[$0>>2]|0;
   $192 = ((($191)) + 428|0);
   $193 = HEAP32[$192>>2]|0;
   $194 = (($193) + ($190<<2)|0);
   HEAP32[$194>>2] = $187;
  }
 }
 $195 = HEAP32[$0>>2]|0;
 $196 = ((($195)) + 400|0);
 $197 = HEAP16[$196>>1]|0;
 $198 = $197 << 16 >> 16;
 $199 = HEAP32[$db>>2]|0;
 $200 = ((($199)) + 96|0);
 $201 = ((($200)) + 36|0);
 $202 = HEAP32[$201>>2]|0;
 $203 = ($198|0)>($202|0);
 if (!($203)) {
  STACKTOP = sp;return;
 }
 $204 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($204,34238,$vararg_buffer2);
 STACKTOP = sp;return;
}
function _sqlite3ExprAddCollateToken($pParse,$pExpr,$pCollName,$dequote) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pCollName = $pCollName|0;
 $dequote = $dequote|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pCollName;
 HEAP32[$3>>2] = $dequote;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6>>>0)>(0);
 if (!($7)) {
  $23 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = (_sqlite3ExprAlloc($9,53,$10,$11)|0);
 HEAP32[$pNew>>2] = $12;
 $13 = HEAP32[$pNew>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  $23 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$pNew>>2]|0;
 $17 = ((($16)) + 12|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20 | 4352;
 HEAP32[$19>>2] = $21;
 $22 = HEAP32[$pNew>>2]|0;
 HEAP32[$1>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3ExprFunction($pParse,$pList,$pToken) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $pToken = $pToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $db = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pNew = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $pToken;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3ExprAlloc($6,151,$7,1)|0);
 HEAP32[$pNew>>2] = $8;
 $9 = HEAP32[$pNew>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  _sqlite3ExprListDelete($11,$12);
  HEAP32[$0>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$pNew>>2]|0;
  $15 = ((($14)) + 20|0);
  HEAP32[$15>>2] = $13;
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$pNew>>2]|0;
  _sqlite3ExprSetHeightAndFlags($16,$17);
  $18 = HEAP32[$pNew>>2]|0;
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _spanBinaryExpr($pParse,$op,$pLeft,$pRight) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3PExpr($4,$5,$7,$9,0)|0);
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 8|0);
 HEAP32[$16>>2] = $14;
 STACKTOP = sp;return;
}
function _exprNot($pParse,$doNot,$pSpan) {
 $pParse = $pParse|0;
 $doNot = $doNot|0;
 $pSpan = $pSpan|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $doNot;
 HEAP32[$2>>2] = $pSpan;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PExpr($5,19,$7,0,0)|0);
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$9>>2] = $8;
 STACKTOP = sp;return;
}
function _spanUnaryPostfix($pParse,$op,$pOperand,$pPostOp) {
 $pParse = $pParse|0;
 $op = $op|0;
 $pOperand = $pOperand|0;
 $pPostOp = $pPostOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $pOperand;
 HEAP32[$3>>2] = $pPostOp;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PExpr($4,$5,$7,0,0)|0);
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$3>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($12)|0);
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP32[$17>>2] = $15;
 STACKTOP = sp;return;
}
function _binaryToUnaryIfNull($pParse,$pY,$pA,$op) {
 $pParse = $pParse|0;
 $pY = $pY|0;
 $pA = $pA|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pY;
 HEAP32[$2>>2] = $pA;
 HEAP32[$3>>2] = $op;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)!=(0|0);
 $or$cond = $7 & $9;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(101);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = $14&255;
 $16 = HEAP32[$2>>2]|0;
 HEAP8[$16>>0] = $15;
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 _sqlite3ExprDelete($17,$20);
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 16|0);
 HEAP32[$22>>2] = 0;
 STACKTOP = sp;return;
}
function _spanUnaryPrefix($pOut,$pParse,$op,$pOperand,$pPreOp) {
 $pOut = $pOut|0;
 $pParse = $pParse|0;
 $op = $op|0;
 $pOperand = $pOperand|0;
 $pPreOp = $pPreOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 HEAP32[$0>>2] = $pOut;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pOperand;
 HEAP32[$4>>2] = $pPreOp;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3PExpr($9,$10,$12,0,0)|0);
 $14 = HEAP32[$0>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = $17;
 STACKTOP = sp;return;
}
function _sqlite3ExprSetHeightAndFlags($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 _exprSetHeight($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 (_sqlite3ExprCheckHeight($7,$10)|0);
 STACKTOP = sp;return;
}
function _sqlite3PExprAddSelect($pParse,$pExpr,$pSelect) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 20|0);
  HEAP32[$7>>2] = $5;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 | 2099200;
  HEAP32[$9>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  _sqlite3ExprSetHeightAndFlags($12,$13);
  STACKTOP = sp;return;
 } else {
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  _sqlite3SelectDelete($15,$16);
  STACKTOP = sp;return;
 }
}
function _parserAddExprIdListTerm($pParse,$pPrior,$pIdToken,$hasCollate,$sortOrder) {
 $pParse = $pParse|0;
 $pPrior = $pPrior|0;
 $pIdToken = $pIdToken|0;
 $hasCollate = $hasCollate|0;
 $sortOrder = $sortOrder|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pPrior;
 HEAP32[$2>>2] = $pIdToken;
 HEAP32[$3>>2] = $hasCollate;
 HEAP32[$4>>2] = $sortOrder;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3ExprListAppend($5,$6,0)|0);
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$4>>2]|0;
 $11 = ($10|0)!=(-1);
 $or$cond = $9 | $11;
 if ($or$cond) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ((($13)) + 148|0);
  $15 = ((($14)) + 5|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 4|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$2>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$vararg_buffer>>2] = $22;
   $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr2>>2] = $24;
   _sqlite3ErrorMsg($19,34155,$vararg_buffer);
  }
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$p>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 _sqlite3ExprListSetName($25,$26,$27,1);
 $28 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3DropIndex($pParse,$pName,$ifExists) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $ifExists = $ifExists|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $pIndex = 0, $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_ptr1 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0;
 var $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $pIndex = sp + 56|0;
 $v = sp + 52|0;
 $db = sp + 48|0;
 $iDb = sp + 44|0;
 $code = sp + 40|0;
 $pTab = sp + 36|0;
 $zDb = sp + 32|0;
 $zTab = sp + 28|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $ifExists;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 69|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3ReadSchema($9)|0);
 $11 = (0)!=($10|0);
 if ($11) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3FindIndex($12,$16,$20)|0);
 HEAP32[$pIndex>>2] = $21;
 $22 = HEAP32[$pIndex>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$2>>2]|0;
  $25 = ($24|0)!=(0);
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  if ($25) {
   $28 = ((($27)) + 8|0);
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   _sqlite3CodeVerifyNamedSchema($26,$30);
  } else {
   HEAP32[$vararg_buffer>>2] = $27;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 0;
   _sqlite3ErrorMsg($26,33945,$vararg_buffer);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = ((($31)) + 17|0);
  HEAP8[$32>>0] = 1;
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$pIndex>>2]|0;
 $34 = ((($33)) + 55|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35 & 3;
 $37 = $36&255;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  HEAP32[$vararg_buffer2>>2] = 0;
  _sqlite3ErrorMsg($39,33963,$vararg_buffer2);
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$db>>2]|0;
 $41 = HEAP32[$pIndex>>2]|0;
 $42 = ((($41)) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3SchemaToIndex($40,$43)|0);
 HEAP32[$iDb>>2] = $44;
 HEAP32[$code>>2] = 10;
 $45 = HEAP32[$pIndex>>2]|0;
 $46 = ((($45)) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pTab>>2] = $47;
 $48 = HEAP32[$iDb>>2]|0;
 $49 = HEAP32[$db>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48<<4)|0);
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$zDb>>2] = $53;
 $54 = HEAP32[$iDb>>2]|0;
 $55 = ($54|0)==(1);
 $56 = $55 ? 19905 : 19924;
 HEAP32[$zTab>>2] = $56;
 $57 = HEAP32[$0>>2]|0;
 $58 = HEAP32[$zTab>>2]|0;
 $59 = HEAP32[$zDb>>2]|0;
 $60 = (_sqlite3AuthCheck($57,9,$58,0,$59)|0);
 $61 = ($60|0)!=(0);
 if ($61) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $62 = HEAP32[$iDb>>2]|0;
 $63 = ($62|0)!=(0);
 if ($63) {
  HEAP32[$code>>2] = 12;
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$code>>2]|0;
 $66 = HEAP32[$pIndex>>2]|0;
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$pTab>>2]|0;
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$zDb>>2]|0;
 $71 = (_sqlite3AuthCheck($64,$65,$67,$69,$70)|0);
 $72 = ($71|0)!=(0);
 if ($72) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$0>>2]|0;
 $74 = (_sqlite3GetVdbe($73)|0);
 HEAP32[$v>>2] = $74;
 $75 = HEAP32[$v>>2]|0;
 $76 = ($75|0)!=(0|0);
 if (!($76)) {
  $106 = HEAP32[$db>>2]|0;
  $107 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($106,$107);
  STACKTOP = sp;return;
 }
 $77 = HEAP32[$0>>2]|0;
 $78 = HEAP32[$iDb>>2]|0;
 _sqlite3BeginWriteOperation($77,1,$78);
 $79 = HEAP32[$0>>2]|0;
 $80 = HEAP32[$iDb>>2]|0;
 $81 = HEAP32[$db>>2]|0;
 $82 = ((($81)) + 16|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = (($83) + ($80<<4)|0);
 $85 = HEAP32[$84>>2]|0;
 $86 = HEAP32[$iDb>>2]|0;
 $87 = ($86|0)==(1);
 $88 = $87 ? 19905 : 19924;
 $89 = HEAP32[$pIndex>>2]|0;
 $90 = HEAP32[$89>>2]|0;
 HEAP32[$vararg_buffer5>>2] = $85;
 $vararg_ptr8 = ((($vararg_buffer5)) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $88;
 $vararg_ptr9 = ((($vararg_buffer5)) + 8|0);
 HEAP32[$vararg_ptr9>>2] = $90;
 _sqlite3NestedParse($79,34036,$vararg_buffer5);
 $91 = HEAP32[$0>>2]|0;
 $92 = HEAP32[$iDb>>2]|0;
 $93 = HEAP32[$pIndex>>2]|0;
 $94 = HEAP32[$93>>2]|0;
 _sqlite3ClearStatTables($91,$92,30854,$94);
 $95 = HEAP32[$0>>2]|0;
 $96 = HEAP32[$iDb>>2]|0;
 _sqlite3ChangeCookie($95,$96);
 $97 = HEAP32[$0>>2]|0;
 $98 = HEAP32[$pIndex>>2]|0;
 $99 = ((($98)) + 44|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = HEAP32[$iDb>>2]|0;
 _destroyRootPage($97,$100,$101);
 $102 = HEAP32[$v>>2]|0;
 $103 = HEAP32[$iDb>>2]|0;
 $104 = HEAP32[$pIndex>>2]|0;
 $105 = HEAP32[$104>>2]|0;
 (_sqlite3VdbeAddOp4($102,139,$103,0,0,$105,0)|0);
 $106 = HEAP32[$db>>2]|0;
 $107 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($106,$107);
 STACKTOP = sp;return;
}
function _sqlite3Vacuum($pParse,$pNm) {
 $pParse = $pParse|0;
 $pNm = $pNm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $iDb = 0, $or$cond = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $v = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pNm;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetVdbe($2)|0);
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (_sqlite3TwoPartName($6,$7,$8,$1)|0);
  $10 = $9;
 } else {
  $10 = 0;
 }
 HEAP32[$iDb>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$iDb>>2]|0;
 $14 = ($13|0)>=(2);
 $15 = HEAP32[$iDb>>2]|0;
 $16 = ($15|0)==(0);
 $or$cond = $14 | $16;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$v>>2]|0;
 $18 = HEAP32[$iDb>>2]|0;
 (_sqlite3VdbeAddOp1($17,10,$18)|0);
 $19 = HEAP32[$v>>2]|0;
 $20 = HEAP32[$iDb>>2]|0;
 _sqlite3VdbeUsesBtree($19,$20);
 STACKTOP = sp;return;
}
function _sqlite3Pragma($pParse,$pId1,$pId2,$pValue,$minusFlag) {
 $pParse = $pParse|0;
 $pId1 = $pId1|0;
 $pId2 = $pId2|0;
 $pValue = $pValue|0;
 $minusFlag = $minusFlag|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0;
 var $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0;
 var $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0;
 var $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0;
 var $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0;
 var $1087 = 0, $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0;
 var $1104 = 0, $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0;
 var $1122 = 0, $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0;
 var $1140 = 0, $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0;
 var $1159 = 0, $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0;
 var $1177 = 0, $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0;
 var $1195 = 0, $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0;
 var $1212 = 0, $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0;
 var $1230 = 0, $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0;
 var $1249 = 0, $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0;
 var $1267 = 0, $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0;
 var $1285 = 0, $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0;
 var $1302 = 0, $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0;
 var $1320 = 0, $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0;
 var $1339 = 0, $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0;
 var $1357 = 0, $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0;
 var $1375 = 0, $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0;
 var $1393 = 0, $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0;
 var $1410 = 0, $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0;
 var $1429 = 0, $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0;
 var $1447 = 0, $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0;
 var $1465 = 0, $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0;
 var $1483 = 0, $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0;
 var $1500 = 0, $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0;
 var $1519 = 0, $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0;
 var $1537 = 0, $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0;
 var $1555 = 0, $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0;
 var $1573 = 0, $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0;
 var $1591 = 0, $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0;
 var $1609 = 0, $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0;
 var $1627 = 0, $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0;
 var $1645 = 0, $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0;
 var $1663 = 0, $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0;
 var $1681 = 0, $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0;
 var $17 = 0, $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0;
 var $1717 = 0, $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0;
 var $1735 = 0, $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0, $1752 = 0;
 var $1753 = 0, $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0, $1770 = 0;
 var $1771 = 0, $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0, $1789 = 0;
 var $179 = 0, $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0, $1806 = 0;
 var $1807 = 0, $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0, $1824 = 0;
 var $1825 = 0, $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0, $1842 = 0;
 var $1843 = 0, $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0, $1860 = 0;
 var $1861 = 0, $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0, $1879 = 0;
 var $188 = 0, $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0, $1897 = 0;
 var $1898 = 0, $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0, $1914 = 0;
 var $1915 = 0, $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0, $1932 = 0;
 var $1933 = 0, $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0, $1950 = 0;
 var $1951 = 0, $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0, $1969 = 0;
 var $197 = 0, $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0, $1987 = 0;
 var $1988 = 0, $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0, $2003 = 0;
 var $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0;
 var $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0;
 var $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0;
 var $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0;
 var $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0;
 var $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0;
 var $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0;
 var $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0;
 var $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0;
 var $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0;
 var $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0;
 var $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0;
 var $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0;
 var $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0;
 var $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0;
 var $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0;
 var $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0;
 var $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0;
 var $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0;
 var $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0;
 var $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0;
 var $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0;
 var $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0;
 var $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0;
 var $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0;
 var $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0;
 var $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0;
 var $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0;
 var $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0;
 var $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0;
 var $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0;
 var $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0;
 var $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0;
 var $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0;
 var $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0;
 var $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0;
 var $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0;
 var $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0;
 var $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0;
 var $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0;
 var $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0;
 var $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0;
 var $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0;
 var $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0;
 var $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $N = 0, $N48 = 0, $aFcntl = 0, $aOp = 0, $aOp43 = 0, $aOp44 = 0, $aOp45 = 0, $aOp8 = 0, $aRoot = 0, $addr = 0, $addr32 = 0, $addrOk = 0, $addrTop = 0, $aiCols = 0, $azOrigin = 0;
 var $b = 0, $ckUniq = 0, $cnt = 0, $cnum = 0, $db = 0, $eAuto = 0, $eMode = 0, $eMode4 = 0, $eMode47 = 0, $i = 0, $i12 = 0, $i16 = 0, $i19 = 0, $i20 = 0, $i21 = 0, $i23 = 0, $i27 = 0, $i30 = 0, $i46 = 0, $iAddr = 0;
 var $iBt = 0, $iCol = 0, $iCookie = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $iKey = 0, $iLevel = 0, $iLimit = 0, $iLimit9 = 0, $iReg = 0, $ii = 0, $ii3 = 0, $ii5 = 0, $isQuick = 0, $j = 0, $j28 = 0, $j31 = 0, $jmp2 = 0, $jmp241 = 0;
 var $jmp3 = 0, $jmp342 = 0, $jmp4 = 0, $jmp5 = 0, $jmp6 = 0, $k = 0, $k29 = 0, $kk = 0, $loopTop = 0, $lwr = 0, $mask = 0, $mid = 0, $mx = 0, $mxErr = 0, $mxIdx = 0, $n = 0, $nHidden = 0, $nIdx = 0, $or$cond = 0, $or$cond11 = 0;
 var $or$cond14 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $p = 0, $pBt = 0, $pBt2 = 0, $pBt7 = 0, $pCol = 0, $pColl = 0, $pDb = 0, $pEnc = 0, $pFK = 0, $pFK24 = 0, $pId = 0, $pIdx = 0, $pIdx14 = 0, $pIdx17 = 0, $pIdx26 = 0;
 var $pIdx35 = 0, $pIdx37 = 0, $pIdx39 = 0, $pPager = 0, $pPager6 = 0, $pParent = 0, $pPk = 0, $pPk40 = 0, $pPragma = 0, $pPrior = 0, $pTab = 0, $pTab13 = 0, $pTab15 = 0, $pTab18 = 0, $pTab22 = 0, $pTab25 = 0, $pTab34 = 0, $pTab36 = 0, $pTab38 = 0, $pTbls = 0;
 var $r1 = 0, $rc = 0, $regKey = 0, $regResult = 0, $regRow = 0, $res = 0, $size = 0, $size1 = 0, $size10 = 0, $size11 = 0, $sz = 0, $uniqOk = 0, $upr = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer15 = 0, $vararg_buffer18 = 0, $vararg_buffer20 = 0, $vararg_buffer23 = 0, $vararg_buffer25 = 0;
 var $vararg_buffer33 = 0, $vararg_buffer39 = 0, $vararg_buffer44 = 0, $vararg_buffer49 = 0, $vararg_buffer54 = 0, $vararg_buffer61 = 0, $vararg_buffer66 = 0, $vararg_buffer70 = 0, $vararg_buffer80 = 0, $vararg_buffer84 = 0, $vararg_buffer87 = 0, $vararg_buffer91 = 0, $vararg_ptr28 = 0, $vararg_ptr29 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr32 = 0, $vararg_ptr36 = 0, $vararg_ptr37 = 0, $vararg_ptr38 = 0;
 var $vararg_ptr42 = 0, $vararg_ptr43 = 0, $vararg_ptr47 = 0, $vararg_ptr48 = 0, $vararg_ptr52 = 0, $vararg_ptr53 = 0, $vararg_ptr57 = 0, $vararg_ptr58 = 0, $vararg_ptr59 = 0, $vararg_ptr60 = 0, $vararg_ptr64 = 0, $vararg_ptr65 = 0, $vararg_ptr69 = 0, $vararg_ptr73 = 0, $vararg_ptr74 = 0, $vararg_ptr75 = 0, $vararg_ptr76 = 0, $vararg_ptr77 = 0, $vararg_ptr78 = 0, $vararg_ptr79 = 0;
 var $vararg_ptr83 = 0, $vararg_ptr90 = 0, $x = 0, $x33 = 0, $zDb = 0, $zErr = 0, $zLeft = 0, $zMode = 0, $zOpt = 0, $zRet = 0, $zRight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 816|0;
 $vararg_buffer91 = sp + 264|0;
 $vararg_buffer87 = sp + 256|0;
 $vararg_buffer84 = sp + 248|0;
 $vararg_buffer80 = sp + 240|0;
 $vararg_buffer70 = sp + 208|0;
 $vararg_buffer66 = sp + 200|0;
 $vararg_buffer61 = sp + 184|0;
 $vararg_buffer54 = sp + 160|0;
 $vararg_buffer49 = sp + 144|0;
 $vararg_buffer44 = sp + 128|0;
 $vararg_buffer39 = sp + 112|0;
 $vararg_buffer33 = sp + 96|0;
 $vararg_buffer25 = sp + 72|0;
 $vararg_buffer23 = sp + 64|0;
 $vararg_buffer20 = sp + 56|0;
 $vararg_buffer18 = sp + 48|0;
 $vararg_buffer15 = sp + 40|0;
 $vararg_buffer = sp + 32|0;
 $0 = sp + 808|0;
 $1 = sp + 804|0;
 $2 = sp + 800|0;
 $3 = sp + 796|0;
 $4 = sp + 792|0;
 $zLeft = sp + 788|0;
 $zRight = sp + 784|0;
 $zDb = sp + 780|0;
 $pId = sp + 776|0;
 $aFcntl = sp + 760|0;
 $iDb = sp + 756|0;
 $lwr = sp + 752|0;
 $upr = sp + 748|0;
 $mid = sp + 744|0;
 $rc = sp + 740|0;
 $db = sp + 736|0;
 $pDb = sp + 732|0;
 $v = sp + 728|0;
 $pPragma = sp + 724|0;
 $aOp = sp + 720|0;
 $size = sp + 716|0;
 $pBt = sp + 712|0;
 $size1 = sp + 708|0;
 $pBt2 = sp + 704|0;
 $b = sp + 700|0;
 $ii = sp + 696|0;
 $iReg = sp + 692|0;
 $zRet = sp + 688|0;
 $eMode = sp + 684|0;
 $pPager = sp + 680|0;
 $ii3 = sp + 676|0;
 $eMode4 = sp + 672|0;
 $ii5 = sp + 668|0;
 $zMode = sp + 664|0;
 $n = sp + 660|0;
 $pPager6 = sp + 656|0;
 $iLimit = sp + 24|0;
 $pBt7 = sp + 652|0;
 $eAuto = sp + 648|0;
 $aOp8 = sp + 644|0;
 $iAddr = sp + 640|0;
 $iLimit9 = sp + 636|0;
 $addr = sp + 632|0;
 $size10 = sp + 628|0;
 $size11 = sp + 624|0;
 $sz = sp + 16|0;
 $res = sp + 620|0;
 $iLevel = sp + 616|0;
 $mask = sp + 612|0;
 $pTab = sp + 608|0;
 $i = sp + 604|0;
 $k = sp + 600|0;
 $nHidden = sp + 596|0;
 $pCol = sp + 592|0;
 $pPk = sp + 588|0;
 $pIdx = sp + 584|0;
 $i12 = sp + 580|0;
 $pTab13 = sp + 576|0;
 $pIdx14 = sp + 572|0;
 $pTab15 = sp + 568|0;
 $i16 = sp + 564|0;
 $mx = sp + 560|0;
 $cnum = sp + 812|0;
 $pIdx17 = sp + 556|0;
 $pTab18 = sp + 552|0;
 $i19 = sp + 548|0;
 $azOrigin = sp + 536|0;
 $i20 = sp + 532|0;
 $i21 = sp + 528|0;
 $p = sp + 524|0;
 $pColl = sp + 520|0;
 $pFK = sp + 516|0;
 $pTab22 = sp + 512|0;
 $i23 = sp + 508|0;
 $j = sp + 504|0;
 $pFK24 = sp + 500|0;
 $pTab25 = sp + 496|0;
 $pParent = sp + 492|0;
 $pIdx26 = sp + 488|0;
 $i27 = sp + 484|0;
 $j28 = sp + 480|0;
 $k29 = sp + 476|0;
 $x = sp + 472|0;
 $regResult = sp + 468|0;
 $regKey = sp + 464|0;
 $regRow = sp + 460|0;
 $addrTop = sp + 456|0;
 $addrOk = sp + 452|0;
 $aiCols = sp + 448|0;
 $iKey = sp + 444|0;
 $i30 = sp + 440|0;
 $j31 = sp + 436|0;
 $addr32 = sp + 432|0;
 $mxErr = sp + 428|0;
 $isQuick = sp + 424|0;
 $x33 = sp + 420|0;
 $pTbls = sp + 416|0;
 $aRoot = sp + 412|0;
 $cnt = sp + 408|0;
 $mxIdx = sp + 404|0;
 $nIdx = sp + 400|0;
 $pTab34 = sp + 396|0;
 $pIdx35 = sp + 392|0;
 $pTab36 = sp + 388|0;
 $pIdx37 = sp + 384|0;
 $pTab38 = sp + 380|0;
 $pIdx39 = sp + 376|0;
 $pPk40 = sp + 372|0;
 $pPrior = sp + 368|0;
 $loopTop = sp + 364|0;
 $iDataCur = sp + 360|0;
 $iIdxCur = sp + 356|0;
 $r1 = sp + 352|0;
 $zErr = sp + 348|0;
 $jmp2 = sp + 344|0;
 $jmp3 = sp + 340|0;
 $jmp241 = sp + 336|0;
 $jmp342 = sp + 332|0;
 $jmp4 = sp + 328|0;
 $jmp5 = sp + 324|0;
 $ckUniq = sp + 320|0;
 $uniqOk = sp + 316|0;
 $jmp6 = sp + 312|0;
 $kk = sp + 308|0;
 $iCol = sp + 304|0;
 $aOp43 = sp + 300|0;
 $pEnc = sp + 296|0;
 $iCookie = sp + 292|0;
 $aOp44 = sp + 288|0;
 $aOp45 = sp + 284|0;
 $i46 = sp + 280|0;
 $zOpt = sp + 276|0;
 $iBt = sp + 272|0;
 $eMode47 = sp + 268|0;
 $N = sp + 8|0;
 $N48 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pId1;
 HEAP32[$2>>2] = $pId2;
 HEAP32[$3>>2] = $pValue;
 HEAP32[$4>>2] = $minusFlag;
 HEAP32[$zLeft>>2] = 0;
 HEAP32[$zRight>>2] = 0;
 HEAP32[$zDb>>2] = 0;
 HEAP32[$mid>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$v>>2]|0;
 _sqlite3VdbeRunOnlyOnce($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 44|0);
 HEAP32[$13>>2] = 2;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = (_sqlite3TwoPartName($14,$15,$16,$pId)|0);
 HEAP32[$iDb>>2] = $17;
 $18 = HEAP32[$iDb>>2]|0;
 $19 = ($18|0)<(0);
 if ($19) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$iDb>>2]|0;
 $21 = HEAP32[$db>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20<<4)|0);
 HEAP32[$pDb>>2] = $24;
 $25 = HEAP32[$iDb>>2]|0;
 $26 = ($25|0)==(1);
 if ($26) {
  $27 = HEAP32[$0>>2]|0;
  $28 = (_sqlite3OpenTempDatabase($27)|0);
  $29 = ($28|0)!=(0);
  if ($29) {
   STACKTOP = sp;return;
  }
 }
 $30 = HEAP32[$db>>2]|0;
 $31 = HEAP32[$pId>>2]|0;
 $32 = (_sqlite3NameFromToken($30,$31)|0);
 HEAP32[$zLeft>>2] = $32;
 $33 = HEAP32[$zLeft>>2]|0;
 $34 = ($33|0)!=(0|0);
 if (!($34)) {
  STACKTOP = sp;return;
 }
 $35 = HEAP32[$4>>2]|0;
 $36 = ($35|0)!=(0);
 $37 = HEAP32[$db>>2]|0;
 $38 = HEAP32[$3>>2]|0;
 if ($36) {
  HEAP32[$vararg_buffer>>2] = $38;
  $39 = (_sqlite3MPrintf($37,32103,$vararg_buffer)|0);
  HEAP32[$zRight>>2] = $39;
 } else {
  $40 = (_sqlite3NameFromToken($37,$38)|0);
  HEAP32[$zRight>>2] = $40;
 }
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43>>>0)>(0);
 if ($44) {
  $45 = HEAP32[$pDb>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $46;
 } else {
  $47 = 0;
 }
 HEAP32[$zDb>>2] = $47;
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$zLeft>>2]|0;
 $50 = HEAP32[$zRight>>2]|0;
 $51 = HEAP32[$zDb>>2]|0;
 $52 = (_sqlite3AuthCheck($48,19,$49,$50,$51)|0);
 $53 = ($52|0)!=(0);
 L21: do {
  if (!($53)) {
   HEAP32[$aFcntl>>2] = 0;
   $54 = HEAP32[$zLeft>>2]|0;
   $55 = ((($aFcntl)) + 4|0);
   HEAP32[$55>>2] = $54;
   $56 = HEAP32[$zRight>>2]|0;
   $57 = ((($aFcntl)) + 8|0);
   HEAP32[$57>>2] = $56;
   $58 = ((($aFcntl)) + 12|0);
   HEAP32[$58>>2] = 0;
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 380|0);
   $61 = ((($60)) + 8|0);
   HEAP32[$61>>2] = 0;
   $62 = HEAP32[$db>>2]|0;
   $63 = HEAP32[$zDb>>2]|0;
   $64 = (_sqlite3_file_control($62,$63,14,$aFcntl)|0);
   HEAP32[$rc>>2] = $64;
   $65 = HEAP32[$rc>>2]|0;
   $66 = ($65|0)==(0);
   if ($66) {
    $67 = HEAP32[$v>>2]|0;
    $68 = HEAP32[$aFcntl>>2]|0;
    _returnSingleText($67,32107,$68);
    $69 = HEAP32[$aFcntl>>2]|0;
    _sqlite3_free($69);
    break;
   }
   $70 = HEAP32[$rc>>2]|0;
   $71 = ($70|0)!=(12);
   if ($71) {
    $72 = HEAP32[$aFcntl>>2]|0;
    $73 = ($72|0)!=(0|0);
    if ($73) {
     $74 = HEAP32[$0>>2]|0;
     $75 = HEAP32[$aFcntl>>2]|0;
     HEAP32[$vararg_buffer15>>2] = $75;
     _sqlite3ErrorMsg($74,18391,$vararg_buffer15);
     $76 = HEAP32[$aFcntl>>2]|0;
     _sqlite3_free($76);
    }
    $77 = HEAP32[$0>>2]|0;
    $78 = ((($77)) + 36|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = (($79) + 1)|0;
    HEAP32[$78>>2] = $80;
    $81 = HEAP32[$rc>>2]|0;
    $82 = HEAP32[$0>>2]|0;
    $83 = ((($82)) + 12|0);
    HEAP32[$83>>2] = $81;
    break;
   }
   HEAP32[$lwr>>2] = 0;
   HEAP32[$upr>>2] = 57;
   while(1) {
    $84 = HEAP32[$lwr>>2]|0;
    $85 = HEAP32[$upr>>2]|0;
    $86 = ($84|0)<=($85|0);
    if (!($86)) {
     break;
    }
    $87 = HEAP32[$lwr>>2]|0;
    $88 = HEAP32[$upr>>2]|0;
    $89 = (($87) + ($88))|0;
    $90 = (($89|0) / 2)&-1;
    HEAP32[$mid>>2] = $90;
    $91 = HEAP32[$zLeft>>2]|0;
    $92 = HEAP32[$mid>>2]|0;
    $93 = (4476 + (($92*12)|0)|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = (_sqlite3_stricmp($91,$94)|0);
    HEAP32[$rc>>2] = $95;
    $96 = HEAP32[$rc>>2]|0;
    $97 = ($96|0)==(0);
    if ($97) {
     break;
    }
    $98 = HEAP32[$rc>>2]|0;
    $99 = ($98|0)<(0);
    $100 = HEAP32[$mid>>2]|0;
    if ($99) {
     $101 = (($100) - 1)|0;
     HEAP32[$upr>>2] = $101;
     continue;
    } else {
     $102 = (($100) + 1)|0;
     HEAP32[$lwr>>2] = $102;
     continue;
    }
   }
   $103 = HEAP32[$lwr>>2]|0;
   $104 = HEAP32[$upr>>2]|0;
   $105 = ($103|0)>($104|0);
   if (!($105)) {
    $106 = HEAP32[$mid>>2]|0;
    $107 = (4476 + (($106*12)|0)|0);
    HEAP32[$pPragma>>2] = $107;
    $108 = HEAP32[$pPragma>>2]|0;
    $109 = ((($108)) + 5|0);
    $110 = HEAP8[$109>>0]|0;
    $111 = $110&255;
    $112 = $111 & 1;
    $113 = ($112|0)!=(0);
    if ($113) {
     $114 = HEAP32[$0>>2]|0;
     $115 = (_sqlite3ReadSchema($114)|0);
     $116 = ($115|0)!=(0);
     if ($116) {
      break;
     }
    }
    $117 = HEAP32[$pPragma>>2]|0;
    $118 = ((($117)) + 4|0);
    $119 = HEAP8[$118>>0]|0;
    $120 = $119&255;
    do {
     switch ($120|0) {
     case 11:  {
      $121 = HEAP32[$v>>2]|0;
      $122 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($121,$122);
      $123 = HEAP32[$zRight>>2]|0;
      $124 = ($123|0)!=(0|0);
      if ($124) {
       $142 = HEAP32[$zRight>>2]|0;
       $143 = (_sqlite3Atoi($142)|0);
       $144 = (_sqlite3AbsInt32($143)|0);
       HEAP32[$size>>2] = $144;
       $145 = HEAP32[$0>>2]|0;
       $146 = HEAP32[$iDb>>2]|0;
       _sqlite3BeginWriteOperation($145,0,$146);
       $147 = HEAP32[$v>>2]|0;
       $148 = HEAP32[$iDb>>2]|0;
       $149 = HEAP32[$size>>2]|0;
       (_sqlite3VdbeAddOp3($147,102,$148,3,$149)|0);
       $150 = HEAP32[$size>>2]|0;
       $151 = HEAP32[$pDb>>2]|0;
       $152 = ((($151)) + 12|0);
       $153 = HEAP32[$152>>2]|0;
       $154 = ((($153)) + 80|0);
       HEAP32[$154>>2] = $150;
       $155 = HEAP32[$pDb>>2]|0;
       $156 = ((($155)) + 4|0);
       $157 = HEAP32[$156>>2]|0;
       $158 = HEAP32[$pDb>>2]|0;
       $159 = ((($158)) + 12|0);
       $160 = HEAP32[$159>>2]|0;
       $161 = ((($160)) + 80|0);
       $162 = HEAP32[$161>>2]|0;
       (_sqlite3BtreeSetCacheSize($157,$162)|0);
       break L21;
      } else {
       $125 = HEAP32[$v>>2]|0;
       _setOneColumnName($125,32114);
       $126 = HEAP32[$0>>2]|0;
       $127 = ((($126)) + 44|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = (($128) + 2)|0;
       HEAP32[$127>>2] = $129;
       $130 = HEAP32[$v>>2]|0;
       $131 = (_sqlite3VdbeAddOpList($130,9,32125,0)|0);
       HEAP32[$aOp>>2] = $131;
       $132 = HEAP32[$iDb>>2]|0;
       $133 = HEAP32[$aOp>>2]|0;
       $134 = ((($133)) + 4|0);
       HEAP32[$134>>2] = $132;
       $135 = HEAP32[$iDb>>2]|0;
       $136 = HEAP32[$aOp>>2]|0;
       $137 = ((($136)) + 20|0);
       $138 = ((($137)) + 4|0);
       HEAP32[$138>>2] = $135;
       $139 = HEAP32[$aOp>>2]|0;
       $140 = ((($139)) + 120|0);
       $141 = ((($140)) + 4|0);
       HEAP32[$141>>2] = -2000;
       break L21;
      }
      break;
     }
     case 25:  {
      $163 = HEAP32[$pDb>>2]|0;
      $164 = ((($163)) + 4|0);
      $165 = HEAP32[$164>>2]|0;
      HEAP32[$pBt>>2] = $165;
      $166 = HEAP32[$zRight>>2]|0;
      $167 = ($166|0)!=(0|0);
      if ($167) {
       $177 = HEAP32[$zRight>>2]|0;
       $178 = (_sqlite3Atoi($177)|0);
       $179 = HEAP32[$db>>2]|0;
       $180 = ((($179)) + 80|0);
       HEAP32[$180>>2] = $178;
       $181 = HEAP32[$pBt>>2]|0;
       $182 = HEAP32[$db>>2]|0;
       $183 = ((($182)) + 80|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = (_sqlite3BtreeSetPageSize($181,$184,-1,0)|0);
       $186 = (7)==($185|0);
       if (!($186)) {
        break L21;
       }
       $187 = HEAP32[$db>>2]|0;
       _sqlite3OomFault($187);
       break L21;
      }
      $168 = HEAP32[$pBt>>2]|0;
      $169 = ($168|0)!=(0|0);
      if ($169) {
       $170 = HEAP32[$pBt>>2]|0;
       $171 = (_sqlite3BtreeGetPageSize($170)|0);
       $172 = $171;
      } else {
       $172 = 0;
      }
      HEAP32[$size1>>2] = $172;
      $173 = HEAP32[$v>>2]|0;
      $174 = HEAP32[$size1>>2]|0;
      $175 = ($174|0)<(0);
      $176 = $175 << 31 >> 31;
      _returnSingleInt($173,32161,$174,$176);
      break L21;
      break;
     }
     case 26:  {
      $188 = HEAP32[$pDb>>2]|0;
      $189 = ((($188)) + 4|0);
      $190 = HEAP32[$189>>2]|0;
      HEAP32[$pBt2>>2] = $190;
      HEAP32[$b>>2] = -1;
      $191 = HEAP32[$zRight>>2]|0;
      $192 = ($191|0)!=(0|0);
      if ($192) {
       $193 = HEAP32[$zRight>>2]|0;
       $194 = (_sqlite3GetBoolean($193,0)|0);
       $195 = $194&255;
       HEAP32[$b>>2] = $195;
      }
      $196 = HEAP32[$2>>2]|0;
      $197 = ((($196)) + 4|0);
      $198 = HEAP32[$197>>2]|0;
      $199 = ($198|0)==(0);
      $200 = HEAP32[$b>>2]|0;
      $201 = ($200|0)>=(0);
      $or$cond = $199 & $201;
      L65: do {
       if ($or$cond) {
        HEAP32[$ii>>2] = 0;
        while(1) {
         $202 = HEAP32[$ii>>2]|0;
         $203 = HEAP32[$db>>2]|0;
         $204 = ((($203)) + 20|0);
         $205 = HEAP32[$204>>2]|0;
         $206 = ($202|0)<($205|0);
         if (!($206)) {
          break L65;
         }
         $207 = HEAP32[$ii>>2]|0;
         $208 = HEAP32[$db>>2]|0;
         $209 = ((($208)) + 16|0);
         $210 = HEAP32[$209>>2]|0;
         $211 = (($210) + ($207<<4)|0);
         $212 = ((($211)) + 4|0);
         $213 = HEAP32[$212>>2]|0;
         $214 = HEAP32[$b>>2]|0;
         (_sqlite3BtreeSecureDelete($213,$214)|0);
         $215 = HEAP32[$ii>>2]|0;
         $216 = (($215) + 1)|0;
         HEAP32[$ii>>2] = $216;
        }
       }
      } while(0);
      $217 = HEAP32[$pBt2>>2]|0;
      $218 = HEAP32[$b>>2]|0;
      $219 = (_sqlite3BtreeSecureDelete($217,$218)|0);
      HEAP32[$b>>2] = $219;
      $220 = HEAP32[$v>>2]|0;
      $221 = HEAP32[$b>>2]|0;
      $222 = ($221|0)<(0);
      $223 = $222 << 31 >> 31;
      _returnSingleInt($220,32171,$221,$223);
      break L21;
      break;
     }
     case 23:  {
      $224 = HEAP32[$0>>2]|0;
      $225 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($224,$225);
      $226 = HEAP32[$0>>2]|0;
      $227 = ((($226)) + 44|0);
      $228 = HEAP32[$227>>2]|0;
      $229 = (($228) + 1)|0;
      HEAP32[$227>>2] = $229;
      HEAP32[$iReg>>2] = $229;
      $230 = HEAP32[$zLeft>>2]|0;
      $231 = HEAP8[$230>>0]|0;
      $232 = $231&255;
      $233 = (17613 + ($232)|0);
      $234 = HEAP8[$233>>0]|0;
      $235 = $234&255;
      $236 = ($235|0)==(112);
      $237 = HEAP32[$v>>2]|0;
      $238 = HEAP32[$iDb>>2]|0;
      $239 = HEAP32[$iReg>>2]|0;
      if ($236) {
       (_sqlite3VdbeAddOp2($237,158,$238,$239)|0);
      } else {
       $240 = HEAP32[$zRight>>2]|0;
       $241 = (_sqlite3Atoi($240)|0);
       $242 = (_sqlite3AbsInt32($241)|0);
       (_sqlite3VdbeAddOp3($237,159,$238,$239,$242)|0);
      }
      $243 = HEAP32[$v>>2]|0;
      $244 = HEAP32[$iReg>>2]|0;
      (_sqlite3VdbeAddOp2($243,87,$244,1)|0);
      $245 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($245,1);
      $246 = HEAP32[$v>>2]|0;
      $247 = HEAP32[$zLeft>>2]|0;
      (_sqlite3VdbeSetColName($246,0,0,$247,(-1))|0);
      break L21;
      break;
     }
     case 22:  {
      HEAP32[$zRet>>2] = 32185;
      $248 = HEAP32[$zRight>>2]|0;
      $249 = (_getLockingMode($248)|0);
      HEAP32[$eMode>>2] = $249;
      $250 = HEAP32[$2>>2]|0;
      $251 = ((($250)) + 4|0);
      $252 = HEAP32[$251>>2]|0;
      $253 = ($252|0)==(0);
      $254 = HEAP32[$eMode>>2]|0;
      $255 = ($254|0)==(-1);
      $or$cond3 = $253 & $255;
      if ($or$cond3) {
       $256 = HEAP32[$db>>2]|0;
       $257 = ((($256)) + 71|0);
       $258 = HEAP8[$257>>0]|0;
       $259 = $258&255;
       HEAP32[$eMode>>2] = $259;
      } else {
       $260 = HEAP32[$2>>2]|0;
       $261 = ((($260)) + 4|0);
       $262 = HEAP32[$261>>2]|0;
       $263 = ($262|0)==(0);
       if ($263) {
        HEAP32[$ii3>>2] = 2;
        while(1) {
         $264 = HEAP32[$ii3>>2]|0;
         $265 = HEAP32[$db>>2]|0;
         $266 = ((($265)) + 20|0);
         $267 = HEAP32[$266>>2]|0;
         $268 = ($264|0)<($267|0);
         if (!($268)) {
          break;
         }
         $269 = HEAP32[$ii3>>2]|0;
         $270 = HEAP32[$db>>2]|0;
         $271 = ((($270)) + 16|0);
         $272 = HEAP32[$271>>2]|0;
         $273 = (($272) + ($269<<4)|0);
         $274 = ((($273)) + 4|0);
         $275 = HEAP32[$274>>2]|0;
         $276 = (_sqlite3BtreePager($275)|0);
         HEAP32[$pPager>>2] = $276;
         $277 = HEAP32[$pPager>>2]|0;
         $278 = HEAP32[$eMode>>2]|0;
         (_sqlite3PagerLockingMode($277,$278)|0);
         $279 = HEAP32[$ii3>>2]|0;
         $280 = (($279) + 1)|0;
         HEAP32[$ii3>>2] = $280;
        }
        $281 = HEAP32[$eMode>>2]|0;
        $282 = $281&255;
        $283 = HEAP32[$db>>2]|0;
        $284 = ((($283)) + 71|0);
        HEAP8[$284>>0] = $282;
       }
       $285 = HEAP32[$pDb>>2]|0;
       $286 = ((($285)) + 4|0);
       $287 = HEAP32[$286>>2]|0;
       $288 = (_sqlite3BtreePager($287)|0);
       HEAP32[$pPager>>2] = $288;
       $289 = HEAP32[$pPager>>2]|0;
       $290 = HEAP32[$eMode>>2]|0;
       $291 = (_sqlite3PagerLockingMode($289,$290)|0);
       HEAP32[$eMode>>2] = $291;
      }
      $292 = HEAP32[$eMode>>2]|0;
      $293 = ($292|0)==(1);
      if ($293) {
       HEAP32[$zRet>>2] = 32192;
      }
      $294 = HEAP32[$v>>2]|0;
      $295 = HEAP32[$zRet>>2]|0;
      _returnSingleText($294,32202,$295);
      break L21;
      break;
     }
     case 19:  {
      $296 = HEAP32[$v>>2]|0;
      _setOneColumnName($296,32215);
      $297 = HEAP32[$zRight>>2]|0;
      $298 = ($297|0)==(0|0);
      if ($298) {
       HEAP32[$eMode4>>2] = -1;
      } else {
       $299 = HEAP32[$zRight>>2]|0;
       $300 = (_sqlite3Strlen30($299)|0);
       HEAP32[$n>>2] = $300;
       HEAP32[$eMode4>>2] = 0;
       while(1) {
        $301 = HEAP32[$eMode4>>2]|0;
        $302 = (_sqlite3JournalModename($301)|0);
        HEAP32[$zMode>>2] = $302;
        $303 = ($302|0)!=(0|0);
        if (!($303)) {
         break;
        }
        $304 = HEAP32[$zRight>>2]|0;
        $305 = HEAP32[$zMode>>2]|0;
        $306 = HEAP32[$n>>2]|0;
        $307 = (_sqlite3_strnicmp($304,$305,$306)|0);
        $308 = ($307|0)==(0);
        if ($308) {
         break;
        }
        $309 = HEAP32[$eMode4>>2]|0;
        $310 = (($309) + 1)|0;
        HEAP32[$eMode4>>2] = $310;
       }
       $311 = HEAP32[$zMode>>2]|0;
       $312 = ($311|0)!=(0|0);
       if (!($312)) {
        HEAP32[$eMode4>>2] = -1;
       }
      }
      $313 = HEAP32[$eMode4>>2]|0;
      $314 = ($313|0)==(-1);
      if ($314) {
       $315 = HEAP32[$2>>2]|0;
       $316 = ((($315)) + 4|0);
       $317 = HEAP32[$316>>2]|0;
       $318 = ($317|0)==(0);
       if ($318) {
        HEAP32[$iDb>>2] = 0;
        $319 = HEAP32[$2>>2]|0;
        $320 = ((($319)) + 4|0);
        HEAP32[$320>>2] = 1;
       }
      }
      $321 = HEAP32[$db>>2]|0;
      $322 = ((($321)) + 20|0);
      $323 = HEAP32[$322>>2]|0;
      $324 = (($323) - 1)|0;
      HEAP32[$ii5>>2] = $324;
      while(1) {
       $325 = HEAP32[$ii5>>2]|0;
       $326 = ($325|0)>=(0);
       if (!($326)) {
        break;
       }
       $327 = HEAP32[$ii5>>2]|0;
       $328 = HEAP32[$db>>2]|0;
       $329 = ((($328)) + 16|0);
       $330 = HEAP32[$329>>2]|0;
       $331 = (($330) + ($327<<4)|0);
       $332 = ((($331)) + 4|0);
       $333 = HEAP32[$332>>2]|0;
       $334 = ($333|0)!=(0|0);
       do {
        if ($334) {
         $335 = HEAP32[$ii5>>2]|0;
         $336 = HEAP32[$iDb>>2]|0;
         $337 = ($335|0)==($336|0);
         if (!($337)) {
          $338 = HEAP32[$2>>2]|0;
          $339 = ((($338)) + 4|0);
          $340 = HEAP32[$339>>2]|0;
          $341 = ($340|0)==(0);
          if (!($341)) {
           break;
          }
         }
         $342 = HEAP32[$v>>2]|0;
         $343 = HEAP32[$ii5>>2]|0;
         _sqlite3VdbeUsesBtree($342,$343);
         $344 = HEAP32[$v>>2]|0;
         $345 = HEAP32[$ii5>>2]|0;
         $346 = HEAP32[$eMode4>>2]|0;
         (_sqlite3VdbeAddOp3($344,9,$345,1,$346)|0);
        }
       } while(0);
       $347 = HEAP32[$ii5>>2]|0;
       $348 = (($347) + -1)|0;
       HEAP32[$ii5>>2] = $348;
      }
      $349 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($349,87,1,1)|0);
      break L21;
      break;
     }
     case 20:  {
      $350 = HEAP32[$pDb>>2]|0;
      $351 = ((($350)) + 4|0);
      $352 = HEAP32[$351>>2]|0;
      $353 = (_sqlite3BtreePager($352)|0);
      HEAP32[$pPager6>>2] = $353;
      $354 = $iLimit;
      $355 = $354;
      HEAP32[$355>>2] = -2;
      $356 = (($354) + 4)|0;
      $357 = $356;
      HEAP32[$357>>2] = -1;
      $358 = HEAP32[$zRight>>2]|0;
      $359 = ($358|0)!=(0|0);
      if ($359) {
       $360 = HEAP32[$zRight>>2]|0;
       (_sqlite3DecOrHexToI64($360,$iLimit)|0);
       $361 = $iLimit;
       $362 = $361;
       $363 = HEAP32[$362>>2]|0;
       $364 = (($361) + 4)|0;
       $365 = $364;
       $366 = HEAP32[$365>>2]|0;
       $367 = ($366|0)<(-1);
       $368 = ($363>>>0)<(4294967295);
       $369 = ($366|0)==(-1);
       $370 = $369 & $368;
       $371 = $367 | $370;
       if ($371) {
        $372 = $iLimit;
        $373 = $372;
        HEAP32[$373>>2] = -1;
        $374 = (($372) + 4)|0;
        $375 = $374;
        HEAP32[$375>>2] = -1;
       }
      }
      $376 = HEAP32[$pPager6>>2]|0;
      $377 = $iLimit;
      $378 = $377;
      $379 = HEAP32[$378>>2]|0;
      $380 = (($377) + 4)|0;
      $381 = $380;
      $382 = HEAP32[$381>>2]|0;
      $383 = (_sqlite3PagerJournalSizeLimit($376,$379,$382)|0);
      $384 = tempRet0;
      $385 = $iLimit;
      $386 = $385;
      HEAP32[$386>>2] = $383;
      $387 = (($385) + 4)|0;
      $388 = $387;
      HEAP32[$388>>2] = $384;
      $389 = HEAP32[$v>>2]|0;
      $390 = $iLimit;
      $391 = $390;
      $392 = HEAP32[$391>>2]|0;
      $393 = (($390) + 4)|0;
      $394 = $393;
      $395 = HEAP32[$394>>2]|0;
      _returnSingleInt($389,32228,$392,$395);
      break L21;
      break;
     }
     case 1:  {
      $396 = HEAP32[$pDb>>2]|0;
      $397 = ((($396)) + 4|0);
      $398 = HEAP32[$397>>2]|0;
      HEAP32[$pBt7>>2] = $398;
      $399 = HEAP32[$zRight>>2]|0;
      $400 = ($399|0)!=(0|0);
      if (!($400)) {
       $401 = HEAP32[$v>>2]|0;
       $402 = HEAP32[$pBt7>>2]|0;
       $403 = (_sqlite3BtreeGetAutoVacuum($402)|0);
       $404 = ($403|0)<(0);
       $405 = $404 << 31 >> 31;
       _returnSingleInt($401,32247,$403,$405);
       break L21;
      }
      $406 = HEAP32[$zRight>>2]|0;
      $407 = (_getAutoVacuum($406)|0);
      HEAP32[$eAuto>>2] = $407;
      $408 = HEAP32[$eAuto>>2]|0;
      $409 = $408&255;
      $410 = HEAP32[$db>>2]|0;
      $411 = ((($410)) + 72|0);
      HEAP8[$411>>0] = $409;
      $412 = HEAP32[$pBt7>>2]|0;
      $413 = HEAP32[$eAuto>>2]|0;
      $414 = (_sqlite3BtreeSetAutoVacuum($412,$413)|0);
      HEAP32[$rc>>2] = $414;
      $415 = HEAP32[$rc>>2]|0;
      $416 = ($415|0)==(0);
      if (!($416)) {
       break L21;
      }
      $417 = HEAP32[$eAuto>>2]|0;
      $418 = ($417|0)==(1);
      $419 = HEAP32[$eAuto>>2]|0;
      $420 = ($419|0)==(2);
      $or$cond5 = $418 | $420;
      if (!($or$cond5)) {
       break L21;
      }
      $421 = HEAP32[$v>>2]|0;
      $422 = (_sqlite3VdbeCurrentAddr($421)|0);
      HEAP32[$iAddr>>2] = $422;
      $423 = HEAP32[$v>>2]|0;
      $424 = (_sqlite3VdbeAddOpList($423,5,32259,0)|0);
      HEAP32[$aOp8>>2] = $424;
      $425 = HEAP32[$iDb>>2]|0;
      $426 = HEAP32[$aOp8>>2]|0;
      $427 = ((($426)) + 4|0);
      HEAP32[$427>>2] = $425;
      $428 = HEAP32[$iDb>>2]|0;
      $429 = HEAP32[$aOp8>>2]|0;
      $430 = ((($429)) + 20|0);
      $431 = ((($430)) + 4|0);
      HEAP32[$431>>2] = $428;
      $432 = HEAP32[$iAddr>>2]|0;
      $433 = (($432) + 4)|0;
      $434 = HEAP32[$aOp8>>2]|0;
      $435 = ((($434)) + 40|0);
      $436 = ((($435)) + 8|0);
      HEAP32[$436>>2] = $433;
      $437 = HEAP32[$iDb>>2]|0;
      $438 = HEAP32[$aOp8>>2]|0;
      $439 = ((($438)) + 80|0);
      $440 = ((($439)) + 4|0);
      HEAP32[$440>>2] = $437;
      $441 = HEAP32[$eAuto>>2]|0;
      $442 = (($441) - 1)|0;
      $443 = HEAP32[$aOp8>>2]|0;
      $444 = ((($443)) + 80|0);
      $445 = ((($444)) + 12|0);
      HEAP32[$445>>2] = $442;
      $446 = HEAP32[$v>>2]|0;
      $447 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($446,$447);
      break L21;
      break;
     }
     case 15:  {
      $448 = HEAP32[$zRight>>2]|0;
      $449 = ($448|0)==(0|0);
      if ($449) {
       label = 89;
      } else {
       $450 = HEAP32[$zRight>>2]|0;
       $451 = (_sqlite3GetInt32($450,$iLimit9)|0);
       $452 = ($451|0)==(0);
       $453 = HEAP32[$iLimit9>>2]|0;
       $454 = ($453|0)<=(0);
       $or$cond7 = $452 | $454;
       if ($or$cond7) {
        label = 89;
       }
      }
      if ((label|0) == 89) {
       HEAP32[$iLimit9>>2] = 2147483647;
      }
      $455 = HEAP32[$0>>2]|0;
      $456 = HEAP32[$iDb>>2]|0;
      _sqlite3BeginWriteOperation($455,0,$456);
      $457 = HEAP32[$v>>2]|0;
      $458 = HEAP32[$iLimit9>>2]|0;
      (_sqlite3VdbeAddOp2($457,76,$458,1)|0);
      $459 = HEAP32[$v>>2]|0;
      $460 = HEAP32[$iDb>>2]|0;
      $461 = (_sqlite3VdbeAddOp1($459,69,$460)|0);
      HEAP32[$addr>>2] = $461;
      $462 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp1($462,87,1)|0);
      $463 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($463,91,1,-1)|0);
      $464 = HEAP32[$v>>2]|0;
      $465 = HEAP32[$addr>>2]|0;
      (_sqlite3VdbeAddOp2($464,66,1,$465)|0);
      $466 = HEAP32[$v>>2]|0;
      $467 = HEAP32[$addr>>2]|0;
      _sqlite3VdbeJumpHere($466,$467);
      break L21;
      break;
     }
     case 4:  {
      $468 = HEAP32[$zRight>>2]|0;
      $469 = ($468|0)!=(0|0);
      if ($469) {
       $478 = HEAP32[$zRight>>2]|0;
       $479 = (_sqlite3Atoi($478)|0);
       HEAP32[$size10>>2] = $479;
       $480 = HEAP32[$size10>>2]|0;
       $481 = HEAP32[$pDb>>2]|0;
       $482 = ((($481)) + 12|0);
       $483 = HEAP32[$482>>2]|0;
       $484 = ((($483)) + 80|0);
       HEAP32[$484>>2] = $480;
       $485 = HEAP32[$pDb>>2]|0;
       $486 = ((($485)) + 4|0);
       $487 = HEAP32[$486>>2]|0;
       $488 = HEAP32[$pDb>>2]|0;
       $489 = ((($488)) + 12|0);
       $490 = HEAP32[$489>>2]|0;
       $491 = ((($490)) + 80|0);
       $492 = HEAP32[$491>>2]|0;
       (_sqlite3BtreeSetCacheSize($487,$492)|0);
       break L21;
      } else {
       $470 = HEAP32[$v>>2]|0;
       $471 = HEAP32[$pDb>>2]|0;
       $472 = ((($471)) + 12|0);
       $473 = HEAP32[$472>>2]|0;
       $474 = ((($473)) + 80|0);
       $475 = HEAP32[$474>>2]|0;
       $476 = ($475|0)<(0);
       $477 = $476 << 31 >> 31;
       _returnSingleInt($470,32114,$475,$477);
       break L21;
      }
      break;
     }
     case 5:  {
      $493 = HEAP32[$zRight>>2]|0;
      $494 = ($493|0)!=(0|0);
      if (!($494)) {
       $495 = HEAP32[$v>>2]|0;
       $496 = HEAP32[$db>>2]|0;
       $497 = ((($496)) + 24|0);
       $498 = HEAP32[$497>>2]|0;
       $499 = $498 & 32;
       $500 = ($499|0)==(0);
       if ($500) {
        $506 = 0;
       } else {
        $501 = HEAP32[$pDb>>2]|0;
        $502 = ((($501)) + 4|0);
        $503 = HEAP32[$502>>2]|0;
        $504 = (_sqlite3BtreeSetSpillSize($503,0)|0);
        $506 = $504;
       }
       $505 = ($506|0)<(0);
       $507 = $505 << 31 >> 31;
       _returnSingleInt($495,32279,$506,$507);
       break L21;
      }
      HEAP32[$size11>>2] = 1;
      $508 = HEAP32[$zRight>>2]|0;
      $509 = (_sqlite3GetInt32($508,$size11)|0);
      $510 = ($509|0)!=(0);
      if ($510) {
       $511 = HEAP32[$pDb>>2]|0;
       $512 = ((($511)) + 4|0);
       $513 = HEAP32[$512>>2]|0;
       $514 = HEAP32[$size11>>2]|0;
       (_sqlite3BtreeSetSpillSize($513,$514)|0);
      }
      $515 = HEAP32[$zRight>>2]|0;
      $516 = HEAP32[$size11>>2]|0;
      $517 = ($516|0)!=(0);
      $518 = $517&1;
      $519 = $518&255;
      $520 = (_sqlite3GetBoolean($515,$519)|0);
      $521 = ($520<<24>>24)!=(0);
      $522 = HEAP32[$db>>2]|0;
      $523 = ((($522)) + 24|0);
      $524 = HEAP32[$523>>2]|0;
      if ($521) {
       $525 = $524 | 32;
       HEAP32[$523>>2] = $525;
      } else {
       $526 = $524 & -33;
       HEAP32[$523>>2] = $526;
      }
      $527 = HEAP32[$db>>2]|0;
      _setAllPagerFlags($527);
      break L21;
      break;
     }
     case 24:  {
      $528 = $sz;
      $529 = $528;
      HEAP32[$529>>2] = 0;
      $530 = (($528) + 4)|0;
      $531 = $530;
      HEAP32[$531>>2] = 0;
      HEAP32[$rc>>2] = 0;
      $532 = HEAP32[$rc>>2]|0;
      $533 = ($532|0)==(0);
      if ($533) {
       $534 = HEAP32[$v>>2]|0;
       $535 = $sz;
       $536 = $535;
       $537 = HEAP32[$536>>2]|0;
       $538 = (($535) + 4)|0;
       $539 = $538;
       $540 = HEAP32[$539>>2]|0;
       _returnSingleInt($534,32291,$537,$540);
       break L21;
      }
      $541 = HEAP32[$rc>>2]|0;
      $542 = ($541|0)!=(12);
      if (!($542)) {
       break L21;
      }
      $543 = HEAP32[$0>>2]|0;
      $544 = ((($543)) + 36|0);
      $545 = HEAP32[$544>>2]|0;
      $546 = (($545) + 1)|0;
      HEAP32[$544>>2] = $546;
      $547 = HEAP32[$rc>>2]|0;
      $548 = HEAP32[$0>>2]|0;
      $549 = ((($548)) + 12|0);
      HEAP32[$549>>2] = $547;
      break L21;
      break;
     }
     case 32:  {
      $550 = HEAP32[$zRight>>2]|0;
      $551 = ($550|0)!=(0|0);
      if ($551) {
       $557 = HEAP32[$0>>2]|0;
       $558 = HEAP32[$zRight>>2]|0;
       (_changeTempStorage($557,$558)|0);
       break L21;
      } else {
       $552 = HEAP32[$v>>2]|0;
       $553 = HEAP32[$db>>2]|0;
       $554 = ((($553)) + 68|0);
       $555 = HEAP8[$554>>0]|0;
       $556 = $555&255;
       _returnSingleInt($552,32301,$556,0);
       break L21;
      }
      break;
     }
     case 33:  {
      $559 = HEAP32[$zRight>>2]|0;
      $560 = ($559|0)!=(0|0);
      if (!($560)) {
       $561 = HEAP32[$v>>2]|0;
       $562 = HEAP32[972>>2]|0;
       _returnSingleText($561,32312,$562);
       break L21;
      }
      $563 = HEAP32[$zRight>>2]|0;
      $564 = HEAP8[$563>>0]|0;
      $565 = ($564<<24>>24)!=(0);
      if ($565) {
       $566 = HEAP32[$db>>2]|0;
       $567 = HEAP32[$566>>2]|0;
       $568 = HEAP32[$zRight>>2]|0;
       $569 = (_sqlite3OsAccess($567,$568,1,$res)|0);
       HEAP32[$rc>>2] = $569;
       $570 = HEAP32[$rc>>2]|0;
       $571 = ($570|0)!=(0);
       $572 = HEAP32[$res>>2]|0;
       $573 = ($572|0)==(0);
       $or$cond9 = $571 | $573;
       if ($or$cond9) {
        $574 = HEAP32[$0>>2]|0;
        _sqlite3ErrorMsg($574,32333,$vararg_buffer18);
        break L21;
       }
      }
      $575 = HEAP32[$db>>2]|0;
      $576 = ((($575)) + 68|0);
      $577 = HEAP8[$576>>0]|0;
      $578 = $577&255;
      $579 = ($578|0)<=(1);
      if ($579) {
       $580 = HEAP32[$0>>2]|0;
       (_invalidateTempStorage($580)|0);
      }
      $581 = HEAP32[972>>2]|0;
      _sqlite3_free($581);
      $582 = HEAP32[$zRight>>2]|0;
      $583 = HEAP8[$582>>0]|0;
      $584 = ($583<<24>>24)!=(0);
      if ($584) {
       $585 = HEAP32[$zRight>>2]|0;
       HEAP32[$vararg_buffer20>>2] = $585;
       $586 = (_sqlite3_mprintf(18391,$vararg_buffer20)|0);
       HEAP32[972>>2] = $586;
       break L21;
      } else {
       HEAP32[972>>2] = 0;
       break L21;
      }
      break;
     }
     case 30:  {
      $587 = HEAP32[$zRight>>2]|0;
      $588 = ($587|0)!=(0|0);
      if (!($588)) {
       $589 = HEAP32[$v>>2]|0;
       $590 = HEAP32[$pDb>>2]|0;
       $591 = ((($590)) + 8|0);
       $592 = HEAP8[$591>>0]|0;
       $593 = $592&255;
       $594 = (($593) - 1)|0;
       $595 = ($594|0)<(0);
       $596 = $595 << 31 >> 31;
       _returnSingleInt($589,32358,$594,$596);
       break L21;
      }
      $597 = HEAP32[$db>>2]|0;
      $598 = ((($597)) + 67|0);
      $599 = HEAP8[$598>>0]|0;
      $600 = ($599<<24>>24)!=(0);
      if ($600) {
       $602 = HEAP32[$zRight>>2]|0;
       $603 = (_getSafetyLevel($602,0,1)|0);
       $604 = $603&255;
       $605 = (($604) + 1)|0;
       $606 = $605 & 7;
       HEAP32[$iLevel>>2] = $606;
       $607 = HEAP32[$iLevel>>2]|0;
       $608 = ($607|0)==(0);
       $$ = $608 ? 1 : $606;
       HEAP32[$iLevel>>2] = $$;
       $609 = HEAP32[$iLevel>>2]|0;
       $610 = $609&255;
       $611 = HEAP32[$pDb>>2]|0;
       $612 = ((($611)) + 8|0);
       HEAP8[$612>>0] = $610;
       $613 = HEAP32[$pDb>>2]|0;
       $614 = ((($613)) + 9|0);
       HEAP8[$614>>0] = 1;
       $615 = HEAP32[$db>>2]|0;
       _setAllPagerFlags($615);
       break L21;
      } else {
       $601 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($601,32370,$vararg_buffer23);
       break L21;
      }
      break;
     }
     case 2:  {
      $616 = HEAP32[$zRight>>2]|0;
      $617 = ($616|0)==(0|0);
      if ($617) {
       $618 = HEAP32[$v>>2]|0;
       $619 = HEAP32[$pPragma>>2]|0;
       $620 = HEAP32[$619>>2]|0;
       $621 = HEAP32[$db>>2]|0;
       $622 = ((($621)) + 24|0);
       $623 = HEAP32[$622>>2]|0;
       $624 = HEAP32[$pPragma>>2]|0;
       $625 = ((($624)) + 8|0);
       $626 = HEAP32[$625>>2]|0;
       $627 = $623 & $626;
       $628 = ($627|0)!=(0);
       $629 = $628&1;
       $630 = ($629|0)<(0);
       $631 = $630 << 31 >> 31;
       _returnSingleInt($618,$620,$629,$631);
       break L21;
      }
      $632 = HEAP32[$pPragma>>2]|0;
      $633 = ((($632)) + 8|0);
      $634 = HEAP32[$633>>2]|0;
      HEAP32[$mask>>2] = $634;
      $635 = HEAP32[$db>>2]|0;
      $636 = ((($635)) + 67|0);
      $637 = HEAP8[$636>>0]|0;
      $638 = $637&255;
      $639 = ($638|0)==(0);
      if ($639) {
       $640 = HEAP32[$mask>>2]|0;
       $641 = $640 & -524289;
       HEAP32[$mask>>2] = $641;
      }
      $642 = HEAP32[$zRight>>2]|0;
      $643 = (_sqlite3GetBoolean($642,0)|0);
      $644 = ($643<<24>>24)!=(0);
      $645 = HEAP32[$mask>>2]|0;
      if ($644) {
       $646 = HEAP32[$db>>2]|0;
       $647 = ((($646)) + 24|0);
       $648 = HEAP32[$647>>2]|0;
       $649 = $648 | $645;
       HEAP32[$647>>2] = $649;
      } else {
       $650 = $645 ^ -1;
       $651 = HEAP32[$db>>2]|0;
       $652 = ((($651)) + 24|0);
       $653 = HEAP32[$652>>2]|0;
       $654 = $653 & $650;
       HEAP32[$652>>2] = $654;
       $655 = HEAP32[$mask>>2]|0;
       $656 = ($655|0)==(33554432);
       if ($656) {
        $657 = HEAP32[$db>>2]|0;
        $658 = ((($657)) + 448|0);
        $659 = $658;
        $660 = $659;
        HEAP32[$660>>2] = 0;
        $661 = (($659) + 4)|0;
        $662 = $661;
        HEAP32[$662>>2] = 0;
       }
      }
      $663 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp0($663,150)|0);
      $664 = HEAP32[$db>>2]|0;
      _setAllPagerFlags($664);
      break L21;
      break;
     }
     case 31:  {
      $665 = HEAP32[$zRight>>2]|0;
      $666 = ($665|0)!=(0|0);
      if (!($666)) {
       break L21;
      }
      $667 = HEAP32[$0>>2]|0;
      $668 = HEAP32[$zRight>>2]|0;
      $669 = HEAP32[$zDb>>2]|0;
      $670 = (_sqlite3LocateTable($667,2,$668,$669)|0);
      HEAP32[$pTab>>2] = $670;
      $671 = HEAP32[$pTab>>2]|0;
      $672 = ($671|0)!=(0|0);
      if (!($672)) {
       break L21;
      }
      HEAP32[$nHidden>>2] = 0;
      $673 = HEAP32[$pTab>>2]|0;
      $674 = (_sqlite3PrimaryKeyIndex($673)|0);
      HEAP32[$pPk>>2] = $674;
      $675 = HEAP32[$0>>2]|0;
      $676 = ((($675)) + 44|0);
      HEAP32[$676>>2] = 6;
      $677 = HEAP32[$0>>2]|0;
      $678 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($677,$678);
      $679 = HEAP32[$v>>2]|0;
      _setAllColumnNames($679,6,5172);
      $680 = HEAP32[$0>>2]|0;
      $681 = HEAP32[$pTab>>2]|0;
      (_sqlite3ViewGetColumnNames($680,$681)|0);
      HEAP32[$i>>2] = 0;
      $682 = HEAP32[$pTab>>2]|0;
      $683 = ((($682)) + 4|0);
      $684 = HEAP32[$683>>2]|0;
      HEAP32[$pCol>>2] = $684;
      while(1) {
       $685 = HEAP32[$i>>2]|0;
       $686 = HEAP32[$pTab>>2]|0;
       $687 = ((($686)) + 34|0);
       $688 = HEAP16[$687>>1]|0;
       $689 = $688 << 16 >> 16;
       $690 = ($685|0)<($689|0);
       if (!($690)) {
        break L21;
       }
       $691 = HEAP32[$pCol>>2]|0;
       $692 = ((($691)) + 15|0);
       $693 = HEAP8[$692>>0]|0;
       $694 = $693&255;
       $695 = $694 & 2;
       $696 = ($695|0)!=(0);
       if ($696) {
        $697 = HEAP32[$nHidden>>2]|0;
        $698 = (($697) + 1)|0;
        HEAP32[$nHidden>>2] = $698;
       } else {
        $699 = HEAP32[$pCol>>2]|0;
        $700 = ((($699)) + 15|0);
        $701 = HEAP8[$700>>0]|0;
        $702 = $701&255;
        $703 = $702 & 1;
        $704 = ($703|0)==(0);
        L202: do {
         if ($704) {
          HEAP32[$k>>2] = 0;
         } else {
          $705 = HEAP32[$pPk>>2]|0;
          $706 = ($705|0)==(0|0);
          HEAP32[$k>>2] = 1;
          if ($706) {
           break;
          }
          while(1) {
           $707 = HEAP32[$k>>2]|0;
           $708 = HEAP32[$pTab>>2]|0;
           $709 = ((($708)) + 34|0);
           $710 = HEAP16[$709>>1]|0;
           $711 = $710 << 16 >> 16;
           $712 = ($707|0)<=($711|0);
           if (!($712)) {
            break L202;
           }
           $713 = HEAP32[$k>>2]|0;
           $714 = (($713) - 1)|0;
           $715 = HEAP32[$pPk>>2]|0;
           $716 = ((($715)) + 4|0);
           $717 = HEAP32[$716>>2]|0;
           $718 = (($717) + ($714<<1)|0);
           $719 = HEAP16[$718>>1]|0;
           $720 = $719 << 16 >> 16;
           $721 = HEAP32[$i>>2]|0;
           $722 = ($720|0)!=($721|0);
           if (!($722)) {
            break L202;
           }
           $723 = HEAP32[$k>>2]|0;
           $724 = (($723) + 1)|0;
           HEAP32[$k>>2] = $724;
          }
         }
        } while(0);
        $725 = HEAP32[$v>>2]|0;
        $726 = HEAP32[$i>>2]|0;
        $727 = HEAP32[$nHidden>>2]|0;
        $728 = (($726) - ($727))|0;
        $729 = HEAP32[$pCol>>2]|0;
        $730 = HEAP32[$729>>2]|0;
        $731 = HEAP32[$pCol>>2]|0;
        $732 = (_sqlite3ColumnType($731,18402)|0);
        $733 = HEAP32[$pCol>>2]|0;
        $734 = ((($733)) + 12|0);
        $735 = HEAP8[$734>>0]|0;
        $736 = $735&255;
        $737 = ($736|0)!=(0);
        $738 = $737 ? 1 : 0;
        $739 = HEAP32[$pCol>>2]|0;
        $740 = ((($739)) + 4|0);
        $741 = HEAP32[$740>>2]|0;
        $742 = ($741|0)!=(0|0);
        if ($742) {
         $743 = HEAP32[$pCol>>2]|0;
         $744 = ((($743)) + 4|0);
         $745 = HEAP32[$744>>2]|0;
         $746 = ((($745)) + 8|0);
         $747 = HEAP32[$746>>2]|0;
         $749 = $747;
        } else {
         $749 = 0;
        }
        $748 = HEAP32[$k>>2]|0;
        HEAP32[$vararg_buffer25>>2] = $728;
        $vararg_ptr28 = ((($vararg_buffer25)) + 4|0);
        HEAP32[$vararg_ptr28>>2] = $730;
        $vararg_ptr29 = ((($vararg_buffer25)) + 8|0);
        HEAP32[$vararg_ptr29>>2] = $732;
        $vararg_ptr30 = ((($vararg_buffer25)) + 12|0);
        HEAP32[$vararg_ptr30>>2] = $738;
        $vararg_ptr31 = ((($vararg_buffer25)) + 16|0);
        HEAP32[$vararg_ptr31>>2] = $749;
        $vararg_ptr32 = ((($vararg_buffer25)) + 20|0);
        HEAP32[$vararg_ptr32>>2] = $748;
        _sqlite3VdbeMultiLoad($725,1,32423,$vararg_buffer25);
        $750 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($750,87,1,6)|0);
       }
       $751 = HEAP32[$i>>2]|0;
       $752 = (($751) + 1)|0;
       HEAP32[$i>>2] = $752;
       $753 = HEAP32[$pCol>>2]|0;
       $754 = ((($753)) + 16|0);
       HEAP32[$pCol>>2] = $754;
      }
      break;
     }
     case 29:  {
      $755 = HEAP32[$0>>2]|0;
      $756 = (_sqlite3GetVdbe($755)|0);
      HEAP32[$v>>2] = $756;
      $757 = HEAP32[$0>>2]|0;
      $758 = ((($757)) + 44|0);
      HEAP32[$758>>2] = 4;
      $759 = HEAP32[$0>>2]|0;
      $760 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($759,$760);
      $761 = HEAP32[$v>>2]|0;
      _setAllColumnNames($761,4,5196);
      $762 = HEAP32[$pDb>>2]|0;
      $763 = ((($762)) + 12|0);
      $764 = HEAP32[$763>>2]|0;
      $765 = ((($764)) + 8|0);
      $766 = ((($765)) + 8|0);
      $767 = HEAP32[$766>>2]|0;
      HEAP32[$i12>>2] = $767;
      while(1) {
       $768 = HEAP32[$i12>>2]|0;
       $769 = ($768|0)!=(0|0);
       if (!($769)) {
        break L21;
       }
       $770 = HEAP32[$i12>>2]|0;
       $771 = ((($770)) + 8|0);
       $772 = HEAP32[$771>>2]|0;
       HEAP32[$pTab13>>2] = $772;
       $773 = HEAP32[$v>>2]|0;
       $774 = HEAP32[$pTab13>>2]|0;
       $775 = HEAP32[$774>>2]|0;
       $776 = HEAP32[$pTab13>>2]|0;
       $777 = ((($776)) + 40|0);
       $778 = HEAP16[$777>>1]|0;
       $779 = $778 << 16 >> 16;
       $780 = HEAP32[$pTab13>>2]|0;
       $781 = ((($780)) + 38|0);
       $782 = HEAP16[$781>>1]|0;
       $783 = $782 << 16 >> 16;
       HEAP32[$vararg_buffer33>>2] = $775;
       $vararg_ptr36 = ((($vararg_buffer33)) + 4|0);
       HEAP32[$vararg_ptr36>>2] = 0;
       $vararg_ptr37 = ((($vararg_buffer33)) + 8|0);
       HEAP32[$vararg_ptr37>>2] = $779;
       $vararg_ptr38 = ((($vararg_buffer33)) + 12|0);
       HEAP32[$vararg_ptr38>>2] = $783;
       _sqlite3VdbeMultiLoad($773,1,32430,$vararg_buffer33);
       $784 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($784,87,1,4)|0);
       $785 = HEAP32[$pTab13>>2]|0;
       $786 = ((($785)) + 8|0);
       $787 = HEAP32[$786>>2]|0;
       HEAP32[$pIdx>>2] = $787;
       while(1) {
        $788 = HEAP32[$pIdx>>2]|0;
        $789 = ($788|0)!=(0|0);
        if (!($789)) {
         break;
        }
        $790 = HEAP32[$v>>2]|0;
        $791 = HEAP32[$pIdx>>2]|0;
        $792 = HEAP32[$791>>2]|0;
        $793 = HEAP32[$pIdx>>2]|0;
        $794 = ((($793)) + 48|0);
        $795 = HEAP16[$794>>1]|0;
        $796 = $795 << 16 >> 16;
        $797 = HEAP32[$pIdx>>2]|0;
        $798 = ((($797)) + 8|0);
        $799 = HEAP32[$798>>2]|0;
        $800 = HEAP16[$799>>1]|0;
        $801 = $800 << 16 >> 16;
        HEAP32[$vararg_buffer39>>2] = $792;
        $vararg_ptr42 = ((($vararg_buffer39)) + 4|0);
        HEAP32[$vararg_ptr42>>2] = $796;
        $vararg_ptr43 = ((($vararg_buffer39)) + 8|0);
        HEAP32[$vararg_ptr43>>2] = $801;
        _sqlite3VdbeMultiLoad($790,2,32435,$vararg_buffer39);
        $802 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($802,87,1,4)|0);
        $803 = HEAP32[$pIdx>>2]|0;
        $804 = ((($803)) + 20|0);
        $805 = HEAP32[$804>>2]|0;
        HEAP32[$pIdx>>2] = $805;
       }
       $806 = HEAP32[$i12>>2]|0;
       $807 = HEAP32[$806>>2]|0;
       HEAP32[$i12>>2] = $807;
      }
      break;
     }
     case 16:  {
      $808 = HEAP32[$zRight>>2]|0;
      $809 = ($808|0)!=(0|0);
      if (!($809)) {
       break L21;
      }
      $810 = HEAP32[$db>>2]|0;
      $811 = HEAP32[$zRight>>2]|0;
      $812 = HEAP32[$zDb>>2]|0;
      $813 = (_sqlite3FindIndex($810,$811,$812)|0);
      HEAP32[$pIdx14>>2] = $813;
      $814 = HEAP32[$pIdx14>>2]|0;
      $815 = ($814|0)!=(0|0);
      if (!($815)) {
       break L21;
      }
      $816 = HEAP32[$pPragma>>2]|0;
      $817 = ((($816)) + 8|0);
      $818 = HEAP32[$817>>2]|0;
      $819 = ($818|0)!=(0);
      $820 = HEAP32[$pIdx14>>2]|0;
      if ($819) {
       $821 = ((($820)) + 52|0);
       $822 = HEAP16[$821>>1]|0;
       $823 = $822&65535;
       HEAP32[$mx>>2] = $823;
       $824 = HEAP32[$0>>2]|0;
       $825 = ((($824)) + 44|0);
       HEAP32[$825>>2] = 6;
      } else {
       $826 = ((($820)) + 50|0);
       $827 = HEAP16[$826>>1]|0;
       $828 = $827&65535;
       HEAP32[$mx>>2] = $828;
       $829 = HEAP32[$0>>2]|0;
       $830 = ((($829)) + 44|0);
       HEAP32[$830>>2] = 3;
      }
      $831 = HEAP32[$pIdx14>>2]|0;
      $832 = ((($831)) + 12|0);
      $833 = HEAP32[$832>>2]|0;
      HEAP32[$pTab15>>2] = $833;
      $834 = HEAP32[$0>>2]|0;
      $835 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($834,$835);
      $836 = HEAP32[$v>>2]|0;
      $837 = HEAP32[$0>>2]|0;
      $838 = ((($837)) + 44|0);
      $839 = HEAP32[$838>>2]|0;
      _setAllColumnNames($836,$839,5212);
      HEAP32[$i16>>2] = 0;
      while(1) {
       $840 = HEAP32[$i16>>2]|0;
       $841 = HEAP32[$mx>>2]|0;
       $842 = ($840|0)<($841|0);
       if (!($842)) {
        break L21;
       }
       $843 = HEAP32[$i16>>2]|0;
       $844 = HEAP32[$pIdx14>>2]|0;
       $845 = ((($844)) + 4|0);
       $846 = HEAP32[$845>>2]|0;
       $847 = (($846) + ($843<<1)|0);
       $848 = HEAP16[$847>>1]|0;
       HEAP16[$cnum>>1] = $848;
       $849 = HEAP32[$v>>2]|0;
       $850 = HEAP32[$i16>>2]|0;
       $851 = HEAP16[$cnum>>1]|0;
       $852 = $851 << 16 >> 16;
       $853 = HEAP16[$cnum>>1]|0;
       $854 = $853 << 16 >> 16;
       $855 = ($854|0)<(0);
       if ($855) {
        $863 = 0;
       } else {
        $856 = HEAP16[$cnum>>1]|0;
        $857 = $856 << 16 >> 16;
        $858 = HEAP32[$pTab15>>2]|0;
        $859 = ((($858)) + 4|0);
        $860 = HEAP32[$859>>2]|0;
        $861 = (($860) + ($857<<4)|0);
        $862 = HEAP32[$861>>2]|0;
        $863 = $862;
       }
       HEAP32[$vararg_buffer44>>2] = $850;
       $vararg_ptr47 = ((($vararg_buffer44)) + 4|0);
       HEAP32[$vararg_ptr47>>2] = $852;
       $vararg_ptr48 = ((($vararg_buffer44)) + 8|0);
       HEAP32[$vararg_ptr48>>2] = $863;
       _sqlite3VdbeMultiLoad($849,1,32439,$vararg_buffer44);
       $864 = HEAP32[$pPragma>>2]|0;
       $865 = ((($864)) + 8|0);
       $866 = HEAP32[$865>>2]|0;
       $867 = ($866|0)!=(0);
       if ($867) {
        $868 = HEAP32[$v>>2]|0;
        $869 = HEAP32[$i16>>2]|0;
        $870 = HEAP32[$pIdx14>>2]|0;
        $871 = ((($870)) + 28|0);
        $872 = HEAP32[$871>>2]|0;
        $873 = (($872) + ($869)|0);
        $874 = HEAP8[$873>>0]|0;
        $875 = $874&255;
        $876 = HEAP32[$i16>>2]|0;
        $877 = HEAP32[$pIdx14>>2]|0;
        $878 = ((($877)) + 32|0);
        $879 = HEAP32[$878>>2]|0;
        $880 = (($879) + ($876<<2)|0);
        $881 = HEAP32[$880>>2]|0;
        $882 = HEAP32[$i16>>2]|0;
        $883 = HEAP32[$pIdx14>>2]|0;
        $884 = ((($883)) + 50|0);
        $885 = HEAP16[$884>>1]|0;
        $886 = $885&65535;
        $887 = ($882|0)<($886|0);
        $888 = $887&1;
        HEAP32[$vararg_buffer49>>2] = $875;
        $vararg_ptr52 = ((($vararg_buffer49)) + 4|0);
        HEAP32[$vararg_ptr52>>2] = $881;
        $vararg_ptr53 = ((($vararg_buffer49)) + 8|0);
        HEAP32[$vararg_ptr53>>2] = $888;
        _sqlite3VdbeMultiLoad($868,4,32443,$vararg_buffer49);
       }
       $889 = HEAP32[$v>>2]|0;
       $890 = HEAP32[$0>>2]|0;
       $891 = ((($890)) + 44|0);
       $892 = HEAP32[$891>>2]|0;
       (_sqlite3VdbeAddOp2($889,87,1,$892)|0);
       $893 = HEAP32[$i16>>2]|0;
       $894 = (($893) + 1)|0;
       HEAP32[$i16>>2] = $894;
      }
      break;
     }
     case 17:  {
      $895 = HEAP32[$zRight>>2]|0;
      $896 = ($895|0)!=(0|0);
      if (!($896)) {
       break L21;
      }
      $897 = HEAP32[$db>>2]|0;
      $898 = HEAP32[$zRight>>2]|0;
      $899 = HEAP32[$zDb>>2]|0;
      $900 = (_sqlite3FindTable($897,$898,$899)|0);
      HEAP32[$pTab18>>2] = $900;
      $901 = HEAP32[$pTab18>>2]|0;
      $902 = ($901|0)!=(0|0);
      if (!($902)) {
       break L21;
      }
      $903 = HEAP32[$0>>2]|0;
      $904 = (_sqlite3GetVdbe($903)|0);
      HEAP32[$v>>2] = $904;
      $905 = HEAP32[$0>>2]|0;
      $906 = ((($905)) + 44|0);
      HEAP32[$906>>2] = 5;
      $907 = HEAP32[$0>>2]|0;
      $908 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($907,$908);
      $909 = HEAP32[$v>>2]|0;
      _setAllColumnNames($909,5,5236);
      $910 = HEAP32[$pTab18>>2]|0;
      $911 = ((($910)) + 8|0);
      $912 = HEAP32[$911>>2]|0;
      HEAP32[$pIdx17>>2] = $912;
      HEAP32[$i19>>2] = 0;
      while(1) {
       $913 = HEAP32[$pIdx17>>2]|0;
       $914 = ($913|0)!=(0|0);
       if (!($914)) {
        break L21;
       }
       ;HEAP32[$azOrigin>>2]=HEAP32[5256>>2]|0;HEAP32[$azOrigin+4>>2]=HEAP32[5256+4>>2]|0;HEAP32[$azOrigin+8>>2]=HEAP32[5256+8>>2]|0;
       $915 = HEAP32[$v>>2]|0;
       $916 = HEAP32[$i19>>2]|0;
       $917 = HEAP32[$pIdx17>>2]|0;
       $918 = HEAP32[$917>>2]|0;
       $919 = HEAP32[$pIdx17>>2]|0;
       $920 = ((($919)) + 54|0);
       $921 = HEAP8[$920>>0]|0;
       $922 = $921&255;
       $923 = ($922|0)!=(0);
       $924 = $923&1;
       $925 = HEAP32[$pIdx17>>2]|0;
       $926 = ((($925)) + 55|0);
       $927 = HEAP8[$926>>0]|0;
       $928 = $927 & 3;
       $929 = $928&255;
       $930 = (($azOrigin) + ($929<<2)|0);
       $931 = HEAP32[$930>>2]|0;
       $932 = HEAP32[$pIdx17>>2]|0;
       $933 = ((($932)) + 36|0);
       $934 = HEAP32[$933>>2]|0;
       $935 = ($934|0)!=(0|0);
       $936 = $935&1;
       HEAP32[$vararg_buffer54>>2] = $916;
       $vararg_ptr57 = ((($vararg_buffer54)) + 4|0);
       HEAP32[$vararg_ptr57>>2] = $918;
       $vararg_ptr58 = ((($vararg_buffer54)) + 8|0);
       HEAP32[$vararg_ptr58>>2] = $924;
       $vararg_ptr59 = ((($vararg_buffer54)) + 12|0);
       HEAP32[$vararg_ptr59>>2] = $931;
       $vararg_ptr60 = ((($vararg_buffer54)) + 16|0);
       HEAP32[$vararg_ptr60>>2] = $936;
       _sqlite3VdbeMultiLoad($915,1,32447,$vararg_buffer54);
       $937 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($937,87,1,5)|0);
       $938 = HEAP32[$pIdx17>>2]|0;
       $939 = ((($938)) + 20|0);
       $940 = HEAP32[$939>>2]|0;
       HEAP32[$pIdx17>>2] = $940;
       $941 = HEAP32[$i19>>2]|0;
       $942 = (($941) + 1)|0;
       HEAP32[$i19>>2] = $942;
      }
      break;
     }
     case 10:  {
      $943 = HEAP32[$0>>2]|0;
      $944 = ((($943)) + 44|0);
      HEAP32[$944>>2] = 3;
      $945 = HEAP32[$v>>2]|0;
      _setAllColumnNames($945,3,5268);
      HEAP32[$i20>>2] = 0;
      while(1) {
       $946 = HEAP32[$i20>>2]|0;
       $947 = HEAP32[$db>>2]|0;
       $948 = ((($947)) + 20|0);
       $949 = HEAP32[$948>>2]|0;
       $950 = ($946|0)<($949|0);
       if (!($950)) {
        break L21;
       }
       $951 = HEAP32[$i20>>2]|0;
       $952 = HEAP32[$db>>2]|0;
       $953 = ((($952)) + 16|0);
       $954 = HEAP32[$953>>2]|0;
       $955 = (($954) + ($951<<4)|0);
       $956 = ((($955)) + 4|0);
       $957 = HEAP32[$956>>2]|0;
       $958 = ($957|0)==(0|0);
       if (!($958)) {
        $959 = HEAP32[$v>>2]|0;
        $960 = HEAP32[$i20>>2]|0;
        $961 = HEAP32[$i20>>2]|0;
        $962 = HEAP32[$db>>2]|0;
        $963 = ((($962)) + 16|0);
        $964 = HEAP32[$963>>2]|0;
        $965 = (($964) + ($961<<4)|0);
        $966 = HEAP32[$965>>2]|0;
        $967 = HEAP32[$i20>>2]|0;
        $968 = HEAP32[$db>>2]|0;
        $969 = ((($968)) + 16|0);
        $970 = HEAP32[$969>>2]|0;
        $971 = (($970) + ($967<<4)|0);
        $972 = ((($971)) + 4|0);
        $973 = HEAP32[$972>>2]|0;
        $974 = (_sqlite3BtreeGetFilename($973)|0);
        HEAP32[$vararg_buffer61>>2] = $960;
        $vararg_ptr64 = ((($vararg_buffer61)) + 4|0);
        HEAP32[$vararg_ptr64>>2] = $966;
        $vararg_ptr65 = ((($vararg_buffer61)) + 8|0);
        HEAP32[$vararg_ptr65>>2] = $974;
        _sqlite3VdbeMultiLoad($959,1,32453,$vararg_buffer61);
        $975 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($975,87,1,3)|0);
       }
       $976 = HEAP32[$i20>>2]|0;
       $977 = (($976) + 1)|0;
       HEAP32[$i20>>2] = $977;
      }
      break;
     }
     case 7:  {
      HEAP32[$i21>>2] = 0;
      $978 = HEAP32[$0>>2]|0;
      $979 = ((($978)) + 44|0);
      HEAP32[$979>>2] = 2;
      $980 = HEAP32[$v>>2]|0;
      _setAllColumnNames($980,2,5280);
      $981 = HEAP32[$db>>2]|0;
      $982 = ((($981)) + 364|0);
      $983 = ((($982)) + 8|0);
      $984 = HEAP32[$983>>2]|0;
      HEAP32[$p>>2] = $984;
      while(1) {
       $985 = HEAP32[$p>>2]|0;
       $986 = ($985|0)!=(0|0);
       if (!($986)) {
        break L21;
       }
       $987 = HEAP32[$p>>2]|0;
       $988 = ((($987)) + 8|0);
       $989 = HEAP32[$988>>2]|0;
       HEAP32[$pColl>>2] = $989;
       $990 = HEAP32[$v>>2]|0;
       $991 = HEAP32[$i21>>2]|0;
       $992 = (($991) + 1)|0;
       HEAP32[$i21>>2] = $992;
       $993 = HEAP32[$pColl>>2]|0;
       $994 = HEAP32[$993>>2]|0;
       HEAP32[$vararg_buffer66>>2] = $991;
       $vararg_ptr69 = ((($vararg_buffer66)) + 4|0);
       HEAP32[$vararg_ptr69>>2] = $994;
       _sqlite3VdbeMultiLoad($990,1,32457,$vararg_buffer66);
       $995 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($995,87,1,2)|0);
       $996 = HEAP32[$p>>2]|0;
       $997 = HEAP32[$996>>2]|0;
       HEAP32[$p>>2] = $997;
      }
      break;
     }
     case 14:  {
      $998 = HEAP32[$zRight>>2]|0;
      $999 = ($998|0)!=(0|0);
      if (!($999)) {
       break L21;
      }
      $1000 = HEAP32[$db>>2]|0;
      $1001 = HEAP32[$zRight>>2]|0;
      $1002 = HEAP32[$zDb>>2]|0;
      $1003 = (_sqlite3FindTable($1000,$1001,$1002)|0);
      HEAP32[$pTab22>>2] = $1003;
      $1004 = HEAP32[$pTab22>>2]|0;
      $1005 = ($1004|0)!=(0|0);
      if (!($1005)) {
       break L21;
      }
      $1006 = HEAP32[$0>>2]|0;
      $1007 = (_sqlite3GetVdbe($1006)|0);
      HEAP32[$v>>2] = $1007;
      $1008 = HEAP32[$pTab22>>2]|0;
      $1009 = ((($1008)) + 16|0);
      $1010 = HEAP32[$1009>>2]|0;
      HEAP32[$pFK>>2] = $1010;
      $1011 = HEAP32[$pFK>>2]|0;
      $1012 = ($1011|0)!=(0|0);
      if (!($1012)) {
       break L21;
      }
      HEAP32[$i23>>2] = 0;
      $1013 = HEAP32[$0>>2]|0;
      $1014 = ((($1013)) + 44|0);
      HEAP32[$1014>>2] = 8;
      $1015 = HEAP32[$0>>2]|0;
      $1016 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($1015,$1016);
      $1017 = HEAP32[$v>>2]|0;
      _setAllColumnNames($1017,8,5288);
      while(1) {
       $1018 = HEAP32[$pFK>>2]|0;
       $1019 = ($1018|0)!=(0|0);
       if (!($1019)) {
        break L21;
       }
       HEAP32[$j>>2] = 0;
       while(1) {
        $1020 = HEAP32[$j>>2]|0;
        $1021 = HEAP32[$pFK>>2]|0;
        $1022 = ((($1021)) + 20|0);
        $1023 = HEAP32[$1022>>2]|0;
        $1024 = ($1020|0)<($1023|0);
        if (!($1024)) {
         break;
        }
        $1025 = HEAP32[$v>>2]|0;
        $1026 = HEAP32[$i23>>2]|0;
        $1027 = HEAP32[$j>>2]|0;
        $1028 = HEAP32[$pFK>>2]|0;
        $1029 = ((($1028)) + 8|0);
        $1030 = HEAP32[$1029>>2]|0;
        $1031 = HEAP32[$j>>2]|0;
        $1032 = HEAP32[$pFK>>2]|0;
        $1033 = ((($1032)) + 36|0);
        $1034 = (($1033) + ($1031<<3)|0);
        $1035 = HEAP32[$1034>>2]|0;
        $1036 = HEAP32[$pTab22>>2]|0;
        $1037 = ((($1036)) + 4|0);
        $1038 = HEAP32[$1037>>2]|0;
        $1039 = (($1038) + ($1035<<4)|0);
        $1040 = HEAP32[$1039>>2]|0;
        $1041 = HEAP32[$j>>2]|0;
        $1042 = HEAP32[$pFK>>2]|0;
        $1043 = ((($1042)) + 36|0);
        $1044 = (($1043) + ($1041<<3)|0);
        $1045 = ((($1044)) + 4|0);
        $1046 = HEAP32[$1045>>2]|0;
        $1047 = HEAP32[$pFK>>2]|0;
        $1048 = ((($1047)) + 25|0);
        $1049 = ((($1048)) + 1|0);
        $1050 = HEAP8[$1049>>0]|0;
        $1051 = (_actionName($1050)|0);
        $1052 = HEAP32[$pFK>>2]|0;
        $1053 = ((($1052)) + 25|0);
        $1054 = HEAP8[$1053>>0]|0;
        $1055 = (_actionName($1054)|0);
        HEAP32[$vararg_buffer70>>2] = $1026;
        $vararg_ptr73 = ((($vararg_buffer70)) + 4|0);
        HEAP32[$vararg_ptr73>>2] = $1027;
        $vararg_ptr74 = ((($vararg_buffer70)) + 8|0);
        HEAP32[$vararg_ptr74>>2] = $1030;
        $vararg_ptr75 = ((($vararg_buffer70)) + 12|0);
        HEAP32[$vararg_ptr75>>2] = $1040;
        $vararg_ptr76 = ((($vararg_buffer70)) + 16|0);
        HEAP32[$vararg_ptr76>>2] = $1046;
        $vararg_ptr77 = ((($vararg_buffer70)) + 20|0);
        HEAP32[$vararg_ptr77>>2] = $1051;
        $vararg_ptr78 = ((($vararg_buffer70)) + 24|0);
        HEAP32[$vararg_ptr78>>2] = $1055;
        $vararg_ptr79 = ((($vararg_buffer70)) + 28|0);
        HEAP32[$vararg_ptr79>>2] = 32469;
        _sqlite3VdbeMultiLoad($1025,1,32460,$vararg_buffer70);
        $1056 = HEAP32[$v>>2]|0;
        (_sqlite3VdbeAddOp2($1056,87,1,8)|0);
        $1057 = HEAP32[$j>>2]|0;
        $1058 = (($1057) + 1)|0;
        HEAP32[$j>>2] = $1058;
       }
       $1059 = HEAP32[$i23>>2]|0;
       $1060 = (($1059) + 1)|0;
       HEAP32[$i23>>2] = $1060;
       $1061 = HEAP32[$pFK>>2]|0;
       $1062 = ((($1061)) + 4|0);
       $1063 = HEAP32[$1062>>2]|0;
       HEAP32[$pFK>>2] = $1063;
      }
      break;
     }
     case 13:  {
      $1064 = HEAP32[$0>>2]|0;
      $1065 = ((($1064)) + 44|0);
      $1066 = HEAP32[$1065>>2]|0;
      $1067 = (($1066) + 1)|0;
      HEAP32[$regResult>>2] = $1067;
      $1068 = HEAP32[$0>>2]|0;
      $1069 = ((($1068)) + 44|0);
      $1070 = HEAP32[$1069>>2]|0;
      $1071 = (($1070) + 4)|0;
      HEAP32[$1069>>2] = $1071;
      $1072 = HEAP32[$0>>2]|0;
      $1073 = ((($1072)) + 44|0);
      $1074 = HEAP32[$1073>>2]|0;
      $1075 = (($1074) + 1)|0;
      HEAP32[$1073>>2] = $1075;
      HEAP32[$regKey>>2] = $1075;
      $1076 = HEAP32[$0>>2]|0;
      $1077 = ((($1076)) + 44|0);
      $1078 = HEAP32[$1077>>2]|0;
      $1079 = (($1078) + 1)|0;
      HEAP32[$1077>>2] = $1079;
      HEAP32[$regRow>>2] = $1079;
      $1080 = HEAP32[$0>>2]|0;
      $1081 = (_sqlite3GetVdbe($1080)|0);
      HEAP32[$v>>2] = $1081;
      $1082 = HEAP32[$v>>2]|0;
      _setAllColumnNames($1082,4,5320);
      $1083 = HEAP32[$0>>2]|0;
      $1084 = HEAP32[$iDb>>2]|0;
      _sqlite3CodeVerifySchema($1083,$1084);
      $1085 = HEAP32[$iDb>>2]|0;
      $1086 = HEAP32[$db>>2]|0;
      $1087 = ((($1086)) + 16|0);
      $1088 = HEAP32[$1087>>2]|0;
      $1089 = (($1088) + ($1085<<4)|0);
      $1090 = ((($1089)) + 12|0);
      $1091 = HEAP32[$1090>>2]|0;
      $1092 = ((($1091)) + 8|0);
      $1093 = ((($1092)) + 8|0);
      $1094 = HEAP32[$1093>>2]|0;
      HEAP32[$k29>>2] = $1094;
      while(1) {
       $1095 = HEAP32[$k29>>2]|0;
       $1096 = ($1095|0)!=(0|0);
       if (!($1096)) {
        break L21;
       }
       $1097 = HEAP32[$zRight>>2]|0;
       $1098 = ($1097|0)!=(0|0);
       if ($1098) {
        $1099 = HEAP32[$0>>2]|0;
        $1100 = HEAP32[$zRight>>2]|0;
        $1101 = HEAP32[$zDb>>2]|0;
        $1102 = (_sqlite3LocateTable($1099,0,$1100,$1101)|0);
        HEAP32[$pTab25>>2] = $1102;
        HEAP32[$k29>>2] = 0;
       } else {
        $1103 = HEAP32[$k29>>2]|0;
        $1104 = ((($1103)) + 8|0);
        $1105 = HEAP32[$1104>>2]|0;
        HEAP32[$pTab25>>2] = $1105;
        $1106 = HEAP32[$k29>>2]|0;
        $1107 = HEAP32[$1106>>2]|0;
        HEAP32[$k29>>2] = $1107;
       }
       $1108 = HEAP32[$pTab25>>2]|0;
       $1109 = ($1108|0)==(0|0);
       if ($1109) {
        continue;
       }
       $1110 = HEAP32[$pTab25>>2]|0;
       $1111 = ((($1110)) + 16|0);
       $1112 = HEAP32[$1111>>2]|0;
       $1113 = ($1112|0)==(0|0);
       if ($1113) {
        continue;
       }
       $1114 = HEAP32[$0>>2]|0;
       $1115 = HEAP32[$iDb>>2]|0;
       $1116 = HEAP32[$pTab25>>2]|0;
       $1117 = ((($1116)) + 28|0);
       $1118 = HEAP32[$1117>>2]|0;
       $1119 = HEAP32[$pTab25>>2]|0;
       $1120 = HEAP32[$1119>>2]|0;
       _sqlite3TableLock($1114,$1115,$1118,0,$1120);
       $1121 = HEAP32[$pTab25>>2]|0;
       $1122 = ((($1121)) + 34|0);
       $1123 = HEAP16[$1122>>1]|0;
       $1124 = $1123 << 16 >> 16;
       $1125 = HEAP32[$regRow>>2]|0;
       $1126 = (($1124) + ($1125))|0;
       $1127 = HEAP32[$0>>2]|0;
       $1128 = ((($1127)) + 44|0);
       $1129 = HEAP32[$1128>>2]|0;
       $1130 = ($1126|0)>($1129|0);
       if ($1130) {
        $1131 = HEAP32[$pTab25>>2]|0;
        $1132 = ((($1131)) + 34|0);
        $1133 = HEAP16[$1132>>1]|0;
        $1134 = $1133 << 16 >> 16;
        $1135 = HEAP32[$regRow>>2]|0;
        $1136 = (($1134) + ($1135))|0;
        $1137 = HEAP32[$0>>2]|0;
        $1138 = ((($1137)) + 44|0);
        HEAP32[$1138>>2] = $1136;
       }
       $1139 = HEAP32[$0>>2]|0;
       $1140 = HEAP32[$iDb>>2]|0;
       $1141 = HEAP32[$pTab25>>2]|0;
       _sqlite3OpenTable($1139,0,$1140,$1141,104);
       $1142 = HEAP32[$v>>2]|0;
       $1143 = HEAP32[$regResult>>2]|0;
       $1144 = HEAP32[$pTab25>>2]|0;
       $1145 = HEAP32[$1144>>2]|0;
       (_sqlite3VdbeLoadString($1142,$1143,$1145)|0);
       HEAP32[$i27>>2] = 1;
       $1146 = HEAP32[$pTab25>>2]|0;
       $1147 = ((($1146)) + 16|0);
       $1148 = HEAP32[$1147>>2]|0;
       HEAP32[$pFK24>>2] = $1148;
       L279: while(1) {
        $1149 = HEAP32[$pFK24>>2]|0;
        $1150 = ($1149|0)!=(0|0);
        if (!($1150)) {
         break;
        }
        $1151 = HEAP32[$db>>2]|0;
        $1152 = HEAP32[$pFK24>>2]|0;
        $1153 = ((($1152)) + 8|0);
        $1154 = HEAP32[$1153>>2]|0;
        $1155 = HEAP32[$zDb>>2]|0;
        $1156 = (_sqlite3FindTable($1151,$1154,$1155)|0);
        HEAP32[$pParent>>2] = $1156;
        $1157 = HEAP32[$pParent>>2]|0;
        $1158 = ($1157|0)==(0|0);
        do {
         if (!($1158)) {
          HEAP32[$pIdx26>>2] = 0;
          $1159 = HEAP32[$0>>2]|0;
          $1160 = HEAP32[$iDb>>2]|0;
          $1161 = HEAP32[$pParent>>2]|0;
          $1162 = ((($1161)) + 28|0);
          $1163 = HEAP32[$1162>>2]|0;
          $1164 = HEAP32[$pParent>>2]|0;
          $1165 = HEAP32[$1164>>2]|0;
          _sqlite3TableLock($1159,$1160,$1163,0,$1165);
          $1166 = HEAP32[$0>>2]|0;
          $1167 = HEAP32[$pParent>>2]|0;
          $1168 = HEAP32[$pFK24>>2]|0;
          $1169 = (_sqlite3FkLocateIndex($1166,$1167,$1168,$pIdx26,0)|0);
          HEAP32[$x>>2] = $1169;
          $1170 = HEAP32[$x>>2]|0;
          $1171 = ($1170|0)==(0);
          if (!($1171)) {
           label = 207;
           break L279;
          }
          $1172 = HEAP32[$pIdx26>>2]|0;
          $1173 = ($1172|0)==(0|0);
          if ($1173) {
           $1174 = HEAP32[$0>>2]|0;
           $1175 = HEAP32[$i27>>2]|0;
           $1176 = HEAP32[$iDb>>2]|0;
           $1177 = HEAP32[$pParent>>2]|0;
           _sqlite3OpenTable($1174,$1175,$1176,$1177,104);
           break;
          } else {
           $1178 = HEAP32[$v>>2]|0;
           $1179 = HEAP32[$i27>>2]|0;
           $1180 = HEAP32[$pIdx26>>2]|0;
           $1181 = ((($1180)) + 44|0);
           $1182 = HEAP32[$1181>>2]|0;
           $1183 = HEAP32[$iDb>>2]|0;
           (_sqlite3VdbeAddOp3($1178,104,$1179,$1182,$1183)|0);
           $1184 = HEAP32[$0>>2]|0;
           $1185 = HEAP32[$pIdx26>>2]|0;
           _sqlite3VdbeSetP4KeyInfo($1184,$1185);
           break;
          }
         }
        } while(0);
        $1186 = HEAP32[$i27>>2]|0;
        $1187 = (($1186) + 1)|0;
        HEAP32[$i27>>2] = $1187;
        $1188 = HEAP32[$pFK24>>2]|0;
        $1189 = ((($1188)) + 4|0);
        $1190 = HEAP32[$1189>>2]|0;
        HEAP32[$pFK24>>2] = $1190;
       }
       if ((label|0) == 207) {
        label = 0;
        HEAP32[$k29>>2] = 0;
       }
       $1191 = HEAP32[$pFK24>>2]|0;
       $1192 = ($1191|0)!=(0|0);
       if ($1192) {
        break L21;
       }
       $1193 = HEAP32[$0>>2]|0;
       $1194 = ((($1193)) + 40|0);
       $1195 = HEAP32[$1194>>2]|0;
       $1196 = HEAP32[$i27>>2]|0;
       $1197 = ($1195|0)<($1196|0);
       if ($1197) {
        $1198 = HEAP32[$i27>>2]|0;
        $1199 = HEAP32[$0>>2]|0;
        $1200 = ((($1199)) + 40|0);
        HEAP32[$1200>>2] = $1198;
       }
       $1201 = HEAP32[$v>>2]|0;
       $1202 = (_sqlite3VdbeAddOp1($1201,57,0)|0);
       HEAP32[$addrTop>>2] = $1202;
       HEAP32[$i27>>2] = 1;
       $1203 = HEAP32[$pTab25>>2]|0;
       $1204 = ((($1203)) + 16|0);
       $1205 = HEAP32[$1204>>2]|0;
       HEAP32[$pFK24>>2] = $1205;
       while(1) {
        $1206 = HEAP32[$pFK24>>2]|0;
        $1207 = ($1206|0)!=(0|0);
        if (!($1207)) {
         break;
        }
        $1208 = HEAP32[$db>>2]|0;
        $1209 = HEAP32[$pFK24>>2]|0;
        $1210 = ((($1209)) + 8|0);
        $1211 = HEAP32[$1210>>2]|0;
        $1212 = HEAP32[$zDb>>2]|0;
        $1213 = (_sqlite3FindTable($1208,$1211,$1212)|0);
        HEAP32[$pParent>>2] = $1213;
        HEAP32[$pIdx26>>2] = 0;
        HEAP32[$aiCols>>2] = 0;
        $1214 = HEAP32[$pParent>>2]|0;
        $1215 = ($1214|0)!=(0|0);
        if ($1215) {
         $1216 = HEAP32[$0>>2]|0;
         $1217 = HEAP32[$pParent>>2]|0;
         $1218 = HEAP32[$pFK24>>2]|0;
         $1219 = (_sqlite3FkLocateIndex($1216,$1217,$1218,$pIdx26,$aiCols)|0);
         HEAP32[$x>>2] = $1219;
        }
        $1220 = HEAP32[$v>>2]|0;
        $1221 = (_sqlite3VdbeMakeLabel($1220)|0);
        HEAP32[$addrOk>>2] = $1221;
        $1222 = HEAP32[$pParent>>2]|0;
        $1223 = ($1222|0)!=(0|0);
        $1224 = HEAP32[$pIdx26>>2]|0;
        $1225 = ($1224|0)==(0|0);
        $or$cond11 = $1223 & $1225;
        do {
         if ($or$cond11) {
          $1226 = HEAP32[$pFK24>>2]|0;
          $1227 = ((($1226)) + 36|0);
          $1228 = HEAP32[$1227>>2]|0;
          HEAP32[$iKey>>2] = $1228;
          $1229 = HEAP32[$iKey>>2]|0;
          $1230 = HEAP32[$pTab25>>2]|0;
          $1231 = ((($1230)) + 32|0);
          $1232 = HEAP16[$1231>>1]|0;
          $1233 = $1232 << 16 >> 16;
          $1234 = ($1229|0)!=($1233|0);
          $1235 = HEAP32[$v>>2]|0;
          if ($1234) {
           $1236 = HEAP32[$iKey>>2]|0;
           $1237 = HEAP32[$regRow>>2]|0;
           (_sqlite3VdbeAddOp3($1235,96,0,$1236,$1237)|0);
           $1238 = HEAP32[$v>>2]|0;
           $1239 = HEAP32[$pTab25>>2]|0;
           $1240 = HEAP32[$iKey>>2]|0;
           $1241 = HEAP32[$regRow>>2]|0;
           _sqlite3ColumnDefault($1238,$1239,$1240,$1241);
           $1242 = HEAP32[$v>>2]|0;
           $1243 = HEAP32[$regRow>>2]|0;
           $1244 = HEAP32[$addrOk>>2]|0;
           (_sqlite3VdbeAddOp2($1242,34,$1243,$1244)|0);
          } else {
           $1245 = HEAP32[$regRow>>2]|0;
           (_sqlite3VdbeAddOp2($1235,123,0,$1245)|0);
          }
          $1246 = HEAP32[$v>>2]|0;
          $1247 = HEAP32[$i27>>2]|0;
          $1248 = HEAP32[$regRow>>2]|0;
          (_sqlite3VdbeAddOp3($1246,32,$1247,0,$1248)|0);
          $1249 = HEAP32[$v>>2]|0;
          $1250 = HEAP32[$addrOk>>2]|0;
          (_sqlite3VdbeGoto($1249,$1250)|0);
          $1251 = HEAP32[$v>>2]|0;
          $1252 = HEAP32[$v>>2]|0;
          $1253 = (_sqlite3VdbeCurrentAddr($1252)|0);
          $1254 = (($1253) - 2)|0;
          _sqlite3VdbeJumpHere($1251,$1254);
         } else {
          HEAP32[$j28>>2] = 0;
          while(1) {
           $1255 = HEAP32[$j28>>2]|0;
           $1256 = HEAP32[$pFK24>>2]|0;
           $1257 = ((($1256)) + 20|0);
           $1258 = HEAP32[$1257>>2]|0;
           $1259 = ($1255|0)<($1258|0);
           if (!($1259)) {
            break;
           }
           $1260 = HEAP32[$v>>2]|0;
           $1261 = HEAP32[$pTab25>>2]|0;
           $1262 = HEAP32[$aiCols>>2]|0;
           $1263 = ($1262|0)!=(0|0);
           $1264 = HEAP32[$j28>>2]|0;
           if ($1263) {
            $1265 = HEAP32[$aiCols>>2]|0;
            $1266 = (($1265) + ($1264<<2)|0);
            $1267 = HEAP32[$1266>>2]|0;
            $1275 = $1267;
           } else {
            $1268 = HEAP32[$pFK24>>2]|0;
            $1269 = ((($1268)) + 36|0);
            $1270 = (($1269) + ($1264<<3)|0);
            $1271 = HEAP32[$1270>>2]|0;
            $1275 = $1271;
           }
           $1272 = HEAP32[$regRow>>2]|0;
           $1273 = HEAP32[$j28>>2]|0;
           $1274 = (($1272) + ($1273))|0;
           _sqlite3ExprCodeGetColumnOfTable($1260,$1261,0,$1275,$1274);
           $1276 = HEAP32[$v>>2]|0;
           $1277 = HEAP32[$regRow>>2]|0;
           $1278 = HEAP32[$j28>>2]|0;
           $1279 = (($1277) + ($1278))|0;
           $1280 = HEAP32[$addrOk>>2]|0;
           (_sqlite3VdbeAddOp2($1276,34,$1279,$1280)|0);
           $1281 = HEAP32[$j28>>2]|0;
           $1282 = (($1281) + 1)|0;
           HEAP32[$j28>>2] = $1282;
          }
          $1283 = HEAP32[$pParent>>2]|0;
          $1284 = ($1283|0)!=(0|0);
          if (!($1284)) {
           break;
          }
          $1285 = HEAP32[$v>>2]|0;
          $1286 = HEAP32[$regRow>>2]|0;
          $1287 = HEAP32[$pFK24>>2]|0;
          $1288 = ((($1287)) + 20|0);
          $1289 = HEAP32[$1288>>2]|0;
          $1290 = HEAP32[$regKey>>2]|0;
          $1291 = HEAP32[$db>>2]|0;
          $1292 = HEAP32[$pIdx26>>2]|0;
          $1293 = (_sqlite3IndexAffinityStr($1291,$1292)|0);
          $1294 = HEAP32[$pFK24>>2]|0;
          $1295 = ((($1294)) + 20|0);
          $1296 = HEAP32[$1295>>2]|0;
          (_sqlite3VdbeAddOp4($1285,99,$1286,$1289,$1290,$1293,$1296)|0);
          $1297 = HEAP32[$v>>2]|0;
          $1298 = HEAP32[$i27>>2]|0;
          $1299 = HEAP32[$addrOk>>2]|0;
          $1300 = HEAP32[$regKey>>2]|0;
          (_sqlite3VdbeAddOp4Int($1297,31,$1298,$1299,$1300,0)|0);
         }
        } while(0);
        $1301 = HEAP32[$v>>2]|0;
        $1302 = HEAP32[$regResult>>2]|0;
        $1303 = (($1302) + 1)|0;
        (_sqlite3VdbeAddOp2($1301,123,0,$1303)|0);
        $1304 = HEAP32[$v>>2]|0;
        $1305 = HEAP32[$regResult>>2]|0;
        $1306 = (($1305) + 2)|0;
        $1307 = HEAP32[$pFK24>>2]|0;
        $1308 = ((($1307)) + 8|0);
        $1309 = HEAP32[$1308>>2]|0;
        $1310 = HEAP32[$i27>>2]|0;
        $1311 = (($1310) - 1)|0;
        HEAP32[$vararg_buffer80>>2] = $1309;
        $vararg_ptr83 = ((($vararg_buffer80)) + 4|0);
        HEAP32[$vararg_ptr83>>2] = $1311;
        _sqlite3VdbeMultiLoad($1304,$1306,32474,$vararg_buffer80);
        $1312 = HEAP32[$v>>2]|0;
        $1313 = HEAP32[$regResult>>2]|0;
        (_sqlite3VdbeAddOp2($1312,87,$1313,4)|0);
        $1314 = HEAP32[$v>>2]|0;
        $1315 = HEAP32[$addrOk>>2]|0;
        _sqlite3VdbeResolveLabel($1314,$1315);
        $1316 = HEAP32[$db>>2]|0;
        $1317 = HEAP32[$aiCols>>2]|0;
        _sqlite3DbFree($1316,$1317);
        $1318 = HEAP32[$i27>>2]|0;
        $1319 = (($1318) + 1)|0;
        HEAP32[$i27>>2] = $1319;
        $1320 = HEAP32[$pFK24>>2]|0;
        $1321 = ((($1320)) + 4|0);
        $1322 = HEAP32[$1321>>2]|0;
        HEAP32[$pFK24>>2] = $1322;
       }
       $1323 = HEAP32[$v>>2]|0;
       $1324 = HEAP32[$addrTop>>2]|0;
       $1325 = (($1324) + 1)|0;
       (_sqlite3VdbeAddOp2($1323,7,0,$1325)|0);
       $1326 = HEAP32[$v>>2]|0;
       $1327 = HEAP32[$addrTop>>2]|0;
       _sqlite3VdbeJumpHere($1326,$1327);
      }
      break;
     }
     case 6:  {
      $1328 = HEAP32[$zRight>>2]|0;
      $1329 = ($1328|0)!=(0|0);
      if (!($1329)) {
       break L21;
      }
      $1330 = HEAP32[$db>>2]|0;
      $1331 = HEAP32[$zRight>>2]|0;
      $1332 = (_sqlite3GetBoolean($1331,0)|0);
      $1333 = $1332&255;
      _sqlite3RegisterLikeFunctions($1330,$1333);
      break L21;
      break;
     }
     case 18:  {
      $1334 = HEAP32[$zLeft>>2]|0;
      $1335 = HEAP8[$1334>>0]|0;
      $1336 = $1335&255;
      $1337 = (17613 + ($1336)|0);
      $1338 = HEAP8[$1337>>0]|0;
      $1339 = $1338&255;
      $1340 = ($1339|0)==(113);
      $1341 = $1340&1;
      HEAP32[$isQuick>>2] = $1341;
      $1342 = HEAP32[$2>>2]|0;
      $1343 = HEAP32[$1342>>2]|0;
      $1344 = ($1343|0)==(0|0);
      if ($1344) {
       HEAP32[$iDb>>2] = -1;
      }
      $1345 = HEAP32[$0>>2]|0;
      $1346 = ((($1345)) + 44|0);
      HEAP32[$1346>>2] = 6;
      $1347 = HEAP32[$v>>2]|0;
      _setOneColumnName($1347,32477);
      HEAP32[$mxErr>>2] = 100;
      $1348 = HEAP32[$zRight>>2]|0;
      $1349 = ($1348|0)!=(0|0);
      if ($1349) {
       $1350 = HEAP32[$zRight>>2]|0;
       (_sqlite3GetInt32($1350,$mxErr)|0);
       $1351 = HEAP32[$mxErr>>2]|0;
       $1352 = ($1351|0)<=(0);
       if ($1352) {
        HEAP32[$mxErr>>2] = 100;
       }
      }
      $1353 = HEAP32[$v>>2]|0;
      $1354 = HEAP32[$mxErr>>2]|0;
      (_sqlite3VdbeAddOp2($1353,76,$1354,1)|0);
      HEAP32[$i30>>2] = 0;
      L330: while(1) {
       $1355 = HEAP32[$i30>>2]|0;
       $1356 = HEAP32[$db>>2]|0;
       $1357 = ((($1356)) + 20|0);
       $1358 = HEAP32[$1357>>2]|0;
       $1359 = ($1355|0)<($1358|0);
       if (!($1359)) {
        break;
       }
       HEAP32[$cnt>>2] = 0;
       HEAP32[$mxIdx>>2] = 0;
       $1360 = HEAP32[$iDb>>2]|0;
       $1361 = ($1360|0)>=(0);
       if ($1361) {
        $1362 = HEAP32[$i30>>2]|0;
        $1363 = HEAP32[$iDb>>2]|0;
        $1364 = ($1362|0)!=($1363|0);
        if (!($1364)) {
         label = 242;
        }
       } else {
        label = 242;
       }
       L335: do {
        if ((label|0) == 242) {
         label = 0;
         $1365 = HEAP32[$0>>2]|0;
         $1366 = HEAP32[$i30>>2]|0;
         _sqlite3CodeVerifySchema($1365,$1366);
         $1367 = HEAP32[$v>>2]|0;
         $1368 = (_sqlite3VdbeAddOp1($1367,66,1)|0);
         HEAP32[$addr32>>2] = $1368;
         $1369 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp2($1369,75,0,0)|0);
         $1370 = HEAP32[$v>>2]|0;
         $1371 = HEAP32[$addr32>>2]|0;
         _sqlite3VdbeJumpHere($1370,$1371);
         $1372 = HEAP32[$i30>>2]|0;
         $1373 = HEAP32[$db>>2]|0;
         $1374 = ((($1373)) + 16|0);
         $1375 = HEAP32[$1374>>2]|0;
         $1376 = (($1375) + ($1372<<4)|0);
         $1377 = ((($1376)) + 12|0);
         $1378 = HEAP32[$1377>>2]|0;
         $1379 = ((($1378)) + 8|0);
         HEAP32[$pTbls>>2] = $1379;
         HEAP32[$cnt>>2] = 0;
         $1380 = HEAP32[$pTbls>>2]|0;
         $1381 = ((($1380)) + 8|0);
         $1382 = HEAP32[$1381>>2]|0;
         HEAP32[$x33>>2] = $1382;
         while(1) {
          $1383 = HEAP32[$x33>>2]|0;
          $1384 = ($1383|0)!=(0|0);
          if (!($1384)) {
           break;
          }
          $1385 = HEAP32[$x33>>2]|0;
          $1386 = ((($1385)) + 8|0);
          $1387 = HEAP32[$1386>>2]|0;
          HEAP32[$pTab34>>2] = $1387;
          $1388 = HEAP32[$pTab34>>2]|0;
          $1389 = ((($1388)) + 42|0);
          $1390 = HEAP8[$1389>>0]|0;
          $1391 = $1390&255;
          $1392 = $1391 & 32;
          $1393 = ($1392|0)==(0);
          if ($1393) {
           $1394 = HEAP32[$cnt>>2]|0;
           $1395 = (($1394) + 1)|0;
           HEAP32[$cnt>>2] = $1395;
          }
          HEAP32[$nIdx>>2] = 0;
          $1396 = HEAP32[$pTab34>>2]|0;
          $1397 = ((($1396)) + 8|0);
          $1398 = HEAP32[$1397>>2]|0;
          HEAP32[$pIdx35>>2] = $1398;
          while(1) {
           $1399 = HEAP32[$pIdx35>>2]|0;
           $1400 = ($1399|0)!=(0|0);
           if (!($1400)) {
            break;
           }
           $1401 = HEAP32[$cnt>>2]|0;
           $1402 = (($1401) + 1)|0;
           HEAP32[$cnt>>2] = $1402;
           $1403 = HEAP32[$pIdx35>>2]|0;
           $1404 = ((($1403)) + 20|0);
           $1405 = HEAP32[$1404>>2]|0;
           HEAP32[$pIdx35>>2] = $1405;
           $1406 = HEAP32[$nIdx>>2]|0;
           $1407 = (($1406) + 1)|0;
           HEAP32[$nIdx>>2] = $1407;
          }
          $1408 = HEAP32[$nIdx>>2]|0;
          $1409 = HEAP32[$mxIdx>>2]|0;
          $1410 = ($1408|0)>($1409|0);
          if ($1410) {
           $1411 = HEAP32[$nIdx>>2]|0;
           HEAP32[$mxIdx>>2] = $1411;
          }
          $1412 = HEAP32[$x33>>2]|0;
          $1413 = HEAP32[$1412>>2]|0;
          HEAP32[$x33>>2] = $1413;
         }
         $1414 = HEAP32[$db>>2]|0;
         $1415 = HEAP32[$cnt>>2]|0;
         $1416 = (($1415) + 1)|0;
         $1417 = $1416<<2;
         $1418 = (_sqlite3DbMallocRawNN($1414,$1417,0)|0);
         HEAP32[$aRoot>>2] = $1418;
         $1419 = HEAP32[$aRoot>>2]|0;
         $1420 = ($1419|0)==(0|0);
         if ($1420) {
          break L330;
         }
         HEAP32[$cnt>>2] = 0;
         $1421 = HEAP32[$pTbls>>2]|0;
         $1422 = ((($1421)) + 8|0);
         $1423 = HEAP32[$1422>>2]|0;
         HEAP32[$x33>>2] = $1423;
         while(1) {
          $1424 = HEAP32[$x33>>2]|0;
          $1425 = ($1424|0)!=(0|0);
          if (!($1425)) {
           break;
          }
          $1426 = HEAP32[$x33>>2]|0;
          $1427 = ((($1426)) + 8|0);
          $1428 = HEAP32[$1427>>2]|0;
          HEAP32[$pTab36>>2] = $1428;
          $1429 = HEAP32[$pTab36>>2]|0;
          $1430 = ((($1429)) + 42|0);
          $1431 = HEAP8[$1430>>0]|0;
          $1432 = $1431&255;
          $1433 = $1432 & 32;
          $1434 = ($1433|0)==(0);
          if ($1434) {
           $1435 = HEAP32[$pTab36>>2]|0;
           $1436 = ((($1435)) + 28|0);
           $1437 = HEAP32[$1436>>2]|0;
           $1438 = HEAP32[$cnt>>2]|0;
           $1439 = (($1438) + 1)|0;
           HEAP32[$cnt>>2] = $1439;
           $1440 = HEAP32[$aRoot>>2]|0;
           $1441 = (($1440) + ($1438<<2)|0);
           HEAP32[$1441>>2] = $1437;
          }
          $1442 = HEAP32[$pTab36>>2]|0;
          $1443 = ((($1442)) + 8|0);
          $1444 = HEAP32[$1443>>2]|0;
          HEAP32[$pIdx37>>2] = $1444;
          while(1) {
           $1445 = HEAP32[$pIdx37>>2]|0;
           $1446 = ($1445|0)!=(0|0);
           if (!($1446)) {
            break;
           }
           $1447 = HEAP32[$pIdx37>>2]|0;
           $1448 = ((($1447)) + 44|0);
           $1449 = HEAP32[$1448>>2]|0;
           $1450 = HEAP32[$cnt>>2]|0;
           $1451 = (($1450) + 1)|0;
           HEAP32[$cnt>>2] = $1451;
           $1452 = HEAP32[$aRoot>>2]|0;
           $1453 = (($1452) + ($1450<<2)|0);
           HEAP32[$1453>>2] = $1449;
           $1454 = HEAP32[$pIdx37>>2]|0;
           $1455 = ((($1454)) + 20|0);
           $1456 = HEAP32[$1455>>2]|0;
           HEAP32[$pIdx37>>2] = $1456;
          }
          $1457 = HEAP32[$x33>>2]|0;
          $1458 = HEAP32[$1457>>2]|0;
          HEAP32[$x33>>2] = $1458;
         }
         $1459 = HEAP32[$cnt>>2]|0;
         $1460 = HEAP32[$aRoot>>2]|0;
         $1461 = (($1460) + ($1459<<2)|0);
         HEAP32[$1461>>2] = 0;
         $1462 = HEAP32[$0>>2]|0;
         $1463 = ((($1462)) + 44|0);
         $1464 = HEAP32[$1463>>2]|0;
         $1465 = HEAP32[$mxIdx>>2]|0;
         $1466 = (8 + ($1465))|0;
         $1467 = ($1464|0)>($1466|0);
         if ($1467) {
          $1468 = HEAP32[$0>>2]|0;
          $1469 = ((($1468)) + 44|0);
          $1470 = HEAP32[$1469>>2]|0;
          $1475 = $1470;
         } else {
          $1471 = HEAP32[$mxIdx>>2]|0;
          $1472 = (8 + ($1471))|0;
          $1475 = $1472;
         }
         $1473 = HEAP32[$0>>2]|0;
         $1474 = ((($1473)) + 44|0);
         HEAP32[$1474>>2] = $1475;
         $1476 = HEAP32[$v>>2]|0;
         $1477 = HEAP32[$cnt>>2]|0;
         $1478 = HEAP32[$aRoot>>2]|0;
         (_sqlite3VdbeAddOp4($1476,141,2,$1477,1,$1478,-15)|0);
         $1479 = HEAP32[$v>>2]|0;
         $1480 = HEAP32[$i30>>2]|0;
         $1481 = $1480&255;
         _sqlite3VdbeChangeP5($1479,$1481);
         $1482 = HEAP32[$v>>2]|0;
         $1483 = (_sqlite3VdbeAddOp1($1482,34,2)|0);
         HEAP32[$addr32>>2] = $1483;
         $1484 = HEAP32[$v>>2]|0;
         $1485 = HEAP32[$db>>2]|0;
         $1486 = HEAP32[$i30>>2]|0;
         $1487 = HEAP32[$db>>2]|0;
         $1488 = ((($1487)) + 16|0);
         $1489 = HEAP32[$1488>>2]|0;
         $1490 = (($1489) + ($1486<<4)|0);
         $1491 = HEAP32[$1490>>2]|0;
         HEAP32[$vararg_buffer84>>2] = $1491;
         $1492 = (_sqlite3MPrintf($1485,32493,$vararg_buffer84)|0);
         (_sqlite3VdbeAddOp4($1484,97,0,3,0,$1492,-1)|0);
         $1493 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp3($1493,83,2,4,1)|0);
         $1494 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp3($1494,52,4,3,2)|0);
         $1495 = HEAP32[$v>>2]|0;
         (_sqlite3VdbeAddOp2($1495,87,2,1)|0);
         $1496 = HEAP32[$v>>2]|0;
         $1497 = HEAP32[$addr32>>2]|0;
         _sqlite3VdbeJumpHere($1496,$1497);
         $1498 = HEAP32[$pTbls>>2]|0;
         $1499 = ((($1498)) + 8|0);
         $1500 = HEAP32[$1499>>2]|0;
         HEAP32[$x33>>2] = $1500;
         while(1) {
          $1501 = HEAP32[$x33>>2]|0;
          $1502 = ($1501|0)!=(0|0);
          if (!($1502)) {
           break L335;
          }
          $1503 = HEAP32[$isQuick>>2]|0;
          $1504 = ($1503|0)!=(0);
          $1505 = $1504 ^ 1;
          if (!($1505)) {
           break L335;
          }
          $1506 = HEAP32[$x33>>2]|0;
          $1507 = ((($1506)) + 8|0);
          $1508 = HEAP32[$1507>>2]|0;
          HEAP32[$pTab38>>2] = $1508;
          HEAP32[$pPrior>>2] = 0;
          HEAP32[$r1>>2] = -1;
          $1509 = HEAP32[$pTab38>>2]|0;
          $1510 = ((($1509)) + 8|0);
          $1511 = HEAP32[$1510>>2]|0;
          $1512 = ($1511|0)==(0|0);
          L371: do {
           if (!($1512)) {
            $1513 = HEAP32[$pTab38>>2]|0;
            $1514 = ((($1513)) + 42|0);
            $1515 = HEAP8[$1514>>0]|0;
            $1516 = $1515&255;
            $1517 = $1516 & 32;
            $1518 = ($1517|0)==(0);
            if ($1518) {
             $1521 = 0;
            } else {
             $1519 = HEAP32[$pTab38>>2]|0;
             $1520 = (_sqlite3PrimaryKeyIndex($1519)|0);
             $1521 = $1520;
            }
            HEAP32[$pPk40>>2] = $1521;
            $1522 = HEAP32[$v>>2]|0;
            $1523 = (_sqlite3VdbeAddOp1($1522,66,1)|0);
            HEAP32[$addr32>>2] = $1523;
            $1524 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp2($1524,75,0,0)|0);
            $1525 = HEAP32[$v>>2]|0;
            $1526 = HEAP32[$addr32>>2]|0;
            _sqlite3VdbeJumpHere($1525,$1526);
            $1527 = HEAP32[$0>>2]|0;
            _sqlite3ExprCacheClear($1527);
            $1528 = HEAP32[$0>>2]|0;
            $1529 = HEAP32[$pTab38>>2]|0;
            (_sqlite3OpenTableAndIndices($1528,$1529,104,0,1,0,$iDataCur,$iIdxCur)|0);
            $1530 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeAddOp2($1530,76,0,7)|0);
            HEAP32[$j31>>2] = 0;
            $1531 = HEAP32[$pTab38>>2]|0;
            $1532 = ((($1531)) + 8|0);
            $1533 = HEAP32[$1532>>2]|0;
            HEAP32[$pIdx39>>2] = $1533;
            while(1) {
             $1534 = HEAP32[$pIdx39>>2]|0;
             $1535 = ($1534|0)!=(0|0);
             $1536 = HEAP32[$v>>2]|0;
             if (!($1535)) {
              break;
             }
             $1537 = HEAP32[$j31>>2]|0;
             $1538 = (8 + ($1537))|0;
             (_sqlite3VdbeAddOp2($1536,76,0,$1538)|0);
             $1539 = HEAP32[$pIdx39>>2]|0;
             $1540 = ((($1539)) + 20|0);
             $1541 = HEAP32[$1540>>2]|0;
             HEAP32[$pIdx39>>2] = $1541;
             $1542 = HEAP32[$j31>>2]|0;
             $1543 = (($1542) + 1)|0;
             HEAP32[$j31>>2] = $1543;
            }
            $1544 = HEAP32[$iDataCur>>2]|0;
            (_sqlite3VdbeAddOp2($1536,57,$1544,0)|0);
            $1545 = HEAP32[$v>>2]|0;
            $1546 = (_sqlite3VdbeAddOp2($1545,91,7,1)|0);
            HEAP32[$loopTop>>2] = $1546;
            HEAP32[$j31>>2] = 0;
            while(1) {
             $1547 = HEAP32[$j31>>2]|0;
             $1548 = HEAP32[$pTab38>>2]|0;
             $1549 = ((($1548)) + 34|0);
             $1550 = HEAP16[$1549>>1]|0;
             $1551 = $1550 << 16 >> 16;
             $1552 = ($1547|0)<($1551|0);
             if (!($1552)) {
              break;
             }
             $1553 = HEAP32[$j31>>2]|0;
             $1554 = HEAP32[$pTab38>>2]|0;
             $1555 = ((($1554)) + 32|0);
             $1556 = HEAP16[$1555>>1]|0;
             $1557 = $1556 << 16 >> 16;
             $1558 = ($1553|0)==($1557|0);
             do {
              if (!($1558)) {
               $1559 = HEAP32[$j31>>2]|0;
               $1560 = HEAP32[$pTab38>>2]|0;
               $1561 = ((($1560)) + 4|0);
               $1562 = HEAP32[$1561>>2]|0;
               $1563 = (($1562) + ($1559<<4)|0);
               $1564 = ((($1563)) + 12|0);
               $1565 = HEAP8[$1564>>0]|0;
               $1566 = $1565&255;
               $1567 = ($1566|0)==(0);
               if ($1567) {
                break;
               }
               $1568 = HEAP32[$v>>2]|0;
               $1569 = HEAP32[$pTab38>>2]|0;
               $1570 = HEAP32[$iDataCur>>2]|0;
               $1571 = HEAP32[$j31>>2]|0;
               _sqlite3ExprCodeGetColumnOfTable($1568,$1569,$1570,$1571,3);
               $1572 = HEAP32[$v>>2]|0;
               _sqlite3VdbeChangeP5($1572,-128);
               $1573 = HEAP32[$v>>2]|0;
               $1574 = (_sqlite3VdbeAddOp1($1573,35,3)|0);
               HEAP32[$jmp2>>2] = $1574;
               $1575 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1575,91,1,-1)|0);
               $1576 = HEAP32[$db>>2]|0;
               $1577 = HEAP32[$pTab38>>2]|0;
               $1578 = HEAP32[$1577>>2]|0;
               $1579 = HEAP32[$j31>>2]|0;
               $1580 = HEAP32[$pTab38>>2]|0;
               $1581 = ((($1580)) + 4|0);
               $1582 = HEAP32[$1581>>2]|0;
               $1583 = (($1582) + ($1579<<4)|0);
               $1584 = HEAP32[$1583>>2]|0;
               HEAP32[$vararg_buffer87>>2] = $1578;
               $vararg_ptr90 = ((($vararg_buffer87)) + 4|0);
               HEAP32[$vararg_ptr90>>2] = $1584;
               $1585 = (_sqlite3MPrintf($1576,32517,$vararg_buffer87)|0);
               HEAP32[$zErr>>2] = $1585;
               $1586 = HEAP32[$v>>2]|0;
               $1587 = HEAP32[$zErr>>2]|0;
               (_sqlite3VdbeAddOp4($1586,97,0,3,0,$1587,-1)|0);
               $1588 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1588,87,3,1)|0);
               $1589 = HEAP32[$v>>2]|0;
               $1590 = (_sqlite3VdbeAddOp1($1589,66,1)|0);
               HEAP32[$jmp3>>2] = $1590;
               $1591 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp0($1591,75)|0);
               $1592 = HEAP32[$v>>2]|0;
               $1593 = HEAP32[$jmp2>>2]|0;
               _sqlite3VdbeJumpHere($1592,$1593);
               $1594 = HEAP32[$v>>2]|0;
               $1595 = HEAP32[$jmp3>>2]|0;
               _sqlite3VdbeJumpHere($1594,$1595);
              }
             } while(0);
             $1596 = HEAP32[$j31>>2]|0;
             $1597 = (($1596) + 1)|0;
             HEAP32[$j31>>2] = $1597;
            }
            HEAP32[$j31>>2] = 0;
            $1598 = HEAP32[$pTab38>>2]|0;
            $1599 = ((($1598)) + 8|0);
            $1600 = HEAP32[$1599>>2]|0;
            HEAP32[$pIdx39>>2] = $1600;
            while(1) {
             $1601 = HEAP32[$pIdx39>>2]|0;
             $1602 = ($1601|0)!=(0|0);
             $1603 = HEAP32[$v>>2]|0;
             if (!($1602)) {
              break;
             }
             $1604 = (_sqlite3VdbeMakeLabel($1603)|0);
             HEAP32[$ckUniq>>2] = $1604;
             $1605 = HEAP32[$pPk40>>2]|0;
             $1606 = HEAP32[$pIdx39>>2]|0;
             $1607 = ($1605|0)==($1606|0);
             if (!($1607)) {
              $1608 = HEAP32[$0>>2]|0;
              $1609 = HEAP32[$pIdx39>>2]|0;
              $1610 = HEAP32[$iDataCur>>2]|0;
              $1611 = HEAP32[$pPrior>>2]|0;
              $1612 = HEAP32[$r1>>2]|0;
              $1613 = (_sqlite3GenerateIndexKey($1608,$1609,$1610,0,0,$jmp342,$1611,$1612)|0);
              HEAP32[$r1>>2] = $1613;
              $1614 = HEAP32[$pIdx39>>2]|0;
              HEAP32[$pPrior>>2] = $1614;
              $1615 = HEAP32[$v>>2]|0;
              $1616 = HEAP32[$j31>>2]|0;
              $1617 = (8 + ($1616))|0;
              (_sqlite3VdbeAddOp2($1615,91,$1617,1)|0);
              $1618 = HEAP32[$v>>2]|0;
              $1619 = HEAP32[$iIdxCur>>2]|0;
              $1620 = HEAP32[$j31>>2]|0;
              $1621 = (($1619) + ($1620))|0;
              $1622 = HEAP32[$ckUniq>>2]|0;
              $1623 = HEAP32[$r1>>2]|0;
              $1624 = HEAP32[$pIdx39>>2]|0;
              $1625 = ((($1624)) + 52|0);
              $1626 = HEAP16[$1625>>1]|0;
              $1627 = $1626&65535;
              $1628 = (_sqlite3VdbeAddOp4Int($1618,31,$1621,$1622,$1623,$1627)|0);
              HEAP32[$jmp241>>2] = $1628;
              $1629 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1629,91,1,-1)|0);
              $1630 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeLoadString($1630,3,32537)|0);
              $1631 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1631,52,7,3,3)|0);
              $1632 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeLoadString($1632,4,32542)|0);
              $1633 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1633,52,4,3,3)|0);
              $1634 = HEAP32[$v>>2]|0;
              $1635 = HEAP32[$pIdx39>>2]|0;
              $1636 = HEAP32[$1635>>2]|0;
              $1637 = (_sqlite3VdbeLoadString($1634,4,$1636)|0);
              HEAP32[$jmp5>>2] = $1637;
              $1638 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1638,52,4,3,3)|0);
              $1639 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1639,87,3,1)|0);
              $1640 = HEAP32[$v>>2]|0;
              $1641 = (_sqlite3VdbeAddOp1($1640,66,1)|0);
              HEAP32[$jmp4>>2] = $1641;
              $1642 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp0($1642,75)|0);
              $1643 = HEAP32[$v>>2]|0;
              $1644 = HEAP32[$jmp241>>2]|0;
              _sqlite3VdbeJumpHere($1643,$1644);
              $1645 = HEAP32[$pIdx39>>2]|0;
              $1646 = ((($1645)) + 54|0);
              $1647 = HEAP8[$1646>>0]|0;
              $1648 = $1647&255;
              $1649 = ($1648|0)!=(0);
              if ($1649) {
               $1650 = HEAP32[$v>>2]|0;
               $1651 = (_sqlite3VdbeMakeLabel($1650)|0);
               HEAP32[$uniqOk>>2] = $1651;
               HEAP32[$kk>>2] = 0;
               while(1) {
                $1652 = HEAP32[$kk>>2]|0;
                $1653 = HEAP32[$pIdx39>>2]|0;
                $1654 = ((($1653)) + 50|0);
                $1655 = HEAP16[$1654>>1]|0;
                $1656 = $1655&65535;
                $1657 = ($1652|0)<($1656|0);
                if (!($1657)) {
                 break;
                }
                $1658 = HEAP32[$kk>>2]|0;
                $1659 = HEAP32[$pIdx39>>2]|0;
                $1660 = ((($1659)) + 4|0);
                $1661 = HEAP32[$1660>>2]|0;
                $1662 = (($1661) + ($1658<<1)|0);
                $1663 = HEAP16[$1662>>1]|0;
                $1664 = $1663 << 16 >> 16;
                HEAP32[$iCol>>2] = $1664;
                $1665 = HEAP32[$iCol>>2]|0;
                $1666 = ($1665|0)>=(0);
                if ($1666) {
                 $1667 = HEAP32[$iCol>>2]|0;
                 $1668 = HEAP32[$pTab38>>2]|0;
                 $1669 = ((($1668)) + 4|0);
                 $1670 = HEAP32[$1669>>2]|0;
                 $1671 = (($1670) + ($1667<<4)|0);
                 $1672 = ((($1671)) + 12|0);
                 $1673 = HEAP8[$1672>>0]|0;
                 $1674 = $1673&255;
                 $1675 = ($1674|0)!=(0);
                 if (!($1675)) {
                  label = 287;
                 }
                } else {
                 label = 287;
                }
                if ((label|0) == 287) {
                 label = 0;
                 $1676 = HEAP32[$v>>2]|0;
                 $1677 = HEAP32[$r1>>2]|0;
                 $1678 = HEAP32[$kk>>2]|0;
                 $1679 = (($1677) + ($1678))|0;
                 $1680 = HEAP32[$uniqOk>>2]|0;
                 (_sqlite3VdbeAddOp2($1676,34,$1679,$1680)|0);
                }
                $1681 = HEAP32[$kk>>2]|0;
                $1682 = (($1681) + 1)|0;
                HEAP32[$kk>>2] = $1682;
               }
               $1683 = HEAP32[$v>>2]|0;
               $1684 = HEAP32[$iIdxCur>>2]|0;
               $1685 = HEAP32[$j31>>2]|0;
               $1686 = (($1684) + ($1685))|0;
               $1687 = (_sqlite3VdbeAddOp1($1683,7,$1686)|0);
               HEAP32[$jmp6>>2] = $1687;
               $1688 = HEAP32[$v>>2]|0;
               $1689 = HEAP32[$uniqOk>>2]|0;
               (_sqlite3VdbeGoto($1688,$1689)|0);
               $1690 = HEAP32[$v>>2]|0;
               $1691 = HEAP32[$jmp6>>2]|0;
               _sqlite3VdbeJumpHere($1690,$1691);
               $1692 = HEAP32[$v>>2]|0;
               $1693 = HEAP32[$iIdxCur>>2]|0;
               $1694 = HEAP32[$j31>>2]|0;
               $1695 = (($1693) + ($1694))|0;
               $1696 = HEAP32[$uniqOk>>2]|0;
               $1697 = HEAP32[$r1>>2]|0;
               $1698 = HEAP32[$pIdx39>>2]|0;
               $1699 = ((($1698)) + 50|0);
               $1700 = HEAP16[$1699>>1]|0;
               $1701 = $1700&65535;
               (_sqlite3VdbeAddOp4Int($1692,59,$1695,$1696,$1697,$1701)|0);
               $1702 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeAddOp2($1702,91,1,-1)|0);
               $1703 = HEAP32[$v>>2]|0;
               (_sqlite3VdbeLoadString($1703,3,32563)|0);
               $1704 = HEAP32[$v>>2]|0;
               $1705 = HEAP32[$jmp5>>2]|0;
               (_sqlite3VdbeGoto($1704,$1705)|0);
               $1706 = HEAP32[$v>>2]|0;
               $1707 = HEAP32[$uniqOk>>2]|0;
               _sqlite3VdbeResolveLabel($1706,$1707);
              }
              $1708 = HEAP32[$v>>2]|0;
              $1709 = HEAP32[$jmp4>>2]|0;
              _sqlite3VdbeJumpHere($1708,$1709);
              $1710 = HEAP32[$0>>2]|0;
              $1711 = HEAP32[$jmp342>>2]|0;
              _sqlite3ResolvePartIdxLabel($1710,$1711);
             }
             $1712 = HEAP32[$pIdx39>>2]|0;
             $1713 = ((($1712)) + 20|0);
             $1714 = HEAP32[$1713>>2]|0;
             HEAP32[$pIdx39>>2] = $1714;
             $1715 = HEAP32[$j31>>2]|0;
             $1716 = (($1715) + 1)|0;
             HEAP32[$j31>>2] = $1716;
            }
            $1717 = HEAP32[$iDataCur>>2]|0;
            $1718 = HEAP32[$loopTop>>2]|0;
            (_sqlite3VdbeAddOp2($1603,7,$1717,$1718)|0);
            $1719 = HEAP32[$v>>2]|0;
            $1720 = HEAP32[$loopTop>>2]|0;
            $1721 = (($1720) - 1)|0;
            _sqlite3VdbeJumpHere($1719,$1721);
            $1722 = HEAP32[$v>>2]|0;
            (_sqlite3VdbeLoadString($1722,2,32590)|0);
            HEAP32[$j31>>2] = 0;
            $1723 = HEAP32[$pTab38>>2]|0;
            $1724 = ((($1723)) + 8|0);
            $1725 = HEAP32[$1724>>2]|0;
            HEAP32[$pIdx39>>2] = $1725;
            while(1) {
             $1726 = HEAP32[$pIdx39>>2]|0;
             $1727 = ($1726|0)!=(0|0);
             if (!($1727)) {
              break L371;
             }
             $1728 = HEAP32[$pPk40>>2]|0;
             $1729 = HEAP32[$pIdx39>>2]|0;
             $1730 = ($1728|0)==($1729|0);
             if (!($1730)) {
              $1731 = HEAP32[$v>>2]|0;
              $1732 = (_sqlite3VdbeCurrentAddr($1731)|0);
              HEAP32[$addr32>>2] = $1732;
              $1733 = HEAP32[$v>>2]|0;
              $1734 = HEAP32[$addr32>>2]|0;
              $1735 = (($1734) + 2)|0;
              (_sqlite3VdbeAddOp2($1733,66,1,$1735)|0);
              $1736 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1736,75,0,0)|0);
              $1737 = HEAP32[$v>>2]|0;
              $1738 = HEAP32[$iIdxCur>>2]|0;
              $1739 = HEAP32[$j31>>2]|0;
              $1740 = (($1738) + ($1739))|0;
              (_sqlite3VdbeAddOp2($1737,100,$1740,3)|0);
              $1741 = HEAP32[$v>>2]|0;
              $1742 = HEAP32[$j31>>2]|0;
              $1743 = (8 + ($1742))|0;
              $1744 = HEAP32[$addr32>>2]|0;
              $1745 = (($1744) + 8)|0;
              (_sqlite3VdbeAddOp3($1741,37,$1743,$1745,3)|0);
              $1746 = HEAP32[$v>>2]|0;
              _sqlite3VdbeChangeP5($1746,-112);
              $1747 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1747,91,1,-1)|0);
              $1748 = HEAP32[$v>>2]|0;
              $1749 = HEAP32[$pIdx39>>2]|0;
              $1750 = HEAP32[$1749>>2]|0;
              (_sqlite3VdbeLoadString($1748,3,$1750)|0);
              $1751 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp3($1751,52,3,2,7)|0);
              $1752 = HEAP32[$v>>2]|0;
              (_sqlite3VdbeAddOp2($1752,87,7,1)|0);
             }
             $1753 = HEAP32[$pIdx39>>2]|0;
             $1754 = ((($1753)) + 20|0);
             $1755 = HEAP32[$1754>>2]|0;
             HEAP32[$pIdx39>>2] = $1755;
             $1756 = HEAP32[$j31>>2]|0;
             $1757 = (($1756) + 1)|0;
             HEAP32[$j31>>2] = $1757;
            }
           }
          } while(0);
          $1758 = HEAP32[$x33>>2]|0;
          $1759 = HEAP32[$1758>>2]|0;
          HEAP32[$x33>>2] = $1759;
         }
        }
       } while(0);
       $1760 = HEAP32[$i30>>2]|0;
       $1761 = (($1760) + 1)|0;
       HEAP32[$i30>>2] = $1761;
      }
      $1762 = HEAP32[$v>>2]|0;
      $1763 = (_sqlite3VdbeAddOpList($1762,4,32619,0)|0);
      HEAP32[$aOp43>>2] = $1763;
      $1764 = HEAP32[$aOp43>>2]|0;
      $1765 = ($1764|0)!=(0|0);
      if (!($1765)) {
       break L21;
      }
      $1766 = HEAP32[$mxErr>>2]|0;
      $1767 = (0 - ($1766))|0;
      $1768 = HEAP32[$aOp43>>2]|0;
      $1769 = ((($1768)) + 8|0);
      HEAP32[$1769>>2] = $1767;
      $1770 = HEAP32[$aOp43>>2]|0;
      $1771 = ((($1770)) + 40|0);
      $1772 = ((($1771)) + 1|0);
      HEAP8[$1772>>0] = -2;
      $1773 = HEAP32[$aOp43>>2]|0;
      $1774 = ((($1773)) + 40|0);
      $1775 = ((($1774)) + 16|0);
      HEAP32[$1775>>2] = 32635;
      break L21;
      break;
     }
     case 12:  {
      $1776 = HEAP32[$zRight>>2]|0;
      $1777 = ($1776|0)!=(0|0);
      if (!($1777)) {
       $1778 = HEAP32[$0>>2]|0;
       $1779 = (_sqlite3ReadSchema($1778)|0);
       $1780 = ($1779|0)!=(0);
       if ($1780) {
        break L21;
       }
       $1781 = HEAP32[$v>>2]|0;
       $1782 = HEAP32[$0>>2]|0;
       $1783 = HEAP32[$1782>>2]|0;
       $1784 = ((($1783)) + 66|0);
       $1785 = HEAP8[$1784>>0]|0;
       $1786 = $1785&255;
       $1787 = (5336 + ($1786<<3)|0);
       $1788 = HEAP32[$1787>>2]|0;
       _returnSingleText($1781,32638,$1788);
       break L21;
      }
      $1789 = HEAP32[$db>>2]|0;
      $1790 = ((($1789)) + 16|0);
      $1791 = HEAP32[$1790>>2]|0;
      $1792 = ((($1791)) + 12|0);
      $1793 = HEAP32[$1792>>2]|0;
      $1794 = ((($1793)) + 78|0);
      $1795 = HEAP16[$1794>>1]|0;
      $1796 = $1795&65535;
      $1797 = $1796 & 1;
      $1798 = ($1797|0)==(1);
      if ($1798) {
       $1799 = HEAP32[$db>>2]|0;
       $1800 = ((($1799)) + 16|0);
       $1801 = HEAP32[$1800>>2]|0;
       $1802 = ((($1801)) + 12|0);
       $1803 = HEAP32[$1802>>2]|0;
       $1804 = ((($1803)) + 78|0);
       $1805 = HEAP16[$1804>>1]|0;
       $1806 = $1805&65535;
       $1807 = $1806 & 4;
       $1808 = ($1807|0)==(4);
       if (!($1808)) {
        break L21;
       }
      }
      HEAP32[$pEnc>>2] = 5336;
      while(1) {
       $1809 = HEAP32[$pEnc>>2]|0;
       $1810 = HEAP32[$1809>>2]|0;
       $1811 = ($1810|0)!=(0|0);
       if (!($1811)) {
        break;
       }
       $1812 = HEAP32[$zRight>>2]|0;
       $1813 = HEAP32[$pEnc>>2]|0;
       $1814 = HEAP32[$1813>>2]|0;
       $1815 = (_sqlite3StrICmp($1812,$1814)|0);
       $1816 = (0)==($1815|0);
       $1817 = HEAP32[$pEnc>>2]|0;
       if ($1816) {
        label = 309;
        break;
       }
       $1840 = ((($1817)) + 8|0);
       HEAP32[$pEnc>>2] = $1840;
      }
      if ((label|0) == 309) {
       $1818 = ((($1817)) + 4|0);
       $1819 = HEAP8[$1818>>0]|0;
       $1820 = $1819&255;
       $1821 = ($1820|0)!=(0);
       if ($1821) {
        $1822 = HEAP32[$pEnc>>2]|0;
        $1823 = ((($1822)) + 4|0);
        $1824 = HEAP8[$1823>>0]|0;
        $1825 = $1824&255;
        $1831 = $1825;
       } else {
        $1826 = HEAP8[968>>0]|0;
        $1827 = $1826 << 24 >> 24;
        $1828 = ($1827|0)==(0);
        $1829 = $1828 ? 3 : 2;
        $1831 = $1829;
       }
       $1830 = $1831&255;
       $1832 = HEAP32[$db>>2]|0;
       $1833 = ((($1832)) + 66|0);
       HEAP8[$1833>>0] = $1830;
       $1834 = HEAP32[$db>>2]|0;
       $1835 = ((($1834)) + 16|0);
       $1836 = HEAP32[$1835>>2]|0;
       $1837 = ((($1836)) + 12|0);
       $1838 = HEAP32[$1837>>2]|0;
       $1839 = ((($1838)) + 77|0);
       HEAP8[$1839>>0] = $1830;
      }
      $1841 = HEAP32[$pEnc>>2]|0;
      $1842 = HEAP32[$1841>>2]|0;
      $1843 = ($1842|0)!=(0|0);
      if ($1843) {
       break L21;
      }
      $1844 = HEAP32[$0>>2]|0;
      $1845 = HEAP32[$zRight>>2]|0;
      HEAP32[$vararg_buffer91>>2] = $1845;
      _sqlite3ErrorMsg($1844,32647,$vararg_buffer91);
      break L21;
      break;
     }
     case 0:  {
      $1846 = HEAP32[$pPragma>>2]|0;
      $1847 = ((($1846)) + 8|0);
      $1848 = HEAP32[$1847>>2]|0;
      HEAP32[$iCookie>>2] = $1848;
      $1849 = HEAP32[$v>>2]|0;
      $1850 = HEAP32[$iDb>>2]|0;
      _sqlite3VdbeUsesBtree($1849,$1850);
      $1851 = HEAP32[$zRight>>2]|0;
      $1852 = ($1851|0)!=(0|0);
      if ($1852) {
       $1853 = HEAP32[$pPragma>>2]|0;
       $1854 = ((($1853)) + 5|0);
       $1855 = HEAP8[$1854>>0]|0;
       $1856 = $1855&255;
       $1857 = $1856 & 2;
       $1858 = ($1857|0)==(0);
       if ($1858) {
        $1859 = HEAP32[$v>>2]|0;
        $1860 = (_sqlite3VdbeAddOpList($1859,2,32672,0)|0);
        HEAP32[$aOp44>>2] = $1860;
        $1861 = HEAP32[$iDb>>2]|0;
        $1862 = HEAP32[$aOp44>>2]|0;
        $1863 = ((($1862)) + 4|0);
        HEAP32[$1863>>2] = $1861;
        $1864 = HEAP32[$iDb>>2]|0;
        $1865 = HEAP32[$aOp44>>2]|0;
        $1866 = ((($1865)) + 20|0);
        $1867 = ((($1866)) + 4|0);
        HEAP32[$1867>>2] = $1864;
        $1868 = HEAP32[$iCookie>>2]|0;
        $1869 = HEAP32[$aOp44>>2]|0;
        $1870 = ((($1869)) + 20|0);
        $1871 = ((($1870)) + 8|0);
        HEAP32[$1871>>2] = $1868;
        $1872 = HEAP32[$zRight>>2]|0;
        $1873 = (_sqlite3Atoi($1872)|0);
        $1874 = HEAP32[$aOp44>>2]|0;
        $1875 = ((($1874)) + 20|0);
        $1876 = ((($1875)) + 12|0);
        HEAP32[$1876>>2] = $1873;
        break L21;
       }
      }
      $1877 = HEAP32[$v>>2]|0;
      $1878 = (_sqlite3VdbeAddOpList($1877,3,32680,0)|0);
      HEAP32[$aOp45>>2] = $1878;
      $1879 = HEAP32[$iDb>>2]|0;
      $1880 = HEAP32[$aOp45>>2]|0;
      $1881 = ((($1880)) + 4|0);
      HEAP32[$1881>>2] = $1879;
      $1882 = HEAP32[$iDb>>2]|0;
      $1883 = HEAP32[$aOp45>>2]|0;
      $1884 = ((($1883)) + 20|0);
      $1885 = ((($1884)) + 4|0);
      HEAP32[$1885>>2] = $1882;
      $1886 = HEAP32[$iCookie>>2]|0;
      $1887 = HEAP32[$aOp45>>2]|0;
      $1888 = ((($1887)) + 20|0);
      $1889 = ((($1888)) + 12|0);
      HEAP32[$1889>>2] = $1886;
      $1890 = HEAP32[$v>>2]|0;
      _sqlite3VdbeSetNumCols($1890,1);
      $1891 = HEAP32[$v>>2]|0;
      $1892 = HEAP32[$zLeft>>2]|0;
      (_sqlite3VdbeSetColName($1891,0,0,$1892,(-1))|0);
      $1893 = HEAP32[$v>>2]|0;
      _sqlite3VdbeReusable($1893);
      break L21;
      break;
     }
     case 8:  {
      HEAP32[$i46>>2] = 0;
      $1894 = HEAP32[$0>>2]|0;
      $1895 = ((($1894)) + 44|0);
      HEAP32[$1895>>2] = 1;
      $1896 = HEAP32[$v>>2]|0;
      _setOneColumnName($1896,32692);
      while(1) {
       $1897 = HEAP32[$i46>>2]|0;
       $1898 = (($1897) + 1)|0;
       HEAP32[$i46>>2] = $1898;
       $1899 = (_sqlite3_compileoption_get($1897)|0);
       HEAP32[$zOpt>>2] = $1899;
       $1900 = ($1899|0)!=(0|0);
       $1901 = HEAP32[$v>>2]|0;
       if (!($1900)) {
        break;
       }
       $1902 = HEAP32[$zOpt>>2]|0;
       (_sqlite3VdbeLoadString($1901,1,$1902)|0);
       $1903 = HEAP32[$v>>2]|0;
       (_sqlite3VdbeAddOp2($1903,87,1,1)|0);
      }
      _sqlite3VdbeReusable($1901);
      break L21;
      break;
     }
     case 36:  {
      $1904 = HEAP32[$2>>2]|0;
      $1905 = HEAP32[$1904>>2]|0;
      $1906 = ($1905|0)!=(0|0);
      $1907 = HEAP32[$iDb>>2]|0;
      $1908 = $1906 ? $1907 : 10;
      HEAP32[$iBt>>2] = $1908;
      HEAP32[$eMode47>>2] = 0;
      $1909 = HEAP32[$zRight>>2]|0;
      $1910 = ($1909|0)!=(0|0);
      do {
       if ($1910) {
        $1911 = HEAP32[$zRight>>2]|0;
        $1912 = (_sqlite3StrICmp($1911,32707)|0);
        $1913 = ($1912|0)==(0);
        if ($1913) {
         HEAP32[$eMode47>>2] = 1;
         break;
        }
        $1914 = HEAP32[$zRight>>2]|0;
        $1915 = (_sqlite3StrICmp($1914,32712)|0);
        $1916 = ($1915|0)==(0);
        if ($1916) {
         HEAP32[$eMode47>>2] = 2;
         break;
        }
        $1917 = HEAP32[$zRight>>2]|0;
        $1918 = (_sqlite3StrICmp($1917,32720)|0);
        $1919 = ($1918|0)==(0);
        if ($1919) {
         HEAP32[$eMode47>>2] = 3;
        }
       }
      } while(0);
      $1920 = HEAP32[$v>>2]|0;
      _setAllColumnNames($1920,3,5408);
      $1921 = HEAP32[$0>>2]|0;
      $1922 = ((($1921)) + 44|0);
      HEAP32[$1922>>2] = 3;
      $1923 = HEAP32[$v>>2]|0;
      $1924 = HEAP32[$iBt>>2]|0;
      $1925 = HEAP32[$eMode47>>2]|0;
      (_sqlite3VdbeAddOp3($1923,8,$1924,$1925,1)|0);
      $1926 = HEAP32[$v>>2]|0;
      (_sqlite3VdbeAddOp2($1926,87,1,3)|0);
      break L21;
      break;
     }
     case 35:  {
      $1927 = HEAP32[$zRight>>2]|0;
      $1928 = ($1927|0)!=(0|0);
      if ($1928) {
       $1929 = HEAP32[$db>>2]|0;
       $1930 = HEAP32[$zRight>>2]|0;
       $1931 = (_sqlite3Atoi($1930)|0);
       (_sqlite3_wal_autocheckpoint($1929,$1931)|0);
      }
      $1932 = HEAP32[$v>>2]|0;
      $1933 = HEAP32[$db>>2]|0;
      $1934 = ((($1933)) + 224|0);
      $1935 = HEAP32[$1934>>2]|0;
      $1936 = ($1935|0)==(139|0);
      if ($1936) {
       $1937 = HEAP32[$db>>2]|0;
       $1938 = ((($1937)) + 228|0);
       $1939 = HEAP32[$1938>>2]|0;
       $1940 = $1939;
       $1942 = $1940;
      } else {
       $1942 = 0;
      }
      $1941 = ($1942|0)<(0);
      $1943 = $1941 << 31 >> 31;
      _returnSingleInt($1932,32729,$1942,$1943);
      break L21;
      break;
     }
     case 27:  {
      $1944 = HEAP32[$db>>2]|0;
      (_sqlite3_db_release_memory($1944)|0);
      break L21;
      break;
     }
     case 28:  {
      $1956 = HEAP32[$zRight>>2]|0;
      $1957 = ($1956|0)!=(0|0);
      if ($1957) {
       $1958 = HEAP32[$zRight>>2]|0;
       $1959 = (_sqlite3DecOrHexToI64($1958,$N)|0);
       $1960 = ($1959|0)==(0);
       if ($1960) {
        $1961 = $N;
        $1962 = $1961;
        $1963 = HEAP32[$1962>>2]|0;
        $1964 = (($1961) + 4)|0;
        $1965 = $1964;
        $1966 = HEAP32[$1965>>2]|0;
        (_sqlite3_soft_heap_limit64($1963,$1966)|0);
        $1967 = tempRet0;
       }
      }
      $1968 = HEAP32[$v>>2]|0;
      $1969 = (_sqlite3_soft_heap_limit64(-1,-1)|0);
      $1970 = tempRet0;
      _returnSingleInt($1968,32756,$1969,$1970);
      break L21;
      break;
     }
     case 34:  {
      $1971 = HEAP32[$zRight>>2]|0;
      $1972 = ($1971|0)!=(0|0);
      if ($1972) {
       $1973 = HEAP32[$zRight>>2]|0;
       $1974 = (_sqlite3DecOrHexToI64($1973,$N48)|0);
       $1975 = ($1974|0)==(0);
       $1976 = $N48;
       $1977 = $1976;
       $1978 = HEAP32[$1977>>2]|0;
       $1979 = (($1976) + 4)|0;
       $1980 = $1979;
       $1981 = HEAP32[$1980>>2]|0;
       $1982 = ($1981|0)>(0);
       $1983 = ($1978>>>0)>=(0);
       $1984 = ($1981|0)==(0);
       $1985 = $1984 & $1983;
       $1986 = $1982 | $1985;
       $or$cond14 = $1975 & $1986;
       if ($or$cond14) {
        $1987 = HEAP32[$db>>2]|0;
        $1988 = $N48;
        $1989 = $1988;
        $1990 = HEAP32[$1989>>2]|0;
        $1991 = (($1988) + 4)|0;
        $1992 = $1991;
        $1993 = HEAP32[$1992>>2]|0;
        $1994 = $1990 & 2147483647;
        (_sqlite3_limit($1987,11,$1994)|0);
       }
      }
      $1995 = HEAP32[$v>>2]|0;
      $1996 = HEAP32[$db>>2]|0;
      $1997 = (_sqlite3_limit($1996,11,-1)|0);
      $1998 = ($1997|0)<(0);
      $1999 = $1998 << 31 >> 31;
      _returnSingleInt($1995,32772,$1997,$1999);
      break L21;
      break;
     }
     default: {
      $1945 = HEAP32[$zRight>>2]|0;
      $1946 = ($1945|0)!=(0|0);
      if ($1946) {
       $1947 = HEAP32[$db>>2]|0;
       $1948 = HEAP32[$zRight>>2]|0;
       $1949 = (_sqlite3Atoi($1948)|0);
       (_sqlite3_busy_timeout($1947,$1949)|0);
      }
      $1950 = HEAP32[$v>>2]|0;
      $1951 = HEAP32[$db>>2]|0;
      $1952 = ((($1951)) + 428|0);
      $1953 = HEAP32[$1952>>2]|0;
      $1954 = ($1953|0)<(0);
      $1955 = $1954 << 31 >> 31;
      _returnSingleInt($1950,32748,$1953,$1955);
      break L21;
     }
     }
    } while(0);
   }
  }
 } while(0);
 $2000 = HEAP32[$db>>2]|0;
 $2001 = HEAP32[$zLeft>>2]|0;
 _sqlite3DbFree($2000,$2001);
 $2002 = HEAP32[$db>>2]|0;
 $2003 = HEAP32[$zRight>>2]|0;
 _sqlite3DbFree($2002,$2003);
 STACKTOP = sp;return;
}
function _sqlite3FinishTrigger($pParse,$pStepList,$pAll) {
 $pParse = $pParse|0;
 $pStepList = $pStepList|0;
 $pAll = $pAll|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0;
 var $nameToken = 0, $or$cond = 0, $pHash = 0, $pLink = 0, $pTab = 0, $pTrig = 0, $sFix = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer6 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $z = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $pTrig = sp + 96|0;
 $zName = sp + 92|0;
 $db = sp + 88|0;
 $sFix = sp + 64|0;
 $iDb = sp + 56|0;
 $nameToken = sp + 48|0;
 $v = sp + 44|0;
 $z = sp + 40|0;
 $pLink = sp + 36|0;
 $pHash = sp + 32|0;
 $pTab = sp + 28|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStepList;
 HEAP32[$2>>2] = $pAll;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 444|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTrig>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 444|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 36|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 $14 = HEAP32[$pTrig>>2]|0;
 $15 = ($14|0)!=(0|0);
 $or$cond = $13 & $15;
 do {
  if ($or$cond) {
   $16 = HEAP32[$pTrig>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$zName>>2] = $17;
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$pTrig>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3SchemaToIndex($19,$22)|0);
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$pTrig>>2]|0;
   $26 = ((($25)) + 28|0);
   HEAP32[$26>>2] = $24;
   while(1) {
    $27 = HEAP32[$1>>2]|0;
    $28 = ($27|0)!=(0|0);
    $29 = HEAP32[$pTrig>>2]|0;
    if (!($28)) {
     break;
    }
    $30 = HEAP32[$1>>2]|0;
    $31 = ((($30)) + 4|0);
    HEAP32[$31>>2] = $29;
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 28|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$1>>2] = $34;
   }
   $35 = HEAP32[$29>>2]|0;
   _sqlite3TokenInit($nameToken,$35);
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$iDb>>2]|0;
   _sqlite3FixInit($sFix,$36,$37,18664,$nameToken);
   $38 = HEAP32[$pTrig>>2]|0;
   $39 = ((($38)) + 28|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = (_sqlite3FixTriggerStep($sFix,$40)|0);
   $42 = ($41|0)!=(0);
   if (!($42)) {
    $43 = HEAP32[$pTrig>>2]|0;
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = (_sqlite3FixExpr($sFix,$45)|0);
    $47 = ($46|0)!=(0);
    if (!($47)) {
     $48 = HEAP32[$db>>2]|0;
     $49 = ((($48)) + 148|0);
     $50 = ((($49)) + 5|0);
     $51 = HEAP8[$50>>0]|0;
     $52 = ($51<<24>>24)!=(0);
     if (!($52)) {
      $53 = HEAP32[$0>>2]|0;
      $54 = (_sqlite3GetVdbe($53)|0);
      HEAP32[$v>>2] = $54;
      $55 = HEAP32[$v>>2]|0;
      $56 = ($55|0)==(0|0);
      if ($56) {
       break;
      }
      $57 = HEAP32[$0>>2]|0;
      $58 = HEAP32[$iDb>>2]|0;
      _sqlite3BeginWriteOperation($57,0,$58);
      $59 = HEAP32[$db>>2]|0;
      $60 = HEAP32[$2>>2]|0;
      $61 = HEAP32[$60>>2]|0;
      $62 = HEAP32[$2>>2]|0;
      $63 = ((($62)) + 4|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (_sqlite3DbStrNDup($59,$61,$64,0)|0);
      HEAP32[$z>>2] = $65;
      $66 = HEAP32[$0>>2]|0;
      $67 = HEAP32[$iDb>>2]|0;
      $68 = HEAP32[$db>>2]|0;
      $69 = ((($68)) + 16|0);
      $70 = HEAP32[$69>>2]|0;
      $71 = (($70) + ($67<<4)|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = HEAP32[$iDb>>2]|0;
      $74 = ($73|0)==(1);
      $75 = $74 ? 19905 : 19924;
      $76 = HEAP32[$zName>>2]|0;
      $77 = HEAP32[$pTrig>>2]|0;
      $78 = ((($77)) + 4|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = HEAP32[$z>>2]|0;
      HEAP32[$vararg_buffer>>2] = $72;
      $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
      HEAP32[$vararg_ptr2>>2] = $75;
      $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
      HEAP32[$vararg_ptr3>>2] = $76;
      $vararg_ptr4 = ((($vararg_buffer)) + 12|0);
      HEAP32[$vararg_ptr4>>2] = $79;
      $vararg_ptr5 = ((($vararg_buffer)) + 16|0);
      HEAP32[$vararg_ptr5>>2] = $80;
      _sqlite3NestedParse($66,32010,$vararg_buffer);
      $81 = HEAP32[$db>>2]|0;
      $82 = HEAP32[$z>>2]|0;
      _sqlite3DbFree($81,$82);
      $83 = HEAP32[$0>>2]|0;
      $84 = HEAP32[$iDb>>2]|0;
      _sqlite3ChangeCookie($83,$84);
      $85 = HEAP32[$v>>2]|0;
      $86 = HEAP32[$iDb>>2]|0;
      $87 = HEAP32[$db>>2]|0;
      $88 = HEAP32[$zName>>2]|0;
      HEAP32[$vararg_buffer6>>2] = $88;
      $89 = (_sqlite3MPrintf($87,32074,$vararg_buffer6)|0);
      _sqlite3VdbeAddParseSchemaOp($85,$86,$89);
     }
     $90 = HEAP32[$db>>2]|0;
     $91 = ((($90)) + 148|0);
     $92 = ((($91)) + 5|0);
     $93 = HEAP8[$92>>0]|0;
     $94 = ($93<<24>>24)!=(0);
     if ($94) {
      $95 = HEAP32[$pTrig>>2]|0;
      HEAP32[$pLink>>2] = $95;
      $96 = HEAP32[$iDb>>2]|0;
      $97 = HEAP32[$db>>2]|0;
      $98 = ((($97)) + 16|0);
      $99 = HEAP32[$98>>2]|0;
      $100 = (($99) + ($96<<4)|0);
      $101 = ((($100)) + 12|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = ((($102)) + 40|0);
      HEAP32[$pHash>>2] = $103;
      $104 = HEAP32[$pHash>>2]|0;
      $105 = HEAP32[$zName>>2]|0;
      $106 = HEAP32[$pTrig>>2]|0;
      $107 = (_sqlite3HashInsert($104,$105,$106)|0);
      HEAP32[$pTrig>>2] = $107;
      $108 = HEAP32[$pTrig>>2]|0;
      $109 = ($108|0)!=(0|0);
      if ($109) {
       $110 = HEAP32[$db>>2]|0;
       _sqlite3OomFault($110);
       break;
      }
      $111 = HEAP32[$pLink>>2]|0;
      $112 = ((($111)) + 20|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = HEAP32[$pLink>>2]|0;
      $115 = ((($114)) + 24|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ($113|0)==($116|0);
      if ($117) {
       $118 = HEAP32[$pLink>>2]|0;
       $119 = ((($118)) + 24|0);
       $120 = HEAP32[$119>>2]|0;
       $121 = ((($120)) + 8|0);
       $122 = HEAP32[$pLink>>2]|0;
       $123 = ((($122)) + 4|0);
       $124 = HEAP32[$123>>2]|0;
       $125 = (_sqlite3HashFind($121,$124)|0);
       HEAP32[$pTab>>2] = $125;
       $126 = HEAP32[$pTab>>2]|0;
       $127 = ((($126)) + 60|0);
       $128 = HEAP32[$127>>2]|0;
       $129 = HEAP32[$pLink>>2]|0;
       $130 = ((($129)) + 32|0);
       HEAP32[$130>>2] = $128;
       $131 = HEAP32[$pLink>>2]|0;
       $132 = HEAP32[$pTab>>2]|0;
       $133 = ((($132)) + 60|0);
       HEAP32[$133>>2] = $131;
      }
     }
    }
   }
  }
 } while(0);
 $134 = HEAP32[$db>>2]|0;
 $135 = HEAP32[$pTrig>>2]|0;
 _sqlite3DeleteTrigger($134,$135);
 $136 = HEAP32[$db>>2]|0;
 $137 = HEAP32[$1>>2]|0;
 _sqlite3DeleteTriggerStep($136,$137);
 STACKTOP = sp;return;
}
function _sqlite3BeginTrigger($pParse,$pName1,$pName2,$tr_tm,$op,$pColumns,$pTableName,$pWhen,$isTemp,$noErr) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $tr_tm = $tr_tm|0;
 $op = $op|0;
 $pColumns = $pColumns|0;
 $pTableName = $pTableName|0;
 $pWhen = $pWhen|0;
 $isTemp = $isTemp|0;
 $noErr = $noErr|0;
 var $$old = 0, $$old10 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $code = 0, $db = 0, $iDb = 0, $iTabDb = 0, $or$cond = 0, $or$cond11 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $pName = 0, $pTab = 0, $pTrigger = 0, $sFix = 0, $vararg_buffer = 0, $vararg_buffer12 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0;
 var $vararg_buffer18 = 0, $vararg_buffer23 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $zDb = 0, $zDbTrig = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer23 = sp + 48|0;
 $vararg_buffer18 = sp + 32|0;
 $vararg_buffer16 = sp + 24|0;
 $vararg_buffer14 = sp + 16|0;
 $vararg_buffer12 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 156|0;
 $1 = sp + 152|0;
 $2 = sp + 148|0;
 $3 = sp + 144|0;
 $4 = sp + 140|0;
 $5 = sp + 136|0;
 $6 = sp + 132|0;
 $7 = sp + 128|0;
 $8 = sp + 124|0;
 $9 = sp + 120|0;
 $pTrigger = sp + 116|0;
 $pTab = sp + 112|0;
 $zName = sp + 108|0;
 $db = sp + 104|0;
 $iDb = sp + 100|0;
 $pName = sp + 96|0;
 $sFix = sp + 72|0;
 $iTabDb = sp + 68|0;
 $code = sp + 64|0;
 $zDb = sp + 60|0;
 $zDbTrig = sp + 56|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $tr_tm;
 HEAP32[$4>>2] = $op;
 HEAP32[$5>>2] = $pColumns;
 HEAP32[$6>>2] = $pTableName;
 HEAP32[$7>>2] = $pWhen;
 HEAP32[$8>>2] = $isTemp;
 HEAP32[$9>>2] = $noErr;
 HEAP32[$pTrigger>>2] = 0;
 HEAP32[$zName>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$8>>2]|0;
 $13 = ($12|0)!=(0);
 do {
  if ($13) {
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16>>>0)>(0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($18,31693,$vararg_buffer);
    break;
   } else {
    HEAP32[$iDb>>2] = 1;
    $19 = HEAP32[$1>>2]|0;
    HEAP32[$pName>>2] = $19;
    $$old = HEAP32[$6>>2]|0;
    $$old10 = ($$old|0)!=(0|0);
    if ($$old10) {
     label = 6;
     break;
    } else {
     break;
    }
   }
  } else {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$2>>2]|0;
   $23 = (_sqlite3TwoPartName($20,$21,$22,$pName)|0);
   HEAP32[$iDb>>2] = $23;
   $24 = HEAP32[$iDb>>2]|0;
   $25 = ($24|0)>=(0);
   $26 = HEAP32[$6>>2]|0;
   $27 = ($26|0)!=(0|0);
   $or$cond11 = $25 & $27;
   if ($or$cond11) {
    label = 6;
   }
  }
 } while(0);
 do {
  if ((label|0) == 6) {
   $28 = HEAP32[$db>>2]|0;
   $29 = ((($28)) + 69|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = ($31|0)!=(0);
   if (!($32)) {
    $33 = HEAP32[$db>>2]|0;
    $34 = ((($33)) + 148|0);
    $35 = ((($34)) + 5|0);
    $36 = HEAP8[$35>>0]|0;
    $37 = $36&255;
    $38 = ($37|0)!=(0);
    $39 = HEAP32[$iDb>>2]|0;
    $40 = ($39|0)!=(1);
    $or$cond = $38 & $40;
    if ($or$cond) {
     $41 = HEAP32[$db>>2]|0;
     $42 = HEAP32[$6>>2]|0;
     $43 = ((($42)) + 8|0);
     $44 = ((($43)) + 4|0);
     $45 = HEAP32[$44>>2]|0;
     _sqlite3DbFree($41,$45);
     $46 = HEAP32[$6>>2]|0;
     $47 = ((($46)) + 8|0);
     $48 = ((($47)) + 4|0);
     HEAP32[$48>>2] = 0;
    }
    $49 = HEAP32[$0>>2]|0;
    $50 = HEAP32[$6>>2]|0;
    $51 = (_sqlite3SrcListLookup($49,$50)|0);
    HEAP32[$pTab>>2] = $51;
    $52 = HEAP32[$db>>2]|0;
    $53 = ((($52)) + 148|0);
    $54 = ((($53)) + 5|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = ($56|0)==(0);
    if ($57) {
     $58 = HEAP32[$2>>2]|0;
     $59 = ((($58)) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = ($60|0)==(0);
     $62 = HEAP32[$pTab>>2]|0;
     $63 = ($62|0)!=(0|0);
     $or$cond3 = $61 & $63;
     if ($or$cond3) {
      $64 = HEAP32[$pTab>>2]|0;
      $65 = ((($64)) + 64|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = HEAP32[$db>>2]|0;
      $68 = ((($67)) + 16|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = ((($69)) + 16|0);
      $71 = ((($70)) + 12|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = ($66|0)==($72|0);
      if ($73) {
       HEAP32[$iDb>>2] = 1;
      }
     }
    }
    $74 = HEAP32[$db>>2]|0;
    $75 = ((($74)) + 69|0);
    $76 = HEAP8[$75>>0]|0;
    $77 = ($76<<24>>24)!=(0);
    if (!($77)) {
     $78 = HEAP32[$0>>2]|0;
     $79 = HEAP32[$iDb>>2]|0;
     $80 = HEAP32[$pName>>2]|0;
     _sqlite3FixInit($sFix,$78,$79,18664,$80);
     $81 = HEAP32[$6>>2]|0;
     $82 = (_sqlite3FixSrcList($sFix,$81)|0);
     $83 = ($82|0)!=(0);
     if (!($83)) {
      $84 = HEAP32[$0>>2]|0;
      $85 = HEAP32[$6>>2]|0;
      $86 = (_sqlite3SrcListLookup($84,$85)|0);
      HEAP32[$pTab>>2] = $86;
      $87 = HEAP32[$pTab>>2]|0;
      $88 = ($87|0)!=(0|0);
      if (!($88)) {
       $89 = HEAP32[$db>>2]|0;
       $90 = ((($89)) + 148|0);
       $91 = ((($90)) + 4|0);
       $92 = HEAP8[$91>>0]|0;
       $93 = $92&255;
       $94 = ($93|0)==(1);
       if (!($94)) {
        break;
       }
       $95 = HEAP32[$db>>2]|0;
       $96 = ((($95)) + 148|0);
       $97 = ((($96)) + 6|0);
       HEAP8[$97>>0] = 1;
       break;
      }
      $98 = HEAP32[$pTab>>2]|0;
      $99 = ((($98)) + 42|0);
      $100 = HEAP8[$99>>0]|0;
      $101 = $100&255;
      $102 = $101 & 16;
      $103 = ($102|0)!=(0);
      if ($103) {
       $104 = HEAP32[$0>>2]|0;
       _sqlite3ErrorMsg($104,31739,$vararg_buffer12);
       break;
      }
      $105 = HEAP32[$db>>2]|0;
      $106 = HEAP32[$pName>>2]|0;
      $107 = (_sqlite3NameFromToken($105,$106)|0);
      HEAP32[$zName>>2] = $107;
      $108 = HEAP32[$zName>>2]|0;
      $109 = ($108|0)!=(0|0);
      if ($109) {
       $110 = HEAP32[$0>>2]|0;
       $111 = HEAP32[$zName>>2]|0;
       $112 = (_sqlite3CheckObjectName($110,$111)|0);
       $113 = (0)!=($112|0);
       if (!($113)) {
        $114 = HEAP32[$iDb>>2]|0;
        $115 = HEAP32[$db>>2]|0;
        $116 = ((($115)) + 16|0);
        $117 = HEAP32[$116>>2]|0;
        $118 = (($117) + ($114<<4)|0);
        $119 = ((($118)) + 12|0);
        $120 = HEAP32[$119>>2]|0;
        $121 = ((($120)) + 40|0);
        $122 = HEAP32[$zName>>2]|0;
        $123 = (_sqlite3HashFind($121,$122)|0);
        $124 = ($123|0)!=(0|0);
        if ($124) {
         $125 = HEAP32[$9>>2]|0;
         $126 = ($125|0)!=(0);
         $127 = HEAP32[$0>>2]|0;
         if ($126) {
          $129 = HEAP32[$iDb>>2]|0;
          _sqlite3CodeVerifySchema($127,$129);
          break;
         } else {
          $128 = HEAP32[$pName>>2]|0;
          HEAP32[$vararg_buffer14>>2] = $128;
          _sqlite3ErrorMsg($127,31780,$vararg_buffer14);
          break;
         }
        }
        $130 = HEAP32[$pTab>>2]|0;
        $131 = HEAP32[$130>>2]|0;
        $132 = (_sqlite3_strnicmp($131,27844,7)|0);
        $133 = ($132|0)==(0);
        if ($133) {
         $134 = HEAP32[$0>>2]|0;
         _sqlite3ErrorMsg($134,31806,$vararg_buffer16);
         break;
        }
        $135 = HEAP32[$pTab>>2]|0;
        $136 = ((($135)) + 12|0);
        $137 = HEAP32[$136>>2]|0;
        $138 = ($137|0)!=(0|0);
        $139 = HEAP32[$3>>2]|0;
        $140 = ($139|0)!=(77);
        $or$cond5 = $138 & $140;
        if ($or$cond5) {
         $141 = HEAP32[$0>>2]|0;
         $142 = HEAP32[$3>>2]|0;
         $143 = ($142|0)==(63);
         $144 = $143 ? 31844 : 31851;
         $145 = HEAP32[$6>>2]|0;
         HEAP32[$vararg_buffer18>>2] = $144;
         $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
         HEAP32[$vararg_ptr21>>2] = $145;
         $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
         HEAP32[$vararg_ptr22>>2] = 0;
         _sqlite3ErrorMsg($141,31857,$vararg_buffer18);
         break;
        }
        $146 = HEAP32[$pTab>>2]|0;
        $147 = ((($146)) + 12|0);
        $148 = HEAP32[$147>>2]|0;
        $149 = ($148|0)==(0|0);
        $150 = HEAP32[$3>>2]|0;
        $151 = ($150|0)==(77);
        $or$cond7 = $149 & $151;
        if ($or$cond7) {
         $152 = HEAP32[$0>>2]|0;
         $153 = HEAP32[$6>>2]|0;
         HEAP32[$vararg_buffer23>>2] = $153;
         $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
         HEAP32[$vararg_ptr26>>2] = 0;
         _sqlite3ErrorMsg($152,31894,$vararg_buffer23);
         break;
        }
        $154 = HEAP32[$db>>2]|0;
        $155 = HEAP32[$pTab>>2]|0;
        $156 = ((($155)) + 64|0);
        $157 = HEAP32[$156>>2]|0;
        $158 = (_sqlite3SchemaToIndex($154,$157)|0);
        HEAP32[$iTabDb>>2] = $158;
        HEAP32[$code>>2] = 7;
        $159 = HEAP32[$iTabDb>>2]|0;
        $160 = HEAP32[$db>>2]|0;
        $161 = ((($160)) + 16|0);
        $162 = HEAP32[$161>>2]|0;
        $163 = (($162) + ($159<<4)|0);
        $164 = HEAP32[$163>>2]|0;
        HEAP32[$zDb>>2] = $164;
        $165 = HEAP32[$8>>2]|0;
        $166 = ($165|0)!=(0);
        if ($166) {
         $167 = HEAP32[$db>>2]|0;
         $168 = ((($167)) + 16|0);
         $169 = HEAP32[$168>>2]|0;
         $170 = ((($169)) + 16|0);
         $171 = HEAP32[$170>>2]|0;
         $173 = $171;
        } else {
         $172 = HEAP32[$zDb>>2]|0;
         $173 = $172;
        }
        HEAP32[$zDbTrig>>2] = $173;
        $174 = HEAP32[$iTabDb>>2]|0;
        $175 = ($174|0)==(1);
        $176 = HEAP32[$8>>2]|0;
        $177 = ($176|0)!=(0);
        $or$cond9 = $175 | $177;
        if ($or$cond9) {
         HEAP32[$code>>2] = 5;
        }
        $178 = HEAP32[$0>>2]|0;
        $179 = HEAP32[$code>>2]|0;
        $180 = HEAP32[$zName>>2]|0;
        $181 = HEAP32[$pTab>>2]|0;
        $182 = HEAP32[$181>>2]|0;
        $183 = HEAP32[$zDbTrig>>2]|0;
        $184 = (_sqlite3AuthCheck($178,$179,$180,$182,$183)|0);
        $185 = ($184|0)!=(0);
        if (!($185)) {
         $186 = HEAP32[$0>>2]|0;
         $187 = HEAP32[$iTabDb>>2]|0;
         $188 = ($187|0)==(1);
         $189 = $188 ? 19905 : 19924;
         $190 = HEAP32[$zDb>>2]|0;
         $191 = (_sqlite3AuthCheck($186,18,$189,0,$190)|0);
         $192 = ($191|0)!=(0);
         if (!($192)) {
          $193 = HEAP32[$3>>2]|0;
          $194 = ($193|0)==(77);
          if ($194) {
           HEAP32[$3>>2] = 63;
          }
          $195 = HEAP32[$db>>2]|0;
          $196 = (_sqlite3DbMallocZero($195,36,0)|0);
          HEAP32[$pTrigger>>2] = $196;
          $197 = HEAP32[$pTrigger>>2]|0;
          $198 = ($197|0)==(0|0);
          if ($198) {
           break;
          }
          $199 = HEAP32[$zName>>2]|0;
          $200 = HEAP32[$pTrigger>>2]|0;
          HEAP32[$200>>2] = $199;
          HEAP32[$zName>>2] = 0;
          $201 = HEAP32[$db>>2]|0;
          $202 = HEAP32[$6>>2]|0;
          $203 = ((($202)) + 8|0);
          $204 = ((($203)) + 8|0);
          $205 = HEAP32[$204>>2]|0;
          $206 = (_sqlite3DbStrDup($201,$205)|0);
          $207 = HEAP32[$pTrigger>>2]|0;
          $208 = ((($207)) + 4|0);
          HEAP32[$208>>2] = $206;
          $209 = HEAP32[$iDb>>2]|0;
          $210 = HEAP32[$db>>2]|0;
          $211 = ((($210)) + 16|0);
          $212 = HEAP32[$211>>2]|0;
          $213 = (($212) + ($209<<4)|0);
          $214 = ((($213)) + 12|0);
          $215 = HEAP32[$214>>2]|0;
          $216 = HEAP32[$pTrigger>>2]|0;
          $217 = ((($216)) + 20|0);
          HEAP32[$217>>2] = $215;
          $218 = HEAP32[$pTab>>2]|0;
          $219 = ((($218)) + 64|0);
          $220 = HEAP32[$219>>2]|0;
          $221 = HEAP32[$pTrigger>>2]|0;
          $222 = ((($221)) + 24|0);
          HEAP32[$222>>2] = $220;
          $223 = HEAP32[$4>>2]|0;
          $224 = $223&255;
          $225 = HEAP32[$pTrigger>>2]|0;
          $226 = ((($225)) + 8|0);
          HEAP8[$226>>0] = $224;
          $227 = HEAP32[$3>>2]|0;
          $228 = ($227|0)==(63);
          $229 = $228 ? 1 : 2;
          $230 = $229&255;
          $231 = HEAP32[$pTrigger>>2]|0;
          $232 = ((($231)) + 9|0);
          HEAP8[$232>>0] = $230;
          $233 = HEAP32[$db>>2]|0;
          $234 = HEAP32[$7>>2]|0;
          $235 = (_sqlite3ExprDup($233,$234,1)|0);
          $236 = HEAP32[$pTrigger>>2]|0;
          $237 = ((($236)) + 12|0);
          HEAP32[$237>>2] = $235;
          $238 = HEAP32[$db>>2]|0;
          $239 = HEAP32[$5>>2]|0;
          $240 = (_sqlite3IdListDup($238,$239)|0);
          $241 = HEAP32[$pTrigger>>2]|0;
          $242 = ((($241)) + 16|0);
          HEAP32[$242>>2] = $240;
          $243 = HEAP32[$pTrigger>>2]|0;
          $244 = HEAP32[$0>>2]|0;
          $245 = ((($244)) + 444|0);
          HEAP32[$245>>2] = $243;
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $246 = HEAP32[$db>>2]|0;
 $247 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($246,$247);
 $248 = HEAP32[$db>>2]|0;
 $249 = HEAP32[$6>>2]|0;
 _sqlite3SrcListDelete($248,$249);
 $250 = HEAP32[$db>>2]|0;
 $251 = HEAP32[$5>>2]|0;
 _sqlite3IdListDelete($250,$251);
 $252 = HEAP32[$db>>2]|0;
 $253 = HEAP32[$7>>2]|0;
 _sqlite3ExprDelete($252,$253);
 $254 = HEAP32[$0>>2]|0;
 $255 = ((($254)) + 444|0);
 $256 = HEAP32[$255>>2]|0;
 $257 = ($256|0)!=(0|0);
 if ($257) {
  STACKTOP = sp;return;
 }
 $258 = HEAP32[$db>>2]|0;
 $259 = HEAP32[$pTrigger>>2]|0;
 _sqlite3DeleteTrigger($258,$259);
 STACKTOP = sp;return;
}
function _sqlite3TriggerUpdateStep($db,$pTableName,$pEList,$pWhere,$orconf) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pEList = $pEList|0;
 $pWhere = $pWhere|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 20|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pWhere;
 HEAP8[$4>>0] = $orconf;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_triggerStepAllocate($5,110,$6)|0);
 HEAP32[$pTriggerStep>>2] = $7;
 $8 = HEAP32[$pTriggerStep>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_sqlite3ExprListDup($10,$11,1)|0);
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 20|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_sqlite3ExprDup($15,$16,1)|0);
  $18 = HEAP32[$pTriggerStep>>2]|0;
  $19 = ((($18)) + 16|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP8[$4>>0]|0;
  $21 = HEAP32[$pTriggerStep>>2]|0;
  $22 = ((($21)) + 1|0);
  HEAP8[$22>>0] = $20;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 _sqlite3ExprListDelete($23,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($25,$26);
 $27 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3TriggerInsertStep($db,$pTableName,$pColumn,$pSelect,$orconf) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pColumn = $pColumn|0;
 $pSelect = $pSelect|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 20|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pColumn;
 HEAP32[$3>>2] = $pSelect;
 HEAP8[$4>>0] = $orconf;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_triggerStepAllocate($5,108,$6)|0);
 HEAP32[$pTriggerStep>>2] = $7;
 $8 = HEAP32[$pTriggerStep>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$0>>2]|0;
 if ($9) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3SelectDup($10,$11,1)|0);
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 8|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$pTriggerStep>>2]|0;
  $17 = ((($16)) + 24|0);
  HEAP32[$17>>2] = $15;
  $18 = HEAP8[$4>>0]|0;
  $19 = HEAP32[$pTriggerStep>>2]|0;
  $20 = ((($19)) + 1|0);
  HEAP8[$20>>0] = $18;
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _sqlite3SelectDelete($22,$23);
  $24 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $21 = HEAP32[$2>>2]|0;
  _sqlite3IdListDelete($10,$21);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _sqlite3SelectDelete($22,$23);
  $24 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3TriggerDeleteStep($db,$pTableName,$pWhere) {
 $db = $db|0;
 $pTableName = $pTableName|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pTriggerStep = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTableName;
 HEAP32[$2>>2] = $pWhere;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_triggerStepAllocate($3,109,$4)|0);
 HEAP32[$pTriggerStep>>2] = $5;
 $6 = HEAP32[$pTriggerStep>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3ExprDup($8,$9,1)|0);
  $11 = HEAP32[$pTriggerStep>>2]|0;
  $12 = ((($11)) + 16|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP8[$14>>0] = 10;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($15,$16);
 $17 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3TriggerSelectStep($db,$pSelect) {
 $db = $db|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pTriggerStep = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3DbMallocZero($3,36,0)|0);
 HEAP32[$pTriggerStep>>2] = $4;
 $5 = HEAP32[$pTriggerStep>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  _sqlite3SelectDelete($7,$8);
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 } else {
  $9 = HEAP32[$pTriggerStep>>2]|0;
  HEAP8[$9>>0] = 119;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$pTriggerStep>>2]|0;
  $12 = ((($11)) + 8|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$pTriggerStep>>2]|0;
  $14 = ((($13)) + 1|0);
  HEAP8[$14>>0] = 10;
  $15 = HEAP32[$pTriggerStep>>2]|0;
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _sqlite3DropTrigger($pParse,$pName,$noErr) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $noErr = $noErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $j = 0, $pTrigger = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zDb = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pTrigger = sp + 28|0;
 $i = sp + 24|0;
 $zDb = sp + 20|0;
 $zName = sp + 16|0;
 $db = sp + 12|0;
 $j = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $noErr;
 HEAP32[$pTrigger>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 69|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3ReadSchema($9)|0);
 $11 = (0)!=($10|0);
 if ($11) {
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$zDb>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$zName>>2] = $19;
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$db>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = ($25|0)<(2);
  $27 = HEAP32[$i>>2]|0;
  $28 = $27 ^ 1;
  $29 = $26 ? $28 : $27;
  HEAP32[$j>>2] = $29;
  $30 = HEAP32[$zDb>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = HEAP32[$j>>2]|0;
   $33 = HEAP32[$db>>2]|0;
   $34 = ((($33)) + 16|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (($35) + ($32<<4)|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$zDb>>2]|0;
   $39 = (_sqlite3StrICmp($37,$38)|0);
   $40 = ($39|0)!=(0);
   if (!($40)) {
    label = 7;
   }
  } else {
   label = 7;
  }
  if ((label|0) == 7) {
   label = 0;
   $41 = HEAP32[$j>>2]|0;
   $42 = HEAP32[$db>>2]|0;
   $43 = ((($42)) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + ($41<<4)|0);
   $46 = ((($45)) + 12|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ((($47)) + 40|0);
   $49 = HEAP32[$zName>>2]|0;
   $50 = (_sqlite3HashFind($48,$49)|0);
   HEAP32[$pTrigger>>2] = $50;
   $51 = HEAP32[$pTrigger>>2]|0;
   $52 = ($51|0)!=(0|0);
   if ($52) {
    break;
   }
  }
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$i>>2] = $54;
 }
 $55 = HEAP32[$pTrigger>>2]|0;
 $56 = ($55|0)!=(0|0);
 if ($56) {
  $64 = HEAP32[$0>>2]|0;
  $65 = HEAP32[$pTrigger>>2]|0;
  _sqlite3DropTriggerPtr($64,$65);
  $66 = HEAP32[$db>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  _sqlite3SrcListDelete($66,$67);
  STACKTOP = sp;return;
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = ($57|0)!=(0);
 $59 = HEAP32[$0>>2]|0;
 if ($58) {
  $61 = HEAP32[$zDb>>2]|0;
  _sqlite3CodeVerifyNamedSchema($59,$61);
 } else {
  $60 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $60;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 0;
  _sqlite3ErrorMsg($59,31622,$vararg_buffer);
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = ((($62)) + 17|0);
 HEAP8[$63>>0] = 1;
 $66 = HEAP32[$db>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($66,$67);
 STACKTOP = sp;return;
}
function _sqlite3Attach($pParse,$p,$pDbname,$pKey) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDbname = $pDbname|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDbname;
 HEAP32[$3>>2] = $pKey;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 _codeAttach($4,24,4448,$5,$6,$7,$8);
 STACKTOP = sp;return;
}
function _sqlite3Detach($pParse,$pDbname) {
 $pParse = $pParse|0;
 $pDbname = $pDbname|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pDbname;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _codeAttach($2,25,4420,$3,0,0,$4);
 STACKTOP = sp;return;
}
function _sqlite3Reindex($pParse,$pName1,$pName2) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $pColl = 0, $pIndex = 0, $pObjName = 0;
 var $pTab = 0, $vararg_buffer = 0, $z = 0, $zColl = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $pColl = sp + 36|0;
 $z = sp + 32|0;
 $zDb = sp + 28|0;
 $pTab = sp + 24|0;
 $pIndex = sp + 20|0;
 $iDb = sp + 16|0;
 $db = sp + 12|0;
 $pObjName = sp + 8|0;
 $zColl = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _reindexDatabases($10,0);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  label = 6;
 } else {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0|0);
  if ($15) {
   label = 6;
  }
 }
 do {
  if ((label|0) == 6) {
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = (_sqlite3NameFromToken($17,$18)|0);
   HEAP32[$zColl>>2] = $19;
   $20 = HEAP32[$zColl>>2]|0;
   $21 = ($20|0)!=(0|0);
   if (!($21)) {
    STACKTOP = sp;return;
   }
   $22 = HEAP32[$db>>2]|0;
   $23 = HEAP32[$db>>2]|0;
   $24 = ((($23)) + 66|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = HEAP32[$zColl>>2]|0;
   $27 = (_sqlite3FindCollSeq($22,$25,$26,0)|0);
   HEAP32[$pColl>>2] = $27;
   $28 = HEAP32[$pColl>>2]|0;
   $29 = ($28|0)!=(0|0);
   if (!($29)) {
    $34 = HEAP32[$db>>2]|0;
    $35 = HEAP32[$zColl>>2]|0;
    _sqlite3DbFree($34,$35);
    break;
   }
   $30 = HEAP32[$0>>2]|0;
   $31 = HEAP32[$zColl>>2]|0;
   _reindexDatabases($30,$31);
   $32 = HEAP32[$db>>2]|0;
   $33 = HEAP32[$zColl>>2]|0;
   _sqlite3DbFree($32,$33);
   STACKTOP = sp;return;
  }
 } while(0);
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = (_sqlite3TwoPartName($36,$37,$38,$pObjName)|0);
 HEAP32[$iDb>>2] = $39;
 $40 = HEAP32[$iDb>>2]|0;
 $41 = ($40|0)<(0);
 if ($41) {
  STACKTOP = sp;return;
 }
 $42 = HEAP32[$db>>2]|0;
 $43 = HEAP32[$pObjName>>2]|0;
 $44 = (_sqlite3NameFromToken($42,$43)|0);
 HEAP32[$z>>2] = $44;
 $45 = HEAP32[$z>>2]|0;
 $46 = ($45|0)==(0|0);
 if ($46) {
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$iDb>>2]|0;
 $48 = HEAP32[$db>>2]|0;
 $49 = ((($48)) + 16|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = (($50) + ($47<<4)|0);
 $52 = HEAP32[$51>>2]|0;
 HEAP32[$zDb>>2] = $52;
 $53 = HEAP32[$db>>2]|0;
 $54 = HEAP32[$z>>2]|0;
 $55 = HEAP32[$zDb>>2]|0;
 $56 = (_sqlite3FindTable($53,$54,$55)|0);
 HEAP32[$pTab>>2] = $56;
 $57 = HEAP32[$pTab>>2]|0;
 $58 = ($57|0)!=(0|0);
 if ($58) {
  $59 = HEAP32[$0>>2]|0;
  $60 = HEAP32[$pTab>>2]|0;
  _reindexTable($59,$60,0);
  $61 = HEAP32[$db>>2]|0;
  $62 = HEAP32[$z>>2]|0;
  _sqlite3DbFree($61,$62);
  STACKTOP = sp;return;
 }
 $63 = HEAP32[$db>>2]|0;
 $64 = HEAP32[$z>>2]|0;
 $65 = HEAP32[$zDb>>2]|0;
 $66 = (_sqlite3FindIndex($63,$64,$65)|0);
 HEAP32[$pIndex>>2] = $66;
 $67 = HEAP32[$db>>2]|0;
 $68 = HEAP32[$z>>2]|0;
 _sqlite3DbFree($67,$68);
 $69 = HEAP32[$pIndex>>2]|0;
 $70 = ($69|0)!=(0|0);
 $71 = HEAP32[$0>>2]|0;
 if ($70) {
  $72 = HEAP32[$iDb>>2]|0;
  _sqlite3BeginWriteOperation($71,0,$72);
  $73 = HEAP32[$0>>2]|0;
  $74 = HEAP32[$pIndex>>2]|0;
  _sqlite3RefillIndex($73,$74,-1);
  STACKTOP = sp;return;
 } else {
  _sqlite3ErrorMsg($71,31007,$vararg_buffer);
  STACKTOP = sp;return;
 }
}
function _sqlite3Analyze($pParse,$pName1,$pName2) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $db = 0, $i = 0, $iDb = 0, $pIdx = 0, $pTab = 0, $pTableName = 0, $v = 0;
 var $z = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $db = sp + 32|0;
 $iDb = sp + 28|0;
 $i = sp + 24|0;
 $z = sp + 20|0;
 $zDb = sp + 16|0;
 $pTab = sp + 12|0;
 $pIdx = sp + 8|0;
 $pTableName = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3ReadSchema($5)|0);
 $7 = (0)!=($6|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 L4: do {
  if ($9) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $10 = HEAP32[$i>>2]|0;
    $11 = HEAP32[$db>>2]|0;
    $12 = ((($11)) + 20|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = ($10|0)<($13|0);
    if (!($14)) {
     break L4;
    }
    $15 = HEAP32[$i>>2]|0;
    $16 = ($15|0)==(1);
    if (!($16)) {
     $17 = HEAP32[$0>>2]|0;
     $18 = HEAP32[$i>>2]|0;
     _analyzeDatabase($17,$18);
    }
    $19 = HEAP32[$i>>2]|0;
    $20 = (($19) + 1)|0;
    HEAP32[$i>>2] = $20;
   }
  } else {
   $21 = HEAP32[$2>>2]|0;
   $22 = ((($21)) + 4|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)==(0);
   if (!($24)) {
    $53 = HEAP32[$0>>2]|0;
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$2>>2]|0;
    $56 = (_sqlite3TwoPartName($53,$54,$55,$pTableName)|0);
    HEAP32[$iDb>>2] = $56;
    $57 = HEAP32[$iDb>>2]|0;
    $58 = ($57|0)>=(0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[$iDb>>2]|0;
    $60 = HEAP32[$db>>2]|0;
    $61 = ((($60)) + 16|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + ($59<<4)|0);
    $64 = HEAP32[$63>>2]|0;
    HEAP32[$zDb>>2] = $64;
    $65 = HEAP32[$db>>2]|0;
    $66 = HEAP32[$pTableName>>2]|0;
    $67 = (_sqlite3NameFromToken($65,$66)|0);
    HEAP32[$z>>2] = $67;
    $68 = HEAP32[$z>>2]|0;
    $69 = ($68|0)!=(0|0);
    if (!($69)) {
     break;
    }
    $70 = HEAP32[$db>>2]|0;
    $71 = HEAP32[$z>>2]|0;
    $72 = HEAP32[$zDb>>2]|0;
    $73 = (_sqlite3FindIndex($70,$71,$72)|0);
    HEAP32[$pIdx>>2] = $73;
    $74 = ($73|0)!=(0|0);
    $75 = HEAP32[$0>>2]|0;
    if ($74) {
     $76 = HEAP32[$pIdx>>2]|0;
     $77 = ((($76)) + 12|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP32[$pIdx>>2]|0;
     _analyzeTable($75,$78,$79);
    } else {
     $80 = HEAP32[$z>>2]|0;
     $81 = HEAP32[$zDb>>2]|0;
     $82 = (_sqlite3LocateTable($75,0,$80,$81)|0);
     HEAP32[$pTab>>2] = $82;
     $83 = ($82|0)!=(0|0);
     if ($83) {
      $84 = HEAP32[$0>>2]|0;
      $85 = HEAP32[$pTab>>2]|0;
      _analyzeTable($84,$85,0);
     }
    }
    $86 = HEAP32[$db>>2]|0;
    $87 = HEAP32[$z>>2]|0;
    _sqlite3DbFree($86,$87);
    break;
   }
   $25 = HEAP32[$db>>2]|0;
   $26 = HEAP32[$1>>2]|0;
   $27 = (_sqlite3FindDb($25,$26)|0);
   HEAP32[$iDb>>2] = $27;
   $28 = HEAP32[$iDb>>2]|0;
   $29 = ($28|0)>=(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = HEAP32[$iDb>>2]|0;
    _analyzeDatabase($30,$31);
    break;
   }
   $32 = HEAP32[$db>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = (_sqlite3NameFromToken($32,$33)|0);
   HEAP32[$z>>2] = $34;
   $35 = HEAP32[$z>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    $38 = HEAP32[$z>>2]|0;
    $39 = (_sqlite3FindIndex($37,$38,0)|0);
    HEAP32[$pIdx>>2] = $39;
    $40 = ($39|0)!=(0|0);
    $41 = HEAP32[$0>>2]|0;
    if ($40) {
     $42 = HEAP32[$pIdx>>2]|0;
     $43 = ((($42)) + 12|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$pIdx>>2]|0;
     _analyzeTable($41,$44,$45);
    } else {
     $46 = HEAP32[$z>>2]|0;
     $47 = (_sqlite3LocateTable($41,0,$46,0)|0);
     HEAP32[$pTab>>2] = $47;
     $48 = ($47|0)!=(0|0);
     if ($48) {
      $49 = HEAP32[$0>>2]|0;
      $50 = HEAP32[$pTab>>2]|0;
      _analyzeTable($49,$50,0);
     }
    }
    $51 = HEAP32[$db>>2]|0;
    $52 = HEAP32[$z>>2]|0;
    _sqlite3DbFree($51,$52);
   }
  }
 } while(0);
 $88 = HEAP32[$0>>2]|0;
 $89 = (_sqlite3GetVdbe($88)|0);
 HEAP32[$v>>2] = $89;
 $90 = HEAP32[$v>>2]|0;
 $91 = ($90|0)!=(0|0);
 if (!($91)) {
  STACKTOP = sp;return;
 }
 $92 = HEAP32[$v>>2]|0;
 (_sqlite3VdbeAddOp0($92,150)|0);
 STACKTOP = sp;return;
}
function _sqlite3AlterRenameTable($pParse,$pSrc,$pName) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $nTabName = 0, $p = 0, $pFrom = 0, $pTab = 0, $pVTab = 0, $savedDbFlags = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer22 = 0, $vararg_buffer27 = 0, $vararg_buffer4 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0;
 var $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $vararg_ptr20 = 0, $vararg_ptr21 = 0, $vararg_ptr25 = 0, $vararg_ptr26 = 0, $vararg_ptr30 = 0, $vararg_ptr31 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $zDb = 0, $zName = 0, $zTabName = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer27 = sp + 96|0;
 $vararg_buffer22 = sp + 80|0;
 $vararg_buffer11 = sp + 40|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 172|0;
 $1 = sp + 168|0;
 $2 = sp + 164|0;
 $iDb = sp + 160|0;
 $zDb = sp + 156|0;
 $pTab = sp + 152|0;
 $zName = sp + 148|0;
 $db = sp + 144|0;
 $nTabName = sp + 140|0;
 $zTabName = sp + 136|0;
 $v = sp + 132|0;
 $zWhere = sp + 128|0;
 $pVTab = sp + 124|0;
 $savedDbFlags = sp + 120|0;
 $i = sp + 116|0;
 $p = sp + 112|0;
 $pFrom = sp + 108|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pName;
 HEAP32[$zName>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 HEAP32[$zWhere>>2] = 0;
 HEAP32[$pVTab>>2] = 0;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$savedDbFlags>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 69|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 8|0);
   $15 = (_sqlite3LocateTableItem($12,0,$14)|0);
   HEAP32[$pTab>>2] = $15;
   $16 = HEAP32[$pTab>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$18>>2]|0;
    $20 = HEAP32[$pTab>>2]|0;
    $21 = ((($20)) + 64|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (_sqlite3SchemaToIndex($19,$22)|0);
    HEAP32[$iDb>>2] = $23;
    $24 = HEAP32[$iDb>>2]|0;
    $25 = HEAP32[$db>>2]|0;
    $26 = ((($25)) + 16|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = (($27) + ($24<<4)|0);
    $29 = HEAP32[$28>>2]|0;
    HEAP32[$zDb>>2] = $29;
    $30 = HEAP32[$db>>2]|0;
    $31 = ((($30)) + 24|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 2097152;
    HEAP32[$31>>2] = $33;
    $34 = HEAP32[$db>>2]|0;
    $35 = HEAP32[$2>>2]|0;
    $36 = (_sqlite3NameFromToken($34,$35)|0);
    HEAP32[$zName>>2] = $36;
    $37 = HEAP32[$zName>>2]|0;
    $38 = ($37|0)!=(0|0);
    if ($38) {
     $39 = HEAP32[$db>>2]|0;
     $40 = HEAP32[$zName>>2]|0;
     $41 = HEAP32[$zDb>>2]|0;
     $42 = (_sqlite3FindTable($39,$40,$41)|0);
     $43 = ($42|0)!=(0|0);
     if (!($43)) {
      $44 = HEAP32[$db>>2]|0;
      $45 = HEAP32[$zName>>2]|0;
      $46 = HEAP32[$zDb>>2]|0;
      $47 = (_sqlite3FindIndex($44,$45,$46)|0);
      $48 = ($47|0)!=(0|0);
      if (!($48)) {
       $51 = HEAP32[$0>>2]|0;
       $52 = HEAP32[$pTab>>2]|0;
       $53 = HEAP32[$52>>2]|0;
       $54 = (_isSystemTable($51,$53)|0);
       $55 = (0)!=($54|0);
       if ($55) {
        break;
       }
       $56 = HEAP32[$0>>2]|0;
       $57 = HEAP32[$zName>>2]|0;
       $58 = (_sqlite3CheckObjectName($56,$57)|0);
       $59 = (0)!=($58|0);
       if ($59) {
        break;
       }
       $60 = HEAP32[$pTab>>2]|0;
       $61 = ((($60)) + 12|0);
       $62 = HEAP32[$61>>2]|0;
       $63 = ($62|0)!=(0|0);
       $64 = HEAP32[$0>>2]|0;
       if ($63) {
        $65 = HEAP32[$pTab>>2]|0;
        $66 = HEAP32[$65>>2]|0;
        HEAP32[$vararg_buffer1>>2] = $66;
        _sqlite3ErrorMsg($64,28337,$vararg_buffer1);
        break;
       }
       $67 = HEAP32[$zDb>>2]|0;
       $68 = HEAP32[$pTab>>2]|0;
       $69 = HEAP32[$68>>2]|0;
       $70 = (_sqlite3AuthCheck($64,26,$67,$69,0)|0);
       $71 = ($70|0)!=(0);
       if ($71) {
        break;
       }
       $72 = HEAP32[$0>>2]|0;
       $73 = HEAP32[$pTab>>2]|0;
       $74 = (_sqlite3ViewGetColumnNames($72,$73)|0);
       $75 = ($74|0)!=(0);
       if ($75) {
        break;
       }
       $76 = HEAP32[$pTab>>2]|0;
       $77 = ((($76)) + 42|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78&255;
       $80 = $79 & 16;
       $81 = ($80|0)!=(0);
       if ($81) {
        $82 = HEAP32[$db>>2]|0;
        $83 = HEAP32[$pTab>>2]|0;
        $84 = (_sqlite3GetVTable($82,$83)|0);
        HEAP32[$pVTab>>2] = $84;
        $85 = HEAP32[$pVTab>>2]|0;
        $86 = ((($85)) + 8|0);
        $87 = HEAP32[$86>>2]|0;
        $88 = HEAP32[$87>>2]|0;
        $89 = ((($88)) + 76|0);
        $90 = HEAP32[$89>>2]|0;
        $91 = ($90|0)==(0|0);
        if ($91) {
         HEAP32[$pVTab>>2] = 0;
        }
       }
       $92 = HEAP32[$0>>2]|0;
       $93 = (_sqlite3GetVdbe($92)|0);
       HEAP32[$v>>2] = $93;
       $94 = HEAP32[$v>>2]|0;
       $95 = ($94|0)==(0|0);
       if ($95) {
        break;
       }
       $96 = HEAP32[$0>>2]|0;
       $97 = HEAP32[$pVTab>>2]|0;
       $98 = ($97|0)!=(0|0);
       $99 = $98&1;
       $100 = HEAP32[$iDb>>2]|0;
       _sqlite3BeginWriteOperation($96,$99,$100);
       $101 = HEAP32[$0>>2]|0;
       $102 = HEAP32[$iDb>>2]|0;
       _sqlite3ChangeCookie($101,$102);
       $103 = HEAP32[$pVTab>>2]|0;
       $104 = ($103|0)!=(0|0);
       if ($104) {
        $105 = HEAP32[$0>>2]|0;
        $106 = ((($105)) + 44|0);
        $107 = HEAP32[$106>>2]|0;
        $108 = (($107) + 1)|0;
        HEAP32[$106>>2] = $108;
        HEAP32[$i>>2] = $108;
        $109 = HEAP32[$v>>2]|0;
        $110 = HEAP32[$i>>2]|0;
        $111 = HEAP32[$zName>>2]|0;
        (_sqlite3VdbeLoadString($109,$110,$111)|0);
        $112 = HEAP32[$v>>2]|0;
        $113 = HEAP32[$i>>2]|0;
        $114 = HEAP32[$pVTab>>2]|0;
        (_sqlite3VdbeAddOp4($112,157,$113,0,0,$114,-10)|0);
        $115 = HEAP32[$0>>2]|0;
        _sqlite3MayAbort($115);
       }
       $116 = HEAP32[$pTab>>2]|0;
       $117 = HEAP32[$116>>2]|0;
       HEAP32[$zTabName>>2] = $117;
       $118 = HEAP32[$zTabName>>2]|0;
       $119 = (_sqlite3Utf8CharLen81($118,-1)|0);
       HEAP32[$nTabName>>2] = $119;
       $120 = HEAP32[$db>>2]|0;
       $121 = ((($120)) + 24|0);
       $122 = HEAP32[$121>>2]|0;
       $123 = $122 & 524288;
       $124 = ($123|0)!=(0);
       if ($124) {
        $125 = HEAP32[$0>>2]|0;
        $126 = HEAP32[$pTab>>2]|0;
        $127 = (_whereForeignKeys($125,$126)|0);
        HEAP32[$zWhere>>2] = $127;
        $128 = ($127|0)!=(0|0);
        if ($128) {
         $129 = HEAP32[$0>>2]|0;
         $130 = HEAP32[$zDb>>2]|0;
         $131 = HEAP32[$iDb>>2]|0;
         $132 = ($131|0)==(1);
         $133 = $132 ? 19905 : 19924;
         $134 = HEAP32[$zTabName>>2]|0;
         $135 = HEAP32[$zName>>2]|0;
         $136 = HEAP32[$zWhere>>2]|0;
         HEAP32[$vararg_buffer4>>2] = $130;
         $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
         HEAP32[$vararg_ptr7>>2] = $133;
         $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
         HEAP32[$vararg_ptr8>>2] = $134;
         $vararg_ptr9 = ((($vararg_buffer4)) + 12|0);
         HEAP32[$vararg_ptr9>>2] = $135;
         $vararg_ptr10 = ((($vararg_buffer4)) + 16|0);
         HEAP32[$vararg_ptr10>>2] = $136;
         _sqlite3NestedParse($129,28364,$vararg_buffer4);
         $137 = HEAP32[$db>>2]|0;
         $138 = HEAP32[$zWhere>>2]|0;
         _sqlite3DbFree($137,$138);
        }
       }
       $139 = HEAP32[$0>>2]|0;
       $140 = HEAP32[$zDb>>2]|0;
       $141 = HEAP32[$iDb>>2]|0;
       $142 = ($141|0)==(1);
       $143 = $142 ? 19905 : 19924;
       $144 = HEAP32[$zName>>2]|0;
       $145 = HEAP32[$zName>>2]|0;
       $146 = HEAP32[$zName>>2]|0;
       $147 = HEAP32[$zName>>2]|0;
       $148 = HEAP32[$zName>>2]|0;
       $149 = HEAP32[$nTabName>>2]|0;
       $150 = HEAP32[$zTabName>>2]|0;
       HEAP32[$vararg_buffer11>>2] = $140;
       $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
       HEAP32[$vararg_ptr14>>2] = $143;
       $vararg_ptr15 = ((($vararg_buffer11)) + 8|0);
       HEAP32[$vararg_ptr15>>2] = $144;
       $vararg_ptr16 = ((($vararg_buffer11)) + 12|0);
       HEAP32[$vararg_ptr16>>2] = $145;
       $vararg_ptr17 = ((($vararg_buffer11)) + 16|0);
       HEAP32[$vararg_ptr17>>2] = $146;
       $vararg_ptr18 = ((($vararg_buffer11)) + 20|0);
       HEAP32[$vararg_ptr18>>2] = $147;
       $vararg_ptr19 = ((($vararg_buffer11)) + 24|0);
       HEAP32[$vararg_ptr19>>2] = $148;
       $vararg_ptr20 = ((($vararg_buffer11)) + 28|0);
       HEAP32[$vararg_ptr20>>2] = $149;
       $vararg_ptr21 = ((($vararg_buffer11)) + 32|0);
       HEAP32[$vararg_ptr21>>2] = $150;
       _sqlite3NestedParse($139,28433,$vararg_buffer11);
       $151 = HEAP32[$db>>2]|0;
       $152 = HEAP32[$zDb>>2]|0;
       $153 = (_sqlite3FindTable($151,28817,$152)|0);
       $154 = ($153|0)!=(0|0);
       if ($154) {
        $155 = HEAP32[$0>>2]|0;
        $156 = HEAP32[$zDb>>2]|0;
        $157 = HEAP32[$zName>>2]|0;
        $158 = HEAP32[$pTab>>2]|0;
        $159 = HEAP32[$158>>2]|0;
        HEAP32[$vararg_buffer22>>2] = $156;
        $vararg_ptr25 = ((($vararg_buffer22)) + 4|0);
        HEAP32[$vararg_ptr25>>2] = $157;
        $vararg_ptr26 = ((($vararg_buffer22)) + 8|0);
        HEAP32[$vararg_ptr26>>2] = $159;
        _sqlite3NestedParse($155,28833,$vararg_buffer22);
       }
       $160 = HEAP32[$0>>2]|0;
       $161 = HEAP32[$pTab>>2]|0;
       $162 = (_whereTempTriggers($160,$161)|0);
       HEAP32[$zWhere>>2] = $162;
       $163 = ($162|0)!=(0|0);
       if ($163) {
        $164 = HEAP32[$0>>2]|0;
        $165 = HEAP32[$zName>>2]|0;
        $166 = HEAP32[$zName>>2]|0;
        $167 = HEAP32[$zWhere>>2]|0;
        HEAP32[$vararg_buffer27>>2] = $165;
        $vararg_ptr30 = ((($vararg_buffer27)) + 4|0);
        HEAP32[$vararg_ptr30>>2] = $166;
        $vararg_ptr31 = ((($vararg_buffer27)) + 8|0);
        HEAP32[$vararg_ptr31>>2] = $167;
        _sqlite3NestedParse($164,28891,$vararg_buffer27);
        $168 = HEAP32[$db>>2]|0;
        $169 = HEAP32[$zWhere>>2]|0;
        _sqlite3DbFree($168,$169);
       }
       $170 = HEAP32[$db>>2]|0;
       $171 = ((($170)) + 24|0);
       $172 = HEAP32[$171>>2]|0;
       $173 = $172 & 524288;
       $174 = ($173|0)!=(0);
       L33: do {
        if ($174) {
         $175 = HEAP32[$pTab>>2]|0;
         $176 = (_sqlite3FkReferences($175)|0);
         HEAP32[$p>>2] = $176;
         while(1) {
          $177 = HEAP32[$p>>2]|0;
          $178 = ($177|0)!=(0|0);
          if (!($178)) {
           break L33;
          }
          $179 = HEAP32[$p>>2]|0;
          $180 = HEAP32[$179>>2]|0;
          HEAP32[$pFrom>>2] = $180;
          $181 = HEAP32[$pFrom>>2]|0;
          $182 = HEAP32[$pTab>>2]|0;
          $183 = ($181|0)!=($182|0);
          if ($183) {
           $184 = HEAP32[$0>>2]|0;
           $185 = HEAP32[$p>>2]|0;
           $186 = HEAP32[$185>>2]|0;
           $187 = HEAP32[$pFrom>>2]|0;
           $188 = HEAP32[$187>>2]|0;
           _reloadTableSchema($184,$186,$188);
          }
          $189 = HEAP32[$p>>2]|0;
          $190 = ((($189)) + 12|0);
          $191 = HEAP32[$190>>2]|0;
          HEAP32[$p>>2] = $191;
         }
        }
       } while(0);
       $192 = HEAP32[$0>>2]|0;
       $193 = HEAP32[$pTab>>2]|0;
       $194 = HEAP32[$zName>>2]|0;
       _reloadTableSchema($192,$193,$194);
       break;
      }
     }
     $49 = HEAP32[$0>>2]|0;
     $50 = HEAP32[$zName>>2]|0;
     HEAP32[$vararg_buffer>>2] = $50;
     _sqlite3ErrorMsg($49,28278,$vararg_buffer);
    }
   }
  }
 } while(0);
 $195 = HEAP32[$db>>2]|0;
 $196 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($195,$196);
 $197 = HEAP32[$db>>2]|0;
 $198 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($197,$198);
 $199 = HEAP32[$savedDbFlags>>2]|0;
 $200 = HEAP32[$db>>2]|0;
 $201 = ((($200)) + 24|0);
 HEAP32[$201>>2] = $199;
 STACKTOP = sp;return;
}
function _sqlite3AlterFinishAddColumn($pParse,$pColDef) {
 $pParse = $pParse|0;
 $pColDef = $pColDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $or$cond = 0, $or$cond3 = 0, $pCol = 0, $pDflt = 0, $pNew = 0, $pTab = 0, $pVal = 0, $r1 = 0, $rc = 0, $savedDbFlags = 0, $v = 0, $vararg_buffer = 0;
 var $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer5 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, $vararg_ptr15 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr18 = 0, $vararg_ptr19 = 0, $zCol = 0, $zDb = 0, $zEnd = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer9 = sp + 24|0;
 $vararg_buffer7 = sp + 16|0;
 $vararg_buffer5 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $pNew = sp + 120|0;
 $pTab = sp + 116|0;
 $iDb = sp + 112|0;
 $zDb = sp + 108|0;
 $zTab = sp + 104|0;
 $zCol = sp + 100|0;
 $pCol = sp + 96|0;
 $pDflt = sp + 92|0;
 $db = sp + 88|0;
 $v = sp + 84|0;
 $r1 = sp + 80|0;
 $pVal = sp + 76|0;
 $rc = sp + 72|0;
 $zEnd = sp + 68|0;
 $savedDbFlags = sp + 64|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pColDef;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 69|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)!=(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 440|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$pNew>>2] = $18;
 $19 = HEAP32[$db>>2]|0;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ((($20)) + 64|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3SchemaToIndex($19,$22)|0);
 HEAP32[$iDb>>2] = $23;
 $24 = HEAP32[$iDb>>2]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24<<4)|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$zDb>>2] = $29;
 $30 = HEAP32[$pNew>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ((($31)) + 16|0);
 HEAP32[$zTab>>2] = $32;
 $33 = HEAP32[$pNew>>2]|0;
 $34 = ((($33)) + 34|0);
 $35 = HEAP16[$34>>1]|0;
 $36 = $35 << 16 >> 16;
 $37 = (($36) - 1)|0;
 $38 = HEAP32[$pNew>>2]|0;
 $39 = ((($38)) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + ($37<<4)|0);
 HEAP32[$pCol>>2] = $41;
 $42 = HEAP32[$pCol>>2]|0;
 $43 = ((($42)) + 4|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pDflt>>2] = $44;
 $45 = HEAP32[$db>>2]|0;
 $46 = HEAP32[$zTab>>2]|0;
 $47 = HEAP32[$zDb>>2]|0;
 $48 = (_sqlite3FindTable($45,$46,$47)|0);
 HEAP32[$pTab>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$zDb>>2]|0;
 $51 = HEAP32[$pTab>>2]|0;
 $52 = HEAP32[$51>>2]|0;
 $53 = (_sqlite3AuthCheck($49,26,$50,$52,0)|0);
 $54 = ($53|0)!=(0);
 if ($54) {
  STACKTOP = sp;return;
 }
 $55 = HEAP32[$pDflt>>2]|0;
 $56 = ($55|0)!=(0|0);
 if ($56) {
  $57 = HEAP32[$pDflt>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = ($61|0)==(101);
  if ($62) {
   HEAP32[$pDflt>>2] = 0;
  }
 }
 $63 = HEAP32[$pCol>>2]|0;
 $64 = ((($63)) + 15|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = $65&255;
 $67 = $66 & 1;
 $68 = ($67|0)!=(0);
 if ($68) {
  $69 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($69,27880,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $70 = HEAP32[$pNew>>2]|0;
 $71 = ((($70)) + 8|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = ($72|0)!=(0|0);
 if ($73) {
  $74 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($74,27912,$vararg_buffer5);
  STACKTOP = sp;return;
 }
 $75 = HEAP32[$db>>2]|0;
 $76 = ((($75)) + 24|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = $77 & 524288;
 $79 = ($78|0)!=(0);
 if ($79) {
  $80 = HEAP32[$pNew>>2]|0;
  $81 = ((($80)) + 16|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = ($82|0)!=(0|0);
  $84 = HEAP32[$pDflt>>2]|0;
  $85 = ($84|0)!=(0|0);
  $or$cond = $83 & $85;
  if ($or$cond) {
   $86 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($86,27939,$vararg_buffer7);
   STACKTOP = sp;return;
  }
 }
 $87 = HEAP32[$pCol>>2]|0;
 $88 = ((($87)) + 12|0);
 $89 = HEAP8[$88>>0]|0;
 $90 = $89&255;
 $91 = ($90|0)==(0);
 $92 = HEAP32[$pDflt>>2]|0;
 $93 = ($92|0)!=(0|0);
 $or$cond3 = $91 | $93;
 if (!($or$cond3)) {
  $94 = HEAP32[$0>>2]|0;
  _sqlite3ErrorMsg($94,27998,$vararg_buffer9);
  STACKTOP = sp;return;
 }
 $95 = HEAP32[$pDflt>>2]|0;
 $96 = ($95|0)!=(0|0);
 do {
  if ($96) {
   HEAP32[$pVal>>2] = 0;
   $97 = HEAP32[$db>>2]|0;
   $98 = HEAP32[$pDflt>>2]|0;
   $99 = (_sqlite3ValueFromExpr($97,$98,1,65,$pVal)|0);
   HEAP32[$rc>>2] = $99;
   $100 = HEAP32[$rc>>2]|0;
   $101 = ($100|0)!=(0);
   if ($101) {
    STACKTOP = sp;return;
   }
   $102 = HEAP32[$pVal>>2]|0;
   $103 = ($102|0)!=(0|0);
   if ($103) {
    $105 = HEAP32[$pVal>>2]|0;
    _sqlite3ValueFree($105);
    break;
   }
   $104 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($104,28051,$vararg_buffer11);
   STACKTOP = sp;return;
  }
 } while(0);
 $106 = HEAP32[$db>>2]|0;
 $107 = HEAP32[$1>>2]|0;
 $108 = HEAP32[$107>>2]|0;
 $109 = HEAP32[$1>>2]|0;
 $110 = ((($109)) + 4|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = (_sqlite3DbStrNDup($106,$108,$111,0)|0);
 HEAP32[$zCol>>2] = $112;
 $113 = HEAP32[$zCol>>2]|0;
 $114 = ($113|0)!=(0|0);
 if ($114) {
  $115 = HEAP32[$1>>2]|0;
  $116 = ((($115)) + 4|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = (($117) - 1)|0;
  $119 = HEAP32[$zCol>>2]|0;
  $120 = (($119) + ($118)|0);
  HEAP32[$zEnd>>2] = $120;
  $121 = HEAP32[$db>>2]|0;
  $122 = ((($121)) + 24|0);
  $123 = HEAP32[$122>>2]|0;
  HEAP32[$savedDbFlags>>2] = $123;
  while(1) {
   $124 = HEAP32[$zEnd>>2]|0;
   $125 = HEAP32[$zCol>>2]|0;
   $126 = ($124>>>0)>($125>>>0);
   if (!($126)) {
    break;
   }
   $127 = HEAP32[$zEnd>>2]|0;
   $128 = HEAP8[$127>>0]|0;
   $129 = $128 << 24 >> 24;
   $130 = ($129|0)==(59);
   if (!($130)) {
    $131 = HEAP32[$zEnd>>2]|0;
    $132 = HEAP8[$131>>0]|0;
    $133 = $132&255;
    $134 = (18403 + ($133)|0);
    $135 = HEAP8[$134>>0]|0;
    $136 = $135&255;
    $137 = $136 & 1;
    $138 = ($137|0)!=(0);
    if (!($138)) {
     break;
    }
   }
   $139 = HEAP32[$zEnd>>2]|0;
   $140 = ((($139)) + -1|0);
   HEAP32[$zEnd>>2] = $140;
   HEAP8[$139>>0] = 0;
  }
  $141 = HEAP32[$db>>2]|0;
  $142 = ((($141)) + 24|0);
  $143 = HEAP32[$142>>2]|0;
  $144 = $143 | 2097152;
  HEAP32[$142>>2] = $144;
  $145 = HEAP32[$0>>2]|0;
  $146 = HEAP32[$zDb>>2]|0;
  $147 = HEAP32[$iDb>>2]|0;
  $148 = ($147|0)==(1);
  $149 = $148 ? 19905 : 19924;
  $150 = HEAP32[$pNew>>2]|0;
  $151 = ((($150)) + 44|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = HEAP32[$zCol>>2]|0;
  $154 = HEAP32[$pNew>>2]|0;
  $155 = ((($154)) + 44|0);
  $156 = HEAP32[$155>>2]|0;
  $157 = (($156) + 1)|0;
  $158 = HEAP32[$zTab>>2]|0;
  HEAP32[$vararg_buffer13>>2] = $146;
  $vararg_ptr15 = ((($vararg_buffer13)) + 4|0);
  HEAP32[$vararg_ptr15>>2] = $149;
  $vararg_ptr16 = ((($vararg_buffer13)) + 8|0);
  HEAP32[$vararg_ptr16>>2] = $152;
  $vararg_ptr17 = ((($vararg_buffer13)) + 12|0);
  HEAP32[$vararg_ptr17>>2] = $153;
  $vararg_ptr18 = ((($vararg_buffer13)) + 16|0);
  HEAP32[$vararg_ptr18>>2] = $157;
  $vararg_ptr19 = ((($vararg_buffer13)) + 20|0);
  HEAP32[$vararg_ptr19>>2] = $158;
  _sqlite3NestedParse($145,28097,$vararg_buffer13);
  $159 = HEAP32[$db>>2]|0;
  $160 = HEAP32[$zCol>>2]|0;
  _sqlite3DbFree($159,$160);
  $161 = HEAP32[$savedDbFlags>>2]|0;
  $162 = HEAP32[$db>>2]|0;
  $163 = ((($162)) + 24|0);
  HEAP32[$163>>2] = $161;
 }
 $164 = HEAP32[$0>>2]|0;
 $165 = (_sqlite3GetTempReg($164)|0);
 HEAP32[$r1>>2] = $165;
 $166 = HEAP32[$v>>2]|0;
 $167 = HEAP32[$iDb>>2]|0;
 $168 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp3($166,101,$167,$168,2)|0);
 $169 = HEAP32[$v>>2]|0;
 $170 = HEAP32[$iDb>>2]|0;
 _sqlite3VdbeUsesBtree($169,$170);
 $171 = HEAP32[$v>>2]|0;
 $172 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp2($171,91,$172,-2)|0);
 $173 = HEAP32[$v>>2]|0;
 $174 = HEAP32[$r1>>2]|0;
 $175 = HEAP32[$v>>2]|0;
 $176 = (_sqlite3VdbeCurrentAddr($175)|0);
 $177 = (($176) + 2)|0;
 (_sqlite3VdbeAddOp2($173,66,$174,$177)|0);
 $178 = HEAP32[$v>>2]|0;
 $179 = HEAP32[$iDb>>2]|0;
 (_sqlite3VdbeAddOp3($178,102,$179,2,3)|0);
 $180 = HEAP32[$0>>2]|0;
 $181 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($180,$181);
 $182 = HEAP32[$0>>2]|0;
 $183 = HEAP32[$pTab>>2]|0;
 $184 = HEAP32[$pTab>>2]|0;
 $185 = HEAP32[$184>>2]|0;
 _reloadTableSchema($182,$183,$185);
 STACKTOP = sp;return;
}
function _sqlite3AlterBeginAddColumn($pParse,$pSrc) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iDb = 0, $nAlloc = 0, $pCol = 0, $pNew = 0;
 var $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $pNew = sp + 48|0;
 $pTab = sp + 44|0;
 $v = sp + 40|0;
 $iDb = sp + 36|0;
 $i = sp + 32|0;
 $nAlloc = sp + 28|0;
 $db = sp + 24|0;
 $pCol = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 69|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 do {
  if (!($7)) {
   $8 = HEAP32[$0>>2]|0;
   $9 = HEAP32[$1>>2]|0;
   $10 = ((($9)) + 8|0);
   $11 = (_sqlite3LocateTableItem($8,0,$10)|0);
   HEAP32[$pTab>>2] = $11;
   $12 = HEAP32[$pTab>>2]|0;
   $13 = ($12|0)!=(0|0);
   if ($13) {
    $14 = HEAP32[$pTab>>2]|0;
    $15 = ((($14)) + 42|0);
    $16 = HEAP8[$15>>0]|0;
    $17 = $16&255;
    $18 = $17 & 16;
    $19 = ($18|0)!=(0);
    if ($19) {
     $20 = HEAP32[$0>>2]|0;
     _sqlite3ErrorMsg($20,27691,$vararg_buffer);
     break;
    }
    $21 = HEAP32[$pTab>>2]|0;
    $22 = ((($21)) + 12|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = ($23|0)!=(0|0);
    $25 = HEAP32[$0>>2]|0;
    if ($24) {
     _sqlite3ErrorMsg($25,27725,$vararg_buffer1);
     break;
    }
    $26 = HEAP32[$pTab>>2]|0;
    $27 = HEAP32[$26>>2]|0;
    $28 = (_isSystemTable($25,$27)|0);
    $29 = (0)!=($28|0);
    if (!($29)) {
     $30 = HEAP32[$db>>2]|0;
     $31 = HEAP32[$pTab>>2]|0;
     $32 = ((($31)) + 64|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = (_sqlite3SchemaToIndex($30,$33)|0);
     HEAP32[$iDb>>2] = $34;
     $35 = HEAP32[$db>>2]|0;
     $36 = (_sqlite3DbMallocZero($35,72,0)|0);
     HEAP32[$pNew>>2] = $36;
     $37 = HEAP32[$pNew>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$pNew>>2]|0;
      $40 = HEAP32[$0>>2]|0;
      $41 = ((($40)) + 440|0);
      HEAP32[$41>>2] = $39;
      $42 = HEAP32[$pNew>>2]|0;
      $43 = ((($42)) + 36|0);
      HEAP16[$43>>1] = 1;
      $44 = HEAP32[$pTab>>2]|0;
      $45 = ((($44)) + 34|0);
      $46 = HEAP16[$45>>1]|0;
      $47 = HEAP32[$pNew>>2]|0;
      $48 = ((($47)) + 34|0);
      HEAP16[$48>>1] = $46;
      $49 = HEAP32[$pNew>>2]|0;
      $50 = ((($49)) + 34|0);
      $51 = HEAP16[$50>>1]|0;
      $52 = $51 << 16 >> 16;
      $53 = (($52) - 1)|0;
      $54 = (($53|0) / 8)&-1;
      $55 = $54<<3;
      $56 = (($55) + 8)|0;
      HEAP32[$nAlloc>>2] = $56;
      $57 = HEAP32[$db>>2]|0;
      $58 = HEAP32[$nAlloc>>2]|0;
      $59 = $58<<4;
      $60 = (_sqlite3DbMallocZero($57,$59,0)|0);
      $61 = HEAP32[$pNew>>2]|0;
      $62 = ((($61)) + 4|0);
      HEAP32[$62>>2] = $60;
      $63 = HEAP32[$db>>2]|0;
      $64 = HEAP32[$pTab>>2]|0;
      $65 = HEAP32[$64>>2]|0;
      HEAP32[$vararg_buffer3>>2] = $65;
      $66 = (_sqlite3MPrintf($63,27755,$vararg_buffer3)|0);
      $67 = HEAP32[$pNew>>2]|0;
      HEAP32[$67>>2] = $66;
      $68 = HEAP32[$pNew>>2]|0;
      $69 = ((($68)) + 4|0);
      $70 = HEAP32[$69>>2]|0;
      $71 = ($70|0)!=(0|0);
      if ($71) {
       $72 = HEAP32[$pNew>>2]|0;
       $73 = HEAP32[$72>>2]|0;
       $74 = ($73|0)!=(0|0);
       if ($74) {
        $75 = HEAP32[$pNew>>2]|0;
        $76 = ((($75)) + 4|0);
        $77 = HEAP32[$76>>2]|0;
        $78 = HEAP32[$pTab>>2]|0;
        $79 = ((($78)) + 4|0);
        $80 = HEAP32[$79>>2]|0;
        $81 = HEAP32[$pNew>>2]|0;
        $82 = ((($81)) + 34|0);
        $83 = HEAP16[$82>>1]|0;
        $84 = $83 << 16 >> 16;
        $85 = $84<<4;
        _memcpy(($77|0),($80|0),($85|0))|0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $86 = HEAP32[$i>>2]|0;
         $87 = HEAP32[$pNew>>2]|0;
         $88 = ((($87)) + 34|0);
         $89 = HEAP16[$88>>1]|0;
         $90 = $89 << 16 >> 16;
         $91 = ($86|0)<($90|0);
         if (!($91)) {
          break;
         }
         $92 = HEAP32[$i>>2]|0;
         $93 = HEAP32[$pNew>>2]|0;
         $94 = ((($93)) + 4|0);
         $95 = HEAP32[$94>>2]|0;
         $96 = (($95) + ($92<<4)|0);
         HEAP32[$pCol>>2] = $96;
         $97 = HEAP32[$db>>2]|0;
         $98 = HEAP32[$pCol>>2]|0;
         $99 = HEAP32[$98>>2]|0;
         $100 = (_sqlite3DbStrDup($97,$99)|0);
         $101 = HEAP32[$pCol>>2]|0;
         HEAP32[$101>>2] = $100;
         $102 = HEAP32[$pCol>>2]|0;
         $103 = ((($102)) + 8|0);
         HEAP32[$103>>2] = 0;
         $104 = HEAP32[$pCol>>2]|0;
         $105 = ((($104)) + 4|0);
         HEAP32[$105>>2] = 0;
         $106 = HEAP32[$i>>2]|0;
         $107 = (($106) + 1)|0;
         HEAP32[$i>>2] = $107;
        }
        $108 = HEAP32[$iDb>>2]|0;
        $109 = HEAP32[$db>>2]|0;
        $110 = ((($109)) + 16|0);
        $111 = HEAP32[$110>>2]|0;
        $112 = (($111) + ($108<<4)|0);
        $113 = ((($112)) + 12|0);
        $114 = HEAP32[$113>>2]|0;
        $115 = HEAP32[$pNew>>2]|0;
        $116 = ((($115)) + 64|0);
        HEAP32[$116>>2] = $114;
        $117 = HEAP32[$pTab>>2]|0;
        $118 = ((($117)) + 44|0);
        $119 = HEAP32[$118>>2]|0;
        $120 = HEAP32[$pNew>>2]|0;
        $121 = ((($120)) + 44|0);
        HEAP32[$121>>2] = $119;
        $122 = HEAP32[$pNew>>2]|0;
        $123 = ((($122)) + 36|0);
        HEAP16[$123>>1] = 1;
        $124 = HEAP32[$0>>2]|0;
        $125 = HEAP32[$iDb>>2]|0;
        _sqlite3BeginWriteOperation($124,0,$125);
        $126 = HEAP32[$0>>2]|0;
        $127 = (_sqlite3GetVdbe($126)|0);
        HEAP32[$v>>2] = $127;
        $128 = HEAP32[$v>>2]|0;
        $129 = ($128|0)!=(0|0);
        if ($129) {
         $130 = HEAP32[$0>>2]|0;
         $131 = HEAP32[$iDb>>2]|0;
         _sqlite3ChangeCookie($130,$131);
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 $132 = HEAP32[$db>>2]|0;
 $133 = HEAP32[$1>>2]|0;
 _sqlite3SrcListDelete($132,$133);
 STACKTOP = sp;return;
}
function _sqlite3VtabFinishParse($pParse,$pEnd) {
 $pParse = $pParse|0;
 $pEnd = $pEnd|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iDb = 0, $iReg = 0, $pOld = 0, $pSchema = 0, $pTab = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer9 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0;
 var $zName = 0, $zStmt = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $pTab = sp + 72|0;
 $db = sp + 68|0;
 $zStmt = sp + 64|0;
 $zWhere = sp + 60|0;
 $iDb = sp + 56|0;
 $iReg = sp + 52|0;
 $v = sp + 48|0;
 $pOld = sp + 44|0;
 $pSchema = sp + 40|0;
 $zName = sp + 36|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEnd;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 440|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 _addArgumentToVtab($9);
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 452|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$pTab>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)<(1);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 148|0);
 $18 = ((($17)) + 5|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 if (!($20)) {
  $21 = HEAP32[$1>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 384|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $24;
   $29 = $27;
   $30 = (($28) - ($29))|0;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($30) + ($33))|0;
   $35 = HEAP32[$0>>2]|0;
   $36 = ((($35)) + 384|0);
   $37 = ((($36)) + 4|0);
   HEAP32[$37>>2] = $34;
  }
  $38 = HEAP32[$db>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 384|0);
  HEAP32[$vararg_buffer>>2] = $40;
  $41 = (_sqlite3MPrintf($38,27552,$vararg_buffer)|0);
  HEAP32[$zStmt>>2] = $41;
  $42 = HEAP32[$db>>2]|0;
  $43 = HEAP32[$pTab>>2]|0;
  $44 = ((($43)) + 64|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3SchemaToIndex($42,$45)|0);
  HEAP32[$iDb>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = HEAP32[$iDb>>2]|0;
  $49 = HEAP32[$db>>2]|0;
  $50 = ((($49)) + 16|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + ($48<<4)|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$iDb>>2]|0;
  $55 = ($54|0)==(1);
  $56 = $55 ? 19905 : 19924;
  $57 = HEAP32[$pTab>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$pTab>>2]|0;
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$zStmt>>2]|0;
  $62 = HEAP32[$0>>2]|0;
  $63 = ((($62)) + 100|0);
  $64 = HEAP32[$63>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $53;
  $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $56;
  $vararg_ptr5 = ((($vararg_buffer1)) + 8|0);
  HEAP32[$vararg_ptr5>>2] = $58;
  $vararg_ptr6 = ((($vararg_buffer1)) + 12|0);
  HEAP32[$vararg_ptr6>>2] = $60;
  $vararg_ptr7 = ((($vararg_buffer1)) + 16|0);
  HEAP32[$vararg_ptr7>>2] = $61;
  $vararg_ptr8 = ((($vararg_buffer1)) + 20|0);
  HEAP32[$vararg_ptr8>>2] = $64;
  _sqlite3NestedParse($47,27576,$vararg_buffer1);
  $65 = HEAP32[$db>>2]|0;
  $66 = HEAP32[$zStmt>>2]|0;
  _sqlite3DbFree($65,$66);
  $67 = HEAP32[$0>>2]|0;
  $68 = (_sqlite3GetVdbe($67)|0);
  HEAP32[$v>>2] = $68;
  $69 = HEAP32[$0>>2]|0;
  $70 = HEAP32[$iDb>>2]|0;
  _sqlite3ChangeCookie($69,$70);
  $71 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($71,150)|0);
  $72 = HEAP32[$db>>2]|0;
  $73 = HEAP32[$pTab>>2]|0;
  $74 = HEAP32[$73>>2]|0;
  HEAP32[$vararg_buffer9>>2] = $74;
  $75 = (_sqlite3MPrintf($72,27664,$vararg_buffer9)|0);
  HEAP32[$zWhere>>2] = $75;
  $76 = HEAP32[$v>>2]|0;
  $77 = HEAP32[$iDb>>2]|0;
  $78 = HEAP32[$zWhere>>2]|0;
  _sqlite3VdbeAddParseSchemaOp($76,$77,$78);
  $79 = HEAP32[$0>>2]|0;
  $80 = ((($79)) + 44|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$80>>2] = $82;
  HEAP32[$iReg>>2] = $82;
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$iReg>>2]|0;
  $85 = HEAP32[$pTab>>2]|0;
  $86 = HEAP32[$85>>2]|0;
  (_sqlite3VdbeLoadString($83,$84,$86)|0);
  $87 = HEAP32[$v>>2]|0;
  $88 = HEAP32[$iDb>>2]|0;
  $89 = HEAP32[$iReg>>2]|0;
  (_sqlite3VdbeAddOp2($87,153,$88,$89)|0);
  STACKTOP = sp;return;
 }
 $90 = HEAP32[$pTab>>2]|0;
 $91 = ((($90)) + 64|0);
 $92 = HEAP32[$91>>2]|0;
 HEAP32[$pSchema>>2] = $92;
 $93 = HEAP32[$pTab>>2]|0;
 $94 = HEAP32[$93>>2]|0;
 HEAP32[$zName>>2] = $94;
 $95 = HEAP32[$pSchema>>2]|0;
 $96 = ((($95)) + 8|0);
 $97 = HEAP32[$zName>>2]|0;
 $98 = HEAP32[$pTab>>2]|0;
 $99 = (_sqlite3HashInsert($96,$97,$98)|0);
 HEAP32[$pOld>>2] = $99;
 $100 = HEAP32[$pOld>>2]|0;
 $101 = ($100|0)!=(0|0);
 if ($101) {
  $102 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($102);
  STACKTOP = sp;return;
 } else {
  $103 = HEAP32[$0>>2]|0;
  $104 = ((($103)) + 440|0);
  HEAP32[$104>>2] = 0;
  STACKTOP = sp;return;
 }
}
function _sqlite3VtabBeginParse($pParse,$pName1,$pName2,$pModuleName,$ifNotExists) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pModuleName = $pModuleName|0;
 $ifNotExists = $ifNotExists|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $pTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $iDb = sp + 8|0;
 $pTable = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName1;
 HEAP32[$2>>2] = $pName2;
 HEAP32[$3>>2] = $pModuleName;
 HEAP32[$4>>2] = $ifNotExists;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 _sqlite3StartTable($5,$6,$7,0,0,1,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 440|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pTable>>2] = $11;
 $12 = HEAP32[$pTable>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = HEAP32[$pTable>>2]|0;
 $18 = ((($17)) + 64|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$pTable>>2]|0;
 $22 = ((($21)) + 42|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = $24 | 16;
 $26 = $25&255;
 HEAP8[$22>>0] = $26;
 $27 = HEAP32[$pTable>>2]|0;
 $28 = ((($27)) + 48|0);
 HEAP32[$28>>2] = 0;
 $29 = HEAP32[$db>>2]|0;
 $30 = HEAP32[$pTable>>2]|0;
 $31 = HEAP32[$db>>2]|0;
 $32 = HEAP32[$3>>2]|0;
 $33 = (_sqlite3NameFromToken($31,$32)|0);
 _addModuleArgument($29,$30,$33);
 $34 = HEAP32[$db>>2]|0;
 $35 = HEAP32[$pTable>>2]|0;
 _addModuleArgument($34,$35,0);
 $36 = HEAP32[$db>>2]|0;
 $37 = HEAP32[$pTable>>2]|0;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$pTable>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = (_sqlite3DbStrDup($38,$40)|0);
 _addModuleArgument($36,$37,$41);
 $42 = HEAP32[$3>>2]|0;
 $43 = ((($42)) + 4|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = HEAP32[$3>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + ($44)|0);
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 384|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = $47;
 $52 = $50;
 $53 = (($51) - ($52))|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 384|0);
 $56 = ((($55)) + 4|0);
 HEAP32[$56>>2] = $53;
 $57 = HEAP32[$pTable>>2]|0;
 $58 = ((($57)) + 52|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)!=(0|0);
 if (!($60)) {
  STACKTOP = sp;return;
 }
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$pTable>>2]|0;
 $63 = HEAP32[$62>>2]|0;
 $64 = HEAP32[$pTable>>2]|0;
 $65 = ((($64)) + 52|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$66>>2]|0;
 $68 = HEAP32[$iDb>>2]|0;
 $69 = HEAP32[$0>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 $71 = ((($70)) + 16|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = (($72) + ($68<<4)|0);
 $74 = HEAP32[$73>>2]|0;
 (_sqlite3AuthCheck($61,29,$63,$67,$74)|0);
 STACKTOP = sp;return;
}
function _sqlite3VtabArgInit($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 _addArgumentToVtab($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 452|0);
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 452|0);
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VtabArgExtend($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pArg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pArg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 452|0);
 HEAP32[$pArg>>2] = $3;
 $4 = HEAP32[$pArg>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$pArg>>2]|0;
  HEAP32[$9>>2] = $8;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$pArg>>2]|0;
  $14 = ((($13)) + 4|0);
  HEAP32[$14>>2] = $12;
  STACKTOP = sp;return;
 } else {
  $15 = ((($7)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($16)|0);
  $20 = HEAP32[$pArg>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = $19;
  $23 = $21;
  $24 = (($22) - ($23))|0;
  $25 = HEAP32[$pArg>>2]|0;
  $26 = ((($25)) + 4|0);
  HEAP32[$26>>2] = $24;
  STACKTOP = sp;return;
 }
}
function _sqlite3WithAdd($pParse,$pWith,$pName,$pArglist,$pQuery) {
 $pParse = $pParse|0;
 $pWith = $pWith|0;
 $pName = $pName|0;
 $pArglist = $pArglist|0;
 $pQuery = $pQuery|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $db = 0, $i = 0, $nByte = 0, $or$cond = 0, $pNew = 0, $vararg_buffer = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $db = sp + 20|0;
 $pNew = sp + 16|0;
 $zName = sp + 12|0;
 $i = sp + 8|0;
 $nByte = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pWith;
 HEAP32[$2>>2] = $pName;
 HEAP32[$3>>2] = $pArglist;
 HEAP32[$4>>2] = $pQuery;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (_sqlite3NameFromToken($8,$9)|0);
 HEAP32[$zName>>2] = $10;
 $11 = HEAP32[$zName>>2]|0;
 $12 = ($11|0)!=(0|0);
 $13 = HEAP32[$1>>2]|0;
 $14 = ($13|0)!=(0|0);
 $or$cond = $12 & $14;
 L1: do {
  if ($or$cond) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $15 = HEAP32[$i>>2]|0;
    $16 = HEAP32[$1>>2]|0;
    $17 = HEAP32[$16>>2]|0;
    $18 = ($15|0)<($17|0);
    if (!($18)) {
     break L1;
    }
    $19 = HEAP32[$zName>>2]|0;
    $20 = HEAP32[$i>>2]|0;
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 8|0);
    $23 = (($22) + ($20<<4)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (_sqlite3StrICmp($19,$24)|0);
    $26 = ($25|0)==(0);
    if ($26) {
     $27 = HEAP32[$0>>2]|0;
     $28 = HEAP32[$zName>>2]|0;
     HEAP32[$vararg_buffer>>2] = $28;
     _sqlite3ErrorMsg($27,27484,$vararg_buffer);
    }
    $29 = HEAP32[$i>>2]|0;
    $30 = (($29) + 1)|0;
    HEAP32[$i>>2] = $30;
   }
  }
 } while(0);
 $31 = HEAP32[$1>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = $34<<4;
  $36 = (24 + ($35))|0;
  HEAP32[$nByte>>2] = $36;
  $37 = HEAP32[$db>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$nByte>>2]|0;
  $40 = ($39|0)<(0);
  $41 = $40 << 31 >> 31;
  $42 = (_sqlite3DbRealloc($37,$38,$39,$41)|0);
  HEAP32[$pNew>>2] = $42;
 } else {
  $43 = HEAP32[$db>>2]|0;
  $44 = (_sqlite3DbMallocZero($43,24,0)|0);
  HEAP32[$pNew>>2] = $44;
 }
 $45 = HEAP32[$db>>2]|0;
 $46 = ((($45)) + 69|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = ($47<<24>>24)!=(0);
 if ($48) {
  $49 = HEAP32[$db>>2]|0;
  $50 = HEAP32[$3>>2]|0;
  _sqlite3ExprListDelete($49,$50);
  $51 = HEAP32[$db>>2]|0;
  $52 = HEAP32[$4>>2]|0;
  _sqlite3SelectDelete($51,$52);
  $53 = HEAP32[$db>>2]|0;
  $54 = HEAP32[$zName>>2]|0;
  _sqlite3DbFree($53,$54);
  $55 = HEAP32[$1>>2]|0;
  HEAP32[$pNew>>2] = $55;
  $85 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($85|0);
 } else {
  $56 = HEAP32[$4>>2]|0;
  $57 = HEAP32[$pNew>>2]|0;
  $58 = HEAP32[$57>>2]|0;
  $59 = HEAP32[$pNew>>2]|0;
  $60 = ((($59)) + 8|0);
  $61 = (($60) + ($58<<4)|0);
  $62 = ((($61)) + 8|0);
  HEAP32[$62>>2] = $56;
  $63 = HEAP32[$3>>2]|0;
  $64 = HEAP32[$pNew>>2]|0;
  $65 = HEAP32[$64>>2]|0;
  $66 = HEAP32[$pNew>>2]|0;
  $67 = ((($66)) + 8|0);
  $68 = (($67) + ($65<<4)|0);
  $69 = ((($68)) + 4|0);
  HEAP32[$69>>2] = $63;
  $70 = HEAP32[$zName>>2]|0;
  $71 = HEAP32[$pNew>>2]|0;
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$pNew>>2]|0;
  $74 = ((($73)) + 8|0);
  $75 = (($74) + ($72<<4)|0);
  HEAP32[$75>>2] = $70;
  $76 = HEAP32[$pNew>>2]|0;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$pNew>>2]|0;
  $79 = ((($78)) + 8|0);
  $80 = (($79) + ($77<<4)|0);
  $81 = ((($80)) + 12|0);
  HEAP32[$81>>2] = 0;
  $82 = HEAP32[$pNew>>2]|0;
  $83 = HEAP32[$82>>2]|0;
  $84 = (($83) + 1)|0;
  HEAP32[$82>>2] = $84;
  $85 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 return (0)|0;
}
function _yy_find_reduce_action($stateno,$iLookAhead) {
 $stateno = $stateno|0;
 $iLookAhead = $iLookAhead|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $i = sp;
 HEAP32[$0>>2] = $stateno;
 HEAP8[$1>>0] = $iLookAhead;
 $2 = HEAP32[$0>>2]|0;
 $3 = (10806 + ($2<<1)|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4 << 16 >> 16;
 HEAP32[$i>>2] = $5;
 $6 = HEAP8[$1>>0]|0;
 $7 = $6&255;
 $8 = HEAP32[$i>>2]|0;
 $9 = (($8) + ($7))|0;
 HEAP32[$i>>2] = $9;
 $10 = HEAP32[$i>>2]|0;
 $11 = (11456 + ($10<<1)|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 STACKTOP = sp;return ($13|0);
}
function _yy_accept($yypParser) {
 $yypParser = $yypParser|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pParse = sp;
 HEAP32[$0>>2] = $yypParser;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pParse>>2] = $3;
 $4 = HEAP32[$pParse>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 HEAP32[$6>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3NameFromToken($db,$pName) {
 $db = $db|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3DbStrNDup($4,$6,$9,0)|0);
  HEAP32[$zName>>2] = $10;
  $11 = HEAP32[$zName>>2]|0;
  _sqlite3Dequote($11);
  $12 = HEAP32[$zName>>2]|0;
  STACKTOP = sp;return ($12|0);
 } else {
  HEAP32[$zName>>2] = 0;
  $12 = HEAP32[$zName>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 return (0)|0;
}
function _sqlite3DbRealloc($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $p;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = ($10|0)==(0|0);
 $12 = HEAP32[$3>>2]|0;
 if ($11) {
  $13 = $5;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3DbMallocRawNN($12,$15,$18)|0);
  HEAP32[$2>>2] = $19;
  $49 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $20 = HEAP32[$4>>2]|0;
 $21 = (_isLookaside($12,$20)|0);
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = $5;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = ((($29)) + 256|0);
  $31 = ((($30)) + 4|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32&65535;
  $34 = ($28>>>0)<(0);
  $35 = ($25>>>0)<=($33>>>0);
  $36 = ($28|0)==(0);
  $37 = $36 & $35;
  $38 = $34 | $37;
  if ($38) {
   $39 = HEAP32[$4>>2]|0;
   HEAP32[$2>>2] = $39;
   $49 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 }
 $40 = HEAP32[$3>>2]|0;
 $41 = HEAP32[$4>>2]|0;
 $42 = $5;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (_dbReallocFinish($40,$41,$44,$47)|0);
 HEAP32[$2>>2] = $48;
 $49 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3DbMallocRawNN($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $pBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pBuf = sp + 8|0;
 HEAP32[$3>>2] = $db;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 256|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = $4;
   $14 = $13;
   $15 = HEAP32[$14>>2]|0;
   $16 = (($13) + 4)|0;
   $17 = $16;
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$3>>2]|0;
   $20 = ((($19)) + 256|0);
   $21 = ((($20)) + 4|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22&65535;
   $24 = ($18>>>0)>(0);
   $25 = ($15>>>0)>($23>>>0);
   $26 = ($18|0)==(0);
   $27 = $26 & $25;
   $28 = $24 | $27;
   $29 = HEAP32[$3>>2]|0;
   $30 = ((($29)) + 256|0);
   if ($28) {
    $31 = ((($30)) + 16|0);
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + 1)|0;
    HEAP32[$32>>2] = $34;
    break;
   }
   $35 = ((($30)) + 28|0);
   $36 = HEAP32[$35>>2]|0;
   HEAP32[$pBuf>>2] = $36;
   $37 = ($36|0)==(0|0);
   if ($37) {
    $38 = HEAP32[$3>>2]|0;
    $39 = ((($38)) + 256|0);
    $40 = ((($39)) + 16|0);
    $41 = ((($40)) + 8|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = (($42) + 1)|0;
    HEAP32[$41>>2] = $43;
    break;
   }
   $44 = HEAP32[$pBuf>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$3>>2]|0;
   $47 = ((($46)) + 256|0);
   $48 = ((($47)) + 28|0);
   HEAP32[$48>>2] = $45;
   $49 = HEAP32[$3>>2]|0;
   $50 = ((($49)) + 256|0);
   $51 = ((($50)) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + 1)|0;
   HEAP32[$51>>2] = $53;
   $54 = HEAP32[$3>>2]|0;
   $55 = ((($54)) + 256|0);
   $56 = ((($55)) + 16|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + 1)|0;
   HEAP32[$56>>2] = $58;
   $59 = HEAP32[$3>>2]|0;
   $60 = ((($59)) + 256|0);
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = ((($63)) + 256|0);
   $65 = ((($64)) + 12|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = ($62|0)>($66|0);
   if ($67) {
    $68 = HEAP32[$3>>2]|0;
    $69 = ((($68)) + 256|0);
    $70 = ((($69)) + 8|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = HEAP32[$3>>2]|0;
    $73 = ((($72)) + 256|0);
    $74 = ((($73)) + 12|0);
    HEAP32[$74>>2] = $71;
   }
   $75 = HEAP32[$pBuf>>2]|0;
   HEAP32[$2>>2] = $75;
   $88 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($88|0);
  } else {
   $76 = HEAP32[$3>>2]|0;
   $77 = ((($76)) + 69|0);
   $78 = HEAP8[$77>>0]|0;
   $79 = ($78<<24>>24)!=(0);
   if ($79) {
    HEAP32[$2>>2] = 0;
    $88 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($88|0);
   }
  }
 } while(0);
 $80 = HEAP32[$3>>2]|0;
 $81 = $4;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = (_dbMallocRawFinish($80,$83,$86)|0);
 HEAP32[$2>>2] = $87;
 $88 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($88|0);
}
function _dbReallocFinish($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pNew = sp + 8|0;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$pNew>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 69|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(0);
 if (!($13)) {
  $48 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$3>>2]|0;
 $16 = (_isLookaside($14,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3DbMallocRawNN($18,$21,$24)|0);
  HEAP32[$pNew>>2] = $25;
  $26 = HEAP32[$pNew>>2]|0;
  $27 = ($26|0)!=(0|0);
  if (!($27)) {
   $48 = HEAP32[$pNew>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
  $28 = HEAP32[$pNew>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 256|0);
  $32 = ((($31)) + 4|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  _memcpy(($28|0),($29|0),($34|0))|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = HEAP32[$3>>2]|0;
  _sqlite3DbFree($35,$36);
  $48 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($48|0);
 } else {
  $37 = HEAP32[$3>>2]|0;
  $38 = $4;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (($38) + 4)|0;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = (_sqlite3_realloc64($37,$40,$43)|0);
  HEAP32[$pNew>>2] = $44;
  $45 = HEAP32[$pNew>>2]|0;
  $46 = ($45|0)!=(0|0);
  if ($46) {
   $48 = HEAP32[$pNew>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
  $47 = HEAP32[$2>>2]|0;
  _sqlite3OomFault($47);
  $48 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 return (0)|0;
}
function _dbMallocRawFinish($db,$0,$1) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $db;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3Malloc($10,$13)|0);
 HEAP32[$p>>2] = $14;
 $15 = HEAP32[$p>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $18 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $17 = HEAP32[$2>>2]|0;
 _sqlite3OomFault($17);
 $18 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3DbStrNDup($db,$z,$0,$1) {
 $db = $db|0;
 $z = $z|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp;
 $zNew = sp + 8|0;
 HEAP32[$3>>2] = $db;
 HEAP32[$4>>2] = $z;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$2>>2] = 0;
  $41 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = $5;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (_i64Add(($15|0),($18|0),1,0)|0);
 $20 = tempRet0;
 $21 = (_sqlite3DbMallocRawNN($12,$19,$20)|0);
 HEAP32[$zNew>>2] = $21;
 $22 = HEAP32[$zNew>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$zNew>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = $5;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  _memcpy(($24|0),($25|0),($28|0))|0;
  $32 = $5;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$zNew>>2]|0;
  $39 = (($38) + ($34)|0);
  HEAP8[$39>>0] = 0;
 }
 $40 = HEAP32[$zNew>>2]|0;
 HEAP32[$2>>2] = $40;
 $41 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3Dequote($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $quote = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $quote = sp + 12|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $z;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0|0);
 if ($2) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 HEAP8[$quote>>0] = $4;
 $5 = HEAP8[$quote>>0]|0;
 $6 = $5&255;
 $7 = (18403 + ($6)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & 128;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP8[$quote>>0]|0;
 $13 = $12 << 24 >> 24;
 $14 = ($13|0)==(91);
 if ($14) {
  HEAP8[$quote>>0] = 93;
 }
 HEAP32[$i>>2] = 1;
 HEAP32[$j>>2] = 0;
 while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = (($16) + ($15)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = HEAP8[$quote>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = ($19|0)==($21|0);
  $23 = HEAP32[$i>>2]|0;
  if ($22) {
   $24 = (($23) + 1)|0;
   $25 = HEAP32[$0>>2]|0;
   $26 = (($25) + ($24)|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27 << 24 >> 24;
   $29 = HEAP8[$quote>>0]|0;
   $30 = $29 << 24 >> 24;
   $31 = ($28|0)==($30|0);
   if (!($31)) {
    break;
   }
   $32 = HEAP8[$quote>>0]|0;
   $33 = HEAP32[$j>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$j>>2] = $34;
   $35 = HEAP32[$0>>2]|0;
   $36 = (($35) + ($33)|0);
   HEAP8[$36>>0] = $32;
   $37 = HEAP32[$i>>2]|0;
   $38 = (($37) + 1)|0;
   HEAP32[$i>>2] = $38;
  } else {
   $39 = HEAP32[$0>>2]|0;
   $40 = (($39) + ($23)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = HEAP32[$j>>2]|0;
   $43 = (($42) + 1)|0;
   HEAP32[$j>>2] = $43;
   $44 = HEAP32[$0>>2]|0;
   $45 = (($44) + ($42)|0);
   HEAP8[$45>>0] = $41;
  }
  $46 = HEAP32[$i>>2]|0;
  $47 = (($46) + 1)|0;
  HEAP32[$i>>2] = $47;
 }
 $48 = HEAP32[$j>>2]|0;
 $49 = HEAP32[$0>>2]|0;
 $50 = (($49) + ($48)|0);
 HEAP8[$50>>0] = 0;
 STACKTOP = sp;return;
}
function _addArgumentToVtab($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $z = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 452|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 440|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 452|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$z>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 452|0);
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$db>>2] = $17;
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 440|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$z>>2]|0;
 $24 = HEAP32[$n>>2]|0;
 $25 = ($24|0)<(0);
 $26 = $25 << 31 >> 31;
 $27 = (_sqlite3DbStrNDup($22,$23,$24,$26)|0);
 _addModuleArgument($18,$21,$27);
 STACKTOP = sp;return;
}
function _addModuleArgument($db,$pTable,$zArg) {
 $db = $db|0;
 $pTable = $pTable|0;
 $zArg = $zArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $azModuleArg = 0, $i = 0, $nBytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nBytes = sp + 8|0;
 $azModuleArg = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 HEAP32[$2>>2] = $zArg;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (2 + ($5))|0;
 $7 = $6<<2;
 HEAP32[$nBytes>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$nBytes>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_sqlite3DbRealloc($8,$11,$12,$14)|0);
 HEAP32[$azModuleArg>>2] = $15;
 $16 = HEAP32[$azModuleArg>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  _sqlite3DbFree($18,$19);
  STACKTOP = sp;return;
 } else {
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 48|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$21>>2] = $23;
  HEAP32[$i>>2] = $22;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$azModuleArg>>2]|0;
  $27 = (($26) + ($25<<2)|0);
  HEAP32[$27>>2] = $24;
  $28 = HEAP32[$i>>2]|0;
  $29 = (($28) + 1)|0;
  $30 = HEAP32[$azModuleArg>>2]|0;
  $31 = (($30) + ($29<<2)|0);
  HEAP32[$31>>2] = 0;
  $32 = HEAP32[$azModuleArg>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 52|0);
  HEAP32[$34>>2] = $32;
  STACKTOP = sp;return;
 }
}
function _sqlite3AuthCheck($pParse,$code,$zArg1,$zArg2,$zArg3) {
 $pParse = $pParse|0;
 $code = $code|0;
 $zArg1 = $zArg1|0;
 $zArg2 = $zArg2|0;
 $zArg3 = $zArg3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $db = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $code;
 HEAP32[$3>>2] = $zArg1;
 HEAP32[$4>>2] = $zArg2;
 HEAP32[$5>>2] = $zArg3;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 148|0);
 $10 = ((($9)) + 5|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 410|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)!=(0);
  if (!($18)) {
   $19 = HEAP32[$db>>2]|0;
   $20 = ((($19)) + 296|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)==(0|0);
   if ($22) {
    HEAP32[$0>>2] = 0;
    $48 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($48|0);
   }
   $23 = HEAP32[$db>>2]|0;
   $24 = ((($23)) + 296|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$db>>2]|0;
   $27 = ((($26)) + 300|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   $31 = HEAP32[$4>>2]|0;
   $32 = HEAP32[$5>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   $34 = ((($33)) + 448|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (FUNCTION_TABLE_iiiiiii[$25 & 255]($28,$29,$30,$31,$32,$35)|0);
   HEAP32[$rc>>2] = $36;
   $37 = HEAP32[$rc>>2]|0;
   $38 = ($37|0)==(1);
   if ($38) {
    $39 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($39,27514,$vararg_buffer);
    $40 = HEAP32[$1>>2]|0;
    $41 = ((($40)) + 12|0);
    HEAP32[$41>>2] = 23;
   } else {
    $42 = HEAP32[$rc>>2]|0;
    $43 = ($42|0)!=(0);
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)!=(2);
    $or$cond = $43 & $45;
    if ($or$cond) {
     HEAP32[$rc>>2] = 1;
     $46 = HEAP32[$1>>2]|0;
     _sqliteAuthBadReturnCode($46);
    }
   }
   $47 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $47;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _sqliteAuthBadReturnCode($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3ErrorMsg($1,27529,$vararg_buffer);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 12|0);
 HEAP32[$3>>2] = 1;
 STACKTOP = sp;return;
}
function _sqlite3NestedParse($pParse,$zFormat,$varargs) {
 $pParse = $pParse|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $db = 0, $saveBuf = 0, $zErrMsg = 0, $zSql = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $ap = sp + 16|0;
 $zSql = sp + 8|0;
 $zErrMsg = sp + 4|0;
 $db = sp;
 $saveBuf = sp + 40|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$zErrMsg>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 HEAP32[$ap>>2] = $varargs;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3VMPrintf($8,$9,$ap)|0);
 HEAP32[$zSql>>2] = $10;
 $11 = HEAP32[$zSql>>2]|0;
 $12 = ($11|0)==(0|0);
 if ($12) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 18|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = (($15) + 1)<<24>>24;
 HEAP8[$14>>0] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 400|0);
 dest=$saveBuf; src=$18; stop=dest+80|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 400|0);
 dest=$20; stop=dest+80|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$zSql>>2]|0;
 (_sqlite3RunParser($21,$22,$zErrMsg)|0);
 $23 = HEAP32[$db>>2]|0;
 $24 = HEAP32[$zErrMsg>>2]|0;
 _sqlite3DbFree($23,$24);
 $25 = HEAP32[$db>>2]|0;
 $26 = HEAP32[$zSql>>2]|0;
 _sqlite3DbFree($25,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 400|0);
 dest=$28; src=$saveBuf; stop=dest+80|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 18|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = (($31) + -1)<<24>>24;
 HEAP8[$30>>0] = $32;
 STACKTOP = sp;return;
}
function _sqlite3GetVdbe($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$v>>2]|0;
  $9 = $6;
  STACKTOP = sp;return ($9|0);
 } else {
  $7 = HEAP32[$0>>2]|0;
  $8 = (_allocVdbe($7)|0);
  $9 = $8;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _sqlite3ChangeCookie($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($9<<4)|0);
 $14 = ((($13)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 1)|0;
 (_sqlite3VdbeAddOp3($7,102,$8,1,$17)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOp0($p,$op) {
 $p = $p|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VdbeAddOp3($2,$3,0,0,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeAddParseSchemaOp($p,$iDb,$zWhere) {
 $p = $p|0;
 $iDb = $iDb|0;
 $zWhere = $zWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zWhere;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp4($3,136,$4,0,0,$5,-1)|0);
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ((($8)) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$j>>2]|0;
  _sqlite3VdbeUsesBtree($12,$13);
  $14 = HEAP32[$j>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$j>>2] = $15;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeLoadString($p,$iDest,$zStr) {
 $p = $p|0;
 $iDest = $iDest|0;
 $zStr = $zStr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDest;
 HEAP32[$2>>2] = $zStr;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3VdbeAddOp4($3,97,0,$4,0,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3VdbeAddOp2($p,$op,$p1,$p2) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_sqlite3VdbeAddOp3($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeAddOp3($p,$op,$p1,$p2,$p3) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $i = sp + 4|0;
 $pOp = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $p1;
 HEAP32[$4>>2] = $p2;
 HEAP32[$5>>2] = $p3;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 136|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$i>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 48|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$i>>2]|0;
 $15 = ($13|0)<=($14|0);
 $16 = HEAP32[$1>>2]|0;
 if ($15) {
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$3>>2]|0;
  $19 = HEAP32[$4>>2]|0;
  $20 = HEAP32[$5>>2]|0;
  $21 = (_growOp3($16,$17,$18,$19,$20)|0);
  HEAP32[$0>>2] = $21;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 } else {
  $22 = ((($16)) + 136|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$22>>2] = $24;
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 88|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($25*20)|0)|0);
  HEAP32[$pOp>>2] = $29;
  $30 = HEAP32[$2>>2]|0;
  $31 = $30&255;
  $32 = HEAP32[$pOp>>2]|0;
  HEAP8[$32>>0] = $31;
  $33 = HEAP32[$pOp>>2]|0;
  $34 = ((($33)) + 3|0);
  HEAP8[$34>>0] = 0;
  $35 = HEAP32[$3>>2]|0;
  $36 = HEAP32[$pOp>>2]|0;
  $37 = ((($36)) + 4|0);
  HEAP32[$37>>2] = $35;
  $38 = HEAP32[$4>>2]|0;
  $39 = HEAP32[$pOp>>2]|0;
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $38;
  $41 = HEAP32[$5>>2]|0;
  $42 = HEAP32[$pOp>>2]|0;
  $43 = ((($42)) + 12|0);
  HEAP32[$43>>2] = $41;
  $44 = HEAP32[$pOp>>2]|0;
  $45 = ((($44)) + 16|0);
  HEAP32[$45>>2] = 0;
  $46 = HEAP32[$pOp>>2]|0;
  $47 = ((($46)) + 1|0);
  HEAP8[$47>>0] = 0;
  $48 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $48;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 return (0)|0;
}
function _growOp3($p,$op,$p1,$p2,$p3) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $5 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $p1;
 HEAP32[$4>>2] = $p2;
 HEAP32[$5>>2] = $p3;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_growOpArray($6,1)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP32[$4>>2]|0;
  $13 = HEAP32[$5>>2]|0;
  $14 = (_sqlite3VdbeAddOp3($9,$10,$11,$12,$13)|0);
  HEAP32[$0>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _growOpArray($v,$nOp) {
 $v = $v|0;
 $nOp = $nOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $nNew = 0, $p = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pNew = sp + 8|0;
 $p = sp + 4|0;
 $nNew = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $nOp;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 48|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 48|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11<<1;
  $13 = $12;
 } else {
  $13 = 51;
 }
 HEAP32[$nNew>>2] = $13;
 $14 = HEAP32[$p>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 88|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$nNew>>2]|0;
 $20 = ($19*20)|0;
 $21 = (_sqlite3DbRealloc($15,$18,$20,0)|0);
 HEAP32[$pNew>>2] = $21;
 $22 = HEAP32[$pNew>>2]|0;
 $23 = ($22|0)!=(0|0);
 if (!($23)) {
  $39 = HEAP32[$pNew>>2]|0;
  $40 = ($39|0)!=(0|0);
  $41 = $40 ? 0 : 7;
  STACKTOP = sp;return ($41|0);
 }
 $24 = HEAP32[$p>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$pNew>>2]|0;
 $27 = (_sqlite3DbMallocSize($25,$26)|0);
 $28 = HEAP32[$p>>2]|0;
 $29 = ((($28)) + 52|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 52|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32>>>0) / 20)&-1;
 $34 = HEAP32[$p>>2]|0;
 $35 = ((($34)) + 48|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pNew>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 88|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$pNew>>2]|0;
 $40 = ($39|0)!=(0|0);
 $41 = $40 ? 0 : 7;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3VdbeAddOp4($p,$op,$p1,$p2,$p3,$zP4,$p4type) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $zP4 = $zP4|0;
 $p4type = $p4type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $zP4;
 HEAP32[$6>>2] = $p4type;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_sqlite3VdbeAddOp3($7,$8,$9,$10,$11)|0);
 HEAP32[$addr>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$addr>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$6>>2]|0;
 _sqlite3VdbeChangeP4($13,$14,$15,$16);
 $17 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _allocVdbe($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeCreate($1)|0);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 HEAP32[$4>>2] = $2;
 HEAP32[$v>>2] = $2;
 $5 = HEAP32[$v>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp2($7,71,0,1)|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 124|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0|0);
 if (!($11)) {
  $21 = HEAP32[$v>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 64|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 & 8;
 $18 = ($17|0)==(0);
 if (!($18)) {
  $21 = HEAP32[$v>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 23|0);
 HEAP8[$20>>0] = 1;
 $21 = HEAP32[$v>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3VMPrintf($db,$zFormat,$ap) {
 $db = $db|0;
 $zFormat = $zFormat|0;
 $ap = $ap|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $acc = 0, $z = 0, $zBase = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $z = sp + 28|0;
 $zBase = sp + 44|0;
 $acc = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$2>>2] = $ap;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 96|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3StrAccumInit($acc,$3,$zBase,70,$6);
 $7 = ((($acc)) + 25|0);
 HEAP8[$7>>0] = 1;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 _sqlite3VXPrintf($acc,$8,$9);
 $10 = (_sqlite3StrAccumFinish($acc)|0);
 HEAP32[$z>>2] = $10;
 $11 = ((($acc)) + 24|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(1);
 if (!($14)) {
  $16 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $15 = HEAP32[$0>>2]|0;
 _sqlite3OomFault($15);
 $16 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3LocateTableItem($pParse,$flags,$p) {
 $pParse = $pParse|0;
 $flags = $flags|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zDb = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $flags;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (_sqlite3SchemaToIndex($7,$9)|0);
  HEAP32[$iDb>>2] = $10;
  $11 = HEAP32[$iDb>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ((($13)) + 16|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($11<<4)|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$zDb>>2] = $17;
 } else {
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$zDb>>2] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$zDb>>2]|0;
 $27 = (_sqlite3LocateTable($21,$22,$25,$26)|0);
 STACKTOP = sp;return ($27|0);
}
function _isSystemTable($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3Strlen30($3)|0);
 $5 = ($4|0)>(6);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (_sqlite3_strnicmp($6,27844,7)|0);
  $8 = (0)==($7|0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer>>2] = $10;
   _sqlite3ErrorMsg($9,27852,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $11 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($11|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3BeginWriteOperation($pParse,$setStatement,$iDb) {
 $pParse = $pParse|0;
 $setStatement = $setStatement|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $setStatement;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 124|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = ((($7)) + 124|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $9;
 } else {
  $10 = $7;
 }
 HEAP32[$pToplevel>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 _sqlite3CodeVerifySchema($11,$12);
 $13 = HEAP32[$2>>2]|0;
 $14 = 1 << $13;
 $15 = HEAP32[$pToplevel>>2]|0;
 $16 = ((($15)) + 92|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17 | $14;
 HEAP32[$16>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$pToplevel>>2]|0;
 $21 = ((($20)) + 20|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $23 | $19;
 $25 = $24&255;
 HEAP8[$21>>0] = $25;
 STACKTOP = sp;return;
}
function _sqlite3CodeVerifySchema($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 124|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 124|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $8;
 } else {
  $9 = $6;
 }
 HEAP32[$pToplevel>>2] = $9;
 $10 = HEAP32[$pToplevel>>2]|0;
 $11 = ((($10)) + 96|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = 1 << $13;
 $15 = $12 & $14;
 $16 = ($15|0)!=(0);
 $17 = $16&1;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$1>>2]|0;
 $20 = 1 << $19;
 $21 = HEAP32[$pToplevel>>2]|0;
 $22 = ((($21)) + 96|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = $23 | $20;
 HEAP32[$22>>2] = $24;
 $25 = HEAP32[$1>>2]|0;
 $26 = ($25|0)==(1);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$pToplevel>>2]|0;
 (_sqlite3OpenTempDatabase($27)|0);
 STACKTOP = sp;return;
}
function _sqlite3OpenTempDatabase($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pBt = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $db = sp + 12|0;
 $rc = sp + 8|0;
 $pBt = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 409|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if (!($14)) {
   $15 = HEAP32[$db>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$db>>2]|0;
   $18 = (_sqlite3BtreeOpen($16,0,$17,$pBt,0,542)|0);
   HEAP32[$rc>>2] = $18;
   $19 = HEAP32[$rc>>2]|0;
   $20 = ($19|0)!=(0);
   if ($20) {
    $21 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($21,27774,$vararg_buffer);
    $22 = HEAP32[$rc>>2]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 12|0);
    HEAP32[$24>>2] = $22;
    HEAP32[$0>>2] = 1;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
   $25 = HEAP32[$pBt>>2]|0;
   $26 = HEAP32[$db>>2]|0;
   $27 = ((($26)) + 16|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ((($28)) + 16|0);
   $30 = ((($29)) + 4|0);
   HEAP32[$30>>2] = $25;
   $31 = HEAP32[$pBt>>2]|0;
   $32 = HEAP32[$db>>2]|0;
   $33 = ((($32)) + 80|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (_sqlite3BtreeSetPageSize($31,$34,-1,0)|0);
   $36 = (7)==($35|0);
   if ($36) {
    $37 = HEAP32[$db>>2]|0;
    _sqlite3OomFault($37);
    HEAP32[$0>>2] = 1;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3ValueFromExpr($db,$pExpr,$enc,$affinity,$ppVal) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $enc = $enc|0;
 $affinity = $affinity|0;
 $ppVal = $ppVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 13|0;
 $3 = sp + 12|0;
 $4 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $enc;
 HEAP8[$3>>0] = $affinity;
 HEAP32[$4>>2] = $ppVal;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $13 = 0;
  STACKTOP = sp;return ($13|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$2>>0]|0;
 $10 = HEAP8[$3>>0]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_valueFromExpr($7,$8,$9,$10,$11,0)|0);
 $13 = $12;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3GetTempReg($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 19|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 44|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + 1)|0;
  HEAP32[$8>>2] = $10;
  HEAP32[$0>>2] = $10;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $11 = ((($7)) + 19|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = (($12) + -1)<<24>>24;
  HEAP8[$11>>0] = $13;
  $14 = $13&255;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 352|0);
  $17 = (($16) + ($14<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3VdbeCurrentAddr($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 136|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3ReleaseTempReg($pParse,$iReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 19|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)<(8);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 152|0);
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 25|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = ($11|0)<($15|0);
  if (!($16)) {
   label = 8;
   break;
  }
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ($19|0)==($20|0);
  if ($21) {
   label = 6;
   break;
  }
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
  $26 = HEAP32[$p>>2]|0;
  $27 = ((($26)) + 20|0);
  HEAP32[$p>>2] = $27;
 }
 if ((label|0) == 6) {
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 6|0);
  HEAP8[$23>>0] = 1;
  STACKTOP = sp;return;
 }
 else if ((label|0) == 8) {
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 19|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = (($31) + 1)<<24>>24;
  HEAP8[$30>>0] = $32;
  $33 = $31&255;
  $34 = HEAP32[$0>>2]|0;
  $35 = ((($34)) + 352|0);
  $36 = (($35) + ($33<<2)|0);
  HEAP32[$36>>2] = $28;
  STACKTOP = sp;return;
 }
}
function _reloadTableSchema($pParse,$pTab,$zName) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $iTrigDb = 0, $pTrig = 0, $v = 0, $vararg_buffer = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $v = sp + 20|0;
 $zWhere = sp + 16|0;
 $iDb = sp + 12|0;
 $pTrig = sp + 8|0;
 $iTrigDb = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetVdbe($3)|0);
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$v>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 64|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3SchemaToIndex($8,$11)|0);
 HEAP32[$iDb>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = (_sqlite3TriggerList($13,$14)|0);
 HEAP32[$pTrig>>2] = $15;
 while(1) {
  $16 = HEAP32[$pTrig>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pTrig>>2]|0;
  $21 = ((($20)) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (_sqlite3SchemaToIndex($19,$22)|0);
  HEAP32[$iTrigDb>>2] = $23;
  $24 = HEAP32[$v>>2]|0;
  $25 = HEAP32[$iTrigDb>>2]|0;
  $26 = HEAP32[$pTrig>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3VdbeAddOp4($24,140,$25,0,0,$27,0)|0);
  $28 = HEAP32[$pTrig>>2]|0;
  $29 = ((($28)) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pTrig>>2] = $30;
 }
 $31 = HEAP32[$v>>2]|0;
 $32 = HEAP32[$iDb>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 $34 = HEAP32[$33>>2]|0;
 (_sqlite3VdbeAddOp4($31,138,$32,0,0,$34,0)|0);
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $37;
 $38 = (_sqlite3MPrintf($36,28206,$vararg_buffer)|0);
 HEAP32[$zWhere>>2] = $38;
 $39 = HEAP32[$zWhere>>2]|0;
 $40 = ($39|0)!=(0|0);
 if (!($40)) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$v>>2]|0;
 $42 = HEAP32[$iDb>>2]|0;
 $43 = HEAP32[$zWhere>>2]|0;
 _sqlite3VdbeAddParseSchemaOp($41,$42,$43);
 $44 = HEAP32[$0>>2]|0;
 $45 = HEAP32[$1>>2]|0;
 $46 = (_whereTempTriggers($44,$45)|0);
 HEAP32[$zWhere>>2] = $46;
 $47 = ($46|0)!=(0|0);
 if (!($47)) {
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$v>>2]|0;
 $49 = HEAP32[$zWhere>>2]|0;
 _sqlite3VdbeAddParseSchemaOp($48,1,$49);
 STACKTOP = sp;return;
}
function _sqlite3TriggerList($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pList = 0, $pTmpSchema = 0, $pTrig = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pTmpSchema = sp + 12|0;
 $pList = sp + 8|0;
 $p = sp + 4|0;
 $pTrig = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTmpSchema>>2] = $9;
 HEAP32[$pList>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 150|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $14 = HEAP32[$pTmpSchema>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($14|0)!=($17|0);
 L5: do {
  if ($18) {
   $19 = HEAP32[$pTmpSchema>>2]|0;
   $20 = ((($19)) + 40|0);
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$p>>2] = $22;
   while(1) {
    $23 = HEAP32[$p>>2]|0;
    $24 = ($23|0)!=(0|0);
    if (!($24)) {
     break L5;
    }
    $25 = HEAP32[$p>>2]|0;
    $26 = ((($25)) + 8|0);
    $27 = HEAP32[$26>>2]|0;
    HEAP32[$pTrig>>2] = $27;
    $28 = HEAP32[$pTrig>>2]|0;
    $29 = ((($28)) + 24|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    $32 = ((($31)) + 64|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($30|0)==($33|0);
    if ($34) {
     $35 = HEAP32[$pTrig>>2]|0;
     $36 = ((($35)) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = HEAP32[$2>>2]|0;
     $39 = HEAP32[$38>>2]|0;
     $40 = (_sqlite3StrICmp($37,$39)|0);
     $41 = (0)==($40|0);
     if ($41) {
      $42 = HEAP32[$pList>>2]|0;
      $43 = ($42|0)!=(0|0);
      if ($43) {
       $44 = HEAP32[$pList>>2]|0;
       $50 = $44;
      } else {
       $45 = HEAP32[$2>>2]|0;
       $46 = ((($45)) + 60|0);
       $47 = HEAP32[$46>>2]|0;
       $50 = $47;
      }
      $48 = HEAP32[$pTrig>>2]|0;
      $49 = ((($48)) + 32|0);
      HEAP32[$49>>2] = $50;
      $51 = HEAP32[$pTrig>>2]|0;
      HEAP32[$pList>>2] = $51;
     }
    }
    $52 = HEAP32[$p>>2]|0;
    $53 = HEAP32[$52>>2]|0;
    HEAP32[$p>>2] = $53;
   }
  }
 } while(0);
 $54 = HEAP32[$pList>>2]|0;
 $55 = ($54|0)!=(0|0);
 if ($55) {
  $56 = HEAP32[$pList>>2]|0;
  $60 = $56;
 } else {
  $57 = HEAP32[$2>>2]|0;
  $58 = ((($57)) + 60|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $59;
 }
 HEAP32[$0>>2] = $60;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _whereTempTriggers($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pTempSchema = 0, $pTrig = 0, $vararg_buffer = 0, $zNew = 0, $zWhere = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pTrig = sp + 20|0;
 $zWhere = sp + 16|0;
 $pTempSchema = sp + 12|0;
 $db = sp + 8|0;
 $zNew = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$zWhere>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTempSchema>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 64|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pTempSchema>>2]|0;
 $13 = ($11|0)!=($12|0);
 L1: do {
  if ($13) {
   $14 = HEAP32[$0>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   HEAP32[$db>>2] = $15;
   $16 = HEAP32[$0>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3TriggerList($16,$17)|0);
   HEAP32[$pTrig>>2] = $18;
   while(1) {
    $19 = HEAP32[$pTrig>>2]|0;
    $20 = ($19|0)!=(0|0);
    if (!($20)) {
     break L1;
    }
    $21 = HEAP32[$pTrig>>2]|0;
    $22 = ((($21)) + 20|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$pTempSchema>>2]|0;
    $25 = ($23|0)==($24|0);
    if ($25) {
     $26 = HEAP32[$db>>2]|0;
     $27 = HEAP32[$zWhere>>2]|0;
     $28 = HEAP32[$pTrig>>2]|0;
     $29 = HEAP32[$28>>2]|0;
     $30 = (_whereOrName($26,$27,$29)|0);
     HEAP32[$zWhere>>2] = $30;
    }
    $31 = HEAP32[$pTrig>>2]|0;
    $32 = ((($31)) + 32|0);
    $33 = HEAP32[$32>>2]|0;
    HEAP32[$pTrig>>2] = $33;
   }
  }
 } while(0);
 $34 = HEAP32[$zWhere>>2]|0;
 $35 = ($34|0)!=(0|0);
 if (!($35)) {
  $44 = HEAP32[$zWhere>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$zWhere>>2]|0;
 HEAP32[$vararg_buffer>>2] = $38;
 $39 = (_sqlite3MPrintf($37,28218,$vararg_buffer)|0);
 HEAP32[$zNew>>2] = $39;
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$zWhere>>2]|0;
 _sqlite3DbFree($41,$42);
 $43 = HEAP32[$zNew>>2]|0;
 HEAP32[$zWhere>>2] = $43;
 $44 = HEAP32[$zWhere>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _whereOrName($db,$zWhere,$zConstant) {
 $db = $db|0;
 $zWhere = $zWhere|0;
 $zConstant = $zConstant|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $zNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $zNew = sp + 16|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zWhere;
 HEAP32[$2>>2] = $zConstant;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $8;
  $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $9;
  $10 = (_sqlite3MPrintf($5,28250,$vararg_buffer1)|0);
  HEAP32[$zNew>>2] = $10;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($11,$12);
  $13 = HEAP32[$zNew>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  $6 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer>>2] = $6;
  $7 = (_sqlite3MPrintf($5,28242,$vararg_buffer)|0);
  HEAP32[$zNew>>2] = $7;
  $13 = HEAP32[$zNew>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _valueFromExpr($db,$pExpr,$enc,$affinity,$ppVal,$pCtx) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $enc = $enc|0;
 $affinity = $affinity|0;
 $ppVal = $ppVal|0;
 $pCtx = $pCtx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0.0, $155 = 0.0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aff = 0, $nVal = 0, $negInt = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $pVal = 0, $rc = 0, $vararg_buffer = 0, $vararg_ptr8 = 0;
 var $zNeg = 0, $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 58|0;
 $4 = sp + 57|0;
 $5 = sp + 40|0;
 $6 = sp + 36|0;
 $op = sp + 32|0;
 $zVal = sp + 28|0;
 $pVal = sp + 24|0;
 $negInt = sp + 20|0;
 $zNeg = sp + 16|0;
 $rc = sp + 12|0;
 $aff = sp + 56|0;
 $nVal = sp + 8|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP8[$3>>0] = $enc;
 HEAP8[$4>>0] = $affinity;
 HEAP32[$5>>2] = $ppVal;
 HEAP32[$6>>2] = $pCtx;
 HEAP32[$zVal>>2] = 0;
 HEAP32[$pVal>>2] = 0;
 HEAP32[$negInt>>2] = 1;
 HEAP32[$zNeg>>2] = 18402;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  HEAP32[$op>>2] = $9;
  $10 = ($9|0)==(156);
  $11 = HEAP32[$op>>2]|0;
  $12 = ($11|0)==(161);
  $13 = $10 ? 1 : $12;
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$2>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$2>>2] = $16;
 }
 $17 = HEAP32[$op>>2]|0;
 $18 = ($17|0)==(157);
 if ($18) {
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 38|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  HEAP32[$op>>2] = $22;
 }
 $23 = HEAP32[$op>>2]|0;
 $24 = ($23|0)==(66);
 if ($24) {
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (_sqlite3AffinityType($27,0)|0);
  HEAP8[$aff>>0] = $28;
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP8[$3>>0]|0;
  $34 = HEAP8[$aff>>0]|0;
  $35 = HEAP32[$5>>2]|0;
  $36 = HEAP32[$6>>2]|0;
  $37 = (_valueFromExpr($29,$32,$33,$34,$35,$36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$5>>2]|0;
  $39 = HEAP32[$38>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$5>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP8[$aff>>0]|0;
   _sqlite3VdbeMemCast($42,$43,1);
   $44 = HEAP32[$5>>2]|0;
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP8[$4>>0]|0;
   _sqlite3ValueApplyAffinity($45,$46,1);
  }
  $47 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $47;
  $227 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 $48 = HEAP32[$op>>2]|0;
 $49 = ($48|0)==(155);
 do {
  if ($49) {
   $50 = HEAP32[$2>>2]|0;
   $51 = ((($50)) + 12|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP8[$52>>0]|0;
   $54 = $53&255;
   $55 = ($54|0)==(134);
   if (!($55)) {
    $56 = HEAP32[$2>>2]|0;
    $57 = ((($56)) + 12|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = ($60|0)==(132);
    if (!($61)) {
     break;
    }
   }
   $62 = HEAP32[$2>>2]|0;
   $63 = ((($62)) + 12|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$2>>2] = $64;
   $65 = HEAP32[$2>>2]|0;
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   HEAP32[$op>>2] = $67;
   HEAP32[$negInt>>2] = -1;
   HEAP32[$zNeg>>2] = 28264;
  }
 } while(0);
 $68 = HEAP32[$op>>2]|0;
 $69 = ($68|0)==(97);
 $70 = HEAP32[$op>>2]|0;
 $71 = ($70|0)==(132);
 $or$cond = $69 | $71;
 $72 = HEAP32[$op>>2]|0;
 $73 = ($72|0)==(134);
 $or$cond3 = $or$cond | $73;
 do {
  if ($or$cond3) {
   $74 = HEAP32[$1>>2]|0;
   $75 = HEAP32[$6>>2]|0;
   $76 = (_valueNew($74,$75)|0);
   HEAP32[$pVal>>2] = $76;
   $77 = HEAP32[$pVal>>2]|0;
   $78 = ($77|0)==(0|0);
   if ($78) {
    label = 42;
   } else {
    $79 = HEAP32[$2>>2]|0;
    $80 = ((($79)) + 4|0);
    $81 = HEAP32[$80>>2]|0;
    $82 = $81 & 1024;
    $83 = ($82|0)!=(0);
    if ($83) {
     $84 = HEAP32[$pVal>>2]|0;
     $85 = HEAP32[$2>>2]|0;
     $86 = ((($85)) + 8|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = ($87|0)<(0);
     $89 = $88 << 31 >> 31;
     $90 = HEAP32[$negInt>>2]|0;
     $91 = ($90|0)<(0);
     $92 = $91 << 31 >> 31;
     $93 = (___muldi3(($87|0),($89|0),($90|0),($92|0))|0);
     $94 = tempRet0;
     _sqlite3VdbeMemSetInt64($84,$93,$94);
    } else {
     $95 = HEAP32[$1>>2]|0;
     $96 = HEAP32[$zNeg>>2]|0;
     $97 = HEAP32[$2>>2]|0;
     $98 = ((($97)) + 8|0);
     $99 = HEAP32[$98>>2]|0;
     HEAP32[$vararg_buffer>>2] = $96;
     $vararg_ptr8 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr8>>2] = $99;
     $100 = (_sqlite3MPrintf($95,24705,$vararg_buffer)|0);
     HEAP32[$zVal>>2] = $100;
     $101 = HEAP32[$zVal>>2]|0;
     $102 = ($101|0)==(0|0);
     if ($102) {
      label = 42;
      break;
     }
     $103 = HEAP32[$pVal>>2]|0;
     $104 = HEAP32[$zVal>>2]|0;
     _sqlite3ValueSetStr($103,-1,$104,1,170);
    }
    $105 = HEAP32[$op>>2]|0;
    $106 = ($105|0)==(134);
    $107 = HEAP32[$op>>2]|0;
    $108 = ($107|0)==(132);
    $or$cond5 = $106 | $108;
    if ($or$cond5) {
     $109 = HEAP8[$4>>0]|0;
     $110 = $109&255;
     $111 = ($110|0)==(65);
     if ($111) {
      $112 = HEAP32[$pVal>>2]|0;
      _sqlite3ValueApplyAffinity($112,67,1);
     } else {
      label = 23;
     }
    } else {
     label = 23;
    }
    if ((label|0) == 23) {
     $113 = HEAP32[$pVal>>2]|0;
     $114 = HEAP8[$4>>0]|0;
     _sqlite3ValueApplyAffinity($113,$114,1);
    }
    $115 = HEAP32[$pVal>>2]|0;
    $116 = ((($115)) + 8|0);
    $117 = HEAP16[$116>>1]|0;
    $118 = $117&65535;
    $119 = $118 & 12;
    $120 = ($119|0)!=(0);
    if ($120) {
     $121 = HEAP32[$pVal>>2]|0;
     $122 = ((($121)) + 8|0);
     $123 = HEAP16[$122>>1]|0;
     $124 = $123&65535;
     $125 = $124 & -3;
     $126 = $125&65535;
     HEAP16[$122>>1] = $126;
    }
    $127 = HEAP8[$3>>0]|0;
    $128 = $127&255;
    $129 = ($128|0)!=(1);
    if ($129) {
     $130 = HEAP32[$pVal>>2]|0;
     $131 = HEAP8[$3>>0]|0;
     $132 = $131&255;
     $133 = (_sqlite3VdbeChangeEncoding($130,$132)|0);
     HEAP32[$rc>>2] = $133;
     label = 41;
    } else {
     label = 41;
    }
   }
  } else {
   $134 = HEAP32[$op>>2]|0;
   $135 = ($134|0)==(155);
   if (!($135)) {
    $192 = HEAP32[$op>>2]|0;
    $193 = ($192|0)==(101);
    if ($193) {
     $194 = HEAP32[$1>>2]|0;
     $195 = HEAP32[$6>>2]|0;
     $196 = (_valueNew($194,$195)|0);
     HEAP32[$pVal>>2] = $196;
     $197 = HEAP32[$pVal>>2]|0;
     $198 = ($197|0)==(0|0);
     if ($198) {
      label = 42;
      break;
     } else {
      label = 41;
      break;
     }
    }
    $199 = HEAP32[$op>>2]|0;
    $200 = ($199|0)==(133);
    if (!($200)) {
     label = 41;
     break;
    }
    $201 = HEAP32[$1>>2]|0;
    $202 = HEAP32[$6>>2]|0;
    $203 = (_valueNew($201,$202)|0);
    HEAP32[$pVal>>2] = $203;
    $204 = HEAP32[$pVal>>2]|0;
    $205 = ($204|0)!=(0|0);
    if (!($205)) {
     label = 42;
     break;
    }
    $206 = HEAP32[$2>>2]|0;
    $207 = ((($206)) + 8|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = ((($208)) + 2|0);
    HEAP32[$zVal>>2] = $209;
    $210 = HEAP32[$zVal>>2]|0;
    $211 = (_sqlite3Strlen30($210)|0);
    $212 = (($211) - 1)|0;
    HEAP32[$nVal>>2] = $212;
    $213 = HEAP32[$pVal>>2]|0;
    $214 = HEAP32[$1>>2]|0;
    $215 = HEAP32[$zVal>>2]|0;
    $216 = HEAP32[$nVal>>2]|0;
    $217 = (_sqlite3HexToBlob($214,$215,$216)|0);
    $218 = HEAP32[$nVal>>2]|0;
    $219 = (($218|0) / 2)&-1;
    (_sqlite3VdbeMemSetStr($213,$217,$219,0,170)|0);
    label = 41;
    break;
   }
   $136 = HEAP32[$1>>2]|0;
   $137 = HEAP32[$2>>2]|0;
   $138 = ((($137)) + 12|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = HEAP8[$3>>0]|0;
   $141 = HEAP8[$4>>0]|0;
   $142 = (_sqlite3ValueFromExpr($136,$139,$140,$141,$pVal)|0);
   $143 = (0)==($142|0);
   $144 = HEAP32[$pVal>>2]|0;
   $145 = ($144|0)!=(0|0);
   $or$cond7 = $143 & $145;
   if ($or$cond7) {
    $146 = HEAP32[$pVal>>2]|0;
    (_sqlite3VdbeMemNumerify($146)|0);
    $147 = HEAP32[$pVal>>2]|0;
    $148 = ((($147)) + 8|0);
    $149 = HEAP16[$148>>1]|0;
    $150 = $149&65535;
    $151 = $150 & 8;
    $152 = ($151|0)!=(0);
    $153 = HEAP32[$pVal>>2]|0;
    do {
     if ($152) {
      $154 = +HEAPF64[$153>>3];
      $155 = -$154;
      $156 = HEAP32[$pVal>>2]|0;
      HEAPF64[$156>>3] = $155;
     } else {
      $157 = $153;
      $158 = $157;
      $159 = HEAP32[$158>>2]|0;
      $160 = (($157) + 4)|0;
      $161 = $160;
      $162 = HEAP32[$161>>2]|0;
      $163 = ($159|0)==(0);
      $164 = ($162|0)==(-2147483648);
      $165 = $163 & $164;
      $166 = HEAP32[$pVal>>2]|0;
      if ($165) {
       HEAPF64[$166>>3] = 9.2233720368547758E+18;
       $167 = HEAP32[$pVal>>2]|0;
       $168 = ((($167)) + 8|0);
       $169 = HEAP16[$168>>1]|0;
       $170 = $169&65535;
       $171 = $170 & -49664;
       $172 = $171 | 8;
       $173 = $172&65535;
       $174 = HEAP32[$pVal>>2]|0;
       $175 = ((($174)) + 8|0);
       HEAP16[$175>>1] = $173;
       break;
      } else {
       $176 = $166;
       $177 = $176;
       $178 = HEAP32[$177>>2]|0;
       $179 = (($176) + 4)|0;
       $180 = $179;
       $181 = HEAP32[$180>>2]|0;
       $182 = (_i64Subtract(0,0,($178|0),($181|0))|0);
       $183 = tempRet0;
       $184 = HEAP32[$pVal>>2]|0;
       $185 = $184;
       $186 = $185;
       HEAP32[$186>>2] = $182;
       $187 = (($185) + 4)|0;
       $188 = $187;
       HEAP32[$188>>2] = $183;
       break;
      }
     }
    } while(0);
    $189 = HEAP32[$pVal>>2]|0;
    $190 = HEAP8[$4>>0]|0;
    $191 = HEAP8[$3>>0]|0;
    _sqlite3ValueApplyAffinity($189,$190,$191);
    label = 41;
   } else {
    label = 41;
   }
  }
 } while(0);
 if ((label|0) == 41) {
  $220 = HEAP32[$pVal>>2]|0;
  $221 = HEAP32[$5>>2]|0;
  HEAP32[$221>>2] = $220;
  $222 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $222;
  $227 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 else if ((label|0) == 42) {
  $223 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($223);
  $224 = HEAP32[$1>>2]|0;
  $225 = HEAP32[$zVal>>2]|0;
  _sqlite3DbFree($224,$225);
  $226 = HEAP32[$pVal>>2]|0;
  _sqlite3ValueFree($226);
  HEAP32[$0>>2] = 7;
  $227 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($227|0);
 }
 return (0)|0;
}
function _sqlite3AffinityType($zIn,$pszEst) {
 $zIn = $zIn|0;
 $pszEst = $pszEst|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $aff = 0, $h = 0, $v = 0, $zChar = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $h = sp + 8|0;
 $aff = sp + 20|0;
 $zChar = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $pszEst;
 HEAP32[$h>>2] = 0;
 HEAP8[$aff>>0] = 67;
 HEAP32[$zChar>>2] = 0;
 L1: while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP8[$2>>0]|0;
  $4 = ($3<<24>>24)!=(0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$h>>2]|0;
  $6 = $5 << 8;
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = $9 & 255;
  $11 = (17613 + ($10)|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = (($6) + ($13))|0;
  HEAP32[$h>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 1|0);
  HEAP32[$0>>2] = $16;
  $17 = HEAP32[$h>>2]|0;
  $18 = ($17|0)==(1667785074);
  if ($18) {
   HEAP8[$aff>>0] = 66;
   $19 = HEAP32[$0>>2]|0;
   HEAP32[$zChar>>2] = $19;
   continue;
  }
  $20 = HEAP32[$h>>2]|0;
  $21 = ($20|0)==(1668050786);
  if ($21) {
   HEAP8[$aff>>0] = 66;
   continue;
  }
  $22 = HEAP32[$h>>2]|0;
  $23 = ($22|0)==(1952807028);
  if ($23) {
   HEAP8[$aff>>0] = 66;
   continue;
  }
  $24 = HEAP32[$h>>2]|0;
  $25 = ($24|0)==(1651273570);
  do {
   if ($25) {
    $26 = HEAP8[$aff>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)==(67);
    if (!($28)) {
     $29 = HEAP8[$aff>>0]|0;
     $30 = $29 << 24 >> 24;
     $31 = ($30|0)==(69);
     if (!($31)) {
      break;
     }
    }
    HEAP8[$aff>>0] = 65;
    $32 = HEAP32[$0>>2]|0;
    $33 = HEAP8[$32>>0]|0;
    $34 = $33 << 24 >> 24;
    $35 = ($34|0)==(40);
    if (!($35)) {
     continue L1;
    }
    $36 = HEAP32[$0>>2]|0;
    HEAP32[$zChar>>2] = $36;
    continue L1;
   }
  } while(0);
  $37 = HEAP32[$h>>2]|0;
  $38 = ($37|0)==(1919246700);
  if ($38) {
   $39 = HEAP8[$aff>>0]|0;
   $40 = $39 << 24 >> 24;
   $41 = ($40|0)==(67);
   if ($41) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $42 = HEAP32[$h>>2]|0;
  $43 = ($42|0)==(1718382433);
  if ($43) {
   $44 = HEAP8[$aff>>0]|0;
   $45 = $44 << 24 >> 24;
   $46 = ($45|0)==(67);
   if ($46) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $47 = HEAP32[$h>>2]|0;
  $48 = ($47|0)==(1685026146);
  if ($48) {
   $49 = HEAP8[$aff>>0]|0;
   $50 = $49 << 24 >> 24;
   $51 = ($50|0)==(67);
   if ($51) {
    HEAP8[$aff>>0] = 69;
    continue;
   }
  }
  $52 = HEAP32[$h>>2]|0;
  $53 = $52 & 16777215;
  $54 = ($53|0)==(6909556);
  if ($54) {
   label = 24;
   break;
  }
 }
 if ((label|0) == 24) {
  HEAP8[$aff>>0] = 68;
 }
 $55 = HEAP32[$1>>2]|0;
 $56 = ($55|0)!=(0|0);
 if (!($56)) {
  $86 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($86|0);
 }
 $57 = HEAP32[$1>>2]|0;
 HEAP8[$57>>0] = 1;
 $58 = HEAP8[$aff>>0]|0;
 $59 = $58 << 24 >> 24;
 $60 = ($59|0)<(67);
 if (!($60)) {
  $86 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($86|0);
 }
 $61 = HEAP32[$zChar>>2]|0;
 $62 = ($61|0)!=(0|0);
 if (!($62)) {
  $85 = HEAP32[$1>>2]|0;
  HEAP8[$85>>0] = 5;
  $86 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($86|0);
 }
 while(1) {
  $63 = HEAP32[$zChar>>2]|0;
  $64 = HEAP8[$63>>0]|0;
  $65 = ($64<<24>>24)!=(0);
  if (!($65)) {
   label = 33;
   break;
  }
  $66 = HEAP32[$zChar>>2]|0;
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = (18403 + ($68)|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = $70&255;
  $72 = $71 & 4;
  $73 = ($72|0)!=(0);
  if ($73) {
   break;
  }
  $83 = HEAP32[$zChar>>2]|0;
  $84 = ((($83)) + 1|0);
  HEAP32[$zChar>>2] = $84;
 }
 if ((label|0) == 33) {
  $86 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($86|0);
 }
 HEAP32[$v>>2] = 0;
 $74 = HEAP32[$zChar>>2]|0;
 (_sqlite3GetInt32($74,$v)|0);
 $75 = HEAP32[$v>>2]|0;
 $76 = (($75|0) / 4)&-1;
 $77 = (($76) + 1)|0;
 HEAP32[$v>>2] = $77;
 $78 = HEAP32[$v>>2]|0;
 $79 = ($78|0)>(255);
 $$ = $79 ? 255 : $77;
 HEAP32[$v>>2] = $$;
 $80 = HEAP32[$v>>2]|0;
 $81 = $80&255;
 $82 = HEAP32[$1>>2]|0;
 HEAP8[$82>>0] = $81;
 $86 = HEAP8[$aff>>0]|0;
 STACKTOP = sp;return ($86|0);
}
function _sqlite3VdbeMemCast($pMem,$aff,$encoding) {
 $pMem = $pMem|0;
 $aff = $aff|0;
 $encoding = $encoding|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pMem;
 HEAP8[$1>>0] = $aff;
 HEAP8[$2>>0] = $encoding;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 if ($8) {
  STACKTOP = sp;return;
 }
 $9 = HEAP8[$1>>0]|0;
 $10 = $9&255;
 switch ($10|0) {
 case 65:  {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP16[$12>>1]|0;
  $14 = $13&65535;
  $15 = $14 & 16;
  $16 = ($15|0)==(0);
  $17 = HEAP32[$0>>2]|0;
  if (!($16)) {
   $34 = ((($17)) + 8|0);
   $35 = HEAP16[$34>>1]|0;
   $36 = $35&65535;
   $37 = $36 & -33264;
   $38 = $37&65535;
   HEAP16[$34>>1] = $38;
   STACKTOP = sp;return;
  }
  $18 = HEAP8[$2>>0]|0;
  _sqlite3ValueApplyAffinity($17,66,$18);
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = $21&65535;
  $23 = $22 & 2;
  $24 = ($23|0)!=(0);
  if (!($24)) {
   STACKTOP = sp;return;
  }
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = HEAP16[$26>>1]|0;
  $28 = $27&65535;
  $29 = $28 & -49664;
  $30 = $29 | 16;
  $31 = $30&65535;
  $32 = HEAP32[$0>>2]|0;
  $33 = ((($32)) + 8|0);
  HEAP16[$33>>1] = $31;
  STACKTOP = sp;return;
  break;
 }
 case 67:  {
  $39 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemNumerify($39)|0);
  STACKTOP = sp;return;
  break;
 }
 case 68:  {
  $40 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemIntegerify($40)|0);
  STACKTOP = sp;return;
  break;
 }
 case 69:  {
  $41 = HEAP32[$0>>2]|0;
  (_sqlite3VdbeMemRealify($41)|0);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $42 = HEAP32[$0>>2]|0;
  $43 = ((($42)) + 8|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 & 16;
  $47 = $46 >> 3;
  $48 = HEAP32[$0>>2]|0;
  $49 = ((($48)) + 8|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = $50&65535;
  $52 = $51 | $47;
  $53 = $52&65535;
  HEAP16[$49>>1] = $53;
  $54 = HEAP32[$0>>2]|0;
  $55 = HEAP8[$2>>0]|0;
  _sqlite3ValueApplyAffinity($54,66,$55);
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 8|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58&65535;
  $60 = $59 & -16413;
  $61 = $60&65535;
  HEAP16[$57>>1] = $61;
  STACKTOP = sp;return;
 }
 }
}
function _sqlite3ValueApplyAffinity($pVal,$affinity,$enc) {
 $pVal = $pVal|0;
 $affinity = $affinity|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pVal;
 HEAP8[$1>>0] = $affinity;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP8[$1>>0]|0;
 $5 = HEAP8[$2>>0]|0;
 _applyAffinity($3,$4,$5);
 STACKTOP = sp;return;
}
function _valueNew($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3ValueNew($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _sqlite3VdbeMemNumerify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 13;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = ((($7)) + 16|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$0>>2]|0;
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 12|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 10|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = (_sqlite3Atoi64($9,$10,$13,$16)|0);
   $18 = (0)==($17|0);
   $19 = HEAP32[$0>>2]|0;
   if ($18) {
    $20 = ((($19)) + 8|0);
    $21 = HEAP16[$20>>1]|0;
    $22 = $21&65535;
    $23 = $22 & -49664;
    $24 = $23 | 4;
    $25 = $24&65535;
    $26 = HEAP32[$0>>2]|0;
    $27 = ((($26)) + 8|0);
    HEAP16[$27>>1] = $25;
    break;
   } else {
    $28 = (+_sqlite3VdbeRealValue($19));
    $29 = HEAP32[$0>>2]|0;
    HEAPF64[$29>>3] = $28;
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 8|0);
    $32 = HEAP16[$31>>1]|0;
    $33 = $32&65535;
    $34 = $33 & -49664;
    $35 = $34 | 8;
    $36 = $35&65535;
    $37 = HEAP32[$0>>2]|0;
    $38 = ((($37)) + 8|0);
    HEAP16[$38>>1] = $36;
    $39 = HEAP32[$0>>2]|0;
    _sqlite3VdbeIntegerAffinity($39);
    break;
   }
  }
 } while(0);
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 8|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = $42&65535;
 $44 = $43 & -16403;
 $45 = $44&65535;
 HEAP16[$41>>1] = $45;
 STACKTOP = sp;return 0;
}
function _sqlite3HexToBlob($db,$z,$n) {
 $db = $db|0;
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zBlob = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $zBlob = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($4|0) / 2)&-1;
 $6 = (($5) + 1)|0;
 $7 = ($6|0)<(0);
 $8 = $7 << 31 >> 31;
 $9 = (_sqlite3DbMallocRawNN($3,$6,$8)|0);
 HEAP32[$zBlob>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) + -1)|0;
 HEAP32[$2>>2] = $11;
 $12 = HEAP32[$zBlob>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $44 = HEAP32[$zBlob>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($14|0)<($15|0);
  $17 = HEAP32[$i>>2]|0;
  if (!($16)) {
   break;
  }
  $18 = HEAP32[$1>>2]|0;
  $19 = (($18) + ($17)|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = (_sqlite3HexToInt($21)|0);
  $23 = $22&255;
  $24 = $23 << 4;
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = (($27) + ($26)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = (_sqlite3HexToInt($30)|0);
  $32 = $31&255;
  $33 = $24 | $32;
  $34 = $33&255;
  $35 = HEAP32[$i>>2]|0;
  $36 = (($35|0) / 2)&-1;
  $37 = HEAP32[$zBlob>>2]|0;
  $38 = (($37) + ($36)|0);
  HEAP8[$38>>0] = $34;
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 2)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = (($17|0) / 2)&-1;
 $42 = HEAP32[$zBlob>>2]|0;
 $43 = (($42) + ($41)|0);
 HEAP8[$43>>0] = 0;
 $44 = HEAP32[$zBlob>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3VdbeIntegerAffinity($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $ix = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $ix = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = +HEAPF64[$1>>3];
 $3 = (_doubleToInt64($2)|0);
 $4 = tempRet0;
 $5 = $ix;
 $6 = $5;
 HEAP32[$6>>2] = $3;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $4;
 $9 = HEAP32[$0>>2]|0;
 $10 = +HEAPF64[$9>>3];
 $11 = $ix;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (+($13>>>0)) + (4294967296.0*(+($16|0)));
 $18 = $10 == $17;
 $19 = $ix;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)>(-2147483648);
 $26 = ($21>>>0)>(0);
 $27 = ($24|0)==(-2147483648);
 $28 = $27 & $26;
 $29 = $25 | $28;
 $or$cond = $18 & $29;
 $30 = $ix;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)<(2147483647);
 $37 = ($32>>>0)<(4294967295);
 $38 = ($35|0)==(2147483647);
 $39 = $38 & $37;
 $40 = $36 | $39;
 $or$cond3 = $or$cond & $40;
 if (!($or$cond3)) {
  STACKTOP = sp;return;
 }
 $41 = $ix;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$0>>2]|0;
 $48 = $47;
 $49 = $48;
 HEAP32[$49>>2] = $43;
 $50 = (($48) + 4)|0;
 $51 = $50;
 HEAP32[$51>>2] = $46;
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 8|0);
 $54 = HEAP16[$53>>1]|0;
 $55 = $54&65535;
 $56 = $55 & -49664;
 $57 = $56 | 4;
 $58 = $57&65535;
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 8|0);
 HEAP16[$60>>1] = $58;
 STACKTOP = sp;return;
}
function _doubleToInt64($r) {
 $r = +$r;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0.0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp;
 HEAPF64[$1>>3] = $r;
 $2 = +HEAPF64[$1>>3];
 $3 = $2 <= -9.2233720368547758E+18;
 do {
  if ($3) {
   $4 = $0;
   $5 = $4;
   HEAP32[$5>>2] = 0;
   $6 = (($4) + 4)|0;
   $7 = $6;
   HEAP32[$7>>2] = -2147483648;
  } else {
   $8 = +HEAPF64[$1>>3];
   $9 = $8 >= 9.2233720368547758E+18;
   if ($9) {
    $10 = $0;
    $11 = $10;
    HEAP32[$11>>2] = -1;
    $12 = (($10) + 4)|0;
    $13 = $12;
    HEAP32[$13>>2] = 2147483647;
    break;
   } else {
    $14 = +HEAPF64[$1>>3];
    $15 = (~~$14)>>>0;
    $16 = +Math_abs($14) >= 1.0 ? $14 > 0.0 ? (~~+Math_min(+Math_floor($14 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($14 - +(~~$14 >>> 0)) / 4294967296.0) >>> 0 : 0;
    $17 = $0;
    $18 = $17;
    HEAP32[$18>>2] = $15;
    $19 = (($17) + 4)|0;
    $20 = $19;
    HEAP32[$20>>2] = $16;
    break;
   }
  }
 } while(0);
 $21 = $0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = (($21) + 4)|0;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 tempRet0 = ($26);
 STACKTOP = sp;return ($23|0);
}
function _applyAffinity($pRec,$affinity,$enc) {
 $pRec = $pRec|0;
 $affinity = $affinity|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 5|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $pRec;
 HEAP8[$1>>0] = $affinity;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP8[$1>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)>=(67);
 if (!($5)) {
  $25 = HEAP8[$1>>0]|0;
  $26 = $25 << 24 >> 24;
  $27 = ($26|0)==(66);
  if (!($27)) {
   STACKTOP = sp;return;
  }
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 8|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 & 2;
  $33 = (0)==($32|0);
  if ($33) {
   $34 = HEAP32[$0>>2]|0;
   $35 = ((($34)) + 8|0);
   $36 = HEAP16[$35>>1]|0;
   $37 = $36&65535;
   $38 = $37 & 12;
   $39 = ($38|0)!=(0);
   if ($39) {
    $40 = HEAP32[$0>>2]|0;
    $41 = HEAP8[$2>>0]|0;
    (_sqlite3VdbeMemStringify($40,$41,1)|0);
   }
  }
  $42 = HEAP32[$0>>2]|0;
  $43 = ((($42)) + 8|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 & -13;
  $47 = $46&65535;
  HEAP16[$43>>1] = $47;
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 $10 = $9 & 4;
 $11 = ($10|0)==(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 8;
 $17 = ($16|0)==(0);
 $18 = HEAP32[$0>>2]|0;
 if (!($17)) {
  _sqlite3VdbeIntegerAffinity($18);
  STACKTOP = sp;return;
 }
 $19 = ((($18)) + 8|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 & 2;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 _applyNumericAffinity($24,1);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemStringify($pMem,$enc,$bForce) {
 $pMem = $pMem|0;
 $enc = $enc|0;
 $bForce = $bForce|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fg = 0, $nByte = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 33|0;
 $3 = sp + 32|0;
 $fg = sp + 20|0;
 $nByte = sp + 16|0;
 HEAP32[$1>>2] = $pMem;
 HEAP8[$2>>0] = $enc;
 HEAP8[$3>>0] = $bForce;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$fg>>2] = $7;
 HEAP32[$nByte>>2] = 32;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3VdbeMemClearAndResize($8,32)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 10|0);
  HEAP8[$12>>0] = 0;
  HEAP32[$0>>2] = 7;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $13 = HEAP32[$fg>>2]|0;
 $14 = $13 & 4;
 $15 = ($14|0)!=(0);
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 if ($15) {
  $20 = $19;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($20) + 4)|0;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = $vararg_buffer;
  $27 = $26;
  HEAP32[$27>>2] = $22;
  $28 = (($26) + 4)|0;
  $29 = $28;
  HEAP32[$29>>2] = $25;
  (_sqlite3_snprintf(32,$18,28266,$vararg_buffer)|0);
 } else {
  $30 = +HEAPF64[$19>>3];
  HEAPF64[$vararg_buffer1>>3] = $30;
  (_sqlite3_snprintf(32,$18,28271,$vararg_buffer1)|0);
 }
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 16|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (_sqlite3Strlen30($33)|0);
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 12|0);
 HEAP32[$36>>2] = $34;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 10|0);
 HEAP8[$38>>0] = 1;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 8|0);
 $41 = HEAP16[$40>>1]|0;
 $42 = $41&65535;
 $43 = $42 | 514;
 $44 = $43&65535;
 HEAP16[$40>>1] = $44;
 $45 = HEAP8[$3>>0]|0;
 $46 = ($45<<24>>24)!=(0);
 if ($46) {
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 8|0);
  $49 = HEAP16[$48>>1]|0;
  $50 = $49&65535;
  $51 = $50 & -13;
  $52 = $51&65535;
  HEAP16[$48>>1] = $52;
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = HEAP8[$2>>0]|0;
 $55 = $54&255;
 (_sqlite3VdbeChangeEncoding($53,$55)|0);
 HEAP32[$0>>2] = 0;
 $56 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _sqlite3VdbeMemClearAndResize($pMem,$szNew) {
 $pMem = $pMem|0;
 $szNew = $szNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $szNew;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($5|0)<($6|0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3VdbeMemGrow($8,$9,0)|0);
  HEAP32[$0>>2] = $10;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 } else {
  $11 = ((($8)) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 16|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & 13;
  $20 = $19&65535;
  HEAP16[$16>>1] = $20;
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3VdbeMemGrow($pMem,$n,$bPreserve) {
 $pMem = $pMem|0;
 $n = $n|0;
 $bPreserve = $bPreserve|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $bPreserve;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($6|0)<($7|0);
 do {
  if ($8) {
   $9 = HEAP32[$2>>2]|0;
   $10 = ($9|0)<(32);
   if ($10) {
    HEAP32[$2>>2] = 32;
   }
   $11 = HEAP32[$3>>2]|0;
   $12 = ($11|0)!=(0);
   if ($12) {
    $13 = HEAP32[$1>>2]|0;
    $14 = ((($13)) + 24|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = ($15|0)>(0);
    if ($16) {
     $17 = HEAP32[$1>>2]|0;
     $18 = ((($17)) + 16|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = HEAP32[$1>>2]|0;
     $21 = ((($20)) + 20|0);
     $22 = HEAP32[$21>>2]|0;
     $23 = ($19|0)==($22|0);
     if ($23) {
      $24 = HEAP32[$1>>2]|0;
      $25 = ((($24)) + 32|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = HEAP32[$1>>2]|0;
      $28 = ((($27)) + 16|0);
      $29 = HEAP32[$28>>2]|0;
      $30 = HEAP32[$2>>2]|0;
      $31 = ($30|0)<(0);
      $32 = $31 << 31 >> 31;
      $33 = (_sqlite3DbReallocOrFree($26,$29,$30,$32)|0);
      $34 = HEAP32[$1>>2]|0;
      $35 = ((($34)) + 20|0);
      HEAP32[$35>>2] = $33;
      $36 = HEAP32[$1>>2]|0;
      $37 = ((($36)) + 16|0);
      HEAP32[$37>>2] = $33;
      HEAP32[$3>>2] = 0;
     } else {
      label = 8;
     }
    } else {
     label = 8;
    }
   } else {
    label = 8;
   }
   if ((label|0) == 8) {
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 24|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = ($40|0)>(0);
    if ($41) {
     $42 = HEAP32[$1>>2]|0;
     $43 = ((($42)) + 32|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$1>>2]|0;
     $46 = ((($45)) + 20|0);
     $47 = HEAP32[$46>>2]|0;
     _sqlite3DbFree($44,$47);
    }
    $48 = HEAP32[$1>>2]|0;
    $49 = ((($48)) + 32|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = HEAP32[$2>>2]|0;
    $52 = ($51|0)<(0);
    $53 = $52 << 31 >> 31;
    $54 = (_sqlite3DbMallocRaw($50,$51,$53)|0);
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 20|0);
    HEAP32[$56>>2] = $54;
   }
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 20|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)==(0|0);
   $61 = HEAP32[$1>>2]|0;
   if (!($60)) {
    $66 = ((($61)) + 32|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = HEAP32[$1>>2]|0;
    $69 = ((($68)) + 20|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = (_sqlite3DbMallocSize($67,$70)|0);
    $72 = HEAP32[$1>>2]|0;
    $73 = ((($72)) + 24|0);
    HEAP32[$73>>2] = $71;
    break;
   }
   _sqlite3VdbeMemSetNull($61);
   $62 = HEAP32[$1>>2]|0;
   $63 = ((($62)) + 16|0);
   HEAP32[$63>>2] = 0;
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 24|0);
   HEAP32[$65>>2] = 0;
   HEAP32[$0>>2] = 7;
   $119 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($119|0);
  }
 } while(0);
 $74 = HEAP32[$3>>2]|0;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$1>>2]|0;
  $77 = ((($76)) + 16|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)!=(0|0);
  if ($79) {
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 16|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = HEAP32[$1>>2]|0;
   $84 = ((($83)) + 20|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = ($82|0)!=($85|0);
   if ($86) {
    $87 = HEAP32[$1>>2]|0;
    $88 = ((($87)) + 20|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = HEAP32[$1>>2]|0;
    $91 = ((($90)) + 16|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = HEAP32[$1>>2]|0;
    $94 = ((($93)) + 12|0);
    $95 = HEAP32[$94>>2]|0;
    _memcpy(($89|0),($92|0),($95|0))|0;
   }
  }
 }
 $96 = HEAP32[$1>>2]|0;
 $97 = ((($96)) + 8|0);
 $98 = HEAP16[$97>>1]|0;
 $99 = $98&65535;
 $100 = $99 & 1024;
 $101 = ($100|0)!=(0);
 if ($101) {
  $102 = HEAP32[$1>>2]|0;
  $103 = ((($102)) + 36|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = HEAP32[$1>>2]|0;
  $106 = ((($105)) + 16|0);
  $107 = HEAP32[$106>>2]|0;
  FUNCTION_TABLE_vi[$104 & 255]($107);
 }
 $108 = HEAP32[$1>>2]|0;
 $109 = ((($108)) + 20|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = HEAP32[$1>>2]|0;
 $112 = ((($111)) + 16|0);
 HEAP32[$112>>2] = $110;
 $113 = HEAP32[$1>>2]|0;
 $114 = ((($113)) + 8|0);
 $115 = HEAP16[$114>>1]|0;
 $116 = $115&65535;
 $117 = $116 & -7169;
 $118 = $117&65535;
 HEAP16[$114>>1] = $118;
 HEAP32[$0>>2] = 0;
 $119 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($119|0);
}
function _sqlite3DbReallocOrFree($db,$p,$0,$1) {
 $db = $db|0;
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $pNew = sp + 8|0;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = $4;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3DbRealloc($9,$10,$13,$16)|0);
 HEAP32[$pNew>>2] = $17;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $22 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $20 = HEAP32[$2>>2]|0;
 $21 = HEAP32[$3>>2]|0;
 _sqlite3DbFree($20,$21);
 $22 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3VdbeMemIntegerify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3VdbeIntValue($1)|0);
 $3 = tempRet0;
 $4 = HEAP32[$0>>2]|0;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $2;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $3;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = $12 & -49664;
 $14 = $13 | 4;
 $15 = $14&65535;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 8|0);
 HEAP16[$17>>1] = $15;
 STACKTOP = sp;return 0;
}
function _sqlite3VdbeMemRealify($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = (+_sqlite3VdbeRealValue($1));
 $3 = HEAP32[$0>>2]|0;
 HEAPF64[$3>>3] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & -49664;
 $9 = $8 | 8;
 $10 = $9&65535;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 HEAP16[$12>>1] = $10;
 STACKTOP = sp;return 0;
}
function _sqlite3CheckObjectName($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 148|0);
 $6 = ((($5)) + 5|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 18|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(0);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = ((($15)) + 24|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = $17 & 2048;
   $19 = ($18|0)==(0);
   if ($19) {
    $20 = HEAP32[$2>>2]|0;
    $21 = (_sqlite3_strnicmp($20,27844,7)|0);
    $22 = (0)==($21|0);
    if ($22) {
     $23 = HEAP32[$1>>2]|0;
     $24 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer>>2] = $24;
     _sqlite3ErrorMsg($23,30775,$vararg_buffer);
     HEAP32[$0>>2] = 1;
     $25 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($25|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3ViewGetColumnNames($pParse,$pTable) {
 $pParse = $pParse|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $n = 0, $nErr = 0, $pSel = 0, $pSelTab = 0, $vararg_buffer = 0;
 var $xAuth = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pSelTab = sp + 24|0;
 $pSel = sp + 20|0;
 $nErr = sp + 16|0;
 $n = sp + 12|0;
 $db = sp + 8|0;
 $xAuth = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTable;
 HEAP32[$nErr>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_sqlite3VtabCallConnect($5,$6)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $133 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($133|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 16;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $133 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($133|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 34|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 << 16 >> 16;
 $19 = ($18|0)>(0);
 if ($19) {
  HEAP32[$0>>2] = 0;
  $133 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($133|0);
 }
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 34|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$vararg_buffer>>2] = $27;
  _sqlite3ErrorMsg($25,28983,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $133 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($133|0);
 }
 $28 = HEAP32[$db>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = ((($29)) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (_sqlite3SelectDup($28,$31,0)|0);
 HEAP32[$pSel>>2] = $32;
 $33 = HEAP32[$pSel>>2]|0;
 $34 = ($33|0)!=(0|0);
 if ($34) {
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 40|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$n>>2] = $37;
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$pSel>>2]|0;
  $40 = ((($39)) + 28|0);
  $41 = HEAP32[$40>>2]|0;
  _sqlite3SrcListAssignCursors($38,$41);
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 34|0);
  HEAP16[$43>>1] = -1;
  $44 = HEAP32[$db>>2]|0;
  $45 = ((($44)) + 256|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + 1)|0;
  HEAP32[$45>>2] = $47;
  $48 = HEAP32[$db>>2]|0;
  $49 = ((($48)) + 296|0);
  $50 = HEAP32[$49>>2]|0;
  HEAP32[$xAuth>>2] = $50;
  $51 = HEAP32[$db>>2]|0;
  $52 = ((($51)) + 296|0);
  HEAP32[$52>>2] = 0;
  $53 = HEAP32[$1>>2]|0;
  $54 = HEAP32[$pSel>>2]|0;
  $55 = (_sqlite3ResultSetOfSelect($53,$54)|0);
  HEAP32[$pSelTab>>2] = $55;
  $56 = HEAP32[$xAuth>>2]|0;
  $57 = HEAP32[$db>>2]|0;
  $58 = ((($57)) + 296|0);
  HEAP32[$58>>2] = $56;
  $59 = HEAP32[$n>>2]|0;
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 40|0);
  HEAP32[$61>>2] = $59;
  $62 = HEAP32[$2>>2]|0;
  $63 = ((($62)) + 24|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($64|0)!=(0|0);
  do {
   if ($65) {
    $66 = HEAP32[$1>>2]|0;
    $67 = HEAP32[$2>>2]|0;
    $68 = ((($67)) + 24|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = HEAP32[$2>>2]|0;
    $71 = ((($70)) + 34|0);
    $72 = HEAP32[$2>>2]|0;
    $73 = ((($72)) + 4|0);
    (_sqlite3ColumnsFromExprList($66,$69,$71,$73)|0);
    $74 = HEAP32[$db>>2]|0;
    $75 = ((($74)) + 69|0);
    $76 = HEAP8[$75>>0]|0;
    $77 = $76&255;
    $78 = ($77|0)==(0);
    if ($78) {
     $79 = HEAP32[$1>>2]|0;
     $80 = ((($79)) + 36|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = ($81|0)==(0);
     if ($82) {
      $83 = HEAP32[$2>>2]|0;
      $84 = ((($83)) + 34|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85 << 16 >> 16;
      $87 = HEAP32[$pSel>>2]|0;
      $88 = HEAP32[$87>>2]|0;
      $89 = HEAP32[$88>>2]|0;
      $90 = ($86|0)==($89|0);
      if ($90) {
       $91 = HEAP32[$1>>2]|0;
       $92 = HEAP32[$2>>2]|0;
       $93 = HEAP32[$pSel>>2]|0;
       _sqlite3SelectAddColumnTypeAndCollation($91,$92,$93);
      }
     }
    }
   } else {
    $94 = HEAP32[$pSelTab>>2]|0;
    $95 = ($94|0)!=(0|0);
    if ($95) {
     $96 = HEAP32[$pSelTab>>2]|0;
     $97 = ((($96)) + 34|0);
     $98 = HEAP16[$97>>1]|0;
     $99 = HEAP32[$2>>2]|0;
     $100 = ((($99)) + 34|0);
     HEAP16[$100>>1] = $98;
     $101 = HEAP32[$pSelTab>>2]|0;
     $102 = ((($101)) + 4|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = HEAP32[$2>>2]|0;
     $105 = ((($104)) + 4|0);
     HEAP32[$105>>2] = $103;
     $106 = HEAP32[$pSelTab>>2]|0;
     $107 = ((($106)) + 34|0);
     HEAP16[$107>>1] = 0;
     $108 = HEAP32[$pSelTab>>2]|0;
     $109 = ((($108)) + 4|0);
     HEAP32[$109>>2] = 0;
     break;
    } else {
     $110 = HEAP32[$2>>2]|0;
     $111 = ((($110)) + 34|0);
     HEAP16[$111>>1] = 0;
     $112 = HEAP32[$nErr>>2]|0;
     $113 = (($112) + 1)|0;
     HEAP32[$nErr>>2] = $113;
     break;
    }
   }
  } while(0);
  $114 = HEAP32[$db>>2]|0;
  $115 = HEAP32[$pSelTab>>2]|0;
  _sqlite3DeleteTable($114,$115);
  $116 = HEAP32[$db>>2]|0;
  $117 = HEAP32[$pSel>>2]|0;
  _sqlite3SelectDelete($116,$117);
  $118 = HEAP32[$db>>2]|0;
  $119 = ((($118)) + 256|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = (($120) + -1)|0;
  HEAP32[$119>>2] = $121;
 } else {
  $122 = HEAP32[$nErr>>2]|0;
  $123 = (($122) + 1)|0;
  HEAP32[$nErr>>2] = $123;
 }
 $124 = HEAP32[$2>>2]|0;
 $125 = ((($124)) + 64|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = ((($126)) + 78|0);
 $128 = HEAP16[$127>>1]|0;
 $129 = $128&65535;
 $130 = $129 | 2;
 $131 = $130&65535;
 HEAP16[$127>>1] = $131;
 $132 = HEAP32[$nErr>>2]|0;
 HEAP32[$0>>2] = $132;
 $133 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($133|0);
}
function _sqlite3GetVTable($db,$pTab) {
 $db = $db|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pVtab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pVtab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pVtab>>2] = $4;
 while(1) {
  $5 = HEAP32[$pVtab>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$pVtab>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ($8|0)!=($9|0);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$pVtab>>2]|0;
  $12 = ((($11)) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pVtab>>2] = $13;
 }
 if ((label|0) == 5) {
  $14 = HEAP32[$pVtab>>2]|0;
  STACKTOP = sp;return ($14|0);
 }
 return (0)|0;
}
function _sqlite3MayAbort($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 124|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ((($5)) + 124|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = $7;
 } else {
  $8 = $5;
 }
 HEAP32[$pToplevel>>2] = $8;
 $9 = HEAP32[$pToplevel>>2]|0;
 $10 = ((($9)) + 21|0);
 HEAP8[$10>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3Utf8CharLen81($zIn,$nByte) {
 $zIn = $zIn|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, $z = 0, $zTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $r = sp + 8|0;
 $z = sp + 4|0;
 $zTerm = sp;
 HEAP32[$0>>2] = $zIn;
 HEAP32[$1>>2] = $nByte;
 HEAP32[$r>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$z>>2] = $2;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>=(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$z>>2]|0;
  $7 = (($6) + ($5)|0);
  HEAP32[$zTerm>>2] = $7;
 } else {
  HEAP32[$zTerm>>2] = (-1);
 }
 while(1) {
  $8 = HEAP32[$z>>2]|0;
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = ($10|0)!=(0);
  if (!($11)) {
   label = 10;
   break;
  }
  $12 = HEAP32[$z>>2]|0;
  $13 = HEAP32[$zTerm>>2]|0;
  $14 = ($12>>>0)<($13>>>0);
  if (!($14)) {
   label = 10;
   break;
  }
  $15 = HEAP32[$z>>2]|0;
  $16 = ((($15)) + 1|0);
  HEAP32[$z>>2] = $16;
  $17 = HEAP8[$15>>0]|0;
  $18 = $17&255;
  $19 = ($18|0)>=(192);
  L8: do {
   if ($19) {
    while(1) {
     $20 = HEAP32[$z>>2]|0;
     $21 = HEAP8[$20>>0]|0;
     $22 = $21&255;
     $23 = $22 & 192;
     $24 = ($23|0)==(128);
     if (!($24)) {
      break L8;
     }
     $25 = HEAP32[$z>>2]|0;
     $26 = ((($25)) + 1|0);
     HEAP32[$z>>2] = $26;
    }
   }
  } while(0);
  $27 = HEAP32[$r>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$r>>2] = $28;
 }
 if ((label|0) == 10) {
  $29 = HEAP32[$r>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _whereForeignKeys($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $zWhere = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $zWhere = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$zWhere>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3FkReferences($2)|0);
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP32[$zWhere>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (_whereOrName($7,$8,$11)|0);
  HEAP32[$zWhere>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$p>>2] = $15;
 }
 $16 = HEAP32[$zWhere>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _sqlite3FkReferences($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 56|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3HashFind($4,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _sqlite3VtabCallConnect($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pMod = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $zErr = 0, $zMod = 0, $zModule = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $db = sp + 32|0;
 $zMod = sp + 28|0;
 $pMod = sp + 24|0;
 $rc = sp + 20|0;
 $zModule = sp + 16|0;
 $zErr = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 42|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = $8 & 16;
 $10 = ($9|0)==(0);
 if (!($10)) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_sqlite3GetVTable($11,$12)|0);
  $14 = ($13|0)!=(0|0);
  if (!($14)) {
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 52|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$zMod>>2] = $18;
   $19 = HEAP32[$db>>2]|0;
   $20 = ((($19)) + 320|0);
   $21 = HEAP32[$zMod>>2]|0;
   $22 = (_sqlite3HashFind($20,$21)|0);
   HEAP32[$pMod>>2] = $22;
   $23 = HEAP32[$pMod>>2]|0;
   $24 = ($23|0)!=(0|0);
   if ($24) {
    HEAP32[$zErr>>2] = 0;
    $31 = HEAP32[$db>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = HEAP32[$pMod>>2]|0;
    $34 = HEAP32[$pMod>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    $36 = ((($35)) + 8|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (_vtabCallConstructor($31,$32,$33,$37,$zErr)|0);
    HEAP32[$rc>>2] = $38;
    $39 = HEAP32[$rc>>2]|0;
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$zErr>>2]|0;
     HEAP32[$vararg_buffer1>>2] = $42;
     _sqlite3ErrorMsg($41,18391,$vararg_buffer1);
    }
    $43 = HEAP32[$db>>2]|0;
    $44 = HEAP32[$zErr>>2]|0;
    _sqlite3DbFree($43,$44);
   } else {
    $25 = HEAP32[$2>>2]|0;
    $26 = ((($25)) + 52|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$27>>2]|0;
    HEAP32[$zModule>>2] = $28;
    $29 = HEAP32[$1>>2]|0;
    $30 = HEAP32[$zModule>>2]|0;
    HEAP32[$vararg_buffer>>2] = $30;
    _sqlite3ErrorMsg($29,30631,$vararg_buffer);
    HEAP32[$rc>>2] = 1;
   }
   $45 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $45;
   $46 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($46|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _sqlite3SelectDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $pNew = 0, $pPrior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pNew = sp + 4|0;
 $pPrior = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3DbMallocRawNN($6,68,0)|0);
 HEAP32[$pNew>>2] = $7;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $120 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_sqlite3ExprListDup($10,$12,$13)|0);
 $15 = HEAP32[$pNew>>2]|0;
 HEAP32[$15>>2] = $14;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 28|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = (_sqlite3SrcListDup($16,$19,$20)|0);
 $22 = HEAP32[$pNew>>2]|0;
 $23 = ((($22)) + 28|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$1>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = (_sqlite3ExprDup($24,$27,$28)|0);
 $30 = HEAP32[$pNew>>2]|0;
 $31 = ((($30)) + 32|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 36|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = (_sqlite3ExprListDup($32,$35,$36)|0);
 $38 = HEAP32[$pNew>>2]|0;
 $39 = ((($38)) + 36|0);
 HEAP32[$39>>2] = $37;
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 40|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 $45 = (_sqlite3ExprDup($40,$43,$44)|0);
 $46 = HEAP32[$pNew>>2]|0;
 $47 = ((($46)) + 40|0);
 HEAP32[$47>>2] = $45;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = ((($49)) + 44|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 $53 = (_sqlite3ExprListDup($48,$51,$52)|0);
 $54 = HEAP32[$pNew>>2]|0;
 $55 = ((($54)) + 44|0);
 HEAP32[$55>>2] = $53;
 $56 = HEAP32[$2>>2]|0;
 $57 = ((($56)) + 4|0);
 $58 = HEAP8[$57>>0]|0;
 $59 = HEAP32[$pNew>>2]|0;
 $60 = ((($59)) + 4|0);
 HEAP8[$60>>0] = $58;
 $61 = HEAP32[$1>>2]|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = ((($62)) + 48|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$3>>2]|0;
 $66 = (_sqlite3SelectDup($61,$64,$65)|0);
 HEAP32[$pPrior>>2] = $66;
 $67 = HEAP32[$pNew>>2]|0;
 $68 = ((($67)) + 48|0);
 HEAP32[$68>>2] = $66;
 $69 = HEAP32[$pPrior>>2]|0;
 $70 = ($69|0)!=(0|0);
 if ($70) {
  $71 = HEAP32[$pNew>>2]|0;
  $72 = HEAP32[$pPrior>>2]|0;
  $73 = ((($72)) + 52|0);
  HEAP32[$73>>2] = $71;
 }
 $74 = HEAP32[$pNew>>2]|0;
 $75 = ((($74)) + 52|0);
 HEAP32[$75>>2] = 0;
 $76 = HEAP32[$1>>2]|0;
 $77 = HEAP32[$2>>2]|0;
 $78 = ((($77)) + 56|0);
 $79 = HEAP32[$78>>2]|0;
 $80 = HEAP32[$3>>2]|0;
 $81 = (_sqlite3ExprDup($76,$79,$80)|0);
 $82 = HEAP32[$pNew>>2]|0;
 $83 = ((($82)) + 56|0);
 HEAP32[$83>>2] = $81;
 $84 = HEAP32[$1>>2]|0;
 $85 = HEAP32[$2>>2]|0;
 $86 = ((($85)) + 60|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = HEAP32[$3>>2]|0;
 $89 = (_sqlite3ExprDup($84,$87,$88)|0);
 $90 = HEAP32[$pNew>>2]|0;
 $91 = ((($90)) + 60|0);
 HEAP32[$91>>2] = $89;
 $92 = HEAP32[$pNew>>2]|0;
 $93 = ((($92)) + 12|0);
 HEAP32[$93>>2] = 0;
 $94 = HEAP32[$pNew>>2]|0;
 $95 = ((($94)) + 16|0);
 HEAP32[$95>>2] = 0;
 $96 = HEAP32[$2>>2]|0;
 $97 = ((($96)) + 8|0);
 $98 = HEAP32[$97>>2]|0;
 $99 = $98 & -33;
 $100 = HEAP32[$pNew>>2]|0;
 $101 = ((($100)) + 8|0);
 HEAP32[$101>>2] = $99;
 $102 = HEAP32[$pNew>>2]|0;
 $103 = ((($102)) + 20|0);
 HEAP32[$103>>2] = -1;
 $104 = HEAP32[$pNew>>2]|0;
 $105 = ((($104)) + 20|0);
 $106 = ((($105)) + 4|0);
 HEAP32[$106>>2] = -1;
 $107 = HEAP32[$2>>2]|0;
 $108 = ((($107)) + 6|0);
 $109 = HEAP16[$108>>1]|0;
 $110 = HEAP32[$pNew>>2]|0;
 $111 = ((($110)) + 6|0);
 HEAP16[$111>>1] = $109;
 $112 = HEAP32[$1>>2]|0;
 $113 = HEAP32[$2>>2]|0;
 $114 = ((($113)) + 64|0);
 $115 = HEAP32[$114>>2]|0;
 $116 = (_withDup($112,$115)|0);
 $117 = HEAP32[$pNew>>2]|0;
 $118 = ((($117)) + 64|0);
 HEAP32[$118>>2] = $116;
 $119 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $119;
 $120 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($120|0);
}
function _sqlite3SrcListAssignCursors($pParse,$pList) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP32[$pItem>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$pItem>>2]|0;
  $11 = ((($10)) + 44|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)>=(0);
  if ($13) {
   label = 8;
   break;
  }
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 40|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$15>>2] = $17;
  $18 = HEAP32[$pItem>>2]|0;
  $19 = ((($18)) + 44|0);
  HEAP32[$19>>2] = $16;
  $20 = HEAP32[$pItem>>2]|0;
  $21 = ((($20)) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$pItem>>2]|0;
   $26 = ((($25)) + 20|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ((($27)) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   _sqlite3SrcListAssignCursors($24,$29);
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
  $32 = HEAP32[$pItem>>2]|0;
  $33 = ((($32)) + 72|0);
  HEAP32[$pItem>>2] = $33;
 }
 if ((label|0) == 8) {
  STACKTOP = sp;return;
 }
}
function _sqlite3ResultSetOfSelect($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $pTab = 0, $savedFlags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pTab = sp + 8|0;
 $db = sp + 4|0;
 $savedFlags = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$savedFlags>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & -5;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 | 64;
 HEAP32[$13>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 _sqlite3SelectPrep($16,$17,0);
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 36|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 while(1) {
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 48|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$2>>2]|0;
  $27 = ((($26)) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$2>>2] = $28;
 }
 $29 = HEAP32[$savedFlags>>2]|0;
 $30 = HEAP32[$db>>2]|0;
 $31 = ((($30)) + 24|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$db>>2]|0;
 $33 = (_sqlite3DbMallocZero($32,72,0)|0);
 HEAP32[$pTab>>2] = $33;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = ($34|0)==(0|0);
 if ($35) {
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $36 = HEAP32[$pTab>>2]|0;
 $37 = ((($36)) + 36|0);
 HEAP16[$37>>1] = 1;
 $38 = HEAP32[$pTab>>2]|0;
 HEAP32[$38>>2] = 0;
 $39 = HEAP32[$pTab>>2]|0;
 $40 = ((($39)) + 38|0);
 HEAP16[$40>>1] = 200;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$2>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$pTab>>2]|0;
 $45 = ((($44)) + 34|0);
 $46 = HEAP32[$pTab>>2]|0;
 $47 = ((($46)) + 4|0);
 (_sqlite3ColumnsFromExprList($41,$43,$45,$47)|0);
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$pTab>>2]|0;
 $50 = HEAP32[$2>>2]|0;
 _sqlite3SelectAddColumnTypeAndCollation($48,$49,$50);
 $51 = HEAP32[$pTab>>2]|0;
 $52 = ((($51)) + 32|0);
 HEAP16[$52>>1] = -1;
 $53 = HEAP32[$db>>2]|0;
 $54 = ((($53)) + 69|0);
 $55 = HEAP8[$54>>0]|0;
 $56 = ($55<<24>>24)!=(0);
 if ($56) {
  $57 = HEAP32[$db>>2]|0;
  $58 = HEAP32[$pTab>>2]|0;
  _sqlite3DeleteTable($57,$58);
  HEAP32[$0>>2] = 0;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 } else {
  $59 = HEAP32[$pTab>>2]|0;
  HEAP32[$0>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return (0)|0;
}
function _sqlite3ColumnsFromExprList($pParse,$pEList,$pnCol,$paCol) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pnCol = $pnCol|0;
 $paCol = $paCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aCol = 0, $cnt = 0, $db = 0, $ht = 0, $i = 0, $iCol = 0, $j = 0, $nCol = 0, $nName = 0, $p = 0;
 var $pCol = 0, $pColExpr = 0, $pTab = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $3 = sp + 92|0;
 $4 = sp + 88|0;
 $db = sp + 84|0;
 $i = sp + 80|0;
 $j = sp + 76|0;
 $cnt = sp + 72|0;
 $aCol = sp + 68|0;
 $pCol = sp + 64|0;
 $nCol = sp + 60|0;
 $p = sp + 56|0;
 $zName = sp + 52|0;
 $nName = sp + 48|0;
 $ht = sp + 32|0;
 $pColExpr = sp + 28|0;
 $pTab = sp + 24|0;
 $iCol = sp + 20|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pnCol;
 HEAP32[$4>>2] = $paCol;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 _sqlite3HashInit($ht);
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$nCol>>2] = $10;
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$nCol>>2]|0;
  $13 = $12<<4;
  $14 = (_sqlite3DbMallocZero($11,$13,0)|0);
  HEAP32[$aCol>>2] = $14;
 } else {
  HEAP32[$nCol>>2] = 0;
  HEAP32[$aCol>>2] = 0;
 }
 $15 = HEAP32[$nCol>>2]|0;
 $16 = $15&65535;
 $17 = HEAP32[$3>>2]|0;
 HEAP16[$17>>1] = $16;
 $18 = HEAP32[$aCol>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 HEAP32[$19>>2] = $18;
 HEAP32[$i>>2] = 0;
 $20 = HEAP32[$aCol>>2]|0;
 HEAP32[$pCol>>2] = $20;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$nCol>>2]|0;
  $23 = ($21|0)<($22|0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$db>>2]|0;
  $25 = ((($24)) + 69|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = ($26<<24>>24)!=(0);
  $28 = $27 ^ 1;
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = ((($30)) + 4|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + (($29*20)|0)|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (_sqlite3ExprSkipCollate($34)|0);
  HEAP32[$p>>2] = $35;
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = ((($37)) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + (($36*20)|0)|0);
  $41 = ((($40)) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$zName>>2] = $42;
  $43 = ($42|0)!=(0|0);
  do {
   if (!($43)) {
    $44 = HEAP32[$p>>2]|0;
    HEAP32[$pColExpr>>2] = $44;
    while(1) {
     $45 = HEAP32[$pColExpr>>2]|0;
     $46 = HEAP8[$45>>0]|0;
     $47 = $46&255;
     $48 = ($47|0)==(122);
     $49 = HEAP32[$pColExpr>>2]|0;
     if (!($48)) {
      break;
     }
     $50 = ((($49)) + 16|0);
     $51 = HEAP32[$50>>2]|0;
     HEAP32[$pColExpr>>2] = $51;
    }
    $52 = HEAP8[$49>>0]|0;
    $53 = $52&255;
    $54 = ($53|0)==(152);
    if ($54) {
     $55 = HEAP32[$pColExpr>>2]|0;
     $56 = ((($55)) + 44|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$pColExpr>>2]|0;
      $60 = ((($59)) + 32|0);
      $61 = HEAP16[$60>>1]|0;
      $62 = $61 << 16 >> 16;
      HEAP32[$iCol>>2] = $62;
      $63 = HEAP32[$pColExpr>>2]|0;
      $64 = ((($63)) + 44|0);
      $65 = HEAP32[$64>>2]|0;
      HEAP32[$pTab>>2] = $65;
      $66 = HEAP32[$iCol>>2]|0;
      $67 = ($66|0)<(0);
      if ($67) {
       $68 = HEAP32[$pTab>>2]|0;
       $69 = ((($68)) + 32|0);
       $70 = HEAP16[$69>>1]|0;
       $71 = $70 << 16 >> 16;
       HEAP32[$iCol>>2] = $71;
      }
      $72 = HEAP32[$iCol>>2]|0;
      $73 = ($72|0)>=(0);
      if ($73) {
       $74 = HEAP32[$iCol>>2]|0;
       $75 = HEAP32[$pTab>>2]|0;
       $76 = ((($75)) + 4|0);
       $77 = HEAP32[$76>>2]|0;
       $78 = (($77) + ($74<<4)|0);
       $79 = HEAP32[$78>>2]|0;
       $80 = $79;
      } else {
       $80 = 24745;
      }
      HEAP32[$zName>>2] = $80;
      break;
     }
    }
    $81 = HEAP32[$pColExpr>>2]|0;
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = ($83|0)==(55);
    if ($84) {
     $85 = HEAP32[$pColExpr>>2]|0;
     $86 = ((($85)) + 8|0);
     $87 = HEAP32[$86>>2]|0;
     HEAP32[$zName>>2] = $87;
     break;
    } else {
     $88 = HEAP32[$i>>2]|0;
     $89 = HEAP32[$2>>2]|0;
     $90 = ((($89)) + 4|0);
     $91 = HEAP32[$90>>2]|0;
     $92 = (($91) + (($88*20)|0)|0);
     $93 = ((($92)) + 8|0);
     $94 = HEAP32[$93>>2]|0;
     HEAP32[$zName>>2] = $94;
     break;
    }
   }
  } while(0);
  $95 = HEAP32[$db>>2]|0;
  $96 = HEAP32[$zName>>2]|0;
  HEAP32[$vararg_buffer>>2] = $96;
  $97 = (_sqlite3MPrintf($95,18391,$vararg_buffer)|0);
  HEAP32[$zName>>2] = $97;
  HEAP32[$cnt>>2] = 0;
  while(1) {
   $98 = HEAP32[$zName>>2]|0;
   $99 = ($98|0)!=(0|0);
   if ($99) {
    $100 = HEAP32[$zName>>2]|0;
    $101 = (_sqlite3HashFind($ht,$100)|0);
    $102 = ($101|0)!=(0|0);
    $169 = $102;
   } else {
    $169 = 0;
   }
   $103 = HEAP32[$zName>>2]|0;
   if (!($169)) {
    break;
   }
   $104 = (_sqlite3Strlen30($103)|0);
   HEAP32[$nName>>2] = $104;
   $105 = HEAP32[$nName>>2]|0;
   $106 = ($105|0)>(0);
   if ($106) {
    $107 = HEAP32[$nName>>2]|0;
    $108 = (($107) - 1)|0;
    HEAP32[$j>>2] = $108;
    while(1) {
     $109 = HEAP32[$j>>2]|0;
     $110 = ($109|0)>(0);
     if (!($110)) {
      break;
     }
     $111 = HEAP32[$j>>2]|0;
     $112 = HEAP32[$zName>>2]|0;
     $113 = (($112) + ($111)|0);
     $114 = HEAP8[$113>>0]|0;
     $115 = $114&255;
     $116 = (18403 + ($115)|0);
     $117 = HEAP8[$116>>0]|0;
     $118 = $117&255;
     $119 = $118 & 4;
     $120 = ($119|0)!=(0);
     if (!($120)) {
      break;
     }
     $121 = HEAP32[$j>>2]|0;
     $122 = (($121) + -1)|0;
     HEAP32[$j>>2] = $122;
    }
    $123 = HEAP32[$j>>2]|0;
    $124 = HEAP32[$zName>>2]|0;
    $125 = (($124) + ($123)|0);
    $126 = HEAP8[$125>>0]|0;
    $127 = $126 << 24 >> 24;
    $128 = ($127|0)==(58);
    if ($128) {
     $129 = HEAP32[$j>>2]|0;
     HEAP32[$nName>>2] = $129;
    }
   }
   $130 = HEAP32[$db>>2]|0;
   $131 = HEAP32[$nName>>2]|0;
   $132 = HEAP32[$zName>>2]|0;
   $133 = HEAP32[$cnt>>2]|0;
   $134 = (($133) + 1)|0;
   HEAP32[$cnt>>2] = $134;
   HEAP32[$vararg_buffer2>>2] = $131;
   $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
   HEAP32[$vararg_ptr5>>2] = $132;
   $vararg_ptr6 = ((($vararg_buffer2)) + 8|0);
   HEAP32[$vararg_ptr6>>2] = $134;
   $135 = (_sqlite3MPrintf($130,29047,$vararg_buffer2)|0);
   HEAP32[$zName>>2] = $135;
   $136 = HEAP32[$cnt>>2]|0;
   $137 = ($136>>>0)>(3);
   if (!($137)) {
    continue;
   }
   _sqlite3_randomness(4,$cnt);
  }
  $138 = HEAP32[$pCol>>2]|0;
  HEAP32[$138>>2] = $103;
  $139 = HEAP32[$zName>>2]|0;
  $140 = ($139|0)!=(0|0);
  if ($140) {
   $141 = HEAP32[$zName>>2]|0;
   $142 = HEAP32[$pCol>>2]|0;
   $143 = (_sqlite3HashInsert($ht,$141,$142)|0);
   $144 = HEAP32[$pCol>>2]|0;
   $145 = ($143|0)==($144|0);
   if ($145) {
    $146 = HEAP32[$db>>2]|0;
    _sqlite3OomFault($146);
   }
  }
  $147 = HEAP32[$i>>2]|0;
  $148 = (($147) + 1)|0;
  HEAP32[$i>>2] = $148;
  $149 = HEAP32[$pCol>>2]|0;
  $150 = ((($149)) + 16|0);
  HEAP32[$pCol>>2] = $150;
 }
 _sqlite3HashClear($ht);
 $151 = HEAP32[$db>>2]|0;
 $152 = ((($151)) + 69|0);
 $153 = HEAP8[$152>>0]|0;
 $154 = ($153<<24>>24)!=(0);
 if (!($154)) {
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $155 = HEAP32[$j>>2]|0;
  $156 = HEAP32[$i>>2]|0;
  $157 = ($155|0)<($156|0);
  $158 = HEAP32[$db>>2]|0;
  if (!($157)) {
   break;
  }
  $159 = HEAP32[$j>>2]|0;
  $160 = HEAP32[$aCol>>2]|0;
  $161 = (($160) + ($159<<4)|0);
  $162 = HEAP32[$161>>2]|0;
  _sqlite3DbFree($158,$162);
  $163 = HEAP32[$j>>2]|0;
  $164 = (($163) + 1)|0;
  HEAP32[$j>>2] = $164;
 }
 $165 = HEAP32[$aCol>>2]|0;
 _sqlite3DbFree($158,$165);
 $166 = HEAP32[$4>>2]|0;
 HEAP32[$166>>2] = 0;
 $167 = HEAP32[$3>>2]|0;
 HEAP16[$167>>1] = 0;
 HEAP32[$0>>2] = 7;
 $168 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($168|0);
}
function _sqlite3SelectAddColumnTypeAndCollation($pParse,$pTab,$pSelect) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $db = 0, $i = 0, $m = 0, $n = 0, $p = 0, $pCol = 0, $pColl = 0, $sNC = 0;
 var $szAll = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $db = sp + 72|0;
 $sNC = sp + 40|0;
 $pCol = sp + 36|0;
 $pColl = sp + 32|0;
 $i = sp + 28|0;
 $p = sp + 24|0;
 $a = sp + 20|0;
 $szAll = sp;
 $zType = sp + 16|0;
 $n = sp + 12|0;
 $m = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pSelect;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = $szAll;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 69|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 if ($12) {
  STACKTOP = sp;return;
 }
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 28|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($sNC)) + 4|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$a>>2] = $20;
 HEAP32[$i>>2] = 0;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pCol>>2] = $23;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 34|0);
  $27 = HEAP16[$26>>1]|0;
  $28 = $27 << 16 >> 16;
  $29 = ($24|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$a>>2]|0;
  $32 = (($31) + (($30*20)|0)|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$p>>2] = $33;
  $34 = HEAP32[$p>>2]|0;
  $35 = HEAP32[$pCol>>2]|0;
  $36 = ((($35)) + 14|0);
  $37 = (_columnTypeImpl($sNC,$34,$36)|0);
  HEAP32[$zType>>2] = $37;
  $38 = HEAP32[$pCol>>2]|0;
  $39 = ((($38)) + 14|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $42 = $szAll;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = (_i64Add(($44|0),($47|0),($41|0),0)|0);
  $49 = tempRet0;
  $50 = $szAll;
  $51 = $50;
  HEAP32[$51>>2] = $48;
  $52 = (($50) + 4)|0;
  $53 = $52;
  HEAP32[$53>>2] = $49;
  $54 = HEAP32[$p>>2]|0;
  $55 = (_sqlite3ExprAffinity($54)|0);
  $56 = HEAP32[$pCol>>2]|0;
  $57 = ((($56)) + 13|0);
  HEAP8[$57>>0] = $55;
  $58 = HEAP32[$zType>>2]|0;
  $59 = ($58|0)!=(0|0);
  if ($59) {
   $60 = HEAP32[$zType>>2]|0;
   $61 = (_sqlite3Strlen30($60)|0);
   HEAP32[$m>>2] = $61;
   $62 = ($61|0)>(0);
   if ($62) {
    $63 = HEAP32[$pCol>>2]|0;
    $64 = HEAP32[$63>>2]|0;
    $65 = (_sqlite3Strlen30($64)|0);
    HEAP32[$n>>2] = $65;
    $66 = HEAP32[$db>>2]|0;
    $67 = HEAP32[$pCol>>2]|0;
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP32[$n>>2]|0;
    $70 = HEAP32[$m>>2]|0;
    $71 = (($69) + ($70))|0;
    $72 = (($71) + 2)|0;
    $73 = ($72|0)<(0);
    $74 = $73 << 31 >> 31;
    $75 = (_sqlite3DbReallocOrFree($66,$68,$72,$74)|0);
    $76 = HEAP32[$pCol>>2]|0;
    HEAP32[$76>>2] = $75;
    $77 = HEAP32[$pCol>>2]|0;
    $78 = HEAP32[$77>>2]|0;
    $79 = ($78|0)!=(0|0);
    if ($79) {
     $80 = HEAP32[$n>>2]|0;
     $81 = (($80) + 1)|0;
     $82 = HEAP32[$pCol>>2]|0;
     $83 = HEAP32[$82>>2]|0;
     $84 = (($83) + ($81)|0);
     $85 = HEAP32[$zType>>2]|0;
     $86 = HEAP32[$m>>2]|0;
     $87 = (($86) + 1)|0;
     _memcpy(($84|0),($85|0),($87|0))|0;
     $88 = HEAP32[$pCol>>2]|0;
     $89 = ((($88)) + 15|0);
     $90 = HEAP8[$89>>0]|0;
     $91 = $90&255;
     $92 = $91 | 4;
     $93 = $92&255;
     HEAP8[$89>>0] = $93;
    }
   }
  }
  $94 = HEAP32[$pCol>>2]|0;
  $95 = ((($94)) + 13|0);
  $96 = HEAP8[$95>>0]|0;
  $97 = $96 << 24 >> 24;
  $98 = ($97|0)==(0);
  if ($98) {
   $99 = HEAP32[$pCol>>2]|0;
   $100 = ((($99)) + 13|0);
   HEAP8[$100>>0] = 65;
  }
  $101 = HEAP32[$0>>2]|0;
  $102 = HEAP32[$p>>2]|0;
  $103 = (_sqlite3ExprCollSeq($101,$102)|0);
  HEAP32[$pColl>>2] = $103;
  $104 = HEAP32[$pColl>>2]|0;
  $105 = ($104|0)!=(0|0);
  if ($105) {
   $106 = HEAP32[$pCol>>2]|0;
   $107 = ((($106)) + 8|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = ($108|0)==(0|0);
   if ($109) {
    $110 = HEAP32[$db>>2]|0;
    $111 = HEAP32[$pColl>>2]|0;
    $112 = HEAP32[$111>>2]|0;
    $113 = (_sqlite3DbStrDup($110,$112)|0);
    $114 = HEAP32[$pCol>>2]|0;
    $115 = ((($114)) + 8|0);
    HEAP32[$115>>2] = $113;
   }
  }
  $116 = HEAP32[$i>>2]|0;
  $117 = (($116) + 1)|0;
  HEAP32[$i>>2] = $117;
  $118 = HEAP32[$pCol>>2]|0;
  $119 = ((($118)) + 16|0);
  HEAP32[$pCol>>2] = $119;
 }
 $120 = $szAll;
 $121 = $120;
 $122 = HEAP32[$121>>2]|0;
 $123 = (($120) + 4)|0;
 $124 = $123;
 $125 = HEAP32[$124>>2]|0;
 $126 = (___muldi3(($122|0),($125|0),4,0)|0);
 $127 = tempRet0;
 $128 = (_sqlite3LogEst($126,$127)|0);
 $129 = HEAP32[$1>>2]|0;
 $130 = ((($129)) + 40|0);
 HEAP16[$130>>1] = $128;
 STACKTOP = sp;return;
}
function _columnTypeImpl($pNC,$pExpr,$pEstWidth) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 $pEstWidth = $pEstWidth|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $estWidth = 0, $iCol = 0, $j = 0, $p = 0, $p3 = 0, $pS = 0, $pS2 = 0, $pTab = 0, $pTabList = 0, $sNC = 0, $sNC1 = 0;
 var $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $zType = sp + 100|0;
 $j = sp + 96|0;
 $estWidth = sp + 116|0;
 $pTab = sp + 92|0;
 $pS = sp + 88|0;
 $iCol = sp + 84|0;
 $pTabList = sp + 80|0;
 $sNC = sp + 48|0;
 $p = sp + 40|0;
 $sNC1 = sp + 8|0;
 $pS2 = sp + 4|0;
 $p3 = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pEstWidth;
 HEAP32[$zType>>2] = 0;
 HEAP8[$estWidth>>0] = 1;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 L1: do {
  switch ($5|0) {
  case 152: case 154:  {
   HEAP32[$pTab>>2] = 0;
   HEAP32[$pS>>2] = 0;
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 32|0);
   $8 = HEAP16[$7>>1]|0;
   $9 = $8 << 16 >> 16;
   HEAP32[$iCol>>2] = $9;
   while(1) {
    $10 = HEAP32[$0>>2]|0;
    $11 = ($10|0)!=(0|0);
    if (!($11)) {
     break;
    }
    $12 = HEAP32[$pTab>>2]|0;
    $13 = ($12|0)!=(0|0);
    $14 = $13 ^ 1;
    if (!($14)) {
     break;
    }
    $15 = HEAP32[$0>>2]|0;
    $16 = ((($15)) + 4|0);
    $17 = HEAP32[$16>>2]|0;
    HEAP32[$pTabList>>2] = $17;
    HEAP32[$j>>2] = 0;
    while(1) {
     $18 = HEAP32[$j>>2]|0;
     $19 = HEAP32[$pTabList>>2]|0;
     $20 = HEAP32[$19>>2]|0;
     $21 = ($18|0)<($20|0);
     if (!($21)) {
      break;
     }
     $22 = HEAP32[$j>>2]|0;
     $23 = HEAP32[$pTabList>>2]|0;
     $24 = ((($23)) + 8|0);
     $25 = (($24) + (($22*72)|0)|0);
     $26 = ((($25)) + 44|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 28|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($27|0)!=($30|0);
     if (!($31)) {
      break;
     }
     $32 = HEAP32[$j>>2]|0;
     $33 = (($32) + 1)|0;
     HEAP32[$j>>2] = $33;
    }
    $34 = HEAP32[$j>>2]|0;
    $35 = HEAP32[$pTabList>>2]|0;
    $36 = HEAP32[$35>>2]|0;
    $37 = ($34|0)<($36|0);
    if ($37) {
     $38 = HEAP32[$j>>2]|0;
     $39 = HEAP32[$pTabList>>2]|0;
     $40 = ((($39)) + 8|0);
     $41 = (($40) + (($38*72)|0)|0);
     $42 = ((($41)) + 16|0);
     $43 = HEAP32[$42>>2]|0;
     HEAP32[$pTab>>2] = $43;
     $44 = HEAP32[$j>>2]|0;
     $45 = HEAP32[$pTabList>>2]|0;
     $46 = ((($45)) + 8|0);
     $47 = (($46) + (($44*72)|0)|0);
     $48 = ((($47)) + 20|0);
     $49 = HEAP32[$48>>2]|0;
     HEAP32[$pS>>2] = $49;
     continue;
    } else {
     $50 = HEAP32[$0>>2]|0;
     $51 = ((($50)) + 16|0);
     $52 = HEAP32[$51>>2]|0;
     HEAP32[$0>>2] = $52;
     continue;
    }
   }
   $53 = HEAP32[$pTab>>2]|0;
   $54 = ($53|0)==(0|0);
   if (!($54)) {
    $55 = HEAP32[$pS>>2]|0;
    $56 = ($55|0)!=(0|0);
    if ($56) {
     $57 = HEAP32[$iCol>>2]|0;
     $58 = ($57|0)>=(0);
     if (!($58)) {
      break L1;
     }
     $59 = HEAP32[$iCol>>2]|0;
     $60 = HEAP32[$pS>>2]|0;
     $61 = HEAP32[$60>>2]|0;
     $62 = HEAP32[$61>>2]|0;
     $63 = ($59|0)<($62|0);
     if (!($63)) {
      break L1;
     }
     $64 = HEAP32[$iCol>>2]|0;
     $65 = HEAP32[$pS>>2]|0;
     $66 = HEAP32[$65>>2]|0;
     $67 = ((($66)) + 4|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + (($64*20)|0)|0);
     $70 = HEAP32[$69>>2]|0;
     HEAP32[$p>>2] = $70;
     $71 = HEAP32[$pS>>2]|0;
     $72 = ((($71)) + 28|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = ((($sNC)) + 4|0);
     HEAP32[$74>>2] = $73;
     $75 = HEAP32[$0>>2]|0;
     $76 = ((($sNC)) + 16|0);
     HEAP32[$76>>2] = $75;
     $77 = HEAP32[$0>>2]|0;
     $78 = HEAP32[$77>>2]|0;
     HEAP32[$sNC>>2] = $78;
     $79 = HEAP32[$p>>2]|0;
     $80 = (_columnTypeImpl($sNC,$79,$estWidth)|0);
     HEAP32[$zType>>2] = $80;
     break L1;
    }
    $81 = HEAP32[$pTab>>2]|0;
    $82 = ((($81)) + 64|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ($83|0)!=(0|0);
    if ($84) {
     $85 = HEAP32[$iCol>>2]|0;
     $86 = ($85|0)<(0);
     if ($86) {
      $87 = HEAP32[$pTab>>2]|0;
      $88 = ((($87)) + 32|0);
      $89 = HEAP16[$88>>1]|0;
      $90 = $89 << 16 >> 16;
      HEAP32[$iCol>>2] = $90;
     }
     $91 = HEAP32[$iCol>>2]|0;
     $92 = ($91|0)<(0);
     if ($92) {
      HEAP32[$zType>>2] = 18759;
      break L1;
     } else {
      $93 = HEAP32[$iCol>>2]|0;
      $94 = HEAP32[$pTab>>2]|0;
      $95 = ((($94)) + 4|0);
      $96 = HEAP32[$95>>2]|0;
      $97 = (($96) + ($93<<4)|0);
      $98 = (_sqlite3ColumnType($97,0)|0);
      HEAP32[$zType>>2] = $98;
      $99 = HEAP32[$iCol>>2]|0;
      $100 = HEAP32[$pTab>>2]|0;
      $101 = ((($100)) + 4|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = (($102) + ($99<<4)|0);
      $104 = ((($103)) + 14|0);
      $105 = HEAP8[$104>>0]|0;
      HEAP8[$estWidth>>0] = $105;
      break L1;
     }
    }
   }
   break;
  }
  case 119:  {
   $106 = HEAP32[$1>>2]|0;
   $107 = ((($106)) + 20|0);
   $108 = HEAP32[$107>>2]|0;
   HEAP32[$pS2>>2] = $108;
   $109 = HEAP32[$pS2>>2]|0;
   $110 = HEAP32[$109>>2]|0;
   $111 = ((($110)) + 4|0);
   $112 = HEAP32[$111>>2]|0;
   $113 = HEAP32[$112>>2]|0;
   HEAP32[$p3>>2] = $113;
   $114 = HEAP32[$pS2>>2]|0;
   $115 = ((($114)) + 28|0);
   $116 = HEAP32[$115>>2]|0;
   $117 = ((($sNC1)) + 4|0);
   HEAP32[$117>>2] = $116;
   $118 = HEAP32[$0>>2]|0;
   $119 = ((($sNC1)) + 16|0);
   HEAP32[$119>>2] = $118;
   $120 = HEAP32[$0>>2]|0;
   $121 = HEAP32[$120>>2]|0;
   HEAP32[$sNC1>>2] = $121;
   $122 = HEAP32[$p3>>2]|0;
   $123 = (_columnTypeImpl($sNC1,$122,$estWidth)|0);
   HEAP32[$zType>>2] = $123;
   break;
  }
  default: {
  }
  }
 } while(0);
 $124 = HEAP32[$2>>2]|0;
 $125 = ($124|0)!=(0|0);
 if (!($125)) {
  $128 = HEAP32[$zType>>2]|0;
  STACKTOP = sp;return ($128|0);
 }
 $126 = HEAP8[$estWidth>>0]|0;
 $127 = HEAP32[$2>>2]|0;
 HEAP8[$127>>0] = $126;
 $128 = HEAP32[$zType>>2]|0;
 STACKTOP = sp;return ($128|0);
}
function _sqlite3ExprAffinity($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $op = sp;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3ExprSkipCollate($2)|0);
 HEAP32[$1>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 512;
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP8[$0>>0] = 0;
  $65 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($65|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$op>>2] = $11;
 $12 = HEAP32[$op>>2]|0;
 $13 = ($12|0)==(119);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3ExprAffinity($20)|0);
  HEAP8[$0>>0] = $21;
  $65 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($65|0);
 }
 $22 = HEAP32[$op>>2]|0;
 $23 = ($22|0)==(157);
 if ($23) {
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 38|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  HEAP32[$op>>2] = $27;
 }
 $28 = HEAP32[$op>>2]|0;
 $29 = ($28|0)==(66);
 if ($29) {
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (_sqlite3AffinityType($32,0)|0);
  HEAP8[$0>>0] = $33;
  $65 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($65|0);
 }
 $34 = HEAP32[$op>>2]|0;
 $35 = ($34|0)==(154);
 $36 = HEAP32[$op>>2]|0;
 $37 = ($36|0)==(152);
 $or$cond = $35 | $37;
 if ($or$cond) {
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 44|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$1>>2]|0;
  $42 = ((($41)) + 32|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43 << 16 >> 16;
  $45 = (_sqlite3TableColumnAffinity($40,$44)|0);
  HEAP8[$0>>0] = $45;
  $65 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($65|0);
 }
 $46 = HEAP32[$op>>2]|0;
 $47 = ($46|0)==(159);
 $48 = HEAP32[$1>>2]|0;
 if ($47) {
  $49 = ((($48)) + 32|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = $50 << 16 >> 16;
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ((($54)) + 20|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$56>>2]|0;
  $58 = ((($57)) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (($59) + (($51*20)|0)|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (_sqlite3ExprAffinity($61)|0);
  HEAP8[$0>>0] = $62;
  $65 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($65|0);
 } else {
  $63 = ((($48)) + 1|0);
  $64 = HEAP8[$63>>0]|0;
  HEAP8[$0>>0] = $64;
  $65 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($65|0);
 }
 return (0)|0;
}
function _sqlite3ExprCollSeq($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $op = 0;
 var $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $p = 0, $pColl = 0, $pNext = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $db = sp + 28|0;
 $pColl = sp + 24|0;
 $p = sp + 20|0;
 $op = sp + 16|0;
 $j = sp + 12|0;
 $zColl = sp + 8|0;
 $pNext = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$pColl>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$p>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  HEAP32[$op>>2] = $9;
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $12 & 512;
  $14 = ($13|0)!=(0);
  if ($14) {
   break;
  }
  $15 = HEAP32[$op>>2]|0;
  $16 = ($15|0)==(66);
  $17 = HEAP32[$op>>2]|0;
  $18 = ($17|0)==(156);
  $or$cond = $16 | $18;
  if ($or$cond) {
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 12|0);
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$p>>2] = $21;
   continue;
  }
  $22 = HEAP32[$op>>2]|0;
  $23 = ($22|0)==(53);
  if ($23) {
   label = 9;
   break;
  }
  $24 = HEAP32[$op>>2]|0;
  $25 = ($24|0)==(157);
  if ($25) {
   $26 = HEAP32[$p>>2]|0;
   $27 = ((($26)) + 38|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = ($29|0)==(53);
   if ($30) {
    label = 9;
    break;
   }
  }
  $39 = HEAP32[$op>>2]|0;
  $40 = ($39|0)==(154);
  $41 = HEAP32[$op>>2]|0;
  $42 = ($41|0)==(152);
  $or$cond3 = $40 | $42;
  $43 = HEAP32[$op>>2]|0;
  $44 = ($43|0)==(157);
  $or$cond5 = $or$cond3 | $44;
  $45 = HEAP32[$op>>2]|0;
  $46 = ($45|0)==(88);
  $or$cond7 = $or$cond5 | $46;
  if ($or$cond7) {
   $47 = HEAP32[$p>>2]|0;
   $48 = ((($47)) + 44|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ($49|0)!=(0|0);
   if ($50) {
    label = 12;
    break;
   }
  }
  $72 = HEAP32[$p>>2]|0;
  $73 = ((($72)) + 4|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $74 & 256;
  $76 = ($75|0)!=(0);
  if (!($76)) {
   break;
  }
  $77 = HEAP32[$p>>2]|0;
  $78 = ((($77)) + 12|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0|0);
  if ($80) {
   $81 = HEAP32[$p>>2]|0;
   $82 = ((($81)) + 12|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = ((($83)) + 4|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = $85 & 256;
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = HEAP32[$p>>2]|0;
    $89 = ((($88)) + 12|0);
    $90 = HEAP32[$89>>2]|0;
    HEAP32[$p>>2] = $90;
    continue;
   }
  }
  $91 = HEAP32[$p>>2]|0;
  $92 = ((($91)) + 16|0);
  $93 = HEAP32[$92>>2]|0;
  HEAP32[$pNext>>2] = $93;
  $94 = HEAP32[$p>>2]|0;
  $95 = ((($94)) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)!=(0|0);
  L20: do {
   if ($97) {
    $98 = HEAP32[$p>>2]|0;
    $99 = ((($98)) + 4|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = $100 & 2048;
    $102 = ($101|0)!=(0);
    if (!($102)) {
     HEAP32[$i>>2] = 0;
     while(1) {
      $103 = HEAP32[$i>>2]|0;
      $104 = HEAP32[$p>>2]|0;
      $105 = ((($104)) + 20|0);
      $106 = HEAP32[$105>>2]|0;
      $107 = HEAP32[$106>>2]|0;
      $108 = ($103|0)<($107|0);
      if (!($108)) {
       break L20;
      }
      $109 = HEAP32[$i>>2]|0;
      $110 = HEAP32[$p>>2]|0;
      $111 = ((($110)) + 20|0);
      $112 = HEAP32[$111>>2]|0;
      $113 = ((($112)) + 4|0);
      $114 = HEAP32[$113>>2]|0;
      $115 = (($114) + (($109*20)|0)|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ((($116)) + 4|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = $118 & 256;
      $120 = ($119|0)!=(0);
      $121 = HEAP32[$i>>2]|0;
      if ($120) {
       break;
      }
      $129 = (($121) + 1)|0;
      HEAP32[$i>>2] = $129;
     }
     $122 = HEAP32[$p>>2]|0;
     $123 = ((($122)) + 20|0);
     $124 = HEAP32[$123>>2]|0;
     $125 = ((($124)) + 4|0);
     $126 = HEAP32[$125>>2]|0;
     $127 = (($126) + (($121*20)|0)|0);
     $128 = HEAP32[$127>>2]|0;
     HEAP32[$pNext>>2] = $128;
    }
   }
  } while(0);
  $130 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $130;
 }
 if ((label|0) == 9) {
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$db>>2]|0;
  $33 = ((($32)) + 66|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 8|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3GetCollSeq($31,$34,0,$37)|0);
  HEAP32[$pColl>>2] = $38;
 }
 else if ((label|0) == 12) {
  $51 = HEAP32[$p>>2]|0;
  $52 = ((($51)) + 32|0);
  $53 = HEAP16[$52>>1]|0;
  $54 = $53 << 16 >> 16;
  HEAP32[$j>>2] = $54;
  $55 = HEAP32[$j>>2]|0;
  $56 = ($55|0)>=(0);
  if ($56) {
   $57 = HEAP32[$j>>2]|0;
   $58 = HEAP32[$p>>2]|0;
   $59 = ((($58)) + 44|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = ((($60)) + 4|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = (($62) + ($57<<4)|0);
   $64 = ((($63)) + 8|0);
   $65 = HEAP32[$64>>2]|0;
   HEAP32[$zColl>>2] = $65;
   $66 = HEAP32[$db>>2]|0;
   $67 = HEAP32[$db>>2]|0;
   $68 = ((($67)) + 66|0);
   $69 = HEAP8[$68>>0]|0;
   $70 = HEAP32[$zColl>>2]|0;
   $71 = (_sqlite3FindCollSeq($66,$69,$70,0)|0);
   HEAP32[$pColl>>2] = $71;
  }
 }
 $131 = HEAP32[$0>>2]|0;
 $132 = HEAP32[$pColl>>2]|0;
 $133 = (_sqlite3CheckCollSeq($131,$132)|0);
 $134 = ($133|0)!=(0);
 if (!($134)) {
  $135 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 HEAP32[$pColl>>2] = 0;
 $135 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _sqlite3GetCollSeq($pParse,$enc,$pColl,$zName) {
 $pParse = $pParse|0;
 $enc = $enc|0;
 $pColl = $pColl|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $p = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 24|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $p = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP8[$1>>0] = $enc;
 HEAP32[$2>>2] = $pColl;
 HEAP32[$3>>2] = $zName;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  $9 = HEAP32[$db>>2]|0;
  $10 = HEAP8[$1>>0]|0;
  $11 = HEAP32[$3>>2]|0;
  $12 = (_sqlite3FindCollSeq($9,$10,$11,0)|0);
  HEAP32[$p>>2] = $12;
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ($17|0)!=(0|0);
  if (!($18)) {
   label = 5;
  }
 } else {
  label = 5;
 }
 if ((label|0) == 5) {
  $19 = HEAP32[$db>>2]|0;
  $20 = HEAP8[$1>>0]|0;
  $21 = $20&255;
  $22 = HEAP32[$3>>2]|0;
  _callCollNeeded($19,$21,$22);
  $23 = HEAP32[$db>>2]|0;
  $24 = HEAP8[$1>>0]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = (_sqlite3FindCollSeq($23,$24,$25,0)|0);
  HEAP32[$p>>2] = $26;
 }
 $27 = HEAP32[$p>>2]|0;
 $28 = ($27|0)!=(0|0);
 if ($28) {
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   $33 = HEAP32[$db>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = (_synthCollSeq($33,$34)|0);
   $36 = ($35|0)!=(0);
   if ($36) {
    HEAP32[$p>>2] = 0;
   }
  }
 }
 $37 = HEAP32[$p>>2]|0;
 $38 = ($37|0)==(0|0);
 if (!($38)) {
  $41 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $40;
 _sqlite3ErrorMsg($39,29013,$vararg_buffer);
 $41 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3CheckCollSeq($pParse,$pColl) {
 $pParse = $pParse|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $p = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zName = sp + 8|0;
 $db = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pColl;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$zName>>2] = $6;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$db>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$db>>2]|0;
  $11 = ((($10)) + 66|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$zName>>2]|0;
  $15 = (_sqlite3GetCollSeq($9,$12,$13,$14)|0);
  HEAP32[$p>>2] = $15;
  $16 = HEAP32[$p>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   HEAP32[$0>>2] = 1;
   $18 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($18|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $18 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _callCollNeeded($db,$enc,$zName) {
 $db = $db|0;
 $enc = $enc|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, $zExternal = 0, $zExternal1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zExternal = sp + 8|0;
 $zExternal1 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $enc;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 232|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = HEAP32[$2>>2]|0;
   $9 = (_sqlite3DbStrDup($7,$8)|0);
   HEAP32[$zExternal>>2] = $9;
   $10 = HEAP32[$zExternal>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$0>>2]|0;
    $13 = ((($12)) + 232|0);
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$0>>2]|0;
    $16 = ((($15)) + 240|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$0>>2]|0;
    $19 = HEAP32[$1>>2]|0;
    $20 = HEAP32[$zExternal>>2]|0;
    FUNCTION_TABLE_viiii[$14 & 255]($17,$18,$19,$20);
    $21 = HEAP32[$0>>2]|0;
    $22 = HEAP32[$zExternal>>2]|0;
    _sqlite3DbFree($21,$22);
    break;
   } else {
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 236|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (_sqlite3ValueNew($27)|0);
 HEAP32[$pTmp>>2] = $28;
 $29 = HEAP32[$pTmp>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 _sqlite3ValueSetStr($29,-1,$30,1,0);
 $31 = HEAP32[$pTmp>>2]|0;
 $32 = HEAP8[968>>0]|0;
 $33 = $32 << 24 >> 24;
 $34 = ($33|0)==(0);
 $35 = $34 ? 3 : 2;
 $36 = $35&255;
 $37 = (_sqlite3ValueText($31,$36)|0);
 HEAP32[$zExternal1>>2] = $37;
 $38 = HEAP32[$zExternal1>>2]|0;
 $39 = ($38|0)!=(0|0);
 if ($39) {
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 236|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 240|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 66|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = HEAP32[$zExternal1>>2]|0;
  FUNCTION_TABLE_viiii[$42 & 255]($45,$46,$50,$51);
 }
 $52 = HEAP32[$pTmp>>2]|0;
 _sqlite3ValueFree($52);
 STACKTOP = sp;return;
}
function _synthCollSeq($db,$pColl) {
 $db = $db|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pColl2 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pColl2 = sp + 8|0;
 $z = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pColl;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$z>>2] = $4;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = ($5|0)<(3);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$i>>2]|0;
  $9 = (29044 + ($8)|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = HEAP32[$z>>2]|0;
  $12 = (_sqlite3FindCollSeq($7,$10,$11,0)|0);
  HEAP32[$pColl2>>2] = $12;
  $13 = HEAP32[$pColl2>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   label = 4;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 if ((label|0) == 4) {
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$pColl2>>2]|0;
  ;HEAP32[$17>>2]=HEAP32[$18>>2]|0;HEAP32[$17+4>>2]=HEAP32[$18+4>>2]|0;HEAP32[$17+8>>2]=HEAP32[$18+8>>2]|0;HEAP32[$17+12>>2]=HEAP32[$18+12>>2]|0;HEAP32[$17+16>>2]=HEAP32[$18+16>>2]|0;
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 16|0);
  HEAP32[$20>>2] = 0;
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _sqlite3ExprSkipCollate($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pExpr;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if ($2) {
   $3 = HEAP32[$0>>2]|0;
   $4 = ((($3)) + 4|0);
   $5 = HEAP32[$4>>2]|0;
   $6 = $5 & 4096;
   $7 = ($6|0)!=(0);
   $21 = $7;
  } else {
   $21 = 0;
  }
  $8 = HEAP32[$0>>2]|0;
  if (!($21)) {
   break;
  }
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 & 262144;
  $12 = ($11|0)!=(0);
  $13 = HEAP32[$0>>2]|0;
  if ($12) {
   $14 = ((($13)) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$0>>2] = $18;
   continue;
  } else {
   $19 = ((($13)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   HEAP32[$0>>2] = $20;
   continue;
  }
 }
 STACKTOP = sp;return ($8|0);
}
function _sqlite3TableColumnAffinity($pTab,$iCol) {
 $pTab = $pTab|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$1>>2] = $iCol;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 if (!($3)) {
  $13 = 68;
  $12 = $13&255;
  STACKTOP = sp;return ($12|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 $9 = ((($8)) + 13|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10 << 24 >> 24;
 $13 = $11;
 $12 = $13&255;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3SelectPrep($pParse,$p,$pOuterNC) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pOuterNC = $pOuterNC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOuterNC;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 69|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 128;
 $15 = ($14|0)!=(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 _sqlite3SelectExpand($16,$17);
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 36|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$db>>2]|0;
 $23 = ((($22)) + 69|0);
 $24 = HEAP8[$23>>0]|0;
 $25 = $24&255;
 $26 = ($25|0)!=(0);
 if ($26) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 _sqlite3ResolveSelectNames($27,$28,$29);
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 36|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$db>>2]|0;
 $35 = ((($34)) + 69|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = ($37|0)!=(0);
 if ($38) {
  STACKTOP = sp;return;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 _sqlite3SelectAddTypeInfo($39,$40);
 STACKTOP = sp;return;
}
function _sqlite3SelectExpand($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSelect;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 4|0);
 HEAP32[$2>>2] = 190;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$w>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 22|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = ((($w)) + 8|0);
  HEAP32[$8>>2] = 191;
  $9 = HEAP32[$1>>2]|0;
  (_sqlite3WalkSelect($w,$9)|0);
 }
 $10 = ((($w)) + 8|0);
 HEAP32[$10>>2] = 192;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 1024;
 $15 = ($14|0)==(0);
 if (!($15)) {
  $17 = HEAP32[$1>>2]|0;
  (_sqlite3WalkSelect($w,$17)|0);
  STACKTOP = sp;return;
 }
 $16 = ((($w)) + 12|0);
 HEAP32[$16>>2] = 132;
 $17 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$17)|0);
 STACKTOP = sp;return;
}
function _sqlite3ResolveSelectNames($pParse,$p,$pOuterNC) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pOuterNC = $pOuterNC|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOuterNC;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $3 = ((($w)) + 4|0);
 HEAP32[$3>>2] = 193;
 $4 = ((($w)) + 8|0);
 HEAP32[$4>>2] = 194;
 $5 = HEAP32[$0>>2]|0;
 HEAP32[$w>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($w)) + 24|0);
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$8)|0);
 STACKTOP = sp;return;
}
function _sqlite3SelectAddTypeInfo($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSelect;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 12|0);
 HEAP32[$2>>2] = 133;
 $3 = ((($w)) + 4|0);
 HEAP32[$3>>2] = 190;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$w>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 (_sqlite3WalkSelect($w,$5)|0);
 STACKTOP = sp;return;
}
function _selectAddSubqueryTypeInfo($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pFrom = 0, $pParse = 0, $pSel = 0, $pTab = 0, $pTabList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pParse = sp + 20|0;
 $i = sp + 16|0;
 $pTabList = sp + 12|0;
 $pFrom = sp + 8|0;
 $pTab = sp + 4|0;
 $pSel = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 | 128;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pParse>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pTabList>>2] = $10;
 HEAP32[$i>>2] = 0;
 $11 = HEAP32[$pTabList>>2]|0;
 $12 = ((($11)) + 8|0);
 HEAP32[$pFrom>>2] = $12;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pTabList>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = ($13|0)<($15|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pFrom>>2]|0;
  $18 = ((($17)) + 16|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pTab>>2] = $19;
  $20 = HEAP32[$pTab>>2]|0;
  $21 = ((($20)) + 42|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = $23 & 2;
  $25 = ($24|0)!=(0);
  if ($25) {
   $26 = HEAP32[$pFrom>>2]|0;
   $27 = ((($26)) + 20|0);
   $28 = HEAP32[$27>>2]|0;
   HEAP32[$pSel>>2] = $28;
   $29 = HEAP32[$pSel>>2]|0;
   $30 = ($29|0)!=(0|0);
   if ($30) {
    while(1) {
     $31 = HEAP32[$pSel>>2]|0;
     $32 = ((($31)) + 48|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($33|0)!=(0|0);
     if (!($34)) {
      break;
     }
     $35 = HEAP32[$pSel>>2]|0;
     $36 = ((($35)) + 48|0);
     $37 = HEAP32[$36>>2]|0;
     HEAP32[$pSel>>2] = $37;
    }
    $38 = HEAP32[$pParse>>2]|0;
    $39 = HEAP32[$pTab>>2]|0;
    $40 = HEAP32[$pSel>>2]|0;
    _sqlite3SelectAddColumnTypeAndCollation($38,$39,$40);
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
  $43 = HEAP32[$pFrom>>2]|0;
  $44 = ((($43)) + 72|0);
  HEAP32[$pFrom>>2] = $44;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprWalkNoop($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _sqlite3WalkSelect($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 do {
  if (!($4)) {
   $5 = HEAP32[$1>>2]|0;
   $6 = ((($5)) + 8|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    $9 = HEAP32[$1>>2]|0;
    $10 = ((($9)) + 12|0);
    $11 = HEAP32[$10>>2]|0;
    $12 = ($11|0)==(0|0);
    if ($12) {
     break;
    }
   }
   HEAP32[$rc>>2] = 0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (($15) + 1)|0;
   HEAP32[$14>>2] = $16;
   while(1) {
    $17 = HEAP32[$2>>2]|0;
    $18 = ($17|0)!=(0|0);
    if (!($18)) {
     label = 15;
     break;
    }
    $19 = HEAP32[$1>>2]|0;
    $20 = ((($19)) + 8|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($21|0)!=(0|0);
    if ($22) {
     $23 = HEAP32[$1>>2]|0;
     $24 = ((($23)) + 8|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = HEAP32[$1>>2]|0;
     $27 = HEAP32[$2>>2]|0;
     $28 = (FUNCTION_TABLE_iii[$25 & 255]($26,$27)|0);
     HEAP32[$rc>>2] = $28;
     $29 = HEAP32[$rc>>2]|0;
     $30 = ($29|0)!=(0);
     if ($30) {
      label = 15;
      break;
     }
    }
    $31 = HEAP32[$1>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = (_sqlite3WalkSelectExpr($31,$32)|0);
    $34 = ($33|0)!=(0);
    if ($34) {
     label = 11;
     break;
    }
    $35 = HEAP32[$1>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    $37 = (_sqlite3WalkSelectFrom($35,$36)|0);
    $38 = ($37|0)!=(0);
    if ($38) {
     label = 11;
     break;
    }
    $43 = HEAP32[$1>>2]|0;
    $44 = ((($43)) + 12|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$1>>2]|0;
     $48 = ((($47)) + 12|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = HEAP32[$1>>2]|0;
     $51 = HEAP32[$2>>2]|0;
     FUNCTION_TABLE_vii[$49 & 255]($50,$51);
    }
    $52 = HEAP32[$2>>2]|0;
    $53 = ((($52)) + 48|0);
    $54 = HEAP32[$53>>2]|0;
    HEAP32[$2>>2] = $54;
   }
   if ((label|0) == 11) {
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 16|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = (($41) + -1)|0;
    HEAP32[$40>>2] = $42;
    HEAP32[$0>>2] = 2;
    $61 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($61|0);
   }
   else if ((label|0) == 15) {
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 16|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + -1)|0;
    HEAP32[$56>>2] = $58;
    $59 = HEAP32[$rc>>2]|0;
    $60 = $59 & 2;
    HEAP32[$0>>2] = $60;
    $61 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($61|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _sqlite3WalkSelectExpr($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3WalkExprList($3,$5)|0);
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   HEAP32[$0>>2] = 2;
  } else {
   $8 = HEAP32[$1>>2]|0;
   $9 = HEAP32[$2>>2]|0;
   $10 = ((($9)) + 32|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = (_sqlite3WalkExpr($8,$11)|0);
   $13 = ($12|0)!=(0);
   if ($13) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 36|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3WalkExprList($14,$17)|0);
   $19 = ($18|0)!=(0);
   if ($19) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = ((($21)) + 40|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (_sqlite3WalkExpr($20,$23)|0);
   $25 = ($24|0)!=(0);
   if ($25) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $26 = HEAP32[$1>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = ((($27)) + 44|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (_sqlite3WalkExprList($26,$29)|0);
   $31 = ($30|0)!=(0);
   if ($31) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $32 = HEAP32[$1>>2]|0;
   $33 = HEAP32[$2>>2]|0;
   $34 = ((($33)) + 56|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3WalkExpr($32,$35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    HEAP32[$0>>2] = 2;
    break;
   }
   $38 = HEAP32[$1>>2]|0;
   $39 = HEAP32[$2>>2]|0;
   $40 = ((($39)) + 60|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = (_sqlite3WalkExpr($38,$41)|0);
   $43 = ($42|0)!=(0);
   if ($43) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3WalkSelectFrom($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $pItem = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSrc = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = ($6|0)!=(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$pSrc>>2]|0;
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$i>>2] = $9;
   $10 = HEAP32[$pSrc>>2]|0;
   $11 = ((($10)) + 8|0);
   HEAP32[$pItem>>2] = $11;
   while(1) {
    $12 = HEAP32[$i>>2]|0;
    $13 = ($12|0)>(0);
    if (!($13)) {
     break L1;
    }
    $14 = HEAP32[$1>>2]|0;
    $15 = HEAP32[$pItem>>2]|0;
    $16 = ((($15)) + 20|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = (_sqlite3WalkSelect($14,$17)|0);
    $19 = ($18|0)!=(0);
    if ($19) {
     label = 5;
     break;
    }
    $20 = HEAP32[$pItem>>2]|0;
    $21 = ((($20)) + 36|0);
    $22 = ((($21)) + 1|0);
    $23 = HEAP8[$22>>0]|0;
    $24 = ($23&255) >>> 2;
    $25 = $24 & 1;
    $26 = $25&255;
    $27 = ($26|0)!=(0);
    if ($27) {
     $28 = HEAP32[$1>>2]|0;
     $29 = HEAP32[$pItem>>2]|0;
     $30 = ((($29)) + 64|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = (_sqlite3WalkExprList($28,$31)|0);
     $33 = ($32|0)!=(0);
     if ($33) {
      label = 8;
      break;
     }
    }
    $34 = HEAP32[$i>>2]|0;
    $35 = (($34) + -1)|0;
    HEAP32[$i>>2] = $35;
    $36 = HEAP32[$pItem>>2]|0;
    $37 = ((($36)) + 72|0);
    HEAP32[$pItem>>2] = $37;
   }
   if ((label|0) == 5) {
    HEAP32[$0>>2] = 2;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
   else if ((label|0) == 8) {
    HEAP32[$0>>2] = 2;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3WalkExprList($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   $5 = HEAP32[$2>>2]|0;
   $6 = HEAP32[$5>>2]|0;
   HEAP32[$i>>2] = $6;
   $7 = HEAP32[$2>>2]|0;
   $8 = ((($7)) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   HEAP32[$pItem>>2] = $9;
   while(1) {
    $10 = HEAP32[$i>>2]|0;
    $11 = ($10|0)>(0);
    if (!($11)) {
     break L1;
    }
    $12 = HEAP32[$1>>2]|0;
    $13 = HEAP32[$pItem>>2]|0;
    $14 = HEAP32[$13>>2]|0;
    $15 = (_sqlite3WalkExpr($12,$14)|0);
    $16 = ($15|0)!=(0);
    if ($16) {
     break;
    }
    $17 = HEAP32[$i>>2]|0;
    $18 = (($17) + -1)|0;
    HEAP32[$i>>2] = $18;
    $19 = HEAP32[$pItem>>2]|0;
    $20 = ((($19)) + 20|0);
    HEAP32[$pItem>>2] = $20;
   }
   HEAP32[$0>>2] = 2;
   $21 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($21|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3WalkExpr($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $7 = 0;
  STACKTOP = sp;return ($7|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_walkExpr($4,$5)|0);
 $7 = $6;
 STACKTOP = sp;return ($7|0);
}
function _walkExpr($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $13 & 8404992;
  $15 = ($14|0)!=(0);
  if (!($15)) {
   $18 = HEAP32[$2>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$2>>2]|0;
    $24 = ((($23)) + 12|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (_walkExpr($22,$25)|0);
    $27 = ($26|0)!=(0);
    if ($27) {
     HEAP32[$0>>2] = 2;
     $59 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($59|0);
    }
   }
   $28 = HEAP32[$2>>2]|0;
   $29 = ((($28)) + 16|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = ((($33)) + 16|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = (_walkExpr($32,$35)|0);
    $37 = ($36|0)!=(0);
    if ($37) {
     HEAP32[$0>>2] = 2;
     $59 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($59|0);
    }
   }
   $38 = HEAP32[$2>>2]|0;
   $39 = ((($38)) + 4|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = $40 & 2048;
   $42 = ($41|0)!=(0);
   if ($42) {
    $43 = HEAP32[$1>>2]|0;
    $44 = HEAP32[$2>>2]|0;
    $45 = ((($44)) + 20|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = (_sqlite3WalkSelect($43,$46)|0);
    $48 = ($47|0)!=(0);
    if ($48) {
     HEAP32[$0>>2] = 2;
     $59 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($59|0);
    }
   } else {
    $49 = HEAP32[$2>>2]|0;
    $50 = ((($49)) + 20|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = ($51|0)!=(0|0);
    if ($52) {
     $53 = HEAP32[$1>>2]|0;
     $54 = HEAP32[$2>>2]|0;
     $55 = ((($54)) + 20|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = (_sqlite3WalkExprList($53,$56)|0);
     $58 = ($57|0)!=(0);
     if ($58) {
      HEAP32[$0>>2] = 2;
      $59 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($59|0);
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $59 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($59|0);
  }
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = $16 & 2;
 HEAP32[$0>>2] = $17;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _resolveExprStep($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $auth = 0, $enc = 0, $is_agg = 0, $n = 0, $nId = 0, $nLeft = 0, $nRef = 0, $nRight = 0, $no_such_func = 0, $pDef = 0;
 var $pList = 0, $pNC = 0, $pNC2 = 0, $pParse = 0, $pRight = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer15 = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr6 = 0, $wrong_num_args = 0, $zColumn = 0, $zDb = 0, $zId = 0, $zTable = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer15 = sp + 40|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $pNC = sp + 116|0;
 $pParse = sp + 112|0;
 $zColumn = sp + 108|0;
 $zTable = sp + 104|0;
 $zDb = sp + 100|0;
 $pRight = sp + 96|0;
 $pList = sp + 92|0;
 $n = sp + 88|0;
 $no_such_func = sp + 84|0;
 $wrong_num_args = sp + 80|0;
 $is_agg = sp + 76|0;
 $nId = sp + 72|0;
 $zId = sp + 68|0;
 $pDef = sp + 64|0;
 $enc = sp + 132|0;
 $auth = sp + 60|0;
 $pNC2 = sp + 56|0;
 $nRef = sp + 52|0;
 $nLeft = sp + 48|0;
 $nRight = sp + 44|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pNC>>2] = $5;
 $6 = HEAP32[$pNC>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pParse>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 & 4;
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$0>>2] = 1;
  $329 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($329|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 | 4;
 HEAP32[$14>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 switch ($19|0) {
 case 55:  {
  $20 = HEAP32[$pParse>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = ((($21)) + 8|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$pNC>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = (_lookupName($20,0,0,$23,$24,$25)|0);
  HEAP32[$0>>2] = $26;
  $329 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($329|0);
  break;
 }
 case 122:  {
  $27 = HEAP32[$pParse>>2]|0;
  $28 = HEAP32[$pNC>>2]|0;
  _notValid($27,$28,29561,32);
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pRight>>2] = $31;
  $32 = HEAP32[$pRight>>2]|0;
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  $35 = ($34|0)==(55);
  if ($35) {
   HEAP32[$zDb>>2] = 0;
   $36 = HEAP32[$2>>2]|0;
   $37 = ((($36)) + 12|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ((($38)) + 8|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$zTable>>2] = $40;
   $41 = HEAP32[$pRight>>2]|0;
   $42 = ((($41)) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   HEAP32[$zColumn>>2] = $43;
  } else {
   $44 = HEAP32[$2>>2]|0;
   $45 = ((($44)) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   HEAP32[$zDb>>2] = $48;
   $49 = HEAP32[$pRight>>2]|0;
   $50 = ((($49)) + 12|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ((($51)) + 8|0);
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$zTable>>2] = $53;
   $54 = HEAP32[$pRight>>2]|0;
   $55 = ((($54)) + 16|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = ((($56)) + 8|0);
   $58 = HEAP32[$57>>2]|0;
   HEAP32[$zColumn>>2] = $58;
  }
  $59 = HEAP32[$pParse>>2]|0;
  $60 = HEAP32[$zDb>>2]|0;
  $61 = HEAP32[$zTable>>2]|0;
  $62 = HEAP32[$zColumn>>2]|0;
  $63 = HEAP32[$pNC>>2]|0;
  $64 = HEAP32[$2>>2]|0;
  $65 = (_lookupName($59,$60,$61,$62,$63,$64)|0);
  HEAP32[$0>>2] = $65;
  $329 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($329|0);
  break;
 }
 case 151:  {
  $66 = HEAP32[$2>>2]|0;
  $67 = ((($66)) + 20|0);
  $68 = HEAP32[$67>>2]|0;
  HEAP32[$pList>>2] = $68;
  $69 = HEAP32[$pList>>2]|0;
  $70 = ($69|0)!=(0|0);
  if ($70) {
   $71 = HEAP32[$pList>>2]|0;
   $72 = HEAP32[$71>>2]|0;
   $73 = $72;
  } else {
   $73 = 0;
  }
  HEAP32[$n>>2] = $73;
  HEAP32[$no_such_func>>2] = 0;
  HEAP32[$wrong_num_args>>2] = 0;
  HEAP32[$is_agg>>2] = 0;
  $74 = HEAP32[$pParse>>2]|0;
  $75 = HEAP32[$74>>2]|0;
  $76 = ((($75)) + 66|0);
  $77 = HEAP8[$76>>0]|0;
  HEAP8[$enc>>0] = $77;
  $78 = HEAP32[$2>>2]|0;
  $79 = ((($78)) + 8|0);
  $80 = HEAP32[$79>>2]|0;
  HEAP32[$zId>>2] = $80;
  $81 = HEAP32[$zId>>2]|0;
  $82 = (_sqlite3Strlen30($81)|0);
  HEAP32[$nId>>2] = $82;
  $83 = HEAP32[$pParse>>2]|0;
  $84 = HEAP32[$83>>2]|0;
  $85 = HEAP32[$zId>>2]|0;
  $86 = HEAP32[$n>>2]|0;
  $87 = HEAP8[$enc>>0]|0;
  $88 = (_sqlite3FindFunction($84,$85,$86,$87,0)|0);
  HEAP32[$pDef>>2] = $88;
  $89 = HEAP32[$pDef>>2]|0;
  $90 = ($89|0)==(0|0);
  do {
   if ($90) {
    $91 = HEAP32[$pParse>>2]|0;
    $92 = HEAP32[$91>>2]|0;
    $93 = HEAP32[$zId>>2]|0;
    $94 = HEAP8[$enc>>0]|0;
    $95 = (_sqlite3FindFunction($92,$93,-2,$94,0)|0);
    HEAP32[$pDef>>2] = $95;
    $96 = HEAP32[$pDef>>2]|0;
    $97 = ($96|0)==(0|0);
    if ($97) {
     HEAP32[$no_such_func>>2] = 1;
     break;
    } else {
     HEAP32[$wrong_num_args>>2] = 1;
     break;
    }
   } else {
    $98 = HEAP32[$pDef>>2]|0;
    $99 = ((($98)) + 16|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = ($100|0)!=(0|0);
    $102 = $101&1;
    HEAP32[$is_agg>>2] = $102;
    $103 = HEAP32[$pDef>>2]|0;
    $104 = ((($103)) + 2|0);
    $105 = HEAP16[$104>>1]|0;
    $106 = $105&65535;
    $107 = $106 & 1024;
    $108 = ($107|0)!=(0);
    do {
     if ($108) {
      $109 = HEAP32[$2>>2]|0;
      $110 = ((($109)) + 4|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = $111 | 266240;
      HEAP32[$110>>2] = $112;
      $113 = HEAP32[$n>>2]|0;
      $114 = ($113|0)==(2);
      if (!($114)) {
       $132 = HEAP32[$pDef>>2]|0;
       $133 = ((($132)) + 20|0);
       $134 = HEAP32[$133>>2]|0;
       $135 = HEAP8[$134>>0]|0;
       $136 = $135 << 24 >> 24;
       $137 = ($136|0)==(117);
       $138 = $137 ? 8388608 : 125829120;
       $139 = HEAP32[$2>>2]|0;
       $140 = ((($139)) + 28|0);
       HEAP32[$140>>2] = $138;
       break;
      }
      $115 = HEAP32[$pList>>2]|0;
      $116 = ((($115)) + 4|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = ((($117)) + 20|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = (_exprProbability($119)|0);
      $121 = HEAP32[$2>>2]|0;
      $122 = ((($121)) + 28|0);
      HEAP32[$122>>2] = $120;
      $123 = HEAP32[$2>>2]|0;
      $124 = ((($123)) + 28|0);
      $125 = HEAP32[$124>>2]|0;
      $126 = ($125|0)<(0);
      if ($126) {
       $127 = HEAP32[$pParse>>2]|0;
       _sqlite3ErrorMsg($127,29578,$vararg_buffer);
       $128 = HEAP32[$pNC>>2]|0;
       $129 = ((($128)) + 24|0);
       $130 = HEAP32[$129>>2]|0;
       $131 = (($130) + 1)|0;
       HEAP32[$129>>2] = $131;
      }
     }
    } while(0);
    $141 = HEAP32[$pParse>>2]|0;
    $142 = HEAP32[$pDef>>2]|0;
    $143 = ((($142)) + 20|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = (_sqlite3AuthCheck($141,31,0,$144,0)|0);
    HEAP32[$auth>>2] = $145;
    $146 = HEAP32[$auth>>2]|0;
    $147 = ($146|0)!=(0);
    if ($147) {
     $148 = HEAP32[$auth>>2]|0;
     $149 = ($148|0)==(1);
     if ($149) {
      $150 = HEAP32[$pParse>>2]|0;
      $151 = HEAP32[$pDef>>2]|0;
      $152 = ((($151)) + 20|0);
      $153 = HEAP32[$152>>2]|0;
      HEAP32[$vararg_buffer1>>2] = $153;
      _sqlite3ErrorMsg($150,29649,$vararg_buffer1);
      $154 = HEAP32[$pNC>>2]|0;
      $155 = ((($154)) + 24|0);
      $156 = HEAP32[$155>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$155>>2] = $157;
     }
     $158 = HEAP32[$2>>2]|0;
     HEAP8[$158>>0] = 101;
     HEAP32[$0>>2] = 1;
     $329 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($329|0);
    } else {
     $159 = HEAP32[$pDef>>2]|0;
     $160 = ((($159)) + 2|0);
     $161 = HEAP16[$160>>1]|0;
     $162 = $161&65535;
     $163 = $162 & 10240;
     $164 = ($163|0)!=(0);
     if ($164) {
      $165 = HEAP32[$2>>2]|0;
      $166 = ((($165)) + 4|0);
      $167 = HEAP32[$166>>2]|0;
      $168 = $167 | 524288;
      HEAP32[$166>>2] = $168;
     }
     $169 = HEAP32[$pDef>>2]|0;
     $170 = ((($169)) + 2|0);
     $171 = HEAP16[$170>>1]|0;
     $172 = $171&65535;
     $173 = $172 & 2048;
     $174 = ($173|0)==(0);
     if (!($174)) {
      break;
     }
     $175 = HEAP32[$pParse>>2]|0;
     $176 = HEAP32[$pNC>>2]|0;
     _notValid($175,$176,29684,34);
     break;
    }
   }
  } while(0);
  $177 = HEAP32[$is_agg>>2]|0;
  $178 = ($177|0)!=(0);
  if ($178) {
   $179 = HEAP32[$pNC>>2]|0;
   $180 = ((($179)) + 28|0);
   $181 = HEAP16[$180>>1]|0;
   $182 = $181&65535;
   $183 = $182 & 1;
   $184 = ($183|0)==(0);
   if ($184) {
    $185 = HEAP32[$pParse>>2]|0;
    $186 = HEAP32[$nId>>2]|0;
    $187 = HEAP32[$zId>>2]|0;
    HEAP32[$vararg_buffer3>>2] = $186;
    $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
    HEAP32[$vararg_ptr6>>2] = $187;
    _sqlite3ErrorMsg($185,29712,$vararg_buffer3);
    $188 = HEAP32[$pNC>>2]|0;
    $189 = ((($188)) + 24|0);
    $190 = HEAP32[$189>>2]|0;
    $191 = (($190) + 1)|0;
    HEAP32[$189>>2] = $191;
    HEAP32[$is_agg>>2] = 0;
   } else {
    label = 31;
   }
  } else {
   label = 31;
  }
  do {
   if ((label|0) == 31) {
    $192 = HEAP32[$no_such_func>>2]|0;
    $193 = ($192|0)!=(0);
    if ($193) {
     $194 = HEAP32[$pParse>>2]|0;
     $195 = HEAP32[$194>>2]|0;
     $196 = ((($195)) + 148|0);
     $197 = ((($196)) + 5|0);
     $198 = HEAP8[$197>>0]|0;
     $199 = $198&255;
     $200 = ($199|0)==(0);
     if ($200) {
      $201 = HEAP32[$pParse>>2]|0;
      $202 = HEAP32[$nId>>2]|0;
      $203 = HEAP32[$zId>>2]|0;
      HEAP32[$vararg_buffer7>>2] = $202;
      $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
      HEAP32[$vararg_ptr10>>2] = $203;
      _sqlite3ErrorMsg($201,29748,$vararg_buffer7);
      $204 = HEAP32[$pNC>>2]|0;
      $205 = ((($204)) + 24|0);
      $206 = HEAP32[$205>>2]|0;
      $207 = (($206) + 1)|0;
      HEAP32[$205>>2] = $207;
      break;
     }
    }
    $208 = HEAP32[$wrong_num_args>>2]|0;
    $209 = ($208|0)!=(0);
    if ($209) {
     $210 = HEAP32[$pParse>>2]|0;
     $211 = HEAP32[$nId>>2]|0;
     $212 = HEAP32[$zId>>2]|0;
     HEAP32[$vararg_buffer11>>2] = $211;
     $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
     HEAP32[$vararg_ptr14>>2] = $212;
     _sqlite3ErrorMsg($210,29771,$vararg_buffer11);
     $213 = HEAP32[$pNC>>2]|0;
     $214 = ((($213)) + 24|0);
     $215 = HEAP32[$214>>2]|0;
     $216 = (($215) + 1)|0;
     HEAP32[$214>>2] = $216;
    }
   }
  } while(0);
  $217 = HEAP32[$is_agg>>2]|0;
  $218 = ($217|0)!=(0);
  if ($218) {
   $219 = HEAP32[$pNC>>2]|0;
   $220 = ((($219)) + 28|0);
   $221 = HEAP16[$220>>1]|0;
   $222 = $221&65535;
   $223 = $222 & -2;
   $224 = $223&65535;
   HEAP16[$220>>1] = $224;
  }
  $225 = HEAP32[$1>>2]|0;
  $226 = HEAP32[$pList>>2]|0;
  (_sqlite3WalkExprList($225,$226)|0);
  $227 = HEAP32[$is_agg>>2]|0;
  $228 = ($227|0)!=(0);
  if ($228) {
   $229 = HEAP32[$pNC>>2]|0;
   HEAP32[$pNC2>>2] = $229;
   $230 = HEAP32[$2>>2]|0;
   HEAP8[$230>>0] = -103;
   $231 = HEAP32[$2>>2]|0;
   $232 = ((($231)) + 38|0);
   HEAP8[$232>>0] = 0;
   while(1) {
    $233 = HEAP32[$pNC2>>2]|0;
    $234 = ($233|0)!=(0|0);
    if (!($234)) {
     break;
    }
    $235 = HEAP32[$2>>2]|0;
    $236 = HEAP32[$pNC2>>2]|0;
    $237 = ((($236)) + 4|0);
    $238 = HEAP32[$237>>2]|0;
    $239 = (_sqlite3FunctionUsesThisSrc($235,$238)|0);
    $240 = ($239|0)!=(0);
    $241 = $240 ^ 1;
    if (!($241)) {
     break;
    }
    $242 = HEAP32[$2>>2]|0;
    $243 = ((($242)) + 38|0);
    $244 = HEAP8[$243>>0]|0;
    $245 = (($244) + 1)<<24>>24;
    HEAP8[$243>>0] = $245;
    $246 = HEAP32[$pNC2>>2]|0;
    $247 = ((($246)) + 16|0);
    $248 = HEAP32[$247>>2]|0;
    HEAP32[$pNC2>>2] = $248;
   }
   $249 = HEAP32[$pNC2>>2]|0;
   $250 = ($249|0)!=(0|0);
   if ($250) {
    $251 = HEAP32[$pDef>>2]|0;
    $252 = ((($251)) + 2|0);
    $253 = HEAP16[$252>>1]|0;
    $254 = $253&65535;
    $255 = $254 & 4096;
    $256 = 16 | $255;
    $257 = HEAP32[$pNC2>>2]|0;
    $258 = ((($257)) + 28|0);
    $259 = HEAP16[$258>>1]|0;
    $260 = $259&65535;
    $261 = $260 | $256;
    $262 = $261&65535;
    HEAP16[$258>>1] = $262;
   }
   $263 = HEAP32[$pNC>>2]|0;
   $264 = ((($263)) + 28|0);
   $265 = HEAP16[$264>>1]|0;
   $266 = $265&65535;
   $267 = $266 | 1;
   $268 = $267&65535;
   HEAP16[$264>>1] = $268;
  }
  HEAP32[$0>>2] = 1;
  $329 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($329|0);
  break;
 }
 case 33: case 20: case 119:  {
  $269 = HEAP32[$2>>2]|0;
  $270 = ((($269)) + 4|0);
  $271 = HEAP32[$270>>2]|0;
  $272 = $271 & 2048;
  $273 = ($272|0)!=(0);
  if ($273) {
   $274 = HEAP32[$pNC>>2]|0;
   $275 = ((($274)) + 20|0);
   $276 = HEAP32[$275>>2]|0;
   HEAP32[$nRef>>2] = $276;
   $277 = HEAP32[$pParse>>2]|0;
   $278 = HEAP32[$pNC>>2]|0;
   _notValid($277,$278,29816,38);
   $279 = HEAP32[$1>>2]|0;
   $280 = HEAP32[$2>>2]|0;
   $281 = ((($280)) + 20|0);
   $282 = HEAP32[$281>>2]|0;
   (_sqlite3WalkSelect($279,$282)|0);
   $283 = HEAP32[$nRef>>2]|0;
   $284 = HEAP32[$pNC>>2]|0;
   $285 = ((($284)) + 20|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = ($283|0)!=($286|0);
   if ($287) {
    $288 = HEAP32[$2>>2]|0;
    $289 = ((($288)) + 4|0);
    $290 = HEAP32[$289>>2]|0;
    $291 = $290 | 32;
    HEAP32[$289>>2] = $291;
    $292 = HEAP32[$pNC>>2]|0;
    $293 = ((($292)) + 28|0);
    $294 = HEAP16[$293>>1]|0;
    $295 = $294&65535;
    $296 = $295 | 64;
    $297 = $296&65535;
    HEAP16[$293>>1] = $297;
   }
  }
  break;
 }
 case 135:  {
  $298 = HEAP32[$pParse>>2]|0;
  $299 = HEAP32[$pNC>>2]|0;
  _notValid($298,$299,29827,38);
  break;
 }
 case 148: case 29: case 41: case 38: case 39: case 40: case 36: case 37:  {
  $300 = HEAP32[$pParse>>2]|0;
  $301 = HEAP32[$300>>2]|0;
  $302 = ((($301)) + 69|0);
  $303 = HEAP8[$302>>0]|0;
  $304 = ($303<<24>>24)!=(0);
  if (!($304)) {
   $305 = HEAP32[$2>>2]|0;
   $306 = ((($305)) + 12|0);
   $307 = HEAP32[$306>>2]|0;
   $308 = (_sqlite3ExprVectorSize($307)|0);
   HEAP32[$nLeft>>2] = $308;
   $309 = HEAP32[$2>>2]|0;
   $310 = ((($309)) + 16|0);
   $311 = HEAP32[$310>>2]|0;
   $312 = (_sqlite3ExprVectorSize($311)|0);
   HEAP32[$nRight>>2] = $312;
   $313 = HEAP32[$nLeft>>2]|0;
   $314 = HEAP32[$nRight>>2]|0;
   $315 = ($313|0)!=($314|0);
   if ($315) {
    $316 = HEAP32[$pParse>>2]|0;
    _sqlite3ErrorMsg($316,29838,$vararg_buffer15);
   }
  }
  break;
 }
 default: {
 }
 }
 $317 = HEAP32[$pParse>>2]|0;
 $318 = ((($317)) + 36|0);
 $319 = HEAP32[$318>>2]|0;
 $320 = ($319|0)!=(0);
 if ($320) {
  $328 = 1;
 } else {
  $321 = HEAP32[$pParse>>2]|0;
  $322 = HEAP32[$321>>2]|0;
  $323 = ((($322)) + 69|0);
  $324 = HEAP8[$323>>0]|0;
  $325 = $324&255;
  $326 = ($325|0)!=(0);
  $328 = $326;
 }
 $327 = $328 ? 2 : 0;
 HEAP32[$0>>2] = $327;
 $329 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($329|0);
}
function _resolveSelectStep($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $isCompound = 0, $nCompound = 0, $nRef = 0, $or$cond = 0, $pGroupBy = 0;
 var $pItem = 0, $pItem1 = 0, $pItem3 = 0, $pLeftmost = 0, $pNC = 0, $pOuterNC = 0, $pParse = 0, $pSub = 0, $pSub2 = 0, $sNC = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $zSavedContext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $pOuterNC = sp + 104|0;
 $sNC = sp + 72|0;
 $isCompound = sp + 68|0;
 $nCompound = sp + 64|0;
 $pParse = sp + 60|0;
 $i = sp + 56|0;
 $pGroupBy = sp + 52|0;
 $pLeftmost = sp + 48|0;
 $db = sp + 44|0;
 $pSub = sp + 40|0;
 $pItem = sp + 36|0;
 $pNC = sp + 32|0;
 $nRef = sp + 28|0;
 $zSavedContext = sp + 24|0;
 $pItem1 = sp + 20|0;
 $pSub2 = sp + 16|0;
 $pItem3 = sp + 12|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 4;
 $7 = ($6|0)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 1;
  $332 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($332|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pOuterNC>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pParse>>2] = $12;
 $13 = HEAP32[$pParse>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17 & 64;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$pParse>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$pOuterNC>>2]|0;
  _sqlite3SelectPrep($20,$21,$22);
  $23 = HEAP32[$pParse>>2]|0;
  $24 = ((($23)) + 36|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)!=(0);
  if ($26) {
   $33 = 1;
  } else {
   $27 = HEAP32[$db>>2]|0;
   $28 = ((($27)) + 69|0);
   $29 = HEAP8[$28>>0]|0;
   $30 = $29&255;
   $31 = ($30|0)!=(0);
   $33 = $31;
  }
  $32 = $33 ? 2 : 1;
  HEAP32[$0>>2] = $32;
  $332 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($332|0);
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 48|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)!=(0|0);
 $38 = $37&1;
 HEAP32[$isCompound>>2] = $38;
 HEAP32[$nCompound>>2] = 0;
 $39 = HEAP32[$2>>2]|0;
 HEAP32[$pLeftmost>>2] = $39;
 L12: while(1) {
  $40 = HEAP32[$2>>2]|0;
  $41 = ($40|0)!=(0|0);
  if (!($41)) {
   label = 68;
   break;
  }
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 8|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = $44 | 4;
  HEAP32[$43>>2] = $45;
  ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
  $46 = HEAP32[$pParse>>2]|0;
  HEAP32[$sNC>>2] = $46;
  $47 = HEAP32[$2>>2]|0;
  $48 = ((($47)) + 56|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_sqlite3ResolveExprNames($sNC,$49)|0);
  $51 = ($50|0)!=(0);
  if ($51) {
   label = 11;
   break;
  }
  $52 = HEAP32[$2>>2]|0;
  $53 = ((($52)) + 60|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3ResolveExprNames($sNC,$54)|0);
  $56 = ($55|0)!=(0);
  if ($56) {
   label = 11;
   break;
  }
  $57 = HEAP32[$2>>2]|0;
  $58 = ((($57)) + 8|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $59 & 65536;
  $61 = ($60|0)!=(0);
  if ($61) {
   $62 = HEAP32[$2>>2]|0;
   $63 = ((($62)) + 28|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = ((($64)) + 8|0);
   $66 = ((($65)) + 20|0);
   $67 = HEAP32[$66>>2]|0;
   HEAP32[$pSub>>2] = $67;
   $68 = HEAP32[$2>>2]|0;
   $69 = ((($68)) + 44|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$pSub>>2]|0;
   $72 = ((($71)) + 44|0);
   HEAP32[$72>>2] = $70;
   $73 = HEAP32[$2>>2]|0;
   $74 = ((($73)) + 44|0);
   HEAP32[$74>>2] = 0;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $75 = HEAP32[$i>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = ((($76)) + 28|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = HEAP32[$78>>2]|0;
   $80 = ($75|0)<($79|0);
   if (!($80)) {
    break;
   }
   $81 = HEAP32[$i>>2]|0;
   $82 = HEAP32[$2>>2]|0;
   $83 = ((($82)) + 28|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ((($84)) + 8|0);
   $86 = (($85) + (($81*72)|0)|0);
   HEAP32[$pItem>>2] = $86;
   $87 = HEAP32[$pItem>>2]|0;
   $88 = ((($87)) + 20|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = ($89|0)!=(0|0);
   if ($90) {
    HEAP32[$nRef>>2] = 0;
    $91 = HEAP32[$pParse>>2]|0;
    $92 = ((($91)) + 448|0);
    $93 = HEAP32[$92>>2]|0;
    HEAP32[$zSavedContext>>2] = $93;
    $94 = HEAP32[$pOuterNC>>2]|0;
    HEAP32[$pNC>>2] = $94;
    while(1) {
     $95 = HEAP32[$pNC>>2]|0;
     $96 = ($95|0)!=(0|0);
     if (!($96)) {
      break;
     }
     $97 = HEAP32[$pNC>>2]|0;
     $98 = ((($97)) + 20|0);
     $99 = HEAP32[$98>>2]|0;
     $100 = HEAP32[$nRef>>2]|0;
     $101 = (($100) + ($99))|0;
     HEAP32[$nRef>>2] = $101;
     $102 = HEAP32[$pNC>>2]|0;
     $103 = ((($102)) + 16|0);
     $104 = HEAP32[$103>>2]|0;
     HEAP32[$pNC>>2] = $104;
    }
    $105 = HEAP32[$pItem>>2]|0;
    $106 = ((($105)) + 8|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = ($107|0)!=(0|0);
    if ($108) {
     $109 = HEAP32[$pItem>>2]|0;
     $110 = ((($109)) + 8|0);
     $111 = HEAP32[$110>>2]|0;
     $112 = HEAP32[$pParse>>2]|0;
     $113 = ((($112)) + 448|0);
     HEAP32[$113>>2] = $111;
    }
    $114 = HEAP32[$pParse>>2]|0;
    $115 = HEAP32[$pItem>>2]|0;
    $116 = ((($115)) + 20|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = HEAP32[$pOuterNC>>2]|0;
    _sqlite3ResolveSelectNames($114,$117,$118);
    $119 = HEAP32[$zSavedContext>>2]|0;
    $120 = HEAP32[$pParse>>2]|0;
    $121 = ((($120)) + 448|0);
    HEAP32[$121>>2] = $119;
    $122 = HEAP32[$pParse>>2]|0;
    $123 = ((($122)) + 36|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = ($124|0)!=(0);
    if ($125) {
     label = 24;
     break L12;
    }
    $126 = HEAP32[$db>>2]|0;
    $127 = ((($126)) + 69|0);
    $128 = HEAP8[$127>>0]|0;
    $129 = $128&255;
    $130 = ($129|0)!=(0);
    if ($130) {
     label = 24;
     break L12;
    }
    $131 = HEAP32[$pOuterNC>>2]|0;
    HEAP32[$pNC>>2] = $131;
    while(1) {
     $132 = HEAP32[$pNC>>2]|0;
     $133 = ($132|0)!=(0|0);
     if (!($133)) {
      break;
     }
     $134 = HEAP32[$pNC>>2]|0;
     $135 = ((($134)) + 20|0);
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$nRef>>2]|0;
     $138 = (($137) - ($136))|0;
     HEAP32[$nRef>>2] = $138;
     $139 = HEAP32[$pNC>>2]|0;
     $140 = ((($139)) + 16|0);
     $141 = HEAP32[$140>>2]|0;
     HEAP32[$pNC>>2] = $141;
    }
    $142 = HEAP32[$nRef>>2]|0;
    $143 = ($142|0)!=(0);
    $144 = $143&1;
    $145 = HEAP32[$pItem>>2]|0;
    $146 = ((($145)) + 36|0);
    $147 = ((($146)) + 1|0);
    $148 = $144&255;
    $149 = HEAP8[$147>>0]|0;
    $150 = $148 & 1;
    $151 = ($150 << 3)&255;
    $152 = $149 & -9;
    $153 = $152 | $151;
    HEAP8[$147>>0] = $153;
   }
   $154 = HEAP32[$i>>2]|0;
   $155 = (($154) + 1)|0;
   HEAP32[$i>>2] = $155;
  }
  $156 = ((($sNC)) + 28|0);
  HEAP16[$156>>1] = 1;
  $157 = HEAP32[$2>>2]|0;
  $158 = ((($157)) + 28|0);
  $159 = HEAP32[$158>>2]|0;
  $160 = ((($sNC)) + 4|0);
  HEAP32[$160>>2] = $159;
  $161 = HEAP32[$pOuterNC>>2]|0;
  $162 = ((($sNC)) + 16|0);
  HEAP32[$162>>2] = $161;
  $163 = HEAP32[$2>>2]|0;
  $164 = HEAP32[$163>>2]|0;
  $165 = (_sqlite3ResolveExprListNames($sNC,$164)|0);
  $166 = ($165|0)!=(0);
  if ($166) {
   label = 31;
   break;
  }
  $167 = HEAP32[$2>>2]|0;
  $168 = ((($167)) + 36|0);
  $169 = HEAP32[$168>>2]|0;
  HEAP32[$pGroupBy>>2] = $169;
  $170 = HEAP32[$pGroupBy>>2]|0;
  $171 = ($170|0)!=(0|0);
  if ($171) {
   label = 34;
  } else {
   $172 = ((($sNC)) + 28|0);
   $173 = HEAP16[$172>>1]|0;
   $174 = $173&65535;
   $175 = $174 & 16;
   $176 = ($175|0)!=(0);
   if ($176) {
    label = 34;
   } else {
    $186 = ((($sNC)) + 28|0);
    $187 = HEAP16[$186>>1]|0;
    $188 = $187&65535;
    $189 = $188 & -2;
    $190 = $189&65535;
    HEAP16[$186>>1] = $190;
   }
  }
  if ((label|0) == 34) {
   label = 0;
   $177 = ((($sNC)) + 28|0);
   $178 = HEAP16[$177>>1]|0;
   $179 = $178&65535;
   $180 = $179 & 4096;
   $181 = 8 | $180;
   $182 = HEAP32[$2>>2]|0;
   $183 = ((($182)) + 8|0);
   $184 = HEAP32[$183>>2]|0;
   $185 = $184 | $181;
   HEAP32[$183>>2] = $185;
  }
  $191 = HEAP32[$2>>2]|0;
  $192 = ((($191)) + 40|0);
  $193 = HEAP32[$192>>2]|0;
  $194 = ($193|0)==(0|0);
  $195 = HEAP32[$pGroupBy>>2]|0;
  $196 = ($195|0)!=(0|0);
  $or$cond = $194 | $196;
  if (!($or$cond)) {
   label = 37;
   break;
  }
  $198 = HEAP32[$2>>2]|0;
  $199 = HEAP32[$198>>2]|0;
  $200 = ((($sNC)) + 8|0);
  HEAP32[$200>>2] = $199;
  $201 = HEAP32[$2>>2]|0;
  $202 = ((($201)) + 40|0);
  $203 = HEAP32[$202>>2]|0;
  $204 = (_sqlite3ResolveExprNames($sNC,$203)|0);
  $205 = ($204|0)!=(0);
  if ($205) {
   label = 39;
   break;
  }
  $206 = HEAP32[$2>>2]|0;
  $207 = ((($206)) + 32|0);
  $208 = HEAP32[$207>>2]|0;
  $209 = (_sqlite3ResolveExprNames($sNC,$208)|0);
  $210 = ($209|0)!=(0);
  if ($210) {
   label = 41;
   break;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $211 = HEAP32[$i>>2]|0;
   $212 = HEAP32[$2>>2]|0;
   $213 = ((($212)) + 28|0);
   $214 = HEAP32[$213>>2]|0;
   $215 = HEAP32[$214>>2]|0;
   $216 = ($211|0)<($215|0);
   if (!($216)) {
    break;
   }
   $217 = HEAP32[$i>>2]|0;
   $218 = HEAP32[$2>>2]|0;
   $219 = ((($218)) + 28|0);
   $220 = HEAP32[$219>>2]|0;
   $221 = ((($220)) + 8|0);
   $222 = (($221) + (($217*72)|0)|0);
   HEAP32[$pItem1>>2] = $222;
   $223 = HEAP32[$pItem1>>2]|0;
   $224 = ((($223)) + 36|0);
   $225 = ((($224)) + 1|0);
   $226 = HEAP8[$225>>0]|0;
   $227 = ($226&255) >>> 2;
   $228 = $227 & 1;
   $229 = $228&255;
   $230 = ($229|0)!=(0);
   if ($230) {
    $231 = HEAP32[$pItem1>>2]|0;
    $232 = ((($231)) + 64|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = (_sqlite3ResolveExprListNames($sNC,$233)|0);
    $235 = ($234|0)!=(0);
    if ($235) {
     label = 46;
     break L12;
    }
   }
   $236 = HEAP32[$i>>2]|0;
   $237 = (($236) + 1)|0;
   HEAP32[$i>>2] = $237;
  }
  $238 = ((($sNC)) + 16|0);
  HEAP32[$238>>2] = 0;
  $239 = ((($sNC)) + 28|0);
  $240 = HEAP16[$239>>1]|0;
  $241 = $240&65535;
  $242 = $241 | 1;
  $243 = $242&65535;
  HEAP16[$239>>1] = $243;
  $244 = HEAP32[$2>>2]|0;
  $245 = ((($244)) + 8|0);
  $246 = HEAP32[$245>>2]|0;
  $247 = $246 & 65536;
  $248 = ($247|0)!=(0);
  if ($248) {
   $249 = HEAP32[$2>>2]|0;
   $250 = ((($249)) + 28|0);
   $251 = HEAP32[$250>>2]|0;
   $252 = ((($251)) + 8|0);
   $253 = ((($252)) + 20|0);
   $254 = HEAP32[$253>>2]|0;
   HEAP32[$pSub2>>2] = $254;
   $255 = HEAP32[$pSub2>>2]|0;
   $256 = ((($255)) + 44|0);
   $257 = HEAP32[$256>>2]|0;
   $258 = HEAP32[$2>>2]|0;
   $259 = ((($258)) + 44|0);
   HEAP32[$259>>2] = $257;
   $260 = HEAP32[$pSub2>>2]|0;
   $261 = ((($260)) + 44|0);
   HEAP32[$261>>2] = 0;
  }
  $262 = HEAP32[$isCompound>>2]|0;
  $263 = HEAP32[$nCompound>>2]|0;
  $264 = ($262|0)<=($263|0);
  if ($264) {
   $265 = HEAP32[$2>>2]|0;
   $266 = HEAP32[$2>>2]|0;
   $267 = ((($266)) + 44|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = (_resolveOrderGroupBy($sNC,$265,$268,29099)|0);
   $270 = ($269|0)!=(0);
   if ($270) {
    label = 52;
    break;
   }
  }
  $271 = HEAP32[$db>>2]|0;
  $272 = ((($271)) + 69|0);
  $273 = HEAP8[$272>>0]|0;
  $274 = ($273<<24>>24)!=(0);
  if ($274) {
   label = 54;
   break;
  }
  $275 = HEAP32[$pGroupBy>>2]|0;
  $276 = ($275|0)!=(0|0);
  L64: do {
   if ($276) {
    $277 = HEAP32[$2>>2]|0;
    $278 = HEAP32[$pGroupBy>>2]|0;
    $279 = (_resolveOrderGroupBy($sNC,$277,$278,29105)|0);
    $280 = ($279|0)!=(0);
    if ($280) {
     label = 58;
     break L12;
    }
    $281 = HEAP32[$db>>2]|0;
    $282 = ((($281)) + 69|0);
    $283 = HEAP8[$282>>0]|0;
    $284 = $283&255;
    $285 = ($284|0)!=(0);
    if ($285) {
     label = 58;
     break L12;
    }
    HEAP32[$i>>2] = 0;
    $286 = HEAP32[$pGroupBy>>2]|0;
    $287 = ((($286)) + 4|0);
    $288 = HEAP32[$287>>2]|0;
    HEAP32[$pItem3>>2] = $288;
    while(1) {
     $289 = HEAP32[$i>>2]|0;
     $290 = HEAP32[$pGroupBy>>2]|0;
     $291 = HEAP32[$290>>2]|0;
     $292 = ($289|0)<($291|0);
     if (!($292)) {
      break L64;
     }
     $293 = HEAP32[$pItem3>>2]|0;
     $294 = HEAP32[$293>>2]|0;
     $295 = ((($294)) + 4|0);
     $296 = HEAP32[$295>>2]|0;
     $297 = $296 & 2;
     $298 = ($297|0)!=(0);
     if ($298) {
      label = 62;
      break L12;
     }
     $300 = HEAP32[$i>>2]|0;
     $301 = (($300) + 1)|0;
     HEAP32[$i>>2] = $301;
     $302 = HEAP32[$pItem3>>2]|0;
     $303 = ((($302)) + 20|0);
     HEAP32[$pItem3>>2] = $303;
    }
   }
  } while(0);
  $304 = HEAP32[$2>>2]|0;
  $305 = ((($304)) + 52|0);
  $306 = HEAP32[$305>>2]|0;
  $307 = ($306|0)!=(0|0);
  if ($307) {
   $308 = HEAP32[$2>>2]|0;
   $309 = HEAP32[$308>>2]|0;
   $310 = HEAP32[$309>>2]|0;
   $311 = HEAP32[$2>>2]|0;
   $312 = ((($311)) + 52|0);
   $313 = HEAP32[$312>>2]|0;
   $314 = HEAP32[$313>>2]|0;
   $315 = HEAP32[$314>>2]|0;
   $316 = ($310|0)!=($315|0);
   if ($316) {
    label = 66;
    break;
   }
  }
  $321 = HEAP32[$2>>2]|0;
  $322 = ((($321)) + 48|0);
  $323 = HEAP32[$322>>2]|0;
  HEAP32[$2>>2] = $323;
  $324 = HEAP32[$nCompound>>2]|0;
  $325 = (($324) + 1)|0;
  HEAP32[$nCompound>>2] = $325;
 }
 switch (label|0) {
  case 11: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 24: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 31: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 37: {
   $197 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($197,29055,$vararg_buffer);
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 39: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 41: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 46: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 52: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 54: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 58: {
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 62: {
   $299 = HEAP32[$pParse>>2]|0;
   _sqlite3ErrorMsg($299,29111,$vararg_buffer2);
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 66: {
   $317 = HEAP32[$pParse>>2]|0;
   $318 = HEAP32[$2>>2]|0;
   $319 = ((($318)) + 52|0);
   $320 = HEAP32[$319>>2]|0;
   _sqlite3SelectWrongNumTermsError($317,$320);
   HEAP32[$0>>2] = 2;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
  case 68: {
   $326 = HEAP32[$isCompound>>2]|0;
   $327 = ($326|0)!=(0);
   if ($327) {
    $328 = HEAP32[$pParse>>2]|0;
    $329 = HEAP32[$pLeftmost>>2]|0;
    $330 = (_resolveCompoundOrderBy($328,$329)|0);
    $331 = ($330|0)!=(0);
    if ($331) {
     HEAP32[$0>>2] = 2;
     $332 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($332|0);
    }
   }
   HEAP32[$0>>2] = 1;
   $332 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($332|0);
   break;
  }
 }
 return (0)|0;
}
function _sqlite3ResolveExprNames($pNC,$pExpr) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $pParse = 0, $savedHasAgg = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $savedHasAgg = sp + 44|0;
 $w = sp + 4|0;
 $pParse = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 416|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($10) + ($14))|0;
 $16 = (_sqlite3ExprCheckHeight($7,$15)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 24|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pParse>>2]|0;
 $22 = ((($21)) + 416|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + ($20))|0;
 HEAP32[$22>>2] = $24;
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 28|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27&65535;
 $29 = $28 & 4112;
 $30 = $29&65535;
 HEAP16[$savedHasAgg>>1] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 28|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33&65535;
 $35 = $34 & -4113;
 $36 = $35&65535;
 HEAP16[$32>>1] = $36;
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 HEAP32[$w>>2] = $38;
 $39 = ((($w)) + 4|0);
 HEAP32[$39>>2] = 193;
 $40 = ((($w)) + 8|0);
 HEAP32[$40>>2] = 194;
 $41 = ((($w)) + 12|0);
 HEAP32[$41>>2] = 0;
 $42 = ((($w)) + 16|0);
 HEAP32[$42>>2] = 0;
 $43 = ((($w)) + 20|0);
 HEAP8[$43>>0] = 0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($w)) + 24|0);
 HEAP32[$45>>2] = $44;
 $46 = HEAP32[$2>>2]|0;
 (_sqlite3WalkExpr($w,$46)|0);
 $47 = HEAP32[$2>>2]|0;
 $48 = ((($47)) + 24|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = ((($51)) + 416|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = (($53) - ($49))|0;
 HEAP32[$52>>2] = $54;
 $55 = HEAP32[$1>>2]|0;
 $56 = ((($55)) + 24|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = ($57|0)>(0);
 if ($58) {
  label = 7;
 } else {
  $59 = HEAP32[$w>>2]|0;
  $60 = ((($59)) + 36|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = ($61|0)>(0);
  if ($62) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $63 = HEAP32[$2>>2]|0;
  $64 = ((($63)) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = $65 | 8;
  HEAP32[$64>>2] = $66;
 }
 $67 = HEAP32[$1>>2]|0;
 $68 = ((($67)) + 28|0);
 $69 = HEAP16[$68>>1]|0;
 $70 = $69&65535;
 $71 = $70 & 16;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = HEAP32[$2>>2]|0;
  $74 = ((($73)) + 4|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = $75 | 2;
  HEAP32[$74>>2] = $76;
 }
 $77 = HEAP16[$savedHasAgg>>1]|0;
 $78 = $77&65535;
 $79 = HEAP32[$1>>2]|0;
 $80 = ((($79)) + 28|0);
 $81 = HEAP16[$80>>1]|0;
 $82 = $81&65535;
 $83 = $82 | $78;
 $84 = $83&65535;
 HEAP16[$80>>1] = $84;
 $85 = HEAP32[$2>>2]|0;
 $86 = ((($85)) + 4|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = $87 & 8;
 $89 = ($88|0)!=(0);
 $90 = $89&1;
 HEAP32[$0>>2] = $90;
 $91 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($91|0);
}
function _sqlite3ResolveExprListNames($pNC,$pList) {
 $pNC = $pNC|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $5 = HEAP32[$i>>2]|0;
    $6 = HEAP32[$2>>2]|0;
    $7 = HEAP32[$6>>2]|0;
    $8 = ($5|0)<($7|0);
    if (!($8)) {
     break L1;
    }
    $9 = HEAP32[$1>>2]|0;
    $10 = HEAP32[$i>>2]|0;
    $11 = HEAP32[$2>>2]|0;
    $12 = ((($11)) + 4|0);
    $13 = HEAP32[$12>>2]|0;
    $14 = (($13) + (($10*20)|0)|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = (_sqlite3ResolveExprNames($9,$15)|0);
    $17 = ($16|0)!=(0);
    if ($17) {
     break;
    }
    $18 = HEAP32[$i>>2]|0;
    $19 = (($18) + 1)|0;
    HEAP32[$i>>2] = $19;
   }
   HEAP32[$0>>2] = 2;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _resolveOrderGroupBy($pNC,$pSelect,$pOrderBy,$zType) {
 $pNC = $pNC|0;
 $pSelect = $pSelect|0;
 $pOrderBy = $pOrderBy|0;
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $i = 0, $iCol = 0, $j = 0, $nResult = 0, $or$cond = 0, $pE = 0, $pE2 = 0, $pItem = 0;
 var $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $i = sp + 28|0;
 $j = sp + 24|0;
 $iCol = sp + 20|0;
 $pItem = sp + 16|0;
 $pParse = sp + 12|0;
 $nResult = sp + 8|0;
 $pE = sp + 4|0;
 $pE2 = sp;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pOrderBy;
 HEAP32[$4>>2] = $zType;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$nResult>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pParse>>2] = $11;
 HEAP32[$i>>2] = 0;
 $12 = HEAP32[$3>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pItem>>2] = $14;
 L5: while(1) {
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($15|0)<($17|0);
  if (!($18)) {
   label = 20;
   break;
  }
  $19 = HEAP32[$pItem>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pE>>2] = $20;
  $21 = HEAP32[$pE>>2]|0;
  $22 = (_sqlite3ExprSkipCollate($21)|0);
  HEAP32[$pE2>>2] = $22;
  $23 = HEAP32[$4>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24 << 24 >> 24;
  $26 = ($25|0)!=(71);
  if ($26) {
   $27 = HEAP32[$pParse>>2]|0;
   $28 = HEAP32[$2>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$pE2>>2]|0;
   $31 = (_resolveAsName($27,$29,$30)|0);
   HEAP32[$iCol>>2] = $31;
   $32 = HEAP32[$iCol>>2]|0;
   $33 = ($32|0)>(0);
   if ($33) {
    $34 = HEAP32[$iCol>>2]|0;
    $35 = $34&65535;
    $36 = HEAP32[$pItem>>2]|0;
    $37 = ((($36)) + 16|0);
    HEAP16[$37>>1] = $35;
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  L11: do {
   if ((label|0) == 8) {
    label = 0;
    $38 = HEAP32[$pE2>>2]|0;
    $39 = (_sqlite3ExprIsInteger($38,$iCol)|0);
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$iCol>>2]|0;
     $42 = ($41|0)<(1);
     $43 = HEAP32[$iCol>>2]|0;
     $44 = ($43|0)>(65535);
     $or$cond = $42 | $44;
     if ($or$cond) {
      label = 10;
      break L5;
     }
     $50 = HEAP32[$iCol>>2]|0;
     $51 = $50&65535;
     $52 = HEAP32[$pItem>>2]|0;
     $53 = ((($52)) + 16|0);
     HEAP16[$53>>1] = $51;
     break;
    }
    $54 = HEAP32[$pItem>>2]|0;
    $55 = ((($54)) + 16|0);
    HEAP16[$55>>1] = 0;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$pE>>2]|0;
    $58 = (_sqlite3ResolveExprNames($56,$57)|0);
    $59 = ($58|0)!=(0);
    if ($59) {
     label = 13;
     break L5;
    }
    HEAP32[$j>>2] = 0;
    while(1) {
     $60 = HEAP32[$j>>2]|0;
     $61 = HEAP32[$2>>2]|0;
     $62 = HEAP32[$61>>2]|0;
     $63 = HEAP32[$62>>2]|0;
     $64 = ($60|0)<($63|0);
     if (!($64)) {
      break L11;
     }
     $65 = HEAP32[$pE>>2]|0;
     $66 = HEAP32[$j>>2]|0;
     $67 = HEAP32[$2>>2]|0;
     $68 = HEAP32[$67>>2]|0;
     $69 = ((($68)) + 4|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = (($70) + (($66*20)|0)|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = (_sqlite3ExprCompare($65,$72,-1)|0);
     $74 = ($73|0)==(0);
     if ($74) {
      $75 = HEAP32[$j>>2]|0;
      $76 = (($75) + 1)|0;
      $77 = $76&65535;
      $78 = HEAP32[$pItem>>2]|0;
      $79 = ((($78)) + 16|0);
      HEAP16[$79>>1] = $77;
     }
     $80 = HEAP32[$j>>2]|0;
     $81 = (($80) + 1)|0;
     HEAP32[$j>>2] = $81;
    }
   }
  } while(0);
  $82 = HEAP32[$i>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$i>>2] = $83;
  $84 = HEAP32[$pItem>>2]|0;
  $85 = ((($84)) + 20|0);
  HEAP32[$pItem>>2] = $85;
 }
 if ((label|0) == 10) {
  $45 = HEAP32[$pParse>>2]|0;
  $46 = HEAP32[$4>>2]|0;
  $47 = HEAP32[$i>>2]|0;
  $48 = (($47) + 1)|0;
  $49 = HEAP32[$nResult>>2]|0;
  _resolveOutOfRangeError($45,$46,$48,$49);
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 1;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 else if ((label|0) == 20) {
  $86 = HEAP32[$pParse>>2]|0;
  $87 = HEAP32[$2>>2]|0;
  $88 = HEAP32[$3>>2]|0;
  $89 = HEAP32[$4>>2]|0;
  $90 = (_sqlite3ResolveOrderGroupBy($86,$87,$88,$89)|0);
  HEAP32[$0>>2] = $90;
  $91 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 return (0)|0;
}
function _sqlite3SelectWrongNumTermsError($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 512;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  _sqlite3ErrorMsg($7,29321,$vararg_buffer);
  STACKTOP = sp;return;
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = (_selectOpName($11)|0);
  HEAP32[$vararg_buffer1>>2] = $12;
  _sqlite3ErrorMsg($7,29367,$vararg_buffer1);
  STACKTOP = sp;return;
 }
}
function _resolveCompoundOrderBy($pParse,$pSelect) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $moreToDo = 0, $pDup = 0, $pE = 0, $pEList = 0;
 var $pItem = 0, $pNew = 0, $pOrderBy = 0, $pParent = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $i = sp + 52|0;
 $pOrderBy = sp + 48|0;
 $pEList = sp + 44|0;
 $db = sp + 40|0;
 $moreToDo = sp + 36|0;
 $pItem = sp + 32|0;
 $iCol = sp + 28|0;
 $pE = sp + 24|0;
 $pDup = sp + 20|0;
 $pNew = sp + 16|0;
 $pParent = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$moreToDo>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pOrderBy>>2] = $5;
 $6 = HEAP32[$pOrderBy>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$pOrderBy>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 96|0);
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($11|0)>($15|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($17,29170,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$pOrderBy>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = ($18|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pOrderBy>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + (($22*20)|0)|0);
  $27 = ((($26)) + 13|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 & -2;
  HEAP8[$27>>0] = $29;
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$2>>2]|0;
 $33 = ((($32)) + 52|0);
 HEAP32[$33>>2] = 0;
 while(1) {
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 48|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)!=(0|0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = ((($39)) + 48|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ((($41)) + 52|0);
  HEAP32[$42>>2] = $38;
  $43 = HEAP32[$2>>2]|0;
  $44 = ((($43)) + 48|0);
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$2>>2] = $45;
 }
 L16: while(1) {
  $46 = HEAP32[$2>>2]|0;
  $47 = ($46|0)!=(0|0);
  $48 = HEAP32[$moreToDo>>2]|0;
  $49 = ($48|0)!=(0);
  $50 = $47 ? $49 : 0;
  if (!($50)) {
   label = 36;
   break;
  }
  HEAP32[$moreToDo>>2] = 0;
  $51 = HEAP32[$2>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  HEAP32[$pEList>>2] = $52;
  HEAP32[$i>>2] = 0;
  $53 = HEAP32[$pOrderBy>>2]|0;
  $54 = ((($53)) + 4|0);
  $55 = HEAP32[$54>>2]|0;
  HEAP32[$pItem>>2] = $55;
  while(1) {
   $56 = HEAP32[$i>>2]|0;
   $57 = HEAP32[$pOrderBy>>2]|0;
   $58 = HEAP32[$57>>2]|0;
   $59 = ($56|0)<($58|0);
   if (!($59)) {
    break;
   }
   HEAP32[$iCol>>2] = -1;
   $60 = HEAP32[$pItem>>2]|0;
   $61 = ((($60)) + 13|0);
   $62 = HEAP8[$61>>0]|0;
   $63 = $62 & 1;
   $64 = $63&255;
   $65 = ($64|0)!=(0);
   do {
    if (!($65)) {
     $66 = HEAP32[$pItem>>2]|0;
     $67 = HEAP32[$66>>2]|0;
     $68 = (_sqlite3ExprSkipCollate($67)|0);
     HEAP32[$pE>>2] = $68;
     $69 = HEAP32[$pE>>2]|0;
     $70 = (_sqlite3ExprIsInteger($69,$iCol)|0);
     $71 = ($70|0)!=(0);
     if ($71) {
      $72 = HEAP32[$iCol>>2]|0;
      $73 = ($72|0)<=(0);
      if ($73) {
       label = 18;
       break L16;
      }
      $74 = HEAP32[$iCol>>2]|0;
      $75 = HEAP32[$pEList>>2]|0;
      $76 = HEAP32[$75>>2]|0;
      $77 = ($74|0)>($76|0);
      if ($77) {
       label = 18;
       break L16;
      }
     } else {
      $83 = HEAP32[$1>>2]|0;
      $84 = HEAP32[$pEList>>2]|0;
      $85 = HEAP32[$pE>>2]|0;
      $86 = (_resolveAsName($83,$84,$85)|0);
      HEAP32[$iCol>>2] = $86;
      $87 = HEAP32[$iCol>>2]|0;
      $88 = ($87|0)==(0);
      if ($88) {
       $89 = HEAP32[$db>>2]|0;
       $90 = HEAP32[$pE>>2]|0;
       $91 = (_sqlite3ExprDup($89,$90,0)|0);
       HEAP32[$pDup>>2] = $91;
       $92 = HEAP32[$db>>2]|0;
       $93 = ((($92)) + 69|0);
       $94 = HEAP8[$93>>0]|0;
       $95 = ($94<<24>>24)!=(0);
       if (!($95)) {
        $96 = HEAP32[$1>>2]|0;
        $97 = HEAP32[$2>>2]|0;
        $98 = HEAP32[$pDup>>2]|0;
        $99 = (_resolveOrderByTermToExprList($96,$97,$98)|0);
        HEAP32[$iCol>>2] = $99;
       }
       $100 = HEAP32[$db>>2]|0;
       $101 = HEAP32[$pDup>>2]|0;
       _sqlite3ExprDelete($100,$101);
      }
     }
     $102 = HEAP32[$iCol>>2]|0;
     $103 = ($102|0)>(0);
     if (!($103)) {
      HEAP32[$moreToDo>>2] = 1;
      break;
     }
     $104 = HEAP32[$db>>2]|0;
     $105 = (_sqlite3Expr($104,134,0)|0);
     HEAP32[$pNew>>2] = $105;
     $106 = HEAP32[$pNew>>2]|0;
     $107 = ($106|0)==(0|0);
     if ($107) {
      label = 25;
      break L16;
     }
     $108 = HEAP32[$pNew>>2]|0;
     $109 = ((($108)) + 4|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = $110 | 1024;
     HEAP32[$109>>2] = $111;
     $112 = HEAP32[$iCol>>2]|0;
     $113 = HEAP32[$pNew>>2]|0;
     $114 = ((($113)) + 8|0);
     HEAP32[$114>>2] = $112;
     $115 = HEAP32[$pItem>>2]|0;
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$pE>>2]|0;
     $118 = ($116|0)==($117|0);
     if ($118) {
      $119 = HEAP32[$pNew>>2]|0;
      $120 = HEAP32[$pItem>>2]|0;
      HEAP32[$120>>2] = $119;
     } else {
      $121 = HEAP32[$pItem>>2]|0;
      $122 = HEAP32[$121>>2]|0;
      HEAP32[$pParent>>2] = $122;
      while(1) {
       $123 = HEAP32[$pParent>>2]|0;
       $124 = ((($123)) + 12|0);
       $125 = HEAP32[$124>>2]|0;
       $126 = HEAP8[$125>>0]|0;
       $127 = $126&255;
       $128 = ($127|0)==(53);
       if (!($128)) {
        break;
       }
       $129 = HEAP32[$pParent>>2]|0;
       $130 = ((($129)) + 12|0);
       $131 = HEAP32[$130>>2]|0;
       HEAP32[$pParent>>2] = $131;
      }
      $132 = HEAP32[$pNew>>2]|0;
      $133 = HEAP32[$pParent>>2]|0;
      $134 = ((($133)) + 12|0);
      HEAP32[$134>>2] = $132;
     }
     $135 = HEAP32[$db>>2]|0;
     $136 = HEAP32[$pE>>2]|0;
     _sqlite3ExprDelete($135,$136);
     $137 = HEAP32[$iCol>>2]|0;
     $138 = $137&65535;
     $139 = HEAP32[$pItem>>2]|0;
     $140 = ((($139)) + 16|0);
     HEAP16[$140>>1] = $138;
     $141 = HEAP32[$pItem>>2]|0;
     $142 = ((($141)) + 13|0);
     $143 = HEAP8[$142>>0]|0;
     $144 = $143 & -2;
     $145 = $144 | 1;
     HEAP8[$142>>0] = $145;
    }
   } while(0);
   $146 = HEAP32[$i>>2]|0;
   $147 = (($146) + 1)|0;
   HEAP32[$i>>2] = $147;
   $148 = HEAP32[$pItem>>2]|0;
   $149 = ((($148)) + 20|0);
   HEAP32[$pItem>>2] = $149;
  }
  $150 = HEAP32[$2>>2]|0;
  $151 = ((($150)) + 52|0);
  $152 = HEAP32[$151>>2]|0;
  HEAP32[$2>>2] = $152;
 }
 if ((label|0) == 18) {
  $78 = HEAP32[$1>>2]|0;
  $79 = HEAP32[$i>>2]|0;
  $80 = (($79) + 1)|0;
  $81 = HEAP32[$pEList>>2]|0;
  $82 = HEAP32[$81>>2]|0;
  _resolveOutOfRangeError($78,29099,$80,$82);
  HEAP32[$0>>2] = 1;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 else if ((label|0) == 25) {
  HEAP32[$0>>2] = 1;
  $172 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($172|0);
 }
 else if ((label|0) == 36) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $153 = HEAP32[$i>>2]|0;
   $154 = HEAP32[$pOrderBy>>2]|0;
   $155 = HEAP32[$154>>2]|0;
   $156 = ($153|0)<($155|0);
   if (!($156)) {
    label = 41;
    break;
   }
   $157 = HEAP32[$i>>2]|0;
   $158 = HEAP32[$pOrderBy>>2]|0;
   $159 = ((($158)) + 4|0);
   $160 = HEAP32[$159>>2]|0;
   $161 = (($160) + (($157*20)|0)|0);
   $162 = ((($161)) + 13|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163 & 1;
   $165 = $164&255;
   $166 = ($165|0)==(0);
   if ($166) {
    label = 39;
    break;
   }
   $170 = HEAP32[$i>>2]|0;
   $171 = (($170) + 1)|0;
   HEAP32[$i>>2] = $171;
  }
  if ((label|0) == 39) {
   $167 = HEAP32[$1>>2]|0;
   $168 = HEAP32[$i>>2]|0;
   $169 = (($168) + 1)|0;
   HEAP32[$vararg_buffer1>>2] = $169;
   _sqlite3ErrorMsg($167,29204,$vararg_buffer1);
   HEAP32[$0>>2] = 1;
   $172 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($172|0);
  }
  else if ((label|0) == 41) {
   HEAP32[$0>>2] = 0;
   $172 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($172|0);
  }
 }
 return (0)|0;
}
function _sqlite3ExprIsInteger($p,$pValue) {
 $p = $p|0;
 $pValue = $pValue|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pValue;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 1024;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = ((($8)) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  HEAP32[$11>>2] = $10;
  HEAP32[$0>>2] = 1;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 $12 = HEAP8[$8>>0]|0;
 $13 = $12&255;
 switch ($13|0) {
 case 156:  {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_sqlite3ExprIsInteger($16,$17)|0);
  HEAP32[$rc>>2] = $18;
  break;
 }
 case 155:  {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3ExprIsInteger($21,$v)|0);
  $23 = ($22|0)!=(0);
  if ($23) {
   $24 = HEAP32[$v>>2]|0;
   $25 = (0 - ($24))|0;
   $26 = HEAP32[$2>>2]|0;
   HEAP32[$26>>2] = $25;
   HEAP32[$rc>>2] = 1;
  }
  break;
 }
 default: {
 }
 }
 $27 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $27;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _resolveOutOfRangeError($pParse,$zType,$i,$mx) {
 $pParse = $pParse|0;
 $zType = $zType|0;
 $i = $i|0;
 $mx = $mx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zType;
 HEAP32[$2>>2] = $i;
 HEAP32[$3>>2] = $mx;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $6;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $7;
 _sqlite3ErrorMsg($4,29265,$vararg_buffer);
 STACKTOP = sp;return;
}
function _resolveAsName($pParse,$pEList,$pE) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $pE = $pE|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zAs = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $zCol = sp + 4|0;
 $zAs = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $pE;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(55);
 L1: do {
  if ($7) {
   $8 = HEAP32[$3>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$zCol>>2] = $10;
   HEAP32[$i>>2] = 0;
   while(1) {
    $11 = HEAP32[$i>>2]|0;
    $12 = HEAP32[$2>>2]|0;
    $13 = HEAP32[$12>>2]|0;
    $14 = ($11|0)<($13|0);
    if (!($14)) {
     break L1;
    }
    $15 = HEAP32[$i>>2]|0;
    $16 = HEAP32[$2>>2]|0;
    $17 = ((($16)) + 4|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = (($18) + (($15*20)|0)|0);
    $20 = ((($19)) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$zAs>>2] = $21;
    $22 = HEAP32[$zAs>>2]|0;
    $23 = ($22|0)!=(0|0);
    if ($23) {
     $24 = HEAP32[$zAs>>2]|0;
     $25 = HEAP32[$zCol>>2]|0;
     $26 = (_sqlite3StrICmp($24,$25)|0);
     $27 = ($26|0)==(0);
     if ($27) {
      break;
     }
    }
    $30 = HEAP32[$i>>2]|0;
    $31 = (($30) + 1)|0;
    HEAP32[$i>>2] = $31;
   }
   $28 = HEAP32[$i>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$0>>2] = $29;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3ExprDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $9 = 0;
  STACKTOP = sp;return ($9|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = (_exprDup($5,$6,$7,0)|0);
 $9 = $8;
 STACKTOP = sp;return ($9|0);
}
function _resolveOrderByTermToExprList($pParse,$pSelect,$pE) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 $pE = $pE|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $nc = 0, $pEList = 0, $rc = 0, $savedSuppErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $i = sp + 44|0;
 $pEList = sp + 40|0;
 $nc = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 $savedSuppErr = sp + 64|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pE;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pEList>>2] = $5;
 ;HEAP32[$nc>>2]=0|0;HEAP32[$nc+4>>2]=0|0;HEAP32[$nc+8>>2]=0|0;HEAP32[$nc+12>>2]=0|0;HEAP32[$nc+16>>2]=0|0;HEAP32[$nc+20>>2]=0|0;HEAP32[$nc+24>>2]=0|0;HEAP32[$nc+28>>2]=0|0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$nc>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 28|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($nc)) + 4|0);
 HEAP32[$10>>2] = $9;
 $11 = HEAP32[$pEList>>2]|0;
 $12 = ((($nc)) + 8|0);
 HEAP32[$12>>2] = $11;
 $13 = ((($nc)) + 28|0);
 HEAP16[$13>>1] = 1;
 $14 = ((($nc)) + 24|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$db>>2] = $16;
 $17 = HEAP32[$db>>2]|0;
 $18 = ((($17)) + 73|0);
 $19 = HEAP8[$18>>0]|0;
 HEAP8[$savedSuppErr>>0] = $19;
 $20 = HEAP32[$db>>2]|0;
 $21 = ((($20)) + 73|0);
 HEAP8[$21>>0] = 1;
 $22 = HEAP32[$3>>2]|0;
 $23 = (_sqlite3ResolveExprNames($nc,$22)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP8[$savedSuppErr>>0]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = ((($25)) + 73|0);
 HEAP8[$26>>0] = $24;
 $27 = HEAP32[$rc>>2]|0;
 $28 = ($27|0)!=(0);
 if ($28) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$pEList>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = ($29|0)<($31|0);
  if (!($32)) {
   label = 8;
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$pEList>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($33*20)|0)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$3>>2]|0;
  $40 = (_sqlite3ExprCompare($38,$39,-1)|0);
  $41 = ($40|0)<(2);
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  if ($41) {
   label = 6;
   break;
  }
  HEAP32[$i>>2] = $43;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $43;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $44 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 return (0)|0;
}
function _sqlite3ExprCompare($pA,$pB,$iTab) {
 $pA = $pA|0;
 $pB = $pB|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $combinedFlags = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $combinedFlags = sp;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $or$cond = $5 | $7;
 if ($or$cond) {
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ($8|0)==($9|0);
  $11 = $10 ? 0 : 2;
  HEAP32[$0>>2] = $11;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 4|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $14 | $17;
 HEAP32[$combinedFlags>>2] = $18;
 $19 = HEAP32[$combinedFlags>>2]|0;
 $20 = $19 & 1024;
 $21 = ($20|0)!=(0);
 $22 = HEAP32[$1>>2]|0;
 if ($21) {
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $24 & $27;
  $29 = $28 & 1024;
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 8|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = ((($34)) + 8|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($33|0)==($36|0);
   if ($37) {
    HEAP32[$0>>2] = 0;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
  }
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $38 = HEAP8[$22>>0]|0;
 $39 = $38&255;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = ($39|0)!=($42|0);
 $44 = HEAP32[$1>>2]|0;
 $45 = HEAP8[$44>>0]|0;
 $46 = $45&255;
 if ($43) {
  $47 = ($46|0)==(53);
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = ((($48)) + 12|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$2>>2]|0;
   $52 = HEAP32[$3>>2]|0;
   $53 = (_sqlite3ExprCompare($50,$51,$52)|0);
   $54 = ($53|0)<(2);
   if ($54) {
    HEAP32[$0>>2] = 1;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
  }
  $55 = HEAP32[$2>>2]|0;
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = ($57|0)==(53);
  if ($58) {
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = ((($60)) + 12|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$3>>2]|0;
   $64 = (_sqlite3ExprCompare($59,$62,$63)|0);
   $65 = ($64|0)<(2);
   if ($65) {
    HEAP32[$0>>2] = 1;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
  }
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $66 = ($46|0)!=(152);
 do {
  if ($66) {
   $67 = HEAP32[$1>>2]|0;
   $68 = HEAP8[$67>>0]|0;
   $69 = $68&255;
   $70 = ($69|0)!=(154);
   if ($70) {
    $71 = HEAP32[$1>>2]|0;
    $72 = ((($71)) + 8|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = ($73|0)!=(0|0);
    if ($74) {
     $75 = HEAP32[$1>>2]|0;
     $76 = HEAP8[$75>>0]|0;
     $77 = $76&255;
     $78 = ($77|0)==(151);
     $79 = HEAP32[$1>>2]|0;
     $80 = ((($79)) + 8|0);
     $81 = HEAP32[$80>>2]|0;
     $82 = HEAP32[$2>>2]|0;
     $83 = ((($82)) + 8|0);
     $84 = HEAP32[$83>>2]|0;
     if ($78) {
      $85 = (_sqlite3StrICmp($81,$84)|0);
      $86 = ($85|0)!=(0);
      if (!($86)) {
       break;
      }
      HEAP32[$0>>2] = 2;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     } else {
      $87 = (_strcmp($81,$84)|0);
      $88 = ($87|0)!=(0);
      if (!($88)) {
       break;
      }
      $89 = HEAP32[$1>>2]|0;
      $90 = HEAP8[$89>>0]|0;
      $91 = $90&255;
      $92 = ($91|0)==(53);
      $93 = $92 ? 1 : 2;
      HEAP32[$0>>2] = $93;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     }
    }
   }
  }
 } while(0);
 $94 = HEAP32[$1>>2]|0;
 $95 = ((($94)) + 4|0);
 $96 = HEAP32[$95>>2]|0;
 $97 = $96 & 16;
 $98 = HEAP32[$2>>2]|0;
 $99 = ((($98)) + 4|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = $100 & 16;
 $102 = ($97|0)!=($101|0);
 if ($102) {
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 $103 = HEAP32[$combinedFlags>>2]|0;
 $104 = $103 & 16384;
 $105 = ($104|0)==(0);
 do {
  if ($105) {
   $106 = HEAP32[$combinedFlags>>2]|0;
   $107 = $106 & 2048;
   $108 = ($107|0)!=(0);
   if ($108) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $109 = HEAP32[$1>>2]|0;
   $110 = ((($109)) + 12|0);
   $111 = HEAP32[$110>>2]|0;
   $112 = HEAP32[$2>>2]|0;
   $113 = ((($112)) + 12|0);
   $114 = HEAP32[$113>>2]|0;
   $115 = HEAP32[$3>>2]|0;
   $116 = (_sqlite3ExprCompare($111,$114,$115)|0);
   $117 = ($116|0)!=(0);
   if ($117) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $118 = HEAP32[$1>>2]|0;
   $119 = ((($118)) + 16|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = HEAP32[$2>>2]|0;
   $122 = ((($121)) + 16|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = HEAP32[$3>>2]|0;
   $125 = (_sqlite3ExprCompare($120,$123,$124)|0);
   $126 = ($125|0)!=(0);
   if ($126) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $127 = HEAP32[$1>>2]|0;
   $128 = ((($127)) + 20|0);
   $129 = HEAP32[$128>>2]|0;
   $130 = HEAP32[$2>>2]|0;
   $131 = ((($130)) + 20|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = HEAP32[$3>>2]|0;
   $134 = (_sqlite3ExprListCompare($129,$132,$133)|0);
   $135 = ($134|0)!=(0);
   if ($135) {
    HEAP32[$0>>2] = 2;
    $168 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($168|0);
   }
   $136 = HEAP32[$combinedFlags>>2]|0;
   $137 = $136 & 8192;
   $138 = ($137|0)==(0);
   if ($138) {
    $139 = HEAP32[$1>>2]|0;
    $140 = HEAP8[$139>>0]|0;
    $141 = $140&255;
    $142 = ($141|0)!=(97);
    if ($142) {
     $143 = HEAP32[$1>>2]|0;
     $144 = ((($143)) + 32|0);
     $145 = HEAP16[$144>>1]|0;
     $146 = $145 << 16 >> 16;
     $147 = HEAP32[$2>>2]|0;
     $148 = ((($147)) + 32|0);
     $149 = HEAP16[$148>>1]|0;
     $150 = $149 << 16 >> 16;
     $151 = ($146|0)!=($150|0);
     if ($151) {
      HEAP32[$0>>2] = 2;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     }
     $152 = HEAP32[$1>>2]|0;
     $153 = ((($152)) + 28|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = HEAP32[$2>>2]|0;
     $156 = ((($155)) + 28|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ($154|0)!=($157|0);
     if ($158) {
      $159 = HEAP32[$1>>2]|0;
      $160 = ((($159)) + 28|0);
      $161 = HEAP32[$160>>2]|0;
      $162 = HEAP32[$3>>2]|0;
      $163 = ($161|0)!=($162|0);
      if (!($163)) {
       $164 = HEAP32[$2>>2]|0;
       $165 = ((($164)) + 28|0);
       $166 = HEAP32[$165>>2]|0;
       $167 = ($166|0)>=(0);
       if (!($167)) {
        break;
       }
      }
      HEAP32[$0>>2] = 2;
      $168 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($168|0);
     }
    }
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $168 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($168|0);
}
function _sqlite3ExprListCompare($pA,$pB,$iTab) {
 $pA = $pA|0;
 $pB = $pB|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $or$cond = 0;
 var $or$cond3 = 0, $pExprA = 0, $pExprB = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $pExprA = sp + 4|0;
 $pExprB = sp;
 HEAP32[$1>>2] = $pA;
 HEAP32[$2>>2] = $pB;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0|0);
 $or$cond = $5 & $7;
 if ($or$cond) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0|0);
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 $or$cond3 = $9 | $11;
 if ($or$cond3) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ($13|0)!=($15|0);
 if ($16) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($17|0)<($19|0);
  if (!($20)) {
   label = 14;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + (($21*20)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pExprA>>2] = $26;
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + (($27*20)|0)|0);
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$pExprB>>2] = $32;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($33*20)|0)|0);
  $38 = ((($37)) + 12|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 4|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (($44) + (($41*20)|0)|0);
  $46 = ((($45)) + 12|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = ($40|0)!=($48|0);
  if ($49) {
   label = 10;
   break;
  }
  $50 = HEAP32[$pExprA>>2]|0;
  $51 = HEAP32[$pExprB>>2]|0;
  $52 = HEAP32[$3>>2]|0;
  $53 = (_sqlite3ExprCompare($50,$51,$52)|0);
  $54 = ($53|0)!=(0);
  if ($54) {
   label = 12;
   break;
  }
  $55 = HEAP32[$i>>2]|0;
  $56 = (($55) + 1)|0;
  HEAP32[$i>>2] = $56;
 }
 if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return (0)|0;
}
function _exprDup($db,$p,$dupFlags,$pzBuffer) {
 $db = $db|0;
 $p = $p|0;
 $dupFlags = $dupFlags|0;
 $pzBuffer = $pzBuffer|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nNewSize = 0;
 var $nSize = 0, $nStructSize = 0, $nToken = 0, $pNew = 0, $staticFlag = 0, $zAlloc = 0, $zToken = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pNew = sp + 28|0;
 $zAlloc = sp + 24|0;
 $staticFlag = sp + 20|0;
 $nStructSize = sp + 16|0;
 $nNewSize = sp + 12|0;
 $nToken = sp + 8|0;
 $nSize = sp + 4|0;
 $zToken = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $dupFlags;
 HEAP32[$3>>2] = $pzBuffer;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$3>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$zAlloc>>2] = $7;
  HEAP32[$staticFlag>>2] = 32768;
 } else {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = (_dupedExprSize($9,$10)|0);
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = (_sqlite3DbMallocRawNN($8,$11,$13)|0);
  HEAP32[$zAlloc>>2] = $14;
  HEAP32[$staticFlag>>2] = 0;
 }
 $15 = HEAP32[$zAlloc>>2]|0;
 HEAP32[$pNew>>2] = $15;
 $16 = HEAP32[$pNew>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $178 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($178|0);
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_dupedExprStructSize($18,$19)|0);
 HEAP32[$nStructSize>>2] = $20;
 $21 = HEAP32[$nStructSize>>2]|0;
 $22 = $21 & 4095;
 HEAP32[$nNewSize>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $25 & 1024;
 $27 = ($26|0)!=(0);
 if ($27) {
  label = 8;
 } else {
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 8|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)!=(0|0);
  if ($31) {
   $32 = HEAP32[$1>>2]|0;
   $33 = ((($32)) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (_sqlite3Strlen30($34)|0);
   $36 = (($35) + 1)|0;
   HEAP32[$nToken>>2] = $36;
  } else {
   label = 8;
  }
 }
 if ((label|0) == 8) {
  HEAP32[$nToken>>2] = 0;
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = ($37|0)!=(0);
 if ($38) {
  $39 = HEAP32[$zAlloc>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$nNewSize>>2]|0;
  _memcpy(($39|0),($40|0),($41|0))|0;
 } else {
  $42 = HEAP32[$1>>2]|0;
  $43 = (_exprStructSize($42)|0);
  HEAP32[$nSize>>2] = $43;
  $44 = HEAP32[$zAlloc>>2]|0;
  $45 = HEAP32[$1>>2]|0;
  $46 = HEAP32[$nSize>>2]|0;
  _memcpy(($44|0),($45|0),($46|0))|0;
  $47 = HEAP32[$nSize>>2]|0;
  $48 = ($47>>>0)<(48);
  if ($48) {
   $49 = HEAP32[$nSize>>2]|0;
   $50 = HEAP32[$zAlloc>>2]|0;
   $51 = (($50) + ($49)|0);
   $52 = HEAP32[$nSize>>2]|0;
   $53 = (48 - ($52))|0;
   _memset(($51|0),0,($53|0))|0;
  }
 }
 $54 = HEAP32[$pNew>>2]|0;
 $55 = ((($54)) + 4|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = $56 & -122881;
 HEAP32[$55>>2] = $57;
 $58 = HEAP32[$nStructSize>>2]|0;
 $59 = $58 & 24576;
 $60 = HEAP32[$pNew>>2]|0;
 $61 = ((($60)) + 4|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = $62 | $59;
 HEAP32[$61>>2] = $63;
 $64 = HEAP32[$staticFlag>>2]|0;
 $65 = HEAP32[$pNew>>2]|0;
 $66 = ((($65)) + 4|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = $67 | $64;
 HEAP32[$66>>2] = $68;
 $69 = HEAP32[$nToken>>2]|0;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$nNewSize>>2]|0;
  $72 = HEAP32[$zAlloc>>2]|0;
  $73 = (($72) + ($71)|0);
  $74 = HEAP32[$pNew>>2]|0;
  $75 = ((($74)) + 8|0);
  HEAP32[$75>>2] = $73;
  HEAP32[$zToken>>2] = $73;
  $76 = HEAP32[$zToken>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 8|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = HEAP32[$nToken>>2]|0;
  _memcpy(($76|0),($79|0),($80|0))|0;
 }
 $81 = HEAP32[$1>>2]|0;
 $82 = ((($81)) + 4|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = HEAP32[$pNew>>2]|0;
 $85 = ((($84)) + 4|0);
 $86 = HEAP32[$85>>2]|0;
 $87 = $83 | $86;
 $88 = $87 & 8404992;
 $89 = (0)==($88|0);
 do {
  if ($89) {
   $90 = HEAP32[$1>>2]|0;
   $91 = ((($90)) + 4|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = $92 & 2048;
   $94 = ($93|0)!=(0);
   $95 = HEAP32[$0>>2]|0;
   $96 = HEAP32[$1>>2]|0;
   $97 = ((($96)) + 20|0);
   if ($94) {
    $98 = HEAP32[$97>>2]|0;
    $99 = HEAP32[$2>>2]|0;
    $100 = (_sqlite3SelectDup($95,$98,$99)|0);
    $101 = HEAP32[$pNew>>2]|0;
    $102 = ((($101)) + 20|0);
    HEAP32[$102>>2] = $100;
    break;
   } else {
    $103 = HEAP32[$97>>2]|0;
    $104 = HEAP32[$2>>2]|0;
    $105 = (_sqlite3ExprListDup($95,$103,$104)|0);
    $106 = HEAP32[$pNew>>2]|0;
    $107 = ((($106)) + 20|0);
    HEAP32[$107>>2] = $105;
    break;
   }
  }
 } while(0);
 $108 = HEAP32[$pNew>>2]|0;
 $109 = ((($108)) + 4|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = $110 & 24576;
 $112 = ($111|0)!=(0);
 $113 = HEAP32[$1>>2]|0;
 if (!($112)) {
  $151 = ((($113)) + 4|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = $152 & 8404992;
  $154 = ($153|0)!=(0);
  if ($154) {
   $178 = HEAP32[$pNew>>2]|0;
   STACKTOP = sp;return ($178|0);
  }
  $155 = HEAP32[$pNew>>2]|0;
  $156 = HEAP8[$155>>0]|0;
  $157 = $156&255;
  $158 = ($157|0)==(159);
  if ($158) {
   $159 = HEAP32[$1>>2]|0;
   $160 = ((($159)) + 12|0);
   $161 = HEAP32[$160>>2]|0;
   $162 = HEAP32[$pNew>>2]|0;
   $163 = ((($162)) + 12|0);
   HEAP32[$163>>2] = $161;
  } else {
   $164 = HEAP32[$0>>2]|0;
   $165 = HEAP32[$1>>2]|0;
   $166 = ((($165)) + 12|0);
   $167 = HEAP32[$166>>2]|0;
   $168 = (_sqlite3ExprDup($164,$167,0)|0);
   $169 = HEAP32[$pNew>>2]|0;
   $170 = ((($169)) + 12|0);
   HEAP32[$170>>2] = $168;
  }
  $171 = HEAP32[$0>>2]|0;
  $172 = HEAP32[$1>>2]|0;
  $173 = ((($172)) + 16|0);
  $174 = HEAP32[$173>>2]|0;
  $175 = (_sqlite3ExprDup($171,$174,0)|0);
  $176 = HEAP32[$pNew>>2]|0;
  $177 = ((($176)) + 16|0);
  HEAP32[$177>>2] = $175;
  $178 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($178|0);
 }
 $114 = HEAP32[$2>>2]|0;
 $115 = (_dupedExprNodeSize($113,$114)|0);
 $116 = HEAP32[$zAlloc>>2]|0;
 $117 = (($116) + ($115)|0);
 HEAP32[$zAlloc>>2] = $117;
 $118 = HEAP32[$pNew>>2]|0;
 $119 = ((($118)) + 4|0);
 $120 = HEAP32[$119>>2]|0;
 $121 = $120 & 8404992;
 $122 = ($121|0)!=(0);
 if (!($122)) {
  $123 = HEAP32[$1>>2]|0;
  $124 = ((($123)) + 12|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = ($125|0)!=(0|0);
  if ($126) {
   $127 = HEAP32[$0>>2]|0;
   $128 = HEAP32[$1>>2]|0;
   $129 = ((($128)) + 12|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = (_exprDup($127,$130,1,$zAlloc)|0);
   $134 = $131;
  } else {
   $134 = 0;
  }
  $132 = HEAP32[$pNew>>2]|0;
  $133 = ((($132)) + 12|0);
  HEAP32[$133>>2] = $134;
  $135 = HEAP32[$1>>2]|0;
  $136 = ((($135)) + 16|0);
  $137 = HEAP32[$136>>2]|0;
  $138 = ($137|0)!=(0|0);
  if ($138) {
   $139 = HEAP32[$0>>2]|0;
   $140 = HEAP32[$1>>2]|0;
   $141 = ((($140)) + 16|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = (_exprDup($139,$142,1,$zAlloc)|0);
   $146 = $143;
  } else {
   $146 = 0;
  }
  $144 = HEAP32[$pNew>>2]|0;
  $145 = ((($144)) + 16|0);
  HEAP32[$145>>2] = $146;
 }
 $147 = HEAP32[$3>>2]|0;
 $148 = ($147|0)!=(0|0);
 if (!($148)) {
  $178 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($178|0);
 }
 $149 = HEAP32[$zAlloc>>2]|0;
 $150 = HEAP32[$3>>2]|0;
 HEAP32[$150>>2] = $149;
 $178 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($178|0);
}
function _dupedExprSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 HEAP32[$nByte>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $23 = HEAP32[$nByte>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_dupedExprNodeSize($4,$5)|0);
 HEAP32[$nByte>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = $7 & 1;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  $23 = HEAP32[$nByte>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_dupedExprSize($12,$13)|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = (_dupedExprSize($17,$18)|0);
 $20 = (($14) + ($19))|0;
 $21 = HEAP32[$nByte>>2]|0;
 $22 = (($21) + ($20))|0;
 HEAP32[$nByte>>2] = $22;
 $23 = HEAP32[$nByte>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _dupedExprStructSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nSize = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$1>>2]|0;
 $3 = (0)==($2|0);
 do {
  if ($3) {
   HEAP32[$nSize>>2] = 48;
  } else {
   $4 = HEAP32[$0>>2]|0;
   $5 = ((($4)) + 12|0);
   $6 = HEAP32[$5>>2]|0;
   $7 = ($6|0)!=(0|0);
   if (!($7)) {
    $8 = HEAP32[$0>>2]|0;
    $9 = ((($8)) + 20|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = ($10|0)!=(0|0);
    if (!($11)) {
     HEAP32[$nSize>>2] = 16396;
     break;
    }
   }
   HEAP32[$nSize>>2] = 8220;
  }
 } while(0);
 $12 = HEAP32[$nSize>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _exprStructSize($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 16384;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 12;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = $9 & 8192;
   $11 = ($10|0)!=(0);
   if ($11) {
    HEAP32[$0>>2] = 28;
    break;
   } else {
    HEAP32[$0>>2] = 48;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3ExprListDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $i = 0, $pItem = 0, $pNew = 0, $pOldExpr = 0;
 var $pOldItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pNew = sp + 16|0;
 $pItem = sp + 12|0;
 $pOldItem = sp + 8|0;
 $i = sp + 4|0;
 $pOldExpr = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3DbMallocRawNN($6,8,0)|0);
 HEAP32[$pNew>>2] = $7;
 $8 = HEAP32[$pNew>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$i>>2] = $11;
 $12 = HEAP32[$pNew>>2]|0;
 HEAP32[$12>>2] = $11;
 $13 = HEAP32[$3>>2]|0;
 $14 = $13 & 1;
 $15 = ($14|0)==(0);
 L9: do {
  if ($15) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $16 = HEAP32[$i>>2]|0;
    $17 = HEAP32[$2>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = ($16|0)<($18|0);
    if (!($19)) {
     break L9;
    }
    $20 = HEAP32[$i>>2]|0;
    $21 = HEAP32[$i>>2]|0;
    $22 = (($21) + ($20))|0;
    HEAP32[$i>>2] = $22;
   }
  }
 } while(0);
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$i>>2]|0;
 $25 = ($24*20)|0;
 $26 = (_sqlite3DbMallocRawNN($23,$25,0)|0);
 HEAP32[$pItem>>2] = $26;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = ((($27)) + 4|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pItem>>2]|0;
 $30 = ($29|0)==(0|0);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  _sqlite3DbFree($31,$32);
  HEAP32[$0>>2] = 0;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 4|0);
 $35 = HEAP32[$34>>2]|0;
 HEAP32[$pOldItem>>2] = $35;
 HEAP32[$i>>2] = 0;
 while(1) {
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  $39 = ($36|0)<($38|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$pOldItem>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  HEAP32[$pOldExpr>>2] = $41;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pOldExpr>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  $45 = (_sqlite3ExprDup($42,$43,$44)|0);
  $46 = HEAP32[$pItem>>2]|0;
  HEAP32[$46>>2] = $45;
  $47 = HEAP32[$1>>2]|0;
  $48 = HEAP32[$pOldItem>>2]|0;
  $49 = ((($48)) + 4|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (_sqlite3DbStrDup($47,$50)|0);
  $52 = HEAP32[$pItem>>2]|0;
  $53 = ((($52)) + 4|0);
  HEAP32[$53>>2] = $51;
  $54 = HEAP32[$1>>2]|0;
  $55 = HEAP32[$pOldItem>>2]|0;
  $56 = ((($55)) + 8|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (_sqlite3DbStrDup($54,$57)|0);
  $59 = HEAP32[$pItem>>2]|0;
  $60 = ((($59)) + 8|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$pOldItem>>2]|0;
  $62 = ((($61)) + 12|0);
  $63 = HEAP8[$62>>0]|0;
  $64 = HEAP32[$pItem>>2]|0;
  $65 = ((($64)) + 12|0);
  HEAP8[$65>>0] = $63;
  $66 = HEAP32[$pItem>>2]|0;
  $67 = ((($66)) + 13|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 & -2;
  HEAP8[$67>>0] = $69;
  $70 = HEAP32[$pOldItem>>2]|0;
  $71 = ((($70)) + 13|0);
  $72 = HEAP8[$71>>0]|0;
  $73 = ($72&255) >>> 1;
  $74 = $73 & 1;
  $75 = $74&255;
  $76 = HEAP32[$pItem>>2]|0;
  $77 = ((($76)) + 13|0);
  $78 = $75&255;
  $79 = HEAP8[$77>>0]|0;
  $80 = $78 & 1;
  $81 = ($80 << 1)&255;
  $82 = $79 & -3;
  $83 = $82 | $81;
  HEAP8[$77>>0] = $83;
  $84 = HEAP32[$pItem>>2]|0;
  $85 = ((($84)) + 16|0);
  $86 = HEAP32[$pOldItem>>2]|0;
  $87 = ((($86)) + 16|0);
  ;HEAP32[$85>>2]=HEAP32[$87>>2]|0;
  $88 = HEAP32[$i>>2]|0;
  $89 = (($88) + 1)|0;
  HEAP32[$i>>2] = $89;
  $90 = HEAP32[$pItem>>2]|0;
  $91 = ((($90)) + 20|0);
  HEAP32[$pItem>>2] = $91;
  $92 = HEAP32[$pOldItem>>2]|0;
  $93 = ((($92)) + 20|0);
  HEAP32[$pOldItem>>2] = $93;
 }
 $94 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $94;
 $95 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($95|0);
}
function _dupedExprNodeSize($p,$flags) {
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nByte = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_dupedExprStructSize($2,$3)|0);
 $5 = $4 & 4095;
 HEAP32[$nByte>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 1024;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_sqlite3Strlen30($17)|0);
   $19 = (($18) + 1)|0;
   $20 = HEAP32[$nByte>>2]|0;
   $21 = (($20) + ($19))|0;
   HEAP32[$nByte>>2] = $21;
  }
 }
 $22 = HEAP32[$nByte>>2]|0;
 $23 = (($22) + 7)|0;
 $24 = $23 & -8;
 STACKTOP = sp;return ($24|0);
}
function _selectOpName($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 switch ($1|0) {
 case 116:  {
  HEAP32[$z>>2] = 29449;
  break;
 }
 case 118:  {
  HEAP32[$z>>2] = 29459;
  break;
 }
 case 117:  {
  HEAP32[$z>>2] = 29469;
  break;
 }
 default: {
  HEAP32[$z>>2] = 29476;
 }
 }
 $2 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ResolveOrderGroupBy($pParse,$pSelect,$pOrderBy,$zType) {
 $pParse = $pParse|0;
 $pSelect = $pSelect|0;
 $pOrderBy = $pOrderBy|0;
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pEList = 0, $pItem = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $i = sp + 16|0;
 $db = sp + 12|0;
 $pEList = sp + 8|0;
 $pItem = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $pOrderBy;
 HEAP32[$4>>2] = $zType;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)==(0|0);
 if (!($8)) {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 69|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if (!($14)) {
   $15 = HEAP32[$3>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$db>>2]|0;
   $18 = ((($17)) + 96|0);
   $19 = ((($18)) + 8|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($16|0)>($20|0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = HEAP32[$4>>2]|0;
    HEAP32[$vararg_buffer>>2] = $23;
    _sqlite3ErrorMsg($22,29482,$vararg_buffer);
    HEAP32[$0>>2] = 1;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$pEList>>2] = $25;
   HEAP32[$i>>2] = 0;
   $26 = HEAP32[$3>>2]|0;
   $27 = ((($26)) + 4|0);
   $28 = HEAP32[$27>>2]|0;
   HEAP32[$pItem>>2] = $28;
   while(1) {
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$3>>2]|0;
    $31 = HEAP32[$30>>2]|0;
    $32 = ($29|0)<($31|0);
    if (!($32)) {
     label = 13;
     break;
    }
    $33 = HEAP32[$pItem>>2]|0;
    $34 = ((($33)) + 16|0);
    $35 = HEAP16[$34>>1]|0;
    $36 = ($35<<16>>16)!=(0);
    if ($36) {
     $37 = HEAP32[$pItem>>2]|0;
     $38 = ((($37)) + 16|0);
     $39 = HEAP16[$38>>1]|0;
     $40 = $39&65535;
     $41 = HEAP32[$pEList>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     $43 = ($40|0)>($42|0);
     $44 = HEAP32[$1>>2]|0;
     if ($43) {
      label = 10;
      break;
     }
     $50 = HEAP32[$pEList>>2]|0;
     $51 = HEAP32[$pItem>>2]|0;
     $52 = ((($51)) + 16|0);
     $53 = HEAP16[$52>>1]|0;
     $54 = $53&65535;
     $55 = (($54) - 1)|0;
     $56 = HEAP32[$pItem>>2]|0;
     $57 = HEAP32[$56>>2]|0;
     $58 = HEAP32[$4>>2]|0;
     _resolveAlias($44,$50,$55,$57,$58,0);
    }
    $59 = HEAP32[$i>>2]|0;
    $60 = (($59) + 1)|0;
    HEAP32[$i>>2] = $60;
    $61 = HEAP32[$pItem>>2]|0;
    $62 = ((($61)) + 20|0);
    HEAP32[$pItem>>2] = $62;
   }
   if ((label|0) == 10) {
    $45 = HEAP32[$4>>2]|0;
    $46 = HEAP32[$i>>2]|0;
    $47 = (($46) + 1)|0;
    $48 = HEAP32[$pEList>>2]|0;
    $49 = HEAP32[$48>>2]|0;
    _resolveOutOfRangeError($44,$45,$47,$49);
    HEAP32[$0>>2] = 1;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
   else if ((label|0) == 13) {
    HEAP32[$0>>2] = 0;
    $63 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($63|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $63 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _resolveAlias($pParse,$pEList,$iCol,$pExpr,$zType,$nSubquery) {
 $pParse = $pParse|0;
 $pEList = $pEList|0;
 $iCol = $iCol|0;
 $pExpr = $pExpr|0;
 $zType = $zType|0;
 $nSubquery = $nSubquery|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pDup = 0, $pOrig = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $pOrig = sp + 8|0;
 $pDup = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pEList;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$3>>2] = $pExpr;
 HEAP32[$4>>2] = $zType;
 HEAP32[$5>>2] = $nSubquery;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($6*20)|0)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pOrig>>2] = $11;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$db>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = HEAP32[$pOrig>>2]|0;
 $16 = (_sqlite3ExprDup($14,$15,0)|0);
 HEAP32[$pDup>>2] = $16;
 $17 = HEAP32[$pDup>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP8[$19>>0]|0;
 $21 = $20 << 24 >> 24;
 $22 = ($21|0)!=(71);
 if ($22) {
  $23 = HEAP32[$pDup>>2]|0;
  $24 = HEAP32[$5>>2]|0;
  _incrAggFunctionDepth($23,$24);
 }
 $25 = HEAP32[$3>>2]|0;
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = ($27|0)==(53);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$pDup>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (_sqlite3ExprAddCollateString($29,$30,$33)|0);
  HEAP32[$pDup>>2] = $34;
 }
 $35 = HEAP32[$pDup>>2]|0;
 $36 = ((($35)) + 4|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = $37 | 4194304;
 HEAP32[$36>>2] = $38;
 $39 = HEAP32[$3>>2]|0;
 $40 = ((($39)) + 4|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $41 | 32768;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$db>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($43,$44);
 $45 = HEAP32[$3>>2]|0;
 $46 = HEAP32[$pDup>>2]|0;
 dest=$45; src=$46; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $47 = HEAP32[$3>>2]|0;
 $48 = ((($47)) + 4|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $49 & 1024;
 $51 = ($50|0)!=(0);
 if (!($51)) {
  $52 = HEAP32[$3>>2]|0;
  $53 = ((($52)) + 8|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0|0);
  if ($55) {
   $56 = HEAP32[$db>>2]|0;
   $57 = HEAP32[$3>>2]|0;
   $58 = ((($57)) + 8|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3DbStrDup($56,$59)|0);
   $61 = HEAP32[$3>>2]|0;
   $62 = ((($61)) + 8|0);
   HEAP32[$62>>2] = $60;
   $63 = HEAP32[$3>>2]|0;
   $64 = ((($63)) + 4|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = $65 | 65536;
   HEAP32[$64>>2] = $66;
  }
 }
 $67 = HEAP32[$db>>2]|0;
 $68 = HEAP32[$pDup>>2]|0;
 _sqlite3DbFree($67,$68);
 STACKTOP = sp;return;
}
function _incrAggFunctionDepth($pExpr,$N) {
 $pExpr = $pExpr|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $4 = ((($w)) + 4|0);
 HEAP32[$4>>2] = 195;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($w)) + 24|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$7)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprAddCollateString($pParse,$pExpr,$zC) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $zC = $zC|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $s = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $s = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $zC;
 $3 = HEAP32[$2>>2]|0;
 _sqlite3TokenInit($s,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3ExprAddCollateToken($4,$5,$s,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3TokenInit($p,$z) {
 $p = $p|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$3>>2] = $2;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $5;
 STACKTOP = sp;return;
}
function _incrAggDepth($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(153);
 if (!($5)) {
  STACKTOP = sp;return 0;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 38|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = (($12) + ($8))|0;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 STACKTOP = sp;return 0;
}
function _sqlite3ExprCheckHeight($pParse,$nHeight) {
 $pParse = $pParse|0;
 $nHeight = $nHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mxHeight = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $mxHeight = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $nHeight;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 96|0);
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$mxHeight>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$mxHeight>>2]|0;
 $9 = ($7|0)>($8|0);
 if (!($9)) {
  $12 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($12|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$mxHeight>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 _sqlite3ErrorMsg($10,29513,$vararg_buffer);
 HEAP32[$rc>>2] = 1;
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _lookupName($pParse,$zDb,$zTab,$zCol,$pNC,$pExpr) {
 $pParse = $pParse|0;
 $zDb = $zDb|0;
 $zTab = $zTab|0;
 $zCol = $zCol|0;
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $cnt = 0, $cntTab = 0, $db = 0, $hit = 0, $i = 0, $iCol = 0, $isTrigger = 0, $j = 0, $n = 0, $nSubquery = 0, $op = 0, $or$cond = 0, $or$cond11 = 0, $or$cond13 = 0, $or$cond15 = 0, $or$cond17 = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0;
 var $pCol = 0, $pEList = 0, $pItem = 0, $pMatch = 0, $pOrig = 0, $pSchema = 0, $pSrcList = 0, $pTab = 0, $pTopNC = 0, $vararg_buffer = 0, $vararg_buffer18 = 0, $vararg_buffer24 = 0, $vararg_buffer29 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr23 = 0, $vararg_ptr27 = 0, $vararg_ptr28 = 0, $vararg_ptr32 = 0, $zAs = 0;
 var $zErr = 0, $zTabName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer29 = sp + 40|0;
 $vararg_buffer24 = sp + 24|0;
 $vararg_buffer18 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 164|0;
 $1 = sp + 160|0;
 $2 = sp + 156|0;
 $3 = sp + 152|0;
 $4 = sp + 148|0;
 $5 = sp + 144|0;
 $6 = sp + 140|0;
 $i = sp + 136|0;
 $j = sp + 132|0;
 $cnt = sp + 128|0;
 $cntTab = sp + 124|0;
 $nSubquery = sp + 120|0;
 $db = sp + 116|0;
 $pItem = sp + 112|0;
 $pMatch = sp + 108|0;
 $pTopNC = sp + 104|0;
 $pSchema = sp + 100|0;
 $isTrigger = sp + 96|0;
 $pTab = sp + 92|0;
 $pCol = sp + 88|0;
 $pEList = sp + 84|0;
 $pSrcList = sp + 80|0;
 $hit = sp + 76|0;
 $zTabName = sp + 72|0;
 $op = sp + 68|0;
 $iCol = sp + 64|0;
 $zAs = sp + 60|0;
 $pOrig = sp + 56|0;
 $zErr = sp + 52|0;
 $n = sp + 48|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zDb;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$4>>2] = $zCol;
 HEAP32[$5>>2] = $pNC;
 HEAP32[$6>>2] = $pExpr;
 HEAP32[$cnt>>2] = 0;
 HEAP32[$cntTab>>2] = 0;
 HEAP32[$nSubquery>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 HEAP32[$pMatch>>2] = 0;
 $9 = HEAP32[$5>>2]|0;
 HEAP32[$pTopNC>>2] = $9;
 HEAP32[$pSchema>>2] = 0;
 HEAP32[$isTrigger>>2] = 0;
 HEAP32[$pTab>>2] = 0;
 $10 = HEAP32[$6>>2]|0;
 $11 = ((($10)) + 28|0);
 HEAP32[$11>>2] = -1;
 $12 = HEAP32[$6>>2]|0;
 $13 = ((($12)) + 44|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ($14|0)!=(0|0);
 L1: do {
  if ($15) {
   $16 = HEAP32[$5>>2]|0;
   $17 = ((($16)) + 28|0);
   $18 = HEAP16[$17>>1]|0;
   $19 = $18&65535;
   $20 = $19 & 6;
   $21 = ($20|0)!=(0);
   if ($21) {
    HEAP32[$2>>2] = 0;
    break;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $22 = HEAP32[$i>>2]|0;
    $23 = HEAP32[$db>>2]|0;
    $24 = ((($23)) + 20|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ($22|0)<($25|0);
    if (!($26)) {
     break L1;
    }
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$db>>2]|0;
    $29 = ((($28)) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<4)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = (_sqlite3StrICmp($32,$33)|0);
    $35 = ($34|0)==(0);
    $36 = HEAP32[$i>>2]|0;
    if ($35) {
     break;
    }
    $43 = (($36) + 1)|0;
    HEAP32[$i>>2] = $43;
   }
   $37 = HEAP32[$db>>2]|0;
   $38 = ((($37)) + 16|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($36<<4)|0);
   $41 = ((($40)) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   HEAP32[$pSchema>>2] = $42;
  }
 } while(0);
 L11: while(1) {
  $44 = HEAP32[$5>>2]|0;
  $45 = ($44|0)!=(0|0);
  $46 = HEAP32[$cnt>>2]|0;
  $47 = ($46|0)==(0);
  $48 = $45 ? $47 : 0;
  if (!($48)) {
   label = 89;
   break;
  }
  $49 = HEAP32[$5>>2]|0;
  $50 = ((($49)) + 4|0);
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$pSrcList>>2] = $51;
  $52 = HEAP32[$pSrcList>>2]|0;
  $53 = ($52|0)!=(0|0);
  if ($53) {
   HEAP32[$i>>2] = 0;
   $54 = HEAP32[$pSrcList>>2]|0;
   $55 = ((($54)) + 8|0);
   HEAP32[$pItem>>2] = $55;
   while(1) {
    $56 = HEAP32[$i>>2]|0;
    $57 = HEAP32[$pSrcList>>2]|0;
    $58 = HEAP32[$57>>2]|0;
    $59 = ($56|0)<($58|0);
    if (!($59)) {
     break;
    }
    $60 = HEAP32[$pItem>>2]|0;
    $61 = ((($60)) + 16|0);
    $62 = HEAP32[$61>>2]|0;
    HEAP32[$pTab>>2] = $62;
    $63 = HEAP32[$pItem>>2]|0;
    $64 = ((($63)) + 20|0);
    $65 = HEAP32[$64>>2]|0;
    $66 = ($65|0)!=(0|0);
    if ($66) {
     $67 = HEAP32[$pItem>>2]|0;
     $68 = ((($67)) + 20|0);
     $69 = HEAP32[$68>>2]|0;
     $70 = ((($69)) + 8|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = $71 & 2048;
     $73 = ($72|0)!=(0);
     if ($73) {
      HEAP32[$hit>>2] = 0;
      $74 = HEAP32[$pItem>>2]|0;
      $75 = ((($74)) + 20|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = HEAP32[$76>>2]|0;
      HEAP32[$pEList>>2] = $77;
      HEAP32[$j>>2] = 0;
      while(1) {
       $78 = HEAP32[$j>>2]|0;
       $79 = HEAP32[$pEList>>2]|0;
       $80 = HEAP32[$79>>2]|0;
       $81 = ($78|0)<($80|0);
       if (!($81)) {
        break;
       }
       $82 = HEAP32[$j>>2]|0;
       $83 = HEAP32[$pEList>>2]|0;
       $84 = ((($83)) + 4|0);
       $85 = HEAP32[$84>>2]|0;
       $86 = (($85) + (($82*20)|0)|0);
       $87 = ((($86)) + 8|0);
       $88 = HEAP32[$87>>2]|0;
       $89 = HEAP32[$4>>2]|0;
       $90 = HEAP32[$3>>2]|0;
       $91 = HEAP32[$2>>2]|0;
       $92 = (_sqlite3MatchSpanName($88,$89,$90,$91)|0);
       $93 = ($92|0)!=(0);
       if ($93) {
        $94 = HEAP32[$cnt>>2]|0;
        $95 = (($94) + 1)|0;
        HEAP32[$cnt>>2] = $95;
        HEAP32[$cntTab>>2] = 2;
        $96 = HEAP32[$pItem>>2]|0;
        HEAP32[$pMatch>>2] = $96;
        $97 = HEAP32[$j>>2]|0;
        $98 = $97&65535;
        $99 = HEAP32[$6>>2]|0;
        $100 = ((($99)) + 32|0);
        HEAP16[$100>>1] = $98;
        HEAP32[$hit>>2] = 1;
       }
       $101 = HEAP32[$j>>2]|0;
       $102 = (($101) + 1)|0;
       HEAP32[$j>>2] = $102;
      }
      $103 = HEAP32[$hit>>2]|0;
      $104 = ($103|0)!=(0);
      $105 = HEAP32[$3>>2]|0;
      $106 = ($105|0)==(0|0);
      $or$cond = $104 | $106;
      if (!($or$cond)) {
       label = 21;
      }
     } else {
      label = 21;
     }
    } else {
     label = 21;
    }
    L29: do {
     if ((label|0) == 21) {
      label = 0;
      $107 = HEAP32[$2>>2]|0;
      $108 = ($107|0)!=(0|0);
      if ($108) {
       $109 = HEAP32[$pTab>>2]|0;
       $110 = ((($109)) + 64|0);
       $111 = HEAP32[$110>>2]|0;
       $112 = HEAP32[$pSchema>>2]|0;
       $113 = ($111|0)!=($112|0);
       if ($113) {
        break;
       }
      }
      $114 = HEAP32[$3>>2]|0;
      $115 = ($114|0)!=(0|0);
      if ($115) {
       $116 = HEAP32[$pItem>>2]|0;
       $117 = ((($116)) + 12|0);
       $118 = HEAP32[$117>>2]|0;
       $119 = ($118|0)!=(0|0);
       if ($119) {
        $120 = HEAP32[$pItem>>2]|0;
        $121 = ((($120)) + 12|0);
        $122 = HEAP32[$121>>2]|0;
        $125 = $122;
       } else {
        $123 = HEAP32[$pTab>>2]|0;
        $124 = HEAP32[$123>>2]|0;
        $125 = $124;
       }
       HEAP32[$zTabName>>2] = $125;
       $126 = HEAP32[$zTabName>>2]|0;
       $127 = HEAP32[$3>>2]|0;
       $128 = (_sqlite3StrICmp($126,$127)|0);
       $129 = ($128|0)!=(0);
       if ($129) {
        break;
       }
      }
      $130 = HEAP32[$cntTab>>2]|0;
      $131 = (($130) + 1)|0;
      HEAP32[$cntTab>>2] = $131;
      $132 = (0)==($130|0);
      if ($132) {
       $133 = HEAP32[$pItem>>2]|0;
       HEAP32[$pMatch>>2] = $133;
      }
      HEAP32[$j>>2] = 0;
      $134 = HEAP32[$pTab>>2]|0;
      $135 = ((($134)) + 4|0);
      $136 = HEAP32[$135>>2]|0;
      HEAP32[$pCol>>2] = $136;
      while(1) {
       $137 = HEAP32[$j>>2]|0;
       $138 = HEAP32[$pTab>>2]|0;
       $139 = ((($138)) + 34|0);
       $140 = HEAP16[$139>>1]|0;
       $141 = $140 << 16 >> 16;
       $142 = ($137|0)<($141|0);
       if (!($142)) {
        break L29;
       }
       $143 = HEAP32[$pCol>>2]|0;
       $144 = HEAP32[$143>>2]|0;
       $145 = HEAP32[$4>>2]|0;
       $146 = (_sqlite3StrICmp($144,$145)|0);
       $147 = ($146|0)==(0);
       if ($147) {
        $148 = HEAP32[$cnt>>2]|0;
        $149 = ($148|0)==(1);
        if (!($149)) {
         break;
        }
        $150 = HEAP32[$pItem>>2]|0;
        $151 = ((($150)) + 36|0);
        $152 = HEAP8[$151>>0]|0;
        $153 = $152&255;
        $154 = $153 & 4;
        $155 = ($154|0)!=(0);
        if (!($155)) {
         $156 = HEAP32[$pItem>>2]|0;
         $157 = ((($156)) + 52|0);
         $158 = HEAP32[$157>>2]|0;
         $159 = HEAP32[$4>>2]|0;
         $160 = (_nameInUsingClause($158,$159)|0);
         $161 = ($160|0)!=(0);
         if (!($161)) {
          break;
         }
        }
       }
       $178 = HEAP32[$j>>2]|0;
       $179 = (($178) + 1)|0;
       HEAP32[$j>>2] = $179;
       $180 = HEAP32[$pCol>>2]|0;
       $181 = ((($180)) + 16|0);
       HEAP32[$pCol>>2] = $181;
      }
      $162 = HEAP32[$cnt>>2]|0;
      $163 = (($162) + 1)|0;
      HEAP32[$cnt>>2] = $163;
      $164 = HEAP32[$pItem>>2]|0;
      HEAP32[$pMatch>>2] = $164;
      $165 = HEAP32[$j>>2]|0;
      $166 = HEAP32[$pTab>>2]|0;
      $167 = ((($166)) + 32|0);
      $168 = HEAP16[$167>>1]|0;
      $169 = $168 << 16 >> 16;
      $170 = ($165|0)==($169|0);
      if ($170) {
       $175 = -1;
      } else {
       $171 = HEAP32[$j>>2]|0;
       $172 = $171&65535;
       $173 = $172 << 16 >> 16;
       $175 = $173;
      }
      $174 = $175&65535;
      $176 = HEAP32[$6>>2]|0;
      $177 = ((($176)) + 32|0);
      HEAP16[$177>>1] = $174;
     }
    } while(0);
    $182 = HEAP32[$i>>2]|0;
    $183 = (($182) + 1)|0;
    HEAP32[$i>>2] = $183;
    $184 = HEAP32[$pItem>>2]|0;
    $185 = ((($184)) + 72|0);
    HEAP32[$pItem>>2] = $185;
   }
   $186 = HEAP32[$pMatch>>2]|0;
   $187 = ($186|0)!=(0|0);
   if ($187) {
    $188 = HEAP32[$pMatch>>2]|0;
    $189 = ((($188)) + 44|0);
    $190 = HEAP32[$189>>2]|0;
    $191 = HEAP32[$6>>2]|0;
    $192 = ((($191)) + 28|0);
    HEAP32[$192>>2] = $190;
    $193 = HEAP32[$pMatch>>2]|0;
    $194 = ((($193)) + 16|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = HEAP32[$6>>2]|0;
    $197 = ((($196)) + 44|0);
    HEAP32[$197>>2] = $195;
    $198 = HEAP32[$pMatch>>2]|0;
    $199 = ((($198)) + 36|0);
    $200 = HEAP8[$199>>0]|0;
    $201 = $200&255;
    $202 = $201 & 8;
    $203 = ($202|0)!=(0);
    if ($203) {
     $204 = HEAP32[$6>>2]|0;
     $205 = ((($204)) + 4|0);
     $206 = HEAP32[$205>>2]|0;
     $207 = $206 | 1048576;
     HEAP32[$205>>2] = $207;
    }
    $208 = HEAP32[$6>>2]|0;
    $209 = ((($208)) + 44|0);
    $210 = HEAP32[$209>>2]|0;
    $211 = ((($210)) + 64|0);
    $212 = HEAP32[$211>>2]|0;
    HEAP32[$pSchema>>2] = $212;
   }
  }
  $213 = HEAP32[$2>>2]|0;
  $214 = ($213|0)==(0|0);
  $215 = HEAP32[$3>>2]|0;
  $216 = ($215|0)!=(0|0);
  $or$cond3 = $214 & $216;
  $217 = HEAP32[$cntTab>>2]|0;
  $218 = ($217|0)==(0);
  $or$cond5 = $or$cond3 & $218;
  if ($or$cond5) {
   $219 = HEAP32[$1>>2]|0;
   $220 = ((($219)) + 128|0);
   $221 = HEAP32[$220>>2]|0;
   $222 = ($221|0)!=(0|0);
   if ($222) {
    $223 = HEAP32[$1>>2]|0;
    $224 = ((($223)) + 148|0);
    $225 = HEAP8[$224>>0]|0;
    $226 = $225&255;
    HEAP32[$op>>2] = $226;
    $227 = HEAP32[$op>>2]|0;
    $228 = ($227|0)!=(109);
    if ($228) {
     $229 = HEAP32[$3>>2]|0;
     $230 = (_sqlite3StrICmp(29940,$229)|0);
     $231 = ($230|0)==(0);
     if ($231) {
      $232 = HEAP32[$6>>2]|0;
      $233 = ((($232)) + 28|0);
      HEAP32[$233>>2] = 1;
      $234 = HEAP32[$1>>2]|0;
      $235 = ((($234)) + 128|0);
      $236 = HEAP32[$235>>2]|0;
      HEAP32[$pTab>>2] = $236;
     } else {
      label = 50;
     }
    } else {
     label = 50;
    }
    do {
     if ((label|0) == 50) {
      label = 0;
      $237 = HEAP32[$op>>2]|0;
      $238 = ($237|0)!=(108);
      if ($238) {
       $239 = HEAP32[$3>>2]|0;
       $240 = (_sqlite3StrICmp(29944,$239)|0);
       $241 = ($240|0)==(0);
       if ($241) {
        $242 = HEAP32[$6>>2]|0;
        $243 = ((($242)) + 28|0);
        HEAP32[$243>>2] = 0;
        $244 = HEAP32[$1>>2]|0;
        $245 = ((($244)) + 128|0);
        $246 = HEAP32[$245>>2]|0;
        HEAP32[$pTab>>2] = $246;
        break;
       }
      }
      HEAP32[$pTab>>2] = 0;
     }
    } while(0);
    $247 = HEAP32[$pTab>>2]|0;
    $248 = ($247|0)!=(0|0);
    if ($248) {
     $249 = HEAP32[$pTab>>2]|0;
     $250 = ((($249)) + 64|0);
     $251 = HEAP32[$250>>2]|0;
     HEAP32[$pSchema>>2] = $251;
     $252 = HEAP32[$cntTab>>2]|0;
     $253 = (($252) + 1)|0;
     HEAP32[$cntTab>>2] = $253;
     HEAP32[$iCol>>2] = 0;
     $254 = HEAP32[$pTab>>2]|0;
     $255 = ((($254)) + 4|0);
     $256 = HEAP32[$255>>2]|0;
     HEAP32[$pCol>>2] = $256;
     while(1) {
      $257 = HEAP32[$iCol>>2]|0;
      $258 = HEAP32[$pTab>>2]|0;
      $259 = ((($258)) + 34|0);
      $260 = HEAP16[$259>>1]|0;
      $261 = $260 << 16 >> 16;
      $262 = ($257|0)<($261|0);
      if (!($262)) {
       break;
      }
      $263 = HEAP32[$pCol>>2]|0;
      $264 = HEAP32[$263>>2]|0;
      $265 = HEAP32[$4>>2]|0;
      $266 = (_sqlite3StrICmp($264,$265)|0);
      $267 = ($266|0)==(0);
      $268 = HEAP32[$iCol>>2]|0;
      if ($267) {
       label = 58;
       break;
      }
      $274 = (($268) + 1)|0;
      HEAP32[$iCol>>2] = $274;
      $275 = HEAP32[$pCol>>2]|0;
      $276 = ((($275)) + 16|0);
      HEAP32[$pCol>>2] = $276;
     }
     if ((label|0) == 58) {
      label = 0;
      $269 = HEAP32[$pTab>>2]|0;
      $270 = ((($269)) + 32|0);
      $271 = HEAP16[$270>>1]|0;
      $272 = $271 << 16 >> 16;
      $273 = ($268|0)==($272|0);
      if ($273) {
       HEAP32[$iCol>>2] = -1;
      }
     }
     $277 = HEAP32[$iCol>>2]|0;
     $278 = HEAP32[$pTab>>2]|0;
     $279 = ((($278)) + 34|0);
     $280 = HEAP16[$279>>1]|0;
     $281 = $280 << 16 >> 16;
     $282 = ($277|0)>=($281|0);
     if ($282) {
      $283 = HEAP32[$4>>2]|0;
      $284 = (_sqlite3IsRowid($283)|0);
      $285 = ($284|0)!=(0);
      if ($285) {
       $286 = HEAP32[$pTab>>2]|0;
       $287 = ((($286)) + 42|0);
       $288 = HEAP8[$287>>0]|0;
       $289 = $288&255;
       $290 = $289 & 64;
       $291 = ($290|0)==(0);
       if ($291) {
        HEAP32[$iCol>>2] = -1;
       }
      }
     }
     $292 = HEAP32[$iCol>>2]|0;
     $293 = HEAP32[$pTab>>2]|0;
     $294 = ((($293)) + 34|0);
     $295 = HEAP16[$294>>1]|0;
     $296 = $295 << 16 >> 16;
     $297 = ($292|0)<($296|0);
     if ($297) {
      $298 = HEAP32[$cnt>>2]|0;
      $299 = (($298) + 1)|0;
      HEAP32[$cnt>>2] = $299;
      $300 = HEAP32[$iCol>>2]|0;
      $301 = ($300|0)<(0);
      $302 = HEAP32[$6>>2]|0;
      do {
       if ($301) {
        $303 = ((($302)) + 1|0);
        HEAP8[$303>>0] = 68;
       } else {
        $304 = ((($302)) + 28|0);
        $305 = HEAP32[$304>>2]|0;
        $306 = ($305|0)==(0);
        $307 = HEAP32[$iCol>>2]|0;
        $308 = ($307|0)>=(32);
        $309 = HEAP32[$iCol>>2]|0;
        $310 = 1 << $309;
        $311 = $308 ? -1 : $310;
        $312 = HEAP32[$1>>2]|0;
        if ($306) {
         $313 = ((($312)) + 140|0);
         $314 = HEAP32[$313>>2]|0;
         $315 = $314 | $311;
         HEAP32[$313>>2] = $315;
         break;
        } else {
         $316 = ((($312)) + 144|0);
         $317 = HEAP32[$316>>2]|0;
         $318 = $317 | $311;
         HEAP32[$316>>2] = $318;
         break;
        }
       }
      } while(0);
      $319 = HEAP32[$iCol>>2]|0;
      $320 = $319&65535;
      $321 = HEAP32[$6>>2]|0;
      $322 = ((($321)) + 32|0);
      HEAP16[$322>>1] = $320;
      $323 = HEAP32[$pTab>>2]|0;
      $324 = HEAP32[$6>>2]|0;
      $325 = ((($324)) + 44|0);
      HEAP32[$325>>2] = $323;
      HEAP32[$isTrigger>>2] = 1;
     }
    }
   }
  }
  $326 = HEAP32[$cnt>>2]|0;
  $327 = ($326|0)==(0);
  $328 = HEAP32[$cntTab>>2]|0;
  $329 = ($328|0)==(1);
  $or$cond7 = $327 & $329;
  $330 = HEAP32[$pMatch>>2]|0;
  $331 = ($330|0)!=(0|0);
  $or$cond9 = $or$cond7 & $331;
  if ($or$cond9) {
   $332 = HEAP32[$5>>2]|0;
   $333 = ((($332)) + 28|0);
   $334 = HEAP16[$333>>1]|0;
   $335 = $334&65535;
   $336 = $335 & 32;
   $337 = ($336|0)==(0);
   if ($337) {
    $338 = HEAP32[$4>>2]|0;
    $339 = (_sqlite3IsRowid($338)|0);
    $340 = ($339|0)!=(0);
    if ($340) {
     $341 = HEAP32[$pMatch>>2]|0;
     $342 = ((($341)) + 16|0);
     $343 = HEAP32[$342>>2]|0;
     $344 = ((($343)) + 42|0);
     $345 = HEAP8[$344>>0]|0;
     $346 = $345&255;
     $347 = $346 & 64;
     $348 = ($347|0)==(0);
     if ($348) {
      HEAP32[$cnt>>2] = 1;
      $349 = HEAP32[$6>>2]|0;
      $350 = ((($349)) + 32|0);
      HEAP16[$350>>1] = -1;
      $351 = HEAP32[$6>>2]|0;
      $352 = ((($351)) + 1|0);
      HEAP8[$352>>0] = 68;
     }
    }
   }
  }
  $353 = HEAP32[$5>>2]|0;
  $354 = ((($353)) + 8|0);
  $355 = HEAP32[$354>>2]|0;
  HEAP32[$pEList>>2] = $355;
  $356 = ($355|0)!=(0|0);
  $357 = HEAP32[$3>>2]|0;
  $358 = ($357|0)==(0|0);
  $or$cond11 = $356 & $358;
  $359 = HEAP32[$cnt>>2]|0;
  $360 = ($359|0)==(0);
  $or$cond13 = $or$cond11 & $360;
  L105: do {
   if ($or$cond13) {
    HEAP32[$j>>2] = 0;
    while(1) {
     $361 = HEAP32[$j>>2]|0;
     $362 = HEAP32[$pEList>>2]|0;
     $363 = HEAP32[$362>>2]|0;
     $364 = ($361|0)<($363|0);
     if (!($364)) {
      break L105;
     }
     $365 = HEAP32[$j>>2]|0;
     $366 = HEAP32[$pEList>>2]|0;
     $367 = ((($366)) + 4|0);
     $368 = HEAP32[$367>>2]|0;
     $369 = (($368) + (($365*20)|0)|0);
     $370 = ((($369)) + 4|0);
     $371 = HEAP32[$370>>2]|0;
     HEAP32[$zAs>>2] = $371;
     $372 = HEAP32[$zAs>>2]|0;
     $373 = ($372|0)!=(0|0);
     if ($373) {
      $374 = HEAP32[$zAs>>2]|0;
      $375 = HEAP32[$4>>2]|0;
      $376 = (_sqlite3StrICmp($374,$375)|0);
      $377 = ($376|0)==(0);
      if ($377) {
       label = 82;
       break L11;
      }
     }
     $402 = HEAP32[$j>>2]|0;
     $403 = (($402) + 1)|0;
     HEAP32[$j>>2] = $403;
    }
   }
  } while(0);
  $404 = HEAP32[$cnt>>2]|0;
  $405 = ($404|0)==(0);
  if (!($405)) {
   continue;
  }
  $406 = HEAP32[$5>>2]|0;
  $407 = ((($406)) + 16|0);
  $408 = HEAP32[$407>>2]|0;
  HEAP32[$5>>2] = $408;
  $409 = HEAP32[$nSubquery>>2]|0;
  $410 = (($409) + 1)|0;
  HEAP32[$nSubquery>>2] = $410;
 }
 if ((label|0) == 82) {
  $378 = HEAP32[$j>>2]|0;
  $379 = HEAP32[$pEList>>2]|0;
  $380 = ((($379)) + 4|0);
  $381 = HEAP32[$380>>2]|0;
  $382 = (($381) + (($378*20)|0)|0);
  $383 = HEAP32[$382>>2]|0;
  HEAP32[$pOrig>>2] = $383;
  $384 = HEAP32[$5>>2]|0;
  $385 = ((($384)) + 28|0);
  $386 = HEAP16[$385>>1]|0;
  $387 = $386&65535;
  $388 = $387 & 1;
  $389 = ($388|0)==(0);
  if ($389) {
   $390 = HEAP32[$pOrig>>2]|0;
   $391 = ((($390)) + 4|0);
   $392 = HEAP32[$391>>2]|0;
   $393 = $392 & 2;
   $394 = ($393|0)!=(0);
   if ($394) {
    $395 = HEAP32[$1>>2]|0;
    $396 = HEAP32[$zAs>>2]|0;
    HEAP32[$vararg_buffer>>2] = $396;
    _sqlite3ErrorMsg($395,29948,$vararg_buffer);
    HEAP32[$0>>2] = 2;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
  }
  $397 = HEAP32[$1>>2]|0;
  $398 = HEAP32[$pEList>>2]|0;
  $399 = HEAP32[$j>>2]|0;
  $400 = HEAP32[$6>>2]|0;
  $401 = HEAP32[$nSubquery>>2]|0;
  _resolveAlias($397,$398,$399,$400,18402,$401);
  HEAP32[$cnt>>2] = 1;
  HEAP32[$pMatch>>2] = 0;
 }
 else if ((label|0) == 89) {
  $411 = HEAP32[$cnt>>2]|0;
  $412 = ($411|0)==(0);
  $413 = HEAP32[$3>>2]|0;
  $414 = ($413|0)==(0|0);
  $or$cond15 = $412 & $414;
  if ($or$cond15) {
   $415 = HEAP32[$6>>2]|0;
   $416 = ((($415)) + 4|0);
   $417 = HEAP32[$416>>2]|0;
   $418 = $417 & 64;
   $419 = ($418|0)!=(0);
   if ($419) {
    $420 = HEAP32[$6>>2]|0;
    HEAP8[$420>>0] = 97;
    $421 = HEAP32[$6>>2]|0;
    $422 = ((($421)) + 44|0);
    HEAP32[$422>>2] = 0;
    HEAP32[$0>>2] = 1;
    $518 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($518|0);
   }
  }
  $423 = HEAP32[$cnt>>2]|0;
  $424 = ($423|0)!=(1);
  if ($424) {
   $425 = HEAP32[$cnt>>2]|0;
   $426 = ($425|0)==(0);
   $427 = $426 ? 29979 : 29994;
   HEAP32[$zErr>>2] = $427;
   $428 = HEAP32[$2>>2]|0;
   $429 = ($428|0)!=(0|0);
   do {
    if ($429) {
     $430 = HEAP32[$1>>2]|0;
     $431 = HEAP32[$zErr>>2]|0;
     $432 = HEAP32[$2>>2]|0;
     $433 = HEAP32[$3>>2]|0;
     $434 = HEAP32[$4>>2]|0;
     HEAP32[$vararg_buffer18>>2] = $431;
     $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
     HEAP32[$vararg_ptr21>>2] = $432;
     $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
     HEAP32[$vararg_ptr22>>2] = $433;
     $vararg_ptr23 = ((($vararg_buffer18)) + 12|0);
     HEAP32[$vararg_ptr23>>2] = $434;
     _sqlite3ErrorMsg($430,30016,$vararg_buffer18);
    } else {
     $435 = HEAP32[$3>>2]|0;
     $436 = ($435|0)!=(0|0);
     $437 = HEAP32[$1>>2]|0;
     $438 = HEAP32[$zErr>>2]|0;
     if ($436) {
      $439 = HEAP32[$3>>2]|0;
      $440 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer24>>2] = $438;
      $vararg_ptr27 = ((($vararg_buffer24)) + 4|0);
      HEAP32[$vararg_ptr27>>2] = $439;
      $vararg_ptr28 = ((($vararg_buffer24)) + 8|0);
      HEAP32[$vararg_ptr28>>2] = $440;
      _sqlite3ErrorMsg($437,30029,$vararg_buffer24);
      break;
     } else {
      $441 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer29>>2] = $438;
      $vararg_ptr32 = ((($vararg_buffer29)) + 4|0);
      HEAP32[$vararg_ptr32>>2] = $441;
      _sqlite3ErrorMsg($437,30039,$vararg_buffer29);
      break;
     }
    }
   } while(0);
   $442 = HEAP32[$1>>2]|0;
   $443 = ((($442)) + 17|0);
   HEAP8[$443>>0] = 1;
   $444 = HEAP32[$pTopNC>>2]|0;
   $445 = ((($444)) + 24|0);
   $446 = HEAP32[$445>>2]|0;
   $447 = (($446) + 1)|0;
   HEAP32[$445>>2] = $447;
  }
  $448 = HEAP32[$6>>2]|0;
  $449 = ((($448)) + 32|0);
  $450 = HEAP16[$449>>1]|0;
  $451 = $450 << 16 >> 16;
  $452 = ($451|0)>=(0);
  $453 = HEAP32[$pMatch>>2]|0;
  $454 = ($453|0)!=(0|0);
  $or$cond17 = $452 & $454;
  if ($or$cond17) {
   $455 = HEAP32[$6>>2]|0;
   $456 = ((($455)) + 32|0);
   $457 = HEAP16[$456>>1]|0;
   $458 = $457 << 16 >> 16;
   HEAP32[$n>>2] = $458;
   $459 = HEAP32[$n>>2]|0;
   $460 = ($459|0)>=(64);
   $$ = $460 ? 63 : $458;
   HEAP32[$n>>2] = $$;
   $461 = HEAP32[$n>>2]|0;
   $462 = (_bitshift64Shl(1,0,($461|0))|0);
   $463 = tempRet0;
   $464 = HEAP32[$pMatch>>2]|0;
   $465 = ((($464)) + 56|0);
   $466 = $465;
   $467 = $466;
   $468 = HEAP32[$467>>2]|0;
   $469 = (($466) + 4)|0;
   $470 = $469;
   $471 = HEAP32[$470>>2]|0;
   $472 = $468 | $462;
   $473 = $471 | $463;
   $474 = $465;
   $475 = $474;
   HEAP32[$475>>2] = $472;
   $476 = (($474) + 4)|0;
   $477 = $476;
   HEAP32[$477>>2] = $473;
  }
  $478 = HEAP32[$db>>2]|0;
  $479 = HEAP32[$6>>2]|0;
  $480 = ((($479)) + 12|0);
  $481 = HEAP32[$480>>2]|0;
  _sqlite3ExprDelete($478,$481);
  $482 = HEAP32[$6>>2]|0;
  $483 = ((($482)) + 12|0);
  HEAP32[$483>>2] = 0;
  $484 = HEAP32[$db>>2]|0;
  $485 = HEAP32[$6>>2]|0;
  $486 = ((($485)) + 16|0);
  $487 = HEAP32[$486>>2]|0;
  _sqlite3ExprDelete($484,$487);
  $488 = HEAP32[$6>>2]|0;
  $489 = ((($488)) + 16|0);
  HEAP32[$489>>2] = 0;
  $490 = HEAP32[$isTrigger>>2]|0;
  $491 = ($490|0)!=(0);
  $492 = $491 ? 88 : 152;
  $493 = $492&255;
  $494 = HEAP32[$6>>2]|0;
  HEAP8[$494>>0] = $493;
 }
 $495 = HEAP32[$cnt>>2]|0;
 $496 = ($495|0)==(1);
 if (!($496)) {
  HEAP32[$0>>2] = 2;
  $518 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($518|0);
 }
 $497 = HEAP32[$6>>2]|0;
 $498 = ((($497)) + 4|0);
 $499 = HEAP32[$498>>2]|0;
 $500 = $499 & 4194304;
 $501 = ($500|0)!=(0);
 if (!($501)) {
  $502 = HEAP32[$1>>2]|0;
  $503 = HEAP32[$6>>2]|0;
  $504 = HEAP32[$pSchema>>2]|0;
  $505 = HEAP32[$5>>2]|0;
  $506 = ((($505)) + 4|0);
  $507 = HEAP32[$506>>2]|0;
  _sqlite3AuthRead($502,$503,$504,$507);
 }
 while(1) {
  $508 = HEAP32[$pTopNC>>2]|0;
  $509 = ((($508)) + 20|0);
  $510 = HEAP32[$509>>2]|0;
  $511 = (($510) + 1)|0;
  HEAP32[$509>>2] = $511;
  $512 = HEAP32[$pTopNC>>2]|0;
  $513 = HEAP32[$5>>2]|0;
  $514 = ($512|0)==($513|0);
  if ($514) {
   break;
  }
  $515 = HEAP32[$pTopNC>>2]|0;
  $516 = ((($515)) + 16|0);
  $517 = HEAP32[$516>>2]|0;
  HEAP32[$pTopNC>>2] = $517;
 }
 HEAP32[$0>>2] = 1;
 $518 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($518|0);
}
function _notValid($pParse,$pNC,$zMsg,$validMask) {
 $pParse = $pParse|0;
 $pNC = $pNC|0;
 $zMsg = $zMsg|0;
 $validMask = $validMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $zIn = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pNC;
 HEAP32[$2>>2] = $zMsg;
 HEAP32[$3>>2] = $validMask;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = $7 & $8;
 $10 = ($9|0)!=(0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 HEAP32[$zIn>>2] = 29856;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 32;
 $16 = ($15|0)!=(0);
 if ($16) {
  HEAP32[$zIn>>2] = 29884;
 } else {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 28|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = $19&65535;
  $21 = $20 & 4;
  $22 = ($21|0)!=(0);
  if ($22) {
   HEAP32[$zIn>>2] = 29902;
  }
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$zIn>>2]|0;
 HEAP32[$vararg_buffer>>2] = $24;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $25;
 _sqlite3ErrorMsg($23,29920,$vararg_buffer);
 STACKTOP = sp;return;
}
function _exprProbability($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0.0, $16 = 0.0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $r = sp;
 HEAP32[$1>>2] = $p;
 HEAPF64[$r>>3] = -1.0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(132);
 if ($5) {
  HEAP32[$0>>2] = -1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3Strlen30($11)|0);
 (_sqlite3AtoF($8,$r,$12,1)|0);
 $13 = +HEAPF64[$r>>3];
 $14 = $13 > 1.0;
 if ($14) {
  HEAP32[$0>>2] = -1;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $15 = +HEAPF64[$r>>3];
  $16 = $15 * 134217728.0;
  $17 = (~~(($16)));
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _sqlite3FunctionUsesThisSrc($pExpr,$pSrcList) {
 $pExpr = $pExpr|0;
 $pSrcList = $pSrcList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $w = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $w = sp + 12|0;
 $cnt = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $pSrcList;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 4|0);
 HEAP32[$2>>2] = 196;
 $3 = ((($w)) + 24|0);
 HEAP32[$3>>2] = $cnt;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$cnt>>2] = $4;
 $5 = ((($cnt)) + 4|0);
 HEAP32[$5>>2] = 0;
 $6 = ((($cnt)) + 8|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 (_sqlite3WalkExprList($w,$9)|0);
 $10 = ((($cnt)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 $13 = ((($cnt)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)==(0);
 $16 = $12 ? 1 : $15;
 $17 = $16&1;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3ExprVectorSize($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $op = sp + 8|0;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 HEAP8[$op>>0] = $3;
 $4 = HEAP8[$op>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(157);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 38|0);
  $9 = HEAP8[$8>>0]|0;
  HEAP8[$op>>0] = $9;
 }
 $10 = HEAP8[$op>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(158);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$0>>2] = $16;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $17 = HEAP8[$op>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(119);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 20|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$0>>2] = $24;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _exprSrcCount($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nSrc = 0, $p = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $i = sp + 12|0;
 $p = sp + 8|0;
 $pSrc = sp + 4|0;
 $nSrc = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(152);
 if (!($5)) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(154);
  if (!($9)) {
   STACKTOP = sp;return 0;
  }
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pSrc>>2] = $14;
 $15 = HEAP32[$pSrc>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$pSrc>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = $18;
 } else {
  $19 = 0;
 }
 HEAP32[$nSrc>>2] = $19;
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$nSrc>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 28|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$pSrc>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = (($28) + (($26*72)|0)|0);
  $30 = ((($29)) + 44|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($25|0)==($31|0);
  if ($32) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = HEAP32[$i>>2]|0;
 $36 = HEAP32[$nSrc>>2]|0;
 $37 = ($35|0)<($36|0);
 $38 = HEAP32[$p>>2]|0;
 if ($37) {
  $39 = ((($38)) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$39>>2] = $41;
  STACKTOP = sp;return 0;
 } else {
  $42 = ((($38)) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$42>>2] = $44;
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _sqlite3AtoF($z,$pResult,$length,$enc) {
 $z = $z|0;
 $pResult = $pResult|0;
 $length = $length|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0.0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0.0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0.0, $367 = 0.0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0.0, $379 = 0.0, $38 = 0, $380 = 0.0, $381 = 0.0, $382 = 0.0, $383 = 0.0, $384 = 0.0, $385 = 0.0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0.0, $395 = 0.0, $396 = 0.0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0.0, $401 = 0.0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0.0, $407 = 0.0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0.0, $419 = 0.0, $42 = 0, $420 = 0.0, $421 = 0.0;
 var $422 = 0.0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d = 0, $e = 0, $eValid = 0, $esign = 0;
 var $i = 0, $incr = 0, $nDigits = 0, $nonNum = 0, $or$cond = 0, $or$cond10 = 0, $or$cond12 = 0, $result = 0, $s = 0, $scale = 0, $sign = 0, $zEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 80|0;
 $incr = sp + 60|0;
 $zEnd = sp + 56|0;
 $sign = sp + 52|0;
 $s = sp + 16|0;
 $d = sp + 48|0;
 $esign = sp + 44|0;
 $e = sp + 40|0;
 $eValid = sp + 36|0;
 $result = sp + 8|0;
 $nDigits = sp + 32|0;
 $nonNum = sp + 28|0;
 $i = sp + 24|0;
 $scale = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $pResult;
 HEAP32[$3>>2] = $length;
 HEAP8[$4>>0] = $enc;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (($5) + ($6)|0);
 HEAP32[$zEnd>>2] = $7;
 HEAP32[$sign>>2] = 1;
 $8 = $s;
 $9 = $8;
 HEAP32[$9>>2] = 0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = 0;
 HEAP32[$d>>2] = 0;
 HEAP32[$esign>>2] = 1;
 HEAP32[$e>>2] = 0;
 HEAP32[$eValid>>2] = 1;
 HEAP32[$nDigits>>2] = 0;
 HEAP32[$nonNum>>2] = 0;
 $12 = HEAP32[$2>>2]|0;
 HEAPF64[$12>>3] = 0.0;
 $13 = HEAP8[$4>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(1);
 if ($15) {
  HEAP32[$incr>>2] = 1;
 } else {
  HEAP32[$incr>>2] = 2;
  $16 = HEAP8[$4>>0]|0;
  $17 = $16&255;
  $18 = (3 - ($17))|0;
  HEAP32[$i>>2] = $18;
  while(1) {
   $19 = HEAP32[$i>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = ($19|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$i>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + ($22)|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = ($26|0)==(0);
   if (!($27)) {
    break;
   }
   $28 = HEAP32[$i>>2]|0;
   $29 = (($28) + 2)|0;
   HEAP32[$i>>2] = $29;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ($30|0)<($31|0);
  $33 = $32&1;
  HEAP32[$nonNum>>2] = $33;
  $34 = HEAP32[$i>>2]|0;
  $35 = $34 ^ 1;
  $36 = HEAP32[$1>>2]|0;
  $37 = (($36) + ($35)|0);
  HEAP32[$zEnd>>2] = $37;
  $38 = HEAP8[$4>>0]|0;
  $39 = $38&255;
  $40 = $39 & 1;
  $41 = HEAP32[$1>>2]|0;
  $42 = (($41) + ($40)|0);
  HEAP32[$1>>2] = $42;
 }
 while(1) {
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$zEnd>>2]|0;
  $45 = ($43>>>0)<($44>>>0);
  if (!($45)) {
   break;
  }
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = (18403 + ($48)|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 & 1;
  $53 = ($52|0)!=(0);
  if (!($53)) {
   break;
  }
  $54 = HEAP32[$incr>>2]|0;
  $55 = HEAP32[$1>>2]|0;
  $56 = (($55) + ($54)|0);
  HEAP32[$1>>2] = $56;
 }
 $57 = HEAP32[$1>>2]|0;
 $58 = HEAP32[$zEnd>>2]|0;
 $59 = ($57>>>0)>=($58>>>0);
 if ($59) {
  HEAP32[$0>>2] = 0;
  $435 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($435|0);
 }
 $60 = HEAP32[$1>>2]|0;
 $61 = HEAP8[$60>>0]|0;
 $62 = $61 << 24 >> 24;
 $63 = ($62|0)==(45);
 if ($63) {
  HEAP32[$sign>>2] = -1;
  $64 = HEAP32[$incr>>2]|0;
  $65 = HEAP32[$1>>2]|0;
  $66 = (($65) + ($64)|0);
  HEAP32[$1>>2] = $66;
 } else {
  $67 = HEAP32[$1>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 << 24 >> 24;
  $70 = ($69|0)==(43);
  if ($70) {
   $71 = HEAP32[$incr>>2]|0;
   $72 = HEAP32[$1>>2]|0;
   $73 = (($72) + ($71)|0);
   HEAP32[$1>>2] = $73;
  }
 }
 while(1) {
  $74 = HEAP32[$1>>2]|0;
  $75 = HEAP32[$zEnd>>2]|0;
  $76 = ($74>>>0)<($75>>>0);
  if (!($76)) {
   break;
  }
  $77 = HEAP32[$1>>2]|0;
  $78 = HEAP8[$77>>0]|0;
  $79 = $78&255;
  $80 = (18403 + ($79)|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = $81&255;
  $83 = $82 & 4;
  $84 = ($83|0)!=(0);
  $85 = $s;
  $86 = $85;
  $87 = HEAP32[$86>>2]|0;
  $88 = (($85) + 4)|0;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = ($90|0)<(214748364);
  $92 = ($87>>>0)<(3435973835);
  $93 = ($90|0)==(214748364);
  $94 = $93 & $92;
  $95 = $91 | $94;
  $or$cond12 = $84 & $95;
  if (!($or$cond12)) {
   break;
  }
  $96 = $s;
  $97 = $96;
  $98 = HEAP32[$97>>2]|0;
  $99 = (($96) + 4)|0;
  $100 = $99;
  $101 = HEAP32[$100>>2]|0;
  $102 = (___muldi3(($98|0),($101|0),10,0)|0);
  $103 = tempRet0;
  $104 = HEAP32[$1>>2]|0;
  $105 = HEAP8[$104>>0]|0;
  $106 = $105 << 24 >> 24;
  $107 = (($106) - 48)|0;
  $108 = ($107|0)<(0);
  $109 = $108 << 31 >> 31;
  $110 = (_i64Add(($102|0),($103|0),($107|0),($109|0))|0);
  $111 = tempRet0;
  $112 = $s;
  $113 = $112;
  HEAP32[$113>>2] = $110;
  $114 = (($112) + 4)|0;
  $115 = $114;
  HEAP32[$115>>2] = $111;
  $116 = HEAP32[$incr>>2]|0;
  $117 = HEAP32[$1>>2]|0;
  $118 = (($117) + ($116)|0);
  HEAP32[$1>>2] = $118;
  $119 = HEAP32[$nDigits>>2]|0;
  $120 = (($119) + 1)|0;
  HEAP32[$nDigits>>2] = $120;
 }
 while(1) {
  $121 = HEAP32[$1>>2]|0;
  $122 = HEAP32[$zEnd>>2]|0;
  $123 = ($121>>>0)<($122>>>0);
  if (!($123)) {
   break;
  }
  $124 = HEAP32[$1>>2]|0;
  $125 = HEAP8[$124>>0]|0;
  $126 = $125&255;
  $127 = (18403 + ($126)|0);
  $128 = HEAP8[$127>>0]|0;
  $129 = $128&255;
  $130 = $129 & 4;
  $131 = ($130|0)!=(0);
  if (!($131)) {
   break;
  }
  $132 = HEAP32[$incr>>2]|0;
  $133 = HEAP32[$1>>2]|0;
  $134 = (($133) + ($132)|0);
  HEAP32[$1>>2] = $134;
  $135 = HEAP32[$nDigits>>2]|0;
  $136 = (($135) + 1)|0;
  HEAP32[$nDigits>>2] = $136;
  $137 = HEAP32[$d>>2]|0;
  $138 = (($137) + 1)|0;
  HEAP32[$d>>2] = $138;
 }
 $139 = HEAP32[$1>>2]|0;
 $140 = HEAP32[$zEnd>>2]|0;
 $141 = ($139>>>0)>=($140>>>0);
 L31: do {
  if (!($141)) {
   $142 = HEAP32[$1>>2]|0;
   $143 = HEAP8[$142>>0]|0;
   $144 = $143 << 24 >> 24;
   $145 = ($144|0)==(46);
   L33: do {
    if ($145) {
     $146 = HEAP32[$incr>>2]|0;
     $147 = HEAP32[$1>>2]|0;
     $148 = (($147) + ($146)|0);
     HEAP32[$1>>2] = $148;
     while(1) {
      $149 = HEAP32[$1>>2]|0;
      $150 = HEAP32[$zEnd>>2]|0;
      $151 = ($149>>>0)<($150>>>0);
      if (!($151)) {
       break L33;
      }
      $152 = HEAP32[$1>>2]|0;
      $153 = HEAP8[$152>>0]|0;
      $154 = $153&255;
      $155 = (18403 + ($154)|0);
      $156 = HEAP8[$155>>0]|0;
      $157 = $156&255;
      $158 = $157 & 4;
      $159 = ($158|0)!=(0);
      if (!($159)) {
       break L33;
      }
      $160 = $s;
      $161 = $160;
      $162 = HEAP32[$161>>2]|0;
      $163 = (($160) + 4)|0;
      $164 = $163;
      $165 = HEAP32[$164>>2]|0;
      $166 = ($165|0)<(214748364);
      $167 = ($162>>>0)<(3435973835);
      $168 = ($165|0)==(214748364);
      $169 = $168 & $167;
      $170 = $166 | $169;
      if ($170) {
       $171 = $s;
       $172 = $171;
       $173 = HEAP32[$172>>2]|0;
       $174 = (($171) + 4)|0;
       $175 = $174;
       $176 = HEAP32[$175>>2]|0;
       $177 = (___muldi3(($173|0),($176|0),10,0)|0);
       $178 = tempRet0;
       $179 = HEAP32[$1>>2]|0;
       $180 = HEAP8[$179>>0]|0;
       $181 = $180 << 24 >> 24;
       $182 = (($181) - 48)|0;
       $183 = ($182|0)<(0);
       $184 = $183 << 31 >> 31;
       $185 = (_i64Add(($177|0),($178|0),($182|0),($184|0))|0);
       $186 = tempRet0;
       $187 = $s;
       $188 = $187;
       HEAP32[$188>>2] = $185;
       $189 = (($187) + 4)|0;
       $190 = $189;
       HEAP32[$190>>2] = $186;
       $191 = HEAP32[$d>>2]|0;
       $192 = (($191) + -1)|0;
       HEAP32[$d>>2] = $192;
      }
      $193 = HEAP32[$incr>>2]|0;
      $194 = HEAP32[$1>>2]|0;
      $195 = (($194) + ($193)|0);
      HEAP32[$1>>2] = $195;
      $196 = HEAP32[$nDigits>>2]|0;
      $197 = (($196) + 1)|0;
      HEAP32[$nDigits>>2] = $197;
     }
    }
   } while(0);
   $198 = HEAP32[$1>>2]|0;
   $199 = HEAP32[$zEnd>>2]|0;
   $200 = ($198>>>0)>=($199>>>0);
   if (!($200)) {
    $201 = HEAP32[$1>>2]|0;
    $202 = HEAP8[$201>>0]|0;
    $203 = $202 << 24 >> 24;
    $204 = ($203|0)==(101);
    if ($204) {
     label = 34;
    } else {
     $205 = HEAP32[$1>>2]|0;
     $206 = HEAP8[$205>>0]|0;
     $207 = $206 << 24 >> 24;
     $208 = ($207|0)==(69);
     if ($208) {
      label = 34;
     }
    }
    L46: do {
     if ((label|0) == 34) {
      $209 = HEAP32[$incr>>2]|0;
      $210 = HEAP32[$1>>2]|0;
      $211 = (($210) + ($209)|0);
      HEAP32[$1>>2] = $211;
      HEAP32[$eValid>>2] = 0;
      $212 = HEAP32[$1>>2]|0;
      $213 = HEAP32[$zEnd>>2]|0;
      $214 = ($212>>>0)>=($213>>>0);
      if ($214) {
       break L31;
      }
      $215 = HEAP32[$1>>2]|0;
      $216 = HEAP8[$215>>0]|0;
      $217 = $216 << 24 >> 24;
      $218 = ($217|0)==(45);
      if ($218) {
       HEAP32[$esign>>2] = -1;
       $219 = HEAP32[$incr>>2]|0;
       $220 = HEAP32[$1>>2]|0;
       $221 = (($220) + ($219)|0);
       HEAP32[$1>>2] = $221;
      } else {
       $222 = HEAP32[$1>>2]|0;
       $223 = HEAP8[$222>>0]|0;
       $224 = $223 << 24 >> 24;
       $225 = ($224|0)==(43);
       if ($225) {
        $226 = HEAP32[$incr>>2]|0;
        $227 = HEAP32[$1>>2]|0;
        $228 = (($227) + ($226)|0);
        HEAP32[$1>>2] = $228;
       }
      }
      while(1) {
       $229 = HEAP32[$1>>2]|0;
       $230 = HEAP32[$zEnd>>2]|0;
       $231 = ($229>>>0)<($230>>>0);
       if (!($231)) {
        break L46;
       }
       $232 = HEAP32[$1>>2]|0;
       $233 = HEAP8[$232>>0]|0;
       $234 = $233&255;
       $235 = (18403 + ($234)|0);
       $236 = HEAP8[$235>>0]|0;
       $237 = $236&255;
       $238 = $237 & 4;
       $239 = ($238|0)!=(0);
       if (!($239)) {
        break L46;
       }
       $240 = HEAP32[$e>>2]|0;
       $241 = ($240|0)<(10000);
       if ($241) {
        $242 = HEAP32[$e>>2]|0;
        $243 = ($242*10)|0;
        $244 = HEAP32[$1>>2]|0;
        $245 = HEAP8[$244>>0]|0;
        $246 = $245 << 24 >> 24;
        $247 = (($246) - 48)|0;
        $248 = (($243) + ($247))|0;
        $249 = $248;
       } else {
        $249 = 10000;
       }
       HEAP32[$e>>2] = $249;
       $250 = HEAP32[$incr>>2]|0;
       $251 = HEAP32[$1>>2]|0;
       $252 = (($251) + ($250)|0);
       HEAP32[$1>>2] = $252;
       HEAP32[$eValid>>2] = 1;
      }
     }
    } while(0);
    while(1) {
     $253 = HEAP32[$1>>2]|0;
     $254 = HEAP32[$zEnd>>2]|0;
     $255 = ($253>>>0)<($254>>>0);
     if (!($255)) {
      break L31;
     }
     $256 = HEAP32[$1>>2]|0;
     $257 = HEAP8[$256>>0]|0;
     $258 = $257&255;
     $259 = (18403 + ($258)|0);
     $260 = HEAP8[$259>>0]|0;
     $261 = $260&255;
     $262 = $261 & 1;
     $263 = ($262|0)!=(0);
     if (!($263)) {
      break L31;
     }
     $264 = HEAP32[$incr>>2]|0;
     $265 = HEAP32[$1>>2]|0;
     $266 = (($265) + ($264)|0);
     HEAP32[$1>>2] = $266;
    }
   }
  }
 } while(0);
 $267 = HEAP32[$e>>2]|0;
 $268 = HEAP32[$esign>>2]|0;
 $269 = Math_imul($267, $268)|0;
 $270 = HEAP32[$d>>2]|0;
 $271 = (($269) + ($270))|0;
 HEAP32[$e>>2] = $271;
 $272 = HEAP32[$e>>2]|0;
 $273 = ($272|0)<(0);
 if ($273) {
  HEAP32[$esign>>2] = -1;
  $274 = HEAP32[$e>>2]|0;
  $275 = Math_imul($274, -1)|0;
  HEAP32[$e>>2] = $275;
 } else {
  HEAP32[$esign>>2] = 1;
 }
 $276 = $s;
 $277 = $276;
 $278 = HEAP32[$277>>2]|0;
 $279 = (($276) + 4)|0;
 $280 = $279;
 $281 = HEAP32[$280>>2]|0;
 $282 = ($278|0)==(0);
 $283 = ($281|0)==(0);
 $284 = $282 & $283;
 do {
  if ($284) {
   $285 = HEAP32[$sign>>2]|0;
   $286 = ($285|0)<(0);
   $287 = $286 ? -0.0 : 0.0;
   HEAPF64[$result>>3] = $287;
  } else {
   while(1) {
    $288 = HEAP32[$e>>2]|0;
    $289 = ($288|0)>(0);
    if (!($289)) {
     break;
    }
    $290 = HEAP32[$esign>>2]|0;
    $291 = ($290|0)>(0);
    $292 = $s;
    $293 = $292;
    $294 = HEAP32[$293>>2]|0;
    $295 = (($292) + 4)|0;
    $296 = $295;
    $297 = HEAP32[$296>>2]|0;
    if ($291) {
     $298 = ($297|0)>(214748364);
     $299 = ($294>>>0)>=(3435973836);
     $300 = ($297|0)==(214748364);
     $301 = $300 & $299;
     $302 = $298 | $301;
     if ($302) {
      break;
     }
     $303 = $s;
     $304 = $303;
     $305 = HEAP32[$304>>2]|0;
     $306 = (($303) + 4)|0;
     $307 = $306;
     $308 = HEAP32[$307>>2]|0;
     $309 = (___muldi3(($305|0),($308|0),10,0)|0);
     $310 = tempRet0;
     $311 = $s;
     $312 = $311;
     HEAP32[$312>>2] = $309;
     $313 = (($311) + 4)|0;
     $314 = $313;
     HEAP32[$314>>2] = $310;
    } else {
     $315 = (___remdi3(($294|0),($297|0),10,0)|0);
     $316 = tempRet0;
     $317 = ($315|0)!=(0);
     $318 = ($316|0)!=(0);
     $319 = $317 | $318;
     if ($319) {
      break;
     }
     $320 = $s;
     $321 = $320;
     $322 = HEAP32[$321>>2]|0;
     $323 = (($320) + 4)|0;
     $324 = $323;
     $325 = HEAP32[$324>>2]|0;
     $326 = (___divdi3(($322|0),($325|0),10,0)|0);
     $327 = tempRet0;
     $328 = $s;
     $329 = $328;
     HEAP32[$329>>2] = $326;
     $330 = (($328) + 4)|0;
     $331 = $330;
     HEAP32[$331>>2] = $327;
    }
    $332 = HEAP32[$e>>2]|0;
    $333 = (($332) + -1)|0;
    HEAP32[$e>>2] = $333;
   }
   $334 = HEAP32[$sign>>2]|0;
   $335 = ($334|0)<(0);
   $336 = $s;
   $337 = $336;
   $338 = HEAP32[$337>>2]|0;
   $339 = (($336) + 4)|0;
   $340 = $339;
   $341 = HEAP32[$340>>2]|0;
   $342 = (_i64Subtract(0,0,($338|0),($341|0))|0);
   $343 = tempRet0;
   $344 = $335 ? $342 : $338;
   $345 = $335 ? $343 : $341;
   $346 = $s;
   $347 = $346;
   HEAP32[$347>>2] = $344;
   $348 = (($346) + 4)|0;
   $349 = $348;
   HEAP32[$349>>2] = $345;
   $350 = HEAP32[$e>>2]|0;
   $351 = ($350|0)==(0);
   if ($351) {
    $352 = $s;
    $353 = $352;
    $354 = HEAP32[$353>>2]|0;
    $355 = (($352) + 4)|0;
    $356 = $355;
    $357 = HEAP32[$356>>2]|0;
    $358 = (+($354>>>0)) + (4294967296.0*(+($357|0)));
    HEAPF64[$result>>3] = $358;
    break;
   }
   HEAPF64[$scale>>3] = 1.0;
   $359 = HEAP32[$e>>2]|0;
   $360 = ($359|0)>(307);
   if (!($360)) {
    while(1) {
     $397 = HEAP32[$e>>2]|0;
     $398 = (($397|0) % 22)&-1;
     $399 = ($398|0)!=(0);
     if (!($399)) {
      break;
     }
     $400 = +HEAPF64[$scale>>3];
     $401 = $400 * 10.0;
     HEAPF64[$scale>>3] = $401;
     $402 = HEAP32[$e>>2]|0;
     $403 = (($402) - 1)|0;
     HEAP32[$e>>2] = $403;
    }
    while(1) {
     $404 = HEAP32[$e>>2]|0;
     $405 = ($404|0)>(0);
     if (!($405)) {
      break;
     }
     $406 = +HEAPF64[$scale>>3];
     $407 = $406 * 1.0E+22;
     HEAPF64[$scale>>3] = $407;
     $408 = HEAP32[$e>>2]|0;
     $409 = (($408) - 22)|0;
     HEAP32[$e>>2] = $409;
    }
    $410 = HEAP32[$esign>>2]|0;
    $411 = ($410|0)<(0);
    $412 = $s;
    $413 = $412;
    $414 = HEAP32[$413>>2]|0;
    $415 = (($412) + 4)|0;
    $416 = $415;
    $417 = HEAP32[$416>>2]|0;
    $418 = (+($414>>>0)) + (4294967296.0*(+($417|0)));
    $419 = +HEAPF64[$scale>>3];
    if ($411) {
     $420 = $418 / $419;
     HEAPF64[$result>>3] = $420;
     break;
    } else {
     $421 = $418 * $419;
     HEAPF64[$result>>3] = $421;
     break;
    }
   }
   $361 = HEAP32[$e>>2]|0;
   $362 = ($361|0)<(342);
   if (!($362)) {
    $386 = HEAP32[$esign>>2]|0;
    $387 = ($386|0)<(0);
    $388 = $s;
    $389 = $388;
    $390 = HEAP32[$389>>2]|0;
    $391 = (($388) + 4)|0;
    $392 = $391;
    $393 = HEAP32[$392>>2]|0;
    $394 = (+($390>>>0)) + (4294967296.0*(+($393|0)));
    if ($387) {
     $395 = 0.0 * $394;
     HEAPF64[$result>>3] = $395;
     break;
    } else {
     $396 = inf * $394;
     HEAPF64[$result>>3] = $396;
     break;
    }
   }
   while(1) {
    $363 = HEAP32[$e>>2]|0;
    $364 = (($363|0) % 308)&-1;
    $365 = ($364|0)!=(0);
    if (!($365)) {
     break;
    }
    $366 = +HEAPF64[$scale>>3];
    $367 = $366 * 10.0;
    HEAPF64[$scale>>3] = $367;
    $368 = HEAP32[$e>>2]|0;
    $369 = (($368) - 1)|0;
    HEAP32[$e>>2] = $369;
   }
   $370 = HEAP32[$esign>>2]|0;
   $371 = ($370|0)<(0);
   $372 = $s;
   $373 = $372;
   $374 = HEAP32[$373>>2]|0;
   $375 = (($372) + 4)|0;
   $376 = $375;
   $377 = HEAP32[$376>>2]|0;
   $378 = (+($374>>>0)) + (4294967296.0*(+($377|0)));
   $379 = +HEAPF64[$scale>>3];
   if ($371) {
    $380 = $378 / $379;
    HEAPF64[$result>>3] = $380;
    $381 = +HEAPF64[$result>>3];
    $382 = $381 / 1.0E+308;
    HEAPF64[$result>>3] = $382;
    break;
   } else {
    $383 = $378 * $379;
    HEAPF64[$result>>3] = $383;
    $384 = +HEAPF64[$result>>3];
    $385 = $384 * 1.0E+308;
    HEAPF64[$result>>3] = $385;
    break;
   }
  }
 } while(0);
 $422 = +HEAPF64[$result>>3];
 $423 = HEAP32[$2>>2]|0;
 HEAPF64[$423>>3] = $422;
 $424 = HEAP32[$1>>2]|0;
 $425 = HEAP32[$zEnd>>2]|0;
 $426 = ($424|0)==($425|0);
 $427 = HEAP32[$nDigits>>2]|0;
 $428 = ($427|0)>(0);
 $or$cond = $426 & $428;
 $429 = HEAP32[$eValid>>2]|0;
 $430 = ($429|0)!=(0);
 $or$cond10 = $or$cond & $430;
 if ($or$cond10) {
  $431 = HEAP32[$nonNum>>2]|0;
  $432 = ($431|0)==(0);
  $434 = $432;
 } else {
  $434 = 0;
 }
 $433 = $434&1;
 HEAP32[$0>>2] = $433;
 $435 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($435|0);
}
function _sqlite3MatchSpanName($zSpan,$zCol,$zTab,$zDb) {
 $zSpan = $zSpan|0;
 $zCol = $zCol|0;
 $zTab = $zTab|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $zSpan;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$4>>2] = $zDb;
 HEAP32[$n>>2] = 0;
 while(1) {
  $5 = HEAP32[$n>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) + ($5)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$n>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = ($15|0)!=(46);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$n>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$n>>2] = $18;
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)!=(0|0);
 do {
  if ($20) {
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$4>>2]|0;
   $23 = HEAP32[$n>>2]|0;
   $24 = (_sqlite3_strnicmp($21,$22,$23)|0);
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $26 = HEAP32[$n>>2]|0;
    $27 = HEAP32[$4>>2]|0;
    $28 = (($27) + ($26)|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29 << 24 >> 24;
    $31 = ($30|0)!=(0);
    if (!($31)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $32 = HEAP32[$n>>2]|0;
 $33 = (($32) + 1)|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = (($34) + ($33)|0);
 HEAP32[$1>>2] = $35;
 HEAP32[$n>>2] = 0;
 while(1) {
  $36 = HEAP32[$n>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = (($37) + ($36)|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39 << 24 >> 24;
  $41 = ($40|0)!=(0);
  if (!($41)) {
   break;
  }
  $42 = HEAP32[$n>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = (($43) + ($42)|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = ($46|0)!=(46);
  if (!($47)) {
   break;
  }
  $48 = HEAP32[$n>>2]|0;
  $49 = (($48) + 1)|0;
  HEAP32[$n>>2] = $49;
 }
 $50 = HEAP32[$3>>2]|0;
 $51 = ($50|0)!=(0|0);
 do {
  if ($51) {
   $52 = HEAP32[$1>>2]|0;
   $53 = HEAP32[$3>>2]|0;
   $54 = HEAP32[$n>>2]|0;
   $55 = (_sqlite3_strnicmp($52,$53,$54)|0);
   $56 = ($55|0)!=(0);
   if (!($56)) {
    $57 = HEAP32[$n>>2]|0;
    $58 = HEAP32[$3>>2]|0;
    $59 = (($58) + ($57)|0);
    $60 = HEAP8[$59>>0]|0;
    $61 = $60 << 24 >> 24;
    $62 = ($61|0)!=(0);
    if (!($62)) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 } while(0);
 $63 = HEAP32[$n>>2]|0;
 $64 = (($63) + 1)|0;
 $65 = HEAP32[$1>>2]|0;
 $66 = (($65) + ($64)|0);
 HEAP32[$1>>2] = $66;
 $67 = HEAP32[$2>>2]|0;
 $68 = ($67|0)!=(0|0);
 if ($68) {
  $69 = HEAP32[$1>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = (_sqlite3StrICmp($69,$70)|0);
  $72 = ($71|0)!=(0);
  if ($72) {
   HEAP32[$0>>2] = 0;
   $73 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($73|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $73 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _nameInUsingClause($pUsing,$zCol) {
 $pUsing = $pUsing|0;
 $zCol = $zCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $k = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $k = sp;
 HEAP32[$1>>2] = $pUsing;
 HEAP32[$2>>2] = $zCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$k>>2] = 0;
   while(1) {
    $5 = HEAP32[$k>>2]|0;
    $6 = HEAP32[$1>>2]|0;
    $7 = ((($6)) + 4|0);
    $8 = HEAP32[$7>>2]|0;
    $9 = ($5|0)<($8|0);
    if (!($9)) {
     break L1;
    }
    $10 = HEAP32[$k>>2]|0;
    $11 = HEAP32[$1>>2]|0;
    $12 = HEAP32[$11>>2]|0;
    $13 = (($12) + ($10<<3)|0);
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$2>>2]|0;
    $16 = (_sqlite3StrICmp($14,$15)|0);
    $17 = ($16|0)==(0);
    if ($17) {
     break;
    }
    $18 = HEAP32[$k>>2]|0;
    $19 = (($18) + 1)|0;
    HEAP32[$k>>2] = $19;
   }
   HEAP32[$0>>2] = 1;
   $20 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($20|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3AuthRead($pParse,$pExpr,$pSchema,$pTabList) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pSchema = $pSchema|0;
 $pTabList = $pTabList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $db = 0, $iCol = 0, $iDb = 0, $iSrc = 0, $pTab = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $db = sp + 20|0;
 $pTab = sp + 16|0;
 $zCol = sp + 12|0;
 $iSrc = sp + 8|0;
 $iDb = sp + 4|0;
 $iCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $pSchema;
 HEAP32[$3>>2] = $pTabList;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 HEAP32[$pTab>>2] = 0;
 $6 = HEAP32[$db>>2]|0;
 $7 = ((($6)) + 296|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3SchemaToIndex($11,$12)|0);
 HEAP32[$iDb>>2] = $13;
 $14 = HEAP32[$iDb>>2]|0;
 $15 = ($14|0)<(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(88);
 L7: do {
  if ($19) {
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 128|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pTab>>2] = $22;
  } else {
   HEAP32[$iSrc>>2] = 0;
   while(1) {
    $23 = HEAP32[$iSrc>>2]|0;
    $24 = HEAP32[$3>>2]|0;
    $25 = HEAP32[$24>>2]|0;
    $26 = ($23|0)<($25|0);
    if (!($26)) {
     break L7;
    }
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 28|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$iSrc>>2]|0;
    $31 = HEAP32[$3>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = (($32) + (($30*72)|0)|0);
    $34 = ((($33)) + 44|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = ($29|0)==($35|0);
    $37 = HEAP32[$iSrc>>2]|0;
    if ($36) {
     break;
    }
    $43 = (($37) + 1)|0;
    HEAP32[$iSrc>>2] = $43;
   }
   $38 = HEAP32[$3>>2]|0;
   $39 = ((($38)) + 8|0);
   $40 = (($39) + (($37*72)|0)|0);
   $41 = ((($40)) + 16|0);
   $42 = HEAP32[$41>>2]|0;
   HEAP32[$pTab>>2] = $42;
  }
 } while(0);
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 32|0);
 $46 = HEAP16[$45>>1]|0;
 $47 = $46 << 16 >> 16;
 HEAP32[$iCol>>2] = $47;
 $48 = HEAP32[$pTab>>2]|0;
 $49 = ($48|0)==(0|0);
 if ($49) {
  STACKTOP = sp;return;
 }
 $50 = HEAP32[$iCol>>2]|0;
 $51 = ($50|0)>=(0);
 do {
  if ($51) {
   $52 = HEAP32[$iCol>>2]|0;
   $53 = HEAP32[$pTab>>2]|0;
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + ($52<<4)|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$zCol>>2] = $57;
  } else {
   $58 = HEAP32[$pTab>>2]|0;
   $59 = ((($58)) + 32|0);
   $60 = HEAP16[$59>>1]|0;
   $61 = $60 << 16 >> 16;
   $62 = ($61|0)>=(0);
   if ($62) {
    $63 = HEAP32[$pTab>>2]|0;
    $64 = ((($63)) + 32|0);
    $65 = HEAP16[$64>>1]|0;
    $66 = $65 << 16 >> 16;
    $67 = HEAP32[$pTab>>2]|0;
    $68 = ((($67)) + 4|0);
    $69 = HEAP32[$68>>2]|0;
    $70 = (($69) + ($66<<4)|0);
    $71 = HEAP32[$70>>2]|0;
    HEAP32[$zCol>>2] = $71;
    break;
   } else {
    HEAP32[$zCol>>2] = 19895;
    break;
   }
  }
 } while(0);
 $72 = HEAP32[$0>>2]|0;
 $73 = HEAP32[$pTab>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 $75 = HEAP32[$zCol>>2]|0;
 $76 = HEAP32[$iDb>>2]|0;
 $77 = (_sqlite3AuthReadCol($72,$74,$75,$76)|0);
 $78 = (2)==($77|0);
 if (!($78)) {
  STACKTOP = sp;return;
 }
 $79 = HEAP32[$1>>2]|0;
 HEAP8[$79>>0] = 101;
 STACKTOP = sp;return;
}
function _sqlite3AuthReadCol($pParse,$zTab,$zCol,$iDb) {
 $pParse = $pParse|0;
 $zTab = $zTab|0;
 $zCol = $zCol|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer6 = 0;
 var $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $db = sp + 32|0;
 $zDb = sp + 28|0;
 $rc = sp + 24|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zTab;
 HEAP32[$3>>2] = $zCol;
 HEAP32[$4>>2] = $iDb;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10) + ($7<<4)|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$zDb>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 148|0);
 $15 = ((($14)) + 5|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 0;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 296|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$db>>2]|0;
 $22 = ((($21)) + 300|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$3>>2]|0;
 $26 = HEAP32[$zDb>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 448|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (FUNCTION_TABLE_iiiiiii[$20 & 255]($23,20,$24,$25,$26,$29)|0);
 HEAP32[$rc>>2] = $30;
 $31 = HEAP32[$rc>>2]|0;
 $32 = ($31|0)==(1);
 if ($32) {
  $33 = HEAP32[$db>>2]|0;
  $34 = ((($33)) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($35|0)>(2);
  $37 = HEAP32[$4>>2]|0;
  $38 = ($37|0)!=(0);
  $or$cond = $36 | $38;
  $39 = HEAP32[$1>>2]|0;
  if ($or$cond) {
   $40 = HEAP32[$zDb>>2]|0;
   $41 = HEAP32[$2>>2]|0;
   $42 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer>>2] = $40;
   $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $41;
   $vararg_ptr5 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr5>>2] = $42;
   _sqlite3ErrorMsg($39,30046,$vararg_buffer);
  } else {
   $43 = HEAP32[$2>>2]|0;
   $44 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer6>>2] = $43;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $44;
   _sqlite3ErrorMsg($39,30079,$vararg_buffer6);
  }
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 12|0);
  HEAP32[$46>>2] = 23;
 } else {
  $47 = HEAP32[$rc>>2]|0;
  $48 = ($47|0)!=(2);
  $49 = HEAP32[$rc>>2]|0;
  $50 = ($49|0)!=(0);
  $or$cond3 = $48 & $50;
  if ($or$cond3) {
   $51 = HEAP32[$1>>2]|0;
   _sqliteAuthBadReturnCode($51);
  }
 }
 $52 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $52;
 $53 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _convertCompoundSelectToSubquery($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0, $db = 0, $dummy = 0, $i = 0, $pNew = 0, $pNewSrc = 0, $pParse = 0, $pX = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $i = sp + 32|0;
 $pNew = sp + 28|0;
 $pX = sp + 24|0;
 $db = sp + 20|0;
 $a = sp + 16|0;
 $pNewSrc = sp + 12|0;
 $pParse = sp + 8|0;
 $dummy = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$pX>>2] = $11;
 while(1) {
  $12 = HEAP32[$pX>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pX>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(116);
  if (!($18)) {
   $19 = HEAP32[$pX>>2]|0;
   $20 = ((($19)) + 4|0);
   $21 = HEAP8[$20>>0]|0;
   $22 = $21&255;
   $23 = ($22|0)==(119);
   if (!($23)) {
    break;
   }
  }
  $24 = HEAP32[$pX>>2]|0;
  $25 = ((($24)) + 48|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pX>>2] = $26;
 }
 $27 = HEAP32[$pX>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  HEAP32[$0>>2] = 0;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = ((($29)) + 44|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ((($31)) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$a>>2] = $33;
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 44|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) - 1)|0;
 HEAP32[$i>>2] = $38;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = ($39|0)>=(0);
  if (!($40)) {
   break;
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = HEAP32[$a>>2]|0;
  $43 = (($42) + (($41*20)|0)|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $46 & 256;
  $48 = ($47|0)!=(0);
  if ($48) {
   break;
  }
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + -1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$i>>2]|0;
 $52 = ($51|0)<(0);
 if ($52) {
  HEAP32[$0>>2] = 0;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = HEAP32[$53>>2]|0;
 HEAP32[$pParse>>2] = $54;
 $55 = HEAP32[$pParse>>2]|0;
 $56 = HEAP32[$55>>2]|0;
 HEAP32[$db>>2] = $56;
 $57 = HEAP32[$db>>2]|0;
 $58 = (_sqlite3DbMallocZero($57,68,0)|0);
 HEAP32[$pNew>>2] = $58;
 $59 = HEAP32[$pNew>>2]|0;
 $60 = ($59|0)==(0|0);
 if ($60) {
  HEAP32[$0>>2] = 2;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 ;HEAP32[$dummy>>2]=0|0;HEAP32[$dummy+4>>2]=0|0;
 $61 = HEAP32[$pParse>>2]|0;
 $62 = HEAP32[$pNew>>2]|0;
 $63 = (_sqlite3SrcListAppendFromTerm($61,0,0,0,$dummy,$62,0,0)|0);
 HEAP32[$pNewSrc>>2] = $63;
 $64 = HEAP32[$pNewSrc>>2]|0;
 $65 = ($64|0)==(0|0);
 if ($65) {
  HEAP32[$0>>2] = 2;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 } else {
  $66 = HEAP32[$pNew>>2]|0;
  $67 = HEAP32[$2>>2]|0;
  dest=$66; src=$67; stop=dest+68|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $68 = HEAP32[$pNewSrc>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = ((($69)) + 28|0);
  HEAP32[$70>>2] = $68;
  $71 = HEAP32[$pParse>>2]|0;
  $72 = HEAP32[$db>>2]|0;
  $73 = (_sqlite3Expr($72,160,0)|0);
  $74 = (_sqlite3ExprListAppend($71,0,$73)|0);
  $75 = HEAP32[$2>>2]|0;
  HEAP32[$75>>2] = $74;
  $76 = HEAP32[$2>>2]|0;
  $77 = ((($76)) + 4|0);
  HEAP8[$77>>0] = 119;
  $78 = HEAP32[$2>>2]|0;
  $79 = ((($78)) + 32|0);
  HEAP32[$79>>2] = 0;
  $80 = HEAP32[$pNew>>2]|0;
  $81 = ((($80)) + 36|0);
  HEAP32[$81>>2] = 0;
  $82 = HEAP32[$pNew>>2]|0;
  $83 = ((($82)) + 40|0);
  HEAP32[$83>>2] = 0;
  $84 = HEAP32[$pNew>>2]|0;
  $85 = ((($84)) + 44|0);
  HEAP32[$85>>2] = 0;
  $86 = HEAP32[$2>>2]|0;
  $87 = ((($86)) + 48|0);
  HEAP32[$87>>2] = 0;
  $88 = HEAP32[$2>>2]|0;
  $89 = ((($88)) + 52|0);
  HEAP32[$89>>2] = 0;
  $90 = HEAP32[$2>>2]|0;
  $91 = ((($90)) + 64|0);
  HEAP32[$91>>2] = 0;
  $92 = HEAP32[$2>>2]|0;
  $93 = ((($92)) + 8|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = $94 & -257;
  HEAP32[$93>>2] = $95;
  $96 = HEAP32[$2>>2]|0;
  $97 = ((($96)) + 8|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = $98 | 65536;
  HEAP32[$97>>2] = $99;
  $100 = HEAP32[$pNew>>2]|0;
  $101 = HEAP32[$pNew>>2]|0;
  $102 = ((($101)) + 48|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = ((($103)) + 52|0);
  HEAP32[$104>>2] = $100;
  $105 = HEAP32[$pNew>>2]|0;
  $106 = ((($105)) + 56|0);
  HEAP32[$106>>2] = 0;
  $107 = HEAP32[$pNew>>2]|0;
  $108 = ((($107)) + 60|0);
  HEAP32[$108>>2] = 0;
  HEAP32[$0>>2] = 0;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 return (0)|0;
}
function _selectExpander($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $a = 0, $db = 0, $flags = 0, $i = 0, $iDb = 0, $j = 0, $k = 0, $longNames = 0, $nCol = 0, $or$cond = 0, $or$cond3 = 0, $pE = 0, $pEList = 0, $pExpr = 0, $pFrom = 0, $pLeft = 0, $pNew = 0, $pParse = 0, $pRight = 0;
 var $pSel = 0, $pSub = 0, $pTab = 0, $pTab1 = 0, $pTabList = 0, $pX = 0, $sColname = 0, $selFlags = 0, $tableSeen = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer21 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr10 = 0, $vararg_ptr14 = 0, $vararg_ptr15 = 0, $zColname = 0;
 var $zName = 0, $zSchemaName = 0, $zTName = 0, $zTabName = 0, $zToFree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $vararg_buffer21 = sp + 56|0;
 $vararg_buffer19 = sp + 48|0;
 $vararg_buffer16 = sp + 40|0;
 $vararg_buffer11 = sp + 24|0;
 $vararg_buffer7 = sp + 16|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 196|0;
 $1 = sp + 192|0;
 $2 = sp + 188|0;
 $pParse = sp + 184|0;
 $i = sp + 180|0;
 $j = sp + 176|0;
 $k = sp + 172|0;
 $pTabList = sp + 168|0;
 $pEList = sp + 164|0;
 $pFrom = sp + 160|0;
 $db = sp + 156|0;
 $pE = sp + 152|0;
 $pRight = sp + 148|0;
 $pExpr = sp + 144|0;
 $selFlags = sp + 202|0;
 $pTab = sp + 140|0;
 $pSel = sp + 136|0;
 $nCol = sp + 200|0;
 $a = sp + 132|0;
 $pNew = sp + 128|0;
 $flags = sp + 124|0;
 $longNames = sp + 120|0;
 $tableSeen = sp + 116|0;
 $zTName = sp + 112|0;
 $pTab1 = sp + 108|0;
 $pSub = sp + 104|0;
 $zTabName = sp + 100|0;
 $zSchemaName = sp + 96|0;
 $iDb = sp + 92|0;
 $zName = sp + 88|0;
 $zColname = sp + 84|0;
 $zToFree = sp + 80|0;
 $sColname = sp + 72|0;
 $pLeft = sp + 64|0;
 $pX = sp + 60|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9&65535;
 HEAP16[$selFlags>>1] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 | 64;
 HEAP32[$12>>2] = $14;
 $15 = HEAP32[$db>>2]|0;
 $16 = ((($15)) + 69|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 if ($18) {
  HEAP32[$0>>2] = 2;
  $540 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($540|0);
 }
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 28|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)==(0|0);
 if (!($22)) {
  $23 = HEAP16[$selFlags>>1]|0;
  $24 = $23&65535;
  $25 = $24 & 64;
  $26 = ($25|0)!=(0);
  if (!($26)) {
   $27 = HEAP32[$2>>2]|0;
   $28 = ((($27)) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$pTabList>>2] = $29;
   $30 = HEAP32[$2>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$pEList>>2] = $31;
   $32 = HEAP32[$1>>2]|0;
   $33 = ((($32)) + 12|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = ($34|0)==(132|0);
   if ($35) {
    $36 = HEAP32[$pParse>>2]|0;
    $37 = HEAP32[$2>>2]|0;
    $38 = (_findRightmost($37)|0);
    $39 = ((($38)) + 64|0);
    $40 = HEAP32[$39>>2]|0;
    _sqlite3WithPush($36,$40,0);
   }
   $41 = HEAP32[$pParse>>2]|0;
   $42 = HEAP32[$pTabList>>2]|0;
   _sqlite3SrcListAssignCursors($41,$42);
   HEAP32[$i>>2] = 0;
   $43 = HEAP32[$pTabList>>2]|0;
   $44 = ((($43)) + 8|0);
   HEAP32[$pFrom>>2] = $44;
   L11: while(1) {
    $45 = HEAP32[$i>>2]|0;
    $46 = HEAP32[$pTabList>>2]|0;
    $47 = HEAP32[$46>>2]|0;
    $48 = ($45|0)<($47|0);
    if (!($48)) {
     label = 38;
     break;
    }
    $49 = HEAP32[$pFrom>>2]|0;
    $50 = ((($49)) + 36|0);
    $51 = ((($50)) + 1|0);
    $52 = HEAP8[$51>>0]|0;
    $53 = ($52&255) >>> 5;
    $54 = $53 & 1;
    $55 = $54&255;
    $56 = ($55|0)!=(0);
    if (!($56)) {
     $57 = HEAP32[$1>>2]|0;
     $58 = HEAP32[$pFrom>>2]|0;
     $59 = (_withExpand($57,$58)|0);
     $60 = ($59|0)!=(0);
     if ($60) {
      label = 12;
      break;
     }
     $61 = HEAP32[$pFrom>>2]|0;
     $62 = ((($61)) + 16|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = ($63|0)!=(0|0);
     do {
      if (!($64)) {
       $65 = HEAP32[$pFrom>>2]|0;
       $66 = ((($65)) + 8|0);
       $67 = HEAP32[$66>>2]|0;
       $68 = ($67|0)==(0|0);
       if ($68) {
        $69 = HEAP32[$pFrom>>2]|0;
        $70 = ((($69)) + 20|0);
        $71 = HEAP32[$70>>2]|0;
        HEAP32[$pSel>>2] = $71;
        $72 = HEAP32[$1>>2]|0;
        $73 = HEAP32[$pSel>>2]|0;
        $74 = (_sqlite3WalkSelect($72,$73)|0);
        $75 = ($74|0)!=(0);
        if ($75) {
         label = 16;
         break L11;
        }
        $76 = HEAP32[$db>>2]|0;
        $77 = (_sqlite3DbMallocZero($76,72,0)|0);
        HEAP32[$pTab>>2] = $77;
        $78 = HEAP32[$pFrom>>2]|0;
        $79 = ((($78)) + 16|0);
        HEAP32[$79>>2] = $77;
        $80 = HEAP32[$pTab>>2]|0;
        $81 = ($80|0)==(0|0);
        if ($81) {
         label = 18;
         break L11;
        }
        $82 = HEAP32[$pTab>>2]|0;
        $83 = ((($82)) + 36|0);
        HEAP16[$83>>1] = 1;
        $84 = HEAP32[$db>>2]|0;
        $85 = HEAP32[$pTab>>2]|0;
        HEAP32[$vararg_buffer>>2] = $85;
        $86 = (_sqlite3MPrintf($84,30109,$vararg_buffer)|0);
        $87 = HEAP32[$pTab>>2]|0;
        HEAP32[$87>>2] = $86;
        while(1) {
         $88 = HEAP32[$pSel>>2]|0;
         $89 = ((($88)) + 48|0);
         $90 = HEAP32[$89>>2]|0;
         $91 = ($90|0)!=(0|0);
         if (!($91)) {
          break;
         }
         $92 = HEAP32[$pSel>>2]|0;
         $93 = ((($92)) + 48|0);
         $94 = HEAP32[$93>>2]|0;
         HEAP32[$pSel>>2] = $94;
        }
        $95 = HEAP32[$pParse>>2]|0;
        $96 = HEAP32[$pSel>>2]|0;
        $97 = HEAP32[$96>>2]|0;
        $98 = HEAP32[$pTab>>2]|0;
        $99 = ((($98)) + 34|0);
        $100 = HEAP32[$pTab>>2]|0;
        $101 = ((($100)) + 4|0);
        (_sqlite3ColumnsFromExprList($95,$97,$99,$101)|0);
        $102 = HEAP32[$pTab>>2]|0;
        $103 = ((($102)) + 32|0);
        HEAP16[$103>>1] = -1;
        $104 = HEAP32[$pTab>>2]|0;
        $105 = ((($104)) + 38|0);
        HEAP16[$105>>1] = 200;
        $106 = HEAP32[$pTab>>2]|0;
        $107 = ((($106)) + 42|0);
        $108 = HEAP8[$107>>0]|0;
        $109 = $108&255;
        $110 = $109 | 2;
        $111 = $110&255;
        HEAP8[$107>>0] = $111;
        break;
       }
       $112 = HEAP32[$pParse>>2]|0;
       $113 = HEAP32[$pFrom>>2]|0;
       $114 = (_sqlite3LocateTableItem($112,0,$113)|0);
       HEAP32[$pTab>>2] = $114;
       $115 = HEAP32[$pFrom>>2]|0;
       $116 = ((($115)) + 16|0);
       HEAP32[$116>>2] = $114;
       $117 = HEAP32[$pTab>>2]|0;
       $118 = ($117|0)==(0|0);
       if ($118) {
        label = 24;
        break L11;
       }
       $119 = HEAP32[$pTab>>2]|0;
       $120 = ((($119)) + 36|0);
       $121 = HEAP16[$120>>1]|0;
       $122 = $121&65535;
       $123 = ($122|0)==(65535);
       if ($123) {
        label = 26;
        break L11;
       }
       $129 = HEAP32[$pTab>>2]|0;
       $130 = ((($129)) + 36|0);
       $131 = HEAP16[$130>>1]|0;
       $132 = (($131) + 1)<<16>>16;
       HEAP16[$130>>1] = $132;
       $133 = HEAP32[$pTab>>2]|0;
       $134 = ((($133)) + 42|0);
       $135 = HEAP8[$134>>0]|0;
       $136 = $135&255;
       $137 = $136 & 16;
       $138 = ($137|0)!=(0);
       if (!($138)) {
        $139 = HEAP32[$pParse>>2]|0;
        $140 = HEAP32[$pFrom>>2]|0;
        $141 = (_cannotBeFunction($139,$140)|0);
        $142 = ($141|0)!=(0);
        if ($142) {
         label = 29;
         break L11;
        }
       }
       $143 = HEAP32[$pTab>>2]|0;
       $144 = ((($143)) + 42|0);
       $145 = HEAP8[$144>>0]|0;
       $146 = $145&255;
       $147 = $146 & 16;
       $148 = ($147|0)!=(0);
       if (!($148)) {
        $149 = HEAP32[$pTab>>2]|0;
        $150 = ((($149)) + 12|0);
        $151 = HEAP32[$150>>2]|0;
        $152 = ($151|0)!=(0|0);
        if (!($152)) {
         break;
        }
       }
       $153 = HEAP32[$pParse>>2]|0;
       $154 = HEAP32[$pTab>>2]|0;
       $155 = (_sqlite3ViewGetColumnNames($153,$154)|0);
       $156 = ($155|0)!=(0);
       if ($156) {
        label = 33;
        break L11;
       }
       $157 = HEAP32[$db>>2]|0;
       $158 = HEAP32[$pTab>>2]|0;
       $159 = ((($158)) + 12|0);
       $160 = HEAP32[$159>>2]|0;
       $161 = (_sqlite3SelectDup($157,$160,0)|0);
       $162 = HEAP32[$pFrom>>2]|0;
       $163 = ((($162)) + 20|0);
       HEAP32[$163>>2] = $161;
       $164 = HEAP32[$pTab>>2]|0;
       $165 = ((($164)) + 34|0);
       $166 = HEAP16[$165>>1]|0;
       HEAP16[$nCol>>1] = $166;
       $167 = HEAP32[$pTab>>2]|0;
       $168 = ((($167)) + 34|0);
       HEAP16[$168>>1] = -1;
       $169 = HEAP32[$1>>2]|0;
       $170 = HEAP32[$pFrom>>2]|0;
       $171 = ((($170)) + 20|0);
       $172 = HEAP32[$171>>2]|0;
       (_sqlite3WalkSelect($169,$172)|0);
       $173 = HEAP16[$nCol>>1]|0;
       $174 = HEAP32[$pTab>>2]|0;
       $175 = ((($174)) + 34|0);
       HEAP16[$175>>1] = $173;
      }
     } while(0);
     $176 = HEAP32[$pParse>>2]|0;
     $177 = HEAP32[$pFrom>>2]|0;
     $178 = (_sqlite3IndexedByLookup($176,$177)|0);
     $179 = ($178|0)!=(0);
     if ($179) {
      label = 36;
      break;
     }
    }
    $180 = HEAP32[$i>>2]|0;
    $181 = (($180) + 1)|0;
    HEAP32[$i>>2] = $181;
    $182 = HEAP32[$pFrom>>2]|0;
    $183 = ((($182)) + 72|0);
    HEAP32[$pFrom>>2] = $183;
   }
   if ((label|0) == 12) {
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 16) {
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 18) {
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 24) {
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 26) {
    $124 = HEAP32[$pParse>>2]|0;
    $125 = HEAP32[$pTab>>2]|0;
    $126 = HEAP32[$125>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $126;
    _sqlite3ErrorMsg($124,30122,$vararg_buffer4);
    $127 = HEAP32[$pFrom>>2]|0;
    $128 = ((($127)) + 16|0);
    HEAP32[$128>>2] = 0;
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 29) {
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 33) {
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 36) {
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
   else if ((label|0) == 38) {
    $184 = HEAP32[$db>>2]|0;
    $185 = ((($184)) + 69|0);
    $186 = HEAP8[$185>>0]|0;
    $187 = $186&255;
    $188 = ($187|0)!=(0);
    if (!($188)) {
     $189 = HEAP32[$pParse>>2]|0;
     $190 = HEAP32[$2>>2]|0;
     $191 = (_sqliteProcessJoin($189,$190)|0);
     $192 = ($191|0)!=(0);
     if (!($192)) {
      HEAP32[$k>>2] = 0;
      while(1) {
       $193 = HEAP32[$k>>2]|0;
       $194 = HEAP32[$pEList>>2]|0;
       $195 = HEAP32[$194>>2]|0;
       $196 = ($193|0)<($195|0);
       if (!($196)) {
        break;
       }
       $197 = HEAP32[$k>>2]|0;
       $198 = HEAP32[$pEList>>2]|0;
       $199 = ((($198)) + 4|0);
       $200 = HEAP32[$199>>2]|0;
       $201 = (($200) + (($197*20)|0)|0);
       $202 = HEAP32[$201>>2]|0;
       HEAP32[$pE>>2] = $202;
       $203 = HEAP32[$pE>>2]|0;
       $204 = HEAP8[$203>>0]|0;
       $205 = $204&255;
       $206 = ($205|0)==(160);
       if ($206) {
        break;
       }
       $207 = HEAP32[$pE>>2]|0;
       $208 = HEAP8[$207>>0]|0;
       $209 = $208&255;
       $210 = ($209|0)==(122);
       if ($210) {
        $211 = HEAP32[$pE>>2]|0;
        $212 = ((($211)) + 16|0);
        $213 = HEAP32[$212>>2]|0;
        $214 = HEAP8[$213>>0]|0;
        $215 = $214&255;
        $216 = ($215|0)==(160);
        if ($216) {
         break;
        }
       }
       $217 = HEAP32[$k>>2]|0;
       $218 = (($217) + 1)|0;
       HEAP32[$k>>2] = $218;
      }
      $219 = HEAP32[$k>>2]|0;
      $220 = HEAP32[$pEList>>2]|0;
      $221 = HEAP32[$220>>2]|0;
      $222 = ($219|0)<($221|0);
      if ($222) {
       $223 = HEAP32[$pEList>>2]|0;
       $224 = ((($223)) + 4|0);
       $225 = HEAP32[$224>>2]|0;
       HEAP32[$a>>2] = $225;
       HEAP32[$pNew>>2] = 0;
       $226 = HEAP32[$pParse>>2]|0;
       $227 = HEAP32[$226>>2]|0;
       $228 = ((($227)) + 24|0);
       $229 = HEAP32[$228>>2]|0;
       HEAP32[$flags>>2] = $229;
       $230 = HEAP32[$flags>>2]|0;
       $231 = $230 & 4;
       $232 = ($231|0)!=(0);
       if ($232) {
        $233 = HEAP32[$flags>>2]|0;
        $234 = $233 & 64;
        $235 = ($234|0)==(0);
        $237 = $235;
       } else {
        $237 = 0;
       }
       $236 = $237&1;
       HEAP32[$longNames>>2] = $236;
       HEAP32[$k>>2] = 0;
       while(1) {
        $238 = HEAP32[$k>>2]|0;
        $239 = HEAP32[$pEList>>2]|0;
        $240 = HEAP32[$239>>2]|0;
        $241 = ($238|0)<($240|0);
        if (!($241)) {
         break;
        }
        $242 = HEAP32[$k>>2]|0;
        $243 = HEAP32[$a>>2]|0;
        $244 = (($243) + (($242*20)|0)|0);
        $245 = HEAP32[$244>>2]|0;
        HEAP32[$pE>>2] = $245;
        $246 = HEAP32[$pE>>2]|0;
        $247 = ((($246)) + 16|0);
        $248 = HEAP32[$247>>2]|0;
        HEAP32[$pRight>>2] = $248;
        $249 = HEAP32[$pE>>2]|0;
        $250 = HEAP8[$249>>0]|0;
        $251 = $250&255;
        $252 = ($251|0)!=(160);
        do {
         if ($252) {
          $253 = HEAP32[$pE>>2]|0;
          $254 = HEAP8[$253>>0]|0;
          $255 = $254&255;
          $256 = ($255|0)!=(122);
          if (!($256)) {
           $257 = HEAP32[$pRight>>2]|0;
           $258 = HEAP8[$257>>0]|0;
           $259 = $258&255;
           $260 = ($259|0)!=(160);
           if (!($260)) {
            label = 58;
            break;
           }
          }
          $261 = HEAP32[$pParse>>2]|0;
          $262 = HEAP32[$pNew>>2]|0;
          $263 = HEAP32[$k>>2]|0;
          $264 = HEAP32[$a>>2]|0;
          $265 = (($264) + (($263*20)|0)|0);
          $266 = HEAP32[$265>>2]|0;
          $267 = (_sqlite3ExprListAppend($261,$262,$266)|0);
          HEAP32[$pNew>>2] = $267;
          $268 = HEAP32[$pNew>>2]|0;
          $269 = ($268|0)!=(0|0);
          if ($269) {
           $270 = HEAP32[$k>>2]|0;
           $271 = HEAP32[$a>>2]|0;
           $272 = (($271) + (($270*20)|0)|0);
           $273 = ((($272)) + 4|0);
           $274 = HEAP32[$273>>2]|0;
           $275 = HEAP32[$pNew>>2]|0;
           $276 = HEAP32[$275>>2]|0;
           $277 = (($276) - 1)|0;
           $278 = HEAP32[$pNew>>2]|0;
           $279 = ((($278)) + 4|0);
           $280 = HEAP32[$279>>2]|0;
           $281 = (($280) + (($277*20)|0)|0);
           $282 = ((($281)) + 4|0);
           HEAP32[$282>>2] = $274;
           $283 = HEAP32[$k>>2]|0;
           $284 = HEAP32[$a>>2]|0;
           $285 = (($284) + (($283*20)|0)|0);
           $286 = ((($285)) + 8|0);
           $287 = HEAP32[$286>>2]|0;
           $288 = HEAP32[$pNew>>2]|0;
           $289 = HEAP32[$288>>2]|0;
           $290 = (($289) - 1)|0;
           $291 = HEAP32[$pNew>>2]|0;
           $292 = ((($291)) + 4|0);
           $293 = HEAP32[$292>>2]|0;
           $294 = (($293) + (($290*20)|0)|0);
           $295 = ((($294)) + 8|0);
           HEAP32[$295>>2] = $287;
           $296 = HEAP32[$k>>2]|0;
           $297 = HEAP32[$a>>2]|0;
           $298 = (($297) + (($296*20)|0)|0);
           $299 = ((($298)) + 4|0);
           HEAP32[$299>>2] = 0;
           $300 = HEAP32[$k>>2]|0;
           $301 = HEAP32[$a>>2]|0;
           $302 = (($301) + (($300*20)|0)|0);
           $303 = ((($302)) + 8|0);
           HEAP32[$303>>2] = 0;
          }
          $304 = HEAP32[$k>>2]|0;
          $305 = HEAP32[$a>>2]|0;
          $306 = (($305) + (($304*20)|0)|0);
          HEAP32[$306>>2] = 0;
         } else {
          label = 58;
         }
        } while(0);
        do {
         if ((label|0) == 58) {
          label = 0;
          HEAP32[$tableSeen>>2] = 0;
          HEAP32[$zTName>>2] = 0;
          $307 = HEAP32[$pE>>2]|0;
          $308 = HEAP8[$307>>0]|0;
          $309 = $308&255;
          $310 = ($309|0)==(122);
          if ($310) {
           $311 = HEAP32[$pE>>2]|0;
           $312 = ((($311)) + 12|0);
           $313 = HEAP32[$312>>2]|0;
           $314 = ((($313)) + 8|0);
           $315 = HEAP32[$314>>2]|0;
           HEAP32[$zTName>>2] = $315;
          }
          HEAP32[$i>>2] = 0;
          $316 = HEAP32[$pTabList>>2]|0;
          $317 = ((($316)) + 8|0);
          HEAP32[$pFrom>>2] = $317;
          while(1) {
           $318 = HEAP32[$i>>2]|0;
           $319 = HEAP32[$pTabList>>2]|0;
           $320 = HEAP32[$319>>2]|0;
           $321 = ($318|0)<($320|0);
           if (!($321)) {
            break;
           }
           $322 = HEAP32[$pFrom>>2]|0;
           $323 = ((($322)) + 16|0);
           $324 = HEAP32[$323>>2]|0;
           HEAP32[$pTab1>>2] = $324;
           $325 = HEAP32[$pFrom>>2]|0;
           $326 = ((($325)) + 20|0);
           $327 = HEAP32[$326>>2]|0;
           HEAP32[$pSub>>2] = $327;
           $328 = HEAP32[$pFrom>>2]|0;
           $329 = ((($328)) + 12|0);
           $330 = HEAP32[$329>>2]|0;
           HEAP32[$zTabName>>2] = $330;
           HEAP32[$zSchemaName>>2] = 0;
           $331 = HEAP32[$zTabName>>2]|0;
           $332 = ($331|0)==(0|0);
           if ($332) {
            $333 = HEAP32[$pTab1>>2]|0;
            $334 = HEAP32[$333>>2]|0;
            HEAP32[$zTabName>>2] = $334;
           }
           $335 = HEAP32[$db>>2]|0;
           $336 = ((($335)) + 69|0);
           $337 = HEAP8[$336>>0]|0;
           $338 = ($337<<24>>24)!=(0);
           if ($338) {
            break;
           }
           $339 = HEAP32[$pSub>>2]|0;
           $340 = ($339|0)==(0|0);
           if ($340) {
            label = 67;
           } else {
            $341 = HEAP32[$pSub>>2]|0;
            $342 = ((($341)) + 8|0);
            $343 = HEAP32[$342>>2]|0;
            $344 = $343 & 2048;
            $345 = ($344|0)==(0);
            if ($345) {
             label = 67;
            } else {
             label = 72;
            }
           }
           do {
            if ((label|0) == 67) {
             label = 0;
             HEAP32[$pSub>>2] = 0;
             $346 = HEAP32[$zTName>>2]|0;
             $347 = ($346|0)!=(0|0);
             if ($347) {
              $348 = HEAP32[$zTName>>2]|0;
              $349 = HEAP32[$zTabName>>2]|0;
              $350 = (_sqlite3StrICmp($348,$349)|0);
              $351 = ($350|0)!=(0);
              if ($351) {
               break;
              }
             }
             $352 = HEAP32[$db>>2]|0;
             $353 = HEAP32[$pTab1>>2]|0;
             $354 = ((($353)) + 64|0);
             $355 = HEAP32[$354>>2]|0;
             $356 = (_sqlite3SchemaToIndex($352,$355)|0);
             HEAP32[$iDb>>2] = $356;
             $357 = HEAP32[$iDb>>2]|0;
             $358 = ($357|0)>=(0);
             if ($358) {
              $359 = HEAP32[$iDb>>2]|0;
              $360 = HEAP32[$db>>2]|0;
              $361 = ((($360)) + 16|0);
              $362 = HEAP32[$361>>2]|0;
              $363 = (($362) + ($359<<4)|0);
              $364 = HEAP32[$363>>2]|0;
              $365 = $364;
             } else {
              $365 = 25451;
             }
             HEAP32[$zSchemaName>>2] = $365;
             label = 72;
            }
           } while(0);
           L90: do {
            if ((label|0) == 72) {
             label = 0;
             HEAP32[$j>>2] = 0;
             while(1) {
              $366 = HEAP32[$j>>2]|0;
              $367 = HEAP32[$pTab1>>2]|0;
              $368 = ((($367)) + 34|0);
              $369 = HEAP16[$368>>1]|0;
              $370 = $369 << 16 >> 16;
              $371 = ($366|0)<($370|0);
              if (!($371)) {
               break L90;
              }
              $372 = HEAP32[$j>>2]|0;
              $373 = HEAP32[$pTab1>>2]|0;
              $374 = ((($373)) + 4|0);
              $375 = HEAP32[$374>>2]|0;
              $376 = (($375) + ($372<<4)|0);
              $377 = HEAP32[$376>>2]|0;
              HEAP32[$zName>>2] = $377;
              $378 = HEAP32[$zTName>>2]|0;
              $379 = ($378|0)!=(0|0);
              $380 = HEAP32[$pSub>>2]|0;
              $381 = ($380|0)!=(0|0);
              $or$cond = $379 & $381;
              if ($or$cond) {
               $382 = HEAP32[$j>>2]|0;
               $383 = HEAP32[$pSub>>2]|0;
               $384 = HEAP32[$383>>2]|0;
               $385 = ((($384)) + 4|0);
               $386 = HEAP32[$385>>2]|0;
               $387 = (($386) + (($382*20)|0)|0);
               $388 = ((($387)) + 8|0);
               $389 = HEAP32[$388>>2]|0;
               $390 = HEAP32[$zTName>>2]|0;
               $391 = (_sqlite3MatchSpanName($389,0,$390,0)|0);
               $392 = ($391|0)==(0);
               if (!($392)) {
                label = 76;
               }
              } else {
               label = 76;
              }
              do {
               if ((label|0) == 76) {
                label = 0;
                $393 = HEAP32[$2>>2]|0;
                $394 = ((($393)) + 8|0);
                $395 = HEAP32[$394>>2]|0;
                $396 = $395 & 131072;
                $397 = ($396|0)==(0);
                if ($397) {
                 $398 = HEAP32[$j>>2]|0;
                 $399 = HEAP32[$pTab1>>2]|0;
                 $400 = ((($399)) + 4|0);
                 $401 = HEAP32[$400>>2]|0;
                 $402 = (($401) + ($398<<4)|0);
                 $403 = ((($402)) + 15|0);
                 $404 = HEAP8[$403>>0]|0;
                 $405 = $404&255;
                 $406 = $405 & 2;
                 $407 = ($406|0)!=(0);
                 if ($407) {
                  break;
                 }
                }
                HEAP32[$tableSeen>>2] = 1;
                $408 = HEAP32[$i>>2]|0;
                $409 = ($408|0)>(0);
                $410 = HEAP32[$zTName>>2]|0;
                $411 = ($410|0)==(0|0);
                $or$cond3 = $409 & $411;
                if ($or$cond3) {
                 $412 = HEAP32[$pFrom>>2]|0;
                 $413 = ((($412)) + 36|0);
                 $414 = HEAP8[$413>>0]|0;
                 $415 = $414&255;
                 $416 = $415 & 4;
                 $417 = ($416|0)!=(0);
                 if ($417) {
                  $418 = HEAP32[$pTabList>>2]|0;
                  $419 = HEAP32[$i>>2]|0;
                  $420 = HEAP32[$zName>>2]|0;
                  $421 = (_tableAndColumnIndex($418,$419,$420,0,0)|0);
                  $422 = ($421|0)!=(0);
                  if ($422) {
                   break;
                  }
                 }
                 $423 = HEAP32[$pFrom>>2]|0;
                 $424 = ((($423)) + 52|0);
                 $425 = HEAP32[$424>>2]|0;
                 $426 = HEAP32[$zName>>2]|0;
                 $427 = (_sqlite3IdListIndex($425,$426)|0);
                 $428 = ($427|0)>=(0);
                 if ($428) {
                  break;
                 }
                }
                $429 = HEAP32[$db>>2]|0;
                $430 = HEAP32[$zName>>2]|0;
                $431 = (_sqlite3Expr($429,55,$430)|0);
                HEAP32[$pRight>>2] = $431;
                $432 = HEAP32[$zName>>2]|0;
                HEAP32[$zColname>>2] = $432;
                HEAP32[$zToFree>>2] = 0;
                $433 = HEAP32[$longNames>>2]|0;
                $434 = ($433|0)!=(0);
                do {
                 if ($434) {
                  label = 84;
                 } else {
                  $435 = HEAP32[$pTabList>>2]|0;
                  $436 = HEAP32[$435>>2]|0;
                  $437 = ($436|0)>(1);
                  if ($437) {
                   label = 84;
                   break;
                  }
                  $461 = HEAP32[$pRight>>2]|0;
                  HEAP32[$pExpr>>2] = $461;
                 }
                } while(0);
                do {
                 if ((label|0) == 84) {
                  label = 0;
                  $438 = HEAP32[$db>>2]|0;
                  $439 = HEAP32[$zTabName>>2]|0;
                  $440 = (_sqlite3Expr($438,55,$439)|0);
                  HEAP32[$pLeft>>2] = $440;
                  $441 = HEAP32[$pParse>>2]|0;
                  $442 = HEAP32[$pLeft>>2]|0;
                  $443 = HEAP32[$pRight>>2]|0;
                  $444 = (_sqlite3PExpr($441,122,$442,$443,0)|0);
                  HEAP32[$pExpr>>2] = $444;
                  $445 = HEAP32[$zSchemaName>>2]|0;
                  $446 = ($445|0)!=(0|0);
                  if ($446) {
                   $447 = HEAP32[$db>>2]|0;
                   $448 = HEAP32[$zSchemaName>>2]|0;
                   $449 = (_sqlite3Expr($447,55,$448)|0);
                   HEAP32[$pLeft>>2] = $449;
                   $450 = HEAP32[$pParse>>2]|0;
                   $451 = HEAP32[$pLeft>>2]|0;
                   $452 = HEAP32[$pExpr>>2]|0;
                   $453 = (_sqlite3PExpr($450,122,$451,$452,0)|0);
                   HEAP32[$pExpr>>2] = $453;
                  }
                  $454 = HEAP32[$longNames>>2]|0;
                  $455 = ($454|0)!=(0);
                  if (!($455)) {
                   break;
                  }
                  $456 = HEAP32[$db>>2]|0;
                  $457 = HEAP32[$zTabName>>2]|0;
                  $458 = HEAP32[$zName>>2]|0;
                  HEAP32[$vararg_buffer7>>2] = $457;
                  $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
                  HEAP32[$vararg_ptr10>>2] = $458;
                  $459 = (_sqlite3MPrintf($456,30161,$vararg_buffer7)|0);
                  HEAP32[$zColname>>2] = $459;
                  $460 = HEAP32[$zColname>>2]|0;
                  HEAP32[$zToFree>>2] = $460;
                 }
                } while(0);
                $462 = HEAP32[$pParse>>2]|0;
                $463 = HEAP32[$pNew>>2]|0;
                $464 = HEAP32[$pExpr>>2]|0;
                $465 = (_sqlite3ExprListAppend($462,$463,$464)|0);
                HEAP32[$pNew>>2] = $465;
                $466 = HEAP32[$zColname>>2]|0;
                _sqlite3TokenInit($sColname,$466);
                $467 = HEAP32[$pParse>>2]|0;
                $468 = HEAP32[$pNew>>2]|0;
                _sqlite3ExprListSetName($467,$468,$sColname,0);
                $469 = HEAP32[$pNew>>2]|0;
                $470 = ($469|0)!=(0|0);
                do {
                 if ($470) {
                  $471 = HEAP32[$2>>2]|0;
                  $472 = ((($471)) + 8|0);
                  $473 = HEAP32[$472>>2]|0;
                  $474 = $473 & 2048;
                  $475 = ($474|0)!=(0);
                  if (!($475)) {
                   break;
                  }
                  $476 = HEAP32[$pNew>>2]|0;
                  $477 = HEAP32[$476>>2]|0;
                  $478 = (($477) - 1)|0;
                  $479 = HEAP32[$pNew>>2]|0;
                  $480 = ((($479)) + 4|0);
                  $481 = HEAP32[$480>>2]|0;
                  $482 = (($481) + (($478*20)|0)|0);
                  HEAP32[$pX>>2] = $482;
                  $483 = HEAP32[$pSub>>2]|0;
                  $484 = ($483|0)!=(0|0);
                  $485 = HEAP32[$db>>2]|0;
                  if ($484) {
                   $486 = HEAP32[$j>>2]|0;
                   $487 = HEAP32[$pSub>>2]|0;
                   $488 = HEAP32[$487>>2]|0;
                   $489 = ((($488)) + 4|0);
                   $490 = HEAP32[$489>>2]|0;
                   $491 = (($490) + (($486*20)|0)|0);
                   $492 = ((($491)) + 8|0);
                   $493 = HEAP32[$492>>2]|0;
                   $494 = (_sqlite3DbStrDup($485,$493)|0);
                   $495 = HEAP32[$pX>>2]|0;
                   $496 = ((($495)) + 8|0);
                   HEAP32[$496>>2] = $494;
                  } else {
                   $497 = HEAP32[$zSchemaName>>2]|0;
                   $498 = HEAP32[$zTabName>>2]|0;
                   $499 = HEAP32[$zColname>>2]|0;
                   HEAP32[$vararg_buffer11>>2] = $497;
                   $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
                   HEAP32[$vararg_ptr14>>2] = $498;
                   $vararg_ptr15 = ((($vararg_buffer11)) + 8|0);
                   HEAP32[$vararg_ptr15>>2] = $499;
                   $500 = (_sqlite3MPrintf($485,30167,$vararg_buffer11)|0);
                   $501 = HEAP32[$pX>>2]|0;
                   $502 = ((($501)) + 8|0);
                   HEAP32[$502>>2] = $500;
                  }
                  $503 = HEAP32[$pX>>2]|0;
                  $504 = ((($503)) + 13|0);
                  $505 = HEAP8[$504>>0]|0;
                  $506 = $505 & -3;
                  $507 = $506 | 2;
                  HEAP8[$504>>0] = $507;
                 }
                } while(0);
                $508 = HEAP32[$db>>2]|0;
                $509 = HEAP32[$zToFree>>2]|0;
                _sqlite3DbFree($508,$509);
               }
              } while(0);
              $510 = HEAP32[$j>>2]|0;
              $511 = (($510) + 1)|0;
              HEAP32[$j>>2] = $511;
             }
            }
           } while(0);
           $512 = HEAP32[$i>>2]|0;
           $513 = (($512) + 1)|0;
           HEAP32[$i>>2] = $513;
           $514 = HEAP32[$pFrom>>2]|0;
           $515 = ((($514)) + 72|0);
           HEAP32[$pFrom>>2] = $515;
          }
          $516 = HEAP32[$tableSeen>>2]|0;
          $517 = ($516|0)!=(0);
          if (!($517)) {
           $518 = HEAP32[$zTName>>2]|0;
           $519 = ($518|0)!=(0|0);
           $520 = HEAP32[$pParse>>2]|0;
           if ($519) {
            $521 = HEAP32[$zTName>>2]|0;
            HEAP32[$vararg_buffer16>>2] = $521;
            _sqlite3ErrorMsg($520,30176,$vararg_buffer16);
            break;
           } else {
            _sqlite3ErrorMsg($520,30194,$vararg_buffer19);
            break;
           }
          }
         }
        } while(0);
        $522 = HEAP32[$k>>2]|0;
        $523 = (($522) + 1)|0;
        HEAP32[$k>>2] = $523;
       }
       $524 = HEAP32[$db>>2]|0;
       $525 = HEAP32[$pEList>>2]|0;
       _sqlite3ExprListDelete($524,$525);
       $526 = HEAP32[$pNew>>2]|0;
       $527 = HEAP32[$2>>2]|0;
       HEAP32[$527>>2] = $526;
      }
      $528 = HEAP32[$2>>2]|0;
      $529 = HEAP32[$528>>2]|0;
      $530 = ($529|0)!=(0|0);
      if ($530) {
       $531 = HEAP32[$2>>2]|0;
       $532 = HEAP32[$531>>2]|0;
       $533 = HEAP32[$532>>2]|0;
       $534 = HEAP32[$db>>2]|0;
       $535 = ((($534)) + 96|0);
       $536 = ((($535)) + 8|0);
       $537 = HEAP32[$536>>2]|0;
       $538 = ($533|0)>($537|0);
       if ($538) {
        $539 = HEAP32[$pParse>>2]|0;
        _sqlite3ErrorMsg($539,30214,$vararg_buffer21);
        HEAP32[$0>>2] = 2;
        $540 = HEAP32[$0>>2]|0;
        STACKTOP = sp;return ($540|0);
       }
      }
      HEAP32[$0>>2] = 0;
      $540 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($540|0);
     }
    }
    HEAP32[$0>>2] = 2;
    $540 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($540|0);
   }
  }
 }
 HEAP32[$0>>2] = 1;
 $540 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($540|0);
}
function _selectPopWith($pWalker,$p) {
 $pWalker = $pWalker|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pParse = 0, $pWith = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pParse = sp + 4|0;
 $pWith = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pParse>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_findRightmost($4)|0);
 $6 = ((($5)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pWith>>2] = $7;
 $8 = HEAP32[$pWith>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pWith>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pParse>>2]|0;
 $14 = ((($13)) + 472|0);
 HEAP32[$14>>2] = $12;
 STACKTOP = sp;return;
}
function _findRightmost($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ((($1)) + 52|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  $5 = HEAP32[$0>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 52|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$0>>2] = $7;
 }
 STACKTOP = sp;return ($5|0);
}
function _withExpand($pWalker,$pFrom) {
 $pWalker = $pWalker|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bMayRecursive = 0, $db = 0, $i = 0, $pCte = 0, $pEList = 0, $pItem = 0, $pLeft = 0, $pParse = 0, $pSavedWith = 0;
 var $pSel = 0, $pSrc = 0, $pTab = 0, $pWith = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $pParse = sp + 76|0;
 $db = sp + 72|0;
 $pCte = sp + 68|0;
 $pWith = sp + 64|0;
 $pTab = sp + 60|0;
 $pEList = sp + 56|0;
 $pSel = sp + 52|0;
 $pLeft = sp + 48|0;
 $bMayRecursive = sp + 44|0;
 $pSavedWith = sp + 40|0;
 $i = sp + 36|0;
 $pSrc = sp + 32|0;
 $pItem = sp + 28|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = ((($7)) + 472|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_searchWith($9,$10,$pWith)|0);
 HEAP32[$pCte>>2] = $11;
 $12 = HEAP32[$pCte>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$pCte>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  $18 = HEAP32[$pParse>>2]|0;
  if ($17) {
   $19 = HEAP32[$pCte>>2]|0;
   $20 = ((($19)) + 12|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$pCte>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$vararg_buffer>>2] = $23;
   _sqlite3ErrorMsg($18,$21,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  }
  $24 = HEAP32[$2>>2]|0;
  $25 = (_cannotBeFunction($18,$24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   HEAP32[$0>>2] = 1;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  }
  $27 = HEAP32[$db>>2]|0;
  $28 = (_sqlite3DbMallocZero($27,72,0)|0);
  HEAP32[$pTab>>2] = $28;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  HEAP32[$30>>2] = $28;
  $31 = HEAP32[$pTab>>2]|0;
  $32 = ($31|0)==(0|0);
  if ($32) {
   HEAP32[$0>>2] = 2;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  }
  $33 = HEAP32[$pTab>>2]|0;
  $34 = ((($33)) + 36|0);
  HEAP16[$34>>1] = 1;
  $35 = HEAP32[$db>>2]|0;
  $36 = HEAP32[$pCte>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3DbStrDup($35,$37)|0);
  $39 = HEAP32[$pTab>>2]|0;
  HEAP32[$39>>2] = $38;
  $40 = HEAP32[$pTab>>2]|0;
  $41 = ((($40)) + 32|0);
  HEAP16[$41>>1] = -1;
  $42 = HEAP32[$pTab>>2]|0;
  $43 = ((($42)) + 38|0);
  HEAP16[$43>>1] = 200;
  $44 = HEAP32[$pTab>>2]|0;
  $45 = ((($44)) + 42|0);
  $46 = HEAP8[$45>>0]|0;
  $47 = $46&255;
  $48 = $47 | 66;
  $49 = $48&255;
  HEAP8[$45>>0] = $49;
  $50 = HEAP32[$db>>2]|0;
  $51 = HEAP32[$pCte>>2]|0;
  $52 = ((($51)) + 8|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = (_sqlite3SelectDup($50,$53,0)|0);
  $55 = HEAP32[$2>>2]|0;
  $56 = ((($55)) + 20|0);
  HEAP32[$56>>2] = $54;
  $57 = HEAP32[$db>>2]|0;
  $58 = ((($57)) + 69|0);
  $59 = HEAP8[$58>>0]|0;
  $60 = ($59<<24>>24)!=(0);
  if ($60) {
   HEAP32[$0>>2] = 7;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  }
  $61 = HEAP32[$2>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  HEAP32[$pSel>>2] = $63;
  $64 = HEAP32[$pSel>>2]|0;
  $65 = ((($64)) + 4|0);
  $66 = HEAP8[$65>>0]|0;
  $67 = $66&255;
  $68 = ($67|0)==(116);
  if ($68) {
   $75 = 1;
  } else {
   $69 = HEAP32[$pSel>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = ($72|0)==(115);
   $75 = $73;
  }
  $74 = $75&1;
  HEAP32[$bMayRecursive>>2] = $74;
  $76 = HEAP32[$bMayRecursive>>2]|0;
  $77 = ($76|0)!=(0);
  L22: do {
   if ($77) {
    $78 = HEAP32[$2>>2]|0;
    $79 = ((($78)) + 20|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ((($80)) + 28|0);
    $82 = HEAP32[$81>>2]|0;
    HEAP32[$pSrc>>2] = $82;
    HEAP32[$i>>2] = 0;
    while(1) {
     $83 = HEAP32[$i>>2]|0;
     $84 = HEAP32[$pSrc>>2]|0;
     $85 = HEAP32[$84>>2]|0;
     $86 = ($83|0)<($85|0);
     if (!($86)) {
      break L22;
     }
     $87 = HEAP32[$i>>2]|0;
     $88 = HEAP32[$pSrc>>2]|0;
     $89 = ((($88)) + 8|0);
     $90 = (($89) + (($87*72)|0)|0);
     HEAP32[$pItem>>2] = $90;
     $91 = HEAP32[$pItem>>2]|0;
     $92 = ((($91)) + 4|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = ($93|0)==(0|0);
     if ($94) {
      $95 = HEAP32[$pItem>>2]|0;
      $96 = ((($95)) + 8|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = ($97|0)!=(0|0);
      if ($98) {
       $99 = HEAP32[$pItem>>2]|0;
       $100 = ((($99)) + 8|0);
       $101 = HEAP32[$100>>2]|0;
       $102 = HEAP32[$pCte>>2]|0;
       $103 = HEAP32[$102>>2]|0;
       $104 = (_sqlite3StrICmp($101,$103)|0);
       $105 = (0)==($104|0);
       if ($105) {
        $106 = HEAP32[$pTab>>2]|0;
        $107 = HEAP32[$pItem>>2]|0;
        $108 = ((($107)) + 16|0);
        HEAP32[$108>>2] = $106;
        $109 = HEAP32[$pItem>>2]|0;
        $110 = ((($109)) + 36|0);
        $111 = ((($110)) + 1|0);
        $112 = HEAP8[$111>>0]|0;
        $113 = $112 & -33;
        $114 = $113 | 32;
        HEAP8[$111>>0] = $114;
        $115 = HEAP32[$pTab>>2]|0;
        $116 = ((($115)) + 36|0);
        $117 = HEAP16[$116>>1]|0;
        $118 = (($117) + 1)<<16>>16;
        HEAP16[$116>>1] = $118;
        $119 = HEAP32[$pSel>>2]|0;
        $120 = ((($119)) + 8|0);
        $121 = HEAP32[$120>>2]|0;
        $122 = $121 | 8192;
        HEAP32[$120>>2] = $122;
       }
      }
     }
     $123 = HEAP32[$i>>2]|0;
     $124 = (($123) + 1)|0;
     HEAP32[$i>>2] = $124;
    }
   }
  } while(0);
  $125 = HEAP32[$pTab>>2]|0;
  $126 = ((($125)) + 36|0);
  $127 = HEAP16[$126>>1]|0;
  $128 = $127&65535;
  $129 = ($128|0)>(2);
  if ($129) {
   $130 = HEAP32[$pParse>>2]|0;
   $131 = HEAP32[$pCte>>2]|0;
   $132 = HEAP32[$131>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $132;
   _sqlite3ErrorMsg($130,30455,$vararg_buffer1);
   HEAP32[$0>>2] = 1;
   $210 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($210|0);
  }
  $133 = HEAP32[$pCte>>2]|0;
  $134 = ((($133)) + 12|0);
  HEAP32[$134>>2] = 30498;
  $135 = HEAP32[$pParse>>2]|0;
  $136 = ((($135)) + 472|0);
  $137 = HEAP32[$136>>2]|0;
  HEAP32[$pSavedWith>>2] = $137;
  $138 = HEAP32[$pWith>>2]|0;
  $139 = HEAP32[$pParse>>2]|0;
  $140 = ((($139)) + 472|0);
  HEAP32[$140>>2] = $138;
  $141 = HEAP32[$1>>2]|0;
  $142 = HEAP32[$bMayRecursive>>2]|0;
  $143 = ($142|0)!=(0);
  $144 = HEAP32[$pSel>>2]|0;
  if ($143) {
   $145 = ((($144)) + 48|0);
   $146 = HEAP32[$145>>2]|0;
   $147 = $146;
  } else {
   $147 = $144;
  }
  (_sqlite3WalkSelect($141,$147)|0);
  $148 = HEAP32[$pWith>>2]|0;
  $149 = HEAP32[$pParse>>2]|0;
  $150 = ((($149)) + 472|0);
  HEAP32[$150>>2] = $148;
  $151 = HEAP32[$pSel>>2]|0;
  HEAP32[$pLeft>>2] = $151;
  while(1) {
   $152 = HEAP32[$pLeft>>2]|0;
   $153 = ((($152)) + 48|0);
   $154 = HEAP32[$153>>2]|0;
   $155 = ($154|0)!=(0|0);
   $156 = HEAP32[$pLeft>>2]|0;
   if (!($155)) {
    break;
   }
   $157 = ((($156)) + 48|0);
   $158 = HEAP32[$157>>2]|0;
   HEAP32[$pLeft>>2] = $158;
  }
  $159 = HEAP32[$156>>2]|0;
  HEAP32[$pEList>>2] = $159;
  $160 = HEAP32[$pCte>>2]|0;
  $161 = ((($160)) + 4|0);
  $162 = HEAP32[$161>>2]|0;
  $163 = ($162|0)!=(0|0);
  if ($163) {
   $164 = HEAP32[$pEList>>2]|0;
   $165 = ($164|0)!=(0|0);
   if ($165) {
    $166 = HEAP32[$pEList>>2]|0;
    $167 = HEAP32[$166>>2]|0;
    $168 = HEAP32[$pCte>>2]|0;
    $169 = ((($168)) + 4|0);
    $170 = HEAP32[$169>>2]|0;
    $171 = HEAP32[$170>>2]|0;
    $172 = ($167|0)!=($171|0);
    if ($172) {
     $173 = HEAP32[$pParse>>2]|0;
     $174 = HEAP32[$pCte>>2]|0;
     $175 = HEAP32[$174>>2]|0;
     $176 = HEAP32[$pEList>>2]|0;
     $177 = HEAP32[$176>>2]|0;
     $178 = HEAP32[$pCte>>2]|0;
     $179 = ((($178)) + 4|0);
     $180 = HEAP32[$179>>2]|0;
     $181 = HEAP32[$180>>2]|0;
     HEAP32[$vararg_buffer4>>2] = $175;
     $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $177;
     $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
     HEAP32[$vararg_ptr8>>2] = $181;
     _sqlite3ErrorMsg($173,30521,$vararg_buffer4);
     $182 = HEAP32[$pSavedWith>>2]|0;
     $183 = HEAP32[$pParse>>2]|0;
     $184 = ((($183)) + 472|0);
     HEAP32[$184>>2] = $182;
     HEAP32[$0>>2] = 1;
     $210 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($210|0);
    }
   }
   $185 = HEAP32[$pCte>>2]|0;
   $186 = ((($185)) + 4|0);
   $187 = HEAP32[$186>>2]|0;
   HEAP32[$pEList>>2] = $187;
  }
  $188 = HEAP32[$pParse>>2]|0;
  $189 = HEAP32[$pEList>>2]|0;
  $190 = HEAP32[$pTab>>2]|0;
  $191 = ((($190)) + 34|0);
  $192 = HEAP32[$pTab>>2]|0;
  $193 = ((($192)) + 4|0);
  (_sqlite3ColumnsFromExprList($188,$189,$191,$193)|0);
  $194 = HEAP32[$bMayRecursive>>2]|0;
  $195 = ($194|0)!=(0);
  if ($195) {
   $196 = HEAP32[$pSel>>2]|0;
   $197 = ((($196)) + 8|0);
   $198 = HEAP32[$197>>2]|0;
   $199 = $198 & 8192;
   $200 = ($199|0)!=(0);
   $201 = HEAP32[$pCte>>2]|0;
   $202 = ((($201)) + 12|0);
   if ($200) {
    HEAP32[$202>>2] = 30559;
   } else {
    HEAP32[$202>>2] = 30593;
   }
   $203 = HEAP32[$1>>2]|0;
   $204 = HEAP32[$pSel>>2]|0;
   (_sqlite3WalkSelect($203,$204)|0);
  }
  $205 = HEAP32[$pCte>>2]|0;
  $206 = ((($205)) + 12|0);
  HEAP32[$206>>2] = 0;
  $207 = HEAP32[$pSavedWith>>2]|0;
  $208 = HEAP32[$pParse>>2]|0;
  $209 = ((($208)) + 472|0);
  HEAP32[$209>>2] = $207;
 }
 HEAP32[$0>>2] = 0;
 $210 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($210|0);
}
function _cannotBeFunction($pParse,$pFrom) {
 $pParse = $pParse|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = ((($4)) + 1|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6&255) >>> 2;
 $8 = $7 & 1;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$vararg_buffer>>2] = $14;
  _sqlite3ErrorMsg($11,30432,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  HEAP32[$0>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _sqlite3IndexedByLookup($pParse,$pFrom) {
 $pParse = $pParse|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pIdx = 0, $pTab = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zIndexedBy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pTab = sp + 16|0;
 $zIndexedBy = sp + 12|0;
 $pIdx = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pFrom;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 do {
  if ($6) {
   $7 = HEAP32[$2>>2]|0;
   $8 = ((($7)) + 36|0);
   $9 = ((($8)) + 1|0);
   $10 = HEAP8[$9>>0]|0;
   $11 = ($10&255) >>> 1;
   $12 = $11 & 1;
   $13 = $12&255;
   $14 = ($13|0)!=(0);
   if ($14) {
    $15 = HEAP32[$2>>2]|0;
    $16 = ((($15)) + 16|0);
    $17 = HEAP32[$16>>2]|0;
    HEAP32[$pTab>>2] = $17;
    $18 = HEAP32[$2>>2]|0;
    $19 = ((($18)) + 64|0);
    $20 = HEAP32[$19>>2]|0;
    HEAP32[$zIndexedBy>>2] = $20;
    $21 = HEAP32[$pTab>>2]|0;
    $22 = ((($21)) + 8|0);
    $23 = HEAP32[$22>>2]|0;
    HEAP32[$pIdx>>2] = $23;
    while(1) {
     $24 = HEAP32[$pIdx>>2]|0;
     $25 = ($24|0)!=(0|0);
     if (!($25)) {
      break;
     }
     $26 = HEAP32[$pIdx>>2]|0;
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$zIndexedBy>>2]|0;
     $29 = (_sqlite3StrICmp($27,$28)|0);
     $30 = ($29|0)!=(0);
     if (!($30)) {
      break;
     }
     $31 = HEAP32[$pIdx>>2]|0;
     $32 = ((($31)) + 20|0);
     $33 = HEAP32[$32>>2]|0;
     HEAP32[$pIdx>>2] = $33;
    }
    $34 = HEAP32[$pIdx>>2]|0;
    $35 = ($34|0)!=(0|0);
    if ($35) {
     $40 = HEAP32[$pIdx>>2]|0;
     $41 = HEAP32[$2>>2]|0;
     $42 = ((($41)) + 68|0);
     HEAP32[$42>>2] = $40;
     break;
    }
    $36 = HEAP32[$1>>2]|0;
    $37 = HEAP32[$zIndexedBy>>2]|0;
    HEAP32[$vararg_buffer>>2] = $37;
    $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr1>>2] = 0;
    _sqlite3ErrorMsg($36,30414,$vararg_buffer);
    $38 = HEAP32[$1>>2]|0;
    $39 = ((($38)) + 17|0);
    HEAP8[$39>>0] = 1;
    HEAP32[$0>>2] = 1;
    $43 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($43|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _sqliteProcessJoin($pParse,$p) {
 $pParse = $pParse|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $i = 0, $iLeft = 0, $iLeft2 = 0, $iLeftCol = 0, $iLeftCol3 = 0, $iRightCol = 0, $isOuter = 0, $j = 0, $or$cond = 0, $pLeft = 0, $pLeftTab = 0, $pList = 0, $pRight = 0, $pRightTab = 0, $pSrc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer4 = 0, $zName = 0, $zName1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $pSrc = sp + 80|0;
 $i = sp + 76|0;
 $j = sp + 72|0;
 $pLeft = sp + 68|0;
 $pRight = sp + 64|0;
 $pLeftTab = sp + 60|0;
 $pRightTab = sp + 56|0;
 $isOuter = sp + 52|0;
 $zName = sp + 48|0;
 $iLeft = sp + 44|0;
 $iLeftCol = sp + 40|0;
 $pList = sp + 36|0;
 $zName1 = sp + 32|0;
 $iLeft2 = sp + 28|0;
 $iLeftCol3 = sp + 24|0;
 $iRightCol = sp + 20|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = ((($6)) + 8|0);
 HEAP32[$pLeft>>2] = $7;
 $8 = HEAP32[$pLeft>>2]|0;
 $9 = ((($8)) + 72|0);
 HEAP32[$pRight>>2] = $9;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pSrc>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) - 1)|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   label = 28;
   break;
  }
  $15 = HEAP32[$pLeft>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pLeftTab>>2] = $17;
  $18 = HEAP32[$pRight>>2]|0;
  $19 = ((($18)) + 16|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pRightTab>>2] = $20;
  $21 = HEAP32[$pLeftTab>>2]|0;
  $22 = ($21|0)==(0|0);
  $23 = HEAP32[$pRightTab>>2]|0;
  $24 = ($23|0)==(0|0);
  $or$cond = $22 | $24;
  L4: do {
   if (!($or$cond)) {
    $25 = HEAP32[$pRight>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = $28 & 32;
    $30 = ($29|0)!=(0);
    $31 = $30&1;
    HEAP32[$isOuter>>2] = $31;
    $32 = HEAP32[$pRight>>2]|0;
    $33 = ((($32)) + 36|0);
    $34 = HEAP8[$33>>0]|0;
    $35 = $34&255;
    $36 = $35 & 4;
    $37 = ($36|0)!=(0);
    L6: do {
     if ($37) {
      $38 = HEAP32[$pRight>>2]|0;
      $39 = ((($38)) + 48|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = ($40|0)!=(0|0);
      if ($41) {
       label = 7;
       break L1;
      }
      $42 = HEAP32[$pRight>>2]|0;
      $43 = ((($42)) + 52|0);
      $44 = HEAP32[$43>>2]|0;
      $45 = ($44|0)!=(0|0);
      if ($45) {
       label = 7;
       break L1;
      }
      HEAP32[$j>>2] = 0;
      while(1) {
       $47 = HEAP32[$j>>2]|0;
       $48 = HEAP32[$pRightTab>>2]|0;
       $49 = ((($48)) + 34|0);
       $50 = HEAP16[$49>>1]|0;
       $51 = $50 << 16 >> 16;
       $52 = ($47|0)<($51|0);
       if (!($52)) {
        break L6;
       }
       $53 = HEAP32[$j>>2]|0;
       $54 = HEAP32[$pRightTab>>2]|0;
       $55 = ((($54)) + 4|0);
       $56 = HEAP32[$55>>2]|0;
       $57 = (($56) + ($53<<4)|0);
       $58 = HEAP32[$57>>2]|0;
       HEAP32[$zName>>2] = $58;
       $59 = HEAP32[$pSrc>>2]|0;
       $60 = HEAP32[$i>>2]|0;
       $61 = (($60) + 1)|0;
       $62 = HEAP32[$zName>>2]|0;
       $63 = (_tableAndColumnIndex($59,$61,$62,$iLeft,$iLeftCol)|0);
       $64 = ($63|0)!=(0);
       if ($64) {
        $65 = HEAP32[$1>>2]|0;
        $66 = HEAP32[$pSrc>>2]|0;
        $67 = HEAP32[$iLeft>>2]|0;
        $68 = HEAP32[$iLeftCol>>2]|0;
        $69 = HEAP32[$i>>2]|0;
        $70 = (($69) + 1)|0;
        $71 = HEAP32[$j>>2]|0;
        $72 = HEAP32[$isOuter>>2]|0;
        $73 = HEAP32[$2>>2]|0;
        $74 = ((($73)) + 32|0);
        _addWhereTerm($65,$66,$67,$68,$70,$71,$72,$74);
       }
       $75 = HEAP32[$j>>2]|0;
       $76 = (($75) + 1)|0;
       HEAP32[$j>>2] = $76;
      }
     }
    } while(0);
    $77 = HEAP32[$pRight>>2]|0;
    $78 = ((($77)) + 48|0);
    $79 = HEAP32[$78>>2]|0;
    $80 = ($79|0)!=(0|0);
    if ($80) {
     $81 = HEAP32[$pRight>>2]|0;
     $82 = ((($81)) + 52|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)!=(0|0);
     if ($84) {
      label = 15;
      break L1;
     }
    }
    $86 = HEAP32[$pRight>>2]|0;
    $87 = ((($86)) + 48|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = ($88|0)!=(0|0);
    if ($89) {
     $90 = HEAP32[$isOuter>>2]|0;
     $91 = ($90|0)!=(0);
     if ($91) {
      $92 = HEAP32[$pRight>>2]|0;
      $93 = ((($92)) + 48|0);
      $94 = HEAP32[$93>>2]|0;
      $95 = HEAP32[$pRight>>2]|0;
      $96 = ((($95)) + 44|0);
      $97 = HEAP32[$96>>2]|0;
      _setJoinExpr($94,$97);
     }
     $98 = HEAP32[$1>>2]|0;
     $99 = HEAP32[$98>>2]|0;
     $100 = HEAP32[$2>>2]|0;
     $101 = ((($100)) + 32|0);
     $102 = HEAP32[$101>>2]|0;
     $103 = HEAP32[$pRight>>2]|0;
     $104 = ((($103)) + 48|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (_sqlite3ExprAnd($99,$102,$105)|0);
     $107 = HEAP32[$2>>2]|0;
     $108 = ((($107)) + 32|0);
     HEAP32[$108>>2] = $106;
     $109 = HEAP32[$pRight>>2]|0;
     $110 = ((($109)) + 48|0);
     HEAP32[$110>>2] = 0;
    }
    $111 = HEAP32[$pRight>>2]|0;
    $112 = ((($111)) + 52|0);
    $113 = HEAP32[$112>>2]|0;
    $114 = ($113|0)!=(0|0);
    if ($114) {
     $115 = HEAP32[$pRight>>2]|0;
     $116 = ((($115)) + 52|0);
     $117 = HEAP32[$116>>2]|0;
     HEAP32[$pList>>2] = $117;
     HEAP32[$j>>2] = 0;
     while(1) {
      $118 = HEAP32[$j>>2]|0;
      $119 = HEAP32[$pList>>2]|0;
      $120 = ((($119)) + 4|0);
      $121 = HEAP32[$120>>2]|0;
      $122 = ($118|0)<($121|0);
      if (!($122)) {
       break L4;
      }
      $123 = HEAP32[$j>>2]|0;
      $124 = HEAP32[$pList>>2]|0;
      $125 = HEAP32[$124>>2]|0;
      $126 = (($125) + ($123<<3)|0);
      $127 = HEAP32[$126>>2]|0;
      HEAP32[$zName1>>2] = $127;
      $128 = HEAP32[$pRightTab>>2]|0;
      $129 = HEAP32[$zName1>>2]|0;
      $130 = (_columnIndex($128,$129)|0);
      HEAP32[$iRightCol>>2] = $130;
      $131 = HEAP32[$iRightCol>>2]|0;
      $132 = ($131|0)<(0);
      if ($132) {
       label = 25;
       break L1;
      }
      $133 = HEAP32[$pSrc>>2]|0;
      $134 = HEAP32[$i>>2]|0;
      $135 = (($134) + 1)|0;
      $136 = HEAP32[$zName1>>2]|0;
      $137 = (_tableAndColumnIndex($133,$135,$136,$iLeft2,$iLeftCol3)|0);
      $138 = ($137|0)!=(0);
      if (!($138)) {
       label = 25;
       break L1;
      }
      $141 = HEAP32[$1>>2]|0;
      $142 = HEAP32[$pSrc>>2]|0;
      $143 = HEAP32[$iLeft2>>2]|0;
      $144 = HEAP32[$iLeftCol3>>2]|0;
      $145 = HEAP32[$i>>2]|0;
      $146 = (($145) + 1)|0;
      $147 = HEAP32[$iRightCol>>2]|0;
      $148 = HEAP32[$isOuter>>2]|0;
      $149 = HEAP32[$2>>2]|0;
      $150 = ((($149)) + 32|0);
      _addWhereTerm($141,$142,$143,$144,$146,$147,$148,$150);
      $151 = HEAP32[$j>>2]|0;
      $152 = (($151) + 1)|0;
      HEAP32[$j>>2] = $152;
     }
    }
   }
  } while(0);
  $153 = HEAP32[$i>>2]|0;
  $154 = (($153) + 1)|0;
  HEAP32[$i>>2] = $154;
  $155 = HEAP32[$pRight>>2]|0;
  $156 = ((($155)) + 72|0);
  HEAP32[$pRight>>2] = $156;
  $157 = HEAP32[$pLeft>>2]|0;
  $158 = ((($157)) + 72|0);
  HEAP32[$pLeft>>2] = $158;
 }
 if ((label|0) == 7) {
  $46 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = 0;
  _sqlite3ErrorMsg($46,30245,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 15) {
  $85 = HEAP32[$1>>2]|0;
  _sqlite3ErrorMsg($85,30295,$vararg_buffer2);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 25) {
  $139 = HEAP32[$1>>2]|0;
  $140 = HEAP32[$zName1>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $140;
  _sqlite3ErrorMsg($139,30350,$vararg_buffer4);
  HEAP32[$0>>2] = 1;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 else if ((label|0) == 28) {
  HEAP32[$0>>2] = 0;
  $159 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($159|0);
 }
 return (0)|0;
}
function _tableAndColumnIndex($pSrc,$N,$zCol,$piTab,$piCol) {
 $pSrc = $pSrc|0;
 $N = $N|0;
 $zCol = $zCol|0;
 $piTab = $piTab|0;
 $piCol = $piCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $i = sp + 4|0;
 $iCol = sp;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $N;
 HEAP32[$3>>2] = $zCol;
 HEAP32[$4>>2] = $piTab;
 HEAP32[$5>>2] = $piCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = ($6|0)<($7|0);
  if (!($8)) {
   label = 8;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = (($11) + (($9*72)|0)|0);
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$3>>2]|0;
  $16 = (_columnIndex($14,$15)|0);
  HEAP32[$iCol>>2] = $16;
  $17 = HEAP32[$iCol>>2]|0;
  $18 = ($17|0)>=(0);
  if ($18) {
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$iCol>>2]|0;
  $24 = HEAP32[$5>>2]|0;
  HEAP32[$24>>2] = $23;
 }
 HEAP32[$0>>2] = 1;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _sqlite3IdListIndex($pList,$zName) {
 $pList = $pList|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = (_sqlite3StrICmp($14,$15)|0);
  $17 = ($16|0)==(0);
  $18 = HEAP32[$i>>2]|0;
  if ($17) {
   label = 6;
   break;
  }
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = $18;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _columnIndex($pTab,$zCol) {
 $pTab = $pTab|0;
 $zCol = $zCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 34|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6 << 16 >> 16;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<4)|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = (_sqlite3StrICmp($14,$15)|0);
  $17 = ($16|0)==(0);
  $18 = HEAP32[$i>>2]|0;
  if ($17) {
   label = 4;
   break;
  }
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = $18;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = -1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _addWhereTerm($pParse,$pSrc,$iLeft,$iColLeft,$iRight,$iColRight,$isOuterJoin,$ppWhere) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $iLeft = $iLeft|0;
 $iColLeft = $iColLeft|0;
 $iRight = $iRight|0;
 $iColRight = $iColRight|0;
 $isOuterJoin = $isOuterJoin|0;
 $ppWhere = $ppWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pE1 = 0, $pE2 = 0, $pEq = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $db = sp + 12|0;
 $pE1 = sp + 8|0;
 $pE2 = sp + 4|0;
 $pEq = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $iLeft;
 HEAP32[$3>>2] = $iColLeft;
 HEAP32[$4>>2] = $iRight;
 HEAP32[$5>>2] = $iColRight;
 HEAP32[$6>>2] = $isOuterJoin;
 HEAP32[$7>>2] = $ppWhere;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = (_sqlite3CreateColumnExpr($10,$11,$12,$13)|0);
 HEAP32[$pE1>>2] = $14;
 $15 = HEAP32[$db>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = HEAP32[$5>>2]|0;
 $19 = (_sqlite3CreateColumnExpr($15,$16,$17,$18)|0);
 HEAP32[$pE2>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$pE1>>2]|0;
 $22 = HEAP32[$pE2>>2]|0;
 $23 = (_sqlite3PExpr($20,37,$21,$22,0)|0);
 HEAP32[$pEq>>2] = $23;
 $24 = HEAP32[$pEq>>2]|0;
 $25 = ($24|0)!=(0|0);
 $26 = HEAP32[$6>>2]|0;
 $27 = ($26|0)!=(0);
 $or$cond = $25 & $27;
 if (!($or$cond)) {
  $38 = HEAP32[$db>>2]|0;
  $39 = HEAP32[$7>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pEq>>2]|0;
  $42 = (_sqlite3ExprAnd($38,$40,$41)|0);
  $43 = HEAP32[$7>>2]|0;
  HEAP32[$43>>2] = $42;
  STACKTOP = sp;return;
 }
 $28 = HEAP32[$pEq>>2]|0;
 $29 = ((($28)) + 4|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 | 1;
 HEAP32[$29>>2] = $31;
 $32 = HEAP32[$pE2>>2]|0;
 $33 = ((($32)) + 28|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $34&65535;
 $36 = HEAP32[$pEq>>2]|0;
 $37 = ((($36)) + 36|0);
 HEAP16[$37>>1] = $35;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$7>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$pEq>>2]|0;
 $42 = (_sqlite3ExprAnd($38,$40,$41)|0);
 $43 = HEAP32[$7>>2]|0;
 HEAP32[$43>>2] = $42;
 STACKTOP = sp;return;
}
function _setJoinExpr($p,$iTable) {
 $p = $p|0;
 $iTable = $iTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 4|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = $6 | 1;
  HEAP32[$5>>2] = $7;
  $8 = HEAP32[$1>>2]|0;
  $9 = $8&65535;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 36|0);
  HEAP16[$11>>1] = $9;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($14|0)==(151);
  L4: do {
   if ($15) {
    $16 = HEAP32[$0>>2]|0;
    $17 = ((($16)) + 20|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($18|0)!=(0|0);
    if ($19) {
     HEAP32[$i>>2] = 0;
     while(1) {
      $20 = HEAP32[$i>>2]|0;
      $21 = HEAP32[$0>>2]|0;
      $22 = ((($21)) + 20|0);
      $23 = HEAP32[$22>>2]|0;
      $24 = HEAP32[$23>>2]|0;
      $25 = ($20|0)<($24|0);
      if (!($25)) {
       break L4;
      }
      $26 = HEAP32[$i>>2]|0;
      $27 = HEAP32[$0>>2]|0;
      $28 = ((($27)) + 20|0);
      $29 = HEAP32[$28>>2]|0;
      $30 = ((($29)) + 4|0);
      $31 = HEAP32[$30>>2]|0;
      $32 = (($31) + (($26*20)|0)|0);
      $33 = HEAP32[$32>>2]|0;
      $34 = HEAP32[$1>>2]|0;
      _setJoinExpr($33,$34);
      $35 = HEAP32[$i>>2]|0;
      $36 = (($35) + 1)|0;
      HEAP32[$i>>2] = $36;
     }
    }
   }
  } while(0);
  $37 = HEAP32[$0>>2]|0;
  $38 = ((($37)) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  _setJoinExpr($39,$40);
  $41 = HEAP32[$0>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  HEAP32[$0>>2] = $43;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprAnd($db,$pLeft,$pRight) {
 $db = $db|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 $6 = HEAP32[$3>>2]|0;
 if ($5) {
  HEAP32[$0>>2] = $6;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $7 = ($6|0)==(0|0);
 $8 = HEAP32[$2>>2]|0;
 if ($7) {
  HEAP32[$0>>2] = $8;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $9 = (_exprAlwaysFalse($8)|0);
 $10 = ($9|0)!=(0);
 if (!($10)) {
  $11 = HEAP32[$3>>2]|0;
  $12 = (_exprAlwaysFalse($11)|0);
  $13 = ($12|0)!=(0);
  if (!($13)) {
   $20 = HEAP32[$1>>2]|0;
   $21 = (_sqlite3ExprAlloc($20,28,0,0)|0);
   HEAP32[$pNew>>2] = $21;
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$pNew>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   _sqlite3ExprAttachSubtrees($22,$23,$24,$25);
   $26 = HEAP32[$pNew>>2]|0;
   HEAP32[$0>>2] = $26;
   $27 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($27|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 _sqlite3ExprDelete($14,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 _sqlite3ExprDelete($16,$17);
 $18 = HEAP32[$1>>2]|0;
 $19 = (_sqlite3ExprAlloc($18,134,4276,0)|0);
 HEAP32[$0>>2] = $19;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _exprAlwaysFalse($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$v>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 1;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 0;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3ExprIsInteger($7,$v)|0);
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$v>>2]|0;
    $11 = ($10|0)==(0);
    $12 = $11&1;
    HEAP32[$0>>2] = $12;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3ExprAttachSubtrees($db,$pRoot,$pLeft,$pRight) {
 $db = $db|0;
 $pRoot = $pRoot|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $pLeft;
 HEAP32[$3>>2] = $pRight;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  _sqlite3ExprDelete($6,$7);
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$3>>2]|0;
  _sqlite3ExprDelete($8,$9);
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 16|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$3>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = 2097408 & $17;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21 | $18;
  HEAP32[$20>>2] = $22;
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 12|0);
  HEAP32[$27>>2] = $25;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = 2097408 & $30;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 4|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $34 | $31;
  HEAP32[$33>>2] = $35;
 }
 $36 = HEAP32[$1>>2]|0;
 _exprSetHeight($36);
 STACKTOP = sp;return;
}
function _exprSetHeight($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nHeight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nHeight = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$nHeight>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 _heightOfExpr($3,$nHeight);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 _heightOfExpr($6,$nHeight);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 2048;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 20|0);
 if ($11) {
  $14 = HEAP32[$13>>2]|0;
  _heightOfSelect($14,$nHeight);
 } else {
  $15 = HEAP32[$13>>2]|0;
  $16 = ($15|0)!=(0|0);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   _heightOfExprList($19,$nHeight);
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = (_sqlite3ExprListFlags($22)|0);
   $24 = 2097408 & $23;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 4|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $27 | $24;
   HEAP32[$26>>2] = $28;
  }
 }
 $29 = HEAP32[$nHeight>>2]|0;
 $30 = (($29) + 1)|0;
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 24|0);
 HEAP32[$32>>2] = $30;
 STACKTOP = sp;return;
}
function _heightOfExpr($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($6|0)>($8|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 HEAP32[$13>>2] = $12;
 STACKTOP = sp;return;
}
function _heightOfSelect($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 32|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _heightOfExpr($6,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 _heightOfExpr($10,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 56|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 _heightOfExpr($14,$15);
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 60|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 _heightOfExpr($18,$19);
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 _heightOfExprList($21,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 36|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _heightOfExprList($25,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 44|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 _heightOfExprList($29,$30);
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 48|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 _heightOfSelect($33,$34);
 STACKTOP = sp;return;
}
function _heightOfExprList($p,$pnHeight) {
 $p = $p|0;
 $pnHeight = $pnHeight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnHeight;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  $7 = ($4|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*20)|0)|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  _heightOfExpr($13,$14);
  $15 = HEAP32[$i>>2]|0;
  $16 = (($15) + 1)|0;
  HEAP32[$i>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprListFlags($pList) {
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $m = 0, $pExpr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $m = sp + 4|0;
 $pExpr = sp;
 HEAP32[$0>>2] = $pList;
 HEAP32[$m>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $20 = HEAP32[$m>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*20)|0)|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pExpr>>2] = $12;
  $13 = HEAP32[$pExpr>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$m>>2]|0;
  $17 = $16 | $15;
  HEAP32[$m>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
 }
 $20 = HEAP32[$m>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3CreateColumnExpr($db,$pSrc,$iSrc,$iCol) {
 $db = $db|0;
 $pSrc = $pSrc|0;
 $iSrc = $iSrc|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $p = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $p = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $iSrc;
 HEAP32[$3>>2] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3ExprAlloc($4,152,0,0)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $60 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = (($10) + (($8*72)|0)|0);
 HEAP32[$pItem>>2] = $11;
 $12 = HEAP32[$pItem>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 44|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$pItem>>2]|0;
 $18 = ((($17)) + 44|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 28|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 44|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ((($24)) + 32|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26 << 16 >> 16;
 $28 = HEAP32[$3>>2]|0;
 $29 = ($27|0)==($28|0);
 if ($29) {
  $30 = HEAP32[$p>>2]|0;
  $31 = ((($30)) + 32|0);
  HEAP16[$31>>1] = -1;
 } else {
  $32 = HEAP32[$3>>2]|0;
  $33 = $32&65535;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 32|0);
  HEAP16[$35>>1] = $33;
  $36 = HEAP32[$3>>2]|0;
  $37 = ($36|0)>=(64);
  $38 = HEAP32[$3>>2]|0;
  $39 = $37 ? 63 : $38;
  $40 = (_bitshift64Shl(1,0,($39|0))|0);
  $41 = tempRet0;
  $42 = HEAP32[$pItem>>2]|0;
  $43 = ((($42)) + 56|0);
  $44 = $43;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = $46 | $40;
  $51 = $49 | $41;
  $52 = $43;
  $53 = $52;
  HEAP32[$53>>2] = $50;
  $54 = (($52) + 4)|0;
  $55 = $54;
  HEAP32[$55>>2] = $51;
 }
 $56 = HEAP32[$p>>2]|0;
 $57 = ((($56)) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = $58 | 4;
 HEAP32[$57>>2] = $59;
 $60 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _searchWith($pWith,$pItem,$ppContext) {
 $pWith = $pWith|0;
 $pItem = $pItem|0;
 $ppContext = $ppContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $p = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $zName = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pWith;
 HEAP32[$2>>2] = $pItem;
 HEAP32[$3>>2] = $ppContext;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)==(0|0);
 L1: do {
  if ($7) {
   $8 = HEAP32[$2>>2]|0;
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$zName>>2] = $10;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$1>>2]|0;
    HEAP32[$p>>2] = $12;
    L4: while(1) {
     $13 = HEAP32[$p>>2]|0;
     $14 = ($13|0)!=(0|0);
     if (!($14)) {
      break L1;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $15 = HEAP32[$i>>2]|0;
      $16 = HEAP32[$p>>2]|0;
      $17 = HEAP32[$16>>2]|0;
      $18 = ($15|0)<($17|0);
      if (!($18)) {
       break;
      }
      $19 = HEAP32[$zName>>2]|0;
      $20 = HEAP32[$i>>2]|0;
      $21 = HEAP32[$p>>2]|0;
      $22 = ((($21)) + 8|0);
      $23 = (($22) + ($20<<4)|0);
      $24 = HEAP32[$23>>2]|0;
      $25 = (_sqlite3StrICmp($19,$24)|0);
      $26 = ($25|0)==(0);
      if ($26) {
       break L4;
      }
      $33 = HEAP32[$i>>2]|0;
      $34 = (($33) + 1)|0;
      HEAP32[$i>>2] = $34;
     }
     $35 = HEAP32[$p>>2]|0;
     $36 = ((($35)) + 4|0);
     $37 = HEAP32[$36>>2]|0;
     HEAP32[$p>>2] = $37;
    }
    $27 = HEAP32[$p>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    HEAP32[$28>>2] = $27;
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$p>>2]|0;
    $31 = ((($30)) + 8|0);
    $32 = (($31) + ($29<<4)|0);
    HEAP32[$0>>2] = $32;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3SrcListDup($db,$p,$flags) {
 $db = $db|0;
 $p = $p|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $nByte = 0, $pNew = 0, $pNewItem = 0, $pOldItem = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $pNew = sp + 20|0;
 $i = sp + 16|0;
 $nByte = sp + 12|0;
 $pNewItem = sp + 8|0;
 $pOldItem = sp + 4|0;
 $pTab = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)>(0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) - 1)|0;
  $12 = ($11*72)|0;
  $14 = $12;
 } else {
  $14 = 0;
 }
 $13 = (80 + ($14))|0;
 HEAP32[$nByte>>2] = $13;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$nByte>>2]|0;
 $17 = ($16|0)<(0);
 $18 = $17 << 31 >> 31;
 $19 = (_sqlite3DbMallocRawNN($15,$16,$18)|0);
 HEAP32[$pNew>>2] = $19;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ($20|0)==(0|0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $169 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($169|0);
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$pNew>>2]|0;
 $25 = ((($24)) + 4|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$pNew>>2]|0;
 HEAP32[$26>>2] = $23;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = ($27|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = (($33) + (($31*72)|0)|0);
  HEAP32[$pNewItem>>2] = $34;
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = ((($36)) + 8|0);
  $38 = (($37) + (($35*72)|0)|0);
  HEAP32[$pOldItem>>2] = $38;
  $39 = HEAP32[$pOldItem>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pNewItem>>2]|0;
  HEAP32[$41>>2] = $40;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pOldItem>>2]|0;
  $44 = ((($43)) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3DbStrDup($42,$45)|0);
  $47 = HEAP32[$pNewItem>>2]|0;
  $48 = ((($47)) + 4|0);
  HEAP32[$48>>2] = $46;
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$pOldItem>>2]|0;
  $51 = ((($50)) + 8|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (_sqlite3DbStrDup($49,$52)|0);
  $54 = HEAP32[$pNewItem>>2]|0;
  $55 = ((($54)) + 8|0);
  HEAP32[$55>>2] = $53;
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$pOldItem>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (_sqlite3DbStrDup($56,$59)|0);
  $61 = HEAP32[$pNewItem>>2]|0;
  $62 = ((($61)) + 12|0);
  HEAP32[$62>>2] = $60;
  $63 = HEAP32[$pNewItem>>2]|0;
  $64 = ((($63)) + 36|0);
  $65 = HEAP32[$pOldItem>>2]|0;
  $66 = ((($65)) + 36|0);
  ;HEAP32[$64>>2]=HEAP32[$66>>2]|0;
  $67 = HEAP32[$pOldItem>>2]|0;
  $68 = ((($67)) + 44|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$pNewItem>>2]|0;
  $71 = ((($70)) + 44|0);
  HEAP32[$71>>2] = $69;
  $72 = HEAP32[$pOldItem>>2]|0;
  $73 = ((($72)) + 24|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = HEAP32[$pNewItem>>2]|0;
  $76 = ((($75)) + 24|0);
  HEAP32[$76>>2] = $74;
  $77 = HEAP32[$pOldItem>>2]|0;
  $78 = ((($77)) + 28|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = HEAP32[$pNewItem>>2]|0;
  $81 = ((($80)) + 28|0);
  HEAP32[$81>>2] = $79;
  $82 = HEAP32[$pNewItem>>2]|0;
  $83 = ((($82)) + 36|0);
  $84 = ((($83)) + 1|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = ($85&255) >>> 1;
  $87 = $86 & 1;
  $88 = $87&255;
  $89 = ($88|0)!=(0);
  if ($89) {
   $90 = HEAP32[$1>>2]|0;
   $91 = HEAP32[$pOldItem>>2]|0;
   $92 = ((($91)) + 64|0);
   $93 = HEAP32[$92>>2]|0;
   $94 = (_sqlite3DbStrDup($90,$93)|0);
   $95 = HEAP32[$pNewItem>>2]|0;
   $96 = ((($95)) + 64|0);
   HEAP32[$96>>2] = $94;
  }
  $97 = HEAP32[$pOldItem>>2]|0;
  $98 = ((($97)) + 68|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = HEAP32[$pNewItem>>2]|0;
  $101 = ((($100)) + 68|0);
  HEAP32[$101>>2] = $99;
  $102 = HEAP32[$pNewItem>>2]|0;
  $103 = ((($102)) + 36|0);
  $104 = ((($103)) + 1|0);
  $105 = HEAP8[$104>>0]|0;
  $106 = ($105&255) >>> 2;
  $107 = $106 & 1;
  $108 = $107&255;
  $109 = ($108|0)!=(0);
  if ($109) {
   $110 = HEAP32[$1>>2]|0;
   $111 = HEAP32[$pOldItem>>2]|0;
   $112 = ((($111)) + 64|0);
   $113 = HEAP32[$112>>2]|0;
   $114 = HEAP32[$3>>2]|0;
   $115 = (_sqlite3ExprListDup($110,$113,$114)|0);
   $116 = HEAP32[$pNewItem>>2]|0;
   $117 = ((($116)) + 64|0);
   HEAP32[$117>>2] = $115;
  }
  $118 = HEAP32[$pOldItem>>2]|0;
  $119 = ((($118)) + 16|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = HEAP32[$pNewItem>>2]|0;
  $122 = ((($121)) + 16|0);
  HEAP32[$122>>2] = $120;
  HEAP32[$pTab>>2] = $120;
  $123 = HEAP32[$pTab>>2]|0;
  $124 = ($123|0)!=(0|0);
  if ($124) {
   $125 = HEAP32[$pTab>>2]|0;
   $126 = ((($125)) + 36|0);
   $127 = HEAP16[$126>>1]|0;
   $128 = (($127) + 1)<<16>>16;
   HEAP16[$126>>1] = $128;
  }
  $129 = HEAP32[$1>>2]|0;
  $130 = HEAP32[$pOldItem>>2]|0;
  $131 = ((($130)) + 20|0);
  $132 = HEAP32[$131>>2]|0;
  $133 = HEAP32[$3>>2]|0;
  $134 = (_sqlite3SelectDup($129,$132,$133)|0);
  $135 = HEAP32[$pNewItem>>2]|0;
  $136 = ((($135)) + 20|0);
  HEAP32[$136>>2] = $134;
  $137 = HEAP32[$1>>2]|0;
  $138 = HEAP32[$pOldItem>>2]|0;
  $139 = ((($138)) + 48|0);
  $140 = HEAP32[$139>>2]|0;
  $141 = HEAP32[$3>>2]|0;
  $142 = (_sqlite3ExprDup($137,$140,$141)|0);
  $143 = HEAP32[$pNewItem>>2]|0;
  $144 = ((($143)) + 48|0);
  HEAP32[$144>>2] = $142;
  $145 = HEAP32[$1>>2]|0;
  $146 = HEAP32[$pOldItem>>2]|0;
  $147 = ((($146)) + 52|0);
  $148 = HEAP32[$147>>2]|0;
  $149 = (_sqlite3IdListDup($145,$148)|0);
  $150 = HEAP32[$pNewItem>>2]|0;
  $151 = ((($150)) + 52|0);
  HEAP32[$151>>2] = $149;
  $152 = HEAP32[$pOldItem>>2]|0;
  $153 = ((($152)) + 56|0);
  $154 = $153;
  $155 = $154;
  $156 = HEAP32[$155>>2]|0;
  $157 = (($154) + 4)|0;
  $158 = $157;
  $159 = HEAP32[$158>>2]|0;
  $160 = HEAP32[$pNewItem>>2]|0;
  $161 = ((($160)) + 56|0);
  $162 = $161;
  $163 = $162;
  HEAP32[$163>>2] = $156;
  $164 = (($162) + 4)|0;
  $165 = $164;
  HEAP32[$165>>2] = $159;
  $166 = HEAP32[$i>>2]|0;
  $167 = (($166) + 1)|0;
  HEAP32[$i>>2] = $167;
 }
 $168 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $168;
 $169 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($169|0);
}
function _withDup($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $nByte = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRet = sp + 8|0;
 $nByte = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$pRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $62 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) - 1)|0;
 $7 = $6<<4;
 $8 = (24 + ($7))|0;
 HEAP32[$nByte>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$nByte>>2]|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (_sqlite3DbMallocZero($9,$10,$12)|0);
 HEAP32[$pRet>>2] = $13;
 $14 = HEAP32[$pRet>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $62 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$pRet>>2]|0;
 HEAP32[$18>>2] = $17;
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ($19|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = (($26) + ($24<<4)|0);
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (_sqlite3SelectDup($23,$29,0)|0);
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pRet>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = (($33) + ($31<<4)|0);
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $30;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$i>>2]|0;
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 8|0);
  $40 = (($39) + ($37<<4)|0);
  $41 = ((($40)) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (_sqlite3ExprListDup($36,$42,0)|0);
  $44 = HEAP32[$i>>2]|0;
  $45 = HEAP32[$pRet>>2]|0;
  $46 = ((($45)) + 8|0);
  $47 = (($46) + ($44<<4)|0);
  $48 = ((($47)) + 4|0);
  HEAP32[$48>>2] = $43;
  $49 = HEAP32[$0>>2]|0;
  $50 = HEAP32[$i>>2]|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = ((($51)) + 8|0);
  $53 = (($52) + ($50<<4)|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3DbStrDup($49,$54)|0);
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$pRet>>2]|0;
  $58 = ((($57)) + 8|0);
  $59 = (($58) + ($56<<4)|0);
  HEAP32[$59>>2] = $55;
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 $62 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($62|0);
}
function _sqlite3IdListDup($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pNew = 0, $pNewItem = 0, $pOldItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pNew = sp + 12|0;
 $i = sp + 8|0;
 $pNewItem = sp + 4|0;
 $pOldItem = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3DbMallocRawNN($5,8,0)|0);
 HEAP32[$pNew>>2] = $6;
 $7 = HEAP32[$pNew>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$pNew>>2]|0;
 $13 = ((($12)) + 4|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = $17<<3;
 $19 = (_sqlite3DbMallocRawNN($14,$18,0)|0);
 $20 = HEAP32[$pNew>>2]|0;
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$pNew>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$pNew>>2]|0;
  _sqlite3DbFree($24,$25);
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ($26|0)<($29|0);
  if (!($30)) {
   break;
  }
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$pNew>>2]|0;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($31<<3)|0);
  HEAP32[$pNewItem>>2] = $34;
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($35<<3)|0);
  HEAP32[$pOldItem>>2] = $38;
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$pOldItem>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = (_sqlite3DbStrDup($39,$41)|0);
  $43 = HEAP32[$pNewItem>>2]|0;
  HEAP32[$43>>2] = $42;
  $44 = HEAP32[$pOldItem>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$pNewItem>>2]|0;
  $48 = ((($47)) + 4|0);
  HEAP32[$48>>2] = $46;
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$pNew>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _vtabCallConstructor($db,$pTab,$pMod,$xConstruct,$pzErr) {
 $db = $db|0;
 $pTab = $pTab|0;
 $pMod = $pMod|0;
 $xConstruct = $xConstruct|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $azArg = 0, $i = 0, $iCol = 0, $iDb = 0, $j = 0, $nArg = 0, $nDel = 0, $nType = 0, $oooHidden = 0, $or$cond = 0, $pCtx = 0, $pVTable = 0, $rc = 0, $sCtx = 0;
 var $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer2 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $zErr = 0, $zFormat = 0, $zModuleName = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 132|0;
 $1 = sp + 128|0;
 $2 = sp + 124|0;
 $3 = sp + 120|0;
 $4 = sp + 116|0;
 $5 = sp + 112|0;
 $sCtx = sp + 96|0;
 $pVTable = sp + 92|0;
 $rc = sp + 88|0;
 $azArg = sp + 84|0;
 $nArg = sp + 80|0;
 $zErr = sp + 76|0;
 $zModuleName = sp + 72|0;
 $iDb = sp + 68|0;
 $pCtx = sp + 64|0;
 $zFormat = sp + 60|0;
 $iCol = sp + 56|0;
 $oooHidden = sp + 136|0;
 $zType = sp + 52|0;
 $nType = sp + 48|0;
 $i = sp + 44|0;
 $j = sp + 40|0;
 $nDel = sp + 36|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pMod;
 HEAP32[$4>>2] = $xConstruct;
 HEAP32[$5>>2] = $pzErr;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$azArg>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 48|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$nArg>>2] = $11;
 HEAP32[$zErr>>2] = 0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 336|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pCtx>>2] = $14;
 while(1) {
  $15 = HEAP32[$pCtx>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pCtx>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($19|0)==($20|0);
  if ($21) {
   label = 4;
   break;
  }
  $27 = HEAP32[$pCtx>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  HEAP32[$pCtx>>2] = $29;
 }
 if ((label|0) == 4) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$vararg_buffer>>2] = $24;
  $25 = (_sqlite3MPrintf($22,30650,$vararg_buffer)|0);
  $26 = HEAP32[$5>>2]|0;
  HEAP32[$26>>2] = $25;
  HEAP32[$0>>2] = 6;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 $30 = HEAP32[$1>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $32;
 $33 = (_sqlite3MPrintf($30,18391,$vararg_buffer2)|0);
 HEAP32[$zModuleName>>2] = $33;
 $34 = HEAP32[$zModuleName>>2]|0;
 $35 = ($34|0)!=(0|0);
 if (!($35)) {
  HEAP32[$0>>2] = 7;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = (_sqlite3DbMallocZero($36,28,0)|0);
 HEAP32[$pVTable>>2] = $37;
 $38 = HEAP32[$pVTable>>2]|0;
 $39 = ($38|0)!=(0|0);
 $40 = HEAP32[$1>>2]|0;
 if (!($39)) {
  $41 = HEAP32[$zModuleName>>2]|0;
  _sqlite3DbFree($40,$41);
  HEAP32[$0>>2] = 7;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 $42 = HEAP32[$pVTable>>2]|0;
 HEAP32[$42>>2] = $40;
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$pVTable>>2]|0;
 $45 = ((($44)) + 4|0);
 HEAP32[$45>>2] = $43;
 $46 = HEAP32[$1>>2]|0;
 $47 = HEAP32[$2>>2]|0;
 $48 = ((($47)) + 64|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (_sqlite3SchemaToIndex($46,$49)|0);
 HEAP32[$iDb>>2] = $50;
 $51 = HEAP32[$iDb>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 $53 = ((($52)) + 16|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + ($51<<4)|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$2>>2]|0;
 $58 = ((($57)) + 52|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = ((($59)) + 4|0);
 HEAP32[$60>>2] = $56;
 $61 = HEAP32[$2>>2]|0;
 $62 = ((($sCtx)) + 4|0);
 HEAP32[$62>>2] = $61;
 $63 = HEAP32[$pVTable>>2]|0;
 HEAP32[$sCtx>>2] = $63;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 336|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ((($sCtx)) + 8|0);
 HEAP32[$67>>2] = $66;
 $68 = ((($sCtx)) + 12|0);
 HEAP32[$68>>2] = 0;
 $69 = HEAP32[$1>>2]|0;
 $70 = ((($69)) + 336|0);
 HEAP32[$70>>2] = $sCtx;
 $71 = HEAP32[$4>>2]|0;
 $72 = HEAP32[$1>>2]|0;
 $73 = HEAP32[$3>>2]|0;
 $74 = ((($73)) + 8|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = HEAP32[$nArg>>2]|0;
 $77 = HEAP32[$azArg>>2]|0;
 $78 = HEAP32[$pVTable>>2]|0;
 $79 = ((($78)) + 8|0);
 $80 = (FUNCTION_TABLE_iiiiiii[$71 & 255]($72,$75,$76,$77,$79,$zErr)|0);
 HEAP32[$rc>>2] = $80;
 $81 = ((($sCtx)) + 8|0);
 $82 = HEAP32[$81>>2]|0;
 $83 = HEAP32[$1>>2]|0;
 $84 = ((($83)) + 336|0);
 HEAP32[$84>>2] = $82;
 $85 = HEAP32[$rc>>2]|0;
 $86 = ($85|0)==(7);
 if ($86) {
  $87 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($87);
 }
 $88 = HEAP32[$rc>>2]|0;
 $89 = (0)!=($88|0);
 L20: do {
  if ($89) {
   $90 = HEAP32[$zErr>>2]|0;
   $91 = ($90|0)==(0|0);
   $92 = HEAP32[$1>>2]|0;
   if ($91) {
    $93 = HEAP32[$zModuleName>>2]|0;
    HEAP32[$vararg_buffer5>>2] = $93;
    $94 = (_sqlite3MPrintf($92,30692,$vararg_buffer5)|0);
    $95 = HEAP32[$5>>2]|0;
    HEAP32[$95>>2] = $94;
   } else {
    $96 = HEAP32[$zErr>>2]|0;
    HEAP32[$vararg_buffer8>>2] = $96;
    $97 = (_sqlite3MPrintf($92,18391,$vararg_buffer8)|0);
    $98 = HEAP32[$5>>2]|0;
    HEAP32[$98>>2] = $97;
    $99 = HEAP32[$zErr>>2]|0;
    _sqlite3_free($99);
   }
   $100 = HEAP32[$1>>2]|0;
   $101 = HEAP32[$pVTable>>2]|0;
   _sqlite3DbFree($100,$101);
  } else {
   $102 = HEAP32[$pVTable>>2]|0;
   $103 = ((($102)) + 8|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = ($104|0)!=(0|0);
   if ($105) {
    $106 = HEAP32[$pVTable>>2]|0;
    $107 = ((($106)) + 8|0);
    $108 = HEAP32[$107>>2]|0;
    ;HEAP32[$108>>2]=0|0;HEAP32[$108+4>>2]=0|0;HEAP32[$108+8>>2]=0|0;
    $109 = HEAP32[$3>>2]|0;
    $110 = HEAP32[$109>>2]|0;
    $111 = HEAP32[$pVTable>>2]|0;
    $112 = ((($111)) + 8|0);
    $113 = HEAP32[$112>>2]|0;
    HEAP32[$113>>2] = $110;
    $114 = HEAP32[$pVTable>>2]|0;
    $115 = ((($114)) + 12|0);
    HEAP32[$115>>2] = 1;
    $116 = ((($sCtx)) + 12|0);
    $117 = HEAP32[$116>>2]|0;
    $118 = ($117|0)==(0);
    if ($118) {
     HEAP32[$zFormat>>2] = 30722;
     $119 = HEAP32[$1>>2]|0;
     $120 = HEAP32[$zFormat>>2]|0;
     $121 = HEAP32[$2>>2]|0;
     $122 = HEAP32[$121>>2]|0;
     HEAP32[$vararg_buffer11>>2] = $122;
     $123 = (_sqlite3MPrintf($119,$120,$vararg_buffer11)|0);
     $124 = HEAP32[$5>>2]|0;
     HEAP32[$124>>2] = $123;
     $125 = HEAP32[$pVTable>>2]|0;
     _sqlite3VtabUnlock($125);
     HEAP32[$rc>>2] = 1;
     break;
    }
    HEAP8[$oooHidden>>0] = 0;
    $126 = HEAP32[$2>>2]|0;
    $127 = ((($126)) + 56|0);
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$pVTable>>2]|0;
    $130 = ((($129)) + 24|0);
    HEAP32[$130>>2] = $128;
    $131 = HEAP32[$pVTable>>2]|0;
    $132 = HEAP32[$2>>2]|0;
    $133 = ((($132)) + 56|0);
    HEAP32[$133>>2] = $131;
    HEAP32[$iCol>>2] = 0;
    while(1) {
     $134 = HEAP32[$iCol>>2]|0;
     $135 = HEAP32[$2>>2]|0;
     $136 = ((($135)) + 34|0);
     $137 = HEAP16[$136>>1]|0;
     $138 = $137 << 16 >> 16;
     $139 = ($134|0)<($138|0);
     if (!($139)) {
      break L20;
     }
     $140 = HEAP32[$iCol>>2]|0;
     $141 = HEAP32[$2>>2]|0;
     $142 = ((($141)) + 4|0);
     $143 = HEAP32[$142>>2]|0;
     $144 = (($143) + ($140<<4)|0);
     $145 = (_sqlite3ColumnType($144,18402)|0);
     HEAP32[$zType>>2] = $145;
     HEAP32[$i>>2] = 0;
     $146 = HEAP32[$zType>>2]|0;
     $147 = (_sqlite3Strlen30($146)|0);
     HEAP32[$nType>>2] = $147;
     HEAP32[$i>>2] = 0;
     L34: while(1) {
      $148 = HEAP32[$i>>2]|0;
      $149 = HEAP32[$nType>>2]|0;
      $150 = ($148|0)<($149|0);
      if (!($150)) {
       break;
      }
      $151 = HEAP32[$i>>2]|0;
      $152 = HEAP32[$zType>>2]|0;
      $153 = (($152) + ($151)|0);
      $154 = (_sqlite3_strnicmp(30768,$153,6)|0);
      $155 = (0)==($154|0);
      do {
       if ($155) {
        $156 = HEAP32[$i>>2]|0;
        $157 = ($156|0)==(0);
        if (!($157)) {
         $158 = HEAP32[$i>>2]|0;
         $159 = (($158) - 1)|0;
         $160 = HEAP32[$zType>>2]|0;
         $161 = (($160) + ($159)|0);
         $162 = HEAP8[$161>>0]|0;
         $163 = $162 << 24 >> 24;
         $164 = ($163|0)==(32);
         if (!($164)) {
          break;
         }
        }
        $165 = HEAP32[$i>>2]|0;
        $166 = (($165) + 6)|0;
        $167 = HEAP32[$zType>>2]|0;
        $168 = (($167) + ($166)|0);
        $169 = HEAP8[$168>>0]|0;
        $170 = $169 << 24 >> 24;
        $171 = ($170|0)==(0);
        if ($171) {
         break L34;
        }
        $172 = HEAP32[$i>>2]|0;
        $173 = (($172) + 6)|0;
        $174 = HEAP32[$zType>>2]|0;
        $175 = (($174) + ($173)|0);
        $176 = HEAP8[$175>>0]|0;
        $177 = $176 << 24 >> 24;
        $178 = ($177|0)==(32);
        if ($178) {
         break L34;
        }
       }
      } while(0);
      $179 = HEAP32[$i>>2]|0;
      $180 = (($179) + 1)|0;
      HEAP32[$i>>2] = $180;
     }
     $181 = HEAP32[$i>>2]|0;
     $182 = HEAP32[$nType>>2]|0;
     $183 = ($181|0)<($182|0);
     if ($183) {
      $184 = HEAP32[$i>>2]|0;
      $185 = (($184) + 6)|0;
      $186 = HEAP32[$zType>>2]|0;
      $187 = (($186) + ($185)|0);
      $188 = HEAP8[$187>>0]|0;
      $189 = $188 << 24 >> 24;
      $190 = ($189|0)!=(0);
      $191 = $190 ? 1 : 0;
      $192 = (6 + ($191))|0;
      HEAP32[$nDel>>2] = $192;
      $193 = HEAP32[$i>>2]|0;
      HEAP32[$j>>2] = $193;
      while(1) {
       $194 = HEAP32[$j>>2]|0;
       $195 = HEAP32[$nDel>>2]|0;
       $196 = (($194) + ($195))|0;
       $197 = HEAP32[$nType>>2]|0;
       $198 = ($196|0)<=($197|0);
       if (!($198)) {
        break;
       }
       $199 = HEAP32[$j>>2]|0;
       $200 = HEAP32[$nDel>>2]|0;
       $201 = (($199) + ($200))|0;
       $202 = HEAP32[$zType>>2]|0;
       $203 = (($202) + ($201)|0);
       $204 = HEAP8[$203>>0]|0;
       $205 = HEAP32[$j>>2]|0;
       $206 = HEAP32[$zType>>2]|0;
       $207 = (($206) + ($205)|0);
       HEAP8[$207>>0] = $204;
       $208 = HEAP32[$j>>2]|0;
       $209 = (($208) + 1)|0;
       HEAP32[$j>>2] = $209;
      }
      $210 = HEAP32[$i>>2]|0;
      $211 = HEAP32[$zType>>2]|0;
      $212 = (($211) + ($210)|0);
      $213 = HEAP8[$212>>0]|0;
      $214 = $213 << 24 >> 24;
      $215 = ($214|0)==(0);
      $216 = HEAP32[$i>>2]|0;
      $217 = ($216|0)>(0);
      $or$cond = $215 & $217;
      if ($or$cond) {
       $218 = HEAP32[$i>>2]|0;
       $219 = (($218) - 1)|0;
       $220 = HEAP32[$zType>>2]|0;
       $221 = (($220) + ($219)|0);
       HEAP8[$221>>0] = 0;
      }
      $222 = HEAP32[$iCol>>2]|0;
      $223 = HEAP32[$2>>2]|0;
      $224 = ((($223)) + 4|0);
      $225 = HEAP32[$224>>2]|0;
      $226 = (($225) + ($222<<4)|0);
      $227 = ((($226)) + 15|0);
      $228 = HEAP8[$227>>0]|0;
      $229 = $228&255;
      $230 = $229 | 2;
      $231 = $230&255;
      HEAP8[$227>>0] = $231;
      HEAP8[$oooHidden>>0] = -128;
     } else {
      $232 = HEAP8[$oooHidden>>0]|0;
      $233 = $232&255;
      $234 = HEAP32[$2>>2]|0;
      $235 = ((($234)) + 42|0);
      $236 = HEAP8[$235>>0]|0;
      $237 = $236&255;
      $238 = $237 | $233;
      $239 = $238&255;
      HEAP8[$235>>0] = $239;
     }
     $240 = HEAP32[$iCol>>2]|0;
     $241 = (($240) + 1)|0;
     HEAP32[$iCol>>2] = $241;
    }
   }
  }
 } while(0);
 $242 = HEAP32[$1>>2]|0;
 $243 = HEAP32[$zModuleName>>2]|0;
 _sqlite3DbFree($242,$243);
 $244 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $244;
 $245 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($245|0);
}
function _sqlite3ReadSchema($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $db = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$db>>2]|0;
 $4 = ((($3)) + 148|0);
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  $8 = HEAP32[$db>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 4|0);
  $11 = (_sqlite3Init($8,$10)|0);
  HEAP32[$rc>>2] = $11;
 }
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $21 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $14 = HEAP32[$rc>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 36|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 1)|0;
 HEAP32[$18>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _analyzeDatabase($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iMem = 0, $iStatCur = 0, $iTab = 0, $k = 0, $pSchema = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $db = sp + 24|0;
 $pSchema = sp + 20|0;
 $k = sp + 16|0;
 $iStatCur = sp + 12|0;
 $iMem = sp + 8|0;
 $iTab = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pSchema>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 _sqlite3BeginWriteOperation($11,0,$12);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 40|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$iStatCur>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 40|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + 3)|0;
 HEAP32[$17>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$iStatCur>>2]|0;
 _openStatTable($20,$21,$22,0,0);
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 44|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$iMem>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 40|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$iTab>>2] = $29;
 $30 = HEAP32[$pSchema>>2]|0;
 $31 = ((($30)) + 8|0);
 $32 = ((($31)) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$k>>2] = $33;
 while(1) {
  $34 = HEAP32[$k>>2]|0;
  $35 = ($34|0)!=(0|0);
  if (!($35)) {
   break;
  }
  $36 = HEAP32[$k>>2]|0;
  $37 = ((($36)) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$pTab>>2] = $38;
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$pTab>>2]|0;
  $41 = HEAP32[$iStatCur>>2]|0;
  $42 = HEAP32[$iMem>>2]|0;
  $43 = HEAP32[$iTab>>2]|0;
  _analyzeOneTable($39,$40,0,$41,$42,$43);
  $44 = HEAP32[$k>>2]|0;
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$k>>2] = $45;
 }
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 _loadAnalysis($46,$47);
 STACKTOP = sp;return;
}
function _sqlite3FindDb($db,$pName) {
 $db = $db|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3NameFromToken($2,$3)|0);
 HEAP32[$zName>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$zName>>2]|0;
 $7 = (_sqlite3FindDbName($5,$6)|0);
 HEAP32[$i>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$zName>>2]|0;
 _sqlite3DbFree($8,$9);
 $10 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _analyzeTable($pParse,$pTab,$pOnlyIdx) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pOnlyIdx = $pOnlyIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iDb = 0, $iStatCur = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $iDb = sp + 4|0;
 $iStatCur = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pOnlyIdx;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3SchemaToIndex($4,$7)|0);
 HEAP32[$iDb>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$iDb>>2]|0;
 _sqlite3BeginWriteOperation($9,0,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$iStatCur>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 40|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (($16) + 3)|0;
 HEAP32[$15>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = ($18|0)!=(0|0);
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$iDb>>2]|0;
 $22 = HEAP32[$iStatCur>>2]|0;
 if ($19) {
  $23 = HEAP32[$2>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  _openStatTable($20,$21,$22,$24,30854);
 } else {
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  _openStatTable($20,$21,$22,$26,30858);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$2>>2]|0;
 $30 = HEAP32[$iStatCur>>2]|0;
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 44|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + 1)|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 40|0);
 $37 = HEAP32[$36>>2]|0;
 _analyzeOneTable($27,$28,$29,$30,$34,$37);
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$iDb>>2]|0;
 _loadAnalysis($38,$39);
 STACKTOP = sp;return;
}
function _sqlite3TwoPartName($pParse,$pName1,$pName2,$pUnqual) {
 $pParse = $pParse|0;
 $pName1 = $pName1|0;
 $pName2 = $pName2|0;
 $pUnqual = $pUnqual|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $iDb = sp + 16|0;
 $db = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pName1;
 HEAP32[$3>>2] = $pName2;
 HEAP32[$4>>2] = $pUnqual;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9>>>0)>(0);
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 148|0);
 if ($10) {
  $13 = ((($12)) + 5|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = ($14<<24>>24)!=(0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($16,30817,$vararg_buffer);
   HEAP32[$0>>2] = -1;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
  $17 = HEAP32[$3>>2]|0;
  $18 = HEAP32[$4>>2]|0;
  HEAP32[$18>>2] = $17;
  $19 = HEAP32[$db>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = (_sqlite3FindDb($19,$20)|0);
  HEAP32[$iDb>>2] = $21;
  $22 = HEAP32[$iDb>>2]|0;
  $23 = ($22|0)<(0);
  if ($23) {
   $24 = HEAP32[$1>>2]|0;
   $25 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $25;
   _sqlite3ErrorMsg($24,30834,$vararg_buffer1);
   HEAP32[$0>>2] = -1;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } else {
  $26 = ((($12)) + 4|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27&255;
  HEAP32[$iDb>>2] = $28;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$4>>2]|0;
  HEAP32[$30>>2] = $29;
 }
 $31 = HEAP32[$iDb>>2]|0;
 HEAP32[$0>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _openStatTable($pParse,$iDb,$iStatCur,$zWhere,$zWhereType) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $iStatCur = $iStatCur|0;
 $zWhere = $zWhere|0;
 $zWhereType = $zWhereType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aCreateTbl = 0, $aRoot = 0, $db = 0, $i = 0, $pDb = 0, $pStat = 0;
 var $v = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $i = sp + 64|0;
 $db = sp + 60|0;
 $pDb = sp + 56|0;
 $v = sp + 52|0;
 $aRoot = sp + 40|0;
 $aCreateTbl = sp + 88|0;
 $zTab = sp + 36|0;
 $pStat = sp + 32|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iStatCur;
 HEAP32[$3>>2] = $zWhere;
 HEAP32[$4>>2] = $zWhereType;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetVdbe($7)|0);
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11<<4)|0);
 HEAP32[$pDb>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = ($16|0)<(3);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = (4396 + ($18<<3)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$zTab>>2] = $20;
  $21 = HEAP32[$db>>2]|0;
  $22 = HEAP32[$zTab>>2]|0;
  $23 = HEAP32[$pDb>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = (_sqlite3FindTable($21,$22,$24)|0);
  HEAP32[$pStat>>2] = $25;
  $26 = ($25|0)==(0|0);
  do {
   if ($26) {
    $27 = HEAP32[$i>>2]|0;
    $28 = (4396 + ($27<<3)|0);
    $29 = ((($28)) + 4|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = ($30|0)!=(0|0);
    if ($31) {
     $32 = HEAP32[$0>>2]|0;
     $33 = HEAP32[$pDb>>2]|0;
     $34 = HEAP32[$33>>2]|0;
     $35 = HEAP32[$zTab>>2]|0;
     $36 = HEAP32[$i>>2]|0;
     $37 = (4396 + ($36<<3)|0);
     $38 = ((($37)) + 4|0);
     $39 = HEAP32[$38>>2]|0;
     HEAP32[$vararg_buffer>>2] = $34;
     $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr1>>2] = $35;
     $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
     HEAP32[$vararg_ptr2>>2] = $39;
     _sqlite3NestedParse($32,30915,$vararg_buffer);
     $40 = HEAP32[$0>>2]|0;
     $41 = ((($40)) + 104|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = HEAP32[$i>>2]|0;
     $44 = (($aRoot) + ($43<<2)|0);
     HEAP32[$44>>2] = $42;
     $45 = HEAP32[$i>>2]|0;
     $46 = (($aCreateTbl) + ($45)|0);
     HEAP8[$46>>0] = 16;
    }
   } else {
    $47 = HEAP32[$pStat>>2]|0;
    $48 = ((($47)) + 28|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = HEAP32[$i>>2]|0;
    $51 = (($aRoot) + ($50<<2)|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$i>>2]|0;
    $53 = (($aCreateTbl) + ($52)|0);
    HEAP8[$53>>0] = 0;
    $54 = HEAP32[$0>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = HEAP32[$i>>2]|0;
    $57 = (($aRoot) + ($56<<2)|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$zTab>>2]|0;
    _sqlite3TableLock($54,$55,$58,1,$59);
    $60 = HEAP32[$3>>2]|0;
    $61 = ($60|0)!=(0|0);
    if ($61) {
     $62 = HEAP32[$0>>2]|0;
     $63 = HEAP32[$pDb>>2]|0;
     $64 = HEAP32[$63>>2]|0;
     $65 = HEAP32[$zTab>>2]|0;
     $66 = HEAP32[$4>>2]|0;
     $67 = HEAP32[$3>>2]|0;
     HEAP32[$vararg_buffer3>>2] = $64;
     $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
     HEAP32[$vararg_ptr6>>2] = $65;
     $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
     HEAP32[$vararg_ptr7>>2] = $66;
     $vararg_ptr8 = ((($vararg_buffer3)) + 12|0);
     HEAP32[$vararg_ptr8>>2] = $67;
     _sqlite3NestedParse($62,30938,$vararg_buffer3);
     break;
    } else {
     $68 = HEAP32[$v>>2]|0;
     $69 = HEAP32[$i>>2]|0;
     $70 = (($aRoot) + ($69<<2)|0);
     $71 = HEAP32[$70>>2]|0;
     $72 = HEAP32[$1>>2]|0;
     (_sqlite3VdbeAddOp2($68,131,$71,$72)|0);
     break;
    }
   }
  } while(0);
  $73 = HEAP32[$i>>2]|0;
  $74 = (($73) + 1)|0;
  HEAP32[$i>>2] = $74;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $75 = HEAP32[$i>>2]|0;
  $76 = (4396 + ($75<<3)|0);
  $77 = ((($76)) + 4|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)!=(0|0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$v>>2]|0;
  $81 = HEAP32[$2>>2]|0;
  $82 = HEAP32[$i>>2]|0;
  $83 = (($81) + ($82))|0;
  $84 = HEAP32[$i>>2]|0;
  $85 = (($aRoot) + ($84<<2)|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeAddOp4Int($80,105,$83,$86,$87,3)|0);
  $88 = HEAP32[$v>>2]|0;
  $89 = HEAP32[$i>>2]|0;
  $90 = (($aCreateTbl) + ($89)|0);
  $91 = HEAP8[$90>>0]|0;
  _sqlite3VdbeChangeP5($88,$91);
  $92 = HEAP32[$i>>2]|0;
  $93 = (($92) + 1)|0;
  HEAP32[$i>>2] = $93;
 }
 STACKTOP = sp;return;
}
function _analyzeOneTable($pParse,$pTab,$pOnlyIdx,$iStatCur,$iMem,$iTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pOnlyIdx = $pOnlyIdx|0;
 $iStatCur = $iStatCur|0;
 $iMem = $iMem|0;
 $iTab = $iTab|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aGotoChng = 0, $addrNextRow = 0, $addrRewind = 0;
 var $db = 0, $endDistinctTest = 0, $i = 0, $iDb = 0, $iIdxCur = 0, $iTabCur = 0, $jZeroRows = 0, $nCol = 0, $nColTest = 0, $needTableCnt = 0, $or$cond = 0, $pColl = 0, $pIdx = 0, $regChng = 0, $regIdxname = 0, $regNewRowid = 0, $regPrev = 0, $regStat1 = 0, $regStat4 = 0, $regTabname = 0;
 var $regTemp = 0, $v = 0, $zIdxName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $5 = sp + 96|0;
 $db = sp + 92|0;
 $pIdx = sp + 88|0;
 $iIdxCur = sp + 84|0;
 $iTabCur = sp + 80|0;
 $v = sp + 76|0;
 $i = sp + 72|0;
 $jZeroRows = sp + 68|0;
 $iDb = sp + 64|0;
 $needTableCnt = sp + 120|0;
 $regNewRowid = sp + 60|0;
 $regStat4 = sp + 56|0;
 $regChng = sp + 52|0;
 $regTemp = sp + 48|0;
 $regTabname = sp + 44|0;
 $regIdxname = sp + 40|0;
 $regStat1 = sp + 36|0;
 $regPrev = sp + 32|0;
 $nCol = sp + 28|0;
 $addrRewind = sp + 24|0;
 $addrNextRow = sp + 20|0;
 $zIdxName = sp + 16|0;
 $nColTest = sp + 12|0;
 $endDistinctTest = sp + 8|0;
 $aGotoChng = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pOnlyIdx;
 HEAP32[$3>>2] = $iStatCur;
 HEAP32[$4>>2] = $iMem;
 HEAP32[$5>>2] = $iTab;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$jZeroRows>>2] = -1;
 HEAP8[$needTableCnt>>0] = 1;
 $8 = HEAP32[$4>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$4>>2] = $9;
 HEAP32[$regNewRowid>>2] = $8;
 $10 = HEAP32[$4>>2]|0;
 $11 = (($10) + 1)|0;
 HEAP32[$4>>2] = $11;
 HEAP32[$regStat4>>2] = $10;
 $12 = HEAP32[$4>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$4>>2] = $13;
 HEAP32[$regChng>>2] = $12;
 $14 = HEAP32[$4>>2]|0;
 $15 = (($14) + 1)|0;
 HEAP32[$4>>2] = $15;
 HEAP32[$regTemp>>2] = $14;
 $16 = HEAP32[$4>>2]|0;
 $17 = (($16) + 1)|0;
 HEAP32[$4>>2] = $17;
 HEAP32[$regTabname>>2] = $16;
 $18 = HEAP32[$4>>2]|0;
 $19 = (($18) + 1)|0;
 HEAP32[$4>>2] = $19;
 HEAP32[$regIdxname>>2] = $18;
 $20 = HEAP32[$4>>2]|0;
 $21 = (($20) + 1)|0;
 HEAP32[$4>>2] = $21;
 HEAP32[$regStat1>>2] = $20;
 $22 = HEAP32[$4>>2]|0;
 HEAP32[$regPrev>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 44|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$4>>2]|0;
 $27 = ($25|0)>($26|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 44|0);
  $30 = HEAP32[$29>>2]|0;
  $34 = $30;
 } else {
  $31 = HEAP32[$4>>2]|0;
  $34 = $31;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 44|0);
 HEAP32[$33>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 $36 = (_sqlite3GetVdbe($35)|0);
 HEAP32[$v>>2] = $36;
 $37 = HEAP32[$v>>2]|0;
 $38 = ($37|0)==(0|0);
 $39 = HEAP32[$1>>2]|0;
 $40 = ($39|0)==(0|0);
 $or$cond = $38 | $40;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 28|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$1>>2]|0;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_sqlite3_strlike(30862,$46,0)|0);
 $48 = ($47|0)==(0);
 if ($48) {
  STACKTOP = sp;return;
 }
 $49 = HEAP32[$db>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 64|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (_sqlite3SchemaToIndex($49,$52)|0);
 HEAP32[$iDb>>2] = $53;
 $54 = HEAP32[$0>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$iDb>>2]|0;
 $58 = HEAP32[$db>>2]|0;
 $59 = ((($58)) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = (($60) + ($57<<4)|0);
 $62 = HEAP32[$61>>2]|0;
 $63 = (_sqlite3AuthCheck($54,28,$56,0,$62)|0);
 $64 = ($63|0)!=(0);
 if ($64) {
  STACKTOP = sp;return;
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = HEAP32[$iDb>>2]|0;
 $67 = HEAP32[$1>>2]|0;
 $68 = ((($67)) + 28|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$1>>2]|0;
 $71 = HEAP32[$70>>2]|0;
 _sqlite3TableLock($65,$66,$69,0,$71);
 $72 = HEAP32[$5>>2]|0;
 $73 = (($72) + 1)|0;
 HEAP32[$5>>2] = $73;
 HEAP32[$iTabCur>>2] = $72;
 $74 = HEAP32[$5>>2]|0;
 $75 = (($74) + 1)|0;
 HEAP32[$5>>2] = $75;
 HEAP32[$iIdxCur>>2] = $74;
 $76 = HEAP32[$0>>2]|0;
 $77 = ((($76)) + 40|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = HEAP32[$5>>2]|0;
 $80 = ($78|0)>($79|0);
 if ($80) {
  $81 = HEAP32[$0>>2]|0;
  $82 = ((($81)) + 40|0);
  $83 = HEAP32[$82>>2]|0;
  $87 = $83;
 } else {
  $84 = HEAP32[$5>>2]|0;
  $87 = $84;
 }
 $85 = HEAP32[$0>>2]|0;
 $86 = ((($85)) + 40|0);
 HEAP32[$86>>2] = $87;
 $88 = HEAP32[$0>>2]|0;
 $89 = HEAP32[$iTabCur>>2]|0;
 $90 = HEAP32[$iDb>>2]|0;
 $91 = HEAP32[$1>>2]|0;
 _sqlite3OpenTable($88,$89,$90,$91,104);
 $92 = HEAP32[$v>>2]|0;
 $93 = HEAP32[$regTabname>>2]|0;
 $94 = HEAP32[$1>>2]|0;
 $95 = HEAP32[$94>>2]|0;
 (_sqlite3VdbeLoadString($92,$93,$95)|0);
 $96 = HEAP32[$1>>2]|0;
 $97 = ((($96)) + 8|0);
 $98 = HEAP32[$97>>2]|0;
 HEAP32[$pIdx>>2] = $98;
 while(1) {
  $99 = HEAP32[$pIdx>>2]|0;
  $100 = ($99|0)!=(0|0);
  $101 = HEAP32[$2>>2]|0;
  if (!($100)) {
   break;
  }
  $102 = ($101|0)!=(0|0);
  if ($102) {
   $103 = HEAP32[$2>>2]|0;
   $104 = HEAP32[$pIdx>>2]|0;
   $105 = ($103|0)!=($104|0);
   if (!($105)) {
    label = 15;
   }
  } else {
   label = 15;
  }
  do {
   if ((label|0) == 15) {
    label = 0;
    $106 = HEAP32[$pIdx>>2]|0;
    $107 = ((($106)) + 36|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = ($108|0)==(0|0);
    if ($109) {
     HEAP8[$needTableCnt>>0] = 0;
    }
    $110 = HEAP32[$1>>2]|0;
    $111 = ((($110)) + 42|0);
    $112 = HEAP8[$111>>0]|0;
    $113 = $112&255;
    $114 = $113 & 32;
    $115 = ($114|0)==(0);
    if ($115) {
     label = 20;
    } else {
     $116 = HEAP32[$pIdx>>2]|0;
     $117 = ((($116)) + 55|0);
     $118 = HEAP8[$117>>0]|0;
     $119 = $118 & 3;
     $120 = $119&255;
     $121 = ($120|0)==(2);
     if ($121) {
      $122 = HEAP32[$pIdx>>2]|0;
      $123 = ((($122)) + 50|0);
      $124 = HEAP16[$123>>1]|0;
      $125 = $124&65535;
      HEAP32[$nCol>>2] = $125;
      $126 = HEAP32[$1>>2]|0;
      $127 = HEAP32[$126>>2]|0;
      HEAP32[$zIdxName>>2] = $127;
      $128 = HEAP32[$nCol>>2]|0;
      $129 = (($128) - 1)|0;
      HEAP32[$nColTest>>2] = $129;
     } else {
      label = 20;
     }
    }
    if ((label|0) == 20) {
     label = 0;
     $130 = HEAP32[$pIdx>>2]|0;
     $131 = ((($130)) + 52|0);
     $132 = HEAP16[$131>>1]|0;
     $133 = $132&65535;
     HEAP32[$nCol>>2] = $133;
     $134 = HEAP32[$pIdx>>2]|0;
     $135 = HEAP32[$134>>2]|0;
     HEAP32[$zIdxName>>2] = $135;
     $136 = HEAP32[$pIdx>>2]|0;
     $137 = ((($136)) + 55|0);
     $138 = HEAP8[$137>>0]|0;
     $139 = ($138&255) >>> 3;
     $140 = $139 & 1;
     $141 = $140&255;
     $142 = ($141|0)!=(0);
     if ($142) {
      $143 = HEAP32[$pIdx>>2]|0;
      $144 = ((($143)) + 50|0);
      $145 = HEAP16[$144>>1]|0;
      $146 = $145&65535;
      $$sink = $146;
     } else {
      $147 = HEAP32[$nCol>>2]|0;
      $$sink = $147;
     }
     $148 = (($$sink) - 1)|0;
     HEAP32[$nColTest>>2] = $148;
    }
    $149 = HEAP32[$v>>2]|0;
    $150 = HEAP32[$regIdxname>>2]|0;
    $151 = HEAP32[$zIdxName>>2]|0;
    (_sqlite3VdbeLoadString($149,$150,$151)|0);
    $152 = HEAP32[$0>>2]|0;
    $153 = ((($152)) + 44|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = HEAP32[$regPrev>>2]|0;
    $156 = HEAP32[$nColTest>>2]|0;
    $157 = (($155) + ($156))|0;
    $158 = ($154|0)>($157|0);
    if ($158) {
     $159 = HEAP32[$0>>2]|0;
     $160 = ((($159)) + 44|0);
     $161 = HEAP32[$160>>2]|0;
     $167 = $161;
    } else {
     $162 = HEAP32[$regPrev>>2]|0;
     $163 = HEAP32[$nColTest>>2]|0;
     $164 = (($162) + ($163))|0;
     $167 = $164;
    }
    $165 = HEAP32[$0>>2]|0;
    $166 = ((($165)) + 44|0);
    HEAP32[$166>>2] = $167;
    $168 = HEAP32[$v>>2]|0;
    $169 = HEAP32[$iIdxCur>>2]|0;
    $170 = HEAP32[$pIdx>>2]|0;
    $171 = ((($170)) + 44|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = HEAP32[$iDb>>2]|0;
    (_sqlite3VdbeAddOp3($168,104,$169,$172,$173)|0);
    $174 = HEAP32[$0>>2]|0;
    $175 = HEAP32[$pIdx>>2]|0;
    _sqlite3VdbeSetP4KeyInfo($174,$175);
    $176 = HEAP32[$v>>2]|0;
    $177 = HEAP32[$nCol>>2]|0;
    $178 = HEAP32[$regStat4>>2]|0;
    $179 = (($178) + 1)|0;
    (_sqlite3VdbeAddOp2($176,76,$177,$179)|0);
    $180 = HEAP32[$v>>2]|0;
    $181 = HEAP32[$pIdx>>2]|0;
    $182 = ((($181)) + 50|0);
    $183 = HEAP16[$182>>1]|0;
    $184 = $183&65535;
    $185 = HEAP32[$regStat4>>2]|0;
    $186 = (($185) + 2)|0;
    (_sqlite3VdbeAddOp2($180,76,$184,$186)|0);
    $187 = HEAP32[$v>>2]|0;
    $188 = HEAP32[$regStat4>>2]|0;
    $189 = (($188) + 1)|0;
    $190 = HEAP32[$regStat4>>2]|0;
    (_sqlite3VdbeAddOp4($187,89,0,$189,$190,4292,-5)|0);
    $191 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($191,2);
    $192 = HEAP32[$v>>2]|0;
    $193 = HEAP32[$iIdxCur>>2]|0;
    $194 = (_sqlite3VdbeAddOp1($192,57,$193)|0);
    HEAP32[$addrRewind>>2] = $194;
    $195 = HEAP32[$v>>2]|0;
    $196 = HEAP32[$regChng>>2]|0;
    (_sqlite3VdbeAddOp2($195,76,0,$196)|0);
    $197 = HEAP32[$v>>2]|0;
    $198 = (_sqlite3VdbeCurrentAddr($197)|0);
    HEAP32[$addrNextRow>>2] = $198;
    $199 = HEAP32[$nColTest>>2]|0;
    $200 = ($199|0)>(0);
    if ($200) {
     $201 = HEAP32[$v>>2]|0;
     $202 = (_sqlite3VdbeMakeLabel($201)|0);
     HEAP32[$endDistinctTest>>2] = $202;
     $203 = HEAP32[$db>>2]|0;
     $204 = HEAP32[$nColTest>>2]|0;
     $205 = $204<<2;
     $206 = (_sqlite3DbMallocRawNN($203,$205,0)|0);
     HEAP32[$aGotoChng>>2] = $206;
     $207 = HEAP32[$aGotoChng>>2]|0;
     $208 = ($207|0)==(0|0);
     if ($208) {
      break;
     }
     $209 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp0($209,13)|0);
     $210 = HEAP32[$v>>2]|0;
     $211 = (_sqlite3VdbeCurrentAddr($210)|0);
     HEAP32[$addrNextRow>>2] = $211;
     $212 = HEAP32[$nColTest>>2]|0;
     $213 = ($212|0)==(1);
     if ($213) {
      $214 = HEAP32[$pIdx>>2]|0;
      $215 = ((($214)) + 50|0);
      $216 = HEAP16[$215>>1]|0;
      $217 = $216&65535;
      $218 = ($217|0)==(1);
      if ($218) {
       $219 = HEAP32[$pIdx>>2]|0;
       $220 = ((($219)) + 54|0);
       $221 = HEAP8[$220>>0]|0;
       $222 = $221&255;
       $223 = ($222|0)!=(0);
       if ($223) {
        $224 = HEAP32[$v>>2]|0;
        $225 = HEAP32[$regPrev>>2]|0;
        $226 = HEAP32[$endDistinctTest>>2]|0;
        (_sqlite3VdbeAddOp2($224,35,$225,$226)|0);
       }
      }
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $227 = HEAP32[$i>>2]|0;
      $228 = HEAP32[$nColTest>>2]|0;
      $229 = ($227|0)<($228|0);
      if (!($229)) {
       break;
      }
      $230 = HEAP32[$0>>2]|0;
      $231 = HEAP32[$i>>2]|0;
      $232 = HEAP32[$pIdx>>2]|0;
      $233 = ((($232)) + 32|0);
      $234 = HEAP32[$233>>2]|0;
      $235 = (($234) + ($231<<2)|0);
      $236 = HEAP32[$235>>2]|0;
      $237 = (_sqlite3LocateCollSeq($230,$236)|0);
      HEAP32[$pColl>>2] = $237;
      $238 = HEAP32[$v>>2]|0;
      $239 = HEAP32[$i>>2]|0;
      $240 = HEAP32[$regChng>>2]|0;
      (_sqlite3VdbeAddOp2($238,76,$239,$240)|0);
      $241 = HEAP32[$v>>2]|0;
      $242 = HEAP32[$iIdxCur>>2]|0;
      $243 = HEAP32[$i>>2]|0;
      $244 = HEAP32[$regTemp>>2]|0;
      (_sqlite3VdbeAddOp3($241,96,$242,$243,$244)|0);
      $245 = HEAP32[$v>>2]|0;
      $246 = HEAP32[$regTemp>>2]|0;
      $247 = HEAP32[$regPrev>>2]|0;
      $248 = HEAP32[$i>>2]|0;
      $249 = (($247) + ($248))|0;
      $250 = HEAP32[$pColl>>2]|0;
      $251 = (_sqlite3VdbeAddOp4($245,36,$246,0,$249,$250,-4)|0);
      $252 = HEAP32[$i>>2]|0;
      $253 = HEAP32[$aGotoChng>>2]|0;
      $254 = (($253) + ($252<<2)|0);
      HEAP32[$254>>2] = $251;
      $255 = HEAP32[$v>>2]|0;
      _sqlite3VdbeChangeP5($255,-128);
      $256 = HEAP32[$i>>2]|0;
      $257 = (($256) + 1)|0;
      HEAP32[$i>>2] = $257;
     }
     $258 = HEAP32[$v>>2]|0;
     $259 = HEAP32[$nColTest>>2]|0;
     $260 = HEAP32[$regChng>>2]|0;
     (_sqlite3VdbeAddOp2($258,76,$259,$260)|0);
     $261 = HEAP32[$v>>2]|0;
     $262 = HEAP32[$endDistinctTest>>2]|0;
     (_sqlite3VdbeGoto($261,$262)|0);
     $263 = HEAP32[$v>>2]|0;
     $264 = HEAP32[$addrNextRow>>2]|0;
     $265 = (($264) - 1)|0;
     _sqlite3VdbeJumpHere($263,$265);
     HEAP32[$i>>2] = 0;
     while(1) {
      $266 = HEAP32[$i>>2]|0;
      $267 = HEAP32[$nColTest>>2]|0;
      $268 = ($266|0)<($267|0);
      $269 = HEAP32[$v>>2]|0;
      if (!($268)) {
       break;
      }
      $270 = HEAP32[$i>>2]|0;
      $271 = HEAP32[$aGotoChng>>2]|0;
      $272 = (($271) + ($270<<2)|0);
      $273 = HEAP32[$272>>2]|0;
      _sqlite3VdbeJumpHere($269,$273);
      $274 = HEAP32[$v>>2]|0;
      $275 = HEAP32[$iIdxCur>>2]|0;
      $276 = HEAP32[$i>>2]|0;
      $277 = HEAP32[$regPrev>>2]|0;
      $278 = HEAP32[$i>>2]|0;
      $279 = (($277) + ($278))|0;
      (_sqlite3VdbeAddOp3($274,96,$275,$276,$279)|0);
      $280 = HEAP32[$i>>2]|0;
      $281 = (($280) + 1)|0;
      HEAP32[$i>>2] = $281;
     }
     $282 = HEAP32[$endDistinctTest>>2]|0;
     _sqlite3VdbeResolveLabel($269,$282);
     $283 = HEAP32[$db>>2]|0;
     $284 = HEAP32[$aGotoChng>>2]|0;
     _sqlite3DbFree($283,$284);
    }
    $285 = HEAP32[$v>>2]|0;
    $286 = HEAP32[$regStat4>>2]|0;
    $287 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp4($285,89,1,$286,$287,4320,-5)|0);
    $288 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($288,2);
    $289 = HEAP32[$v>>2]|0;
    $290 = HEAP32[$iIdxCur>>2]|0;
    $291 = HEAP32[$addrNextRow>>2]|0;
    (_sqlite3VdbeAddOp2($289,7,$290,$291)|0);
    $292 = HEAP32[$v>>2]|0;
    $293 = HEAP32[$regStat4>>2]|0;
    $294 = HEAP32[$regStat1>>2]|0;
    _callStatGet($292,$293,0,$294);
    $295 = HEAP32[$v>>2]|0;
    $296 = HEAP32[$regTabname>>2]|0;
    $297 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp4($295,99,$296,3,$297,30871,0)|0);
    $298 = HEAP32[$v>>2]|0;
    $299 = HEAP32[$3>>2]|0;
    $300 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp2($298,114,$299,$300)|0);
    $301 = HEAP32[$v>>2]|0;
    $302 = HEAP32[$3>>2]|0;
    $303 = HEAP32[$regTemp>>2]|0;
    $304 = HEAP32[$regNewRowid>>2]|0;
    (_sqlite3VdbeAddOp3($301,115,$302,$303,$304)|0);
    $305 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($305,8);
    $306 = HEAP32[$v>>2]|0;
    $307 = HEAP32[$addrRewind>>2]|0;
    _sqlite3VdbeJumpHere($306,$307);
   }
  } while(0);
  $308 = HEAP32[$pIdx>>2]|0;
  $309 = ((($308)) + 20|0);
  $310 = HEAP32[$309>>2]|0;
  HEAP32[$pIdx>>2] = $310;
 }
 $311 = ($101|0)==(0|0);
 if (!($311)) {
  STACKTOP = sp;return;
 }
 $312 = HEAP8[$needTableCnt>>0]|0;
 $313 = $312&255;
 $314 = ($313|0)!=(0);
 if (!($314)) {
  STACKTOP = sp;return;
 }
 $315 = HEAP32[$v>>2]|0;
 $316 = HEAP32[$iTabCur>>2]|0;
 $317 = HEAP32[$regStat1>>2]|0;
 (_sqlite3VdbeAddOp2($315,100,$316,$317)|0);
 $318 = HEAP32[$v>>2]|0;
 $319 = HEAP32[$regStat1>>2]|0;
 $320 = (_sqlite3VdbeAddOp1($318,22,$319)|0);
 HEAP32[$jZeroRows>>2] = $320;
 $321 = HEAP32[$v>>2]|0;
 $322 = HEAP32[$regIdxname>>2]|0;
 (_sqlite3VdbeAddOp2($321,79,0,$322)|0);
 $323 = HEAP32[$v>>2]|0;
 $324 = HEAP32[$regTabname>>2]|0;
 $325 = HEAP32[$regTemp>>2]|0;
 (_sqlite3VdbeAddOp4($323,99,$324,3,$325,30871,0)|0);
 $326 = HEAP32[$v>>2]|0;
 $327 = HEAP32[$3>>2]|0;
 $328 = HEAP32[$regNewRowid>>2]|0;
 (_sqlite3VdbeAddOp2($326,114,$327,$328)|0);
 $329 = HEAP32[$v>>2]|0;
 $330 = HEAP32[$3>>2]|0;
 $331 = HEAP32[$regTemp>>2]|0;
 $332 = HEAP32[$regNewRowid>>2]|0;
 (_sqlite3VdbeAddOp3($329,115,$330,$331,$332)|0);
 $333 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($333,8);
 $334 = HEAP32[$v>>2]|0;
 $335 = HEAP32[$jZeroRows>>2]|0;
 _sqlite3VdbeJumpHere($334,$335);
 STACKTOP = sp;return;
}
function _loadAnalysis($pParse,$iDb) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetVdbe($2)|0);
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$v>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$v>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp1($6,137,$7)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeAddOp1($p,$op,$p1) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3VdbeAddOp3($3,$4,$5,0,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3TableLock($pParse,$iDb,$iTab,$isWriteLock,$zName) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $iTab = $iTab|0;
 $isWriteLock = $isWriteLock|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $i = 0, $nBytes = 0, $p = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 32|0;
 $4 = sp + 16|0;
 $pToplevel = sp + 12|0;
 $i = sp + 8|0;
 $nBytes = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iTab;
 HEAP8[$3>>0] = $isWriteLock;
 HEAP32[$4>>2] = $zName;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 124|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = ((($9)) + 124|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = $9;
 }
 HEAP32[$pToplevel>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$pToplevel>>2]|0;
  $15 = ((($14)) + 112|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = HEAP32[$pToplevel>>2]|0;
  $20 = ((($19)) + 116|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18<<4)|0);
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ($24|0)==($25|0);
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = ((($27)) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$2>>2]|0;
   $31 = ($29|0)==($30|0);
   if ($31) {
    label = 7;
    break;
   }
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 8|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = ($35|0)!=(0);
  if ($36) {
   $41 = 1;
  } else {
   $37 = HEAP8[$3>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)!=(0);
   $41 = $39;
  }
  $40 = $41&1;
  $42 = $40&255;
  $43 = HEAP32[$p>>2]|0;
  $44 = ((($43)) + 8|0);
  HEAP8[$44>>0] = $42;
  STACKTOP = sp;return;
 }
 $47 = HEAP32[$pToplevel>>2]|0;
 $48 = ((($47)) + 112|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = (($49) + 1)|0;
 $51 = $50<<4;
 HEAP32[$nBytes>>2] = $51;
 $52 = HEAP32[$pToplevel>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 $54 = HEAP32[$pToplevel>>2]|0;
 $55 = ((($54)) + 116|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$nBytes>>2]|0;
 $58 = ($57|0)<(0);
 $59 = $58 << 31 >> 31;
 $60 = (_sqlite3DbReallocOrFree($53,$56,$57,$59)|0);
 $61 = HEAP32[$pToplevel>>2]|0;
 $62 = ((($61)) + 116|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$pToplevel>>2]|0;
 $64 = ((($63)) + 116|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = ($65|0)!=(0|0);
 $67 = HEAP32[$pToplevel>>2]|0;
 $68 = ((($67)) + 112|0);
 if ($66) {
  $69 = HEAP32[$68>>2]|0;
  $70 = (($69) + 1)|0;
  HEAP32[$68>>2] = $70;
  $71 = HEAP32[$pToplevel>>2]|0;
  $72 = ((($71)) + 116|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + ($69<<4)|0);
  HEAP32[$p>>2] = $74;
  $75 = HEAP32[$1>>2]|0;
  $76 = HEAP32[$p>>2]|0;
  HEAP32[$76>>2] = $75;
  $77 = HEAP32[$2>>2]|0;
  $78 = HEAP32[$p>>2]|0;
  $79 = ((($78)) + 4|0);
  HEAP32[$79>>2] = $77;
  $80 = HEAP8[$3>>0]|0;
  $81 = HEAP32[$p>>2]|0;
  $82 = ((($81)) + 8|0);
  HEAP8[$82>>0] = $80;
  $83 = HEAP32[$4>>2]|0;
  $84 = HEAP32[$p>>2]|0;
  $85 = ((($84)) + 12|0);
  HEAP32[$85>>2] = $83;
  STACKTOP = sp;return;
 } else {
  HEAP32[$68>>2] = 0;
  $86 = HEAP32[$pToplevel>>2]|0;
  $87 = HEAP32[$86>>2]|0;
  _sqlite3OomFault($87);
  STACKTOP = sp;return;
 }
}
function _sqlite3OpenTable($pParse,$iCur,$iDb,$pTab,$opcode) {
 $pParse = $pParse|0;
 $iCur = $iCur|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 $opcode = $opcode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPk = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $v = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$4>>2] = $opcode;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3GetVdbe($5)|0);
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 28|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)==(105);
 $14 = $13 ? 1 : 0;
 $15 = $14&255;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 _sqlite3TableLock($7,$8,$11,$15,$17);
 $18 = HEAP32[$3>>2]|0;
 $19 = ((($18)) + 42|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = $21 & 32;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = HEAP32[$v>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = ((($27)) + 28|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  $31 = HEAP32[$3>>2]|0;
  $32 = ((($31)) + 34|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33 << 16 >> 16;
  (_sqlite3VdbeAddOp4Int($24,$25,$26,$29,$30,$34)|0);
  STACKTOP = sp;return;
 } else {
  $35 = HEAP32[$3>>2]|0;
  $36 = (_sqlite3PrimaryKeyIndex($35)|0);
  HEAP32[$pPk>>2] = $36;
  $37 = HEAP32[$v>>2]|0;
  $38 = HEAP32[$4>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  $40 = HEAP32[$pPk>>2]|0;
  $41 = ((($40)) + 44|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  (_sqlite3VdbeAddOp3($37,$38,$39,$42,$43)|0);
  $44 = HEAP32[$0>>2]|0;
  $45 = HEAP32[$pPk>>2]|0;
  _sqlite3VdbeSetP4KeyInfo($44,$45);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeSetP4KeyInfo($pParse,$pIdx) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$v>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3KeyInfoOfIndex($6,$7)|0);
 _sqlite3VdbeChangeP4($5,-1,$8,-6);
 STACKTOP = sp;return;
}
function _sqlite3VdbeMakeLabel($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 $5 = ((($4)) + 72|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + 1)|0;
 HEAP32[$5>>2] = $7;
 HEAP32[$i>>2] = $6;
 $8 = HEAP32[$i>>2]|0;
 $9 = HEAP32[$i>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = $8 & $10;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = ((($15)) + 76|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$i>>2]|0;
  $19 = $18<<1;
  $20 = (($19) + 1)|0;
  $21 = $20<<2;
  $22 = (_sqlite3DbReallocOrFree($14,$17,$21,0)|0);
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 76|0);
  HEAP32[$24>>2] = $22;
 }
 $25 = HEAP32[$p>>2]|0;
 $26 = ((($25)) + 76|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)!=(0|0);
 if (!($28)) {
  $34 = HEAP32[$i>>2]|0;
  $35 = (-1 - ($34))|0;
  STACKTOP = sp;return ($35|0);
 }
 $29 = HEAP32[$i>>2]|0;
 $30 = HEAP32[$p>>2]|0;
 $31 = ((($30)) + 76|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + ($29<<2)|0);
 HEAP32[$33>>2] = -1;
 $34 = HEAP32[$i>>2]|0;
 $35 = (-1 - ($34))|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3LocateCollSeq($pParse,$zName) {
 $pParse = $pParse|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $enc = 0, $initbusy = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 $enc = sp + 17|0;
 $initbusy = sp + 16|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zName;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 66|0);
 $6 = HEAP8[$5>>0]|0;
 HEAP8[$enc>>0] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 148|0);
 $9 = ((($8)) + 5|0);
 $10 = HEAP8[$9>>0]|0;
 HEAP8[$initbusy>>0] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = HEAP8[$enc>>0]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP8[$initbusy>>0]|0;
 $15 = $14&255;
 $16 = (_sqlite3FindCollSeq($11,$12,$13,$15)|0);
 HEAP32[$pColl>>2] = $16;
 $17 = HEAP8[$initbusy>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 if ($18) {
  $30 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $19 = HEAP32[$pColl>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = HEAP32[$pColl>>2]|0;
  $22 = ((($21)) + 12|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $30 = HEAP32[$pColl>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP8[$enc>>0]|0;
 $27 = HEAP32[$pColl>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 $29 = (_sqlite3GetCollSeq($25,$26,$27,$28)|0);
 HEAP32[$pColl>>2] = $29;
 $30 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3VdbeGoto($p,$iDest) {
 $p = $p|0;
 $iDest = $iDest|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDest;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3VdbeAddOp3($2,13,0,$3,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeJumpHere($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 136|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3VdbeChangeP2($2,$3,$6);
 STACKTOP = sp;return;
}
function _sqlite3VdbeResolveLabel($v,$x) {
 $v = $v|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = (-1 - ($5))|0;
 HEAP32[$j>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 76|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 136|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$j>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 $16 = ((($15)) + 76|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14<<2)|0);
 HEAP32[$18>>2] = $13;
 STACKTOP = sp;return;
}
function _callStatGet($v,$regStat4,$iParam,$regOut) {
 $v = $v|0;
 $regStat4 = $regStat4|0;
 $iParam = $iParam|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $regStat4;
 HEAP32[$2>>2] = $iParam;
 HEAP32[$3>>2] = $regOut;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4($4,89,0,$5,$6,4348,-5)|0);
 $7 = HEAP32[$0>>2]|0;
 _sqlite3VdbeChangeP5($7,1);
 STACKTOP = sp;return;
}
function _statGet($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $i = 0, $iVal = 0, $nDistinct = 0, $p = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $z = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 24|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $p = sp + 44|0;
 $z = sp + 40|0;
 $i = sp + 36|0;
 $zRet = sp + 32|0;
 $nDistinct = sp + 8|0;
 $iVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 $10 = ($9*25)|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (_sqlite3MallocZero($10,$12)|0);
 HEAP32[$zRet>>2] = $13;
 $14 = HEAP32[$zRet>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  $16 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($16);
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$zRet>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = $vararg_buffer;
 $21 = $20;
 HEAP32[$21>>2] = $19;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = 0;
 (_sqlite3_snprintf(24,$17,30884,$vararg_buffer)|0);
 $24 = HEAP32[$zRet>>2]|0;
 $25 = HEAP32[$zRet>>2]|0;
 $26 = (_sqlite3Strlen30($25)|0);
 $27 = (($24) + ($26)|0);
 HEAP32[$z>>2] = $27;
 HEAP32[$i>>2] = 0;
 while(1) {
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$p>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($28|0)<($31|0);
  if (!($32)) {
   break;
  }
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 20|0);
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($33<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + 1)|0;
  $41 = $nDistinct;
  $42 = $41;
  HEAP32[$42>>2] = $40;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = 0;
  $45 = HEAP32[$p>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  $47 = $nDistinct;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (_i64Add(($46|0),0,($49|0),($52|0))|0);
  $54 = tempRet0;
  $55 = (_i64Subtract(($53|0),($54|0),1,0)|0);
  $56 = tempRet0;
  $57 = $nDistinct;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (___udivdi3(($55|0),($56|0),($59|0),($62|0))|0);
  $64 = tempRet0;
  $65 = $iVal;
  $66 = $65;
  HEAP32[$66>>2] = $63;
  $67 = (($65) + 4)|0;
  $68 = $67;
  HEAP32[$68>>2] = $64;
  $69 = HEAP32[$z>>2]|0;
  $70 = $iVal;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $vararg_buffer1;
  $77 = $76;
  HEAP32[$77>>2] = $72;
  $78 = (($76) + 4)|0;
  $79 = $78;
  HEAP32[$79>>2] = $75;
  (_sqlite3_snprintf(24,$69,30889,$vararg_buffer1)|0);
  $80 = HEAP32[$z>>2]|0;
  $81 = (_sqlite3Strlen30($80)|0);
  $82 = HEAP32[$z>>2]|0;
  $83 = (($82) + ($81)|0);
  HEAP32[$z>>2] = $83;
  $84 = HEAP32[$i>>2]|0;
  $85 = (($84) + 1)|0;
  HEAP32[$i>>2] = $85;
 }
 $86 = HEAP32[$0>>2]|0;
 $87 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($86,$87,-1,148);
 STACKTOP = sp;return;
}
function _statPush($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $iChng = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $p = sp + 4|0;
 $iChng = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_int($8)|0);
 HEAP32[$iChng>>2] = $9;
 $10 = HEAP32[$p>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $13 = HEAP32[$i>>2]|0;
   $14 = HEAP32[$p>>2]|0;
   $15 = ((($14)) + 8|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($13|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 20|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = (($21) + ($18<<2)|0);
   HEAP32[$22>>2] = 1;
   $23 = HEAP32[$i>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$i>>2] = $24;
  }
  $60 = HEAP32[$p>>2]|0;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + 1)|0;
  HEAP32[$60>>2] = $62;
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$p>>2]|0;
 $26 = HEAP32[$iChng>>2]|0;
 _samplePushPrevious($25,$26);
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$iChng>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$p>>2]|0;
  $32 = ((($31)) + 20|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($30<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35) + 1)|0;
  HEAP32[$34>>2] = $36;
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 $39 = HEAP32[$iChng>>2]|0;
 HEAP32[$i>>2] = $39;
 while(1) {
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 8|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = ($40|0)<($43|0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$p>>2]|0;
  $47 = ((($46)) + 20|0);
  $48 = ((($47)) + 4|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($45<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$50>>2] = $52;
  $53 = HEAP32[$i>>2]|0;
  $54 = HEAP32[$p>>2]|0;
  $55 = ((($54)) + 20|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (($56) + ($53<<2)|0);
  HEAP32[$57>>2] = 1;
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
 }
 $60 = HEAP32[$p>>2]|0;
 $61 = HEAP32[$60>>2]|0;
 $62 = (($61) + 1)|0;
 HEAP32[$60>>2] = $62;
 STACKTOP = sp;return;
}
function _samplePushPrevious($p,$iChng) {
 $p = $p|0;
 $iChng = $iChng|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iChng;
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP2($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 $7 = ((($6)) + 8|0);
 HEAP32[$7>>2] = $3;
 STACKTOP = sp;return;
}
function _sqlite3VdbeGetOp($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $addr;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)<(0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 136|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) - 1)|0;
  HEAP32[$2>>2] = $8;
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 69|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 if ($13) {
  HEAP32[$0>>2] = 4376;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 88|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($14*20)|0)|0);
  HEAP32[$0>>2] = $18;
  $19 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _statInit($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $nCol = 0, $nColUp = 0, $nKeyCol = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $p = sp + 20|0;
 $nCol = sp + 16|0;
 $nKeyCol = sp + 12|0;
 $nColUp = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$nCol>>2] = $5;
 $6 = HEAP32[$nCol>>2]|0;
 $7 = (($6) + 1)|0;
 $8 = $7 & -2;
 HEAP32[$nColUp>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3_value_int($11)|0);
 HEAP32[$nKeyCol>>2] = $12;
 $13 = HEAP32[$nColUp>>2]|0;
 $14 = $13<<2;
 $15 = (56 + ($14))|0;
 $16 = HEAP32[$nColUp>>2]|0;
 $17 = $16<<2;
 $18 = (($15) + ($17))|0;
 HEAP32[$n>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = (_sqlite3_context_db_handle($19)|0);
 HEAP32[$db>>2] = $20;
 $21 = HEAP32[$db>>2]|0;
 $22 = HEAP32[$n>>2]|0;
 $23 = ($22|0)<(0);
 $24 = $23 << 31 >> 31;
 $25 = (_sqlite3DbMallocZero($21,$22,$24)|0);
 HEAP32[$p>>2] = $25;
 $26 = HEAP32[$p>>2]|0;
 $27 = ($26|0)==(0|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($28);
  STACKTOP = sp;return;
 } else {
  $29 = HEAP32[$db>>2]|0;
  $30 = HEAP32[$p>>2]|0;
  $31 = ((($30)) + 52|0);
  HEAP32[$31>>2] = $29;
  $32 = HEAP32[$p>>2]|0;
  HEAP32[$32>>2] = 0;
  $33 = HEAP32[$nCol>>2]|0;
  $34 = HEAP32[$p>>2]|0;
  $35 = ((($34)) + 8|0);
  HEAP32[$35>>2] = $33;
  $36 = HEAP32[$nKeyCol>>2]|0;
  $37 = HEAP32[$p>>2]|0;
  $38 = ((($37)) + 12|0);
  HEAP32[$38>>2] = $36;
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 56|0);
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 20|0);
  $43 = ((($42)) + 4|0);
  HEAP32[$43>>2] = $40;
  $44 = HEAP32[$nColUp>>2]|0;
  $45 = HEAP32[$p>>2]|0;
  $46 = ((($45)) + 20|0);
  $47 = ((($46)) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (($48) + ($44<<2)|0);
  $50 = HEAP32[$p>>2]|0;
  $51 = ((($50)) + 20|0);
  HEAP32[$51>>2] = $49;
  $52 = HEAP32[$0>>2]|0;
  $53 = HEAP32[$p>>2]|0;
  _sqlite3_result_blob($52,$53,56,153);
  STACKTOP = sp;return;
 }
}
function _stat4Destructor($pOld) {
 $pOld = $pOld|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pOld;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 52|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$p>>2]|0;
 _sqlite3DbFree($4,$5);
 STACKTOP = sp;return;
}
function _sqlite3KeyInfoOfIndex($pParse,$pIdx) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $i = 0, $nCol = 0, $nKey = 0, $pKey = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $nCol = sp + 12|0;
 $nKey = sp + 8|0;
 $pKey = sp + 4|0;
 $zColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 HEAP32[$nCol>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 50|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 HEAP32[$nKey>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 36|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = ((($15)) + 55|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = ($17&255) >>> 3;
 $19 = $18 & 1;
 $20 = $19&255;
 $21 = ($20|0)!=(0);
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 if ($21) {
  $24 = HEAP32[$nKey>>2]|0;
  $25 = HEAP32[$nCol>>2]|0;
  $26 = HEAP32[$nKey>>2]|0;
  $27 = (($25) - ($26))|0;
  $28 = (_sqlite3KeyInfoAlloc($23,$24,$27)|0);
  HEAP32[$pKey>>2] = $28;
 } else {
  $29 = HEAP32[$nCol>>2]|0;
  $30 = (_sqlite3KeyInfoAlloc($23,$29,0)|0);
  HEAP32[$pKey>>2] = $30;
 }
 $31 = HEAP32[$pKey>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $33 = HEAP32[$i>>2]|0;
   $34 = HEAP32[$nCol>>2]|0;
   $35 = ($33|0)<($34|0);
   if (!($35)) {
    break;
   }
   $36 = HEAP32[$i>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = ((($37)) + 32|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + ($36<<2)|0);
   $41 = HEAP32[$40>>2]|0;
   HEAP32[$zColl>>2] = $41;
   $42 = HEAP32[$zColl>>2]|0;
   $43 = ($42|0)==(34834|0);
   if ($43) {
    $51 = 0;
   } else {
    $44 = HEAP32[$1>>2]|0;
    $45 = HEAP32[$zColl>>2]|0;
    $46 = (_sqlite3LocateCollSeq($44,$45)|0);
    $51 = $46;
   }
   $47 = HEAP32[$i>>2]|0;
   $48 = HEAP32[$pKey>>2]|0;
   $49 = ((($48)) + 20|0);
   $50 = (($49) + ($47<<2)|0);
   HEAP32[$50>>2] = $51;
   $52 = HEAP32[$i>>2]|0;
   $53 = HEAP32[$2>>2]|0;
   $54 = ((($53)) + 28|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + ($52)|0);
   $57 = HEAP8[$56>>0]|0;
   $58 = HEAP32[$i>>2]|0;
   $59 = HEAP32[$pKey>>2]|0;
   $60 = ((($59)) + 16|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + ($58)|0);
   HEAP8[$62>>0] = $57;
   $63 = HEAP32[$i>>2]|0;
   $64 = (($63) + 1)|0;
   HEAP32[$i>>2] = $64;
  }
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 36|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)!=(0);
  if ($68) {
   $69 = HEAP32[$pKey>>2]|0;
   _sqlite3KeyInfoUnref($69);
   HEAP32[$pKey>>2] = 0;
  }
 }
 $70 = HEAP32[$pKey>>2]|0;
 HEAP32[$0>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($71|0);
}
function _sqlite3KeyInfoAlloc($db,$N,$X) {
 $db = $db|0;
 $N = $N|0;
 $X = $X|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $nExtra = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nExtra = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $X;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (($3) + ($4))|0;
 $6 = ($5*5)|0;
 HEAP32[$nExtra>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$nExtra>>2]|0;
 $9 = (24 + ($8))|0;
 $10 = (_sqlite3DbMallocRawNN($7,$9,0)|0);
 HEAP32[$p>>2] = $10;
 $11 = HEAP32[$p>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = (($13) + ($14))|0;
  $16 = HEAP32[$p>>2]|0;
  $17 = ((($16)) + 20|0);
  $18 = (($17) + ($15<<2)|0);
  $19 = HEAP32[$p>>2]|0;
  $20 = ((($19)) + 16|0);
  HEAP32[$20>>2] = $18;
  $21 = HEAP32[$1>>2]|0;
  $22 = $21&65535;
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 6|0);
  HEAP16[$24>>1] = $22;
  $25 = HEAP32[$2>>2]|0;
  $26 = $25&65535;
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 8|0);
  HEAP16[$28>>1] = $26;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 66|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 4|0);
  HEAP8[$33>>0] = $31;
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 12|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$p>>2]|0;
  HEAP32[$37>>2] = 1;
  $38 = HEAP32[$p>>2]|0;
  $39 = ((($38)) + 24|0);
  $40 = HEAP32[$nExtra>>2]|0;
  _memset(($39|0),0,($40|0))|0;
  $42 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($42|0);
 } else {
  $41 = HEAP32[$0>>2]|0;
  _sqlite3OomFault($41);
  $42 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _sqlite3KeyInfoUnref($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + -1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($11,$12);
 STACKTOP = sp;return;
}
function _reindexDatabases($pParse,$zColl) {
 $pParse = $pParse|0;
 $zColl = $zColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $k = 0, $pDb = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pDb = sp + 16|0;
 $iDb = sp + 12|0;
 $db = sp + 8|0;
 $k = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zColl;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$iDb>>2] = 0;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pDb>>2] = $6;
 while(1) {
  $7 = HEAP32[$iDb>>2]|0;
  $8 = HEAP32[$db>>2]|0;
  $9 = ((($8)) + 20|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ($7|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pDb>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$k>>2] = $17;
  while(1) {
   $18 = HEAP32[$k>>2]|0;
   $19 = ($18|0)!=(0|0);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$k>>2]|0;
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pTab>>2] = $22;
   $23 = HEAP32[$0>>2]|0;
   $24 = HEAP32[$pTab>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   _reindexTable($23,$24,$25);
   $26 = HEAP32[$k>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$k>>2] = $27;
  }
  $28 = HEAP32[$iDb>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$iDb>>2] = $29;
  $30 = HEAP32[$pDb>>2]|0;
  $31 = ((($30)) + 16|0);
  HEAP32[$pDb>>2] = $31;
 }
 STACKTOP = sp;return;
}
function _reindexTable($pParse,$pTab,$zColl) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $zColl = $zColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $pIndex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pIndex = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $zColl;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pIndex>>2] = $5;
 while(1) {
  $6 = HEAP32[$pIndex>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$2>>2]|0;
  $9 = ($8|0)==(0|0);
  if ($9) {
   label = 5;
  } else {
   $10 = HEAP32[$2>>2]|0;
   $11 = HEAP32[$pIndex>>2]|0;
   $12 = (_collationMatch($10,$11)|0);
   $13 = ($12|0)!=(0);
   if ($13) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $14 = HEAP32[$0>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 64|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = (_sqlite3SchemaToIndex($15,$18)|0);
   HEAP32[$iDb>>2] = $19;
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$iDb>>2]|0;
   _sqlite3BeginWriteOperation($20,0,$21);
   $22 = HEAP32[$0>>2]|0;
   $23 = HEAP32[$pIndex>>2]|0;
   _sqlite3RefillIndex($22,$23,-1);
  }
  $24 = HEAP32[$pIndex>>2]|0;
  $25 = ((($24)) + 20|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pIndex>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _sqlite3RefillIndex($pParse,$pIndex,$memRootPage) {
 $pParse = $pParse|0;
 $pIndex = $pIndex|0;
 $memRootPage = $memRootPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $addr2 = 0, $db = 0, $iDb = 0, $iIdx = 0, $iPartIdxLabel = 0, $iSorter = 0, $iTab = 0;
 var $j2 = 0, $pKey = 0, $pTab = 0, $regRecord = 0, $tnum = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $pTab = sp + 52|0;
 $iTab = sp + 48|0;
 $iIdx = sp + 44|0;
 $iSorter = sp + 40|0;
 $addr1 = sp + 36|0;
 $addr2 = sp + 32|0;
 $tnum = sp + 28|0;
 $iPartIdxLabel = sp + 24|0;
 $v = sp + 20|0;
 $pKey = sp + 16|0;
 $regRecord = sp + 12|0;
 $db = sp + 8|0;
 $iDb = sp + 4|0;
 $j2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIndex;
 HEAP32[$2>>2] = $memRootPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + 1)|0;
 HEAP32[$7>>2] = $9;
 HEAP32[$iTab>>2] = $8;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 40|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$11>>2] = $13;
 HEAP32[$iIdx>>2] = $12;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$db>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 24|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$iDb>>2]|0;
 $25 = HEAP32[$db>>2]|0;
 $26 = ((($25)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24<<4)|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3AuthCheck($21,27,$23,0,$29)|0);
 $31 = ($30|0)!=(0);
 if ($31) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$iDb>>2]|0;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = ((($34)) + 28|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = HEAP32[$pTab>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 _sqlite3TableLock($32,$33,$36,1,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = (_sqlite3GetVdbe($39)|0);
 HEAP32[$v>>2] = $40;
 $41 = HEAP32[$v>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$2>>2]|0;
 $44 = ($43|0)>=(0);
 if ($44) {
  $45 = HEAP32[$2>>2]|0;
  HEAP32[$tnum>>2] = $45;
 } else {
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 44|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$tnum>>2] = $48;
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = (_sqlite3KeyInfoOfIndex($49,$50)|0);
 HEAP32[$pKey>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 40|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = (($54) + 1)|0;
 HEAP32[$53>>2] = $55;
 HEAP32[$iSorter>>2] = $54;
 $56 = HEAP32[$v>>2]|0;
 $57 = HEAP32[$iSorter>>2]|0;
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 50|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60&65535;
 $62 = HEAP32[$pKey>>2]|0;
 $63 = (_sqlite3KeyInfoRef($62)|0);
 (_sqlite3VdbeAddOp4($56,108,$57,0,$61,$63,-6)|0);
 $64 = HEAP32[$0>>2]|0;
 $65 = HEAP32[$iTab>>2]|0;
 $66 = HEAP32[$iDb>>2]|0;
 $67 = HEAP32[$pTab>>2]|0;
 _sqlite3OpenTable($64,$65,$66,$67,104);
 $68 = HEAP32[$v>>2]|0;
 $69 = HEAP32[$iTab>>2]|0;
 $70 = (_sqlite3VdbeAddOp2($68,57,$69,0)|0);
 HEAP32[$addr1>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 $72 = (_sqlite3GetTempReg($71)|0);
 HEAP32[$regRecord>>2] = $72;
 $73 = HEAP32[$0>>2]|0;
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$iTab>>2]|0;
 $76 = HEAP32[$regRecord>>2]|0;
 (_sqlite3GenerateIndexKey($73,$74,$75,$76,0,$iPartIdxLabel,0,0)|0);
 $77 = HEAP32[$v>>2]|0;
 $78 = HEAP32[$iSorter>>2]|0;
 $79 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($77,125,$78,$79)|0);
 $80 = HEAP32[$0>>2]|0;
 $81 = HEAP32[$iPartIdxLabel>>2]|0;
 _sqlite3ResolvePartIdxLabel($80,$81);
 $82 = HEAP32[$v>>2]|0;
 $83 = HEAP32[$iTab>>2]|0;
 $84 = HEAP32[$addr1>>2]|0;
 $85 = (($84) + 1)|0;
 (_sqlite3VdbeAddOp2($82,7,$83,$85)|0);
 $86 = HEAP32[$v>>2]|0;
 $87 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($86,$87);
 $88 = HEAP32[$2>>2]|0;
 $89 = ($88|0)<(0);
 if ($89) {
  $90 = HEAP32[$v>>2]|0;
  $91 = HEAP32[$tnum>>2]|0;
  $92 = HEAP32[$iDb>>2]|0;
  (_sqlite3VdbeAddOp2($90,131,$91,$92)|0);
 }
 $93 = HEAP32[$v>>2]|0;
 $94 = HEAP32[$iIdx>>2]|0;
 $95 = HEAP32[$tnum>>2]|0;
 $96 = HEAP32[$iDb>>2]|0;
 $97 = HEAP32[$pKey>>2]|0;
 (_sqlite3VdbeAddOp4($93,105,$94,$95,$96,$97,-6)|0);
 $98 = HEAP32[$v>>2]|0;
 $99 = HEAP32[$2>>2]|0;
 $100 = ($99|0)>=(0);
 $101 = $100 ? 16 : 0;
 $102 = 1 | $101;
 $103 = $102&255;
 _sqlite3VdbeChangeP5($98,$103);
 $104 = HEAP32[$v>>2]|0;
 $105 = HEAP32[$iSorter>>2]|0;
 $106 = (_sqlite3VdbeAddOp2($104,55,$105,0)|0);
 HEAP32[$addr1>>2] = $106;
 $107 = HEAP32[$1>>2]|0;
 $108 = ((($107)) + 54|0);
 $109 = HEAP8[$108>>0]|0;
 $110 = $109&255;
 $111 = ($110|0)!=(0);
 $112 = HEAP32[$v>>2]|0;
 $113 = (_sqlite3VdbeCurrentAddr($112)|0);
 if ($111) {
  $114 = (($113) + 3)|0;
  HEAP32[$j2>>2] = $114;
  $115 = HEAP32[$v>>2]|0;
  $116 = HEAP32[$j2>>2]|0;
  (_sqlite3VdbeGoto($115,$116)|0);
  $117 = HEAP32[$v>>2]|0;
  $118 = (_sqlite3VdbeCurrentAddr($117)|0);
  HEAP32[$addr2>>2] = $118;
  $119 = HEAP32[$v>>2]|0;
  $120 = HEAP32[$iSorter>>2]|0;
  $121 = HEAP32[$j2>>2]|0;
  $122 = HEAP32[$regRecord>>2]|0;
  $123 = HEAP32[$1>>2]|0;
  $124 = ((($123)) + 50|0);
  $125 = HEAP16[$124>>1]|0;
  $126 = $125&65535;
  (_sqlite3VdbeAddOp4Int($119,119,$120,$121,$122,$126)|0);
  $127 = HEAP32[$0>>2]|0;
  $128 = HEAP32[$1>>2]|0;
  _sqlite3UniqueConstraint($127,2,$128);
 } else {
  HEAP32[$addr2>>2] = $113;
 }
 $129 = HEAP32[$v>>2]|0;
 $130 = HEAP32[$iSorter>>2]|0;
 $131 = HEAP32[$regRecord>>2]|0;
 $132 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp3($129,120,$130,$131,$132)|0);
 $133 = HEAP32[$v>>2]|0;
 $134 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp3($133,53,$134,0,-1)|0);
 $135 = HEAP32[$v>>2]|0;
 $136 = HEAP32[$iIdx>>2]|0;
 $137 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp3($135,126,$136,$137,0)|0);
 $138 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($138,16);
 $139 = HEAP32[$0>>2]|0;
 $140 = HEAP32[$regRecord>>2]|0;
 _sqlite3ReleaseTempReg($139,$140);
 $141 = HEAP32[$v>>2]|0;
 $142 = HEAP32[$iSorter>>2]|0;
 $143 = HEAP32[$addr2>>2]|0;
 (_sqlite3VdbeAddOp2($141,3,$142,$143)|0);
 $144 = HEAP32[$v>>2]|0;
 $145 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($144,$145);
 $146 = HEAP32[$v>>2]|0;
 $147 = HEAP32[$iTab>>2]|0;
 (_sqlite3VdbeAddOp1($146,111,$147)|0);
 $148 = HEAP32[$v>>2]|0;
 $149 = HEAP32[$iIdx>>2]|0;
 (_sqlite3VdbeAddOp1($148,111,$149)|0);
 $150 = HEAP32[$v>>2]|0;
 $151 = HEAP32[$iSorter>>2]|0;
 (_sqlite3VdbeAddOp1($150,111,$151)|0);
 STACKTOP = sp;return;
}
function _sqlite3KeyInfoRef($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if ($2) {
  $3 = HEAP32[$0>>2]|0;
  $4 = HEAP32[$3>>2]|0;
  $5 = (($4) + 1)|0;
  HEAP32[$3>>2] = $5;
 }
 $6 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3GenerateIndexKey($pParse,$pIdx,$iDataCur,$regOut,$prefixOnly,$piPartIdxLabel,$pPrior,$regPrior) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 $iDataCur = $iDataCur|0;
 $regOut = $regOut|0;
 $prefixOnly = $prefixOnly|0;
 $piPartIdxLabel = $piPartIdxLabel|0;
 $pPrior = $pPrior|0;
 $regPrior = $regPrior|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $j = 0, $nCol = 0, $regBase = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $7 = sp + 16|0;
 $v = sp + 12|0;
 $j = sp + 8|0;
 $regBase = sp + 4|0;
 $nCol = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $regOut;
 HEAP32[$4>>2] = $prefixOnly;
 HEAP32[$5>>2] = $piPartIdxLabel;
 HEAP32[$6>>2] = $pPrior;
 HEAP32[$7>>2] = $regPrior;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$5>>2]|0;
 $12 = ($11|0)!=(0|0);
 do {
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 36|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    $17 = HEAP32[$v>>2]|0;
    $18 = (_sqlite3VdbeMakeLabel($17)|0);
    $19 = HEAP32[$5>>2]|0;
    HEAP32[$19>>2] = $18;
    $20 = HEAP32[$2>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ((($21)) + 60|0);
    HEAP32[$22>>2] = $20;
    $23 = HEAP32[$0>>2]|0;
    _sqlite3ExprCachePush($23);
    $24 = HEAP32[$0>>2]|0;
    $25 = HEAP32[$1>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = HEAP32[$5>>2]|0;
    $29 = HEAP32[$28>>2]|0;
    _sqlite3ExprIfFalseDup($24,$27,$29,16);
    break;
   } else {
    $30 = HEAP32[$5>>2]|0;
    HEAP32[$30>>2] = 0;
    break;
   }
  }
 } while(0);
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 55|0);
  $35 = HEAP8[$34>>0]|0;
  $36 = ($35&255) >>> 3;
  $37 = $36 & 1;
  $38 = $37&255;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 50|0);
   $42 = HEAP16[$41>>1]|0;
   $$sink = $42;
  } else {
   label = 8;
  }
 } else {
  label = 8;
 }
 if ((label|0) == 8) {
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 52|0);
  $45 = HEAP16[$44>>1]|0;
  $$sink = $45;
 }
 $46 = $$sink&65535;
 HEAP32[$nCol>>2] = $46;
 $47 = HEAP32[$0>>2]|0;
 $48 = HEAP32[$nCol>>2]|0;
 $49 = (_sqlite3GetTempRange($47,$48)|0);
 HEAP32[$regBase>>2] = $49;
 $50 = HEAP32[$6>>2]|0;
 $51 = ($50|0)!=(0|0);
 do {
  if ($51) {
   $52 = HEAP32[$regBase>>2]|0;
   $53 = HEAP32[$7>>2]|0;
   $54 = ($52|0)!=($53|0);
   if (!($54)) {
    $55 = HEAP32[$6>>2]|0;
    $56 = ((($55)) + 36|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0|0);
    if (!($58)) {
     break;
    }
   }
   HEAP32[$6>>2] = 0;
  }
 } while(0);
 HEAP32[$j>>2] = 0;
 while(1) {
  $59 = HEAP32[$j>>2]|0;
  $60 = HEAP32[$nCol>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$6>>2]|0;
  $63 = ($62|0)!=(0|0);
  if ($63) {
   $64 = HEAP32[$j>>2]|0;
   $65 = HEAP32[$6>>2]|0;
   $66 = ((($65)) + 4|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (($67) + ($64<<1)|0);
   $69 = HEAP16[$68>>1]|0;
   $70 = $69 << 16 >> 16;
   $71 = HEAP32[$j>>2]|0;
   $72 = HEAP32[$1>>2]|0;
   $73 = ((($72)) + 4|0);
   $74 = HEAP32[$73>>2]|0;
   $75 = (($74) + ($71<<1)|0);
   $76 = HEAP16[$75>>1]|0;
   $77 = $76 << 16 >> 16;
   $78 = ($70|0)==($77|0);
   if ($78) {
    $79 = HEAP32[$j>>2]|0;
    $80 = HEAP32[$6>>2]|0;
    $81 = ((($80)) + 4|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = (($82) + ($79<<1)|0);
    $84 = HEAP16[$83>>1]|0;
    $85 = $84 << 16 >> 16;
    $86 = ($85|0)!=(-2);
    if (!($86)) {
     label = 18;
    }
   } else {
    label = 18;
   }
  } else {
   label = 18;
  }
  if ((label|0) == 18) {
   label = 0;
   $87 = HEAP32[$0>>2]|0;
   $88 = HEAP32[$1>>2]|0;
   $89 = HEAP32[$2>>2]|0;
   $90 = HEAP32[$j>>2]|0;
   $91 = HEAP32[$regBase>>2]|0;
   $92 = HEAP32[$j>>2]|0;
   $93 = (($91) + ($92))|0;
   _sqlite3ExprCodeLoadIndexColumn($87,$88,$89,$90,$93);
   $94 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeDeletePriorOpcode($94,92)|0);
  }
  $95 = HEAP32[$j>>2]|0;
  $96 = (($95) + 1)|0;
  HEAP32[$j>>2] = $96;
 }
 $97 = HEAP32[$3>>2]|0;
 $98 = ($97|0)!=(0);
 if (!($98)) {
  $103 = HEAP32[$0>>2]|0;
  $104 = HEAP32[$regBase>>2]|0;
  $105 = HEAP32[$nCol>>2]|0;
  _sqlite3ReleaseTempRange($103,$104,$105);
  $106 = HEAP32[$regBase>>2]|0;
  STACKTOP = sp;return ($106|0);
 }
 $99 = HEAP32[$v>>2]|0;
 $100 = HEAP32[$regBase>>2]|0;
 $101 = HEAP32[$nCol>>2]|0;
 $102 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp3($99,99,$100,$101,$102)|0);
 $103 = HEAP32[$0>>2]|0;
 $104 = HEAP32[$regBase>>2]|0;
 $105 = HEAP32[$nCol>>2]|0;
 _sqlite3ReleaseTempRange($103,$104,$105);
 $106 = HEAP32[$regBase>>2]|0;
 STACKTOP = sp;return ($106|0);
}
function _sqlite3ResolvePartIdxLabel($pParse,$iLabel) {
 $pParse = $pParse|0;
 $iLabel = $iLabel|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iLabel;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3VdbeResolveLabel($6,$7);
 $8 = HEAP32[$0>>2]|0;
 _sqlite3ExprCachePop($8);
 STACKTOP = sp;return;
}
function _sqlite3UniqueConstraint($pParse,$onError,$pIdx) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $errMsg = 0, $j = 0, $pTab = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $zCol = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $zErr = sp + 56|0;
 $j = sp + 52|0;
 $errMsg = sp + 24|0;
 $pTab = sp + 20|0;
 $zCol = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pTab>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 _sqlite3StrAccumInit($errMsg,$7,0,0,200);
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  _sqlite3XPrintf($errMsg,31053,$vararg_buffer);
  $39 = (_sqlite3StrAccumFinish($errMsg)|0);
  HEAP32[$zErr>>2] = $39;
  $40 = HEAP32[$0>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 55|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43 & 3;
  $45 = $44&255;
  $46 = ($45|0)==(2);
  $47 = $46 ? 1555 : 2067;
  $48 = HEAP32[$1>>2]|0;
  $49 = HEAP32[$zErr>>2]|0;
  _sqlite3HaltConstraint($40,$47,$48,$49,-1,2);
  STACKTOP = sp;return;
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $14 = HEAP32[$j>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ((($15)) + 50|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = ($14|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$j>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + ($20<<1)|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25 << 16 >> 16;
  $27 = HEAP32[$pTab>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<4)|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$zCol>>2] = $31;
  $32 = HEAP32[$j>>2]|0;
  $33 = ($32|0)!=(0);
  if ($33) {
   _sqlite3StrAccumAppend($errMsg,31064,2);
  }
  $34 = HEAP32[$pTab>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$zCol>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $35;
  $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
  HEAP32[$vararg_ptr4>>2] = $36;
  _sqlite3XPrintf($errMsg,30161,$vararg_buffer1);
  $37 = HEAP32[$j>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$j>>2] = $38;
 }
 $39 = (_sqlite3StrAccumFinish($errMsg)|0);
 HEAP32[$zErr>>2] = $39;
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ((($41)) + 55|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = $43 & 3;
 $45 = $44&255;
 $46 = ($45|0)==(2);
 $47 = $46 ? 1555 : 2067;
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$zErr>>2]|0;
 _sqlite3HaltConstraint($40,$47,$48,$49,-1,2);
 STACKTOP = sp;return;
}
function _sqlite3XPrintf($p,$zFormat,$varargs) {
 $p = $p|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$ap>>2] = $varargs;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _sqlite3VXPrintf($2,$3,$ap);
 STACKTOP = sp;return;
}
function _sqlite3StrAccumAppend($p,$z,$N) {
 $p = $p|0;
 $z = $z|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (($5) + ($6))|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($7>>>0)>=($10>>>0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  _enlargeAndAppend($12,$13,$14);
  STACKTOP = sp;return;
 } else {
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15))|0;
  HEAP32[$17>>2] = $19;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($22) - ($23))|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 8|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + ($24)|0);
  $29 = HEAP32[$1>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  _memcpy(($28|0),($29|0),($30|0))|0;
  STACKTOP = sp;return;
 }
}
function _sqlite3HaltConstraint($pParse,$errCode,$onError,$p4,$p4type,$p5Errmsg) {
 $pParse = $pParse|0;
 $errCode = $errCode|0;
 $onError = $onError|0;
 $p4 = $p4|0;
 $p4type = $p4type|0;
 $p5Errmsg = $p5Errmsg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp + 21|0;
 $5 = sp + 20|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $errCode;
 HEAP32[$2>>2] = $onError;
 HEAP32[$3>>2] = $p4;
 HEAP8[$4>>0] = $p4type;
 HEAP8[$5>>0] = $p5Errmsg;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3GetVdbe($6)|0);
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _sqlite3MayAbort($10);
 }
 $11 = HEAP32[$v>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = HEAP8[$4>>0]|0;
 $16 = $15 << 24 >> 24;
 (_sqlite3VdbeAddOp4($11,75,$12,$13,0,$14,$16)|0);
 $17 = HEAP32[$v>>2]|0;
 $18 = HEAP8[$5>>0]|0;
 _sqlite3VdbeChangeP5($17,$18);
 STACKTOP = sp;return;
}
function _enlargeAndAppend($p,$z,$N) {
 $p = $p|0;
 $z = $z|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3StrAccumEnlarge($3,$4)|0);
 HEAP32[$2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10)|0);
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _memcpy(($14|0),($15|0),($16|0))|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17))|0;
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _sqlite3StrAccumEnlarge($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $szNew = 0, $zNew = 0, $zOld = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $zNew = sp + 12|0;
 $zOld = sp + 8|0;
 $szNew = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$1>>2]|0;
 if ($10) {
  $12 = ((($11)) + 16|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13) - ($16))|0;
  $18 = (($17) - 1)|0;
  HEAP32[$2>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  _setStrAccumError($19,2);
  $20 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $20;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $21 = ((($11)) + 25|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $23 & 4;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = $28;
 } else {
  $29 = 0;
 }
 HEAP32[$zOld>>2] = $29;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 12|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = $szNew;
 $34 = $33;
 HEAP32[$34>>2] = $32;
 $35 = (($33) + 4)|0;
 $36 = $35;
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$2>>2]|0;
 $38 = (($37) + 1)|0;
 $39 = ($38|0)<(0);
 $40 = $39 << 31 >> 31;
 $41 = $szNew;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + 4)|0;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (_i64Add(($43|0),($46|0),($38|0),($40|0))|0);
 $48 = tempRet0;
 $49 = $szNew;
 $50 = $49;
 HEAP32[$50>>2] = $47;
 $51 = (($49) + 4)|0;
 $52 = $51;
 HEAP32[$52>>2] = $48;
 $53 = $szNew;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($53) + 4)|0;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP32[$1>>2]|0;
 $60 = ((($59)) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (_i64Add(($55|0),($58|0),($61|0),0)|0);
 $63 = tempRet0;
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 20|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($63|0)<(0);
 $68 = ($62>>>0)<=($66>>>0);
 $69 = ($63|0)==(0);
 $70 = $69 & $68;
 $71 = $67 | $70;
 if ($71) {
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 12|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $szNew;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (_i64Add(($77|0),($80|0),($74|0),0)|0);
  $82 = tempRet0;
  $83 = $szNew;
  $84 = $83;
  HEAP32[$84>>2] = $81;
  $85 = (($83) + 4)|0;
  $86 = $85;
  HEAP32[$86>>2] = $82;
 }
 $87 = $szNew;
 $88 = $87;
 $89 = HEAP32[$88>>2]|0;
 $90 = (($87) + 4)|0;
 $91 = $90;
 $92 = HEAP32[$91>>2]|0;
 $93 = HEAP32[$1>>2]|0;
 $94 = ((($93)) + 20|0);
 $95 = HEAP32[$94>>2]|0;
 $96 = ($92|0)>(0);
 $97 = ($89>>>0)>($95>>>0);
 $98 = ($92|0)==(0);
 $99 = $98 & $97;
 $100 = $96 | $99;
 if ($100) {
  $101 = HEAP32[$1>>2]|0;
  _sqlite3StrAccumReset($101);
  $102 = HEAP32[$1>>2]|0;
  _setStrAccumError($102,2);
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $103 = $szNew;
 $104 = $103;
 $105 = HEAP32[$104>>2]|0;
 $106 = (($103) + 4)|0;
 $107 = $106;
 $108 = HEAP32[$107>>2]|0;
 $109 = HEAP32[$1>>2]|0;
 $110 = ((($109)) + 16|0);
 HEAP32[$110>>2] = $105;
 $111 = HEAP32[$1>>2]|0;
 $112 = HEAP32[$111>>2]|0;
 $113 = ($112|0)!=(0|0);
 if ($113) {
  $114 = HEAP32[$1>>2]|0;
  $115 = HEAP32[$114>>2]|0;
  $116 = HEAP32[$zOld>>2]|0;
  $117 = HEAP32[$1>>2]|0;
  $118 = ((($117)) + 16|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = (_sqlite3DbRealloc($115,$116,$119,0)|0);
  HEAP32[$zNew>>2] = $120;
 } else {
  $121 = HEAP32[$zOld>>2]|0;
  $122 = HEAP32[$1>>2]|0;
  $123 = ((($122)) + 16|0);
  $124 = HEAP32[$123>>2]|0;
  $125 = (_sqlite3_realloc64($121,$124,0)|0);
  HEAP32[$zNew>>2] = $125;
 }
 $126 = HEAP32[$zNew>>2]|0;
 $127 = ($126|0)!=(0|0);
 $128 = HEAP32[$1>>2]|0;
 if (!($127)) {
  _sqlite3StrAccumReset($128);
  $161 = HEAP32[$1>>2]|0;
  _setStrAccumError($161,1);
  HEAP32[$0>>2] = 0;
  $162 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $129 = ((($128)) + 25|0);
 $130 = HEAP8[$129>>0]|0;
 $131 = $130&255;
 $132 = $131 & 4;
 $133 = ($132|0)!=(0);
 if (!($133)) {
  $134 = HEAP32[$1>>2]|0;
  $135 = ((($134)) + 12|0);
  $136 = HEAP32[$135>>2]|0;
  $137 = ($136>>>0)>(0);
  if ($137) {
   $138 = HEAP32[$zNew>>2]|0;
   $139 = HEAP32[$1>>2]|0;
   $140 = ((($139)) + 8|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = HEAP32[$1>>2]|0;
   $143 = ((($142)) + 12|0);
   $144 = HEAP32[$143>>2]|0;
   _memcpy(($138|0),($141|0),($144|0))|0;
  }
 }
 $145 = HEAP32[$zNew>>2]|0;
 $146 = HEAP32[$1>>2]|0;
 $147 = ((($146)) + 8|0);
 HEAP32[$147>>2] = $145;
 $148 = HEAP32[$1>>2]|0;
 $149 = HEAP32[$148>>2]|0;
 $150 = HEAP32[$zNew>>2]|0;
 $151 = (_sqlite3DbMallocSize($149,$150)|0);
 $152 = HEAP32[$1>>2]|0;
 $153 = ((($152)) + 16|0);
 HEAP32[$153>>2] = $151;
 $154 = HEAP32[$1>>2]|0;
 $155 = ((($154)) + 25|0);
 $156 = HEAP8[$155>>0]|0;
 $157 = $156&255;
 $158 = $157 | 4;
 $159 = $158&255;
 HEAP8[$155>>0] = $159;
 $160 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $160;
 $162 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _setStrAccumError($p,$eError) {
 $p = $p|0;
 $eError = $eError|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $eError;
 $2 = HEAP8[$1>>0]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 24|0);
 HEAP8[$4>>0] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 HEAP32[$6>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3StrAccumReset($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 25|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = $4 & 4;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 8|0);
  HEAP32[$19>>2] = 0;
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3DbFree($8,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 25|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = $15 & -5;
 $17 = $16&255;
 HEAP8[$13>>0] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3ExprCachePop($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$i>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 25|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  $10 = ($5|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 152|0);
  $14 = (($13) + (($11*20)|0)|0);
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 64|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)>($19|0);
  if ($20) {
   $21 = HEAP32[$0>>2]|0;
   $22 = HEAP32[$i>>2]|0;
   _cacheEntryClear($21,$22);
   continue;
  } else {
   $23 = HEAP32[$i>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$i>>2] = $24;
   continue;
  }
 }
 STACKTOP = sp;return;
}
function _cacheEntryClear($pParse,$i) {
 $pParse = $pParse|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $i;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 152|0);
 $5 = (($4) + (($2*20)|0)|0);
 $6 = ((($5)) + 6|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 19|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)<(8);
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 152|0);
   $17 = (($16) + (($14*20)|0)|0);
   $18 = ((($17)) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 19|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = (($22) + 1)<<24>>24;
   HEAP8[$21>>0] = $23;
   $24 = $22&255;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 352|0);
   $27 = (($26) + ($24<<2)|0);
   HEAP32[$27>>2] = $19;
  }
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 25|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = (($30) + -1)<<24>>24;
 HEAP8[$29>>0] = $31;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 25|0);
 $35 = HEAP8[$34>>0]|0;
 $36 = $35&255;
 $37 = ($32|0)<($36|0);
 if (!($37)) {
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 152|0);
 $41 = (($40) + (($38*20)|0)|0);
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 25|0);
 $44 = HEAP8[$43>>0]|0;
 $45 = $44&255;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 152|0);
 $48 = (($47) + (($45*20)|0)|0);
 ;HEAP32[$41>>2]=HEAP32[$48>>2]|0;HEAP32[$41+4>>2]=HEAP32[$48+4>>2]|0;HEAP32[$41+8>>2]=HEAP32[$48+8>>2]|0;HEAP32[$41+12>>2]=HEAP32[$48+12>>2]|0;HEAP32[$41+16>>2]=HEAP32[$48+16>>2]|0;
 STACKTOP = sp;return;
}
function _sqlite3ExprCachePush($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3ExprIfFalseDup($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $db = 0, $pCopy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $db = sp + 4|0;
 $pCopy = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3ExprDup($6,$7,0)|0);
 HEAP32[$pCopy>>2] = $8;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 69|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(0);
 if (!($13)) {
  $18 = HEAP32[$db>>2]|0;
  $19 = HEAP32[$pCopy>>2]|0;
  _sqlite3ExprDelete($18,$19);
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$pCopy>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 _sqlite3ExprIfFalse($14,$15,$16,$17);
 $18 = HEAP32[$db>>2]|0;
 $19 = HEAP32[$pCopy>>2]|0;
 _sqlite3ExprDelete($18,$19);
 STACKTOP = sp;return;
}
function _sqlite3GetTempRange($pParse,$nReg) {
 $pParse = $pParse|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $nReg;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(1);
 $5 = HEAP32[$1>>2]|0;
 if ($4) {
  $6 = (_sqlite3GetTempReg($5)|0);
  HEAP32[$0>>2] = $6;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $7 = ((($5)) + 32|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$i>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 28|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$n>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$n>>2]|0;
 $14 = ($12|0)<=($13|0);
 if ($14) {
  $15 = HEAP32[$2>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15))|0;
  HEAP32[$17>>2] = $19;
  $20 = HEAP32[$2>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 28|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) - ($20))|0;
  HEAP32[$22>>2] = $24;
 } else {
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 44|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 44|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + ($29))|0;
  HEAP32[$31>>2] = $33;
 }
 $34 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3ExprCodeLoadIndexColumn($pParse,$pIdx,$iTabCur,$iIdxCol,$regOut) {
 $pParse = $pParse|0;
 $pIdx = $pIdx|0;
 $iTabCur = $iTabCur|0;
 $iIdxCol = $iIdxCol|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iTabCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $4 = sp;
 $iTabCol = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iTabCur;
 HEAP32[$3>>2] = $iIdxCol;
 HEAP32[$4>>2] = $regOut;
 $5 = HEAP32[$3>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($5<<1)|0);
 $10 = HEAP16[$9>>1]|0;
 HEAP16[$iTabCol>>1] = $10;
 $11 = HEAP16[$iTabCol>>1]|0;
 $12 = $11 << 16 >> 16;
 $13 = ($12|0)==(-2);
 if ($13) {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 60|0);
  HEAP32[$16>>2] = $14;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$3>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 40|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($18*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  _sqlite3ExprCodeCopy($17,$25,$26);
  STACKTOP = sp;return;
 } else {
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 8|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 12|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP16[$iTabCol>>1]|0;
  $35 = $34 << 16 >> 16;
  $36 = HEAP32[$4>>2]|0;
  _sqlite3ExprCodeGetColumnOfTable($29,$32,$33,$35,$36);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeDeletePriorOpcode($p,$op) {
 $p = $p|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 HEAP8[$2>>0] = $op;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 136|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 136|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) - 1)|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 88|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + (($10*20)|0)|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = HEAP8[$2>>0]|0;
  $18 = $17&255;
  $19 = ($16|0)==($18|0);
  if ($19) {
   $20 = HEAP32[$1>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 136|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) - 1)|0;
   $25 = (_sqlite3VdbeChangeToNoop($20,$24)|0);
   HEAP32[$0>>2] = $25;
   $26 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($26|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3ReleaseTempRange($pParse,$iReg,$nReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$2>>2] = $nReg;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(1);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 if ($4) {
  _sqlite3ReleaseTempReg($5,$6);
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheRemove($5,$6,$7);
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 28|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)>($11|0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 28|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 32|0);
 HEAP32[$18>>2] = $16;
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheRemove($pParse,$iReg,$nReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$2>>2] = $nReg;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 25|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 152|0);
  $12 = (($11) + (($9*20)|0)|0);
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ($15|0)>=($16|0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$2>>2]|0;
   $23 = (($21) + ($22))|0;
   $24 = ($20|0)<($23|0);
   if ($24) {
    $25 = HEAP32[$0>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    _cacheEntryClear($25,$26);
    continue;
   }
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeToNoop($p,$addr) {
 $p = $p|0;
 $addr = $addr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOp = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $addr;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 69|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 88|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + (($8*20)|0)|0);
  HEAP32[$pOp>>2] = $12;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$pOp>>2]|0;
  $16 = ((($15)) + 1|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17 << 24 >> 24;
  $19 = HEAP32[$pOp>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  _freeP4($14,$18,$21);
  $22 = HEAP32[$pOp>>2]|0;
  $23 = ((($22)) + 1|0);
  HEAP8[$23>>0] = 0;
  $24 = HEAP32[$pOp>>2]|0;
  $25 = ((($24)) + 16|0);
  HEAP32[$25>>2] = 0;
  $26 = HEAP32[$pOp>>2]|0;
  HEAP8[$26>>0] = -95;
  HEAP32[$0>>2] = 1;
  $27 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _freeP4($db,$p4type,$p4) {
 $db = $db|0;
 $p4type = $p4type|0;
 $p4 = $p4|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p4type;
 HEAP32[$2>>2] = $p4;
 $3 = HEAP32[$1>>2]|0;
 L1: do {
  switch ($3|0) {
  case -21:  {
   $4 = HEAP32[$0>>2]|0;
   $5 = HEAP32[$2>>2]|0;
   _freeP4FuncCtx($4,$5);
   break;
  }
  case -15: case -1: case -13: case -12:  {
   $6 = HEAP32[$0>>2]|0;
   $7 = HEAP32[$2>>2]|0;
   _sqlite3DbFree($6,$7);
   break;
  }
  case -6:  {
   $8 = HEAP32[$0>>2]|0;
   $9 = ((($8)) + 456|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)==(0|0);
   if ($11) {
    $12 = HEAP32[$2>>2]|0;
    _sqlite3KeyInfoUnref($12);
   }
   break;
  }
  case -11:  {
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 456|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)==(0|0);
   if ($16) {
    $17 = HEAP32[$2>>2]|0;
    _sqlite3_free($17);
   }
   break;
  }
  case -5:  {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   _freeEphemeralFunction($18,$19);
   break;
  }
  case -8:  {
   $20 = HEAP32[$0>>2]|0;
   $21 = ((($20)) + 456|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)==(0|0);
   if ($23) {
    $24 = HEAP32[$2>>2]|0;
    _sqlite3ValueFree($24);
    break L1;
   } else {
    $25 = HEAP32[$0>>2]|0;
    $26 = HEAP32[$2>>2]|0;
    _freeP4Mem($25,$26);
    break L1;
   }
   break;
  }
  case -10:  {
   $27 = HEAP32[$0>>2]|0;
   $28 = ((($27)) + 456|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0|0);
   if ($30) {
    $31 = HEAP32[$2>>2]|0;
    _sqlite3VtabUnlock($31);
   }
   break;
  }
  default: {
  }
  }
 } while(0);
 STACKTOP = sp;return;
}
function _freeP4FuncCtx($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _freeEphemeralFunction($2,$5);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($6,$7);
 STACKTOP = sp;return;
}
function _freeEphemeralFunction($db,$pDef) {
 $db = $db|0;
 $pDef = $pDef|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pDef;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 2|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($8,$9);
 STACKTOP = sp;return;
}
function _freeP4Mem($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  _sqlite3DbFree($6,$9);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($10,$11);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeCopy($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3ExprDup($5,$6,0)|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = ((($8)) + 69|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 if ($11) {
  $15 = HEAP32[$db>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  _sqlite3ExprDelete($15,$16);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($12,$13,$14);
 $15 = HEAP32[$db>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 _sqlite3ExprDelete($15,$16);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeGetColumnOfTable($v,$pTab,$iTabCur,$iCol,$regOut) {
 $v = $v|0;
 $pTab = $pTab|0;
 $iTabCur = $iTabCur|0;
 $iCol = $iCol|0;
 $regOut = $regOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $op = sp + 4|0;
 $x = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iTabCur;
 HEAP32[$3>>2] = $iCol;
 HEAP32[$4>>2] = $regOut;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)<(0);
 if ($6) {
  label = 3;
 } else {
  $7 = HEAP32[$3>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 32|0);
  $10 = HEAP16[$9>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = ($7|0)==($11|0);
  if ($12) {
   label = 3;
  } else {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 42|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $19 & 16;
   $21 = ($20|0)!=(0);
   $22 = $21 ? 156 : 96;
   HEAP32[$op>>2] = $22;
   $23 = HEAP32[$3>>2]|0;
   HEAP32[$x>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 42|0);
   $26 = HEAP8[$25>>0]|0;
   $27 = $26&255;
   $28 = $27 & 32;
   $29 = ($28|0)==(0);
   if (!($29)) {
    $30 = HEAP32[$1>>2]|0;
    $31 = ((($30)) + 42|0);
    $32 = HEAP8[$31>>0]|0;
    $33 = $32&255;
    $34 = $33 & 16;
    $35 = ($34|0)!=(0);
    if (!($35)) {
     $36 = HEAP32[$1>>2]|0;
     $37 = (_sqlite3PrimaryKeyIndex($36)|0);
     $38 = HEAP32[$3>>2]|0;
     $39 = $38&65535;
     $40 = (_sqlite3ColumnOfIndex($37,$39)|0);
     $41 = $40 << 16 >> 16;
     HEAP32[$x>>2] = $41;
    }
   }
   $42 = HEAP32[$0>>2]|0;
   $43 = HEAP32[$op>>2]|0;
   $44 = HEAP32[$2>>2]|0;
   $45 = HEAP32[$x>>2]|0;
   $46 = HEAP32[$4>>2]|0;
   (_sqlite3VdbeAddOp3($42,$43,$44,$45,$46)|0);
  }
 }
 if ((label|0) == 3) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  (_sqlite3VdbeAddOp2($13,123,$14,$15)|0);
 }
 $47 = HEAP32[$3>>2]|0;
 $48 = ($47|0)>=(0);
 if (!($48)) {
  STACKTOP = sp;return;
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$3>>2]|0;
 $52 = HEAP32[$4>>2]|0;
 _sqlite3ColumnDefault($49,$50,$51,$52);
 STACKTOP = sp;return;
}
function _sqlite3ColumnOfIndex($pIdx,$iCol) {
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 10|0;
 $1 = sp + 4|0;
 $2 = sp + 8|0;
 $i = sp;
 HEAP32[$1>>2] = $pIdx;
 HEAP16[$2>>1] = $iCol;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 52|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 6;
   break;
  }
  $9 = HEAP16[$2>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11<<1)|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = ($10|0)==($17|0);
  $19 = HEAP32[$i>>2]|0;
  if ($18) {
   label = 4;
   break;
  }
  $21 = (($19) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 if ((label|0) == 4) {
  $20 = $19&65535;
  HEAP16[$0>>1] = $20;
  $22 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($22|0);
 }
 else if ((label|0) == 6) {
  HEAP16[$0>>1] = -1;
  $22 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _sqlite3ColumnDefault($v,$pTab,$i,$iReg) {
 $v = $v|0;
 $pTab = $pTab|0;
 $i = $i|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $enc = 0, $pCol = 0, $pValue = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pValue = sp + 4|0;
 $enc = sp + 24|0;
 $pCol = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $i;
 HEAP32[$3>>2] = $iReg;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 HEAP32[$pValue>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3VdbeDb($8)|0);
 $10 = ((($9)) + 66|0);
 $11 = HEAP8[$10>>0]|0;
 HEAP8[$enc>>0] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<4)|0);
 HEAP32[$pCol>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = (_sqlite3VdbeDb($17)|0);
 $19 = HEAP32[$pCol>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP8[$enc>>0]|0;
 $23 = HEAP32[$pCol>>2]|0;
 $24 = ((($23)) + 13|0);
 $25 = HEAP8[$24>>0]|0;
 (_sqlite3ValueFromExpr($18,$21,$22,$25,$pValue)|0);
 $26 = HEAP32[$pValue>>2]|0;
 $27 = ($26|0)!=(0|0);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  $29 = HEAP32[$pValue>>2]|0;
  _sqlite3VdbeChangeP4($28,-1,$29,-8);
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 4|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = (($33) + ($30<<4)|0);
 $35 = ((($34)) + 13|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36 << 24 >> 24;
 $38 = ($37|0)==(69);
 if (!($38)) {
  STACKTOP = sp;return;
 }
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp1($39,92,$40)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeDb($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ExprCode($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $inReg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $inReg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)==(157);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 8|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 28|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($11,84,$14,$15)|0);
   STACKTOP = sp;return;
  }
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$2>>2]|0;
 $19 = (_sqlite3ExprCodeTarget($16,$17,$18)|0);
 HEAP32[$inReg>>2] = $19;
 $20 = HEAP32[$inReg>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = ($20|0)!=($21|0);
 if (!($22)) {
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  STACKTOP = sp;return;
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$inReg>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($29,85,$30,$31)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeTarget($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $aListelem = 0, $addr = 0, $constMask = 0, $db = 0, $destIfFalse = 0, $destIfNull = 0, $enc = 0, $endCoalesce = 0, $endLabel = 0, $exprOp = 0, $i = 0, $i2 = 0, $iTab = 0, $inReg = 0, $n = 0, $nCol = 0, $nExpr = 0;
 var $nFarg = 0, $nextCase = 0, $op = 0, $opCompare = 0, $or$cond = 0, $or$cond3 = 0, $p1 = 0, $p5 = 0, $pAggInfo = 0, $pCol = 0, $pColl = 0, $pDef = 0, $pEList = 0, $pFarg = 0, $pInfo = 0, $pLeft = 0, $pLeft1 = 0, $pTab = 0, $pTest = 0, $pX = 0;
 var $r1 = 0, $r2 = 0, $regFree1 = 0, $regFree2 = 0, $tempX = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_buffer9 = 0, $z = 0, $zBlob = 0, $zId = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 320|0;
 $vararg_buffer9 = sp + 24|0;
 $vararg_buffer7 = sp + 16|0;
 $vararg_buffer4 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 304|0;
 $1 = sp + 300|0;
 $2 = sp + 296|0;
 $3 = sp + 292|0;
 $v = sp + 288|0;
 $op = sp + 284|0;
 $inReg = sp + 280|0;
 $regFree1 = sp + 276|0;
 $regFree2 = sp + 272|0;
 $r1 = sp + 268|0;
 $r2 = sp + 264|0;
 $tempX = sp + 216|0;
 $p5 = sp + 208|0;
 $pAggInfo = sp + 204|0;
 $pCol = sp + 200|0;
 $iTab = sp + 196|0;
 $n = sp + 192|0;
 $z = sp + 188|0;
 $zBlob = sp + 184|0;
 $pLeft = sp + 180|0;
 $pLeft1 = sp + 176|0;
 $addr = sp + 172|0;
 $pInfo = sp + 168|0;
 $pFarg = sp + 164|0;
 $nFarg = sp + 160|0;
 $pDef = sp + 156|0;
 $zId = sp + 152|0;
 $constMask = sp + 148|0;
 $i = sp + 144|0;
 $db = sp + 140|0;
 $enc = sp + 309|0;
 $pColl = sp + 136|0;
 $endCoalesce = sp + 132|0;
 $exprOp = sp + 308|0;
 $nCol = sp + 128|0;
 $destIfFalse = sp + 124|0;
 $destIfNull = sp + 120|0;
 $pTab = sp + 116|0;
 $p1 = sp + 112|0;
 $endLabel = sp + 108|0;
 $nextCase = sp + 104|0;
 $nExpr = sp + 100|0;
 $i2 = sp + 96|0;
 $pEList = sp + 92|0;
 $aListelem = sp + 88|0;
 $opCompare = sp + 40|0;
 $pX = sp + 32|0;
 $pTest = sp + 28|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $target;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 HEAP32[$inReg>>2] = $7;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 HEAP32[$p5>>2] = 0;
 $8 = HEAP32[$v>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $722 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($722|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  HEAP32[$op>>2] = 101;
 } else {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  HEAP32[$op>>2] = $14;
 }
 $15 = HEAP32[$op>>2]|0;
 L9: do {
  switch ($15|0) {
  case 154:  {
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 40|0);
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$pAggInfo>>2] = $18;
   $19 = HEAP32[$2>>2]|0;
   $20 = ((($19)) + 34|0);
   $21 = HEAP16[$20>>1]|0;
   $22 = $21 << 16 >> 16;
   $23 = HEAP32[$pAggInfo>>2]|0;
   $24 = ((($23)) + 28|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (($25) + (($22*24)|0)|0);
   HEAP32[$pCol>>2] = $26;
   $27 = HEAP32[$pAggInfo>>2]|0;
   $28 = HEAP8[$27>>0]|0;
   $29 = ($28<<24>>24)!=(0);
   if (!($29)) {
    $30 = HEAP32[$pCol>>2]|0;
    $31 = ((($30)) + 16|0);
    $32 = HEAP32[$31>>2]|0;
    HEAP32[$0>>2] = $32;
    $722 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($722|0);
   }
   $33 = HEAP32[$pAggInfo>>2]|0;
   $34 = ((($33)) + 1|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = ($35<<24>>24)!=(0);
   if ($36) {
    $37 = HEAP32[$v>>2]|0;
    $38 = HEAP32[$pAggInfo>>2]|0;
    $39 = ((($38)) + 8|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$pCol>>2]|0;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp3($37,96,$40,$43,$44)|0);
    $45 = HEAP32[$3>>2]|0;
    HEAP32[$0>>2] = $45;
    $722 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($722|0);
   } else {
    label = 11;
   }
   break;
  }
  case 152:  {
   label = 11;
   break;
  }
  case 134:  {
   $80 = HEAP32[$1>>2]|0;
   $81 = HEAP32[$2>>2]|0;
   $82 = HEAP32[$3>>2]|0;
   _codeInteger($80,$81,0,$82);
   $83 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $83;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 132:  {
   $84 = HEAP32[$v>>2]|0;
   $85 = HEAP32[$2>>2]|0;
   $86 = ((($85)) + 8|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = HEAP32[$3>>2]|0;
   _codeReal($84,$87,0,$88);
   $89 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $89;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 97:  {
   $90 = HEAP32[$v>>2]|0;
   $91 = HEAP32[$3>>2]|0;
   $92 = HEAP32[$2>>2]|0;
   $93 = ((($92)) + 8|0);
   $94 = HEAP32[$93>>2]|0;
   (_sqlite3VdbeLoadString($90,$91,$94)|0);
   $95 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $95;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 101:  {
   $96 = HEAP32[$v>>2]|0;
   $97 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($96,79,0,$97)|0);
   $98 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $98;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 133:  {
   $99 = HEAP32[$2>>2]|0;
   $100 = ((($99)) + 8|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = ((($101)) + 2|0);
   HEAP32[$z>>2] = $102;
   $103 = HEAP32[$z>>2]|0;
   $104 = (_sqlite3Strlen30($103)|0);
   $105 = (($104) - 1)|0;
   HEAP32[$n>>2] = $105;
   $106 = HEAP32[$v>>2]|0;
   $107 = (_sqlite3VdbeDb($106)|0);
   $108 = HEAP32[$z>>2]|0;
   $109 = HEAP32[$n>>2]|0;
   $110 = (_sqlite3HexToBlob($107,$108,$109)|0);
   HEAP32[$zBlob>>2] = $110;
   $111 = HEAP32[$v>>2]|0;
   $112 = HEAP32[$n>>2]|0;
   $113 = (($112|0) / 2)&-1;
   $114 = HEAP32[$3>>2]|0;
   $115 = HEAP32[$zBlob>>2]|0;
   (_sqlite3VdbeAddOp4($111,81,$113,$114,0,$115,-1)|0);
   $116 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $116;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 135:  {
   $117 = HEAP32[$v>>2]|0;
   $118 = HEAP32[$2>>2]|0;
   $119 = ((($118)) + 32|0);
   $120 = HEAP16[$119>>1]|0;
   $121 = $120 << 16 >> 16;
   $122 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($117,82,$121,$122)|0);
   $123 = HEAP32[$2>>2]|0;
   $124 = ((($123)) + 8|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = ((($125)) + 1|0);
   $127 = HEAP8[$126>>0]|0;
   $128 = $127 << 24 >> 24;
   $129 = ($128|0)!=(0);
   if ($129) {
    $130 = HEAP32[$v>>2]|0;
    $131 = HEAP32[$2>>2]|0;
    $132 = ((($131)) + 32|0);
    $133 = HEAP16[$132>>1]|0;
    $134 = $133 << 16 >> 16;
    $135 = (($134) - 1)|0;
    $136 = HEAP32[$1>>2]|0;
    $137 = ((($136)) + 428|0);
    $138 = HEAP32[$137>>2]|0;
    $139 = (($138) + ($135<<2)|0);
    $140 = HEAP32[$139>>2]|0;
    _sqlite3VdbeChangeP4($130,-1,$140,-2);
   }
   $141 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $141;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 157:  {
   $142 = HEAP32[$2>>2]|0;
   $143 = ((($142)) + 28|0);
   $144 = HEAP32[$143>>2]|0;
   HEAP32[$0>>2] = $144;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 66:  {
   $145 = HEAP32[$1>>2]|0;
   $146 = HEAP32[$2>>2]|0;
   $147 = ((($146)) + 12|0);
   $148 = HEAP32[$147>>2]|0;
   $149 = HEAP32[$3>>2]|0;
   $150 = (_sqlite3ExprCodeTarget($145,$148,$149)|0);
   HEAP32[$inReg>>2] = $150;
   $151 = HEAP32[$inReg>>2]|0;
   $152 = HEAP32[$3>>2]|0;
   $153 = ($151|0)!=($152|0);
   if ($153) {
    $154 = HEAP32[$v>>2]|0;
    $155 = HEAP32[$inReg>>2]|0;
    $156 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($154,85,$155,$156)|0);
    $157 = HEAP32[$3>>2]|0;
    HEAP32[$inReg>>2] = $157;
   }
   $158 = HEAP32[$v>>2]|0;
   $159 = HEAP32[$3>>2]|0;
   $160 = HEAP32[$2>>2]|0;
   $161 = ((($160)) + 8|0);
   $162 = HEAP32[$161>>2]|0;
   $163 = (_sqlite3AffinityType($162,0)|0);
   $164 = $163 << 24 >> 24;
   (_sqlite3VdbeAddOp2($158,93,$159,$164)|0);
   $165 = HEAP32[$1>>2]|0;
   $166 = HEAP32[$inReg>>2]|0;
   _sqlite3ExprCacheAffinityChange($165,$166,1);
   $167 = HEAP32[$inReg>>2]|0;
   HEAP32[$0>>2] = $167;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 148: case 29:  {
   $168 = HEAP32[$op>>2]|0;
   $169 = ($168|0)==(29);
   $170 = $169 ? 37 : 36;
   HEAP32[$op>>2] = $170;
   HEAP32[$p5>>2] = 128;
   label = 29;
   break;
  }
  case 37: case 36: case 41: case 38: case 39: case 40:  {
   label = 29;
   break;
  }
  case 52: case 46: case 45: case 50: case 44: case 43: case 51: case 48: case 49: case 47: case 27: case 28:  {
   $202 = HEAP32[$1>>2]|0;
   $203 = HEAP32[$2>>2]|0;
   $204 = ((($203)) + 12|0);
   $205 = HEAP32[$204>>2]|0;
   $206 = (_sqlite3ExprCodeTemp($202,$205,$regFree1)|0);
   HEAP32[$r1>>2] = $206;
   $207 = HEAP32[$1>>2]|0;
   $208 = HEAP32[$2>>2]|0;
   $209 = ((($208)) + 16|0);
   $210 = HEAP32[$209>>2]|0;
   $211 = (_sqlite3ExprCodeTemp($207,$210,$regFree2)|0);
   HEAP32[$r2>>2] = $211;
   $212 = HEAP32[$v>>2]|0;
   $213 = HEAP32[$op>>2]|0;
   $214 = HEAP32[$r2>>2]|0;
   $215 = HEAP32[$r1>>2]|0;
   $216 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp3($212,$213,$214,$215,$216)|0);
   break;
  }
  case 155:  {
   $217 = HEAP32[$2>>2]|0;
   $218 = ((($217)) + 12|0);
   $219 = HEAP32[$218>>2]|0;
   HEAP32[$pLeft1>>2] = $219;
   $220 = HEAP32[$pLeft1>>2]|0;
   $221 = HEAP8[$220>>0]|0;
   $222 = $221&255;
   $223 = ($222|0)==(134);
   if ($223) {
    $224 = HEAP32[$1>>2]|0;
    $225 = HEAP32[$pLeft1>>2]|0;
    $226 = HEAP32[$3>>2]|0;
    _codeInteger($224,$225,1,$226);
    $227 = HEAP32[$3>>2]|0;
    HEAP32[$0>>2] = $227;
    $722 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($722|0);
   }
   $228 = HEAP32[$pLeft1>>2]|0;
   $229 = HEAP8[$228>>0]|0;
   $230 = $229&255;
   $231 = ($230|0)==(132);
   if (!($231)) {
    HEAP8[$tempX>>0] = -122;
    $238 = ((($tempX)) + 4|0);
    HEAP32[$238>>2] = 17408;
    $239 = ((($tempX)) + 8|0);
    HEAP32[$239>>2] = 0;
    $240 = HEAP32[$1>>2]|0;
    $241 = (_sqlite3ExprCodeTemp($240,$tempX,$regFree1)|0);
    HEAP32[$r1>>2] = $241;
    $242 = HEAP32[$1>>2]|0;
    $243 = HEAP32[$2>>2]|0;
    $244 = ((($243)) + 12|0);
    $245 = HEAP32[$244>>2]|0;
    $246 = (_sqlite3ExprCodeTemp($242,$245,$regFree2)|0);
    HEAP32[$r2>>2] = $246;
    $247 = HEAP32[$v>>2]|0;
    $248 = HEAP32[$r2>>2]|0;
    $249 = HEAP32[$r1>>2]|0;
    $250 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp3($247,48,$248,$249,$250)|0);
    break L9;
   }
   $232 = HEAP32[$v>>2]|0;
   $233 = HEAP32[$pLeft1>>2]|0;
   $234 = ((($233)) + 8|0);
   $235 = HEAP32[$234>>2]|0;
   $236 = HEAP32[$3>>2]|0;
   _codeReal($232,$235,1,$236);
   $237 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $237;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 19: case 54:  {
   $251 = HEAP32[$1>>2]|0;
   $252 = HEAP32[$2>>2]|0;
   $253 = ((($252)) + 12|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = (_sqlite3ExprCodeTemp($251,$254,$regFree1)|0);
   HEAP32[$r1>>2] = $255;
   $256 = HEAP32[$v>>2]|0;
   $257 = HEAP32[$op>>2]|0;
   $258 = HEAP32[$r1>>2]|0;
   $259 = HEAP32[$inReg>>2]|0;
   (_sqlite3VdbeAddOp2($256,$257,$258,$259)|0);
   break;
  }
  case 35: case 34:  {
   $260 = HEAP32[$v>>2]|0;
   $261 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($260,76,1,$261)|0);
   $262 = HEAP32[$1>>2]|0;
   $263 = HEAP32[$2>>2]|0;
   $264 = ((($263)) + 12|0);
   $265 = HEAP32[$264>>2]|0;
   $266 = (_sqlite3ExprCodeTemp($262,$265,$regFree1)|0);
   HEAP32[$r1>>2] = $266;
   $267 = HEAP32[$v>>2]|0;
   $268 = HEAP32[$op>>2]|0;
   $269 = HEAP32[$r1>>2]|0;
   $270 = (_sqlite3VdbeAddOp1($267,$268,$269)|0);
   HEAP32[$addr>>2] = $270;
   $271 = HEAP32[$v>>2]|0;
   $272 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($271,76,0,$272)|0);
   $273 = HEAP32[$v>>2]|0;
   $274 = HEAP32[$addr>>2]|0;
   _sqlite3VdbeJumpHere($273,$274);
   break;
  }
  case 153:  {
   $275 = HEAP32[$2>>2]|0;
   $276 = ((($275)) + 40|0);
   $277 = HEAP32[$276>>2]|0;
   HEAP32[$pInfo>>2] = $277;
   $278 = HEAP32[$pInfo>>2]|0;
   $279 = ($278|0)==(0|0);
   if ($279) {
    $280 = HEAP32[$1>>2]|0;
    $281 = HEAP32[$2>>2]|0;
    $282 = ((($281)) + 8|0);
    $283 = HEAP32[$282>>2]|0;
    HEAP32[$vararg_buffer>>2] = $283;
    _sqlite3ErrorMsg($280,31067,$vararg_buffer);
    break L9;
   }
   $284 = HEAP32[$2>>2]|0;
   $285 = ((($284)) + 34|0);
   $286 = HEAP16[$285>>1]|0;
   $287 = $286 << 16 >> 16;
   $288 = HEAP32[$pInfo>>2]|0;
   $289 = ((($288)) + 40|0);
   $290 = HEAP32[$289>>2]|0;
   $291 = (($290) + ($287<<4)|0);
   $292 = ((($291)) + 8|0);
   $293 = HEAP32[$292>>2]|0;
   HEAP32[$0>>2] = $293;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 151:  {
   HEAP32[$constMask>>2] = 0;
   $294 = HEAP32[$1>>2]|0;
   $295 = HEAP32[$294>>2]|0;
   HEAP32[$db>>2] = $295;
   $296 = HEAP32[$db>>2]|0;
   $297 = ((($296)) + 66|0);
   $298 = HEAP8[$297>>0]|0;
   HEAP8[$enc>>0] = $298;
   HEAP32[$pColl>>2] = 0;
   $299 = HEAP32[$2>>2]|0;
   $300 = ((($299)) + 4|0);
   $301 = HEAP32[$300>>2]|0;
   $302 = $301 & 16384;
   $303 = ($302|0)!=(0);
   if ($303) {
    HEAP32[$pFarg>>2] = 0;
   } else {
    $304 = HEAP32[$2>>2]|0;
    $305 = ((($304)) + 20|0);
    $306 = HEAP32[$305>>2]|0;
    HEAP32[$pFarg>>2] = $306;
   }
   $307 = HEAP32[$pFarg>>2]|0;
   $308 = ($307|0)!=(0|0);
   if ($308) {
    $309 = HEAP32[$pFarg>>2]|0;
    $310 = HEAP32[$309>>2]|0;
    $311 = $310;
   } else {
    $311 = 0;
   }
   HEAP32[$nFarg>>2] = $311;
   $312 = HEAP32[$2>>2]|0;
   $313 = ((($312)) + 8|0);
   $314 = HEAP32[$313>>2]|0;
   HEAP32[$zId>>2] = $314;
   $315 = HEAP32[$db>>2]|0;
   $316 = HEAP32[$zId>>2]|0;
   $317 = HEAP32[$nFarg>>2]|0;
   $318 = HEAP8[$enc>>0]|0;
   $319 = (_sqlite3FindFunction($315,$316,$317,$318,0)|0);
   HEAP32[$pDef>>2] = $319;
   $320 = HEAP32[$pDef>>2]|0;
   $321 = ($320|0)==(0|0);
   if (!($321)) {
    $322 = HEAP32[$pDef>>2]|0;
    $323 = ((($322)) + 16|0);
    $324 = HEAP32[$323>>2]|0;
    $325 = ($324|0)!=(0|0);
    if (!($325)) {
     $328 = HEAP32[$pDef>>2]|0;
     $329 = ((($328)) + 2|0);
     $330 = HEAP16[$329>>1]|0;
     $331 = $330&65535;
     $332 = $331 & 512;
     $333 = ($332|0)!=(0);
     if ($333) {
      $334 = HEAP32[$v>>2]|0;
      $335 = (_sqlite3VdbeMakeLabel($334)|0);
      HEAP32[$endCoalesce>>2] = $335;
      $336 = HEAP32[$1>>2]|0;
      $337 = HEAP32[$pFarg>>2]|0;
      $338 = ((($337)) + 4|0);
      $339 = HEAP32[$338>>2]|0;
      $340 = HEAP32[$339>>2]|0;
      $341 = HEAP32[$3>>2]|0;
      _sqlite3ExprCode($336,$340,$341);
      HEAP32[$i>>2] = 1;
      while(1) {
       $342 = HEAP32[$i>>2]|0;
       $343 = HEAP32[$nFarg>>2]|0;
       $344 = ($342|0)<($343|0);
       $345 = HEAP32[$v>>2]|0;
       if (!($344)) {
        break;
       }
       $346 = HEAP32[$3>>2]|0;
       $347 = HEAP32[$endCoalesce>>2]|0;
       (_sqlite3VdbeAddOp2($345,35,$346,$347)|0);
       $348 = HEAP32[$1>>2]|0;
       $349 = HEAP32[$3>>2]|0;
       _sqlite3ExprCacheRemove($348,$349,1);
       $350 = HEAP32[$1>>2]|0;
       _sqlite3ExprCachePush($350);
       $351 = HEAP32[$1>>2]|0;
       $352 = HEAP32[$i>>2]|0;
       $353 = HEAP32[$pFarg>>2]|0;
       $354 = ((($353)) + 4|0);
       $355 = HEAP32[$354>>2]|0;
       $356 = (($355) + (($352*20)|0)|0);
       $357 = HEAP32[$356>>2]|0;
       $358 = HEAP32[$3>>2]|0;
       _sqlite3ExprCode($351,$357,$358);
       $359 = HEAP32[$1>>2]|0;
       _sqlite3ExprCachePop($359);
       $360 = HEAP32[$i>>2]|0;
       $361 = (($360) + 1)|0;
       HEAP32[$i>>2] = $361;
      }
      $362 = HEAP32[$endCoalesce>>2]|0;
      _sqlite3VdbeResolveLabel($345,$362);
      break L9;
     }
     $363 = HEAP32[$pDef>>2]|0;
     $364 = ((($363)) + 2|0);
     $365 = HEAP16[$364>>1]|0;
     $366 = $365&65535;
     $367 = $366 & 1024;
     $368 = ($367|0)!=(0);
     if ($368) {
      $369 = HEAP32[$1>>2]|0;
      $370 = HEAP32[$pFarg>>2]|0;
      $371 = ((($370)) + 4|0);
      $372 = HEAP32[$371>>2]|0;
      $373 = HEAP32[$372>>2]|0;
      $374 = HEAP32[$3>>2]|0;
      $375 = (_sqlite3ExprCodeTarget($369,$373,$374)|0);
      HEAP32[$0>>2] = $375;
      $722 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($722|0);
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $376 = HEAP32[$i>>2]|0;
      $377 = HEAP32[$nFarg>>2]|0;
      $378 = ($376|0)<($377|0);
      if (!($378)) {
       break;
      }
      $379 = HEAP32[$i>>2]|0;
      $380 = ($379|0)<(32);
      if ($380) {
       $381 = HEAP32[$i>>2]|0;
       $382 = HEAP32[$pFarg>>2]|0;
       $383 = ((($382)) + 4|0);
       $384 = HEAP32[$383>>2]|0;
       $385 = (($384) + (($381*20)|0)|0);
       $386 = HEAP32[$385>>2]|0;
       $387 = (_sqlite3ExprIsConstant($386)|0);
       $388 = ($387|0)!=(0);
       if ($388) {
        $389 = HEAP32[$i>>2]|0;
        $390 = 1 << $389;
        $391 = HEAP32[$constMask>>2]|0;
        $392 = $391 | $390;
        HEAP32[$constMask>>2] = $392;
       }
      }
      $393 = HEAP32[$pDef>>2]|0;
      $394 = ((($393)) + 2|0);
      $395 = HEAP16[$394>>1]|0;
      $396 = $395&65535;
      $397 = $396 & 32;
      $398 = ($397|0)==(0);
      $399 = HEAP32[$pColl>>2]|0;
      $400 = ($399|0)!=(0|0);
      $or$cond = $398 | $400;
      if (!($or$cond)) {
       $401 = HEAP32[$1>>2]|0;
       $402 = HEAP32[$i>>2]|0;
       $403 = HEAP32[$pFarg>>2]|0;
       $404 = ((($403)) + 4|0);
       $405 = HEAP32[$404>>2]|0;
       $406 = (($405) + (($402*20)|0)|0);
       $407 = HEAP32[$406>>2]|0;
       $408 = (_sqlite3ExprCollSeq($401,$407)|0);
       HEAP32[$pColl>>2] = $408;
      }
      $409 = HEAP32[$i>>2]|0;
      $410 = (($409) + 1)|0;
      HEAP32[$i>>2] = $410;
     }
     $411 = HEAP32[$pFarg>>2]|0;
     $412 = ($411|0)!=(0|0);
     if ($412) {
      $413 = HEAP32[$constMask>>2]|0;
      $414 = ($413|0)!=(0);
      $415 = HEAP32[$1>>2]|0;
      if ($414) {
       $416 = ((($415)) + 44|0);
       $417 = HEAP32[$416>>2]|0;
       $418 = (($417) + 1)|0;
       HEAP32[$r1>>2] = $418;
       $419 = HEAP32[$nFarg>>2]|0;
       $420 = HEAP32[$1>>2]|0;
       $421 = ((($420)) + 44|0);
       $422 = HEAP32[$421>>2]|0;
       $423 = (($422) + ($419))|0;
       HEAP32[$421>>2] = $423;
      } else {
       $424 = HEAP32[$nFarg>>2]|0;
       $425 = (_sqlite3GetTempRange($415,$424)|0);
       HEAP32[$r1>>2] = $425;
      }
      $426 = HEAP32[$pDef>>2]|0;
      $427 = ((($426)) + 2|0);
      $428 = HEAP16[$427>>1]|0;
      $429 = $428&65535;
      $430 = $429 & 192;
      $431 = ($430|0)!=(0);
      do {
       if ($431) {
        $432 = HEAP32[$pFarg>>2]|0;
        $433 = ((($432)) + 4|0);
        $434 = HEAP32[$433>>2]|0;
        $435 = HEAP32[$434>>2]|0;
        $436 = HEAP8[$435>>0]|0;
        HEAP8[$exprOp>>0] = $436;
        $437 = HEAP8[$exprOp>>0]|0;
        $438 = $437&255;
        $439 = ($438|0)==(152);
        if (!($439)) {
         $440 = HEAP8[$exprOp>>0]|0;
         $441 = $440&255;
         $442 = ($441|0)==(154);
         if (!($442)) {
          break;
         }
        }
        $443 = HEAP32[$pDef>>2]|0;
        $444 = ((($443)) + 2|0);
        $445 = HEAP16[$444>>1]|0;
        $446 = $445&65535;
        $447 = $446 & 192;
        $448 = $447&255;
        $449 = HEAP32[$pFarg>>2]|0;
        $450 = ((($449)) + 4|0);
        $451 = HEAP32[$450>>2]|0;
        $452 = HEAP32[$451>>2]|0;
        $453 = ((($452)) + 38|0);
        HEAP8[$453>>0] = $448;
       }
      } while(0);
      $454 = HEAP32[$1>>2]|0;
      _sqlite3ExprCachePush($454);
      $455 = HEAP32[$1>>2]|0;
      $456 = HEAP32[$pFarg>>2]|0;
      $457 = HEAP32[$r1>>2]|0;
      (_sqlite3ExprCodeExprList($455,$456,$457,0,3)|0);
      $458 = HEAP32[$1>>2]|0;
      _sqlite3ExprCachePop($458);
     } else {
      HEAP32[$r1>>2] = 0;
     }
     $459 = HEAP32[$nFarg>>2]|0;
     $460 = ($459|0)>=(2);
     if ($460) {
      $461 = HEAP32[$2>>2]|0;
      $462 = ((($461)) + 4|0);
      $463 = HEAP32[$462>>2]|0;
      $464 = $463 & 128;
      $465 = ($464|0)!=(0);
      if ($465) {
       $466 = HEAP32[$db>>2]|0;
       $467 = HEAP32[$pDef>>2]|0;
       $468 = HEAP32[$nFarg>>2]|0;
       $469 = HEAP32[$pFarg>>2]|0;
       $470 = ((($469)) + 4|0);
       $471 = HEAP32[$470>>2]|0;
       $472 = ((($471)) + 20|0);
       $473 = HEAP32[$472>>2]|0;
       $474 = (_sqlite3VtabOverloadFunction($466,$467,$468,$473)|0);
       HEAP32[$pDef>>2] = $474;
      } else {
       label = 79;
      }
     } else {
      label = 79;
     }
     if ((label|0) == 79) {
      $475 = HEAP32[$nFarg>>2]|0;
      $476 = ($475|0)>(0);
      if ($476) {
       $477 = HEAP32[$db>>2]|0;
       $478 = HEAP32[$pDef>>2]|0;
       $479 = HEAP32[$nFarg>>2]|0;
       $480 = HEAP32[$pFarg>>2]|0;
       $481 = ((($480)) + 4|0);
       $482 = HEAP32[$481>>2]|0;
       $483 = HEAP32[$482>>2]|0;
       $484 = (_sqlite3VtabOverloadFunction($477,$478,$479,$483)|0);
       HEAP32[$pDef>>2] = $484;
      }
     }
     $485 = HEAP32[$pDef>>2]|0;
     $486 = ((($485)) + 2|0);
     $487 = HEAP16[$486>>1]|0;
     $488 = $487&65535;
     $489 = $488 & 32;
     $490 = ($489|0)!=(0);
     if ($490) {
      $491 = HEAP32[$pColl>>2]|0;
      $492 = ($491|0)!=(0|0);
      if (!($492)) {
       $493 = HEAP32[$db>>2]|0;
       $494 = ((($493)) + 8|0);
       $495 = HEAP32[$494>>2]|0;
       HEAP32[$pColl>>2] = $495;
      }
      $496 = HEAP32[$v>>2]|0;
      $497 = HEAP32[$pColl>>2]|0;
      (_sqlite3VdbeAddOp4($496,88,0,0,0,$497,-4)|0);
     }
     $498 = HEAP32[$v>>2]|0;
     $499 = HEAP32[$constMask>>2]|0;
     $500 = HEAP32[$r1>>2]|0;
     $501 = HEAP32[$3>>2]|0;
     $502 = HEAP32[$pDef>>2]|0;
     (_sqlite3VdbeAddOp4($498,89,$499,$500,$501,$502,-5)|0);
     $503 = HEAP32[$v>>2]|0;
     $504 = HEAP32[$nFarg>>2]|0;
     $505 = $504&255;
     _sqlite3VdbeChangeP5($503,$505);
     $506 = HEAP32[$nFarg>>2]|0;
     $507 = ($506|0)!=(0);
     $508 = HEAP32[$constMask>>2]|0;
     $509 = ($508|0)==(0);
     $or$cond3 = $507 & $509;
     if ($or$cond3) {
      $510 = HEAP32[$1>>2]|0;
      $511 = HEAP32[$r1>>2]|0;
      $512 = HEAP32[$nFarg>>2]|0;
      _sqlite3ReleaseTempRange($510,$511,$512);
     }
     $513 = HEAP32[$3>>2]|0;
     HEAP32[$0>>2] = $513;
     $722 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($722|0);
    }
   }
   $326 = HEAP32[$1>>2]|0;
   $327 = HEAP32[$zId>>2]|0;
   HEAP32[$vararg_buffer4>>2] = $327;
   _sqlite3ErrorMsg($326,31093,$vararg_buffer4);
   break;
  }
  case 119: case 20:  {
   $514 = HEAP32[$op>>2]|0;
   $515 = ($514|0)==(119);
   if ($515) {
    $516 = HEAP32[$2>>2]|0;
    $517 = ((($516)) + 20|0);
    $518 = HEAP32[$517>>2]|0;
    $519 = HEAP32[$518>>2]|0;
    $520 = HEAP32[$519>>2]|0;
    HEAP32[$nCol>>2] = $520;
    $521 = ($520|0)!=(1);
    if ($521) {
     $522 = HEAP32[$1>>2]|0;
     $523 = HEAP32[$nCol>>2]|0;
     _sqlite3SubselectError($522,$523,1);
     break L9;
    }
   }
   $524 = HEAP32[$1>>2]|0;
   $525 = HEAP32[$2>>2]|0;
   $526 = (_sqlite3CodeSubselect($524,$525,0,0)|0);
   HEAP32[$0>>2] = $526;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 159:  {
   $527 = HEAP32[$2>>2]|0;
   $528 = ((($527)) + 12|0);
   $529 = HEAP32[$528>>2]|0;
   $530 = ((($529)) + 28|0);
   $531 = HEAP32[$530>>2]|0;
   $532 = ($531|0)==(0);
   if ($532) {
    $533 = HEAP32[$1>>2]|0;
    $534 = HEAP32[$2>>2]|0;
    $535 = ((($534)) + 12|0);
    $536 = HEAP32[$535>>2]|0;
    $537 = (_sqlite3CodeSubselect($533,$536,0,0)|0);
    $538 = HEAP32[$2>>2]|0;
    $539 = ((($538)) + 12|0);
    $540 = HEAP32[$539>>2]|0;
    $541 = ((($540)) + 28|0);
    HEAP32[$541>>2] = $537;
   }
   $542 = HEAP32[$2>>2]|0;
   $543 = ((($542)) + 12|0);
   $544 = HEAP32[$543>>2]|0;
   $545 = ((($544)) + 28|0);
   $546 = HEAP32[$545>>2]|0;
   $547 = HEAP32[$2>>2]|0;
   $548 = ((($547)) + 32|0);
   $549 = HEAP16[$548>>1]|0;
   $550 = $549 << 16 >> 16;
   $551 = (($546) + ($550))|0;
   HEAP32[$0>>2] = $551;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 33:  {
   $552 = HEAP32[$v>>2]|0;
   $553 = (_sqlite3VdbeMakeLabel($552)|0);
   HEAP32[$destIfFalse>>2] = $553;
   $554 = HEAP32[$v>>2]|0;
   $555 = (_sqlite3VdbeMakeLabel($554)|0);
   HEAP32[$destIfNull>>2] = $555;
   $556 = HEAP32[$v>>2]|0;
   $557 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($556,79,0,$557)|0);
   $558 = HEAP32[$1>>2]|0;
   $559 = HEAP32[$2>>2]|0;
   $560 = HEAP32[$destIfFalse>>2]|0;
   $561 = HEAP32[$destIfNull>>2]|0;
   _sqlite3ExprCodeIN($558,$559,$560,$561);
   $562 = HEAP32[$v>>2]|0;
   $563 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($562,76,1,$563)|0);
   $564 = HEAP32[$v>>2]|0;
   $565 = HEAP32[$destIfFalse>>2]|0;
   _sqlite3VdbeResolveLabel($564,$565);
   $566 = HEAP32[$v>>2]|0;
   $567 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($566,91,$567,0)|0);
   $568 = HEAP32[$v>>2]|0;
   $569 = HEAP32[$destIfNull>>2]|0;
   _sqlite3VdbeResolveLabel($568,$569);
   $570 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $570;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 32:  {
   $571 = HEAP32[$1>>2]|0;
   $572 = HEAP32[$2>>2]|0;
   $573 = HEAP32[$3>>2]|0;
   _exprCodeBetween($571,$572,$573,0,0);
   $574 = HEAP32[$3>>2]|0;
   HEAP32[$0>>2] = $574;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 156: case 53: case 161:  {
   $575 = HEAP32[$1>>2]|0;
   $576 = HEAP32[$2>>2]|0;
   $577 = ((($576)) + 12|0);
   $578 = HEAP32[$577>>2]|0;
   $579 = HEAP32[$3>>2]|0;
   $580 = (_sqlite3ExprCodeTarget($575,$578,$579)|0);
   HEAP32[$0>>2] = $580;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
   break;
  }
  case 88:  {
   $581 = HEAP32[$2>>2]|0;
   $582 = ((($581)) + 44|0);
   $583 = HEAP32[$582>>2]|0;
   HEAP32[$pTab>>2] = $583;
   $584 = HEAP32[$2>>2]|0;
   $585 = ((($584)) + 28|0);
   $586 = HEAP32[$585>>2]|0;
   $587 = HEAP32[$pTab>>2]|0;
   $588 = ((($587)) + 34|0);
   $589 = HEAP16[$588>>1]|0;
   $590 = $589 << 16 >> 16;
   $591 = (($590) + 1)|0;
   $592 = Math_imul($586, $591)|0;
   $593 = (($592) + 1)|0;
   $594 = HEAP32[$2>>2]|0;
   $595 = ((($594)) + 32|0);
   $596 = HEAP16[$595>>1]|0;
   $597 = $596 << 16 >> 16;
   $598 = (($593) + ($597))|0;
   HEAP32[$p1>>2] = $598;
   $599 = HEAP32[$v>>2]|0;
   $600 = HEAP32[$p1>>2]|0;
   $601 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp2($599,143,$600,$601)|0);
   $602 = HEAP32[$2>>2]|0;
   $603 = ((($602)) + 32|0);
   $604 = HEAP16[$603>>1]|0;
   $605 = $604 << 16 >> 16;
   $606 = ($605|0)>=(0);
   if ($606) {
    $607 = HEAP32[$2>>2]|0;
    $608 = ((($607)) + 32|0);
    $609 = HEAP16[$608>>1]|0;
    $610 = $609 << 16 >> 16;
    $611 = HEAP32[$pTab>>2]|0;
    $612 = ((($611)) + 4|0);
    $613 = HEAP32[$612>>2]|0;
    $614 = (($613) + ($610<<4)|0);
    $615 = ((($614)) + 13|0);
    $616 = HEAP8[$615>>0]|0;
    $617 = $616 << 24 >> 24;
    $618 = ($617|0)==(69);
    if ($618) {
     $619 = HEAP32[$v>>2]|0;
     $620 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeAddOp1($619,92,$620)|0);
    }
   }
   break;
  }
  case 158:  {
   $621 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($621,29838,$vararg_buffer7);
   break;
  }
  case 83:  {
   $689 = HEAP32[$1>>2]|0;
   $690 = ((($689)) + 128|0);
   $691 = HEAP32[$690>>2]|0;
   $692 = ($691|0)!=(0|0);
   if (!($692)) {
    $693 = HEAP32[$1>>2]|0;
    _sqlite3ErrorMsg($693,31116,$vararg_buffer9);
    HEAP32[$0>>2] = 0;
    $722 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($722|0);
   }
   $694 = HEAP32[$2>>2]|0;
   $695 = ((($694)) + 1|0);
   $696 = HEAP8[$695>>0]|0;
   $697 = $696 << 24 >> 24;
   $698 = ($697|0)==(2);
   if ($698) {
    $699 = HEAP32[$1>>2]|0;
    _sqlite3MayAbort($699);
   }
   $700 = HEAP32[$2>>2]|0;
   $701 = ((($700)) + 1|0);
   $702 = HEAP8[$701>>0]|0;
   $703 = $702 << 24 >> 24;
   $704 = ($703|0)==(4);
   if ($704) {
    $705 = HEAP32[$v>>2]|0;
    $706 = HEAP32[$2>>2]|0;
    $707 = ((($706)) + 8|0);
    $708 = HEAP32[$707>>2]|0;
    (_sqlite3VdbeAddOp4($705,75,0,4,0,$708,0)|0);
    break L9;
   } else {
    $709 = HEAP32[$1>>2]|0;
    $710 = HEAP32[$2>>2]|0;
    $711 = ((($710)) + 1|0);
    $712 = HEAP8[$711>>0]|0;
    $713 = $712 << 24 >> 24;
    $714 = HEAP32[$2>>2]|0;
    $715 = ((($714)) + 8|0);
    $716 = HEAP32[$715>>2]|0;
    _sqlite3HaltConstraint($709,1811,$713,$716,0,0);
    break L9;
   }
   break;
  }
  default: {
   HEAP32[$pTest>>2] = 0;
   $622 = HEAP32[$2>>2]|0;
   $623 = ((($622)) + 20|0);
   $624 = HEAP32[$623>>2]|0;
   HEAP32[$pEList>>2] = $624;
   $625 = HEAP32[$pEList>>2]|0;
   $626 = ((($625)) + 4|0);
   $627 = HEAP32[$626>>2]|0;
   HEAP32[$aListelem>>2] = $627;
   $628 = HEAP32[$pEList>>2]|0;
   $629 = HEAP32[$628>>2]|0;
   HEAP32[$nExpr>>2] = $629;
   $630 = HEAP32[$v>>2]|0;
   $631 = (_sqlite3VdbeMakeLabel($630)|0);
   HEAP32[$endLabel>>2] = $631;
   $632 = HEAP32[$2>>2]|0;
   $633 = ((($632)) + 12|0);
   $634 = HEAP32[$633>>2]|0;
   HEAP32[$pX>>2] = $634;
   $635 = ($634|0)!=(0|0);
   if ($635) {
    $636 = HEAP32[$pX>>2]|0;
    dest=$tempX; src=$636; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
    $637 = HEAP32[$1>>2]|0;
    $638 = (_exprCodeVector($637,$tempX,$regFree1)|0);
    _exprToRegister($tempX,$638);
    dest=$opCompare; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
    HEAP8[$opCompare>>0] = 37;
    $639 = ((($opCompare)) + 12|0);
    HEAP32[$639>>2] = $tempX;
    HEAP32[$pTest>>2] = $opCompare;
    HEAP32[$regFree1>>2] = 0;
   }
   HEAP32[$i2>>2] = 0;
   while(1) {
    $640 = HEAP32[$i2>>2]|0;
    $641 = HEAP32[$nExpr>>2]|0;
    $642 = (($641) - 1)|0;
    $643 = ($640|0)<($642|0);
    if (!($643)) {
     break;
    }
    $644 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePush($644);
    $645 = HEAP32[$pX>>2]|0;
    $646 = ($645|0)!=(0|0);
    $647 = HEAP32[$i2>>2]|0;
    $648 = HEAP32[$aListelem>>2]|0;
    $649 = (($648) + (($647*20)|0)|0);
    $650 = HEAP32[$649>>2]|0;
    if ($646) {
     $651 = ((($opCompare)) + 16|0);
     HEAP32[$651>>2] = $650;
    } else {
     HEAP32[$pTest>>2] = $650;
    }
    $652 = HEAP32[$v>>2]|0;
    $653 = (_sqlite3VdbeMakeLabel($652)|0);
    HEAP32[$nextCase>>2] = $653;
    $654 = HEAP32[$1>>2]|0;
    $655 = HEAP32[$pTest>>2]|0;
    $656 = HEAP32[$nextCase>>2]|0;
    _sqlite3ExprIfFalse($654,$655,$656,16);
    $657 = HEAP32[$1>>2]|0;
    $658 = HEAP32[$i2>>2]|0;
    $659 = (($658) + 1)|0;
    $660 = HEAP32[$aListelem>>2]|0;
    $661 = (($660) + (($659*20)|0)|0);
    $662 = HEAP32[$661>>2]|0;
    $663 = HEAP32[$3>>2]|0;
    _sqlite3ExprCode($657,$662,$663);
    $664 = HEAP32[$v>>2]|0;
    $665 = HEAP32[$endLabel>>2]|0;
    (_sqlite3VdbeGoto($664,$665)|0);
    $666 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePop($666);
    $667 = HEAP32[$v>>2]|0;
    $668 = HEAP32[$nextCase>>2]|0;
    _sqlite3VdbeResolveLabel($667,$668);
    $669 = HEAP32[$i2>>2]|0;
    $670 = (($669) + 2)|0;
    HEAP32[$i2>>2] = $670;
   }
   $671 = HEAP32[$nExpr>>2]|0;
   $672 = $671 & 1;
   $673 = ($672|0)!=(0);
   if ($673) {
    $674 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePush($674);
    $675 = HEAP32[$1>>2]|0;
    $676 = HEAP32[$nExpr>>2]|0;
    $677 = (($676) - 1)|0;
    $678 = HEAP32[$pEList>>2]|0;
    $679 = ((($678)) + 4|0);
    $680 = HEAP32[$679>>2]|0;
    $681 = (($680) + (($677*20)|0)|0);
    $682 = HEAP32[$681>>2]|0;
    $683 = HEAP32[$3>>2]|0;
    _sqlite3ExprCode($675,$682,$683);
    $684 = HEAP32[$1>>2]|0;
    _sqlite3ExprCachePop($684);
   } else {
    $685 = HEAP32[$v>>2]|0;
    $686 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($685,79,0,$686)|0);
   }
   $687 = HEAP32[$v>>2]|0;
   $688 = HEAP32[$endLabel>>2]|0;
   _sqlite3VdbeResolveLabel($687,$688);
  }
  }
 } while(0);
 do {
  if ((label|0) == 11) {
   $46 = HEAP32[$2>>2]|0;
   $47 = ((($46)) + 28|0);
   $48 = HEAP32[$47>>2]|0;
   HEAP32[$iTab>>2] = $48;
   $49 = HEAP32[$iTab>>2]|0;
   $50 = ($49|0)<(0);
   do {
    if ($50) {
     $51 = HEAP32[$1>>2]|0;
     $52 = ((($51)) + 56|0);
     $53 = HEAP32[$52>>2]|0;
     $54 = ($53|0)>(0);
     if (!($54)) {
      $63 = HEAP32[$1>>2]|0;
      $64 = ((($63)) + 60|0);
      $65 = HEAP32[$64>>2]|0;
      HEAP32[$iTab>>2] = $65;
      break;
     }
     $55 = HEAP32[$2>>2]|0;
     $56 = ((($55)) + 32|0);
     $57 = HEAP16[$56>>1]|0;
     $58 = $57 << 16 >> 16;
     $59 = HEAP32[$1>>2]|0;
     $60 = ((($59)) + 56|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = (($58) + ($61))|0;
     HEAP32[$0>>2] = $62;
     $722 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($722|0);
    }
   } while(0);
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$2>>2]|0;
   $68 = ((($67)) + 44|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = HEAP32[$2>>2]|0;
   $71 = ((($70)) + 32|0);
   $72 = HEAP16[$71>>1]|0;
   $73 = $72 << 16 >> 16;
   $74 = HEAP32[$iTab>>2]|0;
   $75 = HEAP32[$3>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = ((($76)) + 38|0);
   $78 = HEAP8[$77>>0]|0;
   $79 = (_sqlite3ExprCodeGetColumn($66,$69,$73,$74,$75,$78)|0);
   HEAP32[$0>>2] = $79;
   $722 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($722|0);
  }
  else if ((label|0) == 29) {
   $171 = HEAP32[$2>>2]|0;
   $172 = ((($171)) + 12|0);
   $173 = HEAP32[$172>>2]|0;
   HEAP32[$pLeft>>2] = $173;
   $174 = HEAP32[$pLeft>>2]|0;
   $175 = (_sqlite3ExprIsVector($174)|0);
   $176 = ($175|0)!=(0);
   $177 = HEAP32[$1>>2]|0;
   if ($176) {
    $178 = HEAP32[$2>>2]|0;
    $179 = HEAP32[$3>>2]|0;
    $180 = HEAP32[$op>>2]|0;
    $181 = $180&255;
    $182 = HEAP32[$p5>>2]|0;
    $183 = $182&255;
    _codeVectorCompare($177,$178,$179,$181,$183);
    break;
   } else {
    $184 = HEAP32[$pLeft>>2]|0;
    $185 = (_sqlite3ExprCodeTemp($177,$184,$regFree1)|0);
    HEAP32[$r1>>2] = $185;
    $186 = HEAP32[$1>>2]|0;
    $187 = HEAP32[$2>>2]|0;
    $188 = ((($187)) + 16|0);
    $189 = HEAP32[$188>>2]|0;
    $190 = (_sqlite3ExprCodeTemp($186,$189,$regFree2)|0);
    HEAP32[$r2>>2] = $190;
    $191 = HEAP32[$1>>2]|0;
    $192 = HEAP32[$pLeft>>2]|0;
    $193 = HEAP32[$2>>2]|0;
    $194 = ((($193)) + 16|0);
    $195 = HEAP32[$194>>2]|0;
    $196 = HEAP32[$op>>2]|0;
    $197 = HEAP32[$r1>>2]|0;
    $198 = HEAP32[$r2>>2]|0;
    $199 = HEAP32[$inReg>>2]|0;
    $200 = HEAP32[$p5>>2]|0;
    $201 = 32 | $200;
    (_codeCompare($191,$192,$195,$196,$197,$198,$199,$201)|0);
    break;
   }
  }
 } while(0);
 $717 = HEAP32[$1>>2]|0;
 $718 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($717,$718);
 $719 = HEAP32[$1>>2]|0;
 $720 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($719,$720);
 $721 = HEAP32[$inReg>>2]|0;
 HEAP32[$0>>2] = $721;
 $722 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($722|0);
}
function _sqlite3ExprCodeGetColumn($pParse,$pTab,$iColumn,$iTable,$iReg,$p5) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iColumn = $iColumn|0;
 $iTable = $iTable|0;
 $iReg = $iReg|0;
 $p5 = $p5|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, $p = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $6 = sp + 36|0;
 $v = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $iColumn;
 HEAP32[$4>>2] = $iTable;
 HEAP32[$5>>2] = $iReg;
 HEAP8[$6>>0] = $p5;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 HEAP32[$i>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 152|0);
 HEAP32[$p>>2] = $11;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 25|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($12|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$p>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = ($19|0)==($20|0);
  if ($21) {
   $22 = HEAP32[$p>>2]|0;
   $23 = ((($22)) + 4|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24 << 16 >> 16;
   $26 = HEAP32[$3>>2]|0;
   $27 = ($25|0)==($26|0);
   if ($27) {
    label = 5;
    break;
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
  $43 = HEAP32[$p>>2]|0;
  $44 = ((($43)) + 20|0);
  HEAP32[$p>>2] = $44;
 }
 if ((label|0) == 5) {
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 68|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$29>>2] = $31;
  $32 = HEAP32[$p>>2]|0;
  $33 = ((($32)) + 16|0);
  HEAP32[$33>>2] = $30;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3ExprCachePinRegister($34,$37);
  $38 = HEAP32[$p>>2]|0;
  $39 = ((($38)) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$0>>2] = $40;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $45 = HEAP32[$v>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = HEAP32[$4>>2]|0;
 $48 = HEAP32[$3>>2]|0;
 $49 = HEAP32[$5>>2]|0;
 _sqlite3ExprCodeGetColumnOfTable($45,$46,$47,$48,$49);
 $50 = HEAP8[$6>>0]|0;
 $51 = ($50<<24>>24)!=(0);
 if ($51) {
  $52 = HEAP32[$v>>2]|0;
  $53 = HEAP8[$6>>0]|0;
  _sqlite3VdbeChangeP5($52,$53);
 } else {
  $54 = HEAP32[$1>>2]|0;
  $55 = HEAP32[$4>>2]|0;
  $56 = HEAP32[$3>>2]|0;
  $57 = HEAP32[$5>>2]|0;
  _sqlite3ExprCacheStore($54,$55,$56,$57);
 }
 $58 = HEAP32[$5>>2]|0;
 HEAP32[$0>>2] = $58;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _codeInteger($pParse,$pExpr,$negFlag,$iMem) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $negFlag = $negFlag|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $c = 0, $i = 0, $or$cond = 0, $v = 0, $value = 0, $vararg_buffer = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $c = sp + 16|0;
 $value = sp;
 $z = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $negFlag;
 HEAP32[$3>>2] = $iMem;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 1024;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 8|0);
 if ($11) {
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$i>>2] = $14;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = HEAP32[$i>>2]|0;
   $18 = (0 - ($17))|0;
   HEAP32[$i>>2] = $18;
  }
  $19 = HEAP32[$v>>2]|0;
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  (_sqlite3VdbeAddOp2($19,76,$20,$21)|0);
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$13>>2]|0;
 HEAP32[$z>>2] = $22;
 $23 = HEAP32[$z>>2]|0;
 $24 = (_sqlite3DecOrHexToI64($23,$value)|0);
 HEAP32[$c>>2] = $24;
 $25 = HEAP32[$c>>2]|0;
 $26 = ($25|0)==(0);
 if (!($26)) {
  $27 = HEAP32[$c>>2]|0;
  $28 = ($27|0)==(2);
  $29 = HEAP32[$2>>2]|0;
  $30 = ($29|0)!=(0);
  $or$cond = $28 & $30;
  if (!($or$cond)) {
   $51 = HEAP32[$z>>2]|0;
   $52 = (_sqlite3_strnicmp($51,31290,2)|0);
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = HEAP32[$0>>2]|0;
    $55 = HEAP32[$z>>2]|0;
    HEAP32[$vararg_buffer>>2] = $55;
    _sqlite3ErrorMsg($54,31293,$vararg_buffer);
    STACKTOP = sp;return;
   } else {
    $56 = HEAP32[$v>>2]|0;
    $57 = HEAP32[$z>>2]|0;
    $58 = HEAP32[$2>>2]|0;
    $59 = HEAP32[$3>>2]|0;
    _codeReal($56,$57,$58,$59);
    STACKTOP = sp;return;
   }
  }
 }
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[$c>>2]|0;
  $34 = ($33|0)==(2);
  $35 = $value;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (_i64Subtract(0,0,($37|0),($40|0))|0);
  $42 = tempRet0;
  $43 = $34 ? 0 : $41;
  $44 = $34 ? -2147483648 : $42;
  $45 = $value;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
 }
 $49 = HEAP32[$v>>2]|0;
 $50 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4Dup8($49,77,0,$50,0,$value,-13)|0);
 STACKTOP = sp;return;
}
function _codeReal($v,$z,$negateFlag,$iMem) {
 $v = $v|0;
 $z = $z|0;
 $negateFlag = $negateFlag|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $value = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $value = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $negateFlag;
 HEAP32[$3>>2] = $iMem;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3Strlen30($7)|0);
 (_sqlite3AtoF($6,$value,$8,1)|0);
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = +HEAPF64[$value>>3];
  $12 = -$11;
  HEAPF64[$value>>3] = $12;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4Dup8($13,132,0,$14,0,$value,-12)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheAffinityChange($pParse,$iStart,$iCount) {
 $pParse = $pParse|0;
 $iStart = $iStart|0;
 $iCount = $iCount|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iStart;
 HEAP32[$2>>2] = $iCount;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheRemove($3,$4,$5);
 STACKTOP = sp;return;
}
function _sqlite3ExprIsVector($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pExpr;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3ExprVectorSize($1)|0);
 $3 = ($2|0)>(1);
 $4 = $3&1;
 STACKTOP = sp;return ($4|0);
}
function _codeVectorCompare($pParse,$pExpr,$dest,$op,$p5) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $op = $op|0;
 $p5 = $p5|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $addrDone = 0;
 var $i = 0, $nLeft = 0, $opx = 0, $pL = 0, $pLeft = 0, $pR = 0, $pRight = 0, $r1 = 0, $r2 = 0, $regFree1 = 0, $regFree2 = 0, $regLeft = 0, $regRight = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 70|0;
 $4 = sp + 69|0;
 $v = sp + 52|0;
 $pLeft = sp + 48|0;
 $pRight = sp + 44|0;
 $nLeft = sp + 40|0;
 $i = sp + 36|0;
 $regLeft = sp + 32|0;
 $regRight = sp + 28|0;
 $opx = sp + 68|0;
 $addrDone = sp + 24|0;
 $regFree1 = sp + 20|0;
 $regFree2 = sp + 16|0;
 $pL = sp + 12|0;
 $pR = sp + 8|0;
 $r1 = sp + 4|0;
 $r2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP8[$3>>0] = $op;
 HEAP8[$4>>0] = $p5;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pLeft>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pRight>>2] = $13;
 $14 = HEAP32[$pLeft>>2]|0;
 $15 = (_sqlite3ExprVectorSize($14)|0);
 HEAP32[$nLeft>>2] = $15;
 HEAP32[$regLeft>>2] = 0;
 HEAP32[$regRight>>2] = 0;
 $16 = HEAP8[$3>>0]|0;
 HEAP8[$opx>>0] = $16;
 $17 = HEAP32[$v>>2]|0;
 $18 = (_sqlite3VdbeMakeLabel($17)|0);
 HEAP32[$addrDone>>2] = $18;
 $19 = HEAP8[$4>>0]|0;
 $20 = $19&255;
 $21 = $20 | 32;
 $22 = $21&255;
 HEAP8[$4>>0] = $22;
 $23 = HEAP8[$opx>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)==(39);
 if ($25) {
  HEAP8[$opx>>0] = 40;
 }
 $26 = HEAP8[$opx>>0]|0;
 $27 = $26&255;
 $28 = ($27|0)==(41);
 if ($28) {
  HEAP8[$opx>>0] = 38;
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$pLeft>>2]|0;
 $31 = (_exprCodeSubselect($29,$30)|0);
 HEAP32[$regLeft>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$pRight>>2]|0;
 $34 = (_exprCodeSubselect($32,$33)|0);
 HEAP32[$regRight>>2] = $34;
 HEAP32[$i>>2] = 0;
 while(1) {
  HEAP32[$regFree1>>2] = 0;
  HEAP32[$regFree2>>2] = 0;
  $35 = HEAP32[$i>>2]|0;
  $36 = ($35|0)>(0);
  if ($36) {
   $37 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($37);
  }
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$pLeft>>2]|0;
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$regLeft>>2]|0;
  $42 = (_exprVectorRegister($38,$39,$40,$41,$pL,$regFree1)|0);
  HEAP32[$r1>>2] = $42;
  $43 = HEAP32[$0>>2]|0;
  $44 = HEAP32[$pRight>>2]|0;
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$regRight>>2]|0;
  $47 = (_exprVectorRegister($43,$44,$45,$46,$pR,$regFree2)|0);
  HEAP32[$r2>>2] = $47;
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$pL>>2]|0;
  $50 = HEAP32[$pR>>2]|0;
  $51 = HEAP8[$opx>>0]|0;
  $52 = $51&255;
  $53 = HEAP32[$r1>>2]|0;
  $54 = HEAP32[$r2>>2]|0;
  $55 = HEAP32[$2>>2]|0;
  $56 = HEAP8[$4>>0]|0;
  $57 = $56&255;
  (_codeCompare($48,$49,$50,$52,$53,$54,$55,$57)|0);
  $58 = HEAP32[$0>>2]|0;
  $59 = HEAP32[$regFree1>>2]|0;
  _sqlite3ReleaseTempReg($58,$59);
  $60 = HEAP32[$0>>2]|0;
  $61 = HEAP32[$regFree2>>2]|0;
  _sqlite3ReleaseTempReg($60,$61);
  $62 = HEAP32[$i>>2]|0;
  $63 = ($62|0)>(0);
  if ($63) {
   $64 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($64);
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$nLeft>>2]|0;
  $67 = (($66) - 1)|0;
  $68 = ($65|0)==($67|0);
  if ($68) {
   break;
  }
  $69 = HEAP8[$opx>>0]|0;
  $70 = $69&255;
  $71 = ($70|0)==(37);
  do {
   if ($71) {
    $72 = HEAP32[$v>>2]|0;
    $73 = HEAP32[$2>>2]|0;
    $74 = HEAP32[$addrDone>>2]|0;
    (_sqlite3VdbeAddOp2($72,22,$73,$74)|0);
    $75 = HEAP8[$4>>0]|0;
    $76 = $75&255;
    $77 = $76 | 8;
    $78 = $77&255;
    HEAP8[$4>>0] = $78;
   } else {
    $79 = HEAP8[$opx>>0]|0;
    $80 = $79&255;
    $81 = ($80|0)==(36);
    $82 = HEAP32[$v>>2]|0;
    if ($81) {
     $83 = HEAP32[$2>>2]|0;
     $84 = HEAP32[$addrDone>>2]|0;
     (_sqlite3VdbeAddOp2($82,21,$83,$84)|0);
     $85 = HEAP8[$4>>0]|0;
     $86 = $85&255;
     $87 = $86 | 8;
     $88 = $87&255;
     HEAP8[$4>>0] = $88;
     break;
    }
    $89 = HEAP32[$addrDone>>2]|0;
    (_sqlite3VdbeAddOp2($82,42,0,$89)|0);
    $90 = HEAP32[$i>>2]|0;
    $91 = HEAP32[$nLeft>>2]|0;
    $92 = (($91) - 2)|0;
    $93 = ($90|0)==($92|0);
    if ($93) {
     $94 = HEAP8[$3>>0]|0;
     HEAP8[$opx>>0] = $94;
    }
   }
  } while(0);
  $95 = HEAP32[$i>>2]|0;
  $96 = (($95) + 1)|0;
  HEAP32[$i>>2] = $96;
 }
 $97 = HEAP32[$v>>2]|0;
 $98 = HEAP32[$addrDone>>2]|0;
 _sqlite3VdbeResolveLabel($97,$98);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeTemp($pParse,$pExpr,$pReg) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $pReg = $pReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pItem = 0, $r1 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $r2 = sp + 16|0;
 $p = sp + 12|0;
 $i = sp + 8|0;
 $pItem = sp + 4|0;
 $r1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $pReg;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3ExprSkipCollate($4)|0);
 HEAP32[$2>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 23|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(157);
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = (_sqlite3ExprIsConstantNotJoin($15)|0);
   $17 = ($16|0)!=(0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 80|0);
    $20 = HEAP32[$19>>2]|0;
    HEAP32[$p>>2] = $20;
    $21 = HEAP32[$3>>2]|0;
    HEAP32[$21>>2] = 0;
    $22 = HEAP32[$p>>2]|0;
    $23 = ($22|0)!=(0|0);
    L5: do {
     if ($23) {
      $24 = HEAP32[$p>>2]|0;
      $25 = ((($24)) + 4|0);
      $26 = HEAP32[$25>>2]|0;
      HEAP32[$pItem>>2] = $26;
      $27 = HEAP32[$p>>2]|0;
      $28 = HEAP32[$27>>2]|0;
      HEAP32[$i>>2] = $28;
      while(1) {
       $29 = HEAP32[$i>>2]|0;
       $30 = ($29|0)>(0);
       if (!($30)) {
        break L5;
       }
       $31 = HEAP32[$pItem>>2]|0;
       $32 = ((($31)) + 13|0);
       $33 = HEAP8[$32>>0]|0;
       $34 = ($33&255) >>> 2;
       $35 = $34 & 1;
       $36 = $35&255;
       $37 = ($36|0)!=(0);
       if ($37) {
        $38 = HEAP32[$pItem>>2]|0;
        $39 = HEAP32[$38>>2]|0;
        $40 = HEAP32[$2>>2]|0;
        $41 = (_sqlite3ExprCompare($39,$40,-1)|0);
        $42 = ($41|0)==(0);
        if ($42) {
         break;
        }
       }
       $46 = HEAP32[$pItem>>2]|0;
       $47 = ((($46)) + 20|0);
       HEAP32[$pItem>>2] = $47;
       $48 = HEAP32[$i>>2]|0;
       $49 = (($48) + -1)|0;
       HEAP32[$i>>2] = $49;
      }
      $43 = HEAP32[$pItem>>2]|0;
      $44 = ((($43)) + 16|0);
      $45 = HEAP32[$44>>2]|0;
      HEAP32[$0>>2] = $45;
      $72 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($72|0);
     }
    } while(0);
    $50 = HEAP32[$1>>2]|0;
    $51 = ((($50)) + 44|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (($52) + 1)|0;
    HEAP32[$51>>2] = $53;
    HEAP32[$r2>>2] = $53;
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$2>>2]|0;
    $56 = HEAP32[$r2>>2]|0;
    _sqlite3ExprCodeAtInit($54,$55,$56,1);
   } else {
    label = 12;
   }
  } else {
   label = 12;
  }
 } else {
  label = 12;
 }
 do {
  if ((label|0) == 12) {
   $57 = HEAP32[$1>>2]|0;
   $58 = (_sqlite3GetTempReg($57)|0);
   HEAP32[$r1>>2] = $58;
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = HEAP32[$r1>>2]|0;
   $62 = (_sqlite3ExprCodeTarget($59,$60,$61)|0);
   HEAP32[$r2>>2] = $62;
   $63 = HEAP32[$r2>>2]|0;
   $64 = HEAP32[$r1>>2]|0;
   $65 = ($63|0)==($64|0);
   if ($65) {
    $66 = HEAP32[$r1>>2]|0;
    $67 = HEAP32[$3>>2]|0;
    HEAP32[$67>>2] = $66;
    break;
   } else {
    $68 = HEAP32[$1>>2]|0;
    $69 = HEAP32[$r1>>2]|0;
    _sqlite3ReleaseTempReg($68,$69);
    $70 = HEAP32[$3>>2]|0;
    HEAP32[$70>>2] = 0;
    break;
   }
  }
 } while(0);
 $71 = HEAP32[$r2>>2]|0;
 HEAP32[$0>>2] = $71;
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _codeCompare($pParse,$pLeft,$pRight,$opcode,$in1,$in2,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 $opcode = $opcode|0;
 $in1 = $in1|0;
 $in2 = $in2|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr = 0, $p4 = 0, $p5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $7 = sp + 12|0;
 $p5 = sp + 8|0;
 $addr = sp + 4|0;
 $p4 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLeft;
 HEAP32[$2>>2] = $pRight;
 HEAP32[$3>>2] = $opcode;
 HEAP32[$4>>2] = $in1;
 HEAP32[$5>>2] = $in2;
 HEAP32[$6>>2] = $dest;
 HEAP32[$7>>2] = $jumpIfNull;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (_sqlite3BinaryCompareCollSeq($8,$9,$10)|0);
 HEAP32[$p4>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$7>>2]|0;
 $15 = (_binaryCompareP5($12,$13,$14)|0);
 $16 = $15&255;
 HEAP32[$p5>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$5>>2]|0;
 $22 = HEAP32[$6>>2]|0;
 $23 = HEAP32[$4>>2]|0;
 $24 = HEAP32[$p4>>2]|0;
 $25 = (_sqlite3VdbeAddOp4($19,$20,$21,$22,$23,$24,-4)|0);
 HEAP32[$addr>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 8|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$p5>>2]|0;
 $30 = $29&255;
 _sqlite3VdbeChangeP5($28,$30);
 $31 = HEAP32[$addr>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _sqlite3ExprIsConstant($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_exprIsConst($1,1,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _sqlite3ExprCodeExprList($pParse,$pList,$target,$srcReg,$flags) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $target = $target|0;
 $srcReg = $srcReg|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $copyOp = 0, $i = 0, $inReg = 0, $j = 0, $n = 0, $pExpr = 0, $pItem = 0, $pOp = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 49|0;
 $pItem = sp + 28|0;
 $i = sp + 24|0;
 $j = sp + 20|0;
 $n = sp + 16|0;
 $copyOp = sp + 48|0;
 $v = sp + 12|0;
 $pExpr = sp + 8|0;
 $inReg = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $target;
 HEAP32[$3>>2] = $srcReg;
 HEAP8[$4>>0] = $flags;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = $6 & 1;
 $8 = ($7|0)!=(0);
 $9 = $8 ? 84 : 85;
 $10 = $9&255;
 HEAP8[$copyOp>>0] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$v>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 23|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18<<24>>24)!=(0);
 if (!($19)) {
  $20 = HEAP8[$4>>0]|0;
  $21 = $20&255;
  $22 = $21 & -3;
  $23 = $22&255;
  HEAP8[$4>>0] = $23;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pItem>>2] = $26;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$n>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$pItem>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pExpr>>2] = $31;
  $32 = HEAP8[$4>>0]|0;
  $33 = $32&255;
  $34 = $33 & 4;
  $35 = ($34|0)!=(0);
  if ($35) {
   $36 = HEAP32[$i>>2]|0;
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + (($36*20)|0)|0);
   $41 = ((($40)) + 16|0);
   $42 = HEAP16[$41>>1]|0;
   $43 = $42&65535;
   HEAP32[$j>>2] = $43;
   $44 = ($43|0)>(0);
   if ($44) {
    $45 = HEAP32[$v>>2]|0;
    $46 = HEAP8[$copyOp>>0]|0;
    $47 = $46&255;
    $48 = HEAP32[$j>>2]|0;
    $49 = HEAP32[$3>>2]|0;
    $50 = (($48) + ($49))|0;
    $51 = (($50) - 1)|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = HEAP32[$i>>2]|0;
    $54 = (($52) + ($53))|0;
    (_sqlite3VdbeAddOp2($45,$47,$51,$54)|0);
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  do {
   if ((label|0) == 8) {
    label = 0;
    $55 = HEAP8[$4>>0]|0;
    $56 = $55&255;
    $57 = $56 & 2;
    $58 = ($57|0)!=(0);
    if ($58) {
     $59 = HEAP32[$pExpr>>2]|0;
     $60 = (_sqlite3ExprIsConstant($59)|0);
     $61 = ($60|0)!=(0);
     if ($61) {
      $62 = HEAP32[$0>>2]|0;
      $63 = HEAP32[$pExpr>>2]|0;
      $64 = HEAP32[$2>>2]|0;
      $65 = HEAP32[$i>>2]|0;
      $66 = (($64) + ($65))|0;
      _sqlite3ExprCodeAtInit($62,$63,$66,0);
      break;
     }
    }
    $67 = HEAP32[$0>>2]|0;
    $68 = HEAP32[$pExpr>>2]|0;
    $69 = HEAP32[$2>>2]|0;
    $70 = HEAP32[$i>>2]|0;
    $71 = (($69) + ($70))|0;
    $72 = (_sqlite3ExprCodeTarget($67,$68,$71)|0);
    HEAP32[$inReg>>2] = $72;
    $73 = HEAP32[$inReg>>2]|0;
    $74 = HEAP32[$2>>2]|0;
    $75 = HEAP32[$i>>2]|0;
    $76 = (($74) + ($75))|0;
    $77 = ($73|0)!=($76|0);
    if ($77) {
     $78 = HEAP8[$copyOp>>0]|0;
     $79 = $78&255;
     $80 = ($79|0)==(84);
     if ($80) {
      $81 = HEAP32[$v>>2]|0;
      $82 = (_sqlite3VdbeGetOp($81,-1)|0);
      HEAP32[$pOp>>2] = $82;
      $83 = HEAP8[$82>>0]|0;
      $84 = $83&255;
      $85 = ($84|0)==(84);
      if ($85) {
       $86 = HEAP32[$pOp>>2]|0;
       $87 = ((($86)) + 4|0);
       $88 = HEAP32[$87>>2]|0;
       $89 = HEAP32[$pOp>>2]|0;
       $90 = ((($89)) + 12|0);
       $91 = HEAP32[$90>>2]|0;
       $92 = (($88) + ($91))|0;
       $93 = (($92) + 1)|0;
       $94 = HEAP32[$inReg>>2]|0;
       $95 = ($93|0)==($94|0);
       if ($95) {
        $96 = HEAP32[$pOp>>2]|0;
        $97 = ((($96)) + 8|0);
        $98 = HEAP32[$97>>2]|0;
        $99 = HEAP32[$pOp>>2]|0;
        $100 = ((($99)) + 12|0);
        $101 = HEAP32[$100>>2]|0;
        $102 = (($98) + ($101))|0;
        $103 = (($102) + 1)|0;
        $104 = HEAP32[$2>>2]|0;
        $105 = HEAP32[$i>>2]|0;
        $106 = (($104) + ($105))|0;
        $107 = ($103|0)==($106|0);
        if ($107) {
         $108 = HEAP32[$pOp>>2]|0;
         $109 = ((($108)) + 12|0);
         $110 = HEAP32[$109>>2]|0;
         $111 = (($110) + 1)|0;
         HEAP32[$109>>2] = $111;
         break;
        }
       }
      }
     }
     $112 = HEAP32[$v>>2]|0;
     $113 = HEAP8[$copyOp>>0]|0;
     $114 = $113&255;
     $115 = HEAP32[$inReg>>2]|0;
     $116 = HEAP32[$2>>2]|0;
     $117 = HEAP32[$i>>2]|0;
     $118 = (($116) + ($117))|0;
     (_sqlite3VdbeAddOp2($112,$114,$115,$118)|0);
    }
   }
  } while(0);
  $119 = HEAP32[$i>>2]|0;
  $120 = (($119) + 1)|0;
  HEAP32[$i>>2] = $120;
  $121 = HEAP32[$pItem>>2]|0;
  $122 = ((($121)) + 20|0);
  HEAP32[$pItem>>2] = $122;
 }
 $123 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _sqlite3VtabOverloadFunction($db,$pDef,$nArg,$pExpr) {
 $db = $db|0;
 $pDef = $pDef|0;
 $nArg = $nArg|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0;
 var $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pArg = 0, $pMod = 0, $pNew = 0, $pTab = 0, $pVtab = 0, $rc = 0, $xSFunc = 0, $z = 0, $zLowerName = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $pTab = sp + 32|0;
 $pVtab = sp + 28|0;
 $pMod = sp + 24|0;
 $xSFunc = sp + 20|0;
 $pArg = sp + 16|0;
 $pNew = sp + 12|0;
 $rc = sp + 8|0;
 $zLowerName = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pDef;
 HEAP32[$3>>2] = $nArg;
 HEAP32[$4>>2] = $pExpr;
 HEAP32[$xSFunc>>2] = 0;
 HEAP32[$pArg>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $7;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $8 = HEAP32[$4>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(152);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $12;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $13 = HEAP32[$4>>2]|0;
 $14 = ((($13)) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTab>>2] = $15;
 $16 = HEAP32[$pTab>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $18;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $19 = HEAP32[$pTab>>2]|0;
 $20 = ((($19)) + 42|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = $22 & 16;
 $24 = ($23|0)==(0);
 if ($24) {
  $25 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $25;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = HEAP32[$pTab>>2]|0;
 $28 = (_sqlite3GetVTable($26,$27)|0);
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$pVtab>>2] = $30;
 $31 = HEAP32[$pVtab>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$pMod>>2] = $32;
 $33 = HEAP32[$pMod>>2]|0;
 $34 = ((($33)) + 72|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)==(0|0);
 if ($36) {
  $37 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $37;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = ((($39)) + 20|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (_sqlite3DbStrDup($38,$41)|0);
 HEAP32[$zLowerName>>2] = $42;
 $43 = HEAP32[$zLowerName>>2]|0;
 $44 = ($43|0)!=(0|0);
 if ($44) {
  $45 = HEAP32[$zLowerName>>2]|0;
  HEAP32[$z>>2] = $45;
  while(1) {
   $46 = HEAP32[$z>>2]|0;
   $47 = HEAP8[$46>>0]|0;
   $48 = ($47<<24>>24)!=(0);
   if (!($48)) {
    break;
   }
   $49 = HEAP32[$z>>2]|0;
   $50 = HEAP8[$49>>0]|0;
   $51 = $50&255;
   $52 = (17613 + ($51)|0);
   $53 = HEAP8[$52>>0]|0;
   $54 = HEAP32[$z>>2]|0;
   HEAP8[$54>>0] = $53;
   $55 = HEAP32[$z>>2]|0;
   $56 = ((($55)) + 1|0);
   HEAP32[$z>>2] = $56;
  }
  $57 = HEAP32[$pMod>>2]|0;
  $58 = ((($57)) + 72|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP32[$pVtab>>2]|0;
  $61 = HEAP32[$3>>2]|0;
  $62 = HEAP32[$zLowerName>>2]|0;
  $63 = (FUNCTION_TABLE_iiiiii[$59 & 255]($60,$61,$62,$xSFunc,$pArg)|0);
  HEAP32[$rc>>2] = $63;
  $64 = HEAP32[$1>>2]|0;
  $65 = HEAP32[$zLowerName>>2]|0;
  _sqlite3DbFree($64,$65);
 }
 $66 = HEAP32[$rc>>2]|0;
 $67 = ($66|0)==(0);
 if ($67) {
  $68 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $68;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 $69 = HEAP32[$1>>2]|0;
 $70 = HEAP32[$2>>2]|0;
 $71 = ((($70)) + 20|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = (_sqlite3Strlen30($72)|0);
 $74 = (28 + ($73))|0;
 $75 = (($74) + 1)|0;
 $76 = (_sqlite3DbMallocZero($69,$75,0)|0);
 HEAP32[$pNew>>2] = $76;
 $77 = HEAP32[$pNew>>2]|0;
 $78 = ($77|0)==(0|0);
 if ($78) {
  $79 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $79;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 } else {
  $80 = HEAP32[$pNew>>2]|0;
  $81 = HEAP32[$2>>2]|0;
  ;HEAP32[$80>>2]=HEAP32[$81>>2]|0;HEAP32[$80+4>>2]=HEAP32[$81+4>>2]|0;HEAP32[$80+8>>2]=HEAP32[$81+8>>2]|0;HEAP32[$80+12>>2]=HEAP32[$81+12>>2]|0;HEAP32[$80+16>>2]=HEAP32[$81+16>>2]|0;HEAP32[$80+20>>2]=HEAP32[$81+20>>2]|0;HEAP32[$80+24>>2]=HEAP32[$81+24>>2]|0;
  $82 = HEAP32[$pNew>>2]|0;
  $83 = ((($82)) + 28|0);
  $84 = HEAP32[$pNew>>2]|0;
  $85 = ((($84)) + 20|0);
  HEAP32[$85>>2] = $83;
  $86 = HEAP32[$pNew>>2]|0;
  $87 = ((($86)) + 28|0);
  $88 = HEAP32[$2>>2]|0;
  $89 = ((($88)) + 20|0);
  $90 = HEAP32[$89>>2]|0;
  $91 = HEAP32[$2>>2]|0;
  $92 = ((($91)) + 20|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (_sqlite3Strlen30($93)|0);
  $95 = (($94) + 1)|0;
  _memcpy(($87|0),($90|0),($95|0))|0;
  $96 = HEAP32[$xSFunc>>2]|0;
  $97 = HEAP32[$pNew>>2]|0;
  $98 = ((($97)) + 12|0);
  HEAP32[$98>>2] = $96;
  $99 = HEAP32[$pArg>>2]|0;
  $100 = HEAP32[$pNew>>2]|0;
  $101 = ((($100)) + 4|0);
  HEAP32[$101>>2] = $99;
  $102 = HEAP32[$pNew>>2]|0;
  $103 = ((($102)) + 2|0);
  $104 = HEAP16[$103>>1]|0;
  $105 = $104&65535;
  $106 = $105 | 16;
  $107 = $106&65535;
  HEAP16[$103>>1] = $107;
  $108 = HEAP32[$pNew>>2]|0;
  HEAP32[$0>>2] = $108;
  $109 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($109|0);
 }
 return (0)|0;
}
function _sqlite3SubselectError($pParse,$nActual,$nExpect) {
 $pParse = $pParse|0;
 $nActual = $nActual|0;
 $nExpect = $nExpect|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zFmt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $zFmt = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $nActual;
 HEAP32[$2>>2] = $nExpect;
 HEAP32[$zFmt>>2] = 31246;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$zFmt>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $6;
 _sqlite3ErrorMsg($3,$4,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3CodeSubselect($pParse,$pExpr,$rHasNullFlag,$isRowid) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $rHasNullFlag = $rHasNullFlag|0;
 $isRowid = $isRowid|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $affinity = 0, $cond = 0, $dest = 0, $dest2 = 0, $i = 0, $i1 = 0, $iValToIns = 0, $jmpIfDynamic = 0;
 var $nReg = 0, $nVal = 0, $p = 0, $pE2 = 0, $pEList = 0, $pItem = 0, $pKeyInfo = 0, $pLeft = 0, $pList = 0, $pSel = 0, $pSelect = 0, $r1 = 0, $r2 = 0, $r3 = 0, $rReg = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zMsg = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp;
 $0 = sp + 168|0;
 $1 = sp + 164|0;
 $2 = sp + 160|0;
 $3 = sp + 156|0;
 $4 = sp + 152|0;
 $jmpIfDynamic = sp + 148|0;
 $rReg = sp + 144|0;
 $v = sp + 140|0;
 $zMsg = sp + 136|0;
 $addr = sp + 132|0;
 $pLeft = sp + 128|0;
 $pKeyInfo = sp + 124|0;
 $nVal = sp + 120|0;
 $pSelect = sp + 116|0;
 $pEList = sp + 112|0;
 $dest = sp + 88|0;
 $i = sp + 80|0;
 $p = sp + 76|0;
 $affinity = sp + 172|0;
 $i1 = sp + 72|0;
 $pList = sp + 68|0;
 $pItem = sp + 64|0;
 $r1 = sp + 60|0;
 $r2 = sp + 56|0;
 $r3 = sp + 52|0;
 $pE2 = sp + 48|0;
 $iValToIns = sp + 44|0;
 $pSel = sp + 40|0;
 $dest2 = sp + 16|0;
 $nReg = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $rHasNullFlag;
 HEAP32[$4>>2] = $isRowid;
 HEAP32[$jmpIfDynamic>>2] = -1;
 HEAP32[$rReg>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3GetVdbe($5)|0);
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $289 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($289|0);
 }
 $9 = HEAP32[$1>>2]|0;
 _sqlite3ExprCachePush($9);
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $12 & 32;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$v>>2]|0;
  $16 = (_sqlite3VdbeAddOp0($15,20)|0);
  HEAP32[$jmpIfDynamic>>2] = $16;
 }
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 409|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20|0)==(2);
 if ($21) {
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$jmpIfDynamic>>2]|0;
  $25 = ($24|0)>=(0);
  $26 = $25 ? 18402 : 31197;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(33);
  $31 = $30 ? 31209 : 31214;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 424|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$vararg_buffer>>2] = $26;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $31;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $34;
  $35 = (_sqlite3MPrintf($23,31221,$vararg_buffer)|0);
  HEAP32[$zMsg>>2] = $35;
  $36 = HEAP32[$v>>2]|0;
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 420|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$zMsg>>2]|0;
  (_sqlite3VdbeAddOp4($36,162,$39,0,0,$40,-1)|0);
 }
 $41 = HEAP32[$2>>2]|0;
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $cond = ($43|0)==(33);
 $44 = HEAP32[$2>>2]|0;
 do {
  if ($cond) {
   $45 = ((($44)) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$pLeft>>2] = $46;
   HEAP32[$pKeyInfo>>2] = 0;
   $47 = HEAP32[$pLeft>>2]|0;
   $48 = (_sqlite3ExprVectorSize($47)|0);
   HEAP32[$nVal>>2] = $48;
   $49 = HEAP32[$1>>2]|0;
   $50 = ((($49)) + 40|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = (($51) + 1)|0;
   HEAP32[$50>>2] = $52;
   $53 = HEAP32[$2>>2]|0;
   $54 = ((($53)) + 28|0);
   HEAP32[$54>>2] = $51;
   $55 = HEAP32[$v>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   $57 = ((($56)) + 28|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$4>>2]|0;
   $60 = ($59|0)!=(0);
   $61 = HEAP32[$nVal>>2]|0;
   $62 = $60 ? 0 : $61;
   $63 = (_sqlite3VdbeAddOp2($55,107,$58,$62)|0);
   HEAP32[$addr>>2] = $63;
   $64 = HEAP32[$4>>2]|0;
   $65 = ($64|0)!=(0);
   if ($65) {
    $70 = 0;
   } else {
    $66 = HEAP32[$1>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = HEAP32[$nVal>>2]|0;
    $69 = (_sqlite3KeyInfoAlloc($67,$68,1)|0);
    $70 = $69;
   }
   HEAP32[$pKeyInfo>>2] = $70;
   $71 = HEAP32[$2>>2]|0;
   $72 = ((($71)) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = $73 & 2048;
   $75 = ($74|0)!=(0);
   $76 = HEAP32[$2>>2]|0;
   $77 = ((($76)) + 20|0);
   L16: do {
    if ($75) {
     $78 = HEAP32[$77>>2]|0;
     HEAP32[$pSelect>>2] = $78;
     $79 = HEAP32[$pSelect>>2]|0;
     $80 = HEAP32[$79>>2]|0;
     HEAP32[$pEList>>2] = $80;
     $81 = HEAP32[$pEList>>2]|0;
     $82 = HEAP32[$81>>2]|0;
     $83 = HEAP32[$nVal>>2]|0;
     $84 = ($82|0)==($83|0);
     if ($84) {
      $85 = HEAP32[$2>>2]|0;
      $86 = ((($85)) + 28|0);
      $87 = HEAP32[$86>>2]|0;
      _sqlite3SelectDestInit($dest,11,$87);
      $88 = HEAP32[$1>>2]|0;
      $89 = HEAP32[$2>>2]|0;
      $90 = (_exprINAffinity($88,$89)|0);
      $91 = ((($dest)) + 4|0);
      HEAP32[$91>>2] = $90;
      $92 = HEAP32[$pSelect>>2]|0;
      $93 = ((($92)) + 12|0);
      HEAP32[$93>>2] = 0;
      $94 = HEAP32[$1>>2]|0;
      $95 = HEAP32[$pSelect>>2]|0;
      $96 = (_sqlite3Select($94,$95,$dest)|0);
      $97 = ($96|0)!=(0);
      $98 = HEAP32[$1>>2]|0;
      $99 = HEAP32[$98>>2]|0;
      $100 = ((($dest)) + 4|0);
      $101 = HEAP32[$100>>2]|0;
      _sqlite3DbFree($99,$101);
      if ($97) {
       $102 = HEAP32[$pKeyInfo>>2]|0;
       _sqlite3KeyInfoUnref($102);
       HEAP32[$0>>2] = 0;
       $289 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($289|0);
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $103 = HEAP32[$i>>2]|0;
       $104 = HEAP32[$nVal>>2]|0;
       $105 = ($103|0)<($104|0);
       if (!($105)) {
        break L16;
       }
       $106 = HEAP32[$pLeft>>2]|0;
       $107 = HEAP32[$i>>2]|0;
       $108 = (_sqlite3VectorFieldSubexpr($106,$107)|0);
       HEAP32[$p>>2] = $108;
       $109 = HEAP32[$1>>2]|0;
       $110 = HEAP32[$p>>2]|0;
       $111 = HEAP32[$i>>2]|0;
       $112 = HEAP32[$pEList>>2]|0;
       $113 = ((($112)) + 4|0);
       $114 = HEAP32[$113>>2]|0;
       $115 = (($114) + (($111*20)|0)|0);
       $116 = HEAP32[$115>>2]|0;
       $117 = (_sqlite3BinaryCompareCollSeq($109,$110,$116)|0);
       $118 = HEAP32[$i>>2]|0;
       $119 = HEAP32[$pKeyInfo>>2]|0;
       $120 = ((($119)) + 20|0);
       $121 = (($120) + ($118<<2)|0);
       HEAP32[$121>>2] = $117;
       $122 = HEAP32[$i>>2]|0;
       $123 = (($122) + 1)|0;
       HEAP32[$i>>2] = $123;
      }
     }
    } else {
     $124 = HEAP32[$77>>2]|0;
     $125 = ($124|0)!=(0|0);
     if ($125) {
      $126 = HEAP32[$2>>2]|0;
      $127 = ((($126)) + 20|0);
      $128 = HEAP32[$127>>2]|0;
      HEAP32[$pList>>2] = $128;
      $129 = HEAP32[$pLeft>>2]|0;
      $130 = (_sqlite3ExprAffinity($129)|0);
      HEAP8[$affinity>>0] = $130;
      $131 = HEAP8[$affinity>>0]|0;
      $132 = ($131<<24>>24)!=(0);
      $$ = $132 ? $130 : 65;
      HEAP8[$affinity>>0] = $$;
      $133 = HEAP32[$pKeyInfo>>2]|0;
      $134 = ($133|0)!=(0|0);
      if ($134) {
       $135 = HEAP32[$1>>2]|0;
       $136 = HEAP32[$2>>2]|0;
       $137 = ((($136)) + 12|0);
       $138 = HEAP32[$137>>2]|0;
       $139 = (_sqlite3ExprCollSeq($135,$138)|0);
       $140 = HEAP32[$pKeyInfo>>2]|0;
       $141 = ((($140)) + 20|0);
       HEAP32[$141>>2] = $139;
      }
      $142 = HEAP32[$1>>2]|0;
      $143 = (_sqlite3GetTempReg($142)|0);
      HEAP32[$r1>>2] = $143;
      $144 = HEAP32[$1>>2]|0;
      $145 = (_sqlite3GetTempReg($144)|0);
      HEAP32[$r2>>2] = $145;
      $146 = HEAP32[$4>>2]|0;
      $147 = ($146|0)!=(0);
      if ($147) {
       $148 = HEAP32[$v>>2]|0;
       $149 = HEAP32[$r2>>2]|0;
       (_sqlite3VdbeAddOp2($148,79,0,$149)|0);
      }
      $150 = HEAP32[$pList>>2]|0;
      $151 = HEAP32[$150>>2]|0;
      HEAP32[$i1>>2] = $151;
      $152 = HEAP32[$pList>>2]|0;
      $153 = ((($152)) + 4|0);
      $154 = HEAP32[$153>>2]|0;
      HEAP32[$pItem>>2] = $154;
      while(1) {
       $155 = HEAP32[$i1>>2]|0;
       $156 = ($155|0)>(0);
       if (!($156)) {
        break;
       }
       $157 = HEAP32[$pItem>>2]|0;
       $158 = HEAP32[$157>>2]|0;
       HEAP32[$pE2>>2] = $158;
       $159 = HEAP32[$jmpIfDynamic>>2]|0;
       $160 = ($159|0)>=(0);
       if ($160) {
        $161 = HEAP32[$pE2>>2]|0;
        $162 = (_sqlite3ExprIsConstant($161)|0);
        $163 = ($162|0)!=(0);
        if (!($163)) {
         $164 = HEAP32[$v>>2]|0;
         $165 = HEAP32[$jmpIfDynamic>>2]|0;
         (_sqlite3VdbeChangeToNoop($164,$165)|0);
         HEAP32[$jmpIfDynamic>>2] = -1;
        }
       }
       $166 = HEAP32[$4>>2]|0;
       $167 = ($166|0)!=(0);
       if ($167) {
        $168 = HEAP32[$pE2>>2]|0;
        $169 = (_sqlite3ExprIsInteger($168,$iValToIns)|0);
        $170 = ($169|0)!=(0);
        if ($170) {
         $171 = HEAP32[$v>>2]|0;
         $172 = HEAP32[$2>>2]|0;
         $173 = ((($172)) + 28|0);
         $174 = HEAP32[$173>>2]|0;
         $175 = HEAP32[$r2>>2]|0;
         $176 = HEAP32[$iValToIns>>2]|0;
         (_sqlite3VdbeAddOp3($171,116,$174,$175,$176)|0);
        } else {
         label = 30;
        }
       } else {
        label = 30;
       }
       do {
        if ((label|0) == 30) {
         label = 0;
         $177 = HEAP32[$1>>2]|0;
         $178 = HEAP32[$pE2>>2]|0;
         $179 = HEAP32[$r1>>2]|0;
         $180 = (_sqlite3ExprCodeTarget($177,$178,$179)|0);
         HEAP32[$r3>>2] = $180;
         $181 = HEAP32[$4>>2]|0;
         $182 = ($181|0)!=(0);
         $183 = HEAP32[$v>>2]|0;
         $184 = HEAP32[$r3>>2]|0;
         if ($182) {
          $185 = HEAP32[$v>>2]|0;
          $186 = (_sqlite3VdbeCurrentAddr($185)|0);
          $187 = (($186) + 2)|0;
          (_sqlite3VdbeAddOp2($183,17,$184,$187)|0);
          $188 = HEAP32[$v>>2]|0;
          $189 = HEAP32[$2>>2]|0;
          $190 = ((($189)) + 28|0);
          $191 = HEAP32[$190>>2]|0;
          $192 = HEAP32[$r2>>2]|0;
          $193 = HEAP32[$r3>>2]|0;
          (_sqlite3VdbeAddOp3($188,115,$191,$192,$193)|0);
          break;
         } else {
          $194 = HEAP32[$r2>>2]|0;
          (_sqlite3VdbeAddOp4($183,99,$184,1,$194,$affinity,1)|0);
          $195 = HEAP32[$1>>2]|0;
          $196 = HEAP32[$r3>>2]|0;
          _sqlite3ExprCacheAffinityChange($195,$196,1);
          $197 = HEAP32[$v>>2]|0;
          $198 = HEAP32[$2>>2]|0;
          $199 = ((($198)) + 28|0);
          $200 = HEAP32[$199>>2]|0;
          $201 = HEAP32[$r2>>2]|0;
          (_sqlite3VdbeAddOp2($197,126,$200,$201)|0);
          break;
         }
        }
       } while(0);
       $202 = HEAP32[$i1>>2]|0;
       $203 = (($202) + -1)|0;
       HEAP32[$i1>>2] = $203;
       $204 = HEAP32[$pItem>>2]|0;
       $205 = ((($204)) + 20|0);
       HEAP32[$pItem>>2] = $205;
      }
      $206 = HEAP32[$1>>2]|0;
      $207 = HEAP32[$r1>>2]|0;
      _sqlite3ReleaseTempReg($206,$207);
      $208 = HEAP32[$1>>2]|0;
      $209 = HEAP32[$r2>>2]|0;
      _sqlite3ReleaseTempReg($208,$209);
     }
    }
   } while(0);
   $210 = HEAP32[$pKeyInfo>>2]|0;
   $211 = ($210|0)!=(0|0);
   if ($211) {
    $212 = HEAP32[$v>>2]|0;
    $213 = HEAP32[$addr>>2]|0;
    $214 = HEAP32[$pKeyInfo>>2]|0;
    _sqlite3VdbeChangeP4($212,$213,$214,-6);
   }
  } else {
   $215 = ((($44)) + 20|0);
   $216 = HEAP32[$215>>2]|0;
   HEAP32[$pSel>>2] = $216;
   $217 = HEAP32[$2>>2]|0;
   $218 = HEAP8[$217>>0]|0;
   $219 = $218&255;
   $220 = ($219|0)==(119);
   if ($220) {
    $221 = HEAP32[$pSel>>2]|0;
    $222 = HEAP32[$221>>2]|0;
    $223 = HEAP32[$222>>2]|0;
    $224 = $223;
   } else {
    $224 = 1;
   }
   HEAP32[$nReg>>2] = $224;
   $225 = HEAP32[$1>>2]|0;
   $226 = ((($225)) + 44|0);
   $227 = HEAP32[$226>>2]|0;
   $228 = (($227) + 1)|0;
   _sqlite3SelectDestInit($dest2,0,$228);
   $229 = HEAP32[$nReg>>2]|0;
   $230 = HEAP32[$1>>2]|0;
   $231 = ((($230)) + 44|0);
   $232 = HEAP32[$231>>2]|0;
   $233 = (($232) + ($229))|0;
   HEAP32[$231>>2] = $233;
   $234 = HEAP32[$2>>2]|0;
   $235 = HEAP8[$234>>0]|0;
   $236 = $235&255;
   $237 = ($236|0)==(119);
   if ($237) {
    HEAP8[$dest2>>0] = 10;
    $238 = ((($dest2)) + 8|0);
    $239 = HEAP32[$238>>2]|0;
    $240 = ((($dest2)) + 12|0);
    HEAP32[$240>>2] = $239;
    $241 = HEAP32[$nReg>>2]|0;
    $242 = ((($dest2)) + 16|0);
    HEAP32[$242>>2] = $241;
    $243 = HEAP32[$v>>2]|0;
    $244 = ((($dest2)) + 8|0);
    $245 = HEAP32[$244>>2]|0;
    $246 = ((($dest2)) + 8|0);
    $247 = HEAP32[$246>>2]|0;
    $248 = HEAP32[$nReg>>2]|0;
    $249 = (($247) + ($248))|0;
    $250 = (($249) - 1)|0;
    (_sqlite3VdbeAddOp3($243,79,0,$245,$250)|0);
   } else {
    HEAP8[$dest2>>0] = 3;
    $251 = HEAP32[$v>>2]|0;
    $252 = ((($dest2)) + 8|0);
    $253 = HEAP32[$252>>2]|0;
    (_sqlite3VdbeAddOp2($251,76,0,$253)|0);
   }
   $254 = HEAP32[$1>>2]|0;
   $255 = HEAP32[$254>>2]|0;
   $256 = HEAP32[$pSel>>2]|0;
   $257 = ((($256)) + 56|0);
   $258 = HEAP32[$257>>2]|0;
   _sqlite3ExprDelete($255,$258);
   $259 = HEAP32[$1>>2]|0;
   $260 = HEAP32[$259>>2]|0;
   $261 = (_sqlite3ExprAlloc($260,134,(4284),0)|0);
   $262 = HEAP32[$pSel>>2]|0;
   $263 = ((($262)) + 56|0);
   HEAP32[$263>>2] = $261;
   $264 = HEAP32[$pSel>>2]|0;
   $265 = ((($264)) + 12|0);
   HEAP32[$265>>2] = 0;
   $266 = HEAP32[$pSel>>2]|0;
   $267 = ((($266)) + 8|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = $268 & -1025;
   HEAP32[$267>>2] = $269;
   $270 = HEAP32[$1>>2]|0;
   $271 = HEAP32[$pSel>>2]|0;
   $272 = (_sqlite3Select($270,$271,$dest2)|0);
   $273 = ($272|0)!=(0);
   if (!($273)) {
    $274 = ((($dest2)) + 8|0);
    $275 = HEAP32[$274>>2]|0;
    HEAP32[$rReg>>2] = $275;
    break;
   }
   HEAP32[$0>>2] = 0;
   $289 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($289|0);
  }
 } while(0);
 $276 = HEAP32[$3>>2]|0;
 $277 = ($276|0)!=(0);
 if ($277) {
  $278 = HEAP32[$v>>2]|0;
  $279 = HEAP32[$2>>2]|0;
  $280 = ((($279)) + 28|0);
  $281 = HEAP32[$280>>2]|0;
  $282 = HEAP32[$3>>2]|0;
  _sqlite3SetHasNullFlag($278,$281,$282);
 }
 $283 = HEAP32[$jmpIfDynamic>>2]|0;
 $284 = ($283|0)>=(0);
 if ($284) {
  $285 = HEAP32[$v>>2]|0;
  $286 = HEAP32[$jmpIfDynamic>>2]|0;
  _sqlite3VdbeJumpHere($285,$286);
 }
 $287 = HEAP32[$1>>2]|0;
 _sqlite3ExprCachePop($287);
 $288 = HEAP32[$rReg>>2]|0;
 HEAP32[$0>>2] = $288;
 $289 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($289|0);
}
function _sqlite3ExprCodeIN($pParse,$pExpr,$destIfFalse,$destIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $destIfFalse = $destIfFalse|0;
 $destIfNull = $destIfNull|0;
 var $$rRhsHasNull = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $addrTop = 0, $addrTruthOp = 0, $aiMap = 0, $destNotNull = 0, $destStep2 = 0, $destStep6 = 0, $eType = 0, $i = 0, $iDummy = 0, $ii = 0, $labelOk = 0, $nVector = 0, $or$cond = 0, $p = 0, $p1 = 0, $pColl = 0, $pColl2 = 0, $pLeft = 0, $pList = 0, $r2 = 0;
 var $r3 = 0, $rLhs = 0, $rLhsOrig = 0, $rRhsHasNull = 0, $regCkNull = 0, $regToFree = 0, $v = 0, $zAff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $rRhsHasNull = sp + 104|0;
 $eType = sp + 100|0;
 $rLhs = sp + 96|0;
 $rLhsOrig = sp + 92|0;
 $v = sp + 88|0;
 $aiMap = sp + 84|0;
 $zAff = sp + 80|0;
 $nVector = sp + 76|0;
 $iDummy = sp + 72|0;
 $pLeft = sp + 68|0;
 $i = sp + 64|0;
 $destStep2 = sp + 60|0;
 $destStep6 = sp + 56|0;
 $addrTruthOp = sp + 52|0;
 $destNotNull = sp + 48|0;
 $addrTop = sp + 44|0;
 $pList = sp + 40|0;
 $pColl = sp + 36|0;
 $labelOk = sp + 32|0;
 $r2 = sp + 28|0;
 $regToFree = sp + 24|0;
 $regCkNull = sp + 20|0;
 $ii = sp + 16|0;
 $p = sp + 12|0;
 $p1 = sp + 8|0;
 $pColl2 = sp + 4|0;
 $r3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $destIfFalse;
 HEAP32[$3>>2] = $destIfNull;
 HEAP32[$rRhsHasNull>>2] = 0;
 HEAP32[$aiMap>>2] = 0;
 HEAP32[$zAff>>2] = 0;
 HEAP32[$destStep6>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pLeft>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3ExprCheckIN($7,$8)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (_exprINAffinity($11,$12)|0);
 HEAP32[$zAff>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3ExprVectorSize($16)|0);
 HEAP32[$nVector>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$nVector>>2]|0;
 $21 = ($20*5)|0;
 $22 = (($21) + 1)|0;
 $23 = (_sqlite3DbMallocZero($19,$22,0)|0);
 HEAP32[$aiMap>>2] = $23;
 $24 = HEAP32[$0>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = ((($25)) + 69|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = ($27<<24>>24)!=(0);
 if (!($28)) {
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 8|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$v>>2] = $31;
  $32 = HEAP32[$0>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = HEAP32[$3>>2]|0;
  $36 = ($34|0)==($35|0);
  $$rRhsHasNull = $36 ? 0 : $rRhsHasNull;
  $37 = HEAP32[$aiMap>>2]|0;
  $38 = (_sqlite3FindInIndex($32,$33,3,$$rRhsHasNull,$37)|0);
  HEAP32[$eType>>2] = $38;
  $39 = HEAP32[$0>>2]|0;
  _sqlite3ExprCachePush($39);
  $40 = HEAP32[$0>>2]|0;
  $41 = HEAP32[$pLeft>>2]|0;
  $42 = (_exprCodeVector($40,$41,$iDummy)|0);
  HEAP32[$rLhsOrig>>2] = $42;
  HEAP32[$i>>2] = 0;
  while(1) {
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$nVector>>2]|0;
   $45 = ($43|0)<($44|0);
   if (!($45)) {
    break;
   }
   $46 = HEAP32[$i>>2]|0;
   $47 = HEAP32[$aiMap>>2]|0;
   $48 = (($47) + ($46<<2)|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$i>>2]|0;
   $51 = ($49|0)==($50|0);
   if (!($51)) {
    break;
   }
   $52 = HEAP32[$i>>2]|0;
   $53 = (($52) + 1)|0;
   HEAP32[$i>>2] = $53;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = HEAP32[$nVector>>2]|0;
  $56 = ($54|0)==($55|0);
  L11: do {
   if ($56) {
    $57 = HEAP32[$rLhsOrig>>2]|0;
    HEAP32[$rLhs>>2] = $57;
   } else {
    $58 = HEAP32[$0>>2]|0;
    $59 = HEAP32[$nVector>>2]|0;
    $60 = (_sqlite3GetTempRange($58,$59)|0);
    HEAP32[$rLhs>>2] = $60;
    HEAP32[$i>>2] = 0;
    while(1) {
     $61 = HEAP32[$i>>2]|0;
     $62 = HEAP32[$nVector>>2]|0;
     $63 = ($61|0)<($62|0);
     if (!($63)) {
      break L11;
     }
     $64 = HEAP32[$v>>2]|0;
     $65 = HEAP32[$rLhsOrig>>2]|0;
     $66 = HEAP32[$i>>2]|0;
     $67 = (($65) + ($66))|0;
     $68 = HEAP32[$rLhs>>2]|0;
     $69 = HEAP32[$i>>2]|0;
     $70 = HEAP32[$aiMap>>2]|0;
     $71 = (($70) + ($69<<2)|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = (($68) + ($72))|0;
     (_sqlite3VdbeAddOp3($64,84,$67,$73,0)|0);
     $74 = HEAP32[$i>>2]|0;
     $75 = (($74) + 1)|0;
     HEAP32[$i>>2] = $75;
    }
   }
  } while(0);
  $76 = HEAP32[$eType>>2]|0;
  $77 = ($76|0)==(5);
  L18: do {
   if ($77) {
    $78 = HEAP32[$1>>2]|0;
    $79 = ((($78)) + 20|0);
    $80 = HEAP32[$79>>2]|0;
    HEAP32[$pList>>2] = $80;
    $81 = HEAP32[$0>>2]|0;
    $82 = HEAP32[$1>>2]|0;
    $83 = ((($82)) + 12|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = (_sqlite3ExprCollSeq($81,$84)|0);
    HEAP32[$pColl>>2] = $85;
    $86 = HEAP32[$v>>2]|0;
    $87 = (_sqlite3VdbeMakeLabel($86)|0);
    HEAP32[$labelOk>>2] = $87;
    HEAP32[$regCkNull>>2] = 0;
    $88 = HEAP32[$3>>2]|0;
    $89 = HEAP32[$2>>2]|0;
    $90 = ($88|0)!=($89|0);
    if ($90) {
     $91 = HEAP32[$0>>2]|0;
     $92 = (_sqlite3GetTempReg($91)|0);
     HEAP32[$regCkNull>>2] = $92;
     $93 = HEAP32[$v>>2]|0;
     $94 = HEAP32[$rLhs>>2]|0;
     $95 = HEAP32[$rLhs>>2]|0;
     $96 = HEAP32[$regCkNull>>2]|0;
     (_sqlite3VdbeAddOp3($93,43,$94,$95,$96)|0);
    }
    HEAP32[$ii>>2] = 0;
    while(1) {
     $97 = HEAP32[$ii>>2]|0;
     $98 = HEAP32[$pList>>2]|0;
     $99 = HEAP32[$98>>2]|0;
     $100 = ($97|0)<($99|0);
     if (!($100)) {
      break;
     }
     $101 = HEAP32[$0>>2]|0;
     $102 = HEAP32[$ii>>2]|0;
     $103 = HEAP32[$pList>>2]|0;
     $104 = ((($103)) + 4|0);
     $105 = HEAP32[$104>>2]|0;
     $106 = (($105) + (($102*20)|0)|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = (_sqlite3ExprCodeTemp($101,$107,$regToFree)|0);
     HEAP32[$r2>>2] = $108;
     $109 = HEAP32[$regCkNull>>2]|0;
     $110 = ($109|0)!=(0);
     if ($110) {
      $111 = HEAP32[$ii>>2]|0;
      $112 = HEAP32[$pList>>2]|0;
      $113 = ((($112)) + 4|0);
      $114 = HEAP32[$113>>2]|0;
      $115 = (($114) + (($111*20)|0)|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = (_sqlite3ExprCanBeNull($116)|0);
      $118 = ($117|0)!=(0);
      if ($118) {
       $119 = HEAP32[$v>>2]|0;
       $120 = HEAP32[$regCkNull>>2]|0;
       $121 = HEAP32[$r2>>2]|0;
       $122 = HEAP32[$regCkNull>>2]|0;
       (_sqlite3VdbeAddOp3($119,43,$120,$121,$122)|0);
      }
     }
     $123 = HEAP32[$ii>>2]|0;
     $124 = HEAP32[$pList>>2]|0;
     $125 = HEAP32[$124>>2]|0;
     $126 = (($125) - 1)|0;
     $127 = ($123|0)<($126|0);
     if ($127) {
      label = 22;
     } else {
      $128 = HEAP32[$3>>2]|0;
      $129 = HEAP32[$2>>2]|0;
      $130 = ($128|0)!=($129|0);
      if ($130) {
       label = 22;
      } else {
       $139 = HEAP32[$v>>2]|0;
       $140 = HEAP32[$rLhs>>2]|0;
       $141 = HEAP32[$2>>2]|0;
       $142 = HEAP32[$r2>>2]|0;
       $143 = HEAP32[$pColl>>2]|0;
       (_sqlite3VdbeAddOp4($139,36,$140,$141,$142,$143,-4)|0);
       $144 = HEAP32[$v>>2]|0;
       $145 = HEAP32[$zAff>>2]|0;
       $146 = HEAP8[$145>>0]|0;
       $147 = $146 << 24 >> 24;
       $148 = $147 | 16;
       $149 = $148&255;
       _sqlite3VdbeChangeP5($144,$149);
      }
     }
     if ((label|0) == 22) {
      label = 0;
      $131 = HEAP32[$v>>2]|0;
      $132 = HEAP32[$rLhs>>2]|0;
      $133 = HEAP32[$labelOk>>2]|0;
      $134 = HEAP32[$r2>>2]|0;
      $135 = HEAP32[$pColl>>2]|0;
      (_sqlite3VdbeAddOp4($131,37,$132,$133,$134,$135,-4)|0);
      $136 = HEAP32[$v>>2]|0;
      $137 = HEAP32[$zAff>>2]|0;
      $138 = HEAP8[$137>>0]|0;
      _sqlite3VdbeChangeP5($136,$138);
     }
     $150 = HEAP32[$0>>2]|0;
     $151 = HEAP32[$regToFree>>2]|0;
     _sqlite3ReleaseTempReg($150,$151);
     $152 = HEAP32[$ii>>2]|0;
     $153 = (($152) + 1)|0;
     HEAP32[$ii>>2] = $153;
    }
    $154 = HEAP32[$regCkNull>>2]|0;
    $155 = ($154|0)!=(0);
    if ($155) {
     $156 = HEAP32[$v>>2]|0;
     $157 = HEAP32[$regCkNull>>2]|0;
     $158 = HEAP32[$3>>2]|0;
     (_sqlite3VdbeAddOp2($156,34,$157,$158)|0);
     $159 = HEAP32[$v>>2]|0;
     $160 = HEAP32[$2>>2]|0;
     (_sqlite3VdbeGoto($159,$160)|0);
    }
    $161 = HEAP32[$v>>2]|0;
    $162 = HEAP32[$labelOk>>2]|0;
    _sqlite3VdbeResolveLabel($161,$162);
    $163 = HEAP32[$0>>2]|0;
    $164 = HEAP32[$regCkNull>>2]|0;
    _sqlite3ReleaseTempReg($163,$164);
   } else {
    $165 = HEAP32[$3>>2]|0;
    $166 = HEAP32[$2>>2]|0;
    $167 = ($165|0)==($166|0);
    if ($167) {
     $168 = HEAP32[$2>>2]|0;
     HEAP32[$destStep2>>2] = $168;
    } else {
     $169 = HEAP32[$v>>2]|0;
     $170 = (_sqlite3VdbeMakeLabel($169)|0);
     HEAP32[$destStep6>>2] = $170;
     HEAP32[$destStep2>>2] = $170;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $171 = HEAP32[$i>>2]|0;
     $172 = HEAP32[$nVector>>2]|0;
     $173 = ($171|0)<($172|0);
     if (!($173)) {
      break;
     }
     $174 = HEAP32[$1>>2]|0;
     $175 = ((($174)) + 12|0);
     $176 = HEAP32[$175>>2]|0;
     $177 = HEAP32[$i>>2]|0;
     $178 = (_sqlite3VectorFieldSubexpr($176,$177)|0);
     HEAP32[$p>>2] = $178;
     $179 = HEAP32[$p>>2]|0;
     $180 = (_sqlite3ExprCanBeNull($179)|0);
     $181 = ($180|0)!=(0);
     if ($181) {
      $182 = HEAP32[$v>>2]|0;
      $183 = HEAP32[$rLhs>>2]|0;
      $184 = HEAP32[$i>>2]|0;
      $185 = (($183) + ($184))|0;
      $186 = HEAP32[$destStep2>>2]|0;
      (_sqlite3VdbeAddOp2($182,34,$185,$186)|0);
     }
     $187 = HEAP32[$i>>2]|0;
     $188 = (($187) + 1)|0;
     HEAP32[$i>>2] = $188;
    }
    $189 = HEAP32[$eType>>2]|0;
    $190 = ($189|0)==(1);
    $191 = HEAP32[$v>>2]|0;
    do {
     if ($190) {
      $192 = HEAP32[$1>>2]|0;
      $193 = ((($192)) + 28|0);
      $194 = HEAP32[$193>>2]|0;
      $195 = HEAP32[$2>>2]|0;
      $196 = HEAP32[$rLhs>>2]|0;
      (_sqlite3VdbeAddOp3($191,32,$194,$195,$196)|0);
      $197 = HEAP32[$v>>2]|0;
      $198 = (_sqlite3VdbeAddOp0($197,13)|0);
      HEAP32[$addrTruthOp>>2] = $198;
     } else {
      $199 = HEAP32[$rLhs>>2]|0;
      $200 = HEAP32[$nVector>>2]|0;
      $201 = HEAP32[$zAff>>2]|0;
      $202 = HEAP32[$nVector>>2]|0;
      (_sqlite3VdbeAddOp4($191,98,$199,$200,0,$201,$202)|0);
      $203 = HEAP32[$2>>2]|0;
      $204 = HEAP32[$3>>2]|0;
      $205 = ($203|0)==($204|0);
      $206 = HEAP32[$v>>2]|0;
      $207 = HEAP32[$1>>2]|0;
      $208 = ((($207)) + 28|0);
      $209 = HEAP32[$208>>2]|0;
      if ($205) {
       $210 = HEAP32[$2>>2]|0;
       $211 = HEAP32[$rLhs>>2]|0;
       $212 = HEAP32[$nVector>>2]|0;
       (_sqlite3VdbeAddOp4Int($206,30,$209,$210,$211,$212)|0);
       break L18;
      } else {
       $213 = HEAP32[$rLhs>>2]|0;
       $214 = HEAP32[$nVector>>2]|0;
       $215 = (_sqlite3VdbeAddOp4Int($206,31,$209,0,$213,$214)|0);
       HEAP32[$addrTruthOp>>2] = $215;
       break;
      }
     }
    } while(0);
    $216 = HEAP32[$rRhsHasNull>>2]|0;
    $217 = ($216|0)!=(0);
    $218 = HEAP32[$nVector>>2]|0;
    $219 = ($218|0)==(1);
    $or$cond = $217 & $219;
    if ($or$cond) {
     $220 = HEAP32[$v>>2]|0;
     $221 = HEAP32[$rRhsHasNull>>2]|0;
     $222 = HEAP32[$2>>2]|0;
     (_sqlite3VdbeAddOp2($220,35,$221,$222)|0);
    }
    $223 = HEAP32[$2>>2]|0;
    $224 = HEAP32[$3>>2]|0;
    $225 = ($223|0)==($224|0);
    if ($225) {
     $226 = HEAP32[$v>>2]|0;
     $227 = HEAP32[$2>>2]|0;
     (_sqlite3VdbeGoto($226,$227)|0);
    }
    $228 = HEAP32[$destStep6>>2]|0;
    $229 = ($228|0)!=(0);
    if ($229) {
     $230 = HEAP32[$v>>2]|0;
     $231 = HEAP32[$destStep6>>2]|0;
     _sqlite3VdbeResolveLabel($230,$231);
    }
    $232 = HEAP32[$v>>2]|0;
    $233 = HEAP32[$1>>2]|0;
    $234 = ((($233)) + 28|0);
    $235 = HEAP32[$234>>2]|0;
    $236 = HEAP32[$2>>2]|0;
    $237 = (_sqlite3VdbeAddOp2($232,57,$235,$236)|0);
    HEAP32[$addrTop>>2] = $237;
    $238 = HEAP32[$nVector>>2]|0;
    $239 = ($238|0)>(1);
    if ($239) {
     $240 = HEAP32[$v>>2]|0;
     $241 = (_sqlite3VdbeMakeLabel($240)|0);
     HEAP32[$destNotNull>>2] = $241;
    } else {
     $242 = HEAP32[$2>>2]|0;
     HEAP32[$destNotNull>>2] = $242;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $243 = HEAP32[$i>>2]|0;
     $244 = HEAP32[$nVector>>2]|0;
     $245 = ($243|0)<($244|0);
     if (!($245)) {
      break;
     }
     $246 = HEAP32[$0>>2]|0;
     $247 = (_sqlite3GetTempReg($246)|0);
     HEAP32[$r3>>2] = $247;
     $248 = HEAP32[$pLeft>>2]|0;
     $249 = HEAP32[$i>>2]|0;
     $250 = (_sqlite3VectorFieldSubexpr($248,$249)|0);
     HEAP32[$p1>>2] = $250;
     $251 = HEAP32[$0>>2]|0;
     $252 = HEAP32[$p1>>2]|0;
     $253 = (_sqlite3ExprCollSeq($251,$252)|0);
     HEAP32[$pColl2>>2] = $253;
     $254 = HEAP32[$v>>2]|0;
     $255 = HEAP32[$1>>2]|0;
     $256 = ((($255)) + 28|0);
     $257 = HEAP32[$256>>2]|0;
     $258 = HEAP32[$i>>2]|0;
     $259 = HEAP32[$r3>>2]|0;
     (_sqlite3VdbeAddOp3($254,96,$257,$258,$259)|0);
     $260 = HEAP32[$v>>2]|0;
     $261 = HEAP32[$rLhs>>2]|0;
     $262 = HEAP32[$i>>2]|0;
     $263 = (($261) + ($262))|0;
     $264 = HEAP32[$destNotNull>>2]|0;
     $265 = HEAP32[$r3>>2]|0;
     $266 = HEAP32[$pColl2>>2]|0;
     (_sqlite3VdbeAddOp4($260,36,$263,$264,$265,$266,-4)|0);
     $267 = HEAP32[$0>>2]|0;
     $268 = HEAP32[$r3>>2]|0;
     _sqlite3ReleaseTempReg($267,$268);
     $269 = HEAP32[$i>>2]|0;
     $270 = (($269) + 1)|0;
     HEAP32[$i>>2] = $270;
    }
    $271 = HEAP32[$v>>2]|0;
    $272 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($271,13,0,$272)|0);
    $273 = HEAP32[$nVector>>2]|0;
    $274 = ($273|0)>(1);
    if ($274) {
     $275 = HEAP32[$v>>2]|0;
     $276 = HEAP32[$destNotNull>>2]|0;
     _sqlite3VdbeResolveLabel($275,$276);
     $277 = HEAP32[$v>>2]|0;
     $278 = HEAP32[$1>>2]|0;
     $279 = ((($278)) + 28|0);
     $280 = HEAP32[$279>>2]|0;
     $281 = HEAP32[$addrTop>>2]|0;
     $282 = (($281) + 1)|0;
     (_sqlite3VdbeAddOp2($277,7,$280,$282)|0);
     $283 = HEAP32[$v>>2]|0;
     $284 = HEAP32[$2>>2]|0;
     (_sqlite3VdbeAddOp2($283,13,0,$284)|0);
    }
    $285 = HEAP32[$v>>2]|0;
    $286 = HEAP32[$addrTruthOp>>2]|0;
    _sqlite3VdbeJumpHere($285,$286);
   }
  } while(0);
  $287 = HEAP32[$rLhs>>2]|0;
  $288 = HEAP32[$rLhsOrig>>2]|0;
  $289 = ($287|0)!=($288|0);
  if ($289) {
   $290 = HEAP32[$0>>2]|0;
   $291 = HEAP32[$rLhs>>2]|0;
   _sqlite3ReleaseTempReg($290,$291);
  }
  $292 = HEAP32[$0>>2]|0;
  _sqlite3ExprCachePop($292);
 }
 $293 = HEAP32[$0>>2]|0;
 $294 = HEAP32[$293>>2]|0;
 $295 = HEAP32[$aiMap>>2]|0;
 _sqlite3DbFree($294,$295);
 $296 = HEAP32[$0>>2]|0;
 $297 = HEAP32[$296>>2]|0;
 $298 = HEAP32[$zAff>>2]|0;
 _sqlite3DbFree($297,$298);
 STACKTOP = sp;return;
}
function _exprCodeBetween($pParse,$pExpr,$dest,$xJump,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $xJump = $xJump|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $compLeft = 0, $compRight = 0, $exprAnd = 0, $exprX = 0, $regFree1 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 216|0;
 $1 = sp + 212|0;
 $2 = sp + 208|0;
 $3 = sp + 204|0;
 $4 = sp + 200|0;
 $exprAnd = sp + 152|0;
 $compLeft = sp + 104|0;
 $compRight = sp + 56|0;
 $exprX = sp + 8|0;
 $regFree1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $xJump;
 HEAP32[$4>>2] = $jumpIfNull;
 HEAP32[$regFree1>>2] = 0;
 dest=$compLeft; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 dest=$compRight; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 dest=$exprAnd; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 dest=$exprX; src=$7; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 HEAP8[$exprAnd>>0] = 28;
 $8 = ((($exprAnd)) + 12|0);
 HEAP32[$8>>2] = $compLeft;
 $9 = ((($exprAnd)) + 16|0);
 HEAP32[$9>>2] = $compRight;
 HEAP8[$compLeft>>0] = 41;
 $10 = ((($compLeft)) + 12|0);
 HEAP32[$10>>2] = $exprX;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($compLeft)) + 16|0);
 HEAP32[$17>>2] = $16;
 HEAP8[$compRight>>0] = 39;
 $18 = ((($compRight)) + 12|0);
 HEAP32[$18>>2] = $exprX;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 20|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ((($compRight)) + 16|0);
 HEAP32[$26>>2] = $25;
 $27 = HEAP32[$0>>2]|0;
 $28 = (_exprCodeVector($27,$exprX,$regFree1)|0);
 _exprToRegister($exprX,$28);
 $29 = HEAP32[$3>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  $31 = HEAP32[$3>>2]|0;
  $32 = HEAP32[$0>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$4>>2]|0;
  FUNCTION_TABLE_viiii[$31 & 255]($32,$exprAnd,$33,$34);
  $40 = HEAP32[$0>>2]|0;
  $41 = HEAP32[$regFree1>>2]|0;
  _sqlite3ReleaseTempReg($40,$41);
  STACKTOP = sp;return;
 } else {
  $35 = ((($exprX)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = $36 | 1;
  HEAP32[$35>>2] = $37;
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  (_sqlite3ExprCodeTarget($38,$exprAnd,$39)|0);
  $40 = HEAP32[$0>>2]|0;
  $41 = HEAP32[$regFree1>>2]|0;
  _sqlite3ReleaseTempReg($40,$41);
  STACKTOP = sp;return;
 }
}
function _exprCodeVector($pParse,$p,$piFreeable) {
 $pParse = $pParse|0;
 $p = $p|0;
 $piFreeable = $piFreeable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iResult = 0, $nResult = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $iResult = sp + 8|0;
 $nResult = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $piFreeable;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprVectorSize($3)|0);
 HEAP32[$nResult>>2] = $4;
 $5 = HEAP32[$nResult>>2]|0;
 $6 = ($5|0)==(1);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3ExprCodeTemp($7,$8,$9)|0);
  HEAP32[$iResult>>2] = $10;
  $44 = HEAP32[$iResult>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $11 = HEAP32[$2>>2]|0;
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(119);
 $16 = HEAP32[$0>>2]|0;
 if ($15) {
  $17 = HEAP32[$1>>2]|0;
  $18 = (_sqlite3CodeSubselect($16,$17,0,0)|0);
  HEAP32[$iResult>>2] = $18;
  $44 = HEAP32[$iResult>>2]|0;
  STACKTOP = sp;return ($44|0);
 }
 $19 = ((($16)) + 44|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + 1)|0;
 HEAP32[$iResult>>2] = $21;
 $22 = HEAP32[$nResult>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 44|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22))|0;
 HEAP32[$24>>2] = $26;
 HEAP32[$i>>2] = 0;
 while(1) {
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$nResult>>2]|0;
  $29 = ($27|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = ((($34)) + 4|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = (($36) + (($31*20)|0)|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$iResult>>2]|0;
  $41 = (($39) + ($40))|0;
  _sqlite3ExprCode($30,$38,$41);
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$i>>2] = $43;
 }
 $44 = HEAP32[$iResult>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _exprToRegister($p,$iReg) {
 $p = $p|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iReg;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 38|0);
 HEAP8[$5>>0] = $3;
 $6 = HEAP32[$0>>2]|0;
 HEAP8[$6>>0] = -99;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 28|0);
 HEAP32[$9>>2] = $7;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $12 & -4097;
 HEAP32[$11>>2] = $13;
 STACKTOP = sp;return;
}
function _sqlite3ExprIfFalse($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d2 = 0;
 var $destIfNull = 0, $op = 0, $or$cond = 0, $r1 = 0, $r2 = 0, $regFree1 = 0, $regFree2 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $v = sp + 28|0;
 $op = sp + 24|0;
 $regFree1 = sp + 20|0;
 $regFree2 = sp + 16|0;
 $r1 = sp + 12|0;
 $r2 = sp + 8|0;
 $d2 = sp + 4|0;
 $destIfNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 HEAP32[$op>>2] = 0;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 | $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = (($13) + 0)|0;
 $15 = $14 ^ 1;
 $16 = (($15) - 0)|0;
 HEAP32[$op>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 L4: do {
  switch ($19|0) {
  case 28:  {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = ((($21)) + 12|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($20,$23,$24,$25);
   $26 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($26);
   $27 = HEAP32[$0>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 16|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$2>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($27,$30,$31,$32);
   $33 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($33);
   break;
  }
  case 27:  {
   $34 = HEAP32[$v>>2]|0;
   $35 = (_sqlite3VdbeMakeLabel($34)|0);
   HEAP32[$d2>>2] = $35;
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$d2>>2]|0;
   $41 = HEAP32[$3>>2]|0;
   $42 = $41 ^ 16;
   _sqlite3ExprIfTrue($36,$39,$40,$42);
   $43 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePush($43);
   $44 = HEAP32[$0>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 16|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$2>>2]|0;
   $49 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfFalse($44,$47,$48,$49);
   $50 = HEAP32[$v>>2]|0;
   $51 = HEAP32[$d2>>2]|0;
   _sqlite3VdbeResolveLabel($50,$51);
   $52 = HEAP32[$0>>2]|0;
   _sqlite3ExprCachePop($52);
   break;
  }
  case 19:  {
   $53 = HEAP32[$0>>2]|0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$2>>2]|0;
   $58 = HEAP32[$3>>2]|0;
   _sqlite3ExprIfTrue($53,$56,$57,$58);
   break;
  }
  case 148: case 29:  {
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP8[$59>>0]|0;
   $61 = $60&255;
   $62 = ($61|0)==(29);
   $63 = $62 ? 36 : 37;
   HEAP32[$op>>2] = $63;
   HEAP32[$3>>2] = 128;
   label = 7;
   break;
  }
  case 37: case 36: case 41: case 38: case 39: case 40:  {
   label = 7;
   break;
  }
  case 35: case 34:  {
   $91 = HEAP32[$0>>2]|0;
   $92 = HEAP32[$1>>2]|0;
   $93 = ((($92)) + 12|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (_sqlite3ExprCodeTemp($91,$94,$regFree1)|0);
   HEAP32[$r1>>2] = $95;
   $96 = HEAP32[$v>>2]|0;
   $97 = HEAP32[$op>>2]|0;
   $98 = HEAP32[$r1>>2]|0;
   $99 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($96,$97,$98,$99)|0);
   break;
  }
  case 32:  {
   $100 = HEAP32[$0>>2]|0;
   $101 = HEAP32[$1>>2]|0;
   $102 = HEAP32[$2>>2]|0;
   $103 = HEAP32[$3>>2]|0;
   _exprCodeBetween($100,$101,$102,133,$103);
   break;
  }
  case 33:  {
   $104 = HEAP32[$3>>2]|0;
   $105 = ($104|0)!=(0);
   if ($105) {
    $106 = HEAP32[$0>>2]|0;
    $107 = HEAP32[$1>>2]|0;
    $108 = HEAP32[$2>>2]|0;
    $109 = HEAP32[$2>>2]|0;
    _sqlite3ExprCodeIN($106,$107,$108,$109);
    break L4;
   } else {
    $110 = HEAP32[$v>>2]|0;
    $111 = (_sqlite3VdbeMakeLabel($110)|0);
    HEAP32[$destIfNull>>2] = $111;
    $112 = HEAP32[$0>>2]|0;
    $113 = HEAP32[$1>>2]|0;
    $114 = HEAP32[$2>>2]|0;
    $115 = HEAP32[$destIfNull>>2]|0;
    _sqlite3ExprCodeIN($112,$113,$114,$115);
    $116 = HEAP32[$v>>2]|0;
    $117 = HEAP32[$destIfNull>>2]|0;
    _sqlite3VdbeResolveLabel($116,$117);
    break L4;
   }
   break;
  }
  default: {
   label = 14;
  }
  }
 } while(0);
 if ((label|0) == 7) {
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 12|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = (_sqlite3ExprIsVector($66)|0);
  $68 = ($67|0)!=(0);
  if ($68) {
   label = 14;
  } else {
   $69 = HEAP32[$0>>2]|0;
   $70 = HEAP32[$1>>2]|0;
   $71 = ((($70)) + 12|0);
   $72 = HEAP32[$71>>2]|0;
   $73 = (_sqlite3ExprCodeTemp($69,$72,$regFree1)|0);
   HEAP32[$r1>>2] = $73;
   $74 = HEAP32[$0>>2]|0;
   $75 = HEAP32[$1>>2]|0;
   $76 = ((($75)) + 16|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (_sqlite3ExprCodeTemp($74,$77,$regFree2)|0);
   HEAP32[$r2>>2] = $78;
   $79 = HEAP32[$0>>2]|0;
   $80 = HEAP32[$1>>2]|0;
   $81 = ((($80)) + 12|0);
   $82 = HEAP32[$81>>2]|0;
   $83 = HEAP32[$1>>2]|0;
   $84 = ((($83)) + 16|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = HEAP32[$op>>2]|0;
   $87 = HEAP32[$r1>>2]|0;
   $88 = HEAP32[$r2>>2]|0;
   $89 = HEAP32[$2>>2]|0;
   $90 = HEAP32[$3>>2]|0;
   (_codeCompare($79,$82,$85,$86,$87,$88,$89,$90)|0);
  }
 }
 do {
  if ((label|0) == 14) {
   $118 = HEAP32[$1>>2]|0;
   $119 = (_exprAlwaysFalse($118)|0);
   $120 = ($119|0)!=(0);
   if ($120) {
    $121 = HEAP32[$v>>2]|0;
    $122 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeGoto($121,$122)|0);
    break;
   }
   $123 = HEAP32[$1>>2]|0;
   $124 = (_exprAlwaysTrue($123)|0);
   $125 = ($124|0)!=(0);
   if (!($125)) {
    $126 = HEAP32[$0>>2]|0;
    $127 = HEAP32[$1>>2]|0;
    $128 = (_sqlite3ExprCodeTemp($126,$127,$regFree1)|0);
    HEAP32[$r1>>2] = $128;
    $129 = HEAP32[$v>>2]|0;
    $130 = HEAP32[$r1>>2]|0;
    $131 = HEAP32[$2>>2]|0;
    $132 = HEAP32[$3>>2]|0;
    $133 = ($132|0)!=(0);
    $134 = $133&1;
    (_sqlite3VdbeAddOp3($129,22,$130,$131,$134)|0);
   }
  }
 } while(0);
 $135 = HEAP32[$0>>2]|0;
 $136 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($135,$136);
 $137 = HEAP32[$0>>2]|0;
 $138 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($137,$138);
 STACKTOP = sp;return;
}
function _sqlite3ExprIfTrue($pParse,$pExpr,$dest,$jumpIfNull) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $dest = $dest|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $d2 = 0, $destIfFalse = 0, $destIfNull = 0, $op = 0, $or$cond = 0, $r1 = 0, $r2 = 0;
 var $regFree1 = 0, $regFree2 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $v = sp + 32|0;
 $op = sp + 28|0;
 $regFree1 = sp + 24|0;
 $regFree2 = sp + 20|0;
 $r1 = sp + 16|0;
 $r2 = sp + 12|0;
 $d2 = sp + 8|0;
 $destIfFalse = sp + 4|0;
 $destIfNull = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $dest;
 HEAP32[$3>>2] = $jumpIfNull;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 HEAP32[$op>>2] = 0;
 HEAP32[$regFree1>>2] = 0;
 HEAP32[$regFree2>>2] = 0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ($7|0)==(0|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ($9|0)==(0|0);
 $or$cond = $8 | $10;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 HEAP32[$op>>2] = $13;
 $14 = HEAP32[$op>>2]|0;
 switch ($14|0) {
 case 28:  {
  $15 = HEAP32[$v>>2]|0;
  $16 = (_sqlite3VdbeMakeLabel($15)|0);
  HEAP32[$d2>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$d2>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = $22 ^ 16;
  _sqlite3ExprIfFalse($17,$20,$21,$23);
  $24 = HEAP32[$0>>2]|0;
  _sqlite3ExprCachePush($24);
  $25 = HEAP32[$0>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = HEAP32[$3>>2]|0;
  _sqlite3ExprIfTrue($25,$28,$29,$30);
  $31 = HEAP32[$v>>2]|0;
  $32 = HEAP32[$d2>>2]|0;
  _sqlite3VdbeResolveLabel($31,$32);
  $33 = HEAP32[$0>>2]|0;
  _sqlite3ExprCachePop($33);
  break;
 }
 case 27:  {
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$3>>2]|0;
  _sqlite3ExprIfTrue($34,$37,$38,$39);
  $40 = HEAP32[$0>>2]|0;
  _sqlite3ExprCachePush($40);
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$1>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$2>>2]|0;
  $46 = HEAP32[$3>>2]|0;
  _sqlite3ExprIfTrue($41,$44,$45,$46);
  $47 = HEAP32[$0>>2]|0;
  _sqlite3ExprCachePop($47);
  break;
 }
 case 19:  {
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 12|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = HEAP32[$3>>2]|0;
  _sqlite3ExprIfFalse($48,$51,$52,$53);
  break;
 }
 case 148: case 29:  {
  $54 = HEAP32[$op>>2]|0;
  $55 = ($54|0)==(29);
  $56 = $55 ? 37 : 36;
  HEAP32[$op>>2] = $56;
  HEAP32[$3>>2] = 128;
  label = 7;
  break;
 }
 case 37: case 36: case 41: case 38: case 39: case 40:  {
  label = 7;
  break;
 }
 case 35: case 34:  {
  $84 = HEAP32[$0>>2]|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 12|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (_sqlite3ExprCodeTemp($84,$87,$regFree1)|0);
  HEAP32[$r1>>2] = $88;
  $89 = HEAP32[$v>>2]|0;
  $90 = HEAP32[$op>>2]|0;
  $91 = HEAP32[$r1>>2]|0;
  $92 = HEAP32[$2>>2]|0;
  (_sqlite3VdbeAddOp2($89,$90,$91,$92)|0);
  break;
 }
 case 32:  {
  $93 = HEAP32[$0>>2]|0;
  $94 = HEAP32[$1>>2]|0;
  $95 = HEAP32[$2>>2]|0;
  $96 = HEAP32[$3>>2]|0;
  _exprCodeBetween($93,$94,$95,134,$96);
  break;
 }
 case 33:  {
  $97 = HEAP32[$v>>2]|0;
  $98 = (_sqlite3VdbeMakeLabel($97)|0);
  HEAP32[$destIfFalse>>2] = $98;
  $99 = HEAP32[$3>>2]|0;
  $100 = ($99|0)!=(0);
  $101 = HEAP32[$2>>2]|0;
  $102 = HEAP32[$destIfFalse>>2]|0;
  $103 = $100 ? $101 : $102;
  HEAP32[$destIfNull>>2] = $103;
  $104 = HEAP32[$0>>2]|0;
  $105 = HEAP32[$1>>2]|0;
  $106 = HEAP32[$destIfFalse>>2]|0;
  $107 = HEAP32[$destIfNull>>2]|0;
  _sqlite3ExprCodeIN($104,$105,$106,$107);
  $108 = HEAP32[$v>>2]|0;
  $109 = HEAP32[$2>>2]|0;
  (_sqlite3VdbeGoto($108,$109)|0);
  $110 = HEAP32[$v>>2]|0;
  $111 = HEAP32[$destIfFalse>>2]|0;
  _sqlite3VdbeResolveLabel($110,$111);
  break;
 }
 default: {
  label = 12;
 }
 }
 if ((label|0) == 7) {
  $57 = HEAP32[$1>>2]|0;
  $58 = ((($57)) + 12|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = (_sqlite3ExprIsVector($59)|0);
  $61 = ($60|0)!=(0);
  if ($61) {
   label = 12;
  } else {
   $62 = HEAP32[$0>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   $64 = ((($63)) + 12|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (_sqlite3ExprCodeTemp($62,$65,$regFree1)|0);
   HEAP32[$r1>>2] = $66;
   $67 = HEAP32[$0>>2]|0;
   $68 = HEAP32[$1>>2]|0;
   $69 = ((($68)) + 16|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (_sqlite3ExprCodeTemp($67,$70,$regFree2)|0);
   HEAP32[$r2>>2] = $71;
   $72 = HEAP32[$0>>2]|0;
   $73 = HEAP32[$1>>2]|0;
   $74 = ((($73)) + 12|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$1>>2]|0;
   $77 = ((($76)) + 16|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = HEAP32[$op>>2]|0;
   $80 = HEAP32[$r1>>2]|0;
   $81 = HEAP32[$r2>>2]|0;
   $82 = HEAP32[$2>>2]|0;
   $83 = HEAP32[$3>>2]|0;
   (_codeCompare($72,$75,$78,$79,$80,$81,$82,$83)|0);
  }
 }
 do {
  if ((label|0) == 12) {
   $112 = HEAP32[$1>>2]|0;
   $113 = (_exprAlwaysTrue($112)|0);
   $114 = ($113|0)!=(0);
   if ($114) {
    $115 = HEAP32[$v>>2]|0;
    $116 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeGoto($115,$116)|0);
    break;
   }
   $117 = HEAP32[$1>>2]|0;
   $118 = (_exprAlwaysFalse($117)|0);
   $119 = ($118|0)!=(0);
   if (!($119)) {
    $120 = HEAP32[$0>>2]|0;
    $121 = HEAP32[$1>>2]|0;
    $122 = (_sqlite3ExprCodeTemp($120,$121,$regFree1)|0);
    HEAP32[$r1>>2] = $122;
    $123 = HEAP32[$v>>2]|0;
    $124 = HEAP32[$r1>>2]|0;
    $125 = HEAP32[$2>>2]|0;
    $126 = HEAP32[$3>>2]|0;
    $127 = ($126|0)!=(0);
    $128 = $127&1;
    (_sqlite3VdbeAddOp3($123,21,$124,$125,$128)|0);
   }
  }
 } while(0);
 $129 = HEAP32[$0>>2]|0;
 $130 = HEAP32[$regFree1>>2]|0;
 _sqlite3ReleaseTempReg($129,$130);
 $131 = HEAP32[$0>>2]|0;
 $132 = HEAP32[$regFree2>>2]|0;
 _sqlite3ReleaseTempReg($131,$132);
 STACKTOP = sp;return;
}
function _exprAlwaysTrue($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$v>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 1;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   HEAP32[$0>>2] = 0;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3ExprIsInteger($7,$v)|0);
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$v>>2]|0;
    $11 = ($10|0)!=(0);
    $12 = $11&1;
    HEAP32[$0>>2] = $12;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3ExprCheckIN($pParse,$pIn) {
 $pParse = $pParse|0;
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nVector = 0, $vararg_buffer = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nVector = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pIn;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3ExprVectorSize($5)|0);
 HEAP32[$nVector>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 2048;
 $11 = ($10|0)!=(0);
 $12 = HEAP32[$nVector>>2]|0;
 if ($11) {
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  $18 = ($12|0)!=($17|0);
  if ($18) {
   $19 = HEAP32[$1>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 20|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$nVector>>2]|0;
   _sqlite3SubselectError($19,$24,$25);
   HEAP32[$0>>2] = 1;
   $36 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
 } else {
  $26 = ($12|0)!=(1);
  if ($26) {
   $27 = HEAP32[$2>>2]|0;
   $28 = ((($27)) + 12|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ((($29)) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = $31 & 2048;
   $33 = ($32|0)!=(0);
   $34 = HEAP32[$1>>2]|0;
   if ($33) {
    $35 = HEAP32[$nVector>>2]|0;
    _sqlite3SubselectError($34,$35,1);
   } else {
    _sqlite3ErrorMsg($34,29838,$vararg_buffer);
   }
   HEAP32[$0>>2] = 1;
   $36 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _exprINAffinity($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $i = 0, $nVal = 0;
 var $pA = 0, $pLeft = 0, $pSelect = 0, $zRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pLeft = sp + 20|0;
 $nVal = sp + 16|0;
 $pSelect = sp + 12|0;
 $zRet = sp + 8|0;
 $i = sp + 4|0;
 $pA = sp;
 $a = sp + 32|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pLeft>>2] = $4;
 $5 = HEAP32[$pLeft>>2]|0;
 $6 = (_sqlite3ExprVectorSize($5)|0);
 HEAP32[$nVal>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 2048;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 20|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = $14;
 } else {
  $15 = 0;
 }
 HEAP32[$pSelect>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$nVal>>2]|0;
 $19 = (($18) + 1)|0;
 $20 = ($19|0)<(0);
 $21 = $20 << 31 >> 31;
 $22 = (_sqlite3DbMallocZero($17,$19,$21)|0);
 HEAP32[$zRet>>2] = $22;
 $23 = HEAP32[$zRet>>2]|0;
 $24 = ($23|0)!=(0|0);
 if (!($24)) {
  $56 = HEAP32[$zRet>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$nVal>>2]|0;
  $27 = ($25|0)<($26|0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$pLeft>>2]|0;
  $29 = HEAP32[$i>>2]|0;
  $30 = (_sqlite3VectorFieldSubexpr($28,$29)|0);
  HEAP32[$pA>>2] = $30;
  $31 = HEAP32[$pA>>2]|0;
  $32 = (_sqlite3ExprAffinity($31)|0);
  HEAP8[$a>>0] = $32;
  $33 = HEAP32[$pSelect>>2]|0;
  $34 = ($33|0)!=(0|0);
  if ($34) {
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$pSelect>>2]|0;
   $37 = HEAP32[$36>>2]|0;
   $38 = ((($37)) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (($39) + (($35*20)|0)|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP8[$a>>0]|0;
   $43 = (_sqlite3CompareAffinity($41,$42)|0);
   $44 = HEAP32[$i>>2]|0;
   $45 = HEAP32[$zRet>>2]|0;
   $46 = (($45) + ($44)|0);
   HEAP8[$46>>0] = $43;
  } else {
   $47 = HEAP8[$a>>0]|0;
   $48 = HEAP32[$i>>2]|0;
   $49 = HEAP32[$zRet>>2]|0;
   $50 = (($49) + ($48)|0);
   HEAP8[$50>>0] = $47;
  }
  $51 = HEAP32[$i>>2]|0;
  $52 = (($51) + 1)|0;
  HEAP32[$i>>2] = $52;
 }
 $53 = HEAP32[$nVal>>2]|0;
 $54 = HEAP32[$zRet>>2]|0;
 $55 = (($54) + ($53)|0);
 HEAP8[$55>>0] = 0;
 $56 = HEAP32[$zRet>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _sqlite3FindInIndex($pParse,$pX,$inFlags,$prRhsHasNull,$aiMap) {
 $pParse = $pParse|0;
 $pX = $pX|0;
 $inFlags = $inFlags|0;
 $prRhsHasNull = $prRhsHasNull|0;
 $aiMap = $aiMap|0;
 var $$off = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $affinity_ok = 0, $cmpaff = 0, $colUsed = 0, $db = 0, $eType = 0, $i = 0, $i2 = 0, $i5 = 0, $iAddr = 0;
 var $iAddr4 = 0, $iCol = 0, $iDb = 0, $iTab = 0, $idxaff = 0, $j = 0, $mCol = 0, $mustBeUnique = 0, $n = 0, $nExpr = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pEList = 0, $pEList1 = 0, $pIdx = 0, $pLhs = 0, $pLhs3 = 0, $pReq = 0, $pRhs = 0;
 var $pTab = 0, $rMayHaveNull = 0, $savedNQueryLoop = 0, $switch = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 140|0;
 $1 = sp + 136|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $p = sp + 120|0;
 $eType = sp + 116|0;
 $iTab = sp + 112|0;
 $mustBeUnique = sp + 108|0;
 $v = sp + 104|0;
 $i = sp + 100|0;
 $pEList = sp + 96|0;
 $db = sp + 92|0;
 $pTab = sp + 88|0;
 $iDb = sp + 144|0;
 $pEList1 = sp + 84|0;
 $nExpr = sp + 80|0;
 $iAddr = sp + 76|0;
 $pIdx = sp + 72|0;
 $affinity_ok = sp + 68|0;
 $i2 = sp + 64|0;
 $pLhs = sp + 60|0;
 $iCol = sp + 56|0;
 $idxaff = sp + 147|0;
 $cmpaff = sp + 146|0;
 $colUsed = sp + 8|0;
 $mCol = sp;
 $pLhs3 = sp + 52|0;
 $pRhs = sp + 48|0;
 $pReq = sp + 44|0;
 $j = sp + 40|0;
 $iAddr4 = sp + 36|0;
 $savedNQueryLoop = sp + 32|0;
 $rMayHaveNull = sp + 28|0;
 $i5 = sp + 24|0;
 $n = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pX;
 HEAP32[$2>>2] = $inFlags;
 HEAP32[$3>>2] = $prRhsHasNull;
 HEAP32[$4>>2] = $aiMap;
 HEAP32[$eType>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 40|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + 1)|0;
 HEAP32[$6>>2] = $8;
 HEAP32[$iTab>>2] = $7;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = $11 & 4;
 $13 = ($12|0)!=(0);
 $14 = $13&1;
 HEAP32[$mustBeUnique>>2] = $14;
 $15 = HEAP32[$3>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = $19 & 2048;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 20|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$pEList>>2] = $25;
   HEAP32[$i>>2] = 0;
   while(1) {
    $26 = HEAP32[$i>>2]|0;
    $27 = HEAP32[$pEList>>2]|0;
    $28 = HEAP32[$27>>2]|0;
    $29 = ($26|0)<($28|0);
    if (!($29)) {
     break;
    }
    $30 = HEAP32[$i>>2]|0;
    $31 = HEAP32[$pEList>>2]|0;
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = (($33) + (($30*20)|0)|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = (_sqlite3ExprCanBeNull($35)|0);
    $37 = ($36|0)!=(0);
    if ($37) {
     break;
    }
    $38 = HEAP32[$i>>2]|0;
    $39 = (($38) + 1)|0;
    HEAP32[$i>>2] = $39;
   }
   $40 = HEAP32[$i>>2]|0;
   $41 = HEAP32[$pEList>>2]|0;
   $42 = HEAP32[$41>>2]|0;
   $43 = ($40|0)==($42|0);
   if ($43) {
    HEAP32[$3>>2] = 0;
   }
  }
 }
 $44 = HEAP32[$0>>2]|0;
 $45 = ((($44)) + 36|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)==(0);
 L11: do {
  if ($47) {
   $48 = HEAP32[$1>>2]|0;
   $49 = (_isCandidateForInOpt($48)|0);
   HEAP32[$p>>2] = $49;
   $50 = ($49|0)!=(0|0);
   if ($50) {
    $51 = HEAP32[$0>>2]|0;
    $52 = HEAP32[$51>>2]|0;
    HEAP32[$db>>2] = $52;
    $53 = HEAP32[$p>>2]|0;
    $54 = HEAP32[$53>>2]|0;
    HEAP32[$pEList1>>2] = $54;
    $55 = HEAP32[$pEList1>>2]|0;
    $56 = HEAP32[$55>>2]|0;
    HEAP32[$nExpr>>2] = $56;
    $57 = HEAP32[$p>>2]|0;
    $58 = ((($57)) + 28|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = ((($59)) + 8|0);
    $61 = ((($60)) + 16|0);
    $62 = HEAP32[$61>>2]|0;
    HEAP32[$pTab>>2] = $62;
    $63 = HEAP32[$db>>2]|0;
    $64 = HEAP32[$pTab>>2]|0;
    $65 = ((($64)) + 64|0);
    $66 = HEAP32[$65>>2]|0;
    $67 = (_sqlite3SchemaToIndex($63,$66)|0);
    $68 = $67&65535;
    HEAP16[$iDb>>1] = $68;
    $69 = HEAP32[$0>>2]|0;
    $70 = HEAP16[$iDb>>1]|0;
    $71 = $70 << 16 >> 16;
    _sqlite3CodeVerifySchema($69,$71);
    $72 = HEAP32[$0>>2]|0;
    $73 = HEAP16[$iDb>>1]|0;
    $74 = $73 << 16 >> 16;
    $75 = HEAP32[$pTab>>2]|0;
    $76 = ((($75)) + 28|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$pTab>>2]|0;
    $79 = HEAP32[$78>>2]|0;
    _sqlite3TableLock($72,$74,$77,0,$79);
    $80 = HEAP32[$nExpr>>2]|0;
    $81 = ($80|0)==(1);
    if ($81) {
     $82 = HEAP32[$pEList1>>2]|0;
     $83 = ((($82)) + 4|0);
     $84 = HEAP32[$83>>2]|0;
     $85 = HEAP32[$84>>2]|0;
     $86 = ((($85)) + 32|0);
     $87 = HEAP16[$86>>1]|0;
     $88 = $87 << 16 >> 16;
     $89 = ($88|0)<(0);
     if ($89) {
      $90 = HEAP32[$v>>2]|0;
      $91 = (_sqlite3VdbeAddOp0($90,20)|0);
      HEAP32[$iAddr>>2] = $91;
      $92 = HEAP32[$0>>2]|0;
      $93 = HEAP32[$iTab>>2]|0;
      $94 = HEAP16[$iDb>>1]|0;
      $95 = $94 << 16 >> 16;
      $96 = HEAP32[$pTab>>2]|0;
      _sqlite3OpenTable($92,$93,$95,$96,104);
      HEAP32[$eType>>2] = 1;
      $97 = HEAP32[$v>>2]|0;
      $98 = HEAP32[$iAddr>>2]|0;
      _sqlite3VdbeJumpHere($97,$98);
      break;
     }
    }
    HEAP32[$affinity_ok>>2] = 1;
    HEAP32[$i2>>2] = 0;
    while(1) {
     $99 = HEAP32[$i2>>2]|0;
     $100 = HEAP32[$nExpr>>2]|0;
     $101 = ($99|0)<($100|0);
     $102 = HEAP32[$affinity_ok>>2]|0;
     $103 = ($102|0)!=(0);
     $104 = $101 ? $103 : 0;
     if (!($104)) {
      break;
     }
     $105 = HEAP32[$1>>2]|0;
     $106 = ((($105)) + 12|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = HEAP32[$i2>>2]|0;
     $109 = (_sqlite3VectorFieldSubexpr($107,$108)|0);
     HEAP32[$pLhs>>2] = $109;
     $110 = HEAP32[$i2>>2]|0;
     $111 = HEAP32[$pEList1>>2]|0;
     $112 = ((($111)) + 4|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = (($113) + (($110*20)|0)|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = ((($115)) + 32|0);
     $117 = HEAP16[$116>>1]|0;
     $118 = $117 << 16 >> 16;
     HEAP32[$iCol>>2] = $118;
     $119 = HEAP32[$pTab>>2]|0;
     $120 = HEAP32[$iCol>>2]|0;
     $121 = (_sqlite3TableColumnAffinity($119,$120)|0);
     HEAP8[$idxaff>>0] = $121;
     $122 = HEAP32[$pLhs>>2]|0;
     $123 = HEAP8[$idxaff>>0]|0;
     $124 = (_sqlite3CompareAffinity($122,$123)|0);
     HEAP8[$cmpaff>>0] = $124;
     $125 = HEAP8[$cmpaff>>0]|0;
     $126 = $125 << 24 >> 24;
     $$off = (($126) + -65)|0;
     $switch = ($$off>>>0)<(2);
     if (!($switch)) {
      $127 = HEAP8[$idxaff>>0]|0;
      $128 = $127 << 24 >> 24;
      $129 = ($128|0)>=(67);
      $130 = $129&1;
      HEAP32[$affinity_ok>>2] = $130;
     }
     $131 = HEAP32[$i2>>2]|0;
     $132 = (($131) + 1)|0;
     HEAP32[$i2>>2] = $132;
    }
    $133 = HEAP32[$affinity_ok>>2]|0;
    $134 = ($133|0)!=(0);
    if ($134) {
     $135 = HEAP32[$pTab>>2]|0;
     $136 = ((($135)) + 8|0);
     $137 = HEAP32[$136>>2]|0;
     HEAP32[$pIdx>>2] = $137;
     while(1) {
      $138 = HEAP32[$pIdx>>2]|0;
      $139 = ($138|0)!=(0|0);
      $140 = HEAP32[$eType>>2]|0;
      $141 = ($140|0)==(0);
      $142 = $139 ? $141 : 0;
      if (!($142)) {
       break L11;
      }
      $143 = HEAP32[$pIdx>>2]|0;
      $144 = ((($143)) + 52|0);
      $145 = HEAP16[$144>>1]|0;
      $146 = $145&65535;
      $147 = HEAP32[$nExpr>>2]|0;
      $148 = ($146|0)<($147|0);
      do {
       if (!($148)) {
        $149 = HEAP32[$pIdx>>2]|0;
        $150 = ((($149)) + 52|0);
        $151 = HEAP16[$150>>1]|0;
        $152 = $151&65535;
        $153 = ($152|0)>=(63);
        if (!($153)) {
         $154 = HEAP32[$mustBeUnique>>2]|0;
         $155 = ($154|0)!=(0);
         if ($155) {
          $156 = HEAP32[$pIdx>>2]|0;
          $157 = ((($156)) + 50|0);
          $158 = HEAP16[$157>>1]|0;
          $159 = $158&65535;
          $160 = HEAP32[$nExpr>>2]|0;
          $161 = ($159|0)>($160|0);
          if ($161) {
           break;
          }
          $162 = HEAP32[$pIdx>>2]|0;
          $163 = ((($162)) + 52|0);
          $164 = HEAP16[$163>>1]|0;
          $165 = $164&65535;
          $166 = HEAP32[$nExpr>>2]|0;
          $167 = ($165|0)>($166|0);
          if ($167) {
           $168 = HEAP32[$pIdx>>2]|0;
           $169 = ((($168)) + 54|0);
           $170 = HEAP8[$169>>0]|0;
           $171 = $170&255;
           $172 = ($171|0)!=(0);
           if (!($172)) {
            break;
           }
          }
         }
         $173 = $colUsed;
         $174 = $173;
         HEAP32[$174>>2] = 0;
         $175 = (($173) + 4)|0;
         $176 = $175;
         HEAP32[$176>>2] = 0;
         HEAP32[$i2>>2] = 0;
         while(1) {
          $177 = HEAP32[$i2>>2]|0;
          $178 = HEAP32[$nExpr>>2]|0;
          $179 = ($177|0)<($178|0);
          if (!($179)) {
           break;
          }
          $180 = HEAP32[$1>>2]|0;
          $181 = ((($180)) + 12|0);
          $182 = HEAP32[$181>>2]|0;
          $183 = HEAP32[$i2>>2]|0;
          $184 = (_sqlite3VectorFieldSubexpr($182,$183)|0);
          HEAP32[$pLhs3>>2] = $184;
          $185 = HEAP32[$i2>>2]|0;
          $186 = HEAP32[$pEList1>>2]|0;
          $187 = ((($186)) + 4|0);
          $188 = HEAP32[$187>>2]|0;
          $189 = (($188) + (($185*20)|0)|0);
          $190 = HEAP32[$189>>2]|0;
          HEAP32[$pRhs>>2] = $190;
          $191 = HEAP32[$0>>2]|0;
          $192 = HEAP32[$pLhs3>>2]|0;
          $193 = HEAP32[$pRhs>>2]|0;
          $194 = (_sqlite3BinaryCompareCollSeq($191,$192,$193)|0);
          HEAP32[$pReq>>2] = $194;
          HEAP32[$j>>2] = 0;
          while(1) {
           $195 = HEAP32[$j>>2]|0;
           $196 = HEAP32[$nExpr>>2]|0;
           $197 = ($195|0)<($196|0);
           if (!($197)) {
            break;
           }
           $198 = HEAP32[$j>>2]|0;
           $199 = HEAP32[$pIdx>>2]|0;
           $200 = ((($199)) + 4|0);
           $201 = HEAP32[$200>>2]|0;
           $202 = (($201) + ($198<<1)|0);
           $203 = HEAP16[$202>>1]|0;
           $204 = $203 << 16 >> 16;
           $205 = HEAP32[$pRhs>>2]|0;
           $206 = ((($205)) + 32|0);
           $207 = HEAP16[$206>>1]|0;
           $208 = $207 << 16 >> 16;
           $209 = ($204|0)!=($208|0);
           if (!($209)) {
            $210 = HEAP32[$pReq>>2]|0;
            $211 = ($210|0)!=(0|0);
            if (!($211)) {
             break;
            }
            $212 = HEAP32[$pReq>>2]|0;
            $213 = HEAP32[$212>>2]|0;
            $214 = HEAP32[$j>>2]|0;
            $215 = HEAP32[$pIdx>>2]|0;
            $216 = ((($215)) + 32|0);
            $217 = HEAP32[$216>>2]|0;
            $218 = (($217) + ($214<<2)|0);
            $219 = HEAP32[$218>>2]|0;
            $220 = (_sqlite3StrICmp($213,$219)|0);
            $221 = ($220|0)!=(0);
            if (!($221)) {
             break;
            }
           }
           $222 = HEAP32[$j>>2]|0;
           $223 = (($222) + 1)|0;
           HEAP32[$j>>2] = $223;
          }
          $224 = HEAP32[$j>>2]|0;
          $225 = HEAP32[$nExpr>>2]|0;
          $226 = ($224|0)==($225|0);
          if ($226) {
           break;
          }
          $227 = HEAP32[$j>>2]|0;
          $228 = (_bitshift64Shl(1,0,($227|0))|0);
          $229 = tempRet0;
          $230 = $mCol;
          $231 = $230;
          HEAP32[$231>>2] = $228;
          $232 = (($230) + 4)|0;
          $233 = $232;
          HEAP32[$233>>2] = $229;
          $234 = $mCol;
          $235 = $234;
          $236 = HEAP32[$235>>2]|0;
          $237 = (($234) + 4)|0;
          $238 = $237;
          $239 = HEAP32[$238>>2]|0;
          $240 = $colUsed;
          $241 = $240;
          $242 = HEAP32[$241>>2]|0;
          $243 = (($240) + 4)|0;
          $244 = $243;
          $245 = HEAP32[$244>>2]|0;
          $246 = $236 & $242;
          $247 = $239 & $245;
          $248 = ($246|0)!=(0);
          $249 = ($247|0)!=(0);
          $250 = $248 | $249;
          if ($250) {
           break;
          }
          $251 = $mCol;
          $252 = $251;
          $253 = HEAP32[$252>>2]|0;
          $254 = (($251) + 4)|0;
          $255 = $254;
          $256 = HEAP32[$255>>2]|0;
          $257 = $colUsed;
          $258 = $257;
          $259 = HEAP32[$258>>2]|0;
          $260 = (($257) + 4)|0;
          $261 = $260;
          $262 = HEAP32[$261>>2]|0;
          $263 = $259 | $253;
          $264 = $262 | $256;
          $265 = $colUsed;
          $266 = $265;
          HEAP32[$266>>2] = $263;
          $267 = (($265) + 4)|0;
          $268 = $267;
          HEAP32[$268>>2] = $264;
          $269 = HEAP32[$4>>2]|0;
          $270 = ($269|0)!=(0|0);
          if ($270) {
           $271 = HEAP32[$j>>2]|0;
           $272 = HEAP32[$i2>>2]|0;
           $273 = HEAP32[$4>>2]|0;
           $274 = (($273) + ($272<<2)|0);
           HEAP32[$274>>2] = $271;
          }
          $275 = HEAP32[$i2>>2]|0;
          $276 = (($275) + 1)|0;
          HEAP32[$i2>>2] = $276;
         }
         $277 = $colUsed;
         $278 = $277;
         $279 = HEAP32[$278>>2]|0;
         $280 = (($277) + 4)|0;
         $281 = $280;
         $282 = HEAP32[$281>>2]|0;
         $283 = HEAP32[$nExpr>>2]|0;
         $284 = (_bitshift64Shl(1,0,($283|0))|0);
         $285 = tempRet0;
         $286 = (_i64Subtract(($284|0),($285|0),1,0)|0);
         $287 = tempRet0;
         $288 = ($279|0)==($286|0);
         $289 = ($282|0)==($287|0);
         $290 = $288 & $289;
         if ($290) {
          $291 = HEAP32[$v>>2]|0;
          $292 = (_sqlite3VdbeAddOp0($291,20)|0);
          HEAP32[$iAddr4>>2] = $292;
          $293 = HEAP32[$v>>2]|0;
          $294 = HEAP32[$db>>2]|0;
          $295 = HEAP32[$pIdx>>2]|0;
          $296 = HEAP32[$295>>2]|0;
          HEAP32[$vararg_buffer>>2] = $296;
          $297 = (_sqlite3MPrintf($294,31166,$vararg_buffer)|0);
          (_sqlite3VdbeAddOp4($293,162,0,0,0,$297,-1)|0);
          $298 = HEAP32[$v>>2]|0;
          $299 = HEAP32[$iTab>>2]|0;
          $300 = HEAP32[$pIdx>>2]|0;
          $301 = ((($300)) + 44|0);
          $302 = HEAP32[$301>>2]|0;
          $303 = HEAP16[$iDb>>1]|0;
          $304 = $303 << 16 >> 16;
          (_sqlite3VdbeAddOp3($298,104,$299,$302,$304)|0);
          $305 = HEAP32[$0>>2]|0;
          $306 = HEAP32[$pIdx>>2]|0;
          _sqlite3VdbeSetP4KeyInfo($305,$306);
          $307 = HEAP32[$pIdx>>2]|0;
          $308 = ((($307)) + 28|0);
          $309 = HEAP32[$308>>2]|0;
          $310 = HEAP8[$309>>0]|0;
          $311 = $310&255;
          $312 = (3 + ($311))|0;
          HEAP32[$eType>>2] = $312;
          $313 = HEAP32[$3>>2]|0;
          $314 = ($313|0)!=(0|0);
          if ($314) {
           $315 = HEAP32[$0>>2]|0;
           $316 = ((($315)) + 44|0);
           $317 = HEAP32[$316>>2]|0;
           $318 = (($317) + 1)|0;
           HEAP32[$316>>2] = $318;
           $319 = HEAP32[$3>>2]|0;
           HEAP32[$319>>2] = $318;
           $320 = HEAP32[$nExpr>>2]|0;
           $321 = ($320|0)==(1);
           if ($321) {
            $322 = HEAP32[$v>>2]|0;
            $323 = HEAP32[$iTab>>2]|0;
            $324 = HEAP32[$3>>2]|0;
            $325 = HEAP32[$324>>2]|0;
            _sqlite3SetHasNullFlag($322,$323,$325);
           }
          }
          $326 = HEAP32[$v>>2]|0;
          $327 = HEAP32[$iAddr4>>2]|0;
          _sqlite3VdbeJumpHere($326,$327);
         }
        }
       }
      } while(0);
      $328 = HEAP32[$pIdx>>2]|0;
      $329 = ((($328)) + 20|0);
      $330 = HEAP32[$329>>2]|0;
      HEAP32[$pIdx>>2] = $330;
     }
    }
   }
  }
 } while(0);
 $331 = HEAP32[$eType>>2]|0;
 $332 = ($331|0)==(0);
 do {
  if ($332) {
   $333 = HEAP32[$2>>2]|0;
   $334 = $333 & 1;
   $335 = ($334|0)!=(0);
   if ($335) {
    $336 = HEAP32[$1>>2]|0;
    $337 = ((($336)) + 4|0);
    $338 = HEAP32[$337>>2]|0;
    $339 = $338 & 2048;
    $340 = ($339|0)!=(0);
    if (!($340)) {
     $341 = HEAP32[$1>>2]|0;
     $342 = (_sqlite3InRhsIsConstant($341)|0);
     $343 = ($342|0)!=(0);
     if ($343) {
      $344 = HEAP32[$1>>2]|0;
      $345 = ((($344)) + 20|0);
      $346 = HEAP32[$345>>2]|0;
      $347 = HEAP32[$346>>2]|0;
      $348 = ($347|0)<=(2);
      if (!($348)) {
       break;
      }
     }
     HEAP32[$eType>>2] = 5;
    }
   }
  }
 } while(0);
 $349 = HEAP32[$eType>>2]|0;
 $350 = ($349|0)==(0);
 if ($350) {
  $351 = HEAP32[$0>>2]|0;
  $352 = ((($351)) + 136|0);
  $353 = HEAP32[$352>>2]|0;
  HEAP32[$savedNQueryLoop>>2] = $353;
  HEAP32[$rMayHaveNull>>2] = 0;
  HEAP32[$eType>>2] = 2;
  $354 = HEAP32[$2>>2]|0;
  $355 = $354 & 4;
  $356 = ($355|0)!=(0);
  if ($356) {
   $357 = HEAP32[$0>>2]|0;
   $358 = ((($357)) + 136|0);
   HEAP32[$358>>2] = 0;
   $359 = HEAP32[$1>>2]|0;
   $360 = ((($359)) + 12|0);
   $361 = HEAP32[$360>>2]|0;
   $362 = ((($361)) + 32|0);
   $363 = HEAP16[$362>>1]|0;
   $364 = $363 << 16 >> 16;
   $365 = ($364|0)<(0);
   if ($365) {
    $366 = HEAP32[$1>>2]|0;
    $367 = ((($366)) + 4|0);
    $368 = HEAP32[$367>>2]|0;
    $369 = $368 & 2048;
    $370 = ($369|0)!=(0);
    if (!($370)) {
     HEAP32[$eType>>2] = 1;
    }
   }
  } else {
   $371 = HEAP32[$3>>2]|0;
   $372 = ($371|0)!=(0|0);
   if ($372) {
    $373 = HEAP32[$0>>2]|0;
    $374 = ((($373)) + 44|0);
    $375 = HEAP32[$374>>2]|0;
    $376 = (($375) + 1)|0;
    HEAP32[$374>>2] = $376;
    HEAP32[$rMayHaveNull>>2] = $376;
    $377 = HEAP32[$3>>2]|0;
    HEAP32[$377>>2] = $376;
   }
  }
  $378 = HEAP32[$0>>2]|0;
  $379 = HEAP32[$1>>2]|0;
  $380 = HEAP32[$rMayHaveNull>>2]|0;
  $381 = HEAP32[$eType>>2]|0;
  $382 = ($381|0)==(1);
  $383 = $382&1;
  (_sqlite3CodeSubselect($378,$379,$380,$383)|0);
  $384 = HEAP32[$savedNQueryLoop>>2]|0;
  $385 = HEAP32[$0>>2]|0;
  $386 = ((($385)) + 136|0);
  HEAP32[$386>>2] = $384;
 } else {
  $387 = HEAP32[$iTab>>2]|0;
  $388 = HEAP32[$1>>2]|0;
  $389 = ((($388)) + 28|0);
  HEAP32[$389>>2] = $387;
 }
 $390 = HEAP32[$4>>2]|0;
 $391 = ($390|0)!=(0|0);
 $392 = HEAP32[$eType>>2]|0;
 $393 = ($392|0)!=(3);
 $or$cond = $391 & $393;
 $394 = HEAP32[$eType>>2]|0;
 $395 = ($394|0)!=(4);
 $or$cond3 = $or$cond & $395;
 if (!($or$cond3)) {
  $409 = HEAP32[$eType>>2]|0;
  STACKTOP = sp;return ($409|0);
 }
 $396 = HEAP32[$1>>2]|0;
 $397 = ((($396)) + 12|0);
 $398 = HEAP32[$397>>2]|0;
 $399 = (_sqlite3ExprVectorSize($398)|0);
 HEAP32[$n>>2] = $399;
 HEAP32[$i5>>2] = 0;
 while(1) {
  $400 = HEAP32[$i5>>2]|0;
  $401 = HEAP32[$n>>2]|0;
  $402 = ($400|0)<($401|0);
  if (!($402)) {
   break;
  }
  $403 = HEAP32[$i5>>2]|0;
  $404 = HEAP32[$i5>>2]|0;
  $405 = HEAP32[$4>>2]|0;
  $406 = (($405) + ($404<<2)|0);
  HEAP32[$406>>2] = $403;
  $407 = HEAP32[$i5>>2]|0;
  $408 = (($407) + 1)|0;
  HEAP32[$i5>>2] = $408;
 }
 $409 = HEAP32[$eType>>2]|0;
 STACKTOP = sp;return ($409|0);
}
function _sqlite3ExprCanBeNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $op = sp + 8|0;
 HEAP32[$1>>2] = $p;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = HEAP8[$2>>0]|0;
  $4 = $3&255;
  $5 = ($4|0)==(156);
  if ($5) {
   $49 = 1;
  } else {
   $6 = HEAP32[$1>>2]|0;
   $7 = HEAP8[$6>>0]|0;
   $8 = $7&255;
   $9 = ($8|0)==(155);
   $49 = $9;
  }
  $10 = HEAP32[$1>>2]|0;
  if (!($49)) {
   break;
  }
  $11 = ((($10)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$1>>2] = $12;
 }
 $13 = HEAP8[$10>>0]|0;
 HEAP8[$op>>0] = $13;
 $14 = HEAP8[$op>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(157);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 38|0);
  $19 = HEAP8[$18>>0]|0;
  HEAP8[$op>>0] = $19;
 }
 $20 = HEAP8[$op>>0]|0;
 $21 = $20&255;
 switch ($21|0) {
 case 133: case 132: case 97: case 134:  {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
  break;
 }
 case 152:  {
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24 & 1048576;
  $26 = ($25|0)!=(0);
  if ($26) {
   $47 = 1;
  } else {
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 32|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = ($30|0)>=(0);
   if ($31) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 32|0);
    $34 = HEAP16[$33>>1]|0;
    $35 = $34 << 16 >> 16;
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 44|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = ((($38)) + 4|0);
    $40 = HEAP32[$39>>2]|0;
    $41 = (($40) + ($35<<4)|0);
    $42 = ((($41)) + 12|0);
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = ($44|0)==(0);
    $47 = $45;
   } else {
    $47 = 0;
   }
  }
  $46 = $47&1;
  HEAP32[$0>>2] = $46;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
  break;
 }
 default: {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 }
 return (0)|0;
}
function _sqlite3VectorFieldSubexpr($pVector,$i) {
 $pVector = $pVector|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pVector;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprIsVector($3)|0);
 $5 = ($4|0)!=(0);
 $6 = HEAP32[$1>>2]|0;
 if (!($5)) {
  HEAP32[$0>>2] = $6;
  $32 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(119);
 if (!($9)) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 38|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)==(119);
  if (!($14)) {
   $24 = HEAP32[$2>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 20|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ((($27)) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + (($24*20)|0)|0);
   $31 = HEAP32[$30>>2]|0;
   HEAP32[$0>>2] = $31;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 4|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) + (($15*20)|0)|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$0>>2] = $23;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _isCandidateForInOpt($pX) {
 $pX = $pX|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pEList = 0, $pRes = 0, $pSrc = 0, $pTab = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $p = sp + 20|0;
 $pSrc = sp + 16|0;
 $pEList = sp + 12|0;
 $pTab = sp + 8|0;
 $i = sp + 4|0;
 $pRes = sp;
 HEAP32[$1>>2] = $pX;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 2048;
 $6 = ($5|0)!=(0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = $9 & 32;
   $11 = ($10|0)!=(0);
   if ($11) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 20|0);
   $14 = HEAP32[$13>>2]|0;
   HEAP32[$p>>2] = $14;
   $15 = HEAP32[$p>>2]|0;
   $16 = ((($15)) + 48|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = $21 & 9;
   $23 = ($22|0)!=(0);
   if ($23) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $24 = HEAP32[$p>>2]|0;
   $25 = ((($24)) + 56|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0|0);
   if ($27) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $28 = HEAP32[$p>>2]|0;
   $29 = ((($28)) + 32|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = ($30|0)!=(0|0);
   if ($31) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $32 = HEAP32[$p>>2]|0;
   $33 = ((($32)) + 28|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pSrc>>2] = $34;
   $35 = HEAP32[$pSrc>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)!=(1);
   if ($37) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $38 = HEAP32[$pSrc>>2]|0;
   $39 = ((($38)) + 8|0);
   $40 = ((($39)) + 20|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = ($41|0)!=(0|0);
   if ($42) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $43 = HEAP32[$pSrc>>2]|0;
   $44 = ((($43)) + 8|0);
   $45 = ((($44)) + 16|0);
   $46 = HEAP32[$45>>2]|0;
   HEAP32[$pTab>>2] = $46;
   $47 = HEAP32[$pTab>>2]|0;
   $48 = ((($47)) + 42|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = $50 & 16;
   $52 = ($51|0)!=(0);
   if ($52) {
    HEAP32[$0>>2] = 0;
    break;
   }
   $53 = HEAP32[$p>>2]|0;
   $54 = HEAP32[$53>>2]|0;
   HEAP32[$pEList>>2] = $54;
   HEAP32[$i>>2] = 0;
   while(1) {
    $55 = HEAP32[$i>>2]|0;
    $56 = HEAP32[$pEList>>2]|0;
    $57 = HEAP32[$56>>2]|0;
    $58 = ($55|0)<($57|0);
    if (!($58)) {
     label = 24;
     break;
    }
    $59 = HEAP32[$i>>2]|0;
    $60 = HEAP32[$pEList>>2]|0;
    $61 = ((($60)) + 4|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = (($62) + (($59*20)|0)|0);
    $64 = HEAP32[$63>>2]|0;
    HEAP32[$pRes>>2] = $64;
    $65 = HEAP32[$pRes>>2]|0;
    $66 = HEAP8[$65>>0]|0;
    $67 = $66&255;
    $68 = ($67|0)!=(152);
    if ($68) {
     label = 22;
     break;
    }
    $69 = HEAP32[$i>>2]|0;
    $70 = (($69) + 1)|0;
    HEAP32[$i>>2] = $70;
   }
   if ((label|0) == 22) {
    HEAP32[$0>>2] = 0;
    break;
   }
   else if ((label|0) == 24) {
    $71 = HEAP32[$p>>2]|0;
    HEAP32[$0>>2] = $71;
    break;
   }
  } else {
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _sqlite3CompareAffinity($pExpr,$aff2) {
 $pExpr = $pExpr|0;
 $aff2 = $aff2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff1 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 6|0;
 $1 = sp;
 $2 = sp + 5|0;
 $aff1 = sp + 4|0;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $aff2;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff1>>0] = $4;
 $5 = HEAP8[$aff1>>0]|0;
 $6 = $5 << 24 >> 24;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP8[$2>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP8[$aff1>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = ($12|0)>=(67);
   if (!($13)) {
    $14 = HEAP8[$2>>0]|0;
    $15 = $14 << 24 >> 24;
    $16 = ($15|0)>=(67);
    if (!($16)) {
     HEAP8[$0>>0] = 65;
     $27 = HEAP8[$0>>0]|0;
     STACKTOP = sp;return ($27|0);
    }
   }
   HEAP8[$0>>0] = 67;
   $27 = HEAP8[$0>>0]|0;
   STACKTOP = sp;return ($27|0);
  }
 }
 $17 = HEAP8[$aff1>>0]|0;
 $18 = ($17<<24>>24)!=(0);
 $19 = HEAP8[$2>>0]|0;
 $20 = ($19<<24>>24)!=(0);
 $or$cond = $18 | $20;
 if ($or$cond) {
  $21 = HEAP8[$aff1>>0]|0;
  $22 = $21 << 24 >> 24;
  $23 = HEAP8[$2>>0]|0;
  $24 = $23 << 24 >> 24;
  $25 = (($22) + ($24))|0;
  $26 = $25&255;
  HEAP8[$0>>0] = $26;
  $27 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  HEAP8[$0>>0] = 65;
  $27 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _sqlite3BinaryCompareCollSeq($pParse,$pLeft,$pRight) {
 $pParse = $pParse|0;
 $pLeft = $pLeft|0;
 $pRight = $pRight|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLeft;
 HEAP32[$2>>2] = $pRight;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 256;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = (_sqlite3ExprCollSeq($8,$9)|0);
  HEAP32[$pColl>>2] = $10;
  $29 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = $15 & 256;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (_sqlite3ExprCollSeq($18,$19)|0);
   HEAP32[$pColl>>2] = $20;
   $29 = HEAP32[$pColl>>2]|0;
   STACKTOP = sp;return ($29|0);
  }
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = (_sqlite3ExprCollSeq($21,$22)|0);
 HEAP32[$pColl>>2] = $23;
 $24 = HEAP32[$pColl>>2]|0;
 $25 = ($24|0)!=(0|0);
 if ($25) {
  $29 = HEAP32[$pColl>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (_sqlite3ExprCollSeq($26,$27)|0);
 HEAP32[$pColl>>2] = $28;
 $29 = HEAP32[$pColl>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3SetHasNullFlag($v,$iCur,$regHasNull) {
 $v = $v|0;
 $iCur = $iCur|0;
 $regHasNull = $regHasNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $addr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $addr1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $regHasNull;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($3,76,0,$4)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3VdbeAddOp1($5,57,$6)|0);
 HEAP32[$addr1>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($8,96,$9,0,$10)|0);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3VdbeChangeP5($11,-128);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($12,$13);
 STACKTOP = sp;return;
}
function _sqlite3InRhsIsConstant($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pLHS = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pLHS = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pIn;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pLHS>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3ExprIsConstant($6)|0);
 HEAP32[$res>>2] = $7;
 $8 = HEAP32[$pLHS>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 12|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3SelectDestInit($pDest,$eDest,$iParm) {
 $pDest = $pDest|0;
 $eDest = $eDest|0;
 $iParm = $iParm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pDest;
 HEAP32[$1>>2] = $eDest;
 HEAP32[$2>>2] = $iParm;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3&255;
 $5 = HEAP32[$0>>2]|0;
 HEAP8[$5>>0] = $4;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 4|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 16|0);
 HEAP32[$14>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeAtInit($pParse,$pExpr,$regDest,$reusable) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $regDest = $regDest|0;
 $reusable = $reusable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $p = sp + 4|0;
 $pItem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $regDest;
 HEAP8[$3>>0] = $reusable;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 80|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3ExprDup($8,$9,0)|0);
 HEAP32[$1>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$p>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_sqlite3ExprListAppend($11,$12,$13)|0);
 HEAP32[$p>>2] = $14;
 $15 = HEAP32[$p>>2]|0;
 $16 = ($15|0)!=(0|0);
 if (!($16)) {
  $37 = HEAP32[$p>>2]|0;
  $38 = HEAP32[$0>>2]|0;
  $39 = ((($38)) + 80|0);
  HEAP32[$39>>2] = $37;
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$p>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) - 1)|0;
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + (($19*20)|0)|0);
 HEAP32[$pItem>>2] = $23;
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$pItem>>2]|0;
 $26 = ((($25)) + 16|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP8[$3>>0]|0;
 $28 = $27&255;
 $29 = HEAP32[$pItem>>2]|0;
 $30 = ((($29)) + 13|0);
 $31 = $28&255;
 $32 = HEAP8[$30>>0]|0;
 $33 = $31 & 1;
 $34 = ($33 << 2)&255;
 $35 = $32 & -5;
 $36 = $35 | $34;
 HEAP8[$30>>0] = $36;
 $37 = HEAP32[$p>>2]|0;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 80|0);
 HEAP32[$39>>2] = $37;
 STACKTOP = sp;return;
}
function _exprIsConst($p,$initFlag,$iCur) {
 $p = $p|0;
 $initFlag = $initFlag|0;
 $iCur = $iCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $initFlag;
 HEAP32[$2>>2] = $iCur;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3&255;
 $5 = ((($w)) + 20|0);
 HEAP8[$5>>0] = $4;
 $6 = ((($w)) + 4|0);
 HEAP32[$6>>2] = 197;
 $7 = ((($w)) + 8|0);
 HEAP32[$7>>2] = 198;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($w)) + 24|0);
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$10)|0);
 $11 = ((($w)) + 20|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 STACKTOP = sp;return ($13|0);
}
function _exprNodeIsConstant($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if ($7) {
  $8 = HEAP32[$2>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10 & 1;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 20|0);
   HEAP8[$14>>0] = 0;
   HEAP32[$0>>2] = 2;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 L4: do {
  if ((label|0) == 4) {
   $15 = HEAP32[$2>>2]|0;
   $16 = HEAP8[$15>>0]|0;
   $17 = $16&255;
   L6: do {
    switch ($17|0) {
    case 151:  {
     $18 = HEAP32[$1>>2]|0;
     $19 = ((($18)) + 20|0);
     $20 = HEAP8[$19>>0]|0;
     $21 = $20&255;
     $22 = ($21|0)>=(4);
     if (!($22)) {
      $23 = HEAP32[$2>>2]|0;
      $24 = ((($23)) + 4|0);
      $25 = HEAP32[$24>>2]|0;
      $26 = $25 & 524288;
      $27 = ($26|0)!=(0);
      if (!($27)) {
       $28 = HEAP32[$1>>2]|0;
       $29 = ((($28)) + 20|0);
       HEAP8[$29>>0] = 0;
       HEAP32[$0>>2] = 2;
       break L4;
      }
     }
     HEAP32[$0>>2] = 0;
     break L4;
     break;
    }
    case 154: case 153: case 152: case 55:  {
     $30 = HEAP32[$1>>2]|0;
     $31 = ((($30)) + 20|0);
     $32 = HEAP8[$31>>0]|0;
     $33 = $32&255;
     $34 = ($33|0)==(3);
     if ($34) {
      $35 = HEAP32[$2>>2]|0;
      $36 = ((($35)) + 28|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = HEAP32[$1>>2]|0;
      $39 = ((($38)) + 24|0);
      $40 = HEAP32[$39>>2]|0;
      $41 = ($37|0)==($40|0);
      if ($41) {
       HEAP32[$0>>2] = 0;
       break L4;
      }
     }
     $42 = HEAP32[$1>>2]|0;
     $43 = ((($42)) + 20|0);
     HEAP8[$43>>0] = 0;
     HEAP32[$0>>2] = 2;
     break L4;
     break;
    }
    case 135:  {
     $44 = HEAP32[$1>>2]|0;
     $45 = ((($44)) + 20|0);
     $46 = HEAP8[$45>>0]|0;
     $47 = $46&255;
     $48 = ($47|0)==(5);
     if ($48) {
      $49 = HEAP32[$2>>2]|0;
      HEAP8[$49>>0] = 101;
      break L6;
     }
     $50 = HEAP32[$1>>2]|0;
     $51 = ((($50)) + 20|0);
     $52 = HEAP8[$51>>0]|0;
     $53 = $52&255;
     $54 = ($53|0)==(4);
     if ($54) {
      $55 = HEAP32[$1>>2]|0;
      $56 = ((($55)) + 20|0);
      HEAP8[$56>>0] = 0;
      HEAP32[$0>>2] = 2;
      break L4;
     }
     break;
    }
    default: {
    }
    }
   } while(0);
   HEAP32[$0>>2] = 0;
  }
 } while(0);
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _selectNodeIsConstant($pWalker,$NotUsed) {
 $pWalker = $pWalker|0;
 $NotUsed = $NotUsed|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $NotUsed;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 20|0);
 HEAP8[$3>>0] = 0;
 STACKTOP = sp;return 2;
}
function _binaryCompareP5($pExpr1,$pExpr2,$jumpIfNull) {
 $pExpr1 = $pExpr1|0;
 $pExpr2 = $pExpr2|0;
 $jumpIfNull = $jumpIfNull|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 $aff = sp + 12|0;
 HEAP32[$0>>2] = $pExpr1;
 HEAP32[$1>>2] = $pExpr2;
 HEAP32[$2>>2] = $jumpIfNull;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP8[$aff>>0]|0;
 $7 = (_sqlite3CompareAffinity($5,$6)|0);
 $8 = $7&255;
 $9 = HEAP32[$2>>2]|0;
 $10 = $9&255;
 $11 = $10&255;
 $12 = $8 | $11;
 $13 = $12&255;
 HEAP8[$aff>>0] = $13;
 $14 = HEAP8[$aff>>0]|0;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3ExprIsConstantNotJoin($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_exprIsConst($1,2,0)|0);
 STACKTOP = sp;return ($2|0);
}
function _exprCodeSubselect($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $reg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $reg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$reg>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(119);
 if (!($5)) {
  $9 = HEAP32[$reg>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3CodeSubselect($6,$7,0,0)|0);
 HEAP32[$reg>>2] = $8;
 $9 = HEAP32[$reg>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _exprVectorRegister($pParse,$pVector,$iField,$regSelect,$ppExpr,$pRegFree) {
 $pParse = $pParse|0;
 $pVector = $pVector|0;
 $iField = $iField|0;
 $regSelect = $regSelect|0;
 $ppExpr = $ppExpr|0;
 $pRegFree = $pRegFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $6 = sp;
 $op = sp + 28|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pVector;
 HEAP32[$3>>2] = $iField;
 HEAP32[$4>>2] = $regSelect;
 HEAP32[$5>>2] = $ppExpr;
 HEAP32[$6>>2] = $pRegFree;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 HEAP8[$op>>0] = $8;
 $9 = HEAP8[$op>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)==(157);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$3>>2]|0;
  $14 = (_sqlite3VectorFieldSubexpr($12,$13)|0);
  $15 = HEAP32[$5>>2]|0;
  HEAP32[$15>>2] = $14;
  $16 = HEAP32[$2>>2]|0;
  $17 = ((($16)) + 28|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$3>>2]|0;
  $20 = (($18) + ($19))|0;
  HEAP32[$0>>2] = $20;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $21 = HEAP8[$op>>0]|0;
 $22 = $21&255;
 $23 = ($22|0)==(119);
 $24 = HEAP32[$3>>2]|0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 20|0);
 if ($23) {
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + (($24*20)|0)|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$5>>2]|0;
  HEAP32[$33>>2] = $32;
  $34 = HEAP32[$4>>2]|0;
  $35 = HEAP32[$3>>2]|0;
  $36 = (($34) + ($35))|0;
  HEAP32[$0>>2] = $36;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 } else {
  $37 = HEAP32[$26>>2]|0;
  $38 = ((($37)) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + (($24*20)|0)|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = HEAP32[$5>>2]|0;
  HEAP32[$42>>2] = $41;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$5>>2]|0;
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$6>>2]|0;
  $47 = (_sqlite3ExprCodeTemp($43,$45,$46)|0);
  HEAP32[$0>>2] = $47;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 return (0)|0;
}
function _sqlite3VdbeAddOp4Dup8($p,$op,$p1,$p2,$p3,$zP4,$p4type) {
 $p = $p|0;
 $op = $op|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 $p3 = $p3|0;
 $zP4 = $zP4|0;
 $p4type = $p4type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p4copy = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $6 = sp + 4|0;
 $p4copy = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $p1;
 HEAP32[$3>>2] = $p2;
 HEAP32[$4>>2] = $p3;
 HEAP32[$5>>2] = $zP4;
 HEAP32[$6>>2] = $p4type;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3VdbeDb($7)|0);
 $9 = (_sqlite3DbMallocRawNN($8,8,0)|0);
 HEAP32[$p4copy>>2] = $9;
 $10 = HEAP32[$p4copy>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$p4copy>>2]|0;
  $13 = HEAP32[$5>>2]|0;
  ;HEAP8[$12>>0]=HEAP8[$13>>0]|0;HEAP8[$12+1>>0]=HEAP8[$13+1>>0]|0;HEAP8[$12+2>>0]=HEAP8[$13+2>>0]|0;HEAP8[$12+3>>0]=HEAP8[$13+3>>0]|0;HEAP8[$12+4>>0]=HEAP8[$13+4>>0]|0;HEAP8[$12+5>>0]=HEAP8[$13+5>>0]|0;HEAP8[$12+6>>0]=HEAP8[$13+6>>0]|0;HEAP8[$12+7>>0]=HEAP8[$13+7>>0]|0;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$p4copy>>2]|0;
 $20 = HEAP32[$6>>2]|0;
 $21 = (_sqlite3VdbeAddOp4($14,$15,$16,$17,$18,$19,$20)|0);
 STACKTOP = sp;return ($21|0);
}
function _sqlite3ExprCachePinRegister($pParse,$iReg) {
 $pParse = $pParse|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iReg;
 HEAP32[$i>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 152|0);
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 25|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$p>>2]|0;
  $11 = ((($10)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)==($13|0);
  if ($14) {
   $15 = HEAP32[$p>>2]|0;
   $16 = ((($15)) + 6|0);
   HEAP8[$16>>0] = 0;
  }
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
  $19 = HEAP32[$p>>2]|0;
  $20 = ((($19)) + 20|0);
  HEAP32[$p>>2] = $20;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheStore($pParse,$iTab,$iCol,$iReg) {
 $pParse = $pParse|0;
 $iTab = $iTab|0;
 $iCol = $iCol|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $idxLru = 0, $minLru = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $i = sp + 12|0;
 $minLru = sp + 8|0;
 $idxLru = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $iCol;
 HEAP32[$3>>2] = $iReg;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 64|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = $8 & 2;
 $10 = ($9|0)!=(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 25|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)>=(10);
 if ($15) {
  HEAP32[$minLru>>2] = 2147483647;
  HEAP32[$idxLru>>2] = -1;
  HEAP32[$i>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 152|0);
  HEAP32[$p>>2] = $17;
  while(1) {
   $18 = HEAP32[$i>>2]|0;
   $19 = ($18|0)<(10);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$minLru>>2]|0;
   $24 = ($22|0)<($23|0);
   if ($24) {
    $25 = HEAP32[$i>>2]|0;
    HEAP32[$idxLru>>2] = $25;
    $26 = HEAP32[$p>>2]|0;
    $27 = ((($26)) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    HEAP32[$minLru>>2] = $28;
   }
   $29 = HEAP32[$i>>2]|0;
   $30 = (($29) + 1)|0;
   HEAP32[$i>>2] = $30;
   $31 = HEAP32[$p>>2]|0;
   $32 = ((($31)) + 20|0);
   HEAP32[$p>>2] = $32;
  }
  $33 = HEAP32[$idxLru>>2]|0;
  $34 = HEAP32[$0>>2]|0;
  $35 = ((($34)) + 152|0);
  $36 = (($35) + (($33*20)|0)|0);
  HEAP32[$p>>2] = $36;
 } else {
  $37 = HEAP32[$0>>2]|0;
  $38 = ((($37)) + 25|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = (($39) + 1)<<24>>24;
  HEAP8[$38>>0] = $40;
  $41 = $39&255;
  $42 = HEAP32[$0>>2]|0;
  $43 = ((($42)) + 152|0);
  $44 = (($43) + (($41*20)|0)|0);
  HEAP32[$p>>2] = $44;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 64|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$p>>2]|0;
 $49 = ((($48)) + 8|0);
 HEAP32[$49>>2] = $47;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$p>>2]|0;
 HEAP32[$51>>2] = $50;
 $52 = HEAP32[$2>>2]|0;
 $53 = $52&65535;
 $54 = HEAP32[$p>>2]|0;
 $55 = ((($54)) + 4|0);
 HEAP16[$55>>1] = $53;
 $56 = HEAP32[$3>>2]|0;
 $57 = HEAP32[$p>>2]|0;
 $58 = ((($57)) + 12|0);
 HEAP32[$58>>2] = $56;
 $59 = HEAP32[$p>>2]|0;
 $60 = ((($59)) + 6|0);
 HEAP8[$60>>0] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 68|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = (($63) + 1)|0;
 HEAP32[$62>>2] = $64;
 $65 = HEAP32[$p>>2]|0;
 $66 = ((($65)) + 16|0);
 HEAP32[$66>>2] = $63;
 STACKTOP = sp;return;
}
function _collationMatch($zColl,$pIndex) {
 $zColl = $zColl|0;
 $pIndex = $pIndex|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $z = sp;
 HEAP32[$1>>2] = $zColl;
 HEAP32[$2>>2] = $pIndex;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = ((($4)) + 52|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = ($3|0)<($7|0);
  if (!($8)) {
   label = 7;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 32|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<2)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$z>>2] = $14;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<1)|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20 << 16 >> 16;
  $22 = ($21|0)>=(0);
  if ($22) {
   $23 = HEAP32[$z>>2]|0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_sqlite3StrICmp($23,$24)|0);
   $26 = (0)==($25|0);
   if ($26) {
    label = 5;
    break;
   }
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 1;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _codeAttach($pParse,$type,$pFunc,$pAuthArg,$pFilename,$pDbname,$pKey) {
 $pParse = $pParse|0;
 $type = $type|0;
 $pFunc = $pFunc|0;
 $pAuthArg = $pAuthArg|0;
 $pFilename = $pFilename|0;
 $pDbname = $pDbname|0;
 $pKey = $pKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, $regArgs = 0, $sName = 0, $v = 0, $zAuthArg = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $5 = sp + 56|0;
 $6 = sp + 52|0;
 $rc = sp + 48|0;
 $sName = sp + 16|0;
 $v = sp + 12|0;
 $db = sp + 8|0;
 $regArgs = sp + 4|0;
 $zAuthArg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $type;
 HEAP32[$2>>2] = $pFunc;
 HEAP32[$3>>2] = $pAuthArg;
 HEAP32[$4>>2] = $pFilename;
 HEAP32[$5>>2] = $pDbname;
 HEAP32[$6>>2] = $pKey;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 ;HEAP32[$sName>>2]=0|0;HEAP32[$sName+4>>2]=0|0;HEAP32[$sName+8>>2]=0|0;HEAP32[$sName+12>>2]=0|0;HEAP32[$sName+16>>2]=0|0;HEAP32[$sName+20>>2]=0|0;HEAP32[$sName+24>>2]=0|0;HEAP32[$sName+28>>2]=0|0;
 $9 = HEAP32[$0>>2]|0;
 HEAP32[$sName>>2] = $9;
 $10 = HEAP32[$4>>2]|0;
 $11 = (_resolveAttachExpr($sName,$10)|0);
 HEAP32[$rc>>2] = $11;
 $12 = (0)!=($11|0);
 do {
  if (!($12)) {
   $13 = HEAP32[$5>>2]|0;
   $14 = (_resolveAttachExpr($sName,$13)|0);
   HEAP32[$rc>>2] = $14;
   $15 = (0)!=($14|0);
   if (!($15)) {
    $16 = HEAP32[$6>>2]|0;
    $17 = (_resolveAttachExpr($sName,$16)|0);
    HEAP32[$rc>>2] = $17;
    $18 = (0)!=($17|0);
    if (!($18)) {
     $19 = HEAP32[$3>>2]|0;
     $20 = ($19|0)!=(0|0);
     if ($20) {
      $21 = HEAP32[$3>>2]|0;
      $22 = HEAP8[$21>>0]|0;
      $23 = $22&255;
      $24 = ($23|0)==(97);
      if ($24) {
       $25 = HEAP32[$3>>2]|0;
       $26 = ((($25)) + 8|0);
       $27 = HEAP32[$26>>2]|0;
       HEAP32[$zAuthArg>>2] = $27;
      } else {
       HEAP32[$zAuthArg>>2] = 0;
      }
      $28 = HEAP32[$0>>2]|0;
      $29 = HEAP32[$1>>2]|0;
      $30 = HEAP32[$zAuthArg>>2]|0;
      $31 = (_sqlite3AuthCheck($28,$29,$30,0,0)|0);
      HEAP32[$rc>>2] = $31;
      $32 = HEAP32[$rc>>2]|0;
      $33 = ($32|0)!=(0);
      if ($33) {
       break;
      }
     }
     $34 = HEAP32[$0>>2]|0;
     $35 = (_sqlite3GetVdbe($34)|0);
     HEAP32[$v>>2] = $35;
     $36 = HEAP32[$0>>2]|0;
     $37 = (_sqlite3GetTempRange($36,4)|0);
     HEAP32[$regArgs>>2] = $37;
     $38 = HEAP32[$0>>2]|0;
     $39 = HEAP32[$4>>2]|0;
     $40 = HEAP32[$regArgs>>2]|0;
     _sqlite3ExprCode($38,$39,$40);
     $41 = HEAP32[$0>>2]|0;
     $42 = HEAP32[$5>>2]|0;
     $43 = HEAP32[$regArgs>>2]|0;
     $44 = (($43) + 1)|0;
     _sqlite3ExprCode($41,$42,$44);
     $45 = HEAP32[$0>>2]|0;
     $46 = HEAP32[$6>>2]|0;
     $47 = HEAP32[$regArgs>>2]|0;
     $48 = (($47) + 2)|0;
     _sqlite3ExprCode($45,$46,$48);
     $49 = HEAP32[$v>>2]|0;
     $50 = ($49|0)!=(0|0);
     if ($50) {
      $51 = HEAP32[$v>>2]|0;
      $52 = HEAP32[$regArgs>>2]|0;
      $53 = (($52) + 3)|0;
      $54 = HEAP32[$2>>2]|0;
      $55 = HEAP8[$54>>0]|0;
      $56 = $55 << 24 >> 24;
      $57 = (($53) - ($56))|0;
      $58 = HEAP32[$regArgs>>2]|0;
      $59 = (($58) + 3)|0;
      $60 = HEAP32[$2>>2]|0;
      (_sqlite3VdbeAddOp4($51,89,0,$57,$59,$60,-5)|0);
      $61 = HEAP32[$v>>2]|0;
      $62 = HEAP32[$2>>2]|0;
      $63 = HEAP8[$62>>0]|0;
      _sqlite3VdbeChangeP5($61,$63);
      $64 = HEAP32[$v>>2]|0;
      $65 = HEAP32[$1>>2]|0;
      $66 = ($65|0)==(24);
      $67 = $66&1;
      (_sqlite3VdbeAddOp1($64,150,$67)|0);
     }
    }
   }
  }
 } while(0);
 $68 = HEAP32[$db>>2]|0;
 $69 = HEAP32[$4>>2]|0;
 _sqlite3ExprDelete($68,$69);
 $70 = HEAP32[$db>>2]|0;
 $71 = HEAP32[$5>>2]|0;
 _sqlite3ExprDelete($70,$71);
 $72 = HEAP32[$db>>2]|0;
 $73 = HEAP32[$6>>2]|0;
 _sqlite3ExprDelete($72,$73);
 STACKTOP = sp;return;
}
function _resolveAttachExpr($pName,$pExpr) {
 $pName = $pName|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pName;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = HEAP8[$4>>0]|0;
   $6 = $5&255;
   $7 = ($6|0)!=(55);
   if ($7) {
    $8 = HEAP32[$0>>2]|0;
    $9 = HEAP32[$1>>2]|0;
    $10 = (_sqlite3ResolveExprNames($8,$9)|0);
    HEAP32[$rc>>2] = $10;
    break;
   } else {
    $11 = HEAP32[$1>>2]|0;
    HEAP8[$11>>0] = 97;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _detachFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pDb = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_buffer6 = 0, $zErr = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer6 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $zName = sp + 40|0;
 $db = sp + 36|0;
 $i = sp + 32|0;
 $pDb = sp + 28|0;
 $zErr = sp + 56|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zName>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3_context_db_handle($6)|0);
 HEAP32[$db>>2] = $7;
 HEAP32[$pDb>>2] = 0;
 $8 = HEAP32[$zName>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$zName>>2] = 18402;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$db>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$db>>2]|0;
  $17 = ((($16)) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  HEAP32[$pDb>>2] = $19;
  $20 = HEAP32[$pDb>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)==(0|0);
  if (!($23)) {
   $24 = HEAP32[$pDb>>2]|0;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$zName>>2]|0;
   $27 = (_sqlite3StrICmp($25,$26)|0);
   $28 = ($27|0)==(0);
   if ($28) {
    break;
   }
  }
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
 }
 $31 = HEAP32[$i>>2]|0;
 $32 = HEAP32[$db>>2]|0;
 $33 = ((($32)) + 20|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($31|0)>=($34|0);
 do {
  if ($35) {
   $36 = HEAP32[$zName>>2]|0;
   HEAP32[$vararg_buffer>>2] = $36;
   (_sqlite3_snprintf(128,$zErr,31331,$vararg_buffer)|0);
  } else {
   $37 = HEAP32[$i>>2]|0;
   $38 = ($37|0)<(2);
   if ($38) {
    $39 = HEAP32[$zName>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $39;
    (_sqlite3_snprintf(128,$zErr,31352,$vararg_buffer1)|0);
    break;
   }
   $40 = HEAP32[$db>>2]|0;
   $41 = ((($40)) + 67|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = ($42<<24>>24)!=(0);
   if (!($43)) {
    (_sqlite3_snprintf(128,$zErr,31378,$vararg_buffer4)|0);
    break;
   }
   $44 = HEAP32[$pDb>>2]|0;
   $45 = ((($44)) + 4|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = (_sqlite3BtreeIsInReadTrans($46)|0);
   $48 = ($47|0)!=(0);
   if (!($48)) {
    $49 = HEAP32[$pDb>>2]|0;
    $50 = ((($49)) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = (_sqlite3BtreeIsInBackup($51)|0);
    $53 = ($52|0)!=(0);
    if (!($53)) {
     $55 = HEAP32[$pDb>>2]|0;
     $56 = ((($55)) + 4|0);
     $57 = HEAP32[$56>>2]|0;
     (_sqlite3BtreeClose($57)|0);
     $58 = HEAP32[$pDb>>2]|0;
     $59 = ((($58)) + 4|0);
     HEAP32[$59>>2] = 0;
     $60 = HEAP32[$pDb>>2]|0;
     $61 = ((($60)) + 12|0);
     HEAP32[$61>>2] = 0;
     $62 = HEAP32[$db>>2]|0;
     _sqlite3CollapseDatabaseArray($62);
     STACKTOP = sp;return;
    }
   }
   $54 = HEAP32[$zName>>2]|0;
   HEAP32[$vararg_buffer6>>2] = $54;
   (_sqlite3_snprintf(128,$zErr,31420,$vararg_buffer6)|0);
  }
 } while(0);
 $63 = HEAP32[$0>>2]|0;
 _sqlite3_result_error($63,$zErr,-1);
 STACKTOP = sp;return;
}
function _sqlite3BtreeClose($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $pBt = 0, $pCur = 0, $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pBt = sp + 8|0;
 $pCur = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pCur>>2] = $7;
 while(1) {
  $8 = HEAP32[$pCur>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pCur>>2]|0;
  HEAP32[$pTmp>>2] = $10;
  $11 = HEAP32[$pCur>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pCur>>2] = $13;
  $14 = HEAP32[$pTmp>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ($15|0)==($16|0);
  if (!($17)) {
   continue;
  }
  $18 = HEAP32[$pTmp>>2]|0;
  (_sqlite3BtreeCloseCursor($18)|0);
 }
 $19 = HEAP32[$0>>2]|0;
 (_sqlite3BtreeRollback($19,0,0)|0);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 9|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = ($22<<24>>24)!=(0);
 if ($23) {
  $24 = HEAP32[$pBt>>2]|0;
  $25 = (_removeFromSharingList($24)|0);
  $26 = ($25|0)!=(0);
  if ($26) {
   label = 7;
  }
 } else {
  label = 7;
 }
 if ((label|0) == 7) {
  $27 = HEAP32[$pBt>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  (_sqlite3PagerClose($28)|0);
  $29 = HEAP32[$pBt>>2]|0;
  $30 = ((($29)) + 52|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if ($32) {
   $33 = HEAP32[$pBt>>2]|0;
   $34 = ((($33)) + 48|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)!=(0|0);
   if ($36) {
    $37 = HEAP32[$pBt>>2]|0;
    $38 = ((($37)) + 52|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pBt>>2]|0;
    $41 = ((($40)) + 48|0);
    $42 = HEAP32[$41>>2]|0;
    FUNCTION_TABLE_vi[$39 & 255]($42);
   }
  }
  $43 = HEAP32[$pBt>>2]|0;
  $44 = ((($43)) + 48|0);
  $45 = HEAP32[$44>>2]|0;
  _sqlite3DbFree(0,$45);
  $46 = HEAP32[$pBt>>2]|0;
  _freeTempSpace($46);
  $47 = HEAP32[$pBt>>2]|0;
  _sqlite3_free($47);
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 28|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50|0)!=(0|0);
 if ($51) {
  $52 = HEAP32[$0>>2]|0;
  $53 = ((($52)) + 24|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$0>>2]|0;
  $56 = ((($55)) + 28|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = ((($57)) + 24|0);
  HEAP32[$58>>2] = $54;
 }
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 24|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = ($61|0)!=(0|0);
 if (!($62)) {
  $70 = HEAP32[$0>>2]|0;
  _sqlite3_free($70);
  STACKTOP = sp;return 0;
 }
 $63 = HEAP32[$0>>2]|0;
 $64 = ((($63)) + 28|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$0>>2]|0;
 $67 = ((($66)) + 24|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ((($68)) + 28|0);
 HEAP32[$69>>2] = $65;
 $70 = HEAP32[$0>>2]|0;
 _sqlite3_free($70);
 STACKTOP = sp;return 0;
}
function _sqlite3CollapseDatabaseArray($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $j = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$j>>2] = 2;
 HEAP32[$i>>2] = 2;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  HEAP32[$pDb>>2] = $10;
  $11 = HEAP32[$pDb>>2]|0;
  $12 = ((($11)) + 4|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)==(0|0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$pDb>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   _sqlite3DbFree($15,$17);
   $18 = HEAP32[$pDb>>2]|0;
   HEAP32[$18>>2] = 0;
  } else {
   $19 = HEAP32[$j>>2]|0;
   $20 = HEAP32[$i>>2]|0;
   $21 = ($19|0)<($20|0);
   if ($21) {
    $22 = HEAP32[$j>>2]|0;
    $23 = HEAP32[$0>>2]|0;
    $24 = ((($23)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = (($25) + ($22<<4)|0);
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$0>>2]|0;
    $29 = ((($28)) + 16|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = (($30) + ($27<<4)|0);
    ;HEAP32[$26>>2]=HEAP32[$31>>2]|0;HEAP32[$26+4>>2]=HEAP32[$31+4>>2]|0;HEAP32[$26+8>>2]=HEAP32[$31+8>>2]|0;HEAP32[$26+12>>2]=HEAP32[$31+12>>2]|0;
   }
   $32 = HEAP32[$j>>2]|0;
   $33 = (($32) + 1)|0;
   HEAP32[$j>>2] = $33;
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 $36 = HEAP32[$j>>2]|0;
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 20|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 20|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)<=(2);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = ((($43)) + 16|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 392|0);
 $48 = ($45|0)!=($47|0);
 if (!($48)) {
  STACKTOP = sp;return;
 }
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 392|0);
 $51 = HEAP32[$0>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$52>>2]|0;
 ;HEAP32[$50>>2]=HEAP32[$53>>2]|0;HEAP32[$50+4>>2]=HEAP32[$53+4>>2]|0;HEAP32[$50+8>>2]=HEAP32[$53+8>>2]|0;HEAP32[$50+12>>2]=HEAP32[$53+12>>2]|0;HEAP32[$50+16>>2]=HEAP32[$53+16>>2]|0;HEAP32[$50+20>>2]=HEAP32[$53+20>>2]|0;HEAP32[$50+24>>2]=HEAP32[$53+24>>2]|0;HEAP32[$50+28>>2]=HEAP32[$53+28>>2]|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 16|0);
 $57 = HEAP32[$56>>2]|0;
 _sqlite3DbFree($54,$57);
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 392|0);
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 16|0);
 HEAP32[$61>>2] = $59;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCloseCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pBt = 0, $pBtree = 0, $pPrev = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $pBtree = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $pPrev = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pBtree>>2] = $2;
 $3 = HEAP32[$pBtree>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return 0;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$pBtree>>2]|0;
 _sqlite3BtreeEnter($8);
 $9 = HEAP32[$0>>2]|0;
 _sqlite3BtreeClearCursor($9);
 $10 = HEAP32[$pBt>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ($12|0)==($13|0);
 L4: do {
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$pBt>>2]|0;
   $19 = ((($18)) + 8|0);
   HEAP32[$19>>2] = $17;
  } else {
   $20 = HEAP32[$pBt>>2]|0;
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   HEAP32[$pPrev>>2] = $22;
   while(1) {
    $23 = HEAP32[$pPrev>>2]|0;
    $24 = ((($23)) + 8|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$0>>2]|0;
    $27 = ($25|0)==($26|0);
    if ($27) {
     break;
    }
    $33 = HEAP32[$pPrev>>2]|0;
    $34 = ((($33)) + 8|0);
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$pPrev>>2] = $35;
    $36 = HEAP32[$pPrev>>2]|0;
    $37 = ($36|0)!=(0|0);
    if (!($37)) {
     break L4;
    }
   }
   $28 = HEAP32[$0>>2]|0;
   $29 = ((($28)) + 8|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$pPrev>>2]|0;
   $32 = ((($31)) + 8|0);
   HEAP32[$32>>2] = $30;
  }
 } while(0);
 HEAP32[$i>>2] = 0;
 while(1) {
  $38 = HEAP32[$i>>2]|0;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 68|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = ($38|0)<=($42|0);
  if (!($43)) {
   break;
  }
  $44 = HEAP32[$i>>2]|0;
  $45 = HEAP32[$0>>2]|0;
  $46 = ((($45)) + 120|0);
  $47 = (($46) + ($44<<2)|0);
  $48 = HEAP32[$47>>2]|0;
  _releasePage($48);
  $49 = HEAP32[$i>>2]|0;
  $50 = (($49) + 1)|0;
  HEAP32[$i>>2] = $50;
 }
 $51 = HEAP32[$pBt>>2]|0;
 _unlockBtreeIfUnused($51);
 $52 = HEAP32[$0>>2]|0;
 $53 = ((($52)) + 12|0);
 $54 = HEAP32[$53>>2]|0;
 _sqlite3_free($54);
 STACKTOP = sp;return 0;
}
function _removeFromSharingList($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pList = 0, $removed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pList = sp + 4|0;
 $removed = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$removed>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + -1)|0;
 HEAP32[$2>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 64|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<=(0);
 if (!($8)) {
  $32 = HEAP32[$removed>>2]|0;
  STACKTOP = sp;return ($32|0);
 }
 $9 = HEAP32[4024>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ($9|0)==($10|0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 68|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[4024>>2] = $14;
 } else {
  $15 = HEAP32[4024>>2]|0;
  HEAP32[$pList>>2] = $15;
  while(1) {
   $16 = HEAP32[$pList>>2]|0;
   $17 = ($16|0)!=(0|0);
   if ($17) {
    $18 = HEAP32[$pList>>2]|0;
    $19 = ((($18)) + 68|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ($20|0)!=($21|0);
    $33 = $22;
   } else {
    $33 = 0;
   }
   $23 = HEAP32[$pList>>2]|0;
   if (!($33)) {
    break;
   }
   $24 = ((($23)) + 68|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$pList>>2] = $25;
  }
  $26 = ($23|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$0>>2]|0;
   $28 = ((($27)) + 68|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$pList>>2]|0;
   $31 = ((($30)) + 68|0);
   HEAP32[$31>>2] = $29;
  }
 }
 HEAP32[$removed>>2] = 1;
 $32 = HEAP32[$removed>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _sqlite3BtreeClearCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 48|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 48|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 66|0);
 HEAP8[$7>>0] = 0;
 STACKTOP = sp;return;
}
function _releasePage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 _releasePageNotNull($3);
 STACKTOP = sp;return;
}
function _unlockBtreeIfUnused($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 20|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pPage1>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 12|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$pPage1>>2]|0;
 _releasePageNotNull($15);
 STACKTOP = sp;return;
}
function _releasePageNotNull($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 72|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3PagerUnrefNotNull($3);
 STACKTOP = sp;return;
}
function _attachFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $db = 0, $flags = 0, $i = 0, $iDb = 0, $or$cond = 0, $pPager = 0, $pVfs = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer13 = 0, $vararg_buffer2 = 0, $vararg_buffer4 = 0;
 var $vararg_buffer7 = 0, $vararg_buffer9 = 0, $z = 0, $zErr = 0, $zErrDyn = 0, $zFile = 0, $zName = 0, $zPath = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer11 = sp + 40|0;
 $vararg_buffer9 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $i = sp + 104|0;
 $rc = sp + 100|0;
 $db = sp + 96|0;
 $zName = sp + 92|0;
 $zFile = sp + 88|0;
 $zPath = sp + 84|0;
 $zErr = sp + 80|0;
 $flags = sp + 76|0;
 $aNew = sp + 72|0;
 $zErrDyn = sp + 68|0;
 $pVfs = sp + 64|0;
 $z = sp + 60|0;
 $pPager = sp + 56|0;
 $iDb = sp + 52|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 HEAP32[$zPath>>2] = 0;
 HEAP32[$zErr>>2] = 0;
 HEAP32[$zErrDyn>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zFile>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_text($10)|0);
 HEAP32[$zName>>2] = $11;
 $12 = HEAP32[$zFile>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  HEAP32[$zFile>>2] = 18402;
 }
 $14 = HEAP32[$zName>>2]|0;
 $15 = ($14|0)==(0|0);
 if ($15) {
  HEAP32[$zName>>2] = 18402;
 }
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$db>>2]|0;
 $20 = ((($19)) + 96|0);
 $21 = ((($20)) + 28|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($22) + 2)|0;
 $24 = ($18|0)>=($23|0);
 $25 = HEAP32[$db>>2]|0;
 do {
  if ($24) {
   $26 = HEAP32[$db>>2]|0;
   $27 = ((($26)) + 96|0);
   $28 = ((($27)) + 28|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$vararg_buffer>>2] = $29;
   $30 = (_sqlite3MPrintf($25,31456,$vararg_buffer)|0);
   HEAP32[$zErrDyn>>2] = $30;
  } else {
   $31 = ((($25)) + 67|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = ($32<<24>>24)!=(0);
   if (!($33)) {
    $34 = HEAP32[$db>>2]|0;
    $35 = (_sqlite3MPrintf($34,31493,$vararg_buffer2)|0);
    HEAP32[$zErrDyn>>2] = $35;
    break;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $36 = HEAP32[$i>>2]|0;
    $37 = HEAP32[$db>>2]|0;
    $38 = ((($37)) + 20|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($36|0)<($39|0);
    if (!($40)) {
     break;
    }
    $41 = HEAP32[$i>>2]|0;
    $42 = HEAP32[$db>>2]|0;
    $43 = ((($42)) + 16|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + ($41<<4)|0);
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$z>>2] = $46;
    $47 = HEAP32[$z>>2]|0;
    $48 = HEAP32[$zName>>2]|0;
    $49 = (_sqlite3StrICmp($47,$48)|0);
    $50 = ($49|0)==(0);
    if ($50) {
     label = 12;
     break;
    }
    $54 = HEAP32[$i>>2]|0;
    $55 = (($54) + 1)|0;
    HEAP32[$i>>2] = $55;
   }
   if ((label|0) == 12) {
    $51 = HEAP32[$db>>2]|0;
    $52 = HEAP32[$zName>>2]|0;
    HEAP32[$vararg_buffer4>>2] = $52;
    $53 = (_sqlite3MPrintf($51,31535,$vararg_buffer4)|0);
    HEAP32[$zErrDyn>>2] = $53;
    break;
   }
   $56 = HEAP32[$db>>2]|0;
   $57 = ((($56)) + 16|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$db>>2]|0;
   $60 = ((($59)) + 392|0);
   $61 = ($58|0)==($60|0);
   $62 = HEAP32[$db>>2]|0;
   do {
    if ($61) {
     $63 = (_sqlite3DbMallocRawNN($62,48,0)|0);
     HEAP32[$aNew>>2] = $63;
     $64 = HEAP32[$aNew>>2]|0;
     $65 = ($64|0)==(0|0);
     if ($65) {
      STACKTOP = sp;return;
     } else {
      $66 = HEAP32[$aNew>>2]|0;
      $67 = HEAP32[$db>>2]|0;
      $68 = ((($67)) + 16|0);
      $69 = HEAP32[$68>>2]|0;
      ;HEAP32[$66>>2]=HEAP32[$69>>2]|0;HEAP32[$66+4>>2]=HEAP32[$69+4>>2]|0;HEAP32[$66+8>>2]=HEAP32[$69+8>>2]|0;HEAP32[$66+12>>2]=HEAP32[$69+12>>2]|0;HEAP32[$66+16>>2]=HEAP32[$69+16>>2]|0;HEAP32[$66+20>>2]=HEAP32[$69+20>>2]|0;HEAP32[$66+24>>2]=HEAP32[$69+24>>2]|0;HEAP32[$66+28>>2]=HEAP32[$69+28>>2]|0;
      break;
     }
    } else {
     $70 = HEAP32[$db>>2]|0;
     $71 = ((($70)) + 16|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = HEAP32[$db>>2]|0;
     $74 = ((($73)) + 20|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + 1)|0;
     $77 = $76<<4;
     $78 = (_sqlite3DbRealloc($62,$72,$77,0)|0);
     HEAP32[$aNew>>2] = $78;
     $79 = HEAP32[$aNew>>2]|0;
     $80 = ($79|0)==(0|0);
     if ($80) {
      STACKTOP = sp;return;
     }
    }
   } while(0);
   $81 = HEAP32[$aNew>>2]|0;
   $82 = HEAP32[$db>>2]|0;
   $83 = ((($82)) + 16|0);
   HEAP32[$83>>2] = $81;
   $84 = HEAP32[$db>>2]|0;
   $85 = ((($84)) + 20|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = HEAP32[$db>>2]|0;
   $88 = ((($87)) + 16|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = (($89) + ($86<<4)|0);
   HEAP32[$aNew>>2] = $90;
   $91 = HEAP32[$aNew>>2]|0;
   ;HEAP32[$91>>2]=0|0;HEAP32[$91+4>>2]=0|0;HEAP32[$91+8>>2]=0|0;HEAP32[$91+12>>2]=0|0;
   $92 = HEAP32[$db>>2]|0;
   $93 = ((($92)) + 48|0);
   $94 = HEAP32[$93>>2]|0;
   HEAP32[$flags>>2] = $94;
   $95 = HEAP32[$db>>2]|0;
   $96 = HEAP32[$95>>2]|0;
   $97 = ((($96)) + 16|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$zFile>>2]|0;
   $100 = (_sqlite3ParseUri($98,$99,$flags,$pVfs,$zPath,$zErr)|0);
   HEAP32[$rc>>2] = $100;
   $101 = HEAP32[$rc>>2]|0;
   $102 = ($101|0)!=(0);
   if ($102) {
    $103 = HEAP32[$rc>>2]|0;
    $104 = ($103|0)==(7);
    if ($104) {
     $105 = HEAP32[$db>>2]|0;
     _sqlite3OomFault($105);
    }
    $106 = HEAP32[$0>>2]|0;
    $107 = HEAP32[$zErr>>2]|0;
    _sqlite3_result_error($106,$107,-1);
    $108 = HEAP32[$zErr>>2]|0;
    _sqlite3_free($108);
    STACKTOP = sp;return;
   }
   $109 = HEAP32[$flags>>2]|0;
   $110 = $109 | 256;
   HEAP32[$flags>>2] = $110;
   $111 = HEAP32[$pVfs>>2]|0;
   $112 = HEAP32[$zPath>>2]|0;
   $113 = HEAP32[$db>>2]|0;
   $114 = HEAP32[$aNew>>2]|0;
   $115 = ((($114)) + 4|0);
   $116 = HEAP32[$flags>>2]|0;
   $117 = (_sqlite3BtreeOpen($111,$112,$113,$115,0,$116)|0);
   HEAP32[$rc>>2] = $117;
   $118 = HEAP32[$zPath>>2]|0;
   _sqlite3_free($118);
   $119 = HEAP32[$db>>2]|0;
   $120 = ((($119)) + 20|0);
   $121 = HEAP32[$120>>2]|0;
   $122 = (($121) + 1)|0;
   HEAP32[$120>>2] = $122;
   $123 = HEAP32[$rc>>2]|0;
   $124 = ($123|0)==(19);
   if ($124) {
    HEAP32[$rc>>2] = 1;
    $125 = HEAP32[$db>>2]|0;
    $126 = (_sqlite3MPrintf($125,31565,$vararg_buffer7)|0);
    HEAP32[$zErrDyn>>2] = $126;
   } else {
    $127 = HEAP32[$rc>>2]|0;
    $128 = ($127|0)==(0);
    if ($128) {
     $129 = HEAP32[$db>>2]|0;
     $130 = HEAP32[$aNew>>2]|0;
     $131 = ((($130)) + 4|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = (_sqlite3SchemaGet($129,$132)|0);
     $134 = HEAP32[$aNew>>2]|0;
     $135 = ((($134)) + 12|0);
     HEAP32[$135>>2] = $133;
     $136 = HEAP32[$aNew>>2]|0;
     $137 = ((($136)) + 12|0);
     $138 = HEAP32[$137>>2]|0;
     $139 = ($138|0)!=(0|0);
     if ($139) {
      $140 = HEAP32[$aNew>>2]|0;
      $141 = ((($140)) + 12|0);
      $142 = HEAP32[$141>>2]|0;
      $143 = ((($142)) + 76|0);
      $144 = HEAP8[$143>>0]|0;
      $145 = $144&255;
      $146 = ($145|0)!=(0);
      if ($146) {
       $147 = HEAP32[$aNew>>2]|0;
       $148 = ((($147)) + 12|0);
       $149 = HEAP32[$148>>2]|0;
       $150 = ((($149)) + 77|0);
       $151 = HEAP8[$150>>0]|0;
       $152 = $151&255;
       $153 = HEAP32[$db>>2]|0;
       $154 = ((($153)) + 66|0);
       $155 = HEAP8[$154>>0]|0;
       $156 = $155&255;
       $157 = ($152|0)!=($156|0);
       if ($157) {
        $158 = HEAP32[$db>>2]|0;
        $159 = (_sqlite3MPrintf($158,20016,$vararg_buffer9)|0);
        HEAP32[$zErrDyn>>2] = $159;
        HEAP32[$rc>>2] = 1;
       }
      }
     } else {
      HEAP32[$rc>>2] = 7;
     }
     $160 = HEAP32[$aNew>>2]|0;
     $161 = ((($160)) + 4|0);
     $162 = HEAP32[$161>>2]|0;
     _sqlite3BtreeEnter($162);
     $163 = HEAP32[$aNew>>2]|0;
     $164 = ((($163)) + 4|0);
     $165 = HEAP32[$164>>2]|0;
     $166 = (_sqlite3BtreePager($165)|0);
     HEAP32[$pPager>>2] = $166;
     $167 = HEAP32[$pPager>>2]|0;
     $168 = HEAP32[$db>>2]|0;
     $169 = ((($168)) + 71|0);
     $170 = HEAP8[$169>>0]|0;
     $171 = $170&255;
     (_sqlite3PagerLockingMode($167,$171)|0);
     $172 = HEAP32[$aNew>>2]|0;
     $173 = ((($172)) + 4|0);
     $174 = HEAP32[$173>>2]|0;
     $175 = HEAP32[$db>>2]|0;
     $176 = ((($175)) + 16|0);
     $177 = HEAP32[$176>>2]|0;
     $178 = ((($177)) + 4|0);
     $179 = HEAP32[$178>>2]|0;
     $180 = (_sqlite3BtreeSecureDelete($179,-1)|0);
     (_sqlite3BtreeSecureDelete($174,$180)|0);
     $181 = HEAP32[$aNew>>2]|0;
     $182 = ((($181)) + 4|0);
     $183 = HEAP32[$182>>2]|0;
     $184 = HEAP32[$db>>2]|0;
     $185 = ((($184)) + 24|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = $186 & 56;
     $188 = 3 | $187;
     (_sqlite3BtreeSetPagerFlags($183,$188)|0);
    }
   }
   $189 = HEAP32[$aNew>>2]|0;
   $190 = ((($189)) + 8|0);
   HEAP8[$190>>0] = 3;
   $191 = HEAP32[$db>>2]|0;
   $192 = HEAP32[$zName>>2]|0;
   $193 = (_sqlite3DbStrDup($191,$192)|0);
   $194 = HEAP32[$aNew>>2]|0;
   HEAP32[$194>>2] = $193;
   $195 = HEAP32[$rc>>2]|0;
   $196 = ($195|0)==(0);
   if ($196) {
    $197 = HEAP32[$aNew>>2]|0;
    $198 = HEAP32[$197>>2]|0;
    $199 = ($198|0)==(0|0);
    if ($199) {
     HEAP32[$rc>>2] = 7;
    }
   }
   $200 = HEAP32[$rc>>2]|0;
   $201 = ($200|0)==(0);
   if ($201) {
    $202 = HEAP32[$db>>2]|0;
    _sqlite3BtreeEnterAll($202);
    $203 = HEAP32[$db>>2]|0;
    $204 = (_sqlite3Init($203,$zErrDyn)|0);
    HEAP32[$rc>>2] = $204;
   }
   $205 = HEAP32[$rc>>2]|0;
   $206 = ($205|0)!=(0);
   if (!($206)) {
    STACKTOP = sp;return;
   }
   $207 = HEAP32[$db>>2]|0;
   $208 = ((($207)) + 20|0);
   $209 = HEAP32[$208>>2]|0;
   $210 = (($209) - 1)|0;
   HEAP32[$iDb>>2] = $210;
   $211 = HEAP32[$iDb>>2]|0;
   $212 = HEAP32[$db>>2]|0;
   $213 = ((($212)) + 16|0);
   $214 = HEAP32[$213>>2]|0;
   $215 = (($214) + ($211<<4)|0);
   $216 = ((($215)) + 4|0);
   $217 = HEAP32[$216>>2]|0;
   $218 = ($217|0)!=(0|0);
   if ($218) {
    $219 = HEAP32[$iDb>>2]|0;
    $220 = HEAP32[$db>>2]|0;
    $221 = ((($220)) + 16|0);
    $222 = HEAP32[$221>>2]|0;
    $223 = (($222) + ($219<<4)|0);
    $224 = ((($223)) + 4|0);
    $225 = HEAP32[$224>>2]|0;
    (_sqlite3BtreeClose($225)|0);
    $226 = HEAP32[$iDb>>2]|0;
    $227 = HEAP32[$db>>2]|0;
    $228 = ((($227)) + 16|0);
    $229 = HEAP32[$228>>2]|0;
    $230 = (($229) + ($226<<4)|0);
    $231 = ((($230)) + 4|0);
    HEAP32[$231>>2] = 0;
    $232 = HEAP32[$iDb>>2]|0;
    $233 = HEAP32[$db>>2]|0;
    $234 = ((($233)) + 16|0);
    $235 = HEAP32[$234>>2]|0;
    $236 = (($235) + ($232<<4)|0);
    $237 = ((($236)) + 12|0);
    HEAP32[$237>>2] = 0;
   }
   $238 = HEAP32[$db>>2]|0;
   _sqlite3ResetAllSchemasOfConnection($238);
   $239 = HEAP32[$iDb>>2]|0;
   $240 = HEAP32[$db>>2]|0;
   $241 = ((($240)) + 20|0);
   HEAP32[$241>>2] = $239;
   $242 = HEAP32[$rc>>2]|0;
   $243 = ($242|0)==(7);
   $244 = HEAP32[$rc>>2]|0;
   $245 = ($244|0)==(3082);
   $or$cond = $243 | $245;
   if ($or$cond) {
    $246 = HEAP32[$db>>2]|0;
    _sqlite3OomFault($246);
    $247 = HEAP32[$db>>2]|0;
    $248 = HEAP32[$zErrDyn>>2]|0;
    _sqlite3DbFree($247,$248);
    $249 = HEAP32[$db>>2]|0;
    $250 = (_sqlite3MPrintf($249,25016,$vararg_buffer11)|0);
    HEAP32[$zErrDyn>>2] = $250;
    break;
   }
   $251 = HEAP32[$zErrDyn>>2]|0;
   $252 = ($251|0)==(0|0);
   if ($252) {
    $253 = HEAP32[$db>>2]|0;
    $254 = HEAP32[$zFile>>2]|0;
    HEAP32[$vararg_buffer13>>2] = $254;
    $255 = (_sqlite3MPrintf($253,31594,$vararg_buffer13)|0);
    HEAP32[$zErrDyn>>2] = $255;
   }
  }
 } while(0);
 $256 = HEAP32[$zErrDyn>>2]|0;
 $257 = ($256|0)!=(0|0);
 if ($257) {
  $258 = HEAP32[$0>>2]|0;
  $259 = HEAP32[$zErrDyn>>2]|0;
  _sqlite3_result_error($258,$259,-1);
  $260 = HEAP32[$db>>2]|0;
  $261 = HEAP32[$zErrDyn>>2]|0;
  _sqlite3DbFree($260,$261);
 }
 $262 = HEAP32[$rc>>2]|0;
 $263 = ($262|0)!=(0);
 if (!($263)) {
  STACKTOP = sp;return;
 }
 $264 = HEAP32[$0>>2]|0;
 $265 = HEAP32[$rc>>2]|0;
 _sqlite3_result_error_code($264,$265);
 STACKTOP = sp;return;
}
function _sqlite3PagerLockingMode($pPager,$eMode) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>=(0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 13|0);
  $6 = HEAP8[$5>>0]|0;
  $7 = ($6<<24>>24)!=(0);
  if (!($7)) {
   $8 = HEAP32[$0>>2]|0;
   $9 = ((($8)) + 216|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3WalHeapMemory($10)|0);
   $12 = ($11|0)!=(0);
   if (!($12)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = $13&255;
    $15 = HEAP32[$0>>2]|0;
    $16 = ((($15)) + 4|0);
    HEAP8[$16>>0] = $14;
   }
  }
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3BtreeSecureDelete($p,$newFlag) {
 $p = $p|0;
 $newFlag = $newFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $b = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $newFlag;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $5 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 22|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12&65535;
  $14 = $13 & -5;
  $15 = $14&65535;
  HEAP16[$11>>1] = $15;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ((($20)) + 22|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22&65535;
   $24 = $23 | 4;
   $25 = $24&65535;
   HEAP16[$21>>1] = $25;
  }
 }
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ((($28)) + 22|0);
 $30 = HEAP16[$29>>1]|0;
 $31 = $30&65535;
 $32 = $31 & 4;
 $33 = ($32|0)!=(0);
 $34 = $33&1;
 HEAP32[$b>>2] = $34;
 $35 = HEAP32[$b>>2]|0;
 HEAP32[$0>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3BtreeSetPagerFlags($p,$pgFlags) {
 $p = $p|0;
 $pgFlags = $pgFlags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pgFlags;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _sqlite3PagerSetFlags($7,$8);
 STACKTOP = sp;return 0;
}
function _sqlite3PagerSetFlags($pPager,$pgFlags) {
 $pPager = $pPager|0;
 $pgFlags = $pgFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $level = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $level = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pgFlags;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 7;
 HEAP32[$level>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 13|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 7|0);
  HEAP8[$9>>0] = 1;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 8|0);
  HEAP8[$11>>0] = 0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 9|0);
  HEAP8[$13>>0] = 0;
 } else {
  $14 = HEAP32[$level>>2]|0;
  $15 = ($14|0)==(1);
  $16 = $15 ? 1 : 0;
  $17 = $16&255;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 7|0);
  HEAP8[$19>>0] = $17;
  $20 = HEAP32[$level>>2]|0;
  $21 = ($20>>>0)>=(3);
  $22 = $21 ? 1 : 0;
  $23 = $22&255;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 8|0);
  HEAP8[$25>>0] = $23;
  $26 = HEAP32[$level>>2]|0;
  $27 = ($26|0)==(4);
  $28 = $27 ? 1 : 0;
  $29 = $28&255;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 9|0);
  HEAP8[$31>>0] = $29;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 7|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = ($34<<24>>24)!=(0);
 do {
  if ($35) {
   $36 = HEAP32[$0>>2]|0;
   $37 = ((($36)) + 12|0);
   HEAP8[$37>>0] = 0;
   $38 = HEAP32[$0>>2]|0;
   $39 = ((($38)) + 10|0);
   HEAP8[$39>>0] = 0;
  } else {
   $40 = HEAP32[$1>>2]|0;
   $41 = $40 & 8;
   $42 = ($41|0)!=(0);
   if ($42) {
    $43 = HEAP32[$0>>2]|0;
    $44 = ((($43)) + 12|0);
    HEAP8[$44>>0] = 3;
    $45 = HEAP32[$0>>2]|0;
    $46 = ((($45)) + 10|0);
    HEAP8[$46>>0] = 3;
    break;
   }
   $47 = HEAP32[$1>>2]|0;
   $48 = $47 & 16;
   $49 = ($48|0)!=(0);
   $50 = HEAP32[$0>>2]|0;
   $51 = ((($50)) + 12|0);
   HEAP8[$51>>0] = 2;
   $52 = HEAP32[$0>>2]|0;
   $53 = ((($52)) + 10|0);
   if ($49) {
    HEAP8[$53>>0] = 3;
    break;
   } else {
    HEAP8[$53>>0] = 2;
    break;
   }
  }
 } while(0);
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 12|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 11|0);
 HEAP8[$58>>0] = $56;
 $59 = HEAP32[$0>>2]|0;
 $60 = ((($59)) + 8|0);
 $61 = HEAP8[$60>>0]|0;
 $62 = ($61<<24>>24)!=(0);
 if ($62) {
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 11|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = $66 | 32;
  $68 = $67&255;
  HEAP8[$64>>0] = $68;
 }
 $69 = HEAP32[$1>>2]|0;
 $70 = $69 & 32;
 $71 = ($70|0)!=(0);
 $72 = HEAP32[$0>>2]|0;
 $73 = ((($72)) + 21|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 if ($71) {
  $76 = $75 & -2;
  $77 = $76&255;
  HEAP8[$73>>0] = $77;
  STACKTOP = sp;return;
 } else {
  $78 = $75 | 1;
  $79 = $78&255;
  HEAP8[$73>>0] = $79;
  STACKTOP = sp;return;
 }
}
function _sqlite3WalHeapMemory($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWal;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $9 = 0;
  $8 = $9&1;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 43|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 $9 = $7;
 $8 = $9&1;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3CodeVerifyNamedSchema($pParse,$zDb) {
 $pParse = $pParse|0;
 $zDb = $zDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $db = sp + 8|0;
 $i = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$db>>2]|0;
  $6 = ((($5)) + 20|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($4|0)<($7|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$db>>2]|0;
  $11 = ((($10)) + 16|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (($12) + ($9<<4)|0);
  HEAP32[$pDb>>2] = $13;
  $14 = HEAP32[$pDb>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  do {
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ($18|0)!=(0|0);
    if ($19) {
     $20 = HEAP32[$1>>2]|0;
     $21 = HEAP32[$pDb>>2]|0;
     $22 = HEAP32[$21>>2]|0;
     $23 = (_sqlite3StrICmp($20,$22)|0);
     $24 = (0)==($23|0);
     if (!($24)) {
      break;
     }
    }
    $25 = HEAP32[$0>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    _sqlite3CodeVerifySchema($25,$26);
   }
  } while(0);
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _sqlite3DropTriggerPtr($pParse,$pTrigger) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $code = 0, $db = 0;
 var $iDb = 0, $pTable = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zDb = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $pTable = sp + 36|0;
 $v = sp + 32|0;
 $db = sp + 28|0;
 $iDb = sp + 24|0;
 $code = sp + 20|0;
 $zDb = sp + 16|0;
 $zTab = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3SchemaToIndex($5,$8)|0);
 HEAP32[$iDb>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = (_tableOfTrigger($10)|0);
 HEAP32[$pTable>>2] = $11;
 HEAP32[$code>>2] = 16;
 $12 = HEAP32[$iDb>>2]|0;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (($15) + ($12<<4)|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$zDb>>2] = $17;
 $18 = HEAP32[$iDb>>2]|0;
 $19 = ($18|0)==(1);
 $20 = $19 ? 19905 : 19924;
 HEAP32[$zTab>>2] = $20;
 $21 = HEAP32[$iDb>>2]|0;
 $22 = ($21|0)==(1);
 if ($22) {
  HEAP32[$code>>2] = 14;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$code>>2]|0;
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$pTable>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$zDb>>2]|0;
 $30 = (_sqlite3AuthCheck($23,$24,$26,$28,$29)|0);
 $31 = ($30|0)!=(0);
 if ($31) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$zTab>>2]|0;
 $34 = HEAP32[$zDb>>2]|0;
 $35 = (_sqlite3AuthCheck($32,9,$33,0,$34)|0);
 $36 = ($35|0)!=(0);
 if ($36) {
  STACKTOP = sp;return;
 }
 $37 = HEAP32[$0>>2]|0;
 $38 = (_sqlite3GetVdbe($37)|0);
 HEAP32[$v>>2] = $38;
 $39 = ($38|0)!=(0|0);
 if (!($39)) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = HEAP32[$iDb>>2]|0;
 $42 = HEAP32[$db>>2]|0;
 $43 = ((($42)) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<4)|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$iDb>>2]|0;
 $48 = ($47|0)==(1);
 $49 = $48 ? 19905 : 19924;
 $50 = HEAP32[$1>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 HEAP32[$vararg_buffer>>2] = $46;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $49;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $51;
 _sqlite3NestedParse($40,31642,$vararg_buffer);
 $52 = HEAP32[$0>>2]|0;
 $53 = HEAP32[$iDb>>2]|0;
 _sqlite3ChangeCookie($52,$53);
 $54 = HEAP32[$v>>2]|0;
 $55 = HEAP32[$iDb>>2]|0;
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 (_sqlite3VdbeAddOp4($54,140,$55,0,0,$57,0)|0);
 STACKTOP = sp;return;
}
function _tableOfTrigger($pTrigger) {
 $pTrigger = $pTrigger|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pTrigger;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3HashFind($4,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _triggerStepAllocate($db,$op,$pName) {
 $db = $db|0;
 $op = $op|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTriggerStep = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 16|0;
 $2 = sp + 8|0;
 $pTriggerStep = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP8[$1>>0] = $op;
 HEAP32[$2>>2] = $pName;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (36 + ($6))|0;
 $8 = (($7) + 1)|0;
 $9 = (_sqlite3DbMallocZero($3,$8,0)|0);
 HEAP32[$pTriggerStep>>2] = $9;
 $10 = HEAP32[$pTriggerStep>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  $26 = HEAP32[$pTriggerStep>>2]|0;
  STACKTOP = sp;return ($26|0);
 }
 $12 = HEAP32[$pTriggerStep>>2]|0;
 $13 = ((($12)) + 36|0);
 HEAP32[$z>>2] = $13;
 $14 = HEAP32[$z>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 _memcpy(($14|0),($16|0),($19|0))|0;
 $20 = HEAP32[$z>>2]|0;
 _sqlite3Dequote($20);
 $21 = HEAP32[$z>>2]|0;
 $22 = HEAP32[$pTriggerStep>>2]|0;
 $23 = ((($22)) + 12|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP8[$1>>0]|0;
 $25 = HEAP32[$pTriggerStep>>2]|0;
 HEAP8[$25>>0] = $24;
 $26 = HEAP32[$pTriggerStep>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3SrcListLookup($pParse,$pSrc) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pItem = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pItem = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 HEAP32[$pItem>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$pItem>>2]|0;
 $6 = (_sqlite3LocateTableItem($4,0,$5)|0);
 HEAP32[$pTab>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$pItem>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3DeleteTable($8,$11);
 $12 = HEAP32[$pTab>>2]|0;
 $13 = HEAP32[$pItem>>2]|0;
 $14 = ((($13)) + 16|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$pTab>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$pTab>>2]|0;
  $18 = ((($17)) + 36|0);
  $19 = HEAP16[$18>>1]|0;
  $20 = (($19) + 1)<<16>>16;
  HEAP16[$18>>1] = $20;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = HEAP32[$pItem>>2]|0;
 $23 = (_sqlite3IndexedByLookup($21,$22)|0);
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $25 = HEAP32[$pTab>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 HEAP32[$pTab>>2] = 0;
 $25 = HEAP32[$pTab>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3FixInit($pFix,$pParse,$iDb,$zType,$pName) {
 $pFix = $pFix|0;
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $zType = $zType|0;
 $pName = $pName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pFix;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $zType;
 HEAP32[$4>>2] = $pName;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$8>>2] = $7;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + ($9<<4)|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17<<4)|0);
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 4|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$3>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = ((($27)) + 16|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$4>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 20|0);
 HEAP32[$31>>2] = $29;
 $32 = HEAP32[$2>>2]|0;
 $33 = ($32|0)==(1);
 $34 = $33&1;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 8|0);
 HEAP32[$36>>2] = $34;
 STACKTOP = sp;return;
}
function _sqlite3FixSrcList($pFix,$pList) {
 $pFix = $pFix|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $i = sp + 20|0;
 $zDb = sp + 16|0;
 $pItem = sp + 12|0;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$zDb>>2] = $7;
 HEAP32[$i>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$pItem>>2] = $9;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = ($10|0)<($12|0);
  if (!($13)) {
   label = 15;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 8|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)==(0);
  if ($17) {
   $18 = HEAP32[$pItem>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$pItem>>2]|0;
    $23 = ((($22)) + 4|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$zDb>>2]|0;
    $26 = (_sqlite3StrICmp($24,$25)|0);
    $27 = ($26|0)!=(0);
    if ($27) {
     label = 8;
     break;
    }
   }
   $39 = HEAP32[$1>>2]|0;
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$pItem>>2]|0;
   $43 = ((($42)) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   _sqlite3DbFree($41,$44);
   $45 = HEAP32[$pItem>>2]|0;
   $46 = ((($45)) + 4|0);
   HEAP32[$46>>2] = 0;
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = HEAP32[$pItem>>2]|0;
   HEAP32[$50>>2] = $49;
  }
  $51 = HEAP32[$1>>2]|0;
  $52 = HEAP32[$pItem>>2]|0;
  $53 = ((($52)) + 20|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (_sqlite3FixSelect($51,$54)|0);
  $56 = ($55|0)!=(0);
  if ($56) {
   label = 11;
   break;
  }
  $57 = HEAP32[$1>>2]|0;
  $58 = HEAP32[$pItem>>2]|0;
  $59 = ((($58)) + 48|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = (_sqlite3FixExpr($57,$60)|0);
  $62 = ($61|0)!=(0);
  if ($62) {
   label = 13;
   break;
  }
  $63 = HEAP32[$i>>2]|0;
  $64 = (($63) + 1)|0;
  HEAP32[$i>>2] = $64;
  $65 = HEAP32[$pItem>>2]|0;
  $66 = ((($65)) + 72|0);
  HEAP32[$pItem>>2] = $66;
 }
 if ((label|0) == 8) {
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 16|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = ((($33)) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$pItem>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$vararg_buffer>>2] = $32;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $35;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $38;
  _sqlite3ErrorMsg($29,31940,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 1;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 else if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $67 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($67|0);
 }
 return (0)|0;
}
function _sqlite3FixSelect($pFix,$pSelect) {
 $pFix = $pFix|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pSelect;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 20;
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3FixExprList($5,$7)|0);
  $9 = ($8|0)!=(0);
  if ($9) {
   label = 4;
   break;
  }
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 28|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3FixSrcList($10,$13)|0);
  $15 = ($14|0)!=(0);
  if ($15) {
   label = 6;
   break;
  }
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ((($17)) + 32|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3FixExpr($16,$19)|0);
  $21 = ($20|0)!=(0);
  if ($21) {
   label = 8;
   break;
  }
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 36|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3FixExprList($22,$25)|0);
  $27 = ($26|0)!=(0);
  if ($27) {
   label = 10;
   break;
  }
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 40|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3FixExpr($28,$31)|0);
  $33 = ($32|0)!=(0);
  if ($33) {
   label = 12;
   break;
  }
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 44|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3FixExprList($34,$37)|0);
  $39 = ($38|0)!=(0);
  if ($39) {
   label = 14;
   break;
  }
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 56|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (_sqlite3FixExpr($40,$43)|0);
  $45 = ($44|0)!=(0);
  if ($45) {
   label = 16;
   break;
  }
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP32[$2>>2]|0;
  $48 = ((($47)) + 60|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_sqlite3FixExpr($46,$49)|0);
  $51 = ($50|0)!=(0);
  if ($51) {
   label = 18;
   break;
  }
  $52 = HEAP32[$2>>2]|0;
  $53 = ((($52)) + 48|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$2>>2] = $54;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 18) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 20) {
  HEAP32[$0>>2] = 0;
 }
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3FixExpr($pFix,$pExpr) {
 $pFix = $pFix|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pExpr;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 16;
   break;
  }
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP8[$5>>0]|0;
  $7 = $6&255;
  $8 = ($7|0)==(135);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$10>>2]|0;
   $12 = ((($11)) + 148|0);
   $13 = ((($12)) + 5|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = ($14<<24>>24)!=(0);
   if (!($15)) {
    label = 6;
    break;
   }
   $16 = HEAP32[$2>>2]|0;
   HEAP8[$16>>0] = 101;
  }
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = $24 & 8404992;
  $26 = ($25|0)!=(0);
  if ($26) {
   label = 16;
   break;
  }
  $27 = HEAP32[$2>>2]|0;
  $28 = ((($27)) + 4|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = $29 & 2048;
  $31 = ($30|0)!=(0);
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = ((($33)) + 20|0);
  if ($31) {
   $35 = HEAP32[$34>>2]|0;
   $36 = (_sqlite3FixSelect($32,$35)|0);
   $37 = ($36|0)!=(0);
   if ($37) {
    label = 10;
    break;
   }
  } else {
   $38 = HEAP32[$34>>2]|0;
   $39 = (_sqlite3FixExprList($32,$38)|0);
   $40 = ($39|0)!=(0);
   if ($40) {
    label = 12;
    break;
   }
  }
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = ((($42)) + 16|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = (_sqlite3FixExpr($41,$44)|0);
  $46 = ($45|0)!=(0);
  if ($46) {
   label = 14;
   break;
  }
  $47 = HEAP32[$2>>2]|0;
  $48 = ((($47)) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  HEAP32[$2>>2] = $49;
 }
 if ((label|0) == 6) {
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$vararg_buffer>>2] = $21;
  _sqlite3ErrorMsg($18,31986,$vararg_buffer);
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $50 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 return (0)|0;
}
function _sqlite3FixExprList($pFix,$pList) {
 $pFix = $pFix|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $pItem = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pItem>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   label = 8;
   break;
  }
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$pItem>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (_sqlite3FixExpr($12,$14)|0);
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 6;
   break;
  }
  $17 = HEAP32[$i>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$i>>2] = $18;
  $19 = HEAP32[$pItem>>2]|0;
  $20 = ((($19)) + 20|0);
  HEAP32[$pItem>>2] = $20;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 return (0)|0;
}
function _sqlite3FixTriggerStep($pFix,$pStep) {
 $pFix = $pFix|0;
 $pStep = $pStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pFix;
 HEAP32[$2>>2] = $pStep;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   label = 10;
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = ((($6)) + 8|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3FixSelect($5,$8)|0);
  $10 = ($9|0)!=(0);
  if ($10) {
   label = 4;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (_sqlite3FixExpr($11,$14)|0);
  $16 = ($15|0)!=(0);
  if ($16) {
   label = 6;
   break;
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3FixExprList($17,$20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   label = 8;
   break;
  }
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 28|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$2>>2] = $25;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = 0;
 }
 $26 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($26|0);
}
function _sqlite3VdbeRunOnlyOnce($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 144|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 & -33;
 $5 = $4 | 32;
 HEAP16[$2>>1] = $5;
 STACKTOP = sp;return;
}
function _returnSingleText($v,$zLabel,$zValue) {
 $v = $v|0;
 $zLabel = $zLabel|0;
 $zValue = $zValue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $zLabel;
 HEAP32[$2>>2] = $zValue;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeLoadString($5,1,$6)|0);
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 _setOneColumnName($7,$8);
 $9 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeAddOp2($9,87,1,1)|0);
 STACKTOP = sp;return;
}
function _setOneColumnName($v,$z) {
 $v = $v|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$0>>2]|0;
 _setAllColumnNames($2,1,$1);
 STACKTOP = sp;return;
}
function _returnSingleInt($v,$zLabel,$0,$1) {
 $v = $v|0;
 $zLabel = $zLabel|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$2>>2] = $v;
 HEAP32[$3>>2] = $zLabel;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp4Dup8($9,77,0,1,0,$4,-13)|0);
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 _setOneColumnName($10,$11);
 $12 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($12,87,1,1)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeSetNumCols($p,$nResColumn) {
 $p = $p|0;
 $nResColumn = $nResColumn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0;
 var $pColName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pColName = sp + 8|0;
 $n = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $nResColumn;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 100|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 140|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = $10<<1;
 _releaseMemArray($6,$11);
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 100|0);
 $15 = HEAP32[$14>>2]|0;
 _sqlite3DbFree($12,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = $16<<1;
 HEAP32[$n>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = $18&65535;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 140|0);
 HEAP16[$21>>1] = $19;
 $22 = HEAP32[$db>>2]|0;
 $23 = HEAP32[$n>>2]|0;
 $24 = ($23*40)|0;
 $25 = (_sqlite3DbMallocRawNN($22,$24,0)|0);
 HEAP32[$pColName>>2] = $25;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 100|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 100|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($30|0)==(0|0);
 if ($31) {
  STACKTOP = sp;return;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 100|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$n>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 _initMemArray($34,$35,$37,1);
 STACKTOP = sp;return;
}
function _sqlite3VdbeSetColName($p,$idx,$var,$zName,$xDel) {
 $p = $p|0;
 $idx = $idx|0;
 $var = $var|0;
 $zName = $zName|0;
 $xDel = $xDel|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColName = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $rc = sp + 4|0;
 $pColName = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $idx;
 HEAP32[$3>>2] = $var;
 HEAP32[$4>>2] = $zName;
 HEAP32[$5>>2] = $xDel;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 69|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 7;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 } else {
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 140|0);
  $15 = HEAP16[$14>>1]|0;
  $16 = $15&65535;
  $17 = Math_imul($12, $16)|0;
  $18 = (($11) + ($17))|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 100|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + (($18*40)|0)|0);
  HEAP32[$pColName>>2] = $22;
  $23 = HEAP32[$pColName>>2]|0;
  $24 = HEAP32[$4>>2]|0;
  $25 = HEAP32[$5>>2]|0;
  $26 = (_sqlite3VdbeMemSetStr($23,$24,-1,1,$25)|0);
  HEAP32[$rc>>2] = $26;
  $27 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $27;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _getLockingMode($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 do {
  if ($3) {
   $4 = HEAP32[$1>>2]|0;
   $5 = (_sqlite3StrICmp($4,32192)|0);
   $6 = (0)==($5|0);
   if ($6) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $7 = HEAP32[$1>>2]|0;
   $8 = (_sqlite3StrICmp($7,32185)|0);
   $9 = (0)==($8|0);
   if ($9) {
    HEAP32[$0>>2] = 0;
   } else {
    label = 6;
   }
  } else {
   label = 6;
  }
 } while(0);
 if ((label|0) == 6) {
  HEAP32[$0>>2] = -1;
 }
 $10 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3JournalModename($eMode) {
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(6);
 if ($3) {
  HEAP32[$0>>2] = 0;
 } else {
  $4 = HEAP32[$1>>2]|0;
  $5 = (5420 + ($4<<2)|0);
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$0>>2] = $6;
 }
 $7 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _sqlite3PagerJournalSizeLimit($pPager,$0,$1) {
 $pPager = $pPager|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pPager;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)>(-1);
 $15 = ($10>>>0)>=(4294967295);
 $16 = ($13|0)==(-1);
 $17 = $16 & $15;
 $18 = $14 | $17;
 if ($18) {
  $19 = $3;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = ((($25)) + 168|0);
  $27 = $26;
  $28 = $27;
  HEAP32[$28>>2] = $21;
  $29 = (($27) + 4)|0;
  $30 = $29;
  HEAP32[$30>>2] = $24;
  $31 = HEAP32[$2>>2]|0;
  $32 = ((($31)) + 216|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = $3;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  _sqlite3WalLimit($33,$36,$39);
 }
 $40 = HEAP32[$2>>2]|0;
 $41 = ((($40)) + 168|0);
 $42 = $41;
 $43 = $42;
 $44 = HEAP32[$43>>2]|0;
 $45 = (($42) + 4)|0;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 tempRet0 = ($47);
 STACKTOP = sp;return ($44|0);
}
function _sqlite3BtreeGetAutoVacuum($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  $16 = 0;
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 18|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = ($12<<24>>24)!=(0);
 $14 = $13 ^ 1;
 $15 = $14 ? 1 : 2;
 $16 = $15;
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _getAutoVacuum($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = (_sqlite3StrICmp($2,33263)|0);
 $4 = (0)==($3|0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 0;
  } else {
   $5 = HEAP32[$1>>2]|0;
   $6 = (_sqlite3StrICmp($5,32707)|0);
   $7 = (0)==($6|0);
   if ($7) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $8 = HEAP32[$1>>2]|0;
   $9 = (_sqlite3StrICmp($8,33268)|0);
   $10 = (0)==($9|0);
   if ($10) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_sqlite3Atoi($11)|0);
    HEAP32[$i>>2] = $12;
    $13 = HEAP32[$i>>2]|0;
    $14 = ($13|0)>=(0);
    $15 = HEAP32[$i>>2]|0;
    $16 = ($15|0)<=(2);
    $or$cond = $14 & $16;
    $17 = HEAP32[$i>>2]|0;
    $18 = $or$cond ? $17 : 0;
    $19 = $18&255;
    $20 = $19&255;
    HEAP32[$0>>2] = $20;
    break;
   }
  }
 } while(0);
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _sqlite3BtreeSetAutoVacuum($p,$autoVacuum) {
 $p = $p|0;
 $autoVacuum = $autoVacuum|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $av = 0, $pBt = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pBt = sp + 4|0;
 $rc = sp;
 $av = sp + 16|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $autoVacuum;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = $5&255;
 HEAP8[$av>>0] = $6;
 $7 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($7);
 $8 = HEAP32[$pBt>>2]|0;
 $9 = ((($8)) + 22|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 2;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP8[$av>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)!=(0);
  $17 = $16 ? 1 : 0;
  $18 = HEAP32[$pBt>>2]|0;
  $19 = ((($18)) + 17|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  $22 = ($17|0)!=($21|0);
  if ($22) {
   HEAP32[$rc>>2] = 8;
   $37 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $23 = HEAP8[$av>>0]|0;
 $24 = $23&255;
 $25 = ($24|0)!=(0);
 $26 = $25 ? 1 : 0;
 $27 = $26&255;
 $28 = HEAP32[$pBt>>2]|0;
 $29 = ((($28)) + 17|0);
 HEAP8[$29>>0] = $27;
 $30 = HEAP8[$av>>0]|0;
 $31 = $30&255;
 $32 = ($31|0)==(2);
 $33 = $32 ? 1 : 0;
 $34 = $33&255;
 $35 = HEAP32[$pBt>>2]|0;
 $36 = ((($35)) + 18|0);
 HEAP8[$36>>0] = $34;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3BtreeSetSpillSize($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pBt = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3PagerSetSpillsize($7,$8)|0);
 HEAP32[$res>>2] = $9;
 $10 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _setAllPagerFlags($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pDb = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 67|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pDb>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 20|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$n>>2] = $10;
 while(1) {
  $11 = HEAP32[$n>>2]|0;
  $12 = (($11) + -1)|0;
  HEAP32[$n>>2] = $12;
  $13 = ($11|0)>(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$pDb>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$pDb>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$pDb>>2]|0;
   $22 = ((($21)) + 8|0);
   $23 = HEAP8[$22>>0]|0;
   $24 = $23&255;
   $25 = HEAP32[$0>>2]|0;
   $26 = ((($25)) + 24|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = $27 & 56;
   $29 = $24 | $28;
   (_sqlite3BtreeSetPagerFlags($20,$29)|0);
  }
  $30 = HEAP32[$pDb>>2]|0;
  $31 = ((($30)) + 16|0);
  HEAP32[$pDb>>2] = $31;
 }
 STACKTOP = sp;return;
}
function _changeTempStorage($pParse,$zStorageType) {
 $pParse = $pParse|0;
 $zStorageType = $zStorageType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $db = 0, $ts = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $ts = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $zStorageType;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_getTempStore($3)|0);
 HEAP32[$ts>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$ts>>2]|0;
 $12 = ($10|0)==($11|0);
 if ($12) {
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 $13 = HEAP32[$1>>2]|0;
 $14 = (_invalidateTempStorage($13)|0);
 $15 = ($14|0)!=(0);
 if ($15) {
  HEAP32[$0>>2] = 1;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 } else {
  $16 = HEAP32[$ts>>2]|0;
  $17 = $16&255;
  $18 = HEAP32[$db>>2]|0;
  $19 = ((($18)) + 68|0);
  HEAP8[$19>>0] = $17;
  HEAP32[$0>>2] = 0;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _invalidateTempStorage($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 do {
  if ($10) {
   $11 = HEAP32[$db>>2]|0;
   $12 = ((($11)) + 67|0);
   $13 = HEAP8[$12>>0]|0;
   $14 = ($13<<24>>24)!=(0);
   if ($14) {
    $15 = HEAP32[$db>>2]|0;
    $16 = ((($15)) + 16|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ((($17)) + 16|0);
    $19 = ((($18)) + 4|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (_sqlite3BtreeIsInReadTrans($20)|0);
    $22 = ($21|0)!=(0);
    if (!($22)) {
     $24 = HEAP32[$db>>2]|0;
     $25 = ((($24)) + 16|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ((($26)) + 16|0);
     $28 = ((($27)) + 4|0);
     $29 = HEAP32[$28>>2]|0;
     (_sqlite3BtreeClose($29)|0);
     $30 = HEAP32[$db>>2]|0;
     $31 = ((($30)) + 16|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = ((($32)) + 16|0);
     $34 = ((($33)) + 4|0);
     HEAP32[$34>>2] = 0;
     $35 = HEAP32[$db>>2]|0;
     _sqlite3ResetAllSchemasOfConnection($35);
     break;
    }
   }
   $23 = HEAP32[$1>>2]|0;
   _sqlite3ErrorMsg($23,33201,$vararg_buffer);
   HEAP32[$0>>2] = 1;
   $36 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($36|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _setAllColumnNames($v,$N,$azCol) {
 $v = $v|0;
 $N = $N|0;
 $azCol = $azCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $azCol;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3VdbeSetNumCols($3,$4);
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$1>>2]|0;
  $7 = ($5|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + ($10<<2)|0);
  $13 = HEAP32[$12>>2]|0;
  (_sqlite3VdbeSetColName($8,$9,0,$13,0)|0);
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeMultiLoad($p,$iDest,$zTypes,$varargs) {
 $p = $p|0;
 $iDest = $iDest|0;
 $zTypes = $zTypes|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $arglist_current = 0;
 var $arglist_current2 = 0, $arglist_next = 0, $arglist_next3 = 0, $c = 0, $expanded = 0, $expanded10 = 0, $expanded12 = 0, $expanded13 = 0, $expanded14 = 0, $expanded3 = 0, $expanded5 = 0, $expanded6 = 0, $expanded7 = 0, $expanded9 = 0, $i = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $ap = sp + 8|0;
 $i = sp + 4|0;
 $c = sp + 36|0;
 $z = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDest;
 HEAP32[$2>>2] = $zTypes;
 HEAP32[$ap>>2] = $varargs;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = (($4) + ($3)|0);
  $6 = HEAP8[$5>>0]|0;
  HEAP8[$c>>0] = $6;
  $7 = $6 << 24 >> 24;
  $8 = ($7|0)!=(0);
  if (!($8)) {
   break;
  }
  $9 = HEAP8[$c>>0]|0;
  $10 = $9 << 24 >> 24;
  $11 = ($10|0)==(115);
  if ($11) {
   $arglist_current = HEAP32[$ap>>2]|0;
   $12 = $arglist_current;
   $13 = ((0) + 4|0);
   $expanded3 = $13;
   $expanded = (($expanded3) - 1)|0;
   $14 = (($12) + ($expanded))|0;
   $15 = ((0) + 4|0);
   $expanded7 = $15;
   $expanded6 = (($expanded7) - 1)|0;
   $expanded5 = $expanded6 ^ -1;
   $16 = $14 & $expanded5;
   $17 = $16;
   $18 = HEAP32[$17>>2]|0;
   $arglist_next = ((($17)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next;
   HEAP32[$z>>2] = $18;
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$z>>2]|0;
   $21 = ($20|0)==(0|0);
   $22 = $21 ? 79 : 97;
   $23 = HEAP32[$1>>2]|0;
   $24 = (($23) + 1)|0;
   HEAP32[$1>>2] = $24;
   $25 = HEAP32[$z>>2]|0;
   (_sqlite3VdbeAddOp4($19,$22,0,$23,0,$25,0)|0);
  } else {
   $26 = HEAP32[$0>>2]|0;
   $arglist_current2 = HEAP32[$ap>>2]|0;
   $27 = $arglist_current2;
   $28 = ((0) + 4|0);
   $expanded10 = $28;
   $expanded9 = (($expanded10) - 1)|0;
   $29 = (($27) + ($expanded9))|0;
   $30 = ((0) + 4|0);
   $expanded14 = $30;
   $expanded13 = (($expanded14) - 1)|0;
   $expanded12 = $expanded13 ^ -1;
   $31 = $29 & $expanded12;
   $32 = $31;
   $33 = HEAP32[$32>>2]|0;
   $arglist_next3 = ((($32)) + 4|0);
   HEAP32[$ap>>2] = $arglist_next3;
   $34 = HEAP32[$1>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$1>>2] = $35;
   (_sqlite3VdbeAddOp2($26,76,$33,$34)|0);
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _actionName($action) {
 $action = $action|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $zName = sp;
 HEAP8[$0>>0] = $action;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 switch ($2|0) {
 case 7:  {
  HEAP32[$zName>>2] = 33011;
  break;
 }
 case 8:  {
  HEAP32[$zName>>2] = 33020;
  break;
 }
 case 9:  {
  HEAP32[$zName>>2] = 33032;
  break;
 }
 case 6:  {
  HEAP32[$zName>>2] = 33040;
  break;
 }
 default: {
  HEAP32[$zName>>2] = 33049;
 }
 }
 $3 = HEAP32[$zName>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3FkLocateIndex($pParse,$pParent,$pFKey,$ppIdx,$paiCol) {
 $pParse = $pParse|0;
 $pParent = $pParent|0;
 $pFKey = $pFKey|0;
 $ppIdx = $ppIdx|0;
 $paiCol = $paiCol|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiCol = 0, $i = 0, $i1 = 0;
 var $iCol = 0, $j = 0, $nCol = 0, $pIdx = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zDfltColl = 0, $zIdxCol = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $pIdx = sp + 40|0;
 $aiCol = sp + 36|0;
 $nCol = sp + 32|0;
 $zKey = sp + 28|0;
 $i = sp + 24|0;
 $i1 = sp + 20|0;
 $j = sp + 16|0;
 $iCol = sp + 68|0;
 $zDfltColl = sp + 12|0;
 $zIdxCol = sp + 8|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pParent;
 HEAP32[$3>>2] = $pFKey;
 HEAP32[$4>>2] = $ppIdx;
 HEAP32[$5>>2] = $paiCol;
 HEAP32[$pIdx>>2] = 0;
 HEAP32[$aiCol>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nCol>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 36|0);
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$zKey>>2] = $12;
 $13 = HEAP32[$nCol>>2]|0;
 $14 = ($13|0)==(1);
 do {
  if ($14) {
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 32|0);
   $17 = HEAP16[$16>>1]|0;
   $18 = $17 << 16 >> 16;
   $19 = ($18|0)>=(0);
   if ($19) {
    $20 = HEAP32[$zKey>>2]|0;
    $21 = ($20|0)!=(0|0);
    if (!($21)) {
     HEAP32[$0>>2] = 0;
     $175 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($175|0);
    }
    $22 = HEAP32[$2>>2]|0;
    $23 = ((($22)) + 32|0);
    $24 = HEAP16[$23>>1]|0;
    $25 = $24 << 16 >> 16;
    $26 = HEAP32[$2>>2]|0;
    $27 = ((($26)) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = (($28) + ($25<<4)|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$zKey>>2]|0;
    $32 = (_sqlite3StrICmp($30,$31)|0);
    $33 = ($32|0)!=(0);
    if (!($33)) {
     HEAP32[$0>>2] = 0;
     $175 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($175|0);
    }
   }
  } else {
   $34 = HEAP32[$5>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    $36 = HEAP32[$1>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$nCol>>2]|0;
    $39 = $38<<2;
    $40 = (_sqlite3DbMallocRawNN($37,$39,0)|0);
    HEAP32[$aiCol>>2] = $40;
    $41 = HEAP32[$aiCol>>2]|0;
    $42 = ($41|0)!=(0|0);
    if ($42) {
     $43 = HEAP32[$aiCol>>2]|0;
     $44 = HEAP32[$5>>2]|0;
     HEAP32[$44>>2] = $43;
     break;
    }
    HEAP32[$0>>2] = 1;
    $175 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($175|0);
   }
  }
 } while(0);
 $45 = HEAP32[$2>>2]|0;
 $46 = ((($45)) + 8|0);
 $47 = HEAP32[$46>>2]|0;
 HEAP32[$pIdx>>2] = $47;
 L17: while(1) {
  $48 = HEAP32[$pIdx>>2]|0;
  $49 = ($48|0)!=(0|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[$pIdx>>2]|0;
  $51 = ((($50)) + 50|0);
  $52 = HEAP16[$51>>1]|0;
  $53 = $52&65535;
  $54 = HEAP32[$nCol>>2]|0;
  $55 = ($53|0)==($54|0);
  do {
   if ($55) {
    $56 = HEAP32[$pIdx>>2]|0;
    $57 = ((($56)) + 54|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58&255;
    $60 = ($59|0)!=(0);
    if ($60) {
     $61 = HEAP32[$zKey>>2]|0;
     $62 = ($61|0)==(0|0);
     if ($62) {
      $63 = HEAP32[$pIdx>>2]|0;
      $64 = ((($63)) + 55|0);
      $65 = HEAP8[$64>>0]|0;
      $66 = $65 & 3;
      $67 = $66&255;
      $68 = ($67|0)==(2);
      if ($68) {
       label = 17;
       break L17;
      } else {
       break;
      }
     }
     HEAP32[$i1>>2] = 0;
     while(1) {
      $84 = HEAP32[$i1>>2]|0;
      $85 = HEAP32[$nCol>>2]|0;
      $86 = ($84|0)<($85|0);
      if (!($86)) {
       break;
      }
      $87 = HEAP32[$i1>>2]|0;
      $88 = HEAP32[$pIdx>>2]|0;
      $89 = ((($88)) + 4|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (($90) + ($87<<1)|0);
      $92 = HEAP16[$91>>1]|0;
      HEAP16[$iCol>>1] = $92;
      $93 = HEAP16[$iCol>>1]|0;
      $94 = $93 << 16 >> 16;
      $95 = ($94|0)<(0);
      if ($95) {
       break;
      }
      $96 = HEAP16[$iCol>>1]|0;
      $97 = $96 << 16 >> 16;
      $98 = HEAP32[$2>>2]|0;
      $99 = ((($98)) + 4|0);
      $100 = HEAP32[$99>>2]|0;
      $101 = (($100) + ($97<<4)|0);
      $102 = ((($101)) + 8|0);
      $103 = HEAP32[$102>>2]|0;
      HEAP32[$zDfltColl>>2] = $103;
      $104 = HEAP32[$zDfltColl>>2]|0;
      $105 = ($104|0)!=(0|0);
      $$ = $105 ? $103 : 34834;
      HEAP32[$zDfltColl>>2] = $$;
      $106 = HEAP32[$i1>>2]|0;
      $107 = HEAP32[$pIdx>>2]|0;
      $108 = ((($107)) + 32|0);
      $109 = HEAP32[$108>>2]|0;
      $110 = (($109) + ($106<<2)|0);
      $111 = HEAP32[$110>>2]|0;
      $112 = HEAP32[$zDfltColl>>2]|0;
      $113 = (_sqlite3StrICmp($111,$112)|0);
      $114 = ($113|0)!=(0);
      if ($114) {
       break;
      }
      $115 = HEAP16[$iCol>>1]|0;
      $116 = $115 << 16 >> 16;
      $117 = HEAP32[$2>>2]|0;
      $118 = ((($117)) + 4|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = (($119) + ($116<<4)|0);
      $121 = HEAP32[$120>>2]|0;
      HEAP32[$zIdxCol>>2] = $121;
      HEAP32[$j>>2] = 0;
      while(1) {
       $122 = HEAP32[$j>>2]|0;
       $123 = HEAP32[$nCol>>2]|0;
       $124 = ($122|0)<($123|0);
       if (!($124)) {
        break;
       }
       $125 = HEAP32[$j>>2]|0;
       $126 = HEAP32[$3>>2]|0;
       $127 = ((($126)) + 36|0);
       $128 = (($127) + ($125<<3)|0);
       $129 = ((($128)) + 4|0);
       $130 = HEAP32[$129>>2]|0;
       $131 = HEAP32[$zIdxCol>>2]|0;
       $132 = (_sqlite3StrICmp($130,$131)|0);
       $133 = ($132|0)==(0);
       if ($133) {
        label = 28;
        break;
       }
       $144 = HEAP32[$j>>2]|0;
       $145 = (($144) + 1)|0;
       HEAP32[$j>>2] = $145;
      }
      if ((label|0) == 28) {
       label = 0;
       $134 = HEAP32[$aiCol>>2]|0;
       $135 = ($134|0)!=(0|0);
       if ($135) {
        $136 = HEAP32[$j>>2]|0;
        $137 = HEAP32[$3>>2]|0;
        $138 = ((($137)) + 36|0);
        $139 = (($138) + ($136<<3)|0);
        $140 = HEAP32[$139>>2]|0;
        $141 = HEAP32[$i1>>2]|0;
        $142 = HEAP32[$aiCol>>2]|0;
        $143 = (($142) + ($141<<2)|0);
        HEAP32[$143>>2] = $140;
       }
      }
      $146 = HEAP32[$j>>2]|0;
      $147 = HEAP32[$nCol>>2]|0;
      $148 = ($146|0)==($147|0);
      if ($148) {
       break;
      }
      $149 = HEAP32[$i1>>2]|0;
      $150 = (($149) + 1)|0;
      HEAP32[$i1>>2] = $150;
     }
     $151 = HEAP32[$i1>>2]|0;
     $152 = HEAP32[$nCol>>2]|0;
     $153 = ($151|0)==($152|0);
     if ($153) {
      break L17;
     }
    }
   }
  } while(0);
  $154 = HEAP32[$pIdx>>2]|0;
  $155 = ((($154)) + 20|0);
  $156 = HEAP32[$155>>2]|0;
  HEAP32[$pIdx>>2] = $156;
 }
 L42: do {
  if ((label|0) == 17) {
   $69 = HEAP32[$aiCol>>2]|0;
   $70 = ($69|0)!=(0|0);
   if ($70) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $71 = HEAP32[$i>>2]|0;
     $72 = HEAP32[$nCol>>2]|0;
     $73 = ($71|0)<($72|0);
     if (!($73)) {
      break L42;
     }
     $74 = HEAP32[$i>>2]|0;
     $75 = HEAP32[$3>>2]|0;
     $76 = ((($75)) + 36|0);
     $77 = (($76) + ($74<<3)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = HEAP32[$i>>2]|0;
     $80 = HEAP32[$aiCol>>2]|0;
     $81 = (($80) + ($79<<2)|0);
     HEAP32[$81>>2] = $78;
     $82 = HEAP32[$i>>2]|0;
     $83 = (($82) + 1)|0;
     HEAP32[$i>>2] = $83;
    }
   }
  }
 } while(0);
 $157 = HEAP32[$pIdx>>2]|0;
 $158 = ($157|0)!=(0|0);
 if ($158) {
  $173 = HEAP32[$pIdx>>2]|0;
  $174 = HEAP32[$4>>2]|0;
  HEAP32[$174>>2] = $173;
  HEAP32[$0>>2] = 0;
  $175 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($175|0);
 }
 $159 = HEAP32[$1>>2]|0;
 $160 = ((($159)) + 150|0);
 $161 = HEAP8[$160>>0]|0;
 $162 = ($161<<24>>24)!=(0);
 if (!($162)) {
  $163 = HEAP32[$1>>2]|0;
  $164 = HEAP32[$3>>2]|0;
  $165 = HEAP32[$164>>2]|0;
  $166 = HEAP32[$165>>2]|0;
  $167 = HEAP32[$3>>2]|0;
  $168 = ((($167)) + 8|0);
  $169 = HEAP32[$168>>2]|0;
  HEAP32[$vararg_buffer>>2] = $166;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $169;
  _sqlite3ErrorMsg($163,32948,$vararg_buffer);
 }
 $170 = HEAP32[$1>>2]|0;
 $171 = HEAP32[$170>>2]|0;
 $172 = HEAP32[$aiCol>>2]|0;
 _sqlite3DbFree($171,$172);
 HEAP32[$0>>2] = 1;
 $175 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($175|0);
}
function _sqlite3IndexAffinityStr($db,$pIdx) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $9 = 0, $aff = 0, $n = 0, $pTab = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $n = sp + 4|0;
 $pTab = sp;
 $x = sp + 20|0;
 $aff = sp + 22|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$2>>2]|0;
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pTab>>2] = $9;
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 52|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12&65535;
  $14 = (($13) + 1)|0;
  $15 = ($14|0)<(0);
  $16 = $15 << 31 >> 31;
  $17 = (_sqlite3DbMallocRaw(0,$14,$16)|0);
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 16|0);
  HEAP32[$19>>2] = $17;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  if (!($23)) {
   $24 = HEAP32[$1>>2]|0;
   _sqlite3OomFault($24);
   HEAP32[$0>>2] = 0;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  }
  HEAP32[$n>>2] = 0;
  while(1) {
   $25 = HEAP32[$n>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = ((($26)) + 52|0);
   $28 = HEAP16[$27>>1]|0;
   $29 = $28&65535;
   $30 = ($25|0)<($29|0);
   $31 = HEAP32[$n>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   if (!($30)) {
    break;
   }
   $33 = ((($32)) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<1)|0);
   $36 = HEAP16[$35>>1]|0;
   HEAP16[$x>>1] = $36;
   $37 = HEAP16[$x>>1]|0;
   $38 = $37 << 16 >> 16;
   $39 = ($38|0)>=(0);
   $40 = HEAP16[$x>>1]|0;
   $41 = $40 << 16 >> 16;
   do {
    if ($39) {
     $42 = HEAP32[$pTab>>2]|0;
     $43 = ((($42)) + 4|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = (($44) + ($41<<4)|0);
     $46 = ((($45)) + 13|0);
     $47 = HEAP8[$46>>0]|0;
     $48 = HEAP32[$n>>2]|0;
     $49 = HEAP32[$2>>2]|0;
     $50 = ((($49)) + 16|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = (($51) + ($48)|0);
     HEAP8[$52>>0] = $47;
    } else {
     $53 = ($41|0)==(-1);
     $54 = HEAP32[$n>>2]|0;
     $55 = HEAP32[$2>>2]|0;
     if ($53) {
      $56 = ((($55)) + 16|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = (($57) + ($54)|0);
      HEAP8[$58>>0] = 68;
      break;
     } else {
      $59 = ((($55)) + 40|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ((($60)) + 4|0);
      $62 = HEAP32[$61>>2]|0;
      $63 = (($62) + (($54*20)|0)|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (_sqlite3ExprAffinity($64)|0);
      HEAP8[$aff>>0] = $65;
      $66 = HEAP8[$aff>>0]|0;
      $67 = $66 << 24 >> 24;
      $68 = ($67|0)==(0);
      $$ = $68 ? 65 : $65;
      HEAP8[$aff>>0] = $$;
      $69 = HEAP8[$aff>>0]|0;
      $70 = HEAP32[$n>>2]|0;
      $71 = HEAP32[$2>>2]|0;
      $72 = ((($71)) + 16|0);
      $73 = HEAP32[$72>>2]|0;
      $74 = (($73) + ($70)|0);
      HEAP8[$74>>0] = $69;
      break;
     }
    }
   } while(0);
   $75 = HEAP32[$n>>2]|0;
   $76 = (($75) + 1)|0;
   HEAP32[$n>>2] = $76;
  }
  $77 = ((($32)) + 16|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = (($78) + ($31)|0);
  HEAP8[$79>>0] = 0;
 }
 $80 = HEAP32[$2>>2]|0;
 $81 = ((($80)) + 16|0);
 $82 = HEAP32[$81>>2]|0;
 HEAP32[$0>>2] = $82;
 $83 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($83|0);
}
function _sqlite3RegisterLikeFunctions($db,$caseSensitive) {
 $db = $db|0;
 $caseSensitive = $caseSensitive|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pInfo = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $caseSensitive;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  HEAP32[$pInfo>>2] = 32856;
 } else {
  HEAP32[$pInfo>>2] = 18398;
 }
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$pInfo>>2]|0;
 (_sqlite3CreateFunc($4,32860,2,1,$5,212,0,0,0)|0);
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$pInfo>>2]|0;
 (_sqlite3CreateFunc($6,32860,3,1,$7,212,0,0,0)|0);
 $8 = HEAP32[$0>>2]|0;
 (_sqlite3CreateFunc($8,32865,2,1,18394,212,0,0,0)|0);
 $9 = HEAP32[$0>>2]|0;
 _setLikeOptFlag($9,32865,12);
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ($11|0)!=(0);
 $13 = $12 ? 12 : 4;
 $14 = $13&255;
 _setLikeOptFlag($10,32860,$14);
 STACKTOP = sp;return;
}
function _sqlite3ExprCacheClear($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 25|0);
  $4 = HEAP8[$3>>0]|0;
  $5 = $4&255;
  $6 = ($1|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 152|0);
  $10 = (($9) + (($7*20)|0)|0);
  $11 = ((($10)) + 6|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = ($13|0)!=(0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = ((($15)) + 19|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17&255;
   $19 = ($18|0)<(8);
   if ($19) {
    $20 = HEAP32[$i>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ((($21)) + 152|0);
    $23 = (($22) + (($20*20)|0)|0);
    $24 = ((($23)) + 12|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = HEAP32[$0>>2]|0;
    $27 = ((($26)) + 19|0);
    $28 = HEAP8[$27>>0]|0;
    $29 = (($28) + 1)<<24>>24;
    HEAP8[$27>>0] = $29;
    $30 = $28&255;
    $31 = HEAP32[$0>>2]|0;
    $32 = ((($31)) + 352|0);
    $33 = (($32) + ($30<<2)|0);
    HEAP32[$33>>2] = $25;
   }
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = (($34) + 1)|0;
  HEAP32[$i>>2] = $35;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 25|0);
 HEAP8[$37>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3OpenTableAndIndices($pParse,$pTab,$op,$p5,$iBase,$aToOpen,$piDataCur,$piIdxCur) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $op = $op|0;
 $p5 = $p5|0;
 $iBase = $iBase|0;
 $aToOpen = $aToOpen|0;
 $piDataCur = $piDataCur|0;
 $piIdxCur = $piIdxCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $i = 0, $iDataCur = 0, $iDb = 0, $iIdxCur = 0, $pIdx = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 56|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $8 = sp + 24|0;
 $i = sp + 20|0;
 $iDb = sp + 16|0;
 $iDataCur = sp + 12|0;
 $pIdx = sp + 8|0;
 $v = sp + 4|0;
 $iIdxCur = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $op;
 HEAP8[$4>>0] = $p5;
 HEAP32[$5>>2] = $iBase;
 HEAP32[$6>>2] = $aToOpen;
 HEAP32[$7>>2] = $piDataCur;
 HEAP32[$8>>2] = $piIdxCur;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 16;
 $14 = ($13|0)!=(0);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 64|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3SchemaToIndex($16,$19)|0);
 HEAP32[$iDb>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = (_sqlite3GetVdbe($21)|0);
 HEAP32[$v>>2] = $22;
 $23 = HEAP32[$5>>2]|0;
 $24 = ($23|0)<(0);
 if ($24) {
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 40|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$5>>2] = $27;
 }
 $28 = HEAP32[$5>>2]|0;
 $29 = (($28) + 1)|0;
 HEAP32[$5>>2] = $29;
 HEAP32[$iDataCur>>2] = $28;
 $30 = HEAP32[$7>>2]|0;
 $31 = ($30|0)!=(0|0);
 if ($31) {
  $32 = HEAP32[$iDataCur>>2]|0;
  $33 = HEAP32[$7>>2]|0;
  HEAP32[$33>>2] = $32;
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 42|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $37 & 32;
 $39 = ($38|0)==(0);
 do {
  if ($39) {
   $40 = HEAP32[$6>>2]|0;
   $41 = ($40|0)==(0|0);
   if (!($41)) {
    $42 = HEAP32[$6>>2]|0;
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    $45 = ($44|0)!=(0);
    if (!($45)) {
     label = 11;
     break;
    }
   }
   $46 = HEAP32[$1>>2]|0;
   $47 = HEAP32[$iDataCur>>2]|0;
   $48 = HEAP32[$iDb>>2]|0;
   $49 = HEAP32[$2>>2]|0;
   $50 = HEAP32[$3>>2]|0;
   _sqlite3OpenTable($46,$47,$48,$49,$50);
  } else {
   label = 11;
  }
 } while(0);
 if ((label|0) == 11) {
  $51 = HEAP32[$1>>2]|0;
  $52 = HEAP32[$iDb>>2]|0;
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 28|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$3>>2]|0;
  $57 = ($56|0)==(105);
  $58 = $57&1;
  $59 = $58&255;
  $60 = HEAP32[$2>>2]|0;
  $61 = HEAP32[$60>>2]|0;
  _sqlite3TableLock($51,$52,$55,$59,$61);
 }
 $62 = HEAP32[$8>>2]|0;
 $63 = ($62|0)!=(0|0);
 if ($63) {
  $64 = HEAP32[$5>>2]|0;
  $65 = HEAP32[$8>>2]|0;
  HEAP32[$65>>2] = $64;
 }
 HEAP32[$i>>2] = 0;
 $66 = HEAP32[$2>>2]|0;
 $67 = ((($66)) + 8|0);
 $68 = HEAP32[$67>>2]|0;
 HEAP32[$pIdx>>2] = $68;
 while(1) {
  $69 = HEAP32[$pIdx>>2]|0;
  $70 = ($69|0)!=(0|0);
  $71 = HEAP32[$5>>2]|0;
  if (!($70)) {
   break;
  }
  $72 = (($71) + 1)|0;
  HEAP32[$5>>2] = $72;
  HEAP32[$iIdxCur>>2] = $71;
  $73 = HEAP32[$pIdx>>2]|0;
  $74 = ((($73)) + 55|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75 & 3;
  $77 = $76&255;
  $78 = ($77|0)==(2);
  if ($78) {
   $79 = HEAP32[$2>>2]|0;
   $80 = ((($79)) + 42|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = $81&255;
   $83 = $82 & 32;
   $84 = ($83|0)==(0);
   if (!($84)) {
    $85 = HEAP32[$7>>2]|0;
    $86 = ($85|0)!=(0|0);
    if ($86) {
     $87 = HEAP32[$iIdxCur>>2]|0;
     $88 = HEAP32[$7>>2]|0;
     HEAP32[$88>>2] = $87;
    }
    HEAP8[$4>>0] = 0;
   }
  }
  $89 = HEAP32[$6>>2]|0;
  $90 = ($89|0)==(0|0);
  if ($90) {
   label = 23;
  } else {
   $91 = HEAP32[$i>>2]|0;
   $92 = (($91) + 1)|0;
   $93 = HEAP32[$6>>2]|0;
   $94 = (($93) + ($92)|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = $95&255;
   $97 = ($96|0)!=(0);
   if ($97) {
    label = 23;
   }
  }
  if ((label|0) == 23) {
   label = 0;
   $98 = HEAP32[$v>>2]|0;
   $99 = HEAP32[$3>>2]|0;
   $100 = HEAP32[$iIdxCur>>2]|0;
   $101 = HEAP32[$pIdx>>2]|0;
   $102 = ((($101)) + 44|0);
   $103 = HEAP32[$102>>2]|0;
   $104 = HEAP32[$iDb>>2]|0;
   (_sqlite3VdbeAddOp3($98,$99,$100,$103,$104)|0);
   $105 = HEAP32[$1>>2]|0;
   $106 = HEAP32[$pIdx>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($105,$106);
   $107 = HEAP32[$v>>2]|0;
   $108 = HEAP8[$4>>0]|0;
   _sqlite3VdbeChangeP5($107,$108);
  }
  $109 = HEAP32[$pIdx>>2]|0;
  $110 = ((($109)) + 20|0);
  $111 = HEAP32[$110>>2]|0;
  HEAP32[$pIdx>>2] = $111;
  $112 = HEAP32[$i>>2]|0;
  $113 = (($112) + 1)|0;
  HEAP32[$i>>2] = $113;
 }
 $114 = HEAP32[$1>>2]|0;
 $115 = ((($114)) + 40|0);
 $116 = HEAP32[$115>>2]|0;
 $117 = ($71|0)>($116|0);
 if ($117) {
  $118 = HEAP32[$5>>2]|0;
  $119 = HEAP32[$1>>2]|0;
  $120 = ((($119)) + 40|0);
  HEAP32[$120>>2] = $118;
 }
 $121 = HEAP32[$i>>2]|0;
 HEAP32[$0>>2] = $121;
 $122 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($122|0);
}
function _sqlite3VdbeReusable($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 144|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 & -33;
 HEAP16[$2>>1] = $4;
 STACKTOP = sp;return;
}
function _likeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $escape = 0, $nPat = 0, $or$cond = 0, $pInfo = 0, $zA = 0, $zB = 0, $zEsc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $zA = sp + 24|0;
 $zB = sp + 20|0;
 $escape = sp + 16|0;
 $nPat = sp + 12|0;
 $db = sp + 8|0;
 $pInfo = sp + 4|0;
 $zEsc = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3_user_data($5)|0);
 HEAP32[$pInfo>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zB>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3_value_text($12)|0);
 HEAP32[$zA>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_bytes($15)|0);
 HEAP32[$nPat>>2] = $16;
 $17 = HEAP32[$nPat>>2]|0;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 96|0);
 $20 = ((($19)) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($17|0)>($21|0);
 if ($22) {
  $23 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($23,32870,-1);
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ($24|0)==(3);
 do {
  if ($25) {
   $26 = HEAP32[$2>>2]|0;
   $27 = ((($26)) + 8|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = (_sqlite3_value_text($28)|0);
   HEAP32[$zEsc>>2] = $29;
   $30 = HEAP32[$zEsc>>2]|0;
   $31 = ($30|0)==(0|0);
   if ($31) {
    STACKTOP = sp;return;
   }
   $32 = HEAP32[$zEsc>>2]|0;
   $33 = (_sqlite3Utf8CharLen81($32,-1)|0);
   $34 = ($33|0)!=(1);
   if (!($34)) {
    $36 = (_sqlite3Utf8Read($zEsc)|0);
    HEAP32[$escape>>2] = $36;
    break;
   }
   $35 = HEAP32[$0>>2]|0;
   _sqlite3_result_error($35,32903,-1);
   STACKTOP = sp;return;
  } else {
   $37 = HEAP32[$pInfo>>2]|0;
   $38 = ((($37)) + 2|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   HEAP32[$escape>>2] = $40;
  }
 } while(0);
 $41 = HEAP32[$zA>>2]|0;
 $42 = ($41|0)!=(0|0);
 $43 = HEAP32[$zB>>2]|0;
 $44 = ($43|0)!=(0|0);
 $or$cond = $42 & $44;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$zB>>2]|0;
 $47 = HEAP32[$zA>>2]|0;
 $48 = HEAP32[$pInfo>>2]|0;
 $49 = HEAP32[$escape>>2]|0;
 $50 = (_patternCompare($46,$47,$48,$49)|0);
 _sqlite3_result_int($45,$50);
 STACKTOP = sp;return;
}
function _setLikeOptFlag($db,$zName,$flagVal) {
 $db = $db|0;
 $zName = $zName|0;
 $flagVal = $flagVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $pDef = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zName;
 HEAP8[$2>>0] = $flagVal;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3FindFunction($3,$4,2,1,0)|0);
 HEAP32[$pDef>>2] = $5;
 $6 = HEAP32[$pDef>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP8[$2>>0]|0;
 $9 = $8&255;
 $10 = HEAP32[$pDef>>2]|0;
 $11 = ((($10)) + 2|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 | $9;
 $15 = $14&65535;
 HEAP16[$11>>1] = $15;
 STACKTOP = sp;return;
}
function _sqlite3Utf8Read($pz) {
 $pz = $pz|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $c = sp;
 HEAP32[$0>>2] = $pz;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 1|0);
 HEAP32[$1>>2] = $3;
 $4 = HEAP8[$2>>0]|0;
 $5 = $4&255;
 HEAP32[$c>>2] = $5;
 $6 = HEAP32[$c>>2]|0;
 $7 = ($6>>>0)>=(192);
 if (!($7)) {
  $35 = HEAP32[$c>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$c>>2]|0;
 $9 = (($8) - 192)|0;
 $10 = (25563 + ($9)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 HEAP32[$c>>2] = $12;
 while(1) {
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = $16 & 192;
  $18 = ($17|0)==(128);
  $19 = HEAP32[$c>>2]|0;
  if (!($18)) {
   break;
  }
  $20 = $19 << 6;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = ((($22)) + 1|0);
  HEAP32[$21>>2] = $23;
  $24 = HEAP8[$22>>0]|0;
  $25 = $24&255;
  $26 = 63 & $25;
  $27 = (($20) + ($26))|0;
  HEAP32[$c>>2] = $27;
 }
 $28 = ($19>>>0)<(128);
 if (!($28)) {
  $29 = HEAP32[$c>>2]|0;
  $30 = $29 & -2048;
  $31 = ($30|0)==(55296);
  if (!($31)) {
   $32 = HEAP32[$c>>2]|0;
   $33 = $32 & -2;
   $34 = ($33|0)==(65534);
   if (!($34)) {
    $35 = HEAP32[$c>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
  }
 }
 HEAP32[$c>>2] = 65533;
 $35 = HEAP32[$c>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _getTempStore($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)>=(48);
 if ($5) {
  $6 = HEAP32[$1>>2]|0;
  $7 = HEAP8[$6>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)<=(50);
  if ($9) {
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP8[$10>>0]|0;
   $12 = $11 << 24 >> 24;
   $13 = (($12) - 48)|0;
   HEAP32[$0>>2] = $13;
  } else {
   label = 4;
  }
 } else {
  label = 4;
 }
 do {
  if ((label|0) == 4) {
   $14 = HEAP32[$1>>2]|0;
   $15 = (_sqlite3StrICmp($14,33105)|0);
   $16 = ($15|0)==(0);
   if ($16) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $17 = HEAP32[$1>>2]|0;
   $18 = (_sqlite3StrICmp($17,26159)|0);
   $19 = ($18|0)==(0);
   if ($19) {
    HEAP32[$0>>2] = 2;
    break;
   } else {
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $20 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($20|0);
}
function _sqlite3PagerSetSpillsize($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 212|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3PcacheSetSpillsize($4,$5)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3PcacheSetSpillsize($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = ($4|0)<(0);
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ($6|0)<(0);
   $8 = $7 << 31 >> 31;
   $9 = (___muldi3(-1024,-1,($6|0),($8|0))|0);
   $10 = tempRet0;
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 24|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 28|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (($13) + ($16))|0;
   $18 = ($17|0)<(0);
   $19 = $18 << 31 >> 31;
   $20 = (___divdi3(($9|0),($10|0),($17|0),($19|0))|0);
   $21 = tempRet0;
   HEAP32[$1>>2] = $20;
  }
  $22 = HEAP32[$1>>2]|0;
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 20|0);
  HEAP32[$24>>2] = $22;
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = (_numberOfCachePages($25)|0);
 HEAP32[$res>>2] = $26;
 $27 = HEAP32[$res>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 20|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ($27|0)<($30|0);
 if (!($31)) {
  $35 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 20|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$res>>2] = $34;
 $35 = HEAP32[$res>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _sqlite3WalLimit($pWal,$0,$1) {
 $pWal = $pWal|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pWal;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = $17;
 $19 = $18;
 HEAP32[$19>>2] = $12;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $15;
 STACKTOP = sp;return;
}
function _releaseMemArray($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $or$cond = 0, $pEnd = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pEnd = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0);
 $or$cond = $3 & $5;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + (($6*40)|0)|0);
 HEAP32[$pEnd>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 456|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)!=(0|0);
 if ($15) {
  while(1) {
   $16 = HEAP32[$0>>2]|0;
   $17 = ((($16)) + 24|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)!=(0);
   if ($19) {
    $20 = HEAP32[$db>>2]|0;
    $21 = HEAP32[$0>>2]|0;
    $22 = ((($21)) + 20|0);
    $23 = HEAP32[$22>>2]|0;
    _sqlite3DbFree($20,$23);
   }
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 40|0);
   HEAP32[$0>>2] = $25;
   $26 = HEAP32[$pEnd>>2]|0;
   $27 = ($25>>>0)<($26>>>0);
   if (!($27)) {
    break;
   }
  }
  STACKTOP = sp;return;
 }
 while(1) {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 8|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 & 9312;
  $33 = ($32|0)!=(0);
  $34 = HEAP32[$0>>2]|0;
  if ($33) {
   _sqlite3VdbeMemRelease($34);
  } else {
   $35 = ((($34)) + 24|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)!=(0);
   if ($37) {
    $38 = HEAP32[$db>>2]|0;
    $39 = HEAP32[$0>>2]|0;
    $40 = ((($39)) + 20|0);
    $41 = HEAP32[$40>>2]|0;
    _sqlite3DbFree($38,$41);
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 24|0);
    HEAP32[$43>>2] = 0;
   }
  }
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 8|0);
  HEAP16[$45>>1] = 128;
  $46 = HEAP32[$0>>2]|0;
  $47 = ((($46)) + 40|0);
  HEAP32[$0>>2] = $47;
  $48 = HEAP32[$pEnd>>2]|0;
  $49 = ($47>>>0)<($48>>>0);
  if (!($49)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _initMemArray($p,$N,$db,$flags) {
 $p = $p|0;
 $N = $N|0;
 $db = $db|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 $3 = sp + 12|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 HEAP32[$2>>2] = $db;
 HEAP16[$3>>1] = $flags;
 while(1) {
  $4 = HEAP32[$1>>2]|0;
  $5 = (($4) + -1)|0;
  HEAP32[$1>>2] = $5;
  $6 = ($4|0)>(0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 32|0);
  HEAP32[$9>>2] = $7;
  $10 = HEAP16[$3>>1]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  HEAP16[$12>>1] = $10;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 24|0);
  HEAP32[$14>>2] = 0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 40|0);
  HEAP32[$0>>2] = $16;
 }
 STACKTOP = sp;return;
}
function _sqlite3ClearStatTables($pParse,$iDb,$zType,$zName) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $zType = $zType|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, $zDbName = 0, $zTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $i = sp + 28|0;
 $zDbName = sp + 24|0;
 $zTab = sp + 48|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zType;
 HEAP32[$3>>2] = $zName;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($4<<4)|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$zDbName>>2] = $10;
 HEAP32[$i>>2] = 1;
 while(1) {
  $11 = HEAP32[$i>>2]|0;
  $12 = ($11|0)<=(4);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  (_sqlite3_snprintf(24,$zTab,34141,$vararg_buffer)|0);
  $14 = HEAP32[$0>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$zDbName>>2]|0;
  $17 = (_sqlite3FindTable($15,$zTab,$16)|0);
  $18 = ($17|0)!=(0|0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = HEAP32[$zDbName>>2]|0;
   $21 = HEAP32[$2>>2]|0;
   $22 = HEAP32[$3>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $20;
   $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
   HEAP32[$vararg_ptr4>>2] = $zTab;
   $vararg_ptr5 = ((($vararg_buffer1)) + 8|0);
   HEAP32[$vararg_ptr5>>2] = $21;
   $vararg_ptr6 = ((($vararg_buffer1)) + 12|0);
   HEAP32[$vararg_ptr6>>2] = $22;
   _sqlite3NestedParse($19,30938,$vararg_buffer1);
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 STACKTOP = sp;return;
}
function _destroyRootPage($pParse,$iTable,$iDb) {
 $pParse = $pParse|0;
 $iTable = $iTable|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $v = sp + 24|0;
 $r1 = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $iDb;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetVdbe($3)|0);
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3GetTempReg($5)|0);
 HEAP32[$r1>>2] = $6;
 $7 = HEAP32[$v>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$r1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($7,130,$8,$9,$10)|0);
 $11 = HEAP32[$0>>2]|0;
 _sqlite3MayAbort($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($13<<4)|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ($20|0)==(1);
 $22 = $21 ? 19905 : 19924;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$r1>>2]|0;
 $25 = HEAP32[$r1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $19;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $22;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $23;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $24;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $25;
 _sqlite3NestedParse($12,34085,$vararg_buffer);
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($26,$27);
 STACKTOP = sp;return;
}
function _sqlite3ArrayAllocate($db,$pArray,$szEntry,$pnEntry,$pIdx) {
 $db = $db|0;
 $pArray = $pArray|0;
 $szEntry = $szEntry|0;
 $pnEntry = $pnEntry|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $pNew = 0, $sz = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $z = sp + 12|0;
 $n = sp + 8|0;
 $sz = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pArray;
 HEAP32[$3>>2] = $szEntry;
 HEAP32[$4>>2] = $pnEntry;
 HEAP32[$5>>2] = $pIdx;
 $6 = HEAP32[$4>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$n>>2] = $7;
 $8 = HEAP32[$n>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = (($9) - 1)|0;
 $11 = $8 & $10;
 $12 = ($11|0)==(0);
 do {
  if ($12) {
   $13 = HEAP32[$n>>2]|0;
   $14 = ($13|0)==(0);
   $15 = HEAP32[$n>>2]|0;
   $16 = $15<<1;
   $17 = $14 ? 1 : $16;
   HEAP32[$sz>>2] = $17;
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = HEAP32[$sz>>2]|0;
   $21 = HEAP32[$3>>2]|0;
   $22 = Math_imul($20, $21)|0;
   $23 = ($22|0)<(0);
   $24 = $23 << 31 >> 31;
   $25 = (_sqlite3DbRealloc($18,$19,$22,$24)|0);
   HEAP32[$pNew>>2] = $25;
   $26 = HEAP32[$pNew>>2]|0;
   $27 = ($26|0)==(0|0);
   if (!($27)) {
    $30 = HEAP32[$pNew>>2]|0;
    HEAP32[$2>>2] = $30;
    break;
   }
   $28 = HEAP32[$5>>2]|0;
   HEAP32[$28>>2] = -1;
   $29 = HEAP32[$2>>2]|0;
   HEAP32[$0>>2] = $29;
   $44 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($44|0);
  }
 } while(0);
 $31 = HEAP32[$2>>2]|0;
 HEAP32[$z>>2] = $31;
 $32 = HEAP32[$n>>2]|0;
 $33 = HEAP32[$3>>2]|0;
 $34 = Math_imul($32, $33)|0;
 $35 = HEAP32[$z>>2]|0;
 $36 = (($35) + ($34)|0);
 $37 = HEAP32[$3>>2]|0;
 _memset(($36|0),0,($37|0))|0;
 $38 = HEAP32[$n>>2]|0;
 $39 = HEAP32[$5>>2]|0;
 HEAP32[$39>>2] = $38;
 $40 = HEAP32[$4>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 1)|0;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $43;
 $44 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3TriggersExist($pParse,$pTab,$op,$pChanges,$pMask) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $op = $op|0;
 $pChanges = $pChanges|0;
 $pMask = $pMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mask = 0, $p = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $mask = sp + 8|0;
 $pList = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $pMask;
 HEAP32[$mask>>2] = 0;
 HEAP32[$pList>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 16777216;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = (_sqlite3TriggerList($11,$12)|0);
  HEAP32[$pList>>2] = $13;
 }
 $14 = HEAP32[$pList>>2]|0;
 HEAP32[$p>>2] = $14;
 while(1) {
  $15 = HEAP32[$p>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = HEAP32[$2>>2]|0;
  $22 = ($20|0)==($21|0);
  if ($22) {
   $23 = HEAP32[$p>>2]|0;
   $24 = ((($23)) + 16|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$3>>2]|0;
   $27 = (_checkColumnOverlap($25,$26)|0);
   $28 = ($27|0)!=(0);
   if ($28) {
    $29 = HEAP32[$p>>2]|0;
    $30 = ((($29)) + 9|0);
    $31 = HEAP8[$30>>0]|0;
    $32 = $31&255;
    $33 = HEAP32[$mask>>2]|0;
    $34 = $33 | $32;
    HEAP32[$mask>>2] = $34;
   }
  }
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 32|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$p>>2] = $37;
 }
 $38 = HEAP32[$4>>2]|0;
 $39 = ($38|0)!=(0|0);
 if (!($39)) {
  $42 = HEAP32[$mask>>2]|0;
  $43 = ($42|0)!=(0);
  $44 = HEAP32[$pList>>2]|0;
  $45 = $43 ? $44 : 0;
  STACKTOP = sp;return ($45|0);
 }
 $40 = HEAP32[$mask>>2]|0;
 $41 = HEAP32[$4>>2]|0;
 HEAP32[$41>>2] = $40;
 $42 = HEAP32[$mask>>2]|0;
 $43 = ($42|0)!=(0);
 $44 = HEAP32[$pList>>2]|0;
 $45 = $43 ? $44 : 0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3IsReadOnly($pParse,$pTab,$viewOk) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $viewOk = $viewOk|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $viewOk;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 42|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & 16;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = (_sqlite3GetVTable($11,$12)|0);
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 52|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)==(0|0);
  if (!($19)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 do {
  if ((label|0) == 3) {
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 42|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $23 & 1;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = HEAP32[$1>>2]|0;
    $27 = HEAP32[$26>>2]|0;
    $28 = ((($27)) + 24|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = $29 & 2048;
    $31 = ($30|0)==(0);
    if ($31) {
     $32 = HEAP32[$1>>2]|0;
     $33 = ((($32)) + 18|0);
     $34 = HEAP8[$33>>0]|0;
     $35 = $34&255;
     $36 = ($35|0)==(0);
     if ($36) {
      break;
     }
    }
   }
   $40 = HEAP32[$3>>2]|0;
   $41 = ($40|0)!=(0);
   if (!($41)) {
    $42 = HEAP32[$2>>2]|0;
    $43 = ((($42)) + 12|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = ($44|0)!=(0|0);
    if ($45) {
     $46 = HEAP32[$1>>2]|0;
     $47 = HEAP32[$2>>2]|0;
     $48 = HEAP32[$47>>2]|0;
     HEAP32[$vararg_buffer1>>2] = $48;
     _sqlite3ErrorMsg($46,35257,$vararg_buffer1);
     HEAP32[$0>>2] = 1;
     $49 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($49|0);
    }
   }
   HEAP32[$0>>2] = 0;
   $49 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 } while(0);
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 HEAP32[$vararg_buffer>>2] = $39;
 _sqlite3ErrorMsg($37,35228,$vararg_buffer);
 HEAP32[$0>>2] = 1;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3VdbeCountChanges($v) {
 $v = $v|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $v;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 144|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 & -17;
 $5 = $4 | 16;
 HEAP16[$2>>1] = $5;
 STACKTOP = sp;return;
}
function _xferOptimization($pParse,$pDest,$pSelect,$onError,$iDbDest) {
 $pParse = $pParse|0;
 $pDest = $pDest|0;
 $pSelect = $pSelect|0;
 $onError = $onError|0;
 $iDbDest = $iDbDest|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0;
 var $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0;
 var $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0;
 var $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0;
 var $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0;
 var $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0;
 var $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0;
 var $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0;
 var $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0;
 var $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0;
 var $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0;
 var $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0;
 var $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0;
 var $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0, $addr2 = 0, $db = 0, $destHasUniqueIdx = 0, $emptyDestTest = 0, $emptySrcTest = 0, $i = 0, $iDbSrc = 0, $iDest = 0, $iSrc = 0, $idxInsFlags = 0, $or$cond = 0, $or$cond4 = 0, $pDestCol = 0, $pDestIdx = 0, $pEList = 0;
 var $pItem = 0, $pSrc = 0, $pSrcCol = 0, $pSrcIdx = 0, $regAutoinc = 0, $regData = 0, $regRowid = 0, $v = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $4 = sp + 92|0;
 $5 = sp + 88|0;
 $db = sp + 84|0;
 $pEList = sp + 80|0;
 $pSrc = sp + 76|0;
 $pSrcIdx = sp + 72|0;
 $pDestIdx = sp + 68|0;
 $pItem = sp + 64|0;
 $i = sp + 60|0;
 $iDbSrc = sp + 56|0;
 $iSrc = sp + 52|0;
 $iDest = sp + 48|0;
 $addr1 = sp + 44|0;
 $addr2 = sp + 40|0;
 $emptyDestTest = sp + 36|0;
 $emptySrcTest = sp + 32|0;
 $v = sp + 28|0;
 $regAutoinc = sp + 24|0;
 $destHasUniqueIdx = sp + 20|0;
 $regData = sp + 16|0;
 $regRowid = sp + 12|0;
 $pDestCol = sp + 8|0;
 $pSrcCol = sp + 4|0;
 $idxInsFlags = sp + 112|0;
 $zColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$3>>2] = $pSelect;
 HEAP32[$4>>2] = $onError;
 HEAP32[$5>>2] = $iDbDest;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$emptyDestTest>>2] = 0;
 HEAP32[$emptySrcTest>>2] = 0;
 HEAP32[$destHasUniqueIdx>>2] = 0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $521 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($521|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 472|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  $14 = HEAP32[$3>>2]|0;
  $15 = ((($14)) + 64|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (_sqlite3TriggerList($18,$19)|0);
   $21 = ($20|0)!=(0|0);
   if ($21) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $22 = HEAP32[$2>>2]|0;
   $23 = ((($22)) + 42|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = $25 & 16;
   $27 = ($26|0)!=(0);
   if ($27) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $28 = HEAP32[$4>>2]|0;
   $29 = ($28|0)==(10);
   if ($29) {
    $30 = HEAP32[$2>>2]|0;
    $31 = ((($30)) + 32|0);
    $32 = HEAP16[$31>>1]|0;
    $33 = $32 << 16 >> 16;
    $34 = ($33|0)>=(0);
    if ($34) {
     $35 = HEAP32[$2>>2]|0;
     $36 = ((($35)) + 43|0);
     $37 = HEAP8[$36>>0]|0;
     $38 = $37&255;
     HEAP32[$4>>2] = $38;
    }
    $39 = HEAP32[$4>>2]|0;
    $40 = ($39|0)==(10);
    if ($40) {
     HEAP32[$4>>2] = 2;
    }
   }
   $41 = HEAP32[$3>>2]|0;
   $42 = ((($41)) + 28|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(1);
   if ($45) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $46 = HEAP32[$3>>2]|0;
   $47 = ((($46)) + 28|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = ((($48)) + 8|0);
   $50 = ((($49)) + 20|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = ($51|0)!=(0|0);
   if ($52) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $53 = HEAP32[$3>>2]|0;
   $54 = ((($53)) + 32|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = ($55|0)!=(0|0);
   if ($56) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $57 = HEAP32[$3>>2]|0;
   $58 = ((($57)) + 44|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)!=(0|0);
   if ($60) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $61 = HEAP32[$3>>2]|0;
   $62 = ((($61)) + 36|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($63|0)!=(0|0);
   if ($64) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $65 = HEAP32[$3>>2]|0;
   $66 = ((($65)) + 56|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = ($67|0)!=(0|0);
   if ($68) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $69 = HEAP32[$3>>2]|0;
   $70 = ((($69)) + 48|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)!=(0|0);
   if ($72) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $73 = HEAP32[$3>>2]|0;
   $74 = ((($73)) + 8|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = $75 & 1;
   $77 = ($76|0)!=(0);
   if ($77) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $78 = HEAP32[$3>>2]|0;
   $79 = HEAP32[$78>>2]|0;
   HEAP32[$pEList>>2] = $79;
   $80 = HEAP32[$pEList>>2]|0;
   $81 = HEAP32[$80>>2]|0;
   $82 = ($81|0)!=(1);
   if ($82) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $83 = HEAP32[$pEList>>2]|0;
   $84 = ((($83)) + 4|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = HEAP32[$85>>2]|0;
   $87 = HEAP8[$86>>0]|0;
   $88 = $87&255;
   $89 = ($88|0)!=(160);
   if ($89) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $90 = HEAP32[$3>>2]|0;
   $91 = ((($90)) + 28|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = ((($92)) + 8|0);
   HEAP32[$pItem>>2] = $93;
   $94 = HEAP32[$1>>2]|0;
   $95 = HEAP32[$pItem>>2]|0;
   $96 = (_sqlite3LocateTableItem($94,0,$95)|0);
   HEAP32[$pSrc>>2] = $96;
   $97 = HEAP32[$pSrc>>2]|0;
   $98 = ($97|0)==(0|0);
   if ($98) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $99 = HEAP32[$pSrc>>2]|0;
   $100 = HEAP32[$2>>2]|0;
   $101 = ($99|0)==($100|0);
   if ($101) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $102 = HEAP32[$2>>2]|0;
   $103 = ((($102)) + 42|0);
   $104 = HEAP8[$103>>0]|0;
   $105 = $104&255;
   $106 = $105 & 32;
   $107 = ($106|0)==(0);
   $108 = $107&1;
   $109 = HEAP32[$pSrc>>2]|0;
   $110 = ((($109)) + 42|0);
   $111 = HEAP8[$110>>0]|0;
   $112 = $111&255;
   $113 = $112 & 32;
   $114 = ($113|0)==(0);
   $115 = $114&1;
   $116 = ($108|0)!=($115|0);
   if ($116) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $117 = HEAP32[$pSrc>>2]|0;
   $118 = ((($117)) + 42|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = $119&255;
   $121 = $120 & 16;
   $122 = ($121|0)!=(0);
   if ($122) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $123 = HEAP32[$pSrc>>2]|0;
   $124 = ((($123)) + 12|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = ($125|0)!=(0|0);
   if ($126) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $127 = HEAP32[$2>>2]|0;
   $128 = ((($127)) + 34|0);
   $129 = HEAP16[$128>>1]|0;
   $130 = $129 << 16 >> 16;
   $131 = HEAP32[$pSrc>>2]|0;
   $132 = ((($131)) + 34|0);
   $133 = HEAP16[$132>>1]|0;
   $134 = $133 << 16 >> 16;
   $135 = ($130|0)!=($134|0);
   if ($135) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   $136 = HEAP32[$2>>2]|0;
   $137 = ((($136)) + 32|0);
   $138 = HEAP16[$137>>1]|0;
   $139 = $138 << 16 >> 16;
   $140 = HEAP32[$pSrc>>2]|0;
   $141 = ((($140)) + 32|0);
   $142 = HEAP16[$141>>1]|0;
   $143 = $142 << 16 >> 16;
   $144 = ($139|0)!=($143|0);
   if ($144) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   HEAP32[$i>>2] = 0;
   L91: while(1) {
    $145 = HEAP32[$i>>2]|0;
    $146 = HEAP32[$2>>2]|0;
    $147 = ((($146)) + 34|0);
    $148 = HEAP16[$147>>1]|0;
    $149 = $148 << 16 >> 16;
    $150 = ($145|0)<($149|0);
    if (!($150)) {
     label = 64;
     break;
    }
    $151 = HEAP32[$i>>2]|0;
    $152 = HEAP32[$2>>2]|0;
    $153 = ((($152)) + 4|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = (($154) + ($151<<4)|0);
    HEAP32[$pDestCol>>2] = $155;
    $156 = HEAP32[$i>>2]|0;
    $157 = HEAP32[$pSrc>>2]|0;
    $158 = ((($157)) + 4|0);
    $159 = HEAP32[$158>>2]|0;
    $160 = (($159) + ($156<<4)|0);
    HEAP32[$pSrcCol>>2] = $160;
    $161 = HEAP32[$pDestCol>>2]|0;
    $162 = ((($161)) + 13|0);
    $163 = HEAP8[$162>>0]|0;
    $164 = $163 << 24 >> 24;
    $165 = HEAP32[$pSrcCol>>2]|0;
    $166 = ((($165)) + 13|0);
    $167 = HEAP8[$166>>0]|0;
    $168 = $167 << 24 >> 24;
    $169 = ($164|0)!=($168|0);
    if ($169) {
     label = 52;
     break;
    }
    $170 = HEAP32[$pDestCol>>2]|0;
    $171 = ((($170)) + 8|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = HEAP32[$pSrcCol>>2]|0;
    $174 = ((($173)) + 8|0);
    $175 = HEAP32[$174>>2]|0;
    $176 = (_sqlite3_stricmp($172,$175)|0);
    $177 = ($176|0)!=(0);
    if ($177) {
     label = 54;
     break;
    }
    $178 = HEAP32[$pDestCol>>2]|0;
    $179 = ((($178)) + 12|0);
    $180 = HEAP8[$179>>0]|0;
    $181 = $180&255;
    $182 = ($181|0)!=(0);
    if ($182) {
     $183 = HEAP32[$pSrcCol>>2]|0;
     $184 = ((($183)) + 12|0);
     $185 = HEAP8[$184>>0]|0;
     $186 = ($185<<24>>24)!=(0);
     if (!($186)) {
      label = 57;
      break;
     }
    }
    $187 = HEAP32[$i>>2]|0;
    $188 = ($187|0)>(0);
    do {
     if ($188) {
      $189 = HEAP32[$pDestCol>>2]|0;
      $190 = ((($189)) + 4|0);
      $191 = HEAP32[$190>>2]|0;
      $192 = ($191|0)==(0|0);
      $193 = $192&1;
      $194 = HEAP32[$pSrcCol>>2]|0;
      $195 = ((($194)) + 4|0);
      $196 = HEAP32[$195>>2]|0;
      $197 = ($196|0)==(0|0);
      $198 = $197&1;
      $199 = ($193|0)!=($198|0);
      if ($199) {
       label = 62;
       break L91;
      }
      $200 = HEAP32[$pDestCol>>2]|0;
      $201 = ((($200)) + 4|0);
      $202 = HEAP32[$201>>2]|0;
      $203 = ($202|0)!=(0|0);
      if (!($203)) {
       break;
      }
      $204 = HEAP32[$pDestCol>>2]|0;
      $205 = ((($204)) + 4|0);
      $206 = HEAP32[$205>>2]|0;
      $207 = ((($206)) + 8|0);
      $208 = HEAP32[$207>>2]|0;
      $209 = HEAP32[$pSrcCol>>2]|0;
      $210 = ((($209)) + 4|0);
      $211 = HEAP32[$210>>2]|0;
      $212 = ((($211)) + 8|0);
      $213 = HEAP32[$212>>2]|0;
      $214 = (_strcmp($208,$213)|0);
      $215 = ($214|0)!=(0);
      if ($215) {
       label = 62;
       break L91;
      }
     }
    } while(0);
    $216 = HEAP32[$i>>2]|0;
    $217 = (($216) + 1)|0;
    HEAP32[$i>>2] = $217;
   }
   if ((label|0) == 52) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   else if ((label|0) == 54) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   else if ((label|0) == 57) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   else if ((label|0) == 62) {
    HEAP32[$0>>2] = 0;
    $521 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($521|0);
   }
   else if ((label|0) == 64) {
    $218 = HEAP32[$2>>2]|0;
    $219 = ((($218)) + 8|0);
    $220 = HEAP32[$219>>2]|0;
    HEAP32[$pDestIdx>>2] = $220;
    while(1) {
     $221 = HEAP32[$pDestIdx>>2]|0;
     $222 = ($221|0)!=(0|0);
     if (!($222)) {
      break;
     }
     $223 = HEAP32[$pDestIdx>>2]|0;
     $224 = ((($223)) + 54|0);
     $225 = HEAP8[$224>>0]|0;
     $226 = $225&255;
     $227 = ($226|0)!=(0);
     if ($227) {
      HEAP32[$destHasUniqueIdx>>2] = 1;
     }
     $228 = HEAP32[$pSrc>>2]|0;
     $229 = ((($228)) + 8|0);
     $230 = HEAP32[$229>>2]|0;
     HEAP32[$pSrcIdx>>2] = $230;
     while(1) {
      $231 = HEAP32[$pSrcIdx>>2]|0;
      $232 = ($231|0)!=(0|0);
      if (!($232)) {
       break;
      }
      $233 = HEAP32[$pDestIdx>>2]|0;
      $234 = HEAP32[$pSrcIdx>>2]|0;
      $235 = (_xferCompatibleIndex($233,$234)|0);
      $236 = ($235|0)!=(0);
      if ($236) {
       break;
      }
      $237 = HEAP32[$pSrcIdx>>2]|0;
      $238 = ((($237)) + 20|0);
      $239 = HEAP32[$238>>2]|0;
      HEAP32[$pSrcIdx>>2] = $239;
     }
     $240 = HEAP32[$pSrcIdx>>2]|0;
     $241 = ($240|0)==(0|0);
     if ($241) {
      label = 73;
      break;
     }
     $242 = HEAP32[$pDestIdx>>2]|0;
     $243 = ((($242)) + 20|0);
     $244 = HEAP32[$243>>2]|0;
     HEAP32[$pDestIdx>>2] = $244;
    }
    if ((label|0) == 73) {
     HEAP32[$0>>2] = 0;
     $521 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($521|0);
    }
    $245 = HEAP32[$2>>2]|0;
    $246 = ((($245)) + 24|0);
    $247 = HEAP32[$246>>2]|0;
    $248 = ($247|0)!=(0|0);
    do {
     if ($248) {
      $249 = HEAP32[$pSrc>>2]|0;
      $250 = ((($249)) + 24|0);
      $251 = HEAP32[$250>>2]|0;
      $252 = HEAP32[$2>>2]|0;
      $253 = ((($252)) + 24|0);
      $254 = HEAP32[$253>>2]|0;
      $255 = (_sqlite3ExprListCompare($251,$254,-1)|0);
      $256 = ($255|0)!=(0);
      if (!($256)) {
       break;
      }
      HEAP32[$0>>2] = 0;
      $521 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($521|0);
     }
    } while(0);
    $257 = HEAP32[$db>>2]|0;
    $258 = ((($257)) + 24|0);
    $259 = HEAP32[$258>>2]|0;
    $260 = $259 & 524288;
    $261 = ($260|0)!=(0);
    do {
     if ($261) {
      $262 = HEAP32[$2>>2]|0;
      $263 = ((($262)) + 16|0);
      $264 = HEAP32[$263>>2]|0;
      $265 = ($264|0)!=(0|0);
      if (!($265)) {
       break;
      }
      HEAP32[$0>>2] = 0;
      $521 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($521|0);
     }
    } while(0);
    $266 = HEAP32[$db>>2]|0;
    $267 = ((($266)) + 24|0);
    $268 = HEAP32[$267>>2]|0;
    $269 = $268 & 128;
    $270 = ($269|0)!=(0);
    if ($270) {
     HEAP32[$0>>2] = 0;
     $521 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($521|0);
    }
    $271 = HEAP32[$db>>2]|0;
    $272 = HEAP32[$pSrc>>2]|0;
    $273 = ((($272)) + 64|0);
    $274 = HEAP32[$273>>2]|0;
    $275 = (_sqlite3SchemaToIndex($271,$274)|0);
    HEAP32[$iDbSrc>>2] = $275;
    $276 = HEAP32[$1>>2]|0;
    $277 = (_sqlite3GetVdbe($276)|0);
    HEAP32[$v>>2] = $277;
    $278 = HEAP32[$1>>2]|0;
    $279 = HEAP32[$iDbSrc>>2]|0;
    _sqlite3CodeVerifySchema($278,$279);
    $280 = HEAP32[$1>>2]|0;
    $281 = ((($280)) + 40|0);
    $282 = HEAP32[$281>>2]|0;
    $283 = (($282) + 1)|0;
    HEAP32[$281>>2] = $283;
    HEAP32[$iSrc>>2] = $282;
    $284 = HEAP32[$1>>2]|0;
    $285 = ((($284)) + 40|0);
    $286 = HEAP32[$285>>2]|0;
    $287 = (($286) + 1)|0;
    HEAP32[$285>>2] = $287;
    HEAP32[$iDest>>2] = $286;
    $288 = HEAP32[$1>>2]|0;
    $289 = HEAP32[$5>>2]|0;
    $290 = HEAP32[$2>>2]|0;
    $291 = (_autoIncBegin($288,$289,$290)|0);
    HEAP32[$regAutoinc>>2] = $291;
    $292 = HEAP32[$1>>2]|0;
    $293 = (_sqlite3GetTempReg($292)|0);
    HEAP32[$regData>>2] = $293;
    $294 = HEAP32[$1>>2]|0;
    $295 = (_sqlite3GetTempReg($294)|0);
    HEAP32[$regRowid>>2] = $295;
    $296 = HEAP32[$1>>2]|0;
    $297 = HEAP32[$iDest>>2]|0;
    $298 = HEAP32[$5>>2]|0;
    $299 = HEAP32[$2>>2]|0;
    _sqlite3OpenTable($296,$297,$298,$299,105);
    $300 = HEAP32[$db>>2]|0;
    $301 = ((($300)) + 24|0);
    $302 = HEAP32[$301>>2]|0;
    $303 = $302 & 268435456;
    $304 = ($303|0)==(0);
    do {
     if ($304) {
      $305 = HEAP32[$2>>2]|0;
      $306 = ((($305)) + 32|0);
      $307 = HEAP16[$306>>1]|0;
      $308 = $307 << 16 >> 16;
      $309 = ($308|0)<(0);
      if ($309) {
       $310 = HEAP32[$2>>2]|0;
       $311 = ((($310)) + 8|0);
       $312 = HEAP32[$311>>2]|0;
       $313 = ($312|0)!=(0|0);
       $314 = HEAP32[$destHasUniqueIdx>>2]|0;
       $315 = ($314|0)!=(0);
       $or$cond = $313 | $315;
       if (!($or$cond)) {
        label = 87;
       }
      } else {
       $$old = HEAP32[$destHasUniqueIdx>>2]|0;
       $$old1 = ($$old|0)!=(0);
       if (!($$old1)) {
        label = 87;
       }
      }
      if ((label|0) == 87) {
       $316 = HEAP32[$4>>2]|0;
       $317 = ($316|0)!=(2);
       $318 = HEAP32[$4>>2]|0;
       $319 = ($318|0)!=(1);
       $or$cond4 = $317 & $319;
       if (!($or$cond4)) {
        break;
       }
      }
      $320 = HEAP32[$v>>2]|0;
      $321 = HEAP32[$iDest>>2]|0;
      $322 = (_sqlite3VdbeAddOp2($320,57,$321,0)|0);
      HEAP32[$addr1>>2] = $322;
      $323 = HEAP32[$v>>2]|0;
      $324 = (_sqlite3VdbeAddOp0($323,13)|0);
      HEAP32[$emptyDestTest>>2] = $324;
      $325 = HEAP32[$v>>2]|0;
      $326 = HEAP32[$addr1>>2]|0;
      _sqlite3VdbeJumpHere($325,$326);
     }
    } while(0);
    $327 = HEAP32[$pSrc>>2]|0;
    $328 = ((($327)) + 42|0);
    $329 = HEAP8[$328>>0]|0;
    $330 = $329&255;
    $331 = $330 & 32;
    $332 = ($331|0)==(0);
    $333 = HEAP32[$1>>2]|0;
    if ($332) {
     $334 = HEAP32[$iSrc>>2]|0;
     $335 = HEAP32[$iDbSrc>>2]|0;
     $336 = HEAP32[$pSrc>>2]|0;
     _sqlite3OpenTable($333,$334,$335,$336,104);
     $337 = HEAP32[$v>>2]|0;
     $338 = HEAP32[$iSrc>>2]|0;
     $339 = (_sqlite3VdbeAddOp2($337,57,$338,0)|0);
     HEAP32[$emptySrcTest>>2] = $339;
     $340 = HEAP32[$2>>2]|0;
     $341 = ((($340)) + 32|0);
     $342 = HEAP16[$341>>1]|0;
     $343 = $342 << 16 >> 16;
     $344 = ($343|0)>=(0);
     do {
      if ($344) {
       $345 = HEAP32[$v>>2]|0;
       $346 = HEAP32[$iSrc>>2]|0;
       $347 = HEAP32[$regRowid>>2]|0;
       $348 = (_sqlite3VdbeAddOp2($345,123,$346,$347)|0);
       HEAP32[$addr1>>2] = $348;
       $349 = HEAP32[$v>>2]|0;
       $350 = HEAP32[$iDest>>2]|0;
       $351 = HEAP32[$regRowid>>2]|0;
       $352 = (_sqlite3VdbeAddOp3($349,33,$350,0,$351)|0);
       HEAP32[$addr2>>2] = $352;
       $353 = HEAP32[$1>>2]|0;
       $354 = HEAP32[$4>>2]|0;
       $355 = HEAP32[$2>>2]|0;
       _sqlite3RowidConstraint($353,$354,$355);
       $356 = HEAP32[$v>>2]|0;
       $357 = HEAP32[$addr2>>2]|0;
       _sqlite3VdbeJumpHere($356,$357);
       $358 = HEAP32[$1>>2]|0;
       $359 = HEAP32[$regAutoinc>>2]|0;
       $360 = HEAP32[$regRowid>>2]|0;
       _autoIncStep($358,$359,$360);
      } else {
       $361 = HEAP32[$2>>2]|0;
       $362 = ((($361)) + 8|0);
       $363 = HEAP32[$362>>2]|0;
       $364 = ($363|0)==(0|0);
       $365 = HEAP32[$v>>2]|0;
       if ($364) {
        $366 = HEAP32[$iDest>>2]|0;
        $367 = HEAP32[$regRowid>>2]|0;
        $368 = (_sqlite3VdbeAddOp2($365,114,$366,$367)|0);
        HEAP32[$addr1>>2] = $368;
        break;
       } else {
        $369 = HEAP32[$iSrc>>2]|0;
        $370 = HEAP32[$regRowid>>2]|0;
        $371 = (_sqlite3VdbeAddOp2($365,123,$369,$370)|0);
        HEAP32[$addr1>>2] = $371;
        break;
       }
      }
     } while(0);
     $372 = HEAP32[$v>>2]|0;
     $373 = HEAP32[$iSrc>>2]|0;
     $374 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp2($372,122,$373,$374)|0);
     $375 = HEAP32[$v>>2]|0;
     $376 = HEAP32[$iDest>>2]|0;
     $377 = HEAP32[$regData>>2]|0;
     $378 = HEAP32[$regRowid>>2]|0;
     $379 = HEAP32[$2>>2]|0;
     (_sqlite3VdbeAddOp4($375,115,$376,$377,$378,$379,-20)|0);
     $380 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($380,11);
     $381 = HEAP32[$v>>2]|0;
     $382 = HEAP32[$iSrc>>2]|0;
     $383 = HEAP32[$addr1>>2]|0;
     (_sqlite3VdbeAddOp2($381,7,$382,$383)|0);
     $384 = HEAP32[$v>>2]|0;
     $385 = HEAP32[$iSrc>>2]|0;
     (_sqlite3VdbeAddOp2($384,111,$385,0)|0);
     $386 = HEAP32[$v>>2]|0;
     $387 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($386,111,$387,0)|0);
    } else {
     $388 = HEAP32[$5>>2]|0;
     $389 = HEAP32[$2>>2]|0;
     $390 = ((($389)) + 28|0);
     $391 = HEAP32[$390>>2]|0;
     $392 = HEAP32[$2>>2]|0;
     $393 = HEAP32[$392>>2]|0;
     _sqlite3TableLock($333,$388,$391,1,$393);
     $394 = HEAP32[$1>>2]|0;
     $395 = HEAP32[$iDbSrc>>2]|0;
     $396 = HEAP32[$pSrc>>2]|0;
     $397 = ((($396)) + 28|0);
     $398 = HEAP32[$397>>2]|0;
     $399 = HEAP32[$pSrc>>2]|0;
     $400 = HEAP32[$399>>2]|0;
     _sqlite3TableLock($394,$395,$398,0,$400);
    }
    $401 = HEAP32[$2>>2]|0;
    $402 = ((($401)) + 8|0);
    $403 = HEAP32[$402>>2]|0;
    HEAP32[$pDestIdx>>2] = $403;
    while(1) {
     $404 = HEAP32[$pDestIdx>>2]|0;
     $405 = ($404|0)!=(0|0);
     if (!($405)) {
      break;
     }
     HEAP8[$idxInsFlags>>0] = 0;
     $406 = HEAP32[$pSrc>>2]|0;
     $407 = ((($406)) + 8|0);
     $408 = HEAP32[$407>>2]|0;
     HEAP32[$pSrcIdx>>2] = $408;
     while(1) {
      $409 = HEAP32[$pSrcIdx>>2]|0;
      $410 = ($409|0)!=(0|0);
      if (!($410)) {
       break;
      }
      $411 = HEAP32[$pDestIdx>>2]|0;
      $412 = HEAP32[$pSrcIdx>>2]|0;
      $413 = (_xferCompatibleIndex($411,$412)|0);
      $414 = ($413|0)!=(0);
      if ($414) {
       break;
      }
      $415 = HEAP32[$pSrcIdx>>2]|0;
      $416 = ((($415)) + 20|0);
      $417 = HEAP32[$416>>2]|0;
      HEAP32[$pSrcIdx>>2] = $417;
     }
     $418 = HEAP32[$v>>2]|0;
     $419 = HEAP32[$iSrc>>2]|0;
     $420 = HEAP32[$pSrcIdx>>2]|0;
     $421 = ((($420)) + 44|0);
     $422 = HEAP32[$421>>2]|0;
     $423 = HEAP32[$iDbSrc>>2]|0;
     (_sqlite3VdbeAddOp3($418,104,$419,$422,$423)|0);
     $424 = HEAP32[$1>>2]|0;
     $425 = HEAP32[$pSrcIdx>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($424,$425);
     $426 = HEAP32[$v>>2]|0;
     $427 = HEAP32[$iDest>>2]|0;
     $428 = HEAP32[$pDestIdx>>2]|0;
     $429 = ((($428)) + 44|0);
     $430 = HEAP32[$429>>2]|0;
     $431 = HEAP32[$5>>2]|0;
     (_sqlite3VdbeAddOp3($426,105,$427,$430,$431)|0);
     $432 = HEAP32[$1>>2]|0;
     $433 = HEAP32[$pDestIdx>>2]|0;
     _sqlite3VdbeSetP4KeyInfo($432,$433);
     $434 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($434,1);
     $435 = HEAP32[$v>>2]|0;
     $436 = HEAP32[$iSrc>>2]|0;
     $437 = (_sqlite3VdbeAddOp2($435,57,$436,0)|0);
     HEAP32[$addr1>>2] = $437;
     $438 = HEAP32[$v>>2]|0;
     $439 = HEAP32[$iSrc>>2]|0;
     $440 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp2($438,121,$439,$440)|0);
     $441 = HEAP32[$db>>2]|0;
     $442 = ((($441)) + 24|0);
     $443 = HEAP32[$442>>2]|0;
     $444 = $443 & 268435456;
     $445 = ($444|0)!=(0);
     do {
      if ($445) {
       HEAP32[$i>>2] = 0;
       while(1) {
        $446 = HEAP32[$i>>2]|0;
        $447 = HEAP32[$pSrcIdx>>2]|0;
        $448 = ((($447)) + 52|0);
        $449 = HEAP16[$448>>1]|0;
        $450 = $449&65535;
        $451 = ($446|0)<($450|0);
        if (!($451)) {
         break;
        }
        $452 = HEAP32[$i>>2]|0;
        $453 = HEAP32[$pSrcIdx>>2]|0;
        $454 = ((($453)) + 32|0);
        $455 = HEAP32[$454>>2]|0;
        $456 = (($455) + ($452<<2)|0);
        $457 = HEAP32[$456>>2]|0;
        HEAP32[$zColl>>2] = $457;
        $458 = HEAP32[$zColl>>2]|0;
        $459 = (_sqlite3_stricmp(34834,$458)|0);
        $460 = ($459|0)!=(0);
        if ($460) {
         break;
        }
        $461 = HEAP32[$i>>2]|0;
        $462 = (($461) + 1)|0;
        HEAP32[$i>>2] = $462;
       }
       $463 = HEAP32[$i>>2]|0;
       $464 = HEAP32[$pSrcIdx>>2]|0;
       $465 = ((($464)) + 52|0);
       $466 = HEAP16[$465>>1]|0;
       $467 = $466&65535;
       $468 = ($463|0)==($467|0);
       if (!($468)) {
        break;
       }
       HEAP8[$idxInsFlags>>0] = 16;
       $469 = HEAP32[$v>>2]|0;
       $470 = HEAP32[$iDest>>2]|0;
       (_sqlite3VdbeAddOp3($469,53,$470,0,-1)|0);
      }
     } while(0);
     $471 = HEAP32[$pSrc>>2]|0;
     $472 = ((($471)) + 42|0);
     $473 = HEAP8[$472>>0]|0;
     $474 = $473&255;
     $475 = $474 & 32;
     $476 = ($475|0)==(0);
     do {
      if (!($476)) {
       $477 = HEAP32[$pDestIdx>>2]|0;
       $478 = ((($477)) + 55|0);
       $479 = HEAP8[$478>>0]|0;
       $480 = $479 & 3;
       $481 = $480&255;
       $482 = ($481|0)==(2);
       if (!($482)) {
        break;
       }
       $483 = HEAP8[$idxInsFlags>>0]|0;
       $484 = $483&255;
       $485 = $484 | 1;
       $486 = $485&255;
       HEAP8[$idxInsFlags>>0] = $486;
      }
     } while(0);
     $487 = HEAP32[$v>>2]|0;
     $488 = HEAP32[$iDest>>2]|0;
     $489 = HEAP32[$regData>>2]|0;
     (_sqlite3VdbeAddOp3($487,126,$488,$489,1)|0);
     $490 = HEAP32[$v>>2]|0;
     $491 = HEAP8[$idxInsFlags>>0]|0;
     _sqlite3VdbeChangeP5($490,$491);
     $492 = HEAP32[$v>>2]|0;
     $493 = HEAP32[$iSrc>>2]|0;
     $494 = HEAP32[$addr1>>2]|0;
     $495 = (($494) + 1)|0;
     (_sqlite3VdbeAddOp2($492,7,$493,$495)|0);
     $496 = HEAP32[$v>>2]|0;
     $497 = HEAP32[$addr1>>2]|0;
     _sqlite3VdbeJumpHere($496,$497);
     $498 = HEAP32[$v>>2]|0;
     $499 = HEAP32[$iSrc>>2]|0;
     (_sqlite3VdbeAddOp2($498,111,$499,0)|0);
     $500 = HEAP32[$v>>2]|0;
     $501 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($500,111,$501,0)|0);
     $502 = HEAP32[$pDestIdx>>2]|0;
     $503 = ((($502)) + 20|0);
     $504 = HEAP32[$503>>2]|0;
     HEAP32[$pDestIdx>>2] = $504;
    }
    $505 = HEAP32[$emptySrcTest>>2]|0;
    $506 = ($505|0)!=(0);
    if ($506) {
     $507 = HEAP32[$v>>2]|0;
     $508 = HEAP32[$emptySrcTest>>2]|0;
     _sqlite3VdbeJumpHere($507,$508);
    }
    $509 = HEAP32[$1>>2]|0;
    $510 = HEAP32[$regRowid>>2]|0;
    _sqlite3ReleaseTempReg($509,$510);
    $511 = HEAP32[$1>>2]|0;
    $512 = HEAP32[$regData>>2]|0;
    _sqlite3ReleaseTempReg($511,$512);
    $513 = HEAP32[$emptyDestTest>>2]|0;
    $514 = ($513|0)!=(0);
    if ($514) {
     $515 = HEAP32[$1>>2]|0;
     _sqlite3AutoincrementEnd($515);
     $516 = HEAP32[$v>>2]|0;
     (_sqlite3VdbeAddOp2($516,75,0,0)|0);
     $517 = HEAP32[$v>>2]|0;
     $518 = HEAP32[$emptyDestTest>>2]|0;
     _sqlite3VdbeJumpHere($517,$518);
     $519 = HEAP32[$v>>2]|0;
     $520 = HEAP32[$iDest>>2]|0;
     (_sqlite3VdbeAddOp2($519,111,$520,0)|0);
     HEAP32[$0>>2] = 0;
     $521 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($521|0);
    } else {
     HEAP32[$0>>2] = 1;
     $521 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($521|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $521 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($521|0);
}
function _autoIncBegin($pParse,$iDb,$pTab) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $memId = 0, $pInfo = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $memId = sp + 8|0;
 $pToplevel = sp + 4|0;
 $pInfo = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$memId>>2] = 0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 42|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & 8;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ((($11)) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $13 & 268435456;
  $15 = ($14|0)==(0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 124|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = ($18|0)!=(0|0);
   $20 = HEAP32[$1>>2]|0;
   if ($19) {
    $21 = ((($20)) + 124|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = $22;
   } else {
    $23 = $20;
   }
   HEAP32[$pToplevel>>2] = $23;
   $24 = HEAP32[$pToplevel>>2]|0;
   $25 = ((($24)) + 120|0);
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$pInfo>>2] = $26;
   while(1) {
    $27 = HEAP32[$pInfo>>2]|0;
    $28 = ($27|0)!=(0|0);
    if ($28) {
     $29 = HEAP32[$pInfo>>2]|0;
     $30 = ((($29)) + 4|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = HEAP32[$3>>2]|0;
     $33 = ($31|0)!=($32|0);
     $74 = $33;
    } else {
     $74 = 0;
    }
    $34 = HEAP32[$pInfo>>2]|0;
    if (!($74)) {
     break;
    }
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$pInfo>>2] = $35;
   }
   $36 = ($34|0)==(0|0);
   do {
    if ($36) {
     $37 = HEAP32[$1>>2]|0;
     $38 = HEAP32[$37>>2]|0;
     $39 = (_sqlite3DbMallocRawNN($38,16,0)|0);
     HEAP32[$pInfo>>2] = $39;
     $40 = HEAP32[$pInfo>>2]|0;
     $41 = ($40|0)==(0|0);
     if (!($41)) {
      $42 = HEAP32[$pToplevel>>2]|0;
      $43 = ((($42)) + 120|0);
      $44 = HEAP32[$43>>2]|0;
      $45 = HEAP32[$pInfo>>2]|0;
      HEAP32[$45>>2] = $44;
      $46 = HEAP32[$pInfo>>2]|0;
      $47 = HEAP32[$pToplevel>>2]|0;
      $48 = ((($47)) + 120|0);
      HEAP32[$48>>2] = $46;
      $49 = HEAP32[$3>>2]|0;
      $50 = HEAP32[$pInfo>>2]|0;
      $51 = ((($50)) + 4|0);
      HEAP32[$51>>2] = $49;
      $52 = HEAP32[$2>>2]|0;
      $53 = HEAP32[$pInfo>>2]|0;
      $54 = ((($53)) + 8|0);
      HEAP32[$54>>2] = $52;
      $55 = HEAP32[$pToplevel>>2]|0;
      $56 = ((($55)) + 44|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = (($57) + 1)|0;
      HEAP32[$56>>2] = $58;
      $59 = HEAP32[$pToplevel>>2]|0;
      $60 = ((($59)) + 44|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = (($61) + 1)|0;
      HEAP32[$60>>2] = $62;
      $63 = HEAP32[$pInfo>>2]|0;
      $64 = ((($63)) + 12|0);
      HEAP32[$64>>2] = $62;
      $65 = HEAP32[$pToplevel>>2]|0;
      $66 = ((($65)) + 44|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = (($67) + 1)|0;
      HEAP32[$66>>2] = $68;
      break;
     }
     HEAP32[$0>>2] = 0;
     $73 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($73|0);
    }
   } while(0);
   $69 = HEAP32[$pInfo>>2]|0;
   $70 = ((($69)) + 12|0);
   $71 = HEAP32[$70>>2]|0;
   HEAP32[$memId>>2] = $71;
  }
 }
 $72 = HEAP32[$memId>>2]|0;
 HEAP32[$0>>2] = $72;
 $73 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _sqlite3VdbeEndCoroutine($v,$regYield) {
 $v = $v|0;
 $regYield = $regYield|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $regYield;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp1($2,73,$3)|0);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 19|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 28|0);
 HEAP32[$11>>2] = 0;
 STACKTOP = sp;return;
}
function _readsTable($p,$iDb,$pTab) {
 $p = $p|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iEnd = 0, $pIndex = 0, $pOp = 0, $pVTab = 0, $tnum = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $iEnd = sp + 16|0;
 $pVTab = sp + 12|0;
 $pOp = sp + 8|0;
 $pIndex = sp + 4|0;
 $tnum = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $pTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3GetVdbe($4)|0);
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$v>>2]|0;
 $7 = (_sqlite3VdbeCurrentAddr($6)|0);
 HEAP32[$iEnd>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 42|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = $11 & 16;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_sqlite3GetVTable($15,$16)|0);
  $18 = $17;
 } else {
  $18 = 0;
 }
 HEAP32[$pVTab>>2] = $18;
 HEAP32[$i>>2] = 1;
 L4: while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$iEnd>>2]|0;
  $21 = ($19|0)<($20|0);
  if (!($21)) {
   label = 18;
   break;
  }
  $22 = HEAP32[$v>>2]|0;
  $23 = HEAP32[$i>>2]|0;
  $24 = (_sqlite3VdbeGetOp($22,$23)|0);
  HEAP32[$pOp>>2] = $24;
  $25 = HEAP32[$pOp>>2]|0;
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = ($27|0)==(104);
  L7: do {
   if ($28) {
    $29 = HEAP32[$pOp>>2]|0;
    $30 = ((($29)) + 12|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = HEAP32[$2>>2]|0;
    $33 = ($31|0)==($32|0);
    if ($33) {
     $34 = HEAP32[$pOp>>2]|0;
     $35 = ((($34)) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     HEAP32[$tnum>>2] = $36;
     $37 = HEAP32[$tnum>>2]|0;
     $38 = HEAP32[$3>>2]|0;
     $39 = ((($38)) + 28|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($37|0)==($40|0);
     if ($41) {
      label = 8;
      break L4;
     }
     $42 = HEAP32[$3>>2]|0;
     $43 = ((($42)) + 8|0);
     $44 = HEAP32[$43>>2]|0;
     HEAP32[$pIndex>>2] = $44;
     while(1) {
      $45 = HEAP32[$pIndex>>2]|0;
      $46 = ($45|0)!=(0|0);
      if (!($46)) {
       break L7;
      }
      $47 = HEAP32[$tnum>>2]|0;
      $48 = HEAP32[$pIndex>>2]|0;
      $49 = ((($48)) + 44|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = ($47|0)==($50|0);
      if ($51) {
       label = 12;
       break L4;
      }
      $52 = HEAP32[$pIndex>>2]|0;
      $53 = ((($52)) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      HEAP32[$pIndex>>2] = $54;
     }
    }
   }
  } while(0);
  $55 = HEAP32[$pOp>>2]|0;
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = ($57|0)==(155);
  if ($58) {
   $59 = HEAP32[$pOp>>2]|0;
   $60 = ((($59)) + 16|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$pVTab>>2]|0;
   $63 = ($61|0)==($62|0);
   if ($63) {
    label = 16;
    break;
   }
  }
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 1;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 18) {
  HEAP32[$0>>2] = 0;
  $66 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 return (0)|0;
}
function _sqlite3ExprCodeAndCache($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iMem = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $v = sp + 4|0;
 $iMem = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($6,$7,$8);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 44|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (($11) + 1)|0;
 HEAP32[$10>>2] = $12;
 HEAP32[$iMem>>2] = $12;
 $13 = HEAP32[$v>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$iMem>>2]|0;
 (_sqlite3VdbeAddOp2($13,84,$14,$15)|0);
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$iMem>>2]|0;
 _exprToRegister($16,$17);
 STACKTOP = sp;return;
}
function _sqlite3TableAffinity($v,$pTab,$iReg) {
 $v = $v|0;
 $pTab = $pTab|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $zColAff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 $zColAff = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iReg;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$zColAff>>2] = $5;
 $6 = HEAP32[$zColAff>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = (_sqlite3VdbeDb($8)|0);
  HEAP32[$db>>2] = $9;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 34|0);
  $12 = HEAP16[$11>>1]|0;
  $13 = $12 << 16 >> 16;
  $14 = (($13) + 1)|0;
  $15 = ($14|0)<(0);
  $16 = $15 << 31 >> 31;
  $17 = (_sqlite3DbMallocRaw(0,$14,$16)|0);
  HEAP32[$zColAff>>2] = $17;
  $18 = HEAP32[$zColAff>>2]|0;
  $19 = ($18|0)!=(0|0);
  if (!($19)) {
   $20 = HEAP32[$db>>2]|0;
   _sqlite3OomFault($20);
   STACKTOP = sp;return;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 34|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24 << 16 >> 16;
   $26 = ($21|0)<($25|0);
   if (!($26)) {
    break;
   }
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + ($27<<4)|0);
   $32 = ((($31)) + 13|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = HEAP32[$i>>2]|0;
   $35 = HEAP32[$zColAff>>2]|0;
   $36 = (($35) + ($34)|0);
   HEAP8[$36>>0] = $33;
   $37 = HEAP32[$i>>2]|0;
   $38 = (($37) + 1)|0;
   HEAP32[$i>>2] = $38;
  }
  while(1) {
   $39 = HEAP32[$i>>2]|0;
   $40 = (($39) + -1)|0;
   HEAP32[$i>>2] = $40;
   $41 = HEAP32[$zColAff>>2]|0;
   $42 = (($41) + ($39)|0);
   HEAP8[$42>>0] = 0;
   $43 = HEAP32[$i>>2]|0;
   $44 = ($43|0)>=(0);
   if (!($44)) {
    break;
   }
   $45 = HEAP32[$i>>2]|0;
   $46 = HEAP32[$zColAff>>2]|0;
   $47 = (($46) + ($45)|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48 << 24 >> 24;
   $50 = ($49|0)==(65);
   if (!($50)) {
    break;
   }
  }
  $51 = HEAP32[$zColAff>>2]|0;
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 20|0);
  HEAP32[$53>>2] = $51;
 }
 $54 = HEAP32[$zColAff>>2]|0;
 $55 = (_sqlite3Strlen30($54)|0);
 HEAP32[$i>>2] = $55;
 $56 = HEAP32[$i>>2]|0;
 $57 = ($56|0)!=(0);
 if (!($57)) {
  STACKTOP = sp;return;
 }
 $58 = HEAP32[$2>>2]|0;
 $59 = ($58|0)!=(0);
 $60 = HEAP32[$0>>2]|0;
 if ($59) {
  $61 = HEAP32[$2>>2]|0;
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$zColAff>>2]|0;
  $64 = HEAP32[$i>>2]|0;
  (_sqlite3VdbeAddOp4($60,98,$61,$62,0,$63,$64)|0);
  STACKTOP = sp;return;
 } else {
  $65 = HEAP32[$zColAff>>2]|0;
  $66 = HEAP32[$i>>2]|0;
  _sqlite3VdbeChangeP4($60,-1,$65,$66);
  STACKTOP = sp;return;
 }
}
function _sqlite3CodeRowTrigger($pParse,$pTrigger,$op,$pChanges,$tr_tm,$pTab,$reg,$orconf,$ignoreJump) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $op = $op|0;
 $pChanges = $pChanges|0;
 $tr_tm = $tr_tm|0;
 $pTab = $pTab|0;
 $reg = $reg|0;
 $orconf = $orconf|0;
 $ignoreJump = $ignoreJump|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp + 12|0;
 $7 = sp + 8|0;
 $8 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $tr_tm;
 HEAP32[$5>>2] = $pTab;
 HEAP32[$6>>2] = $reg;
 HEAP32[$7>>2] = $orconf;
 HEAP32[$8>>2] = $ignoreJump;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $9;
 while(1) {
  $10 = HEAP32[$p>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($15|0)==($16|0);
  if ($17) {
   $18 = HEAP32[$p>>2]|0;
   $19 = ((($18)) + 9|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = HEAP32[$4>>2]|0;
   $23 = ($21|0)==($22|0);
   if ($23) {
    $24 = HEAP32[$p>>2]|0;
    $25 = ((($24)) + 16|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = HEAP32[$3>>2]|0;
    $28 = (_checkColumnOverlap($26,$27)|0);
    $29 = ($28|0)!=(0);
    if ($29) {
     $30 = HEAP32[$0>>2]|0;
     $31 = HEAP32[$p>>2]|0;
     $32 = HEAP32[$5>>2]|0;
     $33 = HEAP32[$6>>2]|0;
     $34 = HEAP32[$7>>2]|0;
     $35 = HEAP32[$8>>2]|0;
     _sqlite3CodeRowTriggerDirect($30,$31,$32,$33,$34,$35);
    }
   }
  }
  $36 = HEAP32[$p>>2]|0;
  $37 = ((($36)) + 32|0);
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$p>>2] = $38;
 }
 STACKTOP = sp;return;
}
function _autoIncStep($pParse,$memId,$regRowid) {
 $pParse = $pParse|0;
 $memId = $memId|0;
 $regRowid = $regRowid|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $memId;
 HEAP32[$2>>2] = $regRowid;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp2($7,145,$8,$9)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeFactorable($pParse,$pExpr,$target) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $target = $target|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $target;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 23|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_sqlite3ExprIsConstant($8)|0);
  $10 = ($9|0)!=(0);
  if ($10) {
   $11 = HEAP32[$0>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$2>>2]|0;
   _sqlite3ExprCodeAtInit($11,$12,$13,0);
   STACKTOP = sp;return;
  }
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 _sqlite3ExprCode($14,$15,$16);
 STACKTOP = sp;return;
}
function _sqlite3VtabMakeWritable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $apVtabLock = 0, $i = 0, $n = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pToplevel = sp + 12|0;
 $i = sp + 8|0;
 $n = sp + 4|0;
 $apVtabLock = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 124|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 124|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = $8;
 } else {
  $9 = $6;
 }
 HEAP32[$pToplevel>>2] = $9;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$pToplevel>>2]|0;
  $12 = ((($11)) + 412|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$pToplevel>>2]|0;
  $18 = ((($17)) + 460|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16<<2)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($15|0)==($21|0);
  if ($22) {
   label = 10;
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
 }
 if ((label|0) == 10) {
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$pToplevel>>2]|0;
 $26 = ((($25)) + 412|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 $29 = $28<<2;
 HEAP32[$n>>2] = $29;
 $30 = HEAP32[$pToplevel>>2]|0;
 $31 = ((($30)) + 460|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$n>>2]|0;
 $34 = ($33|0)<(0);
 $35 = $34 << 31 >> 31;
 $36 = (_sqlite3_realloc64($32,$33,$35)|0);
 HEAP32[$apVtabLock>>2] = $36;
 $37 = HEAP32[$apVtabLock>>2]|0;
 $38 = ($37|0)!=(0|0);
 if ($38) {
  $39 = HEAP32[$apVtabLock>>2]|0;
  $40 = HEAP32[$pToplevel>>2]|0;
  $41 = ((($40)) + 460|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$pToplevel>>2]|0;
  $44 = ((($43)) + 412|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$44>>2] = $46;
  $47 = HEAP32[$pToplevel>>2]|0;
  $48 = ((($47)) + 460|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + ($45<<2)|0);
  HEAP32[$50>>2] = $42;
  STACKTOP = sp;return;
 } else {
  $51 = HEAP32[$pToplevel>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  _sqlite3OomFault($52);
  STACKTOP = sp;return;
 }
}
function _sqlite3GenerateConstraintChecks($pParse,$pTab,$aRegIdx,$iDataCur,$iIdxCur,$regNewData,$regOldData,$pkChng,$overrideError,$ignoreDest,$pbMayReplace,$aiChng) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $aRegIdx = $aRegIdx|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $regNewData = $regNewData|0;
 $regOldData = $regOldData|0;
 $pkChng = $pkChng|0;
 $overrideError = $overrideError|0;
 $ignoreDest = $ignoreDest|0;
 $pbMayReplace = $pbMayReplace|0;
 $aiChng = $aiChng|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr1 = 0;
 var $addrJump = 0, $addrRowidOk = 0, $addrUniqueOk = 0, $allOk = 0, $bAffinityDone = 0, $db = 0, $i = 0, $iField = 0, $iThisCur = 0, $ipkBottom = 0, $ipkTop = 0, $isUpdate = 0, $ix = 0, $nCol = 0, $nPkField = 0, $onError = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $p4 = 0;
 var $pCheck = 0, $pExpr = 0, $pIdx = 0, $pPk = 0, $pTrigger = 0, $pTrigger2 = 0, $regCmp = 0, $regIdx = 0, $regR = 0, $regRowid = 0, $seenReplace = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr4 = 0, $x = 0, $x1 = 0, $zMsg = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer = sp;
 $0 = sp + 176|0;
 $1 = sp + 172|0;
 $2 = sp + 168|0;
 $3 = sp + 164|0;
 $4 = sp + 160|0;
 $5 = sp + 156|0;
 $6 = sp + 152|0;
 $7 = sp + 183|0;
 $8 = sp + 182|0;
 $9 = sp + 148|0;
 $10 = sp + 144|0;
 $11 = sp + 140|0;
 $v = sp + 136|0;
 $pIdx = sp + 132|0;
 $pPk = sp + 128|0;
 $db = sp + 124|0;
 $i = sp + 120|0;
 $ix = sp + 116|0;
 $nCol = sp + 112|0;
 $onError = sp + 108|0;
 $addr1 = sp + 104|0;
 $seenReplace = sp + 100|0;
 $nPkField = sp + 96|0;
 $ipkTop = sp + 92|0;
 $ipkBottom = sp + 88|0;
 $isUpdate = sp + 181|0;
 $bAffinityDone = sp + 180|0;
 $regRowid = sp + 84|0;
 $zMsg = sp + 80|0;
 $pCheck = sp + 76|0;
 $allOk = sp + 72|0;
 $pExpr = sp + 68|0;
 $zName = sp + 64|0;
 $addrRowidOk = sp + 60|0;
 $pTrigger = sp + 56|0;
 $regIdx = sp + 52|0;
 $regR = sp + 48|0;
 $iThisCur = sp + 44|0;
 $addrUniqueOk = sp + 40|0;
 $iField = sp + 36|0;
 $x = sp + 32|0;
 $x1 = sp + 28|0;
 $addrJump = sp + 24|0;
 $op = sp + 20|0;
 $regCmp = sp + 16|0;
 $p4 = sp + 12|0;
 $pTrigger2 = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $aRegIdx;
 HEAP32[$3>>2] = $iDataCur;
 HEAP32[$4>>2] = $iIdxCur;
 HEAP32[$5>>2] = $regNewData;
 HEAP32[$6>>2] = $regOldData;
 HEAP8[$7>>0] = $pkChng;
 HEAP8[$8>>0] = $overrideError;
 HEAP32[$9>>2] = $ignoreDest;
 HEAP32[$10>>2] = $pbMayReplace;
 HEAP32[$11>>2] = $aiChng;
 HEAP32[$pPk>>2] = 0;
 HEAP32[$seenReplace>>2] = 0;
 HEAP32[$ipkTop>>2] = 0;
 HEAP32[$ipkBottom>>2] = 0;
 HEAP8[$bAffinityDone>>0] = 0;
 HEAP32[$regRowid>>2] = -1;
 $12 = HEAP32[$6>>2]|0;
 $13 = ($12|0)!=(0);
 $14 = $13&1;
 $15 = $14&255;
 HEAP8[$isUpdate>>0] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$db>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = (_sqlite3GetVdbe($18)|0);
 HEAP32[$v>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 34|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 HEAP32[$nCol>>2] = $23;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 42|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = $26&255;
 $28 = $27 & 32;
 $29 = ($28|0)==(0);
 if ($29) {
  HEAP32[$pPk>>2] = 0;
  HEAP32[$nPkField>>2] = 1;
 } else {
  $30 = HEAP32[$1>>2]|0;
  $31 = (_sqlite3PrimaryKeyIndex($30)|0);
  HEAP32[$pPk>>2] = $31;
  $32 = HEAP32[$pPk>>2]|0;
  $33 = ((($32)) + 50|0);
  $34 = HEAP16[$33>>1]|0;
  $35 = $34&65535;
  HEAP32[$nPkField>>2] = $35;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$nCol>>2]|0;
  $38 = ($36|0)<($37|0);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 32|0);
  $42 = HEAP16[$41>>1]|0;
  $43 = $42 << 16 >> 16;
  $44 = ($39|0)==($43|0);
  L8: do {
   if (!($44)) {
    $45 = HEAP32[$11>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$i>>2]|0;
     $48 = HEAP32[$11>>2]|0;
     $49 = (($48) + ($47<<2)|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = ($50|0)<(0);
     if ($51) {
      break;
     }
    }
    $52 = HEAP32[$i>>2]|0;
    $53 = HEAP32[$1>>2]|0;
    $54 = ((($53)) + 4|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = (($55) + ($52<<4)|0);
    $57 = ((($56)) + 12|0);
    $58 = HEAP8[$57>>0]|0;
    $59 = $58&255;
    HEAP32[$onError>>2] = $59;
    $60 = HEAP32[$onError>>2]|0;
    $61 = ($60|0)==(0);
    if (!($61)) {
     $62 = HEAP8[$8>>0]|0;
     $63 = $62&255;
     $64 = ($63|0)!=(10);
     if ($64) {
      $65 = HEAP8[$8>>0]|0;
      $66 = $65&255;
      HEAP32[$onError>>2] = $66;
     } else {
      $67 = HEAP32[$onError>>2]|0;
      $68 = ($67|0)==(10);
      if ($68) {
       HEAP32[$onError>>2] = 2;
      }
     }
     $69 = HEAP32[$onError>>2]|0;
     $70 = ($69|0)==(5);
     if ($70) {
      $71 = HEAP32[$i>>2]|0;
      $72 = HEAP32[$1>>2]|0;
      $73 = ((($72)) + 4|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + ($71<<4)|0);
      $76 = ((($75)) + 4|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = ($77|0)==(0|0);
      if ($78) {
       HEAP32[$onError>>2] = 2;
      }
     }
     $79 = HEAP32[$onError>>2]|0;
     switch ($79|0) {
     case 2:  {
      $80 = HEAP32[$0>>2]|0;
      _sqlite3MayAbort($80);
      break;
     }
     case 3: case 1:  {
      break;
     }
     case 4:  {
      $99 = HEAP32[$v>>2]|0;
      $100 = HEAP32[$5>>2]|0;
      $101 = (($100) + 1)|0;
      $102 = HEAP32[$i>>2]|0;
      $103 = (($101) + ($102))|0;
      $104 = HEAP32[$9>>2]|0;
      (_sqlite3VdbeAddOp2($99,34,$103,$104)|0);
      break L8;
      break;
     }
     default: {
      $105 = HEAP32[$v>>2]|0;
      $106 = HEAP32[$5>>2]|0;
      $107 = (($106) + 1)|0;
      $108 = HEAP32[$i>>2]|0;
      $109 = (($107) + ($108))|0;
      $110 = (_sqlite3VdbeAddOp1($105,35,$109)|0);
      HEAP32[$addr1>>2] = $110;
      $111 = HEAP32[$0>>2]|0;
      $112 = HEAP32[$i>>2]|0;
      $113 = HEAP32[$1>>2]|0;
      $114 = ((($113)) + 4|0);
      $115 = HEAP32[$114>>2]|0;
      $116 = (($115) + ($112<<4)|0);
      $117 = ((($116)) + 4|0);
      $118 = HEAP32[$117>>2]|0;
      $119 = HEAP32[$5>>2]|0;
      $120 = (($119) + 1)|0;
      $121 = HEAP32[$i>>2]|0;
      $122 = (($120) + ($121))|0;
      _sqlite3ExprCode($111,$118,$122);
      $123 = HEAP32[$v>>2]|0;
      $124 = HEAP32[$addr1>>2]|0;
      _sqlite3VdbeJumpHere($123,$124);
      break L8;
     }
     }
     $81 = HEAP32[$db>>2]|0;
     $82 = HEAP32[$1>>2]|0;
     $83 = HEAP32[$82>>2]|0;
     $84 = HEAP32[$i>>2]|0;
     $85 = HEAP32[$1>>2]|0;
     $86 = ((($85)) + 4|0);
     $87 = HEAP32[$86>>2]|0;
     $88 = (($87) + ($84<<4)|0);
     $89 = HEAP32[$88>>2]|0;
     HEAP32[$vararg_buffer>>2] = $83;
     $vararg_ptr4 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr4>>2] = $89;
     $90 = (_sqlite3MPrintf($81,30161,$vararg_buffer)|0);
     HEAP32[$zMsg>>2] = $90;
     $91 = HEAP32[$v>>2]|0;
     $92 = HEAP32[$onError>>2]|0;
     $93 = HEAP32[$5>>2]|0;
     $94 = (($93) + 1)|0;
     $95 = HEAP32[$i>>2]|0;
     $96 = (($94) + ($95))|0;
     $97 = HEAP32[$zMsg>>2]|0;
     (_sqlite3VdbeAddOp4($91,74,1299,$92,$96,$97,-1)|0);
     $98 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($98,1);
    }
   }
  } while(0);
  $125 = HEAP32[$i>>2]|0;
  $126 = (($125) + 1)|0;
  HEAP32[$i>>2] = $126;
 }
 $127 = HEAP32[$1>>2]|0;
 $128 = ((($127)) + 24|0);
 $129 = HEAP32[$128>>2]|0;
 $130 = ($129|0)!=(0|0);
 L30: do {
  if ($130) {
   $131 = HEAP32[$db>>2]|0;
   $132 = ((($131)) + 24|0);
   $133 = HEAP32[$132>>2]|0;
   $134 = $133 & 8192;
   $135 = ($134|0)==(0);
   if ($135) {
    $136 = HEAP32[$1>>2]|0;
    $137 = ((($136)) + 24|0);
    $138 = HEAP32[$137>>2]|0;
    HEAP32[$pCheck>>2] = $138;
    $139 = HEAP32[$5>>2]|0;
    $140 = (($139) + 1)|0;
    $141 = HEAP32[$0>>2]|0;
    $142 = ((($141)) + 56|0);
    HEAP32[$142>>2] = $140;
    $143 = HEAP8[$8>>0]|0;
    $144 = $143&255;
    $145 = ($144|0)!=(10);
    $146 = HEAP8[$8>>0]|0;
    $147 = $146&255;
    $148 = $145 ? $147 : 2;
    HEAP32[$onError>>2] = $148;
    HEAP32[$i>>2] = 0;
    while(1) {
     $149 = HEAP32[$i>>2]|0;
     $150 = HEAP32[$pCheck>>2]|0;
     $151 = HEAP32[$150>>2]|0;
     $152 = ($149|0)<($151|0);
     if (!($152)) {
      break L30;
     }
     $153 = HEAP32[$i>>2]|0;
     $154 = HEAP32[$pCheck>>2]|0;
     $155 = ((($154)) + 4|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = (($156) + (($153*20)|0)|0);
     $158 = HEAP32[$157>>2]|0;
     HEAP32[$pExpr>>2] = $158;
     $159 = HEAP32[$11>>2]|0;
     $160 = ($159|0)!=(0|0);
     if ($160) {
      $161 = HEAP32[$pExpr>>2]|0;
      $162 = HEAP32[$11>>2]|0;
      $163 = HEAP8[$7>>0]|0;
      $164 = $163&255;
      $165 = (_checkConstraintUnchanged($161,$162,$164)|0);
      $166 = ($165|0)!=(0);
      if (!($166)) {
       label = 29;
      }
     } else {
      label = 29;
     }
     if ((label|0) == 29) {
      label = 0;
      $167 = HEAP32[$v>>2]|0;
      $168 = (_sqlite3VdbeMakeLabel($167)|0);
      HEAP32[$allOk>>2] = $168;
      $169 = HEAP32[$0>>2]|0;
      $170 = HEAP32[$pExpr>>2]|0;
      $171 = HEAP32[$allOk>>2]|0;
      _sqlite3ExprIfTrue($169,$170,$171,16);
      $172 = HEAP32[$onError>>2]|0;
      $173 = ($172|0)==(4);
      if ($173) {
       $174 = HEAP32[$v>>2]|0;
       $175 = HEAP32[$9>>2]|0;
       (_sqlite3VdbeGoto($174,$175)|0);
      } else {
       $176 = HEAP32[$i>>2]|0;
       $177 = HEAP32[$pCheck>>2]|0;
       $178 = ((($177)) + 4|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = (($179) + (($176*20)|0)|0);
       $181 = ((($180)) + 4|0);
       $182 = HEAP32[$181>>2]|0;
       HEAP32[$zName>>2] = $182;
       $183 = HEAP32[$zName>>2]|0;
       $184 = ($183|0)==(0|0);
       if ($184) {
        $185 = HEAP32[$1>>2]|0;
        $186 = HEAP32[$185>>2]|0;
        HEAP32[$zName>>2] = $186;
       }
       $187 = HEAP32[$onError>>2]|0;
       $188 = ($187|0)==(5);
       if ($188) {
        HEAP32[$onError>>2] = 2;
       }
       $189 = HEAP32[$0>>2]|0;
       $190 = HEAP32[$onError>>2]|0;
       $191 = HEAP32[$zName>>2]|0;
       _sqlite3HaltConstraint($189,275,$190,$191,0,3);
      }
      $192 = HEAP32[$v>>2]|0;
      $193 = HEAP32[$allOk>>2]|0;
      _sqlite3VdbeResolveLabel($192,$193);
     }
     $194 = HEAP32[$i>>2]|0;
     $195 = (($194) + 1)|0;
     HEAP32[$i>>2] = $195;
    }
   }
  }
 } while(0);
 $196 = HEAP8[$7>>0]|0;
 $197 = $196&255;
 $198 = ($197|0)!=(0);
 $199 = HEAP32[$pPk>>2]|0;
 $200 = ($199|0)==(0|0);
 $or$cond = $198 & $200;
 if ($or$cond) {
  $201 = HEAP32[$v>>2]|0;
  $202 = (_sqlite3VdbeMakeLabel($201)|0);
  HEAP32[$addrRowidOk>>2] = $202;
  $203 = HEAP32[$1>>2]|0;
  $204 = ((($203)) + 43|0);
  $205 = HEAP8[$204>>0]|0;
  $206 = $205&255;
  HEAP32[$onError>>2] = $206;
  $207 = HEAP8[$8>>0]|0;
  $208 = $207&255;
  $209 = ($208|0)!=(10);
  if ($209) {
   $210 = HEAP8[$8>>0]|0;
   $211 = $210&255;
   HEAP32[$onError>>2] = $211;
  } else {
   $212 = HEAP32[$onError>>2]|0;
   $213 = ($212|0)==(10);
   if ($213) {
    HEAP32[$onError>>2] = 2;
   }
  }
  $214 = HEAP8[$isUpdate>>0]|0;
  $215 = ($214<<24>>24)!=(0);
  if ($215) {
   $216 = HEAP32[$v>>2]|0;
   $217 = HEAP32[$5>>2]|0;
   $218 = HEAP32[$addrRowidOk>>2]|0;
   $219 = HEAP32[$6>>2]|0;
   (_sqlite3VdbeAddOp3($216,37,$217,$218,$219)|0);
   $220 = HEAP32[$v>>2]|0;
   _sqlite3VdbeChangeP5($220,-112);
  }
  $221 = HEAP32[$onError>>2]|0;
  $222 = ($221|0)==(5);
  L62: do {
   if ($222) {
    $223 = HEAP8[$8>>0]|0;
    $224 = $223&255;
    $225 = ($224|0)!=(5);
    if ($225) {
     $226 = HEAP32[$1>>2]|0;
     $227 = ((($226)) + 8|0);
     $228 = HEAP32[$227>>2]|0;
     HEAP32[$pIdx>>2] = $228;
     while(1) {
      $229 = HEAP32[$pIdx>>2]|0;
      $230 = ($229|0)!=(0|0);
      if (!($230)) {
       break L62;
      }
      $231 = HEAP32[$pIdx>>2]|0;
      $232 = ((($231)) + 54|0);
      $233 = HEAP8[$232>>0]|0;
      $234 = $233&255;
      $235 = ($234|0)==(4);
      if ($235) {
       break;
      }
      $236 = HEAP32[$pIdx>>2]|0;
      $237 = ((($236)) + 54|0);
      $238 = HEAP8[$237>>0]|0;
      $239 = $238&255;
      $240 = ($239|0)==(3);
      if ($240) {
       break;
      }
      $243 = HEAP32[$pIdx>>2]|0;
      $244 = ((($243)) + 20|0);
      $245 = HEAP32[$244>>2]|0;
      HEAP32[$pIdx>>2] = $245;
     }
     $241 = HEAP32[$v>>2]|0;
     $242 = (_sqlite3VdbeAddOp0($241,13)|0);
     HEAP32[$ipkTop>>2] = $242;
    }
   }
  } while(0);
  $246 = HEAP32[$v>>2]|0;
  $247 = HEAP32[$3>>2]|0;
  $248 = HEAP32[$addrRowidOk>>2]|0;
  $249 = HEAP32[$5>>2]|0;
  (_sqlite3VdbeAddOp3($246,33,$247,$248,$249)|0);
  $250 = HEAP32[$onError>>2]|0;
  switch ($250|0) {
  case 3: case 2: case 1:  {
   label = 55;
   break;
  }
  case 5:  {
   HEAP32[$pTrigger>>2] = 0;
   $254 = HEAP32[$db>>2]|0;
   $255 = ((($254)) + 24|0);
   $256 = HEAP32[$255>>2]|0;
   $257 = $256 & 262144;
   $258 = ($257|0)!=(0);
   if ($258) {
    $259 = HEAP32[$0>>2]|0;
    $260 = HEAP32[$1>>2]|0;
    $261 = (_sqlite3TriggersExist($259,$260,109,0,0)|0);
    HEAP32[$pTrigger>>2] = $261;
   }
   $262 = HEAP32[$pTrigger>>2]|0;
   $263 = ($262|0)!=(0|0);
   if ($263) {
    label = 60;
   } else {
    $264 = HEAP32[$0>>2]|0;
    $265 = HEAP32[$1>>2]|0;
    $266 = (_sqlite3FkRequired($264,$265,0,0)|0);
    $267 = ($266|0)!=(0);
    if ($267) {
     label = 60;
    } else {
     $275 = HEAP32[$1>>2]|0;
     $276 = ((($275)) + 8|0);
     $277 = HEAP32[$276>>2]|0;
     $278 = ($277|0)!=(0|0);
     if ($278) {
      $279 = HEAP32[$0>>2]|0;
      _sqlite3MultiWrite($279);
      $280 = HEAP32[$0>>2]|0;
      $281 = HEAP32[$1>>2]|0;
      $282 = HEAP32[$3>>2]|0;
      $283 = HEAP32[$4>>2]|0;
      _sqlite3GenerateRowIndexDelete($280,$281,$282,$283,0,-1);
     }
    }
   }
   if ((label|0) == 60) {
    $268 = HEAP32[$0>>2]|0;
    _sqlite3MultiWrite($268);
    $269 = HEAP32[$0>>2]|0;
    $270 = HEAP32[$1>>2]|0;
    $271 = HEAP32[$pTrigger>>2]|0;
    $272 = HEAP32[$3>>2]|0;
    $273 = HEAP32[$4>>2]|0;
    $274 = HEAP32[$5>>2]|0;
    _sqlite3GenerateRowDelete($269,$270,$271,$272,$273,$274,1,0,5,1,-1);
   }
   HEAP32[$seenReplace>>2] = 1;
   break;
  }
  case 4:  {
   $284 = HEAP32[$v>>2]|0;
   $285 = HEAP32[$9>>2]|0;
   (_sqlite3VdbeGoto($284,$285)|0);
   break;
  }
  default: {
   HEAP32[$onError>>2] = 2;
   label = 55;
  }
  }
  if ((label|0) == 55) {
   $251 = HEAP32[$0>>2]|0;
   $252 = HEAP32[$onError>>2]|0;
   $253 = HEAP32[$1>>2]|0;
   _sqlite3RowidConstraint($251,$252,$253);
  }
  $286 = HEAP32[$v>>2]|0;
  $287 = HEAP32[$addrRowidOk>>2]|0;
  _sqlite3VdbeResolveLabel($286,$287);
  $288 = HEAP32[$ipkTop>>2]|0;
  $289 = ($288|0)!=(0);
  if ($289) {
   $290 = HEAP32[$v>>2]|0;
   $291 = (_sqlite3VdbeAddOp0($290,13)|0);
   HEAP32[$ipkBottom>>2] = $291;
   $292 = HEAP32[$v>>2]|0;
   $293 = HEAP32[$ipkTop>>2]|0;
   _sqlite3VdbeJumpHere($292,$293);
  }
 }
 HEAP32[$ix>>2] = 0;
 $294 = HEAP32[$1>>2]|0;
 $295 = ((($294)) + 8|0);
 $296 = HEAP32[$295>>2]|0;
 HEAP32[$pIdx>>2] = $296;
 while(1) {
  $297 = HEAP32[$pIdx>>2]|0;
  $298 = ($297|0)!=(0|0);
  if (!($298)) {
   break;
  }
  $299 = HEAP32[$ix>>2]|0;
  $300 = HEAP32[$2>>2]|0;
  $301 = (($300) + ($299<<2)|0);
  $302 = HEAP32[$301>>2]|0;
  $303 = ($302|0)==(0);
  do {
   if (!($303)) {
    $304 = HEAP8[$bAffinityDone>>0]|0;
    $305 = $304&255;
    $306 = ($305|0)==(0);
    if ($306) {
     $307 = HEAP32[$v>>2]|0;
     $308 = HEAP32[$1>>2]|0;
     $309 = HEAP32[$5>>2]|0;
     $310 = (($309) + 1)|0;
     _sqlite3TableAffinity($307,$308,$310);
     HEAP8[$bAffinityDone>>0] = 1;
    }
    $311 = HEAP32[$4>>2]|0;
    $312 = HEAP32[$ix>>2]|0;
    $313 = (($311) + ($312))|0;
    HEAP32[$iThisCur>>2] = $313;
    $314 = HEAP32[$v>>2]|0;
    $315 = (_sqlite3VdbeMakeLabel($314)|0);
    HEAP32[$addrUniqueOk>>2] = $315;
    $316 = HEAP32[$pIdx>>2]|0;
    $317 = ((($316)) + 36|0);
    $318 = HEAP32[$317>>2]|0;
    $319 = ($318|0)!=(0|0);
    if ($319) {
     $320 = HEAP32[$v>>2]|0;
     $321 = HEAP32[$ix>>2]|0;
     $322 = HEAP32[$2>>2]|0;
     $323 = (($322) + ($321<<2)|0);
     $324 = HEAP32[$323>>2]|0;
     (_sqlite3VdbeAddOp2($320,79,0,$324)|0);
     $325 = HEAP32[$5>>2]|0;
     $326 = (($325) + 1)|0;
     $327 = HEAP32[$0>>2]|0;
     $328 = ((($327)) + 56|0);
     HEAP32[$328>>2] = $326;
     $329 = HEAP32[$0>>2]|0;
     $330 = HEAP32[$pIdx>>2]|0;
     $331 = ((($330)) + 36|0);
     $332 = HEAP32[$331>>2]|0;
     $333 = HEAP32[$addrUniqueOk>>2]|0;
     _sqlite3ExprIfFalseDup($329,$332,$333,16);
     $334 = HEAP32[$0>>2]|0;
     $335 = ((($334)) + 56|0);
     HEAP32[$335>>2] = 0;
    }
    $336 = HEAP32[$0>>2]|0;
    $337 = HEAP32[$pIdx>>2]|0;
    $338 = ((($337)) + 52|0);
    $339 = HEAP16[$338>>1]|0;
    $340 = $339&65535;
    $341 = (_sqlite3GetTempRange($336,$340)|0);
    HEAP32[$regIdx>>2] = $341;
    HEAP32[$i>>2] = 0;
    while(1) {
     $342 = HEAP32[$i>>2]|0;
     $343 = HEAP32[$pIdx>>2]|0;
     $344 = ((($343)) + 52|0);
     $345 = HEAP16[$344>>1]|0;
     $346 = $345&65535;
     $347 = ($342|0)<($346|0);
     if (!($347)) {
      break;
     }
     $348 = HEAP32[$i>>2]|0;
     $349 = HEAP32[$pIdx>>2]|0;
     $350 = ((($349)) + 4|0);
     $351 = HEAP32[$350>>2]|0;
     $352 = (($351) + ($348<<1)|0);
     $353 = HEAP16[$352>>1]|0;
     $354 = $353 << 16 >> 16;
     HEAP32[$iField>>2] = $354;
     $355 = HEAP32[$iField>>2]|0;
     $356 = ($355|0)==(-2);
     do {
      if ($356) {
       $357 = HEAP32[$5>>2]|0;
       $358 = (($357) + 1)|0;
       $359 = HEAP32[$0>>2]|0;
       $360 = ((($359)) + 56|0);
       HEAP32[$360>>2] = $358;
       $361 = HEAP32[$0>>2]|0;
       $362 = HEAP32[$i>>2]|0;
       $363 = HEAP32[$pIdx>>2]|0;
       $364 = ((($363)) + 40|0);
       $365 = HEAP32[$364>>2]|0;
       $366 = ((($365)) + 4|0);
       $367 = HEAP32[$366>>2]|0;
       $368 = (($367) + (($362*20)|0)|0);
       $369 = HEAP32[$368>>2]|0;
       $370 = HEAP32[$regIdx>>2]|0;
       $371 = HEAP32[$i>>2]|0;
       $372 = (($370) + ($371))|0;
       _sqlite3ExprCodeCopy($361,$369,$372);
       $373 = HEAP32[$0>>2]|0;
       $374 = ((($373)) + 56|0);
       HEAP32[$374>>2] = 0;
      } else {
       $375 = HEAP32[$iField>>2]|0;
       $376 = ($375|0)==(-1);
       if ($376) {
        label = 80;
       } else {
        $377 = HEAP32[$iField>>2]|0;
        $378 = HEAP32[$1>>2]|0;
        $379 = ((($378)) + 32|0);
        $380 = HEAP16[$379>>1]|0;
        $381 = $380 << 16 >> 16;
        $382 = ($377|0)==($381|0);
        if ($382) {
         label = 80;
        } else {
         $397 = HEAP32[$iField>>2]|0;
         $398 = HEAP32[$5>>2]|0;
         $399 = (($397) + ($398))|0;
         $400 = (($399) + 1)|0;
         HEAP32[$x>>2] = $400;
        }
       }
       if ((label|0) == 80) {
        label = 0;
        $383 = HEAP32[$regRowid>>2]|0;
        $384 = HEAP32[$regIdx>>2]|0;
        $385 = HEAP32[$i>>2]|0;
        $386 = (($384) + ($385))|0;
        $387 = ($383|0)==($386|0);
        if ($387) {
         break;
        }
        $388 = HEAP32[$5>>2]|0;
        HEAP32[$x>>2] = $388;
        $389 = HEAP32[$pIdx>>2]|0;
        $390 = ((($389)) + 36|0);
        $391 = HEAP32[$390>>2]|0;
        $392 = ($391|0)!=(0|0);
        if ($392) {
         $396 = -1;
        } else {
         $393 = HEAP32[$regIdx>>2]|0;
         $394 = HEAP32[$i>>2]|0;
         $395 = (($393) + ($394))|0;
         $396 = $395;
        }
        HEAP32[$regRowid>>2] = $396;
       }
       $401 = HEAP32[$v>>2]|0;
       $402 = HEAP32[$iField>>2]|0;
       $403 = ($402|0)<(0);
       $404 = $403 ? 86 : 85;
       $405 = HEAP32[$x>>2]|0;
       $406 = HEAP32[$regIdx>>2]|0;
       $407 = HEAP32[$i>>2]|0;
       $408 = (($406) + ($407))|0;
       (_sqlite3VdbeAddOp2($401,$404,$405,$408)|0);
      }
     } while(0);
     $409 = HEAP32[$i>>2]|0;
     $410 = (($409) + 1)|0;
     HEAP32[$i>>2] = $410;
    }
    $411 = HEAP32[$v>>2]|0;
    $412 = HEAP32[$regIdx>>2]|0;
    $413 = HEAP32[$pIdx>>2]|0;
    $414 = ((($413)) + 52|0);
    $415 = HEAP16[$414>>1]|0;
    $416 = $415&65535;
    $417 = HEAP32[$ix>>2]|0;
    $418 = HEAP32[$2>>2]|0;
    $419 = (($418) + ($417<<2)|0);
    $420 = HEAP32[$419>>2]|0;
    (_sqlite3VdbeAddOp3($411,99,$412,$416,$420)|0);
    $421 = HEAP32[$0>>2]|0;
    $422 = HEAP32[$regIdx>>2]|0;
    $423 = HEAP32[$pIdx>>2]|0;
    $424 = ((($423)) + 52|0);
    $425 = HEAP16[$424>>1]|0;
    $426 = $425&65535;
    _sqlite3ExprCacheAffinityChange($421,$422,$426);
    $427 = HEAP8[$isUpdate>>0]|0;
    $428 = $427&255;
    $429 = ($428|0)!=(0);
    if ($429) {
     $430 = HEAP32[$pPk>>2]|0;
     $431 = HEAP32[$pIdx>>2]|0;
     $432 = ($430|0)==($431|0);
     if ($432) {
      $433 = HEAP8[$7>>0]|0;
      $434 = $433&255;
      $435 = ($434|0)==(0);
      if ($435) {
       $436 = HEAP32[$v>>2]|0;
       $437 = HEAP32[$addrUniqueOk>>2]|0;
       _sqlite3VdbeResolveLabel($436,$437);
       break;
      }
     }
    }
    $438 = HEAP32[$pIdx>>2]|0;
    $439 = ((($438)) + 54|0);
    $440 = HEAP8[$439>>0]|0;
    $441 = $440&255;
    HEAP32[$onError>>2] = $441;
    $442 = HEAP32[$onError>>2]|0;
    $443 = ($442|0)==(0);
    if ($443) {
     $444 = HEAP32[$0>>2]|0;
     $445 = HEAP32[$regIdx>>2]|0;
     $446 = HEAP32[$pIdx>>2]|0;
     $447 = ((($446)) + 52|0);
     $448 = HEAP16[$447>>1]|0;
     $449 = $448&65535;
     _sqlite3ReleaseTempRange($444,$445,$449);
     $450 = HEAP32[$v>>2]|0;
     $451 = HEAP32[$addrUniqueOk>>2]|0;
     _sqlite3VdbeResolveLabel($450,$451);
     break;
    }
    $452 = HEAP8[$8>>0]|0;
    $453 = $452&255;
    $454 = ($453|0)!=(10);
    if ($454) {
     $455 = HEAP8[$8>>0]|0;
     $456 = $455&255;
     HEAP32[$onError>>2] = $456;
    } else {
     $457 = HEAP32[$onError>>2]|0;
     $458 = ($457|0)==(10);
     if ($458) {
      HEAP32[$onError>>2] = 2;
     }
    }
    $459 = HEAP32[$v>>2]|0;
    $460 = HEAP32[$iThisCur>>2]|0;
    $461 = HEAP32[$addrUniqueOk>>2]|0;
    $462 = HEAP32[$regIdx>>2]|0;
    $463 = HEAP32[$pIdx>>2]|0;
    $464 = ((($463)) + 50|0);
    $465 = HEAP16[$464>>1]|0;
    $466 = $465&65535;
    (_sqlite3VdbeAddOp4Int($459,29,$460,$461,$462,$466)|0);
    $467 = HEAP32[$pIdx>>2]|0;
    $468 = HEAP32[$pPk>>2]|0;
    $469 = ($467|0)==($468|0);
    if ($469) {
     $470 = HEAP32[$regIdx>>2]|0;
     $474 = $470;
    } else {
     $471 = HEAP32[$0>>2]|0;
     $472 = HEAP32[$nPkField>>2]|0;
     $473 = (_sqlite3GetTempRange($471,$472)|0);
     $474 = $473;
    }
    HEAP32[$regR>>2] = $474;
    $475 = HEAP8[$isUpdate>>0]|0;
    $476 = $475&255;
    $477 = ($476|0)!=(0);
    $478 = HEAP32[$onError>>2]|0;
    $479 = ($478|0)==(5);
    $or$cond3 = $477 | $479;
    L137: do {
     if ($or$cond3) {
      $480 = HEAP32[$1>>2]|0;
      $481 = ((($480)) + 42|0);
      $482 = HEAP8[$481>>0]|0;
      $483 = $482&255;
      $484 = $483 & 32;
      $485 = ($484|0)==(0);
      if ($485) {
       $486 = HEAP32[$v>>2]|0;
       $487 = HEAP32[$iThisCur>>2]|0;
       $488 = HEAP32[$regR>>2]|0;
       (_sqlite3VdbeAddOp2($486,129,$487,$488)|0);
       $489 = HEAP8[$isUpdate>>0]|0;
       $490 = ($489<<24>>24)!=(0);
       if (!($490)) {
        break;
       }
       $491 = HEAP32[$v>>2]|0;
       $492 = HEAP32[$regR>>2]|0;
       $493 = HEAP32[$addrUniqueOk>>2]|0;
       $494 = HEAP32[$6>>2]|0;
       (_sqlite3VdbeAddOp3($491,37,$492,$493,$494)|0);
       $495 = HEAP32[$v>>2]|0;
       _sqlite3VdbeChangeP5($495,-112);
       break;
      }
      $496 = HEAP32[$pIdx>>2]|0;
      $497 = HEAP32[$pPk>>2]|0;
      $498 = ($496|0)!=($497|0);
      L143: do {
       if ($498) {
        HEAP32[$i>>2] = 0;
        while(1) {
         $499 = HEAP32[$i>>2]|0;
         $500 = HEAP32[$pPk>>2]|0;
         $501 = ((($500)) + 50|0);
         $502 = HEAP16[$501>>1]|0;
         $503 = $502&65535;
         $504 = ($499|0)<($503|0);
         if (!($504)) {
          break L143;
         }
         $505 = HEAP32[$pIdx>>2]|0;
         $506 = HEAP32[$i>>2]|0;
         $507 = HEAP32[$pPk>>2]|0;
         $508 = ((($507)) + 4|0);
         $509 = HEAP32[$508>>2]|0;
         $510 = (($509) + ($506<<1)|0);
         $511 = HEAP16[$510>>1]|0;
         $512 = (_sqlite3ColumnOfIndex($505,$511)|0);
         $513 = $512 << 16 >> 16;
         HEAP32[$x1>>2] = $513;
         $514 = HEAP32[$v>>2]|0;
         $515 = HEAP32[$iThisCur>>2]|0;
         $516 = HEAP32[$x1>>2]|0;
         $517 = HEAP32[$regR>>2]|0;
         $518 = HEAP32[$i>>2]|0;
         $519 = (($517) + ($518))|0;
         (_sqlite3VdbeAddOp3($514,96,$515,$516,$519)|0);
         $520 = HEAP32[$i>>2]|0;
         $521 = (($520) + 1)|0;
         HEAP32[$i>>2] = $521;
        }
       }
      } while(0);
      $522 = HEAP8[$isUpdate>>0]|0;
      $523 = ($522<<24>>24)!=(0);
      if ($523) {
       $524 = HEAP32[$v>>2]|0;
       $525 = (_sqlite3VdbeCurrentAddr($524)|0);
       $526 = HEAP32[$pPk>>2]|0;
       $527 = ((($526)) + 50|0);
       $528 = HEAP16[$527>>1]|0;
       $529 = $528&65535;
       $530 = (($525) + ($529))|0;
       HEAP32[$addrJump>>2] = $530;
       HEAP32[$op>>2] = 36;
       $531 = HEAP32[$pIdx>>2]|0;
       $532 = ((($531)) + 55|0);
       $533 = HEAP8[$532>>0]|0;
       $534 = $533 & 3;
       $535 = $534&255;
       $536 = ($535|0)==(2);
       $537 = HEAP32[$regIdx>>2]|0;
       $538 = HEAP32[$regR>>2]|0;
       $539 = $536 ? $537 : $538;
       HEAP32[$regCmp>>2] = $539;
       HEAP32[$i>>2] = 0;
       while(1) {
        $540 = HEAP32[$i>>2]|0;
        $541 = HEAP32[$pPk>>2]|0;
        $542 = ((($541)) + 50|0);
        $543 = HEAP16[$542>>1]|0;
        $544 = $543&65535;
        $545 = ($540|0)<($544|0);
        if (!($545)) {
         break L137;
        }
        $546 = HEAP32[$0>>2]|0;
        $547 = HEAP32[$i>>2]|0;
        $548 = HEAP32[$pPk>>2]|0;
        $549 = ((($548)) + 32|0);
        $550 = HEAP32[$549>>2]|0;
        $551 = (($550) + ($547<<2)|0);
        $552 = HEAP32[$551>>2]|0;
        $553 = (_sqlite3LocateCollSeq($546,$552)|0);
        HEAP32[$p4>>2] = $553;
        $554 = HEAP32[$i>>2]|0;
        $555 = HEAP32[$pPk>>2]|0;
        $556 = ((($555)) + 4|0);
        $557 = HEAP32[$556>>2]|0;
        $558 = (($557) + ($554<<1)|0);
        $559 = HEAP16[$558>>1]|0;
        $560 = $559 << 16 >> 16;
        HEAP32[$x1>>2] = $560;
        $561 = HEAP32[$i>>2]|0;
        $562 = HEAP32[$pPk>>2]|0;
        $563 = ((($562)) + 50|0);
        $564 = HEAP16[$563>>1]|0;
        $565 = $564&65535;
        $566 = (($565) - 1)|0;
        $567 = ($561|0)==($566|0);
        if ($567) {
         $568 = HEAP32[$addrUniqueOk>>2]|0;
         HEAP32[$addrJump>>2] = $568;
         HEAP32[$op>>2] = 37;
        }
        $569 = HEAP32[$v>>2]|0;
        $570 = HEAP32[$op>>2]|0;
        $571 = HEAP32[$6>>2]|0;
        $572 = (($571) + 1)|0;
        $573 = HEAP32[$x1>>2]|0;
        $574 = (($572) + ($573))|0;
        $575 = HEAP32[$addrJump>>2]|0;
        $576 = HEAP32[$regCmp>>2]|0;
        $577 = HEAP32[$i>>2]|0;
        $578 = (($576) + ($577))|0;
        $579 = HEAP32[$p4>>2]|0;
        (_sqlite3VdbeAddOp4($569,$570,$574,$575,$578,$579,-4)|0);
        $580 = HEAP32[$v>>2]|0;
        _sqlite3VdbeChangeP5($580,-112);
        $581 = HEAP32[$i>>2]|0;
        $582 = (($581) + 1)|0;
        HEAP32[$i>>2] = $582;
       }
      }
     }
    } while(0);
    $583 = HEAP32[$onError>>2]|0;
    switch ($583|0) {
    case 3: case 2: case 1:  {
     $584 = HEAP32[$0>>2]|0;
     $585 = HEAP32[$onError>>2]|0;
     $586 = HEAP32[$pIdx>>2]|0;
     _sqlite3UniqueConstraint($584,$585,$586);
     break;
    }
    case 4:  {
     $587 = HEAP32[$v>>2]|0;
     $588 = HEAP32[$9>>2]|0;
     (_sqlite3VdbeGoto($587,$588)|0);
     break;
    }
    default: {
     HEAP32[$pTrigger2>>2] = 0;
     $589 = HEAP32[$0>>2]|0;
     _sqlite3MultiWrite($589);
     $590 = HEAP32[$db>>2]|0;
     $591 = ((($590)) + 24|0);
     $592 = HEAP32[$591>>2]|0;
     $593 = $592 & 262144;
     $594 = ($593|0)!=(0);
     if ($594) {
      $595 = HEAP32[$0>>2]|0;
      $596 = HEAP32[$1>>2]|0;
      $597 = (_sqlite3TriggersExist($595,$596,109,0,0)|0);
      HEAP32[$pTrigger2>>2] = $597;
     }
     $598 = HEAP32[$0>>2]|0;
     $599 = HEAP32[$1>>2]|0;
     $600 = HEAP32[$pTrigger2>>2]|0;
     $601 = HEAP32[$3>>2]|0;
     $602 = HEAP32[$4>>2]|0;
     $603 = HEAP32[$regR>>2]|0;
     $604 = HEAP32[$nPkField>>2]|0;
     $605 = $604&65535;
     $606 = HEAP32[$pIdx>>2]|0;
     $607 = HEAP32[$pPk>>2]|0;
     $608 = ($606|0)==($607|0);
     $609 = $608 ? 1 : 0;
     $610 = $609&255;
     _sqlite3GenerateRowDelete($598,$599,$600,$601,$602,$603,$605,0,5,$610,-1);
     HEAP32[$seenReplace>>2] = 1;
    }
    }
    $611 = HEAP32[$v>>2]|0;
    $612 = HEAP32[$addrUniqueOk>>2]|0;
    _sqlite3VdbeResolveLabel($611,$612);
    $613 = HEAP32[$0>>2]|0;
    $614 = HEAP32[$regIdx>>2]|0;
    $615 = HEAP32[$pIdx>>2]|0;
    $616 = ((($615)) + 52|0);
    $617 = HEAP16[$616>>1]|0;
    $618 = $617&65535;
    _sqlite3ReleaseTempRange($613,$614,$618);
    $619 = HEAP32[$regR>>2]|0;
    $620 = HEAP32[$regIdx>>2]|0;
    $621 = ($619|0)!=($620|0);
    if ($621) {
     $622 = HEAP32[$0>>2]|0;
     $623 = HEAP32[$regR>>2]|0;
     $624 = HEAP32[$nPkField>>2]|0;
     _sqlite3ReleaseTempRange($622,$623,$624);
    }
   }
  } while(0);
  $625 = HEAP32[$pIdx>>2]|0;
  $626 = ((($625)) + 20|0);
  $627 = HEAP32[$626>>2]|0;
  HEAP32[$pIdx>>2] = $627;
  $628 = HEAP32[$ix>>2]|0;
  $629 = (($628) + 1)|0;
  HEAP32[$ix>>2] = $629;
 }
 $630 = HEAP32[$ipkTop>>2]|0;
 $631 = ($630|0)!=(0);
 if (!($631)) {
  $637 = HEAP32[$seenReplace>>2]|0;
  $638 = HEAP32[$10>>2]|0;
  HEAP32[$638>>2] = $637;
  STACKTOP = sp;return;
 }
 $632 = HEAP32[$v>>2]|0;
 $633 = HEAP32[$ipkTop>>2]|0;
 $634 = (($633) + 1)|0;
 (_sqlite3VdbeGoto($632,$634)|0);
 $635 = HEAP32[$v>>2]|0;
 $636 = HEAP32[$ipkBottom>>2]|0;
 _sqlite3VdbeJumpHere($635,$636);
 $637 = HEAP32[$seenReplace>>2]|0;
 $638 = HEAP32[$10>>2]|0;
 HEAP32[$638>>2] = $637;
 STACKTOP = sp;return;
}
function _sqlite3FkCheck($pParse,$pTab,$regOld,$regNew,$aChange,$bChngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $regOld = $regOld|0;
 $regNew = $regNew|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aiCol = 0, $aiCol2 = 0, $aiFree = 0, $bIgnore = 0, $db = 0, $eAction = 0, $i = 0, $iCol = 0, $iDb = 0, $iJump = 0, $iReg = 0, $isIgnoreErrors = 0, $or$cond = 0, $or$cond3 = 0, $pFKey = 0, $pIdx = 0, $pIdx1 = 0, $pItem = 0, $pSrc = 0, $pTo = 0;
 var $rcauth = 0, $v = 0, $zCol = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 108|0;
 $1 = sp + 104|0;
 $2 = sp + 100|0;
 $3 = sp + 96|0;
 $4 = sp + 92|0;
 $5 = sp + 88|0;
 $db = sp + 84|0;
 $pFKey = sp + 80|0;
 $iDb = sp + 76|0;
 $zDb = sp + 72|0;
 $isIgnoreErrors = sp + 68|0;
 $pTo = sp + 64|0;
 $pIdx = sp + 60|0;
 $aiFree = sp + 56|0;
 $aiCol = sp + 52|0;
 $iCol = sp + 48|0;
 $i = sp + 44|0;
 $bIgnore = sp + 40|0;
 $v = sp + 36|0;
 $iJump = sp + 32|0;
 $iReg = sp + 28|0;
 $rcauth = sp + 24|0;
 $zCol = sp + 20|0;
 $pIdx1 = sp + 16|0;
 $pSrc = sp + 12|0;
 $aiCol2 = sp + 8|0;
 $pItem = sp + 4|0;
 $eAction = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $regOld;
 HEAP32[$3>>2] = $regNew;
 HEAP32[$4>>2] = $aChange;
 HEAP32[$5>>2] = $bChngRowid;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 150|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$isIgnoreErrors>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = ((($12)) + 24|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 & 524288;
 $16 = ($15|0)==(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 64|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3SchemaToIndex($17,$20)|0);
 HEAP32[$iDb>>2] = $21;
 $22 = HEAP32[$iDb>>2]|0;
 $23 = HEAP32[$db>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<4)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$zDb>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$pFKey>>2] = $30;
 L4: while(1) {
  $31 = HEAP32[$pFKey>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   label = 39;
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  HEAP32[$aiFree>>2] = 0;
  HEAP32[$bIgnore>>2] = 0;
  $33 = HEAP32[$4>>2]|0;
  $34 = ($33|0)!=(0|0);
  if ($34) {
   $35 = HEAP32[$1>>2]|0;
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pFKey>>2]|0;
   $38 = ((($37)) + 8|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = (_sqlite3_stricmp($36,$39)|0);
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP32[$pFKey>>2]|0;
    $44 = HEAP32[$4>>2]|0;
    $45 = HEAP32[$5>>2]|0;
    $46 = (_fkChildIsModified($42,$43,$44,$45)|0);
    $47 = ($46|0)==(0);
    if (!($47)) {
     label = 7;
    }
   } else {
    label = 7;
   }
  } else {
   label = 7;
  }
  do {
   if ((label|0) == 7) {
    label = 0;
    $48 = HEAP32[$0>>2]|0;
    $49 = ((($48)) + 150|0);
    $50 = HEAP8[$49>>0]|0;
    $51 = ($50<<24>>24)!=(0);
    if ($51) {
     $52 = HEAP32[$db>>2]|0;
     $53 = HEAP32[$pFKey>>2]|0;
     $54 = ((($53)) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = HEAP32[$zDb>>2]|0;
     $57 = (_sqlite3FindTable($52,$55,$56)|0);
     HEAP32[$pTo>>2] = $57;
    } else {
     $58 = HEAP32[$0>>2]|0;
     $59 = HEAP32[$pFKey>>2]|0;
     $60 = ((($59)) + 8|0);
     $61 = HEAP32[$60>>2]|0;
     $62 = HEAP32[$zDb>>2]|0;
     $63 = (_sqlite3LocateTable($58,0,$61,$62)|0);
     HEAP32[$pTo>>2] = $63;
    }
    $64 = HEAP32[$pTo>>2]|0;
    $65 = ($64|0)!=(0|0);
    if ($65) {
     $66 = HEAP32[$0>>2]|0;
     $67 = HEAP32[$pTo>>2]|0;
     $68 = HEAP32[$pFKey>>2]|0;
     $69 = (_sqlite3FkLocateIndex($66,$67,$68,$pIdx,$aiFree)|0);
     $70 = ($69|0)!=(0);
     if (!($70)) {
      $112 = HEAP32[$aiFree>>2]|0;
      $113 = ($112|0)!=(0|0);
      if ($113) {
       $114 = HEAP32[$aiFree>>2]|0;
       HEAP32[$aiCol>>2] = $114;
      } else {
       $115 = HEAP32[$pFKey>>2]|0;
       $116 = ((($115)) + 36|0);
       $117 = HEAP32[$116>>2]|0;
       HEAP32[$iCol>>2] = $117;
       HEAP32[$aiCol>>2] = $iCol;
      }
      HEAP32[$i>>2] = 0;
      while(1) {
       $118 = HEAP32[$i>>2]|0;
       $119 = HEAP32[$pFKey>>2]|0;
       $120 = ((($119)) + 20|0);
       $121 = HEAP32[$120>>2]|0;
       $122 = ($118|0)<($121|0);
       if (!($122)) {
        break;
       }
       $123 = HEAP32[$i>>2]|0;
       $124 = HEAP32[$aiCol>>2]|0;
       $125 = (($124) + ($123<<2)|0);
       $126 = HEAP32[$125>>2]|0;
       $127 = HEAP32[$1>>2]|0;
       $128 = ((($127)) + 32|0);
       $129 = HEAP16[$128>>1]|0;
       $130 = $129 << 16 >> 16;
       $131 = ($126|0)==($130|0);
       if ($131) {
        $132 = HEAP32[$i>>2]|0;
        $133 = HEAP32[$aiCol>>2]|0;
        $134 = (($133) + ($132<<2)|0);
        HEAP32[$134>>2] = -1;
       }
       $135 = HEAP32[$db>>2]|0;
       $136 = ((($135)) + 296|0);
       $137 = HEAP32[$136>>2]|0;
       $138 = ($137|0)!=(0|0);
       if ($138) {
        $139 = HEAP32[$pIdx>>2]|0;
        $140 = ($139|0)!=(0|0);
        if ($140) {
         $141 = HEAP32[$i>>2]|0;
         $142 = HEAP32[$pIdx>>2]|0;
         $143 = ((($142)) + 4|0);
         $144 = HEAP32[$143>>2]|0;
         $145 = (($144) + ($141<<1)|0);
         $146 = HEAP16[$145>>1]|0;
         $$sink = $146;
        } else {
         $147 = HEAP32[$pTo>>2]|0;
         $148 = ((($147)) + 32|0);
         $149 = HEAP16[$148>>1]|0;
         $$sink = $149;
        }
        $150 = $$sink << 16 >> 16;
        $151 = HEAP32[$pTo>>2]|0;
        $152 = ((($151)) + 4|0);
        $153 = HEAP32[$152>>2]|0;
        $154 = (($153) + ($150<<4)|0);
        $155 = HEAP32[$154>>2]|0;
        HEAP32[$zCol>>2] = $155;
        $156 = HEAP32[$0>>2]|0;
        $157 = HEAP32[$pTo>>2]|0;
        $158 = HEAP32[$157>>2]|0;
        $159 = HEAP32[$zCol>>2]|0;
        $160 = HEAP32[$iDb>>2]|0;
        $161 = (_sqlite3AuthReadCol($156,$158,$159,$160)|0);
        HEAP32[$rcauth>>2] = $161;
        $162 = HEAP32[$rcauth>>2]|0;
        $163 = ($162|0)==(2);
        $164 = $163&1;
        HEAP32[$bIgnore>>2] = $164;
       }
       $165 = HEAP32[$i>>2]|0;
       $166 = (($165) + 1)|0;
       HEAP32[$i>>2] = $166;
      }
      $167 = HEAP32[$0>>2]|0;
      $168 = HEAP32[$iDb>>2]|0;
      $169 = HEAP32[$pTo>>2]|0;
      $170 = ((($169)) + 28|0);
      $171 = HEAP32[$170>>2]|0;
      $172 = HEAP32[$pTo>>2]|0;
      $173 = HEAP32[$172>>2]|0;
      _sqlite3TableLock($167,$168,$171,0,$173);
      $174 = HEAP32[$0>>2]|0;
      $175 = ((($174)) + 40|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = (($176) + 1)|0;
      HEAP32[$175>>2] = $177;
      $178 = HEAP32[$2>>2]|0;
      $179 = ($178|0)!=(0);
      if ($179) {
       $180 = HEAP32[$0>>2]|0;
       $181 = HEAP32[$iDb>>2]|0;
       $182 = HEAP32[$pTo>>2]|0;
       $183 = HEAP32[$pIdx>>2]|0;
       $184 = HEAP32[$pFKey>>2]|0;
       $185 = HEAP32[$aiCol>>2]|0;
       $186 = HEAP32[$2>>2]|0;
       $187 = HEAP32[$bIgnore>>2]|0;
       _fkLookupParent($180,$181,$182,$183,$184,$185,$186,-1,$187);
      }
      $188 = HEAP32[$3>>2]|0;
      $189 = ($188|0)!=(0);
      if ($189) {
       $190 = HEAP32[$0>>2]|0;
       $191 = HEAP32[$pFKey>>2]|0;
       $192 = (_isSetNullAction($190,$191)|0);
       $193 = ($192|0)!=(0);
       if (!($193)) {
        $194 = HEAP32[$0>>2]|0;
        $195 = HEAP32[$iDb>>2]|0;
        $196 = HEAP32[$pTo>>2]|0;
        $197 = HEAP32[$pIdx>>2]|0;
        $198 = HEAP32[$pFKey>>2]|0;
        $199 = HEAP32[$aiCol>>2]|0;
        $200 = HEAP32[$3>>2]|0;
        $201 = HEAP32[$bIgnore>>2]|0;
        _fkLookupParent($194,$195,$196,$197,$198,$199,$200,1,$201);
       }
      }
      $202 = HEAP32[$db>>2]|0;
      $203 = HEAP32[$aiFree>>2]|0;
      _sqlite3DbFree($202,$203);
      break;
     }
    }
    $71 = HEAP32[$isIgnoreErrors>>2]|0;
    $72 = ($71|0)!=(0);
    if (!($72)) {
     label = 59;
     break L4;
    }
    $73 = HEAP32[$db>>2]|0;
    $74 = ((($73)) + 69|0);
    $75 = HEAP8[$74>>0]|0;
    $76 = $75&255;
    $77 = ($76|0)!=(0);
    if ($77) {
     label = 59;
     break L4;
    }
    $78 = HEAP32[$pTo>>2]|0;
    $79 = ($78|0)==(0|0);
    if ($79) {
     $80 = HEAP32[$0>>2]|0;
     $81 = (_sqlite3GetVdbe($80)|0);
     HEAP32[$v>>2] = $81;
     $82 = HEAP32[$v>>2]|0;
     $83 = (_sqlite3VdbeCurrentAddr($82)|0);
     $84 = HEAP32[$pFKey>>2]|0;
     $85 = ((($84)) + 20|0);
     $86 = HEAP32[$85>>2]|0;
     $87 = (($83) + ($86))|0;
     $88 = (($87) + 1)|0;
     HEAP32[$iJump>>2] = $88;
     HEAP32[$i>>2] = 0;
     while(1) {
      $89 = HEAP32[$i>>2]|0;
      $90 = HEAP32[$pFKey>>2]|0;
      $91 = ((($90)) + 20|0);
      $92 = HEAP32[$91>>2]|0;
      $93 = ($89|0)<($92|0);
      if (!($93)) {
       break;
      }
      $94 = HEAP32[$i>>2]|0;
      $95 = HEAP32[$pFKey>>2]|0;
      $96 = ((($95)) + 36|0);
      $97 = (($96) + ($94<<3)|0);
      $98 = HEAP32[$97>>2]|0;
      $99 = HEAP32[$2>>2]|0;
      $100 = (($98) + ($99))|0;
      $101 = (($100) + 1)|0;
      HEAP32[$iReg>>2] = $101;
      $102 = HEAP32[$v>>2]|0;
      $103 = HEAP32[$iReg>>2]|0;
      $104 = HEAP32[$iJump>>2]|0;
      (_sqlite3VdbeAddOp2($102,34,$103,$104)|0);
      $105 = HEAP32[$i>>2]|0;
      $106 = (($105) + 1)|0;
      HEAP32[$i>>2] = $106;
     }
     $107 = HEAP32[$v>>2]|0;
     $108 = HEAP32[$pFKey>>2]|0;
     $109 = ((($108)) + 24|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     (_sqlite3VdbeAddOp2($107,144,$111,-1)|0);
    }
   }
  } while(0);
  $204 = HEAP32[$pFKey>>2]|0;
  $205 = ((($204)) + 4|0);
  $206 = HEAP32[$205>>2]|0;
  HEAP32[$pFKey>>2] = $206;
 }
 if ((label|0) == 39) {
  $207 = HEAP32[$1>>2]|0;
  $208 = (_sqlite3FkReferences($207)|0);
  HEAP32[$pFKey>>2] = $208;
  L55: while(1) {
   $209 = HEAP32[$pFKey>>2]|0;
   $210 = ($209|0)!=(0|0);
   if (!($210)) {
    label = 59;
    break;
   }
   HEAP32[$pIdx1>>2] = 0;
   HEAP32[$aiCol2>>2] = 0;
   $211 = HEAP32[$4>>2]|0;
   $212 = ($211|0)!=(0|0);
   if ($212) {
    $213 = HEAP32[$1>>2]|0;
    $214 = HEAP32[$pFKey>>2]|0;
    $215 = HEAP32[$4>>2]|0;
    $216 = HEAP32[$5>>2]|0;
    $217 = (_fkParentIsModified($213,$214,$215,$216)|0);
    $218 = ($217|0)==(0);
    if (!($218)) {
     label = 43;
    }
   } else {
    label = 43;
   }
   do {
    if ((label|0) == 43) {
     label = 0;
     $219 = HEAP32[$pFKey>>2]|0;
     $220 = ((($219)) + 24|0);
     $221 = HEAP8[$220>>0]|0;
     $222 = ($221<<24>>24)!=(0);
     if (!($222)) {
      $223 = HEAP32[$db>>2]|0;
      $224 = ((($223)) + 24|0);
      $225 = HEAP32[$224>>2]|0;
      $226 = $225 & 33554432;
      $227 = ($226|0)!=(0);
      if (!($227)) {
       $228 = HEAP32[$0>>2]|0;
       $229 = ((($228)) + 124|0);
       $230 = HEAP32[$229>>2]|0;
       $231 = ($230|0)!=(0|0);
       if (!($231)) {
        $232 = HEAP32[$0>>2]|0;
        $233 = ((($232)) + 20|0);
        $234 = HEAP8[$233>>0]|0;
        $235 = ($234<<24>>24)!=(0);
        if (!($235)) {
         break;
        }
       }
      }
     }
     $236 = HEAP32[$0>>2]|0;
     $237 = HEAP32[$1>>2]|0;
     $238 = HEAP32[$pFKey>>2]|0;
     $239 = (_sqlite3FkLocateIndex($236,$237,$238,$pIdx1,$aiCol2)|0);
     $240 = ($239|0)!=(0);
     if ($240) {
      $241 = HEAP32[$isIgnoreErrors>>2]|0;
      $242 = ($241|0)!=(0);
      if (!($242)) {
       label = 59;
       break L55;
      }
      $243 = HEAP32[$db>>2]|0;
      $244 = ((($243)) + 69|0);
      $245 = HEAP8[$244>>0]|0;
      $246 = $245&255;
      $247 = ($246|0)!=(0);
      if ($247) {
       label = 59;
       break L55;
      } else {
       break;
      }
     }
     $248 = HEAP32[$db>>2]|0;
     $249 = (_sqlite3SrcListAppend($248,0,0,0)|0);
     HEAP32[$pSrc>>2] = $249;
     $250 = HEAP32[$pSrc>>2]|0;
     $251 = ($250|0)!=(0|0);
     if ($251) {
      $252 = HEAP32[$pSrc>>2]|0;
      $253 = ((($252)) + 8|0);
      HEAP32[$pItem>>2] = $253;
      $254 = HEAP32[$pFKey>>2]|0;
      $255 = HEAP32[$254>>2]|0;
      $256 = HEAP32[$pItem>>2]|0;
      $257 = ((($256)) + 16|0);
      HEAP32[$257>>2] = $255;
      $258 = HEAP32[$pFKey>>2]|0;
      $259 = HEAP32[$258>>2]|0;
      $260 = HEAP32[$259>>2]|0;
      $261 = HEAP32[$pItem>>2]|0;
      $262 = ((($261)) + 8|0);
      HEAP32[$262>>2] = $260;
      $263 = HEAP32[$pItem>>2]|0;
      $264 = ((($263)) + 16|0);
      $265 = HEAP32[$264>>2]|0;
      $266 = ((($265)) + 36|0);
      $267 = HEAP16[$266>>1]|0;
      $268 = (($267) + 1)<<16>>16;
      HEAP16[$266>>1] = $268;
      $269 = HEAP32[$0>>2]|0;
      $270 = ((($269)) + 40|0);
      $271 = HEAP32[$270>>2]|0;
      $272 = (($271) + 1)|0;
      HEAP32[$270>>2] = $272;
      $273 = HEAP32[$pItem>>2]|0;
      $274 = ((($273)) + 44|0);
      HEAP32[$274>>2] = $271;
      $275 = HEAP32[$3>>2]|0;
      $276 = ($275|0)!=(0);
      if ($276) {
       $277 = HEAP32[$0>>2]|0;
       $278 = HEAP32[$pSrc>>2]|0;
       $279 = HEAP32[$1>>2]|0;
       $280 = HEAP32[$pIdx1>>2]|0;
       $281 = HEAP32[$pFKey>>2]|0;
       $282 = HEAP32[$aiCol2>>2]|0;
       $283 = HEAP32[$3>>2]|0;
       _fkScanChildren($277,$278,$279,$280,$281,$282,$283,-1);
      }
      $284 = HEAP32[$2>>2]|0;
      $285 = ($284|0)!=(0);
      if ($285) {
       $286 = HEAP32[$4>>2]|0;
       $287 = ($286|0)!=(0|0);
       $288 = $287&1;
       $289 = HEAP32[$pFKey>>2]|0;
       $290 = ((($289)) + 25|0);
       $291 = (($290) + ($288)|0);
       $292 = HEAP8[$291>>0]|0;
       $293 = $292&255;
       HEAP32[$eAction>>2] = $293;
       $294 = HEAP32[$0>>2]|0;
       $295 = HEAP32[$pSrc>>2]|0;
       $296 = HEAP32[$1>>2]|0;
       $297 = HEAP32[$pIdx1>>2]|0;
       $298 = HEAP32[$pFKey>>2]|0;
       $299 = HEAP32[$aiCol2>>2]|0;
       $300 = HEAP32[$2>>2]|0;
       _fkScanChildren($294,$295,$296,$297,$298,$299,$300,1);
       $301 = HEAP32[$pFKey>>2]|0;
       $302 = ((($301)) + 24|0);
       $303 = HEAP8[$302>>0]|0;
       $304 = ($303<<24>>24)==(0);
       $305 = HEAP32[$eAction>>2]|0;
       $306 = ($305|0)!=(9);
       $or$cond = $304 & $306;
       $307 = HEAP32[$eAction>>2]|0;
       $308 = ($307|0)!=(7);
       $or$cond3 = $or$cond & $308;
       if ($or$cond3) {
        $309 = HEAP32[$0>>2]|0;
        _sqlite3MayAbort($309);
       }
      }
      $310 = HEAP32[$pItem>>2]|0;
      $311 = ((($310)) + 8|0);
      HEAP32[$311>>2] = 0;
      $312 = HEAP32[$db>>2]|0;
      $313 = HEAP32[$pSrc>>2]|0;
      _sqlite3SrcListDelete($312,$313);
     }
     $314 = HEAP32[$db>>2]|0;
     $315 = HEAP32[$aiCol2>>2]|0;
     _sqlite3DbFree($314,$315);
    }
   } while(0);
   $316 = HEAP32[$pFKey>>2]|0;
   $317 = ((($316)) + 12|0);
   $318 = HEAP32[$317>>2]|0;
   HEAP32[$pFKey>>2] = $318;
  }
  if ((label|0) == 59) {
   STACKTOP = sp;return;
  }
 }
 else if ((label|0) == 59) {
  STACKTOP = sp;return;
 }
}
function _sqlite3CompleteInsertion($pParse,$pTab,$iDataCur,$iIdxCur,$regNewData,$aRegIdx,$isUpdate,$appendBias,$useSeekResult) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $regNewData = $regNewData|0;
 $aRegIdx = $aRegIdx|0;
 $isUpdate = $isUpdate|0;
 $appendBias = $appendBias|0;
 $useSeekResult = $useSeekResult|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bAffinityDone = 0, $i = 0, $pIdx = 0, $pik_flags = 0, $regData = 0, $regRec = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $v = sp + 16|0;
 $pIdx = sp + 12|0;
 $pik_flags = sp + 57|0;
 $regData = sp + 8|0;
 $regRec = sp + 4|0;
 $i = sp;
 $bAffinityDone = sp + 56|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $iIdxCur;
 HEAP32[$4>>2] = $regNewData;
 HEAP32[$5>>2] = $aRegIdx;
 HEAP32[$6>>2] = $isUpdate;
 HEAP32[$7>>2] = $appendBias;
 HEAP32[$8>>2] = $useSeekResult;
 HEAP8[$bAffinityDone>>0] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 HEAP32[$i>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pIdx>>2] = $13;
 while(1) {
  $14 = HEAP32[$pIdx>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$5>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)==(0);
  if (!($20)) {
   HEAP8[$bAffinityDone>>0] = 1;
   $21 = HEAP32[$pIdx>>2]|0;
   $22 = ((($21)) + 36|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = ($23|0)!=(0|0);
   if ($24) {
    $25 = HEAP32[$v>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    $27 = HEAP32[$5>>2]|0;
    $28 = (($27) + ($26<<2)|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$v>>2]|0;
    $31 = (_sqlite3VdbeCurrentAddr($30)|0);
    $32 = (($31) + 2)|0;
    (_sqlite3VdbeAddOp2($25,34,$29,$32)|0);
   }
   $33 = HEAP32[$v>>2]|0;
   $34 = HEAP32[$3>>2]|0;
   $35 = HEAP32[$i>>2]|0;
   $36 = (($34) + ($35))|0;
   $37 = HEAP32[$i>>2]|0;
   $38 = HEAP32[$5>>2]|0;
   $39 = (($38) + ($37<<2)|0);
   $40 = HEAP32[$39>>2]|0;
   (_sqlite3VdbeAddOp2($33,126,$36,$40)|0);
   HEAP8[$pik_flags>>0] = 0;
   $41 = HEAP32[$8>>2]|0;
   $42 = ($41|0)!=(0);
   $$ = $42 ? 16 : 0;
   HEAP8[$pik_flags>>0] = $$;
   $43 = HEAP32[$pIdx>>2]|0;
   $44 = ((($43)) + 55|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = $45 & 3;
   $47 = $46&255;
   $48 = ($47|0)==(2);
   if ($48) {
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 42|0);
    $51 = HEAP8[$50>>0]|0;
    $52 = $51&255;
    $53 = $52 & 32;
    $54 = ($53|0)==(0);
    if (!($54)) {
     $55 = HEAP8[$pik_flags>>0]|0;
     $56 = $55&255;
     $57 = $56 | 1;
     $58 = $57&255;
     HEAP8[$pik_flags>>0] = $58;
    }
   }
   $59 = HEAP32[$v>>2]|0;
   $60 = HEAP8[$pik_flags>>0]|0;
   _sqlite3VdbeChangeP5($59,$60);
  }
  $61 = HEAP32[$pIdx>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  HEAP32[$pIdx>>2] = $63;
  $64 = HEAP32[$i>>2]|0;
  $65 = (($64) + 1)|0;
  HEAP32[$i>>2] = $65;
 }
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 42|0);
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = $69 & 32;
 $71 = ($70|0)==(0);
 if (!($71)) {
  STACKTOP = sp;return;
 }
 $72 = HEAP32[$4>>2]|0;
 $73 = (($72) + 1)|0;
 HEAP32[$regData>>2] = $73;
 $74 = HEAP32[$0>>2]|0;
 $75 = (_sqlite3GetTempReg($74)|0);
 HEAP32[$regRec>>2] = $75;
 $76 = HEAP32[$v>>2]|0;
 $77 = HEAP32[$regData>>2]|0;
 $78 = HEAP32[$1>>2]|0;
 $79 = ((($78)) + 34|0);
 $80 = HEAP16[$79>>1]|0;
 $81 = $80 << 16 >> 16;
 $82 = HEAP32[$regRec>>2]|0;
 (_sqlite3VdbeAddOp3($76,99,$77,$81,$82)|0);
 $83 = HEAP8[$bAffinityDone>>0]|0;
 $84 = ($83<<24>>24)!=(0);
 if (!($84)) {
  $85 = HEAP32[$v>>2]|0;
  $86 = HEAP32[$1>>2]|0;
  _sqlite3TableAffinity($85,$86,0);
 }
 $87 = HEAP32[$0>>2]|0;
 $88 = HEAP32[$regData>>2]|0;
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 34|0);
 $91 = HEAP16[$90>>1]|0;
 $92 = $91 << 16 >> 16;
 _sqlite3ExprCacheAffinityChange($87,$88,$92);
 $93 = HEAP32[$0>>2]|0;
 $94 = ((($93)) + 18|0);
 $95 = HEAP8[$94>>0]|0;
 $96 = ($95<<24>>24)!=(0);
 if ($96) {
  HEAP8[$pik_flags>>0] = 0;
 } else {
  HEAP8[$pik_flags>>0] = 1;
  $97 = HEAP32[$6>>2]|0;
  $98 = ($97|0)!=(0);
  $99 = $98 ? 4 : 2;
  $100 = HEAP8[$pik_flags>>0]|0;
  $101 = $100&255;
  $102 = $101 | $99;
  $103 = $102&255;
  HEAP8[$pik_flags>>0] = $103;
 }
 $104 = HEAP32[$7>>2]|0;
 $105 = ($104|0)!=(0);
 if ($105) {
  $106 = HEAP8[$pik_flags>>0]|0;
  $107 = $106&255;
  $108 = $107 | 8;
  $109 = $108&255;
  HEAP8[$pik_flags>>0] = $109;
 }
 $110 = HEAP32[$8>>2]|0;
 $111 = ($110|0)!=(0);
 if ($111) {
  $112 = HEAP8[$pik_flags>>0]|0;
  $113 = $112&255;
  $114 = $113 | 16;
  $115 = $114&255;
  HEAP8[$pik_flags>>0] = $115;
 }
 $116 = HEAP32[$v>>2]|0;
 $117 = HEAP32[$2>>2]|0;
 $118 = HEAP32[$regRec>>2]|0;
 $119 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeAddOp3($116,115,$117,$118,$119)|0);
 $120 = HEAP32[$0>>2]|0;
 $121 = ((($120)) + 18|0);
 $122 = HEAP8[$121>>0]|0;
 $123 = ($122<<24>>24)!=(0);
 if (!($123)) {
  $124 = HEAP32[$v>>2]|0;
  $125 = HEAP32[$1>>2]|0;
  _sqlite3VdbeChangeP4($124,-1,$125,-20);
 }
 $126 = HEAP32[$v>>2]|0;
 $127 = HEAP8[$pik_flags>>0]|0;
 _sqlite3VdbeChangeP5($126,$127);
 STACKTOP = sp;return;
}
function _sqlite3AutoincrementEnd($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 120|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 _autoIncrementEnd($5);
 STACKTOP = sp;return;
}
function _autoIncrementEnd($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $aOp = 0, $db = 0, $iRec = 0, $memId = 0, $p = 0, $pDb = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $p = sp + 24|0;
 $v = sp + 20|0;
 $db = sp + 16|0;
 $aOp = sp + 12|0;
 $pDb = sp + 8|0;
 $iRec = sp + 4|0;
 $memId = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 120|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$db>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  HEAP32[$pDb>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$memId>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  $22 = (_sqlite3GetTempReg($21)|0);
  HEAP32[$iRec>>2] = $22;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$pDb>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = ((($29)) + 72|0);
  $31 = HEAP32[$30>>2]|0;
  _sqlite3OpenTable($23,0,$26,$31,105);
  $32 = HEAP32[$v>>2]|0;
  $33 = (_sqlite3VdbeAddOpList($32,5,34384,0)|0);
  HEAP32[$aOp>>2] = $33;
  $34 = HEAP32[$aOp>>2]|0;
  $35 = ($34|0)==(0|0);
  if ($35) {
   label = 5;
   break;
  }
  $36 = HEAP32[$memId>>2]|0;
  $37 = (($36) + 1)|0;
  $38 = HEAP32[$aOp>>2]|0;
  $39 = ((($38)) + 4|0);
  HEAP32[$39>>2] = $37;
  $40 = HEAP32[$memId>>2]|0;
  $41 = (($40) + 1)|0;
  $42 = HEAP32[$aOp>>2]|0;
  $43 = ((($42)) + 20|0);
  $44 = ((($43)) + 8|0);
  HEAP32[$44>>2] = $41;
  $45 = HEAP32[$memId>>2]|0;
  $46 = (($45) - 1)|0;
  $47 = HEAP32[$aOp>>2]|0;
  $48 = ((($47)) + 40|0);
  $49 = ((($48)) + 4|0);
  HEAP32[$49>>2] = $46;
  $50 = HEAP32[$iRec>>2]|0;
  $51 = HEAP32[$aOp>>2]|0;
  $52 = ((($51)) + 40|0);
  $53 = ((($52)) + 12|0);
  HEAP32[$53>>2] = $50;
  $54 = HEAP32[$iRec>>2]|0;
  $55 = HEAP32[$aOp>>2]|0;
  $56 = ((($55)) + 60|0);
  $57 = ((($56)) + 8|0);
  HEAP32[$57>>2] = $54;
  $58 = HEAP32[$memId>>2]|0;
  $59 = (($58) + 1)|0;
  $60 = HEAP32[$aOp>>2]|0;
  $61 = ((($60)) + 60|0);
  $62 = ((($61)) + 12|0);
  HEAP32[$62>>2] = $59;
  $63 = HEAP32[$aOp>>2]|0;
  $64 = ((($63)) + 60|0);
  $65 = ((($64)) + 3|0);
  HEAP8[$65>>0] = 8;
  $66 = HEAP32[$0>>2]|0;
  $67 = HEAP32[$iRec>>2]|0;
  _sqlite3ReleaseTempReg($66,$67);
  $68 = HEAP32[$p>>2]|0;
  $69 = HEAP32[$68>>2]|0;
  HEAP32[$p>>2] = $69;
 }
 if ((label|0) == 5) {
  STACKTOP = sp;return;
 }
}
function _fkChildIsModified($pTab,$p,$aChange,$bChngRowid) {
 $pTab = $pTab|0;
 $p = $p|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iChildKey = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $i = sp + 4|0;
 $iChildKey = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $bChngRowid;
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 8;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 36|0);
  $13 = (($12) + ($10<<3)|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$iChildKey>>2] = $14;
  $15 = HEAP32[$iChildKey>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = ($18|0)>=(0);
  if ($19) {
   label = 4;
   break;
  }
  $20 = HEAP32[$iChildKey>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 32|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23 << 16 >> 16;
  $25 = ($20|0)==($24|0);
  $26 = HEAP32[$4>>2]|0;
  $27 = ($26|0)!=(0);
  $or$cond = $25 & $27;
  if ($or$cond) {
   label = 6;
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$i>>2] = $29;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _fkLookupParent($pParse,$iDb,$pTab,$pIdx,$pFKey,$aiCol,$regData,$nIncr,$isIgnore) {
 $pParse = $pParse|0;
 $iDb = $iDb|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 $pFKey = $pFKey|0;
 $aiCol = $aiCol|0;
 $regData = $regData|0;
 $nIncr = $nIncr|0;
 $isIgnore = $isIgnore|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iChild = 0, $iCur = 0, $iJump = 0, $iMustBeInt = 0, $iOk = 0, $iParent = 0, $iReg = 0, $nCol = 0, $or$cond = 0, $or$cond3 = 0, $regRec = 0, $regTemp = 0, $regTemp1 = 0;
 var $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $5 = sp + 64|0;
 $6 = sp + 60|0;
 $7 = sp + 56|0;
 $8 = sp + 52|0;
 $i = sp + 48|0;
 $v = sp + 44|0;
 $iCur = sp + 40|0;
 $iOk = sp + 36|0;
 $iReg = sp + 32|0;
 $iMustBeInt = sp + 28|0;
 $regTemp = sp + 24|0;
 $nCol = sp + 20|0;
 $regTemp1 = sp + 16|0;
 $regRec = sp + 12|0;
 $iJump = sp + 8|0;
 $iChild = sp + 4|0;
 $iParent = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pIdx;
 HEAP32[$4>>2] = $pFKey;
 HEAP32[$5>>2] = $aiCol;
 HEAP32[$6>>2] = $regData;
 HEAP32[$7>>2] = $nIncr;
 HEAP32[$8>>2] = $isIgnore;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3GetVdbe($9)|0);
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) - 1)|0;
 HEAP32[$iCur>>2] = $14;
 $15 = HEAP32[$v>>2]|0;
 $16 = (_sqlite3VdbeMakeLabel($15)|0);
 HEAP32[$iOk>>2] = $16;
 $17 = HEAP32[$7>>2]|0;
 $18 = ($17|0)<(0);
 if ($18) {
  $19 = HEAP32[$v>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = ((($20)) + 24|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = HEAP32[$iOk>>2]|0;
  (_sqlite3VdbeAddOp2($19,65,$23,$24)|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  $27 = ((($26)) + 20|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($25|0)<($28|0);
  if (!($29)) {
   break;
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$5>>2]|0;
  $32 = (($31) + ($30<<2)|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$6>>2]|0;
  $35 = (($33) + ($34))|0;
  $36 = (($35) + 1)|0;
  HEAP32[$iReg>>2] = $36;
  $37 = HEAP32[$v>>2]|0;
  $38 = HEAP32[$iReg>>2]|0;
  $39 = HEAP32[$iOk>>2]|0;
  (_sqlite3VdbeAddOp2($37,34,$38,$39)|0);
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = HEAP32[$8>>2]|0;
 $43 = ($42|0)==(0);
 do {
  if ($43) {
   $44 = HEAP32[$3>>2]|0;
   $45 = ($44|0)==(0|0);
   if ($45) {
    $46 = HEAP32[$0>>2]|0;
    $47 = (_sqlite3GetTempReg($46)|0);
    HEAP32[$regTemp>>2] = $47;
    $48 = HEAP32[$v>>2]|0;
    $49 = HEAP32[$5>>2]|0;
    $50 = HEAP32[$49>>2]|0;
    $51 = (($50) + 1)|0;
    $52 = HEAP32[$6>>2]|0;
    $53 = (($51) + ($52))|0;
    $54 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp2($48,85,$53,$54)|0);
    $55 = HEAP32[$v>>2]|0;
    $56 = HEAP32[$regTemp>>2]|0;
    $57 = (_sqlite3VdbeAddOp2($55,17,$56,0)|0);
    HEAP32[$iMustBeInt>>2] = $57;
    $58 = HEAP32[$2>>2]|0;
    $59 = HEAP32[$4>>2]|0;
    $60 = HEAP32[$59>>2]|0;
    $61 = ($58|0)==($60|0);
    $62 = HEAP32[$7>>2]|0;
    $63 = ($62|0)==(1);
    $or$cond = $61 & $63;
    if ($or$cond) {
     $64 = HEAP32[$v>>2]|0;
     $65 = HEAP32[$6>>2]|0;
     $66 = HEAP32[$iOk>>2]|0;
     $67 = HEAP32[$regTemp>>2]|0;
     (_sqlite3VdbeAddOp3($64,37,$65,$66,$67)|0);
     $68 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($68,-112);
    }
    $69 = HEAP32[$0>>2]|0;
    $70 = HEAP32[$iCur>>2]|0;
    $71 = HEAP32[$1>>2]|0;
    $72 = HEAP32[$2>>2]|0;
    _sqlite3OpenTable($69,$70,$71,$72,104);
    $73 = HEAP32[$v>>2]|0;
    $74 = HEAP32[$iCur>>2]|0;
    $75 = HEAP32[$regTemp>>2]|0;
    (_sqlite3VdbeAddOp3($73,33,$74,0,$75)|0);
    $76 = HEAP32[$v>>2]|0;
    $77 = HEAP32[$iOk>>2]|0;
    (_sqlite3VdbeGoto($76,$77)|0);
    $78 = HEAP32[$v>>2]|0;
    $79 = HEAP32[$v>>2]|0;
    $80 = (_sqlite3VdbeCurrentAddr($79)|0);
    $81 = (($80) - 2)|0;
    _sqlite3VdbeJumpHere($78,$81);
    $82 = HEAP32[$v>>2]|0;
    $83 = HEAP32[$iMustBeInt>>2]|0;
    _sqlite3VdbeJumpHere($82,$83);
    $84 = HEAP32[$0>>2]|0;
    $85 = HEAP32[$regTemp>>2]|0;
    _sqlite3ReleaseTempReg($84,$85);
    break;
   }
   $86 = HEAP32[$4>>2]|0;
   $87 = ((($86)) + 20|0);
   $88 = HEAP32[$87>>2]|0;
   HEAP32[$nCol>>2] = $88;
   $89 = HEAP32[$0>>2]|0;
   $90 = HEAP32[$nCol>>2]|0;
   $91 = (_sqlite3GetTempRange($89,$90)|0);
   HEAP32[$regTemp1>>2] = $91;
   $92 = HEAP32[$0>>2]|0;
   $93 = (_sqlite3GetTempReg($92)|0);
   HEAP32[$regRec>>2] = $93;
   $94 = HEAP32[$v>>2]|0;
   $95 = HEAP32[$iCur>>2]|0;
   $96 = HEAP32[$3>>2]|0;
   $97 = ((($96)) + 44|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = HEAP32[$1>>2]|0;
   (_sqlite3VdbeAddOp3($94,104,$95,$98,$99)|0);
   $100 = HEAP32[$0>>2]|0;
   $101 = HEAP32[$3>>2]|0;
   _sqlite3VdbeSetP4KeyInfo($100,$101);
   HEAP32[$i>>2] = 0;
   while(1) {
    $102 = HEAP32[$i>>2]|0;
    $103 = HEAP32[$nCol>>2]|0;
    $104 = ($102|0)<($103|0);
    if (!($104)) {
     break;
    }
    $105 = HEAP32[$v>>2]|0;
    $106 = HEAP32[$i>>2]|0;
    $107 = HEAP32[$5>>2]|0;
    $108 = (($107) + ($106<<2)|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + 1)|0;
    $111 = HEAP32[$6>>2]|0;
    $112 = (($110) + ($111))|0;
    $113 = HEAP32[$regTemp1>>2]|0;
    $114 = HEAP32[$i>>2]|0;
    $115 = (($113) + ($114))|0;
    (_sqlite3VdbeAddOp2($105,84,$112,$115)|0);
    $116 = HEAP32[$i>>2]|0;
    $117 = (($116) + 1)|0;
    HEAP32[$i>>2] = $117;
   }
   $118 = HEAP32[$2>>2]|0;
   $119 = HEAP32[$4>>2]|0;
   $120 = HEAP32[$119>>2]|0;
   $121 = ($118|0)==($120|0);
   $122 = HEAP32[$7>>2]|0;
   $123 = ($122|0)==(1);
   $or$cond3 = $121 & $123;
   if ($or$cond3) {
    $124 = HEAP32[$v>>2]|0;
    $125 = (_sqlite3VdbeCurrentAddr($124)|0);
    $126 = HEAP32[$nCol>>2]|0;
    $127 = (($125) + ($126))|0;
    $128 = (($127) + 1)|0;
    HEAP32[$iJump>>2] = $128;
    HEAP32[$i>>2] = 0;
    while(1) {
     $129 = HEAP32[$i>>2]|0;
     $130 = HEAP32[$nCol>>2]|0;
     $131 = ($129|0)<($130|0);
     if (!($131)) {
      break;
     }
     $132 = HEAP32[$i>>2]|0;
     $133 = HEAP32[$5>>2]|0;
     $134 = (($133) + ($132<<2)|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = (($135) + 1)|0;
     $137 = HEAP32[$6>>2]|0;
     $138 = (($136) + ($137))|0;
     HEAP32[$iChild>>2] = $138;
     $139 = HEAP32[$i>>2]|0;
     $140 = HEAP32[$3>>2]|0;
     $141 = ((($140)) + 4|0);
     $142 = HEAP32[$141>>2]|0;
     $143 = (($142) + ($139<<1)|0);
     $144 = HEAP16[$143>>1]|0;
     $145 = $144 << 16 >> 16;
     $146 = (($145) + 1)|0;
     $147 = HEAP32[$6>>2]|0;
     $148 = (($146) + ($147))|0;
     HEAP32[$iParent>>2] = $148;
     $149 = HEAP32[$i>>2]|0;
     $150 = HEAP32[$3>>2]|0;
     $151 = ((($150)) + 4|0);
     $152 = HEAP32[$151>>2]|0;
     $153 = (($152) + ($149<<1)|0);
     $154 = HEAP16[$153>>1]|0;
     $155 = $154 << 16 >> 16;
     $156 = HEAP32[$2>>2]|0;
     $157 = ((($156)) + 32|0);
     $158 = HEAP16[$157>>1]|0;
     $159 = $158 << 16 >> 16;
     $160 = ($155|0)==($159|0);
     if ($160) {
      $161 = HEAP32[$6>>2]|0;
      HEAP32[$iParent>>2] = $161;
     }
     $162 = HEAP32[$v>>2]|0;
     $163 = HEAP32[$iChild>>2]|0;
     $164 = HEAP32[$iJump>>2]|0;
     $165 = HEAP32[$iParent>>2]|0;
     (_sqlite3VdbeAddOp3($162,36,$163,$164,$165)|0);
     $166 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($166,16);
     $167 = HEAP32[$i>>2]|0;
     $168 = (($167) + 1)|0;
     HEAP32[$i>>2] = $168;
    }
    $169 = HEAP32[$v>>2]|0;
    $170 = HEAP32[$iOk>>2]|0;
    (_sqlite3VdbeGoto($169,$170)|0);
   }
   $171 = HEAP32[$v>>2]|0;
   $172 = HEAP32[$regTemp1>>2]|0;
   $173 = HEAP32[$nCol>>2]|0;
   $174 = HEAP32[$regRec>>2]|0;
   $175 = HEAP32[$0>>2]|0;
   $176 = HEAP32[$175>>2]|0;
   $177 = HEAP32[$3>>2]|0;
   $178 = (_sqlite3IndexAffinityStr($176,$177)|0);
   $179 = HEAP32[$nCol>>2]|0;
   (_sqlite3VdbeAddOp4($171,99,$172,$173,$174,$178,$179)|0);
   $180 = HEAP32[$v>>2]|0;
   $181 = HEAP32[$iCur>>2]|0;
   $182 = HEAP32[$iOk>>2]|0;
   $183 = HEAP32[$regRec>>2]|0;
   (_sqlite3VdbeAddOp4Int($180,31,$181,$182,$183,0)|0);
   $184 = HEAP32[$0>>2]|0;
   $185 = HEAP32[$regRec>>2]|0;
   _sqlite3ReleaseTempReg($184,$185);
   $186 = HEAP32[$0>>2]|0;
   $187 = HEAP32[$regTemp1>>2]|0;
   $188 = HEAP32[$nCol>>2]|0;
   _sqlite3ReleaseTempRange($186,$187,$188);
  }
 } while(0);
 $189 = HEAP32[$4>>2]|0;
 $190 = ((($189)) + 24|0);
 $191 = HEAP8[$190>>0]|0;
 $192 = ($191<<24>>24)!=(0);
 if (!($192)) {
  $193 = HEAP32[$0>>2]|0;
  $194 = HEAP32[$193>>2]|0;
  $195 = ((($194)) + 24|0);
  $196 = HEAP32[$195>>2]|0;
  $197 = $196 & 33554432;
  $198 = ($197|0)!=(0);
  if (!($198)) {
   $199 = HEAP32[$0>>2]|0;
   $200 = ((($199)) + 124|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = ($201|0)!=(0|0);
   if (!($202)) {
    $203 = HEAP32[$0>>2]|0;
    $204 = ((($203)) + 20|0);
    $205 = HEAP8[$204>>0]|0;
    $206 = ($205<<24>>24)!=(0);
    if (!($206)) {
     $207 = HEAP32[$0>>2]|0;
     _sqlite3HaltConstraint($207,787,2,0,-2,4);
     $222 = HEAP32[$v>>2]|0;
     $223 = HEAP32[$iOk>>2]|0;
     _sqlite3VdbeResolveLabel($222,$223);
     $224 = HEAP32[$v>>2]|0;
     $225 = HEAP32[$iCur>>2]|0;
     (_sqlite3VdbeAddOp1($224,111,$225)|0);
     STACKTOP = sp;return;
    }
   }
  }
 }
 $208 = HEAP32[$7>>2]|0;
 $209 = ($208|0)>(0);
 if ($209) {
  $210 = HEAP32[$4>>2]|0;
  $211 = ((($210)) + 24|0);
  $212 = HEAP8[$211>>0]|0;
  $213 = $212&255;
  $214 = ($213|0)==(0);
  if ($214) {
   $215 = HEAP32[$0>>2]|0;
   _sqlite3MayAbort($215);
  }
 }
 $216 = HEAP32[$v>>2]|0;
 $217 = HEAP32[$4>>2]|0;
 $218 = ((($217)) + 24|0);
 $219 = HEAP8[$218>>0]|0;
 $220 = $219&255;
 $221 = HEAP32[$7>>2]|0;
 (_sqlite3VdbeAddOp2($216,144,$220,$221)|0);
 $222 = HEAP32[$v>>2]|0;
 $223 = HEAP32[$iOk>>2]|0;
 _sqlite3VdbeResolveLabel($222,$223);
 $224 = HEAP32[$v>>2]|0;
 $225 = HEAP32[$iCur>>2]|0;
 (_sqlite3VdbeAddOp1($224,111,$225)|0);
 STACKTOP = sp;return;
}
function _isSetNullAction($pParse,$pFKey) {
 $pParse = $pParse|0;
 $pFKey = $pFKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pTop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pTop = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pFKey;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 124|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 124|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $9;
 } else {
  $10 = $7;
 }
 HEAP32[$pTop>>2] = $10;
 $11 = HEAP32[$pTop>>2]|0;
 $12 = ((($11)) + 468|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 do {
  if ($14) {
   $15 = HEAP32[$pTop>>2]|0;
   $16 = ((($15)) + 468|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$p>>2] = $18;
   $19 = HEAP32[$p>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 28|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($19|0)==($22|0);
   if ($23) {
    $24 = HEAP32[$2>>2]|0;
    $25 = ((($24)) + 25|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = ($27|0)==(7);
    if (!($28)) {
     label = 6;
    }
   } else {
    label = 6;
   }
   if ((label|0) == 6) {
    $29 = HEAP32[$p>>2]|0;
    $30 = HEAP32[$2>>2]|0;
    $31 = ((($30)) + 28|0);
    $32 = ((($31)) + 4|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($29|0)==($33|0);
    if (!($34)) {
     break;
    }
    $35 = HEAP32[$2>>2]|0;
    $36 = ((($35)) + 25|0);
    $37 = ((($36)) + 1|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = ($39|0)==(7);
    if (!($40)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $41 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _fkParentIsModified($pTab,$p,$aChange,$bChngRowid) {
 $pTab = $pTab|0;
 $p = $p|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iKey = 0, $or$cond = 0;
 var $pCol = 0, $zKey = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $zKey = sp + 8|0;
 $iKey = sp + 4|0;
 $pCol = sp;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $bChngRowid;
 HEAP32[$i>>2] = 0;
 L1: while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$2>>2]|0;
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   label = 14;
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 36|0);
  $13 = (($12) + ($10<<3)|0);
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$zKey>>2] = $15;
  HEAP32[$iKey>>2] = 0;
  while(1) {
   $16 = HEAP32[$iKey>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = ((($17)) + 34|0);
   $19 = HEAP16[$18>>1]|0;
   $20 = $19 << 16 >> 16;
   $21 = ($16|0)<($20|0);
   if (!($21)) {
    break;
   }
   $22 = HEAP32[$iKey>>2]|0;
   $23 = HEAP32[$3>>2]|0;
   $24 = (($23) + ($22<<2)|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ($25|0)>=(0);
   if ($26) {
    label = 7;
   } else {
    $27 = HEAP32[$iKey>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 32|0);
    $30 = HEAP16[$29>>1]|0;
    $31 = $30 << 16 >> 16;
    $32 = ($27|0)==($31|0);
    $33 = HEAP32[$4>>2]|0;
    $34 = ($33|0)!=(0);
    $or$cond = $32 & $34;
    if ($or$cond) {
     label = 7;
    }
   }
   do {
    if ((label|0) == 7) {
     label = 0;
     $35 = HEAP32[$iKey>>2]|0;
     $36 = HEAP32[$1>>2]|0;
     $37 = ((($36)) + 4|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = (($38) + ($35<<4)|0);
     HEAP32[$pCol>>2] = $39;
     $40 = HEAP32[$zKey>>2]|0;
     $41 = ($40|0)!=(0|0);
     $42 = HEAP32[$pCol>>2]|0;
     if ($41) {
      $43 = HEAP32[$42>>2]|0;
      $44 = HEAP32[$zKey>>2]|0;
      $45 = (_sqlite3StrICmp($43,$44)|0);
      $46 = (0)==($45|0);
      if ($46) {
       label = 9;
       break L1;
      } else {
       break;
      }
     } else {
      $47 = ((($42)) + 15|0);
      $48 = HEAP8[$47>>0]|0;
      $49 = $48&255;
      $50 = $49 & 1;
      $51 = ($50|0)!=(0);
      if ($51) {
       label = 11;
       break L1;
      } else {
       break;
      }
     }
    }
   } while(0);
   $52 = HEAP32[$iKey>>2]|0;
   $53 = (($52) + 1)|0;
   HEAP32[$iKey>>2] = $53;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 9) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return (0)|0;
}
function _fkScanChildren($pParse,$pSrc,$pTab,$pIdx,$pFKey,$aiCol,$regData,$nIncr) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 $pFKey = $pFKey|0;
 $aiCol = $aiCol|0;
 $regData = $regData|0;
 $nIncr = $nIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $iCol = 0, $iCol4 = 0, $iFkIfZero = 0, $or$cond = 0, $pAll = 0, $pEq = 0, $pEq3 = 0, $pLeft = 0, $pLeft1 = 0, $pNe = 0, $pPk = 0;
 var $pRight = 0, $pRight2 = 0, $pWInfo = 0, $pWhere = 0, $sNameContext = 0, $v = 0, $zCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 116|0;
 $4 = sp + 112|0;
 $5 = sp + 108|0;
 $6 = sp + 104|0;
 $7 = sp + 100|0;
 $db = sp + 96|0;
 $i = sp + 92|0;
 $pWhere = sp + 88|0;
 $sNameContext = sp + 56|0;
 $pWInfo = sp + 48|0;
 $iFkIfZero = sp + 44|0;
 $v = sp + 40|0;
 $pLeft = sp + 36|0;
 $pRight = sp + 32|0;
 $pEq = sp + 28|0;
 $iCol = sp + 134|0;
 $zCol = sp + 24|0;
 $pNe = sp + 20|0;
 $pLeft1 = sp + 16|0;
 $pRight2 = sp + 12|0;
 $pEq3 = sp + 8|0;
 $pAll = sp + 4|0;
 $pPk = sp;
 $iCol4 = sp + 132|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pIdx;
 HEAP32[$4>>2] = $pFKey;
 HEAP32[$5>>2] = $aiCol;
 HEAP32[$6>>2] = $regData;
 HEAP32[$7>>2] = $nIncr;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$db>>2] = $9;
 HEAP32[$pWhere>>2] = 0;
 HEAP32[$iFkIfZero>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3GetVdbe($10)|0);
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$7>>2]|0;
 $13 = ($12|0)<(0);
 if ($13) {
  $14 = HEAP32[$v>>2]|0;
  $15 = HEAP32[$4>>2]|0;
  $16 = ((($15)) + 24|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = (_sqlite3VdbeAddOp2($14,65,$18,0)|0);
  HEAP32[$iFkIfZero>>2] = $19;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($20|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$3>>2]|0;
  $26 = ($25|0)!=(0|0);
  if ($26) {
   $27 = HEAP32[$i>>2]|0;
   $28 = HEAP32[$3>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + ($27<<1)|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = $32 << 16 >> 16;
   $35 = $33;
  } else {
   $35 = -1;
  }
  $34 = $35&65535;
  HEAP16[$iCol>>1] = $34;
  $36 = HEAP32[$0>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$6>>2]|0;
  $39 = HEAP16[$iCol>>1]|0;
  $40 = (_exprTableRegister($36,$37,$38,$39)|0);
  HEAP32[$pLeft>>2] = $40;
  $41 = HEAP32[$5>>2]|0;
  $42 = ($41|0)!=(0|0);
  if ($42) {
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$5>>2]|0;
   $45 = (($44) + ($43<<2)|0);
   $46 = HEAP32[$45>>2]|0;
   $51 = $46;
  } else {
   $47 = HEAP32[$4>>2]|0;
   $48 = ((($47)) + 36|0);
   $49 = HEAP32[$48>>2]|0;
   $51 = $49;
  }
  $50 = $51&65535;
  HEAP16[$iCol>>1] = $50;
  $52 = HEAP16[$iCol>>1]|0;
  $53 = $52 << 16 >> 16;
  $54 = HEAP32[$4>>2]|0;
  $55 = HEAP32[$54>>2]|0;
  $56 = ((($55)) + 4|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = (($57) + ($53<<4)|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$zCol>>2] = $59;
  $60 = HEAP32[$db>>2]|0;
  $61 = HEAP32[$zCol>>2]|0;
  $62 = (_sqlite3Expr($60,55,$61)|0);
  HEAP32[$pRight>>2] = $62;
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$pLeft>>2]|0;
  $65 = HEAP32[$pRight>>2]|0;
  $66 = (_sqlite3PExpr($63,37,$64,$65,0)|0);
  HEAP32[$pEq>>2] = $66;
  $67 = HEAP32[$db>>2]|0;
  $68 = HEAP32[$pWhere>>2]|0;
  $69 = HEAP32[$pEq>>2]|0;
  $70 = (_sqlite3ExprAnd($67,$68,$69)|0);
  HEAP32[$pWhere>>2] = $70;
  $71 = HEAP32[$i>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$i>>2] = $72;
 }
 $73 = HEAP32[$2>>2]|0;
 $74 = HEAP32[$4>>2]|0;
 $75 = HEAP32[$74>>2]|0;
 $76 = ($73|0)==($75|0);
 $77 = HEAP32[$7>>2]|0;
 $78 = ($77|0)>(0);
 $or$cond = $76 & $78;
 if ($or$cond) {
  $79 = HEAP32[$2>>2]|0;
  $80 = ((($79)) + 42|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = $81&255;
  $83 = $82 & 32;
  $84 = ($83|0)==(0);
  if ($84) {
   $85 = HEAP32[$0>>2]|0;
   $86 = HEAP32[$2>>2]|0;
   $87 = HEAP32[$6>>2]|0;
   $88 = (_exprTableRegister($85,$86,$87,-1)|0);
   HEAP32[$pLeft1>>2] = $88;
   $89 = HEAP32[$db>>2]|0;
   $90 = HEAP32[$2>>2]|0;
   $91 = HEAP32[$1>>2]|0;
   $92 = ((($91)) + 8|0);
   $93 = ((($92)) + 44|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (_exprTableColumn($89,$90,$94,-1)|0);
   HEAP32[$pRight2>>2] = $95;
   $96 = HEAP32[$0>>2]|0;
   $97 = HEAP32[$pLeft1>>2]|0;
   $98 = HEAP32[$pRight2>>2]|0;
   $99 = (_sqlite3PExpr($96,36,$97,$98,0)|0);
   HEAP32[$pNe>>2] = $99;
  } else {
   HEAP32[$pAll>>2] = 0;
   $100 = HEAP32[$2>>2]|0;
   $101 = (_sqlite3PrimaryKeyIndex($100)|0);
   HEAP32[$pPk>>2] = $101;
   HEAP32[$i>>2] = 0;
   while(1) {
    $102 = HEAP32[$i>>2]|0;
    $103 = HEAP32[$pPk>>2]|0;
    $104 = ((($103)) + 50|0);
    $105 = HEAP16[$104>>1]|0;
    $106 = $105&65535;
    $107 = ($102|0)<($106|0);
    if (!($107)) {
     break;
    }
    $108 = HEAP32[$i>>2]|0;
    $109 = HEAP32[$3>>2]|0;
    $110 = ((($109)) + 4|0);
    $111 = HEAP32[$110>>2]|0;
    $112 = (($111) + ($108<<1)|0);
    $113 = HEAP16[$112>>1]|0;
    HEAP16[$iCol4>>1] = $113;
    $114 = HEAP32[$0>>2]|0;
    $115 = HEAP32[$2>>2]|0;
    $116 = HEAP32[$6>>2]|0;
    $117 = HEAP16[$iCol4>>1]|0;
    $118 = (_exprTableRegister($114,$115,$116,$117)|0);
    HEAP32[$pLeft1>>2] = $118;
    $119 = HEAP32[$db>>2]|0;
    $120 = HEAP32[$2>>2]|0;
    $121 = HEAP32[$1>>2]|0;
    $122 = ((($121)) + 8|0);
    $123 = ((($122)) + 44|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = HEAP16[$iCol4>>1]|0;
    $126 = (_exprTableColumn($119,$120,$124,$125)|0);
    HEAP32[$pRight2>>2] = $126;
    $127 = HEAP32[$0>>2]|0;
    $128 = HEAP32[$pLeft1>>2]|0;
    $129 = HEAP32[$pRight2>>2]|0;
    $130 = (_sqlite3PExpr($127,37,$128,$129,0)|0);
    HEAP32[$pEq3>>2] = $130;
    $131 = HEAP32[$db>>2]|0;
    $132 = HEAP32[$pAll>>2]|0;
    $133 = HEAP32[$pEq3>>2]|0;
    $134 = (_sqlite3ExprAnd($131,$132,$133)|0);
    HEAP32[$pAll>>2] = $134;
    $135 = HEAP32[$i>>2]|0;
    $136 = (($135) + 1)|0;
    HEAP32[$i>>2] = $136;
   }
   $137 = HEAP32[$0>>2]|0;
   $138 = HEAP32[$pAll>>2]|0;
   $139 = (_sqlite3PExpr($137,19,$138,0,0)|0);
   HEAP32[$pNe>>2] = $139;
  }
  $140 = HEAP32[$db>>2]|0;
  $141 = HEAP32[$pWhere>>2]|0;
  $142 = HEAP32[$pNe>>2]|0;
  $143 = (_sqlite3ExprAnd($140,$141,$142)|0);
  HEAP32[$pWhere>>2] = $143;
 }
 ;HEAP32[$sNameContext>>2]=0|0;HEAP32[$sNameContext+4>>2]=0|0;HEAP32[$sNameContext+8>>2]=0|0;HEAP32[$sNameContext+12>>2]=0|0;HEAP32[$sNameContext+16>>2]=0|0;HEAP32[$sNameContext+20>>2]=0|0;HEAP32[$sNameContext+24>>2]=0|0;HEAP32[$sNameContext+28>>2]=0|0;
 $144 = HEAP32[$1>>2]|0;
 $145 = ((($sNameContext)) + 4|0);
 HEAP32[$145>>2] = $144;
 $146 = HEAP32[$0>>2]|0;
 HEAP32[$sNameContext>>2] = $146;
 $147 = HEAP32[$pWhere>>2]|0;
 (_sqlite3ResolveExprNames($sNameContext,$147)|0);
 $148 = HEAP32[$0>>2]|0;
 $149 = HEAP32[$1>>2]|0;
 $150 = HEAP32[$pWhere>>2]|0;
 $151 = (_sqlite3WhereBegin($148,$149,$150,0,0,0,0)|0);
 HEAP32[$pWInfo>>2] = $151;
 $152 = HEAP32[$v>>2]|0;
 $153 = HEAP32[$4>>2]|0;
 $154 = ((($153)) + 24|0);
 $155 = HEAP8[$154>>0]|0;
 $156 = $155&255;
 $157 = HEAP32[$7>>2]|0;
 (_sqlite3VdbeAddOp2($152,144,$156,$157)|0);
 $158 = HEAP32[$pWInfo>>2]|0;
 $159 = ($158|0)!=(0|0);
 if ($159) {
  $160 = HEAP32[$pWInfo>>2]|0;
  _sqlite3WhereEnd($160);
 }
 $161 = HEAP32[$db>>2]|0;
 $162 = HEAP32[$pWhere>>2]|0;
 _sqlite3ExprDelete($161,$162);
 $163 = HEAP32[$iFkIfZero>>2]|0;
 $164 = ($163|0)!=(0);
 if (!($164)) {
  STACKTOP = sp;return;
 }
 $165 = HEAP32[$v>>2]|0;
 $166 = HEAP32[$iFkIfZero>>2]|0;
 _sqlite3VdbeJumpHere($165,$166);
 STACKTOP = sp;return;
}
function _exprTableRegister($pParse,$pTab,$regBase,$iCol) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $regBase = $regBase|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $pCol = 0, $pExpr = 0;
 var $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 28|0;
 $pExpr = sp + 12|0;
 $pCol = sp + 8|0;
 $zColl = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $regBase;
 HEAP16[$3>>1] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3Expr($6,157,0)|0);
 HEAP32[$pExpr>>2] = $7;
 $8 = HEAP32[$pExpr>>2]|0;
 $9 = ($8|0)!=(0|0);
 if (!($9)) {
  $56 = HEAP32[$pExpr>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $10 = HEAP16[$3>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = ($11|0)>=(0);
 if ($12) {
  $13 = HEAP16[$3>>1]|0;
  $14 = $13 << 16 >> 16;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 32|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  $19 = ($14|0)!=($18|0);
  if ($19) {
   $20 = HEAP16[$3>>1]|0;
   $21 = $20 << 16 >> 16;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<4)|0);
   HEAP32[$pCol>>2] = $25;
   $26 = HEAP32[$2>>2]|0;
   $27 = HEAP16[$3>>1]|0;
   $28 = $27 << 16 >> 16;
   $29 = (($26) + ($28))|0;
   $30 = (($29) + 1)|0;
   $31 = HEAP32[$pExpr>>2]|0;
   $32 = ((($31)) + 28|0);
   HEAP32[$32>>2] = $30;
   $33 = HEAP32[$pCol>>2]|0;
   $34 = ((($33)) + 13|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = HEAP32[$pExpr>>2]|0;
   $37 = ((($36)) + 1|0);
   HEAP8[$37>>0] = $35;
   $38 = HEAP32[$pCol>>2]|0;
   $39 = ((($38)) + 8|0);
   $40 = HEAP32[$39>>2]|0;
   HEAP32[$zColl>>2] = $40;
   $41 = HEAP32[$zColl>>2]|0;
   $42 = ($41|0)==(0|0);
   if ($42) {
    $43 = HEAP32[$db>>2]|0;
    $44 = ((($43)) + 8|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$zColl>>2] = $46;
   }
   $47 = HEAP32[$0>>2]|0;
   $48 = HEAP32[$pExpr>>2]|0;
   $49 = HEAP32[$zColl>>2]|0;
   $50 = (_sqlite3ExprAddCollateString($47,$48,$49)|0);
   HEAP32[$pExpr>>2] = $50;
   $56 = HEAP32[$pExpr>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 }
 $51 = HEAP32[$2>>2]|0;
 $52 = HEAP32[$pExpr>>2]|0;
 $53 = ((($52)) + 28|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$pExpr>>2]|0;
 $55 = ((($54)) + 1|0);
 HEAP8[$55>>0] = 68;
 $56 = HEAP32[$pExpr>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _exprTableColumn($db,$pTab,$iCursor,$iCol) {
 $db = $db|0;
 $pTab = $pTab|0;
 $iCursor = $iCursor|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pExpr = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp + 16|0;
 $pExpr = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iCursor;
 HEAP16[$3>>1] = $iCol;
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3Expr($4,152,0)|0);
 HEAP32[$pExpr>>2] = $5;
 $6 = HEAP32[$pExpr>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $17 = HEAP32[$pExpr>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$pExpr>>2]|0;
 $10 = ((($9)) + 44|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$pExpr>>2]|0;
 $13 = ((($12)) + 28|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP16[$3>>1]|0;
 $15 = HEAP32[$pExpr>>2]|0;
 $16 = ((($15)) + 32|0);
 HEAP16[$16>>1] = $14;
 $17 = HEAP32[$pExpr>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3WhereBegin($pParse,$pTabList,$pWhere,$pOrderBy,$pDistinctSet,$wctrlFlags,$iAuxArg) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWhere = $pWhere|0;
 $pOrderBy = $pOrderBy|0;
 $pDistinctSet = $pDistinctSet|0;
 $wctrlFlags = $wctrlFlags|0;
 $iAuxArg = $iAuxArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $addrExplain = 0, $b = 0, $bFordelete = 0, $bOnerow = 0, $db = 0, $iCur = 0, $iDb = 0, $iIndexCur = 0, $ii = 0, $n = 0, $nByteWInfo = 0, $nTabList = 0, $notReady = 0, $op = 0, $op1 = 0, $or$cond = 0, $pEnd = 0, $pIx = 0, $pJ = 0, $pLevel = 0;
 var $pLoop = 0, $pMaskSet = 0, $pTab = 0, $pTabItem = 0, $pTerm = 0, $pVTab = 0, $pWInfo = 0, $rc = 0, $sWLB = 0, $tabUsed = 0, $v = 0, $vararg_buffer = 0, $wsFlags = 0, $wsFlags2 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer = sp + 24|0;
 $0 = sp + 180|0;
 $1 = sp + 176|0;
 $2 = sp + 172|0;
 $3 = sp + 168|0;
 $4 = sp + 164|0;
 $5 = sp + 160|0;
 $6 = sp + 184|0;
 $7 = sp + 156|0;
 $nByteWInfo = sp + 152|0;
 $nTabList = sp + 148|0;
 $pWInfo = sp + 144|0;
 $v = sp + 140|0;
 $notReady = sp + 16|0;
 $sWLB = sp + 120|0;
 $pMaskSet = sp + 116|0;
 $pLevel = sp + 112|0;
 $pLoop = sp + 108|0;
 $ii = sp + 104|0;
 $db = sp + 100|0;
 $rc = sp + 96|0;
 $bFordelete = sp + 186|0;
 $tabUsed = sp + 8|0;
 $pTerm = sp + 92|0;
 $pEnd = sp + 88|0;
 $wsFlags = sp + 84|0;
 $bOnerow = sp + 80|0;
 $pTab = sp + 76|0;
 $iDb = sp + 72|0;
 $pTabItem = sp + 68|0;
 $pVTab = sp + 64|0;
 $iCur = sp + 60|0;
 $op = sp + 56|0;
 $b = sp;
 $n = sp + 52|0;
 $pIx = sp + 48|0;
 $iIndexCur = sp + 44|0;
 $op1 = sp + 40|0;
 $pJ = sp + 36|0;
 $addrExplain = sp + 32|0;
 $wsFlags2 = sp + 28|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $pOrderBy;
 HEAP32[$5>>2] = $pDistinctSet;
 HEAP16[$6>>1] = $wctrlFlags;
 HEAP32[$7>>2] = $iAuxArg;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 HEAP8[$bFordelete>>0] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 ;HEAP32[$sWLB>>2]=0|0;HEAP32[$sWLB+4>>2]=0|0;HEAP32[$sWLB+8>>2]=0|0;HEAP32[$sWLB+12>>2]=0|0;HEAP32[$sWLB+16>>2]=0|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$4>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)>=(64);
  if ($17) {
   HEAP32[$4>>2] = 0;
  }
 }
 $18 = HEAP32[$4>>2]|0;
 $19 = ((($sWLB)) + 8|0);
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$db>>2]|0;
 $21 = ((($20)) + 64|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & 32;
 $25 = ($24|0)!=(0);
 if ($25) {
  $26 = HEAP16[$6>>1]|0;
  $27 = $26&65535;
  $28 = $27 & -257;
  $29 = $28&65535;
  HEAP16[$6>>1] = $29;
 }
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)>(64);
 if ($32) {
  $33 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = 64;
  _sqlite3ErrorMsg($33,34404,$vararg_buffer);
  HEAP32[$0>>2] = 0;
  $839 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($839|0);
 }
 $34 = HEAP16[$6>>1]|0;
 $35 = $34&65535;
 $36 = $35 & 32;
 $37 = ($36|0)!=(0);
 if ($37) {
  $40 = 1;
 } else {
  $38 = HEAP32[$2>>2]|0;
  $39 = HEAP32[$38>>2]|0;
  $40 = $39;
 }
 HEAP32[$nTabList>>2] = $40;
 $41 = HEAP32[$nTabList>>2]|0;
 $42 = (($41) - 1)|0;
 $43 = ($42*80)|0;
 $44 = (832 + ($43))|0;
 $45 = (($44) + 7)|0;
 $46 = $45 & -8;
 HEAP32[$nByteWInfo>>2] = $46;
 $47 = HEAP32[$db>>2]|0;
 $48 = HEAP32[$nByteWInfo>>2]|0;
 $49 = (($48) + 72)|0;
 $50 = (_sqlite3DbMallocRawNN($47,$49,0)|0);
 HEAP32[$pWInfo>>2] = $50;
 $51 = HEAP32[$db>>2]|0;
 $52 = ((($51)) + 69|0);
 $53 = HEAP8[$52>>0]|0;
 $54 = ($53<<24>>24)!=(0);
 L15: do {
  if ($54) {
   $55 = HEAP32[$db>>2]|0;
   $56 = HEAP32[$pWInfo>>2]|0;
   _sqlite3DbFree($55,$56);
   HEAP32[$pWInfo>>2] = 0;
  } else {
   $57 = HEAP32[$1>>2]|0;
   $58 = HEAP32[$pWInfo>>2]|0;
   HEAP32[$58>>2] = $57;
   $59 = HEAP32[$2>>2]|0;
   $60 = HEAP32[$pWInfo>>2]|0;
   $61 = ((($60)) + 4|0);
   HEAP32[$61>>2] = $59;
   $62 = HEAP32[$4>>2]|0;
   $63 = HEAP32[$pWInfo>>2]|0;
   $64 = ((($63)) + 8|0);
   HEAP32[$64>>2] = $62;
   $65 = HEAP32[$5>>2]|0;
   $66 = HEAP32[$pWInfo>>2]|0;
   $67 = ((($66)) + 12|0);
   HEAP32[$67>>2] = $65;
   $68 = HEAP32[$pWInfo>>2]|0;
   $69 = ((($68)) + 20|0);
   $70 = ((($69)) + 4|0);
   HEAP32[$70>>2] = -1;
   $71 = HEAP32[$pWInfo>>2]|0;
   $72 = ((($71)) + 20|0);
   HEAP32[$72>>2] = -1;
   $73 = HEAP32[$nTabList>>2]|0;
   $74 = $73&255;
   $75 = HEAP32[$pWInfo>>2]|0;
   $76 = ((($75)) + 42|0);
   HEAP8[$76>>0] = $74;
   $77 = HEAP32[$v>>2]|0;
   $78 = (_sqlite3VdbeMakeLabel($77)|0);
   $79 = HEAP32[$pWInfo>>2]|0;
   $80 = ((($79)) + 28|0);
   HEAP32[$80>>2] = $78;
   $81 = HEAP32[$pWInfo>>2]|0;
   $82 = ((($81)) + 32|0);
   HEAP32[$82>>2] = $78;
   $83 = HEAP16[$6>>1]|0;
   $84 = HEAP32[$pWInfo>>2]|0;
   $85 = ((($84)) + 40|0);
   HEAP16[$85>>1] = $83;
   $86 = HEAP32[$7>>2]|0;
   $87 = $86&65535;
   $88 = HEAP32[$pWInfo>>2]|0;
   $89 = ((($88)) + 16|0);
   HEAP16[$89>>1] = $87;
   $90 = HEAP32[$1>>2]|0;
   $91 = ((($90)) + 136|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = HEAP32[$pWInfo>>2]|0;
   $94 = ((($93)) + 36|0);
   HEAP32[$94>>2] = $92;
   $95 = HEAP32[$pWInfo>>2]|0;
   $96 = ((($95)) + 43|0);
   dest=$96; stop=dest+37|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
   $97 = HEAP32[$pWInfo>>2]|0;
   $98 = ((($97)) + 752|0);
   $99 = HEAP32[$nTabList>>2]|0;
   $100 = ($99*80)|0;
   $101 = (72 + ($100))|0;
   _memset(($98|0),0,($101|0))|0;
   $102 = HEAP32[$pWInfo>>2]|0;
   $103 = ((($102)) + 488|0);
   HEAP32[$pMaskSet>>2] = $103;
   $104 = HEAP32[$pWInfo>>2]|0;
   HEAP32[$sWLB>>2] = $104;
   $105 = HEAP32[$pWInfo>>2]|0;
   $106 = ((($105)) + 80|0);
   $107 = ((($sWLB)) + 4|0);
   HEAP32[$107>>2] = $106;
   $108 = HEAP32[$pWInfo>>2]|0;
   $109 = HEAP32[$nByteWInfo>>2]|0;
   $110 = (($108) + ($109)|0);
   $111 = ((($sWLB)) + 12|0);
   HEAP32[$111>>2] = $110;
   $112 = ((($sWLB)) + 12|0);
   $113 = HEAP32[$112>>2]|0;
   _whereLoopInit($113);
   $114 = HEAP32[$pMaskSet>>2]|0;
   HEAP32[$114>>2] = 0;
   $115 = HEAP32[$pWInfo>>2]|0;
   $116 = ((($115)) + 80|0);
   $117 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereClauseInit($116,$117);
   $118 = HEAP32[$pWInfo>>2]|0;
   $119 = ((($118)) + 80|0);
   $120 = HEAP32[$3>>2]|0;
   _sqlite3WhereSplit($119,$120,28);
   HEAP32[$ii>>2] = 0;
   while(1) {
    $121 = HEAP32[$ii>>2]|0;
    $122 = ((($sWLB)) + 4|0);
    $123 = HEAP32[$122>>2]|0;
    $124 = ((($123)) + 12|0);
    $125 = HEAP32[$124>>2]|0;
    $126 = ($121|0)<($125|0);
    $127 = HEAP32[$nTabList>>2]|0;
    $128 = ($127|0)==(0);
    if (!($126)) {
     break;
    }
    if ($128) {
     label = 16;
    } else {
     $129 = HEAP32[$ii>>2]|0;
     $130 = ((($sWLB)) + 4|0);
     $131 = HEAP32[$130>>2]|0;
     $132 = ((($131)) + 20|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = (($133) + (($129*48)|0)|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = (_sqlite3ExprIsConstantNotJoin($135)|0);
     $137 = ($136|0)!=(0);
     if ($137) {
      label = 16;
     }
    }
    if ((label|0) == 16) {
     label = 0;
     $138 = HEAP32[$1>>2]|0;
     $139 = HEAP32[$ii>>2]|0;
     $140 = ((($sWLB)) + 4|0);
     $141 = HEAP32[$140>>2]|0;
     $142 = ((($141)) + 20|0);
     $143 = HEAP32[$142>>2]|0;
     $144 = (($143) + (($139*48)|0)|0);
     $145 = HEAP32[$144>>2]|0;
     $146 = HEAP32[$pWInfo>>2]|0;
     $147 = ((($146)) + 32|0);
     $148 = HEAP32[$147>>2]|0;
     _sqlite3ExprIfFalse($138,$145,$148,16);
     $149 = HEAP32[$ii>>2]|0;
     $150 = ((($sWLB)) + 4|0);
     $151 = HEAP32[$150>>2]|0;
     $152 = ((($151)) + 20|0);
     $153 = HEAP32[$152>>2]|0;
     $154 = (($153) + (($149*48)|0)|0);
     $155 = ((($154)) + 10|0);
     $156 = HEAP16[$155>>1]|0;
     $157 = $156&65535;
     $158 = $157 | 4;
     $159 = $158&65535;
     HEAP16[$155>>1] = $159;
    }
    $160 = HEAP32[$ii>>2]|0;
    $161 = (($160) + 1)|0;
    HEAP32[$ii>>2] = $161;
   }
   if ($128) {
    $162 = HEAP32[$4>>2]|0;
    $163 = ($162|0)!=(0|0);
    if ($163) {
     $164 = HEAP32[$4>>2]|0;
     $165 = HEAP32[$164>>2]|0;
     $166 = $165&255;
     $167 = HEAP32[$pWInfo>>2]|0;
     $168 = ((($167)) + 43|0);
     HEAP8[$168>>0] = $166;
    }
    $169 = HEAP16[$6>>1]|0;
    $170 = $169&65535;
    $171 = $170 & 256;
    $172 = ($171|0)!=(0);
    if ($172) {
     $173 = HEAP32[$pWInfo>>2]|0;
     $174 = ((($173)) + 47|0);
     HEAP8[$174>>0] = 1;
    }
   }
   HEAP32[$ii>>2] = 0;
   while(1) {
    $175 = HEAP32[$ii>>2]|0;
    $176 = HEAP32[$2>>2]|0;
    $177 = HEAP32[$176>>2]|0;
    $178 = ($175|0)<($177|0);
    if (!($178)) {
     break;
    }
    $179 = HEAP32[$pMaskSet>>2]|0;
    $180 = HEAP32[$ii>>2]|0;
    $181 = HEAP32[$2>>2]|0;
    $182 = ((($181)) + 8|0);
    $183 = (($182) + (($180*72)|0)|0);
    $184 = ((($183)) + 44|0);
    $185 = HEAP32[$184>>2]|0;
    _createMask($179,$185);
    $186 = HEAP32[$1>>2]|0;
    $187 = HEAP32[$ii>>2]|0;
    $188 = HEAP32[$2>>2]|0;
    $189 = ((($188)) + 8|0);
    $190 = (($189) + (($187*72)|0)|0);
    $191 = HEAP32[$pWInfo>>2]|0;
    $192 = ((($191)) + 80|0);
    _sqlite3WhereTabFuncArgs($186,$190,$192);
    $193 = HEAP32[$ii>>2]|0;
    $194 = (($193) + 1)|0;
    HEAP32[$ii>>2] = $194;
   }
   $195 = HEAP32[$2>>2]|0;
   $196 = HEAP32[$pWInfo>>2]|0;
   $197 = ((($196)) + 80|0);
   _sqlite3WhereExprAnalyze($195,$197);
   $198 = HEAP32[$db>>2]|0;
   $199 = ((($198)) + 69|0);
   $200 = HEAP8[$199>>0]|0;
   $201 = ($200<<24>>24)!=(0);
   if (!($201)) {
    $202 = HEAP16[$6>>1]|0;
    $203 = $202&65535;
    $204 = $203 & 256;
    $205 = ($204|0)!=(0);
    do {
     if ($205) {
      $206 = HEAP32[$1>>2]|0;
      $207 = HEAP32[$2>>2]|0;
      $208 = HEAP32[$pWInfo>>2]|0;
      $209 = ((($208)) + 80|0);
      $210 = HEAP32[$5>>2]|0;
      $211 = (_isDistinctRedundant($206,$207,$209,$210)|0);
      $212 = ($211|0)!=(0);
      if ($212) {
       $213 = HEAP32[$pWInfo>>2]|0;
       $214 = ((($213)) + 47|0);
       HEAP8[$214>>0] = 1;
       break;
      }
      $215 = HEAP32[$4>>2]|0;
      $216 = ($215|0)==(0|0);
      if ($216) {
       $217 = HEAP32[$pWInfo>>2]|0;
       $218 = ((($217)) + 40|0);
       $219 = HEAP16[$218>>1]|0;
       $220 = $219&65535;
       $221 = $220 | 128;
       $222 = $221&65535;
       HEAP16[$218>>1] = $222;
       $223 = HEAP32[$5>>2]|0;
       $224 = HEAP32[$pWInfo>>2]|0;
       $225 = ((($224)) + 8|0);
       HEAP32[$225>>2] = $223;
      }
     }
    } while(0);
    $226 = HEAP32[$nTabList>>2]|0;
    $227 = ($226|0)!=(1);
    if ($227) {
     label = 34;
    } else {
     $228 = (_whereShortCut($sWLB)|0);
     $229 = ($228|0)==(0);
     if ($229) {
      label = 34;
     }
    }
    if ((label|0) == 34) {
     $230 = (_whereLoopAddAll($sWLB)|0);
     HEAP32[$rc>>2] = $230;
     $231 = HEAP32[$rc>>2]|0;
     $232 = ($231|0)!=(0);
     if ($232) {
      break;
     }
     $233 = HEAP32[$pWInfo>>2]|0;
     (_wherePathSolver($233,0)|0);
     $234 = HEAP32[$db>>2]|0;
     $235 = ((($234)) + 69|0);
     $236 = HEAP8[$235>>0]|0;
     $237 = ($236<<24>>24)!=(0);
     if ($237) {
      break;
     }
     $238 = HEAP32[$pWInfo>>2]|0;
     $239 = ((($238)) + 8|0);
     $240 = HEAP32[$239>>2]|0;
     $241 = ($240|0)!=(0|0);
     if ($241) {
      $242 = HEAP32[$pWInfo>>2]|0;
      $243 = HEAP32[$pWInfo>>2]|0;
      $244 = ((($243)) + 72|0);
      $245 = HEAP16[$244>>1]|0;
      $246 = $245 << 16 >> 16;
      $247 = (($246) + 1)|0;
      $248 = $247&65535;
      (_wherePathSolver($242,$248)|0);
      $249 = HEAP32[$db>>2]|0;
      $250 = ((($249)) + 69|0);
      $251 = HEAP8[$250>>0]|0;
      $252 = ($251<<24>>24)!=(0);
      if ($252) {
       break;
      }
     }
    }
    $253 = HEAP32[$pWInfo>>2]|0;
    $254 = ((($253)) + 8|0);
    $255 = HEAP32[$254>>2]|0;
    $256 = ($255|0)==(0|0);
    if ($256) {
     $257 = HEAP32[$db>>2]|0;
     $258 = ((($257)) + 24|0);
     $259 = HEAP32[$258>>2]|0;
     $260 = $259 & 131072;
     $261 = ($260|0)!=(0);
     if ($261) {
      $262 = HEAP32[$pWInfo>>2]|0;
      $263 = ((($262)) + 64|0);
      $264 = $263;
      $265 = $264;
      HEAP32[$265>>2] = -1;
      $266 = (($264) + 4)|0;
      $267 = $266;
      HEAP32[$267>>2] = -1;
     }
    }
    $268 = HEAP32[$1>>2]|0;
    $269 = ((($268)) + 36|0);
    $270 = HEAP32[$269>>2]|0;
    $271 = ($270|0)!=(0);
    if (!($271)) {
     $272 = HEAP32[$db>>2]|0;
     $273 = ((($272)) + 69|0);
     $274 = HEAP8[$273>>0]|0;
     $275 = $274&255;
     $276 = ($275|0)!=(0);
     if (!($276)) {
      $277 = HEAP32[$pWInfo>>2]|0;
      $278 = ((($277)) + 42|0);
      $279 = HEAP8[$278>>0]|0;
      $280 = $279&255;
      $281 = ($280|0)>=(2);
      $282 = HEAP32[$5>>2]|0;
      $283 = ($282|0)!=(0|0);
      $or$cond = $281 & $283;
      L60: do {
       if ($or$cond) {
        $284 = HEAP32[$db>>2]|0;
        $285 = ((($284)) + 64|0);
        $286 = HEAP16[$285>>1]|0;
        $287 = $286&65535;
        $288 = $287 & 1024;
        $289 = ($288|0)==(0);
        if ($289) {
         $290 = HEAP32[$pMaskSet>>2]|0;
         $291 = HEAP32[$5>>2]|0;
         $292 = (_sqlite3WhereExprListUsage($290,$291)|0);
         $293 = tempRet0;
         $294 = $tabUsed;
         $295 = $294;
         HEAP32[$295>>2] = $292;
         $296 = (($294) + 4)|0;
         $297 = $296;
         HEAP32[$297>>2] = $293;
         $298 = ((($sWLB)) + 8|0);
         $299 = HEAP32[$298>>2]|0;
         $300 = ($299|0)!=(0|0);
         if ($300) {
          $301 = HEAP32[$pMaskSet>>2]|0;
          $302 = ((($sWLB)) + 8|0);
          $303 = HEAP32[$302>>2]|0;
          $304 = (_sqlite3WhereExprListUsage($301,$303)|0);
          $305 = tempRet0;
          $306 = $tabUsed;
          $307 = $306;
          $308 = HEAP32[$307>>2]|0;
          $309 = (($306) + 4)|0;
          $310 = $309;
          $311 = HEAP32[$310>>2]|0;
          $312 = $308 | $304;
          $313 = $311 | $305;
          $314 = $tabUsed;
          $315 = $314;
          HEAP32[$315>>2] = $312;
          $316 = (($314) + 4)|0;
          $317 = $316;
          HEAP32[$317>>2] = $313;
         }
         while(1) {
          $318 = HEAP32[$pWInfo>>2]|0;
          $319 = ((($318)) + 42|0);
          $320 = HEAP8[$319>>0]|0;
          $321 = $320&255;
          $322 = ($321|0)>=(2);
          if (!($322)) {
           break L60;
          }
          $323 = HEAP32[$pWInfo>>2]|0;
          $324 = ((($323)) + 42|0);
          $325 = HEAP8[$324>>0]|0;
          $326 = $325&255;
          $327 = (($326) - 1)|0;
          $328 = HEAP32[$pWInfo>>2]|0;
          $329 = ((($328)) + 752|0);
          $330 = (($329) + (($327*80)|0)|0);
          $331 = ((($330)) + 64|0);
          $332 = HEAP32[$331>>2]|0;
          HEAP32[$pLoop>>2] = $332;
          $333 = HEAP32[$pLoop>>2]|0;
          $334 = ((($333)) + 16|0);
          $335 = HEAP8[$334>>0]|0;
          $336 = $335&255;
          $337 = HEAP32[$pWInfo>>2]|0;
          $338 = ((($337)) + 4|0);
          $339 = HEAP32[$338>>2]|0;
          $340 = ((($339)) + 8|0);
          $341 = (($340) + (($336*72)|0)|0);
          $342 = ((($341)) + 36|0);
          $343 = HEAP8[$342>>0]|0;
          $344 = $343&255;
          $345 = $344 & 8;
          $346 = ($345|0)==(0);
          if ($346) {
           break L60;
          }
          $347 = HEAP16[$6>>1]|0;
          $348 = $347&65535;
          $349 = $348 & 256;
          $350 = ($349|0)==(0);
          if ($350) {
           $351 = HEAP32[$pLoop>>2]|0;
           $352 = ((($351)) + 36|0);
           $353 = HEAP32[$352>>2]|0;
           $354 = $353 & 4096;
           $355 = ($354|0)==(0);
           if ($355) {
            break L60;
           }
          }
          $356 = $tabUsed;
          $357 = $356;
          $358 = HEAP32[$357>>2]|0;
          $359 = (($356) + 4)|0;
          $360 = $359;
          $361 = HEAP32[$360>>2]|0;
          $362 = HEAP32[$pLoop>>2]|0;
          $363 = ((($362)) + 8|0);
          $364 = $363;
          $365 = $364;
          $366 = HEAP32[$365>>2]|0;
          $367 = (($364) + 4)|0;
          $368 = $367;
          $369 = HEAP32[$368>>2]|0;
          $370 = $358 & $366;
          $371 = $361 & $369;
          $372 = ($370|0)!=(0);
          $373 = ($371|0)!=(0);
          $374 = $372 | $373;
          if ($374) {
           break L60;
          }
          $375 = ((($sWLB)) + 4|0);
          $376 = HEAP32[$375>>2]|0;
          $377 = ((($376)) + 20|0);
          $378 = HEAP32[$377>>2]|0;
          $379 = ((($sWLB)) + 4|0);
          $380 = HEAP32[$379>>2]|0;
          $381 = ((($380)) + 12|0);
          $382 = HEAP32[$381>>2]|0;
          $383 = (($378) + (($382*48)|0)|0);
          HEAP32[$pEnd>>2] = $383;
          $384 = ((($sWLB)) + 4|0);
          $385 = HEAP32[$384>>2]|0;
          $386 = ((($385)) + 20|0);
          $387 = HEAP32[$386>>2]|0;
          HEAP32[$pTerm>>2] = $387;
          while(1) {
           $388 = HEAP32[$pTerm>>2]|0;
           $389 = HEAP32[$pEnd>>2]|0;
           $390 = ($388>>>0)<($389>>>0);
           if (!($390)) {
            break;
           }
           $391 = HEAP32[$pTerm>>2]|0;
           $392 = ((($391)) + 40|0);
           $393 = $392;
           $394 = $393;
           $395 = HEAP32[$394>>2]|0;
           $396 = (($393) + 4)|0;
           $397 = $396;
           $398 = HEAP32[$397>>2]|0;
           $399 = HEAP32[$pLoop>>2]|0;
           $400 = ((($399)) + 8|0);
           $401 = $400;
           $402 = $401;
           $403 = HEAP32[$402>>2]|0;
           $404 = (($401) + 4)|0;
           $405 = $404;
           $406 = HEAP32[$405>>2]|0;
           $407 = $395 & $403;
           $408 = $398 & $406;
           $409 = ($407|0)!=(0);
           $410 = ($408|0)!=(0);
           $411 = $409 | $410;
           if ($411) {
            $412 = HEAP32[$pTerm>>2]|0;
            $413 = HEAP32[$412>>2]|0;
            $414 = ((($413)) + 4|0);
            $415 = HEAP32[$414>>2]|0;
            $416 = $415 & 1;
            $417 = ($416|0)!=(0);
            if (!($417)) {
             break;
            }
           }
           $418 = HEAP32[$pTerm>>2]|0;
           $419 = ((($418)) + 48|0);
           HEAP32[$pTerm>>2] = $419;
          }
          $420 = HEAP32[$pTerm>>2]|0;
          $421 = HEAP32[$pEnd>>2]|0;
          $422 = ($420>>>0)<($421>>>0);
          if ($422) {
           break L60;
          }
          $423 = HEAP32[$pWInfo>>2]|0;
          $424 = ((($423)) + 42|0);
          $425 = HEAP8[$424>>0]|0;
          $426 = (($425) + -1)<<24>>24;
          HEAP8[$424>>0] = $426;
          $427 = HEAP32[$nTabList>>2]|0;
          $428 = (($427) + -1)|0;
          HEAP32[$nTabList>>2] = $428;
         }
        }
       }
      } while(0);
      $429 = HEAP32[$pWInfo>>2]|0;
      $430 = ((($429)) + 72|0);
      $431 = HEAP16[$430>>1]|0;
      $432 = $431 << 16 >> 16;
      $433 = HEAP32[$pWInfo>>2]|0;
      $434 = HEAP32[$433>>2]|0;
      $435 = ((($434)) + 136|0);
      $436 = HEAP32[$435>>2]|0;
      $437 = (($436) + ($432))|0;
      HEAP32[$435>>2] = $437;
      $438 = HEAP16[$6>>1]|0;
      $439 = $438&65535;
      $440 = $439 & 4;
      $441 = ($440|0)!=(0);
      do {
       if ($441) {
        $442 = HEAP32[$pWInfo>>2]|0;
        $443 = ((($442)) + 752|0);
        $444 = ((($443)) + 64|0);
        $445 = HEAP32[$444>>2]|0;
        $446 = ((($445)) + 36|0);
        $447 = HEAP32[$446>>2]|0;
        HEAP32[$wsFlags>>2] = $447;
        $448 = HEAP32[$wsFlags>>2]|0;
        $449 = $448 & 4096;
        $450 = ($449|0)!=(0);
        $451 = $450&1;
        HEAP32[$bOnerow>>2] = $451;
        $452 = HEAP32[$bOnerow>>2]|0;
        $453 = ($452|0)!=(0);
        if (!($453)) {
         $454 = HEAP16[$6>>1]|0;
         $455 = $454&65535;
         $456 = $455 & 8;
         $457 = ($456|0)!=(0);
         if (!($457)) {
          break;
         }
         $458 = HEAP32[$wsFlags>>2]|0;
         $459 = $458 & 1024;
         $460 = (0)==($459|0);
         if (!($460)) {
          break;
         }
        }
        $461 = HEAP32[$bOnerow>>2]|0;
        $462 = ($461|0)!=(0);
        $463 = $462 ? 1 : 2;
        $464 = $463&255;
        $465 = HEAP32[$pWInfo>>2]|0;
        $466 = ((($465)) + 45|0);
        HEAP8[$466>>0] = $464;
        $467 = HEAP32[$2>>2]|0;
        $468 = ((($467)) + 8|0);
        $469 = ((($468)) + 16|0);
        $470 = HEAP32[$469>>2]|0;
        $471 = ((($470)) + 42|0);
        $472 = HEAP8[$471>>0]|0;
        $473 = $472&255;
        $474 = $473 & 32;
        $475 = ($474|0)==(0);
        if ($475) {
         $476 = HEAP32[$wsFlags>>2]|0;
         $477 = $476 & 64;
         $478 = ($477|0)!=(0);
         if ($478) {
          $479 = HEAP16[$6>>1]|0;
          $480 = $479&65535;
          $481 = $480 & 8;
          $482 = ($481|0)!=(0);
          if ($482) {
           HEAP8[$bFordelete>>0] = 8;
          }
          $483 = HEAP32[$wsFlags>>2]|0;
          $484 = $483 & -65;
          $485 = HEAP32[$pWInfo>>2]|0;
          $486 = ((($485)) + 752|0);
          $487 = ((($486)) + 64|0);
          $488 = HEAP32[$487>>2]|0;
          $489 = ((($488)) + 36|0);
          HEAP32[$489>>2] = $484;
         }
        }
       }
      } while(0);
      HEAP32[$ii>>2] = 0;
      $490 = HEAP32[$pWInfo>>2]|0;
      $491 = ((($490)) + 752|0);
      HEAP32[$pLevel>>2] = $491;
      while(1) {
       $492 = HEAP32[$ii>>2]|0;
       $493 = HEAP32[$nTabList>>2]|0;
       $494 = ($492|0)<($493|0);
       if (!($494)) {
        break;
       }
       $495 = HEAP32[$pLevel>>2]|0;
       $496 = ((($495)) + 44|0);
       $497 = HEAP8[$496>>0]|0;
       $498 = $497&255;
       $499 = HEAP32[$2>>2]|0;
       $500 = ((($499)) + 8|0);
       $501 = (($500) + (($498*72)|0)|0);
       HEAP32[$pTabItem>>2] = $501;
       $502 = HEAP32[$pTabItem>>2]|0;
       $503 = ((($502)) + 16|0);
       $504 = HEAP32[$503>>2]|0;
       HEAP32[$pTab>>2] = $504;
       $505 = HEAP32[$db>>2]|0;
       $506 = HEAP32[$pTab>>2]|0;
       $507 = ((($506)) + 64|0);
       $508 = HEAP32[$507>>2]|0;
       $509 = (_sqlite3SchemaToIndex($505,$508)|0);
       HEAP32[$iDb>>2] = $509;
       $510 = HEAP32[$pLevel>>2]|0;
       $511 = ((($510)) + 64|0);
       $512 = HEAP32[$511>>2]|0;
       HEAP32[$pLoop>>2] = $512;
       $513 = HEAP32[$pTab>>2]|0;
       $514 = ((($513)) + 42|0);
       $515 = HEAP8[$514>>0]|0;
       $516 = $515&255;
       $517 = $516 & 2;
       $518 = ($517|0)!=(0);
       L97: do {
        if (!($518)) {
         $519 = HEAP32[$pTab>>2]|0;
         $520 = ((($519)) + 12|0);
         $521 = HEAP32[$520>>2]|0;
         $522 = ($521|0)!=(0|0);
         if (!($522)) {
          $523 = HEAP32[$pLoop>>2]|0;
          $524 = ((($523)) + 36|0);
          $525 = HEAP32[$524>>2]|0;
          $526 = $525 & 1024;
          $527 = ($526|0)!=(0);
          if ($527) {
           $528 = HEAP32[$db>>2]|0;
           $529 = HEAP32[$pTab>>2]|0;
           $530 = (_sqlite3GetVTable($528,$529)|0);
           HEAP32[$pVTab>>2] = $530;
           $531 = HEAP32[$pTabItem>>2]|0;
           $532 = ((($531)) + 44|0);
           $533 = HEAP32[$532>>2]|0;
           HEAP32[$iCur>>2] = $533;
           $534 = HEAP32[$v>>2]|0;
           $535 = HEAP32[$iCur>>2]|0;
           $536 = HEAP32[$pVTab>>2]|0;
           (_sqlite3VdbeAddOp4($534,155,$535,0,0,$536,-10)|0);
           break;
          }
          $537 = HEAP32[$pTab>>2]|0;
          $538 = ((($537)) + 42|0);
          $539 = HEAP8[$538>>0]|0;
          $540 = $539&255;
          $541 = $540 & 16;
          $542 = ($541|0)!=(0);
          if ($542) {
           break;
          }
          $543 = HEAP32[$pLoop>>2]|0;
          $544 = ((($543)) + 36|0);
          $545 = HEAP32[$544>>2]|0;
          $546 = $545 & 64;
          $547 = ($546|0)==(0);
          do {
           if ($547) {
            $548 = HEAP16[$6>>1]|0;
            $549 = $548&65535;
            $550 = $549 & 32;
            $551 = ($550|0)==(0);
            if (!($551)) {
             break;
            }
            HEAP32[$op>>2] = 104;
            $552 = HEAP32[$pWInfo>>2]|0;
            $553 = ((($552)) + 45|0);
            $554 = HEAP8[$553>>0]|0;
            $555 = $554&255;
            $556 = ($555|0)!=(0);
            if ($556) {
             HEAP32[$op>>2] = 105;
             $557 = HEAP32[$pTabItem>>2]|0;
             $558 = ((($557)) + 44|0);
             $559 = HEAP32[$558>>2]|0;
             $560 = HEAP32[$pWInfo>>2]|0;
             $561 = ((($560)) + 20|0);
             HEAP32[$561>>2] = $559;
            }
            $562 = HEAP32[$1>>2]|0;
            $563 = HEAP32[$pTabItem>>2]|0;
            $564 = ((($563)) + 44|0);
            $565 = HEAP32[$564>>2]|0;
            $566 = HEAP32[$iDb>>2]|0;
            $567 = HEAP32[$pTab>>2]|0;
            $568 = HEAP32[$op>>2]|0;
            _sqlite3OpenTable($562,$565,$566,$567,$568);
            $569 = HEAP32[$pWInfo>>2]|0;
            $570 = ((($569)) + 45|0);
            $571 = HEAP8[$570>>0]|0;
            $572 = $571&255;
            $573 = ($572|0)==(0);
            do {
             if ($573) {
              $574 = HEAP32[$pTab>>2]|0;
              $575 = ((($574)) + 34|0);
              $576 = HEAP16[$575>>1]|0;
              $577 = $576 << 16 >> 16;
              $578 = ($577|0)<(64);
              if (!($578)) {
               break;
              }
              $579 = HEAP32[$pTab>>2]|0;
              $580 = ((($579)) + 42|0);
              $581 = HEAP8[$580>>0]|0;
              $582 = $581&255;
              $583 = $582 & 32;
              $584 = ($583|0)==(0);
              if (!($584)) {
               break;
              }
              $585 = HEAP32[$pTabItem>>2]|0;
              $586 = ((($585)) + 56|0);
              $587 = $586;
              $588 = $587;
              $589 = HEAP32[$588>>2]|0;
              $590 = (($587) + 4)|0;
              $591 = $590;
              $592 = HEAP32[$591>>2]|0;
              $593 = $b;
              $594 = $593;
              HEAP32[$594>>2] = $589;
              $595 = (($593) + 4)|0;
              $596 = $595;
              HEAP32[$596>>2] = $592;
              HEAP32[$n>>2] = 0;
              while(1) {
               $597 = $b;
               $598 = $597;
               $599 = HEAP32[$598>>2]|0;
               $600 = (($597) + 4)|0;
               $601 = $600;
               $602 = HEAP32[$601>>2]|0;
               $603 = ($599|0)!=(0);
               $604 = ($602|0)!=(0);
               $605 = $603 | $604;
               if (!($605)) {
                break;
               }
               $606 = $b;
               $607 = $606;
               $608 = HEAP32[$607>>2]|0;
               $609 = (($606) + 4)|0;
               $610 = $609;
               $611 = HEAP32[$610>>2]|0;
               $612 = (_bitshift64Lshr(($608|0),($611|0),1)|0);
               $613 = tempRet0;
               $614 = $b;
               $615 = $614;
               HEAP32[$615>>2] = $612;
               $616 = (($614) + 4)|0;
               $617 = $616;
               HEAP32[$617>>2] = $613;
               $618 = HEAP32[$n>>2]|0;
               $619 = (($618) + 1)|0;
               HEAP32[$n>>2] = $619;
              }
              $620 = HEAP32[$v>>2]|0;
              $621 = HEAP32[$n>>2]|0;
              $622 = $621;
              _sqlite3VdbeChangeP4($620,-1,$622,-14);
             }
            } while(0);
            $623 = HEAP32[$v>>2]|0;
            $624 = HEAP8[$bFordelete>>0]|0;
            _sqlite3VdbeChangeP5($623,$624);
            break L97;
           }
          } while(0);
          $625 = HEAP32[$1>>2]|0;
          $626 = HEAP32[$iDb>>2]|0;
          $627 = HEAP32[$pTab>>2]|0;
          $628 = ((($627)) + 28|0);
          $629 = HEAP32[$628>>2]|0;
          $630 = HEAP32[$pTab>>2]|0;
          $631 = HEAP32[$630>>2]|0;
          _sqlite3TableLock($625,$626,$629,0,$631);
         }
        }
       } while(0);
       $632 = HEAP32[$pLoop>>2]|0;
       $633 = ((($632)) + 36|0);
       $634 = HEAP32[$633>>2]|0;
       $635 = $634 & 512;
       $636 = ($635|0)!=(0);
       do {
        if ($636) {
         $637 = HEAP32[$pLoop>>2]|0;
         $638 = ((($637)) + 24|0);
         $639 = ((($638)) + 8|0);
         $640 = HEAP32[$639>>2]|0;
         HEAP32[$pIx>>2] = $640;
         HEAP32[$op1>>2] = 104;
         $641 = HEAP32[$pTab>>2]|0;
         $642 = ((($641)) + 42|0);
         $643 = HEAP8[$642>>0]|0;
         $644 = $643&255;
         $645 = $644 & 32;
         $646 = ($645|0)==(0);
         do {
          if ($646) {
           label = 93;
          } else {
           $647 = HEAP32[$pIx>>2]|0;
           $648 = ((($647)) + 55|0);
           $649 = HEAP8[$648>>0]|0;
           $650 = $649 & 3;
           $651 = $650&255;
           $652 = ($651|0)==(2);
           if (!($652)) {
            label = 93;
            break;
           }
           $653 = HEAP16[$6>>1]|0;
           $654 = $653&65535;
           $655 = $654 & 32;
           $656 = ($655|0)!=(0);
           if (!($656)) {
            label = 93;
            break;
           }
           $657 = HEAP32[$pLevel>>2]|0;
           $658 = ((($657)) + 4|0);
           $659 = HEAP32[$658>>2]|0;
           HEAP32[$iIndexCur>>2] = $659;
           HEAP32[$op1>>2] = 0;
          }
         } while(0);
         L127: do {
          if ((label|0) == 93) {
           label = 0;
           $660 = HEAP32[$pWInfo>>2]|0;
           $661 = ((($660)) + 45|0);
           $662 = HEAP8[$661>>0]|0;
           $663 = $662&255;
           $664 = ($663|0)!=(0);
           if (!($664)) {
            $685 = HEAP32[$7>>2]|0;
            $686 = ($685|0)!=(0);
            do {
             if ($686) {
              $687 = HEAP16[$6>>1]|0;
              $688 = $687&65535;
              $689 = $688 & 32;
              $690 = ($689|0)!=(0);
              if (!($690)) {
               break;
              }
              $691 = HEAP32[$7>>2]|0;
              HEAP32[$iIndexCur>>2] = $691;
              HEAP32[$op1>>2] = 103;
              break L127;
             }
            } while(0);
            $692 = HEAP32[$1>>2]|0;
            $693 = ((($692)) + 40|0);
            $694 = HEAP32[$693>>2]|0;
            $695 = (($694) + 1)|0;
            HEAP32[$693>>2] = $695;
            HEAP32[$iIndexCur>>2] = $694;
            break;
           }
           $665 = HEAP32[$pTabItem>>2]|0;
           $666 = ((($665)) + 16|0);
           $667 = HEAP32[$666>>2]|0;
           $668 = ((($667)) + 8|0);
           $669 = HEAP32[$668>>2]|0;
           HEAP32[$pJ>>2] = $669;
           $670 = HEAP32[$7>>2]|0;
           HEAP32[$iIndexCur>>2] = $670;
           while(1) {
            $671 = HEAP32[$pJ>>2]|0;
            $672 = ($671|0)!=(0|0);
            if (!($672)) {
             break;
            }
            $673 = HEAP32[$pJ>>2]|0;
            $674 = HEAP32[$pIx>>2]|0;
            $675 = ($673|0)!=($674|0);
            if (!($675)) {
             break;
            }
            $676 = HEAP32[$iIndexCur>>2]|0;
            $677 = (($676) + 1)|0;
            HEAP32[$iIndexCur>>2] = $677;
            $678 = HEAP32[$pJ>>2]|0;
            $679 = ((($678)) + 20|0);
            $680 = HEAP32[$679>>2]|0;
            HEAP32[$pJ>>2] = $680;
           }
           HEAP32[$op1>>2] = 105;
           $681 = HEAP32[$iIndexCur>>2]|0;
           $682 = HEAP32[$pWInfo>>2]|0;
           $683 = ((($682)) + 20|0);
           $684 = ((($683)) + 4|0);
           HEAP32[$684>>2] = $681;
          }
         } while(0);
         $696 = HEAP32[$iIndexCur>>2]|0;
         $697 = HEAP32[$pLevel>>2]|0;
         $698 = ((($697)) + 8|0);
         HEAP32[$698>>2] = $696;
         $699 = HEAP32[$op1>>2]|0;
         $700 = ($699|0)!=(0);
         if (!($700)) {
          break;
         }
         $701 = HEAP32[$v>>2]|0;
         $702 = HEAP32[$op1>>2]|0;
         $703 = HEAP32[$iIndexCur>>2]|0;
         $704 = HEAP32[$pIx>>2]|0;
         $705 = ((($704)) + 44|0);
         $706 = HEAP32[$705>>2]|0;
         $707 = HEAP32[$iDb>>2]|0;
         (_sqlite3VdbeAddOp3($701,$702,$703,$706,$707)|0);
         $708 = HEAP32[$1>>2]|0;
         $709 = HEAP32[$pIx>>2]|0;
         _sqlite3VdbeSetP4KeyInfo($708,$709);
         $710 = HEAP32[$pLoop>>2]|0;
         $711 = ((($710)) + 36|0);
         $712 = HEAP32[$711>>2]|0;
         $713 = $712 & 15;
         $714 = ($713|0)!=(0);
         if (!($714)) {
          break;
         }
         $715 = HEAP32[$pLoop>>2]|0;
         $716 = ((($715)) + 36|0);
         $717 = HEAP32[$716>>2]|0;
         $718 = $717 & 32770;
         $719 = ($718|0)==(0);
         if (!($719)) {
          break;
         }
         $720 = HEAP32[$pWInfo>>2]|0;
         $721 = ((($720)) + 40|0);
         $722 = HEAP16[$721>>1]|0;
         $723 = $722&65535;
         $724 = $723 & 1;
         $725 = ($724|0)==(0);
         if (!($725)) {
          break;
         }
         $726 = HEAP32[$v>>2]|0;
         _sqlite3VdbeChangeP5($726,2);
        }
       } while(0);
       $727 = HEAP32[$iDb>>2]|0;
       $728 = ($727|0)>=(0);
       if ($728) {
        $729 = HEAP32[$1>>2]|0;
        $730 = HEAP32[$iDb>>2]|0;
        _sqlite3CodeVerifySchema($729,$730);
       }
       $731 = HEAP32[$ii>>2]|0;
       $732 = (($731) + 1)|0;
       HEAP32[$ii>>2] = $732;
       $733 = HEAP32[$pLevel>>2]|0;
       $734 = ((($733)) + 80|0);
       HEAP32[$pLevel>>2] = $734;
      }
      $735 = HEAP32[$v>>2]|0;
      $736 = (_sqlite3VdbeCurrentAddr($735)|0);
      $737 = HEAP32[$pWInfo>>2]|0;
      $738 = ((($737)) + 52|0);
      HEAP32[$738>>2] = $736;
      $739 = HEAP32[$db>>2]|0;
      $740 = ((($739)) + 69|0);
      $741 = HEAP8[$740>>0]|0;
      $742 = ($741<<24>>24)!=(0);
      if (!($742)) {
       $743 = $notReady;
       $744 = $743;
       HEAP32[$744>>2] = -1;
       $745 = (($743) + 4)|0;
       $746 = $745;
       HEAP32[$746>>2] = -1;
       HEAP32[$ii>>2] = 0;
       while(1) {
        $747 = HEAP32[$ii>>2]|0;
        $748 = HEAP32[$nTabList>>2]|0;
        $749 = ($747|0)<($748|0);
        if (!($749)) {
         break;
        }
        $750 = HEAP32[$ii>>2]|0;
        $751 = HEAP32[$pWInfo>>2]|0;
        $752 = ((($751)) + 752|0);
        $753 = (($752) + (($750*80)|0)|0);
        HEAP32[$pLevel>>2] = $753;
        $754 = HEAP32[$pLevel>>2]|0;
        $755 = ((($754)) + 64|0);
        $756 = HEAP32[$755>>2]|0;
        $757 = ((($756)) + 36|0);
        $758 = HEAP32[$757>>2]|0;
        HEAP32[$wsFlags2>>2] = $758;
        $759 = HEAP32[$pLevel>>2]|0;
        $760 = ((($759)) + 64|0);
        $761 = HEAP32[$760>>2]|0;
        $762 = ((($761)) + 36|0);
        $763 = HEAP32[$762>>2]|0;
        $764 = $763 & 16384;
        $765 = ($764|0)!=(0);
        if ($765) {
         $766 = HEAP32[$1>>2]|0;
         $767 = HEAP32[$pWInfo>>2]|0;
         $768 = ((($767)) + 80|0);
         $769 = HEAP32[$pLevel>>2]|0;
         $770 = ((($769)) + 44|0);
         $771 = HEAP8[$770>>0]|0;
         $772 = $771&255;
         $773 = HEAP32[$2>>2]|0;
         $774 = ((($773)) + 8|0);
         $775 = (($774) + (($772*72)|0)|0);
         $776 = $notReady;
         $777 = $776;
         $778 = HEAP32[$777>>2]|0;
         $779 = (($776) + 4)|0;
         $780 = $779;
         $781 = HEAP32[$780>>2]|0;
         $782 = HEAP32[$pLevel>>2]|0;
         _constructAutomaticIndex($766,$768,$775,$778,$781,$782);
         $783 = HEAP32[$db>>2]|0;
         $784 = ((($783)) + 69|0);
         $785 = HEAP8[$784>>0]|0;
         $786 = ($785<<24>>24)!=(0);
         if ($786) {
          break L15;
         }
        }
        $787 = HEAP32[$1>>2]|0;
        $788 = HEAP32[$2>>2]|0;
        $789 = HEAP32[$pLevel>>2]|0;
        $790 = HEAP32[$ii>>2]|0;
        $791 = HEAP32[$pLevel>>2]|0;
        $792 = ((($791)) + 44|0);
        $793 = HEAP8[$792>>0]|0;
        $794 = $793&255;
        $795 = HEAP16[$6>>1]|0;
        $796 = (_sqlite3WhereExplainOneScan($787,$788,$789,$790,$794,$795)|0);
        HEAP32[$addrExplain>>2] = $796;
        $797 = HEAP32[$v>>2]|0;
        $798 = (_sqlite3VdbeCurrentAddr($797)|0);
        $799 = HEAP32[$pLevel>>2]|0;
        $800 = ((($799)) + 32|0);
        HEAP32[$800>>2] = $798;
        $801 = HEAP32[$pWInfo>>2]|0;
        $802 = HEAP32[$ii>>2]|0;
        $803 = $notReady;
        $804 = $803;
        $805 = HEAP32[$804>>2]|0;
        $806 = (($803) + 4)|0;
        $807 = $806;
        $808 = HEAP32[$807>>2]|0;
        $809 = (_sqlite3WhereCodeOneLoopStart($801,$802,$805,$808)|0);
        $810 = tempRet0;
        $811 = $notReady;
        $812 = $811;
        HEAP32[$812>>2] = $809;
        $813 = (($811) + 4)|0;
        $814 = $813;
        HEAP32[$814>>2] = $810;
        $815 = HEAP32[$pLevel>>2]|0;
        $816 = ((($815)) + 24|0);
        $817 = HEAP32[$816>>2]|0;
        $818 = HEAP32[$pWInfo>>2]|0;
        $819 = ((($818)) + 28|0);
        HEAP32[$819>>2] = $817;
        $820 = HEAP32[$wsFlags2>>2]|0;
        $821 = $820 & 8192;
        $822 = ($821|0)==(0);
        do {
         if ($822) {
          $823 = HEAP16[$6>>1]|0;
          $824 = $823&65535;
          $825 = $824 & 32;
          $826 = ($825|0)==(0);
          if (!($826)) {
           break;
          }
         }
        } while(0);
        $827 = HEAP32[$ii>>2]|0;
        $828 = (($827) + 1)|0;
        HEAP32[$ii>>2] = $828;
       }
       $829 = HEAP32[$pWInfo>>2]|0;
       HEAP32[$0>>2] = $829;
       $839 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($839|0);
      }
     }
    }
   }
  }
 } while(0);
 $830 = HEAP32[$pWInfo>>2]|0;
 $831 = ($830|0)!=(0|0);
 if ($831) {
  $832 = HEAP32[$pWInfo>>2]|0;
  $833 = ((($832)) + 36|0);
  $834 = HEAP32[$833>>2]|0;
  $835 = HEAP32[$1>>2]|0;
  $836 = ((($835)) + 136|0);
  HEAP32[$836>>2] = $834;
  $837 = HEAP32[$db>>2]|0;
  $838 = HEAP32[$pWInfo>>2]|0;
  _whereInfoFree($837,$838);
 }
 HEAP32[$0>>2] = 0;
 $839 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($839|0);
}
function _sqlite3WhereEnd($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $addr = 0, $db = 0, $i = 0, $j = 0, $k = 0, $last = 0, $pIdx = 0, $pIn = 0, $pLevel = 0, $pLoop = 0, $pOp = 0, $pParse = 0, $pPk = 0, $pTab = 0, $pTabItem = 0, $pTabList = 0, $v = 0;
 var $ws = 0, $ws1 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $pParse = sp + 76|0;
 $v = sp + 72|0;
 $i = sp + 68|0;
 $pLevel = sp + 64|0;
 $pLoop = sp + 60|0;
 $pTabList = sp + 56|0;
 $db = sp + 52|0;
 $addr = sp + 48|0;
 $pIn = sp + 44|0;
 $j = sp + 40|0;
 $ws = sp + 36|0;
 $k = sp + 32|0;
 $last = sp + 28|0;
 $pOp = sp + 24|0;
 $pIdx = sp + 20|0;
 $pTabItem = sp + 16|0;
 $pTab = sp + 12|0;
 $ws1 = sp + 8|0;
 $x = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pParse>>2] = $2;
 $3 = HEAP32[$pParse>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTabList>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$pParse>>2]|0;
 _sqlite3ExprCacheClear($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 42|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = (($15) - 1)|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = ($17|0)>=(0);
  if (!($18)) {
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 752|0);
  $22 = (($21) + (($19*80)|0)|0);
  HEAP32[$pLevel>>2] = $22;
  $23 = HEAP32[$pLevel>>2]|0;
  $24 = ((($23)) + 64|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pLoop>>2] = $25;
  $26 = HEAP32[$v>>2]|0;
  $27 = HEAP32[$pLevel>>2]|0;
  $28 = ((($27)) + 24|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3VdbeResolveLabel($26,$29);
  $30 = HEAP32[$pLevel>>2]|0;
  $31 = ((($30)) + 45|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = ($33|0)!=(161);
  if ($34) {
   $35 = HEAP32[$v>>2]|0;
   $36 = HEAP32[$pLevel>>2]|0;
   $37 = ((($36)) + 45|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   $40 = HEAP32[$pLevel>>2]|0;
   $41 = ((($40)) + 48|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$pLevel>>2]|0;
   $44 = ((($43)) + 52|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$pLevel>>2]|0;
   $47 = ((($46)) + 46|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48&255;
   (_sqlite3VdbeAddOp3($35,$39,$42,$45,$49)|0);
   $50 = HEAP32[$v>>2]|0;
   $51 = HEAP32[$pLevel>>2]|0;
   $52 = ((($51)) + 47|0);
   $53 = HEAP8[$52>>0]|0;
   _sqlite3VdbeChangeP5($50,$53);
  }
  $54 = HEAP32[$pLoop>>2]|0;
  $55 = ((($54)) + 36|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = $56 & 2048;
  $58 = ($57|0)!=(0);
  L7: do {
   if ($58) {
    $59 = HEAP32[$pLevel>>2]|0;
    $60 = ((($59)) + 56|0);
    $61 = HEAP32[$60>>2]|0;
    $62 = ($61|0)>(0);
    if ($62) {
     $63 = HEAP32[$v>>2]|0;
     $64 = HEAP32[$pLevel>>2]|0;
     $65 = ((($64)) + 16|0);
     $66 = HEAP32[$65>>2]|0;
     _sqlite3VdbeResolveLabel($63,$66);
     $67 = HEAP32[$pLevel>>2]|0;
     $68 = ((($67)) + 56|0);
     $69 = HEAP32[$68>>2]|0;
     HEAP32[$j>>2] = $69;
     $70 = HEAP32[$j>>2]|0;
     $71 = (($70) - 1)|0;
     $72 = HEAP32[$pLevel>>2]|0;
     $73 = ((($72)) + 56|0);
     $74 = ((($73)) + 4|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = (($75) + (($71*12)|0)|0);
     HEAP32[$pIn>>2] = $76;
     while(1) {
      $77 = HEAP32[$j>>2]|0;
      $78 = ($77|0)>(0);
      if (!($78)) {
       break L7;
      }
      $79 = HEAP32[$v>>2]|0;
      $80 = HEAP32[$pIn>>2]|0;
      $81 = ((($80)) + 4|0);
      $82 = HEAP32[$81>>2]|0;
      $83 = (($82) + 1)|0;
      _sqlite3VdbeJumpHere($79,$83);
      $84 = HEAP32[$pIn>>2]|0;
      $85 = ((($84)) + 8|0);
      $86 = HEAP8[$85>>0]|0;
      $87 = $86&255;
      $88 = ($87|0)!=(161);
      if ($88) {
       $89 = HEAP32[$v>>2]|0;
       $90 = HEAP32[$pIn>>2]|0;
       $91 = ((($90)) + 8|0);
       $92 = HEAP8[$91>>0]|0;
       $93 = $92&255;
       $94 = HEAP32[$pIn>>2]|0;
       $95 = HEAP32[$94>>2]|0;
       $96 = HEAP32[$pIn>>2]|0;
       $97 = ((($96)) + 4|0);
       $98 = HEAP32[$97>>2]|0;
       (_sqlite3VdbeAddOp2($89,$93,$95,$98)|0);
      }
      $99 = HEAP32[$v>>2]|0;
      $100 = HEAP32[$pIn>>2]|0;
      $101 = ((($100)) + 4|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = (($102) - 1)|0;
      _sqlite3VdbeJumpHere($99,$103);
      $104 = HEAP32[$j>>2]|0;
      $105 = (($104) + -1)|0;
      HEAP32[$j>>2] = $105;
      $106 = HEAP32[$pIn>>2]|0;
      $107 = ((($106)) + -12|0);
      HEAP32[$pIn>>2] = $107;
     }
    }
   }
  } while(0);
  $108 = HEAP32[$v>>2]|0;
  $109 = HEAP32[$pLevel>>2]|0;
  $110 = ((($109)) + 12|0);
  $111 = HEAP32[$110>>2]|0;
  _sqlite3VdbeResolveLabel($108,$111);
  $112 = HEAP32[$pLevel>>2]|0;
  $113 = ((($112)) + 20|0);
  $114 = HEAP32[$113>>2]|0;
  $115 = ($114|0)!=(0);
  if ($115) {
   $116 = HEAP32[$v>>2]|0;
   $117 = HEAP32[$pLevel>>2]|0;
   $118 = ((($117)) + 20|0);
   $119 = HEAP32[$118>>2]|0;
   (_sqlite3VdbeGoto($116,$119)|0);
   $120 = HEAP32[$v>>2]|0;
   $121 = HEAP32[$pLevel>>2]|0;
   $122 = ((($121)) + 20|0);
   $123 = HEAP32[$122>>2]|0;
   _sqlite3VdbeJumpHere($120,$123);
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$pLevel>>2]|0;
   $126 = ((($125)) + 20|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = (($127) - 2)|0;
   _sqlite3VdbeJumpHere($124,$128);
  }
  $129 = HEAP32[$pLevel>>2]|0;
  $130 = ((($129)) + 40|0);
  $131 = HEAP32[$130>>2]|0;
  $132 = ($131|0)!=(0);
  if ($132) {
   $133 = HEAP32[$v>>2]|0;
   $134 = HEAP32[$pLevel>>2]|0;
   $135 = ((($134)) + 36|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = $136 >>> 1;
   $138 = HEAP32[$pLevel>>2]|0;
   $139 = ((($138)) + 40|0);
   $140 = HEAP32[$139>>2]|0;
   (_sqlite3VdbeAddOp2($133,68,$137,$140)|0);
  }
  $141 = HEAP32[$pLevel>>2]|0;
  $142 = HEAP32[$141>>2]|0;
  $143 = ($142|0)!=(0);
  if ($143) {
   $144 = HEAP32[$pLoop>>2]|0;
   $145 = ((($144)) + 36|0);
   $146 = HEAP32[$145>>2]|0;
   HEAP32[$ws>>2] = $146;
   $147 = HEAP32[$v>>2]|0;
   $148 = HEAP32[$pLevel>>2]|0;
   $149 = HEAP32[$148>>2]|0;
   $150 = (_sqlite3VdbeAddOp1($147,66,$149)|0);
   HEAP32[$addr>>2] = $150;
   $151 = HEAP32[$ws>>2]|0;
   $152 = $151 & 64;
   $153 = ($152|0)==(0);
   if ($153) {
    $154 = HEAP32[$v>>2]|0;
    $155 = HEAP32[$i>>2]|0;
    $156 = HEAP32[$pTabList>>2]|0;
    $157 = ((($156)) + 8|0);
    $158 = (($157) + (($155*72)|0)|0);
    $159 = ((($158)) + 44|0);
    $160 = HEAP32[$159>>2]|0;
    (_sqlite3VdbeAddOp1($154,124,$160)|0);
   }
   $161 = HEAP32[$ws>>2]|0;
   $162 = $161 & 512;
   $163 = ($162|0)!=(0);
   if ($163) {
    label = 22;
   } else {
    $164 = HEAP32[$ws>>2]|0;
    $165 = $164 & 8192;
    $166 = ($165|0)!=(0);
    if ($166) {
     $167 = HEAP32[$pLevel>>2]|0;
     $168 = ((($167)) + 56|0);
     $169 = HEAP32[$168>>2]|0;
     $170 = ($169|0)!=(0|0);
     if ($170) {
      label = 22;
     }
    }
   }
   if ((label|0) == 22) {
    label = 0;
    $171 = HEAP32[$v>>2]|0;
    $172 = HEAP32[$pLevel>>2]|0;
    $173 = ((($172)) + 8|0);
    $174 = HEAP32[$173>>2]|0;
    (_sqlite3VdbeAddOp1($171,124,$174)|0);
   }
   $175 = HEAP32[$pLevel>>2]|0;
   $176 = ((($175)) + 45|0);
   $177 = HEAP8[$176>>0]|0;
   $178 = $177&255;
   $179 = ($178|0)==(72);
   $180 = HEAP32[$v>>2]|0;
   $181 = HEAP32[$pLevel>>2]|0;
   if ($179) {
    $182 = ((($181)) + 48|0);
    $183 = HEAP32[$182>>2]|0;
    $184 = HEAP32[$pLevel>>2]|0;
    $185 = ((($184)) + 28|0);
    $186 = HEAP32[$185>>2]|0;
    (_sqlite3VdbeAddOp2($180,14,$183,$186)|0);
   } else {
    $187 = ((($181)) + 28|0);
    $188 = HEAP32[$187>>2]|0;
    (_sqlite3VdbeGoto($180,$188)|0);
   }
   $189 = HEAP32[$v>>2]|0;
   $190 = HEAP32[$addr>>2]|0;
   _sqlite3VdbeJumpHere($189,$190);
  }
  $191 = HEAP32[$i>>2]|0;
  $192 = (($191) + -1)|0;
  HEAP32[$i>>2] = $192;
 }
 $193 = HEAP32[$v>>2]|0;
 $194 = HEAP32[$0>>2]|0;
 $195 = ((($194)) + 32|0);
 $196 = HEAP32[$195>>2]|0;
 _sqlite3VdbeResolveLabel($193,$196);
 HEAP32[$i>>2] = 0;
 $197 = HEAP32[$0>>2]|0;
 $198 = ((($197)) + 752|0);
 HEAP32[$pLevel>>2] = $198;
 while(1) {
  $199 = HEAP32[$i>>2]|0;
  $200 = HEAP32[$0>>2]|0;
  $201 = ((($200)) + 42|0);
  $202 = HEAP8[$201>>0]|0;
  $203 = $202&255;
  $204 = ($199|0)<($203|0);
  if (!($204)) {
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  $205 = HEAP32[$pLevel>>2]|0;
  $206 = ((($205)) + 44|0);
  $207 = HEAP8[$206>>0]|0;
  $208 = $207&255;
  $209 = HEAP32[$pTabList>>2]|0;
  $210 = ((($209)) + 8|0);
  $211 = (($210) + (($208*72)|0)|0);
  HEAP32[$pTabItem>>2] = $211;
  $212 = HEAP32[$pTabItem>>2]|0;
  $213 = ((($212)) + 16|0);
  $214 = HEAP32[$213>>2]|0;
  HEAP32[$pTab>>2] = $214;
  $215 = HEAP32[$pLevel>>2]|0;
  $216 = ((($215)) + 64|0);
  $217 = HEAP32[$216>>2]|0;
  HEAP32[$pLoop>>2] = $217;
  $218 = HEAP32[$pTabItem>>2]|0;
  $219 = ((($218)) + 36|0);
  $220 = ((($219)) + 1|0);
  $221 = HEAP8[$220>>0]|0;
  $222 = ($221&255) >>> 4;
  $223 = $222 & 1;
  $224 = $223&255;
  $225 = ($224|0)!=(0);
  if ($225) {
   $226 = HEAP32[$db>>2]|0;
   $227 = ((($226)) + 69|0);
   $228 = HEAP8[$227>>0]|0;
   $229 = ($228<<24>>24)!=(0);
   if ($229) {
    label = 33;
   } else {
    $230 = HEAP32[$v>>2]|0;
    $231 = HEAP32[$pLevel>>2]|0;
    $232 = ((($231)) + 32|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = HEAP32[$pLevel>>2]|0;
    $235 = ((($234)) + 4|0);
    $236 = HEAP32[$235>>2]|0;
    $237 = HEAP32[$pTabItem>>2]|0;
    $238 = ((($237)) + 32|0);
    $239 = HEAP32[$238>>2]|0;
    _translateColumnToCopy($230,$233,$236,$239,0);
   }
  } else {
   label = 33;
  }
  L46: do {
   if ((label|0) == 33) {
    label = 0;
    $240 = HEAP32[$pTab>>2]|0;
    $241 = ((($240)) + 42|0);
    $242 = HEAP8[$241>>0]|0;
    $243 = $242&255;
    $244 = $243 & 2;
    $245 = ($244|0)==(0);
    if ($245) {
     $246 = HEAP32[$pTab>>2]|0;
     $247 = ((($246)) + 12|0);
     $248 = HEAP32[$247>>2]|0;
     $249 = ($248|0)==(0|0);
     if ($249) {
      $250 = HEAP32[$0>>2]|0;
      $251 = ((($250)) + 40|0);
      $252 = HEAP16[$251>>1]|0;
      $253 = $252&65535;
      $254 = $253 & 32;
      $255 = ($254|0)==(0);
      if ($255) {
       $256 = HEAP32[$pLoop>>2]|0;
       $257 = ((($256)) + 36|0);
       $258 = HEAP32[$257>>2]|0;
       HEAP32[$ws1>>2] = $258;
       $259 = HEAP32[$0>>2]|0;
       $260 = ((($259)) + 45|0);
       $261 = HEAP8[$260>>0]|0;
       $262 = $261&255;
       $263 = ($262|0)==(0);
       if ($263) {
        $264 = HEAP32[$ws1>>2]|0;
        $265 = $264 & 64;
        $266 = ($265|0)==(0);
        if ($266) {
         $267 = HEAP32[$v>>2]|0;
         $268 = HEAP32[$pTabItem>>2]|0;
         $269 = ((($268)) + 44|0);
         $270 = HEAP32[$269>>2]|0;
         (_sqlite3VdbeAddOp1($267,111,$270)|0);
        }
       }
       $271 = HEAP32[$ws1>>2]|0;
       $272 = $271 & 512;
       $273 = ($272|0)!=(0);
       if ($273) {
        $274 = HEAP32[$ws1>>2]|0;
        $275 = $274 & 16640;
        $276 = ($275|0)==(0);
        if ($276) {
         $277 = HEAP32[$pLevel>>2]|0;
         $278 = ((($277)) + 8|0);
         $279 = HEAP32[$278>>2]|0;
         $280 = HEAP32[$0>>2]|0;
         $281 = ((($280)) + 20|0);
         $282 = ((($281)) + 4|0);
         $283 = HEAP32[$282>>2]|0;
         $284 = ($279|0)!=($283|0);
         if ($284) {
          $285 = HEAP32[$v>>2]|0;
          $286 = HEAP32[$pLevel>>2]|0;
          $287 = ((($286)) + 8|0);
          $288 = HEAP32[$287>>2]|0;
          (_sqlite3VdbeAddOp1($285,111,$288)|0);
         }
        }
       }
      }
     }
    }
    $289 = HEAP32[$pLoop>>2]|0;
    $290 = ((($289)) + 36|0);
    $291 = HEAP32[$290>>2]|0;
    $292 = $291 & 576;
    $293 = ($292|0)!=(0);
    $294 = HEAP32[$pLoop>>2]|0;
    if ($293) {
     $295 = ((($294)) + 24|0);
     $296 = ((($295)) + 8|0);
     $297 = HEAP32[$296>>2]|0;
     HEAP32[$pIdx>>2] = $297;
    } else {
     $298 = ((($294)) + 36|0);
     $299 = HEAP32[$298>>2]|0;
     $300 = $299 & 8192;
     $301 = ($300|0)!=(0);
     if ($301) {
      $302 = HEAP32[$pLevel>>2]|0;
      $303 = ((($302)) + 56|0);
      $304 = HEAP32[$303>>2]|0;
      HEAP32[$pIdx>>2] = $304;
     }
    }
    $305 = HEAP32[$pIdx>>2]|0;
    $306 = ($305|0)!=(0|0);
    if ($306) {
     $307 = HEAP32[$0>>2]|0;
     $308 = ((($307)) + 45|0);
     $309 = HEAP8[$308>>0]|0;
     $310 = $309&255;
     $311 = ($310|0)==(0);
     if (!($311)) {
      $312 = HEAP32[$pIdx>>2]|0;
      $313 = ((($312)) + 12|0);
      $314 = HEAP32[$313>>2]|0;
      $315 = ((($314)) + 42|0);
      $316 = HEAP8[$315>>0]|0;
      $317 = $316&255;
      $318 = $317 & 32;
      $319 = ($318|0)==(0);
      if ($319) {
       break;
      }
     }
     $320 = HEAP32[$db>>2]|0;
     $321 = ((($320)) + 69|0);
     $322 = HEAP8[$321>>0]|0;
     $323 = ($322<<24>>24)!=(0);
     if (!($323)) {
      $324 = HEAP32[$v>>2]|0;
      $325 = (_sqlite3VdbeCurrentAddr($324)|0);
      HEAP32[$last>>2] = $325;
      $326 = HEAP32[$pLevel>>2]|0;
      $327 = ((($326)) + 32|0);
      $328 = HEAP32[$327>>2]|0;
      HEAP32[$k>>2] = $328;
      $329 = HEAP32[$v>>2]|0;
      $330 = HEAP32[$k>>2]|0;
      $331 = (_sqlite3VdbeGetOp($329,$330)|0);
      HEAP32[$pOp>>2] = $331;
      while(1) {
       $332 = HEAP32[$k>>2]|0;
       $333 = HEAP32[$last>>2]|0;
       $334 = ($332|0)<($333|0);
       if (!($334)) {
        break L46;
       }
       $335 = HEAP32[$pOp>>2]|0;
       $336 = ((($335)) + 4|0);
       $337 = HEAP32[$336>>2]|0;
       $338 = HEAP32[$pLevel>>2]|0;
       $339 = ((($338)) + 4|0);
       $340 = HEAP32[$339>>2]|0;
       $341 = ($337|0)!=($340|0);
       do {
        if (!($341)) {
         $342 = HEAP32[$pOp>>2]|0;
         $343 = HEAP8[$342>>0]|0;
         $344 = $343&255;
         $345 = ($344|0)==(96);
         $346 = HEAP32[$pOp>>2]|0;
         if (!($345)) {
          $379 = HEAP8[$346>>0]|0;
          $380 = $379&255;
          $381 = ($380|0)==(123);
          if (!($381)) {
           break;
          }
          $382 = HEAP32[$pLevel>>2]|0;
          $383 = ((($382)) + 8|0);
          $384 = HEAP32[$383>>2]|0;
          $385 = HEAP32[$pOp>>2]|0;
          $386 = ((($385)) + 4|0);
          HEAP32[$386>>2] = $384;
          $387 = HEAP32[$pOp>>2]|0;
          HEAP8[$387>>0] = -127;
          break;
         }
         $347 = ((($346)) + 8|0);
         $348 = HEAP32[$347>>2]|0;
         HEAP32[$x>>2] = $348;
         $349 = HEAP32[$pTab>>2]|0;
         $350 = ((($349)) + 42|0);
         $351 = HEAP8[$350>>0]|0;
         $352 = $351&255;
         $353 = $352 & 32;
         $354 = ($353|0)==(0);
         if (!($354)) {
          $355 = HEAP32[$pTab>>2]|0;
          $356 = (_sqlite3PrimaryKeyIndex($355)|0);
          HEAP32[$pPk>>2] = $356;
          $357 = HEAP32[$x>>2]|0;
          $358 = HEAP32[$pPk>>2]|0;
          $359 = ((($358)) + 4|0);
          $360 = HEAP32[$359>>2]|0;
          $361 = (($360) + ($357<<1)|0);
          $362 = HEAP16[$361>>1]|0;
          $363 = $362 << 16 >> 16;
          HEAP32[$x>>2] = $363;
         }
         $364 = HEAP32[$pIdx>>2]|0;
         $365 = HEAP32[$x>>2]|0;
         $366 = $365&65535;
         $367 = (_sqlite3ColumnOfIndex($364,$366)|0);
         $368 = $367 << 16 >> 16;
         HEAP32[$x>>2] = $368;
         $369 = HEAP32[$x>>2]|0;
         $370 = ($369|0)>=(0);
         if ($370) {
          $371 = HEAP32[$x>>2]|0;
          $372 = HEAP32[$pOp>>2]|0;
          $373 = ((($372)) + 8|0);
          HEAP32[$373>>2] = $371;
          $374 = HEAP32[$pLevel>>2]|0;
          $375 = ((($374)) + 8|0);
          $376 = HEAP32[$375>>2]|0;
          $377 = HEAP32[$pOp>>2]|0;
          $378 = ((($377)) + 4|0);
          HEAP32[$378>>2] = $376;
         }
        }
       } while(0);
       $388 = HEAP32[$k>>2]|0;
       $389 = (($388) + 1)|0;
       HEAP32[$k>>2] = $389;
       $390 = HEAP32[$pOp>>2]|0;
       $391 = ((($390)) + 20|0);
       HEAP32[$pOp>>2] = $391;
      }
     }
    }
   }
  } while(0);
  $392 = HEAP32[$i>>2]|0;
  $393 = (($392) + 1)|0;
  HEAP32[$i>>2] = $393;
  $394 = HEAP32[$pLevel>>2]|0;
  $395 = ((($394)) + 80|0);
  HEAP32[$pLevel>>2] = $395;
 }
 $396 = HEAP32[$0>>2]|0;
 $397 = ((($396)) + 36|0);
 $398 = HEAP32[$397>>2]|0;
 $399 = HEAP32[$pParse>>2]|0;
 $400 = ((($399)) + 136|0);
 HEAP32[$400>>2] = $398;
 $401 = HEAP32[$db>>2]|0;
 $402 = HEAP32[$0>>2]|0;
 _whereInfoFree($401,$402);
 STACKTOP = sp;return;
}
function _translateColumnToCopy($v,$iStart,$iTabCur,$iRegister,$bIncrRowid) {
 $v = $v|0;
 $iStart = $iStart|0;
 $iTabCur = $iTabCur|0;
 $iRegister = $iRegister|0;
 $bIncrRowid = $bIncrRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iEnd = 0, $pOp = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $pOp = sp + 4|0;
 $iEnd = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iStart;
 HEAP32[$2>>2] = $iTabCur;
 HEAP32[$3>>2] = $iRegister;
 HEAP32[$4>>2] = $bIncrRowid;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_sqlite3VdbeGetOp($5,$6)|0);
 HEAP32[$pOp>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3VdbeCurrentAddr($8)|0);
 HEAP32[$iEnd>>2] = $9;
 while(1) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$iEnd>>2]|0;
  $12 = ($10|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$pOp>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = ($15|0)!=($16|0);
  do {
   if (!($17)) {
    $18 = HEAP32[$pOp>>2]|0;
    $19 = HEAP8[$18>>0]|0;
    $20 = $19&255;
    $21 = ($20|0)==(96);
    $22 = HEAP32[$pOp>>2]|0;
    if ($21) {
     HEAP8[$22>>0] = 84;
     $23 = HEAP32[$pOp>>2]|0;
     $24 = ((($23)) + 8|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = HEAP32[$3>>2]|0;
     $27 = (($25) + ($26))|0;
     $28 = HEAP32[$pOp>>2]|0;
     $29 = ((($28)) + 4|0);
     HEAP32[$29>>2] = $27;
     $30 = HEAP32[$pOp>>2]|0;
     $31 = ((($30)) + 12|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = HEAP32[$pOp>>2]|0;
     $34 = ((($33)) + 8|0);
     HEAP32[$34>>2] = $32;
     $35 = HEAP32[$pOp>>2]|0;
     $36 = ((($35)) + 12|0);
     HEAP32[$36>>2] = 0;
     break;
    }
    $37 = HEAP8[$22>>0]|0;
    $38 = $37&255;
    $39 = ($38|0)==(123);
    if ($39) {
     $40 = HEAP32[$4>>2]|0;
     $41 = ($40|0)!=(0);
     $42 = HEAP32[$pOp>>2]|0;
     if ($41) {
      HEAP8[$42>>0] = 91;
      $43 = HEAP32[$pOp>>2]|0;
      $44 = ((($43)) + 8|0);
      $45 = HEAP32[$44>>2]|0;
      $46 = HEAP32[$pOp>>2]|0;
      $47 = ((($46)) + 4|0);
      HEAP32[$47>>2] = $45;
      $48 = HEAP32[$pOp>>2]|0;
      $49 = ((($48)) + 8|0);
      HEAP32[$49>>2] = 1;
      break;
     } else {
      HEAP8[$42>>0] = 79;
      $50 = HEAP32[$pOp>>2]|0;
      $51 = ((($50)) + 4|0);
      HEAP32[$51>>2] = 0;
      $52 = HEAP32[$pOp>>2]|0;
      $53 = ((($52)) + 12|0);
      HEAP32[$53>>2] = 0;
      break;
     }
    }
   }
  } while(0);
  $54 = HEAP32[$1>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$1>>2] = $55;
  $56 = HEAP32[$pOp>>2]|0;
  $57 = ((($56)) + 20|0);
  HEAP32[$pOp>>2] = $57;
 }
 STACKTOP = sp;return;
}
function _whereInfoFree($db,$pWInfo) {
 $db = $db|0;
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pLevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $pLevel = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pWInfo;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $4 = HEAP32[$i>>2]|0;
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 42|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 752|0);
  $13 = (($12) + (($10*80)|0)|0);
  HEAP32[$pLevel>>2] = $13;
  $14 = HEAP32[$pLevel>>2]|0;
  $15 = ((($14)) + 64|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$pLevel>>2]|0;
   $19 = ((($18)) + 64|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ((($20)) + 36|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = $22 & 2048;
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$0>>2]|0;
    $26 = HEAP32[$pLevel>>2]|0;
    $27 = ((($26)) + 56|0);
    $28 = ((($27)) + 4|0);
    $29 = HEAP32[$28>>2]|0;
    _sqlite3DbFree($25,$29);
   }
  }
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + 1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 80|0);
 _sqlite3WhereClauseClear($33);
 while(1) {
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 56|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = ($36|0)!=(0|0);
  if (!($37)) {
   break;
  }
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 56|0);
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$p>>2] = $40;
  $41 = HEAP32[$p>>2]|0;
  $42 = ((($41)) + 52|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 56|0);
  HEAP32[$45>>2] = $43;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$p>>2]|0;
  _whereLoopDelete($46,$47);
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($48,$49);
 STACKTOP = sp;return;
}
function _sqlite3WhereClauseClear($pWC) {
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $db = 0, $i = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $a = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pWC;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) - 1)|0;
 HEAP32[$i>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$a>>2] = $11;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = ($12|0)>=(0);
  if (!($13)) {
   break;
  }
  $14 = HEAP32[$a>>2]|0;
  $15 = ((($14)) + 10|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16&65535;
  $18 = $17 & 1;
  $19 = ($18|0)!=(0);
  if ($19) {
   $20 = HEAP32[$db>>2]|0;
   $21 = HEAP32[$a>>2]|0;
   $22 = HEAP32[$21>>2]|0;
   _sqlite3ExprDelete($20,$22);
  }
  $23 = HEAP32[$a>>2]|0;
  $24 = ((($23)) + 10|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26 & 16;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$db>>2]|0;
   $30 = HEAP32[$a>>2]|0;
   $31 = ((($30)) + 28|0);
   $32 = HEAP32[$31>>2]|0;
   _whereOrInfoDelete($29,$32);
  } else {
   $33 = HEAP32[$a>>2]|0;
   $34 = ((($33)) + 10|0);
   $35 = HEAP16[$34>>1]|0;
   $36 = $35&65535;
   $37 = $36 & 32;
   $38 = ($37|0)!=(0);
   if ($38) {
    $39 = HEAP32[$db>>2]|0;
    $40 = HEAP32[$a>>2]|0;
    $41 = ((($40)) + 28|0);
    $42 = HEAP32[$41>>2]|0;
    _whereAndInfoDelete($39,$42);
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + -1)|0;
  HEAP32[$i>>2] = $44;
  $45 = HEAP32[$a>>2]|0;
  $46 = ((($45)) + 48|0);
  HEAP32[$a>>2] = $46;
 }
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 20|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = ($49|0)!=($51|0);
 if (!($52)) {
  STACKTOP = sp;return;
 }
 $53 = HEAP32[$db>>2]|0;
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 20|0);
 $56 = HEAP32[$55>>2]|0;
 _sqlite3DbFree($53,$56);
 STACKTOP = sp;return;
}
function _whereLoopDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _whereLoopClear($2,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($4,$5);
 STACKTOP = sp;return;
}
function _whereLoopClear($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = ($4|0)!=($6|0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 48|0);
  $11 = HEAP32[$10>>2]|0;
  _sqlite3DbFree($8,$11);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 _whereLoopClearUnion($12,$13);
 $14 = HEAP32[$1>>2]|0;
 _whereLoopInit($14);
 STACKTOP = sp;return;
}
function _whereLoopClearUnion($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 17408;
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 1024;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 24|0);
  $14 = ((($13)) + 4|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 24|0);
   $20 = ((($19)) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   _sqlite3_free($21);
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 24|0);
   $24 = ((($23)) + 4|0);
   HEAP8[$24>>0] = 0;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 24|0);
   $27 = ((($26)) + 8|0);
   HEAP32[$27>>2] = 0;
   STACKTOP = sp;return;
  }
 }
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 & 16384;
 $32 = ($31|0)!=(0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 24|0);
 $35 = ((($34)) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = ($36|0)!=(0|0);
 if (!($37)) {
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 24|0);
 $41 = ((($40)) + 8|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ((($42)) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 _sqlite3DbFree($38,$44);
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$1>>2]|0;
 $47 = ((($46)) + 24|0);
 $48 = ((($47)) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 _sqlite3DbFree($45,$49);
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = ((($51)) + 8|0);
 HEAP32[$52>>2] = 0;
 STACKTOP = sp;return;
}
function _whereLoopInit($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 56|0);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 48|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 40|0);
 HEAP16[$6>>1] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 HEAP16[$8>>1] = 3;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 36|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _whereOrInfoDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3WhereClauseClear($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($3,$4);
 STACKTOP = sp;return;
}
function _whereAndInfoDelete($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3WhereClauseClear($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($3,$4);
 STACKTOP = sp;return;
}
function _sqlite3WhereClauseInit($pWC,$pWInfo) {
 $pWC = $pWC|0;
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pWInfo;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 HEAP32[$3>>2] = $2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 4|0);
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 16|0);
 HEAP32[$9>>2] = 8;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 20|0);
 HEAP32[$13>>2] = $11;
 STACKTOP = sp;return;
}
function _sqlite3WhereSplit($pWC,$pExpr,$op) {
 $pWC = $pWC|0;
 $pExpr = $pExpr|0;
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pE2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 12|0;
 $pE2 = sp;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $op;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3ExprSkipCollate($3)|0);
 HEAP32[$pE2>>2] = $4;
 $5 = HEAP8[$2>>0]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 HEAP8[$7>>0] = $5;
 $8 = HEAP32[$pE2>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$pE2>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = HEAP8[$2>>0]|0;
 $14 = $13&255;
 $15 = ($12|0)!=($14|0);
 $16 = HEAP32[$0>>2]|0;
 if ($15) {
  $17 = HEAP32[$1>>2]|0;
  (_whereClauseInsert($16,$17,0)|0);
  STACKTOP = sp;return;
 } else {
  $18 = HEAP32[$pE2>>2]|0;
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP8[$2>>0]|0;
  _sqlite3WhereSplit($16,$20,$21);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$pE2>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP8[$2>>0]|0;
  _sqlite3WhereSplit($22,$25,$26);
  STACKTOP = sp;return;
 }
}
function _createMask($pMaskSet,$iCursor) {
 $pMaskSet = $pMaskSet|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $iCursor;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) + 1)|0;
 HEAP32[$3>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = (($7) + ($4<<2)|0);
 HEAP32[$8>>2] = $2;
 STACKTOP = sp;return;
}
function _sqlite3WhereTabFuncArgs($pParse,$pItem,$pWC) {
 $pParse = $pParse|0;
 $pItem = $pItem|0;
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $j = 0, $k = 0, $pArgs = 0, $pColRef = 0, $pTab = 0, $pTerm = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pTab = sp + 28|0;
 $j = sp + 24|0;
 $k = sp + 20|0;
 $pArgs = sp + 16|0;
 $pColRef = sp + 12|0;
 $pTerm = sp + 8|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pItem;
 HEAP32[$2>>2] = $pWC;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = ((($4)) + 1|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6&255) >>> 2;
 $8 = $7 & 1;
 $9 = $8&255;
 $10 = ($9|0)==(0);
 if ($10) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pTab>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 64|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pArgs>>2] = $16;
 $17 = HEAP32[$pArgs>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  STACKTOP = sp;return;
 }
 HEAP32[$k>>2] = 0;
 HEAP32[$j>>2] = 0;
 while(1) {
  $19 = HEAP32[$j>>2]|0;
  $20 = HEAP32[$pArgs>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ($19|0)<($21|0);
  if (!($22)) {
   label = 13;
   break;
  }
  while(1) {
   $23 = HEAP32[$k>>2]|0;
   $24 = HEAP32[$pTab>>2]|0;
   $25 = ((($24)) + 34|0);
   $26 = HEAP16[$25>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = ($23|0)<($27|0);
   if ($28) {
    $29 = HEAP32[$k>>2]|0;
    $30 = HEAP32[$pTab>>2]|0;
    $31 = ((($30)) + 4|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = (($32) + ($29<<4)|0);
    $34 = ((($33)) + 15|0);
    $35 = HEAP8[$34>>0]|0;
    $36 = $35&255;
    $37 = $36 & 2;
    $38 = ($37|0)==(0);
    $83 = $38;
   } else {
    $83 = 0;
   }
   $39 = HEAP32[$k>>2]|0;
   if (!($83)) {
    break;
   }
   $40 = (($39) + 1)|0;
   HEAP32[$k>>2] = $40;
  }
  $41 = HEAP32[$pTab>>2]|0;
  $42 = ((($41)) + 34|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43 << 16 >> 16;
  $45 = ($39|0)>=($44|0);
  $46 = HEAP32[$0>>2]|0;
  if ($45) {
   label = 10;
   break;
  }
  $50 = HEAP32[$46>>2]|0;
  $51 = (_sqlite3ExprAlloc($50,152,0,0)|0);
  HEAP32[$pColRef>>2] = $51;
  $52 = HEAP32[$pColRef>>2]|0;
  $53 = ($52|0)==(0|0);
  if ($53) {
   label = 13;
   break;
  }
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 44|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$pColRef>>2]|0;
  $58 = ((($57)) + 28|0);
  HEAP32[$58>>2] = $56;
  $59 = HEAP32[$k>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$k>>2] = $60;
  $61 = $59&65535;
  $62 = HEAP32[$pColRef>>2]|0;
  $63 = ((($62)) + 32|0);
  HEAP16[$63>>1] = $61;
  $64 = HEAP32[$pTab>>2]|0;
  $65 = HEAP32[$pColRef>>2]|0;
  $66 = ((($65)) + 44|0);
  HEAP32[$66>>2] = $64;
  $67 = HEAP32[$0>>2]|0;
  $68 = HEAP32[$pColRef>>2]|0;
  $69 = HEAP32[$0>>2]|0;
  $70 = HEAP32[$69>>2]|0;
  $71 = HEAP32[$j>>2]|0;
  $72 = HEAP32[$pArgs>>2]|0;
  $73 = ((($72)) + 4|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + (($71*20)|0)|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = (_sqlite3ExprDup($70,$76,0)|0);
  $78 = (_sqlite3PExpr($67,37,$68,$77,0)|0);
  HEAP32[$pTerm>>2] = $78;
  $79 = HEAP32[$2>>2]|0;
  $80 = HEAP32[$pTerm>>2]|0;
  (_whereClauseInsert($79,$80,1)|0);
  $81 = HEAP32[$j>>2]|0;
  $82 = (($81) + 1)|0;
  HEAP32[$j>>2] = $82;
 }
 if ((label|0) == 10) {
  $47 = HEAP32[$pTab>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = HEAP32[$j>>2]|0;
  HEAP32[$vararg_buffer>>2] = $48;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $49;
  _sqlite3ErrorMsg($46,34848,$vararg_buffer);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 13) {
  STACKTOP = sp;return;
 }
}
function _sqlite3WhereExprAnalyze($pTabList,$pWC) {
 $pTabList = $pTabList|0;
 $pWC = $pWC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTabList;
 HEAP32[$1>>2] = $pWC;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4) - 1)|0;
 HEAP32[$i>>2] = $5;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = ($6|0)>=(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  _exprAnalyze($8,$9,$10);
  $11 = HEAP32[$i>>2]|0;
  $12 = (($11) + -1)|0;
  HEAP32[$i>>2] = $12;
 }
 STACKTOP = sp;return;
}
function _isDistinctRedundant($pParse,$pTabList,$pWC,$pDistinct) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pWC = $pWC|0;
 $pDistinct = $pDistinct|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $i = 0, $iBase = 0, $p = 0, $pIdx = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pTab = sp + 16|0;
 $pIdx = sp + 12|0;
 $i = sp + 8|0;
 $iBase = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $pDistinct;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(1);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = ((($9)) + 44|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$iBase>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTab>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$4>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($16|0)<($18|0);
  if (!($19)) {
   break;
  }
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$4>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3ExprSkipCollate($25)|0);
  HEAP32[$p>>2] = $26;
  $27 = HEAP32[$p>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(152);
  if ($30) {
   $31 = HEAP32[$p>>2]|0;
   $32 = ((($31)) + 28|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$iBase>>2]|0;
   $35 = ($33|0)==($34|0);
   if ($35) {
    $36 = HEAP32[$p>>2]|0;
    $37 = ((($36)) + 32|0);
    $38 = HEAP16[$37>>1]|0;
    $39 = $38 << 16 >> 16;
    $40 = ($39|0)<(0);
    if ($40) {
     label = 8;
     break;
    }
   }
  }
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 1;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 $43 = HEAP32[$pTab>>2]|0;
 $44 = ((($43)) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 HEAP32[$pIdx>>2] = $45;
 while(1) {
  $46 = HEAP32[$pIdx>>2]|0;
  $47 = ($46|0)!=(0|0);
  if (!($47)) {
   label = 22;
   break;
  }
  $48 = HEAP32[$pIdx>>2]|0;
  $49 = ((($48)) + 54|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = ($51|0)!=(0);
  if ($52) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $53 = HEAP32[$i>>2]|0;
    $54 = HEAP32[$pIdx>>2]|0;
    $55 = ((($54)) + 50|0);
    $56 = HEAP16[$55>>1]|0;
    $57 = $56&65535;
    $58 = ($53|0)<($57|0);
    if (!($58)) {
     break;
    }
    $59 = HEAP32[$3>>2]|0;
    $60 = HEAP32[$iBase>>2]|0;
    $61 = HEAP32[$i>>2]|0;
    $62 = HEAP32[$pIdx>>2]|0;
    $63 = (_sqlite3WhereFindTerm($59,$60,$61,-1,-1,2,$62)|0);
    $64 = (0|0)==($63|0);
    if ($64) {
     $65 = HEAP32[$1>>2]|0;
     $66 = HEAP32[$4>>2]|0;
     $67 = HEAP32[$iBase>>2]|0;
     $68 = HEAP32[$pIdx>>2]|0;
     $69 = HEAP32[$i>>2]|0;
     $70 = (_findIndexCol($65,$66,$67,$68,$69)|0);
     $71 = ($70|0)<(0);
     if ($71) {
      break;
     }
     $72 = HEAP32[$pIdx>>2]|0;
     $73 = HEAP32[$i>>2]|0;
     $74 = (_indexColumnNotNull($72,$73)|0);
     $75 = ($74|0)==(0);
     if ($75) {
      break;
     }
    }
    $76 = HEAP32[$i>>2]|0;
    $77 = (($76) + 1)|0;
    HEAP32[$i>>2] = $77;
   }
   $78 = HEAP32[$i>>2]|0;
   $79 = HEAP32[$pIdx>>2]|0;
   $80 = ((($79)) + 50|0);
   $81 = HEAP16[$80>>1]|0;
   $82 = $81&65535;
   $83 = ($78|0)==($82|0);
   if ($83) {
    label = 20;
    break;
   }
  }
  $84 = HEAP32[$pIdx>>2]|0;
  $85 = ((($84)) + 20|0);
  $86 = HEAP32[$85>>2]|0;
  HEAP32[$pIdx>>2] = $86;
 }
 if ((label|0) == 20) {
  HEAP32[$0>>2] = 1;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 else if ((label|0) == 22) {
  HEAP32[$0>>2] = 0;
  $87 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($87|0);
 }
 return (0)|0;
}
function _whereShortCut($pBuilder) {
 $pBuilder = $pBuilder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iCur = 0, $j = 0, $opMask = 0, $pIdx = 0, $pItem = 0, $pLoop = 0, $pTab = 0, $pTerm = 0, $pWC = 0, $pWInfo = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $pWInfo = sp + 36|0;
 $pItem = sp + 32|0;
 $pWC = sp + 28|0;
 $pTerm = sp + 24|0;
 $pLoop = sp + 20|0;
 $iCur = sp + 16|0;
 $j = sp + 12|0;
 $pTab = sp + 8|0;
 $pIdx = sp + 4|0;
 $opMask = sp;
 HEAP32[$1>>2] = $pBuilder;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pWInfo>>2] = $3;
 $4 = HEAP32[$pWInfo>>2]|0;
 $5 = ((($4)) + 40|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 32;
 $9 = ($8|0)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $10 = HEAP32[$pWInfo>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 8|0);
 HEAP32[$pItem>>2] = $13;
 $14 = HEAP32[$pItem>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pTab>>2] = $16;
 $17 = HEAP32[$pTab>>2]|0;
 $18 = ((($17)) + 42|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = $20 & 16;
 $22 = ($21|0)!=(0);
 if ($22) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $23 = HEAP32[$pItem>>2]|0;
 $24 = ((($23)) + 36|0);
 $25 = ((($24)) + 1|0);
 $26 = HEAP8[$25>>0]|0;
 $27 = ($26&255) >>> 1;
 $28 = $27 & 1;
 $29 = $28&255;
 $30 = ($29|0)!=(0);
 if ($30) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $31 = HEAP32[$pItem>>2]|0;
 $32 = ((($31)) + 44|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$iCur>>2] = $33;
 $34 = HEAP32[$pWInfo>>2]|0;
 $35 = ((($34)) + 80|0);
 HEAP32[$pWC>>2] = $35;
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 12|0);
 $38 = HEAP32[$37>>2]|0;
 HEAP32[$pLoop>>2] = $38;
 $39 = HEAP32[$pLoop>>2]|0;
 $40 = ((($39)) + 36|0);
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$pLoop>>2]|0;
 $42 = ((($41)) + 42|0);
 HEAP16[$42>>1] = 0;
 $43 = HEAP32[$pWC>>2]|0;
 $44 = HEAP32[$iCur>>2]|0;
 $45 = (_sqlite3WhereFindTerm($43,$44,-1,0,0,130,0)|0);
 HEAP32[$pTerm>>2] = $45;
 $46 = HEAP32[$pTerm>>2]|0;
 $47 = ($46|0)!=(0|0);
 L13: do {
  if ($47) {
   $48 = HEAP32[$pLoop>>2]|0;
   $49 = ((($48)) + 36|0);
   HEAP32[$49>>2] = 4353;
   $50 = HEAP32[$pTerm>>2]|0;
   $51 = HEAP32[$pLoop>>2]|0;
   $52 = ((($51)) + 48|0);
   $53 = HEAP32[$52>>2]|0;
   HEAP32[$53>>2] = $50;
   $54 = HEAP32[$pLoop>>2]|0;
   $55 = ((($54)) + 40|0);
   HEAP16[$55>>1] = 1;
   $56 = HEAP32[$pLoop>>2]|0;
   $57 = ((($56)) + 24|0);
   HEAP16[$57>>1] = 1;
   $58 = HEAP32[$pLoop>>2]|0;
   $59 = ((($58)) + 20|0);
   HEAP16[$59>>1] = 33;
  } else {
   $60 = HEAP32[$pTab>>2]|0;
   $61 = ((($60)) + 8|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$pIdx>>2] = $62;
   while(1) {
    $63 = HEAP32[$pIdx>>2]|0;
    $64 = ($63|0)!=(0|0);
    if (!($64)) {
     break L13;
    }
    $65 = HEAP32[$pIdx>>2]|0;
    $66 = ((($65)) + 54|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = ($68|0)!=(0);
    if ($69) {
     $70 = HEAP32[$pIdx>>2]|0;
     $71 = ((($70)) + 36|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = ($72|0)!=(0|0);
     if (!($73)) {
      $74 = HEAP32[$pIdx>>2]|0;
      $75 = ((($74)) + 50|0);
      $76 = HEAP16[$75>>1]|0;
      $77 = $76&65535;
      $78 = ($77|0)>(3);
      if (!($78)) {
       $79 = HEAP32[$pIdx>>2]|0;
       $80 = ((($79)) + 55|0);
       $81 = HEAP8[$80>>0]|0;
       $82 = ($81&255) >>> 3;
       $83 = $82 & 1;
       $84 = $83&255;
       $85 = ($84|0)!=(0);
       $86 = $85 ? 130 : 2;
       HEAP32[$opMask>>2] = $86;
       HEAP32[$j>>2] = 0;
       while(1) {
        $87 = HEAP32[$j>>2]|0;
        $88 = HEAP32[$pIdx>>2]|0;
        $89 = ((($88)) + 50|0);
        $90 = HEAP16[$89>>1]|0;
        $91 = $90&65535;
        $92 = ($87|0)<($91|0);
        if (!($92)) {
         break;
        }
        $93 = HEAP32[$pWC>>2]|0;
        $94 = HEAP32[$iCur>>2]|0;
        $95 = HEAP32[$j>>2]|0;
        $96 = HEAP32[$opMask>>2]|0;
        $97 = HEAP32[$pIdx>>2]|0;
        $98 = (_sqlite3WhereFindTerm($93,$94,$95,0,0,$96,$97)|0);
        HEAP32[$pTerm>>2] = $98;
        $99 = HEAP32[$pTerm>>2]|0;
        $100 = ($99|0)==(0|0);
        if ($100) {
         break;
        }
        $101 = HEAP32[$pTerm>>2]|0;
        $102 = HEAP32[$j>>2]|0;
        $103 = HEAP32[$pLoop>>2]|0;
        $104 = ((($103)) + 48|0);
        $105 = HEAP32[$104>>2]|0;
        $106 = (($105) + ($102<<2)|0);
        HEAP32[$106>>2] = $101;
        $107 = HEAP32[$j>>2]|0;
        $108 = (($107) + 1)|0;
        HEAP32[$j>>2] = $108;
       }
       $109 = HEAP32[$j>>2]|0;
       $110 = HEAP32[$pIdx>>2]|0;
       $111 = ((($110)) + 50|0);
       $112 = HEAP16[$111>>1]|0;
       $113 = $112&65535;
       $114 = ($109|0)!=($113|0);
       if (!($114)) {
        break;
       }
      }
     }
    }
    $160 = HEAP32[$pIdx>>2]|0;
    $161 = ((($160)) + 20|0);
    $162 = HEAP32[$161>>2]|0;
    HEAP32[$pIdx>>2] = $162;
   }
   $115 = HEAP32[$pLoop>>2]|0;
   $116 = ((($115)) + 36|0);
   HEAP32[$116>>2] = 4609;
   $117 = HEAP32[$pIdx>>2]|0;
   $118 = ((($117)) + 55|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = ($119&255) >>> 5;
   $121 = $120 & 1;
   $122 = $121&255;
   $123 = ($122|0)!=(0);
   if ($123) {
    label = 21;
   } else {
    $124 = HEAP32[$pItem>>2]|0;
    $125 = ((($124)) + 56|0);
    $126 = $125;
    $127 = $126;
    $128 = HEAP32[$127>>2]|0;
    $129 = (($126) + 4)|0;
    $130 = $129;
    $131 = HEAP32[$130>>2]|0;
    $132 = HEAP32[$pIdx>>2]|0;
    $133 = (_columnsInIndex($132)|0);
    $134 = tempRet0;
    $135 = $133 ^ -1;
    $136 = $134 ^ -1;
    $137 = $128 & $135;
    $138 = $131 & $136;
    $139 = ($137|0)==(0);
    $140 = ($138|0)==(0);
    $141 = $139 & $140;
    if ($141) {
     label = 21;
    }
   }
   if ((label|0) == 21) {
    $142 = HEAP32[$pLoop>>2]|0;
    $143 = ((($142)) + 36|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = $144 | 64;
    HEAP32[$143>>2] = $145;
   }
   $146 = HEAP32[$j>>2]|0;
   $147 = $146&65535;
   $148 = HEAP32[$pLoop>>2]|0;
   $149 = ((($148)) + 40|0);
   HEAP16[$149>>1] = $147;
   $150 = HEAP32[$j>>2]|0;
   $151 = $150&65535;
   $152 = HEAP32[$pLoop>>2]|0;
   $153 = ((($152)) + 24|0);
   HEAP16[$153>>1] = $151;
   $154 = HEAP32[$pIdx>>2]|0;
   $155 = HEAP32[$pLoop>>2]|0;
   $156 = ((($155)) + 24|0);
   $157 = ((($156)) + 8|0);
   HEAP32[$157>>2] = $154;
   $158 = HEAP32[$pLoop>>2]|0;
   $159 = ((($158)) + 20|0);
   HEAP16[$159>>1] = 39;
  }
 } while(0);
 $163 = HEAP32[$pLoop>>2]|0;
 $164 = ((($163)) + 36|0);
 $165 = HEAP32[$164>>2]|0;
 $166 = ($165|0)!=(0);
 if (!($166)) {
  HEAP32[$0>>2] = 0;
  $209 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($209|0);
 }
 $167 = HEAP32[$pLoop>>2]|0;
 $168 = ((($167)) + 22|0);
 HEAP16[$168>>1] = 1;
 $169 = HEAP32[$pLoop>>2]|0;
 $170 = HEAP32[$pWInfo>>2]|0;
 $171 = ((($170)) + 752|0);
 $172 = ((($171)) + 64|0);
 HEAP32[$172>>2] = $169;
 $173 = HEAP32[$pWInfo>>2]|0;
 $174 = ((($173)) + 488|0);
 $175 = HEAP32[$iCur>>2]|0;
 $176 = (_sqlite3WhereGetMask($174,$175)|0);
 $177 = tempRet0;
 $178 = HEAP32[$pLoop>>2]|0;
 $179 = ((($178)) + 8|0);
 $180 = $179;
 $181 = $180;
 HEAP32[$181>>2] = $176;
 $182 = (($180) + 4)|0;
 $183 = $182;
 HEAP32[$183>>2] = $177;
 $184 = HEAP32[$iCur>>2]|0;
 $185 = HEAP32[$pWInfo>>2]|0;
 $186 = ((($185)) + 752|0);
 $187 = ((($186)) + 4|0);
 HEAP32[$187>>2] = $184;
 $188 = HEAP32[$pWInfo>>2]|0;
 $189 = ((($188)) + 72|0);
 HEAP16[$189>>1] = 1;
 $190 = HEAP32[$pWInfo>>2]|0;
 $191 = ((($190)) + 8|0);
 $192 = HEAP32[$191>>2]|0;
 $193 = ($192|0)!=(0|0);
 if ($193) {
  $194 = HEAP32[$pWInfo>>2]|0;
  $195 = ((($194)) + 8|0);
  $196 = HEAP32[$195>>2]|0;
  $197 = HEAP32[$196>>2]|0;
  $198 = $197&255;
  $199 = HEAP32[$pWInfo>>2]|0;
  $200 = ((($199)) + 43|0);
  HEAP8[$200>>0] = $198;
 }
 $201 = HEAP32[$pWInfo>>2]|0;
 $202 = ((($201)) + 40|0);
 $203 = HEAP16[$202>>1]|0;
 $204 = $203&65535;
 $205 = $204 & 256;
 $206 = ($205|0)!=(0);
 if ($206) {
  $207 = HEAP32[$pWInfo>>2]|0;
  $208 = ((($207)) + 47|0);
  HEAP8[$208>>0] = 1;
 }
 HEAP32[$0>>2] = 1;
 $209 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($209|0);
}
function _whereLoopAddAll($pBuilder) {
 $pBuilder = $pBuilder|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $iTab = 0, $mPrereq = 0;
 var $mPrior = 0, $mUnusable = 0, $p = 0, $pEnd = 0, $pItem = 0, $pNew = 0, $pTabList = 0, $pWInfo = 0, $priorJointype = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $pWInfo = sp + 56|0;
 $mPrereq = sp + 16|0;
 $mPrior = sp + 8|0;
 $iTab = sp + 52|0;
 $pTabList = sp + 48|0;
 $pItem = sp + 44|0;
 $pEnd = sp + 40|0;
 $db = sp + 36|0;
 $rc = sp + 32|0;
 $pNew = sp + 28|0;
 $priorJointype = sp + 64|0;
 $mUnusable = sp;
 $p = sp + 24|0;
 HEAP32[$0>>2] = $pBuilder;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pWInfo>>2] = $2;
 $3 = $mPrereq;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = $mPrior;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pWInfo>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pTabList>>2] = $13;
 $14 = HEAP32[$pWInfo>>2]|0;
 $15 = ((($14)) + 42|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = HEAP32[$pTabList>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = (($19) + (($17*72)|0)|0);
 HEAP32[$pEnd>>2] = $20;
 $21 = HEAP32[$pWInfo>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$db>>2] = $23;
 HEAP32[$rc>>2] = 0;
 HEAP8[$priorJointype>>0] = 0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 12|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pNew>>2] = $26;
 $27 = HEAP32[$pNew>>2]|0;
 _whereLoopInit($27);
 HEAP32[$iTab>>2] = 0;
 $28 = HEAP32[$pTabList>>2]|0;
 $29 = ((($28)) + 8|0);
 HEAP32[$pItem>>2] = $29;
 while(1) {
  $30 = HEAP32[$pItem>>2]|0;
  $31 = HEAP32[$pEnd>>2]|0;
  $32 = ($30>>>0)<($31>>>0);
  if (!($32)) {
   label = 19;
   break;
  }
  $33 = $mUnusable;
  $34 = $33;
  HEAP32[$34>>2] = 0;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = 0;
  $37 = HEAP32[$iTab>>2]|0;
  $38 = $37&255;
  $39 = HEAP32[$pNew>>2]|0;
  $40 = ((($39)) + 16|0);
  HEAP8[$40>>0] = $38;
  $41 = HEAP32[$pWInfo>>2]|0;
  $42 = ((($41)) + 488|0);
  $43 = HEAP32[$pItem>>2]|0;
  $44 = ((($43)) + 44|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3WhereGetMask($42,$45)|0);
  $47 = tempRet0;
  $48 = HEAP32[$pNew>>2]|0;
  $49 = ((($48)) + 8|0);
  $50 = $49;
  $51 = $50;
  HEAP32[$51>>2] = $46;
  $52 = (($50) + 4)|0;
  $53 = $52;
  HEAP32[$53>>2] = $47;
  $54 = HEAP32[$pItem>>2]|0;
  $55 = ((($54)) + 36|0);
  $56 = HEAP8[$55>>0]|0;
  $57 = $56&255;
  $58 = HEAP8[$priorJointype>>0]|0;
  $59 = $58&255;
  $60 = $57 | $59;
  $61 = $60 & 10;
  $62 = ($61|0)!=(0);
  if ($62) {
   $63 = $mPrior;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = (($63) + 4)|0;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = $mPrereq;
   $70 = $69;
   HEAP32[$70>>2] = $65;
   $71 = (($69) + 4)|0;
   $72 = $71;
   HEAP32[$72>>2] = $68;
  }
  $73 = HEAP32[$pItem>>2]|0;
  $74 = ((($73)) + 36|0);
  $75 = HEAP8[$74>>0]|0;
  HEAP8[$priorJointype>>0] = $75;
  $76 = HEAP32[$pItem>>2]|0;
  $77 = ((($76)) + 16|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ((($78)) + 42|0);
  $80 = HEAP8[$79>>0]|0;
  $81 = $80&255;
  $82 = $81 & 16;
  $83 = ($82|0)!=(0);
  if ($83) {
   $84 = HEAP32[$pItem>>2]|0;
   $85 = ((($84)) + 72|0);
   HEAP32[$p>>2] = $85;
   while(1) {
    $86 = HEAP32[$p>>2]|0;
    $87 = HEAP32[$pEnd>>2]|0;
    $88 = ($86>>>0)<($87>>>0);
    if (!($88)) {
     break;
    }
    $89 = $mUnusable;
    $90 = $89;
    $91 = HEAP32[$90>>2]|0;
    $92 = (($89) + 4)|0;
    $93 = $92;
    $94 = HEAP32[$93>>2]|0;
    $95 = ($91|0)!=(0);
    $96 = ($94|0)!=(0);
    $97 = $95 | $96;
    if ($97) {
     label = 10;
    } else {
     $98 = HEAP32[$p>>2]|0;
     $99 = ((($98)) + 36|0);
     $100 = HEAP8[$99>>0]|0;
     $101 = $100&255;
     $102 = $101 & 10;
     $103 = ($102|0)!=(0);
     if ($103) {
      label = 10;
     }
    }
    if ((label|0) == 10) {
     label = 0;
     $104 = HEAP32[$pWInfo>>2]|0;
     $105 = ((($104)) + 488|0);
     $106 = HEAP32[$p>>2]|0;
     $107 = ((($106)) + 44|0);
     $108 = HEAP32[$107>>2]|0;
     $109 = (_sqlite3WhereGetMask($105,$108)|0);
     $110 = tempRet0;
     $111 = $mUnusable;
     $112 = $111;
     $113 = HEAP32[$112>>2]|0;
     $114 = (($111) + 4)|0;
     $115 = $114;
     $116 = HEAP32[$115>>2]|0;
     $117 = $113 | $109;
     $118 = $116 | $110;
     $119 = $mUnusable;
     $120 = $119;
     HEAP32[$120>>2] = $117;
     $121 = (($119) + 4)|0;
     $122 = $121;
     HEAP32[$122>>2] = $118;
    }
    $123 = HEAP32[$p>>2]|0;
    $124 = ((($123)) + 72|0);
    HEAP32[$p>>2] = $124;
   }
   $125 = HEAP32[$0>>2]|0;
   $126 = $mPrereq;
   $127 = $126;
   $128 = HEAP32[$127>>2]|0;
   $129 = (($126) + 4)|0;
   $130 = $129;
   $131 = HEAP32[$130>>2]|0;
   $132 = $mUnusable;
   $133 = $132;
   $134 = HEAP32[$133>>2]|0;
   $135 = (($132) + 4)|0;
   $136 = $135;
   $137 = HEAP32[$136>>2]|0;
   $138 = (_whereLoopAddVirtual($125,$128,$131,$134,$137)|0);
   HEAP32[$rc>>2] = $138;
  } else {
   $139 = HEAP32[$0>>2]|0;
   $140 = $mPrereq;
   $141 = $140;
   $142 = HEAP32[$141>>2]|0;
   $143 = (($140) + 4)|0;
   $144 = $143;
   $145 = HEAP32[$144>>2]|0;
   $146 = (_whereLoopAddBtree($139,$142,$145)|0);
   HEAP32[$rc>>2] = $146;
  }
  $147 = HEAP32[$rc>>2]|0;
  $148 = ($147|0)==(0);
  if ($148) {
   $149 = HEAP32[$0>>2]|0;
   $150 = $mPrereq;
   $151 = $150;
   $152 = HEAP32[$151>>2]|0;
   $153 = (($150) + 4)|0;
   $154 = $153;
   $155 = HEAP32[$154>>2]|0;
   $156 = $mUnusable;
   $157 = $156;
   $158 = HEAP32[$157>>2]|0;
   $159 = (($156) + 4)|0;
   $160 = $159;
   $161 = HEAP32[$160>>2]|0;
   $162 = (_whereLoopAddOr($149,$152,$155,$158,$161)|0);
   HEAP32[$rc>>2] = $162;
  }
  $163 = HEAP32[$pNew>>2]|0;
  $164 = ((($163)) + 8|0);
  $165 = $164;
  $166 = $165;
  $167 = HEAP32[$166>>2]|0;
  $168 = (($165) + 4)|0;
  $169 = $168;
  $170 = HEAP32[$169>>2]|0;
  $171 = $mPrior;
  $172 = $171;
  $173 = HEAP32[$172>>2]|0;
  $174 = (($171) + 4)|0;
  $175 = $174;
  $176 = HEAP32[$175>>2]|0;
  $177 = $173 | $167;
  $178 = $176 | $170;
  $179 = $mPrior;
  $180 = $179;
  HEAP32[$180>>2] = $177;
  $181 = (($179) + 4)|0;
  $182 = $181;
  HEAP32[$182>>2] = $178;
  $183 = HEAP32[$rc>>2]|0;
  $184 = ($183|0)!=(0);
  if ($184) {
   label = 19;
   break;
  }
  $185 = HEAP32[$db>>2]|0;
  $186 = ((($185)) + 69|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $187&255;
  $189 = ($188|0)!=(0);
  if ($189) {
   label = 19;
   break;
  }
  $190 = HEAP32[$iTab>>2]|0;
  $191 = (($190) + 1)|0;
  HEAP32[$iTab>>2] = $191;
  $192 = HEAP32[$pItem>>2]|0;
  $193 = ((($192)) + 72|0);
  HEAP32[$pItem>>2] = $193;
 }
 if ((label|0) == 19) {
  $194 = HEAP32[$db>>2]|0;
  $195 = HEAP32[$pNew>>2]|0;
  _whereLoopClear($194,$195);
  $196 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($196|0);
 }
 return (0)|0;
}
function _wherePathSolver($pWInfo,$nRowEst) {
 $pWInfo = $pWInfo|0;
 $nRowEst = $nRowEst|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aFrom = 0, $aSortCost = 0, $aTo = 0, $db = 0, $iLoop = 0, $ii = 0, $isOrdered = 0, $jj = 0, $m = 0, $maskNew = 0, $mxChoice = 0, $mxCost = 0;
 var $mxI = 0, $mxUnsorted = 0, $nFrom = 0, $nLoop = 0, $nOrder = 0, $nOrderBy = 0, $nOut = 0, $nSpace = 0, $nTo = 0, $notUsed = 0, $or$cond = 0, $pFrom = 0, $pLevel = 0, $pParse = 0, $pSpace = 0, $pTo = 0, $pWLoop = 0, $pX = 0, $rCost = 0, $rUnsorted = 0;
 var $rc = 0, $rc1 = 0, $revMask = 0, $revMask2 = 0, $vararg_buffer = 0, $wsFlags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 40|0;
 $0 = sp + 148|0;
 $1 = sp + 144|0;
 $2 = sp + 162|0;
 $mxChoice = sp + 140|0;
 $nLoop = sp + 136|0;
 $pParse = sp + 132|0;
 $db = sp + 128|0;
 $iLoop = sp + 124|0;
 $ii = sp + 120|0;
 $jj = sp + 116|0;
 $mxI = sp + 112|0;
 $nOrderBy = sp + 108|0;
 $mxCost = sp + 160|0;
 $mxUnsorted = sp + 158|0;
 $nTo = sp + 104|0;
 $nFrom = sp + 100|0;
 $aFrom = sp + 96|0;
 $aTo = sp + 92|0;
 $pFrom = sp + 88|0;
 $pTo = sp + 84|0;
 $pWLoop = sp + 80|0;
 $pX = sp + 76|0;
 $aSortCost = sp + 72|0;
 $pSpace = sp + 68|0;
 $nSpace = sp + 64|0;
 $nOut = sp + 156|0;
 $rCost = sp + 154|0;
 $rUnsorted = sp + 152|0;
 $isOrdered = sp + 164|0;
 $maskNew = sp + 32|0;
 $revMask = sp + 24|0;
 $pLevel = sp + 60|0;
 $notUsed = sp + 16|0;
 $rc = sp + 56|0;
 $wsFlags = sp + 52|0;
 $m = sp + 8|0;
 $rc1 = sp + 48|0;
 $revMask2 = sp;
 $nOrder = sp + 44|0;
 HEAP32[$1>>2] = $pWInfo;
 HEAP16[$2>>1] = $nRowEst;
 HEAP32[$mxI>>2] = 0;
 HEAP16[$mxCost>>1] = 0;
 HEAP16[$mxUnsorted>>1] = 0;
 HEAP32[$aSortCost>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pParse>>2] = $4;
 $5 = HEAP32[$pParse>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 42|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$nLoop>>2] = $10;
 $11 = HEAP32[$nLoop>>2]|0;
 $12 = ($11|0)<=(1);
 if ($12) {
  $16 = 1;
 } else {
  $13 = HEAP32[$nLoop>>2]|0;
  $14 = ($13|0)==(2);
  $15 = $14 ? 5 : 10;
  $16 = $15;
 }
 HEAP32[$mxChoice>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)==(0|0);
 if ($20) {
  label = 5;
 } else {
  $21 = HEAP16[$2>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = ($22|0)==(0);
  if ($23) {
   label = 5;
  } else {
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 8|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$nOrderBy>>2] = $27;
  }
 }
 if ((label|0) == 5) {
  HEAP32[$nOrderBy>>2] = 0;
 }
 $28 = HEAP32[$nLoop>>2]|0;
 $29 = $28<<2;
 $30 = (32 + ($29))|0;
 $31 = HEAP32[$mxChoice>>2]|0;
 $32 = Math_imul($30, $31)|0;
 $33 = $32<<1;
 HEAP32[$nSpace>>2] = $33;
 $34 = HEAP32[$nOrderBy>>2]|0;
 $35 = $34<<1;
 $36 = HEAP32[$nSpace>>2]|0;
 $37 = (($36) + ($35))|0;
 HEAP32[$nSpace>>2] = $37;
 $38 = HEAP32[$db>>2]|0;
 $39 = HEAP32[$nSpace>>2]|0;
 $40 = ($39|0)<(0);
 $41 = $40 << 31 >> 31;
 $42 = (_sqlite3DbMallocRawNN($38,$39,$41)|0);
 HEAP32[$pSpace>>2] = $42;
 $43 = HEAP32[$pSpace>>2]|0;
 $44 = ($43|0)==(0|0);
 if ($44) {
  HEAP32[$0>>2] = 7;
  $747 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($747|0);
 }
 $45 = HEAP32[$pSpace>>2]|0;
 HEAP32[$aTo>>2] = $45;
 $46 = HEAP32[$aTo>>2]|0;
 $47 = HEAP32[$mxChoice>>2]|0;
 $48 = (($46) + ($47<<5)|0);
 HEAP32[$aFrom>>2] = $48;
 $49 = HEAP32[$aFrom>>2]|0;
 ;HEAP32[$49>>2]=0|0;HEAP32[$49+4>>2]=0|0;HEAP32[$49+8>>2]=0|0;HEAP32[$49+12>>2]=0|0;HEAP32[$49+16>>2]=0|0;HEAP32[$49+20>>2]=0|0;HEAP32[$49+24>>2]=0|0;HEAP32[$49+28>>2]=0|0;
 $50 = HEAP32[$aFrom>>2]|0;
 $51 = HEAP32[$mxChoice>>2]|0;
 $52 = (($50) + ($51<<5)|0);
 HEAP32[$pX>>2] = $52;
 $53 = HEAP32[$mxChoice>>2]|0;
 $54 = $53<<1;
 HEAP32[$ii>>2] = $54;
 $55 = HEAP32[$aTo>>2]|0;
 HEAP32[$pFrom>>2] = $55;
 while(1) {
  $56 = HEAP32[$ii>>2]|0;
  $57 = ($56|0)>(0);
  if (!($57)) {
   break;
  }
  $58 = HEAP32[$pX>>2]|0;
  $59 = HEAP32[$pFrom>>2]|0;
  $60 = ((($59)) + 24|0);
  HEAP32[$60>>2] = $58;
  $61 = HEAP32[$ii>>2]|0;
  $62 = (($61) + -1)|0;
  HEAP32[$ii>>2] = $62;
  $63 = HEAP32[$pFrom>>2]|0;
  $64 = ((($63)) + 32|0);
  HEAP32[$pFrom>>2] = $64;
  $65 = HEAP32[$nLoop>>2]|0;
  $66 = HEAP32[$pX>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  HEAP32[$pX>>2] = $67;
 }
 $68 = HEAP32[$nOrderBy>>2]|0;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$pX>>2]|0;
  HEAP32[$aSortCost>>2] = $70;
  $71 = HEAP32[$aSortCost>>2]|0;
  $72 = HEAP32[$nOrderBy>>2]|0;
  $73 = $72<<1;
  _memset(($71|0),0,($73|0))|0;
 }
 $74 = HEAP32[$pParse>>2]|0;
 $75 = ((($74)) + 136|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = ($76>>>0)<(48);
 if ($77) {
  $78 = HEAP32[$pParse>>2]|0;
  $79 = ((($78)) + 136|0);
  $80 = HEAP32[$79>>2]|0;
  $82 = $80;
 } else {
  $82 = 48;
 }
 $81 = $82&65535;
 $83 = HEAP32[$aFrom>>2]|0;
 $84 = ((($83)) + 16|0);
 HEAP16[$84>>1] = $81;
 HEAP32[$nFrom>>2] = 1;
 $85 = HEAP32[$nOrderBy>>2]|0;
 $86 = ($85|0)!=(0);
 if ($86) {
  $87 = HEAP32[$nLoop>>2]|0;
  $88 = ($87|0)>(0);
  $89 = HEAP32[$nOrderBy>>2]|0;
  $90 = $88 ? -1 : $89;
  $91 = $90&255;
  $92 = HEAP32[$aFrom>>2]|0;
  $93 = ((($92)) + 22|0);
  HEAP8[$93>>0] = $91;
 }
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $94 = HEAP32[$iLoop>>2]|0;
  $95 = HEAP32[$nLoop>>2]|0;
  $96 = ($94|0)<($95|0);
  if (!($96)) {
   break;
  }
  HEAP32[$nTo>>2] = 0;
  HEAP32[$ii>>2] = 0;
  $97 = HEAP32[$aFrom>>2]|0;
  HEAP32[$pFrom>>2] = $97;
  while(1) {
   $98 = HEAP32[$ii>>2]|0;
   $99 = HEAP32[$nFrom>>2]|0;
   $100 = ($98|0)<($99|0);
   if (!($100)) {
    break;
   }
   $101 = HEAP32[$1>>2]|0;
   $102 = ((($101)) + 56|0);
   $103 = HEAP32[$102>>2]|0;
   HEAP32[$pWLoop>>2] = $103;
   while(1) {
    $104 = HEAP32[$pWLoop>>2]|0;
    $105 = ($104|0)!=(0|0);
    if (!($105)) {
     break;
    }
    $106 = HEAP32[$pFrom>>2]|0;
    $107 = ((($106)) + 22|0);
    $108 = HEAP8[$107>>0]|0;
    HEAP8[$isOrdered>>0] = $108;
    $109 = $revMask;
    $110 = $109;
    HEAP32[$110>>2] = 0;
    $111 = (($109) + 4)|0;
    $112 = $111;
    HEAP32[$112>>2] = 0;
    $113 = HEAP32[$pWLoop>>2]|0;
    $114 = $113;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($114) + 4)|0;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = HEAP32[$pFrom>>2]|0;
    $121 = $120;
    $122 = $121;
    $123 = HEAP32[$122>>2]|0;
    $124 = (($121) + 4)|0;
    $125 = $124;
    $126 = HEAP32[$125>>2]|0;
    $127 = $123 ^ -1;
    $128 = $126 ^ -1;
    $129 = $116 & $127;
    $130 = $119 & $128;
    $131 = ($129|0)!=(0);
    $132 = ($130|0)!=(0);
    $133 = $131 | $132;
    L36: do {
     if (!($133)) {
      $134 = HEAP32[$pWLoop>>2]|0;
      $135 = ((($134)) + 8|0);
      $136 = $135;
      $137 = $136;
      $138 = HEAP32[$137>>2]|0;
      $139 = (($136) + 4)|0;
      $140 = $139;
      $141 = HEAP32[$140>>2]|0;
      $142 = HEAP32[$pFrom>>2]|0;
      $143 = $142;
      $144 = $143;
      $145 = HEAP32[$144>>2]|0;
      $146 = (($143) + 4)|0;
      $147 = $146;
      $148 = HEAP32[$147>>2]|0;
      $149 = $138 & $145;
      $150 = $141 & $148;
      $151 = ($149|0)!=(0);
      $152 = ($150|0)!=(0);
      $153 = $151 | $152;
      if (!($153)) {
       $154 = HEAP32[$pWLoop>>2]|0;
       $155 = ((($154)) + 36|0);
       $156 = HEAP32[$155>>2]|0;
       $157 = $156 & 16384;
       $158 = ($157|0)!=(0);
       if ($158) {
        $159 = HEAP32[$pFrom>>2]|0;
        $160 = ((($159)) + 16|0);
        $161 = HEAP16[$160>>1]|0;
        $162 = $161 << 16 >> 16;
        $163 = ($162|0)<(10);
        if ($163) {
         break;
        }
       }
       $164 = HEAP32[$pWLoop>>2]|0;
       $165 = ((($164)) + 18|0);
       $166 = HEAP16[$165>>1]|0;
       $167 = HEAP32[$pWLoop>>2]|0;
       $168 = ((($167)) + 20|0);
       $169 = HEAP16[$168>>1]|0;
       $170 = $169 << 16 >> 16;
       $171 = HEAP32[$pFrom>>2]|0;
       $172 = ((($171)) + 16|0);
       $173 = HEAP16[$172>>1]|0;
       $174 = $173 << 16 >> 16;
       $175 = (($170) + ($174))|0;
       $176 = $175&65535;
       $177 = (_sqlite3LogEstAdd($166,$176)|0);
       HEAP16[$rUnsorted>>1] = $177;
       $178 = HEAP16[$rUnsorted>>1]|0;
       $179 = HEAP32[$pFrom>>2]|0;
       $180 = ((($179)) + 20|0);
       $181 = HEAP16[$180>>1]|0;
       $182 = (_sqlite3LogEstAdd($178,$181)|0);
       HEAP16[$rUnsorted>>1] = $182;
       $183 = HEAP32[$pFrom>>2]|0;
       $184 = ((($183)) + 16|0);
       $185 = HEAP16[$184>>1]|0;
       $186 = $185 << 16 >> 16;
       $187 = HEAP32[$pWLoop>>2]|0;
       $188 = ((($187)) + 22|0);
       $189 = HEAP16[$188>>1]|0;
       $190 = $189 << 16 >> 16;
       $191 = (($186) + ($190))|0;
       $192 = $191&65535;
       HEAP16[$nOut>>1] = $192;
       $193 = HEAP32[$pFrom>>2]|0;
       $194 = $193;
       $195 = $194;
       $196 = HEAP32[$195>>2]|0;
       $197 = (($194) + 4)|0;
       $198 = $197;
       $199 = HEAP32[$198>>2]|0;
       $200 = HEAP32[$pWLoop>>2]|0;
       $201 = ((($200)) + 8|0);
       $202 = $201;
       $203 = $202;
       $204 = HEAP32[$203>>2]|0;
       $205 = (($202) + 4)|0;
       $206 = $205;
       $207 = HEAP32[$206>>2]|0;
       $208 = $196 | $204;
       $209 = $199 | $207;
       $210 = $maskNew;
       $211 = $210;
       HEAP32[$211>>2] = $208;
       $212 = (($210) + 4)|0;
       $213 = $212;
       HEAP32[$213>>2] = $209;
       $214 = HEAP8[$isOrdered>>0]|0;
       $215 = $214 << 24 >> 24;
       $216 = ($215|0)<(0);
       if ($216) {
        $217 = HEAP32[$1>>2]|0;
        $218 = HEAP32[$1>>2]|0;
        $219 = ((($218)) + 8|0);
        $220 = HEAP32[$219>>2]|0;
        $221 = HEAP32[$pFrom>>2]|0;
        $222 = HEAP32[$1>>2]|0;
        $223 = ((($222)) + 40|0);
        $224 = HEAP16[$223>>1]|0;
        $225 = HEAP32[$iLoop>>2]|0;
        $226 = $225&65535;
        $227 = HEAP32[$pWLoop>>2]|0;
        $228 = (_wherePathSatisfiesOrderBy($217,$220,$221,$224,$226,$227,$revMask)|0);
        HEAP8[$isOrdered>>0] = $228;
       } else {
        $229 = HEAP32[$pFrom>>2]|0;
        $230 = ((($229)) + 8|0);
        $231 = $230;
        $232 = $231;
        $233 = HEAP32[$232>>2]|0;
        $234 = (($231) + 4)|0;
        $235 = $234;
        $236 = HEAP32[$235>>2]|0;
        $237 = $revMask;
        $238 = $237;
        HEAP32[$238>>2] = $233;
        $239 = (($237) + 4)|0;
        $240 = $239;
        HEAP32[$240>>2] = $236;
       }
       $241 = HEAP8[$isOrdered>>0]|0;
       $242 = $241 << 24 >> 24;
       $243 = ($242|0)>=(0);
       if ($243) {
        $244 = HEAP8[$isOrdered>>0]|0;
        $245 = $244 << 24 >> 24;
        $246 = HEAP32[$nOrderBy>>2]|0;
        $247 = ($245|0)<($246|0);
        if ($247) {
         $248 = HEAP8[$isOrdered>>0]|0;
         $249 = $248 << 24 >> 24;
         $250 = HEAP32[$aSortCost>>2]|0;
         $251 = (($250) + ($249<<1)|0);
         $252 = HEAP16[$251>>1]|0;
         $253 = $252 << 16 >> 16;
         $254 = ($253|0)==(0);
         if ($254) {
          $255 = HEAP32[$1>>2]|0;
          $256 = HEAP16[$2>>1]|0;
          $257 = HEAP32[$nOrderBy>>2]|0;
          $258 = HEAP8[$isOrdered>>0]|0;
          $259 = $258 << 24 >> 24;
          $260 = (_whereSortingCost($255,$256,$257,$259)|0);
          $261 = HEAP8[$isOrdered>>0]|0;
          $262 = $261 << 24 >> 24;
          $263 = HEAP32[$aSortCost>>2]|0;
          $264 = (($263) + ($262<<1)|0);
          HEAP16[$264>>1] = $260;
         }
         $265 = HEAP16[$rUnsorted>>1]|0;
         $266 = HEAP8[$isOrdered>>0]|0;
         $267 = $266 << 24 >> 24;
         $268 = HEAP32[$aSortCost>>2]|0;
         $269 = (($268) + ($267<<1)|0);
         $270 = HEAP16[$269>>1]|0;
         $271 = (_sqlite3LogEstAdd($265,$270)|0);
         HEAP16[$rCost>>1] = $271;
        } else {
         label = 36;
        }
       } else {
        label = 36;
       }
       if ((label|0) == 36) {
        label = 0;
        $272 = HEAP16[$rUnsorted>>1]|0;
        HEAP16[$rCost>>1] = $272;
       }
       HEAP32[$jj>>2] = 0;
       $273 = HEAP32[$aTo>>2]|0;
       HEAP32[$pTo>>2] = $273;
       while(1) {
        $274 = HEAP32[$jj>>2]|0;
        $275 = HEAP32[$nTo>>2]|0;
        $276 = ($274|0)<($275|0);
        if (!($276)) {
         break;
        }
        $277 = HEAP32[$pTo>>2]|0;
        $278 = $277;
        $279 = $278;
        $280 = HEAP32[$279>>2]|0;
        $281 = (($278) + 4)|0;
        $282 = $281;
        $283 = HEAP32[$282>>2]|0;
        $284 = $maskNew;
        $285 = $284;
        $286 = HEAP32[$285>>2]|0;
        $287 = (($284) + 4)|0;
        $288 = $287;
        $289 = HEAP32[$288>>2]|0;
        $290 = ($280|0)==($286|0);
        $291 = ($283|0)==($289|0);
        $292 = $290 & $291;
        if ($292) {
         $293 = HEAP32[$pTo>>2]|0;
         $294 = ((($293)) + 22|0);
         $295 = HEAP8[$294>>0]|0;
         $296 = $295 << 24 >> 24;
         $297 = HEAP8[$isOrdered>>0]|0;
         $298 = $297 << 24 >> 24;
         $299 = $296 ^ $298;
         $300 = $299 & 128;
         $301 = ($300|0)==(0);
         if ($301) {
          break;
         }
        }
        $302 = HEAP32[$jj>>2]|0;
        $303 = (($302) + 1)|0;
        HEAP32[$jj>>2] = $303;
        $304 = HEAP32[$pTo>>2]|0;
        $305 = ((($304)) + 32|0);
        HEAP32[$pTo>>2] = $305;
       }
       $306 = HEAP32[$jj>>2]|0;
       $307 = HEAP32[$nTo>>2]|0;
       $308 = ($306|0)>=($307|0);
       do {
        if ($308) {
         $309 = HEAP32[$nTo>>2]|0;
         $310 = HEAP32[$mxChoice>>2]|0;
         $311 = ($309|0)>=($310|0);
         do {
          if ($311) {
           $312 = HEAP16[$rCost>>1]|0;
           $313 = $312 << 16 >> 16;
           $314 = HEAP16[$mxCost>>1]|0;
           $315 = $314 << 16 >> 16;
           $316 = ($313|0)>($315|0);
           if ($316) {
            break L36;
           }
           $317 = HEAP16[$rCost>>1]|0;
           $318 = $317 << 16 >> 16;
           $319 = HEAP16[$mxCost>>1]|0;
           $320 = $319 << 16 >> 16;
           $321 = ($318|0)==($320|0);
           if (!($321)) {
            break;
           }
           $322 = HEAP16[$rUnsorted>>1]|0;
           $323 = $322 << 16 >> 16;
           $324 = HEAP16[$mxUnsorted>>1]|0;
           $325 = $324 << 16 >> 16;
           $326 = ($323|0)>=($325|0);
           if ($326) {
            break L36;
           }
          }
         } while(0);
         $327 = HEAP32[$nTo>>2]|0;
         $328 = HEAP32[$mxChoice>>2]|0;
         $329 = ($327|0)<($328|0);
         if ($329) {
          $330 = HEAP32[$nTo>>2]|0;
          $331 = (($330) + 1)|0;
          HEAP32[$nTo>>2] = $331;
          HEAP32[$jj>>2] = $330;
         } else {
          $332 = HEAP32[$mxI>>2]|0;
          HEAP32[$jj>>2] = $332;
         }
         $333 = HEAP32[$jj>>2]|0;
         $334 = HEAP32[$aTo>>2]|0;
         $335 = (($334) + ($333<<5)|0);
         HEAP32[$pTo>>2] = $335;
        } else {
         $336 = HEAP32[$pTo>>2]|0;
         $337 = ((($336)) + 18|0);
         $338 = HEAP16[$337>>1]|0;
         $339 = $338 << 16 >> 16;
         $340 = HEAP16[$rCost>>1]|0;
         $341 = $340 << 16 >> 16;
         $342 = ($339|0)<($341|0);
         if ($342) {
          break L36;
         }
         $343 = HEAP32[$pTo>>2]|0;
         $344 = ((($343)) + 18|0);
         $345 = HEAP16[$344>>1]|0;
         $346 = $345 << 16 >> 16;
         $347 = HEAP16[$rCost>>1]|0;
         $348 = $347 << 16 >> 16;
         $349 = ($346|0)==($348|0);
         if (!($349)) {
          break;
         }
         $350 = HEAP32[$pTo>>2]|0;
         $351 = ((($350)) + 16|0);
         $352 = HEAP16[$351>>1]|0;
         $353 = $352 << 16 >> 16;
         $354 = HEAP16[$nOut>>1]|0;
         $355 = $354 << 16 >> 16;
         $356 = ($353|0)<=($355|0);
         if ($356) {
          break L36;
         }
        }
       } while(0);
       $357 = HEAP32[$pFrom>>2]|0;
       $358 = $357;
       $359 = $358;
       $360 = HEAP32[$359>>2]|0;
       $361 = (($358) + 4)|0;
       $362 = $361;
       $363 = HEAP32[$362>>2]|0;
       $364 = HEAP32[$pWLoop>>2]|0;
       $365 = ((($364)) + 8|0);
       $366 = $365;
       $367 = $366;
       $368 = HEAP32[$367>>2]|0;
       $369 = (($366) + 4)|0;
       $370 = $369;
       $371 = HEAP32[$370>>2]|0;
       $372 = $360 | $368;
       $373 = $363 | $371;
       $374 = HEAP32[$pTo>>2]|0;
       $375 = $374;
       $376 = $375;
       HEAP32[$376>>2] = $372;
       $377 = (($375) + 4)|0;
       $378 = $377;
       HEAP32[$378>>2] = $373;
       $379 = $revMask;
       $380 = $379;
       $381 = HEAP32[$380>>2]|0;
       $382 = (($379) + 4)|0;
       $383 = $382;
       $384 = HEAP32[$383>>2]|0;
       $385 = HEAP32[$pTo>>2]|0;
       $386 = ((($385)) + 8|0);
       $387 = $386;
       $388 = $387;
       HEAP32[$388>>2] = $381;
       $389 = (($387) + 4)|0;
       $390 = $389;
       HEAP32[$390>>2] = $384;
       $391 = HEAP16[$nOut>>1]|0;
       $392 = HEAP32[$pTo>>2]|0;
       $393 = ((($392)) + 16|0);
       HEAP16[$393>>1] = $391;
       $394 = HEAP16[$rCost>>1]|0;
       $395 = HEAP32[$pTo>>2]|0;
       $396 = ((($395)) + 18|0);
       HEAP16[$396>>1] = $394;
       $397 = HEAP16[$rUnsorted>>1]|0;
       $398 = HEAP32[$pTo>>2]|0;
       $399 = ((($398)) + 20|0);
       HEAP16[$399>>1] = $397;
       $400 = HEAP8[$isOrdered>>0]|0;
       $401 = HEAP32[$pTo>>2]|0;
       $402 = ((($401)) + 22|0);
       HEAP8[$402>>0] = $400;
       $403 = HEAP32[$pTo>>2]|0;
       $404 = ((($403)) + 24|0);
       $405 = HEAP32[$404>>2]|0;
       $406 = HEAP32[$pFrom>>2]|0;
       $407 = ((($406)) + 24|0);
       $408 = HEAP32[$407>>2]|0;
       $409 = HEAP32[$iLoop>>2]|0;
       $410 = $409<<2;
       _memcpy(($405|0),($408|0),($410|0))|0;
       $411 = HEAP32[$pWLoop>>2]|0;
       $412 = HEAP32[$iLoop>>2]|0;
       $413 = HEAP32[$pTo>>2]|0;
       $414 = ((($413)) + 24|0);
       $415 = HEAP32[$414>>2]|0;
       $416 = (($415) + ($412<<2)|0);
       HEAP32[$416>>2] = $411;
       $417 = HEAP32[$nTo>>2]|0;
       $418 = HEAP32[$mxChoice>>2]|0;
       $419 = ($417|0)>=($418|0);
       if ($419) {
        HEAP32[$mxI>>2] = 0;
        $420 = HEAP32[$aTo>>2]|0;
        $421 = ((($420)) + 18|0);
        $422 = HEAP16[$421>>1]|0;
        HEAP16[$mxCost>>1] = $422;
        $423 = HEAP32[$aTo>>2]|0;
        $424 = ((($423)) + 16|0);
        $425 = HEAP16[$424>>1]|0;
        HEAP16[$mxUnsorted>>1] = $425;
        HEAP32[$jj>>2] = 1;
        $426 = HEAP32[$aTo>>2]|0;
        $427 = ((($426)) + 32|0);
        HEAP32[$pTo>>2] = $427;
        while(1) {
         $428 = HEAP32[$jj>>2]|0;
         $429 = HEAP32[$mxChoice>>2]|0;
         $430 = ($428|0)<($429|0);
         if (!($430)) {
          break L36;
         }
         $431 = HEAP32[$pTo>>2]|0;
         $432 = ((($431)) + 18|0);
         $433 = HEAP16[$432>>1]|0;
         $434 = $433 << 16 >> 16;
         $435 = HEAP16[$mxCost>>1]|0;
         $436 = $435 << 16 >> 16;
         $437 = ($434|0)>($436|0);
         do {
          if ($437) {
           label = 60;
          } else {
           $438 = HEAP32[$pTo>>2]|0;
           $439 = ((($438)) + 18|0);
           $440 = HEAP16[$439>>1]|0;
           $441 = $440 << 16 >> 16;
           $442 = HEAP16[$mxCost>>1]|0;
           $443 = $442 << 16 >> 16;
           $444 = ($441|0)==($443|0);
           if (!($444)) {
            break;
           }
           $445 = HEAP32[$pTo>>2]|0;
           $446 = ((($445)) + 20|0);
           $447 = HEAP16[$446>>1]|0;
           $448 = $447 << 16 >> 16;
           $449 = HEAP16[$mxUnsorted>>1]|0;
           $450 = $449 << 16 >> 16;
           $451 = ($448|0)>($450|0);
           if ($451) {
            label = 60;
           }
          }
         } while(0);
         if ((label|0) == 60) {
          label = 0;
          $452 = HEAP32[$pTo>>2]|0;
          $453 = ((($452)) + 18|0);
          $454 = HEAP16[$453>>1]|0;
          HEAP16[$mxCost>>1] = $454;
          $455 = HEAP32[$pTo>>2]|0;
          $456 = ((($455)) + 20|0);
          $457 = HEAP16[$456>>1]|0;
          HEAP16[$mxUnsorted>>1] = $457;
          $458 = HEAP32[$jj>>2]|0;
          HEAP32[$mxI>>2] = $458;
         }
         $459 = HEAP32[$jj>>2]|0;
         $460 = (($459) + 1)|0;
         HEAP32[$jj>>2] = $460;
         $461 = HEAP32[$pTo>>2]|0;
         $462 = ((($461)) + 32|0);
         HEAP32[$pTo>>2] = $462;
        }
       }
      }
     }
    } while(0);
    $463 = HEAP32[$pWLoop>>2]|0;
    $464 = ((($463)) + 52|0);
    $465 = HEAP32[$464>>2]|0;
    HEAP32[$pWLoop>>2] = $465;
   }
   $466 = HEAP32[$ii>>2]|0;
   $467 = (($466) + 1)|0;
   HEAP32[$ii>>2] = $467;
   $468 = HEAP32[$pFrom>>2]|0;
   $469 = ((($468)) + 32|0);
   HEAP32[$pFrom>>2] = $469;
  }
  $470 = HEAP32[$aTo>>2]|0;
  HEAP32[$pFrom>>2] = $470;
  $471 = HEAP32[$aFrom>>2]|0;
  HEAP32[$aTo>>2] = $471;
  $472 = HEAP32[$pFrom>>2]|0;
  HEAP32[$aFrom>>2] = $472;
  $473 = HEAP32[$nTo>>2]|0;
  HEAP32[$nFrom>>2] = $473;
  $474 = HEAP32[$iLoop>>2]|0;
  $475 = (($474) + 1)|0;
  HEAP32[$iLoop>>2] = $475;
 }
 $476 = HEAP32[$nFrom>>2]|0;
 $477 = ($476|0)==(0);
 if ($477) {
  $478 = HEAP32[$pParse>>2]|0;
  _sqlite3ErrorMsg($478,34756,$vararg_buffer);
  $479 = HEAP32[$db>>2]|0;
  $480 = HEAP32[$pSpace>>2]|0;
  _sqlite3DbFree($479,$480);
  HEAP32[$0>>2] = 1;
  $747 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($747|0);
 }
 $481 = HEAP32[$aFrom>>2]|0;
 HEAP32[$pFrom>>2] = $481;
 HEAP32[$ii>>2] = 1;
 while(1) {
  $482 = HEAP32[$ii>>2]|0;
  $483 = HEAP32[$nFrom>>2]|0;
  $484 = ($482|0)<($483|0);
  if (!($484)) {
   break;
  }
  $485 = HEAP32[$pFrom>>2]|0;
  $486 = ((($485)) + 18|0);
  $487 = HEAP16[$486>>1]|0;
  $488 = $487 << 16 >> 16;
  $489 = HEAP32[$ii>>2]|0;
  $490 = HEAP32[$aFrom>>2]|0;
  $491 = (($490) + ($489<<5)|0);
  $492 = ((($491)) + 18|0);
  $493 = HEAP16[$492>>1]|0;
  $494 = $493 << 16 >> 16;
  $495 = ($488|0)>($494|0);
  if ($495) {
   $496 = HEAP32[$ii>>2]|0;
   $497 = HEAP32[$aFrom>>2]|0;
   $498 = (($497) + ($496<<5)|0);
   HEAP32[$pFrom>>2] = $498;
  }
  $499 = HEAP32[$ii>>2]|0;
  $500 = (($499) + 1)|0;
  HEAP32[$ii>>2] = $500;
 }
 HEAP32[$iLoop>>2] = 0;
 while(1) {
  $501 = HEAP32[$iLoop>>2]|0;
  $502 = HEAP32[$nLoop>>2]|0;
  $503 = ($501|0)<($502|0);
  $504 = HEAP32[$1>>2]|0;
  if (!($503)) {
   break;
  }
  $505 = ((($504)) + 752|0);
  $506 = HEAP32[$iLoop>>2]|0;
  $507 = (($505) + (($506*80)|0)|0);
  HEAP32[$pLevel>>2] = $507;
  $508 = HEAP32[$iLoop>>2]|0;
  $509 = HEAP32[$pFrom>>2]|0;
  $510 = ((($509)) + 24|0);
  $511 = HEAP32[$510>>2]|0;
  $512 = (($511) + ($508<<2)|0);
  $513 = HEAP32[$512>>2]|0;
  HEAP32[$pWLoop>>2] = $513;
  $514 = HEAP32[$pLevel>>2]|0;
  $515 = ((($514)) + 64|0);
  HEAP32[$515>>2] = $513;
  $516 = HEAP32[$pWLoop>>2]|0;
  $517 = ((($516)) + 16|0);
  $518 = HEAP8[$517>>0]|0;
  $519 = HEAP32[$pLevel>>2]|0;
  $520 = ((($519)) + 44|0);
  HEAP8[$520>>0] = $518;
  $521 = HEAP32[$pLevel>>2]|0;
  $522 = ((($521)) + 44|0);
  $523 = HEAP8[$522>>0]|0;
  $524 = $523&255;
  $525 = HEAP32[$1>>2]|0;
  $526 = ((($525)) + 4|0);
  $527 = HEAP32[$526>>2]|0;
  $528 = ((($527)) + 8|0);
  $529 = (($528) + (($524*72)|0)|0);
  $530 = ((($529)) + 44|0);
  $531 = HEAP32[$530>>2]|0;
  $532 = HEAP32[$pLevel>>2]|0;
  $533 = ((($532)) + 4|0);
  HEAP32[$533>>2] = $531;
  $534 = HEAP32[$iLoop>>2]|0;
  $535 = (($534) + 1)|0;
  HEAP32[$iLoop>>2] = $535;
 }
 $536 = ((($504)) + 40|0);
 $537 = HEAP16[$536>>1]|0;
 $538 = $537&65535;
 $539 = $538 & 256;
 $540 = ($539|0)!=(0);
 if ($540) {
  $541 = HEAP32[$1>>2]|0;
  $542 = ((($541)) + 40|0);
  $543 = HEAP16[$542>>1]|0;
  $544 = $543&65535;
  $545 = $544 & 128;
  $546 = ($545|0)==(0);
  if ($546) {
   $547 = HEAP32[$1>>2]|0;
   $548 = ((($547)) + 47|0);
   $549 = HEAP8[$548>>0]|0;
   $550 = $549&255;
   $551 = ($550|0)==(0);
   if ($551) {
    $552 = HEAP16[$2>>1]|0;
    $553 = $552 << 16 >> 16;
    $554 = ($553|0)!=(0);
    if ($554) {
     $555 = HEAP32[$1>>2]|0;
     $556 = HEAP32[$1>>2]|0;
     $557 = ((($556)) + 12|0);
     $558 = HEAP32[$557>>2]|0;
     $559 = HEAP32[$pFrom>>2]|0;
     $560 = HEAP32[$nLoop>>2]|0;
     $561 = (($560) - 1)|0;
     $562 = $561&65535;
     $563 = HEAP32[$nLoop>>2]|0;
     $564 = (($563) - 1)|0;
     $565 = HEAP32[$pFrom>>2]|0;
     $566 = ((($565)) + 24|0);
     $567 = HEAP32[$566>>2]|0;
     $568 = (($567) + ($564<<2)|0);
     $569 = HEAP32[$568>>2]|0;
     $570 = (_wherePathSatisfiesOrderBy($555,$558,$559,128,$562,$569,$notUsed)|0);
     $571 = $570 << 24 >> 24;
     HEAP32[$rc>>2] = $571;
     $572 = HEAP32[$rc>>2]|0;
     $573 = HEAP32[$1>>2]|0;
     $574 = ((($573)) + 12|0);
     $575 = HEAP32[$574>>2]|0;
     $576 = HEAP32[$575>>2]|0;
     $577 = ($572|0)==($576|0);
     if ($577) {
      $578 = HEAP32[$1>>2]|0;
      $579 = ((($578)) + 47|0);
      HEAP8[$579>>0] = 2;
     }
    }
   }
  }
 }
 $580 = HEAP32[$1>>2]|0;
 $581 = ((($580)) + 8|0);
 $582 = HEAP32[$581>>2]|0;
 $583 = ($582|0)!=(0|0);
 if ($583) {
  $584 = HEAP32[$1>>2]|0;
  $585 = ((($584)) + 40|0);
  $586 = HEAP16[$585>>1]|0;
  $587 = $586&65535;
  $588 = $587 & 128;
  $589 = ($588|0)!=(0);
  $590 = HEAP32[$pFrom>>2]|0;
  $591 = ((($590)) + 22|0);
  $592 = HEAP8[$591>>0]|0;
  if ($589) {
   $593 = $592 << 24 >> 24;
   $594 = HEAP32[$1>>2]|0;
   $595 = ((($594)) + 8|0);
   $596 = HEAP32[$595>>2]|0;
   $597 = HEAP32[$596>>2]|0;
   $598 = ($593|0)==($597|0);
   if ($598) {
    $599 = HEAP32[$1>>2]|0;
    $600 = ((($599)) + 47|0);
    HEAP8[$600>>0] = 2;
   }
  } else {
   $601 = HEAP32[$1>>2]|0;
   $602 = ((($601)) + 43|0);
   HEAP8[$602>>0] = $592;
   $603 = HEAP32[$pFrom>>2]|0;
   $604 = ((($603)) + 8|0);
   $605 = $604;
   $606 = $605;
   $607 = HEAP32[$606>>2]|0;
   $608 = (($605) + 4)|0;
   $609 = $608;
   $610 = HEAP32[$609>>2]|0;
   $611 = HEAP32[$1>>2]|0;
   $612 = ((($611)) + 64|0);
   $613 = $612;
   $614 = $613;
   HEAP32[$614>>2] = $607;
   $615 = (($613) + 4)|0;
   $616 = $615;
   HEAP32[$616>>2] = $610;
   $617 = HEAP32[$1>>2]|0;
   $618 = ((($617)) + 43|0);
   $619 = HEAP8[$618>>0]|0;
   $620 = $619 << 24 >> 24;
   $621 = ($620|0)<=(0);
   if ($621) {
    $622 = HEAP32[$1>>2]|0;
    $623 = ((($622)) + 43|0);
    HEAP8[$623>>0] = 0;
    $624 = HEAP32[$nLoop>>2]|0;
    $625 = ($624|0)>(0);
    if ($625) {
     $626 = HEAP32[$nLoop>>2]|0;
     $627 = (($626) - 1)|0;
     $628 = HEAP32[$pFrom>>2]|0;
     $629 = ((($628)) + 24|0);
     $630 = HEAP32[$629>>2]|0;
     $631 = (($630) + ($627<<2)|0);
     $632 = HEAP32[$631>>2]|0;
     $633 = ((($632)) + 36|0);
     $634 = HEAP32[$633>>2]|0;
     HEAP32[$wsFlags>>2] = $634;
     $635 = HEAP32[$wsFlags>>2]|0;
     $636 = $635 & 4096;
     $637 = ($636|0)==(0);
     if ($637) {
      $638 = HEAP32[$wsFlags>>2]|0;
      $639 = $638 & 260;
      $640 = ($639|0)!=(260);
      if ($640) {
       $641 = $m;
       $642 = $641;
       HEAP32[$642>>2] = 0;
       $643 = (($641) + 4)|0;
       $644 = $643;
       HEAP32[$644>>2] = 0;
       $645 = HEAP32[$1>>2]|0;
       $646 = HEAP32[$1>>2]|0;
       $647 = ((($646)) + 8|0);
       $648 = HEAP32[$647>>2]|0;
       $649 = HEAP32[$pFrom>>2]|0;
       $650 = HEAP32[$nLoop>>2]|0;
       $651 = (($650) - 1)|0;
       $652 = $651&65535;
       $653 = HEAP32[$nLoop>>2]|0;
       $654 = (($653) - 1)|0;
       $655 = HEAP32[$pFrom>>2]|0;
       $656 = ((($655)) + 24|0);
       $657 = HEAP32[$656>>2]|0;
       $658 = (($657) + ($654<<2)|0);
       $659 = HEAP32[$658>>2]|0;
       $660 = (_wherePathSatisfiesOrderBy($645,$648,$649,2048,$652,$659,$m)|0);
       $661 = $660 << 24 >> 24;
       HEAP32[$rc1>>2] = $661;
       $662 = HEAP32[$rc1>>2]|0;
       $663 = HEAP32[$1>>2]|0;
       $664 = ((($663)) + 8|0);
       $665 = HEAP32[$664>>2]|0;
       $666 = HEAP32[$665>>2]|0;
       $667 = ($662|0)==($666|0);
       if ($667) {
        $668 = HEAP32[$1>>2]|0;
        $669 = ((($668)) + 48|0);
        HEAP8[$669>>0] = 1;
        $670 = $m;
        $671 = $670;
        $672 = HEAP32[$671>>2]|0;
        $673 = (($670) + 4)|0;
        $674 = $673;
        $675 = HEAP32[$674>>2]|0;
        $676 = HEAP32[$1>>2]|0;
        $677 = ((($676)) + 64|0);
        $678 = $677;
        $679 = $678;
        HEAP32[$679>>2] = $672;
        $680 = (($678) + 4)|0;
        $681 = $680;
        HEAP32[$681>>2] = $675;
       }
      }
     }
    }
   }
  }
  $682 = HEAP32[$1>>2]|0;
  $683 = ((($682)) + 40|0);
  $684 = HEAP16[$683>>1]|0;
  $685 = $684&65535;
  $686 = $685 & 512;
  $687 = ($686|0)!=(0);
  if ($687) {
   $688 = HEAP32[$1>>2]|0;
   $689 = ((($688)) + 43|0);
   $690 = HEAP8[$689>>0]|0;
   $691 = $690 << 24 >> 24;
   $692 = HEAP32[$1>>2]|0;
   $693 = ((($692)) + 8|0);
   $694 = HEAP32[$693>>2]|0;
   $695 = HEAP32[$694>>2]|0;
   $696 = ($691|0)==($695|0);
   $697 = HEAP32[$nLoop>>2]|0;
   $698 = ($697|0)>(0);
   $or$cond = $696 & $698;
   if ($or$cond) {
    $699 = $revMask2;
    $700 = $699;
    HEAP32[$700>>2] = 0;
    $701 = (($699) + 4)|0;
    $702 = $701;
    HEAP32[$702>>2] = 0;
    $703 = HEAP32[$1>>2]|0;
    $704 = HEAP32[$1>>2]|0;
    $705 = ((($704)) + 8|0);
    $706 = HEAP32[$705>>2]|0;
    $707 = HEAP32[$pFrom>>2]|0;
    $708 = HEAP32[$nLoop>>2]|0;
    $709 = (($708) - 1)|0;
    $710 = $709&65535;
    $711 = HEAP32[$nLoop>>2]|0;
    $712 = (($711) - 1)|0;
    $713 = HEAP32[$pFrom>>2]|0;
    $714 = ((($713)) + 24|0);
    $715 = HEAP32[$714>>2]|0;
    $716 = (($715) + ($712<<2)|0);
    $717 = HEAP32[$716>>2]|0;
    $718 = (_wherePathSatisfiesOrderBy($703,$706,$707,0,$710,$717,$revMask2)|0);
    $719 = $718 << 24 >> 24;
    HEAP32[$nOrder>>2] = $719;
    $720 = HEAP32[$nOrder>>2]|0;
    $721 = HEAP32[$1>>2]|0;
    $722 = ((($721)) + 8|0);
    $723 = HEAP32[$722>>2]|0;
    $724 = HEAP32[$723>>2]|0;
    $725 = ($720|0)==($724|0);
    if ($725) {
     $726 = HEAP32[$1>>2]|0;
     $727 = ((($726)) + 44|0);
     HEAP8[$727>>0] = 1;
     $728 = $revMask2;
     $729 = $728;
     $730 = HEAP32[$729>>2]|0;
     $731 = (($728) + 4)|0;
     $732 = $731;
     $733 = HEAP32[$732>>2]|0;
     $734 = HEAP32[$1>>2]|0;
     $735 = ((($734)) + 64|0);
     $736 = $735;
     $737 = $736;
     HEAP32[$737>>2] = $730;
     $738 = (($736) + 4)|0;
     $739 = $738;
     HEAP32[$739>>2] = $733;
    }
   }
  }
 }
 $740 = HEAP32[$pFrom>>2]|0;
 $741 = ((($740)) + 16|0);
 $742 = HEAP16[$741>>1]|0;
 $743 = HEAP32[$1>>2]|0;
 $744 = ((($743)) + 72|0);
 HEAP16[$744>>1] = $742;
 $745 = HEAP32[$db>>2]|0;
 $746 = HEAP32[$pSpace>>2]|0;
 _sqlite3DbFree($745,$746);
 HEAP32[$0>>2] = 0;
 $747 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($747|0);
}
function _sqlite3WhereExprListUsage($pMaskSet,$pList) {
 $pMaskSet = $pMaskSet|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $mask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp + 8|0;
 $mask = sp;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $pList;
 $2 = $mask;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $35 = $mask;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = (($35) + 4)|0;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  tempRet0 = ($40);
  STACKTOP = sp;return ($37|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = ($8|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*20)|0)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3WhereExprUsage($12,$18)|0);
  $20 = tempRet0;
  $21 = $mask;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = $23 | $19;
  $28 = $26 | $20;
  $29 = $mask;
  $30 = $29;
  HEAP32[$30>>2] = $27;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = $mask;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 tempRet0 = ($40);
 STACKTOP = sp;return ($37|0);
}
function _constructAutomaticIndex($pParse,$pWC,$pSrc,$0,$1,$pLevel) {
 $pParse = $pParse|0;
 $pWC = $pWC|0;
 $pSrc = $pSrc|0;
 $0 = $0|0;
 $1 = $1|0;
 $pLevel = $pLevel|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0;
 var $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0;
 var $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0;
 var $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0;
 var $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addrCounter = 0, $addrInit = 0, $addrTop = 0, $cMask = 0, $cMask2 = 0, $extraCols = 0, $i = 0, $iCol = 0, $iCol1 = 0, $iContinue = 0, $idxCols = 0, $mxBitCol = 0, $n = 0, $nKeyCol = 0, $pColl = 0, $pExpr = 0, $pIdx = 0, $pLoop = 0, $pPartial = 0;
 var $pTabItem = 0, $pTable = 0, $pTerm = 0, $pWCEnd = 0, $pX = 0, $regBase = 0, $regRecord = 0, $regYield = 0, $sentWarning = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $zNotUsed = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 40|0;
 $2 = sp + 160|0;
 $3 = sp + 156|0;
 $4 = sp + 152|0;
 $5 = sp + 32|0;
 $6 = sp + 148|0;
 $nKeyCol = sp + 144|0;
 $pTerm = sp + 140|0;
 $pWCEnd = sp + 136|0;
 $pIdx = sp + 132|0;
 $v = sp + 128|0;
 $addrInit = sp + 124|0;
 $pTable = sp + 120|0;
 $addrTop = sp + 116|0;
 $regRecord = sp + 112|0;
 $n = sp + 108|0;
 $i = sp + 104|0;
 $mxBitCol = sp + 100|0;
 $pColl = sp + 96|0;
 $pLoop = sp + 92|0;
 $zNotUsed = sp + 88|0;
 $idxCols = sp + 24|0;
 $extraCols = sp + 16|0;
 $sentWarning = sp + 164|0;
 $pPartial = sp + 84|0;
 $iContinue = sp + 80|0;
 $pTabItem = sp + 76|0;
 $addrCounter = sp + 72|0;
 $regBase = sp + 68|0;
 $pExpr = sp + 64|0;
 $iCol = sp + 60|0;
 $cMask = sp + 8|0;
 $iCol1 = sp + 56|0;
 $cMask2 = sp;
 $pX = sp + 52|0;
 $regYield = sp + 48|0;
 HEAP32[$2>>2] = $pParse;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $pSrc;
 $7 = $5;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$6>>2] = $pLevel;
 HEAP8[$sentWarning>>0] = 0;
 HEAP32[$pPartial>>2] = 0;
 HEAP32[$iContinue>>2] = 0;
 HEAP32[$addrCounter>>2] = 0;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$v>>2] = $13;
 $14 = HEAP32[$v>>2]|0;
 $15 = (_sqlite3VdbeAddOp0($14,20)|0);
 HEAP32[$addrInit>>2] = $15;
 HEAP32[$nKeyCol>>2] = 0;
 $16 = HEAP32[$4>>2]|0;
 $17 = ((($16)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$pTable>>2] = $18;
 $19 = HEAP32[$3>>2]|0;
 $20 = ((($19)) + 12|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$3>>2]|0;
 $23 = ((($22)) + 20|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) + (($21*48)|0)|0);
 HEAP32[$pWCEnd>>2] = $25;
 $26 = HEAP32[$6>>2]|0;
 $27 = ((($26)) + 64|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pLoop>>2] = $28;
 $29 = $idxCols;
 $30 = $29;
 HEAP32[$30>>2] = 0;
 $31 = (($29) + 4)|0;
 $32 = $31;
 HEAP32[$32>>2] = 0;
 $33 = HEAP32[$3>>2]|0;
 $34 = ((($33)) + 20|0);
 $35 = HEAP32[$34>>2]|0;
 HEAP32[$pTerm>>2] = $35;
 while(1) {
  $36 = HEAP32[$pTerm>>2]|0;
  $37 = HEAP32[$pWCEnd>>2]|0;
  $38 = ($36>>>0)<($37>>>0);
  if (!($38)) {
   break;
  }
  $39 = HEAP32[$pTerm>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$pExpr>>2] = $40;
  $41 = HEAP32[$pLoop>>2]|0;
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = ($44|0)==(0);
  $49 = ($47|0)==(0);
  $50 = $48 & $49;
  if ($50) {
   $51 = HEAP32[$pTerm>>2]|0;
   $52 = ((($51)) + 10|0);
   $53 = HEAP16[$52>>1]|0;
   $54 = $53&65535;
   $55 = $54 & 2;
   $56 = ($55|0)==(0);
   if ($56) {
    $57 = HEAP32[$pExpr>>2]|0;
    $58 = ((($57)) + 4|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = $59 & 1;
    $61 = ($60|0)!=(0);
    if (!($61)) {
     $62 = HEAP32[$pExpr>>2]|0;
     $63 = HEAP32[$4>>2]|0;
     $64 = ((($63)) + 44|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = (_sqlite3ExprIsTableConstant($62,$65)|0);
     $67 = ($66|0)!=(0);
     if ($67) {
      $68 = HEAP32[$2>>2]|0;
      $69 = HEAP32[$68>>2]|0;
      $70 = HEAP32[$pPartial>>2]|0;
      $71 = HEAP32[$2>>2]|0;
      $72 = HEAP32[$71>>2]|0;
      $73 = HEAP32[$pExpr>>2]|0;
      $74 = (_sqlite3ExprDup($72,$73,0)|0);
      $75 = (_sqlite3ExprAnd($69,$70,$74)|0);
      HEAP32[$pPartial>>2] = $75;
     }
    }
   }
  }
  $76 = HEAP32[$pTerm>>2]|0;
  $77 = HEAP32[$4>>2]|0;
  $78 = $5;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = (_termCanDriveIndex($76,$77,$80,$83)|0);
  $85 = ($84|0)!=(0);
  if ($85) {
   $86 = HEAP32[$pTerm>>2]|0;
   $87 = ((($86)) + 28|0);
   $88 = HEAP32[$87>>2]|0;
   HEAP32[$iCol>>2] = $88;
   $89 = HEAP32[$iCol>>2]|0;
   $90 = ($89|0)>=(64);
   if ($90) {
    $96 = 0;$99 = -2147483648;
   } else {
    $91 = HEAP32[$iCol>>2]|0;
    $92 = (_bitshift64Shl(1,0,($91|0))|0);
    $93 = tempRet0;
    $96 = $92;$99 = $93;
   }
   $94 = $cMask;
   $95 = $94;
   HEAP32[$95>>2] = $96;
   $97 = (($94) + 4)|0;
   $98 = $97;
   HEAP32[$98>>2] = $99;
   $100 = HEAP8[$sentWarning>>0]|0;
   $101 = ($100<<24>>24)!=(0);
   if (!($101)) {
    $102 = HEAP32[$pTable>>2]|0;
    $103 = HEAP32[$102>>2]|0;
    $104 = HEAP32[$iCol>>2]|0;
    $105 = HEAP32[$pTable>>2]|0;
    $106 = ((($105)) + 4|0);
    $107 = HEAP32[$106>>2]|0;
    $108 = (($107) + ($104<<4)|0);
    $109 = HEAP32[$108>>2]|0;
    HEAP32[$vararg_buffer>>2] = $103;
    $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr1>>2] = $109;
    _sqlite3_log(284,34719,$vararg_buffer);
    HEAP8[$sentWarning>>0] = 1;
   }
   $110 = $idxCols;
   $111 = $110;
   $112 = HEAP32[$111>>2]|0;
   $113 = (($110) + 4)|0;
   $114 = $113;
   $115 = HEAP32[$114>>2]|0;
   $116 = $cMask;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($116) + 4)|0;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = $112 & $118;
   $123 = $115 & $121;
   $124 = ($122|0)==(0);
   $125 = ($123|0)==(0);
   $126 = $124 & $125;
   if ($126) {
    $127 = HEAP32[$2>>2]|0;
    $128 = HEAP32[$127>>2]|0;
    $129 = HEAP32[$pLoop>>2]|0;
    $130 = HEAP32[$nKeyCol>>2]|0;
    $131 = (($130) + 1)|0;
    $132 = (_whereLoopResize($128,$129,$131)|0);
    $133 = ($132|0)!=(0);
    if ($133) {
     label = 57;
     break;
    }
    $134 = HEAP32[$pTerm>>2]|0;
    $135 = HEAP32[$nKeyCol>>2]|0;
    $136 = (($135) + 1)|0;
    HEAP32[$nKeyCol>>2] = $136;
    $137 = HEAP32[$pLoop>>2]|0;
    $138 = ((($137)) + 48|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = (($139) + ($135<<2)|0);
    HEAP32[$140>>2] = $134;
    $141 = $cMask;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = (($141) + 4)|0;
    $145 = $144;
    $146 = HEAP32[$145>>2]|0;
    $147 = $idxCols;
    $148 = $147;
    $149 = HEAP32[$148>>2]|0;
    $150 = (($147) + 4)|0;
    $151 = $150;
    $152 = HEAP32[$151>>2]|0;
    $153 = $149 | $143;
    $154 = $152 | $146;
    $155 = $idxCols;
    $156 = $155;
    HEAP32[$156>>2] = $153;
    $157 = (($155) + 4)|0;
    $158 = $157;
    HEAP32[$158>>2] = $154;
   }
  }
  $159 = HEAP32[$pTerm>>2]|0;
  $160 = ((($159)) + 48|0);
  HEAP32[$pTerm>>2] = $160;
 }
 if ((label|0) == 57) {
  $560 = HEAP32[$2>>2]|0;
  $561 = HEAP32[$560>>2]|0;
  $562 = HEAP32[$pPartial>>2]|0;
  _sqlite3ExprDelete($561,$562);
  STACKTOP = sp;return;
 }
 $161 = HEAP32[$nKeyCol>>2]|0;
 $162 = $161&65535;
 $163 = HEAP32[$pLoop>>2]|0;
 $164 = ((($163)) + 40|0);
 HEAP16[$164>>1] = $162;
 $165 = HEAP32[$pLoop>>2]|0;
 $166 = ((($165)) + 24|0);
 HEAP16[$166>>1] = $162;
 $167 = HEAP32[$pLoop>>2]|0;
 $168 = ((($167)) + 36|0);
 HEAP32[$168>>2] = 16961;
 $169 = HEAP32[$4>>2]|0;
 $170 = ((($169)) + 56|0);
 $171 = $170;
 $172 = $171;
 $173 = HEAP32[$172>>2]|0;
 $174 = (($171) + 4)|0;
 $175 = $174;
 $176 = HEAP32[$175>>2]|0;
 $177 = $idxCols;
 $178 = $177;
 $179 = HEAP32[$178>>2]|0;
 $180 = (($177) + 4)|0;
 $181 = $180;
 $182 = HEAP32[$181>>2]|0;
 $183 = $179 ^ -1;
 $184 = $182 ^ -1;
 $185 = $184 | -2147483648;
 $186 = $173 & $183;
 $187 = $176 & $185;
 $188 = $extraCols;
 $189 = $188;
 HEAP32[$189>>2] = $186;
 $190 = (($188) + 4)|0;
 $191 = $190;
 HEAP32[$191>>2] = $187;
 $192 = HEAP32[$pTable>>2]|0;
 $193 = ((($192)) + 34|0);
 $194 = HEAP16[$193>>1]|0;
 $195 = $194 << 16 >> 16;
 $196 = (63)<($195|0);
 if ($196) {
  $201 = 63;
 } else {
  $197 = HEAP32[$pTable>>2]|0;
  $198 = ((($197)) + 34|0);
  $199 = HEAP16[$198>>1]|0;
  $200 = $199 << 16 >> 16;
  $201 = $200;
 }
 HEAP32[$mxBitCol>>2] = $201;
 HEAP32[$i>>2] = 0;
 while(1) {
  $202 = HEAP32[$i>>2]|0;
  $203 = HEAP32[$mxBitCol>>2]|0;
  $204 = ($202|0)<($203|0);
  if (!($204)) {
   break;
  }
  $205 = $extraCols;
  $206 = $205;
  $207 = HEAP32[$206>>2]|0;
  $208 = (($205) + 4)|0;
  $209 = $208;
  $210 = HEAP32[$209>>2]|0;
  $211 = HEAP32[$i>>2]|0;
  $212 = (_bitshift64Shl(1,0,($211|0))|0);
  $213 = tempRet0;
  $214 = $207 & $212;
  $215 = $210 & $213;
  $216 = ($214|0)!=(0);
  $217 = ($215|0)!=(0);
  $218 = $216 | $217;
  if ($218) {
   $219 = HEAP32[$nKeyCol>>2]|0;
   $220 = (($219) + 1)|0;
   HEAP32[$nKeyCol>>2] = $220;
  }
  $221 = HEAP32[$i>>2]|0;
  $222 = (($221) + 1)|0;
  HEAP32[$i>>2] = $222;
 }
 $223 = HEAP32[$4>>2]|0;
 $224 = ((($223)) + 56|0);
 $225 = $224;
 $226 = $225;
 $227 = HEAP32[$226>>2]|0;
 $228 = (($225) + 4)|0;
 $229 = $228;
 $230 = HEAP32[$229>>2]|0;
 $231 = $230 & -2147483648;
 $232 = (0)!=(0);
 $233 = ($231|0)!=(0);
 $234 = $232 | $233;
 if ($234) {
  $235 = HEAP32[$pTable>>2]|0;
  $236 = ((($235)) + 34|0);
  $237 = HEAP16[$236>>1]|0;
  $238 = $237 << 16 >> 16;
  $239 = (($238) - 64)|0;
  $240 = (($239) + 1)|0;
  $241 = HEAP32[$nKeyCol>>2]|0;
  $242 = (($241) + ($240))|0;
  HEAP32[$nKeyCol>>2] = $242;
 }
 $243 = HEAP32[$2>>2]|0;
 $244 = HEAP32[$243>>2]|0;
 $245 = HEAP32[$nKeyCol>>2]|0;
 $246 = (($245) + 1)|0;
 $247 = $246&65535;
 $248 = (_sqlite3AllocateIndexObject($244,$247,0,$zNotUsed)|0);
 HEAP32[$pIdx>>2] = $248;
 $249 = HEAP32[$pIdx>>2]|0;
 $250 = ($249|0)==(0|0);
 if ($250) {
  $560 = HEAP32[$2>>2]|0;
  $561 = HEAP32[$560>>2]|0;
  $562 = HEAP32[$pPartial>>2]|0;
  _sqlite3ExprDelete($561,$562);
  STACKTOP = sp;return;
 }
 $251 = HEAP32[$pIdx>>2]|0;
 $252 = HEAP32[$pLoop>>2]|0;
 $253 = ((($252)) + 24|0);
 $254 = ((($253)) + 8|0);
 HEAP32[$254>>2] = $251;
 $255 = HEAP32[$pIdx>>2]|0;
 HEAP32[$255>>2] = 34745;
 $256 = HEAP32[$pTable>>2]|0;
 $257 = HEAP32[$pIdx>>2]|0;
 $258 = ((($257)) + 12|0);
 HEAP32[$258>>2] = $256;
 HEAP32[$n>>2] = 0;
 $259 = $idxCols;
 $260 = $259;
 HEAP32[$260>>2] = 0;
 $261 = (($259) + 4)|0;
 $262 = $261;
 HEAP32[$262>>2] = 0;
 $263 = HEAP32[$3>>2]|0;
 $264 = ((($263)) + 20|0);
 $265 = HEAP32[$264>>2]|0;
 HEAP32[$pTerm>>2] = $265;
 while(1) {
  $266 = HEAP32[$pTerm>>2]|0;
  $267 = HEAP32[$pWCEnd>>2]|0;
  $268 = ($266>>>0)<($267>>>0);
  if (!($268)) {
   break;
  }
  $269 = HEAP32[$pTerm>>2]|0;
  $270 = HEAP32[$4>>2]|0;
  $271 = $5;
  $272 = $271;
  $273 = HEAP32[$272>>2]|0;
  $274 = (($271) + 4)|0;
  $275 = $274;
  $276 = HEAP32[$275>>2]|0;
  $277 = (_termCanDriveIndex($269,$270,$273,$276)|0);
  $278 = ($277|0)!=(0);
  if ($278) {
   $279 = HEAP32[$pTerm>>2]|0;
   $280 = ((($279)) + 28|0);
   $281 = HEAP32[$280>>2]|0;
   HEAP32[$iCol1>>2] = $281;
   $282 = HEAP32[$iCol1>>2]|0;
   $283 = ($282|0)>=(64);
   if ($283) {
    $289 = 0;$292 = -2147483648;
   } else {
    $284 = HEAP32[$iCol1>>2]|0;
    $285 = (_bitshift64Shl(1,0,($284|0))|0);
    $286 = tempRet0;
    $289 = $285;$292 = $286;
   }
   $287 = $cMask2;
   $288 = $287;
   HEAP32[$288>>2] = $289;
   $290 = (($287) + 4)|0;
   $291 = $290;
   HEAP32[$291>>2] = $292;
   $293 = $idxCols;
   $294 = $293;
   $295 = HEAP32[$294>>2]|0;
   $296 = (($293) + 4)|0;
   $297 = $296;
   $298 = HEAP32[$297>>2]|0;
   $299 = $cMask2;
   $300 = $299;
   $301 = HEAP32[$300>>2]|0;
   $302 = (($299) + 4)|0;
   $303 = $302;
   $304 = HEAP32[$303>>2]|0;
   $305 = $295 & $301;
   $306 = $298 & $304;
   $307 = ($305|0)==(0);
   $308 = ($306|0)==(0);
   $309 = $307 & $308;
   if ($309) {
    $310 = HEAP32[$pTerm>>2]|0;
    $311 = HEAP32[$310>>2]|0;
    HEAP32[$pX>>2] = $311;
    $312 = $cMask2;
    $313 = $312;
    $314 = HEAP32[$313>>2]|0;
    $315 = (($312) + 4)|0;
    $316 = $315;
    $317 = HEAP32[$316>>2]|0;
    $318 = $idxCols;
    $319 = $318;
    $320 = HEAP32[$319>>2]|0;
    $321 = (($318) + 4)|0;
    $322 = $321;
    $323 = HEAP32[$322>>2]|0;
    $324 = $320 | $314;
    $325 = $323 | $317;
    $326 = $idxCols;
    $327 = $326;
    HEAP32[$327>>2] = $324;
    $328 = (($326) + 4)|0;
    $329 = $328;
    HEAP32[$329>>2] = $325;
    $330 = HEAP32[$pTerm>>2]|0;
    $331 = ((($330)) + 28|0);
    $332 = HEAP32[$331>>2]|0;
    $333 = $332&65535;
    $334 = HEAP32[$n>>2]|0;
    $335 = HEAP32[$pIdx>>2]|0;
    $336 = ((($335)) + 4|0);
    $337 = HEAP32[$336>>2]|0;
    $338 = (($337) + ($334<<1)|0);
    HEAP16[$338>>1] = $333;
    $339 = HEAP32[$2>>2]|0;
    $340 = HEAP32[$pX>>2]|0;
    $341 = ((($340)) + 12|0);
    $342 = HEAP32[$341>>2]|0;
    $343 = HEAP32[$pX>>2]|0;
    $344 = ((($343)) + 16|0);
    $345 = HEAP32[$344>>2]|0;
    $346 = (_sqlite3BinaryCompareCollSeq($339,$342,$345)|0);
    HEAP32[$pColl>>2] = $346;
    $347 = HEAP32[$pColl>>2]|0;
    $348 = ($347|0)!=(0|0);
    if ($348) {
     $349 = HEAP32[$pColl>>2]|0;
     $350 = HEAP32[$349>>2]|0;
     $356 = $350;
    } else {
     $356 = 34834;
    }
    $351 = HEAP32[$n>>2]|0;
    $352 = HEAP32[$pIdx>>2]|0;
    $353 = ((($352)) + 32|0);
    $354 = HEAP32[$353>>2]|0;
    $355 = (($354) + ($351<<2)|0);
    HEAP32[$355>>2] = $356;
    $357 = HEAP32[$n>>2]|0;
    $358 = (($357) + 1)|0;
    HEAP32[$n>>2] = $358;
   }
  }
  $359 = HEAP32[$pTerm>>2]|0;
  $360 = ((($359)) + 48|0);
  HEAP32[$pTerm>>2] = $360;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $361 = HEAP32[$i>>2]|0;
  $362 = HEAP32[$mxBitCol>>2]|0;
  $363 = ($361|0)<($362|0);
  if (!($363)) {
   break;
  }
  $364 = $extraCols;
  $365 = $364;
  $366 = HEAP32[$365>>2]|0;
  $367 = (($364) + 4)|0;
  $368 = $367;
  $369 = HEAP32[$368>>2]|0;
  $370 = HEAP32[$i>>2]|0;
  $371 = (_bitshift64Shl(1,0,($370|0))|0);
  $372 = tempRet0;
  $373 = $366 & $371;
  $374 = $369 & $372;
  $375 = ($373|0)!=(0);
  $376 = ($374|0)!=(0);
  $377 = $375 | $376;
  if ($377) {
   $378 = HEAP32[$i>>2]|0;
   $379 = $378&65535;
   $380 = HEAP32[$n>>2]|0;
   $381 = HEAP32[$pIdx>>2]|0;
   $382 = ((($381)) + 4|0);
   $383 = HEAP32[$382>>2]|0;
   $384 = (($383) + ($380<<1)|0);
   HEAP16[$384>>1] = $379;
   $385 = HEAP32[$n>>2]|0;
   $386 = HEAP32[$pIdx>>2]|0;
   $387 = ((($386)) + 32|0);
   $388 = HEAP32[$387>>2]|0;
   $389 = (($388) + ($385<<2)|0);
   HEAP32[$389>>2] = 34834;
   $390 = HEAP32[$n>>2]|0;
   $391 = (($390) + 1)|0;
   HEAP32[$n>>2] = $391;
  }
  $392 = HEAP32[$i>>2]|0;
  $393 = (($392) + 1)|0;
  HEAP32[$i>>2] = $393;
 }
 $394 = HEAP32[$4>>2]|0;
 $395 = ((($394)) + 56|0);
 $396 = $395;
 $397 = $396;
 $398 = HEAP32[$397>>2]|0;
 $399 = (($396) + 4)|0;
 $400 = $399;
 $401 = HEAP32[$400>>2]|0;
 $402 = $401 & -2147483648;
 $403 = (0)!=(0);
 $404 = ($402|0)!=(0);
 $405 = $403 | $404;
 L61: do {
  if ($405) {
   HEAP32[$i>>2] = 63;
   while(1) {
    $406 = HEAP32[$i>>2]|0;
    $407 = HEAP32[$pTable>>2]|0;
    $408 = ((($407)) + 34|0);
    $409 = HEAP16[$408>>1]|0;
    $410 = $409 << 16 >> 16;
    $411 = ($406|0)<($410|0);
    if (!($411)) {
     break L61;
    }
    $412 = HEAP32[$i>>2]|0;
    $413 = $412&65535;
    $414 = HEAP32[$n>>2]|0;
    $415 = HEAP32[$pIdx>>2]|0;
    $416 = ((($415)) + 4|0);
    $417 = HEAP32[$416>>2]|0;
    $418 = (($417) + ($414<<1)|0);
    HEAP16[$418>>1] = $413;
    $419 = HEAP32[$n>>2]|0;
    $420 = HEAP32[$pIdx>>2]|0;
    $421 = ((($420)) + 32|0);
    $422 = HEAP32[$421>>2]|0;
    $423 = (($422) + ($419<<2)|0);
    HEAP32[$423>>2] = 34834;
    $424 = HEAP32[$n>>2]|0;
    $425 = (($424) + 1)|0;
    HEAP32[$n>>2] = $425;
    $426 = HEAP32[$i>>2]|0;
    $427 = (($426) + 1)|0;
    HEAP32[$i>>2] = $427;
   }
  }
 } while(0);
 $428 = HEAP32[$n>>2]|0;
 $429 = HEAP32[$pIdx>>2]|0;
 $430 = ((($429)) + 4|0);
 $431 = HEAP32[$430>>2]|0;
 $432 = (($431) + ($428<<1)|0);
 HEAP16[$432>>1] = -1;
 $433 = HEAP32[$n>>2]|0;
 $434 = HEAP32[$pIdx>>2]|0;
 $435 = ((($434)) + 32|0);
 $436 = HEAP32[$435>>2]|0;
 $437 = (($436) + ($433<<2)|0);
 HEAP32[$437>>2] = 34834;
 $438 = HEAP32[$2>>2]|0;
 $439 = ((($438)) + 40|0);
 $440 = HEAP32[$439>>2]|0;
 $441 = (($440) + 1)|0;
 HEAP32[$439>>2] = $441;
 $442 = HEAP32[$6>>2]|0;
 $443 = ((($442)) + 8|0);
 HEAP32[$443>>2] = $440;
 $444 = HEAP32[$v>>2]|0;
 $445 = HEAP32[$6>>2]|0;
 $446 = ((($445)) + 8|0);
 $447 = HEAP32[$446>>2]|0;
 $448 = HEAP32[$nKeyCol>>2]|0;
 $449 = (($448) + 1)|0;
 (_sqlite3VdbeAddOp2($444,106,$447,$449)|0);
 $450 = HEAP32[$2>>2]|0;
 $451 = HEAP32[$pIdx>>2]|0;
 _sqlite3VdbeSetP4KeyInfo($450,$451);
 $452 = HEAP32[$2>>2]|0;
 _sqlite3ExprCachePush($452);
 $453 = HEAP32[$6>>2]|0;
 $454 = ((($453)) + 44|0);
 $455 = HEAP8[$454>>0]|0;
 $456 = $455&255;
 $457 = HEAP32[$3>>2]|0;
 $458 = HEAP32[$457>>2]|0;
 $459 = ((($458)) + 4|0);
 $460 = HEAP32[$459>>2]|0;
 $461 = ((($460)) + 8|0);
 $462 = (($461) + (($456*72)|0)|0);
 HEAP32[$pTabItem>>2] = $462;
 $463 = HEAP32[$pTabItem>>2]|0;
 $464 = ((($463)) + 36|0);
 $465 = ((($464)) + 1|0);
 $466 = HEAP8[$465>>0]|0;
 $467 = ($466&255) >>> 4;
 $468 = $467 & 1;
 $469 = $468&255;
 $470 = ($469|0)!=(0);
 if ($470) {
  $471 = HEAP32[$pTabItem>>2]|0;
  $472 = ((($471)) + 28|0);
  $473 = HEAP32[$472>>2]|0;
  HEAP32[$regYield>>2] = $473;
  $474 = HEAP32[$v>>2]|0;
  $475 = (_sqlite3VdbeAddOp2($474,76,0,0)|0);
  HEAP32[$addrCounter>>2] = $475;
  $476 = HEAP32[$v>>2]|0;
  $477 = HEAP32[$regYield>>2]|0;
  $478 = HEAP32[$pTabItem>>2]|0;
  $479 = ((($478)) + 24|0);
  $480 = HEAP32[$479>>2]|0;
  (_sqlite3VdbeAddOp3($476,15,$477,0,$480)|0);
  $481 = HEAP32[$v>>2]|0;
  $482 = HEAP32[$regYield>>2]|0;
  $483 = (_sqlite3VdbeAddOp1($481,16,$482)|0);
  HEAP32[$addrTop>>2] = $483;
 } else {
  $484 = HEAP32[$v>>2]|0;
  $485 = HEAP32[$6>>2]|0;
  $486 = ((($485)) + 4|0);
  $487 = HEAP32[$486>>2]|0;
  $488 = (_sqlite3VdbeAddOp1($484,57,$487)|0);
  HEAP32[$addrTop>>2] = $488;
 }
 $489 = HEAP32[$pPartial>>2]|0;
 $490 = ($489|0)!=(0|0);
 if ($490) {
  $491 = HEAP32[$v>>2]|0;
  $492 = (_sqlite3VdbeMakeLabel($491)|0);
  HEAP32[$iContinue>>2] = $492;
  $493 = HEAP32[$2>>2]|0;
  $494 = HEAP32[$pPartial>>2]|0;
  $495 = HEAP32[$iContinue>>2]|0;
  _sqlite3ExprIfFalse($493,$494,$495,16);
  $496 = HEAP32[$pLoop>>2]|0;
  $497 = ((($496)) + 36|0);
  $498 = HEAP32[$497>>2]|0;
  $499 = $498 | 131072;
  HEAP32[$497>>2] = $499;
 }
 $500 = HEAP32[$2>>2]|0;
 $501 = (_sqlite3GetTempReg($500)|0);
 HEAP32[$regRecord>>2] = $501;
 $502 = HEAP32[$2>>2]|0;
 $503 = HEAP32[$pIdx>>2]|0;
 $504 = HEAP32[$6>>2]|0;
 $505 = ((($504)) + 4|0);
 $506 = HEAP32[$505>>2]|0;
 $507 = HEAP32[$regRecord>>2]|0;
 $508 = (_sqlite3GenerateIndexKey($502,$503,$506,$507,0,0,0,0)|0);
 HEAP32[$regBase>>2] = $508;
 $509 = HEAP32[$v>>2]|0;
 $510 = HEAP32[$6>>2]|0;
 $511 = ((($510)) + 8|0);
 $512 = HEAP32[$511>>2]|0;
 $513 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($509,126,$512,$513)|0);
 $514 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($514,16);
 $515 = HEAP32[$pPartial>>2]|0;
 $516 = ($515|0)!=(0|0);
 if ($516) {
  $517 = HEAP32[$v>>2]|0;
  $518 = HEAP32[$iContinue>>2]|0;
  _sqlite3VdbeResolveLabel($517,$518);
 }
 $519 = HEAP32[$pTabItem>>2]|0;
 $520 = ((($519)) + 36|0);
 $521 = ((($520)) + 1|0);
 $522 = HEAP8[$521>>0]|0;
 $523 = ($522&255) >>> 4;
 $524 = $523 & 1;
 $525 = $524&255;
 $526 = ($525|0)!=(0);
 $527 = HEAP32[$v>>2]|0;
 if ($526) {
  $528 = HEAP32[$addrCounter>>2]|0;
  $529 = HEAP32[$regBase>>2]|0;
  $530 = HEAP32[$n>>2]|0;
  $531 = (($529) + ($530))|0;
  _sqlite3VdbeChangeP2($527,$528,$531);
  $532 = HEAP32[$v>>2]|0;
  $533 = HEAP32[$addrTop>>2]|0;
  $534 = HEAP32[$6>>2]|0;
  $535 = ((($534)) + 4|0);
  $536 = HEAP32[$535>>2]|0;
  $537 = HEAP32[$pTabItem>>2]|0;
  $538 = ((($537)) + 32|0);
  $539 = HEAP32[$538>>2]|0;
  _translateColumnToCopy($532,$533,$536,$539,1);
  $540 = HEAP32[$v>>2]|0;
  $541 = HEAP32[$addrTop>>2]|0;
  (_sqlite3VdbeGoto($540,$541)|0);
  $542 = HEAP32[$pTabItem>>2]|0;
  $543 = ((($542)) + 36|0);
  $544 = ((($543)) + 1|0);
  $545 = HEAP8[$544>>0]|0;
  $546 = $545 & -17;
  HEAP8[$544>>0] = $546;
 } else {
  $547 = HEAP32[$6>>2]|0;
  $548 = ((($547)) + 4|0);
  $549 = HEAP32[$548>>2]|0;
  $550 = HEAP32[$addrTop>>2]|0;
  $551 = (($550) + 1)|0;
  (_sqlite3VdbeAddOp2($527,7,$549,$551)|0);
 }
 $552 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($552,3);
 $553 = HEAP32[$v>>2]|0;
 $554 = HEAP32[$addrTop>>2]|0;
 _sqlite3VdbeJumpHere($553,$554);
 $555 = HEAP32[$2>>2]|0;
 $556 = HEAP32[$regRecord>>2]|0;
 _sqlite3ReleaseTempReg($555,$556);
 $557 = HEAP32[$2>>2]|0;
 _sqlite3ExprCachePop($557);
 $558 = HEAP32[$v>>2]|0;
 $559 = HEAP32[$addrInit>>2]|0;
 _sqlite3VdbeJumpHere($558,$559);
 $560 = HEAP32[$2>>2]|0;
 $561 = HEAP32[$560>>2]|0;
 $562 = HEAP32[$pPartial>>2]|0;
 _sqlite3ExprDelete($561,$562);
 STACKTOP = sp;return;
}
function _sqlite3WhereExplainOneScan($pParse,$pTabList,$pLevel,$iLevel,$iFrom,$wctrlFlags) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pLevel = $pLevel|0;
 $iLevel = $iLevel|0;
 $iFrom = $iFrom|0;
 $wctrlFlags = $wctrlFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $flags = 0, $iId = 0, $isSearch = 0, $pIdx = 0, $pItem = 0, $pLoop = 0, $ret = 0, $str = 0;
 var $v = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr16 = 0, $zBuf = 0, $zFmt = 0, $zMsg = 0, $zRangeOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer13 = sp + 40|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 144|0;
 $1 = sp + 140|0;
 $2 = sp + 136|0;
 $3 = sp + 132|0;
 $4 = sp + 128|0;
 $5 = sp + 124|0;
 $6 = sp + 148|0;
 $ret = sp + 120|0;
 $pItem = sp + 116|0;
 $v = sp + 112|0;
 $db = sp + 108|0;
 $iId = sp + 104|0;
 $isSearch = sp + 100|0;
 $pLoop = sp + 96|0;
 $flags = sp + 92|0;
 $zMsg = sp + 88|0;
 $str = sp + 60|0;
 $zBuf = sp + 152|0;
 $zFmt = sp + 56|0;
 $pIdx = sp + 52|0;
 $zRangeOp = sp + 48|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTabList;
 HEAP32[$3>>2] = $pLevel;
 HEAP32[$4>>2] = $iLevel;
 HEAP32[$5>>2] = $iFrom;
 HEAP16[$6>>1] = $wctrlFlags;
 HEAP32[$ret>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 409|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)==(2);
 do {
  if ($11) {
   $12 = HEAP32[$3>>2]|0;
   $13 = ((($12)) + 44|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 8|0);
   $18 = (($17) + (($15*72)|0)|0);
   HEAP32[$pItem>>2] = $18;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$v>>2] = $21;
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$db>>2] = $23;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 420|0);
   $26 = HEAP32[$25>>2]|0;
   HEAP32[$iId>>2] = $26;
   $27 = HEAP32[$3>>2]|0;
   $28 = ((($27)) + 64|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$pLoop>>2] = $29;
   $30 = HEAP32[$pLoop>>2]|0;
   $31 = ((($30)) + 36|0);
   $32 = HEAP32[$31>>2]|0;
   HEAP32[$flags>>2] = $32;
   $33 = HEAP32[$flags>>2]|0;
   $34 = $33 & 8192;
   $35 = ($34|0)!=(0);
   if (!($35)) {
    $36 = HEAP16[$6>>1]|0;
    $37 = $36&65535;
    $38 = $37 & 32;
    $39 = ($38|0)!=(0);
    if (!($39)) {
     $40 = HEAP32[$flags>>2]|0;
     $41 = $40 & 48;
     $42 = ($41|0)!=(0);
     do {
      if ($42) {
       $56 = 1;
      } else {
       $43 = HEAP32[$flags>>2]|0;
       $44 = $43 & 1024;
       $45 = ($44|0)==(0);
       if ($45) {
        $46 = HEAP32[$pLoop>>2]|0;
        $47 = ((($46)) + 24|0);
        $48 = HEAP16[$47>>1]|0;
        $49 = $48&65535;
        $50 = ($49|0)>(0);
        if ($50) {
         $56 = 1;
         break;
        }
       }
       $51 = HEAP16[$6>>1]|0;
       $52 = $51&65535;
       $53 = $52 & 3;
       $54 = ($53|0)!=(0);
       $56 = $54;
      }
     } while(0);
     $55 = $56&1;
     HEAP32[$isSearch>>2] = $55;
     $57 = HEAP32[$db>>2]|0;
     _sqlite3StrAccumInit($str,$57,$zBuf,100,1000000000);
     $58 = HEAP32[$isSearch>>2]|0;
     $59 = ($58|0)!=(0);
     $60 = $59 ? 34452 : 34459;
     _sqlite3StrAccumAppendAll($str,$60);
     $61 = HEAP32[$pItem>>2]|0;
     $62 = ((($61)) + 20|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = ($63|0)!=(0|0);
     $65 = HEAP32[$pItem>>2]|0;
     if ($64) {
      $66 = ((($65)) + 40|0);
      $67 = HEAP8[$66>>0]|0;
      $68 = $67&255;
      HEAP32[$vararg_buffer>>2] = $68;
      _sqlite3XPrintf($str,34464,$vararg_buffer);
     } else {
      $69 = ((($65)) + 8|0);
      $70 = HEAP32[$69>>2]|0;
      HEAP32[$vararg_buffer1>>2] = $70;
      _sqlite3XPrintf($str,34477,$vararg_buffer1);
     }
     $71 = HEAP32[$pItem>>2]|0;
     $72 = ((($71)) + 12|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = ($73|0)!=(0|0);
     if ($74) {
      $75 = HEAP32[$pItem>>2]|0;
      $76 = ((($75)) + 12|0);
      $77 = HEAP32[$76>>2]|0;
      HEAP32[$vararg_buffer4>>2] = $77;
      _sqlite3XPrintf($str,34487,$vararg_buffer4);
     }
     $78 = HEAP32[$flags>>2]|0;
     $79 = $78 & 1280;
     $80 = ($79|0)==(0);
     do {
      if ($80) {
       HEAP32[$zFmt>>2] = 0;
       $81 = HEAP32[$pLoop>>2]|0;
       $82 = ((($81)) + 24|0);
       $83 = ((($82)) + 8|0);
       $84 = HEAP32[$83>>2]|0;
       HEAP32[$pIdx>>2] = $84;
       $85 = HEAP32[$pItem>>2]|0;
       $86 = ((($85)) + 16|0);
       $87 = HEAP32[$86>>2]|0;
       $88 = ((($87)) + 42|0);
       $89 = HEAP8[$88>>0]|0;
       $90 = $89&255;
       $91 = $90 & 32;
       $92 = ($91|0)==(0);
       if ($92) {
        label = 19;
       } else {
        $93 = HEAP32[$pIdx>>2]|0;
        $94 = ((($93)) + 55|0);
        $95 = HEAP8[$94>>0]|0;
        $96 = $95 & 3;
        $97 = $96&255;
        $98 = ($97|0)==(2);
        if ($98) {
         $99 = HEAP32[$isSearch>>2]|0;
         $100 = ($99|0)!=(0);
         if ($100) {
          HEAP32[$zFmt>>2] = 34494;
         }
        } else {
         label = 19;
        }
       }
       do {
        if ((label|0) == 19) {
         $101 = HEAP32[$flags>>2]|0;
         $102 = $101 & 131072;
         $103 = ($102|0)!=(0);
         if ($103) {
          HEAP32[$zFmt>>2] = 34506;
          break;
         }
         $104 = HEAP32[$flags>>2]|0;
         $105 = $104 & 16384;
         $106 = ($105|0)!=(0);
         if ($106) {
          HEAP32[$zFmt>>2] = 34539;
          break;
         }
         $107 = HEAP32[$flags>>2]|0;
         $108 = $107 & 64;
         $109 = ($108|0)!=(0);
         if ($109) {
          HEAP32[$zFmt>>2] = 34564;
          break;
         } else {
          HEAP32[$zFmt>>2] = 34582;
          break;
         }
        }
       } while(0);
       $110 = HEAP32[$zFmt>>2]|0;
       $111 = ($110|0)!=(0|0);
       if ($111) {
        _sqlite3StrAccumAppend($str,34591,7);
        $112 = HEAP32[$zFmt>>2]|0;
        $113 = HEAP32[$pIdx>>2]|0;
        $114 = HEAP32[$113>>2]|0;
        HEAP32[$vararg_buffer7>>2] = $114;
        _sqlite3XPrintf($str,$112,$vararg_buffer7);
        $115 = HEAP32[$pLoop>>2]|0;
        _explainIndexRange($str,$115);
       }
      } else {
       $116 = HEAP32[$flags>>2]|0;
       $117 = $116 & 256;
       $118 = ($117|0)!=(0);
       if ($118) {
        $119 = HEAP32[$flags>>2]|0;
        $120 = $119 & 15;
        $121 = ($120|0)!=(0);
        if ($121) {
         $122 = HEAP32[$flags>>2]|0;
         $123 = $122 & 5;
         $124 = ($123|0)!=(0);
         do {
          if ($124) {
           HEAP32[$zRangeOp>>2] = 34599;
          } else {
           $125 = HEAP32[$flags>>2]|0;
           $126 = $125 & 48;
           $127 = ($126|0)==(48);
           if ($127) {
            HEAP32[$zRangeOp>>2] = 34601;
            break;
           }
           $128 = HEAP32[$flags>>2]|0;
           $129 = $128 & 32;
           $130 = ($129|0)!=(0);
           if ($130) {
            HEAP32[$zRangeOp>>2] = 34615;
            break;
           } else {
            HEAP32[$zRangeOp>>2] = 34617;
            break;
           }
          }
         } while(0);
         $131 = HEAP32[$zRangeOp>>2]|0;
         HEAP32[$vararg_buffer10>>2] = $131;
         _sqlite3XPrintf($str,34619,$vararg_buffer10);
         break;
        }
       }
       $132 = HEAP32[$flags>>2]|0;
       $133 = $132 & 1024;
       $134 = ($133|0)!=(0);
       if ($134) {
        $135 = HEAP32[$pLoop>>2]|0;
        $136 = ((($135)) + 24|0);
        $137 = HEAP32[$136>>2]|0;
        $138 = HEAP32[$pLoop>>2]|0;
        $139 = ((($138)) + 24|0);
        $140 = ((($139)) + 8|0);
        $141 = HEAP32[$140>>2]|0;
        HEAP32[$vararg_buffer13>>2] = $137;
        $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
        HEAP32[$vararg_ptr16>>2] = $141;
        _sqlite3XPrintf($str,34657,$vararg_buffer13);
       }
      }
     } while(0);
     $142 = (_sqlite3StrAccumFinish($str)|0);
     HEAP32[$zMsg>>2] = $142;
     $143 = HEAP32[$v>>2]|0;
     $144 = HEAP32[$iId>>2]|0;
     $145 = HEAP32[$4>>2]|0;
     $146 = HEAP32[$5>>2]|0;
     $147 = HEAP32[$zMsg>>2]|0;
     $148 = (_sqlite3VdbeAddOp4($143,162,$144,$145,$146,$147,-1)|0);
     HEAP32[$ret>>2] = $148;
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $150 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($150|0);
  }
 } while(0);
 $149 = HEAP32[$ret>>2]|0;
 HEAP32[$0>>2] = $149;
 $150 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($150|0);
}
function _sqlite3WhereCodeOneLoopStart($pWInfo,$iLevel,$0,$1) {
 $pWInfo = $pWInfo|0;
 $iLevel = $iLevel|0;
 $0 = $0|0;
 $1 = $1|0;
 var $$old1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0;
 var $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0;
 var $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0;
 var $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0;
 var $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0, $1088 = 0;
 var $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0, $1105 = 0;
 var $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0, $1123 = 0;
 var $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0, $1141 = 0;
 var $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0, $116 = 0;
 var $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0, $1178 = 0;
 var $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0, $1196 = 0;
 var $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0, $1213 = 0;
 var $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0, $1231 = 0;
 var $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0, $125 = 0;
 var $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0, $1268 = 0;
 var $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0, $1286 = 0;
 var $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0, $1303 = 0;
 var $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0, $1321 = 0;
 var $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0, $134 = 0;
 var $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0, $1358 = 0;
 var $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0, $1376 = 0;
 var $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0, $1392 = 0, $1393 = 0, $1394 = 0;
 var $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0, $141 = 0, $1410 = 0, $1411 = 0;
 var $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0, $1428 = 0, $1429 = 0, $143 = 0;
 var $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0, $1446 = 0, $1447 = 0, $1448 = 0;
 var $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0, $1464 = 0, $1465 = 0, $1466 = 0;
 var $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0, $1482 = 0, $1483 = 0, $1484 = 0;
 var $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0, $150 = 0, $1500 = 0, $1501 = 0;
 var $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0, $1518 = 0, $1519 = 0, $152 = 0;
 var $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0, $1536 = 0, $1537 = 0, $1538 = 0;
 var $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0, $1554 = 0, $1555 = 0, $1556 = 0;
 var $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0, $1572 = 0, $1573 = 0, $1574 = 0;
 var $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0, $1590 = 0, $1591 = 0, $1592 = 0;
 var $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0, $1608 = 0, $1609 = 0, $161 = 0;
 var $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0, $1626 = 0, $1627 = 0, $1628 = 0;
 var $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0, $1644 = 0, $1645 = 0, $1646 = 0;
 var $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0, $1662 = 0, $1663 = 0, $1664 = 0;
 var $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0, $1680 = 0, $1681 = 0, $1682 = 0;
 var $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0, $1699 = 0, $17 = 0, $170 = 0;
 var $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0, $1716 = 0, $1717 = 0, $1718 = 0;
 var $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0, $1734 = 0, $1735 = 0, $1736 = 0;
 var $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0;
 var $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0;
 var $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0;
 var $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0;
 var $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0;
 var $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0;
 var $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0;
 var $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0;
 var $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0;
 var $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0;
 var $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0;
 var $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0;
 var $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0;
 var $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0;
 var $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0;
 var $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0;
 var $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0;
 var $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0;
 var $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0;
 var $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0;
 var $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0;
 var $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0;
 var $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0;
 var $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0;
 var $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0;
 var $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0;
 var $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0;
 var $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0;
 var $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0;
 var $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0;
 var $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0;
 var $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0;
 var $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0;
 var $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0;
 var $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0;
 var $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0;
 var $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0;
 var $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0;
 var $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0;
 var $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0;
 var $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0;
 var $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0;
 var $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0;
 var $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0;
 var $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0;
 var $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0;
 var $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $addrBrk = 0, $addrCont = 0, $addrExplain = 0, $addrNotFound = 0, $addrNxt = 0, $bRev = 0, $bSeekPastNull = 0, $bStopAtNull = 0, $db = 0, $endEq = 0;
 var $iCol = 0, $iCovCur = 0, $iCur = 0, $iIdxCur = 0, $iIn = 0, $iLoopBody = 0, $iPk = 0, $iReg = 0, $iReleaseReg = 0, $iRetInit = 0, $iRowidReg = 0, $iSet = 0, $iTarget = 0, $iTerm = 0, $ii = 0, $j = 0, $jmp1 = 0, $k = 0, $memEndValue = 0, $nBtm = 0;
 var $nConstraint = 0, $nConstraint3 = 0, $nEq = 0, $nExtraReg = 0, $nNotReady = 0, $nPk = 0, $nTop = 0, $omitTable = 0, $op = 0, $op4 = 0, $or$cond = 0, $origSrc = 0, $pAlt = 0, $pAndExpr = 0, $pCompare = 0, $pCov = 0, $pE = 0, $pE11 = 0, $pEnd = 0, $pExpr = 0;
 var $pIdx = 0, $pLevel = 0, $pLoop = 0, $pOp = 0, $pOrExpr = 0, $pOrTab = 0, $pOrTerm = 0, $pOrWc = 0, $pParse = 0, $pPk = 0, $pPk10 = 0, $pPk9 = 0, $pRangeEnd = 0, $pRangeStart = 0, $pRight = 0, $pRight1 = 0, $pRight7 = 0, $pRight8 = 0, $pStart = 0, $pSubLoop = 0;
 var $pSubWInfo = 0, $pTab = 0, $pTabItem = 0, $pTerm = 0, $pWC = 0, $pX = 0, $pX2 = 0, $r = 0, $r1 = 0, $rTemp = 0, $regBase = 0, $regReturn = 0, $regRowid = 0, $regRowset = 0, $regYield = 0, $sEAlt = 0, $skipLikeAddr = 0, $start = 0, $startEq = 0, $start_constraints = 0;
 var $t = 0, $t5 = 0, $t6 = 0, $testOp = 0, $untestedTerms = 0, $v = 0, $wctrlFlags = 0, $x = 0, $zEndAff = 0, $zStartAff = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 448|0;
 $2 = sp + 8|0;
 $3 = sp + 432|0;
 $4 = sp + 428|0;
 $5 = sp;
 $j = sp + 424|0;
 $k = sp + 420|0;
 $iCur = sp + 416|0;
 $addrNxt = sp + 412|0;
 $omitTable = sp + 408|0;
 $bRev = sp + 404|0;
 $pLevel = sp + 400|0;
 $pLoop = sp + 396|0;
 $pWC = sp + 392|0;
 $pTerm = sp + 388|0;
 $pParse = sp + 384|0;
 $db = sp + 380|0;
 $v = sp + 376|0;
 $pTabItem = sp + 372|0;
 $addrBrk = sp + 368|0;
 $addrCont = sp + 364|0;
 $iRowidReg = sp + 360|0;
 $iReleaseReg = sp + 356|0;
 $regYield = sp + 352|0;
 $iReg = sp + 348|0;
 $addrNotFound = sp + 344|0;
 $nConstraint = sp + 340|0;
 $iIn = sp + 336|0;
 $iTarget = sp + 332|0;
 $pRight = sp + 328|0;
 $pCompare = sp + 324|0;
 $pRight1 = sp + 320|0;
 $pOp = sp + 316|0;
 $testOp = sp + 312|0;
 $start = sp + 308|0;
 $memEndValue = sp + 304|0;
 $pStart = sp + 300|0;
 $pEnd = sp + 296|0;
 $pX = sp + 292|0;
 $r1 = sp + 288|0;
 $rTemp = sp + 284|0;
 $op = sp + 280|0;
 $pX2 = sp + 276|0;
 $nEq = sp + 442|0;
 $nBtm = sp + 440|0;
 $nTop = sp + 438|0;
 $regBase = sp + 272|0;
 $pRangeStart = sp + 268|0;
 $pRangeEnd = sp + 264|0;
 $startEq = sp + 260|0;
 $endEq = sp + 256|0;
 $start_constraints = sp + 252|0;
 $nConstraint3 = sp + 248|0;
 $pIdx = sp + 244|0;
 $iIdxCur = sp + 240|0;
 $nExtraReg = sp + 236|0;
 $op4 = sp + 232|0;
 $zStartAff = sp + 228|0;
 $zEndAff = sp + 224|0;
 $bSeekPastNull = sp + 447|0;
 $bStopAtNull = sp + 446|0;
 $t = sp + 220|0;
 $t5 = sp + 445|0;
 $t6 = sp + 444|0;
 $pRight7 = sp + 216|0;
 $pRight8 = sp + 212|0;
 $pPk = sp + 208|0;
 $pOrWc = sp + 204|0;
 $pOrTab = sp + 200|0;
 $pCov = sp + 196|0;
 $iCovCur = sp + 192|0;
 $regReturn = sp + 188|0;
 $regRowset = sp + 184|0;
 $regRowid = sp + 180|0;
 $iLoopBody = sp + 176|0;
 $iRetInit = sp + 172|0;
 $untestedTerms = sp + 168|0;
 $ii = sp + 164|0;
 $wctrlFlags = sp + 436|0;
 $pAndExpr = sp + 160|0;
 $pTab = sp + 156|0;
 $nNotReady = sp + 152|0;
 $origSrc = sp + 148|0;
 $pPk9 = sp + 144|0;
 $iTerm = sp + 140|0;
 $pExpr = sp + 136|0;
 $pOrTerm = sp + 132|0;
 $pSubWInfo = sp + 128|0;
 $pOrExpr = sp + 124|0;
 $jmp1 = sp + 120|0;
 $pSubLoop = sp + 116|0;
 $addrExplain = sp + 112|0;
 $r = sp + 108|0;
 $iSet = sp + 104|0;
 $pPk10 = sp + 100|0;
 $nPk = sp + 96|0;
 $iPk = sp + 92|0;
 $iCol = sp + 88|0;
 $pE = sp + 84|0;
 $skipLikeAddr = sp + 80|0;
 $x = sp + 76|0;
 $pE11 = sp + 72|0;
 $sEAlt = sp + 24|0;
 $pAlt = sp + 16|0;
 HEAP32[$3>>2] = $pWInfo;
 HEAP32[$4>>2] = $iLevel;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$iRowidReg>>2] = 0;
 HEAP32[$iReleaseReg>>2] = 0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pParse>>2] = $11;
 $12 = HEAP32[$pParse>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$v>>2] = $14;
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 80|0);
 HEAP32[$pWC>>2] = $16;
 $17 = HEAP32[$pParse>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$db>>2] = $18;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = ((($20)) + 752|0);
 $22 = (($21) + (($19*80)|0)|0);
 HEAP32[$pLevel>>2] = $22;
 $23 = HEAP32[$pLevel>>2]|0;
 $24 = ((($23)) + 64|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pLoop>>2] = $25;
 $26 = HEAP32[$pLevel>>2]|0;
 $27 = ((($26)) + 44|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$3>>2]|0;
 $31 = ((($30)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = (($33) + (($29*72)|0)|0);
 HEAP32[$pTabItem>>2] = $34;
 $35 = HEAP32[$pTabItem>>2]|0;
 $36 = ((($35)) + 44|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$iCur>>2] = $37;
 $38 = $5;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$3>>2]|0;
 $45 = ((($44)) + 488|0);
 $46 = HEAP32[$iCur>>2]|0;
 $47 = (_sqlite3WhereGetMask($45,$46)|0);
 $48 = tempRet0;
 $49 = $47 ^ -1;
 $50 = $48 ^ -1;
 $51 = $40 & $49;
 $52 = $43 & $50;
 $53 = HEAP32[$pLevel>>2]|0;
 $54 = ((($53)) + 72|0);
 $55 = $54;
 $56 = $55;
 HEAP32[$56>>2] = $51;
 $57 = (($55) + 4)|0;
 $58 = $57;
 HEAP32[$58>>2] = $52;
 $59 = HEAP32[$3>>2]|0;
 $60 = ((($59)) + 64|0);
 $61 = $60;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (($61) + 4)|0;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$4>>2]|0;
 $68 = (_bitshift64Lshr(($63|0),($66|0),($67|0))|0);
 $69 = tempRet0;
 $70 = $68 & 1;
 HEAP32[$bRev>>2] = $70;
 $71 = HEAP32[$pLoop>>2]|0;
 $72 = ((($71)) + 36|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $73 & 64;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$3>>2]|0;
  $77 = ((($76)) + 40|0);
  $78 = HEAP16[$77>>1]|0;
  $79 = $78&65535;
  $80 = $79 & 32;
  $81 = ($80|0)==(0);
  $83 = $81;
 } else {
  $83 = 0;
 }
 $82 = $83&1;
 HEAP32[$omitTable>>2] = $82;
 $84 = HEAP32[$v>>2]|0;
 $85 = (_sqlite3VdbeMakeLabel($84)|0);
 $86 = HEAP32[$pLevel>>2]|0;
 $87 = ((($86)) + 16|0);
 HEAP32[$87>>2] = $85;
 $88 = HEAP32[$pLevel>>2]|0;
 $89 = ((($88)) + 12|0);
 HEAP32[$89>>2] = $85;
 HEAP32[$addrBrk>>2] = $85;
 $90 = HEAP32[$v>>2]|0;
 $91 = (_sqlite3VdbeMakeLabel($90)|0);
 $92 = HEAP32[$pLevel>>2]|0;
 $93 = ((($92)) + 24|0);
 HEAP32[$93>>2] = $91;
 HEAP32[$addrCont>>2] = $91;
 $94 = HEAP32[$pLevel>>2]|0;
 $95 = ((($94)) + 44|0);
 $96 = HEAP8[$95>>0]|0;
 $97 = $96&255;
 $98 = ($97|0)>(0);
 if ($98) {
  $99 = HEAP32[$pTabItem>>2]|0;
  $100 = ((($99)) + 36|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = $101&255;
  $103 = $102 & 8;
  $104 = ($103|0)!=(0);
  if ($104) {
   $105 = HEAP32[$pParse>>2]|0;
   $106 = ((($105)) + 44|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = (($107) + 1)|0;
   HEAP32[$106>>2] = $108;
   $109 = HEAP32[$pLevel>>2]|0;
   HEAP32[$109>>2] = $108;
   $110 = HEAP32[$v>>2]|0;
   $111 = HEAP32[$pLevel>>2]|0;
   $112 = HEAP32[$111>>2]|0;
   (_sqlite3VdbeAddOp2($110,76,0,$112)|0);
  }
 }
 $113 = HEAP32[$pTabItem>>2]|0;
 $114 = ((($113)) + 36|0);
 $115 = ((($114)) + 1|0);
 $116 = HEAP8[$115>>0]|0;
 $117 = ($116&255) >>> 4;
 $118 = $117 & 1;
 $119 = $118&255;
 $120 = ($119|0)!=(0);
 do {
  if ($120) {
   $121 = HEAP32[$pTabItem>>2]|0;
   $122 = ((($121)) + 28|0);
   $123 = HEAP32[$122>>2]|0;
   HEAP32[$regYield>>2] = $123;
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$regYield>>2]|0;
   $126 = HEAP32[$pTabItem>>2]|0;
   $127 = ((($126)) + 24|0);
   $128 = HEAP32[$127>>2]|0;
   (_sqlite3VdbeAddOp3($124,15,$125,0,$128)|0);
   $129 = HEAP32[$v>>2]|0;
   $130 = HEAP32[$regYield>>2]|0;
   $131 = HEAP32[$addrBrk>>2]|0;
   $132 = (_sqlite3VdbeAddOp2($129,16,$130,$131)|0);
   $133 = HEAP32[$pLevel>>2]|0;
   $134 = ((($133)) + 52|0);
   HEAP32[$134>>2] = $132;
   $135 = HEAP32[$pLevel>>2]|0;
   $136 = ((($135)) + 45|0);
   HEAP8[$136>>0] = 13;
  } else {
   $137 = HEAP32[$pLoop>>2]|0;
   $138 = ((($137)) + 36|0);
   $139 = HEAP32[$138>>2]|0;
   $140 = $139 & 1024;
   $141 = ($140|0)!=(0);
   $142 = HEAP32[$pLoop>>2]|0;
   if ($141) {
    $143 = ((($142)) + 40|0);
    $144 = HEAP16[$143>>1]|0;
    $145 = $144&65535;
    HEAP32[$nConstraint>>2] = $145;
    $146 = HEAP32[$pParse>>2]|0;
    _sqlite3ExprCachePush($146);
    $147 = HEAP32[$pParse>>2]|0;
    $148 = HEAP32[$nConstraint>>2]|0;
    $149 = (($148) + 2)|0;
    $150 = (_sqlite3GetTempRange($147,$149)|0);
    HEAP32[$iReg>>2] = $150;
    $151 = HEAP32[$pLevel>>2]|0;
    $152 = ((($151)) + 12|0);
    $153 = HEAP32[$152>>2]|0;
    HEAP32[$addrNotFound>>2] = $153;
    HEAP32[$j>>2] = 0;
    while(1) {
     $154 = HEAP32[$j>>2]|0;
     $155 = HEAP32[$nConstraint>>2]|0;
     $156 = ($154|0)<($155|0);
     if (!($156)) {
      break;
     }
     $157 = HEAP32[$iReg>>2]|0;
     $158 = HEAP32[$j>>2]|0;
     $159 = (($157) + ($158))|0;
     $160 = (($159) + 2)|0;
     HEAP32[$iTarget>>2] = $160;
     $161 = HEAP32[$j>>2]|0;
     $162 = HEAP32[$pLoop>>2]|0;
     $163 = ((($162)) + 48|0);
     $164 = HEAP32[$163>>2]|0;
     $165 = (($164) + ($161<<2)|0);
     $166 = HEAP32[$165>>2]|0;
     HEAP32[$pTerm>>2] = $166;
     $167 = HEAP32[$pTerm>>2]|0;
     $168 = ($167|0)==(0|0);
     do {
      if (!($168)) {
       $169 = HEAP32[$pTerm>>2]|0;
       $170 = ((($169)) + 12|0);
       $171 = HEAP16[$170>>1]|0;
       $172 = $171&65535;
       $173 = $172 & 1;
       $174 = ($173|0)!=(0);
       if ($174) {
        $175 = HEAP32[$pParse>>2]|0;
        $176 = HEAP32[$pTerm>>2]|0;
        $177 = HEAP32[$pLevel>>2]|0;
        $178 = HEAP32[$j>>2]|0;
        $179 = HEAP32[$bRev>>2]|0;
        $180 = HEAP32[$iTarget>>2]|0;
        (_codeEqualityTerm($175,$176,$177,$178,$179,$180)|0);
        $181 = HEAP32[$pLevel>>2]|0;
        $182 = ((($181)) + 16|0);
        $183 = HEAP32[$182>>2]|0;
        HEAP32[$addrNotFound>>2] = $183;
        break;
       } else {
        $184 = HEAP32[$pTerm>>2]|0;
        $185 = HEAP32[$184>>2]|0;
        $186 = ((($185)) + 16|0);
        $187 = HEAP32[$186>>2]|0;
        HEAP32[$pRight>>2] = $187;
        $188 = HEAP32[$pParse>>2]|0;
        $189 = HEAP32[$pRight>>2]|0;
        $190 = HEAP32[$iTarget>>2]|0;
        _codeExprOrVector($188,$189,$190,1);
        break;
       }
      }
     } while(0);
     $191 = HEAP32[$j>>2]|0;
     $192 = (($191) + 1)|0;
     HEAP32[$j>>2] = $192;
    }
    $193 = HEAP32[$v>>2]|0;
    $194 = HEAP32[$pLoop>>2]|0;
    $195 = ((($194)) + 24|0);
    $196 = HEAP32[$195>>2]|0;
    $197 = HEAP32[$iReg>>2]|0;
    (_sqlite3VdbeAddOp2($193,76,$196,$197)|0);
    $198 = HEAP32[$v>>2]|0;
    $199 = HEAP32[$nConstraint>>2]|0;
    $200 = HEAP32[$iReg>>2]|0;
    $201 = (($200) + 1)|0;
    (_sqlite3VdbeAddOp2($198,76,$199,$201)|0);
    $202 = HEAP32[$v>>2]|0;
    $203 = HEAP32[$iCur>>2]|0;
    $204 = HEAP32[$addrNotFound>>2]|0;
    $205 = HEAP32[$iReg>>2]|0;
    $206 = HEAP32[$pLoop>>2]|0;
    $207 = ((($206)) + 24|0);
    $208 = ((($207)) + 8|0);
    $209 = HEAP32[$208>>2]|0;
    $210 = HEAP32[$pLoop>>2]|0;
    $211 = ((($210)) + 24|0);
    $212 = ((($211)) + 4|0);
    $213 = HEAP8[$212>>0]|0;
    $214 = $213&255;
    $215 = ($214|0)!=(0);
    $216 = $215 ? -11 : -2;
    (_sqlite3VdbeAddOp4($202,11,$203,$204,$205,$209,$216)|0);
    $217 = HEAP32[$pLoop>>2]|0;
    $218 = ((($217)) + 24|0);
    $219 = ((($218)) + 4|0);
    HEAP8[$219>>0] = 0;
    $220 = HEAP32[$iCur>>2]|0;
    $221 = HEAP32[$pLevel>>2]|0;
    $222 = ((($221)) + 48|0);
    HEAP32[$222>>2] = $220;
    $223 = HEAP32[$3>>2]|0;
    $224 = ((($223)) + 45|0);
    $225 = HEAP8[$224>>0]|0;
    $226 = $225&255;
    $227 = ($226|0)!=(0);
    $228 = $227 ? 161 : 70;
    $229 = $228&255;
    $230 = HEAP32[$pLevel>>2]|0;
    $231 = ((($230)) + 45|0);
    HEAP8[$231>>0] = $229;
    $232 = HEAP32[$v>>2]|0;
    $233 = (_sqlite3VdbeCurrentAddr($232)|0);
    $234 = HEAP32[$pLevel>>2]|0;
    $235 = ((($234)) + 52|0);
    HEAP32[$235>>2] = $233;
    $236 = HEAP32[$pLevel>>2]|0;
    $237 = ((($236)) + 56|0);
    $238 = HEAP32[$237>>2]|0;
    HEAP32[$iIn>>2] = $238;
    $239 = HEAP32[$nConstraint>>2]|0;
    $240 = (($239) - 1)|0;
    HEAP32[$j>>2] = $240;
    while(1) {
     $241 = HEAP32[$j>>2]|0;
     $242 = ($241|0)>=(0);
     if (!($242)) {
      break;
     }
     $243 = HEAP32[$j>>2]|0;
     $244 = HEAP32[$pLoop>>2]|0;
     $245 = ((($244)) + 48|0);
     $246 = HEAP32[$245>>2]|0;
     $247 = (($246) + ($243<<2)|0);
     $248 = HEAP32[$247>>2]|0;
     HEAP32[$pTerm>>2] = $248;
     $249 = HEAP32[$j>>2]|0;
     $250 = ($249|0)<(16);
     if ($250) {
      $251 = HEAP32[$pLoop>>2]|0;
      $252 = ((($251)) + 24|0);
      $253 = ((($252)) + 6|0);
      $254 = HEAP16[$253>>1]|0;
      $255 = $254&65535;
      $256 = HEAP32[$j>>2]|0;
      $257 = $255 >> $256;
      $258 = $257 & 1;
      $259 = ($258|0)!=(0);
      if ($259) {
       $260 = HEAP32[$pLevel>>2]|0;
       $261 = HEAP32[$pTerm>>2]|0;
       _disableTerm($260,$261);
      } else {
       label = 21;
      }
     } else {
      label = 21;
     }
     if ((label|0) == 21) {
      label = 0;
      $262 = HEAP32[$pTerm>>2]|0;
      $263 = ((($262)) + 12|0);
      $264 = HEAP16[$263>>1]|0;
      $265 = $264&65535;
      $266 = $265 & 1;
      $267 = ($266|0)!=(0);
      if ($267) {
       $268 = HEAP32[$db>>2]|0;
       $269 = ((($268)) + 69|0);
       $270 = HEAP8[$269>>0]|0;
       $271 = ($270<<24>>24)!=(0);
       if (!($271)) {
        $272 = HEAP32[$v>>2]|0;
        $273 = HEAP32[$iIn>>2]|0;
        $274 = (($273) + -1)|0;
        HEAP32[$iIn>>2] = $274;
        $275 = HEAP32[$pLevel>>2]|0;
        $276 = ((($275)) + 56|0);
        $277 = ((($276)) + 4|0);
        $278 = HEAP32[$277>>2]|0;
        $279 = (($278) + (($274*12)|0)|0);
        $280 = ((($279)) + 4|0);
        $281 = HEAP32[$280>>2]|0;
        $282 = (_sqlite3VdbeGetOp($272,$281)|0);
        HEAP32[$pOp>>2] = $282;
        $283 = HEAP32[$v>>2]|0;
        $284 = HEAP32[$pOp>>2]|0;
        $285 = HEAP8[$284>>0]|0;
        $286 = $285&255;
        $287 = HEAP32[$pOp>>2]|0;
        $288 = ((($287)) + 4|0);
        $289 = HEAP32[$288>>2]|0;
        $290 = HEAP32[$pOp>>2]|0;
        $291 = ((($290)) + 8|0);
        $292 = HEAP32[$291>>2]|0;
        $293 = HEAP32[$pOp>>2]|0;
        $294 = ((($293)) + 12|0);
        $295 = HEAP32[$294>>2]|0;
        (_sqlite3VdbeAddOp3($283,$286,$289,$292,$295)|0);
       }
       $296 = HEAP32[$pParse>>2]|0;
       $297 = (_sqlite3PExpr($296,37,0,0,0)|0);
       HEAP32[$pCompare>>2] = $297;
       $298 = HEAP32[$pCompare>>2]|0;
       $299 = ($298|0)!=(0|0);
       if ($299) {
        $300 = HEAP32[$pTerm>>2]|0;
        $301 = HEAP32[$300>>2]|0;
        $302 = ((($301)) + 12|0);
        $303 = HEAP32[$302>>2]|0;
        $304 = HEAP32[$pCompare>>2]|0;
        $305 = ((($304)) + 12|0);
        HEAP32[$305>>2] = $303;
        $306 = HEAP32[$db>>2]|0;
        $307 = (_sqlite3Expr($306,157,0)|0);
        HEAP32[$pRight1>>2] = $307;
        $308 = HEAP32[$pCompare>>2]|0;
        $309 = ((($308)) + 16|0);
        HEAP32[$309>>2] = $307;
        $310 = HEAP32[$pRight1>>2]|0;
        $311 = ($310|0)!=(0|0);
        if ($311) {
         $312 = HEAP32[$iReg>>2]|0;
         $313 = HEAP32[$j>>2]|0;
         $314 = (($312) + ($313))|0;
         $315 = (($314) + 2)|0;
         $316 = HEAP32[$pRight1>>2]|0;
         $317 = ((($316)) + 28|0);
         HEAP32[$317>>2] = $315;
         $318 = HEAP32[$pParse>>2]|0;
         $319 = HEAP32[$pCompare>>2]|0;
         $320 = HEAP32[$pLevel>>2]|0;
         $321 = ((($320)) + 24|0);
         $322 = HEAP32[$321>>2]|0;
         _sqlite3ExprIfFalse($318,$319,$322,0);
        }
        $323 = HEAP32[$pCompare>>2]|0;
        $324 = ((($323)) + 12|0);
        HEAP32[$324>>2] = 0;
        $325 = HEAP32[$db>>2]|0;
        $326 = HEAP32[$pCompare>>2]|0;
        _sqlite3ExprDelete($325,$326);
       }
      }
     }
     $327 = HEAP32[$j>>2]|0;
     $328 = (($327) + -1)|0;
     HEAP32[$j>>2] = $328;
    }
    $329 = HEAP32[$pParse>>2]|0;
    _sqlite3ExprCachePop($329);
    break;
   }
   $330 = ((($142)) + 36|0);
   $331 = HEAP32[$330>>2]|0;
   $332 = $331 & 256;
   $333 = ($332|0)!=(0);
   if ($333) {
    $334 = HEAP32[$pLoop>>2]|0;
    $335 = ((($334)) + 36|0);
    $336 = HEAP32[$335>>2]|0;
    $337 = $336 & 5;
    $338 = ($337|0)!=(0);
    if ($338) {
     $339 = HEAP32[$pLoop>>2]|0;
     $340 = ((($339)) + 48|0);
     $341 = HEAP32[$340>>2]|0;
     $342 = HEAP32[$341>>2]|0;
     HEAP32[$pTerm>>2] = $342;
     $343 = HEAP32[$pParse>>2]|0;
     $344 = ((($343)) + 44|0);
     $345 = HEAP32[$344>>2]|0;
     $346 = (($345) + 1)|0;
     HEAP32[$344>>2] = $346;
     HEAP32[$iReleaseReg>>2] = $346;
     $347 = HEAP32[$pParse>>2]|0;
     $348 = HEAP32[$pTerm>>2]|0;
     $349 = HEAP32[$pLevel>>2]|0;
     $350 = HEAP32[$bRev>>2]|0;
     $351 = HEAP32[$iReleaseReg>>2]|0;
     $352 = (_codeEqualityTerm($347,$348,$349,0,$350,$351)|0);
     HEAP32[$iRowidReg>>2] = $352;
     $353 = HEAP32[$iRowidReg>>2]|0;
     $354 = HEAP32[$iReleaseReg>>2]|0;
     $355 = ($353|0)!=($354|0);
     if ($355) {
      $356 = HEAP32[$pParse>>2]|0;
      $357 = HEAP32[$iReleaseReg>>2]|0;
      _sqlite3ReleaseTempReg($356,$357);
     }
     $358 = HEAP32[$pLevel>>2]|0;
     $359 = ((($358)) + 16|0);
     $360 = HEAP32[$359>>2]|0;
     HEAP32[$addrNxt>>2] = $360;
     $361 = HEAP32[$v>>2]|0;
     $362 = HEAP32[$iCur>>2]|0;
     $363 = HEAP32[$addrNxt>>2]|0;
     $364 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp3($361,32,$362,$363,$364)|0);
     $365 = HEAP32[$pParse>>2]|0;
     $366 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheAffinityChange($365,$366,1);
     $367 = HEAP32[$pParse>>2]|0;
     $368 = HEAP32[$iCur>>2]|0;
     $369 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheStore($367,$368,-1,$369);
     $370 = HEAP32[$pLevel>>2]|0;
     $371 = ((($370)) + 45|0);
     HEAP8[$371>>0] = -95;
     break;
    }
   }
   $372 = HEAP32[$pLoop>>2]|0;
   $373 = ((($372)) + 36|0);
   $374 = HEAP32[$373>>2]|0;
   $375 = $374 & 256;
   $376 = ($375|0)!=(0);
   if ($376) {
    $377 = HEAP32[$pLoop>>2]|0;
    $378 = ((($377)) + 36|0);
    $379 = HEAP32[$378>>2]|0;
    $380 = $379 & 2;
    $381 = ($380|0)!=(0);
    if ($381) {
     HEAP32[$testOp>>2] = 161;
     HEAP32[$memEndValue>>2] = 0;
     HEAP32[$j>>2] = 0;
     HEAP32[$pEnd>>2] = 0;
     HEAP32[$pStart>>2] = 0;
     $382 = HEAP32[$pLoop>>2]|0;
     $383 = ((($382)) + 36|0);
     $384 = HEAP32[$383>>2]|0;
     $385 = $384 & 32;
     $386 = ($385|0)!=(0);
     if ($386) {
      $387 = HEAP32[$j>>2]|0;
      $388 = (($387) + 1)|0;
      HEAP32[$j>>2] = $388;
      $389 = HEAP32[$pLoop>>2]|0;
      $390 = ((($389)) + 48|0);
      $391 = HEAP32[$390>>2]|0;
      $392 = (($391) + ($387<<2)|0);
      $393 = HEAP32[$392>>2]|0;
      HEAP32[$pStart>>2] = $393;
     }
     $394 = HEAP32[$pLoop>>2]|0;
     $395 = ((($394)) + 36|0);
     $396 = HEAP32[$395>>2]|0;
     $397 = $396 & 16;
     $398 = ($397|0)!=(0);
     if ($398) {
      $399 = HEAP32[$j>>2]|0;
      $400 = (($399) + 1)|0;
      HEAP32[$j>>2] = $400;
      $401 = HEAP32[$pLoop>>2]|0;
      $402 = ((($401)) + 48|0);
      $403 = HEAP32[$402>>2]|0;
      $404 = (($403) + ($399<<2)|0);
      $405 = HEAP32[$404>>2]|0;
      HEAP32[$pEnd>>2] = $405;
     }
     $406 = HEAP32[$bRev>>2]|0;
     $407 = ($406|0)!=(0);
     if ($407) {
      $408 = HEAP32[$pStart>>2]|0;
      HEAP32[$pTerm>>2] = $408;
      $409 = HEAP32[$pEnd>>2]|0;
      HEAP32[$pStart>>2] = $409;
      $410 = HEAP32[$pTerm>>2]|0;
      HEAP32[$pEnd>>2] = $410;
     }
     $411 = HEAP32[$pStart>>2]|0;
     $412 = ($411|0)!=(0|0);
     if ($412) {
      $413 = HEAP32[$pStart>>2]|0;
      $414 = HEAP32[$413>>2]|0;
      HEAP32[$pX>>2] = $414;
      $415 = HEAP32[$pX>>2]|0;
      $416 = ((($415)) + 16|0);
      $417 = HEAP32[$416>>2]|0;
      $418 = (_sqlite3ExprIsVector($417)|0);
      $419 = ($418|0)!=(0);
      $420 = HEAP32[$pParse>>2]|0;
      if ($419) {
       $421 = (_sqlite3GetTempReg($420)|0);
       HEAP32[$rTemp>>2] = $421;
       HEAP32[$r1>>2] = $421;
       $422 = HEAP32[$pParse>>2]|0;
       $423 = HEAP32[$pX>>2]|0;
       $424 = ((($423)) + 16|0);
       $425 = HEAP32[$424>>2]|0;
       $426 = HEAP32[$r1>>2]|0;
       _codeExprOrVector($422,$425,$426,1);
       $427 = HEAP32[$pX>>2]|0;
       $428 = HEAP8[$427>>0]|0;
       $429 = $428&255;
       $430 = (($429) - 38)|0;
       $431 = $430 | 1;
       $432 = (34432 + ($431)|0);
       $433 = HEAP8[$432>>0]|0;
       $434 = $433&255;
       HEAP32[$op>>2] = $434;
      } else {
       $435 = HEAP32[$pX>>2]|0;
       $436 = ((($435)) + 16|0);
       $437 = HEAP32[$436>>2]|0;
       $438 = (_sqlite3ExprCodeTemp($420,$437,$rTemp)|0);
       HEAP32[$r1>>2] = $438;
       $439 = HEAP32[$pLevel>>2]|0;
       $440 = HEAP32[$pStart>>2]|0;
       _disableTerm($439,$440);
       $441 = HEAP32[$pX>>2]|0;
       $442 = HEAP8[$441>>0]|0;
       $443 = $442&255;
       $444 = (($443) - 38)|0;
       $445 = (34432 + ($444)|0);
       $446 = HEAP8[$445>>0]|0;
       $447 = $446&255;
       HEAP32[$op>>2] = $447;
      }
      $448 = HEAP32[$v>>2]|0;
      $449 = HEAP32[$op>>2]|0;
      $450 = HEAP32[$iCur>>2]|0;
      $451 = HEAP32[$addrBrk>>2]|0;
      $452 = HEAP32[$r1>>2]|0;
      (_sqlite3VdbeAddOp3($448,$449,$450,$451,$452)|0);
      $453 = HEAP32[$pParse>>2]|0;
      $454 = HEAP32[$r1>>2]|0;
      _sqlite3ExprCacheAffinityChange($453,$454,1);
      $455 = HEAP32[$pParse>>2]|0;
      $456 = HEAP32[$rTemp>>2]|0;
      _sqlite3ReleaseTempReg($455,$456);
     } else {
      $457 = HEAP32[$v>>2]|0;
      $458 = HEAP32[$bRev>>2]|0;
      $459 = ($458|0)!=(0);
      $460 = $459 ? 53 : 57;
      $461 = HEAP32[$iCur>>2]|0;
      $462 = HEAP32[$addrBrk>>2]|0;
      (_sqlite3VdbeAddOp2($457,$460,$461,$462)|0);
     }
     $463 = HEAP32[$pEnd>>2]|0;
     $464 = ($463|0)!=(0|0);
     if ($464) {
      $465 = HEAP32[$pEnd>>2]|0;
      $466 = HEAP32[$465>>2]|0;
      HEAP32[$pX2>>2] = $466;
      $467 = HEAP32[$pParse>>2]|0;
      $468 = ((($467)) + 44|0);
      $469 = HEAP32[$468>>2]|0;
      $470 = (($469) + 1)|0;
      HEAP32[$468>>2] = $470;
      HEAP32[$memEndValue>>2] = $470;
      $471 = HEAP32[$pParse>>2]|0;
      $472 = HEAP32[$pX2>>2]|0;
      $473 = ((($472)) + 16|0);
      $474 = HEAP32[$473>>2]|0;
      $475 = HEAP32[$memEndValue>>2]|0;
      _codeExprOrVector($471,$474,$475,1);
      $476 = HEAP32[$pX2>>2]|0;
      $477 = ((($476)) + 16|0);
      $478 = HEAP32[$477>>2]|0;
      $479 = (_sqlite3ExprIsVector($478)|0);
      $480 = (0)==($479|0);
      do {
       if ($480) {
        $481 = HEAP32[$pX2>>2]|0;
        $482 = HEAP8[$481>>0]|0;
        $483 = $482&255;
        $484 = ($483|0)==(40);
        if (!($484)) {
         $485 = HEAP32[$pX2>>2]|0;
         $486 = HEAP8[$485>>0]|0;
         $487 = $486&255;
         $488 = ($487|0)==(38);
         if (!($488)) {
          label = 54;
          break;
         }
        }
        $489 = HEAP32[$bRev>>2]|0;
        $490 = ($489|0)!=(0);
        $491 = $490 ? 39 : 41;
        HEAP32[$testOp>>2] = $491;
       } else {
        label = 54;
       }
      } while(0);
      if ((label|0) == 54) {
       $492 = HEAP32[$bRev>>2]|0;
       $493 = ($492|0)!=(0);
       $494 = $493 ? 40 : 38;
       HEAP32[$testOp>>2] = $494;
      }
      $495 = HEAP32[$pX2>>2]|0;
      $496 = ((($495)) + 16|0);
      $497 = HEAP32[$496>>2]|0;
      $498 = (_sqlite3ExprIsVector($497)|0);
      $499 = (0)==($498|0);
      if ($499) {
       $500 = HEAP32[$pLevel>>2]|0;
       $501 = HEAP32[$pEnd>>2]|0;
       _disableTerm($500,$501);
      }
     }
     $502 = HEAP32[$v>>2]|0;
     $503 = (_sqlite3VdbeCurrentAddr($502)|0);
     HEAP32[$start>>2] = $503;
     $504 = HEAP32[$bRev>>2]|0;
     $505 = ($504|0)!=(0);
     $506 = $505 ? 6 : 7;
     $507 = $506&255;
     $508 = HEAP32[$pLevel>>2]|0;
     $509 = ((($508)) + 45|0);
     HEAP8[$509>>0] = $507;
     $510 = HEAP32[$iCur>>2]|0;
     $511 = HEAP32[$pLevel>>2]|0;
     $512 = ((($511)) + 48|0);
     HEAP32[$512>>2] = $510;
     $513 = HEAP32[$start>>2]|0;
     $514 = HEAP32[$pLevel>>2]|0;
     $515 = ((($514)) + 52|0);
     HEAP32[$515>>2] = $513;
     $516 = HEAP32[$testOp>>2]|0;
     $517 = ($516|0)!=(161);
     if (!($517)) {
      break;
     }
     $518 = HEAP32[$pParse>>2]|0;
     $519 = ((($518)) + 44|0);
     $520 = HEAP32[$519>>2]|0;
     $521 = (($520) + 1)|0;
     HEAP32[$519>>2] = $521;
     HEAP32[$iRowidReg>>2] = $521;
     $522 = HEAP32[$v>>2]|0;
     $523 = HEAP32[$iCur>>2]|0;
     $524 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp2($522,123,$523,$524)|0);
     $525 = HEAP32[$pParse>>2]|0;
     $526 = HEAP32[$iCur>>2]|0;
     $527 = HEAP32[$iRowidReg>>2]|0;
     _sqlite3ExprCacheStore($525,$526,-1,$527);
     $528 = HEAP32[$v>>2]|0;
     $529 = HEAP32[$testOp>>2]|0;
     $530 = HEAP32[$memEndValue>>2]|0;
     $531 = HEAP32[$addrBrk>>2]|0;
     $532 = HEAP32[$iRowidReg>>2]|0;
     (_sqlite3VdbeAddOp3($528,$529,$530,$531,$532)|0);
     $533 = HEAP32[$v>>2]|0;
     _sqlite3VdbeChangeP5($533,83);
     break;
    }
   }
   $534 = HEAP32[$pLoop>>2]|0;
   $535 = ((($534)) + 36|0);
   $536 = HEAP32[$535>>2]|0;
   $537 = $536 & 512;
   $538 = ($537|0)!=(0);
   $539 = HEAP32[$pLoop>>2]|0;
   if (!($538)) {
    $1075 = ((($539)) + 36|0);
    $1076 = HEAP32[$1075>>2]|0;
    $1077 = $1076 & 8192;
    $1078 = ($1077|0)!=(0);
    if (!($1078)) {
     $1493 = HEAP32[$pTabItem>>2]|0;
     $1494 = ((($1493)) + 36|0);
     $1495 = ((($1494)) + 1|0);
     $1496 = HEAP8[$1495>>0]|0;
     $1497 = ($1496&255) >>> 5;
     $1498 = $1497 & 1;
     $1499 = $1498&255;
     $1500 = ($1499|0)!=(0);
     if ($1500) {
      $1501 = HEAP32[$pLevel>>2]|0;
      $1502 = ((($1501)) + 45|0);
      HEAP8[$1502>>0] = -95;
      break;
     } else {
      $1503 = HEAP32[$bRev>>2]|0;
      $1504 = (34448 + ($1503)|0);
      $1505 = HEAP8[$1504>>0]|0;
      $1506 = HEAP32[$pLevel>>2]|0;
      $1507 = ((($1506)) + 45|0);
      HEAP8[$1507>>0] = $1505;
      $1508 = HEAP32[$iCur>>2]|0;
      $1509 = HEAP32[$pLevel>>2]|0;
      $1510 = ((($1509)) + 48|0);
      HEAP32[$1510>>2] = $1508;
      $1511 = HEAP32[$v>>2]|0;
      $1512 = HEAP32[$bRev>>2]|0;
      $1513 = (34450 + ($1512)|0);
      $1514 = HEAP8[$1513>>0]|0;
      $1515 = $1514&255;
      $1516 = HEAP32[$iCur>>2]|0;
      $1517 = HEAP32[$addrBrk>>2]|0;
      $1518 = (_sqlite3VdbeAddOp2($1511,$1515,$1516,$1517)|0);
      $1519 = (1 + ($1518))|0;
      $1520 = HEAP32[$pLevel>>2]|0;
      $1521 = ((($1520)) + 52|0);
      HEAP32[$1521>>2] = $1519;
      $1522 = HEAP32[$pLevel>>2]|0;
      $1523 = ((($1522)) + 47|0);
      HEAP8[$1523>>0] = 1;
      break;
     }
    }
    HEAP32[$pCov>>2] = 0;
    $1079 = HEAP32[$pParse>>2]|0;
    $1080 = ((($1079)) + 40|0);
    $1081 = HEAP32[$1080>>2]|0;
    $1082 = (($1081) + 1)|0;
    HEAP32[$1080>>2] = $1082;
    HEAP32[$iCovCur>>2] = $1081;
    $1083 = HEAP32[$pParse>>2]|0;
    $1084 = ((($1083)) + 44|0);
    $1085 = HEAP32[$1084>>2]|0;
    $1086 = (($1085) + 1)|0;
    HEAP32[$1084>>2] = $1086;
    HEAP32[$regReturn>>2] = $1086;
    HEAP32[$regRowset>>2] = 0;
    HEAP32[$regRowid>>2] = 0;
    $1087 = HEAP32[$v>>2]|0;
    $1088 = (_sqlite3VdbeMakeLabel($1087)|0);
    HEAP32[$iLoopBody>>2] = $1088;
    HEAP32[$untestedTerms>>2] = 0;
    HEAP32[$pAndExpr>>2] = 0;
    $1089 = HEAP32[$pTabItem>>2]|0;
    $1090 = ((($1089)) + 16|0);
    $1091 = HEAP32[$1090>>2]|0;
    HEAP32[$pTab>>2] = $1091;
    $1092 = HEAP32[$pLoop>>2]|0;
    $1093 = ((($1092)) + 48|0);
    $1094 = HEAP32[$1093>>2]|0;
    $1095 = HEAP32[$1094>>2]|0;
    HEAP32[$pTerm>>2] = $1095;
    $1096 = HEAP32[$pTerm>>2]|0;
    $1097 = ((($1096)) + 28|0);
    $1098 = HEAP32[$1097>>2]|0;
    HEAP32[$pOrWc>>2] = $1098;
    $1099 = HEAP32[$pLevel>>2]|0;
    $1100 = ((($1099)) + 45|0);
    HEAP8[$1100>>0] = 72;
    $1101 = HEAP32[$regReturn>>2]|0;
    $1102 = HEAP32[$pLevel>>2]|0;
    $1103 = ((($1102)) + 48|0);
    HEAP32[$1103>>2] = $1101;
    $1104 = HEAP32[$3>>2]|0;
    $1105 = ((($1104)) + 42|0);
    $1106 = HEAP8[$1105>>0]|0;
    $1107 = $1106&255;
    $1108 = ($1107|0)>(1);
    $1109 = HEAP32[$3>>2]|0;
    L91: do {
     if ($1108) {
      $1110 = ((($1109)) + 42|0);
      $1111 = HEAP8[$1110>>0]|0;
      $1112 = $1111&255;
      $1113 = HEAP32[$4>>2]|0;
      $1114 = (($1112) - ($1113))|0;
      $1115 = (($1114) - 1)|0;
      HEAP32[$nNotReady>>2] = $1115;
      $1116 = HEAP32[$db>>2]|0;
      $1117 = HEAP32[$nNotReady>>2]|0;
      $1118 = ($1117*72)|0;
      $1119 = (80 + ($1118))|0;
      $1120 = (_sqlite3DbMallocRaw($1116,$1119,0)|0);
      HEAP32[$pOrTab>>2] = $1120;
      $1121 = HEAP32[$pOrTab>>2]|0;
      $1122 = ($1121|0)==(0|0);
      if ($1122) {
       $1123 = $5;
       $1124 = $1123;
       $1125 = HEAP32[$1124>>2]|0;
       $1126 = (($1123) + 4)|0;
       $1127 = $1126;
       $1128 = HEAP32[$1127>>2]|0;
       $1129 = $2;
       $1130 = $1129;
       HEAP32[$1130>>2] = $1125;
       $1131 = (($1129) + 4)|0;
       $1132 = $1131;
       HEAP32[$1132>>2] = $1128;
       $1745 = $2;
       $1746 = $1745;
       $1747 = HEAP32[$1746>>2]|0;
       $1748 = (($1745) + 4)|0;
       $1749 = $1748;
       $1750 = HEAP32[$1749>>2]|0;
       tempRet0 = ($1750);
       STACKTOP = sp;return ($1747|0);
      }
      $1133 = HEAP32[$nNotReady>>2]|0;
      $1134 = (($1133) + 1)|0;
      $1135 = $1134&255;
      $1136 = $1135&255;
      $1137 = HEAP32[$pOrTab>>2]|0;
      $1138 = ((($1137)) + 4|0);
      HEAP32[$1138>>2] = $1136;
      $1139 = HEAP32[$pOrTab>>2]|0;
      $1140 = ((($1139)) + 4|0);
      $1141 = HEAP32[$1140>>2]|0;
      $1142 = HEAP32[$pOrTab>>2]|0;
      HEAP32[$1142>>2] = $1141;
      $1143 = HEAP32[$pOrTab>>2]|0;
      $1144 = ((($1143)) + 8|0);
      $1145 = HEAP32[$pTabItem>>2]|0;
      dest=$1144; src=$1145; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
      $1146 = HEAP32[$3>>2]|0;
      $1147 = ((($1146)) + 4|0);
      $1148 = HEAP32[$1147>>2]|0;
      $1149 = ((($1148)) + 8|0);
      HEAP32[$origSrc>>2] = $1149;
      HEAP32[$k>>2] = 1;
      while(1) {
       $1150 = HEAP32[$k>>2]|0;
       $1151 = HEAP32[$nNotReady>>2]|0;
       $1152 = ($1150|0)<=($1151|0);
       if (!($1152)) {
        break L91;
       }
       $1153 = HEAP32[$k>>2]|0;
       $1154 = HEAP32[$pOrTab>>2]|0;
       $1155 = ((($1154)) + 8|0);
       $1156 = (($1155) + (($1153*72)|0)|0);
       $1157 = HEAP32[$k>>2]|0;
       $1158 = HEAP32[$pLevel>>2]|0;
       $1159 = (($1158) + (($1157*80)|0)|0);
       $1160 = ((($1159)) + 44|0);
       $1161 = HEAP8[$1160>>0]|0;
       $1162 = $1161&255;
       $1163 = HEAP32[$origSrc>>2]|0;
       $1164 = (($1163) + (($1162*72)|0)|0);
       dest=$1156; src=$1164; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
       $1165 = HEAP32[$k>>2]|0;
       $1166 = (($1165) + 1)|0;
       HEAP32[$k>>2] = $1166;
      }
     } else {
      $1167 = ((($1109)) + 4|0);
      $1168 = HEAP32[$1167>>2]|0;
      HEAP32[$pOrTab>>2] = $1168;
     }
    } while(0);
    $1169 = HEAP32[$3>>2]|0;
    $1170 = ((($1169)) + 40|0);
    $1171 = HEAP16[$1170>>1]|0;
    $1172 = $1171&65535;
    $1173 = $1172 & 16;
    $1174 = ($1173|0)==(0);
    if ($1174) {
     $1175 = HEAP32[$pTab>>2]|0;
     $1176 = ((($1175)) + 42|0);
     $1177 = HEAP8[$1176>>0]|0;
     $1178 = $1177&255;
     $1179 = $1178 & 32;
     $1180 = ($1179|0)==(0);
     if ($1180) {
      $1181 = HEAP32[$pParse>>2]|0;
      $1182 = ((($1181)) + 44|0);
      $1183 = HEAP32[$1182>>2]|0;
      $1184 = (($1183) + 1)|0;
      HEAP32[$1182>>2] = $1184;
      HEAP32[$regRowset>>2] = $1184;
      $1185 = HEAP32[$v>>2]|0;
      $1186 = HEAP32[$regRowset>>2]|0;
      (_sqlite3VdbeAddOp2($1185,79,0,$1186)|0);
     } else {
      $1187 = HEAP32[$pTab>>2]|0;
      $1188 = (_sqlite3PrimaryKeyIndex($1187)|0);
      HEAP32[$pPk9>>2] = $1188;
      $1189 = HEAP32[$pParse>>2]|0;
      $1190 = ((($1189)) + 40|0);
      $1191 = HEAP32[$1190>>2]|0;
      $1192 = (($1191) + 1)|0;
      HEAP32[$1190>>2] = $1192;
      HEAP32[$regRowset>>2] = $1191;
      $1193 = HEAP32[$v>>2]|0;
      $1194 = HEAP32[$regRowset>>2]|0;
      $1195 = HEAP32[$pPk9>>2]|0;
      $1196 = ((($1195)) + 50|0);
      $1197 = HEAP16[$1196>>1]|0;
      $1198 = $1197&65535;
      (_sqlite3VdbeAddOp2($1193,107,$1194,$1198)|0);
      $1199 = HEAP32[$pParse>>2]|0;
      $1200 = HEAP32[$pPk9>>2]|0;
      _sqlite3VdbeSetP4KeyInfo($1199,$1200);
     }
     $1201 = HEAP32[$pParse>>2]|0;
     $1202 = ((($1201)) + 44|0);
     $1203 = HEAP32[$1202>>2]|0;
     $1204 = (($1203) + 1)|0;
     HEAP32[$1202>>2] = $1204;
     HEAP32[$regRowid>>2] = $1204;
    }
    $1205 = HEAP32[$v>>2]|0;
    $1206 = HEAP32[$regReturn>>2]|0;
    $1207 = (_sqlite3VdbeAddOp2($1205,76,0,$1206)|0);
    HEAP32[$iRetInit>>2] = $1207;
    $1208 = HEAP32[$pWC>>2]|0;
    $1209 = ((($1208)) + 12|0);
    $1210 = HEAP32[$1209>>2]|0;
    $1211 = ($1210|0)>(1);
    if ($1211) {
     HEAP32[$iTerm>>2] = 0;
     while(1) {
      $1212 = HEAP32[$iTerm>>2]|0;
      $1213 = HEAP32[$pWC>>2]|0;
      $1214 = ((($1213)) + 12|0);
      $1215 = HEAP32[$1214>>2]|0;
      $1216 = ($1212|0)<($1215|0);
      if (!($1216)) {
       break;
      }
      $1217 = HEAP32[$iTerm>>2]|0;
      $1218 = HEAP32[$pWC>>2]|0;
      $1219 = ((($1218)) + 20|0);
      $1220 = HEAP32[$1219>>2]|0;
      $1221 = (($1220) + (($1217*48)|0)|0);
      $1222 = HEAP32[$1221>>2]|0;
      HEAP32[$pExpr>>2] = $1222;
      $1223 = HEAP32[$iTerm>>2]|0;
      $1224 = HEAP32[$pWC>>2]|0;
      $1225 = ((($1224)) + 20|0);
      $1226 = HEAP32[$1225>>2]|0;
      $1227 = (($1226) + (($1223*48)|0)|0);
      $1228 = HEAP32[$pTerm>>2]|0;
      $1229 = ($1227|0)==($1228|0);
      do {
       if (!($1229)) {
        $1230 = HEAP32[$pExpr>>2]|0;
        $1231 = ((($1230)) + 4|0);
        $1232 = HEAP32[$1231>>2]|0;
        $1233 = $1232 & 1;
        $1234 = ($1233|0)!=(0);
        if (!($1234)) {
         $1235 = HEAP32[$iTerm>>2]|0;
         $1236 = HEAP32[$pWC>>2]|0;
         $1237 = ((($1236)) + 20|0);
         $1238 = HEAP32[$1237>>2]|0;
         $1239 = (($1238) + (($1235*48)|0)|0);
         $1240 = ((($1239)) + 10|0);
         $1241 = HEAP16[$1240>>1]|0;
         $1242 = $1241&65535;
         $1243 = $1242 & 6;
         $1244 = ($1243|0)!=(0);
         if (!($1244)) {
          $1245 = HEAP32[$iTerm>>2]|0;
          $1246 = HEAP32[$pWC>>2]|0;
          $1247 = ((($1246)) + 20|0);
          $1248 = HEAP32[$1247>>2]|0;
          $1249 = (($1248) + (($1245*48)|0)|0);
          $1250 = ((($1249)) + 12|0);
          $1251 = HEAP16[$1250>>1]|0;
          $1252 = $1251&65535;
          $1253 = $1252 & 8191;
          $1254 = ($1253|0)==(0);
          if ($1254) {
           break;
          }
          $1255 = HEAP32[$db>>2]|0;
          $1256 = HEAP32[$pExpr>>2]|0;
          $1257 = (_sqlite3ExprDup($1255,$1256,0)|0);
          HEAP32[$pExpr>>2] = $1257;
          $1258 = HEAP32[$db>>2]|0;
          $1259 = HEAP32[$pAndExpr>>2]|0;
          $1260 = HEAP32[$pExpr>>2]|0;
          $1261 = (_sqlite3ExprAnd($1258,$1259,$1260)|0);
          HEAP32[$pAndExpr>>2] = $1261;
         }
        }
       }
      } while(0);
      $1262 = HEAP32[$iTerm>>2]|0;
      $1263 = (($1262) + 1)|0;
      HEAP32[$iTerm>>2] = $1263;
     }
     $1264 = HEAP32[$pAndExpr>>2]|0;
     $1265 = ($1264|0)!=(0|0);
     if ($1265) {
      $1266 = HEAP32[$pParse>>2]|0;
      $1267 = HEAP32[$pAndExpr>>2]|0;
      $1268 = (_sqlite3PExpr($1266,284,0,$1267,0)|0);
      HEAP32[$pAndExpr>>2] = $1268;
     }
    }
    $1269 = HEAP32[$3>>2]|0;
    $1270 = ((($1269)) + 40|0);
    $1271 = HEAP16[$1270>>1]|0;
    $1272 = $1271&65535;
    $1273 = $1272 & 1024;
    $1274 = 32 | $1273;
    $1275 = $1274&65535;
    HEAP16[$wctrlFlags>>1] = $1275;
    HEAP32[$ii>>2] = 0;
    while(1) {
     $1276 = HEAP32[$ii>>2]|0;
     $1277 = HEAP32[$pOrWc>>2]|0;
     $1278 = ((($1277)) + 12|0);
     $1279 = HEAP32[$1278>>2]|0;
     $1280 = ($1276|0)<($1279|0);
     if (!($1280)) {
      break;
     }
     $1281 = HEAP32[$ii>>2]|0;
     $1282 = HEAP32[$pOrWc>>2]|0;
     $1283 = ((($1282)) + 20|0);
     $1284 = HEAP32[$1283>>2]|0;
     $1285 = (($1284) + (($1281*48)|0)|0);
     HEAP32[$pOrTerm>>2] = $1285;
     $1286 = HEAP32[$pOrTerm>>2]|0;
     $1287 = ((($1286)) + 20|0);
     $1288 = HEAP32[$1287>>2]|0;
     $1289 = HEAP32[$iCur>>2]|0;
     $1290 = ($1288|0)==($1289|0);
     if ($1290) {
      label = 167;
     } else {
      $1291 = HEAP32[$pOrTerm>>2]|0;
      $1292 = ((($1291)) + 12|0);
      $1293 = HEAP16[$1292>>1]|0;
      $1294 = $1293&65535;
      $1295 = $1294 & 1024;
      $1296 = ($1295|0)!=(0);
      if ($1296) {
       label = 167;
      }
     }
     if ((label|0) == 167) {
      label = 0;
      $1297 = HEAP32[$pOrTerm>>2]|0;
      $1298 = HEAP32[$1297>>2]|0;
      HEAP32[$pOrExpr>>2] = $1298;
      HEAP32[$jmp1>>2] = 0;
      $1299 = HEAP32[$pAndExpr>>2]|0;
      $1300 = ($1299|0)!=(0|0);
      if ($1300) {
       $1301 = HEAP32[$pOrExpr>>2]|0;
       $1302 = ((($1301)) + 4|0);
       $1303 = HEAP32[$1302>>2]|0;
       $1304 = $1303 & 1;
       $1305 = ($1304|0)!=(0);
       if (!($1305)) {
        $1306 = HEAP32[$pOrExpr>>2]|0;
        $1307 = HEAP32[$pAndExpr>>2]|0;
        $1308 = ((($1307)) + 12|0);
        HEAP32[$1308>>2] = $1306;
        $1309 = HEAP32[$pAndExpr>>2]|0;
        HEAP32[$pOrExpr>>2] = $1309;
       }
      }
      $1310 = HEAP32[$pParse>>2]|0;
      $1311 = HEAP32[$pOrTab>>2]|0;
      $1312 = HEAP32[$pOrExpr>>2]|0;
      $1313 = HEAP16[$wctrlFlags>>1]|0;
      $1314 = HEAP32[$iCovCur>>2]|0;
      $1315 = (_sqlite3WhereBegin($1310,$1311,$1312,0,0,$1313,$1314)|0);
      HEAP32[$pSubWInfo>>2] = $1315;
      $1316 = HEAP32[$pSubWInfo>>2]|0;
      $1317 = ($1316|0)!=(0|0);
      if ($1317) {
       $1318 = HEAP32[$pParse>>2]|0;
       $1319 = HEAP32[$pOrTab>>2]|0;
       $1320 = HEAP32[$pSubWInfo>>2]|0;
       $1321 = ((($1320)) + 752|0);
       $1322 = HEAP32[$4>>2]|0;
       $1323 = HEAP32[$pLevel>>2]|0;
       $1324 = ((($1323)) + 44|0);
       $1325 = HEAP8[$1324>>0]|0;
       $1326 = $1325&255;
       $1327 = (_sqlite3WhereExplainOneScan($1318,$1319,$1321,$1322,$1326,0)|0);
       HEAP32[$addrExplain>>2] = $1327;
       $1328 = HEAP32[$3>>2]|0;
       $1329 = ((($1328)) + 40|0);
       $1330 = HEAP16[$1329>>1]|0;
       $1331 = $1330&65535;
       $1332 = $1331 & 16;
       $1333 = ($1332|0)==(0);
       do {
        if ($1333) {
         $1334 = HEAP32[$ii>>2]|0;
         $1335 = HEAP32[$pOrWc>>2]|0;
         $1336 = ((($1335)) + 12|0);
         $1337 = HEAP32[$1336>>2]|0;
         $1338 = (($1337) - 1)|0;
         $1339 = ($1334|0)==($1338|0);
         $1340 = HEAP32[$ii>>2]|0;
         $1341 = $1339 ? -1 : $1340;
         HEAP32[$iSet>>2] = $1341;
         $1342 = HEAP32[$pTab>>2]|0;
         $1343 = ((($1342)) + 42|0);
         $1344 = HEAP8[$1343>>0]|0;
         $1345 = $1344&255;
         $1346 = $1345 & 32;
         $1347 = ($1346|0)==(0);
         if ($1347) {
          $1348 = HEAP32[$pParse>>2]|0;
          $1349 = HEAP32[$pTab>>2]|0;
          $1350 = HEAP32[$iCur>>2]|0;
          $1351 = HEAP32[$regRowid>>2]|0;
          $1352 = (_sqlite3ExprCodeGetColumn($1348,$1349,-1,$1350,$1351,0)|0);
          HEAP32[$r>>2] = $1352;
          $1353 = HEAP32[$v>>2]|0;
          $1354 = HEAP32[$regRowset>>2]|0;
          $1355 = HEAP32[$r>>2]|0;
          $1356 = HEAP32[$iSet>>2]|0;
          $1357 = (_sqlite3VdbeAddOp4Int($1353,63,$1354,0,$1355,$1356)|0);
          HEAP32[$jmp1>>2] = $1357;
          break;
         }
         $1358 = HEAP32[$pTab>>2]|0;
         $1359 = (_sqlite3PrimaryKeyIndex($1358)|0);
         HEAP32[$pPk10>>2] = $1359;
         $1360 = HEAP32[$pPk10>>2]|0;
         $1361 = ((($1360)) + 50|0);
         $1362 = HEAP16[$1361>>1]|0;
         $1363 = $1362&65535;
         HEAP32[$nPk>>2] = $1363;
         $1364 = HEAP32[$pParse>>2]|0;
         $1365 = HEAP32[$nPk>>2]|0;
         $1366 = (_sqlite3GetTempRange($1364,$1365)|0);
         HEAP32[$r>>2] = $1366;
         HEAP32[$iPk>>2] = 0;
         while(1) {
          $1367 = HEAP32[$iPk>>2]|0;
          $1368 = HEAP32[$nPk>>2]|0;
          $1369 = ($1367|0)<($1368|0);
          if (!($1369)) {
           break;
          }
          $1370 = HEAP32[$iPk>>2]|0;
          $1371 = HEAP32[$pPk10>>2]|0;
          $1372 = ((($1371)) + 4|0);
          $1373 = HEAP32[$1372>>2]|0;
          $1374 = (($1373) + ($1370<<1)|0);
          $1375 = HEAP16[$1374>>1]|0;
          $1376 = $1375 << 16 >> 16;
          HEAP32[$iCol>>2] = $1376;
          $1377 = HEAP32[$pParse>>2]|0;
          $1378 = HEAP32[$pTab>>2]|0;
          $1379 = HEAP32[$iCol>>2]|0;
          $1380 = HEAP32[$iCur>>2]|0;
          $1381 = HEAP32[$r>>2]|0;
          $1382 = HEAP32[$iPk>>2]|0;
          $1383 = (($1381) + ($1382))|0;
          _sqlite3ExprCodeGetColumnToReg($1377,$1378,$1379,$1380,$1383);
          $1384 = HEAP32[$iPk>>2]|0;
          $1385 = (($1384) + 1)|0;
          HEAP32[$iPk>>2] = $1385;
         }
         $1386 = HEAP32[$iSet>>2]|0;
         $1387 = ($1386|0)!=(0);
         if ($1387) {
          $1388 = HEAP32[$v>>2]|0;
          $1389 = HEAP32[$regRowset>>2]|0;
          $1390 = HEAP32[$r>>2]|0;
          $1391 = HEAP32[$nPk>>2]|0;
          $1392 = (_sqlite3VdbeAddOp4Int($1388,31,$1389,0,$1390,$1391)|0);
          HEAP32[$jmp1>>2] = $1392;
         }
         $1393 = HEAP32[$iSet>>2]|0;
         $1394 = ($1393|0)>=(0);
         do {
          if ($1394) {
           $1395 = HEAP32[$v>>2]|0;
           $1396 = HEAP32[$r>>2]|0;
           $1397 = HEAP32[$nPk>>2]|0;
           $1398 = HEAP32[$regRowid>>2]|0;
           (_sqlite3VdbeAddOp3($1395,99,$1396,$1397,$1398)|0);
           $1399 = HEAP32[$v>>2]|0;
           $1400 = HEAP32[$regRowset>>2]|0;
           $1401 = HEAP32[$regRowid>>2]|0;
           (_sqlite3VdbeAddOp3($1399,126,$1400,$1401,0)|0);
           $1402 = HEAP32[$iSet>>2]|0;
           $1403 = ($1402|0)!=(0);
           if (!($1403)) {
            break;
           }
           $1404 = HEAP32[$v>>2]|0;
           _sqlite3VdbeChangeP5($1404,16);
          }
         } while(0);
         $1405 = HEAP32[$pParse>>2]|0;
         $1406 = HEAP32[$r>>2]|0;
         $1407 = HEAP32[$nPk>>2]|0;
         _sqlite3ReleaseTempRange($1405,$1406,$1407);
        }
       } while(0);
       $1408 = HEAP32[$v>>2]|0;
       $1409 = HEAP32[$regReturn>>2]|0;
       $1410 = HEAP32[$iLoopBody>>2]|0;
       (_sqlite3VdbeAddOp2($1408,14,$1409,$1410)|0);
       $1411 = HEAP32[$jmp1>>2]|0;
       $1412 = ($1411|0)!=(0);
       if ($1412) {
        $1413 = HEAP32[$v>>2]|0;
        $1414 = HEAP32[$jmp1>>2]|0;
        _sqlite3VdbeJumpHere($1413,$1414);
       }
       $1415 = HEAP32[$pSubWInfo>>2]|0;
       $1416 = ((($1415)) + 46|0);
       $1417 = HEAP8[$1416>>0]|0;
       $1418 = ($1417<<24>>24)!=(0);
       if ($1418) {
        HEAP32[$untestedTerms>>2] = 1;
       }
       $1419 = HEAP32[$pSubWInfo>>2]|0;
       $1420 = ((($1419)) + 752|0);
       $1421 = ((($1420)) + 64|0);
       $1422 = HEAP32[$1421>>2]|0;
       HEAP32[$pSubLoop>>2] = $1422;
       $1423 = HEAP32[$pSubLoop>>2]|0;
       $1424 = ((($1423)) + 36|0);
       $1425 = HEAP32[$1424>>2]|0;
       $1426 = $1425 & 512;
       $1427 = ($1426|0)!=(0);
       do {
        if ($1427) {
         $1428 = HEAP32[$ii>>2]|0;
         $1429 = ($1428|0)==(0);
         if (!($1429)) {
          $1430 = HEAP32[$pSubLoop>>2]|0;
          $1431 = ((($1430)) + 24|0);
          $1432 = ((($1431)) + 8|0);
          $1433 = HEAP32[$1432>>2]|0;
          $1434 = HEAP32[$pCov>>2]|0;
          $1435 = ($1433|0)==($1434|0);
          if (!($1435)) {
           label = 193;
           break;
          }
         }
         $1436 = HEAP32[$pTab>>2]|0;
         $1437 = ((($1436)) + 42|0);
         $1438 = HEAP8[$1437>>0]|0;
         $1439 = $1438&255;
         $1440 = $1439 & 32;
         $1441 = ($1440|0)==(0);
         if (!($1441)) {
          $1442 = HEAP32[$pSubLoop>>2]|0;
          $1443 = ((($1442)) + 24|0);
          $1444 = ((($1443)) + 8|0);
          $1445 = HEAP32[$1444>>2]|0;
          $1446 = ((($1445)) + 55|0);
          $1447 = HEAP8[$1446>>0]|0;
          $1448 = $1447 & 3;
          $1449 = $1448&255;
          $1450 = ($1449|0)==(2);
          if ($1450) {
           label = 193;
           break;
          }
         }
         $1451 = HEAP32[$pSubLoop>>2]|0;
         $1452 = ((($1451)) + 24|0);
         $1453 = ((($1452)) + 8|0);
         $1454 = HEAP32[$1453>>2]|0;
         HEAP32[$pCov>>2] = $1454;
        } else {
         label = 193;
        }
       } while(0);
       if ((label|0) == 193) {
        label = 0;
        HEAP32[$pCov>>2] = 0;
       }
       $1455 = HEAP32[$pSubWInfo>>2]|0;
       _sqlite3WhereEnd($1455);
      }
     }
     $1456 = HEAP32[$ii>>2]|0;
     $1457 = (($1456) + 1)|0;
     HEAP32[$ii>>2] = $1457;
    }
    $1458 = HEAP32[$pCov>>2]|0;
    $1459 = HEAP32[$pLevel>>2]|0;
    $1460 = ((($1459)) + 56|0);
    HEAP32[$1460>>2] = $1458;
    $1461 = HEAP32[$pCov>>2]|0;
    $1462 = ($1461|0)!=(0|0);
    if ($1462) {
     $1463 = HEAP32[$iCovCur>>2]|0;
     $1464 = HEAP32[$pLevel>>2]|0;
     $1465 = ((($1464)) + 8|0);
     HEAP32[$1465>>2] = $1463;
    }
    $1466 = HEAP32[$pAndExpr>>2]|0;
    $1467 = ($1466|0)!=(0|0);
    if ($1467) {
     $1468 = HEAP32[$pAndExpr>>2]|0;
     $1469 = ((($1468)) + 12|0);
     HEAP32[$1469>>2] = 0;
     $1470 = HEAP32[$db>>2]|0;
     $1471 = HEAP32[$pAndExpr>>2]|0;
     _sqlite3ExprDelete($1470,$1471);
    }
    $1472 = HEAP32[$v>>2]|0;
    $1473 = HEAP32[$iRetInit>>2]|0;
    $1474 = HEAP32[$v>>2]|0;
    $1475 = (_sqlite3VdbeCurrentAddr($1474)|0);
    _sqlite3VdbeChangeP1($1472,$1473,$1475);
    $1476 = HEAP32[$v>>2]|0;
    $1477 = HEAP32[$pLevel>>2]|0;
    $1478 = ((($1477)) + 12|0);
    $1479 = HEAP32[$1478>>2]|0;
    (_sqlite3VdbeGoto($1476,$1479)|0);
    $1480 = HEAP32[$v>>2]|0;
    $1481 = HEAP32[$iLoopBody>>2]|0;
    _sqlite3VdbeResolveLabel($1480,$1481);
    $1482 = HEAP32[$3>>2]|0;
    $1483 = ((($1482)) + 42|0);
    $1484 = HEAP8[$1483>>0]|0;
    $1485 = $1484&255;
    $1486 = ($1485|0)>(1);
    if ($1486) {
     $1487 = HEAP32[$db>>2]|0;
     $1488 = HEAP32[$pOrTab>>2]|0;
     _sqlite3DbFree($1487,$1488);
    }
    $1489 = HEAP32[$untestedTerms>>2]|0;
    $1490 = ($1489|0)!=(0);
    if ($1490) {
     break;
    }
    $1491 = HEAP32[$pLevel>>2]|0;
    $1492 = HEAP32[$pTerm>>2]|0;
    _disableTerm($1491,$1492);
    break;
   }
   $540 = ((($539)) + 24|0);
   $541 = HEAP16[$540>>1]|0;
   HEAP16[$nEq>>1] = $541;
   $542 = HEAP32[$pLoop>>2]|0;
   $543 = ((($542)) + 24|0);
   $544 = ((($543)) + 2|0);
   $545 = HEAP16[$544>>1]|0;
   HEAP16[$nBtm>>1] = $545;
   $546 = HEAP32[$pLoop>>2]|0;
   $547 = ((($546)) + 24|0);
   $548 = ((($547)) + 4|0);
   $549 = HEAP16[$548>>1]|0;
   HEAP16[$nTop>>1] = $549;
   HEAP32[$pRangeStart>>2] = 0;
   HEAP32[$pRangeEnd>>2] = 0;
   HEAP32[$nExtraReg>>2] = 0;
   HEAP32[$zEndAff>>2] = 0;
   HEAP8[$bSeekPastNull>>0] = 0;
   HEAP8[$bStopAtNull>>0] = 0;
   $550 = HEAP32[$pLoop>>2]|0;
   $551 = ((($550)) + 24|0);
   $552 = ((($551)) + 8|0);
   $553 = HEAP32[$552>>2]|0;
   HEAP32[$pIdx>>2] = $553;
   $554 = HEAP32[$pLevel>>2]|0;
   $555 = ((($554)) + 8|0);
   $556 = HEAP32[$555>>2]|0;
   HEAP32[$iIdxCur>>2] = $556;
   $557 = HEAP32[$3>>2]|0;
   $558 = ((($557)) + 40|0);
   $559 = HEAP16[$558>>1]|0;
   $560 = $559&65535;
   $561 = $560 & 1;
   $562 = ($561|0)!=(0);
   if ($562) {
    $563 = HEAP32[$3>>2]|0;
    $564 = ((($563)) + 43|0);
    $565 = HEAP8[$564>>0]|0;
    $566 = $565 << 24 >> 24;
    $567 = ($566|0)>(0);
    if ($567) {
     $568 = HEAP32[$pIdx>>2]|0;
     $569 = ((($568)) + 50|0);
     $570 = HEAP16[$569>>1]|0;
     $571 = $570&65535;
     $572 = HEAP16[$nEq>>1]|0;
     $573 = $572&65535;
     $574 = ($571|0)>($573|0);
     if ($574) {
      HEAP8[$bSeekPastNull>>0] = 1;
      HEAP32[$nExtraReg>>2] = 1;
     }
    }
   }
   $575 = HEAP16[$nEq>>1]|0;
   $576 = $575&65535;
   HEAP32[$j>>2] = $576;
   $577 = HEAP32[$pLoop>>2]|0;
   $578 = ((($577)) + 36|0);
   $579 = HEAP32[$578>>2]|0;
   $580 = $579 & 32;
   $581 = ($580|0)!=(0);
   if ($581) {
    $582 = HEAP32[$j>>2]|0;
    $583 = (($582) + 1)|0;
    HEAP32[$j>>2] = $583;
    $584 = HEAP32[$pLoop>>2]|0;
    $585 = ((($584)) + 48|0);
    $586 = HEAP32[$585>>2]|0;
    $587 = (($586) + ($582<<2)|0);
    $588 = HEAP32[$587>>2]|0;
    HEAP32[$pRangeStart>>2] = $588;
    $589 = HEAP32[$nExtraReg>>2]|0;
    $590 = HEAP32[$pLoop>>2]|0;
    $591 = ((($590)) + 24|0);
    $592 = ((($591)) + 2|0);
    $593 = HEAP16[$592>>1]|0;
    $594 = $593&65535;
    $595 = ($589|0)>($594|0);
    if ($595) {
     $596 = HEAP32[$nExtraReg>>2]|0;
     $602 = $596;
    } else {
     $597 = HEAP32[$pLoop>>2]|0;
     $598 = ((($597)) + 24|0);
     $599 = ((($598)) + 2|0);
     $600 = HEAP16[$599>>1]|0;
     $601 = $600&65535;
     $602 = $601;
    }
    HEAP32[$nExtraReg>>2] = $602;
   }
   $603 = HEAP32[$pLoop>>2]|0;
   $604 = ((($603)) + 36|0);
   $605 = HEAP32[$604>>2]|0;
   $606 = $605 & 16;
   $607 = ($606|0)!=(0);
   do {
    if ($607) {
     $608 = HEAP32[$j>>2]|0;
     $609 = (($608) + 1)|0;
     HEAP32[$j>>2] = $609;
     $610 = HEAP32[$pLoop>>2]|0;
     $611 = ((($610)) + 48|0);
     $612 = HEAP32[$611>>2]|0;
     $613 = (($612) + ($608<<2)|0);
     $614 = HEAP32[$613>>2]|0;
     HEAP32[$pRangeEnd>>2] = $614;
     $615 = HEAP32[$nExtraReg>>2]|0;
     $616 = HEAP32[$pLoop>>2]|0;
     $617 = ((($616)) + 24|0);
     $618 = ((($617)) + 4|0);
     $619 = HEAP16[$618>>1]|0;
     $620 = $619&65535;
     $621 = ($615|0)>($620|0);
     if ($621) {
      $622 = HEAP32[$nExtraReg>>2]|0;
      $628 = $622;
     } else {
      $623 = HEAP32[$pLoop>>2]|0;
      $624 = ((($623)) + 24|0);
      $625 = ((($624)) + 4|0);
      $626 = HEAP16[$625>>1]|0;
      $627 = $626&65535;
      $628 = $627;
     }
     HEAP32[$nExtraReg>>2] = $628;
     $629 = HEAP32[$pRangeEnd>>2]|0;
     $630 = ((($629)) + 10|0);
     $631 = HEAP16[$630>>1]|0;
     $632 = $631&65535;
     $633 = $632 & 256;
     $634 = ($633|0)!=(0);
     if ($634) {
      $635 = HEAP32[$pParse>>2]|0;
      $636 = ((($635)) + 44|0);
      $637 = HEAP32[$636>>2]|0;
      $638 = (($637) + 1)|0;
      HEAP32[$636>>2] = $638;
      $639 = HEAP32[$pLevel>>2]|0;
      $640 = ((($639)) + 36|0);
      HEAP32[$640>>2] = $638;
      $641 = HEAP32[$v>>2]|0;
      $642 = HEAP32[$pLevel>>2]|0;
      $643 = ((($642)) + 36|0);
      $644 = HEAP32[$643>>2]|0;
      (_sqlite3VdbeAddOp2($641,76,1,$644)|0);
      $645 = HEAP32[$v>>2]|0;
      $646 = (_sqlite3VdbeCurrentAddr($645)|0);
      $647 = HEAP32[$pLevel>>2]|0;
      $648 = ((($647)) + 40|0);
      HEAP32[$648>>2] = $646;
      $649 = HEAP32[$pLevel>>2]|0;
      $650 = ((($649)) + 36|0);
      $651 = HEAP32[$650>>2]|0;
      $652 = $651 << 1;
      HEAP32[$650>>2] = $652;
      $653 = HEAP32[$bRev>>2]|0;
      $654 = HEAP16[$nEq>>1]|0;
      $655 = $654&65535;
      $656 = HEAP32[$pIdx>>2]|0;
      $657 = ((($656)) + 28|0);
      $658 = HEAP32[$657>>2]|0;
      $659 = (($658) + ($655)|0);
      $660 = HEAP8[$659>>0]|0;
      $661 = $660&255;
      $662 = ($661|0)==(1);
      $663 = $662&1;
      $664 = $653 ^ $663;
      $665 = HEAP32[$pLevel>>2]|0;
      $666 = ((($665)) + 36|0);
      $667 = HEAP32[$666>>2]|0;
      $668 = $667 | $664;
      HEAP32[$666>>2] = $668;
     }
     $669 = HEAP32[$pRangeStart>>2]|0;
     $670 = ($669|0)==(0|0);
     if ($670) {
      $671 = HEAP16[$nEq>>1]|0;
      $672 = $671&65535;
      $673 = HEAP32[$pIdx>>2]|0;
      $674 = ((($673)) + 4|0);
      $675 = HEAP32[$674>>2]|0;
      $676 = (($675) + ($672<<1)|0);
      $677 = HEAP16[$676>>1]|0;
      $678 = $677 << 16 >> 16;
      HEAP32[$j>>2] = $678;
      $679 = HEAP32[$j>>2]|0;
      $680 = ($679|0)>=(0);
      $681 = HEAP32[$j>>2]|0;
      if ($680) {
       $682 = HEAP32[$pIdx>>2]|0;
       $683 = ((($682)) + 12|0);
       $684 = HEAP32[$683>>2]|0;
       $685 = ((($684)) + 4|0);
       $686 = HEAP32[$685>>2]|0;
       $687 = (($686) + ($681<<4)|0);
       $688 = ((($687)) + 12|0);
       $689 = HEAP8[$688>>0]|0;
       $690 = $689&255;
       $691 = ($690|0)==(0);
       $692 = HEAP32[$j>>2]|0;
       $693 = ($692|0)==(-2);
       $or$cond = $691 | $693;
       if (!($or$cond)) {
        break;
       }
      } else {
       $$old1 = ($681|0)==(-2);
       if (!($$old1)) {
        break;
       }
      }
      HEAP8[$bSeekPastNull>>0] = 1;
     }
    }
   } while(0);
   $694 = HEAP16[$nEq>>1]|0;
   $695 = $694&65535;
   $696 = HEAP32[$pIdx>>2]|0;
   $697 = ((($696)) + 50|0);
   $698 = HEAP16[$697>>1]|0;
   $699 = $698&65535;
   $700 = ($695|0)<($699|0);
   if ($700) {
    $701 = HEAP32[$bRev>>2]|0;
    $702 = HEAP16[$nEq>>1]|0;
    $703 = $702&65535;
    $704 = HEAP32[$pIdx>>2]|0;
    $705 = ((($704)) + 28|0);
    $706 = HEAP32[$705>>2]|0;
    $707 = (($706) + ($703)|0);
    $708 = HEAP8[$707>>0]|0;
    $709 = $708&255;
    $710 = ($709|0)==(0);
    $711 = $710&1;
    $712 = ($701|0)==($711|0);
    if ($712) {
     label = 84;
    } else {
     label = 82;
    }
   } else {
    label = 82;
   }
   if ((label|0) == 82) {
    $713 = HEAP32[$bRev>>2]|0;
    $714 = ($713|0)!=(0);
    if ($714) {
     $715 = HEAP32[$pIdx>>2]|0;
     $716 = ((($715)) + 50|0);
     $717 = HEAP16[$716>>1]|0;
     $718 = $717&65535;
     $719 = HEAP16[$nEq>>1]|0;
     $720 = $719&65535;
     $721 = ($718|0)==($720|0);
     if ($721) {
      label = 84;
     }
    }
   }
   if ((label|0) == 84) {
    $722 = HEAP32[$pRangeEnd>>2]|0;
    HEAP32[$t>>2] = $722;
    $723 = HEAP32[$pRangeStart>>2]|0;
    HEAP32[$pRangeEnd>>2] = $723;
    $724 = HEAP32[$t>>2]|0;
    HEAP32[$pRangeStart>>2] = $724;
    $725 = HEAP8[$bSeekPastNull>>0]|0;
    HEAP8[$t5>>0] = $725;
    $726 = HEAP8[$bStopAtNull>>0]|0;
    HEAP8[$bSeekPastNull>>0] = $726;
    $727 = HEAP8[$t5>>0]|0;
    HEAP8[$bStopAtNull>>0] = $727;
    $728 = HEAP16[$nBtm>>1]|0;
    $729 = $728&255;
    HEAP8[$t6>>0] = $729;
    $730 = HEAP16[$nTop>>1]|0;
    HEAP16[$nBtm>>1] = $730;
    $731 = HEAP8[$t6>>0]|0;
    $732 = $731&255;
    HEAP16[$nTop>>1] = $732;
   }
   $733 = HEAP32[$pParse>>2]|0;
   $734 = HEAP32[$pLevel>>2]|0;
   $735 = HEAP32[$bRev>>2]|0;
   $736 = HEAP32[$nExtraReg>>2]|0;
   $737 = (_codeAllEqualityTerms($733,$734,$735,$736,$zStartAff)|0);
   HEAP32[$regBase>>2] = $737;
   $738 = HEAP32[$zStartAff>>2]|0;
   $739 = ($738|0)!=(0|0);
   if ($739) {
    $740 = HEAP16[$nTop>>1]|0;
    $741 = $740&65535;
    $742 = ($741|0)!=(0);
    if ($742) {
     $743 = HEAP32[$db>>2]|0;
     $744 = HEAP16[$nEq>>1]|0;
     $745 = $744&65535;
     $746 = HEAP32[$zStartAff>>2]|0;
     $747 = (($746) + ($745)|0);
     $748 = (_sqlite3DbStrDup($743,$747)|0);
     HEAP32[$zEndAff>>2] = $748;
    }
   }
   $749 = HEAP32[$pLevel>>2]|0;
   $750 = ((($749)) + 16|0);
   $751 = HEAP32[$750>>2]|0;
   HEAP32[$addrNxt>>2] = $751;
   $752 = HEAP32[$pRangeStart>>2]|0;
   $753 = ($752|0)!=(0|0);
   if ($753) {
    $754 = HEAP32[$pRangeStart>>2]|0;
    $755 = ((($754)) + 12|0);
    $756 = HEAP16[$755>>1]|0;
    $757 = $756&65535;
    $758 = $757 & 40;
    $759 = ($758|0)!=(0);
    $761 = $759;
   } else {
    $761 = 1;
   }
   $760 = $761&1;
   HEAP32[$startEq>>2] = $760;
   $762 = HEAP32[$pRangeEnd>>2]|0;
   $763 = ($762|0)!=(0|0);
   if ($763) {
    $764 = HEAP32[$pRangeEnd>>2]|0;
    $765 = ((($764)) + 12|0);
    $766 = HEAP16[$765>>1]|0;
    $767 = $766&65535;
    $768 = $767 & 40;
    $769 = ($768|0)!=(0);
    $771 = $769;
   } else {
    $771 = 1;
   }
   $770 = $771&1;
   HEAP32[$endEq>>2] = $770;
   $772 = HEAP32[$pRangeStart>>2]|0;
   $773 = ($772|0)!=(0|0);
   if ($773) {
    $778 = 1;
   } else {
    $774 = HEAP16[$nEq>>1]|0;
    $775 = $774&65535;
    $776 = ($775|0)>(0);
    $778 = $776;
   }
   $777 = $778&1;
   HEAP32[$start_constraints>>2] = $777;
   $779 = HEAP16[$nEq>>1]|0;
   $780 = $779&65535;
   HEAP32[$nConstraint3>>2] = $780;
   $781 = HEAP32[$pRangeStart>>2]|0;
   $782 = ($781|0)!=(0|0);
   if ($782) {
    $783 = HEAP32[$pRangeStart>>2]|0;
    $784 = HEAP32[$783>>2]|0;
    $785 = ((($784)) + 16|0);
    $786 = HEAP32[$785>>2]|0;
    HEAP32[$pRight7>>2] = $786;
    $787 = HEAP32[$pParse>>2]|0;
    $788 = HEAP32[$pRight7>>2]|0;
    $789 = HEAP32[$regBase>>2]|0;
    $790 = HEAP16[$nEq>>1]|0;
    $791 = $790&65535;
    $792 = (($789) + ($791))|0;
    $793 = HEAP16[$nBtm>>1]|0;
    $794 = $793&65535;
    _codeExprOrVector($787,$788,$792,$794);
    $795 = HEAP32[$v>>2]|0;
    $796 = HEAP32[$pLevel>>2]|0;
    $797 = HEAP32[$pRangeStart>>2]|0;
    _whereLikeOptimizationStringFixup($795,$796,$797);
    $798 = HEAP32[$pRangeStart>>2]|0;
    $799 = ((($798)) + 10|0);
    $800 = HEAP16[$799>>1]|0;
    $801 = $800&65535;
    $802 = $801 & 0;
    $803 = ($802|0)==(0);
    do {
     if ($803) {
      $804 = HEAP32[$pRight7>>2]|0;
      $805 = (_sqlite3ExprCanBeNull($804)|0);
      $806 = ($805|0)!=(0);
      if (!($806)) {
       break;
      }
      $807 = HEAP32[$v>>2]|0;
      $808 = HEAP32[$regBase>>2]|0;
      $809 = HEAP16[$nEq>>1]|0;
      $810 = $809&65535;
      $811 = (($808) + ($810))|0;
      $812 = HEAP32[$addrNxt>>2]|0;
      (_sqlite3VdbeAddOp2($807,34,$811,$812)|0);
     }
    } while(0);
    $813 = HEAP32[$zStartAff>>2]|0;
    $814 = ($813|0)!=(0|0);
    if ($814) {
     $815 = HEAP32[$pRight7>>2]|0;
     $816 = HEAP16[$nBtm>>1]|0;
     $817 = $816&65535;
     $818 = HEAP16[$nEq>>1]|0;
     $819 = $818&65535;
     $820 = HEAP32[$zStartAff>>2]|0;
     $821 = (($820) + ($819)|0);
     _updateRangeAffinityStr($815,$817,$821);
    }
    $822 = HEAP16[$nBtm>>1]|0;
    $823 = $822&65535;
    $824 = HEAP32[$nConstraint3>>2]|0;
    $825 = (($824) + ($823))|0;
    HEAP32[$nConstraint3>>2] = $825;
    $826 = HEAP32[$pRight7>>2]|0;
    $827 = (_sqlite3ExprIsVector($826)|0);
    $828 = ($827|0)==(0);
    if ($828) {
     $829 = HEAP32[$pLevel>>2]|0;
     $830 = HEAP32[$pRangeStart>>2]|0;
     _disableTerm($829,$830);
    } else {
     HEAP32[$startEq>>2] = 1;
    }
    HEAP8[$bSeekPastNull>>0] = 0;
   } else {
    $831 = HEAP8[$bSeekPastNull>>0]|0;
    $832 = ($831<<24>>24)!=(0);
    if ($832) {
     $833 = HEAP32[$v>>2]|0;
     $834 = HEAP32[$regBase>>2]|0;
     $835 = HEAP16[$nEq>>1]|0;
     $836 = $835&65535;
     $837 = (($834) + ($836))|0;
     (_sqlite3VdbeAddOp2($833,79,0,$837)|0);
     $838 = HEAP32[$nConstraint3>>2]|0;
     $839 = (($838) + 1)|0;
     HEAP32[$nConstraint3>>2] = $839;
     HEAP32[$startEq>>2] = 0;
     HEAP32[$start_constraints>>2] = 1;
    }
   }
   $840 = HEAP32[$pParse>>2]|0;
   $841 = HEAP32[$regBase>>2]|0;
   $842 = HEAP32[$nConstraint3>>2]|0;
   $843 = HEAP8[$bSeekPastNull>>0]|0;
   $844 = $843&255;
   $845 = (($842) - ($844))|0;
   $846 = HEAP32[$zStartAff>>2]|0;
   _codeApplyAffinity($840,$841,$845,$846);
   $847 = HEAP32[$pLoop>>2]|0;
   $848 = ((($847)) + 42|0);
   $849 = HEAP16[$848>>1]|0;
   $850 = $849&65535;
   $851 = ($850|0)>(0);
   if ($851) {
    $852 = HEAP32[$nConstraint3>>2]|0;
    $853 = HEAP32[$pLoop>>2]|0;
    $854 = ((($853)) + 42|0);
    $855 = HEAP16[$854>>1]|0;
    $856 = $855&65535;
    $857 = ($852|0)==($856|0);
    if (!($857)) {
     label = 108;
    }
   } else {
    label = 108;
   }
   if ((label|0) == 108) {
    $858 = HEAP32[$start_constraints>>2]|0;
    $859 = $858 << 2;
    $860 = HEAP32[$startEq>>2]|0;
    $861 = $860 << 1;
    $862 = (($859) + ($861))|0;
    $863 = HEAP32[$bRev>>2]|0;
    $864 = (($862) + ($863))|0;
    $865 = (34436 + ($864)|0);
    $866 = HEAP8[$865>>0]|0;
    $867 = $866&255;
    HEAP32[$op4>>2] = $867;
    $868 = HEAP32[$v>>2]|0;
    $869 = HEAP32[$op4>>2]|0;
    $870 = HEAP32[$iIdxCur>>2]|0;
    $871 = HEAP32[$addrNxt>>2]|0;
    $872 = HEAP32[$regBase>>2]|0;
    $873 = HEAP32[$nConstraint3>>2]|0;
    (_sqlite3VdbeAddOp4Int($868,$869,$870,$871,$872,$873)|0);
   }
   $874 = HEAP16[$nEq>>1]|0;
   $875 = $874&65535;
   HEAP32[$nConstraint3>>2] = $875;
   $876 = HEAP32[$pRangeEnd>>2]|0;
   $877 = ($876|0)!=(0|0);
   do {
    if ($877) {
     $878 = HEAP32[$pRangeEnd>>2]|0;
     $879 = HEAP32[$878>>2]|0;
     $880 = ((($879)) + 16|0);
     $881 = HEAP32[$880>>2]|0;
     HEAP32[$pRight8>>2] = $881;
     $882 = HEAP32[$pParse>>2]|0;
     $883 = HEAP32[$regBase>>2]|0;
     $884 = HEAP16[$nEq>>1]|0;
     $885 = $884&65535;
     $886 = (($883) + ($885))|0;
     _sqlite3ExprCacheRemove($882,$886,1);
     $887 = HEAP32[$pParse>>2]|0;
     $888 = HEAP32[$pRight8>>2]|0;
     $889 = HEAP32[$regBase>>2]|0;
     $890 = HEAP16[$nEq>>1]|0;
     $891 = $890&65535;
     $892 = (($889) + ($891))|0;
     $893 = HEAP16[$nTop>>1]|0;
     $894 = $893&65535;
     _codeExprOrVector($887,$888,$892,$894);
     $895 = HEAP32[$v>>2]|0;
     $896 = HEAP32[$pLevel>>2]|0;
     $897 = HEAP32[$pRangeEnd>>2]|0;
     _whereLikeOptimizationStringFixup($895,$896,$897);
     $898 = HEAP32[$pRangeEnd>>2]|0;
     $899 = ((($898)) + 10|0);
     $900 = HEAP16[$899>>1]|0;
     $901 = $900&65535;
     $902 = $901 & 0;
     $903 = ($902|0)==(0);
     do {
      if ($903) {
       $904 = HEAP32[$pRight8>>2]|0;
       $905 = (_sqlite3ExprCanBeNull($904)|0);
       $906 = ($905|0)!=(0);
       if (!($906)) {
        break;
       }
       $907 = HEAP32[$v>>2]|0;
       $908 = HEAP32[$regBase>>2]|0;
       $909 = HEAP16[$nEq>>1]|0;
       $910 = $909&65535;
       $911 = (($908) + ($910))|0;
       $912 = HEAP32[$addrNxt>>2]|0;
       (_sqlite3VdbeAddOp2($907,34,$911,$912)|0);
      }
     } while(0);
     $913 = HEAP32[$zEndAff>>2]|0;
     $914 = ($913|0)!=(0|0);
     if ($914) {
      $915 = HEAP32[$pRight8>>2]|0;
      $916 = HEAP16[$nTop>>1]|0;
      $917 = $916&65535;
      $918 = HEAP32[$zEndAff>>2]|0;
      _updateRangeAffinityStr($915,$917,$918);
      $919 = HEAP32[$pParse>>2]|0;
      $920 = HEAP32[$regBase>>2]|0;
      $921 = HEAP16[$nEq>>1]|0;
      $922 = $921&65535;
      $923 = (($920) + ($922))|0;
      $924 = HEAP16[$nTop>>1]|0;
      $925 = $924&65535;
      $926 = HEAP32[$zEndAff>>2]|0;
      _codeApplyAffinity($919,$923,$925,$926);
     }
     $927 = HEAP16[$nTop>>1]|0;
     $928 = $927&65535;
     $929 = HEAP32[$nConstraint3>>2]|0;
     $930 = (($929) + ($928))|0;
     HEAP32[$nConstraint3>>2] = $930;
     $931 = HEAP32[$pRight8>>2]|0;
     $932 = (_sqlite3ExprIsVector($931)|0);
     $933 = ($932|0)==(0);
     if ($933) {
      $934 = HEAP32[$pLevel>>2]|0;
      $935 = HEAP32[$pRangeEnd>>2]|0;
      _disableTerm($934,$935);
      break;
     } else {
      HEAP32[$endEq>>2] = 1;
      break;
     }
    } else {
     $936 = HEAP8[$bStopAtNull>>0]|0;
     $937 = ($936<<24>>24)!=(0);
     if (!($937)) {
      break;
     }
     $938 = HEAP32[$v>>2]|0;
     $939 = HEAP32[$regBase>>2]|0;
     $940 = HEAP16[$nEq>>1]|0;
     $941 = $940&65535;
     $942 = (($939) + ($941))|0;
     (_sqlite3VdbeAddOp2($938,79,0,$942)|0);
     HEAP32[$endEq>>2] = 0;
     $943 = HEAP32[$nConstraint3>>2]|0;
     $944 = (($943) + 1)|0;
     HEAP32[$nConstraint3>>2] = $944;
    }
   } while(0);
   $945 = HEAP32[$db>>2]|0;
   $946 = HEAP32[$zStartAff>>2]|0;
   _sqlite3DbFree($945,$946);
   $947 = HEAP32[$db>>2]|0;
   $948 = HEAP32[$zEndAff>>2]|0;
   _sqlite3DbFree($947,$948);
   $949 = HEAP32[$v>>2]|0;
   $950 = (_sqlite3VdbeCurrentAddr($949)|0);
   $951 = HEAP32[$pLevel>>2]|0;
   $952 = ((($951)) + 52|0);
   HEAP32[$952>>2] = $950;
   $953 = HEAP32[$nConstraint3>>2]|0;
   $954 = ($953|0)!=(0);
   if ($954) {
    $955 = HEAP32[$bRev>>2]|0;
    $956 = $955<<1;
    $957 = HEAP32[$endEq>>2]|0;
    $958 = (($956) + ($957))|0;
    $959 = (34444 + ($958)|0);
    $960 = HEAP8[$959>>0]|0;
    $961 = $960&255;
    HEAP32[$op4>>2] = $961;
    $962 = HEAP32[$v>>2]|0;
    $963 = HEAP32[$op4>>2]|0;
    $964 = HEAP32[$iIdxCur>>2]|0;
    $965 = HEAP32[$addrNxt>>2]|0;
    $966 = HEAP32[$regBase>>2]|0;
    $967 = HEAP32[$nConstraint3>>2]|0;
    (_sqlite3VdbeAddOp4Int($962,$963,$964,$965,$966,$967)|0);
   }
   $968 = HEAP32[$omitTable>>2]|0;
   $969 = ($968|0)!=(0);
   do {
    if (!($969)) {
     $970 = HEAP32[$pIdx>>2]|0;
     $971 = ((($970)) + 12|0);
     $972 = HEAP32[$971>>2]|0;
     $973 = ((($972)) + 42|0);
     $974 = HEAP8[$973>>0]|0;
     $975 = $974&255;
     $976 = $975 & 32;
     $977 = ($976|0)==(0);
     if ($977) {
      $978 = HEAP32[$3>>2]|0;
      $979 = ((($978)) + 40|0);
      $980 = HEAP16[$979>>1]|0;
      $981 = $980&65535;
      $982 = $981 & 1024;
      $983 = ($982|0)!=(0);
      if ($983) {
       $984 = HEAP32[$pParse>>2]|0;
       $985 = ((($984)) + 44|0);
       $986 = HEAP32[$985>>2]|0;
       $987 = (($986) + 1)|0;
       HEAP32[$985>>2] = $987;
       HEAP32[$iRowidReg>>2] = $987;
       $988 = HEAP32[$v>>2]|0;
       $989 = HEAP32[$iIdxCur>>2]|0;
       $990 = HEAP32[$iRowidReg>>2]|0;
       (_sqlite3VdbeAddOp2($988,129,$989,$990)|0);
       $991 = HEAP32[$pParse>>2]|0;
       $992 = HEAP32[$iCur>>2]|0;
       $993 = HEAP32[$iRowidReg>>2]|0;
       _sqlite3ExprCacheStore($991,$992,-1,$993);
       $994 = HEAP32[$v>>2]|0;
       $995 = HEAP32[$iCur>>2]|0;
       $996 = HEAP32[$iRowidReg>>2]|0;
       (_sqlite3VdbeAddOp3($994,33,$995,0,$996)|0);
       break;
      } else {
       $997 = HEAP32[$3>>2]|0;
       $998 = HEAP32[$pIdx>>2]|0;
       $999 = HEAP32[$iCur>>2]|0;
       $1000 = HEAP32[$iIdxCur>>2]|0;
       _codeDeferredSeek($997,$998,$999,$1000);
       break;
      }
     }
     $1001 = HEAP32[$iCur>>2]|0;
     $1002 = HEAP32[$iIdxCur>>2]|0;
     $1003 = ($1001|0)!=($1002|0);
     if (!($1003)) {
      break;
     }
     $1004 = HEAP32[$pIdx>>2]|0;
     $1005 = ((($1004)) + 12|0);
     $1006 = HEAP32[$1005>>2]|0;
     $1007 = (_sqlite3PrimaryKeyIndex($1006)|0);
     HEAP32[$pPk>>2] = $1007;
     $1008 = HEAP32[$pParse>>2]|0;
     $1009 = HEAP32[$pPk>>2]|0;
     $1010 = ((($1009)) + 50|0);
     $1011 = HEAP16[$1010>>1]|0;
     $1012 = $1011&65535;
     $1013 = (_sqlite3GetTempRange($1008,$1012)|0);
     HEAP32[$iRowidReg>>2] = $1013;
     HEAP32[$j>>2] = 0;
     while(1) {
      $1014 = HEAP32[$j>>2]|0;
      $1015 = HEAP32[$pPk>>2]|0;
      $1016 = ((($1015)) + 50|0);
      $1017 = HEAP16[$1016>>1]|0;
      $1018 = $1017&65535;
      $1019 = ($1014|0)<($1018|0);
      if (!($1019)) {
       break;
      }
      $1020 = HEAP32[$pIdx>>2]|0;
      $1021 = HEAP32[$j>>2]|0;
      $1022 = HEAP32[$pPk>>2]|0;
      $1023 = ((($1022)) + 4|0);
      $1024 = HEAP32[$1023>>2]|0;
      $1025 = (($1024) + ($1021<<1)|0);
      $1026 = HEAP16[$1025>>1]|0;
      $1027 = (_sqlite3ColumnOfIndex($1020,$1026)|0);
      $1028 = $1027 << 16 >> 16;
      HEAP32[$k>>2] = $1028;
      $1029 = HEAP32[$v>>2]|0;
      $1030 = HEAP32[$iIdxCur>>2]|0;
      $1031 = HEAP32[$k>>2]|0;
      $1032 = HEAP32[$iRowidReg>>2]|0;
      $1033 = HEAP32[$j>>2]|0;
      $1034 = (($1032) + ($1033))|0;
      (_sqlite3VdbeAddOp3($1029,96,$1030,$1031,$1034)|0);
      $1035 = HEAP32[$j>>2]|0;
      $1036 = (($1035) + 1)|0;
      HEAP32[$j>>2] = $1036;
     }
     $1037 = HEAP32[$v>>2]|0;
     $1038 = HEAP32[$iCur>>2]|0;
     $1039 = HEAP32[$addrCont>>2]|0;
     $1040 = HEAP32[$iRowidReg>>2]|0;
     $1041 = HEAP32[$pPk>>2]|0;
     $1042 = ((($1041)) + 50|0);
     $1043 = HEAP16[$1042>>1]|0;
     $1044 = $1043&65535;
     (_sqlite3VdbeAddOp4Int($1037,30,$1038,$1039,$1040,$1044)|0);
    }
   } while(0);
   $1045 = HEAP32[$pLoop>>2]|0;
   $1046 = ((($1045)) + 36|0);
   $1047 = HEAP32[$1046>>2]|0;
   $1048 = $1047 & 4096;
   $1049 = ($1048|0)!=(0);
   do {
    if ($1049) {
     $1050 = HEAP32[$pLevel>>2]|0;
     $1051 = ((($1050)) + 45|0);
     HEAP8[$1051>>0] = -95;
    } else {
     $1052 = HEAP32[$bRev>>2]|0;
     $1053 = ($1052|0)!=(0);
     $1054 = HEAP32[$pLevel>>2]|0;
     $1055 = ((($1054)) + 45|0);
     if ($1053) {
      HEAP8[$1055>>0] = 6;
      break;
     } else {
      HEAP8[$1055>>0] = 7;
      break;
     }
    }
   } while(0);
   $1056 = HEAP32[$iIdxCur>>2]|0;
   $1057 = HEAP32[$pLevel>>2]|0;
   $1058 = ((($1057)) + 48|0);
   HEAP32[$1058>>2] = $1056;
   $1059 = HEAP32[$pLoop>>2]|0;
   $1060 = ((($1059)) + 36|0);
   $1061 = HEAP32[$1060>>2]|0;
   $1062 = $1061 & 65536;
   $1063 = ($1062|0)!=(0);
   $1064 = $1063 ? 1 : 0;
   $1065 = $1064&255;
   $1066 = HEAP32[$pLevel>>2]|0;
   $1067 = ((($1066)) + 46|0);
   HEAP8[$1067>>0] = $1065;
   $1068 = HEAP32[$pLoop>>2]|0;
   $1069 = ((($1068)) + 36|0);
   $1070 = HEAP32[$1069>>2]|0;
   $1071 = $1070 & 15;
   $1072 = ($1071|0)==(0);
   if (!($1072)) {
    break;
   }
   $1073 = HEAP32[$pLevel>>2]|0;
   $1074 = ((($1073)) + 47|0);
   HEAP8[$1074>>0] = 1;
  }
 } while(0);
 $1524 = HEAP32[$pWC>>2]|0;
 $1525 = ((($1524)) + 20|0);
 $1526 = HEAP32[$1525>>2]|0;
 HEAP32[$pTerm>>2] = $1526;
 $1527 = HEAP32[$pWC>>2]|0;
 $1528 = ((($1527)) + 12|0);
 $1529 = HEAP32[$1528>>2]|0;
 HEAP32[$j>>2] = $1529;
 while(1) {
  $1530 = HEAP32[$j>>2]|0;
  $1531 = ($1530|0)>(0);
  if (!($1531)) {
   break;
  }
  HEAP32[$skipLikeAddr>>2] = 0;
  $1532 = HEAP32[$pTerm>>2]|0;
  $1533 = ((($1532)) + 10|0);
  $1534 = HEAP16[$1533>>1]|0;
  $1535 = $1534&65535;
  $1536 = $1535 & 6;
  $1537 = ($1536|0)!=(0);
  do {
   if (!($1537)) {
    $1538 = HEAP32[$pTerm>>2]|0;
    $1539 = ((($1538)) + 40|0);
    $1540 = $1539;
    $1541 = $1540;
    $1542 = HEAP32[$1541>>2]|0;
    $1543 = (($1540) + 4)|0;
    $1544 = $1543;
    $1545 = HEAP32[$1544>>2]|0;
    $1546 = HEAP32[$pLevel>>2]|0;
    $1547 = ((($1546)) + 72|0);
    $1548 = $1547;
    $1549 = $1548;
    $1550 = HEAP32[$1549>>2]|0;
    $1551 = (($1548) + 4)|0;
    $1552 = $1551;
    $1553 = HEAP32[$1552>>2]|0;
    $1554 = $1542 & $1550;
    $1555 = $1545 & $1553;
    $1556 = ($1554|0)!=(0);
    $1557 = ($1555|0)!=(0);
    $1558 = $1556 | $1557;
    if ($1558) {
     $1559 = HEAP32[$3>>2]|0;
     $1560 = ((($1559)) + 46|0);
     HEAP8[$1560>>0] = 1;
     break;
    }
    $1561 = HEAP32[$pTerm>>2]|0;
    $1562 = HEAP32[$1561>>2]|0;
    HEAP32[$pE>>2] = $1562;
    $1563 = HEAP32[$pLevel>>2]|0;
    $1564 = HEAP32[$1563>>2]|0;
    $1565 = ($1564|0)!=(0);
    if ($1565) {
     $1566 = HEAP32[$pE>>2]|0;
     $1567 = ((($1566)) + 4|0);
     $1568 = HEAP32[$1567>>2]|0;
     $1569 = $1568 & 1;
     $1570 = ($1569|0)!=(0);
     if (!($1570)) {
      break;
     }
    }
    $1571 = HEAP32[$pTerm>>2]|0;
    $1572 = ((($1571)) + 10|0);
    $1573 = HEAP16[$1572>>1]|0;
    $1574 = $1573&65535;
    $1575 = $1574 & 512;
    $1576 = ($1575|0)!=(0);
    if ($1576) {
     $1577 = HEAP32[$pLevel>>2]|0;
     $1578 = ((($1577)) + 36|0);
     $1579 = HEAP32[$1578>>2]|0;
     HEAP32[$x>>2] = $1579;
     $1580 = HEAP32[$v>>2]|0;
     $1581 = HEAP32[$x>>2]|0;
     $1582 = $1581 & 1;
     $1583 = ($1582|0)!=(0);
     $1584 = $1583 ? 22 : 21;
     $1585 = HEAP32[$x>>2]|0;
     $1586 = $1585 >>> 1;
     $1587 = (_sqlite3VdbeAddOp1($1580,$1584,$1586)|0);
     HEAP32[$skipLikeAddr>>2] = $1587;
    }
    $1588 = HEAP32[$pParse>>2]|0;
    $1589 = HEAP32[$pE>>2]|0;
    $1590 = HEAP32[$addrCont>>2]|0;
    _sqlite3ExprIfFalse($1588,$1589,$1590,16);
    $1591 = HEAP32[$skipLikeAddr>>2]|0;
    $1592 = ($1591|0)!=(0);
    if ($1592) {
     $1593 = HEAP32[$v>>2]|0;
     $1594 = HEAP32[$skipLikeAddr>>2]|0;
     _sqlite3VdbeJumpHere($1593,$1594);
    }
    $1595 = HEAP32[$pTerm>>2]|0;
    $1596 = ((($1595)) + 10|0);
    $1597 = HEAP16[$1596>>1]|0;
    $1598 = $1597&65535;
    $1599 = $1598 | 4;
    $1600 = $1599&65535;
    HEAP16[$1596>>1] = $1600;
   }
  } while(0);
  $1601 = HEAP32[$j>>2]|0;
  $1602 = (($1601) + -1)|0;
  HEAP32[$j>>2] = $1602;
  $1603 = HEAP32[$pTerm>>2]|0;
  $1604 = ((($1603)) + 48|0);
  HEAP32[$pTerm>>2] = $1604;
 }
 $1605 = HEAP32[$pWC>>2]|0;
 $1606 = ((($1605)) + 20|0);
 $1607 = HEAP32[$1606>>2]|0;
 HEAP32[$pTerm>>2] = $1607;
 $1608 = HEAP32[$pWC>>2]|0;
 $1609 = ((($1608)) + 12|0);
 $1610 = HEAP32[$1609>>2]|0;
 HEAP32[$j>>2] = $1610;
 while(1) {
  $1611 = HEAP32[$j>>2]|0;
  $1612 = ($1611|0)>(0);
  if (!($1612)) {
   break;
  }
  $1613 = HEAP32[$pTerm>>2]|0;
  $1614 = ((($1613)) + 10|0);
  $1615 = HEAP16[$1614>>1]|0;
  $1616 = $1615&65535;
  $1617 = $1616 & 6;
  $1618 = ($1617|0)!=(0);
  if (!($1618)) {
   $1619 = HEAP32[$pTerm>>2]|0;
   $1620 = ((($1619)) + 12|0);
   $1621 = HEAP16[$1620>>1]|0;
   $1622 = $1621&65535;
   $1623 = $1622 & 130;
   $1624 = ($1623|0)==(0);
   if (!($1624)) {
    $1625 = HEAP32[$pTerm>>2]|0;
    $1626 = ((($1625)) + 12|0);
    $1627 = HEAP16[$1626>>1]|0;
    $1628 = $1627&65535;
    $1629 = $1628 & 2048;
    $1630 = ($1629|0)==(0);
    if (!($1630)) {
     $1631 = HEAP32[$pTerm>>2]|0;
     $1632 = ((($1631)) + 20|0);
     $1633 = HEAP32[$1632>>2]|0;
     $1634 = HEAP32[$iCur>>2]|0;
     $1635 = ($1633|0)!=($1634|0);
     if (!($1635)) {
      $1636 = HEAP32[$pLevel>>2]|0;
      $1637 = HEAP32[$1636>>2]|0;
      $1638 = ($1637|0)!=(0);
      if (!($1638)) {
       $1639 = HEAP32[$pTerm>>2]|0;
       $1640 = HEAP32[$1639>>2]|0;
       HEAP32[$pE11>>2] = $1640;
       $1641 = HEAP32[$pWC>>2]|0;
       $1642 = HEAP32[$iCur>>2]|0;
       $1643 = HEAP32[$pTerm>>2]|0;
       $1644 = ((($1643)) + 28|0);
       $1645 = HEAP32[$1644>>2]|0;
       $1646 = $5;
       $1647 = $1646;
       $1648 = HEAP32[$1647>>2]|0;
       $1649 = (($1646) + 4)|0;
       $1650 = $1649;
       $1651 = HEAP32[$1650>>2]|0;
       $1652 = (_sqlite3WhereFindTerm($1641,$1642,$1645,$1648,$1651,131,0)|0);
       HEAP32[$pAlt>>2] = $1652;
       $1653 = HEAP32[$pAlt>>2]|0;
       $1654 = ($1653|0)==(0|0);
       if (!($1654)) {
        $1655 = HEAP32[$pAlt>>2]|0;
        $1656 = ((($1655)) + 10|0);
        $1657 = HEAP16[$1656>>1]|0;
        $1658 = $1657&65535;
        $1659 = $1658 & 4;
        $1660 = ($1659|0)!=(0);
        if (!($1660)) {
         $1661 = HEAP32[$pAlt>>2]|0;
         $1662 = HEAP32[$1661>>2]|0;
         dest=$sEAlt; src=$1662; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
         $1663 = HEAP32[$pE11>>2]|0;
         $1664 = ((($1663)) + 12|0);
         $1665 = HEAP32[$1664>>2]|0;
         $1666 = ((($sEAlt)) + 12|0);
         HEAP32[$1666>>2] = $1665;
         $1667 = HEAP32[$pParse>>2]|0;
         $1668 = HEAP32[$addrCont>>2]|0;
         _sqlite3ExprIfFalse($1667,$sEAlt,$1668,16);
        }
       }
      }
     }
    }
   }
  }
  $1669 = HEAP32[$j>>2]|0;
  $1670 = (($1669) + -1)|0;
  HEAP32[$j>>2] = $1670;
  $1671 = HEAP32[$pTerm>>2]|0;
  $1672 = ((($1671)) + 48|0);
  HEAP32[$pTerm>>2] = $1672;
 }
 $1673 = HEAP32[$pLevel>>2]|0;
 $1674 = HEAP32[$1673>>2]|0;
 $1675 = ($1674|0)!=(0);
 L324: do {
  if ($1675) {
   $1676 = HEAP32[$v>>2]|0;
   $1677 = (_sqlite3VdbeCurrentAddr($1676)|0);
   $1678 = HEAP32[$pLevel>>2]|0;
   $1679 = ((($1678)) + 28|0);
   HEAP32[$1679>>2] = $1677;
   $1680 = HEAP32[$v>>2]|0;
   $1681 = HEAP32[$pLevel>>2]|0;
   $1682 = HEAP32[$1681>>2]|0;
   (_sqlite3VdbeAddOp2($1680,76,1,$1682)|0);
   $1683 = HEAP32[$pParse>>2]|0;
   _sqlite3ExprCacheClear($1683);
   $1684 = HEAP32[$pWC>>2]|0;
   $1685 = ((($1684)) + 20|0);
   $1686 = HEAP32[$1685>>2]|0;
   HEAP32[$pTerm>>2] = $1686;
   HEAP32[$j>>2] = 0;
   while(1) {
    $1687 = HEAP32[$j>>2]|0;
    $1688 = HEAP32[$pWC>>2]|0;
    $1689 = ((($1688)) + 12|0);
    $1690 = HEAP32[$1689>>2]|0;
    $1691 = ($1687|0)<($1690|0);
    if (!($1691)) {
     break L324;
    }
    $1692 = HEAP32[$pTerm>>2]|0;
    $1693 = ((($1692)) + 10|0);
    $1694 = HEAP16[$1693>>1]|0;
    $1695 = $1694&65535;
    $1696 = $1695 & 6;
    $1697 = ($1696|0)!=(0);
    if (!($1697)) {
     $1698 = HEAP32[$pTerm>>2]|0;
     $1699 = ((($1698)) + 40|0);
     $1700 = $1699;
     $1701 = $1700;
     $1702 = HEAP32[$1701>>2]|0;
     $1703 = (($1700) + 4)|0;
     $1704 = $1703;
     $1705 = HEAP32[$1704>>2]|0;
     $1706 = HEAP32[$pLevel>>2]|0;
     $1707 = ((($1706)) + 72|0);
     $1708 = $1707;
     $1709 = $1708;
     $1710 = HEAP32[$1709>>2]|0;
     $1711 = (($1708) + 4)|0;
     $1712 = $1711;
     $1713 = HEAP32[$1712>>2]|0;
     $1714 = $1702 & $1710;
     $1715 = $1705 & $1713;
     $1716 = ($1714|0)!=(0);
     $1717 = ($1715|0)!=(0);
     $1718 = $1716 | $1717;
     if (!($1718)) {
      $1719 = HEAP32[$pParse>>2]|0;
      $1720 = HEAP32[$pTerm>>2]|0;
      $1721 = HEAP32[$1720>>2]|0;
      $1722 = HEAP32[$addrCont>>2]|0;
      _sqlite3ExprIfFalse($1719,$1721,$1722,16);
      $1723 = HEAP32[$pTerm>>2]|0;
      $1724 = ((($1723)) + 10|0);
      $1725 = HEAP16[$1724>>1]|0;
      $1726 = $1725&65535;
      $1727 = $1726 | 4;
      $1728 = $1727&65535;
      HEAP16[$1724>>1] = $1728;
     }
    }
    $1729 = HEAP32[$j>>2]|0;
    $1730 = (($1729) + 1)|0;
    HEAP32[$j>>2] = $1730;
    $1731 = HEAP32[$pTerm>>2]|0;
    $1732 = ((($1731)) + 48|0);
    HEAP32[$pTerm>>2] = $1732;
   }
  }
 } while(0);
 $1733 = HEAP32[$pLevel>>2]|0;
 $1734 = ((($1733)) + 72|0);
 $1735 = $1734;
 $1736 = $1735;
 $1737 = HEAP32[$1736>>2]|0;
 $1738 = (($1735) + 4)|0;
 $1739 = $1738;
 $1740 = HEAP32[$1739>>2]|0;
 $1741 = $2;
 $1742 = $1741;
 HEAP32[$1742>>2] = $1737;
 $1743 = (($1741) + 4)|0;
 $1744 = $1743;
 HEAP32[$1744>>2] = $1740;
 $1745 = $2;
 $1746 = $1745;
 $1747 = HEAP32[$1746>>2]|0;
 $1748 = (($1745) + 4)|0;
 $1749 = $1748;
 $1750 = HEAP32[$1749>>2]|0;
 tempRet0 = ($1750);
 STACKTOP = sp;return ($1747|0);
}
function _sqlite3WhereGetMask($pMaskSet,$iCursor) {
 $pMaskSet = $pMaskSet|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $i = sp + 8|0;
 HEAP32[$1>>2] = $pMaskSet;
 HEAP32[$2>>2] = $iCursor;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   label = 6;
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = ($11|0)==($12|0);
  $14 = HEAP32[$i>>2]|0;
  if ($13) {
   label = 4;
   break;
  }
  $21 = (($14) + 1)|0;
  HEAP32[$i>>2] = $21;
 }
 if ((label|0) == 4) {
  $15 = (_bitshift64Shl(1,0,($14|0))|0);
  $16 = tempRet0;
  $17 = $0;
  $18 = $17;
  HEAP32[$18>>2] = $15;
  $19 = (($17) + 4)|0;
  $20 = $19;
  HEAP32[$20>>2] = $16;
  $26 = $0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  tempRet0 = ($31);
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 6) {
  $22 = $0;
  $23 = $22;
  HEAP32[$23>>2] = 0;
  $24 = (($22) + 4)|0;
  $25 = $24;
  HEAP32[$25>>2] = 0;
  $26 = $0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = (($26) + 4)|0;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  tempRet0 = ($31);
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _codeEqualityTerm($pParse,$pTerm,$pLevel,$iEq,$bRev,$iTarget) {
 $pParse = $pParse|0;
 $pTerm = $pTerm|0;
 $pLevel = $pLevel|0;
 $iEq = $iEq|0;
 $bRev = $bRev|0;
 $iTarget = $iTarget|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $aiMap = 0, $db = 0, $eType = 0, $i = 0, $iCol = 0, $iField = 0, $iMap = 0, $iOut = 0, $iReg = 0, $iTab = 0, $nEq = 0, $pIn = 0, $pLeft = 0, $pLhs = 0, $pLoop = 0, $pNewLhs = 0, $pNewRhs = 0, $pOrderBy = 0, $pOrigLhs = 0;
 var $pOrigRhs = 0, $pRhs = 0, $pSelect = 0, $pX = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $4 = sp + 104|0;
 $5 = sp + 100|0;
 $6 = sp + 96|0;
 $pX = sp + 92|0;
 $v = sp + 88|0;
 $iReg = sp + 84|0;
 $eType = sp + 80|0;
 $iTab = sp + 76|0;
 $pIn = sp + 72|0;
 $pLoop = sp + 68|0;
 $i = sp + 64|0;
 $nEq = sp + 60|0;
 $aiMap = sp + 56|0;
 $pSelect = sp + 52|0;
 $db = sp + 48|0;
 $pOrigRhs = sp + 44|0;
 $pOrigLhs = sp + 40|0;
 $pRhs = sp + 36|0;
 $pLhs = sp + 32|0;
 $iField = sp + 28|0;
 $pNewRhs = sp + 24|0;
 $pNewLhs = sp + 20|0;
 $pLeft = sp + 16|0;
 $pOrderBy = sp + 12|0;
 $iMap = sp + 8|0;
 $iOut = sp + 4|0;
 $iCol = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTerm;
 HEAP32[$3>>2] = $pLevel;
 HEAP32[$4>>2] = $iEq;
 HEAP32[$5>>2] = $bRev;
 HEAP32[$6>>2] = $iTarget;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pX>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$pX>>2]|0;
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(37);
 do {
  if ($15) {
   label = 3;
  } else {
   $16 = HEAP32[$pX>>2]|0;
   $17 = HEAP8[$16>>0]|0;
   $18 = $17&255;
   $19 = ($18|0)==(29);
   if ($19) {
    label = 3;
   } else {
    $26 = HEAP32[$pX>>2]|0;
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = ($28|0)==(34);
    if ($29) {
     $30 = HEAP32[$6>>2]|0;
     HEAP32[$iReg>>2] = $30;
     $31 = HEAP32[$v>>2]|0;
     $32 = HEAP32[$iReg>>2]|0;
     (_sqlite3VdbeAddOp2($31,79,0,$32)|0);
     break;
    }
    HEAP32[$eType>>2] = 5;
    $33 = HEAP32[$3>>2]|0;
    $34 = ((($33)) + 64|0);
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$pLoop>>2] = $35;
    HEAP32[$nEq>>2] = 0;
    HEAP32[$aiMap>>2] = 0;
    $36 = HEAP32[$pLoop>>2]|0;
    $37 = ((($36)) + 36|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = $38 & 1024;
    $40 = ($39|0)==(0);
    if ($40) {
     $41 = HEAP32[$pLoop>>2]|0;
     $42 = ((($41)) + 24|0);
     $43 = ((($42)) + 8|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = ($44|0)!=(0|0);
     if ($45) {
      $46 = HEAP32[$4>>2]|0;
      $47 = HEAP32[$pLoop>>2]|0;
      $48 = ((($47)) + 24|0);
      $49 = ((($48)) + 8|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = ((($50)) + 28|0);
      $52 = HEAP32[$51>>2]|0;
      $53 = (($52) + ($46)|0);
      $54 = HEAP8[$53>>0]|0;
      $55 = $54&255;
      $56 = ($55|0)!=(0);
      if ($56) {
       $57 = HEAP32[$5>>2]|0;
       $58 = ($57|0)!=(0);
       $59 = $58 ^ 1;
       $60 = $59&1;
       HEAP32[$5>>2] = $60;
      }
     }
    }
    $61 = HEAP32[$6>>2]|0;
    HEAP32[$iReg>>2] = $61;
    HEAP32[$i>>2] = 0;
    while(1) {
     $62 = HEAP32[$i>>2]|0;
     $63 = HEAP32[$4>>2]|0;
     $64 = ($62|0)<($63|0);
     if (!($64)) {
      break;
     }
     $65 = HEAP32[$i>>2]|0;
     $66 = HEAP32[$pLoop>>2]|0;
     $67 = ((($66)) + 48|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = (($68) + ($65<<2)|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($70|0)!=(0|0);
     if ($71) {
      $72 = HEAP32[$i>>2]|0;
      $73 = HEAP32[$pLoop>>2]|0;
      $74 = ((($73)) + 48|0);
      $75 = HEAP32[$74>>2]|0;
      $76 = (($75) + ($72<<2)|0);
      $77 = HEAP32[$76>>2]|0;
      $78 = HEAP32[$77>>2]|0;
      $79 = HEAP32[$pX>>2]|0;
      $80 = ($78|0)==($79|0);
      if ($80) {
       label = 14;
       break;
      }
     }
     $84 = HEAP32[$i>>2]|0;
     $85 = (($84) + 1)|0;
     HEAP32[$i>>2] = $85;
    }
    if ((label|0) == 14) {
     $81 = HEAP32[$3>>2]|0;
     $82 = HEAP32[$2>>2]|0;
     _disableTerm($81,$82);
     $83 = HEAP32[$6>>2]|0;
     HEAP32[$0>>2] = $83;
     $380 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($380|0);
    }
    $86 = HEAP32[$4>>2]|0;
    HEAP32[$i>>2] = $86;
    while(1) {
     $87 = HEAP32[$i>>2]|0;
     $88 = HEAP32[$pLoop>>2]|0;
     $89 = ((($88)) + 40|0);
     $90 = HEAP16[$89>>1]|0;
     $91 = $90&65535;
     $92 = ($87|0)<($91|0);
     if (!($92)) {
      break;
     }
     $93 = HEAP32[$i>>2]|0;
     $94 = HEAP32[$pLoop>>2]|0;
     $95 = ((($94)) + 48|0);
     $96 = HEAP32[$95>>2]|0;
     $97 = (($96) + ($93<<2)|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = ($98|0)!=(0|0);
     if ($99) {
      $100 = HEAP32[$i>>2]|0;
      $101 = HEAP32[$pLoop>>2]|0;
      $102 = ((($101)) + 48|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = (($103) + ($100<<2)|0);
      $105 = HEAP32[$104>>2]|0;
      $106 = HEAP32[$105>>2]|0;
      $107 = HEAP32[$pX>>2]|0;
      $108 = ($106|0)==($107|0);
      if ($108) {
       $109 = HEAP32[$nEq>>2]|0;
       $110 = (($109) + 1)|0;
       HEAP32[$nEq>>2] = $110;
      }
     }
     $111 = HEAP32[$i>>2]|0;
     $112 = (($111) + 1)|0;
     HEAP32[$i>>2] = $112;
    }
    $113 = HEAP32[$pX>>2]|0;
    $114 = ((($113)) + 4|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = $115 & 2048;
    $117 = ($116|0)==(0);
    if ($117) {
     label = 24;
    } else {
     $118 = HEAP32[$pX>>2]|0;
     $119 = ((($118)) + 20|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = HEAP32[$120>>2]|0;
     $122 = HEAP32[$121>>2]|0;
     $123 = ($122|0)==(1);
     if ($123) {
      label = 24;
     } else {
      $127 = HEAP32[$pX>>2]|0;
      $128 = ((($127)) + 20|0);
      $129 = HEAP32[$128>>2]|0;
      HEAP32[$pSelect>>2] = $129;
      $130 = HEAP32[$1>>2]|0;
      $131 = HEAP32[$130>>2]|0;
      HEAP32[$db>>2] = $131;
      $132 = HEAP32[$pSelect>>2]|0;
      $133 = HEAP32[$132>>2]|0;
      HEAP32[$pOrigRhs>>2] = $133;
      $134 = HEAP32[$pX>>2]|0;
      $135 = ((($134)) + 12|0);
      $136 = HEAP32[$135>>2]|0;
      $137 = ((($136)) + 20|0);
      $138 = HEAP32[$137>>2]|0;
      HEAP32[$pOrigLhs>>2] = $138;
      HEAP32[$pRhs>>2] = 0;
      HEAP32[$pLhs>>2] = 0;
      $139 = HEAP32[$4>>2]|0;
      HEAP32[$i>>2] = $139;
      while(1) {
       $140 = HEAP32[$i>>2]|0;
       $141 = HEAP32[$pLoop>>2]|0;
       $142 = ((($141)) + 40|0);
       $143 = HEAP16[$142>>1]|0;
       $144 = $143&65535;
       $145 = ($140|0)<($144|0);
       if (!($145)) {
        break;
       }
       $146 = HEAP32[$i>>2]|0;
       $147 = HEAP32[$pLoop>>2]|0;
       $148 = ((($147)) + 48|0);
       $149 = HEAP32[$148>>2]|0;
       $150 = (($149) + ($146<<2)|0);
       $151 = HEAP32[$150>>2]|0;
       $152 = HEAP32[$151>>2]|0;
       $153 = HEAP32[$pX>>2]|0;
       $154 = ($152|0)==($153|0);
       if ($154) {
        $155 = HEAP32[$i>>2]|0;
        $156 = HEAP32[$pLoop>>2]|0;
        $157 = ((($156)) + 48|0);
        $158 = HEAP32[$157>>2]|0;
        $159 = (($158) + ($155<<2)|0);
        $160 = HEAP32[$159>>2]|0;
        $161 = ((($160)) + 24|0);
        $162 = HEAP32[$161>>2]|0;
        $163 = (($162) - 1)|0;
        HEAP32[$iField>>2] = $163;
        $164 = HEAP32[$db>>2]|0;
        $165 = HEAP32[$iField>>2]|0;
        $166 = HEAP32[$pOrigRhs>>2]|0;
        $167 = ((($166)) + 4|0);
        $168 = HEAP32[$167>>2]|0;
        $169 = (($168) + (($165*20)|0)|0);
        $170 = HEAP32[$169>>2]|0;
        $171 = (_sqlite3ExprDup($164,$170,0)|0);
        HEAP32[$pNewRhs>>2] = $171;
        $172 = HEAP32[$db>>2]|0;
        $173 = HEAP32[$iField>>2]|0;
        $174 = HEAP32[$pOrigLhs>>2]|0;
        $175 = ((($174)) + 4|0);
        $176 = HEAP32[$175>>2]|0;
        $177 = (($176) + (($173*20)|0)|0);
        $178 = HEAP32[$177>>2]|0;
        $179 = (_sqlite3ExprDup($172,$178,0)|0);
        HEAP32[$pNewLhs>>2] = $179;
        $180 = HEAP32[$1>>2]|0;
        $181 = HEAP32[$pRhs>>2]|0;
        $182 = HEAP32[$pNewRhs>>2]|0;
        $183 = (_sqlite3ExprListAppend($180,$181,$182)|0);
        HEAP32[$pRhs>>2] = $183;
        $184 = HEAP32[$1>>2]|0;
        $185 = HEAP32[$pLhs>>2]|0;
        $186 = HEAP32[$pNewLhs>>2]|0;
        $187 = (_sqlite3ExprListAppend($184,$185,$186)|0);
        HEAP32[$pLhs>>2] = $187;
       }
       $188 = HEAP32[$i>>2]|0;
       $189 = (($188) + 1)|0;
       HEAP32[$i>>2] = $189;
      }
      $190 = HEAP32[$db>>2]|0;
      $191 = ((($190)) + 69|0);
      $192 = HEAP8[$191>>0]|0;
      $193 = ($192<<24>>24)!=(0);
      if (!($193)) {
       $194 = HEAP32[$pX>>2]|0;
       $195 = ((($194)) + 12|0);
       $196 = HEAP32[$195>>2]|0;
       HEAP32[$pLeft>>2] = $196;
       $197 = HEAP32[$pSelect>>2]|0;
       $198 = ((($197)) + 44|0);
       $199 = HEAP32[$198>>2]|0;
       $200 = ($199|0)!=(0|0);
       L42: do {
        if ($200) {
         $201 = HEAP32[$pSelect>>2]|0;
         $202 = ((($201)) + 44|0);
         $203 = HEAP32[$202>>2]|0;
         HEAP32[$pOrderBy>>2] = $203;
         HEAP32[$i>>2] = 0;
         while(1) {
          $204 = HEAP32[$i>>2]|0;
          $205 = HEAP32[$pOrderBy>>2]|0;
          $206 = HEAP32[$205>>2]|0;
          $207 = ($204|0)<($206|0);
          if (!($207)) {
           break L42;
          }
          $208 = HEAP32[$i>>2]|0;
          $209 = HEAP32[$pOrderBy>>2]|0;
          $210 = ((($209)) + 4|0);
          $211 = HEAP32[$210>>2]|0;
          $212 = (($211) + (($208*20)|0)|0);
          $213 = ((($212)) + 16|0);
          HEAP16[$213>>1] = 0;
          $214 = HEAP32[$i>>2]|0;
          $215 = (($214) + 1)|0;
          HEAP32[$i>>2] = $215;
         }
        }
       } while(0);
       $216 = HEAP32[$pLhs>>2]|0;
       $217 = HEAP32[$216>>2]|0;
       $218 = ($217|0)==(1);
       $219 = HEAP32[$pLhs>>2]|0;
       if ($218) {
        $220 = ((($219)) + 4|0);
        $221 = HEAP32[$220>>2]|0;
        $222 = HEAP32[$221>>2]|0;
        $223 = HEAP32[$pX>>2]|0;
        $224 = ((($223)) + 12|0);
        HEAP32[$224>>2] = $222;
       } else {
        $225 = HEAP32[$pLeft>>2]|0;
        $226 = ((($225)) + 20|0);
        HEAP32[$226>>2] = $219;
        $227 = HEAP32[$1>>2]|0;
        $228 = HEAP32[$227>>2]|0;
        $229 = HEAP32[$nEq>>2]|0;
        $230 = $229<<2;
        $231 = (_sqlite3DbMallocZero($228,$230,0)|0);
        HEAP32[$aiMap>>2] = $231;
       }
       $232 = HEAP32[$pRhs>>2]|0;
       $233 = HEAP32[$pSelect>>2]|0;
       HEAP32[$233>>2] = $232;
       $234 = HEAP32[$1>>2]|0;
       $235 = HEAP32[$pX>>2]|0;
       $236 = HEAP32[$aiMap>>2]|0;
       $237 = (_sqlite3FindInIndex($234,$235,4,0,$236)|0);
       HEAP32[$eType>>2] = $237;
       $238 = HEAP32[$pOrigRhs>>2]|0;
       $239 = HEAP32[$pSelect>>2]|0;
       HEAP32[$239>>2] = $238;
       $240 = HEAP32[$pOrigLhs>>2]|0;
       $241 = HEAP32[$pLeft>>2]|0;
       $242 = ((($241)) + 20|0);
       HEAP32[$242>>2] = $240;
       $243 = HEAP32[$pLeft>>2]|0;
       $244 = HEAP32[$pX>>2]|0;
       $245 = ((($244)) + 12|0);
       HEAP32[$245>>2] = $243;
      }
      $246 = HEAP32[$1>>2]|0;
      $247 = HEAP32[$246>>2]|0;
      $248 = HEAP32[$pLhs>>2]|0;
      _sqlite3ExprListDelete($247,$248);
      $249 = HEAP32[$1>>2]|0;
      $250 = HEAP32[$249>>2]|0;
      $251 = HEAP32[$pRhs>>2]|0;
      _sqlite3ExprListDelete($250,$251);
     }
    }
    if ((label|0) == 24) {
     $124 = HEAP32[$1>>2]|0;
     $125 = HEAP32[$pX>>2]|0;
     $126 = (_sqlite3FindInIndex($124,$125,4,0,0)|0);
     HEAP32[$eType>>2] = $126;
    }
    $252 = HEAP32[$eType>>2]|0;
    $253 = ($252|0)==(4);
    if ($253) {
     $254 = HEAP32[$5>>2]|0;
     $255 = ($254|0)!=(0);
     $256 = $255 ^ 1;
     $257 = $256&1;
     HEAP32[$5>>2] = $257;
    }
    $258 = HEAP32[$pX>>2]|0;
    $259 = ((($258)) + 28|0);
    $260 = HEAP32[$259>>2]|0;
    HEAP32[$iTab>>2] = $260;
    $261 = HEAP32[$v>>2]|0;
    $262 = HEAP32[$5>>2]|0;
    $263 = ($262|0)!=(0);
    $264 = $263 ? 53 : 57;
    $265 = HEAP32[$iTab>>2]|0;
    (_sqlite3VdbeAddOp2($261,$264,$265,0)|0);
    $266 = HEAP32[$pLoop>>2]|0;
    $267 = ((($266)) + 36|0);
    $268 = HEAP32[$267>>2]|0;
    $269 = $268 | 2048;
    HEAP32[$267>>2] = $269;
    $270 = HEAP32[$3>>2]|0;
    $271 = ((($270)) + 56|0);
    $272 = HEAP32[$271>>2]|0;
    $273 = ($272|0)==(0);
    if ($273) {
     $274 = HEAP32[$v>>2]|0;
     $275 = (_sqlite3VdbeMakeLabel($274)|0);
     $276 = HEAP32[$3>>2]|0;
     $277 = ((($276)) + 16|0);
     HEAP32[$277>>2] = $275;
    }
    $278 = HEAP32[$3>>2]|0;
    $279 = ((($278)) + 56|0);
    $280 = HEAP32[$279>>2]|0;
    HEAP32[$i>>2] = $280;
    $281 = HEAP32[$nEq>>2]|0;
    $282 = HEAP32[$3>>2]|0;
    $283 = ((($282)) + 56|0);
    $284 = HEAP32[$283>>2]|0;
    $285 = (($284) + ($281))|0;
    HEAP32[$283>>2] = $285;
    $286 = HEAP32[$1>>2]|0;
    $287 = HEAP32[$286>>2]|0;
    $288 = HEAP32[$3>>2]|0;
    $289 = ((($288)) + 56|0);
    $290 = ((($289)) + 4|0);
    $291 = HEAP32[$290>>2]|0;
    $292 = HEAP32[$3>>2]|0;
    $293 = ((($292)) + 56|0);
    $294 = HEAP32[$293>>2]|0;
    $295 = ($294*12)|0;
    $296 = (_sqlite3DbReallocOrFree($287,$291,$295,0)|0);
    $297 = HEAP32[$3>>2]|0;
    $298 = ((($297)) + 56|0);
    $299 = ((($298)) + 4|0);
    HEAP32[$299>>2] = $296;
    $300 = HEAP32[$3>>2]|0;
    $301 = ((($300)) + 56|0);
    $302 = ((($301)) + 4|0);
    $303 = HEAP32[$302>>2]|0;
    HEAP32[$pIn>>2] = $303;
    $304 = HEAP32[$pIn>>2]|0;
    $305 = ($304|0)!=(0|0);
    L62: do {
     if ($305) {
      HEAP32[$iMap>>2] = 0;
      $306 = HEAP32[$i>>2]|0;
      $307 = HEAP32[$pIn>>2]|0;
      $308 = (($307) + (($306*12)|0)|0);
      HEAP32[$pIn>>2] = $308;
      $309 = HEAP32[$4>>2]|0;
      HEAP32[$i>>2] = $309;
      while(1) {
       $310 = HEAP32[$i>>2]|0;
       $311 = HEAP32[$pLoop>>2]|0;
       $312 = ((($311)) + 40|0);
       $313 = HEAP16[$312>>1]|0;
       $314 = $313&65535;
       $315 = ($310|0)<($314|0);
       if (!($315)) {
        break L62;
       }
       $316 = HEAP32[$i>>2]|0;
       $317 = HEAP32[$pLoop>>2]|0;
       $318 = ((($317)) + 48|0);
       $319 = HEAP32[$318>>2]|0;
       $320 = (($319) + ($316<<2)|0);
       $321 = HEAP32[$320>>2]|0;
       $322 = HEAP32[$321>>2]|0;
       $323 = HEAP32[$pX>>2]|0;
       $324 = ($322|0)==($323|0);
       if ($324) {
        $325 = HEAP32[$iReg>>2]|0;
        $326 = HEAP32[$i>>2]|0;
        $327 = (($325) + ($326))|0;
        $328 = HEAP32[$4>>2]|0;
        $329 = (($327) - ($328))|0;
        HEAP32[$iOut>>2] = $329;
        $330 = HEAP32[$eType>>2]|0;
        $331 = ($330|0)==(1);
        if ($331) {
         $332 = HEAP32[$v>>2]|0;
         $333 = HEAP32[$iTab>>2]|0;
         $334 = HEAP32[$iOut>>2]|0;
         $335 = (_sqlite3VdbeAddOp2($332,123,$333,$334)|0);
         $336 = HEAP32[$pIn>>2]|0;
         $337 = ((($336)) + 4|0);
         HEAP32[$337>>2] = $335;
        } else {
         $338 = HEAP32[$aiMap>>2]|0;
         $339 = ($338|0)!=(0|0);
         if ($339) {
          $340 = HEAP32[$iMap>>2]|0;
          $341 = (($340) + 1)|0;
          HEAP32[$iMap>>2] = $341;
          $342 = HEAP32[$aiMap>>2]|0;
          $343 = (($342) + ($340<<2)|0);
          $344 = HEAP32[$343>>2]|0;
          $345 = $344;
         } else {
          $345 = 0;
         }
         HEAP32[$iCol>>2] = $345;
         $346 = HEAP32[$v>>2]|0;
         $347 = HEAP32[$iTab>>2]|0;
         $348 = HEAP32[$iCol>>2]|0;
         $349 = HEAP32[$iOut>>2]|0;
         $350 = (_sqlite3VdbeAddOp3($346,96,$347,$348,$349)|0);
         $351 = HEAP32[$pIn>>2]|0;
         $352 = ((($351)) + 4|0);
         HEAP32[$352>>2] = $350;
        }
        $353 = HEAP32[$v>>2]|0;
        $354 = HEAP32[$iOut>>2]|0;
        (_sqlite3VdbeAddOp1($353,34,$354)|0);
        $355 = HEAP32[$i>>2]|0;
        $356 = HEAP32[$4>>2]|0;
        $357 = ($355|0)==($356|0);
        if ($357) {
         $358 = HEAP32[$iTab>>2]|0;
         $359 = HEAP32[$pIn>>2]|0;
         HEAP32[$359>>2] = $358;
         $360 = HEAP32[$5>>2]|0;
         $361 = ($360|0)!=(0);
         $362 = $361 ? 4 : 5;
         $363 = $362&255;
         $364 = HEAP32[$pIn>>2]|0;
         $365 = ((($364)) + 8|0);
         HEAP8[$365>>0] = $363;
        } else {
         $366 = HEAP32[$pIn>>2]|0;
         $367 = ((($366)) + 8|0);
         HEAP8[$367>>0] = -95;
        }
        $368 = HEAP32[$pIn>>2]|0;
        $369 = ((($368)) + 12|0);
        HEAP32[$pIn>>2] = $369;
       }
       $370 = HEAP32[$i>>2]|0;
       $371 = (($370) + 1)|0;
       HEAP32[$i>>2] = $371;
      }
     } else {
      $372 = HEAP32[$3>>2]|0;
      $373 = ((($372)) + 56|0);
      HEAP32[$373>>2] = 0;
     }
    } while(0);
    $374 = HEAP32[$1>>2]|0;
    $375 = HEAP32[$374>>2]|0;
    $376 = HEAP32[$aiMap>>2]|0;
    _sqlite3DbFree($375,$376);
   }
  }
 } while(0);
 if ((label|0) == 3) {
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$pX>>2]|0;
  $22 = ((($21)) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$6>>2]|0;
  $25 = (_sqlite3ExprCodeTarget($20,$23,$24)|0);
  HEAP32[$iReg>>2] = $25;
 }
 $377 = HEAP32[$3>>2]|0;
 $378 = HEAP32[$2>>2]|0;
 _disableTerm($377,$378);
 $379 = HEAP32[$iReg>>2]|0;
 HEAP32[$0>>2] = $379;
 $380 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($380|0);
}
function _codeExprOrVector($pParse,$p,$iReg,$nReg) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iReg = $iReg|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iSelect = 0, $pList = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $v = sp + 12|0;
 $iSelect = sp + 8|0;
 $i = sp + 4|0;
 $pList = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iReg;
 HEAP32[$3>>2] = $nReg;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3ExprIsVector($4)|0);
 $6 = ($5|0)!=(0);
 if (!($6)) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  _sqlite3ExprCode($41,$42,$43);
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 2048;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$v>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$1>>2]|0;
  $17 = (_sqlite3CodeSubselect($15,$16,0,0)|0);
  HEAP32[$iSelect>>2] = $17;
  $18 = HEAP32[$v>>2]|0;
  $19 = HEAP32[$iSelect>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = HEAP32[$3>>2]|0;
  $22 = (($21) - 1)|0;
  (_sqlite3VdbeAddOp3($18,84,$19,$20,$22)|0);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pList>>2] = $25;
 HEAP32[$i>>2] = 0;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = ($26|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$pList>>2]|0;
  $32 = ((($31)) + 4|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + (($30*20)|0)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = HEAP32[$i>>2]|0;
  $38 = (($36) + ($37))|0;
  _sqlite3ExprCode($29,$35,$38);
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 1)|0;
  HEAP32[$i>>2] = $40;
 }
 STACKTOP = sp;return;
}
function _disableTerm($pLevel,$pTerm) {
 $pLevel = $pLevel|0;
 $pTerm = $pTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $nLoop = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $nLoop = sp;
 HEAP32[$0>>2] = $pLevel;
 HEAP32[$1>>2] = $pTerm;
 HEAP32[$nLoop>>2] = 0;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ($2|0)!=(0|0);
  if (!($3)) {
   label = 14;
   break;
  }
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 10|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6&65535;
  $8 = $7 & 4;
  $9 = ($8|0)==(0);
  if (!($9)) {
   label = 14;
   break;
  }
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)==(0);
  if (!($12)) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = ((($14)) + 4|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $16 & 1;
   $18 = ($17|0)!=(0);
   if (!($18)) {
    label = 14;
    break;
   }
  }
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 72|0);
  $21 = $20;
  $22 = $21;
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + 4)|0;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 40|0);
  $29 = $28;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = $23 & $31;
  $36 = $26 & $34;
  $37 = ($35|0)==(0);
  $38 = ($36|0)==(0);
  $39 = $37 & $38;
  if (!($39)) {
   label = 14;
   break;
  }
  $40 = HEAP32[$nLoop>>2]|0;
  $41 = ($40|0)!=(0);
  if ($41) {
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 10|0);
   $44 = HEAP16[$43>>1]|0;
   $45 = $44&65535;
   $46 = $45 & 1024;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$1>>2]|0;
    $49 = ((($48)) + 10|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = $50&65535;
    $52 = $51 | 512;
    $53 = $52&65535;
    HEAP16[$49>>1] = $53;
   } else {
    label = 10;
   }
  } else {
   label = 10;
  }
  if ((label|0) == 10) {
   label = 0;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 10|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   $58 = $57 | 4;
   $59 = $58&65535;
   HEAP16[$55>>1] = $59;
  }
  $60 = HEAP32[$1>>2]|0;
  $61 = ((($60)) + 16|0);
  $62 = HEAP32[$61>>2]|0;
  $63 = ($62|0)<(0);
  if ($63) {
   label = 14;
   break;
  }
  $64 = HEAP32[$1>>2]|0;
  $65 = ((($64)) + 16|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = HEAP32[$1>>2]|0;
  $68 = ((($67)) + 4|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = ((($69)) + 20|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + (($66*48)|0)|0);
  HEAP32[$1>>2] = $72;
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 14|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = (($75) + -1)<<24>>24;
  HEAP8[$74>>0] = $76;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 14|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = ($80|0)!=(0);
  if ($81) {
   label = 14;
   break;
  }
  $82 = HEAP32[$nLoop>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$nLoop>>2] = $83;
 }
 if ((label|0) == 14) {
  STACKTOP = sp;return;
 }
}
function _codeAllEqualityTerms($pParse,$pLevel,$bRev,$nExtraReg,$pzAff) {
 $pParse = $pParse|0;
 $pLevel = $pLevel|0;
 $bRev = $bRev|0;
 $nExtraReg = $nExtraReg|0;
 $pzAff = $pzAff|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iIdxCur = 0, $j = 0, $nEq = 0, $nReg = 0, $nSkip = 0, $or$cond = 0, $pIdx = 0, $pLoop = 0, $pRight = 0, $pTerm = 0, $r1 = 0, $regBase = 0, $v = 0, $zAff = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $3 = sp + 48|0;
 $4 = sp + 44|0;
 $nEq = sp + 66|0;
 $nSkip = sp + 64|0;
 $v = sp + 40|0;
 $pIdx = sp + 36|0;
 $pTerm = sp + 32|0;
 $pLoop = sp + 28|0;
 $j = sp + 24|0;
 $regBase = sp + 20|0;
 $nReg = sp + 16|0;
 $zAff = sp + 12|0;
 $iIdxCur = sp + 8|0;
 $r1 = sp + 4|0;
 $pRight = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pLevel;
 HEAP32[$2>>2] = $bRev;
 HEAP32[$3>>2] = $nExtraReg;
 HEAP32[$4>>2] = $pzAff;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 64|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pLoop>>2] = $10;
 $11 = HEAP32[$pLoop>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP16[$12>>1]|0;
 HEAP16[$nEq>>1] = $13;
 $14 = HEAP32[$pLoop>>2]|0;
 $15 = ((($14)) + 42|0);
 $16 = HEAP16[$15>>1]|0;
 HEAP16[$nSkip>>1] = $16;
 $17 = HEAP32[$pLoop>>2]|0;
 $18 = ((($17)) + 24|0);
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pIdx>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 44|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = (($23) + 1)|0;
 HEAP32[$regBase>>2] = $24;
 $25 = HEAP32[$pLoop>>2]|0;
 $26 = ((($25)) + 24|0);
 $27 = HEAP16[$26>>1]|0;
 $28 = $27&65535;
 $29 = HEAP32[$3>>2]|0;
 $30 = (($28) + ($29))|0;
 HEAP32[$nReg>>2] = $30;
 $31 = HEAP32[$nReg>>2]|0;
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 44|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31))|0;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$pIdx>>2]|0;
 $41 = (_sqlite3IndexAffinityStr($39,$40)|0);
 $42 = (_sqlite3DbStrDup($37,$41)|0);
 HEAP32[$zAff>>2] = $42;
 $43 = HEAP16[$nSkip>>1]|0;
 $44 = ($43<<16>>16)!=(0);
 L1: do {
  if ($44) {
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 8|0);
   $47 = HEAP32[$46>>2]|0;
   HEAP32[$iIdxCur>>2] = $47;
   $48 = HEAP32[$v>>2]|0;
   $49 = HEAP32[$2>>2]|0;
   $50 = ($49|0)!=(0);
   $51 = $50 ? 53 : 57;
   $52 = HEAP32[$iIdxCur>>2]|0;
   (_sqlite3VdbeAddOp1($48,$51,$52)|0);
   $53 = HEAP32[$v>>2]|0;
   $54 = (_sqlite3VdbeAddOp0($53,13)|0);
   HEAP32[$j>>2] = $54;
   $55 = HEAP32[$v>>2]|0;
   $56 = HEAP32[$2>>2]|0;
   $57 = ($56|0)!=(0);
   $58 = $57 ? 23 : 26;
   $59 = HEAP32[$iIdxCur>>2]|0;
   $60 = HEAP32[$regBase>>2]|0;
   $61 = HEAP16[$nSkip>>1]|0;
   $62 = $61&65535;
   $63 = (_sqlite3VdbeAddOp4Int($55,$58,$59,0,$60,$62)|0);
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 20|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP32[$v>>2]|0;
   $67 = HEAP32[$j>>2]|0;
   _sqlite3VdbeJumpHere($66,$67);
   HEAP32[$j>>2] = 0;
   while(1) {
    $68 = HEAP32[$j>>2]|0;
    $69 = HEAP16[$nSkip>>1]|0;
    $70 = $69&65535;
    $71 = ($68|0)<($70|0);
    if (!($71)) {
     break L1;
    }
    $72 = HEAP32[$v>>2]|0;
    $73 = HEAP32[$iIdxCur>>2]|0;
    $74 = HEAP32[$j>>2]|0;
    $75 = HEAP32[$regBase>>2]|0;
    $76 = HEAP32[$j>>2]|0;
    $77 = (($75) + ($76))|0;
    (_sqlite3VdbeAddOp3($72,96,$73,$74,$77)|0);
    $78 = HEAP32[$j>>2]|0;
    $79 = (($78) + 1)|0;
    HEAP32[$j>>2] = $79;
   }
  }
 } while(0);
 $80 = HEAP16[$nSkip>>1]|0;
 $81 = $80&65535;
 HEAP32[$j>>2] = $81;
 while(1) {
  $82 = HEAP32[$j>>2]|0;
  $83 = HEAP16[$nEq>>1]|0;
  $84 = $83&65535;
  $85 = ($82|0)<($84|0);
  if (!($85)) {
   break;
  }
  $86 = HEAP32[$j>>2]|0;
  $87 = HEAP32[$pLoop>>2]|0;
  $88 = ((($87)) + 48|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = (($89) + ($86<<2)|0);
  $91 = HEAP32[$90>>2]|0;
  HEAP32[$pTerm>>2] = $91;
  $92 = HEAP32[$0>>2]|0;
  $93 = HEAP32[$pTerm>>2]|0;
  $94 = HEAP32[$1>>2]|0;
  $95 = HEAP32[$j>>2]|0;
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$regBase>>2]|0;
  $98 = HEAP32[$j>>2]|0;
  $99 = (($97) + ($98))|0;
  $100 = (_codeEqualityTerm($92,$93,$94,$95,$96,$99)|0);
  HEAP32[$r1>>2] = $100;
  $101 = HEAP32[$r1>>2]|0;
  $102 = HEAP32[$regBase>>2]|0;
  $103 = HEAP32[$j>>2]|0;
  $104 = (($102) + ($103))|0;
  $105 = ($101|0)!=($104|0);
  do {
   if ($105) {
    $106 = HEAP32[$nReg>>2]|0;
    $107 = ($106|0)==(1);
    if ($107) {
     $108 = HEAP32[$0>>2]|0;
     $109 = HEAP32[$regBase>>2]|0;
     _sqlite3ReleaseTempReg($108,$109);
     $110 = HEAP32[$r1>>2]|0;
     HEAP32[$regBase>>2] = $110;
     break;
    } else {
     $111 = HEAP32[$v>>2]|0;
     $112 = HEAP32[$r1>>2]|0;
     $113 = HEAP32[$regBase>>2]|0;
     $114 = HEAP32[$j>>2]|0;
     $115 = (($113) + ($114))|0;
     (_sqlite3VdbeAddOp2($111,85,$112,$115)|0);
     break;
    }
   }
  } while(0);
  $116 = HEAP32[$pTerm>>2]|0;
  $117 = ((($116)) + 12|0);
  $118 = HEAP16[$117>>1]|0;
  $119 = $118&65535;
  $120 = $119 & 1;
  $121 = ($120|0)!=(0);
  $122 = HEAP32[$pTerm>>2]|0;
  if ($121) {
   $123 = HEAP32[$122>>2]|0;
   $124 = ((($123)) + 4|0);
   $125 = HEAP32[$124>>2]|0;
   $126 = $125 & 2048;
   $127 = ($126|0)!=(0);
   $128 = HEAP32[$zAff>>2]|0;
   $129 = ($128|0)!=(0|0);
   $or$cond = $127 & $129;
   if ($or$cond) {
    $130 = HEAP32[$j>>2]|0;
    $131 = HEAP32[$zAff>>2]|0;
    $132 = (($131) + ($130)|0);
    HEAP8[$132>>0] = 65;
   }
  } else {
   $133 = ((($122)) + 12|0);
   $134 = HEAP16[$133>>1]|0;
   $135 = $134&65535;
   $136 = $135 & 256;
   $137 = ($136|0)==(0);
   if ($137) {
    $138 = HEAP32[$pTerm>>2]|0;
    $139 = HEAP32[$138>>2]|0;
    $140 = ((($139)) + 16|0);
    $141 = HEAP32[$140>>2]|0;
    HEAP32[$pRight>>2] = $141;
    $142 = HEAP32[$pTerm>>2]|0;
    $143 = ((($142)) + 10|0);
    $144 = HEAP16[$143>>1]|0;
    $145 = $144&65535;
    $146 = $145 & 2048;
    $147 = ($146|0)==(0);
    if ($147) {
     $148 = HEAP32[$pRight>>2]|0;
     $149 = (_sqlite3ExprCanBeNull($148)|0);
     $150 = ($149|0)!=(0);
     if ($150) {
      $151 = HEAP32[$v>>2]|0;
      $152 = HEAP32[$regBase>>2]|0;
      $153 = HEAP32[$j>>2]|0;
      $154 = (($152) + ($153))|0;
      $155 = HEAP32[$1>>2]|0;
      $156 = ((($155)) + 12|0);
      $157 = HEAP32[$156>>2]|0;
      (_sqlite3VdbeAddOp2($151,34,$154,$157)|0);
     }
    }
    $158 = HEAP32[$zAff>>2]|0;
    $159 = ($158|0)!=(0|0);
    if ($159) {
     $160 = HEAP32[$pRight>>2]|0;
     $161 = HEAP32[$j>>2]|0;
     $162 = HEAP32[$zAff>>2]|0;
     $163 = (($162) + ($161)|0);
     $164 = HEAP8[$163>>0]|0;
     $165 = (_sqlite3CompareAffinity($160,$164)|0);
     $166 = $165 << 24 >> 24;
     $167 = ($166|0)==(65);
     if ($167) {
      $168 = HEAP32[$j>>2]|0;
      $169 = HEAP32[$zAff>>2]|0;
      $170 = (($169) + ($168)|0);
      HEAP8[$170>>0] = 65;
     }
     $171 = HEAP32[$pRight>>2]|0;
     $172 = HEAP32[$j>>2]|0;
     $173 = HEAP32[$zAff>>2]|0;
     $174 = (($173) + ($172)|0);
     $175 = HEAP8[$174>>0]|0;
     $176 = (_sqlite3ExprNeedsNoAffinityChange($171,$175)|0);
     $177 = ($176|0)!=(0);
     if ($177) {
      $178 = HEAP32[$j>>2]|0;
      $179 = HEAP32[$zAff>>2]|0;
      $180 = (($179) + ($178)|0);
      HEAP8[$180>>0] = 65;
     }
    }
   }
  }
  $181 = HEAP32[$j>>2]|0;
  $182 = (($181) + 1)|0;
  HEAP32[$j>>2] = $182;
 }
 $183 = HEAP32[$zAff>>2]|0;
 $184 = HEAP32[$4>>2]|0;
 HEAP32[$184>>2] = $183;
 $185 = HEAP32[$regBase>>2]|0;
 STACKTOP = sp;return ($185|0);
}
function _whereLikeOptimizationStringFixup($v,$pLevel,$pTerm) {
 $v = $v|0;
 $pLevel = $pLevel|0;
 $pTerm = $pTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $pLevel;
 HEAP32[$2>>2] = $pTerm;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 10|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 256;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_sqlite3VdbeGetOp($9,-1)|0);
 HEAP32[$pOp>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 36|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 >>> 1;
 $15 = HEAP32[$pOp>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 36|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = $19 & 1;
 $21 = $20&255;
 $22 = HEAP32[$pOp>>2]|0;
 $23 = ((($22)) + 3|0);
 HEAP8[$23>>0] = $21;
 STACKTOP = sp;return;
}
function _updateRangeAffinityStr($pRight,$n,$zAff) {
 $pRight = $pRight|0;
 $n = $n|0;
 $zAff = $zAff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pRight;
 HEAP32[$1>>2] = $n;
 HEAP32[$2>>2] = $zAff;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$1>>2]|0;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$i>>2]|0;
  $8 = (_sqlite3VectorFieldSubexpr($6,$7)|0);
  HEAP32[$p>>2] = $8;
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (($11) + ($10)|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = (_sqlite3CompareAffinity($9,$13)|0);
  $15 = $14 << 24 >> 24;
  $16 = ($15|0)==(65);
  if ($16) {
   label = 5;
  } else {
   $17 = HEAP32[$p>>2]|0;
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = (($19) + ($18)|0);
   $21 = HEAP8[$20>>0]|0;
   $22 = (_sqlite3ExprNeedsNoAffinityChange($17,$21)|0);
   $23 = ($22|0)!=(0);
   if ($23) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $24 = HEAP32[$i>>2]|0;
   $25 = HEAP32[$2>>2]|0;
   $26 = (($25) + ($24)|0);
   HEAP8[$26>>0] = 65;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _codeApplyAffinity($pParse,$base,$n,$zAff) {
 $pParse = $pParse|0;
 $base = $base|0;
 $n = $n|0;
 $zAff = $zAff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $base;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $zAff;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  STACKTOP = sp;return;
 }
 while(1) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ($9|0)>(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$3>>2]|0;
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = ($13|0)==(65);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$2>>2]|0;
  $16 = (($15) + -1)|0;
  HEAP32[$2>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = (($17) + 1)|0;
  HEAP32[$1>>2] = $18;
  $19 = HEAP32[$3>>2]|0;
  $20 = ((($19)) + 1|0);
  HEAP32[$3>>2] = $20;
 }
 while(1) {
  $21 = HEAP32[$2>>2]|0;
  $22 = ($21|0)>(1);
  if ($22) {
   $23 = HEAP32[$2>>2]|0;
   $24 = (($23) - 1)|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = (($25) + ($24)|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27 << 24 >> 24;
   $29 = ($28|0)==(65);
   $41 = $29;
  } else {
   $41 = 0;
  }
  $30 = HEAP32[$2>>2]|0;
  if (!($41)) {
   break;
  }
  $31 = (($30) + -1)|0;
  HEAP32[$2>>2] = $31;
 }
 $32 = ($30|0)>(0);
 if (!($32)) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$v>>2]|0;
 $34 = HEAP32[$1>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 $36 = HEAP32[$3>>2]|0;
 $37 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp4($33,98,$34,$35,0,$36,$37)|0);
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$1>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 _sqlite3ExprCacheAffinityChange($38,$39,$40);
 STACKTOP = sp;return;
}
function _codeDeferredSeek($pWInfo,$pIdx,$iCur,$iIdxCur) {
 $pWInfo = $pWInfo|0;
 $pIdx = $pIdx|0;
 $iCur = $iCur|0;
 $iIdxCur = $iIdxCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $ai = 0;
 var $i = 0, $pParse = 0, $pTab = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pParse = sp + 16|0;
 $v = sp + 12|0;
 $i = sp + 8|0;
 $pTab = sp + 4|0;
 $ai = sp;
 HEAP32[$0>>2] = $pWInfo;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iCur;
 HEAP32[$3>>2] = $iIdxCur;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pParse>>2] = $5;
 $6 = HEAP32[$pParse>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = HEAP32[$3>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($9,128,$10,0,$11)|0);
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 40|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = $15 & 32;
 $17 = ($16|0)!=(0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$pParse>>2]|0;
 $19 = ((($18)) + 124|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0|0);
 $22 = HEAP32[$pParse>>2]|0;
 if ($21) {
  $23 = ((($22)) + 124|0);
  $24 = HEAP32[$23>>2]|0;
  $26 = $24;
 } else {
  $26 = $22;
 }
 $25 = ((($26)) + 92|0);
 $27 = HEAP32[$25>>2]|0;
 $28 = ($27|0)==(0);
 if (!($28)) {
  STACKTOP = sp;return;
 }
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pTab>>2] = $31;
 $32 = HEAP32[$pParse>>2]|0;
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$pTab>>2]|0;
 $35 = ((($34)) + 34|0);
 $36 = HEAP16[$35>>1]|0;
 $37 = $36 << 16 >> 16;
 $38 = (($37) + 1)|0;
 $39 = $38<<2;
 $40 = (_sqlite3DbMallocZero($33,$39,0)|0);
 HEAP32[$ai>>2] = $40;
 $41 = HEAP32[$ai>>2]|0;
 $42 = ($41|0)!=(0|0);
 if (!($42)) {
  STACKTOP = sp;return;
 }
 $43 = HEAP32[$pTab>>2]|0;
 $44 = ((($43)) + 34|0);
 $45 = HEAP16[$44>>1]|0;
 $46 = $45 << 16 >> 16;
 $47 = HEAP32[$ai>>2]|0;
 HEAP32[$47>>2] = $46;
 HEAP32[$i>>2] = 0;
 while(1) {
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 52|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51&65535;
  $53 = (($52) - 1)|0;
  $54 = ($48|0)<($53|0);
  if (!($54)) {
   break;
  }
  $55 = HEAP32[$i>>2]|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 4|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($55<<1)|0);
  $60 = HEAP16[$59>>1]|0;
  $61 = $60 << 16 >> 16;
  $62 = ($61|0)>=(0);
  if ($62) {
   $63 = HEAP32[$i>>2]|0;
   $64 = (($63) + 1)|0;
   $65 = HEAP32[$i>>2]|0;
   $66 = HEAP32[$1>>2]|0;
   $67 = ((($66)) + 4|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (($68) + ($65<<1)|0);
   $70 = HEAP16[$69>>1]|0;
   $71 = $70 << 16 >> 16;
   $72 = (($71) + 1)|0;
   $73 = HEAP32[$ai>>2]|0;
   $74 = (($73) + ($72<<2)|0);
   HEAP32[$74>>2] = $64;
  }
  $75 = HEAP32[$i>>2]|0;
  $76 = (($75) + 1)|0;
  HEAP32[$i>>2] = $76;
 }
 $77 = HEAP32[$v>>2]|0;
 $78 = HEAP32[$ai>>2]|0;
 _sqlite3VdbeChangeP4($77,-1,$78,-15);
 STACKTOP = sp;return;
}
function _sqlite3ExprCodeGetColumnToReg($pParse,$pTab,$iColumn,$iTable,$iReg) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iColumn = $iColumn|0;
 $iTable = $iTable|0;
 $iReg = $iReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $r1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iColumn;
 HEAP32[$3>>2] = $iTable;
 HEAP32[$4>>2] = $iReg;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$4>>2]|0;
 $10 = (_sqlite3ExprCodeGetColumn($5,$6,$7,$8,$9,0)|0);
 HEAP32[$r1>>2] = $10;
 $11 = HEAP32[$r1>>2]|0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($11|0)!=($12|0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 8|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$r1>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 (_sqlite3VdbeAddOp2($16,85,$17,$18)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeP1($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $3;
 STACKTOP = sp;return;
}
function _sqlite3WhereFindTerm($pWC,$iCur,$iColumn,$0,$1,$op,$pIdx) {
 $pWC = $pWC|0;
 $iCur = $iCur|0;
 $iColumn = $iColumn|0;
 $0 = $0|0;
 $1 = $1|0;
 $op = $op|0;
 $pIdx = $pIdx|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pResult = 0, $scan = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $2 = sp + 132|0;
 $3 = sp + 128|0;
 $4 = sp + 124|0;
 $5 = sp + 120|0;
 $6 = sp;
 $7 = sp + 116|0;
 $8 = sp + 112|0;
 $pResult = sp + 108|0;
 $p = sp + 104|0;
 $scan = sp + 8|0;
 HEAP32[$3>>2] = $pWC;
 HEAP32[$4>>2] = $iCur;
 HEAP32[$5>>2] = $iColumn;
 $9 = $6;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$7>>2] = $op;
 HEAP32[$8>>2] = $pIdx;
 HEAP32[$pResult>>2] = 0;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP32[$4>>2]|0;
 $15 = HEAP32[$5>>2]|0;
 $16 = HEAP32[$7>>2]|0;
 $17 = HEAP32[$8>>2]|0;
 $18 = (_whereScanInit($scan,$13,$14,$15,$16,$17)|0);
 HEAP32[$p>>2] = $18;
 $19 = HEAP32[$7>>2]|0;
 $20 = $19 & 130;
 HEAP32[$7>>2] = $20;
 while(1) {
  $21 = HEAP32[$p>>2]|0;
  $22 = ($21|0)!=(0|0);
  if (!($22)) {
   label = 10;
   break;
  }
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 32|0);
  $25 = $24;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = $6;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = $27 & $33;
  $38 = $30 & $36;
  $39 = ($37|0)==(0);
  $40 = ($38|0)==(0);
  $41 = $39 & $40;
  if ($41) {
   $42 = HEAP32[$p>>2]|0;
   $43 = ((($42)) + 32|0);
   $44 = $43;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = (($44) + 4)|0;
   $48 = $47;
   $49 = HEAP32[$48>>2]|0;
   $50 = ($46|0)==(0);
   $51 = ($49|0)==(0);
   $52 = $50 & $51;
   if ($52) {
    $53 = HEAP32[$p>>2]|0;
    $54 = ((($53)) + 12|0);
    $55 = HEAP16[$54>>1]|0;
    $56 = $55&65535;
    $57 = HEAP32[$7>>2]|0;
    $58 = $56 & $57;
    $59 = ($58|0)!=(0);
    if ($59) {
     label = 6;
     break;
    }
   }
   $61 = HEAP32[$pResult>>2]|0;
   $62 = ($61|0)==(0|0);
   if ($62) {
    $63 = HEAP32[$p>>2]|0;
    HEAP32[$pResult>>2] = $63;
   }
  }
  $64 = (_whereScanNext($scan)|0);
  HEAP32[$p>>2] = $64;
 }
 if ((label|0) == 6) {
  $60 = HEAP32[$p>>2]|0;
  HEAP32[$2>>2] = $60;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 else if ((label|0) == 10) {
  $65 = HEAP32[$pResult>>2]|0;
  HEAP32[$2>>2] = $65;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 return (0)|0;
}
function _whereScanInit($pScan,$pWC,$iCur,$iColumn,$opMask,$pIdx) {
 $pScan = $pScan|0;
 $pWC = $pWC|0;
 $iCur = $iCur|0;
 $iColumn = $iColumn|0;
 $opMask = $opMask|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $j = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $pScan;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $iCur;
 HEAP32[$3>>2] = $iColumn;
 HEAP32[$4>>2] = $opMask;
 HEAP32[$5>>2] = $pIdx;
 HEAP32[$j>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 4|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$5>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  HEAP32[$j>>2] = $15;
  $16 = HEAP32[$j>>2]|0;
  $17 = HEAP32[$5>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($16<<1)|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = $21 << 16 >> 16;
  HEAP32[$3>>2] = $22;
  $23 = HEAP32[$3>>2]|0;
  $24 = ($23|0)==(-2);
  if ($24) {
   $25 = HEAP32[$j>>2]|0;
   $26 = HEAP32[$5>>2]|0;
   $27 = ((($26)) + 40|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ((($28)) + 4|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = (($30) + (($25*20)|0)|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$0>>2]|0;
   $34 = ((($33)) + 12|0);
   HEAP32[$34>>2] = $32;
  }
  $35 = HEAP32[$3>>2]|0;
  $36 = HEAP32[$5>>2]|0;
  $37 = ((($36)) + 12|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ((($38)) + 32|0);
  $40 = HEAP16[$39>>1]|0;
  $41 = $40 << 16 >> 16;
  $42 = ($35|0)==($41|0);
  if ($42) {
   HEAP32[$3>>2] = -1;
  }
 }
 $43 = HEAP32[$5>>2]|0;
 $44 = ($43|0)!=(0|0);
 $45 = HEAP32[$3>>2]|0;
 $46 = ($45|0)>=(0);
 $or$cond = $44 & $46;
 if ($or$cond) {
  $47 = HEAP32[$3>>2]|0;
  $48 = HEAP32[$5>>2]|0;
  $49 = ((($48)) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = ((($50)) + 4|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + ($47<<4)|0);
  $54 = ((($53)) + 13|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = HEAP32[$0>>2]|0;
  $57 = ((($56)) + 16|0);
  HEAP8[$57>>0] = $55;
  $58 = HEAP32[$j>>2]|0;
  $59 = HEAP32[$5>>2]|0;
  $60 = ((($59)) + 32|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + ($58<<2)|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = HEAP32[$0>>2]|0;
  $65 = ((($64)) + 8|0);
  HEAP32[$65>>2] = $63;
 } else {
  $66 = HEAP32[$0>>2]|0;
  $67 = ((($66)) + 16|0);
  HEAP8[$67>>0] = 0;
  $68 = HEAP32[$0>>2]|0;
  $69 = ((($68)) + 8|0);
  HEAP32[$69>>2] = 0;
 }
 $70 = HEAP32[$4>>2]|0;
 $71 = HEAP32[$0>>2]|0;
 $72 = ((($71)) + 20|0);
 HEAP32[$72>>2] = $70;
 $73 = HEAP32[$0>>2]|0;
 $74 = ((($73)) + 24|0);
 HEAP32[$74>>2] = 0;
 $75 = HEAP32[$2>>2]|0;
 $76 = HEAP32[$0>>2]|0;
 $77 = ((($76)) + 28|0);
 HEAP32[$77>>2] = $75;
 $78 = HEAP32[$3>>2]|0;
 $79 = $78&65535;
 $80 = HEAP32[$0>>2]|0;
 $81 = ((($80)) + 72|0);
 HEAP16[$81>>1] = $79;
 $82 = HEAP32[$0>>2]|0;
 $83 = ((($82)) + 17|0);
 HEAP8[$83>>0] = 1;
 $84 = HEAP32[$0>>2]|0;
 $85 = ((($84)) + 18|0);
 HEAP8[$85>>0] = 1;
 $86 = HEAP32[$0>>2]|0;
 $87 = (_whereScanNext($86)|0);
 STACKTOP = sp;return ($87|0);
}
function _whereScanNext($pScan) {
 $pScan = $pScan|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iColumn = 0, $iCur = 0, $j = 0, $k = 0, $pColl = 0, $pParse = 0, $pTerm = 0, $pWC = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $iCur = sp + 28|0;
 $iColumn = sp + 40|0;
 $pX = sp + 24|0;
 $pWC = sp + 20|0;
 $pTerm = sp + 16|0;
 $k = sp + 12|0;
 $j = sp + 8|0;
 $pColl = sp + 4|0;
 $pParse = sp;
 HEAP32[$1>>2] = $pScan;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$k>>2] = $4;
 L1: while(1) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 18|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 17|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($8|0)<=($12|0);
  if (!($13)) {
   label = 40;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 18|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($17) - 1)|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 28|0);
  $21 = (($20) + ($18<<2)|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$iCur>>2] = $22;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 18|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = (($26) - 1)|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 72|0);
  $30 = (($29) + ($27<<1)|0);
  $31 = HEAP16[$30>>1]|0;
  HEAP16[$iColumn>>1] = $31;
  $32 = HEAP16[$iColumn>>1]|0;
  $33 = $32 << 16 >> 16;
  $34 = ($33|0)==(-2);
  if ($34) {
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = ($37|0)==(0|0);
   if ($38) {
    label = 5;
    break;
   }
  }
  while(1) {
   $39 = HEAP32[$1>>2]|0;
   $40 = ((($39)) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   HEAP32[$pWC>>2] = $41;
   $42 = ($41|0)!=(0|0);
   if (!($42)) {
    break;
   }
   $43 = HEAP32[$pWC>>2]|0;
   $44 = ((($43)) + 20|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = HEAP32[$k>>2]|0;
   $47 = (($45) + (($46*48)|0)|0);
   HEAP32[$pTerm>>2] = $47;
   while(1) {
    $48 = HEAP32[$k>>2]|0;
    $49 = HEAP32[$pWC>>2]|0;
    $50 = ((($49)) + 12|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = ($48|0)<($51|0);
    if (!($52)) {
     break;
    }
    $53 = HEAP32[$pTerm>>2]|0;
    $54 = ((($53)) + 20|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = HEAP32[$iCur>>2]|0;
    $57 = ($55|0)==($56|0);
    do {
     if ($57) {
      $58 = HEAP32[$pTerm>>2]|0;
      $59 = ((($58)) + 28|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = HEAP16[$iColumn>>1]|0;
      $62 = $61 << 16 >> 16;
      $63 = ($60|0)==($62|0);
      if ($63) {
       $64 = HEAP16[$iColumn>>1]|0;
       $65 = $64 << 16 >> 16;
       $66 = ($65|0)!=(-2);
       if (!($66)) {
        $67 = HEAP32[$pTerm>>2]|0;
        $68 = HEAP32[$67>>2]|0;
        $69 = ((($68)) + 12|0);
        $70 = HEAP32[$69>>2]|0;
        $71 = HEAP32[$1>>2]|0;
        $72 = ((($71)) + 12|0);
        $73 = HEAP32[$72>>2]|0;
        $74 = HEAP32[$iCur>>2]|0;
        $75 = (_sqlite3ExprCompare($70,$73,$74)|0);
        $76 = ($75|0)==(0);
        if (!($76)) {
         break;
        }
       }
       $77 = HEAP32[$1>>2]|0;
       $78 = ((($77)) + 18|0);
       $79 = HEAP8[$78>>0]|0;
       $80 = $79&255;
       $81 = ($80|0)<=(1);
       if (!($81)) {
        $82 = HEAP32[$pTerm>>2]|0;
        $83 = HEAP32[$82>>2]|0;
        $84 = ((($83)) + 4|0);
        $85 = HEAP32[$84>>2]|0;
        $86 = $85 & 1;
        $87 = ($86|0)!=(0);
        if ($87) {
         break;
        }
       }
       $88 = HEAP32[$pTerm>>2]|0;
       $89 = ((($88)) + 12|0);
       $90 = HEAP16[$89>>1]|0;
       $91 = $90&65535;
       $92 = $91 & 2048;
       $93 = ($92|0)!=(0);
       if ($93) {
        $94 = HEAP32[$1>>2]|0;
        $95 = ((($94)) + 17|0);
        $96 = HEAP8[$95>>0]|0;
        $97 = $96&255;
        $98 = ($97|0)<(11);
        if ($98) {
         $99 = HEAP32[$pTerm>>2]|0;
         $100 = HEAP32[$99>>2]|0;
         $101 = ((($100)) + 16|0);
         $102 = HEAP32[$101>>2]|0;
         $103 = (_sqlite3ExprSkipCollate($102)|0);
         HEAP32[$pX>>2] = $103;
         $104 = HEAP8[$103>>0]|0;
         $105 = $104&255;
         $106 = ($105|0)==(152);
         if ($106) {
          HEAP32[$j>>2] = 0;
          while(1) {
           $107 = HEAP32[$j>>2]|0;
           $108 = HEAP32[$1>>2]|0;
           $109 = ((($108)) + 17|0);
           $110 = HEAP8[$109>>0]|0;
           $111 = $110&255;
           $112 = ($107|0)<($111|0);
           if (!($112)) {
            break;
           }
           $113 = HEAP32[$j>>2]|0;
           $114 = HEAP32[$1>>2]|0;
           $115 = ((($114)) + 28|0);
           $116 = (($115) + ($113<<2)|0);
           $117 = HEAP32[$116>>2]|0;
           $118 = HEAP32[$pX>>2]|0;
           $119 = ((($118)) + 28|0);
           $120 = HEAP32[$119>>2]|0;
           $121 = ($117|0)==($120|0);
           if ($121) {
            $122 = HEAP32[$j>>2]|0;
            $123 = HEAP32[$1>>2]|0;
            $124 = ((($123)) + 72|0);
            $125 = (($124) + ($122<<1)|0);
            $126 = HEAP16[$125>>1]|0;
            $127 = $126 << 16 >> 16;
            $128 = HEAP32[$pX>>2]|0;
            $129 = ((($128)) + 32|0);
            $130 = HEAP16[$129>>1]|0;
            $131 = $130 << 16 >> 16;
            $132 = ($127|0)==($131|0);
            if ($132) {
             break;
            }
           }
           $133 = HEAP32[$j>>2]|0;
           $134 = (($133) + 1)|0;
           HEAP32[$j>>2] = $134;
          }
          $135 = HEAP32[$j>>2]|0;
          $136 = HEAP32[$1>>2]|0;
          $137 = ((($136)) + 17|0);
          $138 = HEAP8[$137>>0]|0;
          $139 = $138&255;
          $140 = ($135|0)==($139|0);
          if ($140) {
           $141 = HEAP32[$pX>>2]|0;
           $142 = ((($141)) + 28|0);
           $143 = HEAP32[$142>>2]|0;
           $144 = HEAP32[$j>>2]|0;
           $145 = HEAP32[$1>>2]|0;
           $146 = ((($145)) + 28|0);
           $147 = (($146) + ($144<<2)|0);
           HEAP32[$147>>2] = $143;
           $148 = HEAP32[$pX>>2]|0;
           $149 = ((($148)) + 32|0);
           $150 = HEAP16[$149>>1]|0;
           $151 = HEAP32[$j>>2]|0;
           $152 = HEAP32[$1>>2]|0;
           $153 = ((($152)) + 72|0);
           $154 = (($153) + ($151<<1)|0);
           HEAP16[$154>>1] = $150;
           $155 = HEAP32[$1>>2]|0;
           $156 = ((($155)) + 17|0);
           $157 = HEAP8[$156>>0]|0;
           $158 = (($157) + 1)<<24>>24;
           HEAP8[$156>>0] = $158;
          }
         }
        }
       }
       $159 = HEAP32[$pTerm>>2]|0;
       $160 = ((($159)) + 12|0);
       $161 = HEAP16[$160>>1]|0;
       $162 = $161&65535;
       $163 = HEAP32[$1>>2]|0;
       $164 = ((($163)) + 20|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = $162 & $165;
       $167 = ($166|0)!=(0);
       if ($167) {
        $168 = HEAP32[$1>>2]|0;
        $169 = ((($168)) + 8|0);
        $170 = HEAP32[$169>>2]|0;
        $171 = ($170|0)!=(0|0);
        if ($171) {
         $172 = HEAP32[$pTerm>>2]|0;
         $173 = ((($172)) + 12|0);
         $174 = HEAP16[$173>>1]|0;
         $175 = $174&65535;
         $176 = $175 & 256;
         $177 = ($176|0)==(0);
         if ($177) {
          $178 = HEAP32[$pWC>>2]|0;
          $179 = HEAP32[$178>>2]|0;
          $180 = HEAP32[$179>>2]|0;
          HEAP32[$pParse>>2] = $180;
          $181 = HEAP32[$pTerm>>2]|0;
          $182 = HEAP32[$181>>2]|0;
          HEAP32[$pX>>2] = $182;
          $183 = HEAP32[$pX>>2]|0;
          $184 = HEAP32[$1>>2]|0;
          $185 = ((($184)) + 16|0);
          $186 = HEAP8[$185>>0]|0;
          $187 = (_sqlite3IndexAffinityOk($183,$186)|0);
          $188 = ($187|0)!=(0);
          if (!($188)) {
           break;
          }
          $189 = HEAP32[$pParse>>2]|0;
          $190 = HEAP32[$pX>>2]|0;
          $191 = ((($190)) + 12|0);
          $192 = HEAP32[$191>>2]|0;
          $193 = HEAP32[$pX>>2]|0;
          $194 = ((($193)) + 16|0);
          $195 = HEAP32[$194>>2]|0;
          $196 = (_sqlite3BinaryCompareCollSeq($189,$192,$195)|0);
          HEAP32[$pColl>>2] = $196;
          $197 = HEAP32[$pColl>>2]|0;
          $198 = ($197|0)==(0|0);
          if ($198) {
           $199 = HEAP32[$pParse>>2]|0;
           $200 = HEAP32[$199>>2]|0;
           $201 = ((($200)) + 8|0);
           $202 = HEAP32[$201>>2]|0;
           HEAP32[$pColl>>2] = $202;
          }
          $203 = HEAP32[$pColl>>2]|0;
          $204 = HEAP32[$203>>2]|0;
          $205 = HEAP32[$1>>2]|0;
          $206 = ((($205)) + 8|0);
          $207 = HEAP32[$206>>2]|0;
          $208 = (_sqlite3StrICmp($204,$207)|0);
          $209 = ($208|0)!=(0);
          if ($209) {
           break;
          }
         }
        }
        $210 = HEAP32[$pTerm>>2]|0;
        $211 = ((($210)) + 12|0);
        $212 = HEAP16[$211>>1]|0;
        $213 = $212&65535;
        $214 = $213 & 130;
        $215 = ($214|0)!=(0);
        if (!($215)) {
         label = 36;
         break L1;
        }
        $216 = HEAP32[$pTerm>>2]|0;
        $217 = HEAP32[$216>>2]|0;
        $218 = ((($217)) + 16|0);
        $219 = HEAP32[$218>>2]|0;
        HEAP32[$pX>>2] = $219;
        $220 = HEAP8[$219>>0]|0;
        $221 = $220&255;
        $222 = ($221|0)==(152);
        if (!($222)) {
         label = 36;
         break L1;
        }
        $223 = HEAP32[$pX>>2]|0;
        $224 = ((($223)) + 28|0);
        $225 = HEAP32[$224>>2]|0;
        $226 = HEAP32[$1>>2]|0;
        $227 = ((($226)) + 28|0);
        $228 = HEAP32[$227>>2]|0;
        $229 = ($225|0)==($228|0);
        if (!($229)) {
         label = 36;
         break L1;
        }
        $230 = HEAP32[$pX>>2]|0;
        $231 = ((($230)) + 32|0);
        $232 = HEAP16[$231>>1]|0;
        $233 = $232 << 16 >> 16;
        $234 = HEAP32[$1>>2]|0;
        $235 = ((($234)) + 72|0);
        $236 = HEAP16[$235>>1]|0;
        $237 = $236 << 16 >> 16;
        $238 = ($233|0)==($237|0);
        if (!($238)) {
         label = 36;
         break L1;
        }
       }
      }
     }
    } while(0);
    $244 = HEAP32[$k>>2]|0;
    $245 = (($244) + 1)|0;
    HEAP32[$k>>2] = $245;
    $246 = HEAP32[$pTerm>>2]|0;
    $247 = ((($246)) + 48|0);
    HEAP32[$pTerm>>2] = $247;
   }
   $248 = HEAP32[$1>>2]|0;
   $249 = ((($248)) + 4|0);
   $250 = HEAP32[$249>>2]|0;
   $251 = ((($250)) + 4|0);
   $252 = HEAP32[$251>>2]|0;
   $253 = HEAP32[$1>>2]|0;
   $254 = ((($253)) + 4|0);
   HEAP32[$254>>2] = $252;
   HEAP32[$k>>2] = 0;
  }
  $255 = HEAP32[$1>>2]|0;
  $256 = HEAP32[$255>>2]|0;
  $257 = HEAP32[$1>>2]|0;
  $258 = ((($257)) + 4|0);
  HEAP32[$258>>2] = $256;
  HEAP32[$k>>2] = 0;
  $259 = HEAP32[$1>>2]|0;
  $260 = ((($259)) + 18|0);
  $261 = HEAP8[$260>>0]|0;
  $262 = (($261) + 1)<<24>>24;
  HEAP8[$260>>0] = $262;
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 else if ((label|0) == 36) {
  $239 = HEAP32[$k>>2]|0;
  $240 = (($239) + 1)|0;
  $241 = HEAP32[$1>>2]|0;
  $242 = ((($241)) + 24|0);
  HEAP32[$242>>2] = $240;
  $243 = HEAP32[$pTerm>>2]|0;
  HEAP32[$0>>2] = $243;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 else if ((label|0) == 40) {
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 return (0)|0;
}
function _sqlite3IndexAffinityOk($pExpr,$idx_affinity) {
 $pExpr = $pExpr|0;
 $idx_affinity = $idx_affinity|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 9|0;
 $aff = sp + 8|0;
 HEAP32[$1>>2] = $pExpr;
 HEAP8[$2>>0] = $idx_affinity;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_comparisonAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP8[$aff>>0]|0;
 $6 = $5 << 24 >> 24;
 switch ($6|0) {
 case 65:  {
  HEAP32[$0>>2] = 1;
  break;
 }
 case 66:  {
  $7 = HEAP8[$2>>0]|0;
  $8 = $7 << 24 >> 24;
  $9 = ($8|0)==(66);
  $10 = $9&1;
  HEAP32[$0>>2] = $10;
  break;
 }
 default: {
  $11 = HEAP8[$2>>0]|0;
  $12 = $11 << 24 >> 24;
  $13 = ($12|0)>=(67);
  $14 = $13&1;
  HEAP32[$0>>2] = $14;
 }
 }
 $15 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($15|0);
}
function _comparisonAffinity($pExpr) {
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $aff = sp + 4|0;
 HEAP32[$0>>2] = $pExpr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3ExprAffinity($3)|0);
 HEAP8[$aff>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP8[$aff>>0]|0;
  $13 = (_sqlite3CompareAffinity($11,$12)|0);
  HEAP8[$aff>>0] = $13;
  $30 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($30|0);
 }
 $14 = ((($9)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 & 2048;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP8[$aff>>0]|0;
  $26 = (_sqlite3CompareAffinity($24,$25)|0);
  HEAP8[$aff>>0] = $26;
  $30 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($30|0);
 }
 $27 = HEAP8[$aff>>0]|0;
 $28 = $27 << 24 >> 24;
 $29 = ($28|0)==(0);
 if (!($29)) {
  $30 = HEAP8[$aff>>0]|0;
  STACKTOP = sp;return ($30|0);
 }
 HEAP8[$aff>>0] = 65;
 $30 = HEAP8[$aff>>0]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3ExprNeedsNoAffinityChange($p,$aff) {
 $p = $p|0;
 $aff = $aff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $op = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 9|0;
 $op = sp + 8|0;
 HEAP32[$1>>2] = $p;
 HEAP8[$2>>0] = $aff;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 $5 = ($4|0)==(65);
 L1: do {
  if ($5) {
   HEAP32[$0>>2] = 1;
  } else {
   while(1) {
    $6 = HEAP32[$1>>2]|0;
    $7 = HEAP8[$6>>0]|0;
    $8 = $7&255;
    $9 = ($8|0)==(156);
    if ($9) {
     $60 = 1;
    } else {
     $10 = HEAP32[$1>>2]|0;
     $11 = HEAP8[$10>>0]|0;
     $12 = $11&255;
     $13 = ($12|0)==(155);
     $60 = $13;
    }
    $14 = HEAP32[$1>>2]|0;
    if (!($60)) {
     break;
    }
    $15 = ((($14)) + 12|0);
    $16 = HEAP32[$15>>2]|0;
    HEAP32[$1>>2] = $16;
   }
   $17 = HEAP8[$14>>0]|0;
   HEAP8[$op>>0] = $17;
   $18 = HEAP8[$op>>0]|0;
   $19 = $18&255;
   $20 = ($19|0)==(157);
   if ($20) {
    $21 = HEAP32[$1>>2]|0;
    $22 = ((($21)) + 38|0);
    $23 = HEAP8[$22>>0]|0;
    HEAP8[$op>>0] = $23;
   }
   $24 = HEAP8[$op>>0]|0;
   $25 = $24&255;
   switch ($25|0) {
   case 134:  {
    $26 = HEAP8[$2>>0]|0;
    $27 = $26 << 24 >> 24;
    $28 = ($27|0)==(68);
    if ($28) {
     $33 = 1;
    } else {
     $29 = HEAP8[$2>>0]|0;
     $30 = $29 << 24 >> 24;
     $31 = ($30|0)==(67);
     $33 = $31;
    }
    $32 = $33&1;
    HEAP32[$0>>2] = $32;
    break L1;
    break;
   }
   case 132:  {
    $34 = HEAP8[$2>>0]|0;
    $35 = $34 << 24 >> 24;
    $36 = ($35|0)==(69);
    if ($36) {
     $41 = 1;
    } else {
     $37 = HEAP8[$2>>0]|0;
     $38 = $37 << 24 >> 24;
     $39 = ($38|0)==(67);
     $41 = $39;
    }
    $40 = $41&1;
    HEAP32[$0>>2] = $40;
    break L1;
    break;
   }
   case 97:  {
    $42 = HEAP8[$2>>0]|0;
    $43 = $42 << 24 >> 24;
    $44 = ($43|0)==(66);
    $45 = $44&1;
    HEAP32[$0>>2] = $45;
    break L1;
    break;
   }
   case 133:  {
    HEAP32[$0>>2] = 1;
    break L1;
    break;
   }
   case 152:  {
    $46 = HEAP32[$1>>2]|0;
    $47 = ((($46)) + 32|0);
    $48 = HEAP16[$47>>1]|0;
    $49 = $48 << 16 >> 16;
    $50 = ($49|0)<(0);
    if ($50) {
     $51 = HEAP8[$2>>0]|0;
     $52 = $51 << 24 >> 24;
     $53 = ($52|0)==(68);
     if ($53) {
      $58 = 1;
     } else {
      $54 = HEAP8[$2>>0]|0;
      $55 = $54 << 24 >> 24;
      $56 = ($55|0)==(67);
      $58 = $56;
     }
    } else {
     $58 = 0;
    }
    $57 = $58&1;
    HEAP32[$0>>2] = $57;
    break L1;
    break;
   }
   default: {
    HEAP32[$0>>2] = 0;
    break L1;
   }
   }
  }
 } while(0);
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _sqlite3StrAccumAppendAll($p,$z) {
 $p = $p|0;
 $z = $z|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 _sqlite3StrAccumAppend($2,$3,$5);
 STACKTOP = sp;return;
}
function _explainIndexRange($pStr,$pLoop) {
 $pStr = $pStr|0;
 $pLoop = $pLoop|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $nEq = 0, $nSkip = 0, $pIndex = 0, $vararg_buffer = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $pIndex = sp + 16|0;
 $nEq = sp + 30|0;
 $nSkip = sp + 28|0;
 $i = sp + 12|0;
 $j = sp + 8|0;
 $z = sp + 4|0;
 HEAP32[$0>>2] = $pStr;
 HEAP32[$1>>2] = $pLoop;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pIndex>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP16[$7>>1]|0;
 HEAP16[$nEq>>1] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP16[$10>>1]|0;
 HEAP16[$nSkip>>1] = $11;
 $12 = HEAP16[$nEq>>1]|0;
 $13 = $12&65535;
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 36|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $17 & 48;
  $19 = ($18|0)==(0);
  if ($19) {
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($20,34684,2);
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP16[$nEq>>1]|0;
  $23 = $22&65535;
  $24 = ($21|0)<($23|0);
  if (!($24)) {
   break;
  }
  $25 = HEAP32[$pIndex>>2]|0;
  $26 = HEAP32[$i>>2]|0;
  $27 = (_explainIndexColumnName($25,$26)|0);
  HEAP32[$z>>2] = $27;
  $28 = HEAP32[$i>>2]|0;
  $29 = ($28|0)!=(0);
  if ($29) {
   $30 = HEAP32[$0>>2]|0;
   _sqlite3StrAccumAppend($30,34687,5);
  }
  $31 = HEAP32[$0>>2]|0;
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP16[$nSkip>>1]|0;
  $34 = $33&65535;
  $35 = ($32|0)>=($34|0);
  $36 = $35 ? 34693 : 34698;
  $37 = HEAP32[$z>>2]|0;
  HEAP32[$vararg_buffer>>2] = $37;
  _sqlite3XPrintf($31,$36,$vararg_buffer);
  $38 = HEAP32[$i>>2]|0;
  $39 = (($38) + 1)|0;
  HEAP32[$i>>2] = $39;
 }
 $40 = HEAP32[$i>>2]|0;
 HEAP32[$j>>2] = $40;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 36|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = $43 & 32;
 $45 = ($44|0)!=(0);
 if ($45) {
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$pIndex>>2]|0;
  $48 = HEAP32[$1>>2]|0;
  $49 = ((($48)) + 24|0);
  $50 = ((($49)) + 2|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51&65535;
  $53 = HEAP32[$j>>2]|0;
  $54 = HEAP32[$i>>2]|0;
  _explainAppendTerm($46,$47,$52,$53,$54,34615);
  HEAP32[$i>>2] = 1;
 }
 $55 = HEAP32[$1>>2]|0;
 $56 = ((($55)) + 36|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $57 & 16;
 $59 = ($58|0)!=(0);
 if ($59) {
  $60 = HEAP32[$0>>2]|0;
  $61 = HEAP32[$pIndex>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 24|0);
  $64 = ((($63)) + 4|0);
  $65 = HEAP16[$64>>1]|0;
  $66 = $65&65535;
  $67 = HEAP32[$j>>2]|0;
  $68 = HEAP32[$i>>2]|0;
  _explainAppendTerm($60,$61,$66,$67,$68,34617);
 }
 $69 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($69,34706,1);
 STACKTOP = sp;return;
}
function _explainIndexColumnName($pIdx,$i) {
 $pIdx = $pIdx|0;
 $i = $i|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $i;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<1)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 HEAP32[$2>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)==(-2);
 if ($11) {
  HEAP32[$0>>2] = 34712;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = ($12|0)==(-1);
 if ($13) {
  HEAP32[$0>>2] = 24745;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 } else {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + ($14<<4)|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$0>>2] = $21;
  $22 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _explainAppendTerm($pStr,$pIdx,$nTerm,$iTerm,$bAnd,$zOp) {
 $pStr = $pStr|0;
 $pIdx = $pIdx|0;
 $nTerm = $nTerm|0;
 $iTerm = $iTerm|0;
 $bAnd = $bAnd|0;
 $zOp = $zOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pStr;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $nTerm;
 HEAP32[$3>>2] = $iTerm;
 HEAP32[$4>>2] = $bAnd;
 HEAP32[$5>>2] = $zOp;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($8,34687,5);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)>(1);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($11,34708,1);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = ($15|0)!=(0);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   _sqlite3StrAccumAppend($17,34710,1);
  }
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$3>>2]|0;
  $21 = HEAP32[$i>>2]|0;
  $22 = (($20) + ($21))|0;
  $23 = (_explainIndexColumnName($19,$22)|0);
  _sqlite3StrAccumAppendAll($18,$23);
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
 }
 $26 = HEAP32[$2>>2]|0;
 $27 = ($26|0)>(1);
 if ($27) {
  $28 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($28,34706,1);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$5>>2]|0;
 _sqlite3StrAccumAppend($29,$30,1);
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)>(1);
 if ($32) {
  $33 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($33,34708,1);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ($34|0)<($35|0);
  if (!($36)) {
   break;
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = ($37|0)!=(0);
  if ($38) {
   $39 = HEAP32[$0>>2]|0;
   _sqlite3StrAccumAppend($39,34710,1);
  }
  $40 = HEAP32[$0>>2]|0;
  _sqlite3StrAccumAppend($40,20275,1);
  $41 = HEAP32[$i>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$i>>2] = $42;
 }
 $43 = HEAP32[$2>>2]|0;
 $44 = ($43|0)>(1);
 if (!($44)) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 _sqlite3StrAccumAppend($45,34706,1);
 STACKTOP = sp;return;
}
function _sqlite3ExprIsTableConstant($p,$iCur) {
 $p = $p|0;
 $iCur = $iCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCur;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_exprIsConst($2,3,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _termCanDriveIndex($pTerm,$pSrc,$0,$1) {
 $pTerm = $pTerm|0;
 $pSrc = $pSrc|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $aff = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 $aff = sp + 20|0;
 HEAP32[$3>>2] = $pTerm;
 HEAP32[$4>>2] = $pSrc;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$3>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = ((($13)) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($12|0)!=($15|0);
 if ($16) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP16[$18>>1]|0;
 $20 = $19&65535;
 $21 = $20 & 130;
 $22 = ($21|0)==(0);
 if ($22) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $23 = HEAP32[$3>>2]|0;
 $24 = ((($23)) + 32|0);
 $25 = $24;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = $5;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = $27 & $33;
 $38 = $30 & $36;
 $39 = ($37|0)!=(0);
 $40 = ($38|0)!=(0);
 $41 = $39 | $40;
 if ($41) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $42 = HEAP32[$3>>2]|0;
 $43 = ((($42)) + 28|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)<(0);
 if ($45) {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 $46 = HEAP32[$3>>2]|0;
 $47 = ((($46)) + 28|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$4>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ((($51)) + 4|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = (($53) + ($48<<4)|0);
 $55 = ((($54)) + 13|0);
 $56 = HEAP8[$55>>0]|0;
 HEAP8[$aff>>0] = $56;
 $57 = HEAP32[$3>>2]|0;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP8[$aff>>0]|0;
 $60 = (_sqlite3IndexAffinityOk($58,$59)|0);
 $61 = ($60|0)!=(0);
 if ($61) {
  HEAP32[$2>>2] = 1;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 } else {
  HEAP32[$2>>2] = 0;
  $62 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 return (0)|0;
}
function _whereLoopResize($db,$p,$n) {
 $db = $db|0;
 $p = $p|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $paNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $paNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = (($10) + 7)|0;
 $12 = $11 & -8;
 HEAP32[$3>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 $15 = $14<<2;
 $16 = (_sqlite3DbMallocRawNN($13,$15,0)|0);
 HEAP32[$paNew>>2] = $16;
 $17 = HEAP32[$paNew>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$0>>2] = 7;
  $45 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 $19 = HEAP32[$paNew>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 44|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26<<2;
 _memcpy(($19|0),($22|0),($27|0))|0;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 48|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 $32 = ((($31)) + 56|0);
 $33 = ($30|0)!=($32|0);
 if ($33) {
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 48|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3DbFree($34,$37);
 }
 $38 = HEAP32[$paNew>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = ((($39)) + 48|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$3>>2]|0;
 $42 = $41&65535;
 $43 = HEAP32[$2>>2]|0;
 $44 = ((($43)) + 44|0);
 HEAP16[$44>>1] = $42;
 HEAP32[$0>>2] = 0;
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _sqlite3AllocateIndexObject($db,$nCol,$nExtra,$ppExtra) {
 $db = $db|0;
 $nCol = $nCol|0;
 $nExtra = $nExtra|0;
 $ppExtra = $ppExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $nByte = 0, $p = 0, $pExtra = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 24|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $p = sp + 8|0;
 $nByte = sp + 4|0;
 $pExtra = sp;
 HEAP32[$0>>2] = $db;
 HEAP16[$1>>1] = $nCol;
 HEAP32[$2>>2] = $nExtra;
 HEAP32[$3>>2] = $ppExtra;
 $4 = HEAP16[$1>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = $5<<2;
 $7 = (($6) + 7)|0;
 $8 = $7 & -8;
 $9 = (56 + ($8))|0;
 $10 = HEAP16[$1>>1]|0;
 $11 = $10 << 16 >> 16;
 $12 = (($11) + 1)|0;
 $13 = $12<<1;
 $14 = HEAP16[$1>>1]|0;
 $15 = $14 << 16 >> 16;
 $16 = $15<<1;
 $17 = (($13) + ($16))|0;
 $18 = HEAP16[$1>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = $19;
 $21 = (($17) + ($20))|0;
 $22 = (($21) + 7)|0;
 $23 = $22 & -8;
 $24 = (($9) + ($23))|0;
 HEAP32[$nByte>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$nByte>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = (($26) + ($27))|0;
 $29 = ($28|0)<(0);
 $30 = $29 << 31 >> 31;
 $31 = (_sqlite3DbMallocZero($25,$28,$30)|0);
 HEAP32[$p>>2] = $31;
 $32 = HEAP32[$p>>2]|0;
 $33 = ($32|0)!=(0|0);
 if (!($33)) {
  $79 = HEAP32[$p>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $34 = HEAP32[$p>>2]|0;
 $35 = ((($34)) + 56|0);
 HEAP32[$pExtra>>2] = $35;
 $36 = HEAP32[$pExtra>>2]|0;
 $37 = HEAP32[$p>>2]|0;
 $38 = ((($37)) + 32|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP16[$1>>1]|0;
 $40 = $39 << 16 >> 16;
 $41 = $40<<2;
 $42 = (($41) + 7)|0;
 $43 = $42 & -8;
 $44 = HEAP32[$pExtra>>2]|0;
 $45 = (($44) + ($43)|0);
 HEAP32[$pExtra>>2] = $45;
 $46 = HEAP32[$pExtra>>2]|0;
 $47 = HEAP32[$p>>2]|0;
 $48 = ((($47)) + 8|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP16[$1>>1]|0;
 $50 = $49 << 16 >> 16;
 $51 = (($50) + 1)|0;
 $52 = $51<<1;
 $53 = HEAP32[$pExtra>>2]|0;
 $54 = (($53) + ($52)|0);
 HEAP32[$pExtra>>2] = $54;
 $55 = HEAP32[$pExtra>>2]|0;
 $56 = HEAP32[$p>>2]|0;
 $57 = ((($56)) + 4|0);
 HEAP32[$57>>2] = $55;
 $58 = HEAP16[$1>>1]|0;
 $59 = $58 << 16 >> 16;
 $60 = $59<<1;
 $61 = HEAP32[$pExtra>>2]|0;
 $62 = (($61) + ($60)|0);
 HEAP32[$pExtra>>2] = $62;
 $63 = HEAP32[$pExtra>>2]|0;
 $64 = HEAP32[$p>>2]|0;
 $65 = ((($64)) + 28|0);
 HEAP32[$65>>2] = $63;
 $66 = HEAP16[$1>>1]|0;
 $67 = HEAP32[$p>>2]|0;
 $68 = ((($67)) + 52|0);
 HEAP16[$68>>1] = $66;
 $69 = HEAP16[$1>>1]|0;
 $70 = $69 << 16 >> 16;
 $71 = (($70) - 1)|0;
 $72 = $71&65535;
 $73 = HEAP32[$p>>2]|0;
 $74 = ((($73)) + 50|0);
 HEAP16[$74>>1] = $72;
 $75 = HEAP32[$p>>2]|0;
 $76 = HEAP32[$nByte>>2]|0;
 $77 = (($75) + ($76)|0);
 $78 = HEAP32[$3>>2]|0;
 HEAP32[$78>>2] = $77;
 $79 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _sqlite3WhereExprUsage($pMaskSet,$p) {
 $pMaskSet = $pMaskSet|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $mask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $mask = sp;
 HEAP32[$1>>2] = $pMaskSet;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  $5 = $0;
  $6 = $5;
  HEAP32[$6>>2] = 0;
  $7 = (($5) + 4)|0;
  $8 = $7;
  HEAP32[$8>>2] = 0;
  $126 = $0;
  $127 = $126;
  $128 = HEAP32[$127>>2]|0;
  $129 = (($126) + 4)|0;
  $130 = $129;
  $131 = HEAP32[$130>>2]|0;
  tempRet0 = ($131);
  STACKTOP = sp;return ($128|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(152);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = ((($14)) + 28|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3WhereGetMask($13,$16)|0);
  $18 = tempRet0;
  $19 = $mask;
  $20 = $19;
  HEAP32[$20>>2] = $17;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $18;
  $23 = $mask;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = $0;
  $30 = $29;
  HEAP32[$30>>2] = $25;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $28;
  $126 = $0;
  $127 = $126;
  $128 = HEAP32[$127>>2]|0;
  $129 = (($126) + 4)|0;
  $130 = $129;
  $131 = HEAP32[$130>>2]|0;
  tempRet0 = ($131);
  STACKTOP = sp;return ($128|0);
 }
 $33 = HEAP32[$2>>2]|0;
 $34 = ((($33)) + 16|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)!=(0|0);
 if ($36) {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP32[$2>>2]|0;
  $39 = ((($38)) + 16|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (_sqlite3WhereExprUsage($37,$40)|0);
  $42 = tempRet0;
  $45 = $41;$48 = $42;
 } else {
  $45 = 0;$48 = 0;
 }
 $43 = $mask;
 $44 = $43;
 HEAP32[$44>>2] = $45;
 $46 = (($43) + 4)|0;
 $47 = $46;
 HEAP32[$47>>2] = $48;
 $49 = HEAP32[$2>>2]|0;
 $50 = ((($49)) + 12|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = ($51|0)!=(0|0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  $55 = ((($54)) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = (_sqlite3WhereExprUsage($53,$56)|0);
  $58 = tempRet0;
  $59 = $mask;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($59) + 4)|0;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = $61 | $57;
  $66 = $64 | $58;
  $67 = $mask;
  $68 = $67;
  HEAP32[$68>>2] = $65;
  $69 = (($67) + 4)|0;
  $70 = $69;
  HEAP32[$70>>2] = $66;
 }
 $71 = HEAP32[$2>>2]|0;
 $72 = ((($71)) + 4|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = $73 & 2048;
 $75 = ($74|0)!=(0);
 if ($75) {
  $76 = HEAP32[$1>>2]|0;
  $77 = HEAP32[$2>>2]|0;
  $78 = ((($77)) + 20|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = (_exprSelectUsage($76,$79)|0);
  $81 = tempRet0;
  $82 = $mask;
  $83 = $82;
  $84 = HEAP32[$83>>2]|0;
  $85 = (($82) + 4)|0;
  $86 = $85;
  $87 = HEAP32[$86>>2]|0;
  $88 = $84 | $80;
  $89 = $87 | $81;
  $90 = $mask;
  $91 = $90;
  HEAP32[$91>>2] = $88;
  $92 = (($90) + 4)|0;
  $93 = $92;
  HEAP32[$93>>2] = $89;
 } else {
  $94 = HEAP32[$2>>2]|0;
  $95 = ((($94)) + 20|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)!=(0|0);
  if ($97) {
   $98 = HEAP32[$1>>2]|0;
   $99 = HEAP32[$2>>2]|0;
   $100 = ((($99)) + 20|0);
   $101 = HEAP32[$100>>2]|0;
   $102 = (_sqlite3WhereExprListUsage($98,$101)|0);
   $103 = tempRet0;
   $104 = $mask;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = (($104) + 4)|0;
   $108 = $107;
   $109 = HEAP32[$108>>2]|0;
   $110 = $106 | $102;
   $111 = $109 | $103;
   $112 = $mask;
   $113 = $112;
   HEAP32[$113>>2] = $110;
   $114 = (($112) + 4)|0;
   $115 = $114;
   HEAP32[$115>>2] = $111;
  }
 }
 $116 = $mask;
 $117 = $116;
 $118 = HEAP32[$117>>2]|0;
 $119 = (($116) + 4)|0;
 $120 = $119;
 $121 = HEAP32[$120>>2]|0;
 $122 = $0;
 $123 = $122;
 HEAP32[$123>>2] = $118;
 $124 = (($122) + 4)|0;
 $125 = $124;
 HEAP32[$125>>2] = $121;
 $126 = $0;
 $127 = $126;
 $128 = HEAP32[$127>>2]|0;
 $129 = (($126) + 4)|0;
 $130 = $129;
 $131 = HEAP32[$130>>2]|0;
 tempRet0 = ($131);
 STACKTOP = sp;return ($128|0);
}
function _exprSelectUsage($pMaskSet,$pS) {
 $pMaskSet = $pMaskSet|0;
 $pS = $pS|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0;
 var $mask = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $mask = sp;
 $pSrc = sp + 12|0;
 $i = sp + 8|0;
 HEAP32[$0>>2] = $pMaskSet;
 HEAP32[$1>>2] = $pS;
 $2 = $mask;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 while(1) {
  $6 = HEAP32[$1>>2]|0;
  $7 = ($6|0)!=(0|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 28|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pSrc>>2] = $10;
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_sqlite3WhereExprListUsage($11,$13)|0);
  $15 = tempRet0;
  $16 = $mask;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = (($16) + 4)|0;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = $18 | $14;
  $23 = $21 | $15;
  $24 = $mask;
  $25 = $24;
  HEAP32[$25>>2] = $22;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  $28 = HEAP32[$0>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 36|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3WhereExprListUsage($28,$31)|0);
  $33 = tempRet0;
  $34 = $mask;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = $36 | $32;
  $41 = $39 | $33;
  $42 = $mask;
  $43 = $42;
  HEAP32[$43>>2] = $40;
  $44 = (($42) + 4)|0;
  $45 = $44;
  HEAP32[$45>>2] = $41;
  $46 = HEAP32[$0>>2]|0;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 44|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (_sqlite3WhereExprListUsage($46,$49)|0);
  $51 = tempRet0;
  $52 = $mask;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (($52) + 4)|0;
  $56 = $55;
  $57 = HEAP32[$56>>2]|0;
  $58 = $54 | $50;
  $59 = $57 | $51;
  $60 = $mask;
  $61 = $60;
  HEAP32[$61>>2] = $58;
  $62 = (($60) + 4)|0;
  $63 = $62;
  HEAP32[$63>>2] = $59;
  $64 = HEAP32[$0>>2]|0;
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 32|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (_sqlite3WhereExprUsage($64,$67)|0);
  $69 = tempRet0;
  $70 = $mask;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = (($70) + 4)|0;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = $72 | $68;
  $77 = $75 | $69;
  $78 = $mask;
  $79 = $78;
  HEAP32[$79>>2] = $76;
  $80 = (($78) + 4)|0;
  $81 = $80;
  HEAP32[$81>>2] = $77;
  $82 = HEAP32[$0>>2]|0;
  $83 = HEAP32[$1>>2]|0;
  $84 = ((($83)) + 40|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = (_sqlite3WhereExprUsage($82,$85)|0);
  $87 = tempRet0;
  $88 = $mask;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = $90 | $86;
  $95 = $93 | $87;
  $96 = $mask;
  $97 = $96;
  HEAP32[$97>>2] = $94;
  $98 = (($96) + 4)|0;
  $99 = $98;
  HEAP32[$99>>2] = $95;
  $100 = HEAP32[$pSrc>>2]|0;
  $101 = ($100|0)!=(0|0);
  L4: do {
   if ($101) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $102 = HEAP32[$i>>2]|0;
     $103 = HEAP32[$pSrc>>2]|0;
     $104 = HEAP32[$103>>2]|0;
     $105 = ($102|0)<($104|0);
     if (!($105)) {
      break L4;
     }
     $106 = HEAP32[$0>>2]|0;
     $107 = HEAP32[$i>>2]|0;
     $108 = HEAP32[$pSrc>>2]|0;
     $109 = ((($108)) + 8|0);
     $110 = (($109) + (($107*72)|0)|0);
     $111 = ((($110)) + 20|0);
     $112 = HEAP32[$111>>2]|0;
     $113 = (_exprSelectUsage($106,$112)|0);
     $114 = tempRet0;
     $115 = $mask;
     $116 = $115;
     $117 = HEAP32[$116>>2]|0;
     $118 = (($115) + 4)|0;
     $119 = $118;
     $120 = HEAP32[$119>>2]|0;
     $121 = $117 | $113;
     $122 = $120 | $114;
     $123 = $mask;
     $124 = $123;
     HEAP32[$124>>2] = $121;
     $125 = (($123) + 4)|0;
     $126 = $125;
     HEAP32[$126>>2] = $122;
     $127 = HEAP32[$0>>2]|0;
     $128 = HEAP32[$i>>2]|0;
     $129 = HEAP32[$pSrc>>2]|0;
     $130 = ((($129)) + 8|0);
     $131 = (($130) + (($128*72)|0)|0);
     $132 = ((($131)) + 48|0);
     $133 = HEAP32[$132>>2]|0;
     $134 = (_sqlite3WhereExprUsage($127,$133)|0);
     $135 = tempRet0;
     $136 = $mask;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = (($136) + 4)|0;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = $138 | $134;
     $143 = $141 | $135;
     $144 = $mask;
     $145 = $144;
     HEAP32[$145>>2] = $142;
     $146 = (($144) + 4)|0;
     $147 = $146;
     HEAP32[$147>>2] = $143;
     $148 = HEAP32[$i>>2]|0;
     $149 = (($148) + 1)|0;
     HEAP32[$i>>2] = $149;
    }
   }
  } while(0);
  $150 = HEAP32[$1>>2]|0;
  $151 = ((($150)) + 48|0);
  $152 = HEAP32[$151>>2]|0;
  HEAP32[$1>>2] = $152;
 }
 $153 = $mask;
 $154 = $153;
 $155 = HEAP32[$154>>2]|0;
 $156 = (($153) + 4)|0;
 $157 = $156;
 $158 = HEAP32[$157>>2]|0;
 tempRet0 = ($158);
 STACKTOP = sp;return ($155|0);
}
function _sqlite3LogEstAdd($a,$b) {
 $a = $a|0;
 $b = $b|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 2|0;
 $2 = sp;
 HEAP16[$1>>1] = $a;
 HEAP16[$2>>1] = $b;
 $3 = HEAP16[$1>>1]|0;
 $4 = $3 << 16 >> 16;
 $5 = HEAP16[$2>>1]|0;
 $6 = $5 << 16 >> 16;
 $7 = ($4|0)>=($6|0);
 if ($7) {
  $8 = HEAP16[$1>>1]|0;
  $9 = $8 << 16 >> 16;
  $10 = HEAP16[$2>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = (($11) + 49)|0;
  $13 = ($9|0)>($12|0);
  $14 = HEAP16[$1>>1]|0;
  if ($13) {
   HEAP16[$0>>1] = $14;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
  $15 = $14 << 16 >> 16;
  $16 = HEAP16[$2>>1]|0;
  $17 = $16 << 16 >> 16;
  $18 = (($17) + 31)|0;
  $19 = ($15|0)>($18|0);
  $20 = HEAP16[$1>>1]|0;
  $21 = $20 << 16 >> 16;
  if ($19) {
   $22 = (($21) + 1)|0;
   $23 = $22&65535;
   HEAP16[$0>>1] = $23;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  } else {
   $24 = HEAP16[$1>>1]|0;
   $25 = $24 << 16 >> 16;
   $26 = HEAP16[$2>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = (($25) - ($27))|0;
   $29 = (34774 + ($28)|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = (($21) + ($31))|0;
   $33 = $32&65535;
   HEAP16[$0>>1] = $33;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
 } else {
  $34 = HEAP16[$2>>1]|0;
  $35 = $34 << 16 >> 16;
  $36 = HEAP16[$1>>1]|0;
  $37 = $36 << 16 >> 16;
  $38 = (($37) + 49)|0;
  $39 = ($35|0)>($38|0);
  $40 = HEAP16[$2>>1]|0;
  if ($39) {
   HEAP16[$0>>1] = $40;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
  $41 = $40 << 16 >> 16;
  $42 = HEAP16[$1>>1]|0;
  $43 = $42 << 16 >> 16;
  $44 = (($43) + 31)|0;
  $45 = ($41|0)>($44|0);
  $46 = HEAP16[$2>>1]|0;
  $47 = $46 << 16 >> 16;
  if ($45) {
   $48 = (($47) + 1)|0;
   $49 = $48&65535;
   HEAP16[$0>>1] = $49;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  } else {
   $50 = HEAP16[$2>>1]|0;
   $51 = $50 << 16 >> 16;
   $52 = HEAP16[$1>>1]|0;
   $53 = $52 << 16 >> 16;
   $54 = (($51) - ($53))|0;
   $55 = (34774 + ($54)|0);
   $56 = HEAP8[$55>>0]|0;
   $57 = $56&255;
   $58 = (($47) + ($57))|0;
   $59 = $58&65535;
   HEAP16[$0>>1] = $59;
   $60 = HEAP16[$0>>1]|0;
   STACKTOP = sp;return ($60|0);
  }
 }
 return (0)|0;
}
function _wherePathSatisfiesOrderBy($pWInfo,$pOrderBy,$pPath,$wctrlFlags,$nLoop,$pLast,$pRevMask) {
 $pWInfo = $pWInfo|0;
 $pOrderBy = $pOrderBy|0;
 $pPath = $pPath|0;
 $wctrlFlags = $wctrlFlags|0;
 $nLoop = $nLoop|0;
 $pLast = $pLast|0;
 $pRevMask = $pRevMask|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bOnce = 0, $db = 0, $distinctColumns = 0, $eOp = 0, $eqOpMask = 0, $i = 0, $iColumn = 0, $iCur = 0, $iLoop = 0, $isMatch = 0, $isOrderDistinct = 0, $j = 0, $m = 0, $mTerm = 0, $nColumn = 0;
 var $nKeyCol = 0, $nOrderBy = 0, $obDone = 0, $obSat = 0, $or$cond = 0, $orderDistinctMask = 0, $p = 0, $pColl = 0, $pIndex = 0, $pLoop = 0, $pOBExpr = 0, $pTerm = 0, $pX = 0, $ready = 0, $rev = 0, $revIdx = 0, $revSet = 0, $z1 = 0, $z2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 149|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 116|0;
 $4 = sp + 140|0;
 $5 = sp + 138|0;
 $6 = sp + 112|0;
 $7 = sp + 108|0;
 $revSet = sp + 148|0;
 $rev = sp + 147|0;
 $revIdx = sp + 146|0;
 $isOrderDistinct = sp + 145|0;
 $distinctColumns = sp + 144|0;
 $isMatch = sp + 143|0;
 $eqOpMask = sp + 136|0;
 $nKeyCol = sp + 134|0;
 $nColumn = sp + 132|0;
 $nOrderBy = sp + 130|0;
 $iLoop = sp + 104|0;
 $i = sp + 100|0;
 $j = sp + 96|0;
 $iCur = sp + 92|0;
 $iColumn = sp + 88|0;
 $pLoop = sp + 84|0;
 $pTerm = sp + 80|0;
 $pOBExpr = sp + 76|0;
 $pColl = sp + 72|0;
 $pIndex = sp + 68|0;
 $db = sp + 64|0;
 $obSat = sp + 40|0;
 $obDone = sp + 32|0;
 $orderDistinctMask = sp + 24|0;
 $ready = sp + 16|0;
 $z1 = sp + 60|0;
 $z2 = sp + 56|0;
 $bOnce = sp + 142|0;
 $eOp = sp + 128|0;
 $pX = sp + 52|0;
 $p = sp + 48|0;
 $mTerm = sp + 8|0;
 $m = sp;
 HEAP32[$1>>2] = $pWInfo;
 HEAP32[$2>>2] = $pOrderBy;
 HEAP32[$3>>2] = $pPath;
 HEAP16[$4>>1] = $wctrlFlags;
 HEAP16[$5>>1] = $nLoop;
 HEAP32[$6>>2] = $pLast;
 HEAP32[$7>>2] = $pRevMask;
 HEAP32[$pLoop>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = $obSat;
 $12 = $11;
 HEAP32[$12>>2] = 0;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = 0;
 $15 = HEAP16[$5>>1]|0;
 $16 = $15&65535;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = ((($18)) + 64|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20&65535;
  $22 = $21 & 128;
  $23 = ($22|0)!=(0);
  if ($23) {
   HEAP8[$0>>0] = 0;
   $764 = HEAP8[$0>>0]|0;
   STACKTOP = sp;return ($764|0);
  }
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 $26 = $25&65535;
 HEAP16[$nOrderBy>>1] = $26;
 $27 = HEAP16[$nOrderBy>>1]|0;
 $28 = $27&65535;
 $29 = ($28|0)>(63);
 if ($29) {
  HEAP8[$0>>0] = 0;
  $764 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($764|0);
 }
 HEAP8[$isOrderDistinct>>0] = 1;
 $30 = HEAP16[$nOrderBy>>1]|0;
 $31 = $30&65535;
 $32 = (_bitshift64Shl(1,0,($31|0))|0);
 $33 = tempRet0;
 $34 = (_i64Subtract(($32|0),($33|0),1,0)|0);
 $35 = tempRet0;
 $36 = $obDone;
 $37 = $36;
 HEAP32[$37>>2] = $34;
 $38 = (($36) + 4)|0;
 $39 = $38;
 HEAP32[$39>>2] = $35;
 $40 = $orderDistinctMask;
 $41 = $40;
 HEAP32[$41>>2] = 0;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = 0;
 $44 = $ready;
 $45 = $44;
 HEAP32[$45>>2] = 0;
 $46 = (($44) + 4)|0;
 $47 = $46;
 HEAP32[$47>>2] = 0;
 HEAP16[$eqOpMask>>1] = 386;
 $48 = HEAP16[$4>>1]|0;
 $49 = $48&65535;
 $50 = $49 & 2048;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = HEAP16[$eqOpMask>>1]|0;
  $53 = $52&65535;
  $54 = $53 | 1;
  $55 = $54&65535;
  HEAP16[$eqOpMask>>1] = $55;
 }
 HEAP32[$iLoop>>2] = 0;
 L13: while(1) {
  $56 = HEAP8[$isOrderDistinct>>0]|0;
  $57 = $56&255;
  $58 = ($57|0)!=(0);
  if (!($58)) {
   break;
  }
  $59 = $obSat;
  $60 = $59;
  $61 = HEAP32[$60>>2]|0;
  $62 = (($59) + 4)|0;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = $obDone;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = (($65) + 4)|0;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = ($64>>>0)<($70>>>0);
  $72 = ($61>>>0)<($67>>>0);
  $73 = ($64|0)==($70|0);
  $74 = $73 & $72;
  $75 = $71 | $74;
  if (!($75)) {
   break;
  }
  $76 = HEAP32[$iLoop>>2]|0;
  $77 = HEAP16[$5>>1]|0;
  $78 = $77&65535;
  $79 = ($76|0)<=($78|0);
  if (!($79)) {
   break;
  }
  $80 = HEAP32[$iLoop>>2]|0;
  $81 = ($80|0)>(0);
  if ($81) {
   $82 = HEAP32[$pLoop>>2]|0;
   $83 = ((($82)) + 8|0);
   $84 = $83;
   $85 = $84;
   $86 = HEAP32[$85>>2]|0;
   $87 = (($84) + 4)|0;
   $88 = $87;
   $89 = HEAP32[$88>>2]|0;
   $90 = $ready;
   $91 = $90;
   $92 = HEAP32[$91>>2]|0;
   $93 = (($90) + 4)|0;
   $94 = $93;
   $95 = HEAP32[$94>>2]|0;
   $96 = $92 | $86;
   $97 = $95 | $89;
   $98 = $ready;
   $99 = $98;
   HEAP32[$99>>2] = $96;
   $100 = (($98) + 4)|0;
   $101 = $100;
   HEAP32[$101>>2] = $97;
  }
  $102 = HEAP32[$iLoop>>2]|0;
  $103 = HEAP16[$5>>1]|0;
  $104 = $103&65535;
  $105 = ($102|0)<($104|0);
  if ($105) {
   $106 = HEAP32[$iLoop>>2]|0;
   $107 = HEAP32[$3>>2]|0;
   $108 = ((($107)) + 24|0);
   $109 = HEAP32[$108>>2]|0;
   $110 = (($109) + ($106<<2)|0);
   $111 = HEAP32[$110>>2]|0;
   HEAP32[$pLoop>>2] = $111;
   $112 = HEAP16[$4>>1]|0;
   $113 = $112&65535;
   $114 = $113 & 2048;
   $115 = ($114|0)!=(0);
   if (!($115)) {
    label = 17;
   }
  } else {
   $116 = HEAP32[$6>>2]|0;
   HEAP32[$pLoop>>2] = $116;
   label = 17;
  }
  L24: do {
   if ((label|0) == 17) {
    label = 0;
    $117 = HEAP32[$pLoop>>2]|0;
    $118 = ((($117)) + 36|0);
    $119 = HEAP32[$118>>2]|0;
    $120 = $119 & 1024;
    $121 = ($120|0)!=(0);
    $122 = HEAP32[$pLoop>>2]|0;
    if ($121) {
     label = 18;
     break L13;
    }
    $137 = ((($122)) + 16|0);
    $138 = HEAP8[$137>>0]|0;
    $139 = $138&255;
    $140 = HEAP32[$1>>2]|0;
    $141 = ((($140)) + 4|0);
    $142 = HEAP32[$141>>2]|0;
    $143 = ((($142)) + 8|0);
    $144 = (($143) + (($139*72)|0)|0);
    $145 = ((($144)) + 44|0);
    $146 = HEAP32[$145>>2]|0;
    HEAP32[$iCur>>2] = $146;
    HEAP32[$i>>2] = 0;
    while(1) {
     $147 = HEAP32[$i>>2]|0;
     $148 = HEAP16[$nOrderBy>>1]|0;
     $149 = $148&65535;
     $150 = ($147|0)<($149|0);
     if (!($150)) {
      break;
     }
     $151 = HEAP32[$i>>2]|0;
     $152 = (_bitshift64Shl(1,0,($151|0))|0);
     $153 = tempRet0;
     $154 = $obSat;
     $155 = $154;
     $156 = HEAP32[$155>>2]|0;
     $157 = (($154) + 4)|0;
     $158 = $157;
     $159 = HEAP32[$158>>2]|0;
     $160 = $152 & $156;
     $161 = $153 & $159;
     $162 = ($160|0)!=(0);
     $163 = ($161|0)!=(0);
     $164 = $162 | $163;
     L30: do {
      if (!($164)) {
       $165 = HEAP32[$i>>2]|0;
       $166 = HEAP32[$2>>2]|0;
       $167 = ((($166)) + 4|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = (($168) + (($165*20)|0)|0);
       $170 = HEAP32[$169>>2]|0;
       $171 = (_sqlite3ExprSkipCollate($170)|0);
       HEAP32[$pOBExpr>>2] = $171;
       $172 = HEAP32[$pOBExpr>>2]|0;
       $173 = HEAP8[$172>>0]|0;
       $174 = $173&255;
       $175 = ($174|0)!=(152);
       if (!($175)) {
        $176 = HEAP32[$pOBExpr>>2]|0;
        $177 = ((($176)) + 28|0);
        $178 = HEAP32[$177>>2]|0;
        $179 = HEAP32[$iCur>>2]|0;
        $180 = ($178|0)!=($179|0);
        if (!($180)) {
         $181 = HEAP32[$1>>2]|0;
         $182 = ((($181)) + 80|0);
         $183 = HEAP32[$iCur>>2]|0;
         $184 = HEAP32[$pOBExpr>>2]|0;
         $185 = ((($184)) + 32|0);
         $186 = HEAP16[$185>>1]|0;
         $187 = $186 << 16 >> 16;
         $188 = $ready;
         $189 = $188;
         $190 = HEAP32[$189>>2]|0;
         $191 = (($188) + 4)|0;
         $192 = $191;
         $193 = HEAP32[$192>>2]|0;
         $194 = $190 ^ -1;
         $195 = $193 ^ -1;
         $196 = HEAP16[$eqOpMask>>1]|0;
         $197 = $196&65535;
         $198 = (_sqlite3WhereFindTerm($182,$183,$187,$194,$195,$197,0)|0);
         HEAP32[$pTerm>>2] = $198;
         $199 = HEAP32[$pTerm>>2]|0;
         $200 = ($199|0)==(0|0);
         if (!($200)) {
          $201 = HEAP32[$pTerm>>2]|0;
          $202 = ((($201)) + 12|0);
          $203 = HEAP16[$202>>1]|0;
          $204 = $203&65535;
          $205 = ($204|0)==(1);
          if ($205) {
           HEAP32[$j>>2] = 0;
           while(1) {
            $206 = HEAP32[$j>>2]|0;
            $207 = HEAP32[$pLoop>>2]|0;
            $208 = ((($207)) + 40|0);
            $209 = HEAP16[$208>>1]|0;
            $210 = $209&65535;
            $211 = ($206|0)<($210|0);
            if (!($211)) {
             break;
            }
            $212 = HEAP32[$pTerm>>2]|0;
            $213 = HEAP32[$j>>2]|0;
            $214 = HEAP32[$pLoop>>2]|0;
            $215 = ((($214)) + 48|0);
            $216 = HEAP32[$215>>2]|0;
            $217 = (($216) + ($213<<2)|0);
            $218 = HEAP32[$217>>2]|0;
            $219 = ($212|0)!=($218|0);
            if (!($219)) {
             break;
            }
            $220 = HEAP32[$j>>2]|0;
            $221 = (($220) + 1)|0;
            HEAP32[$j>>2] = $221;
           }
           $222 = HEAP32[$j>>2]|0;
           $223 = HEAP32[$pLoop>>2]|0;
           $224 = ((($223)) + 40|0);
           $225 = HEAP16[$224>>1]|0;
           $226 = $225&65535;
           $227 = ($222|0)>=($226|0);
           if ($227) {
            break;
           }
          }
          $228 = HEAP32[$pTerm>>2]|0;
          $229 = ((($228)) + 12|0);
          $230 = HEAP16[$229>>1]|0;
          $231 = $230&65535;
          $232 = $231 & 130;
          $233 = ($232|0)!=(0);
          do {
           if ($233) {
            $234 = HEAP32[$pOBExpr>>2]|0;
            $235 = ((($234)) + 32|0);
            $236 = HEAP16[$235>>1]|0;
            $237 = $236 << 16 >> 16;
            $238 = ($237|0)>=(0);
            if (!($238)) {
             break;
            }
            $239 = HEAP32[$1>>2]|0;
            $240 = HEAP32[$239>>2]|0;
            $241 = HEAP32[$i>>2]|0;
            $242 = HEAP32[$2>>2]|0;
            $243 = ((($242)) + 4|0);
            $244 = HEAP32[$243>>2]|0;
            $245 = (($244) + (($241*20)|0)|0);
            $246 = HEAP32[$245>>2]|0;
            $247 = (_sqlite3ExprCollSeq($240,$246)|0);
            HEAP32[$pColl>>2] = $247;
            $248 = HEAP32[$pColl>>2]|0;
            $249 = ($248|0)!=(0|0);
            if (!($249)) {
             $250 = HEAP32[$db>>2]|0;
             $251 = ((($250)) + 8|0);
             $252 = HEAP32[$251>>2]|0;
             HEAP32[$pColl>>2] = $252;
            }
            $253 = HEAP32[$pColl>>2]|0;
            $254 = HEAP32[$253>>2]|0;
            HEAP32[$z1>>2] = $254;
            $255 = HEAP32[$1>>2]|0;
            $256 = HEAP32[$255>>2]|0;
            $257 = HEAP32[$pTerm>>2]|0;
            $258 = HEAP32[$257>>2]|0;
            $259 = (_sqlite3ExprCollSeq($256,$258)|0);
            HEAP32[$pColl>>2] = $259;
            $260 = HEAP32[$pColl>>2]|0;
            $261 = ($260|0)!=(0|0);
            if (!($261)) {
             $262 = HEAP32[$db>>2]|0;
             $263 = ((($262)) + 8|0);
             $264 = HEAP32[$263>>2]|0;
             HEAP32[$pColl>>2] = $264;
            }
            $265 = HEAP32[$pColl>>2]|0;
            $266 = HEAP32[$265>>2]|0;
            HEAP32[$z2>>2] = $266;
            $267 = HEAP32[$z1>>2]|0;
            $268 = HEAP32[$z2>>2]|0;
            $269 = (_sqlite3StrICmp($267,$268)|0);
            $270 = ($269|0)!=(0);
            if ($270) {
             break L30;
            }
           }
          } while(0);
          $271 = HEAP32[$i>>2]|0;
          $272 = (_bitshift64Shl(1,0,($271|0))|0);
          $273 = tempRet0;
          $274 = $obSat;
          $275 = $274;
          $276 = HEAP32[$275>>2]|0;
          $277 = (($274) + 4)|0;
          $278 = $277;
          $279 = HEAP32[$278>>2]|0;
          $280 = $276 | $272;
          $281 = $279 | $273;
          $282 = $obSat;
          $283 = $282;
          HEAP32[$283>>2] = $280;
          $284 = (($282) + 4)|0;
          $285 = $284;
          HEAP32[$285>>2] = $281;
         }
        }
       }
      }
     } while(0);
     $286 = HEAP32[$i>>2]|0;
     $287 = (($286) + 1)|0;
     HEAP32[$i>>2] = $287;
    }
    $288 = HEAP32[$pLoop>>2]|0;
    $289 = ((($288)) + 36|0);
    $290 = HEAP32[$289>>2]|0;
    $291 = $290 & 4096;
    $292 = ($291|0)==(0);
    if ($292) {
     $293 = HEAP32[$pLoop>>2]|0;
     $294 = ((($293)) + 36|0);
     $295 = HEAP32[$294>>2]|0;
     $296 = $295 & 256;
     $297 = ($296|0)!=(0);
     if ($297) {
      HEAP32[$pIndex>>2] = 0;
      HEAP16[$nKeyCol>>1] = 0;
      HEAP16[$nColumn>>1] = 1;
     } else {
      $298 = HEAP32[$pLoop>>2]|0;
      $299 = ((($298)) + 24|0);
      $300 = ((($299)) + 8|0);
      $301 = HEAP32[$300>>2]|0;
      HEAP32[$pIndex>>2] = $301;
      $302 = ($301|0)==(0|0);
      if ($302) {
       label = 46;
       break L13;
      }
      $303 = HEAP32[$pIndex>>2]|0;
      $304 = ((($303)) + 55|0);
      $305 = HEAP8[$304>>0]|0;
      $306 = ($305&255) >>> 2;
      $307 = $306 & 1;
      $308 = $307&255;
      $309 = ($308|0)!=(0);
      if ($309) {
       label = 46;
       break L13;
      }
      $310 = HEAP32[$pIndex>>2]|0;
      $311 = ((($310)) + 50|0);
      $312 = HEAP16[$311>>1]|0;
      HEAP16[$nKeyCol>>1] = $312;
      $313 = HEAP32[$pIndex>>2]|0;
      $314 = ((($313)) + 52|0);
      $315 = HEAP16[$314>>1]|0;
      HEAP16[$nColumn>>1] = $315;
      $316 = HEAP32[$pIndex>>2]|0;
      $317 = ((($316)) + 54|0);
      $318 = HEAP8[$317>>0]|0;
      $319 = $318&255;
      $320 = ($319|0)!=(0);
      $321 = $320&1;
      $322 = $321&255;
      HEAP8[$isOrderDistinct>>0] = $322;
     }
     HEAP8[$revSet>>0] = 0;
     HEAP8[$rev>>0] = 0;
     HEAP8[$distinctColumns>>0] = 0;
     HEAP32[$j>>2] = 0;
     while(1) {
      $323 = HEAP32[$j>>2]|0;
      $324 = HEAP16[$nColumn>>1]|0;
      $325 = $324&65535;
      $326 = ($323|0)<($325|0);
      if (!($326)) {
       break;
      }
      HEAP8[$bOnce>>0] = 1;
      $327 = HEAP32[$j>>2]|0;
      $328 = HEAP32[$pLoop>>2]|0;
      $329 = ((($328)) + 24|0);
      $330 = HEAP16[$329>>1]|0;
      $331 = $330&65535;
      $332 = ($327|0)<($331|0);
      L66: do {
       if ($332) {
        $333 = HEAP32[$j>>2]|0;
        $334 = HEAP32[$pLoop>>2]|0;
        $335 = ((($334)) + 42|0);
        $336 = HEAP16[$335>>1]|0;
        $337 = $336&65535;
        $338 = ($333|0)>=($337|0);
        if ($338) {
         $339 = HEAP32[$j>>2]|0;
         $340 = HEAP32[$pLoop>>2]|0;
         $341 = ((($340)) + 48|0);
         $342 = HEAP32[$341>>2]|0;
         $343 = (($342) + ($339<<2)|0);
         $344 = HEAP32[$343>>2]|0;
         $345 = ((($344)) + 12|0);
         $346 = HEAP16[$345>>1]|0;
         HEAP16[$eOp>>1] = $346;
         $347 = HEAP16[$eOp>>1]|0;
         $348 = $347&65535;
         $349 = HEAP16[$eqOpMask>>1]|0;
         $350 = $349&65535;
         $351 = $348 & $350;
         $352 = ($351|0)!=(0);
         $353 = HEAP16[$eOp>>1]|0;
         $354 = $353&65535;
         if ($352) {
          $355 = $354 & 256;
          $356 = ($355|0)!=(0);
          if (!($356)) {
           break;
          }
          HEAP8[$isOrderDistinct>>0] = 0;
          break;
         }
         $357 = $354 & 1;
         $358 = ($357|0)!=(0);
         if (!($358)) {
          label = 61;
          break;
         }
         $359 = HEAP32[$j>>2]|0;
         $360 = HEAP32[$pLoop>>2]|0;
         $361 = ((($360)) + 48|0);
         $362 = HEAP32[$361>>2]|0;
         $363 = (($362) + ($359<<2)|0);
         $364 = HEAP32[$363>>2]|0;
         $365 = HEAP32[$364>>2]|0;
         HEAP32[$pX>>2] = $365;
         $366 = HEAP32[$j>>2]|0;
         $367 = (($366) + 1)|0;
         HEAP32[$i>>2] = $367;
         while(1) {
          $368 = HEAP32[$i>>2]|0;
          $369 = HEAP32[$pLoop>>2]|0;
          $370 = ((($369)) + 24|0);
          $371 = HEAP16[$370>>1]|0;
          $372 = $371&65535;
          $373 = ($368|0)<($372|0);
          if (!($373)) {
           label = 61;
           break L66;
          }
          $374 = HEAP32[$i>>2]|0;
          $375 = HEAP32[$pLoop>>2]|0;
          $376 = ((($375)) + 48|0);
          $377 = HEAP32[$376>>2]|0;
          $378 = (($377) + ($374<<2)|0);
          $379 = HEAP32[$378>>2]|0;
          $380 = HEAP32[$379>>2]|0;
          $381 = HEAP32[$pX>>2]|0;
          $382 = ($380|0)==($381|0);
          if ($382) {
           break;
          }
          $383 = HEAP32[$i>>2]|0;
          $384 = (($383) + 1)|0;
          HEAP32[$i>>2] = $384;
         }
         HEAP8[$bOnce>>0] = 0;
         label = 61;
        } else {
         label = 61;
        }
       } else {
        label = 61;
       }
      } while(0);
      if ((label|0) == 61) {
       label = 0;
       $385 = HEAP32[$pIndex>>2]|0;
       $386 = ($385|0)!=(0|0);
       do {
        if ($386) {
         $387 = HEAP32[$j>>2]|0;
         $388 = HEAP32[$pIndex>>2]|0;
         $389 = ((($388)) + 4|0);
         $390 = HEAP32[$389>>2]|0;
         $391 = (($390) + ($387<<1)|0);
         $392 = HEAP16[$391>>1]|0;
         $393 = $392 << 16 >> 16;
         HEAP32[$iColumn>>2] = $393;
         $394 = HEAP32[$j>>2]|0;
         $395 = HEAP32[$pIndex>>2]|0;
         $396 = ((($395)) + 28|0);
         $397 = HEAP32[$396>>2]|0;
         $398 = (($397) + ($394)|0);
         $399 = HEAP8[$398>>0]|0;
         HEAP8[$revIdx>>0] = $399;
         $400 = HEAP32[$iColumn>>2]|0;
         $401 = HEAP32[$pIndex>>2]|0;
         $402 = ((($401)) + 12|0);
         $403 = HEAP32[$402>>2]|0;
         $404 = ((($403)) + 32|0);
         $405 = HEAP16[$404>>1]|0;
         $406 = $405 << 16 >> 16;
         $407 = ($400|0)==($406|0);
         if (!($407)) {
          break;
         }
         HEAP32[$iColumn>>2] = -1;
        } else {
         HEAP32[$iColumn>>2] = -1;
         HEAP8[$revIdx>>0] = 0;
        }
       } while(0);
       $408 = HEAP8[$isOrderDistinct>>0]|0;
       $409 = $408&255;
       $410 = ($409|0)!=(0);
       $411 = HEAP32[$iColumn>>2]|0;
       $412 = ($411|0)>=(0);
       $or$cond = $410 & $412;
       do {
        if ($or$cond) {
         $413 = HEAP32[$j>>2]|0;
         $414 = HEAP32[$pLoop>>2]|0;
         $415 = ((($414)) + 24|0);
         $416 = HEAP16[$415>>1]|0;
         $417 = $416&65535;
         $418 = ($413|0)>=($417|0);
         if (!($418)) {
          break;
         }
         $419 = HEAP32[$iColumn>>2]|0;
         $420 = HEAP32[$pIndex>>2]|0;
         $421 = ((($420)) + 12|0);
         $422 = HEAP32[$421>>2]|0;
         $423 = ((($422)) + 4|0);
         $424 = HEAP32[$423>>2]|0;
         $425 = (($424) + ($419<<4)|0);
         $426 = ((($425)) + 12|0);
         $427 = HEAP8[$426>>0]|0;
         $428 = $427&255;
         $429 = ($428|0)==(0);
         if (!($429)) {
          break;
         }
         HEAP8[$isOrderDistinct>>0] = 0;
        }
       } while(0);
       HEAP8[$isMatch>>0] = 0;
       HEAP32[$i>>2] = 0;
       L91: while(1) {
        $430 = HEAP8[$bOnce>>0]|0;
        $431 = $430&255;
        $432 = ($431|0)!=(0);
        if (!($432)) {
         break;
        }
        $433 = HEAP32[$i>>2]|0;
        $434 = HEAP16[$nOrderBy>>1]|0;
        $435 = $434&65535;
        $436 = ($433|0)<($435|0);
        if (!($436)) {
         break;
        }
        $437 = HEAP32[$i>>2]|0;
        $438 = (_bitshift64Shl(1,0,($437|0))|0);
        $439 = tempRet0;
        $440 = $obSat;
        $441 = $440;
        $442 = HEAP32[$441>>2]|0;
        $443 = (($440) + 4)|0;
        $444 = $443;
        $445 = HEAP32[$444>>2]|0;
        $446 = $438 & $442;
        $447 = $439 & $445;
        $448 = ($446|0)!=(0);
        $449 = ($447|0)!=(0);
        $450 = $448 | $449;
        do {
         if (!($450)) {
          $451 = HEAP32[$i>>2]|0;
          $452 = HEAP32[$2>>2]|0;
          $453 = ((($452)) + 4|0);
          $454 = HEAP32[$453>>2]|0;
          $455 = (($454) + (($451*20)|0)|0);
          $456 = HEAP32[$455>>2]|0;
          $457 = (_sqlite3ExprSkipCollate($456)|0);
          HEAP32[$pOBExpr>>2] = $457;
          $458 = HEAP16[$4>>1]|0;
          $459 = $458&65535;
          $460 = $459 & 192;
          $461 = ($460|0)==(0);
          if ($461) {
           HEAP8[$bOnce>>0] = 0;
          }
          $462 = HEAP32[$iColumn>>2]|0;
          $463 = ($462|0)>=(-1);
          $464 = HEAP32[$pOBExpr>>2]|0;
          if ($463) {
           $465 = HEAP8[$464>>0]|0;
           $466 = $465&255;
           $467 = ($466|0)!=(152);
           if ($467) {
            break;
           }
           $468 = HEAP32[$pOBExpr>>2]|0;
           $469 = ((($468)) + 28|0);
           $470 = HEAP32[$469>>2]|0;
           $471 = HEAP32[$iCur>>2]|0;
           $472 = ($470|0)!=($471|0);
           if ($472) {
            break;
           }
           $473 = HEAP32[$pOBExpr>>2]|0;
           $474 = ((($473)) + 32|0);
           $475 = HEAP16[$474>>1]|0;
           $476 = $475 << 16 >> 16;
           $477 = HEAP32[$iColumn>>2]|0;
           $478 = ($476|0)!=($477|0);
           if ($478) {
            break;
           }
          } else {
           $479 = HEAP32[$j>>2]|0;
           $480 = HEAP32[$pIndex>>2]|0;
           $481 = ((($480)) + 40|0);
           $482 = HEAP32[$481>>2]|0;
           $483 = ((($482)) + 4|0);
           $484 = HEAP32[$483>>2]|0;
           $485 = (($484) + (($479*20)|0)|0);
           $486 = HEAP32[$485>>2]|0;
           $487 = HEAP32[$iCur>>2]|0;
           $488 = (_sqlite3ExprCompare($464,$486,$487)|0);
           $489 = ($488|0)!=(0);
           if ($489) {
            break;
           }
          }
          $490 = HEAP32[$iColumn>>2]|0;
          $491 = ($490|0)>=(0);
          if (!($491)) {
           label = 84;
           break L91;
          }
          $492 = HEAP32[$1>>2]|0;
          $493 = HEAP32[$492>>2]|0;
          $494 = HEAP32[$i>>2]|0;
          $495 = HEAP32[$2>>2]|0;
          $496 = ((($495)) + 4|0);
          $497 = HEAP32[$496>>2]|0;
          $498 = (($497) + (($494*20)|0)|0);
          $499 = HEAP32[$498>>2]|0;
          $500 = (_sqlite3ExprCollSeq($493,$499)|0);
          HEAP32[$pColl>>2] = $500;
          $501 = HEAP32[$pColl>>2]|0;
          $502 = ($501|0)!=(0|0);
          if (!($502)) {
           $503 = HEAP32[$db>>2]|0;
           $504 = ((($503)) + 8|0);
           $505 = HEAP32[$504>>2]|0;
           HEAP32[$pColl>>2] = $505;
          }
          $506 = HEAP32[$pColl>>2]|0;
          $507 = HEAP32[$506>>2]|0;
          $508 = HEAP32[$j>>2]|0;
          $509 = HEAP32[$pIndex>>2]|0;
          $510 = ((($509)) + 32|0);
          $511 = HEAP32[$510>>2]|0;
          $512 = (($511) + ($508<<2)|0);
          $513 = HEAP32[$512>>2]|0;
          $514 = (_sqlite3StrICmp($507,$513)|0);
          $515 = ($514|0)!=(0);
          if (!($515)) {
           label = 84;
           break L91;
          }
         }
        } while(0);
        $516 = HEAP32[$i>>2]|0;
        $517 = (($516) + 1)|0;
        HEAP32[$i>>2] = $517;
       }
       if ((label|0) == 84) {
        label = 0;
        HEAP8[$isMatch>>0] = 1;
       }
       $518 = HEAP8[$isMatch>>0]|0;
       $519 = $518&255;
       $520 = ($519|0)!=(0);
       do {
        if ($520) {
         $521 = HEAP16[$4>>1]|0;
         $522 = $521&65535;
         $523 = $522 & 64;
         $524 = ($523|0)==(0);
         if (!($524)) {
          break;
         }
         $525 = HEAP8[$revSet>>0]|0;
         $526 = ($525<<24>>24)!=(0);
         if ($526) {
          $527 = HEAP8[$rev>>0]|0;
          $528 = $527&255;
          $529 = HEAP8[$revIdx>>0]|0;
          $530 = $529&255;
          $531 = $528 ^ $530;
          $532 = HEAP32[$i>>2]|0;
          $533 = HEAP32[$2>>2]|0;
          $534 = ((($533)) + 4|0);
          $535 = HEAP32[$534>>2]|0;
          $536 = (($535) + (($532*20)|0)|0);
          $537 = ((($536)) + 12|0);
          $538 = HEAP8[$537>>0]|0;
          $539 = $538&255;
          $540 = ($531|0)!=($539|0);
          if (!($540)) {
           break;
          }
          HEAP8[$isMatch>>0] = 0;
          break;
         }
         $541 = HEAP8[$revIdx>>0]|0;
         $542 = $541&255;
         $543 = HEAP32[$i>>2]|0;
         $544 = HEAP32[$2>>2]|0;
         $545 = ((($544)) + 4|0);
         $546 = HEAP32[$545>>2]|0;
         $547 = (($546) + (($543*20)|0)|0);
         $548 = ((($547)) + 12|0);
         $549 = HEAP8[$548>>0]|0;
         $550 = $549&255;
         $551 = $542 ^ $550;
         $552 = $551&255;
         HEAP8[$rev>>0] = $552;
         $553 = HEAP8[$rev>>0]|0;
         $554 = ($553<<24>>24)!=(0);
         if ($554) {
          $555 = HEAP32[$iLoop>>2]|0;
          $556 = (_bitshift64Shl(1,0,($555|0))|0);
          $557 = tempRet0;
          $558 = HEAP32[$7>>2]|0;
          $559 = $558;
          $560 = $559;
          $561 = HEAP32[$560>>2]|0;
          $562 = (($559) + 4)|0;
          $563 = $562;
          $564 = HEAP32[$563>>2]|0;
          $565 = $561 | $556;
          $566 = $564 | $557;
          $567 = $558;
          $568 = $567;
          HEAP32[$568>>2] = $565;
          $569 = (($567) + 4)|0;
          $570 = $569;
          HEAP32[$570>>2] = $566;
         }
         HEAP8[$revSet>>0] = 1;
        }
       } while(0);
       $571 = HEAP8[$isMatch>>0]|0;
       $572 = ($571<<24>>24)!=(0);
       if (!($572)) {
        label = 98;
        break;
       }
       $573 = HEAP32[$iColumn>>2]|0;
       $574 = ($573|0)==(-1);
       if ($574) {
        HEAP8[$distinctColumns>>0] = 1;
       }
       $575 = HEAP32[$i>>2]|0;
       $576 = (_bitshift64Shl(1,0,($575|0))|0);
       $577 = tempRet0;
       $578 = $obSat;
       $579 = $578;
       $580 = HEAP32[$579>>2]|0;
       $581 = (($578) + 4)|0;
       $582 = $581;
       $583 = HEAP32[$582>>2]|0;
       $584 = $580 | $576;
       $585 = $583 | $577;
       $586 = $obSat;
       $587 = $586;
       HEAP32[$587>>2] = $584;
       $588 = (($586) + 4)|0;
       $589 = $588;
       HEAP32[$589>>2] = $585;
      }
      $596 = HEAP32[$j>>2]|0;
      $597 = (($596) + 1)|0;
      HEAP32[$j>>2] = $597;
     }
     do {
      if ((label|0) == 98) {
       label = 0;
       $590 = HEAP32[$j>>2]|0;
       $591 = ($590|0)==(0);
       if (!($591)) {
        $592 = HEAP32[$j>>2]|0;
        $593 = HEAP16[$nKeyCol>>1]|0;
        $594 = $593&65535;
        $595 = ($592|0)<($594|0);
        if (!($595)) {
         break;
        }
       }
       HEAP8[$isOrderDistinct>>0] = 0;
      }
     } while(0);
     $598 = HEAP8[$distinctColumns>>0]|0;
     $599 = ($598<<24>>24)!=(0);
     if ($599) {
      HEAP8[$isOrderDistinct>>0] = 1;
     }
    }
    $600 = HEAP8[$isOrderDistinct>>0]|0;
    $601 = ($600<<24>>24)!=(0);
    if ($601) {
     $602 = HEAP32[$pLoop>>2]|0;
     $603 = ((($602)) + 8|0);
     $604 = $603;
     $605 = $604;
     $606 = HEAP32[$605>>2]|0;
     $607 = (($604) + 4)|0;
     $608 = $607;
     $609 = HEAP32[$608>>2]|0;
     $610 = $orderDistinctMask;
     $611 = $610;
     $612 = HEAP32[$611>>2]|0;
     $613 = (($610) + 4)|0;
     $614 = $613;
     $615 = HEAP32[$614>>2]|0;
     $616 = $612 | $606;
     $617 = $615 | $609;
     $618 = $orderDistinctMask;
     $619 = $618;
     HEAP32[$619>>2] = $616;
     $620 = (($618) + 4)|0;
     $621 = $620;
     HEAP32[$621>>2] = $617;
     HEAP32[$i>>2] = 0;
     while(1) {
      $622 = HEAP32[$i>>2]|0;
      $623 = HEAP16[$nOrderBy>>1]|0;
      $624 = $623&65535;
      $625 = ($622|0)<($624|0);
      if (!($625)) {
       break L24;
      }
      $626 = HEAP32[$i>>2]|0;
      $627 = (_bitshift64Shl(1,0,($626|0))|0);
      $628 = tempRet0;
      $629 = $obSat;
      $630 = $629;
      $631 = HEAP32[$630>>2]|0;
      $632 = (($629) + 4)|0;
      $633 = $632;
      $634 = HEAP32[$633>>2]|0;
      $635 = $627 & $631;
      $636 = $628 & $634;
      $637 = ($635|0)!=(0);
      $638 = ($636|0)!=(0);
      $639 = $637 | $638;
      do {
       if (!($639)) {
        $640 = HEAP32[$i>>2]|0;
        $641 = HEAP32[$2>>2]|0;
        $642 = ((($641)) + 4|0);
        $643 = HEAP32[$642>>2]|0;
        $644 = (($643) + (($640*20)|0)|0);
        $645 = HEAP32[$644>>2]|0;
        HEAP32[$p>>2] = $645;
        $646 = HEAP32[$1>>2]|0;
        $647 = ((($646)) + 488|0);
        $648 = HEAP32[$p>>2]|0;
        $649 = (_sqlite3WhereExprUsage($647,$648)|0);
        $650 = tempRet0;
        $651 = $mTerm;
        $652 = $651;
        HEAP32[$652>>2] = $649;
        $653 = (($651) + 4)|0;
        $654 = $653;
        HEAP32[$654>>2] = $650;
        $655 = $mTerm;
        $656 = $655;
        $657 = HEAP32[$656>>2]|0;
        $658 = (($655) + 4)|0;
        $659 = $658;
        $660 = HEAP32[$659>>2]|0;
        $661 = ($657|0)==(0);
        $662 = ($660|0)==(0);
        $663 = $661 & $662;
        if ($663) {
         $664 = HEAP32[$p>>2]|0;
         $665 = (_sqlite3ExprIsConstant($664)|0);
         $666 = ($665|0)!=(0);
         if (!($666)) {
          break;
         }
        }
        $667 = $mTerm;
        $668 = $667;
        $669 = HEAP32[$668>>2]|0;
        $670 = (($667) + 4)|0;
        $671 = $670;
        $672 = HEAP32[$671>>2]|0;
        $673 = $orderDistinctMask;
        $674 = $673;
        $675 = HEAP32[$674>>2]|0;
        $676 = (($673) + 4)|0;
        $677 = $676;
        $678 = HEAP32[$677>>2]|0;
        $679 = $675 ^ -1;
        $680 = $678 ^ -1;
        $681 = $669 & $679;
        $682 = $672 & $680;
        $683 = ($681|0)==(0);
        $684 = ($682|0)==(0);
        $685 = $683 & $684;
        if ($685) {
         $686 = HEAP32[$i>>2]|0;
         $687 = (_bitshift64Shl(1,0,($686|0))|0);
         $688 = tempRet0;
         $689 = $obSat;
         $690 = $689;
         $691 = HEAP32[$690>>2]|0;
         $692 = (($689) + 4)|0;
         $693 = $692;
         $694 = HEAP32[$693>>2]|0;
         $695 = $691 | $687;
         $696 = $694 | $688;
         $697 = $obSat;
         $698 = $697;
         HEAP32[$698>>2] = $695;
         $699 = (($697) + 4)|0;
         $700 = $699;
         HEAP32[$700>>2] = $696;
        }
       }
      } while(0);
      $701 = HEAP32[$i>>2]|0;
      $702 = (($701) + 1)|0;
      HEAP32[$i>>2] = $702;
     }
    }
   }
  } while(0);
  $703 = HEAP32[$iLoop>>2]|0;
  $704 = (($703) + 1)|0;
  HEAP32[$iLoop>>2] = $704;
 }
 if ((label|0) == 18) {
  $123 = ((($122)) + 24|0);
  $124 = ((($123)) + 5|0);
  $125 = HEAP8[$124>>0]|0;
  $126 = ($125<<24>>24)!=(0);
  if ($126) {
   $127 = $obDone;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = (($127) + 4)|0;
   $131 = $130;
   $132 = HEAP32[$131>>2]|0;
   $133 = $obSat;
   $134 = $133;
   HEAP32[$134>>2] = $129;
   $135 = (($133) + 4)|0;
   $136 = $135;
   HEAP32[$136>>2] = $132;
  }
 }
 else if ((label|0) == 46) {
  HEAP8[$0>>0] = 0;
  $764 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($764|0);
 }
 $705 = $obSat;
 $706 = $705;
 $707 = HEAP32[$706>>2]|0;
 $708 = (($705) + 4)|0;
 $709 = $708;
 $710 = HEAP32[$709>>2]|0;
 $711 = $obDone;
 $712 = $711;
 $713 = HEAP32[$712>>2]|0;
 $714 = (($711) + 4)|0;
 $715 = $714;
 $716 = HEAP32[$715>>2]|0;
 $717 = ($707|0)==($713|0);
 $718 = ($710|0)==($716|0);
 $719 = $717 & $718;
 if ($719) {
  $720 = HEAP16[$nOrderBy>>1]|0;
  $721 = $720&255;
  HEAP8[$0>>0] = $721;
  $764 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($764|0);
 }
 $722 = HEAP8[$isOrderDistinct>>0]|0;
 $723 = ($722<<24>>24)!=(0);
 if ($723) {
  HEAP8[$0>>0] = -1;
  $764 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($764|0);
 }
 $724 = HEAP16[$nOrderBy>>1]|0;
 $725 = $724&65535;
 $726 = (($725) - 1)|0;
 HEAP32[$i>>2] = $726;
 while(1) {
  $727 = HEAP32[$i>>2]|0;
  $728 = ($727|0)>(0);
  if (!($728)) {
   label = 122;
   break;
  }
  $729 = HEAP32[$i>>2]|0;
  $730 = (_bitshift64Shl(1,0,($729|0))|0);
  $731 = tempRet0;
  $732 = (_i64Subtract(($730|0),($731|0),1,0)|0);
  $733 = tempRet0;
  $734 = $m;
  $735 = $734;
  HEAP32[$735>>2] = $732;
  $736 = (($734) + 4)|0;
  $737 = $736;
  HEAP32[$737>>2] = $733;
  $738 = $obSat;
  $739 = $738;
  $740 = HEAP32[$739>>2]|0;
  $741 = (($738) + 4)|0;
  $742 = $741;
  $743 = HEAP32[$742>>2]|0;
  $744 = $m;
  $745 = $744;
  $746 = HEAP32[$745>>2]|0;
  $747 = (($744) + 4)|0;
  $748 = $747;
  $749 = HEAP32[$748>>2]|0;
  $750 = $740 & $746;
  $751 = $743 & $749;
  $752 = $m;
  $753 = $752;
  $754 = HEAP32[$753>>2]|0;
  $755 = (($752) + 4)|0;
  $756 = $755;
  $757 = HEAP32[$756>>2]|0;
  $758 = ($750|0)==($754|0);
  $759 = ($751|0)==($757|0);
  $760 = $758 & $759;
  $761 = HEAP32[$i>>2]|0;
  if ($760) {
   label = 120;
   break;
  }
  $763 = (($761) + -1)|0;
  HEAP32[$i>>2] = $763;
 }
 if ((label|0) == 120) {
  $762 = $761&255;
  HEAP8[$0>>0] = $762;
  $764 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($764|0);
 }
 else if ((label|0) == 122) {
  HEAP8[$0>>0] = 0;
  $764 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($764|0);
 }
 return (0)|0;
}
function _whereSortingCost($pWInfo,$nRow,$nOrderBy,$nSorted) {
 $pWInfo = $pWInfo|0;
 $nRow = $nRow|0;
 $nOrderBy = $nOrderBy|0;
 $nSorted = $nSorted|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rScale = 0, $rSortCost = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 16|0;
 $2 = sp + 4|0;
 $3 = sp;
 $rScale = sp + 14|0;
 $rSortCost = sp + 12|0;
 HEAP32[$0>>2] = $pWInfo;
 HEAP16[$1>>1] = $nRow;
 HEAP32[$2>>2] = $nOrderBy;
 HEAP32[$3>>2] = $nSorted;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (($4) - ($5))|0;
 $7 = ($6*100)|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($7|0) / ($8|0))&-1;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3LogEst($9,$11)|0);
 $13 = $12 << 16 >> 16;
 $14 = (($13) - 66)|0;
 $15 = $14&65535;
 HEAP16[$rScale>>1] = $15;
 $16 = HEAP16[$1>>1]|0;
 $17 = $16 << 16 >> 16;
 $18 = HEAP16[$rScale>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = (($17) + ($19))|0;
 $21 = (($20) + 16)|0;
 $22 = $21&65535;
 HEAP16[$rSortCost>>1] = $22;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 40|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26 & 16384;
 $28 = ($27|0)!=(0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP16[$30>>1]|0;
  $32 = $31 << 16 >> 16;
  $33 = HEAP16[$1>>1]|0;
  $34 = $33 << 16 >> 16;
  $35 = ($32|0)<($34|0);
  if ($35) {
   $36 = HEAP32[$0>>2]|0;
   $37 = ((($36)) + 16|0);
   $38 = HEAP16[$37>>1]|0;
   HEAP16[$1>>1] = $38;
  }
 }
 $39 = HEAP16[$1>>1]|0;
 $40 = (_estLog($39)|0);
 $41 = $40 << 16 >> 16;
 $42 = HEAP16[$rSortCost>>1]|0;
 $43 = $42 << 16 >> 16;
 $44 = (($43) + ($41))|0;
 $45 = $44&65535;
 HEAP16[$rSortCost>>1] = $45;
 $46 = HEAP16[$rSortCost>>1]|0;
 STACKTOP = sp;return ($46|0);
}
function _estLog($N) {
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP16[$0>>1] = $N;
 $1 = HEAP16[$0>>1]|0;
 $2 = $1 << 16 >> 16;
 $3 = ($2|0)<=(10);
 if ($3) {
  $12 = 0;
  $11 = $12&65535;
  STACKTOP = sp;return ($11|0);
 }
 $4 = HEAP16[$0>>1]|0;
 $5 = $4 << 16 >> 16;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3LogEst($5,$7)|0);
 $9 = $8 << 16 >> 16;
 $10 = (($9) - 33)|0;
 $12 = $10;
 $11 = $12&65535;
 STACKTOP = sp;return ($11|0);
}
function _whereLoopAddVirtual($pBuilder,$0,$1,$2,$3) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0;
 var $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0;
 var $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0;
 var $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0;
 var $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0;
 var $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0;
 var $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bIn = 0, $i = 0, $mBest = 0, $mBestNoIn = 0, $mNext = 0, $mNoOmit = 0, $mPrev = 0, $mThis = 0, $nConstraint = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pNew = 0, $pParse = 0, $pSrc = 0, $pWC = 0;
 var $pWInfo = 0, $rc = 0, $seenZero = 0, $seenZeroNoIN = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $4 = sp + 108|0;
 $5 = sp + 104|0;
 $6 = sp + 48|0;
 $7 = sp + 40|0;
 $rc = sp + 100|0;
 $pWInfo = sp + 96|0;
 $pParse = sp + 92|0;
 $pWC = sp + 88|0;
 $pSrc = sp + 84|0;
 $p = sp + 80|0;
 $nConstraint = sp + 76|0;
 $bIn = sp + 72|0;
 $pNew = sp + 68|0;
 $mBest = sp + 32|0;
 $mNoOmit = sp + 112|0;
 $seenZero = sp + 64|0;
 $seenZeroNoIN = sp + 60|0;
 $mPrev = sp + 24|0;
 $mBestNoIn = sp + 16|0;
 $i = sp + 56|0;
 $mNext = sp + 8|0;
 $mThis = sp;
 HEAP32[$5>>2] = $pBuilder;
 $8 = $6;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 $12 = $7;
 $13 = $12;
 HEAP32[$13>>2] = $2;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pWInfo>>2] = $17;
 $18 = HEAP32[$pWInfo>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pParse>>2] = $19;
 $20 = HEAP32[$5>>2]|0;
 $21 = ((($20)) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pWC>>2] = $22;
 $23 = HEAP32[$5>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pNew>>2] = $25;
 $26 = HEAP32[$pNew>>2]|0;
 $27 = ((($26)) + 16|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$pWInfo>>2]|0;
 $31 = ((($30)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = (($33) + (($29*72)|0)|0);
 HEAP32[$pSrc>>2] = $34;
 $35 = HEAP32[$pParse>>2]|0;
 $36 = HEAP32[$pWC>>2]|0;
 $37 = $7;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) + 4)|0;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = HEAP32[$pSrc>>2]|0;
 $44 = HEAP32[$5>>2]|0;
 $45 = ((($44)) + 8|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (_allocateIndexInfo($35,$36,$39,$42,$43,$46,$mNoOmit)|0);
 HEAP32[$p>>2] = $47;
 $48 = HEAP32[$p>>2]|0;
 $49 = ($48|0)==(0|0);
 if ($49) {
  HEAP32[$4>>2] = 7;
  $387 = HEAP32[$4>>2]|0;
  STACKTOP = sp;return ($387|0);
 }
 $50 = HEAP32[$pNew>>2]|0;
 $51 = ((($50)) + 18|0);
 HEAP16[$51>>1] = 0;
 $52 = HEAP32[$pNew>>2]|0;
 $53 = ((($52)) + 36|0);
 HEAP32[$53>>2] = 1024;
 $54 = HEAP32[$pNew>>2]|0;
 $55 = ((($54)) + 40|0);
 HEAP16[$55>>1] = 0;
 $56 = HEAP32[$pNew>>2]|0;
 $57 = ((($56)) + 24|0);
 $58 = ((($57)) + 4|0);
 HEAP8[$58>>0] = 0;
 $59 = HEAP32[$p>>2]|0;
 $60 = HEAP32[$59>>2]|0;
 HEAP32[$nConstraint>>2] = $60;
 $61 = HEAP32[$pParse>>2]|0;
 $62 = HEAP32[$61>>2]|0;
 $63 = HEAP32[$pNew>>2]|0;
 $64 = HEAP32[$nConstraint>>2]|0;
 $65 = (_whereLoopResize($62,$63,$64)|0);
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = HEAP32[$pParse>>2]|0;
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($68,$69);
  HEAP32[$4>>2] = 7;
  $387 = HEAP32[$4>>2]|0;
  STACKTOP = sp;return ($387|0);
 }
 $70 = HEAP32[$5>>2]|0;
 $71 = $6;
 $72 = $71;
 $73 = HEAP32[$72>>2]|0;
 $74 = (($71) + 4)|0;
 $75 = $74;
 $76 = HEAP32[$75>>2]|0;
 $77 = HEAP32[$p>>2]|0;
 $78 = HEAP16[$mNoOmit>>1]|0;
 $79 = (_whereLoopAddVirtualOne($70,$73,$76,-1,-1,0,$77,$78,$bIn)|0);
 HEAP32[$rc>>2] = $79;
 $80 = HEAP32[$rc>>2]|0;
 $81 = ($80|0)==(0);
 if ($81) {
  $82 = HEAP32[$pNew>>2]|0;
  $83 = $82;
  $84 = $83;
  $85 = HEAP32[$84>>2]|0;
  $86 = (($83) + 4)|0;
  $87 = $86;
  $88 = HEAP32[$87>>2]|0;
  $89 = $6;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = $91 ^ -1;
  $96 = $94 ^ -1;
  $97 = $85 & $95;
  $98 = $88 & $96;
  $99 = $mBest;
  $100 = $99;
  HEAP32[$100>>2] = $97;
  $101 = (($99) + 4)|0;
  $102 = $101;
  HEAP32[$102>>2] = $98;
  $103 = ($97|0)!=(0);
  $104 = ($98|0)!=(0);
  $105 = $103 | $104;
  if ($105) {
   HEAP32[$seenZero>>2] = 0;
   HEAP32[$seenZeroNoIN>>2] = 0;
   $106 = $mPrev;
   $107 = $106;
   HEAP32[$107>>2] = 0;
   $108 = (($106) + 4)|0;
   $109 = $108;
   HEAP32[$109>>2] = 0;
   $110 = $mBestNoIn;
   $111 = $110;
   HEAP32[$111>>2] = 0;
   $112 = (($110) + 4)|0;
   $113 = $112;
   HEAP32[$113>>2] = 0;
   $114 = HEAP32[$bIn>>2]|0;
   $115 = ($114|0)!=(0);
   if ($115) {
    $116 = HEAP32[$5>>2]|0;
    $117 = $6;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = (($117) + 4)|0;
    $121 = $120;
    $122 = HEAP32[$121>>2]|0;
    $123 = HEAP32[$p>>2]|0;
    $124 = HEAP16[$mNoOmit>>1]|0;
    $125 = (_whereLoopAddVirtualOne($116,$119,$122,-1,-1,1,$123,$124,$bIn)|0);
    HEAP32[$rc>>2] = $125;
    $126 = HEAP32[$pNew>>2]|0;
    $127 = $126;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = $6;
    $134 = $133;
    $135 = HEAP32[$134>>2]|0;
    $136 = (($133) + 4)|0;
    $137 = $136;
    $138 = HEAP32[$137>>2]|0;
    $139 = $135 ^ -1;
    $140 = $138 ^ -1;
    $141 = $129 & $139;
    $142 = $132 & $140;
    $143 = $mBestNoIn;
    $144 = $143;
    HEAP32[$144>>2] = $141;
    $145 = (($143) + 4)|0;
    $146 = $145;
    HEAP32[$146>>2] = $142;
    $147 = $mBestNoIn;
    $148 = $147;
    $149 = HEAP32[$148>>2]|0;
    $150 = (($147) + 4)|0;
    $151 = $150;
    $152 = HEAP32[$151>>2]|0;
    $153 = ($149|0)==(0);
    $154 = ($152|0)==(0);
    $155 = $153 & $154;
    if ($155) {
     HEAP32[$seenZero>>2] = 1;
     HEAP32[$seenZeroNoIN>>2] = 1;
    }
   }
   while(1) {
    $156 = HEAP32[$rc>>2]|0;
    $157 = ($156|0)==(0);
    if (!($157)) {
     break;
    }
    $158 = $mNext;
    $159 = $158;
    HEAP32[$159>>2] = -1;
    $160 = (($158) + 4)|0;
    $161 = $160;
    HEAP32[$161>>2] = -1;
    HEAP32[$i>>2] = 0;
    while(1) {
     $162 = HEAP32[$i>>2]|0;
     $163 = HEAP32[$nConstraint>>2]|0;
     $164 = ($162|0)<($163|0);
     if (!($164)) {
      break;
     }
     $165 = HEAP32[$i>>2]|0;
     $166 = HEAP32[$p>>2]|0;
     $167 = ((($166)) + 4|0);
     $168 = HEAP32[$167>>2]|0;
     $169 = (($168) + (($165*12)|0)|0);
     $170 = ((($169)) + 8|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = HEAP32[$pWC>>2]|0;
     $173 = ((($172)) + 20|0);
     $174 = HEAP32[$173>>2]|0;
     $175 = (($174) + (($171*48)|0)|0);
     $176 = ((($175)) + 32|0);
     $177 = $176;
     $178 = $177;
     $179 = HEAP32[$178>>2]|0;
     $180 = (($177) + 4)|0;
     $181 = $180;
     $182 = HEAP32[$181>>2]|0;
     $183 = $6;
     $184 = $183;
     $185 = HEAP32[$184>>2]|0;
     $186 = (($183) + 4)|0;
     $187 = $186;
     $188 = HEAP32[$187>>2]|0;
     $189 = $185 ^ -1;
     $190 = $188 ^ -1;
     $191 = $179 & $189;
     $192 = $182 & $190;
     $193 = $mThis;
     $194 = $193;
     HEAP32[$194>>2] = $191;
     $195 = (($193) + 4)|0;
     $196 = $195;
     HEAP32[$196>>2] = $192;
     $197 = $mThis;
     $198 = $197;
     $199 = HEAP32[$198>>2]|0;
     $200 = (($197) + 4)|0;
     $201 = $200;
     $202 = HEAP32[$201>>2]|0;
     $203 = $mPrev;
     $204 = $203;
     $205 = HEAP32[$204>>2]|0;
     $206 = (($203) + 4)|0;
     $207 = $206;
     $208 = HEAP32[$207>>2]|0;
     $209 = ($202>>>0)>($208>>>0);
     $210 = ($199>>>0)>($205>>>0);
     $211 = ($202|0)==($208|0);
     $212 = $211 & $210;
     $213 = $209 | $212;
     if ($213) {
      $214 = $mThis;
      $215 = $214;
      $216 = HEAP32[$215>>2]|0;
      $217 = (($214) + 4)|0;
      $218 = $217;
      $219 = HEAP32[$218>>2]|0;
      $220 = $mNext;
      $221 = $220;
      $222 = HEAP32[$221>>2]|0;
      $223 = (($220) + 4)|0;
      $224 = $223;
      $225 = HEAP32[$224>>2]|0;
      $226 = ($219>>>0)<($225>>>0);
      $227 = ($216>>>0)<($222>>>0);
      $228 = ($219|0)==($225|0);
      $229 = $228 & $227;
      $230 = $226 | $229;
      if ($230) {
       $231 = $mThis;
       $232 = $231;
       $233 = HEAP32[$232>>2]|0;
       $234 = (($231) + 4)|0;
       $235 = $234;
       $236 = HEAP32[$235>>2]|0;
       $237 = $mNext;
       $238 = $237;
       HEAP32[$238>>2] = $233;
       $239 = (($237) + 4)|0;
       $240 = $239;
       HEAP32[$240>>2] = $236;
      }
     }
     $241 = HEAP32[$i>>2]|0;
     $242 = (($241) + 1)|0;
     HEAP32[$i>>2] = $242;
    }
    $243 = $mNext;
    $244 = $243;
    $245 = HEAP32[$244>>2]|0;
    $246 = (($243) + 4)|0;
    $247 = $246;
    $248 = HEAP32[$247>>2]|0;
    $249 = $mPrev;
    $250 = $249;
    HEAP32[$250>>2] = $245;
    $251 = (($249) + 4)|0;
    $252 = $251;
    HEAP32[$252>>2] = $248;
    $253 = $mNext;
    $254 = $253;
    $255 = HEAP32[$254>>2]|0;
    $256 = (($253) + 4)|0;
    $257 = $256;
    $258 = HEAP32[$257>>2]|0;
    $259 = ($255|0)==(-1);
    $260 = ($258|0)==(-1);
    $261 = $259 & $260;
    if ($261) {
     break;
    }
    $262 = $mNext;
    $263 = $262;
    $264 = HEAP32[$263>>2]|0;
    $265 = (($262) + 4)|0;
    $266 = $265;
    $267 = HEAP32[$266>>2]|0;
    $268 = $mBest;
    $269 = $268;
    $270 = HEAP32[$269>>2]|0;
    $271 = (($268) + 4)|0;
    $272 = $271;
    $273 = HEAP32[$272>>2]|0;
    $274 = ($264|0)==($270|0);
    $275 = ($267|0)==($273|0);
    $276 = $274 & $275;
    if ($276) {
     continue;
    }
    $277 = $mNext;
    $278 = $277;
    $279 = HEAP32[$278>>2]|0;
    $280 = (($277) + 4)|0;
    $281 = $280;
    $282 = HEAP32[$281>>2]|0;
    $283 = $mBestNoIn;
    $284 = $283;
    $285 = HEAP32[$284>>2]|0;
    $286 = (($283) + 4)|0;
    $287 = $286;
    $288 = HEAP32[$287>>2]|0;
    $289 = ($279|0)==($285|0);
    $290 = ($282|0)==($288|0);
    $291 = $289 & $290;
    if ($291) {
     continue;
    }
    $292 = HEAP32[$5>>2]|0;
    $293 = $6;
    $294 = $293;
    $295 = HEAP32[$294>>2]|0;
    $296 = (($293) + 4)|0;
    $297 = $296;
    $298 = HEAP32[$297>>2]|0;
    $299 = $mNext;
    $300 = $299;
    $301 = HEAP32[$300>>2]|0;
    $302 = (($299) + 4)|0;
    $303 = $302;
    $304 = HEAP32[$303>>2]|0;
    $305 = $6;
    $306 = $305;
    $307 = HEAP32[$306>>2]|0;
    $308 = (($305) + 4)|0;
    $309 = $308;
    $310 = HEAP32[$309>>2]|0;
    $311 = $301 | $307;
    $312 = $304 | $310;
    $313 = HEAP32[$p>>2]|0;
    $314 = HEAP16[$mNoOmit>>1]|0;
    $315 = (_whereLoopAddVirtualOne($292,$295,$298,$311,$312,0,$313,$314,$bIn)|0);
    HEAP32[$rc>>2] = $315;
    $316 = HEAP32[$pNew>>2]|0;
    $317 = $316;
    $318 = $317;
    $319 = HEAP32[$318>>2]|0;
    $320 = (($317) + 4)|0;
    $321 = $320;
    $322 = HEAP32[$321>>2]|0;
    $323 = $6;
    $324 = $323;
    $325 = HEAP32[$324>>2]|0;
    $326 = (($323) + 4)|0;
    $327 = $326;
    $328 = HEAP32[$327>>2]|0;
    $329 = ($319|0)==($325|0);
    $330 = ($322|0)==($328|0);
    $331 = $329 & $330;
    if (!($331)) {
     continue;
    }
    HEAP32[$seenZero>>2] = 1;
    $332 = HEAP32[$bIn>>2]|0;
    $333 = ($332|0)==(0);
    if (!($333)) {
     continue;
    }
    HEAP32[$seenZeroNoIN>>2] = 1;
   }
   $334 = HEAP32[$rc>>2]|0;
   $335 = ($334|0)==(0);
   $336 = HEAP32[$seenZero>>2]|0;
   $337 = ($336|0)==(0);
   $or$cond = $335 & $337;
   if ($or$cond) {
    $338 = HEAP32[$5>>2]|0;
    $339 = $6;
    $340 = $339;
    $341 = HEAP32[$340>>2]|0;
    $342 = (($339) + 4)|0;
    $343 = $342;
    $344 = HEAP32[$343>>2]|0;
    $345 = $6;
    $346 = $345;
    $347 = HEAP32[$346>>2]|0;
    $348 = (($345) + 4)|0;
    $349 = $348;
    $350 = HEAP32[$349>>2]|0;
    $351 = HEAP32[$p>>2]|0;
    $352 = HEAP16[$mNoOmit>>1]|0;
    $353 = (_whereLoopAddVirtualOne($338,$341,$344,$347,$350,0,$351,$352,$bIn)|0);
    HEAP32[$rc>>2] = $353;
    $354 = HEAP32[$bIn>>2]|0;
    $355 = ($354|0)==(0);
    if ($355) {
     HEAP32[$seenZeroNoIN>>2] = 1;
    }
   }
   $356 = HEAP32[$rc>>2]|0;
   $357 = ($356|0)==(0);
   $358 = HEAP32[$seenZeroNoIN>>2]|0;
   $359 = ($358|0)==(0);
   $or$cond3 = $357 & $359;
   if ($or$cond3) {
    $360 = HEAP32[$5>>2]|0;
    $361 = $6;
    $362 = $361;
    $363 = HEAP32[$362>>2]|0;
    $364 = (($361) + 4)|0;
    $365 = $364;
    $366 = HEAP32[$365>>2]|0;
    $367 = $6;
    $368 = $367;
    $369 = HEAP32[$368>>2]|0;
    $370 = (($367) + 4)|0;
    $371 = $370;
    $372 = HEAP32[$371>>2]|0;
    $373 = HEAP32[$p>>2]|0;
    $374 = HEAP16[$mNoOmit>>1]|0;
    $375 = (_whereLoopAddVirtualOne($360,$363,$366,$369,$372,1,$373,$374,$bIn)|0);
    HEAP32[$rc>>2] = $375;
   }
  }
 }
 $376 = HEAP32[$p>>2]|0;
 $377 = ((($376)) + 28|0);
 $378 = HEAP32[$377>>2]|0;
 $379 = ($378|0)!=(0);
 if ($379) {
  $380 = HEAP32[$p>>2]|0;
  $381 = ((($380)) + 24|0);
  $382 = HEAP32[$381>>2]|0;
  _sqlite3_free($382);
 }
 $383 = HEAP32[$pParse>>2]|0;
 $384 = HEAP32[$383>>2]|0;
 $385 = HEAP32[$p>>2]|0;
 _sqlite3DbFree($384,$385);
 $386 = HEAP32[$rc>>2]|0;
 HEAP32[$4>>2] = $386;
 $387 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($387|0);
}
function _whereLoopAddBtree($pBuilder,$0,$1) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0;
 var $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0;
 var $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0;
 var $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0;
 var $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aiColumnPk = 0, $aiRowEstPk = 0, $b = 0, $iCur = 0, $iSortIdx = 0;
 var $ii = 0, $m = 0, $nLookup = 0, $or$cond = 0, $pFirst = 0, $pNew = 0, $pProbe = 0, $pSrc = 0, $pTab = 0, $pTabList = 0, $pTerm = 0, $pTerm1 = 0, $pWC = 0, $pWC2 = 0, $pWCEnd = 0, $pWInfo = 0, $rLogSize = 0, $rSize = 0, $rc = 0, $sPk = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $2 = sp + 144|0;
 $3 = sp + 8|0;
 $pWInfo = sp + 140|0;
 $pProbe = sp + 136|0;
 $sPk = sp + 80|0;
 $aiRowEstPk = sp + 156|0;
 $aiColumnPk = sp + 154|0;
 $pTabList = sp + 72|0;
 $pSrc = sp + 68|0;
 $pNew = sp + 64|0;
 $rc = sp + 60|0;
 $iSortIdx = sp + 56|0;
 $b = sp + 52|0;
 $rSize = sp + 152|0;
 $rLogSize = sp + 150|0;
 $pWC = sp + 48|0;
 $pTab = sp + 44|0;
 $pFirst = sp + 40|0;
 $pTerm = sp + 36|0;
 $pWCEnd = sp + 32|0;
 $m = sp;
 $nLookup = sp + 148|0;
 $ii = sp + 28|0;
 $iCur = sp + 24|0;
 $pWC2 = sp + 20|0;
 $pTerm1 = sp + 16|0;
 HEAP32[$2>>2] = $pBuilder;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 HEAP16[$aiColumnPk>>1] = -1;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iSortIdx>>2] = 1;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pNew>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pWInfo>>2] = $12;
 $13 = HEAP32[$pWInfo>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTabList>>2] = $15;
 $16 = HEAP32[$pTabList>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = (($17) + (($21*72)|0)|0);
 HEAP32[$pSrc>>2] = $22;
 $23 = HEAP32[$pSrc>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pTab>>2] = $25;
 $26 = HEAP32[$2>>2]|0;
 $27 = ((($26)) + 4|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pWC>>2] = $28;
 $29 = HEAP32[$pSrc>>2]|0;
 $30 = ((($29)) + 68|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)!=(0|0);
 do {
  if ($32) {
   $33 = HEAP32[$pSrc>>2]|0;
   $34 = ((($33)) + 68|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$pProbe>>2] = $35;
  } else {
   $36 = HEAP32[$pTab>>2]|0;
   $37 = ((($36)) + 42|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   $40 = $39 & 32;
   $41 = ($40|0)==(0);
   if (!($41)) {
    $42 = HEAP32[$pTab>>2]|0;
    $43 = ((($42)) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    HEAP32[$pProbe>>2] = $44;
    break;
   }
   dest=$sPk; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $45 = ((($sPk)) + 50|0);
   HEAP16[$45>>1] = 1;
   $46 = ((($sPk)) + 52|0);
   HEAP16[$46>>1] = 1;
   $47 = ((($sPk)) + 4|0);
   HEAP32[$47>>2] = $aiColumnPk;
   $48 = ((($sPk)) + 8|0);
   HEAP32[$48>>2] = $aiRowEstPk;
   $49 = ((($sPk)) + 54|0);
   HEAP8[$49>>0] = 5;
   $50 = HEAP32[$pTab>>2]|0;
   $51 = ((($sPk)) + 12|0);
   HEAP32[$51>>2] = $50;
   $52 = HEAP32[$pTab>>2]|0;
   $53 = ((($52)) + 40|0);
   $54 = HEAP16[$53>>1]|0;
   $55 = ((($sPk)) + 48|0);
   HEAP16[$55>>1] = $54;
   $56 = HEAP32[$pTab>>2]|0;
   $57 = ((($56)) + 38|0);
   $58 = HEAP16[$57>>1]|0;
   HEAP16[$aiRowEstPk>>1] = $58;
   $59 = ((($aiRowEstPk)) + 2|0);
   HEAP16[$59>>1] = 0;
   $60 = HEAP32[$pSrc>>2]|0;
   $61 = ((($60)) + 16|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = ((($62)) + 8|0);
   $64 = HEAP32[$63>>2]|0;
   HEAP32[$pFirst>>2] = $64;
   $65 = HEAP32[$pSrc>>2]|0;
   $66 = ((($65)) + 36|0);
   $67 = ((($66)) + 1|0);
   $68 = HEAP8[$67>>0]|0;
   $69 = $68 & 1;
   $70 = $69&255;
   $71 = ($70|0)==(0);
   if ($71) {
    $72 = HEAP32[$pFirst>>2]|0;
    $73 = ((($sPk)) + 20|0);
    HEAP32[$73>>2] = $72;
   }
   HEAP32[$pProbe>>2] = $sPk;
  }
 } while(0);
 $74 = HEAP32[$pTab>>2]|0;
 $75 = ((($74)) + 38|0);
 $76 = HEAP16[$75>>1]|0;
 HEAP16[$rSize>>1] = $76;
 $77 = HEAP16[$rSize>>1]|0;
 $78 = (_estLog($77)|0);
 HEAP16[$rLogSize>>1] = $78;
 $79 = HEAP32[$2>>2]|0;
 $80 = ((($79)) + 16|0);
 $81 = HEAP32[$80>>2]|0;
 $82 = ($81|0)!=(0|0);
 L11: do {
  if (!($82)) {
   $83 = HEAP32[$pWInfo>>2]|0;
   $84 = ((($83)) + 40|0);
   $85 = HEAP16[$84>>1]|0;
   $86 = $85&65535;
   $87 = $86 & 32;
   $88 = ($87|0)==(0);
   if ($88) {
    $89 = HEAP32[$pWInfo>>2]|0;
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$90>>2]|0;
    $92 = ((($91)) + 24|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = $93 & 1048576;
    $95 = ($94|0)!=(0);
    if ($95) {
     $96 = HEAP32[$pSrc>>2]|0;
     $97 = ((($96)) + 68|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = ($98|0)==(0|0);
     if ($99) {
      $100 = HEAP32[$pSrc>>2]|0;
      $101 = ((($100)) + 36|0);
      $102 = ((($101)) + 1|0);
      $103 = HEAP8[$102>>0]|0;
      $104 = $103 & 1;
      $105 = $104&255;
      $106 = ($105|0)!=(0);
      if (!($106)) {
       $107 = HEAP32[$pTab>>2]|0;
       $108 = ((($107)) + 42|0);
       $109 = HEAP8[$108>>0]|0;
       $110 = $109&255;
       $111 = $110 & 32;
       $112 = ($111|0)==(0);
       if ($112) {
        $113 = HEAP32[$pSrc>>2]|0;
        $114 = ((($113)) + 36|0);
        $115 = ((($114)) + 1|0);
        $116 = HEAP8[$115>>0]|0;
        $117 = ($116&255) >>> 3;
        $118 = $117 & 1;
        $119 = $118&255;
        $120 = ($119|0)!=(0);
        if (!($120)) {
         $121 = HEAP32[$pSrc>>2]|0;
         $122 = ((($121)) + 36|0);
         $123 = ((($122)) + 1|0);
         $124 = HEAP8[$123>>0]|0;
         $125 = ($124&255) >>> 5;
         $126 = $125 & 1;
         $127 = $126&255;
         $128 = ($127|0)!=(0);
         if (!($128)) {
          $129 = HEAP32[$pWC>>2]|0;
          $130 = ((($129)) + 20|0);
          $131 = HEAP32[$130>>2]|0;
          $132 = HEAP32[$pWC>>2]|0;
          $133 = ((($132)) + 12|0);
          $134 = HEAP32[$133>>2]|0;
          $135 = (($131) + (($134*48)|0)|0);
          HEAP32[$pWCEnd>>2] = $135;
          $136 = HEAP32[$pWC>>2]|0;
          $137 = ((($136)) + 20|0);
          $138 = HEAP32[$137>>2]|0;
          HEAP32[$pTerm>>2] = $138;
          while(1) {
           $139 = HEAP32[$rc>>2]|0;
           $140 = ($139|0)==(0);
           if (!($140)) {
            break L11;
           }
           $141 = HEAP32[$pTerm>>2]|0;
           $142 = HEAP32[$pWCEnd>>2]|0;
           $143 = ($141>>>0)<($142>>>0);
           if (!($143)) {
            break L11;
           }
           $144 = HEAP32[$pTerm>>2]|0;
           $145 = ((($144)) + 32|0);
           $146 = $145;
           $147 = $146;
           $148 = HEAP32[$147>>2]|0;
           $149 = (($146) + 4)|0;
           $150 = $149;
           $151 = HEAP32[$150>>2]|0;
           $152 = HEAP32[$pNew>>2]|0;
           $153 = ((($152)) + 8|0);
           $154 = $153;
           $155 = $154;
           $156 = HEAP32[$155>>2]|0;
           $157 = (($154) + 4)|0;
           $158 = $157;
           $159 = HEAP32[$158>>2]|0;
           $160 = $148 & $156;
           $161 = $151 & $159;
           $162 = ($160|0)!=(0);
           $163 = ($161|0)!=(0);
           $164 = $162 | $163;
           if (!($164)) {
            $165 = HEAP32[$pTerm>>2]|0;
            $166 = HEAP32[$pSrc>>2]|0;
            $167 = (_termCanDriveIndex($165,$166,0,0)|0);
            $168 = ($167|0)!=(0);
            if ($168) {
             $169 = HEAP32[$pNew>>2]|0;
             $170 = ((($169)) + 24|0);
             HEAP16[$170>>1] = 1;
             $171 = HEAP32[$pNew>>2]|0;
             $172 = ((($171)) + 42|0);
             HEAP16[$172>>1] = 0;
             $173 = HEAP32[$pNew>>2]|0;
             $174 = ((($173)) + 24|0);
             $175 = ((($174)) + 8|0);
             HEAP32[$175>>2] = 0;
             $176 = HEAP32[$pNew>>2]|0;
             $177 = ((($176)) + 40|0);
             HEAP16[$177>>1] = 1;
             $178 = HEAP32[$pTerm>>2]|0;
             $179 = HEAP32[$pNew>>2]|0;
             $180 = ((($179)) + 48|0);
             $181 = HEAP32[$180>>2]|0;
             HEAP32[$181>>2] = $178;
             $182 = HEAP16[$rLogSize>>1]|0;
             $183 = $182 << 16 >> 16;
             $184 = HEAP16[$rSize>>1]|0;
             $185 = $184 << 16 >> 16;
             $186 = (($183) + ($185))|0;
             $187 = (($186) + 4)|0;
             $188 = $187&65535;
             $189 = HEAP32[$pNew>>2]|0;
             $190 = ((($189)) + 18|0);
             HEAP16[$190>>1] = $188;
             $191 = HEAP32[$pTab>>2]|0;
             $192 = ((($191)) + 12|0);
             $193 = HEAP32[$192>>2]|0;
             $194 = ($193|0)==(0|0);
             if ($194) {
              $195 = HEAP32[$pTab>>2]|0;
              $196 = ((($195)) + 42|0);
              $197 = HEAP8[$196>>0]|0;
              $198 = $197&255;
              $199 = $198 & 2;
              $200 = ($199|0)==(0);
              if ($200) {
               $201 = HEAP32[$pNew>>2]|0;
               $202 = ((($201)) + 18|0);
               $203 = HEAP16[$202>>1]|0;
               $204 = $203 << 16 >> 16;
               $205 = (($204) + 24)|0;
               $206 = $205&65535;
               HEAP16[$202>>1] = $206;
              }
             }
             $207 = HEAP32[$pNew>>2]|0;
             $208 = ((($207)) + 18|0);
             $209 = HEAP16[$208>>1]|0;
             $210 = $209 << 16 >> 16;
             $211 = ($210|0)<(0);
             if ($211) {
              $212 = HEAP32[$pNew>>2]|0;
              $213 = ((($212)) + 18|0);
              HEAP16[$213>>1] = 0;
             }
             $214 = HEAP32[$pNew>>2]|0;
             $215 = ((($214)) + 22|0);
             HEAP16[$215>>1] = 43;
             $216 = HEAP16[$rLogSize>>1]|0;
             $217 = HEAP32[$pNew>>2]|0;
             $218 = ((($217)) + 22|0);
             $219 = HEAP16[$218>>1]|0;
             $220 = (_sqlite3LogEstAdd($216,$219)|0);
             $221 = HEAP32[$pNew>>2]|0;
             $222 = ((($221)) + 20|0);
             HEAP16[$222>>1] = $220;
             $223 = HEAP32[$pNew>>2]|0;
             $224 = ((($223)) + 36|0);
             HEAP32[$224>>2] = 16384;
             $225 = $3;
             $226 = $225;
             $227 = HEAP32[$226>>2]|0;
             $228 = (($225) + 4)|0;
             $229 = $228;
             $230 = HEAP32[$229>>2]|0;
             $231 = HEAP32[$pTerm>>2]|0;
             $232 = ((($231)) + 32|0);
             $233 = $232;
             $234 = $233;
             $235 = HEAP32[$234>>2]|0;
             $236 = (($233) + 4)|0;
             $237 = $236;
             $238 = HEAP32[$237>>2]|0;
             $239 = $227 | $235;
             $240 = $230 | $238;
             $241 = HEAP32[$pNew>>2]|0;
             $242 = $241;
             $243 = $242;
             HEAP32[$243>>2] = $239;
             $244 = (($242) + 4)|0;
             $245 = $244;
             HEAP32[$245>>2] = $240;
             $246 = HEAP32[$2>>2]|0;
             $247 = HEAP32[$pNew>>2]|0;
             $248 = (_whereLoopInsert($246,$247)|0);
             HEAP32[$rc>>2] = $248;
            }
           }
           $249 = HEAP32[$pTerm>>2]|0;
           $250 = ((($249)) + 48|0);
           HEAP32[$pTerm>>2] = $250;
          }
         }
        }
       }
      }
     }
    }
   }
  }
 } while(0);
 L35: while(1) {
  $251 = HEAP32[$rc>>2]|0;
  $252 = ($251|0)==(0);
  $253 = HEAP32[$pProbe>>2]|0;
  $254 = ($253|0)!=(0|0);
  $255 = $252 ? $254 : 0;
  if (!($255)) {
   label = 57;
   break;
  }
  $256 = HEAP32[$pProbe>>2]|0;
  $257 = ((($256)) + 36|0);
  $258 = HEAP32[$257>>2]|0;
  $259 = ($258|0)!=(0|0);
  if ($259) {
   $260 = HEAP32[$pSrc>>2]|0;
   $261 = ((($260)) + 44|0);
   $262 = HEAP32[$261>>2]|0;
   $263 = HEAP32[$pWC>>2]|0;
   $264 = HEAP32[$pProbe>>2]|0;
   $265 = ((($264)) + 36|0);
   $266 = HEAP32[$265>>2]|0;
   $267 = (_whereUsablePartialIndex($262,$263,$266)|0);
   $268 = ($267|0)!=(0);
   if ($268) {
    label = 31;
   }
  } else {
   label = 31;
  }
  if ((label|0) == 31) {
   label = 0;
   $269 = HEAP32[$pProbe>>2]|0;
   $270 = ((($269)) + 8|0);
   $271 = HEAP32[$270>>2]|0;
   $272 = HEAP16[$271>>1]|0;
   HEAP16[$rSize>>1] = $272;
   $273 = HEAP32[$pNew>>2]|0;
   $274 = ((($273)) + 24|0);
   HEAP16[$274>>1] = 0;
   $275 = HEAP32[$pNew>>2]|0;
   $276 = ((($275)) + 24|0);
   $277 = ((($276)) + 2|0);
   HEAP16[$277>>1] = 0;
   $278 = HEAP32[$pNew>>2]|0;
   $279 = ((($278)) + 24|0);
   $280 = ((($279)) + 4|0);
   HEAP16[$280>>1] = 0;
   $281 = HEAP32[$pNew>>2]|0;
   $282 = ((($281)) + 42|0);
   HEAP16[$282>>1] = 0;
   $283 = HEAP32[$pNew>>2]|0;
   $284 = ((($283)) + 40|0);
   HEAP16[$284>>1] = 0;
   $285 = HEAP32[$pNew>>2]|0;
   $286 = ((($285)) + 17|0);
   HEAP8[$286>>0] = 0;
   $287 = HEAP32[$pNew>>2]|0;
   $288 = ((($287)) + 18|0);
   HEAP16[$288>>1] = 0;
   $289 = $3;
   $290 = $289;
   $291 = HEAP32[$290>>2]|0;
   $292 = (($289) + 4)|0;
   $293 = $292;
   $294 = HEAP32[$293>>2]|0;
   $295 = HEAP32[$pNew>>2]|0;
   $296 = $295;
   $297 = $296;
   HEAP32[$297>>2] = $291;
   $298 = (($296) + 4)|0;
   $299 = $298;
   HEAP32[$299>>2] = $294;
   $300 = HEAP16[$rSize>>1]|0;
   $301 = HEAP32[$pNew>>2]|0;
   $302 = ((($301)) + 22|0);
   HEAP16[$302>>1] = $300;
   $303 = HEAP32[$pProbe>>2]|0;
   $304 = HEAP32[$pNew>>2]|0;
   $305 = ((($304)) + 24|0);
   $306 = ((($305)) + 8|0);
   HEAP32[$306>>2] = $303;
   $307 = HEAP32[$2>>2]|0;
   $308 = HEAP32[$pProbe>>2]|0;
   $309 = HEAP32[$pSrc>>2]|0;
   $310 = ((($309)) + 44|0);
   $311 = HEAP32[$310>>2]|0;
   $312 = (_indexMightHelpWithOrderBy($307,$308,$311)|0);
   HEAP32[$b>>2] = $312;
   $313 = HEAP32[$pProbe>>2]|0;
   $314 = ((($313)) + 44|0);
   $315 = HEAP32[$314>>2]|0;
   $316 = ($315|0)<=(0);
   do {
    if ($316) {
     $317 = HEAP32[$pNew>>2]|0;
     $318 = ((($317)) + 36|0);
     HEAP32[$318>>2] = 256;
     $319 = HEAP32[$b>>2]|0;
     $320 = ($319|0)!=(0);
     $321 = HEAP32[$iSortIdx>>2]|0;
     $322 = $320 ? $321 : 0;
     $323 = $322&255;
     $324 = HEAP32[$pNew>>2]|0;
     $325 = ((($324)) + 17|0);
     HEAP8[$325>>0] = $323;
     $326 = HEAP16[$rSize>>1]|0;
     $327 = $326 << 16 >> 16;
     $328 = (($327) + 16)|0;
     $329 = $328&65535;
     $330 = HEAP32[$pNew>>2]|0;
     $331 = ((($330)) + 20|0);
     HEAP16[$331>>1] = $329;
     $332 = HEAP32[$pWC>>2]|0;
     $333 = HEAP32[$pNew>>2]|0;
     $334 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($332,$333,$334);
     $335 = HEAP32[$2>>2]|0;
     $336 = HEAP32[$pNew>>2]|0;
     $337 = (_whereLoopInsert($335,$336)|0);
     HEAP32[$rc>>2] = $337;
     $338 = HEAP16[$rSize>>1]|0;
     $339 = HEAP32[$pNew>>2]|0;
     $340 = ((($339)) + 22|0);
     HEAP16[$340>>1] = $338;
     $341 = HEAP32[$rc>>2]|0;
     $342 = ($341|0)!=(0);
     if ($342) {
      label = 57;
      break L35;
     }
    } else {
     $343 = HEAP32[$pProbe>>2]|0;
     $344 = ((($343)) + 55|0);
     $345 = HEAP8[$344>>0]|0;
     $346 = ($345&255) >>> 5;
     $347 = $346 & 1;
     $348 = $347&255;
     $349 = ($348|0)!=(0);
     if ($349) {
      $350 = HEAP32[$pNew>>2]|0;
      $351 = ((($350)) + 36|0);
      HEAP32[$351>>2] = 576;
      $352 = $m;
      $353 = $352;
      HEAP32[$353>>2] = 0;
      $354 = (($352) + 4)|0;
      $355 = $354;
      HEAP32[$355>>2] = 0;
     } else {
      $356 = HEAP32[$pSrc>>2]|0;
      $357 = ((($356)) + 56|0);
      $358 = $357;
      $359 = $358;
      $360 = HEAP32[$359>>2]|0;
      $361 = (($358) + 4)|0;
      $362 = $361;
      $363 = HEAP32[$362>>2]|0;
      $364 = HEAP32[$pProbe>>2]|0;
      $365 = (_columnsInIndex($364)|0);
      $366 = tempRet0;
      $367 = $365 ^ -1;
      $368 = $366 ^ -1;
      $369 = $360 & $367;
      $370 = $363 & $368;
      $371 = $m;
      $372 = $371;
      HEAP32[$372>>2] = $369;
      $373 = (($371) + 4)|0;
      $374 = $373;
      HEAP32[$374>>2] = $370;
      $375 = $m;
      $376 = $375;
      $377 = HEAP32[$376>>2]|0;
      $378 = (($375) + 4)|0;
      $379 = $378;
      $380 = HEAP32[$379>>2]|0;
      $381 = ($377|0)==(0);
      $382 = ($380|0)==(0);
      $383 = $381 & $382;
      $384 = $383 ? 576 : 512;
      $385 = HEAP32[$pNew>>2]|0;
      $386 = ((($385)) + 36|0);
      HEAP32[$386>>2] = $384;
     }
     $387 = HEAP32[$b>>2]|0;
     $388 = ($387|0)!=(0);
     if (!($388)) {
      $389 = HEAP32[$pTab>>2]|0;
      $390 = ((($389)) + 42|0);
      $391 = HEAP8[$390>>0]|0;
      $392 = $391&255;
      $393 = $392 & 32;
      $394 = ($393|0)==(0);
      if ($394) {
       $395 = HEAP32[$pProbe>>2]|0;
       $396 = ((($395)) + 36|0);
       $397 = HEAP32[$396>>2]|0;
       $398 = ($397|0)!=(0|0);
       if (!($398)) {
        $399 = $m;
        $400 = $399;
        $401 = HEAP32[$400>>2]|0;
        $402 = (($399) + 4)|0;
        $403 = $402;
        $404 = HEAP32[$403>>2]|0;
        $405 = ($401|0)==(0);
        $406 = ($404|0)==(0);
        $407 = $405 & $406;
        if (!($407)) {
         break;
        }
        $408 = HEAP32[$pProbe>>2]|0;
        $409 = ((($408)) + 55|0);
        $410 = HEAP8[$409>>0]|0;
        $411 = ($410&255) >>> 2;
        $412 = $411 & 1;
        $413 = $412&255;
        $414 = ($413|0)==(0);
        if (!($414)) {
         break;
        }
        $415 = HEAP32[$pProbe>>2]|0;
        $416 = ((($415)) + 48|0);
        $417 = HEAP16[$416>>1]|0;
        $418 = $417 << 16 >> 16;
        $419 = HEAP32[$pTab>>2]|0;
        $420 = ((($419)) + 40|0);
        $421 = HEAP16[$420>>1]|0;
        $422 = $421 << 16 >> 16;
        $423 = ($418|0)<($422|0);
        if (!($423)) {
         break;
        }
        $424 = HEAP32[$pWInfo>>2]|0;
        $425 = ((($424)) + 40|0);
        $426 = HEAP16[$425>>1]|0;
        $427 = $426&65535;
        $428 = $427 & 4;
        $429 = ($428|0)==(0);
        $430 = HEAP32[(24)>>2]|0;
        $431 = ($430|0)!=(0);
        $or$cond = $429 & $431;
        if (!($or$cond)) {
         break;
        }
        $432 = HEAP32[$pWInfo>>2]|0;
        $433 = HEAP32[$432>>2]|0;
        $434 = HEAP32[$433>>2]|0;
        $435 = ((($434)) + 64|0);
        $436 = HEAP16[$435>>1]|0;
        $437 = $436&65535;
        $438 = $437 & 64;
        $439 = ($438|0)==(0);
        if (!($439)) {
         break;
        }
       }
      }
     }
     $440 = HEAP32[$b>>2]|0;
     $441 = ($440|0)!=(0);
     $442 = HEAP32[$iSortIdx>>2]|0;
     $443 = $441 ? $442 : 0;
     $444 = $443&255;
     $445 = HEAP32[$pNew>>2]|0;
     $446 = ((($445)) + 17|0);
     HEAP8[$446>>0] = $444;
     $447 = HEAP16[$rSize>>1]|0;
     $448 = $447 << 16 >> 16;
     $449 = (($448) + 1)|0;
     $450 = HEAP32[$pProbe>>2]|0;
     $451 = ((($450)) + 48|0);
     $452 = HEAP16[$451>>1]|0;
     $453 = $452 << 16 >> 16;
     $454 = ($453*15)|0;
     $455 = HEAP32[$pTab>>2]|0;
     $456 = ((($455)) + 40|0);
     $457 = HEAP16[$456>>1]|0;
     $458 = $457 << 16 >> 16;
     $459 = (($454|0) / ($458|0))&-1;
     $460 = (($449) + ($459))|0;
     $461 = $460&65535;
     $462 = HEAP32[$pNew>>2]|0;
     $463 = ((($462)) + 20|0);
     HEAP16[$463>>1] = $461;
     $464 = $m;
     $465 = $464;
     $466 = HEAP32[$465>>2]|0;
     $467 = (($464) + 4)|0;
     $468 = $467;
     $469 = HEAP32[$468>>2]|0;
     $470 = ($466|0)!=(0);
     $471 = ($469|0)!=(0);
     $472 = $470 | $471;
     if ($472) {
      $473 = HEAP16[$rSize>>1]|0;
      $474 = $473 << 16 >> 16;
      $475 = (($474) + 16)|0;
      $476 = $475&65535;
      HEAP16[$nLookup>>1] = $476;
      $477 = HEAP32[$pSrc>>2]|0;
      $478 = ((($477)) + 44|0);
      $479 = HEAP32[$478>>2]|0;
      HEAP32[$iCur>>2] = $479;
      $480 = HEAP32[$pWInfo>>2]|0;
      $481 = ((($480)) + 80|0);
      HEAP32[$pWC2>>2] = $481;
      HEAP32[$ii>>2] = 0;
      while(1) {
       $482 = HEAP32[$ii>>2]|0;
       $483 = HEAP32[$pWC2>>2]|0;
       $484 = ((($483)) + 12|0);
       $485 = HEAP32[$484>>2]|0;
       $486 = ($482|0)<($485|0);
       if (!($486)) {
        break;
       }
       $487 = HEAP32[$ii>>2]|0;
       $488 = HEAP32[$pWC2>>2]|0;
       $489 = ((($488)) + 20|0);
       $490 = HEAP32[$489>>2]|0;
       $491 = (($490) + (($487*48)|0)|0);
       HEAP32[$pTerm1>>2] = $491;
       $492 = HEAP32[$pTerm1>>2]|0;
       $493 = HEAP32[$492>>2]|0;
       $494 = HEAP32[$iCur>>2]|0;
       $495 = HEAP32[$pProbe>>2]|0;
       $496 = (_sqlite3ExprCoveredByIndex($493,$494,$495)|0);
       $497 = ($496|0)!=(0);
       if (!($497)) {
        break;
       }
       $498 = HEAP32[$pTerm1>>2]|0;
       $499 = ((($498)) + 8|0);
       $500 = HEAP16[$499>>1]|0;
       $501 = $500 << 16 >> 16;
       $502 = ($501|0)<=(0);
       if ($502) {
        $503 = HEAP32[$pTerm1>>2]|0;
        $504 = ((($503)) + 8|0);
        $505 = HEAP16[$504>>1]|0;
        $506 = $505 << 16 >> 16;
        $507 = HEAP16[$nLookup>>1]|0;
        $508 = $507 << 16 >> 16;
        $509 = (($508) + ($506))|0;
        $510 = $509&65535;
        HEAP16[$nLookup>>1] = $510;
       } else {
        $511 = HEAP16[$nLookup>>1]|0;
        $512 = (($511) + -1)<<16>>16;
        HEAP16[$nLookup>>1] = $512;
        $513 = HEAP32[$pTerm1>>2]|0;
        $514 = ((($513)) + 12|0);
        $515 = HEAP16[$514>>1]|0;
        $516 = $515&65535;
        $517 = $516 & 130;
        $518 = ($517|0)!=(0);
        if ($518) {
         $519 = HEAP16[$nLookup>>1]|0;
         $520 = $519 << 16 >> 16;
         $521 = (($520) - 19)|0;
         $522 = $521&65535;
         HEAP16[$nLookup>>1] = $522;
        }
       }
       $523 = HEAP32[$ii>>2]|0;
       $524 = (($523) + 1)|0;
       HEAP32[$ii>>2] = $524;
      }
      $525 = HEAP32[$pNew>>2]|0;
      $526 = ((($525)) + 20|0);
      $527 = HEAP16[$526>>1]|0;
      $528 = HEAP16[$nLookup>>1]|0;
      $529 = (_sqlite3LogEstAdd($527,$528)|0);
      $530 = HEAP32[$pNew>>2]|0;
      $531 = ((($530)) + 20|0);
      HEAP16[$531>>1] = $529;
     }
     $532 = HEAP32[$pWC>>2]|0;
     $533 = HEAP32[$pNew>>2]|0;
     $534 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($532,$533,$534);
     $535 = HEAP32[$2>>2]|0;
     $536 = HEAP32[$pNew>>2]|0;
     $537 = (_whereLoopInsert($535,$536)|0);
     HEAP32[$rc>>2] = $537;
     $538 = HEAP16[$rSize>>1]|0;
     $539 = HEAP32[$pNew>>2]|0;
     $540 = ((($539)) + 22|0);
     HEAP16[$540>>1] = $538;
     $541 = HEAP32[$rc>>2]|0;
     $542 = ($541|0)!=(0);
     if ($542) {
      label = 57;
      break L35;
     }
    }
   } while(0);
   $543 = HEAP32[$2>>2]|0;
   $544 = HEAP32[$pSrc>>2]|0;
   $545 = HEAP32[$pProbe>>2]|0;
   $546 = (_whereLoopAddBtreeIndex($543,$544,$545,0)|0);
   HEAP32[$rc>>2] = $546;
   $547 = HEAP32[$pSrc>>2]|0;
   $548 = ((($547)) + 68|0);
   $549 = HEAP32[$548>>2]|0;
   $550 = ($549|0)!=(0|0);
   if ($550) {
    label = 57;
    break;
   }
  }
  $551 = HEAP32[$pProbe>>2]|0;
  $552 = ((($551)) + 20|0);
  $553 = HEAP32[$552>>2]|0;
  HEAP32[$pProbe>>2] = $553;
  $554 = HEAP32[$iSortIdx>>2]|0;
  $555 = (($554) + 1)|0;
  HEAP32[$iSortIdx>>2] = $555;
 }
 if ((label|0) == 57) {
  $556 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($556|0);
 }
 return (0)|0;
}
function _whereLoopAddOr($pBuilder,$0,$1,$2,$3) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $28 = 0, $29 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCur = 0, $j = 0, $once = 0, $pItem = 0, $pNew = 0;
 var $pOrTerm = 0, $pOrWC = 0, $pOrWCEnd = 0, $pTerm = 0, $pWC = 0, $pWCEnd = 0, $pWInfo = 0, $rc = 0, $sCur = 0, $sPrev = 0, $sSubBuild = 0, $sSum = 0, $tempWC = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 672|0;
 $4 = sp + 668|0;
 $5 = sp + 584|0;
 $6 = sp + 576|0;
 $pWInfo = sp + 664|0;
 $pWC = sp + 660|0;
 $pNew = sp + 656|0;
 $pTerm = sp + 652|0;
 $pWCEnd = sp + 648|0;
 $rc = sp + 644|0;
 $iCur = sp + 640|0;
 $tempWC = sp + 168|0;
 $sSubBuild = sp + 620|0;
 $sSum = sp + 112|0;
 $sCur = sp + 56|0;
 $pItem = sp + 616|0;
 $pOrWC = sp + 612|0;
 $pOrWCEnd = sp + 608|0;
 $pOrTerm = sp + 604|0;
 $once = sp + 600|0;
 $i = sp + 596|0;
 $j = sp + 592|0;
 $sPrev = sp;
 HEAP32[$4>>2] = $pBuilder;
 $7 = $5;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = $6;
 $12 = $11;
 HEAP32[$12>>2] = $2;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $3;
 $15 = HEAP32[$4>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pWInfo>>2] = $16;
 HEAP32[$rc>>2] = 0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pWC>>2] = $19;
 $20 = HEAP32[$pWC>>2]|0;
 $21 = ((($20)) + 20|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pWC>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($22) + (($25*48)|0)|0);
 HEAP32[$pWCEnd>>2] = $26;
 $27 = HEAP32[$4>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$pNew>>2] = $29;
 dest=$sSum; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $30 = HEAP32[$pWInfo>>2]|0;
 $31 = ((($30)) + 4|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = HEAP32[$pNew>>2]|0;
 $35 = ((($34)) + 16|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = (($33) + (($37*72)|0)|0);
 HEAP32[$pItem>>2] = $38;
 $39 = HEAP32[$pItem>>2]|0;
 $40 = ((($39)) + 44|0);
 $41 = HEAP32[$40>>2]|0;
 HEAP32[$iCur>>2] = $41;
 $42 = HEAP32[$pWC>>2]|0;
 $43 = ((($42)) + 20|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pTerm>>2] = $44;
 while(1) {
  $45 = HEAP32[$pTerm>>2]|0;
  $46 = HEAP32[$pWCEnd>>2]|0;
  $47 = ($45>>>0)<($46>>>0);
  $48 = HEAP32[$rc>>2]|0;
  $49 = ($48|0)==(0);
  $50 = $47 ? $49 : 0;
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$pTerm>>2]|0;
  $52 = ((($51)) + 12|0);
  $53 = HEAP16[$52>>1]|0;
  $54 = $53&65535;
  $55 = $54 & 512;
  $56 = ($55|0)!=(0);
  L4: do {
   if ($56) {
    $57 = HEAP32[$pTerm>>2]|0;
    $58 = ((($57)) + 28|0);
    $59 = HEAP32[$58>>2]|0;
    $60 = ((($59)) + 408|0);
    $61 = $60;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = (($61) + 4)|0;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = HEAP32[$pNew>>2]|0;
    $68 = ((($67)) + 8|0);
    $69 = $68;
    $70 = $69;
    $71 = HEAP32[$70>>2]|0;
    $72 = (($69) + 4)|0;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = $63 & $71;
    $76 = $66 & $74;
    $77 = ($75|0)!=(0);
    $78 = ($76|0)!=(0);
    $79 = $77 | $78;
    if ($79) {
     $80 = HEAP32[$pTerm>>2]|0;
     $81 = ((($80)) + 28|0);
     $82 = HEAP32[$81>>2]|0;
     HEAP32[$pOrWC>>2] = $82;
     $83 = HEAP32[$pOrWC>>2]|0;
     $84 = ((($83)) + 12|0);
     $85 = HEAP32[$84>>2]|0;
     $86 = HEAP32[$pOrWC>>2]|0;
     $87 = ((($86)) + 20|0);
     $88 = HEAP32[$87>>2]|0;
     $89 = (($88) + (($85*48)|0)|0);
     HEAP32[$pOrWCEnd>>2] = $89;
     HEAP32[$once>>2] = 1;
     $90 = HEAP32[$4>>2]|0;
     ;HEAP32[$sSubBuild>>2]=HEAP32[$90>>2]|0;HEAP32[$sSubBuild+4>>2]=HEAP32[$90+4>>2]|0;HEAP32[$sSubBuild+8>>2]=HEAP32[$90+8>>2]|0;HEAP32[$sSubBuild+12>>2]=HEAP32[$90+12>>2]|0;HEAP32[$sSubBuild+16>>2]=HEAP32[$90+16>>2]|0;
     $91 = ((($sSubBuild)) + 8|0);
     HEAP32[$91>>2] = 0;
     $92 = ((($sSubBuild)) + 16|0);
     HEAP32[$92>>2] = $sCur;
     $93 = HEAP32[$pOrWC>>2]|0;
     $94 = ((($93)) + 20|0);
     $95 = HEAP32[$94>>2]|0;
     HEAP32[$pOrTerm>>2] = $95;
     L7: while(1) {
      $96 = HEAP32[$pOrTerm>>2]|0;
      $97 = HEAP32[$pOrWCEnd>>2]|0;
      $98 = ($96>>>0)<($97>>>0);
      if (!($98)) {
       break;
      }
      $99 = HEAP32[$pOrTerm>>2]|0;
      $100 = ((($99)) + 12|0);
      $101 = HEAP16[$100>>1]|0;
      $102 = $101&65535;
      $103 = $102 & 1024;
      $104 = ($103|0)!=(0);
      $105 = HEAP32[$pOrTerm>>2]|0;
      if ($104) {
       $106 = ((($105)) + 28|0);
       $107 = HEAP32[$106>>2]|0;
       $108 = ((($sSubBuild)) + 4|0);
       HEAP32[$108>>2] = $107;
       label = 11;
      } else {
       $109 = ((($105)) + 20|0);
       $110 = HEAP32[$109>>2]|0;
       $111 = HEAP32[$iCur>>2]|0;
       $112 = ($110|0)==($111|0);
       if ($112) {
        $113 = HEAP32[$pWC>>2]|0;
        $114 = HEAP32[$113>>2]|0;
        HEAP32[$tempWC>>2] = $114;
        $115 = HEAP32[$pWC>>2]|0;
        $116 = ((($tempWC)) + 4|0);
        HEAP32[$116>>2] = $115;
        $117 = ((($tempWC)) + 8|0);
        HEAP8[$117>>0] = 28;
        $118 = ((($tempWC)) + 12|0);
        HEAP32[$118>>2] = 1;
        $119 = HEAP32[$pOrTerm>>2]|0;
        $120 = ((($tempWC)) + 20|0);
        HEAP32[$120>>2] = $119;
        $121 = ((($sSubBuild)) + 4|0);
        HEAP32[$121>>2] = $tempWC;
        label = 11;
       }
      }
      L14: do {
       if ((label|0) == 11) {
        label = 0;
        HEAP16[$sCur>>1] = 0;
        $122 = HEAP32[$pItem>>2]|0;
        $123 = ((($122)) + 16|0);
        $124 = HEAP32[$123>>2]|0;
        $125 = ((($124)) + 42|0);
        $126 = HEAP8[$125>>0]|0;
        $127 = $126&255;
        $128 = $127 & 16;
        $129 = ($128|0)!=(0);
        $130 = $5;
        $131 = $130;
        $132 = HEAP32[$131>>2]|0;
        $133 = (($130) + 4)|0;
        $134 = $133;
        $135 = HEAP32[$134>>2]|0;
        if ($129) {
         $136 = $6;
         $137 = $136;
         $138 = HEAP32[$137>>2]|0;
         $139 = (($136) + 4)|0;
         $140 = $139;
         $141 = HEAP32[$140>>2]|0;
         $142 = (_whereLoopAddVirtual($sSubBuild,$132,$135,$138,$141)|0);
         HEAP32[$rc>>2] = $142;
        } else {
         $143 = (_whereLoopAddBtree($sSubBuild,$132,$135)|0);
         HEAP32[$rc>>2] = $143;
        }
        $144 = HEAP32[$rc>>2]|0;
        $145 = ($144|0)==(0);
        if ($145) {
         $146 = $5;
         $147 = $146;
         $148 = HEAP32[$147>>2]|0;
         $149 = (($146) + 4)|0;
         $150 = $149;
         $151 = HEAP32[$150>>2]|0;
         $152 = $6;
         $153 = $152;
         $154 = HEAP32[$153>>2]|0;
         $155 = (($152) + 4)|0;
         $156 = $155;
         $157 = HEAP32[$156>>2]|0;
         $158 = (_whereLoopAddOr($sSubBuild,$148,$151,$154,$157)|0);
         HEAP32[$rc>>2] = $158;
        }
        $159 = HEAP16[$sCur>>1]|0;
        $160 = $159&65535;
        $161 = ($160|0)==(0);
        if ($161) {
         label = 17;
         break L7;
        }
        $162 = HEAP32[$once>>2]|0;
        $163 = ($162|0)!=(0);
        if ($163) {
         _whereOrMove($sSum,$sCur);
         HEAP32[$once>>2] = 0;
         break;
        }
        _whereOrMove($sPrev,$sSum);
        HEAP16[$sSum>>1] = 0;
        HEAP32[$i>>2] = 0;
        while(1) {
         $164 = HEAP32[$i>>2]|0;
         $165 = HEAP16[$sPrev>>1]|0;
         $166 = $165&65535;
         $167 = ($164|0)<($166|0);
         if (!($167)) {
          break L14;
         }
         HEAP32[$j>>2] = 0;
         while(1) {
          $168 = HEAP32[$j>>2]|0;
          $169 = HEAP16[$sCur>>1]|0;
          $170 = $169&65535;
          $171 = ($168|0)<($170|0);
          $172 = HEAP32[$i>>2]|0;
          if (!($171)) {
           break;
          }
          $173 = ((($sPrev)) + 8|0);
          $174 = (($173) + ($172<<4)|0);
          $175 = $174;
          $176 = $175;
          $177 = HEAP32[$176>>2]|0;
          $178 = (($175) + 4)|0;
          $179 = $178;
          $180 = HEAP32[$179>>2]|0;
          $181 = HEAP32[$j>>2]|0;
          $182 = ((($sCur)) + 8|0);
          $183 = (($182) + ($181<<4)|0);
          $184 = $183;
          $185 = $184;
          $186 = HEAP32[$185>>2]|0;
          $187 = (($184) + 4)|0;
          $188 = $187;
          $189 = HEAP32[$188>>2]|0;
          $190 = $177 | $186;
          $191 = $180 | $189;
          $192 = HEAP32[$i>>2]|0;
          $193 = ((($sPrev)) + 8|0);
          $194 = (($193) + ($192<<4)|0);
          $195 = ((($194)) + 8|0);
          $196 = HEAP16[$195>>1]|0;
          $197 = HEAP32[$j>>2]|0;
          $198 = ((($sCur)) + 8|0);
          $199 = (($198) + ($197<<4)|0);
          $200 = ((($199)) + 8|0);
          $201 = HEAP16[$200>>1]|0;
          $202 = (_sqlite3LogEstAdd($196,$201)|0);
          $203 = HEAP32[$i>>2]|0;
          $204 = ((($sPrev)) + 8|0);
          $205 = (($204) + ($203<<4)|0);
          $206 = ((($205)) + 10|0);
          $207 = HEAP16[$206>>1]|0;
          $208 = HEAP32[$j>>2]|0;
          $209 = ((($sCur)) + 8|0);
          $210 = (($209) + ($208<<4)|0);
          $211 = ((($210)) + 10|0);
          $212 = HEAP16[$211>>1]|0;
          $213 = (_sqlite3LogEstAdd($207,$212)|0);
          (_whereOrInsert($sSum,$190,$191,$202,$213)|0);
          $214 = HEAP32[$j>>2]|0;
          $215 = (($214) + 1)|0;
          HEAP32[$j>>2] = $215;
         }
         $216 = (($172) + 1)|0;
         HEAP32[$i>>2] = $216;
        }
       }
      } while(0);
      $217 = HEAP32[$pOrTerm>>2]|0;
      $218 = ((($217)) + 48|0);
      HEAP32[$pOrTerm>>2] = $218;
     }
     if ((label|0) == 17) {
      label = 0;
      HEAP16[$sSum>>1] = 0;
     }
     $219 = HEAP32[$pNew>>2]|0;
     $220 = ((($219)) + 40|0);
     HEAP16[$220>>1] = 1;
     $221 = HEAP32[$pTerm>>2]|0;
     $222 = HEAP32[$pNew>>2]|0;
     $223 = ((($222)) + 48|0);
     $224 = HEAP32[$223>>2]|0;
     HEAP32[$224>>2] = $221;
     $225 = HEAP32[$pNew>>2]|0;
     $226 = ((($225)) + 36|0);
     HEAP32[$226>>2] = 8192;
     $227 = HEAP32[$pNew>>2]|0;
     $228 = ((($227)) + 18|0);
     HEAP16[$228>>1] = 0;
     $229 = HEAP32[$pNew>>2]|0;
     $230 = ((($229)) + 17|0);
     HEAP8[$230>>0] = 0;
     $231 = HEAP32[$pNew>>2]|0;
     $232 = ((($231)) + 24|0);
     ;HEAP32[$232>>2]=0|0;HEAP32[$232+4>>2]=0|0;HEAP32[$232+8>>2]=0|0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $233 = HEAP32[$rc>>2]|0;
      $234 = ($233|0)==(0);
      if (!($234)) {
       break L4;
      }
      $235 = HEAP32[$i>>2]|0;
      $236 = HEAP16[$sSum>>1]|0;
      $237 = $236&65535;
      $238 = ($235|0)<($237|0);
      if (!($238)) {
       break L4;
      }
      $239 = HEAP32[$i>>2]|0;
      $240 = ((($sSum)) + 8|0);
      $241 = (($240) + ($239<<4)|0);
      $242 = ((($241)) + 8|0);
      $243 = HEAP16[$242>>1]|0;
      $244 = $243 << 16 >> 16;
      $245 = (($244) + 1)|0;
      $246 = $245&65535;
      $247 = HEAP32[$pNew>>2]|0;
      $248 = ((($247)) + 20|0);
      HEAP16[$248>>1] = $246;
      $249 = HEAP32[$i>>2]|0;
      $250 = ((($sSum)) + 8|0);
      $251 = (($250) + ($249<<4)|0);
      $252 = ((($251)) + 10|0);
      $253 = HEAP16[$252>>1]|0;
      $254 = HEAP32[$pNew>>2]|0;
      $255 = ((($254)) + 22|0);
      HEAP16[$255>>1] = $253;
      $256 = HEAP32[$i>>2]|0;
      $257 = ((($sSum)) + 8|0);
      $258 = (($257) + ($256<<4)|0);
      $259 = $258;
      $260 = $259;
      $261 = HEAP32[$260>>2]|0;
      $262 = (($259) + 4)|0;
      $263 = $262;
      $264 = HEAP32[$263>>2]|0;
      $265 = HEAP32[$pNew>>2]|0;
      $266 = $265;
      $267 = $266;
      HEAP32[$267>>2] = $261;
      $268 = (($266) + 4)|0;
      $269 = $268;
      HEAP32[$269>>2] = $264;
      $270 = HEAP32[$4>>2]|0;
      $271 = HEAP32[$pNew>>2]|0;
      $272 = (_whereLoopInsert($270,$271)|0);
      HEAP32[$rc>>2] = $272;
      $273 = HEAP32[$i>>2]|0;
      $274 = (($273) + 1)|0;
      HEAP32[$i>>2] = $274;
     }
    }
   }
  } while(0);
  $275 = HEAP32[$pTerm>>2]|0;
  $276 = ((($275)) + 48|0);
  HEAP32[$pTerm>>2] = $276;
 }
 $277 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($277|0);
}
function _whereOrMove($pDest,$pSrc) {
 $pDest = $pDest|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pDest;
 HEAP32[$1>>2] = $pSrc;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 HEAP16[$4>>1] = $3;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11<<4;
 _memcpy(($6|0),($8|0),($12|0))|0;
 STACKTOP = sp;return;
}
function _whereOrInsert($pSet,$0,$1,$rRun,$nOut) {
 $pSet = $pSet|0;
 $0 = $0|0;
 $1 = $1|0;
 $rRun = $rRun|0;
 $nOut = $nOut|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp;
 $5 = sp + 24|0;
 $6 = sp + 22|0;
 $i = sp + 20|0;
 $p = sp + 8|0;
 HEAP32[$3>>2] = $pSet;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP16[$5>>1] = $rRun;
 HEAP16[$6>>1] = $nOut;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP16[$11>>1]|0;
 HEAP16[$i>>1] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = ((($13)) + 8|0);
 HEAP32[$p>>2] = $14;
 while(1) {
  $15 = HEAP16[$i>>1]|0;
  $16 = $15&65535;
  $17 = ($16|0)>(0);
  if (!($17)) {
   label = 9;
   break;
  }
  $18 = HEAP16[$5>>1]|0;
  $19 = $18 << 16 >> 16;
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP16[$21>>1]|0;
  $23 = $22 << 16 >> 16;
  $24 = ($19|0)<=($23|0);
  if ($24) {
   $25 = $4;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (($25) + 4)|0;
   $29 = $28;
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$p>>2]|0;
   $32 = $31;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = (($32) + 4)|0;
   $36 = $35;
   $37 = HEAP32[$36>>2]|0;
   $38 = $27 & $34;
   $39 = $30 & $37;
   $40 = $4;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = (($40) + 4)|0;
   $44 = $43;
   $45 = HEAP32[$44>>2]|0;
   $46 = ($38|0)==($42|0);
   $47 = ($39|0)==($45|0);
   $48 = $46 & $47;
   if ($48) {
    break;
   }
  }
  $49 = HEAP32[$p>>2]|0;
  $50 = ((($49)) + 8|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51 << 16 >> 16;
  $53 = HEAP16[$5>>1]|0;
  $54 = $53 << 16 >> 16;
  $55 = ($52|0)<=($54|0);
  if ($55) {
   $56 = HEAP32[$p>>2]|0;
   $57 = $56;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (($57) + 4)|0;
   $61 = $60;
   $62 = HEAP32[$61>>2]|0;
   $63 = $4;
   $64 = $63;
   $65 = HEAP32[$64>>2]|0;
   $66 = (($63) + 4)|0;
   $67 = $66;
   $68 = HEAP32[$67>>2]|0;
   $69 = $59 & $65;
   $70 = $62 & $68;
   $71 = HEAP32[$p>>2]|0;
   $72 = $71;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = ($69|0)==($74|0);
   $79 = ($70|0)==($77|0);
   $80 = $78 & $79;
   if ($80) {
    label = 7;
    break;
   }
  }
  $81 = HEAP16[$i>>1]|0;
  $82 = (($81) + -1)<<16>>16;
  HEAP16[$i>>1] = $82;
  $83 = HEAP32[$p>>2]|0;
  $84 = ((($83)) + 16|0);
  HEAP32[$p>>2] = $84;
 }
 if ((label|0) == 7) {
  HEAP32[$2>>2] = 0;
  $153 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($153|0);
 }
 do {
  if ((label|0) == 9) {
   $85 = HEAP32[$3>>2]|0;
   $86 = HEAP16[$85>>1]|0;
   $87 = $86&65535;
   $88 = ($87|0)<(3);
   $89 = HEAP32[$3>>2]|0;
   if ($88) {
    $90 = HEAP16[$89>>1]|0;
    $91 = (($90) + 1)<<16>>16;
    HEAP16[$89>>1] = $91;
    $92 = $90&65535;
    $93 = HEAP32[$3>>2]|0;
    $94 = ((($93)) + 8|0);
    $95 = (($94) + ($92<<4)|0);
    HEAP32[$p>>2] = $95;
    $96 = HEAP16[$6>>1]|0;
    $97 = HEAP32[$p>>2]|0;
    $98 = ((($97)) + 10|0);
    HEAP16[$98>>1] = $96;
    break;
   }
   $99 = ((($89)) + 8|0);
   HEAP32[$p>>2] = $99;
   HEAP16[$i>>1] = 1;
   while(1) {
    $100 = HEAP16[$i>>1]|0;
    $101 = $100&65535;
    $102 = HEAP32[$3>>2]|0;
    $103 = HEAP16[$102>>1]|0;
    $104 = $103&65535;
    $105 = ($101|0)<($104|0);
    $106 = HEAP32[$p>>2]|0;
    $107 = ((($106)) + 8|0);
    $108 = HEAP16[$107>>1]|0;
    $109 = $108 << 16 >> 16;
    if (!($105)) {
     break;
    }
    $110 = HEAP16[$i>>1]|0;
    $111 = $110&65535;
    $112 = HEAP32[$3>>2]|0;
    $113 = ((($112)) + 8|0);
    $114 = (($113) + ($111<<4)|0);
    $115 = ((($114)) + 8|0);
    $116 = HEAP16[$115>>1]|0;
    $117 = $116 << 16 >> 16;
    $118 = ($109|0)>($117|0);
    if ($118) {
     $119 = HEAP32[$3>>2]|0;
     $120 = ((($119)) + 8|0);
     $121 = HEAP16[$i>>1]|0;
     $122 = $121&65535;
     $123 = (($120) + ($122<<4)|0);
     HEAP32[$p>>2] = $123;
    }
    $124 = HEAP16[$i>>1]|0;
    $125 = (($124) + 1)<<16>>16;
    HEAP16[$i>>1] = $125;
   }
   $126 = HEAP16[$5>>1]|0;
   $127 = $126 << 16 >> 16;
   $128 = ($109|0)<=($127|0);
   if ($128) {
    HEAP32[$2>>2] = 0;
    $153 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($153|0);
   }
  }
 } while(0);
 $129 = $4;
 $130 = $129;
 $131 = HEAP32[$130>>2]|0;
 $132 = (($129) + 4)|0;
 $133 = $132;
 $134 = HEAP32[$133>>2]|0;
 $135 = HEAP32[$p>>2]|0;
 $136 = $135;
 $137 = $136;
 HEAP32[$137>>2] = $131;
 $138 = (($136) + 4)|0;
 $139 = $138;
 HEAP32[$139>>2] = $134;
 $140 = HEAP16[$5>>1]|0;
 $141 = HEAP32[$p>>2]|0;
 $142 = ((($141)) + 8|0);
 HEAP16[$142>>1] = $140;
 $143 = HEAP32[$p>>2]|0;
 $144 = ((($143)) + 10|0);
 $145 = HEAP16[$144>>1]|0;
 $146 = $145 << 16 >> 16;
 $147 = HEAP16[$6>>1]|0;
 $148 = $147 << 16 >> 16;
 $149 = ($146|0)>($148|0);
 if ($149) {
  $150 = HEAP16[$6>>1]|0;
  $151 = HEAP32[$p>>2]|0;
  $152 = ((($151)) + 10|0);
  HEAP16[$152>>1] = $150;
 }
 HEAP32[$2>>2] = 1;
 $153 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($153|0);
}
function _whereLoopInsert($pBuilder,$pTemplate) {
 $pBuilder = $pBuilder|0;
 $pTemplate = $pTemplate|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $db = 0, $p = 0;
 var $pIndex = 0, $pToDel = 0, $pWInfo = 0, $ppPrev = 0, $ppTail = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $ppPrev = sp + 28|0;
 $p = sp + 24|0;
 $pWInfo = sp + 20|0;
 $db = sp + 16|0;
 $rc = sp + 12|0;
 $ppTail = sp + 8|0;
 $pToDel = sp + 4|0;
 $pIndex = sp;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pTemplate;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = HEAP32[$pWInfo>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 40|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = ($14<<16>>16)!=(0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$2>>2]|0;
   $20 = $19;
   $21 = $20;
   $22 = HEAP32[$21>>2]|0;
   $23 = (($20) + 4)|0;
   $24 = $23;
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$2>>2]|0;
   $27 = ((($26)) + 20|0);
   $28 = HEAP16[$27>>1]|0;
   $29 = HEAP32[$2>>2]|0;
   $30 = ((($29)) + 22|0);
   $31 = HEAP16[$30>>1]|0;
   (_whereOrInsert($18,$22,$25,$28,$31)|0);
  }
  HEAP32[$0>>2] = 0;
  $97 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($97|0);
 }
 $32 = HEAP32[$pWInfo>>2]|0;
 $33 = ((($32)) + 56|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = HEAP32[$2>>2]|0;
 _whereLoopAdjustCost($34,$35);
 $36 = HEAP32[$pWInfo>>2]|0;
 $37 = ((($36)) + 56|0);
 $38 = HEAP32[$2>>2]|0;
 $39 = (_whereLoopFindLesser($37,$38)|0);
 HEAP32[$ppPrev>>2] = $39;
 $40 = HEAP32[$ppPrev>>2]|0;
 $41 = ($40|0)==(0|0);
 if ($41) {
  HEAP32[$0>>2] = 0;
  $97 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($97|0);
 }
 $42 = HEAP32[$ppPrev>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$p>>2] = $43;
 $44 = HEAP32[$p>>2]|0;
 $45 = ($44|0)==(0|0);
 L12: do {
  if ($45) {
   $46 = HEAP32[$db>>2]|0;
   $47 = (_sqlite3DbMallocRawNN($46,72,0)|0);
   HEAP32[$p>>2] = $47;
   $48 = HEAP32[$ppPrev>>2]|0;
   HEAP32[$48>>2] = $47;
   $49 = HEAP32[$p>>2]|0;
   $50 = ($49|0)==(0|0);
   if (!($50)) {
    $51 = HEAP32[$p>>2]|0;
    _whereLoopInit($51);
    $52 = HEAP32[$p>>2]|0;
    $53 = ((($52)) + 52|0);
    HEAP32[$53>>2] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $97 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($97|0);
  } else {
   $54 = HEAP32[$p>>2]|0;
   $55 = ((($54)) + 52|0);
   HEAP32[$ppTail>>2] = $55;
   while(1) {
    $56 = HEAP32[$ppTail>>2]|0;
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0|0);
    if (!($58)) {
     break L12;
    }
    $59 = HEAP32[$ppTail>>2]|0;
    $60 = HEAP32[$2>>2]|0;
    $61 = (_whereLoopFindLesser($59,$60)|0);
    HEAP32[$ppTail>>2] = $61;
    $62 = HEAP32[$ppTail>>2]|0;
    $63 = ($62|0)==(0|0);
    if ($63) {
     break L12;
    }
    $64 = HEAP32[$ppTail>>2]|0;
    $65 = HEAP32[$64>>2]|0;
    HEAP32[$pToDel>>2] = $65;
    $66 = HEAP32[$pToDel>>2]|0;
    $67 = ($66|0)==(0|0);
    if ($67) {
     break L12;
    }
    $68 = HEAP32[$pToDel>>2]|0;
    $69 = ((($68)) + 52|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = HEAP32[$ppTail>>2]|0;
    HEAP32[$71>>2] = $70;
    $72 = HEAP32[$db>>2]|0;
    $73 = HEAP32[$pToDel>>2]|0;
    _whereLoopDelete($72,$73);
   }
  }
 } while(0);
 $74 = HEAP32[$db>>2]|0;
 $75 = HEAP32[$p>>2]|0;
 $76 = HEAP32[$2>>2]|0;
 $77 = (_whereLoopXfer($74,$75,$76)|0);
 HEAP32[$rc>>2] = $77;
 $78 = HEAP32[$p>>2]|0;
 $79 = ((($78)) + 36|0);
 $80 = HEAP32[$79>>2]|0;
 $81 = $80 & 1024;
 $82 = ($81|0)==(0);
 if ($82) {
  $83 = HEAP32[$p>>2]|0;
  $84 = ((($83)) + 24|0);
  $85 = ((($84)) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  HEAP32[$pIndex>>2] = $86;
  $87 = HEAP32[$pIndex>>2]|0;
  $88 = ($87|0)!=(0|0);
  if ($88) {
   $89 = HEAP32[$pIndex>>2]|0;
   $90 = ((($89)) + 44|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = ($91|0)==(0);
   if ($92) {
    $93 = HEAP32[$p>>2]|0;
    $94 = ((($93)) + 24|0);
    $95 = ((($94)) + 8|0);
    HEAP32[$95>>2] = 0;
   }
  }
 }
 $96 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $96;
 $97 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($97|0);
}
function _whereLoopAdjustCost($p,$pTemplate) {
 $p = $p|0;
 $pTemplate = $pTemplate|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pTemplate;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4 & 512;
 $6 = ($5|0)!=(0);
 $7 = HEAP32[$0>>2]|0;
 $8 = ($7|0)!=(0|0);
 $or$cond = $6 & $8;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 16|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($12|0)!=($16|0);
  do {
   if (!($17)) {
    $18 = HEAP32[$0>>2]|0;
    $19 = ((($18)) + 36|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $20 & 512;
    $22 = ($21|0)==(0);
    if (!($22)) {
     $23 = HEAP32[$0>>2]|0;
     $24 = HEAP32[$1>>2]|0;
     $25 = (_whereLoopCheaperProperSubset($23,$24)|0);
     $26 = ($25|0)!=(0);
     if ($26) {
      $27 = HEAP32[$0>>2]|0;
      $28 = ((($27)) + 20|0);
      $29 = HEAP16[$28>>1]|0;
      $30 = HEAP32[$1>>2]|0;
      $31 = ((($30)) + 20|0);
      HEAP16[$31>>1] = $29;
      $32 = HEAP32[$0>>2]|0;
      $33 = ((($32)) + 22|0);
      $34 = HEAP16[$33>>1]|0;
      $35 = $34 << 16 >> 16;
      $36 = (($35) - 1)|0;
      $37 = $36&65535;
      $38 = HEAP32[$1>>2]|0;
      $39 = ((($38)) + 22|0);
      HEAP16[$39>>1] = $37;
      break;
     }
     $40 = HEAP32[$1>>2]|0;
     $41 = HEAP32[$0>>2]|0;
     $42 = (_whereLoopCheaperProperSubset($40,$41)|0);
     $43 = ($42|0)!=(0);
     if ($43) {
      $44 = HEAP32[$0>>2]|0;
      $45 = ((($44)) + 20|0);
      $46 = HEAP16[$45>>1]|0;
      $47 = HEAP32[$1>>2]|0;
      $48 = ((($47)) + 20|0);
      HEAP16[$48>>1] = $46;
      $49 = HEAP32[$0>>2]|0;
      $50 = ((($49)) + 22|0);
      $51 = HEAP16[$50>>1]|0;
      $52 = $51 << 16 >> 16;
      $53 = (($52) + 1)|0;
      $54 = $53&65535;
      $55 = HEAP32[$1>>2]|0;
      $56 = ((($55)) + 22|0);
      HEAP16[$56>>1] = $54;
     }
    }
   }
  } while(0);
  $57 = HEAP32[$0>>2]|0;
  $58 = ((($57)) + 52|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$0>>2] = $59;
  $$old = HEAP32[$0>>2]|0;
  $$old1 = ($$old|0)!=(0|0);
  if (!($$old1)) {
   break;
  }
 }
 STACKTOP = sp;return;
}
function _whereLoopFindLesser($ppPrev,$pTemplate) {
 $ppPrev = $ppPrev|0;
 $pTemplate = $pTemplate|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $ppPrev;
 HEAP32[$2>>2] = $pTemplate;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 while(1) {
  $5 = HEAP32[$p>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   label = 19;
   break;
  }
  $7 = HEAP32[$p>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9&255;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 16|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = ($10|0)!=($14|0);
  if (!($15)) {
   $16 = HEAP32[$p>>2]|0;
   $17 = ((($16)) + 17|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 17|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = ($19|0)!=($23|0);
   if (!($24)) {
    $25 = HEAP32[$p>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = $27 & 16384;
    $29 = ($28|0)!=(0);
    if ($29) {
     $30 = HEAP32[$2>>2]|0;
     $31 = ((($30)) + 42|0);
     $32 = HEAP16[$31>>1]|0;
     $33 = $32&65535;
     $34 = ($33|0)==(0);
     if ($34) {
      $35 = HEAP32[$2>>2]|0;
      $36 = ((($35)) + 36|0);
      $37 = HEAP32[$36>>2]|0;
      $38 = $37 & 512;
      $39 = ($38|0)!=(0);
      if ($39) {
       $40 = HEAP32[$2>>2]|0;
       $41 = ((($40)) + 36|0);
       $42 = HEAP32[$41>>2]|0;
       $43 = $42 & 1;
       $44 = ($43|0)!=(0);
       if ($44) {
        $45 = HEAP32[$p>>2]|0;
        $46 = $45;
        $47 = $46;
        $48 = HEAP32[$47>>2]|0;
        $49 = (($46) + 4)|0;
        $50 = $49;
        $51 = HEAP32[$50>>2]|0;
        $52 = HEAP32[$2>>2]|0;
        $53 = $52;
        $54 = $53;
        $55 = HEAP32[$54>>2]|0;
        $56 = (($53) + 4)|0;
        $57 = $56;
        $58 = HEAP32[$57>>2]|0;
        $59 = $48 & $55;
        $60 = $51 & $58;
        $61 = HEAP32[$2>>2]|0;
        $62 = $61;
        $63 = $62;
        $64 = HEAP32[$63>>2]|0;
        $65 = (($62) + 4)|0;
        $66 = $65;
        $67 = HEAP32[$66>>2]|0;
        $68 = ($59|0)==($64|0);
        $69 = ($60|0)==($67|0);
        $70 = $68 & $69;
        if ($70) {
         label = 19;
         break;
        }
       }
      }
     }
    }
    $71 = HEAP32[$p>>2]|0;
    $72 = $71;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = (($72) + 4)|0;
    $76 = $75;
    $77 = HEAP32[$76>>2]|0;
    $78 = HEAP32[$2>>2]|0;
    $79 = $78;
    $80 = $79;
    $81 = HEAP32[$80>>2]|0;
    $82 = (($79) + 4)|0;
    $83 = $82;
    $84 = HEAP32[$83>>2]|0;
    $85 = $74 & $81;
    $86 = $77 & $84;
    $87 = HEAP32[$p>>2]|0;
    $88 = $87;
    $89 = $88;
    $90 = HEAP32[$89>>2]|0;
    $91 = (($88) + 4)|0;
    $92 = $91;
    $93 = HEAP32[$92>>2]|0;
    $94 = ($85|0)==($90|0);
    $95 = ($86|0)==($93|0);
    $96 = $94 & $95;
    if ($96) {
     $97 = HEAP32[$p>>2]|0;
     $98 = ((($97)) + 18|0);
     $99 = HEAP16[$98>>1]|0;
     $100 = $99 << 16 >> 16;
     $101 = HEAP32[$2>>2]|0;
     $102 = ((($101)) + 18|0);
     $103 = HEAP16[$102>>1]|0;
     $104 = $103 << 16 >> 16;
     $105 = ($100|0)<=($104|0);
     if ($105) {
      $106 = HEAP32[$p>>2]|0;
      $107 = ((($106)) + 20|0);
      $108 = HEAP16[$107>>1]|0;
      $109 = $108 << 16 >> 16;
      $110 = HEAP32[$2>>2]|0;
      $111 = ((($110)) + 20|0);
      $112 = HEAP16[$111>>1]|0;
      $113 = $112 << 16 >> 16;
      $114 = ($109|0)<=($113|0);
      if ($114) {
       $115 = HEAP32[$p>>2]|0;
       $116 = ((($115)) + 22|0);
       $117 = HEAP16[$116>>1]|0;
       $118 = $117 << 16 >> 16;
       $119 = HEAP32[$2>>2]|0;
       $120 = ((($119)) + 22|0);
       $121 = HEAP16[$120>>1]|0;
       $122 = $121 << 16 >> 16;
       $123 = ($118|0)<=($122|0);
       if ($123) {
        label = 14;
        break;
       }
      }
     }
    }
    $124 = HEAP32[$p>>2]|0;
    $125 = $124;
    $126 = $125;
    $127 = HEAP32[$126>>2]|0;
    $128 = (($125) + 4)|0;
    $129 = $128;
    $130 = HEAP32[$129>>2]|0;
    $131 = HEAP32[$2>>2]|0;
    $132 = $131;
    $133 = $132;
    $134 = HEAP32[$133>>2]|0;
    $135 = (($132) + 4)|0;
    $136 = $135;
    $137 = HEAP32[$136>>2]|0;
    $138 = $127 & $134;
    $139 = $130 & $137;
    $140 = HEAP32[$2>>2]|0;
    $141 = $140;
    $142 = $141;
    $143 = HEAP32[$142>>2]|0;
    $144 = (($141) + 4)|0;
    $145 = $144;
    $146 = HEAP32[$145>>2]|0;
    $147 = ($138|0)==($143|0);
    $148 = ($139|0)==($146|0);
    $149 = $147 & $148;
    if ($149) {
     $150 = HEAP32[$p>>2]|0;
     $151 = ((($150)) + 20|0);
     $152 = HEAP16[$151>>1]|0;
     $153 = $152 << 16 >> 16;
     $154 = HEAP32[$2>>2]|0;
     $155 = ((($154)) + 20|0);
     $156 = HEAP16[$155>>1]|0;
     $157 = $156 << 16 >> 16;
     $158 = ($153|0)>=($157|0);
     if ($158) {
      $159 = HEAP32[$p>>2]|0;
      $160 = ((($159)) + 22|0);
      $161 = HEAP16[$160>>1]|0;
      $162 = $161 << 16 >> 16;
      $163 = HEAP32[$2>>2]|0;
      $164 = ((($163)) + 22|0);
      $165 = HEAP16[$164>>1]|0;
      $166 = $165 << 16 >> 16;
      $167 = ($162|0)>=($166|0);
      if ($167) {
       label = 19;
       break;
      }
     }
    }
   }
  }
  $168 = HEAP32[$p>>2]|0;
  $169 = ((($168)) + 52|0);
  HEAP32[$1>>2] = $169;
  $170 = HEAP32[$1>>2]|0;
  $171 = HEAP32[$170>>2]|0;
  HEAP32[$p>>2] = $171;
 }
 if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $173 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($173|0);
 }
 else if ((label|0) == 19) {
  $172 = HEAP32[$1>>2]|0;
  HEAP32[$0>>2] = $172;
  $173 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($173|0);
 }
 return (0)|0;
}
function _whereLoopXfer($db,$pTo,$pFrom) {
 $db = $db|0;
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pTo;
 HEAP32[$3>>2] = $pFrom;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _whereLoopClearUnion($4,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 40|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = (_whereLoopResize($6,$7,$11)|0);
 $13 = ($12|0)!=(0);
 $14 = HEAP32[$2>>2]|0;
 if ($13) {
  $15 = ((($14)) + 24|0);
  ;HEAP32[$15>>2]=0|0;HEAP32[$15+4>>2]=0|0;HEAP32[$15+8>>2]=0|0;
  HEAP32[$0>>2] = 7;
  $43 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($43|0);
 }
 $16 = HEAP32[$3>>2]|0;
 dest=$14; src=$16; stop=dest+44|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$3>>2]|0;
 $21 = ((($20)) + 48|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 40|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26<<2;
 _memcpy(($19|0),($22|0),($27|0))|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = ((($28)) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 & 1024;
 $32 = ($31|0)!=(0);
 $33 = HEAP32[$3>>2]|0;
 if ($32) {
  $34 = ((($33)) + 24|0);
  $35 = ((($34)) + 4|0);
  HEAP8[$35>>0] = 0;
 } else {
  $36 = ((($33)) + 36|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = $37 & 16384;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$3>>2]|0;
   $41 = ((($40)) + 24|0);
   $42 = ((($41)) + 8|0);
   HEAP32[$42>>2] = 0;
  }
 }
 HEAP32[$0>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _whereLoopCheaperProperSubset($pX,$pY) {
 $pX = $pX|0;
 $pY = $pY|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $i = 0;
 var $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pX;
 HEAP32[$2>>2] = $pY;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 42|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = (($6) - ($10))|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 40|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = $14&65535;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 42|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = (($15) - ($19))|0;
 $21 = ($11|0)>=($20|0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = ((($22)) + 42|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 42|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = ($25|0)>($29|0);
 if ($30) {
  HEAP32[$0>>2] = 0;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 20|0);
 $33 = HEAP16[$32>>1]|0;
 $34 = $33 << 16 >> 16;
 $35 = HEAP32[$2>>2]|0;
 $36 = ((($35)) + 20|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37 << 16 >> 16;
 $39 = ($34|0)>=($38|0);
 if ($39) {
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 20|0);
  $42 = HEAP16[$41>>1]|0;
  $43 = $42 << 16 >> 16;
  $44 = HEAP32[$2>>2]|0;
  $45 = ((($44)) + 20|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46 << 16 >> 16;
  $48 = ($43|0)>($47|0);
  if ($48) {
   HEAP32[$0>>2] = 0;
   $98 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($98|0);
  }
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 22|0);
  $51 = HEAP16[$50>>1]|0;
  $52 = $51 << 16 >> 16;
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 22|0);
  $55 = HEAP16[$54>>1]|0;
  $56 = $55 << 16 >> 16;
  $57 = ($52|0)>($56|0);
  if ($57) {
   HEAP32[$0>>2] = 0;
   $98 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($98|0);
  }
 }
 $58 = HEAP32[$1>>2]|0;
 $59 = ((($58)) + 40|0);
 $60 = HEAP16[$59>>1]|0;
 $61 = $60&65535;
 $62 = (($61) - 1)|0;
 HEAP32[$i>>2] = $62;
 while(1) {
  $63 = HEAP32[$i>>2]|0;
  $64 = ($63|0)>=(0);
  if (!($64)) {
   label = 20;
   break;
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$1>>2]|0;
  $67 = ((($66)) + 48|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = (($68) + ($65<<2)|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = ($70|0)==(0|0);
  if (!($71)) {
   $72 = HEAP32[$2>>2]|0;
   $73 = ((($72)) + 40|0);
   $74 = HEAP16[$73>>1]|0;
   $75 = $74&65535;
   $76 = (($75) - 1)|0;
   HEAP32[$j>>2] = $76;
   while(1) {
    $77 = HEAP32[$j>>2]|0;
    $78 = ($77|0)>=(0);
    if (!($78)) {
     break;
    }
    $79 = HEAP32[$j>>2]|0;
    $80 = HEAP32[$2>>2]|0;
    $81 = ((($80)) + 48|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = (($82) + ($79<<2)|0);
    $84 = HEAP32[$83>>2]|0;
    $85 = HEAP32[$i>>2]|0;
    $86 = HEAP32[$1>>2]|0;
    $87 = ((($86)) + 48|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = (($88) + ($85<<2)|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = ($84|0)==($90|0);
    if ($91) {
     break;
    }
    $92 = HEAP32[$j>>2]|0;
    $93 = (($92) + -1)|0;
    HEAP32[$j>>2] = $93;
   }
   $94 = HEAP32[$j>>2]|0;
   $95 = ($94|0)<(0);
   if ($95) {
    label = 18;
    break;
   }
  }
  $96 = HEAP32[$i>>2]|0;
  $97 = (($96) + -1)|0;
  HEAP32[$i>>2] = $97;
 }
 if ((label|0) == 18) {
  HEAP32[$0>>2] = 0;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 else if ((label|0) == 20) {
  HEAP32[$0>>2] = 1;
  $98 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($98|0);
 }
 return (0)|0;
}
function _whereUsablePartialIndex($iTab,$pWC,$pWhere) {
 $iTab = $iTab|0;
 $pWC = $pWC|0;
 $pWhere = $pWhere|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pExpr = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $i = sp + 8|0;
 $pTerm = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $pWC;
 HEAP32[$3>>2] = $pWhere;
 while(1) {
  $4 = HEAP32[$3>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5&255;
  $7 = ($6|0)==(28);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = HEAP32[$3>>2]|0;
  $11 = ((($10)) + 12|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_whereUsablePartialIndex($8,$9,$12)|0);
  $14 = ($13|0)!=(0);
  if (!($14)) {
   label = 4;
   break;
  }
  $15 = HEAP32[$3>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$3>>2] = $17;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 HEAP32[$i>>2] = 0;
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pTerm>>2] = $20;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($21|0)<($24|0);
  if (!($25)) {
   label = 13;
   break;
  }
  $26 = HEAP32[$pTerm>>2]|0;
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pExpr>>2] = $27;
  $28 = HEAP32[$pExpr>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = HEAP32[$1>>2]|0;
  $31 = (_sqlite3ExprImpliesExpr($28,$29,$30)|0);
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = HEAP32[$pExpr>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = $35 & 1;
   $37 = ($36|0)!=(0);
   if (!($37)) {
    label = 11;
    break;
   }
   $38 = HEAP32[$pExpr>>2]|0;
   $39 = ((($38)) + 36|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40 << 16 >> 16;
   $42 = HEAP32[$1>>2]|0;
   $43 = ($41|0)==($42|0);
   if ($43) {
    label = 11;
    break;
   }
  }
  $44 = HEAP32[$i>>2]|0;
  $45 = (($44) + 1)|0;
  HEAP32[$i>>2] = $45;
  $46 = HEAP32[$pTerm>>2]|0;
  $47 = ((($46)) + 48|0);
  HEAP32[$pTerm>>2] = $47;
 }
 if ((label|0) == 11) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 return (0)|0;
}
function _indexMightHelpWithOrderBy($pBuilder,$pIndex,$iCursor) {
 $pBuilder = $pBuilder|0;
 $pIndex = $pIndex|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $aColExpr = 0, $ii = 0, $jj = 0, $pExpr = 0, $pOB = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pOB = sp + 16|0;
 $aColExpr = sp + 12|0;
 $ii = sp + 8|0;
 $jj = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pIndex;
 HEAP32[$3>>2] = $iCursor;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 55|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6&255) >>> 2;
 $8 = $7 & 1;
 $9 = $8&255;
 $10 = ($9|0)!=(0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pOB>>2] = $14;
 $15 = ($14|0)==(0|0);
 if ($15) {
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 HEAP32[$ii>>2] = 0;
 L9: while(1) {
  $16 = HEAP32[$ii>>2]|0;
  $17 = HEAP32[$pOB>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ($16|0)<($18|0);
  if (!($19)) {
   label = 24;
   break;
  }
  $20 = HEAP32[$ii>>2]|0;
  $21 = HEAP32[$pOB>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (_sqlite3ExprSkipCollate($25)|0);
  HEAP32[$pExpr>>2] = $26;
  $27 = HEAP32[$pExpr>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(152);
  L12: do {
   if ($30) {
    $31 = HEAP32[$pExpr>>2]|0;
    $32 = ((($31)) + 28|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = HEAP32[$3>>2]|0;
    $35 = ($33|0)==($34|0);
    if ($35) {
     $36 = HEAP32[$pExpr>>2]|0;
     $37 = ((($36)) + 32|0);
     $38 = HEAP16[$37>>1]|0;
     $39 = $38 << 16 >> 16;
     $40 = ($39|0)<(0);
     if ($40) {
      label = 10;
      break L9;
     }
     HEAP32[$jj>>2] = 0;
     while(1) {
      $41 = HEAP32[$jj>>2]|0;
      $42 = HEAP32[$2>>2]|0;
      $43 = ((($42)) + 50|0);
      $44 = HEAP16[$43>>1]|0;
      $45 = $44&65535;
      $46 = ($41|0)<($45|0);
      if (!($46)) {
       break L12;
      }
      $47 = HEAP32[$pExpr>>2]|0;
      $48 = ((($47)) + 32|0);
      $49 = HEAP16[$48>>1]|0;
      $50 = $49 << 16 >> 16;
      $51 = HEAP32[$jj>>2]|0;
      $52 = HEAP32[$2>>2]|0;
      $53 = ((($52)) + 4|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = (($54) + ($51<<1)|0);
      $56 = HEAP16[$55>>1]|0;
      $57 = $56 << 16 >> 16;
      $58 = ($50|0)==($57|0);
      if ($58) {
       label = 14;
       break L9;
      }
      $59 = HEAP32[$jj>>2]|0;
      $60 = (($59) + 1)|0;
      HEAP32[$jj>>2] = $60;
     }
    } else {
     label = 16;
    }
   } else {
    label = 16;
   }
  } while(0);
  L20: do {
   if ((label|0) == 16) {
    label = 0;
    $61 = HEAP32[$2>>2]|0;
    $62 = ((($61)) + 40|0);
    $63 = HEAP32[$62>>2]|0;
    HEAP32[$aColExpr>>2] = $63;
    $64 = ($63|0)!=(0|0);
    if ($64) {
     HEAP32[$jj>>2] = 0;
     while(1) {
      $65 = HEAP32[$jj>>2]|0;
      $66 = HEAP32[$2>>2]|0;
      $67 = ((($66)) + 50|0);
      $68 = HEAP16[$67>>1]|0;
      $69 = $68&65535;
      $70 = ($65|0)<($69|0);
      if (!($70)) {
       break L20;
      }
      $71 = HEAP32[$jj>>2]|0;
      $72 = HEAP32[$2>>2]|0;
      $73 = ((($72)) + 4|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = (($74) + ($71<<1)|0);
      $76 = HEAP16[$75>>1]|0;
      $77 = $76 << 16 >> 16;
      $78 = ($77|0)!=(-2);
      if (!($78)) {
       $79 = HEAP32[$pExpr>>2]|0;
       $80 = HEAP32[$jj>>2]|0;
       $81 = HEAP32[$aColExpr>>2]|0;
       $82 = ((($81)) + 4|0);
       $83 = HEAP32[$82>>2]|0;
       $84 = (($83) + (($80*20)|0)|0);
       $85 = HEAP32[$84>>2]|0;
       $86 = HEAP32[$3>>2]|0;
       $87 = (_sqlite3ExprCompare($79,$85,$86)|0);
       $88 = ($87|0)==(0);
       if ($88) {
        label = 21;
        break L9;
       }
      }
      $89 = HEAP32[$jj>>2]|0;
      $90 = (($89) + 1)|0;
      HEAP32[$jj>>2] = $90;
     }
    }
   }
  } while(0);
  $91 = HEAP32[$ii>>2]|0;
  $92 = (($91) + 1)|0;
  HEAP32[$ii>>2] = $92;
 }
 if ((label|0) == 10) {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 else if ((label|0) == 21) {
  HEAP32[$0>>2] = 1;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 else if ((label|0) == 24) {
  HEAP32[$0>>2] = 0;
  $93 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 return (0)|0;
}
function _whereLoopOutputAdjust($pWC,$pLoop,$nRow) {
 $pWC = $pWC|0;
 $pLoop = $pLoop|0;
 $nRow = $nRow|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0;
 var $iReduce = 0, $j = 0, $k = 0, $notAllowed = 0, $or$cond = 0, $or$cond3 = 0, $pRight = 0, $pTerm = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 42|0;
 $pTerm = sp + 28|0;
 $pX = sp + 24|0;
 $notAllowed = sp;
 $i = sp + 20|0;
 $j = sp + 16|0;
 $k = sp + 12|0;
 $iReduce = sp + 40|0;
 $pRight = sp + 8|0;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $pLoop;
 HEAP16[$2>>1] = $nRow;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = $6 | $14;
 $19 = $9 | $17;
 $20 = $18 ^ -1;
 $21 = $19 ^ -1;
 $22 = $notAllowed;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 HEAP16[$iReduce>>1] = 0;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$i>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pTerm>>2] = $31;
 while(1) {
  $32 = HEAP32[$i>>2]|0;
  $33 = ($32|0)>(0);
  if (!($33)) {
   break;
  }
  $34 = HEAP32[$pTerm>>2]|0;
  $35 = ((($34)) + 10|0);
  $36 = HEAP16[$35>>1]|0;
  $37 = $36&65535;
  $38 = $37 & 2;
  $39 = ($38|0)!=(0);
  if ($39) {
   break;
  }
  $40 = HEAP32[$pTerm>>2]|0;
  $41 = ((($40)) + 40|0);
  $42 = $41;
  $43 = $42;
  $44 = HEAP32[$43>>2]|0;
  $45 = (($42) + 4)|0;
  $46 = $45;
  $47 = HEAP32[$46>>2]|0;
  $48 = HEAP32[$1>>2]|0;
  $49 = ((($48)) + 8|0);
  $50 = $49;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = $44 & $52;
  $57 = $47 & $55;
  $58 = ($56|0)==(0);
  $59 = ($57|0)==(0);
  $60 = $58 & $59;
  do {
   if (!($60)) {
    $61 = HEAP32[$pTerm>>2]|0;
    $62 = ((($61)) + 40|0);
    $63 = $62;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (($63) + 4)|0;
    $67 = $66;
    $68 = HEAP32[$67>>2]|0;
    $69 = $notAllowed;
    $70 = $69;
    $71 = HEAP32[$70>>2]|0;
    $72 = (($69) + 4)|0;
    $73 = $72;
    $74 = HEAP32[$73>>2]|0;
    $75 = $65 & $71;
    $76 = $68 & $74;
    $77 = ($75|0)!=(0);
    $78 = ($76|0)!=(0);
    $79 = $77 | $78;
    if (!($79)) {
     $80 = HEAP32[$1>>2]|0;
     $81 = ((($80)) + 40|0);
     $82 = HEAP16[$81>>1]|0;
     $83 = $82&65535;
     $84 = (($83) - 1)|0;
     HEAP32[$j>>2] = $84;
     while(1) {
      $85 = HEAP32[$j>>2]|0;
      $86 = ($85|0)>=(0);
      if (!($86)) {
       break;
      }
      $87 = HEAP32[$j>>2]|0;
      $88 = HEAP32[$1>>2]|0;
      $89 = ((($88)) + 48|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = (($90) + ($87<<2)|0);
      $92 = HEAP32[$91>>2]|0;
      HEAP32[$pX>>2] = $92;
      $93 = HEAP32[$pX>>2]|0;
      $94 = ($93|0)==(0|0);
      if (!($94)) {
       $95 = HEAP32[$pX>>2]|0;
       $96 = HEAP32[$pTerm>>2]|0;
       $97 = ($95|0)==($96|0);
       if ($97) {
        break;
       }
       $98 = HEAP32[$pX>>2]|0;
       $99 = ((($98)) + 16|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = ($100|0)>=(0);
       if ($101) {
        $102 = HEAP32[$pX>>2]|0;
        $103 = ((($102)) + 16|0);
        $104 = HEAP32[$103>>2]|0;
        $105 = HEAP32[$0>>2]|0;
        $106 = ((($105)) + 20|0);
        $107 = HEAP32[$106>>2]|0;
        $108 = (($107) + (($104*48)|0)|0);
        $109 = HEAP32[$pTerm>>2]|0;
        $110 = ($108|0)==($109|0);
        if ($110) {
         break;
        }
       }
      }
      $111 = HEAP32[$j>>2]|0;
      $112 = (($111) + -1)|0;
      HEAP32[$j>>2] = $112;
     }
     $113 = HEAP32[$j>>2]|0;
     $114 = ($113|0)<(0);
     if ($114) {
      $115 = HEAP32[$pTerm>>2]|0;
      $116 = ((($115)) + 8|0);
      $117 = HEAP16[$116>>1]|0;
      $118 = $117 << 16 >> 16;
      $119 = ($118|0)<=(0);
      if ($119) {
       $120 = HEAP32[$pTerm>>2]|0;
       $121 = ((($120)) + 8|0);
       $122 = HEAP16[$121>>1]|0;
       $123 = $122 << 16 >> 16;
       $124 = HEAP32[$1>>2]|0;
       $125 = ((($124)) + 22|0);
       $126 = HEAP16[$125>>1]|0;
       $127 = $126 << 16 >> 16;
       $128 = (($127) + ($123))|0;
       $129 = $128&65535;
       HEAP16[$125>>1] = $129;
       break;
      }
      $130 = HEAP32[$1>>2]|0;
      $131 = ((($130)) + 22|0);
      $132 = HEAP16[$131>>1]|0;
      $133 = (($132) + -1)<<16>>16;
      HEAP16[$131>>1] = $133;
      $134 = HEAP32[$pTerm>>2]|0;
      $135 = ((($134)) + 12|0);
      $136 = HEAP16[$135>>1]|0;
      $137 = $136&65535;
      $138 = $137 & 130;
      $139 = ($138|0)!=(0);
      if ($139) {
       $140 = HEAP32[$pTerm>>2]|0;
       $141 = HEAP32[$140>>2]|0;
       $142 = ((($141)) + 16|0);
       $143 = HEAP32[$142>>2]|0;
       HEAP32[$pRight>>2] = $143;
       $144 = HEAP32[$pRight>>2]|0;
       $145 = (_sqlite3ExprIsInteger($144,$k)|0);
       $146 = ($145|0)!=(0);
       $147 = HEAP32[$k>>2]|0;
       $148 = ($147|0)>=(-1);
       $or$cond = $146 & $148;
       $149 = HEAP32[$k>>2]|0;
       $150 = ($149|0)<=(1);
       $or$cond3 = $or$cond & $150;
       if ($or$cond3) {
        HEAP32[$k>>2] = 10;
       } else {
        HEAP32[$k>>2] = 20;
       }
       $151 = HEAP16[$iReduce>>1]|0;
       $152 = $151 << 16 >> 16;
       $153 = HEAP32[$k>>2]|0;
       $154 = ($152|0)<($153|0);
       if ($154) {
        $155 = HEAP32[$k>>2]|0;
        $156 = $155&65535;
        HEAP16[$iReduce>>1] = $156;
       }
      }
     }
    }
   }
  } while(0);
  $157 = HEAP32[$i>>2]|0;
  $158 = (($157) + -1)|0;
  HEAP32[$i>>2] = $158;
  $159 = HEAP32[$pTerm>>2]|0;
  $160 = ((($159)) + 48|0);
  HEAP32[$pTerm>>2] = $160;
 }
 $161 = HEAP32[$1>>2]|0;
 $162 = ((($161)) + 22|0);
 $163 = HEAP16[$162>>1]|0;
 $164 = $163 << 16 >> 16;
 $165 = HEAP16[$2>>1]|0;
 $166 = $165 << 16 >> 16;
 $167 = HEAP16[$iReduce>>1]|0;
 $168 = $167 << 16 >> 16;
 $169 = (($166) - ($168))|0;
 $170 = ($164|0)>($169|0);
 if (!($170)) {
  STACKTOP = sp;return;
 }
 $171 = HEAP16[$2>>1]|0;
 $172 = $171 << 16 >> 16;
 $173 = HEAP16[$iReduce>>1]|0;
 $174 = $173 << 16 >> 16;
 $175 = (($172) - ($174))|0;
 $176 = $175&65535;
 $177 = HEAP32[$1>>2]|0;
 $178 = ((($177)) + 22|0);
 HEAP16[$178>>1] = $176;
 STACKTOP = sp;return;
}
function _columnsInIndex($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, $m = 0, $or$cond = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $m = sp;
 $j = sp + 12|0;
 $x = sp + 8|0;
 HEAP32[$0>>2] = $pIdx;
 $1 = $m;
 $2 = $1;
 HEAP32[$2>>2] = 0;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = (($8) - 1)|0;
 HEAP32[$j>>2] = $9;
 while(1) {
  $10 = HEAP32[$j>>2]|0;
  $11 = ($10|0)>=(0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$j>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<1)|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17 << 16 >> 16;
  HEAP32[$x>>2] = $18;
  $19 = HEAP32[$x>>2]|0;
  $20 = ($19|0)>=(0);
  $21 = HEAP32[$x>>2]|0;
  $22 = ($21|0)<(63);
  $or$cond = $20 & $22;
  if ($or$cond) {
   $23 = HEAP32[$x>>2]|0;
   $24 = (_bitshift64Shl(1,0,($23|0))|0);
   $25 = tempRet0;
   $26 = $m;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = $28 | $24;
   $33 = $31 | $25;
   $34 = $m;
   $35 = $34;
   HEAP32[$35>>2] = $32;
   $36 = (($34) + 4)|0;
   $37 = $36;
   HEAP32[$37>>2] = $33;
  }
  $38 = HEAP32[$j>>2]|0;
  $39 = (($38) + -1)|0;
  HEAP32[$j>>2] = $39;
 }
 $40 = $m;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($40) + 4)|0;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 tempRet0 = ($45);
 STACKTOP = sp;return ($42|0);
}
function _sqlite3ExprCoveredByIndex($pExpr,$iCur,$pIdx) {
 $pExpr = $pExpr|0;
 $iCur = $iCur|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w = 0, $xcov = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $w = sp + 8|0;
 $xcov = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $pIdx;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($xcov)) + 4|0);
 HEAP32[$4>>2] = $3;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$xcov>>2] = $5;
 $6 = ((($w)) + 4|0);
 HEAP32[$6>>2] = 199;
 $7 = ((($w)) + 24|0);
 HEAP32[$7>>2] = $xcov;
 $8 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$8)|0);
 $9 = ((($w)) + 20|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 $12 = $11 ^ 1;
 $13 = $12&1;
 STACKTOP = sp;return ($13|0);
}
function _whereLoopAddBtreeIndex($pBuilder,$pSrc,$pProbe,$nInMul) {
 $pBuilder = $pBuilder|0;
 $pSrc = $pSrc|0;
 $pProbe = $pProbe|0;
 $nInMul = $nInMul|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0;
 var $eOp = 0, $i = 0, $iCol = 0, $nEq = 0, $nIn = 0, $nIter = 0, $nOutUnadjusted = 0, $opMask = 0, $pBtm = 0, $pExpr = 0, $pNew = 0, $pParse = 0, $pTerm = 0, $pTop = 0, $pWInfo = 0, $rCostIdx = 0, $rLogSize = 0, $rSize = 0, $rc = 0, $saved_nBtm = 0;
 var $saved_nEq = 0, $saved_nLTerm = 0, $saved_nOut = 0, $saved_nSkip = 0, $saved_nTop = 0, $saved_prereq = 0, $saved_wsFlags = 0, $scan = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 180|0;
 $1 = sp + 176|0;
 $2 = sp + 172|0;
 $3 = sp + 168|0;
 $4 = sp + 208|0;
 $pWInfo = sp + 164|0;
 $pParse = sp + 160|0;
 $db = sp + 156|0;
 $pNew = sp + 152|0;
 $pTerm = sp + 148|0;
 $opMask = sp + 144|0;
 $scan = sp + 48|0;
 $saved_prereq = sp;
 $saved_nLTerm = sp + 206|0;
 $saved_nEq = sp + 204|0;
 $saved_nBtm = sp + 202|0;
 $saved_nTop = sp + 200|0;
 $saved_nSkip = sp + 198|0;
 $saved_wsFlags = sp + 40|0;
 $saved_nOut = sp + 196|0;
 $rc = sp + 36|0;
 $rSize = sp + 194|0;
 $rLogSize = sp + 192|0;
 $pTop = sp + 32|0;
 $pBtm = sp + 28|0;
 $eOp = sp + 190|0;
 $rCostIdx = sp + 188|0;
 $nOutUnadjusted = sp + 186|0;
 $nIn = sp + 24|0;
 $pExpr = sp + 20|0;
 $i = sp + 16|0;
 $iCol = sp + 12|0;
 $nEq = sp + 8|0;
 $nIter = sp + 184|0;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $pProbe;
 HEAP16[$4>>1] = $nInMul;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pWInfo>>2] = $6;
 $7 = HEAP32[$pWInfo>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pParse>>2] = $8;
 $9 = HEAP32[$pParse>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pTop>>2] = 0;
 HEAP32[$pBtm>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pNew>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = ((($14)) + 69|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)!=(0);
 if ($17) {
  HEAP32[$0>>2] = 7;
  $758 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($758|0);
 }
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 36|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20 & 32;
 $22 = ($21|0)!=(0);
 if ($22) {
  HEAP32[$opMask>>2] = 24;
 } else {
  HEAP32[$opMask>>2] = 447;
 }
 $23 = HEAP32[$3>>2]|0;
 $24 = ((($23)) + 55|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = ($25&255) >>> 2;
 $27 = $26 & 1;
 $28 = $27&255;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$opMask>>2]|0;
  $31 = $30 & -61;
  HEAP32[$opMask>>2] = $31;
 }
 $32 = HEAP32[$pNew>>2]|0;
 $33 = ((($32)) + 24|0);
 $34 = HEAP16[$33>>1]|0;
 HEAP16[$saved_nEq>>1] = $34;
 $35 = HEAP32[$pNew>>2]|0;
 $36 = ((($35)) + 24|0);
 $37 = ((($36)) + 2|0);
 $38 = HEAP16[$37>>1]|0;
 HEAP16[$saved_nBtm>>1] = $38;
 $39 = HEAP32[$pNew>>2]|0;
 $40 = ((($39)) + 24|0);
 $41 = ((($40)) + 4|0);
 $42 = HEAP16[$41>>1]|0;
 HEAP16[$saved_nTop>>1] = $42;
 $43 = HEAP32[$pNew>>2]|0;
 $44 = ((($43)) + 42|0);
 $45 = HEAP16[$44>>1]|0;
 HEAP16[$saved_nSkip>>1] = $45;
 $46 = HEAP32[$pNew>>2]|0;
 $47 = ((($46)) + 40|0);
 $48 = HEAP16[$47>>1]|0;
 HEAP16[$saved_nLTerm>>1] = $48;
 $49 = HEAP32[$pNew>>2]|0;
 $50 = ((($49)) + 36|0);
 $51 = HEAP32[$50>>2]|0;
 HEAP32[$saved_wsFlags>>2] = $51;
 $52 = HEAP32[$pNew>>2]|0;
 $53 = $52;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($53) + 4)|0;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = $saved_prereq;
 $60 = $59;
 HEAP32[$60>>2] = $55;
 $61 = (($59) + 4)|0;
 $62 = $61;
 HEAP32[$62>>2] = $58;
 $63 = HEAP32[$pNew>>2]|0;
 $64 = ((($63)) + 22|0);
 $65 = HEAP16[$64>>1]|0;
 HEAP16[$saved_nOut>>1] = $65;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 4|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = HEAP32[$2>>2]|0;
 $70 = ((($69)) + 44|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP16[$saved_nEq>>1]|0;
 $73 = $72&65535;
 $74 = HEAP32[$opMask>>2]|0;
 $75 = HEAP32[$3>>2]|0;
 $76 = (_whereScanInit($scan,$68,$71,$73,$74,$75)|0);
 HEAP32[$pTerm>>2] = $76;
 $77 = HEAP32[$pNew>>2]|0;
 $78 = ((($77)) + 18|0);
 HEAP16[$78>>1] = 0;
 $79 = HEAP32[$3>>2]|0;
 $80 = ((($79)) + 8|0);
 $81 = HEAP32[$80>>2]|0;
 $82 = HEAP16[$81>>1]|0;
 HEAP16[$rSize>>1] = $82;
 $83 = HEAP16[$rSize>>1]|0;
 $84 = (_estLog($83)|0);
 HEAP16[$rLogSize>>1] = $84;
 L12: while(1) {
  $85 = HEAP32[$rc>>2]|0;
  $86 = ($85|0)==(0);
  $87 = HEAP32[$pTerm>>2]|0;
  $88 = ($87|0)!=(0|0);
  $89 = $86 ? $88 : 0;
  if (!($89)) {
   break;
  }
  $90 = HEAP32[$pTerm>>2]|0;
  $91 = ((($90)) + 12|0);
  $92 = HEAP16[$91>>1]|0;
  HEAP16[$eOp>>1] = $92;
  HEAP32[$nIn>>2] = 0;
  $93 = HEAP16[$eOp>>1]|0;
  $94 = $93&65535;
  $95 = ($94|0)==(256);
  if ($95) {
   label = 12;
  } else {
   $96 = HEAP32[$pTerm>>2]|0;
   $97 = ((($96)) + 10|0);
   $98 = HEAP16[$97>>1]|0;
   $99 = $98&65535;
   $100 = $99 & 0;
   $101 = ($100|0)!=(0);
   if ($101) {
    label = 12;
   } else {
    label = 13;
   }
  }
  if ((label|0) == 12) {
   label = 0;
   $102 = HEAP32[$3>>2]|0;
   $103 = HEAP16[$saved_nEq>>1]|0;
   $104 = $103&65535;
   $105 = (_indexColumnNotNull($102,$104)|0);
   $106 = ($105|0)!=(0);
   if (!($106)) {
    label = 13;
   }
  }
  do {
   if ((label|0) == 13) {
    label = 0;
    $107 = HEAP32[$pTerm>>2]|0;
    $108 = ((($107)) + 32|0);
    $109 = $108;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = (($109) + 4)|0;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = HEAP32[$pNew>>2]|0;
    $116 = ((($115)) + 8|0);
    $117 = $116;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = (($117) + 4)|0;
    $121 = $120;
    $122 = HEAP32[$121>>2]|0;
    $123 = $111 & $119;
    $124 = $114 & $122;
    $125 = ($123|0)!=(0);
    $126 = ($124|0)!=(0);
    $127 = $125 | $126;
    if (!($127)) {
     $128 = HEAP32[$pTerm>>2]|0;
     $129 = ((($128)) + 10|0);
     $130 = HEAP16[$129>>1]|0;
     $131 = $130&65535;
     $132 = $131 & 256;
     $133 = ($132|0)!=(0);
     if ($133) {
      $134 = HEAP32[$pTerm>>2]|0;
      $135 = ((($134)) + 12|0);
      $136 = HEAP16[$135>>1]|0;
      $137 = $136&65535;
      $138 = ($137|0)==(16);
      if ($138) {
       break;
      }
     }
     $139 = HEAP32[$2>>2]|0;
     $140 = ((($139)) + 36|0);
     $141 = HEAP8[$140>>0]|0;
     $142 = $141&255;
     $143 = $142 & 8;
     $144 = ($143|0)!=(0);
     if ($144) {
      $145 = HEAP32[$pTerm>>2]|0;
      $146 = HEAP32[$145>>2]|0;
      $147 = ((($146)) + 4|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = $148 & 1;
      $150 = ($149|0)!=(0);
      if (!($150)) {
       $151 = HEAP16[$eOp>>1]|0;
       $152 = $151&65535;
       $153 = $152 & 384;
       $154 = ($153|0)!=(0);
       if ($154) {
        break;
       }
      }
     }
     $155 = HEAP32[$saved_wsFlags>>2]|0;
     $156 = HEAP32[$pNew>>2]|0;
     $157 = ((($156)) + 36|0);
     HEAP32[$157>>2] = $155;
     $158 = HEAP16[$saved_nEq>>1]|0;
     $159 = HEAP32[$pNew>>2]|0;
     $160 = ((($159)) + 24|0);
     HEAP16[$160>>1] = $158;
     $161 = HEAP16[$saved_nBtm>>1]|0;
     $162 = HEAP32[$pNew>>2]|0;
     $163 = ((($162)) + 24|0);
     $164 = ((($163)) + 2|0);
     HEAP16[$164>>1] = $161;
     $165 = HEAP16[$saved_nTop>>1]|0;
     $166 = HEAP32[$pNew>>2]|0;
     $167 = ((($166)) + 24|0);
     $168 = ((($167)) + 4|0);
     HEAP16[$168>>1] = $165;
     $169 = HEAP16[$saved_nLTerm>>1]|0;
     $170 = HEAP32[$pNew>>2]|0;
     $171 = ((($170)) + 40|0);
     HEAP16[$171>>1] = $169;
     $172 = HEAP32[$db>>2]|0;
     $173 = HEAP32[$pNew>>2]|0;
     $174 = HEAP32[$pNew>>2]|0;
     $175 = ((($174)) + 40|0);
     $176 = HEAP16[$175>>1]|0;
     $177 = $176&65535;
     $178 = (($177) + 1)|0;
     $179 = (_whereLoopResize($172,$173,$178)|0);
     $180 = ($179|0)!=(0);
     if ($180) {
      break L12;
     }
     $181 = HEAP32[$pTerm>>2]|0;
     $182 = HEAP32[$pNew>>2]|0;
     $183 = ((($182)) + 40|0);
     $184 = HEAP16[$183>>1]|0;
     $185 = (($184) + 1)<<16>>16;
     HEAP16[$183>>1] = $185;
     $186 = $184&65535;
     $187 = HEAP32[$pNew>>2]|0;
     $188 = ((($187)) + 48|0);
     $189 = HEAP32[$188>>2]|0;
     $190 = (($189) + ($186<<2)|0);
     HEAP32[$190>>2] = $181;
     $191 = $saved_prereq;
     $192 = $191;
     $193 = HEAP32[$192>>2]|0;
     $194 = (($191) + 4)|0;
     $195 = $194;
     $196 = HEAP32[$195>>2]|0;
     $197 = HEAP32[$pTerm>>2]|0;
     $198 = ((($197)) + 32|0);
     $199 = $198;
     $200 = $199;
     $201 = HEAP32[$200>>2]|0;
     $202 = (($199) + 4)|0;
     $203 = $202;
     $204 = HEAP32[$203>>2]|0;
     $205 = $193 | $201;
     $206 = $196 | $204;
     $207 = HEAP32[$pNew>>2]|0;
     $208 = ((($207)) + 8|0);
     $209 = $208;
     $210 = $209;
     $211 = HEAP32[$210>>2]|0;
     $212 = (($209) + 4)|0;
     $213 = $212;
     $214 = HEAP32[$213>>2]|0;
     $215 = $211 ^ -1;
     $216 = $214 ^ -1;
     $217 = $205 & $215;
     $218 = $206 & $216;
     $219 = HEAP32[$pNew>>2]|0;
     $220 = $219;
     $221 = $220;
     HEAP32[$221>>2] = $217;
     $222 = (($220) + 4)|0;
     $223 = $222;
     HEAP32[$223>>2] = $218;
     $224 = HEAP16[$eOp>>1]|0;
     $225 = $224&65535;
     $226 = $225 & 1;
     $227 = ($226|0)!=(0);
     L30: do {
      if ($227) {
       $228 = HEAP32[$pTerm>>2]|0;
       $229 = HEAP32[$228>>2]|0;
       HEAP32[$pExpr>>2] = $229;
       $230 = HEAP32[$pNew>>2]|0;
       $231 = ((($230)) + 36|0);
       $232 = HEAP32[$231>>2]|0;
       $233 = $232 | 4;
       HEAP32[$231>>2] = $233;
       $234 = HEAP32[$pExpr>>2]|0;
       $235 = ((($234)) + 4|0);
       $236 = HEAP32[$235>>2]|0;
       $237 = $236 & 2048;
       $238 = ($237|0)!=(0);
       if (!($238)) {
        $264 = HEAP32[$pExpr>>2]|0;
        $265 = ((($264)) + 20|0);
        $266 = HEAP32[$265>>2]|0;
        $267 = ($266|0)!=(0|0);
        if (!($267)) {
         break;
        }
        $268 = HEAP32[$pExpr>>2]|0;
        $269 = ((($268)) + 20|0);
        $270 = HEAP32[$269>>2]|0;
        $271 = HEAP32[$270>>2]|0;
        $272 = ($271|0)!=(0);
        if (!($272)) {
         break;
        }
        $273 = HEAP32[$pExpr>>2]|0;
        $274 = ((($273)) + 20|0);
        $275 = HEAP32[$274>>2]|0;
        $276 = HEAP32[$275>>2]|0;
        $277 = ($276|0)<(0);
        $278 = $277 << 31 >> 31;
        $279 = (_sqlite3LogEst($276,$278)|0);
        $280 = $279 << 16 >> 16;
        HEAP32[$nIn>>2] = $280;
        break;
       }
       HEAP32[$nIn>>2] = 46;
       HEAP32[$i>>2] = 0;
       while(1) {
        $239 = HEAP32[$i>>2]|0;
        $240 = HEAP32[$pNew>>2]|0;
        $241 = ((($240)) + 40|0);
        $242 = HEAP16[$241>>1]|0;
        $243 = $242&65535;
        $244 = (($243) - 1)|0;
        $245 = ($239|0)<($244|0);
        if (!($245)) {
         break L30;
        }
        $246 = HEAP32[$i>>2]|0;
        $247 = HEAP32[$pNew>>2]|0;
        $248 = ((($247)) + 48|0);
        $249 = HEAP32[$248>>2]|0;
        $250 = (($249) + ($246<<2)|0);
        $251 = HEAP32[$250>>2]|0;
        $252 = ($251|0)!=(0|0);
        if ($252) {
         $253 = HEAP32[$i>>2]|0;
         $254 = HEAP32[$pNew>>2]|0;
         $255 = ((($254)) + 48|0);
         $256 = HEAP32[$255>>2]|0;
         $257 = (($256) + ($253<<2)|0);
         $258 = HEAP32[$257>>2]|0;
         $259 = HEAP32[$258>>2]|0;
         $260 = HEAP32[$pExpr>>2]|0;
         $261 = ($259|0)==($260|0);
         if ($261) {
          HEAP32[$nIn>>2] = 0;
         }
        }
        $262 = HEAP32[$i>>2]|0;
        $263 = (($262) + 1)|0;
        HEAP32[$i>>2] = $263;
       }
      } else {
       $281 = HEAP16[$eOp>>1]|0;
       $282 = $281&65535;
       $283 = $282 & 130;
       $284 = ($283|0)!=(0);
       if ($284) {
        $285 = HEAP16[$saved_nEq>>1]|0;
        $286 = $285&65535;
        $287 = HEAP32[$3>>2]|0;
        $288 = ((($287)) + 4|0);
        $289 = HEAP32[$288>>2]|0;
        $290 = (($289) + ($286<<1)|0);
        $291 = HEAP16[$290>>1]|0;
        $292 = $291 << 16 >> 16;
        HEAP32[$iCol>>2] = $292;
        $293 = HEAP32[$pNew>>2]|0;
        $294 = ((($293)) + 36|0);
        $295 = HEAP32[$294>>2]|0;
        $296 = $295 | 1;
        HEAP32[$294>>2] = $296;
        $297 = HEAP32[$iCol>>2]|0;
        $298 = ($297|0)==(-1);
        if (!($298)) {
         $299 = HEAP32[$iCol>>2]|0;
         $300 = ($299|0)>(0);
         if (!($300)) {
          break;
         }
         $301 = HEAP16[$4>>1]|0;
         $302 = $301 << 16 >> 16;
         $303 = ($302|0)==(0);
         if (!($303)) {
          break;
         }
         $304 = HEAP16[$saved_nEq>>1]|0;
         $305 = $304&65535;
         $306 = HEAP32[$3>>2]|0;
         $307 = ((($306)) + 50|0);
         $308 = HEAP16[$307>>1]|0;
         $309 = $308&65535;
         $310 = (($309) - 1)|0;
         $311 = ($305|0)==($310|0);
         if (!($311)) {
          break;
         }
        }
        $312 = HEAP32[$iCol>>2]|0;
        $313 = ($312|0)>=(0);
        if ($313) {
         $314 = HEAP32[$3>>2]|0;
         $315 = ((($314)) + 55|0);
         $316 = HEAP8[$315>>0]|0;
         $317 = ($316&255) >>> 3;
         $318 = $317 & 1;
         $319 = $318&255;
         $320 = ($319|0)==(0);
         if ($320) {
          $321 = HEAP32[$pNew>>2]|0;
          $322 = ((($321)) + 36|0);
          $323 = HEAP32[$322>>2]|0;
          $324 = $323 | 65536;
          HEAP32[$322>>2] = $324;
          break;
         }
        }
        $325 = HEAP32[$pNew>>2]|0;
        $326 = ((($325)) + 36|0);
        $327 = HEAP32[$326>>2]|0;
        $328 = $327 | 4096;
        HEAP32[$326>>2] = $328;
        break;
       }
       $329 = HEAP16[$eOp>>1]|0;
       $330 = $329&65535;
       $331 = $330 & 256;
       $332 = ($331|0)!=(0);
       if ($332) {
        $333 = HEAP32[$pNew>>2]|0;
        $334 = ((($333)) + 36|0);
        $335 = HEAP32[$334>>2]|0;
        $336 = $335 | 8;
        HEAP32[$334>>2] = $336;
        break;
       }
       $337 = HEAP16[$eOp>>1]|0;
       $338 = $337&65535;
       $339 = $338 & 36;
       $340 = ($339|0)!=(0);
       $341 = HEAP32[$pNew>>2]|0;
       $342 = ((($341)) + 36|0);
       $343 = HEAP32[$342>>2]|0;
       if ($340) {
        $344 = $343 | 34;
        HEAP32[$342>>2] = $344;
        $345 = HEAP32[$pParse>>2]|0;
        $346 = HEAP32[$2>>2]|0;
        $347 = ((($346)) + 44|0);
        $348 = HEAP32[$347>>2]|0;
        $349 = HEAP32[$3>>2]|0;
        $350 = HEAP16[$saved_nEq>>1]|0;
        $351 = $350&65535;
        $352 = HEAP32[$pTerm>>2]|0;
        $353 = (_whereRangeVectorLen($345,$348,$349,$351,$352)|0);
        $354 = $353&65535;
        $355 = HEAP32[$pNew>>2]|0;
        $356 = ((($355)) + 24|0);
        $357 = ((($356)) + 2|0);
        HEAP16[$357>>1] = $354;
        $358 = HEAP32[$pTerm>>2]|0;
        HEAP32[$pBtm>>2] = $358;
        HEAP32[$pTop>>2] = 0;
        $359 = HEAP32[$pTerm>>2]|0;
        $360 = ((($359)) + 10|0);
        $361 = HEAP16[$360>>1]|0;
        $362 = $361&65535;
        $363 = $362 & 256;
        $364 = ($363|0)!=(0);
        if (!($364)) {
         break;
        }
        $365 = HEAP32[$pTerm>>2]|0;
        $366 = ((($365)) + 48|0);
        HEAP32[$pTop>>2] = $366;
        $367 = HEAP32[$db>>2]|0;
        $368 = HEAP32[$pNew>>2]|0;
        $369 = HEAP32[$pNew>>2]|0;
        $370 = ((($369)) + 40|0);
        $371 = HEAP16[$370>>1]|0;
        $372 = $371&65535;
        $373 = (($372) + 1)|0;
        $374 = (_whereLoopResize($367,$368,$373)|0);
        $375 = ($374|0)!=(0);
        if ($375) {
         break L12;
        }
        $376 = HEAP32[$pTop>>2]|0;
        $377 = HEAP32[$pNew>>2]|0;
        $378 = ((($377)) + 40|0);
        $379 = HEAP16[$378>>1]|0;
        $380 = (($379) + 1)<<16>>16;
        HEAP16[$378>>1] = $380;
        $381 = $379&65535;
        $382 = HEAP32[$pNew>>2]|0;
        $383 = ((($382)) + 48|0);
        $384 = HEAP32[$383>>2]|0;
        $385 = (($384) + ($381<<2)|0);
        HEAP32[$385>>2] = $376;
        $386 = HEAP32[$pNew>>2]|0;
        $387 = ((($386)) + 36|0);
        $388 = HEAP32[$387>>2]|0;
        $389 = $388 | 16;
        HEAP32[$387>>2] = $389;
        $390 = HEAP32[$pNew>>2]|0;
        $391 = ((($390)) + 24|0);
        $392 = ((($391)) + 4|0);
        HEAP16[$392>>1] = 1;
        break;
       } else {
        $393 = $343 | 18;
        HEAP32[$342>>2] = $393;
        $394 = HEAP32[$pParse>>2]|0;
        $395 = HEAP32[$2>>2]|0;
        $396 = ((($395)) + 44|0);
        $397 = HEAP32[$396>>2]|0;
        $398 = HEAP32[$3>>2]|0;
        $399 = HEAP16[$saved_nEq>>1]|0;
        $400 = $399&65535;
        $401 = HEAP32[$pTerm>>2]|0;
        $402 = (_whereRangeVectorLen($394,$397,$398,$400,$401)|0);
        $403 = $402&65535;
        $404 = HEAP32[$pNew>>2]|0;
        $405 = ((($404)) + 24|0);
        $406 = ((($405)) + 4|0);
        HEAP16[$406>>1] = $403;
        $407 = HEAP32[$pTerm>>2]|0;
        HEAP32[$pTop>>2] = $407;
        $408 = HEAP32[$pNew>>2]|0;
        $409 = ((($408)) + 36|0);
        $410 = HEAP32[$409>>2]|0;
        $411 = $410 & 32;
        $412 = ($411|0)!=(0);
        if ($412) {
         $413 = HEAP32[$pNew>>2]|0;
         $414 = ((($413)) + 40|0);
         $415 = HEAP16[$414>>1]|0;
         $416 = $415&65535;
         $417 = (($416) - 2)|0;
         $418 = HEAP32[$pNew>>2]|0;
         $419 = ((($418)) + 48|0);
         $420 = HEAP32[$419>>2]|0;
         $421 = (($420) + ($417<<2)|0);
         $422 = HEAP32[$421>>2]|0;
         $423 = $422;
        } else {
         $423 = 0;
        }
        HEAP32[$pBtm>>2] = $423;
        break;
       }
      }
     } while(0);
     $424 = HEAP32[$pNew>>2]|0;
     $425 = ((($424)) + 36|0);
     $426 = HEAP32[$425>>2]|0;
     $427 = $426 & 2;
     $428 = ($427|0)!=(0);
     do {
      if ($428) {
       $429 = HEAP32[$pParse>>2]|0;
       $430 = HEAP32[$1>>2]|0;
       $431 = HEAP32[$pBtm>>2]|0;
       $432 = HEAP32[$pTop>>2]|0;
       $433 = HEAP32[$pNew>>2]|0;
       (_whereRangeScanEst($429,$430,$431,$432,$433)|0);
      } else {
       $434 = HEAP32[$pNew>>2]|0;
       $435 = ((($434)) + 24|0);
       $436 = HEAP16[$435>>1]|0;
       $437 = (($436) + 1)<<16>>16;
       HEAP16[$435>>1] = $437;
       $438 = $437&65535;
       HEAP32[$nEq>>2] = $438;
       $439 = HEAP32[$pTerm>>2]|0;
       $440 = ((($439)) + 8|0);
       $441 = HEAP16[$440>>1]|0;
       $442 = $441 << 16 >> 16;
       $443 = ($442|0)<=(0);
       if ($443) {
        $444 = HEAP16[$saved_nEq>>1]|0;
        $445 = $444&65535;
        $446 = HEAP32[$3>>2]|0;
        $447 = ((($446)) + 4|0);
        $448 = HEAP32[$447>>2]|0;
        $449 = (($448) + ($445<<1)|0);
        $450 = HEAP16[$449>>1]|0;
        $451 = $450 << 16 >> 16;
        $452 = ($451|0)>=(0);
        if ($452) {
         $453 = HEAP32[$pTerm>>2]|0;
         $454 = ((($453)) + 8|0);
         $455 = HEAP16[$454>>1]|0;
         $456 = $455 << 16 >> 16;
         $457 = HEAP32[$pNew>>2]|0;
         $458 = ((($457)) + 22|0);
         $459 = HEAP16[$458>>1]|0;
         $460 = $459 << 16 >> 16;
         $461 = (($460) + ($456))|0;
         $462 = $461&65535;
         HEAP16[$458>>1] = $462;
         $463 = HEAP32[$nIn>>2]|0;
         $464 = HEAP32[$pNew>>2]|0;
         $465 = ((($464)) + 22|0);
         $466 = HEAP16[$465>>1]|0;
         $467 = $466 << 16 >> 16;
         $468 = (($467) - ($463))|0;
         $469 = $468&65535;
         HEAP16[$465>>1] = $469;
         break;
        }
       }
       $470 = HEAP32[$nEq>>2]|0;
       $471 = HEAP32[$3>>2]|0;
       $472 = ((($471)) + 8|0);
       $473 = HEAP32[$472>>2]|0;
       $474 = (($473) + ($470<<1)|0);
       $475 = HEAP16[$474>>1]|0;
       $476 = $475 << 16 >> 16;
       $477 = HEAP32[$nEq>>2]|0;
       $478 = (($477) - 1)|0;
       $479 = HEAP32[$3>>2]|0;
       $480 = ((($479)) + 8|0);
       $481 = HEAP32[$480>>2]|0;
       $482 = (($481) + ($478<<1)|0);
       $483 = HEAP16[$482>>1]|0;
       $484 = $483 << 16 >> 16;
       $485 = (($476) - ($484))|0;
       $486 = HEAP32[$pNew>>2]|0;
       $487 = ((($486)) + 22|0);
       $488 = HEAP16[$487>>1]|0;
       $489 = $488 << 16 >> 16;
       $490 = (($489) + ($485))|0;
       $491 = $490&65535;
       HEAP16[$487>>1] = $491;
       $492 = HEAP16[$eOp>>1]|0;
       $493 = $492&65535;
       $494 = $493 & 256;
       $495 = ($494|0)!=(0);
       if ($495) {
        $496 = HEAP32[$pNew>>2]|0;
        $497 = ((($496)) + 22|0);
        $498 = HEAP16[$497>>1]|0;
        $499 = $498 << 16 >> 16;
        $500 = (($499) + 10)|0;
        $501 = $500&65535;
        HEAP16[$497>>1] = $501;
       }
      }
     } while(0);
     $502 = HEAP32[$pNew>>2]|0;
     $503 = ((($502)) + 22|0);
     $504 = HEAP16[$503>>1]|0;
     $505 = $504 << 16 >> 16;
     $506 = (($505) + 1)|0;
     $507 = HEAP32[$3>>2]|0;
     $508 = ((($507)) + 48|0);
     $509 = HEAP16[$508>>1]|0;
     $510 = $509 << 16 >> 16;
     $511 = ($510*15)|0;
     $512 = HEAP32[$2>>2]|0;
     $513 = ((($512)) + 16|0);
     $514 = HEAP32[$513>>2]|0;
     $515 = ((($514)) + 40|0);
     $516 = HEAP16[$515>>1]|0;
     $517 = $516 << 16 >> 16;
     $518 = (($511|0) / ($517|0))&-1;
     $519 = (($506) + ($518))|0;
     $520 = $519&65535;
     HEAP16[$rCostIdx>>1] = $520;
     $521 = HEAP16[$rLogSize>>1]|0;
     $522 = HEAP16[$rCostIdx>>1]|0;
     $523 = (_sqlite3LogEstAdd($521,$522)|0);
     $524 = HEAP32[$pNew>>2]|0;
     $525 = ((($524)) + 20|0);
     HEAP16[$525>>1] = $523;
     $526 = HEAP32[$pNew>>2]|0;
     $527 = ((($526)) + 36|0);
     $528 = HEAP32[$527>>2]|0;
     $529 = $528 & 320;
     $530 = ($529|0)==(0);
     if ($530) {
      $531 = HEAP32[$pNew>>2]|0;
      $532 = ((($531)) + 20|0);
      $533 = HEAP16[$532>>1]|0;
      $534 = HEAP32[$pNew>>2]|0;
      $535 = ((($534)) + 22|0);
      $536 = HEAP16[$535>>1]|0;
      $537 = $536 << 16 >> 16;
      $538 = (($537) + 16)|0;
      $539 = $538&65535;
      $540 = (_sqlite3LogEstAdd($533,$539)|0);
      $541 = HEAP32[$pNew>>2]|0;
      $542 = ((($541)) + 20|0);
      HEAP16[$542>>1] = $540;
     }
     $543 = HEAP32[$pNew>>2]|0;
     $544 = ((($543)) + 22|0);
     $545 = HEAP16[$544>>1]|0;
     HEAP16[$nOutUnadjusted>>1] = $545;
     $546 = HEAP16[$4>>1]|0;
     $547 = $546 << 16 >> 16;
     $548 = HEAP32[$nIn>>2]|0;
     $549 = (($547) + ($548))|0;
     $550 = HEAP32[$pNew>>2]|0;
     $551 = ((($550)) + 20|0);
     $552 = HEAP16[$551>>1]|0;
     $553 = $552 << 16 >> 16;
     $554 = (($553) + ($549))|0;
     $555 = $554&65535;
     HEAP16[$551>>1] = $555;
     $556 = HEAP16[$4>>1]|0;
     $557 = $556 << 16 >> 16;
     $558 = HEAP32[$nIn>>2]|0;
     $559 = (($557) + ($558))|0;
     $560 = HEAP32[$pNew>>2]|0;
     $561 = ((($560)) + 22|0);
     $562 = HEAP16[$561>>1]|0;
     $563 = $562 << 16 >> 16;
     $564 = (($563) + ($559))|0;
     $565 = $564&65535;
     HEAP16[$561>>1] = $565;
     $566 = HEAP32[$1>>2]|0;
     $567 = ((($566)) + 4|0);
     $568 = HEAP32[$567>>2]|0;
     $569 = HEAP32[$pNew>>2]|0;
     $570 = HEAP16[$rSize>>1]|0;
     _whereLoopOutputAdjust($568,$569,$570);
     $571 = HEAP32[$1>>2]|0;
     $572 = HEAP32[$pNew>>2]|0;
     $573 = (_whereLoopInsert($571,$572)|0);
     HEAP32[$rc>>2] = $573;
     $574 = HEAP32[$pNew>>2]|0;
     $575 = ((($574)) + 36|0);
     $576 = HEAP32[$575>>2]|0;
     $577 = $576 & 2;
     $578 = ($577|0)!=(0);
     if ($578) {
      $579 = HEAP16[$saved_nOut>>1]|0;
      $580 = HEAP32[$pNew>>2]|0;
      $581 = ((($580)) + 22|0);
      HEAP16[$581>>1] = $579;
     } else {
      $582 = HEAP16[$nOutUnadjusted>>1]|0;
      $583 = HEAP32[$pNew>>2]|0;
      $584 = ((($583)) + 22|0);
      HEAP16[$584>>1] = $582;
     }
     $585 = HEAP32[$pNew>>2]|0;
     $586 = ((($585)) + 36|0);
     $587 = HEAP32[$586>>2]|0;
     $588 = $587 & 16;
     $589 = ($588|0)==(0);
     if ($589) {
      $590 = HEAP32[$pNew>>2]|0;
      $591 = ((($590)) + 24|0);
      $592 = HEAP16[$591>>1]|0;
      $593 = $592&65535;
      $594 = HEAP32[$3>>2]|0;
      $595 = ((($594)) + 52|0);
      $596 = HEAP16[$595>>1]|0;
      $597 = $596&65535;
      $598 = ($593|0)<($597|0);
      if ($598) {
       $599 = HEAP32[$1>>2]|0;
       $600 = HEAP32[$2>>2]|0;
       $601 = HEAP32[$3>>2]|0;
       $602 = HEAP16[$4>>1]|0;
       $603 = $602 << 16 >> 16;
       $604 = HEAP32[$nIn>>2]|0;
       $605 = (($603) + ($604))|0;
       $606 = $605&65535;
       (_whereLoopAddBtreeIndex($599,$600,$601,$606)|0);
      }
     }
     $607 = HEAP16[$saved_nOut>>1]|0;
     $608 = HEAP32[$pNew>>2]|0;
     $609 = ((($608)) + 22|0);
     HEAP16[$609>>1] = $607;
    }
   }
  } while(0);
  $610 = (_whereScanNext($scan)|0);
  HEAP32[$pTerm>>2] = $610;
 }
 $611 = $saved_prereq;
 $612 = $611;
 $613 = HEAP32[$612>>2]|0;
 $614 = (($611) + 4)|0;
 $615 = $614;
 $616 = HEAP32[$615>>2]|0;
 $617 = HEAP32[$pNew>>2]|0;
 $618 = $617;
 $619 = $618;
 HEAP32[$619>>2] = $613;
 $620 = (($618) + 4)|0;
 $621 = $620;
 HEAP32[$621>>2] = $616;
 $622 = HEAP16[$saved_nEq>>1]|0;
 $623 = HEAP32[$pNew>>2]|0;
 $624 = ((($623)) + 24|0);
 HEAP16[$624>>1] = $622;
 $625 = HEAP16[$saved_nBtm>>1]|0;
 $626 = HEAP32[$pNew>>2]|0;
 $627 = ((($626)) + 24|0);
 $628 = ((($627)) + 2|0);
 HEAP16[$628>>1] = $625;
 $629 = HEAP16[$saved_nTop>>1]|0;
 $630 = HEAP32[$pNew>>2]|0;
 $631 = ((($630)) + 24|0);
 $632 = ((($631)) + 4|0);
 HEAP16[$632>>1] = $629;
 $633 = HEAP16[$saved_nSkip>>1]|0;
 $634 = HEAP32[$pNew>>2]|0;
 $635 = ((($634)) + 42|0);
 HEAP16[$635>>1] = $633;
 $636 = HEAP32[$saved_wsFlags>>2]|0;
 $637 = HEAP32[$pNew>>2]|0;
 $638 = ((($637)) + 36|0);
 HEAP32[$638>>2] = $636;
 $639 = HEAP16[$saved_nOut>>1]|0;
 $640 = HEAP32[$pNew>>2]|0;
 $641 = ((($640)) + 22|0);
 HEAP16[$641>>1] = $639;
 $642 = HEAP16[$saved_nLTerm>>1]|0;
 $643 = HEAP32[$pNew>>2]|0;
 $644 = ((($643)) + 40|0);
 HEAP16[$644>>1] = $642;
 $645 = HEAP16[$saved_nEq>>1]|0;
 $646 = $645&65535;
 $647 = HEAP16[$saved_nSkip>>1]|0;
 $648 = $647&65535;
 $649 = ($646|0)==($648|0);
 if ($649) {
  $650 = HEAP16[$saved_nEq>>1]|0;
  $651 = $650&65535;
  $652 = (($651) + 1)|0;
  $653 = HEAP32[$3>>2]|0;
  $654 = ((($653)) + 50|0);
  $655 = HEAP16[$654>>1]|0;
  $656 = $655&65535;
  $657 = ($652|0)<($656|0);
  if ($657) {
   $658 = HEAP32[$3>>2]|0;
   $659 = ((($658)) + 55|0);
   $660 = HEAP8[$659>>0]|0;
   $661 = ($660&255) >>> 6;
   $662 = $661 & 1;
   $663 = $662&255;
   $664 = ($663|0)==(0);
   if ($664) {
    $665 = HEAP16[$saved_nEq>>1]|0;
    $666 = $665&65535;
    $667 = (($666) + 1)|0;
    $668 = HEAP32[$3>>2]|0;
    $669 = ((($668)) + 8|0);
    $670 = HEAP32[$669>>2]|0;
    $671 = (($670) + ($667<<1)|0);
    $672 = HEAP16[$671>>1]|0;
    $673 = $672 << 16 >> 16;
    $674 = ($673|0)>=(42);
    if ($674) {
     $675 = HEAP32[$db>>2]|0;
     $676 = HEAP32[$pNew>>2]|0;
     $677 = HEAP32[$pNew>>2]|0;
     $678 = ((($677)) + 40|0);
     $679 = HEAP16[$678>>1]|0;
     $680 = $679&65535;
     $681 = (($680) + 1)|0;
     $682 = (_whereLoopResize($675,$676,$681)|0);
     HEAP32[$rc>>2] = $682;
     $683 = ($682|0)==(0);
     if ($683) {
      $684 = HEAP32[$pNew>>2]|0;
      $685 = ((($684)) + 24|0);
      $686 = HEAP16[$685>>1]|0;
      $687 = (($686) + 1)<<16>>16;
      HEAP16[$685>>1] = $687;
      $688 = HEAP32[$pNew>>2]|0;
      $689 = ((($688)) + 42|0);
      $690 = HEAP16[$689>>1]|0;
      $691 = (($690) + 1)<<16>>16;
      HEAP16[$689>>1] = $691;
      $692 = HEAP32[$pNew>>2]|0;
      $693 = ((($692)) + 40|0);
      $694 = HEAP16[$693>>1]|0;
      $695 = (($694) + 1)<<16>>16;
      HEAP16[$693>>1] = $695;
      $696 = $694&65535;
      $697 = HEAP32[$pNew>>2]|0;
      $698 = ((($697)) + 48|0);
      $699 = HEAP32[$698>>2]|0;
      $700 = (($699) + ($696<<2)|0);
      HEAP32[$700>>2] = 0;
      $701 = HEAP32[$pNew>>2]|0;
      $702 = ((($701)) + 36|0);
      $703 = HEAP32[$702>>2]|0;
      $704 = $703 | 32768;
      HEAP32[$702>>2] = $704;
      $705 = HEAP16[$saved_nEq>>1]|0;
      $706 = $705&65535;
      $707 = HEAP32[$3>>2]|0;
      $708 = ((($707)) + 8|0);
      $709 = HEAP32[$708>>2]|0;
      $710 = (($709) + ($706<<1)|0);
      $711 = HEAP16[$710>>1]|0;
      $712 = $711 << 16 >> 16;
      $713 = HEAP16[$saved_nEq>>1]|0;
      $714 = $713&65535;
      $715 = (($714) + 1)|0;
      $716 = HEAP32[$3>>2]|0;
      $717 = ((($716)) + 8|0);
      $718 = HEAP32[$717>>2]|0;
      $719 = (($718) + ($715<<1)|0);
      $720 = HEAP16[$719>>1]|0;
      $721 = $720 << 16 >> 16;
      $722 = (($712) - ($721))|0;
      $723 = $722&65535;
      HEAP16[$nIter>>1] = $723;
      $724 = HEAP16[$nIter>>1]|0;
      $725 = $724 << 16 >> 16;
      $726 = HEAP32[$pNew>>2]|0;
      $727 = ((($726)) + 22|0);
      $728 = HEAP16[$727>>1]|0;
      $729 = $728 << 16 >> 16;
      $730 = (($729) - ($725))|0;
      $731 = $730&65535;
      HEAP16[$727>>1] = $731;
      $732 = HEAP16[$nIter>>1]|0;
      $733 = $732 << 16 >> 16;
      $734 = (($733) + 5)|0;
      $735 = $734&65535;
      HEAP16[$nIter>>1] = $735;
      $736 = HEAP32[$1>>2]|0;
      $737 = HEAP32[$2>>2]|0;
      $738 = HEAP32[$3>>2]|0;
      $739 = HEAP16[$nIter>>1]|0;
      $740 = $739 << 16 >> 16;
      $741 = HEAP16[$4>>1]|0;
      $742 = $741 << 16 >> 16;
      $743 = (($740) + ($742))|0;
      $744 = $743&65535;
      (_whereLoopAddBtreeIndex($736,$737,$738,$744)|0);
      $745 = HEAP16[$saved_nOut>>1]|0;
      $746 = HEAP32[$pNew>>2]|0;
      $747 = ((($746)) + 22|0);
      HEAP16[$747>>1] = $745;
      $748 = HEAP16[$saved_nEq>>1]|0;
      $749 = HEAP32[$pNew>>2]|0;
      $750 = ((($749)) + 24|0);
      HEAP16[$750>>1] = $748;
      $751 = HEAP16[$saved_nSkip>>1]|0;
      $752 = HEAP32[$pNew>>2]|0;
      $753 = ((($752)) + 42|0);
      HEAP16[$753>>1] = $751;
      $754 = HEAP32[$saved_wsFlags>>2]|0;
      $755 = HEAP32[$pNew>>2]|0;
      $756 = ((($755)) + 36|0);
      HEAP32[$756>>2] = $754;
     }
    }
   }
  }
 }
 $757 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $757;
 $758 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($758|0);
}
function _indexColumnNotNull($pIdx,$iCol) {
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $j = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $j = sp;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<1)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 HEAP32[$j>>2] = $9;
 $10 = HEAP32[$j>>2]|0;
 $11 = ($10|0)>=(0);
 $12 = HEAP32[$j>>2]|0;
 if ($11) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($12<<4)|0);
  $19 = ((($18)) + 12|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20&255;
  HEAP32[$0>>2] = $21;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $22 = ($12|0)==(-1);
 if ($22) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 } else {
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _whereRangeVectorLen($pParse,$iCur,$pIdx,$nEq,$pTerm) {
 $pParse = $pParse|0;
 $iCur = $iCur|0;
 $pIdx = $pIdx|0;
 $nEq = $nEq|0;
 $pTerm = $pTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $aff = 0, $i = 0, $idxaff = 0, $nCmp = 0, $pColl = 0, $pLhs = 0, $pRhs = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $nCmp = sp + 16|0;
 $i = sp + 12|0;
 $aff = sp + 41|0;
 $idxaff = sp + 40|0;
 $pColl = sp + 8|0;
 $pLhs = sp + 4|0;
 $pRhs = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $pIdx;
 HEAP32[$3>>2] = $nEq;
 HEAP32[$4>>2] = $pTerm;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3ExprVectorSize($8)|0);
 HEAP32[$nCmp>>2] = $9;
 $10 = HEAP32[$nCmp>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 52|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = HEAP32[$3>>2]|0;
 $16 = (($14) - ($15))|0;
 $17 = ($10|0)<($16|0);
 if ($17) {
  $18 = HEAP32[$nCmp>>2]|0;
  $25 = $18;
 } else {
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 52|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = $21&65535;
  $23 = HEAP32[$3>>2]|0;
  $24 = (($22) - ($23))|0;
  $25 = $24;
 }
 HEAP32[$nCmp>>2] = $25;
 HEAP32[$i>>2] = 1;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$nCmp>>2]|0;
  $28 = ($26|0)<($27|0);
  if (!($28)) {
   label = 17;
   break;
  }
  HEAP8[$idxaff>>0] = 0;
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$4>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ((($33)) + 20|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + (($29*20)|0)|0);
  $39 = HEAP32[$38>>2]|0;
  HEAP32[$pLhs>>2] = $39;
  $40 = HEAP32[$4>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  HEAP32[$pRhs>>2] = $43;
  $44 = HEAP32[$pRhs>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = $46 & 2048;
  $48 = ($47|0)!=(0);
  $49 = HEAP32[$i>>2]|0;
  $50 = HEAP32[$pRhs>>2]|0;
  $51 = ((($50)) + 20|0);
  if ($48) {
   $52 = HEAP32[$51>>2]|0;
   $53 = HEAP32[$52>>2]|0;
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + (($49*20)|0)|0);
   $57 = HEAP32[$56>>2]|0;
   HEAP32[$pRhs>>2] = $57;
  } else {
   $58 = HEAP32[$51>>2]|0;
   $59 = ((($58)) + 4|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = (($60) + (($49*20)|0)|0);
   $62 = HEAP32[$61>>2]|0;
   HEAP32[$pRhs>>2] = $62;
  }
  $63 = HEAP32[$pLhs>>2]|0;
  $64 = HEAP8[$63>>0]|0;
  $65 = $64&255;
  $66 = ($65|0)!=(152);
  if ($66) {
   label = 17;
   break;
  }
  $67 = HEAP32[$pLhs>>2]|0;
  $68 = ((($67)) + 28|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$1>>2]|0;
  $71 = ($69|0)!=($70|0);
  if ($71) {
   label = 17;
   break;
  }
  $72 = HEAP32[$pLhs>>2]|0;
  $73 = ((($72)) + 32|0);
  $74 = HEAP16[$73>>1]|0;
  $75 = $74 << 16 >> 16;
  $76 = HEAP32[$i>>2]|0;
  $77 = HEAP32[$3>>2]|0;
  $78 = (($76) + ($77))|0;
  $79 = HEAP32[$2>>2]|0;
  $80 = ((($79)) + 4|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = (($81) + ($78<<1)|0);
  $83 = HEAP16[$82>>1]|0;
  $84 = $83 << 16 >> 16;
  $85 = ($75|0)!=($84|0);
  if ($85) {
   label = 17;
   break;
  }
  $86 = HEAP32[$i>>2]|0;
  $87 = HEAP32[$3>>2]|0;
  $88 = (($86) + ($87))|0;
  $89 = HEAP32[$2>>2]|0;
  $90 = ((($89)) + 28|0);
  $91 = HEAP32[$90>>2]|0;
  $92 = (($91) + ($88)|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $93&255;
  $95 = HEAP32[$3>>2]|0;
  $96 = HEAP32[$2>>2]|0;
  $97 = ((($96)) + 28|0);
  $98 = HEAP32[$97>>2]|0;
  $99 = (($98) + ($95)|0);
  $100 = HEAP8[$99>>0]|0;
  $101 = $100&255;
  $102 = ($94|0)!=($101|0);
  if ($102) {
   label = 17;
   break;
  }
  $103 = HEAP32[$pRhs>>2]|0;
  $104 = HEAP32[$pLhs>>2]|0;
  $105 = (_sqlite3ExprAffinity($104)|0);
  $106 = (_sqlite3CompareAffinity($103,$105)|0);
  HEAP8[$aff>>0] = $106;
  $107 = HEAP32[$2>>2]|0;
  $108 = ((($107)) + 12|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = HEAP32[$pLhs>>2]|0;
  $111 = ((($110)) + 32|0);
  $112 = HEAP16[$111>>1]|0;
  $113 = $112 << 16 >> 16;
  $114 = (_sqlite3TableColumnAffinity($109,$113)|0);
  HEAP8[$idxaff>>0] = $114;
  $115 = HEAP8[$aff>>0]|0;
  $116 = $115 << 24 >> 24;
  $117 = HEAP8[$idxaff>>0]|0;
  $118 = $117 << 24 >> 24;
  $119 = ($116|0)!=($118|0);
  if ($119) {
   label = 17;
   break;
  }
  $120 = HEAP32[$0>>2]|0;
  $121 = HEAP32[$pLhs>>2]|0;
  $122 = HEAP32[$pRhs>>2]|0;
  $123 = (_sqlite3BinaryCompareCollSeq($120,$121,$122)|0);
  HEAP32[$pColl>>2] = $123;
  $124 = HEAP32[$pColl>>2]|0;
  $125 = ($124|0)==(0|0);
  if ($125) {
   label = 17;
   break;
  }
  $126 = HEAP32[$pColl>>2]|0;
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$i>>2]|0;
  $129 = HEAP32[$3>>2]|0;
  $130 = (($128) + ($129))|0;
  $131 = HEAP32[$2>>2]|0;
  $132 = ((($131)) + 32|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = (($133) + ($130<<2)|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (_sqlite3StrICmp($127,$135)|0);
  $137 = ($136|0)!=(0);
  if ($137) {
   label = 17;
   break;
  }
  $138 = HEAP32[$i>>2]|0;
  $139 = (($138) + 1)|0;
  HEAP32[$i>>2] = $139;
 }
 if ((label|0) == 17) {
  $140 = HEAP32[$i>>2]|0;
  STACKTOP = sp;return ($140|0);
 }
 return (0)|0;
}
function _whereRangeScanEst($pParse,$pBuilder,$pLower,$pUpper,$pLoop) {
 $pParse = $pParse|0;
 $pBuilder = $pBuilder|0;
 $pLower = $pLower|0;
 $pUpper = $pUpper|0;
 $pLoop = $pLoop|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nNew = 0, $nOut = 0, $or$cond = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $nOut = sp;
 $nNew = sp + 28|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pBuilder;
 HEAP32[$2>>2] = $pLower;
 HEAP32[$3>>2] = $pUpper;
 HEAP32[$4>>2] = $pLoop;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 22|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7 << 16 >> 16;
 HEAP32[$nOut>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$nOut>>2]|0;
 $11 = $10&65535;
 $12 = (_whereRangeAdjust($9,$11)|0);
 HEAP16[$nNew>>1] = $12;
 $13 = HEAP32[$3>>2]|0;
 $14 = HEAP16[$nNew>>1]|0;
 $15 = (_whereRangeAdjust($13,$14)|0);
 HEAP16[$nNew>>1] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = ((($18)) + 8|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20 << 16 >> 16;
  $22 = ($21|0)>(0);
  $23 = HEAP32[$3>>2]|0;
  $24 = ($23|0)!=(0|0);
  $or$cond = $22 & $24;
  if ($or$cond) {
   $25 = HEAP32[$3>>2]|0;
   $26 = ((($25)) + 8|0);
   $27 = HEAP16[$26>>1]|0;
   $28 = $27 << 16 >> 16;
   $29 = ($28|0)>(0);
   if ($29) {
    $30 = HEAP16[$nNew>>1]|0;
    $31 = $30 << 16 >> 16;
    $32 = (($31) - 20)|0;
    $33 = $32&65535;
    HEAP16[$nNew>>1] = $33;
   }
  }
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = ($34|0)!=(0|0);
 $36 = $35&1;
 $37 = HEAP32[$3>>2]|0;
 $38 = ($37|0)!=(0|0);
 $39 = $38&1;
 $40 = (($36) + ($39))|0;
 $41 = HEAP32[$nOut>>2]|0;
 $42 = (($41) - ($40))|0;
 HEAP32[$nOut>>2] = $42;
 $43 = HEAP16[$nNew>>1]|0;
 $44 = $43 << 16 >> 16;
 $45 = ($44|0)<(10);
 if ($45) {
  HEAP16[$nNew>>1] = 10;
 }
 $46 = HEAP16[$nNew>>1]|0;
 $47 = $46 << 16 >> 16;
 $48 = HEAP32[$nOut>>2]|0;
 $49 = ($47|0)<($48|0);
 if (!($49)) {
  $52 = HEAP32[$nOut>>2]|0;
  $53 = $52&65535;
  $54 = HEAP32[$4>>2]|0;
  $55 = ((($54)) + 22|0);
  HEAP16[$55>>1] = $53;
  $56 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $50 = HEAP16[$nNew>>1]|0;
 $51 = $50 << 16 >> 16;
 HEAP32[$nOut>>2] = $51;
 $52 = HEAP32[$nOut>>2]|0;
 $53 = $52&65535;
 $54 = HEAP32[$4>>2]|0;
 $55 = ((($54)) + 22|0);
 HEAP16[$55>>1] = $53;
 $56 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _whereRangeAdjust($pTerm,$nNew) {
 $pTerm = $pTerm|0;
 $nNew = $nNew|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 6|0;
 $nRet = sp + 4|0;
 HEAP32[$0>>2] = $pTerm;
 HEAP16[$1>>1] = $nNew;
 $2 = HEAP16[$1>>1]|0;
 HEAP16[$nRet>>1] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 do {
  if ($4) {
   $5 = HEAP32[$0>>2]|0;
   $6 = ((($5)) + 8|0);
   $7 = HEAP16[$6>>1]|0;
   $8 = $7 << 16 >> 16;
   $9 = ($8|0)<=(0);
   $10 = HEAP32[$0>>2]|0;
   if ($9) {
    $11 = ((($10)) + 8|0);
    $12 = HEAP16[$11>>1]|0;
    $13 = $12 << 16 >> 16;
    $14 = HEAP16[$nRet>>1]|0;
    $15 = $14 << 16 >> 16;
    $16 = (($15) + ($13))|0;
    $17 = $16&65535;
    HEAP16[$nRet>>1] = $17;
    break;
   }
   $18 = ((($10)) + 10|0);
   $19 = HEAP16[$18>>1]|0;
   $20 = $19&65535;
   $21 = $20 & 0;
   $22 = ($21|0)==(0);
   if ($22) {
    $23 = HEAP16[$nRet>>1]|0;
    $24 = $23 << 16 >> 16;
    $25 = (($24) - 20)|0;
    $26 = $25&65535;
    HEAP16[$nRet>>1] = $26;
   }
  }
 } while(0);
 $27 = HEAP16[$nRet>>1]|0;
 STACKTOP = sp;return ($27|0);
}
function _exprIdxCover($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(152);
 if ($6) {
  $7 = HEAP32[$2>>2]|0;
  $8 = ((($7)) + 28|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($9|0)==($14|0);
  if ($15) {
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 24|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = HEAP32[$2>>2]|0;
   $21 = ((($20)) + 32|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = (_sqlite3ColumnOfIndex($19,$22)|0);
   $24 = $23 << 16 >> 16;
   $25 = ($24|0)<(0);
   if ($25) {
    $26 = HEAP32[$1>>2]|0;
    $27 = ((($26)) + 20|0);
    HEAP8[$27>>0] = 1;
    HEAP32[$0>>2] = 2;
    $28 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($28|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3ExprImpliesExpr($pE1,$pE2,$iTab) {
 $pE1 = $pE1|0;
 $pE2 = $pE2|0;
 $iTab = $iTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $pE1;
 HEAP32[$2>>2] = $pE2;
 HEAP32[$3>>2] = $iTab;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_sqlite3ExprCompare($4,$5,$6)|0);
 $8 = ($7|0)==(0);
 if ($8) {
  HEAP32[$0>>2] = 1;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(27);
 do {
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = ((($14)) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$3>>2]|0;
   $18 = (_sqlite3ExprImpliesExpr($13,$16,$17)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$1>>2]|0;
    $21 = HEAP32[$2>>2]|0;
    $22 = ((($21)) + 16|0);
    $23 = HEAP32[$22>>2]|0;
    $24 = HEAP32[$3>>2]|0;
    $25 = (_sqlite3ExprImpliesExpr($20,$23,$24)|0);
    $26 = ($25|0)!=(0);
    if (!($26)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 } while(0);
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)==(35);
 if ($30) {
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = ((($34)) + 12|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$3>>2]|0;
  $38 = (_sqlite3ExprCompare($33,$36,$37)|0);
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   $43 = ($42|0)!=(34);
   if ($43) {
    $44 = HEAP32[$1>>2]|0;
    $45 = HEAP8[$44>>0]|0;
    $46 = $45&255;
    $47 = ($46|0)!=(29);
    if ($47) {
     HEAP32[$0>>2] = 1;
     $48 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($48|0);
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _allocateIndexInfo($pParse,$pWC,$0,$1,$pSrc,$pOrderBy,$pmNoOmit) {
 $pParse = $pParse|0;
 $pWC = $pWC|0;
 $0 = $0|0;
 $1 = $1|0;
 $pSrc = $pSrc|0;
 $pOrderBy = $pOrderBy|0;
 $pmNoOmit = $pmNoOmit|0;
 var $$ = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0;
 var $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0;
 var $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0;
 var $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0;
 var $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0;
 var $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0;
 var $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0;
 var $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0;
 var $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0;
 var $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $j = 0, $mNoOmit = 0;
 var $n = 0, $nOrderBy = 0, $nTerm = 0, $op = 0, $pExpr = 0, $pExpr1 = 0, $pIdxCons = 0, $pIdxInfo = 0, $pIdxOrderBy = 0, $pTerm = 0, $pUsage = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp + 8|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp;
 $6 = sp + 68|0;
 $7 = sp + 64|0;
 $8 = sp + 60|0;
 $i = sp + 56|0;
 $j = sp + 52|0;
 $nTerm = sp + 48|0;
 $pIdxCons = sp + 44|0;
 $pIdxOrderBy = sp + 40|0;
 $pUsage = sp + 36|0;
 $pTerm = sp + 32|0;
 $nOrderBy = sp + 28|0;
 $pIdxInfo = sp + 24|0;
 $mNoOmit = sp + 84|0;
 $n = sp + 20|0;
 $pExpr = sp + 16|0;
 $op = sp + 86|0;
 $pExpr1 = sp + 12|0;
 HEAP32[$3>>2] = $pParse;
 HEAP32[$4>>2] = $pWC;
 $9 = $5;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$6>>2] = $pSrc;
 HEAP32[$7>>2] = $pOrderBy;
 HEAP32[$8>>2] = $pmNoOmit;
 HEAP16[$mNoOmit>>1] = 0;
 HEAP32[$nTerm>>2] = 0;
 HEAP32[$i>>2] = 0;
 $13 = HEAP32[$4>>2]|0;
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pTerm>>2] = $15;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$4>>2]|0;
  $18 = ((($17)) + 12|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$pTerm>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$6>>2]|0;
  $25 = ((($24)) + 44|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($23|0)!=($26|0);
  if (!($27)) {
   $28 = HEAP32[$pTerm>>2]|0;
   $29 = ((($28)) + 32|0);
   $30 = $29;
   $31 = $30;
   $32 = HEAP32[$31>>2]|0;
   $33 = (($30) + 4)|0;
   $34 = $33;
   $35 = HEAP32[$34>>2]|0;
   $36 = $5;
   $37 = $36;
   $38 = HEAP32[$37>>2]|0;
   $39 = (($36) + 4)|0;
   $40 = $39;
   $41 = HEAP32[$40>>2]|0;
   $42 = $32 & $38;
   $43 = $35 & $41;
   $44 = ($42|0)!=(0);
   $45 = ($43|0)!=(0);
   $46 = $44 | $45;
   if (!($46)) {
    $47 = HEAP32[$pTerm>>2]|0;
    $48 = ((($47)) + 12|0);
    $49 = HEAP16[$48>>1]|0;
    $50 = $49&65535;
    $51 = $50 & -2433;
    $52 = ($51|0)==(0);
    if (!($52)) {
     $53 = HEAP32[$pTerm>>2]|0;
     $54 = ((($53)) + 10|0);
     $55 = HEAP16[$54>>1]|0;
     $56 = $55&65535;
     $57 = $56 & 0;
     $58 = ($57|0)!=(0);
     if (!($58)) {
      $59 = HEAP32[$nTerm>>2]|0;
      $60 = (($59) + 1)|0;
      HEAP32[$nTerm>>2] = $60;
     }
    }
   }
  }
  $61 = HEAP32[$i>>2]|0;
  $62 = (($61) + 1)|0;
  HEAP32[$i>>2] = $62;
  $63 = HEAP32[$pTerm>>2]|0;
  $64 = ((($63)) + 48|0);
  HEAP32[$pTerm>>2] = $64;
 }
 HEAP32[$nOrderBy>>2] = 0;
 $65 = HEAP32[$7>>2]|0;
 $66 = ($65|0)!=(0|0);
 if ($66) {
  $67 = HEAP32[$7>>2]|0;
  $68 = HEAP32[$67>>2]|0;
  HEAP32[$n>>2] = $68;
  HEAP32[$i>>2] = 0;
  while(1) {
   $69 = HEAP32[$i>>2]|0;
   $70 = HEAP32[$n>>2]|0;
   $71 = ($69|0)<($70|0);
   if (!($71)) {
    break;
   }
   $72 = HEAP32[$i>>2]|0;
   $73 = HEAP32[$7>>2]|0;
   $74 = ((($73)) + 4|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = (($75) + (($72*20)|0)|0);
   $77 = HEAP32[$76>>2]|0;
   HEAP32[$pExpr>>2] = $77;
   $78 = HEAP32[$pExpr>>2]|0;
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   $81 = ($80|0)!=(152);
   if ($81) {
    break;
   }
   $82 = HEAP32[$pExpr>>2]|0;
   $83 = ((($82)) + 28|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$6>>2]|0;
   $86 = ((($85)) + 44|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = ($84|0)!=($87|0);
   if ($88) {
    break;
   }
   $89 = HEAP32[$i>>2]|0;
   $90 = (($89) + 1)|0;
   HEAP32[$i>>2] = $90;
  }
  $91 = HEAP32[$i>>2]|0;
  $92 = HEAP32[$n>>2]|0;
  $93 = ($91|0)==($92|0);
  if ($93) {
   $94 = HEAP32[$n>>2]|0;
   HEAP32[$nOrderBy>>2] = $94;
  }
 }
 $95 = HEAP32[$3>>2]|0;
 $96 = HEAP32[$95>>2]|0;
 $97 = HEAP32[$nTerm>>2]|0;
 $98 = ($97*20)|0;
 $99 = (72 + ($98))|0;
 $100 = HEAP32[$nOrderBy>>2]|0;
 $101 = $100<<3;
 $102 = (($99) + ($101))|0;
 $103 = (_sqlite3DbMallocZero($96,$102,0)|0);
 HEAP32[$pIdxInfo>>2] = $103;
 $104 = HEAP32[$pIdxInfo>>2]|0;
 $105 = ($104|0)==(0|0);
 if ($105) {
  $106 = HEAP32[$3>>2]|0;
  _sqlite3ErrorMsg($106,25016,$vararg_buffer);
  HEAP32[$2>>2] = 0;
  $277 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($277|0);
 }
 $107 = HEAP32[$pIdxInfo>>2]|0;
 $108 = ((($107)) + 72|0);
 HEAP32[$pIdxCons>>2] = $108;
 $109 = HEAP32[$nTerm>>2]|0;
 $110 = HEAP32[$pIdxCons>>2]|0;
 $111 = (($110) + (($109*12)|0)|0);
 HEAP32[$pIdxOrderBy>>2] = $111;
 $112 = HEAP32[$nOrderBy>>2]|0;
 $113 = HEAP32[$pIdxOrderBy>>2]|0;
 $114 = (($113) + ($112<<3)|0);
 HEAP32[$pUsage>>2] = $114;
 $115 = HEAP32[$nTerm>>2]|0;
 $116 = HEAP32[$pIdxInfo>>2]|0;
 HEAP32[$116>>2] = $115;
 $117 = HEAP32[$nOrderBy>>2]|0;
 $118 = HEAP32[$pIdxInfo>>2]|0;
 $119 = ((($118)) + 8|0);
 HEAP32[$119>>2] = $117;
 $120 = HEAP32[$pIdxCons>>2]|0;
 $121 = HEAP32[$pIdxInfo>>2]|0;
 $122 = ((($121)) + 4|0);
 HEAP32[$122>>2] = $120;
 $123 = HEAP32[$pIdxOrderBy>>2]|0;
 $124 = HEAP32[$pIdxInfo>>2]|0;
 $125 = ((($124)) + 12|0);
 HEAP32[$125>>2] = $123;
 $126 = HEAP32[$pUsage>>2]|0;
 $127 = HEAP32[$pIdxInfo>>2]|0;
 $128 = ((($127)) + 16|0);
 HEAP32[$128>>2] = $126;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 $129 = HEAP32[$4>>2]|0;
 $130 = ((($129)) + 20|0);
 $131 = HEAP32[$130>>2]|0;
 HEAP32[$pTerm>>2] = $131;
 while(1) {
  $132 = HEAP32[$i>>2]|0;
  $133 = HEAP32[$4>>2]|0;
  $134 = ((($133)) + 12|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = ($132|0)<($135|0);
  if (!($136)) {
   break;
  }
  $137 = HEAP32[$pTerm>>2]|0;
  $138 = ((($137)) + 20|0);
  $139 = HEAP32[$138>>2]|0;
  $140 = HEAP32[$6>>2]|0;
  $141 = ((($140)) + 44|0);
  $142 = HEAP32[$141>>2]|0;
  $143 = ($139|0)!=($142|0);
  if (!($143)) {
   $144 = HEAP32[$pTerm>>2]|0;
   $145 = ((($144)) + 32|0);
   $146 = $145;
   $147 = $146;
   $148 = HEAP32[$147>>2]|0;
   $149 = (($146) + 4)|0;
   $150 = $149;
   $151 = HEAP32[$150>>2]|0;
   $152 = $5;
   $153 = $152;
   $154 = HEAP32[$153>>2]|0;
   $155 = (($152) + 4)|0;
   $156 = $155;
   $157 = HEAP32[$156>>2]|0;
   $158 = $148 & $154;
   $159 = $151 & $157;
   $160 = ($158|0)!=(0);
   $161 = ($159|0)!=(0);
   $162 = $160 | $161;
   if (!($162)) {
    $163 = HEAP32[$pTerm>>2]|0;
    $164 = ((($163)) + 12|0);
    $165 = HEAP16[$164>>1]|0;
    $166 = $165&65535;
    $167 = $166 & -2433;
    $168 = ($167|0)==(0);
    if (!($168)) {
     $169 = HEAP32[$pTerm>>2]|0;
     $170 = ((($169)) + 10|0);
     $171 = HEAP16[$170>>1]|0;
     $172 = $171&65535;
     $173 = $172 & 0;
     $174 = ($173|0)!=(0);
     if (!($174)) {
      $175 = HEAP32[$pTerm>>2]|0;
      $176 = ((($175)) + 28|0);
      $177 = HEAP32[$176>>2]|0;
      $178 = HEAP32[$j>>2]|0;
      $179 = HEAP32[$pIdxCons>>2]|0;
      $180 = (($179) + (($178*12)|0)|0);
      HEAP32[$180>>2] = $177;
      $181 = HEAP32[$i>>2]|0;
      $182 = HEAP32[$j>>2]|0;
      $183 = HEAP32[$pIdxCons>>2]|0;
      $184 = (($183) + (($182*12)|0)|0);
      $185 = ((($184)) + 8|0);
      HEAP32[$185>>2] = $181;
      $186 = HEAP32[$pTerm>>2]|0;
      $187 = ((($186)) + 12|0);
      $188 = HEAP16[$187>>1]|0;
      $189 = $188&255;
      $190 = $189&255;
      $191 = $190 & 8191;
      $192 = $191&255;
      HEAP8[$op>>0] = $192;
      $193 = HEAP8[$op>>0]|0;
      $194 = $193&255;
      $195 = ($194|0)==(1);
      $$ = $195 ? 2 : $192;
      HEAP8[$op>>0] = $$;
      $196 = HEAP8[$op>>0]|0;
      $197 = $196&255;
      $198 = ($197|0)==(64);
      if ($198) {
       $199 = HEAP32[$pTerm>>2]|0;
       $200 = ((($199)) + 15|0);
       $201 = HEAP8[$200>>0]|0;
       HEAP8[$op>>0] = $201;
      }
      $202 = HEAP8[$op>>0]|0;
      $203 = HEAP32[$j>>2]|0;
      $204 = HEAP32[$pIdxCons>>2]|0;
      $205 = (($204) + (($203*12)|0)|0);
      $206 = ((($205)) + 4|0);
      HEAP8[$206>>0] = $202;
      $207 = HEAP8[$op>>0]|0;
      $208 = $207&255;
      $209 = $208 & 60;
      $210 = ($209|0)!=(0);
      if ($210) {
       $211 = HEAP32[$pTerm>>2]|0;
       $212 = HEAP32[$211>>2]|0;
       $213 = ((($212)) + 16|0);
       $214 = HEAP32[$213>>2]|0;
       $215 = (_sqlite3ExprIsVector($214)|0);
       $216 = ($215|0)!=(0);
       if ($216) {
        $217 = HEAP32[$i>>2]|0;
        $218 = ($217|0)<(16);
        if ($218) {
         $219 = HEAP32[$i>>2]|0;
         $220 = 1 << $219;
         $221 = HEAP16[$mNoOmit>>1]|0;
         $222 = $221&65535;
         $223 = $222 | $220;
         $224 = $223&65535;
         HEAP16[$mNoOmit>>1] = $224;
        }
        $225 = HEAP8[$op>>0]|0;
        $226 = $225&255;
        $227 = ($226|0)==(16);
        if ($227) {
         $228 = HEAP32[$j>>2]|0;
         $229 = HEAP32[$pIdxCons>>2]|0;
         $230 = (($229) + (($228*12)|0)|0);
         $231 = ((($230)) + 4|0);
         HEAP8[$231>>0] = 8;
        }
        $232 = HEAP8[$op>>0]|0;
        $233 = $232&255;
        $234 = ($233|0)==(4);
        if ($234) {
         $235 = HEAP32[$j>>2]|0;
         $236 = HEAP32[$pIdxCons>>2]|0;
         $237 = (($236) + (($235*12)|0)|0);
         $238 = ((($237)) + 4|0);
         HEAP8[$238>>0] = 32;
        }
       }
      }
      $239 = HEAP32[$j>>2]|0;
      $240 = (($239) + 1)|0;
      HEAP32[$j>>2] = $240;
     }
    }
   }
  }
  $241 = HEAP32[$i>>2]|0;
  $242 = (($241) + 1)|0;
  HEAP32[$i>>2] = $242;
  $243 = HEAP32[$pTerm>>2]|0;
  $244 = ((($243)) + 48|0);
  HEAP32[$pTerm>>2] = $244;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $245 = HEAP32[$i>>2]|0;
  $246 = HEAP32[$nOrderBy>>2]|0;
  $247 = ($245|0)<($246|0);
  if (!($247)) {
   break;
  }
  $248 = HEAP32[$i>>2]|0;
  $249 = HEAP32[$7>>2]|0;
  $250 = ((($249)) + 4|0);
  $251 = HEAP32[$250>>2]|0;
  $252 = (($251) + (($248*20)|0)|0);
  $253 = HEAP32[$252>>2]|0;
  HEAP32[$pExpr1>>2] = $253;
  $254 = HEAP32[$pExpr1>>2]|0;
  $255 = ((($254)) + 32|0);
  $256 = HEAP16[$255>>1]|0;
  $257 = $256 << 16 >> 16;
  $258 = HEAP32[$i>>2]|0;
  $259 = HEAP32[$pIdxOrderBy>>2]|0;
  $260 = (($259) + ($258<<3)|0);
  HEAP32[$260>>2] = $257;
  $261 = HEAP32[$i>>2]|0;
  $262 = HEAP32[$7>>2]|0;
  $263 = ((($262)) + 4|0);
  $264 = HEAP32[$263>>2]|0;
  $265 = (($264) + (($261*20)|0)|0);
  $266 = ((($265)) + 12|0);
  $267 = HEAP8[$266>>0]|0;
  $268 = HEAP32[$i>>2]|0;
  $269 = HEAP32[$pIdxOrderBy>>2]|0;
  $270 = (($269) + ($268<<3)|0);
  $271 = ((($270)) + 4|0);
  HEAP8[$271>>0] = $267;
  $272 = HEAP32[$i>>2]|0;
  $273 = (($272) + 1)|0;
  HEAP32[$i>>2] = $273;
 }
 $274 = HEAP16[$mNoOmit>>1]|0;
 $275 = HEAP32[$8>>2]|0;
 HEAP16[$275>>1] = $274;
 $276 = HEAP32[$pIdxInfo>>2]|0;
 HEAP32[$2>>2] = $276;
 $277 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($277|0);
}
function _whereLoopAddVirtualOne($pBuilder,$0,$1,$2,$3,$mExclude,$pIdxInfo,$mNoOmit,$pbIn) {
 $pBuilder = $pBuilder|0;
 $0 = $0|0;
 $1 = $1|0;
 $2 = $2|0;
 $3 = $3|0;
 $mExclude = $mExclude|0;
 $pIdxInfo = $pIdxInfo|0;
 $mNoOmit = $mNoOmit|0;
 $pbIn = $pbIn|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0;
 var $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0;
 var $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0;
 var $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0;
 var $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0;
 var $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0;
 var $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0;
 var $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0.0, $333 = 0, $334 = 0, $335 = 0;
 var $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0;
 var $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0;
 var $372 = 0, $373 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iTerm = 0, $j = 0, $mxTerm = 0, $nConstraint = 0, $or$cond = 0, $pIdxCons = 0, $pNew = 0, $pParse = 0, $pSrc = 0;
 var $pTerm = 0, $pTerm1 = 0, $pUsage = 0, $pWC = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp + 16|0;
 $4 = sp + 88|0;
 $5 = sp + 84|0;
 $6 = sp + 8|0;
 $7 = sp;
 $8 = sp + 94|0;
 $9 = sp + 80|0;
 $10 = sp + 92|0;
 $11 = sp + 76|0;
 $pWC = sp + 72|0;
 $pIdxCons = sp + 68|0;
 $pUsage = sp + 64|0;
 $i = sp + 60|0;
 $mxTerm = sp + 56|0;
 $rc = sp + 52|0;
 $pNew = sp + 48|0;
 $pParse = sp + 44|0;
 $pSrc = sp + 40|0;
 $nConstraint = sp + 36|0;
 $pTerm = sp + 32|0;
 $iTerm = sp + 28|0;
 $pTerm1 = sp + 24|0;
 $j = sp + 20|0;
 HEAP32[$5>>2] = $pBuilder;
 $12 = $6;
 $13 = $12;
 HEAP32[$13>>2] = $0;
 $14 = (($12) + 4)|0;
 $15 = $14;
 HEAP32[$15>>2] = $1;
 $16 = $7;
 $17 = $16;
 HEAP32[$17>>2] = $2;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $3;
 HEAP16[$8>>1] = $mExclude;
 HEAP32[$9>>2] = $pIdxInfo;
 HEAP16[$10>>1] = $mNoOmit;
 HEAP32[$11>>2] = $pbIn;
 $20 = HEAP32[$5>>2]|0;
 $21 = ((($20)) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pWC>>2] = $22;
 $23 = HEAP32[$9>>2]|0;
 $24 = ((($23)) + 16|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pUsage>>2] = $25;
 HEAP32[$rc>>2] = 0;
 $26 = HEAP32[$5>>2]|0;
 $27 = ((($26)) + 12|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pNew>>2] = $28;
 $29 = HEAP32[$5>>2]|0;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$pParse>>2] = $31;
 $32 = HEAP32[$pNew>>2]|0;
 $33 = ((($32)) + 16|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = HEAP32[$5>>2]|0;
 $37 = HEAP32[$36>>2]|0;
 $38 = ((($37)) + 4|0);
 $39 = HEAP32[$38>>2]|0;
 $40 = ((($39)) + 8|0);
 $41 = (($40) + (($35*72)|0)|0);
 HEAP32[$pSrc>>2] = $41;
 $42 = HEAP32[$9>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$nConstraint>>2] = $43;
 $44 = HEAP32[$11>>2]|0;
 HEAP32[$44>>2] = 0;
 $45 = $6;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($45) + 4)|0;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$pNew>>2]|0;
 $52 = $51;
 $53 = $52;
 HEAP32[$53>>2] = $47;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = $50;
 $56 = HEAP32[$9>>2]|0;
 $57 = ((($56)) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 HEAP32[$pIdxCons>>2] = $58;
 HEAP32[$i>>2] = 0;
 while(1) {
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$nConstraint>>2]|0;
  $61 = ($59|0)<($60|0);
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$pIdxCons>>2]|0;
  $63 = ((($62)) + 8|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = HEAP32[$pWC>>2]|0;
  $66 = ((($65)) + 20|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + (($64*48)|0)|0);
  HEAP32[$pTerm>>2] = $68;
  $69 = HEAP32[$pIdxCons>>2]|0;
  $70 = ((($69)) + 5|0);
  HEAP8[$70>>0] = 0;
  $71 = HEAP32[$pTerm>>2]|0;
  $72 = ((($71)) + 32|0);
  $73 = $72;
  $74 = $73;
  $75 = HEAP32[$74>>2]|0;
  $76 = (($73) + 4)|0;
  $77 = $76;
  $78 = HEAP32[$77>>2]|0;
  $79 = $7;
  $80 = $79;
  $81 = HEAP32[$80>>2]|0;
  $82 = (($79) + 4)|0;
  $83 = $82;
  $84 = HEAP32[$83>>2]|0;
  $85 = $75 & $81;
  $86 = $78 & $84;
  $87 = HEAP32[$pTerm>>2]|0;
  $88 = ((($87)) + 32|0);
  $89 = $88;
  $90 = $89;
  $91 = HEAP32[$90>>2]|0;
  $92 = (($89) + 4)|0;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = ($85|0)==($91|0);
  $96 = ($86|0)==($94|0);
  $97 = $95 & $96;
  if ($97) {
   $98 = HEAP32[$pTerm>>2]|0;
   $99 = ((($98)) + 12|0);
   $100 = HEAP16[$99>>1]|0;
   $101 = $100&65535;
   $102 = HEAP16[$8>>1]|0;
   $103 = $102&65535;
   $104 = $101 & $103;
   $105 = ($104|0)==(0);
   if ($105) {
    $106 = HEAP32[$pIdxCons>>2]|0;
    $107 = ((($106)) + 5|0);
    HEAP8[$107>>0] = 1;
   }
  }
  $108 = HEAP32[$i>>2]|0;
  $109 = (($108) + 1)|0;
  HEAP32[$i>>2] = $109;
  $110 = HEAP32[$pIdxCons>>2]|0;
  $111 = ((($110)) + 12|0);
  HEAP32[$pIdxCons>>2] = $111;
 }
 $112 = HEAP32[$pUsage>>2]|0;
 $113 = HEAP32[$nConstraint>>2]|0;
 $114 = $113<<3;
 _memset(($112|0),0,($114|0))|0;
 $115 = HEAP32[$9>>2]|0;
 $116 = ((($115)) + 24|0);
 HEAP32[$116>>2] = 0;
 $117 = HEAP32[$9>>2]|0;
 $118 = ((($117)) + 20|0);
 HEAP32[$118>>2] = 0;
 $119 = HEAP32[$9>>2]|0;
 $120 = ((($119)) + 32|0);
 HEAP32[$120>>2] = 0;
 $121 = HEAP32[$9>>2]|0;
 $122 = ((($121)) + 40|0);
 HEAPF64[$122>>3] = 4.9999999999999998E+98;
 $123 = HEAP32[$9>>2]|0;
 $124 = ((($123)) + 48|0);
 $125 = $124;
 $126 = $125;
 HEAP32[$126>>2] = 25;
 $127 = (($125) + 4)|0;
 $128 = $127;
 HEAP32[$128>>2] = 0;
 $129 = HEAP32[$9>>2]|0;
 $130 = ((($129)) + 56|0);
 HEAP32[$130>>2] = 0;
 $131 = HEAP32[$pSrc>>2]|0;
 $132 = ((($131)) + 56|0);
 $133 = $132;
 $134 = $133;
 $135 = HEAP32[$134>>2]|0;
 $136 = (($133) + 4)|0;
 $137 = $136;
 $138 = HEAP32[$137>>2]|0;
 $139 = HEAP32[$9>>2]|0;
 $140 = ((($139)) + 64|0);
 $141 = $140;
 $142 = $141;
 HEAP32[$142>>2] = $135;
 $143 = (($141) + 4)|0;
 $144 = $143;
 HEAP32[$144>>2] = $138;
 $145 = HEAP32[$pParse>>2]|0;
 $146 = HEAP32[$pSrc>>2]|0;
 $147 = ((($146)) + 16|0);
 $148 = HEAP32[$147>>2]|0;
 $149 = HEAP32[$9>>2]|0;
 $150 = (_vtabBestIndex($145,$148,$149)|0);
 HEAP32[$rc>>2] = $150;
 $151 = HEAP32[$rc>>2]|0;
 $152 = ($151|0)!=(0);
 if ($152) {
  $153 = HEAP32[$rc>>2]|0;
  HEAP32[$4>>2] = $153;
  $373 = HEAP32[$4>>2]|0;
  STACKTOP = sp;return ($373|0);
 }
 HEAP32[$mxTerm>>2] = -1;
 HEAP32[$i>>2] = 0;
 while(1) {
  $154 = HEAP32[$i>>2]|0;
  $155 = HEAP32[$nConstraint>>2]|0;
  $156 = ($154|0)<($155|0);
  if (!($156)) {
   break;
  }
  $157 = HEAP32[$i>>2]|0;
  $158 = HEAP32[$pNew>>2]|0;
  $159 = ((($158)) + 48|0);
  $160 = HEAP32[$159>>2]|0;
  $161 = (($160) + ($157<<2)|0);
  HEAP32[$161>>2] = 0;
  $162 = HEAP32[$i>>2]|0;
  $163 = (($162) + 1)|0;
  HEAP32[$i>>2] = $163;
 }
 $164 = HEAP32[$pNew>>2]|0;
 $165 = ((($164)) + 24|0);
 $166 = ((($165)) + 6|0);
 HEAP16[$166>>1] = 0;
 $167 = HEAP32[$9>>2]|0;
 $168 = ((($167)) + 4|0);
 $169 = HEAP32[$168>>2]|0;
 HEAP32[$pIdxCons>>2] = $169;
 HEAP32[$i>>2] = 0;
 while(1) {
  $170 = HEAP32[$i>>2]|0;
  $171 = HEAP32[$nConstraint>>2]|0;
  $172 = ($170|0)<($171|0);
  if (!($172)) {
   break;
  }
  $173 = HEAP32[$i>>2]|0;
  $174 = HEAP32[$pUsage>>2]|0;
  $175 = (($174) + ($173<<3)|0);
  $176 = HEAP32[$175>>2]|0;
  $177 = (($176) - 1)|0;
  HEAP32[$iTerm>>2] = $177;
  $178 = ($177|0)>=(0);
  if ($178) {
   $179 = HEAP32[$pIdxCons>>2]|0;
   $180 = ((($179)) + 8|0);
   $181 = HEAP32[$180>>2]|0;
   HEAP32[$j>>2] = $181;
   $182 = HEAP32[$iTerm>>2]|0;
   $183 = HEAP32[$nConstraint>>2]|0;
   $184 = ($182|0)>=($183|0);
   $185 = HEAP32[$j>>2]|0;
   $186 = ($185|0)<(0);
   $or$cond = $184 | $186;
   if ($or$cond) {
    label = 19;
    break;
   }
   $187 = HEAP32[$j>>2]|0;
   $188 = HEAP32[$pWC>>2]|0;
   $189 = ((($188)) + 12|0);
   $190 = HEAP32[$189>>2]|0;
   $191 = ($187|0)>=($190|0);
   if ($191) {
    label = 19;
    break;
   }
   $192 = HEAP32[$iTerm>>2]|0;
   $193 = HEAP32[$pNew>>2]|0;
   $194 = ((($193)) + 48|0);
   $195 = HEAP32[$194>>2]|0;
   $196 = (($195) + ($192<<2)|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = ($197|0)!=(0|0);
   if ($198) {
    label = 19;
    break;
   }
   $199 = HEAP32[$pIdxCons>>2]|0;
   $200 = ((($199)) + 5|0);
   $201 = HEAP8[$200>>0]|0;
   $202 = $201&255;
   $203 = ($202|0)==(0);
   if ($203) {
    label = 19;
    break;
   }
   $210 = HEAP32[$j>>2]|0;
   $211 = HEAP32[$pWC>>2]|0;
   $212 = ((($211)) + 20|0);
   $213 = HEAP32[$212>>2]|0;
   $214 = (($213) + (($210*48)|0)|0);
   HEAP32[$pTerm1>>2] = $214;
   $215 = HEAP32[$pTerm1>>2]|0;
   $216 = ((($215)) + 32|0);
   $217 = $216;
   $218 = $217;
   $219 = HEAP32[$218>>2]|0;
   $220 = (($217) + 4)|0;
   $221 = $220;
   $222 = HEAP32[$221>>2]|0;
   $223 = HEAP32[$pNew>>2]|0;
   $224 = $223;
   $225 = $224;
   $226 = HEAP32[$225>>2]|0;
   $227 = (($224) + 4)|0;
   $228 = $227;
   $229 = HEAP32[$228>>2]|0;
   $230 = $226 | $219;
   $231 = $229 | $222;
   $232 = $223;
   $233 = $232;
   HEAP32[$233>>2] = $230;
   $234 = (($232) + 4)|0;
   $235 = $234;
   HEAP32[$235>>2] = $231;
   $236 = HEAP32[$pTerm1>>2]|0;
   $237 = HEAP32[$iTerm>>2]|0;
   $238 = HEAP32[$pNew>>2]|0;
   $239 = ((($238)) + 48|0);
   $240 = HEAP32[$239>>2]|0;
   $241 = (($240) + ($237<<2)|0);
   HEAP32[$241>>2] = $236;
   $242 = HEAP32[$iTerm>>2]|0;
   $243 = HEAP32[$mxTerm>>2]|0;
   $244 = ($242|0)>($243|0);
   if ($244) {
    $245 = HEAP32[$iTerm>>2]|0;
    HEAP32[$mxTerm>>2] = $245;
   }
   $246 = HEAP32[$iTerm>>2]|0;
   $247 = ($246|0)<(16);
   if ($247) {
    $248 = HEAP32[$i>>2]|0;
    $249 = HEAP32[$pUsage>>2]|0;
    $250 = (($249) + ($248<<3)|0);
    $251 = ((($250)) + 4|0);
    $252 = HEAP8[$251>>0]|0;
    $253 = $252&255;
    $254 = ($253|0)!=(0);
    if ($254) {
     $255 = HEAP32[$iTerm>>2]|0;
     $256 = 1 << $255;
     $257 = HEAP32[$pNew>>2]|0;
     $258 = ((($257)) + 24|0);
     $259 = ((($258)) + 6|0);
     $260 = HEAP16[$259>>1]|0;
     $261 = $260&65535;
     $262 = $261 | $256;
     $263 = $262&65535;
     HEAP16[$259>>1] = $263;
    }
   }
   $264 = HEAP32[$pTerm1>>2]|0;
   $265 = ((($264)) + 12|0);
   $266 = HEAP16[$265>>1]|0;
   $267 = $266&65535;
   $268 = $267 & 1;
   $269 = ($268|0)!=(0);
   if ($269) {
    $270 = HEAP32[$9>>2]|0;
    $271 = ((($270)) + 32|0);
    HEAP32[$271>>2] = 0;
    $272 = HEAP32[$9>>2]|0;
    $273 = ((($272)) + 56|0);
    $274 = HEAP32[$273>>2]|0;
    $275 = $274 & -2;
    HEAP32[$273>>2] = $275;
    $276 = HEAP32[$11>>2]|0;
    HEAP32[$276>>2] = 1;
   }
  }
  $277 = HEAP32[$i>>2]|0;
  $278 = (($277) + 1)|0;
  HEAP32[$i>>2] = $278;
  $279 = HEAP32[$pIdxCons>>2]|0;
  $280 = ((($279)) + 12|0);
  HEAP32[$pIdxCons>>2] = $280;
 }
 if ((label|0) == 19) {
  HEAP32[$rc>>2] = 1;
  $204 = HEAP32[$pParse>>2]|0;
  $205 = HEAP32[$pSrc>>2]|0;
  $206 = ((($205)) + 16|0);
  $207 = HEAP32[$206>>2]|0;
  $208 = HEAP32[$207>>2]|0;
  HEAP32[$vararg_buffer>>2] = $208;
  _sqlite3ErrorMsg($204,34806,$vararg_buffer);
  $209 = HEAP32[$rc>>2]|0;
  HEAP32[$4>>2] = $209;
  $373 = HEAP32[$4>>2]|0;
  STACKTOP = sp;return ($373|0);
 }
 $281 = HEAP16[$10>>1]|0;
 $282 = $281&65535;
 $283 = $282 ^ -1;
 $284 = HEAP32[$pNew>>2]|0;
 $285 = ((($284)) + 24|0);
 $286 = ((($285)) + 6|0);
 $287 = HEAP16[$286>>1]|0;
 $288 = $287&65535;
 $289 = $288 & $283;
 $290 = $289&65535;
 HEAP16[$286>>1] = $290;
 $291 = HEAP32[$mxTerm>>2]|0;
 $292 = (($291) + 1)|0;
 $293 = $292&65535;
 $294 = HEAP32[$pNew>>2]|0;
 $295 = ((($294)) + 40|0);
 HEAP16[$295>>1] = $293;
 $296 = HEAP32[$9>>2]|0;
 $297 = ((($296)) + 20|0);
 $298 = HEAP32[$297>>2]|0;
 $299 = HEAP32[$pNew>>2]|0;
 $300 = ((($299)) + 24|0);
 HEAP32[$300>>2] = $298;
 $301 = HEAP32[$9>>2]|0;
 $302 = ((($301)) + 28|0);
 $303 = HEAP32[$302>>2]|0;
 $304 = $303&255;
 $305 = HEAP32[$pNew>>2]|0;
 $306 = ((($305)) + 24|0);
 $307 = ((($306)) + 4|0);
 HEAP8[$307>>0] = $304;
 $308 = HEAP32[$9>>2]|0;
 $309 = ((($308)) + 28|0);
 HEAP32[$309>>2] = 0;
 $310 = HEAP32[$9>>2]|0;
 $311 = ((($310)) + 24|0);
 $312 = HEAP32[$311>>2]|0;
 $313 = HEAP32[$pNew>>2]|0;
 $314 = ((($313)) + 24|0);
 $315 = ((($314)) + 8|0);
 HEAP32[$315>>2] = $312;
 $316 = HEAP32[$9>>2]|0;
 $317 = ((($316)) + 32|0);
 $318 = HEAP32[$317>>2]|0;
 $319 = ($318|0)!=(0);
 if ($319) {
  $320 = HEAP32[$9>>2]|0;
  $321 = ((($320)) + 8|0);
  $322 = HEAP32[$321>>2]|0;
  $324 = $322;
 } else {
  $324 = 0;
 }
 $323 = $324&255;
 $325 = HEAP32[$pNew>>2]|0;
 $326 = ((($325)) + 24|0);
 $327 = ((($326)) + 5|0);
 HEAP8[$327>>0] = $323;
 $328 = HEAP32[$pNew>>2]|0;
 $329 = ((($328)) + 18|0);
 HEAP16[$329>>1] = 0;
 $330 = HEAP32[$9>>2]|0;
 $331 = ((($330)) + 40|0);
 $332 = +HEAPF64[$331>>3];
 $333 = (_sqlite3LogEstFromDouble($332)|0);
 $334 = HEAP32[$pNew>>2]|0;
 $335 = ((($334)) + 20|0);
 HEAP16[$335>>1] = $333;
 $336 = HEAP32[$9>>2]|0;
 $337 = ((($336)) + 48|0);
 $338 = $337;
 $339 = $338;
 $340 = HEAP32[$339>>2]|0;
 $341 = (($338) + 4)|0;
 $342 = $341;
 $343 = HEAP32[$342>>2]|0;
 $344 = (_sqlite3LogEst($340,$343)|0);
 $345 = HEAP32[$pNew>>2]|0;
 $346 = ((($345)) + 22|0);
 HEAP16[$346>>1] = $344;
 $347 = HEAP32[$9>>2]|0;
 $348 = ((($347)) + 56|0);
 $349 = HEAP32[$348>>2]|0;
 $350 = $349 & 1;
 $351 = ($350|0)!=(0);
 $352 = HEAP32[$pNew>>2]|0;
 $353 = ((($352)) + 36|0);
 $354 = HEAP32[$353>>2]|0;
 if ($351) {
  $355 = $354 | 4096;
  HEAP32[$353>>2] = $355;
 } else {
  $356 = $354 & -4097;
  HEAP32[$353>>2] = $356;
 }
 $357 = HEAP32[$5>>2]|0;
 $358 = HEAP32[$pNew>>2]|0;
 $359 = (_whereLoopInsert($357,$358)|0);
 HEAP32[$rc>>2] = $359;
 $360 = HEAP32[$pNew>>2]|0;
 $361 = ((($360)) + 24|0);
 $362 = ((($361)) + 4|0);
 $363 = HEAP8[$362>>0]|0;
 $364 = ($363<<24>>24)!=(0);
 if ($364) {
  $365 = HEAP32[$pNew>>2]|0;
  $366 = ((($365)) + 24|0);
  $367 = ((($366)) + 8|0);
  $368 = HEAP32[$367>>2]|0;
  _sqlite3_free($368);
  $369 = HEAP32[$pNew>>2]|0;
  $370 = ((($369)) + 24|0);
  $371 = ((($370)) + 4|0);
  HEAP8[$371>>0] = 0;
 }
 $372 = HEAP32[$rc>>2]|0;
 HEAP32[$4>>2] = $372;
 $373 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($373|0);
}
function _vtabBestIndex($pParse,$pTab,$p) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pVtab = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pVtab = sp + 16|0;
 $rc = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3GetVTable($4,$5)|0);
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pVtab>>2] = $8;
 $9 = HEAP32[$pVtab>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$pVtab>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (FUNCTION_TABLE_iii[$12 & 255]($13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 do {
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   $19 = ($18|0)==(7);
   if ($19) {
    $20 = HEAP32[$0>>2]|0;
    $21 = HEAP32[$20>>2]|0;
    _sqlite3OomFault($21);
    break;
   }
   $22 = HEAP32[$pVtab>>2]|0;
   $23 = ((($22)) + 8|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($24|0)!=(0|0);
   $26 = HEAP32[$0>>2]|0;
   if ($25) {
    $29 = HEAP32[$pVtab>>2]|0;
    $30 = ((($29)) + 8|0);
    $31 = HEAP32[$30>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $31;
    _sqlite3ErrorMsg($26,18391,$vararg_buffer1);
    break;
   } else {
    $27 = HEAP32[$rc>>2]|0;
    $28 = (_sqlite3ErrStr($27)|0);
    HEAP32[$vararg_buffer>>2] = $28;
    _sqlite3ErrorMsg($26,18391,$vararg_buffer);
    break;
   }
  }
 } while(0);
 $32 = HEAP32[$pVtab>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = HEAP32[$33>>2]|0;
 _sqlite3_free($34);
 $35 = HEAP32[$pVtab>>2]|0;
 $36 = ((($35)) + 8|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$0>>2]|0;
 $38 = ((($37)) + 36|0);
 $39 = HEAP32[$38>>2]|0;
 STACKTOP = sp;return ($39|0);
}
function _sqlite3LogEstFromDouble($x) {
 $x = +$x;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0.0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0.0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0, $9 = 0, $a = 0, $e = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 18|0;
 $1 = sp + 8|0;
 $a = sp;
 $e = sp + 16|0;
 HEAPF64[$1>>3] = $x;
 $2 = +HEAPF64[$1>>3];
 $3 = $2 <= 1.0;
 if ($3) {
  HEAP16[$0>>1] = 0;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 }
 $4 = +HEAPF64[$1>>3];
 $5 = $4 <= 2.0E+9;
 if ($5) {
  $6 = +HEAPF64[$1>>3];
  $7 = (~~$6)>>>0;
  $8 = +Math_abs($6) >= 1.0 ? $6 > 0.0 ? (~~+Math_min(+Math_floor($6 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($6 - +(~~$6 >>> 0)) / 4294967296.0) >>> 0 : 0;
  $9 = (_sqlite3LogEst($7,$8)|0);
  HEAP16[$0>>1] = $9;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  ;HEAP32[$a>>2]=HEAP32[$1>>2]|0;HEAP32[$a+4>>2]=HEAP32[$1+4>>2]|0;
  $10 = $a;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (_bitshift64Lshr(($12|0),($15|0),52)|0);
  $17 = tempRet0;
  $18 = (_i64Subtract(($16|0),($17|0),1022,0)|0);
  $19 = tempRet0;
  $20 = $18&65535;
  HEAP16[$e>>1] = $20;
  $21 = HEAP16[$e>>1]|0;
  $22 = $21 << 16 >> 16;
  $23 = ($22*10)|0;
  $24 = $23&65535;
  HEAP16[$0>>1] = $24;
  $25 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _findIndexCol($pParse,$pList,$iBase,$pIdx,$iCol) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $iBase = $iBase|0;
 $pIdx = $pIdx|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $pColl = 0, $zColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $i = sp + 12|0;
 $zColl = sp + 8|0;
 $p = sp + 4|0;
 $pColl = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pList;
 HEAP32[$3>>2] = $iBase;
 HEAP32[$4>>2] = $pIdx;
 HEAP32[$5>>2] = $iCol;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = ((($7)) + 32|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<2)|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$zColl>>2] = $11;
 HEAP32[$i>>2] = 0;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ($12|0)<($14|0);
  if (!($15)) {
   label = 10;
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (($19) + (($16*20)|0)|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (_sqlite3ExprSkipCollate($21)|0);
  HEAP32[$p>>2] = $22;
  $23 = HEAP32[$p>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)==(152);
  if ($26) {
   $27 = HEAP32[$p>>2]|0;
   $28 = ((($27)) + 32|0);
   $29 = HEAP16[$28>>1]|0;
   $30 = $29 << 16 >> 16;
   $31 = HEAP32[$5>>2]|0;
   $32 = HEAP32[$4>>2]|0;
   $33 = ((($32)) + 4|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<1)|0);
   $36 = HEAP16[$35>>1]|0;
   $37 = $36 << 16 >> 16;
   $38 = ($30|0)==($37|0);
   if ($38) {
    $39 = HEAP32[$p>>2]|0;
    $40 = ((($39)) + 28|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = HEAP32[$3>>2]|0;
    $43 = ($41|0)==($42|0);
    if ($43) {
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP32[$i>>2]|0;
     $46 = HEAP32[$2>>2]|0;
     $47 = ((($46)) + 4|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = (($48) + (($45*20)|0)|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = (_sqlite3ExprCollSeq($44,$50)|0);
     HEAP32[$pColl>>2] = $51;
     $52 = HEAP32[$pColl>>2]|0;
     $53 = ($52|0)!=(0|0);
     if ($53) {
      $54 = HEAP32[$pColl>>2]|0;
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$zColl>>2]|0;
      $57 = (_sqlite3StrICmp($55,$56)|0);
      $58 = (0)==($57|0);
      if ($58) {
       label = 8;
       break;
      }
     }
    }
   }
  }
  $60 = HEAP32[$i>>2]|0;
  $61 = (($60) + 1)|0;
  HEAP32[$i>>2] = $61;
 }
 if ((label|0) == 8) {
  $59 = HEAP32[$i>>2]|0;
  HEAP32[$0>>2] = $59;
  $62 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 else if ((label|0) == 10) {
  HEAP32[$0>>2] = -1;
  $62 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($62|0);
 }
 return (0)|0;
}
function _exprAnalyze($pSrc,$pWC,$idxTerm) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $idxTerm = $idxTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0;
 var $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0;
 var $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0;
 var $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0;
 var $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0;
 var $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0;
 var $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0;
 var $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0;
 var $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0;
 var $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0;
 var $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0;
 var $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0;
 var $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0;
 var $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0;
 var $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0;
 var $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0;
 var $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $c5 = 0, $db = 0, $eExtraOp = 0, $eOp2 = 0, $extraRight = 0, $i = 0, $i10 = 0, $i15 = 0, $i4 = 0;
 var $iColumn = 0, $iCur = 0, $idxNew = 0, $idxNew1 = 0, $idxNew11 = 0, $idxNew13 = 0, $idxNew16 = 0, $idxNew2 = 0, $idxNew6 = 0, $isComplete = 0, $nLeft = 0, $noCase = 0, $op = 0, $opMask = 0, $pC = 0, $pDup = 0, $pExpr = 0, $pLeft = 0, $pLeft13 = 0, $pLeft2 = 0;
 var $pLeft8 = 0, $pList = 0, $pMaskSet = 0, $pNew = 0, $pNew12 = 0, $pNewExpr = 0, $pNewExpr1 = 0, $pNewExpr2 = 0, $pNewExpr9 = 0, $pNewTerm = 0, $pParse = 0, $pRight = 0, $pRight14 = 0, $pRight7 = 0, $pStr1 = 0, $pStr2 = 0, $pTerm = 0, $pWInfo = 0, $prereqAll = 0, $prereqColumn = 0;
 var $prereqExpr = 0, $prereqLeft = 0, $wtFlags = 0, $x = 0, $zCollSeqName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $0 = sp + 228|0;
 $1 = sp + 224|0;
 $2 = sp + 220|0;
 $pWInfo = sp + 216|0;
 $pTerm = sp + 212|0;
 $pMaskSet = sp + 208|0;
 $pExpr = sp + 204|0;
 $prereqLeft = sp + 40|0;
 $prereqAll = sp + 32|0;
 $extraRight = sp + 24|0;
 $pStr1 = sp + 200|0;
 $isComplete = sp + 196|0;
 $noCase = sp + 192|0;
 $op = sp + 188|0;
 $pParse = sp + 184|0;
 $db = sp + 180|0;
 $eOp2 = sp + 240|0;
 $x = sp + 16|0;
 $iCur = sp + 176|0;
 $iColumn = sp + 172|0;
 $pLeft = sp + 168|0;
 $pRight = sp + 164|0;
 $opMask = sp + 236|0;
 $pNew = sp + 160|0;
 $pDup = sp + 156|0;
 $eExtraOp = sp + 234|0;
 $idxNew = sp + 152|0;
 $pList = sp + 148|0;
 $i = sp + 144|0;
 $pNewExpr = sp + 140|0;
 $idxNew1 = sp + 136|0;
 $pLeft2 = sp + 132|0;
 $pStr2 = sp + 128|0;
 $pNewExpr1 = sp + 124|0;
 $pNewExpr2 = sp + 120|0;
 $idxNew13 = sp + 116|0;
 $idxNew2 = sp + 112|0;
 $zCollSeqName = sp + 108|0;
 $wtFlags = sp + 232|0;
 $i4 = sp + 104|0;
 $c = sp + 239|0;
 $c5 = sp + 238|0;
 $pC = sp + 100|0;
 $idxNew6 = sp + 96|0;
 $pRight7 = sp + 92|0;
 $pLeft8 = sp + 88|0;
 $pNewTerm = sp + 84|0;
 $prereqColumn = sp + 8|0;
 $prereqExpr = sp;
 $pNewExpr9 = sp + 80|0;
 $nLeft = sp + 76|0;
 $i10 = sp + 72|0;
 $idxNew11 = sp + 68|0;
 $pNew12 = sp + 64|0;
 $pLeft13 = sp + 60|0;
 $pRight14 = sp + 56|0;
 $i15 = sp + 52|0;
 $idxNew16 = sp + 48|0;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $idxTerm;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = $extraRight;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 HEAP32[$pStr1>>2] = 0;
 HEAP32[$isComplete>>2] = 0;
 HEAP32[$noCase>>2] = 0;
 $9 = HEAP32[$pWInfo>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pParse>>2] = $10;
 $11 = HEAP32[$pParse>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 69|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = ($15<<24>>24)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + (($17*48)|0)|0);
 HEAP32[$pTerm>>2] = $21;
 $22 = HEAP32[$pWInfo>>2]|0;
 $23 = ((($22)) + 488|0);
 HEAP32[$pMaskSet>>2] = $23;
 $24 = HEAP32[$pTerm>>2]|0;
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pExpr>>2] = $25;
 $26 = HEAP32[$pMaskSet>>2]|0;
 $27 = HEAP32[$pExpr>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3WhereExprUsage($26,$29)|0);
 $31 = tempRet0;
 $32 = $prereqLeft;
 $33 = $32;
 HEAP32[$33>>2] = $30;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = HEAP32[$pExpr>>2]|0;
 $37 = HEAP8[$36>>0]|0;
 $38 = $37&255;
 HEAP32[$op>>2] = $38;
 $39 = HEAP32[$op>>2]|0;
 $40 = ($39|0)==(33);
 do {
  if ($40) {
   $41 = HEAP32[$pParse>>2]|0;
   $42 = HEAP32[$pExpr>>2]|0;
   $43 = (_sqlite3ExprCheckIN($41,$42)|0);
   $44 = ($43|0)!=(0);
   if ($44) {
    STACKTOP = sp;return;
   }
   $45 = HEAP32[$pExpr>>2]|0;
   $46 = ((($45)) + 4|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = $47 & 2048;
   $49 = ($48|0)!=(0);
   $50 = HEAP32[$pMaskSet>>2]|0;
   $51 = HEAP32[$pExpr>>2]|0;
   $52 = ((($51)) + 20|0);
   if ($49) {
    $53 = HEAP32[$52>>2]|0;
    $54 = (_exprSelectUsage($50,$53)|0);
    $55 = tempRet0;
    $56 = HEAP32[$pTerm>>2]|0;
    $57 = ((($56)) + 32|0);
    $58 = $57;
    $59 = $58;
    HEAP32[$59>>2] = $54;
    $60 = (($58) + 4)|0;
    $61 = $60;
    HEAP32[$61>>2] = $55;
    break;
   } else {
    $62 = HEAP32[$52>>2]|0;
    $63 = (_sqlite3WhereExprListUsage($50,$62)|0);
    $64 = tempRet0;
    $65 = HEAP32[$pTerm>>2]|0;
    $66 = ((($65)) + 32|0);
    $67 = $66;
    $68 = $67;
    HEAP32[$68>>2] = $63;
    $69 = (($67) + 4)|0;
    $70 = $69;
    HEAP32[$70>>2] = $64;
    break;
   }
  } else {
   $71 = HEAP32[$op>>2]|0;
   $72 = ($71|0)==(34);
   if ($72) {
    $73 = HEAP32[$pTerm>>2]|0;
    $74 = ((($73)) + 32|0);
    $75 = $74;
    $76 = $75;
    HEAP32[$76>>2] = 0;
    $77 = (($75) + 4)|0;
    $78 = $77;
    HEAP32[$78>>2] = 0;
    break;
   } else {
    $79 = HEAP32[$pMaskSet>>2]|0;
    $80 = HEAP32[$pExpr>>2]|0;
    $81 = ((($80)) + 16|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = (_sqlite3WhereExprUsage($79,$82)|0);
    $84 = tempRet0;
    $85 = HEAP32[$pTerm>>2]|0;
    $86 = ((($85)) + 32|0);
    $87 = $86;
    $88 = $87;
    HEAP32[$88>>2] = $83;
    $89 = (($87) + 4)|0;
    $90 = $89;
    HEAP32[$90>>2] = $84;
    break;
   }
  }
 } while(0);
 $91 = HEAP32[$pMaskSet>>2]|0;
 $92 = HEAP32[$pExpr>>2]|0;
 $93 = (_sqlite3WhereExprUsage($91,$92)|0);
 $94 = tempRet0;
 $95 = $prereqAll;
 $96 = $95;
 HEAP32[$96>>2] = $93;
 $97 = (($95) + 4)|0;
 $98 = $97;
 HEAP32[$98>>2] = $94;
 $99 = HEAP32[$pExpr>>2]|0;
 $100 = ((($99)) + 4|0);
 $101 = HEAP32[$100>>2]|0;
 $102 = $101 & 1;
 $103 = ($102|0)!=(0);
 if ($103) {
  $104 = HEAP32[$pMaskSet>>2]|0;
  $105 = HEAP32[$pExpr>>2]|0;
  $106 = ((($105)) + 36|0);
  $107 = HEAP16[$106>>1]|0;
  $108 = $107 << 16 >> 16;
  $109 = (_sqlite3WhereGetMask($104,$108)|0);
  $110 = tempRet0;
  $111 = $x;
  $112 = $111;
  HEAP32[$112>>2] = $109;
  $113 = (($111) + 4)|0;
  $114 = $113;
  HEAP32[$114>>2] = $110;
  $115 = $x;
  $116 = $115;
  $117 = HEAP32[$116>>2]|0;
  $118 = (($115) + 4)|0;
  $119 = $118;
  $120 = HEAP32[$119>>2]|0;
  $121 = $prereqAll;
  $122 = $121;
  $123 = HEAP32[$122>>2]|0;
  $124 = (($121) + 4)|0;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = $123 | $117;
  $128 = $126 | $120;
  $129 = $prereqAll;
  $130 = $129;
  HEAP32[$130>>2] = $127;
  $131 = (($129) + 4)|0;
  $132 = $131;
  HEAP32[$132>>2] = $128;
  $133 = $x;
  $134 = $133;
  $135 = HEAP32[$134>>2]|0;
  $136 = (($133) + 4)|0;
  $137 = $136;
  $138 = HEAP32[$137>>2]|0;
  $139 = (_i64Subtract(($135|0),($138|0),1,0)|0);
  $140 = tempRet0;
  $141 = $extraRight;
  $142 = $141;
  HEAP32[$142>>2] = $139;
  $143 = (($141) + 4)|0;
  $144 = $143;
  HEAP32[$144>>2] = $140;
 }
 $145 = $prereqAll;
 $146 = $145;
 $147 = HEAP32[$146>>2]|0;
 $148 = (($145) + 4)|0;
 $149 = $148;
 $150 = HEAP32[$149>>2]|0;
 $151 = HEAP32[$pTerm>>2]|0;
 $152 = ((($151)) + 40|0);
 $153 = $152;
 $154 = $153;
 HEAP32[$154>>2] = $147;
 $155 = (($153) + 4)|0;
 $156 = $155;
 HEAP32[$156>>2] = $150;
 $157 = HEAP32[$pTerm>>2]|0;
 $158 = ((($157)) + 20|0);
 HEAP32[$158>>2] = -1;
 $159 = HEAP32[$pTerm>>2]|0;
 $160 = ((($159)) + 16|0);
 HEAP32[$160>>2] = -1;
 $161 = HEAP32[$pTerm>>2]|0;
 $162 = ((($161)) + 12|0);
 HEAP16[$162>>1] = 0;
 $163 = HEAP32[$op>>2]|0;
 $164 = (_allowedOp($163)|0);
 $165 = ($164|0)!=(0);
 $166 = HEAP32[$pExpr>>2]|0;
 L20: do {
  if ($165) {
   $167 = ((($166)) + 12|0);
   $168 = HEAP32[$167>>2]|0;
   $169 = (_sqlite3ExprSkipCollate($168)|0);
   HEAP32[$pLeft>>2] = $169;
   $170 = HEAP32[$pExpr>>2]|0;
   $171 = ((($170)) + 16|0);
   $172 = HEAP32[$171>>2]|0;
   $173 = (_sqlite3ExprSkipCollate($172)|0);
   HEAP32[$pRight>>2] = $173;
   $174 = HEAP32[$pTerm>>2]|0;
   $175 = ((($174)) + 32|0);
   $176 = $175;
   $177 = $176;
   $178 = HEAP32[$177>>2]|0;
   $179 = (($176) + 4)|0;
   $180 = $179;
   $181 = HEAP32[$180>>2]|0;
   $182 = $prereqLeft;
   $183 = $182;
   $184 = HEAP32[$183>>2]|0;
   $185 = (($182) + 4)|0;
   $186 = $185;
   $187 = HEAP32[$186>>2]|0;
   $188 = $178 & $184;
   $189 = $181 & $187;
   $190 = ($188|0)==(0);
   $191 = ($189|0)==(0);
   $192 = $190 & $191;
   $193 = $192 ? 8191 : 2048;
   $194 = $193&65535;
   HEAP16[$opMask>>1] = $194;
   $195 = HEAP32[$pTerm>>2]|0;
   $196 = ((($195)) + 24|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = ($197|0)>(0);
   if ($198) {
    $199 = HEAP32[$pTerm>>2]|0;
    $200 = ((($199)) + 24|0);
    $201 = HEAP32[$200>>2]|0;
    $202 = (($201) - 1)|0;
    $203 = HEAP32[$pLeft>>2]|0;
    $204 = ((($203)) + 20|0);
    $205 = HEAP32[$204>>2]|0;
    $206 = ((($205)) + 4|0);
    $207 = HEAP32[$206>>2]|0;
    $208 = (($207) + (($202*20)|0)|0);
    $209 = HEAP32[$208>>2]|0;
    HEAP32[$pLeft>>2] = $209;
   }
   $210 = HEAP32[$0>>2]|0;
   $211 = HEAP32[$op>>2]|0;
   $212 = $prereqLeft;
   $213 = $212;
   $214 = HEAP32[$213>>2]|0;
   $215 = (($212) + 4)|0;
   $216 = $215;
   $217 = HEAP32[$216>>2]|0;
   $218 = HEAP32[$pLeft>>2]|0;
   $219 = (_exprMightBeIndexed($210,$211,$214,$217,$218,$iCur,$iColumn)|0);
   $220 = ($219|0)!=(0);
   if ($220) {
    $221 = HEAP32[$iCur>>2]|0;
    $222 = HEAP32[$pTerm>>2]|0;
    $223 = ((($222)) + 20|0);
    HEAP32[$223>>2] = $221;
    $224 = HEAP32[$iColumn>>2]|0;
    $225 = HEAP32[$pTerm>>2]|0;
    $226 = ((($225)) + 28|0);
    HEAP32[$226>>2] = $224;
    $227 = HEAP32[$op>>2]|0;
    $228 = (_operatorMask($227)|0);
    $229 = $228&65535;
    $230 = HEAP16[$opMask>>1]|0;
    $231 = $230&65535;
    $232 = $229 & $231;
    $233 = $232&65535;
    $234 = HEAP32[$pTerm>>2]|0;
    $235 = ((($234)) + 12|0);
    HEAP16[$235>>1] = $233;
   }
   $236 = HEAP32[$op>>2]|0;
   $237 = ($236|0)==(29);
   if ($237) {
    $238 = HEAP32[$pTerm>>2]|0;
    $239 = ((($238)) + 10|0);
    $240 = HEAP16[$239>>1]|0;
    $241 = $240&65535;
    $242 = $241 | 2048;
    $243 = $242&65535;
    HEAP16[$239>>1] = $243;
   }
   $244 = HEAP32[$pRight>>2]|0;
   $245 = ($244|0)!=(0|0);
   if ($245) {
    $246 = HEAP32[$0>>2]|0;
    $247 = HEAP32[$op>>2]|0;
    $248 = HEAP32[$pTerm>>2]|0;
    $249 = ((($248)) + 32|0);
    $250 = $249;
    $251 = $250;
    $252 = HEAP32[$251>>2]|0;
    $253 = (($250) + 4)|0;
    $254 = $253;
    $255 = HEAP32[$254>>2]|0;
    $256 = HEAP32[$pRight>>2]|0;
    $257 = (_exprMightBeIndexed($246,$247,$252,$255,$256,$iCur,$iColumn)|0);
    $258 = ($257|0)!=(0);
    if ($258) {
     HEAP16[$eExtraOp>>1] = 0;
     $259 = HEAP32[$pTerm>>2]|0;
     $260 = ((($259)) + 20|0);
     $261 = HEAP32[$260>>2]|0;
     $262 = ($261|0)>=(0);
     if ($262) {
      $263 = HEAP32[$db>>2]|0;
      $264 = HEAP32[$pExpr>>2]|0;
      $265 = (_sqlite3ExprDup($263,$264,0)|0);
      HEAP32[$pDup>>2] = $265;
      $266 = HEAP32[$db>>2]|0;
      $267 = ((($266)) + 69|0);
      $268 = HEAP8[$267>>0]|0;
      $269 = ($268<<24>>24)!=(0);
      if ($269) {
       $270 = HEAP32[$db>>2]|0;
       $271 = HEAP32[$pDup>>2]|0;
       _sqlite3ExprDelete($270,$271);
       STACKTOP = sp;return;
      }
      $272 = HEAP32[$1>>2]|0;
      $273 = HEAP32[$pDup>>2]|0;
      $274 = (_whereClauseInsert($272,$273,3)|0);
      HEAP32[$idxNew>>2] = $274;
      $275 = HEAP32[$idxNew>>2]|0;
      $276 = ($275|0)==(0);
      if ($276) {
       STACKTOP = sp;return;
      }
      $277 = HEAP32[$idxNew>>2]|0;
      $278 = HEAP32[$1>>2]|0;
      $279 = ((($278)) + 20|0);
      $280 = HEAP32[$279>>2]|0;
      $281 = (($280) + (($277*48)|0)|0);
      HEAP32[$pNew>>2] = $281;
      $282 = HEAP32[$1>>2]|0;
      $283 = HEAP32[$idxNew>>2]|0;
      $284 = HEAP32[$2>>2]|0;
      _markTermAsChild($282,$283,$284);
      $285 = HEAP32[$op>>2]|0;
      $286 = ($285|0)==(29);
      if ($286) {
       $287 = HEAP32[$pNew>>2]|0;
       $288 = ((($287)) + 10|0);
       $289 = HEAP16[$288>>1]|0;
       $290 = $289&65535;
       $291 = $290 | 2048;
       $292 = $291&65535;
       HEAP16[$288>>1] = $292;
      }
      $293 = HEAP32[$2>>2]|0;
      $294 = HEAP32[$1>>2]|0;
      $295 = ((($294)) + 20|0);
      $296 = HEAP32[$295>>2]|0;
      $297 = (($296) + (($293*48)|0)|0);
      HEAP32[$pTerm>>2] = $297;
      $298 = HEAP32[$pTerm>>2]|0;
      $299 = ((($298)) + 10|0);
      $300 = HEAP16[$299>>1]|0;
      $301 = $300&65535;
      $302 = $301 | 8;
      $303 = $302&65535;
      HEAP16[$299>>1] = $303;
      $304 = HEAP32[$pParse>>2]|0;
      $305 = HEAP32[$pDup>>2]|0;
      $306 = (_termIsEquivalence($304,$305)|0);
      $307 = ($306|0)!=(0);
      if ($307) {
       $308 = HEAP32[$pTerm>>2]|0;
       $309 = ((($308)) + 12|0);
       $310 = HEAP16[$309>>1]|0;
       $311 = $310&65535;
       $312 = $311 | 2048;
       $313 = $312&65535;
       HEAP16[$309>>1] = $313;
       HEAP16[$eExtraOp>>1] = 2048;
      }
     } else {
      $314 = HEAP32[$pExpr>>2]|0;
      HEAP32[$pDup>>2] = $314;
      $315 = HEAP32[$pTerm>>2]|0;
      HEAP32[$pNew>>2] = $315;
     }
     $316 = HEAP32[$pParse>>2]|0;
     $317 = HEAP32[$pDup>>2]|0;
     _exprCommute($316,$317);
     $318 = HEAP32[$iCur>>2]|0;
     $319 = HEAP32[$pNew>>2]|0;
     $320 = ((($319)) + 20|0);
     HEAP32[$320>>2] = $318;
     $321 = HEAP32[$iColumn>>2]|0;
     $322 = HEAP32[$pNew>>2]|0;
     $323 = ((($322)) + 28|0);
     HEAP32[$323>>2] = $321;
     $324 = $prereqLeft;
     $325 = $324;
     $326 = HEAP32[$325>>2]|0;
     $327 = (($324) + 4)|0;
     $328 = $327;
     $329 = HEAP32[$328>>2]|0;
     $330 = $extraRight;
     $331 = $330;
     $332 = HEAP32[$331>>2]|0;
     $333 = (($330) + 4)|0;
     $334 = $333;
     $335 = HEAP32[$334>>2]|0;
     $336 = $326 | $332;
     $337 = $329 | $335;
     $338 = HEAP32[$pNew>>2]|0;
     $339 = ((($338)) + 32|0);
     $340 = $339;
     $341 = $340;
     HEAP32[$341>>2] = $336;
     $342 = (($340) + 4)|0;
     $343 = $342;
     HEAP32[$343>>2] = $337;
     $344 = $prereqAll;
     $345 = $344;
     $346 = HEAP32[$345>>2]|0;
     $347 = (($344) + 4)|0;
     $348 = $347;
     $349 = HEAP32[$348>>2]|0;
     $350 = HEAP32[$pNew>>2]|0;
     $351 = ((($350)) + 40|0);
     $352 = $351;
     $353 = $352;
     HEAP32[$353>>2] = $346;
     $354 = (($352) + 4)|0;
     $355 = $354;
     HEAP32[$355>>2] = $349;
     $356 = HEAP32[$pDup>>2]|0;
     $357 = HEAP8[$356>>0]|0;
     $358 = $357&255;
     $359 = (_operatorMask($358)|0);
     $360 = $359&65535;
     $361 = HEAP16[$eExtraOp>>1]|0;
     $362 = $361&65535;
     $363 = (($360) + ($362))|0;
     $364 = HEAP16[$opMask>>1]|0;
     $365 = $364&65535;
     $366 = $363 & $365;
     $367 = $366&65535;
     $368 = HEAP32[$pNew>>2]|0;
     $369 = ((($368)) + 12|0);
     HEAP16[$369>>1] = $367;
    }
   }
  } else {
   $370 = HEAP8[$166>>0]|0;
   $371 = $370&255;
   $372 = ($371|0)==(32);
   if ($372) {
    $373 = HEAP32[$1>>2]|0;
    $374 = ((($373)) + 8|0);
    $375 = HEAP8[$374>>0]|0;
    $376 = $375&255;
    $377 = ($376|0)==(28);
    if ($377) {
     $378 = HEAP32[$pExpr>>2]|0;
     $379 = ((($378)) + 20|0);
     $380 = HEAP32[$379>>2]|0;
     HEAP32[$pList>>2] = $380;
     HEAP32[$i>>2] = 0;
     while(1) {
      $381 = HEAP32[$i>>2]|0;
      $382 = ($381|0)<(2);
      if (!($382)) {
       break L20;
      }
      $383 = HEAP32[$pParse>>2]|0;
      $384 = HEAP32[$i>>2]|0;
      $385 = (34832 + ($384)|0);
      $386 = HEAP8[$385>>0]|0;
      $387 = $386&255;
      $388 = HEAP32[$db>>2]|0;
      $389 = HEAP32[$pExpr>>2]|0;
      $390 = ((($389)) + 12|0);
      $391 = HEAP32[$390>>2]|0;
      $392 = (_sqlite3ExprDup($388,$391,0)|0);
      $393 = HEAP32[$db>>2]|0;
      $394 = HEAP32[$i>>2]|0;
      $395 = HEAP32[$pList>>2]|0;
      $396 = ((($395)) + 4|0);
      $397 = HEAP32[$396>>2]|0;
      $398 = (($397) + (($394*20)|0)|0);
      $399 = HEAP32[$398>>2]|0;
      $400 = (_sqlite3ExprDup($393,$399,0)|0);
      $401 = (_sqlite3PExpr($383,$387,$392,$400,0)|0);
      HEAP32[$pNewExpr>>2] = $401;
      $402 = HEAP32[$pNewExpr>>2]|0;
      $403 = HEAP32[$pExpr>>2]|0;
      _transferJoinMarkings($402,$403);
      $404 = HEAP32[$1>>2]|0;
      $405 = HEAP32[$pNewExpr>>2]|0;
      $406 = (_whereClauseInsert($404,$405,3)|0);
      HEAP32[$idxNew1>>2] = $406;
      $407 = HEAP32[$0>>2]|0;
      $408 = HEAP32[$1>>2]|0;
      $409 = HEAP32[$idxNew1>>2]|0;
      _exprAnalyze($407,$408,$409);
      $410 = HEAP32[$2>>2]|0;
      $411 = HEAP32[$1>>2]|0;
      $412 = ((($411)) + 20|0);
      $413 = HEAP32[$412>>2]|0;
      $414 = (($413) + (($410*48)|0)|0);
      HEAP32[$pTerm>>2] = $414;
      $415 = HEAP32[$1>>2]|0;
      $416 = HEAP32[$idxNew1>>2]|0;
      $417 = HEAP32[$2>>2]|0;
      _markTermAsChild($415,$416,$417);
      $418 = HEAP32[$i>>2]|0;
      $419 = (($418) + 1)|0;
      HEAP32[$i>>2] = $419;
     }
    }
   }
   $420 = HEAP32[$pExpr>>2]|0;
   $421 = HEAP8[$420>>0]|0;
   $422 = $421&255;
   $423 = ($422|0)==(27);
   if ($423) {
    $424 = HEAP32[$0>>2]|0;
    $425 = HEAP32[$1>>2]|0;
    $426 = HEAP32[$2>>2]|0;
    _exprAnalyzeOrTerm($424,$425,$426);
    $427 = HEAP32[$2>>2]|0;
    $428 = HEAP32[$1>>2]|0;
    $429 = ((($428)) + 20|0);
    $430 = HEAP32[$429>>2]|0;
    $431 = (($430) + (($427*48)|0)|0);
    HEAP32[$pTerm>>2] = $431;
   }
  }
 } while(0);
 $432 = HEAP32[$1>>2]|0;
 $433 = ((($432)) + 8|0);
 $434 = HEAP8[$433>>0]|0;
 $435 = $434&255;
 $436 = ($435|0)==(28);
 if ($436) {
  $437 = HEAP32[$pParse>>2]|0;
  $438 = HEAP32[$pExpr>>2]|0;
  $439 = (_isLikeOrGlob($437,$438,$pStr1,$isComplete,$noCase)|0);
  $440 = ($439|0)!=(0);
  if ($440) {
   HEAP16[$wtFlags>>1] = 259;
   $441 = HEAP32[$pExpr>>2]|0;
   $442 = ((($441)) + 20|0);
   $443 = HEAP32[$442>>2]|0;
   $444 = ((($443)) + 4|0);
   $445 = HEAP32[$444>>2]|0;
   $446 = ((($445)) + 20|0);
   $447 = HEAP32[$446>>2]|0;
   HEAP32[$pLeft2>>2] = $447;
   $448 = HEAP32[$db>>2]|0;
   $449 = HEAP32[$pStr1>>2]|0;
   $450 = (_sqlite3ExprDup($448,$449,0)|0);
   HEAP32[$pStr2>>2] = $450;
   $451 = HEAP32[$noCase>>2]|0;
   $452 = ($451|0)!=(0);
   L61: do {
    if ($452) {
     $453 = HEAP32[$pParse>>2]|0;
     $454 = HEAP32[$453>>2]|0;
     $455 = ((($454)) + 69|0);
     $456 = HEAP8[$455>>0]|0;
     $457 = ($456<<24>>24)!=(0);
     if (!($457)) {
      $458 = HEAP32[$pTerm>>2]|0;
      $459 = ((($458)) + 10|0);
      $460 = HEAP16[$459>>1]|0;
      $461 = $460&65535;
      $462 = $461 | 1024;
      $463 = $462&65535;
      HEAP16[$459>>1] = $463;
      HEAP32[$i4>>2] = 0;
      while(1) {
       $464 = HEAP32[$i4>>2]|0;
       $465 = HEAP32[$pStr1>>2]|0;
       $466 = ((($465)) + 8|0);
       $467 = HEAP32[$466>>2]|0;
       $468 = (($467) + ($464)|0);
       $469 = HEAP8[$468>>0]|0;
       HEAP8[$c>>0] = $469;
       $470 = $469 << 24 >> 24;
       $471 = ($470|0)!=(0);
       if (!($471)) {
        break L61;
       }
       $472 = HEAP8[$c>>0]|0;
       $473 = $472 << 24 >> 24;
       $474 = HEAP8[$c>>0]|0;
       $475 = $474&255;
       $476 = (18403 + ($475)|0);
       $477 = HEAP8[$476>>0]|0;
       $478 = $477&255;
       $479 = $478 & 32;
       $480 = $479 ^ -1;
       $481 = $473 & $480;
       $482 = $481&255;
       $483 = HEAP32[$i4>>2]|0;
       $484 = HEAP32[$pStr1>>2]|0;
       $485 = ((($484)) + 8|0);
       $486 = HEAP32[$485>>2]|0;
       $487 = (($486) + ($483)|0);
       HEAP8[$487>>0] = $482;
       $488 = HEAP8[$c>>0]|0;
       $489 = $488&255;
       $490 = (17613 + ($489)|0);
       $491 = HEAP8[$490>>0]|0;
       $492 = HEAP32[$i4>>2]|0;
       $493 = HEAP32[$pStr2>>2]|0;
       $494 = ((($493)) + 8|0);
       $495 = HEAP32[$494>>2]|0;
       $496 = (($495) + ($492)|0);
       HEAP8[$496>>0] = $491;
       $497 = HEAP32[$i4>>2]|0;
       $498 = (($497) + 1)|0;
       HEAP32[$i4>>2] = $498;
      }
     }
    }
   } while(0);
   $499 = HEAP32[$db>>2]|0;
   $500 = ((($499)) + 69|0);
   $501 = HEAP8[$500>>0]|0;
   $502 = ($501<<24>>24)!=(0);
   if (!($502)) {
    $503 = HEAP32[$pStr2>>2]|0;
    $504 = ((($503)) + 8|0);
    $505 = HEAP32[$504>>2]|0;
    $506 = (_sqlite3Strlen30($505)|0);
    $507 = (($506) - 1)|0;
    $508 = HEAP32[$pStr2>>2]|0;
    $509 = ((($508)) + 8|0);
    $510 = HEAP32[$509>>2]|0;
    $511 = (($510) + ($507)|0);
    HEAP32[$pC>>2] = $511;
    $512 = HEAP32[$pC>>2]|0;
    $513 = HEAP8[$512>>0]|0;
    HEAP8[$c5>>0] = $513;
    $514 = HEAP32[$noCase>>2]|0;
    $515 = ($514|0)!=(0);
    if ($515) {
     $516 = HEAP8[$c5>>0]|0;
     $517 = $516&255;
     $518 = ($517|0)==(64);
     if ($518) {
      HEAP32[$isComplete>>2] = 0;
     }
     $519 = HEAP8[$c5>>0]|0;
     $520 = $519&255;
     $521 = (17613 + ($520)|0);
     $522 = HEAP8[$521>>0]|0;
     HEAP8[$c5>>0] = $522;
    }
    $523 = HEAP8[$c5>>0]|0;
    $524 = $523&255;
    $525 = (($524) + 1)|0;
    $526 = $525&255;
    $527 = HEAP32[$pC>>2]|0;
    HEAP8[$527>>0] = $526;
   }
   $528 = HEAP32[$noCase>>2]|0;
   $529 = ($528|0)!=(0);
   $530 = $529 ? 20429 : 34834;
   HEAP32[$zCollSeqName>>2] = $530;
   $531 = HEAP32[$db>>2]|0;
   $532 = HEAP32[$pLeft2>>2]|0;
   $533 = (_sqlite3ExprDup($531,$532,0)|0);
   HEAP32[$pNewExpr1>>2] = $533;
   $534 = HEAP32[$pParse>>2]|0;
   $535 = HEAP32[$pParse>>2]|0;
   $536 = HEAP32[$pNewExpr1>>2]|0;
   $537 = HEAP32[$zCollSeqName>>2]|0;
   $538 = (_sqlite3ExprAddCollateString($535,$536,$537)|0);
   $539 = HEAP32[$pStr1>>2]|0;
   $540 = (_sqlite3PExpr($534,41,$538,$539,0)|0);
   HEAP32[$pNewExpr1>>2] = $540;
   $541 = HEAP32[$pNewExpr1>>2]|0;
   $542 = HEAP32[$pExpr>>2]|0;
   _transferJoinMarkings($541,$542);
   $543 = HEAP32[$1>>2]|0;
   $544 = HEAP32[$pNewExpr1>>2]|0;
   $545 = (_whereClauseInsert($543,$544,259)|0);
   HEAP32[$idxNew13>>2] = $545;
   $546 = HEAP32[$0>>2]|0;
   $547 = HEAP32[$1>>2]|0;
   $548 = HEAP32[$idxNew13>>2]|0;
   _exprAnalyze($546,$547,$548);
   $549 = HEAP32[$db>>2]|0;
   $550 = HEAP32[$pLeft2>>2]|0;
   $551 = (_sqlite3ExprDup($549,$550,0)|0);
   HEAP32[$pNewExpr2>>2] = $551;
   $552 = HEAP32[$pParse>>2]|0;
   $553 = HEAP32[$pParse>>2]|0;
   $554 = HEAP32[$pNewExpr2>>2]|0;
   $555 = HEAP32[$zCollSeqName>>2]|0;
   $556 = (_sqlite3ExprAddCollateString($553,$554,$555)|0);
   $557 = HEAP32[$pStr2>>2]|0;
   $558 = (_sqlite3PExpr($552,40,$556,$557,0)|0);
   HEAP32[$pNewExpr2>>2] = $558;
   $559 = HEAP32[$pNewExpr2>>2]|0;
   $560 = HEAP32[$pExpr>>2]|0;
   _transferJoinMarkings($559,$560);
   $561 = HEAP32[$1>>2]|0;
   $562 = HEAP32[$pNewExpr2>>2]|0;
   $563 = (_whereClauseInsert($561,$562,259)|0);
   HEAP32[$idxNew2>>2] = $563;
   $564 = HEAP32[$0>>2]|0;
   $565 = HEAP32[$1>>2]|0;
   $566 = HEAP32[$idxNew2>>2]|0;
   _exprAnalyze($564,$565,$566);
   $567 = HEAP32[$2>>2]|0;
   $568 = HEAP32[$1>>2]|0;
   $569 = ((($568)) + 20|0);
   $570 = HEAP32[$569>>2]|0;
   $571 = (($570) + (($567*48)|0)|0);
   HEAP32[$pTerm>>2] = $571;
   $572 = HEAP32[$isComplete>>2]|0;
   $573 = ($572|0)!=(0);
   if ($573) {
    $574 = HEAP32[$1>>2]|0;
    $575 = HEAP32[$idxNew13>>2]|0;
    $576 = HEAP32[$2>>2]|0;
    _markTermAsChild($574,$575,$576);
    $577 = HEAP32[$1>>2]|0;
    $578 = HEAP32[$idxNew2>>2]|0;
    $579 = HEAP32[$2>>2]|0;
    _markTermAsChild($577,$578,$579);
   }
  }
 }
 $580 = HEAP32[$1>>2]|0;
 $581 = ((($580)) + 8|0);
 $582 = HEAP8[$581>>0]|0;
 $583 = $582&255;
 $584 = ($583|0)==(28);
 if ($584) {
  $585 = HEAP32[$pExpr>>2]|0;
  $586 = (_isMatchOfColumn($585,$eOp2)|0);
  $587 = ($586|0)!=(0);
  if ($587) {
   $588 = HEAP32[$pExpr>>2]|0;
   $589 = ((($588)) + 20|0);
   $590 = HEAP32[$589>>2]|0;
   $591 = ((($590)) + 4|0);
   $592 = HEAP32[$591>>2]|0;
   $593 = HEAP32[$592>>2]|0;
   HEAP32[$pRight7>>2] = $593;
   $594 = HEAP32[$pExpr>>2]|0;
   $595 = ((($594)) + 20|0);
   $596 = HEAP32[$595>>2]|0;
   $597 = ((($596)) + 4|0);
   $598 = HEAP32[$597>>2]|0;
   $599 = ((($598)) + 20|0);
   $600 = HEAP32[$599>>2]|0;
   HEAP32[$pLeft8>>2] = $600;
   $601 = HEAP32[$pMaskSet>>2]|0;
   $602 = HEAP32[$pRight7>>2]|0;
   $603 = (_sqlite3WhereExprUsage($601,$602)|0);
   $604 = tempRet0;
   $605 = $prereqExpr;
   $606 = $605;
   HEAP32[$606>>2] = $603;
   $607 = (($605) + 4)|0;
   $608 = $607;
   HEAP32[$608>>2] = $604;
   $609 = HEAP32[$pMaskSet>>2]|0;
   $610 = HEAP32[$pLeft8>>2]|0;
   $611 = (_sqlite3WhereExprUsage($609,$610)|0);
   $612 = tempRet0;
   $613 = $prereqColumn;
   $614 = $613;
   HEAP32[$614>>2] = $611;
   $615 = (($613) + 4)|0;
   $616 = $615;
   HEAP32[$616>>2] = $612;
   $617 = $prereqExpr;
   $618 = $617;
   $619 = HEAP32[$618>>2]|0;
   $620 = (($617) + 4)|0;
   $621 = $620;
   $622 = HEAP32[$621>>2]|0;
   $623 = $prereqColumn;
   $624 = $623;
   $625 = HEAP32[$624>>2]|0;
   $626 = (($623) + 4)|0;
   $627 = $626;
   $628 = HEAP32[$627>>2]|0;
   $629 = $619 & $625;
   $630 = $622 & $628;
   $631 = ($629|0)==(0);
   $632 = ($630|0)==(0);
   $633 = $631 & $632;
   if ($633) {
    $634 = HEAP32[$pParse>>2]|0;
    $635 = HEAP32[$db>>2]|0;
    $636 = HEAP32[$pRight7>>2]|0;
    $637 = (_sqlite3ExprDup($635,$636,0)|0);
    $638 = (_sqlite3PExpr($634,30,0,$637,0)|0);
    HEAP32[$pNewExpr9>>2] = $638;
    $639 = HEAP32[$1>>2]|0;
    $640 = HEAP32[$pNewExpr9>>2]|0;
    $641 = (_whereClauseInsert($639,$640,3)|0);
    HEAP32[$idxNew6>>2] = $641;
    $642 = HEAP32[$idxNew6>>2]|0;
    $643 = HEAP32[$1>>2]|0;
    $644 = ((($643)) + 20|0);
    $645 = HEAP32[$644>>2]|0;
    $646 = (($645) + (($642*48)|0)|0);
    HEAP32[$pNewTerm>>2] = $646;
    $647 = $prereqExpr;
    $648 = $647;
    $649 = HEAP32[$648>>2]|0;
    $650 = (($647) + 4)|0;
    $651 = $650;
    $652 = HEAP32[$651>>2]|0;
    $653 = HEAP32[$pNewTerm>>2]|0;
    $654 = ((($653)) + 32|0);
    $655 = $654;
    $656 = $655;
    HEAP32[$656>>2] = $649;
    $657 = (($655) + 4)|0;
    $658 = $657;
    HEAP32[$658>>2] = $652;
    $659 = HEAP32[$pLeft8>>2]|0;
    $660 = ((($659)) + 28|0);
    $661 = HEAP32[$660>>2]|0;
    $662 = HEAP32[$pNewTerm>>2]|0;
    $663 = ((($662)) + 20|0);
    HEAP32[$663>>2] = $661;
    $664 = HEAP32[$pLeft8>>2]|0;
    $665 = ((($664)) + 32|0);
    $666 = HEAP16[$665>>1]|0;
    $667 = $666 << 16 >> 16;
    $668 = HEAP32[$pNewTerm>>2]|0;
    $669 = ((($668)) + 28|0);
    HEAP32[$669>>2] = $667;
    $670 = HEAP32[$pNewTerm>>2]|0;
    $671 = ((($670)) + 12|0);
    HEAP16[$671>>1] = 64;
    $672 = HEAP8[$eOp2>>0]|0;
    $673 = HEAP32[$pNewTerm>>2]|0;
    $674 = ((($673)) + 15|0);
    HEAP8[$674>>0] = $672;
    $675 = HEAP32[$1>>2]|0;
    $676 = HEAP32[$idxNew6>>2]|0;
    $677 = HEAP32[$2>>2]|0;
    _markTermAsChild($675,$676,$677);
    $678 = HEAP32[$2>>2]|0;
    $679 = HEAP32[$1>>2]|0;
    $680 = ((($679)) + 20|0);
    $681 = HEAP32[$680>>2]|0;
    $682 = (($681) + (($678*48)|0)|0);
    HEAP32[$pTerm>>2] = $682;
    $683 = HEAP32[$pTerm>>2]|0;
    $684 = ((($683)) + 10|0);
    $685 = HEAP16[$684>>1]|0;
    $686 = $685&65535;
    $687 = $686 | 8;
    $688 = $687&65535;
    HEAP16[$684>>1] = $688;
    $689 = HEAP32[$pTerm>>2]|0;
    $690 = ((($689)) + 40|0);
    $691 = $690;
    $692 = $691;
    $693 = HEAP32[$692>>2]|0;
    $694 = (($691) + 4)|0;
    $695 = $694;
    $696 = HEAP32[$695>>2]|0;
    $697 = HEAP32[$pNewTerm>>2]|0;
    $698 = ((($697)) + 40|0);
    $699 = $698;
    $700 = $699;
    HEAP32[$700>>2] = $693;
    $701 = (($699) + 4)|0;
    $702 = $701;
    HEAP32[$702>>2] = $696;
   }
  }
 }
 $703 = HEAP32[$1>>2]|0;
 $704 = ((($703)) + 8|0);
 $705 = HEAP8[$704>>0]|0;
 $706 = $705&255;
 $707 = ($706|0)==(28);
 do {
  if ($707) {
   $708 = HEAP32[$pExpr>>2]|0;
   $709 = HEAP8[$708>>0]|0;
   $710 = $709&255;
   $711 = ($710|0)==(37);
   if (!($711)) {
    $712 = HEAP32[$pExpr>>2]|0;
    $713 = HEAP8[$712>>0]|0;
    $714 = $713&255;
    $715 = ($714|0)==(29);
    if (!($715)) {
     break;
    }
   }
   $716 = HEAP32[$pExpr>>2]|0;
   $717 = ((($716)) + 12|0);
   $718 = HEAP32[$717>>2]|0;
   $719 = (_sqlite3ExprIsVector($718)|0);
   $720 = ($719|0)!=(0);
   if ($720) {
    $721 = HEAP32[$pExpr>>2]|0;
    $722 = ((($721)) + 12|0);
    $723 = HEAP32[$722>>2]|0;
    $724 = ((($723)) + 4|0);
    $725 = HEAP32[$724>>2]|0;
    $726 = $725 & 2048;
    $727 = ($726|0)==(0);
    if (!($727)) {
     $728 = HEAP32[$pExpr>>2]|0;
     $729 = ((($728)) + 16|0);
     $730 = HEAP32[$729>>2]|0;
     $731 = ((($730)) + 4|0);
     $732 = HEAP32[$731>>2]|0;
     $733 = $732 & 2048;
     $734 = ($733|0)==(0);
     if (!($734)) {
      break;
     }
    }
    $735 = HEAP32[$pExpr>>2]|0;
    $736 = ((($735)) + 12|0);
    $737 = HEAP32[$736>>2]|0;
    $738 = (_sqlite3ExprVectorSize($737)|0);
    HEAP32[$nLeft>>2] = $738;
    HEAP32[$i10>>2] = 0;
    while(1) {
     $739 = HEAP32[$i10>>2]|0;
     $740 = HEAP32[$nLeft>>2]|0;
     $741 = ($739|0)<($740|0);
     if (!($741)) {
      break;
     }
     $742 = HEAP32[$pParse>>2]|0;
     $743 = HEAP32[$pExpr>>2]|0;
     $744 = ((($743)) + 12|0);
     $745 = HEAP32[$744>>2]|0;
     $746 = HEAP32[$i10>>2]|0;
     $747 = (_sqlite3ExprForVectorField($742,$745,$746)|0);
     HEAP32[$pLeft13>>2] = $747;
     $748 = HEAP32[$pParse>>2]|0;
     $749 = HEAP32[$pExpr>>2]|0;
     $750 = ((($749)) + 16|0);
     $751 = HEAP32[$750>>2]|0;
     $752 = HEAP32[$i10>>2]|0;
     $753 = (_sqlite3ExprForVectorField($748,$751,$752)|0);
     HEAP32[$pRight14>>2] = $753;
     $754 = HEAP32[$pParse>>2]|0;
     $755 = HEAP32[$pExpr>>2]|0;
     $756 = HEAP8[$755>>0]|0;
     $757 = $756&255;
     $758 = HEAP32[$pLeft13>>2]|0;
     $759 = HEAP32[$pRight14>>2]|0;
     $760 = (_sqlite3PExpr($754,$757,$758,$759,0)|0);
     HEAP32[$pNew12>>2] = $760;
     $761 = HEAP32[$pNew12>>2]|0;
     $762 = HEAP32[$pExpr>>2]|0;
     _transferJoinMarkings($761,$762);
     $763 = HEAP32[$1>>2]|0;
     $764 = HEAP32[$pNew12>>2]|0;
     $765 = (_whereClauseInsert($763,$764,1)|0);
     HEAP32[$idxNew11>>2] = $765;
     $766 = HEAP32[$0>>2]|0;
     $767 = HEAP32[$1>>2]|0;
     $768 = HEAP32[$idxNew11>>2]|0;
     _exprAnalyze($766,$767,$768);
     $769 = HEAP32[$i10>>2]|0;
     $770 = (($769) + 1)|0;
     HEAP32[$i10>>2] = $770;
    }
    $771 = HEAP32[$2>>2]|0;
    $772 = HEAP32[$1>>2]|0;
    $773 = ((($772)) + 20|0);
    $774 = HEAP32[$773>>2]|0;
    $775 = (($774) + (($771*48)|0)|0);
    HEAP32[$pTerm>>2] = $775;
    $776 = HEAP32[$pTerm>>2]|0;
    $777 = ((($776)) + 10|0);
    HEAP16[$777>>1] = 6;
    $778 = HEAP32[$pTerm>>2]|0;
    $779 = ((($778)) + 12|0);
    HEAP16[$779>>1] = 0;
   }
  }
 } while(0);
 $780 = HEAP32[$1>>2]|0;
 $781 = ((($780)) + 8|0);
 $782 = HEAP8[$781>>0]|0;
 $783 = $782&255;
 $784 = ($783|0)==(28);
 L98: do {
  if ($784) {
   $785 = HEAP32[$pExpr>>2]|0;
   $786 = HEAP8[$785>>0]|0;
   $787 = $786&255;
   $788 = ($787|0)==(33);
   if ($788) {
    $789 = HEAP32[$pTerm>>2]|0;
    $790 = ((($789)) + 24|0);
    $791 = HEAP32[$790>>2]|0;
    $792 = ($791|0)==(0);
    if ($792) {
     $793 = HEAP32[$pExpr>>2]|0;
     $794 = ((($793)) + 12|0);
     $795 = HEAP32[$794>>2]|0;
     $796 = HEAP8[$795>>0]|0;
     $797 = $796&255;
     $798 = ($797|0)==(158);
     if ($798) {
      $799 = HEAP32[$pExpr>>2]|0;
      $800 = ((($799)) + 20|0);
      $801 = HEAP32[$800>>2]|0;
      $802 = ((($801)) + 48|0);
      $803 = HEAP32[$802>>2]|0;
      $804 = ($803|0)==(0|0);
      if ($804) {
       HEAP32[$i15>>2] = 0;
       while(1) {
        $805 = HEAP32[$i15>>2]|0;
        $806 = HEAP32[$pExpr>>2]|0;
        $807 = ((($806)) + 12|0);
        $808 = HEAP32[$807>>2]|0;
        $809 = (_sqlite3ExprVectorSize($808)|0);
        $810 = ($805|0)<($809|0);
        if (!($810)) {
         break L98;
        }
        $811 = HEAP32[$1>>2]|0;
        $812 = HEAP32[$pExpr>>2]|0;
        $813 = (_whereClauseInsert($811,$812,2)|0);
        HEAP32[$idxNew16>>2] = $813;
        $814 = HEAP32[$i15>>2]|0;
        $815 = (($814) + 1)|0;
        $816 = HEAP32[$idxNew16>>2]|0;
        $817 = HEAP32[$1>>2]|0;
        $818 = ((($817)) + 20|0);
        $819 = HEAP32[$818>>2]|0;
        $820 = (($819) + (($816*48)|0)|0);
        $821 = ((($820)) + 24|0);
        HEAP32[$821>>2] = $815;
        $822 = HEAP32[$0>>2]|0;
        $823 = HEAP32[$1>>2]|0;
        $824 = HEAP32[$idxNew16>>2]|0;
        _exprAnalyze($822,$823,$824);
        $825 = HEAP32[$1>>2]|0;
        $826 = HEAP32[$idxNew16>>2]|0;
        $827 = HEAP32[$2>>2]|0;
        _markTermAsChild($825,$826,$827);
        $828 = HEAP32[$i15>>2]|0;
        $829 = (($828) + 1)|0;
        HEAP32[$i15>>2] = $829;
       }
      }
     }
    }
   }
  }
 } while(0);
 $830 = $extraRight;
 $831 = $830;
 $832 = HEAP32[$831>>2]|0;
 $833 = (($830) + 4)|0;
 $834 = $833;
 $835 = HEAP32[$834>>2]|0;
 $836 = HEAP32[$pTerm>>2]|0;
 $837 = ((($836)) + 32|0);
 $838 = $837;
 $839 = $838;
 $840 = HEAP32[$839>>2]|0;
 $841 = (($838) + 4)|0;
 $842 = $841;
 $843 = HEAP32[$842>>2]|0;
 $844 = $840 | $832;
 $845 = $843 | $835;
 $846 = $837;
 $847 = $846;
 HEAP32[$847>>2] = $844;
 $848 = (($846) + 4)|0;
 $849 = $848;
 HEAP32[$849>>2] = $845;
 STACKTOP = sp;return;
}
function _allowedOp($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(33);
 if ($2) {
  $12 = 1;
 } else {
  $3 = HEAP32[$0>>2]|0;
  $4 = ($3|0)>=(37);
  $5 = HEAP32[$0>>2]|0;
  $6 = ($5|0)<=(41);
  $or$cond = $4 & $6;
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)==(34);
  $or$cond3 = $or$cond | $8;
  if ($or$cond3) {
   $12 = 1;
  } else {
   $9 = HEAP32[$0>>2]|0;
   $10 = ($9|0)==(29);
   $12 = $10;
  }
 }
 $11 = $12&1;
 STACKTOP = sp;return ($11|0);
}
function _exprMightBeIndexed($pFrom,$op,$0,$1,$pExpr,$piCur,$piColumn) {
 $pFrom = $pFrom|0;
 $op = $op|0;
 $0 = $0|0;
 $1 = $1|0;
 $pExpr = $pExpr|0;
 $piCur = $piCur|0;
 $piColumn = $piColumn|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iCur = 0, $or$cond = 0, $or$cond3 = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp;
 $6 = sp + 28|0;
 $7 = sp + 24|0;
 $8 = sp + 20|0;
 $pIdx = sp + 16|0;
 $i = sp + 12|0;
 $iCur = sp + 8|0;
 HEAP32[$3>>2] = $pFrom;
 HEAP32[$4>>2] = $op;
 $9 = $5;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$6>>2] = $pExpr;
 HEAP32[$7>>2] = $piCur;
 HEAP32[$8>>2] = $piColumn;
 $13 = HEAP32[$6>>2]|0;
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)==(158);
 $17 = HEAP32[$4>>2]|0;
 $18 = ($17|0)>=(38);
 $or$cond = $16 & $18;
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)<=(41);
 $or$cond3 = $or$cond & $20;
 if ($or$cond3) {
  $21 = HEAP32[$6>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$6>>2] = $26;
 }
 $27 = HEAP32[$6>>2]|0;
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)==(152);
 if ($30) {
  $31 = HEAP32[$6>>2]|0;
  $32 = ((($31)) + 28|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = HEAP32[$7>>2]|0;
  HEAP32[$34>>2] = $33;
  $35 = HEAP32[$6>>2]|0;
  $36 = ((($35)) + 32|0);
  $37 = HEAP16[$36>>1]|0;
  $38 = $37 << 16 >> 16;
  $39 = HEAP32[$8>>2]|0;
  HEAP32[$39>>2] = $38;
  HEAP32[$2>>2] = 1;
  $146 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($146|0);
 }
 $40 = $5;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = (($40) + 4)|0;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 $46 = ($42|0)==(0);
 $47 = ($45|0)==(0);
 $48 = $46 & $47;
 if ($48) {
  HEAP32[$2>>2] = 0;
  $146 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($146|0);
 }
 $49 = $5;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = (($49) + 4)|0;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = $5;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = (($55) + 4)|0;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (_i64Subtract(($57|0),($60|0),1,0)|0);
 $62 = tempRet0;
 $63 = $51 & $61;
 $64 = $54 & $62;
 $65 = ($63|0)!=(0);
 $66 = ($64|0)!=(0);
 $67 = $65 | $66;
 if ($67) {
  HEAP32[$2>>2] = 0;
  $146 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($146|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $68 = $5;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = ($73>>>0)>(0);
  $75 = ($70>>>0)>(1);
  $76 = ($73|0)==(0);
  $77 = $76 & $75;
  $78 = $74 | $77;
  $79 = HEAP32[$i>>2]|0;
  if (!($78)) {
   break;
  }
  $80 = (($79) + 1)|0;
  HEAP32[$i>>2] = $80;
  $81 = $5;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = (($81) + 4)|0;
  $85 = $84;
  $86 = HEAP32[$85>>2]|0;
  $87 = (_bitshift64Lshr(($83|0),($86|0),1)|0);
  $88 = tempRet0;
  $89 = $5;
  $90 = $89;
  HEAP32[$90>>2] = $87;
  $91 = (($89) + 4)|0;
  $92 = $91;
  HEAP32[$92>>2] = $88;
 }
 $93 = HEAP32[$3>>2]|0;
 $94 = ((($93)) + 8|0);
 $95 = (($94) + (($79*72)|0)|0);
 $96 = ((($95)) + 44|0);
 $97 = HEAP32[$96>>2]|0;
 HEAP32[$iCur>>2] = $97;
 $98 = HEAP32[$i>>2]|0;
 $99 = HEAP32[$3>>2]|0;
 $100 = ((($99)) + 8|0);
 $101 = (($100) + (($98*72)|0)|0);
 $102 = ((($101)) + 16|0);
 $103 = HEAP32[$102>>2]|0;
 $104 = ((($103)) + 8|0);
 $105 = HEAP32[$104>>2]|0;
 HEAP32[$pIdx>>2] = $105;
 L20: while(1) {
  $106 = HEAP32[$pIdx>>2]|0;
  $107 = ($106|0)!=(0|0);
  if (!($107)) {
   label = 22;
   break;
  }
  $108 = HEAP32[$pIdx>>2]|0;
  $109 = ((($108)) + 40|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = ($110|0)==(0|0);
  L23: do {
   if (!($111)) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $112 = HEAP32[$i>>2]|0;
     $113 = HEAP32[$pIdx>>2]|0;
     $114 = ((($113)) + 50|0);
     $115 = HEAP16[$114>>1]|0;
     $116 = $115&65535;
     $117 = ($112|0)<($116|0);
     if (!($117)) {
      break L23;
     }
     $118 = HEAP32[$i>>2]|0;
     $119 = HEAP32[$pIdx>>2]|0;
     $120 = ((($119)) + 4|0);
     $121 = HEAP32[$120>>2]|0;
     $122 = (($121) + ($118<<1)|0);
     $123 = HEAP16[$122>>1]|0;
     $124 = $123 << 16 >> 16;
     $125 = ($124|0)!=(-2);
     if (!($125)) {
      $126 = HEAP32[$6>>2]|0;
      $127 = HEAP32[$i>>2]|0;
      $128 = HEAP32[$pIdx>>2]|0;
      $129 = ((($128)) + 40|0);
      $130 = HEAP32[$129>>2]|0;
      $131 = ((($130)) + 4|0);
      $132 = HEAP32[$131>>2]|0;
      $133 = (($132) + (($127*20)|0)|0);
      $134 = HEAP32[$133>>2]|0;
      $135 = HEAP32[$iCur>>2]|0;
      $136 = (_sqlite3ExprCompare($126,$134,$135)|0);
      $137 = ($136|0)==(0);
      if ($137) {
       label = 19;
       break L20;
      }
     }
     $141 = HEAP32[$i>>2]|0;
     $142 = (($141) + 1)|0;
     HEAP32[$i>>2] = $142;
    }
   }
  } while(0);
  $143 = HEAP32[$pIdx>>2]|0;
  $144 = ((($143)) + 20|0);
  $145 = HEAP32[$144>>2]|0;
  HEAP32[$pIdx>>2] = $145;
 }
 if ((label|0) == 19) {
  $138 = HEAP32[$iCur>>2]|0;
  $139 = HEAP32[$7>>2]|0;
  HEAP32[$139>>2] = $138;
  $140 = HEAP32[$8>>2]|0;
  HEAP32[$140>>2] = -2;
  HEAP32[$2>>2] = 1;
  $146 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($146|0);
 }
 else if ((label|0) == 22) {
  HEAP32[$2>>2] = 0;
  $146 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($146|0);
 }
 return (0)|0;
}
function _operatorMask($op) {
 $op = $op|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $c = sp + 4|0;
 HEAP32[$0>>2] = $op;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(33);
 do {
  if ($2) {
   HEAP16[$c>>1] = 1;
  } else {
   $3 = HEAP32[$0>>2]|0;
   $4 = ($3|0)==(34);
   if ($4) {
    HEAP16[$c>>1] = 256;
    break;
   }
   $5 = HEAP32[$0>>2]|0;
   $6 = ($5|0)==(29);
   if ($6) {
    HEAP16[$c>>1] = 128;
    break;
   } else {
    $7 = HEAP32[$0>>2]|0;
    $8 = (($7) - 37)|0;
    $9 = 2 << $8;
    $10 = $9&65535;
    HEAP16[$c>>1] = $10;
    break;
   }
  }
 } while(0);
 $11 = HEAP16[$c>>1]|0;
 STACKTOP = sp;return ($11|0);
}
function _whereClauseInsert($pWC,$p,$wtFlags) {
 $pWC = $pWC|0;
 $p = $p|0;
 $wtFlags = $wtFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $idx = 0, $pOld = 0, $pTerm = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 28|0;
 $pTerm = sp + 12|0;
 $idx = sp + 8|0;
 $pOld = sp + 4|0;
 $db = sp;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $p;
 HEAP16[$3>>1] = $wtFlags;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($6|0)>=($9|0);
 do {
  if ($10) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pOld>>2] = $13;
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$16>>2]|0;
   HEAP32[$db>>2] = $17;
   $18 = HEAP32[$db>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21*48)|0;
   $23 = $22<<1;
   $24 = (_sqlite3DbMallocRawNN($18,$23,0)|0);
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 20|0);
   HEAP32[$26>>2] = $24;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 20|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)==(0|0);
   if (!($30)) {
    $40 = HEAP32[$1>>2]|0;
    $41 = ((($40)) + 20|0);
    $42 = HEAP32[$41>>2]|0;
    $43 = HEAP32[$pOld>>2]|0;
    $44 = HEAP32[$1>>2]|0;
    $45 = ((($44)) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = ($46*48)|0;
    _memcpy(($42|0),($43|0),($47|0))|0;
    $48 = HEAP32[$pOld>>2]|0;
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 24|0);
    $51 = ($48|0)!=($50|0);
    if ($51) {
     $52 = HEAP32[$db>>2]|0;
     $53 = HEAP32[$pOld>>2]|0;
     _sqlite3DbFree($52,$53);
    }
    $54 = HEAP32[$db>>2]|0;
    $55 = HEAP32[$1>>2]|0;
    $56 = ((($55)) + 20|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (_sqlite3DbMallocSize($54,$57)|0);
    $59 = (($58>>>0) / 48)&-1;
    $60 = HEAP32[$1>>2]|0;
    $61 = ((($60)) + 16|0);
    HEAP32[$61>>2] = $59;
    break;
   }
   $31 = HEAP16[$3>>1]|0;
   $32 = $31&65535;
   $33 = $32 & 1;
   $34 = ($33|0)!=(0);
   if ($34) {
    $35 = HEAP32[$db>>2]|0;
    $36 = HEAP32[$2>>2]|0;
    _sqlite3ExprDelete($35,$36);
   }
   $37 = HEAP32[$pOld>>2]|0;
   $38 = HEAP32[$1>>2]|0;
   $39 = ((($38)) + 20|0);
   HEAP32[$39>>2] = $37;
   HEAP32[$0>>2] = 0;
   $104 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($104|0);
  }
 } while(0);
 $62 = HEAP32[$1>>2]|0;
 $63 = ((($62)) + 12|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = (($64) + 1)|0;
 HEAP32[$63>>2] = $65;
 HEAP32[$idx>>2] = $64;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = (($68) + (($64*48)|0)|0);
 HEAP32[$pTerm>>2] = $69;
 $70 = HEAP32[$2>>2]|0;
 $71 = ($70|0)!=(0|0);
 if ($71) {
  $72 = HEAP32[$2>>2]|0;
  $73 = ((($72)) + 4|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = $74 & 262144;
  $76 = ($75|0)!=(0);
  if ($76) {
   $77 = HEAP32[$2>>2]|0;
   $78 = ((($77)) + 28|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = ($79|0)<(0);
   $81 = $80 << 31 >> 31;
   $82 = (_sqlite3LogEst($79,$81)|0);
   $83 = $82 << 16 >> 16;
   $84 = (($83) - 270)|0;
   $85 = $84&65535;
   $86 = HEAP32[$pTerm>>2]|0;
   $87 = ((($86)) + 8|0);
   HEAP16[$87>>1] = $85;
  } else {
   label = 12;
  }
 } else {
  label = 12;
 }
 if ((label|0) == 12) {
  $88 = HEAP32[$pTerm>>2]|0;
  $89 = ((($88)) + 8|0);
  HEAP16[$89>>1] = 1;
 }
 $90 = HEAP32[$2>>2]|0;
 $91 = (_sqlite3ExprSkipCollate($90)|0);
 $92 = HEAP32[$pTerm>>2]|0;
 HEAP32[$92>>2] = $91;
 $93 = HEAP16[$3>>1]|0;
 $94 = HEAP32[$pTerm>>2]|0;
 $95 = ((($94)) + 10|0);
 HEAP16[$95>>1] = $93;
 $96 = HEAP32[$1>>2]|0;
 $97 = HEAP32[$pTerm>>2]|0;
 $98 = ((($97)) + 4|0);
 HEAP32[$98>>2] = $96;
 $99 = HEAP32[$pTerm>>2]|0;
 $100 = ((($99)) + 16|0);
 HEAP32[$100>>2] = -1;
 $101 = HEAP32[$pTerm>>2]|0;
 $102 = ((($101)) + 12|0);
 dest=$102; stop=dest+36|0; do { HEAP16[dest>>1]=0|0; dest=dest+2|0; } while ((dest|0) < (stop|0));
 $103 = HEAP32[$idx>>2]|0;
 HEAP32[$0>>2] = $103;
 $104 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _markTermAsChild($pWC,$iChild,$iParent) {
 $pWC = $pWC|0;
 $iChild = $iChild|0;
 $iParent = $iParent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pWC;
 HEAP32[$1>>2] = $iChild;
 HEAP32[$2>>2] = $iParent;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 20|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + (($4*48)|0)|0);
 $9 = ((($8)) + 16|0);
 HEAP32[$9>>2] = $3;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*48)|0)|0);
 $15 = ((($14)) + 8|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + (($17*48)|0)|0);
 $22 = ((($21)) + 8|0);
 HEAP16[$22>>1] = $16;
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 20|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (($26) + (($23*48)|0)|0);
 $28 = ((($27)) + 14|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = (($29) + 1)<<24>>24;
 HEAP8[$28>>0] = $30;
 STACKTOP = sp;return;
}
function _termIsEquivalence($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $aff1 = 0, $aff2 = 0, $pColl = 0, $zColl1 = 0, $zColl2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $aff1 = sp + 25|0;
 $aff2 = sp + 24|0;
 $pColl = sp + 8|0;
 $zColl1 = sp + 4|0;
 $zColl2 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 64|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = $7 & 512;
 $9 = ($8|0)==(0);
 if (!($9)) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)!=(37);
 if ($13) {
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(29);
  if ($17) {
   HEAP32[$0>>2] = 0;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = $20 & 1;
 $22 = ($21|0)!=(0);
 if ($22) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (_sqlite3ExprAffinity($25)|0);
 HEAP8[$aff1>>0] = $26;
 $27 = HEAP32[$2>>2]|0;
 $28 = ((($27)) + 16|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3ExprAffinity($29)|0);
 HEAP8[$aff2>>0] = $30;
 $31 = HEAP8[$aff1>>0]|0;
 $32 = $31 << 24 >> 24;
 $33 = HEAP8[$aff2>>0]|0;
 $34 = $33 << 24 >> 24;
 $35 = ($32|0)!=($34|0);
 do {
  if ($35) {
   $36 = HEAP8[$aff1>>0]|0;
   $37 = $36 << 24 >> 24;
   $38 = ($37|0)>=(67);
   if ($38) {
    $39 = HEAP8[$aff2>>0]|0;
    $40 = $39 << 24 >> 24;
    $41 = ($40|0)>=(67);
    if ($41) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 } while(0);
 $42 = HEAP32[$1>>2]|0;
 $43 = HEAP32[$2>>2]|0;
 $44 = ((($43)) + 12|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = ((($46)) + 16|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (_sqlite3BinaryCompareCollSeq($42,$45,$48)|0);
 HEAP32[$pColl>>2] = $49;
 $50 = HEAP32[$pColl>>2]|0;
 $51 = ($50|0)==(0|0);
 if (!($51)) {
  $52 = HEAP32[$pColl>>2]|0;
  $53 = HEAP32[$52>>2]|0;
  $54 = (_sqlite3StrICmp($53,34834)|0);
  $55 = ($54|0)==(0);
  if (!($55)) {
   $56 = HEAP32[$1>>2]|0;
   $57 = HEAP32[$2>>2]|0;
   $58 = ((($57)) + 12|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3ExprCollSeq($56,$59)|0);
   HEAP32[$pColl>>2] = $60;
   $61 = HEAP32[$pColl>>2]|0;
   $62 = ($61|0)!=(0|0);
   if ($62) {
    $63 = HEAP32[$pColl>>2]|0;
    $64 = HEAP32[$63>>2]|0;
    $65 = $64;
   } else {
    $65 = 0;
   }
   HEAP32[$zColl1>>2] = $65;
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$2>>2]|0;
   $68 = ((($67)) + 16|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = (_sqlite3ExprCollSeq($66,$69)|0);
   HEAP32[$pColl>>2] = $70;
   $71 = HEAP32[$pColl>>2]|0;
   $72 = ($71|0)!=(0|0);
   if ($72) {
    $73 = HEAP32[$pColl>>2]|0;
    $74 = HEAP32[$73>>2]|0;
    $75 = $74;
   } else {
    $75 = 0;
   }
   HEAP32[$zColl2>>2] = $75;
   $76 = HEAP32[$zColl1>>2]|0;
   $77 = HEAP32[$zColl2>>2]|0;
   $78 = (_sqlite3_stricmp($76,$77)|0);
   $79 = ($78|0)==(0);
   $80 = $79&1;
   HEAP32[$0>>2] = $80;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $81 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($81|0);
}
function _exprCommute($pParse,$pExpr) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $expLeft = 0, $expRight = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $expRight = sp + 14|0;
 $expLeft = sp + 12|0;
 $t = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 256;
 $8 = $7&65535;
 HEAP16[$expRight>>1] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = $13 & 256;
 $15 = $14&65535;
 HEAP16[$expLeft>>1] = $15;
 $16 = HEAP16[$expRight>>1]|0;
 $17 = $16&65535;
 $18 = HEAP16[$expLeft>>1]|0;
 $19 = $18&65535;
 $20 = ($17|0)==($19|0);
 do {
  if ($20) {
   $21 = HEAP16[$expRight>>1]|0;
   $22 = ($21<<16>>16)!=(0);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 16|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ((($25)) + 4|0);
    $27 = HEAP32[$26>>2]|0;
    $28 = $27 & -257;
    HEAP32[$26>>2] = $28;
    break;
   }
   $29 = HEAP32[$0>>2]|0;
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 12|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3ExprCollSeq($29,$32)|0);
   $34 = ($33|0)!=(0|0);
   if ($34) {
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 12|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = $39 | 256;
    HEAP32[$38>>2] = $40;
   }
  }
 } while(0);
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 16|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$t>>2] = $43;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 16|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP32[$t>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 12|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$1>>2]|0;
 $53 = HEAP8[$52>>0]|0;
 $54 = $53&255;
 $55 = ($54|0)>=(38);
 if (!($55)) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$1>>2]|0;
 $57 = HEAP8[$56>>0]|0;
 $58 = $57&255;
 $59 = (($58) - 38)|0;
 $60 = $59 ^ 2;
 $61 = (($60) + 38)|0;
 $62 = $61&255;
 $63 = HEAP32[$1>>2]|0;
 HEAP8[$63>>0] = $62;
 STACKTOP = sp;return;
}
function _transferJoinMarkings($pDerived,$pBase) {
 $pDerived = $pDerived|0;
 $pBase = $pBase|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pDerived;
 HEAP32[$1>>2] = $pBase;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = $6 & 1;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = $10 | $7;
 HEAP32[$9>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 36|0);
 $14 = HEAP16[$13>>1]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 36|0);
 HEAP16[$16>>1] = $14;
 STACKTOP = sp;return;
}
function _exprAnalyzeOrTerm($pSrc,$pWC,$idxTerm) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $idxTerm = $idxTerm|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $affLeft = 0, $affRight = 0, $b = 0, $b1 = 0, $chngToIN = 0, $db = 0, $i = 0, $iColumn = 0, $iCursor = 0;
 var $iOne = 0, $iTwo = 0, $idxNew = 0, $indexable = 0, $j = 0, $j2 = 0, $okToChngToIN = 0, $pAndInfo = 0, $pAndTerm = 0, $pAndWC = 0, $pDup = 0, $pExpr = 0, $pLeft = 0, $pList = 0, $pNew = 0, $pOne = 0, $pOrInfo = 0, $pOrTerm = 0, $pOrWc = 0, $pOther = 0;
 var $pParse = 0, $pTerm = 0, $pTwo = 0, $pWInfo = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 156|0;
 $1 = sp + 152|0;
 $2 = sp + 148|0;
 $pWInfo = sp + 144|0;
 $pParse = sp + 140|0;
 $db = sp + 136|0;
 $pTerm = sp + 132|0;
 $pExpr = sp + 128|0;
 $i = sp + 124|0;
 $pOrWc = sp + 120|0;
 $pOrTerm = sp + 116|0;
 $pOrInfo = sp + 112|0;
 $chngToIN = sp + 24|0;
 $indexable = sp + 16|0;
 $pAndInfo = sp + 108|0;
 $pAndWC = sp + 104|0;
 $pAndTerm = sp + 100|0;
 $j = sp + 96|0;
 $b = sp + 8|0;
 $b1 = sp;
 $pOther = sp + 92|0;
 $iOne = sp + 88|0;
 $pOne = sp + 84|0;
 $iTwo = sp + 80|0;
 $pTwo = sp + 76|0;
 $okToChngToIN = sp + 72|0;
 $iColumn = sp + 68|0;
 $iCursor = sp + 64|0;
 $j2 = sp + 60|0;
 $affLeft = sp + 56|0;
 $affRight = sp + 52|0;
 $pDup = sp + 48|0;
 $pList = sp + 44|0;
 $pLeft = sp + 40|0;
 $pNew = sp + 36|0;
 $idxNew = sp + 32|0;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $idxTerm;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pWInfo>>2] = $4;
 $5 = HEAP32[$pWInfo>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$db>>2] = $8;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + (($9*48)|0)|0);
 HEAP32[$pTerm>>2] = $13;
 $14 = HEAP32[$pTerm>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pExpr>>2] = $15;
 $16 = HEAP32[$db>>2]|0;
 $17 = (_sqlite3DbMallocZero($16,416,0)|0);
 HEAP32[$pOrInfo>>2] = $17;
 $18 = HEAP32[$pTerm>>2]|0;
 $19 = ((($18)) + 28|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$pOrInfo>>2]|0;
 $21 = ($20|0)==(0|0);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$pTerm>>2]|0;
 $23 = ((($22)) + 10|0);
 $24 = HEAP16[$23>>1]|0;
 $25 = $24&65535;
 $26 = $25 | 16;
 $27 = $26&65535;
 HEAP16[$23>>1] = $27;
 $28 = HEAP32[$pOrInfo>>2]|0;
 HEAP32[$pOrWc>>2] = $28;
 $29 = HEAP32[$pOrWc>>2]|0;
 $30 = ((($29)) + 24|0);
 _memset(($30|0),0,384)|0;
 $31 = HEAP32[$pOrWc>>2]|0;
 $32 = HEAP32[$pWInfo>>2]|0;
 _sqlite3WhereClauseInit($31,$32);
 $33 = HEAP32[$pOrWc>>2]|0;
 $34 = HEAP32[$pExpr>>2]|0;
 _sqlite3WhereSplit($33,$34,27);
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$pOrWc>>2]|0;
 _sqlite3WhereExprAnalyze($35,$36);
 $37 = HEAP32[$db>>2]|0;
 $38 = ((($37)) + 69|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = ($39<<24>>24)!=(0);
 if ($40) {
  STACKTOP = sp;return;
 }
 $41 = $indexable;
 $42 = $41;
 HEAP32[$42>>2] = -1;
 $43 = (($41) + 4)|0;
 $44 = $43;
 HEAP32[$44>>2] = -1;
 $45 = $chngToIN;
 $46 = $45;
 HEAP32[$46>>2] = -1;
 $47 = (($45) + 4)|0;
 $48 = $47;
 HEAP32[$48>>2] = -1;
 $49 = HEAP32[$pOrWc>>2]|0;
 $50 = ((($49)) + 12|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) - 1)|0;
 HEAP32[$i>>2] = $52;
 $53 = HEAP32[$pOrWc>>2]|0;
 $54 = ((($53)) + 20|0);
 $55 = HEAP32[$54>>2]|0;
 HEAP32[$pOrTerm>>2] = $55;
 while(1) {
  $56 = HEAP32[$i>>2]|0;
  $57 = ($56|0)>=(0);
  $58 = $indexable;
  $59 = $58;
  $60 = HEAP32[$59>>2]|0;
  $61 = (($58) + 4)|0;
  $62 = $61;
  $63 = HEAP32[$62>>2]|0;
  $64 = ($60|0)!=(0);
  $65 = ($63|0)!=(0);
  $66 = $64 | $65;
  $67 = $57 ? $66 : 0;
  if (!($67)) {
   break;
  }
  $68 = HEAP32[$pOrTerm>>2]|0;
  $69 = ((($68)) + 12|0);
  $70 = HEAP16[$69>>1]|0;
  $71 = $70&65535;
  $72 = $71 & 511;
  $73 = ($72|0)==(0);
  do {
   if ($73) {
    $74 = $chngToIN;
    $75 = $74;
    HEAP32[$75>>2] = 0;
    $76 = (($74) + 4)|0;
    $77 = $76;
    HEAP32[$77>>2] = 0;
    $78 = HEAP32[$db>>2]|0;
    $79 = (_sqlite3DbMallocRawNN($78,408,0)|0);
    HEAP32[$pAndInfo>>2] = $79;
    $80 = HEAP32[$pAndInfo>>2]|0;
    $81 = ($80|0)!=(0|0);
    if ($81) {
     $82 = $b;
     $83 = $82;
     HEAP32[$83>>2] = 0;
     $84 = (($82) + 4)|0;
     $85 = $84;
     HEAP32[$85>>2] = 0;
     $86 = HEAP32[$pAndInfo>>2]|0;
     $87 = HEAP32[$pOrTerm>>2]|0;
     $88 = ((($87)) + 28|0);
     HEAP32[$88>>2] = $86;
     $89 = HEAP32[$pOrTerm>>2]|0;
     $90 = ((($89)) + 10|0);
     $91 = HEAP16[$90>>1]|0;
     $92 = $91&65535;
     $93 = $92 | 32;
     $94 = $93&65535;
     HEAP16[$90>>1] = $94;
     $95 = HEAP32[$pOrTerm>>2]|0;
     $96 = ((($95)) + 12|0);
     HEAP16[$96>>1] = 1024;
     $97 = HEAP32[$pAndInfo>>2]|0;
     HEAP32[$pAndWC>>2] = $97;
     $98 = HEAP32[$pAndWC>>2]|0;
     $99 = ((($98)) + 24|0);
     _memset(($99|0),0,384)|0;
     $100 = HEAP32[$pAndWC>>2]|0;
     $101 = HEAP32[$1>>2]|0;
     $102 = HEAP32[$101>>2]|0;
     _sqlite3WhereClauseInit($100,$102);
     $103 = HEAP32[$pAndWC>>2]|0;
     $104 = HEAP32[$pOrTerm>>2]|0;
     $105 = HEAP32[$104>>2]|0;
     _sqlite3WhereSplit($103,$105,28);
     $106 = HEAP32[$0>>2]|0;
     $107 = HEAP32[$pAndWC>>2]|0;
     _sqlite3WhereExprAnalyze($106,$107);
     $108 = HEAP32[$1>>2]|0;
     $109 = HEAP32[$pAndWC>>2]|0;
     $110 = ((($109)) + 4|0);
     HEAP32[$110>>2] = $108;
     $111 = HEAP32[$db>>2]|0;
     $112 = ((($111)) + 69|0);
     $113 = HEAP8[$112>>0]|0;
     $114 = ($113<<24>>24)!=(0);
     L13: do {
      if (!($114)) {
       HEAP32[$j>>2] = 0;
       $115 = HEAP32[$pAndWC>>2]|0;
       $116 = ((($115)) + 20|0);
       $117 = HEAP32[$116>>2]|0;
       HEAP32[$pAndTerm>>2] = $117;
       while(1) {
        $118 = HEAP32[$j>>2]|0;
        $119 = HEAP32[$pAndWC>>2]|0;
        $120 = ((($119)) + 12|0);
        $121 = HEAP32[$120>>2]|0;
        $122 = ($118|0)<($121|0);
        if (!($122)) {
         break L13;
        }
        $123 = HEAP32[$pAndTerm>>2]|0;
        $124 = HEAP32[$123>>2]|0;
        $125 = HEAP8[$124>>0]|0;
        $126 = $125&255;
        $127 = (_allowedOp($126)|0);
        $128 = ($127|0)!=(0);
        if ($128) {
         label = 12;
        } else {
         $129 = HEAP32[$pAndTerm>>2]|0;
         $130 = ((($129)) + 12|0);
         $131 = HEAP16[$130>>1]|0;
         $132 = $131&65535;
         $133 = ($132|0)==(64);
         if ($133) {
          label = 12;
         }
        }
        if ((label|0) == 12) {
         label = 0;
         $134 = HEAP32[$pWInfo>>2]|0;
         $135 = ((($134)) + 488|0);
         $136 = HEAP32[$pAndTerm>>2]|0;
         $137 = ((($136)) + 20|0);
         $138 = HEAP32[$137>>2]|0;
         $139 = (_sqlite3WhereGetMask($135,$138)|0);
         $140 = tempRet0;
         $141 = $b;
         $142 = $141;
         $143 = HEAP32[$142>>2]|0;
         $144 = (($141) + 4)|0;
         $145 = $144;
         $146 = HEAP32[$145>>2]|0;
         $147 = $143 | $139;
         $148 = $146 | $140;
         $149 = $b;
         $150 = $149;
         HEAP32[$150>>2] = $147;
         $151 = (($149) + 4)|0;
         $152 = $151;
         HEAP32[$152>>2] = $148;
        }
        $153 = HEAP32[$j>>2]|0;
        $154 = (($153) + 1)|0;
        HEAP32[$j>>2] = $154;
        $155 = HEAP32[$pAndTerm>>2]|0;
        $156 = ((($155)) + 48|0);
        HEAP32[$pAndTerm>>2] = $156;
       }
      }
     } while(0);
     $157 = $b;
     $158 = $157;
     $159 = HEAP32[$158>>2]|0;
     $160 = (($157) + 4)|0;
     $161 = $160;
     $162 = HEAP32[$161>>2]|0;
     $163 = $indexable;
     $164 = $163;
     $165 = HEAP32[$164>>2]|0;
     $166 = (($163) + 4)|0;
     $167 = $166;
     $168 = HEAP32[$167>>2]|0;
     $169 = $165 & $159;
     $170 = $168 & $162;
     $171 = $indexable;
     $172 = $171;
     HEAP32[$172>>2] = $169;
     $173 = (($171) + 4)|0;
     $174 = $173;
     HEAP32[$174>>2] = $170;
    }
   } else {
    $175 = HEAP32[$pOrTerm>>2]|0;
    $176 = ((($175)) + 10|0);
    $177 = HEAP16[$176>>1]|0;
    $178 = $177&65535;
    $179 = $178 & 8;
    $180 = ($179|0)!=(0);
    if (!($180)) {
     $181 = HEAP32[$pWInfo>>2]|0;
     $182 = ((($181)) + 488|0);
     $183 = HEAP32[$pOrTerm>>2]|0;
     $184 = ((($183)) + 20|0);
     $185 = HEAP32[$184>>2]|0;
     $186 = (_sqlite3WhereGetMask($182,$185)|0);
     $187 = tempRet0;
     $188 = $b1;
     $189 = $188;
     HEAP32[$189>>2] = $186;
     $190 = (($188) + 4)|0;
     $191 = $190;
     HEAP32[$191>>2] = $187;
     $192 = HEAP32[$pOrTerm>>2]|0;
     $193 = ((($192)) + 10|0);
     $194 = HEAP16[$193>>1]|0;
     $195 = $194&65535;
     $196 = $195 & 2;
     $197 = ($196|0)!=(0);
     if ($197) {
      $198 = HEAP32[$pOrTerm>>2]|0;
      $199 = ((($198)) + 16|0);
      $200 = HEAP32[$199>>2]|0;
      $201 = HEAP32[$pOrWc>>2]|0;
      $202 = ((($201)) + 20|0);
      $203 = HEAP32[$202>>2]|0;
      $204 = (($203) + (($200*48)|0)|0);
      HEAP32[$pOther>>2] = $204;
      $205 = HEAP32[$pWInfo>>2]|0;
      $206 = ((($205)) + 488|0);
      $207 = HEAP32[$pOther>>2]|0;
      $208 = ((($207)) + 20|0);
      $209 = HEAP32[$208>>2]|0;
      $210 = (_sqlite3WhereGetMask($206,$209)|0);
      $211 = tempRet0;
      $212 = $b1;
      $213 = $212;
      $214 = HEAP32[$213>>2]|0;
      $215 = (($212) + 4)|0;
      $216 = $215;
      $217 = HEAP32[$216>>2]|0;
      $218 = $214 | $210;
      $219 = $217 | $211;
      $220 = $b1;
      $221 = $220;
      HEAP32[$221>>2] = $218;
      $222 = (($220) + 4)|0;
      $223 = $222;
      HEAP32[$223>>2] = $219;
     }
     $224 = $b1;
     $225 = $224;
     $226 = HEAP32[$225>>2]|0;
     $227 = (($224) + 4)|0;
     $228 = $227;
     $229 = HEAP32[$228>>2]|0;
     $230 = $indexable;
     $231 = $230;
     $232 = HEAP32[$231>>2]|0;
     $233 = (($230) + 4)|0;
     $234 = $233;
     $235 = HEAP32[$234>>2]|0;
     $236 = $232 & $226;
     $237 = $235 & $229;
     $238 = $indexable;
     $239 = $238;
     HEAP32[$239>>2] = $236;
     $240 = (($238) + 4)|0;
     $241 = $240;
     HEAP32[$241>>2] = $237;
     $242 = HEAP32[$pOrTerm>>2]|0;
     $243 = ((($242)) + 12|0);
     $244 = HEAP16[$243>>1]|0;
     $245 = $244&65535;
     $246 = $245 & 2;
     $247 = ($246|0)==(0);
     if ($247) {
      $248 = $chngToIN;
      $249 = $248;
      HEAP32[$249>>2] = 0;
      $250 = (($248) + 4)|0;
      $251 = $250;
      HEAP32[$251>>2] = 0;
      break;
     } else {
      $252 = $b1;
      $253 = $252;
      $254 = HEAP32[$253>>2]|0;
      $255 = (($252) + 4)|0;
      $256 = $255;
      $257 = HEAP32[$256>>2]|0;
      $258 = $chngToIN;
      $259 = $258;
      $260 = HEAP32[$259>>2]|0;
      $261 = (($258) + 4)|0;
      $262 = $261;
      $263 = HEAP32[$262>>2]|0;
      $264 = $260 & $254;
      $265 = $263 & $257;
      $266 = $chngToIN;
      $267 = $266;
      HEAP32[$267>>2] = $264;
      $268 = (($266) + 4)|0;
      $269 = $268;
      HEAP32[$269>>2] = $265;
      break;
     }
    }
   }
  } while(0);
  $270 = HEAP32[$i>>2]|0;
  $271 = (($270) + -1)|0;
  HEAP32[$i>>2] = $271;
  $272 = HEAP32[$pOrTerm>>2]|0;
  $273 = ((($272)) + 48|0);
  HEAP32[$pOrTerm>>2] = $273;
 }
 $274 = $indexable;
 $275 = $274;
 $276 = HEAP32[$275>>2]|0;
 $277 = (($274) + 4)|0;
 $278 = $277;
 $279 = HEAP32[$278>>2]|0;
 $280 = HEAP32[$pOrInfo>>2]|0;
 $281 = ((($280)) + 408|0);
 $282 = $281;
 $283 = $282;
 HEAP32[$283>>2] = $276;
 $284 = (($282) + 4)|0;
 $285 = $284;
 HEAP32[$285>>2] = $279;
 $286 = $indexable;
 $287 = $286;
 $288 = HEAP32[$287>>2]|0;
 $289 = (($286) + 4)|0;
 $290 = $289;
 $291 = HEAP32[$290>>2]|0;
 $292 = ($288|0)==(0);
 $293 = ($291|0)==(0);
 $294 = $292 & $293;
 $295 = $294 ? 0 : 512;
 $296 = $295&65535;
 $297 = HEAP32[$pTerm>>2]|0;
 $298 = ((($297)) + 12|0);
 HEAP16[$298>>1] = $296;
 $299 = $indexable;
 $300 = $299;
 $301 = HEAP32[$300>>2]|0;
 $302 = (($299) + 4)|0;
 $303 = $302;
 $304 = HEAP32[$303>>2]|0;
 $305 = ($301|0)!=(0);
 $306 = ($304|0)!=(0);
 $307 = $305 | $306;
 L34: do {
  if ($307) {
   $308 = HEAP32[$pOrWc>>2]|0;
   $309 = ((($308)) + 12|0);
   $310 = HEAP32[$309>>2]|0;
   $311 = ($310|0)==(2);
   if ($311) {
    HEAP32[$iOne>>2] = 0;
    L37: while(1) {
     $312 = HEAP32[$pOrWc>>2]|0;
     $313 = ((($312)) + 20|0);
     $314 = HEAP32[$313>>2]|0;
     $315 = HEAP32[$iOne>>2]|0;
     $316 = (($315) + 1)|0;
     HEAP32[$iOne>>2] = $316;
     $317 = (_whereNthSubterm($314,$315)|0);
     HEAP32[$pOne>>2] = $317;
     $318 = ($317|0)!=(0|0);
     if (!($318)) {
      break L34;
     }
     HEAP32[$iTwo>>2] = 0;
     while(1) {
      $319 = HEAP32[$pOrWc>>2]|0;
      $320 = ((($319)) + 20|0);
      $321 = HEAP32[$320>>2]|0;
      $322 = ((($321)) + 48|0);
      $323 = HEAP32[$iTwo>>2]|0;
      $324 = (($323) + 1)|0;
      HEAP32[$iTwo>>2] = $324;
      $325 = (_whereNthSubterm($322,$323)|0);
      HEAP32[$pTwo>>2] = $325;
      $326 = ($325|0)!=(0|0);
      if (!($326)) {
       continue L37;
      }
      $327 = HEAP32[$0>>2]|0;
      $328 = HEAP32[$1>>2]|0;
      $329 = HEAP32[$pOne>>2]|0;
      $330 = HEAP32[$pTwo>>2]|0;
      _whereCombineDisjuncts($327,$328,$329,$330);
     }
    }
   }
  }
 } while(0);
 $331 = $chngToIN;
 $332 = $331;
 $333 = HEAP32[$332>>2]|0;
 $334 = (($331) + 4)|0;
 $335 = $334;
 $336 = HEAP32[$335>>2]|0;
 $337 = ($333|0)!=(0);
 $338 = ($336|0)!=(0);
 $339 = $337 | $338;
 if (!($339)) {
  STACKTOP = sp;return;
 }
 HEAP32[$okToChngToIN>>2] = 0;
 HEAP32[$iColumn>>2] = -1;
 HEAP32[$iCursor>>2] = -1;
 HEAP32[$j2>>2] = 0;
 HEAP32[$j2>>2] = 0;
 while(1) {
  $340 = HEAP32[$j2>>2]|0;
  $341 = ($340|0)<(2);
  if (!($341)) {
   break;
  }
  $342 = HEAP32[$okToChngToIN>>2]|0;
  $343 = ($342|0)!=(0);
  $344 = $343 ^ 1;
  if (!($344)) {
   break;
  }
  $345 = HEAP32[$pOrWc>>2]|0;
  $346 = ((($345)) + 20|0);
  $347 = HEAP32[$346>>2]|0;
  HEAP32[$pOrTerm>>2] = $347;
  $348 = HEAP32[$pOrWc>>2]|0;
  $349 = ((($348)) + 12|0);
  $350 = HEAP32[$349>>2]|0;
  $351 = (($350) - 1)|0;
  HEAP32[$i>>2] = $351;
  while(1) {
   $352 = HEAP32[$i>>2]|0;
   $353 = ($352|0)>=(0);
   if (!($353)) {
    break;
   }
   $354 = HEAP32[$pOrTerm>>2]|0;
   $355 = ((($354)) + 10|0);
   $356 = HEAP16[$355>>1]|0;
   $357 = $356&65535;
   $358 = $357 & -65;
   $359 = $358&65535;
   HEAP16[$355>>1] = $359;
   $360 = HEAP32[$pOrTerm>>2]|0;
   $361 = ((($360)) + 20|0);
   $362 = HEAP32[$361>>2]|0;
   $363 = HEAP32[$iCursor>>2]|0;
   $364 = ($362|0)==($363|0);
   if (!($364)) {
    $365 = $chngToIN;
    $366 = $365;
    $367 = HEAP32[$366>>2]|0;
    $368 = (($365) + 4)|0;
    $369 = $368;
    $370 = HEAP32[$369>>2]|0;
    $371 = HEAP32[$pWInfo>>2]|0;
    $372 = ((($371)) + 488|0);
    $373 = HEAP32[$pOrTerm>>2]|0;
    $374 = ((($373)) + 20|0);
    $375 = HEAP32[$374>>2]|0;
    $376 = (_sqlite3WhereGetMask($372,$375)|0);
    $377 = tempRet0;
    $378 = $367 & $376;
    $379 = $370 & $377;
    $380 = ($378|0)==(0);
    $381 = ($379|0)==(0);
    $382 = $380 & $381;
    if (!($382)) {
     label = 37;
     break;
    }
   }
   $389 = HEAP32[$i>>2]|0;
   $390 = (($389) + -1)|0;
   HEAP32[$i>>2] = $390;
   $391 = HEAP32[$pOrTerm>>2]|0;
   $392 = ((($391)) + 48|0);
   HEAP32[$pOrTerm>>2] = $392;
  }
  if ((label|0) == 37) {
   label = 0;
   $383 = HEAP32[$pOrTerm>>2]|0;
   $384 = ((($383)) + 28|0);
   $385 = HEAP32[$384>>2]|0;
   HEAP32[$iColumn>>2] = $385;
   $386 = HEAP32[$pOrTerm>>2]|0;
   $387 = ((($386)) + 20|0);
   $388 = HEAP32[$387>>2]|0;
   HEAP32[$iCursor>>2] = $388;
  }
  $393 = HEAP32[$i>>2]|0;
  $394 = ($393|0)<(0);
  if ($394) {
   break;
  }
  HEAP32[$okToChngToIN>>2] = 1;
  while(1) {
   $395 = HEAP32[$i>>2]|0;
   $396 = ($395|0)>=(0);
   $397 = HEAP32[$okToChngToIN>>2]|0;
   $398 = ($397|0)!=(0);
   $399 = $396 ? $398 : 0;
   if (!($399)) {
    break;
   }
   $400 = HEAP32[$pOrTerm>>2]|0;
   $401 = ((($400)) + 20|0);
   $402 = HEAP32[$401>>2]|0;
   $403 = HEAP32[$iCursor>>2]|0;
   $404 = ($402|0)!=($403|0);
   $405 = HEAP32[$pOrTerm>>2]|0;
   do {
    if ($404) {
     $406 = ((($405)) + 10|0);
     $407 = HEAP16[$406>>1]|0;
     $408 = $407&65535;
     $409 = $408 & -65;
     $410 = $409&65535;
     HEAP16[$406>>1] = $410;
    } else {
     $411 = ((($405)) + 28|0);
     $412 = HEAP32[$411>>2]|0;
     $413 = HEAP32[$iColumn>>2]|0;
     $414 = ($412|0)!=($413|0);
     if ($414) {
      HEAP32[$okToChngToIN>>2] = 0;
      break;
     }
     $415 = HEAP32[$pOrTerm>>2]|0;
     $416 = HEAP32[$415>>2]|0;
     $417 = ((($416)) + 16|0);
     $418 = HEAP32[$417>>2]|0;
     $419 = (_sqlite3ExprAffinity($418)|0);
     $420 = $419 << 24 >> 24;
     HEAP32[$affRight>>2] = $420;
     $421 = HEAP32[$pOrTerm>>2]|0;
     $422 = HEAP32[$421>>2]|0;
     $423 = ((($422)) + 12|0);
     $424 = HEAP32[$423>>2]|0;
     $425 = (_sqlite3ExprAffinity($424)|0);
     $426 = $425 << 24 >> 24;
     HEAP32[$affLeft>>2] = $426;
     $427 = HEAP32[$affRight>>2]|0;
     $428 = ($427|0)!=(0);
     if ($428) {
      $429 = HEAP32[$affRight>>2]|0;
      $430 = HEAP32[$affLeft>>2]|0;
      $431 = ($429|0)!=($430|0);
      if ($431) {
       HEAP32[$okToChngToIN>>2] = 0;
       break;
      }
     }
     $432 = HEAP32[$pOrTerm>>2]|0;
     $433 = ((($432)) + 10|0);
     $434 = HEAP16[$433>>1]|0;
     $435 = $434&65535;
     $436 = $435 | 64;
     $437 = $436&65535;
     HEAP16[$433>>1] = $437;
    }
   } while(0);
   $438 = HEAP32[$i>>2]|0;
   $439 = (($438) + -1)|0;
   HEAP32[$i>>2] = $439;
   $440 = HEAP32[$pOrTerm>>2]|0;
   $441 = ((($440)) + 48|0);
   HEAP32[$pOrTerm>>2] = $441;
  }
  $442 = HEAP32[$j2>>2]|0;
  $443 = (($442) + 1)|0;
  HEAP32[$j2>>2] = $443;
 }
 $444 = HEAP32[$okToChngToIN>>2]|0;
 $445 = ($444|0)!=(0);
 if (!($445)) {
  STACKTOP = sp;return;
 }
 HEAP32[$pList>>2] = 0;
 HEAP32[$pLeft>>2] = 0;
 $446 = HEAP32[$pOrWc>>2]|0;
 $447 = ((($446)) + 12|0);
 $448 = HEAP32[$447>>2]|0;
 $449 = (($448) - 1)|0;
 HEAP32[$i>>2] = $449;
 $450 = HEAP32[$pOrWc>>2]|0;
 $451 = ((($450)) + 20|0);
 $452 = HEAP32[$451>>2]|0;
 HEAP32[$pOrTerm>>2] = $452;
 while(1) {
  $453 = HEAP32[$i>>2]|0;
  $454 = ($453|0)>=(0);
  if (!($454)) {
   break;
  }
  $455 = HEAP32[$pOrTerm>>2]|0;
  $456 = ((($455)) + 10|0);
  $457 = HEAP16[$456>>1]|0;
  $458 = $457&65535;
  $459 = $458 & 64;
  $460 = ($459|0)==(0);
  if (!($460)) {
   $461 = HEAP32[$db>>2]|0;
   $462 = HEAP32[$pOrTerm>>2]|0;
   $463 = HEAP32[$462>>2]|0;
   $464 = ((($463)) + 16|0);
   $465 = HEAP32[$464>>2]|0;
   $466 = (_sqlite3ExprDup($461,$465,0)|0);
   HEAP32[$pDup>>2] = $466;
   $467 = HEAP32[$pWInfo>>2]|0;
   $468 = HEAP32[$467>>2]|0;
   $469 = HEAP32[$pList>>2]|0;
   $470 = HEAP32[$pDup>>2]|0;
   $471 = (_sqlite3ExprListAppend($468,$469,$470)|0);
   HEAP32[$pList>>2] = $471;
   $472 = HEAP32[$pOrTerm>>2]|0;
   $473 = HEAP32[$472>>2]|0;
   $474 = ((($473)) + 12|0);
   $475 = HEAP32[$474>>2]|0;
   HEAP32[$pLeft>>2] = $475;
  }
  $476 = HEAP32[$i>>2]|0;
  $477 = (($476) + -1)|0;
  HEAP32[$i>>2] = $477;
  $478 = HEAP32[$pOrTerm>>2]|0;
  $479 = ((($478)) + 48|0);
  HEAP32[$pOrTerm>>2] = $479;
 }
 $480 = HEAP32[$db>>2]|0;
 $481 = HEAP32[$pLeft>>2]|0;
 $482 = (_sqlite3ExprDup($480,$481,0)|0);
 HEAP32[$pDup>>2] = $482;
 $483 = HEAP32[$pParse>>2]|0;
 $484 = HEAP32[$pDup>>2]|0;
 $485 = (_sqlite3PExpr($483,33,$484,0,0)|0);
 HEAP32[$pNew>>2] = $485;
 $486 = HEAP32[$pNew>>2]|0;
 $487 = ($486|0)!=(0|0);
 if ($487) {
  $488 = HEAP32[$pNew>>2]|0;
  $489 = HEAP32[$pExpr>>2]|0;
  _transferJoinMarkings($488,$489);
  $490 = HEAP32[$pList>>2]|0;
  $491 = HEAP32[$pNew>>2]|0;
  $492 = ((($491)) + 20|0);
  HEAP32[$492>>2] = $490;
  $493 = HEAP32[$1>>2]|0;
  $494 = HEAP32[$pNew>>2]|0;
  $495 = (_whereClauseInsert($493,$494,3)|0);
  HEAP32[$idxNew>>2] = $495;
  $496 = HEAP32[$0>>2]|0;
  $497 = HEAP32[$1>>2]|0;
  $498 = HEAP32[$idxNew>>2]|0;
  _exprAnalyze($496,$497,$498);
  $499 = HEAP32[$2>>2]|0;
  $500 = HEAP32[$1>>2]|0;
  $501 = ((($500)) + 20|0);
  $502 = HEAP32[$501>>2]|0;
  $503 = (($502) + (($499*48)|0)|0);
  HEAP32[$pTerm>>2] = $503;
  $504 = HEAP32[$1>>2]|0;
  $505 = HEAP32[$idxNew>>2]|0;
  $506 = HEAP32[$2>>2]|0;
  _markTermAsChild($504,$505,$506);
 } else {
  $507 = HEAP32[$db>>2]|0;
  $508 = HEAP32[$pList>>2]|0;
  _sqlite3ExprListDelete($507,$508);
 }
 $509 = HEAP32[$pTerm>>2]|0;
 $510 = ((($509)) + 12|0);
 HEAP16[$510>>1] = 4096;
 STACKTOP = sp;return;
}
function _isLikeOrGlob($pParse,$pExpr,$ppPrefix,$pisComplete,$pnoCase) {
 $pParse = $pParse|0;
 $pExpr = $pExpr|0;
 $ppPrefix = $ppPrefix|0;
 $pisComplete = $pisComplete|0;
 $pnoCase = $pnoCase|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $cnt = 0, $db = 0, $iCol = 0, $op = 0, $pLeft = 0, $pList = 0, $pPrefix = 0, $pReprepare = 0, $pRight = 0, $pVal = 0;
 var $r1 = 0, $rc = 0, $v = 0, $wc = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $3 = sp + 68|0;
 $4 = sp + 64|0;
 $5 = sp + 60|0;
 $z = sp + 56|0;
 $pRight = sp + 52|0;
 $pLeft = sp + 48|0;
 $pList = sp + 44|0;
 $c = sp + 40|0;
 $cnt = sp + 36|0;
 $wc = sp + 84|0;
 $db = sp + 32|0;
 $pVal = sp + 28|0;
 $op = sp + 24|0;
 $rc = sp + 20|0;
 $pReprepare = sp + 16|0;
 $iCol = sp + 12|0;
 $pPrefix = sp + 8|0;
 $v = sp + 4|0;
 $r1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $ppPrefix;
 HEAP32[$4>>2] = $pisComplete;
 HEAP32[$5>>2] = $pnoCase;
 HEAP32[$z>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 HEAP32[$pVal>>2] = 0;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$5>>2]|0;
 $11 = (_sqlite3IsLikeFunction($8,$9,$10,$wc)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  HEAP32[$0>>2] = 0;
  $167 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($167|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pList>>2] = $15;
 $16 = HEAP32[$pList>>2]|0;
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pLeft>>2] = $20;
 $21 = HEAP32[$pLeft>>2]|0;
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(152);
 if (!($24)) {
  $25 = HEAP32[$pLeft>>2]|0;
  $26 = (_sqlite3ExprAffinity($25)|0);
  $27 = $26 << 24 >> 24;
  $28 = ($27|0)!=(66);
  if (!($28)) {
   $29 = HEAP32[$pLeft>>2]|0;
   $30 = ((($29)) + 44|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ((($31)) + 42|0);
   $33 = HEAP8[$32>>0]|0;
   $34 = $33&255;
   $35 = $34 & 16;
   $36 = ($35|0)!=(0);
   if (!($36)) {
    $37 = HEAP32[$pList>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$39>>2]|0;
    $41 = (_sqlite3ExprSkipCollate($40)|0);
    HEAP32[$pRight>>2] = $41;
    $42 = HEAP32[$pRight>>2]|0;
    $43 = HEAP8[$42>>0]|0;
    $44 = $43&255;
    HEAP32[$op>>2] = $44;
    $45 = HEAP32[$op>>2]|0;
    $46 = ($45|0)==(135);
    if ($46) {
     $47 = HEAP32[$1>>2]|0;
     $48 = ((($47)) + 432|0);
     $49 = HEAP32[$48>>2]|0;
     HEAP32[$pReprepare>>2] = $49;
     $50 = HEAP32[$pRight>>2]|0;
     $51 = ((($50)) + 32|0);
     $52 = HEAP16[$51>>1]|0;
     $53 = $52 << 16 >> 16;
     HEAP32[$iCol>>2] = $53;
     $54 = HEAP32[$pReprepare>>2]|0;
     $55 = HEAP32[$iCol>>2]|0;
     $56 = (_sqlite3VdbeGetBoundValue($54,$55,65)|0);
     HEAP32[$pVal>>2] = $56;
     $57 = HEAP32[$pVal>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$pVal>>2]|0;
      $60 = (_sqlite3_value_type($59)|0);
      $61 = ($60|0)==(3);
      if ($61) {
       $62 = HEAP32[$pVal>>2]|0;
       $63 = (_sqlite3_value_text($62)|0);
       HEAP32[$z>>2] = $63;
      }
     }
     $64 = HEAP32[$1>>2]|0;
     $65 = ((($64)) + 8|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$iCol>>2]|0;
     _sqlite3VdbeSetVarmask($66,$67);
    } else {
     $68 = HEAP32[$op>>2]|0;
     $69 = ($68|0)==(97);
     if ($69) {
      $70 = HEAP32[$pRight>>2]|0;
      $71 = ((($70)) + 8|0);
      $72 = HEAP32[$71>>2]|0;
      HEAP32[$z>>2] = $72;
     }
    }
    $73 = HEAP32[$z>>2]|0;
    $74 = ($73|0)!=(0|0);
    do {
     if ($74) {
      HEAP32[$cnt>>2] = 0;
      while(1) {
       $75 = HEAP32[$cnt>>2]|0;
       $76 = HEAP32[$z>>2]|0;
       $77 = (($76) + ($75)|0);
       $78 = HEAP8[$77>>0]|0;
       $79 = $78 << 24 >> 24;
       HEAP32[$c>>2] = $79;
       $80 = ($79|0)!=(0);
       if ($80) {
        $81 = HEAP32[$c>>2]|0;
        $82 = HEAP8[$wc>>0]|0;
        $83 = $82 << 24 >> 24;
        $84 = ($81|0)!=($83|0);
        if ($84) {
         $85 = HEAP32[$c>>2]|0;
         $86 = ((($wc)) + 1|0);
         $87 = HEAP8[$86>>0]|0;
         $88 = $87 << 24 >> 24;
         $89 = ($85|0)!=($88|0);
         if ($89) {
          $90 = HEAP32[$c>>2]|0;
          $91 = ((($wc)) + 2|0);
          $92 = HEAP8[$91>>0]|0;
          $93 = $92 << 24 >> 24;
          $94 = ($90|0)!=($93|0);
          $168 = $94;
         } else {
          $168 = 0;
         }
        } else {
         $168 = 0;
        }
       } else {
        $168 = 0;
       }
       $95 = HEAP32[$cnt>>2]|0;
       if (!($168)) {
        break;
       }
       $96 = (($95) + 1)|0;
       HEAP32[$cnt>>2] = $96;
      }
      $97 = ($95|0)!=(0);
      if ($97) {
       $98 = HEAP32[$cnt>>2]|0;
       $99 = (($98) - 1)|0;
       $100 = HEAP32[$z>>2]|0;
       $101 = (($100) + ($99)|0);
       $102 = HEAP8[$101>>0]|0;
       $103 = $102&255;
       $104 = (255)!=($103|0);
       if ($104) {
        $105 = HEAP32[$c>>2]|0;
        $106 = HEAP8[$wc>>0]|0;
        $107 = $106 << 24 >> 24;
        $108 = ($105|0)==($107|0);
        if ($108) {
         $109 = HEAP32[$cnt>>2]|0;
         $110 = (($109) + 1)|0;
         $111 = HEAP32[$z>>2]|0;
         $112 = (($111) + ($110)|0);
         $113 = HEAP8[$112>>0]|0;
         $114 = $113 << 24 >> 24;
         $115 = ($114|0)==(0);
         $117 = $115;
        } else {
         $117 = 0;
        }
        $116 = $117&1;
        $118 = HEAP32[$4>>2]|0;
        HEAP32[$118>>2] = $116;
        $119 = HEAP32[$db>>2]|0;
        $120 = HEAP32[$z>>2]|0;
        $121 = (_sqlite3Expr($119,97,$120)|0);
        HEAP32[$pPrefix>>2] = $121;
        $122 = HEAP32[$pPrefix>>2]|0;
        $123 = ($122|0)!=(0|0);
        if ($123) {
         $124 = HEAP32[$cnt>>2]|0;
         $125 = HEAP32[$pPrefix>>2]|0;
         $126 = ((($125)) + 8|0);
         $127 = HEAP32[$126>>2]|0;
         $128 = (($127) + ($124)|0);
         HEAP8[$128>>0] = 0;
        }
        $129 = HEAP32[$pPrefix>>2]|0;
        $130 = HEAP32[$3>>2]|0;
        HEAP32[$130>>2] = $129;
        $131 = HEAP32[$op>>2]|0;
        $132 = ($131|0)==(135);
        if (!($132)) {
         break;
        }
        $133 = HEAP32[$1>>2]|0;
        $134 = ((($133)) + 8|0);
        $135 = HEAP32[$134>>2]|0;
        HEAP32[$v>>2] = $135;
        $136 = HEAP32[$v>>2]|0;
        $137 = HEAP32[$pRight>>2]|0;
        $138 = ((($137)) + 32|0);
        $139 = HEAP16[$138>>1]|0;
        $140 = $139 << 16 >> 16;
        _sqlite3VdbeSetVarmask($136,$140);
        $141 = HEAP32[$4>>2]|0;
        $142 = HEAP32[$141>>2]|0;
        $143 = ($142|0)!=(0);
        if (!($143)) {
         break;
        }
        $144 = HEAP32[$pRight>>2]|0;
        $145 = ((($144)) + 8|0);
        $146 = HEAP32[$145>>2]|0;
        $147 = ((($146)) + 1|0);
        $148 = HEAP8[$147>>0]|0;
        $149 = $148 << 24 >> 24;
        $150 = ($149|0)!=(0);
        if (!($150)) {
         break;
        }
        $151 = HEAP32[$1>>2]|0;
        $152 = (_sqlite3GetTempReg($151)|0);
        HEAP32[$r1>>2] = $152;
        $153 = HEAP32[$1>>2]|0;
        $154 = HEAP32[$pRight>>2]|0;
        $155 = HEAP32[$r1>>2]|0;
        (_sqlite3ExprCodeTarget($153,$154,$155)|0);
        $156 = HEAP32[$v>>2]|0;
        $157 = HEAP32[$v>>2]|0;
        $158 = (_sqlite3VdbeCurrentAddr($157)|0);
        $159 = (($158) - 1)|0;
        _sqlite3VdbeChangeP3($156,$159,0);
        $160 = HEAP32[$1>>2]|0;
        $161 = HEAP32[$r1>>2]|0;
        _sqlite3ReleaseTempReg($160,$161);
        break;
       }
      }
      HEAP32[$z>>2] = 0;
     }
    } while(0);
    $162 = HEAP32[$z>>2]|0;
    $163 = ($162|0)!=(0|0);
    $164 = $163&1;
    HEAP32[$rc>>2] = $164;
    $165 = HEAP32[$pVal>>2]|0;
    _sqlite3ValueFree($165);
    $166 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $166;
    $167 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($167|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $167 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($167|0);
}
function _isMatchOfColumn($pExpr,$peOp2) {
 $pExpr = $pExpr|0;
 $peOp2 = $peOp2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pCol = 0, $pList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pList = sp + 8|0;
 $pCol = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pExpr;
 HEAP32[$2>>2] = $peOp2;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)!=(151);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pList>>2] = $9;
 $10 = HEAP32[$pList>>2]|0;
 $11 = ($10|0)==(0|0);
 if (!($11)) {
  $12 = HEAP32[$pList>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = ($13|0)!=(2);
  if (!($14)) {
   $15 = HEAP32[$pList>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ((($17)) + 20|0);
   $19 = HEAP32[$18>>2]|0;
   HEAP32[$pCol>>2] = $19;
   $20 = HEAP32[$pCol>>2]|0;
   $21 = HEAP8[$20>>0]|0;
   $22 = $21&255;
   $23 = ($22|0)!=(152);
   if (!($23)) {
    $24 = HEAP32[$pCol>>2]|0;
    $25 = ((($24)) + 44|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ((($26)) + 42|0);
    $28 = HEAP8[$27>>0]|0;
    $29 = $28&255;
    $30 = $29 & 16;
    $31 = ($30|0)!=(0);
    if ($31) {
     HEAP32[$i>>2] = 0;
     while(1) {
      $32 = HEAP32[$i>>2]|0;
      $33 = ($32|0)<(4);
      if (!($33)) {
       label = 14;
       break;
      }
      $34 = HEAP32[$1>>2]|0;
      $35 = ((($34)) + 8|0);
      $36 = HEAP32[$35>>2]|0;
      $37 = HEAP32[$i>>2]|0;
      $38 = (5444 + ($37<<3)|0);
      $39 = HEAP32[$38>>2]|0;
      $40 = (_sqlite3StrICmp($36,$39)|0);
      $41 = ($40|0)==(0);
      $42 = HEAP32[$i>>2]|0;
      if ($41) {
       label = 12;
       break;
      }
      $47 = (($42) + 1)|0;
      HEAP32[$i>>2] = $47;
     }
     if ((label|0) == 12) {
      $43 = (5444 + ($42<<3)|0);
      $44 = ((($43)) + 4|0);
      $45 = HEAP8[$44>>0]|0;
      $46 = HEAP32[$2>>2]|0;
      HEAP8[$46>>0] = $45;
      HEAP32[$0>>2] = 1;
      $48 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($48|0);
     }
     else if ((label|0) == 14) {
      HEAP32[$0>>2] = 0;
      $48 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($48|0);
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $48 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($48|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _sqlite3ExprForVectorField($pParse,$pVector,$iField) {
 $pParse = $pParse|0;
 $pVector = $pVector|0;
 $iField = $iField|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pVector;
 HEAP32[$2>>2] = $iField;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(119);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = (_sqlite3PExpr($7,159,0,0,0)|0);
  HEAP32[$pRet>>2] = $8;
  $9 = HEAP32[$pRet>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   $34 = HEAP32[$pRet>>2]|0;
   STACKTOP = sp;return ($34|0);
  }
  $11 = HEAP32[$2>>2]|0;
  $12 = $11&65535;
  $13 = HEAP32[$pRet>>2]|0;
  $14 = ((($13)) + 32|0);
  HEAP16[$14>>1] = $12;
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$pRet>>2]|0;
  $17 = ((($16)) + 12|0);
  HEAP32[$17>>2] = $15;
  $34 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $18 = HEAP32[$1>>2]|0;
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = ($20|0)==(158);
  if ($21) {
   $22 = HEAP32[$2>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 20|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = ((($25)) + 4|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + (($22*20)|0)|0);
   $29 = HEAP32[$28>>2]|0;
   HEAP32[$1>>2] = $29;
  }
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  $33 = (_sqlite3ExprDup($31,$32,0)|0);
  HEAP32[$pRet>>2] = $33;
  $34 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return (0)|0;
}
function _sqlite3IsLikeFunction($db,$pExpr,$pIsNocase,$aWc) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $pIsNocase = $pIsNocase|0;
 $aWc = $aWc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDef = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pDef = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $pIsNocase;
 HEAP32[$4>>2] = $aWc;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)!=(151);
 if (!($8)) {
  $9 = HEAP32[$2>>2]|0;
  $10 = ((($9)) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0|0);
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = ((($13)) + 20|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)!=(2);
   if (!($17)) {
    $18 = HEAP32[$1>>2]|0;
    $19 = HEAP32[$2>>2]|0;
    $20 = ((($19)) + 8|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = (_sqlite3FindFunction($18,$21,2,1,0)|0);
    HEAP32[$pDef>>2] = $22;
    $23 = HEAP32[$pDef>>2]|0;
    $24 = ($23|0)==(0|0);
    if (!($24)) {
     $25 = HEAP32[$pDef>>2]|0;
     $26 = ((($25)) + 2|0);
     $27 = HEAP16[$26>>1]|0;
     $28 = $27&65535;
     $29 = $28 & 4;
     $30 = ($29|0)==(0);
     if (!($30)) {
      $31 = HEAP32[$4>>2]|0;
      $32 = HEAP32[$pDef>>2]|0;
      $33 = ((($32)) + 4|0);
      $34 = HEAP32[$33>>2]|0;
      ;HEAP8[$31>>0]=HEAP8[$34>>0]|0;HEAP8[$31+1>>0]=HEAP8[$34+1>>0]|0;HEAP8[$31+2>>0]=HEAP8[$34+2>>0]|0;
      $35 = HEAP32[$pDef>>2]|0;
      $36 = ((($35)) + 2|0);
      $37 = HEAP16[$36>>1]|0;
      $38 = $37&65535;
      $39 = $38 & 8;
      $40 = ($39|0)==(0);
      $41 = $40&1;
      $42 = HEAP32[$3>>2]|0;
      HEAP32[$42>>2] = $41;
      HEAP32[$0>>2] = 1;
      $43 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($43|0);
     }
    }
    HEAP32[$0>>2] = 0;
    $43 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($43|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _sqlite3VdbeGetBoundValue($v,$iVar,$aff) {
 $v = $v|0;
 $iVar = $iVar|0;
 $aff = $aff|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMem = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $pMem = sp + 4|0;
 $pRet = sp;
 HEAP32[$1>>2] = $v;
 HEAP32[$2>>2] = $iVar;
 HEAP8[$3>>0] = $aff;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$2>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 116|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*40)|0)|0);
  HEAP32[$pMem>>2] = $11;
  $12 = HEAP32[$pMem>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14&65535;
  $16 = $15 & 1;
  $17 = (0)==($16|0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3ValueNew($19)|0);
   HEAP32[$pRet>>2] = $20;
   $21 = HEAP32[$pRet>>2]|0;
   $22 = ($21|0)!=(0|0);
   if ($22) {
    $23 = HEAP32[$pRet>>2]|0;
    $24 = HEAP32[$pMem>>2]|0;
    (_sqlite3VdbeMemCopy($23,$24)|0);
    $25 = HEAP32[$pRet>>2]|0;
    $26 = HEAP8[$3>>0]|0;
    _sqlite3ValueApplyAffinity($25,$26,1);
   }
   $27 = HEAP32[$pRet>>2]|0;
   HEAP32[$0>>2] = $27;
   $28 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($28|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _sqlite3VdbeSetVarmask($v,$iVar) {
 $v = $v|0;
 $iVar = $iVar|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iVar;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(32);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 196|0);
  HEAP32[$5>>2] = -1;
  STACKTOP = sp;return;
 } else {
  $6 = HEAP32[$1>>2]|0;
  $7 = (($6) - 1)|0;
  $8 = 1 << $7;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 196|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11 | $8;
  HEAP32[$10>>2] = $12;
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeChangeP3($p,$addr,$val) {
 $p = $p|0;
 $addr = $addr|0;
 $val = $val|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP32[$2>>2] = $val;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = $3;
 STACKTOP = sp;return;
}
function _whereNthSubterm($pTerm,$N) {
 $pTerm = $pTerm|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pTerm;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = ($6|0)!=(1024);
 $8 = HEAP32[$2>>2]|0;
 if ($7) {
  $9 = ($8|0)==(0);
  $10 = HEAP32[$1>>2]|0;
  $11 = $9 ? $10 : 0;
  HEAP32[$0>>2] = $11;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 28|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($8|0)<($16|0);
 if ($17) {
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 28|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 20|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($18*48)|0)|0);
  HEAP32[$0>>2] = $24;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _whereCombineDisjuncts($pSrc,$pWC,$pOne,$pTwo) {
 $pSrc = $pSrc|0;
 $pWC = $pWC|0;
 $pOne = $pOne|0;
 $pTwo = $pTwo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $db = 0, $eOp = 0, $idxNew = 0, $op = 0, $pNew = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $eOp = sp + 32|0;
 $db = sp + 12|0;
 $pNew = sp + 8|0;
 $op = sp + 4|0;
 $idxNew = sp;
 HEAP32[$0>>2] = $pSrc;
 HEAP32[$1>>2] = $pWC;
 HEAP32[$2>>2] = $pOne;
 HEAP32[$3>>2] = $pTwo;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $7 | $11;
 $13 = $12&65535;
 HEAP16[$eOp>>1] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 12|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 & 62;
 $19 = ($18|0)==(0);
 if ($19) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$3>>2]|0;
 $21 = ((($20)) + 12|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = $23 & 62;
 $25 = ($24|0)==(0);
 if ($25) {
  STACKTOP = sp;return;
 }
 $26 = HEAP16[$eOp>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 26;
 $29 = HEAP16[$eOp>>1]|0;
 $30 = $29&65535;
 $31 = ($28|0)!=($30|0);
 if ($31) {
  $32 = HEAP16[$eOp>>1]|0;
  $33 = $32&65535;
  $34 = $33 & 38;
  $35 = HEAP16[$eOp>>1]|0;
  $36 = $35&65535;
  $37 = ($34|0)!=($36|0);
  if ($37) {
   STACKTOP = sp;return;
  }
 }
 $38 = HEAP32[$2>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = ((($39)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = ((($43)) + 12|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = (_sqlite3ExprCompare($41,$45,-1)|0);
 $47 = ($46|0)!=(0);
 if ($47) {
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$2>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$3>>2]|0;
 $53 = HEAP32[$52>>2]|0;
 $54 = ((($53)) + 16|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (_sqlite3ExprCompare($51,$55,-1)|0);
 $57 = ($56|0)!=(0);
 if ($57) {
  STACKTOP = sp;return;
 }
 $58 = HEAP16[$eOp>>1]|0;
 $59 = $58&65535;
 $60 = HEAP16[$eOp>>1]|0;
 $61 = $60&65535;
 $62 = (($61) - 1)|0;
 $63 = $59 & $62;
 $64 = ($63|0)!=(0);
 do {
  if ($64) {
   $65 = HEAP16[$eOp>>1]|0;
   $66 = $65&65535;
   $67 = $66 & 24;
   $68 = ($67|0)!=(0);
   if ($68) {
    HEAP16[$eOp>>1] = 8;
    break;
   } else {
    HEAP16[$eOp>>1] = 32;
    break;
   }
  }
 } while(0);
 $69 = HEAP32[$1>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP32[$71>>2]|0;
 HEAP32[$db>>2] = $72;
 $73 = HEAP32[$db>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 $75 = HEAP32[$74>>2]|0;
 $76 = (_sqlite3ExprDup($73,$75,0)|0);
 HEAP32[$pNew>>2] = $76;
 $77 = HEAP32[$pNew>>2]|0;
 $78 = ($77|0)==(0|0);
 if ($78) {
  STACKTOP = sp;return;
 }
 HEAP32[$op>>2] = 37;
 while(1) {
  $79 = HEAP16[$eOp>>1]|0;
  $80 = $79&65535;
  $81 = HEAP32[$op>>2]|0;
  $82 = (($81) - 37)|0;
  $83 = 2 << $82;
  $84 = ($80|0)!=($83|0);
  $85 = HEAP32[$op>>2]|0;
  if (!($84)) {
   break;
  }
  $86 = (($85) + 1)|0;
  HEAP32[$op>>2] = $86;
 }
 $87 = $85&255;
 $88 = HEAP32[$pNew>>2]|0;
 HEAP8[$88>>0] = $87;
 $89 = HEAP32[$1>>2]|0;
 $90 = HEAP32[$pNew>>2]|0;
 $91 = (_whereClauseInsert($89,$90,3)|0);
 HEAP32[$idxNew>>2] = $91;
 $92 = HEAP32[$0>>2]|0;
 $93 = HEAP32[$1>>2]|0;
 $94 = HEAP32[$idxNew>>2]|0;
 _exprAnalyze($92,$93,$94);
 STACKTOP = sp;return;
}
function _checkConstraintUnchanged($pExpr,$aiChng,$chngRowid) {
 $pExpr = $pExpr|0;
 $aiChng = $aiChng|0;
 $chngRowid = $chngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pExpr;
 HEAP32[$1>>2] = $aiChng;
 HEAP32[$2>>2] = $chngRowid;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $3 = ((($w)) + 20|0);
 HEAP8[$3>>0] = 0;
 $4 = ((($w)) + 4|0);
 HEAP32[$4>>2] = 200;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($w)) + 24|0);
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 (_sqlite3WalkExpr($w,$7)|0);
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $15 = ((($w)) + 20|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  $18 = $17 ^ 1;
  $19 = $18&1;
  STACKTOP = sp;return ($19|0);
 }
 $10 = ((($w)) + 20|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & -3;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 $15 = ((($w)) + 20|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = ($16<<24>>24)!=(0);
 $18 = $17 ^ 1;
 $19 = $18&1;
 STACKTOP = sp;return ($19|0);
}
function _sqlite3RowidConstraint($pParse,$onError,$pTab) {
 $pParse = $pParse|0;
 $onError = $onError|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $zMsg = sp + 16|0;
 $rc = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $onError;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5 << 16 >> 16;
 $7 = ($6|0)>=(0);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 if ($7) {
  $12 = HEAP32[$2>>2]|0;
  $13 = ((($12)) + 32|0);
  $14 = HEAP16[$13>>1]|0;
  $15 = $14 << 16 >> 16;
  $16 = HEAP32[$2>>2]|0;
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$vararg_buffer>>2] = $11;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $20;
  $21 = (_sqlite3MPrintf($9,30161,$vararg_buffer)|0);
  HEAP32[$zMsg>>2] = $21;
  HEAP32[$rc>>2] = 1555;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$zMsg>>2]|0;
  _sqlite3HaltConstraint($23,$24,$25,$26,-1,2);
  STACKTOP = sp;return;
 } else {
  HEAP32[$vararg_buffer2>>2] = $11;
  $22 = (_sqlite3MPrintf($9,35219,$vararg_buffer2)|0);
  HEAP32[$zMsg>>2] = $22;
  HEAP32[$rc>>2] = 2579;
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$rc>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$zMsg>>2]|0;
  _sqlite3HaltConstraint($23,$24,$25,$26,-1,2);
  STACKTOP = sp;return;
 }
}
function _sqlite3FkRequired($pParse,$pTab,$aChange,$chngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $aChange = $aChange|0;
 $chngRowid = $chngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $aChange;
 HEAP32[$4>>2] = $chngRowid;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 24|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 524288;
 $10 = ($9|0)!=(0);
 L1: do {
  if ($10) {
   $11 = HEAP32[$3>>2]|0;
   $12 = ($11|0)!=(0|0);
   $13 = HEAP32[$2>>2]|0;
   if (!($12)) {
    $14 = (_sqlite3FkReferences($13)|0);
    $15 = ($14|0)!=(0|0);
    if ($15) {
     $21 = 1;
    } else {
     $16 = HEAP32[$2>>2]|0;
     $17 = ((($16)) + 16|0);
     $18 = HEAP32[$17>>2]|0;
     $19 = ($18|0)!=(0|0);
     $21 = $19;
    }
    $20 = $21&1;
    HEAP32[$0>>2] = $20;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
   $22 = ((($13)) + 16|0);
   $23 = HEAP32[$22>>2]|0;
   HEAP32[$p>>2] = $23;
   while(1) {
    $24 = HEAP32[$p>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$2>>2]|0;
    if (!($25)) {
     break;
    }
    $27 = HEAP32[$p>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = HEAP32[$4>>2]|0;
    $30 = (_fkChildIsModified($26,$27,$28,$29)|0);
    $31 = ($30|0)!=(0);
    if ($31) {
     label = 9;
     break;
    }
    $32 = HEAP32[$p>>2]|0;
    $33 = ((($32)) + 4|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$p>>2] = $34;
   }
   if ((label|0) == 9) {
    HEAP32[$0>>2] = 1;
    $47 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($47|0);
   }
   $35 = (_sqlite3FkReferences($26)|0);
   HEAP32[$p>>2] = $35;
   while(1) {
    $36 = HEAP32[$p>>2]|0;
    $37 = ($36|0)!=(0|0);
    if (!($37)) {
     break L1;
    }
    $38 = HEAP32[$2>>2]|0;
    $39 = HEAP32[$p>>2]|0;
    $40 = HEAP32[$3>>2]|0;
    $41 = HEAP32[$4>>2]|0;
    $42 = (_fkParentIsModified($38,$39,$40,$41)|0);
    $43 = ($42|0)!=(0);
    if ($43) {
     break;
    }
    $44 = HEAP32[$p>>2]|0;
    $45 = ((($44)) + 12|0);
    $46 = HEAP32[$45>>2]|0;
    HEAP32[$p>>2] = $46;
   }
   HEAP32[$0>>2] = 1;
   $47 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($47|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _sqlite3MultiWrite($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pToplevel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pToplevel = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 124|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = ((($5)) + 124|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = $7;
 } else {
  $8 = $5;
 }
 HEAP32[$pToplevel>>2] = $8;
 $9 = HEAP32[$pToplevel>>2]|0;
 $10 = ((($9)) + 20|0);
 HEAP8[$10>>0] = 1;
 STACKTOP = sp;return;
}
function _sqlite3GenerateRowDelete($pParse,$pTab,$pTrigger,$iDataCur,$iIdxCur,$iPk,$nPk,$count,$onconf,$eMode,$iIdxNoSeek) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pTrigger = $pTrigger|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $iPk = $iPk|0;
 $nPk = $nPk|0;
 $count = $count|0;
 $onconf = $onconf|0;
 $eMode = $eMode|0;
 $iIdxNoSeek = $iIdxNoSeek|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $addrStart = 0, $iCol = 0, $iLabel = 0, $iOld = 0, $mask = 0, $opSeek = 0, $or$cond = 0, $p5 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $6 = sp + 52|0;
 $7 = sp + 58|0;
 $8 = sp + 57|0;
 $9 = sp + 56|0;
 $10 = sp + 24|0;
 $v = sp + 20|0;
 $iOld = sp + 16|0;
 $iLabel = sp + 12|0;
 $opSeek = sp + 55|0;
 $mask = sp + 8|0;
 $iCol = sp + 4|0;
 $addrStart = sp;
 $p5 = sp + 54|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pTrigger;
 HEAP32[$3>>2] = $iDataCur;
 HEAP32[$4>>2] = $iIdxCur;
 HEAP32[$5>>2] = $iPk;
 HEAP16[$6>>1] = $nPk;
 HEAP8[$7>>0] = $count;
 HEAP8[$8>>0] = $onconf;
 HEAP8[$9>>0] = $eMode;
 HEAP32[$10>>2] = $iIdxNoSeek;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$v>>2] = $13;
 HEAP32[$iOld>>2] = 0;
 $14 = HEAP32[$v>>2]|0;
 $15 = (_sqlite3VdbeMakeLabel($14)|0);
 HEAP32[$iLabel>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 42|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = $19 & 32;
 $21 = ($20|0)==(0);
 $22 = $21 ? 33 : 30;
 $23 = $22&255;
 HEAP8[$opSeek>>0] = $23;
 $24 = HEAP8[$9>>0]|0;
 $25 = $24&255;
 $26 = ($25|0)==(0);
 if ($26) {
  $27 = HEAP32[$v>>2]|0;
  $28 = HEAP8[$opSeek>>0]|0;
  $29 = $28&255;
  $30 = HEAP32[$3>>2]|0;
  $31 = HEAP32[$iLabel>>2]|0;
  $32 = HEAP32[$5>>2]|0;
  $33 = HEAP16[$6>>1]|0;
  $34 = $33 << 16 >> 16;
  (_sqlite3VdbeAddOp4Int($27,$29,$30,$31,$32,$34)|0);
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 $37 = (_sqlite3FkRequired($35,$36,0,0)|0);
 $38 = ($37|0)!=(0);
 $39 = HEAP32[$2>>2]|0;
 $40 = ($39|0)!=(0|0);
 $or$cond = $38 | $40;
 if ($or$cond) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$2>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP8[$8>>0]|0;
  $45 = $44&255;
  $46 = (_sqlite3TriggerColmask($41,$42,0,0,3,$43,$45)|0);
  HEAP32[$mask>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = HEAP32[$1>>2]|0;
  $49 = (_sqlite3FkOldmask($47,$48)|0);
  $50 = HEAP32[$mask>>2]|0;
  $51 = $50 | $49;
  HEAP32[$mask>>2] = $51;
  $52 = HEAP32[$0>>2]|0;
  $53 = ((($52)) + 44|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$iOld>>2] = $55;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 34|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58 << 16 >> 16;
  $60 = (1 + ($59))|0;
  $61 = HEAP32[$0>>2]|0;
  $62 = ((($61)) + 44|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = (($63) + ($60))|0;
  HEAP32[$62>>2] = $64;
  $65 = HEAP32[$v>>2]|0;
  $66 = HEAP32[$5>>2]|0;
  $67 = HEAP32[$iOld>>2]|0;
  (_sqlite3VdbeAddOp2($65,84,$66,$67)|0);
  HEAP32[$iCol>>2] = 0;
  while(1) {
   $68 = HEAP32[$iCol>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 34|0);
   $71 = HEAP16[$70>>1]|0;
   $72 = $71 << 16 >> 16;
   $73 = ($68|0)<($72|0);
   if (!($73)) {
    break;
   }
   $74 = HEAP32[$mask>>2]|0;
   $75 = ($74|0)==(-1);
   if ($75) {
    label = 9;
   } else {
    $76 = HEAP32[$iCol>>2]|0;
    $77 = ($76|0)<=(31);
    if ($77) {
     $78 = HEAP32[$mask>>2]|0;
     $79 = HEAP32[$iCol>>2]|0;
     $80 = 1 << $79;
     $81 = $78 & $80;
     $82 = ($81|0)!=(0);
     if ($82) {
      label = 9;
     }
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $83 = HEAP32[$v>>2]|0;
    $84 = HEAP32[$1>>2]|0;
    $85 = HEAP32[$3>>2]|0;
    $86 = HEAP32[$iCol>>2]|0;
    $87 = HEAP32[$iOld>>2]|0;
    $88 = HEAP32[$iCol>>2]|0;
    $89 = (($87) + ($88))|0;
    $90 = (($89) + 1)|0;
    _sqlite3ExprCodeGetColumnOfTable($83,$84,$85,$86,$90);
   }
   $91 = HEAP32[$iCol>>2]|0;
   $92 = (($91) + 1)|0;
   HEAP32[$iCol>>2] = $92;
  }
  $93 = HEAP32[$v>>2]|0;
  $94 = (_sqlite3VdbeCurrentAddr($93)|0);
  HEAP32[$addrStart>>2] = $94;
  $95 = HEAP32[$0>>2]|0;
  $96 = HEAP32[$2>>2]|0;
  $97 = HEAP32[$1>>2]|0;
  $98 = HEAP32[$iOld>>2]|0;
  $99 = HEAP8[$8>>0]|0;
  $100 = $99&255;
  $101 = HEAP32[$iLabel>>2]|0;
  _sqlite3CodeRowTrigger($95,$96,109,0,1,$97,$98,$100,$101);
  $102 = HEAP32[$addrStart>>2]|0;
  $103 = HEAP32[$v>>2]|0;
  $104 = (_sqlite3VdbeCurrentAddr($103)|0);
  $105 = ($102|0)<($104|0);
  if ($105) {
   $106 = HEAP32[$v>>2]|0;
   $107 = HEAP8[$opSeek>>0]|0;
   $108 = $107&255;
   $109 = HEAP32[$3>>2]|0;
   $110 = HEAP32[$iLabel>>2]|0;
   $111 = HEAP32[$5>>2]|0;
   $112 = HEAP16[$6>>1]|0;
   $113 = $112 << 16 >> 16;
   (_sqlite3VdbeAddOp4Int($106,$108,$109,$110,$111,$113)|0);
  }
  $114 = HEAP32[$0>>2]|0;
  $115 = HEAP32[$1>>2]|0;
  $116 = HEAP32[$iOld>>2]|0;
  _sqlite3FkCheck($114,$115,$116,0,0,0);
 }
 $117 = HEAP32[$1>>2]|0;
 $118 = ((($117)) + 12|0);
 $119 = HEAP32[$118>>2]|0;
 $120 = ($119|0)==(0|0);
 if (!($120)) {
  $151 = HEAP32[$0>>2]|0;
  $152 = HEAP32[$1>>2]|0;
  $153 = HEAP32[$iOld>>2]|0;
  _sqlite3FkActions($151,$152,0,$153,0,0);
  $154 = HEAP32[$0>>2]|0;
  $155 = HEAP32[$2>>2]|0;
  $156 = HEAP32[$1>>2]|0;
  $157 = HEAP32[$iOld>>2]|0;
  $158 = HEAP8[$8>>0]|0;
  $159 = $158&255;
  $160 = HEAP32[$iLabel>>2]|0;
  _sqlite3CodeRowTrigger($154,$155,109,0,2,$156,$157,$159,$160);
  $161 = HEAP32[$v>>2]|0;
  $162 = HEAP32[$iLabel>>2]|0;
  _sqlite3VdbeResolveLabel($161,$162);
  STACKTOP = sp;return;
 }
 HEAP8[$p5>>0] = 0;
 $121 = HEAP32[$0>>2]|0;
 $122 = HEAP32[$1>>2]|0;
 $123 = HEAP32[$3>>2]|0;
 $124 = HEAP32[$4>>2]|0;
 $125 = HEAP32[$10>>2]|0;
 _sqlite3GenerateRowIndexDelete($121,$122,$123,$124,0,$125);
 $126 = HEAP32[$v>>2]|0;
 $127 = HEAP32[$3>>2]|0;
 $128 = HEAP8[$7>>0]|0;
 $129 = $128&255;
 $130 = ($129|0)!=(0);
 $131 = $130 ? 1 : 0;
 (_sqlite3VdbeAddOp2($126,117,$127,$131)|0);
 $132 = HEAP32[$v>>2]|0;
 $133 = HEAP32[$1>>2]|0;
 _sqlite3VdbeChangeP4($132,-1,$133,-20);
 $134 = HEAP8[$9>>0]|0;
 $135 = $134&255;
 $136 = ($135|0)!=(0);
 if ($136) {
  $137 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($137,4);
 }
 $138 = HEAP32[$10>>2]|0;
 $139 = ($138|0)>=(0);
 if ($139) {
  $140 = HEAP32[$v>>2]|0;
  $141 = HEAP32[$10>>2]|0;
  (_sqlite3VdbeAddOp1($140,117,$141)|0);
 }
 $142 = HEAP8[$9>>0]|0;
 $143 = $142&255;
 $144 = ($143|0)==(2);
 if ($144) {
  $145 = HEAP8[$p5>>0]|0;
  $146 = $145&255;
  $147 = $146 | 2;
  $148 = $147&255;
  HEAP8[$p5>>0] = $148;
 }
 $149 = HEAP32[$v>>2]|0;
 $150 = HEAP8[$p5>>0]|0;
 _sqlite3VdbeChangeP5($149,$150);
 $151 = HEAP32[$0>>2]|0;
 $152 = HEAP32[$1>>2]|0;
 $153 = HEAP32[$iOld>>2]|0;
 _sqlite3FkActions($151,$152,0,$153,0,0);
 $154 = HEAP32[$0>>2]|0;
 $155 = HEAP32[$2>>2]|0;
 $156 = HEAP32[$1>>2]|0;
 $157 = HEAP32[$iOld>>2]|0;
 $158 = HEAP8[$8>>0]|0;
 $159 = $158&255;
 $160 = HEAP32[$iLabel>>2]|0;
 _sqlite3CodeRowTrigger($154,$155,109,0,2,$156,$157,$159,$160);
 $161 = HEAP32[$v>>2]|0;
 $162 = HEAP32[$iLabel>>2]|0;
 _sqlite3VdbeResolveLabel($161,$162);
 STACKTOP = sp;return;
}
function _sqlite3GenerateRowIndexDelete($pParse,$pTab,$iDataCur,$iIdxCur,$aRegIdx,$iIdxNoSeek) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDataCur = $iDataCur|0;
 $iIdxCur = $iIdxCur|0;
 $aRegIdx = $aRegIdx|0;
 $iIdxNoSeek = $iIdxNoSeek|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iPartIdxLabel = 0, $pIdx = 0, $pPk = 0, $pPrior = 0, $r1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $i = sp + 24|0;
 $r1 = sp + 20|0;
 $iPartIdxLabel = sp + 16|0;
 $pIdx = sp + 12|0;
 $pPrior = sp + 8|0;
 $v = sp + 4|0;
 $pPk = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDataCur;
 HEAP32[$3>>2] = $iIdxCur;
 HEAP32[$4>>2] = $aRegIdx;
 HEAP32[$5>>2] = $iIdxNoSeek;
 HEAP32[$r1>>2] = -1;
 HEAP32[$pPrior>>2] = 0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 42|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 & 32;
 $14 = ($13|0)==(0);
 if ($14) {
  $17 = 0;
 } else {
  $15 = HEAP32[$1>>2]|0;
  $16 = (_sqlite3PrimaryKeyIndex($15)|0);
  $17 = $16;
 }
 HEAP32[$pPk>>2] = $17;
 HEAP32[$i>>2] = 0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pIdx>>2] = $20;
 while(1) {
  $21 = HEAP32[$pIdx>>2]|0;
  $22 = ($21|0)!=(0|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$4>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$i>>2]|0;
   $26 = HEAP32[$4>>2]|0;
   $27 = (($26) + ($25<<2)|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ($28|0)==(0);
   if (!($29)) {
    label = 7;
   }
  } else {
   label = 7;
  }
  if ((label|0) == 7) {
   label = 0;
   $30 = HEAP32[$pIdx>>2]|0;
   $31 = HEAP32[$pPk>>2]|0;
   $32 = ($30|0)==($31|0);
   if (!($32)) {
    $33 = HEAP32[$3>>2]|0;
    $34 = HEAP32[$i>>2]|0;
    $35 = (($33) + ($34))|0;
    $36 = HEAP32[$5>>2]|0;
    $37 = ($35|0)==($36|0);
    if (!($37)) {
     $38 = HEAP32[$0>>2]|0;
     $39 = HEAP32[$pIdx>>2]|0;
     $40 = HEAP32[$2>>2]|0;
     $41 = HEAP32[$pPrior>>2]|0;
     $42 = HEAP32[$r1>>2]|0;
     $43 = (_sqlite3GenerateIndexKey($38,$39,$40,0,1,$iPartIdxLabel,$41,$42)|0);
     HEAP32[$r1>>2] = $43;
     $44 = HEAP32[$v>>2]|0;
     $45 = HEAP32[$3>>2]|0;
     $46 = HEAP32[$i>>2]|0;
     $47 = (($45) + ($46))|0;
     $48 = HEAP32[$r1>>2]|0;
     $49 = HEAP32[$pIdx>>2]|0;
     $50 = ((($49)) + 55|0);
     $51 = HEAP8[$50>>0]|0;
     $52 = ($51&255) >>> 3;
     $53 = $52 & 1;
     $54 = $53&255;
     $55 = ($54|0)!=(0);
     $56 = HEAP32[$pIdx>>2]|0;
     if ($55) {
      $57 = ((($56)) + 50|0);
      $58 = HEAP16[$57>>1]|0;
      $$sink = $58;
     } else {
      $59 = ((($56)) + 52|0);
      $60 = HEAP16[$59>>1]|0;
      $$sink = $60;
     }
     $61 = $$sink&65535;
     (_sqlite3VdbeAddOp3($44,127,$47,$48,$61)|0);
     $62 = HEAP32[$0>>2]|0;
     $63 = HEAP32[$iPartIdxLabel>>2]|0;
     _sqlite3ResolvePartIdxLabel($62,$63);
     $64 = HEAP32[$pIdx>>2]|0;
     HEAP32[$pPrior>>2] = $64;
    }
   }
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = (($65) + 1)|0;
  HEAP32[$i>>2] = $66;
  $67 = HEAP32[$pIdx>>2]|0;
  $68 = ((($67)) + 20|0);
  $69 = HEAP32[$68>>2]|0;
  HEAP32[$pIdx>>2] = $69;
 }
 STACKTOP = sp;return;
}
function _sqlite3TriggerColmask($pParse,$pTrigger,$pChanges,$isNew,$tr_tm,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pChanges = $pChanges|0;
 $isNew = $isNew|0;
 $tr_tm = $tr_tm|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $mask = 0, $op = 0, $p = 0, $pPrg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $5 = sp + 20|0;
 $6 = sp + 16|0;
 $op = sp + 12|0;
 $mask = sp + 8|0;
 $p = sp + 4|0;
 $pPrg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $isNew;
 HEAP32[$4>>2] = $tr_tm;
 HEAP32[$5>>2] = $pTab;
 HEAP32[$6>>2] = $orconf;
 $7 = HEAP32[$2>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8 ? 110 : 109;
 HEAP32[$op>>2] = $9;
 HEAP32[$mask>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 8|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = HEAP32[$op>>2]|0;
  $18 = ($16|0)==($17|0);
  if ($18) {
   $19 = HEAP32[$4>>2]|0;
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 9|0);
   $22 = HEAP8[$21>>0]|0;
   $23 = $22&255;
   $24 = $19 & $23;
   $25 = ($24|0)!=(0);
   if ($25) {
    $26 = HEAP32[$p>>2]|0;
    $27 = ((($26)) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$2>>2]|0;
    $30 = (_checkColumnOverlap($28,$29)|0);
    $31 = ($30|0)!=(0);
    if ($31) {
     $32 = HEAP32[$0>>2]|0;
     $33 = HEAP32[$p>>2]|0;
     $34 = HEAP32[$5>>2]|0;
     $35 = HEAP32[$6>>2]|0;
     $36 = (_getRowTrigger($32,$33,$34,$35)|0);
     HEAP32[$pPrg>>2] = $36;
     $37 = HEAP32[$pPrg>>2]|0;
     $38 = ($37|0)!=(0|0);
     if ($38) {
      $39 = HEAP32[$3>>2]|0;
      $40 = HEAP32[$pPrg>>2]|0;
      $41 = ((($40)) + 16|0);
      $42 = (($41) + ($39<<2)|0);
      $43 = HEAP32[$42>>2]|0;
      $44 = HEAP32[$mask>>2]|0;
      $45 = $44 | $43;
      HEAP32[$mask>>2] = $45;
     }
    }
   }
  }
  $46 = HEAP32[$p>>2]|0;
  $47 = ((($46)) + 32|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$p>>2] = $48;
 }
 $49 = HEAP32[$mask>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _sqlite3FkOldmask($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $i = 0, $mask = 0;
 var $p = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $mask = sp + 12|0;
 $p = sp + 8|0;
 $i = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$mask>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 524288;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $77 = HEAP32[$mask>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   break;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $13 = HEAP32[$i>>2]|0;
   $14 = HEAP32[$p>>2]|0;
   $15 = ((($14)) + 20|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($13|0)<($16|0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$i>>2]|0;
   $19 = HEAP32[$p>>2]|0;
   $20 = ((($19)) + 36|0);
   $21 = (($20) + ($18<<3)|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)>(31);
   if ($23) {
    $32 = -1;
   } else {
    $24 = HEAP32[$i>>2]|0;
    $25 = HEAP32[$p>>2]|0;
    $26 = ((($25)) + 36|0);
    $27 = (($26) + ($24<<3)|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = 1 << $28;
    $32 = $29;
   }
   $30 = HEAP32[$mask>>2]|0;
   $31 = $30 | $32;
   HEAP32[$mask>>2] = $31;
   $33 = HEAP32[$i>>2]|0;
   $34 = (($33) + 1)|0;
   HEAP32[$i>>2] = $34;
  }
  $35 = HEAP32[$p>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$p>>2] = $37;
 }
 $38 = HEAP32[$1>>2]|0;
 $39 = (_sqlite3FkReferences($38)|0);
 HEAP32[$p>>2] = $39;
 while(1) {
  $40 = HEAP32[$p>>2]|0;
  $41 = ($40|0)!=(0|0);
  if (!($41)) {
   break;
  }
  HEAP32[$pIdx>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$p>>2]|0;
  (_sqlite3FkLocateIndex($42,$43,$44,$pIdx,0)|0);
  $45 = HEAP32[$pIdx>>2]|0;
  $46 = ($45|0)!=(0|0);
  L18: do {
   if ($46) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $47 = HEAP32[$i>>2]|0;
     $48 = HEAP32[$pIdx>>2]|0;
     $49 = ((($48)) + 50|0);
     $50 = HEAP16[$49>>1]|0;
     $51 = $50&65535;
     $52 = ($47|0)<($51|0);
     if (!($52)) {
      break L18;
     }
     $53 = HEAP32[$i>>2]|0;
     $54 = HEAP32[$pIdx>>2]|0;
     $55 = ((($54)) + 4|0);
     $56 = HEAP32[$55>>2]|0;
     $57 = (($56) + ($53<<1)|0);
     $58 = HEAP16[$57>>1]|0;
     $59 = $58 << 16 >> 16;
     $60 = ($59|0)>(31);
     if ($60) {
      $71 = -1;
     } else {
      $61 = HEAP32[$i>>2]|0;
      $62 = HEAP32[$pIdx>>2]|0;
      $63 = ((($62)) + 4|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = (($64) + ($61<<1)|0);
      $66 = HEAP16[$65>>1]|0;
      $67 = $66 << 16 >> 16;
      $68 = 1 << $67;
      $71 = $68;
     }
     $69 = HEAP32[$mask>>2]|0;
     $70 = $69 | $71;
     HEAP32[$mask>>2] = $70;
     $72 = HEAP32[$i>>2]|0;
     $73 = (($72) + 1)|0;
     HEAP32[$i>>2] = $73;
    }
   }
  } while(0);
  $74 = HEAP32[$p>>2]|0;
  $75 = ((($74)) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  HEAP32[$p>>2] = $76;
 }
 $77 = HEAP32[$mask>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _sqlite3FkActions($pParse,$pTab,$pChanges,$regOld,$aChange,$bChngRowid) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pChanges = $pChanges|0;
 $regOld = $regOld|0;
 $aChange = $aChange|0;
 $bChngRowid = $bChngRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAct = 0, $pFKey = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $pFKey = sp + 4|0;
 $pAct = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pChanges;
 HEAP32[$3>>2] = $regOld;
 HEAP32[$4>>2] = $aChange;
 HEAP32[$5>>2] = $bChngRowid;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 24|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 524288;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = (_sqlite3FkReferences($12)|0);
 HEAP32[$pFKey>>2] = $13;
 while(1) {
  $14 = HEAP32[$pFKey>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$4>>2]|0;
  $17 = ($16|0)==(0|0);
  if ($17) {
   label = 6;
  } else {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$pFKey>>2]|0;
   $20 = HEAP32[$4>>2]|0;
   $21 = HEAP32[$5>>2]|0;
   $22 = (_fkParentIsModified($18,$19,$20,$21)|0);
   $23 = ($22|0)!=(0);
   if ($23) {
    label = 6;
   }
  }
  if ((label|0) == 6) {
   label = 0;
   $24 = HEAP32[$0>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = HEAP32[$pFKey>>2]|0;
   $27 = HEAP32[$2>>2]|0;
   $28 = (_fkActionTrigger($24,$25,$26,$27)|0);
   HEAP32[$pAct>>2] = $28;
   $29 = HEAP32[$pAct>>2]|0;
   $30 = ($29|0)!=(0|0);
   if ($30) {
    $31 = HEAP32[$0>>2]|0;
    $32 = HEAP32[$pAct>>2]|0;
    $33 = HEAP32[$1>>2]|0;
    $34 = HEAP32[$3>>2]|0;
    _sqlite3CodeRowTriggerDirect($31,$32,$33,$34,2,0);
   }
  }
  $35 = HEAP32[$pFKey>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$pFKey>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _fkActionTrigger($pParse,$pTab,$pFKey,$pChanges) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pFKey = $pFKey|0;
 $pChanges = $pChanges|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $action = 0, $aiCol = 0, $db = 0;
 var $i = 0, $iAction = 0, $iFromCol = 0, $nFrom = 0, $or$cond = 0, $or$cond3 = 0, $pDflt = 0, $pEq = 0, $pIdx = 0, $pList = 0, $pNew = 0, $pRaise = 0, $pSelect = 0, $pStep = 0, $pTrigger = 0, $pWhen = 0, $pWhere = 0, $tFrom = 0, $tFromCol = 0, $tNew = 0;
 var $tOld = 0, $tToCol = 0, $zFrom = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 136|0;
 $1 = sp + 132|0;
 $2 = sp + 128|0;
 $3 = sp + 124|0;
 $4 = sp + 120|0;
 $db = sp + 116|0;
 $action = sp + 112|0;
 $pTrigger = sp + 108|0;
 $iAction = sp + 104|0;
 $zFrom = sp + 100|0;
 $nFrom = sp + 96|0;
 $pIdx = sp + 92|0;
 $aiCol = sp + 88|0;
 $pStep = sp + 84|0;
 $pWhere = sp + 80|0;
 $pList = sp + 76|0;
 $pSelect = sp + 72|0;
 $i = sp + 68|0;
 $pWhen = sp + 64|0;
 $tOld = sp + 56|0;
 $tNew = sp + 48|0;
 $tFromCol = sp + 40|0;
 $tToCol = sp + 32|0;
 $iFromCol = sp + 28|0;
 $pEq = sp + 24|0;
 $pNew = sp + 20|0;
 $pDflt = sp + 16|0;
 $tFrom = sp + 8|0;
 $pRaise = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pFKey;
 HEAP32[$4>>2] = $pChanges;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8&1;
 HEAP32[$iAction>>2] = $9;
 $10 = HEAP32[$iAction>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = ((($11)) + 25|0);
 $13 = (($12) + ($10)|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 HEAP32[$action>>2] = $15;
 $16 = HEAP32[$action>>2]|0;
 $17 = ($16|0)==(6);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = ((($18)) + 24|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = $20 & 33554432;
  $22 = ($21|0)!=(0);
  if ($22) {
   HEAP32[$0>>2] = 0;
   $275 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($275|0);
  }
 }
 $23 = HEAP32[$iAction>>2]|0;
 $24 = HEAP32[$3>>2]|0;
 $25 = ((($24)) + 28|0);
 $26 = (($25) + ($23<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pTrigger>>2] = $27;
 $28 = HEAP32[$action>>2]|0;
 $29 = ($28|0)==(0);
 $30 = HEAP32[$pTrigger>>2]|0;
 $31 = ($30|0)!=(0|0);
 $or$cond = $29 | $31;
 if (!($or$cond)) {
  HEAP32[$pIdx>>2] = 0;
  HEAP32[$aiCol>>2] = 0;
  HEAP32[$pStep>>2] = 0;
  HEAP32[$pWhere>>2] = 0;
  HEAP32[$pList>>2] = 0;
  HEAP32[$pSelect>>2] = 0;
  HEAP32[$pWhen>>2] = 0;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$3>>2]|0;
  $35 = (_sqlite3FkLocateIndex($32,$33,$34,$pIdx,$aiCol)|0);
  $36 = ($35|0)!=(0);
  if ($36) {
   HEAP32[$0>>2] = 0;
   $275 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($275|0);
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $37 = HEAP32[$i>>2]|0;
   $38 = HEAP32[$3>>2]|0;
   $39 = ((($38)) + 20|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = ($37|0)<($40|0);
   if (!($41)) {
    break;
   }
   ;HEAP32[$tOld>>2]=HEAP32[5476>>2]|0;HEAP32[$tOld+4>>2]=HEAP32[5476+4>>2]|0;
   ;HEAP32[$tNew>>2]=HEAP32[5484>>2]|0;HEAP32[$tNew+4>>2]=HEAP32[5484+4>>2]|0;
   $42 = HEAP32[$aiCol>>2]|0;
   $43 = ($42|0)!=(0|0);
   if ($43) {
    $44 = HEAP32[$i>>2]|0;
    $45 = HEAP32[$aiCol>>2]|0;
    $46 = (($45) + ($44<<2)|0);
    $47 = HEAP32[$46>>2]|0;
    $51 = $47;
   } else {
    $48 = HEAP32[$3>>2]|0;
    $49 = ((($48)) + 36|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = $50;
   }
   HEAP32[$iFromCol>>2] = $51;
   $52 = HEAP32[$pIdx>>2]|0;
   $53 = ($52|0)!=(0|0);
   if ($53) {
    $54 = HEAP32[$i>>2]|0;
    $55 = HEAP32[$pIdx>>2]|0;
    $56 = ((($55)) + 4|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = (($57) + ($54<<1)|0);
    $59 = HEAP16[$58>>1]|0;
    $$sink = $59;
   } else {
    $60 = HEAP32[$2>>2]|0;
    $61 = ((($60)) + 32|0);
    $62 = HEAP16[$61>>1]|0;
    $$sink = $62;
   }
   $63 = $$sink << 16 >> 16;
   $64 = HEAP32[$2>>2]|0;
   $65 = ((($64)) + 4|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($66) + ($63<<4)|0);
   $68 = HEAP32[$67>>2]|0;
   _sqlite3TokenInit($tToCol,$68);
   $69 = HEAP32[$iFromCol>>2]|0;
   $70 = HEAP32[$3>>2]|0;
   $71 = HEAP32[$70>>2]|0;
   $72 = ((($71)) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($73) + ($69<<4)|0);
   $75 = HEAP32[$74>>2]|0;
   _sqlite3TokenInit($tFromCol,$75);
   $76 = HEAP32[$1>>2]|0;
   $77 = HEAP32[$1>>2]|0;
   $78 = HEAP32[$db>>2]|0;
   $79 = (_sqlite3ExprAlloc($78,55,$tOld,0)|0);
   $80 = HEAP32[$db>>2]|0;
   $81 = (_sqlite3ExprAlloc($80,55,$tToCol,0)|0);
   $82 = (_sqlite3PExpr($77,122,$79,$81,0)|0);
   $83 = HEAP32[$db>>2]|0;
   $84 = (_sqlite3ExprAlloc($83,55,$tFromCol,0)|0);
   $85 = (_sqlite3PExpr($76,37,$82,$84,0)|0);
   HEAP32[$pEq>>2] = $85;
   $86 = HEAP32[$db>>2]|0;
   $87 = HEAP32[$pWhere>>2]|0;
   $88 = HEAP32[$pEq>>2]|0;
   $89 = (_sqlite3ExprAnd($86,$87,$88)|0);
   HEAP32[$pWhere>>2] = $89;
   $90 = HEAP32[$4>>2]|0;
   $91 = ($90|0)!=(0|0);
   if ($91) {
    $92 = HEAP32[$1>>2]|0;
    $93 = HEAP32[$1>>2]|0;
    $94 = HEAP32[$db>>2]|0;
    $95 = (_sqlite3ExprAlloc($94,55,$tOld,0)|0);
    $96 = HEAP32[$db>>2]|0;
    $97 = (_sqlite3ExprAlloc($96,55,$tToCol,0)|0);
    $98 = (_sqlite3PExpr($93,122,$95,$97,0)|0);
    $99 = HEAP32[$1>>2]|0;
    $100 = HEAP32[$db>>2]|0;
    $101 = (_sqlite3ExprAlloc($100,55,$tNew,0)|0);
    $102 = HEAP32[$db>>2]|0;
    $103 = (_sqlite3ExprAlloc($102,55,$tToCol,0)|0);
    $104 = (_sqlite3PExpr($99,122,$101,$103,0)|0);
    $105 = (_sqlite3PExpr($92,29,$98,$104,0)|0);
    HEAP32[$pEq>>2] = $105;
    $106 = HEAP32[$db>>2]|0;
    $107 = HEAP32[$pWhen>>2]|0;
    $108 = HEAP32[$pEq>>2]|0;
    $109 = (_sqlite3ExprAnd($106,$107,$108)|0);
    HEAP32[$pWhen>>2] = $109;
   }
   $110 = HEAP32[$action>>2]|0;
   $111 = ($110|0)!=(6);
   if ($111) {
    $112 = HEAP32[$action>>2]|0;
    $113 = ($112|0)!=(9);
    $114 = HEAP32[$4>>2]|0;
    $115 = ($114|0)!=(0|0);
    $or$cond3 = $113 | $115;
    if ($or$cond3) {
     $116 = HEAP32[$action>>2]|0;
     $117 = ($116|0)==(9);
     do {
      if ($117) {
       $118 = HEAP32[$1>>2]|0;
       $119 = HEAP32[$db>>2]|0;
       $120 = (_sqlite3ExprAlloc($119,55,$tNew,0)|0);
       $121 = HEAP32[$db>>2]|0;
       $122 = (_sqlite3ExprAlloc($121,55,$tToCol,0)|0);
       $123 = (_sqlite3PExpr($118,122,$120,$122,0)|0);
       HEAP32[$pNew>>2] = $123;
      } else {
       $124 = HEAP32[$action>>2]|0;
       $125 = ($124|0)==(8);
       if (!($125)) {
        $140 = HEAP32[$db>>2]|0;
        $141 = (_sqlite3ExprAlloc($140,101,0,0)|0);
        HEAP32[$pNew>>2] = $141;
        break;
       }
       $126 = HEAP32[$iFromCol>>2]|0;
       $127 = HEAP32[$3>>2]|0;
       $128 = HEAP32[$127>>2]|0;
       $129 = ((($128)) + 4|0);
       $130 = HEAP32[$129>>2]|0;
       $131 = (($130) + ($126<<4)|0);
       $132 = ((($131)) + 4|0);
       $133 = HEAP32[$132>>2]|0;
       HEAP32[$pDflt>>2] = $133;
       $134 = HEAP32[$pDflt>>2]|0;
       $135 = ($134|0)!=(0|0);
       $136 = HEAP32[$db>>2]|0;
       if ($135) {
        $137 = HEAP32[$pDflt>>2]|0;
        $138 = (_sqlite3ExprDup($136,$137,0)|0);
        HEAP32[$pNew>>2] = $138;
        break;
       } else {
        $139 = (_sqlite3ExprAlloc($136,101,0,0)|0);
        HEAP32[$pNew>>2] = $139;
        break;
       }
      }
     } while(0);
     $142 = HEAP32[$1>>2]|0;
     $143 = HEAP32[$pList>>2]|0;
     $144 = HEAP32[$pNew>>2]|0;
     $145 = (_sqlite3ExprListAppend($142,$143,$144)|0);
     HEAP32[$pList>>2] = $145;
     $146 = HEAP32[$1>>2]|0;
     $147 = HEAP32[$pList>>2]|0;
     _sqlite3ExprListSetName($146,$147,$tFromCol,0);
    }
   }
   $148 = HEAP32[$i>>2]|0;
   $149 = (($148) + 1)|0;
   HEAP32[$i>>2] = $149;
  }
  $150 = HEAP32[$db>>2]|0;
  $151 = HEAP32[$aiCol>>2]|0;
  _sqlite3DbFree($150,$151);
  $152 = HEAP32[$3>>2]|0;
  $153 = HEAP32[$152>>2]|0;
  $154 = HEAP32[$153>>2]|0;
  HEAP32[$zFrom>>2] = $154;
  $155 = HEAP32[$zFrom>>2]|0;
  $156 = (_sqlite3Strlen30($155)|0);
  HEAP32[$nFrom>>2] = $156;
  $157 = HEAP32[$action>>2]|0;
  $158 = ($157|0)==(6);
  if ($158) {
   $159 = HEAP32[$zFrom>>2]|0;
   HEAP32[$tFrom>>2] = $159;
   $160 = HEAP32[$nFrom>>2]|0;
   $161 = ((($tFrom)) + 4|0);
   HEAP32[$161>>2] = $160;
   $162 = HEAP32[$db>>2]|0;
   $163 = (_sqlite3Expr($162,83,35189)|0);
   HEAP32[$pRaise>>2] = $163;
   $164 = HEAP32[$pRaise>>2]|0;
   $165 = ($164|0)!=(0|0);
   if ($165) {
    $166 = HEAP32[$pRaise>>2]|0;
    $167 = ((($166)) + 1|0);
    HEAP8[$167>>0] = 2;
   }
   $168 = HEAP32[$1>>2]|0;
   $169 = HEAP32[$1>>2]|0;
   $170 = HEAP32[$pRaise>>2]|0;
   $171 = (_sqlite3ExprListAppend($169,0,$170)|0);
   $172 = HEAP32[$db>>2]|0;
   $173 = (_sqlite3SrcListAppend($172,0,$tFrom,0)|0);
   $174 = HEAP32[$pWhere>>2]|0;
   $175 = (_sqlite3SelectNew($168,$171,$173,$174,0,0,0,0,0,0)|0);
   HEAP32[$pSelect>>2] = $175;
   HEAP32[$pWhere>>2] = 0;
  }
  $176 = HEAP32[$db>>2]|0;
  $177 = ((($176)) + 256|0);
  $178 = HEAP32[$177>>2]|0;
  $179 = (($178) + 1)|0;
  HEAP32[$177>>2] = $179;
  $180 = HEAP32[$db>>2]|0;
  $181 = HEAP32[$nFrom>>2]|0;
  $182 = (72 + ($181))|0;
  $183 = (($182) + 1)|0;
  $184 = (_sqlite3DbMallocZero($180,$183,0)|0);
  HEAP32[$pTrigger>>2] = $184;
  $185 = HEAP32[$pTrigger>>2]|0;
  $186 = ($185|0)!=(0|0);
  if ($186) {
   $187 = HEAP32[$pTrigger>>2]|0;
   $188 = ((($187)) + 36|0);
   $189 = HEAP32[$pTrigger>>2]|0;
   $190 = ((($189)) + 28|0);
   HEAP32[$190>>2] = $188;
   HEAP32[$pStep>>2] = $188;
   $191 = HEAP32[$pStep>>2]|0;
   $192 = ((($191)) + 36|0);
   $193 = HEAP32[$pStep>>2]|0;
   $194 = ((($193)) + 12|0);
   HEAP32[$194>>2] = $192;
   $195 = HEAP32[$pStep>>2]|0;
   $196 = ((($195)) + 12|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = HEAP32[$zFrom>>2]|0;
   $199 = HEAP32[$nFrom>>2]|0;
   _memcpy(($197|0),($198|0),($199|0))|0;
   $200 = HEAP32[$db>>2]|0;
   $201 = HEAP32[$pWhere>>2]|0;
   $202 = (_sqlite3ExprDup($200,$201,1)|0);
   $203 = HEAP32[$pStep>>2]|0;
   $204 = ((($203)) + 16|0);
   HEAP32[$204>>2] = $202;
   $205 = HEAP32[$db>>2]|0;
   $206 = HEAP32[$pList>>2]|0;
   $207 = (_sqlite3ExprListDup($205,$206,1)|0);
   $208 = HEAP32[$pStep>>2]|0;
   $209 = ((($208)) + 20|0);
   HEAP32[$209>>2] = $207;
   $210 = HEAP32[$db>>2]|0;
   $211 = HEAP32[$pSelect>>2]|0;
   $212 = (_sqlite3SelectDup($210,$211,1)|0);
   $213 = HEAP32[$pStep>>2]|0;
   $214 = ((($213)) + 8|0);
   HEAP32[$214>>2] = $212;
   $215 = HEAP32[$pWhen>>2]|0;
   $216 = ($215|0)!=(0|0);
   if ($216) {
    $217 = HEAP32[$1>>2]|0;
    $218 = HEAP32[$pWhen>>2]|0;
    $219 = (_sqlite3PExpr($217,19,$218,0,0)|0);
    HEAP32[$pWhen>>2] = $219;
    $220 = HEAP32[$db>>2]|0;
    $221 = HEAP32[$pWhen>>2]|0;
    $222 = (_sqlite3ExprDup($220,$221,1)|0);
    $223 = HEAP32[$pTrigger>>2]|0;
    $224 = ((($223)) + 12|0);
    HEAP32[$224>>2] = $222;
   }
  }
  $225 = HEAP32[$db>>2]|0;
  $226 = ((($225)) + 256|0);
  $227 = HEAP32[$226>>2]|0;
  $228 = (($227) + -1)|0;
  HEAP32[$226>>2] = $228;
  $229 = HEAP32[$db>>2]|0;
  $230 = HEAP32[$pWhere>>2]|0;
  _sqlite3ExprDelete($229,$230);
  $231 = HEAP32[$db>>2]|0;
  $232 = HEAP32[$pWhen>>2]|0;
  _sqlite3ExprDelete($231,$232);
  $233 = HEAP32[$db>>2]|0;
  $234 = HEAP32[$pList>>2]|0;
  _sqlite3ExprListDelete($233,$234);
  $235 = HEAP32[$db>>2]|0;
  $236 = HEAP32[$pSelect>>2]|0;
  _sqlite3SelectDelete($235,$236);
  $237 = HEAP32[$db>>2]|0;
  $238 = ((($237)) + 69|0);
  $239 = HEAP8[$238>>0]|0;
  $240 = $239&255;
  $241 = ($240|0)==(1);
  if ($241) {
   $242 = HEAP32[$db>>2]|0;
   $243 = HEAP32[$pTrigger>>2]|0;
   _fkTriggerDelete($242,$243);
   HEAP32[$0>>2] = 0;
   $275 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($275|0);
  }
  $244 = HEAP32[$action>>2]|0;
  switch ($244|0) {
  case 6:  {
   $245 = HEAP32[$pStep>>2]|0;
   HEAP8[$245>>0] = 119;
   break;
  }
  case 9:  {
   $246 = HEAP32[$4>>2]|0;
   $247 = ($246|0)!=(0|0);
   if ($247) {
    label = 41;
   } else {
    $248 = HEAP32[$pStep>>2]|0;
    HEAP8[$248>>0] = 109;
   }
   break;
  }
  default: {
   label = 41;
  }
  }
  if ((label|0) == 41) {
   $249 = HEAP32[$pStep>>2]|0;
   HEAP8[$249>>0] = 110;
  }
  $250 = HEAP32[$pTrigger>>2]|0;
  $251 = HEAP32[$pStep>>2]|0;
  $252 = ((($251)) + 4|0);
  HEAP32[$252>>2] = $250;
  $253 = HEAP32[$2>>2]|0;
  $254 = ((($253)) + 64|0);
  $255 = HEAP32[$254>>2]|0;
  $256 = HEAP32[$pTrigger>>2]|0;
  $257 = ((($256)) + 20|0);
  HEAP32[$257>>2] = $255;
  $258 = HEAP32[$2>>2]|0;
  $259 = ((($258)) + 64|0);
  $260 = HEAP32[$259>>2]|0;
  $261 = HEAP32[$pTrigger>>2]|0;
  $262 = ((($261)) + 24|0);
  HEAP32[$262>>2] = $260;
  $263 = HEAP32[$pTrigger>>2]|0;
  $264 = HEAP32[$iAction>>2]|0;
  $265 = HEAP32[$3>>2]|0;
  $266 = ((($265)) + 28|0);
  $267 = (($266) + ($264<<2)|0);
  HEAP32[$267>>2] = $263;
  $268 = HEAP32[$4>>2]|0;
  $269 = ($268|0)!=(0|0);
  $270 = $269 ? 110 : 109;
  $271 = $270&255;
  $272 = HEAP32[$pTrigger>>2]|0;
  $273 = ((($272)) + 8|0);
  HEAP8[$273>>0] = $271;
 }
 $274 = HEAP32[$pTrigger>>2]|0;
 HEAP32[$0>>2] = $274;
 $275 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($275|0);
}
function _sqlite3CodeRowTriggerDirect($pParse,$p,$pTab,$reg,$orconf,$ignoreJump) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pTab = $pTab|0;
 $reg = $reg|0;
 $orconf = $orconf|0;
 $ignoreJump = $ignoreJump|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bRecursive = 0;
 var $pPrg = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $5 = sp + 12|0;
 $v = sp + 8|0;
 $pPrg = sp + 4|0;
 $bRecursive = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $reg;
 HEAP32[$4>>2] = $orconf;
 HEAP32[$5>>2] = $ignoreJump;
 $6 = HEAP32[$0>>2]|0;
 $7 = (_sqlite3GetVdbe($6)|0);
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$4>>2]|0;
 $12 = (_getRowTrigger($8,$9,$10,$11)|0);
 HEAP32[$pPrg>>2] = $12;
 $13 = HEAP32[$pPrg>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = ((($19)) + 24|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = $21 & 262144;
  $23 = (0)==($22|0);
  $25 = $23;
 } else {
  $25 = 0;
 }
 $24 = $25&1;
 HEAP32[$bRecursive>>2] = $24;
 $26 = HEAP32[$v>>2]|0;
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$5>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 44|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + 1)|0;
 HEAP32[$30>>2] = $32;
 $33 = HEAP32[$pPrg>>2]|0;
 $34 = ((($33)) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 (_sqlite3VdbeAddOp4($26,64,$27,$28,$32,$35,-18)|0);
 $36 = HEAP32[$v>>2]|0;
 $37 = HEAP32[$bRecursive>>2]|0;
 $38 = $37&255;
 _sqlite3VdbeChangeP5($36,$38);
 STACKTOP = sp;return;
}
function _getRowTrigger($pParse,$pTrigger,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPrg = 0, $pRoot = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pRoot = sp + 4|0;
 $pPrg = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTrigger;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $orconf;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 124|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$0>>2]|0;
 if ($7) {
  $9 = ((($8)) + 124|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = $10;
 } else {
  $11 = $8;
 }
 HEAP32[$pRoot>>2] = $11;
 $12 = HEAP32[$pRoot>>2]|0;
 $13 = ((($12)) + 468|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pPrg>>2] = $14;
 while(1) {
  $15 = HEAP32[$pPrg>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pPrg>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ($18|0)!=($19|0);
  if (!($20)) {
   $21 = HEAP32[$pPrg>>2]|0;
   $22 = ((($21)) + 12|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = HEAP32[$3>>2]|0;
   $25 = ($23|0)!=($24|0);
   if (!($25)) {
    break;
   }
  }
  $26 = HEAP32[$pPrg>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pPrg>>2] = $28;
 }
 $29 = HEAP32[$pPrg>>2]|0;
 $30 = ($29|0)!=(0|0);
 if ($30) {
  $36 = HEAP32[$pPrg>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = HEAP32[$2>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = (_codeRowTrigger($31,$32,$33,$34)|0);
 HEAP32[$pPrg>>2] = $35;
 $36 = HEAP32[$pPrg>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _codeRowTrigger($pParse,$pTrigger,$pTab,$orconf) {
 $pParse = $pParse|0;
 $pTrigger = $pTrigger|0;
 $pTab = $pTab|0;
 $orconf = $orconf|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $db = 0, $iEndTrigger = 0, $pPrg = 0, $pProgram = 0, $pSubParse = 0, $pTop = 0, $pWhen = 0, $sNC = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $vararg_buffer = sp;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $pTop = sp + 64|0;
 $db = sp + 60|0;
 $pPrg = sp + 56|0;
 $pWhen = sp + 52|0;
 $v = sp + 48|0;
 $sNC = sp + 16|0;
 $pProgram = sp + 12|0;
 $pSubParse = sp + 8|0;
 $iEndTrigger = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pTrigger;
 HEAP32[$3>>2] = $pTab;
 HEAP32[$4>>2] = $orconf;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 124|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 if ($8) {
  $10 = ((($9)) + 124|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = $11;
 } else {
  $12 = $9;
 }
 HEAP32[$pTop>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$db>>2] = $14;
 HEAP32[$pWhen>>2] = 0;
 HEAP32[$pProgram>>2] = 0;
 HEAP32[$iEndTrigger>>2] = 0;
 $15 = HEAP32[$db>>2]|0;
 $16 = (_sqlite3DbMallocZero($15,24,0)|0);
 HEAP32[$pPrg>>2] = $16;
 $17 = HEAP32[$pPrg>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  HEAP32[$0>>2] = 0;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $19 = HEAP32[$pTop>>2]|0;
 $20 = ((($19)) + 468|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$pPrg>>2]|0;
 $23 = ((($22)) + 4|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$pPrg>>2]|0;
 $25 = HEAP32[$pTop>>2]|0;
 $26 = ((($25)) + 468|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$db>>2]|0;
 $28 = (_sqlite3DbMallocZero($27,24,0)|0);
 HEAP32[$pProgram>>2] = $28;
 $29 = HEAP32[$pPrg>>2]|0;
 $30 = ((($29)) + 8|0);
 HEAP32[$30>>2] = $28;
 $31 = HEAP32[$pProgram>>2]|0;
 $32 = ($31|0)!=(0|0);
 if (!($32)) {
  HEAP32[$0>>2] = 0;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 $33 = HEAP32[$pTop>>2]|0;
 $34 = ((($33)) + 8|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = HEAP32[$pProgram>>2]|0;
 _sqlite3VdbeLinkSubProgram($35,$36);
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$pPrg>>2]|0;
 HEAP32[$38>>2] = $37;
 $39 = HEAP32[$4>>2]|0;
 $40 = HEAP32[$pPrg>>2]|0;
 $41 = ((($40)) + 12|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$pPrg>>2]|0;
 $43 = ((($42)) + 16|0);
 HEAP32[$43>>2] = -1;
 $44 = HEAP32[$pPrg>>2]|0;
 $45 = ((($44)) + 16|0);
 $46 = ((($45)) + 4|0);
 HEAP32[$46>>2] = -1;
 $47 = HEAP32[$db>>2]|0;
 $48 = (_sqlite3DbMallocZero($47,480,0)|0);
 HEAP32[$pSubParse>>2] = $48;
 $49 = HEAP32[$pSubParse>>2]|0;
 $50 = ($49|0)!=(0|0);
 if (!($50)) {
  HEAP32[$0>>2] = 0;
  $160 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($160|0);
 }
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 $51 = HEAP32[$pSubParse>>2]|0;
 HEAP32[$sNC>>2] = $51;
 $52 = HEAP32[$db>>2]|0;
 $53 = HEAP32[$pSubParse>>2]|0;
 HEAP32[$53>>2] = $52;
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$pSubParse>>2]|0;
 $56 = ((($55)) + 128|0);
 HEAP32[$56>>2] = $54;
 $57 = HEAP32[$pTop>>2]|0;
 $58 = HEAP32[$pSubParse>>2]|0;
 $59 = ((($58)) + 124|0);
 HEAP32[$59>>2] = $57;
 $60 = HEAP32[$2>>2]|0;
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$pSubParse>>2]|0;
 $63 = ((($62)) + 448|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$2>>2]|0;
 $65 = ((($64)) + 8|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = HEAP32[$pSubParse>>2]|0;
 $68 = ((($67)) + 148|0);
 HEAP8[$68>>0] = $66;
 $69 = HEAP32[$1>>2]|0;
 $70 = ((($69)) + 136|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = HEAP32[$pSubParse>>2]|0;
 $73 = ((($72)) + 136|0);
 HEAP32[$73>>2] = $71;
 $74 = HEAP32[$pSubParse>>2]|0;
 $75 = (_sqlite3GetVdbe($74)|0);
 HEAP32[$v>>2] = $75;
 $76 = HEAP32[$v>>2]|0;
 $77 = ($76|0)!=(0|0);
 if ($77) {
  $78 = HEAP32[$v>>2]|0;
  $79 = HEAP32[$db>>2]|0;
  $80 = HEAP32[$2>>2]|0;
  $81 = HEAP32[$80>>2]|0;
  HEAP32[$vararg_buffer>>2] = $81;
  $82 = (_sqlite3MPrintf($79,34884,$vararg_buffer)|0);
  _sqlite3VdbeChangeP4($78,-1,$82,-1);
  $83 = HEAP32[$2>>2]|0;
  $84 = ((($83)) + 12|0);
  $85 = HEAP32[$84>>2]|0;
  $86 = ($85|0)!=(0|0);
  if ($86) {
   $87 = HEAP32[$db>>2]|0;
   $88 = HEAP32[$2>>2]|0;
   $89 = ((($88)) + 12|0);
   $90 = HEAP32[$89>>2]|0;
   $91 = (_sqlite3ExprDup($87,$90,0)|0);
   HEAP32[$pWhen>>2] = $91;
   $92 = HEAP32[$pWhen>>2]|0;
   $93 = (_sqlite3ResolveExprNames($sNC,$92)|0);
   $94 = (0)==($93|0);
   if ($94) {
    $95 = HEAP32[$db>>2]|0;
    $96 = ((($95)) + 69|0);
    $97 = HEAP8[$96>>0]|0;
    $98 = $97&255;
    $99 = ($98|0)==(0);
    if ($99) {
     $100 = HEAP32[$v>>2]|0;
     $101 = (_sqlite3VdbeMakeLabel($100)|0);
     HEAP32[$iEndTrigger>>2] = $101;
     $102 = HEAP32[$pSubParse>>2]|0;
     $103 = HEAP32[$pWhen>>2]|0;
     $104 = HEAP32[$iEndTrigger>>2]|0;
     _sqlite3ExprIfFalse($102,$103,$104,16);
    }
   }
   $105 = HEAP32[$db>>2]|0;
   $106 = HEAP32[$pWhen>>2]|0;
   _sqlite3ExprDelete($105,$106);
  }
  $107 = HEAP32[$pSubParse>>2]|0;
  $108 = HEAP32[$2>>2]|0;
  $109 = ((($108)) + 28|0);
  $110 = HEAP32[$109>>2]|0;
  $111 = HEAP32[$4>>2]|0;
  (_codeTriggerProgram($107,$110,$111)|0);
  $112 = HEAP32[$iEndTrigger>>2]|0;
  $113 = ($112|0)!=(0);
  if ($113) {
   $114 = HEAP32[$v>>2]|0;
   $115 = HEAP32[$iEndTrigger>>2]|0;
   _sqlite3VdbeResolveLabel($114,$115);
  }
  $116 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($116,75)|0);
  $117 = HEAP32[$1>>2]|0;
  $118 = HEAP32[$pSubParse>>2]|0;
  _transferParseError($117,$118);
  $119 = HEAP32[$db>>2]|0;
  $120 = ((($119)) + 69|0);
  $121 = HEAP8[$120>>0]|0;
  $122 = $121&255;
  $123 = ($122|0)==(0);
  if ($123) {
   $124 = HEAP32[$v>>2]|0;
   $125 = HEAP32[$pProgram>>2]|0;
   $126 = ((($125)) + 4|0);
   $127 = HEAP32[$pTop>>2]|0;
   $128 = ((($127)) + 108|0);
   $129 = (_sqlite3VdbeTakeOpArray($124,$126,$128)|0);
   $130 = HEAP32[$pProgram>>2]|0;
   HEAP32[$130>>2] = $129;
  }
  $131 = HEAP32[$pSubParse>>2]|0;
  $132 = ((($131)) + 44|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = HEAP32[$pProgram>>2]|0;
  $135 = ((($134)) + 8|0);
  HEAP32[$135>>2] = $133;
  $136 = HEAP32[$pSubParse>>2]|0;
  $137 = ((($136)) + 40|0);
  $138 = HEAP32[$137>>2]|0;
  $139 = HEAP32[$pProgram>>2]|0;
  $140 = ((($139)) + 12|0);
  HEAP32[$140>>2] = $138;
  $141 = HEAP32[$2>>2]|0;
  $142 = HEAP32[$pProgram>>2]|0;
  $143 = ((($142)) + 16|0);
  HEAP32[$143>>2] = $141;
  $144 = HEAP32[$pSubParse>>2]|0;
  $145 = ((($144)) + 140|0);
  $146 = HEAP32[$145>>2]|0;
  $147 = HEAP32[$pPrg>>2]|0;
  $148 = ((($147)) + 16|0);
  HEAP32[$148>>2] = $146;
  $149 = HEAP32[$pSubParse>>2]|0;
  $150 = ((($149)) + 144|0);
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$pPrg>>2]|0;
  $153 = ((($152)) + 16|0);
  $154 = ((($153)) + 4|0);
  HEAP32[$154>>2] = $151;
  $155 = HEAP32[$v>>2]|0;
  _sqlite3VdbeDelete($155);
 }
 $156 = HEAP32[$pSubParse>>2]|0;
 _sqlite3ParserReset($156);
 $157 = HEAP32[$db>>2]|0;
 $158 = HEAP32[$pSubParse>>2]|0;
 _sqlite3DbFree($157,$158);
 $159 = HEAP32[$pPrg>>2]|0;
 HEAP32[$0>>2] = $159;
 $160 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($160|0);
}
function _sqlite3VdbeLinkSubProgram($pVdbe,$p) {
 $pVdbe = $pVdbe|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pVdbe;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 200|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 HEAP32[$6>>2] = $4;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 200|0);
 HEAP32[$9>>2] = $7;
 STACKTOP = sp;return;
}
function _codeTriggerProgram($pParse,$pStepList,$orconf) {
 $pParse = $pParse|0;
 $pStepList = $pStepList|0;
 $orconf = $orconf|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $db = 0, $pSelect = 0, $pStep = 0, $sDest = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $pStep = sp + 40|0;
 $v = sp + 36|0;
 $db = sp + 32|0;
 $sDest = sp + 8|0;
 $pSelect = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStepList;
 HEAP32[$2>>2] = $orconf;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 HEAP32[$pStep>>2] = $8;
 while(1) {
  $9 = HEAP32[$pStep>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$2>>2]|0;
  $12 = ($11|0)==(10);
  if ($12) {
   $13 = HEAP32[$pStep>>2]|0;
   $14 = ((($13)) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $$sink = $15;
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = $16&255;
   $$sink = $17;
  }
  $18 = $$sink&255;
  $19 = $18&255;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 149|0);
  HEAP8[$21>>0] = $19;
  $22 = HEAP32[$pStep>>2]|0;
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  switch ($24|0) {
  case 110:  {
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$0>>2]|0;
   $27 = HEAP32[$pStep>>2]|0;
   $28 = (_targetSrcList($26,$27)|0);
   $29 = HEAP32[$db>>2]|0;
   $30 = HEAP32[$pStep>>2]|0;
   $31 = ((($30)) + 20|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = (_sqlite3ExprListDup($29,$32,0)|0);
   $34 = HEAP32[$db>>2]|0;
   $35 = HEAP32[$pStep>>2]|0;
   $36 = ((($35)) + 16|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (_sqlite3ExprDup($34,$37,0)|0);
   $39 = HEAP32[$0>>2]|0;
   $40 = ((($39)) + 149|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41&255;
   _sqlite3Update($25,$28,$33,$38,$42);
   break;
  }
  case 108:  {
   $43 = HEAP32[$0>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = HEAP32[$pStep>>2]|0;
   $46 = (_targetSrcList($44,$45)|0);
   $47 = HEAP32[$db>>2]|0;
   $48 = HEAP32[$pStep>>2]|0;
   $49 = ((($48)) + 8|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (_sqlite3SelectDup($47,$50,0)|0);
   $52 = HEAP32[$db>>2]|0;
   $53 = HEAP32[$pStep>>2]|0;
   $54 = ((($53)) + 24|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (_sqlite3IdListDup($52,$55)|0);
   $57 = HEAP32[$0>>2]|0;
   $58 = ((($57)) + 149|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = $59&255;
   _sqlite3Insert($43,$46,$51,$56,$60);
   break;
  }
  case 109:  {
   $61 = HEAP32[$0>>2]|0;
   $62 = HEAP32[$0>>2]|0;
   $63 = HEAP32[$pStep>>2]|0;
   $64 = (_targetSrcList($62,$63)|0);
   $65 = HEAP32[$db>>2]|0;
   $66 = HEAP32[$pStep>>2]|0;
   $67 = ((($66)) + 16|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = (_sqlite3ExprDup($65,$68,0)|0);
   _sqlite3DeleteFrom($61,$64,$69);
   break;
  }
  default: {
   $70 = HEAP32[$db>>2]|0;
   $71 = HEAP32[$pStep>>2]|0;
   $72 = ((($71)) + 8|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (_sqlite3SelectDup($70,$73,0)|0);
   HEAP32[$pSelect>>2] = $74;
   _sqlite3SelectDestInit($sDest,4,0);
   $75 = HEAP32[$0>>2]|0;
   $76 = HEAP32[$pSelect>>2]|0;
   (_sqlite3Select($75,$76,$sDest)|0);
   $77 = HEAP32[$db>>2]|0;
   $78 = HEAP32[$pSelect>>2]|0;
   _sqlite3SelectDelete($77,$78);
  }
  }
  $79 = HEAP32[$pStep>>2]|0;
  $80 = HEAP8[$79>>0]|0;
  $81 = $80&255;
  $82 = ($81|0)!=(119);
  if ($82) {
   $83 = HEAP32[$v>>2]|0;
   (_sqlite3VdbeAddOp0($83,118)|0);
  }
  $84 = HEAP32[$pStep>>2]|0;
  $85 = ((($84)) + 28|0);
  $86 = HEAP32[$85>>2]|0;
  HEAP32[$pStep>>2] = $86;
 }
 STACKTOP = sp;return 0;
}
function _transferParseError($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 36|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = ((($6)) + 4|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 4|0);
  HEAP32[$10>>2] = $8;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 36|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 36|0);
  HEAP32[$15>>2] = $13;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 12|0);
  HEAP32[$20>>2] = $18;
  STACKTOP = sp;return;
 } else {
  $21 = HEAP32[$6>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 4|0);
  $24 = HEAP32[$23>>2]|0;
  _sqlite3DbFree($21,$24);
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeTakeOpArray($p,$pnOp,$pnMaxArg) {
 $p = $p|0;
 $pnOp = $pnOp|0;
 $pnMaxArg = $pnMaxArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $aOp = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pnOp;
 HEAP32[$2>>2] = $pnMaxArg;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 88|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$aOp>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 _resolveP2Values($6,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 136|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 88|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$aOp>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _resolveP2Values($p,$pMaxFuncArgs) {
 $p = $p|0;
 $pMaxFuncArgs = $pMaxFuncArgs|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aLabel = 0;
 var $n = 0, $nMaxArgs = 0, $pOp = 0, $pParse = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $nMaxArgs = sp + 16|0;
 $pOp = sp + 12|0;
 $pParse = sp + 8|0;
 $aLabel = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pMaxFuncArgs;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$nMaxArgs>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pParse>>2] = $6;
 $7 = HEAP32[$pParse>>2]|0;
 $8 = ((($7)) + 76|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aLabel>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 144|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12 & -129;
 $14 = $13 | 128;
 HEAP16[$11>>1] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 144|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17 & -257;
 HEAP16[$16>>1] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 136|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 88|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + (($22*20)|0)|0);
 HEAP32[$pOp>>2] = $26;
 while(1) {
  $27 = HEAP32[$pOp>>2]|0;
  $28 = HEAP8[$27>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)<=(71);
  if ($30) {
   $31 = HEAP32[$pOp>>2]|0;
   $32 = HEAP8[$31>>0]|0;
   $33 = $32&255;
   switch ($33|0) {
   case 2:  {
    $34 = HEAP32[$pOp>>2]|0;
    $35 = ((($34)) + 8|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = ($36|0)!=(0);
    if ($37) {
     $38 = HEAP32[$0>>2]|0;
     $39 = ((($38)) + 144|0);
     $40 = HEAP16[$39>>1]|0;
     $41 = $40 & -129;
     HEAP16[$39>>1] = $41;
     label = 6;
    } else {
     label = 6;
    }
    break;
   }
   case 0: case 1:  {
    label = 6;
    break;
   }
   case 9: case 10: case 8:  {
    $47 = HEAP32[$0>>2]|0;
    $48 = ((($47)) + 144|0);
    $49 = HEAP16[$48>>1]|0;
    $50 = $49 & -129;
    HEAP16[$48>>1] = $50;
    $51 = HEAP32[$0>>2]|0;
    $52 = ((($51)) + 144|0);
    $53 = HEAP16[$52>>1]|0;
    $54 = $53 & -257;
    $55 = $54 | 256;
    HEAP16[$52>>1] = $55;
    break;
   }
   case 12:  {
    $56 = HEAP32[$pOp>>2]|0;
    $57 = ((($56)) + 8|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = HEAP32[$nMaxArgs>>2]|0;
    $60 = ($58|0)>($59|0);
    if ($60) {
     $61 = HEAP32[$pOp>>2]|0;
     $62 = ((($61)) + 8|0);
     $63 = HEAP32[$62>>2]|0;
     HEAP32[$nMaxArgs>>2] = $63;
    }
    break;
   }
   case 11:  {
    $64 = HEAP32[$pOp>>2]|0;
    $65 = ((($64)) + -20|0);
    $66 = ((($65)) + 4|0);
    $67 = HEAP32[$66>>2]|0;
    HEAP32[$n>>2] = $67;
    $68 = HEAP32[$n>>2]|0;
    $69 = HEAP32[$nMaxArgs>>2]|0;
    $70 = ($68|0)>($69|0);
    if ($70) {
     $71 = HEAP32[$n>>2]|0;
     HEAP32[$nMaxArgs>>2] = $71;
    }
    break;
   }
   case 3: case 5: case 7:  {
    $72 = HEAP32[$pOp>>2]|0;
    $73 = ((($72)) + 16|0);
    HEAP32[$73>>2] = 201;
    $74 = HEAP32[$pOp>>2]|0;
    $75 = ((($74)) + 1|0);
    HEAP8[$75>>0] = -19;
    break;
   }
   case 4: case 6:  {
    $76 = HEAP32[$pOp>>2]|0;
    $77 = ((($76)) + 16|0);
    HEAP32[$77>>2] = 202;
    $78 = HEAP32[$pOp>>2]|0;
    $79 = ((($78)) + 1|0);
    HEAP8[$79>>0] = -19;
    break;
   }
   default: {
   }
   }
   if ((label|0) == 6) {
    label = 0;
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 144|0);
    $44 = HEAP16[$43>>1]|0;
    $45 = $44 & -257;
    $46 = $45 | 256;
    HEAP16[$43>>1] = $46;
   }
   $80 = HEAP32[$pOp>>2]|0;
   $81 = HEAP8[$80>>0]|0;
   $82 = $81&255;
   $83 = (34898 + ($82)|0);
   $84 = HEAP8[$83>>0]|0;
   $85 = $84&255;
   $86 = $85 & 1;
   $87 = ($86|0)!=(0);
   if ($87) {
    $88 = HEAP32[$pOp>>2]|0;
    $89 = ((($88)) + 8|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = ($90|0)<(0);
    if ($91) {
     $92 = HEAP32[$pOp>>2]|0;
     $93 = ((($92)) + 8|0);
     $94 = HEAP32[$93>>2]|0;
     $95 = (-1 - ($94))|0;
     $96 = HEAP32[$aLabel>>2]|0;
     $97 = (($96) + ($95<<2)|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = HEAP32[$pOp>>2]|0;
     $100 = ((($99)) + 8|0);
     HEAP32[$100>>2] = $98;
    }
   }
  }
  $101 = HEAP32[$pOp>>2]|0;
  $102 = HEAP32[$0>>2]|0;
  $103 = ((($102)) + 88|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = ($101|0)==($104|0);
  if ($105) {
   break;
  }
  $106 = HEAP32[$pOp>>2]|0;
  $107 = ((($106)) + -20|0);
  HEAP32[$pOp>>2] = $107;
 }
 $108 = HEAP32[$0>>2]|0;
 $109 = HEAP32[$108>>2]|0;
 $110 = HEAP32[$pParse>>2]|0;
 $111 = ((($110)) + 76|0);
 $112 = HEAP32[$111>>2]|0;
 _sqlite3DbFree($109,$112);
 $113 = HEAP32[$pParse>>2]|0;
 $114 = ((($113)) + 76|0);
 HEAP32[$114>>2] = 0;
 $115 = HEAP32[$pParse>>2]|0;
 $116 = ((($115)) + 72|0);
 HEAP32[$116>>2] = 0;
 $117 = HEAP32[$nMaxArgs>>2]|0;
 $118 = HEAP32[$1>>2]|0;
 HEAP32[$118>>2] = $117;
 STACKTOP = sp;return;
}
function _sqlite3BtreeNext($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = ((($4)) + 18|0);
 HEAP16[$5>>1] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 64|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = $9 & -7;
 $11 = $10&255;
 HEAP8[$7>>0] = $11;
 $12 = HEAP32[$2>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 66|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(1);
 $18 = HEAP32[$1>>2]|0;
 if ($17) {
  $19 = HEAP32[$2>>2]|0;
  $20 = (_btreeNext($18,$19)|0);
  HEAP32[$0>>2] = $20;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $21 = ((($18)) + 68|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22 << 24 >> 24;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 120|0);
 $26 = (($25) + ($23<<2)|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pPage>>2] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 68|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30 << 24 >> 24;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 80|0);
 $34 = (($33) + ($31<<1)|0);
 $35 = HEAP16[$34>>1]|0;
 $36 = (($35) + 1)<<16>>16;
 HEAP16[$34>>1] = $36;
 $37 = $36&65535;
 $38 = HEAP32[$pPage>>2]|0;
 $39 = ((($38)) + 18|0);
 $40 = HEAP16[$39>>1]|0;
 $41 = $40&65535;
 $42 = ($37|0)>=($41|0);
 if ($42) {
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 68|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45 << 24 >> 24;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 80|0);
  $49 = (($48) + ($46<<1)|0);
  $50 = HEAP16[$49>>1]|0;
  $51 = (($50) + -1)<<16>>16;
  HEAP16[$49>>1] = $51;
  $52 = HEAP32[$1>>2]|0;
  $53 = HEAP32[$2>>2]|0;
  $54 = (_btreeNext($52,$53)|0);
  HEAP32[$0>>2] = $54;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $55 = HEAP32[$pPage>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = HEAP8[$56>>0]|0;
 $58 = ($57<<24>>24)!=(0);
 if ($58) {
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $59 = HEAP32[$1>>2]|0;
  $60 = (_moveToLeftmost($59)|0);
  HEAP32[$0>>2] = $60;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _sqlite3BtreePrevious($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$3>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 64|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & -15;
 $9 = $8&255;
 HEAP8[$5>>0] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = ((($11)) + 18|0);
 HEAP16[$12>>1] = 0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 66|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = ($16|0)!=(1);
 if (!($17)) {
  $18 = HEAP32[$1>>2]|0;
  $19 = ((($18)) + 68|0);
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 80|0);
  $24 = (($23) + ($21<<1)|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = ($26|0)==(0);
  if (!($27)) {
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 68|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30 << 24 >> 24;
   $32 = HEAP32[$1>>2]|0;
   $33 = ((($32)) + 120|0);
   $34 = (($33) + ($31<<2)|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ((($35)) + 4|0);
   $37 = HEAP8[$36>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)==(0);
   if (!($39)) {
    $43 = HEAP32[$1>>2]|0;
    $44 = ((($43)) + 68|0);
    $45 = HEAP8[$44>>0]|0;
    $46 = $45 << 24 >> 24;
    $47 = HEAP32[$1>>2]|0;
    $48 = ((($47)) + 80|0);
    $49 = (($48) + ($46<<1)|0);
    $50 = HEAP16[$49>>1]|0;
    $51 = (($50) + -1)<<16>>16;
    HEAP16[$49>>1] = $51;
    HEAP32[$0>>2] = 0;
    $52 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($52|0);
   }
  }
 }
 $40 = HEAP32[$1>>2]|0;
 $41 = HEAP32[$2>>2]|0;
 $42 = (_btreePrevious($40,$41)|0);
 HEAP32[$0>>2] = $42;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _btreePrevious($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $idx = 0;
 var $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $pPage = sp + 4|0;
 $idx = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 66|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)>=(3);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = (_btreeRestoreCursorPosition($13)|0);
   $15 = $14;
  } else {
   $15 = 0;
  }
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $18;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 66|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (0)==($22|0);
  if ($23) {
   $24 = HEAP32[$2>>2]|0;
   HEAP32[$24>>2] = 1;
   HEAP32[$0>>2] = 0;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 60|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 66|0);
   HEAP8[$30>>0] = 1;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 60|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)<(0);
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 60|0);
   HEAP32[$36>>2] = 0;
   if ($34) {
    HEAP32[$0>>2] = 0;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
  }
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 68|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = $39 << 24 >> 24;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 120|0);
 $43 = (($42) + ($40<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pPage>>2] = $44;
 $45 = HEAP32[$pPage>>2]|0;
 $46 = ((($45)) + 4|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = ($47<<24>>24)!=(0);
 do {
  if ($48) {
   while(1) {
    $94 = HEAP32[$1>>2]|0;
    $95 = ((($94)) + 68|0);
    $96 = HEAP8[$95>>0]|0;
    $97 = $96 << 24 >> 24;
    $98 = HEAP32[$1>>2]|0;
    $99 = ((($98)) + 80|0);
    $100 = (($99) + ($97<<1)|0);
    $101 = HEAP16[$100>>1]|0;
    $102 = $101&65535;
    $103 = ($102|0)==(0);
    $104 = HEAP32[$1>>2]|0;
    $105 = ((($104)) + 68|0);
    $106 = HEAP8[$105>>0]|0;
    $107 = $106 << 24 >> 24;
    if (!($103)) {
     break;
    }
    $108 = ($107|0)==(0);
    $109 = HEAP32[$1>>2]|0;
    if ($108) {
     label = 17;
     break;
    }
    _moveToParent($109);
   }
   if ((label|0) == 17) {
    $110 = ((($109)) + 66|0);
    HEAP8[$110>>0] = 0;
    $111 = HEAP32[$2>>2]|0;
    HEAP32[$111>>2] = 1;
    HEAP32[$0>>2] = 0;
    $138 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($138|0);
   }
   $112 = HEAP32[$1>>2]|0;
   $113 = ((($112)) + 80|0);
   $114 = (($113) + ($107<<1)|0);
   $115 = HEAP16[$114>>1]|0;
   $116 = (($115) + -1)<<16>>16;
   HEAP16[$114>>1] = $116;
   $117 = HEAP32[$1>>2]|0;
   $118 = ((($117)) + 68|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = $119 << 24 >> 24;
   $121 = HEAP32[$1>>2]|0;
   $122 = ((($121)) + 120|0);
   $123 = (($122) + ($120<<2)|0);
   $124 = HEAP32[$123>>2]|0;
   HEAP32[$pPage>>2] = $124;
   $125 = HEAP32[$pPage>>2]|0;
   $126 = ((($125)) + 2|0);
   $127 = HEAP8[$126>>0]|0;
   $128 = $127&255;
   $129 = ($128|0)!=(0);
   if ($129) {
    $130 = HEAP32[$pPage>>2]|0;
    $131 = ((($130)) + 4|0);
    $132 = HEAP8[$131>>0]|0;
    $133 = ($132<<24>>24)!=(0);
    if (!($133)) {
     $134 = HEAP32[$1>>2]|0;
     $135 = HEAP32[$2>>2]|0;
     $136 = (_sqlite3BtreePrevious($134,$135)|0);
     HEAP32[$rc>>2] = $136;
     break;
    }
   }
   HEAP32[$rc>>2] = 0;
  } else {
   $49 = HEAP32[$1>>2]|0;
   $50 = ((($49)) + 68|0);
   $51 = HEAP8[$50>>0]|0;
   $52 = $51 << 24 >> 24;
   $53 = HEAP32[$1>>2]|0;
   $54 = ((($53)) + 80|0);
   $55 = (($54) + ($52<<1)|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   HEAP32[$idx>>2] = $57;
   $58 = HEAP32[$1>>2]|0;
   $59 = HEAP32[$pPage>>2]|0;
   $60 = ((($59)) + 56|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$pPage>>2]|0;
   $63 = ((($62)) + 20|0);
   $64 = HEAP16[$63>>1]|0;
   $65 = $64&65535;
   $66 = HEAP32[$idx>>2]|0;
   $67 = $66<<1;
   $68 = HEAP32[$pPage>>2]|0;
   $69 = ((($68)) + 64|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = (($70) + ($67)|0);
   $72 = HEAP8[$71>>0]|0;
   $73 = $72&255;
   $74 = $73 << 8;
   $75 = HEAP32[$idx>>2]|0;
   $76 = $75<<1;
   $77 = HEAP32[$pPage>>2]|0;
   $78 = ((($77)) + 64|0);
   $79 = HEAP32[$78>>2]|0;
   $80 = (($79) + ($76)|0);
   $81 = ((($80)) + 1|0);
   $82 = HEAP8[$81>>0]|0;
   $83 = $82&255;
   $84 = $74 | $83;
   $85 = $65 & $84;
   $86 = (($61) + ($85)|0);
   $87 = (_sqlite3Get4byte($86)|0);
   $88 = (_moveToChild($58,$87)|0);
   HEAP32[$rc>>2] = $88;
   $89 = HEAP32[$rc>>2]|0;
   $90 = ($89|0)!=(0);
   if (!($90)) {
    $92 = HEAP32[$1>>2]|0;
    $93 = (_moveToRightmost($92)|0);
    HEAP32[$rc>>2] = $93;
    break;
   }
   $91 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $91;
   $138 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($138|0);
  }
 } while(0);
 $137 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $137;
 $138 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($138|0);
}
function _btreeRestoreCursorPosition($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $skipNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $skipNext = sp;
 HEAP32[$1>>2] = $pCur;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 66|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(4);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = ((($7)) + 60|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$0>>2] = $9;
  $48 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $10 = ((($7)) + 66|0);
 HEAP8[$10>>0] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 40|0);
 $17 = $16;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (($17) + 4)|0;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (_btreeMoveto($11,$14,$19,$22,0,$skipNext)|0);
 HEAP32[$rc>>2] = $23;
 $24 = HEAP32[$rc>>2]|0;
 $25 = ($24|0)==(0);
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  _sqlite3_free($28);
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 48|0);
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$skipNext>>2]|0;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 60|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = $34 | $31;
  HEAP32[$33>>2] = $35;
  $36 = HEAP32[$1>>2]|0;
  $37 = ((($36)) + 60|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 66|0);
   $42 = HEAP8[$41>>0]|0;
   $43 = $42&255;
   $44 = ($43|0)==(1);
   if ($44) {
    $45 = HEAP32[$1>>2]|0;
    $46 = ((($45)) + 66|0);
    HEAP8[$46>>0] = 2;
   }
  }
 }
 $47 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $47;
 $48 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _moveToChild($pCur,$newPgno) {
 $pCur = $pCur|0;
 $newPgno = $newPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $newPgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 68|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = ($9|0)>=(19);
 if ($10) {
  $11 = (_sqlite3CorruptError(63009)|0);
  HEAP32[$0>>2] = $11;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 } else {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = ((($13)) + 18|0);
  HEAP16[$14>>1] = 0;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 64|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = $18 & -7;
  $20 = $19&255;
  HEAP8[$16>>0] = $20;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 68|0);
  $23 = HEAP8[$22>>0]|0;
  $24 = (($23) + 1)<<24>>24;
  HEAP8[$22>>0] = $24;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 68|0);
  $27 = HEAP8[$26>>0]|0;
  $28 = $27 << 24 >> 24;
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 80|0);
  $31 = (($30) + ($28<<1)|0);
  HEAP16[$31>>1] = 0;
  $32 = HEAP32[$pBt>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 68|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36 << 24 >> 24;
  $38 = HEAP32[$1>>2]|0;
  $39 = ((($38)) + 120|0);
  $40 = (($39) + ($37<<2)|0);
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$1>>2]|0;
  $43 = ((($42)) + 65|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = (_getAndInitPage($32,$33,$40,$41,$45)|0);
  HEAP32[$0>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 return (0)|0;
}
function _moveToRightmost($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pgno = sp + 8|0;
 $rc = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pPage>>2] = 0;
 while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = ((($2)) + 68|0);
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = HEAP32[$1>>2]|0;
  $7 = ((($6)) + 120|0);
  $8 = (($7) + ($5<<2)|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pPage>>2] = $9;
  $10 = ((($9)) + 4|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = ($11<<24>>24)!=(0);
  $13 = $12 ^ 1;
  $14 = HEAP32[$pPage>>2]|0;
  if (!($13)) {
   label = 5;
   break;
  }
  $15 = ((($14)) + 5|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = (($17) + 8)|0;
  $19 = HEAP32[$pPage>>2]|0;
  $20 = ((($19)) + 56|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($21) + ($18)|0);
  $23 = (_sqlite3Get4byte($22)|0);
  HEAP32[$pgno>>2] = $23;
  $24 = HEAP32[$pPage>>2]|0;
  $25 = ((($24)) + 18|0);
  $26 = HEAP16[$25>>1]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ((($27)) + 68|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 80|0);
  $33 = (($32) + ($30<<1)|0);
  HEAP16[$33>>1] = $26;
  $34 = HEAP32[$1>>2]|0;
  $35 = HEAP32[$pgno>>2]|0;
  $36 = (_moveToChild($34,$35)|0);
  HEAP32[$rc>>2] = $36;
  $37 = HEAP32[$rc>>2]|0;
  $38 = ($37|0)!=(0);
  if ($38) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $39 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $39;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 else if ((label|0) == 5) {
  $40 = ((($14)) + 18|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = (($42) - 1)|0;
  $44 = $43&65535;
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 68|0);
  $47 = HEAP8[$46>>0]|0;
  $48 = $47 << 24 >> 24;
  $49 = HEAP32[$1>>2]|0;
  $50 = ((($49)) + 80|0);
  $51 = (($50) + ($48<<1)|0);
  HEAP16[$51>>1] = $44;
  HEAP32[$0>>2] = 0;
  $52 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 return (0)|0;
}
function _moveToParent($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = ((($2)) + 18|0);
 HEAP16[$3>>1] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 64|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = $7 & -7;
 $9 = $8&255;
 HEAP8[$5>>0] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 68|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = (($12) + -1)<<24>>24;
 HEAP8[$11>>0] = $13;
 $14 = $12 << 24 >> 24;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 120|0);
 $17 = (($16) + ($14<<2)|0);
 $18 = HEAP32[$17>>2]|0;
 _releasePageNotNull($18);
 STACKTOP = sp;return;
}
function _getAndInitPage($pBt,$pgno,$ppPage,$pCur,$bReadOnly) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $pCur = $pCur|0;
 $bReadOnly = $bReadOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $rc = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $pCur;
 HEAP32[$5>>2] = $bReadOnly;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_btreePagecount($7)|0);
 $9 = ($6>>>0)>($8>>>0);
 L1: do {
  if ($9) {
   $10 = (_sqlite3CorruptError(60244)|0);
   HEAP32[$rc>>2] = $10;
  } else {
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$2>>2]|0;
   $14 = HEAP32[$5>>2]|0;
   $15 = (_sqlite3PagerGet($12,$13,$pDbPage,$14)|0);
   HEAP32[$rc>>2] = $15;
   $16 = HEAP32[$rc>>2]|0;
   $17 = ($16|0)!=(0);
   if (!($17)) {
    $18 = HEAP32[$pDbPage>>2]|0;
    $19 = (_sqlite3PagerGetExtra($18)|0);
    $20 = HEAP32[$3>>2]|0;
    HEAP32[$20>>2] = $19;
    $21 = HEAP32[$3>>2]|0;
    $22 = HEAP32[$21>>2]|0;
    $23 = HEAP8[$22>>0]|0;
    $24 = $23&255;
    $25 = ($24|0)==(0);
    if ($25) {
     $26 = HEAP32[$pDbPage>>2]|0;
     $27 = HEAP32[$2>>2]|0;
     $28 = HEAP32[$1>>2]|0;
     (_btreePageFromDbPage($26,$27,$28)|0);
     $29 = HEAP32[$3>>2]|0;
     $30 = HEAP32[$29>>2]|0;
     $31 = (_btreeInitPage($30)|0);
     HEAP32[$rc>>2] = $31;
     $32 = HEAP32[$rc>>2]|0;
     $33 = ($32|0)!=(0);
     if ($33) {
      $34 = HEAP32[$3>>2]|0;
      $35 = HEAP32[$34>>2]|0;
      _releasePage($35);
      break;
     }
    }
    $36 = HEAP32[$4>>2]|0;
    $37 = ($36|0)!=(0|0);
    do {
     if ($37) {
      $38 = HEAP32[$3>>2]|0;
      $39 = HEAP32[$38>>2]|0;
      $40 = ((($39)) + 18|0);
      $41 = HEAP16[$40>>1]|0;
      $42 = $41&65535;
      $43 = ($42|0)<(1);
      if (!($43)) {
       $44 = HEAP32[$3>>2]|0;
       $45 = HEAP32[$44>>2]|0;
       $46 = ((($45)) + 2|0);
       $47 = HEAP8[$46>>0]|0;
       $48 = $47&255;
       $49 = HEAP32[$4>>2]|0;
       $50 = ((($49)) + 69|0);
       $51 = HEAP8[$50>>0]|0;
       $52 = $51&255;
       $53 = ($48|0)!=($52|0);
       if (!($53)) {
        break;
       }
      }
      $54 = (_sqlite3CorruptError(60266)|0);
      HEAP32[$rc>>2] = $54;
      $55 = HEAP32[$3>>2]|0;
      $56 = HEAP32[$55>>2]|0;
      _releasePage($56);
      break L1;
     }
    } while(0);
    HEAP32[$0>>2] = 0;
    $64 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($64|0);
   }
  }
 } while(0);
 $57 = HEAP32[$4>>2]|0;
 $58 = ($57|0)!=(0|0);
 if ($58) {
  $59 = HEAP32[$4>>2]|0;
  $60 = ((($59)) + 68|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = (($61) + -1)<<24>>24;
  HEAP8[$60>>0] = $62;
 }
 $63 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $63;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _btreePagecount($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _btreePageFromDbPage($pDbPage,$pgno,$pBt) {
 $pDbPage = $pDbPage|0;
 $pgno = $pgno|0;
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pDbPage;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$2>>2] = $pBt;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3PagerGetExtra($3)|0);
 HEAP32[$pPage>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$pPage>>2]|0;
 $7 = ((($6)) + 84|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($5|0)!=($8|0);
 if (!($9)) {
  $29 = HEAP32[$pPage>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3PagerGetData($10)|0);
 $12 = HEAP32[$pPage>>2]|0;
 $13 = ((($12)) + 56|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$pPage>>2]|0;
 $16 = ((($15)) + 72|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$pPage>>2]|0;
 $19 = ((($18)) + 52|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$pPage>>2]|0;
 $22 = ((($21)) + 84|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$1>>2]|0;
 $24 = ($23|0)==(1);
 $25 = $24 ? 100 : 0;
 $26 = $25&255;
 $27 = HEAP32[$pPage>>2]|0;
 $28 = ((($27)) + 5|0);
 HEAP8[$28>>0] = $26;
 $29 = HEAP32[$pPage>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _btreeMoveto($pCur,$pKey,$0,$1,$bias,$pRes) {
 $pCur = $pCur|0;
 $pKey = $pKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $bias = $bias|0;
 $pRes = $pRes|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $aSpace = 0, $pFree = 0, $pIdxKey = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 432|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp;
 $6 = sp + 24|0;
 $7 = sp + 20|0;
 $rc = sp + 16|0;
 $pIdxKey = sp + 12|0;
 $aSpace = sp + 40|0;
 $pFree = sp + 8|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pKey;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $bias;
 HEAP32[$7>>2] = $pRes;
 HEAP32[$pFree>>2] = 0;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)!=(0|0);
 if ($13) {
  $14 = HEAP32[$3>>2]|0;
  $15 = ((($14)) + 72|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3VdbeAllocUnpackedRecord($16,$aSpace,384,$pFree)|0);
  HEAP32[$pIdxKey>>2] = $17;
  $18 = HEAP32[$pIdxKey>>2]|0;
  $19 = ($18|0)==(0|0);
  if ($19) {
   HEAP32[$2>>2] = 7;
   $63 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
  $20 = HEAP32[$3>>2]|0;
  $21 = ((($20)) + 72|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $5;
  $24 = $23;
  $25 = HEAP32[$24>>2]|0;
  $26 = (($23) + 4)|0;
  $27 = $26;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = HEAP32[$pIdxKey>>2]|0;
  _sqlite3VdbeRecordUnpack($22,$25,$29,$30);
  $31 = HEAP32[$pIdxKey>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  $35 = ($34|0)==(0);
  if ($35) {
   $36 = HEAP32[$3>>2]|0;
   $37 = ((($36)) + 72|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = ((($38)) + 12|0);
   $40 = HEAP32[$39>>2]|0;
   $41 = HEAP32[$pFree>>2]|0;
   _sqlite3DbFree($40,$41);
   $42 = (_sqlite3CorruptError(59021)|0);
   HEAP32[$2>>2] = $42;
   $63 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
 } else {
  HEAP32[$pIdxKey>>2] = 0;
 }
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$pIdxKey>>2]|0;
 $45 = $5;
 $46 = $45;
 $47 = HEAP32[$46>>2]|0;
 $48 = (($45) + 4)|0;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$6>>2]|0;
 $52 = HEAP32[$7>>2]|0;
 $53 = (_sqlite3BtreeMovetoUnpacked($43,$44,$47,$50,$51,$52)|0);
 HEAP32[$rc>>2] = $53;
 $54 = HEAP32[$pFree>>2]|0;
 $55 = ($54|0)!=(0|0);
 if ($55) {
  $56 = HEAP32[$3>>2]|0;
  $57 = ((($56)) + 72|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = ((($58)) + 12|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$pFree>>2]|0;
  _sqlite3DbFree($60,$61);
 }
 $62 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $62;
 $63 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _sqlite3VdbeAllocUnpackedRecord($pKeyInfo,$pSpace,$szSpace,$ppFree) {
 $pKeyInfo = $pKeyInfo|0;
 $pSpace = $pSpace|0;
 $szSpace = $szSpace|0;
 $ppFree = $ppFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nByte = 0, $nOff = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 $nOff = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $pKeyInfo;
 HEAP32[$2>>2] = $pSpace;
 HEAP32[$3>>2] = $szSpace;
 HEAP32[$4>>2] = $ppFree;
 $5 = HEAP32[$2>>2]|0;
 $6 = $5;
 $7 = $6 & 7;
 $8 = (8 - ($7))|0;
 $9 = $8 & 7;
 HEAP32[$nOff>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 6|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = (($13) + 1)|0;
 $15 = ($14*40)|0;
 $16 = (16 + ($15))|0;
 HEAP32[$nByte>>2] = $16;
 $17 = HEAP32[$nByte>>2]|0;
 $18 = HEAP32[$3>>2]|0;
 $19 = HEAP32[$nOff>>2]|0;
 $20 = (($18) + ($19))|0;
 $21 = ($17|0)>($20|0);
 if ($21) {
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$nByte>>2]|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (_sqlite3DbMallocRaw($24,$25,$27)|0);
  HEAP32[$p>>2] = $28;
  $29 = HEAP32[$p>>2]|0;
  $30 = HEAP32[$4>>2]|0;
  HEAP32[$30>>2] = $29;
  $31 = HEAP32[$p>>2]|0;
  $32 = ($31|0)!=(0|0);
  if (!($32)) {
   HEAP32[$0>>2] = 0;
   $52 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($52|0);
  }
 } else {
  $33 = HEAP32[$nOff>>2]|0;
  $34 = HEAP32[$2>>2]|0;
  $35 = (($34) + ($33)|0);
  HEAP32[$p>>2] = $35;
  $36 = HEAP32[$4>>2]|0;
  HEAP32[$36>>2] = 0;
 }
 $37 = HEAP32[$p>>2]|0;
 $38 = ((($37)) + 16|0);
 $39 = HEAP32[$p>>2]|0;
 $40 = ((($39)) + 4|0);
 HEAP32[$40>>2] = $38;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$p>>2]|0;
 HEAP32[$42>>2] = $41;
 $43 = HEAP32[$1>>2]|0;
 $44 = ((($43)) + 6|0);
 $45 = HEAP16[$44>>1]|0;
 $46 = $45&65535;
 $47 = (($46) + 1)|0;
 $48 = $47&65535;
 $49 = HEAP32[$p>>2]|0;
 $50 = ((($49)) + 8|0);
 HEAP16[$50>>1] = $48;
 $51 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $51;
 $52 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _sqlite3VdbeRecordUnpack($pKeyInfo,$nKey,$pKey,$p) {
 $pKeyInfo = $pKeyInfo|0;
 $nKey = $nKey|0;
 $pKey = $pKey|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $aKey = 0, $d = 0, $idx = 0, $pMem = 0, $serial_type = 0, $szHdr = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aKey = sp + 20|0;
 $d = sp + 16|0;
 $idx = sp + 12|0;
 $u = sp + 40|0;
 $szHdr = sp + 8|0;
 $pMem = sp + 4|0;
 $serial_type = sp;
 HEAP32[$0>>2] = $pKeyInfo;
 HEAP32[$1>>2] = $nKey;
 HEAP32[$2>>2] = $pKey;
 HEAP32[$3>>2] = $p;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$aKey>>2] = $4;
 $5 = HEAP32[$3>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pMem>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = ((($8)) + 10|0);
 HEAP8[$9>>0] = 0;
 $10 = HEAP32[$aKey>>2]|0;
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)<(128);
 $14 = HEAP32[$aKey>>2]|0;
 if ($13) {
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  HEAP32[$szHdr>>2] = $16;
  $20 = 1;
 } else {
  $17 = (_sqlite3GetVarint32($14,$szHdr)|0);
  $18 = $17&255;
  $20 = $18;
 }
 $19 = $20&255;
 $21 = $19&255;
 HEAP32[$idx>>2] = $21;
 $22 = HEAP32[$szHdr>>2]|0;
 HEAP32[$d>>2] = $22;
 HEAP16[$u>>1] = 0;
 while(1) {
  $23 = HEAP32[$idx>>2]|0;
  $24 = HEAP32[$szHdr>>2]|0;
  $25 = ($23>>>0)<($24>>>0);
  if (!($25)) {
   label = 11;
   break;
  }
  $26 = HEAP32[$d>>2]|0;
  $27 = HEAP32[$1>>2]|0;
  $28 = ($26|0)<=($27|0);
  if (!($28)) {
   label = 11;
   break;
  }
  $29 = HEAP32[$idx>>2]|0;
  $30 = HEAP32[$aKey>>2]|0;
  $31 = (($30) + ($29)|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = ($33|0)<(128);
  $35 = HEAP32[$idx>>2]|0;
  $36 = HEAP32[$aKey>>2]|0;
  $37 = (($36) + ($35)|0);
  if ($34) {
   $38 = HEAP8[$37>>0]|0;
   $39 = $38&255;
   HEAP32[$serial_type>>2] = $39;
   $43 = 1;
  } else {
   $40 = (_sqlite3GetVarint32($37,$serial_type)|0);
   $41 = $40&255;
   $43 = $41;
  }
  $42 = $43&255;
  $44 = $42&255;
  $45 = HEAP32[$idx>>2]|0;
  $46 = (($45) + ($44))|0;
  HEAP32[$idx>>2] = $46;
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 4|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = HEAP32[$pMem>>2]|0;
  $51 = ((($50)) + 10|0);
  HEAP8[$51>>0] = $49;
  $52 = HEAP32[$0>>2]|0;
  $53 = ((($52)) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$pMem>>2]|0;
  $56 = ((($55)) + 32|0);
  HEAP32[$56>>2] = $54;
  $57 = HEAP32[$pMem>>2]|0;
  $58 = ((($57)) + 24|0);
  HEAP32[$58>>2] = 0;
  $59 = HEAP32[$pMem>>2]|0;
  $60 = ((($59)) + 16|0);
  HEAP32[$60>>2] = 0;
  $61 = HEAP32[$d>>2]|0;
  $62 = HEAP32[$aKey>>2]|0;
  $63 = (($62) + ($61)|0);
  $64 = HEAP32[$serial_type>>2]|0;
  $65 = HEAP32[$pMem>>2]|0;
  $66 = (_sqlite3VdbeSerialGet($63,$64,$65)|0);
  $67 = HEAP32[$d>>2]|0;
  $68 = (($67) + ($66))|0;
  HEAP32[$d>>2] = $68;
  $69 = HEAP32[$pMem>>2]|0;
  $70 = ((($69)) + 40|0);
  HEAP32[$pMem>>2] = $70;
  $71 = HEAP16[$u>>1]|0;
  $72 = (($71) + 1)<<16>>16;
  HEAP16[$u>>1] = $72;
  $73 = $72&65535;
  $74 = HEAP32[$3>>2]|0;
  $75 = ((($74)) + 8|0);
  $76 = HEAP16[$75>>1]|0;
  $77 = $76&65535;
  $78 = ($73|0)>=($77|0);
  if ($78) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 11) {
  $79 = HEAP16[$u>>1]|0;
  $80 = HEAP32[$3>>2]|0;
  $81 = ((($80)) + 8|0);
  HEAP16[$81>>1] = $79;
  STACKTOP = sp;return;
 }
}
function _sqlite3BtreeMovetoUnpacked($pCur,$pIdxKey,$0,$1,$biasRight,$pRes) {
 $pCur = $pCur|0;
 $pIdxKey = $pIdxKey|0;
 $0 = $0|0;
 $1 = $1|0;
 $biasRight = $biasRight|0;
 $pRes = $pRes|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $chldPg = 0, $idx = 0, $lwr = 0, $nCell = 0, $nCellKey = 0, $pCell = 0, $pCellBody = 0;
 var $pCellKey = 0, $pPage = 0, $rc = 0, $upr = 0, $xRecordCompare = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 8|0;
 $6 = sp + 68|0;
 $7 = sp + 64|0;
 $rc = sp + 60|0;
 $xRecordCompare = sp + 56|0;
 $lwr = sp + 52|0;
 $upr = sp + 48|0;
 $idx = sp + 44|0;
 $c = sp + 40|0;
 $chldPg = sp + 36|0;
 $pPage = sp + 32|0;
 $pCell = sp + 28|0;
 $nCellKey = sp;
 $nCell = sp + 24|0;
 $pCellKey = sp + 20|0;
 $pCellBody = sp + 16|0;
 HEAP32[$3>>2] = $pCur;
 HEAP32[$4>>2] = $pIdxKey;
 $8 = $5;
 $9 = $8;
 HEAP32[$9>>2] = $0;
 $10 = (($8) + 4)|0;
 $11 = $10;
 HEAP32[$11>>2] = $1;
 HEAP32[$6>>2] = $biasRight;
 HEAP32[$7>>2] = $pRes;
 $12 = HEAP32[$4>>2]|0;
 $13 = ($12|0)==(0|0);
 if ($13) {
  $14 = HEAP32[$3>>2]|0;
  $15 = ((($14)) + 66|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = ($17|0)==(1);
  if ($18) {
   $19 = HEAP32[$3>>2]|0;
   $20 = ((($19)) + 64|0);
   $21 = HEAP8[$20>>0]|0;
   $22 = $21&255;
   $23 = $22 & 2;
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$3>>2]|0;
    $26 = ((($25)) + 16|0);
    $27 = $26;
    $28 = $27;
    $29 = HEAP32[$28>>2]|0;
    $30 = (($27) + 4)|0;
    $31 = $30;
    $32 = HEAP32[$31>>2]|0;
    $33 = $5;
    $34 = $33;
    $35 = HEAP32[$34>>2]|0;
    $36 = (($33) + 4)|0;
    $37 = $36;
    $38 = HEAP32[$37>>2]|0;
    $39 = ($29|0)==($35|0);
    $40 = ($32|0)==($38|0);
    $41 = $39 & $40;
    if ($41) {
     $42 = HEAP32[$7>>2]|0;
     HEAP32[$42>>2] = 0;
     HEAP32[$2>>2] = 0;
     $473 = HEAP32[$2>>2]|0;
     STACKTOP = sp;return ($473|0);
    }
    $43 = HEAP32[$3>>2]|0;
    $44 = ((($43)) + 64|0);
    $45 = HEAP8[$44>>0]|0;
    $46 = $45&255;
    $47 = $46 & 8;
    $48 = ($47|0)!=(0);
    if ($48) {
     $49 = HEAP32[$3>>2]|0;
     $50 = ((($49)) + 16|0);
     $51 = $50;
     $52 = $51;
     $53 = HEAP32[$52>>2]|0;
     $54 = (($51) + 4)|0;
     $55 = $54;
     $56 = HEAP32[$55>>2]|0;
     $57 = $5;
     $58 = $57;
     $59 = HEAP32[$58>>2]|0;
     $60 = (($57) + 4)|0;
     $61 = $60;
     $62 = HEAP32[$61>>2]|0;
     $63 = ($56|0)<($62|0);
     $64 = ($53>>>0)<($59>>>0);
     $65 = ($56|0)==($62|0);
     $66 = $65 & $64;
     $67 = $63 | $66;
     if ($67) {
      $68 = HEAP32[$7>>2]|0;
      HEAP32[$68>>2] = -1;
      HEAP32[$2>>2] = 0;
      $473 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($473|0);
     }
    }
   }
  }
 }
 $69 = HEAP32[$4>>2]|0;
 $70 = ($69|0)!=(0|0);
 if ($70) {
  $71 = HEAP32[$4>>2]|0;
  $72 = (_sqlite3VdbeFindCompare($71)|0);
  HEAP32[$xRecordCompare>>2] = $72;
  $73 = HEAP32[$4>>2]|0;
  $74 = ((($73)) + 11|0);
  HEAP8[$74>>0] = 0;
 } else {
  HEAP32[$xRecordCompare>>2] = 0;
 }
 $75 = HEAP32[$3>>2]|0;
 $76 = (_moveToRoot($75)|0);
 HEAP32[$rc>>2] = $76;
 $77 = HEAP32[$rc>>2]|0;
 $78 = ($77|0)!=(0);
 if ($78) {
  $79 = HEAP32[$rc>>2]|0;
  HEAP32[$2>>2] = $79;
  $473 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($473|0);
 }
 $80 = HEAP32[$3>>2]|0;
 $81 = ((($80)) + 66|0);
 $82 = HEAP8[$81>>0]|0;
 $83 = $82&255;
 $84 = ($83|0)==(0);
 if ($84) {
  $85 = HEAP32[$7>>2]|0;
  HEAP32[$85>>2] = -1;
  HEAP32[$2>>2] = 0;
  $473 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($473|0);
 }
 L24: while(1) {
  $86 = HEAP32[$3>>2]|0;
  $87 = ((($86)) + 68|0);
  $88 = HEAP8[$87>>0]|0;
  $89 = $88 << 24 >> 24;
  $90 = HEAP32[$3>>2]|0;
  $91 = ((($90)) + 120|0);
  $92 = (($91) + ($89<<2)|0);
  $93 = HEAP32[$92>>2]|0;
  HEAP32[$pPage>>2] = $93;
  HEAP32[$lwr>>2] = 0;
  $94 = HEAP32[$pPage>>2]|0;
  $95 = ((($94)) + 18|0);
  $96 = HEAP16[$95>>1]|0;
  $97 = $96&65535;
  $98 = (($97) - 1)|0;
  HEAP32[$upr>>2] = $98;
  $99 = HEAP32[$upr>>2]|0;
  $100 = HEAP32[$6>>2]|0;
  $101 = (1 - ($100))|0;
  $102 = $99 >> $101;
  HEAP32[$idx>>2] = $102;
  $103 = HEAP32[$idx>>2]|0;
  $104 = $103&65535;
  $105 = HEAP32[$3>>2]|0;
  $106 = ((($105)) + 68|0);
  $107 = HEAP8[$106>>0]|0;
  $108 = $107 << 24 >> 24;
  $109 = HEAP32[$3>>2]|0;
  $110 = ((($109)) + 80|0);
  $111 = (($110) + ($108<<1)|0);
  HEAP16[$111>>1] = $104;
  $112 = HEAP32[$xRecordCompare>>2]|0;
  $113 = ($112|0)==(0|0);
  L26: do {
   if ($113) {
    while(1) {
     $114 = HEAP32[$pPage>>2]|0;
     $115 = ((($114)) + 68|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$pPage>>2]|0;
     $118 = ((($117)) + 20|0);
     $119 = HEAP16[$118>>1]|0;
     $120 = $119&65535;
     $121 = HEAP32[$idx>>2]|0;
     $122 = $121<<1;
     $123 = HEAP32[$pPage>>2]|0;
     $124 = ((($123)) + 64|0);
     $125 = HEAP32[$124>>2]|0;
     $126 = (($125) + ($122)|0);
     $127 = HEAP8[$126>>0]|0;
     $128 = $127&255;
     $129 = $128 << 8;
     $130 = HEAP32[$idx>>2]|0;
     $131 = $130<<1;
     $132 = HEAP32[$pPage>>2]|0;
     $133 = ((($132)) + 64|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = (($134) + ($131)|0);
     $136 = ((($135)) + 1|0);
     $137 = HEAP8[$136>>0]|0;
     $138 = $137&255;
     $139 = $129 | $138;
     $140 = $120 & $139;
     $141 = (($116) + ($140)|0);
     HEAP32[$pCell>>2] = $141;
     $142 = HEAP32[$pPage>>2]|0;
     $143 = ((($142)) + 3|0);
     $144 = HEAP8[$143>>0]|0;
     $145 = ($144<<24>>24)!=(0);
     L29: do {
      if ($145) {
       while(1) {
        $146 = HEAP32[$pCell>>2]|0;
        $147 = ((($146)) + 1|0);
        HEAP32[$pCell>>2] = $147;
        $148 = HEAP8[$146>>0]|0;
        $149 = $148&255;
        $150 = (128)<=($149|0);
        if (!($150)) {
         break L29;
        }
        $151 = HEAP32[$pCell>>2]|0;
        $152 = HEAP32[$pPage>>2]|0;
        $153 = ((($152)) + 60|0);
        $154 = HEAP32[$153>>2]|0;
        $155 = ($151>>>0)>=($154>>>0);
        if ($155) {
         label = 20;
         break L24;
        }
       }
      }
     } while(0);
     $157 = HEAP32[$pCell>>2]|0;
     (_sqlite3GetVarint($157,$nCellKey)|0);
     $158 = $nCellKey;
     $159 = $158;
     $160 = HEAP32[$159>>2]|0;
     $161 = (($158) + 4)|0;
     $162 = $161;
     $163 = HEAP32[$162>>2]|0;
     $164 = $5;
     $165 = $164;
     $166 = HEAP32[$165>>2]|0;
     $167 = (($164) + 4)|0;
     $168 = $167;
     $169 = HEAP32[$168>>2]|0;
     $170 = ($163|0)<($169|0);
     $171 = ($160>>>0)<($166>>>0);
     $172 = ($163|0)==($169|0);
     $173 = $172 & $171;
     $174 = $170 | $173;
     if ($174) {
      $175 = HEAP32[$idx>>2]|0;
      $176 = (($175) + 1)|0;
      HEAP32[$lwr>>2] = $176;
      $177 = HEAP32[$lwr>>2]|0;
      $178 = HEAP32[$upr>>2]|0;
      $179 = ($177|0)>($178|0);
      if ($179) {
       label = 23;
       break;
      }
     } else {
      $180 = $nCellKey;
      $181 = $180;
      $182 = HEAP32[$181>>2]|0;
      $183 = (($180) + 4)|0;
      $184 = $183;
      $185 = HEAP32[$184>>2]|0;
      $186 = $5;
      $187 = $186;
      $188 = HEAP32[$187>>2]|0;
      $189 = (($186) + 4)|0;
      $190 = $189;
      $191 = HEAP32[$190>>2]|0;
      $192 = ($185|0)>($191|0);
      $193 = ($182>>>0)>($188>>>0);
      $194 = ($185|0)==($191|0);
      $195 = $194 & $193;
      $196 = $192 | $195;
      if (!($196)) {
       label = 27;
       break;
      }
      $197 = HEAP32[$idx>>2]|0;
      $198 = (($197) - 1)|0;
      HEAP32[$upr>>2] = $198;
      $199 = HEAP32[$lwr>>2]|0;
      $200 = HEAP32[$upr>>2]|0;
      $201 = ($199|0)>($200|0);
      if ($201) {
       label = 26;
       break;
      }
     }
     $235 = HEAP32[$lwr>>2]|0;
     $236 = HEAP32[$upr>>2]|0;
     $237 = (($235) + ($236))|0;
     $238 = $237 >> 1;
     HEAP32[$idx>>2] = $238;
    }
    if ((label|0) == 23) {
     label = 0;
     HEAP32[$c>>2] = -1;
     label = 51;
     break;
    }
    else if ((label|0) == 26) {
     label = 0;
     HEAP32[$c>>2] = 1;
     label = 51;
     break;
    }
    else if ((label|0) == 27) {
     label = 0;
     $202 = HEAP32[$3>>2]|0;
     $203 = ((($202)) + 64|0);
     $204 = HEAP8[$203>>0]|0;
     $205 = $204&255;
     $206 = $205 | 2;
     $207 = $206&255;
     HEAP8[$203>>0] = $207;
     $208 = $nCellKey;
     $209 = $208;
     $210 = HEAP32[$209>>2]|0;
     $211 = (($208) + 4)|0;
     $212 = $211;
     $213 = HEAP32[$212>>2]|0;
     $214 = HEAP32[$3>>2]|0;
     $215 = ((($214)) + 16|0);
     $216 = $215;
     $217 = $216;
     HEAP32[$217>>2] = $210;
     $218 = (($216) + 4)|0;
     $219 = $218;
     HEAP32[$219>>2] = $213;
     $220 = HEAP32[$idx>>2]|0;
     $221 = $220&65535;
     $222 = HEAP32[$3>>2]|0;
     $223 = ((($222)) + 68|0);
     $224 = HEAP8[$223>>0]|0;
     $225 = $224 << 24 >> 24;
     $226 = HEAP32[$3>>2]|0;
     $227 = ((($226)) + 80|0);
     $228 = (($227) + ($225<<1)|0);
     HEAP16[$228>>1] = $221;
     $229 = HEAP32[$pPage>>2]|0;
     $230 = ((($229)) + 4|0);
     $231 = HEAP8[$230>>0]|0;
     $232 = ($231<<24>>24)!=(0);
     if ($232) {
      label = 29;
      break L24;
     }
     $233 = HEAP32[$idx>>2]|0;
     HEAP32[$lwr>>2] = $233;
     break;
    }
   } else {
    while(1) {
     $239 = HEAP32[$pPage>>2]|0;
     $240 = ((($239)) + 68|0);
     $241 = HEAP32[$240>>2]|0;
     $242 = HEAP32[$pPage>>2]|0;
     $243 = ((($242)) + 20|0);
     $244 = HEAP16[$243>>1]|0;
     $245 = $244&65535;
     $246 = HEAP32[$idx>>2]|0;
     $247 = $246<<1;
     $248 = HEAP32[$pPage>>2]|0;
     $249 = ((($248)) + 64|0);
     $250 = HEAP32[$249>>2]|0;
     $251 = (($250) + ($247)|0);
     $252 = HEAP8[$251>>0]|0;
     $253 = $252&255;
     $254 = $253 << 8;
     $255 = HEAP32[$idx>>2]|0;
     $256 = $255<<1;
     $257 = HEAP32[$pPage>>2]|0;
     $258 = ((($257)) + 64|0);
     $259 = HEAP32[$258>>2]|0;
     $260 = (($259) + ($256)|0);
     $261 = ((($260)) + 1|0);
     $262 = HEAP8[$261>>0]|0;
     $263 = $262&255;
     $264 = $254 | $263;
     $265 = $245 & $264;
     $266 = (($241) + ($265)|0);
     HEAP32[$pCell>>2] = $266;
     $267 = HEAP32[$pCell>>2]|0;
     $268 = HEAP8[$267>>0]|0;
     $269 = $268&255;
     HEAP32[$nCell>>2] = $269;
     $270 = HEAP32[$nCell>>2]|0;
     $271 = HEAP32[$pPage>>2]|0;
     $272 = ((($271)) + 7|0);
     $273 = HEAP8[$272>>0]|0;
     $274 = $273&255;
     $275 = ($270|0)<=($274|0);
     do {
      if ($275) {
       $276 = HEAP32[$xRecordCompare>>2]|0;
       $277 = HEAP32[$nCell>>2]|0;
       $278 = HEAP32[$pCell>>2]|0;
       $279 = ((($278)) + 1|0);
       $280 = HEAP32[$4>>2]|0;
       $281 = (FUNCTION_TABLE_iiii[$276 & 255]($277,$279,$280)|0);
       HEAP32[$c>>2] = $281;
      } else {
       $282 = HEAP32[$pCell>>2]|0;
       $283 = ((($282)) + 1|0);
       $284 = HEAP8[$283>>0]|0;
       $285 = $284&255;
       $286 = $285 & 128;
       $287 = ($286|0)!=(0);
       if (!($287)) {
        $288 = HEAP32[$nCell>>2]|0;
        $289 = $288 & 127;
        $290 = $289 << 7;
        $291 = HEAP32[$pCell>>2]|0;
        $292 = ((($291)) + 1|0);
        $293 = HEAP8[$292>>0]|0;
        $294 = $293&255;
        $295 = (($290) + ($294))|0;
        HEAP32[$nCell>>2] = $295;
        $296 = HEAP32[$pPage>>2]|0;
        $297 = ((($296)) + 10|0);
        $298 = HEAP16[$297>>1]|0;
        $299 = $298&65535;
        $300 = ($295|0)<=($299|0);
        if ($300) {
         $301 = HEAP32[$xRecordCompare>>2]|0;
         $302 = HEAP32[$nCell>>2]|0;
         $303 = HEAP32[$pCell>>2]|0;
         $304 = ((($303)) + 2|0);
         $305 = HEAP32[$4>>2]|0;
         $306 = (FUNCTION_TABLE_iiii[$301 & 255]($302,$304,$305)|0);
         HEAP32[$c>>2] = $306;
         break;
        }
       }
       $307 = HEAP32[$pCell>>2]|0;
       $308 = HEAP32[$pPage>>2]|0;
       $309 = ((($308)) + 6|0);
       $310 = HEAP8[$309>>0]|0;
       $311 = $310&255;
       $312 = (0 - ($311))|0;
       $313 = (($307) + ($312)|0);
       HEAP32[$pCellBody>>2] = $313;
       $314 = HEAP32[$pPage>>2]|0;
       $315 = ((($314)) + 80|0);
       $316 = HEAP32[$315>>2]|0;
       $317 = HEAP32[$pPage>>2]|0;
       $318 = HEAP32[$pCellBody>>2]|0;
       $319 = HEAP32[$3>>2]|0;
       $320 = ((($319)) + 16|0);
       FUNCTION_TABLE_viii[$316 & 255]($317,$318,$320);
       $321 = HEAP32[$3>>2]|0;
       $322 = ((($321)) + 16|0);
       $323 = $322;
       $324 = $323;
       $325 = HEAP32[$324>>2]|0;
       $326 = (($323) + 4)|0;
       $327 = $326;
       $328 = HEAP32[$327>>2]|0;
       HEAP32[$nCell>>2] = $325;
       $329 = HEAP32[$nCell>>2]|0;
       $330 = ($329|0)<(2);
       if ($330) {
        label = 37;
        break L24;
       }
       $332 = HEAP32[$nCell>>2]|0;
       $333 = (($332) + 18)|0;
       $334 = ($333|0)<(0);
       $335 = $334 << 31 >> 31;
       $336 = (_sqlite3Malloc($333,$335)|0);
       HEAP32[$pCellKey>>2] = $336;
       $337 = HEAP32[$pCellKey>>2]|0;
       $338 = ($337|0)==(0|0);
       if ($338) {
        label = 39;
        break L24;
       }
       $339 = HEAP32[$idx>>2]|0;
       $340 = $339&65535;
       $341 = HEAP32[$3>>2]|0;
       $342 = ((($341)) + 68|0);
       $343 = HEAP8[$342>>0]|0;
       $344 = $343 << 24 >> 24;
       $345 = HEAP32[$3>>2]|0;
       $346 = ((($345)) + 80|0);
       $347 = (($346) + ($344<<1)|0);
       HEAP16[$347>>1] = $340;
       $348 = HEAP32[$3>>2]|0;
       $349 = HEAP32[$nCell>>2]|0;
       $350 = HEAP32[$pCellKey>>2]|0;
       $351 = (_accessPayload($348,0,$349,$350,2)|0);
       HEAP32[$rc>>2] = $351;
       $352 = HEAP32[$rc>>2]|0;
       $353 = ($352|0)!=(0);
       if ($353) {
        label = 41;
        break L24;
       }
       $355 = HEAP32[$xRecordCompare>>2]|0;
       $356 = HEAP32[$nCell>>2]|0;
       $357 = HEAP32[$pCellKey>>2]|0;
       $358 = HEAP32[$4>>2]|0;
       $359 = (FUNCTION_TABLE_iiii[$355 & 255]($356,$357,$358)|0);
       HEAP32[$c>>2] = $359;
       $360 = HEAP32[$pCellKey>>2]|0;
       _sqlite3_free($360);
      }
     } while(0);
     $361 = HEAP32[$c>>2]|0;
     $362 = ($361|0)<(0);
     if ($362) {
      $363 = HEAP32[$idx>>2]|0;
      $364 = (($363) + 1)|0;
      HEAP32[$lwr>>2] = $364;
     } else {
      $365 = HEAP32[$c>>2]|0;
      $366 = ($365|0)>(0);
      if (!($366)) {
       label = 47;
       break L24;
      }
      $367 = HEAP32[$idx>>2]|0;
      $368 = (($367) - 1)|0;
      HEAP32[$upr>>2] = $368;
     }
     $383 = HEAP32[$lwr>>2]|0;
     $384 = HEAP32[$upr>>2]|0;
     $385 = ($383|0)>($384|0);
     if ($385) {
      label = 51;
      break L26;
     }
     $386 = HEAP32[$lwr>>2]|0;
     $387 = HEAP32[$upr>>2]|0;
     $388 = (($386) + ($387))|0;
     $389 = $388 >> 1;
     HEAP32[$idx>>2] = $389;
    }
   }
  } while(0);
  if ((label|0) == 51) {
   label = 0;
   $390 = HEAP32[$pPage>>2]|0;
   $391 = ((($390)) + 4|0);
   $392 = HEAP8[$391>>0]|0;
   $393 = ($392<<24>>24)!=(0);
   if ($393) {
    label = 52;
    break;
   }
  }
  $405 = HEAP32[$lwr>>2]|0;
  $406 = HEAP32[$pPage>>2]|0;
  $407 = ((($406)) + 18|0);
  $408 = HEAP16[$407>>1]|0;
  $409 = $408&65535;
  $410 = ($405|0)>=($409|0);
  $411 = HEAP32[$pPage>>2]|0;
  if ($410) {
   $412 = ((($411)) + 5|0);
   $413 = HEAP8[$412>>0]|0;
   $414 = $413&255;
   $415 = (($414) + 8)|0;
   $416 = HEAP32[$pPage>>2]|0;
   $417 = ((($416)) + 56|0);
   $418 = HEAP32[$417>>2]|0;
   $419 = (($418) + ($415)|0);
   $420 = (_sqlite3Get4byte($419)|0);
   HEAP32[$chldPg>>2] = $420;
  } else {
   $421 = ((($411)) + 56|0);
   $422 = HEAP32[$421>>2]|0;
   $423 = HEAP32[$pPage>>2]|0;
   $424 = ((($423)) + 20|0);
   $425 = HEAP16[$424>>1]|0;
   $426 = $425&65535;
   $427 = HEAP32[$lwr>>2]|0;
   $428 = $427<<1;
   $429 = HEAP32[$pPage>>2]|0;
   $430 = ((($429)) + 64|0);
   $431 = HEAP32[$430>>2]|0;
   $432 = (($431) + ($428)|0);
   $433 = HEAP8[$432>>0]|0;
   $434 = $433&255;
   $435 = $434 << 8;
   $436 = HEAP32[$lwr>>2]|0;
   $437 = $436<<1;
   $438 = HEAP32[$pPage>>2]|0;
   $439 = ((($438)) + 64|0);
   $440 = HEAP32[$439>>2]|0;
   $441 = (($440) + ($437)|0);
   $442 = ((($441)) + 1|0);
   $443 = HEAP8[$442>>0]|0;
   $444 = $443&255;
   $445 = $435 | $444;
   $446 = $426 & $445;
   $447 = (($422) + ($446)|0);
   $448 = (_sqlite3Get4byte($447)|0);
   HEAP32[$chldPg>>2] = $448;
  }
  $449 = HEAP32[$lwr>>2]|0;
  $450 = $449&65535;
  $451 = HEAP32[$3>>2]|0;
  $452 = ((($451)) + 68|0);
  $453 = HEAP8[$452>>0]|0;
  $454 = $453 << 24 >> 24;
  $455 = HEAP32[$3>>2]|0;
  $456 = ((($455)) + 80|0);
  $457 = (($456) + ($454<<1)|0);
  HEAP16[$457>>1] = $450;
  $458 = HEAP32[$3>>2]|0;
  $459 = HEAP32[$chldPg>>2]|0;
  $460 = (_moveToChild($458,$459)|0);
  HEAP32[$rc>>2] = $460;
  $461 = HEAP32[$rc>>2]|0;
  $462 = ($461|0)!=(0);
  if ($462) {
   break;
  }
 }
 if ((label|0) == 20) {
  $156 = (_sqlite3CorruptError(63387)|0);
  HEAP32[$2>>2] = $156;
  $473 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($473|0);
 }
 else if ((label|0) == 29) {
  $234 = HEAP32[$7>>2]|0;
  HEAP32[$234>>2] = 0;
  HEAP32[$rc>>2] = 0;
 }
 else if ((label|0) == 37) {
  $331 = (_sqlite3CorruptError(63460)|0);
  HEAP32[$rc>>2] = $331;
 }
 else if ((label|0) == 39) {
  HEAP32[$rc>>2] = 7;
 }
 else if ((label|0) == 41) {
  $354 = HEAP32[$pCellKey>>2]|0;
  _sqlite3_free($354);
 }
 else if ((label|0) == 47) {
  $369 = HEAP32[$7>>2]|0;
  HEAP32[$369>>2] = 0;
  HEAP32[$rc>>2] = 0;
  $370 = HEAP32[$idx>>2]|0;
  $371 = $370&65535;
  $372 = HEAP32[$3>>2]|0;
  $373 = ((($372)) + 68|0);
  $374 = HEAP8[$373>>0]|0;
  $375 = $374 << 24 >> 24;
  $376 = HEAP32[$3>>2]|0;
  $377 = ((($376)) + 80|0);
  $378 = (($377) + ($375<<1)|0);
  HEAP16[$378>>1] = $371;
  $379 = HEAP32[$4>>2]|0;
  $380 = ((($379)) + 11|0);
  $381 = HEAP8[$380>>0]|0;
  $382 = ($381<<24>>24)!=(0);
  if ($382) {
   HEAP32[$rc>>2] = 11;
  }
 }
 else if ((label|0) == 52) {
  $394 = HEAP32[$idx>>2]|0;
  $395 = $394&65535;
  $396 = HEAP32[$3>>2]|0;
  $397 = ((($396)) + 68|0);
  $398 = HEAP8[$397>>0]|0;
  $399 = $398 << 24 >> 24;
  $400 = HEAP32[$3>>2]|0;
  $401 = ((($400)) + 80|0);
  $402 = (($401) + ($399<<1)|0);
  HEAP16[$402>>1] = $395;
  $403 = HEAP32[$c>>2]|0;
  $404 = HEAP32[$7>>2]|0;
  HEAP32[$404>>2] = $403;
  HEAP32[$rc>>2] = 0;
 }
 $463 = HEAP32[$3>>2]|0;
 $464 = ((($463)) + 16|0);
 $465 = ((($464)) + 18|0);
 HEAP16[$465>>1] = 0;
 $466 = HEAP32[$3>>2]|0;
 $467 = ((($466)) + 64|0);
 $468 = HEAP8[$467>>0]|0;
 $469 = $468&255;
 $470 = $469 & -7;
 $471 = $470&255;
 HEAP8[$467>>0] = $471;
 $472 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $472;
 $473 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($473|0);
}
function _sqlite3VdbeFindCompare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $flags = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $flags = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 6|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = (($6) + ($11))|0;
 $13 = ($12|0)<=(13);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP16[$17>>1]|0;
  $19 = $18&65535;
  HEAP32[$flags>>2] = $19;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$20>>2]|0;
  $22 = ((($21)) + 16|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  $25 = ($24<<24>>24)!=(0);
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 12|0);
  if ($25) {
   HEAP8[$27>>0] = 1;
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 13|0);
   HEAP8[$29>>0] = -1;
  } else {
   HEAP8[$27>>0] = -1;
   $30 = HEAP32[$1>>2]|0;
   $31 = ((($30)) + 13|0);
   HEAP8[$31>>0] = 1;
  }
  $32 = HEAP32[$flags>>2]|0;
  $33 = $32 & 4;
  $34 = ($33|0)!=(0);
  if ($34) {
   HEAP32[$0>>2] = 160;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  }
  $35 = HEAP32[$flags>>2]|0;
  $36 = $35 & 25;
  $37 = ($36|0)==(0);
  if ($37) {
   $38 = HEAP32[$1>>2]|0;
   $39 = HEAP32[$38>>2]|0;
   $40 = ((($39)) + 20|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = ($41|0)==(0|0);
   if ($42) {
    HEAP32[$0>>2] = 161;
    $43 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($43|0);
   }
  }
 }
 HEAP32[$0>>2] = 162;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _moveToRoot($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pRoot = 0, $rc = 0, $subpage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pRoot = sp + 8|0;
 $rc = sp + 4|0;
 $subpage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 66|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)>=(3);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 66|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9&255;
   $11 = ($10|0)==(4);
   $12 = HEAP32[$1>>2]|0;
   if (!($11)) {
    _sqlite3BtreeClearCursor($12);
    break;
   }
   $13 = ((($12)) + 60|0);
   $14 = HEAP32[$13>>2]|0;
   HEAP32[$0>>2] = $14;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 68|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17 << 24 >> 24;
 $19 = ($18|0)>=(0);
 L8: do {
  if ($19) {
   while(1) {
    $20 = HEAP32[$1>>2]|0;
    $21 = ((($20)) + 68|0);
    $22 = HEAP8[$21>>0]|0;
    $23 = ($22<<24>>24)!=(0);
    if (!($23)) {
     break L8;
    }
    $24 = HEAP32[$1>>2]|0;
    $25 = ((($24)) + 68|0);
    $26 = HEAP8[$25>>0]|0;
    $27 = (($26) + -1)<<24>>24;
    HEAP8[$25>>0] = $27;
    $28 = $26 << 24 >> 24;
    $29 = HEAP32[$1>>2]|0;
    $30 = ((($29)) + 120|0);
    $31 = (($30) + ($28<<2)|0);
    $32 = HEAP32[$31>>2]|0;
    _releasePageNotNull($32);
   }
  } else {
   $33 = HEAP32[$1>>2]|0;
   $34 = ((($33)) + 52|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)==(0);
   $37 = HEAP32[$1>>2]|0;
   if ($36) {
    $38 = ((($37)) + 66|0);
    HEAP8[$38>>0] = 0;
    HEAP32[$0>>2] = 0;
    $128 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($128|0);
   }
   $39 = HEAP32[$37>>2]|0;
   $40 = ((($39)) + 4|0);
   $41 = HEAP32[$40>>2]|0;
   $42 = HEAP32[$1>>2]|0;
   $43 = ((($42)) + 52|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$1>>2]|0;
   $46 = ((($45)) + 120|0);
   $47 = HEAP32[$1>>2]|0;
   $48 = ((($47)) + 65|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = $49&255;
   $51 = (_getAndInitPage($41,$44,$46,0,$50)|0);
   HEAP32[$rc>>2] = $51;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)!=(0);
   $54 = HEAP32[$1>>2]|0;
   if (!($53)) {
    $57 = ((($54)) + 68|0);
    HEAP8[$57>>0] = 0;
    $58 = HEAP32[$1>>2]|0;
    $59 = ((($58)) + 120|0);
    $60 = HEAP32[$59>>2]|0;
    $61 = ((($60)) + 2|0);
    $62 = HEAP8[$61>>0]|0;
    $63 = HEAP32[$1>>2]|0;
    $64 = ((($63)) + 69|0);
    HEAP8[$64>>0] = $62;
    break;
   }
   $55 = ((($54)) + 66|0);
   HEAP8[$55>>0] = 0;
   $56 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $56;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 120|0);
 $67 = HEAP32[$66>>2]|0;
 HEAP32[$pRoot>>2] = $67;
 $68 = HEAP32[$pRoot>>2]|0;
 $69 = HEAP8[$68>>0]|0;
 $70 = $69&255;
 $71 = ($70|0)==(0);
 if (!($71)) {
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 72|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = ($74|0)==(0|0);
  $76 = $75&1;
  $77 = HEAP32[$pRoot>>2]|0;
  $78 = ((($77)) + 2|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = ($76|0)!=($80|0);
  if (!($81)) {
   $83 = HEAP32[$1>>2]|0;
   $84 = ((($83)) + 80|0);
   HEAP16[$84>>1] = 0;
   $85 = HEAP32[$1>>2]|0;
   $86 = ((($85)) + 16|0);
   $87 = ((($86)) + 18|0);
   HEAP16[$87>>1] = 0;
   $88 = HEAP32[$1>>2]|0;
   $89 = ((($88)) + 64|0);
   $90 = HEAP8[$89>>0]|0;
   $91 = $90&255;
   $92 = $91 & -15;
   $93 = $92&255;
   HEAP8[$89>>0] = $93;
   $94 = HEAP32[$pRoot>>2]|0;
   $95 = ((($94)) + 18|0);
   $96 = HEAP16[$95>>1]|0;
   $97 = $96&65535;
   $98 = ($97|0)>(0);
   do {
    if ($98) {
     $99 = HEAP32[$1>>2]|0;
     $100 = ((($99)) + 66|0);
     HEAP8[$100>>0] = 1;
    } else {
     $101 = HEAP32[$pRoot>>2]|0;
     $102 = ((($101)) + 4|0);
     $103 = HEAP8[$102>>0]|0;
     $104 = ($103<<24>>24)!=(0);
     if ($104) {
      $125 = HEAP32[$1>>2]|0;
      $126 = ((($125)) + 66|0);
      HEAP8[$126>>0] = 0;
      break;
     }
     $105 = HEAP32[$pRoot>>2]|0;
     $106 = ((($105)) + 84|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = ($107|0)!=(1);
     if (!($108)) {
      $110 = HEAP32[$pRoot>>2]|0;
      $111 = ((($110)) + 5|0);
      $112 = HEAP8[$111>>0]|0;
      $113 = $112&255;
      $114 = (($113) + 8)|0;
      $115 = HEAP32[$pRoot>>2]|0;
      $116 = ((($115)) + 56|0);
      $117 = HEAP32[$116>>2]|0;
      $118 = (($117) + ($114)|0);
      $119 = (_sqlite3Get4byte($118)|0);
      HEAP32[$subpage>>2] = $119;
      $120 = HEAP32[$1>>2]|0;
      $121 = ((($120)) + 66|0);
      HEAP8[$121>>0] = 1;
      $122 = HEAP32[$1>>2]|0;
      $123 = HEAP32[$subpage>>2]|0;
      $124 = (_moveToChild($122,$123)|0);
      HEAP32[$rc>>2] = $124;
      break;
     }
     $109 = (_sqlite3CorruptError(63147)|0);
     HEAP32[$0>>2] = $109;
     $128 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($128|0);
    }
   } while(0);
   $127 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $127;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 }
 $82 = (_sqlite3CorruptError(63136)|0);
 HEAP32[$0>>2] = $82;
 $128 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($128|0);
}
function _accessPayload($pCur,$offset,$amt,$pBuf,$eOp) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 $eOp = $eOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $a = 0;
 var $a1 = 0, $aNew = 0, $aPayload = 0, $iIdx = 0, $nOvfl = 0, $nextPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $ovflSize = 0, $pBt = 0, $pDbPage = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $aPayload = sp + 44|0;
 $rc = sp + 40|0;
 $iIdx = sp + 36|0;
 $pPage = sp + 32|0;
 $pBt = sp + 28|0;
 $a = sp + 24|0;
 $ovflSize = sp + 20|0;
 $nextPage = sp + 16|0;
 $nOvfl = sp + 12|0;
 $aNew = sp + 8|0;
 $a1 = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $offset;
 HEAP32[$3>>2] = $amt;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $eOp;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iIdx>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 68|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 120|0);
 $12 = (($11) + ($9<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pPage>>2] = $13;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pBt>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 _getCellInfo($17);
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 16|0);
 $20 = ((($19)) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$aPayload>>2] = $21;
 $22 = HEAP32[$aPayload>>2]|0;
 $23 = HEAP32[$pPage>>2]|0;
 $24 = ((($23)) + 56|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = $22;
 $27 = $25;
 $28 = (($26) - ($27))|0;
 $29 = HEAP32[$pBt>>2]|0;
 $30 = ((($29)) + 36|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 16|0);
 $34 = ((($33)) + 16|0);
 $35 = HEAP16[$34>>1]|0;
 $36 = $35&65535;
 $37 = (($31) - ($36))|0;
 $38 = ($28>>>0)>($37>>>0);
 if ($38) {
  $39 = (_sqlite3CorruptError(62723)|0);
  HEAP32[$0>>2] = $39;
  $278 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($278|0);
 }
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 16|0);
 $43 = ((($42)) + 16|0);
 $44 = HEAP16[$43>>1]|0;
 $45 = $44&65535;
 $46 = ($40>>>0)<($45>>>0);
 if ($46) {
  $47 = HEAP32[$3>>2]|0;
  HEAP32[$a>>2] = $47;
  $48 = HEAP32[$a>>2]|0;
  $49 = HEAP32[$2>>2]|0;
  $50 = (($48) + ($49))|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = ((($51)) + 16|0);
  $53 = ((($52)) + 16|0);
  $54 = HEAP16[$53>>1]|0;
  $55 = $54&65535;
  $56 = ($50>>>0)>($55>>>0);
  if ($56) {
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 16|0);
   $59 = ((($58)) + 16|0);
   $60 = HEAP16[$59>>1]|0;
   $61 = $60&65535;
   $62 = HEAP32[$2>>2]|0;
   $63 = (($61) - ($62))|0;
   HEAP32[$a>>2] = $63;
  }
  $64 = HEAP32[$2>>2]|0;
  $65 = HEAP32[$aPayload>>2]|0;
  $66 = (($65) + ($64)|0);
  $67 = HEAP32[$4>>2]|0;
  $68 = HEAP32[$a>>2]|0;
  $69 = HEAP32[$5>>2]|0;
  $70 = $69 & 1;
  $71 = HEAP32[$pPage>>2]|0;
  $72 = ((($71)) + 72|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (_copyPayload($66,$67,$68,$70,$73)|0);
  HEAP32[$rc>>2] = $74;
  HEAP32[$2>>2] = 0;
  $75 = HEAP32[$a>>2]|0;
  $76 = HEAP32[$4>>2]|0;
  $77 = (($76) + ($75)|0);
  HEAP32[$4>>2] = $77;
  $78 = HEAP32[$a>>2]|0;
  $79 = HEAP32[$3>>2]|0;
  $80 = (($79) - ($78))|0;
  HEAP32[$3>>2] = $80;
 } else {
  $81 = HEAP32[$1>>2]|0;
  $82 = ((($81)) + 16|0);
  $83 = ((($82)) + 16|0);
  $84 = HEAP16[$83>>1]|0;
  $85 = $84&65535;
  $86 = HEAP32[$2>>2]|0;
  $87 = (($86) - ($85))|0;
  HEAP32[$2>>2] = $87;
 }
 $88 = HEAP32[$rc>>2]|0;
 $89 = ($88|0)==(0);
 $90 = HEAP32[$3>>2]|0;
 $91 = ($90>>>0)>(0);
 $or$cond = $89 & $91;
 L12: do {
  if ($or$cond) {
   $92 = HEAP32[$pBt>>2]|0;
   $93 = ((($92)) + 36|0);
   $94 = HEAP32[$93>>2]|0;
   $95 = (($94) - 4)|0;
   HEAP32[$ovflSize>>2] = $95;
   $96 = HEAP32[$1>>2]|0;
   $97 = ((($96)) + 16|0);
   $98 = ((($97)) + 16|0);
   $99 = HEAP16[$98>>1]|0;
   $100 = $99&65535;
   $101 = HEAP32[$aPayload>>2]|0;
   $102 = (($101) + ($100)|0);
   $103 = (_sqlite3Get4byte($102)|0);
   HEAP32[$nextPage>>2] = $103;
   $104 = HEAP32[$5>>2]|0;
   $105 = ($104|0)!=(2);
   if ($105) {
    $106 = HEAP32[$1>>2]|0;
    $107 = ((($106)) + 64|0);
    $108 = HEAP8[$107>>0]|0;
    $109 = $108&255;
    $110 = $109 & 4;
    $111 = ($110|0)==(0);
    if ($111) {
     $112 = HEAP32[$1>>2]|0;
     $113 = ((($112)) + 16|0);
     $114 = ((($113)) + 12|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = HEAP32[$1>>2]|0;
     $117 = ((($116)) + 16|0);
     $118 = ((($117)) + 16|0);
     $119 = HEAP16[$118>>1]|0;
     $120 = $119&65535;
     $121 = (($115) - ($120))|0;
     $122 = HEAP32[$ovflSize>>2]|0;
     $123 = (($121) + ($122))|0;
     $124 = (($123) - 1)|0;
     $125 = HEAP32[$ovflSize>>2]|0;
     $126 = (($124>>>0) / ($125>>>0))&-1;
     HEAP32[$nOvfl>>2] = $126;
     $127 = HEAP32[$nOvfl>>2]|0;
     $128 = HEAP32[$1>>2]|0;
     $129 = ((($128)) + 56|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = ($127|0)>($130|0);
     do {
      if ($131) {
       $132 = HEAP32[$1>>2]|0;
       $133 = ((($132)) + 12|0);
       $134 = HEAP32[$133>>2]|0;
       $135 = HEAP32[$nOvfl>>2]|0;
       $136 = $135<<1;
       $137 = $136<<2;
       $138 = (_sqlite3Realloc($134,$137,0)|0);
       HEAP32[$aNew>>2] = $138;
       $139 = HEAP32[$aNew>>2]|0;
       $140 = ($139|0)==(0|0);
       if ($140) {
        HEAP32[$rc>>2] = 7;
        break;
       } else {
        $141 = HEAP32[$nOvfl>>2]|0;
        $142 = $141<<1;
        $143 = HEAP32[$1>>2]|0;
        $144 = ((($143)) + 56|0);
        HEAP32[$144>>2] = $142;
        $145 = HEAP32[$aNew>>2]|0;
        $146 = HEAP32[$1>>2]|0;
        $147 = ((($146)) + 12|0);
        HEAP32[$147>>2] = $145;
        break;
       }
      }
     } while(0);
     $148 = HEAP32[$rc>>2]|0;
     $149 = ($148|0)==(0);
     if ($149) {
      $150 = HEAP32[$1>>2]|0;
      $151 = ((($150)) + 12|0);
      $152 = HEAP32[$151>>2]|0;
      $153 = HEAP32[$nOvfl>>2]|0;
      $154 = $153<<2;
      _memset(($152|0),0,($154|0))|0;
      $155 = HEAP32[$1>>2]|0;
      $156 = ((($155)) + 64|0);
      $157 = HEAP8[$156>>0]|0;
      $158 = $157&255;
      $159 = $158 | 4;
      $160 = $159&255;
      HEAP8[$156>>0] = $160;
     }
    }
   }
   $161 = HEAP32[$1>>2]|0;
   $162 = ((($161)) + 64|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163&255;
   $165 = $164 & 4;
   $166 = ($165|0)!=(0);
   if ($166) {
    $167 = HEAP32[$2>>2]|0;
    $168 = HEAP32[$ovflSize>>2]|0;
    $169 = (($167>>>0) / ($168>>>0))&-1;
    $170 = HEAP32[$1>>2]|0;
    $171 = ((($170)) + 12|0);
    $172 = HEAP32[$171>>2]|0;
    $173 = (($172) + ($169<<2)|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = ($174|0)!=(0);
    if ($175) {
     $176 = HEAP32[$2>>2]|0;
     $177 = HEAP32[$ovflSize>>2]|0;
     $178 = (($176>>>0) / ($177>>>0))&-1;
     HEAP32[$iIdx>>2] = $178;
     $179 = HEAP32[$iIdx>>2]|0;
     $180 = HEAP32[$1>>2]|0;
     $181 = ((($180)) + 12|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = (($182) + ($179<<2)|0);
     $184 = HEAP32[$183>>2]|0;
     HEAP32[$nextPage>>2] = $184;
     $185 = HEAP32[$2>>2]|0;
     $186 = HEAP32[$ovflSize>>2]|0;
     $187 = (($185>>>0) % ($186>>>0))&-1;
     HEAP32[$2>>2] = $187;
    }
   }
   while(1) {
    $188 = HEAP32[$rc>>2]|0;
    $189 = ($188|0)==(0);
    $190 = HEAP32[$3>>2]|0;
    $191 = ($190>>>0)>(0);
    $or$cond3 = $189 & $191;
    $192 = HEAP32[$nextPage>>2]|0;
    $193 = ($192|0)!=(0);
    $or$cond7 = $or$cond3 & $193;
    if (!($or$cond7)) {
     break L12;
    }
    $194 = HEAP32[$1>>2]|0;
    $195 = ((($194)) + 64|0);
    $196 = HEAP8[$195>>0]|0;
    $197 = $196&255;
    $198 = $197 & 4;
    $199 = ($198|0)!=(0);
    if ($199) {
     $200 = HEAP32[$nextPage>>2]|0;
     $201 = HEAP32[$iIdx>>2]|0;
     $202 = HEAP32[$1>>2]|0;
     $203 = ((($202)) + 12|0);
     $204 = HEAP32[$203>>2]|0;
     $205 = (($204) + ($201<<2)|0);
     HEAP32[$205>>2] = $200;
    }
    $206 = HEAP32[$2>>2]|0;
    $207 = HEAP32[$ovflSize>>2]|0;
    $208 = ($206>>>0)>=($207>>>0);
    if ($208) {
     $209 = HEAP32[$iIdx>>2]|0;
     $210 = (($209) + 1)|0;
     $211 = HEAP32[$1>>2]|0;
     $212 = ((($211)) + 12|0);
     $213 = HEAP32[$212>>2]|0;
     $214 = (($213) + ($210<<2)|0);
     $215 = HEAP32[$214>>2]|0;
     $216 = ($215|0)!=(0);
     if ($216) {
      $217 = HEAP32[$iIdx>>2]|0;
      $218 = (($217) + 1)|0;
      $219 = HEAP32[$1>>2]|0;
      $220 = ((($219)) + 12|0);
      $221 = HEAP32[$220>>2]|0;
      $222 = (($221) + ($218<<2)|0);
      $223 = HEAP32[$222>>2]|0;
      HEAP32[$nextPage>>2] = $223;
     } else {
      $224 = HEAP32[$pBt>>2]|0;
      $225 = HEAP32[$nextPage>>2]|0;
      $226 = (_getOverflowPage($224,$225,0,$nextPage)|0);
      HEAP32[$rc>>2] = $226;
     }
     $227 = HEAP32[$ovflSize>>2]|0;
     $228 = HEAP32[$2>>2]|0;
     $229 = (($228) - ($227))|0;
     HEAP32[$2>>2] = $229;
    } else {
     $230 = HEAP32[$3>>2]|0;
     HEAP32[$a1>>2] = $230;
     $231 = HEAP32[$a1>>2]|0;
     $232 = HEAP32[$2>>2]|0;
     $233 = (($231) + ($232))|0;
     $234 = HEAP32[$ovflSize>>2]|0;
     $235 = ($233>>>0)>($234>>>0);
     if ($235) {
      $236 = HEAP32[$ovflSize>>2]|0;
      $237 = HEAP32[$2>>2]|0;
      $238 = (($236) - ($237))|0;
      HEAP32[$a1>>2] = $238;
     }
     $239 = HEAP32[$pBt>>2]|0;
     $240 = HEAP32[$239>>2]|0;
     $241 = HEAP32[$nextPage>>2]|0;
     $242 = HEAP32[$5>>2]|0;
     $243 = $242 & 1;
     $244 = ($243|0)==(0);
     $245 = $244 ? 2 : 0;
     $246 = (_sqlite3PagerGet($240,$241,$pDbPage,$245)|0);
     HEAP32[$rc>>2] = $246;
     $247 = HEAP32[$rc>>2]|0;
     $248 = ($247|0)==(0);
     if ($248) {
      $249 = HEAP32[$pDbPage>>2]|0;
      $250 = (_sqlite3PagerGetData($249)|0);
      HEAP32[$aPayload>>2] = $250;
      $251 = HEAP32[$aPayload>>2]|0;
      $252 = (_sqlite3Get4byte($251)|0);
      HEAP32[$nextPage>>2] = $252;
      $253 = HEAP32[$2>>2]|0;
      $254 = (($253) + 4)|0;
      $255 = HEAP32[$aPayload>>2]|0;
      $256 = (($255) + ($254)|0);
      $257 = HEAP32[$4>>2]|0;
      $258 = HEAP32[$a1>>2]|0;
      $259 = HEAP32[$5>>2]|0;
      $260 = $259 & 1;
      $261 = HEAP32[$pDbPage>>2]|0;
      $262 = (_copyPayload($256,$257,$258,$260,$261)|0);
      HEAP32[$rc>>2] = $262;
      $263 = HEAP32[$pDbPage>>2]|0;
      _sqlite3PagerUnref($263);
      HEAP32[$2>>2] = 0;
     }
     $264 = HEAP32[$a1>>2]|0;
     $265 = HEAP32[$3>>2]|0;
     $266 = (($265) - ($264))|0;
     HEAP32[$3>>2] = $266;
     $267 = HEAP32[$a1>>2]|0;
     $268 = HEAP32[$4>>2]|0;
     $269 = (($268) + ($267)|0);
     HEAP32[$4>>2] = $269;
    }
    $270 = HEAP32[$iIdx>>2]|0;
    $271 = (($270) + 1)|0;
    HEAP32[$iIdx>>2] = $271;
   }
  }
 } while(0);
 $272 = HEAP32[$rc>>2]|0;
 $273 = ($272|0)==(0);
 $274 = HEAP32[$3>>2]|0;
 $275 = ($274>>>0)>(0);
 $or$cond5 = $273 & $275;
 if ($or$cond5) {
  $276 = (_sqlite3CorruptError(62880)|0);
  HEAP32[$0>>2] = $276;
  $278 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($278|0);
 } else {
  $277 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $277;
  $278 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($278|0);
 }
 return (0)|0;
}
function _getCellInfo($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $iPage = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = ((($2)) + 18|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = ($5|0)==(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 HEAP32[$iPage>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 | 2;
 $16 = $15&255;
 HEAP8[$12>>0] = $16;
 $17 = HEAP32[$iPage>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 120|0);
 $20 = (($19) + ($17<<2)|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$iPage>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 80|0);
 $25 = (($24) + ($22<<1)|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 16|0);
 _btreeParseCell($21,$27,$29);
 STACKTOP = sp;return;
}
function _copyPayload($pPayload,$pBuf,$nByte,$eOp,$pDbPage) {
 $pPayload = $pPayload|0;
 $pBuf = $pBuf|0;
 $nByte = $nByte|0;
 $eOp = $eOp|0;
 $pDbPage = $pDbPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPayload;
 HEAP32[$2>>2] = $pBuf;
 HEAP32[$3>>2] = $nByte;
 HEAP32[$4>>2] = $eOp;
 HEAP32[$5>>2] = $pDbPage;
 $6 = HEAP32[$4>>2]|0;
 $7 = ($6|0)!=(0);
 do {
  if ($7) {
   $8 = HEAP32[$5>>2]|0;
   $9 = (_sqlite3PagerWrite($8)|0);
   HEAP32[$rc>>2] = $9;
   $10 = HEAP32[$rc>>2]|0;
   $11 = ($10|0)!=(0);
   if (!($11)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = HEAP32[$2>>2]|0;
    $15 = HEAP32[$3>>2]|0;
    _memcpy(($13|0),($14|0),($15|0))|0;
    break;
   }
   $12 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $12;
   $19 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($19|0);
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$3>>2]|0;
   _memcpy(($16|0),($17|0),($18|0))|0;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($19|0);
}
function _getOverflowPage($pBt,$ovfl,$ppPage,$pPgnoNext) {
 $pBt = $pBt|0;
 $ovfl = $ovfl|0;
 $ppPage = $ppPage|0;
 $pPgnoNext = $pPgnoNext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $iGuess = 0, $next = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $next = sp + 16|0;
 $pPage = sp + 12|0;
 $rc = sp + 8|0;
 $pgno = sp + 4|0;
 $iGuess = sp;
 $eType = sp + 36|0;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $ovfl;
 HEAP32[$2>>2] = $ppPage;
 HEAP32[$3>>2] = $pPgnoNext;
 HEAP32[$next>>2] = 0;
 HEAP32[$pPage>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 17|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + 1)|0;
  HEAP32[$iGuess>>2] = $9;
  while(1) {
   $10 = HEAP32[$0>>2]|0;
   $11 = HEAP32[$iGuess>>2]|0;
   $12 = (_ptrmapPageno($10,$11)|0);
   $13 = HEAP32[$iGuess>>2]|0;
   $14 = ($12|0)==($13|0);
   if ($14) {
    $64 = 1;
   } else {
    $15 = HEAP32[$iGuess>>2]|0;
    $16 = HEAP32[1536>>2]|0;
    $17 = HEAP32[$0>>2]|0;
    $18 = ((($17)) + 32|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = (($16>>>0) / ($19>>>0))&-1;
    $21 = (($20) + 1)|0;
    $22 = ($15|0)==($21|0);
    $64 = $22;
   }
   $23 = HEAP32[$iGuess>>2]|0;
   if (!($64)) {
    break;
   }
   $24 = (($23) + 1)|0;
   HEAP32[$iGuess>>2] = $24;
  }
  $25 = HEAP32[$0>>2]|0;
  $26 = (_btreePagecount($25)|0);
  $27 = ($23>>>0)<=($26>>>0);
  if ($27) {
   $28 = HEAP32[$0>>2]|0;
   $29 = HEAP32[$iGuess>>2]|0;
   $30 = (_ptrmapGet($28,$29,$eType,$pgno)|0);
   HEAP32[$rc>>2] = $30;
   $31 = HEAP32[$rc>>2]|0;
   $32 = ($31|0)==(0);
   if ($32) {
    $33 = HEAP8[$eType>>0]|0;
    $34 = $33&255;
    $35 = ($34|0)==(4);
    if ($35) {
     $36 = HEAP32[$pgno>>2]|0;
     $37 = HEAP32[$1>>2]|0;
     $38 = ($36|0)==($37|0);
     if ($38) {
      $39 = HEAP32[$iGuess>>2]|0;
      HEAP32[$next>>2] = $39;
      HEAP32[$rc>>2] = 101;
     }
    }
   }
  }
 }
 $40 = HEAP32[$rc>>2]|0;
 $41 = ($40|0)==(0);
 if ($41) {
  $42 = HEAP32[$0>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = HEAP32[$2>>2]|0;
  $45 = ($44|0)==(0|0);
  $46 = $45 ? 2 : 0;
  $47 = (_btreeGetPage($42,$43,$pPage,$46)|0);
  HEAP32[$rc>>2] = $47;
  $48 = HEAP32[$rc>>2]|0;
  $49 = ($48|0)==(0);
  if ($49) {
   $50 = HEAP32[$pPage>>2]|0;
   $51 = ((($50)) + 56|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3Get4byte($52)|0);
   HEAP32[$next>>2] = $53;
  }
 }
 $54 = HEAP32[$next>>2]|0;
 $55 = HEAP32[$3>>2]|0;
 HEAP32[$55>>2] = $54;
 $56 = HEAP32[$2>>2]|0;
 $57 = ($56|0)!=(0|0);
 $58 = HEAP32[$pPage>>2]|0;
 if ($57) {
  $59 = HEAP32[$2>>2]|0;
  HEAP32[$59>>2] = $58;
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(101);
  $62 = HEAP32[$rc>>2]|0;
  $63 = $61 ? 0 : $62;
  STACKTOP = sp;return ($63|0);
 } else {
  _releasePage($58);
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)==(101);
  $62 = HEAP32[$rc>>2]|0;
  $63 = $61 ? 0 : $62;
  STACKTOP = sp;return ($63|0);
 }
 return (0)|0;
}
function _ptrmapPageno($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrMap = 0, $nPagesPerMapPage = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nPagesPerMapPage = sp + 8|0;
 $iPtrMap = sp + 4|0;
 $ret = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3>>>0)<(2);
 if ($4) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7>>>0) / 5)&-1;
 $9 = (($8) + 1)|0;
 HEAP32[$nPagesPerMapPage>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($10) - 2)|0;
 $12 = HEAP32[$nPagesPerMapPage>>2]|0;
 $13 = (($11>>>0) / ($12>>>0))&-1;
 HEAP32[$iPtrMap>>2] = $13;
 $14 = HEAP32[$iPtrMap>>2]|0;
 $15 = HEAP32[$nPagesPerMapPage>>2]|0;
 $16 = Math_imul($14, $15)|0;
 $17 = (($16) + 2)|0;
 HEAP32[$ret>>2] = $17;
 $18 = HEAP32[$ret>>2]|0;
 $19 = HEAP32[1536>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 32|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (($19>>>0) / ($22>>>0))&-1;
 $24 = (($23) + 1)|0;
 $25 = ($18|0)==($24|0);
 if ($25) {
  $26 = HEAP32[$ret>>2]|0;
  $27 = (($26) + 1)|0;
  HEAP32[$ret>>2] = $27;
 }
 $28 = HEAP32[$ret>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _ptrmapGet($pBt,$key,$pEType,$pPgno) {
 $pBt = $pBt|0;
 $key = $key|0;
 $pEType = $pEType|0;
 $pPgno = $pPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrmap = 0, $offset = 0, $pDbPage = 0, $pPtrmap = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pDbPage = sp + 16|0;
 $iPtrmap = sp + 12|0;
 $pPtrmap = sp + 8|0;
 $offset = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $key;
 HEAP32[$3>>2] = $pEType;
 HEAP32[$4>>2] = $pPgno;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_ptrmapPageno($5,$6)|0);
 HEAP32[$iPtrmap>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$iPtrmap>>2]|0;
 $11 = (_sqlite3PagerGet($9,$10,$pDbPage,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $15 = HEAP32[$pDbPage>>2]|0;
 $16 = (_sqlite3PagerGetData($15)|0);
 HEAP32[$pPtrmap>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$iPtrmap>>2]|0;
 $19 = (($17) - ($18))|0;
 $20 = (($19) - 1)|0;
 $21 = ($20*5)|0;
 HEAP32[$offset>>2] = $21;
 $22 = HEAP32[$offset>>2]|0;
 $23 = ($22|0)<(0);
 if ($23) {
  $24 = HEAP32[$pDbPage>>2]|0;
  _sqlite3PagerUnref($24);
  $25 = (_sqlite3CorruptError(59240)|0);
  HEAP32[$0>>2] = $25;
  $49 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $26 = HEAP32[$offset>>2]|0;
 $27 = HEAP32[$pPtrmap>>2]|0;
 $28 = (($27) + ($26)|0);
 $29 = HEAP8[$28>>0]|0;
 $30 = HEAP32[$3>>2]|0;
 HEAP8[$30>>0] = $29;
 $31 = HEAP32[$4>>2]|0;
 $32 = ($31|0)!=(0|0);
 if ($32) {
  $33 = HEAP32[$offset>>2]|0;
  $34 = (($33) + 1)|0;
  $35 = HEAP32[$pPtrmap>>2]|0;
  $36 = (($35) + ($34)|0);
  $37 = (_sqlite3Get4byte($36)|0);
  $38 = HEAP32[$4>>2]|0;
  HEAP32[$38>>2] = $37;
 }
 $39 = HEAP32[$pDbPage>>2]|0;
 _sqlite3PagerUnref($39);
 $40 = HEAP32[$3>>2]|0;
 $41 = HEAP8[$40>>0]|0;
 $42 = $41&255;
 $43 = ($42|0)<(1);
 if (!($43)) {
  $44 = HEAP32[$3>>2]|0;
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = ($46|0)>(5);
  if (!($47)) {
   HEAP32[$0>>2] = 0;
   $49 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($49|0);
  }
 }
 $48 = (_sqlite3CorruptError(59248)|0);
 HEAP32[$0>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _btreeGetPage($pBt,$pgno,$ppPage,$flags) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 $9 = (_sqlite3PagerGet($6,$7,$pDbPage,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $13 = HEAP32[$pDbPage>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (_btreePageFromDbPage($13,$14,$15)|0);
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _btreeParseCell($pPage,$iCell,$pInfo) {
 $pPage = $pPage|0;
 $iCell = $iCell|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $iCell;
 HEAP32[$2>>2] = $pInfo;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 80|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 56|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = HEAP32[$1>>2]|0;
 $15 = $14<<1;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = (($18) + ($15)|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = $21 << 8;
 $23 = HEAP32[$1>>2]|0;
 $24 = $23<<1;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 64|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + ($24)|0);
 $29 = ((($28)) + 1|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = $22 | $31;
 $33 = $13 & $32;
 $34 = (($9) + ($33)|0);
 $35 = HEAP32[$2>>2]|0;
 FUNCTION_TABLE_viii[$5 & 255]($6,$34,$35);
 STACKTOP = sp;return;
}
function _vdbeRecordCompareInt($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey = 0, $lhs = 0, $res = 0, $serial_type = 0, $v = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $aKey = sp + 36|0;
 $serial_type = sp + 32|0;
 $res = sp + 28|0;
 $y = sp + 24|0;
 $x = sp + 16|0;
 $v = sp + 8|0;
 $lhs = sp;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = $6 & 63;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + ($7)|0);
 HEAP32[$aKey>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 1|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 HEAP32[$serial_type>>2] = $13;
 $14 = HEAP32[$serial_type>>2]|0;
 do {
  switch ($14|0) {
  case 1:  {
   $15 = HEAP32[$aKey>>2]|0;
   $16 = HEAP8[$15>>0]|0;
   $17 = $16 << 24 >> 24;
   $18 = ($17|0)<(0);
   $19 = $18 << 31 >> 31;
   $20 = $lhs;
   $21 = $20;
   HEAP32[$21>>2] = $17;
   $22 = (($20) + 4)|0;
   $23 = $22;
   HEAP32[$23>>2] = $19;
   break;
  }
  case 2:  {
   $24 = HEAP32[$aKey>>2]|0;
   $25 = HEAP8[$24>>0]|0;
   $26 = $25 << 24 >> 24;
   $27 = $26<<8;
   $28 = HEAP32[$aKey>>2]|0;
   $29 = ((($28)) + 1|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = $27 | $31;
   $33 = ($32|0)<(0);
   $34 = $33 << 31 >> 31;
   $35 = $lhs;
   $36 = $35;
   HEAP32[$36>>2] = $32;
   $37 = (($35) + 4)|0;
   $38 = $37;
   HEAP32[$38>>2] = $34;
   break;
  }
  case 3:  {
   $39 = HEAP32[$aKey>>2]|0;
   $40 = HEAP8[$39>>0]|0;
   $41 = $40 << 24 >> 24;
   $42 = $41<<16;
   $43 = HEAP32[$aKey>>2]|0;
   $44 = ((($43)) + 1|0);
   $45 = HEAP8[$44>>0]|0;
   $46 = $45&255;
   $47 = $46 << 8;
   $48 = $42 | $47;
   $49 = HEAP32[$aKey>>2]|0;
   $50 = ((($49)) + 2|0);
   $51 = HEAP8[$50>>0]|0;
   $52 = $51&255;
   $53 = $48 | $52;
   $54 = ($53|0)<(0);
   $55 = $54 << 31 >> 31;
   $56 = $lhs;
   $57 = $56;
   HEAP32[$57>>2] = $53;
   $58 = (($56) + 4)|0;
   $59 = $58;
   HEAP32[$59>>2] = $55;
   break;
  }
  case 4:  {
   $60 = HEAP32[$aKey>>2]|0;
   $61 = HEAP8[$60>>0]|0;
   $62 = $61&255;
   $63 = $62 << 24;
   $64 = HEAP32[$aKey>>2]|0;
   $65 = ((($64)) + 1|0);
   $66 = HEAP8[$65>>0]|0;
   $67 = $66&255;
   $68 = $67 << 16;
   $69 = $63 | $68;
   $70 = HEAP32[$aKey>>2]|0;
   $71 = ((($70)) + 2|0);
   $72 = HEAP8[$71>>0]|0;
   $73 = $72&255;
   $74 = $73 << 8;
   $75 = $69 | $74;
   $76 = HEAP32[$aKey>>2]|0;
   $77 = ((($76)) + 3|0);
   $78 = HEAP8[$77>>0]|0;
   $79 = $78&255;
   $80 = $75 | $79;
   HEAP32[$y>>2] = $80;
   $81 = HEAP32[$y>>2]|0;
   $82 = ($81|0)<(0);
   $83 = $82 << 31 >> 31;
   $84 = $lhs;
   $85 = $84;
   HEAP32[$85>>2] = $81;
   $86 = (($84) + 4)|0;
   $87 = $86;
   HEAP32[$87>>2] = $83;
   break;
  }
  case 5:  {
   $88 = HEAP32[$aKey>>2]|0;
   $89 = ((($88)) + 2|0);
   $90 = HEAP8[$89>>0]|0;
   $91 = $90&255;
   $92 = $91 << 24;
   $93 = HEAP32[$aKey>>2]|0;
   $94 = ((($93)) + 2|0);
   $95 = ((($94)) + 1|0);
   $96 = HEAP8[$95>>0]|0;
   $97 = $96&255;
   $98 = $97 << 16;
   $99 = $92 | $98;
   $100 = HEAP32[$aKey>>2]|0;
   $101 = ((($100)) + 2|0);
   $102 = ((($101)) + 2|0);
   $103 = HEAP8[$102>>0]|0;
   $104 = $103&255;
   $105 = $104 << 8;
   $106 = $99 | $105;
   $107 = HEAP32[$aKey>>2]|0;
   $108 = ((($107)) + 2|0);
   $109 = ((($108)) + 3|0);
   $110 = HEAP8[$109>>0]|0;
   $111 = $110&255;
   $112 = $106 | $111;
   $113 = HEAP32[$aKey>>2]|0;
   $114 = HEAP8[$113>>0]|0;
   $115 = $114 << 24 >> 24;
   $116 = $115<<8;
   $117 = HEAP32[$aKey>>2]|0;
   $118 = ((($117)) + 1|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = $119&255;
   $121 = $116 | $120;
   $122 = ($121|0)<(0);
   $123 = $122 << 31 >> 31;
   $124 = (___muldi3(0,1,($121|0),($123|0))|0);
   $125 = tempRet0;
   $126 = (_i64Add(($112|0),0,($124|0),($125|0))|0);
   $127 = tempRet0;
   $128 = $lhs;
   $129 = $128;
   HEAP32[$129>>2] = $126;
   $130 = (($128) + 4)|0;
   $131 = $130;
   HEAP32[$131>>2] = $127;
   break;
  }
  case 6:  {
   $132 = HEAP32[$aKey>>2]|0;
   $133 = HEAP8[$132>>0]|0;
   $134 = $133&255;
   $135 = $134 << 24;
   $136 = HEAP32[$aKey>>2]|0;
   $137 = ((($136)) + 1|0);
   $138 = HEAP8[$137>>0]|0;
   $139 = $138&255;
   $140 = $139 << 16;
   $141 = $135 | $140;
   $142 = HEAP32[$aKey>>2]|0;
   $143 = ((($142)) + 2|0);
   $144 = HEAP8[$143>>0]|0;
   $145 = $144&255;
   $146 = $145 << 8;
   $147 = $141 | $146;
   $148 = HEAP32[$aKey>>2]|0;
   $149 = ((($148)) + 3|0);
   $150 = HEAP8[$149>>0]|0;
   $151 = $150&255;
   $152 = $147 | $151;
   $153 = $x;
   $154 = $153;
   HEAP32[$154>>2] = $152;
   $155 = (($153) + 4)|0;
   $156 = $155;
   HEAP32[$156>>2] = 0;
   $157 = $x;
   $158 = $157;
   $159 = HEAP32[$158>>2]|0;
   $160 = (($157) + 4)|0;
   $161 = $160;
   $162 = HEAP32[$161>>2]|0;
   $163 = HEAP32[$aKey>>2]|0;
   $164 = ((($163)) + 4|0);
   $165 = HEAP8[$164>>0]|0;
   $166 = $165&255;
   $167 = $166 << 24;
   $168 = HEAP32[$aKey>>2]|0;
   $169 = ((($168)) + 4|0);
   $170 = ((($169)) + 1|0);
   $171 = HEAP8[$170>>0]|0;
   $172 = $171&255;
   $173 = $172 << 16;
   $174 = $167 | $173;
   $175 = HEAP32[$aKey>>2]|0;
   $176 = ((($175)) + 4|0);
   $177 = ((($176)) + 2|0);
   $178 = HEAP8[$177>>0]|0;
   $179 = $178&255;
   $180 = $179 << 8;
   $181 = $174 | $180;
   $182 = HEAP32[$aKey>>2]|0;
   $183 = ((($182)) + 4|0);
   $184 = ((($183)) + 3|0);
   $185 = HEAP8[$184>>0]|0;
   $186 = $185&255;
   $187 = $181 | $186;
   $188 = $x;
   $189 = $188;
   HEAP32[$189>>2] = $187;
   $190 = (($188) + 4)|0;
   $191 = $190;
   HEAP32[$191>>2] = $159;
   $192 = $x;
   $193 = $192;
   $194 = HEAP32[$193>>2]|0;
   $195 = (($192) + 4)|0;
   $196 = $195;
   $197 = HEAP32[$196>>2]|0;
   $198 = $lhs;
   $199 = $198;
   HEAP32[$199>>2] = $194;
   $200 = (($198) + 4)|0;
   $201 = $200;
   HEAP32[$201>>2] = $197;
   break;
  }
  case 8:  {
   $202 = $lhs;
   $203 = $202;
   HEAP32[$203>>2] = 0;
   $204 = (($202) + 4)|0;
   $205 = $204;
   HEAP32[$205>>2] = 0;
   break;
  }
  case 9:  {
   $206 = $lhs;
   $207 = $206;
   HEAP32[$207>>2] = 1;
   $208 = (($206) + 4)|0;
   $209 = $208;
   HEAP32[$209>>2] = 0;
   break;
  }
  case 7: case 0:  {
   $210 = HEAP32[$1>>2]|0;
   $211 = HEAP32[$2>>2]|0;
   $212 = HEAP32[$3>>2]|0;
   $213 = (_sqlite3VdbeRecordCompare($210,$211,$212)|0);
   HEAP32[$0>>2] = $213;
   $288 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($288|0);
   break;
  }
  default: {
   $214 = HEAP32[$1>>2]|0;
   $215 = HEAP32[$2>>2]|0;
   $216 = HEAP32[$3>>2]|0;
   $217 = (_sqlite3VdbeRecordCompare($214,$215,$216)|0);
   HEAP32[$0>>2] = $217;
   $288 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($288|0);
  }
  }
 } while(0);
 $218 = HEAP32[$3>>2]|0;
 $219 = ((($218)) + 4|0);
 $220 = HEAP32[$219>>2]|0;
 $221 = $220;
 $222 = $221;
 $223 = HEAP32[$222>>2]|0;
 $224 = (($221) + 4)|0;
 $225 = $224;
 $226 = HEAP32[$225>>2]|0;
 $227 = $v;
 $228 = $227;
 HEAP32[$228>>2] = $223;
 $229 = (($227) + 4)|0;
 $230 = $229;
 HEAP32[$230>>2] = $226;
 $231 = $v;
 $232 = $231;
 $233 = HEAP32[$232>>2]|0;
 $234 = (($231) + 4)|0;
 $235 = $234;
 $236 = HEAP32[$235>>2]|0;
 $237 = $lhs;
 $238 = $237;
 $239 = HEAP32[$238>>2]|0;
 $240 = (($237) + 4)|0;
 $241 = $240;
 $242 = HEAP32[$241>>2]|0;
 $243 = ($236|0)>($242|0);
 $244 = ($233>>>0)>($239>>>0);
 $245 = ($236|0)==($242|0);
 $246 = $245 & $244;
 $247 = $243 | $246;
 do {
  if ($247) {
   $248 = HEAP32[$3>>2]|0;
   $249 = ((($248)) + 12|0);
   $250 = HEAP8[$249>>0]|0;
   $251 = $250 << 24 >> 24;
   HEAP32[$res>>2] = $251;
  } else {
   $252 = $v;
   $253 = $252;
   $254 = HEAP32[$253>>2]|0;
   $255 = (($252) + 4)|0;
   $256 = $255;
   $257 = HEAP32[$256>>2]|0;
   $258 = $lhs;
   $259 = $258;
   $260 = HEAP32[$259>>2]|0;
   $261 = (($258) + 4)|0;
   $262 = $261;
   $263 = HEAP32[$262>>2]|0;
   $264 = ($257|0)<($263|0);
   $265 = ($254>>>0)<($260>>>0);
   $266 = ($257|0)==($263|0);
   $267 = $266 & $265;
   $268 = $264 | $267;
   $269 = HEAP32[$3>>2]|0;
   if ($268) {
    $270 = ((($269)) + 13|0);
    $271 = HEAP8[$270>>0]|0;
    $272 = $271 << 24 >> 24;
    HEAP32[$res>>2] = $272;
    break;
   }
   $273 = ((($269)) + 8|0);
   $274 = HEAP16[$273>>1]|0;
   $275 = $274&65535;
   $276 = ($275|0)>(1);
   if ($276) {
    $277 = HEAP32[$1>>2]|0;
    $278 = HEAP32[$2>>2]|0;
    $279 = HEAP32[$3>>2]|0;
    $280 = (_sqlite3VdbeRecordCompareWithSkip($277,$278,$279,1)|0);
    HEAP32[$res>>2] = $280;
    break;
   } else {
    $281 = HEAP32[$3>>2]|0;
    $282 = ((($281)) + 10|0);
    $283 = HEAP8[$282>>0]|0;
    $284 = $283 << 24 >> 24;
    HEAP32[$res>>2] = $284;
    $285 = HEAP32[$3>>2]|0;
    $286 = ((($285)) + 14|0);
    HEAP8[$286>>0] = 1;
    break;
   }
  }
 } while(0);
 $287 = HEAP32[$res>>2]|0;
 HEAP32[$0>>2] = $287;
 $288 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($288|0);
}
function _vdbeRecordCompareString($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0;
 var $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey1 = 0, $nCmp = 0, $nStr = 0, $res = 0, $serial_type = 0, $szHdr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $aKey1 = sp + 20|0;
 $serial_type = sp + 16|0;
 $res = sp + 12|0;
 $nCmp = sp + 8|0;
 $nStr = sp + 4|0;
 $szHdr = sp;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 $4 = HEAP32[$2>>2]|0;
 HEAP32[$aKey1>>2] = $4;
 $5 = HEAP32[$aKey1>>2]|0;
 $6 = ((($5)) + 1|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)<(128);
 $10 = HEAP32[$aKey1>>2]|0;
 $11 = ((($10)) + 1|0);
 if ($9) {
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  HEAP32[$serial_type>>2] = $13;
 } else {
  (_sqlite3GetVarint32($11,$serial_type)|0);
 }
 $14 = HEAP32[$serial_type>>2]|0;
 $15 = ($14|0)<(12);
 do {
  if ($15) {
   $16 = HEAP32[$3>>2]|0;
   $17 = ((($16)) + 12|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   HEAP32[$res>>2] = $19;
  } else {
   $20 = HEAP32[$serial_type>>2]|0;
   $21 = $20 & 1;
   $22 = ($21|0)!=(0);
   if (!($22)) {
    $23 = HEAP32[$3>>2]|0;
    $24 = ((($23)) + 13|0);
    $25 = HEAP8[$24>>0]|0;
    $26 = $25 << 24 >> 24;
    HEAP32[$res>>2] = $26;
    break;
   }
   $27 = HEAP32[$aKey1>>2]|0;
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   HEAP32[$szHdr>>2] = $29;
   $30 = HEAP32[$serial_type>>2]|0;
   $31 = (($30) - 12)|0;
   $32 = (($31|0) / 2)&-1;
   HEAP32[$nStr>>2] = $32;
   $33 = HEAP32[$szHdr>>2]|0;
   $34 = HEAP32[$nStr>>2]|0;
   $35 = (($33) + ($34))|0;
   $36 = HEAP32[$1>>2]|0;
   $37 = ($35|0)>($36|0);
   if ($37) {
    $38 = (_sqlite3CorruptError(74742)|0);
    $39 = $38&255;
    $40 = HEAP32[$3>>2]|0;
    $41 = ((($40)) + 11|0);
    HEAP8[$41>>0] = $39;
    HEAP32[$0>>2] = 0;
    $111 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($111|0);
   }
   $42 = HEAP32[$3>>2]|0;
   $43 = ((($42)) + 4|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ((($44)) + 12|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = HEAP32[$nStr>>2]|0;
   $48 = ($46|0)<($47|0);
   if ($48) {
    $49 = HEAP32[$3>>2]|0;
    $50 = ((($49)) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = ((($51)) + 12|0);
    $53 = HEAP32[$52>>2]|0;
    $55 = $53;
   } else {
    $54 = HEAP32[$nStr>>2]|0;
    $55 = $54;
   }
   HEAP32[$nCmp>>2] = $55;
   $56 = HEAP32[$szHdr>>2]|0;
   $57 = HEAP32[$aKey1>>2]|0;
   $58 = (($57) + ($56)|0);
   $59 = HEAP32[$3>>2]|0;
   $60 = ((($59)) + 4|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ((($61)) + 16|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$nCmp>>2]|0;
   $65 = (_memcmp($58,$63,$64)|0);
   HEAP32[$res>>2] = $65;
   $66 = HEAP32[$res>>2]|0;
   $67 = ($66|0)==(0);
   if (!($67)) {
    $101 = HEAP32[$res>>2]|0;
    $102 = ($101|0)>(0);
    $103 = HEAP32[$3>>2]|0;
    if ($102) {
     $104 = ((($103)) + 13|0);
     $105 = HEAP8[$104>>0]|0;
     $106 = $105 << 24 >> 24;
     HEAP32[$res>>2] = $106;
     break;
    } else {
     $107 = ((($103)) + 12|0);
     $108 = HEAP8[$107>>0]|0;
     $109 = $108 << 24 >> 24;
     HEAP32[$res>>2] = $109;
     break;
    }
   }
   $68 = HEAP32[$nStr>>2]|0;
   $69 = HEAP32[$3>>2]|0;
   $70 = ((($69)) + 4|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ((($71)) + 12|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = (($68) - ($73))|0;
   HEAP32[$res>>2] = $74;
   $75 = HEAP32[$res>>2]|0;
   $76 = ($75|0)==(0);
   if ($76) {
    $77 = HEAP32[$3>>2]|0;
    $78 = ((($77)) + 8|0);
    $79 = HEAP16[$78>>1]|0;
    $80 = $79&65535;
    $81 = ($80|0)>(1);
    if ($81) {
     $82 = HEAP32[$1>>2]|0;
     $83 = HEAP32[$2>>2]|0;
     $84 = HEAP32[$3>>2]|0;
     $85 = (_sqlite3VdbeRecordCompareWithSkip($82,$83,$84,1)|0);
     HEAP32[$res>>2] = $85;
     break;
    } else {
     $86 = HEAP32[$3>>2]|0;
     $87 = ((($86)) + 10|0);
     $88 = HEAP8[$87>>0]|0;
     $89 = $88 << 24 >> 24;
     HEAP32[$res>>2] = $89;
     $90 = HEAP32[$3>>2]|0;
     $91 = ((($90)) + 14|0);
     HEAP8[$91>>0] = 1;
     break;
    }
   } else {
    $92 = HEAP32[$res>>2]|0;
    $93 = ($92|0)>(0);
    $94 = HEAP32[$3>>2]|0;
    if ($93) {
     $95 = ((($94)) + 13|0);
     $96 = HEAP8[$95>>0]|0;
     $97 = $96 << 24 >> 24;
     HEAP32[$res>>2] = $97;
     break;
    } else {
     $98 = ((($94)) + 12|0);
     $99 = HEAP8[$98>>0]|0;
     $100 = $99 << 24 >> 24;
     HEAP32[$res>>2] = $100;
     break;
    }
   }
  }
 } while(0);
 $110 = HEAP32[$res>>2]|0;
 HEAP32[$0>>2] = $110;
 $111 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($111|0);
}
function _sqlite3VdbeRecordCompare($nKey1,$pKey1,$pPKey2) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $nKey1;
 HEAP32[$1>>2] = $pKey1;
 HEAP32[$2>>2] = $pPKey2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3VdbeRecordCompareWithSkip($3,$4,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3VdbeRecordCompareWithSkip($nKey1,$pKey1,$pPKey2,$bSkip) {
 $nKey1 = $nKey1|0;
 $pKey1 = $pKey1|0;
 $pPKey2 = $pPKey2|0;
 $bSkip = $bSkip|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0.0, $164 = 0, $165 = 0.0, $166 = 0, $167 = 0.0, $168 = 0, $169 = 0.0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0.0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0.0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aKey1 = 0, $d1 = 0, $i = 0, $idx1 = 0, $lhs = 0, $mem1 = 0, $nCmp = 0, $nCmp1 = 0;
 var $nStr = 0, $pKeyInfo = 0, $pRhs = 0, $rc = 0, $rhs = 0, $s1 = 0, $serial_type = 0, $szHdr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 108|0;
 $d1 = sp + 104|0;
 $i = sp + 100|0;
 $szHdr1 = sp + 96|0;
 $idx1 = sp + 92|0;
 $rc = sp + 88|0;
 $pRhs = sp + 84|0;
 $pKeyInfo = sp + 80|0;
 $aKey1 = sp + 76|0;
 $mem1 = sp + 16|0;
 $s1 = sp + 72|0;
 $serial_type = sp + 68|0;
 $lhs = sp + 8|0;
 $rhs = sp;
 $nCmp = sp + 64|0;
 $nStr = sp + 60|0;
 $nCmp1 = sp + 56|0;
 HEAP32[$1>>2] = $nKey1;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $pPKey2;
 HEAP32[$4>>2] = $bSkip;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pRhs>>2] = $7;
 $8 = HEAP32[$3>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pKeyInfo>>2] = $9;
 $10 = HEAP32[$2>>2]|0;
 HEAP32[$aKey1>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = ($11|0)!=(0);
 $13 = HEAP32[$aKey1>>2]|0;
 do {
  if ($12) {
   $14 = ((($13)) + 1|0);
   $15 = HEAP8[$14>>0]|0;
   $16 = $15&255;
   $17 = ($16|0)<(128);
   $18 = HEAP32[$aKey1>>2]|0;
   $19 = ((($18)) + 1|0);
   if ($17) {
    $20 = HEAP8[$19>>0]|0;
    $21 = $20&255;
    HEAP32[$s1>>2] = $21;
    $25 = 1;
   } else {
    $22 = (_sqlite3GetVarint32($19,$s1)|0);
    $23 = $22&255;
    $25 = $23;
   }
   $24 = $25&255;
   $26 = $24&255;
   $27 = (1 + ($26))|0;
   HEAP32[$idx1>>2] = $27;
   $28 = HEAP32[$aKey1>>2]|0;
   $29 = HEAP8[$28>>0]|0;
   $30 = $29&255;
   HEAP32[$szHdr1>>2] = $30;
   $31 = HEAP32[$szHdr1>>2]|0;
   $32 = HEAP32[$s1>>2]|0;
   $33 = (_sqlite3VdbeSerialTypeLen($32)|0);
   $34 = (($31) + ($33))|0;
   HEAP32[$d1>>2] = $34;
   HEAP32[$i>>2] = 1;
   $35 = HEAP32[$pRhs>>2]|0;
   $36 = ((($35)) + 40|0);
   HEAP32[$pRhs>>2] = $36;
  } else {
   $37 = HEAP8[$13>>0]|0;
   $38 = $37&255;
   $39 = ($38|0)<(128);
   $40 = HEAP32[$aKey1>>2]|0;
   if ($39) {
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    HEAP32[$szHdr1>>2] = $42;
    $46 = 1;
   } else {
    $43 = (_sqlite3GetVarint32($40,$szHdr1)|0);
    $44 = $43&255;
    $46 = $44;
   }
   $45 = $46&255;
   $47 = $45&255;
   HEAP32[$idx1>>2] = $47;
   $48 = HEAP32[$szHdr1>>2]|0;
   HEAP32[$d1>>2] = $48;
   $49 = HEAP32[$d1>>2]|0;
   $50 = HEAP32[$1>>2]|0;
   $51 = ($49>>>0)>($50>>>0);
   if (!($51)) {
    HEAP32[$i>>2] = 0;
    break;
   }
   $52 = (_sqlite3CorruptError(74452)|0);
   $53 = $52&255;
   $54 = HEAP32[$3>>2]|0;
   $55 = ((($54)) + 11|0);
   HEAP8[$55>>0] = $53;
   HEAP32[$0>>2] = 0;
   $391 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($391|0);
  }
 } while(0);
 L16: while(1) {
  $56 = HEAP32[$pRhs>>2]|0;
  $57 = ((($56)) + 8|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58&65535;
  $60 = $59 & 4;
  $61 = ($60|0)!=(0);
  do {
   if ($61) {
    $62 = HEAP32[$idx1>>2]|0;
    $63 = HEAP32[$aKey1>>2]|0;
    $64 = (($63) + ($62)|0);
    $65 = HEAP8[$64>>0]|0;
    $66 = $65&255;
    HEAP32[$serial_type>>2] = $66;
    $67 = HEAP32[$serial_type>>2]|0;
    $68 = ($67>>>0)>=(10);
    if ($68) {
     HEAP32[$rc>>2] = 1;
     break;
    }
    $69 = HEAP32[$serial_type>>2]|0;
    $70 = ($69|0)==(0);
    if ($70) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $71 = HEAP32[$serial_type>>2]|0;
    $72 = ($71|0)==(7);
    if ($72) {
     $73 = HEAP32[$d1>>2]|0;
     $74 = HEAP32[$aKey1>>2]|0;
     $75 = (($74) + ($73)|0);
     $76 = HEAP32[$serial_type>>2]|0;
     (_sqlite3VdbeSerialGet($75,$76,$mem1)|0);
     $77 = HEAP32[$pRhs>>2]|0;
     $78 = $77;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $81 = (($78) + 4)|0;
     $82 = $81;
     $83 = HEAP32[$82>>2]|0;
     $84 = +HEAPF64[$mem1>>3];
     $85 = (_sqlite3IntFloatCompare($80,$83,$84)|0);
     $86 = (0 - ($85))|0;
     HEAP32[$rc>>2] = $86;
     break;
    }
    $87 = HEAP32[$serial_type>>2]|0;
    $88 = HEAP32[$d1>>2]|0;
    $89 = HEAP32[$aKey1>>2]|0;
    $90 = (($89) + ($88)|0);
    $91 = (_vdbeRecordDecodeInt($87,$90)|0);
    $92 = tempRet0;
    $93 = $lhs;
    $94 = $93;
    HEAP32[$94>>2] = $91;
    $95 = (($93) + 4)|0;
    $96 = $95;
    HEAP32[$96>>2] = $92;
    $97 = HEAP32[$pRhs>>2]|0;
    $98 = $97;
    $99 = $98;
    $100 = HEAP32[$99>>2]|0;
    $101 = (($98) + 4)|0;
    $102 = $101;
    $103 = HEAP32[$102>>2]|0;
    $104 = $rhs;
    $105 = $104;
    HEAP32[$105>>2] = $100;
    $106 = (($104) + 4)|0;
    $107 = $106;
    HEAP32[$107>>2] = $103;
    $108 = $lhs;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (($108) + 4)|0;
    $112 = $111;
    $113 = HEAP32[$112>>2]|0;
    $114 = $rhs;
    $115 = $114;
    $116 = HEAP32[$115>>2]|0;
    $117 = (($114) + 4)|0;
    $118 = $117;
    $119 = HEAP32[$118>>2]|0;
    $120 = ($113|0)<($119|0);
    $121 = ($110>>>0)<($116>>>0);
    $122 = ($113|0)==($119|0);
    $123 = $122 & $121;
    $124 = $120 | $123;
    if ($124) {
     HEAP32[$rc>>2] = -1;
     break;
    }
    $125 = $lhs;
    $126 = $125;
    $127 = HEAP32[$126>>2]|0;
    $128 = (($125) + 4)|0;
    $129 = $128;
    $130 = HEAP32[$129>>2]|0;
    $131 = $rhs;
    $132 = $131;
    $133 = HEAP32[$132>>2]|0;
    $134 = (($131) + 4)|0;
    $135 = $134;
    $136 = HEAP32[$135>>2]|0;
    $137 = ($130|0)>($136|0);
    $138 = ($127>>>0)>($133>>>0);
    $139 = ($130|0)==($136|0);
    $140 = $139 & $138;
    $141 = $137 | $140;
    if ($141) {
     HEAP32[$rc>>2] = 1;
    }
   } else {
    $142 = HEAP32[$pRhs>>2]|0;
    $143 = ((($142)) + 8|0);
    $144 = HEAP16[$143>>1]|0;
    $145 = $144&65535;
    $146 = $145 & 8;
    $147 = ($146|0)!=(0);
    if ($147) {
     $148 = HEAP32[$idx1>>2]|0;
     $149 = HEAP32[$aKey1>>2]|0;
     $150 = (($149) + ($148)|0);
     $151 = HEAP8[$150>>0]|0;
     $152 = $151&255;
     HEAP32[$serial_type>>2] = $152;
     $153 = HEAP32[$serial_type>>2]|0;
     $154 = ($153>>>0)>=(10);
     if ($154) {
      HEAP32[$rc>>2] = 1;
      break;
     }
     $155 = HEAP32[$serial_type>>2]|0;
     $156 = ($155|0)==(0);
     if ($156) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $157 = HEAP32[$d1>>2]|0;
     $158 = HEAP32[$aKey1>>2]|0;
     $159 = (($158) + ($157)|0);
     $160 = HEAP32[$serial_type>>2]|0;
     (_sqlite3VdbeSerialGet($159,$160,$mem1)|0);
     $161 = HEAP32[$serial_type>>2]|0;
     $162 = ($161|0)==(7);
     if (!($162)) {
      $171 = $mem1;
      $172 = $171;
      $173 = HEAP32[$172>>2]|0;
      $174 = (($171) + 4)|0;
      $175 = $174;
      $176 = HEAP32[$175>>2]|0;
      $177 = HEAP32[$pRhs>>2]|0;
      $178 = +HEAPF64[$177>>3];
      $179 = (_sqlite3IntFloatCompare($173,$176,$178)|0);
      HEAP32[$rc>>2] = $179;
      break;
     }
     $163 = +HEAPF64[$mem1>>3];
     $164 = HEAP32[$pRhs>>2]|0;
     $165 = +HEAPF64[$164>>3];
     $166 = $163 < $165;
     if ($166) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $167 = +HEAPF64[$mem1>>3];
     $168 = HEAP32[$pRhs>>2]|0;
     $169 = +HEAPF64[$168>>3];
     $170 = $167 > $169;
     if (!($170)) {
      break;
     }
     HEAP32[$rc>>2] = 1;
     break;
    }
    $180 = HEAP32[$pRhs>>2]|0;
    $181 = ((($180)) + 8|0);
    $182 = HEAP16[$181>>1]|0;
    $183 = $182&65535;
    $184 = $183 & 2;
    $185 = ($184|0)!=(0);
    if ($185) {
     $186 = HEAP32[$idx1>>2]|0;
     $187 = HEAP32[$aKey1>>2]|0;
     $188 = (($187) + ($186)|0);
     $189 = HEAP8[$188>>0]|0;
     $190 = $189&255;
     $191 = ($190|0)<(128);
     $192 = HEAP32[$idx1>>2]|0;
     $193 = HEAP32[$aKey1>>2]|0;
     $194 = (($193) + ($192)|0);
     if ($191) {
      $195 = HEAP8[$194>>0]|0;
      $196 = $195&255;
      HEAP32[$serial_type>>2] = $196;
     } else {
      (_sqlite3GetVarint32($194,$serial_type)|0);
     }
     $197 = HEAP32[$serial_type>>2]|0;
     $198 = ($197>>>0)<(12);
     if ($198) {
      HEAP32[$rc>>2] = -1;
      break;
     }
     $199 = HEAP32[$serial_type>>2]|0;
     $200 = $199 & 1;
     $201 = ($200|0)!=(0);
     if (!($201)) {
      HEAP32[$rc>>2] = 1;
      break;
     }
     $202 = HEAP32[$serial_type>>2]|0;
     $203 = (($202) - 12)|0;
     $204 = (($203>>>0) / 2)&-1;
     $205 = ((($mem1)) + 12|0);
     HEAP32[$205>>2] = $204;
     $206 = HEAP32[$d1>>2]|0;
     $207 = ((($mem1)) + 12|0);
     $208 = HEAP32[$207>>2]|0;
     $209 = (($206) + ($208))|0;
     $210 = HEAP32[$1>>2]|0;
     $211 = ($209>>>0)>($210>>>0);
     if ($211) {
      label = 43;
      break L16;
     }
     $216 = HEAP32[$i>>2]|0;
     $217 = HEAP32[$pKeyInfo>>2]|0;
     $218 = ((($217)) + 20|0);
     $219 = (($218) + ($216<<2)|0);
     $220 = HEAP32[$219>>2]|0;
     $221 = ($220|0)!=(0|0);
     if ($221) {
      $222 = HEAP32[$pKeyInfo>>2]|0;
      $223 = ((($222)) + 4|0);
      $224 = HEAP8[$223>>0]|0;
      $225 = ((($mem1)) + 10|0);
      HEAP8[$225>>0] = $224;
      $226 = HEAP32[$pKeyInfo>>2]|0;
      $227 = ((($226)) + 12|0);
      $228 = HEAP32[$227>>2]|0;
      $229 = ((($mem1)) + 32|0);
      HEAP32[$229>>2] = $228;
      $230 = ((($mem1)) + 8|0);
      HEAP16[$230>>1] = 2;
      $231 = HEAP32[$d1>>2]|0;
      $232 = HEAP32[$aKey1>>2]|0;
      $233 = (($232) + ($231)|0);
      $234 = ((($mem1)) + 16|0);
      HEAP32[$234>>2] = $233;
      $235 = HEAP32[$pRhs>>2]|0;
      $236 = HEAP32[$i>>2]|0;
      $237 = HEAP32[$pKeyInfo>>2]|0;
      $238 = ((($237)) + 20|0);
      $239 = (($238) + ($236<<2)|0);
      $240 = HEAP32[$239>>2]|0;
      $241 = HEAP32[$3>>2]|0;
      $242 = ((($241)) + 11|0);
      $243 = (_vdbeCompareMemString($mem1,$235,$240,$242)|0);
      HEAP32[$rc>>2] = $243;
      break;
     }
     $244 = ((($mem1)) + 12|0);
     $245 = HEAP32[$244>>2]|0;
     $246 = HEAP32[$pRhs>>2]|0;
     $247 = ((($246)) + 12|0);
     $248 = HEAP32[$247>>2]|0;
     $249 = ($245|0)<($248|0);
     if ($249) {
      $250 = ((($mem1)) + 12|0);
      $251 = HEAP32[$250>>2]|0;
      $255 = $251;
     } else {
      $252 = HEAP32[$pRhs>>2]|0;
      $253 = ((($252)) + 12|0);
      $254 = HEAP32[$253>>2]|0;
      $255 = $254;
     }
     HEAP32[$nCmp>>2] = $255;
     $256 = HEAP32[$d1>>2]|0;
     $257 = HEAP32[$aKey1>>2]|0;
     $258 = (($257) + ($256)|0);
     $259 = HEAP32[$pRhs>>2]|0;
     $260 = ((($259)) + 16|0);
     $261 = HEAP32[$260>>2]|0;
     $262 = HEAP32[$nCmp>>2]|0;
     $263 = (_memcmp($258,$261,$262)|0);
     HEAP32[$rc>>2] = $263;
     $264 = HEAP32[$rc>>2]|0;
     $265 = ($264|0)==(0);
     if (!($265)) {
      break;
     }
     $266 = ((($mem1)) + 12|0);
     $267 = HEAP32[$266>>2]|0;
     $268 = HEAP32[$pRhs>>2]|0;
     $269 = ((($268)) + 12|0);
     $270 = HEAP32[$269>>2]|0;
     $271 = (($267) - ($270))|0;
     HEAP32[$rc>>2] = $271;
     break;
    }
    $272 = HEAP32[$pRhs>>2]|0;
    $273 = ((($272)) + 8|0);
    $274 = HEAP16[$273>>1]|0;
    $275 = $274&65535;
    $276 = $275 & 16;
    $277 = ($276|0)!=(0);
    $278 = HEAP32[$idx1>>2]|0;
    $279 = HEAP32[$aKey1>>2]|0;
    $280 = (($279) + ($278)|0);
    $281 = HEAP8[$280>>0]|0;
    $282 = $281&255;
    if (!($277)) {
     HEAP32[$serial_type>>2] = $282;
     $347 = HEAP32[$serial_type>>2]|0;
     $348 = ($347|0)!=(0);
     $349 = $348&1;
     HEAP32[$rc>>2] = $349;
     break;
    }
    $283 = ($282|0)<(128);
    $284 = HEAP32[$idx1>>2]|0;
    $285 = HEAP32[$aKey1>>2]|0;
    $286 = (($285) + ($284)|0);
    if ($283) {
     $287 = HEAP8[$286>>0]|0;
     $288 = $287&255;
     HEAP32[$serial_type>>2] = $288;
    } else {
     (_sqlite3GetVarint32($286,$serial_type)|0);
    }
    $289 = HEAP32[$serial_type>>2]|0;
    $290 = ($289>>>0)<(12);
    if (!($290)) {
     $291 = HEAP32[$serial_type>>2]|0;
     $292 = $291 & 1;
     $293 = ($292|0)!=(0);
     if (!($293)) {
      $294 = HEAP32[$serial_type>>2]|0;
      $295 = (($294) - 12)|0;
      $296 = (($295>>>0) / 2)&-1;
      HEAP32[$nStr>>2] = $296;
      $297 = HEAP32[$d1>>2]|0;
      $298 = HEAP32[$nStr>>2]|0;
      $299 = (($297) + ($298))|0;
      $300 = HEAP32[$1>>2]|0;
      $301 = ($299>>>0)>($300>>>0);
      if ($301) {
       label = 59;
       break L16;
      }
      $306 = HEAP32[$pRhs>>2]|0;
      $307 = ((($306)) + 8|0);
      $308 = HEAP16[$307>>1]|0;
      $309 = $308&65535;
      $310 = $309 & 16384;
      $311 = ($310|0)!=(0);
      if ($311) {
       $312 = HEAP32[$d1>>2]|0;
       $313 = HEAP32[$aKey1>>2]|0;
       $314 = (($313) + ($312)|0);
       $315 = HEAP32[$nStr>>2]|0;
       $316 = (_isAllZero($314,$315)|0);
       $317 = ($316|0)!=(0);
       if ($317) {
        $318 = HEAP32[$nStr>>2]|0;
        $319 = HEAP32[$pRhs>>2]|0;
        $320 = HEAP32[$319>>2]|0;
        $321 = (($318) - ($320))|0;
        HEAP32[$rc>>2] = $321;
        break;
       } else {
        HEAP32[$rc>>2] = 1;
        break;
       }
      }
      $322 = HEAP32[$nStr>>2]|0;
      $323 = HEAP32[$pRhs>>2]|0;
      $324 = ((($323)) + 12|0);
      $325 = HEAP32[$324>>2]|0;
      $326 = ($322|0)<($325|0);
      if ($326) {
       $327 = HEAP32[$nStr>>2]|0;
       $331 = $327;
      } else {
       $328 = HEAP32[$pRhs>>2]|0;
       $329 = ((($328)) + 12|0);
       $330 = HEAP32[$329>>2]|0;
       $331 = $330;
      }
      HEAP32[$nCmp1>>2] = $331;
      $332 = HEAP32[$d1>>2]|0;
      $333 = HEAP32[$aKey1>>2]|0;
      $334 = (($333) + ($332)|0);
      $335 = HEAP32[$pRhs>>2]|0;
      $336 = ((($335)) + 16|0);
      $337 = HEAP32[$336>>2]|0;
      $338 = HEAP32[$nCmp1>>2]|0;
      $339 = (_memcmp($334,$337,$338)|0);
      HEAP32[$rc>>2] = $339;
      $340 = HEAP32[$rc>>2]|0;
      $341 = ($340|0)==(0);
      if (!($341)) {
       break;
      }
      $342 = HEAP32[$nStr>>2]|0;
      $343 = HEAP32[$pRhs>>2]|0;
      $344 = ((($343)) + 12|0);
      $345 = HEAP32[$344>>2]|0;
      $346 = (($342) - ($345))|0;
      HEAP32[$rc>>2] = $346;
      break;
     }
    }
    HEAP32[$rc>>2] = -1;
   }
  } while(0);
  $350 = HEAP32[$rc>>2]|0;
  $351 = ($350|0)!=(0);
  $352 = HEAP32[$i>>2]|0;
  if ($351) {
   label = 71;
   break;
  }
  $362 = (($352) + 1)|0;
  HEAP32[$i>>2] = $362;
  $363 = HEAP32[$pRhs>>2]|0;
  $364 = ((($363)) + 40|0);
  HEAP32[$pRhs>>2] = $364;
  $365 = HEAP32[$serial_type>>2]|0;
  $366 = (_sqlite3VdbeSerialTypeLen($365)|0);
  $367 = HEAP32[$d1>>2]|0;
  $368 = (($367) + ($366))|0;
  HEAP32[$d1>>2] = $368;
  $369 = HEAP32[$serial_type>>2]|0;
  $370 = (_sqlite3VarintLen($369,0)|0);
  $371 = HEAP32[$idx1>>2]|0;
  $372 = (($371) + ($370))|0;
  HEAP32[$idx1>>2] = $372;
  $373 = HEAP32[$idx1>>2]|0;
  $374 = HEAP32[$szHdr1>>2]|0;
  $375 = ($373>>>0)<($374>>>0);
  if (!($375)) {
   label = 77;
   break;
  }
  $376 = HEAP32[$i>>2]|0;
  $377 = HEAP32[$3>>2]|0;
  $378 = ((($377)) + 8|0);
  $379 = HEAP16[$378>>1]|0;
  $380 = $379&65535;
  $381 = ($376|0)<($380|0);
  if (!($381)) {
   label = 77;
   break;
  }
  $382 = HEAP32[$d1>>2]|0;
  $383 = HEAP32[$1>>2]|0;
  $384 = ($382>>>0)<=($383>>>0);
  if (!($384)) {
   label = 77;
   break;
  }
 }
 if ((label|0) == 43) {
  $212 = (_sqlite3CorruptError(74527)|0);
  $213 = $212&255;
  $214 = HEAP32[$3>>2]|0;
  $215 = ((($214)) + 11|0);
  HEAP8[$215>>0] = $213;
  HEAP32[$0>>2] = 0;
  $391 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($391|0);
 }
 else if ((label|0) == 59) {
  $302 = (_sqlite3CorruptError(74557)|0);
  $303 = $302&255;
  $304 = HEAP32[$3>>2]|0;
  $305 = ((($304)) + 11|0);
  HEAP8[$305>>0] = $303;
  HEAP32[$0>>2] = 0;
  $391 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($391|0);
 }
 else if ((label|0) == 71) {
  $353 = HEAP32[$pKeyInfo>>2]|0;
  $354 = ((($353)) + 16|0);
  $355 = HEAP32[$354>>2]|0;
  $356 = (($355) + ($352)|0);
  $357 = HEAP8[$356>>0]|0;
  $358 = ($357<<24>>24)!=(0);
  if ($358) {
   $359 = HEAP32[$rc>>2]|0;
   $360 = (0 - ($359))|0;
   HEAP32[$rc>>2] = $360;
  }
  $361 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $361;
  $391 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($391|0);
 }
 else if ((label|0) == 77) {
  $385 = HEAP32[$3>>2]|0;
  $386 = ((($385)) + 14|0);
  HEAP8[$386>>0] = 1;
  $387 = HEAP32[$3>>2]|0;
  $388 = ((($387)) + 10|0);
  $389 = HEAP8[$388>>0]|0;
  $390 = $389 << 24 >> 24;
  HEAP32[$0>>2] = $390;
  $391 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($391|0);
 }
 return (0)|0;
}
function _sqlite3GetVarint32($p,$v) {
 $p = $p|0;
 $v = $v|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $a = 0, $b = 0, $n = 0, $v64 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 25|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $a = sp + 12|0;
 $b = sp + 8|0;
 $v64 = sp;
 $n = sp + 24|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $v;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 HEAP32[$a>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 1|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 HEAP32[$b>>2] = $10;
 $11 = HEAP32[$b>>2]|0;
 $12 = $11 & 128;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  $14 = HEAP32[$a>>2]|0;
  $15 = $14 & 127;
  HEAP32[$a>>2] = $15;
  $16 = HEAP32[$a>>2]|0;
  $17 = $16 << 7;
  HEAP32[$a>>2] = $17;
  $18 = HEAP32[$a>>2]|0;
  $19 = HEAP32[$b>>2]|0;
  $20 = $18 | $19;
  $21 = HEAP32[$2>>2]|0;
  HEAP32[$21>>2] = $20;
  HEAP8[$0>>0] = 2;
  $72 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($72|0);
 }
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 1|0);
 HEAP32[$1>>2] = $23;
 $24 = HEAP32[$a>>2]|0;
 $25 = $24 << 14;
 HEAP32[$a>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 $29 = HEAP32[$a>>2]|0;
 $30 = $29 | $28;
 HEAP32[$a>>2] = $30;
 $31 = HEAP32[$a>>2]|0;
 $32 = $31 & 128;
 $33 = ($32|0)!=(0);
 if (!($33)) {
  $34 = HEAP32[$a>>2]|0;
  $35 = $34 & 2080895;
  HEAP32[$a>>2] = $35;
  $36 = HEAP32[$b>>2]|0;
  $37 = $36 & 127;
  HEAP32[$b>>2] = $37;
  $38 = HEAP32[$b>>2]|0;
  $39 = $38 << 7;
  HEAP32[$b>>2] = $39;
  $40 = HEAP32[$a>>2]|0;
  $41 = HEAP32[$b>>2]|0;
  $42 = $40 | $41;
  $43 = HEAP32[$2>>2]|0;
  HEAP32[$43>>2] = $42;
  HEAP8[$0>>0] = 3;
  $72 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($72|0);
 }
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + -2|0);
 HEAP32[$1>>2] = $45;
 $46 = HEAP32[$1>>2]|0;
 $47 = (_sqlite3GetVarint($46,$v64)|0);
 HEAP8[$n>>0] = $47;
 $48 = $v64;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = $v64;
 $55 = $54;
 $56 = HEAP32[$55>>2]|0;
 $57 = (($54) + 4)|0;
 $58 = $57;
 $59 = HEAP32[$58>>2]|0;
 $60 = ($50|0)!=($56|0);
 $61 = (0)!=($59|0);
 $62 = $60 | $61;
 if ($62) {
  $63 = HEAP32[$2>>2]|0;
  HEAP32[$63>>2] = -1;
 } else {
  $64 = $v64;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = (($64) + 4)|0;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  HEAP32[$70>>2] = $66;
 }
 $71 = HEAP8[$n>>0]|0;
 HEAP8[$0>>0] = $71;
 $72 = HEAP8[$0>>0]|0;
 STACKTOP = sp;return ($72|0);
}
function _sqlite3VdbeSerialTypeLen($serial_type) {
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $serial_type;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2>>>0)>=(128);
 $4 = HEAP32[$1>>2]|0;
 if ($3) {
  $5 = (($4) - 12)|0;
  $6 = (($5>>>0) / 2)&-1;
  HEAP32[$0>>2] = $6;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $7 = (35061 + ($4)|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  HEAP32[$0>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSerialGet($buf,$serial_type,$pMem) {
 $buf = $buf|0;
 $serial_type = $serial_type|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $buf;
 HEAP32[$2>>2] = $serial_type;
 HEAP32[$3>>2] = $pMem;
 $4 = HEAP32[$2>>2]|0;
 switch ($4|0) {
 case 0: case 11: case 10:  {
  $5 = HEAP32[$3>>2]|0;
  $6 = ((($5)) + 8|0);
  HEAP16[$6>>1] = 1;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 1:  {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  $9 = $8 << 24 >> 24;
  $10 = ($9|0)<(0);
  $11 = $10 << 31 >> 31;
  $12 = HEAP32[$3>>2]|0;
  $13 = $12;
  $14 = $13;
  HEAP32[$14>>2] = $9;
  $15 = (($13) + 4)|0;
  $16 = $15;
  HEAP32[$16>>2] = $11;
  $17 = HEAP32[$3>>2]|0;
  $18 = ((($17)) + 8|0);
  HEAP16[$18>>1] = 4;
  HEAP32[$0>>2] = 1;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 2:  {
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = $21<<8;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 1|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $22 | $26;
  $28 = ($27|0)<(0);
  $29 = $28 << 31 >> 31;
  $30 = HEAP32[$3>>2]|0;
  $31 = $30;
  $32 = $31;
  HEAP32[$32>>2] = $27;
  $33 = (($31) + 4)|0;
  $34 = $33;
  HEAP32[$34>>2] = $29;
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP16[$36>>1] = 4;
  HEAP32[$0>>2] = 2;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 3:  {
  $37 = HEAP32[$1>>2]|0;
  $38 = HEAP8[$37>>0]|0;
  $39 = $38 << 24 >> 24;
  $40 = $39<<16;
  $41 = HEAP32[$1>>2]|0;
  $42 = ((($41)) + 1|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = $44 << 8;
  $46 = $40 | $45;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 2|0);
  $49 = HEAP8[$48>>0]|0;
  $50 = $49&255;
  $51 = $46 | $50;
  $52 = ($51|0)<(0);
  $53 = $52 << 31 >> 31;
  $54 = HEAP32[$3>>2]|0;
  $55 = $54;
  $56 = $55;
  HEAP32[$56>>2] = $51;
  $57 = (($55) + 4)|0;
  $58 = $57;
  HEAP32[$58>>2] = $53;
  $59 = HEAP32[$3>>2]|0;
  $60 = ((($59)) + 8|0);
  HEAP16[$60>>1] = 4;
  HEAP32[$0>>2] = 3;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 4:  {
  $61 = HEAP32[$1>>2]|0;
  $62 = HEAP8[$61>>0]|0;
  $63 = $62 << 24 >> 24;
  $64 = $63<<24;
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 1|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = $68 << 16;
  $70 = $64 | $69;
  $71 = HEAP32[$1>>2]|0;
  $72 = ((($71)) + 2|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = $74 << 8;
  $76 = $70 | $75;
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 3|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $76 | $80;
  $82 = ($81|0)<(0);
  $83 = $82 << 31 >> 31;
  $84 = HEAP32[$3>>2]|0;
  $85 = $84;
  $86 = $85;
  HEAP32[$86>>2] = $81;
  $87 = (($85) + 4)|0;
  $88 = $87;
  HEAP32[$88>>2] = $83;
  $89 = HEAP32[$3>>2]|0;
  $90 = ((($89)) + 8|0);
  HEAP16[$90>>1] = 4;
  HEAP32[$0>>2] = 4;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 5:  {
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 2|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $93&255;
  $95 = $94 << 24;
  $96 = HEAP32[$1>>2]|0;
  $97 = ((($96)) + 2|0);
  $98 = ((($97)) + 1|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = $99&255;
  $101 = $100 << 16;
  $102 = $95 | $101;
  $103 = HEAP32[$1>>2]|0;
  $104 = ((($103)) + 2|0);
  $105 = ((($104)) + 2|0);
  $106 = HEAP8[$105>>0]|0;
  $107 = $106&255;
  $108 = $107 << 8;
  $109 = $102 | $108;
  $110 = HEAP32[$1>>2]|0;
  $111 = ((($110)) + 2|0);
  $112 = ((($111)) + 3|0);
  $113 = HEAP8[$112>>0]|0;
  $114 = $113&255;
  $115 = $109 | $114;
  $116 = HEAP32[$1>>2]|0;
  $117 = HEAP8[$116>>0]|0;
  $118 = $117 << 24 >> 24;
  $119 = $118<<8;
  $120 = HEAP32[$1>>2]|0;
  $121 = ((($120)) + 1|0);
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $119 | $123;
  $125 = ($124|0)<(0);
  $126 = $125 << 31 >> 31;
  $127 = (___muldi3(0,1,($124|0),($126|0))|0);
  $128 = tempRet0;
  $129 = (_i64Add(($115|0),0,($127|0),($128|0))|0);
  $130 = tempRet0;
  $131 = HEAP32[$3>>2]|0;
  $132 = $131;
  $133 = $132;
  HEAP32[$133>>2] = $129;
  $134 = (($132) + 4)|0;
  $135 = $134;
  HEAP32[$135>>2] = $130;
  $136 = HEAP32[$3>>2]|0;
  $137 = ((($136)) + 8|0);
  HEAP16[$137>>1] = 4;
  HEAP32[$0>>2] = 6;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 7: case 6:  {
  $138 = HEAP32[$1>>2]|0;
  $139 = HEAP32[$2>>2]|0;
  $140 = HEAP32[$3>>2]|0;
  $141 = (_serialGet($138,$139,$140)|0);
  HEAP32[$0>>2] = $141;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 case 9: case 8:  {
  $142 = HEAP32[$2>>2]|0;
  $143 = (($142) - 8)|0;
  $144 = HEAP32[$3>>2]|0;
  $145 = $144;
  $146 = $145;
  HEAP32[$146>>2] = $143;
  $147 = (($145) + 4)|0;
  $148 = $147;
  HEAP32[$148>>2] = 0;
  $149 = HEAP32[$3>>2]|0;
  $150 = ((($149)) + 8|0);
  HEAP16[$150>>1] = 4;
  HEAP32[$0>>2] = 0;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
  break;
 }
 default: {
  $151 = HEAP32[$1>>2]|0;
  $152 = HEAP32[$3>>2]|0;
  $153 = ((($152)) + 16|0);
  HEAP32[$153>>2] = $151;
  $154 = HEAP32[$2>>2]|0;
  $155 = (($154) - 12)|0;
  $156 = (($155>>>0) / 2)&-1;
  $157 = HEAP32[$3>>2]|0;
  $158 = ((($157)) + 12|0);
  HEAP32[$158>>2] = $156;
  $159 = HEAP32[$2>>2]|0;
  $160 = $159 & 1;
  $161 = (14590 + ($160<<1)|0);
  $162 = HEAP16[$161>>1]|0;
  $163 = HEAP32[$3>>2]|0;
  $164 = ((($163)) + 8|0);
  HEAP16[$164>>1] = $162;
  $165 = HEAP32[$3>>2]|0;
  $166 = ((($165)) + 12|0);
  $167 = HEAP32[$166>>2]|0;
  HEAP32[$0>>2] = $167;
  $168 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($168|0);
 }
 }
 return (0)|0;
}
function _sqlite3IntFloatCompare($0,$1,$r) {
 $0 = $0|0;
 $1 = $1|0;
 $r = +$r;
 var $10 = 0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0.0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0.0, $72 = 0.0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0, $8 = 0, $9 = 0.0, $or$cond = 0, $s = 0, $y = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 24|0;
 $4 = sp + 16|0;
 $y = sp + 8|0;
 $s = sp;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAPF64[$4>>3] = $r;
 $9 = +HEAPF64[$4>>3];
 $10 = $9 < -9.2233720368547758E+18;
 if ($10) {
  HEAP32[$2>>2] = 1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $11 = +HEAPF64[$4>>3];
 $12 = $11 > 9.2233720368547758E+18;
 if ($12) {
  HEAP32[$2>>2] = -1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $13 = +HEAPF64[$4>>3];
 $14 = (~~$13)>>>0;
 $15 = +Math_abs($13) >= 1.0 ? $13 > 0.0 ? (~~+Math_min(+Math_floor($13 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($13 - +(~~$13 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $16 = $y;
 $17 = $16;
 HEAP32[$17>>2] = $14;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $3;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = $y;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($25|0)<($31|0);
 $33 = ($22>>>0)<($28>>>0);
 $34 = ($25|0)==($31|0);
 $35 = $34 & $33;
 $36 = $32 | $35;
 if ($36) {
  HEAP32[$2>>2] = -1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $37 = $3;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = (($37) + 4)|0;
 $41 = $40;
 $42 = HEAP32[$41>>2]|0;
 $43 = $y;
 $44 = $43;
 $45 = HEAP32[$44>>2]|0;
 $46 = (($43) + 4)|0;
 $47 = $46;
 $48 = HEAP32[$47>>2]|0;
 $49 = ($42|0)>($48|0);
 $50 = ($39>>>0)>($45>>>0);
 $51 = ($42|0)==($48|0);
 $52 = $51 & $50;
 $53 = $49 | $52;
 if ($53) {
  $54 = $y;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($54) + 4)|0;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = ($56|0)==(0);
  $61 = ($59|0)==(-2147483648);
  $62 = $60 & $61;
  $63 = +HEAPF64[$4>>3];
  $64 = $63 > 0.0;
  $or$cond = $62 & $64;
  if ($or$cond) {
   HEAP32[$2>>2] = -1;
   $78 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($78|0);
  } else {
   HEAP32[$2>>2] = 1;
   $78 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($78|0);
  }
 }
 $65 = $3;
 $66 = $65;
 $67 = HEAP32[$66>>2]|0;
 $68 = (($65) + 4)|0;
 $69 = $68;
 $70 = HEAP32[$69>>2]|0;
 $71 = (+($67>>>0)) + (4294967296.0*(+($70|0)));
 HEAPF64[$s>>3] = $71;
 $72 = +HEAPF64[$s>>3];
 $73 = +HEAPF64[$4>>3];
 $74 = $72 < $73;
 if ($74) {
  HEAP32[$2>>2] = -1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $75 = +HEAPF64[$s>>3];
 $76 = +HEAPF64[$4>>3];
 $77 = $75 > $76;
 if ($77) {
  HEAP32[$2>>2] = 1;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 } else {
  HEAP32[$2>>2] = 0;
  $78 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 return (0)|0;
}
function _vdbeRecordDecodeInt($serial_type,$aKey) {
 $serial_type = $serial_type|0;
 $aKey = $aKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 8|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $y = sp + 16|0;
 $x = sp;
 HEAP32[$1>>2] = $serial_type;
 HEAP32[$2>>2] = $aKey;
 $3 = HEAP32[$1>>2]|0;
 switch ($3|0) {
 case 1: case 0:  {
  $4 = HEAP32[$2>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)<(0);
  $8 = $7 << 31 >> 31;
  $9 = $0;
  $10 = $9;
  HEAP32[$10>>2] = $6;
  $11 = (($9) + 4)|0;
  $12 = $11;
  HEAP32[$12>>2] = $8;
  break;
 }
 case 2:  {
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP8[$13>>0]|0;
  $15 = $14 << 24 >> 24;
  $16 = $15<<8;
  $17 = HEAP32[$2>>2]|0;
  $18 = ((($17)) + 1|0);
  $19 = HEAP8[$18>>0]|0;
  $20 = $19&255;
  $21 = $16 | $20;
  $22 = ($21|0)<(0);
  $23 = $22 << 31 >> 31;
  $24 = $0;
  $25 = $24;
  HEAP32[$25>>2] = $21;
  $26 = (($24) + 4)|0;
  $27 = $26;
  HEAP32[$27>>2] = $23;
  break;
 }
 case 3:  {
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP8[$28>>0]|0;
  $30 = $29 << 24 >> 24;
  $31 = $30<<16;
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 1|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = $35 << 8;
  $37 = $31 | $36;
  $38 = HEAP32[$2>>2]|0;
  $39 = ((($38)) + 2|0);
  $40 = HEAP8[$39>>0]|0;
  $41 = $40&255;
  $42 = $37 | $41;
  $43 = ($42|0)<(0);
  $44 = $43 << 31 >> 31;
  $45 = $0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  break;
 }
 case 4:  {
  $49 = HEAP32[$2>>2]|0;
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 << 24;
  $53 = HEAP32[$2>>2]|0;
  $54 = ((($53)) + 1|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = $55&255;
  $57 = $56 << 16;
  $58 = $52 | $57;
  $59 = HEAP32[$2>>2]|0;
  $60 = ((($59)) + 2|0);
  $61 = HEAP8[$60>>0]|0;
  $62 = $61&255;
  $63 = $62 << 8;
  $64 = $58 | $63;
  $65 = HEAP32[$2>>2]|0;
  $66 = ((($65)) + 3|0);
  $67 = HEAP8[$66>>0]|0;
  $68 = $67&255;
  $69 = $64 | $68;
  HEAP32[$y>>2] = $69;
  $70 = HEAP32[$y>>2]|0;
  $71 = ($70|0)<(0);
  $72 = $71 << 31 >> 31;
  $73 = $0;
  $74 = $73;
  HEAP32[$74>>2] = $70;
  $75 = (($73) + 4)|0;
  $76 = $75;
  HEAP32[$76>>2] = $72;
  break;
 }
 case 5:  {
  $77 = HEAP32[$2>>2]|0;
  $78 = ((($77)) + 2|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $80 << 24;
  $82 = HEAP32[$2>>2]|0;
  $83 = ((($82)) + 2|0);
  $84 = ((($83)) + 1|0);
  $85 = HEAP8[$84>>0]|0;
  $86 = $85&255;
  $87 = $86 << 16;
  $88 = $81 | $87;
  $89 = HEAP32[$2>>2]|0;
  $90 = ((($89)) + 2|0);
  $91 = ((($90)) + 2|0);
  $92 = HEAP8[$91>>0]|0;
  $93 = $92&255;
  $94 = $93 << 8;
  $95 = $88 | $94;
  $96 = HEAP32[$2>>2]|0;
  $97 = ((($96)) + 2|0);
  $98 = ((($97)) + 3|0);
  $99 = HEAP8[$98>>0]|0;
  $100 = $99&255;
  $101 = $95 | $100;
  $102 = HEAP32[$2>>2]|0;
  $103 = HEAP8[$102>>0]|0;
  $104 = $103 << 24 >> 24;
  $105 = $104<<8;
  $106 = HEAP32[$2>>2]|0;
  $107 = ((($106)) + 1|0);
  $108 = HEAP8[$107>>0]|0;
  $109 = $108&255;
  $110 = $105 | $109;
  $111 = ($110|0)<(0);
  $112 = $111 << 31 >> 31;
  $113 = (___muldi3(0,1,($110|0),($112|0))|0);
  $114 = tempRet0;
  $115 = (_i64Add(($101|0),0,($113|0),($114|0))|0);
  $116 = tempRet0;
  $117 = $0;
  $118 = $117;
  HEAP32[$118>>2] = $115;
  $119 = (($117) + 4)|0;
  $120 = $119;
  HEAP32[$120>>2] = $116;
  break;
 }
 case 6:  {
  $121 = HEAP32[$2>>2]|0;
  $122 = HEAP8[$121>>0]|0;
  $123 = $122&255;
  $124 = $123 << 24;
  $125 = HEAP32[$2>>2]|0;
  $126 = ((($125)) + 1|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = $127&255;
  $129 = $128 << 16;
  $130 = $124 | $129;
  $131 = HEAP32[$2>>2]|0;
  $132 = ((($131)) + 2|0);
  $133 = HEAP8[$132>>0]|0;
  $134 = $133&255;
  $135 = $134 << 8;
  $136 = $130 | $135;
  $137 = HEAP32[$2>>2]|0;
  $138 = ((($137)) + 3|0);
  $139 = HEAP8[$138>>0]|0;
  $140 = $139&255;
  $141 = $136 | $140;
  $142 = $x;
  $143 = $142;
  HEAP32[$143>>2] = $141;
  $144 = (($142) + 4)|0;
  $145 = $144;
  HEAP32[$145>>2] = 0;
  $146 = $x;
  $147 = $146;
  $148 = HEAP32[$147>>2]|0;
  $149 = (($146) + 4)|0;
  $150 = $149;
  $151 = HEAP32[$150>>2]|0;
  $152 = HEAP32[$2>>2]|0;
  $153 = ((($152)) + 4|0);
  $154 = HEAP8[$153>>0]|0;
  $155 = $154&255;
  $156 = $155 << 24;
  $157 = HEAP32[$2>>2]|0;
  $158 = ((($157)) + 4|0);
  $159 = ((($158)) + 1|0);
  $160 = HEAP8[$159>>0]|0;
  $161 = $160&255;
  $162 = $161 << 16;
  $163 = $156 | $162;
  $164 = HEAP32[$2>>2]|0;
  $165 = ((($164)) + 4|0);
  $166 = ((($165)) + 2|0);
  $167 = HEAP8[$166>>0]|0;
  $168 = $167&255;
  $169 = $168 << 8;
  $170 = $163 | $169;
  $171 = HEAP32[$2>>2]|0;
  $172 = ((($171)) + 4|0);
  $173 = ((($172)) + 3|0);
  $174 = HEAP8[$173>>0]|0;
  $175 = $174&255;
  $176 = $170 | $175;
  $177 = $x;
  $178 = $177;
  HEAP32[$178>>2] = $176;
  $179 = (($177) + 4)|0;
  $180 = $179;
  HEAP32[$180>>2] = $148;
  $181 = $x;
  $182 = $181;
  $183 = HEAP32[$182>>2]|0;
  $184 = (($181) + 4)|0;
  $185 = $184;
  $186 = HEAP32[$185>>2]|0;
  $187 = $0;
  $188 = $187;
  HEAP32[$188>>2] = $183;
  $189 = (($187) + 4)|0;
  $190 = $189;
  HEAP32[$190>>2] = $186;
  break;
 }
 default: {
  $191 = HEAP32[$1>>2]|0;
  $192 = (($191) - 8)|0;
  $193 = $0;
  $194 = $193;
  HEAP32[$194>>2] = $192;
  $195 = (($193) + 4)|0;
  $196 = $195;
  HEAP32[$196>>2] = 0;
 }
 }
 $197 = $0;
 $198 = $197;
 $199 = HEAP32[$198>>2]|0;
 $200 = (($197) + 4)|0;
 $201 = $200;
 $202 = HEAP32[$201>>2]|0;
 tempRet0 = ($202);
 STACKTOP = sp;return ($199|0);
}
function _vdbeCompareMemString($pMem1,$pMem2,$pColl,$prcErr) {
 $pMem1 = $pMem1|0;
 $pMem2 = $pMem2|0;
 $pColl = $pColl|0;
 $prcErr = $prcErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $c1 = 0;
 var $c2 = 0, $n1 = 0, $n2 = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $v1 = 0, $v2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $4 = sp + 100|0;
 $rc = sp + 96|0;
 $v1 = sp + 92|0;
 $v2 = sp + 88|0;
 $n1 = sp + 84|0;
 $n2 = sp + 80|0;
 $c1 = sp + 40|0;
 $c2 = sp;
 HEAP32[$1>>2] = $pMem1;
 HEAP32[$2>>2] = $pMem2;
 HEAP32[$3>>2] = $pColl;
 HEAP32[$4>>2] = $prcErr;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 10|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($8|0)==($12|0);
 if ($13) {
  $14 = HEAP32[$3>>2]|0;
  $15 = ((($14)) + 12|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (FUNCTION_TABLE_iiiiii[$16 & 255]($19,$22,$25,$28,$31)|0);
  HEAP32[$0>>2] = $32;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 32|0);
 $35 = HEAP32[$34>>2]|0;
 _sqlite3VdbeMemInit($c1,$35,1);
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 32|0);
 $38 = HEAP32[$37>>2]|0;
 _sqlite3VdbeMemInit($c2,$38,1);
 $39 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemShallowCopy($c1,$39,4096);
 $40 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemShallowCopy($c2,$40,4096);
 $41 = HEAP32[$3>>2]|0;
 $42 = ((($41)) + 4|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = (_sqlite3ValueText($c1,$43)|0);
 HEAP32[$v1>>2] = $44;
 $45 = HEAP32[$v1>>2]|0;
 $46 = ($45|0)==(0|0);
 $47 = ((($c1)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = $46 ? 0 : $48;
 HEAP32[$n1>>2] = $49;
 $50 = HEAP32[$3>>2]|0;
 $51 = ((($50)) + 4|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = (_sqlite3ValueText($c2,$52)|0);
 HEAP32[$v2>>2] = $53;
 $54 = HEAP32[$v2>>2]|0;
 $55 = ($54|0)==(0|0);
 $56 = ((($c2)) + 12|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = $55 ? 0 : $57;
 HEAP32[$n2>>2] = $58;
 $59 = HEAP32[$3>>2]|0;
 $60 = ((($59)) + 12|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$3>>2]|0;
 $63 = ((($62)) + 8|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = HEAP32[$n1>>2]|0;
 $66 = HEAP32[$v1>>2]|0;
 $67 = HEAP32[$n2>>2]|0;
 $68 = HEAP32[$v2>>2]|0;
 $69 = (FUNCTION_TABLE_iiiiii[$61 & 255]($64,$65,$66,$67,$68)|0);
 HEAP32[$rc>>2] = $69;
 $70 = HEAP32[$v1>>2]|0;
 $71 = ($70|0)==(0|0);
 $72 = HEAP32[$v2>>2]|0;
 $73 = ($72|0)==(0|0);
 $or$cond = $71 | $73;
 $74 = HEAP32[$4>>2]|0;
 $75 = ($74|0)!=(0|0);
 $or$cond3 = $or$cond & $75;
 if ($or$cond3) {
  $76 = HEAP32[$4>>2]|0;
  HEAP8[$76>>0] = 7;
 }
 _sqlite3VdbeMemRelease($c1);
 _sqlite3VdbeMemRelease($c2);
 $77 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _isAllZero($z,$n) {
 $z = $z|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$2>>2]|0;
  $5 = ($3|0)<($4|0);
  if (!($5)) {
   label = 6;
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) + ($6)|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = ($9<<24>>24)!=(0);
  if ($10) {
   label = 4;
   break;
  }
  $11 = HEAP32[$i>>2]|0;
  $12 = (($11) + 1)|0;
  HEAP32[$i>>2] = $12;
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _sqlite3VarintLen($0,$1) {
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp;
 $i = sp + 8|0;
 $3 = $2;
 $4 = $3;
 HEAP32[$4>>2] = $0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = $1;
 HEAP32[$i>>2] = 1;
 while(1) {
  $7 = $2;
  $8 = $7;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($7) + 4)|0;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (_bitshift64Lshr(($9|0),($12|0),7)|0);
  $14 = tempRet0;
  $15 = $2;
  $16 = $15;
  HEAP32[$16>>2] = $13;
  $17 = (($15) + 4)|0;
  $18 = $17;
  HEAP32[$18>>2] = $14;
  $19 = ($13|0)!=(0);
  $20 = ($14|0)!=(0);
  $21 = $19 | $20;
  $22 = HEAP32[$i>>2]|0;
  if (!($21)) {
   break;
  }
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 STACKTOP = sp;return ($22|0);
}
function _sqlite3VdbeMemInit($pMem,$db,$flags) {
 $pMem = $pMem|0;
 $db = $db|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $db;
 HEAP16[$2>>1] = $flags;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 HEAP16[$5>>1] = $3;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 32|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 24|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemShallowCopy($pTo,$pFrom,$srcType) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 $srcType = $srcType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$2>>2] = $srcType;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 9312;
 $8 = ($7|0)!=(0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  _vdbeClrCopy($9,$10,$11);
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$1>>2]|0;
 ;HEAP32[$9>>2]=HEAP32[$12>>2]|0;HEAP32[$9+4>>2]=HEAP32[$12+4>>2]|0;HEAP32[$9+8>>2]=HEAP32[$12+8>>2]|0;HEAP32[$9+12>>2]=HEAP32[$12+12>>2]|0;HEAP32[$9+16>>2]=HEAP32[$12+16>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $16 & 2048;
 $18 = ($17|0)==(0);
 if (!($18)) {
  STACKTOP = sp;return;
 }
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 8|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 $23 = $22 & -7169;
 $24 = $23&65535;
 HEAP16[$20>>1] = $24;
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 8|0);
 $28 = HEAP16[$27>>1]|0;
 $29 = $28&65535;
 $30 = $29 | $25;
 $31 = $30&65535;
 HEAP16[$27>>1] = $31;
 STACKTOP = sp;return;
}
function _vdbeClrCopy($pTo,$pFrom,$eType) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 $eType = $eType|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 HEAP32[$2>>2] = $eType;
 $3 = HEAP32[$0>>2]|0;
 _vdbeMemClearExternAndSetNull($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemShallowCopy($4,$5,$6);
 STACKTOP = sp;return;
}
function _vdbeMemClearExternAndSetNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFrame = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFrame = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 8192;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  (_sqlite3VdbeMemFinalize($7,$9)|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 & 1024;
 $15 = ($14|0)!=(0);
 $16 = HEAP32[$0>>2]|0;
 do {
  if ($15) {
   $17 = ((($16)) + 36|0);
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 16|0);
   $21 = HEAP32[$20>>2]|0;
   FUNCTION_TABLE_vi[$18 & 255]($21);
  } else {
   $22 = ((($16)) + 8|0);
   $23 = HEAP16[$22>>1]|0;
   $24 = $23&65535;
   $25 = $24 & 32;
   $26 = ($25|0)!=(0);
   $27 = HEAP32[$0>>2]|0;
   if ($26) {
    $28 = HEAP32[$27>>2]|0;
    _sqlite3RowSetClear($28);
    break;
   }
   $29 = ((($27)) + 8|0);
   $30 = HEAP16[$29>>1]|0;
   $31 = $30&65535;
   $32 = $31 & 64;
   $33 = ($32|0)!=(0);
   if ($33) {
    $34 = HEAP32[$0>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    HEAP32[$pFrame>>2] = $35;
    $36 = HEAP32[$pFrame>>2]|0;
    $37 = HEAP32[$36>>2]|0;
    $38 = ((($37)) + 188|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$pFrame>>2]|0;
    $41 = ((($40)) + 4|0);
    HEAP32[$41>>2] = $39;
    $42 = HEAP32[$pFrame>>2]|0;
    $43 = HEAP32[$pFrame>>2]|0;
    $44 = HEAP32[$43>>2]|0;
    $45 = ((($44)) + 188|0);
    HEAP32[$45>>2] = $42;
   }
  }
 } while(0);
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 8|0);
 HEAP16[$47>>1] = 1;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemFinalize($pMem,$pFunc) {
 $pMem = $pMem|0;
 $pFunc = $pFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ctx = 0, $rc = 0, $t = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $rc = sp + 72|0;
 $ctx = sp + 40|0;
 $t = sp;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$1>>2] = $pFunc;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 ;HEAP32[$ctx>>2]=0|0;HEAP32[$ctx+4>>2]=0|0;HEAP32[$ctx+8>>2]=0|0;HEAP32[$ctx+12>>2]=0|0;HEAP32[$ctx+16>>2]=0|0;HEAP32[$ctx+20>>2]=0|0;HEAP32[$ctx+24>>2]=0|0;HEAP32[$ctx+28>>2]=0|0;
 dest=$t; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $8 = ((($t)) + 8|0);
 HEAP16[$8>>1] = 1;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 32|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($t)) + 32|0);
 HEAP32[$12>>2] = $11;
 HEAP32[$ctx>>2] = $t;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($ctx)) + 8|0);
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($ctx)) + 4|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = HEAP32[$18>>2]|0;
 FUNCTION_TABLE_vi[$19 & 255]($ctx);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 24|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)>(0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 32|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 20|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3DbFree($26,$29);
 }
 $30 = HEAP32[$0>>2]|0;
 dest=$30; src=$t; stop=dest+40|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $31 = ((($ctx)) + 20|0);
 $32 = HEAP32[$31>>2]|0;
 HEAP32[$rc>>2] = $32;
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3RowSetClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pChunk = 0, $pNextChunk = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pChunk = sp + 4|0;
 $pNextChunk = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$pChunk>>2] = $2;
 while(1) {
  $3 = HEAP32[$pChunk>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$pChunk>>2]|0;
  $6 = HEAP32[$5>>2]|0;
  HEAP32[$pNextChunk>>2] = $6;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 4|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$pChunk>>2]|0;
  _sqlite3DbFree($9,$10);
  $11 = HEAP32[$pNextChunk>>2]|0;
  HEAP32[$pChunk>>2] = $11;
 }
 $12 = HEAP32[$0>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 24|0);
 HEAP16[$14>>1] = 0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 8|0);
 HEAP32[$16>>2] = 0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 12|0);
 HEAP32[$18>>2] = 0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 20|0);
 HEAP32[$20>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 26|0);
 HEAP16[$22>>1] = 1;
 STACKTOP = sp;return;
}
function _serialGet($buf,$serial_type,$pMem) {
 $buf = $buf|0;
 $serial_type = $serial_type|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0.0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $x = 0, $y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $x = sp;
 $y = sp + 8|0;
 HEAP32[$0>>2] = $buf;
 HEAP32[$1>>2] = $serial_type;
 HEAP32[$2>>2] = $pMem;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = $5 << 24;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 1|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = $10 << 16;
 $12 = $6 | $11;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 2|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = $16 << 8;
 $18 = $12 | $17;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 3|0);
 $21 = HEAP8[$20>>0]|0;
 $22 = $21&255;
 $23 = $18 | $22;
 $24 = $x;
 $25 = $24;
 HEAP32[$25>>2] = $23;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 4|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = $31 << 24;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 4|0);
 $35 = ((($34)) + 1|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $37 << 16;
 $39 = $32 | $38;
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 4|0);
 $42 = ((($41)) + 2|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = $43&255;
 $45 = $44 << 8;
 $46 = $39 | $45;
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 4|0);
 $49 = ((($48)) + 3|0);
 $50 = HEAP8[$49>>0]|0;
 $51 = $50&255;
 $52 = $46 | $51;
 HEAP32[$y>>2] = $52;
 $53 = $x;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (($53) + 4)|0;
 $57 = $56;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP32[$y>>2]|0;
 $60 = (_i64Add(0,($55|0),($59|0),0)|0);
 $61 = tempRet0;
 $62 = $x;
 $63 = $62;
 HEAP32[$63>>2] = $60;
 $64 = (($62) + 4)|0;
 $65 = $64;
 HEAP32[$65>>2] = $61;
 $66 = HEAP32[$1>>2]|0;
 $67 = ($66|0)==(6);
 if ($67) {
  $68 = $x;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  $75 = $74;
  $76 = $75;
  HEAP32[$76>>2] = $70;
  $77 = (($75) + 4)|0;
  $78 = $77;
  HEAP32[$78>>2] = $73;
  $79 = HEAP32[$2>>2]|0;
  $80 = ((($79)) + 8|0);
  HEAP16[$80>>1] = 4;
  STACKTOP = sp;return 8;
 } else {
  $81 = HEAP32[$2>>2]|0;
  ;HEAP32[$81>>2]=HEAP32[$x>>2]|0;HEAP32[$81+4>>2]=HEAP32[$x+4>>2]|0;
  $82 = HEAP32[$2>>2]|0;
  $83 = +HEAPF64[$82>>3];
  $84 = (_sqlite3IsNaN($83)|0);
  $85 = ($84|0)!=(0);
  $86 = $85 ? 1 : 8;
  $87 = $86&65535;
  $88 = HEAP32[$2>>2]|0;
  $89 = ((($88)) + 8|0);
  HEAP16[$89>>1] = $87;
  STACKTOP = sp;return 8;
 }
 return (0)|0;
}
function _sqlite3IsNaN($x) {
 $x = +$x;
 var $0 = 0, $1 = 0.0, $2 = 0.0, $3 = 0.0, $4 = 0.0, $5 = 0, $6 = 0, $7 = 0, $rc = 0, $y = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $rc = sp + 24|0;
 $y = sp + 8|0;
 $z = sp;
 HEAPF64[$0>>3] = $x;
 $1 = +HEAPF64[$0>>3];
 HEAPF64[$y>>3] = $1;
 $2 = +HEAPF64[$y>>3];
 HEAPF64[$z>>3] = $2;
 $3 = +HEAPF64[$y>>3];
 $4 = +HEAPF64[$z>>3];
 $5 = $3 != $4;
 $6 = $5&1;
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($7|0);
}
function _btreeNext($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $idx = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $rc = sp + 8|0;
 $idx = sp + 4|0;
 $pPage = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)!=(1);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 66|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)>=(3);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = (_btreeRestoreCursorPosition($13)|0);
   $15 = $14;
  } else {
   $15 = 0;
  }
  HEAP32[$rc>>2] = $15;
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $18;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 66|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = $21&255;
  $23 = (0)==($22|0);
  if ($23) {
   $24 = HEAP32[$2>>2]|0;
   HEAP32[$24>>2] = 1;
   HEAP32[$0>>2] = 0;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 60|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$1>>2]|0;
   $30 = ((($29)) + 66|0);
   HEAP8[$30>>0] = 1;
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 60|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = ($33|0)>(0);
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 60|0);
   HEAP32[$36>>2] = 0;
   if ($34) {
    HEAP32[$0>>2] = 0;
    $122 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($122|0);
   }
  }
 }
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 68|0);
 $39 = HEAP8[$38>>0]|0;
 $40 = $39 << 24 >> 24;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 120|0);
 $43 = (($42) + ($40<<2)|0);
 $44 = HEAP32[$43>>2]|0;
 HEAP32[$pPage>>2] = $44;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 68|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47 << 24 >> 24;
 $49 = HEAP32[$1>>2]|0;
 $50 = ((($49)) + 80|0);
 $51 = (($50) + ($48<<1)|0);
 $52 = HEAP16[$51>>1]|0;
 $53 = (($52) + 1)<<16>>16;
 HEAP16[$51>>1] = $53;
 $54 = $53&65535;
 HEAP32[$idx>>2] = $54;
 $55 = HEAP32[$idx>>2]|0;
 $56 = HEAP32[$pPage>>2]|0;
 $57 = ((($56)) + 18|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = ($55|0)>=($59|0);
 $61 = HEAP32[$pPage>>2]|0;
 $62 = ((($61)) + 4|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = ($63<<24>>24)!=(0);
 if (!($60)) {
  if ($64) {
   HEAP32[$0>>2] = 0;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  } else {
   $120 = HEAP32[$1>>2]|0;
   $121 = (_moveToLeftmost($120)|0);
   HEAP32[$0>>2] = $121;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
 }
 if (!($64)) {
  $65 = HEAP32[$1>>2]|0;
  $66 = HEAP32[$pPage>>2]|0;
  $67 = ((($66)) + 5|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = (($69) + 8)|0;
  $71 = HEAP32[$pPage>>2]|0;
  $72 = ((($71)) + 56|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + ($70)|0);
  $75 = (_sqlite3Get4byte($74)|0);
  $76 = (_moveToChild($65,$75)|0);
  HEAP32[$rc>>2] = $76;
  $77 = HEAP32[$rc>>2]|0;
  $78 = ($77|0)!=(0);
  if ($78) {
   $79 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $79;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  } else {
   $80 = HEAP32[$1>>2]|0;
   $81 = (_moveToLeftmost($80)|0);
   HEAP32[$0>>2] = $81;
   $122 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($122|0);
  }
 }
 while(1) {
  $82 = HEAP32[$1>>2]|0;
  $83 = ((($82)) + 68|0);
  $84 = HEAP8[$83>>0]|0;
  $85 = $84 << 24 >> 24;
  $86 = ($85|0)==(0);
  if ($86) {
   label = 17;
   break;
  }
  $90 = HEAP32[$1>>2]|0;
  _moveToParent($90);
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 68|0);
  $93 = HEAP8[$92>>0]|0;
  $94 = $93 << 24 >> 24;
  $95 = HEAP32[$1>>2]|0;
  $96 = ((($95)) + 120|0);
  $97 = (($96) + ($94<<2)|0);
  $98 = HEAP32[$97>>2]|0;
  HEAP32[$pPage>>2] = $98;
  $99 = HEAP32[$1>>2]|0;
  $100 = ((($99)) + 68|0);
  $101 = HEAP8[$100>>0]|0;
  $102 = $101 << 24 >> 24;
  $103 = HEAP32[$1>>2]|0;
  $104 = ((($103)) + 80|0);
  $105 = (($104) + ($102<<1)|0);
  $106 = HEAP16[$105>>1]|0;
  $107 = $106&65535;
  $108 = HEAP32[$pPage>>2]|0;
  $109 = ((($108)) + 18|0);
  $110 = HEAP16[$109>>1]|0;
  $111 = $110&65535;
  $112 = ($107|0)>=($111|0);
  if (!($112)) {
   break;
  }
 }
 if ((label|0) == 17) {
  $87 = HEAP32[$2>>2]|0;
  HEAP32[$87>>2] = 1;
  $88 = HEAP32[$1>>2]|0;
  $89 = ((($88)) + 66|0);
  HEAP8[$89>>0] = 0;
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 $113 = HEAP32[$pPage>>2]|0;
 $114 = ((($113)) + 2|0);
 $115 = HEAP8[$114>>0]|0;
 $116 = ($115<<24>>24)!=(0);
 if ($116) {
  $117 = HEAP32[$1>>2]|0;
  $118 = HEAP32[$2>>2]|0;
  $119 = (_sqlite3BtreeNext($117,$118)|0);
  HEAP32[$0>>2] = $119;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 } else {
  HEAP32[$0>>2] = 0;
  $122 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($122|0);
 }
 return (0)|0;
}
function _moveToLeftmost($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $pPage = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pgno = sp + 8|0;
 $rc = sp + 4|0;
 $pPage = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $1 = HEAP32[$rc>>2]|0;
  $2 = ($1|0)==(0);
  if (!($2)) {
   label = 5;
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 68|0);
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 120|0);
  $9 = (($8) + ($6<<2)|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pPage>>2] = $10;
  $11 = ((($10)) + 4|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = ($12<<24>>24)!=(0);
  $14 = $13 ^ 1;
  if (!($14)) {
   label = 5;
   break;
  }
  $15 = HEAP32[$pPage>>2]|0;
  $16 = ((($15)) + 56|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pPage>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP16[$19>>1]|0;
  $21 = $20&65535;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 68|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24 << 24 >> 24;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 80|0);
  $28 = (($27) + ($25<<1)|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  $31 = $30<<1;
  $32 = HEAP32[$pPage>>2]|0;
  $33 = ((($32)) + 64|0);
  $34 = HEAP32[$33>>2]|0;
  $35 = (($34) + ($31)|0);
  $36 = HEAP8[$35>>0]|0;
  $37 = $36&255;
  $38 = $37 << 8;
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 68|0);
  $41 = HEAP8[$40>>0]|0;
  $42 = $41 << 24 >> 24;
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 80|0);
  $45 = (($44) + ($42<<1)|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46&65535;
  $48 = $47<<1;
  $49 = HEAP32[$pPage>>2]|0;
  $50 = ((($49)) + 64|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = (($51) + ($48)|0);
  $53 = ((($52)) + 1|0);
  $54 = HEAP8[$53>>0]|0;
  $55 = $54&255;
  $56 = $38 | $55;
  $57 = $21 & $56;
  $58 = (($17) + ($57)|0);
  $59 = (_sqlite3Get4byte($58)|0);
  HEAP32[$pgno>>2] = $59;
  $60 = HEAP32[$0>>2]|0;
  $61 = HEAP32[$pgno>>2]|0;
  $62 = (_moveToChild($60,$61)|0);
  HEAP32[$rc>>2] = $62;
 }
 if ((label|0) == 5) {
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 return (0)|0;
}
function _targetSrcList($pParse,$pStep) {
 $pParse = $pParse|0;
 $pStep = $pStep|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $iDb = 0, $or$cond = 0, $pSrc = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $db = sp + 12|0;
 $iDb = sp + 8|0;
 $pSrc = sp + 4|0;
 $zDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pStep;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = (_sqlite3SrcListAppend($4,0,0,0)|0);
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$pSrc>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  $47 = HEAP32[$pSrc>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3DbStrDup($8,$11)|0);
 $13 = HEAP32[$pSrc>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) - 1)|0;
 $16 = HEAP32[$pSrc>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = (($17) + (($15*72)|0)|0);
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = $12;
 $20 = HEAP32[$db>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 4|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 20|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (_sqlite3SchemaToIndex($20,$25)|0);
 HEAP32[$iDb>>2] = $26;
 $27 = HEAP32[$iDb>>2]|0;
 $28 = ($27|0)==(0);
 $29 = HEAP32[$iDb>>2]|0;
 $30 = ($29|0)>=(2);
 $or$cond = $28 | $30;
 if (!($or$cond)) {
  $47 = HEAP32[$pSrc>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 $31 = HEAP32[$iDb>>2]|0;
 $32 = HEAP32[$db>>2]|0;
 $33 = ((($32)) + 16|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) + ($31<<4)|0);
 $36 = HEAP32[$35>>2]|0;
 HEAP32[$zDb>>2] = $36;
 $37 = HEAP32[$db>>2]|0;
 $38 = HEAP32[$zDb>>2]|0;
 $39 = (_sqlite3DbStrDup($37,$38)|0);
 $40 = HEAP32[$pSrc>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) - 1)|0;
 $43 = HEAP32[$pSrc>>2]|0;
 $44 = ((($43)) + 8|0);
 $45 = (($44) + (($42*72)|0)|0);
 $46 = ((($45)) + 4|0);
 HEAP32[$46>>2] = $39;
 $47 = HEAP32[$pSrc>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _fkTriggerDelete($dbMem,$p) {
 $dbMem = $dbMem|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pStep = sp;
 HEAP32[$0>>2] = $dbMem;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pStep>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pStep>>2]|0;
 $9 = ((($8)) + 16|0);
 $10 = HEAP32[$9>>2]|0;
 _sqlite3ExprDelete($7,$10);
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$pStep>>2]|0;
 $13 = ((($12)) + 20|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3ExprListDelete($11,$14);
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$pStep>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP32[$17>>2]|0;
 _sqlite3SelectDelete($15,$18);
 $19 = HEAP32[$0>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 12|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3ExprDelete($19,$22);
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($23,$24);
 STACKTOP = sp;return;
}
function _checkColumnOverlap($pIdList,$pEList) {
 $pIdList = $pIdList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $e = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $e = sp;
 HEAP32[$1>>2] = $pIdList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 $or$cond = $4 | $6;
 if ($or$cond) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 HEAP32[$e>>2] = 0;
 while(1) {
  $7 = HEAP32[$e>>2]|0;
  $8 = HEAP32[$2>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   label = 8;
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$e>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*20)|0)|0);
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3IdListIndex($11,$18)|0);
  $20 = ($19|0)>=(0);
  if ($20) {
   label = 6;
   break;
  }
  $21 = HEAP32[$e>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$e>>2] = $22;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = 1;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _checkConstraintExprNode($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pExpr;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(152);
 if (!($5)) {
  STACKTOP = sp;return 0;
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 32|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8 << 16 >> 16;
 $10 = ($9|0)>=(0);
 if (!($10)) {
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 20|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = $29&255;
  $31 = $30 | 2;
  $32 = $31&255;
  HEAP8[$28>>0] = $32;
  STACKTOP = sp;return 0;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 32|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13 << 16 >> 16;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14<<2)|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)>=(0);
 if (!($20)) {
  STACKTOP = sp;return 0;
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 20|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = $24 | 1;
 $26 = $25&255;
 HEAP8[$22>>0] = $26;
 STACKTOP = sp;return 0;
}
function _xferCompatibleIndex($pDest,$pSrc) {
 $pDest = $pDest|0;
 $pSrc = $pSrc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pDest;
 HEAP32[$2>>2] = $pSrc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 50|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 50|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = ($6|0)!=($10|0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 54|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 54|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = ($15|0)!=($19|0);
 if ($20) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 50|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = $24&65535;
  $26 = ($21|0)<($25|0);
  if (!($26)) {
   label = 17;
   break;
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (($30) + ($27<<1)|0);
  $32 = HEAP16[$31>>1]|0;
  $33 = $32 << 16 >> 16;
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<1)|0);
  $39 = HEAP16[$38>>1]|0;
  $40 = $39 << 16 >> 16;
  $41 = ($33|0)!=($40|0);
  if ($41) {
   label = 8;
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = ((($43)) + 4|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<1)|0);
  $47 = HEAP16[$46>>1]|0;
  $48 = $47 << 16 >> 16;
  $49 = ($48|0)==(-2);
  if ($49) {
   $50 = HEAP32[$i>>2]|0;
   $51 = HEAP32[$2>>2]|0;
   $52 = ((($51)) + 40|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   $56 = (($55) + (($50*20)|0)|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = HEAP32[$i>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 40|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = ((($61)) + 4|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = (($63) + (($58*20)|0)|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (_sqlite3ExprCompare($57,$65,-1)|0);
   $67 = ($66|0)!=(0);
   if ($67) {
    label = 11;
    break;
   }
  }
  $68 = HEAP32[$i>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = ((($69)) + 28|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + ($68)|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = HEAP32[$i>>2]|0;
  $76 = HEAP32[$1>>2]|0;
  $77 = ((($76)) + 28|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = (($78) + ($75)|0);
  $80 = HEAP8[$79>>0]|0;
  $81 = $80&255;
  $82 = ($74|0)!=($81|0);
  if ($82) {
   label = 13;
   break;
  }
  $83 = HEAP32[$i>>2]|0;
  $84 = HEAP32[$2>>2]|0;
  $85 = ((($84)) + 32|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = (($86) + ($83<<2)|0);
  $88 = HEAP32[$87>>2]|0;
  $89 = HEAP32[$i>>2]|0;
  $90 = HEAP32[$1>>2]|0;
  $91 = ((($90)) + 32|0);
  $92 = HEAP32[$91>>2]|0;
  $93 = (($92) + ($89<<2)|0);
  $94 = HEAP32[$93>>2]|0;
  $95 = (_sqlite3_stricmp($88,$94)|0);
  $96 = ($95|0)!=(0);
  if ($96) {
   label = 15;
   break;
  }
  $97 = HEAP32[$i>>2]|0;
  $98 = (($97) + 1)|0;
  HEAP32[$i>>2] = $98;
 }
 if ((label|0) == 8) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 13) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $107 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($107|0);
 }
 else if ((label|0) == 17) {
  $99 = HEAP32[$2>>2]|0;
  $100 = ((($99)) + 36|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = HEAP32[$1>>2]|0;
  $103 = ((($102)) + 36|0);
  $104 = HEAP32[$103>>2]|0;
  $105 = (_sqlite3ExprCompare($101,$104,-1)|0);
  $106 = ($105|0)!=(0);
  if ($106) {
   HEAP32[$0>>2] = 0;
   $107 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($107|0);
  } else {
   HEAP32[$0>>2] = 1;
   $107 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($107|0);
  }
 }
 return (0)|0;
}
function _sqlite3AuthContextPush($pParse,$pContext,$zContext) {
 $pParse = $pParse|0;
 $pContext = $pContext|0;
 $zContext = $zContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pContext;
 HEAP32[$2>>2] = $zContext;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 HEAP32[$5>>2] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 448|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$2>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 448|0);
 HEAP32[$12>>2] = $10;
 STACKTOP = sp;return;
}
function _sqlite3MaterializeView($pParse,$pView,$pWhere,$iCur) {
 $pParse = $pParse|0;
 $pView = $pView|0;
 $pWhere = $pWhere|0;
 $iCur = $iCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $dest = 0, $iDb = 0, $pFrom = 0, $pSel = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $dest = sp + 16|0;
 $pSel = sp + 12|0;
 $pFrom = sp + 8|0;
 $db = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pView;
 HEAP32[$2>>2] = $pWhere;
 HEAP32[$3>>2] = $iCur;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$db>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 64|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3SchemaToIndex($6,$9)|0);
 HEAP32[$iDb>>2] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3ExprDup($11,$12,0)|0);
 HEAP32[$2>>2] = $13;
 $14 = HEAP32[$db>>2]|0;
 $15 = (_sqlite3SrcListAppend($14,0,0,0)|0);
 HEAP32[$pFrom>>2] = $15;
 $16 = HEAP32[$pFrom>>2]|0;
 $17 = ($16|0)!=(0|0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3DbStrDup($18,$20)|0);
  $22 = HEAP32[$pFrom>>2]|0;
  $23 = ((($22)) + 8|0);
  $24 = ((($23)) + 8|0);
  HEAP32[$24>>2] = $21;
  $25 = HEAP32[$db>>2]|0;
  $26 = HEAP32[$iDb>>2]|0;
  $27 = HEAP32[$db>>2]|0;
  $28 = ((($27)) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + ($26<<4)|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (_sqlite3DbStrDup($25,$31)|0);
  $33 = HEAP32[$pFrom>>2]|0;
  $34 = ((($33)) + 8|0);
  $35 = ((($34)) + 4|0);
  HEAP32[$35>>2] = $32;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pFrom>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = (_sqlite3SelectNew($36,0,$37,$38,0,0,0,131072,0,0)|0);
 HEAP32[$pSel>>2] = $39;
 $40 = HEAP32[$3>>2]|0;
 _sqlite3SelectDestInit($dest,12,$40);
 $41 = HEAP32[$0>>2]|0;
 $42 = HEAP32[$pSel>>2]|0;
 (_sqlite3Select($41,$42,$dest)|0);
 $43 = HEAP32[$db>>2]|0;
 $44 = HEAP32[$pSel>>2]|0;
 _sqlite3SelectDelete($43,$44);
 STACKTOP = sp;return;
}
function _updateVirtualTable($pParse,$pSrc,$pTab,$pChanges,$pRowid,$aXRef,$pWhere,$onError) {
 $pParse = $pParse|0;
 $pSrc = $pSrc|0;
 $pTab = $pTab|0;
 $pChanges = $pChanges|0;
 $pRowid = $pRowid|0;
 $aXRef = $aXRef|0;
 $pWhere = $pWhere|0;
 $onError = $onError|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aDummy = 0, $addr = 0, $bOnePass = 0, $db = 0, $ephemTab = 0, $i = 0, $iCsr = 0, $nArg = 0, $pVTab = 0, $pWInfo = 0, $regArg = 0, $regRec = 0, $regRowid = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $4 = sp + 72|0;
 $5 = sp + 68|0;
 $6 = sp + 64|0;
 $7 = sp + 60|0;
 $v = sp + 56|0;
 $ephemTab = sp + 52|0;
 $i = sp + 48|0;
 $db = sp + 44|0;
 $pVTab = sp + 40|0;
 $pWInfo = sp + 36|0;
 $nArg = sp + 32|0;
 $regArg = sp + 28|0;
 $regRec = sp + 24|0;
 $regRowid = sp + 20|0;
 $iCsr = sp + 16|0;
 $aDummy = sp + 8|0;
 $bOnePass = sp + 4|0;
 $addr = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSrc;
 HEAP32[$2>>2] = $pTab;
 HEAP32[$3>>2] = $pChanges;
 HEAP32[$4>>2] = $pRowid;
 HEAP32[$5>>2] = $aXRef;
 HEAP32[$6>>2] = $pWhere;
 HEAP32[$7>>2] = $onError;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$v>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$db>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (_sqlite3GetVTable($13,$14)|0);
 HEAP32[$pVTab>>2] = $15;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 34|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18 << 16 >> 16;
 $20 = (2 + ($19))|0;
 HEAP32[$nArg>>2] = $20;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = ((($22)) + 44|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$iCsr>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 40|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 HEAP32[$26>>2] = $28;
 HEAP32[$ephemTab>>2] = $27;
 $29 = HEAP32[$v>>2]|0;
 $30 = HEAP32[$ephemTab>>2]|0;
 $31 = HEAP32[$nArg>>2]|0;
 $32 = (_sqlite3VdbeAddOp2($29,107,$30,$31)|0);
 HEAP32[$addr>>2] = $32;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 44|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (($35) + 1)|0;
 HEAP32[$regArg>>2] = $36;
 $37 = HEAP32[$nArg>>2]|0;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 44|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (($40) + ($37))|0;
 HEAP32[$39>>2] = $41;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 44|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + 1)|0;
 HEAP32[$43>>2] = $45;
 HEAP32[$regRec>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 44|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = (($48) + 1)|0;
 HEAP32[$47>>2] = $49;
 HEAP32[$regRowid>>2] = $49;
 $50 = HEAP32[$0>>2]|0;
 $51 = HEAP32[$1>>2]|0;
 $52 = HEAP32[$6>>2]|0;
 $53 = (_sqlite3WhereBegin($50,$51,$52,0,0,4,0)|0);
 HEAP32[$pWInfo>>2] = $53;
 $54 = HEAP32[$pWInfo>>2]|0;
 $55 = ($54|0)==(0|0);
 if ($55) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$v>>2]|0;
 $57 = HEAP32[$iCsr>>2]|0;
 $58 = HEAP32[$regArg>>2]|0;
 (_sqlite3VdbeAddOp2($56,123,$57,$58)|0);
 $59 = HEAP32[$4>>2]|0;
 $60 = ($59|0)!=(0|0);
 if ($60) {
  $61 = HEAP32[$0>>2]|0;
  $62 = HEAP32[$4>>2]|0;
  $63 = HEAP32[$regArg>>2]|0;
  $64 = (($63) + 1)|0;
  _sqlite3ExprCode($61,$62,$64);
 } else {
  $65 = HEAP32[$v>>2]|0;
  $66 = HEAP32[$iCsr>>2]|0;
  $67 = HEAP32[$regArg>>2]|0;
  $68 = (($67) + 1)|0;
  (_sqlite3VdbeAddOp2($65,123,$66,$68)|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $69 = HEAP32[$i>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = ((($70)) + 34|0);
  $72 = HEAP16[$71>>1]|0;
  $73 = $72 << 16 >> 16;
  $74 = ($69|0)<($73|0);
  if (!($74)) {
   break;
  }
  $75 = HEAP32[$i>>2]|0;
  $76 = HEAP32[$5>>2]|0;
  $77 = (($76) + ($75<<2)|0);
  $78 = HEAP32[$77>>2]|0;
  $79 = ($78|0)>=(0);
  if ($79) {
   $80 = HEAP32[$0>>2]|0;
   $81 = HEAP32[$i>>2]|0;
   $82 = HEAP32[$5>>2]|0;
   $83 = (($82) + ($81<<2)|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$3>>2]|0;
   $86 = ((($85)) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = (($87) + (($84*20)|0)|0);
   $89 = HEAP32[$88>>2]|0;
   $90 = HEAP32[$regArg>>2]|0;
   $91 = (($90) + 2)|0;
   $92 = HEAP32[$i>>2]|0;
   $93 = (($91) + ($92))|0;
   _sqlite3ExprCode($80,$89,$93);
  } else {
   $94 = HEAP32[$v>>2]|0;
   $95 = HEAP32[$iCsr>>2]|0;
   $96 = HEAP32[$i>>2]|0;
   $97 = HEAP32[$regArg>>2]|0;
   $98 = (($97) + 2)|0;
   $99 = HEAP32[$i>>2]|0;
   $100 = (($98) + ($99))|0;
   (_sqlite3VdbeAddOp3($94,156,$95,$96,$100)|0);
  }
  $101 = HEAP32[$i>>2]|0;
  $102 = (($101) + 1)|0;
  HEAP32[$i>>2] = $102;
 }
 $103 = HEAP32[$pWInfo>>2]|0;
 $104 = (_sqlite3WhereOkOnePass($103,$aDummy)|0);
 HEAP32[$bOnePass>>2] = $104;
 $105 = HEAP32[$bOnePass>>2]|0;
 $106 = ($105|0)!=(0);
 $107 = HEAP32[$v>>2]|0;
 if ($106) {
  $108 = HEAP32[$addr>>2]|0;
  (_sqlite3VdbeChangeToNoop($107,$108)|0);
  $109 = HEAP32[$0>>2]|0;
  $110 = ((($109)) + 124|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = ($111|0)==(0|0);
  if ($112) {
   $113 = HEAP32[$0>>2]|0;
   $114 = ((($113)) + 20|0);
   HEAP8[$114>>0] = 0;
  }
 } else {
  $115 = HEAP32[$regArg>>2]|0;
  $116 = HEAP32[$nArg>>2]|0;
  $117 = HEAP32[$regRec>>2]|0;
  (_sqlite3VdbeAddOp3($107,99,$115,$116,$117)|0);
  $118 = HEAP32[$v>>2]|0;
  $119 = HEAP32[$ephemTab>>2]|0;
  $120 = HEAP32[$regRowid>>2]|0;
  (_sqlite3VdbeAddOp2($118,114,$119,$120)|0);
  $121 = HEAP32[$v>>2]|0;
  $122 = HEAP32[$ephemTab>>2]|0;
  $123 = HEAP32[$regRec>>2]|0;
  $124 = HEAP32[$regRowid>>2]|0;
  (_sqlite3VdbeAddOp3($121,115,$122,$123,$124)|0);
 }
 $125 = HEAP32[$bOnePass>>2]|0;
 $126 = ($125|0)==(0);
 L21: do {
  if ($126) {
   $127 = HEAP32[$pWInfo>>2]|0;
   _sqlite3WhereEnd($127);
   $128 = HEAP32[$v>>2]|0;
   $129 = HEAP32[$ephemTab>>2]|0;
   $130 = (_sqlite3VdbeAddOp1($128,57,$129)|0);
   HEAP32[$addr>>2] = $130;
   HEAP32[$i>>2] = 0;
   while(1) {
    $131 = HEAP32[$i>>2]|0;
    $132 = HEAP32[$nArg>>2]|0;
    $133 = ($131|0)<($132|0);
    if (!($133)) {
     break L21;
    }
    $134 = HEAP32[$v>>2]|0;
    $135 = HEAP32[$ephemTab>>2]|0;
    $136 = HEAP32[$i>>2]|0;
    $137 = HEAP32[$regArg>>2]|0;
    $138 = HEAP32[$i>>2]|0;
    $139 = (($137) + ($138))|0;
    (_sqlite3VdbeAddOp3($134,96,$135,$136,$139)|0);
    $140 = HEAP32[$i>>2]|0;
    $141 = (($140) + 1)|0;
    HEAP32[$i>>2] = $141;
   }
  }
 } while(0);
 $142 = HEAP32[$0>>2]|0;
 $143 = HEAP32[$2>>2]|0;
 _sqlite3VtabMakeWritable($142,$143);
 $144 = HEAP32[$v>>2]|0;
 $145 = HEAP32[$nArg>>2]|0;
 $146 = HEAP32[$regArg>>2]|0;
 $147 = HEAP32[$pVTab>>2]|0;
 (_sqlite3VdbeAddOp4($144,12,0,$145,$146,$147,-10)|0);
 $148 = HEAP32[$v>>2]|0;
 $149 = HEAP32[$7>>2]|0;
 $150 = ($149|0)==(10);
 $151 = HEAP32[$7>>2]|0;
 $152 = $150 ? 2 : $151;
 $153 = $152&255;
 _sqlite3VdbeChangeP5($148,$153);
 $154 = HEAP32[$0>>2]|0;
 _sqlite3MayAbort($154);
 $155 = HEAP32[$bOnePass>>2]|0;
 $156 = ($155|0)==(0);
 if ($156) {
  $157 = HEAP32[$v>>2]|0;
  $158 = HEAP32[$ephemTab>>2]|0;
  $159 = HEAP32[$addr>>2]|0;
  $160 = (($159) + 1)|0;
  (_sqlite3VdbeAddOp2($157,7,$158,$160)|0);
  $161 = HEAP32[$v>>2]|0;
  $162 = HEAP32[$addr>>2]|0;
  _sqlite3VdbeJumpHere($161,$162);
  $163 = HEAP32[$v>>2]|0;
  $164 = HEAP32[$ephemTab>>2]|0;
  (_sqlite3VdbeAddOp2($163,111,$164,0)|0);
  STACKTOP = sp;return;
 } else {
  $165 = HEAP32[$pWInfo>>2]|0;
  _sqlite3WhereEnd($165);
  STACKTOP = sp;return;
 }
}
function _sqlite3WhereOkOnePass($pWInfo,$aiCur) {
 $pWInfo = $pWInfo|0;
 $aiCur = $aiCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWInfo;
 HEAP32[$1>>2] = $aiCur;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 ;HEAP32[$2>>2]=HEAP32[$4>>2]|0;HEAP32[$2+4>>2]=HEAP32[$4+4>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 45|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3AuthContextPop($pContext) {
 $pContext = $pContext|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pContext;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 448|0);
 HEAP32[$10>>2] = $6;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3SrcListEnlarge($db,$pSrc,$nExtra,$iStart) {
 $db = $db|0;
 $pSrc = $pSrc|0;
 $nExtra = $nExtra|0;
 $iStart = $iStart|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $i = 0, $nAlloc = 0, $nGot = 0, $pNew = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp + 16|0;
 $i = sp + 12|0;
 $pNew = sp + 8|0;
 $nAlloc = sp + 4|0;
 $nGot = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSrc;
 HEAP32[$3>>2] = $nExtra;
 HEAP32[$4>>2] = $iStart;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (($6) + ($7))|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8>>>0)>($11>>>0);
 do {
  if ($12) {
   $13 = HEAP32[$2>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$3>>2]|0;
   $16 = (($14) + ($15))|0;
   HEAP32[$nAlloc>>2] = $16;
   $17 = HEAP32[$1>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = HEAP32[$nAlloc>>2]|0;
   $20 = (($19) - 1)|0;
   $21 = ($20*72)|0;
   $22 = (80 + ($21))|0;
   $23 = (_sqlite3DbRealloc($17,$18,$22,0)|0);
   HEAP32[$pNew>>2] = $23;
   $24 = HEAP32[$pNew>>2]|0;
   $25 = ($24|0)==(0|0);
   if (!($25)) {
    $27 = HEAP32[$pNew>>2]|0;
    HEAP32[$2>>2] = $27;
    $28 = HEAP32[$1>>2]|0;
    $29 = HEAP32[$pNew>>2]|0;
    $30 = (_sqlite3DbMallocSize($28,$29)|0);
    $31 = (($30) - 80)|0;
    $32 = (($31>>>0) / 72)&-1;
    $33 = (($32) + 1)|0;
    HEAP32[$nGot>>2] = $33;
    $34 = HEAP32[$nGot>>2]|0;
    $35 = HEAP32[$2>>2]|0;
    $36 = ((($35)) + 4|0);
    HEAP32[$36>>2] = $34;
    break;
   }
   $26 = HEAP32[$2>>2]|0;
   HEAP32[$0>>2] = $26;
   $79 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($79|0);
  }
 } while(0);
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) - 1)|0;
 HEAP32[$i>>2] = $39;
 while(1) {
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$4>>2]|0;
  $42 = ($40|0)>=($41|0);
  if (!($42)) {
   break;
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = HEAP32[$3>>2]|0;
  $45 = (($43) + ($44))|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = ((($46)) + 8|0);
  $48 = (($47) + (($45*72)|0)|0);
  $49 = HEAP32[$i>>2]|0;
  $50 = HEAP32[$2>>2]|0;
  $51 = ((($50)) + 8|0);
  $52 = (($51) + (($49*72)|0)|0);
  dest=$48; src=$52; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + -1)|0;
  HEAP32[$i>>2] = $54;
 }
 $55 = HEAP32[$3>>2]|0;
 $56 = HEAP32[$2>>2]|0;
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + ($55))|0;
 HEAP32[$56>>2] = $58;
 $59 = HEAP32[$4>>2]|0;
 $60 = HEAP32[$2>>2]|0;
 $61 = ((($60)) + 8|0);
 $62 = (($61) + (($59*72)|0)|0);
 $63 = HEAP32[$3>>2]|0;
 $64 = ($63*72)|0;
 _memset(($62|0),0,($64|0))|0;
 $65 = HEAP32[$4>>2]|0;
 HEAP32[$i>>2] = $65;
 while(1) {
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$4>>2]|0;
  $68 = HEAP32[$3>>2]|0;
  $69 = (($67) + ($68))|0;
  $70 = ($66|0)<($69|0);
  if (!($70)) {
   break;
  }
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  $73 = ((($72)) + 8|0);
  $74 = (($73) + (($71*72)|0)|0);
  $75 = ((($74)) + 44|0);
  HEAP32[$75>>2] = -1;
  $76 = HEAP32[$i>>2]|0;
  $77 = (($76) + 1)|0;
  HEAP32[$i>>2] = $77;
 }
 $78 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $78;
 $79 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _flattenSubquery($pParse,$p,$iFrom,$isAgg,$subqueryIsAgg) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iFrom = $iFrom|0;
 $isAgg = $isAgg|0;
 $subqueryIsAgg = $subqueryIsAgg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $db = 0, $i = 0, $iParent = 0, $ii = 0, $jointype = 0, $nSubSrc = 0, $or$cond = 0, $or$cond3 = 0, $pLimit = 0, $pList = 0, $pNew = 0, $pOffset = 0, $pOrderBy = 0, $pOrderBy1 = 0, $pParent = 0, $pPrior = 0, $pSrc = 0, $pSub = 0;
 var $pSub1 = 0, $pSubSrc = 0, $pSubitem = 0, $pTabToDel = 0, $pToplevel = 0, $pWhere = 0, $zName = 0, $zSavedAuthContext = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 112|0;
 $1 = sp + 108|0;
 $2 = sp + 104|0;
 $3 = sp + 100|0;
 $4 = sp + 96|0;
 $5 = sp + 92|0;
 $zSavedAuthContext = sp + 88|0;
 $pParent = sp + 84|0;
 $pSub = sp + 80|0;
 $pSub1 = sp + 76|0;
 $pSrc = sp + 72|0;
 $pSubSrc = sp + 68|0;
 $pList = sp + 64|0;
 $iParent = sp + 60|0;
 $i = sp + 56|0;
 $pWhere = sp + 52|0;
 $pSubitem = sp + 48|0;
 $db = sp + 44|0;
 $ii = sp + 40|0;
 $pNew = sp + 36|0;
 $pOrderBy = sp + 32|0;
 $pLimit = sp + 28|0;
 $pOffset = sp + 24|0;
 $pPrior = sp + 20|0;
 $pTabToDel = sp + 16|0;
 $pToplevel = sp + 12|0;
 $nSubSrc = sp + 8|0;
 $jointype = sp + 116|0;
 $zName = sp + 4|0;
 $pOrderBy1 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $iFrom;
 HEAP32[$4>>2] = $isAgg;
 HEAP32[$5>>2] = $subqueryIsAgg;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 448|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$zSavedAuthContext>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$db>>2] = $10;
 $11 = HEAP32[$db>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 1;
 $16 = ($15|0)!=(0);
 if ($16) {
  HEAP32[$0>>2] = 0;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 28|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pSrc>>2] = $19;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$pSrc>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = (($22) + (($20*72)|0)|0);
 HEAP32[$pSubitem>>2] = $23;
 $24 = HEAP32[$pSubitem>>2]|0;
 $25 = ((($24)) + 44|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$iParent>>2] = $26;
 $27 = HEAP32[$pSubitem>>2]|0;
 $28 = ((($27)) + 20|0);
 $29 = HEAP32[$28>>2]|0;
 HEAP32[$pSub>>2] = $29;
 $30 = HEAP32[$5>>2]|0;
 $31 = ($30|0)!=(0);
 do {
  if ($31) {
   $32 = HEAP32[$4>>2]|0;
   $33 = ($32|0)!=(0);
   if ($33) {
    HEAP32[$0>>2] = 0;
    $561 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($561|0);
   }
   $34 = HEAP32[$pSrc>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = ($35|0)>(1);
   if ($36) {
    HEAP32[$0>>2] = 0;
    $561 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($561|0);
   }
   $37 = HEAP32[$2>>2]|0;
   $38 = ((($37)) + 32|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)!=(0|0);
   if ($40) {
    $41 = HEAP32[$2>>2]|0;
    $42 = ((($41)) + 32|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ((($43)) + 4|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = $45 & 2097152;
    $47 = ($46|0)!=(0);
    if (!($47)) {
     label = 10;
    }
   } else {
    label = 10;
   }
   if ((label|0) == 10) {
    $48 = HEAP32[$2>>2]|0;
    $49 = HEAP32[$48>>2]|0;
    $50 = (_sqlite3ExprListFlags($49)|0);
    $51 = $50 & 2097152;
    $52 = ($51|0)!=(0);
    if (!($52)) {
     $53 = HEAP32[$2>>2]|0;
     $54 = ((($53)) + 44|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = (_sqlite3ExprListFlags($55)|0);
     $57 = $56 & 2097152;
     $58 = ($57|0)!=(0);
     if (!($58)) {
      break;
     }
    }
   }
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 } while(0);
 $59 = HEAP32[$pSub>>2]|0;
 $60 = ((($59)) + 28|0);
 $61 = HEAP32[$60>>2]|0;
 HEAP32[$pSubSrc>>2] = $61;
 $62 = HEAP32[$pSub>>2]|0;
 $63 = ((($62)) + 56|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = ($64|0)!=(0|0);
 if ($65) {
  $66 = HEAP32[$2>>2]|0;
  $67 = ((($66)) + 56|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = ($68|0)!=(0|0);
  if ($69) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $70 = HEAP32[$pSub>>2]|0;
 $71 = ((($70)) + 60|0);
 $72 = HEAP32[$71>>2]|0;
 $73 = ($72|0)!=(0|0);
 if ($73) {
  HEAP32[$0>>2] = 0;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $74 = HEAP32[$2>>2]|0;
 $75 = ((($74)) + 8|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = $76 & 256;
 $78 = ($77|0)!=(0);
 if ($78) {
  $79 = HEAP32[$pSub>>2]|0;
  $80 = ((($79)) + 56|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = ($81|0)!=(0|0);
  if ($82) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $83 = HEAP32[$pSubSrc>>2]|0;
 $84 = HEAP32[$83>>2]|0;
 $85 = ($84|0)==(0);
 if ($85) {
  HEAP32[$0>>2] = 0;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $86 = HEAP32[$pSub>>2]|0;
 $87 = ((($86)) + 8|0);
 $88 = HEAP32[$87>>2]|0;
 $89 = $88 & 1;
 $90 = ($89|0)!=(0);
 if ($90) {
  HEAP32[$0>>2] = 0;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $91 = HEAP32[$pSub>>2]|0;
 $92 = ((($91)) + 56|0);
 $93 = HEAP32[$92>>2]|0;
 $94 = ($93|0)!=(0|0);
 if ($94) {
  $95 = HEAP32[$pSrc>>2]|0;
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)>(1);
  $98 = HEAP32[$4>>2]|0;
  $99 = ($98|0)!=(0);
  $or$cond = $97 | $99;
  if ($or$cond) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $100 = HEAP32[$2>>2]|0;
 $101 = ((($100)) + 8|0);
 $102 = HEAP32[$101>>2]|0;
 $103 = $102 & 1;
 $104 = ($103|0)!=(0);
 $105 = HEAP32[$5>>2]|0;
 $106 = ($105|0)!=(0);
 $or$cond3 = $104 & $106;
 if ($or$cond3) {
  HEAP32[$0>>2] = 0;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $107 = HEAP32[$2>>2]|0;
 $108 = ((($107)) + 44|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = ($109|0)!=(0|0);
 if ($110) {
  $111 = HEAP32[$pSub>>2]|0;
  $112 = ((($111)) + 44|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = ($113|0)!=(0|0);
  if ($114) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $115 = HEAP32[$4>>2]|0;
 $116 = ($115|0)!=(0);
 if ($116) {
  $117 = HEAP32[$pSub>>2]|0;
  $118 = ((($117)) + 44|0);
  $119 = HEAP32[$118>>2]|0;
  $120 = ($119|0)!=(0|0);
  if ($120) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $121 = HEAP32[$pSub>>2]|0;
 $122 = ((($121)) + 56|0);
 $123 = HEAP32[$122>>2]|0;
 $124 = ($123|0)!=(0|0);
 if ($124) {
  $125 = HEAP32[$2>>2]|0;
  $126 = ((($125)) + 32|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = ($127|0)!=(0|0);
  if ($128) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $129 = HEAP32[$pSub>>2]|0;
 $130 = ((($129)) + 56|0);
 $131 = HEAP32[$130>>2]|0;
 $132 = ($131|0)!=(0|0);
 if ($132) {
  $133 = HEAP32[$2>>2]|0;
  $134 = ((($133)) + 8|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = $135 & 1;
  $137 = ($136|0)!=(0);
  if ($137) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $138 = HEAP32[$pSub>>2]|0;
 $139 = ((($138)) + 8|0);
 $140 = HEAP32[$139>>2]|0;
 $141 = $140 & 12288;
 $142 = ($141|0)!=(0);
 if ($142) {
  HEAP32[$0>>2] = 0;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $143 = HEAP32[$2>>2]|0;
 $144 = ((($143)) + 8|0);
 $145 = HEAP32[$144>>2]|0;
 $146 = $145 & 8192;
 $147 = ($146|0)!=(0);
 if ($147) {
  $148 = HEAP32[$pSub>>2]|0;
  $149 = ((($148)) + 48|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = ($150|0)!=(0|0);
  if ($151) {
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 }
 $152 = HEAP32[$pSubitem>>2]|0;
 $153 = ((($152)) + 36|0);
 $154 = HEAP8[$153>>0]|0;
 $155 = $154&255;
 $156 = $155 & 32;
 $157 = ($156|0)!=(0);
 if ($157) {
  HEAP32[$0>>2] = 0;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $158 = HEAP32[$pSub>>2]|0;
 $159 = ((($158)) + 48|0);
 $160 = HEAP32[$159>>2]|0;
 $161 = ($160|0)!=(0|0);
 L87: do {
  if ($161) {
   $162 = HEAP32[$pSub>>2]|0;
   $163 = ((($162)) + 44|0);
   $164 = HEAP32[$163>>2]|0;
   $165 = ($164|0)!=(0|0);
   if ($165) {
    HEAP32[$0>>2] = 0;
    $561 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($561|0);
   }
   $166 = HEAP32[$4>>2]|0;
   $167 = ($166|0)!=(0);
   do {
    if (!($167)) {
     $168 = HEAP32[$2>>2]|0;
     $169 = ((($168)) + 8|0);
     $170 = HEAP32[$169>>2]|0;
     $171 = $170 & 1;
     $172 = ($171|0)!=(0);
     if ($172) {
      break;
     }
     $173 = HEAP32[$pSrc>>2]|0;
     $174 = HEAP32[$173>>2]|0;
     $175 = ($174|0)!=(1);
     if ($175) {
      break;
     }
     $176 = HEAP32[$pSub>>2]|0;
     HEAP32[$pSub1>>2] = $176;
     while(1) {
      $177 = HEAP32[$pSub1>>2]|0;
      $178 = ($177|0)!=(0|0);
      if (!($178)) {
       break;
      }
      $179 = HEAP32[$pSub1>>2]|0;
      $180 = ((($179)) + 8|0);
      $181 = HEAP32[$180>>2]|0;
      $182 = $181 & 9;
      $183 = ($182|0)!=(0);
      if ($183) {
       label = 62;
       break;
      }
      $184 = HEAP32[$pSub1>>2]|0;
      $185 = ((($184)) + 48|0);
      $186 = HEAP32[$185>>2]|0;
      $187 = ($186|0)!=(0|0);
      if ($187) {
       $188 = HEAP32[$pSub1>>2]|0;
       $189 = ((($188)) + 4|0);
       $190 = HEAP8[$189>>0]|0;
       $191 = $190&255;
       $192 = ($191|0)!=(116);
       if ($192) {
        label = 62;
        break;
       }
      }
      $193 = HEAP32[$pSub1>>2]|0;
      $194 = ((($193)) + 28|0);
      $195 = HEAP32[$194>>2]|0;
      $196 = HEAP32[$195>>2]|0;
      $197 = ($196|0)<(1);
      if ($197) {
       label = 62;
       break;
      }
      $198 = HEAP32[$pSub1>>2]|0;
      $199 = ((($198)) + 48|0);
      $200 = HEAP32[$199>>2]|0;
      HEAP32[$pSub1>>2] = $200;
     }
     if ((label|0) == 62) {
      HEAP32[$0>>2] = 0;
      $561 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($561|0);
     }
     $201 = HEAP32[$2>>2]|0;
     $202 = ((($201)) + 44|0);
     $203 = HEAP32[$202>>2]|0;
     $204 = ($203|0)!=(0|0);
     if (!($204)) {
      break L87;
     }
     HEAP32[$ii>>2] = 0;
     while(1) {
      $205 = HEAP32[$ii>>2]|0;
      $206 = HEAP32[$2>>2]|0;
      $207 = ((($206)) + 44|0);
      $208 = HEAP32[$207>>2]|0;
      $209 = HEAP32[$208>>2]|0;
      $210 = ($205|0)<($209|0);
      if (!($210)) {
       break L87;
      }
      $211 = HEAP32[$ii>>2]|0;
      $212 = HEAP32[$2>>2]|0;
      $213 = ((($212)) + 44|0);
      $214 = HEAP32[$213>>2]|0;
      $215 = ((($214)) + 4|0);
      $216 = HEAP32[$215>>2]|0;
      $217 = (($216) + (($211*20)|0)|0);
      $218 = ((($217)) + 16|0);
      $219 = HEAP16[$218>>1]|0;
      $220 = $219&65535;
      $221 = ($220|0)==(0);
      if ($221) {
       break;
      }
      $222 = HEAP32[$ii>>2]|0;
      $223 = (($222) + 1)|0;
      HEAP32[$ii>>2] = $223;
     }
     HEAP32[$0>>2] = 0;
     $561 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($561|0);
    }
   } while(0);
   HEAP32[$0>>2] = 0;
   $561 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($561|0);
  }
 } while(0);
 $224 = HEAP32[$pSubitem>>2]|0;
 $225 = ((($224)) + 8|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = HEAP32[$1>>2]|0;
 $228 = ((($227)) + 448|0);
 HEAP32[$228>>2] = $226;
 $229 = HEAP32[$1>>2]|0;
 (_sqlite3AuthCheck($229,21,0,0,0)|0);
 $230 = HEAP32[$zSavedAuthContext>>2]|0;
 $231 = HEAP32[$1>>2]|0;
 $232 = ((($231)) + 448|0);
 HEAP32[$232>>2] = $230;
 $233 = HEAP32[$pSub>>2]|0;
 $234 = ((($233)) + 48|0);
 $235 = HEAP32[$234>>2]|0;
 HEAP32[$pSub>>2] = $235;
 while(1) {
  $236 = HEAP32[$pSub>>2]|0;
  $237 = ($236|0)!=(0|0);
  if (!($237)) {
   break;
  }
  $238 = HEAP32[$2>>2]|0;
  $239 = ((($238)) + 44|0);
  $240 = HEAP32[$239>>2]|0;
  HEAP32[$pOrderBy>>2] = $240;
  $241 = HEAP32[$2>>2]|0;
  $242 = ((($241)) + 56|0);
  $243 = HEAP32[$242>>2]|0;
  HEAP32[$pLimit>>2] = $243;
  $244 = HEAP32[$2>>2]|0;
  $245 = ((($244)) + 60|0);
  $246 = HEAP32[$245>>2]|0;
  HEAP32[$pOffset>>2] = $246;
  $247 = HEAP32[$2>>2]|0;
  $248 = ((($247)) + 48|0);
  $249 = HEAP32[$248>>2]|0;
  HEAP32[$pPrior>>2] = $249;
  $250 = HEAP32[$2>>2]|0;
  $251 = ((($250)) + 44|0);
  HEAP32[$251>>2] = 0;
  $252 = HEAP32[$2>>2]|0;
  $253 = ((($252)) + 28|0);
  HEAP32[$253>>2] = 0;
  $254 = HEAP32[$2>>2]|0;
  $255 = ((($254)) + 48|0);
  HEAP32[$255>>2] = 0;
  $256 = HEAP32[$2>>2]|0;
  $257 = ((($256)) + 56|0);
  HEAP32[$257>>2] = 0;
  $258 = HEAP32[$2>>2]|0;
  $259 = ((($258)) + 60|0);
  HEAP32[$259>>2] = 0;
  $260 = HEAP32[$db>>2]|0;
  $261 = HEAP32[$2>>2]|0;
  $262 = (_sqlite3SelectDup($260,$261,0)|0);
  HEAP32[$pNew>>2] = $262;
  $263 = HEAP32[$pOffset>>2]|0;
  $264 = HEAP32[$2>>2]|0;
  $265 = ((($264)) + 60|0);
  HEAP32[$265>>2] = $263;
  $266 = HEAP32[$pLimit>>2]|0;
  $267 = HEAP32[$2>>2]|0;
  $268 = ((($267)) + 56|0);
  HEAP32[$268>>2] = $266;
  $269 = HEAP32[$pOrderBy>>2]|0;
  $270 = HEAP32[$2>>2]|0;
  $271 = ((($270)) + 44|0);
  HEAP32[$271>>2] = $269;
  $272 = HEAP32[$pSrc>>2]|0;
  $273 = HEAP32[$2>>2]|0;
  $274 = ((($273)) + 28|0);
  HEAP32[$274>>2] = $272;
  $275 = HEAP32[$2>>2]|0;
  $276 = ((($275)) + 4|0);
  HEAP8[$276>>0] = 116;
  $277 = HEAP32[$pNew>>2]|0;
  $278 = ($277|0)==(0|0);
  $279 = HEAP32[$pPrior>>2]|0;
  if ($278) {
   $280 = HEAP32[$2>>2]|0;
   $281 = ((($280)) + 48|0);
   HEAP32[$281>>2] = $279;
  } else {
   $282 = HEAP32[$pNew>>2]|0;
   $283 = ((($282)) + 48|0);
   HEAP32[$283>>2] = $279;
   $284 = HEAP32[$pPrior>>2]|0;
   $285 = ($284|0)!=(0|0);
   if ($285) {
    $286 = HEAP32[$pNew>>2]|0;
    $287 = HEAP32[$pPrior>>2]|0;
    $288 = ((($287)) + 52|0);
    HEAP32[$288>>2] = $286;
   }
   $289 = HEAP32[$2>>2]|0;
   $290 = HEAP32[$pNew>>2]|0;
   $291 = ((($290)) + 52|0);
   HEAP32[$291>>2] = $289;
   $292 = HEAP32[$pNew>>2]|0;
   $293 = HEAP32[$2>>2]|0;
   $294 = ((($293)) + 48|0);
   HEAP32[$294>>2] = $292;
  }
  $295 = HEAP32[$db>>2]|0;
  $296 = ((($295)) + 69|0);
  $297 = HEAP8[$296>>0]|0;
  $298 = ($297<<24>>24)!=(0);
  if ($298) {
   label = 78;
   break;
  }
  $299 = HEAP32[$pSub>>2]|0;
  $300 = ((($299)) + 48|0);
  $301 = HEAP32[$300>>2]|0;
  HEAP32[$pSub>>2] = $301;
 }
 if ((label|0) == 78) {
  HEAP32[$0>>2] = 1;
  $561 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($561|0);
 }
 $302 = HEAP32[$pSubitem>>2]|0;
 $303 = ((($302)) + 20|0);
 $304 = HEAP32[$303>>2]|0;
 HEAP32[$pSub1>>2] = $304;
 HEAP32[$pSub>>2] = $304;
 $305 = HEAP32[$db>>2]|0;
 $306 = HEAP32[$pSubitem>>2]|0;
 $307 = ((($306)) + 4|0);
 $308 = HEAP32[$307>>2]|0;
 _sqlite3DbFree($305,$308);
 $309 = HEAP32[$db>>2]|0;
 $310 = HEAP32[$pSubitem>>2]|0;
 $311 = ((($310)) + 8|0);
 $312 = HEAP32[$311>>2]|0;
 _sqlite3DbFree($309,$312);
 $313 = HEAP32[$db>>2]|0;
 $314 = HEAP32[$pSubitem>>2]|0;
 $315 = ((($314)) + 12|0);
 $316 = HEAP32[$315>>2]|0;
 _sqlite3DbFree($313,$316);
 $317 = HEAP32[$pSubitem>>2]|0;
 $318 = ((($317)) + 4|0);
 HEAP32[$318>>2] = 0;
 $319 = HEAP32[$pSubitem>>2]|0;
 $320 = ((($319)) + 8|0);
 HEAP32[$320>>2] = 0;
 $321 = HEAP32[$pSubitem>>2]|0;
 $322 = ((($321)) + 12|0);
 HEAP32[$322>>2] = 0;
 $323 = HEAP32[$pSubitem>>2]|0;
 $324 = ((($323)) + 20|0);
 HEAP32[$324>>2] = 0;
 $325 = HEAP32[$pSubitem>>2]|0;
 $326 = ((($325)) + 16|0);
 $327 = HEAP32[$326>>2]|0;
 $328 = ($327|0)!=(0|0);
 if ($328) {
  $329 = HEAP32[$pSubitem>>2]|0;
  $330 = ((($329)) + 16|0);
  $331 = HEAP32[$330>>2]|0;
  HEAP32[$pTabToDel>>2] = $331;
  $332 = HEAP32[$pTabToDel>>2]|0;
  $333 = ((($332)) + 36|0);
  $334 = HEAP16[$333>>1]|0;
  $335 = $334&65535;
  $336 = ($335|0)==(1);
  if ($336) {
   $337 = HEAP32[$1>>2]|0;
   $338 = ((($337)) + 124|0);
   $339 = HEAP32[$338>>2]|0;
   $340 = ($339|0)!=(0|0);
   $341 = HEAP32[$1>>2]|0;
   if ($340) {
    $342 = ((($341)) + 124|0);
    $343 = HEAP32[$342>>2]|0;
    $344 = $343;
   } else {
    $344 = $341;
   }
   HEAP32[$pToplevel>>2] = $344;
   $345 = HEAP32[$pToplevel>>2]|0;
   $346 = ((($345)) + 464|0);
   $347 = HEAP32[$346>>2]|0;
   $348 = HEAP32[$pTabToDel>>2]|0;
   $349 = ((($348)) + 68|0);
   HEAP32[$349>>2] = $347;
   $350 = HEAP32[$pTabToDel>>2]|0;
   $351 = HEAP32[$pToplevel>>2]|0;
   $352 = ((($351)) + 464|0);
   HEAP32[$352>>2] = $350;
  } else {
   $353 = HEAP32[$pTabToDel>>2]|0;
   $354 = ((($353)) + 36|0);
   $355 = HEAP16[$354>>1]|0;
   $356 = (($355) + -1)<<16>>16;
   HEAP16[$354>>1] = $356;
  }
  $357 = HEAP32[$pSubitem>>2]|0;
  $358 = ((($357)) + 16|0);
  HEAP32[$358>>2] = 0;
 }
 $359 = HEAP32[$2>>2]|0;
 HEAP32[$pParent>>2] = $359;
 while(1) {
  $360 = HEAP32[$pParent>>2]|0;
  $361 = ($360|0)!=(0|0);
  if (!($361)) {
   break;
  }
  HEAP8[$jointype>>0] = 0;
  $362 = HEAP32[$pSub>>2]|0;
  $363 = ((($362)) + 28|0);
  $364 = HEAP32[$363>>2]|0;
  HEAP32[$pSubSrc>>2] = $364;
  $365 = HEAP32[$pSubSrc>>2]|0;
  $366 = HEAP32[$365>>2]|0;
  HEAP32[$nSubSrc>>2] = $366;
  $367 = HEAP32[$pParent>>2]|0;
  $368 = ((($367)) + 28|0);
  $369 = HEAP32[$368>>2]|0;
  HEAP32[$pSrc>>2] = $369;
  $370 = HEAP32[$pSrc>>2]|0;
  $371 = ($370|0)!=(0|0);
  if ($371) {
   $372 = HEAP32[$pSubitem>>2]|0;
   $373 = ((($372)) + 36|0);
   $374 = HEAP8[$373>>0]|0;
   HEAP8[$jointype>>0] = $374;
  } else {
   $375 = HEAP32[$db>>2]|0;
   $376 = (_sqlite3SrcListAppend($375,0,0,0)|0);
   $377 = HEAP32[$pParent>>2]|0;
   $378 = ((($377)) + 28|0);
   HEAP32[$378>>2] = $376;
   HEAP32[$pSrc>>2] = $376;
   $379 = HEAP32[$pSrc>>2]|0;
   $380 = ($379|0)==(0|0);
   if ($380) {
    break;
   }
  }
  $381 = HEAP32[$nSubSrc>>2]|0;
  $382 = ($381|0)>(1);
  if ($382) {
   $383 = HEAP32[$db>>2]|0;
   $384 = HEAP32[$pSrc>>2]|0;
   $385 = HEAP32[$nSubSrc>>2]|0;
   $386 = (($385) - 1)|0;
   $387 = HEAP32[$3>>2]|0;
   $388 = (($387) + 1)|0;
   $389 = (_sqlite3SrcListEnlarge($383,$384,$386,$388)|0);
   HEAP32[$pSrc>>2] = $389;
   $390 = HEAP32[$pParent>>2]|0;
   $391 = ((($390)) + 28|0);
   HEAP32[$391>>2] = $389;
   $392 = HEAP32[$db>>2]|0;
   $393 = ((($392)) + 69|0);
   $394 = HEAP8[$393>>0]|0;
   $395 = ($394<<24>>24)!=(0);
   if ($395) {
    break;
   }
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $396 = HEAP32[$i>>2]|0;
   $397 = HEAP32[$nSubSrc>>2]|0;
   $398 = ($396|0)<($397|0);
   if (!($398)) {
    break;
   }
   $399 = HEAP32[$db>>2]|0;
   $400 = HEAP32[$i>>2]|0;
   $401 = HEAP32[$3>>2]|0;
   $402 = (($400) + ($401))|0;
   $403 = HEAP32[$pSrc>>2]|0;
   $404 = ((($403)) + 8|0);
   $405 = (($404) + (($402*72)|0)|0);
   $406 = ((($405)) + 52|0);
   $407 = HEAP32[$406>>2]|0;
   _sqlite3IdListDelete($399,$407);
   $408 = HEAP32[$i>>2]|0;
   $409 = HEAP32[$3>>2]|0;
   $410 = (($408) + ($409))|0;
   $411 = HEAP32[$pSrc>>2]|0;
   $412 = ((($411)) + 8|0);
   $413 = (($412) + (($410*72)|0)|0);
   $414 = HEAP32[$i>>2]|0;
   $415 = HEAP32[$pSubSrc>>2]|0;
   $416 = ((($415)) + 8|0);
   $417 = (($416) + (($414*72)|0)|0);
   dest=$413; src=$417; stop=dest+72|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
   $418 = HEAP32[$i>>2]|0;
   $419 = HEAP32[$pSubSrc>>2]|0;
   $420 = ((($419)) + 8|0);
   $421 = (($420) + (($418*72)|0)|0);
   dest=$421; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $422 = HEAP32[$i>>2]|0;
   $423 = (($422) + 1)|0;
   HEAP32[$i>>2] = $423;
  }
  $424 = HEAP8[$jointype>>0]|0;
  $425 = HEAP32[$3>>2]|0;
  $426 = HEAP32[$pSrc>>2]|0;
  $427 = ((($426)) + 8|0);
  $428 = (($427) + (($425*72)|0)|0);
  $429 = ((($428)) + 36|0);
  HEAP8[$429>>0] = $424;
  $430 = HEAP32[$pParent>>2]|0;
  $431 = HEAP32[$430>>2]|0;
  HEAP32[$pList>>2] = $431;
  HEAP32[$i>>2] = 0;
  while(1) {
   $432 = HEAP32[$i>>2]|0;
   $433 = HEAP32[$pList>>2]|0;
   $434 = HEAP32[$433>>2]|0;
   $435 = ($432|0)<($434|0);
   if (!($435)) {
    break;
   }
   $436 = HEAP32[$i>>2]|0;
   $437 = HEAP32[$pList>>2]|0;
   $438 = ((($437)) + 4|0);
   $439 = HEAP32[$438>>2]|0;
   $440 = (($439) + (($436*20)|0)|0);
   $441 = ((($440)) + 4|0);
   $442 = HEAP32[$441>>2]|0;
   $443 = ($442|0)==(0|0);
   if ($443) {
    $444 = HEAP32[$db>>2]|0;
    $445 = HEAP32[$i>>2]|0;
    $446 = HEAP32[$pList>>2]|0;
    $447 = ((($446)) + 4|0);
    $448 = HEAP32[$447>>2]|0;
    $449 = (($448) + (($445*20)|0)|0);
    $450 = ((($449)) + 8|0);
    $451 = HEAP32[$450>>2]|0;
    $452 = (_sqlite3DbStrDup($444,$451)|0);
    HEAP32[$zName>>2] = $452;
    $453 = HEAP32[$zName>>2]|0;
    _sqlite3Dequote($453);
    $454 = HEAP32[$zName>>2]|0;
    $455 = HEAP32[$i>>2]|0;
    $456 = HEAP32[$pList>>2]|0;
    $457 = ((($456)) + 4|0);
    $458 = HEAP32[$457>>2]|0;
    $459 = (($458) + (($455*20)|0)|0);
    $460 = ((($459)) + 4|0);
    HEAP32[$460>>2] = $454;
   }
   $461 = HEAP32[$i>>2]|0;
   $462 = (($461) + 1)|0;
   HEAP32[$i>>2] = $462;
  }
  $463 = HEAP32[$pSub>>2]|0;
  $464 = ((($463)) + 44|0);
  $465 = HEAP32[$464>>2]|0;
  $466 = ($465|0)!=(0|0);
  if ($466) {
   $467 = HEAP32[$pSub>>2]|0;
   $468 = ((($467)) + 44|0);
   $469 = HEAP32[$468>>2]|0;
   HEAP32[$pOrderBy1>>2] = $469;
   HEAP32[$i>>2] = 0;
   while(1) {
    $470 = HEAP32[$i>>2]|0;
    $471 = HEAP32[$pOrderBy1>>2]|0;
    $472 = HEAP32[$471>>2]|0;
    $473 = ($470|0)<($472|0);
    if (!($473)) {
     break;
    }
    $474 = HEAP32[$i>>2]|0;
    $475 = HEAP32[$pOrderBy1>>2]|0;
    $476 = ((($475)) + 4|0);
    $477 = HEAP32[$476>>2]|0;
    $478 = (($477) + (($474*20)|0)|0);
    $479 = ((($478)) + 16|0);
    HEAP16[$479>>1] = 0;
    $480 = HEAP32[$i>>2]|0;
    $481 = (($480) + 1)|0;
    HEAP32[$i>>2] = $481;
   }
   $482 = HEAP32[$pOrderBy1>>2]|0;
   $483 = HEAP32[$pParent>>2]|0;
   $484 = ((($483)) + 44|0);
   HEAP32[$484>>2] = $482;
   $485 = HEAP32[$pSub>>2]|0;
   $486 = ((($485)) + 44|0);
   HEAP32[$486>>2] = 0;
  }
  $487 = HEAP32[$db>>2]|0;
  $488 = HEAP32[$pSub>>2]|0;
  $489 = ((($488)) + 32|0);
  $490 = HEAP32[$489>>2]|0;
  $491 = (_sqlite3ExprDup($487,$490,0)|0);
  HEAP32[$pWhere>>2] = $491;
  $492 = HEAP32[$5>>2]|0;
  $493 = ($492|0)!=(0);
  if ($493) {
   $494 = HEAP32[$pParent>>2]|0;
   $495 = ((($494)) + 32|0);
   $496 = HEAP32[$495>>2]|0;
   $497 = HEAP32[$pParent>>2]|0;
   $498 = ((($497)) + 40|0);
   HEAP32[$498>>2] = $496;
   $499 = HEAP32[$pWhere>>2]|0;
   $500 = HEAP32[$pParent>>2]|0;
   $501 = ((($500)) + 32|0);
   HEAP32[$501>>2] = $499;
   $502 = HEAP32[$db>>2]|0;
   $503 = HEAP32[$db>>2]|0;
   $504 = HEAP32[$pSub>>2]|0;
   $505 = ((($504)) + 40|0);
   $506 = HEAP32[$505>>2]|0;
   $507 = (_sqlite3ExprDup($503,$506,0)|0);
   $508 = HEAP32[$pParent>>2]|0;
   $509 = ((($508)) + 40|0);
   $510 = HEAP32[$509>>2]|0;
   $511 = (_sqlite3ExprAnd($502,$507,$510)|0);
   $512 = HEAP32[$pParent>>2]|0;
   $513 = ((($512)) + 40|0);
   HEAP32[$513>>2] = $511;
   $514 = HEAP32[$db>>2]|0;
   $515 = HEAP32[$pSub>>2]|0;
   $516 = ((($515)) + 36|0);
   $517 = HEAP32[$516>>2]|0;
   $518 = (_sqlite3ExprListDup($514,$517,0)|0);
   $519 = HEAP32[$pParent>>2]|0;
   $520 = ((($519)) + 36|0);
   HEAP32[$520>>2] = $518;
  } else {
   $521 = HEAP32[$db>>2]|0;
   $522 = HEAP32[$pWhere>>2]|0;
   $523 = HEAP32[$pParent>>2]|0;
   $524 = ((($523)) + 32|0);
   $525 = HEAP32[$524>>2]|0;
   $526 = (_sqlite3ExprAnd($521,$522,$525)|0);
   $527 = HEAP32[$pParent>>2]|0;
   $528 = ((($527)) + 32|0);
   HEAP32[$528>>2] = $526;
  }
  $529 = HEAP32[$db>>2]|0;
  $530 = HEAP32[$pParent>>2]|0;
  $531 = HEAP32[$iParent>>2]|0;
  $532 = HEAP32[$pSub>>2]|0;
  $533 = HEAP32[$532>>2]|0;
  _substSelect($529,$530,$531,$533,0);
  $534 = HEAP32[$pSub>>2]|0;
  $535 = ((($534)) + 8|0);
  $536 = HEAP32[$535>>2]|0;
  $537 = $536 & 1;
  $538 = HEAP32[$pParent>>2]|0;
  $539 = ((($538)) + 8|0);
  $540 = HEAP32[$539>>2]|0;
  $541 = $540 | $537;
  HEAP32[$539>>2] = $541;
  $542 = HEAP32[$pSub>>2]|0;
  $543 = ((($542)) + 56|0);
  $544 = HEAP32[$543>>2]|0;
  $545 = ($544|0)!=(0|0);
  if ($545) {
   $546 = HEAP32[$pSub>>2]|0;
   $547 = ((($546)) + 56|0);
   $548 = HEAP32[$547>>2]|0;
   $549 = HEAP32[$pParent>>2]|0;
   $550 = ((($549)) + 56|0);
   HEAP32[$550>>2] = $548;
   $551 = HEAP32[$pSub>>2]|0;
   $552 = ((($551)) + 56|0);
   HEAP32[$552>>2] = 0;
  }
  $553 = HEAP32[$pParent>>2]|0;
  $554 = ((($553)) + 48|0);
  $555 = HEAP32[$554>>2]|0;
  HEAP32[$pParent>>2] = $555;
  $556 = HEAP32[$pSub>>2]|0;
  $557 = ((($556)) + 48|0);
  $558 = HEAP32[$557>>2]|0;
  HEAP32[$pSub>>2] = $558;
 }
 $559 = HEAP32[$db>>2]|0;
 $560 = HEAP32[$pSub1>>2]|0;
 _sqlite3SelectDelete($559,$560);
 HEAP32[$0>>2] = 1;
 $561 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($561|0);
}
function _multiSelect($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0;
 var $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0;
 var $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0;
 var $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0;
 var $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0;
 var $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addr = 0, $addr1 = 0, $addr7 = 0, $addr9 = 0, $apColl = 0, $db = 0, $dest = 0, $i = 0, $iBreak = 0, $iBreak3 = 0, $iCont = 0, $iCont2 = 0, $iStart = 0, $iStart4 = 0, $iSub1 = 0, $iSub2 = 0, $intersectdest = 0, $nCol = 0, $nLimit = 0;
 var $op = 0, $or$cond = 0, $pDelete = 0, $pFirst = 0, $pFirst8 = 0, $pKeyInfo = 0, $pLimit = 0, $pLimit5 = 0, $pLoop = 0, $pOffset = 0, $pOffset6 = 0, $pPrior = 0, $priorOp = 0, $r1 = 0, $rc = 0, $tab1 = 0, $tab2 = 0, $unionTab = 0, $uniondest = 0, $v = 0;
 var $vararg_buffer = 0, $vararg_buffer2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 240|0;
 $1 = sp + 236|0;
 $2 = sp + 232|0;
 $3 = sp + 228|0;
 $rc = sp + 224|0;
 $pPrior = sp + 220|0;
 $v = sp + 216|0;
 $dest = sp + 192|0;
 $pDelete = sp + 184|0;
 $db = sp + 180|0;
 $iSub1 = sp + 176|0;
 $iSub2 = sp + 172|0;
 $addr = sp + 168|0;
 $nLimit = sp + 164|0;
 $unionTab = sp + 160|0;
 $op = sp + 244|0;
 $priorOp = sp + 156|0;
 $pLimit = sp + 152|0;
 $pOffset = sp + 148|0;
 $addr1 = sp + 144|0;
 $uniondest = sp + 120|0;
 $iCont = sp + 116|0;
 $iBreak = sp + 112|0;
 $iStart = sp + 108|0;
 $pFirst = sp + 104|0;
 $tab1 = sp + 100|0;
 $tab2 = sp + 96|0;
 $iCont2 = sp + 92|0;
 $iBreak3 = sp + 88|0;
 $iStart4 = sp + 84|0;
 $pLimit5 = sp + 80|0;
 $pOffset6 = sp + 76|0;
 $addr7 = sp + 72|0;
 $intersectdest = sp + 48|0;
 $r1 = sp + 40|0;
 $pFirst8 = sp + 36|0;
 $i = sp + 32|0;
 $pKeyInfo = sp + 28|0;
 $pLoop = sp + 24|0;
 $apColl = sp + 20|0;
 $nCol = sp + 16|0;
 $addr9 = sp + 12|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDelete>>2] = 0;
 HEAP32[$iSub1>>2] = 0;
 HEAP32[$iSub2>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 48|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pPrior>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 ;HEAP32[$dest>>2]=HEAP32[$9>>2]|0;HEAP32[$dest+4>>2]=HEAP32[$9+4>>2]|0;HEAP32[$dest+8>>2]=HEAP32[$9+8>>2]|0;HEAP32[$dest+12>>2]=HEAP32[$9+12>>2]|0;HEAP32[$dest+16>>2]=HEAP32[$9+16>>2]|0;HEAP32[$dest+20>>2]=HEAP32[$9+20>>2]|0;
 $10 = HEAP32[$pPrior>>2]|0;
 $11 = ((($10)) + 44|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)!=(0|0);
 L1: do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   $16 = ((($15)) + 4|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17&255;
   $19 = (_selectOpName($18)|0);
   HEAP32[$vararg_buffer>>2] = $19;
   _sqlite3ErrorMsg($14,35831,$vararg_buffer);
   HEAP32[$rc>>2] = 1;
  } else {
   $20 = HEAP32[$pPrior>>2]|0;
   $21 = ((($20)) + 56|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = ($22|0)!=(0|0);
   $24 = HEAP32[$1>>2]|0;
   if ($23) {
    $25 = HEAP32[$2>>2]|0;
    $26 = ((($25)) + 4|0);
    $27 = HEAP8[$26>>0]|0;
    $28 = $27&255;
    $29 = (_selectOpName($28)|0);
    HEAP32[$vararg_buffer2>>2] = $29;
    _sqlite3ErrorMsg($24,35879,$vararg_buffer2);
    HEAP32[$rc>>2] = 1;
    break;
   }
   $30 = (_sqlite3GetVdbe($24)|0);
   HEAP32[$v>>2] = $30;
   $31 = HEAP8[$dest>>0]|0;
   $32 = $31&255;
   $33 = ($32|0)==(12);
   if ($33) {
    $34 = HEAP32[$v>>2]|0;
    $35 = ((($dest)) + 8|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = HEAP32[$2>>2]|0;
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    (_sqlite3VdbeAddOp2($34,107,$36,$39)|0);
    HEAP8[$dest>>0] = 14;
   }
   $40 = HEAP32[$2>>2]|0;
   $41 = ((($40)) + 8|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = $42 & 1024;
   $44 = ($43|0)!=(0);
   if ($44) {
    $45 = HEAP32[$1>>2]|0;
    $46 = HEAP32[$2>>2]|0;
    $47 = (_multiSelectValues($45,$46,$dest)|0);
    HEAP32[$rc>>2] = $47;
    break;
   }
   $48 = HEAP32[$2>>2]|0;
   $49 = ((($48)) + 8|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = $50 & 8192;
   $52 = ($51|0)!=(0);
   L13: do {
    if ($52) {
     $53 = HEAP32[$1>>2]|0;
     $54 = HEAP32[$2>>2]|0;
     _generateWithRecursiveQuery($53,$54,$dest);
    } else {
     $55 = HEAP32[$2>>2]|0;
     $56 = ((($55)) + 44|0);
     $57 = HEAP32[$56>>2]|0;
     $58 = ($57|0)!=(0|0);
     if ($58) {
      $59 = HEAP32[$1>>2]|0;
      $60 = HEAP32[$2>>2]|0;
      $61 = HEAP32[$3>>2]|0;
      $62 = (_multiSelectOrderBy($59,$60,$61)|0);
      HEAP32[$0>>2] = $62;
      $560 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($560|0);
     }
     $63 = HEAP32[$2>>2]|0;
     $64 = ((($63)) + 4|0);
     $65 = HEAP8[$64>>0]|0;
     $66 = $65&255;
     switch ($66|0) {
     case 116:  {
      HEAP32[$addr>>2] = 0;
      $67 = HEAP32[$2>>2]|0;
      $68 = ((($67)) + 12|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = HEAP32[$pPrior>>2]|0;
      $71 = ((($70)) + 12|0);
      HEAP32[$71>>2] = $69;
      $72 = HEAP32[$2>>2]|0;
      $73 = ((($72)) + 16|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = HEAP32[$pPrior>>2]|0;
      $76 = ((($75)) + 16|0);
      HEAP32[$76>>2] = $74;
      $77 = HEAP32[$2>>2]|0;
      $78 = ((($77)) + 56|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = HEAP32[$pPrior>>2]|0;
      $81 = ((($80)) + 56|0);
      HEAP32[$81>>2] = $79;
      $82 = HEAP32[$2>>2]|0;
      $83 = ((($82)) + 60|0);
      $84 = HEAP32[$83>>2]|0;
      $85 = HEAP32[$pPrior>>2]|0;
      $86 = ((($85)) + 60|0);
      HEAP32[$86>>2] = $84;
      $87 = HEAP32[$1>>2]|0;
      $88 = ((($87)) + 424|0);
      $89 = HEAP32[$88>>2]|0;
      HEAP32[$iSub1>>2] = $89;
      $90 = HEAP32[$1>>2]|0;
      $91 = HEAP32[$pPrior>>2]|0;
      $92 = (_sqlite3Select($90,$91,$dest)|0);
      HEAP32[$rc>>2] = $92;
      $93 = HEAP32[$2>>2]|0;
      $94 = ((($93)) + 56|0);
      HEAP32[$94>>2] = 0;
      $95 = HEAP32[$2>>2]|0;
      $96 = ((($95)) + 60|0);
      HEAP32[$96>>2] = 0;
      $97 = HEAP32[$rc>>2]|0;
      $98 = ($97|0)!=(0);
      if ($98) {
       break L1;
      }
      $99 = HEAP32[$2>>2]|0;
      $100 = ((($99)) + 48|0);
      HEAP32[$100>>2] = 0;
      $101 = HEAP32[$pPrior>>2]|0;
      $102 = ((($101)) + 12|0);
      $103 = HEAP32[$102>>2]|0;
      $104 = HEAP32[$2>>2]|0;
      $105 = ((($104)) + 12|0);
      HEAP32[$105>>2] = $103;
      $106 = HEAP32[$pPrior>>2]|0;
      $107 = ((($106)) + 16|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = HEAP32[$2>>2]|0;
      $110 = ((($109)) + 16|0);
      HEAP32[$110>>2] = $108;
      $111 = HEAP32[$2>>2]|0;
      $112 = ((($111)) + 12|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = ($113|0)!=(0);
      if ($114) {
       $115 = HEAP32[$v>>2]|0;
       $116 = HEAP32[$2>>2]|0;
       $117 = ((($116)) + 12|0);
       $118 = HEAP32[$117>>2]|0;
       $119 = (_sqlite3VdbeAddOp1($115,22,$118)|0);
       HEAP32[$addr>>2] = $119;
       $120 = HEAP32[$2>>2]|0;
       $121 = ((($120)) + 16|0);
       $122 = HEAP32[$121>>2]|0;
       $123 = ($122|0)!=(0);
       if ($123) {
        $124 = HEAP32[$v>>2]|0;
        $125 = HEAP32[$2>>2]|0;
        $126 = ((($125)) + 12|0);
        $127 = HEAP32[$126>>2]|0;
        $128 = HEAP32[$2>>2]|0;
        $129 = ((($128)) + 16|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = (($130) + 1)|0;
        $132 = HEAP32[$2>>2]|0;
        $133 = ((($132)) + 16|0);
        $134 = HEAP32[$133>>2]|0;
        (_sqlite3VdbeAddOp3($124,146,$127,$131,$134)|0);
       }
      }
      $135 = HEAP32[$1>>2]|0;
      $136 = ((($135)) + 424|0);
      $137 = HEAP32[$136>>2]|0;
      HEAP32[$iSub2>>2] = $137;
      $138 = HEAP32[$1>>2]|0;
      $139 = HEAP32[$2>>2]|0;
      $140 = (_sqlite3Select($138,$139,$dest)|0);
      HEAP32[$rc>>2] = $140;
      $141 = HEAP32[$2>>2]|0;
      $142 = ((($141)) + 48|0);
      $143 = HEAP32[$142>>2]|0;
      HEAP32[$pDelete>>2] = $143;
      $144 = HEAP32[$pPrior>>2]|0;
      $145 = HEAP32[$2>>2]|0;
      $146 = ((($145)) + 48|0);
      HEAP32[$146>>2] = $144;
      $147 = HEAP32[$2>>2]|0;
      $148 = ((($147)) + 6|0);
      $149 = HEAP16[$148>>1]|0;
      $150 = HEAP32[$pPrior>>2]|0;
      $151 = ((($150)) + 6|0);
      $152 = HEAP16[$151>>1]|0;
      $153 = (_sqlite3LogEstAdd($149,$152)|0);
      $154 = HEAP32[$2>>2]|0;
      $155 = ((($154)) + 6|0);
      HEAP16[$155>>1] = $153;
      $156 = HEAP32[$pPrior>>2]|0;
      $157 = ((($156)) + 56|0);
      $158 = HEAP32[$157>>2]|0;
      $159 = ($158|0)!=(0|0);
      if ($159) {
       $160 = HEAP32[$pPrior>>2]|0;
       $161 = ((($160)) + 56|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = (_sqlite3ExprIsInteger($162,$nLimit)|0);
       $164 = ($163|0)!=(0);
       $165 = HEAP32[$nLimit>>2]|0;
       $166 = ($165|0)>(0);
       $or$cond = $164 & $166;
       if ($or$cond) {
        $167 = HEAP32[$2>>2]|0;
        $168 = ((($167)) + 6|0);
        $169 = HEAP16[$168>>1]|0;
        $170 = $169 << 16 >> 16;
        $171 = HEAP32[$nLimit>>2]|0;
        $172 = ($171|0)<(0);
        $173 = $172 << 31 >> 31;
        $174 = (_sqlite3LogEst($171,$173)|0);
        $175 = $174 << 16 >> 16;
        $176 = ($170|0)>($175|0);
        if ($176) {
         $177 = HEAP32[$nLimit>>2]|0;
         $178 = ($177|0)<(0);
         $179 = $178 << 31 >> 31;
         $180 = (_sqlite3LogEst($177,$179)|0);
         $181 = HEAP32[$2>>2]|0;
         $182 = ((($181)) + 6|0);
         HEAP16[$182>>1] = $180;
        }
       }
      }
      $183 = HEAP32[$addr>>2]|0;
      $184 = ($183|0)!=(0);
      if (!($184)) {
       break L13;
      }
      $185 = HEAP32[$v>>2]|0;
      $186 = HEAP32[$addr>>2]|0;
      _sqlite3VdbeJumpHere($185,$186);
      break L13;
      break;
     }
     case 115: case 117:  {
      HEAP8[$op>>0] = 0;
      HEAP32[$priorOp>>2] = 1;
      $187 = HEAP8[$dest>>0]|0;
      $188 = $187&255;
      $189 = HEAP32[$priorOp>>2]|0;
      $190 = ($188|0)==($189|0);
      if ($190) {
       $191 = ((($dest)) + 8|0);
       $192 = HEAP32[$191>>2]|0;
       HEAP32[$unionTab>>2] = $192;
      } else {
       $193 = HEAP32[$1>>2]|0;
       $194 = ((($193)) + 40|0);
       $195 = HEAP32[$194>>2]|0;
       $196 = (($195) + 1)|0;
       HEAP32[$194>>2] = $196;
       HEAP32[$unionTab>>2] = $195;
       $197 = HEAP32[$v>>2]|0;
       $198 = HEAP32[$unionTab>>2]|0;
       $199 = (_sqlite3VdbeAddOp2($197,107,$198,0)|0);
       HEAP32[$addr1>>2] = $199;
       $200 = HEAP32[$addr1>>2]|0;
       $201 = HEAP32[$2>>2]|0;
       $202 = ((($201)) + 20|0);
       HEAP32[$202>>2] = $200;
       $203 = HEAP32[$2>>2]|0;
       $204 = (_findRightmost($203)|0);
       $205 = ((($204)) + 8|0);
       $206 = HEAP32[$205>>2]|0;
       $207 = $206 | 32;
       HEAP32[$205>>2] = $207;
      }
      $208 = HEAP32[$priorOp>>2]|0;
      $209 = HEAP32[$unionTab>>2]|0;
      _sqlite3SelectDestInit($uniondest,$208,$209);
      $210 = HEAP32[$1>>2]|0;
      $211 = ((($210)) + 424|0);
      $212 = HEAP32[$211>>2]|0;
      HEAP32[$iSub1>>2] = $212;
      $213 = HEAP32[$1>>2]|0;
      $214 = HEAP32[$pPrior>>2]|0;
      $215 = (_sqlite3Select($213,$214,$uniondest)|0);
      HEAP32[$rc>>2] = $215;
      $216 = HEAP32[$rc>>2]|0;
      $217 = ($216|0)!=(0);
      if ($217) {
       break L1;
      }
      $218 = HEAP32[$2>>2]|0;
      $219 = ((($218)) + 4|0);
      $220 = HEAP8[$219>>0]|0;
      $221 = $220&255;
      $222 = ($221|0)==(117);
      if ($222) {
       HEAP8[$op>>0] = 2;
      } else {
       HEAP8[$op>>0] = 1;
      }
      $223 = HEAP32[$2>>2]|0;
      $224 = ((($223)) + 48|0);
      HEAP32[$224>>2] = 0;
      $225 = HEAP32[$2>>2]|0;
      $226 = ((($225)) + 56|0);
      $227 = HEAP32[$226>>2]|0;
      HEAP32[$pLimit>>2] = $227;
      $228 = HEAP32[$2>>2]|0;
      $229 = ((($228)) + 56|0);
      HEAP32[$229>>2] = 0;
      $230 = HEAP32[$2>>2]|0;
      $231 = ((($230)) + 60|0);
      $232 = HEAP32[$231>>2]|0;
      HEAP32[$pOffset>>2] = $232;
      $233 = HEAP32[$2>>2]|0;
      $234 = ((($233)) + 60|0);
      HEAP32[$234>>2] = 0;
      $235 = HEAP8[$op>>0]|0;
      HEAP8[$uniondest>>0] = $235;
      $236 = HEAP32[$1>>2]|0;
      $237 = ((($236)) + 424|0);
      $238 = HEAP32[$237>>2]|0;
      HEAP32[$iSub2>>2] = $238;
      $239 = HEAP32[$1>>2]|0;
      $240 = HEAP32[$2>>2]|0;
      $241 = (_sqlite3Select($239,$240,$uniondest)|0);
      HEAP32[$rc>>2] = $241;
      $242 = HEAP32[$db>>2]|0;
      $243 = HEAP32[$2>>2]|0;
      $244 = ((($243)) + 44|0);
      $245 = HEAP32[$244>>2]|0;
      _sqlite3ExprListDelete($242,$245);
      $246 = HEAP32[$2>>2]|0;
      $247 = ((($246)) + 48|0);
      $248 = HEAP32[$247>>2]|0;
      HEAP32[$pDelete>>2] = $248;
      $249 = HEAP32[$pPrior>>2]|0;
      $250 = HEAP32[$2>>2]|0;
      $251 = ((($250)) + 48|0);
      HEAP32[$251>>2] = $249;
      $252 = HEAP32[$2>>2]|0;
      $253 = ((($252)) + 44|0);
      HEAP32[$253>>2] = 0;
      $254 = HEAP32[$2>>2]|0;
      $255 = ((($254)) + 4|0);
      $256 = HEAP8[$255>>0]|0;
      $257 = $256&255;
      $258 = ($257|0)==(115);
      if ($258) {
       $259 = HEAP32[$2>>2]|0;
       $260 = ((($259)) + 6|0);
       $261 = HEAP16[$260>>1]|0;
       $262 = HEAP32[$pPrior>>2]|0;
       $263 = ((($262)) + 6|0);
       $264 = HEAP16[$263>>1]|0;
       $265 = (_sqlite3LogEstAdd($261,$264)|0);
       $266 = HEAP32[$2>>2]|0;
       $267 = ((($266)) + 6|0);
       HEAP16[$267>>1] = $265;
      }
      $268 = HEAP32[$db>>2]|0;
      $269 = HEAP32[$2>>2]|0;
      $270 = ((($269)) + 56|0);
      $271 = HEAP32[$270>>2]|0;
      _sqlite3ExprDelete($268,$271);
      $272 = HEAP32[$pLimit>>2]|0;
      $273 = HEAP32[$2>>2]|0;
      $274 = ((($273)) + 56|0);
      HEAP32[$274>>2] = $272;
      $275 = HEAP32[$pOffset>>2]|0;
      $276 = HEAP32[$2>>2]|0;
      $277 = ((($276)) + 60|0);
      HEAP32[$277>>2] = $275;
      $278 = HEAP32[$2>>2]|0;
      $279 = ((($278)) + 12|0);
      HEAP32[$279>>2] = 0;
      $280 = HEAP32[$2>>2]|0;
      $281 = ((($280)) + 16|0);
      HEAP32[$281>>2] = 0;
      $282 = HEAP8[$dest>>0]|0;
      $283 = $282&255;
      $284 = HEAP32[$priorOp>>2]|0;
      $285 = ($283|0)!=($284|0);
      if (!($285)) {
       break L13;
      }
      $286 = HEAP8[$dest>>0]|0;
      $287 = $286&255;
      $288 = ($287|0)==(9);
      if ($288) {
       $289 = HEAP32[$2>>2]|0;
       HEAP32[$pFirst>>2] = $289;
       while(1) {
        $290 = HEAP32[$pFirst>>2]|0;
        $291 = ((($290)) + 48|0);
        $292 = HEAP32[$291>>2]|0;
        $293 = ($292|0)!=(0|0);
        if (!($293)) {
         break;
        }
        $294 = HEAP32[$pFirst>>2]|0;
        $295 = ((($294)) + 48|0);
        $296 = HEAP32[$295>>2]|0;
        HEAP32[$pFirst>>2] = $296;
       }
       $297 = HEAP32[$1>>2]|0;
       $298 = HEAP32[$pFirst>>2]|0;
       $299 = ((($298)) + 28|0);
       $300 = HEAP32[$299>>2]|0;
       $301 = HEAP32[$pFirst>>2]|0;
       $302 = HEAP32[$301>>2]|0;
       _generateColumnNames($297,$300,$302);
      }
      $303 = HEAP32[$v>>2]|0;
      $304 = (_sqlite3VdbeMakeLabel($303)|0);
      HEAP32[$iBreak>>2] = $304;
      $305 = HEAP32[$v>>2]|0;
      $306 = (_sqlite3VdbeMakeLabel($305)|0);
      HEAP32[$iCont>>2] = $306;
      $307 = HEAP32[$1>>2]|0;
      $308 = HEAP32[$2>>2]|0;
      $309 = HEAP32[$iBreak>>2]|0;
      _computeLimitRegisters($307,$308,$309);
      $310 = HEAP32[$v>>2]|0;
      $311 = HEAP32[$unionTab>>2]|0;
      $312 = HEAP32[$iBreak>>2]|0;
      (_sqlite3VdbeAddOp2($310,57,$311,$312)|0);
      $313 = HEAP32[$v>>2]|0;
      $314 = (_sqlite3VdbeCurrentAddr($313)|0);
      HEAP32[$iStart>>2] = $314;
      $315 = HEAP32[$1>>2]|0;
      $316 = HEAP32[$2>>2]|0;
      $317 = HEAP32[$2>>2]|0;
      $318 = HEAP32[$317>>2]|0;
      $319 = HEAP32[$unionTab>>2]|0;
      $320 = HEAP32[$iCont>>2]|0;
      $321 = HEAP32[$iBreak>>2]|0;
      _selectInnerLoop($315,$316,$318,$319,0,0,$dest,$320,$321);
      $322 = HEAP32[$v>>2]|0;
      $323 = HEAP32[$iCont>>2]|0;
      _sqlite3VdbeResolveLabel($322,$323);
      $324 = HEAP32[$v>>2]|0;
      $325 = HEAP32[$unionTab>>2]|0;
      $326 = HEAP32[$iStart>>2]|0;
      (_sqlite3VdbeAddOp2($324,7,$325,$326)|0);
      $327 = HEAP32[$v>>2]|0;
      $328 = HEAP32[$iBreak>>2]|0;
      _sqlite3VdbeResolveLabel($327,$328);
      $329 = HEAP32[$v>>2]|0;
      $330 = HEAP32[$unionTab>>2]|0;
      (_sqlite3VdbeAddOp2($329,111,$330,0)|0);
      break L13;
      break;
     }
     default: {
      $331 = HEAP32[$1>>2]|0;
      $332 = ((($331)) + 40|0);
      $333 = HEAP32[$332>>2]|0;
      $334 = (($333) + 1)|0;
      HEAP32[$332>>2] = $334;
      HEAP32[$tab1>>2] = $333;
      $335 = HEAP32[$1>>2]|0;
      $336 = ((($335)) + 40|0);
      $337 = HEAP32[$336>>2]|0;
      $338 = (($337) + 1)|0;
      HEAP32[$336>>2] = $338;
      HEAP32[$tab2>>2] = $337;
      $339 = HEAP32[$v>>2]|0;
      $340 = HEAP32[$tab1>>2]|0;
      $341 = (_sqlite3VdbeAddOp2($339,107,$340,0)|0);
      HEAP32[$addr7>>2] = $341;
      $342 = HEAP32[$addr7>>2]|0;
      $343 = HEAP32[$2>>2]|0;
      $344 = ((($343)) + 20|0);
      HEAP32[$344>>2] = $342;
      $345 = HEAP32[$2>>2]|0;
      $346 = (_findRightmost($345)|0);
      $347 = ((($346)) + 8|0);
      $348 = HEAP32[$347>>2]|0;
      $349 = $348 | 32;
      HEAP32[$347>>2] = $349;
      $350 = HEAP32[$tab1>>2]|0;
      _sqlite3SelectDestInit($intersectdest,1,$350);
      $351 = HEAP32[$1>>2]|0;
      $352 = ((($351)) + 424|0);
      $353 = HEAP32[$352>>2]|0;
      HEAP32[$iSub1>>2] = $353;
      $354 = HEAP32[$1>>2]|0;
      $355 = HEAP32[$pPrior>>2]|0;
      $356 = (_sqlite3Select($354,$355,$intersectdest)|0);
      HEAP32[$rc>>2] = $356;
      $357 = HEAP32[$rc>>2]|0;
      $358 = ($357|0)!=(0);
      if ($358) {
       break L1;
      }
      $359 = HEAP32[$v>>2]|0;
      $360 = HEAP32[$tab2>>2]|0;
      $361 = (_sqlite3VdbeAddOp2($359,107,$360,0)|0);
      HEAP32[$addr7>>2] = $361;
      $362 = HEAP32[$addr7>>2]|0;
      $363 = HEAP32[$2>>2]|0;
      $364 = ((($363)) + 20|0);
      $365 = ((($364)) + 4|0);
      HEAP32[$365>>2] = $362;
      $366 = HEAP32[$2>>2]|0;
      $367 = ((($366)) + 48|0);
      HEAP32[$367>>2] = 0;
      $368 = HEAP32[$2>>2]|0;
      $369 = ((($368)) + 56|0);
      $370 = HEAP32[$369>>2]|0;
      HEAP32[$pLimit5>>2] = $370;
      $371 = HEAP32[$2>>2]|0;
      $372 = ((($371)) + 56|0);
      HEAP32[$372>>2] = 0;
      $373 = HEAP32[$2>>2]|0;
      $374 = ((($373)) + 60|0);
      $375 = HEAP32[$374>>2]|0;
      HEAP32[$pOffset6>>2] = $375;
      $376 = HEAP32[$2>>2]|0;
      $377 = ((($376)) + 60|0);
      HEAP32[$377>>2] = 0;
      $378 = HEAP32[$tab2>>2]|0;
      $379 = ((($intersectdest)) + 8|0);
      HEAP32[$379>>2] = $378;
      $380 = HEAP32[$1>>2]|0;
      $381 = ((($380)) + 424|0);
      $382 = HEAP32[$381>>2]|0;
      HEAP32[$iSub2>>2] = $382;
      $383 = HEAP32[$1>>2]|0;
      $384 = HEAP32[$2>>2]|0;
      $385 = (_sqlite3Select($383,$384,$intersectdest)|0);
      HEAP32[$rc>>2] = $385;
      $386 = HEAP32[$2>>2]|0;
      $387 = ((($386)) + 48|0);
      $388 = HEAP32[$387>>2]|0;
      HEAP32[$pDelete>>2] = $388;
      $389 = HEAP32[$pPrior>>2]|0;
      $390 = HEAP32[$2>>2]|0;
      $391 = ((($390)) + 48|0);
      HEAP32[$391>>2] = $389;
      $392 = HEAP32[$2>>2]|0;
      $393 = ((($392)) + 6|0);
      $394 = HEAP16[$393>>1]|0;
      $395 = $394 << 16 >> 16;
      $396 = HEAP32[$pPrior>>2]|0;
      $397 = ((($396)) + 6|0);
      $398 = HEAP16[$397>>1]|0;
      $399 = $398 << 16 >> 16;
      $400 = ($395|0)>($399|0);
      if ($400) {
       $401 = HEAP32[$pPrior>>2]|0;
       $402 = ((($401)) + 6|0);
       $403 = HEAP16[$402>>1]|0;
       $404 = HEAP32[$2>>2]|0;
       $405 = ((($404)) + 6|0);
       HEAP16[$405>>1] = $403;
      }
      $406 = HEAP32[$db>>2]|0;
      $407 = HEAP32[$2>>2]|0;
      $408 = ((($407)) + 56|0);
      $409 = HEAP32[$408>>2]|0;
      _sqlite3ExprDelete($406,$409);
      $410 = HEAP32[$pLimit5>>2]|0;
      $411 = HEAP32[$2>>2]|0;
      $412 = ((($411)) + 56|0);
      HEAP32[$412>>2] = $410;
      $413 = HEAP32[$pOffset6>>2]|0;
      $414 = HEAP32[$2>>2]|0;
      $415 = ((($414)) + 60|0);
      HEAP32[$415>>2] = $413;
      $416 = HEAP8[$dest>>0]|0;
      $417 = $416&255;
      $418 = ($417|0)==(9);
      if ($418) {
       $419 = HEAP32[$2>>2]|0;
       HEAP32[$pFirst8>>2] = $419;
       while(1) {
        $420 = HEAP32[$pFirst8>>2]|0;
        $421 = ((($420)) + 48|0);
        $422 = HEAP32[$421>>2]|0;
        $423 = ($422|0)!=(0|0);
        if (!($423)) {
         break;
        }
        $424 = HEAP32[$pFirst8>>2]|0;
        $425 = ((($424)) + 48|0);
        $426 = HEAP32[$425>>2]|0;
        HEAP32[$pFirst8>>2] = $426;
       }
       $427 = HEAP32[$1>>2]|0;
       $428 = HEAP32[$pFirst8>>2]|0;
       $429 = ((($428)) + 28|0);
       $430 = HEAP32[$429>>2]|0;
       $431 = HEAP32[$pFirst8>>2]|0;
       $432 = HEAP32[$431>>2]|0;
       _generateColumnNames($427,$430,$432);
      }
      $433 = HEAP32[$v>>2]|0;
      $434 = (_sqlite3VdbeMakeLabel($433)|0);
      HEAP32[$iBreak3>>2] = $434;
      $435 = HEAP32[$v>>2]|0;
      $436 = (_sqlite3VdbeMakeLabel($435)|0);
      HEAP32[$iCont2>>2] = $436;
      $437 = HEAP32[$1>>2]|0;
      $438 = HEAP32[$2>>2]|0;
      $439 = HEAP32[$iBreak3>>2]|0;
      _computeLimitRegisters($437,$438,$439);
      $440 = HEAP32[$v>>2]|0;
      $441 = HEAP32[$tab1>>2]|0;
      $442 = HEAP32[$iBreak3>>2]|0;
      (_sqlite3VdbeAddOp2($440,57,$441,$442)|0);
      $443 = HEAP32[$1>>2]|0;
      $444 = (_sqlite3GetTempReg($443)|0);
      HEAP32[$r1>>2] = $444;
      $445 = HEAP32[$v>>2]|0;
      $446 = HEAP32[$tab1>>2]|0;
      $447 = HEAP32[$r1>>2]|0;
      $448 = (_sqlite3VdbeAddOp2($445,121,$446,$447)|0);
      HEAP32[$iStart4>>2] = $448;
      $449 = HEAP32[$v>>2]|0;
      $450 = HEAP32[$tab2>>2]|0;
      $451 = HEAP32[$iCont2>>2]|0;
      $452 = HEAP32[$r1>>2]|0;
      (_sqlite3VdbeAddOp4Int($449,30,$450,$451,$452,0)|0);
      $453 = HEAP32[$1>>2]|0;
      $454 = HEAP32[$r1>>2]|0;
      _sqlite3ReleaseTempReg($453,$454);
      $455 = HEAP32[$1>>2]|0;
      $456 = HEAP32[$2>>2]|0;
      $457 = HEAP32[$2>>2]|0;
      $458 = HEAP32[$457>>2]|0;
      $459 = HEAP32[$tab1>>2]|0;
      $460 = HEAP32[$iCont2>>2]|0;
      $461 = HEAP32[$iBreak3>>2]|0;
      _selectInnerLoop($455,$456,$458,$459,0,0,$dest,$460,$461);
      $462 = HEAP32[$v>>2]|0;
      $463 = HEAP32[$iCont2>>2]|0;
      _sqlite3VdbeResolveLabel($462,$463);
      $464 = HEAP32[$v>>2]|0;
      $465 = HEAP32[$tab1>>2]|0;
      $466 = HEAP32[$iStart4>>2]|0;
      (_sqlite3VdbeAddOp2($464,7,$465,$466)|0);
      $467 = HEAP32[$v>>2]|0;
      $468 = HEAP32[$iBreak3>>2]|0;
      _sqlite3VdbeResolveLabel($467,$468);
      $469 = HEAP32[$v>>2]|0;
      $470 = HEAP32[$tab2>>2]|0;
      (_sqlite3VdbeAddOp2($469,111,$470,0)|0);
      $471 = HEAP32[$v>>2]|0;
      $472 = HEAP32[$tab1>>2]|0;
      (_sqlite3VdbeAddOp2($471,111,$472,0)|0);
      break L13;
     }
     }
    }
   } while(0);
   $473 = HEAP32[$1>>2]|0;
   $474 = HEAP32[$2>>2]|0;
   $475 = ((($474)) + 4|0);
   $476 = HEAP8[$475>>0]|0;
   $477 = $476&255;
   $478 = HEAP32[$iSub1>>2]|0;
   $479 = HEAP32[$iSub2>>2]|0;
   $480 = HEAP32[$2>>2]|0;
   $481 = ((($480)) + 4|0);
   $482 = HEAP8[$481>>0]|0;
   $483 = $482&255;
   $484 = ($483|0)!=(116);
   $485 = $484&1;
   _explainComposite($473,$477,$478,$479,$485);
   $486 = HEAP32[$2>>2]|0;
   $487 = ((($486)) + 8|0);
   $488 = HEAP32[$487>>2]|0;
   $489 = $488 & 32;
   $490 = ($489|0)!=(0);
   if ($490) {
    $491 = HEAP32[$2>>2]|0;
    $492 = HEAP32[$491>>2]|0;
    $493 = HEAP32[$492>>2]|0;
    HEAP32[$nCol>>2] = $493;
    $494 = HEAP32[$db>>2]|0;
    $495 = HEAP32[$nCol>>2]|0;
    $496 = (_sqlite3KeyInfoAlloc($494,$495,1)|0);
    HEAP32[$pKeyInfo>>2] = $496;
    $497 = HEAP32[$pKeyInfo>>2]|0;
    $498 = ($497|0)!=(0|0);
    if (!($498)) {
     HEAP32[$rc>>2] = 7;
     break;
    }
    HEAP32[$i>>2] = 0;
    $499 = HEAP32[$pKeyInfo>>2]|0;
    $500 = ((($499)) + 20|0);
    HEAP32[$apColl>>2] = $500;
    while(1) {
     $501 = HEAP32[$i>>2]|0;
     $502 = HEAP32[$nCol>>2]|0;
     $503 = ($501|0)<($502|0);
     if (!($503)) {
      break;
     }
     $504 = HEAP32[$1>>2]|0;
     $505 = HEAP32[$2>>2]|0;
     $506 = HEAP32[$i>>2]|0;
     $507 = (_multiSelectCollSeq($504,$505,$506)|0);
     $508 = HEAP32[$apColl>>2]|0;
     HEAP32[$508>>2] = $507;
     $509 = HEAP32[$apColl>>2]|0;
     $510 = HEAP32[$509>>2]|0;
     $511 = (0|0)==($510|0);
     if ($511) {
      $512 = HEAP32[$db>>2]|0;
      $513 = ((($512)) + 8|0);
      $514 = HEAP32[$513>>2]|0;
      $515 = HEAP32[$apColl>>2]|0;
      HEAP32[$515>>2] = $514;
     }
     $516 = HEAP32[$i>>2]|0;
     $517 = (($516) + 1)|0;
     HEAP32[$i>>2] = $517;
     $518 = HEAP32[$apColl>>2]|0;
     $519 = ((($518)) + 4|0);
     HEAP32[$apColl>>2] = $519;
    }
    $520 = HEAP32[$2>>2]|0;
    HEAP32[$pLoop>>2] = $520;
    while(1) {
     $521 = HEAP32[$pLoop>>2]|0;
     $522 = ($521|0)!=(0|0);
     if (!($522)) {
      break;
     }
     HEAP32[$i>>2] = 0;
     while(1) {
      $523 = HEAP32[$i>>2]|0;
      $524 = ($523|0)<(2);
      if (!($524)) {
       break;
      }
      $525 = HEAP32[$i>>2]|0;
      $526 = HEAP32[$pLoop>>2]|0;
      $527 = ((($526)) + 20|0);
      $528 = (($527) + ($525<<2)|0);
      $529 = HEAP32[$528>>2]|0;
      HEAP32[$addr9>>2] = $529;
      $530 = HEAP32[$addr9>>2]|0;
      $531 = ($530|0)<(0);
      if ($531) {
       break;
      }
      $532 = HEAP32[$v>>2]|0;
      $533 = HEAP32[$addr9>>2]|0;
      $534 = HEAP32[$nCol>>2]|0;
      _sqlite3VdbeChangeP2($532,$533,$534);
      $535 = HEAP32[$v>>2]|0;
      $536 = HEAP32[$addr9>>2]|0;
      $537 = HEAP32[$pKeyInfo>>2]|0;
      $538 = (_sqlite3KeyInfoRef($537)|0);
      _sqlite3VdbeChangeP4($535,$536,$538,-6);
      $539 = HEAP32[$i>>2]|0;
      $540 = HEAP32[$pLoop>>2]|0;
      $541 = ((($540)) + 20|0);
      $542 = (($541) + ($539<<2)|0);
      HEAP32[$542>>2] = -1;
      $543 = HEAP32[$i>>2]|0;
      $544 = (($543) + 1)|0;
      HEAP32[$i>>2] = $544;
     }
     $545 = HEAP32[$pLoop>>2]|0;
     $546 = ((($545)) + 48|0);
     $547 = HEAP32[$546>>2]|0;
     HEAP32[$pLoop>>2] = $547;
    }
    $548 = HEAP32[$pKeyInfo>>2]|0;
    _sqlite3KeyInfoUnref($548);
   }
  }
 } while(0);
 $549 = ((($dest)) + 12|0);
 $550 = HEAP32[$549>>2]|0;
 $551 = HEAP32[$3>>2]|0;
 $552 = ((($551)) + 12|0);
 HEAP32[$552>>2] = $550;
 $553 = ((($dest)) + 16|0);
 $554 = HEAP32[$553>>2]|0;
 $555 = HEAP32[$3>>2]|0;
 $556 = ((($555)) + 16|0);
 HEAP32[$556>>2] = $554;
 $557 = HEAP32[$db>>2]|0;
 $558 = HEAP32[$pDelete>>2]|0;
 _sqlite3SelectDelete($557,$558);
 $559 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $559;
 $560 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($560|0);
}
function _sqlite3SelectExprHeight($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $nHeight = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $nHeight = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$nHeight>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 _heightOfSelect($1,$nHeight);
 $2 = HEAP32[$nHeight>>2]|0;
 STACKTOP = sp;return ($2|0);
}
function _pushDownWhereTerms($db,$pSubq,$pWhere,$iCursor) {
 $db = $db|0;
 $pSubq = $pSubq|0;
 $pWhere = $pWhere|0;
 $iCursor = $iCursor|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $8 = 0, $9 = 0, $nChng = 0, $pNew = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pNew = sp + 8|0;
 $nChng = sp + 4|0;
 $pX = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pSubq;
 HEAP32[$3>>2] = $pWhere;
 HEAP32[$4>>2] = $iCursor;
 HEAP32[$nChng>>2] = 0;
 $5 = HEAP32[$3>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 $7 = HEAP32[$2>>2]|0;
 HEAP32[$pX>>2] = $7;
 while(1) {
  $8 = HEAP32[$pX>>2]|0;
  $9 = ($8|0)!=(0|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pX>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = $12 & 8200;
  $14 = ($13|0)!=(0);
  if ($14) {
   label = 6;
   break;
  }
  $15 = HEAP32[$pX>>2]|0;
  $16 = ((($15)) + 48|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pX>>2] = $17;
 }
 if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 56|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ($20|0)!=(0|0);
 if ($21) {
  HEAP32[$0>>2] = 0;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 while(1) {
  $22 = HEAP32[$3>>2]|0;
  $23 = HEAP8[$22>>0]|0;
  $24 = $23&255;
  $25 = ($24|0)==(28);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$1>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$3>>2]|0;
  $29 = ((($28)) + 16|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$4>>2]|0;
  $32 = (_pushDownWhereTerms($26,$27,$30,$31)|0);
  $33 = HEAP32[$nChng>>2]|0;
  $34 = (($33) + ($32))|0;
  HEAP32[$nChng>>2] = $34;
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 12|0);
  $37 = HEAP32[$36>>2]|0;
  HEAP32[$3>>2] = $37;
 }
 $38 = HEAP32[$3>>2]|0;
 $39 = ((($38)) + 4|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $40 & 1;
 $42 = ($41|0)!=(0);
 if ($42) {
  HEAP32[$0>>2] = 0;
  $72 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($72|0);
 }
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$4>>2]|0;
 $45 = (_sqlite3ExprIsTableConstant($43,$44)|0);
 $46 = ($45|0)!=(0);
 L24: do {
  if ($46) {
   $47 = HEAP32[$nChng>>2]|0;
   $48 = (($47) + 1)|0;
   HEAP32[$nChng>>2] = $48;
   while(1) {
    $49 = HEAP32[$2>>2]|0;
    $50 = ($49|0)!=(0|0);
    if (!($50)) {
     break L24;
    }
    $51 = HEAP32[$1>>2]|0;
    $52 = HEAP32[$3>>2]|0;
    $53 = (_sqlite3ExprDup($51,$52,0)|0);
    HEAP32[$pNew>>2] = $53;
    $54 = HEAP32[$1>>2]|0;
    $55 = HEAP32[$pNew>>2]|0;
    $56 = HEAP32[$4>>2]|0;
    $57 = HEAP32[$2>>2]|0;
    $58 = HEAP32[$57>>2]|0;
    $59 = (_substExpr($54,$55,$56,$58)|0);
    HEAP32[$pNew>>2] = $59;
    $60 = HEAP32[$1>>2]|0;
    $61 = HEAP32[$2>>2]|0;
    $62 = ((($61)) + 32|0);
    $63 = HEAP32[$62>>2]|0;
    $64 = HEAP32[$pNew>>2]|0;
    $65 = (_sqlite3ExprAnd($60,$63,$64)|0);
    $66 = HEAP32[$2>>2]|0;
    $67 = ((($66)) + 32|0);
    HEAP32[$67>>2] = $65;
    $68 = HEAP32[$2>>2]|0;
    $69 = ((($68)) + 48|0);
    $70 = HEAP32[$69>>2]|0;
    HEAP32[$2>>2] = $70;
   }
  }
 } while(0);
 $71 = HEAP32[$nChng>>2]|0;
 HEAP32[$0>>2] = $71;
 $72 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($72|0);
}
function _sqlite3ClearTempRegCache($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 19|0);
 HEAP8[$2>>0] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 28|0);
 HEAP32[$4>>2] = 0;
 STACKTOP = sp;return;
}
function _keyInfoFromExprList($pParse,$pList,$iStart,$nExtra) {
 $pParse = $pParse|0;
 $pList = $pList|0;
 $iStart = $iStart|0;
 $nExtra = $nExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $i = 0, $nExpr = 0;
 var $pColl = 0, $pInfo = 0, $pItem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $nExpr = sp + 20|0;
 $pInfo = sp + 16|0;
 $pItem = sp + 12|0;
 $db = sp + 8|0;
 $i = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $iStart;
 HEAP32[$3>>2] = $nExtra;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$nExpr>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$nExpr>>2]|0;
 $10 = HEAP32[$2>>2]|0;
 $11 = (($9) - ($10))|0;
 $12 = HEAP32[$3>>2]|0;
 $13 = (($12) + 1)|0;
 $14 = (_sqlite3KeyInfoAlloc($8,$11,$13)|0);
 HEAP32[$pInfo>>2] = $14;
 $15 = HEAP32[$pInfo>>2]|0;
 $16 = ($15|0)!=(0|0);
 if (!($16)) {
  $56 = HEAP32[$pInfo>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $17 = HEAP32[$2>>2]|0;
 HEAP32[$i>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (($20) + (($21*20)|0)|0);
 HEAP32[$pItem>>2] = $22;
 while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$nExpr>>2]|0;
  $25 = ($23|0)<($24|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$0>>2]|0;
  $27 = HEAP32[$pItem>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = (_sqlite3ExprCollSeq($26,$28)|0);
  HEAP32[$pColl>>2] = $29;
  $30 = HEAP32[$pColl>>2]|0;
  $31 = ($30|0)!=(0|0);
  if (!($31)) {
   $32 = HEAP32[$db>>2]|0;
   $33 = ((($32)) + 8|0);
   $34 = HEAP32[$33>>2]|0;
   HEAP32[$pColl>>2] = $34;
  }
  $35 = HEAP32[$pColl>>2]|0;
  $36 = HEAP32[$i>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = (($36) - ($37))|0;
  $39 = HEAP32[$pInfo>>2]|0;
  $40 = ((($39)) + 20|0);
  $41 = (($40) + ($38<<2)|0);
  HEAP32[$41>>2] = $35;
  $42 = HEAP32[$pItem>>2]|0;
  $43 = ((($42)) + 12|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = HEAP32[$i>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = (($45) - ($46))|0;
  $48 = HEAP32[$pInfo>>2]|0;
  $49 = ((($48)) + 16|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = (($50) + ($47)|0);
  HEAP8[$51>>0] = $44;
  $52 = HEAP32[$i>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$i>>2] = $53;
  $54 = HEAP32[$pItem>>2]|0;
  $55 = ((($54)) + 20|0);
  HEAP32[$pItem>>2] = $55;
 }
 $56 = HEAP32[$pInfo>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _computeLimitRegisters($pParse,$p,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $iLimit = 0, $iOffset = 0, $n = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $v = sp + 12|0;
 $iLimit = sp + 8|0;
 $iOffset = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iBreak;
 HEAP32[$v>>2] = 0;
 HEAP32[$iLimit>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($7);
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 44|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + 1)|0;
 HEAP32[$13>>2] = $15;
 HEAP32[$iLimit>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 12|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = (_sqlite3GetVdbe($18)|0);
 HEAP32[$v>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 56|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = (_sqlite3ExprIsInteger($22,$n)|0);
 $24 = ($23|0)!=(0);
 do {
  if ($24) {
   $25 = HEAP32[$v>>2]|0;
   $26 = HEAP32[$n>>2]|0;
   $27 = HEAP32[$iLimit>>2]|0;
   (_sqlite3VdbeAddOp2($25,76,$26,$27)|0);
   $28 = HEAP32[$n>>2]|0;
   $29 = ($28|0)==(0);
   if ($29) {
    $30 = HEAP32[$v>>2]|0;
    $31 = HEAP32[$2>>2]|0;
    (_sqlite3VdbeGoto($30,$31)|0);
    break;
   }
   $32 = HEAP32[$n>>2]|0;
   $33 = ($32|0)>=(0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 6|0);
    $36 = HEAP16[$35>>1]|0;
    $37 = $36 << 16 >> 16;
    $38 = HEAP32[$n>>2]|0;
    $39 = ($38|0)<(0);
    $40 = $39 << 31 >> 31;
    $41 = (_sqlite3LogEst($38,$40)|0);
    $42 = $41 << 16 >> 16;
    $43 = ($37|0)>($42|0);
    if ($43) {
     $44 = HEAP32[$n>>2]|0;
     $45 = ($44|0)<(0);
     $46 = $45 << 31 >> 31;
     $47 = (_sqlite3LogEst($44,$46)|0);
     $48 = HEAP32[$1>>2]|0;
     $49 = ((($48)) + 6|0);
     HEAP16[$49>>1] = $47;
     $50 = HEAP32[$1>>2]|0;
     $51 = ((($50)) + 8|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = $52 | 16384;
     HEAP32[$51>>2] = $53;
    }
   }
  } else {
   $54 = HEAP32[$0>>2]|0;
   $55 = HEAP32[$1>>2]|0;
   $56 = ((($55)) + 56|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = HEAP32[$iLimit>>2]|0;
   _sqlite3ExprCode($54,$57,$58);
   $59 = HEAP32[$v>>2]|0;
   $60 = HEAP32[$iLimit>>2]|0;
   (_sqlite3VdbeAddOp1($59,17,$60)|0);
   $61 = HEAP32[$v>>2]|0;
   $62 = HEAP32[$iLimit>>2]|0;
   $63 = HEAP32[$2>>2]|0;
   (_sqlite3VdbeAddOp2($61,22,$62,$63)|0);
  }
 } while(0);
 $64 = HEAP32[$1>>2]|0;
 $65 = ((($64)) + 60|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = ($66|0)!=(0|0);
 if (!($67)) {
  STACKTOP = sp;return;
 }
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 44|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = (($70) + 1)|0;
 HEAP32[$69>>2] = $71;
 HEAP32[$iOffset>>2] = $71;
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 16|0);
 HEAP32[$73>>2] = $71;
 $74 = HEAP32[$0>>2]|0;
 $75 = ((($74)) + 44|0);
 $76 = HEAP32[$75>>2]|0;
 $77 = (($76) + 1)|0;
 HEAP32[$75>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 $79 = HEAP32[$1>>2]|0;
 $80 = ((($79)) + 60|0);
 $81 = HEAP32[$80>>2]|0;
 $82 = HEAP32[$iOffset>>2]|0;
 _sqlite3ExprCode($78,$81,$82);
 $83 = HEAP32[$v>>2]|0;
 $84 = HEAP32[$iOffset>>2]|0;
 (_sqlite3VdbeAddOp1($83,17,$84)|0);
 $85 = HEAP32[$v>>2]|0;
 $86 = HEAP32[$iLimit>>2]|0;
 $87 = HEAP32[$iOffset>>2]|0;
 $88 = (($87) + 1)|0;
 $89 = HEAP32[$iOffset>>2]|0;
 (_sqlite3VdbeAddOp3($85,146,$86,$88,$89)|0);
 STACKTOP = sp;return;
}
function _sqlite3VdbeChangeOpcode($p,$addr,$iNewOpcode) {
 $p = $p|0;
 $addr = $addr|0;
 $iNewOpcode = $iNewOpcode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $addr;
 HEAP8[$2>>0] = $iNewOpcode;
 $3 = HEAP8[$2>>0]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = (_sqlite3VdbeGetOp($4,$5)|0);
 HEAP8[$6>>0] = $3;
 STACKTOP = sp;return;
}
function _sqlite3WhereOutputRowCount($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 72|0);
 $3 = HEAP16[$2>>1]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3WhereIsDistinct($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 47|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WhereIsOrdered($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 43|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3 << 24 >> 24;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WhereOrderedInnerLoop($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 48|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WhereContinueLabel($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _sqlite3WhereBreakLabel($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _selectInnerLoop($pParse,$p,$pEList,$srcTab,$pSort,$pDistinct,$pDest,$iContinue,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pEList = $pEList|0;
 $srcTab = $srcTab|0;
 $pSort = $pSort|0;
 $pDistinct = $pDistinct|0;
 $pDest = $pDest|0;
 $iContinue = $iContinue|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addrTest = 0, $eDest = 0;
 var $ecelFlags = 0, $hasDistinct = 0, $i = 0, $iJump = 0, $iParm = 0, $nKey = 0, $nPrefixReg = 0, $nResultCol = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pColl = 0, $pOp = 0, $pSO = 0, $r1 = 0, $r11 = 0, $r12 = 0, $r13 = 0, $r2 = 0, $r24 = 0;
 var $r3 = 0, $regPrev = 0, $regResult = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 108|0;
 $5 = sp + 104|0;
 $6 = sp + 100|0;
 $7 = sp + 96|0;
 $8 = sp + 92|0;
 $v = sp + 88|0;
 $i = sp + 84|0;
 $hasDistinct = sp + 80|0;
 $regResult = sp + 76|0;
 $eDest = sp + 72|0;
 $iParm = sp + 68|0;
 $nResultCol = sp + 64|0;
 $nPrefixReg = sp + 60|0;
 $ecelFlags = sp + 128|0;
 $pOp = sp + 56|0;
 $iJump = sp + 52|0;
 $regPrev = sp + 48|0;
 $pColl = sp + 44|0;
 $r1 = sp + 40|0;
 $r11 = sp + 36|0;
 $addr = sp + 32|0;
 $r2 = sp + 28|0;
 $r12 = sp + 24|0;
 $nKey = sp + 20|0;
 $r13 = sp + 16|0;
 $r24 = sp + 12|0;
 $r3 = sp + 8|0;
 $addrTest = sp + 4|0;
 $pSO = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pEList;
 HEAP32[$3>>2] = $srcTab;
 HEAP32[$4>>2] = $pSort;
 HEAP32[$5>>2] = $pDistinct;
 HEAP32[$6>>2] = $pDest;
 HEAP32[$7>>2] = $iContinue;
 HEAP32[$8>>2] = $iBreak;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$6>>2]|0;
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 HEAP32[$eDest>>2] = $14;
 $15 = HEAP32[$6>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$iParm>>2] = $17;
 HEAP32[$nPrefixReg>>2] = 0;
 $18 = HEAP32[$5>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  $20 = HEAP32[$5>>2]|0;
  $21 = ((($20)) + 1|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = $23;
 } else {
  $24 = 0;
 }
 HEAP32[$hasDistinct>>2] = $24;
 $25 = HEAP32[$4>>2]|0;
 $26 = ($25|0)!=(0|0);
 if ($26) {
  $27 = HEAP32[$4>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)==(0|0);
  if ($29) {
   HEAP32[$4>>2] = 0;
  }
 }
 $30 = HEAP32[$4>>2]|0;
 $31 = ($30|0)!=(0|0);
 $32 = HEAP32[$hasDistinct>>2]|0;
 $33 = ($32|0)!=(0);
 $or$cond = $31 | $33;
 if (!($or$cond)) {
  $34 = HEAP32[$v>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 16|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$7>>2]|0;
  _codeOffset($34,$37,$38);
 }
 $39 = HEAP32[$2>>2]|0;
 $40 = HEAP32[$39>>2]|0;
 HEAP32[$nResultCol>>2] = $40;
 $41 = HEAP32[$6>>2]|0;
 $42 = ((($41)) + 12|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  $45 = HEAP32[$4>>2]|0;
  $46 = ($45|0)!=(0|0);
  if ($46) {
   $47 = HEAP32[$4>>2]|0;
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$nPrefixReg>>2] = $49;
   $50 = HEAP32[$4>>2]|0;
   $51 = ((($50)) + 28|0);
   $52 = HEAP8[$51>>0]|0;
   $53 = $52&255;
   $54 = $53 & 1;
   $55 = ($54|0)!=(0);
   if (!($55)) {
    $56 = HEAP32[$nPrefixReg>>2]|0;
    $57 = (($56) + 1)|0;
    HEAP32[$nPrefixReg>>2] = $57;
   }
   $58 = HEAP32[$nPrefixReg>>2]|0;
   $59 = HEAP32[$0>>2]|0;
   $60 = ((($59)) + 44|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (($61) + ($58))|0;
   HEAP32[$60>>2] = $62;
  }
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 44|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = (($65) + 1)|0;
  $67 = HEAP32[$6>>2]|0;
  $68 = ((($67)) + 12|0);
  HEAP32[$68>>2] = $66;
  $69 = HEAP32[$nResultCol>>2]|0;
  $70 = HEAP32[$0>>2]|0;
  $71 = ((($70)) + 44|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = (($72) + ($69))|0;
  HEAP32[$71>>2] = $73;
 } else {
  $74 = HEAP32[$6>>2]|0;
  $75 = ((($74)) + 12|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$nResultCol>>2]|0;
  $78 = (($76) + ($77))|0;
  $79 = HEAP32[$0>>2]|0;
  $80 = ((($79)) + 44|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = ($78|0)>($81|0);
  if ($82) {
   $83 = HEAP32[$nResultCol>>2]|0;
   $84 = HEAP32[$0>>2]|0;
   $85 = ((($84)) + 44|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = (($86) + ($83))|0;
   HEAP32[$85>>2] = $87;
  }
 }
 $88 = HEAP32[$nResultCol>>2]|0;
 $89 = HEAP32[$6>>2]|0;
 $90 = ((($89)) + 16|0);
 HEAP32[$90>>2] = $88;
 $91 = HEAP32[$6>>2]|0;
 $92 = ((($91)) + 12|0);
 $93 = HEAP32[$92>>2]|0;
 HEAP32[$regResult>>2] = $93;
 $94 = HEAP32[$3>>2]|0;
 $95 = ($94|0)>=(0);
 L22: do {
  if ($95) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $96 = HEAP32[$i>>2]|0;
    $97 = HEAP32[$nResultCol>>2]|0;
    $98 = ($96|0)<($97|0);
    if (!($98)) {
     break L22;
    }
    $99 = HEAP32[$v>>2]|0;
    $100 = HEAP32[$3>>2]|0;
    $101 = HEAP32[$i>>2]|0;
    $102 = HEAP32[$regResult>>2]|0;
    $103 = HEAP32[$i>>2]|0;
    $104 = (($102) + ($103))|0;
    (_sqlite3VdbeAddOp3($99,96,$100,$101,$104)|0);
    $105 = HEAP32[$i>>2]|0;
    $106 = (($105) + 1)|0;
    HEAP32[$i>>2] = $106;
   }
  } else {
   $107 = HEAP32[$eDest>>2]|0;
   $108 = ($107|0)!=(3);
   if ($108) {
    $109 = HEAP32[$eDest>>2]|0;
    $110 = ($109|0)==(10);
    $111 = HEAP32[$eDest>>2]|0;
    $112 = ($111|0)==(9);
    $or$cond3 = $110 | $112;
    $113 = HEAP32[$eDest>>2]|0;
    $114 = ($113|0)==(13);
    $or$cond5 = $or$cond3 | $114;
    if ($or$cond5) {
     HEAP8[$ecelFlags>>0] = 1;
    } else {
     HEAP8[$ecelFlags>>0] = 0;
    }
    $115 = HEAP32[$0>>2]|0;
    $116 = HEAP32[$2>>2]|0;
    $117 = HEAP32[$regResult>>2]|0;
    $118 = HEAP8[$ecelFlags>>0]|0;
    (_sqlite3ExprCodeExprList($115,$116,$117,0,$118)|0);
   }
  }
 } while(0);
 $119 = HEAP32[$hasDistinct>>2]|0;
 $120 = ($119|0)!=(0);
 if ($120) {
  $121 = HEAP32[$5>>2]|0;
  $122 = ((($121)) + 1|0);
  $123 = HEAP8[$122>>0]|0;
  $124 = $123&255;
  switch ($124|0) {
  case 2:  {
   $125 = HEAP32[$0>>2]|0;
   $126 = ((($125)) + 44|0);
   $127 = HEAP32[$126>>2]|0;
   $128 = (($127) + 1)|0;
   HEAP32[$regPrev>>2] = $128;
   $129 = HEAP32[$nResultCol>>2]|0;
   $130 = HEAP32[$0>>2]|0;
   $131 = ((($130)) + 44|0);
   $132 = HEAP32[$131>>2]|0;
   $133 = (($132) + ($129))|0;
   HEAP32[$131>>2] = $133;
   $134 = HEAP32[$v>>2]|0;
   $135 = HEAP32[$5>>2]|0;
   $136 = ((($135)) + 8|0);
   $137 = HEAP32[$136>>2]|0;
   (_sqlite3VdbeChangeToNoop($134,$137)|0);
   $138 = HEAP32[$v>>2]|0;
   $139 = HEAP32[$5>>2]|0;
   $140 = ((($139)) + 8|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = (_sqlite3VdbeGetOp($138,$141)|0);
   HEAP32[$pOp>>2] = $142;
   $143 = HEAP32[$pOp>>2]|0;
   HEAP8[$143>>0] = 79;
   $144 = HEAP32[$pOp>>2]|0;
   $145 = ((($144)) + 4|0);
   HEAP32[$145>>2] = 1;
   $146 = HEAP32[$regPrev>>2]|0;
   $147 = HEAP32[$pOp>>2]|0;
   $148 = ((($147)) + 8|0);
   HEAP32[$148>>2] = $146;
   $149 = HEAP32[$v>>2]|0;
   $150 = (_sqlite3VdbeCurrentAddr($149)|0);
   $151 = HEAP32[$nResultCol>>2]|0;
   $152 = (($150) + ($151))|0;
   HEAP32[$iJump>>2] = $152;
   HEAP32[$i>>2] = 0;
   while(1) {
    $153 = HEAP32[$i>>2]|0;
    $154 = HEAP32[$nResultCol>>2]|0;
    $155 = ($153|0)<($154|0);
    if (!($155)) {
     break;
    }
    $156 = HEAP32[$0>>2]|0;
    $157 = HEAP32[$i>>2]|0;
    $158 = HEAP32[$2>>2]|0;
    $159 = ((($158)) + 4|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (($160) + (($157*20)|0)|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = (_sqlite3ExprCollSeq($156,$162)|0);
    HEAP32[$pColl>>2] = $163;
    $164 = HEAP32[$i>>2]|0;
    $165 = HEAP32[$nResultCol>>2]|0;
    $166 = (($165) - 1)|0;
    $167 = ($164|0)<($166|0);
    $168 = HEAP32[$v>>2]|0;
    $169 = HEAP32[$regResult>>2]|0;
    $170 = HEAP32[$i>>2]|0;
    $171 = (($169) + ($170))|0;
    if ($167) {
     $172 = HEAP32[$iJump>>2]|0;
     $173 = HEAP32[$regPrev>>2]|0;
     $174 = HEAP32[$i>>2]|0;
     $175 = (($173) + ($174))|0;
     (_sqlite3VdbeAddOp3($168,36,$171,$172,$175)|0);
    } else {
     $176 = HEAP32[$7>>2]|0;
     $177 = HEAP32[$regPrev>>2]|0;
     $178 = HEAP32[$i>>2]|0;
     $179 = (($177) + ($178))|0;
     (_sqlite3VdbeAddOp3($168,37,$171,$176,$179)|0);
    }
    $180 = HEAP32[$v>>2]|0;
    $181 = HEAP32[$pColl>>2]|0;
    _sqlite3VdbeChangeP4($180,-1,$181,-4);
    $182 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($182,-128);
    $183 = HEAP32[$i>>2]|0;
    $184 = (($183) + 1)|0;
    HEAP32[$i>>2] = $184;
   }
   $185 = HEAP32[$v>>2]|0;
   $186 = HEAP32[$regResult>>2]|0;
   $187 = HEAP32[$regPrev>>2]|0;
   $188 = HEAP32[$nResultCol>>2]|0;
   $189 = (($188) - 1)|0;
   (_sqlite3VdbeAddOp3($185,84,$186,$187,$189)|0);
   break;
  }
  case 1:  {
   $190 = HEAP32[$v>>2]|0;
   $191 = HEAP32[$5>>2]|0;
   $192 = ((($191)) + 8|0);
   $193 = HEAP32[$192>>2]|0;
   (_sqlite3VdbeChangeToNoop($190,$193)|0);
   break;
  }
  default: {
   $194 = HEAP32[$0>>2]|0;
   $195 = HEAP32[$5>>2]|0;
   $196 = ((($195)) + 4|0);
   $197 = HEAP32[$196>>2]|0;
   $198 = HEAP32[$7>>2]|0;
   $199 = HEAP32[$nResultCol>>2]|0;
   $200 = HEAP32[$regResult>>2]|0;
   _codeDistinct($194,$197,$198,$199,$200);
  }
  }
  $201 = HEAP32[$4>>2]|0;
  $202 = ($201|0)==(0|0);
  if ($202) {
   $203 = HEAP32[$v>>2]|0;
   $204 = HEAP32[$1>>2]|0;
   $205 = ((($204)) + 16|0);
   $206 = HEAP32[$205>>2]|0;
   $207 = HEAP32[$7>>2]|0;
   _codeOffset($203,$206,$207);
  }
 }
 $208 = HEAP32[$eDest>>2]|0;
 L51: do {
  switch ($208|0) {
  case 1:  {
   $209 = HEAP32[$0>>2]|0;
   $210 = (_sqlite3GetTempReg($209)|0);
   HEAP32[$r1>>2] = $210;
   $211 = HEAP32[$v>>2]|0;
   $212 = HEAP32[$regResult>>2]|0;
   $213 = HEAP32[$nResultCol>>2]|0;
   $214 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp3($211,99,$212,$213,$214)|0);
   $215 = HEAP32[$v>>2]|0;
   $216 = HEAP32[$iParm>>2]|0;
   $217 = HEAP32[$r1>>2]|0;
   (_sqlite3VdbeAddOp2($215,126,$216,$217)|0);
   $218 = HEAP32[$0>>2]|0;
   $219 = HEAP32[$r1>>2]|0;
   _sqlite3ReleaseTempReg($218,$219);
   break;
  }
  case 2:  {
   $220 = HEAP32[$v>>2]|0;
   $221 = HEAP32[$iParm>>2]|0;
   $222 = HEAP32[$regResult>>2]|0;
   $223 = HEAP32[$nResultCol>>2]|0;
   (_sqlite3VdbeAddOp3($220,127,$221,$222,$223)|0);
   break;
  }
  case 12: case 14: case 6: case 5:  {
   $224 = HEAP32[$0>>2]|0;
   $225 = HEAP32[$nPrefixReg>>2]|0;
   $226 = (($225) + 1)|0;
   $227 = (_sqlite3GetTempRange($224,$226)|0);
   HEAP32[$r11>>2] = $227;
   $228 = HEAP32[$v>>2]|0;
   $229 = HEAP32[$regResult>>2]|0;
   $230 = HEAP32[$nResultCol>>2]|0;
   $231 = HEAP32[$r11>>2]|0;
   $232 = HEAP32[$nPrefixReg>>2]|0;
   $233 = (($231) + ($232))|0;
   (_sqlite3VdbeAddOp3($228,99,$229,$230,$233)|0);
   $234 = HEAP32[$eDest>>2]|0;
   $235 = ($234|0)==(6);
   if ($235) {
    $236 = HEAP32[$v>>2]|0;
    $237 = (_sqlite3VdbeCurrentAddr($236)|0);
    $238 = (($237) + 4)|0;
    HEAP32[$addr>>2] = $238;
    $239 = HEAP32[$v>>2]|0;
    $240 = HEAP32[$iParm>>2]|0;
    $241 = (($240) + 1)|0;
    $242 = HEAP32[$addr>>2]|0;
    $243 = HEAP32[$r11>>2]|0;
    (_sqlite3VdbeAddOp4Int($239,31,$241,$242,$243,0)|0);
    $244 = HEAP32[$v>>2]|0;
    $245 = HEAP32[$iParm>>2]|0;
    $246 = (($245) + 1)|0;
    $247 = HEAP32[$r11>>2]|0;
    (_sqlite3VdbeAddOp2($244,126,$246,$247)|0);
   }
   $248 = HEAP32[$4>>2]|0;
   $249 = ($248|0)!=(0|0);
   $250 = HEAP32[$0>>2]|0;
   if ($249) {
    $251 = HEAP32[$4>>2]|0;
    $252 = HEAP32[$1>>2]|0;
    $253 = HEAP32[$r11>>2]|0;
    $254 = HEAP32[$nPrefixReg>>2]|0;
    $255 = (($253) + ($254))|0;
    $256 = HEAP32[$regResult>>2]|0;
    $257 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($250,$251,$252,$255,$256,1,$257);
   } else {
    $258 = (_sqlite3GetTempReg($250)|0);
    HEAP32[$r2>>2] = $258;
    $259 = HEAP32[$v>>2]|0;
    $260 = HEAP32[$iParm>>2]|0;
    $261 = HEAP32[$r2>>2]|0;
    (_sqlite3VdbeAddOp2($259,114,$260,$261)|0);
    $262 = HEAP32[$v>>2]|0;
    $263 = HEAP32[$iParm>>2]|0;
    $264 = HEAP32[$r11>>2]|0;
    $265 = HEAP32[$r2>>2]|0;
    (_sqlite3VdbeAddOp3($262,115,$263,$264,$265)|0);
    $266 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($266,8);
    $267 = HEAP32[$0>>2]|0;
    $268 = HEAP32[$r2>>2]|0;
    _sqlite3ReleaseTempReg($267,$268);
   }
   $269 = HEAP32[$0>>2]|0;
   $270 = HEAP32[$r11>>2]|0;
   $271 = HEAP32[$nPrefixReg>>2]|0;
   $272 = (($271) + 1)|0;
   _sqlite3ReleaseTempRange($269,$270,$272);
   break;
  }
  case 11:  {
   $273 = HEAP32[$4>>2]|0;
   $274 = ($273|0)!=(0|0);
   $275 = HEAP32[$0>>2]|0;
   if ($274) {
    $276 = HEAP32[$4>>2]|0;
    $277 = HEAP32[$1>>2]|0;
    $278 = HEAP32[$regResult>>2]|0;
    $279 = HEAP32[$regResult>>2]|0;
    $280 = HEAP32[$nResultCol>>2]|0;
    $281 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($275,$276,$277,$278,$279,$280,$281);
    break L51;
   } else {
    $282 = (_sqlite3GetTempReg($275)|0);
    HEAP32[$r12>>2] = $282;
    $283 = HEAP32[$v>>2]|0;
    $284 = HEAP32[$regResult>>2]|0;
    $285 = HEAP32[$nResultCol>>2]|0;
    $286 = HEAP32[$r12>>2]|0;
    $287 = HEAP32[$6>>2]|0;
    $288 = ((($287)) + 4|0);
    $289 = HEAP32[$288>>2]|0;
    $290 = HEAP32[$nResultCol>>2]|0;
    (_sqlite3VdbeAddOp4($283,99,$284,$285,$286,$289,$290)|0);
    $291 = HEAP32[$0>>2]|0;
    $292 = HEAP32[$regResult>>2]|0;
    $293 = HEAP32[$nResultCol>>2]|0;
    _sqlite3ExprCacheAffinityChange($291,$292,$293);
    $294 = HEAP32[$v>>2]|0;
    $295 = HEAP32[$iParm>>2]|0;
    $296 = HEAP32[$r12>>2]|0;
    (_sqlite3VdbeAddOp2($294,126,$295,$296)|0);
    $297 = HEAP32[$0>>2]|0;
    $298 = HEAP32[$r12>>2]|0;
    _sqlite3ReleaseTempReg($297,$298);
    break L51;
   }
   break;
  }
  case 3:  {
   $299 = HEAP32[$v>>2]|0;
   $300 = HEAP32[$iParm>>2]|0;
   (_sqlite3VdbeAddOp2($299,76,1,$300)|0);
   break;
  }
  case 10:  {
   $301 = HEAP32[$4>>2]|0;
   $302 = ($301|0)!=(0|0);
   if ($302) {
    $303 = HEAP32[$0>>2]|0;
    $304 = HEAP32[$4>>2]|0;
    $305 = HEAP32[$1>>2]|0;
    $306 = HEAP32[$regResult>>2]|0;
    $307 = HEAP32[$regResult>>2]|0;
    $308 = HEAP32[$nResultCol>>2]|0;
    $309 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($303,$304,$305,$306,$307,$308,$309);
   }
   break;
  }
  case 9: case 13:  {
   $310 = HEAP32[$4>>2]|0;
   $311 = ($310|0)!=(0|0);
   if ($311) {
    $312 = HEAP32[$0>>2]|0;
    $313 = HEAP32[$4>>2]|0;
    $314 = HEAP32[$1>>2]|0;
    $315 = HEAP32[$regResult>>2]|0;
    $316 = HEAP32[$regResult>>2]|0;
    $317 = HEAP32[$nResultCol>>2]|0;
    $318 = HEAP32[$nPrefixReg>>2]|0;
    _pushOntoSorter($312,$313,$314,$315,$316,$317,$318);
    break L51;
   }
   $319 = HEAP32[$eDest>>2]|0;
   $320 = ($319|0)==(13);
   $321 = HEAP32[$v>>2]|0;
   if ($320) {
    $322 = HEAP32[$6>>2]|0;
    $323 = ((($322)) + 8|0);
    $324 = HEAP32[$323>>2]|0;
    (_sqlite3VdbeAddOp1($321,16,$324)|0);
    break L51;
   } else {
    $325 = HEAP32[$regResult>>2]|0;
    $326 = HEAP32[$nResultCol>>2]|0;
    (_sqlite3VdbeAddOp2($321,87,$325,$326)|0);
    $327 = HEAP32[$0>>2]|0;
    $328 = HEAP32[$regResult>>2]|0;
    $329 = HEAP32[$nResultCol>>2]|0;
    _sqlite3ExprCacheAffinityChange($327,$328,$329);
    break L51;
   }
   break;
  }
  case 7: case 8:  {
   HEAP32[$addrTest>>2] = 0;
   $330 = HEAP32[$6>>2]|0;
   $331 = ((($330)) + 20|0);
   $332 = HEAP32[$331>>2]|0;
   HEAP32[$pSO>>2] = $332;
   $333 = HEAP32[$pSO>>2]|0;
   $334 = HEAP32[$333>>2]|0;
   HEAP32[$nKey>>2] = $334;
   $335 = HEAP32[$0>>2]|0;
   $336 = (_sqlite3GetTempReg($335)|0);
   HEAP32[$r13>>2] = $336;
   $337 = HEAP32[$0>>2]|0;
   $338 = HEAP32[$nKey>>2]|0;
   $339 = (($338) + 2)|0;
   $340 = (_sqlite3GetTempRange($337,$339)|0);
   HEAP32[$r24>>2] = $340;
   $341 = HEAP32[$r24>>2]|0;
   $342 = HEAP32[$nKey>>2]|0;
   $343 = (($341) + ($342))|0;
   $344 = (($343) + 1)|0;
   HEAP32[$r3>>2] = $344;
   $345 = HEAP32[$eDest>>2]|0;
   $346 = ($345|0)==(8);
   if ($346) {
    $347 = HEAP32[$v>>2]|0;
    $348 = HEAP32[$iParm>>2]|0;
    $349 = (($348) + 1)|0;
    $350 = HEAP32[$regResult>>2]|0;
    $351 = HEAP32[$nResultCol>>2]|0;
    $352 = (_sqlite3VdbeAddOp4Int($347,31,$349,0,$350,$351)|0);
    HEAP32[$addrTest>>2] = $352;
   }
   $353 = HEAP32[$v>>2]|0;
   $354 = HEAP32[$regResult>>2]|0;
   $355 = HEAP32[$nResultCol>>2]|0;
   $356 = HEAP32[$r3>>2]|0;
   (_sqlite3VdbeAddOp3($353,99,$354,$355,$356)|0);
   $357 = HEAP32[$eDest>>2]|0;
   $358 = ($357|0)==(8);
   if ($358) {
    $359 = HEAP32[$v>>2]|0;
    $360 = HEAP32[$iParm>>2]|0;
    $361 = (($360) + 1)|0;
    $362 = HEAP32[$r3>>2]|0;
    (_sqlite3VdbeAddOp2($359,126,$361,$362)|0);
    $363 = HEAP32[$v>>2]|0;
    _sqlite3VdbeChangeP5($363,16);
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $364 = HEAP32[$i>>2]|0;
    $365 = HEAP32[$nKey>>2]|0;
    $366 = ($364|0)<($365|0);
    $367 = HEAP32[$v>>2]|0;
    if (!($366)) {
     break;
    }
    $368 = HEAP32[$regResult>>2]|0;
    $369 = HEAP32[$i>>2]|0;
    $370 = HEAP32[$pSO>>2]|0;
    $371 = ((($370)) + 4|0);
    $372 = HEAP32[$371>>2]|0;
    $373 = (($372) + (($369*20)|0)|0);
    $374 = ((($373)) + 16|0);
    $375 = HEAP16[$374>>1]|0;
    $376 = $375&65535;
    $377 = (($368) + ($376))|0;
    $378 = (($377) - 1)|0;
    $379 = HEAP32[$r24>>2]|0;
    $380 = HEAP32[$i>>2]|0;
    $381 = (($379) + ($380))|0;
    (_sqlite3VdbeAddOp2($367,85,$378,$381)|0);
    $382 = HEAP32[$i>>2]|0;
    $383 = (($382) + 1)|0;
    HEAP32[$i>>2] = $383;
   }
   $384 = HEAP32[$iParm>>2]|0;
   $385 = HEAP32[$r24>>2]|0;
   $386 = HEAP32[$nKey>>2]|0;
   $387 = (($385) + ($386))|0;
   (_sqlite3VdbeAddOp2($367,113,$384,$387)|0);
   $388 = HEAP32[$v>>2]|0;
   $389 = HEAP32[$r24>>2]|0;
   $390 = HEAP32[$nKey>>2]|0;
   $391 = (($390) + 2)|0;
   $392 = HEAP32[$r13>>2]|0;
   (_sqlite3VdbeAddOp3($388,99,$389,$391,$392)|0);
   $393 = HEAP32[$v>>2]|0;
   $394 = HEAP32[$iParm>>2]|0;
   $395 = HEAP32[$r13>>2]|0;
   (_sqlite3VdbeAddOp2($393,126,$394,$395)|0);
   $396 = HEAP32[$addrTest>>2]|0;
   $397 = ($396|0)!=(0);
   if ($397) {
    $398 = HEAP32[$v>>2]|0;
    $399 = HEAP32[$addrTest>>2]|0;
    _sqlite3VdbeJumpHere($398,$399);
   }
   $400 = HEAP32[$0>>2]|0;
   $401 = HEAP32[$r13>>2]|0;
   _sqlite3ReleaseTempReg($400,$401);
   $402 = HEAP32[$0>>2]|0;
   $403 = HEAP32[$r24>>2]|0;
   $404 = HEAP32[$nKey>>2]|0;
   $405 = (($404) + 2)|0;
   _sqlite3ReleaseTempRange($402,$403,$405);
   break;
  }
  default: {
  }
  }
 } while(0);
 $406 = HEAP32[$4>>2]|0;
 $407 = ($406|0)==(0|0);
 if (!($407)) {
  STACKTOP = sp;return;
 }
 $408 = HEAP32[$1>>2]|0;
 $409 = ((($408)) + 12|0);
 $410 = HEAP32[$409>>2]|0;
 $411 = ($410|0)!=(0);
 if (!($411)) {
  STACKTOP = sp;return;
 }
 $412 = HEAP32[$v>>2]|0;
 $413 = HEAP32[$1>>2]|0;
 $414 = ((($413)) + 12|0);
 $415 = HEAP32[$414>>2]|0;
 $416 = HEAP32[$8>>2]|0;
 (_sqlite3VdbeAddOp2($412,68,$415,$416)|0);
 STACKTOP = sp;return;
}
function _sqlite3ExprAnalyzeAggList($pNC,$pList) {
 $pNC = $pNC|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pItem = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pItem = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pItem>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($7|0)<($9|0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$pItem>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  _sqlite3ExprAnalyzeAggregates($11,$13);
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
  $16 = HEAP32[$pItem>>2]|0;
  $17 = ((($16)) + 20|0);
  HEAP32[$pItem>>2] = $17;
 }
 STACKTOP = sp;return;
}
function _sqlite3ExprAnalyzeAggregates($pNC,$pExpr) {
 $pNC = $pNC|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $w = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $w = sp;
 HEAP32[$0>>2] = $pNC;
 HEAP32[$1>>2] = $pExpr;
 ;HEAP32[$w>>2]=0|0;HEAP32[$w+4>>2]=0|0;HEAP32[$w+8>>2]=0|0;HEAP32[$w+12>>2]=0|0;HEAP32[$w+16>>2]=0|0;HEAP32[$w+20>>2]=0|0;HEAP32[$w+24>>2]=0|0;
 $2 = ((($w)) + 4|0);
 HEAP32[$2>>2] = 203;
 $3 = ((($w)) + 8|0);
 HEAP32[$3>>2] = 204;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($w)) + 24|0);
 HEAP32[$5>>2] = $4;
 $6 = HEAP32[$1>>2]|0;
 (_sqlite3WalkExpr($w,$6)|0);
 STACKTOP = sp;return;
}
function _explainTempTable($pParse,$zUsage) {
 $pParse = $pParse|0;
 $zUsage = $zUsage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0;
 var $vararg_buffer = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $v = sp + 8|0;
 $zMsg = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $zUsage;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 409|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(2);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $12;
 $13 = (_sqlite3MPrintf($11,35808,$vararg_buffer)|0);
 HEAP32[$zMsg>>2] = $13;
 $14 = HEAP32[$v>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 420|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$zMsg>>2]|0;
 (_sqlite3VdbeAddOp4($14,162,$17,0,0,$18,-1)|0);
 STACKTOP = sp;return;
}
function _sqlite3WhereIsSorted($pWInfo) {
 $pWInfo = $pWInfo|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pWInfo;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 44|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3ExprCodeMove($pParse,$iFrom,$iTo,$nReg) {
 $pParse = $pParse|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 $nReg = $nReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iFrom;
 HEAP32[$2>>2] = $iTo;
 HEAP32[$3>>2] = $nReg;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp3($6,83,$7,$8,$9)|0);
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$3>>2]|0;
 _sqlite3ExprCacheRemove($10,$11,$12);
 STACKTOP = sp;return;
}
function _updateAccumulator($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $addrHitTest = 0, $addrNext = 0, $i = 0, $j = 0, $nArg = 0, $pC = 0, $pColl = 0, $pF = 0, $pItem = 0, $pList = 0, $regAgg = 0, $regHit = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $v = sp + 48|0;
 $i = sp + 44|0;
 $regHit = sp + 40|0;
 $addrHitTest = sp + 36|0;
 $pF = sp + 32|0;
 $pC = sp + 28|0;
 $nArg = sp + 24|0;
 $addrNext = sp + 20|0;
 $regAgg = sp + 16|0;
 $pList = sp + 12|0;
 $pColl = sp + 8|0;
 $pItem = sp + 4|0;
 $j = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 HEAP32[$regHit>>2] = 0;
 HEAP32[$addrHitTest>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 HEAP8[$5>>0] = 1;
 HEAP32[$i>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 40|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pF>>2] = $8;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 44|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   break;
  }
  HEAP32[$addrNext>>2] = 0;
  $14 = HEAP32[$pF>>2]|0;
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 20|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pList>>2] = $17;
  $18 = HEAP32[$pList>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$pList>>2]|0;
   $21 = HEAP32[$20>>2]|0;
   HEAP32[$nArg>>2] = $21;
   $22 = HEAP32[$0>>2]|0;
   $23 = HEAP32[$nArg>>2]|0;
   $24 = (_sqlite3GetTempRange($22,$23)|0);
   HEAP32[$regAgg>>2] = $24;
   $25 = HEAP32[$0>>2]|0;
   $26 = HEAP32[$pList>>2]|0;
   $27 = HEAP32[$regAgg>>2]|0;
   (_sqlite3ExprCodeExprList($25,$26,$27,0,1)|0);
  } else {
   HEAP32[$nArg>>2] = 0;
   HEAP32[$regAgg>>2] = 0;
  }
  $28 = HEAP32[$pF>>2]|0;
  $29 = ((($28)) + 12|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)>=(0);
  if ($31) {
   $32 = HEAP32[$v>>2]|0;
   $33 = (_sqlite3VdbeMakeLabel($32)|0);
   HEAP32[$addrNext>>2] = $33;
   $34 = HEAP32[$0>>2]|0;
   $35 = HEAP32[$pF>>2]|0;
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$addrNext>>2]|0;
   $39 = HEAP32[$regAgg>>2]|0;
   _codeDistinct($34,$37,$38,1,$39);
  }
  $40 = HEAP32[$pF>>2]|0;
  $41 = ((($40)) + 4|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ((($42)) + 2|0);
  $44 = HEAP16[$43>>1]|0;
  $45 = $44&65535;
  $46 = $45 & 32;
  $47 = ($46|0)!=(0);
  if ($47) {
   HEAP32[$pColl>>2] = 0;
   HEAP32[$j>>2] = 0;
   $48 = HEAP32[$pList>>2]|0;
   $49 = ((($48)) + 4|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pItem>>2] = $50;
   while(1) {
    $51 = HEAP32[$pColl>>2]|0;
    $52 = ($51|0)!=(0|0);
    if ($52) {
     break;
    }
    $53 = HEAP32[$j>>2]|0;
    $54 = HEAP32[$nArg>>2]|0;
    $55 = ($53|0)<($54|0);
    if (!($55)) {
     break;
    }
    $56 = HEAP32[$0>>2]|0;
    $57 = HEAP32[$pItem>>2]|0;
    $58 = HEAP32[$57>>2]|0;
    $59 = (_sqlite3ExprCollSeq($56,$58)|0);
    HEAP32[$pColl>>2] = $59;
    $60 = HEAP32[$j>>2]|0;
    $61 = (($60) + 1)|0;
    HEAP32[$j>>2] = $61;
    $62 = HEAP32[$pItem>>2]|0;
    $63 = ((($62)) + 20|0);
    HEAP32[$pItem>>2] = $63;
   }
   $64 = HEAP32[$pColl>>2]|0;
   $65 = ($64|0)!=(0|0);
   if (!($65)) {
    $66 = HEAP32[$0>>2]|0;
    $67 = HEAP32[$66>>2]|0;
    $68 = ((($67)) + 8|0);
    $69 = HEAP32[$68>>2]|0;
    HEAP32[$pColl>>2] = $69;
   }
   $70 = HEAP32[$regHit>>2]|0;
   $71 = ($70|0)==(0);
   if ($71) {
    $72 = HEAP32[$1>>2]|0;
    $73 = ((($72)) + 36|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($74|0)!=(0);
    if ($75) {
     $76 = HEAP32[$0>>2]|0;
     $77 = ((($76)) + 44|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = (($78) + 1)|0;
     HEAP32[$77>>2] = $79;
     HEAP32[$regHit>>2] = $79;
    }
   }
   $80 = HEAP32[$v>>2]|0;
   $81 = HEAP32[$regHit>>2]|0;
   $82 = HEAP32[$pColl>>2]|0;
   (_sqlite3VdbeAddOp4($80,88,$81,0,0,$82,-4)|0);
  }
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$regAgg>>2]|0;
  $85 = HEAP32[$pF>>2]|0;
  $86 = ((($85)) + 8|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = HEAP32[$pF>>2]|0;
  $89 = ((($88)) + 4|0);
  $90 = HEAP32[$89>>2]|0;
  (_sqlite3VdbeAddOp4($83,147,0,$84,$87,$90,-5)|0);
  $91 = HEAP32[$v>>2]|0;
  $92 = HEAP32[$nArg>>2]|0;
  $93 = $92&255;
  _sqlite3VdbeChangeP5($91,$93);
  $94 = HEAP32[$0>>2]|0;
  $95 = HEAP32[$regAgg>>2]|0;
  $96 = HEAP32[$nArg>>2]|0;
  _sqlite3ExprCacheAffinityChange($94,$95,$96);
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$regAgg>>2]|0;
  $99 = HEAP32[$nArg>>2]|0;
  _sqlite3ReleaseTempRange($97,$98,$99);
  $100 = HEAP32[$addrNext>>2]|0;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$v>>2]|0;
   $103 = HEAP32[$addrNext>>2]|0;
   _sqlite3VdbeResolveLabel($102,$103);
   $104 = HEAP32[$0>>2]|0;
   _sqlite3ExprCacheClear($104);
  }
  $105 = HEAP32[$i>>2]|0;
  $106 = (($105) + 1)|0;
  HEAP32[$i>>2] = $106;
  $107 = HEAP32[$pF>>2]|0;
  $108 = ((($107)) + 16|0);
  HEAP32[$pF>>2] = $108;
 }
 $109 = HEAP32[$regHit>>2]|0;
 $110 = ($109|0)!=(0);
 if ($110) {
  $111 = HEAP32[$v>>2]|0;
  $112 = HEAP32[$regHit>>2]|0;
  $113 = (_sqlite3VdbeAddOp1($111,21,$112)|0);
  HEAP32[$addrHitTest>>2] = $113;
 }
 $114 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($114);
 HEAP32[$i>>2] = 0;
 $115 = HEAP32[$1>>2]|0;
 $116 = ((($115)) + 28|0);
 $117 = HEAP32[$116>>2]|0;
 HEAP32[$pC>>2] = $117;
 while(1) {
  $118 = HEAP32[$i>>2]|0;
  $119 = HEAP32[$1>>2]|0;
  $120 = ((($119)) + 36|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($118|0)<($121|0);
  if (!($122)) {
   break;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = HEAP32[$pC>>2]|0;
  $125 = ((($124)) + 20|0);
  $126 = HEAP32[$125>>2]|0;
  $127 = HEAP32[$pC>>2]|0;
  $128 = ((($127)) + 16|0);
  $129 = HEAP32[$128>>2]|0;
  _sqlite3ExprCode($123,$126,$129);
  $130 = HEAP32[$i>>2]|0;
  $131 = (($130) + 1)|0;
  HEAP32[$i>>2] = $131;
  $132 = HEAP32[$pC>>2]|0;
  $133 = ((($132)) + 24|0);
  HEAP32[$pC>>2] = $133;
 }
 $134 = HEAP32[$1>>2]|0;
 HEAP8[$134>>0] = 0;
 $135 = HEAP32[$0>>2]|0;
 _sqlite3ExprCacheClear($135);
 $136 = HEAP32[$addrHitTest>>2]|0;
 $137 = ($136|0)!=(0);
 if (!($137)) {
  STACKTOP = sp;return;
 }
 $138 = HEAP32[$v>>2]|0;
 $139 = HEAP32[$addrHitTest>>2]|0;
 _sqlite3VdbeJumpHere($138,$139);
 STACKTOP = sp;return;
}
function _finalizeAggFunctions($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pF = 0, $pList = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $v = sp + 12|0;
 $i = sp + 8|0;
 $pF = sp + 4|0;
 $pList = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 HEAP32[$i>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 40|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pF>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 44|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$pF>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pList>>2] = $16;
  $17 = HEAP32[$v>>2]|0;
  $18 = HEAP32[$pF>>2]|0;
  $19 = ((($18)) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$pList>>2]|0;
  $22 = ($21|0)!=(0|0);
  if ($22) {
   $23 = HEAP32[$pList>>2]|0;
   $24 = HEAP32[$23>>2]|0;
   $28 = $24;
  } else {
   $28 = 0;
  }
  $25 = HEAP32[$pF>>2]|0;
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  (_sqlite3VdbeAddOp4($17,149,$20,$28,0,$27,-5)|0);
  $29 = HEAP32[$i>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$i>>2] = $30;
  $31 = HEAP32[$pF>>2]|0;
  $32 = ((($31)) + 16|0);
  HEAP32[$pF>>2] = $32;
 }
 STACKTOP = sp;return;
}
function _resetAccumulator($pParse,$pAggInfo) {
 $pParse = $pParse|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $nReg = 0, $pE = 0, $pFunc = 0, $pKeyInfo = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $v = sp + 24|0;
 $i = sp + 20|0;
 $pFunc = sp + 16|0;
 $nReg = sp + 12|0;
 $pE = sp + 8|0;
 $pKeyInfo = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pAggInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$v>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 44|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($7) + ($10))|0;
 HEAP32[$nReg>>2] = $11;
 $12 = HEAP32[$nReg>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$v>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 20|0);
 $20 = HEAP32[$19>>2]|0;
 (_sqlite3VdbeAddOp3($14,79,0,$17,$20)|0);
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 40|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$pFunc>>2] = $23;
 HEAP32[$i>>2] = 0;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 44|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ($24|0)<($27|0);
  if (!($28)) {
   break;
  }
  $29 = HEAP32[$pFunc>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)>=(0);
  do {
   if ($32) {
    $33 = HEAP32[$pFunc>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$pE>>2] = $34;
    $35 = HEAP32[$pE>>2]|0;
    $36 = ((($35)) + 20|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)==(0|0);
    if (!($38)) {
     $39 = HEAP32[$pE>>2]|0;
     $40 = ((($39)) + 20|0);
     $41 = HEAP32[$40>>2]|0;
     $42 = HEAP32[$41>>2]|0;
     $43 = ($42|0)!=(1);
     if (!($43)) {
      $47 = HEAP32[$0>>2]|0;
      $48 = HEAP32[$pE>>2]|0;
      $49 = ((($48)) + 20|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = (_keyInfoFromExprList($47,$50,0,0)|0);
      HEAP32[$pKeyInfo>>2] = $51;
      $52 = HEAP32[$v>>2]|0;
      $53 = HEAP32[$pFunc>>2]|0;
      $54 = ((($53)) + 12|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$pKeyInfo>>2]|0;
      (_sqlite3VdbeAddOp4($52,107,$55,0,0,$56,-6)|0);
      break;
     }
    }
    $44 = HEAP32[$0>>2]|0;
    _sqlite3ErrorMsg($44,35757,$vararg_buffer);
    $45 = HEAP32[$pFunc>>2]|0;
    $46 = ((($45)) + 12|0);
    HEAP32[$46>>2] = -1;
   }
  } while(0);
  $57 = HEAP32[$i>>2]|0;
  $58 = (($57) + 1)|0;
  HEAP32[$i>>2] = $58;
  $59 = HEAP32[$pFunc>>2]|0;
  $60 = ((($59)) + 16|0);
  HEAP32[$pFunc>>2] = $60;
 }
 STACKTOP = sp;return;
}
function _isSimpleCount($p,$pAggInfo) {
 $p = $p|0;
 $pAggInfo = $pAggInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $pExpr = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pTab = sp + 4|0;
 $pExpr = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pAggInfo;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)!=(1);
  if (!($10)) {
   $11 = HEAP32[$1>>2]|0;
   $12 = ((($11)) + 28|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)!=(1);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 28|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ((($18)) + 8|0);
    $20 = ((($19)) + 20|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = ($21|0)!=(0|0);
    if (!($22)) {
     $23 = HEAP32[$1>>2]|0;
     $24 = ((($23)) + 28|0);
     $25 = HEAP32[$24>>2]|0;
     $26 = ((($25)) + 8|0);
     $27 = ((($26)) + 16|0);
     $28 = HEAP32[$27>>2]|0;
     HEAP32[$pTab>>2] = $28;
     $29 = HEAP32[$1>>2]|0;
     $30 = HEAP32[$29>>2]|0;
     $31 = ((($30)) + 4|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = HEAP32[$32>>2]|0;
     HEAP32[$pExpr>>2] = $33;
     $34 = HEAP32[$pTab>>2]|0;
     $35 = ((($34)) + 42|0);
     $36 = HEAP8[$35>>0]|0;
     $37 = $36&255;
     $38 = $37 & 16;
     $39 = ($38|0)!=(0);
     if ($39) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $40 = HEAP32[$pExpr>>2]|0;
     $41 = HEAP8[$40>>0]|0;
     $42 = $41&255;
     $43 = ($42|0)!=(153);
     if ($43) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $44 = HEAP32[$2>>2]|0;
     $45 = ((($44)) + 44|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ($46|0)==(0);
     if ($47) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $48 = HEAP32[$2>>2]|0;
     $49 = ((($48)) + 40|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = ((($50)) + 4|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ((($52)) + 2|0);
     $54 = HEAP16[$53>>1]|0;
     $55 = $54&65535;
     $56 = $55 & 256;
     $57 = ($56|0)==(0);
     if ($57) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
     $58 = HEAP32[$pExpr>>2]|0;
     $59 = ((($58)) + 4|0);
     $60 = HEAP32[$59>>2]|0;
     $61 = $60 & 16;
     $62 = ($61|0)!=(0);
     if ($62) {
      HEAP32[$0>>2] = 0;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     } else {
      $63 = HEAP32[$pTab>>2]|0;
      HEAP32[$0>>2] = $63;
      $64 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($64|0);
     }
    }
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _explainSimpleCount($pParse,$pTab,$pIdx) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $bCover = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zEqp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $bCover = sp + 16|0;
 $zEqp = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $pIdx;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 409|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(2);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 42|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = $13 & 32;
  $15 = ($14|0)==(0);
  if ($15) {
   $24 = 1;
  } else {
   $16 = HEAP32[$2>>2]|0;
   $17 = ((($16)) + 55|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 & 3;
   $20 = $19&255;
   $21 = ($20|0)==(2);
   $22 = $21 ^ 1;
   $24 = $22;
  }
 } else {
  $24 = 0;
 }
 $23 = $24&1;
 HEAP32[$bCover>>2] = $23;
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$bCover>>2]|0;
 $30 = ($29|0)!=(0);
 $31 = $30 ? 35716 : 18402;
 $32 = HEAP32[$bCover>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = HEAP32[$2>>2]|0;
  $35 = HEAP32[$34>>2]|0;
  $36 = $35;
 } else {
  $36 = 18402;
 }
 HEAP32[$vararg_buffer>>2] = $28;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $31;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $36;
 $37 = (_sqlite3MPrintf($26,35739,$vararg_buffer)|0);
 HEAP32[$zEqp>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 8|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 420|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$zEqp>>2]|0;
 (_sqlite3VdbeAddOp4($40,162,$43,0,0,$44,-1)|0);
 STACKTOP = sp;return;
}
function _minMaxQuery($pAggInfo,$ppMinMax) {
 $pAggInfo = $pAggInfo|0;
 $ppMinMax = $ppMinMax|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $eRet = 0, $pEList = 0, $pExpr = 0, $zFunc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $eRet = sp + 12|0;
 $pExpr = sp + 8|0;
 $pEList = sp + 4|0;
 $zFunc = sp;
 HEAP32[$0>>2] = $pAggInfo;
 HEAP32[$1>>2] = $ppMinMax;
 HEAP32[$eRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)==(1);
 do {
  if ($6) {
   $7 = HEAP32[$0>>2]|0;
   $8 = ((($7)) + 40|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   HEAP32[$pExpr>>2] = $10;
   $11 = HEAP32[$pExpr>>2]|0;
   $12 = ((($11)) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$pEList>>2] = $13;
   $14 = HEAP32[$pEList>>2]|0;
   $15 = ($14|0)!=(0|0);
   if ($15) {
    $16 = HEAP32[$pEList>>2]|0;
    $17 = HEAP32[$16>>2]|0;
    $18 = ($17|0)==(1);
    if ($18) {
     $19 = HEAP32[$pEList>>2]|0;
     $20 = ((($19)) + 4|0);
     $21 = HEAP32[$20>>2]|0;
     $22 = HEAP32[$21>>2]|0;
     $23 = HEAP8[$22>>0]|0;
     $24 = $23&255;
     $25 = ($24|0)==(154);
     if ($25) {
      $26 = HEAP32[$pExpr>>2]|0;
      $27 = ((($26)) + 8|0);
      $28 = HEAP32[$27>>2]|0;
      HEAP32[$zFunc>>2] = $28;
      $29 = HEAP32[$zFunc>>2]|0;
      $30 = (_sqlite3StrICmp($29,35708)|0);
      $31 = ($30|0)==(0);
      if ($31) {
       HEAP32[$eRet>>2] = 1;
       $32 = HEAP32[$pEList>>2]|0;
       $33 = HEAP32[$1>>2]|0;
       HEAP32[$33>>2] = $32;
       break;
      }
      $34 = HEAP32[$zFunc>>2]|0;
      $35 = (_sqlite3StrICmp($34,35712)|0);
      $36 = ($35|0)==(0);
      if ($36) {
       HEAP32[$eRet>>2] = 2;
       $37 = HEAP32[$pEList>>2]|0;
       $38 = HEAP32[$1>>2]|0;
       HEAP32[$38>>2] = $37;
      }
     }
    }
   }
  }
 } while(0);
 $39 = HEAP32[$eRet>>2]|0;
 $40 = $39&255;
 STACKTOP = sp;return ($40|0);
}
function _generateSortTail($pParse,$p,$pSort,$nColumn,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pSort = $pSort|0;
 $nColumn = $nColumn|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $addr = 0, $addrBreak = 0, $addrContinue = 0, $addrOnce = 0, $bSeq = 0, $eDest = 0, $i = 0, $iParm = 0, $iSortTab = 0, $iTab = 0, $nKey = 0, $nSortData = 0, $or$cond = 0, $or$cond3 = 0, $pOrderBy = 0, $regRow = 0, $regRowid = 0, $regSortOut = 0, $v = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $1 = sp + 80|0;
 $2 = sp + 76|0;
 $3 = sp + 72|0;
 $4 = sp + 68|0;
 $v = sp + 64|0;
 $addrBreak = sp + 60|0;
 $addrContinue = sp + 56|0;
 $addr = sp + 52|0;
 $addrOnce = sp + 48|0;
 $iTab = sp + 44|0;
 $pOrderBy = sp + 40|0;
 $eDest = sp + 36|0;
 $iParm = sp + 32|0;
 $regRow = sp + 28|0;
 $regRowid = sp + 24|0;
 $nKey = sp + 20|0;
 $iSortTab = sp + 16|0;
 $nSortData = sp + 12|0;
 $i = sp + 8|0;
 $bSeq = sp + 4|0;
 $regSortOut = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pSort;
 HEAP32[$3>>2] = $nColumn;
 HEAP32[$4>>2] = $pDest;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$addrBreak>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = (_sqlite3VdbeMakeLabel($11)|0);
 HEAP32[$addrContinue>>2] = $12;
 HEAP32[$addrOnce>>2] = 0;
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pOrderBy>>2] = $14;
 $15 = HEAP32[$4>>2]|0;
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 HEAP32[$eDest>>2] = $17;
 $18 = HEAP32[$4>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$iParm>>2] = $20;
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$v>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  (_sqlite3VdbeAddOp2($25,14,$28,$31)|0);
  $32 = HEAP32[$v>>2]|0;
  $33 = HEAP32[$addrBreak>>2]|0;
  (_sqlite3VdbeGoto($32,$33)|0);
  $34 = HEAP32[$v>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = ((($35)) + 16|0);
  $37 = HEAP32[$36>>2]|0;
  _sqlite3VdbeResolveLabel($34,$37);
 }
 $38 = HEAP32[$2>>2]|0;
 $39 = ((($38)) + 8|0);
 $40 = HEAP32[$39>>2]|0;
 HEAP32[$iTab>>2] = $40;
 $41 = HEAP32[$eDest>>2]|0;
 $42 = ($41|0)==(9);
 $43 = HEAP32[$eDest>>2]|0;
 $44 = ($43|0)==(13);
 $or$cond = $42 | $44;
 $45 = HEAP32[$eDest>>2]|0;
 $46 = ($45|0)==(10);
 $or$cond3 = $or$cond | $46;
 if ($or$cond3) {
  HEAP32[$regRowid>>2] = 0;
  $47 = HEAP32[$4>>2]|0;
  $48 = ((($47)) + 12|0);
  $49 = HEAP32[$48>>2]|0;
  HEAP32[$regRow>>2] = $49;
  $50 = HEAP32[$3>>2]|0;
  HEAP32[$nSortData>>2] = $50;
 } else {
  $51 = HEAP32[$0>>2]|0;
  $52 = (_sqlite3GetTempReg($51)|0);
  HEAP32[$regRowid>>2] = $52;
  $53 = HEAP32[$0>>2]|0;
  $54 = HEAP32[$3>>2]|0;
  $55 = (_sqlite3GetTempRange($53,$54)|0);
  HEAP32[$regRow>>2] = $55;
  $56 = HEAP32[$3>>2]|0;
  HEAP32[$nSortData>>2] = $56;
 }
 $57 = HEAP32[$pOrderBy>>2]|0;
 $58 = HEAP32[$57>>2]|0;
 $59 = HEAP32[$2>>2]|0;
 $60 = ((($59)) + 4|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = (($58) - ($61))|0;
 HEAP32[$nKey>>2] = $62;
 $63 = HEAP32[$2>>2]|0;
 $64 = ((($63)) + 28|0);
 $65 = HEAP8[$64>>0]|0;
 $66 = $65&255;
 $67 = $66 & 1;
 $68 = ($67|0)!=(0);
 if ($68) {
  $69 = HEAP32[$0>>2]|0;
  $70 = ((($69)) + 44|0);
  $71 = HEAP32[$70>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$70>>2] = $72;
  HEAP32[$regSortOut>>2] = $72;
  $73 = HEAP32[$0>>2]|0;
  $74 = ((($73)) + 40|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = (($75) + 1)|0;
  HEAP32[$74>>2] = $76;
  HEAP32[$iSortTab>>2] = $75;
  $77 = HEAP32[$2>>2]|0;
  $78 = ((($77)) + 16|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0);
  if ($80) {
   $81 = HEAP32[$v>>2]|0;
   $82 = (_sqlite3VdbeAddOp0($81,20)|0);
   HEAP32[$addrOnce>>2] = $82;
  }
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$iSortTab>>2]|0;
  $85 = HEAP32[$regSortOut>>2]|0;
  $86 = HEAP32[$nKey>>2]|0;
  $87 = (($86) + 1)|0;
  $88 = HEAP32[$nSortData>>2]|0;
  $89 = (($87) + ($88))|0;
  (_sqlite3VdbeAddOp3($83,110,$84,$85,$89)|0);
  $90 = HEAP32[$addrOnce>>2]|0;
  $91 = ($90|0)!=(0);
  if ($91) {
   $92 = HEAP32[$v>>2]|0;
   $93 = HEAP32[$addrOnce>>2]|0;
   _sqlite3VdbeJumpHere($92,$93);
  }
  $94 = HEAP32[$v>>2]|0;
  $95 = HEAP32[$iTab>>2]|0;
  $96 = HEAP32[$addrBreak>>2]|0;
  $97 = (_sqlite3VdbeAddOp2($94,55,$95,$96)|0);
  $98 = (1 + ($97))|0;
  HEAP32[$addr>>2] = $98;
  $99 = HEAP32[$v>>2]|0;
  $100 = HEAP32[$1>>2]|0;
  $101 = ((($100)) + 16|0);
  $102 = HEAP32[$101>>2]|0;
  $103 = HEAP32[$addrContinue>>2]|0;
  _codeOffset($99,$102,$103);
  $104 = HEAP32[$v>>2]|0;
  $105 = HEAP32[$iTab>>2]|0;
  $106 = HEAP32[$regSortOut>>2]|0;
  $107 = HEAP32[$iSortTab>>2]|0;
  (_sqlite3VdbeAddOp3($104,120,$105,$106,$107)|0);
  HEAP32[$bSeq>>2] = 0;
 } else {
  $108 = HEAP32[$v>>2]|0;
  $109 = HEAP32[$iTab>>2]|0;
  $110 = HEAP32[$addrBreak>>2]|0;
  $111 = (_sqlite3VdbeAddOp2($108,56,$109,$110)|0);
  $112 = (1 + ($111))|0;
  HEAP32[$addr>>2] = $112;
  $113 = HEAP32[$v>>2]|0;
  $114 = HEAP32[$1>>2]|0;
  $115 = ((($114)) + 16|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = HEAP32[$addrContinue>>2]|0;
  _codeOffset($113,$116,$117);
  $118 = HEAP32[$iTab>>2]|0;
  HEAP32[$iSortTab>>2] = $118;
  HEAP32[$bSeq>>2] = 1;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $119 = HEAP32[$i>>2]|0;
  $120 = HEAP32[$nSortData>>2]|0;
  $121 = ($119|0)<($120|0);
  if (!($121)) {
   break;
  }
  $122 = HEAP32[$v>>2]|0;
  $123 = HEAP32[$iSortTab>>2]|0;
  $124 = HEAP32[$nKey>>2]|0;
  $125 = HEAP32[$bSeq>>2]|0;
  $126 = (($124) + ($125))|0;
  $127 = HEAP32[$i>>2]|0;
  $128 = (($126) + ($127))|0;
  $129 = HEAP32[$regRow>>2]|0;
  $130 = HEAP32[$i>>2]|0;
  $131 = (($129) + ($130))|0;
  (_sqlite3VdbeAddOp3($122,96,$123,$128,$131)|0);
  $132 = HEAP32[$i>>2]|0;
  $133 = (($132) + 1)|0;
  HEAP32[$i>>2] = $133;
 }
 $134 = HEAP32[$eDest>>2]|0;
 L22: do {
  switch ($134|0) {
  case 12:  {
   $135 = HEAP32[$v>>2]|0;
   $136 = HEAP32[$iParm>>2]|0;
   $137 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp2($135,114,$136,$137)|0);
   $138 = HEAP32[$v>>2]|0;
   $139 = HEAP32[$iParm>>2]|0;
   $140 = HEAP32[$regRow>>2]|0;
   $141 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp3($138,115,$139,$140,$141)|0);
   $142 = HEAP32[$v>>2]|0;
   _sqlite3VdbeChangeP5($142,8);
   break;
  }
  case 11:  {
   $143 = HEAP32[$v>>2]|0;
   $144 = HEAP32[$regRow>>2]|0;
   $145 = HEAP32[$3>>2]|0;
   $146 = HEAP32[$regRowid>>2]|0;
   $147 = HEAP32[$4>>2]|0;
   $148 = ((($147)) + 4|0);
   $149 = HEAP32[$148>>2]|0;
   $150 = HEAP32[$3>>2]|0;
   (_sqlite3VdbeAddOp4($143,99,$144,$145,$146,$149,$150)|0);
   $151 = HEAP32[$0>>2]|0;
   $152 = HEAP32[$regRow>>2]|0;
   $153 = HEAP32[$3>>2]|0;
   _sqlite3ExprCacheAffinityChange($151,$152,$153);
   $154 = HEAP32[$v>>2]|0;
   $155 = HEAP32[$iParm>>2]|0;
   $156 = HEAP32[$regRowid>>2]|0;
   (_sqlite3VdbeAddOp2($154,126,$155,$156)|0);
   break;
  }
  case 10:  {
   break;
  }
  default: {
   $157 = HEAP32[$eDest>>2]|0;
   $158 = ($157|0)==(9);
   $159 = HEAP32[$v>>2]|0;
   $160 = HEAP32[$4>>2]|0;
   if ($158) {
    $161 = ((($160)) + 12|0);
    $162 = HEAP32[$161>>2]|0;
    $163 = HEAP32[$3>>2]|0;
    (_sqlite3VdbeAddOp2($159,87,$162,$163)|0);
    $164 = HEAP32[$0>>2]|0;
    $165 = HEAP32[$4>>2]|0;
    $166 = ((($165)) + 12|0);
    $167 = HEAP32[$166>>2]|0;
    $168 = HEAP32[$3>>2]|0;
    _sqlite3ExprCacheAffinityChange($164,$167,$168);
    break L22;
   } else {
    $169 = ((($160)) + 8|0);
    $170 = HEAP32[$169>>2]|0;
    (_sqlite3VdbeAddOp1($159,16,$170)|0);
    break L22;
   }
  }
  }
 } while(0);
 $171 = HEAP32[$regRowid>>2]|0;
 $172 = ($171|0)!=(0);
 if ($172) {
  $173 = HEAP32[$eDest>>2]|0;
  $174 = ($173|0)==(11);
  $175 = HEAP32[$0>>2]|0;
  $176 = HEAP32[$regRow>>2]|0;
  if ($174) {
   $177 = HEAP32[$3>>2]|0;
   _sqlite3ReleaseTempRange($175,$176,$177);
  } else {
   _sqlite3ReleaseTempReg($175,$176);
  }
  $178 = HEAP32[$0>>2]|0;
  $179 = HEAP32[$regRowid>>2]|0;
  _sqlite3ReleaseTempReg($178,$179);
 }
 $180 = HEAP32[$v>>2]|0;
 $181 = HEAP32[$addrContinue>>2]|0;
 _sqlite3VdbeResolveLabel($180,$181);
 $182 = HEAP32[$2>>2]|0;
 $183 = ((($182)) + 28|0);
 $184 = HEAP8[$183>>0]|0;
 $185 = $184&255;
 $186 = $185 & 1;
 $187 = ($186|0)!=(0);
 $188 = HEAP32[$v>>2]|0;
 $189 = HEAP32[$iTab>>2]|0;
 $190 = HEAP32[$addr>>2]|0;
 if ($187) {
  (_sqlite3VdbeAddOp2($188,3,$189,$190)|0);
 } else {
  (_sqlite3VdbeAddOp2($188,7,$189,$190)|0);
 }
 $191 = HEAP32[$2>>2]|0;
 $192 = ((($191)) + 12|0);
 $193 = HEAP32[$192>>2]|0;
 $194 = ($193|0)!=(0);
 if (!($194)) {
  $199 = HEAP32[$v>>2]|0;
  $200 = HEAP32[$addrBreak>>2]|0;
  _sqlite3VdbeResolveLabel($199,$200);
  STACKTOP = sp;return;
 }
 $195 = HEAP32[$v>>2]|0;
 $196 = HEAP32[$2>>2]|0;
 $197 = ((($196)) + 12|0);
 $198 = HEAP32[$197>>2]|0;
 (_sqlite3VdbeAddOp1($195,72,$198)|0);
 $199 = HEAP32[$v>>2]|0;
 $200 = HEAP32[$addrBreak>>2]|0;
 _sqlite3VdbeResolveLabel($199,$200);
 STACKTOP = sp;return;
}
function _generateColumnNames($pParse,$pTabList,$pEList) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $fullNames = 0, $i = 0, $iCol = 0, $j = 0, $or$cond = 0, $p = 0, $pTab = 0, $shortNames = 0, $v = 0, $vararg_buffer = 0;
 var $vararg_buffer3 = 0, $vararg_ptr2 = 0, $z = 0, $zCol = 0, $zName = 0, $zName1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $v = sp + 60|0;
 $i = sp + 56|0;
 $j = sp + 52|0;
 $db = sp + 48|0;
 $fullNames = sp + 44|0;
 $shortNames = sp + 40|0;
 $p = sp + 36|0;
 $zName = sp + 32|0;
 $pTab = sp + 28|0;
 $zCol = sp + 24|0;
 $iCol = sp + 20|0;
 $zName1 = sp + 16|0;
 $z = sp + 12|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 409|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = ($15|0)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$db>>2]|0;
 $18 = ((($17)) + 69|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($20|0)!=(0);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 16|0);
 HEAP8[$23>>0] = 1;
 $24 = HEAP32[$db>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = $26 & 4;
 $28 = ($27|0)!=(0);
 $29 = $28&1;
 HEAP32[$fullNames>>2] = $29;
 $30 = HEAP32[$db>>2]|0;
 $31 = ((($30)) + 24|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = $32 & 64;
 $34 = ($33|0)!=(0);
 $35 = $34&1;
 HEAP32[$shortNames>>2] = $35;
 $36 = HEAP32[$v>>2]|0;
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 _sqlite3VdbeSetNumCols($36,$38);
 HEAP32[$i>>2] = 0;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = HEAP32[$40>>2]|0;
  $42 = ($39|0)<($41|0);
  if (!($42)) {
   break;
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = HEAP32[$2>>2]|0;
  $45 = ((($44)) + 4|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + (($43*20)|0)|0);
  $48 = HEAP32[$47>>2]|0;
  HEAP32[$p>>2] = $48;
  $49 = HEAP32[$p>>2]|0;
  $50 = ($49|0)==(0|0);
  do {
   if (!($50)) {
    $51 = HEAP32[$i>>2]|0;
    $52 = HEAP32[$2>>2]|0;
    $53 = ((($52)) + 4|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($54) + (($51*20)|0)|0);
    $56 = ((($55)) + 4|0);
    $57 = HEAP32[$56>>2]|0;
    $58 = ($57|0)!=(0|0);
    if ($58) {
     $59 = HEAP32[$i>>2]|0;
     $60 = HEAP32[$2>>2]|0;
     $61 = ((($60)) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = (($62) + (($59*20)|0)|0);
     $64 = ((($63)) + 4|0);
     $65 = HEAP32[$64>>2]|0;
     HEAP32[$zName>>2] = $65;
     $66 = HEAP32[$v>>2]|0;
     $67 = HEAP32[$i>>2]|0;
     $68 = HEAP32[$zName>>2]|0;
     (_sqlite3VdbeSetColName($66,$67,0,$68,(-1))|0);
     break;
    }
    $69 = HEAP32[$p>>2]|0;
    $70 = HEAP8[$69>>0]|0;
    $71 = $70&255;
    $72 = ($71|0)==(152);
    if (!($72)) {
     $73 = HEAP32[$p>>2]|0;
     $74 = HEAP8[$73>>0]|0;
     $75 = $74&255;
     $76 = ($75|0)==(154);
     if (!($76)) {
      $145 = HEAP32[$i>>2]|0;
      $146 = HEAP32[$2>>2]|0;
      $147 = ((($146)) + 4|0);
      $148 = HEAP32[$147>>2]|0;
      $149 = (($148) + (($145*20)|0)|0);
      $150 = ((($149)) + 8|0);
      $151 = HEAP32[$150>>2]|0;
      HEAP32[$z>>2] = $151;
      $152 = HEAP32[$z>>2]|0;
      $153 = ($152|0)==(0|0);
      $154 = HEAP32[$db>>2]|0;
      if ($153) {
       $155 = HEAP32[$i>>2]|0;
       $156 = (($155) + 1)|0;
       HEAP32[$vararg_buffer3>>2] = $156;
       $157 = (_sqlite3MPrintf($154,35699,$vararg_buffer3)|0);
       $160 = $157;
      } else {
       $158 = HEAP32[$z>>2]|0;
       $159 = (_sqlite3DbStrDup($154,$158)|0);
       $160 = $159;
      }
      HEAP32[$z>>2] = $160;
      $161 = HEAP32[$v>>2]|0;
      $162 = HEAP32[$i>>2]|0;
      $163 = HEAP32[$z>>2]|0;
      (_sqlite3VdbeSetColName($161,$162,0,$163,170)|0);
      break;
     }
    }
    $77 = HEAP32[$p>>2]|0;
    $78 = ((($77)) + 32|0);
    $79 = HEAP16[$78>>1]|0;
    $80 = $79 << 16 >> 16;
    HEAP32[$iCol>>2] = $80;
    HEAP32[$j>>2] = 0;
    while(1) {
     $81 = HEAP32[$j>>2]|0;
     $82 = HEAP32[$1>>2]|0;
     $83 = HEAP32[$82>>2]|0;
     $84 = ($81|0)<($83|0);
     if (!($84)) {
      break;
     }
     $85 = HEAP32[$j>>2]|0;
     $86 = HEAP32[$1>>2]|0;
     $87 = ((($86)) + 8|0);
     $88 = (($87) + (($85*72)|0)|0);
     $89 = ((($88)) + 44|0);
     $90 = HEAP32[$89>>2]|0;
     $91 = HEAP32[$p>>2]|0;
     $92 = ((($91)) + 28|0);
     $93 = HEAP32[$92>>2]|0;
     $94 = ($90|0)==($93|0);
     if ($94) {
      break;
     }
     $95 = HEAP32[$j>>2]|0;
     $96 = (($95) + 1)|0;
     HEAP32[$j>>2] = $96;
    }
    $97 = HEAP32[$j>>2]|0;
    $98 = HEAP32[$1>>2]|0;
    $99 = ((($98)) + 8|0);
    $100 = (($99) + (($97*72)|0)|0);
    $101 = ((($100)) + 16|0);
    $102 = HEAP32[$101>>2]|0;
    HEAP32[$pTab>>2] = $102;
    $103 = HEAP32[$iCol>>2]|0;
    $104 = ($103|0)<(0);
    if ($104) {
     $105 = HEAP32[$pTab>>2]|0;
     $106 = ((($105)) + 32|0);
     $107 = HEAP16[$106>>1]|0;
     $108 = $107 << 16 >> 16;
     HEAP32[$iCol>>2] = $108;
    }
    $109 = HEAP32[$iCol>>2]|0;
    $110 = ($109|0)<(0);
    if ($110) {
     HEAP32[$zCol>>2] = 24745;
    } else {
     $111 = HEAP32[$iCol>>2]|0;
     $112 = HEAP32[$pTab>>2]|0;
     $113 = ((($112)) + 4|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + ($111<<4)|0);
     $116 = HEAP32[$115>>2]|0;
     HEAP32[$zCol>>2] = $116;
    }
    $117 = HEAP32[$shortNames>>2]|0;
    $118 = ($117|0)!=(0);
    $119 = HEAP32[$fullNames>>2]|0;
    $120 = ($119|0)!=(0);
    $or$cond = $118 | $120;
    if (!($or$cond)) {
     $121 = HEAP32[$v>>2]|0;
     $122 = HEAP32[$i>>2]|0;
     $123 = HEAP32[$db>>2]|0;
     $124 = HEAP32[$i>>2]|0;
     $125 = HEAP32[$2>>2]|0;
     $126 = ((($125)) + 4|0);
     $127 = HEAP32[$126>>2]|0;
     $128 = (($127) + (($124*20)|0)|0);
     $129 = ((($128)) + 8|0);
     $130 = HEAP32[$129>>2]|0;
     $131 = (_sqlite3DbStrDup($123,$130)|0);
     (_sqlite3VdbeSetColName($121,$122,0,$131,170)|0);
     break;
    }
    $132 = HEAP32[$fullNames>>2]|0;
    $133 = ($132|0)!=(0);
    if ($133) {
     HEAP32[$zName1>>2] = 0;
     $134 = HEAP32[$db>>2]|0;
     $135 = HEAP32[$pTab>>2]|0;
     $136 = HEAP32[$135>>2]|0;
     $137 = HEAP32[$zCol>>2]|0;
     HEAP32[$vararg_buffer>>2] = $136;
     $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
     HEAP32[$vararg_ptr2>>2] = $137;
     $138 = (_sqlite3MPrintf($134,30161,$vararg_buffer)|0);
     HEAP32[$zName1>>2] = $138;
     $139 = HEAP32[$v>>2]|0;
     $140 = HEAP32[$i>>2]|0;
     $141 = HEAP32[$zName1>>2]|0;
     (_sqlite3VdbeSetColName($139,$140,0,$141,170)|0);
     break;
    } else {
     $142 = HEAP32[$v>>2]|0;
     $143 = HEAP32[$i>>2]|0;
     $144 = HEAP32[$zCol>>2]|0;
     (_sqlite3VdbeSetColName($142,$143,0,$144,(-1))|0);
     break;
    }
   }
  } while(0);
  $164 = HEAP32[$i>>2]|0;
  $165 = (($164) + 1)|0;
  HEAP32[$i>>2] = $165;
 }
 $166 = HEAP32[$0>>2]|0;
 $167 = HEAP32[$1>>2]|0;
 $168 = HEAP32[$2>>2]|0;
 _generateColumnTypes($166,$167,$168);
 STACKTOP = sp;return;
}
function _generateColumnTypes($pParse,$pTabList,$pEList) {
 $pParse = $pParse|0;
 $pTabList = $pTabList|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $sNC = 0, $v = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $v = sp + 44|0;
 $i = sp + 40|0;
 $sNC = sp + 8|0;
 $p = sp + 4|0;
 $zType = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTabList;
 HEAP32[$2>>2] = $pEList;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($sNC)) + 4|0);
 HEAP32[$7>>2] = $6;
 $8 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $8;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = ($9|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + (($13*20)|0)|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$p>>2] = $18;
  $19 = HEAP32[$p>>2]|0;
  $20 = (_columnTypeImpl($sNC,$19,0)|0);
  HEAP32[$zType>>2] = $20;
  $21 = HEAP32[$v>>2]|0;
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$zType>>2]|0;
  (_sqlite3VdbeSetColName($21,$22,1,$23,(-1))|0);
  $24 = HEAP32[$i>>2]|0;
  $25 = (($24) + 1)|0;
  HEAP32[$i>>2] = $25;
 }
 STACKTOP = sp;return;
}
function _codeOffset($v,$iOffset,$iContinue) {
 $v = $v|0;
 $iOffset = $iOffset|0;
 $iContinue = $iContinue|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $v;
 HEAP32[$1>>2] = $iOffset;
 HEAP32[$2>>2] = $iContinue;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)>(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 (_sqlite3VdbeAddOp3($5,66,$6,$7,1)|0);
 STACKTOP = sp;return;
}
function _codeDistinct($pParse,$iTab,$addrRepeat,$N,$iMem) {
 $pParse = $pParse|0;
 $iTab = $iTab|0;
 $addrRepeat = $addrRepeat|0;
 $N = $N|0;
 $iMem = $iMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r1 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $v = sp + 4|0;
 $r1 = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $iTab;
 HEAP32[$2>>2] = $addrRepeat;
 HEAP32[$3>>2] = $N;
 HEAP32[$4>>2] = $iMem;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$v>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = (_sqlite3GetTempReg($8)|0);
 HEAP32[$r1>>2] = $9;
 $10 = HEAP32[$v>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$4>>2]|0;
 $14 = HEAP32[$3>>2]|0;
 (_sqlite3VdbeAddOp4Int($10,31,$11,$12,$13,$14)|0);
 $15 = HEAP32[$v>>2]|0;
 $16 = HEAP32[$4>>2]|0;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp3($15,99,$16,$17,$18)|0);
 $19 = HEAP32[$v>>2]|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$r1>>2]|0;
 (_sqlite3VdbeAddOp2($19,126,$20,$21)|0);
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$r1>>2]|0;
 _sqlite3ReleaseTempReg($22,$23);
 STACKTOP = sp;return;
}
function _analyzeAggregate($pWalker,$pExpr) {
 $pWalker = $pWalker|0;
 $pExpr = $pExpr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $enc = 0, $i = 0, $j = 0, $k = 0, $n = 0, $pAggInfo = 0, $pCol = 0, $pE = 0, $pGB = 0, $pItem = 0, $pItem1 = 0, $pNC = 0, $pParse = 0, $pSrcList = 0, $pTerm = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $i = sp + 52|0;
 $pNC = sp + 48|0;
 $pParse = sp + 44|0;
 $pSrcList = sp + 40|0;
 $pAggInfo = sp + 36|0;
 $pItem = sp + 32|0;
 $pCol = sp + 28|0;
 $k = sp + 24|0;
 $j = sp + 20|0;
 $n = sp + 16|0;
 $pGB = sp + 12|0;
 $pTerm = sp + 8|0;
 $pE = sp + 4|0;
 $pItem1 = sp;
 $enc = sp + 68|0;
 HEAP32[$1>>2] = $pWalker;
 HEAP32[$2>>2] = $pExpr;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pNC>>2] = $5;
 $6 = HEAP32[$pNC>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pParse>>2] = $7;
 $8 = HEAP32[$pNC>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pSrcList>>2] = $10;
 $11 = HEAP32[$pNC>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pAggInfo>>2] = $13;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 switch ($16|0) {
 case 152: case 154:  {
  $17 = HEAP32[$pSrcList>>2]|0;
  $18 = ($17|0)!=(0|0);
  L5: do {
   if ($18) {
    $19 = HEAP32[$pSrcList>>2]|0;
    $20 = ((($19)) + 8|0);
    HEAP32[$pItem>>2] = $20;
    HEAP32[$i>>2] = 0;
    while(1) {
     $21 = HEAP32[$i>>2]|0;
     $22 = HEAP32[$pSrcList>>2]|0;
     $23 = HEAP32[$22>>2]|0;
     $24 = ($21|0)<($23|0);
     if (!($24)) {
      break L5;
     }
     $25 = HEAP32[$2>>2]|0;
     $26 = ((($25)) + 28|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = HEAP32[$pItem>>2]|0;
     $29 = ((($28)) + 44|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($27|0)==($30|0);
     if ($31) {
      break;
     }
     $162 = HEAP32[$i>>2]|0;
     $163 = (($162) + 1)|0;
     HEAP32[$i>>2] = $163;
     $164 = HEAP32[$pItem>>2]|0;
     $165 = ((($164)) + 72|0);
     HEAP32[$pItem>>2] = $165;
    }
    $32 = HEAP32[$pAggInfo>>2]|0;
    $33 = ((($32)) + 28|0);
    $34 = HEAP32[$33>>2]|0;
    HEAP32[$pCol>>2] = $34;
    HEAP32[$k>>2] = 0;
    while(1) {
     $35 = HEAP32[$k>>2]|0;
     $36 = HEAP32[$pAggInfo>>2]|0;
     $37 = ((($36)) + 32|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = ($35|0)<($38|0);
     if (!($39)) {
      break;
     }
     $40 = HEAP32[$pCol>>2]|0;
     $41 = ((($40)) + 4|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = HEAP32[$2>>2]|0;
     $44 = ((($43)) + 28|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($42|0)==($45|0);
     if ($46) {
      $47 = HEAP32[$pCol>>2]|0;
      $48 = ((($47)) + 8|0);
      $49 = HEAP32[$48>>2]|0;
      $50 = HEAP32[$2>>2]|0;
      $51 = ((($50)) + 32|0);
      $52 = HEAP16[$51>>1]|0;
      $53 = $52 << 16 >> 16;
      $54 = ($49|0)==($53|0);
      if ($54) {
       break;
      }
     }
     $55 = HEAP32[$k>>2]|0;
     $56 = (($55) + 1)|0;
     HEAP32[$k>>2] = $56;
     $57 = HEAP32[$pCol>>2]|0;
     $58 = ((($57)) + 24|0);
     HEAP32[$pCol>>2] = $58;
    }
    $59 = HEAP32[$k>>2]|0;
    $60 = HEAP32[$pAggInfo>>2]|0;
    $61 = ((($60)) + 32|0);
    $62 = HEAP32[$61>>2]|0;
    $63 = ($59|0)>=($62|0);
    if ($63) {
     $64 = HEAP32[$pParse>>2]|0;
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$pAggInfo>>2]|0;
     $67 = (_addAggInfoColumn($65,$66)|0);
     HEAP32[$k>>2] = $67;
     $68 = ($67|0)>=(0);
     if ($68) {
      $69 = HEAP32[$k>>2]|0;
      $70 = HEAP32[$pAggInfo>>2]|0;
      $71 = ((($70)) + 28|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = (($72) + (($69*24)|0)|0);
      HEAP32[$pCol>>2] = $73;
      $74 = HEAP32[$2>>2]|0;
      $75 = ((($74)) + 44|0);
      $76 = HEAP32[$75>>2]|0;
      $77 = HEAP32[$pCol>>2]|0;
      HEAP32[$77>>2] = $76;
      $78 = HEAP32[$2>>2]|0;
      $79 = ((($78)) + 28|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = HEAP32[$pCol>>2]|0;
      $82 = ((($81)) + 4|0);
      HEAP32[$82>>2] = $80;
      $83 = HEAP32[$2>>2]|0;
      $84 = ((($83)) + 32|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85 << 16 >> 16;
      $87 = HEAP32[$pCol>>2]|0;
      $88 = ((($87)) + 8|0);
      HEAP32[$88>>2] = $86;
      $89 = HEAP32[$pParse>>2]|0;
      $90 = ((($89)) + 44|0);
      $91 = HEAP32[$90>>2]|0;
      $92 = (($91) + 1)|0;
      HEAP32[$90>>2] = $92;
      $93 = HEAP32[$pCol>>2]|0;
      $94 = ((($93)) + 16|0);
      HEAP32[$94>>2] = $92;
      $95 = HEAP32[$pCol>>2]|0;
      $96 = ((($95)) + 12|0);
      HEAP32[$96>>2] = -1;
      $97 = HEAP32[$2>>2]|0;
      $98 = HEAP32[$pCol>>2]|0;
      $99 = ((($98)) + 20|0);
      HEAP32[$99>>2] = $97;
      $100 = HEAP32[$pAggInfo>>2]|0;
      $101 = ((($100)) + 24|0);
      $102 = HEAP32[$101>>2]|0;
      $103 = ($102|0)!=(0|0);
      L22: do {
       if ($103) {
        $104 = HEAP32[$pAggInfo>>2]|0;
        $105 = ((($104)) + 24|0);
        $106 = HEAP32[$105>>2]|0;
        HEAP32[$pGB>>2] = $106;
        $107 = HEAP32[$pGB>>2]|0;
        $108 = ((($107)) + 4|0);
        $109 = HEAP32[$108>>2]|0;
        HEAP32[$pTerm>>2] = $109;
        $110 = HEAP32[$pGB>>2]|0;
        $111 = HEAP32[$110>>2]|0;
        HEAP32[$n>>2] = $111;
        HEAP32[$j>>2] = 0;
        while(1) {
         $112 = HEAP32[$j>>2]|0;
         $113 = HEAP32[$n>>2]|0;
         $114 = ($112|0)<($113|0);
         if (!($114)) {
          break L22;
         }
         $115 = HEAP32[$pTerm>>2]|0;
         $116 = HEAP32[$115>>2]|0;
         HEAP32[$pE>>2] = $116;
         $117 = HEAP32[$pE>>2]|0;
         $118 = HEAP8[$117>>0]|0;
         $119 = $118&255;
         $120 = ($119|0)==(152);
         if ($120) {
          $121 = HEAP32[$pE>>2]|0;
          $122 = ((($121)) + 28|0);
          $123 = HEAP32[$122>>2]|0;
          $124 = HEAP32[$2>>2]|0;
          $125 = ((($124)) + 28|0);
          $126 = HEAP32[$125>>2]|0;
          $127 = ($123|0)==($126|0);
          if ($127) {
           $128 = HEAP32[$pE>>2]|0;
           $129 = ((($128)) + 32|0);
           $130 = HEAP16[$129>>1]|0;
           $131 = $130 << 16 >> 16;
           $132 = HEAP32[$2>>2]|0;
           $133 = ((($132)) + 32|0);
           $134 = HEAP16[$133>>1]|0;
           $135 = $134 << 16 >> 16;
           $136 = ($131|0)==($135|0);
           if ($136) {
            break;
           }
          }
         }
         $140 = HEAP32[$j>>2]|0;
         $141 = (($140) + 1)|0;
         HEAP32[$j>>2] = $141;
         $142 = HEAP32[$pTerm>>2]|0;
         $143 = ((($142)) + 20|0);
         HEAP32[$pTerm>>2] = $143;
        }
        $137 = HEAP32[$j>>2]|0;
        $138 = HEAP32[$pCol>>2]|0;
        $139 = ((($138)) + 12|0);
        HEAP32[$139>>2] = $137;
       }
      } while(0);
      $144 = HEAP32[$pCol>>2]|0;
      $145 = ((($144)) + 12|0);
      $146 = HEAP32[$145>>2]|0;
      $147 = ($146|0)<(0);
      if ($147) {
       $148 = HEAP32[$pAggInfo>>2]|0;
       $149 = ((($148)) + 12|0);
       $150 = HEAP32[$149>>2]|0;
       $151 = (($150) + 1)|0;
       HEAP32[$149>>2] = $151;
       $152 = HEAP32[$pCol>>2]|0;
       $153 = ((($152)) + 12|0);
       HEAP32[$153>>2] = $150;
      }
     }
    }
    $154 = HEAP32[$pAggInfo>>2]|0;
    $155 = HEAP32[$2>>2]|0;
    $156 = ((($155)) + 40|0);
    HEAP32[$156>>2] = $154;
    $157 = HEAP32[$2>>2]|0;
    HEAP8[$157>>0] = -102;
    $158 = HEAP32[$k>>2]|0;
    $159 = $158&65535;
    $160 = HEAP32[$2>>2]|0;
    $161 = ((($160)) + 34|0);
    HEAP16[$161>>1] = $159;
   }
  } while(0);
  HEAP32[$0>>2] = 1;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
  break;
 }
 case 153:  {
  $166 = HEAP32[$pNC>>2]|0;
  $167 = ((($166)) + 28|0);
  $168 = HEAP16[$167>>1]|0;
  $169 = $168&65535;
  $170 = $169 & 8;
  $171 = ($170|0)==(0);
  if ($171) {
   $172 = HEAP32[$1>>2]|0;
   $173 = ((($172)) + 16|0);
   $174 = HEAP32[$173>>2]|0;
   $175 = HEAP32[$2>>2]|0;
   $176 = ((($175)) + 38|0);
   $177 = HEAP8[$176>>0]|0;
   $178 = $177&255;
   $179 = ($174|0)==($178|0);
   if ($179) {
    $180 = HEAP32[$pAggInfo>>2]|0;
    $181 = ((($180)) + 40|0);
    $182 = HEAP32[$181>>2]|0;
    HEAP32[$pItem1>>2] = $182;
    HEAP32[$i>>2] = 0;
    while(1) {
     $183 = HEAP32[$i>>2]|0;
     $184 = HEAP32[$pAggInfo>>2]|0;
     $185 = ((($184)) + 44|0);
     $186 = HEAP32[$185>>2]|0;
     $187 = ($183|0)<($186|0);
     if (!($187)) {
      break;
     }
     $188 = HEAP32[$pItem1>>2]|0;
     $189 = HEAP32[$188>>2]|0;
     $190 = HEAP32[$2>>2]|0;
     $191 = (_sqlite3ExprCompare($189,$190,-1)|0);
     $192 = ($191|0)==(0);
     if ($192) {
      break;
     }
     $193 = HEAP32[$i>>2]|0;
     $194 = (($193) + 1)|0;
     HEAP32[$i>>2] = $194;
     $195 = HEAP32[$pItem1>>2]|0;
     $196 = ((($195)) + 16|0);
     HEAP32[$pItem1>>2] = $196;
    }
    $197 = HEAP32[$i>>2]|0;
    $198 = HEAP32[$pAggInfo>>2]|0;
    $199 = ((($198)) + 44|0);
    $200 = HEAP32[$199>>2]|0;
    $201 = ($197|0)>=($200|0);
    do {
     if ($201) {
      $202 = HEAP32[$pParse>>2]|0;
      $203 = HEAP32[$202>>2]|0;
      $204 = ((($203)) + 66|0);
      $205 = HEAP8[$204>>0]|0;
      HEAP8[$enc>>0] = $205;
      $206 = HEAP32[$pParse>>2]|0;
      $207 = HEAP32[$206>>2]|0;
      $208 = HEAP32[$pAggInfo>>2]|0;
      $209 = (_addAggInfoFunc($207,$208)|0);
      HEAP32[$i>>2] = $209;
      $210 = HEAP32[$i>>2]|0;
      $211 = ($210|0)>=(0);
      if ($211) {
       $212 = HEAP32[$i>>2]|0;
       $213 = HEAP32[$pAggInfo>>2]|0;
       $214 = ((($213)) + 40|0);
       $215 = HEAP32[$214>>2]|0;
       $216 = (($215) + ($212<<4)|0);
       HEAP32[$pItem1>>2] = $216;
       $217 = HEAP32[$2>>2]|0;
       $218 = HEAP32[$pItem1>>2]|0;
       HEAP32[$218>>2] = $217;
       $219 = HEAP32[$pParse>>2]|0;
       $220 = ((($219)) + 44|0);
       $221 = HEAP32[$220>>2]|0;
       $222 = (($221) + 1)|0;
       HEAP32[$220>>2] = $222;
       $223 = HEAP32[$pItem1>>2]|0;
       $224 = ((($223)) + 8|0);
       HEAP32[$224>>2] = $222;
       $225 = HEAP32[$pParse>>2]|0;
       $226 = HEAP32[$225>>2]|0;
       $227 = HEAP32[$2>>2]|0;
       $228 = ((($227)) + 8|0);
       $229 = HEAP32[$228>>2]|0;
       $230 = HEAP32[$2>>2]|0;
       $231 = ((($230)) + 20|0);
       $232 = HEAP32[$231>>2]|0;
       $233 = ($232|0)!=(0|0);
       if ($233) {
        $234 = HEAP32[$2>>2]|0;
        $235 = ((($234)) + 20|0);
        $236 = HEAP32[$235>>2]|0;
        $237 = HEAP32[$236>>2]|0;
        $239 = $237;
       } else {
        $239 = 0;
       }
       $238 = HEAP8[$enc>>0]|0;
       $240 = (_sqlite3FindFunction($226,$229,$239,$238,0)|0);
       $241 = HEAP32[$pItem1>>2]|0;
       $242 = ((($241)) + 4|0);
       HEAP32[$242>>2] = $240;
       $243 = HEAP32[$2>>2]|0;
       $244 = ((($243)) + 4|0);
       $245 = HEAP32[$244>>2]|0;
       $246 = $245 & 16;
       $247 = ($246|0)!=(0);
       if ($247) {
        $248 = HEAP32[$pParse>>2]|0;
        $249 = ((($248)) + 40|0);
        $250 = HEAP32[$249>>2]|0;
        $251 = (($250) + 1)|0;
        HEAP32[$249>>2] = $251;
        $252 = HEAP32[$pItem1>>2]|0;
        $253 = ((($252)) + 12|0);
        HEAP32[$253>>2] = $250;
        break;
       } else {
        $254 = HEAP32[$pItem1>>2]|0;
        $255 = ((($254)) + 12|0);
        HEAP32[$255>>2] = -1;
        break;
       }
      }
     }
    } while(0);
    $256 = HEAP32[$i>>2]|0;
    $257 = $256&65535;
    $258 = HEAP32[$2>>2]|0;
    $259 = ((($258)) + 34|0);
    HEAP16[$259>>1] = $257;
    $260 = HEAP32[$pAggInfo>>2]|0;
    $261 = HEAP32[$2>>2]|0;
    $262 = ((($261)) + 40|0);
    HEAP32[$262>>2] = $260;
    HEAP32[$0>>2] = 1;
    $263 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($263|0);
   }
  }
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
  break;
 }
 default: {
  HEAP32[$0>>2] = 0;
  $263 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($263|0);
 }
 }
 return (0)|0;
}
function _analyzeAggregatesInSelect($pWalker,$pSelect) {
 $pWalker = $pWalker|0;
 $pSelect = $pSelect|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWalker;
 HEAP32[$1>>2] = $pSelect;
 STACKTOP = sp;return 0;
}
function _addAggInfoColumn($db,$pInfo) {
 $db = $db|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 32|0);
 $8 = (_sqlite3ArrayAllocate($2,$5,24,$7,$i)|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 28|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _addAggInfoFunc($db,$pInfo) {
 $db = $db|0;
 $pInfo = $pInfo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pInfo;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = (_sqlite3ArrayAllocate($2,$5,16,$7,$i)|0);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 40|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$i>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _pushOntoSorter($pParse,$pSort,$pSelect,$regData,$regOrigData,$nData,$nPrefixReg) {
 $pParse = $pParse|0;
 $pSort = $pSort|0;
 $pSelect = $pSelect|0;
 $regData = $regData|0;
 $regOrigData = $regOrigData|0;
 $nData = $nData|0;
 $nPrefixReg = $nPrefixReg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addrFirst = 0, $addrJmp = 0, $bSeq = 0, $iBrk = 0, $iLimit = 0, $nBase = 0, $nExpr = 0, $nKey = 0;
 var $nOBSat = 0, $op = 0, $pKI = 0, $pOp = 0, $r1 = 0, $regBase = 0, $regPrevKey = 0, $regRecord = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 76|0;
 $6 = sp + 72|0;
 $v = sp + 68|0;
 $bSeq = sp + 64|0;
 $nExpr = sp + 60|0;
 $nBase = sp + 56|0;
 $regBase = sp + 52|0;
 $regRecord = sp + 48|0;
 $nOBSat = sp + 44|0;
 $op = sp + 40|0;
 $iLimit = sp + 36|0;
 $regPrevKey = sp + 32|0;
 $addrFirst = sp + 28|0;
 $addrJmp = sp + 24|0;
 $pOp = sp + 20|0;
 $nKey = sp + 16|0;
 $pKI = sp + 12|0;
 $addr = sp + 8|0;
 $r1 = sp + 4|0;
 $iBrk = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pSort;
 HEAP32[$2>>2] = $pSelect;
 HEAP32[$3>>2] = $regData;
 HEAP32[$4>>2] = $regOrigData;
 HEAP32[$5>>2] = $nData;
 HEAP32[$6>>2] = $nPrefixReg;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$v>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 28|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13 & 1;
 $15 = ($14|0)==(0);
 $16 = $15&1;
 HEAP32[$bSeq>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$nExpr>>2] = $19;
 $20 = HEAP32[$nExpr>>2]|0;
 $21 = HEAP32[$bSeq>>2]|0;
 $22 = (($20) + ($21))|0;
 $23 = HEAP32[$5>>2]|0;
 $24 = (($22) + ($23))|0;
 HEAP32[$nBase>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 44|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 HEAP32[$26>>2] = $28;
 HEAP32[$regRecord>>2] = $28;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 4|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$nOBSat>>2] = $31;
 $32 = HEAP32[$6>>2]|0;
 $33 = ($32|0)!=(0);
 if ($33) {
  $34 = HEAP32[$3>>2]|0;
  $35 = HEAP32[$nExpr>>2]|0;
  $36 = (($34) - ($35))|0;
  $37 = HEAP32[$bSeq>>2]|0;
  $38 = (($36) - ($37))|0;
  HEAP32[$regBase>>2] = $38;
 } else {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 44|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = (($41) + 1)|0;
  HEAP32[$regBase>>2] = $42;
  $43 = HEAP32[$nBase>>2]|0;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 44|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (($46) + ($43))|0;
  HEAP32[$45>>2] = $47;
 }
 $48 = HEAP32[$2>>2]|0;
 $49 = ((($48)) + 16|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($50|0)!=(0);
 $52 = HEAP32[$2>>2]|0;
 if ($51) {
  $53 = ((($52)) + 16|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + 1)|0;
  $58 = $55;
 } else {
  $56 = ((($52)) + 12|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = $57;
 }
 HEAP32[$iLimit>>2] = $58;
 $59 = HEAP32[$v>>2]|0;
 $60 = (_sqlite3VdbeMakeLabel($59)|0);
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 24|0);
 HEAP32[$62>>2] = $60;
 $63 = HEAP32[$0>>2]|0;
 $64 = HEAP32[$1>>2]|0;
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$regBase>>2]|0;
 $67 = HEAP32[$4>>2]|0;
 (_sqlite3ExprCodeExprList($63,$65,$66,$67,5)|0);
 $68 = HEAP32[$bSeq>>2]|0;
 $69 = ($68|0)!=(0);
 if ($69) {
  $70 = HEAP32[$v>>2]|0;
  $71 = HEAP32[$1>>2]|0;
  $72 = ((($71)) + 8|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$regBase>>2]|0;
  $75 = HEAP32[$nExpr>>2]|0;
  $76 = (($74) + ($75))|0;
  (_sqlite3VdbeAddOp2($70,113,$73,$76)|0);
 }
 $77 = HEAP32[$6>>2]|0;
 $78 = ($77|0)==(0);
 if ($78) {
  $79 = HEAP32[$0>>2]|0;
  $80 = HEAP32[$3>>2]|0;
  $81 = HEAP32[$regBase>>2]|0;
  $82 = HEAP32[$nExpr>>2]|0;
  $83 = (($81) + ($82))|0;
  $84 = HEAP32[$bSeq>>2]|0;
  $85 = (($83) + ($84))|0;
  $86 = HEAP32[$5>>2]|0;
  _sqlite3ExprCodeMove($79,$80,$85,$86);
 }
 $87 = HEAP32[$v>>2]|0;
 $88 = HEAP32[$regBase>>2]|0;
 $89 = HEAP32[$nOBSat>>2]|0;
 $90 = (($88) + ($89))|0;
 $91 = HEAP32[$nBase>>2]|0;
 $92 = HEAP32[$nOBSat>>2]|0;
 $93 = (($91) - ($92))|0;
 $94 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp3($87,99,$90,$93,$94)|0);
 $95 = HEAP32[$nOBSat>>2]|0;
 $96 = ($95|0)>(0);
 if ($96) {
  $97 = HEAP32[$0>>2]|0;
  $98 = ((($97)) + 44|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = (($99) + 1)|0;
  HEAP32[$regPrevKey>>2] = $100;
  $101 = HEAP32[$1>>2]|0;
  $102 = ((($101)) + 4|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = HEAP32[$0>>2]|0;
  $105 = ((($104)) + 44|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = (($106) + ($103))|0;
  HEAP32[$105>>2] = $107;
  $108 = HEAP32[$nExpr>>2]|0;
  $109 = HEAP32[$1>>2]|0;
  $110 = ((($109)) + 4|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = (($108) - ($111))|0;
  $113 = HEAP32[$bSeq>>2]|0;
  $114 = (($112) + ($113))|0;
  HEAP32[$nKey>>2] = $114;
  $115 = HEAP32[$bSeq>>2]|0;
  $116 = ($115|0)!=(0);
  $117 = HEAP32[$v>>2]|0;
  if ($116) {
   $118 = HEAP32[$regBase>>2]|0;
   $119 = HEAP32[$nExpr>>2]|0;
   $120 = (($118) + ($119))|0;
   $121 = (_sqlite3VdbeAddOp1($117,22,$120)|0);
   HEAP32[$addrFirst>>2] = $121;
  } else {
   $122 = HEAP32[$1>>2]|0;
   $123 = ((($122)) + 8|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = (_sqlite3VdbeAddOp1($117,109,$124)|0);
   HEAP32[$addrFirst>>2] = $125;
  }
  $126 = HEAP32[$v>>2]|0;
  $127 = HEAP32[$regPrevKey>>2]|0;
  $128 = HEAP32[$regBase>>2]|0;
  $129 = HEAP32[$1>>2]|0;
  $130 = ((($129)) + 4|0);
  $131 = HEAP32[$130>>2]|0;
  (_sqlite3VdbeAddOp3($126,95,$127,$128,$131)|0);
  $132 = HEAP32[$v>>2]|0;
  $133 = HEAP32[$1>>2]|0;
  $134 = ((($133)) + 20|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = (_sqlite3VdbeGetOp($132,$135)|0);
  HEAP32[$pOp>>2] = $136;
  $137 = HEAP32[$0>>2]|0;
  $138 = HEAP32[$137>>2]|0;
  $139 = ((($138)) + 69|0);
  $140 = HEAP8[$139>>0]|0;
  $141 = ($140<<24>>24)!=(0);
  if ($141) {
   STACKTOP = sp;return;
  }
  $142 = HEAP32[$nKey>>2]|0;
  $143 = HEAP32[$5>>2]|0;
  $144 = (($142) + ($143))|0;
  $145 = HEAP32[$pOp>>2]|0;
  $146 = ((($145)) + 8|0);
  HEAP32[$146>>2] = $144;
  $147 = HEAP32[$pOp>>2]|0;
  $148 = ((($147)) + 16|0);
  $149 = HEAP32[$148>>2]|0;
  HEAP32[$pKI>>2] = $149;
  $150 = HEAP32[$pKI>>2]|0;
  $151 = ((($150)) + 16|0);
  $152 = HEAP32[$151>>2]|0;
  $153 = HEAP32[$pKI>>2]|0;
  $154 = ((($153)) + 6|0);
  $155 = HEAP16[$154>>1]|0;
  $156 = $155&65535;
  _memset(($152|0),0,($156|0))|0;
  $157 = HEAP32[$v>>2]|0;
  $158 = HEAP32[$pKI>>2]|0;
  _sqlite3VdbeChangeP4($157,-1,$158,-6);
  $159 = HEAP32[$0>>2]|0;
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$160>>2]|0;
  $162 = HEAP32[$nOBSat>>2]|0;
  $163 = HEAP32[$pKI>>2]|0;
  $164 = ((($163)) + 8|0);
  $165 = HEAP16[$164>>1]|0;
  $166 = $165&65535;
  $167 = (($166) - 1)|0;
  $168 = (_keyInfoFromExprList($159,$161,$162,$167)|0);
  $169 = HEAP32[$pOp>>2]|0;
  $170 = ((($169)) + 16|0);
  HEAP32[$170>>2] = $168;
  $171 = HEAP32[$v>>2]|0;
  $172 = (_sqlite3VdbeCurrentAddr($171)|0);
  HEAP32[$addrJmp>>2] = $172;
  $173 = HEAP32[$v>>2]|0;
  $174 = HEAP32[$addrJmp>>2]|0;
  $175 = (($174) + 1)|0;
  $176 = HEAP32[$addrJmp>>2]|0;
  $177 = (($176) + 1)|0;
  (_sqlite3VdbeAddOp3($173,18,$175,0,$177)|0);
  $178 = HEAP32[$v>>2]|0;
  $179 = (_sqlite3VdbeMakeLabel($178)|0);
  $180 = HEAP32[$1>>2]|0;
  $181 = ((($180)) + 16|0);
  HEAP32[$181>>2] = $179;
  $182 = HEAP32[$0>>2]|0;
  $183 = ((($182)) + 44|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = (($184) + 1)|0;
  HEAP32[$183>>2] = $185;
  $186 = HEAP32[$1>>2]|0;
  $187 = ((($186)) + 12|0);
  HEAP32[$187>>2] = $185;
  $188 = HEAP32[$v>>2]|0;
  $189 = HEAP32[$1>>2]|0;
  $190 = ((($189)) + 12|0);
  $191 = HEAP32[$190>>2]|0;
  $192 = HEAP32[$1>>2]|0;
  $193 = ((($192)) + 16|0);
  $194 = HEAP32[$193>>2]|0;
  (_sqlite3VdbeAddOp2($188,14,$191,$194)|0);
  $195 = HEAP32[$v>>2]|0;
  $196 = HEAP32[$1>>2]|0;
  $197 = ((($196)) + 8|0);
  $198 = HEAP32[$197>>2]|0;
  (_sqlite3VdbeAddOp1($195,133,$198)|0);
  $199 = HEAP32[$iLimit>>2]|0;
  $200 = ($199|0)!=(0);
  if ($200) {
   $201 = HEAP32[$v>>2]|0;
   $202 = HEAP32[$iLimit>>2]|0;
   $203 = HEAP32[$1>>2]|0;
   $204 = ((($203)) + 24|0);
   $205 = HEAP32[$204>>2]|0;
   (_sqlite3VdbeAddOp2($201,22,$202,$205)|0);
  }
  $206 = HEAP32[$v>>2]|0;
  $207 = HEAP32[$addrFirst>>2]|0;
  _sqlite3VdbeJumpHere($206,$207);
  $208 = HEAP32[$0>>2]|0;
  $209 = HEAP32[$regBase>>2]|0;
  $210 = HEAP32[$regPrevKey>>2]|0;
  $211 = HEAP32[$1>>2]|0;
  $212 = ((($211)) + 4|0);
  $213 = HEAP32[$212>>2]|0;
  _sqlite3ExprCodeMove($208,$209,$210,$213);
  $214 = HEAP32[$v>>2]|0;
  $215 = HEAP32[$addrJmp>>2]|0;
  _sqlite3VdbeJumpHere($214,$215);
 }
 $216 = HEAP32[$1>>2]|0;
 $217 = ((($216)) + 28|0);
 $218 = HEAP8[$217>>0]|0;
 $219 = $218&255;
 $220 = $219 & 1;
 $221 = ($220|0)!=(0);
 if ($221) {
  HEAP32[$op>>2] = 125;
 } else {
  HEAP32[$op>>2] = 126;
 }
 $222 = HEAP32[$v>>2]|0;
 $223 = HEAP32[$op>>2]|0;
 $224 = HEAP32[$1>>2]|0;
 $225 = ((($224)) + 8|0);
 $226 = HEAP32[$225>>2]|0;
 $227 = HEAP32[$regRecord>>2]|0;
 (_sqlite3VdbeAddOp2($222,$223,$226,$227)|0);
 $228 = HEAP32[$iLimit>>2]|0;
 $229 = ($228|0)!=(0);
 if (!($229)) {
  STACKTOP = sp;return;
 }
 HEAP32[$r1>>2] = 0;
 $230 = HEAP32[$v>>2]|0;
 $231 = HEAP32[$iLimit>>2]|0;
 $232 = (_sqlite3VdbeAddOp3($230,67,$231,0,1)|0);
 HEAP32[$addr>>2] = $232;
 $233 = HEAP32[$v>>2]|0;
 $234 = HEAP32[$1>>2]|0;
 $235 = ((($234)) + 8|0);
 $236 = HEAP32[$235>>2]|0;
 (_sqlite3VdbeAddOp1($233,53,$236)|0);
 $237 = HEAP32[$1>>2]|0;
 $238 = ((($237)) + 29|0);
 $239 = HEAP8[$238>>0]|0;
 $240 = ($239<<24>>24)!=(0);
 if ($240) {
  $241 = HEAP32[$0>>2]|0;
  $242 = ((($241)) + 44|0);
  $243 = HEAP32[$242>>2]|0;
  $244 = (($243) + 1)|0;
  HEAP32[$242>>2] = $244;
  HEAP32[$r1>>2] = $244;
  $245 = HEAP32[$v>>2]|0;
  $246 = HEAP32[$1>>2]|0;
  $247 = ((($246)) + 8|0);
  $248 = HEAP32[$247>>2]|0;
  $249 = HEAP32[$nExpr>>2]|0;
  $250 = HEAP32[$r1>>2]|0;
  (_sqlite3VdbeAddOp3($245,96,$248,$249,$250)|0);
 }
 $251 = HEAP32[$v>>2]|0;
 $252 = HEAP32[$1>>2]|0;
 $253 = ((($252)) + 8|0);
 $254 = HEAP32[$253>>2]|0;
 (_sqlite3VdbeAddOp1($251,117,$254)|0);
 $255 = HEAP32[$1>>2]|0;
 $256 = ((($255)) + 29|0);
 $257 = HEAP8[$256>>0]|0;
 $258 = ($257<<24>>24)!=(0);
 if ($258) {
  $259 = HEAP32[$v>>2]|0;
  $260 = (_sqlite3VdbeCurrentAddr($259)|0);
  $261 = (($260) + 2)|0;
  HEAP32[$iBrk>>2] = $261;
  $262 = HEAP32[$v>>2]|0;
  $263 = HEAP32[$regBase>>2]|0;
  $264 = HEAP32[$nExpr>>2]|0;
  $265 = (($263) + ($264))|0;
  $266 = HEAP32[$iBrk>>2]|0;
  $267 = HEAP32[$r1>>2]|0;
  (_sqlite3VdbeAddOp3($262,37,$265,$266,$267)|0);
  $268 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($268,-128);
 }
 $269 = HEAP32[$v>>2]|0;
 $270 = HEAP32[$addr>>2]|0;
 _sqlite3VdbeJumpHere($269,$270);
 STACKTOP = sp;return;
}
function _substExpr($db,$pExpr,$iTable,$pEList) {
 $db = $db|0;
 $pExpr = $pExpr|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pExpr;
 HEAP32[$3>>2] = $iTable;
 HEAP32[$4>>2] = $pEList;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $69 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($69|0);
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(152);
 do {
  if ($10) {
   $11 = HEAP32[$2>>2]|0;
   $12 = ((($11)) + 28|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = ($13|0)==($14|0);
   if ($15) {
    $16 = HEAP32[$2>>2]|0;
    $17 = ((($16)) + 32|0);
    $18 = HEAP16[$17>>1]|0;
    $19 = $18 << 16 >> 16;
    $20 = ($19|0)<(0);
    if ($20) {
     $21 = HEAP32[$2>>2]|0;
     HEAP8[$21>>0] = 101;
     break;
    } else {
     $22 = HEAP32[$1>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = ((($23)) + 32|0);
     $25 = HEAP16[$24>>1]|0;
     $26 = $25 << 16 >> 16;
     $27 = HEAP32[$4>>2]|0;
     $28 = ((($27)) + 4|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = (($29) + (($26*20)|0)|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = (_sqlite3ExprDup($22,$31,0)|0);
     HEAP32[$pNew>>2] = $32;
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$2>>2]|0;
     _sqlite3ExprDelete($33,$34);
     $35 = HEAP32[$pNew>>2]|0;
     HEAP32[$2>>2] = $35;
     break;
    }
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
 } while(0);
 do {
  if ((label|0) == 8) {
   $36 = HEAP32[$1>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$3>>2]|0;
   $41 = HEAP32[$4>>2]|0;
   $42 = (_substExpr($36,$39,$40,$41)|0);
   $43 = HEAP32[$2>>2]|0;
   $44 = ((($43)) + 12|0);
   HEAP32[$44>>2] = $42;
   $45 = HEAP32[$1>>2]|0;
   $46 = HEAP32[$2>>2]|0;
   $47 = ((($46)) + 16|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = HEAP32[$3>>2]|0;
   $50 = HEAP32[$4>>2]|0;
   $51 = (_substExpr($45,$48,$49,$50)|0);
   $52 = HEAP32[$2>>2]|0;
   $53 = ((($52)) + 16|0);
   HEAP32[$53>>2] = $51;
   $54 = HEAP32[$2>>2]|0;
   $55 = ((($54)) + 4|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = $56 & 2048;
   $58 = ($57|0)!=(0);
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = ((($60)) + 20|0);
   if ($58) {
    $62 = HEAP32[$61>>2]|0;
    $63 = HEAP32[$3>>2]|0;
    $64 = HEAP32[$4>>2]|0;
    _substSelect($59,$62,$63,$64,1);
    break;
   } else {
    $65 = HEAP32[$61>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    $67 = HEAP32[$4>>2]|0;
    _substExprList($59,$65,$66,$67);
    break;
   }
  }
 } while(0);
 $68 = HEAP32[$2>>2]|0;
 HEAP32[$0>>2] = $68;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _substSelect($db,$p,$iTable,$pEList,$doPrior) {
 $db = $db|0;
 $p = $p|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 $doPrior = $doPrior|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $i = 0, $pItem = 0, $pSrc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $pSrc = sp + 8|0;
 $pItem = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $pEList;
 HEAP32[$4>>2] = $doPrior;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$3>>2]|0;
  _substExprList($7,$9,$10,$11);
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 36|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  _substExprList($12,$15,$16,$17);
  $18 = HEAP32[$0>>2]|0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 44|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _substExprList($18,$21,$22,$23);
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 40|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$3>>2]|0;
  $30 = (_substExpr($24,$27,$28,$29)|0);
  $31 = HEAP32[$1>>2]|0;
  $32 = ((($31)) + 40|0);
  HEAP32[$32>>2] = $30;
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$1>>2]|0;
  $35 = ((($34)) + 32|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = (_substExpr($33,$36,$37,$38)|0);
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 32|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$1>>2]|0;
  $43 = ((($42)) + 28|0);
  $44 = HEAP32[$43>>2]|0;
  HEAP32[$pSrc>>2] = $44;
  $45 = HEAP32[$pSrc>>2]|0;
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$i>>2] = $46;
  $47 = HEAP32[$pSrc>>2]|0;
  $48 = ((($47)) + 8|0);
  HEAP32[$pItem>>2] = $48;
  while(1) {
   $49 = HEAP32[$i>>2]|0;
   $50 = ($49|0)>(0);
   if (!($50)) {
    break;
   }
   $51 = HEAP32[$0>>2]|0;
   $52 = HEAP32[$pItem>>2]|0;
   $53 = ((($52)) + 20|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = HEAP32[$2>>2]|0;
   $56 = HEAP32[$3>>2]|0;
   _substSelect($51,$54,$55,$56,1);
   $57 = HEAP32[$pItem>>2]|0;
   $58 = ((($57)) + 36|0);
   $59 = ((($58)) + 1|0);
   $60 = HEAP8[$59>>0]|0;
   $61 = ($60&255) >>> 2;
   $62 = $61 & 1;
   $63 = $62&255;
   $64 = ($63|0)!=(0);
   if ($64) {
    $65 = HEAP32[$0>>2]|0;
    $66 = HEAP32[$pItem>>2]|0;
    $67 = ((($66)) + 64|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = HEAP32[$2>>2]|0;
    $70 = HEAP32[$3>>2]|0;
    _substExprList($65,$68,$69,$70);
   }
   $71 = HEAP32[$i>>2]|0;
   $72 = (($71) + -1)|0;
   HEAP32[$i>>2] = $72;
   $73 = HEAP32[$pItem>>2]|0;
   $74 = ((($73)) + 72|0);
   HEAP32[$pItem>>2] = $74;
  }
  $75 = HEAP32[$4>>2]|0;
  $76 = ($75|0)!=(0);
  if (!($76)) {
   label = 9;
   break;
  }
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 48|0);
  $79 = HEAP32[$78>>2]|0;
  HEAP32[$1>>2] = $79;
  $80 = ($79|0)!=(0|0);
  if (!($80)) {
   label = 9;
   break;
  }
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
}
function _substExprList($db,$pList,$iTable,$pEList) {
 $db = $db|0;
 $pList = $pList|0;
 $iTable = $iTable|0;
 $pEList = $pEList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pList;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $pEList;
 $4 = HEAP32[$1>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = ($6|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$0>>2]|0;
  $11 = HEAP32[$i>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + (($11*20)|0)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = HEAP32[$3>>2]|0;
  $19 = (_substExpr($10,$16,$17,$18)|0);
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($23) + (($20*20)|0)|0);
  HEAP32[$24>>2] = $19;
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _multiSelectValues($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nRow = 0, $pPrior = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pPrior = sp + 8|0;
 $nRow = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDest;
 HEAP32[$nRow>>2] = 1;
 HEAP32[$rc>>2] = 0;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = ((($3)) + 48|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)==(0|0);
  if ($6) {
   break;
  }
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 48|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$1>>2] = $9;
  $10 = HEAP32[$nRow>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$nRow>>2] = $11;
 }
 while(1) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ($12|0)!=(0|0);
  if (!($13)) {
   label = 7;
   break;
  }
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 48|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pPrior>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 48|0);
  HEAP32[$18>>2] = 0;
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = HEAP32[$2>>2]|0;
  $22 = (_sqlite3Select($19,$20,$21)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$pPrior>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 48|0);
  HEAP32[$25>>2] = $23;
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)!=(0);
  if ($27) {
   label = 7;
   break;
  }
  $28 = HEAP32[$nRow>>2]|0;
  $29 = $28&65535;
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 6|0);
  HEAP16[$31>>1] = $29;
  $32 = HEAP32[$1>>2]|0;
  $33 = ((($32)) + 52|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$1>>2] = $34;
 }
 if ((label|0) == 7) {
  $35 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _generateWithRecursiveQuery($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $addrBreak = 0, $addrCont = 0, $addrTop = 0, $destQueue = 0, $eDest = 0, $i = 0, $iCurrent = 0, $iDistinct = 0, $iQueue = 0, $nCol = 0, $pKeyInfo = 0, $pLimit = 0, $pOffset = 0, $pOrderBy = 0, $pSetup = 0, $pSrc = 0, $rc = 0, $regCurrent = 0;
 var $regLimit = 0, $regOffset = 0, $v = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer = sp;
 $0 = sp + 120|0;
 $1 = sp + 116|0;
 $2 = sp + 112|0;
 $pSrc = sp + 108|0;
 $nCol = sp + 104|0;
 $v = sp + 100|0;
 $pSetup = sp + 96|0;
 $addrTop = sp + 92|0;
 $addrCont = sp + 88|0;
 $addrBreak = sp + 84|0;
 $iCurrent = sp + 80|0;
 $regCurrent = sp + 76|0;
 $iQueue = sp + 72|0;
 $iDistinct = sp + 68|0;
 $eDest = sp + 64|0;
 $destQueue = sp + 40|0;
 $i = sp + 32|0;
 $rc = sp + 28|0;
 $pOrderBy = sp + 24|0;
 $pLimit = sp + 20|0;
 $pOffset = sp + 16|0;
 $regLimit = sp + 12|0;
 $regOffset = sp + 8|0;
 $pKeyInfo = sp + 4|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pDest;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 28|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSrc>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$nCol>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$pSetup>>2] = $14;
 HEAP32[$iCurrent>>2] = 0;
 HEAP32[$iDistinct>>2] = 0;
 HEAP32[$eDest>>2] = 5;
 $15 = HEAP32[$0>>2]|0;
 $16 = (_sqlite3AuthCheck($15,33,0,0,0)|0);
 $17 = ($16|0)!=(0);
 if ($17) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$v>>2]|0;
 $19 = (_sqlite3VdbeMakeLabel($18)|0);
 HEAP32[$addrBreak>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = HEAP32[$addrBreak>>2]|0;
 _computeLimitRegisters($20,$21,$22);
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 56|0);
 $25 = HEAP32[$24>>2]|0;
 HEAP32[$pLimit>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 60|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$pOffset>>2] = $28;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 12|0);
 $31 = HEAP32[$30>>2]|0;
 HEAP32[$regLimit>>2] = $31;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 16|0);
 $34 = HEAP32[$33>>2]|0;
 HEAP32[$regOffset>>2] = $34;
 $35 = HEAP32[$1>>2]|0;
 $36 = ((($35)) + 60|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 56|0);
 HEAP32[$38>>2] = 0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 16|0);
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 12|0);
 HEAP32[$42>>2] = 0;
 $43 = HEAP32[$1>>2]|0;
 $44 = ((($43)) + 44|0);
 $45 = HEAP32[$44>>2]|0;
 HEAP32[$pOrderBy>>2] = $45;
 HEAP32[$i>>2] = 0;
 while(1) {
  $46 = HEAP32[$i>>2]|0;
  $47 = HEAP32[$pSrc>>2]|0;
  $48 = HEAP32[$47>>2]|0;
  $49 = ($46|0)<($48|0);
  if (!($49)) {
   break;
  }
  $50 = HEAP32[$i>>2]|0;
  $51 = HEAP32[$pSrc>>2]|0;
  $52 = ((($51)) + 8|0);
  $53 = (($52) + (($50*72)|0)|0);
  $54 = ((($53)) + 36|0);
  $55 = ((($54)) + 1|0);
  $56 = HEAP8[$55>>0]|0;
  $57 = ($56&255) >>> 5;
  $58 = $57 & 1;
  $59 = $58&255;
  $60 = ($59|0)!=(0);
  $61 = HEAP32[$i>>2]|0;
  if ($60) {
   label = 5;
   break;
  }
  $67 = (($61) + 1)|0;
  HEAP32[$i>>2] = $67;
 }
 if ((label|0) == 5) {
  $62 = HEAP32[$pSrc>>2]|0;
  $63 = ((($62)) + 8|0);
  $64 = (($63) + (($61*72)|0)|0);
  $65 = ((($64)) + 44|0);
  $66 = HEAP32[$65>>2]|0;
  HEAP32[$iCurrent>>2] = $66;
 }
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 40|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = (($70) + 1)|0;
 HEAP32[$69>>2] = $71;
 HEAP32[$iQueue>>2] = $70;
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 4|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 $76 = ($75|0)==(115);
 $77 = HEAP32[$pOrderBy>>2]|0;
 $78 = ($77|0)!=(0|0);
 if ($76) {
  $79 = $78 ? 8 : 6;
  HEAP32[$eDest>>2] = $79;
  $80 = HEAP32[$0>>2]|0;
  $81 = ((($80)) + 40|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + 1)|0;
  HEAP32[$81>>2] = $83;
  HEAP32[$iDistinct>>2] = $82;
 } else {
  $84 = $78 ? 7 : 5;
  HEAP32[$eDest>>2] = $84;
 }
 $85 = HEAP32[$eDest>>2]|0;
 $86 = HEAP32[$iQueue>>2]|0;
 _sqlite3SelectDestInit($destQueue,$85,$86);
 $87 = HEAP32[$0>>2]|0;
 $88 = ((($87)) + 44|0);
 $89 = HEAP32[$88>>2]|0;
 $90 = (($89) + 1)|0;
 HEAP32[$88>>2] = $90;
 HEAP32[$regCurrent>>2] = $90;
 $91 = HEAP32[$v>>2]|0;
 $92 = HEAP32[$iCurrent>>2]|0;
 $93 = HEAP32[$regCurrent>>2]|0;
 $94 = HEAP32[$nCol>>2]|0;
 (_sqlite3VdbeAddOp3($91,110,$92,$93,$94)|0);
 $95 = HEAP32[$pOrderBy>>2]|0;
 $96 = ($95|0)!=(0|0);
 if ($96) {
  $97 = HEAP32[$0>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = (_multiSelectOrderByKeyInfo($97,$98,1)|0);
  HEAP32[$pKeyInfo>>2] = $99;
  $100 = HEAP32[$v>>2]|0;
  $101 = HEAP32[$iQueue>>2]|0;
  $102 = HEAP32[$pOrderBy>>2]|0;
  $103 = HEAP32[$102>>2]|0;
  $104 = (($103) + 2)|0;
  $105 = HEAP32[$pKeyInfo>>2]|0;
  (_sqlite3VdbeAddOp4($100,107,$101,$104,0,$105,-6)|0);
  $106 = HEAP32[$pOrderBy>>2]|0;
  $107 = ((($destQueue)) + 20|0);
  HEAP32[$107>>2] = $106;
 } else {
  $108 = HEAP32[$v>>2]|0;
  $109 = HEAP32[$iQueue>>2]|0;
  $110 = HEAP32[$nCol>>2]|0;
  (_sqlite3VdbeAddOp2($108,107,$109,$110)|0);
 }
 $111 = HEAP32[$iDistinct>>2]|0;
 $112 = ($111|0)!=(0);
 if ($112) {
  $113 = HEAP32[$v>>2]|0;
  $114 = HEAP32[$iDistinct>>2]|0;
  $115 = (_sqlite3VdbeAddOp2($113,107,$114,0)|0);
  $116 = HEAP32[$1>>2]|0;
  $117 = ((($116)) + 20|0);
  HEAP32[$117>>2] = $115;
  $118 = HEAP32[$1>>2]|0;
  $119 = ((($118)) + 8|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = $120 | 32;
  HEAP32[$119>>2] = $121;
 }
 $122 = HEAP32[$1>>2]|0;
 $123 = ((($122)) + 44|0);
 HEAP32[$123>>2] = 0;
 $124 = HEAP32[$pSetup>>2]|0;
 $125 = ((($124)) + 52|0);
 HEAP32[$125>>2] = 0;
 $126 = HEAP32[$0>>2]|0;
 $127 = HEAP32[$pSetup>>2]|0;
 $128 = (_sqlite3Select($126,$127,$destQueue)|0);
 HEAP32[$rc>>2] = $128;
 $129 = HEAP32[$1>>2]|0;
 $130 = HEAP32[$pSetup>>2]|0;
 $131 = ((($130)) + 52|0);
 HEAP32[$131>>2] = $129;
 $132 = HEAP32[$rc>>2]|0;
 $133 = ($132|0)!=(0);
 if (!($133)) {
  $134 = HEAP32[$v>>2]|0;
  $135 = HEAP32[$iQueue>>2]|0;
  $136 = HEAP32[$addrBreak>>2]|0;
  $137 = (_sqlite3VdbeAddOp2($134,57,$135,$136)|0);
  HEAP32[$addrTop>>2] = $137;
  $138 = HEAP32[$v>>2]|0;
  $139 = HEAP32[$iCurrent>>2]|0;
  (_sqlite3VdbeAddOp1($138,124,$139)|0);
  $140 = HEAP32[$pOrderBy>>2]|0;
  $141 = ($140|0)!=(0|0);
  $142 = HEAP32[$v>>2]|0;
  $143 = HEAP32[$iQueue>>2]|0;
  if ($141) {
   $144 = HEAP32[$pOrderBy>>2]|0;
   $145 = HEAP32[$144>>2]|0;
   $146 = (($145) + 1)|0;
   $147 = HEAP32[$regCurrent>>2]|0;
   (_sqlite3VdbeAddOp3($142,96,$143,$146,$147)|0);
  } else {
   $148 = HEAP32[$regCurrent>>2]|0;
   (_sqlite3VdbeAddOp2($142,122,$143,$148)|0);
  }
  $149 = HEAP32[$v>>2]|0;
  $150 = HEAP32[$iQueue>>2]|0;
  (_sqlite3VdbeAddOp1($149,117,$150)|0);
  $151 = HEAP32[$v>>2]|0;
  $152 = (_sqlite3VdbeMakeLabel($151)|0);
  HEAP32[$addrCont>>2] = $152;
  $153 = HEAP32[$v>>2]|0;
  $154 = HEAP32[$regOffset>>2]|0;
  $155 = HEAP32[$addrCont>>2]|0;
  _codeOffset($153,$154,$155);
  $156 = HEAP32[$0>>2]|0;
  $157 = HEAP32[$1>>2]|0;
  $158 = HEAP32[$1>>2]|0;
  $159 = HEAP32[$158>>2]|0;
  $160 = HEAP32[$iCurrent>>2]|0;
  $161 = HEAP32[$2>>2]|0;
  $162 = HEAP32[$addrCont>>2]|0;
  $163 = HEAP32[$addrBreak>>2]|0;
  _selectInnerLoop($156,$157,$159,$160,0,0,$161,$162,$163);
  $164 = HEAP32[$regLimit>>2]|0;
  $165 = ($164|0)!=(0);
  if ($165) {
   $166 = HEAP32[$v>>2]|0;
   $167 = HEAP32[$regLimit>>2]|0;
   $168 = HEAP32[$addrBreak>>2]|0;
   (_sqlite3VdbeAddOp2($166,68,$167,$168)|0);
  }
  $169 = HEAP32[$v>>2]|0;
  $170 = HEAP32[$addrCont>>2]|0;
  _sqlite3VdbeResolveLabel($169,$170);
  $171 = HEAP32[$1>>2]|0;
  $172 = ((($171)) + 8|0);
  $173 = HEAP32[$172>>2]|0;
  $174 = $173 & 8;
  $175 = ($174|0)!=(0);
  if ($175) {
   $176 = HEAP32[$0>>2]|0;
   _sqlite3ErrorMsg($176,35980,$vararg_buffer);
  } else {
   $177 = HEAP32[$1>>2]|0;
   $178 = ((($177)) + 48|0);
   HEAP32[$178>>2] = 0;
   $179 = HEAP32[$0>>2]|0;
   $180 = HEAP32[$1>>2]|0;
   (_sqlite3Select($179,$180,$destQueue)|0);
   $181 = HEAP32[$pSetup>>2]|0;
   $182 = HEAP32[$1>>2]|0;
   $183 = ((($182)) + 48|0);
   HEAP32[$183>>2] = $181;
  }
  $184 = HEAP32[$v>>2]|0;
  $185 = HEAP32[$addrTop>>2]|0;
  (_sqlite3VdbeGoto($184,$185)|0);
  $186 = HEAP32[$v>>2]|0;
  $187 = HEAP32[$addrBreak>>2]|0;
  _sqlite3VdbeResolveLabel($186,$187);
 }
 $188 = HEAP32[$0>>2]|0;
 $189 = HEAP32[$188>>2]|0;
 $190 = HEAP32[$1>>2]|0;
 $191 = ((($190)) + 44|0);
 $192 = HEAP32[$191>>2]|0;
 _sqlite3ExprListDelete($189,$192);
 $193 = HEAP32[$pOrderBy>>2]|0;
 $194 = HEAP32[$1>>2]|0;
 $195 = ((($194)) + 44|0);
 HEAP32[$195>>2] = $193;
 $196 = HEAP32[$pLimit>>2]|0;
 $197 = HEAP32[$1>>2]|0;
 $198 = ((($197)) + 56|0);
 HEAP32[$198>>2] = $196;
 $199 = HEAP32[$pOffset>>2]|0;
 $200 = HEAP32[$1>>2]|0;
 $201 = ((($200)) + 60|0);
 HEAP32[$201>>2] = $199;
 STACKTOP = sp;return;
}
function _multiSelectOrderBy($pParse,$p,$pDest) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pDest = $pDest|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aPermute = 0, $addr1 = 0, $addrAeqB = 0, $addrAgtB = 0, $addrAltB = 0, $addrEofA = 0, $addrEofA_noB = 0;
 var $addrEofB = 0, $addrOutA = 0, $addrOutB = 0, $addrSelectA = 0, $addrSelectB = 0, $db = 0, $destA = 0, $destB = 0, $i = 0, $iSub1 = 0, $iSub2 = 0, $j = 0, $labelCmpr = 0, $labelEnd = 0, $nExpr = 0, $nOrderBy = 0, $op = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0;
 var $or$cond7 = 0, $pFirst = 0, $pItem = 0, $pItem1 = 0, $pKeyDup = 0, $pKeyMerge = 0, $pNew = 0, $pOrderBy = 0, $pPrior = 0, $regAddrA = 0, $regAddrB = 0, $regLimitA = 0, $regLimitB = 0, $regOutA = 0, $regOutB = 0, $regPrev = 0, $savedLimit = 0, $savedOffset = 0, $v = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 220|0;
 $1 = sp + 216|0;
 $2 = sp + 212|0;
 $3 = sp + 208|0;
 $i = sp + 204|0;
 $j = sp + 200|0;
 $pPrior = sp + 196|0;
 $v = sp + 192|0;
 $destA = sp + 168|0;
 $destB = sp + 144|0;
 $regAddrA = sp + 140|0;
 $regAddrB = sp + 136|0;
 $addrSelectA = sp + 132|0;
 $addrSelectB = sp + 128|0;
 $regOutA = sp + 124|0;
 $regOutB = sp + 120|0;
 $addrOutA = sp + 116|0;
 $addrOutB = sp + 112|0;
 $addrEofA = sp + 108|0;
 $addrEofA_noB = sp + 104|0;
 $addrEofB = sp + 100|0;
 $addrAltB = sp + 96|0;
 $addrAeqB = sp + 92|0;
 $addrAgtB = sp + 88|0;
 $regLimitA = sp + 84|0;
 $regLimitB = sp + 80|0;
 $regPrev = sp + 76|0;
 $savedLimit = sp + 72|0;
 $savedOffset = sp + 68|0;
 $labelCmpr = sp + 64|0;
 $labelEnd = sp + 60|0;
 $addr1 = sp + 56|0;
 $op = sp + 52|0;
 $pKeyDup = sp + 48|0;
 $pKeyMerge = sp + 44|0;
 $db = sp + 40|0;
 $pOrderBy = sp + 36|0;
 $nOrderBy = sp + 32|0;
 $aPermute = sp + 28|0;
 $iSub1 = sp + 24|0;
 $iSub2 = sp + 20|0;
 $pItem = sp + 16|0;
 $pNew = sp + 12|0;
 $pItem1 = sp + 8|0;
 $nExpr = sp + 4|0;
 $pFirst = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pDest;
 HEAP32[$addrOutB>>2] = 0;
 HEAP32[$pKeyDup>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$v>>2] = $8;
 $9 = HEAP32[$v>>2]|0;
 $10 = (_sqlite3VdbeMakeLabel($9)|0);
 HEAP32[$labelEnd>>2] = $10;
 $11 = HEAP32[$v>>2]|0;
 $12 = (_sqlite3VdbeMakeLabel($11)|0);
 HEAP32[$labelCmpr>>2] = $12;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 HEAP32[$op>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pPrior>>2] = $19;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 44|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pOrderBy>>2] = $22;
 $23 = HEAP32[$pOrderBy>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$nOrderBy>>2] = $24;
 $25 = HEAP32[$op>>2]|0;
 $26 = ($25|0)!=(116);
 L1: do {
  if ($26) {
   HEAP32[$i>>2] = 1;
   while(1) {
    $27 = HEAP32[$db>>2]|0;
    $28 = ((($27)) + 69|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(0);
    if (!($31)) {
     break L1;
    }
    $32 = HEAP32[$i>>2]|0;
    $33 = HEAP32[$2>>2]|0;
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$34>>2]|0;
    $36 = ($32|0)<=($35|0);
    if (!($36)) {
     break L1;
    }
    HEAP32[$j>>2] = 0;
    $37 = HEAP32[$pOrderBy>>2]|0;
    $38 = ((($37)) + 4|0);
    $39 = HEAP32[$38>>2]|0;
    HEAP32[$pItem>>2] = $39;
    while(1) {
     $40 = HEAP32[$j>>2]|0;
     $41 = HEAP32[$nOrderBy>>2]|0;
     $42 = ($40|0)<($41|0);
     if (!($42)) {
      break;
     }
     $43 = HEAP32[$pItem>>2]|0;
     $44 = ((($43)) + 16|0);
     $45 = HEAP16[$44>>1]|0;
     $46 = $45&65535;
     $47 = HEAP32[$i>>2]|0;
     $48 = ($46|0)==($47|0);
     if ($48) {
      break;
     }
     $49 = HEAP32[$j>>2]|0;
     $50 = (($49) + 1)|0;
     HEAP32[$j>>2] = $50;
     $51 = HEAP32[$pItem>>2]|0;
     $52 = ((($51)) + 20|0);
     HEAP32[$pItem>>2] = $52;
    }
    $53 = HEAP32[$j>>2]|0;
    $54 = HEAP32[$nOrderBy>>2]|0;
    $55 = ($53|0)==($54|0);
    if ($55) {
     $56 = HEAP32[$db>>2]|0;
     $57 = (_sqlite3Expr($56,134,0)|0);
     HEAP32[$pNew>>2] = $57;
     $58 = HEAP32[$pNew>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      break;
     }
     $60 = HEAP32[$pNew>>2]|0;
     $61 = ((($60)) + 4|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = $62 | 1024;
     HEAP32[$61>>2] = $63;
     $64 = HEAP32[$i>>2]|0;
     $65 = HEAP32[$pNew>>2]|0;
     $66 = ((($65)) + 8|0);
     HEAP32[$66>>2] = $64;
     $67 = HEAP32[$1>>2]|0;
     $68 = HEAP32[$pOrderBy>>2]|0;
     $69 = HEAP32[$pNew>>2]|0;
     $70 = (_sqlite3ExprListAppend($67,$68,$69)|0);
     HEAP32[$pOrderBy>>2] = $70;
     $71 = HEAP32[$pOrderBy>>2]|0;
     $72 = ($71|0)!=(0|0);
     if ($72) {
      $73 = HEAP32[$i>>2]|0;
      $74 = $73&65535;
      $75 = HEAP32[$nOrderBy>>2]|0;
      $76 = (($75) + 1)|0;
      HEAP32[$nOrderBy>>2] = $76;
      $77 = HEAP32[$pOrderBy>>2]|0;
      $78 = ((($77)) + 4|0);
      $79 = HEAP32[$78>>2]|0;
      $80 = (($79) + (($75*20)|0)|0);
      $81 = ((($80)) + 16|0);
      HEAP16[$81>>1] = $74;
     }
    }
    $82 = HEAP32[$i>>2]|0;
    $83 = (($82) + 1)|0;
    HEAP32[$i>>2] = $83;
   }
   HEAP32[$0>>2] = 7;
   $472 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($472|0);
  }
 } while(0);
 $84 = HEAP32[$db>>2]|0;
 $85 = HEAP32[$nOrderBy>>2]|0;
 $86 = (($85) + 1)|0;
 $87 = $86<<2;
 $88 = (_sqlite3DbMallocRawNN($84,$87,0)|0);
 HEAP32[$aPermute>>2] = $88;
 $89 = HEAP32[$aPermute>>2]|0;
 $90 = ($89|0)!=(0|0);
 if ($90) {
  $91 = HEAP32[$nOrderBy>>2]|0;
  $92 = HEAP32[$aPermute>>2]|0;
  HEAP32[$92>>2] = $91;
  HEAP32[$i>>2] = 1;
  $93 = HEAP32[$pOrderBy>>2]|0;
  $94 = ((($93)) + 4|0);
  $95 = HEAP32[$94>>2]|0;
  HEAP32[$pItem1>>2] = $95;
  while(1) {
   $96 = HEAP32[$i>>2]|0;
   $97 = HEAP32[$nOrderBy>>2]|0;
   $98 = ($96|0)<=($97|0);
   if (!($98)) {
    break;
   }
   $99 = HEAP32[$pItem1>>2]|0;
   $100 = ((($99)) + 16|0);
   $101 = HEAP16[$100>>1]|0;
   $102 = $101&65535;
   $103 = (($102) - 1)|0;
   $104 = HEAP32[$i>>2]|0;
   $105 = HEAP32[$aPermute>>2]|0;
   $106 = (($105) + ($104<<2)|0);
   HEAP32[$106>>2] = $103;
   $107 = HEAP32[$i>>2]|0;
   $108 = (($107) + 1)|0;
   HEAP32[$i>>2] = $108;
   $109 = HEAP32[$pItem1>>2]|0;
   $110 = ((($109)) + 20|0);
   HEAP32[$pItem1>>2] = $110;
  }
  $111 = HEAP32[$1>>2]|0;
  $112 = HEAP32[$2>>2]|0;
  $113 = (_multiSelectOrderByKeyInfo($111,$112,1)|0);
  HEAP32[$pKeyMerge>>2] = $113;
 } else {
  HEAP32[$pKeyMerge>>2] = 0;
 }
 $114 = HEAP32[$pOrderBy>>2]|0;
 $115 = HEAP32[$2>>2]|0;
 $116 = ((($115)) + 44|0);
 HEAP32[$116>>2] = $114;
 $117 = HEAP32[$1>>2]|0;
 $118 = HEAP32[$117>>2]|0;
 $119 = HEAP32[$pOrderBy>>2]|0;
 $120 = (_sqlite3ExprListDup($118,$119,0)|0);
 $121 = HEAP32[$pPrior>>2]|0;
 $122 = ((($121)) + 44|0);
 HEAP32[$122>>2] = $120;
 $123 = HEAP32[$op>>2]|0;
 $124 = ($123|0)==(116);
 L28: do {
  if ($124) {
   HEAP32[$regPrev>>2] = 0;
  } else {
   $125 = HEAP32[$2>>2]|0;
   $126 = HEAP32[$125>>2]|0;
   $127 = HEAP32[$126>>2]|0;
   HEAP32[$nExpr>>2] = $127;
   $128 = HEAP32[$1>>2]|0;
   $129 = ((($128)) + 44|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = (($130) + 1)|0;
   HEAP32[$regPrev>>2] = $131;
   $132 = HEAP32[$nExpr>>2]|0;
   $133 = (($132) + 1)|0;
   $134 = HEAP32[$1>>2]|0;
   $135 = ((($134)) + 44|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = (($136) + ($133))|0;
   HEAP32[$135>>2] = $137;
   $138 = HEAP32[$v>>2]|0;
   $139 = HEAP32[$regPrev>>2]|0;
   (_sqlite3VdbeAddOp2($138,76,0,$139)|0);
   $140 = HEAP32[$db>>2]|0;
   $141 = HEAP32[$nExpr>>2]|0;
   $142 = (_sqlite3KeyInfoAlloc($140,$141,1)|0);
   HEAP32[$pKeyDup>>2] = $142;
   $143 = HEAP32[$pKeyDup>>2]|0;
   $144 = ($143|0)!=(0|0);
   if ($144) {
    HEAP32[$i>>2] = 0;
    while(1) {
     $145 = HEAP32[$i>>2]|0;
     $146 = HEAP32[$nExpr>>2]|0;
     $147 = ($145|0)<($146|0);
     if (!($147)) {
      break L28;
     }
     $148 = HEAP32[$1>>2]|0;
     $149 = HEAP32[$2>>2]|0;
     $150 = HEAP32[$i>>2]|0;
     $151 = (_multiSelectCollSeq($148,$149,$150)|0);
     $152 = HEAP32[$i>>2]|0;
     $153 = HEAP32[$pKeyDup>>2]|0;
     $154 = ((($153)) + 20|0);
     $155 = (($154) + ($152<<2)|0);
     HEAP32[$155>>2] = $151;
     $156 = HEAP32[$i>>2]|0;
     $157 = HEAP32[$pKeyDup>>2]|0;
     $158 = ((($157)) + 16|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = (($159) + ($156)|0);
     HEAP8[$160>>0] = 0;
     $161 = HEAP32[$i>>2]|0;
     $162 = (($161) + 1)|0;
     HEAP32[$i>>2] = $162;
    }
   }
  }
 } while(0);
 $163 = HEAP32[$2>>2]|0;
 $164 = ((($163)) + 48|0);
 HEAP32[$164>>2] = 0;
 $165 = HEAP32[$pPrior>>2]|0;
 $166 = ((($165)) + 52|0);
 HEAP32[$166>>2] = 0;
 $167 = HEAP32[$1>>2]|0;
 $168 = HEAP32[$2>>2]|0;
 $169 = HEAP32[$2>>2]|0;
 $170 = ((($169)) + 44|0);
 $171 = HEAP32[$170>>2]|0;
 (_sqlite3ResolveOrderGroupBy($167,$168,$171,29099)|0);
 $172 = HEAP32[$pPrior>>2]|0;
 $173 = ((($172)) + 48|0);
 $174 = HEAP32[$173>>2]|0;
 $175 = ($174|0)==(0|0);
 if ($175) {
  $176 = HEAP32[$1>>2]|0;
  $177 = HEAP32[$pPrior>>2]|0;
  $178 = HEAP32[$pPrior>>2]|0;
  $179 = ((($178)) + 44|0);
  $180 = HEAP32[$179>>2]|0;
  (_sqlite3ResolveOrderGroupBy($176,$177,$180,29099)|0);
 }
 $181 = HEAP32[$1>>2]|0;
 $182 = HEAP32[$2>>2]|0;
 $183 = HEAP32[$labelEnd>>2]|0;
 _computeLimitRegisters($181,$182,$183);
 $184 = HEAP32[$2>>2]|0;
 $185 = ((($184)) + 12|0);
 $186 = HEAP32[$185>>2]|0;
 $187 = ($186|0)!=(0);
 $188 = HEAP32[$op>>2]|0;
 $189 = ($188|0)==(116);
 $or$cond = $187 & $189;
 if ($or$cond) {
  $190 = HEAP32[$1>>2]|0;
  $191 = ((($190)) + 44|0);
  $192 = HEAP32[$191>>2]|0;
  $193 = (($192) + 1)|0;
  HEAP32[$191>>2] = $193;
  HEAP32[$regLimitA>>2] = $193;
  $194 = HEAP32[$1>>2]|0;
  $195 = ((($194)) + 44|0);
  $196 = HEAP32[$195>>2]|0;
  $197 = (($196) + 1)|0;
  HEAP32[$195>>2] = $197;
  HEAP32[$regLimitB>>2] = $197;
  $198 = HEAP32[$v>>2]|0;
  $199 = HEAP32[$2>>2]|0;
  $200 = ((($199)) + 16|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = ($201|0)!=(0);
  $203 = HEAP32[$2>>2]|0;
  if ($202) {
   $204 = ((($203)) + 16|0);
   $205 = HEAP32[$204>>2]|0;
   $206 = (($205) + 1)|0;
   $210 = $206;
  } else {
   $207 = ((($203)) + 12|0);
   $208 = HEAP32[$207>>2]|0;
   $210 = $208;
  }
  $209 = HEAP32[$regLimitA>>2]|0;
  (_sqlite3VdbeAddOp2($198,84,$210,$209)|0);
  $211 = HEAP32[$v>>2]|0;
  $212 = HEAP32[$regLimitA>>2]|0;
  $213 = HEAP32[$regLimitB>>2]|0;
  (_sqlite3VdbeAddOp2($211,84,$212,$213)|0);
 } else {
  HEAP32[$regLimitB>>2] = 0;
  HEAP32[$regLimitA>>2] = 0;
 }
 $214 = HEAP32[$db>>2]|0;
 $215 = HEAP32[$2>>2]|0;
 $216 = ((($215)) + 56|0);
 $217 = HEAP32[$216>>2]|0;
 _sqlite3ExprDelete($214,$217);
 $218 = HEAP32[$2>>2]|0;
 $219 = ((($218)) + 56|0);
 HEAP32[$219>>2] = 0;
 $220 = HEAP32[$db>>2]|0;
 $221 = HEAP32[$2>>2]|0;
 $222 = ((($221)) + 60|0);
 $223 = HEAP32[$222>>2]|0;
 _sqlite3ExprDelete($220,$223);
 $224 = HEAP32[$2>>2]|0;
 $225 = ((($224)) + 60|0);
 HEAP32[$225>>2] = 0;
 $226 = HEAP32[$1>>2]|0;
 $227 = ((($226)) + 44|0);
 $228 = HEAP32[$227>>2]|0;
 $229 = (($228) + 1)|0;
 HEAP32[$227>>2] = $229;
 HEAP32[$regAddrA>>2] = $229;
 $230 = HEAP32[$1>>2]|0;
 $231 = ((($230)) + 44|0);
 $232 = HEAP32[$231>>2]|0;
 $233 = (($232) + 1)|0;
 HEAP32[$231>>2] = $233;
 HEAP32[$regAddrB>>2] = $233;
 $234 = HEAP32[$1>>2]|0;
 $235 = ((($234)) + 44|0);
 $236 = HEAP32[$235>>2]|0;
 $237 = (($236) + 1)|0;
 HEAP32[$235>>2] = $237;
 HEAP32[$regOutA>>2] = $237;
 $238 = HEAP32[$1>>2]|0;
 $239 = ((($238)) + 44|0);
 $240 = HEAP32[$239>>2]|0;
 $241 = (($240) + 1)|0;
 HEAP32[$239>>2] = $241;
 HEAP32[$regOutB>>2] = $241;
 $242 = HEAP32[$regAddrA>>2]|0;
 _sqlite3SelectDestInit($destA,13,$242);
 $243 = HEAP32[$regAddrB>>2]|0;
 _sqlite3SelectDestInit($destB,13,$243);
 $244 = HEAP32[$v>>2]|0;
 $245 = (_sqlite3VdbeCurrentAddr($244)|0);
 $246 = (($245) + 1)|0;
 HEAP32[$addrSelectA>>2] = $246;
 $247 = HEAP32[$v>>2]|0;
 $248 = HEAP32[$regAddrA>>2]|0;
 $249 = HEAP32[$addrSelectA>>2]|0;
 $250 = (_sqlite3VdbeAddOp3($247,15,$248,0,$249)|0);
 HEAP32[$addr1>>2] = $250;
 $251 = HEAP32[$regLimitA>>2]|0;
 $252 = HEAP32[$pPrior>>2]|0;
 $253 = ((($252)) + 12|0);
 HEAP32[$253>>2] = $251;
 $254 = HEAP32[$1>>2]|0;
 $255 = ((($254)) + 424|0);
 $256 = HEAP32[$255>>2]|0;
 HEAP32[$iSub1>>2] = $256;
 $257 = HEAP32[$1>>2]|0;
 $258 = HEAP32[$pPrior>>2]|0;
 (_sqlite3Select($257,$258,$destA)|0);
 $259 = HEAP32[$v>>2]|0;
 $260 = HEAP32[$regAddrA>>2]|0;
 _sqlite3VdbeEndCoroutine($259,$260);
 $261 = HEAP32[$v>>2]|0;
 $262 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($261,$262);
 $263 = HEAP32[$v>>2]|0;
 $264 = (_sqlite3VdbeCurrentAddr($263)|0);
 $265 = (($264) + 1)|0;
 HEAP32[$addrSelectB>>2] = $265;
 $266 = HEAP32[$v>>2]|0;
 $267 = HEAP32[$regAddrB>>2]|0;
 $268 = HEAP32[$addrSelectB>>2]|0;
 $269 = (_sqlite3VdbeAddOp3($266,15,$267,0,$268)|0);
 HEAP32[$addr1>>2] = $269;
 $270 = HEAP32[$2>>2]|0;
 $271 = ((($270)) + 12|0);
 $272 = HEAP32[$271>>2]|0;
 HEAP32[$savedLimit>>2] = $272;
 $273 = HEAP32[$2>>2]|0;
 $274 = ((($273)) + 16|0);
 $275 = HEAP32[$274>>2]|0;
 HEAP32[$savedOffset>>2] = $275;
 $276 = HEAP32[$regLimitB>>2]|0;
 $277 = HEAP32[$2>>2]|0;
 $278 = ((($277)) + 12|0);
 HEAP32[$278>>2] = $276;
 $279 = HEAP32[$2>>2]|0;
 $280 = ((($279)) + 16|0);
 HEAP32[$280>>2] = 0;
 $281 = HEAP32[$1>>2]|0;
 $282 = ((($281)) + 424|0);
 $283 = HEAP32[$282>>2]|0;
 HEAP32[$iSub2>>2] = $283;
 $284 = HEAP32[$1>>2]|0;
 $285 = HEAP32[$2>>2]|0;
 (_sqlite3Select($284,$285,$destB)|0);
 $286 = HEAP32[$savedLimit>>2]|0;
 $287 = HEAP32[$2>>2]|0;
 $288 = ((($287)) + 12|0);
 HEAP32[$288>>2] = $286;
 $289 = HEAP32[$savedOffset>>2]|0;
 $290 = HEAP32[$2>>2]|0;
 $291 = ((($290)) + 16|0);
 HEAP32[$291>>2] = $289;
 $292 = HEAP32[$v>>2]|0;
 $293 = HEAP32[$regAddrB>>2]|0;
 _sqlite3VdbeEndCoroutine($292,$293);
 $294 = HEAP32[$1>>2]|0;
 $295 = HEAP32[$2>>2]|0;
 $296 = HEAP32[$3>>2]|0;
 $297 = HEAP32[$regOutA>>2]|0;
 $298 = HEAP32[$regPrev>>2]|0;
 $299 = HEAP32[$pKeyDup>>2]|0;
 $300 = HEAP32[$labelEnd>>2]|0;
 $301 = (_generateOutputSubroutine($294,$295,$destA,$296,$297,$298,$299,$300)|0);
 HEAP32[$addrOutA>>2] = $301;
 $302 = HEAP32[$op>>2]|0;
 $303 = ($302|0)==(116);
 $304 = HEAP32[$op>>2]|0;
 $305 = ($304|0)==(115);
 $or$cond3 = $303 | $305;
 if ($or$cond3) {
  $306 = HEAP32[$1>>2]|0;
  $307 = HEAP32[$2>>2]|0;
  $308 = HEAP32[$3>>2]|0;
  $309 = HEAP32[$regOutB>>2]|0;
  $310 = HEAP32[$regPrev>>2]|0;
  $311 = HEAP32[$pKeyDup>>2]|0;
  $312 = HEAP32[$labelEnd>>2]|0;
  $313 = (_generateOutputSubroutine($306,$307,$destB,$308,$309,$310,$311,$312)|0);
  HEAP32[$addrOutB>>2] = $313;
 }
 $314 = HEAP32[$pKeyDup>>2]|0;
 _sqlite3KeyInfoUnref($314);
 $315 = HEAP32[$op>>2]|0;
 $316 = ($315|0)==(117);
 $317 = HEAP32[$op>>2]|0;
 $318 = ($317|0)==(118);
 $or$cond5 = $316 | $318;
 if ($or$cond5) {
  $319 = HEAP32[$labelEnd>>2]|0;
  HEAP32[$addrEofA>>2] = $319;
  HEAP32[$addrEofA_noB>>2] = $319;
 } else {
  $320 = HEAP32[$v>>2]|0;
  $321 = HEAP32[$regOutB>>2]|0;
  $322 = HEAP32[$addrOutB>>2]|0;
  $323 = (_sqlite3VdbeAddOp2($320,14,$321,$322)|0);
  HEAP32[$addrEofA>>2] = $323;
  $324 = HEAP32[$v>>2]|0;
  $325 = HEAP32[$regAddrB>>2]|0;
  $326 = HEAP32[$labelEnd>>2]|0;
  $327 = (_sqlite3VdbeAddOp2($324,16,$325,$326)|0);
  HEAP32[$addrEofA_noB>>2] = $327;
  $328 = HEAP32[$v>>2]|0;
  $329 = HEAP32[$addrEofA>>2]|0;
  (_sqlite3VdbeGoto($328,$329)|0);
  $330 = HEAP32[$2>>2]|0;
  $331 = ((($330)) + 6|0);
  $332 = HEAP16[$331>>1]|0;
  $333 = HEAP32[$pPrior>>2]|0;
  $334 = ((($333)) + 6|0);
  $335 = HEAP16[$334>>1]|0;
  $336 = (_sqlite3LogEstAdd($332,$335)|0);
  $337 = HEAP32[$2>>2]|0;
  $338 = ((($337)) + 6|0);
  HEAP16[$338>>1] = $336;
 }
 $339 = HEAP32[$op>>2]|0;
 $340 = ($339|0)==(118);
 if ($340) {
  $341 = HEAP32[$addrEofA>>2]|0;
  HEAP32[$addrEofB>>2] = $341;
  $342 = HEAP32[$2>>2]|0;
  $343 = ((($342)) + 6|0);
  $344 = HEAP16[$343>>1]|0;
  $345 = $344 << 16 >> 16;
  $346 = HEAP32[$pPrior>>2]|0;
  $347 = ((($346)) + 6|0);
  $348 = HEAP16[$347>>1]|0;
  $349 = $348 << 16 >> 16;
  $350 = ($345|0)>($349|0);
  if ($350) {
   $351 = HEAP32[$pPrior>>2]|0;
   $352 = ((($351)) + 6|0);
   $353 = HEAP16[$352>>1]|0;
   $354 = HEAP32[$2>>2]|0;
   $355 = ((($354)) + 6|0);
   HEAP16[$355>>1] = $353;
  }
 } else {
  $356 = HEAP32[$v>>2]|0;
  $357 = HEAP32[$regOutA>>2]|0;
  $358 = HEAP32[$addrOutA>>2]|0;
  $359 = (_sqlite3VdbeAddOp2($356,14,$357,$358)|0);
  HEAP32[$addrEofB>>2] = $359;
  $360 = HEAP32[$v>>2]|0;
  $361 = HEAP32[$regAddrA>>2]|0;
  $362 = HEAP32[$labelEnd>>2]|0;
  (_sqlite3VdbeAddOp2($360,16,$361,$362)|0);
  $363 = HEAP32[$v>>2]|0;
  $364 = HEAP32[$addrEofB>>2]|0;
  (_sqlite3VdbeGoto($363,$364)|0);
 }
 $365 = HEAP32[$v>>2]|0;
 $366 = HEAP32[$regOutA>>2]|0;
 $367 = HEAP32[$addrOutA>>2]|0;
 $368 = (_sqlite3VdbeAddOp2($365,14,$366,$367)|0);
 HEAP32[$addrAltB>>2] = $368;
 $369 = HEAP32[$v>>2]|0;
 $370 = HEAP32[$regAddrA>>2]|0;
 $371 = HEAP32[$addrEofA>>2]|0;
 (_sqlite3VdbeAddOp2($369,16,$370,$371)|0);
 $372 = HEAP32[$v>>2]|0;
 $373 = HEAP32[$labelCmpr>>2]|0;
 (_sqlite3VdbeGoto($372,$373)|0);
 $374 = HEAP32[$op>>2]|0;
 $375 = ($374|0)==(116);
 do {
  if ($375) {
   $376 = HEAP32[$addrAltB>>2]|0;
   HEAP32[$addrAeqB>>2] = $376;
  } else {
   $377 = HEAP32[$op>>2]|0;
   $378 = ($377|0)==(118);
   if ($378) {
    $379 = HEAP32[$addrAltB>>2]|0;
    HEAP32[$addrAeqB>>2] = $379;
    $380 = HEAP32[$addrAltB>>2]|0;
    $381 = (($380) + 1)|0;
    HEAP32[$addrAltB>>2] = $381;
    break;
   } else {
    $382 = HEAP32[$v>>2]|0;
    $383 = HEAP32[$regAddrA>>2]|0;
    $384 = HEAP32[$addrEofA>>2]|0;
    $385 = (_sqlite3VdbeAddOp2($382,16,$383,$384)|0);
    HEAP32[$addrAeqB>>2] = $385;
    $386 = HEAP32[$v>>2]|0;
    $387 = HEAP32[$labelCmpr>>2]|0;
    (_sqlite3VdbeGoto($386,$387)|0);
    break;
   }
  }
 } while(0);
 $388 = HEAP32[$v>>2]|0;
 $389 = (_sqlite3VdbeCurrentAddr($388)|0);
 HEAP32[$addrAgtB>>2] = $389;
 $390 = HEAP32[$op>>2]|0;
 $391 = ($390|0)==(116);
 $392 = HEAP32[$op>>2]|0;
 $393 = ($392|0)==(115);
 $or$cond7 = $391 | $393;
 if ($or$cond7) {
  $394 = HEAP32[$v>>2]|0;
  $395 = HEAP32[$regOutB>>2]|0;
  $396 = HEAP32[$addrOutB>>2]|0;
  (_sqlite3VdbeAddOp2($394,14,$395,$396)|0);
 }
 $397 = HEAP32[$v>>2]|0;
 $398 = HEAP32[$regAddrB>>2]|0;
 $399 = HEAP32[$addrEofB>>2]|0;
 (_sqlite3VdbeAddOp2($397,16,$398,$399)|0);
 $400 = HEAP32[$v>>2]|0;
 $401 = HEAP32[$labelCmpr>>2]|0;
 (_sqlite3VdbeGoto($400,$401)|0);
 $402 = HEAP32[$v>>2]|0;
 $403 = HEAP32[$addr1>>2]|0;
 _sqlite3VdbeJumpHere($402,$403);
 $404 = HEAP32[$v>>2]|0;
 $405 = HEAP32[$regAddrA>>2]|0;
 $406 = HEAP32[$addrEofA_noB>>2]|0;
 (_sqlite3VdbeAddOp2($404,16,$405,$406)|0);
 $407 = HEAP32[$v>>2]|0;
 $408 = HEAP32[$regAddrB>>2]|0;
 $409 = HEAP32[$addrEofB>>2]|0;
 (_sqlite3VdbeAddOp2($407,16,$408,$409)|0);
 $410 = HEAP32[$v>>2]|0;
 $411 = HEAP32[$labelCmpr>>2]|0;
 _sqlite3VdbeResolveLabel($410,$411);
 $412 = HEAP32[$v>>2]|0;
 $413 = HEAP32[$aPermute>>2]|0;
 (_sqlite3VdbeAddOp4($412,94,0,0,0,$413,-15)|0);
 $414 = HEAP32[$v>>2]|0;
 $415 = ((($destA)) + 12|0);
 $416 = HEAP32[$415>>2]|0;
 $417 = ((($destB)) + 12|0);
 $418 = HEAP32[$417>>2]|0;
 $419 = HEAP32[$nOrderBy>>2]|0;
 $420 = HEAP32[$pKeyMerge>>2]|0;
 (_sqlite3VdbeAddOp4($414,95,$416,$418,$419,$420,-6)|0);
 $421 = HEAP32[$v>>2]|0;
 _sqlite3VdbeChangeP5($421,1);
 $422 = HEAP32[$v>>2]|0;
 $423 = HEAP32[$addrAltB>>2]|0;
 $424 = HEAP32[$addrAeqB>>2]|0;
 $425 = HEAP32[$addrAgtB>>2]|0;
 (_sqlite3VdbeAddOp3($422,18,$423,$424,$425)|0);
 $426 = HEAP32[$v>>2]|0;
 $427 = HEAP32[$labelEnd>>2]|0;
 _sqlite3VdbeResolveLabel($426,$427);
 $428 = HEAP32[$3>>2]|0;
 $429 = HEAP8[$428>>0]|0;
 $430 = $429&255;
 $431 = ($430|0)==(9);
 if ($431) {
  $432 = HEAP32[$pPrior>>2]|0;
  HEAP32[$pFirst>>2] = $432;
  while(1) {
   $433 = HEAP32[$pFirst>>2]|0;
   $434 = ((($433)) + 48|0);
   $435 = HEAP32[$434>>2]|0;
   $436 = ($435|0)!=(0|0);
   if (!($436)) {
    break;
   }
   $437 = HEAP32[$pFirst>>2]|0;
   $438 = ((($437)) + 48|0);
   $439 = HEAP32[$438>>2]|0;
   HEAP32[$pFirst>>2] = $439;
  }
  $440 = HEAP32[$1>>2]|0;
  $441 = HEAP32[$pFirst>>2]|0;
  $442 = ((($441)) + 28|0);
  $443 = HEAP32[$442>>2]|0;
  $444 = HEAP32[$pFirst>>2]|0;
  $445 = HEAP32[$444>>2]|0;
  _generateColumnNames($440,$443,$445);
 }
 $446 = HEAP32[$2>>2]|0;
 $447 = ((($446)) + 48|0);
 $448 = HEAP32[$447>>2]|0;
 $449 = ($448|0)!=(0|0);
 if ($449) {
  $450 = HEAP32[$db>>2]|0;
  $451 = HEAP32[$2>>2]|0;
  $452 = ((($451)) + 48|0);
  $453 = HEAP32[$452>>2]|0;
  _sqlite3SelectDelete($450,$453);
 }
 $454 = HEAP32[$pPrior>>2]|0;
 $455 = HEAP32[$2>>2]|0;
 $456 = ((($455)) + 48|0);
 HEAP32[$456>>2] = $454;
 $457 = HEAP32[$2>>2]|0;
 $458 = HEAP32[$pPrior>>2]|0;
 $459 = ((($458)) + 52|0);
 HEAP32[$459>>2] = $457;
 $460 = HEAP32[$1>>2]|0;
 $461 = HEAP32[$2>>2]|0;
 $462 = ((($461)) + 4|0);
 $463 = HEAP8[$462>>0]|0;
 $464 = $463&255;
 $465 = HEAP32[$iSub1>>2]|0;
 $466 = HEAP32[$iSub2>>2]|0;
 _explainComposite($460,$464,$465,$466,0);
 $467 = HEAP32[$1>>2]|0;
 $468 = ((($467)) + 36|0);
 $469 = HEAP32[$468>>2]|0;
 $470 = ($469|0)!=(0);
 $471 = $470&1;
 HEAP32[$0>>2] = $471;
 $472 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($472|0);
}
function _explainComposite($pParse,$op,$iSub1,$iSub2,$bUseTmp) {
 $pParse = $pParse|0;
 $op = $op|0;
 $iSub1 = $iSub1|0;
 $iSub2 = $iSub2|0;
 $bUseTmp = $bUseTmp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zMsg = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $v = sp + 20|0;
 $zMsg = sp + 16|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSub1;
 HEAP32[$3>>2] = $iSub2;
 HEAP32[$4>>2] = $bUseTmp;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 409|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$v>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ($17|0)!=(0);
 $19 = $18 ? 35924 : 18402;
 $20 = HEAP32[$1>>2]|0;
 $21 = (_selectOpName($20)|0);
 HEAP32[$vararg_buffer>>2] = $15;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $16;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $19;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $21;
 $22 = (_sqlite3MPrintf($14,35943,$vararg_buffer)|0);
 HEAP32[$zMsg>>2] = $22;
 $23 = HEAP32[$v>>2]|0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 420|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$zMsg>>2]|0;
 (_sqlite3VdbeAddOp4($23,162,$26,0,0,$27,-1)|0);
 STACKTOP = sp;return;
}
function _multiSelectCollSeq($pParse,$p,$iCol) {
 $pParse = $pParse|0;
 $p = $p|0;
 $iCol = $iCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRet = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pRet = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 48|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 48|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = (_multiSelectCollSeq($7,$10,$11)|0);
  HEAP32[$pRet>>2] = $12;
 } else {
  HEAP32[$pRet>>2] = 0;
 }
 $13 = HEAP32[$pRet>>2]|0;
 $14 = ($13|0)==(0|0);
 if (!($14)) {
  $29 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 $19 = ($15|0)<($18|0);
 if (!($19)) {
  $29 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + (($21*20)|0)|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (_sqlite3ExprCollSeq($20,$27)|0);
 HEAP32[$pRet>>2] = $28;
 $29 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _multiSelectOrderByKeyInfo($pParse,$p,$nExtra) {
 $pParse = $pParse|0;
 $p = $p|0;
 $nExtra = $nExtra|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $db = 0;
 var $i = 0, $nOrderBy = 0, $pColl = 0, $pItem = 0, $pOrderBy = 0, $pRet = 0, $pTerm = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $pOrderBy = sp + 28|0;
 $nOrderBy = sp + 24|0;
 $db = sp + 20|0;
 $pRet = sp + 16|0;
 $i = sp + 12|0;
 $pItem = sp + 8|0;
 $pTerm = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nExtra;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pOrderBy>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$nOrderBy>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$nOrderBy>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = (_sqlite3KeyInfoAlloc($12,$15,1)|0);
 HEAP32[$pRet>>2] = $16;
 $17 = HEAP32[$pRet>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  $78 = HEAP32[$pRet>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $19 = HEAP32[$i>>2]|0;
  $20 = HEAP32[$nOrderBy>>2]|0;
  $21 = ($19|0)<($20|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$i>>2]|0;
  $23 = HEAP32[$pOrderBy>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + (($22*20)|0)|0);
  HEAP32[$pItem>>2] = $26;
  $27 = HEAP32[$pItem>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pTerm>>2] = $28;
  $29 = HEAP32[$pTerm>>2]|0;
  $30 = ((($29)) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = $31 & 256;
  $33 = ($32|0)!=(0);
  $34 = HEAP32[$0>>2]|0;
  if ($33) {
   $35 = HEAP32[$pTerm>>2]|0;
   $36 = (_sqlite3ExprCollSeq($34,$35)|0);
   HEAP32[$pColl>>2] = $36;
  } else {
   $37 = HEAP32[$1>>2]|0;
   $38 = HEAP32[$pItem>>2]|0;
   $39 = ((($38)) + 16|0);
   $40 = HEAP16[$39>>1]|0;
   $41 = $40&65535;
   $42 = (($41) - 1)|0;
   $43 = (_multiSelectCollSeq($34,$37,$42)|0);
   HEAP32[$pColl>>2] = $43;
   $44 = HEAP32[$pColl>>2]|0;
   $45 = ($44|0)==(0|0);
   if ($45) {
    $46 = HEAP32[$db>>2]|0;
    $47 = ((($46)) + 8|0);
    $48 = HEAP32[$47>>2]|0;
    HEAP32[$pColl>>2] = $48;
   }
   $49 = HEAP32[$0>>2]|0;
   $50 = HEAP32[$pTerm>>2]|0;
   $51 = HEAP32[$pColl>>2]|0;
   $52 = HEAP32[$51>>2]|0;
   $53 = (_sqlite3ExprAddCollateString($49,$50,$52)|0);
   $54 = HEAP32[$i>>2]|0;
   $55 = HEAP32[$pOrderBy>>2]|0;
   $56 = ((($55)) + 4|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = (($57) + (($54*20)|0)|0);
   HEAP32[$58>>2] = $53;
  }
  $59 = HEAP32[$pColl>>2]|0;
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$pRet>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = (($62) + ($60<<2)|0);
  HEAP32[$63>>2] = $59;
  $64 = HEAP32[$i>>2]|0;
  $65 = HEAP32[$pOrderBy>>2]|0;
  $66 = ((($65)) + 4|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (($67) + (($64*20)|0)|0);
  $69 = ((($68)) + 12|0);
  $70 = HEAP8[$69>>0]|0;
  $71 = HEAP32[$i>>2]|0;
  $72 = HEAP32[$pRet>>2]|0;
  $73 = ((($72)) + 16|0);
  $74 = HEAP32[$73>>2]|0;
  $75 = (($74) + ($71)|0);
  HEAP8[$75>>0] = $70;
  $76 = HEAP32[$i>>2]|0;
  $77 = (($76) + 1)|0;
  HEAP32[$i>>2] = $77;
 }
 $78 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _generateOutputSubroutine($pParse,$p,$pIn,$pDest,$regReturn,$regPrev,$pKeyInfo,$iBreak) {
 $pParse = $pParse|0;
 $p = $p|0;
 $pIn = $pIn|0;
 $pDest = $pDest|0;
 $regReturn = $regReturn|0;
 $regPrev = $regPrev|0;
 $pKeyInfo = $pKeyInfo|0;
 $iBreak = $iBreak|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $addr = 0, $addr1 = 0, $addr2 = 0, $iContinue = 0, $r1 = 0, $r11 = 0, $r2 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $4 = sp + 48|0;
 $5 = sp + 44|0;
 $6 = sp + 40|0;
 $7 = sp + 36|0;
 $8 = sp + 32|0;
 $v = sp + 28|0;
 $iContinue = sp + 24|0;
 $addr = sp + 20|0;
 $addr1 = sp + 16|0;
 $addr2 = sp + 12|0;
 $r1 = sp + 8|0;
 $r2 = sp + 4|0;
 $r11 = sp;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $p;
 HEAP32[$3>>2] = $pIn;
 HEAP32[$4>>2] = $pDest;
 HEAP32[$5>>2] = $regReturn;
 HEAP32[$6>>2] = $regPrev;
 HEAP32[$7>>2] = $pKeyInfo;
 HEAP32[$8>>2] = $iBreak;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$v>>2] = $11;
 $12 = HEAP32[$v>>2]|0;
 $13 = (_sqlite3VdbeCurrentAddr($12)|0);
 HEAP32[$addr>>2] = $13;
 $14 = HEAP32[$v>>2]|0;
 $15 = (_sqlite3VdbeMakeLabel($14)|0);
 HEAP32[$iContinue>>2] = $15;
 $16 = HEAP32[$6>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$v>>2]|0;
  $19 = HEAP32[$6>>2]|0;
  $20 = (_sqlite3VdbeAddOp1($18,22,$19)|0);
  HEAP32[$addr1>>2] = $20;
  $21 = HEAP32[$v>>2]|0;
  $22 = HEAP32[$3>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$6>>2]|0;
  $26 = (($25) + 1)|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = ((($27)) + 16|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$7>>2]|0;
  $31 = (_sqlite3KeyInfoRef($30)|0);
  $32 = (_sqlite3VdbeAddOp4($21,95,$24,$26,$29,$31,-6)|0);
  HEAP32[$addr2>>2] = $32;
  $33 = HEAP32[$v>>2]|0;
  $34 = HEAP32[$addr2>>2]|0;
  $35 = (($34) + 2)|0;
  $36 = HEAP32[$iContinue>>2]|0;
  $37 = HEAP32[$addr2>>2]|0;
  $38 = (($37) + 2)|0;
  (_sqlite3VdbeAddOp3($33,18,$35,$36,$38)|0);
  $39 = HEAP32[$v>>2]|0;
  $40 = HEAP32[$addr1>>2]|0;
  _sqlite3VdbeJumpHere($39,$40);
  $41 = HEAP32[$v>>2]|0;
  $42 = HEAP32[$3>>2]|0;
  $43 = ((($42)) + 12|0);
  $44 = HEAP32[$43>>2]|0;
  $45 = HEAP32[$6>>2]|0;
  $46 = (($45) + 1)|0;
  $47 = HEAP32[$3>>2]|0;
  $48 = ((($47)) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) - 1)|0;
  (_sqlite3VdbeAddOp3($41,84,$44,$46,$50)|0);
  $51 = HEAP32[$v>>2]|0;
  $52 = HEAP32[$6>>2]|0;
  (_sqlite3VdbeAddOp2($51,76,1,$52)|0);
 }
 $53 = HEAP32[$1>>2]|0;
 $54 = HEAP32[$53>>2]|0;
 $55 = ((($54)) + 69|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = ($56<<24>>24)!=(0);
 if ($57) {
  HEAP32[$0>>2] = 0;
  $189 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($189|0);
 }
 $58 = HEAP32[$v>>2]|0;
 $59 = HEAP32[$2>>2]|0;
 $60 = ((($59)) + 16|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$iContinue>>2]|0;
 _codeOffset($58,$61,$62);
 $63 = HEAP32[$4>>2]|0;
 $64 = HEAP8[$63>>0]|0;
 $65 = $64&255;
 switch ($65|0) {
 case 12:  {
  $66 = HEAP32[$1>>2]|0;
  $67 = (_sqlite3GetTempReg($66)|0);
  HEAP32[$r1>>2] = $67;
  $68 = HEAP32[$1>>2]|0;
  $69 = (_sqlite3GetTempReg($68)|0);
  HEAP32[$r2>>2] = $69;
  $70 = HEAP32[$v>>2]|0;
  $71 = HEAP32[$3>>2]|0;
  $72 = ((($71)) + 12|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = HEAP32[$3>>2]|0;
  $75 = ((($74)) + 16|0);
  $76 = HEAP32[$75>>2]|0;
  $77 = HEAP32[$r1>>2]|0;
  (_sqlite3VdbeAddOp3($70,99,$73,$76,$77)|0);
  $78 = HEAP32[$v>>2]|0;
  $79 = HEAP32[$4>>2]|0;
  $80 = ((($79)) + 8|0);
  $81 = HEAP32[$80>>2]|0;
  $82 = HEAP32[$r2>>2]|0;
  (_sqlite3VdbeAddOp2($78,114,$81,$82)|0);
  $83 = HEAP32[$v>>2]|0;
  $84 = HEAP32[$4>>2]|0;
  $85 = ((($84)) + 8|0);
  $86 = HEAP32[$85>>2]|0;
  $87 = HEAP32[$r1>>2]|0;
  $88 = HEAP32[$r2>>2]|0;
  (_sqlite3VdbeAddOp3($83,115,$86,$87,$88)|0);
  $89 = HEAP32[$v>>2]|0;
  _sqlite3VdbeChangeP5($89,8);
  $90 = HEAP32[$1>>2]|0;
  $91 = HEAP32[$r2>>2]|0;
  _sqlite3ReleaseTempReg($90,$91);
  $92 = HEAP32[$1>>2]|0;
  $93 = HEAP32[$r1>>2]|0;
  _sqlite3ReleaseTempReg($92,$93);
  break;
 }
 case 11:  {
  $94 = HEAP32[$1>>2]|0;
  $95 = (_sqlite3GetTempReg($94)|0);
  HEAP32[$r11>>2] = $95;
  $96 = HEAP32[$v>>2]|0;
  $97 = HEAP32[$3>>2]|0;
  $98 = ((($97)) + 12|0);
  $99 = HEAP32[$98>>2]|0;
  $100 = HEAP32[$3>>2]|0;
  $101 = ((($100)) + 16|0);
  $102 = HEAP32[$101>>2]|0;
  $103 = HEAP32[$r11>>2]|0;
  $104 = HEAP32[$4>>2]|0;
  $105 = ((($104)) + 4|0);
  $106 = HEAP32[$105>>2]|0;
  $107 = HEAP32[$3>>2]|0;
  $108 = ((($107)) + 16|0);
  $109 = HEAP32[$108>>2]|0;
  (_sqlite3VdbeAddOp4($96,99,$99,$102,$103,$106,$109)|0);
  $110 = HEAP32[$1>>2]|0;
  $111 = HEAP32[$3>>2]|0;
  $112 = ((($111)) + 12|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = HEAP32[$3>>2]|0;
  $115 = ((($114)) + 16|0);
  $116 = HEAP32[$115>>2]|0;
  _sqlite3ExprCacheAffinityChange($110,$113,$116);
  $117 = HEAP32[$v>>2]|0;
  $118 = HEAP32[$4>>2]|0;
  $119 = ((($118)) + 8|0);
  $120 = HEAP32[$119>>2]|0;
  $121 = HEAP32[$r11>>2]|0;
  (_sqlite3VdbeAddOp2($117,126,$120,$121)|0);
  $122 = HEAP32[$1>>2]|0;
  $123 = HEAP32[$r11>>2]|0;
  _sqlite3ReleaseTempReg($122,$123);
  break;
 }
 case 10:  {
  $124 = HEAP32[$1>>2]|0;
  $125 = HEAP32[$3>>2]|0;
  $126 = ((($125)) + 12|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$4>>2]|0;
  $129 = ((($128)) + 8|0);
  $130 = HEAP32[$129>>2]|0;
  _sqlite3ExprCodeMove($124,$127,$130,1);
  break;
 }
 case 13:  {
  $131 = HEAP32[$4>>2]|0;
  $132 = ((($131)) + 12|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = ($133|0)==(0);
  if ($134) {
   $135 = HEAP32[$1>>2]|0;
   $136 = HEAP32[$3>>2]|0;
   $137 = ((($136)) + 16|0);
   $138 = HEAP32[$137>>2]|0;
   $139 = (_sqlite3GetTempRange($135,$138)|0);
   $140 = HEAP32[$4>>2]|0;
   $141 = ((($140)) + 12|0);
   HEAP32[$141>>2] = $139;
   $142 = HEAP32[$3>>2]|0;
   $143 = ((($142)) + 16|0);
   $144 = HEAP32[$143>>2]|0;
   $145 = HEAP32[$4>>2]|0;
   $146 = ((($145)) + 16|0);
   HEAP32[$146>>2] = $144;
  }
  $147 = HEAP32[$1>>2]|0;
  $148 = HEAP32[$3>>2]|0;
  $149 = ((($148)) + 12|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = HEAP32[$4>>2]|0;
  $152 = ((($151)) + 12|0);
  $153 = HEAP32[$152>>2]|0;
  $154 = HEAP32[$3>>2]|0;
  $155 = ((($154)) + 16|0);
  $156 = HEAP32[$155>>2]|0;
  _sqlite3ExprCodeMove($147,$150,$153,$156);
  $157 = HEAP32[$v>>2]|0;
  $158 = HEAP32[$4>>2]|0;
  $159 = ((($158)) + 8|0);
  $160 = HEAP32[$159>>2]|0;
  (_sqlite3VdbeAddOp1($157,16,$160)|0);
  break;
 }
 default: {
  $161 = HEAP32[$v>>2]|0;
  $162 = HEAP32[$3>>2]|0;
  $163 = ((($162)) + 12|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = HEAP32[$3>>2]|0;
  $166 = ((($165)) + 16|0);
  $167 = HEAP32[$166>>2]|0;
  (_sqlite3VdbeAddOp2($161,87,$164,$167)|0);
  $168 = HEAP32[$1>>2]|0;
  $169 = HEAP32[$3>>2]|0;
  $170 = ((($169)) + 12|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = HEAP32[$3>>2]|0;
  $173 = ((($172)) + 16|0);
  $174 = HEAP32[$173>>2]|0;
  _sqlite3ExprCacheAffinityChange($168,$171,$174);
 }
 }
 $175 = HEAP32[$2>>2]|0;
 $176 = ((($175)) + 12|0);
 $177 = HEAP32[$176>>2]|0;
 $178 = ($177|0)!=(0);
 if ($178) {
  $179 = HEAP32[$v>>2]|0;
  $180 = HEAP32[$2>>2]|0;
  $181 = ((($180)) + 12|0);
  $182 = HEAP32[$181>>2]|0;
  $183 = HEAP32[$8>>2]|0;
  (_sqlite3VdbeAddOp2($179,68,$182,$183)|0);
 }
 $184 = HEAP32[$v>>2]|0;
 $185 = HEAP32[$iContinue>>2]|0;
 _sqlite3VdbeResolveLabel($184,$185);
 $186 = HEAP32[$v>>2]|0;
 $187 = HEAP32[$5>>2]|0;
 (_sqlite3VdbeAddOp1($186,72,$187)|0);
 $188 = HEAP32[$addr>>2]|0;
 HEAP32[$0>>2] = $188;
 $189 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($189|0);
}
function _sqlite3FkDropTable($pParse,$pName,$pTab) {
 $pParse = $pParse|0;
 $pName = $pName|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $db = 0, $iSkip = 0, $p = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $db = sp + 12|0;
 $iSkip = sp + 8|0;
 $v = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pName;
 HEAP32[$2>>2] = $pTab;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$db>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = $7 & 524288;
 $9 = ($8|0)!=(0);
 if (!($9)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 42|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = $13 & 16;
 $15 = ($14|0)!=(0);
 if ($15) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 if ($19) {
  STACKTOP = sp;return;
 }
 HEAP32[$iSkip>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = (_sqlite3GetVdbe($20)|0);
 HEAP32[$v>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = (_sqlite3FkReferences($22)|0);
 $24 = ($23|0)==(0|0);
 do {
  if ($24) {
   $25 = HEAP32[$2>>2]|0;
   $26 = ((($25)) + 16|0);
   $27 = HEAP32[$26>>2]|0;
   HEAP32[$p>>2] = $27;
   while(1) {
    $28 = HEAP32[$p>>2]|0;
    $29 = ($28|0)!=(0|0);
    if (!($29)) {
     break;
    }
    $30 = HEAP32[$p>>2]|0;
    $31 = ((($30)) + 24|0);
    $32 = HEAP8[$31>>0]|0;
    $33 = $32&255;
    $34 = ($33|0)!=(0);
    if ($34) {
     break;
    }
    $35 = HEAP32[$db>>2]|0;
    $36 = ((($35)) + 24|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = $37 & 33554432;
    $39 = ($38|0)!=(0);
    if ($39) {
     break;
    }
    $40 = HEAP32[$p>>2]|0;
    $41 = ((($40)) + 4|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$p>>2] = $42;
   }
   $43 = HEAP32[$p>>2]|0;
   $44 = ($43|0)!=(0|0);
   if ($44) {
    $45 = HEAP32[$v>>2]|0;
    $46 = (_sqlite3VdbeMakeLabel($45)|0);
    HEAP32[$iSkip>>2] = $46;
    $47 = HEAP32[$v>>2]|0;
    $48 = HEAP32[$iSkip>>2]|0;
    (_sqlite3VdbeAddOp2($47,65,1,$48)|0);
    break;
   } else {
    STACKTOP = sp;return;
   }
  }
 } while(0);
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 150|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$db>>2]|0;
 $53 = HEAP32[$1>>2]|0;
 $54 = (_sqlite3SrcListDup($52,$53,0)|0);
 _sqlite3DeleteFrom($51,$54,0);
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 150|0);
 HEAP8[$56>>0] = 0;
 $57 = HEAP32[$db>>2]|0;
 $58 = ((($57)) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = $59 & 33554432;
 $61 = ($60|0)==(0);
 if ($61) {
  $62 = HEAP32[$v>>2]|0;
  $63 = HEAP32[$v>>2]|0;
  $64 = (_sqlite3VdbeCurrentAddr($63)|0);
  $65 = (($64) + 2)|0;
  (_sqlite3VdbeAddOp2($62,65,0,$65)|0);
  $66 = HEAP32[$0>>2]|0;
  _sqlite3HaltConstraint($66,787,2,0,-2,4);
 }
 $67 = HEAP32[$iSkip>>2]|0;
 $68 = ($67|0)!=(0);
 if (!($68)) {
  STACKTOP = sp;return;
 }
 $69 = HEAP32[$v>>2]|0;
 $70 = HEAP32[$iSkip>>2]|0;
 _sqlite3VdbeResolveLabel($69,$70);
 STACKTOP = sp;return;
}
function _sqlite3CodeDropTable($pParse,$pTab,$iDb,$isView) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $iDb = $iDb|0;
 $isView = $isView|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pDb = 0, $pTrigger = 0, $v = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $v = sp + 32|0;
 $db = sp + 28|0;
 $pTrigger = sp + 24|0;
 $pDb = sp + 20|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $isView;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 HEAP32[$pDb>>2] = $10;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3GetVdbe($11)|0);
 HEAP32[$v>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 _sqlite3BeginWriteOperation($13,1,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 42|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 & 16;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$v>>2]|0;
  (_sqlite3VdbeAddOp0($21,152)|0);
 }
 $22 = HEAP32[$0>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = (_sqlite3TriggerList($22,$23)|0);
 HEAP32[$pTrigger>>2] = $24;
 while(1) {
  $25 = HEAP32[$pTrigger>>2]|0;
  $26 = ($25|0)!=(0|0);
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$pTrigger>>2]|0;
  _sqlite3DropTriggerPtr($27,$28);
  $29 = HEAP32[$pTrigger>>2]|0;
  $30 = ((($29)) + 32|0);
  $31 = HEAP32[$30>>2]|0;
  HEAP32[$pTrigger>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 42|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = $35 & 8;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$0>>2]|0;
  $39 = HEAP32[$pDb>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$1>>2]|0;
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$vararg_buffer>>2] = $40;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $42;
  _sqlite3NestedParse($38,36169,$vararg_buffer);
 }
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$pDb>>2]|0;
 $45 = HEAP32[$44>>2]|0;
 $46 = HEAP32[$2>>2]|0;
 $47 = ($46|0)==(1);
 $48 = $47 ? 19905 : 19924;
 $49 = HEAP32[$1>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $45;
 $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $48;
 $vararg_ptr6 = ((($vararg_buffer2)) + 8|0);
 HEAP32[$vararg_ptr6>>2] = $50;
 _sqlite3NestedParse($43,36214,$vararg_buffer2);
 $51 = HEAP32[$3>>2]|0;
 $52 = ($51|0)!=(0);
 if (!($52)) {
  $53 = HEAP32[$1>>2]|0;
  $54 = ((($53)) + 42|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = $55&255;
  $57 = $56 & 16;
  $58 = ($57|0)!=(0);
  if (!($58)) {
   $59 = HEAP32[$0>>2]|0;
   $60 = HEAP32[$1>>2]|0;
   _destroyTable($59,$60);
  }
 }
 $61 = HEAP32[$1>>2]|0;
 $62 = ((($61)) + 42|0);
 $63 = HEAP8[$62>>0]|0;
 $64 = $63&255;
 $65 = $64 & 16;
 $66 = ($65|0)!=(0);
 if (!($66)) {
  $71 = HEAP32[$v>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  $73 = HEAP32[$1>>2]|0;
  $74 = HEAP32[$73>>2]|0;
  (_sqlite3VdbeAddOp4($71,138,$72,0,0,$74,0)|0);
  $75 = HEAP32[$0>>2]|0;
  $76 = HEAP32[$2>>2]|0;
  _sqlite3ChangeCookie($75,$76);
  $77 = HEAP32[$db>>2]|0;
  $78 = HEAP32[$2>>2]|0;
  _sqliteViewResetAll($77,$78);
  STACKTOP = sp;return;
 }
 $67 = HEAP32[$v>>2]|0;
 $68 = HEAP32[$2>>2]|0;
 $69 = HEAP32[$1>>2]|0;
 $70 = HEAP32[$69>>2]|0;
 (_sqlite3VdbeAddOp4($67,154,$68,0,0,$70,0)|0);
 $71 = HEAP32[$v>>2]|0;
 $72 = HEAP32[$2>>2]|0;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 (_sqlite3VdbeAddOp4($71,138,$72,0,0,$74,0)|0);
 $75 = HEAP32[$0>>2]|0;
 $76 = HEAP32[$2>>2]|0;
 _sqlite3ChangeCookie($75,$76);
 $77 = HEAP32[$db>>2]|0;
 $78 = HEAP32[$2>>2]|0;
 _sqliteViewResetAll($77,$78);
 STACKTOP = sp;return;
}
function _destroyTable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iDb = 0, $iDestroyed = 0, $iIdx = 0, $iLargest = 0, $iTab = 0, $pIdx = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $iTab = sp + 20|0;
 $iDestroyed = sp + 16|0;
 $pIdx = sp + 12|0;
 $iLargest = sp + 8|0;
 $iIdx = sp + 4|0;
 $iDb = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$iTab>>2] = $4;
 HEAP32[$iDestroyed>>2] = 0;
 while(1) {
  HEAP32[$iLargest>>2] = 0;
  $5 = HEAP32[$iDestroyed>>2]|0;
  $6 = ($5|0)==(0);
  if ($6) {
   label = 4;
  } else {
   $7 = HEAP32[$iTab>>2]|0;
   $8 = HEAP32[$iDestroyed>>2]|0;
   $9 = ($7|0)<($8|0);
   if ($9) {
    label = 4;
   }
  }
  if ((label|0) == 4) {
   label = 0;
   $10 = HEAP32[$iTab>>2]|0;
   HEAP32[$iLargest>>2] = $10;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pIdx>>2] = $13;
  while(1) {
   $14 = HEAP32[$pIdx>>2]|0;
   $15 = ($14|0)!=(0|0);
   if (!($15)) {
    break;
   }
   $16 = HEAP32[$pIdx>>2]|0;
   $17 = ((($16)) + 44|0);
   $18 = HEAP32[$17>>2]|0;
   HEAP32[$iIdx>>2] = $18;
   $19 = HEAP32[$iDestroyed>>2]|0;
   $20 = ($19|0)==(0);
   if ($20) {
    label = 9;
   } else {
    $21 = HEAP32[$iIdx>>2]|0;
    $22 = HEAP32[$iDestroyed>>2]|0;
    $23 = ($21|0)<($22|0);
    if ($23) {
     label = 9;
    }
   }
   if ((label|0) == 9) {
    label = 0;
    $24 = HEAP32[$iIdx>>2]|0;
    $25 = HEAP32[$iLargest>>2]|0;
    $26 = ($24|0)>($25|0);
    if ($26) {
     $27 = HEAP32[$iIdx>>2]|0;
     HEAP32[$iLargest>>2] = $27;
    }
   }
   $28 = HEAP32[$pIdx>>2]|0;
   $29 = ((($28)) + 20|0);
   $30 = HEAP32[$29>>2]|0;
   HEAP32[$pIdx>>2] = $30;
  }
  $31 = HEAP32[$iLargest>>2]|0;
  $32 = ($31|0)==(0);
  if ($32) {
   break;
  }
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 64|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (_sqlite3SchemaToIndex($34,$37)|0);
  HEAP32[$iDb>>2] = $38;
  $39 = HEAP32[$0>>2]|0;
  $40 = HEAP32[$iLargest>>2]|0;
  $41 = HEAP32[$iDb>>2]|0;
  _destroyRootPage($39,$40,$41);
  $42 = HEAP32[$iLargest>>2]|0;
  HEAP32[$iDestroyed>>2] = $42;
 }
 STACKTOP = sp;return;
}
function _sqliteViewResetAll($db,$idx) {
 $db = $db|0;
 $idx = $idx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $idx;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + ($2<<4)|0);
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 78|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 2;
 $13 = ($12|0)==(2);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 16|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14<<4)|0);
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$i>>2] = $23;
 while(1) {
  $24 = HEAP32[$i>>2]|0;
  $25 = ($24|0)!=(0|0);
  if (!($25)) {
   break;
  }
  $26 = HEAP32[$i>>2]|0;
  $27 = ((($26)) + 8|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pTab>>2] = $28;
  $29 = HEAP32[$pTab>>2]|0;
  $30 = ((($29)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)!=(0|0);
  if ($32) {
   $33 = HEAP32[$0>>2]|0;
   $34 = HEAP32[$pTab>>2]|0;
   _sqlite3DeleteColumnNames($33,$34);
   $35 = HEAP32[$pTab>>2]|0;
   $36 = ((($35)) + 4|0);
   HEAP32[$36>>2] = 0;
   $37 = HEAP32[$pTab>>2]|0;
   $38 = ((($37)) + 34|0);
   HEAP16[$38>>1] = 0;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$39>>2]|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = (($44) + ($41<<4)|0);
 $46 = ((($45)) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = ((($47)) + 78|0);
 $49 = HEAP16[$48>>1]|0;
 $50 = $49&65535;
 $51 = $50 & -3;
 $52 = $51&65535;
 HEAP16[$48>>1] = $52;
 STACKTOP = sp;return;
}
function _sqlite3DeleteColumnNames($db,$pTable) {
 $db = $db|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pCol = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pCol = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCol>>2] = $4;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 34|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($6|0)<($10|0);
  $12 = HEAP32[$0>>2]|0;
  if (!($11)) {
   break;
  }
  $13 = HEAP32[$pCol>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  _sqlite3DbFree($12,$14);
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$pCol>>2]|0;
  $17 = ((($16)) + 4|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3ExprDelete($15,$18);
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$pCol>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP32[$21>>2]|0;
  _sqlite3DbFree($19,$22);
  $23 = HEAP32[$i>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$i>>2] = $24;
  $25 = HEAP32[$pCol>>2]|0;
  $26 = ((($25)) + 16|0);
  HEAP32[$pCol>>2] = $26;
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 _sqlite3DbFree($12,$29);
 STACKTOP = sp;return;
}
function _sqlite3ResolveSelfReference($pParse,$pTab,$type,$pExpr,$pList) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 $type = $type|0;
 $pExpr = $pExpr|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $sNC = 0, $sSrc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $3 = sp + 116|0;
 $4 = sp + 112|0;
 $sSrc = sp;
 $sNC = sp + 80|0;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 HEAP32[$2>>2] = $type;
 HEAP32[$3>>2] = $pExpr;
 HEAP32[$4>>2] = $pList;
 ;HEAP32[$sNC>>2]=0|0;HEAP32[$sNC+4>>2]=0|0;HEAP32[$sNC+8>>2]=0|0;HEAP32[$sNC+12>>2]=0|0;HEAP32[$sNC+16>>2]=0|0;HEAP32[$sNC+20>>2]=0|0;HEAP32[$sNC+24>>2]=0|0;HEAP32[$sNC+28>>2]=0|0;
 dest=$sSrc; stop=dest+80|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 HEAP32[$sSrc>>2] = 1;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($sSrc)) + 8|0);
 $8 = ((($7)) + 8|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($sSrc)) + 8|0);
 $11 = ((($10)) + 16|0);
 HEAP32[$11>>2] = $9;
 $12 = ((($sSrc)) + 8|0);
 $13 = ((($12)) + 44|0);
 HEAP32[$13>>2] = -1;
 $14 = HEAP32[$0>>2]|0;
 HEAP32[$sNC>>2] = $14;
 $15 = ((($sNC)) + 4|0);
 HEAP32[$15>>2] = $sSrc;
 $16 = HEAP32[$2>>2]|0;
 $17 = $16&65535;
 $18 = ((($sNC)) + 28|0);
 HEAP16[$18>>1] = $17;
 $19 = HEAP32[$3>>2]|0;
 $20 = (_sqlite3ResolveExprNames($sNC,$19)|0);
 $21 = ($20|0)==(0);
 $22 = HEAP32[$4>>2]|0;
 $23 = ($22|0)!=(0|0);
 $or$cond = $21 & $23;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$4>>2]|0;
 (_sqlite3ResolveExprListNames($sNC,$24)|0);
 STACKTOP = sp;return;
}
function _sqlite3StringToId($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP8[$1>>0]|0;
 $3 = $2&255;
 $4 = ($3|0)==(97);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  HEAP8[$5>>0] = 55;
  STACKTOP = sp;return;
 }
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(53);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)==(97);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 HEAP8[$17>>0] = 55;
 STACKTOP = sp;return;
}
function _hasColumn($aiCol,$nCol,$x) {
 $aiCol = $aiCol|0;
 $nCol = $nCol|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$1>>2] = $aiCol;
 HEAP32[$2>>2] = $nCol;
 HEAP32[$3>>2] = $x;
 while(1) {
  $4 = HEAP32[$2>>2]|0;
  $5 = (($4) + -1)|0;
  HEAP32[$2>>2] = $5;
  $6 = ($4|0)>(0);
  if (!($6)) {
   label = 5;
   break;
  }
  $7 = HEAP32[$3>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 2|0);
  HEAP32[$1>>2] = $9;
  $10 = HEAP16[$8>>1]|0;
  $11 = $10 << 16 >> 16;
  $12 = ($7|0)==($11|0);
  if ($12) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  HEAP32[$0>>2] = 1;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 else if ((label|0) == 5) {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _estimateIndexWidth($pIdx) {
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $aCol = 0, $i = 0, $wIndex = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $wIndex = sp + 8|0;
 $i = sp + 4|0;
 $aCol = sp;
 $x = sp + 16|0;
 HEAP32[$0>>2] = $pIdx;
 HEAP32[$wIndex>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$aCol>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 52|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9&65535;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + ($12<<1)|0);
  $17 = HEAP16[$16>>1]|0;
  HEAP16[$x>>1] = $17;
  $18 = HEAP16[$x>>1]|0;
  $19 = $18 << 16 >> 16;
  $20 = ($19|0)<(0);
  if ($20) {
   $35 = 1;
  } else {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<1)|0);
   $26 = HEAP16[$25>>1]|0;
   $27 = $26 << 16 >> 16;
   $28 = HEAP32[$aCol>>2]|0;
   $29 = (($28) + ($27<<4)|0);
   $30 = ((($29)) + 14|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31&255;
   $35 = $32;
  }
  $33 = HEAP32[$wIndex>>2]|0;
  $34 = (($33) + ($35))|0;
  HEAP32[$wIndex>>2] = $34;
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 $38 = HEAP32[$wIndex>>2]|0;
 $39 = $38<<2;
 $40 = (_sqlite3LogEst($39,0)|0);
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 48|0);
 HEAP16[$42>>1] = $40;
 STACKTOP = sp;return;
}
function _freeIndex($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 _sqlite3DeleteIndexSamples($2,$3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 _sqlite3ExprDelete($4,$7);
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3ExprListDelete($8,$11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 _sqlite3DbFree($12,$15);
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 55|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18&255) >>> 4;
 $20 = $19 & 1;
 $21 = $20&255;
 $22 = ($21|0)!=(0);
 if (!($22)) {
  $27 = HEAP32[$0>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($27,$28);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 32|0);
 $26 = HEAP32[$25>>2]|0;
 _sqlite3DbFree($23,$26);
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($27,$28);
 STACKTOP = sp;return;
}
function _sqlite3DeleteIndexSamples($db,$pIdx) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pIdx;
 STACKTOP = sp;return;
}
function _sqlite3ExprIsConstantOrFunction($p,$isInit) {
 $p = $p|0;
 $isInit = $isInit|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $p;
 HEAP8[$1>>0] = $isInit;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = $3&255;
 $5 = (4 + ($4))|0;
 $6 = (_exprIsConst($2,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _convertToWithoutRowidTable($pParse,$pTab) {
 $pParse = $pParse|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $ipkToken = 0, $j = 0, $n = 0, $nPk = 0, $pIdx = 0;
 var $pList = 0, $pPk = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $pIdx = sp + 44|0;
 $pPk = sp + 40|0;
 $nPk = sp + 36|0;
 $i = sp + 32|0;
 $j = sp + 28|0;
 $db = sp + 24|0;
 $v = sp + 20|0;
 $pList = sp + 16|0;
 $ipkToken = sp + 8|0;
 $n = sp;
 HEAP32[$0>>2] = $pParse;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$v>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 148|0);
 $9 = ((($8)) + 7|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = ($10<<24>>24)!=(0);
 L1: do {
  if (!($11)) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $12 = HEAP32[$i>>2]|0;
    $13 = HEAP32[$1>>2]|0;
    $14 = ((($13)) + 34|0);
    $15 = HEAP16[$14>>1]|0;
    $16 = $15 << 16 >> 16;
    $17 = ($12|0)<($16|0);
    if (!($17)) {
     break L1;
    }
    $18 = HEAP32[$i>>2]|0;
    $19 = HEAP32[$1>>2]|0;
    $20 = ((($19)) + 4|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = (($21) + ($18<<4)|0);
    $23 = ((($22)) + 15|0);
    $24 = HEAP8[$23>>0]|0;
    $25 = $24&255;
    $26 = $25 & 1;
    $27 = ($26|0)!=(0);
    if ($27) {
     $28 = HEAP32[$i>>2]|0;
     $29 = HEAP32[$1>>2]|0;
     $30 = ((($29)) + 4|0);
     $31 = HEAP32[$30>>2]|0;
     $32 = (($31) + ($28<<4)|0);
     $33 = ((($32)) + 12|0);
     HEAP8[$33>>0] = 2;
    }
    $34 = HEAP32[$i>>2]|0;
    $35 = (($34) + 1)|0;
    HEAP32[$i>>2] = $35;
   }
  }
 } while(0);
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 410|0);
 $38 = HEAP8[$37>>0]|0;
 $39 = ($38<<24>>24)!=(0);
 if ($39) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$0>>2]|0;
 $41 = ((($40)) + 132|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = HEAP32[$v>>2]|0;
  $45 = HEAP32[$0>>2]|0;
  $46 = ((($45)) + 132|0);
  $47 = HEAP32[$46>>2]|0;
  _sqlite3VdbeChangeOpcode($44,$47,-122);
 }
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 32|0);
 $50 = HEAP16[$49>>1]|0;
 $51 = $50 << 16 >> 16;
 $52 = ($51|0)>=(0);
 $53 = HEAP32[$1>>2]|0;
 do {
  if ($52) {
   $54 = ((($53)) + 32|0);
   $55 = HEAP16[$54>>1]|0;
   $56 = $55 << 16 >> 16;
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + ($56<<4)|0);
   $61 = HEAP32[$60>>2]|0;
   _sqlite3TokenInit($ipkToken,$61);
   $62 = HEAP32[$0>>2]|0;
   $63 = HEAP32[$db>>2]|0;
   $64 = (_sqlite3ExprAlloc($63,55,$ipkToken,0)|0);
   $65 = (_sqlite3ExprListAppend($62,0,$64)|0);
   HEAP32[$pList>>2] = $65;
   $66 = HEAP32[$pList>>2]|0;
   $67 = ($66|0)==(0|0);
   if ($67) {
    STACKTOP = sp;return;
   }
   $68 = HEAP32[$0>>2]|0;
   $69 = ((($68)) + 408|0);
   $70 = HEAP8[$69>>0]|0;
   $71 = HEAP32[$pList>>2]|0;
   $72 = ((($71)) + 4|0);
   $73 = HEAP32[$72>>2]|0;
   $74 = ((($73)) + 12|0);
   HEAP8[$74>>0] = $70;
   $75 = HEAP32[$0>>2]|0;
   $76 = HEAP32[$pList>>2]|0;
   $77 = HEAP32[$1>>2]|0;
   $78 = ((($77)) + 43|0);
   $79 = HEAP8[$78>>0]|0;
   $80 = $79&255;
   _sqlite3CreateIndex($75,0,0,0,$76,$80,0,0,0,0,2);
   $81 = HEAP32[$db>>2]|0;
   $82 = ((($81)) + 69|0);
   $83 = HEAP8[$82>>0]|0;
   $84 = ($83<<24>>24)!=(0);
   if ($84) {
    STACKTOP = sp;return;
   } else {
    $85 = HEAP32[$1>>2]|0;
    $86 = (_sqlite3PrimaryKeyIndex($85)|0);
    HEAP32[$pPk>>2] = $86;
    $87 = HEAP32[$1>>2]|0;
    $88 = ((($87)) + 32|0);
    HEAP16[$88>>1] = -1;
    break;
   }
  } else {
   $89 = (_sqlite3PrimaryKeyIndex($53)|0);
   HEAP32[$pPk>>2] = $89;
   $90 = HEAP32[$v>>2]|0;
   $91 = ($90|0)!=(0|0);
   if ($91) {
    $92 = HEAP32[$v>>2]|0;
    $93 = HEAP32[$pPk>>2]|0;
    $94 = ((($93)) + 44|0);
    $95 = HEAP32[$94>>2]|0;
    _sqlite3VdbeChangeOpcode($92,$95,13);
   }
   HEAP32[$j>>2] = 1;
   HEAP32[$i>>2] = 1;
   while(1) {
    $96 = HEAP32[$i>>2]|0;
    $97 = HEAP32[$pPk>>2]|0;
    $98 = ((($97)) + 50|0);
    $99 = HEAP16[$98>>1]|0;
    $100 = $99&65535;
    $101 = ($96|0)<($100|0);
    if (!($101)) {
     break;
    }
    $102 = HEAP32[$pPk>>2]|0;
    $103 = ((($102)) + 4|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = HEAP32[$j>>2]|0;
    $106 = HEAP32[$i>>2]|0;
    $107 = HEAP32[$pPk>>2]|0;
    $108 = ((($107)) + 4|0);
    $109 = HEAP32[$108>>2]|0;
    $110 = (($109) + ($106<<1)|0);
    $111 = HEAP16[$110>>1]|0;
    $112 = $111 << 16 >> 16;
    $113 = (_hasColumn($104,$105,$112)|0);
    $114 = ($113|0)!=(0);
    if ($114) {
     $115 = HEAP32[$pPk>>2]|0;
     $116 = ((($115)) + 52|0);
     $117 = HEAP16[$116>>1]|0;
     $118 = (($117) + -1)<<16>>16;
     HEAP16[$116>>1] = $118;
    } else {
     $119 = HEAP32[$i>>2]|0;
     $120 = HEAP32[$pPk>>2]|0;
     $121 = ((($120)) + 4|0);
     $122 = HEAP32[$121>>2]|0;
     $123 = (($122) + ($119<<1)|0);
     $124 = HEAP16[$123>>1]|0;
     $125 = HEAP32[$j>>2]|0;
     $126 = (($125) + 1)|0;
     HEAP32[$j>>2] = $126;
     $127 = HEAP32[$pPk>>2]|0;
     $128 = ((($127)) + 4|0);
     $129 = HEAP32[$128>>2]|0;
     $130 = (($129) + ($125<<1)|0);
     HEAP16[$130>>1] = $124;
    }
    $131 = HEAP32[$i>>2]|0;
    $132 = (($131) + 1)|0;
    HEAP32[$i>>2] = $132;
   }
   $133 = HEAP32[$j>>2]|0;
   $134 = $133&65535;
   $135 = HEAP32[$pPk>>2]|0;
   $136 = ((($135)) + 50|0);
   HEAP16[$136>>1] = $134;
  }
 } while(0);
 $137 = HEAP32[$pPk>>2]|0;
 $138 = ((($137)) + 55|0);
 $139 = HEAP8[$138>>0]|0;
 $140 = $139 & -33;
 $141 = $140 | 32;
 HEAP8[$138>>0] = $141;
 $142 = HEAP32[$db>>2]|0;
 $143 = ((($142)) + 148|0);
 $144 = ((($143)) + 7|0);
 $145 = HEAP8[$144>>0]|0;
 $146 = ($145<<24>>24)!=(0);
 if (!($146)) {
  $147 = HEAP32[$pPk>>2]|0;
  $148 = ((($147)) + 55|0);
  $149 = HEAP8[$148>>0]|0;
  $150 = $149 & -9;
  $151 = $150 | 8;
  HEAP8[$148>>0] = $151;
 }
 $152 = HEAP32[$pPk>>2]|0;
 $153 = ((($152)) + 50|0);
 $154 = HEAP16[$153>>1]|0;
 $155 = $154&65535;
 HEAP32[$nPk>>2] = $155;
 $156 = HEAP32[$1>>2]|0;
 $157 = ((($156)) + 28|0);
 $158 = HEAP32[$157>>2]|0;
 $159 = HEAP32[$pPk>>2]|0;
 $160 = ((($159)) + 44|0);
 HEAP32[$160>>2] = $158;
 $161 = HEAP32[$1>>2]|0;
 $162 = ((($161)) + 8|0);
 $163 = HEAP32[$162>>2]|0;
 HEAP32[$pIdx>>2] = $163;
 L40: while(1) {
  $164 = HEAP32[$pIdx>>2]|0;
  $165 = ($164|0)!=(0|0);
  if (!($165)) {
   break;
  }
  $166 = HEAP32[$pIdx>>2]|0;
  $167 = ((($166)) + 55|0);
  $168 = HEAP8[$167>>0]|0;
  $169 = $168 & 3;
  $170 = $169&255;
  $171 = ($170|0)==(2);
  L43: do {
   if (!($171)) {
    HEAP32[$n>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $172 = HEAP32[$i>>2]|0;
     $173 = HEAP32[$nPk>>2]|0;
     $174 = ($172|0)<($173|0);
     if (!($174)) {
      break;
     }
     $175 = HEAP32[$pIdx>>2]|0;
     $176 = ((($175)) + 4|0);
     $177 = HEAP32[$176>>2]|0;
     $178 = HEAP32[$pIdx>>2]|0;
     $179 = ((($178)) + 50|0);
     $180 = HEAP16[$179>>1]|0;
     $181 = $180&65535;
     $182 = HEAP32[$i>>2]|0;
     $183 = HEAP32[$pPk>>2]|0;
     $184 = ((($183)) + 4|0);
     $185 = HEAP32[$184>>2]|0;
     $186 = (($185) + ($182<<1)|0);
     $187 = HEAP16[$186>>1]|0;
     $188 = $187 << 16 >> 16;
     $189 = (_hasColumn($177,$181,$188)|0);
     $190 = ($189|0)!=(0);
     if (!($190)) {
      $191 = HEAP32[$n>>2]|0;
      $192 = (($191) + 1)|0;
      HEAP32[$n>>2] = $192;
     }
     $193 = HEAP32[$i>>2]|0;
     $194 = (($193) + 1)|0;
     HEAP32[$i>>2] = $194;
    }
    $195 = HEAP32[$n>>2]|0;
    $196 = ($195|0)==(0);
    if ($196) {
     $197 = HEAP32[$pIdx>>2]|0;
     $198 = ((($197)) + 50|0);
     $199 = HEAP16[$198>>1]|0;
     $200 = HEAP32[$pIdx>>2]|0;
     $201 = ((($200)) + 52|0);
     HEAP16[$201>>1] = $199;
     break;
    }
    $202 = HEAP32[$db>>2]|0;
    $203 = HEAP32[$pIdx>>2]|0;
    $204 = HEAP32[$pIdx>>2]|0;
    $205 = ((($204)) + 50|0);
    $206 = HEAP16[$205>>1]|0;
    $207 = $206&65535;
    $208 = HEAP32[$n>>2]|0;
    $209 = (($207) + ($208))|0;
    $210 = (_resizeIndexObject($202,$203,$209)|0);
    $211 = ($210|0)!=(0);
    if ($211) {
     label = 50;
     break L40;
    }
    HEAP32[$i>>2] = 0;
    $212 = HEAP32[$pIdx>>2]|0;
    $213 = ((($212)) + 50|0);
    $214 = HEAP16[$213>>1]|0;
    $215 = $214&65535;
    HEAP32[$j>>2] = $215;
    while(1) {
     $216 = HEAP32[$i>>2]|0;
     $217 = HEAP32[$nPk>>2]|0;
     $218 = ($216|0)<($217|0);
     if (!($218)) {
      break L43;
     }
     $219 = HEAP32[$pIdx>>2]|0;
     $220 = ((($219)) + 4|0);
     $221 = HEAP32[$220>>2]|0;
     $222 = HEAP32[$pIdx>>2]|0;
     $223 = ((($222)) + 50|0);
     $224 = HEAP16[$223>>1]|0;
     $225 = $224&65535;
     $226 = HEAP32[$i>>2]|0;
     $227 = HEAP32[$pPk>>2]|0;
     $228 = ((($227)) + 4|0);
     $229 = HEAP32[$228>>2]|0;
     $230 = (($229) + ($226<<1)|0);
     $231 = HEAP16[$230>>1]|0;
     $232 = $231 << 16 >> 16;
     $233 = (_hasColumn($221,$225,$232)|0);
     $234 = ($233|0)!=(0);
     if (!($234)) {
      $235 = HEAP32[$i>>2]|0;
      $236 = HEAP32[$pPk>>2]|0;
      $237 = ((($236)) + 4|0);
      $238 = HEAP32[$237>>2]|0;
      $239 = (($238) + ($235<<1)|0);
      $240 = HEAP16[$239>>1]|0;
      $241 = HEAP32[$j>>2]|0;
      $242 = HEAP32[$pIdx>>2]|0;
      $243 = ((($242)) + 4|0);
      $244 = HEAP32[$243>>2]|0;
      $245 = (($244) + ($241<<1)|0);
      HEAP16[$245>>1] = $240;
      $246 = HEAP32[$i>>2]|0;
      $247 = HEAP32[$pPk>>2]|0;
      $248 = ((($247)) + 32|0);
      $249 = HEAP32[$248>>2]|0;
      $250 = (($249) + ($246<<2)|0);
      $251 = HEAP32[$250>>2]|0;
      $252 = HEAP32[$j>>2]|0;
      $253 = HEAP32[$pIdx>>2]|0;
      $254 = ((($253)) + 32|0);
      $255 = HEAP32[$254>>2]|0;
      $256 = (($255) + ($252<<2)|0);
      HEAP32[$256>>2] = $251;
      $257 = HEAP32[$j>>2]|0;
      $258 = (($257) + 1)|0;
      HEAP32[$j>>2] = $258;
     }
     $259 = HEAP32[$i>>2]|0;
     $260 = (($259) + 1)|0;
     HEAP32[$i>>2] = $260;
    }
   }
  } while(0);
  $261 = HEAP32[$pIdx>>2]|0;
  $262 = ((($261)) + 20|0);
  $263 = HEAP32[$262>>2]|0;
  HEAP32[$pIdx>>2] = $263;
 }
 if ((label|0) == 50) {
  STACKTOP = sp;return;
 }
 $264 = HEAP32[$nPk>>2]|0;
 $265 = HEAP32[$1>>2]|0;
 $266 = ((($265)) + 34|0);
 $267 = HEAP16[$266>>1]|0;
 $268 = $267 << 16 >> 16;
 $269 = ($264|0)<($268|0);
 if (!($269)) {
  $308 = HEAP32[$1>>2]|0;
  $309 = ((($308)) + 34|0);
  $310 = HEAP16[$309>>1]|0;
  $311 = HEAP32[$pPk>>2]|0;
  $312 = ((($311)) + 52|0);
  HEAP16[$312>>1] = $310;
  STACKTOP = sp;return;
 }
 $270 = HEAP32[$db>>2]|0;
 $271 = HEAP32[$pPk>>2]|0;
 $272 = HEAP32[$1>>2]|0;
 $273 = ((($272)) + 34|0);
 $274 = HEAP16[$273>>1]|0;
 $275 = $274 << 16 >> 16;
 $276 = (_resizeIndexObject($270,$271,$275)|0);
 $277 = ($276|0)!=(0);
 if ($277) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 $278 = HEAP32[$nPk>>2]|0;
 HEAP32[$j>>2] = $278;
 while(1) {
  $279 = HEAP32[$i>>2]|0;
  $280 = HEAP32[$1>>2]|0;
  $281 = ((($280)) + 34|0);
  $282 = HEAP16[$281>>1]|0;
  $283 = $282 << 16 >> 16;
  $284 = ($279|0)<($283|0);
  if (!($284)) {
   break;
  }
  $285 = HEAP32[$pPk>>2]|0;
  $286 = ((($285)) + 4|0);
  $287 = HEAP32[$286>>2]|0;
  $288 = HEAP32[$j>>2]|0;
  $289 = HEAP32[$i>>2]|0;
  $290 = (_hasColumn($287,$288,$289)|0);
  $291 = ($290|0)!=(0);
  if (!($291)) {
   $292 = HEAP32[$i>>2]|0;
   $293 = $292&65535;
   $294 = HEAP32[$j>>2]|0;
   $295 = HEAP32[$pPk>>2]|0;
   $296 = ((($295)) + 4|0);
   $297 = HEAP32[$296>>2]|0;
   $298 = (($297) + ($294<<1)|0);
   HEAP16[$298>>1] = $293;
   $299 = HEAP32[$j>>2]|0;
   $300 = HEAP32[$pPk>>2]|0;
   $301 = ((($300)) + 32|0);
   $302 = HEAP32[$301>>2]|0;
   $303 = (($302) + ($299<<2)|0);
   HEAP32[$303>>2] = 34834;
   $304 = HEAP32[$j>>2]|0;
   $305 = (($304) + 1)|0;
   HEAP32[$j>>2] = $305;
  }
  $306 = HEAP32[$i>>2]|0;
  $307 = (($306) + 1)|0;
  HEAP32[$i>>2] = $307;
 }
 STACKTOP = sp;return;
}
function _estimateTableWidth($pTab) {
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTabCol = 0, $wTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $wTable = sp + 8|0;
 $pTabCol = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTab;
 HEAP32[$wTable>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 34|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3 << 16 >> 16;
 HEAP32[$i>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pTabCol>>2] = $7;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = ($8|0)>(0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$pTabCol>>2]|0;
  $11 = ((($10)) + 14|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12&255;
  $14 = HEAP32[$wTable>>2]|0;
  $15 = (($14) + ($13))|0;
  HEAP32[$wTable>>2] = $15;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + -1)|0;
  HEAP32[$i>>2] = $17;
  $18 = HEAP32[$pTabCol>>2]|0;
  $19 = ((($18)) + 16|0);
  HEAP32[$pTabCol>>2] = $19;
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 32|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22 << 16 >> 16;
 $24 = ($23|0)<(0);
 if (!($24)) {
  $27 = HEAP32[$wTable>>2]|0;
  $28 = $27<<2;
  $29 = (_sqlite3LogEst($28,0)|0);
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 40|0);
  HEAP16[$31>>1] = $29;
  STACKTOP = sp;return;
 }
 $25 = HEAP32[$wTable>>2]|0;
 $26 = (($25) + 1)|0;
 HEAP32[$wTable>>2] = $26;
 $27 = HEAP32[$wTable>>2]|0;
 $28 = $27<<2;
 $29 = (_sqlite3LogEst($28,0)|0);
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 40|0);
 HEAP16[$31>>1] = $29;
 STACKTOP = sp;return;
}
function _createTableStmt($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $k = 0, $len = 0, $n = 0, $pCol = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0, $zEnd = 0, $zSep = 0, $zSep2 = 0, $zStmt = 0, $zType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $i = sp + 56|0;
 $k = sp + 52|0;
 $n = sp + 48|0;
 $zStmt = sp + 44|0;
 $zSep = sp + 40|0;
 $zSep2 = sp + 36|0;
 $zEnd = sp + 32|0;
 $pCol = sp + 28|0;
 $len = sp + 24|0;
 $zType = sp + 20|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$n>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pCol>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = ((($7)) + 34|0);
  $9 = HEAP16[$8>>1]|0;
  $10 = $9 << 16 >> 16;
  $11 = ($6|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pCol>>2]|0;
  $13 = HEAP32[$12>>2]|0;
  $14 = (_identLength($13)|0);
  $15 = (($14) + 5)|0;
  $16 = HEAP32[$n>>2]|0;
  $17 = (($16) + ($15))|0;
  HEAP32[$n>>2] = $17;
  $18 = HEAP32[$i>>2]|0;
  $19 = (($18) + 1)|0;
  HEAP32[$i>>2] = $19;
  $20 = HEAP32[$pCol>>2]|0;
  $21 = ((($20)) + 16|0);
  HEAP32[$pCol>>2] = $21;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (_identLength($23)|0);
 $25 = HEAP32[$n>>2]|0;
 $26 = (($25) + ($24))|0;
 HEAP32[$n>>2] = $26;
 $27 = HEAP32[$n>>2]|0;
 $28 = ($27|0)<(50);
 if ($28) {
  HEAP32[$zSep>>2] = 18402;
  HEAP32[$zSep2>>2] = 34710;
  HEAP32[$zEnd>>2] = 34706;
 } else {
  HEAP32[$zSep>>2] = 37369;
  HEAP32[$zSep2>>2] = 37373;
  HEAP32[$zEnd>>2] = 37378;
 }
 $29 = HEAP32[$2>>2]|0;
 $30 = ((($29)) + 34|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31 << 16 >> 16;
 $33 = ($32*6)|0;
 $34 = (35 + ($33))|0;
 $35 = HEAP32[$n>>2]|0;
 $36 = (($35) + ($34))|0;
 HEAP32[$n>>2] = $36;
 $37 = HEAP32[$n>>2]|0;
 $38 = ($37|0)<(0);
 $39 = $38 << 31 >> 31;
 $40 = (_sqlite3DbMallocRaw(0,$37,$39)|0);
 HEAP32[$zStmt>>2] = $40;
 $41 = HEAP32[$zStmt>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  $43 = HEAP32[$1>>2]|0;
  _sqlite3OomFault($43);
  HEAP32[$0>>2] = 0;
  $104 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($104|0);
 }
 $44 = HEAP32[$n>>2]|0;
 $45 = HEAP32[$zStmt>>2]|0;
 (_sqlite3_snprintf($44,$45,37381,$vararg_buffer)|0);
 $46 = HEAP32[$zStmt>>2]|0;
 $47 = (_sqlite3Strlen30($46)|0);
 HEAP32[$k>>2] = $47;
 $48 = HEAP32[$zStmt>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = HEAP32[$49>>2]|0;
 _identPut($48,$k,$50);
 $51 = HEAP32[$k>>2]|0;
 $52 = (($51) + 1)|0;
 HEAP32[$k>>2] = $52;
 $53 = HEAP32[$zStmt>>2]|0;
 $54 = (($53) + ($51)|0);
 HEAP8[$54>>0] = 40;
 $55 = HEAP32[$2>>2]|0;
 $56 = ((($55)) + 4|0);
 $57 = HEAP32[$56>>2]|0;
 HEAP32[$pCol>>2] = $57;
 HEAP32[$i>>2] = 0;
 while(1) {
  $58 = HEAP32[$i>>2]|0;
  $59 = HEAP32[$2>>2]|0;
  $60 = ((($59)) + 34|0);
  $61 = HEAP16[$60>>1]|0;
  $62 = $61 << 16 >> 16;
  $63 = ($58|0)<($62|0);
  $64 = HEAP32[$n>>2]|0;
  $65 = HEAP32[$k>>2]|0;
  $66 = (($64) - ($65))|0;
  $67 = HEAP32[$k>>2]|0;
  $68 = HEAP32[$zStmt>>2]|0;
  $69 = (($68) + ($67)|0);
  if (!($63)) {
   break;
  }
  $70 = HEAP32[$zSep>>2]|0;
  (_sqlite3_snprintf($66,$69,$70,$vararg_buffer1)|0);
  $71 = HEAP32[$k>>2]|0;
  $72 = HEAP32[$zStmt>>2]|0;
  $73 = (($72) + ($71)|0);
  $74 = (_sqlite3Strlen30($73)|0);
  $75 = HEAP32[$k>>2]|0;
  $76 = (($75) + ($74))|0;
  HEAP32[$k>>2] = $76;
  $77 = HEAP32[$zSep2>>2]|0;
  HEAP32[$zSep>>2] = $77;
  $78 = HEAP32[$zStmt>>2]|0;
  $79 = HEAP32[$pCol>>2]|0;
  $80 = HEAP32[$79>>2]|0;
  _identPut($78,$k,$80);
  $81 = HEAP32[$pCol>>2]|0;
  $82 = ((($81)) + 13|0);
  $83 = HEAP8[$82>>0]|0;
  $84 = $83 << 24 >> 24;
  $85 = (($84) - 65)|0;
  $86 = (5492 + ($85<<2)|0);
  $87 = HEAP32[$86>>2]|0;
  HEAP32[$zType>>2] = $87;
  $88 = HEAP32[$zType>>2]|0;
  $89 = (_sqlite3Strlen30($88)|0);
  HEAP32[$len>>2] = $89;
  $90 = HEAP32[$k>>2]|0;
  $91 = HEAP32[$zStmt>>2]|0;
  $92 = (($91) + ($90)|0);
  $93 = HEAP32[$zType>>2]|0;
  $94 = HEAP32[$len>>2]|0;
  _memcpy(($92|0),($93|0),($94|0))|0;
  $95 = HEAP32[$len>>2]|0;
  $96 = HEAP32[$k>>2]|0;
  $97 = (($96) + ($95))|0;
  HEAP32[$k>>2] = $97;
  $98 = HEAP32[$i>>2]|0;
  $99 = (($98) + 1)|0;
  HEAP32[$i>>2] = $99;
  $100 = HEAP32[$pCol>>2]|0;
  $101 = ((($100)) + 16|0);
  HEAP32[$pCol>>2] = $101;
 }
 $102 = HEAP32[$zEnd>>2]|0;
 HEAP32[$vararg_buffer3>>2] = $102;
 (_sqlite3_snprintf($66,$69,18391,$vararg_buffer3)|0);
 $103 = HEAP32[$zStmt>>2]|0;
 HEAP32[$0>>2] = $103;
 $104 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _identLength($z) {
 $z = $z|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$n>>2] = 0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = HEAP8[$1>>0]|0;
  $3 = ($2<<24>>24)!=(0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP8[$4>>0]|0;
  $6 = $5 << 24 >> 24;
  $7 = ($6|0)==(34);
  if ($7) {
   $8 = HEAP32[$n>>2]|0;
   $9 = (($8) + 1)|0;
   HEAP32[$n>>2] = $9;
  }
  $10 = HEAP32[$n>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$n>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$0>>2] = $13;
 }
 $14 = HEAP32[$n>>2]|0;
 $15 = (($14) + 2)|0;
 STACKTOP = sp;return ($15|0);
}
function _identPut($z,$pIdx,$zSignedIdent) {
 $z = $z|0;
 $pIdx = $pIdx|0;
 $zSignedIdent = $zSignedIdent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $i = 0, $j = 0, $needQuote = 0, $zIdent = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $zIdent = sp + 12|0;
 $i = sp + 8|0;
 $j = sp + 4|0;
 $needQuote = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $pIdx;
 HEAP32[$2>>2] = $zSignedIdent;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$zIdent>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$i>>2] = $5;
 HEAP32[$j>>2] = 0;
 while(1) {
  $6 = HEAP32[$j>>2]|0;
  $7 = HEAP32[$zIdent>>2]|0;
  $8 = (($7) + ($6)|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = ($9<<24>>24)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$j>>2]|0;
  $12 = HEAP32[$zIdent>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = (18403 + ($15)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = $18 & 6;
  $20 = ($19|0)!=(0);
  if (!($20)) {
   $21 = HEAP32[$j>>2]|0;
   $22 = HEAP32[$zIdent>>2]|0;
   $23 = (($22) + ($21)|0);
   $24 = HEAP8[$23>>0]|0;
   $25 = $24&255;
   $26 = ($25|0)!=(95);
   if ($26) {
    break;
   }
  }
  $27 = HEAP32[$j>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$j>>2] = $28;
 }
 $29 = HEAP32[$zIdent>>2]|0;
 $30 = HEAP8[$29>>0]|0;
 $31 = $30&255;
 $32 = (18403 + ($31)|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $35 = $34 & 4;
 $36 = ($35|0)!=(0);
 if ($36) {
  $50 = 1;
 } else {
  $37 = HEAP32[$zIdent>>2]|0;
  $38 = HEAP32[$j>>2]|0;
  $39 = (_sqlite3KeywordCode($37,$38)|0);
  $40 = ($39|0)!=(55);
  if ($40) {
   $50 = 1;
  } else {
   $41 = HEAP32[$j>>2]|0;
   $42 = HEAP32[$zIdent>>2]|0;
   $43 = (($42) + ($41)|0);
   $44 = HEAP8[$43>>0]|0;
   $45 = $44&255;
   $46 = ($45|0)!=(0);
   if ($46) {
    $50 = 1;
   } else {
    $47 = HEAP32[$j>>2]|0;
    $48 = ($47|0)==(0);
    $50 = $48;
   }
  }
 }
 $49 = $50&1;
 HEAP32[$needQuote>>2] = $49;
 $51 = HEAP32[$needQuote>>2]|0;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$i>>2]|0;
  $54 = (($53) + 1)|0;
  HEAP32[$i>>2] = $54;
  $55 = HEAP32[$0>>2]|0;
  $56 = (($55) + ($53)|0);
  HEAP8[$56>>0] = 34;
 }
 HEAP32[$j>>2] = 0;
 while(1) {
  $57 = HEAP32[$j>>2]|0;
  $58 = HEAP32[$zIdent>>2]|0;
  $59 = (($58) + ($57)|0);
  $60 = HEAP8[$59>>0]|0;
  $61 = ($60<<24>>24)!=(0);
  if (!($61)) {
   break;
  }
  $62 = HEAP32[$j>>2]|0;
  $63 = HEAP32[$zIdent>>2]|0;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = HEAP32[$i>>2]|0;
  $67 = (($66) + 1)|0;
  HEAP32[$i>>2] = $67;
  $68 = HEAP32[$0>>2]|0;
  $69 = (($68) + ($66)|0);
  HEAP8[$69>>0] = $65;
  $70 = HEAP32[$j>>2]|0;
  $71 = HEAP32[$zIdent>>2]|0;
  $72 = (($71) + ($70)|0);
  $73 = HEAP8[$72>>0]|0;
  $74 = $73&255;
  $75 = ($74|0)==(34);
  if ($75) {
   $76 = HEAP32[$i>>2]|0;
   $77 = (($76) + 1)|0;
   HEAP32[$i>>2] = $77;
   $78 = HEAP32[$0>>2]|0;
   $79 = (($78) + ($76)|0);
   HEAP8[$79>>0] = 34;
  }
  $80 = HEAP32[$j>>2]|0;
  $81 = (($80) + 1)|0;
  HEAP32[$j>>2] = $81;
 }
 $82 = HEAP32[$needQuote>>2]|0;
 $83 = ($82|0)!=(0);
 if (!($83)) {
  $88 = HEAP32[$i>>2]|0;
  $89 = HEAP32[$0>>2]|0;
  $90 = (($89) + ($88)|0);
  HEAP8[$90>>0] = 0;
  $91 = HEAP32[$i>>2]|0;
  $92 = HEAP32[$1>>2]|0;
  HEAP32[$92>>2] = $91;
  STACKTOP = sp;return;
 }
 $84 = HEAP32[$i>>2]|0;
 $85 = (($84) + 1)|0;
 HEAP32[$i>>2] = $85;
 $86 = HEAP32[$0>>2]|0;
 $87 = (($86) + ($84)|0);
 HEAP8[$87>>0] = 34;
 $88 = HEAP32[$i>>2]|0;
 $89 = HEAP32[$0>>2]|0;
 $90 = (($89) + ($88)|0);
 HEAP8[$90>>0] = 0;
 $91 = HEAP32[$i>>2]|0;
 $92 = HEAP32[$1>>2]|0;
 HEAP32[$92>>2] = $91;
 STACKTOP = sp;return;
}
function _resizeIndexObject($db,$pIdx,$N) {
 $db = $db|0;
 $pIdx = $pIdx|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $8 = 0, $9 = 0, $nByte = 0, $zExtra = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $zExtra = sp + 4|0;
 $nByte = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pIdx;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10*7)|0;
 HEAP32[$nByte>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = HEAP32[$nByte>>2]|0;
 $14 = ($13|0)<(0);
 $15 = $14 << 31 >> 31;
 $16 = (_sqlite3DbMallocZero($12,$13,$15)|0);
 HEAP32[$zExtra>>2] = $16;
 $17 = HEAP32[$zExtra>>2]|0;
 $18 = ($17|0)==(0|0);
 if ($18) {
  HEAP32[$0>>2] = 7;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 } else {
  $19 = HEAP32[$zExtra>>2]|0;
  $20 = HEAP32[$2>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 52|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26<<2;
  _memcpy(($19|0),($22|0),($27|0))|0;
  $28 = HEAP32[$zExtra>>2]|0;
  $29 = HEAP32[$2>>2]|0;
  $30 = ((($29)) + 32|0);
  HEAP32[$30>>2] = $28;
  $31 = HEAP32[$3>>2]|0;
  $32 = $31<<2;
  $33 = HEAP32[$zExtra>>2]|0;
  $34 = (($33) + ($32)|0);
  HEAP32[$zExtra>>2] = $34;
  $35 = HEAP32[$zExtra>>2]|0;
  $36 = HEAP32[$2>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = ((($39)) + 52|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = $42<<1;
  _memcpy(($35|0),($38|0),($43|0))|0;
  $44 = HEAP32[$zExtra>>2]|0;
  $45 = HEAP32[$2>>2]|0;
  $46 = ((($45)) + 4|0);
  HEAP32[$46>>2] = $44;
  $47 = HEAP32[$3>>2]|0;
  $48 = $47<<1;
  $49 = HEAP32[$zExtra>>2]|0;
  $50 = (($49) + ($48)|0);
  HEAP32[$zExtra>>2] = $50;
  $51 = HEAP32[$zExtra>>2]|0;
  $52 = HEAP32[$2>>2]|0;
  $53 = ((($52)) + 28|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = HEAP32[$2>>2]|0;
  $56 = ((($55)) + 52|0);
  $57 = HEAP16[$56>>1]|0;
  $58 = $57&65535;
  _memcpy(($51|0),($54|0),($58|0))|0;
  $59 = HEAP32[$zExtra>>2]|0;
  $60 = HEAP32[$2>>2]|0;
  $61 = ((($60)) + 28|0);
  HEAP32[$61>>2] = $59;
  $62 = HEAP32[$3>>2]|0;
  $63 = $62&65535;
  $64 = HEAP32[$2>>2]|0;
  $65 = ((($64)) + 52|0);
  HEAP16[$65>>1] = $63;
  $66 = HEAP32[$2>>2]|0;
  $67 = ((($66)) + 55|0);
  $68 = HEAP8[$67>>0]|0;
  $69 = $68 & -17;
  $70 = $69 | 16;
  HEAP8[$67>>0] = $70;
  HEAP32[$0>>2] = 0;
  $71 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($71|0);
 }
 return (0)|0;
}
function _sqlite3OpenMasterTable($p,$iDb) {
 $p = $p|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iDb;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3GetVdbe($2)|0);
 HEAP32[$v>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)==(1);
 $8 = $7 ? 19905 : 19924;
 _sqlite3TableLock($4,$5,1,1,$8);
 $9 = HEAP32[$v>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 (_sqlite3VdbeAddOp4Int($9,105,0,1,$10,5)|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 40|0);
 HEAP32[$16>>2] = 1;
 STACKTOP = sp;return;
}
function _codeTableLocks($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $p1 = 0, $pVdbe = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $pVdbe = sp + 8|0;
 $p = sp + 4|0;
 $p1 = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3GetVdbe($1)|0);
 HEAP32[$pVdbe>>2] = $2;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 112|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 116|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p1>>2] = $14;
  $15 = HEAP32[$pVdbe>>2]|0;
  $16 = HEAP32[$p1>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$p>>2]|0;
  $21 = ((($20)) + 8|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22&255;
  $24 = HEAP32[$p>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  (_sqlite3VdbeAddOp4($15,151,$16,$19,$23,$26,-2)|0);
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 STACKTOP = sp;return;
}
function _sqlite3AutoincrementBegin($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $aOp = 0;
 var $db = 0, $memId = 0, $p = 0, $pDb = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $p = sp + 20|0;
 $db = sp + 16|0;
 $pDb = sp + 12|0;
 $memId = sp + 8|0;
 $v = sp + 4|0;
 $aOp = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$v>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 120|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$p>>2] = $8;
 while(1) {
  $9 = HEAP32[$p>>2]|0;
  $10 = ($9|0)!=(0|0);
  if (!($10)) {
   label = 5;
   break;
  }
  $11 = HEAP32[$p>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$db>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  HEAP32[$pDb>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ((($18)) + 12|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$memId>>2] = $20;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$p>>2]|0;
  $23 = ((($22)) + 8|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$pDb>>2]|0;
  $26 = ((($25)) + 12|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = ((($27)) + 72|0);
  $29 = HEAP32[$28>>2]|0;
  _sqlite3OpenTable($21,0,$24,$29,104);
  $30 = HEAP32[$v>>2]|0;
  $31 = HEAP32[$memId>>2]|0;
  $32 = (($31) - 1)|0;
  $33 = HEAP32[$p>>2]|0;
  $34 = ((($33)) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$35>>2]|0;
  (_sqlite3VdbeLoadString($30,$32,$36)|0);
  $37 = HEAP32[$v>>2]|0;
  $38 = (_sqlite3VdbeAddOpList($37,10,37557,0)|0);
  HEAP32[$aOp>>2] = $38;
  $39 = HEAP32[$aOp>>2]|0;
  $40 = ($39|0)==(0|0);
  if ($40) {
   label = 5;
   break;
  }
  $41 = HEAP32[$memId>>2]|0;
  $42 = HEAP32[$aOp>>2]|0;
  $43 = ((($42)) + 8|0);
  HEAP32[$43>>2] = $41;
  $44 = HEAP32[$memId>>2]|0;
  $45 = (($44) + 1)|0;
  $46 = HEAP32[$aOp>>2]|0;
  $47 = ((($46)) + 12|0);
  HEAP32[$47>>2] = $45;
  $48 = HEAP32[$memId>>2]|0;
  $49 = HEAP32[$aOp>>2]|0;
  $50 = ((($49)) + 40|0);
  $51 = ((($50)) + 12|0);
  HEAP32[$51>>2] = $48;
  $52 = HEAP32[$memId>>2]|0;
  $53 = (($52) - 1)|0;
  $54 = HEAP32[$aOp>>2]|0;
  $55 = ((($54)) + 60|0);
  $56 = ((($55)) + 4|0);
  HEAP32[$56>>2] = $53;
  $57 = HEAP32[$memId>>2]|0;
  $58 = HEAP32[$aOp>>2]|0;
  $59 = ((($58)) + 60|0);
  $60 = ((($59)) + 12|0);
  HEAP32[$60>>2] = $57;
  $61 = HEAP32[$aOp>>2]|0;
  $62 = ((($61)) + 60|0);
  $63 = ((($62)) + 3|0);
  HEAP8[$63>>0] = 16;
  $64 = HEAP32[$memId>>2]|0;
  $65 = (($64) + 1)|0;
  $66 = HEAP32[$aOp>>2]|0;
  $67 = ((($66)) + 80|0);
  $68 = ((($67)) + 8|0);
  HEAP32[$68>>2] = $65;
  $69 = HEAP32[$memId>>2]|0;
  $70 = HEAP32[$aOp>>2]|0;
  $71 = ((($70)) + 100|0);
  $72 = ((($71)) + 12|0);
  HEAP32[$72>>2] = $69;
  $73 = HEAP32[$memId>>2]|0;
  $74 = HEAP32[$aOp>>2]|0;
  $75 = ((($74)) + 160|0);
  $76 = ((($75)) + 8|0);
  HEAP32[$76>>2] = $73;
  $77 = HEAP32[$p>>2]|0;
  $78 = HEAP32[$77>>2]|0;
  HEAP32[$p>>2] = $78;
 }
 if ((label|0) == 5) {
  STACKTOP = sp;return;
 }
}
function _logBadConnection($zType) {
 $zType = $zType|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 4|0;
 HEAP32[$0>>2] = $zType;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 _sqlite3_log(21,39546,$vararg_buffer);
 STACKTOP = sp;return;
}
function _sqlite3Prepare($db,$zSql,$nBytes,$saveSqlFlag,$pReprepare,$ppStmt,$pzTail) {
 $db = $db|0;
 $zSql = $zSql|0;
 $nBytes = $nBytes|0;
 $saveSqlFlag = $saveSqlFlag|0;
 $pReprepare = $pReprepare|0;
 $ppStmt = $ppStmt|0;
 $pzTail = $pzTail|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0;
 var $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0;
 var $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0;
 var $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0;
 var $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $iFirst = 0, $mx = 0, $mxLen = 0, $pBt = 0, $pT = 0, $pVdbe = 0, $rc = 0, $sParse = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer3 = 0;
 var $zDb = 0, $zErrMsg = 0, $zSqlCopy = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 576|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 572|0;
 $1 = sp + 568|0;
 $2 = sp + 564|0;
 $3 = sp + 560|0;
 $4 = sp + 556|0;
 $5 = sp + 552|0;
 $6 = sp + 548|0;
 $zErrMsg = sp + 544|0;
 $rc = sp + 540|0;
 $i = sp + 536|0;
 $sParse = sp + 56|0;
 $pBt = sp + 48|0;
 $zDb = sp + 44|0;
 $zSqlCopy = sp + 40|0;
 $mxLen = sp + 36|0;
 $iFirst = sp + 32|0;
 $mx = sp + 28|0;
 $pVdbe = sp + 24|0;
 $pT = sp + 20|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $zSql;
 HEAP32[$2>>2] = $nBytes;
 HEAP32[$3>>2] = $saveSqlFlag;
 HEAP32[$4>>2] = $pReprepare;
 HEAP32[$5>>2] = $ppStmt;
 HEAP32[$6>>2] = $pzTail;
 HEAP32[$zErrMsg>>2] = 0;
 HEAP32[$rc>>2] = 0;
 _memset(($sParse|0),0,152)|0;
 $7 = ((($sParse)) + 400|0);
 dest=$7; stop=dest+80|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
 $8 = HEAP32[$4>>2]|0;
 $9 = ((($sParse)) + 432|0);
 HEAP32[$9>>2] = $8;
 HEAP32[$i>>2] = 0;
 while(1) {
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = ($10|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 16|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<4)|0);
  $20 = ((($19)) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pBt>>2] = $21;
  $22 = HEAP32[$pBt>>2]|0;
  $23 = ($22|0)!=(0|0);
  if ($23) {
   $24 = HEAP32[$pBt>>2]|0;
   $25 = (_sqlite3BtreeSchemaLocked($24)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)!=(0);
   if ($27) {
    label = 5;
    break;
   }
  }
  $37 = HEAP32[$i>>2]|0;
  $38 = (($37) + 1)|0;
  HEAP32[$i>>2] = $38;
 }
 if ((label|0) == 5) {
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 16|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28<<4)|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$zDb>>2] = $33;
  $34 = HEAP32[$0>>2]|0;
  $35 = HEAP32[$rc>>2]|0;
  $36 = HEAP32[$zDb>>2]|0;
  HEAP32[$vararg_buffer>>2] = $36;
  _sqlite3ErrorWithMsg($34,$35,39591,$vararg_buffer);
  _sqlite3ParserReset($sParse);
  $184 = HEAP32[$0>>2]|0;
  $185 = HEAP32[$rc>>2]|0;
  $186 = (_sqlite3ApiExit($184,$185)|0);
  HEAP32[$rc>>2] = $186;
  $187 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($187|0);
 }
 $39 = HEAP32[$0>>2]|0;
 _sqlite3VtabUnlockList($39);
 $40 = HEAP32[$0>>2]|0;
 HEAP32[$sParse>>2] = $40;
 $41 = HEAP32[$2>>2]|0;
 $42 = ($41|0)>=(0);
 do {
  if ($42) {
   $43 = HEAP32[$2>>2]|0;
   $44 = ($43|0)==(0);
   if (!($44)) {
    $45 = HEAP32[$2>>2]|0;
    $46 = (($45) - 1)|0;
    $47 = HEAP32[$1>>2]|0;
    $48 = (($47) + ($46)|0);
    $49 = HEAP8[$48>>0]|0;
    $50 = $49 << 24 >> 24;
    $51 = ($50|0)!=(0);
    if (!($51)) {
     label = 15;
     break;
    }
   }
   $52 = HEAP32[$0>>2]|0;
   $53 = ((($52)) + 96|0);
   $54 = ((($53)) + 4|0);
   $55 = HEAP32[$54>>2]|0;
   HEAP32[$mxLen>>2] = $55;
   $56 = HEAP32[$2>>2]|0;
   $57 = HEAP32[$mxLen>>2]|0;
   $58 = ($56|0)>($57|0);
   $59 = HEAP32[$0>>2]|0;
   if ($58) {
    _sqlite3ErrorWithMsg($59,18,39621,$vararg_buffer1);
    $60 = HEAP32[$0>>2]|0;
    $61 = (_sqlite3ApiExit($60,18)|0);
    HEAP32[$rc>>2] = $61;
    _sqlite3ParserReset($sParse);
    $184 = HEAP32[$0>>2]|0;
    $185 = HEAP32[$rc>>2]|0;
    $186 = (_sqlite3ApiExit($184,$185)|0);
    HEAP32[$rc>>2] = $186;
    $187 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($187|0);
   }
   $62 = HEAP32[$1>>2]|0;
   $63 = HEAP32[$2>>2]|0;
   $64 = ($63|0)<(0);
   $65 = $64 << 31 >> 31;
   $66 = (_sqlite3DbStrNDup($59,$62,$63,$65)|0);
   HEAP32[$zSqlCopy>>2] = $66;
   $67 = HEAP32[$zSqlCopy>>2]|0;
   $68 = ($67|0)!=(0|0);
   if ($68) {
    $69 = HEAP32[$zSqlCopy>>2]|0;
    (_sqlite3RunParser($sParse,$69,$zErrMsg)|0);
    $70 = ((($sParse)) + 436|0);
    $71 = HEAP32[$70>>2]|0;
    $72 = HEAP32[$zSqlCopy>>2]|0;
    $73 = $71;
    $74 = $72;
    $75 = (($73) - ($74))|0;
    $76 = HEAP32[$1>>2]|0;
    $77 = (($76) + ($75)|0);
    $78 = ((($sParse)) + 436|0);
    HEAP32[$78>>2] = $77;
    $79 = HEAP32[$0>>2]|0;
    $80 = HEAP32[$zSqlCopy>>2]|0;
    _sqlite3DbFree($79,$80);
    break;
   } else {
    $81 = HEAP32[$2>>2]|0;
    $82 = HEAP32[$1>>2]|0;
    $83 = (($82) + ($81)|0);
    $84 = ((($sParse)) + 436|0);
    HEAP32[$84>>2] = $83;
    break;
   }
  } else {
   label = 15;
  }
 } while(0);
 if ((label|0) == 15) {
  $85 = HEAP32[$1>>2]|0;
  (_sqlite3RunParser($sParse,$85,$zErrMsg)|0);
 }
 $86 = ((($sParse)) + 12|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = ($87|0)==(101);
 if ($88) {
  $89 = ((($sParse)) + 12|0);
  HEAP32[$89>>2] = 0;
 }
 $90 = ((($sParse)) + 17|0);
 $91 = HEAP8[$90>>0]|0;
 $92 = ($91<<24>>24)!=(0);
 if ($92) {
  _schemaIsValid($sParse);
 }
 $93 = HEAP32[$0>>2]|0;
 $94 = ((($93)) + 69|0);
 $95 = HEAP8[$94>>0]|0;
 $96 = ($95<<24>>24)!=(0);
 if ($96) {
  $97 = ((($sParse)) + 12|0);
  HEAP32[$97>>2] = 7;
 }
 $98 = HEAP32[$6>>2]|0;
 $99 = ($98|0)!=(0|0);
 if ($99) {
  $100 = ((($sParse)) + 436|0);
  $101 = HEAP32[$100>>2]|0;
  $102 = HEAP32[$6>>2]|0;
  HEAP32[$102>>2] = $101;
 }
 $103 = ((($sParse)) + 12|0);
 $104 = HEAP32[$103>>2]|0;
 HEAP32[$rc>>2] = $104;
 $105 = HEAP32[$rc>>2]|0;
 $106 = ($105|0)==(0);
 L38: do {
  if ($106) {
   $107 = ((($sParse)) + 8|0);
   $108 = HEAP32[$107>>2]|0;
   $109 = ($108|0)!=(0|0);
   if ($109) {
    $110 = ((($sParse)) + 409|0);
    $111 = HEAP8[$110>>0]|0;
    $112 = $111&255;
    $113 = ($112|0)!=(0);
    if ($113) {
     $114 = ((($sParse)) + 409|0);
     $115 = HEAP8[$114>>0]|0;
     $116 = $115&255;
     $117 = ($116|0)==(2);
     $118 = ((($sParse)) + 8|0);
     $119 = HEAP32[$118>>2]|0;
     if ($117) {
      _sqlite3VdbeSetNumCols($119,4);
      HEAP32[$iFirst>>2] = 8;
      HEAP32[$mx>>2] = 12;
     } else {
      _sqlite3VdbeSetNumCols($119,8);
      HEAP32[$iFirst>>2] = 0;
      HEAP32[$mx>>2] = 8;
     }
     $120 = HEAP32[$iFirst>>2]|0;
     HEAP32[$i>>2] = $120;
     while(1) {
      $121 = HEAP32[$i>>2]|0;
      $122 = HEAP32[$mx>>2]|0;
      $123 = ($121|0)<($122|0);
      if (!($123)) {
       break L38;
      }
      $124 = ((($sParse)) + 8|0);
      $125 = HEAP32[$124>>2]|0;
      $126 = HEAP32[$i>>2]|0;
      $127 = HEAP32[$iFirst>>2]|0;
      $128 = (($126) - ($127))|0;
      $129 = HEAP32[$i>>2]|0;
      $130 = (5524 + ($129<<2)|0);
      $131 = HEAP32[$130>>2]|0;
      (_sqlite3VdbeSetColName($125,$128,0,$131,0)|0);
      $132 = HEAP32[$i>>2]|0;
      $133 = (($132) + 1)|0;
      HEAP32[$i>>2] = $133;
     }
    }
   }
  }
 } while(0);
 $134 = HEAP32[$0>>2]|0;
 $135 = ((($134)) + 148|0);
 $136 = ((($135)) + 5|0);
 $137 = HEAP8[$136>>0]|0;
 $138 = $137&255;
 $139 = ($138|0)==(0);
 if ($139) {
  $140 = ((($sParse)) + 8|0);
  $141 = HEAP32[$140>>2]|0;
  HEAP32[$pVdbe>>2] = $141;
  $142 = HEAP32[$pVdbe>>2]|0;
  $143 = HEAP32[$1>>2]|0;
  $144 = ((($sParse)) + 436|0);
  $145 = HEAP32[$144>>2]|0;
  $146 = HEAP32[$1>>2]|0;
  $147 = $145;
  $148 = $146;
  $149 = (($147) - ($148))|0;
  $150 = HEAP32[$3>>2]|0;
  _sqlite3VdbeSetSql($142,$143,$149,$150);
 }
 $151 = ((($sParse)) + 8|0);
 $152 = HEAP32[$151>>2]|0;
 $153 = ($152|0)!=(0|0);
 do {
  if ($153) {
   $154 = HEAP32[$rc>>2]|0;
   $155 = ($154|0)!=(0);
   if (!($155)) {
    $156 = HEAP32[$0>>2]|0;
    $157 = ((($156)) + 69|0);
    $158 = HEAP8[$157>>0]|0;
    $159 = $158&255;
    $160 = ($159|0)!=(0);
    if (!($160)) {
     label = 39;
     break;
    }
   }
   $161 = ((($sParse)) + 8|0);
   $162 = HEAP32[$161>>2]|0;
   (_sqlite3VdbeFinalize($162)|0);
  } else {
   label = 39;
  }
 } while(0);
 if ((label|0) == 39) {
  $163 = ((($sParse)) + 8|0);
  $164 = HEAP32[$163>>2]|0;
  $165 = HEAP32[$5>>2]|0;
  HEAP32[$165>>2] = $164;
 }
 $166 = HEAP32[$zErrMsg>>2]|0;
 $167 = ($166|0)!=(0|0);
 $168 = HEAP32[$0>>2]|0;
 $169 = HEAP32[$rc>>2]|0;
 if ($167) {
  $170 = HEAP32[$zErrMsg>>2]|0;
  HEAP32[$vararg_buffer3>>2] = $170;
  _sqlite3ErrorWithMsg($168,$169,18391,$vararg_buffer3);
  $171 = HEAP32[$0>>2]|0;
  $172 = HEAP32[$zErrMsg>>2]|0;
  _sqlite3DbFree($171,$172);
 } else {
  _sqlite3Error($168,$169);
 }
 while(1) {
  $173 = ((($sParse)) + 468|0);
  $174 = HEAP32[$173>>2]|0;
  $175 = ($174|0)!=(0|0);
  if (!($175)) {
   break;
  }
  $176 = ((($sParse)) + 468|0);
  $177 = HEAP32[$176>>2]|0;
  HEAP32[$pT>>2] = $177;
  $178 = HEAP32[$pT>>2]|0;
  $179 = ((($178)) + 4|0);
  $180 = HEAP32[$179>>2]|0;
  $181 = ((($sParse)) + 468|0);
  HEAP32[$181>>2] = $180;
  $182 = HEAP32[$0>>2]|0;
  $183 = HEAP32[$pT>>2]|0;
  _sqlite3DbFree($182,$183);
 }
 _sqlite3ParserReset($sParse);
 $184 = HEAP32[$0>>2]|0;
 $185 = HEAP32[$rc>>2]|0;
 $186 = (_sqlite3ApiExit($184,$185)|0);
 HEAP32[$rc>>2] = $186;
 $187 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($187|0);
}
function _sqlite3BtreeSchemaLocked($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_querySharedCacheTableLock($2,1,1)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _schemaIsValid($pParse) {
 $pParse = $pParse|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cookie = 0, $db = 0, $iDb = 0, $openedTransaction = 0, $or$cond = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $db = sp + 20|0;
 $iDb = sp + 16|0;
 $rc = sp + 12|0;
 $cookie = sp + 8|0;
 $openedTransaction = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $pParse;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 HEAP32[$iDb>>2] = 0;
 while(1) {
  $3 = HEAP32[$iDb>>2]|0;
  $4 = HEAP32[$db>>2]|0;
  $5 = ((($4)) + 20|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($3|0)<($6|0);
  if (!($7)) {
   label = 14;
   break;
  }
  HEAP32[$openedTransaction>>2] = 0;
  $8 = HEAP32[$iDb>>2]|0;
  $9 = HEAP32[$db>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = (($11) + ($8<<4)|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$pBt>>2] = $14;
  $15 = HEAP32[$pBt>>2]|0;
  $16 = ($15|0)==(0|0);
  if (!($16)) {
   $17 = HEAP32[$pBt>>2]|0;
   $18 = (_sqlite3BtreeIsInReadTrans($17)|0);
   $19 = ($18|0)!=(0);
   if (!($19)) {
    $20 = HEAP32[$pBt>>2]|0;
    $21 = (_sqlite3BtreeBeginTrans($20,0)|0);
    HEAP32[$rc>>2] = $21;
    $22 = HEAP32[$rc>>2]|0;
    $23 = ($22|0)==(7);
    $24 = HEAP32[$rc>>2]|0;
    $25 = ($24|0)==(3082);
    $or$cond = $23 | $25;
    if ($or$cond) {
     $26 = HEAP32[$db>>2]|0;
     _sqlite3OomFault($26);
    }
    $27 = HEAP32[$rc>>2]|0;
    $28 = ($27|0)!=(0);
    if ($28) {
     label = 14;
     break;
    }
    HEAP32[$openedTransaction>>2] = 1;
   }
   $29 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeGetMeta($29,1,$cookie);
   $30 = HEAP32[$cookie>>2]|0;
   $31 = HEAP32[$iDb>>2]|0;
   $32 = HEAP32[$db>>2]|0;
   $33 = ((($32)) + 16|0);
   $34 = HEAP32[$33>>2]|0;
   $35 = (($34) + ($31<<4)|0);
   $36 = ((($35)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$37>>2]|0;
   $39 = ($30|0)!=($38|0);
   if ($39) {
    $40 = HEAP32[$db>>2]|0;
    $41 = HEAP32[$iDb>>2]|0;
    _sqlite3ResetOneSchema($40,$41);
    $42 = HEAP32[$0>>2]|0;
    $43 = ((($42)) + 12|0);
    HEAP32[$43>>2] = 17;
   }
   $44 = HEAP32[$openedTransaction>>2]|0;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$pBt>>2]|0;
    (_sqlite3BtreeCommit($46)|0);
   }
  }
  $47 = HEAP32[$iDb>>2]|0;
  $48 = (($47) + 1)|0;
  HEAP32[$iDb>>2] = $48;
 }
 if ((label|0) == 14) {
  STACKTOP = sp;return;
 }
}
function _sqlite3VdbeSetSql($p,$z,$n,$isPrepareV2) {
 $p = $p|0;
 $z = $z|0;
 $n = $n|0;
 $isPrepareV2 = $isPrepareV2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $z;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $isPrepareV2;
 $4 = HEAP32[$0>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_sqlite3DbStrNDup($7,$8,$9,$11)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 176|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$3>>2]|0;
 $16 = $15&255;
 $17 = $16&255;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 144|0);
 $20 = $17&65535;
 $21 = HEAP16[$19>>1]|0;
 $22 = $20 & 1;
 $23 = ($22 << 9)&65535;
 $24 = $21 & -513;
 $25 = $24 | $23;
 HEAP16[$19>>1] = $25;
 STACKTOP = sp;return;
}
function _querySharedCacheTableLock($p,$iTab,$eLock) {
 $p = $p|0;
 $iTab = $iTab|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 20|0;
 $pBt = sp + 4|0;
 $pIter = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTab;
 HEAP8[$3>>0] = $eLock;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 9|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 if (!($10)) {
  HEAP32[$0>>2] = 0;
  $55 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 76|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ($13|0)!=($14|0);
 if ($15) {
  $16 = HEAP32[$pBt>>2]|0;
  $17 = ((($16)) + 22|0);
  $18 = HEAP16[$17>>1]|0;
  $19 = $18&65535;
  $20 = $19 & 32;
  $21 = ($20|0)!=(0);
  if ($21) {
   HEAP32[$0>>2] = 262;
   $55 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($55|0);
  }
 }
 $22 = HEAP32[$pBt>>2]|0;
 $23 = ((($22)) + 72|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$pIter>>2] = $24;
 while(1) {
  $25 = HEAP32[$pIter>>2]|0;
  $26 = ($25|0)!=(0|0);
  if (!($26)) {
   label = 15;
   break;
  }
  $27 = HEAP32[$pIter>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  $30 = ($28|0)!=($29|0);
  if ($30) {
   $31 = HEAP32[$pIter>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = ($33|0)==($34|0);
   if ($35) {
    $36 = HEAP32[$pIter>>2]|0;
    $37 = ((($36)) + 8|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = HEAP8[$3>>0]|0;
    $41 = $40&255;
    $42 = ($39|0)!=($41|0);
    if ($42) {
     break;
    }
   }
  }
  $52 = HEAP32[$pIter>>2]|0;
  $53 = ((($52)) + 12|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$pIter>>2] = $54;
 }
 if ((label|0) == 15) {
  HEAP32[$0>>2] = 0;
  $55 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $43 = HEAP8[$3>>0]|0;
 $44 = $43&255;
 $45 = ($44|0)==(2);
 if ($45) {
  $46 = HEAP32[$pBt>>2]|0;
  $47 = ((($46)) + 22|0);
  $48 = HEAP16[$47>>1]|0;
  $49 = $48&65535;
  $50 = $49 | 64;
  $51 = $50&65535;
  HEAP16[$47>>1] = $51;
 }
 HEAP32[$0>>2] = 262;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _saveAllCursors($pBt,$iRoot,$pExcept) {
 $pBt = $pBt|0;
 $iRoot = $iRoot|0;
 $pExcept = $pExcept|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $iRoot;
 HEAP32[$3>>2] = $pExcept;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 while(1) {
  $7 = HEAP32[$p>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = HEAP32[$3>>2]|0;
  $11 = ($9|0)!=($10|0);
  if ($11) {
   $12 = HEAP32[$2>>2]|0;
   $13 = (0)==($12|0);
   if ($13) {
    break;
   }
   $14 = HEAP32[$p>>2]|0;
   $15 = ((($14)) + 52|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$2>>2]|0;
   $18 = ($16|0)==($17|0);
   if ($18) {
    break;
   }
  }
  $19 = HEAP32[$p>>2]|0;
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$p>>2] = $21;
 }
 $22 = HEAP32[$p>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$p>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$3>>2]|0;
  $27 = (_saveCursorsOnList($24,$25,$26)|0);
  HEAP32[$0>>2] = $27;
  $36 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $28 = HEAP32[$3>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$3>>2]|0;
  $31 = ((($30)) + 64|0);
  $32 = HEAP8[$31>>0]|0;
  $33 = $32&255;
  $34 = $33 & -33;
  $35 = $34&255;
  HEAP8[$31>>0] = $35;
 }
 HEAP32[$0>>2] = 0;
 $36 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _saveCursorsOnList($p,$iRoot,$pExcept) {
 $p = $p|0;
 $iRoot = $iRoot|0;
 $pExcept = $pExcept|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iRoot;
 HEAP32[$3>>2] = $pExcept;
 L1: while(1) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$3>>2]|0;
  $6 = ($4|0)!=($5|0);
  do {
   if ($6) {
    $7 = HEAP32[$2>>2]|0;
    $8 = (0)==($7|0);
    if (!($8)) {
     $9 = HEAP32[$1>>2]|0;
     $10 = ((($9)) + 52|0);
     $11 = HEAP32[$10>>2]|0;
     $12 = HEAP32[$2>>2]|0;
     $13 = ($11|0)==($12|0);
     if (!($13)) {
      break;
     }
    }
    $14 = HEAP32[$1>>2]|0;
    $15 = ((($14)) + 66|0);
    $16 = HEAP8[$15>>0]|0;
    $17 = $16&255;
    $18 = ($17|0)==(1);
    if (!($18)) {
     $19 = HEAP32[$1>>2]|0;
     $20 = ((($19)) + 66|0);
     $21 = HEAP8[$20>>0]|0;
     $22 = $21&255;
     $23 = ($22|0)==(2);
     if (!($23)) {
      $29 = HEAP32[$1>>2]|0;
      _btreeReleaseAllCursorPages($29);
      break;
     }
    }
    $24 = HEAP32[$1>>2]|0;
    $25 = (_saveCursorPosition($24)|0);
    HEAP32[$rc>>2] = $25;
    $26 = HEAP32[$rc>>2]|0;
    $27 = (0)!=($26|0);
    if ($27) {
     label = 8;
     break L1;
    }
   }
  } while(0);
  $30 = HEAP32[$1>>2]|0;
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  HEAP32[$1>>2] = $32;
  $33 = HEAP32[$1>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 8) {
  $28 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $28;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _saveCursorPosition($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 66|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(2);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 66|0);
  HEAP8[$7>>0] = 1;
 } else {
  $8 = ((($6)) + 60|0);
  HEAP32[$8>>2] = 0;
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = (_saveCursorKey($9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)==(0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  _btreeReleaseAllCursorPages($13);
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 66|0);
  HEAP8[$15>>0] = 3;
 }
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 64|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = $19 & -15;
 $21 = $20&255;
 HEAP8[$17>>0] = $21;
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _btreeReleaseAllCursorPages($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 68|0);
  $4 = HEAP8[$3>>0]|0;
  $5 = $4 << 24 >> 24;
  $6 = ($1|0)<=($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 120|0);
  $10 = (($9) + ($7<<2)|0);
  $11 = HEAP32[$10>>2]|0;
  _releasePage($11);
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 120|0);
  $15 = (($14) + ($12<<2)|0);
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$i>>2]|0;
  $17 = (($16) + 1)|0;
  HEAP32[$i>>2] = $17;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 68|0);
 HEAP8[$19>>0] = -1;
 STACKTOP = sp;return;
}
function _saveCursorKey($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pKey = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pKey = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 69|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 $5 = HEAP32[$0>>2]|0;
 if ($4) {
  $6 = (_sqlite3BtreeIntegerKey($5)|0);
  $7 = tempRet0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 40|0);
  $10 = $9;
  $11 = $10;
  HEAP32[$11>>2] = $6;
  $12 = (($10) + 4)|0;
  $13 = $12;
  HEAP32[$13>>2] = $7;
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $14 = (_sqlite3BtreePayloadSize($5)|0);
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 40|0);
 $17 = $16;
 $18 = $17;
 HEAP32[$18>>2] = $14;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 40|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (_sqlite3Malloc($25,$28)|0);
 HEAP32[$pKey>>2] = $29;
 $30 = HEAP32[$pKey>>2]|0;
 $31 = ($30|0)!=(0|0);
 if (!($31)) {
  HEAP32[$rc>>2] = 7;
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 40|0);
 $35 = $34;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = (($35) + 4)|0;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$pKey>>2]|0;
 $42 = (_sqlite3BtreeKey($32,0,$37,$41)|0);
 HEAP32[$rc>>2] = $42;
 $43 = HEAP32[$rc>>2]|0;
 $44 = ($43|0)==(0);
 $45 = HEAP32[$pKey>>2]|0;
 if ($44) {
  $46 = HEAP32[$0>>2]|0;
  $47 = ((($46)) + 48|0);
  HEAP32[$47>>2] = $45;
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 } else {
  _sqlite3_free($45);
  $48 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 return (0)|0;
}
function _sqlite3BtreeIntegerKey($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 _getCellInfo($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = $3;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = (($4) + 4)|0;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 tempRet0 = ($9);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BtreePayloadSize($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 _getCellInfo($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _sqlite3BtreeKey($pCur,$offset,$amt,$pBuf) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $pBuf = $pBuf|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $pBuf;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = (_accessPayload($4,$5,$6,$7,0)|0);
 STACKTOP = sp;return ($8|0);
}
function _sqlite3BtreeEnterCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($2);
 STACKTOP = sp;return;
}
function _sqlite3BtreeIncrblobCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = $4 | 16;
 $6 = $5&255;
 HEAP8[$2>>0] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 11|0);
 HEAP8[$9>>0] = 1;
 STACKTOP = sp;return;
}
function _allocSpace($p,$pBuf,$nByte) {
 $p = $p|0;
 $pBuf = $pBuf|0;
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pBuf;
 HEAP32[$2>>2] = $nByte;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if (!($4)) {
  $27 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = (($5) + 7)|0;
 $7 = $6 & -8;
 HEAP32[$2>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 4|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)<=($11|0);
 $13 = HEAP32[$2>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 if ($12) {
  $15 = ((($14)) + 4|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) - ($13))|0;
  HEAP32[$15>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 4|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($20)|0);
  HEAP32[$1>>2] = $23;
  $27 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($27|0);
 } else {
  $24 = ((($14)) + 8|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = (($25) + ($13))|0;
  HEAP32[$24>>2] = $26;
  $27 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($27|0);
 }
 return (0)|0;
}
function _vdbeChangeP4Full($p,$pOp,$zP4,$n) {
 $p = $p|0;
 $pOp = $pOp|0;
 $zP4 = $zP4|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $3 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pOp;
 HEAP32[$2>>2] = $zP4;
 HEAP32[$3>>2] = $n;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 1|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 1|0);
  $12 = HEAP8[$11>>0]|0;
  $13 = $12 << 24 >> 24;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  _freeP4($9,$13,$16);
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 1|0);
  HEAP8[$18>>0] = 0;
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 16|0);
  HEAP32[$20>>2] = 0;
 }
 $21 = HEAP32[$3>>2]|0;
 $22 = ($21|0)<(0);
 if ($22) {
  $23 = HEAP32[$0>>2]|0;
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 88|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = $24;
  $29 = $27;
  $30 = (($28) - ($29))|0;
  $31 = (($30|0) / 20)&-1;
  $32 = HEAP32[$2>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  _sqlite3VdbeChangeP4($23,$31,$32,$33);
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$3>>2]|0;
 $35 = ($34|0)==(0);
 if ($35) {
  $36 = HEAP32[$2>>2]|0;
  $37 = (_sqlite3Strlen30($36)|0);
  HEAP32[$3>>2] = $37;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$3>>2]|0;
 $42 = ($41|0)<(0);
 $43 = $42 << 31 >> 31;
 $44 = (_sqlite3DbStrNDup($39,$40,$41,$43)|0);
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 16|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 1|0);
 HEAP8[$48>>0] = -1;
 STACKTOP = sp;return;
}
function _sqlite3VtabLock($pVTab) {
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVTab;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 STACKTOP = sp;return;
}
function _sqlite3BtreeSharable($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 9|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VtabEponymousTableInit($pParse,$pMod) {
 $pParse = $pParse|0;
 $pMod = $pMod|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $db = 0, $pModule = 0, $pTab = 0, $rc = 0, $vararg_buffer = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $pModule = sp + 20|0;
 $pTab = sp + 16|0;
 $zErr = sp + 12|0;
 $rc = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $pParse;
 HEAP32[$2>>2] = $pMod;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pModule>>2] = $4;
 HEAP32[$zErr>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  HEAP32[$0>>2] = 1;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $11 = HEAP32[$pModule>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if ($14) {
  $15 = HEAP32[$pModule>>2]|0;
  $16 = ((($15)) + 4|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$pModule>>2]|0;
  $19 = ((($18)) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($17|0)!=($20|0);
  if ($21) {
   HEAP32[$0>>2] = 0;
   $88 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($88|0);
  }
 }
 $22 = HEAP32[$db>>2]|0;
 $23 = (_sqlite3DbMallocZero($22,72,0)|0);
 HEAP32[$pTab>>2] = $23;
 $24 = HEAP32[$pTab>>2]|0;
 $25 = ($24|0)==(0|0);
 if ($25) {
  HEAP32[$0>>2] = 0;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $26 = HEAP32[$db>>2]|0;
 $27 = HEAP32[$2>>2]|0;
 $28 = ((($27)) + 4|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (_sqlite3DbStrDup($26,$29)|0);
 $31 = HEAP32[$pTab>>2]|0;
 HEAP32[$31>>2] = $30;
 $32 = HEAP32[$pTab>>2]|0;
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)==(0|0);
 if ($34) {
  $35 = HEAP32[$db>>2]|0;
  $36 = HEAP32[$pTab>>2]|0;
  _sqlite3DbFree($35,$36);
  HEAP32[$0>>2] = 0;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $37 = HEAP32[$pTab>>2]|0;
 $38 = HEAP32[$2>>2]|0;
 $39 = ((($38)) + 16|0);
 HEAP32[$39>>2] = $37;
 $40 = HEAP32[$pTab>>2]|0;
 $41 = ((($40)) + 36|0);
 HEAP16[$41>>1] = 1;
 $42 = HEAP32[$db>>2]|0;
 $43 = ((($42)) + 16|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = HEAP32[$pTab>>2]|0;
 $48 = ((($47)) + 64|0);
 HEAP32[$48>>2] = $46;
 $49 = HEAP32[$pTab>>2]|0;
 $50 = ((($49)) + 42|0);
 $51 = HEAP8[$50>>0]|0;
 $52 = $51&255;
 $53 = $52 | 16;
 $54 = $53&255;
 HEAP8[$50>>0] = $54;
 $55 = HEAP32[$pTab>>2]|0;
 $56 = ((($55)) + 48|0);
 HEAP32[$56>>2] = 0;
 $57 = HEAP32[$pTab>>2]|0;
 $58 = ((($57)) + 32|0);
 HEAP16[$58>>1] = -1;
 $59 = HEAP32[$db>>2]|0;
 $60 = HEAP32[$pTab>>2]|0;
 $61 = HEAP32[$db>>2]|0;
 $62 = HEAP32[$pTab>>2]|0;
 $63 = HEAP32[$62>>2]|0;
 $64 = (_sqlite3DbStrDup($61,$63)|0);
 _addModuleArgument($59,$60,$64);
 $65 = HEAP32[$db>>2]|0;
 $66 = HEAP32[$pTab>>2]|0;
 _addModuleArgument($65,$66,0);
 $67 = HEAP32[$db>>2]|0;
 $68 = HEAP32[$pTab>>2]|0;
 $69 = HEAP32[$db>>2]|0;
 $70 = HEAP32[$pTab>>2]|0;
 $71 = HEAP32[$70>>2]|0;
 $72 = (_sqlite3DbStrDup($69,$71)|0);
 _addModuleArgument($67,$68,$72);
 $73 = HEAP32[$db>>2]|0;
 $74 = HEAP32[$pTab>>2]|0;
 $75 = HEAP32[$2>>2]|0;
 $76 = HEAP32[$pModule>>2]|0;
 $77 = ((($76)) + 8|0);
 $78 = HEAP32[$77>>2]|0;
 $79 = (_vtabCallConstructor($73,$74,$75,$78,$zErr)|0);
 HEAP32[$rc>>2] = $79;
 $80 = HEAP32[$rc>>2]|0;
 $81 = ($80|0)!=(0);
 if ($81) {
  $82 = HEAP32[$1>>2]|0;
  $83 = HEAP32[$zErr>>2]|0;
  HEAP32[$vararg_buffer>>2] = $83;
  _sqlite3ErrorMsg($82,18391,$vararg_buffer);
  $84 = HEAP32[$db>>2]|0;
  $85 = HEAP32[$zErr>>2]|0;
  _sqlite3DbFree($84,$85);
  $86 = HEAP32[$db>>2]|0;
  $87 = HEAP32[$2>>2]|0;
  _sqlite3VtabEponymousTableClear($86,$87);
  HEAP32[$0>>2] = 0;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 } else {
  HEAP32[$0>>2] = 1;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 return (0)|0;
}
function _sqlite3VtabEponymousTableClear($db,$pMod) {
 $db = $db|0;
 $pMod = $pMod|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pTab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pMod;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pTab>>2] = $4;
 $5 = HEAP32[$pTab>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$pTab>>2]|0;
 $8 = ((($7)) + 42|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = $10 | 2;
 $12 = $11&255;
 HEAP8[$8>>0] = $12;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pTab>>2]|0;
 _sqlite3DeleteTable($13,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 16|0);
 HEAP32[$16>>2] = 0;
 STACKTOP = sp;return;
}
function _findNextHostParameter($zSql,$pnToken) {
 $zSql = $zSql|0;
 $pnToken = $pnToken|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $nTotal = 0;
 var $tokenType = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $tokenType = sp + 8|0;
 $nTotal = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $zSql;
 HEAP32[$1>>2] = $pnToken;
 HEAP32[$nTotal>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = ($4<<24>>24)!=(0);
  if (!($5)) {
   label = 6;
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = (_sqlite3GetToken($6,$tokenType)|0);
  HEAP32[$n>>2] = $7;
  $8 = HEAP32[$tokenType>>2]|0;
  $9 = ($8|0)==(135);
  $10 = HEAP32[$n>>2]|0;
  if ($9) {
   break;
  }
  $12 = HEAP32[$nTotal>>2]|0;
  $13 = (($12) + ($10))|0;
  HEAP32[$nTotal>>2] = $13;
  $14 = HEAP32[$n>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = (($15) + ($14)|0);
  HEAP32[$0>>2] = $16;
 }
 if ((label|0) == 6) {
  $17 = HEAP32[$nTotal>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $17 = HEAP32[$nTotal>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3VdbeMemMove($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 dest=$3; src=$4; stop=dest+40|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 24|0);
 HEAP32[$8>>2] = 0;
 STACKTOP = sp;return;
}
function _columnNullValue() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return (320|0);
}
function _sqlite3VdbeSwap($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pTmp = 0, $tmp = 0, $zTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 224|0;
 $0 = sp + 220|0;
 $1 = sp + 216|0;
 $tmp = sp;
 $pTmp = sp + 212|0;
 $zTmp = sp + 208|0;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 $2 = HEAP32[$0>>2]|0;
 _memcpy(($tmp|0),($2|0),208)|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memcpy(($3|0),($4|0),208)|0;
 $5 = HEAP32[$1>>2]|0;
 _memcpy(($5|0),($tmp|0),208)|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pTmp>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$pTmp>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 8|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pTmp>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 4|0);
 HEAP32[$24>>2] = $22;
 $25 = HEAP32[$pTmp>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 4|0);
 HEAP32[$27>>2] = $25;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 176|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$zTmp>>2] = $30;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 176|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 176|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$zTmp>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 176|0);
 HEAP32[$38>>2] = $36;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 144|0);
 $41 = HEAP16[$40>>1]|0;
 $42 = ($41&65535) >>> 9;
 $43 = $42 & 1;
 $44 = $43&65535;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 144|0);
 $47 = $44&65535;
 $48 = HEAP16[$46>>1]|0;
 $49 = $47 & 1;
 $50 = ($49 << 9)&65535;
 $51 = $48 & -513;
 $52 = $51 | $50;
 HEAP16[$46>>1] = $52;
 STACKTOP = sp;return;
}
function _sqlite3VdbeResetStepResult($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 HEAP32[$2>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3OsCurrentTimeInt64($pVfs,$pTimeOut) {
 $pVfs = $pVfs|0;
 $pTimeOut = $pTimeOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $r = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $pTimeOut;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>=(2);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 72|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 72|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = (FUNCTION_TABLE_iii[$11 & 255]($12,$13)|0);
   HEAP32[$rc>>2] = $14;
   $29 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($29|0);
  }
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = (FUNCTION_TABLE_iii[$17 & 255]($18,$r)|0);
 HEAP32[$rc>>2] = $19;
 $20 = +HEAPF64[$r>>3];
 $21 = $20 * 8.64E+7;
 $22 = (~~$21)>>>0;
 $23 = +Math_abs($21) >= 1.0 ? $21 > 0.0 ? (~~+Math_min(+Math_floor($21 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($21 - +(~~$21 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $24 = HEAP32[$1>>2]|0;
 $25 = $24;
 $26 = $25;
 HEAP32[$26>>2] = $22;
 $27 = (($25) + 4)|0;
 $28 = $27;
 HEAP32[$28>>2] = $23;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3VdbeList($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apSub = 0, $db = 0, $i = 0, $j = 0, $j1 = 0, $nByte = 0, $nRow = 0, $nSub = 0, $pMem = 0, $pOp = 0, $pSub = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $zP4 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $nRow = sp + 60|0;
 $nSub = sp + 56|0;
 $apSub = sp + 52|0;
 $pSub = sp + 48|0;
 $db = sp + 44|0;
 $i = sp + 40|0;
 $rc = sp + 36|0;
 $pMem = sp + 32|0;
 $zP4 = sp + 28|0;
 $pOp = sp + 24|0;
 $j = sp + 20|0;
 $nByte = sp + 16|0;
 $j1 = sp + 12|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$nSub>>2] = 0;
 HEAP32[$apSub>>2] = 0;
 HEAP32[$pSub>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 92|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 40|0);
 HEAP32[$pMem>>2] = $7;
 $8 = HEAP32[$pMem>>2]|0;
 _releaseMemArray($8,8);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 104|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 40|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)==(7);
 if ($14) {
  $15 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($15);
  HEAP32[$0>>2] = 1;
  $343 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($343|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 136|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$nRow>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 144|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = ($21&65535) >>> 2;
 $23 = $22 & 3;
 $24 = $23&65535;
 $25 = ($24|0)==(1);
 L5: do {
  if ($25) {
   $26 = HEAP32[$1>>2]|0;
   $27 = ((($26)) + 92|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = ((($28)) + 360|0);
   HEAP32[$pSub>>2] = $29;
   $30 = HEAP32[$pSub>>2]|0;
   $31 = ((($30)) + 8|0);
   $32 = HEAP16[$31>>1]|0;
   $33 = $32&65535;
   $34 = $33 & 16;
   $35 = ($34|0)!=(0);
   if ($35) {
    $36 = HEAP32[$pSub>>2]|0;
    $37 = ((($36)) + 12|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = (($38>>>0) / 4)&-1;
    HEAP32[$nSub>>2] = $39;
    $40 = HEAP32[$pSub>>2]|0;
    $41 = ((($40)) + 16|0);
    $42 = HEAP32[$41>>2]|0;
    HEAP32[$apSub>>2] = $42;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $43 = HEAP32[$i>>2]|0;
    $44 = HEAP32[$nSub>>2]|0;
    $45 = ($43|0)<($44|0);
    if (!($45)) {
     break L5;
    }
    $46 = HEAP32[$i>>2]|0;
    $47 = HEAP32[$apSub>>2]|0;
    $48 = (($47) + ($46<<2)|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = ((($49)) + 4|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$nRow>>2]|0;
    $53 = (($52) + ($51))|0;
    HEAP32[$nRow>>2] = $53;
    $54 = HEAP32[$i>>2]|0;
    $55 = (($54) + 1)|0;
    HEAP32[$i>>2] = $55;
   }
  }
 } while(0);
 while(1) {
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 36|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$57>>2] = $59;
  HEAP32[$i>>2] = $58;
  $60 = HEAP32[$i>>2]|0;
  $61 = HEAP32[$nRow>>2]|0;
  $62 = ($60|0)<($61|0);
  if (!($62)) {
   break;
  }
  $63 = HEAP32[$1>>2]|0;
  $64 = ((($63)) + 144|0);
  $65 = HEAP16[$64>>1]|0;
  $66 = ($65&65535) >>> 2;
  $67 = $66 & 3;
  $68 = $67&65535;
  $69 = ($68|0)==(2);
  if (!($69)) {
   break;
  }
  $70 = HEAP32[$i>>2]|0;
  $71 = HEAP32[$1>>2]|0;
  $72 = ((($71)) + 88|0);
  $73 = HEAP32[$72>>2]|0;
  $74 = (($73) + (($70*20)|0)|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  $77 = ($76|0)!=(162);
  if (!($77)) {
   break;
  }
 }
 $78 = HEAP32[$i>>2]|0;
 $79 = HEAP32[$nRow>>2]|0;
 $80 = ($78|0)>=($79|0);
 do {
  if ($80) {
   $81 = HEAP32[$1>>2]|0;
   $82 = ((($81)) + 40|0);
   HEAP32[$82>>2] = 0;
   HEAP32[$rc>>2] = 101;
  } else {
   $83 = HEAP32[$db>>2]|0;
   $84 = ((($83)) + 248|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = HEAP32[$1>>2]|0;
    $88 = ((($87)) + 40|0);
    HEAP32[$88>>2] = 9;
    HEAP32[$rc>>2] = 1;
    $89 = HEAP32[$1>>2]|0;
    $90 = HEAP32[$1>>2]|0;
    $91 = ((($90)) + 40|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = (_sqlite3ErrStr($92)|0);
    _sqlite3VdbeError($89,$93,$vararg_buffer);
    break;
   }
   $94 = HEAP32[$i>>2]|0;
   $95 = HEAP32[$1>>2]|0;
   $96 = ((($95)) + 136|0);
   $97 = HEAP32[$96>>2]|0;
   $98 = ($94|0)<($97|0);
   if ($98) {
    $99 = HEAP32[$i>>2]|0;
    $100 = HEAP32[$1>>2]|0;
    $101 = ((($100)) + 88|0);
    $102 = HEAP32[$101>>2]|0;
    $103 = (($102) + (($99*20)|0)|0);
    HEAP32[$pOp>>2] = $103;
   } else {
    $104 = HEAP32[$1>>2]|0;
    $105 = ((($104)) + 136|0);
    $106 = HEAP32[$105>>2]|0;
    $107 = HEAP32[$i>>2]|0;
    $108 = (($107) - ($106))|0;
    HEAP32[$i>>2] = $108;
    HEAP32[$j>>2] = 0;
    while(1) {
     $109 = HEAP32[$i>>2]|0;
     $110 = HEAP32[$j>>2]|0;
     $111 = HEAP32[$apSub>>2]|0;
     $112 = (($111) + ($110<<2)|0);
     $113 = HEAP32[$112>>2]|0;
     $114 = ((($113)) + 4|0);
     $115 = HEAP32[$114>>2]|0;
     $116 = ($109|0)>=($115|0);
     if (!($116)) {
      break;
     }
     $117 = HEAP32[$j>>2]|0;
     $118 = HEAP32[$apSub>>2]|0;
     $119 = (($118) + ($117<<2)|0);
     $120 = HEAP32[$119>>2]|0;
     $121 = ((($120)) + 4|0);
     $122 = HEAP32[$121>>2]|0;
     $123 = HEAP32[$i>>2]|0;
     $124 = (($123) - ($122))|0;
     HEAP32[$i>>2] = $124;
     $125 = HEAP32[$j>>2]|0;
     $126 = (($125) + 1)|0;
     HEAP32[$j>>2] = $126;
    }
    $127 = HEAP32[$i>>2]|0;
    $128 = HEAP32[$j>>2]|0;
    $129 = HEAP32[$apSub>>2]|0;
    $130 = (($129) + ($128<<2)|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = HEAP32[$131>>2]|0;
    $133 = (($132) + (($127*20)|0)|0);
    HEAP32[$pOp>>2] = $133;
   }
   $134 = HEAP32[$1>>2]|0;
   $135 = ((($134)) + 144|0);
   $136 = HEAP16[$135>>1]|0;
   $137 = ($136&65535) >>> 2;
   $138 = $137 & 3;
   $139 = $138&65535;
   $140 = ($139|0)==(1);
   if ($140) {
    $141 = HEAP32[$pMem>>2]|0;
    $142 = ((($141)) + 8|0);
    HEAP16[$142>>1] = 4;
    $143 = HEAP32[$i>>2]|0;
    $144 = ($143|0)<(0);
    $145 = $144 << 31 >> 31;
    $146 = HEAP32[$pMem>>2]|0;
    $147 = $146;
    $148 = $147;
    HEAP32[$148>>2] = $143;
    $149 = (($147) + 4)|0;
    $150 = $149;
    HEAP32[$150>>2] = $145;
    $151 = HEAP32[$pMem>>2]|0;
    $152 = ((($151)) + 40|0);
    HEAP32[$pMem>>2] = $152;
    $153 = HEAP32[$pMem>>2]|0;
    $154 = ((($153)) + 8|0);
    HEAP16[$154>>1] = 2562;
    $155 = HEAP32[$pOp>>2]|0;
    $156 = HEAP8[$155>>0]|0;
    $157 = $156&255;
    $158 = (_sqlite3OpcodeName($157)|0);
    $159 = HEAP32[$pMem>>2]|0;
    $160 = ((($159)) + 16|0);
    HEAP32[$160>>2] = $158;
    $161 = HEAP32[$pMem>>2]|0;
    $162 = ((($161)) + 16|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = (_sqlite3Strlen30($163)|0);
    $165 = HEAP32[$pMem>>2]|0;
    $166 = ((($165)) + 12|0);
    HEAP32[$166>>2] = $164;
    $167 = HEAP32[$pMem>>2]|0;
    $168 = ((($167)) + 10|0);
    HEAP8[$168>>0] = 1;
    $169 = HEAP32[$pMem>>2]|0;
    $170 = ((($169)) + 40|0);
    HEAP32[$pMem>>2] = $170;
    $171 = HEAP32[$pOp>>2]|0;
    $172 = ((($171)) + 1|0);
    $173 = HEAP8[$172>>0]|0;
    $174 = $173 << 24 >> 24;
    $175 = ($174|0)==(-18);
    if ($175) {
     $176 = HEAP32[$nSub>>2]|0;
     $177 = (($176) + 1)|0;
     $178 = $177<<2;
     HEAP32[$nByte>>2] = $178;
     HEAP32[$j1>>2] = 0;
     while(1) {
      $179 = HEAP32[$j1>>2]|0;
      $180 = HEAP32[$nSub>>2]|0;
      $181 = ($179|0)<($180|0);
      if (!($181)) {
       break;
      }
      $182 = HEAP32[$j1>>2]|0;
      $183 = HEAP32[$apSub>>2]|0;
      $184 = (($183) + ($182<<2)|0);
      $185 = HEAP32[$184>>2]|0;
      $186 = HEAP32[$pOp>>2]|0;
      $187 = ((($186)) + 16|0);
      $188 = HEAP32[$187>>2]|0;
      $189 = ($185|0)==($188|0);
      if ($189) {
       break;
      }
      $190 = HEAP32[$j1>>2]|0;
      $191 = (($190) + 1)|0;
      HEAP32[$j1>>2] = $191;
     }
     $192 = HEAP32[$j1>>2]|0;
     $193 = HEAP32[$nSub>>2]|0;
     $194 = ($192|0)==($193|0);
     if ($194) {
      $195 = HEAP32[$pSub>>2]|0;
      $196 = HEAP32[$nByte>>2]|0;
      $197 = HEAP32[$nSub>>2]|0;
      $198 = ($197|0)!=(0);
      $199 = $198&1;
      $200 = (_sqlite3VdbeMemGrow($195,$196,$199)|0);
      $201 = (0)==($200|0);
      if ($201) {
       $202 = HEAP32[$pSub>>2]|0;
       $203 = ((($202)) + 16|0);
       $204 = HEAP32[$203>>2]|0;
       HEAP32[$apSub>>2] = $204;
       $205 = HEAP32[$pOp>>2]|0;
       $206 = ((($205)) + 16|0);
       $207 = HEAP32[$206>>2]|0;
       $208 = HEAP32[$nSub>>2]|0;
       $209 = (($208) + 1)|0;
       HEAP32[$nSub>>2] = $209;
       $210 = HEAP32[$apSub>>2]|0;
       $211 = (($210) + ($208<<2)|0);
       HEAP32[$211>>2] = $207;
       $212 = HEAP32[$pSub>>2]|0;
       $213 = ((($212)) + 8|0);
       $214 = HEAP16[$213>>1]|0;
       $215 = $214&65535;
       $216 = $215 | 16;
       $217 = $216&65535;
       HEAP16[$213>>1] = $217;
       $218 = HEAP32[$nSub>>2]|0;
       $219 = $218<<2;
       $220 = HEAP32[$pSub>>2]|0;
       $221 = ((($220)) + 12|0);
       HEAP32[$221>>2] = $219;
      }
     }
    }
   }
   $222 = HEAP32[$pMem>>2]|0;
   $223 = ((($222)) + 8|0);
   HEAP16[$223>>1] = 4;
   $224 = HEAP32[$pOp>>2]|0;
   $225 = ((($224)) + 4|0);
   $226 = HEAP32[$225>>2]|0;
   $227 = ($226|0)<(0);
   $228 = $227 << 31 >> 31;
   $229 = HEAP32[$pMem>>2]|0;
   $230 = $229;
   $231 = $230;
   HEAP32[$231>>2] = $226;
   $232 = (($230) + 4)|0;
   $233 = $232;
   HEAP32[$233>>2] = $228;
   $234 = HEAP32[$pMem>>2]|0;
   $235 = ((($234)) + 40|0);
   HEAP32[$pMem>>2] = $235;
   $236 = HEAP32[$pMem>>2]|0;
   $237 = ((($236)) + 8|0);
   HEAP16[$237>>1] = 4;
   $238 = HEAP32[$pOp>>2]|0;
   $239 = ((($238)) + 8|0);
   $240 = HEAP32[$239>>2]|0;
   $241 = ($240|0)<(0);
   $242 = $241 << 31 >> 31;
   $243 = HEAP32[$pMem>>2]|0;
   $244 = $243;
   $245 = $244;
   HEAP32[$245>>2] = $240;
   $246 = (($244) + 4)|0;
   $247 = $246;
   HEAP32[$247>>2] = $242;
   $248 = HEAP32[$pMem>>2]|0;
   $249 = ((($248)) + 40|0);
   HEAP32[$pMem>>2] = $249;
   $250 = HEAP32[$pMem>>2]|0;
   $251 = ((($250)) + 8|0);
   HEAP16[$251>>1] = 4;
   $252 = HEAP32[$pOp>>2]|0;
   $253 = ((($252)) + 12|0);
   $254 = HEAP32[$253>>2]|0;
   $255 = ($254|0)<(0);
   $256 = $255 << 31 >> 31;
   $257 = HEAP32[$pMem>>2]|0;
   $258 = $257;
   $259 = $258;
   HEAP32[$259>>2] = $254;
   $260 = (($258) + 4)|0;
   $261 = $260;
   HEAP32[$261>>2] = $256;
   $262 = HEAP32[$pMem>>2]|0;
   $263 = ((($262)) + 40|0);
   HEAP32[$pMem>>2] = $263;
   $264 = HEAP32[$pMem>>2]|0;
   $265 = (_sqlite3VdbeMemClearAndResize($264,100)|0);
   $266 = ($265|0)!=(0);
   if ($266) {
    HEAP32[$0>>2] = 1;
    $343 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($343|0);
   }
   $267 = HEAP32[$pMem>>2]|0;
   $268 = ((($267)) + 8|0);
   HEAP16[$268>>1] = 514;
   $269 = HEAP32[$pOp>>2]|0;
   $270 = HEAP32[$pMem>>2]|0;
   $271 = ((($270)) + 16|0);
   $272 = HEAP32[$271>>2]|0;
   $273 = HEAP32[$pMem>>2]|0;
   $274 = ((($273)) + 24|0);
   $275 = HEAP32[$274>>2]|0;
   $276 = (_displayP4($269,$272,$275)|0);
   HEAP32[$zP4>>2] = $276;
   $277 = HEAP32[$zP4>>2]|0;
   $278 = HEAP32[$pMem>>2]|0;
   $279 = ((($278)) + 16|0);
   $280 = HEAP32[$279>>2]|0;
   $281 = ($277|0)!=($280|0);
   $282 = HEAP32[$pMem>>2]|0;
   if ($281) {
    $283 = ((($282)) + 12|0);
    HEAP32[$283>>2] = 0;
    $284 = HEAP32[$pMem>>2]|0;
    $285 = HEAP32[$zP4>>2]|0;
    (_sqlite3VdbeMemSetStr($284,$285,-1,1,0)|0);
   } else {
    $286 = ((($282)) + 16|0);
    $287 = HEAP32[$286>>2]|0;
    $288 = (_sqlite3Strlen30($287)|0);
    $289 = HEAP32[$pMem>>2]|0;
    $290 = ((($289)) + 12|0);
    HEAP32[$290>>2] = $288;
    $291 = HEAP32[$pMem>>2]|0;
    $292 = ((($291)) + 10|0);
    HEAP8[$292>>0] = 1;
   }
   $293 = HEAP32[$pMem>>2]|0;
   $294 = ((($293)) + 40|0);
   HEAP32[$pMem>>2] = $294;
   $295 = HEAP32[$1>>2]|0;
   $296 = ((($295)) + 144|0);
   $297 = HEAP16[$296>>1]|0;
   $298 = ($297&65535) >>> 2;
   $299 = $298 & 3;
   $300 = $299&65535;
   $301 = ($300|0)==(1);
   do {
    if ($301) {
     $302 = HEAP32[$pMem>>2]|0;
     $303 = (_sqlite3VdbeMemClearAndResize($302,4)|0);
     $304 = ($303|0)!=(0);
     if (!($304)) {
      $305 = HEAP32[$pMem>>2]|0;
      $306 = ((($305)) + 8|0);
      HEAP16[$306>>1] = 514;
      $307 = HEAP32[$pMem>>2]|0;
      $308 = ((($307)) + 12|0);
      HEAP32[$308>>2] = 2;
      $309 = HEAP32[$pMem>>2]|0;
      $310 = ((($309)) + 16|0);
      $311 = HEAP32[$310>>2]|0;
      $312 = HEAP32[$pOp>>2]|0;
      $313 = ((($312)) + 3|0);
      $314 = HEAP8[$313>>0]|0;
      $315 = $314&255;
      HEAP32[$vararg_buffer2>>2] = $315;
      (_sqlite3_snprintf(3,$311,41910,$vararg_buffer2)|0);
      $316 = HEAP32[$pMem>>2]|0;
      $317 = ((($316)) + 10|0);
      HEAP8[$317>>0] = 1;
      $318 = HEAP32[$pMem>>2]|0;
      $319 = ((($318)) + 40|0);
      HEAP32[$pMem>>2] = $319;
      $320 = HEAP32[$pMem>>2]|0;
      $321 = ((($320)) + 8|0);
      HEAP16[$321>>1] = 1;
      break;
     }
     HEAP32[$0>>2] = 1;
     $343 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($343|0);
    }
   } while(0);
   $322 = HEAP32[$1>>2]|0;
   $323 = ((($322)) + 144|0);
   $324 = HEAP16[$323>>1]|0;
   $325 = ($324&65535) >>> 2;
   $326 = $325 & 3;
   $327 = $326&65535;
   $328 = (($327) - 1)|0;
   $329 = $328<<2;
   $330 = (8 - ($329))|0;
   $331 = $330&65535;
   $332 = HEAP32[$1>>2]|0;
   $333 = ((($332)) + 140|0);
   HEAP16[$333>>1] = $331;
   $334 = HEAP32[$1>>2]|0;
   $335 = ((($334)) + 92|0);
   $336 = HEAP32[$335>>2]|0;
   $337 = ((($336)) + 40|0);
   $338 = HEAP32[$1>>2]|0;
   $339 = ((($338)) + 104|0);
   HEAP32[$339>>2] = $337;
   $340 = HEAP32[$1>>2]|0;
   $341 = ((($340)) + 40|0);
   HEAP32[$341>>2] = 0;
   HEAP32[$rc>>2] = 100;
  }
 } while(0);
 $342 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $342;
 $343 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($343|0);
}
function _sqlite3VdbeExec($p) {
 $p = $p|0;
 var $$old = 0, $$old27 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0;
 var $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0;
 var $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0;
 var $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0;
 var $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0;
 var $1086 = 0, $1087 = 0, $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0;
 var $1103 = 0, $1104 = 0, $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0;
 var $1121 = 0, $1122 = 0, $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0.0, $1138 = 0, $1139 = 0.0;
 var $114 = 0, $1140 = 0, $1141 = 0, $1142 = 0, $1143 = 0.0, $1144 = 0.0, $1145 = 0.0, $1146 = 0.0, $1147 = 0.0, $1148 = 0.0, $1149 = 0.0, $115 = 0, $1150 = 0.0, $1151 = 0.0, $1152 = 0.0, $1153 = 0, $1154 = 0.0, $1155 = 0.0, $1156 = 0.0, $1157 = 0.0;
 var $1158 = 0, $1159 = 0, $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0.0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0;
 var $1176 = 0, $1177 = 0, $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0;
 var $1194 = 0, $1195 = 0, $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0.0, $1208 = 0.0, $1209 = 0, $121 = 0, $1210 = 0;
 var $1211 = 0.0, $1212 = 0, $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0;
 var $123 = 0, $1230 = 0, $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0;
 var $1248 = 0, $1249 = 0, $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0;
 var $1266 = 0, $1267 = 0, $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0;
 var $1284 = 0, $1285 = 0, $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0;
 var $1301 = 0, $1302 = 0, $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0;
 var $132 = 0, $1320 = 0, $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0;
 var $1338 = 0, $1339 = 0, $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0;
 var $1356 = 0, $1357 = 0, $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0;
 var $1374 = 0, $1375 = 0, $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $1382 = 0, $1383 = 0, $1384 = 0, $1385 = 0, $1386 = 0, $1387 = 0, $1388 = 0, $1389 = 0, $139 = 0, $1390 = 0, $1391 = 0;
 var $1392 = 0, $1393 = 0, $1394 = 0, $1395 = 0, $1396 = 0, $1397 = 0, $1398 = 0, $1399 = 0, $14 = 0, $140 = 0, $1400 = 0, $1401 = 0, $1402 = 0, $1403 = 0, $1404 = 0, $1405 = 0, $1406 = 0, $1407 = 0, $1408 = 0, $1409 = 0;
 var $141 = 0, $1410 = 0, $1411 = 0, $1412 = 0, $1413 = 0, $1414 = 0, $1415 = 0, $1416 = 0, $1417 = 0, $1418 = 0, $1419 = 0, $142 = 0, $1420 = 0, $1421 = 0, $1422 = 0, $1423 = 0, $1424 = 0, $1425 = 0, $1426 = 0, $1427 = 0;
 var $1428 = 0, $1429 = 0, $143 = 0, $1430 = 0, $1431 = 0, $1432 = 0, $1433 = 0, $1434 = 0, $1435 = 0, $1436 = 0, $1437 = 0, $1438 = 0, $1439 = 0, $144 = 0, $1440 = 0, $1441 = 0, $1442 = 0, $1443 = 0, $1444 = 0, $1445 = 0;
 var $1446 = 0, $1447 = 0, $1448 = 0, $1449 = 0, $145 = 0, $1450 = 0, $1451 = 0, $1452 = 0, $1453 = 0, $1454 = 0, $1455 = 0, $1456 = 0, $1457 = 0, $1458 = 0, $1459 = 0, $146 = 0, $1460 = 0, $1461 = 0, $1462 = 0, $1463 = 0;
 var $1464 = 0, $1465 = 0, $1466 = 0, $1467 = 0, $1468 = 0, $1469 = 0, $147 = 0, $1470 = 0, $1471 = 0, $1472 = 0, $1473 = 0, $1474 = 0, $1475 = 0, $1476 = 0, $1477 = 0, $1478 = 0, $1479 = 0, $148 = 0, $1480 = 0, $1481 = 0;
 var $1482 = 0, $1483 = 0, $1484 = 0, $1485 = 0, $1486 = 0, $1487 = 0, $1488 = 0, $1489 = 0, $149 = 0, $1490 = 0, $1491 = 0, $1492 = 0, $1493 = 0, $1494 = 0, $1495 = 0, $1496 = 0, $1497 = 0, $1498 = 0, $1499 = 0, $15 = 0;
 var $150 = 0, $1500 = 0, $1501 = 0, $1502 = 0, $1503 = 0, $1504 = 0, $1505 = 0, $1506 = 0, $1507 = 0, $1508 = 0, $1509 = 0, $151 = 0, $1510 = 0, $1511 = 0, $1512 = 0, $1513 = 0, $1514 = 0, $1515 = 0, $1516 = 0, $1517 = 0;
 var $1518 = 0, $1519 = 0, $152 = 0, $1520 = 0, $1521 = 0, $1522 = 0, $1523 = 0, $1524 = 0, $1525 = 0, $1526 = 0, $1527 = 0, $1528 = 0, $1529 = 0, $153 = 0, $1530 = 0, $1531 = 0, $1532 = 0, $1533 = 0, $1534 = 0, $1535 = 0;
 var $1536 = 0, $1537 = 0, $1538 = 0, $1539 = 0, $154 = 0, $1540 = 0, $1541 = 0, $1542 = 0, $1543 = 0, $1544 = 0, $1545 = 0, $1546 = 0, $1547 = 0, $1548 = 0, $1549 = 0, $155 = 0, $1550 = 0, $1551 = 0, $1552 = 0, $1553 = 0;
 var $1554 = 0, $1555 = 0, $1556 = 0, $1557 = 0, $1558 = 0, $1559 = 0, $156 = 0, $1560 = 0, $1561 = 0, $1562 = 0, $1563 = 0, $1564 = 0, $1565 = 0, $1566 = 0, $1567 = 0, $1568 = 0, $1569 = 0, $157 = 0, $1570 = 0, $1571 = 0;
 var $1572 = 0, $1573 = 0, $1574 = 0, $1575 = 0, $1576 = 0, $1577 = 0, $1578 = 0, $1579 = 0, $158 = 0, $1580 = 0, $1581 = 0, $1582 = 0, $1583 = 0, $1584 = 0, $1585 = 0, $1586 = 0, $1587 = 0, $1588 = 0, $1589 = 0, $159 = 0;
 var $1590 = 0, $1591 = 0, $1592 = 0, $1593 = 0, $1594 = 0, $1595 = 0, $1596 = 0, $1597 = 0, $1598 = 0, $1599 = 0, $16 = 0, $160 = 0, $1600 = 0, $1601 = 0, $1602 = 0, $1603 = 0, $1604 = 0, $1605 = 0, $1606 = 0, $1607 = 0;
 var $1608 = 0, $1609 = 0, $161 = 0, $1610 = 0, $1611 = 0, $1612 = 0, $1613 = 0, $1614 = 0, $1615 = 0, $1616 = 0, $1617 = 0, $1618 = 0, $1619 = 0, $162 = 0, $1620 = 0, $1621 = 0, $1622 = 0, $1623 = 0, $1624 = 0, $1625 = 0;
 var $1626 = 0, $1627 = 0, $1628 = 0, $1629 = 0, $163 = 0, $1630 = 0, $1631 = 0, $1632 = 0, $1633 = 0, $1634 = 0, $1635 = 0, $1636 = 0, $1637 = 0, $1638 = 0, $1639 = 0, $164 = 0, $1640 = 0, $1641 = 0, $1642 = 0, $1643 = 0;
 var $1644 = 0, $1645 = 0, $1646 = 0, $1647 = 0, $1648 = 0, $1649 = 0, $165 = 0, $1650 = 0, $1651 = 0, $1652 = 0, $1653 = 0, $1654 = 0, $1655 = 0, $1656 = 0, $1657 = 0, $1658 = 0, $1659 = 0, $166 = 0, $1660 = 0, $1661 = 0;
 var $1662 = 0, $1663 = 0, $1664 = 0, $1665 = 0, $1666 = 0, $1667 = 0, $1668 = 0, $1669 = 0, $167 = 0, $1670 = 0, $1671 = 0, $1672 = 0, $1673 = 0, $1674 = 0, $1675 = 0, $1676 = 0, $1677 = 0, $1678 = 0, $1679 = 0, $168 = 0;
 var $1680 = 0, $1681 = 0, $1682 = 0, $1683 = 0, $1684 = 0, $1685 = 0, $1686 = 0, $1687 = 0, $1688 = 0, $1689 = 0, $169 = 0, $1690 = 0, $1691 = 0, $1692 = 0, $1693 = 0, $1694 = 0, $1695 = 0, $1696 = 0, $1697 = 0, $1698 = 0;
 var $1699 = 0, $17 = 0, $170 = 0, $1700 = 0, $1701 = 0, $1702 = 0, $1703 = 0, $1704 = 0, $1705 = 0, $1706 = 0, $1707 = 0, $1708 = 0, $1709 = 0, $171 = 0, $1710 = 0, $1711 = 0, $1712 = 0, $1713 = 0, $1714 = 0, $1715 = 0;
 var $1716 = 0, $1717 = 0, $1718 = 0, $1719 = 0, $172 = 0, $1720 = 0, $1721 = 0, $1722 = 0, $1723 = 0, $1724 = 0, $1725 = 0, $1726 = 0, $1727 = 0, $1728 = 0, $1729 = 0, $173 = 0, $1730 = 0, $1731 = 0, $1732 = 0, $1733 = 0;
 var $1734 = 0, $1735 = 0, $1736 = 0, $1737 = 0, $1738 = 0, $1739 = 0, $174 = 0, $1740 = 0, $1741 = 0, $1742 = 0, $1743 = 0, $1744 = 0, $1745 = 0, $1746 = 0, $1747 = 0, $1748 = 0, $1749 = 0, $175 = 0, $1750 = 0, $1751 = 0;
 var $1752 = 0, $1753 = 0, $1754 = 0, $1755 = 0, $1756 = 0, $1757 = 0, $1758 = 0, $1759 = 0, $176 = 0, $1760 = 0, $1761 = 0, $1762 = 0, $1763 = 0, $1764 = 0, $1765 = 0, $1766 = 0, $1767 = 0, $1768 = 0, $1769 = 0, $177 = 0;
 var $1770 = 0, $1771 = 0, $1772 = 0, $1773 = 0, $1774 = 0, $1775 = 0, $1776 = 0, $1777 = 0, $1778 = 0, $1779 = 0, $178 = 0, $1780 = 0, $1781 = 0, $1782 = 0, $1783 = 0, $1784 = 0, $1785 = 0, $1786 = 0, $1787 = 0, $1788 = 0;
 var $1789 = 0, $179 = 0, $1790 = 0, $1791 = 0, $1792 = 0, $1793 = 0, $1794 = 0, $1795 = 0, $1796 = 0, $1797 = 0, $1798 = 0, $1799 = 0, $18 = 0, $180 = 0, $1800 = 0, $1801 = 0, $1802 = 0, $1803 = 0, $1804 = 0, $1805 = 0;
 var $1806 = 0, $1807 = 0, $1808 = 0, $1809 = 0, $181 = 0, $1810 = 0, $1811 = 0, $1812 = 0, $1813 = 0, $1814 = 0, $1815 = 0, $1816 = 0, $1817 = 0, $1818 = 0, $1819 = 0, $182 = 0, $1820 = 0, $1821 = 0, $1822 = 0, $1823 = 0;
 var $1824 = 0, $1825 = 0, $1826 = 0, $1827 = 0, $1828 = 0, $1829 = 0, $183 = 0, $1830 = 0, $1831 = 0, $1832 = 0, $1833 = 0, $1834 = 0, $1835 = 0, $1836 = 0, $1837 = 0, $1838 = 0, $1839 = 0, $184 = 0, $1840 = 0, $1841 = 0;
 var $1842 = 0, $1843 = 0, $1844 = 0, $1845 = 0, $1846 = 0, $1847 = 0, $1848 = 0, $1849 = 0, $185 = 0, $1850 = 0, $1851 = 0, $1852 = 0, $1853 = 0, $1854 = 0, $1855 = 0, $1856 = 0, $1857 = 0, $1858 = 0, $1859 = 0, $186 = 0;
 var $1860 = 0, $1861 = 0, $1862 = 0, $1863 = 0, $1864 = 0, $1865 = 0, $1866 = 0, $1867 = 0, $1868 = 0, $1869 = 0, $187 = 0, $1870 = 0, $1871 = 0, $1872 = 0, $1873 = 0, $1874 = 0, $1875 = 0, $1876 = 0, $1877 = 0, $1878 = 0;
 var $1879 = 0, $188 = 0, $1880 = 0, $1881 = 0, $1882 = 0, $1883 = 0, $1884 = 0, $1885 = 0, $1886 = 0, $1887 = 0, $1888 = 0, $1889 = 0, $189 = 0, $1890 = 0, $1891 = 0, $1892 = 0, $1893 = 0, $1894 = 0, $1895 = 0, $1896 = 0;
 var $1897 = 0, $1898 = 0, $1899 = 0, $19 = 0, $190 = 0, $1900 = 0, $1901 = 0, $1902 = 0, $1903 = 0, $1904 = 0, $1905 = 0, $1906 = 0, $1907 = 0, $1908 = 0, $1909 = 0, $191 = 0, $1910 = 0, $1911 = 0, $1912 = 0, $1913 = 0;
 var $1914 = 0, $1915 = 0, $1916 = 0, $1917 = 0, $1918 = 0, $1919 = 0, $192 = 0, $1920 = 0, $1921 = 0, $1922 = 0, $1923 = 0, $1924 = 0, $1925 = 0, $1926 = 0, $1927 = 0, $1928 = 0, $1929 = 0, $193 = 0, $1930 = 0, $1931 = 0;
 var $1932 = 0, $1933 = 0, $1934 = 0, $1935 = 0, $1936 = 0, $1937 = 0, $1938 = 0, $1939 = 0, $194 = 0, $1940 = 0, $1941 = 0, $1942 = 0, $1943 = 0, $1944 = 0, $1945 = 0, $1946 = 0, $1947 = 0, $1948 = 0, $1949 = 0, $195 = 0;
 var $1950 = 0, $1951 = 0, $1952 = 0, $1953 = 0, $1954 = 0, $1955 = 0, $1956 = 0, $1957 = 0, $1958 = 0, $1959 = 0, $196 = 0, $1960 = 0, $1961 = 0, $1962 = 0, $1963 = 0, $1964 = 0, $1965 = 0, $1966 = 0, $1967 = 0, $1968 = 0;
 var $1969 = 0, $197 = 0, $1970 = 0, $1971 = 0, $1972 = 0, $1973 = 0, $1974 = 0, $1975 = 0, $1976 = 0, $1977 = 0, $1978 = 0, $1979 = 0, $198 = 0, $1980 = 0, $1981 = 0, $1982 = 0, $1983 = 0, $1984 = 0, $1985 = 0, $1986 = 0;
 var $1987 = 0, $1988 = 0, $1989 = 0, $199 = 0, $1990 = 0, $1991 = 0, $1992 = 0, $1993 = 0, $1994 = 0, $1995 = 0, $1996 = 0, $1997 = 0, $1998 = 0, $1999 = 0, $2 = 0, $20 = 0, $200 = 0, $2000 = 0, $2001 = 0, $2002 = 0;
 var $2003 = 0, $2004 = 0, $2005 = 0, $2006 = 0, $2007 = 0, $2008 = 0, $2009 = 0, $201 = 0, $2010 = 0, $2011 = 0, $2012 = 0, $2013 = 0, $2014 = 0, $2015 = 0, $2016 = 0, $2017 = 0, $2018 = 0, $2019 = 0, $202 = 0, $2020 = 0;
 var $2021 = 0, $2022 = 0, $2023 = 0, $2024 = 0, $2025 = 0, $2026 = 0, $2027 = 0, $2028 = 0, $2029 = 0, $203 = 0, $2030 = 0, $2031 = 0, $2032 = 0, $2033 = 0, $2034 = 0, $2035 = 0, $2036 = 0, $2037 = 0, $2038 = 0, $2039 = 0;
 var $204 = 0, $2040 = 0, $2041 = 0, $2042 = 0, $2043 = 0, $2044 = 0, $2045 = 0, $2046 = 0, $2047 = 0, $2048 = 0, $2049 = 0, $205 = 0, $2050 = 0, $2051 = 0, $2052 = 0, $2053 = 0, $2054 = 0, $2055 = 0, $2056 = 0, $2057 = 0;
 var $2058 = 0, $2059 = 0, $206 = 0, $2060 = 0, $2061 = 0, $2062 = 0, $2063 = 0, $2064 = 0, $2065 = 0, $2066 = 0, $2067 = 0, $2068 = 0, $2069 = 0, $207 = 0, $2070 = 0, $2071 = 0, $2072 = 0, $2073 = 0, $2074 = 0, $2075 = 0;
 var $2076 = 0, $2077 = 0, $2078 = 0, $2079 = 0, $208 = 0, $2080 = 0, $2081 = 0, $2082 = 0, $2083 = 0, $2084 = 0, $2085 = 0, $2086 = 0, $2087 = 0, $2088 = 0, $2089 = 0, $209 = 0, $2090 = 0, $2091 = 0, $2092 = 0, $2093 = 0;
 var $2094 = 0, $2095 = 0, $2096 = 0, $2097 = 0, $2098 = 0, $2099 = 0, $21 = 0, $210 = 0, $2100 = 0, $2101 = 0, $2102 = 0, $2103 = 0, $2104 = 0, $2105 = 0, $2106 = 0, $2107 = 0, $2108 = 0, $2109 = 0, $211 = 0, $2110 = 0;
 var $2111 = 0, $2112 = 0, $2113 = 0, $2114 = 0, $2115 = 0, $2116 = 0, $2117 = 0, $2118 = 0, $2119 = 0, $212 = 0, $2120 = 0, $2121 = 0, $2122 = 0, $2123 = 0, $2124 = 0, $2125 = 0, $2126 = 0, $2127 = 0, $2128 = 0, $2129 = 0;
 var $213 = 0, $2130 = 0, $2131 = 0, $2132 = 0, $2133 = 0, $2134 = 0, $2135 = 0, $2136 = 0, $2137 = 0, $2138 = 0, $2139 = 0, $214 = 0, $2140 = 0, $2141 = 0, $2142 = 0, $2143 = 0, $2144 = 0, $2145 = 0, $2146 = 0, $2147 = 0;
 var $2148 = 0, $2149 = 0, $215 = 0, $2150 = 0, $2151 = 0, $2152 = 0, $2153 = 0, $2154 = 0, $2155 = 0, $2156 = 0, $2157 = 0, $2158 = 0, $2159 = 0, $216 = 0, $2160 = 0, $2161 = 0, $2162 = 0, $2163 = 0, $2164 = 0, $2165 = 0;
 var $2166 = 0, $2167 = 0, $2168 = 0, $2169 = 0, $217 = 0, $2170 = 0, $2171 = 0, $2172 = 0, $2173 = 0, $2174 = 0, $2175 = 0, $2176 = 0, $2177 = 0, $2178 = 0, $2179 = 0, $218 = 0, $2180 = 0, $2181 = 0, $2182 = 0, $2183 = 0;
 var $2184 = 0, $2185 = 0, $2186 = 0, $2187 = 0, $2188 = 0, $2189 = 0, $219 = 0, $2190 = 0, $2191 = 0, $2192 = 0, $2193 = 0, $2194 = 0, $2195 = 0, $2196 = 0, $2197 = 0, $2198 = 0, $2199 = 0, $22 = 0, $220 = 0, $2200 = 0;
 var $2201 = 0, $2202 = 0, $2203 = 0, $2204 = 0, $2205 = 0, $2206 = 0, $2207 = 0, $2208 = 0, $2209 = 0, $221 = 0, $2210 = 0, $2211 = 0, $2212 = 0, $2213 = 0, $2214 = 0, $2215 = 0, $2216 = 0, $2217 = 0, $2218 = 0, $2219 = 0;
 var $222 = 0, $2220 = 0, $2221 = 0, $2222 = 0, $2223 = 0, $2224 = 0, $2225 = 0, $2226 = 0, $2227 = 0, $2228 = 0, $2229 = 0, $223 = 0, $2230 = 0, $2231 = 0, $2232 = 0, $2233 = 0, $2234 = 0, $2235 = 0, $2236 = 0, $2237 = 0;
 var $2238 = 0, $2239 = 0, $224 = 0, $2240 = 0, $2241 = 0, $2242 = 0, $2243 = 0, $2244 = 0, $2245 = 0, $2246 = 0, $2247 = 0, $2248 = 0, $2249 = 0, $225 = 0, $2250 = 0, $2251 = 0, $2252 = 0.0, $2253 = 0, $2254 = 0, $2255 = 0;
 var $2256 = 0, $2257 = 0, $2258 = 0, $2259 = 0, $226 = 0, $2260 = 0, $2261 = 0, $2262 = 0, $2263 = 0, $2264 = 0, $2265 = 0, $2266 = 0, $2267 = 0, $2268 = 0, $2269 = 0, $227 = 0, $2270 = 0, $2271 = 0, $2272 = 0, $2273 = 0;
 var $2274 = 0, $2275 = 0, $2276 = 0, $2277 = 0, $2278 = 0, $2279 = 0, $228 = 0, $2280 = 0, $2281 = 0, $2282 = 0, $2283 = 0, $2284 = 0, $2285 = 0, $2286 = 0, $2287 = 0, $2288 = 0, $2289 = 0, $229 = 0, $2290 = 0, $2291 = 0;
 var $2292 = 0, $2293 = 0, $2294 = 0, $2295 = 0, $2296 = 0, $2297 = 0, $2298 = 0, $2299 = 0, $23 = 0, $230 = 0, $2300 = 0, $2301 = 0, $2302 = 0, $2303 = 0, $2304 = 0, $2305 = 0, $2306 = 0, $2307 = 0, $2308 = 0, $2309 = 0;
 var $231 = 0, $2310 = 0, $2311 = 0, $2312 = 0, $2313 = 0, $2314 = 0, $2315 = 0, $2316 = 0, $2317 = 0, $2318 = 0, $2319 = 0, $232 = 0, $2320 = 0, $2321 = 0, $2322 = 0, $2323 = 0, $2324 = 0, $2325 = 0, $2326 = 0, $2327 = 0;
 var $2328 = 0, $2329 = 0, $233 = 0, $2330 = 0, $2331 = 0, $2332 = 0, $2333 = 0, $2334 = 0, $2335 = 0, $2336 = 0, $2337 = 0, $2338 = 0, $2339 = 0, $234 = 0, $2340 = 0, $2341 = 0, $2342 = 0, $2343 = 0, $2344 = 0, $2345 = 0;
 var $2346 = 0, $2347 = 0, $2348 = 0, $2349 = 0, $235 = 0, $2350 = 0, $2351 = 0, $2352 = 0, $2353 = 0, $2354 = 0, $2355 = 0, $2356 = 0, $2357 = 0, $2358 = 0, $2359 = 0, $236 = 0, $2360 = 0, $2361 = 0, $2362 = 0, $2363 = 0;
 var $2364 = 0, $2365 = 0, $2366 = 0, $2367 = 0, $2368 = 0, $2369 = 0, $237 = 0, $2370 = 0, $2371 = 0, $2372 = 0, $2373 = 0, $2374 = 0, $2375 = 0, $2376 = 0, $2377 = 0, $2378 = 0, $2379 = 0, $238 = 0, $2380 = 0, $2381 = 0;
 var $2382 = 0, $2383 = 0, $2384 = 0, $2385 = 0, $2386 = 0, $2387 = 0, $2388 = 0, $2389 = 0, $239 = 0, $2390 = 0, $2391 = 0, $2392 = 0, $2393 = 0, $2394 = 0, $2395 = 0, $2396 = 0, $2397 = 0, $2398 = 0, $2399 = 0, $24 = 0;
 var $240 = 0, $2400 = 0, $2401 = 0, $2402 = 0, $2403 = 0, $2404 = 0, $2405 = 0, $2406 = 0, $2407 = 0, $2408 = 0, $2409 = 0, $241 = 0, $2410 = 0, $2411 = 0, $2412 = 0, $2413 = 0, $2414 = 0, $2415 = 0, $2416 = 0, $2417 = 0;
 var $2418 = 0, $2419 = 0, $242 = 0, $2420 = 0, $2421 = 0, $2422 = 0, $2423 = 0, $2424 = 0, $2425 = 0, $2426 = 0, $2427 = 0, $2428 = 0, $2429 = 0, $243 = 0, $2430 = 0, $2431 = 0, $2432 = 0, $2433 = 0, $2434 = 0, $2435 = 0;
 var $2436 = 0, $2437 = 0, $2438 = 0, $2439 = 0, $244 = 0, $2440 = 0, $2441 = 0, $2442 = 0, $2443 = 0, $2444 = 0, $2445 = 0, $2446 = 0, $2447 = 0, $2448 = 0, $2449 = 0, $245 = 0, $2450 = 0, $2451 = 0, $2452 = 0, $2453 = 0;
 var $2454 = 0, $2455 = 0, $2456 = 0, $2457 = 0, $2458 = 0, $2459 = 0, $246 = 0, $2460 = 0, $2461 = 0, $2462 = 0, $2463 = 0, $2464 = 0, $2465 = 0, $2466 = 0, $2467 = 0, $2468 = 0, $2469 = 0, $247 = 0, $2470 = 0, $2471 = 0;
 var $2472 = 0, $2473 = 0, $2474 = 0, $2475 = 0, $2476 = 0, $2477 = 0, $2478 = 0, $2479 = 0, $248 = 0, $2480 = 0, $2481 = 0, $2482 = 0, $2483 = 0, $2484 = 0, $2485 = 0, $2486 = 0, $2487 = 0, $2488 = 0, $2489 = 0, $249 = 0;
 var $2490 = 0, $2491 = 0, $2492 = 0, $2493 = 0, $2494 = 0, $2495 = 0, $2496 = 0, $2497 = 0, $2498 = 0, $2499 = 0, $25 = 0, $250 = 0, $2500 = 0, $2501 = 0, $2502 = 0, $2503 = 0, $2504 = 0, $2505 = 0, $2506 = 0, $2507 = 0;
 var $2508 = 0, $2509 = 0, $251 = 0, $2510 = 0, $2511 = 0, $2512 = 0, $2513 = 0, $2514 = 0, $2515 = 0, $2516 = 0, $2517 = 0, $2518 = 0, $2519 = 0, $252 = 0, $2520 = 0, $2521 = 0, $2522 = 0, $2523 = 0, $2524 = 0, $2525 = 0;
 var $2526 = 0, $2527 = 0, $2528 = 0, $2529 = 0, $253 = 0, $2530 = 0, $2531 = 0, $2532 = 0, $2533 = 0, $2534 = 0, $2535 = 0, $2536 = 0, $2537 = 0, $2538 = 0, $2539 = 0, $254 = 0, $2540 = 0, $2541 = 0, $2542 = 0, $2543 = 0;
 var $2544 = 0, $2545 = 0, $2546 = 0, $2547 = 0, $2548 = 0, $2549 = 0, $255 = 0, $2550 = 0, $2551 = 0, $2552 = 0, $2553 = 0, $2554 = 0, $2555 = 0, $2556 = 0, $2557 = 0, $2558 = 0, $2559 = 0, $256 = 0, $2560 = 0, $2561 = 0;
 var $2562 = 0, $2563 = 0, $2564 = 0, $2565 = 0, $2566 = 0, $2567 = 0, $2568 = 0, $2569 = 0, $257 = 0, $2570 = 0, $2571 = 0, $2572 = 0, $2573 = 0, $2574 = 0, $2575 = 0, $2576 = 0, $2577 = 0, $2578 = 0, $2579 = 0, $258 = 0;
 var $2580 = 0, $2581 = 0, $2582 = 0, $2583 = 0, $2584 = 0, $2585 = 0, $2586 = 0, $2587 = 0, $2588 = 0, $2589 = 0, $259 = 0, $2590 = 0, $2591 = 0, $2592 = 0, $2593 = 0, $2594 = 0, $2595 = 0, $2596 = 0, $2597 = 0, $2598 = 0;
 var $2599 = 0, $26 = 0, $260 = 0, $2600 = 0, $2601 = 0, $2602 = 0, $2603 = 0, $2604 = 0, $2605 = 0, $2606 = 0, $2607 = 0, $2608 = 0, $2609 = 0, $261 = 0, $2610 = 0, $2611 = 0, $2612 = 0, $2613 = 0, $2614 = 0, $2615 = 0;
 var $2616 = 0, $2617 = 0, $2618 = 0, $2619 = 0, $262 = 0, $2620 = 0, $2621 = 0, $2622 = 0, $2623 = 0, $2624 = 0, $2625 = 0, $2626 = 0, $2627 = 0, $2628 = 0, $2629 = 0, $263 = 0, $2630 = 0, $2631 = 0, $2632 = 0, $2633 = 0;
 var $2634 = 0, $2635 = 0, $2636 = 0, $2637 = 0, $2638 = 0, $2639 = 0, $264 = 0, $2640 = 0, $2641 = 0, $2642 = 0, $2643 = 0, $2644 = 0, $2645 = 0, $2646 = 0, $2647 = 0, $2648 = 0, $2649 = 0, $265 = 0, $2650 = 0, $2651 = 0;
 var $2652 = 0, $2653 = 0, $2654 = 0, $2655 = 0, $2656 = 0, $2657 = 0, $2658 = 0, $2659 = 0, $266 = 0, $2660 = 0, $2661 = 0, $2662 = 0, $2663 = 0, $2664 = 0, $2665 = 0, $2666 = 0, $2667 = 0, $2668 = 0, $2669 = 0, $267 = 0;
 var $2670 = 0, $2671 = 0, $2672 = 0, $2673 = 0, $2674 = 0, $2675 = 0, $2676 = 0, $2677 = 0, $2678 = 0, $2679 = 0, $268 = 0, $2680 = 0, $2681 = 0, $2682 = 0, $2683 = 0, $2684 = 0, $2685 = 0, $2686 = 0, $2687 = 0, $2688 = 0;
 var $2689 = 0, $269 = 0, $2690 = 0, $2691 = 0, $2692 = 0, $2693 = 0, $2694 = 0, $2695 = 0, $2696 = 0, $2697 = 0, $2698 = 0, $2699 = 0, $27 = 0, $270 = 0, $2700 = 0, $2701 = 0, $2702 = 0, $2703 = 0, $2704 = 0, $2705 = 0;
 var $2706 = 0, $2707 = 0, $2708 = 0, $2709 = 0, $271 = 0, $2710 = 0, $2711 = 0, $2712 = 0, $2713 = 0, $2714 = 0, $2715 = 0, $2716 = 0, $2717 = 0, $2718 = 0, $2719 = 0, $272 = 0, $2720 = 0, $2721 = 0, $2722 = 0, $2723 = 0;
 var $2724 = 0, $2725 = 0, $2726 = 0, $2727 = 0, $2728 = 0, $2729 = 0, $273 = 0, $2730 = 0, $2731 = 0, $2732 = 0, $2733 = 0, $2734 = 0, $2735 = 0, $2736 = 0, $2737 = 0, $2738 = 0, $2739 = 0, $274 = 0, $2740 = 0, $2741 = 0;
 var $2742 = 0, $2743 = 0, $2744 = 0, $2745 = 0, $2746 = 0, $2747 = 0, $2748 = 0, $2749 = 0, $275 = 0, $2750 = 0, $2751 = 0, $2752 = 0, $2753 = 0, $2754 = 0, $2755 = 0, $2756 = 0, $2757 = 0, $2758 = 0, $2759 = 0, $276 = 0;
 var $2760 = 0, $2761 = 0, $2762 = 0, $2763 = 0, $2764 = 0, $2765 = 0, $2766 = 0, $2767 = 0, $2768 = 0, $2769 = 0, $277 = 0, $2770 = 0, $2771 = 0, $2772 = 0, $2773 = 0, $2774 = 0, $2775 = 0, $2776 = 0, $2777 = 0, $2778 = 0;
 var $2779 = 0, $278 = 0, $2780 = 0, $2781 = 0, $2782 = 0, $2783 = 0, $2784 = 0, $2785 = 0, $2786 = 0, $2787 = 0, $2788 = 0, $2789 = 0, $279 = 0, $2790 = 0, $2791 = 0, $2792 = 0, $2793 = 0, $2794 = 0, $2795 = 0, $2796 = 0;
 var $2797 = 0, $2798 = 0, $2799 = 0, $28 = 0, $280 = 0, $2800 = 0, $2801 = 0, $2802 = 0, $2803 = 0, $2804 = 0, $2805 = 0, $2806 = 0, $2807 = 0, $2808 = 0, $2809 = 0, $281 = 0, $2810 = 0, $2811 = 0, $2812 = 0, $2813 = 0;
 var $2814 = 0, $2815 = 0, $2816 = 0, $2817 = 0, $2818 = 0, $2819 = 0, $282 = 0, $2820 = 0, $2821 = 0, $2822 = 0, $2823 = 0, $2824 = 0, $2825 = 0, $2826 = 0, $2827 = 0, $2828 = 0, $2829 = 0, $283 = 0, $2830 = 0, $2831 = 0;
 var $2832 = 0, $2833 = 0, $2834 = 0, $2835 = 0, $2836 = 0, $2837 = 0, $2838 = 0, $2839 = 0, $284 = 0, $2840 = 0, $2841 = 0, $2842 = 0, $2843 = 0, $2844 = 0, $2845 = 0, $2846 = 0, $2847 = 0, $2848 = 0, $2849 = 0, $285 = 0;
 var $2850 = 0, $2851 = 0, $2852 = 0, $2853 = 0, $2854 = 0, $2855 = 0, $2856 = 0, $2857 = 0, $2858 = 0, $2859 = 0, $286 = 0, $2860 = 0, $2861 = 0, $2862 = 0, $2863 = 0, $2864 = 0, $2865 = 0, $2866 = 0, $2867 = 0, $2868 = 0;
 var $2869 = 0, $287 = 0, $2870 = 0, $2871 = 0, $2872 = 0, $2873 = 0, $2874 = 0, $2875 = 0, $2876 = 0, $2877 = 0, $2878 = 0, $2879 = 0, $288 = 0, $2880 = 0, $2881 = 0, $2882 = 0, $2883 = 0, $2884 = 0, $2885 = 0, $2886 = 0;
 var $2887 = 0, $2888 = 0, $2889 = 0, $289 = 0, $2890 = 0, $2891 = 0, $2892 = 0, $2893 = 0, $2894 = 0, $2895 = 0, $2896 = 0, $2897 = 0, $2898 = 0, $2899 = 0, $29 = 0, $290 = 0, $2900 = 0, $2901 = 0, $2902 = 0, $2903 = 0;
 var $2904 = 0, $2905 = 0, $2906 = 0, $2907 = 0, $2908 = 0, $2909 = 0, $291 = 0, $2910 = 0, $2911 = 0, $2912 = 0, $2913 = 0, $2914 = 0, $2915 = 0, $2916 = 0, $2917 = 0, $2918 = 0, $2919 = 0, $292 = 0, $2920 = 0, $2921 = 0;
 var $2922 = 0, $2923 = 0, $2924 = 0, $2925 = 0, $2926 = 0, $2927 = 0, $2928 = 0, $2929 = 0, $293 = 0, $2930 = 0, $2931 = 0, $2932 = 0, $2933 = 0, $2934 = 0, $2935 = 0, $2936 = 0, $2937 = 0, $2938 = 0, $2939 = 0, $294 = 0;
 var $2940 = 0, $2941 = 0, $2942 = 0, $2943 = 0, $2944 = 0, $2945 = 0, $2946 = 0, $2947 = 0, $2948 = 0, $2949 = 0, $295 = 0, $2950 = 0, $2951 = 0, $2952 = 0, $2953 = 0, $2954 = 0, $2955 = 0, $2956 = 0, $2957 = 0, $2958 = 0;
 var $2959 = 0, $296 = 0, $2960 = 0, $2961 = 0, $2962 = 0, $2963 = 0, $2964 = 0, $2965 = 0, $2966 = 0, $2967 = 0, $2968 = 0, $2969 = 0, $297 = 0, $2970 = 0, $2971 = 0, $2972 = 0, $2973 = 0, $2974 = 0, $2975 = 0, $2976 = 0;
 var $2977 = 0, $2978 = 0, $2979 = 0, $298 = 0, $2980 = 0, $2981 = 0, $2982 = 0, $2983 = 0, $2984 = 0, $2985 = 0, $2986 = 0, $2987 = 0, $2988 = 0, $2989 = 0, $299 = 0, $2990 = 0, $2991 = 0, $2992 = 0, $2993 = 0, $2994 = 0;
 var $2995 = 0, $2996 = 0, $2997 = 0, $2998 = 0, $2999 = 0, $3 = 0, $30 = 0, $300 = 0, $3000 = 0, $3001 = 0, $3002 = 0, $3003 = 0, $3004 = 0, $3005 = 0, $3006 = 0, $3007 = 0, $3008 = 0, $3009 = 0, $301 = 0, $3010 = 0;
 var $3011 = 0, $3012 = 0, $3013 = 0, $3014 = 0, $3015 = 0, $3016 = 0, $3017 = 0, $3018 = 0, $3019 = 0, $302 = 0, $3020 = 0, $3021 = 0, $3022 = 0, $3023 = 0, $3024 = 0, $3025 = 0, $3026 = 0, $3027 = 0, $3028 = 0, $3029 = 0;
 var $303 = 0, $3030 = 0, $3031 = 0, $3032 = 0, $3033 = 0, $3034 = 0, $3035 = 0, $3036 = 0, $3037 = 0, $3038 = 0, $3039 = 0, $304 = 0, $3040 = 0, $3041 = 0, $3042 = 0, $3043 = 0, $3044 = 0, $3045 = 0, $3046 = 0, $3047 = 0;
 var $3048 = 0, $3049 = 0, $305 = 0, $3050 = 0, $3051 = 0, $3052 = 0, $3053 = 0, $3054 = 0, $3055 = 0, $3056 = 0, $3057 = 0, $3058 = 0, $3059 = 0, $306 = 0, $3060 = 0, $3061 = 0, $3062 = 0, $3063 = 0, $3064 = 0, $3065 = 0;
 var $3066 = 0, $3067 = 0, $3068 = 0, $3069 = 0, $307 = 0, $3070 = 0, $3071 = 0, $3072 = 0, $3073 = 0, $3074 = 0, $3075 = 0, $3076 = 0, $3077 = 0, $3078 = 0, $3079 = 0, $308 = 0, $3080 = 0, $3081 = 0, $3082 = 0, $3083 = 0;
 var $3084 = 0, $3085 = 0, $3086 = 0, $3087 = 0, $3088 = 0, $3089 = 0, $309 = 0, $3090 = 0, $3091 = 0, $3092 = 0, $3093 = 0, $3094 = 0, $3095 = 0, $3096 = 0, $3097 = 0, $3098 = 0, $3099 = 0, $31 = 0, $310 = 0, $3100 = 0;
 var $3101 = 0, $3102 = 0, $3103 = 0, $3104 = 0, $3105 = 0, $3106 = 0, $3107 = 0, $3108 = 0, $3109 = 0, $311 = 0, $3110 = 0, $3111 = 0, $3112 = 0, $3113 = 0, $3114 = 0, $3115 = 0, $3116 = 0, $3117 = 0, $3118 = 0, $3119 = 0;
 var $312 = 0, $3120 = 0, $3121 = 0, $3122 = 0, $3123 = 0, $3124 = 0, $3125 = 0, $3126 = 0, $3127 = 0, $3128 = 0, $3129 = 0, $313 = 0, $3130 = 0, $3131 = 0, $3132 = 0, $3133 = 0, $3134 = 0, $3135 = 0, $3136 = 0, $3137 = 0;
 var $3138 = 0, $3139 = 0, $314 = 0, $3140 = 0, $3141 = 0, $3142 = 0, $3143 = 0, $3144 = 0, $3145 = 0, $3146 = 0, $3147 = 0, $3148 = 0, $3149 = 0, $315 = 0, $3150 = 0, $3151 = 0, $3152 = 0, $3153 = 0, $3154 = 0, $3155 = 0;
 var $3156 = 0, $3157 = 0, $3158 = 0, $3159 = 0, $316 = 0, $3160 = 0, $3161 = 0, $3162 = 0, $3163 = 0, $3164 = 0, $3165 = 0, $3166 = 0, $3167 = 0, $3168 = 0, $3169 = 0, $317 = 0, $3170 = 0, $3171 = 0, $3172 = 0, $3173 = 0;
 var $3174 = 0, $3175 = 0, $3176 = 0, $3177 = 0, $3178 = 0, $3179 = 0, $318 = 0, $3180 = 0, $3181 = 0, $3182 = 0, $3183 = 0, $3184 = 0, $3185 = 0, $3186 = 0, $3187 = 0, $3188 = 0, $3189 = 0, $319 = 0, $3190 = 0, $3191 = 0;
 var $3192 = 0, $3193 = 0, $3194 = 0, $3195 = 0, $3196 = 0, $3197 = 0, $3198 = 0, $3199 = 0, $32 = 0, $320 = 0, $3200 = 0, $3201 = 0, $3202 = 0, $3203 = 0, $3204 = 0, $3205 = 0, $3206 = 0, $3207 = 0, $3208 = 0, $3209 = 0;
 var $321 = 0, $3210 = 0, $3211 = 0, $3212 = 0, $3213 = 0, $3214 = 0, $3215 = 0, $3216 = 0, $3217 = 0, $3218 = 0, $3219 = 0, $322 = 0, $3220 = 0, $3221 = 0, $3222 = 0, $3223 = 0, $3224 = 0, $3225 = 0, $3226 = 0, $3227 = 0;
 var $3228 = 0, $3229 = 0, $323 = 0, $3230 = 0, $3231 = 0, $3232 = 0, $3233 = 0, $3234 = 0, $3235 = 0, $3236 = 0, $3237 = 0, $3238 = 0, $3239 = 0, $324 = 0, $3240 = 0, $3241 = 0, $3242 = 0, $3243 = 0, $3244 = 0, $3245 = 0;
 var $3246 = 0, $3247 = 0, $3248 = 0, $3249 = 0, $325 = 0, $3250 = 0, $3251 = 0, $3252 = 0, $3253 = 0, $3254 = 0, $3255 = 0, $3256 = 0, $3257 = 0, $3258 = 0, $3259 = 0, $326 = 0, $3260 = 0, $3261 = 0, $3262 = 0, $3263 = 0;
 var $3264 = 0, $3265 = 0, $3266 = 0, $3267 = 0, $3268 = 0, $3269 = 0, $327 = 0, $3270 = 0, $3271 = 0, $3272 = 0, $3273 = 0, $3274 = 0, $3275 = 0, $3276 = 0, $3277 = 0, $3278 = 0, $3279 = 0, $328 = 0, $3280 = 0, $3281 = 0;
 var $3282 = 0, $3283 = 0, $3284 = 0, $3285 = 0, $3286 = 0, $3287 = 0, $3288 = 0, $3289 = 0, $329 = 0, $3290 = 0, $3291 = 0, $3292 = 0, $3293 = 0, $3294 = 0, $3295 = 0, $3296 = 0, $3297 = 0, $3298 = 0, $3299 = 0, $33 = 0;
 var $330 = 0, $3300 = 0, $3301 = 0, $3302 = 0, $3303 = 0, $3304 = 0, $3305 = 0, $3306 = 0, $3307 = 0, $3308 = 0, $3309 = 0, $331 = 0, $3310 = 0, $3311 = 0, $3312 = 0, $3313 = 0, $3314 = 0, $3315 = 0, $3316 = 0, $3317 = 0;
 var $3318 = 0, $3319 = 0, $332 = 0, $3320 = 0, $3321 = 0, $3322 = 0, $3323 = 0, $3324 = 0, $3325 = 0, $3326 = 0, $3327 = 0, $3328 = 0, $3329 = 0, $333 = 0, $3330 = 0, $3331 = 0, $3332 = 0, $3333 = 0, $3334 = 0, $3335 = 0;
 var $3336 = 0, $3337 = 0, $3338 = 0, $3339 = 0, $334 = 0, $3340 = 0, $3341 = 0, $3342 = 0, $3343 = 0, $3344 = 0, $3345 = 0, $3346 = 0, $3347 = 0, $3348 = 0, $3349 = 0, $335 = 0, $3350 = 0, $3351 = 0, $3352 = 0, $3353 = 0;
 var $3354 = 0, $3355 = 0, $3356 = 0, $3357 = 0, $3358 = 0, $3359 = 0, $336 = 0, $3360 = 0, $3361 = 0, $3362 = 0, $3363 = 0, $3364 = 0, $3365 = 0, $3366 = 0, $3367 = 0, $3368 = 0, $3369 = 0, $337 = 0, $3370 = 0, $3371 = 0;
 var $3372 = 0, $3373 = 0, $3374 = 0, $3375 = 0, $3376 = 0, $3377 = 0, $3378 = 0, $3379 = 0, $338 = 0, $3380 = 0, $3381 = 0, $3382 = 0, $3383 = 0, $3384 = 0, $3385 = 0, $3386 = 0, $3387 = 0, $3388 = 0, $3389 = 0, $339 = 0;
 var $3390 = 0, $3391 = 0, $3392 = 0, $3393 = 0, $3394 = 0, $3395 = 0, $3396 = 0, $3397 = 0, $3398 = 0, $3399 = 0, $34 = 0, $340 = 0, $3400 = 0, $3401 = 0, $3402 = 0, $3403 = 0, $3404 = 0, $3405 = 0, $3406 = 0, $3407 = 0;
 var $3408 = 0, $3409 = 0, $341 = 0, $3410 = 0, $3411 = 0, $3412 = 0, $3413 = 0, $3414 = 0, $3415 = 0, $3416 = 0, $3417 = 0, $3418 = 0, $3419 = 0, $342 = 0, $3420 = 0, $3421 = 0, $3422 = 0, $3423 = 0, $3424 = 0, $3425 = 0;
 var $3426 = 0, $3427 = 0, $3428 = 0, $3429 = 0, $343 = 0, $3430 = 0, $3431 = 0, $3432 = 0, $3433 = 0, $3434 = 0, $3435 = 0, $3436 = 0, $3437 = 0, $3438 = 0, $3439 = 0, $344 = 0, $3440 = 0, $3441 = 0, $3442 = 0, $3443 = 0;
 var $3444 = 0, $3445 = 0, $3446 = 0, $3447 = 0, $3448 = 0, $3449 = 0, $345 = 0, $3450 = 0, $3451 = 0, $3452 = 0, $3453 = 0, $3454 = 0, $3455 = 0, $3456 = 0, $3457 = 0, $3458 = 0, $3459 = 0, $346 = 0, $3460 = 0, $3461 = 0;
 var $3462 = 0, $3463 = 0, $3464 = 0, $3465 = 0, $3466 = 0, $3467 = 0, $3468 = 0, $3469 = 0, $347 = 0, $3470 = 0, $3471 = 0, $3472 = 0, $3473 = 0, $3474 = 0, $3475 = 0, $3476 = 0, $3477 = 0, $3478 = 0, $3479 = 0, $348 = 0;
 var $3480 = 0, $3481 = 0, $3482 = 0, $3483 = 0, $3484 = 0, $3485 = 0, $3486 = 0, $3487 = 0, $3488 = 0, $3489 = 0, $349 = 0, $3490 = 0, $3491 = 0, $3492 = 0, $3493 = 0, $3494 = 0, $3495 = 0, $3496 = 0, $3497 = 0, $3498 = 0;
 var $3499 = 0, $35 = 0, $350 = 0, $3500 = 0, $3501 = 0, $3502 = 0, $3503 = 0, $3504 = 0, $3505 = 0, $3506 = 0, $3507 = 0, $3508 = 0, $3509 = 0, $351 = 0, $3510 = 0, $3511 = 0, $3512 = 0, $3513 = 0, $3514 = 0, $3515 = 0;
 var $3516 = 0, $3517 = 0, $3518 = 0, $3519 = 0, $352 = 0, $3520 = 0, $3521 = 0, $3522 = 0, $3523 = 0, $3524 = 0, $3525 = 0, $3526 = 0, $3527 = 0, $3528 = 0, $3529 = 0, $353 = 0, $3530 = 0, $3531 = 0, $3532 = 0, $3533 = 0;
 var $3534 = 0, $3535 = 0, $3536 = 0, $3537 = 0, $3538 = 0, $3539 = 0, $354 = 0, $3540 = 0, $3541 = 0, $3542 = 0, $3543 = 0, $3544 = 0, $3545 = 0, $3546 = 0, $3547 = 0, $3548 = 0, $3549 = 0, $355 = 0, $3550 = 0, $3551 = 0;
 var $3552 = 0, $3553 = 0, $3554 = 0, $3555 = 0, $3556 = 0, $3557 = 0, $3558 = 0, $3559 = 0, $356 = 0, $3560 = 0, $3561 = 0, $3562 = 0, $3563 = 0, $3564 = 0, $3565 = 0, $3566 = 0, $3567 = 0, $3568 = 0, $3569 = 0, $357 = 0;
 var $3570 = 0, $3571 = 0, $3572 = 0, $3573 = 0, $3574 = 0, $3575 = 0, $3576 = 0, $3577 = 0, $3578 = 0, $3579 = 0, $358 = 0, $3580 = 0, $3581 = 0, $3582 = 0, $3583 = 0, $3584 = 0, $3585 = 0, $3586 = 0, $3587 = 0, $3588 = 0;
 var $3589 = 0, $359 = 0, $3590 = 0, $3591 = 0, $3592 = 0, $3593 = 0, $3594 = 0, $3595 = 0, $3596 = 0, $3597 = 0, $3598 = 0, $3599 = 0, $36 = 0, $360 = 0, $3600 = 0, $3601 = 0, $3602 = 0, $3603 = 0, $3604 = 0, $3605 = 0;
 var $3606 = 0, $3607 = 0, $3608 = 0, $3609 = 0, $361 = 0, $3610 = 0, $3611 = 0, $3612 = 0, $3613 = 0, $3614 = 0, $3615 = 0, $3616 = 0, $3617 = 0, $3618 = 0, $3619 = 0, $362 = 0, $3620 = 0, $3621 = 0, $3622 = 0, $3623 = 0;
 var $3624 = 0, $3625 = 0, $3626 = 0, $3627 = 0, $3628 = 0, $3629 = 0, $363 = 0, $3630 = 0, $3631 = 0, $3632 = 0, $3633 = 0, $3634 = 0, $3635 = 0, $3636 = 0, $3637 = 0, $3638 = 0, $3639 = 0, $364 = 0, $3640 = 0, $3641 = 0;
 var $3642 = 0, $3643 = 0, $3644 = 0, $3645 = 0, $3646 = 0, $3647 = 0, $3648 = 0, $3649 = 0, $365 = 0, $3650 = 0, $3651 = 0, $3652 = 0, $3653 = 0, $3654 = 0, $3655 = 0, $3656 = 0, $3657 = 0, $3658 = 0, $3659 = 0, $366 = 0;
 var $3660 = 0, $3661 = 0, $3662 = 0, $3663 = 0, $3664 = 0, $3665 = 0, $3666 = 0, $3667 = 0, $3668 = 0, $3669 = 0, $367 = 0, $3670 = 0, $3671 = 0, $3672 = 0, $3673 = 0, $3674 = 0, $3675 = 0, $3676 = 0, $3677 = 0, $3678 = 0;
 var $3679 = 0, $368 = 0, $3680 = 0, $3681 = 0, $3682 = 0, $3683 = 0, $3684 = 0, $3685 = 0, $3686 = 0, $3687 = 0, $3688 = 0, $3689 = 0, $369 = 0, $3690 = 0, $3691 = 0, $3692 = 0, $3693 = 0, $3694 = 0, $3695 = 0, $3696 = 0;
 var $3697 = 0, $3698 = 0, $3699 = 0, $37 = 0, $370 = 0, $3700 = 0, $3701 = 0, $3702 = 0, $3703 = 0, $3704 = 0, $3705 = 0, $3706 = 0, $3707 = 0, $3708 = 0, $3709 = 0, $371 = 0, $3710 = 0, $3711 = 0, $3712 = 0, $3713 = 0;
 var $3714 = 0, $3715 = 0, $3716 = 0, $3717 = 0, $3718 = 0, $3719 = 0, $372 = 0, $3720 = 0, $3721 = 0, $3722 = 0, $3723 = 0, $3724 = 0, $3725 = 0, $3726 = 0, $3727 = 0, $3728 = 0, $3729 = 0, $373 = 0, $3730 = 0, $3731 = 0;
 var $3732 = 0, $3733 = 0, $3734 = 0, $3735 = 0, $3736 = 0, $3737 = 0, $3738 = 0, $3739 = 0, $374 = 0, $3740 = 0, $3741 = 0, $3742 = 0, $3743 = 0, $3744 = 0, $3745 = 0, $3746 = 0, $3747 = 0, $3748 = 0, $3749 = 0, $375 = 0;
 var $3750 = 0, $3751 = 0, $3752 = 0, $3753 = 0, $3754 = 0, $3755 = 0, $3756 = 0, $3757 = 0, $3758 = 0, $3759 = 0, $376 = 0, $3760 = 0, $3761 = 0, $3762 = 0, $3763 = 0, $3764 = 0, $3765 = 0, $3766 = 0, $3767 = 0, $3768 = 0;
 var $3769 = 0, $377 = 0, $3770 = 0, $3771 = 0, $3772 = 0, $3773 = 0, $3774 = 0, $3775 = 0, $3776 = 0, $3777 = 0, $3778 = 0, $3779 = 0, $378 = 0, $3780 = 0, $3781 = 0, $3782 = 0, $3783 = 0, $3784 = 0, $3785 = 0, $3786 = 0;
 var $3787 = 0, $3788 = 0, $3789 = 0, $379 = 0, $3790 = 0, $3791 = 0, $3792 = 0, $3793 = 0, $3794 = 0, $3795 = 0, $3796 = 0, $3797 = 0, $3798 = 0, $3799 = 0, $38 = 0, $380 = 0, $3800 = 0, $3801 = 0, $3802 = 0, $3803 = 0;
 var $3804 = 0, $3805 = 0, $3806 = 0, $3807 = 0, $3808 = 0, $3809 = 0, $381 = 0, $3810 = 0, $3811 = 0, $3812 = 0, $3813 = 0, $3814 = 0, $3815 = 0, $3816 = 0, $3817 = 0, $3818 = 0, $3819 = 0, $382 = 0, $3820 = 0, $3821 = 0;
 var $3822 = 0, $3823 = 0, $3824 = 0, $3825 = 0, $3826 = 0, $3827 = 0, $3828 = 0, $3829 = 0, $383 = 0, $3830 = 0, $3831 = 0, $3832 = 0, $3833 = 0, $3834 = 0, $3835 = 0, $3836 = 0, $3837 = 0, $3838 = 0, $3839 = 0, $384 = 0;
 var $3840 = 0, $3841 = 0, $3842 = 0, $3843 = 0, $3844 = 0, $3845 = 0, $3846 = 0, $3847 = 0, $3848 = 0, $3849 = 0, $385 = 0, $3850 = 0, $3851 = 0, $3852 = 0, $3853 = 0, $3854 = 0, $3855 = 0, $3856 = 0, $3857 = 0, $3858 = 0;
 var $3859 = 0, $386 = 0, $3860 = 0, $3861 = 0, $3862 = 0, $3863 = 0, $3864 = 0, $3865 = 0, $3866 = 0, $3867 = 0, $3868 = 0, $3869 = 0, $387 = 0, $3870 = 0, $3871 = 0, $3872 = 0, $3873 = 0, $3874 = 0, $3875 = 0, $3876 = 0;
 var $3877 = 0, $3878 = 0, $3879 = 0, $388 = 0, $3880 = 0, $3881 = 0, $3882 = 0, $3883 = 0, $3884 = 0, $3885 = 0, $3886 = 0, $3887 = 0, $3888 = 0, $3889 = 0, $389 = 0, $3890 = 0, $3891 = 0, $3892 = 0, $3893 = 0, $3894 = 0;
 var $3895 = 0, $3896 = 0, $3897 = 0, $3898 = 0, $3899 = 0, $39 = 0, $390 = 0, $3900 = 0, $3901 = 0, $3902 = 0, $3903 = 0, $3904 = 0, $3905 = 0, $3906 = 0, $3907 = 0, $3908 = 0, $3909 = 0, $391 = 0, $3910 = 0, $3911 = 0;
 var $3912 = 0, $3913 = 0, $3914 = 0, $3915 = 0, $3916 = 0, $3917 = 0, $3918 = 0, $3919 = 0, $392 = 0, $3920 = 0, $3921 = 0, $3922 = 0, $3923 = 0, $3924 = 0, $3925 = 0, $3926 = 0, $3927 = 0, $3928 = 0, $3929 = 0, $393 = 0;
 var $3930 = 0, $3931 = 0, $3932 = 0, $3933 = 0, $3934 = 0, $3935 = 0, $3936 = 0, $3937 = 0, $3938 = 0, $3939 = 0, $394 = 0, $3940 = 0, $3941 = 0, $3942 = 0, $3943 = 0, $3944 = 0, $3945 = 0, $3946 = 0, $3947 = 0, $3948 = 0;
 var $3949 = 0, $395 = 0, $3950 = 0, $3951 = 0, $3952 = 0, $3953 = 0, $3954 = 0, $3955 = 0, $3956 = 0, $3957 = 0, $3958 = 0, $3959 = 0, $396 = 0, $3960 = 0, $3961 = 0, $3962 = 0, $3963 = 0, $3964 = 0, $3965 = 0, $3966 = 0;
 var $3967 = 0, $3968 = 0, $3969 = 0, $397 = 0, $3970 = 0, $3971 = 0, $3972 = 0, $3973 = 0, $3974 = 0, $3975 = 0, $3976 = 0, $3977 = 0, $3978 = 0, $3979 = 0, $398 = 0, $3980 = 0, $3981 = 0, $3982 = 0, $3983 = 0, $3984 = 0;
 var $3985 = 0, $3986 = 0, $3987 = 0, $3988 = 0, $3989 = 0, $399 = 0, $3990 = 0, $3991 = 0, $3992 = 0, $3993 = 0, $3994 = 0, $3995 = 0, $3996 = 0, $3997 = 0, $3998 = 0, $3999 = 0, $4 = 0, $40 = 0, $400 = 0, $4000 = 0;
 var $4001 = 0, $4002 = 0, $4003 = 0, $4004 = 0, $4005 = 0, $4006 = 0, $4007 = 0, $4008 = 0, $4009 = 0, $401 = 0, $4010 = 0, $4011 = 0, $4012 = 0, $4013 = 0, $4014 = 0, $4015 = 0, $4016 = 0, $4017 = 0, $4018 = 0, $4019 = 0;
 var $402 = 0, $4020 = 0, $4021 = 0, $4022 = 0, $4023 = 0, $4024 = 0, $4025 = 0.0, $4026 = 0, $4027 = 0, $4028 = 0, $4029 = 0, $403 = 0, $4030 = 0, $4031 = 0, $4032 = 0.0, $4033 = 0, $4034 = 0, $4035 = 0, $4036 = 0, $4037 = 0;
 var $4038 = 0, $4039 = 0, $404 = 0, $4040 = 0.0, $4041 = 0, $4042 = 0, $4043 = 0, $4044 = 0, $4045 = 0, $4046 = 0, $4047 = 0.0, $4048 = 0, $4049 = 0, $405 = 0.0, $4050 = 0, $4051 = 0, $4052 = 0, $4053 = 0, $4054 = 0, $4055 = 0;
 var $4056 = 0, $4057 = 0, $4058 = 0, $4059 = 0, $406 = 0, $4060 = 0, $4061 = 0, $4062 = 0, $4063 = 0, $4064 = 0, $4065 = 0, $4066 = 0, $4067 = 0, $4068 = 0, $4069 = 0, $407 = 0, $4070 = 0, $4071 = 0, $4072 = 0, $4073 = 0;
 var $4074 = 0, $4075 = 0, $4076 = 0, $4077 = 0, $4078 = 0, $4079 = 0, $408 = 0, $4080 = 0, $4081 = 0, $4082 = 0, $4083 = 0, $4084 = 0, $4085 = 0, $4086 = 0, $4087 = 0, $4088 = 0, $4089 = 0, $409 = 0, $4090 = 0, $4091 = 0;
 var $4092 = 0, $4093 = 0, $4094 = 0, $4095 = 0, $4096 = 0, $4097 = 0, $4098 = 0, $4099 = 0, $41 = 0, $410 = 0, $4100 = 0, $4101 = 0, $4102 = 0, $4103 = 0, $4104 = 0, $4105 = 0, $4106 = 0, $4107 = 0, $4108 = 0, $4109 = 0;
 var $411 = 0, $4110 = 0, $4111 = 0, $4112 = 0, $4113 = 0, $4114 = 0, $4115 = 0, $4116 = 0, $4117 = 0, $4118 = 0, $4119 = 0, $412 = 0, $4120 = 0, $4121 = 0, $4122 = 0, $4123 = 0, $4124 = 0, $4125 = 0, $4126 = 0, $4127 = 0;
 var $4128 = 0, $4129 = 0, $413 = 0, $4130 = 0, $4131 = 0, $4132 = 0, $4133 = 0, $4134 = 0, $4135 = 0, $4136 = 0, $4137 = 0, $4138 = 0, $4139 = 0, $414 = 0, $4140 = 0, $4141 = 0, $4142 = 0, $4143 = 0, $4144 = 0, $4145 = 0;
 var $4146 = 0, $4147 = 0, $4148 = 0, $4149 = 0, $415 = 0, $4150 = 0, $4151 = 0, $4152 = 0, $4153 = 0, $4154 = 0, $4155 = 0, $4156 = 0, $4157 = 0, $4158 = 0, $4159 = 0, $416 = 0, $4160 = 0, $4161 = 0, $4162 = 0, $4163 = 0;
 var $4164 = 0, $4165 = 0, $4166 = 0, $4167 = 0, $4168 = 0, $4169 = 0, $417 = 0, $4170 = 0, $4171 = 0, $4172 = 0, $4173 = 0, $4174 = 0, $4175 = 0, $4176 = 0, $4177 = 0, $4178 = 0, $4179 = 0, $418 = 0, $4180 = 0, $4181 = 0;
 var $4182 = 0, $4183 = 0, $4184 = 0, $4185 = 0, $4186 = 0, $4187 = 0, $4188 = 0, $4189 = 0, $419 = 0, $4190 = 0, $4191 = 0, $4192 = 0, $4193 = 0, $4194 = 0, $4195 = 0, $4196 = 0, $4197 = 0, $4198 = 0, $4199 = 0, $42 = 0;
 var $420 = 0, $4200 = 0, $4201 = 0, $4202 = 0, $4203 = 0, $4204 = 0, $4205 = 0, $4206 = 0, $4207 = 0, $4208 = 0, $4209 = 0, $421 = 0, $4210 = 0, $4211 = 0, $4212 = 0, $4213 = 0, $4214 = 0, $4215 = 0, $4216 = 0, $4217 = 0;
 var $4218 = 0, $4219 = 0, $422 = 0, $4220 = 0, $4221 = 0, $4222 = 0, $4223 = 0, $4224 = 0, $4225 = 0, $4226 = 0, $4227 = 0, $4228 = 0, $4229 = 0, $423 = 0, $4230 = 0, $4231 = 0, $4232 = 0, $4233 = 0, $4234 = 0, $4235 = 0;
 var $4236 = 0, $4237 = 0, $4238 = 0, $4239 = 0, $424 = 0, $4240 = 0, $4241 = 0, $4242 = 0, $4243 = 0, $4244 = 0, $4245 = 0, $4246 = 0, $4247 = 0, $4248 = 0, $4249 = 0, $425 = 0, $4250 = 0, $4251 = 0, $4252 = 0, $4253 = 0;
 var $4254 = 0, $4255 = 0, $4256 = 0, $4257 = 0, $4258 = 0, $4259 = 0, $426 = 0, $4260 = 0, $4261 = 0, $4262 = 0, $4263 = 0, $4264 = 0, $4265 = 0, $4266 = 0, $4267 = 0, $4268 = 0, $4269 = 0, $427 = 0, $4270 = 0, $4271 = 0;
 var $4272 = 0, $4273 = 0, $4274 = 0, $4275 = 0, $4276 = 0, $4277 = 0, $4278 = 0, $4279 = 0, $428 = 0, $4280 = 0, $4281 = 0, $4282 = 0, $4283 = 0, $4284 = 0, $4285 = 0, $4286 = 0, $4287 = 0, $4288 = 0, $4289 = 0, $429 = 0;
 var $4290 = 0, $4291 = 0, $4292 = 0, $4293 = 0, $4294 = 0, $4295 = 0, $4296 = 0, $4297 = 0, $4298 = 0, $4299 = 0, $43 = 0, $430 = 0, $4300 = 0, $4301 = 0, $4302 = 0, $4303 = 0, $4304 = 0, $4305 = 0, $4306 = 0, $4307 = 0;
 var $4308 = 0, $4309 = 0, $431 = 0, $4310 = 0, $4311 = 0, $4312 = 0, $4313 = 0, $4314 = 0, $4315 = 0, $4316 = 0, $4317 = 0, $4318 = 0, $4319 = 0, $432 = 0, $4320 = 0, $4321 = 0, $4322 = 0, $4323 = 0, $4324 = 0, $4325 = 0;
 var $4326 = 0, $4327 = 0, $4328 = 0, $4329 = 0, $433 = 0, $4330 = 0, $4331 = 0, $4332 = 0, $4333 = 0, $4334 = 0, $4335 = 0, $4336 = 0, $4337 = 0, $4338 = 0, $4339 = 0, $434 = 0, $4340 = 0, $4341 = 0, $4342 = 0, $4343 = 0;
 var $4344 = 0, $4345 = 0, $4346 = 0, $4347 = 0, $4348 = 0, $4349 = 0, $435 = 0, $4350 = 0, $4351 = 0, $4352 = 0, $4353 = 0, $4354 = 0, $4355 = 0, $4356 = 0, $4357 = 0, $4358 = 0, $4359 = 0, $436 = 0, $4360 = 0, $4361 = 0;
 var $4362 = 0, $4363 = 0, $4364 = 0, $4365 = 0, $4366 = 0, $4367 = 0, $4368 = 0, $4369 = 0, $437 = 0, $4370 = 0, $4371 = 0, $4372 = 0, $4373 = 0, $4374 = 0, $4375 = 0, $4376 = 0, $4377 = 0, $4378 = 0, $4379 = 0, $438 = 0;
 var $4380 = 0, $4381 = 0, $4382 = 0, $4383 = 0, $4384 = 0, $4385 = 0, $4386 = 0, $4387 = 0, $4388 = 0, $4389 = 0, $439 = 0, $4390 = 0, $4391 = 0, $4392 = 0, $4393 = 0, $4394 = 0, $4395 = 0, $4396 = 0, $4397 = 0, $4398 = 0;
 var $4399 = 0, $44 = 0, $440 = 0, $4400 = 0, $4401 = 0, $4402 = 0, $4403 = 0, $4404 = 0, $4405 = 0, $4406 = 0, $4407 = 0, $4408 = 0, $4409 = 0, $441 = 0, $4410 = 0, $4411 = 0, $4412 = 0, $4413 = 0, $4414 = 0, $4415 = 0;
 var $4416 = 0, $4417 = 0, $4418 = 0, $4419 = 0, $442 = 0, $4420 = 0, $4421 = 0, $4422 = 0, $4423 = 0, $4424 = 0, $4425 = 0, $4426 = 0, $4427 = 0, $4428 = 0, $4429 = 0, $443 = 0, $4430 = 0, $4431 = 0, $4432 = 0, $4433 = 0;
 var $4434 = 0, $4435 = 0, $4436 = 0, $4437 = 0, $4438 = 0, $4439 = 0, $444 = 0, $4440 = 0, $4441 = 0, $4442 = 0, $4443 = 0, $4444 = 0, $4445 = 0, $4446 = 0, $4447 = 0, $4448 = 0, $4449 = 0, $445 = 0, $4450 = 0, $4451 = 0;
 var $4452 = 0, $4453 = 0, $4454 = 0, $4455 = 0, $4456 = 0, $4457 = 0, $4458 = 0, $4459 = 0, $446 = 0, $4460 = 0, $4461 = 0, $4462 = 0, $4463 = 0, $4464 = 0, $4465 = 0, $4466 = 0, $4467 = 0, $4468 = 0, $4469 = 0, $447 = 0;
 var $4470 = 0, $4471 = 0, $4472 = 0, $4473 = 0, $4474 = 0, $4475 = 0, $4476 = 0, $4477 = 0, $4478 = 0, $4479 = 0, $448 = 0, $4480 = 0, $4481 = 0, $4482 = 0, $4483 = 0, $4484 = 0, $4485 = 0, $4486 = 0, $4487 = 0, $4488 = 0;
 var $4489 = 0, $449 = 0, $4490 = 0, $4491 = 0, $4492 = 0, $4493 = 0, $4494 = 0, $4495 = 0, $4496 = 0, $4497 = 0, $4498 = 0, $4499 = 0, $45 = 0, $450 = 0, $4500 = 0, $4501 = 0, $4502 = 0, $4503 = 0, $4504 = 0, $4505 = 0;
 var $4506 = 0, $4507 = 0, $4508 = 0, $4509 = 0, $451 = 0, $4510 = 0, $4511 = 0, $4512 = 0, $4513 = 0, $4514 = 0, $4515 = 0, $4516 = 0, $4517 = 0, $4518 = 0, $4519 = 0, $452 = 0, $4520 = 0, $4521 = 0, $4522 = 0, $4523 = 0;
 var $4524 = 0, $4525 = 0, $4526 = 0, $4527 = 0, $4528 = 0, $4529 = 0, $453 = 0, $4530 = 0, $4531 = 0, $4532 = 0, $4533 = 0, $4534 = 0, $4535 = 0, $4536 = 0, $4537 = 0, $4538 = 0, $4539 = 0, $454 = 0, $4540 = 0, $4541 = 0;
 var $4542 = 0, $4543 = 0, $4544 = 0, $4545 = 0, $4546 = 0, $4547 = 0, $4548 = 0, $4549 = 0, $455 = 0, $4550 = 0, $4551 = 0, $4552 = 0, $4553 = 0, $4554 = 0, $4555 = 0, $4556 = 0, $4557 = 0, $4558 = 0, $4559 = 0, $456 = 0;
 var $4560 = 0, $4561 = 0, $4562 = 0, $4563 = 0, $4564 = 0, $4565 = 0, $4566 = 0, $4567 = 0, $4568 = 0, $4569 = 0, $457 = 0, $4570 = 0, $4571 = 0, $4572 = 0, $4573 = 0, $4574 = 0, $4575 = 0, $4576 = 0, $4577 = 0, $4578 = 0;
 var $4579 = 0, $458 = 0, $4580 = 0, $4581 = 0, $4582 = 0, $4583 = 0, $4584 = 0, $4585 = 0, $4586 = 0, $4587 = 0, $4588 = 0, $4589 = 0, $459 = 0, $4590 = 0, $4591 = 0, $4592 = 0, $4593 = 0, $4594 = 0, $4595 = 0, $4596 = 0;
 var $4597 = 0, $4598 = 0, $4599 = 0, $46 = 0, $460 = 0, $4600 = 0, $4601 = 0, $4602 = 0, $4603 = 0, $4604 = 0, $4605 = 0, $4606 = 0, $4607 = 0, $4608 = 0, $4609 = 0, $461 = 0, $4610 = 0, $4611 = 0, $4612 = 0, $4613 = 0;
 var $4614 = 0, $4615 = 0, $4616 = 0, $4617 = 0, $4618 = 0, $4619 = 0, $462 = 0, $4620 = 0, $4621 = 0, $4622 = 0, $4623 = 0, $4624 = 0, $4625 = 0, $4626 = 0, $4627 = 0, $4628 = 0, $4629 = 0, $463 = 0, $4630 = 0, $4631 = 0;
 var $4632 = 0, $4633 = 0, $4634 = 0, $4635 = 0, $4636 = 0, $4637 = 0, $4638 = 0, $4639 = 0, $464 = 0, $4640 = 0, $4641 = 0, $4642 = 0, $4643 = 0, $4644 = 0, $4645 = 0, $4646 = 0, $4647 = 0, $4648 = 0, $4649 = 0, $465 = 0;
 var $4650 = 0, $4651 = 0, $4652 = 0, $4653 = 0, $4654 = 0, $4655 = 0, $4656 = 0, $4657 = 0, $4658 = 0, $4659 = 0, $466 = 0, $4660 = 0, $4661 = 0, $4662 = 0, $4663 = 0, $4664 = 0, $4665 = 0, $4666 = 0, $4667 = 0, $4668 = 0;
 var $4669 = 0, $467 = 0, $4670 = 0, $4671 = 0, $4672 = 0, $4673 = 0, $4674 = 0, $4675 = 0, $4676 = 0, $4677 = 0, $4678 = 0, $4679 = 0, $468 = 0, $4680 = 0, $4681 = 0, $4682 = 0, $4683 = 0, $4684 = 0, $4685 = 0, $4686 = 0;
 var $4687 = 0, $4688 = 0, $4689 = 0, $469 = 0, $4690 = 0, $4691 = 0, $4692 = 0, $4693 = 0, $4694 = 0, $4695 = 0, $4696 = 0, $4697 = 0, $4698 = 0, $4699 = 0, $47 = 0, $470 = 0, $4700 = 0, $4701 = 0, $4702 = 0, $4703 = 0;
 var $4704 = 0, $4705 = 0, $4706 = 0, $4707 = 0, $4708 = 0, $4709 = 0, $471 = 0, $4710 = 0, $4711 = 0, $4712 = 0, $4713 = 0, $4714 = 0, $4715 = 0, $4716 = 0, $4717 = 0, $4718 = 0, $4719 = 0, $472 = 0, $4720 = 0, $4721 = 0;
 var $4722 = 0, $4723 = 0, $4724 = 0, $4725 = 0, $4726 = 0, $4727 = 0, $4728 = 0, $4729 = 0, $473 = 0, $4730 = 0, $4731 = 0, $4732 = 0, $4733 = 0, $4734 = 0, $4735 = 0, $4736 = 0, $4737 = 0, $4738 = 0, $4739 = 0, $474 = 0;
 var $4740 = 0, $4741 = 0, $4742 = 0, $4743 = 0, $4744 = 0, $4745 = 0, $4746 = 0, $4747 = 0, $4748 = 0, $4749 = 0, $475 = 0, $4750 = 0, $4751 = 0, $4752 = 0, $4753 = 0, $4754 = 0, $4755 = 0, $4756 = 0, $4757 = 0, $4758 = 0;
 var $4759 = 0, $476 = 0, $4760 = 0, $4761 = 0, $4762 = 0, $4763 = 0, $4764 = 0, $4765 = 0, $4766 = 0, $4767 = 0, $4768 = 0, $4769 = 0, $477 = 0, $4770 = 0, $4771 = 0, $4772 = 0, $4773 = 0, $4774 = 0, $4775 = 0, $4776 = 0;
 var $4777 = 0, $4778 = 0, $4779 = 0, $478 = 0, $4780 = 0, $4781 = 0, $4782 = 0, $4783 = 0, $4784 = 0, $4785 = 0, $4786 = 0, $4787 = 0, $4788 = 0, $4789 = 0, $479 = 0, $4790 = 0, $4791 = 0, $4792 = 0, $4793 = 0, $4794 = 0;
 var $4795 = 0, $4796 = 0, $4797 = 0, $4798 = 0, $4799 = 0, $48 = 0, $480 = 0, $4800 = 0, $4801 = 0, $4802 = 0, $4803 = 0, $4804 = 0, $4805 = 0, $4806 = 0, $4807 = 0, $4808 = 0, $4809 = 0, $481 = 0, $4810 = 0, $4811 = 0;
 var $4812 = 0, $4813 = 0, $4814 = 0, $4815 = 0, $4816 = 0, $4817 = 0, $4818 = 0, $4819 = 0, $482 = 0, $4820 = 0, $4821 = 0, $4822 = 0, $4823 = 0, $4824 = 0, $4825 = 0, $4826 = 0, $4827 = 0, $4828 = 0, $4829 = 0, $483 = 0;
 var $4830 = 0, $4831 = 0, $4832 = 0, $4833 = 0, $4834 = 0, $4835 = 0, $4836 = 0, $4837 = 0, $4838 = 0, $4839 = 0, $484 = 0, $4840 = 0, $4841 = 0, $4842 = 0, $4843 = 0, $4844 = 0, $4845 = 0, $4846 = 0, $4847 = 0, $4848 = 0;
 var $4849 = 0, $485 = 0, $4850 = 0, $4851 = 0, $4852 = 0, $4853 = 0, $4854 = 0, $4855 = 0, $4856 = 0, $4857 = 0, $4858 = 0, $4859 = 0, $486 = 0, $4860 = 0, $4861 = 0, $4862 = 0, $4863 = 0, $4864 = 0, $4865 = 0, $4866 = 0;
 var $4867 = 0, $4868 = 0, $4869 = 0, $487 = 0, $4870 = 0, $4871 = 0, $4872 = 0, $4873 = 0, $4874 = 0, $4875 = 0, $4876 = 0, $4877 = 0, $4878 = 0, $4879 = 0, $488 = 0, $4880 = 0, $4881 = 0, $4882 = 0, $4883 = 0, $4884 = 0;
 var $4885 = 0, $4886 = 0, $4887 = 0, $4888 = 0, $4889 = 0, $489 = 0, $4890 = 0, $4891 = 0, $4892 = 0, $4893 = 0, $4894 = 0, $4895 = 0, $4896 = 0, $4897 = 0, $4898 = 0, $4899 = 0, $49 = 0, $490 = 0, $4900 = 0, $4901 = 0;
 var $4902 = 0, $4903 = 0, $4904 = 0, $4905 = 0, $4906 = 0, $4907 = 0, $4908 = 0, $4909 = 0, $491 = 0, $4910 = 0, $4911 = 0, $4912 = 0, $4913 = 0, $4914 = 0, $4915 = 0, $4916 = 0, $4917 = 0, $4918 = 0, $4919 = 0, $492 = 0;
 var $4920 = 0, $4921 = 0, $4922 = 0, $4923 = 0, $4924 = 0, $4925 = 0, $4926 = 0, $4927 = 0, $4928 = 0, $4929 = 0, $493 = 0, $4930 = 0, $4931 = 0, $4932 = 0, $4933 = 0, $4934 = 0, $4935 = 0, $4936 = 0, $4937 = 0, $4938 = 0;
 var $4939 = 0, $494 = 0, $4940 = 0, $4941 = 0, $4942 = 0, $4943 = 0, $4944 = 0, $4945 = 0, $4946 = 0, $4947 = 0, $4948 = 0, $4949 = 0, $495 = 0, $4950 = 0, $4951 = 0, $4952 = 0, $4953 = 0, $4954 = 0, $4955 = 0, $4956 = 0;
 var $4957 = 0, $4958 = 0, $4959 = 0, $496 = 0, $4960 = 0, $4961 = 0, $4962 = 0, $4963 = 0, $4964 = 0, $4965 = 0, $4966 = 0, $4967 = 0, $4968 = 0, $4969 = 0, $497 = 0, $4970 = 0, $4971 = 0, $4972 = 0, $4973 = 0, $4974 = 0;
 var $4975 = 0, $4976 = 0, $4977 = 0, $4978 = 0, $4979 = 0, $498 = 0, $4980 = 0, $4981 = 0, $4982 = 0, $4983 = 0, $4984 = 0, $4985 = 0, $4986 = 0, $4987 = 0, $4988 = 0, $4989 = 0, $499 = 0, $4990 = 0, $4991 = 0, $4992 = 0;
 var $4993 = 0, $4994 = 0, $4995 = 0, $4996 = 0, $4997 = 0, $4998 = 0, $4999 = 0, $5 = 0, $50 = 0, $500 = 0, $5000 = 0, $5001 = 0, $5002 = 0, $5003 = 0, $5004 = 0, $5005 = 0, $5006 = 0, $5007 = 0, $5008 = 0, $5009 = 0;
 var $501 = 0, $5010 = 0, $5011 = 0, $5012 = 0, $5013 = 0, $5014 = 0, $5015 = 0, $5016 = 0, $5017 = 0, $5018 = 0, $5019 = 0, $502 = 0, $5020 = 0, $5021 = 0, $5022 = 0, $5023 = 0, $5024 = 0, $5025 = 0, $5026 = 0, $5027 = 0;
 var $5028 = 0, $5029 = 0, $503 = 0, $5030 = 0, $5031 = 0, $5032 = 0, $5033 = 0, $5034 = 0, $5035 = 0, $5036 = 0, $5037 = 0, $5038 = 0, $5039 = 0, $504 = 0, $5040 = 0, $5041 = 0, $5042 = 0, $5043 = 0, $5044 = 0, $5045 = 0;
 var $5046 = 0, $5047 = 0, $5048 = 0, $5049 = 0, $505 = 0, $5050 = 0, $5051 = 0, $5052 = 0, $5053 = 0, $5054 = 0, $5055 = 0, $5056 = 0, $5057 = 0, $5058 = 0, $5059 = 0, $506 = 0, $5060 = 0, $5061 = 0, $5062 = 0, $5063 = 0;
 var $5064 = 0, $5065 = 0, $5066 = 0, $5067 = 0, $5068 = 0, $5069 = 0, $507 = 0, $5070 = 0, $5071 = 0, $5072 = 0, $5073 = 0, $5074 = 0, $5075 = 0, $5076 = 0, $5077 = 0, $5078 = 0, $5079 = 0, $508 = 0, $5080 = 0, $5081 = 0;
 var $5082 = 0, $5083 = 0, $5084 = 0, $5085 = 0, $5086 = 0, $5087 = 0, $5088 = 0, $5089 = 0, $509 = 0, $5090 = 0, $5091 = 0, $5092 = 0, $5093 = 0, $5094 = 0, $5095 = 0, $5096 = 0, $5097 = 0, $5098 = 0, $5099 = 0, $51 = 0;
 var $510 = 0, $5100 = 0, $5101 = 0, $5102 = 0, $5103 = 0, $5104 = 0, $5105 = 0, $5106 = 0, $5107 = 0, $5108 = 0, $5109 = 0, $511 = 0, $5110 = 0, $5111 = 0, $5112 = 0, $5113 = 0, $5114 = 0, $5115 = 0, $5116 = 0, $5117 = 0;
 var $5118 = 0, $5119 = 0, $512 = 0, $5120 = 0, $5121 = 0, $5122 = 0, $5123 = 0, $5124 = 0, $5125 = 0, $5126 = 0, $5127 = 0, $5128 = 0, $5129 = 0, $513 = 0, $5130 = 0, $5131 = 0, $5132 = 0, $5133 = 0, $5134 = 0, $5135 = 0;
 var $5136 = 0, $5137 = 0, $5138 = 0, $5139 = 0, $514 = 0, $5140 = 0, $5141 = 0, $5142 = 0, $5143 = 0, $5144 = 0, $5145 = 0, $5146 = 0, $5147 = 0, $5148 = 0, $5149 = 0, $515 = 0, $5150 = 0, $5151 = 0, $5152 = 0, $5153 = 0;
 var $5154 = 0, $5155 = 0, $5156 = 0, $5157 = 0, $5158 = 0, $5159 = 0, $516 = 0, $5160 = 0, $5161 = 0, $5162 = 0, $5163 = 0, $5164 = 0, $5165 = 0, $5166 = 0, $5167 = 0, $5168 = 0, $5169 = 0, $517 = 0, $5170 = 0, $5171 = 0;
 var $5172 = 0, $5173 = 0, $5174 = 0, $5175 = 0, $5176 = 0, $5177 = 0, $5178 = 0, $5179 = 0, $518 = 0, $5180 = 0, $5181 = 0, $5182 = 0, $5183 = 0, $5184 = 0, $5185 = 0, $5186 = 0, $5187 = 0, $5188 = 0, $5189 = 0, $519 = 0;
 var $5190 = 0, $5191 = 0, $5192 = 0, $5193 = 0, $5194 = 0, $5195 = 0, $5196 = 0, $5197 = 0, $5198 = 0, $5199 = 0, $52 = 0, $520 = 0, $5200 = 0, $5201 = 0, $5202 = 0, $5203 = 0, $5204 = 0, $5205 = 0, $5206 = 0, $5207 = 0;
 var $5208 = 0, $5209 = 0, $521 = 0, $5210 = 0, $5211 = 0, $5212 = 0, $5213 = 0, $5214 = 0, $5215 = 0, $5216 = 0, $5217 = 0, $5218 = 0, $5219 = 0, $522 = 0, $5220 = 0, $5221 = 0, $5222 = 0, $5223 = 0, $5224 = 0, $5225 = 0;
 var $5226 = 0, $5227 = 0, $5228 = 0, $5229 = 0, $523 = 0, $5230 = 0, $5231 = 0, $5232 = 0, $5233 = 0, $5234 = 0, $5235 = 0, $5236 = 0, $5237 = 0, $5238 = 0, $5239 = 0, $524 = 0, $5240 = 0, $5241 = 0, $5242 = 0, $5243 = 0;
 var $5244 = 0, $5245 = 0, $5246 = 0, $5247 = 0, $5248 = 0, $5249 = 0, $525 = 0, $5250 = 0, $5251 = 0, $5252 = 0, $5253 = 0, $5254 = 0, $5255 = 0, $5256 = 0, $5257 = 0, $5258 = 0, $5259 = 0, $526 = 0, $5260 = 0, $5261 = 0;
 var $5262 = 0, $5263 = 0, $5264 = 0, $5265 = 0, $5266 = 0, $5267 = 0, $5268 = 0, $5269 = 0, $527 = 0, $5270 = 0, $5271 = 0, $5272 = 0, $5273 = 0, $5274 = 0, $5275 = 0, $5276 = 0, $5277 = 0, $5278 = 0, $5279 = 0, $528 = 0;
 var $5280 = 0, $5281 = 0, $5282 = 0, $5283 = 0, $5284 = 0, $5285 = 0, $5286 = 0, $5287 = 0, $5288 = 0, $5289 = 0, $529 = 0, $5290 = 0, $5291 = 0, $5292 = 0, $5293 = 0, $5294 = 0, $5295 = 0, $5296 = 0, $5297 = 0, $5298 = 0;
 var $5299 = 0, $53 = 0, $530 = 0, $5300 = 0, $5301 = 0, $5302 = 0, $5303 = 0, $5304 = 0, $5305 = 0, $5306 = 0, $5307 = 0, $5308 = 0, $5309 = 0, $531 = 0, $5310 = 0, $5311 = 0, $5312 = 0, $5313 = 0, $5314 = 0, $5315 = 0;
 var $5316 = 0, $5317 = 0, $5318 = 0, $5319 = 0, $532 = 0, $5320 = 0, $5321 = 0, $5322 = 0, $5323 = 0, $5324 = 0, $5325 = 0, $5326 = 0, $5327 = 0, $5328 = 0, $5329 = 0, $533 = 0, $5330 = 0, $5331 = 0, $5332 = 0, $5333 = 0;
 var $5334 = 0, $5335 = 0, $5336 = 0, $5337 = 0, $5338 = 0, $5339 = 0, $534 = 0, $5340 = 0, $5341 = 0, $5342 = 0, $5343 = 0, $5344 = 0, $5345 = 0, $5346 = 0, $5347 = 0, $5348 = 0, $5349 = 0, $535 = 0, $5350 = 0, $5351 = 0;
 var $5352 = 0, $5353 = 0, $5354 = 0, $5355 = 0, $5356 = 0, $5357 = 0, $5358 = 0, $5359 = 0, $536 = 0, $5360 = 0, $5361 = 0, $5362 = 0, $5363 = 0, $5364 = 0, $5365 = 0, $5366 = 0, $5367 = 0, $5368 = 0, $5369 = 0, $537 = 0;
 var $5370 = 0, $5371 = 0, $5372 = 0, $5373 = 0, $5374 = 0, $5375 = 0, $5376 = 0, $5377 = 0, $5378 = 0, $5379 = 0, $538 = 0, $5380 = 0, $5381 = 0, $5382 = 0, $5383 = 0, $5384 = 0, $5385 = 0, $5386 = 0, $5387 = 0, $5388 = 0;
 var $5389 = 0, $539 = 0, $5390 = 0, $5391 = 0, $5392 = 0, $5393 = 0, $5394 = 0, $5395 = 0, $5396 = 0, $5397 = 0, $5398 = 0, $5399 = 0, $54 = 0, $540 = 0, $5400 = 0, $5401 = 0, $5402 = 0, $5403 = 0, $5404 = 0, $5405 = 0;
 var $5406 = 0, $5407 = 0, $5408 = 0, $5409 = 0, $541 = 0, $5410 = 0, $5411 = 0, $5412 = 0, $5413 = 0, $5414 = 0, $5415 = 0, $5416 = 0, $5417 = 0, $5418 = 0, $5419 = 0, $542 = 0, $5420 = 0, $5421 = 0, $5422 = 0, $5423 = 0;
 var $5424 = 0, $5425 = 0, $5426 = 0, $5427 = 0, $5428 = 0, $5429 = 0, $543 = 0, $5430 = 0, $5431 = 0, $5432 = 0, $5433 = 0, $5434 = 0, $5435 = 0, $5436 = 0, $5437 = 0, $5438 = 0, $5439 = 0, $544 = 0, $5440 = 0, $5441 = 0;
 var $5442 = 0, $5443 = 0, $5444 = 0, $5445 = 0, $5446 = 0, $5447 = 0, $5448 = 0, $5449 = 0, $545 = 0, $5450 = 0, $5451 = 0, $5452 = 0, $5453 = 0, $5454 = 0, $5455 = 0, $5456 = 0, $5457 = 0, $5458 = 0, $5459 = 0, $546 = 0;
 var $5460 = 0, $5461 = 0, $5462 = 0, $5463 = 0, $5464 = 0, $5465 = 0, $5466 = 0, $5467 = 0, $5468 = 0, $5469 = 0, $547 = 0, $5470 = 0, $5471 = 0, $5472 = 0, $5473 = 0, $5474 = 0, $5475 = 0, $5476 = 0, $5477 = 0, $5478 = 0;
 var $5479 = 0, $548 = 0, $5480 = 0, $5481 = 0, $5482 = 0, $5483 = 0, $5484 = 0, $5485 = 0, $5486 = 0, $5487 = 0, $5488 = 0, $5489 = 0, $549 = 0, $5490 = 0, $5491 = 0, $5492 = 0, $5493 = 0, $5494 = 0, $5495 = 0, $5496 = 0;
 var $5497 = 0, $5498 = 0, $5499 = 0, $55 = 0, $550 = 0, $5500 = 0, $5501 = 0, $5502 = 0, $5503 = 0, $5504 = 0, $5505 = 0, $5506 = 0, $5507 = 0, $5508 = 0, $5509 = 0, $551 = 0, $5510 = 0, $5511 = 0, $5512 = 0, $5513 = 0;
 var $5514 = 0, $5515 = 0, $5516 = 0, $5517 = 0, $5518 = 0, $5519 = 0, $552 = 0, $5520 = 0, $5521 = 0, $5522 = 0, $5523 = 0, $5524 = 0, $5525 = 0, $5526 = 0, $5527 = 0, $5528 = 0, $5529 = 0, $553 = 0, $5530 = 0, $5531 = 0;
 var $5532 = 0, $5533 = 0, $5534 = 0, $5535 = 0, $5536 = 0, $5537 = 0, $5538 = 0, $5539 = 0, $554 = 0, $5540 = 0, $5541 = 0, $5542 = 0, $5543 = 0, $5544 = 0, $5545 = 0, $5546 = 0, $5547 = 0, $5548 = 0, $5549 = 0, $555 = 0;
 var $5550 = 0, $5551 = 0, $5552 = 0, $5553 = 0, $5554 = 0, $5555 = 0, $5556 = 0, $5557 = 0, $5558 = 0, $5559 = 0, $556 = 0, $5560 = 0, $5561 = 0, $5562 = 0, $5563 = 0, $5564 = 0, $5565 = 0, $5566 = 0, $5567 = 0, $5568 = 0;
 var $5569 = 0, $557 = 0, $5570 = 0, $5571 = 0, $5572 = 0, $5573 = 0, $5574 = 0, $5575 = 0, $5576 = 0, $5577 = 0, $5578 = 0, $5579 = 0, $558 = 0, $5580 = 0, $5581 = 0, $5582 = 0, $5583 = 0, $5584 = 0, $5585 = 0, $5586 = 0;
 var $5587 = 0, $5588 = 0, $5589 = 0, $559 = 0, $5590 = 0, $5591 = 0, $5592 = 0, $5593 = 0, $5594 = 0, $5595 = 0, $5596 = 0, $5597 = 0, $5598 = 0, $5599 = 0, $56 = 0, $560 = 0, $5600 = 0, $5601 = 0, $5602 = 0, $5603 = 0;
 var $5604 = 0, $5605 = 0, $5606 = 0, $5607 = 0, $5608 = 0, $5609 = 0, $561 = 0, $5610 = 0, $5611 = 0, $5612 = 0, $5613 = 0, $5614 = 0, $5615 = 0, $5616 = 0, $5617 = 0, $5618 = 0, $5619 = 0, $562 = 0, $5620 = 0, $5621 = 0;
 var $5622 = 0, $5623 = 0, $5624 = 0, $5625 = 0, $5626 = 0, $5627 = 0, $5628 = 0, $5629 = 0, $563 = 0, $5630 = 0, $5631 = 0, $5632 = 0, $5633 = 0, $5634 = 0, $5635 = 0, $5636 = 0, $5637 = 0, $5638 = 0, $5639 = 0, $564 = 0;
 var $5640 = 0, $5641 = 0, $5642 = 0, $5643 = 0, $5644 = 0, $5645 = 0, $5646 = 0, $5647 = 0, $5648 = 0, $5649 = 0, $565 = 0, $5650 = 0, $5651 = 0, $5652 = 0, $5653 = 0, $5654 = 0, $5655 = 0, $5656 = 0, $5657 = 0, $5658 = 0;
 var $5659 = 0, $566 = 0, $5660 = 0, $5661 = 0, $5662 = 0, $5663 = 0, $5664 = 0, $5665 = 0, $5666 = 0, $5667 = 0, $5668 = 0, $5669 = 0, $567 = 0, $5670 = 0, $5671 = 0, $5672 = 0, $5673 = 0, $5674 = 0, $5675 = 0, $5676 = 0;
 var $5677 = 0, $5678 = 0, $5679 = 0, $568 = 0, $5680 = 0, $5681 = 0, $5682 = 0, $5683 = 0, $5684 = 0, $5685 = 0, $5686 = 0, $5687 = 0, $5688 = 0, $5689 = 0, $569 = 0, $5690 = 0, $5691 = 0, $5692 = 0, $5693 = 0, $5694 = 0;
 var $5695 = 0, $5696 = 0, $5697 = 0, $5698 = 0, $5699 = 0, $57 = 0, $570 = 0, $5700 = 0, $5701 = 0, $5702 = 0, $5703 = 0, $5704 = 0, $5705 = 0, $5706 = 0, $5707 = 0, $5708 = 0, $5709 = 0, $571 = 0, $5710 = 0, $5711 = 0;
 var $5712 = 0, $5713 = 0, $5714 = 0, $5715 = 0, $5716 = 0, $5717 = 0, $5718 = 0, $5719 = 0, $572 = 0, $5720 = 0, $5721 = 0, $5722 = 0, $5723 = 0, $5724 = 0, $5725 = 0, $5726 = 0, $5727 = 0, $5728 = 0, $5729 = 0, $573 = 0;
 var $5730 = 0, $5731 = 0, $5732 = 0, $5733 = 0, $5734 = 0, $5735 = 0, $5736 = 0, $5737 = 0, $5738 = 0, $5739 = 0, $574 = 0, $5740 = 0, $5741 = 0, $5742 = 0, $5743 = 0, $5744 = 0, $5745 = 0, $5746 = 0, $5747 = 0, $5748 = 0;
 var $5749 = 0, $575 = 0, $5750 = 0, $5751 = 0, $5752 = 0, $5753 = 0, $5754 = 0, $5755 = 0, $5756 = 0, $5757 = 0, $5758 = 0, $5759 = 0, $576 = 0, $5760 = 0, $5761 = 0, $5762 = 0, $5763 = 0, $5764 = 0, $5765 = 0, $5766 = 0;
 var $5767 = 0, $5768 = 0, $5769 = 0, $577 = 0, $5770 = 0, $5771 = 0, $5772 = 0, $5773 = 0, $5774 = 0, $5775 = 0, $5776 = 0, $5777 = 0, $5778 = 0, $5779 = 0, $578 = 0, $5780 = 0, $5781 = 0, $5782 = 0, $5783 = 0, $5784 = 0;
 var $5785 = 0, $5786 = 0, $5787 = 0, $5788 = 0, $5789 = 0, $579 = 0, $5790 = 0, $5791 = 0, $5792 = 0, $5793 = 0, $5794 = 0, $5795 = 0, $5796 = 0, $5797 = 0, $5798 = 0, $5799 = 0, $58 = 0, $580 = 0, $5800 = 0, $5801 = 0;
 var $5802 = 0, $5803 = 0, $5804 = 0, $5805 = 0, $5806 = 0, $5807 = 0, $5808 = 0, $5809 = 0, $581 = 0, $5810 = 0, $5811 = 0, $5812 = 0, $5813 = 0, $5814 = 0, $5815 = 0, $5816 = 0, $5817 = 0, $5818 = 0, $5819 = 0, $582 = 0;
 var $5820 = 0, $5821 = 0, $5822 = 0, $5823 = 0, $5824 = 0, $5825 = 0, $5826 = 0, $5827 = 0, $5828 = 0, $5829 = 0, $583 = 0, $5830 = 0, $5831 = 0, $5832 = 0, $5833 = 0, $5834 = 0, $5835 = 0, $5836 = 0, $5837 = 0, $5838 = 0;
 var $5839 = 0, $584 = 0, $5840 = 0, $5841 = 0, $5842 = 0, $5843 = 0, $5844 = 0, $5845 = 0, $5846 = 0, $5847 = 0, $5848 = 0, $5849 = 0, $585 = 0, $5850 = 0, $5851 = 0, $5852 = 0, $5853 = 0, $5854 = 0, $5855 = 0, $5856 = 0;
 var $5857 = 0, $5858 = 0, $5859 = 0, $586 = 0, $5860 = 0, $5861 = 0, $5862 = 0, $5863 = 0, $5864 = 0, $5865 = 0, $5866 = 0, $5867 = 0, $5868 = 0, $5869 = 0, $587 = 0, $5870 = 0, $5871 = 0, $5872 = 0, $5873 = 0, $5874 = 0;
 var $5875 = 0, $5876 = 0, $5877 = 0, $5878 = 0, $5879 = 0, $588 = 0, $5880 = 0, $5881 = 0, $5882 = 0, $5883 = 0, $5884 = 0, $5885 = 0, $5886 = 0, $5887 = 0, $5888 = 0, $5889 = 0, $589 = 0, $5890 = 0, $5891 = 0, $5892 = 0;
 var $5893 = 0, $5894 = 0, $5895 = 0, $5896 = 0, $5897 = 0, $5898 = 0, $5899 = 0, $59 = 0, $590 = 0, $5900 = 0, $5901 = 0, $5902 = 0, $5903 = 0, $5904 = 0, $5905 = 0, $5906 = 0, $5907 = 0, $5908 = 0, $5909 = 0, $591 = 0;
 var $5910 = 0, $5911 = 0, $5912 = 0, $5913 = 0, $5914 = 0, $5915 = 0, $5916 = 0, $5917 = 0, $5918 = 0, $5919 = 0, $592 = 0, $5920 = 0, $5921 = 0, $5922 = 0, $5923 = 0, $5924 = 0, $5925 = 0, $5926 = 0, $5927 = 0, $5928 = 0;
 var $5929 = 0, $593 = 0, $5930 = 0, $5931 = 0, $5932 = 0, $5933 = 0, $5934 = 0, $5935 = 0, $5936 = 0, $5937 = 0, $5938 = 0, $5939 = 0, $594 = 0, $5940 = 0, $5941 = 0, $5942 = 0, $5943 = 0, $5944 = 0, $5945 = 0, $5946 = 0;
 var $5947 = 0, $5948 = 0, $5949 = 0, $595 = 0, $5950 = 0, $5951 = 0, $5952 = 0, $5953 = 0, $5954 = 0, $5955 = 0, $5956 = 0, $5957 = 0, $5958 = 0, $5959 = 0, $596 = 0, $5960 = 0, $5961 = 0, $5962 = 0, $5963 = 0, $5964 = 0;
 var $5965 = 0, $5966 = 0, $5967 = 0, $5968 = 0, $5969 = 0, $597 = 0, $5970 = 0, $5971 = 0, $5972 = 0, $5973 = 0, $5974 = 0, $5975 = 0, $5976 = 0, $5977 = 0, $5978 = 0, $5979 = 0, $598 = 0, $5980 = 0, $5981 = 0, $5982 = 0;
 var $5983 = 0, $5984 = 0, $5985 = 0, $5986 = 0, $5987 = 0, $5988 = 0, $5989 = 0, $599 = 0, $5990 = 0, $5991 = 0, $5992 = 0, $5993 = 0, $5994 = 0, $5995 = 0, $5996 = 0, $5997 = 0, $5998 = 0, $5999 = 0, $6 = 0, $60 = 0;
 var $600 = 0, $6000 = 0, $6001 = 0, $6002 = 0, $6003 = 0, $6004 = 0, $6005 = 0, $6006 = 0, $6007 = 0, $6008 = 0, $6009 = 0, $601 = 0, $6010 = 0, $6011 = 0, $6012 = 0, $6013 = 0, $6014 = 0, $6015 = 0, $6016 = 0, $6017 = 0;
 var $6018 = 0, $6019 = 0, $602 = 0, $6020 = 0, $6021 = 0, $6022 = 0, $6023 = 0, $6024 = 0, $6025 = 0, $6026 = 0, $6027 = 0, $6028 = 0, $6029 = 0, $603 = 0, $6030 = 0, $6031 = 0, $6032 = 0, $6033 = 0, $6034 = 0, $6035 = 0;
 var $6036 = 0, $6037 = 0, $6038 = 0, $6039 = 0, $604 = 0, $6040 = 0, $6041 = 0, $6042 = 0, $6043 = 0, $6044 = 0, $6045 = 0, $6046 = 0, $6047 = 0, $6048 = 0, $6049 = 0, $605 = 0, $6050 = 0, $6051 = 0, $6052 = 0, $6053 = 0;
 var $6054 = 0, $6055 = 0, $6056 = 0, $6057 = 0, $6058 = 0, $6059 = 0, $606 = 0, $6060 = 0, $6061 = 0, $6062 = 0, $6063 = 0, $6064 = 0, $6065 = 0, $6066 = 0, $6067 = 0, $6068 = 0, $6069 = 0, $607 = 0, $6070 = 0, $6071 = 0;
 var $6072 = 0, $6073 = 0, $6074 = 0, $6075 = 0, $6076 = 0, $6077 = 0, $6078 = 0, $6079 = 0, $608 = 0, $6080 = 0, $6081 = 0, $6082 = 0, $6083 = 0, $6084 = 0, $6085 = 0, $6086 = 0, $6087 = 0, $6088 = 0, $6089 = 0, $609 = 0;
 var $6090 = 0, $6091 = 0, $6092 = 0, $6093 = 0, $6094 = 0, $6095 = 0, $6096 = 0, $6097 = 0, $6098 = 0, $6099 = 0, $61 = 0, $610 = 0, $6100 = 0, $6101 = 0, $6102 = 0, $6103 = 0, $6104 = 0, $6105 = 0, $6106 = 0, $6107 = 0;
 var $6108 = 0, $6109 = 0, $611 = 0, $6110 = 0, $6111 = 0, $6112 = 0, $6113 = 0, $6114 = 0, $6115 = 0, $6116 = 0, $6117 = 0, $6118 = 0, $6119 = 0, $612 = 0, $6120 = 0, $6121 = 0, $6122 = 0, $6123 = 0, $6124 = 0, $6125 = 0;
 var $6126 = 0, $6127 = 0, $6128 = 0, $6129 = 0, $613 = 0, $6130 = 0, $6131 = 0, $6132 = 0, $6133 = 0, $6134 = 0, $6135 = 0, $6136 = 0, $6137 = 0, $6138 = 0, $6139 = 0, $614 = 0, $6140 = 0, $6141 = 0, $6142 = 0, $6143 = 0;
 var $6144 = 0, $6145 = 0, $6146 = 0, $6147 = 0, $6148 = 0, $6149 = 0, $615 = 0, $6150 = 0, $6151 = 0, $6152 = 0, $6153 = 0, $6154 = 0, $6155 = 0, $6156 = 0, $6157 = 0, $6158 = 0, $6159 = 0, $616 = 0, $6160 = 0, $6161 = 0;
 var $6162 = 0, $6163 = 0, $6164 = 0, $6165 = 0, $6166 = 0, $6167 = 0, $6168 = 0, $6169 = 0, $617 = 0, $6170 = 0, $6171 = 0, $6172 = 0, $6173 = 0, $6174 = 0, $6175 = 0, $6176 = 0, $6177 = 0, $6178 = 0, $6179 = 0, $618 = 0;
 var $6180 = 0, $6181 = 0, $6182 = 0, $6183 = 0, $6184 = 0, $6185 = 0, $6186 = 0, $6187 = 0, $6188 = 0, $6189 = 0, $619 = 0, $6190 = 0, $6191 = 0, $6192 = 0, $6193 = 0, $6194 = 0, $6195 = 0, $6196 = 0, $6197 = 0, $6198 = 0;
 var $6199 = 0, $62 = 0, $620 = 0, $6200 = 0, $6201 = 0, $6202 = 0, $6203 = 0, $6204 = 0, $6205 = 0, $6206 = 0, $6207 = 0, $6208 = 0, $6209 = 0, $621 = 0, $6210 = 0, $6211 = 0, $6212 = 0, $6213 = 0, $6214 = 0, $6215 = 0;
 var $6216 = 0, $6217 = 0, $6218 = 0, $6219 = 0, $622 = 0, $6220 = 0, $6221 = 0, $6222 = 0, $6223 = 0, $6224 = 0, $6225 = 0, $6226 = 0, $6227 = 0, $6228 = 0, $6229 = 0, $623 = 0, $6230 = 0, $6231 = 0, $6232 = 0, $6233 = 0;
 var $6234 = 0, $6235 = 0, $6236 = 0, $6237 = 0, $6238 = 0, $6239 = 0, $624 = 0, $6240 = 0, $6241 = 0, $6242 = 0, $6243 = 0, $6244 = 0, $6245 = 0, $6246 = 0, $6247 = 0, $6248 = 0, $6249 = 0, $625 = 0, $6250 = 0, $6251 = 0;
 var $6252 = 0, $6253 = 0, $6254 = 0, $6255 = 0, $6256 = 0, $6257 = 0, $6258 = 0, $6259 = 0, $626 = 0, $6260 = 0, $6261 = 0, $6262 = 0, $6263 = 0, $6264 = 0, $6265 = 0, $6266 = 0, $6267 = 0, $6268 = 0, $6269 = 0, $627 = 0;
 var $6270 = 0, $6271 = 0, $6272 = 0, $6273 = 0, $6274 = 0, $6275 = 0, $6276 = 0, $6277 = 0, $6278 = 0, $6279 = 0, $628 = 0, $6280 = 0, $6281 = 0, $6282 = 0, $6283 = 0, $6284 = 0, $6285 = 0, $6286 = 0, $6287 = 0, $6288 = 0;
 var $6289 = 0, $629 = 0, $6290 = 0, $6291 = 0, $6292 = 0, $6293 = 0, $6294 = 0, $6295 = 0, $6296 = 0, $6297 = 0, $6298 = 0, $6299 = 0, $63 = 0, $630 = 0, $6300 = 0, $6301 = 0, $6302 = 0, $6303 = 0, $6304 = 0, $6305 = 0;
 var $6306 = 0, $6307 = 0, $6308 = 0, $6309 = 0, $631 = 0, $6310 = 0, $6311 = 0, $6312 = 0, $6313 = 0, $6314 = 0, $6315 = 0, $6316 = 0, $6317 = 0, $6318 = 0, $6319 = 0, $632 = 0, $6320 = 0, $6321 = 0, $6322 = 0, $6323 = 0;
 var $6324 = 0, $6325 = 0, $6326 = 0, $6327 = 0, $6328 = 0, $6329 = 0, $633 = 0, $6330 = 0, $6331 = 0, $6332 = 0, $6333 = 0, $6334 = 0, $6335 = 0, $6336 = 0, $6337 = 0, $6338 = 0, $6339 = 0, $634 = 0, $6340 = 0, $6341 = 0;
 var $6342 = 0, $6343 = 0, $6344 = 0, $6345 = 0, $6346 = 0, $6347 = 0, $6348 = 0, $6349 = 0, $635 = 0, $6350 = 0, $6351 = 0, $6352 = 0, $6353 = 0, $6354 = 0, $6355 = 0, $6356 = 0, $6357 = 0, $6358 = 0, $6359 = 0, $636 = 0;
 var $6360 = 0, $6361 = 0, $6362 = 0, $6363 = 0, $6364 = 0, $6365 = 0, $6366 = 0, $6367 = 0, $6368 = 0, $6369 = 0, $637 = 0, $6370 = 0, $6371 = 0, $6372 = 0, $6373 = 0, $6374 = 0, $6375 = 0, $6376 = 0, $6377 = 0, $6378 = 0;
 var $6379 = 0, $638 = 0, $6380 = 0, $6381 = 0, $6382 = 0, $6383 = 0, $6384 = 0, $6385 = 0, $6386 = 0, $6387 = 0, $6388 = 0, $6389 = 0, $639 = 0, $6390 = 0, $6391 = 0, $6392 = 0, $6393 = 0, $6394 = 0, $6395 = 0, $6396 = 0;
 var $6397 = 0, $6398 = 0, $6399 = 0, $64 = 0, $640 = 0, $6400 = 0, $6401 = 0, $6402 = 0, $6403 = 0, $6404 = 0, $6405 = 0, $6406 = 0, $6407 = 0, $6408 = 0, $6409 = 0, $641 = 0, $6410 = 0, $6411 = 0, $6412 = 0, $6413 = 0;
 var $6414 = 0, $6415 = 0, $6416 = 0, $6417 = 0, $6418 = 0, $6419 = 0, $642 = 0, $6420 = 0, $6421 = 0, $6422 = 0, $6423 = 0, $6424 = 0, $6425 = 0, $6426 = 0, $6427 = 0, $6428 = 0, $6429 = 0, $643 = 0, $6430 = 0, $6431 = 0;
 var $6432 = 0, $6433 = 0, $6434 = 0, $6435 = 0, $6436 = 0, $6437 = 0, $6438 = 0, $6439 = 0, $644 = 0, $6440 = 0, $6441 = 0, $6442 = 0, $6443 = 0, $6444 = 0, $6445 = 0, $6446 = 0, $6447 = 0, $6448 = 0, $6449 = 0, $645 = 0;
 var $6450 = 0, $6451 = 0, $6452 = 0, $6453 = 0, $6454 = 0, $6455 = 0, $6456 = 0, $6457 = 0, $6458 = 0, $6459 = 0, $646 = 0, $6460 = 0, $6461 = 0, $6462 = 0, $6463 = 0, $6464 = 0, $6465 = 0, $6466 = 0, $6467 = 0, $6468 = 0;
 var $6469 = 0, $647 = 0, $6470 = 0, $6471 = 0, $6472 = 0, $6473 = 0, $6474 = 0, $6475 = 0, $6476 = 0, $6477 = 0, $6478 = 0, $6479 = 0, $648 = 0, $6480 = 0, $6481 = 0, $6482 = 0, $6483 = 0, $6484 = 0, $6485 = 0, $6486 = 0;
 var $6487 = 0, $6488 = 0, $6489 = 0, $649 = 0, $6490 = 0, $6491 = 0, $6492 = 0, $6493 = 0, $6494 = 0, $6495 = 0, $6496 = 0, $6497 = 0, $6498 = 0, $6499 = 0, $65 = 0, $650 = 0, $6500 = 0, $6501 = 0, $6502 = 0, $6503 = 0;
 var $6504 = 0, $6505 = 0, $6506 = 0, $6507 = 0, $6508 = 0, $6509 = 0, $651 = 0, $6510 = 0, $6511 = 0, $6512 = 0, $6513 = 0, $6514 = 0, $6515 = 0, $6516 = 0, $6517 = 0, $6518 = 0, $6519 = 0, $652 = 0, $6520 = 0, $6521 = 0;
 var $6522 = 0, $6523 = 0, $6524 = 0, $6525 = 0, $6526 = 0, $6527 = 0, $6528 = 0, $6529 = 0, $653 = 0, $6530 = 0, $6531 = 0, $6532 = 0, $6533 = 0, $6534 = 0, $6535 = 0, $6536 = 0, $6537 = 0, $6538 = 0, $6539 = 0, $654 = 0;
 var $6540 = 0, $6541 = 0, $6542 = 0, $6543 = 0, $6544 = 0, $6545 = 0, $6546 = 0, $6547 = 0, $6548 = 0, $6549 = 0, $655 = 0, $6550 = 0, $6551 = 0, $6552 = 0, $6553 = 0, $6554 = 0, $6555 = 0, $6556 = 0, $6557 = 0, $6558 = 0;
 var $6559 = 0, $656 = 0, $6560 = 0, $6561 = 0, $6562 = 0, $6563 = 0, $6564 = 0, $6565 = 0, $6566 = 0, $6567 = 0, $6568 = 0, $6569 = 0, $657 = 0, $6570 = 0, $6571 = 0, $6572 = 0, $6573 = 0, $6574 = 0, $6575 = 0, $6576 = 0;
 var $6577 = 0, $6578 = 0, $6579 = 0, $658 = 0, $6580 = 0, $6581 = 0, $6582 = 0, $6583 = 0, $6584 = 0, $6585 = 0, $6586 = 0, $6587 = 0, $6588 = 0, $6589 = 0, $659 = 0, $6590 = 0, $6591 = 0, $6592 = 0, $6593 = 0, $6594 = 0;
 var $6595 = 0, $6596 = 0, $6597 = 0, $6598 = 0, $6599 = 0, $66 = 0, $660 = 0, $6600 = 0, $6601 = 0, $6602 = 0, $6603 = 0, $6604 = 0, $6605 = 0, $6606 = 0, $6607 = 0, $6608 = 0, $6609 = 0, $661 = 0, $6610 = 0, $6611 = 0;
 var $6612 = 0, $6613 = 0, $6614 = 0, $6615 = 0, $6616 = 0, $6617 = 0, $6618 = 0, $6619 = 0, $662 = 0, $6620 = 0, $6621 = 0, $6622 = 0, $6623 = 0, $6624 = 0, $6625 = 0, $6626 = 0, $6627 = 0, $6628 = 0, $6629 = 0, $663 = 0;
 var $6630 = 0, $6631 = 0, $6632 = 0, $6633 = 0, $6634 = 0, $6635 = 0, $6636 = 0, $6637 = 0, $6638 = 0, $6639 = 0, $664 = 0, $6640 = 0, $6641 = 0, $6642 = 0, $6643 = 0, $6644 = 0, $6645 = 0, $6646 = 0, $6647 = 0, $6648 = 0;
 var $6649 = 0, $665 = 0, $6650 = 0, $6651 = 0, $6652 = 0, $6653 = 0, $6654 = 0, $6655 = 0, $6656 = 0, $6657 = 0, $6658 = 0, $6659 = 0, $666 = 0, $6660 = 0, $6661 = 0, $6662 = 0, $6663 = 0, $6664 = 0, $6665 = 0, $6666 = 0;
 var $6667 = 0, $6668 = 0, $6669 = 0, $667 = 0, $6670 = 0, $6671 = 0, $6672 = 0, $6673 = 0, $6674 = 0, $6675 = 0, $6676 = 0, $6677 = 0, $6678 = 0, $6679 = 0, $668 = 0, $6680 = 0, $6681 = 0, $6682 = 0, $6683 = 0, $6684 = 0;
 var $6685 = 0, $6686 = 0, $6687 = 0, $6688 = 0, $6689 = 0, $669 = 0, $6690 = 0, $6691 = 0, $6692 = 0, $6693 = 0, $6694 = 0, $6695 = 0, $6696 = 0, $6697 = 0, $6698 = 0, $6699 = 0, $67 = 0, $670 = 0, $6700 = 0, $6701 = 0;
 var $6702 = 0, $6703 = 0, $6704 = 0, $6705 = 0, $6706 = 0, $6707 = 0, $6708 = 0, $6709 = 0, $671 = 0, $6710 = 0, $6711 = 0, $6712 = 0, $6713 = 0, $6714 = 0, $6715 = 0, $6716 = 0, $6717 = 0, $6718 = 0, $6719 = 0, $672 = 0;
 var $6720 = 0, $6721 = 0, $6722 = 0, $6723 = 0, $6724 = 0, $6725 = 0, $6726 = 0, $6727 = 0, $6728 = 0, $6729 = 0, $673 = 0, $6730 = 0, $6731 = 0, $6732 = 0, $6733 = 0, $6734 = 0, $6735 = 0, $6736 = 0, $6737 = 0, $6738 = 0;
 var $6739 = 0, $674 = 0, $6740 = 0, $6741 = 0, $6742 = 0, $6743 = 0, $6744 = 0, $6745 = 0, $6746 = 0, $6747 = 0, $6748 = 0, $6749 = 0, $675 = 0, $6750 = 0, $6751 = 0, $6752 = 0, $6753 = 0, $6754 = 0, $6755 = 0, $6756 = 0;
 var $6757 = 0, $6758 = 0, $6759 = 0, $676 = 0, $6760 = 0, $6761 = 0, $6762 = 0, $6763 = 0, $6764 = 0, $6765 = 0, $6766 = 0, $6767 = 0, $6768 = 0, $6769 = 0, $677 = 0, $6770 = 0, $6771 = 0, $6772 = 0, $6773 = 0, $6774 = 0;
 var $6775 = 0, $6776 = 0, $6777 = 0, $6778 = 0, $6779 = 0, $678 = 0, $6780 = 0, $6781 = 0, $6782 = 0, $6783 = 0, $6784 = 0, $6785 = 0, $6786 = 0, $6787 = 0, $6788 = 0, $6789 = 0, $679 = 0, $6790 = 0, $6791 = 0, $6792 = 0;
 var $6793 = 0, $6794 = 0, $6795 = 0, $6796 = 0, $6797 = 0, $6798 = 0, $6799 = 0, $68 = 0, $680 = 0, $6800 = 0, $6801 = 0, $6802 = 0, $6803 = 0, $6804 = 0, $6805 = 0, $6806 = 0, $6807 = 0, $6808 = 0, $6809 = 0, $681 = 0;
 var $6810 = 0, $6811 = 0, $6812 = 0, $6813 = 0, $6814 = 0, $6815 = 0, $6816 = 0, $6817 = 0, $6818 = 0, $6819 = 0, $682 = 0, $6820 = 0, $6821 = 0, $6822 = 0, $6823 = 0, $6824 = 0, $6825 = 0, $6826 = 0, $6827 = 0, $6828 = 0;
 var $6829 = 0, $683 = 0, $6830 = 0, $6831 = 0, $6832 = 0, $6833 = 0, $6834 = 0, $6835 = 0, $6836 = 0, $6837 = 0, $6838 = 0, $6839 = 0, $684 = 0, $6840 = 0, $6841 = 0, $6842 = 0, $6843 = 0, $6844 = 0, $6845 = 0, $6846 = 0;
 var $6847 = 0, $6848 = 0, $6849 = 0, $685 = 0, $6850 = 0, $6851 = 0, $6852 = 0, $6853 = 0, $6854 = 0, $6855 = 0, $6856 = 0, $6857 = 0, $6858 = 0, $6859 = 0, $686 = 0, $6860 = 0, $6861 = 0, $6862 = 0, $6863 = 0, $6864 = 0;
 var $6865 = 0, $6866 = 0, $6867 = 0, $6868 = 0, $6869 = 0, $687 = 0, $6870 = 0, $6871 = 0, $6872 = 0, $6873 = 0, $6874 = 0, $6875 = 0, $6876 = 0, $6877 = 0, $6878 = 0, $6879 = 0, $688 = 0, $6880 = 0, $6881 = 0, $6882 = 0;
 var $6883 = 0, $6884 = 0, $6885 = 0, $6886 = 0, $6887 = 0, $6888 = 0, $6889 = 0, $689 = 0, $6890 = 0, $6891 = 0, $6892 = 0, $6893 = 0, $6894 = 0, $6895 = 0, $6896 = 0, $6897 = 0, $6898 = 0, $6899 = 0, $69 = 0, $690 = 0;
 var $6900 = 0, $6901 = 0, $6902 = 0, $6903 = 0, $6904 = 0, $6905 = 0, $6906 = 0, $6907 = 0, $6908 = 0, $6909 = 0, $691 = 0, $6910 = 0, $6911 = 0, $6912 = 0, $6913 = 0, $6914 = 0, $6915 = 0, $6916 = 0, $6917 = 0, $6918 = 0;
 var $6919 = 0, $692 = 0, $6920 = 0, $6921 = 0, $6922 = 0, $6923 = 0, $6924 = 0, $6925 = 0, $6926 = 0, $6927 = 0, $6928 = 0, $6929 = 0, $693 = 0, $6930 = 0, $6931 = 0, $6932 = 0, $6933 = 0, $6934 = 0, $6935 = 0, $6936 = 0;
 var $6937 = 0, $6938 = 0, $6939 = 0, $694 = 0, $6940 = 0, $6941 = 0, $6942 = 0, $6943 = 0, $6944 = 0, $6945 = 0, $6946 = 0, $6947 = 0, $6948 = 0, $6949 = 0, $695 = 0, $6950 = 0, $6951 = 0, $6952 = 0, $6953 = 0, $6954 = 0;
 var $6955 = 0, $6956 = 0, $6957 = 0, $6958 = 0, $6959 = 0, $696 = 0, $6960 = 0, $6961 = 0, $6962 = 0, $6963 = 0, $6964 = 0, $6965 = 0, $6966 = 0, $6967 = 0, $6968 = 0, $6969 = 0, $697 = 0, $6970 = 0, $6971 = 0, $6972 = 0;
 var $6973 = 0, $6974 = 0, $6975 = 0, $6976 = 0, $6977 = 0, $6978 = 0, $6979 = 0, $698 = 0, $6980 = 0, $6981 = 0, $6982 = 0, $6983 = 0, $6984 = 0, $6985 = 0, $6986 = 0, $6987 = 0, $6988 = 0, $6989 = 0, $699 = 0, $6990 = 0;
 var $6991 = 0, $6992 = 0, $6993 = 0, $6994 = 0, $6995 = 0, $6996 = 0, $6997 = 0, $6998 = 0, $6999 = 0, $7 = 0, $70 = 0, $700 = 0, $7000 = 0, $7001 = 0, $7002 = 0, $7003 = 0, $7004 = 0, $7005 = 0, $7006 = 0, $7007 = 0;
 var $7008 = 0, $7009 = 0, $701 = 0, $7010 = 0, $7011 = 0, $7012 = 0, $7013 = 0, $7014 = 0, $7015 = 0, $7016 = 0, $7017 = 0, $7018 = 0, $7019 = 0, $702 = 0, $7020 = 0, $7021 = 0, $7022 = 0, $7023 = 0, $7024 = 0, $7025 = 0;
 var $7026 = 0, $7027 = 0, $7028 = 0, $7029 = 0, $703 = 0, $7030 = 0, $7031 = 0, $7032 = 0, $7033 = 0, $7034 = 0, $7035 = 0, $7036 = 0, $7037 = 0, $7038 = 0, $7039 = 0, $704 = 0, $7040 = 0, $7041 = 0, $7042 = 0, $7043 = 0;
 var $7044 = 0, $7045 = 0, $7046 = 0, $7047 = 0, $7048 = 0, $7049 = 0, $705 = 0, $7050 = 0, $7051 = 0, $7052 = 0, $7053 = 0, $7054 = 0, $7055 = 0, $7056 = 0, $7057 = 0, $7058 = 0, $7059 = 0, $706 = 0, $7060 = 0, $7061 = 0;
 var $7062 = 0, $7063 = 0, $7064 = 0, $7065 = 0, $7066 = 0, $7067 = 0, $7068 = 0, $7069 = 0, $707 = 0, $7070 = 0, $7071 = 0, $7072 = 0, $7073 = 0, $7074 = 0, $7075 = 0, $7076 = 0, $7077 = 0, $7078 = 0, $7079 = 0, $708 = 0;
 var $7080 = 0, $7081 = 0, $7082 = 0, $7083 = 0, $7084 = 0, $7085 = 0, $7086 = 0, $7087 = 0, $7088 = 0, $7089 = 0, $709 = 0, $7090 = 0, $7091 = 0, $7092 = 0, $7093 = 0, $7094 = 0, $7095 = 0, $7096 = 0, $7097 = 0, $7098 = 0;
 var $7099 = 0, $71 = 0, $710 = 0, $7100 = 0, $7101 = 0, $7102 = 0, $7103 = 0, $7104 = 0, $7105 = 0, $7106 = 0, $7107 = 0, $7108 = 0, $7109 = 0, $711 = 0, $7110 = 0, $7111 = 0, $7112 = 0, $7113 = 0, $7114 = 0, $7115 = 0;
 var $7116 = 0, $7117 = 0, $7118 = 0, $7119 = 0, $712 = 0, $7120 = 0, $7121 = 0, $7122 = 0, $7123 = 0, $7124 = 0, $7125 = 0, $7126 = 0, $7127 = 0, $7128 = 0, $7129 = 0, $713 = 0, $7130 = 0, $7131 = 0, $7132 = 0, $7133 = 0;
 var $7134 = 0, $7135 = 0, $7136 = 0, $7137 = 0, $7138 = 0, $7139 = 0, $714 = 0, $7140 = 0, $7141 = 0, $7142 = 0, $7143 = 0, $7144 = 0, $7145 = 0, $7146 = 0, $7147 = 0, $7148 = 0, $7149 = 0, $715 = 0, $7150 = 0, $7151 = 0;
 var $7152 = 0, $7153 = 0, $7154 = 0, $7155 = 0, $7156 = 0, $7157 = 0, $7158 = 0, $7159 = 0, $716 = 0, $7160 = 0, $7161 = 0, $7162 = 0, $7163 = 0, $7164 = 0, $7165 = 0, $7166 = 0, $7167 = 0, $7168 = 0, $7169 = 0, $717 = 0;
 var $7170 = 0, $7171 = 0, $7172 = 0, $7173 = 0, $7174 = 0, $7175 = 0, $7176 = 0, $7177 = 0, $7178 = 0, $7179 = 0, $718 = 0, $7180 = 0, $7181 = 0, $7182 = 0, $7183 = 0, $7184 = 0, $7185 = 0, $7186 = 0, $7187 = 0, $7188 = 0;
 var $7189 = 0, $719 = 0, $7190 = 0, $7191 = 0, $7192 = 0, $7193 = 0, $7194 = 0, $7195 = 0, $7196 = 0, $7197 = 0, $7198 = 0, $7199 = 0, $72 = 0, $720 = 0, $7200 = 0, $7201 = 0, $7202 = 0, $7203 = 0, $7204 = 0, $7205 = 0;
 var $7206 = 0, $7207 = 0, $7208 = 0, $7209 = 0, $721 = 0, $7210 = 0, $7211 = 0, $7212 = 0, $7213 = 0, $7214 = 0, $7215 = 0, $7216 = 0, $7217 = 0, $7218 = 0, $7219 = 0, $722 = 0, $7220 = 0, $7221 = 0, $7222 = 0, $7223 = 0;
 var $7224 = 0, $7225 = 0, $7226 = 0, $7227 = 0, $7228 = 0, $7229 = 0, $723 = 0, $7230 = 0, $7231 = 0, $7232 = 0, $7233 = 0, $7234 = 0, $7235 = 0, $7236 = 0, $7237 = 0, $7238 = 0, $7239 = 0, $724 = 0, $7240 = 0, $7241 = 0;
 var $7242 = 0, $7243 = 0, $7244 = 0, $7245 = 0, $7246 = 0, $7247 = 0, $7248 = 0, $7249 = 0, $725 = 0, $7250 = 0, $7251 = 0, $7252 = 0, $7253 = 0, $7254 = 0, $7255 = 0, $7256 = 0, $7257 = 0, $7258 = 0, $7259 = 0, $726 = 0;
 var $7260 = 0, $7261 = 0, $7262 = 0, $7263 = 0, $7264 = 0, $7265 = 0, $7266 = 0, $7267 = 0, $7268 = 0, $7269 = 0, $727 = 0, $7270 = 0, $7271 = 0, $7272 = 0, $7273 = 0, $7274 = 0, $7275 = 0, $7276 = 0, $7277 = 0, $7278 = 0;
 var $7279 = 0, $728 = 0, $7280 = 0, $7281 = 0, $7282 = 0, $7283 = 0, $7284 = 0, $7285 = 0, $7286 = 0, $7287 = 0, $7288 = 0, $7289 = 0, $729 = 0, $7290 = 0, $7291 = 0, $7292 = 0, $7293 = 0, $7294 = 0, $7295 = 0, $7296 = 0;
 var $7297 = 0, $7298 = 0, $7299 = 0, $73 = 0, $730 = 0, $7300 = 0, $7301 = 0, $7302 = 0, $7303 = 0, $7304 = 0, $7305 = 0, $7306 = 0, $7307 = 0, $7308 = 0, $7309 = 0, $731 = 0, $7310 = 0, $7311 = 0, $7312 = 0, $7313 = 0;
 var $7314 = 0, $7315 = 0, $7316 = 0, $7317 = 0, $7318 = 0, $7319 = 0, $732 = 0, $7320 = 0, $7321 = 0, $7322 = 0, $7323 = 0, $7324 = 0, $7325 = 0, $7326 = 0, $7327 = 0, $7328 = 0, $7329 = 0, $733 = 0, $7330 = 0, $7331 = 0;
 var $7332 = 0, $7333 = 0, $7334 = 0, $7335 = 0, $7336 = 0, $7337 = 0, $7338 = 0, $7339 = 0, $734 = 0, $7340 = 0, $7341 = 0, $7342 = 0, $7343 = 0, $7344 = 0, $7345 = 0, $7346 = 0, $7347 = 0, $7348 = 0, $7349 = 0, $735 = 0;
 var $7350 = 0, $7351 = 0, $7352 = 0, $7353 = 0, $7354 = 0, $7355 = 0, $7356 = 0, $7357 = 0, $7358 = 0, $7359 = 0, $736 = 0, $7360 = 0, $7361 = 0, $7362 = 0, $7363 = 0, $7364 = 0, $7365 = 0, $7366 = 0, $7367 = 0, $7368 = 0;
 var $7369 = 0, $737 = 0, $7370 = 0, $7371 = 0, $7372 = 0, $7373 = 0, $7374 = 0, $7375 = 0, $7376 = 0, $7377 = 0, $7378 = 0, $7379 = 0, $738 = 0, $7380 = 0, $7381 = 0, $7382 = 0, $7383 = 0, $7384 = 0, $7385 = 0, $7386 = 0;
 var $7387 = 0, $7388 = 0, $7389 = 0, $739 = 0, $7390 = 0, $7391 = 0, $7392 = 0, $7393 = 0, $7394 = 0, $7395 = 0, $7396 = 0, $7397 = 0, $7398 = 0, $7399 = 0, $74 = 0, $740 = 0, $7400 = 0, $7401 = 0, $7402 = 0, $7403 = 0;
 var $7404 = 0, $7405 = 0, $7406 = 0, $7407 = 0, $7408 = 0, $7409 = 0, $741 = 0, $7410 = 0, $7411 = 0, $7412 = 0, $7413 = 0, $7414 = 0, $7415 = 0, $7416 = 0, $7417 = 0, $7418 = 0, $7419 = 0, $742 = 0, $7420 = 0, $7421 = 0;
 var $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0;
 var $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0;
 var $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0;
 var $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0;
 var $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0;
 var $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0;
 var $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0;
 var $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0;
 var $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0;
 var $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0;
 var $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0;
 var $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0;
 var $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0;
 var $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0;
 var $996 = 0, $997 = 0, $998 = 0, $999 = 0, $aMem = 0, $aOffset = 0, $aOp = 0, $aPermute = 0, $aRes = 0, $aRoot = 0, $aTempRec = 0, $affinity = 0, $alreadyExists = 0, $apArg = 0, $apArg126 = 0, $avail = 0, $bIntint = 0, $bRev = 0, $c = 0, $cAff = 0;
 var $cnt = 0, $cnt42 = 0, $db = 0, $desiredAutoCommit = 0, $eNew = 0, $eOld = 0, $encoding = 0, $eqOnly = 0, $exists = 0, $file_format = 0, $flags = 0, $flags1 = 0, $flags3 = 0, $flags80 = 0, $i = 0, $i111 = 0, $i12 = 0, $i124 = 0, $i129 = 0, $i15 = 0;
 var $i3 = 0, $i8 = 0, $i91 = 0, $i96 = 0, $iA = 0, $iA5 = 0, $iB = 0, $iB6 = 0, $iCompare = 0, $iCookie = 0, $iDb = 0, $iDb23 = 0, $iDb77 = 0, $iDb82 = 0, $iGen = 0, $iKey = 0, $iKey39 = 0, $iMeta = 0, $iMeta19 = 0, $iMoved = 0;
 var $iPrior = 0, $iQuery = 0, $iRollback = 0, $iSavepoint = 0, $iSet = 0, $idx = 0, $ii = 0, $ii32 = 0, $initData = 0, $isSchemaChange = 0, $isTransaction = 0, $isWriteLock = 0, $j = 0, $lastRowid = 0, $len = 0, $len16 = 0, $n = 0, $n1 = 0, $n2 = 0, $n55 = 0;
 var $n7 = 0, $n89 = 0, $nArg = 0, $nArg123 = 0, $nByte = 0, $nByte13 = 0, $nByte83 = 0, $nChange = 0, $nChange$ = 0, $nData = 0, $nEntry = 0, $nErr = 0, $nField = 0, $nField20 = 0, $nField31 = 0, $nHdr = 0, $nKeyCol = 0, $nMem = 0, $nName = 0, $nProgressLimit = 0;
 var $nRoot = 0, $nVarint = 0, $nVmStep = 0, $nZero = 0, $newMax = 0, $nullFlag = 0, $oc = 0, $offset = 0, $offset64 = 0, $op = 0, $op46 = 0, $opflags = 0, $or$cond = 0, $or$cond10 = 0, $or$cond12 = 0, $or$cond14 = 0, $or$cond16 = 0, $or$cond18 = 0, $or$cond21 = 0, $or$cond24 = 0;
 var $or$cond26 = 0, $or$cond28 = 0, $or$cond3 = 0, $or$cond31 = 0, $or$cond6 = 0, $or$cond8 = 0, $p1 = 0, $p1100 = 0, $p118 = 0, $p19 = 0, $p2 = 0, $p210 = 0, $p211 = 0, $p222 = 0, $pArgc = 0, $pBt = 0, $pBt128 = 0, $pBt98 = 0, $pBt99 = 0, $pC = 0;
 var $pC27 = 0, $pC30 = 0, $pC33 = 0, $pC36 = 0, $pC40 = 0, $pC45 = 0, $pC47 = 0, $pC50 = 0, $pC52 = 0, $pC53 = 0, $pC56 = 0, $pC58 = 0, $pC59 = 0, $pC62 = 0, $pC65 = 0, $pC67 = 0, $pC69 = 0, $pC73 = 0, $pC74 = 0, $pC78 = 0;
 var $pCaller = 0, $pColl = 0, $pCrsr = 0, $pCrsr17 = 0, $pCrsr37 = 0, $pCrsr54 = 0, $pCrsr60 = 0, $pCrsr63 = 0, $pCrsr70 = 0, $pCtx = 0, $pCtx4 = 0, $pCtx90 = 0, $pCtx92 = 0, $pCur = 0, $pCur103 = 0, $pCur109 = 0, $pCur115 = 0, $pCur119 = 0, $pCx = 0, $pCx26 = 0;
 var $pCx28 = 0, $pData = 0, $pData0 = 0, $pDb = 0, $pDb24 = 0, $pDb81 = 0, $pDest = 0, $pDest114 = 0, $pEnd = 0, $pFrame = 0, $pFrame44 = 0, $pFrame85 = 0, $pFrame87 = 0, $pFrame88 = 0, $pFree = 0, $pIdxKey = 0, $pIn = 0, $pIn1 = 0, $pIn2 = 0, $pIn3 = 0;
 var $pKey = 0, $pKeyInfo = 0, $pKeyInfo21 = 0, $pKeyInfo25 = 0, $pLast = 0, $pMem = 0, $pMem43 = 0, $pMem84 = 0, $pMem93 = 0, $pMem95 = 0, $pMem97 = 0, $pModule = 0, $pModule105 = 0, $pModule106 = 0, $pModule113 = 0, $pModule117 = 0, $pModule122 = 0, $pName = 0, $pNew = 0, $pOp = 0;
 var $pOut = 0, $pPager = 0, $pProgram = 0, $pQuery = 0, $pRec = 0, $pReg = 0, $pRt = 0, $pSavepoint = 0, $pTab = 0, $pTab49 = 0, $pTabCur = 0, $pTmp = 0, $pVCur = 0, $pVCur107 = 0, $pVTab = 0, $pVar = 0, $pVtab = 0, $pVtab104 = 0, $pVtab108 = 0, $pVtab112 = 0;
 var $pVtab116 = 0, $pVtab120 = 0, $pVtab121 = 0, $pX = 0, $pX127 = 0, $pcDest = 0, $pcx = 0, $pgno = 0, $pgno79 = 0, $pnErr = 0, $r = 0, $r35 = 0, $r72 = 0, $r76 = 0, $rA = 0, $rB = 0, $rc = 0, $res = 0, $res110 = 0, $res118 = 0;
 var $res2 = 0, $res29 = 0, $res34 = 0, $res38 = 0, $res41 = 0, $res51 = 0, $res61 = 0, $res64 = 0, $res66 = 0, $res71 = 0, $res75 = 0, $resetSchemaOnFault = 0, $rowid = 0, $rowid125 = 0, $sContext = 0, $sMem = 0, $sMem102 = 0, $seekResult = 0, $serial_type = 0, $t = 0;
 var $t86 = 0, $t94 = 0, $takeJump = 0, $type1 = 0, $type2 = 0, $uA = 0, $v = 0, $v1 = 0, $v2 = 0, $v57 = 0, $val = 0, $vararg_buffer = 0, $vararg_buffer32 = 0, $vararg_buffer36 = 0, $vararg_buffer39 = 0, $vararg_buffer44 = 0, $vararg_buffer47 = 0, $vararg_buffer49 = 0, $vararg_buffer52 = 0, $vararg_buffer54 = 0;
 var $vararg_buffer56 = 0, $vararg_buffer58 = 0, $vararg_buffer63 = 0, $vararg_buffer65 = 0, $vararg_buffer68 = 0, $vararg_buffer71 = 0, $vararg_buffer74 = 0, $vararg_buffer77 = 0, $vararg_buffer80 = 0, $vararg_buffer85 = 0, $vararg_buffer87 = 0, $vararg_buffer89 = 0, $vararg_ptr35 = 0, $vararg_ptr42 = 0, $vararg_ptr43 = 0, $vararg_ptr61 = 0, $vararg_ptr62 = 0, $vararg_ptr83 = 0, $vararg_ptr84 = 0, $vtabOnConflict = 0;
 var $wrFlag = 0, $x = 0, $x130 = 0, $x68 = 0, $z = 0, $z101 = 0, $z131 = 0, $zAffinity = 0, $zAffinity14 = 0, $zData = 0, $zDb = 0, $zDb48 = 0, $zEndHdr = 0, $zFilename = 0, $zHdr = 0, $zMaster = 0, $zName = 0, $zNewRecord = 0, $zSql = 0, $zTab = 0;
 var $zTrace = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 1872|0;
 $vararg_buffer89 = sp + 536|0;
 $vararg_buffer87 = sp + 528|0;
 $vararg_buffer85 = sp + 520|0;
 $vararg_buffer80 = sp + 504|0;
 $vararg_buffer77 = sp + 496|0;
 $vararg_buffer74 = sp + 488|0;
 $vararg_buffer71 = sp + 480|0;
 $vararg_buffer68 = sp + 472|0;
 $vararg_buffer65 = sp + 464|0;
 $vararg_buffer63 = sp + 456|0;
 $vararg_buffer58 = sp + 440|0;
 $vararg_buffer56 = sp + 432|0;
 $vararg_buffer54 = sp + 424|0;
 $vararg_buffer52 = sp + 416|0;
 $vararg_buffer49 = sp + 408|0;
 $vararg_buffer47 = sp + 400|0;
 $vararg_buffer44 = sp + 392|0;
 $vararg_buffer39 = sp + 376|0;
 $vararg_buffer36 = sp + 368|0;
 $vararg_buffer32 = sp + 360|0;
 $vararg_buffer = sp + 352|0;
 $0 = sp + 1656|0;
 $aOp = sp + 1652|0;
 $pOp = sp + 1648|0;
 $rc = sp + 1644|0;
 $db = sp + 1640|0;
 $resetSchemaOnFault = sp + 1862|0;
 $encoding = sp + 1861|0;
 $iCompare = sp + 1636|0;
 $nVmStep = sp + 1632|0;
 $nProgressLimit = sp + 1628|0;
 $aMem = sp + 1624|0;
 $pIn1 = sp + 1620|0;
 $pIn2 = sp + 1616|0;
 $pIn3 = sp + 1612|0;
 $pOut = sp + 1608|0;
 $aPermute = sp + 1604|0;
 $lastRowid = sp + 344|0;
 $iPrior = sp + 1600|0;
 $pCaller = sp + 1596|0;
 $pcDest = sp + 1592|0;
 $pFrame = sp + 1588|0;
 $pcx = sp + 1584|0;
 $cnt = sp + 1580|0;
 $nullFlag = sp + 1670|0;
 $pVar = sp + 1576|0;
 $n = sp + 1572|0;
 $p1 = sp + 1568|0;
 $p2 = sp + 1564|0;
 $n1 = sp + 1560|0;
 $pMem = sp + 1556|0;
 $i = sp + 1552|0;
 $nByte = sp + 336|0;
 $bIntint = sp + 1860|0;
 $flags = sp + 1668|0;
 $type1 = sp + 1666|0;
 $type2 = sp + 1664|0;
 $iA = sp + 328|0;
 $iB = sp + 320|0;
 $rA = sp + 312|0;
 $rB = sp + 304|0;
 $n2 = sp + 1548|0;
 $pCtx = sp + 1544|0;
 $i3 = sp + 1540|0;
 $pCtx4 = sp + 1536|0;
 $iA5 = sp + 296|0;
 $uA = sp + 288|0;
 $iB6 = sp + 280|0;
 $op = sp + 1859|0;
 $res = sp + 1532|0;
 $res2 = sp + 1528|0;
 $affinity = sp + 1858|0;
 $flags1 = sp + 1662|0;
 $flags3 = sp + 1660|0;
 $n7 = sp + 1524|0;
 $i8 = sp + 1520|0;
 $p19 = sp + 1516|0;
 $p210 = sp + 1512|0;
 $pKeyInfo = sp + 1508|0;
 $idx = sp + 1504|0;
 $pColl = sp + 1500|0;
 $bRev = sp + 1496|0;
 $v1 = sp + 1492|0;
 $v2 = sp + 1488|0;
 $c = sp + 1484|0;
 $p211 = sp + 1480|0;
 $pC = sp + 1476|0;
 $pCrsr = sp + 1472|0;
 $aOffset = sp + 1468|0;
 $len = sp + 1464|0;
 $i12 = sp + 1460|0;
 $pDest = sp + 1456|0;
 $sMem = sp + 240|0;
 $zData = sp + 1452|0;
 $zHdr = sp + 1448|0;
 $zEndHdr = sp + 1444|0;
 $offset = sp + 1440|0;
 $offset64 = sp + 232|0;
 $avail = sp + 1436|0;
 $t = sp + 1432|0;
 $pReg = sp + 1428|0;
 $zAffinity = sp + 1424|0;
 $cAff = sp + 1857|0;
 $zNewRecord = sp + 1420|0;
 $pRec = sp + 1416|0;
 $nData = sp + 224|0;
 $nHdr = sp + 1412|0;
 $nByte13 = sp + 216|0;
 $nZero = sp + 208|0;
 $nVarint = sp + 1408|0;
 $serial_type = sp + 1404|0;
 $pData0 = sp + 1400|0;
 $pLast = sp + 1396|0;
 $nField = sp + 1392|0;
 $zAffinity14 = sp + 1388|0;
 $file_format = sp + 1384|0;
 $i15 = sp + 1380|0;
 $j = sp + 1376|0;
 $len16 = sp + 1372|0;
 $nEntry = sp + 200|0;
 $pCrsr17 = sp + 1368|0;
 $p118 = sp + 1364|0;
 $zName = sp + 1360|0;
 $nName = sp + 1356|0;
 $pNew = sp + 1352|0;
 $pSavepoint = sp + 1348|0;
 $pTmp = sp + 1344|0;
 $iSavepoint = sp + 1340|0;
 $ii = sp + 1336|0;
 $isTransaction = sp + 1332|0;
 $isSchemaChange = sp + 1328|0;
 $desiredAutoCommit = sp + 1324|0;
 $iRollback = sp + 1320|0;
 $pBt = sp + 1316|0;
 $iMeta = sp + 1312|0;
 $iGen = sp + 1308|0;
 $iMeta19 = sp + 1304|0;
 $iDb = sp + 1300|0;
 $iCookie = sp + 1296|0;
 $pDb = sp + 1292|0;
 $nField20 = sp + 1288|0;
 $pKeyInfo21 = sp + 1284|0;
 $p222 = sp + 1280|0;
 $iDb23 = sp + 1276|0;
 $wrFlag = sp + 1272|0;
 $pX = sp + 1268|0;
 $pCur = sp + 1264|0;
 $pDb24 = sp + 1260|0;
 $pCx = sp + 1256|0;
 $pKeyInfo25 = sp + 1252|0;
 $pgno = sp + 1248|0;
 $pCx26 = sp + 1244|0;
 $pC27 = sp + 1240|0;
 $pCx28 = sp + 1236|0;
 $res29 = sp + 1232|0;
 $oc = sp + 1228|0;
 $pC30 = sp + 1224|0;
 $r = sp + 1208|0;
 $nField31 = sp + 1200|0;
 $iKey = sp + 192|0;
 $eqOnly = sp + 1196|0;
 $alreadyExists = sp + 1192|0;
 $takeJump = sp + 1188|0;
 $ii32 = sp + 1184|0;
 $pC33 = sp + 1180|0;
 $res34 = sp + 1176|0;
 $pFree = sp + 1172|0;
 $pIdxKey = sp + 1168|0;
 $r35 = sp + 1152|0;
 $aTempRec = sp + 1674|0;
 $pC36 = sp + 1148|0;
 $pCrsr37 = sp + 1144|0;
 $res38 = sp + 1140|0;
 $iKey39 = sp + 184|0;
 $v = sp + 176|0;
 $pC40 = sp + 1136|0;
 $res41 = sp + 1132|0;
 $cnt42 = sp + 1128|0;
 $pMem43 = sp + 1124|0;
 $pFrame44 = sp + 1120|0;
 $pData = sp + 1116|0;
 $pKey = sp + 1112|0;
 $pC45 = sp + 1108|0;
 $seekResult = sp + 1104|0;
 $zDb = sp + 1100|0;
 $pTab = sp + 1096|0;
 $op46 = sp + 1092|0;
 $x = sp + 144|0;
 $pC47 = sp + 1088|0;
 $zDb48 = sp + 1084|0;
 $pTab49 = sp + 1080|0;
 $opflags = sp + 1076|0;
 $pC50 = sp + 1072|0;
 $res51 = sp + 1068|0;
 $nKeyCol = sp + 1064|0;
 $pC52 = sp + 1060|0;
 $pC53 = sp + 1056|0;
 $pCrsr54 = sp + 1052|0;
 $n55 = sp + 1048|0;
 $pC56 = sp + 1044|0;
 $v57 = sp + 136|0;
 $pVtab = sp + 1040|0;
 $pModule = sp + 1036|0;
 $pC58 = sp + 1032|0;
 $pC59 = sp + 1028|0;
 $pCrsr60 = sp + 1024|0;
 $res61 = sp + 1020|0;
 $pC62 = sp + 1016|0;
 $pCrsr63 = sp + 1012|0;
 $res64 = sp + 1008|0;
 $pC65 = sp + 1004|0;
 $res66 = sp + 1000|0;
 $pC67 = sp + 996|0;
 $x68 = sp + 104|0;
 $pC69 = sp + 992|0;
 $pCrsr70 = sp + 988|0;
 $res71 = sp + 984|0;
 $r72 = sp + 968|0;
 $pC73 = sp + 964|0;
 $pTabCur = sp + 960|0;
 $rowid = sp + 96|0;
 $pC74 = sp + 956|0;
 $res75 = sp + 952|0;
 $r76 = sp + 936|0;
 $iMoved = sp + 932|0;
 $iDb77 = sp + 928|0;
 $nChange = sp + 924|0;
 $pC78 = sp + 920|0;
 $pgno79 = sp + 916|0;
 $flags80 = sp + 912|0;
 $pDb81 = sp + 908|0;
 $iDb82 = sp + 904|0;
 $zMaster = sp + 900|0;
 $zSql = sp + 896|0;
 $initData = sp + 880|0;
 $nRoot = sp + 872|0;
 $aRoot = sp + 868|0;
 $nErr = sp + 864|0;
 $z = sp + 860|0;
 $pnErr = sp + 856|0;
 $val = sp + 88|0;
 $iSet = sp + 852|0;
 $exists = sp + 848|0;
 $nMem = sp + 844|0;
 $nByte83 = sp + 840|0;
 $pRt = sp + 836|0;
 $pMem84 = sp + 832|0;
 $pEnd = sp + 828|0;
 $pFrame85 = sp + 824|0;
 $pProgram = sp + 820|0;
 $t86 = sp + 816|0;
 $pFrame87 = sp + 812|0;
 $pIn = sp + 808|0;
 $pFrame88 = sp + 804|0;
 $n89 = sp + 800|0;
 $pCtx90 = sp + 796|0;
 $i91 = sp + 792|0;
 $pCtx92 = sp + 788|0;
 $pMem93 = sp + 784|0;
 $t94 = sp + 48|0;
 $pMem95 = sp + 780|0;
 $i96 = sp + 776|0;
 $aRes = sp + 764|0;
 $pMem97 = sp + 760|0;
 $pBt98 = sp + 756|0;
 $pPager = sp + 752|0;
 $eNew = sp + 748|0;
 $eOld = sp + 744|0;
 $zFilename = sp + 740|0;
 $pBt99 = sp + 736|0;
 $isWriteLock = sp + 1673|0;
 $p1100 = sp + 732|0;
 $z101 = sp + 728|0;
 $pVTab = sp + 724|0;
 $sMem102 = sp + 8|0;
 $zTab = sp + 720|0;
 $pCur103 = sp + 716|0;
 $pVCur = sp + 712|0;
 $pVtab104 = sp + 708|0;
 $pModule105 = sp + 704|0;
 $nArg = sp + 700|0;
 $iQuery = sp + 696|0;
 $pModule106 = sp + 692|0;
 $pQuery = sp + 688|0;
 $pArgc = sp + 684|0;
 $pVCur107 = sp + 680|0;
 $pVtab108 = sp + 676|0;
 $pCur109 = sp + 672|0;
 $res110 = sp + 668|0;
 $i111 = sp + 664|0;
 $apArg = sp + 660|0;
 $pVtab112 = sp + 656|0;
 $pModule113 = sp + 652|0;
 $pDest114 = sp + 648|0;
 $sContext = sp + 616|0;
 $pCur115 = sp + 612|0;
 $pVtab116 = sp + 608|0;
 $pModule117 = sp + 604|0;
 $res118 = sp + 600|0;
 $pCur119 = sp + 596|0;
 $pVtab120 = sp + 592|0;
 $pName = sp + 588|0;
 $pVtab121 = sp + 584|0;
 $pModule122 = sp + 580|0;
 $nArg123 = sp + 576|0;
 $i124 = sp + 572|0;
 $rowid125 = sp;
 $apArg126 = sp + 568|0;
 $pX127 = sp + 564|0;
 $vtabOnConflict = sp + 1672|0;
 $newMax = sp + 560|0;
 $pBt128 = sp + 556|0;
 $zTrace = sp + 552|0;
 $i129 = sp + 548|0;
 $x130 = sp + 544|0;
 $z131 = sp + 540|0;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 88|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$aOp>>2] = $3;
 $4 = HEAP32[$aOp>>2]|0;
 HEAP32[$pOp>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP8[$resetSchemaOnFault>>0] = 0;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 66|0);
 $9 = HEAP8[$8>>0]|0;
 HEAP8[$encoding>>0] = $9;
 HEAP32[$iCompare>>2] = 0;
 HEAP32[$nVmStep>>2] = 0;
 HEAP32[$nProgressLimit>>2] = 0;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 92|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$aMem>>2] = $12;
 HEAP32[$pIn1>>2] = 0;
 HEAP32[$pIn2>>2] = 0;
 HEAP32[$pIn3>>2] = 0;
 HEAP32[$pOut>>2] = 0;
 HEAP32[$aPermute>>2] = 0;
 $13 = HEAP32[$db>>2]|0;
 $14 = ((($13)) + 32|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = $lastRowid;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = (($21) + 4)|0;
 $24 = $23;
 HEAP32[$24>>2] = $20;
 $25 = HEAP32[$0>>2]|0;
 _sqlite3VdbeEnter($25);
 $26 = HEAP32[$0>>2]|0;
 $27 = ((($26)) + 40|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)==(7);
 L1: do {
  if ($29) {
   label = 889;
  } else {
   $30 = HEAP32[$0>>2]|0;
   $31 = ((($30)) + 40|0);
   HEAP32[$31>>2] = 0;
   $32 = HEAP32[$0>>2]|0;
   $33 = ((($32)) + 56|0);
   $34 = $33;
   $35 = $34;
   HEAP32[$35>>2] = 0;
   $36 = (($34) + 4)|0;
   $37 = $36;
   HEAP32[$37>>2] = 0;
   $38 = HEAP32[$0>>2]|0;
   $39 = ((($38)) + 104|0);
   HEAP32[$39>>2] = 0;
   $40 = HEAP32[$db>>2]|0;
   $41 = ((($40)) + 380|0);
   $42 = ((($41)) + 8|0);
   HEAP32[$42>>2] = 0;
   $43 = HEAP32[$db>>2]|0;
   $44 = ((($43)) + 248|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = ($45|0)!=(0);
   L3: do {
    if (!($46)) {
     $47 = HEAP32[$db>>2]|0;
     $48 = ((($47)) + 304|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49|0)!=(0|0);
     if ($50) {
      $51 = HEAP32[$0>>2]|0;
      $52 = ((($51)) + 156|0);
      $53 = ((($52)) + 16|0);
      $54 = HEAP32[$53>>2]|0;
      HEAP32[$iPrior>>2] = $54;
      $55 = HEAP32[$db>>2]|0;
      $56 = ((($55)) + 312|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = HEAP32[$iPrior>>2]|0;
      $59 = HEAP32[$db>>2]|0;
      $60 = ((($59)) + 312|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = (($58>>>0) % ($61>>>0))&-1;
      $63 = (($57) - ($62))|0;
      HEAP32[$nProgressLimit>>2] = $63;
     }
     $64 = HEAP32[$0>>2]|0;
     $65 = ((($64)) + 36|0);
     $66 = HEAP32[$65>>2]|0;
     $67 = HEAP32[$aOp>>2]|0;
     $68 = (($67) + (($66*20)|0)|0);
     HEAP32[$pOp>>2] = $68;
     L8: while(1) {
      $69 = HEAP32[$nVmStep>>2]|0;
      $70 = (($69) + 1)|0;
      HEAP32[$nVmStep>>2] = $70;
      $71 = HEAP32[$pOp>>2]|0;
      $72 = HEAP8[$71>>0]|0;
      $73 = $72&255;
      L10: do {
       switch ($73|0) {
       case 87:  {
        label = 64;
        break L8;
        break;
       }
       case 1:  {
        label = 393;
        break L8;
        break;
       }
       case 13:  {
        label = 7;
        break;
       }
       case 14:  {
        $110 = HEAP32[$pOp>>2]|0;
        $111 = ((($110)) + 4|0);
        $112 = HEAP32[$111>>2]|0;
        $113 = HEAP32[$aMem>>2]|0;
        $114 = (($113) + (($112*40)|0)|0);
        HEAP32[$pIn1>>2] = $114;
        $115 = HEAP32[$pIn1>>2]|0;
        $116 = ((($115)) + 8|0);
        HEAP16[$116>>1] = 4;
        $117 = HEAP32[$pOp>>2]|0;
        $118 = HEAP32[$aOp>>2]|0;
        $119 = $117;
        $120 = $118;
        $121 = (($119) - ($120))|0;
        $122 = (($121|0) / 20)&-1;
        $123 = ($122|0)<(0);
        $124 = $123 << 31 >> 31;
        $125 = HEAP32[$pIn1>>2]|0;
        $126 = $125;
        $127 = $126;
        HEAP32[$127>>2] = $122;
        $128 = (($126) + 4)|0;
        $129 = $128;
        HEAP32[$129>>2] = $124;
        label = 14;
        break;
       }
       case 72:  {
        $136 = HEAP32[$pOp>>2]|0;
        $137 = ((($136)) + 4|0);
        $138 = HEAP32[$137>>2]|0;
        $139 = HEAP32[$aMem>>2]|0;
        $140 = (($139) + (($138*40)|0)|0);
        HEAP32[$pIn1>>2] = $140;
        $141 = HEAP32[$pIn1>>2]|0;
        $142 = $141;
        $143 = $142;
        $144 = HEAP32[$143>>2]|0;
        $145 = (($142) + 4)|0;
        $146 = $145;
        $147 = HEAP32[$146>>2]|0;
        $148 = HEAP32[$aOp>>2]|0;
        $149 = (($148) + (($144*20)|0)|0);
        HEAP32[$pOp>>2] = $149;
        $150 = HEAP32[$pIn1>>2]|0;
        $151 = ((($150)) + 8|0);
        HEAP16[$151>>1] = 128;
        break;
       }
       case 15:  {
        $152 = HEAP32[$pOp>>2]|0;
        $153 = ((($152)) + 4|0);
        $154 = HEAP32[$153>>2]|0;
        $155 = HEAP32[$aMem>>2]|0;
        $156 = (($155) + (($154*40)|0)|0);
        HEAP32[$pOut>>2] = $156;
        $157 = HEAP32[$pOp>>2]|0;
        $158 = ((($157)) + 12|0);
        $159 = HEAP32[$158>>2]|0;
        $160 = (($159) - 1)|0;
        $161 = ($160|0)<(0);
        $162 = $161 << 31 >> 31;
        $163 = HEAP32[$pOut>>2]|0;
        $164 = $163;
        $165 = $164;
        HEAP32[$165>>2] = $160;
        $166 = (($164) + 4)|0;
        $167 = $166;
        HEAP32[$167>>2] = $162;
        $168 = HEAP32[$pOut>>2]|0;
        $169 = ((($168)) + 8|0);
        HEAP16[$169>>1] = 4;
        $170 = HEAP32[$pOp>>2]|0;
        $171 = ((($170)) + 8|0);
        $172 = HEAP32[$171>>2]|0;
        $173 = ($172|0)!=(0);
        if ($173) {
         label = 14;
        }
        break;
       }
       case 73:  {
        $174 = HEAP32[$pOp>>2]|0;
        $175 = ((($174)) + 4|0);
        $176 = HEAP32[$175>>2]|0;
        $177 = HEAP32[$aMem>>2]|0;
        $178 = (($177) + (($176*40)|0)|0);
        HEAP32[$pIn1>>2] = $178;
        $179 = HEAP32[$pIn1>>2]|0;
        $180 = $179;
        $181 = $180;
        $182 = HEAP32[$181>>2]|0;
        $183 = (($180) + 4)|0;
        $184 = $183;
        $185 = HEAP32[$184>>2]|0;
        $186 = HEAP32[$aOp>>2]|0;
        $187 = (($186) + (($182*20)|0)|0);
        HEAP32[$pCaller>>2] = $187;
        $188 = HEAP32[$pCaller>>2]|0;
        $189 = ((($188)) + 8|0);
        $190 = HEAP32[$189>>2]|0;
        $191 = (($190) - 1)|0;
        $192 = HEAP32[$aOp>>2]|0;
        $193 = (($192) + (($191*20)|0)|0);
        HEAP32[$pOp>>2] = $193;
        $194 = HEAP32[$pIn1>>2]|0;
        $195 = ((($194)) + 8|0);
        HEAP16[$195>>1] = 128;
        break;
       }
       case 16:  {
        $196 = HEAP32[$pOp>>2]|0;
        $197 = ((($196)) + 4|0);
        $198 = HEAP32[$197>>2]|0;
        $199 = HEAP32[$aMem>>2]|0;
        $200 = (($199) + (($198*40)|0)|0);
        HEAP32[$pIn1>>2] = $200;
        $201 = HEAP32[$pIn1>>2]|0;
        $202 = ((($201)) + 8|0);
        HEAP16[$202>>1] = 4;
        $203 = HEAP32[$pIn1>>2]|0;
        $204 = $203;
        $205 = $204;
        $206 = HEAP32[$205>>2]|0;
        $207 = (($204) + 4)|0;
        $208 = $207;
        $209 = HEAP32[$208>>2]|0;
        HEAP32[$pcDest>>2] = $206;
        $210 = HEAP32[$pOp>>2]|0;
        $211 = HEAP32[$aOp>>2]|0;
        $212 = $210;
        $213 = $211;
        $214 = (($212) - ($213))|0;
        $215 = (($214|0) / 20)&-1;
        $216 = ($215|0)<(0);
        $217 = $216 << 31 >> 31;
        $218 = HEAP32[$pIn1>>2]|0;
        $219 = $218;
        $220 = $219;
        HEAP32[$220>>2] = $215;
        $221 = (($219) + 4)|0;
        $222 = $221;
        HEAP32[$222>>2] = $217;
        $223 = HEAP32[$pcDest>>2]|0;
        $224 = HEAP32[$aOp>>2]|0;
        $225 = (($224) + (($223*20)|0)|0);
        HEAP32[$pOp>>2] = $225;
        break;
       }
       case 74:  {
        $226 = HEAP32[$pOp>>2]|0;
        $227 = ((($226)) + 12|0);
        $228 = HEAP32[$227>>2]|0;
        $229 = HEAP32[$aMem>>2]|0;
        $230 = (($229) + (($228*40)|0)|0);
        HEAP32[$pIn3>>2] = $230;
        $231 = HEAP32[$pIn3>>2]|0;
        $232 = ((($231)) + 8|0);
        $233 = HEAP16[$232>>1]|0;
        $234 = $233&65535;
        $235 = $234 & 1;
        $236 = ($235|0)==(0);
        if (!($236)) {
         label = 20;
        }
        break;
       }
       case 75:  {
        label = 20;
        break;
       }
       case 76:  {
        $367 = HEAP32[$0>>2]|0;
        $368 = HEAP32[$pOp>>2]|0;
        $369 = (_out2Prerelease($367,$368)|0);
        HEAP32[$pOut>>2] = $369;
        $370 = HEAP32[$pOp>>2]|0;
        $371 = ((($370)) + 4|0);
        $372 = HEAP32[$371>>2]|0;
        $373 = ($372|0)<(0);
        $374 = $373 << 31 >> 31;
        $375 = HEAP32[$pOut>>2]|0;
        $376 = $375;
        $377 = $376;
        HEAP32[$377>>2] = $372;
        $378 = (($376) + 4)|0;
        $379 = $378;
        HEAP32[$379>>2] = $374;
        break;
       }
       case 77:  {
        $380 = HEAP32[$0>>2]|0;
        $381 = HEAP32[$pOp>>2]|0;
        $382 = (_out2Prerelease($380,$381)|0);
        HEAP32[$pOut>>2] = $382;
        $383 = HEAP32[$pOp>>2]|0;
        $384 = ((($383)) + 16|0);
        $385 = HEAP32[$384>>2]|0;
        $386 = $385;
        $387 = $386;
        $388 = HEAP32[$387>>2]|0;
        $389 = (($386) + 4)|0;
        $390 = $389;
        $391 = HEAP32[$390>>2]|0;
        $392 = HEAP32[$pOut>>2]|0;
        $393 = $392;
        $394 = $393;
        HEAP32[$394>>2] = $388;
        $395 = (($393) + 4)|0;
        $396 = $395;
        HEAP32[$396>>2] = $391;
        break;
       }
       case 132:  {
        $397 = HEAP32[$0>>2]|0;
        $398 = HEAP32[$pOp>>2]|0;
        $399 = (_out2Prerelease($397,$398)|0);
        HEAP32[$pOut>>2] = $399;
        $400 = HEAP32[$pOut>>2]|0;
        $401 = ((($400)) + 8|0);
        HEAP16[$401>>1] = 8;
        $402 = HEAP32[$pOp>>2]|0;
        $403 = ((($402)) + 16|0);
        $404 = HEAP32[$403>>2]|0;
        $405 = +HEAPF64[$404>>3];
        $406 = HEAP32[$pOut>>2]|0;
        HEAPF64[$406>>3] = $405;
        break;
       }
       case 97:  {
        $407 = HEAP32[$0>>2]|0;
        $408 = HEAP32[$pOp>>2]|0;
        $409 = (_out2Prerelease($407,$408)|0);
        HEAP32[$pOut>>2] = $409;
        $410 = HEAP32[$pOp>>2]|0;
        HEAP8[$410>>0] = 78;
        $411 = HEAP32[$pOp>>2]|0;
        $412 = ((($411)) + 16|0);
        $413 = HEAP32[$412>>2]|0;
        $414 = (_sqlite3Strlen30($413)|0);
        $415 = HEAP32[$pOp>>2]|0;
        $416 = ((($415)) + 4|0);
        HEAP32[$416>>2] = $414;
        $417 = HEAP8[$encoding>>0]|0;
        $418 = $417&255;
        $419 = ($418|0)!=(1);
        if ($419) {
         $420 = HEAP32[$pOut>>2]|0;
         $421 = HEAP32[$pOp>>2]|0;
         $422 = ((($421)) + 16|0);
         $423 = HEAP32[$422>>2]|0;
         $424 = (_sqlite3VdbeMemSetStr($420,$423,-1,1,0)|0);
         HEAP32[$rc>>2] = $424;
         $425 = HEAP32[$pOut>>2]|0;
         $426 = HEAP8[$encoding>>0]|0;
         $427 = $426&255;
         $428 = (_sqlite3VdbeChangeEncoding($425,$427)|0);
         $429 = (0)!=($428|0);
         if ($429) {
          label = 889;
          break L1;
         }
         $430 = HEAP32[$pOut>>2]|0;
         $431 = ((($430)) + 24|0);
         HEAP32[$431>>2] = 0;
         $432 = HEAP32[$pOut>>2]|0;
         $433 = ((($432)) + 8|0);
         $434 = HEAP16[$433>>1]|0;
         $435 = $434&65535;
         $436 = $435 | 2048;
         $437 = $436&65535;
         HEAP16[$433>>1] = $437;
         $438 = HEAP32[$pOp>>2]|0;
         $439 = ((($438)) + 1|0);
         $440 = HEAP8[$439>>0]|0;
         $441 = $440 << 24 >> 24;
         $442 = ($441|0)==(-1);
         if ($442) {
          $443 = HEAP32[$db>>2]|0;
          $444 = HEAP32[$pOp>>2]|0;
          $445 = ((($444)) + 16|0);
          $446 = HEAP32[$445>>2]|0;
          _sqlite3DbFree($443,$446);
         }
         $447 = HEAP32[$pOp>>2]|0;
         $448 = ((($447)) + 1|0);
         HEAP8[$448>>0] = -1;
         $449 = HEAP32[$pOut>>2]|0;
         $450 = ((($449)) + 16|0);
         $451 = HEAP32[$450>>2]|0;
         $452 = HEAP32[$pOp>>2]|0;
         $453 = ((($452)) + 16|0);
         HEAP32[$453>>2] = $451;
         $454 = HEAP32[$pOut>>2]|0;
         $455 = ((($454)) + 12|0);
         $456 = HEAP32[$455>>2]|0;
         $457 = HEAP32[$pOp>>2]|0;
         $458 = ((($457)) + 4|0);
         HEAP32[$458>>2] = $456;
        }
        $459 = HEAP32[$pOp>>2]|0;
        $460 = ((($459)) + 4|0);
        $461 = HEAP32[$460>>2]|0;
        $462 = HEAP32[$db>>2]|0;
        $463 = ((($462)) + 96|0);
        $464 = HEAP32[$463>>2]|0;
        $465 = ($461|0)>($464|0);
        if ($465) {
         label = 888;
         break L8;
        } else {
         label = 43;
        }
        break;
       }
       case 78:  {
        label = 43;
        break;
       }
       case 79:  {
        $509 = HEAP32[$0>>2]|0;
        $510 = HEAP32[$pOp>>2]|0;
        $511 = (_out2Prerelease($509,$510)|0);
        HEAP32[$pOut>>2] = $511;
        $512 = HEAP32[$pOp>>2]|0;
        $513 = ((($512)) + 12|0);
        $514 = HEAP32[$513>>2]|0;
        $515 = HEAP32[$pOp>>2]|0;
        $516 = ((($515)) + 8|0);
        $517 = HEAP32[$516>>2]|0;
        $518 = (($514) - ($517))|0;
        HEAP32[$cnt>>2] = $518;
        $519 = HEAP32[$pOp>>2]|0;
        $520 = ((($519)) + 4|0);
        $521 = HEAP32[$520>>2]|0;
        $522 = ($521|0)!=(0);
        $523 = $522 ? 257 : 1;
        $524 = $523&65535;
        HEAP16[$nullFlag>>1] = $524;
        $525 = HEAP32[$pOut>>2]|0;
        $526 = ((($525)) + 8|0);
        HEAP16[$526>>1] = $524;
        $527 = HEAP32[$pOut>>2]|0;
        $528 = ((($527)) + 12|0);
        HEAP32[$528>>2] = 0;
        while(1) {
         $529 = HEAP32[$cnt>>2]|0;
         $530 = ($529|0)>(0);
         if (!($530)) {
          break L10;
         }
         $531 = HEAP32[$pOut>>2]|0;
         $532 = ((($531)) + 40|0);
         HEAP32[$pOut>>2] = $532;
         $533 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($533);
         $534 = HEAP16[$nullFlag>>1]|0;
         $535 = HEAP32[$pOut>>2]|0;
         $536 = ((($535)) + 8|0);
         HEAP16[$536>>1] = $534;
         $537 = HEAP32[$pOut>>2]|0;
         $538 = ((($537)) + 12|0);
         HEAP32[$538>>2] = 0;
         $539 = HEAP32[$cnt>>2]|0;
         $540 = (($539) + -1)|0;
         HEAP32[$cnt>>2] = $540;
        }
        break;
       }
       case 80:  {
        $541 = HEAP32[$pOp>>2]|0;
        $542 = ((($541)) + 4|0);
        $543 = HEAP32[$542>>2]|0;
        $544 = HEAP32[$aMem>>2]|0;
        $545 = (($544) + (($543*40)|0)|0);
        HEAP32[$pOut>>2] = $545;
        $546 = HEAP32[$pOut>>2]|0;
        $547 = ((($546)) + 8|0);
        $548 = HEAP16[$547>>1]|0;
        $549 = $548&65535;
        $550 = $549 | 1;
        $551 = $550 & -129;
        $552 = $551&65535;
        $553 = HEAP32[$pOut>>2]|0;
        $554 = ((($553)) + 8|0);
        HEAP16[$554>>1] = $552;
        break;
       }
       case 81:  {
        $555 = HEAP32[$0>>2]|0;
        $556 = HEAP32[$pOp>>2]|0;
        $557 = (_out2Prerelease($555,$556)|0);
        HEAP32[$pOut>>2] = $557;
        $558 = HEAP32[$pOut>>2]|0;
        $559 = HEAP32[$pOp>>2]|0;
        $560 = ((($559)) + 16|0);
        $561 = HEAP32[$560>>2]|0;
        $562 = HEAP32[$pOp>>2]|0;
        $563 = ((($562)) + 4|0);
        $564 = HEAP32[$563>>2]|0;
        (_sqlite3VdbeMemSetStr($558,$561,$564,0,0)|0);
        $565 = HEAP8[$encoding>>0]|0;
        $566 = HEAP32[$pOut>>2]|0;
        $567 = ((($566)) + 10|0);
        HEAP8[$567>>0] = $565;
        break;
       }
       case 82:  {
        $568 = HEAP32[$pOp>>2]|0;
        $569 = ((($568)) + 4|0);
        $570 = HEAP32[$569>>2]|0;
        $571 = (($570) - 1)|0;
        $572 = HEAP32[$0>>2]|0;
        $573 = ((($572)) + 116|0);
        $574 = HEAP32[$573>>2]|0;
        $575 = (($574) + (($571*40)|0)|0);
        HEAP32[$pVar>>2] = $575;
        $576 = HEAP32[$pVar>>2]|0;
        $577 = (_sqlite3VdbeMemTooBig($576)|0);
        $578 = ($577|0)!=(0);
        if ($578) {
         label = 888;
         break L8;
        }
        $579 = HEAP32[$0>>2]|0;
        $580 = HEAP32[$pOp>>2]|0;
        $581 = (_out2Prerelease($579,$580)|0);
        HEAP32[$pOut>>2] = $581;
        $582 = HEAP32[$pOut>>2]|0;
        $583 = HEAP32[$pVar>>2]|0;
        _sqlite3VdbeMemShallowCopy($582,$583,2048);
        break;
       }
       case 83:  {
        $584 = HEAP32[$pOp>>2]|0;
        $585 = ((($584)) + 12|0);
        $586 = HEAP32[$585>>2]|0;
        HEAP32[$n>>2] = $586;
        $587 = HEAP32[$pOp>>2]|0;
        $588 = ((($587)) + 4|0);
        $589 = HEAP32[$588>>2]|0;
        HEAP32[$p1>>2] = $589;
        $590 = HEAP32[$pOp>>2]|0;
        $591 = ((($590)) + 8|0);
        $592 = HEAP32[$591>>2]|0;
        HEAP32[$p2>>2] = $592;
        $593 = HEAP32[$p1>>2]|0;
        $594 = HEAP32[$aMem>>2]|0;
        $595 = (($594) + (($593*40)|0)|0);
        HEAP32[$pIn1>>2] = $595;
        $596 = HEAP32[$p2>>2]|0;
        $597 = HEAP32[$aMem>>2]|0;
        $598 = (($597) + (($596*40)|0)|0);
        HEAP32[$pOut>>2] = $598;
        while(1) {
         $599 = HEAP32[$pOut>>2]|0;
         $600 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemMove($599,$600);
         $601 = HEAP32[$pOut>>2]|0;
         $602 = ((($601)) + 8|0);
         $603 = HEAP16[$602>>1]|0;
         $604 = $603&65535;
         $605 = $604 & 4096;
         $606 = ($605|0)!=(0);
         if ($606) {
          $607 = HEAP32[$pOut>>2]|0;
          $608 = (_sqlite3VdbeMemMakeWriteable($607)|0);
          $609 = ($608|0)!=(0);
          if ($609) {
           label = 889;
           break L1;
          }
         }
         $610 = HEAP32[$pIn1>>2]|0;
         $611 = ((($610)) + 40|0);
         HEAP32[$pIn1>>2] = $611;
         $612 = HEAP32[$pOut>>2]|0;
         $613 = ((($612)) + 40|0);
         HEAP32[$pOut>>2] = $613;
         $614 = HEAP32[$n>>2]|0;
         $615 = (($614) + -1)|0;
         HEAP32[$n>>2] = $615;
         $616 = ($615|0)!=(0);
         if (!($616)) {
          break;
         }
        }
        break;
       }
       case 84:  {
        $617 = HEAP32[$pOp>>2]|0;
        $618 = ((($617)) + 12|0);
        $619 = HEAP32[$618>>2]|0;
        HEAP32[$n1>>2] = $619;
        $620 = HEAP32[$pOp>>2]|0;
        $621 = ((($620)) + 4|0);
        $622 = HEAP32[$621>>2]|0;
        $623 = HEAP32[$aMem>>2]|0;
        $624 = (($623) + (($622*40)|0)|0);
        HEAP32[$pIn1>>2] = $624;
        $625 = HEAP32[$pOp>>2]|0;
        $626 = ((($625)) + 8|0);
        $627 = HEAP32[$626>>2]|0;
        $628 = HEAP32[$aMem>>2]|0;
        $629 = (($628) + (($627*40)|0)|0);
        HEAP32[$pOut>>2] = $629;
        while(1) {
         $630 = HEAP32[$pOut>>2]|0;
         $631 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemShallowCopy($630,$631,4096);
         $632 = HEAP32[$pOut>>2]|0;
         $633 = ((($632)) + 8|0);
         $634 = HEAP16[$633>>1]|0;
         $635 = $634&65535;
         $636 = $635 & 4096;
         $637 = ($636|0)!=(0);
         if ($637) {
          $638 = HEAP32[$pOut>>2]|0;
          $639 = (_sqlite3VdbeMemMakeWriteable($638)|0);
          $640 = ($639|0)!=(0);
          if ($640) {
           label = 889;
           break L1;
          }
         }
         $641 = HEAP32[$n1>>2]|0;
         $642 = (($641) + -1)|0;
         HEAP32[$n1>>2] = $642;
         $643 = ($641|0)==(0);
         if ($643) {
          break L10;
         }
         $644 = HEAP32[$pOut>>2]|0;
         $645 = ((($644)) + 40|0);
         HEAP32[$pOut>>2] = $645;
         $646 = HEAP32[$pIn1>>2]|0;
         $647 = ((($646)) + 40|0);
         HEAP32[$pIn1>>2] = $647;
        }
        break;
       }
       case 85:  {
        $648 = HEAP32[$pOp>>2]|0;
        $649 = ((($648)) + 4|0);
        $650 = HEAP32[$649>>2]|0;
        $651 = HEAP32[$aMem>>2]|0;
        $652 = (($651) + (($650*40)|0)|0);
        HEAP32[$pIn1>>2] = $652;
        $653 = HEAP32[$pOp>>2]|0;
        $654 = ((($653)) + 8|0);
        $655 = HEAP32[$654>>2]|0;
        $656 = HEAP32[$aMem>>2]|0;
        $657 = (($656) + (($655*40)|0)|0);
        HEAP32[$pOut>>2] = $657;
        $658 = HEAP32[$pOut>>2]|0;
        $659 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemShallowCopy($658,$659,4096);
        break;
       }
       case 86:  {
        $660 = HEAP32[$pOp>>2]|0;
        $661 = ((($660)) + 4|0);
        $662 = HEAP32[$661>>2]|0;
        $663 = HEAP32[$aMem>>2]|0;
        $664 = (($663) + (($662*40)|0)|0);
        HEAP32[$pIn1>>2] = $664;
        $665 = HEAP32[$pOp>>2]|0;
        $666 = ((($665)) + 8|0);
        $667 = HEAP32[$666>>2]|0;
        $668 = HEAP32[$aMem>>2]|0;
        $669 = (($668) + (($667*40)|0)|0);
        HEAP32[$pOut>>2] = $669;
        $670 = HEAP32[$pOut>>2]|0;
        $671 = HEAP32[$pIn1>>2]|0;
        $672 = $671;
        $673 = $672;
        $674 = HEAP32[$673>>2]|0;
        $675 = (($672) + 4)|0;
        $676 = $675;
        $677 = HEAP32[$676>>2]|0;
        _sqlite3VdbeMemSetInt64($670,$674,$677);
        break;
       }
       case 52:  {
        $761 = HEAP32[$pOp>>2]|0;
        $762 = ((($761)) + 4|0);
        $763 = HEAP32[$762>>2]|0;
        $764 = HEAP32[$aMem>>2]|0;
        $765 = (($764) + (($763*40)|0)|0);
        HEAP32[$pIn1>>2] = $765;
        $766 = HEAP32[$pOp>>2]|0;
        $767 = ((($766)) + 8|0);
        $768 = HEAP32[$767>>2]|0;
        $769 = HEAP32[$aMem>>2]|0;
        $770 = (($769) + (($768*40)|0)|0);
        HEAP32[$pIn2>>2] = $770;
        $771 = HEAP32[$pOp>>2]|0;
        $772 = ((($771)) + 12|0);
        $773 = HEAP32[$772>>2]|0;
        $774 = HEAP32[$aMem>>2]|0;
        $775 = (($774) + (($773*40)|0)|0);
        HEAP32[$pOut>>2] = $775;
        $776 = HEAP32[$pIn1>>2]|0;
        $777 = ((($776)) + 8|0);
        $778 = HEAP16[$777>>1]|0;
        $779 = $778&65535;
        $780 = HEAP32[$pIn2>>2]|0;
        $781 = ((($780)) + 8|0);
        $782 = HEAP16[$781>>1]|0;
        $783 = $782&65535;
        $784 = $779 | $783;
        $785 = $784 & 1;
        $786 = ($785|0)!=(0);
        if ($786) {
         $787 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($787);
         break L10;
        }
        $788 = HEAP32[$pIn1>>2]|0;
        $789 = ((($788)) + 8|0);
        $790 = HEAP16[$789>>1]|0;
        $791 = $790&65535;
        $792 = $791 & 16384;
        $793 = ($792|0)!=(0);
        if ($793) {
         $794 = HEAP32[$pIn1>>2]|0;
         $795 = (_sqlite3VdbeMemExpandBlob($794)|0);
         $796 = ($795|0)!=(0);
         if ($796) {
          label = 889;
          break L1;
         }
        }
        $797 = HEAP32[$pIn2>>2]|0;
        $798 = ((($797)) + 8|0);
        $799 = HEAP16[$798>>1]|0;
        $800 = $799&65535;
        $801 = $800 & 16384;
        $802 = ($801|0)!=(0);
        if ($802) {
         $803 = HEAP32[$pIn2>>2]|0;
         $804 = (_sqlite3VdbeMemExpandBlob($803)|0);
         $805 = ($804|0)!=(0);
         if ($805) {
          label = 889;
          break L1;
         }
        }
        $806 = HEAP32[$pIn1>>2]|0;
        $807 = ((($806)) + 8|0);
        $808 = HEAP16[$807>>1]|0;
        $809 = $808&65535;
        $810 = $809 & 18;
        $811 = ($810|0)==(0);
        if ($811) {
         $812 = HEAP32[$pIn1>>2]|0;
         $813 = HEAP8[$encoding>>0]|0;
         $814 = (_sqlite3VdbeMemStringify($812,$813,0)|0);
         $815 = ($814|0)!=(0);
         if ($815) {
          label = 889;
          break L1;
         }
        }
        $816 = HEAP32[$pIn2>>2]|0;
        $817 = ((($816)) + 8|0);
        $818 = HEAP16[$817>>1]|0;
        $819 = $818&65535;
        $820 = $819 & 18;
        $821 = ($820|0)==(0);
        if ($821) {
         $822 = HEAP32[$pIn2>>2]|0;
         $823 = HEAP8[$encoding>>0]|0;
         $824 = (_sqlite3VdbeMemStringify($822,$823,0)|0);
         $825 = ($824|0)!=(0);
         if ($825) {
          label = 889;
          break L1;
         }
        }
        $826 = HEAP32[$pIn1>>2]|0;
        $827 = ((($826)) + 12|0);
        $828 = HEAP32[$827>>2]|0;
        $829 = HEAP32[$pIn2>>2]|0;
        $830 = ((($829)) + 12|0);
        $831 = HEAP32[$830>>2]|0;
        $832 = (($828) + ($831))|0;
        $833 = ($832|0)<(0);
        $834 = $833 << 31 >> 31;
        $835 = $nByte;
        $836 = $835;
        HEAP32[$836>>2] = $832;
        $837 = (($835) + 4)|0;
        $838 = $837;
        HEAP32[$838>>2] = $834;
        $839 = $nByte;
        $840 = $839;
        $841 = HEAP32[$840>>2]|0;
        $842 = (($839) + 4)|0;
        $843 = $842;
        $844 = HEAP32[$843>>2]|0;
        $845 = HEAP32[$db>>2]|0;
        $846 = ((($845)) + 96|0);
        $847 = HEAP32[$846>>2]|0;
        $848 = ($847|0)<(0);
        $849 = $848 << 31 >> 31;
        $850 = ($844|0)>($849|0);
        $851 = ($841>>>0)>($847>>>0);
        $852 = ($844|0)==($849|0);
        $853 = $852 & $851;
        $854 = $850 | $853;
        if ($854) {
         label = 888;
         break L8;
        }
        $855 = HEAP32[$pOut>>2]|0;
        $856 = $nByte;
        $857 = $856;
        $858 = HEAP32[$857>>2]|0;
        $859 = (($856) + 4)|0;
        $860 = $859;
        $861 = HEAP32[$860>>2]|0;
        $862 = (($858) + 2)|0;
        $863 = HEAP32[$pOut>>2]|0;
        $864 = HEAP32[$pIn2>>2]|0;
        $865 = ($863|0)==($864|0);
        $866 = $865&1;
        $867 = (_sqlite3VdbeMemGrow($855,$862,$866)|0);
        $868 = ($867|0)!=(0);
        if ($868) {
         label = 889;
         break L1;
        }
        $869 = HEAP32[$pOut>>2]|0;
        $870 = ((($869)) + 8|0);
        $871 = HEAP16[$870>>1]|0;
        $872 = $871&65535;
        $873 = $872 & -49664;
        $874 = $873 | 2;
        $875 = $874&65535;
        $876 = HEAP32[$pOut>>2]|0;
        $877 = ((($876)) + 8|0);
        HEAP16[$877>>1] = $875;
        $878 = HEAP32[$pOut>>2]|0;
        $879 = HEAP32[$pIn2>>2]|0;
        $880 = ($878|0)!=($879|0);
        if ($880) {
         $881 = HEAP32[$pOut>>2]|0;
         $882 = ((($881)) + 16|0);
         $883 = HEAP32[$882>>2]|0;
         $884 = HEAP32[$pIn2>>2]|0;
         $885 = ((($884)) + 16|0);
         $886 = HEAP32[$885>>2]|0;
         $887 = HEAP32[$pIn2>>2]|0;
         $888 = ((($887)) + 12|0);
         $889 = HEAP32[$888>>2]|0;
         _memcpy(($883|0),($886|0),($889|0))|0;
        }
        $890 = HEAP32[$pIn2>>2]|0;
        $891 = ((($890)) + 12|0);
        $892 = HEAP32[$891>>2]|0;
        $893 = HEAP32[$pOut>>2]|0;
        $894 = ((($893)) + 16|0);
        $895 = HEAP32[$894>>2]|0;
        $896 = (($895) + ($892)|0);
        $897 = HEAP32[$pIn1>>2]|0;
        $898 = ((($897)) + 16|0);
        $899 = HEAP32[$898>>2]|0;
        $900 = HEAP32[$pIn1>>2]|0;
        $901 = ((($900)) + 12|0);
        $902 = HEAP32[$901>>2]|0;
        _memcpy(($896|0),($899|0),($902|0))|0;
        $903 = $nByte;
        $904 = $903;
        $905 = HEAP32[$904>>2]|0;
        $906 = (($903) + 4)|0;
        $907 = $906;
        $908 = HEAP32[$907>>2]|0;
        $909 = HEAP32[$pOut>>2]|0;
        $910 = ((($909)) + 16|0);
        $911 = HEAP32[$910>>2]|0;
        $912 = (($911) + ($905)|0);
        HEAP8[$912>>0] = 0;
        $913 = $nByte;
        $914 = $913;
        $915 = HEAP32[$914>>2]|0;
        $916 = (($913) + 4)|0;
        $917 = $916;
        $918 = HEAP32[$917>>2]|0;
        $919 = (_i64Add(($915|0),($918|0),1,0)|0);
        $920 = tempRet0;
        $921 = HEAP32[$pOut>>2]|0;
        $922 = ((($921)) + 16|0);
        $923 = HEAP32[$922>>2]|0;
        $924 = (($923) + ($919)|0);
        HEAP8[$924>>0] = 0;
        $925 = HEAP32[$pOut>>2]|0;
        $926 = ((($925)) + 8|0);
        $927 = HEAP16[$926>>1]|0;
        $928 = $927&65535;
        $929 = $928 | 512;
        $930 = $929&65535;
        HEAP16[$926>>1] = $930;
        $931 = $nByte;
        $932 = $931;
        $933 = HEAP32[$932>>2]|0;
        $934 = (($931) + 4)|0;
        $935 = $934;
        $936 = HEAP32[$935>>2]|0;
        $937 = HEAP32[$pOut>>2]|0;
        $938 = ((($937)) + 12|0);
        HEAP32[$938>>2] = $933;
        $939 = HEAP8[$encoding>>0]|0;
        $940 = HEAP32[$pOut>>2]|0;
        $941 = ((($940)) + 10|0);
        HEAP8[$941>>0] = $939;
        break;
       }
       case 51: case 50: case 49: case 48: case 47:  {
        $942 = HEAP32[$pOp>>2]|0;
        $943 = ((($942)) + 4|0);
        $944 = HEAP32[$943>>2]|0;
        $945 = HEAP32[$aMem>>2]|0;
        $946 = (($945) + (($944*40)|0)|0);
        HEAP32[$pIn1>>2] = $946;
        $947 = HEAP32[$pIn1>>2]|0;
        $948 = (_numericType($947)|0);
        HEAP16[$type1>>1] = $948;
        $949 = HEAP32[$pOp>>2]|0;
        $950 = ((($949)) + 8|0);
        $951 = HEAP32[$950>>2]|0;
        $952 = HEAP32[$aMem>>2]|0;
        $953 = (($952) + (($951*40)|0)|0);
        HEAP32[$pIn2>>2] = $953;
        $954 = HEAP32[$pIn2>>2]|0;
        $955 = (_numericType($954)|0);
        HEAP16[$type2>>1] = $955;
        $956 = HEAP32[$pOp>>2]|0;
        $957 = ((($956)) + 12|0);
        $958 = HEAP32[$957>>2]|0;
        $959 = HEAP32[$aMem>>2]|0;
        $960 = (($959) + (($958*40)|0)|0);
        HEAP32[$pOut>>2] = $960;
        $961 = HEAP32[$pIn1>>2]|0;
        $962 = ((($961)) + 8|0);
        $963 = HEAP16[$962>>1]|0;
        $964 = $963&65535;
        $965 = HEAP32[$pIn2>>2]|0;
        $966 = ((($965)) + 8|0);
        $967 = HEAP16[$966>>1]|0;
        $968 = $967&65535;
        $969 = $964 | $968;
        $970 = $969&65535;
        HEAP16[$flags>>1] = $970;
        $971 = HEAP16[$flags>>1]|0;
        $972 = $971&65535;
        $973 = $972 & 1;
        $974 = ($973|0)!=(0);
        L73: do {
         if (!($974)) {
          $975 = HEAP16[$type1>>1]|0;
          $976 = $975&65535;
          $977 = HEAP16[$type2>>1]|0;
          $978 = $977&65535;
          $979 = $976 & $978;
          $980 = $979 & 4;
          $981 = ($980|0)!=(0);
          L75: do {
           if ($981) {
            $982 = HEAP32[$pIn1>>2]|0;
            $983 = $982;
            $984 = $983;
            $985 = HEAP32[$984>>2]|0;
            $986 = (($983) + 4)|0;
            $987 = $986;
            $988 = HEAP32[$987>>2]|0;
            $989 = $iA;
            $990 = $989;
            HEAP32[$990>>2] = $985;
            $991 = (($989) + 4)|0;
            $992 = $991;
            HEAP32[$992>>2] = $988;
            $993 = HEAP32[$pIn2>>2]|0;
            $994 = $993;
            $995 = $994;
            $996 = HEAP32[$995>>2]|0;
            $997 = (($994) + 4)|0;
            $998 = $997;
            $999 = HEAP32[$998>>2]|0;
            $1000 = $iB;
            $1001 = $1000;
            HEAP32[$1001>>2] = $996;
            $1002 = (($1000) + 4)|0;
            $1003 = $1002;
            HEAP32[$1003>>2] = $999;
            HEAP8[$bIntint>>0] = 1;
            $1004 = HEAP32[$pOp>>2]|0;
            $1005 = HEAP8[$1004>>0]|0;
            $1006 = $1005&255;
            switch ($1006|0) {
            case 47:  {
             $1007 = $iA;
             $1008 = $1007;
             $1009 = HEAP32[$1008>>2]|0;
             $1010 = (($1007) + 4)|0;
             $1011 = $1010;
             $1012 = HEAP32[$1011>>2]|0;
             $1013 = (_sqlite3AddInt64($iB,$1009,$1012)|0);
             $1014 = ($1013|0)!=(0);
             if ($1014) {
              break L75;
             }
             break;
            }
            case 48:  {
             $1015 = $iA;
             $1016 = $1015;
             $1017 = HEAP32[$1016>>2]|0;
             $1018 = (($1015) + 4)|0;
             $1019 = $1018;
             $1020 = HEAP32[$1019>>2]|0;
             $1021 = (_sqlite3SubInt64($iB,$1017,$1020)|0);
             $1022 = ($1021|0)!=(0);
             if ($1022) {
              break L75;
             }
             break;
            }
            case 49:  {
             $1023 = $iA;
             $1024 = $1023;
             $1025 = HEAP32[$1024>>2]|0;
             $1026 = (($1023) + 4)|0;
             $1027 = $1026;
             $1028 = HEAP32[$1027>>2]|0;
             $1029 = (_sqlite3MulInt64($iB,$1025,$1028)|0);
             $1030 = ($1029|0)!=(0);
             if ($1030) {
              break L75;
             }
             break;
            }
            case 50:  {
             $1031 = $iA;
             $1032 = $1031;
             $1033 = HEAP32[$1032>>2]|0;
             $1034 = (($1031) + 4)|0;
             $1035 = $1034;
             $1036 = HEAP32[$1035>>2]|0;
             $1037 = ($1033|0)==(0);
             $1038 = ($1036|0)==(0);
             $1039 = $1037 & $1038;
             if ($1039) {
              break L73;
             }
             $1040 = $iA;
             $1041 = $1040;
             $1042 = HEAP32[$1041>>2]|0;
             $1043 = (($1040) + 4)|0;
             $1044 = $1043;
             $1045 = HEAP32[$1044>>2]|0;
             $1046 = ($1042|0)==(-1);
             $1047 = ($1045|0)==(-1);
             $1048 = $1046 & $1047;
             $1049 = $iB;
             $1050 = $1049;
             $1051 = HEAP32[$1050>>2]|0;
             $1052 = (($1049) + 4)|0;
             $1053 = $1052;
             $1054 = HEAP32[$1053>>2]|0;
             $1055 = ($1051|0)==(0);
             $1056 = ($1054|0)==(-2147483648);
             $1057 = $1055 & $1056;
             $or$cond = $1048 & $1057;
             if ($or$cond) {
              break L75;
             }
             $1058 = $iA;
             $1059 = $1058;
             $1060 = HEAP32[$1059>>2]|0;
             $1061 = (($1058) + 4)|0;
             $1062 = $1061;
             $1063 = HEAP32[$1062>>2]|0;
             $1064 = $iB;
             $1065 = $1064;
             $1066 = HEAP32[$1065>>2]|0;
             $1067 = (($1064) + 4)|0;
             $1068 = $1067;
             $1069 = HEAP32[$1068>>2]|0;
             $1070 = (___divdi3(($1066|0),($1069|0),($1060|0),($1063|0))|0);
             $1071 = tempRet0;
             $1072 = $iB;
             $1073 = $1072;
             HEAP32[$1073>>2] = $1070;
             $1074 = (($1072) + 4)|0;
             $1075 = $1074;
             HEAP32[$1075>>2] = $1071;
             break;
            }
            default: {
             $1076 = $iA;
             $1077 = $1076;
             $1078 = HEAP32[$1077>>2]|0;
             $1079 = (($1076) + 4)|0;
             $1080 = $1079;
             $1081 = HEAP32[$1080>>2]|0;
             $1082 = ($1078|0)==(0);
             $1083 = ($1081|0)==(0);
             $1084 = $1082 & $1083;
             if ($1084) {
              break L73;
             }
             $1085 = $iA;
             $1086 = $1085;
             $1087 = HEAP32[$1086>>2]|0;
             $1088 = (($1085) + 4)|0;
             $1089 = $1088;
             $1090 = HEAP32[$1089>>2]|0;
             $1091 = ($1087|0)==(-1);
             $1092 = ($1090|0)==(-1);
             $1093 = $1091 & $1092;
             if ($1093) {
              $1094 = $iA;
              $1095 = $1094;
              HEAP32[$1095>>2] = 1;
              $1096 = (($1094) + 4)|0;
              $1097 = $1096;
              HEAP32[$1097>>2] = 0;
             }
             $1098 = $iA;
             $1099 = $1098;
             $1100 = HEAP32[$1099>>2]|0;
             $1101 = (($1098) + 4)|0;
             $1102 = $1101;
             $1103 = HEAP32[$1102>>2]|0;
             $1104 = $iB;
             $1105 = $1104;
             $1106 = HEAP32[$1105>>2]|0;
             $1107 = (($1104) + 4)|0;
             $1108 = $1107;
             $1109 = HEAP32[$1108>>2]|0;
             $1110 = (___remdi3(($1106|0),($1109|0),($1100|0),($1103|0))|0);
             $1111 = tempRet0;
             $1112 = $iB;
             $1113 = $1112;
             HEAP32[$1113>>2] = $1110;
             $1114 = (($1112) + 4)|0;
             $1115 = $1114;
             HEAP32[$1115>>2] = $1111;
            }
            }
            $1116 = $iB;
            $1117 = $1116;
            $1118 = HEAP32[$1117>>2]|0;
            $1119 = (($1116) + 4)|0;
            $1120 = $1119;
            $1121 = HEAP32[$1120>>2]|0;
            $1122 = HEAP32[$pOut>>2]|0;
            $1123 = $1122;
            $1124 = $1123;
            HEAP32[$1124>>2] = $1118;
            $1125 = (($1123) + 4)|0;
            $1126 = $1125;
            HEAP32[$1126>>2] = $1121;
            $1127 = HEAP32[$pOut>>2]|0;
            $1128 = ((($1127)) + 8|0);
            $1129 = HEAP16[$1128>>1]|0;
            $1130 = $1129&65535;
            $1131 = $1130 & -49664;
            $1132 = $1131 | 4;
            $1133 = $1132&65535;
            $1134 = HEAP32[$pOut>>2]|0;
            $1135 = ((($1134)) + 8|0);
            HEAP16[$1135>>1] = $1133;
            break L10;
           } else {
            HEAP8[$bIntint>>0] = 0;
           }
          } while(0);
          $1136 = HEAP32[$pIn1>>2]|0;
          $1137 = (+_sqlite3VdbeRealValue($1136));
          HEAPF64[$rA>>3] = $1137;
          $1138 = HEAP32[$pIn2>>2]|0;
          $1139 = (+_sqlite3VdbeRealValue($1138));
          HEAPF64[$rB>>3] = $1139;
          $1140 = HEAP32[$pOp>>2]|0;
          $1141 = HEAP8[$1140>>0]|0;
          $1142 = $1141&255;
          switch ($1142|0) {
          case 47:  {
           $1143 = +HEAPF64[$rA>>3];
           $1144 = +HEAPF64[$rB>>3];
           $1145 = $1144 + $1143;
           HEAPF64[$rB>>3] = $1145;
           break;
          }
          case 48:  {
           $1146 = +HEAPF64[$rA>>3];
           $1147 = +HEAPF64[$rB>>3];
           $1148 = $1147 - $1146;
           HEAPF64[$rB>>3] = $1148;
           break;
          }
          case 49:  {
           $1149 = +HEAPF64[$rA>>3];
           $1150 = +HEAPF64[$rB>>3];
           $1151 = $1150 * $1149;
           HEAPF64[$rB>>3] = $1151;
           break;
          }
          case 50:  {
           $1152 = +HEAPF64[$rA>>3];
           $1153 = $1152 == 0.0;
           if ($1153) {
            break L73;
           }
           $1154 = +HEAPF64[$rA>>3];
           $1155 = +HEAPF64[$rB>>3];
           $1156 = $1155 / $1154;
           HEAPF64[$rB>>3] = $1156;
           break;
          }
          default: {
           $1157 = +HEAPF64[$rA>>3];
           $1158 = (~~$1157)>>>0;
           $1159 = +Math_abs($1157) >= 1.0 ? $1157 > 0.0 ? (~~+Math_min(+Math_floor($1157 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($1157 - +(~~$1157 >>> 0)) / 4294967296.0) >>> 0 : 0;
           $1160 = $iA;
           $1161 = $1160;
           HEAP32[$1161>>2] = $1158;
           $1162 = (($1160) + 4)|0;
           $1163 = $1162;
           HEAP32[$1163>>2] = $1159;
           $1164 = +HEAPF64[$rB>>3];
           $1165 = (~~$1164)>>>0;
           $1166 = +Math_abs($1164) >= 1.0 ? $1164 > 0.0 ? (~~+Math_min(+Math_floor($1164 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($1164 - +(~~$1164 >>> 0)) / 4294967296.0) >>> 0 : 0;
           $1167 = $iB;
           $1168 = $1167;
           HEAP32[$1168>>2] = $1165;
           $1169 = (($1167) + 4)|0;
           $1170 = $1169;
           HEAP32[$1170>>2] = $1166;
           $1171 = $iA;
           $1172 = $1171;
           $1173 = HEAP32[$1172>>2]|0;
           $1174 = (($1171) + 4)|0;
           $1175 = $1174;
           $1176 = HEAP32[$1175>>2]|0;
           $1177 = ($1173|0)==(0);
           $1178 = ($1176|0)==(0);
           $1179 = $1177 & $1178;
           if ($1179) {
            break L73;
           }
           $1180 = $iA;
           $1181 = $1180;
           $1182 = HEAP32[$1181>>2]|0;
           $1183 = (($1180) + 4)|0;
           $1184 = $1183;
           $1185 = HEAP32[$1184>>2]|0;
           $1186 = ($1182|0)==(-1);
           $1187 = ($1185|0)==(-1);
           $1188 = $1186 & $1187;
           if ($1188) {
            $1189 = $iA;
            $1190 = $1189;
            HEAP32[$1190>>2] = 1;
            $1191 = (($1189) + 4)|0;
            $1192 = $1191;
            HEAP32[$1192>>2] = 0;
           }
           $1193 = $iB;
           $1194 = $1193;
           $1195 = HEAP32[$1194>>2]|0;
           $1196 = (($1193) + 4)|0;
           $1197 = $1196;
           $1198 = HEAP32[$1197>>2]|0;
           $1199 = $iA;
           $1200 = $1199;
           $1201 = HEAP32[$1200>>2]|0;
           $1202 = (($1199) + 4)|0;
           $1203 = $1202;
           $1204 = HEAP32[$1203>>2]|0;
           $1205 = (___remdi3(($1195|0),($1198|0),($1201|0),($1204|0))|0);
           $1206 = tempRet0;
           $1207 = (+($1205>>>0)) + (4294967296.0*(+($1206|0)));
           HEAPF64[$rB>>3] = $1207;
          }
          }
          $1208 = +HEAPF64[$rB>>3];
          $1209 = (_sqlite3IsNaN($1208)|0);
          $1210 = ($1209|0)!=(0);
          if (!($1210)) {
           $1211 = +HEAPF64[$rB>>3];
           $1212 = HEAP32[$pOut>>2]|0;
           HEAPF64[$1212>>3] = $1211;
           $1213 = HEAP32[$pOut>>2]|0;
           $1214 = ((($1213)) + 8|0);
           $1215 = HEAP16[$1214>>1]|0;
           $1216 = $1215&65535;
           $1217 = $1216 & -49664;
           $1218 = $1217 | 8;
           $1219 = $1218&65535;
           $1220 = HEAP32[$pOut>>2]|0;
           $1221 = ((($1220)) + 8|0);
           HEAP16[$1221>>1] = $1219;
           $1222 = HEAP16[$type1>>1]|0;
           $1223 = $1222&65535;
           $1224 = HEAP16[$type2>>1]|0;
           $1225 = $1224&65535;
           $1226 = $1223 | $1225;
           $1227 = $1226 & 8;
           $1228 = ($1227|0)!=(0);
           $1229 = HEAP8[$bIntint>>0]|0;
           $1230 = ($1229<<24>>24)!=(0);
           $or$cond3 = $1228 | $1230;
           if ($or$cond3) {
            break L10;
           }
           $1231 = HEAP32[$pOut>>2]|0;
           _sqlite3VdbeIntegerAffinity($1231);
           break L10;
          }
         }
        } while(0);
        $1232 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($1232);
        break;
       }
       case 88:  {
        $1233 = HEAP32[$pOp>>2]|0;
        $1234 = ((($1233)) + 4|0);
        $1235 = HEAP32[$1234>>2]|0;
        $1236 = ($1235|0)!=(0);
        if ($1236) {
         $1237 = HEAP32[$pOp>>2]|0;
         $1238 = ((($1237)) + 4|0);
         $1239 = HEAP32[$1238>>2]|0;
         $1240 = HEAP32[$aMem>>2]|0;
         $1241 = (($1240) + (($1239*40)|0)|0);
         _sqlite3VdbeMemSetInt64($1241,0,0);
        }
        break;
       }
       case 89:  {
        $1242 = HEAP32[$pOp>>2]|0;
        $1243 = ((($1242)) + 3|0);
        $1244 = HEAP8[$1243>>0]|0;
        $1245 = $1244&255;
        HEAP32[$n2>>2] = $1245;
        $1246 = HEAP32[$db>>2]|0;
        $1247 = HEAP32[$n2>>2]|0;
        $1248 = (($1247) - 1)|0;
        $1249 = $1248<<2;
        $1250 = (32 + ($1249))|0;
        $1251 = (_sqlite3DbMallocRawNN($1246,$1250,0)|0);
        HEAP32[$pCtx>>2] = $1251;
        $1252 = HEAP32[$pCtx>>2]|0;
        $1253 = ($1252|0)==(0|0);
        if ($1253) {
         label = 889;
         break L1;
        }
        $1254 = HEAP32[$pCtx>>2]|0;
        HEAP32[$1254>>2] = 0;
        $1255 = HEAP32[$pOp>>2]|0;
        $1256 = ((($1255)) + 16|0);
        $1257 = HEAP32[$1256>>2]|0;
        $1258 = HEAP32[$pCtx>>2]|0;
        $1259 = ((($1258)) + 4|0);
        HEAP32[$1259>>2] = $1257;
        $1260 = HEAP32[$pOp>>2]|0;
        $1261 = HEAP32[$aOp>>2]|0;
        $1262 = $1260;
        $1263 = $1261;
        $1264 = (($1262) - ($1263))|0;
        $1265 = (($1264|0) / 20)&-1;
        $1266 = HEAP32[$pCtx>>2]|0;
        $1267 = ((($1266)) + 16|0);
        HEAP32[$1267>>2] = $1265;
        $1268 = HEAP32[$0>>2]|0;
        $1269 = HEAP32[$pCtx>>2]|0;
        $1270 = ((($1269)) + 12|0);
        HEAP32[$1270>>2] = $1268;
        $1271 = HEAP32[$n2>>2]|0;
        $1272 = $1271&255;
        $1273 = HEAP32[$pCtx>>2]|0;
        $1274 = ((($1273)) + 26|0);
        HEAP8[$1274>>0] = $1272;
        $1275 = HEAP32[$pOp>>2]|0;
        $1276 = ((($1275)) + 1|0);
        HEAP8[$1276>>0] = -21;
        $1277 = HEAP32[$pCtx>>2]|0;
        $1278 = HEAP32[$pOp>>2]|0;
        $1279 = ((($1278)) + 16|0);
        HEAP32[$1279>>2] = $1277;
        $1280 = HEAP32[$pOp>>2]|0;
        HEAP8[$1280>>0] = 90;
        label = 126;
        break;
       }
       case 90:  {
        label = 126;
        break;
       }
       case 46: case 45: case 44: case 43:  {
        $1404 = HEAP32[$pOp>>2]|0;
        $1405 = ((($1404)) + 4|0);
        $1406 = HEAP32[$1405>>2]|0;
        $1407 = HEAP32[$aMem>>2]|0;
        $1408 = (($1407) + (($1406*40)|0)|0);
        HEAP32[$pIn1>>2] = $1408;
        $1409 = HEAP32[$pOp>>2]|0;
        $1410 = ((($1409)) + 8|0);
        $1411 = HEAP32[$1410>>2]|0;
        $1412 = HEAP32[$aMem>>2]|0;
        $1413 = (($1412) + (($1411*40)|0)|0);
        HEAP32[$pIn2>>2] = $1413;
        $1414 = HEAP32[$pOp>>2]|0;
        $1415 = ((($1414)) + 12|0);
        $1416 = HEAP32[$1415>>2]|0;
        $1417 = HEAP32[$aMem>>2]|0;
        $1418 = (($1417) + (($1416*40)|0)|0);
        HEAP32[$pOut>>2] = $1418;
        $1419 = HEAP32[$pIn1>>2]|0;
        $1420 = ((($1419)) + 8|0);
        $1421 = HEAP16[$1420>>1]|0;
        $1422 = $1421&65535;
        $1423 = HEAP32[$pIn2>>2]|0;
        $1424 = ((($1423)) + 8|0);
        $1425 = HEAP16[$1424>>1]|0;
        $1426 = $1425&65535;
        $1427 = $1422 | $1426;
        $1428 = $1427 & 1;
        $1429 = ($1428|0)!=(0);
        if ($1429) {
         $1430 = HEAP32[$pOut>>2]|0;
         _sqlite3VdbeMemSetNull($1430);
         break L10;
        }
        $1431 = HEAP32[$pIn2>>2]|0;
        $1432 = (_sqlite3VdbeIntValue($1431)|0);
        $1433 = tempRet0;
        $1434 = $iA5;
        $1435 = $1434;
        HEAP32[$1435>>2] = $1432;
        $1436 = (($1434) + 4)|0;
        $1437 = $1436;
        HEAP32[$1437>>2] = $1433;
        $1438 = HEAP32[$pIn1>>2]|0;
        $1439 = (_sqlite3VdbeIntValue($1438)|0);
        $1440 = tempRet0;
        $1441 = $iB6;
        $1442 = $1441;
        HEAP32[$1442>>2] = $1439;
        $1443 = (($1441) + 4)|0;
        $1444 = $1443;
        HEAP32[$1444>>2] = $1440;
        $1445 = HEAP32[$pOp>>2]|0;
        $1446 = HEAP8[$1445>>0]|0;
        HEAP8[$op>>0] = $1446;
        $1447 = HEAP8[$op>>0]|0;
        $1448 = $1447&255;
        $1449 = ($1448|0)==(43);
        do {
         if ($1449) {
          $1450 = $iB6;
          $1451 = $1450;
          $1452 = HEAP32[$1451>>2]|0;
          $1453 = (($1450) + 4)|0;
          $1454 = $1453;
          $1455 = HEAP32[$1454>>2]|0;
          $1456 = $iA5;
          $1457 = $1456;
          $1458 = HEAP32[$1457>>2]|0;
          $1459 = (($1456) + 4)|0;
          $1460 = $1459;
          $1461 = HEAP32[$1460>>2]|0;
          $1462 = $1458 & $1452;
          $1463 = $1461 & $1455;
          $1464 = $iA5;
          $1465 = $1464;
          HEAP32[$1465>>2] = $1462;
          $1466 = (($1464) + 4)|0;
          $1467 = $1466;
          HEAP32[$1467>>2] = $1463;
         } else {
          $1468 = HEAP8[$op>>0]|0;
          $1469 = $1468&255;
          $1470 = ($1469|0)==(44);
          $1471 = $iB6;
          $1472 = $1471;
          $1473 = HEAP32[$1472>>2]|0;
          $1474 = (($1471) + 4)|0;
          $1475 = $1474;
          $1476 = HEAP32[$1475>>2]|0;
          if ($1470) {
           $1477 = $iA5;
           $1478 = $1477;
           $1479 = HEAP32[$1478>>2]|0;
           $1480 = (($1477) + 4)|0;
           $1481 = $1480;
           $1482 = HEAP32[$1481>>2]|0;
           $1483 = $1479 | $1473;
           $1484 = $1482 | $1476;
           $1485 = $iA5;
           $1486 = $1485;
           HEAP32[$1486>>2] = $1483;
           $1487 = (($1485) + 4)|0;
           $1488 = $1487;
           HEAP32[$1488>>2] = $1484;
           break;
          }
          $1489 = ($1473|0)!=(0);
          $1490 = ($1476|0)!=(0);
          $1491 = $1489 | $1490;
          if ($1491) {
           $1492 = $iB6;
           $1493 = $1492;
           $1494 = HEAP32[$1493>>2]|0;
           $1495 = (($1492) + 4)|0;
           $1496 = $1495;
           $1497 = HEAP32[$1496>>2]|0;
           $1498 = ($1497|0)<(0);
           if ($1498) {
            $1499 = HEAP8[$op>>0]|0;
            $1500 = $1499&255;
            $1501 = (91 - ($1500))|0;
            $1502 = $1501&255;
            HEAP8[$op>>0] = $1502;
            $1503 = $iB6;
            $1504 = $1503;
            $1505 = HEAP32[$1504>>2]|0;
            $1506 = (($1503) + 4)|0;
            $1507 = $1506;
            $1508 = HEAP32[$1507>>2]|0;
            $1509 = ($1508|0)>(-1);
            $1510 = ($1505>>>0)>(4294967232);
            $1511 = ($1508|0)==(-1);
            $1512 = $1511 & $1510;
            $1513 = $1509 | $1512;
            $1514 = $iB6;
            $1515 = $1514;
            $1516 = HEAP32[$1515>>2]|0;
            $1517 = (($1514) + 4)|0;
            $1518 = $1517;
            $1519 = HEAP32[$1518>>2]|0;
            $1520 = (_i64Subtract(0,0,($1516|0),($1519|0))|0);
            $1521 = tempRet0;
            $1522 = $1513 ? $1520 : 64;
            $1523 = $1513 ? $1521 : 0;
            $1524 = $iB6;
            $1525 = $1524;
            HEAP32[$1525>>2] = $1522;
            $1526 = (($1524) + 4)|0;
            $1527 = $1526;
            HEAP32[$1527>>2] = $1523;
           }
           $1528 = $iB6;
           $1529 = $1528;
           $1530 = HEAP32[$1529>>2]|0;
           $1531 = (($1528) + 4)|0;
           $1532 = $1531;
           $1533 = HEAP32[$1532>>2]|0;
           $1534 = ($1533|0)>(0);
           $1535 = ($1530>>>0)>=(64);
           $1536 = ($1533|0)==(0);
           $1537 = $1536 & $1535;
           $1538 = $1534 | $1537;
           if ($1538) {
            $1539 = $iA5;
            $1540 = $1539;
            $1541 = HEAP32[$1540>>2]|0;
            $1542 = (($1539) + 4)|0;
            $1543 = $1542;
            $1544 = HEAP32[$1543>>2]|0;
            $1545 = ($1544|0)>(0);
            $1546 = ($1541>>>0)>=(0);
            $1547 = ($1544|0)==(0);
            $1548 = $1547 & $1546;
            $1549 = $1545 | $1548;
            if ($1549) {
             $1554 = 1;
            } else {
             $1550 = HEAP8[$op>>0]|0;
             $1551 = $1550&255;
             $1552 = ($1551|0)==(45);
             $1554 = $1552;
            }
            $1553 = $1554 ? 0 : -1;
            $1555 = ($1553|0)<(0);
            $1556 = $1555 << 31 >> 31;
            $1557 = $iA5;
            $1558 = $1557;
            HEAP32[$1558>>2] = $1553;
            $1559 = (($1557) + 4)|0;
            $1560 = $1559;
            HEAP32[$1560>>2] = $1556;
            break;
           }
           ;HEAP32[$uA>>2]=HEAP32[$iA5>>2]|0;HEAP32[$uA+4>>2]=HEAP32[$iA5+4>>2]|0;
           $1561 = HEAP8[$op>>0]|0;
           $1562 = $1561&255;
           $1563 = ($1562|0)==(45);
           $1564 = $iB6;
           $1565 = $1564;
           $1566 = HEAP32[$1565>>2]|0;
           $1567 = (($1564) + 4)|0;
           $1568 = $1567;
           $1569 = HEAP32[$1568>>2]|0;
           $1570 = $uA;
           $1571 = $1570;
           $1572 = HEAP32[$1571>>2]|0;
           $1573 = (($1570) + 4)|0;
           $1574 = $1573;
           $1575 = HEAP32[$1574>>2]|0;
           if ($1563) {
            $1576 = (_bitshift64Shl(($1572|0),($1575|0),($1566|0))|0);
            $1577 = tempRet0;
            $1578 = $uA;
            $1579 = $1578;
            HEAP32[$1579>>2] = $1576;
            $1580 = (($1578) + 4)|0;
            $1581 = $1580;
            HEAP32[$1581>>2] = $1577;
           } else {
            $1582 = (_bitshift64Lshr(($1572|0),($1575|0),($1566|0))|0);
            $1583 = tempRet0;
            $1584 = $uA;
            $1585 = $1584;
            HEAP32[$1585>>2] = $1582;
            $1586 = (($1584) + 4)|0;
            $1587 = $1586;
            HEAP32[$1587>>2] = $1583;
            $1588 = $iA5;
            $1589 = $1588;
            $1590 = HEAP32[$1589>>2]|0;
            $1591 = (($1588) + 4)|0;
            $1592 = $1591;
            $1593 = HEAP32[$1592>>2]|0;
            $1594 = ($1593|0)<(0);
            if ($1594) {
             $1595 = $iB6;
             $1596 = $1595;
             $1597 = HEAP32[$1596>>2]|0;
             $1598 = (($1595) + 4)|0;
             $1599 = $1598;
             $1600 = HEAP32[$1599>>2]|0;
             $1601 = (_i64Subtract(64,0,($1597|0),($1600|0))|0);
             $1602 = tempRet0;
             $1603 = (_bitshift64Shl(-1,-1,($1601|0))|0);
             $1604 = tempRet0;
             $1605 = $uA;
             $1606 = $1605;
             $1607 = HEAP32[$1606>>2]|0;
             $1608 = (($1605) + 4)|0;
             $1609 = $1608;
             $1610 = HEAP32[$1609>>2]|0;
             $1611 = $1607 | $1603;
             $1612 = $1610 | $1604;
             $1613 = $uA;
             $1614 = $1613;
             HEAP32[$1614>>2] = $1611;
             $1615 = (($1613) + 4)|0;
             $1616 = $1615;
             HEAP32[$1616>>2] = $1612;
            }
           }
           ;HEAP32[$iA5>>2]=HEAP32[$uA>>2]|0;HEAP32[$iA5+4>>2]=HEAP32[$uA+4>>2]|0;
          }
         }
        } while(0);
        $1617 = $iA5;
        $1618 = $1617;
        $1619 = HEAP32[$1618>>2]|0;
        $1620 = (($1617) + 4)|0;
        $1621 = $1620;
        $1622 = HEAP32[$1621>>2]|0;
        $1623 = HEAP32[$pOut>>2]|0;
        $1624 = $1623;
        $1625 = $1624;
        HEAP32[$1625>>2] = $1619;
        $1626 = (($1624) + 4)|0;
        $1627 = $1626;
        HEAP32[$1627>>2] = $1622;
        $1628 = HEAP32[$pOut>>2]|0;
        $1629 = ((($1628)) + 8|0);
        $1630 = HEAP16[$1629>>1]|0;
        $1631 = $1630&65535;
        $1632 = $1631 & -49664;
        $1633 = $1632 | 4;
        $1634 = $1633&65535;
        $1635 = HEAP32[$pOut>>2]|0;
        $1636 = ((($1635)) + 8|0);
        HEAP16[$1636>>1] = $1634;
        break;
       }
       case 91:  {
        $1637 = HEAP32[$pOp>>2]|0;
        $1638 = ((($1637)) + 4|0);
        $1639 = HEAP32[$1638>>2]|0;
        $1640 = HEAP32[$aMem>>2]|0;
        $1641 = (($1640) + (($1639*40)|0)|0);
        HEAP32[$pIn1>>2] = $1641;
        $1642 = HEAP32[$pIn1>>2]|0;
        (_sqlite3VdbeMemIntegerify($1642)|0);
        $1643 = HEAP32[$pOp>>2]|0;
        $1644 = ((($1643)) + 8|0);
        $1645 = HEAP32[$1644>>2]|0;
        $1646 = ($1645|0)<(0);
        $1647 = $1646 << 31 >> 31;
        $1648 = HEAP32[$pIn1>>2]|0;
        $1649 = $1648;
        $1650 = $1649;
        $1651 = HEAP32[$1650>>2]|0;
        $1652 = (($1649) + 4)|0;
        $1653 = $1652;
        $1654 = HEAP32[$1653>>2]|0;
        $1655 = (_i64Add(($1651|0),($1654|0),($1645|0),($1647|0))|0);
        $1656 = tempRet0;
        $1657 = $1648;
        $1658 = $1657;
        HEAP32[$1658>>2] = $1655;
        $1659 = (($1657) + 4)|0;
        $1660 = $1659;
        HEAP32[$1660>>2] = $1656;
        break;
       }
       case 17:  {
        $1661 = HEAP32[$pOp>>2]|0;
        $1662 = ((($1661)) + 4|0);
        $1663 = HEAP32[$1662>>2]|0;
        $1664 = HEAP32[$aMem>>2]|0;
        $1665 = (($1664) + (($1663*40)|0)|0);
        HEAP32[$pIn1>>2] = $1665;
        $1666 = HEAP32[$pIn1>>2]|0;
        $1667 = ((($1666)) + 8|0);
        $1668 = HEAP16[$1667>>1]|0;
        $1669 = $1668&65535;
        $1670 = $1669 & 4;
        $1671 = ($1670|0)==(0);
        if ($1671) {
         $1672 = HEAP32[$pIn1>>2]|0;
         $1673 = HEAP8[$encoding>>0]|0;
         _applyAffinity($1672,67,$1673);
         $1674 = HEAP32[$pIn1>>2]|0;
         $1675 = ((($1674)) + 8|0);
         $1676 = HEAP16[$1675>>1]|0;
         $1677 = $1676&65535;
         $1678 = $1677 & 4;
         $1679 = ($1678|0)==(0);
         if ($1679) {
          $1680 = HEAP32[$pOp>>2]|0;
          $1681 = ((($1680)) + 8|0);
          $1682 = HEAP32[$1681>>2]|0;
          $1683 = ($1682|0)==(0);
          if ($1683) {
           label = 159;
           break L8;
          } else {
           label = 14;
           break L10;
          }
         }
        }
        $1684 = HEAP32[$pIn1>>2]|0;
        $1685 = ((($1684)) + 8|0);
        $1686 = HEAP16[$1685>>1]|0;
        $1687 = $1686&65535;
        $1688 = $1687 & -49664;
        $1689 = $1688 | 4;
        $1690 = $1689&65535;
        $1691 = HEAP32[$pIn1>>2]|0;
        $1692 = ((($1691)) + 8|0);
        HEAP16[$1692>>1] = $1690;
        break;
       }
       case 92:  {
        $1693 = HEAP32[$pOp>>2]|0;
        $1694 = ((($1693)) + 4|0);
        $1695 = HEAP32[$1694>>2]|0;
        $1696 = HEAP32[$aMem>>2]|0;
        $1697 = (($1696) + (($1695*40)|0)|0);
        HEAP32[$pIn1>>2] = $1697;
        $1698 = HEAP32[$pIn1>>2]|0;
        $1699 = ((($1698)) + 8|0);
        $1700 = HEAP16[$1699>>1]|0;
        $1701 = $1700&65535;
        $1702 = $1701 & 4;
        $1703 = ($1702|0)!=(0);
        if ($1703) {
         $1704 = HEAP32[$pIn1>>2]|0;
         (_sqlite3VdbeMemRealify($1704)|0);
        }
        break;
       }
       case 93:  {
        $1705 = HEAP32[$pOp>>2]|0;
        $1706 = ((($1705)) + 4|0);
        $1707 = HEAP32[$1706>>2]|0;
        $1708 = HEAP32[$aMem>>2]|0;
        $1709 = (($1708) + (($1707*40)|0)|0);
        HEAP32[$pIn1>>2] = $1709;
        $1710 = HEAP32[$pIn1>>2]|0;
        $1711 = ((($1710)) + 8|0);
        $1712 = HEAP16[$1711>>1]|0;
        $1713 = $1712&65535;
        $1714 = $1713 & 16384;
        $1715 = ($1714|0)!=(0);
        if ($1715) {
         $1716 = HEAP32[$pIn1>>2]|0;
         $1717 = (_sqlite3VdbeMemExpandBlob($1716)|0);
         $1718 = $1717;
        } else {
         $1718 = 0;
        }
        HEAP32[$rc>>2] = $1718;
        $1719 = HEAP32[$pIn1>>2]|0;
        $1720 = HEAP32[$pOp>>2]|0;
        $1721 = ((($1720)) + 8|0);
        $1722 = HEAP32[$1721>>2]|0;
        $1723 = $1722&255;
        $1724 = HEAP8[$encoding>>0]|0;
        _sqlite3VdbeMemCast($1719,$1723,$1724);
        $1725 = HEAP32[$rc>>2]|0;
        $1726 = ($1725|0)!=(0);
        if ($1726) {
         break L1;
        }
        break;
       }
       case 41: case 38: case 39: case 40: case 36: case 37:  {
        $1727 = HEAP32[$pOp>>2]|0;
        $1728 = ((($1727)) + 4|0);
        $1729 = HEAP32[$1728>>2]|0;
        $1730 = HEAP32[$aMem>>2]|0;
        $1731 = (($1730) + (($1729*40)|0)|0);
        HEAP32[$pIn1>>2] = $1731;
        $1732 = HEAP32[$pOp>>2]|0;
        $1733 = ((($1732)) + 12|0);
        $1734 = HEAP32[$1733>>2]|0;
        $1735 = HEAP32[$aMem>>2]|0;
        $1736 = (($1735) + (($1734*40)|0)|0);
        HEAP32[$pIn3>>2] = $1736;
        $1737 = HEAP32[$pIn1>>2]|0;
        $1738 = ((($1737)) + 8|0);
        $1739 = HEAP16[$1738>>1]|0;
        HEAP16[$flags1>>1] = $1739;
        $1740 = HEAP32[$pIn3>>2]|0;
        $1741 = ((($1740)) + 8|0);
        $1742 = HEAP16[$1741>>1]|0;
        HEAP16[$flags3>>1] = $1742;
        $1743 = HEAP16[$flags1>>1]|0;
        $1744 = $1743&65535;
        $1745 = HEAP16[$flags3>>1]|0;
        $1746 = $1745&65535;
        $1747 = $1744 | $1746;
        $1748 = $1747 & 1;
        $1749 = ($1748|0)!=(0);
        $1750 = HEAP32[$pOp>>2]|0;
        $1751 = ((($1750)) + 3|0);
        $1752 = HEAP8[$1751>>0]|0;
        $1753 = $1752&255;
        do {
         if ($1749) {
          $1754 = $1753 & 128;
          $1755 = ($1754|0)!=(0);
          if (!($1755)) {
           $1768 = HEAP32[$pOp>>2]|0;
           $1769 = ((($1768)) + 3|0);
           $1770 = HEAP8[$1769>>0]|0;
           $1771 = $1770&255;
           $1772 = $1771 & 32;
           $1773 = ($1772|0)!=(0);
           $1774 = HEAP32[$pOp>>2]|0;
           if ($1773) {
            $1775 = ((($1774)) + 8|0);
            $1776 = HEAP32[$1775>>2]|0;
            $1777 = HEAP32[$aMem>>2]|0;
            $1778 = (($1777) + (($1776*40)|0)|0);
            HEAP32[$pOut>>2] = $1778;
            HEAP32[$iCompare>>2] = 1;
            $1779 = HEAP32[$pOut>>2]|0;
            $1780 = ((($1779)) + 8|0);
            $1781 = HEAP16[$1780>>1]|0;
            $1782 = $1781&65535;
            $1783 = $1782 & -49664;
            $1784 = $1783 | 1;
            $1785 = $1784&65535;
            $1786 = HEAP32[$pOut>>2]|0;
            $1787 = ((($1786)) + 8|0);
            HEAP16[$1787>>1] = $1785;
            break L10;
           } else {
            $1788 = ((($1774)) + 3|0);
            $1789 = HEAP8[$1788>>0]|0;
            $1790 = $1789&255;
            $1791 = $1790 & 16;
            $1792 = ($1791|0)!=(0);
            if ($1792) {
             label = 14;
             break L10;
            } else {
             break L10;
            }
           }
          }
          $1756 = HEAP16[$flags1>>1]|0;
          $1757 = $1756&65535;
          $1758 = $1757 & 1;
          $1759 = ($1758|0)!=(0);
          if ($1759) {
           $1760 = HEAP16[$flags3>>1]|0;
           $1761 = $1760&65535;
           $1762 = $1761 & 1;
           $1763 = ($1762|0)!=(0);
           if ($1763) {
            $1764 = HEAP16[$flags3>>1]|0;
            $1765 = $1764&65535;
            $1766 = $1765 & 256;
            $1767 = ($1766|0)==(0);
            if ($1767) {
             HEAP32[$res>>2] = 0;
             break;
            }
           }
          }
          HEAP32[$res>>2] = 1;
         } else {
          $1793 = $1753 & 71;
          $1794 = $1793&255;
          HEAP8[$affinity>>0] = $1794;
          $1795 = HEAP8[$affinity>>0]|0;
          $1796 = $1795 << 24 >> 24;
          $1797 = ($1796|0)>=(67);
          if ($1797) {
           $1798 = HEAP16[$flags1>>1]|0;
           $1799 = $1798&65535;
           $1800 = HEAP16[$flags3>>1]|0;
           $1801 = $1800&65535;
           $1802 = $1799 | $1801;
           $1803 = $1802 & 2;
           $1804 = ($1803|0)!=(0);
           if ($1804) {
            $1805 = HEAP16[$flags1>>1]|0;
            $1806 = $1805&65535;
            $1807 = $1806 & 14;
            $1808 = ($1807|0)==(2);
            if ($1808) {
             $1809 = HEAP32[$pIn1>>2]|0;
             _applyNumericAffinity($1809,0);
             $1810 = HEAP32[$pIn3>>2]|0;
             $1811 = ((($1810)) + 8|0);
             $1812 = HEAP16[$1811>>1]|0;
             HEAP16[$flags3>>1] = $1812;
            }
            $1813 = HEAP16[$flags3>>1]|0;
            $1814 = $1813&65535;
            $1815 = $1814 & 14;
            $1816 = ($1815|0)==(2);
            if ($1816) {
             $1817 = HEAP32[$pIn3>>2]|0;
             _applyNumericAffinity($1817,0);
            }
           }
           $1818 = HEAP32[$pIn1>>2]|0;
           $1819 = ((($1818)) + 8|0);
           $1820 = HEAP16[$1819>>1]|0;
           $1821 = $1820&65535;
           $1822 = HEAP32[$pIn3>>2]|0;
           $1823 = ((($1822)) + 8|0);
           $1824 = HEAP16[$1823>>1]|0;
           $1825 = $1824&65535;
           $1826 = $1821 & $1825;
           $1827 = $1826 & 4;
           $1828 = ($1827|0)!=(0);
           if ($1828) {
            $1829 = HEAP32[$pIn3>>2]|0;
            $1830 = $1829;
            $1831 = $1830;
            $1832 = HEAP32[$1831>>2]|0;
            $1833 = (($1830) + 4)|0;
            $1834 = $1833;
            $1835 = HEAP32[$1834>>2]|0;
            $1836 = HEAP32[$pIn1>>2]|0;
            $1837 = $1836;
            $1838 = $1837;
            $1839 = HEAP32[$1838>>2]|0;
            $1840 = (($1837) + 4)|0;
            $1841 = $1840;
            $1842 = HEAP32[$1841>>2]|0;
            $1843 = ($1835|0)>($1842|0);
            $1844 = ($1832>>>0)>($1839>>>0);
            $1845 = ($1835|0)==($1842|0);
            $1846 = $1845 & $1844;
            $1847 = $1843 | $1846;
            if ($1847) {
             HEAP32[$res>>2] = 1;
             break;
            }
            $1848 = HEAP32[$pIn3>>2]|0;
            $1849 = $1848;
            $1850 = $1849;
            $1851 = HEAP32[$1850>>2]|0;
            $1852 = (($1849) + 4)|0;
            $1853 = $1852;
            $1854 = HEAP32[$1853>>2]|0;
            $1855 = HEAP32[$pIn1>>2]|0;
            $1856 = $1855;
            $1857 = $1856;
            $1858 = HEAP32[$1857>>2]|0;
            $1859 = (($1856) + 4)|0;
            $1860 = $1859;
            $1861 = HEAP32[$1860>>2]|0;
            $1862 = ($1854|0)<($1861|0);
            $1863 = ($1851>>>0)<($1858>>>0);
            $1864 = ($1854|0)==($1861|0);
            $1865 = $1864 & $1863;
            $1866 = $1862 | $1865;
            if ($1866) {
             HEAP32[$res>>2] = -1;
             break;
            } else {
             HEAP32[$res>>2] = 0;
             break;
            }
           }
          } else {
           $1867 = HEAP8[$affinity>>0]|0;
           $1868 = $1867 << 24 >> 24;
           $1869 = ($1868|0)==(66);
           if ($1869) {
            $1870 = HEAP16[$flags1>>1]|0;
            $1871 = $1870&65535;
            $1872 = $1871 & 2;
            $1873 = ($1872|0)==(0);
            if ($1873) {
             $1874 = HEAP16[$flags1>>1]|0;
             $1875 = $1874&65535;
             $1876 = $1875 & 12;
             $1877 = ($1876|0)!=(0);
             if ($1877) {
              $1878 = HEAP32[$pIn1>>2]|0;
              $1879 = HEAP8[$encoding>>0]|0;
              (_sqlite3VdbeMemStringify($1878,$1879,1)|0);
              $1880 = HEAP32[$pIn1>>2]|0;
              $1881 = ((($1880)) + 8|0);
              $1882 = HEAP16[$1881>>1]|0;
              $1883 = $1882&65535;
              $1884 = $1883 & -33280;
              $1885 = HEAP16[$flags1>>1]|0;
              $1886 = $1885&65535;
              $1887 = $1886 & 33279;
              $1888 = $1884 | $1887;
              $1889 = $1888&65535;
              HEAP16[$flags1>>1] = $1889;
             }
            }
            $1890 = HEAP16[$flags3>>1]|0;
            $1891 = $1890&65535;
            $1892 = $1891 & 2;
            $1893 = ($1892|0)==(0);
            if ($1893) {
             $1894 = HEAP16[$flags3>>1]|0;
             $1895 = $1894&65535;
             $1896 = $1895 & 12;
             $1897 = ($1896|0)!=(0);
             if ($1897) {
              $1898 = HEAP32[$pIn3>>2]|0;
              $1899 = HEAP8[$encoding>>0]|0;
              (_sqlite3VdbeMemStringify($1898,$1899,1)|0);
              $1900 = HEAP32[$pIn3>>2]|0;
              $1901 = ((($1900)) + 8|0);
              $1902 = HEAP16[$1901>>1]|0;
              $1903 = $1902&65535;
              $1904 = $1903 & -33280;
              $1905 = HEAP16[$flags3>>1]|0;
              $1906 = $1905&65535;
              $1907 = $1906 & 33279;
              $1908 = $1904 | $1907;
              $1909 = $1908&65535;
              HEAP16[$flags3>>1] = $1909;
             }
            }
           }
          }
          $1910 = HEAP32[$pIn3>>2]|0;
          $1911 = HEAP32[$pIn1>>2]|0;
          $1912 = HEAP32[$pOp>>2]|0;
          $1913 = ((($1912)) + 16|0);
          $1914 = HEAP32[$1913>>2]|0;
          $1915 = (_sqlite3MemCompare($1910,$1911,$1914)|0);
          HEAP32[$res>>2] = $1915;
         }
        } while(0);
        $1916 = HEAP32[$pOp>>2]|0;
        $1917 = HEAP8[$1916>>0]|0;
        $1918 = $1917&255;
        switch ($1918|0) {
        case 37:  {
         $1919 = HEAP32[$res>>2]|0;
         $1920 = ($1919|0)==(0);
         $1921 = $1920&1;
         HEAP32[$res2>>2] = $1921;
         break;
        }
        case 36:  {
         $1922 = HEAP32[$res>>2]|0;
         HEAP32[$res2>>2] = $1922;
         break;
        }
        case 40:  {
         $1923 = HEAP32[$res>>2]|0;
         $1924 = ($1923|0)<(0);
         $1925 = $1924&1;
         HEAP32[$res2>>2] = $1925;
         break;
        }
        case 39:  {
         $1926 = HEAP32[$res>>2]|0;
         $1927 = ($1926|0)<=(0);
         $1928 = $1927&1;
         HEAP32[$res2>>2] = $1928;
         break;
        }
        case 38:  {
         $1929 = HEAP32[$res>>2]|0;
         $1930 = ($1929|0)>(0);
         $1931 = $1930&1;
         HEAP32[$res2>>2] = $1931;
         break;
        }
        default: {
         $1932 = HEAP32[$res>>2]|0;
         $1933 = ($1932|0)>=(0);
         $1934 = $1933&1;
         HEAP32[$res2>>2] = $1934;
        }
        }
        $1935 = HEAP16[$flags1>>1]|0;
        $1936 = HEAP32[$pIn1>>2]|0;
        $1937 = ((($1936)) + 8|0);
        HEAP16[$1937>>1] = $1935;
        $1938 = HEAP16[$flags3>>1]|0;
        $1939 = HEAP32[$pIn3>>2]|0;
        $1940 = ((($1939)) + 8|0);
        HEAP16[$1940>>1] = $1938;
        $1941 = HEAP32[$pOp>>2]|0;
        $1942 = ((($1941)) + 3|0);
        $1943 = HEAP8[$1942>>0]|0;
        $1944 = $1943&255;
        $1945 = $1944 & 32;
        $1946 = ($1945|0)!=(0);
        if (!($1946)) {
         $1986 = HEAP32[$res2>>2]|0;
         $1987 = ($1986|0)!=(0);
         if ($1987) {
          label = 14;
          break L10;
         } else {
          break L10;
         }
        }
        $1947 = HEAP32[$pOp>>2]|0;
        $1948 = ((($1947)) + 8|0);
        $1949 = HEAP32[$1948>>2]|0;
        $1950 = HEAP32[$aMem>>2]|0;
        $1951 = (($1950) + (($1949*40)|0)|0);
        HEAP32[$pOut>>2] = $1951;
        $1952 = HEAP32[$res>>2]|0;
        HEAP32[$iCompare>>2] = $1952;
        $1953 = HEAP32[$res2>>2]|0;
        $1954 = ($1953|0)!=(0);
        $1955 = $1954&1;
        HEAP32[$res2>>2] = $1955;
        $1956 = HEAP32[$pOp>>2]|0;
        $1957 = ((($1956)) + 3|0);
        $1958 = HEAP8[$1957>>0]|0;
        $1959 = $1958&255;
        $1960 = $1959 & 8;
        $1961 = ($1960|0)!=(0);
        if ($1961) {
         $1962 = HEAP32[$pOp>>2]|0;
         $1963 = HEAP8[$1962>>0]|0;
         $1964 = $1963&255;
         $1965 = ($1964|0)==(37);
         $1966 = $1965&1;
         $1967 = HEAP32[$res2>>2]|0;
         $1968 = ($1966|0)==($1967|0);
         if ($1968) {
          break L10;
         }
        }
        $1969 = HEAP32[$pOut>>2]|0;
        $1970 = ((($1969)) + 8|0);
        $1971 = HEAP16[$1970>>1]|0;
        $1972 = $1971&65535;
        $1973 = $1972 & -49664;
        $1974 = $1973 | 4;
        $1975 = $1974&65535;
        $1976 = HEAP32[$pOut>>2]|0;
        $1977 = ((($1976)) + 8|0);
        HEAP16[$1977>>1] = $1975;
        $1978 = HEAP32[$res2>>2]|0;
        $1979 = ($1978|0)<(0);
        $1980 = $1979 << 31 >> 31;
        $1981 = HEAP32[$pOut>>2]|0;
        $1982 = $1981;
        $1983 = $1982;
        HEAP32[$1983>>2] = $1978;
        $1984 = (($1982) + 4)|0;
        $1985 = $1984;
        HEAP32[$1985>>2] = $1980;
        break;
       }
       case 42:  {
        $1988 = HEAP32[$iCompare>>2]|0;
        $1989 = ($1988|0)!=(0);
        if ($1989) {
         label = 14;
        }
        break;
       }
       case 94:  {
        $1990 = HEAP32[$pOp>>2]|0;
        $1991 = ((($1990)) + 16|0);
        $1992 = HEAP32[$1991>>2]|0;
        $1993 = ((($1992)) + 4|0);
        HEAP32[$aPermute>>2] = $1993;
        break;
       }
       case 95:  {
        $1994 = HEAP32[$pOp>>2]|0;
        $1995 = ((($1994)) + 3|0);
        $1996 = HEAP8[$1995>>0]|0;
        $1997 = $1996&255;
        $1998 = $1997 & 1;
        $1999 = ($1998|0)==(0);
        if ($1999) {
         HEAP32[$aPermute>>2] = 0;
        }
        $2000 = HEAP32[$pOp>>2]|0;
        $2001 = ((($2000)) + 12|0);
        $2002 = HEAP32[$2001>>2]|0;
        HEAP32[$n7>>2] = $2002;
        $2003 = HEAP32[$pOp>>2]|0;
        $2004 = ((($2003)) + 16|0);
        $2005 = HEAP32[$2004>>2]|0;
        HEAP32[$pKeyInfo>>2] = $2005;
        $2006 = HEAP32[$pOp>>2]|0;
        $2007 = ((($2006)) + 4|0);
        $2008 = HEAP32[$2007>>2]|0;
        HEAP32[$p19>>2] = $2008;
        $2009 = HEAP32[$pOp>>2]|0;
        $2010 = ((($2009)) + 8|0);
        $2011 = HEAP32[$2010>>2]|0;
        HEAP32[$p210>>2] = $2011;
        HEAP32[$i8>>2] = 0;
        while(1) {
         $2012 = HEAP32[$i8>>2]|0;
         $2013 = HEAP32[$n7>>2]|0;
         $2014 = ($2012|0)<($2013|0);
         if (!($2014)) {
          break;
         }
         $2015 = HEAP32[$aPermute>>2]|0;
         $2016 = ($2015|0)!=(0|0);
         $2017 = HEAP32[$i8>>2]|0;
         if ($2016) {
          $2018 = HEAP32[$aPermute>>2]|0;
          $2019 = (($2018) + ($2017<<2)|0);
          $2020 = HEAP32[$2019>>2]|0;
          $2021 = $2020;
         } else {
          $2021 = $2017;
         }
         HEAP32[$idx>>2] = $2021;
         $2022 = HEAP32[$i8>>2]|0;
         $2023 = HEAP32[$pKeyInfo>>2]|0;
         $2024 = ((($2023)) + 20|0);
         $2025 = (($2024) + ($2022<<2)|0);
         $2026 = HEAP32[$2025>>2]|0;
         HEAP32[$pColl>>2] = $2026;
         $2027 = HEAP32[$i8>>2]|0;
         $2028 = HEAP32[$pKeyInfo>>2]|0;
         $2029 = ((($2028)) + 16|0);
         $2030 = HEAP32[$2029>>2]|0;
         $2031 = (($2030) + ($2027)|0);
         $2032 = HEAP8[$2031>>0]|0;
         $2033 = $2032&255;
         HEAP32[$bRev>>2] = $2033;
         $2034 = HEAP32[$p19>>2]|0;
         $2035 = HEAP32[$idx>>2]|0;
         $2036 = (($2034) + ($2035))|0;
         $2037 = HEAP32[$aMem>>2]|0;
         $2038 = (($2037) + (($2036*40)|0)|0);
         $2039 = HEAP32[$p210>>2]|0;
         $2040 = HEAP32[$idx>>2]|0;
         $2041 = (($2039) + ($2040))|0;
         $2042 = HEAP32[$aMem>>2]|0;
         $2043 = (($2042) + (($2041*40)|0)|0);
         $2044 = HEAP32[$pColl>>2]|0;
         $2045 = (_sqlite3MemCompare($2038,$2043,$2044)|0);
         HEAP32[$iCompare>>2] = $2045;
         $2046 = HEAP32[$iCompare>>2]|0;
         $2047 = ($2046|0)!=(0);
         if ($2047) {
          label = 217;
          break;
         }
         $2052 = HEAP32[$i8>>2]|0;
         $2053 = (($2052) + 1)|0;
         HEAP32[$i8>>2] = $2053;
        }
        if ((label|0) == 217) {
         label = 0;
         $2048 = HEAP32[$bRev>>2]|0;
         $2049 = ($2048|0)!=(0);
         if ($2049) {
          $2050 = HEAP32[$iCompare>>2]|0;
          $2051 = (0 - ($2050))|0;
          HEAP32[$iCompare>>2] = $2051;
         }
        }
        HEAP32[$aPermute>>2] = 0;
        break;
       }
       case 18:  {
        $2054 = HEAP32[$iCompare>>2]|0;
        $2055 = ($2054|0)<(0);
        if ($2055) {
         $2056 = HEAP32[$pOp>>2]|0;
         $2057 = ((($2056)) + 4|0);
         $2058 = HEAP32[$2057>>2]|0;
         $2059 = (($2058) - 1)|0;
         $2060 = HEAP32[$aOp>>2]|0;
         $2061 = (($2060) + (($2059*20)|0)|0);
         HEAP32[$pOp>>2] = $2061;
         break L10;
        }
        $2062 = HEAP32[$iCompare>>2]|0;
        $2063 = ($2062|0)==(0);
        $2064 = HEAP32[$pOp>>2]|0;
        if ($2063) {
         $2065 = ((($2064)) + 8|0);
         $2066 = HEAP32[$2065>>2]|0;
         $2067 = (($2066) - 1)|0;
         $2068 = HEAP32[$aOp>>2]|0;
         $2069 = (($2068) + (($2067*20)|0)|0);
         HEAP32[$pOp>>2] = $2069;
         break L10;
        } else {
         $2070 = ((($2064)) + 12|0);
         $2071 = HEAP32[$2070>>2]|0;
         $2072 = (($2071) - 1)|0;
         $2073 = HEAP32[$aOp>>2]|0;
         $2074 = (($2073) + (($2072*20)|0)|0);
         HEAP32[$pOp>>2] = $2074;
         break L10;
        }
        break;
       }
       case 27: case 28:  {
        $2075 = HEAP32[$pOp>>2]|0;
        $2076 = ((($2075)) + 4|0);
        $2077 = HEAP32[$2076>>2]|0;
        $2078 = HEAP32[$aMem>>2]|0;
        $2079 = (($2078) + (($2077*40)|0)|0);
        HEAP32[$pIn1>>2] = $2079;
        $2080 = HEAP32[$pIn1>>2]|0;
        $2081 = ((($2080)) + 8|0);
        $2082 = HEAP16[$2081>>1]|0;
        $2083 = $2082&65535;
        $2084 = $2083 & 1;
        $2085 = ($2084|0)!=(0);
        if ($2085) {
         HEAP32[$v1>>2] = 2;
        } else {
         $2086 = HEAP32[$pIn1>>2]|0;
         $2087 = (_sqlite3VdbeIntValue($2086)|0);
         $2088 = tempRet0;
         $2089 = ($2087|0)!=(0);
         $2090 = ($2088|0)!=(0);
         $2091 = $2089 | $2090;
         $2092 = $2091&1;
         HEAP32[$v1>>2] = $2092;
        }
        $2093 = HEAP32[$pOp>>2]|0;
        $2094 = ((($2093)) + 8|0);
        $2095 = HEAP32[$2094>>2]|0;
        $2096 = HEAP32[$aMem>>2]|0;
        $2097 = (($2096) + (($2095*40)|0)|0);
        HEAP32[$pIn2>>2] = $2097;
        $2098 = HEAP32[$pIn2>>2]|0;
        $2099 = ((($2098)) + 8|0);
        $2100 = HEAP16[$2099>>1]|0;
        $2101 = $2100&65535;
        $2102 = $2101 & 1;
        $2103 = ($2102|0)!=(0);
        if ($2103) {
         HEAP32[$v2>>2] = 2;
        } else {
         $2104 = HEAP32[$pIn2>>2]|0;
         $2105 = (_sqlite3VdbeIntValue($2104)|0);
         $2106 = tempRet0;
         $2107 = ($2105|0)!=(0);
         $2108 = ($2106|0)!=(0);
         $2109 = $2107 | $2108;
         $2110 = $2109&1;
         HEAP32[$v2>>2] = $2110;
        }
        $2111 = HEAP32[$pOp>>2]|0;
        $2112 = HEAP8[$2111>>0]|0;
        $2113 = $2112&255;
        $2114 = ($2113|0)==(28);
        $2115 = HEAP32[$v1>>2]|0;
        $2116 = ($2115*3)|0;
        $2117 = HEAP32[$v2>>2]|0;
        $2118 = (($2116) + ($2117))|0;
        if ($2114) {
         $2119 = (39925 + ($2118)|0);
         $2120 = HEAP8[$2119>>0]|0;
         $2121 = $2120&255;
         HEAP32[$v1>>2] = $2121;
        } else {
         $2122 = (39934 + ($2118)|0);
         $2123 = HEAP8[$2122>>0]|0;
         $2124 = $2123&255;
         HEAP32[$v1>>2] = $2124;
        }
        $2125 = HEAP32[$pOp>>2]|0;
        $2126 = ((($2125)) + 12|0);
        $2127 = HEAP32[$2126>>2]|0;
        $2128 = HEAP32[$aMem>>2]|0;
        $2129 = (($2128) + (($2127*40)|0)|0);
        HEAP32[$pOut>>2] = $2129;
        $2130 = HEAP32[$v1>>2]|0;
        $2131 = ($2130|0)==(2);
        if ($2131) {
         $2132 = HEAP32[$pOut>>2]|0;
         $2133 = ((($2132)) + 8|0);
         $2134 = HEAP16[$2133>>1]|0;
         $2135 = $2134&65535;
         $2136 = $2135 & -49664;
         $2137 = $2136 | 1;
         $2138 = $2137&65535;
         $2139 = HEAP32[$pOut>>2]|0;
         $2140 = ((($2139)) + 8|0);
         HEAP16[$2140>>1] = $2138;
         break L10;
        } else {
         $2141 = HEAP32[$v1>>2]|0;
         $2142 = ($2141|0)<(0);
         $2143 = $2142 << 31 >> 31;
         $2144 = HEAP32[$pOut>>2]|0;
         $2145 = $2144;
         $2146 = $2145;
         HEAP32[$2146>>2] = $2141;
         $2147 = (($2145) + 4)|0;
         $2148 = $2147;
         HEAP32[$2148>>2] = $2143;
         $2149 = HEAP32[$pOut>>2]|0;
         $2150 = ((($2149)) + 8|0);
         $2151 = HEAP16[$2150>>1]|0;
         $2152 = $2151&65535;
         $2153 = $2152 & -49664;
         $2154 = $2153 | 4;
         $2155 = $2154&65535;
         $2156 = HEAP32[$pOut>>2]|0;
         $2157 = ((($2156)) + 8|0);
         HEAP16[$2157>>1] = $2155;
         break L10;
        }
        break;
       }
       case 19:  {
        $2158 = HEAP32[$pOp>>2]|0;
        $2159 = ((($2158)) + 4|0);
        $2160 = HEAP32[$2159>>2]|0;
        $2161 = HEAP32[$aMem>>2]|0;
        $2162 = (($2161) + (($2160*40)|0)|0);
        HEAP32[$pIn1>>2] = $2162;
        $2163 = HEAP32[$pOp>>2]|0;
        $2164 = ((($2163)) + 8|0);
        $2165 = HEAP32[$2164>>2]|0;
        $2166 = HEAP32[$aMem>>2]|0;
        $2167 = (($2166) + (($2165*40)|0)|0);
        HEAP32[$pOut>>2] = $2167;
        $2168 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($2168);
        $2169 = HEAP32[$pIn1>>2]|0;
        $2170 = ((($2169)) + 8|0);
        $2171 = HEAP16[$2170>>1]|0;
        $2172 = $2171&65535;
        $2173 = $2172 & 1;
        $2174 = ($2173|0)==(0);
        if ($2174) {
         $2175 = HEAP32[$pOut>>2]|0;
         $2176 = ((($2175)) + 8|0);
         HEAP16[$2176>>1] = 4;
         $2177 = HEAP32[$pIn1>>2]|0;
         $2178 = (_sqlite3VdbeIntValue($2177)|0);
         $2179 = tempRet0;
         $2180 = ($2178|0)!=(0);
         $2181 = ($2179|0)!=(0);
         $2182 = $2180 | $2181;
         $2183 = $2182 ^ 1;
         $2184 = $2183&1;
         $2185 = ($2184|0)<(0);
         $2186 = $2185 << 31 >> 31;
         $2187 = HEAP32[$pOut>>2]|0;
         $2188 = $2187;
         $2189 = $2188;
         HEAP32[$2189>>2] = $2184;
         $2190 = (($2188) + 4)|0;
         $2191 = $2190;
         HEAP32[$2191>>2] = $2186;
        }
        break;
       }
       case 54:  {
        $2192 = HEAP32[$pOp>>2]|0;
        $2193 = ((($2192)) + 4|0);
        $2194 = HEAP32[$2193>>2]|0;
        $2195 = HEAP32[$aMem>>2]|0;
        $2196 = (($2195) + (($2194*40)|0)|0);
        HEAP32[$pIn1>>2] = $2196;
        $2197 = HEAP32[$pOp>>2]|0;
        $2198 = ((($2197)) + 8|0);
        $2199 = HEAP32[$2198>>2]|0;
        $2200 = HEAP32[$aMem>>2]|0;
        $2201 = (($2200) + (($2199*40)|0)|0);
        HEAP32[$pOut>>2] = $2201;
        $2202 = HEAP32[$pOut>>2]|0;
        _sqlite3VdbeMemSetNull($2202);
        $2203 = HEAP32[$pIn1>>2]|0;
        $2204 = ((($2203)) + 8|0);
        $2205 = HEAP16[$2204>>1]|0;
        $2206 = $2205&65535;
        $2207 = $2206 & 1;
        $2208 = ($2207|0)==(0);
        if ($2208) {
         $2209 = HEAP32[$pOut>>2]|0;
         $2210 = ((($2209)) + 8|0);
         HEAP16[$2210>>1] = 4;
         $2211 = HEAP32[$pIn1>>2]|0;
         $2212 = (_sqlite3VdbeIntValue($2211)|0);
         $2213 = tempRet0;
         $2214 = $2212 ^ -1;
         $2215 = $2213 ^ -1;
         $2216 = HEAP32[$pOut>>2]|0;
         $2217 = $2216;
         $2218 = $2217;
         HEAP32[$2218>>2] = $2214;
         $2219 = (($2217) + 4)|0;
         $2220 = $2219;
         HEAP32[$2220>>2] = $2215;
        }
        break;
       }
       case 20:  {
        $2221 = HEAP32[$0>>2]|0;
        $2222 = ((($2221)) + 88|0);
        $2223 = HEAP32[$2222>>2]|0;
        $2224 = ((($2223)) + 4|0);
        $2225 = HEAP32[$2224>>2]|0;
        $2226 = HEAP32[$pOp>>2]|0;
        $2227 = ((($2226)) + 4|0);
        $2228 = HEAP32[$2227>>2]|0;
        $2229 = ($2225|0)==($2228|0);
        if ($2229) {
         label = 14;
        } else {
         $2230 = HEAP32[$0>>2]|0;
         $2231 = ((($2230)) + 88|0);
         $2232 = HEAP32[$2231>>2]|0;
         $2233 = ((($2232)) + 4|0);
         $2234 = HEAP32[$2233>>2]|0;
         $2235 = HEAP32[$pOp>>2]|0;
         $2236 = ((($2235)) + 4|0);
         HEAP32[$2236>>2] = $2234;
        }
        break;
       }
       case 22: case 21:  {
        $2237 = HEAP32[$pOp>>2]|0;
        $2238 = ((($2237)) + 4|0);
        $2239 = HEAP32[$2238>>2]|0;
        $2240 = HEAP32[$aMem>>2]|0;
        $2241 = (($2240) + (($2239*40)|0)|0);
        HEAP32[$pIn1>>2] = $2241;
        $2242 = HEAP32[$pIn1>>2]|0;
        $2243 = ((($2242)) + 8|0);
        $2244 = HEAP16[$2243>>1]|0;
        $2245 = $2244&65535;
        $2246 = $2245 & 1;
        $2247 = ($2246|0)!=(0);
        if ($2247) {
         $2248 = HEAP32[$pOp>>2]|0;
         $2249 = ((($2248)) + 12|0);
         $2250 = HEAP32[$2249>>2]|0;
         HEAP32[$c>>2] = $2250;
        } else {
         $2251 = HEAP32[$pIn1>>2]|0;
         $2252 = (+_sqlite3VdbeRealValue($2251));
         $2253 = $2252 != 0.0;
         $2254 = $2253&1;
         HEAP32[$c>>2] = $2254;
         $2255 = HEAP32[$pOp>>2]|0;
         $2256 = HEAP8[$2255>>0]|0;
         $2257 = $2256&255;
         $2258 = ($2257|0)==(22);
         if ($2258) {
          $2259 = HEAP32[$c>>2]|0;
          $2260 = ($2259|0)!=(0);
          $2261 = $2260 ^ 1;
          $2262 = $2261&1;
          HEAP32[$c>>2] = $2262;
         }
        }
        $2263 = HEAP32[$c>>2]|0;
        $2264 = ($2263|0)!=(0);
        if ($2264) {
         label = 14;
        }
        break;
       }
       case 34:  {
        $2265 = HEAP32[$pOp>>2]|0;
        $2266 = ((($2265)) + 4|0);
        $2267 = HEAP32[$2266>>2]|0;
        $2268 = HEAP32[$aMem>>2]|0;
        $2269 = (($2268) + (($2267*40)|0)|0);
        HEAP32[$pIn1>>2] = $2269;
        $2270 = HEAP32[$pIn1>>2]|0;
        $2271 = ((($2270)) + 8|0);
        $2272 = HEAP16[$2271>>1]|0;
        $2273 = $2272&65535;
        $2274 = $2273 & 1;
        $2275 = ($2274|0)!=(0);
        if ($2275) {
         label = 14;
        }
        break;
       }
       case 35:  {
        $2276 = HEAP32[$pOp>>2]|0;
        $2277 = ((($2276)) + 4|0);
        $2278 = HEAP32[$2277>>2]|0;
        $2279 = HEAP32[$aMem>>2]|0;
        $2280 = (($2279) + (($2278*40)|0)|0);
        HEAP32[$pIn1>>2] = $2280;
        $2281 = HEAP32[$pIn1>>2]|0;
        $2282 = ((($2281)) + 8|0);
        $2283 = HEAP16[$2282>>1]|0;
        $2284 = $2283&65535;
        $2285 = $2284 & 1;
        $2286 = ($2285|0)==(0);
        if ($2286) {
         label = 14;
        }
        break;
       }
       case 96:  {
        $2287 = HEAP32[$pOp>>2]|0;
        $2288 = ((($2287)) + 4|0);
        $2289 = HEAP32[$2288>>2]|0;
        $2290 = HEAP32[$0>>2]|0;
        $2291 = ((($2290)) + 112|0);
        $2292 = HEAP32[$2291>>2]|0;
        $2293 = (($2292) + ($2289<<2)|0);
        $2294 = HEAP32[$2293>>2]|0;
        HEAP32[$pC>>2] = $2294;
        $2295 = HEAP32[$pOp>>2]|0;
        $2296 = ((($2295)) + 8|0);
        $2297 = HEAP32[$2296>>2]|0;
        HEAP32[$p211>>2] = $2297;
        $2298 = (_sqlite3VdbeCursorMoveto($pC,$p211)|0);
        HEAP32[$rc>>2] = $2298;
        $2299 = HEAP32[$rc>>2]|0;
        $2300 = ($2299|0)!=(0);
        if ($2300) {
         break L1;
        }
        $2301 = HEAP32[$pOp>>2]|0;
        $2302 = ((($2301)) + 12|0);
        $2303 = HEAP32[$2302>>2]|0;
        $2304 = HEAP32[$aMem>>2]|0;
        $2305 = (($2304) + (($2303*40)|0)|0);
        HEAP32[$pDest>>2] = $2305;
        $2306 = HEAP32[$pC>>2]|0;
        $2307 = ((($2306)) + 76|0);
        $2308 = HEAP32[$2307>>2]|0;
        HEAP32[$aOffset>>2] = $2308;
        $2309 = HEAP32[$pC>>2]|0;
        $2310 = ((($2309)) + 16|0);
        $2311 = HEAP32[$2310>>2]|0;
        HEAP32[$pCrsr>>2] = $2311;
        $2312 = HEAP32[$pC>>2]|0;
        $2313 = ((($2312)) + 56|0);
        $2314 = HEAP32[$2313>>2]|0;
        $2315 = HEAP32[$0>>2]|0;
        $2316 = ((($2315)) + 32|0);
        $2317 = HEAP32[$2316>>2]|0;
        $2318 = ($2314|0)!=($2317|0);
        do {
         if ($2318) {
          $2319 = HEAP32[$pC>>2]|0;
          $2320 = ((($2319)) + 2|0);
          $2321 = HEAP8[$2320>>0]|0;
          $2322 = ($2321<<24>>24)!=(0);
          do {
           if ($2322) {
            $2323 = HEAP32[$pC>>2]|0;
            $2324 = HEAP8[$2323>>0]|0;
            $2325 = $2324&255;
            $2326 = ($2325|0)==(3);
            if ($2326) {
             $2327 = HEAP32[$pC>>2]|0;
             $2328 = ((($2327)) + 16|0);
             $2329 = HEAP32[$2328>>2]|0;
             $2330 = HEAP32[$aMem>>2]|0;
             $2331 = (($2330) + (($2329*40)|0)|0);
             HEAP32[$pReg>>2] = $2331;
             $2332 = HEAP32[$pReg>>2]|0;
             $2333 = ((($2332)) + 12|0);
             $2334 = HEAP32[$2333>>2]|0;
             HEAP32[$avail>>2] = $2334;
             $2335 = HEAP32[$pC>>2]|0;
             $2336 = ((($2335)) + 64|0);
             HEAP32[$2336>>2] = $2334;
             $2337 = HEAP32[$pC>>2]|0;
             $2338 = ((($2337)) + 60|0);
             HEAP32[$2338>>2] = $2334;
             $2339 = HEAP32[$pReg>>2]|0;
             $2340 = ((($2339)) + 16|0);
             $2341 = HEAP32[$2340>>2]|0;
             $2342 = HEAP32[$pC>>2]|0;
             $2343 = ((($2342)) + 72|0);
             HEAP32[$2343>>2] = $2341;
             break;
            } else {
             $2344 = HEAP32[$pDest>>2]|0;
             _sqlite3VdbeMemSetNull($2344);
             break L10;
            }
           } else {
            $2345 = HEAP32[$pCrsr>>2]|0;
            $2346 = (_sqlite3BtreePayloadSize($2345)|0);
            $2347 = HEAP32[$pC>>2]|0;
            $2348 = ((($2347)) + 60|0);
            HEAP32[$2348>>2] = $2346;
            $2349 = HEAP32[$pCrsr>>2]|0;
            $2350 = (_sqlite3BtreePayloadFetch($2349,$avail)|0);
            $2351 = HEAP32[$pC>>2]|0;
            $2352 = ((($2351)) + 72|0);
            HEAP32[$2352>>2] = $2350;
            $2353 = HEAP32[$pC>>2]|0;
            $2354 = ((($2353)) + 60|0);
            $2355 = HEAP32[$2354>>2]|0;
            $2356 = HEAP32[$avail>>2]|0;
            $2357 = ($2355>>>0)<=($2356>>>0);
            $2358 = HEAP32[$pC>>2]|0;
            $2359 = ((($2358)) + 60|0);
            $2360 = HEAP32[$2359>>2]|0;
            if ($2357) {
             $2361 = HEAP32[$pC>>2]|0;
             $2362 = ((($2361)) + 64|0);
             HEAP32[$2362>>2] = $2360;
             break;
            }
            $2363 = HEAP32[$db>>2]|0;
            $2364 = ((($2363)) + 96|0);
            $2365 = HEAP32[$2364>>2]|0;
            $2366 = ($2360>>>0)>($2365>>>0);
            if ($2366) {
             label = 888;
             break L8;
            }
            $2367 = HEAP32[$avail>>2]|0;
            $2368 = HEAP32[$pC>>2]|0;
            $2369 = ((($2368)) + 64|0);
            HEAP32[$2369>>2] = $2367;
           }
          } while(0);
          $2370 = HEAP32[$0>>2]|0;
          $2371 = ((($2370)) + 32|0);
          $2372 = HEAP32[$2371>>2]|0;
          $2373 = HEAP32[$pC>>2]|0;
          $2374 = ((($2373)) + 56|0);
          HEAP32[$2374>>2] = $2372;
          $2375 = HEAP32[$pC>>2]|0;
          $2376 = ((($2375)) + 72|0);
          $2377 = HEAP32[$2376>>2]|0;
          $2378 = HEAP8[$2377>>0]|0;
          $2379 = $2378&255;
          $2380 = ($2379|0)<(128);
          $2381 = HEAP32[$pC>>2]|0;
          $2382 = ((($2381)) + 72|0);
          $2383 = HEAP32[$2382>>2]|0;
          if ($2380) {
           $2384 = HEAP8[$2383>>0]|0;
           $2385 = $2384&255;
           HEAP32[$offset>>2] = $2385;
           $2389 = 1;
          } else {
           $2386 = (_sqlite3GetVarint32($2383,$offset)|0);
           $2387 = $2386&255;
           $2389 = $2387;
          }
          $2388 = $2389&255;
          $2390 = $2388&255;
          $2391 = HEAP32[$pC>>2]|0;
          $2392 = ((($2391)) + 68|0);
          HEAP32[$2392>>2] = $2390;
          $2393 = HEAP32[$pC>>2]|0;
          $2394 = ((($2393)) + 14|0);
          HEAP16[$2394>>1] = 0;
          $2395 = HEAP32[$offset>>2]|0;
          $2396 = HEAP32[$aOffset>>2]|0;
          HEAP32[$2396>>2] = $2395;
          $2397 = HEAP32[$avail>>2]|0;
          $2398 = HEAP32[$offset>>2]|0;
          $2399 = ($2397>>>0)<($2398>>>0);
          if ($2399) {
           $2400 = HEAP32[$pC>>2]|0;
           $2401 = ((($2400)) + 72|0);
           HEAP32[$2401>>2] = 0;
           $2402 = HEAP32[$pC>>2]|0;
           $2403 = ((($2402)) + 64|0);
           HEAP32[$2403>>2] = 0;
           $2404 = HEAP32[$offset>>2]|0;
           $2405 = ($2404>>>0)>(98307);
           if ($2405) {
            label = 267;
            break L8;
           }
           $2406 = HEAP32[$offset>>2]|0;
           $2407 = HEAP32[$pC>>2]|0;
           $2408 = ((($2407)) + 60|0);
           $2409 = HEAP32[$2408>>2]|0;
           $2410 = ($2406>>>0)>($2409>>>0);
           if ($2410) {
            label = 267;
            break L8;
           } else {
            label = 270;
            break;
           }
          } else {
           $2412 = HEAP32[$offset>>2]|0;
           $2413 = ($2412>>>0)>(0);
           if (!($2413)) {
            label = 270;
            break;
           }
           $2414 = HEAP32[$pC>>2]|0;
           $2415 = ((($2414)) + 72|0);
           $2416 = HEAP32[$2415>>2]|0;
           HEAP32[$zData>>2] = $2416;
           label = 276;
           break;
          }
         } else {
          label = 270;
         }
        } while(0);
        do {
         if ((label|0) == 270) {
          label = 0;
          $2417 = HEAP32[$pC>>2]|0;
          $2418 = ((($2417)) + 14|0);
          $2419 = HEAP16[$2418>>1]|0;
          $2420 = $2419&65535;
          $2421 = HEAP32[$p211>>2]|0;
          $2422 = ($2420|0)<=($2421|0);
          if (!($2422)) {
           $2599 = HEAP32[$p211>>2]|0;
           $2600 = HEAP32[$pC>>2]|0;
           $2601 = ((($2600)) + 80|0);
           $2602 = (($2601) + ($2599<<2)|0);
           $2603 = HEAP32[$2602>>2]|0;
           HEAP32[$t>>2] = $2603;
           break;
          }
          $2423 = HEAP32[$pC>>2]|0;
          $2424 = ((($2423)) + 68|0);
          $2425 = HEAP32[$2424>>2]|0;
          $2426 = HEAP32[$aOffset>>2]|0;
          $2427 = HEAP32[$2426>>2]|0;
          $2428 = ($2425>>>0)<($2427>>>0);
          if (!($2428)) {
           HEAP32[$t>>2] = 0;
           label = 292;
           break;
          }
          $2429 = HEAP32[$pC>>2]|0;
          $2430 = ((($2429)) + 72|0);
          $2431 = HEAP32[$2430>>2]|0;
          $2432 = ($2431|0)==(0|0);
          if (!($2432)) {
           $2447 = HEAP32[$pC>>2]|0;
           $2448 = ((($2447)) + 72|0);
           $2449 = HEAP32[$2448>>2]|0;
           HEAP32[$zData>>2] = $2449;
           label = 276;
           break;
          }
          dest=$sMem; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
          $2433 = HEAP32[$pCrsr>>2]|0;
          $2434 = HEAP32[$aOffset>>2]|0;
          $2435 = HEAP32[$2434>>2]|0;
          $2436 = HEAP32[$pC>>2]|0;
          $2437 = ((($2436)) + 4|0);
          $2438 = HEAP8[$2437>>0]|0;
          $2439 = ($2438<<24>>24)!=(0);
          $2440 = $2439 ^ 1;
          $2441 = $2440&1;
          $2442 = (_sqlite3VdbeMemFromBtree($2433,0,$2435,$2441,$sMem)|0);
          HEAP32[$rc>>2] = $2442;
          $2443 = HEAP32[$rc>>2]|0;
          $2444 = ($2443|0)!=(0);
          if ($2444) {
           break L1;
          }
          $2445 = ((($sMem)) + 16|0);
          $2446 = HEAP32[$2445>>2]|0;
          HEAP32[$zData>>2] = $2446;
          label = 276;
         }
        } while(0);
        if ((label|0) == 276) {
         label = 0;
         $2450 = HEAP32[$pC>>2]|0;
         $2451 = ((($2450)) + 14|0);
         $2452 = HEAP16[$2451>>1]|0;
         $2453 = $2452&65535;
         HEAP32[$i12>>2] = $2453;
         $2454 = HEAP32[$i12>>2]|0;
         $2455 = HEAP32[$aOffset>>2]|0;
         $2456 = (($2455) + ($2454<<2)|0);
         $2457 = HEAP32[$2456>>2]|0;
         $2458 = $offset64;
         $2459 = $2458;
         HEAP32[$2459>>2] = $2457;
         $2460 = (($2458) + 4)|0;
         $2461 = $2460;
         HEAP32[$2461>>2] = 0;
         $2462 = HEAP32[$zData>>2]|0;
         $2463 = HEAP32[$pC>>2]|0;
         $2464 = ((($2463)) + 68|0);
         $2465 = HEAP32[$2464>>2]|0;
         $2466 = (($2462) + ($2465)|0);
         HEAP32[$zHdr>>2] = $2466;
         $2467 = HEAP32[$zData>>2]|0;
         $2468 = HEAP32[$aOffset>>2]|0;
         $2469 = HEAP32[$2468>>2]|0;
         $2470 = (($2467) + ($2469)|0);
         HEAP32[$zEndHdr>>2] = $2470;
         while(1) {
          $2471 = HEAP32[$zHdr>>2]|0;
          $2472 = HEAP8[$2471>>0]|0;
          $2473 = $2472&255;
          HEAP32[$t>>2] = $2473;
          $2474 = ($2473>>>0)<(128);
          $2475 = HEAP32[$zHdr>>2]|0;
          if ($2474) {
           $2476 = ((($2475)) + 1|0);
           HEAP32[$zHdr>>2] = $2476;
           $2477 = HEAP32[$t>>2]|0;
           $2478 = $2477&255;
           $2479 = (_sqlite3VdbeOneByteSerialTypeLen($2478)|0);
           $2480 = $2479&255;
           $2481 = $offset64;
           $2482 = $2481;
           $2483 = HEAP32[$2482>>2]|0;
           $2484 = (($2481) + 4)|0;
           $2485 = $2484;
           $2486 = HEAP32[$2485>>2]|0;
           $2487 = (_i64Add(($2483|0),($2486|0),($2480|0),0)|0);
           $2488 = tempRet0;
           $2489 = $offset64;
           $2490 = $2489;
           HEAP32[$2490>>2] = $2487;
           $2491 = (($2489) + 4)|0;
           $2492 = $2491;
           HEAP32[$2492>>2] = $2488;
          } else {
           $2493 = (_sqlite3GetVarint32($2475,$t)|0);
           $2494 = $2493&255;
           $2495 = HEAP32[$zHdr>>2]|0;
           $2496 = (($2495) + ($2494)|0);
           HEAP32[$zHdr>>2] = $2496;
           $2497 = HEAP32[$t>>2]|0;
           $2498 = (_sqlite3VdbeSerialTypeLen($2497)|0);
           $2499 = $offset64;
           $2500 = $2499;
           $2501 = HEAP32[$2500>>2]|0;
           $2502 = (($2499) + 4)|0;
           $2503 = $2502;
           $2504 = HEAP32[$2503>>2]|0;
           $2505 = (_i64Add(($2501|0),($2504|0),($2498|0),0)|0);
           $2506 = tempRet0;
           $2507 = $offset64;
           $2508 = $2507;
           HEAP32[$2508>>2] = $2505;
           $2509 = (($2507) + 4)|0;
           $2510 = $2509;
           HEAP32[$2510>>2] = $2506;
          }
          $2511 = HEAP32[$t>>2]|0;
          $2512 = HEAP32[$i12>>2]|0;
          $2513 = (($2512) + 1)|0;
          HEAP32[$i12>>2] = $2513;
          $2514 = HEAP32[$pC>>2]|0;
          $2515 = ((($2514)) + 80|0);
          $2516 = (($2515) + ($2512<<2)|0);
          HEAP32[$2516>>2] = $2511;
          $2517 = $offset64;
          $2518 = $2517;
          $2519 = HEAP32[$2518>>2]|0;
          $2520 = (($2517) + 4)|0;
          $2521 = $2520;
          $2522 = HEAP32[$2521>>2]|0;
          $2523 = HEAP32[$i12>>2]|0;
          $2524 = HEAP32[$aOffset>>2]|0;
          $2525 = (($2524) + ($2523<<2)|0);
          HEAP32[$2525>>2] = $2519;
          $2526 = HEAP32[$i12>>2]|0;
          $2527 = HEAP32[$p211>>2]|0;
          $2528 = ($2526|0)<=($2527|0);
          if (!($2528)) {
           break;
          }
          $2529 = HEAP32[$zHdr>>2]|0;
          $2530 = HEAP32[$zEndHdr>>2]|0;
          $2531 = ($2529>>>0)<($2530>>>0);
          if (!($2531)) {
           break;
          }
         }
         $2532 = HEAP32[$zHdr>>2]|0;
         $2533 = HEAP32[$zEndHdr>>2]|0;
         $2534 = ($2532>>>0)>=($2533>>>0);
         if ($2534) {
          $2535 = HEAP32[$zHdr>>2]|0;
          $2536 = HEAP32[$zEndHdr>>2]|0;
          $2537 = ($2535>>>0)>($2536>>>0);
          if ($2537) {
           label = 286;
           break L8;
          }
          $2538 = $offset64;
          $2539 = $2538;
          $2540 = HEAP32[$2539>>2]|0;
          $2541 = (($2538) + 4)|0;
          $2542 = $2541;
          $2543 = HEAP32[$2542>>2]|0;
          $2544 = HEAP32[$pC>>2]|0;
          $2545 = ((($2544)) + 60|0);
          $2546 = HEAP32[$2545>>2]|0;
          $2547 = ($2540|0)!=($2546|0);
          $2548 = ($2543|0)!=(0);
          $2549 = $2547 | $2548;
          if ($2549) {
           label = 286;
           break L8;
          }
         }
         $2550 = $offset64;
         $2551 = $2550;
         $2552 = HEAP32[$2551>>2]|0;
         $2553 = (($2550) + 4)|0;
         $2554 = $2553;
         $2555 = HEAP32[$2554>>2]|0;
         $2556 = HEAP32[$pC>>2]|0;
         $2557 = ((($2556)) + 60|0);
         $2558 = HEAP32[$2557>>2]|0;
         $2559 = ($2555>>>0)>(0);
         $2560 = ($2552>>>0)>($2558>>>0);
         $2561 = ($2555|0)==(0);
         $2562 = $2561 & $2560;
         $2563 = $2559 | $2562;
         if ($2563) {
          label = 286;
          break L8;
         }
         $2569 = HEAP32[$i12>>2]|0;
         $2570 = $2569&65535;
         $2571 = HEAP32[$pC>>2]|0;
         $2572 = ((($2571)) + 14|0);
         HEAP16[$2572>>1] = $2570;
         $2573 = HEAP32[$zHdr>>2]|0;
         $2574 = HEAP32[$zData>>2]|0;
         $2575 = $2573;
         $2576 = $2574;
         $2577 = (($2575) - ($2576))|0;
         $2578 = HEAP32[$pC>>2]|0;
         $2579 = ((($2578)) + 68|0);
         HEAP32[$2579>>2] = $2577;
         $2580 = HEAP32[$pC>>2]|0;
         $2581 = ((($2580)) + 72|0);
         $2582 = HEAP32[$2581>>2]|0;
         $2583 = ($2582|0)==(0|0);
         if ($2583) {
          _sqlite3VdbeMemRelease($sMem);
          label = 292;
         } else {
          label = 292;
         }
        }
        if ((label|0) == 292) {
         label = 0;
         $2584 = HEAP32[$pC>>2]|0;
         $2585 = ((($2584)) + 14|0);
         $2586 = HEAP16[$2585>>1]|0;
         $2587 = $2586&65535;
         $2588 = HEAP32[$p211>>2]|0;
         $2589 = ($2587|0)<=($2588|0);
         if ($2589) {
          $2590 = HEAP32[$pOp>>2]|0;
          $2591 = ((($2590)) + 1|0);
          $2592 = HEAP8[$2591>>0]|0;
          $2593 = $2592 << 24 >> 24;
          $2594 = ($2593|0)==(-8);
          $2595 = HEAP32[$pDest>>2]|0;
          if ($2594) {
           $2596 = HEAP32[$pOp>>2]|0;
           $2597 = ((($2596)) + 16|0);
           $2598 = HEAP32[$2597>>2]|0;
           _sqlite3VdbeMemShallowCopy($2595,$2598,2048);
           break L10;
          } else {
           _sqlite3VdbeMemSetNull($2595);
           break L10;
          }
         }
        }
        $2604 = HEAP32[$pDest>>2]|0;
        $2605 = ((($2604)) + 8|0);
        $2606 = HEAP16[$2605>>1]|0;
        $2607 = $2606&65535;
        $2608 = $2607 & 9312;
        $2609 = ($2608|0)!=(0);
        if ($2609) {
         $2610 = HEAP32[$pDest>>2]|0;
         _sqlite3VdbeMemSetNull($2610);
        }
        $2611 = HEAP32[$pC>>2]|0;
        $2612 = ((($2611)) + 64|0);
        $2613 = HEAP32[$2612>>2]|0;
        $2614 = HEAP32[$p211>>2]|0;
        $2615 = (($2614) + 1)|0;
        $2616 = HEAP32[$aOffset>>2]|0;
        $2617 = (($2616) + ($2615<<2)|0);
        $2618 = HEAP32[$2617>>2]|0;
        $2619 = ($2613>>>0)>=($2618>>>0);
        if ($2619) {
         $2620 = HEAP32[$pC>>2]|0;
         $2621 = ((($2620)) + 72|0);
         $2622 = HEAP32[$2621>>2]|0;
         $2623 = HEAP32[$p211>>2]|0;
         $2624 = HEAP32[$aOffset>>2]|0;
         $2625 = (($2624) + ($2623<<2)|0);
         $2626 = HEAP32[$2625>>2]|0;
         $2627 = (($2622) + ($2626)|0);
         HEAP32[$zData>>2] = $2627;
         $2628 = HEAP32[$t>>2]|0;
         $2629 = ($2628>>>0)<(12);
         if ($2629) {
          $2630 = HEAP32[$zData>>2]|0;
          $2631 = HEAP32[$t>>2]|0;
          $2632 = HEAP32[$pDest>>2]|0;
          (_sqlite3VdbeSerialGet($2630,$2631,$2632)|0);
          break L10;
         }
         $2633 = HEAP32[$t>>2]|0;
         $2634 = (($2633) - 12)|0;
         $2635 = (($2634>>>0) / 2)&-1;
         HEAP32[$len>>2] = $2635;
         $2636 = HEAP32[$pDest>>2]|0;
         $2637 = ((($2636)) + 12|0);
         HEAP32[$2637>>2] = $2635;
         $2638 = HEAP8[$encoding>>0]|0;
         $2639 = HEAP32[$pDest>>2]|0;
         $2640 = ((($2639)) + 10|0);
         HEAP8[$2640>>0] = $2638;
         $2641 = HEAP32[$pDest>>2]|0;
         $2642 = ((($2641)) + 24|0);
         $2643 = HEAP32[$2642>>2]|0;
         $2644 = HEAP32[$len>>2]|0;
         $2645 = (($2644) + 2)|0;
         $2646 = ($2643|0)<($2645|0);
         $2647 = HEAP32[$pDest>>2]|0;
         if ($2646) {
          $2648 = ((($2647)) + 8|0);
          HEAP16[$2648>>1] = 1;
          $2649 = HEAP32[$pDest>>2]|0;
          $2650 = HEAP32[$len>>2]|0;
          $2651 = (($2650) + 2)|0;
          $2652 = (_sqlite3VdbeMemGrow($2649,$2651,0)|0);
          $2653 = ($2652|0)!=(0);
          if ($2653) {
           label = 889;
           break L1;
          }
         } else {
          $2654 = ((($2647)) + 20|0);
          $2655 = HEAP32[$2654>>2]|0;
          $2656 = HEAP32[$pDest>>2]|0;
          $2657 = ((($2656)) + 16|0);
          HEAP32[$2657>>2] = $2655;
         }
         $2658 = HEAP32[$pDest>>2]|0;
         $2659 = ((($2658)) + 16|0);
         $2660 = HEAP32[$2659>>2]|0;
         $2661 = HEAP32[$zData>>2]|0;
         $2662 = HEAP32[$len>>2]|0;
         _memcpy(($2660|0),($2661|0),($2662|0))|0;
         $2663 = HEAP32[$len>>2]|0;
         $2664 = HEAP32[$pDest>>2]|0;
         $2665 = ((($2664)) + 16|0);
         $2666 = HEAP32[$2665>>2]|0;
         $2667 = (($2666) + ($2663)|0);
         HEAP8[$2667>>0] = 0;
         $2668 = HEAP32[$len>>2]|0;
         $2669 = (($2668) + 1)|0;
         $2670 = HEAP32[$pDest>>2]|0;
         $2671 = ((($2670)) + 16|0);
         $2672 = HEAP32[$2671>>2]|0;
         $2673 = (($2672) + ($2669)|0);
         HEAP8[$2673>>0] = 0;
         $2674 = HEAP32[$t>>2]|0;
         $2675 = $2674 & 1;
         $2676 = (16418 + ($2675<<1)|0);
         $2677 = HEAP16[$2676>>1]|0;
         $2678 = HEAP32[$pDest>>2]|0;
         $2679 = ((($2678)) + 8|0);
         HEAP16[$2679>>1] = $2677;
         break L10;
        }
        $2680 = HEAP8[$encoding>>0]|0;
        $2681 = HEAP32[$pDest>>2]|0;
        $2682 = ((($2681)) + 10|0);
        HEAP8[$2682>>0] = $2680;
        $2683 = HEAP32[$pOp>>2]|0;
        $2684 = ((($2683)) + 3|0);
        $2685 = HEAP8[$2684>>0]|0;
        $2686 = $2685&255;
        $2687 = $2686 & 192;
        $2688 = ($2687|0)!=(0);
        do {
         if ($2688) {
          $2689 = HEAP32[$t>>2]|0;
          $2690 = ($2689>>>0)>=(12);
          if ($2690) {
           $2691 = HEAP32[$t>>2]|0;
           $2692 = $2691 & 1;
           $2693 = ($2692|0)==(0);
           if ($2693) {
            break;
           }
          }
          $2694 = HEAP32[$pOp>>2]|0;
          $2695 = ((($2694)) + 3|0);
          $2696 = HEAP8[$2695>>0]|0;
          $2697 = $2696&255;
          $2698 = $2697 & 128;
          $2699 = ($2698|0)!=(0);
          if (!($2699)) {
           label = 310;
          }
         } else {
          label = 310;
         }
        } while(0);
        if ((label|0) == 310) {
         label = 0;
         $2700 = HEAP32[$t>>2]|0;
         $2701 = (_sqlite3VdbeSerialTypeLen($2700)|0);
         HEAP32[$len>>2] = $2701;
         $2702 = ($2701|0)==(0);
         if (!($2702)) {
          $2705 = HEAP32[$pCrsr>>2]|0;
          $2706 = HEAP32[$p211>>2]|0;
          $2707 = HEAP32[$aOffset>>2]|0;
          $2708 = (($2707) + ($2706<<2)|0);
          $2709 = HEAP32[$2708>>2]|0;
          $2710 = HEAP32[$len>>2]|0;
          $2711 = HEAP32[$pC>>2]|0;
          $2712 = ((($2711)) + 4|0);
          $2713 = HEAP8[$2712>>0]|0;
          $2714 = ($2713<<24>>24)!=(0);
          $2715 = $2714 ^ 1;
          $2716 = $2715&1;
          $2717 = HEAP32[$pDest>>2]|0;
          $2718 = (_sqlite3VdbeMemFromBtree($2705,$2709,$2710,$2716,$2717)|0);
          HEAP32[$rc>>2] = $2718;
          $2719 = HEAP32[$rc>>2]|0;
          $2720 = ($2719|0)!=(0);
          if ($2720) {
           break L1;
          }
          $2721 = HEAP32[$pDest>>2]|0;
          $2722 = ((($2721)) + 16|0);
          $2723 = HEAP32[$2722>>2]|0;
          $2724 = HEAP32[$t>>2]|0;
          $2725 = HEAP32[$pDest>>2]|0;
          (_sqlite3VdbeSerialGet($2723,$2724,$2725)|0);
          $2726 = HEAP32[$pDest>>2]|0;
          $2727 = ((($2726)) + 8|0);
          $2728 = HEAP16[$2727>>1]|0;
          $2729 = $2728&65535;
          $2730 = $2729 & -4097;
          $2731 = $2730&65535;
          HEAP16[$2727>>1] = $2731;
          break L10;
         }
        }
        $2703 = HEAP32[$t>>2]|0;
        $2704 = HEAP32[$pDest>>2]|0;
        (_sqlite3VdbeSerialGet(39943,$2703,$2704)|0);
        break;
       }
       case 98:  {
        $2732 = HEAP32[$pOp>>2]|0;
        $2733 = ((($2732)) + 16|0);
        $2734 = HEAP32[$2733>>2]|0;
        HEAP32[$zAffinity>>2] = $2734;
        $2735 = HEAP32[$pOp>>2]|0;
        $2736 = ((($2735)) + 4|0);
        $2737 = HEAP32[$2736>>2]|0;
        $2738 = HEAP32[$aMem>>2]|0;
        $2739 = (($2738) + (($2737*40)|0)|0);
        HEAP32[$pIn1>>2] = $2739;
        while(1) {
         $2740 = HEAP32[$zAffinity>>2]|0;
         $2741 = ((($2740)) + 1|0);
         HEAP32[$zAffinity>>2] = $2741;
         $2742 = HEAP8[$2740>>0]|0;
         HEAP8[$cAff>>0] = $2742;
         $2743 = $2742 << 24 >> 24;
         $2744 = ($2743|0)!=(0);
         if (!($2744)) {
          break L10;
         }
         $2745 = HEAP32[$pIn1>>2]|0;
         $2746 = HEAP8[$cAff>>0]|0;
         $2747 = HEAP8[$encoding>>0]|0;
         _applyAffinity($2745,$2746,$2747);
         $2748 = HEAP32[$pIn1>>2]|0;
         $2749 = ((($2748)) + 40|0);
         HEAP32[$pIn1>>2] = $2749;
        }
        break;
       }
       case 99:  {
        $2750 = $nData;
        $2751 = $2750;
        HEAP32[$2751>>2] = 0;
        $2752 = (($2750) + 4)|0;
        $2753 = $2752;
        HEAP32[$2753>>2] = 0;
        HEAP32[$nHdr>>2] = 0;
        $2754 = $nZero;
        $2755 = $2754;
        HEAP32[$2755>>2] = 0;
        $2756 = (($2754) + 4)|0;
        $2757 = $2756;
        HEAP32[$2757>>2] = 0;
        $2758 = HEAP32[$pOp>>2]|0;
        $2759 = ((($2758)) + 4|0);
        $2760 = HEAP32[$2759>>2]|0;
        HEAP32[$nField>>2] = $2760;
        $2761 = HEAP32[$pOp>>2]|0;
        $2762 = ((($2761)) + 16|0);
        $2763 = HEAP32[$2762>>2]|0;
        HEAP32[$zAffinity14>>2] = $2763;
        $2764 = HEAP32[$nField>>2]|0;
        $2765 = HEAP32[$aMem>>2]|0;
        $2766 = (($2765) + (($2764*40)|0)|0);
        HEAP32[$pData0>>2] = $2766;
        $2767 = HEAP32[$pOp>>2]|0;
        $2768 = ((($2767)) + 8|0);
        $2769 = HEAP32[$2768>>2]|0;
        HEAP32[$nField>>2] = $2769;
        $2770 = HEAP32[$nField>>2]|0;
        $2771 = (($2770) - 1)|0;
        $2772 = HEAP32[$pData0>>2]|0;
        $2773 = (($2772) + (($2771*40)|0)|0);
        HEAP32[$pLast>>2] = $2773;
        $2774 = HEAP32[$0>>2]|0;
        $2775 = ((($2774)) + 143|0);
        $2776 = HEAP8[$2775>>0]|0;
        $2777 = $2776&255;
        HEAP32[$file_format>>2] = $2777;
        $2778 = HEAP32[$pOp>>2]|0;
        $2779 = ((($2778)) + 12|0);
        $2780 = HEAP32[$2779>>2]|0;
        $2781 = HEAP32[$aMem>>2]|0;
        $2782 = (($2781) + (($2780*40)|0)|0);
        HEAP32[$pOut>>2] = $2782;
        $2783 = HEAP32[$zAffinity14>>2]|0;
        $2784 = ($2783|0)!=(0|0);
        if ($2784) {
         $2785 = HEAP32[$pData0>>2]|0;
         HEAP32[$pRec>>2] = $2785;
         while(1) {
          $2786 = HEAP32[$pRec>>2]|0;
          $2787 = ((($2786)) + 40|0);
          HEAP32[$pRec>>2] = $2787;
          $2788 = HEAP32[$zAffinity14>>2]|0;
          $2789 = ((($2788)) + 1|0);
          HEAP32[$zAffinity14>>2] = $2789;
          $2790 = HEAP8[$2788>>0]|0;
          $2791 = HEAP8[$encoding>>0]|0;
          _applyAffinity($2786,$2790,$2791);
          $2792 = HEAP32[$zAffinity14>>2]|0;
          $2793 = HEAP8[$2792>>0]|0;
          $2794 = ($2793<<24>>24)!=(0);
          if (!($2794)) {
           break;
          }
         }
        }
        $2795 = HEAP32[$pLast>>2]|0;
        HEAP32[$pRec>>2] = $2795;
        while(1) {
         $2796 = HEAP32[$pRec>>2]|0;
         $2797 = HEAP32[$file_format>>2]|0;
         $2798 = (_sqlite3VdbeSerialType($2796,$2797,$len16)|0);
         HEAP32[$serial_type>>2] = $2798;
         $2799 = HEAP32[$pRec>>2]|0;
         $2800 = ((($2799)) + 28|0);
         HEAP32[$2800>>2] = $2798;
         $2801 = HEAP32[$pRec>>2]|0;
         $2802 = ((($2801)) + 8|0);
         $2803 = HEAP16[$2802>>1]|0;
         $2804 = $2803&65535;
         $2805 = $2804 & 16384;
         $2806 = ($2805|0)!=(0);
         do {
          if ($2806) {
           $2807 = $nData;
           $2808 = $2807;
           $2809 = HEAP32[$2808>>2]|0;
           $2810 = (($2807) + 4)|0;
           $2811 = $2810;
           $2812 = HEAP32[$2811>>2]|0;
           $2813 = ($2809|0)!=(0);
           $2814 = ($2812|0)!=(0);
           $2815 = $2813 | $2814;
           $2816 = HEAP32[$pRec>>2]|0;
           if ($2815) {
            $2817 = (_sqlite3VdbeMemExpandBlob($2816)|0);
            $2818 = ($2817|0)!=(0);
            if ($2818) {
             label = 889;
             break L1;
            } else {
             break;
            }
           } else {
            $2819 = HEAP32[$2816>>2]|0;
            $2820 = ($2819|0)<(0);
            $2821 = $2820 << 31 >> 31;
            $2822 = $nZero;
            $2823 = $2822;
            $2824 = HEAP32[$2823>>2]|0;
            $2825 = (($2822) + 4)|0;
            $2826 = $2825;
            $2827 = HEAP32[$2826>>2]|0;
            $2828 = (_i64Add(($2824|0),($2827|0),($2819|0),($2821|0))|0);
            $2829 = tempRet0;
            $2830 = $nZero;
            $2831 = $2830;
            HEAP32[$2831>>2] = $2828;
            $2832 = (($2830) + 4)|0;
            $2833 = $2832;
            HEAP32[$2833>>2] = $2829;
            $2834 = HEAP32[$pRec>>2]|0;
            $2835 = HEAP32[$2834>>2]|0;
            $2836 = HEAP32[$len16>>2]|0;
            $2837 = (($2836) - ($2835))|0;
            HEAP32[$len16>>2] = $2837;
            break;
           }
          }
         } while(0);
         $2838 = HEAP32[$len16>>2]|0;
         $2839 = $nData;
         $2840 = $2839;
         $2841 = HEAP32[$2840>>2]|0;
         $2842 = (($2839) + 4)|0;
         $2843 = $2842;
         $2844 = HEAP32[$2843>>2]|0;
         $2845 = (_i64Add(($2841|0),($2844|0),($2838|0),0)|0);
         $2846 = tempRet0;
         $2847 = $nData;
         $2848 = $2847;
         HEAP32[$2848>>2] = $2845;
         $2849 = (($2847) + 4)|0;
         $2850 = $2849;
         HEAP32[$2850>>2] = $2846;
         $2851 = HEAP32[$serial_type>>2]|0;
         $2852 = ($2851>>>0)<=(127);
         if ($2852) {
          $2857 = 1;
         } else {
          $2853 = HEAP32[$serial_type>>2]|0;
          $2854 = (_sqlite3VarintLen($2853,0)|0);
          $2857 = $2854;
         }
         $2855 = HEAP32[$nHdr>>2]|0;
         $2856 = (($2855) + ($2857))|0;
         HEAP32[$nHdr>>2] = $2856;
         $2858 = HEAP32[$pRec>>2]|0;
         $2859 = HEAP32[$pData0>>2]|0;
         $2860 = ($2858|0)==($2859|0);
         if ($2860) {
          break;
         }
         $2861 = HEAP32[$pRec>>2]|0;
         $2862 = ((($2861)) + -40|0);
         HEAP32[$pRec>>2] = $2862;
        }
        $2863 = HEAP32[$nHdr>>2]|0;
        $2864 = ($2863|0)<=(126);
        $2865 = HEAP32[$nHdr>>2]|0;
        if ($2864) {
         $2866 = (($2865) + 1)|0;
         HEAP32[$nHdr>>2] = $2866;
        } else {
         $2867 = ($2865|0)<(0);
         $2868 = $2867 << 31 >> 31;
         $2869 = (_sqlite3VarintLen($2865,$2868)|0);
         HEAP32[$nVarint>>2] = $2869;
         $2870 = HEAP32[$nVarint>>2]|0;
         $2871 = HEAP32[$nHdr>>2]|0;
         $2872 = (($2871) + ($2870))|0;
         HEAP32[$nHdr>>2] = $2872;
         $2873 = HEAP32[$nVarint>>2]|0;
         $2874 = HEAP32[$nHdr>>2]|0;
         $2875 = ($2874|0)<(0);
         $2876 = $2875 << 31 >> 31;
         $2877 = (_sqlite3VarintLen($2874,$2876)|0);
         $2878 = ($2873|0)<($2877|0);
         if ($2878) {
          $2879 = HEAP32[$nHdr>>2]|0;
          $2880 = (($2879) + 1)|0;
          HEAP32[$nHdr>>2] = $2880;
         }
        }
        $2881 = HEAP32[$nHdr>>2]|0;
        $2882 = ($2881|0)<(0);
        $2883 = $2882 << 31 >> 31;
        $2884 = $nData;
        $2885 = $2884;
        $2886 = HEAP32[$2885>>2]|0;
        $2887 = (($2884) + 4)|0;
        $2888 = $2887;
        $2889 = HEAP32[$2888>>2]|0;
        $2890 = (_i64Add(($2881|0),($2883|0),($2886|0),($2889|0))|0);
        $2891 = tempRet0;
        $2892 = $nByte13;
        $2893 = $2892;
        HEAP32[$2893>>2] = $2890;
        $2894 = (($2892) + 4)|0;
        $2895 = $2894;
        HEAP32[$2895>>2] = $2891;
        $2896 = $nByte13;
        $2897 = $2896;
        $2898 = HEAP32[$2897>>2]|0;
        $2899 = (($2896) + 4)|0;
        $2900 = $2899;
        $2901 = HEAP32[$2900>>2]|0;
        $2902 = $nZero;
        $2903 = $2902;
        $2904 = HEAP32[$2903>>2]|0;
        $2905 = (($2902) + 4)|0;
        $2906 = $2905;
        $2907 = HEAP32[$2906>>2]|0;
        $2908 = (_i64Add(($2898|0),($2901|0),($2904|0),($2907|0))|0);
        $2909 = tempRet0;
        $2910 = HEAP32[$db>>2]|0;
        $2911 = ((($2910)) + 96|0);
        $2912 = HEAP32[$2911>>2]|0;
        $2913 = ($2912|0)<(0);
        $2914 = $2913 << 31 >> 31;
        $2915 = ($2909|0)>($2914|0);
        $2916 = ($2908>>>0)>($2912>>>0);
        $2917 = ($2909|0)==($2914|0);
        $2918 = $2917 & $2916;
        $2919 = $2915 | $2918;
        if ($2919) {
         label = 888;
         break L8;
        }
        $2920 = HEAP32[$pOut>>2]|0;
        $2921 = $nByte13;
        $2922 = $2921;
        $2923 = HEAP32[$2922>>2]|0;
        $2924 = (($2921) + 4)|0;
        $2925 = $2924;
        $2926 = HEAP32[$2925>>2]|0;
        $2927 = (_sqlite3VdbeMemClearAndResize($2920,$2923)|0);
        $2928 = ($2927|0)!=(0);
        if ($2928) {
         label = 889;
         break L1;
        }
        $2929 = HEAP32[$pOut>>2]|0;
        $2930 = ((($2929)) + 16|0);
        $2931 = HEAP32[$2930>>2]|0;
        HEAP32[$zNewRecord>>2] = $2931;
        $2932 = HEAP32[$nHdr>>2]|0;
        $2933 = ($2932>>>0)<(128);
        if ($2933) {
         $2934 = HEAP32[$nHdr>>2]|0;
         $2935 = $2934&255;
         $2936 = HEAP32[$zNewRecord>>2]|0;
         HEAP8[$2936>>0] = $2935;
         $2943 = 1;
        } else {
         $2937 = HEAP32[$zNewRecord>>2]|0;
         $2938 = HEAP32[$nHdr>>2]|0;
         $2939 = ($2938|0)<(0);
         $2940 = $2939 << 31 >> 31;
         $2941 = (_sqlite3PutVarint($2937,$2938,$2940)|0);
         $2943 = $2941;
        }
        $2942 = $2943&255;
        $2944 = $2942&255;
        HEAP32[$i15>>2] = $2944;
        $2945 = HEAP32[$nHdr>>2]|0;
        HEAP32[$j>>2] = $2945;
        $2946 = HEAP32[$pData0>>2]|0;
        HEAP32[$pRec>>2] = $2946;
        while(1) {
         $2947 = HEAP32[$pRec>>2]|0;
         $2948 = ((($2947)) + 28|0);
         $2949 = HEAP32[$2948>>2]|0;
         HEAP32[$serial_type>>2] = $2949;
         $2950 = HEAP32[$serial_type>>2]|0;
         $2951 = ($2950>>>0)<(128);
         if ($2951) {
          $2952 = HEAP32[$serial_type>>2]|0;
          $2953 = $2952&255;
          $2954 = HEAP32[$i15>>2]|0;
          $2955 = HEAP32[$zNewRecord>>2]|0;
          $2956 = (($2955) + ($2954)|0);
          HEAP8[$2956>>0] = $2953;
          $2963 = 1;
         } else {
          $2957 = HEAP32[$i15>>2]|0;
          $2958 = HEAP32[$zNewRecord>>2]|0;
          $2959 = (($2958) + ($2957)|0);
          $2960 = HEAP32[$serial_type>>2]|0;
          $2961 = (_sqlite3PutVarint($2959,$2960,0)|0);
          $2963 = $2961;
         }
         $2962 = $2963&255;
         $2964 = $2962&255;
         $2965 = HEAP32[$i15>>2]|0;
         $2966 = (($2965) + ($2964))|0;
         HEAP32[$i15>>2] = $2966;
         $2967 = HEAP32[$j>>2]|0;
         $2968 = HEAP32[$zNewRecord>>2]|0;
         $2969 = (($2968) + ($2967)|0);
         $2970 = HEAP32[$pRec>>2]|0;
         $2971 = HEAP32[$serial_type>>2]|0;
         $2972 = (_sqlite3VdbeSerialPut($2969,$2970,$2971)|0);
         $2973 = HEAP32[$j>>2]|0;
         $2974 = (($2973) + ($2972))|0;
         HEAP32[$j>>2] = $2974;
         $2975 = HEAP32[$pRec>>2]|0;
         $2976 = ((($2975)) + 40|0);
         HEAP32[$pRec>>2] = $2976;
         $2977 = HEAP32[$pLast>>2]|0;
         $2978 = ($2976>>>0)<=($2977>>>0);
         if (!($2978)) {
          break;
         }
        }
        $2979 = $nByte13;
        $2980 = $2979;
        $2981 = HEAP32[$2980>>2]|0;
        $2982 = (($2979) + 4)|0;
        $2983 = $2982;
        $2984 = HEAP32[$2983>>2]|0;
        $2985 = HEAP32[$pOut>>2]|0;
        $2986 = ((($2985)) + 12|0);
        HEAP32[$2986>>2] = $2981;
        $2987 = HEAP32[$pOut>>2]|0;
        $2988 = ((($2987)) + 8|0);
        HEAP16[$2988>>1] = 16;
        $2989 = $nZero;
        $2990 = $2989;
        $2991 = HEAP32[$2990>>2]|0;
        $2992 = (($2989) + 4)|0;
        $2993 = $2992;
        $2994 = HEAP32[$2993>>2]|0;
        $2995 = ($2991|0)!=(0);
        $2996 = ($2994|0)!=(0);
        $2997 = $2995 | $2996;
        if ($2997) {
         $2998 = $nZero;
         $2999 = $2998;
         $3000 = HEAP32[$2999>>2]|0;
         $3001 = (($2998) + 4)|0;
         $3002 = $3001;
         $3003 = HEAP32[$3002>>2]|0;
         $3004 = HEAP32[$pOut>>2]|0;
         HEAP32[$3004>>2] = $3000;
         $3005 = HEAP32[$pOut>>2]|0;
         $3006 = ((($3005)) + 8|0);
         $3007 = HEAP16[$3006>>1]|0;
         $3008 = $3007&65535;
         $3009 = $3008 | 16384;
         $3010 = $3009&65535;
         HEAP16[$3006>>1] = $3010;
        }
        $3011 = HEAP32[$pOut>>2]|0;
        $3012 = ((($3011)) + 10|0);
        HEAP8[$3012>>0] = 1;
        break;
       }
       case 100:  {
        $3013 = HEAP32[$pOp>>2]|0;
        $3014 = ((($3013)) + 4|0);
        $3015 = HEAP32[$3014>>2]|0;
        $3016 = HEAP32[$0>>2]|0;
        $3017 = ((($3016)) + 112|0);
        $3018 = HEAP32[$3017>>2]|0;
        $3019 = (($3018) + ($3015<<2)|0);
        $3020 = HEAP32[$3019>>2]|0;
        $3021 = ((($3020)) + 16|0);
        $3022 = HEAP32[$3021>>2]|0;
        HEAP32[$pCrsr17>>2] = $3022;
        $3023 = $nEntry;
        $3024 = $3023;
        HEAP32[$3024>>2] = 0;
        $3025 = (($3023) + 4)|0;
        $3026 = $3025;
        HEAP32[$3026>>2] = 0;
        $3027 = HEAP32[$pCrsr17>>2]|0;
        $3028 = (_sqlite3BtreeCount($3027,$nEntry)|0);
        HEAP32[$rc>>2] = $3028;
        $3029 = HEAP32[$rc>>2]|0;
        $3030 = ($3029|0)!=(0);
        if ($3030) {
         break L1;
        }
        $3031 = HEAP32[$0>>2]|0;
        $3032 = HEAP32[$pOp>>2]|0;
        $3033 = (_out2Prerelease($3031,$3032)|0);
        HEAP32[$pOut>>2] = $3033;
        $3034 = $nEntry;
        $3035 = $3034;
        $3036 = HEAP32[$3035>>2]|0;
        $3037 = (($3034) + 4)|0;
        $3038 = $3037;
        $3039 = HEAP32[$3038>>2]|0;
        $3040 = HEAP32[$pOut>>2]|0;
        $3041 = $3040;
        $3042 = $3041;
        HEAP32[$3042>>2] = $3036;
        $3043 = (($3041) + 4)|0;
        $3044 = $3043;
        HEAP32[$3044>>2] = $3039;
        break;
       }
       case 0:  {
        $3045 = HEAP32[$pOp>>2]|0;
        $3046 = ((($3045)) + 4|0);
        $3047 = HEAP32[$3046>>2]|0;
        HEAP32[$p118>>2] = $3047;
        $3048 = HEAP32[$pOp>>2]|0;
        $3049 = ((($3048)) + 16|0);
        $3050 = HEAP32[$3049>>2]|0;
        HEAP32[$zName>>2] = $3050;
        $3051 = HEAP32[$p118>>2]|0;
        $3052 = ($3051|0)==(0);
        do {
         if ($3052) {
          $3053 = HEAP32[$db>>2]|0;
          $3054 = ((($3053)) + 164|0);
          $3055 = HEAP32[$3054>>2]|0;
          $3056 = ($3055|0)>(0);
          if ($3056) {
           $3057 = HEAP32[$0>>2]|0;
           _sqlite3VdbeError($3057,39951,$vararg_buffer47);
           HEAP32[$rc>>2] = 5;
           break;
          }
          $3058 = HEAP32[$zName>>2]|0;
          $3059 = (_sqlite3Strlen30($3058)|0);
          HEAP32[$nName>>2] = $3059;
          $3060 = HEAP32[$db>>2]|0;
          $3061 = HEAP32[$db>>2]|0;
          $3062 = ((($3061)) + 436|0);
          $3063 = HEAP32[$3062>>2]|0;
          $3064 = HEAP32[$db>>2]|0;
          $3065 = ((($3064)) + 432|0);
          $3066 = HEAP32[$3065>>2]|0;
          $3067 = (($3063) + ($3066))|0;
          $3068 = (_sqlite3VtabSavepoint($3060,0,$3067)|0);
          HEAP32[$rc>>2] = $3068;
          $3069 = HEAP32[$rc>>2]|0;
          $3070 = ($3069|0)!=(0);
          if ($3070) {
           break L1;
          }
          $3071 = HEAP32[$db>>2]|0;
          $3072 = HEAP32[$nName>>2]|0;
          $3073 = (32 + ($3072))|0;
          $3074 = (($3073) + 1)|0;
          $3075 = (_sqlite3DbMallocRawNN($3071,$3074,0)|0);
          HEAP32[$pNew>>2] = $3075;
          $3076 = HEAP32[$pNew>>2]|0;
          $3077 = ($3076|0)!=(0|0);
          if ($3077) {
           $3078 = HEAP32[$pNew>>2]|0;
           $3079 = ((($3078)) + 32|0);
           $3080 = HEAP32[$pNew>>2]|0;
           HEAP32[$3080>>2] = $3079;
           $3081 = HEAP32[$pNew>>2]|0;
           $3082 = HEAP32[$3081>>2]|0;
           $3083 = HEAP32[$zName>>2]|0;
           $3084 = HEAP32[$nName>>2]|0;
           $3085 = (($3084) + 1)|0;
           _memcpy(($3082|0),($3083|0),($3085|0))|0;
           $3086 = HEAP32[$db>>2]|0;
           $3087 = ((($3086)) + 67|0);
           $3088 = HEAP8[$3087>>0]|0;
           $3089 = ($3088<<24>>24)!=(0);
           $3090 = HEAP32[$db>>2]|0;
           if ($3089) {
            $3091 = ((($3090)) + 67|0);
            HEAP8[$3091>>0] = 0;
            $3092 = HEAP32[$db>>2]|0;
            $3093 = ((($3092)) + 75|0);
            HEAP8[$3093>>0] = 1;
           } else {
            $3094 = ((($3090)) + 432|0);
            $3095 = HEAP32[$3094>>2]|0;
            $3096 = (($3095) + 1)|0;
            HEAP32[$3094>>2] = $3096;
           }
           $3097 = HEAP32[$db>>2]|0;
           $3098 = ((($3097)) + 424|0);
           $3099 = HEAP32[$3098>>2]|0;
           $3100 = HEAP32[$pNew>>2]|0;
           $3101 = ((($3100)) + 24|0);
           HEAP32[$3101>>2] = $3099;
           $3102 = HEAP32[$pNew>>2]|0;
           $3103 = HEAP32[$db>>2]|0;
           $3104 = ((($3103)) + 424|0);
           HEAP32[$3104>>2] = $3102;
           $3105 = HEAP32[$db>>2]|0;
           $3106 = ((($3105)) + 440|0);
           $3107 = $3106;
           $3108 = $3107;
           $3109 = HEAP32[$3108>>2]|0;
           $3110 = (($3107) + 4)|0;
           $3111 = $3110;
           $3112 = HEAP32[$3111>>2]|0;
           $3113 = HEAP32[$pNew>>2]|0;
           $3114 = ((($3113)) + 8|0);
           $3115 = $3114;
           $3116 = $3115;
           HEAP32[$3116>>2] = $3109;
           $3117 = (($3115) + 4)|0;
           $3118 = $3117;
           HEAP32[$3118>>2] = $3112;
           $3119 = HEAP32[$db>>2]|0;
           $3120 = ((($3119)) + 448|0);
           $3121 = $3120;
           $3122 = $3121;
           $3123 = HEAP32[$3122>>2]|0;
           $3124 = (($3121) + 4)|0;
           $3125 = $3124;
           $3126 = HEAP32[$3125>>2]|0;
           $3127 = HEAP32[$pNew>>2]|0;
           $3128 = ((($3127)) + 16|0);
           $3129 = $3128;
           $3130 = $3129;
           HEAP32[$3130>>2] = $3123;
           $3131 = (($3129) + 4)|0;
           $3132 = $3131;
           HEAP32[$3132>>2] = $3126;
          }
         } else {
          HEAP32[$iSavepoint>>2] = 0;
          $3133 = HEAP32[$db>>2]|0;
          $3134 = ((($3133)) + 424|0);
          $3135 = HEAP32[$3134>>2]|0;
          HEAP32[$pSavepoint>>2] = $3135;
          while(1) {
           $3136 = HEAP32[$pSavepoint>>2]|0;
           $3137 = ($3136|0)!=(0|0);
           if (!($3137)) {
            break;
           }
           $3138 = HEAP32[$pSavepoint>>2]|0;
           $3139 = HEAP32[$3138>>2]|0;
           $3140 = HEAP32[$zName>>2]|0;
           $3141 = (_sqlite3StrICmp($3139,$3140)|0);
           $3142 = ($3141|0)!=(0);
           if (!($3142)) {
            break;
           }
           $3143 = HEAP32[$iSavepoint>>2]|0;
           $3144 = (($3143) + 1)|0;
           HEAP32[$iSavepoint>>2] = $3144;
           $3145 = HEAP32[$pSavepoint>>2]|0;
           $3146 = ((($3145)) + 24|0);
           $3147 = HEAP32[$3146>>2]|0;
           HEAP32[$pSavepoint>>2] = $3147;
          }
          $3148 = HEAP32[$pSavepoint>>2]|0;
          $3149 = ($3148|0)!=(0|0);
          if (!($3149)) {
           $3150 = HEAP32[$0>>2]|0;
           $3151 = HEAP32[$zName>>2]|0;
           HEAP32[$vararg_buffer49>>2] = $3151;
           _sqlite3VdbeError($3150,40002,$vararg_buffer49);
           HEAP32[$rc>>2] = 1;
           break;
          }
          $3152 = HEAP32[$db>>2]|0;
          $3153 = ((($3152)) + 164|0);
          $3154 = HEAP32[$3153>>2]|0;
          $3155 = ($3154|0)>(0);
          $3156 = HEAP32[$p118>>2]|0;
          $3157 = ($3156|0)==(1);
          $or$cond6 = $3155 & $3157;
          if ($or$cond6) {
           $3158 = HEAP32[$0>>2]|0;
           _sqlite3VdbeError($3158,40024,$vararg_buffer52);
           HEAP32[$rc>>2] = 5;
           break;
          }
          $3159 = HEAP32[$pSavepoint>>2]|0;
          $3160 = ((($3159)) + 24|0);
          $3161 = HEAP32[$3160>>2]|0;
          $3162 = ($3161|0)==(0|0);
          if ($3162) {
           $3163 = HEAP32[$db>>2]|0;
           $3164 = ((($3163)) + 75|0);
           $3165 = HEAP8[$3164>>0]|0;
           $3166 = $3165&255;
           $3167 = ($3166|0)!=(0);
           $3169 = $3167;
          } else {
           $3169 = 0;
          }
          $3168 = $3169&1;
          HEAP32[$isTransaction>>2] = $3168;
          $3170 = HEAP32[$isTransaction>>2]|0;
          $3171 = ($3170|0)!=(0);
          $3172 = HEAP32[$p118>>2]|0;
          $3173 = ($3172|0)==(1);
          $or$cond8 = $3171 & $3173;
          if ($or$cond8) {
           $3174 = HEAP32[$0>>2]|0;
           $3175 = (_sqlite3VdbeCheckFk($3174,1)|0);
           HEAP32[$rc>>2] = $3175;
           $3176 = ($3175|0)!=(0);
           if ($3176) {
            label = 887;
            break L8;
           }
           $3177 = HEAP32[$db>>2]|0;
           $3178 = ((($3177)) + 67|0);
           HEAP8[$3178>>0] = 1;
           $3179 = HEAP32[$0>>2]|0;
           $3180 = (_sqlite3VdbeHalt($3179)|0);
           $3181 = ($3180|0)==(5);
           if ($3181) {
            label = 370;
            break L8;
           }
           $3194 = HEAP32[$db>>2]|0;
           $3195 = ((($3194)) + 75|0);
           HEAP8[$3195>>0] = 0;
           $3196 = HEAP32[$0>>2]|0;
           $3197 = ((($3196)) + 40|0);
           $3198 = HEAP32[$3197>>2]|0;
           HEAP32[$rc>>2] = $3198;
          } else {
           $3199 = HEAP32[$db>>2]|0;
           $3200 = ((($3199)) + 432|0);
           $3201 = HEAP32[$3200>>2]|0;
           $3202 = HEAP32[$iSavepoint>>2]|0;
           $3203 = (($3201) - ($3202))|0;
           $3204 = (($3203) - 1)|0;
           HEAP32[$iSavepoint>>2] = $3204;
           $3205 = HEAP32[$p118>>2]|0;
           $3206 = ($3205|0)==(2);
           L418: do {
            if ($3206) {
             $3207 = HEAP32[$db>>2]|0;
             $3208 = ((($3207)) + 24|0);
             $3209 = HEAP32[$3208>>2]|0;
             $3210 = $3209 & 2;
             $3211 = ($3210|0)!=(0);
             $3212 = $3211&1;
             HEAP32[$isSchemaChange>>2] = $3212;
             HEAP32[$ii>>2] = 0;
             while(1) {
              $3213 = HEAP32[$ii>>2]|0;
              $3214 = HEAP32[$db>>2]|0;
              $3215 = ((($3214)) + 20|0);
              $3216 = HEAP32[$3215>>2]|0;
              $3217 = ($3213|0)<($3216|0);
              if (!($3217)) {
               break L418;
              }
              $3218 = HEAP32[$ii>>2]|0;
              $3219 = HEAP32[$db>>2]|0;
              $3220 = ((($3219)) + 16|0);
              $3221 = HEAP32[$3220>>2]|0;
              $3222 = (($3221) + ($3218<<4)|0);
              $3223 = ((($3222)) + 4|0);
              $3224 = HEAP32[$3223>>2]|0;
              $3225 = HEAP32[$isSchemaChange>>2]|0;
              $3226 = ($3225|0)==(0);
              $3227 = $3226&1;
              $3228 = (_sqlite3BtreeTripAllCursors($3224,516,$3227)|0);
              HEAP32[$rc>>2] = $3228;
              $3229 = HEAP32[$rc>>2]|0;
              $3230 = ($3229|0)!=(0);
              if ($3230) {
               break L1;
              }
              $3231 = HEAP32[$ii>>2]|0;
              $3232 = (($3231) + 1)|0;
              HEAP32[$ii>>2] = $3232;
             }
            } else {
             HEAP32[$isSchemaChange>>2] = 0;
            }
           } while(0);
           HEAP32[$ii>>2] = 0;
           while(1) {
            $3233 = HEAP32[$ii>>2]|0;
            $3234 = HEAP32[$db>>2]|0;
            $3235 = ((($3234)) + 20|0);
            $3236 = HEAP32[$3235>>2]|0;
            $3237 = ($3233|0)<($3236|0);
            if (!($3237)) {
             break;
            }
            $3238 = HEAP32[$ii>>2]|0;
            $3239 = HEAP32[$db>>2]|0;
            $3240 = ((($3239)) + 16|0);
            $3241 = HEAP32[$3240>>2]|0;
            $3242 = (($3241) + ($3238<<4)|0);
            $3243 = ((($3242)) + 4|0);
            $3244 = HEAP32[$3243>>2]|0;
            $3245 = HEAP32[$p118>>2]|0;
            $3246 = HEAP32[$iSavepoint>>2]|0;
            $3247 = (_sqlite3BtreeSavepoint($3244,$3245,$3246)|0);
            HEAP32[$rc>>2] = $3247;
            $3248 = HEAP32[$rc>>2]|0;
            $3249 = ($3248|0)!=(0);
            if ($3249) {
             break L1;
            }
            $3250 = HEAP32[$ii>>2]|0;
            $3251 = (($3250) + 1)|0;
            HEAP32[$ii>>2] = $3251;
           }
           $3252 = HEAP32[$isSchemaChange>>2]|0;
           $3253 = ($3252|0)!=(0);
           if ($3253) {
            $3254 = HEAP32[$db>>2]|0;
            _sqlite3ExpirePreparedStatements($3254);
            $3255 = HEAP32[$db>>2]|0;
            _sqlite3ResetAllSchemasOfConnection($3255);
            $3256 = HEAP32[$db>>2]|0;
            $3257 = ((($3256)) + 24|0);
            $3258 = HEAP32[$3257>>2]|0;
            $3259 = $3258 | 2;
            $3260 = HEAP32[$db>>2]|0;
            $3261 = ((($3260)) + 24|0);
            HEAP32[$3261>>2] = $3259;
           }
          }
          while(1) {
           $3262 = HEAP32[$db>>2]|0;
           $3263 = ((($3262)) + 424|0);
           $3264 = HEAP32[$3263>>2]|0;
           $3265 = HEAP32[$pSavepoint>>2]|0;
           $3266 = ($3264|0)!=($3265|0);
           if (!($3266)) {
            break;
           }
           $3267 = HEAP32[$db>>2]|0;
           $3268 = ((($3267)) + 424|0);
           $3269 = HEAP32[$3268>>2]|0;
           HEAP32[$pTmp>>2] = $3269;
           $3270 = HEAP32[$pTmp>>2]|0;
           $3271 = ((($3270)) + 24|0);
           $3272 = HEAP32[$3271>>2]|0;
           $3273 = HEAP32[$db>>2]|0;
           $3274 = ((($3273)) + 424|0);
           HEAP32[$3274>>2] = $3272;
           $3275 = HEAP32[$db>>2]|0;
           $3276 = HEAP32[$pTmp>>2]|0;
           _sqlite3DbFree($3275,$3276);
           $3277 = HEAP32[$db>>2]|0;
           $3278 = ((($3277)) + 432|0);
           $3279 = HEAP32[$3278>>2]|0;
           $3280 = (($3279) + -1)|0;
           HEAP32[$3278>>2] = $3280;
          }
          $3281 = HEAP32[$p118>>2]|0;
          $3282 = ($3281|0)==(1);
          $3283 = HEAP32[$pSavepoint>>2]|0;
          if ($3282) {
           $3284 = ((($3283)) + 24|0);
           $3285 = HEAP32[$3284>>2]|0;
           $3286 = HEAP32[$db>>2]|0;
           $3287 = ((($3286)) + 424|0);
           HEAP32[$3287>>2] = $3285;
           $3288 = HEAP32[$db>>2]|0;
           $3289 = HEAP32[$pSavepoint>>2]|0;
           _sqlite3DbFree($3288,$3289);
           $3290 = HEAP32[$isTransaction>>2]|0;
           $3291 = ($3290|0)!=(0);
           if (!($3291)) {
            $3292 = HEAP32[$db>>2]|0;
            $3293 = ((($3292)) + 432|0);
            $3294 = HEAP32[$3293>>2]|0;
            $3295 = (($3294) + -1)|0;
            HEAP32[$3293>>2] = $3295;
           }
          } else {
           $3296 = ((($3283)) + 8|0);
           $3297 = $3296;
           $3298 = $3297;
           $3299 = HEAP32[$3298>>2]|0;
           $3300 = (($3297) + 4)|0;
           $3301 = $3300;
           $3302 = HEAP32[$3301>>2]|0;
           $3303 = HEAP32[$db>>2]|0;
           $3304 = ((($3303)) + 440|0);
           $3305 = $3304;
           $3306 = $3305;
           HEAP32[$3306>>2] = $3299;
           $3307 = (($3305) + 4)|0;
           $3308 = $3307;
           HEAP32[$3308>>2] = $3302;
           $3309 = HEAP32[$pSavepoint>>2]|0;
           $3310 = ((($3309)) + 16|0);
           $3311 = $3310;
           $3312 = $3311;
           $3313 = HEAP32[$3312>>2]|0;
           $3314 = (($3311) + 4)|0;
           $3315 = $3314;
           $3316 = HEAP32[$3315>>2]|0;
           $3317 = HEAP32[$db>>2]|0;
           $3318 = ((($3317)) + 448|0);
           $3319 = $3318;
           $3320 = $3319;
           HEAP32[$3320>>2] = $3313;
           $3321 = (($3319) + 4)|0;
           $3322 = $3321;
           HEAP32[$3322>>2] = $3316;
          }
          $3323 = HEAP32[$isTransaction>>2]|0;
          $3324 = ($3323|0)==(0);
          $3325 = HEAP32[$p118>>2]|0;
          $3326 = ($3325|0)==(2);
          $or$cond10 = $3324 | $3326;
          if ($or$cond10) {
           $3327 = HEAP32[$db>>2]|0;
           $3328 = HEAP32[$p118>>2]|0;
           $3329 = HEAP32[$iSavepoint>>2]|0;
           $3330 = (_sqlite3VtabSavepoint($3327,$3328,$3329)|0);
           HEAP32[$rc>>2] = $3330;
           $3331 = HEAP32[$rc>>2]|0;
           $3332 = ($3331|0)!=(0);
           $3333 = HEAP32[$rc>>2]|0;
           $3334 = ($3333|0)!=(0);
           $or$cond28 = $3332 | $3334;
           if ($or$cond28) {
            break L1;
           } else {
            break L10;
           }
          }
         }
        } while(0);
        $$old = HEAP32[$rc>>2]|0;
        $$old27 = ($$old|0)!=(0);
        if ($$old27) {
         break L1;
        }
        break;
       }
       case 2:  {
        $3396 = HEAP32[$pOp>>2]|0;
        $3397 = ((($3396)) + 8|0);
        $3398 = HEAP32[$3397>>2]|0;
        $3399 = ($3398|0)!=(0);
        if ($3399) {
         $3400 = HEAP32[$db>>2]|0;
         $3401 = ((($3400)) + 24|0);
         $3402 = HEAP32[$3401>>2]|0;
         $3403 = $3402 & 67108864;
         $3404 = ($3403|0)!=(0);
         if ($3404) {
          label = 411;
          break L8;
         }
        }
        $3405 = HEAP32[$pOp>>2]|0;
        $3406 = ((($3405)) + 4|0);
        $3407 = HEAP32[$3406>>2]|0;
        $3408 = HEAP32[$db>>2]|0;
        $3409 = ((($3408)) + 16|0);
        $3410 = HEAP32[$3409>>2]|0;
        $3411 = (($3410) + ($3407<<4)|0);
        $3412 = ((($3411)) + 4|0);
        $3413 = HEAP32[$3412>>2]|0;
        HEAP32[$pBt>>2] = $3413;
        $3414 = HEAP32[$pBt>>2]|0;
        $3415 = ($3414|0)!=(0|0);
        if ($3415) {
         $3416 = HEAP32[$pBt>>2]|0;
         $3417 = HEAP32[$pOp>>2]|0;
         $3418 = ((($3417)) + 8|0);
         $3419 = HEAP32[$3418>>2]|0;
         $3420 = (_sqlite3BtreeBeginTrans($3416,$3419)|0);
         HEAP32[$rc>>2] = $3420;
         $3421 = HEAP32[$rc>>2]|0;
         $3422 = ($3421|0)!=(0);
         if ($3422) {
          label = 414;
          break L8;
         }
         $3437 = HEAP32[$pOp>>2]|0;
         $3438 = ((($3437)) + 8|0);
         $3439 = HEAP32[$3438>>2]|0;
         $3440 = ($3439|0)!=(0);
         do {
          if ($3440) {
           $3441 = HEAP32[$0>>2]|0;
           $3442 = ((($3441)) + 144|0);
           $3443 = HEAP16[$3442>>1]|0;
           $3444 = ($3443&65535) >>> 6;
           $3445 = $3444 & 1;
           $3446 = $3445&65535;
           $3447 = ($3446|0)!=(0);
           if ($3447) {
            $3448 = HEAP32[$db>>2]|0;
            $3449 = ((($3448)) + 67|0);
            $3450 = HEAP8[$3449>>0]|0;
            $3451 = $3450&255;
            $3452 = ($3451|0)==(0);
            if (!($3452)) {
             $3453 = HEAP32[$db>>2]|0;
             $3454 = ((($3453)) + 160|0);
             $3455 = HEAP32[$3454>>2]|0;
             $3456 = ($3455|0)>(1);
             if (!($3456)) {
              break;
             }
            }
            $3457 = HEAP32[$0>>2]|0;
            $3458 = ((($3457)) + 48|0);
            $3459 = HEAP32[$3458>>2]|0;
            $3460 = ($3459|0)==(0);
            if ($3460) {
             $3461 = HEAP32[$db>>2]|0;
             $3462 = ((($3461)) + 436|0);
             $3463 = HEAP32[$3462>>2]|0;
             $3464 = (($3463) + 1)|0;
             HEAP32[$3462>>2] = $3464;
             $3465 = HEAP32[$db>>2]|0;
             $3466 = ((($3465)) + 432|0);
             $3467 = HEAP32[$3466>>2]|0;
             $3468 = HEAP32[$db>>2]|0;
             $3469 = ((($3468)) + 436|0);
             $3470 = HEAP32[$3469>>2]|0;
             $3471 = (($3467) + ($3470))|0;
             $3472 = HEAP32[$0>>2]|0;
             $3473 = ((($3472)) + 48|0);
             HEAP32[$3473>>2] = $3471;
            }
            $3474 = HEAP32[$db>>2]|0;
            $3475 = HEAP32[$0>>2]|0;
            $3476 = ((($3475)) + 48|0);
            $3477 = HEAP32[$3476>>2]|0;
            $3478 = (($3477) - 1)|0;
            $3479 = (_sqlite3VtabSavepoint($3474,0,$3478)|0);
            HEAP32[$rc>>2] = $3479;
            $3480 = HEAP32[$rc>>2]|0;
            $3481 = ($3480|0)==(0);
            if ($3481) {
             $3482 = HEAP32[$pBt>>2]|0;
             $3483 = HEAP32[$0>>2]|0;
             $3484 = ((($3483)) + 48|0);
             $3485 = HEAP32[$3484>>2]|0;
             $3486 = (_sqlite3BtreeBeginStmt($3482,$3485)|0);
             HEAP32[$rc>>2] = $3486;
            }
            $3487 = HEAP32[$db>>2]|0;
            $3488 = ((($3487)) + 440|0);
            $3489 = $3488;
            $3490 = $3489;
            $3491 = HEAP32[$3490>>2]|0;
            $3492 = (($3489) + 4)|0;
            $3493 = $3492;
            $3494 = HEAP32[$3493>>2]|0;
            $3495 = HEAP32[$0>>2]|0;
            $3496 = ((($3495)) + 72|0);
            $3497 = $3496;
            $3498 = $3497;
            HEAP32[$3498>>2] = $3491;
            $3499 = (($3497) + 4)|0;
            $3500 = $3499;
            HEAP32[$3500>>2] = $3494;
            $3501 = HEAP32[$db>>2]|0;
            $3502 = ((($3501)) + 448|0);
            $3503 = $3502;
            $3504 = $3503;
            $3505 = HEAP32[$3504>>2]|0;
            $3506 = (($3503) + 4)|0;
            $3507 = $3506;
            $3508 = HEAP32[$3507>>2]|0;
            $3509 = HEAP32[$0>>2]|0;
            $3510 = ((($3509)) + 80|0);
            $3511 = $3510;
            $3512 = $3511;
            HEAP32[$3512>>2] = $3505;
            $3513 = (($3511) + 4)|0;
            $3514 = $3513;
            HEAP32[$3514>>2] = $3508;
           }
          }
         } while(0);
         $3515 = HEAP32[$pBt>>2]|0;
         _sqlite3BtreeGetMeta($3515,1,$iMeta);
         $3516 = HEAP32[$pOp>>2]|0;
         $3517 = ((($3516)) + 4|0);
         $3518 = HEAP32[$3517>>2]|0;
         $3519 = HEAP32[$db>>2]|0;
         $3520 = ((($3519)) + 16|0);
         $3521 = HEAP32[$3520>>2]|0;
         $3522 = (($3521) + ($3518<<4)|0);
         $3523 = ((($3522)) + 12|0);
         $3524 = HEAP32[$3523>>2]|0;
         $3525 = ((($3524)) + 4|0);
         $3526 = HEAP32[$3525>>2]|0;
         HEAP32[$iGen>>2] = $3526;
        } else {
         HEAP32[$iMeta>>2] = 0;
         HEAP32[$iGen>>2] = 0;
        }
        $3527 = HEAP32[$pOp>>2]|0;
        $3528 = ((($3527)) + 3|0);
        $3529 = HEAP8[$3528>>0]|0;
        $3530 = $3529&255;
        $3531 = ($3530|0)!=(0);
        do {
         if ($3531) {
          $3532 = HEAP32[$iMeta>>2]|0;
          $3533 = HEAP32[$pOp>>2]|0;
          $3534 = ((($3533)) + 12|0);
          $3535 = HEAP32[$3534>>2]|0;
          $3536 = ($3532|0)!=($3535|0);
          if (!($3536)) {
           $3537 = HEAP32[$iGen>>2]|0;
           $3538 = HEAP32[$pOp>>2]|0;
           $3539 = ((($3538)) + 16|0);
           $3540 = HEAP32[$3539>>2]|0;
           $3541 = ($3537|0)!=($3540|0);
           if (!($3541)) {
            break;
           }
          }
          $3542 = HEAP32[$db>>2]|0;
          $3543 = HEAP32[$0>>2]|0;
          $3544 = ((($3543)) + 108|0);
          $3545 = HEAP32[$3544>>2]|0;
          _sqlite3DbFree($3542,$3545);
          $3546 = HEAP32[$db>>2]|0;
          $3547 = (_sqlite3DbStrDup($3546,40265)|0);
          $3548 = HEAP32[$0>>2]|0;
          $3549 = ((($3548)) + 108|0);
          HEAP32[$3549>>2] = $3547;
          $3550 = HEAP32[$pOp>>2]|0;
          $3551 = ((($3550)) + 4|0);
          $3552 = HEAP32[$3551>>2]|0;
          $3553 = HEAP32[$db>>2]|0;
          $3554 = ((($3553)) + 16|0);
          $3555 = HEAP32[$3554>>2]|0;
          $3556 = (($3555) + ($3552<<4)|0);
          $3557 = ((($3556)) + 12|0);
          $3558 = HEAP32[$3557>>2]|0;
          $3559 = HEAP32[$3558>>2]|0;
          $3560 = HEAP32[$iMeta>>2]|0;
          $3561 = ($3559|0)!=($3560|0);
          if ($3561) {
           $3562 = HEAP32[$db>>2]|0;
           $3563 = HEAP32[$pOp>>2]|0;
           $3564 = ((($3563)) + 4|0);
           $3565 = HEAP32[$3564>>2]|0;
           _sqlite3ResetOneSchema($3562,$3565);
          }
          $3566 = HEAP32[$0>>2]|0;
          $3567 = ((($3566)) + 144|0);
          $3568 = HEAP16[$3567>>1]|0;
          $3569 = $3568 & -2;
          $3570 = $3569 | 1;
          HEAP16[$3567>>1] = $3570;
          HEAP32[$rc>>2] = 17;
         }
        } while(0);
        $3571 = HEAP32[$rc>>2]|0;
        $3572 = ($3571|0)!=(0);
        if ($3572) {
         break L1;
        }
        break;
       }
       case 101:  {
        $3573 = HEAP32[$pOp>>2]|0;
        $3574 = ((($3573)) + 4|0);
        $3575 = HEAP32[$3574>>2]|0;
        HEAP32[$iDb>>2] = $3575;
        $3576 = HEAP32[$pOp>>2]|0;
        $3577 = ((($3576)) + 12|0);
        $3578 = HEAP32[$3577>>2]|0;
        HEAP32[$iCookie>>2] = $3578;
        $3579 = HEAP32[$iDb>>2]|0;
        $3580 = HEAP32[$db>>2]|0;
        $3581 = ((($3580)) + 16|0);
        $3582 = HEAP32[$3581>>2]|0;
        $3583 = (($3582) + ($3579<<4)|0);
        $3584 = ((($3583)) + 4|0);
        $3585 = HEAP32[$3584>>2]|0;
        $3586 = HEAP32[$iCookie>>2]|0;
        _sqlite3BtreeGetMeta($3585,$3586,$iMeta19);
        $3587 = HEAP32[$0>>2]|0;
        $3588 = HEAP32[$pOp>>2]|0;
        $3589 = (_out2Prerelease($3587,$3588)|0);
        HEAP32[$pOut>>2] = $3589;
        $3590 = HEAP32[$iMeta19>>2]|0;
        $3591 = ($3590|0)<(0);
        $3592 = $3591 << 31 >> 31;
        $3593 = HEAP32[$pOut>>2]|0;
        $3594 = $3593;
        $3595 = $3594;
        HEAP32[$3595>>2] = $3590;
        $3596 = (($3594) + 4)|0;
        $3597 = $3596;
        HEAP32[$3597>>2] = $3592;
        break;
       }
       case 102:  {
        $3598 = HEAP32[$pOp>>2]|0;
        $3599 = ((($3598)) + 4|0);
        $3600 = HEAP32[$3599>>2]|0;
        $3601 = HEAP32[$db>>2]|0;
        $3602 = ((($3601)) + 16|0);
        $3603 = HEAP32[$3602>>2]|0;
        $3604 = (($3603) + ($3600<<4)|0);
        HEAP32[$pDb>>2] = $3604;
        $3605 = HEAP32[$pDb>>2]|0;
        $3606 = ((($3605)) + 4|0);
        $3607 = HEAP32[$3606>>2]|0;
        $3608 = HEAP32[$pOp>>2]|0;
        $3609 = ((($3608)) + 8|0);
        $3610 = HEAP32[$3609>>2]|0;
        $3611 = HEAP32[$pOp>>2]|0;
        $3612 = ((($3611)) + 12|0);
        $3613 = HEAP32[$3612>>2]|0;
        $3614 = (_sqlite3BtreeUpdateMeta($3607,$3610,$3613)|0);
        HEAP32[$rc>>2] = $3614;
        $3615 = HEAP32[$pOp>>2]|0;
        $3616 = ((($3615)) + 8|0);
        $3617 = HEAP32[$3616>>2]|0;
        $3618 = ($3617|0)==(1);
        $3619 = HEAP32[$pOp>>2]|0;
        if ($3618) {
         $3620 = ((($3619)) + 12|0);
         $3621 = HEAP32[$3620>>2]|0;
         $3622 = HEAP32[$pDb>>2]|0;
         $3623 = ((($3622)) + 12|0);
         $3624 = HEAP32[$3623>>2]|0;
         HEAP32[$3624>>2] = $3621;
         $3625 = HEAP32[$db>>2]|0;
         $3626 = ((($3625)) + 24|0);
         $3627 = HEAP32[$3626>>2]|0;
         $3628 = $3627 | 2;
         HEAP32[$3626>>2] = $3628;
        } else {
         $3629 = ((($3619)) + 8|0);
         $3630 = HEAP32[$3629>>2]|0;
         $3631 = ($3630|0)==(2);
         if ($3631) {
          $3632 = HEAP32[$pOp>>2]|0;
          $3633 = ((($3632)) + 12|0);
          $3634 = HEAP32[$3633>>2]|0;
          $3635 = $3634&255;
          $3636 = HEAP32[$pDb>>2]|0;
          $3637 = ((($3636)) + 12|0);
          $3638 = HEAP32[$3637>>2]|0;
          $3639 = ((($3638)) + 76|0);
          HEAP8[$3639>>0] = $3635;
         }
        }
        $3640 = HEAP32[$pOp>>2]|0;
        $3641 = ((($3640)) + 4|0);
        $3642 = HEAP32[$3641>>2]|0;
        $3643 = ($3642|0)==(1);
        if ($3643) {
         $3644 = HEAP32[$db>>2]|0;
         _sqlite3ExpirePreparedStatements($3644);
         $3645 = HEAP32[$0>>2]|0;
         $3646 = ((($3645)) + 144|0);
         $3647 = HEAP16[$3646>>1]|0;
         $3648 = $3647 & -2;
         HEAP16[$3646>>1] = $3648;
        }
        $3649 = HEAP32[$rc>>2]|0;
        $3650 = ($3649|0)!=(0);
        if ($3650) {
         break L1;
        }
        break;
       }
       case 103:  {
        $3651 = HEAP32[$pOp>>2]|0;
        $3652 = ((($3651)) + 4|0);
        $3653 = HEAP32[$3652>>2]|0;
        $3654 = HEAP32[$0>>2]|0;
        $3655 = ((($3654)) + 112|0);
        $3656 = HEAP32[$3655>>2]|0;
        $3657 = (($3656) + ($3653<<2)|0);
        $3658 = HEAP32[$3657>>2]|0;
        HEAP32[$pCur>>2] = $3658;
        $3659 = HEAP32[$pCur>>2]|0;
        $3660 = ($3659|0)!=(0|0);
        if ($3660) {
         $3661 = HEAP32[$pCur>>2]|0;
         $3662 = ((($3661)) + 8|0);
         $3663 = HEAP32[$3662>>2]|0;
         $3664 = HEAP32[$pOp>>2]|0;
         $3665 = ((($3664)) + 8|0);
         $3666 = HEAP32[$3665>>2]|0;
         $3667 = ($3663|0)==($3666|0);
         if ($3667) {
          label = 458;
         } else {
          label = 444;
         }
        } else {
         label = 444;
        }
        break;
       }
       case 105: case 104:  {
        label = 444;
        break;
       }
       case 107: case 106:  {
        $3806 = HEAP32[$0>>2]|0;
        $3807 = HEAP32[$pOp>>2]|0;
        $3808 = ((($3807)) + 4|0);
        $3809 = HEAP32[$3808>>2]|0;
        $3810 = HEAP32[$pOp>>2]|0;
        $3811 = ((($3810)) + 8|0);
        $3812 = HEAP32[$3811>>2]|0;
        $3813 = (_allocateCursor($3806,$3809,$3812,-1,0)|0);
        HEAP32[$pCx>>2] = $3813;
        $3814 = HEAP32[$pCx>>2]|0;
        $3815 = ($3814|0)==(0|0);
        if ($3815) {
         label = 889;
         break L1;
        }
        $3816 = HEAP32[$pCx>>2]|0;
        $3817 = ((($3816)) + 2|0);
        HEAP8[$3817>>0] = 1;
        $3818 = HEAP32[$pCx>>2]|0;
        $3819 = ((($3818)) + 5|0);
        $3820 = HEAP8[$3819>>0]|0;
        $3821 = $3820 & -2;
        $3822 = $3821 | 1;
        HEAP8[$3819>>0] = $3822;
        $3823 = HEAP32[$db>>2]|0;
        $3824 = HEAP32[$3823>>2]|0;
        $3825 = HEAP32[$db>>2]|0;
        $3826 = HEAP32[$pCx>>2]|0;
        $3827 = ((($3826)) + 20|0);
        $3828 = HEAP32[$pOp>>2]|0;
        $3829 = ((($3828)) + 3|0);
        $3830 = HEAP8[$3829>>0]|0;
        $3831 = $3830&255;
        $3832 = 5 | $3831;
        $3833 = (_sqlite3BtreeOpen($3824,0,$3825,$3827,$3832,1054)|0);
        HEAP32[$rc>>2] = $3833;
        $3834 = HEAP32[$rc>>2]|0;
        $3835 = ($3834|0)==(0);
        if ($3835) {
         $3836 = HEAP32[$pCx>>2]|0;
         $3837 = ((($3836)) + 20|0);
         $3838 = HEAP32[$3837>>2]|0;
         $3839 = (_sqlite3BtreeBeginTrans($3838,1)|0);
         HEAP32[$rc>>2] = $3839;
        }
        $3840 = HEAP32[$rc>>2]|0;
        $3841 = ($3840|0)==(0);
        do {
         if ($3841) {
          $3842 = HEAP32[$pOp>>2]|0;
          $3843 = ((($3842)) + 16|0);
          $3844 = HEAP32[$3843>>2]|0;
          HEAP32[$pKeyInfo25>>2] = $3844;
          $3845 = ($3844|0)!=(0|0);
          $3846 = HEAP32[$pCx>>2]|0;
          $3847 = ((($3846)) + 20|0);
          $3848 = HEAP32[$3847>>2]|0;
          if (!($3845)) {
           $3871 = HEAP32[$pCx>>2]|0;
           $3872 = ((($3871)) + 16|0);
           $3873 = HEAP32[$3872>>2]|0;
           $3874 = (_sqlite3BtreeCursor($3848,1,4,0,$3873)|0);
           HEAP32[$rc>>2] = $3874;
           $3875 = HEAP32[$pCx>>2]|0;
           $3876 = ((($3875)) + 4|0);
           HEAP8[$3876>>0] = 1;
           break;
          }
          $3849 = HEAP32[$pOp>>2]|0;
          $3850 = ((($3849)) + 3|0);
          $3851 = HEAP8[$3850>>0]|0;
          $3852 = $3851&255;
          $3853 = 2 | $3852;
          $3854 = (_sqlite3BtreeCreateTable($3848,$pgno,$3853)|0);
          HEAP32[$rc>>2] = $3854;
          $3855 = HEAP32[$rc>>2]|0;
          $3856 = ($3855|0)==(0);
          if ($3856) {
           $3857 = HEAP32[$pKeyInfo25>>2]|0;
           $3858 = HEAP32[$pCx>>2]|0;
           $3859 = ((($3858)) + 24|0);
           HEAP32[$3859>>2] = $3857;
           $3860 = HEAP32[$pCx>>2]|0;
           $3861 = ((($3860)) + 20|0);
           $3862 = HEAP32[$3861>>2]|0;
           $3863 = HEAP32[$pgno>>2]|0;
           $3864 = HEAP32[$pKeyInfo25>>2]|0;
           $3865 = HEAP32[$pCx>>2]|0;
           $3866 = ((($3865)) + 16|0);
           $3867 = HEAP32[$3866>>2]|0;
           $3868 = (_sqlite3BtreeCursor($3862,$3863,4,$3864,$3867)|0);
           HEAP32[$rc>>2] = $3868;
          }
          $3869 = HEAP32[$pCx>>2]|0;
          $3870 = ((($3869)) + 4|0);
          HEAP8[$3870>>0] = 0;
         }
        } while(0);
        $3877 = HEAP32[$rc>>2]|0;
        $3878 = ($3877|0)!=(0);
        if ($3878) {
         break L1;
        }
        $3879 = HEAP32[$pOp>>2]|0;
        $3880 = ((($3879)) + 3|0);
        $3881 = HEAP8[$3880>>0]|0;
        $3882 = $3881&255;
        $3883 = ($3882|0)!=(8);
        $3884 = $3883&1;
        $3885 = HEAP32[$pCx>>2]|0;
        $3886 = ((($3885)) + 5|0);
        $3887 = $3884&255;
        $3888 = HEAP8[$3886>>0]|0;
        $3889 = $3887 & 1;
        $3890 = ($3889 << 2)&255;
        $3891 = $3888 & -5;
        $3892 = $3891 | $3890;
        HEAP8[$3886>>0] = $3892;
        break;
       }
       case 108:  {
        $3893 = HEAP32[$0>>2]|0;
        $3894 = HEAP32[$pOp>>2]|0;
        $3895 = ((($3894)) + 4|0);
        $3896 = HEAP32[$3895>>2]|0;
        $3897 = HEAP32[$pOp>>2]|0;
        $3898 = ((($3897)) + 8|0);
        $3899 = HEAP32[$3898>>2]|0;
        $3900 = (_allocateCursor($3893,$3896,$3899,-1,1)|0);
        HEAP32[$pCx26>>2] = $3900;
        $3901 = HEAP32[$pCx26>>2]|0;
        $3902 = ($3901|0)==(0|0);
        if ($3902) {
         label = 889;
         break L1;
        }
        $3903 = HEAP32[$pOp>>2]|0;
        $3904 = ((($3903)) + 16|0);
        $3905 = HEAP32[$3904>>2]|0;
        $3906 = HEAP32[$pCx26>>2]|0;
        $3907 = ((($3906)) + 24|0);
        HEAP32[$3907>>2] = $3905;
        $3908 = HEAP32[$db>>2]|0;
        $3909 = HEAP32[$pOp>>2]|0;
        $3910 = ((($3909)) + 12|0);
        $3911 = HEAP32[$3910>>2]|0;
        $3912 = HEAP32[$pCx26>>2]|0;
        $3913 = (_sqlite3VdbeSorterInit($3908,$3911,$3912)|0);
        HEAP32[$rc>>2] = $3913;
        $3914 = HEAP32[$rc>>2]|0;
        $3915 = ($3914|0)!=(0);
        if ($3915) {
         break L1;
        }
        break;
       }
       case 109:  {
        $3916 = HEAP32[$pOp>>2]|0;
        $3917 = ((($3916)) + 4|0);
        $3918 = HEAP32[$3917>>2]|0;
        $3919 = HEAP32[$0>>2]|0;
        $3920 = ((($3919)) + 112|0);
        $3921 = HEAP32[$3920>>2]|0;
        $3922 = (($3921) + ($3918<<2)|0);
        $3923 = HEAP32[$3922>>2]|0;
        HEAP32[$pC27>>2] = $3923;
        $3924 = HEAP32[$pC27>>2]|0;
        $3925 = ((($3924)) + 32|0);
        $3926 = $3925;
        $3927 = $3926;
        $3928 = HEAP32[$3927>>2]|0;
        $3929 = (($3926) + 4)|0;
        $3930 = $3929;
        $3931 = HEAP32[$3930>>2]|0;
        $3932 = (_i64Add(($3928|0),($3931|0),1,0)|0);
        $3933 = tempRet0;
        $3934 = $3925;
        $3935 = $3934;
        HEAP32[$3935>>2] = $3932;
        $3936 = (($3934) + 4)|0;
        $3937 = $3936;
        HEAP32[$3937>>2] = $3933;
        $3938 = ($3928|0)==(0);
        $3939 = ($3931|0)==(0);
        $3940 = $3938 & $3939;
        if ($3940) {
         label = 14;
        }
        break;
       }
       case 110:  {
        $3941 = HEAP32[$0>>2]|0;
        $3942 = HEAP32[$pOp>>2]|0;
        $3943 = ((($3942)) + 4|0);
        $3944 = HEAP32[$3943>>2]|0;
        $3945 = HEAP32[$pOp>>2]|0;
        $3946 = ((($3945)) + 12|0);
        $3947 = HEAP32[$3946>>2]|0;
        $3948 = (_allocateCursor($3941,$3944,$3947,-1,3)|0);
        HEAP32[$pCx28>>2] = $3948;
        $3949 = HEAP32[$pCx28>>2]|0;
        $3950 = ($3949|0)==(0|0);
        if ($3950) {
         label = 889;
         break L1;
        }
        $3951 = HEAP32[$pCx28>>2]|0;
        $3952 = ((($3951)) + 2|0);
        HEAP8[$3952>>0] = 1;
        $3953 = HEAP32[$pOp>>2]|0;
        $3954 = ((($3953)) + 8|0);
        $3955 = HEAP32[$3954>>2]|0;
        $3956 = HEAP32[$pCx28>>2]|0;
        $3957 = ((($3956)) + 16|0);
        HEAP32[$3957>>2] = $3955;
        $3958 = HEAP32[$pCx28>>2]|0;
        $3959 = ((($3958)) + 4|0);
        HEAP8[$3959>>0] = 1;
        break;
       }
       case 111:  {
        $3960 = HEAP32[$0>>2]|0;
        $3961 = HEAP32[$pOp>>2]|0;
        $3962 = ((($3961)) + 4|0);
        $3963 = HEAP32[$3962>>2]|0;
        $3964 = HEAP32[$0>>2]|0;
        $3965 = ((($3964)) + 112|0);
        $3966 = HEAP32[$3965>>2]|0;
        $3967 = (($3966) + ($3963<<2)|0);
        $3968 = HEAP32[$3967>>2]|0;
        _sqlite3VdbeFreeCursor($3960,$3968);
        $3969 = HEAP32[$pOp>>2]|0;
        $3970 = ((($3969)) + 4|0);
        $3971 = HEAP32[$3970>>2]|0;
        $3972 = HEAP32[$0>>2]|0;
        $3973 = ((($3972)) + 112|0);
        $3974 = HEAP32[$3973>>2]|0;
        $3975 = (($3974) + ($3971<<2)|0);
        HEAP32[$3975>>2] = 0;
        break;
       }
       case 26: case 25: case 24: case 23:  {
        $3976 = HEAP32[$pOp>>2]|0;
        $3977 = ((($3976)) + 4|0);
        $3978 = HEAP32[$3977>>2]|0;
        $3979 = HEAP32[$0>>2]|0;
        $3980 = ((($3979)) + 112|0);
        $3981 = HEAP32[$3980>>2]|0;
        $3982 = (($3981) + ($3978<<2)|0);
        $3983 = HEAP32[$3982>>2]|0;
        HEAP32[$pC30>>2] = $3983;
        $3984 = HEAP32[$pOp>>2]|0;
        $3985 = HEAP8[$3984>>0]|0;
        $3986 = $3985&255;
        HEAP32[$oc>>2] = $3986;
        HEAP32[$eqOnly>>2] = 0;
        $3987 = HEAP32[$pC30>>2]|0;
        $3988 = ((($3987)) + 2|0);
        HEAP8[$3988>>0] = 0;
        $3989 = HEAP32[$pC30>>2]|0;
        $3990 = ((($3989)) + 4|0);
        $3991 = HEAP8[$3990>>0]|0;
        $3992 = ($3991<<24>>24)!=(0);
        if ($3992) {
         $3993 = HEAP32[$pOp>>2]|0;
         $3994 = ((($3993)) + 12|0);
         $3995 = HEAP32[$3994>>2]|0;
         $3996 = HEAP32[$aMem>>2]|0;
         $3997 = (($3996) + (($3995*40)|0)|0);
         HEAP32[$pIn3>>2] = $3997;
         $3998 = HEAP32[$pIn3>>2]|0;
         $3999 = ((($3998)) + 8|0);
         $4000 = HEAP16[$3999>>1]|0;
         $4001 = $4000&65535;
         $4002 = $4001 & 14;
         $4003 = ($4002|0)==(2);
         if ($4003) {
          $4004 = HEAP32[$pIn3>>2]|0;
          _applyNumericAffinity($4004,0);
         }
         $4005 = HEAP32[$pIn3>>2]|0;
         $4006 = (_sqlite3VdbeIntValue($4005)|0);
         $4007 = tempRet0;
         $4008 = $iKey;
         $4009 = $4008;
         HEAP32[$4009>>2] = $4006;
         $4010 = (($4008) + 4)|0;
         $4011 = $4010;
         HEAP32[$4011>>2] = $4007;
         $4012 = HEAP32[$pIn3>>2]|0;
         $4013 = ((($4012)) + 8|0);
         $4014 = HEAP16[$4013>>1]|0;
         $4015 = $4014&65535;
         $4016 = $4015 & 4;
         $4017 = ($4016|0)==(0);
         do {
          if ($4017) {
           $4018 = HEAP32[$pIn3>>2]|0;
           $4019 = ((($4018)) + 8|0);
           $4020 = HEAP16[$4019>>1]|0;
           $4021 = $4020&65535;
           $4022 = $4021 & 8;
           $4023 = ($4022|0)==(0);
           if ($4023) {
            label = 14;
            break L10;
           }
           $4024 = HEAP32[$pIn3>>2]|0;
           $4025 = +HEAPF64[$4024>>3];
           $4026 = $iKey;
           $4027 = $4026;
           $4028 = HEAP32[$4027>>2]|0;
           $4029 = (($4026) + 4)|0;
           $4030 = $4029;
           $4031 = HEAP32[$4030>>2]|0;
           $4032 = (+($4028>>>0)) + (4294967296.0*(+($4031|0)));
           $4033 = $4025 < $4032;
           if ($4033) {
            $4034 = HEAP32[$oc>>2]|0;
            $4035 = $4034 & 1;
            $4036 = ($4035|0)==(0);
            if (!($4036)) {
             break;
            }
            $4037 = HEAP32[$oc>>2]|0;
            $4038 = (($4037) + -1)|0;
            HEAP32[$oc>>2] = $4038;
            break;
           }
           $4039 = HEAP32[$pIn3>>2]|0;
           $4040 = +HEAPF64[$4039>>3];
           $4041 = $iKey;
           $4042 = $4041;
           $4043 = HEAP32[$4042>>2]|0;
           $4044 = (($4041) + 4)|0;
           $4045 = $4044;
           $4046 = HEAP32[$4045>>2]|0;
           $4047 = (+($4043>>>0)) + (4294967296.0*(+($4046|0)));
           $4048 = $4040 > $4047;
           if ($4048) {
            $4049 = HEAP32[$oc>>2]|0;
            $4050 = $4049 & 1;
            $4051 = ($4050|0)==(1);
            if ($4051) {
             $4052 = HEAP32[$oc>>2]|0;
             $4053 = (($4052) + 1)|0;
             HEAP32[$oc>>2] = $4053;
            }
           }
          }
         } while(0);
         $4054 = HEAP32[$pC30>>2]|0;
         $4055 = ((($4054)) + 16|0);
         $4056 = HEAP32[$4055>>2]|0;
         $4057 = $iKey;
         $4058 = $4057;
         $4059 = HEAP32[$4058>>2]|0;
         $4060 = (($4057) + 4)|0;
         $4061 = $4060;
         $4062 = HEAP32[$4061>>2]|0;
         $4063 = (_sqlite3BtreeMovetoUnpacked($4056,0,$4059,$4062,0,$res29)|0);
         HEAP32[$rc>>2] = $4063;
         $4064 = $iKey;
         $4065 = $4064;
         $4066 = HEAP32[$4065>>2]|0;
         $4067 = (($4064) + 4)|0;
         $4068 = $4067;
         $4069 = HEAP32[$4068>>2]|0;
         $4070 = HEAP32[$pC30>>2]|0;
         $4071 = ((($4070)) + 40|0);
         $4072 = $4071;
         $4073 = $4072;
         HEAP32[$4073>>2] = $4066;
         $4074 = (($4072) + 4)|0;
         $4075 = $4074;
         HEAP32[$4075>>2] = $4069;
         $4076 = HEAP32[$rc>>2]|0;
         $4077 = ($4076|0)!=(0);
         if ($4077) {
          break L1;
         } else {
          label = 493;
         }
        } else {
         $4078 = HEAP32[$pC30>>2]|0;
         $4079 = ((($4078)) + 16|0);
         $4080 = HEAP32[$4079>>2]|0;
         $4081 = (_sqlite3BtreeCursorHasHint($4080,2)|0);
         $4082 = ($4081|0)!=(0);
         if ($4082) {
          HEAP32[$eqOnly>>2] = 1;
         }
         $4083 = HEAP32[$pOp>>2]|0;
         $4084 = ((($4083)) + 16|0);
         $4085 = HEAP32[$4084>>2]|0;
         HEAP32[$nField31>>2] = $4085;
         $4086 = HEAP32[$pC30>>2]|0;
         $4087 = ((($4086)) + 24|0);
         $4088 = HEAP32[$4087>>2]|0;
         HEAP32[$r>>2] = $4088;
         $4089 = HEAP32[$nField31>>2]|0;
         $4090 = $4089&65535;
         $4091 = ((($r)) + 8|0);
         HEAP16[$4091>>1] = $4090;
         $4092 = HEAP32[$oc>>2]|0;
         $4093 = (($4092) - 23)|0;
         $4094 = 1 & $4093;
         $4095 = ($4094|0)!=(0);
         $4096 = $4095 ? -1 : 1;
         $4097 = $4096&255;
         $4098 = ((($r)) + 10|0);
         HEAP8[$4098>>0] = $4097;
         $4099 = HEAP32[$pOp>>2]|0;
         $4100 = ((($4099)) + 12|0);
         $4101 = HEAP32[$4100>>2]|0;
         $4102 = HEAP32[$aMem>>2]|0;
         $4103 = (($4102) + (($4101*40)|0)|0);
         $4104 = ((($r)) + 4|0);
         HEAP32[$4104>>2] = $4103;
         $4105 = ((($r)) + 14|0);
         HEAP8[$4105>>0] = 0;
         $4106 = HEAP32[$pC30>>2]|0;
         $4107 = ((($4106)) + 16|0);
         $4108 = HEAP32[$4107>>2]|0;
         $4109 = (_sqlite3BtreeMovetoUnpacked($4108,$r,0,0,0,$res29)|0);
         HEAP32[$rc>>2] = $4109;
         $4110 = HEAP32[$rc>>2]|0;
         $4111 = ($4110|0)!=(0);
         if ($4111) {
          break L1;
         }
         $4112 = HEAP32[$eqOnly>>2]|0;
         $4113 = ($4112|0)!=(0);
         if ($4113) {
          $4114 = ((($r)) + 14|0);
          $4115 = HEAP8[$4114>>0]|0;
          $4116 = $4115&255;
          $4117 = ($4116|0)==(0);
          if (!($4117)) {
           label = 493;
          }
         } else {
          label = 493;
         }
        }
        do {
         if ((label|0) == 493) {
          label = 0;
          $4118 = HEAP32[$pC30>>2]|0;
          $4119 = ((($4118)) + 3|0);
          HEAP8[$4119>>0] = 0;
          $4120 = HEAP32[$pC30>>2]|0;
          $4121 = ((($4120)) + 56|0);
          HEAP32[$4121>>2] = 0;
          $4122 = HEAP32[$oc>>2]|0;
          $4123 = ($4122|0)>=(25);
          $4124 = HEAP32[$res29>>2]|0;
          if ($4123) {
           $4125 = ($4124|0)<(0);
           if (!($4125)) {
            $4126 = HEAP32[$res29>>2]|0;
            $4127 = ($4126|0)==(0);
            $4128 = HEAP32[$oc>>2]|0;
            $4129 = ($4128|0)==(26);
            $or$cond12 = $4127 & $4129;
            if (!($or$cond12)) {
             HEAP32[$res29>>2] = 0;
             break;
            }
           }
           HEAP32[$res29>>2] = 0;
           $4130 = HEAP32[$pC30>>2]|0;
           $4131 = ((($4130)) + 16|0);
           $4132 = HEAP32[$4131>>2]|0;
           $4133 = (_sqlite3BtreeNext($4132,$res29)|0);
           HEAP32[$rc>>2] = $4133;
           $4134 = HEAP32[$rc>>2]|0;
           $4135 = ($4134|0)!=(0);
           if ($4135) {
            break L1;
           } else {
            break;
           }
          } else {
           $4136 = ($4124|0)>(0);
           if (!($4136)) {
            $4137 = HEAP32[$res29>>2]|0;
            $4138 = ($4137|0)==(0);
            $4139 = HEAP32[$oc>>2]|0;
            $4140 = ($4139|0)==(23);
            $or$cond14 = $4138 & $4140;
            if (!($or$cond14)) {
             $4147 = HEAP32[$pC30>>2]|0;
             $4148 = ((($4147)) + 16|0);
             $4149 = HEAP32[$4148>>2]|0;
             $4150 = (_sqlite3BtreeEof($4149)|0);
             HEAP32[$res29>>2] = $4150;
             break;
            }
           }
           HEAP32[$res29>>2] = 0;
           $4141 = HEAP32[$pC30>>2]|0;
           $4142 = ((($4141)) + 16|0);
           $4143 = HEAP32[$4142>>2]|0;
           $4144 = (_sqlite3BtreePrevious($4143,$res29)|0);
           HEAP32[$rc>>2] = $4144;
           $4145 = HEAP32[$rc>>2]|0;
           $4146 = ($4145|0)!=(0);
           if ($4146) {
            break L1;
           } else {
            break;
           }
          }
         }
        } while(0);
        $4151 = HEAP32[$res29>>2]|0;
        $4152 = ($4151|0)!=(0);
        if ($4152) {
         label = 14;
        } else {
         $4153 = HEAP32[$eqOnly>>2]|0;
         $4154 = ($4153|0)!=(0);
         if ($4154) {
          $4155 = HEAP32[$pOp>>2]|0;
          $4156 = ((($4155)) + 20|0);
          HEAP32[$pOp>>2] = $4156;
         }
        }
        break;
       }
       case 31: case 30: case 29:  {
        $4157 = HEAP32[$pOp>>2]|0;
        $4158 = ((($4157)) + 4|0);
        $4159 = HEAP32[$4158>>2]|0;
        $4160 = HEAP32[$0>>2]|0;
        $4161 = ((($4160)) + 112|0);
        $4162 = HEAP32[$4161>>2]|0;
        $4163 = (($4162) + ($4159<<2)|0);
        $4164 = HEAP32[$4163>>2]|0;
        HEAP32[$pC33>>2] = $4164;
        $4165 = HEAP32[$pOp>>2]|0;
        $4166 = ((($4165)) + 12|0);
        $4167 = HEAP32[$4166>>2]|0;
        $4168 = HEAP32[$aMem>>2]|0;
        $4169 = (($4168) + (($4167*40)|0)|0);
        HEAP32[$pIn3>>2] = $4169;
        HEAP32[$pFree>>2] = 0;
        $4170 = HEAP32[$pOp>>2]|0;
        $4171 = ((($4170)) + 16|0);
        $4172 = HEAP32[$4171>>2]|0;
        $4173 = ($4172|0)>(0);
        $4174 = HEAP32[$pC33>>2]|0;
        $4175 = ((($4174)) + 24|0);
        $4176 = HEAP32[$4175>>2]|0;
        if ($4173) {
         HEAP32[$r35>>2] = $4176;
         $4177 = HEAP32[$pOp>>2]|0;
         $4178 = ((($4177)) + 16|0);
         $4179 = HEAP32[$4178>>2]|0;
         $4180 = $4179&65535;
         $4181 = ((($r35)) + 8|0);
         HEAP16[$4181>>1] = $4180;
         $4182 = HEAP32[$pIn3>>2]|0;
         $4183 = ((($r35)) + 4|0);
         HEAP32[$4183>>2] = $4182;
         HEAP32[$pIdxKey>>2] = $r35;
        } else {
         $4184 = (_sqlite3VdbeAllocUnpackedRecord($4176,$aTempRec,183,$pFree)|0);
         HEAP32[$pIdxKey>>2] = $4184;
         $4185 = HEAP32[$pIdxKey>>2]|0;
         $4186 = ($4185|0)==(0|0);
         if ($4186) {
          label = 889;
          break L1;
         }
         $4187 = HEAP32[$pIn3>>2]|0;
         $4188 = ((($4187)) + 8|0);
         $4189 = HEAP16[$4188>>1]|0;
         $4190 = $4189&65535;
         $4191 = $4190 & 16384;
         $4192 = ($4191|0)!=(0);
         if ($4192) {
          $4193 = HEAP32[$pIn3>>2]|0;
          (_sqlite3VdbeMemExpandBlob($4193)|0);
         }
         $4194 = HEAP32[$pC33>>2]|0;
         $4195 = ((($4194)) + 24|0);
         $4196 = HEAP32[$4195>>2]|0;
         $4197 = HEAP32[$pIn3>>2]|0;
         $4198 = ((($4197)) + 12|0);
         $4199 = HEAP32[$4198>>2]|0;
         $4200 = HEAP32[$pIn3>>2]|0;
         $4201 = ((($4200)) + 16|0);
         $4202 = HEAP32[$4201>>2]|0;
         $4203 = HEAP32[$pIdxKey>>2]|0;
         _sqlite3VdbeRecordUnpack($4196,$4199,$4202,$4203);
        }
        $4204 = HEAP32[$pIdxKey>>2]|0;
        $4205 = ((($4204)) + 10|0);
        HEAP8[$4205>>0] = 0;
        HEAP32[$takeJump>>2] = 0;
        $4206 = HEAP32[$pOp>>2]|0;
        $4207 = HEAP8[$4206>>0]|0;
        $4208 = $4207&255;
        $4209 = ($4208|0)==(29);
        L554: do {
         if ($4209) {
          HEAP32[$ii32>>2] = 0;
          while(1) {
           $4210 = HEAP32[$ii32>>2]|0;
           $4211 = HEAP32[$pIdxKey>>2]|0;
           $4212 = ((($4211)) + 8|0);
           $4213 = HEAP16[$4212>>1]|0;
           $4214 = $4213&65535;
           $4215 = ($4210|0)<($4214|0);
           if (!($4215)) {
            break L554;
           }
           $4216 = HEAP32[$ii32>>2]|0;
           $4217 = HEAP32[$pIdxKey>>2]|0;
           $4218 = ((($4217)) + 4|0);
           $4219 = HEAP32[$4218>>2]|0;
           $4220 = (($4219) + (($4216*40)|0)|0);
           $4221 = ((($4220)) + 8|0);
           $4222 = HEAP16[$4221>>1]|0;
           $4223 = $4222&65535;
           $4224 = $4223 & 1;
           $4225 = ($4224|0)!=(0);
           if ($4225) {
            break;
           }
           $4226 = HEAP32[$ii32>>2]|0;
           $4227 = (($4226) + 1)|0;
           HEAP32[$ii32>>2] = $4227;
          }
          HEAP32[$takeJump>>2] = 1;
         }
        } while(0);
        $4228 = HEAP32[$pC33>>2]|0;
        $4229 = ((($4228)) + 16|0);
        $4230 = HEAP32[$4229>>2]|0;
        $4231 = HEAP32[$pIdxKey>>2]|0;
        $4232 = (_sqlite3BtreeMovetoUnpacked($4230,$4231,0,0,0,$res34)|0);
        HEAP32[$rc>>2] = $4232;
        $4233 = HEAP32[$db>>2]|0;
        $4234 = HEAP32[$pFree>>2]|0;
        _sqlite3DbFree($4233,$4234);
        $4235 = HEAP32[$rc>>2]|0;
        $4236 = ($4235|0)!=(0);
        if ($4236) {
         break L1;
        }
        $4237 = HEAP32[$res34>>2]|0;
        $4238 = HEAP32[$pC33>>2]|0;
        $4239 = ((($4238)) + 28|0);
        HEAP32[$4239>>2] = $4237;
        $4240 = HEAP32[$res34>>2]|0;
        $4241 = ($4240|0)==(0);
        $4242 = $4241&1;
        HEAP32[$alreadyExists>>2] = $4242;
        $4243 = HEAP32[$alreadyExists>>2]|0;
        $4244 = (1 - ($4243))|0;
        $4245 = $4244&255;
        $4246 = HEAP32[$pC33>>2]|0;
        $4247 = ((($4246)) + 2|0);
        HEAP8[$4247>>0] = $4245;
        $4248 = HEAP32[$pC33>>2]|0;
        $4249 = ((($4248)) + 3|0);
        HEAP8[$4249>>0] = 0;
        $4250 = HEAP32[$pC33>>2]|0;
        $4251 = ((($4250)) + 56|0);
        HEAP32[$4251>>2] = 0;
        $4252 = HEAP32[$pOp>>2]|0;
        $4253 = HEAP8[$4252>>0]|0;
        $4254 = $4253&255;
        $4255 = ($4254|0)==(31);
        if ($4255) {
         $4256 = HEAP32[$alreadyExists>>2]|0;
         $4257 = ($4256|0)!=(0);
         if ($4257) {
          label = 14;
          break L10;
         } else {
          break L10;
         }
        } else {
         $4258 = HEAP32[$takeJump>>2]|0;
         $4259 = ($4258|0)==(0);
         $4260 = HEAP32[$alreadyExists>>2]|0;
         $4261 = ($4260|0)!=(0);
         $or$cond16 = $4259 & $4261;
         if ($or$cond16) {
          break L10;
         } else {
          label = 14;
          break L10;
         }
        }
        break;
       }
       case 32:  {
        $4262 = HEAP32[$pOp>>2]|0;
        $4263 = ((($4262)) + 12|0);
        $4264 = HEAP32[$4263>>2]|0;
        $4265 = HEAP32[$aMem>>2]|0;
        $4266 = (($4265) + (($4264*40)|0)|0);
        HEAP32[$pIn3>>2] = $4266;
        $4267 = HEAP32[$pIn3>>2]|0;
        $4268 = ((($4267)) + 8|0);
        $4269 = HEAP16[$4268>>1]|0;
        $4270 = $4269&65535;
        $4271 = $4270 & 4;
        $4272 = ($4271|0)==(0);
        if ($4272) {
         $4273 = HEAP32[$pIn3>>2]|0;
         $4274 = HEAP8[$encoding>>0]|0;
         _applyAffinity($4273,67,$4274);
         $4275 = HEAP32[$pIn3>>2]|0;
         $4276 = ((($4275)) + 8|0);
         $4277 = HEAP16[$4276>>1]|0;
         $4278 = $4277&65535;
         $4279 = $4278 & 4;
         $4280 = ($4279|0)==(0);
         if ($4280) {
          label = 14;
         } else {
          label = 523;
         }
        } else {
         label = 523;
        }
        break;
       }
       case 33:  {
        label = 523;
        break;
       }
       case 113:  {
        $4346 = HEAP32[$0>>2]|0;
        $4347 = HEAP32[$pOp>>2]|0;
        $4348 = (_out2Prerelease($4346,$4347)|0);
        HEAP32[$pOut>>2] = $4348;
        $4349 = HEAP32[$pOp>>2]|0;
        $4350 = ((($4349)) + 4|0);
        $4351 = HEAP32[$4350>>2]|0;
        $4352 = HEAP32[$0>>2]|0;
        $4353 = ((($4352)) + 112|0);
        $4354 = HEAP32[$4353>>2]|0;
        $4355 = (($4354) + ($4351<<2)|0);
        $4356 = HEAP32[$4355>>2]|0;
        $4357 = ((($4356)) + 32|0);
        $4358 = $4357;
        $4359 = $4358;
        $4360 = HEAP32[$4359>>2]|0;
        $4361 = (($4358) + 4)|0;
        $4362 = $4361;
        $4363 = HEAP32[$4362>>2]|0;
        $4364 = (_i64Add(($4360|0),($4363|0),1,0)|0);
        $4365 = tempRet0;
        $4366 = $4357;
        $4367 = $4366;
        HEAP32[$4367>>2] = $4364;
        $4368 = (($4366) + 4)|0;
        $4369 = $4368;
        HEAP32[$4369>>2] = $4365;
        $4370 = HEAP32[$pOut>>2]|0;
        $4371 = $4370;
        $4372 = $4371;
        HEAP32[$4372>>2] = $4360;
        $4373 = (($4371) + 4)|0;
        $4374 = $4373;
        HEAP32[$4374>>2] = $4363;
        break;
       }
       case 114:  {
        $4375 = $v;
        $4376 = $4375;
        HEAP32[$4376>>2] = 0;
        $4377 = (($4375) + 4)|0;
        $4378 = $4377;
        HEAP32[$4378>>2] = 0;
        HEAP32[$res41>>2] = 0;
        $4379 = HEAP32[$0>>2]|0;
        $4380 = HEAP32[$pOp>>2]|0;
        $4381 = (_out2Prerelease($4379,$4380)|0);
        HEAP32[$pOut>>2] = $4381;
        $4382 = HEAP32[$pOp>>2]|0;
        $4383 = ((($4382)) + 4|0);
        $4384 = HEAP32[$4383>>2]|0;
        $4385 = HEAP32[$0>>2]|0;
        $4386 = ((($4385)) + 112|0);
        $4387 = HEAP32[$4386>>2]|0;
        $4388 = (($4387) + ($4384<<2)|0);
        $4389 = HEAP32[$4388>>2]|0;
        HEAP32[$pC40>>2] = $4389;
        $4390 = HEAP32[$pC40>>2]|0;
        $4391 = ((($4390)) + 5|0);
        $4392 = HEAP8[$4391>>0]|0;
        $4393 = ($4392&255) >>> 1;
        $4394 = $4393 & 1;
        $4395 = $4394&255;
        $4396 = ($4395|0)!=(0);
        do {
         if (!($4396)) {
          $4397 = HEAP32[$pC40>>2]|0;
          $4398 = ((($4397)) + 16|0);
          $4399 = HEAP32[$4398>>2]|0;
          $4400 = (_sqlite3BtreeLast($4399,$res41)|0);
          HEAP32[$rc>>2] = $4400;
          $4401 = HEAP32[$rc>>2]|0;
          $4402 = ($4401|0)!=(0);
          if ($4402) {
           break L1;
          }
          $4403 = HEAP32[$res41>>2]|0;
          $4404 = ($4403|0)!=(0);
          if ($4404) {
           $4405 = $v;
           $4406 = $4405;
           HEAP32[$4406>>2] = 1;
           $4407 = (($4405) + 4)|0;
           $4408 = $4407;
           HEAP32[$4408>>2] = 0;
           break;
          }
          $4409 = HEAP32[$pC40>>2]|0;
          $4410 = ((($4409)) + 16|0);
          $4411 = HEAP32[$4410>>2]|0;
          $4412 = (_sqlite3BtreeIntegerKey($4411)|0);
          $4413 = tempRet0;
          $4414 = $v;
          $4415 = $4414;
          HEAP32[$4415>>2] = $4412;
          $4416 = (($4414) + 4)|0;
          $4417 = $4416;
          HEAP32[$4417>>2] = $4413;
          $4418 = $v;
          $4419 = $4418;
          $4420 = HEAP32[$4419>>2]|0;
          $4421 = (($4418) + 4)|0;
          $4422 = $4421;
          $4423 = HEAP32[$4422>>2]|0;
          $4424 = ($4423|0)>(2147483647);
          $4425 = ($4420>>>0)>=(4294967295);
          $4426 = ($4423|0)==(2147483647);
          $4427 = $4426 & $4425;
          $4428 = $4424 | $4427;
          if ($4428) {
           $4429 = HEAP32[$pC40>>2]|0;
           $4430 = ((($4429)) + 5|0);
           $4431 = HEAP8[$4430>>0]|0;
           $4432 = $4431 & -3;
           $4433 = $4432 | 2;
           HEAP8[$4430>>0] = $4433;
           break;
          } else {
           $4434 = $v;
           $4435 = $4434;
           $4436 = HEAP32[$4435>>2]|0;
           $4437 = (($4434) + 4)|0;
           $4438 = $4437;
           $4439 = HEAP32[$4438>>2]|0;
           $4440 = (_i64Add(($4436|0),($4439|0),1,0)|0);
           $4441 = tempRet0;
           $4442 = $v;
           $4443 = $4442;
           HEAP32[$4443>>2] = $4440;
           $4444 = (($4442) + 4)|0;
           $4445 = $4444;
           HEAP32[$4445>>2] = $4441;
           break;
          }
         }
        } while(0);
        $4446 = HEAP32[$pOp>>2]|0;
        $4447 = ((($4446)) + 12|0);
        $4448 = HEAP32[$4447>>2]|0;
        $4449 = ($4448|0)!=(0);
        if ($4449) {
         $4450 = HEAP32[$0>>2]|0;
         $4451 = ((($4450)) + 184|0);
         $4452 = HEAP32[$4451>>2]|0;
         $4453 = ($4452|0)!=(0|0);
         if ($4453) {
          $4454 = HEAP32[$0>>2]|0;
          $4455 = ((($4454)) + 184|0);
          $4456 = HEAP32[$4455>>2]|0;
          HEAP32[$pFrame44>>2] = $4456;
          while(1) {
           $4457 = HEAP32[$pFrame44>>2]|0;
           $4458 = ((($4457)) + 4|0);
           $4459 = HEAP32[$4458>>2]|0;
           $4460 = ($4459|0)!=(0|0);
           if (!($4460)) {
            break;
           }
           $4461 = HEAP32[$pFrame44>>2]|0;
           $4462 = ((($4461)) + 4|0);
           $4463 = HEAP32[$4462>>2]|0;
           HEAP32[$pFrame44>>2] = $4463;
          }
          $4464 = HEAP32[$pOp>>2]|0;
          $4465 = ((($4464)) + 12|0);
          $4466 = HEAP32[$4465>>2]|0;
          $4467 = HEAP32[$pFrame44>>2]|0;
          $4468 = ((($4467)) + 16|0);
          $4469 = HEAP32[$4468>>2]|0;
          $4470 = (($4469) + (($4466*40)|0)|0);
          HEAP32[$pMem43>>2] = $4470;
         } else {
          $4471 = HEAP32[$pOp>>2]|0;
          $4472 = ((($4471)) + 12|0);
          $4473 = HEAP32[$4472>>2]|0;
          $4474 = HEAP32[$aMem>>2]|0;
          $4475 = (($4474) + (($4473*40)|0)|0);
          HEAP32[$pMem43>>2] = $4475;
         }
         $4476 = HEAP32[$pMem43>>2]|0;
         (_sqlite3VdbeMemIntegerify($4476)|0);
         $4477 = HEAP32[$pMem43>>2]|0;
         $4478 = $4477;
         $4479 = $4478;
         $4480 = HEAP32[$4479>>2]|0;
         $4481 = (($4478) + 4)|0;
         $4482 = $4481;
         $4483 = HEAP32[$4482>>2]|0;
         $4484 = ($4480|0)==(-1);
         $4485 = ($4483|0)==(2147483647);
         $4486 = $4484 & $4485;
         if ($4486) {
          label = 544;
          break L8;
         }
         $4487 = HEAP32[$pC40>>2]|0;
         $4488 = ((($4487)) + 5|0);
         $4489 = HEAP8[$4488>>0]|0;
         $4490 = ($4489&255) >>> 1;
         $4491 = $4490 & 1;
         $4492 = $4491&255;
         $4493 = ($4492|0)!=(0);
         if ($4493) {
          label = 544;
          break L8;
         }
         $4494 = $v;
         $4495 = $4494;
         $4496 = HEAP32[$4495>>2]|0;
         $4497 = (($4494) + 4)|0;
         $4498 = $4497;
         $4499 = HEAP32[$4498>>2]|0;
         $4500 = HEAP32[$pMem43>>2]|0;
         $4501 = $4500;
         $4502 = $4501;
         $4503 = HEAP32[$4502>>2]|0;
         $4504 = (($4501) + 4)|0;
         $4505 = $4504;
         $4506 = HEAP32[$4505>>2]|0;
         $4507 = (_i64Add(($4503|0),($4506|0),1,0)|0);
         $4508 = tempRet0;
         $4509 = ($4499|0)<($4508|0);
         $4510 = ($4496>>>0)<($4507>>>0);
         $4511 = ($4499|0)==($4508|0);
         $4512 = $4511 & $4510;
         $4513 = $4509 | $4512;
         if ($4513) {
          $4514 = HEAP32[$pMem43>>2]|0;
          $4515 = $4514;
          $4516 = $4515;
          $4517 = HEAP32[$4516>>2]|0;
          $4518 = (($4515) + 4)|0;
          $4519 = $4518;
          $4520 = HEAP32[$4519>>2]|0;
          $4521 = (_i64Add(($4517|0),($4520|0),1,0)|0);
          $4522 = tempRet0;
          $4523 = $v;
          $4524 = $4523;
          HEAP32[$4524>>2] = $4521;
          $4525 = (($4523) + 4)|0;
          $4526 = $4525;
          HEAP32[$4526>>2] = $4522;
         }
         $4527 = $v;
         $4528 = $4527;
         $4529 = HEAP32[$4528>>2]|0;
         $4530 = (($4527) + 4)|0;
         $4531 = $4530;
         $4532 = HEAP32[$4531>>2]|0;
         $4533 = HEAP32[$pMem43>>2]|0;
         $4534 = $4533;
         $4535 = $4534;
         HEAP32[$4535>>2] = $4529;
         $4536 = (($4534) + 4)|0;
         $4537 = $4536;
         HEAP32[$4537>>2] = $4532;
        }
        $4538 = HEAP32[$pC40>>2]|0;
        $4539 = ((($4538)) + 5|0);
        $4540 = HEAP8[$4539>>0]|0;
        $4541 = ($4540&255) >>> 1;
        $4542 = $4541 & 1;
        $4543 = $4542&255;
        $4544 = ($4543|0)!=(0);
        if ($4544) {
         HEAP32[$cnt42>>2] = 0;
         while(1) {
          _sqlite3_randomness(8,$v);
          $4545 = $v;
          $4546 = $4545;
          $4547 = HEAP32[$4546>>2]|0;
          $4548 = (($4545) + 4)|0;
          $4549 = $4548;
          $4550 = HEAP32[$4549>>2]|0;
          $4551 = $4550 & 1073741823;
          $4552 = $v;
          $4553 = $4552;
          HEAP32[$4553>>2] = $4547;
          $4554 = (($4552) + 4)|0;
          $4555 = $4554;
          HEAP32[$4555>>2] = $4551;
          $4556 = $v;
          $4557 = $4556;
          $4558 = HEAP32[$4557>>2]|0;
          $4559 = (($4556) + 4)|0;
          $4560 = $4559;
          $4561 = HEAP32[$4560>>2]|0;
          $4562 = (_i64Add(($4558|0),($4561|0),1,0)|0);
          $4563 = tempRet0;
          $4564 = $v;
          $4565 = $4564;
          HEAP32[$4565>>2] = $4562;
          $4566 = (($4564) + 4)|0;
          $4567 = $4566;
          HEAP32[$4567>>2] = $4563;
          $4568 = HEAP32[$pC40>>2]|0;
          $4569 = ((($4568)) + 16|0);
          $4570 = HEAP32[$4569>>2]|0;
          $4571 = $v;
          $4572 = $4571;
          $4573 = HEAP32[$4572>>2]|0;
          $4574 = (($4571) + 4)|0;
          $4575 = $4574;
          $4576 = HEAP32[$4575>>2]|0;
          $4577 = (_sqlite3BtreeMovetoUnpacked($4570,0,$4573,$4576,0,$res41)|0);
          HEAP32[$rc>>2] = $4577;
          $4578 = ($4577|0)==(0);
          $4579 = HEAP32[$res41>>2]|0;
          $4580 = ($4579|0)==(0);
          $or$cond18 = $4578 & $4580;
          if (!($or$cond18)) {
           break;
          }
          $4581 = HEAP32[$cnt42>>2]|0;
          $4582 = (($4581) + 1)|0;
          HEAP32[$cnt42>>2] = $4582;
          $4583 = ($4582|0)<(100);
          if (!($4583)) {
           break;
          }
         }
         $4584 = HEAP32[$rc>>2]|0;
         $4585 = ($4584|0)!=(0);
         if ($4585) {
          break L1;
         }
         $4586 = HEAP32[$res41>>2]|0;
         $4587 = ($4586|0)==(0);
         if ($4587) {
          label = 554;
          break L8;
         }
        }
        $4588 = HEAP32[$pC40>>2]|0;
        $4589 = ((($4588)) + 3|0);
        HEAP8[$4589>>0] = 0;
        $4590 = HEAP32[$pC40>>2]|0;
        $4591 = ((($4590)) + 56|0);
        HEAP32[$4591>>2] = 0;
        $4592 = $v;
        $4593 = $4592;
        $4594 = HEAP32[$4593>>2]|0;
        $4595 = (($4592) + 4)|0;
        $4596 = $4595;
        $4597 = HEAP32[$4596>>2]|0;
        $4598 = HEAP32[$pOut>>2]|0;
        $4599 = $4598;
        $4600 = $4599;
        HEAP32[$4600>>2] = $4594;
        $4601 = (($4599) + 4)|0;
        $4602 = $4601;
        HEAP32[$4602>>2] = $4597;
        break;
       }
       case 116: case 115:  {
        HEAP32[$op46>>2] = 0;
        $4603 = HEAP32[$pOp>>2]|0;
        $4604 = ((($4603)) + 8|0);
        $4605 = HEAP32[$4604>>2]|0;
        $4606 = HEAP32[$aMem>>2]|0;
        $4607 = (($4606) + (($4605*40)|0)|0);
        HEAP32[$pData>>2] = $4607;
        $4608 = HEAP32[$pOp>>2]|0;
        $4609 = ((($4608)) + 4|0);
        $4610 = HEAP32[$4609>>2]|0;
        $4611 = HEAP32[$0>>2]|0;
        $4612 = ((($4611)) + 112|0);
        $4613 = HEAP32[$4612>>2]|0;
        $4614 = (($4613) + ($4610<<2)|0);
        $4615 = HEAP32[$4614>>2]|0;
        HEAP32[$pC45>>2] = $4615;
        $4616 = HEAP32[$pOp>>2]|0;
        $4617 = HEAP8[$4616>>0]|0;
        $4618 = $4617&255;
        $4619 = ($4618|0)==(115);
        $4620 = HEAP32[$pOp>>2]|0;
        $4621 = ((($4620)) + 12|0);
        $4622 = HEAP32[$4621>>2]|0;
        if ($4619) {
         $4623 = HEAP32[$aMem>>2]|0;
         $4624 = (($4623) + (($4622*40)|0)|0);
         HEAP32[$pKey>>2] = $4624;
         $4625 = HEAP32[$pKey>>2]|0;
         $4626 = $4625;
         $4627 = $4626;
         $4628 = HEAP32[$4627>>2]|0;
         $4629 = (($4626) + 4)|0;
         $4630 = $4629;
         $4631 = HEAP32[$4630>>2]|0;
         $4632 = ((($x)) + 8|0);
         $4633 = $4632;
         $4634 = $4633;
         HEAP32[$4634>>2] = $4628;
         $4635 = (($4633) + 4)|0;
         $4636 = $4635;
         HEAP32[$4636>>2] = $4631;
        } else {
         $4637 = ($4622|0)<(0);
         $4638 = $4637 << 31 >> 31;
         $4639 = ((($x)) + 8|0);
         $4640 = $4639;
         $4641 = $4640;
         HEAP32[$4641>>2] = $4622;
         $4642 = (($4640) + 4)|0;
         $4643 = $4642;
         HEAP32[$4643>>2] = $4638;
        }
        $4644 = HEAP32[$pOp>>2]|0;
        $4645 = ((($4644)) + 1|0);
        $4646 = HEAP8[$4645>>0]|0;
        $4647 = $4646 << 24 >> 24;
        $4648 = ($4647|0)==(-20);
        if ($4648) {
         $4649 = HEAP32[$db>>2]|0;
         $4650 = ((($4649)) + 220|0);
         $4651 = HEAP32[$4650>>2]|0;
         $4652 = ($4651|0)!=(0|0);
         if ($4652) {
          $4653 = HEAP32[$pC45>>2]|0;
          $4654 = ((($4653)) + 1|0);
          $4655 = HEAP8[$4654>>0]|0;
          $4656 = $4655 << 24 >> 24;
          $4657 = HEAP32[$db>>2]|0;
          $4658 = ((($4657)) + 16|0);
          $4659 = HEAP32[$4658>>2]|0;
          $4660 = (($4659) + ($4656<<4)|0);
          $4661 = HEAP32[$4660>>2]|0;
          HEAP32[$zDb>>2] = $4661;
          $4662 = HEAP32[$pOp>>2]|0;
          $4663 = ((($4662)) + 16|0);
          $4664 = HEAP32[$4663>>2]|0;
          HEAP32[$pTab>>2] = $4664;
          $4665 = HEAP32[$pOp>>2]|0;
          $4666 = ((($4665)) + 3|0);
          $4667 = HEAP8[$4666>>0]|0;
          $4668 = $4667&255;
          $4669 = $4668 & 4;
          $4670 = ($4669|0)!=(0);
          $4671 = $4670 ? 23 : 18;
          HEAP32[$op46>>2] = $4671;
         } else {
          label = 562;
         }
        } else {
         label = 562;
        }
        if ((label|0) == 562) {
         label = 0;
         HEAP32[$pTab>>2] = 0;
         HEAP32[$zDb>>2] = 0;
        }
        $4672 = HEAP32[$pOp>>2]|0;
        $4673 = ((($4672)) + 3|0);
        $4674 = HEAP8[$4673>>0]|0;
        $4675 = $4674&255;
        $4676 = $4675 & 1;
        $4677 = ($4676|0)!=(0);
        if ($4677) {
         $4678 = HEAP32[$0>>2]|0;
         $4679 = ((($4678)) + 44|0);
         $4680 = HEAP32[$4679>>2]|0;
         $4681 = (($4680) + 1)|0;
         HEAP32[$4679>>2] = $4681;
        }
        $4682 = HEAP32[$pOp>>2]|0;
        $4683 = ((($4682)) + 3|0);
        $4684 = HEAP8[$4683>>0]|0;
        $4685 = $4684&255;
        $4686 = $4685 & 2;
        $4687 = ($4686|0)!=(0);
        if ($4687) {
         $4688 = ((($x)) + 8|0);
         $4689 = $4688;
         $4690 = $4689;
         $4691 = HEAP32[$4690>>2]|0;
         $4692 = (($4689) + 4)|0;
         $4693 = $4692;
         $4694 = HEAP32[$4693>>2]|0;
         $4695 = $lastRowid;
         $4696 = $4695;
         HEAP32[$4696>>2] = $4691;
         $4697 = (($4695) + 4)|0;
         $4698 = $4697;
         HEAP32[$4698>>2] = $4694;
         $4699 = HEAP32[$db>>2]|0;
         $4700 = ((($4699)) + 32|0);
         $4701 = $4700;
         $4702 = $4701;
         HEAP32[$4702>>2] = $4691;
         $4703 = (($4701) + 4)|0;
         $4704 = $4703;
         HEAP32[$4704>>2] = $4694;
        }
        $4705 = HEAP32[$pData>>2]|0;
        $4706 = ((($4705)) + 8|0);
        $4707 = HEAP16[$4706>>1]|0;
        $4708 = $4707&65535;
        $4709 = $4708 & 1;
        $4710 = ($4709|0)!=(0);
        if ($4710) {
         $4711 = ((($x)) + 16|0);
         HEAP32[$4711>>2] = 0;
         $4712 = ((($x)) + 20|0);
         HEAP32[$4712>>2] = 0;
        } else {
         $4713 = HEAP32[$pData>>2]|0;
         $4714 = ((($4713)) + 16|0);
         $4715 = HEAP32[$4714>>2]|0;
         $4716 = ((($x)) + 16|0);
         HEAP32[$4716>>2] = $4715;
         $4717 = HEAP32[$pData>>2]|0;
         $4718 = ((($4717)) + 12|0);
         $4719 = HEAP32[$4718>>2]|0;
         $4720 = ((($x)) + 20|0);
         HEAP32[$4720>>2] = $4719;
        }
        $4721 = HEAP32[$pOp>>2]|0;
        $4722 = ((($4721)) + 3|0);
        $4723 = HEAP8[$4722>>0]|0;
        $4724 = $4723&255;
        $4725 = $4724 & 16;
        $4726 = ($4725|0)!=(0);
        if ($4726) {
         $4727 = HEAP32[$pC45>>2]|0;
         $4728 = ((($4727)) + 28|0);
         $4729 = HEAP32[$4728>>2]|0;
         $4730 = $4729;
        } else {
         $4730 = 0;
        }
        HEAP32[$seekResult>>2] = $4730;
        $4731 = HEAP32[$pData>>2]|0;
        $4732 = ((($4731)) + 8|0);
        $4733 = HEAP16[$4732>>1]|0;
        $4734 = $4733&65535;
        $4735 = $4734 & 16384;
        $4736 = ($4735|0)!=(0);
        if ($4736) {
         $4737 = HEAP32[$pData>>2]|0;
         $4738 = HEAP32[$4737>>2]|0;
         $4739 = ((($x)) + 24|0);
         HEAP32[$4739>>2] = $4738;
        } else {
         $4740 = ((($x)) + 24|0);
         HEAP32[$4740>>2] = 0;
        }
        HEAP32[$x>>2] = 0;
        $4741 = HEAP32[$pC45>>2]|0;
        $4742 = ((($4741)) + 16|0);
        $4743 = HEAP32[$4742>>2]|0;
        $4744 = HEAP32[$pOp>>2]|0;
        $4745 = ((($4744)) + 3|0);
        $4746 = HEAP8[$4745>>0]|0;
        $4747 = $4746&255;
        $4748 = $4747 & 8;
        $4749 = ($4748|0)!=(0);
        $4750 = $4749&1;
        $4751 = HEAP32[$seekResult>>2]|0;
        $4752 = (_sqlite3BtreeInsert($4743,$x,$4750,$4751)|0);
        HEAP32[$rc>>2] = $4752;
        $4753 = HEAP32[$pC45>>2]|0;
        $4754 = ((($4753)) + 3|0);
        HEAP8[$4754>>0] = 0;
        $4755 = HEAP32[$pC45>>2]|0;
        $4756 = ((($4755)) + 56|0);
        HEAP32[$4756>>2] = 0;
        $4757 = HEAP32[$rc>>2]|0;
        $4758 = ($4757|0)!=(0);
        if ($4758) {
         break L1;
        }
        $4759 = HEAP32[$db>>2]|0;
        $4760 = ((($4759)) + 220|0);
        $4761 = HEAP32[$4760>>2]|0;
        $4762 = ($4761|0)!=(0|0);
        $4763 = HEAP32[$op46>>2]|0;
        $4764 = ($4763|0)!=(0);
        $or$cond21 = $4762 & $4764;
        if ($or$cond21) {
         $4765 = HEAP32[$db>>2]|0;
         $4766 = ((($4765)) + 220|0);
         $4767 = HEAP32[$4766>>2]|0;
         $4768 = HEAP32[$db>>2]|0;
         $4769 = ((($4768)) + 216|0);
         $4770 = HEAP32[$4769>>2]|0;
         $4771 = HEAP32[$op46>>2]|0;
         $4772 = HEAP32[$zDb>>2]|0;
         $4773 = HEAP32[$pTab>>2]|0;
         $4774 = HEAP32[$4773>>2]|0;
         $4775 = ((($x)) + 8|0);
         $4776 = $4775;
         $4777 = $4776;
         $4778 = HEAP32[$4777>>2]|0;
         $4779 = (($4776) + 4)|0;
         $4780 = $4779;
         $4781 = HEAP32[$4780>>2]|0;
         FUNCTION_TABLE_viiiiii[$4767 & 255]($4770,$4771,$4772,$4774,$4778,$4781);
        }
        break;
       }
       case 117:  {
        $4782 = HEAP32[$pOp>>2]|0;
        $4783 = ((($4782)) + 8|0);
        $4784 = HEAP32[$4783>>2]|0;
        HEAP32[$opflags>>2] = $4784;
        $4785 = HEAP32[$pOp>>2]|0;
        $4786 = ((($4785)) + 4|0);
        $4787 = HEAP32[$4786>>2]|0;
        $4788 = HEAP32[$0>>2]|0;
        $4789 = ((($4788)) + 112|0);
        $4790 = HEAP32[$4789>>2]|0;
        $4791 = (($4790) + ($4787<<2)|0);
        $4792 = HEAP32[$4791>>2]|0;
        HEAP32[$pC47>>2] = $4792;
        $4793 = HEAP32[$pOp>>2]|0;
        $4794 = ((($4793)) + 1|0);
        $4795 = HEAP8[$4794>>0]|0;
        $4796 = $4795 << 24 >> 24;
        $4797 = ($4796|0)==(-20);
        if ($4797) {
         $4798 = HEAP32[$db>>2]|0;
         $4799 = ((($4798)) + 220|0);
         $4800 = HEAP32[$4799>>2]|0;
         $4801 = ($4800|0)!=(0|0);
         if ($4801) {
          $4802 = HEAP32[$pC47>>2]|0;
          $4803 = ((($4802)) + 1|0);
          $4804 = HEAP8[$4803>>0]|0;
          $4805 = $4804 << 24 >> 24;
          $4806 = HEAP32[$db>>2]|0;
          $4807 = ((($4806)) + 16|0);
          $4808 = HEAP32[$4807>>2]|0;
          $4809 = (($4808) + ($4805<<4)|0);
          $4810 = HEAP32[$4809>>2]|0;
          HEAP32[$zDb48>>2] = $4810;
          $4811 = HEAP32[$pOp>>2]|0;
          $4812 = ((($4811)) + 16|0);
          $4813 = HEAP32[$4812>>2]|0;
          HEAP32[$pTab49>>2] = $4813;
          $4814 = HEAP32[$pOp>>2]|0;
          $4815 = ((($4814)) + 3|0);
          $4816 = HEAP8[$4815>>0]|0;
          $4817 = $4816&255;
          $4818 = $4817 & 2;
          $4819 = ($4818|0)!=(0);
          if ($4819) {
           $4820 = HEAP32[$pC47>>2]|0;
           $4821 = ((($4820)) + 4|0);
           $4822 = HEAP8[$4821>>0]|0;
           $4823 = $4822&255;
           $4824 = ($4823|0)!=(0);
           if ($4824) {
            $4825 = HEAP32[$pC47>>2]|0;
            $4826 = ((($4825)) + 16|0);
            $4827 = HEAP32[$4826>>2]|0;
            $4828 = (_sqlite3BtreeIntegerKey($4827)|0);
            $4829 = tempRet0;
            $4830 = HEAP32[$pC47>>2]|0;
            $4831 = ((($4830)) + 40|0);
            $4832 = $4831;
            $4833 = $4832;
            HEAP32[$4833>>2] = $4828;
            $4834 = (($4832) + 4)|0;
            $4835 = $4834;
            HEAP32[$4835>>2] = $4829;
           }
          }
         } else {
          label = 583;
         }
        } else {
         label = 583;
        }
        if ((label|0) == 583) {
         label = 0;
         HEAP32[$zDb48>>2] = 0;
         HEAP32[$pTab49>>2] = 0;
        }
        $4836 = HEAP32[$pC47>>2]|0;
        $4837 = ((($4836)) + 16|0);
        $4838 = HEAP32[$4837>>2]|0;
        $4839 = HEAP32[$pOp>>2]|0;
        $4840 = ((($4839)) + 3|0);
        $4841 = HEAP8[$4840>>0]|0;
        $4842 = (_sqlite3BtreeDelete($4838,$4841)|0);
        HEAP32[$rc>>2] = $4842;
        $4843 = HEAP32[$pC47>>2]|0;
        $4844 = ((($4843)) + 56|0);
        HEAP32[$4844>>2] = 0;
        $4845 = HEAP32[$rc>>2]|0;
        $4846 = ($4845|0)!=(0);
        if ($4846) {
         break L1;
        }
        $4847 = HEAP32[$opflags>>2]|0;
        $4848 = $4847 & 1;
        $4849 = ($4848|0)!=(0);
        if ($4849) {
         $4850 = HEAP32[$0>>2]|0;
         $4851 = ((($4850)) + 44|0);
         $4852 = HEAP32[$4851>>2]|0;
         $4853 = (($4852) + 1)|0;
         HEAP32[$4851>>2] = $4853;
         $4854 = HEAP32[$db>>2]|0;
         $4855 = ((($4854)) + 220|0);
         $4856 = HEAP32[$4855>>2]|0;
         $4857 = ($4856|0)!=(0|0);
         if ($4857) {
          $4858 = HEAP32[$pTab49>>2]|0;
          $4859 = ((($4858)) + 42|0);
          $4860 = HEAP8[$4859>>0]|0;
          $4861 = $4860&255;
          $4862 = $4861 & 32;
          $4863 = ($4862|0)==(0);
          if ($4863) {
           $4864 = HEAP32[$db>>2]|0;
           $4865 = ((($4864)) + 220|0);
           $4866 = HEAP32[$4865>>2]|0;
           $4867 = HEAP32[$db>>2]|0;
           $4868 = ((($4867)) + 216|0);
           $4869 = HEAP32[$4868>>2]|0;
           $4870 = HEAP32[$zDb48>>2]|0;
           $4871 = HEAP32[$pTab49>>2]|0;
           $4872 = HEAP32[$4871>>2]|0;
           $4873 = HEAP32[$pC47>>2]|0;
           $4874 = ((($4873)) + 40|0);
           $4875 = $4874;
           $4876 = $4875;
           $4877 = HEAP32[$4876>>2]|0;
           $4878 = (($4875) + 4)|0;
           $4879 = $4878;
           $4880 = HEAP32[$4879>>2]|0;
           FUNCTION_TABLE_viiiiii[$4866 & 255]($4869,9,$4870,$4872,$4877,$4880);
          }
         }
        }
        break;
       }
       case 118:  {
        $4881 = HEAP32[$db>>2]|0;
        $4882 = HEAP32[$0>>2]|0;
        $4883 = ((($4882)) + 44|0);
        $4884 = HEAP32[$4883>>2]|0;
        _sqlite3VdbeSetChanges($4881,$4884);
        $4885 = HEAP32[$0>>2]|0;
        $4886 = ((($4885)) + 44|0);
        HEAP32[$4886>>2] = 0;
        break;
       }
       case 119:  {
        $4887 = HEAP32[$pOp>>2]|0;
        $4888 = ((($4887)) + 4|0);
        $4889 = HEAP32[$4888>>2]|0;
        $4890 = HEAP32[$0>>2]|0;
        $4891 = ((($4890)) + 112|0);
        $4892 = HEAP32[$4891>>2]|0;
        $4893 = (($4892) + ($4889<<2)|0);
        $4894 = HEAP32[$4893>>2]|0;
        HEAP32[$pC50>>2] = $4894;
        $4895 = HEAP32[$pOp>>2]|0;
        $4896 = ((($4895)) + 12|0);
        $4897 = HEAP32[$4896>>2]|0;
        $4898 = HEAP32[$aMem>>2]|0;
        $4899 = (($4898) + (($4897*40)|0)|0);
        HEAP32[$pIn3>>2] = $4899;
        $4900 = HEAP32[$pOp>>2]|0;
        $4901 = ((($4900)) + 16|0);
        $4902 = HEAP32[$4901>>2]|0;
        HEAP32[$nKeyCol>>2] = $4902;
        HEAP32[$res51>>2] = 0;
        $4903 = HEAP32[$pC50>>2]|0;
        $4904 = HEAP32[$pIn3>>2]|0;
        $4905 = HEAP32[$nKeyCol>>2]|0;
        $4906 = (_sqlite3VdbeSorterCompare($4903,$4904,$4905,$res51)|0);
        HEAP32[$rc>>2] = $4906;
        $4907 = HEAP32[$rc>>2]|0;
        $4908 = ($4907|0)!=(0);
        if ($4908) {
         break L1;
        }
        $4909 = HEAP32[$res51>>2]|0;
        $4910 = ($4909|0)!=(0);
        if ($4910) {
         label = 14;
        }
        break;
       }
       case 120:  {
        $4911 = HEAP32[$pOp>>2]|0;
        $4912 = ((($4911)) + 8|0);
        $4913 = HEAP32[$4912>>2]|0;
        $4914 = HEAP32[$aMem>>2]|0;
        $4915 = (($4914) + (($4913*40)|0)|0);
        HEAP32[$pOut>>2] = $4915;
        $4916 = HEAP32[$pOp>>2]|0;
        $4917 = ((($4916)) + 4|0);
        $4918 = HEAP32[$4917>>2]|0;
        $4919 = HEAP32[$0>>2]|0;
        $4920 = ((($4919)) + 112|0);
        $4921 = HEAP32[$4920>>2]|0;
        $4922 = (($4921) + ($4918<<2)|0);
        $4923 = HEAP32[$4922>>2]|0;
        HEAP32[$pC52>>2] = $4923;
        $4924 = HEAP32[$pC52>>2]|0;
        $4925 = HEAP32[$pOut>>2]|0;
        $4926 = (_sqlite3VdbeSorterRowkey($4924,$4925)|0);
        HEAP32[$rc>>2] = $4926;
        $4927 = HEAP32[$rc>>2]|0;
        $4928 = ($4927|0)!=(0);
        if ($4928) {
         break L1;
        }
        $4929 = HEAP32[$pOp>>2]|0;
        $4930 = ((($4929)) + 12|0);
        $4931 = HEAP32[$4930>>2]|0;
        $4932 = HEAP32[$0>>2]|0;
        $4933 = ((($4932)) + 112|0);
        $4934 = HEAP32[$4933>>2]|0;
        $4935 = (($4934) + ($4931<<2)|0);
        $4936 = HEAP32[$4935>>2]|0;
        $4937 = ((($4936)) + 56|0);
        HEAP32[$4937>>2] = 0;
        break;
       }
       case 122: case 121:  {
        $4938 = HEAP32[$pOp>>2]|0;
        $4939 = ((($4938)) + 8|0);
        $4940 = HEAP32[$4939>>2]|0;
        $4941 = HEAP32[$aMem>>2]|0;
        $4942 = (($4941) + (($4940*40)|0)|0);
        HEAP32[$pOut>>2] = $4942;
        $4943 = HEAP32[$pOp>>2]|0;
        $4944 = ((($4943)) + 4|0);
        $4945 = HEAP32[$4944>>2]|0;
        $4946 = HEAP32[$0>>2]|0;
        $4947 = ((($4946)) + 112|0);
        $4948 = HEAP32[$4947>>2]|0;
        $4949 = (($4948) + ($4945<<2)|0);
        $4950 = HEAP32[$4949>>2]|0;
        HEAP32[$pC53>>2] = $4950;
        $4951 = HEAP32[$pC53>>2]|0;
        $4952 = ((($4951)) + 16|0);
        $4953 = HEAP32[$4952>>2]|0;
        HEAP32[$pCrsr54>>2] = $4953;
        $4954 = HEAP32[$pCrsr54>>2]|0;
        $4955 = (_sqlite3BtreePayloadSize($4954)|0);
        HEAP32[$n55>>2] = $4955;
        $4956 = HEAP32[$n55>>2]|0;
        $4957 = HEAP32[$db>>2]|0;
        $4958 = ((($4957)) + 96|0);
        $4959 = HEAP32[$4958>>2]|0;
        $4960 = ($4956>>>0)>($4959>>>0);
        if ($4960) {
         label = 888;
         break L8;
        }
        $4961 = HEAP32[$pOut>>2]|0;
        $4962 = HEAP32[$n55>>2]|0;
        $4963 = ($4962>>>0)>(32);
        $4964 = HEAP32[$n55>>2]|0;
        $4965 = $4963 ? $4964 : 32;
        $4966 = (_sqlite3VdbeMemClearAndResize($4961,$4965)|0);
        $4967 = ($4966|0)!=(0);
        if ($4967) {
         label = 889;
         break L1;
        }
        $4968 = HEAP32[$n55>>2]|0;
        $4969 = HEAP32[$pOut>>2]|0;
        $4970 = ((($4969)) + 12|0);
        HEAP32[$4970>>2] = $4968;
        $4971 = HEAP32[$pOut>>2]|0;
        $4972 = ((($4971)) + 8|0);
        $4973 = HEAP16[$4972>>1]|0;
        $4974 = $4973&65535;
        $4975 = $4974 & -49664;
        $4976 = $4975 | 16;
        $4977 = $4976&65535;
        $4978 = HEAP32[$pOut>>2]|0;
        $4979 = ((($4978)) + 8|0);
        HEAP16[$4979>>1] = $4977;
        $4980 = HEAP32[$pC53>>2]|0;
        $4981 = ((($4980)) + 4|0);
        $4982 = HEAP8[$4981>>0]|0;
        $4983 = $4982&255;
        $4984 = ($4983|0)==(0);
        $4985 = HEAP32[$pCrsr54>>2]|0;
        $4986 = HEAP32[$n55>>2]|0;
        $4987 = HEAP32[$pOut>>2]|0;
        $4988 = ((($4987)) + 16|0);
        $4989 = HEAP32[$4988>>2]|0;
        if ($4984) {
         $4990 = (_sqlite3BtreeKey($4985,0,$4986,$4989)|0);
         HEAP32[$rc>>2] = $4990;
        } else {
         $4991 = (_sqlite3BtreeData($4985,0,$4986,$4989)|0);
         HEAP32[$rc>>2] = $4991;
        }
        $4992 = HEAP32[$rc>>2]|0;
        $4993 = ($4992|0)!=(0);
        if ($4993) {
         break L1;
        }
        $4994 = HEAP32[$pOut>>2]|0;
        $4995 = ((($4994)) + 10|0);
        HEAP8[$4995>>0] = 1;
        break;
       }
       case 123:  {
        $4996 = HEAP32[$0>>2]|0;
        $4997 = HEAP32[$pOp>>2]|0;
        $4998 = (_out2Prerelease($4996,$4997)|0);
        HEAP32[$pOut>>2] = $4998;
        $4999 = HEAP32[$pOp>>2]|0;
        $5000 = ((($4999)) + 4|0);
        $5001 = HEAP32[$5000>>2]|0;
        $5002 = HEAP32[$0>>2]|0;
        $5003 = ((($5002)) + 112|0);
        $5004 = HEAP32[$5003>>2]|0;
        $5005 = (($5004) + ($5001<<2)|0);
        $5006 = HEAP32[$5005>>2]|0;
        HEAP32[$pC56>>2] = $5006;
        $5007 = HEAP32[$pC56>>2]|0;
        $5008 = ((($5007)) + 2|0);
        $5009 = HEAP8[$5008>>0]|0;
        $5010 = ($5009<<24>>24)!=(0);
        if ($5010) {
         $5011 = HEAP32[$pOut>>2]|0;
         $5012 = ((($5011)) + 8|0);
         HEAP16[$5012>>1] = 1;
         break L10;
        }
        $5013 = HEAP32[$pC56>>2]|0;
        $5014 = ((($5013)) + 3|0);
        $5015 = HEAP8[$5014>>0]|0;
        $5016 = ($5015<<24>>24)!=(0);
        $5017 = HEAP32[$pC56>>2]|0;
        do {
         if ($5016) {
          $5018 = ((($5017)) + 40|0);
          $5019 = $5018;
          $5020 = $5019;
          $5021 = HEAP32[$5020>>2]|0;
          $5022 = (($5019) + 4)|0;
          $5023 = $5022;
          $5024 = HEAP32[$5023>>2]|0;
          $5025 = $v57;
          $5026 = $5025;
          HEAP32[$5026>>2] = $5021;
          $5027 = (($5025) + 4)|0;
          $5028 = $5027;
          HEAP32[$5028>>2] = $5024;
         } else {
          $5029 = HEAP8[$5017>>0]|0;
          $5030 = $5029&255;
          $5031 = ($5030|0)==(2);
          $5032 = HEAP32[$pC56>>2]|0;
          if ($5031) {
           $5033 = ((($5032)) + 16|0);
           $5034 = HEAP32[$5033>>2]|0;
           $5035 = HEAP32[$5034>>2]|0;
           HEAP32[$pVtab>>2] = $5035;
           $5036 = HEAP32[$pVtab>>2]|0;
           $5037 = HEAP32[$5036>>2]|0;
           HEAP32[$pModule>>2] = $5037;
           $5038 = HEAP32[$pModule>>2]|0;
           $5039 = ((($5038)) + 48|0);
           $5040 = HEAP32[$5039>>2]|0;
           $5041 = HEAP32[$pC56>>2]|0;
           $5042 = ((($5041)) + 16|0);
           $5043 = HEAP32[$5042>>2]|0;
           $5044 = (FUNCTION_TABLE_iii[$5040 & 255]($5043,$v57)|0);
           HEAP32[$rc>>2] = $5044;
           $5045 = HEAP32[$0>>2]|0;
           $5046 = HEAP32[$pVtab>>2]|0;
           _sqlite3VtabImportErrmsg($5045,$5046);
           $5047 = HEAP32[$rc>>2]|0;
           $5048 = ($5047|0)!=(0);
           if ($5048) {
            break L1;
           } else {
            break;
           }
          }
          $5049 = (_sqlite3VdbeCursorRestore($5032)|0);
          HEAP32[$rc>>2] = $5049;
          $5050 = HEAP32[$rc>>2]|0;
          $5051 = ($5050|0)!=(0);
          if ($5051) {
           break L1;
          }
          $5052 = HEAP32[$pC56>>2]|0;
          $5053 = ((($5052)) + 2|0);
          $5054 = HEAP8[$5053>>0]|0;
          $5055 = ($5054<<24>>24)!=(0);
          if ($5055) {
           $5056 = HEAP32[$pOut>>2]|0;
           $5057 = ((($5056)) + 8|0);
           HEAP16[$5057>>1] = 1;
           break L10;
          } else {
           $5058 = HEAP32[$pC56>>2]|0;
           $5059 = ((($5058)) + 16|0);
           $5060 = HEAP32[$5059>>2]|0;
           $5061 = (_sqlite3BtreeIntegerKey($5060)|0);
           $5062 = tempRet0;
           $5063 = $v57;
           $5064 = $5063;
           HEAP32[$5064>>2] = $5061;
           $5065 = (($5063) + 4)|0;
           $5066 = $5065;
           HEAP32[$5066>>2] = $5062;
           break;
          }
         }
        } while(0);
        $5067 = $v57;
        $5068 = $5067;
        $5069 = HEAP32[$5068>>2]|0;
        $5070 = (($5067) + 4)|0;
        $5071 = $5070;
        $5072 = HEAP32[$5071>>2]|0;
        $5073 = HEAP32[$pOut>>2]|0;
        $5074 = $5073;
        $5075 = $5074;
        HEAP32[$5075>>2] = $5069;
        $5076 = (($5074) + 4)|0;
        $5077 = $5076;
        HEAP32[$5077>>2] = $5072;
        break;
       }
       case 124:  {
        $5078 = HEAP32[$pOp>>2]|0;
        $5079 = ((($5078)) + 4|0);
        $5080 = HEAP32[$5079>>2]|0;
        $5081 = HEAP32[$0>>2]|0;
        $5082 = ((($5081)) + 112|0);
        $5083 = HEAP32[$5082>>2]|0;
        $5084 = (($5083) + ($5080<<2)|0);
        $5085 = HEAP32[$5084>>2]|0;
        HEAP32[$pC58>>2] = $5085;
        $5086 = HEAP32[$pC58>>2]|0;
        $5087 = ((($5086)) + 2|0);
        HEAP8[$5087>>0] = 1;
        $5088 = HEAP32[$pC58>>2]|0;
        $5089 = ((($5088)) + 56|0);
        HEAP32[$5089>>2] = 0;
        $5090 = HEAP32[$pC58>>2]|0;
        $5091 = HEAP8[$5090>>0]|0;
        $5092 = $5091&255;
        $5093 = ($5092|0)==(0);
        if ($5093) {
         $5094 = HEAP32[$pC58>>2]|0;
         $5095 = ((($5094)) + 16|0);
         $5096 = HEAP32[$5095>>2]|0;
         _sqlite3BtreeClearCursor($5096);
        }
        break;
       }
       case 53:  {
        $5097 = HEAP32[$pOp>>2]|0;
        $5098 = ((($5097)) + 4|0);
        $5099 = HEAP32[$5098>>2]|0;
        $5100 = HEAP32[$0>>2]|0;
        $5101 = ((($5100)) + 112|0);
        $5102 = HEAP32[$5101>>2]|0;
        $5103 = (($5102) + ($5099<<2)|0);
        $5104 = HEAP32[$5103>>2]|0;
        HEAP32[$pC59>>2] = $5104;
        $5105 = HEAP32[$pC59>>2]|0;
        $5106 = ((($5105)) + 16|0);
        $5107 = HEAP32[$5106>>2]|0;
        HEAP32[$pCrsr60>>2] = $5107;
        HEAP32[$res61>>2] = 0;
        $5108 = HEAP32[$pCrsr60>>2]|0;
        $5109 = (_sqlite3BtreeLast($5108,$res61)|0);
        HEAP32[$rc>>2] = $5109;
        $5110 = HEAP32[$res61>>2]|0;
        $5111 = $5110&255;
        $5112 = HEAP32[$pC59>>2]|0;
        $5113 = ((($5112)) + 2|0);
        HEAP8[$5113>>0] = $5111;
        $5114 = HEAP32[$pC59>>2]|0;
        $5115 = ((($5114)) + 3|0);
        HEAP8[$5115>>0] = 0;
        $5116 = HEAP32[$pC59>>2]|0;
        $5117 = ((($5116)) + 56|0);
        HEAP32[$5117>>2] = 0;
        $5118 = HEAP32[$pOp>>2]|0;
        $5119 = ((($5118)) + 12|0);
        $5120 = HEAP32[$5119>>2]|0;
        $5121 = HEAP32[$pC59>>2]|0;
        $5122 = ((($5121)) + 28|0);
        HEAP32[$5122>>2] = $5120;
        $5123 = HEAP32[$rc>>2]|0;
        $5124 = ($5123|0)!=(0);
        if ($5124) {
         break L1;
        }
        $5125 = HEAP32[$pOp>>2]|0;
        $5126 = ((($5125)) + 8|0);
        $5127 = HEAP32[$5126>>2]|0;
        $5128 = ($5127|0)>(0);
        $5129 = HEAP32[$res61>>2]|0;
        $5130 = ($5129|0)!=(0);
        $or$cond31 = $5128 & $5130;
        if ($or$cond31) {
         label = 14;
        }
        break;
       }
       case 56: case 55:  {
        $5131 = HEAP32[$0>>2]|0;
        $5132 = ((($5131)) + 156|0);
        $5133 = ((($5132)) + 8|0);
        $5134 = HEAP32[$5133>>2]|0;
        $5135 = (($5134) + 1)|0;
        HEAP32[$5133>>2] = $5135;
        label = 617;
        break;
       }
       case 57:  {
        label = 617;
        break;
       }
       case 3:  {
        $5166 = HEAP32[$pOp>>2]|0;
        $5167 = ((($5166)) + 4|0);
        $5168 = HEAP32[$5167>>2]|0;
        $5169 = HEAP32[$0>>2]|0;
        $5170 = ((($5169)) + 112|0);
        $5171 = HEAP32[$5170>>2]|0;
        $5172 = (($5171) + ($5168<<2)|0);
        $5173 = HEAP32[$5172>>2]|0;
        HEAP32[$pC65>>2] = $5173;
        HEAP32[$res66>>2] = 0;
        $5174 = HEAP32[$db>>2]|0;
        $5175 = HEAP32[$pC65>>2]|0;
        $5176 = (_sqlite3VdbeSorterNext($5174,$5175,$res66)|0);
        HEAP32[$rc>>2] = $5176;
        label = 625;
        break;
       }
       case 5: case 4:  {
        $5177 = HEAP32[$pOp>>2]|0;
        $5178 = ((($5177)) + 4|0);
        $5179 = HEAP32[$5178>>2]|0;
        $5180 = HEAP32[$0>>2]|0;
        $5181 = ((($5180)) + 112|0);
        $5182 = HEAP32[$5181>>2]|0;
        $5183 = (($5182) + ($5179<<2)|0);
        $5184 = HEAP32[$5183>>2]|0;
        $5185 = ($5184|0)==(0|0);
        if (!($5185)) {
         label = 624;
        }
        break;
       }
       case 7: case 6:  {
        label = 624;
        break;
       }
       case 126: case 125:  {
        $5221 = HEAP32[$pOp>>2]|0;
        $5222 = ((($5221)) + 4|0);
        $5223 = HEAP32[$5222>>2]|0;
        $5224 = HEAP32[$0>>2]|0;
        $5225 = ((($5224)) + 112|0);
        $5226 = HEAP32[$5225>>2]|0;
        $5227 = (($5226) + ($5223<<2)|0);
        $5228 = HEAP32[$5227>>2]|0;
        HEAP32[$pC67>>2] = $5228;
        $5229 = HEAP32[$pOp>>2]|0;
        $5230 = ((($5229)) + 8|0);
        $5231 = HEAP32[$5230>>2]|0;
        $5232 = HEAP32[$aMem>>2]|0;
        $5233 = (($5232) + (($5231*40)|0)|0);
        HEAP32[$pIn2>>2] = $5233;
        $5234 = HEAP32[$pOp>>2]|0;
        $5235 = ((($5234)) + 3|0);
        $5236 = HEAP8[$5235>>0]|0;
        $5237 = $5236&255;
        $5238 = $5237 & 1;
        $5239 = ($5238|0)!=(0);
        if ($5239) {
         $5240 = HEAP32[$0>>2]|0;
         $5241 = ((($5240)) + 44|0);
         $5242 = HEAP32[$5241>>2]|0;
         $5243 = (($5242) + 1)|0;
         HEAP32[$5241>>2] = $5243;
        }
        $5244 = HEAP32[$pIn2>>2]|0;
        $5245 = ((($5244)) + 8|0);
        $5246 = HEAP16[$5245>>1]|0;
        $5247 = $5246&65535;
        $5248 = $5247 & 16384;
        $5249 = ($5248|0)!=(0);
        if ($5249) {
         $5250 = HEAP32[$pIn2>>2]|0;
         $5251 = (_sqlite3VdbeMemExpandBlob($5250)|0);
         $5252 = $5251;
        } else {
         $5252 = 0;
        }
        HEAP32[$rc>>2] = $5252;
        $5253 = HEAP32[$rc>>2]|0;
        $5254 = ($5253|0)!=(0);
        if ($5254) {
         break L1;
        }
        $5255 = HEAP32[$pOp>>2]|0;
        $5256 = HEAP8[$5255>>0]|0;
        $5257 = $5256&255;
        $5258 = ($5257|0)==(125);
        if ($5258) {
         $5259 = HEAP32[$pC67>>2]|0;
         $5260 = HEAP32[$pIn2>>2]|0;
         $5261 = (_sqlite3VdbeSorterWrite($5259,$5260)|0);
         HEAP32[$rc>>2] = $5261;
        } else {
         $5262 = HEAP32[$pIn2>>2]|0;
         $5263 = ((($5262)) + 12|0);
         $5264 = HEAP32[$5263>>2]|0;
         $5265 = ($5264|0)<(0);
         $5266 = $5265 << 31 >> 31;
         $5267 = ((($x68)) + 8|0);
         $5268 = $5267;
         $5269 = $5268;
         HEAP32[$5269>>2] = $5264;
         $5270 = (($5268) + 4)|0;
         $5271 = $5270;
         HEAP32[$5271>>2] = $5266;
         $5272 = HEAP32[$pIn2>>2]|0;
         $5273 = ((($5272)) + 16|0);
         $5274 = HEAP32[$5273>>2]|0;
         HEAP32[$x68>>2] = $5274;
         $5275 = HEAP32[$pC67>>2]|0;
         $5276 = ((($5275)) + 16|0);
         $5277 = HEAP32[$5276>>2]|0;
         $5278 = HEAP32[$pOp>>2]|0;
         $5279 = ((($5278)) + 12|0);
         $5280 = HEAP32[$5279>>2]|0;
         $5281 = HEAP32[$pOp>>2]|0;
         $5282 = ((($5281)) + 3|0);
         $5283 = HEAP8[$5282>>0]|0;
         $5284 = $5283&255;
         $5285 = $5284 & 16;
         $5286 = ($5285|0)!=(0);
         if ($5286) {
          $5287 = HEAP32[$pC67>>2]|0;
          $5288 = ((($5287)) + 28|0);
          $5289 = HEAP32[$5288>>2]|0;
          $5290 = $5289;
         } else {
          $5290 = 0;
         }
         $5291 = (_sqlite3BtreeInsert($5277,$x68,$5280,$5290)|0);
         HEAP32[$rc>>2] = $5291;
         $5292 = HEAP32[$pC67>>2]|0;
         $5293 = ((($5292)) + 56|0);
         HEAP32[$5293>>2] = 0;
        }
        $5294 = HEAP32[$rc>>2]|0;
        $5295 = ($5294|0)!=(0);
        if ($5295) {
         break L1;
        }
        break;
       }
       case 127:  {
        $5296 = HEAP32[$pOp>>2]|0;
        $5297 = ((($5296)) + 4|0);
        $5298 = HEAP32[$5297>>2]|0;
        $5299 = HEAP32[$0>>2]|0;
        $5300 = ((($5299)) + 112|0);
        $5301 = HEAP32[$5300>>2]|0;
        $5302 = (($5301) + ($5298<<2)|0);
        $5303 = HEAP32[$5302>>2]|0;
        HEAP32[$pC69>>2] = $5303;
        $5304 = HEAP32[$pC69>>2]|0;
        $5305 = ((($5304)) + 16|0);
        $5306 = HEAP32[$5305>>2]|0;
        HEAP32[$pCrsr70>>2] = $5306;
        $5307 = HEAP32[$pC69>>2]|0;
        $5308 = ((($5307)) + 24|0);
        $5309 = HEAP32[$5308>>2]|0;
        HEAP32[$r72>>2] = $5309;
        $5310 = HEAP32[$pOp>>2]|0;
        $5311 = ((($5310)) + 12|0);
        $5312 = HEAP32[$5311>>2]|0;
        $5313 = $5312&65535;
        $5314 = ((($r72)) + 8|0);
        HEAP16[$5314>>1] = $5313;
        $5315 = ((($r72)) + 10|0);
        HEAP8[$5315>>0] = 0;
        $5316 = HEAP32[$pOp>>2]|0;
        $5317 = ((($5316)) + 8|0);
        $5318 = HEAP32[$5317>>2]|0;
        $5319 = HEAP32[$aMem>>2]|0;
        $5320 = (($5319) + (($5318*40)|0)|0);
        $5321 = ((($r72)) + 4|0);
        HEAP32[$5321>>2] = $5320;
        $5322 = HEAP32[$pCrsr70>>2]|0;
        $5323 = (_sqlite3BtreeMovetoUnpacked($5322,$r72,0,0,0,$res71)|0);
        HEAP32[$rc>>2] = $5323;
        $5324 = HEAP32[$rc>>2]|0;
        $5325 = ($5324|0)!=(0);
        if ($5325) {
         break L1;
        }
        $5326 = HEAP32[$res71>>2]|0;
        $5327 = ($5326|0)==(0);
        if ($5327) {
         $5328 = HEAP32[$pCrsr70>>2]|0;
         $5329 = (_sqlite3BtreeDelete($5328,4)|0);
         HEAP32[$rc>>2] = $5329;
         $5330 = HEAP32[$rc>>2]|0;
         $5331 = ($5330|0)!=(0);
         if ($5331) {
          break L1;
         }
        }
        $5332 = HEAP32[$pC69>>2]|0;
        $5333 = ((($5332)) + 56|0);
        HEAP32[$5333>>2] = 0;
        break;
       }
       case 129: case 128:  {
        $5334 = HEAP32[$pOp>>2]|0;
        $5335 = ((($5334)) + 4|0);
        $5336 = HEAP32[$5335>>2]|0;
        $5337 = HEAP32[$0>>2]|0;
        $5338 = ((($5337)) + 112|0);
        $5339 = HEAP32[$5338>>2]|0;
        $5340 = (($5339) + ($5336<<2)|0);
        $5341 = HEAP32[$5340>>2]|0;
        HEAP32[$pC73>>2] = $5341;
        $5342 = HEAP32[$pC73>>2]|0;
        $5343 = (_sqlite3VdbeCursorRestore($5342)|0);
        HEAP32[$rc>>2] = $5343;
        $5344 = HEAP32[$rc>>2]|0;
        $5345 = ($5344|0)!=(0);
        if ($5345) {
         break L1;
        }
        $5346 = HEAP32[$pC73>>2]|0;
        $5347 = ((($5346)) + 2|0);
        $5348 = HEAP8[$5347>>0]|0;
        $5349 = ($5348<<24>>24)!=(0);
        if ($5349) {
         $5413 = HEAP32[$pOp>>2]|0;
         $5414 = ((($5413)) + 8|0);
         $5415 = HEAP32[$5414>>2]|0;
         $5416 = HEAP32[$aMem>>2]|0;
         $5417 = (($5416) + (($5415*40)|0)|0);
         _sqlite3VdbeMemSetNull($5417);
         break L10;
        }
        $5350 = $rowid;
        $5351 = $5350;
        HEAP32[$5351>>2] = 0;
        $5352 = (($5350) + 4)|0;
        $5353 = $5352;
        HEAP32[$5353>>2] = 0;
        $5354 = HEAP32[$db>>2]|0;
        $5355 = HEAP32[$pC73>>2]|0;
        $5356 = ((($5355)) + 16|0);
        $5357 = HEAP32[$5356>>2]|0;
        $5358 = (_sqlite3VdbeIdxRowid($5354,$5357,$rowid)|0);
        HEAP32[$rc>>2] = $5358;
        $5359 = HEAP32[$rc>>2]|0;
        $5360 = ($5359|0)!=(0);
        if ($5360) {
         break L1;
        }
        $5361 = HEAP32[$pOp>>2]|0;
        $5362 = HEAP8[$5361>>0]|0;
        $5363 = $5362&255;
        $5364 = ($5363|0)==(128);
        if ($5364) {
         $5365 = HEAP32[$pOp>>2]|0;
         $5366 = ((($5365)) + 12|0);
         $5367 = HEAP32[$5366>>2]|0;
         $5368 = HEAP32[$0>>2]|0;
         $5369 = ((($5368)) + 112|0);
         $5370 = HEAP32[$5369>>2]|0;
         $5371 = (($5370) + ($5367<<2)|0);
         $5372 = HEAP32[$5371>>2]|0;
         HEAP32[$pTabCur>>2] = $5372;
         $5373 = HEAP32[$pTabCur>>2]|0;
         $5374 = ((($5373)) + 2|0);
         HEAP8[$5374>>0] = 0;
         $5375 = $rowid;
         $5376 = $5375;
         $5377 = HEAP32[$5376>>2]|0;
         $5378 = (($5375) + 4)|0;
         $5379 = $5378;
         $5380 = HEAP32[$5379>>2]|0;
         $5381 = HEAP32[$pTabCur>>2]|0;
         $5382 = ((($5381)) + 40|0);
         $5383 = $5382;
         $5384 = $5383;
         HEAP32[$5384>>2] = $5377;
         $5385 = (($5383) + 4)|0;
         $5386 = $5385;
         HEAP32[$5386>>2] = $5380;
         $5387 = HEAP32[$pTabCur>>2]|0;
         $5388 = ((($5387)) + 3|0);
         HEAP8[$5388>>0] = 1;
         $5389 = HEAP32[$pOp>>2]|0;
         $5390 = ((($5389)) + 16|0);
         $5391 = HEAP32[$5390>>2]|0;
         $5392 = HEAP32[$pTabCur>>2]|0;
         $5393 = ((($5392)) + 52|0);
         HEAP32[$5393>>2] = $5391;
         $5394 = HEAP32[$pC73>>2]|0;
         $5395 = HEAP32[$pTabCur>>2]|0;
         $5396 = ((($5395)) + 48|0);
         HEAP32[$5396>>2] = $5394;
         break L10;
        } else {
         $5397 = HEAP32[$0>>2]|0;
         $5398 = HEAP32[$pOp>>2]|0;
         $5399 = (_out2Prerelease($5397,$5398)|0);
         HEAP32[$pOut>>2] = $5399;
         $5400 = $rowid;
         $5401 = $5400;
         $5402 = HEAP32[$5401>>2]|0;
         $5403 = (($5400) + 4)|0;
         $5404 = $5403;
         $5405 = HEAP32[$5404>>2]|0;
         $5406 = HEAP32[$pOut>>2]|0;
         $5407 = $5406;
         $5408 = $5407;
         HEAP32[$5408>>2] = $5402;
         $5409 = (($5407) + 4)|0;
         $5410 = $5409;
         HEAP32[$5410>>2] = $5405;
         $5411 = HEAP32[$pOut>>2]|0;
         $5412 = ((($5411)) + 8|0);
         HEAP16[$5412>>1] = 4;
         break L10;
        }
        break;
       }
       case 61: case 60: case 59: case 58:  {
        $5418 = HEAP32[$pOp>>2]|0;
        $5419 = ((($5418)) + 4|0);
        $5420 = HEAP32[$5419>>2]|0;
        $5421 = HEAP32[$0>>2]|0;
        $5422 = ((($5421)) + 112|0);
        $5423 = HEAP32[$5422>>2]|0;
        $5424 = (($5423) + ($5420<<2)|0);
        $5425 = HEAP32[$5424>>2]|0;
        HEAP32[$pC74>>2] = $5425;
        $5426 = HEAP32[$pC74>>2]|0;
        $5427 = ((($5426)) + 24|0);
        $5428 = HEAP32[$5427>>2]|0;
        HEAP32[$r76>>2] = $5428;
        $5429 = HEAP32[$pOp>>2]|0;
        $5430 = ((($5429)) + 16|0);
        $5431 = HEAP32[$5430>>2]|0;
        $5432 = $5431&65535;
        $5433 = ((($r76)) + 8|0);
        HEAP16[$5433>>1] = $5432;
        $5434 = HEAP32[$pOp>>2]|0;
        $5435 = HEAP8[$5434>>0]|0;
        $5436 = $5435&255;
        $5437 = ($5436|0)<(60);
        $5438 = ((($r76)) + 10|0);
        if ($5437) {
         HEAP8[$5438>>0] = -1;
        } else {
         HEAP8[$5438>>0] = 0;
        }
        $5439 = HEAP32[$pOp>>2]|0;
        $5440 = ((($5439)) + 12|0);
        $5441 = HEAP32[$5440>>2]|0;
        $5442 = HEAP32[$aMem>>2]|0;
        $5443 = (($5442) + (($5441*40)|0)|0);
        $5444 = ((($r76)) + 4|0);
        HEAP32[$5444>>2] = $5443;
        HEAP32[$res75>>2] = 0;
        $5445 = HEAP32[$db>>2]|0;
        $5446 = HEAP32[$pC74>>2]|0;
        $5447 = (_sqlite3VdbeIdxKeyCompare($5445,$5446,$r76,$res75)|0);
        HEAP32[$rc>>2] = $5447;
        $5448 = HEAP32[$pOp>>2]|0;
        $5449 = HEAP8[$5448>>0]|0;
        $5450 = $5449&255;
        $5451 = $5450 & 1;
        $5452 = ($5451|0)==(0);
        $5453 = HEAP32[$res75>>2]|0;
        if ($5452) {
         $5454 = (0 - ($5453))|0;
         HEAP32[$res75>>2] = $5454;
        } else {
         $5455 = (($5453) + 1)|0;
         HEAP32[$res75>>2] = $5455;
        }
        $5456 = HEAP32[$rc>>2]|0;
        $5457 = ($5456|0)!=(0);
        if ($5457) {
         break L1;
        }
        $5458 = HEAP32[$res75>>2]|0;
        $5459 = ($5458|0)>(0);
        if ($5459) {
         label = 14;
        }
        break;
       }
       case 130:  {
        $5460 = HEAP32[$0>>2]|0;
        $5461 = HEAP32[$pOp>>2]|0;
        $5462 = (_out2Prerelease($5460,$5461)|0);
        HEAP32[$pOut>>2] = $5462;
        $5463 = HEAP32[$pOut>>2]|0;
        $5464 = ((($5463)) + 8|0);
        HEAP16[$5464>>1] = 1;
        $5465 = HEAP32[$db>>2]|0;
        $5466 = ((($5465)) + 160|0);
        $5467 = HEAP32[$5466>>2]|0;
        $5468 = HEAP32[$db>>2]|0;
        $5469 = ((($5468)) + 172|0);
        $5470 = HEAP32[$5469>>2]|0;
        $5471 = (($5470) + 1)|0;
        $5472 = ($5467|0)>($5471|0);
        if ($5472) {
         label = 660;
         break L8;
        }
        $5475 = HEAP32[$pOp>>2]|0;
        $5476 = ((($5475)) + 12|0);
        $5477 = HEAP32[$5476>>2]|0;
        HEAP32[$iDb77>>2] = $5477;
        HEAP32[$iMoved>>2] = 0;
        $5478 = HEAP32[$iDb77>>2]|0;
        $5479 = HEAP32[$db>>2]|0;
        $5480 = ((($5479)) + 16|0);
        $5481 = HEAP32[$5480>>2]|0;
        $5482 = (($5481) + ($5478<<4)|0);
        $5483 = ((($5482)) + 4|0);
        $5484 = HEAP32[$5483>>2]|0;
        $5485 = HEAP32[$pOp>>2]|0;
        $5486 = ((($5485)) + 4|0);
        $5487 = HEAP32[$5486>>2]|0;
        $5488 = (_sqlite3BtreeDropTable($5484,$5487,$iMoved)|0);
        HEAP32[$rc>>2] = $5488;
        $5489 = HEAP32[$pOut>>2]|0;
        $5490 = ((($5489)) + 8|0);
        HEAP16[$5490>>1] = 4;
        $5491 = HEAP32[$iMoved>>2]|0;
        $5492 = ($5491|0)<(0);
        $5493 = $5492 << 31 >> 31;
        $5494 = HEAP32[$pOut>>2]|0;
        $5495 = $5494;
        $5496 = $5495;
        HEAP32[$5496>>2] = $5491;
        $5497 = (($5495) + 4)|0;
        $5498 = $5497;
        HEAP32[$5498>>2] = $5493;
        $5499 = HEAP32[$rc>>2]|0;
        $5500 = ($5499|0)!=(0);
        if ($5500) {
         break L1;
        }
        $5501 = HEAP32[$iMoved>>2]|0;
        $5502 = ($5501|0)!=(0);
        if ($5502) {
         $5503 = HEAP32[$db>>2]|0;
         $5504 = HEAP32[$iDb77>>2]|0;
         $5505 = HEAP32[$iMoved>>2]|0;
         $5506 = HEAP32[$pOp>>2]|0;
         $5507 = ((($5506)) + 4|0);
         $5508 = HEAP32[$5507>>2]|0;
         _sqlite3RootPageMoved($5503,$5504,$5505,$5508);
         $5509 = HEAP32[$iDb77>>2]|0;
         $5510 = (($5509) + 1)|0;
         $5511 = $5510&255;
         HEAP8[$resetSchemaOnFault>>0] = $5511;
        }
        break;
       }
       case 131:  {
        HEAP32[$nChange>>2] = 0;
        $5512 = HEAP32[$pOp>>2]|0;
        $5513 = ((($5512)) + 8|0);
        $5514 = HEAP32[$5513>>2]|0;
        $5515 = HEAP32[$db>>2]|0;
        $5516 = ((($5515)) + 16|0);
        $5517 = HEAP32[$5516>>2]|0;
        $5518 = (($5517) + ($5514<<4)|0);
        $5519 = ((($5518)) + 4|0);
        $5520 = HEAP32[$5519>>2]|0;
        $5521 = HEAP32[$pOp>>2]|0;
        $5522 = ((($5521)) + 4|0);
        $5523 = HEAP32[$5522>>2]|0;
        $5524 = HEAP32[$pOp>>2]|0;
        $5525 = ((($5524)) + 12|0);
        $5526 = HEAP32[$5525>>2]|0;
        $5527 = ($5526|0)!=(0);
        $nChange$ = $5527 ? $nChange : 0;
        $5528 = (_sqlite3BtreeClearTable($5520,$5523,$nChange$)|0);
        HEAP32[$rc>>2] = $5528;
        $5529 = HEAP32[$pOp>>2]|0;
        $5530 = ((($5529)) + 12|0);
        $5531 = HEAP32[$5530>>2]|0;
        $5532 = ($5531|0)!=(0);
        if ($5532) {
         $5533 = HEAP32[$nChange>>2]|0;
         $5534 = HEAP32[$0>>2]|0;
         $5535 = ((($5534)) + 44|0);
         $5536 = HEAP32[$5535>>2]|0;
         $5537 = (($5536) + ($5533))|0;
         HEAP32[$5535>>2] = $5537;
         $5538 = HEAP32[$pOp>>2]|0;
         $5539 = ((($5538)) + 12|0);
         $5540 = HEAP32[$5539>>2]|0;
         $5541 = ($5540|0)>(0);
         if ($5541) {
          $5542 = HEAP32[$nChange>>2]|0;
          $5543 = ($5542|0)<(0);
          $5544 = $5543 << 31 >> 31;
          $5545 = HEAP32[$pOp>>2]|0;
          $5546 = ((($5545)) + 12|0);
          $5547 = HEAP32[$5546>>2]|0;
          $5548 = HEAP32[$aMem>>2]|0;
          $5549 = (($5548) + (($5547*40)|0)|0);
          $5550 = $5549;
          $5551 = $5550;
          $5552 = HEAP32[$5551>>2]|0;
          $5553 = (($5550) + 4)|0;
          $5554 = $5553;
          $5555 = HEAP32[$5554>>2]|0;
          $5556 = (_i64Add(($5552|0),($5555|0),($5542|0),($5544|0))|0);
          $5557 = tempRet0;
          $5558 = $5549;
          $5559 = $5558;
          HEAP32[$5559>>2] = $5556;
          $5560 = (($5558) + 4)|0;
          $5561 = $5560;
          HEAP32[$5561>>2] = $5557;
         }
        }
        $5562 = HEAP32[$rc>>2]|0;
        $5563 = ($5562|0)!=(0);
        if ($5563) {
         break L1;
        }
        break;
       }
       case 133:  {
        $5564 = HEAP32[$pOp>>2]|0;
        $5565 = ((($5564)) + 4|0);
        $5566 = HEAP32[$5565>>2]|0;
        $5567 = HEAP32[$0>>2]|0;
        $5568 = ((($5567)) + 112|0);
        $5569 = HEAP32[$5568>>2]|0;
        $5570 = (($5569) + ($5566<<2)|0);
        $5571 = HEAP32[$5570>>2]|0;
        HEAP32[$pC78>>2] = $5571;
        $5572 = HEAP32[$pC78>>2]|0;
        $5573 = HEAP8[$5572>>0]|0;
        $5574 = $5573&255;
        $5575 = ($5574|0)==(1);
        if ($5575) {
         $5576 = HEAP32[$db>>2]|0;
         $5577 = HEAP32[$pC78>>2]|0;
         $5578 = ((($5577)) + 16|0);
         $5579 = HEAP32[$5578>>2]|0;
         _sqlite3VdbeSorterReset($5576,$5579);
         break L10;
        } else {
         $5580 = HEAP32[$pC78>>2]|0;
         $5581 = ((($5580)) + 16|0);
         $5582 = HEAP32[$5581>>2]|0;
         $5583 = (_sqlite3BtreeClearTableOfCursor($5582)|0);
         HEAP32[$rc>>2] = $5583;
         $5584 = HEAP32[$rc>>2]|0;
         $5585 = ($5584|0)!=(0);
         if ($5585) {
          break L1;
         } else {
          break L10;
         }
        }
        break;
       }
       case 135: case 134:  {
        $5586 = HEAP32[$0>>2]|0;
        $5587 = HEAP32[$pOp>>2]|0;
        $5588 = (_out2Prerelease($5586,$5587)|0);
        HEAP32[$pOut>>2] = $5588;
        HEAP32[$pgno79>>2] = 0;
        $5589 = HEAP32[$pOp>>2]|0;
        $5590 = ((($5589)) + 4|0);
        $5591 = HEAP32[$5590>>2]|0;
        $5592 = HEAP32[$db>>2]|0;
        $5593 = ((($5592)) + 16|0);
        $5594 = HEAP32[$5593>>2]|0;
        $5595 = (($5594) + ($5591<<4)|0);
        HEAP32[$pDb81>>2] = $5595;
        $5596 = HEAP32[$pOp>>2]|0;
        $5597 = HEAP8[$5596>>0]|0;
        $5598 = $5597&255;
        $5599 = ($5598|0)==(135);
        if ($5599) {
         HEAP32[$flags80>>2] = 1;
        } else {
         HEAP32[$flags80>>2] = 2;
        }
        $5600 = HEAP32[$pDb81>>2]|0;
        $5601 = ((($5600)) + 4|0);
        $5602 = HEAP32[$5601>>2]|0;
        $5603 = HEAP32[$flags80>>2]|0;
        $5604 = (_sqlite3BtreeCreateTable($5602,$pgno79,$5603)|0);
        HEAP32[$rc>>2] = $5604;
        $5605 = HEAP32[$rc>>2]|0;
        $5606 = ($5605|0)!=(0);
        if ($5606) {
         break L1;
        }
        $5607 = HEAP32[$pgno79>>2]|0;
        $5608 = ($5607|0)<(0);
        $5609 = $5608 << 31 >> 31;
        $5610 = HEAP32[$pOut>>2]|0;
        $5611 = $5610;
        $5612 = $5611;
        HEAP32[$5612>>2] = $5607;
        $5613 = (($5611) + 4)|0;
        $5614 = $5613;
        HEAP32[$5614>>2] = $5609;
        break;
       }
       case 136:  {
        $5615 = HEAP32[$pOp>>2]|0;
        $5616 = ((($5615)) + 4|0);
        $5617 = HEAP32[$5616>>2]|0;
        HEAP32[$iDb82>>2] = $5617;
        $5618 = HEAP32[$iDb82>>2]|0;
        $5619 = ($5618|0)==(1);
        $5620 = $5619 ? 19905 : 19924;
        HEAP32[$zMaster>>2] = $5620;
        $5621 = HEAP32[$db>>2]|0;
        HEAP32[$initData>>2] = $5621;
        $5622 = HEAP32[$pOp>>2]|0;
        $5623 = ((($5622)) + 4|0);
        $5624 = HEAP32[$5623>>2]|0;
        $5625 = ((($initData)) + 8|0);
        HEAP32[$5625>>2] = $5624;
        $5626 = HEAP32[$0>>2]|0;
        $5627 = ((($5626)) + 108|0);
        $5628 = ((($initData)) + 4|0);
        HEAP32[$5628>>2] = $5627;
        $5629 = HEAP32[$db>>2]|0;
        $5630 = HEAP32[$iDb82>>2]|0;
        $5631 = HEAP32[$db>>2]|0;
        $5632 = ((($5631)) + 16|0);
        $5633 = HEAP32[$5632>>2]|0;
        $5634 = (($5633) + ($5630<<4)|0);
        $5635 = HEAP32[$5634>>2]|0;
        $5636 = HEAP32[$zMaster>>2]|0;
        $5637 = HEAP32[$pOp>>2]|0;
        $5638 = ((($5637)) + 16|0);
        $5639 = HEAP32[$5638>>2]|0;
        HEAP32[$vararg_buffer58>>2] = $5635;
        $vararg_ptr61 = ((($vararg_buffer58)) + 4|0);
        HEAP32[$vararg_ptr61>>2] = $5636;
        $vararg_ptr62 = ((($vararg_buffer58)) + 8|0);
        HEAP32[$vararg_ptr62>>2] = $5639;
        $5640 = (_sqlite3MPrintf($5629,40293,$vararg_buffer58)|0);
        HEAP32[$zSql>>2] = $5640;
        $5641 = HEAP32[$zSql>>2]|0;
        $5642 = ($5641|0)==(0|0);
        if ($5642) {
         HEAP32[$rc>>2] = 7;
        } else {
         $5643 = HEAP32[$db>>2]|0;
         $5644 = ((($5643)) + 148|0);
         $5645 = ((($5644)) + 5|0);
         HEAP8[$5645>>0] = 1;
         $5646 = ((($initData)) + 12|0);
         HEAP32[$5646>>2] = 0;
         $5647 = HEAP32[$db>>2]|0;
         $5648 = HEAP32[$zSql>>2]|0;
         $5649 = (_sqlite3_exec($5647,$5648,140,$initData,0)|0);
         HEAP32[$rc>>2] = $5649;
         $5650 = HEAP32[$rc>>2]|0;
         $5651 = ($5650|0)==(0);
         if ($5651) {
          $5652 = ((($initData)) + 12|0);
          $5653 = HEAP32[$5652>>2]|0;
          HEAP32[$rc>>2] = $5653;
         }
         $5654 = HEAP32[$db>>2]|0;
         $5655 = HEAP32[$zSql>>2]|0;
         _sqlite3DbFree($5654,$5655);
         $5656 = HEAP32[$db>>2]|0;
         $5657 = ((($5656)) + 148|0);
         $5658 = ((($5657)) + 5|0);
         HEAP8[$5658>>0] = 0;
        }
        $5659 = HEAP32[$rc>>2]|0;
        $5660 = ($5659|0)!=(0);
        if ($5660) {
         label = 682;
         break L8;
        }
        break;
       }
       case 137:  {
        $5664 = HEAP32[$db>>2]|0;
        $5665 = HEAP32[$pOp>>2]|0;
        $5666 = ((($5665)) + 4|0);
        $5667 = HEAP32[$5666>>2]|0;
        $5668 = (_sqlite3AnalysisLoad($5664,$5667)|0);
        HEAP32[$rc>>2] = $5668;
        $5669 = HEAP32[$rc>>2]|0;
        $5670 = ($5669|0)!=(0);
        if ($5670) {
         break L1;
        }
        break;
       }
       case 138:  {
        $5671 = HEAP32[$db>>2]|0;
        $5672 = HEAP32[$pOp>>2]|0;
        $5673 = ((($5672)) + 4|0);
        $5674 = HEAP32[$5673>>2]|0;
        $5675 = HEAP32[$pOp>>2]|0;
        $5676 = ((($5675)) + 16|0);
        $5677 = HEAP32[$5676>>2]|0;
        _sqlite3UnlinkAndDeleteTable($5671,$5674,$5677);
        break;
       }
       case 139:  {
        $5678 = HEAP32[$db>>2]|0;
        $5679 = HEAP32[$pOp>>2]|0;
        $5680 = ((($5679)) + 4|0);
        $5681 = HEAP32[$5680>>2]|0;
        $5682 = HEAP32[$pOp>>2]|0;
        $5683 = ((($5682)) + 16|0);
        $5684 = HEAP32[$5683>>2]|0;
        _sqlite3UnlinkAndDeleteIndex($5678,$5681,$5684);
        break;
       }
       case 140:  {
        $5685 = HEAP32[$db>>2]|0;
        $5686 = HEAP32[$pOp>>2]|0;
        $5687 = ((($5686)) + 4|0);
        $5688 = HEAP32[$5687>>2]|0;
        $5689 = HEAP32[$pOp>>2]|0;
        $5690 = ((($5689)) + 16|0);
        $5691 = HEAP32[$5690>>2]|0;
        _sqlite3UnlinkAndDeleteTrigger($5685,$5688,$5691);
        break;
       }
       case 141:  {
        $5692 = HEAP32[$pOp>>2]|0;
        $5693 = ((($5692)) + 8|0);
        $5694 = HEAP32[$5693>>2]|0;
        HEAP32[$nRoot>>2] = $5694;
        $5695 = HEAP32[$pOp>>2]|0;
        $5696 = ((($5695)) + 16|0);
        $5697 = HEAP32[$5696>>2]|0;
        HEAP32[$aRoot>>2] = $5697;
        $5698 = HEAP32[$pOp>>2]|0;
        $5699 = ((($5698)) + 12|0);
        $5700 = HEAP32[$5699>>2]|0;
        $5701 = HEAP32[$aMem>>2]|0;
        $5702 = (($5701) + (($5700*40)|0)|0);
        HEAP32[$pnErr>>2] = $5702;
        $5703 = HEAP32[$pOp>>2]|0;
        $5704 = ((($5703)) + 4|0);
        $5705 = HEAP32[$5704>>2]|0;
        $5706 = HEAP32[$aMem>>2]|0;
        $5707 = (($5706) + (($5705*40)|0)|0);
        HEAP32[$pIn1>>2] = $5707;
        $5708 = HEAP32[$pOp>>2]|0;
        $5709 = ((($5708)) + 3|0);
        $5710 = HEAP8[$5709>>0]|0;
        $5711 = $5710&255;
        $5712 = HEAP32[$db>>2]|0;
        $5713 = ((($5712)) + 16|0);
        $5714 = HEAP32[$5713>>2]|0;
        $5715 = (($5714) + ($5711<<4)|0);
        $5716 = ((($5715)) + 4|0);
        $5717 = HEAP32[$5716>>2]|0;
        $5718 = HEAP32[$aRoot>>2]|0;
        $5719 = HEAP32[$nRoot>>2]|0;
        $5720 = HEAP32[$pnErr>>2]|0;
        $5721 = $5720;
        $5722 = $5721;
        $5723 = HEAP32[$5722>>2]|0;
        $5724 = (($5721) + 4)|0;
        $5725 = $5724;
        $5726 = HEAP32[$5725>>2]|0;
        $5727 = (_sqlite3BtreeIntegrityCheck($5717,$5718,$5719,$5723,$nErr)|0);
        HEAP32[$z>>2] = $5727;
        $5728 = HEAP32[$nErr>>2]|0;
        $5729 = ($5728|0)<(0);
        $5730 = $5729 << 31 >> 31;
        $5731 = HEAP32[$pnErr>>2]|0;
        $5732 = $5731;
        $5733 = $5732;
        $5734 = HEAP32[$5733>>2]|0;
        $5735 = (($5732) + 4)|0;
        $5736 = $5735;
        $5737 = HEAP32[$5736>>2]|0;
        $5738 = (_i64Subtract(($5734|0),($5737|0),($5728|0),($5730|0))|0);
        $5739 = tempRet0;
        $5740 = $5731;
        $5741 = $5740;
        HEAP32[$5741>>2] = $5738;
        $5742 = (($5740) + 4)|0;
        $5743 = $5742;
        HEAP32[$5743>>2] = $5739;
        $5744 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemSetNull($5744);
        $5745 = HEAP32[$nErr>>2]|0;
        $5746 = ($5745|0)==(0);
        if (!($5746)) {
         $5747 = HEAP32[$z>>2]|0;
         $5748 = ($5747|0)==(0|0);
         if ($5748) {
          label = 889;
          break L1;
         }
         $5749 = HEAP32[$pIn1>>2]|0;
         $5750 = HEAP32[$z>>2]|0;
         (_sqlite3VdbeMemSetStr($5749,$5750,-1,1,148)|0);
        }
        $5751 = HEAP32[$pIn1>>2]|0;
        $5752 = HEAP8[$encoding>>0]|0;
        $5753 = $5752&255;
        (_sqlite3VdbeChangeEncoding($5751,$5753)|0);
        break;
       }
       case 142:  {
        $5754 = HEAP32[$pOp>>2]|0;
        $5755 = ((($5754)) + 4|0);
        $5756 = HEAP32[$5755>>2]|0;
        $5757 = HEAP32[$aMem>>2]|0;
        $5758 = (($5757) + (($5756*40)|0)|0);
        HEAP32[$pIn1>>2] = $5758;
        $5759 = HEAP32[$pOp>>2]|0;
        $5760 = ((($5759)) + 8|0);
        $5761 = HEAP32[$5760>>2]|0;
        $5762 = HEAP32[$aMem>>2]|0;
        $5763 = (($5762) + (($5761*40)|0)|0);
        HEAP32[$pIn2>>2] = $5763;
        $5764 = HEAP32[$pIn1>>2]|0;
        $5765 = ((($5764)) + 8|0);
        $5766 = HEAP16[$5765>>1]|0;
        $5767 = $5766&65535;
        $5768 = $5767 & 32;
        $5769 = ($5768|0)==(0);
        if ($5769) {
         $5770 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemSetRowSet($5770);
         $5771 = HEAP32[$pIn1>>2]|0;
         $5772 = ((($5771)) + 8|0);
         $5773 = HEAP16[$5772>>1]|0;
         $5774 = $5773&65535;
         $5775 = $5774 & 32;
         $5776 = ($5775|0)==(0);
         if ($5776) {
          label = 889;
          break L1;
         }
        }
        $5777 = HEAP32[$pIn1>>2]|0;
        $5778 = HEAP32[$5777>>2]|0;
        $5779 = HEAP32[$pIn2>>2]|0;
        $5780 = $5779;
        $5781 = $5780;
        $5782 = HEAP32[$5781>>2]|0;
        $5783 = (($5780) + 4)|0;
        $5784 = $5783;
        $5785 = HEAP32[$5784>>2]|0;
        _sqlite3RowSetInsert($5778,$5782,$5785);
        break;
       }
       case 62:  {
        $5786 = HEAP32[$pOp>>2]|0;
        $5787 = ((($5786)) + 4|0);
        $5788 = HEAP32[$5787>>2]|0;
        $5789 = HEAP32[$aMem>>2]|0;
        $5790 = (($5789) + (($5788*40)|0)|0);
        HEAP32[$pIn1>>2] = $5790;
        $5791 = HEAP32[$pIn1>>2]|0;
        $5792 = ((($5791)) + 8|0);
        $5793 = HEAP16[$5792>>1]|0;
        $5794 = $5793&65535;
        $5795 = $5794 & 32;
        $5796 = ($5795|0)==(0);
        if (!($5796)) {
         $5797 = HEAP32[$pIn1>>2]|0;
         $5798 = HEAP32[$5797>>2]|0;
         $5799 = (_sqlite3RowSetNext($5798,$val)|0);
         $5800 = ($5799|0)==(0);
         if (!($5800)) {
          $5802 = HEAP32[$pOp>>2]|0;
          $5803 = ((($5802)) + 12|0);
          $5804 = HEAP32[$5803>>2]|0;
          $5805 = HEAP32[$aMem>>2]|0;
          $5806 = (($5805) + (($5804*40)|0)|0);
          $5807 = $val;
          $5808 = $5807;
          $5809 = HEAP32[$5808>>2]|0;
          $5810 = (($5807) + 4)|0;
          $5811 = $5810;
          $5812 = HEAP32[$5811>>2]|0;
          _sqlite3VdbeMemSetInt64($5806,$5809,$5812);
          label = 8;
          break L10;
         }
        }
        $5801 = HEAP32[$pIn1>>2]|0;
        _sqlite3VdbeMemSetNull($5801);
        label = 7;
        break;
       }
       case 63:  {
        $5813 = HEAP32[$pOp>>2]|0;
        $5814 = ((($5813)) + 4|0);
        $5815 = HEAP32[$5814>>2]|0;
        $5816 = HEAP32[$aMem>>2]|0;
        $5817 = (($5816) + (($5815*40)|0)|0);
        HEAP32[$pIn1>>2] = $5817;
        $5818 = HEAP32[$pOp>>2]|0;
        $5819 = ((($5818)) + 12|0);
        $5820 = HEAP32[$5819>>2]|0;
        $5821 = HEAP32[$aMem>>2]|0;
        $5822 = (($5821) + (($5820*40)|0)|0);
        HEAP32[$pIn3>>2] = $5822;
        $5823 = HEAP32[$pOp>>2]|0;
        $5824 = ((($5823)) + 16|0);
        $5825 = HEAP32[$5824>>2]|0;
        HEAP32[$iSet>>2] = $5825;
        $5826 = HEAP32[$pIn1>>2]|0;
        $5827 = ((($5826)) + 8|0);
        $5828 = HEAP16[$5827>>1]|0;
        $5829 = $5828&65535;
        $5830 = $5829 & 32;
        $5831 = ($5830|0)==(0);
        if ($5831) {
         $5832 = HEAP32[$pIn1>>2]|0;
         _sqlite3VdbeMemSetRowSet($5832);
         $5833 = HEAP32[$pIn1>>2]|0;
         $5834 = ((($5833)) + 8|0);
         $5835 = HEAP16[$5834>>1]|0;
         $5836 = $5835&65535;
         $5837 = $5836 & 32;
         $5838 = ($5837|0)==(0);
         if ($5838) {
          label = 889;
          break L1;
         }
        }
        $5839 = HEAP32[$iSet>>2]|0;
        $5840 = ($5839|0)!=(0);
        if ($5840) {
         $5841 = HEAP32[$pIn1>>2]|0;
         $5842 = HEAP32[$5841>>2]|0;
         $5843 = HEAP32[$iSet>>2]|0;
         $5844 = HEAP32[$pIn3>>2]|0;
         $5845 = $5844;
         $5846 = $5845;
         $5847 = HEAP32[$5846>>2]|0;
         $5848 = (($5845) + 4)|0;
         $5849 = $5848;
         $5850 = HEAP32[$5849>>2]|0;
         $5851 = (_sqlite3RowSetTest($5842,$5843,$5847,$5850)|0);
         HEAP32[$exists>>2] = $5851;
         $5852 = HEAP32[$exists>>2]|0;
         $5853 = ($5852|0)!=(0);
         if ($5853) {
          label = 14;
          break L10;
         }
        }
        $5854 = HEAP32[$iSet>>2]|0;
        $5855 = ($5854|0)>=(0);
        if ($5855) {
         $5856 = HEAP32[$pIn1>>2]|0;
         $5857 = HEAP32[$5856>>2]|0;
         $5858 = HEAP32[$pIn3>>2]|0;
         $5859 = $5858;
         $5860 = $5859;
         $5861 = HEAP32[$5860>>2]|0;
         $5862 = (($5859) + 4)|0;
         $5863 = $5862;
         $5864 = HEAP32[$5863>>2]|0;
         _sqlite3RowSetInsert($5857,$5861,$5864);
        }
        break;
       }
       case 64:  {
        $5865 = HEAP32[$pOp>>2]|0;
        $5866 = ((($5865)) + 16|0);
        $5867 = HEAP32[$5866>>2]|0;
        HEAP32[$pProgram>>2] = $5867;
        $5868 = HEAP32[$pOp>>2]|0;
        $5869 = ((($5868)) + 12|0);
        $5870 = HEAP32[$5869>>2]|0;
        $5871 = HEAP32[$aMem>>2]|0;
        $5872 = (($5871) + (($5870*40)|0)|0);
        HEAP32[$pRt>>2] = $5872;
        $5873 = HEAP32[$pOp>>2]|0;
        $5874 = ((($5873)) + 3|0);
        $5875 = HEAP8[$5874>>0]|0;
        $5876 = ($5875<<24>>24)!=(0);
        if ($5876) {
         $5877 = HEAP32[$pProgram>>2]|0;
         $5878 = ((($5877)) + 16|0);
         $5879 = HEAP32[$5878>>2]|0;
         HEAP32[$t86>>2] = $5879;
         $5880 = HEAP32[$0>>2]|0;
         $5881 = ((($5880)) + 184|0);
         $5882 = HEAP32[$5881>>2]|0;
         HEAP32[$pFrame85>>2] = $5882;
         while(1) {
          $5883 = HEAP32[$pFrame85>>2]|0;
          $5884 = ($5883|0)!=(0|0);
          if (!($5884)) {
           break;
          }
          $5885 = HEAP32[$pFrame85>>2]|0;
          $5886 = ((($5885)) + 24|0);
          $5887 = HEAP32[$5886>>2]|0;
          $5888 = HEAP32[$t86>>2]|0;
          $5889 = ($5887|0)!=($5888|0);
          if (!($5889)) {
           break;
          }
          $5890 = HEAP32[$pFrame85>>2]|0;
          $5891 = ((($5890)) + 4|0);
          $5892 = HEAP32[$5891>>2]|0;
          HEAP32[$pFrame85>>2] = $5892;
         }
         $5893 = HEAP32[$pFrame85>>2]|0;
         $5894 = ($5893|0)!=(0|0);
         if ($5894) {
          break L10;
         }
        }
        $5895 = HEAP32[$0>>2]|0;
        $5896 = ((($5895)) + 192|0);
        $5897 = HEAP32[$5896>>2]|0;
        $5898 = HEAP32[$db>>2]|0;
        $5899 = ((($5898)) + 96|0);
        $5900 = ((($5899)) + 40|0);
        $5901 = HEAP32[$5900>>2]|0;
        $5902 = ($5897|0)>=($5901|0);
        if ($5902) {
         label = 711;
         break L8;
        }
        $5904 = HEAP32[$pRt>>2]|0;
        $5905 = ((($5904)) + 8|0);
        $5906 = HEAP16[$5905>>1]|0;
        $5907 = $5906&65535;
        $5908 = $5907 & 64;
        $5909 = ($5908|0)==(0);
        L784: do {
         if ($5909) {
          $5910 = HEAP32[$pProgram>>2]|0;
          $5911 = ((($5910)) + 8|0);
          $5912 = HEAP32[$5911>>2]|0;
          $5913 = HEAP32[$pProgram>>2]|0;
          $5914 = ((($5913)) + 12|0);
          $5915 = HEAP32[$5914>>2]|0;
          $5916 = (($5912) + ($5915))|0;
          HEAP32[$nMem>>2] = $5916;
          $5917 = HEAP32[$pProgram>>2]|0;
          $5918 = ((($5917)) + 12|0);
          $5919 = HEAP32[$5918>>2]|0;
          $5920 = ($5919|0)==(0);
          if ($5920) {
           $5921 = HEAP32[$nMem>>2]|0;
           $5922 = (($5921) + 1)|0;
           HEAP32[$nMem>>2] = $5922;
          }
          $5923 = HEAP32[$nMem>>2]|0;
          $5924 = ($5923*40)|0;
          $5925 = (80 + ($5924))|0;
          $5926 = HEAP32[$pProgram>>2]|0;
          $5927 = ((($5926)) + 12|0);
          $5928 = HEAP32[$5927>>2]|0;
          $5929 = $5928<<2;
          $5930 = (($5925) + ($5929))|0;
          HEAP32[$nByte83>>2] = $5930;
          $5931 = HEAP32[$db>>2]|0;
          $5932 = HEAP32[$nByte83>>2]|0;
          $5933 = ($5932|0)<(0);
          $5934 = $5933 << 31 >> 31;
          $5935 = (_sqlite3DbMallocZero($5931,$5932,$5934)|0);
          HEAP32[$pFrame85>>2] = $5935;
          $5936 = HEAP32[$pFrame85>>2]|0;
          $5937 = ($5936|0)!=(0|0);
          if (!($5937)) {
           label = 889;
           break L1;
          }
          $5938 = HEAP32[$pRt>>2]|0;
          _sqlite3VdbeMemRelease($5938);
          $5939 = HEAP32[$pRt>>2]|0;
          $5940 = ((($5939)) + 8|0);
          HEAP16[$5940>>1] = 64;
          $5941 = HEAP32[$pFrame85>>2]|0;
          $5942 = HEAP32[$pRt>>2]|0;
          HEAP32[$5942>>2] = $5941;
          $5943 = HEAP32[$0>>2]|0;
          $5944 = HEAP32[$pFrame85>>2]|0;
          HEAP32[$5944>>2] = $5943;
          $5945 = HEAP32[$nMem>>2]|0;
          $5946 = HEAP32[$pFrame85>>2]|0;
          $5947 = ((($5946)) + 60|0);
          HEAP32[$5947>>2] = $5945;
          $5948 = HEAP32[$pProgram>>2]|0;
          $5949 = ((($5948)) + 12|0);
          $5950 = HEAP32[$5949>>2]|0;
          $5951 = HEAP32[$pFrame85>>2]|0;
          $5952 = ((($5951)) + 64|0);
          HEAP32[$5952>>2] = $5950;
          $5953 = HEAP32[$pOp>>2]|0;
          $5954 = HEAP32[$aOp>>2]|0;
          $5955 = $5953;
          $5956 = $5954;
          $5957 = (($5955) - ($5956))|0;
          $5958 = (($5957|0) / 20)&-1;
          $5959 = HEAP32[$pFrame85>>2]|0;
          $5960 = ((($5959)) + 48|0);
          HEAP32[$5960>>2] = $5958;
          $5961 = HEAP32[$0>>2]|0;
          $5962 = ((($5961)) + 92|0);
          $5963 = HEAP32[$5962>>2]|0;
          $5964 = HEAP32[$pFrame85>>2]|0;
          $5965 = ((($5964)) + 16|0);
          HEAP32[$5965>>2] = $5963;
          $5966 = HEAP32[$0>>2]|0;
          $5967 = ((($5966)) + 24|0);
          $5968 = HEAP32[$5967>>2]|0;
          $5969 = HEAP32[$pFrame85>>2]|0;
          $5970 = ((($5969)) + 56|0);
          HEAP32[$5970>>2] = $5968;
          $5971 = HEAP32[$0>>2]|0;
          $5972 = ((($5971)) + 112|0);
          $5973 = HEAP32[$5972>>2]|0;
          $5974 = HEAP32[$pFrame85>>2]|0;
          $5975 = ((($5974)) + 20|0);
          HEAP32[$5975>>2] = $5973;
          $5976 = HEAP32[$0>>2]|0;
          $5977 = ((($5976)) + 28|0);
          $5978 = HEAP32[$5977>>2]|0;
          $5979 = HEAP32[$pFrame85>>2]|0;
          $5980 = ((($5979)) + 44|0);
          HEAP32[$5980>>2] = $5978;
          $5981 = HEAP32[$0>>2]|0;
          $5982 = ((($5981)) + 88|0);
          $5983 = HEAP32[$5982>>2]|0;
          $5984 = HEAP32[$pFrame85>>2]|0;
          $5985 = ((($5984)) + 8|0);
          HEAP32[$5985>>2] = $5983;
          $5986 = HEAP32[$0>>2]|0;
          $5987 = ((($5986)) + 136|0);
          $5988 = HEAP32[$5987>>2]|0;
          $5989 = HEAP32[$pFrame85>>2]|0;
          $5990 = ((($5989)) + 52|0);
          HEAP32[$5990>>2] = $5988;
          $5991 = HEAP32[$pProgram>>2]|0;
          $5992 = ((($5991)) + 16|0);
          $5993 = HEAP32[$5992>>2]|0;
          $5994 = HEAP32[$pFrame85>>2]|0;
          $5995 = ((($5994)) + 24|0);
          HEAP32[$5995>>2] = $5993;
          $5996 = HEAP32[$pFrame85>>2]|0;
          $5997 = ((($5996)) + 60|0);
          $5998 = HEAP32[$5997>>2]|0;
          $5999 = HEAP32[$pFrame85>>2]|0;
          $6000 = ((($5999)) + 80|0);
          $6001 = (($6000) + (($5998*40)|0)|0);
          HEAP32[$pEnd>>2] = $6001;
          $6002 = HEAP32[$pFrame85>>2]|0;
          $6003 = ((($6002)) + 80|0);
          HEAP32[$pMem84>>2] = $6003;
          while(1) {
           $6004 = HEAP32[$pMem84>>2]|0;
           $6005 = HEAP32[$pEnd>>2]|0;
           $6006 = ($6004|0)!=($6005|0);
           if (!($6006)) {
            break L784;
           }
           $6007 = HEAP32[$pMem84>>2]|0;
           $6008 = ((($6007)) + 8|0);
           HEAP16[$6008>>1] = 128;
           $6009 = HEAP32[$db>>2]|0;
           $6010 = HEAP32[$pMem84>>2]|0;
           $6011 = ((($6010)) + 32|0);
           HEAP32[$6011>>2] = $6009;
           $6012 = HEAP32[$pMem84>>2]|0;
           $6013 = ((($6012)) + 40|0);
           HEAP32[$pMem84>>2] = $6013;
          }
         } else {
          $6014 = HEAP32[$pRt>>2]|0;
          $6015 = HEAP32[$6014>>2]|0;
          HEAP32[$pFrame85>>2] = $6015;
         }
        } while(0);
        $6016 = HEAP32[$0>>2]|0;
        $6017 = ((($6016)) + 192|0);
        $6018 = HEAP32[$6017>>2]|0;
        $6019 = (($6018) + 1)|0;
        HEAP32[$6017>>2] = $6019;
        $6020 = HEAP32[$0>>2]|0;
        $6021 = ((($6020)) + 184|0);
        $6022 = HEAP32[$6021>>2]|0;
        $6023 = HEAP32[$pFrame85>>2]|0;
        $6024 = ((($6023)) + 4|0);
        HEAP32[$6024>>2] = $6022;
        $6025 = $lastRowid;
        $6026 = $6025;
        $6027 = HEAP32[$6026>>2]|0;
        $6028 = (($6025) + 4)|0;
        $6029 = $6028;
        $6030 = HEAP32[$6029>>2]|0;
        $6031 = HEAP32[$pFrame85>>2]|0;
        $6032 = ((($6031)) + 32|0);
        $6033 = $6032;
        $6034 = $6033;
        HEAP32[$6034>>2] = $6027;
        $6035 = (($6033) + 4)|0;
        $6036 = $6035;
        HEAP32[$6036>>2] = $6030;
        $6037 = HEAP32[$0>>2]|0;
        $6038 = ((($6037)) + 44|0);
        $6039 = HEAP32[$6038>>2]|0;
        $6040 = HEAP32[$pFrame85>>2]|0;
        $6041 = ((($6040)) + 68|0);
        HEAP32[$6041>>2] = $6039;
        $6042 = HEAP32[$0>>2]|0;
        $6043 = HEAP32[$6042>>2]|0;
        $6044 = ((($6043)) + 88|0);
        $6045 = HEAP32[$6044>>2]|0;
        $6046 = HEAP32[$pFrame85>>2]|0;
        $6047 = ((($6046)) + 72|0);
        HEAP32[$6047>>2] = $6045;
        $6048 = HEAP32[$0>>2]|0;
        $6049 = ((($6048)) + 204|0);
        $6050 = HEAP32[$6049>>2]|0;
        $6051 = HEAP32[$pFrame85>>2]|0;
        $6052 = ((($6051)) + 40|0);
        HEAP32[$6052>>2] = $6050;
        $6053 = HEAP32[$0>>2]|0;
        $6054 = ((($6053)) + 204|0);
        HEAP32[$6054>>2] = 0;
        $6055 = HEAP32[$0>>2]|0;
        $6056 = ((($6055)) + 44|0);
        HEAP32[$6056>>2] = 0;
        $6057 = HEAP32[$pFrame85>>2]|0;
        $6058 = HEAP32[$0>>2]|0;
        $6059 = ((($6058)) + 184|0);
        HEAP32[$6059>>2] = $6057;
        $6060 = HEAP32[$pFrame85>>2]|0;
        $6061 = ((($6060)) + 80|0);
        HEAP32[$aMem>>2] = $6061;
        $6062 = HEAP32[$0>>2]|0;
        $6063 = ((($6062)) + 92|0);
        HEAP32[$6063>>2] = $6061;
        $6064 = HEAP32[$pFrame85>>2]|0;
        $6065 = ((($6064)) + 60|0);
        $6066 = HEAP32[$6065>>2]|0;
        $6067 = HEAP32[$0>>2]|0;
        $6068 = ((($6067)) + 24|0);
        HEAP32[$6068>>2] = $6066;
        $6069 = HEAP32[$pFrame85>>2]|0;
        $6070 = ((($6069)) + 64|0);
        $6071 = HEAP32[$6070>>2]|0;
        $6072 = $6071&65535;
        $6073 = $6072&65535;
        $6074 = HEAP32[$0>>2]|0;
        $6075 = ((($6074)) + 28|0);
        HEAP32[$6075>>2] = $6073;
        $6076 = HEAP32[$0>>2]|0;
        $6077 = ((($6076)) + 24|0);
        $6078 = HEAP32[$6077>>2]|0;
        $6079 = HEAP32[$aMem>>2]|0;
        $6080 = (($6079) + (($6078*40)|0)|0);
        $6081 = HEAP32[$0>>2]|0;
        $6082 = ((($6081)) + 112|0);
        HEAP32[$6082>>2] = $6080;
        $6083 = HEAP32[$pProgram>>2]|0;
        $6084 = HEAP32[$6083>>2]|0;
        HEAP32[$aOp>>2] = $6084;
        $6085 = HEAP32[$0>>2]|0;
        $6086 = ((($6085)) + 88|0);
        HEAP32[$6086>>2] = $6084;
        $6087 = HEAP32[$pProgram>>2]|0;
        $6088 = ((($6087)) + 4|0);
        $6089 = HEAP32[$6088>>2]|0;
        $6090 = HEAP32[$0>>2]|0;
        $6091 = ((($6090)) + 136|0);
        HEAP32[$6091>>2] = $6089;
        $6092 = HEAP32[$aOp>>2]|0;
        $6093 = ((($6092)) + -20|0);
        HEAP32[$pOp>>2] = $6093;
        break;
       }
       case 143:  {
        $6094 = HEAP32[$0>>2]|0;
        $6095 = HEAP32[$pOp>>2]|0;
        $6096 = (_out2Prerelease($6094,$6095)|0);
        HEAP32[$pOut>>2] = $6096;
        $6097 = HEAP32[$0>>2]|0;
        $6098 = ((($6097)) + 184|0);
        $6099 = HEAP32[$6098>>2]|0;
        HEAP32[$pFrame87>>2] = $6099;
        $6100 = HEAP32[$pOp>>2]|0;
        $6101 = ((($6100)) + 4|0);
        $6102 = HEAP32[$6101>>2]|0;
        $6103 = HEAP32[$pFrame87>>2]|0;
        $6104 = ((($6103)) + 48|0);
        $6105 = HEAP32[$6104>>2]|0;
        $6106 = HEAP32[$pFrame87>>2]|0;
        $6107 = ((($6106)) + 8|0);
        $6108 = HEAP32[$6107>>2]|0;
        $6109 = (($6108) + (($6105*20)|0)|0);
        $6110 = ((($6109)) + 4|0);
        $6111 = HEAP32[$6110>>2]|0;
        $6112 = (($6102) + ($6111))|0;
        $6113 = HEAP32[$pFrame87>>2]|0;
        $6114 = ((($6113)) + 16|0);
        $6115 = HEAP32[$6114>>2]|0;
        $6116 = (($6115) + (($6112*40)|0)|0);
        HEAP32[$pIn>>2] = $6116;
        $6117 = HEAP32[$pOut>>2]|0;
        $6118 = HEAP32[$pIn>>2]|0;
        _sqlite3VdbeMemShallowCopy($6117,$6118,4096);
        break;
       }
       case 144:  {
        $6119 = HEAP32[$db>>2]|0;
        $6120 = ((($6119)) + 24|0);
        $6121 = HEAP32[$6120>>2]|0;
        $6122 = $6121 & 33554432;
        $6123 = ($6122|0)!=(0);
        $6124 = HEAP32[$pOp>>2]|0;
        if ($6123) {
         $6125 = ((($6124)) + 8|0);
         $6126 = HEAP32[$6125>>2]|0;
         $6127 = ($6126|0)<(0);
         $6128 = $6127 << 31 >> 31;
         $6129 = HEAP32[$db>>2]|0;
         $6130 = ((($6129)) + 448|0);
         $6131 = $6130;
         $6132 = $6131;
         $6133 = HEAP32[$6132>>2]|0;
         $6134 = (($6131) + 4)|0;
         $6135 = $6134;
         $6136 = HEAP32[$6135>>2]|0;
         $6137 = (_i64Add(($6133|0),($6136|0),($6126|0),($6128|0))|0);
         $6138 = tempRet0;
         $6139 = $6130;
         $6140 = $6139;
         HEAP32[$6140>>2] = $6137;
         $6141 = (($6139) + 4)|0;
         $6142 = $6141;
         HEAP32[$6142>>2] = $6138;
         break L10;
        }
        $6143 = ((($6124)) + 4|0);
        $6144 = HEAP32[$6143>>2]|0;
        $6145 = ($6144|0)!=(0);
        $6146 = HEAP32[$pOp>>2]|0;
        $6147 = ((($6146)) + 8|0);
        $6148 = HEAP32[$6147>>2]|0;
        $6149 = ($6148|0)<(0);
        $6150 = $6149 << 31 >> 31;
        if ($6145) {
         $6151 = HEAP32[$db>>2]|0;
         $6152 = ((($6151)) + 440|0);
         $6153 = $6152;
         $6154 = $6153;
         $6155 = HEAP32[$6154>>2]|0;
         $6156 = (($6153) + 4)|0;
         $6157 = $6156;
         $6158 = HEAP32[$6157>>2]|0;
         $6159 = (_i64Add(($6155|0),($6158|0),($6148|0),($6150|0))|0);
         $6160 = tempRet0;
         $6161 = $6152;
         $6162 = $6161;
         HEAP32[$6162>>2] = $6159;
         $6163 = (($6161) + 4)|0;
         $6164 = $6163;
         HEAP32[$6164>>2] = $6160;
         break L10;
        } else {
         $6165 = HEAP32[$0>>2]|0;
         $6166 = ((($6165)) + 64|0);
         $6167 = $6166;
         $6168 = $6167;
         $6169 = HEAP32[$6168>>2]|0;
         $6170 = (($6167) + 4)|0;
         $6171 = $6170;
         $6172 = HEAP32[$6171>>2]|0;
         $6173 = (_i64Add(($6169|0),($6172|0),($6148|0),($6150|0))|0);
         $6174 = tempRet0;
         $6175 = $6166;
         $6176 = $6175;
         HEAP32[$6176>>2] = $6173;
         $6177 = (($6175) + 4)|0;
         $6178 = $6177;
         HEAP32[$6178>>2] = $6174;
         break L10;
        }
        break;
       }
       case 65:  {
        $6179 = HEAP32[$pOp>>2]|0;
        $6180 = ((($6179)) + 4|0);
        $6181 = HEAP32[$6180>>2]|0;
        $6182 = ($6181|0)!=(0);
        if ($6182) {
         $6183 = HEAP32[$db>>2]|0;
         $6184 = ((($6183)) + 440|0);
         $6185 = $6184;
         $6186 = $6185;
         $6187 = HEAP32[$6186>>2]|0;
         $6188 = (($6185) + 4)|0;
         $6189 = $6188;
         $6190 = HEAP32[$6189>>2]|0;
         $6191 = ($6187|0)==(0);
         $6192 = ($6190|0)==(0);
         $6193 = $6191 & $6192;
         if (!($6193)) {
          break L10;
         }
         $6194 = HEAP32[$db>>2]|0;
         $6195 = ((($6194)) + 448|0);
         $6196 = $6195;
         $6197 = $6196;
         $6198 = HEAP32[$6197>>2]|0;
         $6199 = (($6196) + 4)|0;
         $6200 = $6199;
         $6201 = HEAP32[$6200>>2]|0;
         $6202 = ($6198|0)==(0);
         $6203 = ($6201|0)==(0);
         $6204 = $6202 & $6203;
         if ($6204) {
          label = 14;
          break L10;
         } else {
          break L10;
         }
        } else {
         $6205 = HEAP32[$0>>2]|0;
         $6206 = ((($6205)) + 64|0);
         $6207 = $6206;
         $6208 = $6207;
         $6209 = HEAP32[$6208>>2]|0;
         $6210 = (($6207) + 4)|0;
         $6211 = $6210;
         $6212 = HEAP32[$6211>>2]|0;
         $6213 = ($6209|0)==(0);
         $6214 = ($6212|0)==(0);
         $6215 = $6213 & $6214;
         if (!($6215)) {
          break L10;
         }
         $6216 = HEAP32[$db>>2]|0;
         $6217 = ((($6216)) + 448|0);
         $6218 = $6217;
         $6219 = $6218;
         $6220 = HEAP32[$6219>>2]|0;
         $6221 = (($6218) + 4)|0;
         $6222 = $6221;
         $6223 = HEAP32[$6222>>2]|0;
         $6224 = ($6220|0)==(0);
         $6225 = ($6223|0)==(0);
         $6226 = $6224 & $6225;
         if ($6226) {
          label = 14;
          break L10;
         } else {
          break L10;
         }
        }
        break;
       }
       case 145:  {
        $6227 = HEAP32[$0>>2]|0;
        $6228 = ((($6227)) + 184|0);
        $6229 = HEAP32[$6228>>2]|0;
        $6230 = ($6229|0)!=(0|0);
        if ($6230) {
         $6231 = HEAP32[$0>>2]|0;
         $6232 = ((($6231)) + 184|0);
         $6233 = HEAP32[$6232>>2]|0;
         HEAP32[$pFrame88>>2] = $6233;
         while(1) {
          $6234 = HEAP32[$pFrame88>>2]|0;
          $6235 = ((($6234)) + 4|0);
          $6236 = HEAP32[$6235>>2]|0;
          $6237 = ($6236|0)!=(0|0);
          if (!($6237)) {
           break;
          }
          $6238 = HEAP32[$pFrame88>>2]|0;
          $6239 = ((($6238)) + 4|0);
          $6240 = HEAP32[$6239>>2]|0;
          HEAP32[$pFrame88>>2] = $6240;
         }
         $6241 = HEAP32[$pOp>>2]|0;
         $6242 = ((($6241)) + 4|0);
         $6243 = HEAP32[$6242>>2]|0;
         $6244 = HEAP32[$pFrame88>>2]|0;
         $6245 = ((($6244)) + 16|0);
         $6246 = HEAP32[$6245>>2]|0;
         $6247 = (($6246) + (($6243*40)|0)|0);
         HEAP32[$pIn1>>2] = $6247;
        } else {
         $6248 = HEAP32[$pOp>>2]|0;
         $6249 = ((($6248)) + 4|0);
         $6250 = HEAP32[$6249>>2]|0;
         $6251 = HEAP32[$aMem>>2]|0;
         $6252 = (($6251) + (($6250*40)|0)|0);
         HEAP32[$pIn1>>2] = $6252;
        }
        $6253 = HEAP32[$pIn1>>2]|0;
        (_sqlite3VdbeMemIntegerify($6253)|0);
        $6254 = HEAP32[$pOp>>2]|0;
        $6255 = ((($6254)) + 8|0);
        $6256 = HEAP32[$6255>>2]|0;
        $6257 = HEAP32[$aMem>>2]|0;
        $6258 = (($6257) + (($6256*40)|0)|0);
        HEAP32[$pIn2>>2] = $6258;
        $6259 = HEAP32[$pIn2>>2]|0;
        (_sqlite3VdbeMemIntegerify($6259)|0);
        $6260 = HEAP32[$pIn1>>2]|0;
        $6261 = $6260;
        $6262 = $6261;
        $6263 = HEAP32[$6262>>2]|0;
        $6264 = (($6261) + 4)|0;
        $6265 = $6264;
        $6266 = HEAP32[$6265>>2]|0;
        $6267 = HEAP32[$pIn2>>2]|0;
        $6268 = $6267;
        $6269 = $6268;
        $6270 = HEAP32[$6269>>2]|0;
        $6271 = (($6268) + 4)|0;
        $6272 = $6271;
        $6273 = HEAP32[$6272>>2]|0;
        $6274 = ($6266|0)<($6273|0);
        $6275 = ($6263>>>0)<($6270>>>0);
        $6276 = ($6266|0)==($6273|0);
        $6277 = $6276 & $6275;
        $6278 = $6274 | $6277;
        if ($6278) {
         $6279 = HEAP32[$pIn2>>2]|0;
         $6280 = $6279;
         $6281 = $6280;
         $6282 = HEAP32[$6281>>2]|0;
         $6283 = (($6280) + 4)|0;
         $6284 = $6283;
         $6285 = HEAP32[$6284>>2]|0;
         $6286 = HEAP32[$pIn1>>2]|0;
         $6287 = $6286;
         $6288 = $6287;
         HEAP32[$6288>>2] = $6282;
         $6289 = (($6287) + 4)|0;
         $6290 = $6289;
         HEAP32[$6290>>2] = $6285;
        }
        break;
       }
       case 66:  {
        $6291 = HEAP32[$pOp>>2]|0;
        $6292 = ((($6291)) + 4|0);
        $6293 = HEAP32[$6292>>2]|0;
        $6294 = HEAP32[$aMem>>2]|0;
        $6295 = (($6294) + (($6293*40)|0)|0);
        HEAP32[$pIn1>>2] = $6295;
        $6296 = HEAP32[$pIn1>>2]|0;
        $6297 = $6296;
        $6298 = $6297;
        $6299 = HEAP32[$6298>>2]|0;
        $6300 = (($6297) + 4)|0;
        $6301 = $6300;
        $6302 = HEAP32[$6301>>2]|0;
        $6303 = ($6302|0)>(0);
        $6304 = ($6299>>>0)>(0);
        $6305 = ($6302|0)==(0);
        $6306 = $6305 & $6304;
        $6307 = $6303 | $6306;
        if ($6307) {
         $6308 = HEAP32[$pOp>>2]|0;
         $6309 = ((($6308)) + 12|0);
         $6310 = HEAP32[$6309>>2]|0;
         $6311 = ($6310|0)<(0);
         $6312 = $6311 << 31 >> 31;
         $6313 = HEAP32[$pIn1>>2]|0;
         $6314 = $6313;
         $6315 = $6314;
         $6316 = HEAP32[$6315>>2]|0;
         $6317 = (($6314) + 4)|0;
         $6318 = $6317;
         $6319 = HEAP32[$6318>>2]|0;
         $6320 = (_i64Subtract(($6316|0),($6319|0),($6310|0),($6312|0))|0);
         $6321 = tempRet0;
         $6322 = $6313;
         $6323 = $6322;
         HEAP32[$6323>>2] = $6320;
         $6324 = (($6322) + 4)|0;
         $6325 = $6324;
         HEAP32[$6325>>2] = $6321;
         label = 14;
        }
        break;
       }
       case 146:  {
        $6326 = HEAP32[$pOp>>2]|0;
        $6327 = ((($6326)) + 4|0);
        $6328 = HEAP32[$6327>>2]|0;
        $6329 = HEAP32[$aMem>>2]|0;
        $6330 = (($6329) + (($6328*40)|0)|0);
        HEAP32[$pIn1>>2] = $6330;
        $6331 = HEAP32[$pOp>>2]|0;
        $6332 = ((($6331)) + 12|0);
        $6333 = HEAP32[$6332>>2]|0;
        $6334 = HEAP32[$aMem>>2]|0;
        $6335 = (($6334) + (($6333*40)|0)|0);
        HEAP32[$pIn3>>2] = $6335;
        $6336 = HEAP32[$0>>2]|0;
        $6337 = HEAP32[$pOp>>2]|0;
        $6338 = (_out2Prerelease($6336,$6337)|0);
        HEAP32[$pOut>>2] = $6338;
        $6339 = HEAP32[$pIn1>>2]|0;
        $6340 = $6339;
        $6341 = $6340;
        $6342 = HEAP32[$6341>>2]|0;
        $6343 = (($6340) + 4)|0;
        $6344 = $6343;
        $6345 = HEAP32[$6344>>2]|0;
        $6346 = ($6345|0)<(0);
        $6347 = ($6342>>>0)<=(0);
        $6348 = ($6345|0)==(0);
        $6349 = $6348 & $6347;
        $6350 = $6346 | $6349;
        if ($6350) {
         $6384 = -1;$6387 = -1;
        } else {
         $6351 = HEAP32[$pIn1>>2]|0;
         $6352 = $6351;
         $6353 = $6352;
         $6354 = HEAP32[$6353>>2]|0;
         $6355 = (($6352) + 4)|0;
         $6356 = $6355;
         $6357 = HEAP32[$6356>>2]|0;
         $6358 = HEAP32[$pIn3>>2]|0;
         $6359 = $6358;
         $6360 = $6359;
         $6361 = HEAP32[$6360>>2]|0;
         $6362 = (($6359) + 4)|0;
         $6363 = $6362;
         $6364 = HEAP32[$6363>>2]|0;
         $6365 = ($6364|0)>(0);
         $6366 = ($6361>>>0)>(0);
         $6367 = ($6364|0)==(0);
         $6368 = $6367 & $6366;
         $6369 = $6365 | $6368;
         if ($6369) {
          $6370 = HEAP32[$pIn3>>2]|0;
          $6371 = $6370;
          $6372 = $6371;
          $6373 = HEAP32[$6372>>2]|0;
          $6374 = (($6371) + 4)|0;
          $6375 = $6374;
          $6376 = HEAP32[$6375>>2]|0;
          $6377 = $6373;$6378 = $6376;
         } else {
          $6377 = 0;$6378 = 0;
         }
         $6379 = (_i64Add(($6354|0),($6357|0),($6377|0),($6378|0))|0);
         $6380 = tempRet0;
         $6384 = $6379;$6387 = $6380;
        }
        $6381 = HEAP32[$pOut>>2]|0;
        $6382 = $6381;
        $6383 = $6382;
        HEAP32[$6383>>2] = $6384;
        $6385 = (($6382) + 4)|0;
        $6386 = $6385;
        HEAP32[$6386>>2] = $6387;
        break;
       }
       case 67:  {
        $6388 = HEAP32[$pOp>>2]|0;
        $6389 = ((($6388)) + 4|0);
        $6390 = HEAP32[$6389>>2]|0;
        $6391 = HEAP32[$aMem>>2]|0;
        $6392 = (($6391) + (($6390*40)|0)|0);
        HEAP32[$pIn1>>2] = $6392;
        $6393 = HEAP32[$pIn1>>2]|0;
        $6394 = $6393;
        $6395 = $6394;
        $6396 = HEAP32[$6395>>2]|0;
        $6397 = (($6394) + 4)|0;
        $6398 = $6397;
        $6399 = HEAP32[$6398>>2]|0;
        $6400 = ($6396|0)!=(0);
        $6401 = ($6399|0)!=(0);
        $6402 = $6400 | $6401;
        if ($6402) {
         $6403 = HEAP32[$pOp>>2]|0;
         $6404 = ((($6403)) + 12|0);
         $6405 = HEAP32[$6404>>2]|0;
         $6406 = ($6405|0)<(0);
         $6407 = $6406 << 31 >> 31;
         $6408 = HEAP32[$pIn1>>2]|0;
         $6409 = $6408;
         $6410 = $6409;
         $6411 = HEAP32[$6410>>2]|0;
         $6412 = (($6409) + 4)|0;
         $6413 = $6412;
         $6414 = HEAP32[$6413>>2]|0;
         $6415 = (_i64Subtract(($6411|0),($6414|0),($6405|0),($6407|0))|0);
         $6416 = tempRet0;
         $6417 = $6408;
         $6418 = $6417;
         HEAP32[$6418>>2] = $6415;
         $6419 = (($6417) + 4)|0;
         $6420 = $6419;
         HEAP32[$6420>>2] = $6416;
         label = 14;
        }
        break;
       }
       case 68:  {
        $6421 = HEAP32[$pOp>>2]|0;
        $6422 = ((($6421)) + 4|0);
        $6423 = HEAP32[$6422>>2]|0;
        $6424 = HEAP32[$aMem>>2]|0;
        $6425 = (($6424) + (($6423*40)|0)|0);
        HEAP32[$pIn1>>2] = $6425;
        $6426 = HEAP32[$pIn1>>2]|0;
        $6427 = $6426;
        $6428 = $6427;
        $6429 = HEAP32[$6428>>2]|0;
        $6430 = (($6427) + 4)|0;
        $6431 = $6430;
        $6432 = HEAP32[$6431>>2]|0;
        $6433 = (_i64Add(($6429|0),($6432|0),-1,-1)|0);
        $6434 = tempRet0;
        $6435 = $6426;
        $6436 = $6435;
        HEAP32[$6436>>2] = $6433;
        $6437 = (($6435) + 4)|0;
        $6438 = $6437;
        HEAP32[$6438>>2] = $6434;
        $6439 = HEAP32[$pIn1>>2]|0;
        $6440 = $6439;
        $6441 = $6440;
        $6442 = HEAP32[$6441>>2]|0;
        $6443 = (($6440) + 4)|0;
        $6444 = $6443;
        $6445 = HEAP32[$6444>>2]|0;
        $6446 = ($6442|0)==(0);
        $6447 = ($6445|0)==(0);
        $6448 = $6446 & $6447;
        if ($6448) {
         label = 14;
        }
        break;
       }
       case 147:  {
        $6449 = HEAP32[$pOp>>2]|0;
        $6450 = ((($6449)) + 3|0);
        $6451 = HEAP8[$6450>>0]|0;
        $6452 = $6451&255;
        HEAP32[$n89>>2] = $6452;
        $6453 = HEAP32[$db>>2]|0;
        $6454 = HEAP32[$n89>>2]|0;
        $6455 = (($6454) - 1)|0;
        $6456 = $6455<<2;
        $6457 = (32 + ($6456))|0;
        $6458 = (_sqlite3DbMallocRawNN($6453,$6457,0)|0);
        HEAP32[$pCtx90>>2] = $6458;
        $6459 = HEAP32[$pCtx90>>2]|0;
        $6460 = ($6459|0)==(0|0);
        if ($6460) {
         label = 889;
         break L1;
        }
        $6461 = HEAP32[$pCtx90>>2]|0;
        $6462 = ((($6461)) + 8|0);
        HEAP32[$6462>>2] = 0;
        $6463 = HEAP32[$pOp>>2]|0;
        $6464 = ((($6463)) + 16|0);
        $6465 = HEAP32[$6464>>2]|0;
        $6466 = HEAP32[$pCtx90>>2]|0;
        $6467 = ((($6466)) + 4|0);
        HEAP32[$6467>>2] = $6465;
        $6468 = HEAP32[$pOp>>2]|0;
        $6469 = HEAP32[$aOp>>2]|0;
        $6470 = $6468;
        $6471 = $6469;
        $6472 = (($6470) - ($6471))|0;
        $6473 = (($6472|0) / 20)&-1;
        $6474 = HEAP32[$pCtx90>>2]|0;
        $6475 = ((($6474)) + 16|0);
        HEAP32[$6475>>2] = $6473;
        $6476 = HEAP32[$0>>2]|0;
        $6477 = HEAP32[$pCtx90>>2]|0;
        $6478 = ((($6477)) + 12|0);
        HEAP32[$6478>>2] = $6476;
        $6479 = HEAP32[$n89>>2]|0;
        $6480 = $6479&255;
        $6481 = HEAP32[$pCtx90>>2]|0;
        $6482 = ((($6481)) + 26|0);
        HEAP8[$6482>>0] = $6480;
        $6483 = HEAP32[$pOp>>2]|0;
        $6484 = ((($6483)) + 1|0);
        HEAP8[$6484>>0] = -21;
        $6485 = HEAP32[$pCtx90>>2]|0;
        $6486 = HEAP32[$pOp>>2]|0;
        $6487 = ((($6486)) + 16|0);
        HEAP32[$6487>>2] = $6485;
        $6488 = HEAP32[$pOp>>2]|0;
        HEAP8[$6488>>0] = -108;
        label = 752;
        break;
       }
       case 148:  {
        label = 752;
        break;
       }
       case 149:  {
        $6575 = HEAP32[$pOp>>2]|0;
        $6576 = ((($6575)) + 4|0);
        $6577 = HEAP32[$6576>>2]|0;
        $6578 = HEAP32[$aMem>>2]|0;
        $6579 = (($6578) + (($6577*40)|0)|0);
        HEAP32[$pMem95>>2] = $6579;
        $6580 = HEAP32[$pMem95>>2]|0;
        $6581 = HEAP32[$pOp>>2]|0;
        $6582 = ((($6581)) + 16|0);
        $6583 = HEAP32[$6582>>2]|0;
        $6584 = (_sqlite3VdbeMemFinalize($6580,$6583)|0);
        HEAP32[$rc>>2] = $6584;
        $6585 = HEAP32[$rc>>2]|0;
        $6586 = ($6585|0)!=(0);
        if ($6586) {
         label = 764;
         break L8;
        }
        $6590 = HEAP32[$pMem95>>2]|0;
        $6591 = HEAP8[$encoding>>0]|0;
        $6592 = $6591&255;
        (_sqlite3VdbeChangeEncoding($6590,$6592)|0);
        $6593 = HEAP32[$pMem95>>2]|0;
        $6594 = (_sqlite3VdbeMemTooBig($6593)|0);
        $6595 = ($6594|0)!=(0);
        if ($6595) {
         label = 888;
         break L8;
        }
        break;
       }
       case 8:  {
        HEAP32[$aRes>>2] = 0;
        $6596 = ((($aRes)) + 8|0);
        HEAP32[$6596>>2] = -1;
        $6597 = ((($aRes)) + 4|0);
        HEAP32[$6597>>2] = -1;
        $6598 = HEAP32[$db>>2]|0;
        $6599 = HEAP32[$pOp>>2]|0;
        $6600 = ((($6599)) + 4|0);
        $6601 = HEAP32[$6600>>2]|0;
        $6602 = HEAP32[$pOp>>2]|0;
        $6603 = ((($6602)) + 8|0);
        $6604 = HEAP32[$6603>>2]|0;
        $6605 = ((($aRes)) + 4|0);
        $6606 = ((($aRes)) + 8|0);
        $6607 = (_sqlite3Checkpoint($6598,$6601,$6604,$6605,$6606)|0);
        HEAP32[$rc>>2] = $6607;
        $6608 = HEAP32[$rc>>2]|0;
        $6609 = ($6608|0)!=(0);
        if ($6609) {
         $6610 = HEAP32[$rc>>2]|0;
         $6611 = ($6610|0)!=(5);
         if ($6611) {
          break L1;
         }
         HEAP32[$rc>>2] = 0;
         HEAP32[$aRes>>2] = 1;
        }
        HEAP32[$i96>>2] = 0;
        $6612 = HEAP32[$pOp>>2]|0;
        $6613 = ((($6612)) + 12|0);
        $6614 = HEAP32[$6613>>2]|0;
        $6615 = HEAP32[$aMem>>2]|0;
        $6616 = (($6615) + (($6614*40)|0)|0);
        HEAP32[$pMem97>>2] = $6616;
        while(1) {
         $6617 = HEAP32[$i96>>2]|0;
         $6618 = ($6617|0)<(3);
         if (!($6618)) {
          break L10;
         }
         $6619 = HEAP32[$pMem97>>2]|0;
         $6620 = HEAP32[$i96>>2]|0;
         $6621 = (($aRes) + ($6620<<2)|0);
         $6622 = HEAP32[$6621>>2]|0;
         $6623 = ($6622|0)<(0);
         $6624 = $6623 << 31 >> 31;
         _sqlite3VdbeMemSetInt64($6619,$6622,$6624);
         $6625 = HEAP32[$i96>>2]|0;
         $6626 = (($6625) + 1)|0;
         HEAP32[$i96>>2] = $6626;
         $6627 = HEAP32[$pMem97>>2]|0;
         $6628 = ((($6627)) + 40|0);
         HEAP32[$pMem97>>2] = $6628;
        }
        break;
       }
       case 9:  {
        $6629 = HEAP32[$0>>2]|0;
        $6630 = HEAP32[$pOp>>2]|0;
        $6631 = (_out2Prerelease($6629,$6630)|0);
        HEAP32[$pOut>>2] = $6631;
        $6632 = HEAP32[$pOp>>2]|0;
        $6633 = ((($6632)) + 12|0);
        $6634 = HEAP32[$6633>>2]|0;
        HEAP32[$eNew>>2] = $6634;
        $6635 = HEAP32[$pOp>>2]|0;
        $6636 = ((($6635)) + 4|0);
        $6637 = HEAP32[$6636>>2]|0;
        $6638 = HEAP32[$db>>2]|0;
        $6639 = ((($6638)) + 16|0);
        $6640 = HEAP32[$6639>>2]|0;
        $6641 = (($6640) + ($6637<<4)|0);
        $6642 = ((($6641)) + 4|0);
        $6643 = HEAP32[$6642>>2]|0;
        HEAP32[$pBt98>>2] = $6643;
        $6644 = HEAP32[$pBt98>>2]|0;
        $6645 = (_sqlite3BtreePager($6644)|0);
        HEAP32[$pPager>>2] = $6645;
        $6646 = HEAP32[$pPager>>2]|0;
        $6647 = (_sqlite3PagerGetJournalMode($6646)|0);
        HEAP32[$eOld>>2] = $6647;
        $6648 = HEAP32[$eNew>>2]|0;
        $6649 = ($6648|0)==(-1);
        if ($6649) {
         $6650 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6650;
        }
        $6651 = HEAP32[$pPager>>2]|0;
        $6652 = (_sqlite3PagerOkToChangeJournalMode($6651)|0);
        $6653 = ($6652|0)!=(0);
        if (!($6653)) {
         $6654 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6654;
        }
        $6655 = HEAP32[$pPager>>2]|0;
        $6656 = (_sqlite3PagerFilename($6655,1)|0);
        HEAP32[$zFilename>>2] = $6656;
        $6657 = HEAP32[$eNew>>2]|0;
        $6658 = ($6657|0)==(5);
        do {
         if ($6658) {
          $6659 = HEAP32[$zFilename>>2]|0;
          $6660 = (_sqlite3Strlen30($6659)|0);
          $6661 = ($6660|0)==(0);
          if (!($6661)) {
           $6662 = HEAP32[$pPager>>2]|0;
           $6663 = (_sqlite3PagerWalSupported($6662)|0);
           $6664 = ($6663|0)!=(0);
           if ($6664) {
            break;
           }
          }
          $6665 = HEAP32[$eOld>>2]|0;
          HEAP32[$eNew>>2] = $6665;
         }
        } while(0);
        $6666 = HEAP32[$eNew>>2]|0;
        $6667 = HEAP32[$eOld>>2]|0;
        $6668 = ($6666|0)!=($6667|0);
        if ($6668) {
         $6669 = HEAP32[$eOld>>2]|0;
         $6670 = ($6669|0)==(5);
         $6671 = HEAP32[$eNew>>2]|0;
         $6672 = ($6671|0)==(5);
         $or$cond24 = $6670 | $6672;
         if ($or$cond24) {
          $6673 = HEAP32[$db>>2]|0;
          $6674 = ((($6673)) + 67|0);
          $6675 = HEAP8[$6674>>0]|0;
          $6676 = ($6675<<24>>24)!=(0);
          if (!($6676)) {
           label = 784;
           break L8;
          }
          $6677 = HEAP32[$db>>2]|0;
          $6678 = ((($6677)) + 160|0);
          $6679 = HEAP32[$6678>>2]|0;
          $6680 = ($6679|0)>(1);
          if ($6680) {
           label = 784;
           break L8;
          }
          $6685 = HEAP32[$eOld>>2]|0;
          $6686 = ($6685|0)==(5);
          if ($6686) {
           $6687 = HEAP32[$pPager>>2]|0;
           $6688 = (_sqlite3PagerCloseWal($6687)|0);
           HEAP32[$rc>>2] = $6688;
           $6689 = HEAP32[$rc>>2]|0;
           $6690 = ($6689|0)==(0);
           if ($6690) {
            $6691 = HEAP32[$pPager>>2]|0;
            $6692 = HEAP32[$eNew>>2]|0;
            (_sqlite3PagerSetJournalMode($6691,$6692)|0);
           }
          } else {
           $6693 = HEAP32[$eOld>>2]|0;
           $6694 = ($6693|0)==(4);
           if ($6694) {
            $6695 = HEAP32[$pPager>>2]|0;
            (_sqlite3PagerSetJournalMode($6695,2)|0);
           }
          }
          $6696 = HEAP32[$rc>>2]|0;
          $6697 = ($6696|0)==(0);
          if ($6697) {
           $6698 = HEAP32[$pBt98>>2]|0;
           $6699 = HEAP32[$eNew>>2]|0;
           $6700 = ($6699|0)==(5);
           $6701 = $6700 ? 2 : 1;
           $6702 = (_sqlite3BtreeSetVersion($6698,$6701)|0);
           HEAP32[$rc>>2] = $6702;
          }
         }
        }
        $6703 = HEAP32[$rc>>2]|0;
        $6704 = ($6703|0)!=(0);
        if ($6704) {
         $6705 = HEAP32[$eOld>>2]|0;
         HEAP32[$eNew>>2] = $6705;
        }
        $6706 = HEAP32[$pPager>>2]|0;
        $6707 = HEAP32[$eNew>>2]|0;
        $6708 = (_sqlite3PagerSetJournalMode($6706,$6707)|0);
        HEAP32[$eNew>>2] = $6708;
        $6709 = HEAP32[$pOut>>2]|0;
        $6710 = ((($6709)) + 8|0);
        HEAP16[$6710>>1] = 2562;
        $6711 = HEAP32[$eNew>>2]|0;
        $6712 = (_sqlite3JournalModename($6711)|0);
        $6713 = HEAP32[$pOut>>2]|0;
        $6714 = ((($6713)) + 16|0);
        HEAP32[$6714>>2] = $6712;
        $6715 = HEAP32[$pOut>>2]|0;
        $6716 = ((($6715)) + 16|0);
        $6717 = HEAP32[$6716>>2]|0;
        $6718 = (_sqlite3Strlen30($6717)|0);
        $6719 = HEAP32[$pOut>>2]|0;
        $6720 = ((($6719)) + 12|0);
        HEAP32[$6720>>2] = $6718;
        $6721 = HEAP32[$pOut>>2]|0;
        $6722 = ((($6721)) + 10|0);
        HEAP8[$6722>>0] = 1;
        $6723 = HEAP32[$pOut>>2]|0;
        $6724 = HEAP8[$encoding>>0]|0;
        $6725 = $6724&255;
        (_sqlite3VdbeChangeEncoding($6723,$6725)|0);
        $6726 = HEAP32[$rc>>2]|0;
        $6727 = ($6726|0)!=(0);
        if ($6727) {
         break L1;
        }
        break;
       }
       case 10:  {
        $6728 = HEAP32[$0>>2]|0;
        $6729 = ((($6728)) + 108|0);
        $6730 = HEAP32[$db>>2]|0;
        $6731 = HEAP32[$pOp>>2]|0;
        $6732 = ((($6731)) + 4|0);
        $6733 = HEAP32[$6732>>2]|0;
        $6734 = (_sqlite3RunVacuum($6729,$6730,$6733)|0);
        HEAP32[$rc>>2] = $6734;
        $6735 = HEAP32[$rc>>2]|0;
        $6736 = ($6735|0)!=(0);
        if ($6736) {
         break L1;
        }
        break;
       }
       case 69:  {
        $6737 = HEAP32[$pOp>>2]|0;
        $6738 = ((($6737)) + 4|0);
        $6739 = HEAP32[$6738>>2]|0;
        $6740 = HEAP32[$db>>2]|0;
        $6741 = ((($6740)) + 16|0);
        $6742 = HEAP32[$6741>>2]|0;
        $6743 = (($6742) + ($6739<<4)|0);
        $6744 = ((($6743)) + 4|0);
        $6745 = HEAP32[$6744>>2]|0;
        HEAP32[$pBt99>>2] = $6745;
        $6746 = HEAP32[$pBt99>>2]|0;
        $6747 = (_sqlite3BtreeIncrVacuum($6746)|0);
        HEAP32[$rc>>2] = $6747;
        $6748 = HEAP32[$rc>>2]|0;
        $6749 = ($6748|0)!=(0);
        if ($6749) {
         $6750 = HEAP32[$rc>>2]|0;
         $6751 = ($6750|0)!=(101);
         if ($6751) {
          break L1;
         }
         HEAP32[$rc>>2] = 0;
         label = 14;
        }
        break;
       }
       case 150:  {
        $6752 = HEAP32[$pOp>>2]|0;
        $6753 = ((($6752)) + 4|0);
        $6754 = HEAP32[$6753>>2]|0;
        $6755 = ($6754|0)!=(0);
        if ($6755) {
         $6757 = HEAP32[$0>>2]|0;
         $6758 = ((($6757)) + 144|0);
         $6759 = HEAP16[$6758>>1]|0;
         $6760 = $6759 & -2;
         $6761 = $6760 | 1;
         HEAP16[$6758>>1] = $6761;
         break L10;
        } else {
         $6756 = HEAP32[$db>>2]|0;
         _sqlite3ExpirePreparedStatements($6756);
         break L10;
        }
        break;
       }
       case 151:  {
        $6762 = HEAP32[$pOp>>2]|0;
        $6763 = ((($6762)) + 12|0);
        $6764 = HEAP32[$6763>>2]|0;
        $6765 = $6764&255;
        HEAP8[$isWriteLock>>0] = $6765;
        $6766 = HEAP8[$isWriteLock>>0]|0;
        $6767 = $6766&255;
        $6768 = ($6767|0)!=(0);
        if (!($6768)) {
         $6769 = HEAP32[$db>>2]|0;
         $6770 = ((($6769)) + 24|0);
         $6771 = HEAP32[$6770>>2]|0;
         $6772 = $6771 & 16384;
         $6773 = (0)==($6772|0);
         if (!($6773)) {
          break L10;
         }
        }
        $6774 = HEAP32[$pOp>>2]|0;
        $6775 = ((($6774)) + 4|0);
        $6776 = HEAP32[$6775>>2]|0;
        HEAP32[$p1100>>2] = $6776;
        $6777 = HEAP32[$p1100>>2]|0;
        $6778 = HEAP32[$db>>2]|0;
        $6779 = ((($6778)) + 16|0);
        $6780 = HEAP32[$6779>>2]|0;
        $6781 = (($6780) + ($6777<<4)|0);
        $6782 = ((($6781)) + 4|0);
        $6783 = HEAP32[$6782>>2]|0;
        $6784 = HEAP32[$pOp>>2]|0;
        $6785 = ((($6784)) + 8|0);
        $6786 = HEAP32[$6785>>2]|0;
        $6787 = HEAP8[$isWriteLock>>0]|0;
        $6788 = (_sqlite3BtreeLockTable($6783,$6786,$6787)|0);
        HEAP32[$rc>>2] = $6788;
        $6789 = HEAP32[$rc>>2]|0;
        $6790 = ($6789|0)!=(0);
        if ($6790) {
         label = 805;
         break L8;
        }
        break;
       }
       case 152:  {
        $6799 = HEAP32[$pOp>>2]|0;
        $6800 = ((($6799)) + 16|0);
        $6801 = HEAP32[$6800>>2]|0;
        HEAP32[$pVTab>>2] = $6801;
        $6802 = HEAP32[$db>>2]|0;
        $6803 = HEAP32[$pVTab>>2]|0;
        $6804 = (_sqlite3VtabBegin($6802,$6803)|0);
        HEAP32[$rc>>2] = $6804;
        $6805 = HEAP32[$pVTab>>2]|0;
        $6806 = ($6805|0)!=(0|0);
        if ($6806) {
         $6807 = HEAP32[$0>>2]|0;
         $6808 = HEAP32[$pVTab>>2]|0;
         $6809 = ((($6808)) + 8|0);
         $6810 = HEAP32[$6809>>2]|0;
         _sqlite3VtabImportErrmsg($6807,$6810);
        }
        $6811 = HEAP32[$rc>>2]|0;
        $6812 = ($6811|0)!=(0);
        if ($6812) {
         break L1;
        }
        break;
       }
       case 153:  {
        dest=$sMem102; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
        $6813 = HEAP32[$db>>2]|0;
        $6814 = ((($sMem102)) + 32|0);
        HEAP32[$6814>>2] = $6813;
        $6815 = HEAP32[$pOp>>2]|0;
        $6816 = ((($6815)) + 8|0);
        $6817 = HEAP32[$6816>>2]|0;
        $6818 = HEAP32[$aMem>>2]|0;
        $6819 = (($6818) + (($6817*40)|0)|0);
        $6820 = (_sqlite3VdbeMemCopy($sMem102,$6819)|0);
        HEAP32[$rc>>2] = $6820;
        $6821 = (_sqlite3_value_text($sMem102)|0);
        HEAP32[$zTab>>2] = $6821;
        $6822 = HEAP32[$zTab>>2]|0;
        $6823 = ($6822|0)!=(0|0);
        if ($6823) {
         $6824 = HEAP32[$db>>2]|0;
         $6825 = HEAP32[$pOp>>2]|0;
         $6826 = ((($6825)) + 4|0);
         $6827 = HEAP32[$6826>>2]|0;
         $6828 = HEAP32[$zTab>>2]|0;
         $6829 = HEAP32[$0>>2]|0;
         $6830 = ((($6829)) + 108|0);
         $6831 = (_sqlite3VtabCallCreate($6824,$6827,$6828,$6830)|0);
         HEAP32[$rc>>2] = $6831;
        }
        _sqlite3VdbeMemRelease($sMem102);
        $6832 = HEAP32[$rc>>2]|0;
        $6833 = ($6832|0)!=(0);
        if ($6833) {
         break L1;
        }
        break;
       }
       case 154:  {
        $6834 = HEAP32[$db>>2]|0;
        $6835 = ((($6834)) + 172|0);
        $6836 = HEAP32[$6835>>2]|0;
        $6837 = (($6836) + 1)|0;
        HEAP32[$6835>>2] = $6837;
        $6838 = HEAP32[$db>>2]|0;
        $6839 = HEAP32[$pOp>>2]|0;
        $6840 = ((($6839)) + 4|0);
        $6841 = HEAP32[$6840>>2]|0;
        $6842 = HEAP32[$pOp>>2]|0;
        $6843 = ((($6842)) + 16|0);
        $6844 = HEAP32[$6843>>2]|0;
        $6845 = (_sqlite3VtabCallDestroy($6838,$6841,$6844)|0);
        HEAP32[$rc>>2] = $6845;
        $6846 = HEAP32[$db>>2]|0;
        $6847 = ((($6846)) + 172|0);
        $6848 = HEAP32[$6847>>2]|0;
        $6849 = (($6848) + -1)|0;
        HEAP32[$6847>>2] = $6849;
        $6850 = HEAP32[$rc>>2]|0;
        $6851 = ($6850|0)!=(0);
        if ($6851) {
         break L1;
        }
        break;
       }
       case 155:  {
        HEAP32[$pCur103>>2] = 0;
        HEAP32[$pVCur>>2] = 0;
        $6852 = HEAP32[$pOp>>2]|0;
        $6853 = ((($6852)) + 16|0);
        $6854 = HEAP32[$6853>>2]|0;
        $6855 = ((($6854)) + 8|0);
        $6856 = HEAP32[$6855>>2]|0;
        HEAP32[$pVtab104>>2] = $6856;
        $6857 = HEAP32[$pVtab104>>2]|0;
        $6858 = ($6857|0)==(0|0);
        if ($6858) {
         label = 816;
         break L8;
        }
        $6859 = HEAP32[$pVtab104>>2]|0;
        $6860 = HEAP32[$6859>>2]|0;
        $6861 = ($6860|0)==(0|0);
        if ($6861) {
         label = 816;
         break L8;
        }
        $6862 = HEAP32[$pVtab104>>2]|0;
        $6863 = HEAP32[$6862>>2]|0;
        HEAP32[$pModule105>>2] = $6863;
        $6864 = HEAP32[$pModule105>>2]|0;
        $6865 = ((($6864)) + 24|0);
        $6866 = HEAP32[$6865>>2]|0;
        $6867 = HEAP32[$pVtab104>>2]|0;
        $6868 = (FUNCTION_TABLE_iii[$6866 & 255]($6867,$pVCur)|0);
        HEAP32[$rc>>2] = $6868;
        $6869 = HEAP32[$0>>2]|0;
        $6870 = HEAP32[$pVtab104>>2]|0;
        _sqlite3VtabImportErrmsg($6869,$6870);
        $6871 = HEAP32[$rc>>2]|0;
        $6872 = ($6871|0)!=(0);
        if ($6872) {
         break L1;
        }
        $6873 = HEAP32[$pVtab104>>2]|0;
        $6874 = HEAP32[$pVCur>>2]|0;
        HEAP32[$6874>>2] = $6873;
        $6875 = HEAP32[$0>>2]|0;
        $6876 = HEAP32[$pOp>>2]|0;
        $6877 = ((($6876)) + 4|0);
        $6878 = HEAP32[$6877>>2]|0;
        $6879 = (_allocateCursor($6875,$6878,0,-1,2)|0);
        HEAP32[$pCur103>>2] = $6879;
        $6880 = HEAP32[$pCur103>>2]|0;
        $6881 = ($6880|0)!=(0|0);
        if (!($6881)) {
         label = 820;
         break L8;
        }
        $6882 = HEAP32[$pVCur>>2]|0;
        $6883 = HEAP32[$pCur103>>2]|0;
        $6884 = ((($6883)) + 16|0);
        HEAP32[$6884>>2] = $6882;
        $6885 = HEAP32[$pVtab104>>2]|0;
        $6886 = ((($6885)) + 4|0);
        $6887 = HEAP32[$6886>>2]|0;
        $6888 = (($6887) + 1)|0;
        HEAP32[$6886>>2] = $6888;
        break;
       }
       case 11:  {
        $6893 = HEAP32[$pOp>>2]|0;
        $6894 = ((($6893)) + 12|0);
        $6895 = HEAP32[$6894>>2]|0;
        $6896 = HEAP32[$aMem>>2]|0;
        $6897 = (($6896) + (($6895*40)|0)|0);
        HEAP32[$pQuery>>2] = $6897;
        $6898 = HEAP32[$pQuery>>2]|0;
        $6899 = ((($6898)) + 40|0);
        HEAP32[$pArgc>>2] = $6899;
        $6900 = HEAP32[$pOp>>2]|0;
        $6901 = ((($6900)) + 4|0);
        $6902 = HEAP32[$6901>>2]|0;
        $6903 = HEAP32[$0>>2]|0;
        $6904 = ((($6903)) + 112|0);
        $6905 = HEAP32[$6904>>2]|0;
        $6906 = (($6905) + ($6902<<2)|0);
        $6907 = HEAP32[$6906>>2]|0;
        HEAP32[$pCur109>>2] = $6907;
        $6908 = HEAP32[$pCur109>>2]|0;
        $6909 = ((($6908)) + 16|0);
        $6910 = HEAP32[$6909>>2]|0;
        HEAP32[$pVCur107>>2] = $6910;
        $6911 = HEAP32[$pVCur107>>2]|0;
        $6912 = HEAP32[$6911>>2]|0;
        HEAP32[$pVtab108>>2] = $6912;
        $6913 = HEAP32[$pVtab108>>2]|0;
        $6914 = HEAP32[$6913>>2]|0;
        HEAP32[$pModule106>>2] = $6914;
        $6915 = HEAP32[$pArgc>>2]|0;
        $6916 = $6915;
        $6917 = $6916;
        $6918 = HEAP32[$6917>>2]|0;
        $6919 = (($6916) + 4)|0;
        $6920 = $6919;
        $6921 = HEAP32[$6920>>2]|0;
        HEAP32[$nArg>>2] = $6918;
        $6922 = HEAP32[$pQuery>>2]|0;
        $6923 = $6922;
        $6924 = $6923;
        $6925 = HEAP32[$6924>>2]|0;
        $6926 = (($6923) + 4)|0;
        $6927 = $6926;
        $6928 = HEAP32[$6927>>2]|0;
        HEAP32[$iQuery>>2] = $6925;
        HEAP32[$res110>>2] = 0;
        $6929 = HEAP32[$0>>2]|0;
        $6930 = ((($6929)) + 96|0);
        $6931 = HEAP32[$6930>>2]|0;
        HEAP32[$apArg>>2] = $6931;
        HEAP32[$i111>>2] = 0;
        while(1) {
         $6932 = HEAP32[$i111>>2]|0;
         $6933 = HEAP32[$nArg>>2]|0;
         $6934 = ($6932|0)<($6933|0);
         if (!($6934)) {
          break;
         }
         $6935 = HEAP32[$i111>>2]|0;
         $6936 = (($6935) + 1)|0;
         $6937 = HEAP32[$pArgc>>2]|0;
         $6938 = (($6937) + (($6936*40)|0)|0);
         $6939 = HEAP32[$i111>>2]|0;
         $6940 = HEAP32[$apArg>>2]|0;
         $6941 = (($6940) + ($6939<<2)|0);
         HEAP32[$6941>>2] = $6938;
         $6942 = HEAP32[$i111>>2]|0;
         $6943 = (($6942) + 1)|0;
         HEAP32[$i111>>2] = $6943;
        }
        $6944 = HEAP32[$pModule106>>2]|0;
        $6945 = ((($6944)) + 32|0);
        $6946 = HEAP32[$6945>>2]|0;
        $6947 = HEAP32[$pVCur107>>2]|0;
        $6948 = HEAP32[$iQuery>>2]|0;
        $6949 = HEAP32[$pOp>>2]|0;
        $6950 = ((($6949)) + 16|0);
        $6951 = HEAP32[$6950>>2]|0;
        $6952 = HEAP32[$nArg>>2]|0;
        $6953 = HEAP32[$apArg>>2]|0;
        $6954 = (FUNCTION_TABLE_iiiiii[$6946 & 255]($6947,$6948,$6951,$6952,$6953)|0);
        HEAP32[$rc>>2] = $6954;
        $6955 = HEAP32[$0>>2]|0;
        $6956 = HEAP32[$pVtab108>>2]|0;
        _sqlite3VtabImportErrmsg($6955,$6956);
        $6957 = HEAP32[$rc>>2]|0;
        $6958 = ($6957|0)!=(0);
        if ($6958) {
         break L1;
        }
        $6959 = HEAP32[$pModule106>>2]|0;
        $6960 = ((($6959)) + 40|0);
        $6961 = HEAP32[$6960>>2]|0;
        $6962 = HEAP32[$pVCur107>>2]|0;
        $6963 = (FUNCTION_TABLE_ii[$6961 & 255]($6962)|0);
        HEAP32[$res110>>2] = $6963;
        $6964 = HEAP32[$pCur109>>2]|0;
        $6965 = ((($6964)) + 2|0);
        HEAP8[$6965>>0] = 0;
        $6966 = HEAP32[$res110>>2]|0;
        $6967 = ($6966|0)!=(0);
        if ($6967) {
         label = 14;
        }
        break;
       }
       case 156:  {
        $6968 = HEAP32[$pOp>>2]|0;
        $6969 = ((($6968)) + 4|0);
        $6970 = HEAP32[$6969>>2]|0;
        $6971 = HEAP32[$0>>2]|0;
        $6972 = ((($6971)) + 112|0);
        $6973 = HEAP32[$6972>>2]|0;
        $6974 = (($6973) + ($6970<<2)|0);
        $6975 = HEAP32[$6974>>2]|0;
        HEAP32[$pCur115>>2] = $6975;
        $6976 = HEAP32[$pOp>>2]|0;
        $6977 = ((($6976)) + 12|0);
        $6978 = HEAP32[$6977>>2]|0;
        $6979 = HEAP32[$aMem>>2]|0;
        $6980 = (($6979) + (($6978*40)|0)|0);
        HEAP32[$pDest114>>2] = $6980;
        $6981 = HEAP32[$pCur115>>2]|0;
        $6982 = ((($6981)) + 2|0);
        $6983 = HEAP8[$6982>>0]|0;
        $6984 = ($6983<<24>>24)!=(0);
        if ($6984) {
         $6985 = HEAP32[$pDest114>>2]|0;
         _sqlite3VdbeMemSetNull($6985);
         break L10;
        }
        $6986 = HEAP32[$pCur115>>2]|0;
        $6987 = ((($6986)) + 16|0);
        $6988 = HEAP32[$6987>>2]|0;
        $6989 = HEAP32[$6988>>2]|0;
        HEAP32[$pVtab112>>2] = $6989;
        $6990 = HEAP32[$pVtab112>>2]|0;
        $6991 = HEAP32[$6990>>2]|0;
        HEAP32[$pModule113>>2] = $6991;
        ;HEAP32[$sContext>>2]=0|0;HEAP32[$sContext+4>>2]=0|0;HEAP32[$sContext+8>>2]=0|0;HEAP32[$sContext+12>>2]=0|0;HEAP32[$sContext+16>>2]=0|0;HEAP32[$sContext+20>>2]=0|0;HEAP32[$sContext+24>>2]=0|0;HEAP32[$sContext+28>>2]=0|0;
        $6992 = HEAP32[$pDest114>>2]|0;
        HEAP32[$sContext>>2] = $6992;
        $6993 = HEAP32[$pDest114>>2]|0;
        $6994 = ((($6993)) + 8|0);
        $6995 = HEAP16[$6994>>1]|0;
        $6996 = $6995&65535;
        $6997 = $6996 & -49664;
        $6998 = $6997 | 1;
        $6999 = $6998&65535;
        $7000 = HEAP32[$pDest114>>2]|0;
        $7001 = ((($7000)) + 8|0);
        HEAP16[$7001>>1] = $6999;
        $7002 = HEAP32[$pModule113>>2]|0;
        $7003 = ((($7002)) + 44|0);
        $7004 = HEAP32[$7003>>2]|0;
        $7005 = HEAP32[$pCur115>>2]|0;
        $7006 = ((($7005)) + 16|0);
        $7007 = HEAP32[$7006>>2]|0;
        $7008 = HEAP32[$pOp>>2]|0;
        $7009 = ((($7008)) + 8|0);
        $7010 = HEAP32[$7009>>2]|0;
        $7011 = (FUNCTION_TABLE_iiii[$7004 & 255]($7007,$sContext,$7010)|0);
        HEAP32[$rc>>2] = $7011;
        $7012 = HEAP32[$0>>2]|0;
        $7013 = HEAP32[$pVtab112>>2]|0;
        _sqlite3VtabImportErrmsg($7012,$7013);
        $7014 = ((($sContext)) + 20|0);
        $7015 = HEAP32[$7014>>2]|0;
        $7016 = ($7015|0)!=(0);
        if ($7016) {
         $7017 = ((($sContext)) + 20|0);
         $7018 = HEAP32[$7017>>2]|0;
         HEAP32[$rc>>2] = $7018;
        }
        $7019 = HEAP32[$pDest114>>2]|0;
        $7020 = HEAP8[$encoding>>0]|0;
        $7021 = $7020&255;
        (_sqlite3VdbeChangeEncoding($7019,$7021)|0);
        $7022 = HEAP32[$pDest114>>2]|0;
        $7023 = (_sqlite3VdbeMemTooBig($7022)|0);
        $7024 = ($7023|0)!=(0);
        if ($7024) {
         label = 888;
         break L8;
        }
        $7025 = HEAP32[$rc>>2]|0;
        $7026 = ($7025|0)!=(0);
        if ($7026) {
         break L1;
        }
        break;
       }
       case 70:  {
        HEAP32[$res118>>2] = 0;
        $7027 = HEAP32[$pOp>>2]|0;
        $7028 = ((($7027)) + 4|0);
        $7029 = HEAP32[$7028>>2]|0;
        $7030 = HEAP32[$0>>2]|0;
        $7031 = ((($7030)) + 112|0);
        $7032 = HEAP32[$7031>>2]|0;
        $7033 = (($7032) + ($7029<<2)|0);
        $7034 = HEAP32[$7033>>2]|0;
        HEAP32[$pCur119>>2] = $7034;
        $7035 = HEAP32[$pCur119>>2]|0;
        $7036 = ((($7035)) + 2|0);
        $7037 = HEAP8[$7036>>0]|0;
        $7038 = ($7037<<24>>24)!=(0);
        if (!($7038)) {
         $7039 = HEAP32[$pCur119>>2]|0;
         $7040 = ((($7039)) + 16|0);
         $7041 = HEAP32[$7040>>2]|0;
         $7042 = HEAP32[$7041>>2]|0;
         HEAP32[$pVtab116>>2] = $7042;
         $7043 = HEAP32[$pVtab116>>2]|0;
         $7044 = HEAP32[$7043>>2]|0;
         HEAP32[$pModule117>>2] = $7044;
         $7045 = HEAP32[$pModule117>>2]|0;
         $7046 = ((($7045)) + 36|0);
         $7047 = HEAP32[$7046>>2]|0;
         $7048 = HEAP32[$pCur119>>2]|0;
         $7049 = ((($7048)) + 16|0);
         $7050 = HEAP32[$7049>>2]|0;
         $7051 = (FUNCTION_TABLE_ii[$7047 & 255]($7050)|0);
         HEAP32[$rc>>2] = $7051;
         $7052 = HEAP32[$0>>2]|0;
         $7053 = HEAP32[$pVtab116>>2]|0;
         _sqlite3VtabImportErrmsg($7052,$7053);
         $7054 = HEAP32[$rc>>2]|0;
         $7055 = ($7054|0)!=(0);
         if ($7055) {
          break L1;
         }
         $7056 = HEAP32[$pModule117>>2]|0;
         $7057 = ((($7056)) + 40|0);
         $7058 = HEAP32[$7057>>2]|0;
         $7059 = HEAP32[$pCur119>>2]|0;
         $7060 = ((($7059)) + 16|0);
         $7061 = HEAP32[$7060>>2]|0;
         $7062 = (FUNCTION_TABLE_ii[$7058 & 255]($7061)|0);
         HEAP32[$res118>>2] = $7062;
         $7063 = HEAP32[$res118>>2]|0;
         $7064 = ($7063|0)!=(0);
         if ($7064) {
          label = 8;
         } else {
          label = 7;
         }
        }
        break;
       }
       case 157:  {
        $7065 = HEAP32[$pOp>>2]|0;
        $7066 = ((($7065)) + 16|0);
        $7067 = HEAP32[$7066>>2]|0;
        $7068 = ((($7067)) + 8|0);
        $7069 = HEAP32[$7068>>2]|0;
        HEAP32[$pVtab120>>2] = $7069;
        $7070 = HEAP32[$pOp>>2]|0;
        $7071 = ((($7070)) + 4|0);
        $7072 = HEAP32[$7071>>2]|0;
        $7073 = HEAP32[$aMem>>2]|0;
        $7074 = (($7073) + (($7072*40)|0)|0);
        HEAP32[$pName>>2] = $7074;
        $7075 = HEAP32[$pName>>2]|0;
        $7076 = (_sqlite3VdbeChangeEncoding($7075,1)|0);
        HEAP32[$rc>>2] = $7076;
        $7077 = HEAP32[$rc>>2]|0;
        $7078 = ($7077|0)!=(0);
        if ($7078) {
         break L1;
        }
        $7079 = HEAP32[$pVtab120>>2]|0;
        $7080 = HEAP32[$7079>>2]|0;
        $7081 = ((($7080)) + 76|0);
        $7082 = HEAP32[$7081>>2]|0;
        $7083 = HEAP32[$pVtab120>>2]|0;
        $7084 = HEAP32[$pName>>2]|0;
        $7085 = ((($7084)) + 16|0);
        $7086 = HEAP32[$7085>>2]|0;
        $7087 = (FUNCTION_TABLE_iii[$7082 & 255]($7083,$7086)|0);
        HEAP32[$rc>>2] = $7087;
        $7088 = HEAP32[$0>>2]|0;
        $7089 = HEAP32[$pVtab120>>2]|0;
        _sqlite3VtabImportErrmsg($7088,$7089);
        $7090 = HEAP32[$0>>2]|0;
        $7091 = ((($7090)) + 144|0);
        $7092 = HEAP16[$7091>>1]|0;
        $7093 = $7092 & -2;
        HEAP16[$7091>>1] = $7093;
        $7094 = HEAP32[$rc>>2]|0;
        $7095 = ($7094|0)!=(0);
        if ($7095) {
         break L1;
        }
        break;
       }
       case 12:  {
        $7096 = HEAP32[$pOp>>2]|0;
        $7097 = ((($7096)) + 16|0);
        $7098 = HEAP32[$7097>>2]|0;
        $7099 = ((($7098)) + 8|0);
        $7100 = HEAP32[$7099>>2]|0;
        HEAP32[$pVtab121>>2] = $7100;
        $7101 = HEAP32[$pVtab121>>2]|0;
        $7102 = ($7101|0)==(0|0);
        if ($7102) {
         label = 839;
         break L8;
        }
        $7103 = HEAP32[$pVtab121>>2]|0;
        $7104 = HEAP32[$7103>>2]|0;
        $7105 = ($7104|0)==(0|0);
        if ($7105) {
         label = 839;
         break L8;
        }
        $7106 = HEAP32[$pVtab121>>2]|0;
        $7107 = HEAP32[$7106>>2]|0;
        HEAP32[$pModule122>>2] = $7107;
        $7108 = HEAP32[$pOp>>2]|0;
        $7109 = ((($7108)) + 8|0);
        $7110 = HEAP32[$7109>>2]|0;
        HEAP32[$nArg123>>2] = $7110;
        $7111 = HEAP32[$pModule122>>2]|0;
        $7112 = ((($7111)) + 52|0);
        $7113 = HEAP32[$7112>>2]|0;
        $7114 = ($7113|0)!=(0|0);
        if ($7114) {
         $7115 = HEAP32[$db>>2]|0;
         $7116 = ((($7115)) + 74|0);
         $7117 = HEAP8[$7116>>0]|0;
         HEAP8[$vtabOnConflict>>0] = $7117;
         $7118 = HEAP32[$0>>2]|0;
         $7119 = ((($7118)) + 96|0);
         $7120 = HEAP32[$7119>>2]|0;
         HEAP32[$apArg126>>2] = $7120;
         $7121 = HEAP32[$pOp>>2]|0;
         $7122 = ((($7121)) + 12|0);
         $7123 = HEAP32[$7122>>2]|0;
         $7124 = HEAP32[$aMem>>2]|0;
         $7125 = (($7124) + (($7123*40)|0)|0);
         HEAP32[$pX127>>2] = $7125;
         HEAP32[$i124>>2] = 0;
         while(1) {
          $7126 = HEAP32[$i124>>2]|0;
          $7127 = HEAP32[$nArg123>>2]|0;
          $7128 = ($7126|0)<($7127|0);
          if (!($7128)) {
           break;
          }
          $7129 = HEAP32[$pX127>>2]|0;
          $7130 = HEAP32[$i124>>2]|0;
          $7131 = HEAP32[$apArg126>>2]|0;
          $7132 = (($7131) + ($7130<<2)|0);
          HEAP32[$7132>>2] = $7129;
          $7133 = HEAP32[$pX127>>2]|0;
          $7134 = ((($7133)) + 40|0);
          HEAP32[$pX127>>2] = $7134;
          $7135 = HEAP32[$i124>>2]|0;
          $7136 = (($7135) + 1)|0;
          HEAP32[$i124>>2] = $7136;
         }
         $7137 = HEAP32[$pOp>>2]|0;
         $7138 = ((($7137)) + 3|0);
         $7139 = HEAP8[$7138>>0]|0;
         $7140 = HEAP32[$db>>2]|0;
         $7141 = ((($7140)) + 74|0);
         HEAP8[$7141>>0] = $7139;
         $7142 = HEAP32[$pModule122>>2]|0;
         $7143 = ((($7142)) + 52|0);
         $7144 = HEAP32[$7143>>2]|0;
         $7145 = HEAP32[$pVtab121>>2]|0;
         $7146 = HEAP32[$nArg123>>2]|0;
         $7147 = HEAP32[$apArg126>>2]|0;
         $7148 = (FUNCTION_TABLE_iiiii[$7144 & 255]($7145,$7146,$7147,$rowid125)|0);
         HEAP32[$rc>>2] = $7148;
         $7149 = HEAP8[$vtabOnConflict>>0]|0;
         $7150 = HEAP32[$db>>2]|0;
         $7151 = ((($7150)) + 74|0);
         HEAP8[$7151>>0] = $7149;
         $7152 = HEAP32[$0>>2]|0;
         $7153 = HEAP32[$pVtab121>>2]|0;
         _sqlite3VtabImportErrmsg($7152,$7153);
         $7154 = HEAP32[$rc>>2]|0;
         $7155 = ($7154|0)==(0);
         if ($7155) {
          $7156 = HEAP32[$pOp>>2]|0;
          $7157 = ((($7156)) + 4|0);
          $7158 = HEAP32[$7157>>2]|0;
          $7159 = ($7158|0)!=(0);
          if ($7159) {
           $7160 = $rowid125;
           $7161 = $7160;
           $7162 = HEAP32[$7161>>2]|0;
           $7163 = (($7160) + 4)|0;
           $7164 = $7163;
           $7165 = HEAP32[$7164>>2]|0;
           $7166 = $lastRowid;
           $7167 = $7166;
           HEAP32[$7167>>2] = $7162;
           $7168 = (($7166) + 4)|0;
           $7169 = $7168;
           HEAP32[$7169>>2] = $7165;
           $7170 = HEAP32[$db>>2]|0;
           $7171 = ((($7170)) + 32|0);
           $7172 = $7171;
           $7173 = $7172;
           HEAP32[$7173>>2] = $7162;
           $7174 = (($7172) + 4)|0;
           $7175 = $7174;
           HEAP32[$7175>>2] = $7165;
          }
         }
         $7176 = HEAP32[$rc>>2]|0;
         $7177 = $7176 & 255;
         $7178 = ($7177|0)==(19);
         do {
          if ($7178) {
           $7179 = HEAP32[$pOp>>2]|0;
           $7180 = ((($7179)) + 16|0);
           $7181 = HEAP32[$7180>>2]|0;
           $7182 = ((($7181)) + 16|0);
           $7183 = HEAP8[$7182>>0]|0;
           $7184 = $7183&255;
           $7185 = ($7184|0)!=(0);
           if ($7185) {
            $7186 = HEAP32[$pOp>>2]|0;
            $7187 = ((($7186)) + 3|0);
            $7188 = HEAP8[$7187>>0]|0;
            $7189 = $7188&255;
            $7190 = ($7189|0)==(4);
            if ($7190) {
             HEAP32[$rc>>2] = 0;
             break;
            }
            $7191 = HEAP32[$pOp>>2]|0;
            $7192 = ((($7191)) + 3|0);
            $7193 = HEAP8[$7192>>0]|0;
            $7194 = $7193&255;
            $7195 = ($7194|0)==(5);
            if ($7195) {
             $7201 = 2;
            } else {
             $7196 = HEAP32[$pOp>>2]|0;
             $7197 = ((($7196)) + 3|0);
             $7198 = HEAP8[$7197>>0]|0;
             $7199 = $7198&255;
             $7201 = $7199;
            }
            $7200 = $7201&255;
            $7202 = HEAP32[$0>>2]|0;
            $7203 = ((($7202)) + 142|0);
            HEAP8[$7203>>0] = $7200;
           } else {
            label = 854;
           }
          } else {
           label = 854;
          }
         } while(0);
         if ((label|0) == 854) {
          label = 0;
          $7204 = HEAP32[$0>>2]|0;
          $7205 = ((($7204)) + 44|0);
          $7206 = HEAP32[$7205>>2]|0;
          $7207 = (($7206) + 1)|0;
          HEAP32[$7205>>2] = $7207;
         }
         $7208 = HEAP32[$rc>>2]|0;
         $7209 = ($7208|0)!=(0);
         if ($7209) {
          break L1;
         }
        }
        break;
       }
       case 158:  {
        $7210 = HEAP32[$0>>2]|0;
        $7211 = HEAP32[$pOp>>2]|0;
        $7212 = (_out2Prerelease($7210,$7211)|0);
        HEAP32[$pOut>>2] = $7212;
        $7213 = HEAP32[$pOp>>2]|0;
        $7214 = ((($7213)) + 4|0);
        $7215 = HEAP32[$7214>>2]|0;
        $7216 = HEAP32[$db>>2]|0;
        $7217 = ((($7216)) + 16|0);
        $7218 = HEAP32[$7217>>2]|0;
        $7219 = (($7218) + ($7215<<4)|0);
        $7220 = ((($7219)) + 4|0);
        $7221 = HEAP32[$7220>>2]|0;
        $7222 = (_sqlite3BtreeLastPage($7221)|0);
        $7223 = HEAP32[$pOut>>2]|0;
        $7224 = $7223;
        $7225 = $7224;
        HEAP32[$7225>>2] = $7222;
        $7226 = (($7224) + 4)|0;
        $7227 = $7226;
        HEAP32[$7227>>2] = 0;
        break;
       }
       case 159:  {
        $7228 = HEAP32[$0>>2]|0;
        $7229 = HEAP32[$pOp>>2]|0;
        $7230 = (_out2Prerelease($7228,$7229)|0);
        HEAP32[$pOut>>2] = $7230;
        $7231 = HEAP32[$pOp>>2]|0;
        $7232 = ((($7231)) + 4|0);
        $7233 = HEAP32[$7232>>2]|0;
        $7234 = HEAP32[$db>>2]|0;
        $7235 = ((($7234)) + 16|0);
        $7236 = HEAP32[$7235>>2]|0;
        $7237 = (($7236) + ($7233<<4)|0);
        $7238 = ((($7237)) + 4|0);
        $7239 = HEAP32[$7238>>2]|0;
        HEAP32[$pBt128>>2] = $7239;
        HEAP32[$newMax>>2] = 0;
        $7240 = HEAP32[$pOp>>2]|0;
        $7241 = ((($7240)) + 12|0);
        $7242 = HEAP32[$7241>>2]|0;
        $7243 = ($7242|0)!=(0);
        if ($7243) {
         $7244 = HEAP32[$pBt128>>2]|0;
         $7245 = (_sqlite3BtreeLastPage($7244)|0);
         HEAP32[$newMax>>2] = $7245;
         $7246 = HEAP32[$newMax>>2]|0;
         $7247 = HEAP32[$pOp>>2]|0;
         $7248 = ((($7247)) + 12|0);
         $7249 = HEAP32[$7248>>2]|0;
         $7250 = ($7246>>>0)<($7249>>>0);
         if ($7250) {
          $7251 = HEAP32[$pOp>>2]|0;
          $7252 = ((($7251)) + 12|0);
          $7253 = HEAP32[$7252>>2]|0;
          HEAP32[$newMax>>2] = $7253;
         }
        }
        $7254 = HEAP32[$pBt128>>2]|0;
        $7255 = HEAP32[$newMax>>2]|0;
        $7256 = (_sqlite3BtreeMaxPageCount($7254,$7255)|0);
        $7257 = ($7256|0)<(0);
        $7258 = $7257 << 31 >> 31;
        $7259 = HEAP32[$pOut>>2]|0;
        $7260 = $7259;
        $7261 = $7260;
        HEAP32[$7261>>2] = $7256;
        $7262 = (($7260) + 4)|0;
        $7263 = $7262;
        HEAP32[$7263>>2] = $7258;
        break;
       }
       case 71:  {
        $7264 = HEAP32[$db>>2]|0;
        $7265 = ((($7264)) + 76|0);
        $7266 = HEAP8[$7265>>0]|0;
        $7267 = $7266&255;
        $7268 = $7267 & 129;
        $7269 = ($7268|0)!=(0);
        do {
         if ($7269) {
          $7270 = HEAP32[$0>>2]|0;
          $7271 = ((($7270)) + 144|0);
          $7272 = HEAP16[$7271>>1]|0;
          $7273 = ($7272&65535) >>> 1;
          $7274 = $7273 & 1;
          $7275 = $7274&65535;
          $7276 = ($7275|0)!=(0);
          if (!($7276)) {
           $7277 = HEAP32[$pOp>>2]|0;
           $7278 = ((($7277)) + 16|0);
           $7279 = HEAP32[$7278>>2]|0;
           $7280 = ($7279|0)!=(0|0);
           if ($7280) {
            $7281 = HEAP32[$pOp>>2]|0;
            $7282 = ((($7281)) + 16|0);
            $7283 = HEAP32[$7282>>2]|0;
            $7287 = $7283;
           } else {
            $7284 = HEAP32[$0>>2]|0;
            $7285 = ((($7284)) + 176|0);
            $7286 = HEAP32[$7285>>2]|0;
            $7287 = $7286;
           }
           HEAP32[$zTrace>>2] = $7287;
           $7288 = ($7287|0)!=(0|0);
           if ($7288) {
            $7289 = HEAP32[$db>>2]|0;
            $7290 = ((($7289)) + 76|0);
            $7291 = HEAP8[$7290>>0]|0;
            $7292 = $7291&255;
            $7293 = $7292 & 128;
            $7294 = ($7293|0)!=(0);
            $7295 = HEAP32[$db>>2]|0;
            $7296 = ((($7295)) + 184|0);
            $7297 = HEAP32[$7296>>2]|0;
            if ($7294) {
             HEAP32[$x130>>2] = $7297;
             $7298 = HEAP32[$0>>2]|0;
             $7299 = HEAP32[$zTrace>>2]|0;
             $7300 = (_sqlite3VdbeExpandSql($7298,$7299)|0);
             HEAP32[$z131>>2] = $7300;
             $7301 = HEAP32[$x130>>2]|0;
             $7302 = HEAP32[$db>>2]|0;
             $7303 = ((($7302)) + 188|0);
             $7304 = HEAP32[$7303>>2]|0;
             $7305 = HEAP32[$z131>>2]|0;
             FUNCTION_TABLE_vii[$7301 & 255]($7304,$7305);
             $7306 = HEAP32[$z131>>2]|0;
             _sqlite3_free($7306);
             break;
            } else {
             $7307 = HEAP32[$db>>2]|0;
             $7308 = ((($7307)) + 188|0);
             $7309 = HEAP32[$7308>>2]|0;
             $7310 = HEAP32[$0>>2]|0;
             $7311 = HEAP32[$zTrace>>2]|0;
             (FUNCTION_TABLE_iiiii[$7297 & 255](1,$7309,$7310,$7311)|0);
             break;
            }
           }
          }
         }
        } while(0);
        $7312 = HEAP32[$pOp>>2]|0;
        $7313 = ((($7312)) + 4|0);
        $7314 = HEAP32[$7313>>2]|0;
        $7315 = HEAP32[(280)>>2]|0;
        $7316 = ($7314|0)>=($7315|0);
        if ($7316) {
         HEAP32[$i129>>2] = 1;
         while(1) {
          $7317 = HEAP32[$i129>>2]|0;
          $7318 = HEAP32[$0>>2]|0;
          $7319 = ((($7318)) + 136|0);
          $7320 = HEAP32[$7319>>2]|0;
          $7321 = ($7317|0)<($7320|0);
          if (!($7321)) {
           break;
          }
          $7322 = HEAP32[$i129>>2]|0;
          $7323 = HEAP32[$0>>2]|0;
          $7324 = ((($7323)) + 88|0);
          $7325 = HEAP32[$7324>>2]|0;
          $7326 = (($7325) + (($7322*20)|0)|0);
          $7327 = HEAP8[$7326>>0]|0;
          $7328 = $7327&255;
          $7329 = ($7328|0)==(20);
          if ($7329) {
           $7330 = HEAP32[$i129>>2]|0;
           $7331 = HEAP32[$0>>2]|0;
           $7332 = ((($7331)) + 88|0);
           $7333 = HEAP32[$7332>>2]|0;
           $7334 = (($7333) + (($7330*20)|0)|0);
           $7335 = ((($7334)) + 4|0);
           HEAP32[$7335>>2] = 0;
          }
          $7336 = HEAP32[$i129>>2]|0;
          $7337 = (($7336) + 1)|0;
          HEAP32[$i129>>2] = $7337;
         }
         $7338 = HEAP32[$pOp>>2]|0;
         $7339 = ((($7338)) + 4|0);
         HEAP32[$7339>>2] = 0;
        }
        $7340 = HEAP32[$pOp>>2]|0;
        $7341 = ((($7340)) + 4|0);
        $7342 = HEAP32[$7341>>2]|0;
        $7343 = (($7342) + 1)|0;
        HEAP32[$7341>>2] = $7343;
        label = 14;
        break;
       }
       default: {
       }
       }
      } while(0);
      do {
       if ((label|0) == 20) {
        label = 0;
        $237 = HEAP32[$pOp>>2]|0;
        $238 = HEAP32[$aOp>>2]|0;
        $239 = $237;
        $240 = $238;
        $241 = (($239) - ($240))|0;
        $242 = (($241|0) / 20)&-1;
        HEAP32[$pcx>>2] = $242;
        $243 = HEAP32[$pOp>>2]|0;
        $244 = ((($243)) + 4|0);
        $245 = HEAP32[$244>>2]|0;
        $246 = ($245|0)==(0);
        if (!($246)) {
         label = 25;
         break L8;
        }
        $247 = HEAP32[$0>>2]|0;
        $248 = ((($247)) + 184|0);
        $249 = HEAP32[$248>>2]|0;
        $250 = ($249|0)!=(0|0);
        if (!($250)) {
         label = 25;
         break L8;
        }
        $251 = HEAP32[$0>>2]|0;
        $252 = ((($251)) + 184|0);
        $253 = HEAP32[$252>>2]|0;
        HEAP32[$pFrame>>2] = $253;
        $254 = HEAP32[$pFrame>>2]|0;
        $255 = ((($254)) + 4|0);
        $256 = HEAP32[$255>>2]|0;
        $257 = HEAP32[$0>>2]|0;
        $258 = ((($257)) + 184|0);
        HEAP32[$258>>2] = $256;
        $259 = HEAP32[$0>>2]|0;
        $260 = ((($259)) + 192|0);
        $261 = HEAP32[$260>>2]|0;
        $262 = (($261) + -1)|0;
        HEAP32[$260>>2] = $262;
        $263 = HEAP32[$db>>2]|0;
        $264 = HEAP32[$0>>2]|0;
        $265 = ((($264)) + 44|0);
        $266 = HEAP32[$265>>2]|0;
        _sqlite3VdbeSetChanges($263,$266);
        $267 = HEAP32[$pFrame>>2]|0;
        $268 = (_sqlite3VdbeFrameRestore($267)|0);
        HEAP32[$pcx>>2] = $268;
        $269 = HEAP32[$db>>2]|0;
        $270 = ((($269)) + 32|0);
        $271 = $270;
        $272 = $271;
        $273 = HEAP32[$272>>2]|0;
        $274 = (($271) + 4)|0;
        $275 = $274;
        $276 = HEAP32[$275>>2]|0;
        $277 = $lastRowid;
        $278 = $277;
        HEAP32[$278>>2] = $273;
        $279 = (($277) + 4)|0;
        $280 = $279;
        HEAP32[$280>>2] = $276;
        $281 = HEAP32[$pOp>>2]|0;
        $282 = ((($281)) + 8|0);
        $283 = HEAP32[$282>>2]|0;
        $284 = ($283|0)==(4);
        if ($284) {
         $285 = HEAP32[$pcx>>2]|0;
         $286 = HEAP32[$0>>2]|0;
         $287 = ((($286)) + 88|0);
         $288 = HEAP32[$287>>2]|0;
         $289 = (($288) + (($285*20)|0)|0);
         $290 = ((($289)) + 8|0);
         $291 = HEAP32[$290>>2]|0;
         $292 = (($291) - 1)|0;
         HEAP32[$pcx>>2] = $292;
        }
        $293 = HEAP32[$0>>2]|0;
        $294 = ((($293)) + 88|0);
        $295 = HEAP32[$294>>2]|0;
        HEAP32[$aOp>>2] = $295;
        $296 = HEAP32[$0>>2]|0;
        $297 = ((($296)) + 92|0);
        $298 = HEAP32[$297>>2]|0;
        HEAP32[$aMem>>2] = $298;
        $299 = HEAP32[$pcx>>2]|0;
        $300 = HEAP32[$aOp>>2]|0;
        $301 = (($300) + (($299*20)|0)|0);
        HEAP32[$pOp>>2] = $301;
       }
       else if ((label|0) == 43) {
        label = 0;
        $466 = HEAP32[$0>>2]|0;
        $467 = HEAP32[$pOp>>2]|0;
        $468 = (_out2Prerelease($466,$467)|0);
        HEAP32[$pOut>>2] = $468;
        $469 = HEAP32[$pOut>>2]|0;
        $470 = ((($469)) + 8|0);
        HEAP16[$470>>1] = 2562;
        $471 = HEAP32[$pOp>>2]|0;
        $472 = ((($471)) + 16|0);
        $473 = HEAP32[$472>>2]|0;
        $474 = HEAP32[$pOut>>2]|0;
        $475 = ((($474)) + 16|0);
        HEAP32[$475>>2] = $473;
        $476 = HEAP32[$pOp>>2]|0;
        $477 = ((($476)) + 4|0);
        $478 = HEAP32[$477>>2]|0;
        $479 = HEAP32[$pOut>>2]|0;
        $480 = ((($479)) + 12|0);
        HEAP32[$480>>2] = $478;
        $481 = HEAP8[$encoding>>0]|0;
        $482 = HEAP32[$pOut>>2]|0;
        $483 = ((($482)) + 10|0);
        HEAP8[$483>>0] = $481;
        $484 = HEAP32[$pOp>>2]|0;
        $485 = ((($484)) + 12|0);
        $486 = HEAP32[$485>>2]|0;
        $487 = ($486|0)>(0);
        if ($487) {
         $488 = HEAP32[$pOp>>2]|0;
         $489 = ((($488)) + 12|0);
         $490 = HEAP32[$489>>2]|0;
         $491 = HEAP32[$aMem>>2]|0;
         $492 = (($491) + (($490*40)|0)|0);
         HEAP32[$pIn3>>2] = $492;
         $493 = HEAP32[$pIn3>>2]|0;
         $494 = $493;
         $495 = $494;
         $496 = HEAP32[$495>>2]|0;
         $497 = (($494) + 4)|0;
         $498 = $497;
         $499 = HEAP32[$498>>2]|0;
         $500 = HEAP32[$pOp>>2]|0;
         $501 = ((($500)) + 3|0);
         $502 = HEAP8[$501>>0]|0;
         $503 = $502&255;
         $504 = ($496|0)==($503|0);
         $505 = ($499|0)==(0);
         $506 = $504 & $505;
         if ($506) {
          $507 = HEAP32[$pOut>>2]|0;
          $508 = ((($507)) + 8|0);
          HEAP16[$508>>1] = 2576;
         }
        }
       }
       else if ((label|0) == 126) {
        label = 0;
        $1281 = HEAP32[$pOp>>2]|0;
        $1282 = ((($1281)) + 16|0);
        $1283 = HEAP32[$1282>>2]|0;
        HEAP32[$pCtx4>>2] = $1283;
        $1284 = HEAP32[$pOp>>2]|0;
        $1285 = ((($1284)) + 12|0);
        $1286 = HEAP32[$1285>>2]|0;
        $1287 = HEAP32[$aMem>>2]|0;
        $1288 = (($1287) + (($1286*40)|0)|0);
        HEAP32[$pOut>>2] = $1288;
        $1289 = HEAP32[$pCtx4>>2]|0;
        $1290 = HEAP32[$1289>>2]|0;
        $1291 = HEAP32[$pOut>>2]|0;
        $1292 = ($1290|0)!=($1291|0);
        L981: do {
         if ($1292) {
          $1293 = HEAP32[$pOut>>2]|0;
          $1294 = HEAP32[$pCtx4>>2]|0;
          HEAP32[$1294>>2] = $1293;
          $1295 = HEAP32[$pCtx4>>2]|0;
          $1296 = ((($1295)) + 26|0);
          $1297 = HEAP8[$1296>>0]|0;
          $1298 = $1297&255;
          $1299 = (($1298) - 1)|0;
          HEAP32[$i3>>2] = $1299;
          while(1) {
           $1300 = HEAP32[$i3>>2]|0;
           $1301 = ($1300|0)>=(0);
           if (!($1301)) {
            break L981;
           }
           $1302 = HEAP32[$pOp>>2]|0;
           $1303 = ((($1302)) + 8|0);
           $1304 = HEAP32[$1303>>2]|0;
           $1305 = HEAP32[$i3>>2]|0;
           $1306 = (($1304) + ($1305))|0;
           $1307 = HEAP32[$aMem>>2]|0;
           $1308 = (($1307) + (($1306*40)|0)|0);
           $1309 = HEAP32[$i3>>2]|0;
           $1310 = HEAP32[$pCtx4>>2]|0;
           $1311 = ((($1310)) + 28|0);
           $1312 = (($1311) + ($1309<<2)|0);
           HEAP32[$1312>>2] = $1308;
           $1313 = HEAP32[$i3>>2]|0;
           $1314 = (($1313) + -1)|0;
           HEAP32[$i3>>2] = $1314;
          }
         }
        } while(0);
        $1315 = HEAP32[$pCtx4>>2]|0;
        $1316 = HEAP32[$1315>>2]|0;
        $1317 = ((($1316)) + 8|0);
        $1318 = HEAP16[$1317>>1]|0;
        $1319 = $1318&65535;
        $1320 = $1319 & -49664;
        $1321 = $1320 | 1;
        $1322 = $1321&65535;
        $1323 = HEAP32[$pCtx4>>2]|0;
        $1324 = HEAP32[$1323>>2]|0;
        $1325 = ((($1324)) + 8|0);
        HEAP16[$1325>>1] = $1322;
        $1326 = HEAP32[$pCtx4>>2]|0;
        $1327 = ((($1326)) + 25|0);
        HEAP8[$1327>>0] = 0;
        $1328 = $lastRowid;
        $1329 = $1328;
        $1330 = HEAP32[$1329>>2]|0;
        $1331 = (($1328) + 4)|0;
        $1332 = $1331;
        $1333 = HEAP32[$1332>>2]|0;
        $1334 = HEAP32[$db>>2]|0;
        $1335 = ((($1334)) + 32|0);
        $1336 = $1335;
        $1337 = $1336;
        HEAP32[$1337>>2] = $1330;
        $1338 = (($1336) + 4)|0;
        $1339 = $1338;
        HEAP32[$1339>>2] = $1333;
        $1340 = HEAP32[$pCtx4>>2]|0;
        $1341 = ((($1340)) + 4|0);
        $1342 = HEAP32[$1341>>2]|0;
        $1343 = ((($1342)) + 12|0);
        $1344 = HEAP32[$1343>>2]|0;
        $1345 = HEAP32[$pCtx4>>2]|0;
        $1346 = HEAP32[$pCtx4>>2]|0;
        $1347 = ((($1346)) + 26|0);
        $1348 = HEAP8[$1347>>0]|0;
        $1349 = $1348&255;
        $1350 = HEAP32[$pCtx4>>2]|0;
        $1351 = ((($1350)) + 28|0);
        FUNCTION_TABLE_viii[$1344 & 255]($1345,$1349,$1351);
        $1352 = HEAP32[$db>>2]|0;
        $1353 = ((($1352)) + 32|0);
        $1354 = $1353;
        $1355 = $1354;
        $1356 = HEAP32[$1355>>2]|0;
        $1357 = (($1354) + 4)|0;
        $1358 = $1357;
        $1359 = HEAP32[$1358>>2]|0;
        $1360 = $lastRowid;
        $1361 = $1360;
        HEAP32[$1361>>2] = $1356;
        $1362 = (($1360) + 4)|0;
        $1363 = $1362;
        HEAP32[$1363>>2] = $1359;
        $1364 = HEAP32[$pCtx4>>2]|0;
        $1365 = ((($1364)) + 25|0);
        $1366 = HEAP8[$1365>>0]|0;
        $1367 = ($1366<<24>>24)!=(0);
        if ($1367) {
         $1368 = HEAP32[$pCtx4>>2]|0;
         $1369 = ((($1368)) + 20|0);
         $1370 = HEAP32[$1369>>2]|0;
         $1371 = ($1370|0)!=(0);
         if ($1371) {
          $1372 = HEAP32[$0>>2]|0;
          $1373 = HEAP32[$pCtx4>>2]|0;
          $1374 = HEAP32[$1373>>2]|0;
          $1375 = (_sqlite3_value_text($1374)|0);
          HEAP32[$vararg_buffer44>>2] = $1375;
          _sqlite3VdbeError($1372,18391,$vararg_buffer44);
          $1376 = HEAP32[$pCtx4>>2]|0;
          $1377 = ((($1376)) + 20|0);
          $1378 = HEAP32[$1377>>2]|0;
          HEAP32[$rc>>2] = $1378;
         }
         $1379 = HEAP32[$db>>2]|0;
         $1380 = HEAP32[$0>>2]|0;
         $1381 = ((($1380)) + 204|0);
         $1382 = HEAP32[$pCtx4>>2]|0;
         $1383 = ((($1382)) + 16|0);
         $1384 = HEAP32[$1383>>2]|0;
         $1385 = HEAP32[$pOp>>2]|0;
         $1386 = ((($1385)) + 4|0);
         $1387 = HEAP32[$1386>>2]|0;
         _sqlite3VdbeDeleteAuxData($1379,$1381,$1384,$1387);
         $1388 = HEAP32[$rc>>2]|0;
         $1389 = ($1388|0)!=(0);
         if ($1389) {
          break L1;
         }
        }
        $1390 = HEAP32[$pOut>>2]|0;
        $1391 = ((($1390)) + 8|0);
        $1392 = HEAP16[$1391>>1]|0;
        $1393 = $1392&65535;
        $1394 = $1393 & 18;
        $1395 = ($1394|0)!=(0);
        if ($1395) {
         $1396 = HEAP32[$pCtx4>>2]|0;
         $1397 = HEAP32[$1396>>2]|0;
         $1398 = HEAP8[$encoding>>0]|0;
         $1399 = $1398&255;
         (_sqlite3VdbeChangeEncoding($1397,$1399)|0);
         $1400 = HEAP32[$pCtx4>>2]|0;
         $1401 = HEAP32[$1400>>2]|0;
         $1402 = (_sqlite3VdbeMemTooBig($1401)|0);
         $1403 = ($1402|0)!=(0);
         if ($1403) {
          label = 888;
          break L8;
         }
        }
       }
       else if ((label|0) == 444) {
        label = 0;
        $3668 = HEAP32[$0>>2]|0;
        $3669 = ((($3668)) + 144|0);
        $3670 = HEAP16[$3669>>1]|0;
        $3671 = $3670 & 1;
        $3672 = $3671&65535;
        $3673 = ($3672|0)!=(0);
        if ($3673) {
         label = 445;
         break L8;
        }
        HEAP32[$nField20>>2] = 0;
        HEAP32[$pKeyInfo21>>2] = 0;
        $3674 = HEAP32[$pOp>>2]|0;
        $3675 = ((($3674)) + 8|0);
        $3676 = HEAP32[$3675>>2]|0;
        HEAP32[$p222>>2] = $3676;
        $3677 = HEAP32[$pOp>>2]|0;
        $3678 = ((($3677)) + 12|0);
        $3679 = HEAP32[$3678>>2]|0;
        HEAP32[$iDb23>>2] = $3679;
        $3680 = HEAP32[$iDb23>>2]|0;
        $3681 = HEAP32[$db>>2]|0;
        $3682 = ((($3681)) + 16|0);
        $3683 = HEAP32[$3682>>2]|0;
        $3684 = (($3683) + ($3680<<4)|0);
        HEAP32[$pDb24>>2] = $3684;
        $3685 = HEAP32[$pDb24>>2]|0;
        $3686 = ((($3685)) + 4|0);
        $3687 = HEAP32[$3686>>2]|0;
        HEAP32[$pX>>2] = $3687;
        $3688 = HEAP32[$pOp>>2]|0;
        $3689 = HEAP8[$3688>>0]|0;
        $3690 = $3689&255;
        $3691 = ($3690|0)==(105);
        if ($3691) {
         $3692 = HEAP32[$pOp>>2]|0;
         $3693 = ((($3692)) + 3|0);
         $3694 = HEAP8[$3693>>0]|0;
         $3695 = $3694&255;
         $3696 = $3695 & 8;
         $3697 = 4 | $3696;
         HEAP32[$wrFlag>>2] = $3697;
         $3698 = HEAP32[$pDb24>>2]|0;
         $3699 = ((($3698)) + 12|0);
         $3700 = HEAP32[$3699>>2]|0;
         $3701 = ((($3700)) + 76|0);
         $3702 = HEAP8[$3701>>0]|0;
         $3703 = $3702&255;
         $3704 = HEAP32[$0>>2]|0;
         $3705 = ((($3704)) + 143|0);
         $3706 = HEAP8[$3705>>0]|0;
         $3707 = $3706&255;
         $3708 = ($3703|0)<($3707|0);
         if ($3708) {
          $3709 = HEAP32[$pDb24>>2]|0;
          $3710 = ((($3709)) + 12|0);
          $3711 = HEAP32[$3710>>2]|0;
          $3712 = ((($3711)) + 76|0);
          $3713 = HEAP8[$3712>>0]|0;
          $3714 = HEAP32[$0>>2]|0;
          $3715 = ((($3714)) + 143|0);
          HEAP8[$3715>>0] = $3713;
         }
        } else {
         HEAP32[$wrFlag>>2] = 0;
        }
        $3716 = HEAP32[$pOp>>2]|0;
        $3717 = ((($3716)) + 3|0);
        $3718 = HEAP8[$3717>>0]|0;
        $3719 = $3718&255;
        $3720 = $3719 & 16;
        $3721 = ($3720|0)!=(0);
        if ($3721) {
         $3722 = HEAP32[$p222>>2]|0;
         $3723 = HEAP32[$aMem>>2]|0;
         $3724 = (($3723) + (($3722*40)|0)|0);
         HEAP32[$pIn2>>2] = $3724;
         $3725 = HEAP32[$pIn2>>2]|0;
         (_sqlite3VdbeMemIntegerify($3725)|0);
         $3726 = HEAP32[$pIn2>>2]|0;
         $3727 = $3726;
         $3728 = $3727;
         $3729 = HEAP32[$3728>>2]|0;
         $3730 = (($3727) + 4)|0;
         $3731 = $3730;
         $3732 = HEAP32[$3731>>2]|0;
         HEAP32[$p222>>2] = $3729;
        }
        $3733 = HEAP32[$pOp>>2]|0;
        $3734 = ((($3733)) + 1|0);
        $3735 = HEAP8[$3734>>0]|0;
        $3736 = $3735 << 24 >> 24;
        $3737 = ($3736|0)==(-6);
        $3738 = HEAP32[$pOp>>2]|0;
        if ($3737) {
         $3739 = ((($3738)) + 16|0);
         $3740 = HEAP32[$3739>>2]|0;
         HEAP32[$pKeyInfo21>>2] = $3740;
         $3741 = HEAP32[$pKeyInfo21>>2]|0;
         $3742 = ((($3741)) + 6|0);
         $3743 = HEAP16[$3742>>1]|0;
         $3744 = $3743&65535;
         $3745 = HEAP32[$pKeyInfo21>>2]|0;
         $3746 = ((($3745)) + 8|0);
         $3747 = HEAP16[$3746>>1]|0;
         $3748 = $3747&65535;
         $3749 = (($3744) + ($3748))|0;
         HEAP32[$nField20>>2] = $3749;
        } else {
         $3750 = ((($3738)) + 1|0);
         $3751 = HEAP8[$3750>>0]|0;
         $3752 = $3751 << 24 >> 24;
         $3753 = ($3752|0)==(-14);
         if ($3753) {
          $3754 = HEAP32[$pOp>>2]|0;
          $3755 = ((($3754)) + 16|0);
          $3756 = HEAP32[$3755>>2]|0;
          HEAP32[$nField20>>2] = $3756;
         }
        }
        $3757 = HEAP32[$0>>2]|0;
        $3758 = HEAP32[$pOp>>2]|0;
        $3759 = ((($3758)) + 4|0);
        $3760 = HEAP32[$3759>>2]|0;
        $3761 = HEAP32[$nField20>>2]|0;
        $3762 = HEAP32[$iDb23>>2]|0;
        $3763 = (_allocateCursor($3757,$3760,$3761,$3762,0)|0);
        HEAP32[$pCur>>2] = $3763;
        $3764 = HEAP32[$pCur>>2]|0;
        $3765 = ($3764|0)==(0|0);
        if ($3765) {
         label = 889;
         break L1;
        }
        $3766 = HEAP32[$pCur>>2]|0;
        $3767 = ((($3766)) + 2|0);
        HEAP8[$3767>>0] = 1;
        $3768 = HEAP32[$pCur>>2]|0;
        $3769 = ((($3768)) + 5|0);
        $3770 = HEAP8[$3769>>0]|0;
        $3771 = $3770 & -5;
        $3772 = $3771 | 4;
        HEAP8[$3769>>0] = $3772;
        $3773 = HEAP32[$p222>>2]|0;
        $3774 = HEAP32[$pCur>>2]|0;
        $3775 = ((($3774)) + 8|0);
        HEAP32[$3775>>2] = $3773;
        $3776 = HEAP32[$pX>>2]|0;
        $3777 = HEAP32[$p222>>2]|0;
        $3778 = HEAP32[$wrFlag>>2]|0;
        $3779 = HEAP32[$pKeyInfo21>>2]|0;
        $3780 = HEAP32[$pCur>>2]|0;
        $3781 = ((($3780)) + 16|0);
        $3782 = HEAP32[$3781>>2]|0;
        $3783 = (_sqlite3BtreeCursor($3776,$3777,$3778,$3779,$3782)|0);
        HEAP32[$rc>>2] = $3783;
        $3784 = HEAP32[$pKeyInfo21>>2]|0;
        $3785 = HEAP32[$pCur>>2]|0;
        $3786 = ((($3785)) + 24|0);
        HEAP32[$3786>>2] = $3784;
        $3787 = HEAP32[$pOp>>2]|0;
        $3788 = ((($3787)) + 1|0);
        $3789 = HEAP8[$3788>>0]|0;
        $3790 = $3789 << 24 >> 24;
        $3791 = ($3790|0)!=(-6);
        $3792 = $3791&1;
        $3793 = $3792&255;
        $3794 = HEAP32[$pCur>>2]|0;
        $3795 = ((($3794)) + 4|0);
        HEAP8[$3795>>0] = $3793;
        label = 458;
       }
       else if ((label|0) == 523) {
        label = 0;
        $4281 = HEAP32[$pOp>>2]|0;
        $4282 = ((($4281)) + 12|0);
        $4283 = HEAP32[$4282>>2]|0;
        $4284 = HEAP32[$aMem>>2]|0;
        $4285 = (($4284) + (($4283*40)|0)|0);
        HEAP32[$pIn3>>2] = $4285;
        $4286 = HEAP32[$pOp>>2]|0;
        $4287 = ((($4286)) + 4|0);
        $4288 = HEAP32[$4287>>2]|0;
        $4289 = HEAP32[$0>>2]|0;
        $4290 = ((($4289)) + 112|0);
        $4291 = HEAP32[$4290>>2]|0;
        $4292 = (($4291) + ($4288<<2)|0);
        $4293 = HEAP32[$4292>>2]|0;
        HEAP32[$pC36>>2] = $4293;
        $4294 = HEAP32[$pC36>>2]|0;
        $4295 = ((($4294)) + 16|0);
        $4296 = HEAP32[$4295>>2]|0;
        HEAP32[$pCrsr37>>2] = $4296;
        HEAP32[$res38>>2] = 0;
        $4297 = HEAP32[$pIn3>>2]|0;
        $4298 = $4297;
        $4299 = $4298;
        $4300 = HEAP32[$4299>>2]|0;
        $4301 = (($4298) + 4)|0;
        $4302 = $4301;
        $4303 = HEAP32[$4302>>2]|0;
        $4304 = $iKey39;
        $4305 = $4304;
        HEAP32[$4305>>2] = $4300;
        $4306 = (($4304) + 4)|0;
        $4307 = $4306;
        HEAP32[$4307>>2] = $4303;
        $4308 = HEAP32[$pCrsr37>>2]|0;
        $4309 = $iKey39;
        $4310 = $4309;
        $4311 = HEAP32[$4310>>2]|0;
        $4312 = (($4309) + 4)|0;
        $4313 = $4312;
        $4314 = HEAP32[$4313>>2]|0;
        $4315 = (_sqlite3BtreeMovetoUnpacked($4308,0,$4311,$4314,0,$res38)|0);
        HEAP32[$rc>>2] = $4315;
        $4316 = $iKey39;
        $4317 = $4316;
        $4318 = HEAP32[$4317>>2]|0;
        $4319 = (($4316) + 4)|0;
        $4320 = $4319;
        $4321 = HEAP32[$4320>>2]|0;
        $4322 = HEAP32[$pC36>>2]|0;
        $4323 = ((($4322)) + 40|0);
        $4324 = $4323;
        $4325 = $4324;
        HEAP32[$4325>>2] = $4318;
        $4326 = (($4324) + 4)|0;
        $4327 = $4326;
        HEAP32[$4327>>2] = $4321;
        $4328 = HEAP32[$pC36>>2]|0;
        $4329 = ((($4328)) + 2|0);
        HEAP8[$4329>>0] = 0;
        $4330 = HEAP32[$pC36>>2]|0;
        $4331 = ((($4330)) + 56|0);
        HEAP32[$4331>>2] = 0;
        $4332 = HEAP32[$pC36>>2]|0;
        $4333 = ((($4332)) + 3|0);
        HEAP8[$4333>>0] = 0;
        $4334 = HEAP32[$res38>>2]|0;
        $4335 = HEAP32[$pC36>>2]|0;
        $4336 = ((($4335)) + 28|0);
        HEAP32[$4336>>2] = $4334;
        $4337 = HEAP32[$res38>>2]|0;
        $4338 = ($4337|0)!=(0);
        if ($4338) {
         $4339 = HEAP32[$pOp>>2]|0;
         $4340 = ((($4339)) + 8|0);
         $4341 = HEAP32[$4340>>2]|0;
         $4342 = ($4341|0)==(0);
         if (!($4342)) {
          label = 14;
          break;
         }
         $4343 = (_sqlite3CorruptError(81458)|0);
         HEAP32[$rc>>2] = $4343;
        }
        $4344 = HEAP32[$rc>>2]|0;
        $4345 = ($4344|0)!=(0);
        if ($4345) {
         break L1;
        }
       }
       else if ((label|0) == 617) {
        label = 0;
        $5136 = HEAP32[$pOp>>2]|0;
        $5137 = ((($5136)) + 4|0);
        $5138 = HEAP32[$5137>>2]|0;
        $5139 = HEAP32[$0>>2]|0;
        $5140 = ((($5139)) + 112|0);
        $5141 = HEAP32[$5140>>2]|0;
        $5142 = (($5141) + ($5138<<2)|0);
        $5143 = HEAP32[$5142>>2]|0;
        HEAP32[$pC62>>2] = $5143;
        HEAP32[$res64>>2] = 1;
        $5144 = HEAP32[$pC62>>2]|0;
        $5145 = HEAP8[$5144>>0]|0;
        $5146 = $5145&255;
        $5147 = ($5146|0)==(1);
        $5148 = HEAP32[$pC62>>2]|0;
        if ($5147) {
         $5149 = (_sqlite3VdbeSorterRewind($5148,$res64)|0);
         HEAP32[$rc>>2] = $5149;
        } else {
         $5150 = ((($5148)) + 16|0);
         $5151 = HEAP32[$5150>>2]|0;
         HEAP32[$pCrsr63>>2] = $5151;
         $5152 = HEAP32[$pCrsr63>>2]|0;
         $5153 = (_sqlite3BtreeFirst($5152,$res64)|0);
         HEAP32[$rc>>2] = $5153;
         $5154 = HEAP32[$pC62>>2]|0;
         $5155 = ((($5154)) + 3|0);
         HEAP8[$5155>>0] = 0;
         $5156 = HEAP32[$pC62>>2]|0;
         $5157 = ((($5156)) + 56|0);
         HEAP32[$5157>>2] = 0;
        }
        $5158 = HEAP32[$rc>>2]|0;
        $5159 = ($5158|0)!=(0);
        if ($5159) {
         break L1;
        }
        $5160 = HEAP32[$res64>>2]|0;
        $5161 = $5160&255;
        $5162 = HEAP32[$pC62>>2]|0;
        $5163 = ((($5162)) + 2|0);
        HEAP8[$5163>>0] = $5161;
        $5164 = HEAP32[$res64>>2]|0;
        $5165 = ($5164|0)!=(0);
        if ($5165) {
         label = 14;
        }
       }
       else if ((label|0) == 624) {
        label = 0;
        $5186 = HEAP32[$pOp>>2]|0;
        $5187 = ((($5186)) + 4|0);
        $5188 = HEAP32[$5187>>2]|0;
        $5189 = HEAP32[$0>>2]|0;
        $5190 = ((($5189)) + 112|0);
        $5191 = HEAP32[$5190>>2]|0;
        $5192 = (($5191) + ($5188<<2)|0);
        $5193 = HEAP32[$5192>>2]|0;
        HEAP32[$pC65>>2] = $5193;
        $5194 = HEAP32[$pOp>>2]|0;
        $5195 = ((($5194)) + 12|0);
        $5196 = HEAP32[$5195>>2]|0;
        HEAP32[$res66>>2] = $5196;
        $5197 = HEAP32[$pOp>>2]|0;
        $5198 = ((($5197)) + 16|0);
        $5199 = HEAP32[$5198>>2]|0;
        $5200 = HEAP32[$pC65>>2]|0;
        $5201 = ((($5200)) + 16|0);
        $5202 = HEAP32[$5201>>2]|0;
        $5203 = (FUNCTION_TABLE_iii[$5199 & 255]($5202,$res66)|0);
        HEAP32[$rc>>2] = $5203;
        label = 625;
       }
       else if ((label|0) == 752) {
        label = 0;
        $6489 = HEAP32[$pOp>>2]|0;
        $6490 = ((($6489)) + 16|0);
        $6491 = HEAP32[$6490>>2]|0;
        HEAP32[$pCtx92>>2] = $6491;
        $6492 = HEAP32[$pOp>>2]|0;
        $6493 = ((($6492)) + 12|0);
        $6494 = HEAP32[$6493>>2]|0;
        $6495 = HEAP32[$aMem>>2]|0;
        $6496 = (($6495) + (($6494*40)|0)|0);
        HEAP32[$pMem93>>2] = $6496;
        $6497 = HEAP32[$pCtx92>>2]|0;
        $6498 = ((($6497)) + 8|0);
        $6499 = HEAP32[$6498>>2]|0;
        $6500 = HEAP32[$pMem93>>2]|0;
        $6501 = ($6499|0)!=($6500|0);
        L1023: do {
         if ($6501) {
          $6502 = HEAP32[$pMem93>>2]|0;
          $6503 = HEAP32[$pCtx92>>2]|0;
          $6504 = ((($6503)) + 8|0);
          HEAP32[$6504>>2] = $6502;
          $6505 = HEAP32[$pCtx92>>2]|0;
          $6506 = ((($6505)) + 26|0);
          $6507 = HEAP8[$6506>>0]|0;
          $6508 = $6507&255;
          $6509 = (($6508) - 1)|0;
          HEAP32[$i91>>2] = $6509;
          while(1) {
           $6510 = HEAP32[$i91>>2]|0;
           $6511 = ($6510|0)>=(0);
           if (!($6511)) {
            break L1023;
           }
           $6512 = HEAP32[$pOp>>2]|0;
           $6513 = ((($6512)) + 8|0);
           $6514 = HEAP32[$6513>>2]|0;
           $6515 = HEAP32[$i91>>2]|0;
           $6516 = (($6514) + ($6515))|0;
           $6517 = HEAP32[$aMem>>2]|0;
           $6518 = (($6517) + (($6516*40)|0)|0);
           $6519 = HEAP32[$i91>>2]|0;
           $6520 = HEAP32[$pCtx92>>2]|0;
           $6521 = ((($6520)) + 28|0);
           $6522 = (($6521) + ($6519<<2)|0);
           HEAP32[$6522>>2] = $6518;
           $6523 = HEAP32[$i91>>2]|0;
           $6524 = (($6523) + -1)|0;
           HEAP32[$i91>>2] = $6524;
          }
         }
        } while(0);
        $6525 = HEAP32[$pMem93>>2]|0;
        $6526 = ((($6525)) + 12|0);
        $6527 = HEAP32[$6526>>2]|0;
        $6528 = (($6527) + 1)|0;
        HEAP32[$6526>>2] = $6528;
        $6529 = HEAP32[$db>>2]|0;
        _sqlite3VdbeMemInit($t94,$6529,1);
        $6530 = HEAP32[$pCtx92>>2]|0;
        HEAP32[$6530>>2] = $t94;
        $6531 = HEAP32[$pCtx92>>2]|0;
        $6532 = ((($6531)) + 25|0);
        HEAP8[$6532>>0] = 0;
        $6533 = HEAP32[$pCtx92>>2]|0;
        $6534 = ((($6533)) + 24|0);
        HEAP8[$6534>>0] = 0;
        $6535 = HEAP32[$pCtx92>>2]|0;
        $6536 = ((($6535)) + 4|0);
        $6537 = HEAP32[$6536>>2]|0;
        $6538 = ((($6537)) + 12|0);
        $6539 = HEAP32[$6538>>2]|0;
        $6540 = HEAP32[$pCtx92>>2]|0;
        $6541 = HEAP32[$pCtx92>>2]|0;
        $6542 = ((($6541)) + 26|0);
        $6543 = HEAP8[$6542>>0]|0;
        $6544 = $6543&255;
        $6545 = HEAP32[$pCtx92>>2]|0;
        $6546 = ((($6545)) + 28|0);
        FUNCTION_TABLE_viii[$6539 & 255]($6540,$6544,$6546);
        $6547 = HEAP32[$pCtx92>>2]|0;
        $6548 = ((($6547)) + 25|0);
        $6549 = HEAP8[$6548>>0]|0;
        $6550 = ($6549<<24>>24)!=(0);
        if ($6550) {
         $6551 = HEAP32[$pCtx92>>2]|0;
         $6552 = ((($6551)) + 20|0);
         $6553 = HEAP32[$6552>>2]|0;
         $6554 = ($6553|0)!=(0);
         if ($6554) {
          $6555 = HEAP32[$0>>2]|0;
          $6556 = (_sqlite3_value_text($t94)|0);
          HEAP32[$vararg_buffer65>>2] = $6556;
          _sqlite3VdbeError($6555,18391,$vararg_buffer65);
          $6557 = HEAP32[$pCtx92>>2]|0;
          $6558 = ((($6557)) + 20|0);
          $6559 = HEAP32[$6558>>2]|0;
          HEAP32[$rc>>2] = $6559;
         }
         _sqlite3VdbeMemRelease($t94);
         $6560 = HEAP32[$rc>>2]|0;
         $6561 = ($6560|0)!=(0);
         if ($6561) {
          break L1;
         }
        }
        $6562 = HEAP32[$pCtx92>>2]|0;
        $6563 = ((($6562)) + 24|0);
        $6564 = HEAP8[$6563>>0]|0;
        $6565 = ($6564<<24>>24)!=(0);
        if ($6565) {
         $6566 = HEAP32[$pOp>>2]|0;
         $6567 = ((($6566)) + -20|0);
         $6568 = ((($6567)) + 4|0);
         $6569 = HEAP32[$6568>>2]|0;
         HEAP32[$i91>>2] = $6569;
         $6570 = HEAP32[$i91>>2]|0;
         $6571 = ($6570|0)!=(0);
         if ($6571) {
          $6572 = HEAP32[$i91>>2]|0;
          $6573 = HEAP32[$aMem>>2]|0;
          $6574 = (($6573) + (($6572*40)|0)|0);
          _sqlite3VdbeMemSetInt64($6574,1,0);
         }
        }
       }
      } while(0);
      do {
       if ((label|0) == 14) {
        label = 0;
        $130 = HEAP32[$pOp>>2]|0;
        $131 = ((($130)) + 8|0);
        $132 = HEAP32[$131>>2]|0;
        $133 = (($132) - 1)|0;
        $134 = HEAP32[$aOp>>2]|0;
        $135 = (($134) + (($133*20)|0)|0);
        HEAP32[$pOp>>2] = $135;
       }
       else if ((label|0) == 458) {
        label = 0;
        $3796 = HEAP32[$pCur>>2]|0;
        $3797 = ((($3796)) + 16|0);
        $3798 = HEAP32[$3797>>2]|0;
        $3799 = HEAP32[$pOp>>2]|0;
        $3800 = ((($3799)) + 3|0);
        $3801 = HEAP8[$3800>>0]|0;
        $3802 = $3801&255;
        $3803 = $3802 & 3;
        _sqlite3BtreeCursorHintFlags($3798,$3803);
        $3804 = HEAP32[$rc>>2]|0;
        $3805 = ($3804|0)!=(0);
        if ($3805) {
         break L1;
        }
       }
       else if ((label|0) == 625) {
        label = 0;
        $5204 = HEAP32[$pC65>>2]|0;
        $5205 = ((($5204)) + 56|0);
        HEAP32[$5205>>2] = 0;
        $5206 = HEAP32[$rc>>2]|0;
        $5207 = ($5206|0)!=(0);
        if ($5207) {
         break L1;
        }
        $5208 = HEAP32[$res66>>2]|0;
        $5209 = ($5208|0)==(0);
        $5210 = HEAP32[$pC65>>2]|0;
        $5211 = ((($5210)) + 2|0);
        if ($5209) {
         HEAP8[$5211>>0] = 0;
         $5212 = HEAP32[$pOp>>2]|0;
         $5213 = ((($5212)) + 3|0);
         $5214 = HEAP8[$5213>>0]|0;
         $5215 = $5214&255;
         $5216 = HEAP32[$0>>2]|0;
         $5217 = ((($5216)) + 156|0);
         $5218 = (($5217) + ($5215<<2)|0);
         $5219 = HEAP32[$5218>>2]|0;
         $5220 = (($5219) + 1)|0;
         HEAP32[$5218>>2] = $5220;
         label = 7;
         break;
        } else {
         HEAP8[$5211>>0] = 1;
         label = 8;
         break;
        }
       }
      } while(0);
      if ((label|0) == 7) {
       label = 0;
       $74 = HEAP32[$pOp>>2]|0;
       $75 = ((($74)) + 8|0);
       $76 = HEAP32[$75>>2]|0;
       $77 = (($76) - 1)|0;
       $78 = HEAP32[$aOp>>2]|0;
       $79 = (($78) + (($77*20)|0)|0);
       HEAP32[$pOp>>2] = $79;
       label = 8;
      }
      if ((label|0) == 8) {
       label = 0;
       $80 = HEAP32[$db>>2]|0;
       $81 = ((($80)) + 248|0);
       $82 = HEAP32[$81>>2]|0;
       $83 = ($82|0)!=(0);
       if ($83) {
        break L3;
       }
       $84 = HEAP32[$db>>2]|0;
       $85 = ((($84)) + 304|0);
       $86 = HEAP32[$85>>2]|0;
       $87 = ($86|0)!=(0|0);
       if ($87) {
        $88 = HEAP32[$nVmStep>>2]|0;
        $89 = HEAP32[$nProgressLimit>>2]|0;
        $90 = ($88>>>0)>=($89>>>0);
        if ($90) {
         $91 = HEAP32[$nVmStep>>2]|0;
         $92 = HEAP32[$db>>2]|0;
         $93 = ((($92)) + 312|0);
         $94 = HEAP32[$93>>2]|0;
         $95 = (($91) + ($94))|0;
         $96 = HEAP32[$nVmStep>>2]|0;
         $97 = HEAP32[$db>>2]|0;
         $98 = ((($97)) + 312|0);
         $99 = HEAP32[$98>>2]|0;
         $100 = (($96>>>0) % ($99>>>0))&-1;
         $101 = (($95) - ($100))|0;
         HEAP32[$nProgressLimit>>2] = $101;
         $102 = HEAP32[$db>>2]|0;
         $103 = ((($102)) + 304|0);
         $104 = HEAP32[$103>>2]|0;
         $105 = HEAP32[$db>>2]|0;
         $106 = ((($105)) + 308|0);
         $107 = HEAP32[$106>>2]|0;
         $108 = (FUNCTION_TABLE_ii[$104 & 255]($107)|0);
         $109 = ($108|0)!=(0);
         if ($109) {
          label = 12;
          break;
         }
        }
       }
      }
      $7344 = HEAP32[$pOp>>2]|0;
      $7345 = ((($7344)) + 20|0);
      HEAP32[$pOp>>2] = $7345;
     }
     switch (label|0) {
      case 12: {
       HEAP32[$rc>>2] = 9;
       break L1;
       break;
      }
      case 25: {
       $302 = HEAP32[$pOp>>2]|0;
       $303 = ((($302)) + 4|0);
       $304 = HEAP32[$303>>2]|0;
       $305 = HEAP32[$0>>2]|0;
       $306 = ((($305)) + 40|0);
       HEAP32[$306>>2] = $304;
       $307 = HEAP32[$pOp>>2]|0;
       $308 = ((($307)) + 8|0);
       $309 = HEAP32[$308>>2]|0;
       $310 = $309&255;
       $311 = HEAP32[$0>>2]|0;
       $312 = ((($311)) + 142|0);
       HEAP8[$312>>0] = $310;
       $313 = HEAP32[$pcx>>2]|0;
       $314 = HEAP32[$0>>2]|0;
       $315 = ((($314)) + 36|0);
       HEAP32[$315>>2] = $313;
       $316 = HEAP32[$0>>2]|0;
       $317 = ((($316)) + 40|0);
       $318 = HEAP32[$317>>2]|0;
       $319 = ($318|0)!=(0);
       if ($319) {
        $320 = HEAP32[$pOp>>2]|0;
        $321 = ((($320)) + 3|0);
        $322 = HEAP8[$321>>0]|0;
        $323 = ($322<<24>>24)!=(0);
        $324 = HEAP32[$0>>2]|0;
        $325 = HEAP32[$pOp>>2]|0;
        if ($323) {
         $326 = ((($325)) + 3|0);
         $327 = HEAP8[$326>>0]|0;
         $328 = $327&255;
         $329 = (($328) - 1)|0;
         $330 = (5572 + ($329<<2)|0);
         $331 = HEAP32[$330>>2]|0;
         HEAP32[$vararg_buffer>>2] = $331;
         _sqlite3VdbeError($324,39873,$vararg_buffer);
         $332 = HEAP32[$pOp>>2]|0;
         $333 = ((($332)) + 16|0);
         $334 = HEAP32[$333>>2]|0;
         $335 = ($334|0)!=(0|0);
         if ($335) {
          $336 = HEAP32[$db>>2]|0;
          $337 = HEAP32[$0>>2]|0;
          $338 = ((($337)) + 108|0);
          $339 = HEAP32[$338>>2]|0;
          $340 = HEAP32[$pOp>>2]|0;
          $341 = ((($340)) + 16|0);
          $342 = HEAP32[$341>>2]|0;
          HEAP32[$vararg_buffer32>>2] = $339;
          $vararg_ptr35 = ((($vararg_buffer32)) + 4|0);
          HEAP32[$vararg_ptr35>>2] = $342;
          $343 = (_sqlite3MPrintf($336,39894,$vararg_buffer32)|0);
          $344 = HEAP32[$0>>2]|0;
          $345 = ((($344)) + 108|0);
          HEAP32[$345>>2] = $343;
         }
        } else {
         $346 = ((($325)) + 16|0);
         $347 = HEAP32[$346>>2]|0;
         HEAP32[$vararg_buffer36>>2] = $347;
         _sqlite3VdbeError($324,18391,$vararg_buffer36);
        }
        $348 = HEAP32[$pOp>>2]|0;
        $349 = ((($348)) + 4|0);
        $350 = HEAP32[$349>>2]|0;
        $351 = HEAP32[$pcx>>2]|0;
        $352 = HEAP32[$0>>2]|0;
        $353 = ((($352)) + 176|0);
        $354 = HEAP32[$353>>2]|0;
        $355 = HEAP32[$0>>2]|0;
        $356 = ((($355)) + 108|0);
        $357 = HEAP32[$356>>2]|0;
        HEAP32[$vararg_buffer39>>2] = $351;
        $vararg_ptr42 = ((($vararg_buffer39)) + 4|0);
        HEAP32[$vararg_ptr42>>2] = $354;
        $vararg_ptr43 = ((($vararg_buffer39)) + 8|0);
        HEAP32[$vararg_ptr43>>2] = $357;
        _sqlite3_log($350,39901,$vararg_buffer39);
       }
       $358 = HEAP32[$0>>2]|0;
       $359 = (_sqlite3VdbeHalt($358)|0);
       HEAP32[$rc>>2] = $359;
       $360 = HEAP32[$rc>>2]|0;
       $361 = ($360|0)==(5);
       $362 = HEAP32[$0>>2]|0;
       $363 = ((($362)) + 40|0);
       if ($361) {
        HEAP32[$363>>2] = 5;
        $7388 = $lastRowid;
        $7389 = $7388;
        $7390 = HEAP32[$7389>>2]|0;
        $7391 = (($7388) + 4)|0;
        $7392 = $7391;
        $7393 = HEAP32[$7392>>2]|0;
        $7394 = HEAP32[$db>>2]|0;
        $7395 = ((($7394)) + 32|0);
        $7396 = $7395;
        $7397 = $7396;
        HEAP32[$7397>>2] = $7390;
        $7398 = (($7396) + 4)|0;
        $7399 = $7398;
        HEAP32[$7399>>2] = $7393;
        $7400 = HEAP32[$nVmStep>>2]|0;
        $7401 = HEAP32[$0>>2]|0;
        $7402 = ((($7401)) + 156|0);
        $7403 = ((($7402)) + 16|0);
        $7404 = HEAP32[$7403>>2]|0;
        $7405 = (($7404) + ($7400))|0;
        HEAP32[$7403>>2] = $7405;
        $7406 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7406|0);
       } else {
        $364 = HEAP32[$363>>2]|0;
        $365 = ($364|0)!=(0);
        $366 = $365 ? 1 : 101;
        HEAP32[$rc>>2] = $366;
        $7388 = $lastRowid;
        $7389 = $7388;
        $7390 = HEAP32[$7389>>2]|0;
        $7391 = (($7388) + 4)|0;
        $7392 = $7391;
        $7393 = HEAP32[$7392>>2]|0;
        $7394 = HEAP32[$db>>2]|0;
        $7395 = ((($7394)) + 32|0);
        $7396 = $7395;
        $7397 = $7396;
        HEAP32[$7397>>2] = $7390;
        $7398 = (($7396) + 4)|0;
        $7399 = $7398;
        HEAP32[$7399>>2] = $7393;
        $7400 = HEAP32[$nVmStep>>2]|0;
        $7401 = HEAP32[$0>>2]|0;
        $7402 = ((($7401)) + 156|0);
        $7403 = ((($7402)) + 16|0);
        $7404 = HEAP32[$7403>>2]|0;
        $7405 = (($7404) + ($7400))|0;
        HEAP32[$7403>>2] = $7405;
        $7406 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7406|0);
       }
       break;
      }
      case 64: {
       $678 = HEAP32[$db>>2]|0;
       $679 = ((($678)) + 304|0);
       $680 = HEAP32[$679>>2]|0;
       $681 = ($680|0)!=(0|0);
       if ($681) {
        $682 = HEAP32[$nVmStep>>2]|0;
        $683 = HEAP32[$nProgressLimit>>2]|0;
        $684 = ($682>>>0)>=($683>>>0);
        if ($684) {
         $685 = HEAP32[$db>>2]|0;
         $686 = ((($685)) + 304|0);
         $687 = HEAP32[$686>>2]|0;
         $688 = HEAP32[$db>>2]|0;
         $689 = ((($688)) + 308|0);
         $690 = HEAP32[$689>>2]|0;
         $691 = (FUNCTION_TABLE_ii[$687 & 255]($690)|0);
         $692 = ($691|0)!=(0);
         if ($692) {
          HEAP32[$rc>>2] = 9;
          break L1;
         }
        }
       }
       $693 = HEAP32[$0>>2]|0;
       $694 = (_sqlite3VdbeCheckFk($693,0)|0);
       HEAP32[$rc>>2] = $694;
       $695 = (0)!=($694|0);
       if ($695) {
        break L1;
       }
       $696 = HEAP32[$0>>2]|0;
       $697 = (_sqlite3VdbeCloseStatement($696,1)|0);
       HEAP32[$rc>>2] = $697;
       $698 = HEAP32[$0>>2]|0;
       $699 = ((($698)) + 32|0);
       $700 = HEAP32[$699>>2]|0;
       $701 = (($700) + 2)|0;
       $702 = $701 | 1;
       $703 = HEAP32[$0>>2]|0;
       $704 = ((($703)) + 32|0);
       HEAP32[$704>>2] = $702;
       $705 = HEAP32[$pOp>>2]|0;
       $706 = ((($705)) + 4|0);
       $707 = HEAP32[$706>>2]|0;
       $708 = HEAP32[$aMem>>2]|0;
       $709 = (($708) + (($707*40)|0)|0);
       $710 = HEAP32[$0>>2]|0;
       $711 = ((($710)) + 104|0);
       HEAP32[$711>>2] = $709;
       HEAP32[$pMem>>2] = $709;
       HEAP32[$i>>2] = 0;
       while(1) {
        $712 = HEAP32[$i>>2]|0;
        $713 = HEAP32[$pOp>>2]|0;
        $714 = ((($713)) + 8|0);
        $715 = HEAP32[$714>>2]|0;
        $716 = ($712|0)<($715|0);
        if (!($716)) {
         break;
        }
        $717 = HEAP32[$i>>2]|0;
        $718 = HEAP32[$pMem>>2]|0;
        $719 = (($718) + (($717*40)|0)|0);
        $720 = ((($719)) + 8|0);
        $721 = HEAP16[$720>>1]|0;
        $722 = $721&65535;
        $723 = $722 & 4096;
        $724 = ($723|0)!=(0);
        if ($724) {
         $725 = HEAP32[$i>>2]|0;
         $726 = HEAP32[$pMem>>2]|0;
         $727 = (($726) + (($725*40)|0)|0);
         $728 = (_sqlite3VdbeMemMakeWriteable($727)|0);
         $729 = ($728|0)!=(0);
         if ($729) {
          label = 889;
          break L1;
         }
        }
        $730 = HEAP32[$i>>2]|0;
        $731 = HEAP32[$pMem>>2]|0;
        $732 = (($731) + (($730*40)|0)|0);
        (_sqlite3VdbeMemNulTerminate($732)|0);
        $733 = HEAP32[$i>>2]|0;
        $734 = (($733) + 1)|0;
        HEAP32[$i>>2] = $734;
       }
       $735 = HEAP32[$db>>2]|0;
       $736 = ((($735)) + 69|0);
       $737 = HEAP8[$736>>0]|0;
       $738 = ($737<<24>>24)!=(0);
       if ($738) {
        label = 889;
        break L1;
       }
       $739 = HEAP32[$db>>2]|0;
       $740 = ((($739)) + 76|0);
       $741 = HEAP8[$740>>0]|0;
       $742 = $741&255;
       $743 = $742 & 4;
       $744 = ($743|0)!=(0);
       if ($744) {
        $745 = HEAP32[$db>>2]|0;
        $746 = ((($745)) + 184|0);
        $747 = HEAP32[$746>>2]|0;
        $748 = HEAP32[$db>>2]|0;
        $749 = ((($748)) + 188|0);
        $750 = HEAP32[$749>>2]|0;
        $751 = HEAP32[$0>>2]|0;
        (FUNCTION_TABLE_iiiii[$747 & 255](4,$750,$751,0)|0);
       }
       $752 = HEAP32[$pOp>>2]|0;
       $753 = HEAP32[$aOp>>2]|0;
       $754 = $752;
       $755 = $753;
       $756 = (($754) - ($755))|0;
       $757 = (($756|0) / 20)&-1;
       $758 = (($757) + 1)|0;
       $759 = HEAP32[$0>>2]|0;
       $760 = ((($759)) + 36|0);
       HEAP32[$760>>2] = $758;
       HEAP32[$rc>>2] = 100;
       $7388 = $lastRowid;
       $7389 = $7388;
       $7390 = HEAP32[$7389>>2]|0;
       $7391 = (($7388) + 4)|0;
       $7392 = $7391;
       $7393 = HEAP32[$7392>>2]|0;
       $7394 = HEAP32[$db>>2]|0;
       $7395 = ((($7394)) + 32|0);
       $7396 = $7395;
       $7397 = $7396;
       HEAP32[$7397>>2] = $7390;
       $7398 = (($7396) + 4)|0;
       $7399 = $7398;
       HEAP32[$7399>>2] = $7393;
       $7400 = HEAP32[$nVmStep>>2]|0;
       $7401 = HEAP32[$0>>2]|0;
       $7402 = ((($7401)) + 156|0);
       $7403 = ((($7402)) + 16|0);
       $7404 = HEAP32[$7403>>2]|0;
       $7405 = (($7404) + ($7400))|0;
       HEAP32[$7403>>2] = $7405;
       $7406 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7406|0);
       break;
      }
      case 159: {
       HEAP32[$rc>>2] = 20;
       break L1;
       break;
      }
      case 267: {
       $2411 = (_sqlite3CorruptError(79835)|0);
       HEAP32[$rc>>2] = $2411;
       break L1;
       break;
      }
      case 286: {
       $2564 = HEAP32[$pC>>2]|0;
       $2565 = ((($2564)) + 72|0);
       $2566 = HEAP32[$2565>>2]|0;
       $2567 = ($2566|0)==(0|0);
       if ($2567) {
        _sqlite3VdbeMemRelease($sMem);
       }
       $2568 = (_sqlite3CorruptError(79894)|0);
       HEAP32[$rc>>2] = $2568;
       break L1;
       break;
      }
      case 370: {
       $3182 = HEAP32[$pOp>>2]|0;
       $3183 = HEAP32[$aOp>>2]|0;
       $3184 = $3182;
       $3185 = $3183;
       $3186 = (($3184) - ($3185))|0;
       $3187 = (($3186|0) / 20)&-1;
       $3188 = HEAP32[$0>>2]|0;
       $3189 = ((($3188)) + 36|0);
       HEAP32[$3189>>2] = $3187;
       $3190 = HEAP32[$db>>2]|0;
       $3191 = ((($3190)) + 67|0);
       HEAP8[$3191>>0] = 0;
       HEAP32[$rc>>2] = 5;
       $3192 = HEAP32[$0>>2]|0;
       $3193 = ((($3192)) + 40|0);
       HEAP32[$3193>>2] = 5;
       $7388 = $lastRowid;
       $7389 = $7388;
       $7390 = HEAP32[$7389>>2]|0;
       $7391 = (($7388) + 4)|0;
       $7392 = $7391;
       $7393 = HEAP32[$7392>>2]|0;
       $7394 = HEAP32[$db>>2]|0;
       $7395 = ((($7394)) + 32|0);
       $7396 = $7395;
       $7397 = $7396;
       HEAP32[$7397>>2] = $7390;
       $7398 = (($7396) + 4)|0;
       $7399 = $7398;
       HEAP32[$7399>>2] = $7393;
       $7400 = HEAP32[$nVmStep>>2]|0;
       $7401 = HEAP32[$0>>2]|0;
       $7402 = ((($7401)) + 156|0);
       $7403 = ((($7402)) + 16|0);
       $7404 = HEAP32[$7403>>2]|0;
       $7405 = (($7404) + ($7400))|0;
       HEAP32[$7403>>2] = $7405;
       $7406 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7406|0);
       break;
      }
      case 393: {
       $3335 = HEAP32[$pOp>>2]|0;
       $3336 = ((($3335)) + 4|0);
       $3337 = HEAP32[$3336>>2]|0;
       HEAP32[$desiredAutoCommit>>2] = $3337;
       $3338 = HEAP32[$pOp>>2]|0;
       $3339 = ((($3338)) + 8|0);
       $3340 = HEAP32[$3339>>2]|0;
       HEAP32[$iRollback>>2] = $3340;
       $3341 = HEAP32[$desiredAutoCommit>>2]|0;
       $3342 = HEAP32[$db>>2]|0;
       $3343 = ((($3342)) + 67|0);
       $3344 = HEAP8[$3343>>0]|0;
       $3345 = $3344&255;
       $3346 = ($3341|0)!=($3345|0);
       if (!($3346)) {
        $3389 = HEAP32[$0>>2]|0;
        $3390 = HEAP32[$desiredAutoCommit>>2]|0;
        $3391 = ($3390|0)!=(0);
        if ($3391) {
         $3392 = HEAP32[$iRollback>>2]|0;
         $3393 = ($3392|0)!=(0);
         $3394 = $3393 ? 40133 : 40176;
         $3395 = $3394;
        } else {
         $3395 = 40217;
        }
        _sqlite3VdbeError($3389,$3395,$vararg_buffer56);
        HEAP32[$rc>>2] = 1;
        break L1;
       }
       $3347 = HEAP32[$iRollback>>2]|0;
       $3348 = ($3347|0)!=(0);
       do {
        if ($3348) {
         $3349 = HEAP32[$db>>2]|0;
         _sqlite3RollbackAll($3349,516);
         $3350 = HEAP32[$db>>2]|0;
         $3351 = ((($3350)) + 67|0);
         HEAP8[$3351>>0] = 1;
        } else {
         $3352 = HEAP32[$desiredAutoCommit>>2]|0;
         $3353 = ($3352|0)!=(0);
         if ($3353) {
          $3354 = HEAP32[$db>>2]|0;
          $3355 = ((($3354)) + 164|0);
          $3356 = HEAP32[$3355>>2]|0;
          $3357 = ($3356|0)>(0);
          if ($3357) {
           $3358 = HEAP32[$0>>2]|0;
           _sqlite3VdbeError($3358,40078,$vararg_buffer54);
           HEAP32[$rc>>2] = 5;
           break L1;
          }
         }
         $3359 = HEAP32[$0>>2]|0;
         $3360 = (_sqlite3VdbeCheckFk($3359,1)|0);
         HEAP32[$rc>>2] = $3360;
         $3361 = ($3360|0)!=(0);
         if ($3361) {
          $7388 = $lastRowid;
          $7389 = $7388;
          $7390 = HEAP32[$7389>>2]|0;
          $7391 = (($7388) + 4)|0;
          $7392 = $7391;
          $7393 = HEAP32[$7392>>2]|0;
          $7394 = HEAP32[$db>>2]|0;
          $7395 = ((($7394)) + 32|0);
          $7396 = $7395;
          $7397 = $7396;
          HEAP32[$7397>>2] = $7390;
          $7398 = (($7396) + 4)|0;
          $7399 = $7398;
          HEAP32[$7399>>2] = $7393;
          $7400 = HEAP32[$nVmStep>>2]|0;
          $7401 = HEAP32[$0>>2]|0;
          $7402 = ((($7401)) + 156|0);
          $7403 = ((($7402)) + 16|0);
          $7404 = HEAP32[$7403>>2]|0;
          $7405 = (($7404) + ($7400))|0;
          HEAP32[$7403>>2] = $7405;
          $7406 = HEAP32[$rc>>2]|0;
          STACKTOP = sp;return ($7406|0);
         } else {
          $3362 = HEAP32[$desiredAutoCommit>>2]|0;
          $3363 = $3362&255;
          $3364 = HEAP32[$db>>2]|0;
          $3365 = ((($3364)) + 67|0);
          HEAP8[$3365>>0] = $3363;
          break;
         }
        }
       } while(0);
       $3366 = HEAP32[$0>>2]|0;
       $3367 = (_sqlite3VdbeHalt($3366)|0);
       $3368 = ($3367|0)==(5);
       if ($3368) {
        $3369 = HEAP32[$pOp>>2]|0;
        $3370 = HEAP32[$aOp>>2]|0;
        $3371 = $3369;
        $3372 = $3370;
        $3373 = (($3371) - ($3372))|0;
        $3374 = (($3373|0) / 20)&-1;
        $3375 = HEAP32[$0>>2]|0;
        $3376 = ((($3375)) + 36|0);
        HEAP32[$3376>>2] = $3374;
        $3377 = HEAP32[$desiredAutoCommit>>2]|0;
        $3378 = (1 - ($3377))|0;
        $3379 = $3378&255;
        $3380 = HEAP32[$db>>2]|0;
        $3381 = ((($3380)) + 67|0);
        HEAP8[$3381>>0] = $3379;
        HEAP32[$rc>>2] = 5;
        $3382 = HEAP32[$0>>2]|0;
        $3383 = ((($3382)) + 40|0);
        HEAP32[$3383>>2] = 5;
        $7388 = $lastRowid;
        $7389 = $7388;
        $7390 = HEAP32[$7389>>2]|0;
        $7391 = (($7388) + 4)|0;
        $7392 = $7391;
        $7393 = HEAP32[$7392>>2]|0;
        $7394 = HEAP32[$db>>2]|0;
        $7395 = ((($7394)) + 32|0);
        $7396 = $7395;
        $7397 = $7396;
        HEAP32[$7397>>2] = $7390;
        $7398 = (($7396) + 4)|0;
        $7399 = $7398;
        HEAP32[$7399>>2] = $7393;
        $7400 = HEAP32[$nVmStep>>2]|0;
        $7401 = HEAP32[$0>>2]|0;
        $7402 = ((($7401)) + 156|0);
        $7403 = ((($7402)) + 16|0);
        $7404 = HEAP32[$7403>>2]|0;
        $7405 = (($7404) + ($7400))|0;
        HEAP32[$7403>>2] = $7405;
        $7406 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7406|0);
       }
       $3384 = HEAP32[$db>>2]|0;
       _sqlite3CloseSavepoints($3384);
       $3385 = HEAP32[$0>>2]|0;
       $3386 = ((($3385)) + 40|0);
       $3387 = HEAP32[$3386>>2]|0;
       $3388 = ($3387|0)==(0);
       if ($3388) {
        HEAP32[$rc>>2] = 101;
        $7388 = $lastRowid;
        $7389 = $7388;
        $7390 = HEAP32[$7389>>2]|0;
        $7391 = (($7388) + 4)|0;
        $7392 = $7391;
        $7393 = HEAP32[$7392>>2]|0;
        $7394 = HEAP32[$db>>2]|0;
        $7395 = ((($7394)) + 32|0);
        $7396 = $7395;
        $7397 = $7396;
        HEAP32[$7397>>2] = $7390;
        $7398 = (($7396) + 4)|0;
        $7399 = $7398;
        HEAP32[$7399>>2] = $7393;
        $7400 = HEAP32[$nVmStep>>2]|0;
        $7401 = HEAP32[$0>>2]|0;
        $7402 = ((($7401)) + 156|0);
        $7403 = ((($7402)) + 16|0);
        $7404 = HEAP32[$7403>>2]|0;
        $7405 = (($7404) + ($7400))|0;
        HEAP32[$7403>>2] = $7405;
        $7406 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7406|0);
       } else {
        HEAP32[$rc>>2] = 1;
        $7388 = $lastRowid;
        $7389 = $7388;
        $7390 = HEAP32[$7389>>2]|0;
        $7391 = (($7388) + 4)|0;
        $7392 = $7391;
        $7393 = HEAP32[$7392>>2]|0;
        $7394 = HEAP32[$db>>2]|0;
        $7395 = ((($7394)) + 32|0);
        $7396 = $7395;
        $7397 = $7396;
        HEAP32[$7397>>2] = $7390;
        $7398 = (($7396) + 4)|0;
        $7399 = $7398;
        HEAP32[$7399>>2] = $7393;
        $7400 = HEAP32[$nVmStep>>2]|0;
        $7401 = HEAP32[$0>>2]|0;
        $7402 = ((($7401)) + 156|0);
        $7403 = ((($7402)) + 16|0);
        $7404 = HEAP32[$7403>>2]|0;
        $7405 = (($7404) + ($7400))|0;
        HEAP32[$7403>>2] = $7405;
        $7406 = HEAP32[$rc>>2]|0;
        STACKTOP = sp;return ($7406|0);
       }
       break;
      }
      case 411: {
       HEAP32[$rc>>2] = 8;
       break L1;
       break;
      }
      case 414: {
       $3423 = HEAP32[$rc>>2]|0;
       $3424 = $3423 & 255;
       $3425 = ($3424|0)==(5);
       if (!($3425)) {
        break L1;
       }
       $3426 = HEAP32[$pOp>>2]|0;
       $3427 = HEAP32[$aOp>>2]|0;
       $3428 = $3426;
       $3429 = $3427;
       $3430 = (($3428) - ($3429))|0;
       $3431 = (($3430|0) / 20)&-1;
       $3432 = HEAP32[$0>>2]|0;
       $3433 = ((($3432)) + 36|0);
       HEAP32[$3433>>2] = $3431;
       $3434 = HEAP32[$rc>>2]|0;
       $3435 = HEAP32[$0>>2]|0;
       $3436 = ((($3435)) + 40|0);
       HEAP32[$3436>>2] = $3434;
       $7388 = $lastRowid;
       $7389 = $7388;
       $7390 = HEAP32[$7389>>2]|0;
       $7391 = (($7388) + 4)|0;
       $7392 = $7391;
       $7393 = HEAP32[$7392>>2]|0;
       $7394 = HEAP32[$db>>2]|0;
       $7395 = ((($7394)) + 32|0);
       $7396 = $7395;
       $7397 = $7396;
       HEAP32[$7397>>2] = $7390;
       $7398 = (($7396) + 4)|0;
       $7399 = $7398;
       HEAP32[$7399>>2] = $7393;
       $7400 = HEAP32[$nVmStep>>2]|0;
       $7401 = HEAP32[$0>>2]|0;
       $7402 = ((($7401)) + 156|0);
       $7403 = ((($7402)) + 16|0);
       $7404 = HEAP32[$7403>>2]|0;
       $7405 = (($7404) + ($7400))|0;
       HEAP32[$7403>>2] = $7405;
       $7406 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7406|0);
       break;
      }
      case 445: {
       HEAP32[$rc>>2] = 516;
       break L1;
       break;
      }
      case 544: {
       HEAP32[$rc>>2] = 13;
       break L1;
       break;
      }
      case 554: {
       HEAP32[$rc>>2] = 13;
       break L1;
       break;
      }
      case 660: {
       HEAP32[$rc>>2] = 6;
       $5473 = HEAP32[$0>>2]|0;
       $5474 = ((($5473)) + 142|0);
       HEAP8[$5474>>0] = 2;
       break L1;
       break;
      }
      case 682: {
       $5661 = HEAP32[$db>>2]|0;
       _sqlite3ResetAllSchemasOfConnection($5661);
       $5662 = HEAP32[$rc>>2]|0;
       $5663 = ($5662|0)==(7);
       if ($5663) {
        label = 889;
        break L1;
       } else {
        break L1;
       }
       break;
      }
      case 711: {
       HEAP32[$rc>>2] = 1;
       $5903 = HEAP32[$0>>2]|0;
       _sqlite3VdbeError($5903,40357,$vararg_buffer63);
       break L1;
       break;
      }
      case 764: {
       $6587 = HEAP32[$0>>2]|0;
       $6588 = HEAP32[$pMem95>>2]|0;
       $6589 = (_sqlite3_value_text($6588)|0);
       HEAP32[$vararg_buffer68>>2] = $6589;
       _sqlite3VdbeError($6587,18391,$vararg_buffer68);
       break L1;
       break;
      }
      case 784: {
       HEAP32[$rc>>2] = 1;
       $6681 = HEAP32[$0>>2]|0;
       $6682 = HEAP32[$eNew>>2]|0;
       $6683 = ($6682|0)==(5);
       $6684 = $6683 ? 40394 : 40399;
       HEAP32[$vararg_buffer71>>2] = $6684;
       _sqlite3VdbeError($6681,40406,$vararg_buffer71);
       break L1;
       break;
      }
      case 805: {
       $6791 = HEAP32[$rc>>2]|0;
       $6792 = $6791 & 255;
       $6793 = ($6792|0)==(6);
       if (!($6793)) {
        break L1;
       }
       $6794 = HEAP32[$pOp>>2]|0;
       $6795 = ((($6794)) + 16|0);
       $6796 = HEAP32[$6795>>2]|0;
       HEAP32[$z101>>2] = $6796;
       $6797 = HEAP32[$0>>2]|0;
       $6798 = HEAP32[$z101>>2]|0;
       HEAP32[$vararg_buffer74>>2] = $6798;
       _sqlite3VdbeError($6797,40458,$vararg_buffer74);
       break L1;
       break;
      }
      case 816: {
       HEAP32[$rc>>2] = 6;
       break L1;
       break;
      }
      case 820: {
       $6889 = HEAP32[$pModule105>>2]|0;
       $6890 = ((($6889)) + 28|0);
       $6891 = HEAP32[$6890>>2]|0;
       $6892 = HEAP32[$pVCur>>2]|0;
       (FUNCTION_TABLE_ii[$6891 & 255]($6892)|0);
       label = 889;
       break L1;
       break;
      }
      case 839: {
       HEAP32[$rc>>2] = 6;
       break L1;
       break;
      }
      case 887: {
       $7388 = $lastRowid;
       $7389 = $7388;
       $7390 = HEAP32[$7389>>2]|0;
       $7391 = (($7388) + 4)|0;
       $7392 = $7391;
       $7393 = HEAP32[$7392>>2]|0;
       $7394 = HEAP32[$db>>2]|0;
       $7395 = ((($7394)) + 32|0);
       $7396 = $7395;
       $7397 = $7396;
       HEAP32[$7397>>2] = $7390;
       $7398 = (($7396) + 4)|0;
       $7399 = $7398;
       HEAP32[$7399>>2] = $7393;
       $7400 = HEAP32[$nVmStep>>2]|0;
       $7401 = HEAP32[$0>>2]|0;
       $7402 = ((($7401)) + 156|0);
       $7403 = ((($7402)) + 16|0);
       $7404 = HEAP32[$7403>>2]|0;
       $7405 = (($7404) + ($7400))|0;
       HEAP32[$7403>>2] = $7405;
       $7406 = HEAP32[$rc>>2]|0;
       STACKTOP = sp;return ($7406|0);
       break;
      }
      case 888: {
       $7407 = HEAP32[$0>>2]|0;
       _sqlite3VdbeError($7407,18170,$vararg_buffer85);
       HEAP32[$rc>>2] = 18;
       break L1;
       break;
      }
     }
    }
   } while(0);
   $7410 = HEAP32[$db>>2]|0;
   $7411 = ((($7410)) + 69|0);
   $7412 = HEAP8[$7411>>0]|0;
   $7413 = $7412&255;
   $7414 = ($7413|0)!=(0);
   $7415 = $7414 ? 7 : 9;
   HEAP32[$rc>>2] = $7415;
   $7416 = HEAP32[$rc>>2]|0;
   $7417 = HEAP32[$0>>2]|0;
   $7418 = ((($7417)) + 40|0);
   HEAP32[$7418>>2] = $7416;
   $7419 = HEAP32[$0>>2]|0;
   $7420 = HEAP32[$rc>>2]|0;
   $7421 = (_sqlite3ErrStr($7420)|0);
   HEAP32[$vararg_buffer89>>2] = $7421;
   _sqlite3VdbeError($7419,18391,$vararg_buffer89);
  }
 } while(0);
 if ((label|0) == 889) {
  $7408 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($7408);
  $7409 = HEAP32[$0>>2]|0;
  _sqlite3VdbeError($7409,25016,$vararg_buffer87);
  HEAP32[$rc>>2] = 7;
 }
 $7346 = HEAP32[$db>>2]|0;
 $7347 = ((($7346)) + 69|0);
 $7348 = HEAP8[$7347>>0]|0;
 $7349 = ($7348<<24>>24)!=(0);
 if ($7349) {
  HEAP32[$rc>>2] = 7;
 }
 $7350 = HEAP32[$0>>2]|0;
 $7351 = ((($7350)) + 108|0);
 $7352 = HEAP32[$7351>>2]|0;
 $7353 = ($7352|0)==(0|0);
 $7354 = HEAP32[$rc>>2]|0;
 $7355 = ($7354|0)!=(3082);
 $or$cond26 = $7353 & $7355;
 if ($or$cond26) {
  $7356 = HEAP32[$0>>2]|0;
  $7357 = HEAP32[$rc>>2]|0;
  $7358 = (_sqlite3ErrStr($7357)|0);
  HEAP32[$vararg_buffer77>>2] = $7358;
  _sqlite3VdbeError($7356,18391,$vararg_buffer77);
 }
 $7359 = HEAP32[$rc>>2]|0;
 $7360 = HEAP32[$0>>2]|0;
 $7361 = ((($7360)) + 40|0);
 HEAP32[$7361>>2] = $7359;
 $7362 = HEAP32[$db>>2]|0;
 $7363 = HEAP32[$rc>>2]|0;
 _sqlite3SystemError($7362,$7363);
 $7364 = HEAP32[$rc>>2]|0;
 $7365 = HEAP32[$pOp>>2]|0;
 $7366 = HEAP32[$aOp>>2]|0;
 $7367 = $7365;
 $7368 = $7366;
 $7369 = (($7367) - ($7368))|0;
 $7370 = (($7369|0) / 20)&-1;
 $7371 = HEAP32[$0>>2]|0;
 $7372 = ((($7371)) + 176|0);
 $7373 = HEAP32[$7372>>2]|0;
 $7374 = HEAP32[$0>>2]|0;
 $7375 = ((($7374)) + 108|0);
 $7376 = HEAP32[$7375>>2]|0;
 HEAP32[$vararg_buffer80>>2] = $7370;
 $vararg_ptr83 = ((($vararg_buffer80)) + 4|0);
 HEAP32[$vararg_ptr83>>2] = $7373;
 $vararg_ptr84 = ((($vararg_buffer80)) + 8|0);
 HEAP32[$vararg_ptr84>>2] = $7376;
 _sqlite3_log($7364,40487,$vararg_buffer80);
 $7377 = HEAP32[$0>>2]|0;
 (_sqlite3VdbeHalt($7377)|0);
 $7378 = HEAP32[$rc>>2]|0;
 $7379 = ($7378|0)==(3082);
 if ($7379) {
  $7380 = HEAP32[$db>>2]|0;
  _sqlite3OomFault($7380);
 }
 HEAP32[$rc>>2] = 1;
 $7381 = HEAP8[$resetSchemaOnFault>>0]|0;
 $7382 = $7381&255;
 $7383 = ($7382|0)>(0);
 if (!($7383)) {
  $7388 = $lastRowid;
  $7389 = $7388;
  $7390 = HEAP32[$7389>>2]|0;
  $7391 = (($7388) + 4)|0;
  $7392 = $7391;
  $7393 = HEAP32[$7392>>2]|0;
  $7394 = HEAP32[$db>>2]|0;
  $7395 = ((($7394)) + 32|0);
  $7396 = $7395;
  $7397 = $7396;
  HEAP32[$7397>>2] = $7390;
  $7398 = (($7396) + 4)|0;
  $7399 = $7398;
  HEAP32[$7399>>2] = $7393;
  $7400 = HEAP32[$nVmStep>>2]|0;
  $7401 = HEAP32[$0>>2]|0;
  $7402 = ((($7401)) + 156|0);
  $7403 = ((($7402)) + 16|0);
  $7404 = HEAP32[$7403>>2]|0;
  $7405 = (($7404) + ($7400))|0;
  HEAP32[$7403>>2] = $7405;
  $7406 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($7406|0);
 }
 $7384 = HEAP32[$db>>2]|0;
 $7385 = HEAP8[$resetSchemaOnFault>>0]|0;
 $7386 = $7385&255;
 $7387 = (($7386) - 1)|0;
 _sqlite3ResetOneSchema($7384,$7387);
 $7388 = $lastRowid;
 $7389 = $7388;
 $7390 = HEAP32[$7389>>2]|0;
 $7391 = (($7388) + 4)|0;
 $7392 = $7391;
 $7393 = HEAP32[$7392>>2]|0;
 $7394 = HEAP32[$db>>2]|0;
 $7395 = ((($7394)) + 32|0);
 $7396 = $7395;
 $7397 = $7396;
 HEAP32[$7397>>2] = $7390;
 $7398 = (($7396) + 4)|0;
 $7399 = $7398;
 HEAP32[$7399>>2] = $7393;
 $7400 = HEAP32[$nVmStep>>2]|0;
 $7401 = HEAP32[$0>>2]|0;
 $7402 = ((($7401)) + 156|0);
 $7403 = ((($7402)) + 16|0);
 $7404 = HEAP32[$7403>>2]|0;
 $7405 = (($7404) + ($7400))|0;
 HEAP32[$7403>>2] = $7405;
 $7406 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($7406|0);
}
function _doWalCallbacks($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nEntry = 0, $or$cond = 0, $or$cond3 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $rc = sp + 12|0;
 $i = sp + 8|0;
 $pBt = sp + 4|0;
 $nEntry = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$rc>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $1 = HEAP32[$i>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ((($2)) + 20|0);
  $4 = HEAP32[$3>>2]|0;
  $5 = ($1|0)<($4|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$i>>2]|0;
  $7 = HEAP32[$0>>2]|0;
  $8 = ((($7)) + 16|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9) + ($6<<4)|0);
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  HEAP32[$pBt>>2] = $12;
  $13 = HEAP32[$pBt>>2]|0;
  $14 = ($13|0)!=(0|0);
  if ($14) {
   $15 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeEnter($15);
   $16 = HEAP32[$pBt>>2]|0;
   $17 = (_sqlite3BtreePager($16)|0);
   $18 = (_sqlite3PagerWalCallback($17)|0);
   HEAP32[$nEntry>>2] = $18;
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 224|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = ($21|0)!=(0|0);
   $23 = HEAP32[$nEntry>>2]|0;
   $24 = ($23|0)>(0);
   $or$cond = $22 & $24;
   $25 = HEAP32[$rc>>2]|0;
   $26 = ($25|0)==(0);
   $or$cond3 = $or$cond & $26;
   if ($or$cond3) {
    $27 = HEAP32[$0>>2]|0;
    $28 = ((($27)) + 224|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 228|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = HEAP32[$0>>2]|0;
    $34 = HEAP32[$i>>2]|0;
    $35 = HEAP32[$0>>2]|0;
    $36 = ((($35)) + 16|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (($37) + ($34<<4)|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = HEAP32[$nEntry>>2]|0;
    $41 = (FUNCTION_TABLE_iiiii[$29 & 255]($32,$33,$39,$40)|0);
    HEAP32[$rc>>2] = $41;
   }
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = (($42) + 1)|0;
  HEAP32[$i>>2] = $43;
 }
 $44 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($44|0);
}
function _sqlite3VdbeTransferError($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $db = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 40|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$rc>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 108|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$db>>2]|0;
 if (!($9)) {
  $35 = HEAP32[$rc>>2]|0;
  _sqlite3Error($10,$35);
  $36 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($36|0);
 }
 $11 = ((($10)) + 70|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = (($12) + 1)<<24>>24;
 HEAP8[$11>>0] = $13;
 _sqlite3BeginBenignMalloc();
 $14 = HEAP32[$db>>2]|0;
 $15 = ((($14)) + 244|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0|0);
 if ($17) {
  $18 = HEAP32[$db>>2]|0;
  $19 = (_sqlite3ValueNew($18)|0);
  $20 = HEAP32[$db>>2]|0;
  $21 = ((($20)) + 244|0);
  HEAP32[$21>>2] = $19;
 }
 $22 = HEAP32[$db>>2]|0;
 $23 = ((($22)) + 244|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 108|0);
 $27 = HEAP32[$26>>2]|0;
 _sqlite3ValueSetStr($24,-1,$27,1,(-1));
 _sqlite3EndBenignMalloc();
 $28 = HEAP32[$db>>2]|0;
 $29 = ((($28)) + 70|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = (($30) + -1)<<24>>24;
 HEAP8[$29>>0] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = HEAP32[$db>>2]|0;
 $34 = ((($33)) + 52|0);
 HEAP32[$34>>2] = $32;
 $36 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _sqlite3PagerWalCallback($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3WalCallback($3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3WalCallback($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $ret = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $ret = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$ret>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $8 = HEAP32[$ret>>2]|0;
  STACKTOP = sp;return ($8|0);
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$ret>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$ret>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeEnter($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aDb = 0, $db = 0;
 var $i = 0, $nDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $i = sp + 12|0;
 $db = sp + 8|0;
 $aDb = sp + 4|0;
 $nDb = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 152|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 $7 = HEAP32[$db>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$aDb>>2] = $9;
 $10 = HEAP32[$db>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$nDb>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$nDb>>2]|0;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = ($16|0)!=(1);
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 152|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$i>>2]|0;
   $22 = 1 << $21;
   $23 = $20 & $22;
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$i>>2]|0;
    $26 = HEAP32[$aDb>>2]|0;
    $27 = (($26) + ($25<<4)|0);
    $28 = ((($27)) + 4|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ($29|0)!=(0|0);
    if ($30) {
     $31 = HEAP32[$i>>2]|0;
     $32 = HEAP32[$aDb>>2]|0;
     $33 = (($32) + ($31<<4)|0);
     $34 = ((($33)) + 4|0);
     $35 = HEAP32[$34>>2]|0;
     _sqlite3BtreeEnter($35);
    }
   }
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeSetChanges($db,$nChange) {
 $db = $db|0;
 $nChange = $nChange|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nChange;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 88|0);
 HEAP32[$4>>2] = $2;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 92|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($5))|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _sqlite3VdbeFrameRestore($pFrame) {
 $pFrame = $pFrame|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $v = sp;
 HEAP32[$0>>2] = $pFrame;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$v>>2] = $2;
 $3 = HEAP32[$v>>2]|0;
 _closeCursorsInFrame($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$v>>2]|0;
 $8 = ((($7)) + 88|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$v>>2]|0;
 $13 = ((($12)) + 136|0);
 HEAP32[$13>>2] = $11;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$v>>2]|0;
 $18 = ((($17)) + 92|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 56|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = HEAP32[$v>>2]|0;
 $23 = ((($22)) + 24|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 20|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$v>>2]|0;
 $28 = ((($27)) + 112|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 44|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$v>>2]|0;
 $33 = ((($32)) + 28|0);
 HEAP32[$33>>2] = $31;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 32|0);
 $36 = $35;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = HEAP32[$v>>2]|0;
 $43 = HEAP32[$42>>2]|0;
 $44 = ((($43)) + 32|0);
 $45 = $44;
 $46 = $45;
 HEAP32[$46>>2] = $38;
 $47 = (($45) + 4)|0;
 $48 = $47;
 HEAP32[$48>>2] = $41;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 68|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$v>>2]|0;
 $53 = ((($52)) + 44|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 72|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = HEAP32[$v>>2]|0;
 $58 = HEAP32[$57>>2]|0;
 $59 = ((($58)) + 88|0);
 HEAP32[$59>>2] = $56;
 $60 = HEAP32[$v>>2]|0;
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$v>>2]|0;
 $63 = ((($62)) + 204|0);
 _sqlite3VdbeDeleteAuxData($61,$63,-1,0);
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 40|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = HEAP32[$v>>2]|0;
 $68 = ((($67)) + 204|0);
 HEAP32[$68>>2] = $66;
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 40|0);
 HEAP32[$70>>2] = 0;
 $71 = HEAP32[$0>>2]|0;
 $72 = ((($71)) + 48|0);
 $73 = HEAP32[$72>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _sqlite3VdbeError($p,$zFormat,$varargs) {
 $p = $p|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $ap = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3DbFree($3,$6);
 HEAP32[$ap>>2] = $varargs;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3VMPrintf($8,$9,$ap)|0);
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 108|0);
 HEAP32[$12>>2] = $10;
 STACKTOP = sp;return;
}
function _sqlite3VdbeHalt($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0;
 var $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0;
 var $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0;
 var $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0;
 var $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $eStatementOp = 0, $isSpecialError = 0, $mrc = 0, $or$cond = 0;
 var $or$cond3 = 0, $or$cond5 = 0, $or$cond7 = 0, $or$cond9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $db = sp + 12|0;
 $mrc = sp + 8|0;
 $eStatementOp = sp + 4|0;
 $isSpecialError = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 69|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 40|0);
  HEAP32[$9>>2] = 7;
 }
 $10 = HEAP32[$1>>2]|0;
 _closeAllCursors($10);
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(770837923);
 if ($14) {
  HEAP32[$0>>2] = 0;
  $274 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($274|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 36|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)>=(0);
 if ($18) {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 144|0);
  $21 = HEAP16[$20>>1]|0;
  $22 = ($21&65535) >>> 8;
  $23 = $22 & 1;
  $24 = $23&65535;
  $25 = ($24|0)!=(0);
  if ($25) {
   HEAP32[$eStatementOp>>2] = 0;
   $26 = HEAP32[$1>>2]|0;
   _sqlite3VdbeEnter($26);
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 40|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = $29 & 255;
   HEAP32[$mrc>>2] = $30;
   $31 = HEAP32[$mrc>>2]|0;
   $32 = ($31|0)==(7);
   $33 = HEAP32[$mrc>>2]|0;
   $34 = ($33|0)==(10);
   $or$cond = $32 | $34;
   $35 = HEAP32[$mrc>>2]|0;
   $36 = ($35|0)==(9);
   $or$cond3 = $or$cond | $36;
   if ($or$cond3) {
    $40 = 1;
   } else {
    $37 = HEAP32[$mrc>>2]|0;
    $38 = ($37|0)==(13);
    $40 = $38;
   }
   $39 = $40&1;
   HEAP32[$isSpecialError>>2] = $39;
   $41 = HEAP32[$isSpecialError>>2]|0;
   $42 = ($41|0)!=(0);
   do {
    if ($42) {
     $43 = HEAP32[$1>>2]|0;
     $44 = ((($43)) + 144|0);
     $45 = HEAP16[$44>>1]|0;
     $46 = ($45&65535) >>> 7;
     $47 = $46 & 1;
     $48 = $47&65535;
     $49 = ($48|0)==(0);
     $50 = HEAP32[$mrc>>2]|0;
     $51 = ($50|0)!=(9);
     $or$cond5 = $49 | $51;
     if ($or$cond5) {
      $52 = HEAP32[$mrc>>2]|0;
      $53 = ($52|0)==(7);
      $54 = HEAP32[$mrc>>2]|0;
      $55 = ($54|0)==(13);
      $or$cond7 = $53 | $55;
      if ($or$cond7) {
       $56 = HEAP32[$1>>2]|0;
       $57 = ((($56)) + 144|0);
       $58 = HEAP16[$57>>1]|0;
       $59 = ($58&65535) >>> 6;
       $60 = $59 & 1;
       $61 = $60&65535;
       $62 = ($61|0)!=(0);
       if ($62) {
        HEAP32[$eStatementOp>>2] = 2;
        break;
       }
      }
      $63 = HEAP32[$db>>2]|0;
      _sqlite3RollbackAll($63,516);
      $64 = HEAP32[$db>>2]|0;
      _sqlite3CloseSavepoints($64);
      $65 = HEAP32[$db>>2]|0;
      $66 = ((($65)) + 67|0);
      HEAP8[$66>>0] = 1;
      $67 = HEAP32[$1>>2]|0;
      $68 = ((($67)) + 44|0);
      HEAP32[$68>>2] = 0;
     }
    }
   } while(0);
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 40|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ($71|0)==(0);
   if ($72) {
    $73 = HEAP32[$1>>2]|0;
    (_sqlite3VdbeCheckFk($73,0)|0);
   }
   $74 = HEAP32[$db>>2]|0;
   $75 = ((($74)) + 316|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = ($76|0)>(0);
   if ($77) {
    $78 = HEAP32[$db>>2]|0;
    $79 = ((($78)) + 340|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ($80|0)==(0|0);
    if ($81) {
     label = 36;
    } else {
     label = 19;
    }
   } else {
    label = 19;
   }
   if ((label|0) == 19) {
    $82 = HEAP32[$db>>2]|0;
    $83 = ((($82)) + 67|0);
    $84 = HEAP8[$83>>0]|0;
    $85 = $84&255;
    $86 = ($85|0)!=(0);
    if ($86) {
     $87 = HEAP32[$db>>2]|0;
     $88 = ((($87)) + 164|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = HEAP32[$1>>2]|0;
     $91 = ((($90)) + 144|0);
     $92 = HEAP16[$91>>1]|0;
     $93 = ($92&65535) >>> 7;
     $94 = $93 & 1;
     $95 = $94&65535;
     $96 = ($95|0)==(0);
     $97 = $96&1;
     $98 = ($89|0)==($97|0);
     if ($98) {
      $99 = HEAP32[$1>>2]|0;
      $100 = ((($99)) + 40|0);
      $101 = HEAP32[$100>>2]|0;
      $102 = ($101|0)==(0);
      if ($102) {
       label = 23;
      } else {
       $103 = HEAP32[$1>>2]|0;
       $104 = ((($103)) + 142|0);
       $105 = HEAP8[$104>>0]|0;
       $106 = $105&255;
       $107 = ($106|0)!=(3);
       $108 = HEAP32[$isSpecialError>>2]|0;
       $109 = ($108|0)!=(0);
       $or$cond9 = $107 | $109;
       if ($or$cond9) {
        $158 = HEAP32[$db>>2]|0;
        _sqlite3RollbackAll($158,0);
        $159 = HEAP32[$1>>2]|0;
        $160 = ((($159)) + 44|0);
        HEAP32[$160>>2] = 0;
       } else {
        label = 23;
       }
      }
      do {
       if ((label|0) == 23) {
        $110 = HEAP32[$1>>2]|0;
        $111 = (_sqlite3VdbeCheckFk($110,1)|0);
        HEAP32[$rc>>2] = $111;
        $112 = HEAP32[$rc>>2]|0;
        $113 = ($112|0)!=(0);
        do {
         if ($113) {
          $114 = HEAP32[$1>>2]|0;
          $115 = ((($114)) + 144|0);
          $116 = HEAP16[$115>>1]|0;
          $117 = ($116&65535) >>> 7;
          $118 = $117 & 1;
          $119 = $118&65535;
          $120 = ($119|0)!=(0);
          if (!($120)) {
           HEAP32[$rc>>2] = 787;
           break;
          }
          HEAP32[$0>>2] = 1;
          $274 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($274|0);
         } else {
          $121 = HEAP32[$db>>2]|0;
          $122 = HEAP32[$1>>2]|0;
          $123 = (_vdbeCommit($121,$122)|0);
          HEAP32[$rc>>2] = $123;
         }
        } while(0);
        $124 = HEAP32[$rc>>2]|0;
        $125 = ($124|0)==(5);
        if ($125) {
         $126 = HEAP32[$1>>2]|0;
         $127 = ((($126)) + 144|0);
         $128 = HEAP16[$127>>1]|0;
         $129 = ($128&65535) >>> 7;
         $130 = $129 & 1;
         $131 = $130&65535;
         $132 = ($131|0)!=(0);
         if ($132) {
          HEAP32[$0>>2] = 5;
          $274 = HEAP32[$0>>2]|0;
          STACKTOP = sp;return ($274|0);
         }
        }
        $133 = HEAP32[$rc>>2]|0;
        $134 = ($133|0)!=(0);
        if ($134) {
         $135 = HEAP32[$rc>>2]|0;
         $136 = HEAP32[$1>>2]|0;
         $137 = ((($136)) + 40|0);
         HEAP32[$137>>2] = $135;
         $138 = HEAP32[$db>>2]|0;
         _sqlite3RollbackAll($138,0);
         $139 = HEAP32[$1>>2]|0;
         $140 = ((($139)) + 44|0);
         HEAP32[$140>>2] = 0;
         break;
        } else {
         $141 = HEAP32[$db>>2]|0;
         $142 = ((($141)) + 440|0);
         $143 = $142;
         $144 = $143;
         HEAP32[$144>>2] = 0;
         $145 = (($143) + 4)|0;
         $146 = $145;
         HEAP32[$146>>2] = 0;
         $147 = HEAP32[$db>>2]|0;
         $148 = ((($147)) + 448|0);
         $149 = $148;
         $150 = $149;
         HEAP32[$150>>2] = 0;
         $151 = (($149) + 4)|0;
         $152 = $151;
         HEAP32[$152>>2] = 0;
         $153 = HEAP32[$db>>2]|0;
         $154 = ((($153)) + 24|0);
         $155 = HEAP32[$154>>2]|0;
         $156 = $155 & -33554433;
         HEAP32[$154>>2] = $156;
         $157 = HEAP32[$db>>2]|0;
         _sqlite3CommitInternalChanges($157);
         break;
        }
       }
      } while(0);
      $161 = HEAP32[$db>>2]|0;
      $162 = ((($161)) + 436|0);
      HEAP32[$162>>2] = 0;
     } else {
      label = 36;
     }
    } else {
     label = 36;
    }
   }
   do {
    if ((label|0) == 36) {
     $163 = HEAP32[$eStatementOp>>2]|0;
     $164 = ($163|0)==(0);
     if ($164) {
      $165 = HEAP32[$1>>2]|0;
      $166 = ((($165)) + 40|0);
      $167 = HEAP32[$166>>2]|0;
      $168 = ($167|0)==(0);
      if (!($168)) {
       $169 = HEAP32[$1>>2]|0;
       $170 = ((($169)) + 142|0);
       $171 = HEAP8[$170>>0]|0;
       $172 = $171&255;
       $173 = ($172|0)==(3);
       if (!($173)) {
        $174 = HEAP32[$1>>2]|0;
        $175 = ((($174)) + 142|0);
        $176 = HEAP8[$175>>0]|0;
        $177 = $176&255;
        $178 = ($177|0)==(2);
        if ($178) {
         HEAP32[$eStatementOp>>2] = 2;
         break;
        } else {
         $179 = HEAP32[$db>>2]|0;
         _sqlite3RollbackAll($179,516);
         $180 = HEAP32[$db>>2]|0;
         _sqlite3CloseSavepoints($180);
         $181 = HEAP32[$db>>2]|0;
         $182 = ((($181)) + 67|0);
         HEAP8[$182>>0] = 1;
         $183 = HEAP32[$1>>2]|0;
         $184 = ((($183)) + 44|0);
         HEAP32[$184>>2] = 0;
         break;
        }
       }
      }
      HEAP32[$eStatementOp>>2] = 1;
     }
    }
   } while(0);
   $185 = HEAP32[$eStatementOp>>2]|0;
   $186 = ($185|0)!=(0);
   if ($186) {
    $187 = HEAP32[$1>>2]|0;
    $188 = HEAP32[$eStatementOp>>2]|0;
    $189 = (_sqlite3VdbeCloseStatement($187,$188)|0);
    HEAP32[$rc>>2] = $189;
    $190 = HEAP32[$rc>>2]|0;
    $191 = ($190|0)!=(0);
    if ($191) {
     $192 = HEAP32[$1>>2]|0;
     $193 = ((($192)) + 40|0);
     $194 = HEAP32[$193>>2]|0;
     $195 = ($194|0)==(0);
     if ($195) {
      label = 47;
     } else {
      $196 = HEAP32[$1>>2]|0;
      $197 = ((($196)) + 40|0);
      $198 = HEAP32[$197>>2]|0;
      $199 = $198 & 255;
      $200 = ($199|0)==(19);
      if ($200) {
       label = 47;
      }
     }
     if ((label|0) == 47) {
      $201 = HEAP32[$rc>>2]|0;
      $202 = HEAP32[$1>>2]|0;
      $203 = ((($202)) + 40|0);
      HEAP32[$203>>2] = $201;
      $204 = HEAP32[$db>>2]|0;
      $205 = HEAP32[$1>>2]|0;
      $206 = ((($205)) + 108|0);
      $207 = HEAP32[$206>>2]|0;
      _sqlite3DbFree($204,$207);
      $208 = HEAP32[$1>>2]|0;
      $209 = ((($208)) + 108|0);
      HEAP32[$209>>2] = 0;
     }
     $210 = HEAP32[$db>>2]|0;
     _sqlite3RollbackAll($210,516);
     $211 = HEAP32[$db>>2]|0;
     _sqlite3CloseSavepoints($211);
     $212 = HEAP32[$db>>2]|0;
     $213 = ((($212)) + 67|0);
     HEAP8[$213>>0] = 1;
     $214 = HEAP32[$1>>2]|0;
     $215 = ((($214)) + 44|0);
     HEAP32[$215>>2] = 0;
    }
   }
   $216 = HEAP32[$1>>2]|0;
   $217 = ((($216)) + 144|0);
   $218 = HEAP16[$217>>1]|0;
   $219 = ($218&65535) >>> 4;
   $220 = $219 & 1;
   $221 = $220&65535;
   $222 = ($221|0)!=(0);
   if ($222) {
    $223 = HEAP32[$eStatementOp>>2]|0;
    $224 = ($223|0)!=(2);
    $225 = HEAP32[$db>>2]|0;
    if ($224) {
     $226 = HEAP32[$1>>2]|0;
     $227 = ((($226)) + 44|0);
     $228 = HEAP32[$227>>2]|0;
     _sqlite3VdbeSetChanges($225,$228);
    } else {
     _sqlite3VdbeSetChanges($225,0);
    }
    $229 = HEAP32[$1>>2]|0;
    $230 = ((($229)) + 44|0);
    HEAP32[$230>>2] = 0;
   }
  }
 }
 $231 = HEAP32[$1>>2]|0;
 $232 = ((($231)) + 36|0);
 $233 = HEAP32[$232>>2]|0;
 $234 = ($233|0)>=(0);
 if ($234) {
  $235 = HEAP32[$db>>2]|0;
  $236 = ((($235)) + 156|0);
  $237 = HEAP32[$236>>2]|0;
  $238 = (($237) + -1)|0;
  HEAP32[$236>>2] = $238;
  $239 = HEAP32[$1>>2]|0;
  $240 = ((($239)) + 144|0);
  $241 = HEAP16[$240>>1]|0;
  $242 = ($241&65535) >>> 7;
  $243 = $242 & 1;
  $244 = $243&65535;
  $245 = ($244|0)!=(0);
  if (!($245)) {
   $246 = HEAP32[$db>>2]|0;
   $247 = ((($246)) + 164|0);
   $248 = HEAP32[$247>>2]|0;
   $249 = (($248) + -1)|0;
   HEAP32[$247>>2] = $249;
  }
  $250 = HEAP32[$1>>2]|0;
  $251 = ((($250)) + 144|0);
  $252 = HEAP16[$251>>1]|0;
  $253 = ($252&65535) >>> 8;
  $254 = $253 & 1;
  $255 = $254&65535;
  $256 = ($255|0)!=(0);
  if ($256) {
   $257 = HEAP32[$db>>2]|0;
   $258 = ((($257)) + 160|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = (($259) + -1)|0;
   HEAP32[$258>>2] = $260;
  }
 }
 $261 = HEAP32[$1>>2]|0;
 $262 = ((($261)) + 20|0);
 HEAP32[$262>>2] = 832317811;
 $263 = HEAP32[$db>>2]|0;
 $264 = ((($263)) + 69|0);
 $265 = HEAP8[$264>>0]|0;
 $266 = ($265<<24>>24)!=(0);
 if ($266) {
  $267 = HEAP32[$1>>2]|0;
  $268 = ((($267)) + 40|0);
  HEAP32[$268>>2] = 7;
 }
 $269 = HEAP32[$1>>2]|0;
 $270 = ((($269)) + 40|0);
 $271 = HEAP32[$270>>2]|0;
 $272 = ($271|0)==(5);
 $273 = $272 ? 5 : 0;
 HEAP32[$0>>2] = $273;
 $274 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($274|0);
}
function _out2Prerelease($p,$pOp) {
 $p = $p|0;
 $pOp = $pOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pOut = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOut = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pOp;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 92|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + (($5*40)|0)|0);
 HEAP32[$pOut>>2] = $9;
 $10 = HEAP32[$pOut>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = $13 & 9312;
 $15 = ($14|0)!=(0);
 $16 = HEAP32[$pOut>>2]|0;
 if ($15) {
  $17 = (_out2PrereleaseWithClear($16)|0);
  HEAP32[$0>>2] = $17;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 } else {
  $18 = ((($16)) + 8|0);
  HEAP16[$18>>1] = 4;
  $19 = HEAP32[$pOut>>2]|0;
  HEAP32[$0>>2] = $19;
  $20 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($20|0);
 }
 return (0)|0;
}
function _sqlite3VdbeMemTooBig($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $n = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 18;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$n>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 16384;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$n>>2]|0;
  $20 = (($19) + ($18))|0;
  HEAP32[$n>>2] = $20;
 }
 $21 = HEAP32[$n>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 32|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ((($24)) + 96|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = ($21|0)>($26|0);
 $28 = $27&1;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3VdbeCheckFk($p,$deferred) {
 $p = $p|0;
 $deferred = $deferred|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $deferred;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$db>>2]|0;
  $8 = ((($7)) + 440|0);
  $9 = $8;
  $10 = $9;
  $11 = HEAP32[$10>>2]|0;
  $12 = (($9) + 4)|0;
  $13 = $12;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$db>>2]|0;
  $16 = ((($15)) + 448|0);
  $17 = $16;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (($17) + 4)|0;
  $21 = $20;
  $22 = HEAP32[$21>>2]|0;
  $23 = (_i64Add(($11|0),($14|0),($19|0),($22|0))|0);
  $24 = tempRet0;
  $25 = ($24|0)>(0);
  $26 = ($23>>>0)>(0);
  $27 = ($24|0)==(0);
  $28 = $27 & $26;
  $29 = $25 | $28;
  if (!($29)) {
   label = 3;
  }
 } else {
  label = 3;
 }
 do {
  if ((label|0) == 3) {
   $30 = HEAP32[$2>>2]|0;
   $31 = ($30|0)!=(0);
   if (!($31)) {
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 64|0);
    $34 = $33;
    $35 = $34;
    $36 = HEAP32[$35>>2]|0;
    $37 = (($34) + 4)|0;
    $38 = $37;
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)>(0);
    $41 = ($36>>>0)>(0);
    $42 = ($39|0)==(0);
    $43 = $42 & $41;
    $44 = $40 | $43;
    if ($44) {
     break;
    }
   }
   HEAP32[$0>>2] = 0;
   $50 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($50|0);
  }
 } while(0);
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 40|0);
 HEAP32[$46>>2] = 787;
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 142|0);
 HEAP8[$48>>0] = 2;
 $49 = HEAP32[$1>>2]|0;
 _sqlite3VdbeError($49,35189,$vararg_buffer);
 HEAP32[$0>>2] = 1;
 $50 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($50|0);
}
function _sqlite3VdbeCloseStatement($p,$eOp) {
 $p = $p|0;
 $eOp = $eOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $db = 0, $i = 0, $iSavepoint = 0, $pBt = 0, $rc = 0, $rc2 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $db = sp + 20|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $iSavepoint = sp + 8|0;
 $rc2 = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $eOp;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$db>>2]|0;
 $5 = ((($4)) + 436|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 48|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 48|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) - 1)|0;
 HEAP32[$iSavepoint>>2] = $15;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$db>>2]|0;
  $18 = ((($17)) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   break;
  }
  HEAP32[$rc2>>2] = 0;
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$db>>2]|0;
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pBt>>2] = $27;
  $28 = HEAP32[$pBt>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$1>>2]|0;
   $31 = ($30|0)==(2);
   if ($31) {
    $32 = HEAP32[$pBt>>2]|0;
    $33 = HEAP32[$iSavepoint>>2]|0;
    $34 = (_sqlite3BtreeSavepoint($32,2,$33)|0);
    HEAP32[$rc2>>2] = $34;
   }
   $35 = HEAP32[$rc2>>2]|0;
   $36 = ($35|0)==(0);
   if ($36) {
    $37 = HEAP32[$pBt>>2]|0;
    $38 = HEAP32[$iSavepoint>>2]|0;
    $39 = (_sqlite3BtreeSavepoint($37,1,$38)|0);
    HEAP32[$rc2>>2] = $39;
   }
   $40 = HEAP32[$rc>>2]|0;
   $41 = ($40|0)==(0);
   if ($41) {
    $42 = HEAP32[$rc2>>2]|0;
    HEAP32[$rc>>2] = $42;
   }
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 $45 = HEAP32[$db>>2]|0;
 $46 = ((($45)) + 436|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + -1)|0;
 HEAP32[$46>>2] = $48;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 48|0);
 HEAP32[$50>>2] = 0;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$1>>2]|0;
  $54 = ($53|0)==(2);
  if ($54) {
   $55 = HEAP32[$db>>2]|0;
   $56 = HEAP32[$iSavepoint>>2]|0;
   $57 = (_sqlite3VtabSavepoint($55,2,$56)|0);
   HEAP32[$rc>>2] = $57;
  }
  $58 = HEAP32[$rc>>2]|0;
  $59 = ($58|0)==(0);
  if ($59) {
   $60 = HEAP32[$db>>2]|0;
   $61 = HEAP32[$iSavepoint>>2]|0;
   $62 = (_sqlite3VtabSavepoint($60,1,$61)|0);
   HEAP32[$rc>>2] = $62;
  }
 }
 $63 = HEAP32[$1>>2]|0;
 $64 = ($63|0)==(2);
 if (!($64)) {
  $93 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $65 = HEAP32[$0>>2]|0;
 $66 = ((($65)) + 72|0);
 $67 = $66;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($67) + 4)|0;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = HEAP32[$db>>2]|0;
 $74 = ((($73)) + 440|0);
 $75 = $74;
 $76 = $75;
 HEAP32[$76>>2] = $69;
 $77 = (($75) + 4)|0;
 $78 = $77;
 HEAP32[$78>>2] = $72;
 $79 = HEAP32[$0>>2]|0;
 $80 = ((($79)) + 80|0);
 $81 = $80;
 $82 = $81;
 $83 = HEAP32[$82>>2]|0;
 $84 = (($81) + 4)|0;
 $85 = $84;
 $86 = HEAP32[$85>>2]|0;
 $87 = HEAP32[$db>>2]|0;
 $88 = ((($87)) + 448|0);
 $89 = $88;
 $90 = $89;
 HEAP32[$90>>2] = $83;
 $91 = (($89) + 4)|0;
 $92 = $91;
 HEAP32[$92>>2] = $86;
 $93 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3VdbeMemNulTerminate($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 514;
 $7 = ($6|0)!=(2);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_vdbeMemAddTerminator($8)|0);
  HEAP32[$0>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 return (0)|0;
}
function _numericType($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 12;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 do {
  if ($7) {
   $12 = $11 & 12;
   $13 = $12&65535;
   HEAP16[$0>>1] = $13;
  } else {
   $14 = $11 & 18;
   $15 = ($14|0)!=(0);
   if ($15) {
    $16 = HEAP32[$1>>2]|0;
    $17 = (_computeNumericType($16)|0);
    HEAP16[$0>>1] = $17;
    break;
   } else {
    HEAP16[$0>>1] = 0;
    break;
   }
  }
 } while(0);
 $18 = HEAP16[$0>>1]|0;
 STACKTOP = sp;return ($18|0);
}
function _sqlite3AddInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iA = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 8|0;
 $iA = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = $iA;
 $17 = $16;
 HEAP32[$17>>2] = $12;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $4;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)>(0);
 $27 = ($22>>>0)>=(0);
 $28 = ($25|0)==(0);
 $29 = $28 & $27;
 $30 = $26 | $29;
 $31 = $iA;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 $34 = (($31) + 4)|0;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 if ($30) {
  $37 = ($36|0)>(0);
  $38 = ($33>>>0)>(0);
  $39 = ($36|0)==(0);
  $40 = $39 & $38;
  $41 = $37 | $40;
  if ($41) {
   $42 = $iA;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = (_i64Subtract(-1,2147483647,($44|0),($47|0))|0);
   $49 = tempRet0;
   $50 = $4;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = ($49|0)<($55|0);
   $57 = ($48>>>0)<($52>>>0);
   $58 = ($49|0)==($55|0);
   $59 = $58 & $57;
   $60 = $56 | $59;
   if ($60) {
    HEAP32[$2>>2] = 1;
    $104 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($104|0);
   }
  }
 } else {
  $61 = ($36|0)<(0);
  if ($61) {
   $62 = $iA;
   $63 = $62;
   $64 = HEAP32[$63>>2]|0;
   $65 = (($62) + 4)|0;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (_i64Add(($64|0),($67|0),-1,2147483647)|0);
   $69 = tempRet0;
   $70 = (_i64Subtract(0,0,($68|0),($69|0))|0);
   $71 = tempRet0;
   $72 = $4;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = (($72) + 4)|0;
   $76 = $75;
   $77 = HEAP32[$76>>2]|0;
   $78 = (_i64Add(($74|0),($77|0),1,0)|0);
   $79 = tempRet0;
   $80 = ($71|0)>($79|0);
   $81 = ($70>>>0)>($78>>>0);
   $82 = ($71|0)==($79|0);
   $83 = $82 & $81;
   $84 = $80 | $83;
   if ($84) {
    HEAP32[$2>>2] = 1;
    $104 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($104|0);
   }
  }
 }
 $85 = $4;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (($85) + 4)|0;
 $89 = $88;
 $90 = HEAP32[$89>>2]|0;
 $91 = HEAP32[$3>>2]|0;
 $92 = $91;
 $93 = $92;
 $94 = HEAP32[$93>>2]|0;
 $95 = (($92) + 4)|0;
 $96 = $95;
 $97 = HEAP32[$96>>2]|0;
 $98 = (_i64Add(($94|0),($97|0),($87|0),($90|0))|0);
 $99 = tempRet0;
 $100 = $91;
 $101 = $100;
 HEAP32[$101>>2] = $98;
 $102 = (($100) + 4)|0;
 $103 = $102;
 HEAP32[$103>>2] = $99;
 HEAP32[$2>>2] = 0;
 $104 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($104|0);
}
function _sqlite3SubInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($11|0)==(0);
 $16 = ($14|0)==(-2147483648);
 $17 = $15 & $16;
 $18 = HEAP32[$3>>2]|0;
 if (!($17)) {
  $49 = $4;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = (_i64Subtract(0,0,($51|0),($54|0))|0);
  $56 = tempRet0;
  $57 = (_sqlite3AddInt64($18,$55,$56)|0);
  HEAP32[$2>>2] = $57;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24|0)>(0);
 $26 = ($21>>>0)>=(0);
 $27 = ($24|0)==(0);
 $28 = $27 & $26;
 $29 = $25 | $28;
 if ($29) {
  HEAP32[$2>>2] = 1;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 } else {
  $30 = $4;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$3>>2]|0;
  $37 = $36;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Subtract(($39|0),($42|0),($32|0),($35|0))|0);
  $44 = tempRet0;
  $45 = $36;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  HEAP32[$2>>2] = 0;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 return (0)|0;
}
function _sqlite3MulInt64($pA,$0,$1) {
 $pA = $pA|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iA = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 8|0;
 $iA = sp;
 HEAP32[$3>>2] = $pA;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = $9;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = $iA;
 $17 = $16;
 HEAP32[$17>>2] = $12;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $15;
 $20 = $4;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)>(0);
 $27 = ($22>>>0)>(0);
 $28 = ($25|0)==(0);
 $29 = $28 & $27;
 $30 = $26 | $29;
 do {
  if ($30) {
   $31 = $iA;
   $32 = $31;
   $33 = HEAP32[$32>>2]|0;
   $34 = (($31) + 4)|0;
   $35 = $34;
   $36 = HEAP32[$35>>2]|0;
   $37 = $4;
   $38 = $37;
   $39 = HEAP32[$38>>2]|0;
   $40 = (($37) + 4)|0;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = (___divdi3(-1,2147483647,($39|0),($42|0))|0);
   $44 = tempRet0;
   $45 = ($36|0)>($44|0);
   $46 = ($33>>>0)>($43>>>0);
   $47 = ($36|0)==($44|0);
   $48 = $47 & $46;
   $49 = $45 | $48;
   if ($49) {
    HEAP32[$2>>2] = 1;
    $173 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($173|0);
   }
   $50 = $iA;
   $51 = $50;
   $52 = HEAP32[$51>>2]|0;
   $53 = (($50) + 4)|0;
   $54 = $53;
   $55 = HEAP32[$54>>2]|0;
   $56 = $4;
   $57 = $56;
   $58 = HEAP32[$57>>2]|0;
   $59 = (($56) + 4)|0;
   $60 = $59;
   $61 = HEAP32[$60>>2]|0;
   $62 = (___divdi3(0,-2147483648,($58|0),($61|0))|0);
   $63 = tempRet0;
   $64 = ($55|0)<($63|0);
   $65 = ($52>>>0)<($62>>>0);
   $66 = ($55|0)==($63|0);
   $67 = $66 & $65;
   $68 = $64 | $67;
   if ($68) {
    HEAP32[$2>>2] = 1;
    $173 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($173|0);
   }
  } else {
   $69 = $4;
   $70 = $69;
   $71 = HEAP32[$70>>2]|0;
   $72 = (($69) + 4)|0;
   $73 = $72;
   $74 = HEAP32[$73>>2]|0;
   $75 = ($74|0)<(0);
   if ($75) {
    $76 = $iA;
    $77 = $76;
    $78 = HEAP32[$77>>2]|0;
    $79 = (($76) + 4)|0;
    $80 = $79;
    $81 = HEAP32[$80>>2]|0;
    $82 = ($81|0)>(0);
    $83 = ($78>>>0)>(0);
    $84 = ($81|0)==(0);
    $85 = $84 & $83;
    $86 = $82 | $85;
    if ($86) {
     $87 = $4;
     $88 = $87;
     $89 = HEAP32[$88>>2]|0;
     $90 = (($87) + 4)|0;
     $91 = $90;
     $92 = HEAP32[$91>>2]|0;
     $93 = $iA;
     $94 = $93;
     $95 = HEAP32[$94>>2]|0;
     $96 = (($93) + 4)|0;
     $97 = $96;
     $98 = HEAP32[$97>>2]|0;
     $99 = (___divdi3(0,-2147483648,($95|0),($98|0))|0);
     $100 = tempRet0;
     $101 = ($92|0)<($100|0);
     $102 = ($89>>>0)<($99>>>0);
     $103 = ($92|0)==($100|0);
     $104 = $103 & $102;
     $105 = $101 | $104;
     if (!($105)) {
      break;
     }
     HEAP32[$2>>2] = 1;
     $173 = HEAP32[$2>>2]|0;
     STACKTOP = sp;return ($173|0);
    }
    $106 = $iA;
    $107 = $106;
    $108 = HEAP32[$107>>2]|0;
    $109 = (($106) + 4)|0;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = ($111|0)<(0);
    if ($112) {
     $113 = $4;
     $114 = $113;
     $115 = HEAP32[$114>>2]|0;
     $116 = (($113) + 4)|0;
     $117 = $116;
     $118 = HEAP32[$117>>2]|0;
     $119 = ($115|0)==(0);
     $120 = ($118|0)==(-2147483648);
     $121 = $119 & $120;
     if ($121) {
      HEAP32[$2>>2] = 1;
      $173 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($173|0);
     }
     $122 = $iA;
     $123 = $122;
     $124 = HEAP32[$123>>2]|0;
     $125 = (($122) + 4)|0;
     $126 = $125;
     $127 = HEAP32[$126>>2]|0;
     $128 = ($124|0)==(0);
     $129 = ($127|0)==(-2147483648);
     $130 = $128 & $129;
     if ($130) {
      HEAP32[$2>>2] = 1;
      $173 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($173|0);
     }
     $131 = $iA;
     $132 = $131;
     $133 = HEAP32[$132>>2]|0;
     $134 = (($131) + 4)|0;
     $135 = $134;
     $136 = HEAP32[$135>>2]|0;
     $137 = (_i64Subtract(0,0,($133|0),($136|0))|0);
     $138 = tempRet0;
     $139 = $4;
     $140 = $139;
     $141 = HEAP32[$140>>2]|0;
     $142 = (($139) + 4)|0;
     $143 = $142;
     $144 = HEAP32[$143>>2]|0;
     $145 = (_i64Subtract(0,0,($141|0),($144|0))|0);
     $146 = tempRet0;
     $147 = (___divdi3(-1,2147483647,($145|0),($146|0))|0);
     $148 = tempRet0;
     $149 = ($138|0)>($148|0);
     $150 = ($137>>>0)>($147>>>0);
     $151 = ($138|0)==($148|0);
     $152 = $151 & $150;
     $153 = $149 | $152;
     if ($153) {
      HEAP32[$2>>2] = 1;
      $173 = HEAP32[$2>>2]|0;
      STACKTOP = sp;return ($173|0);
     }
    }
   }
  }
 } while(0);
 $154 = $iA;
 $155 = $154;
 $156 = HEAP32[$155>>2]|0;
 $157 = (($154) + 4)|0;
 $158 = $157;
 $159 = HEAP32[$158>>2]|0;
 $160 = $4;
 $161 = $160;
 $162 = HEAP32[$161>>2]|0;
 $163 = (($160) + 4)|0;
 $164 = $163;
 $165 = HEAP32[$164>>2]|0;
 $166 = (___muldi3(($156|0),($159|0),($162|0),($165|0))|0);
 $167 = tempRet0;
 $168 = HEAP32[$3>>2]|0;
 $169 = $168;
 $170 = $169;
 HEAP32[$170>>2] = $166;
 $171 = (($169) + 4)|0;
 $172 = $171;
 HEAP32[$172>>2] = $167;
 HEAP32[$2>>2] = 0;
 $173 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($173|0);
}
function _sqlite3VdbeDeleteAuxData($db,$pp,$iOp,$mask) {
 $db = $db|0;
 $pp = $pp|0;
 $iOp = $iOp|0;
 $mask = $mask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAux = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $pAux = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pp;
 HEAP32[$2>>2] = $iOp;
 HEAP32[$3>>2] = $mask;
 L1: while(1) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$1>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$pAux>>2] = $8;
  $9 = HEAP32[$2>>2]|0;
  $10 = ($9|0)<(0);
  do {
   if (!($10)) {
    $11 = HEAP32[$pAux>>2]|0;
    $12 = HEAP32[$11>>2]|0;
    $13 = HEAP32[$2>>2]|0;
    $14 = ($12|0)==($13|0);
    if ($14) {
     $15 = HEAP32[$pAux>>2]|0;
     $16 = ((($15)) + 4|0);
     $17 = HEAP32[$16>>2]|0;
     $18 = ($17|0)>(31);
     if ($18) {
      break;
     }
     $19 = HEAP32[$3>>2]|0;
     $20 = HEAP32[$pAux>>2]|0;
     $21 = ((($20)) + 4|0);
     $22 = HEAP32[$21>>2]|0;
     $23 = 1 << $22;
     $24 = $19 & $23;
     $25 = ($24|0)!=(0);
     if (!($25)) {
      break;
     }
    }
    $42 = HEAP32[$pAux>>2]|0;
    $43 = ((($42)) + 16|0);
    HEAP32[$1>>2] = $43;
    continue L1;
   }
  } while(0);
  $26 = HEAP32[$pAux>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$pAux>>2]|0;
   $31 = ((($30)) + 12|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$pAux>>2]|0;
   $34 = ((($33)) + 8|0);
   $35 = HEAP32[$34>>2]|0;
   FUNCTION_TABLE_vi[$32 & 255]($35);
  }
  $36 = HEAP32[$pAux>>2]|0;
  $37 = ((($36)) + 16|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = HEAP32[$1>>2]|0;
  HEAP32[$39>>2] = $38;
  $40 = HEAP32[$0>>2]|0;
  $41 = HEAP32[$pAux>>2]|0;
  _sqlite3DbFree($40,$41);
 }
 STACKTOP = sp;return;
}
function _sqlite3MemCompare($pMem1,$pMem2,$pColl) {
 $pMem1 = $pMem1|0;
 $pMem2 = $pMem2|0;
 $pColl = $pColl|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0.0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0, $77 = 0.0, $78 = 0, $79 = 0, $8 = 0, $80 = 0.0, $81 = 0, $82 = 0.0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0.0, $99 = 0, $combined_flags = 0, $f1 = 0, $f2 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $f1 = sp + 8|0;
 $f2 = sp + 4|0;
 $combined_flags = sp;
 HEAP32[$1>>2] = $pMem1;
 HEAP32[$2>>2] = $pMem2;
 HEAP32[$3>>2] = $pColl;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$f1>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 HEAP32[$f2>>2] = $11;
 $12 = HEAP32[$f1>>2]|0;
 $13 = HEAP32[$f2>>2]|0;
 $14 = $12 | $13;
 HEAP32[$combined_flags>>2] = $14;
 $15 = HEAP32[$combined_flags>>2]|0;
 $16 = $15 & 1;
 $17 = ($16|0)!=(0);
 do {
  if ($17) {
   $18 = HEAP32[$f2>>2]|0;
   $19 = $18 & 1;
   $20 = HEAP32[$f1>>2]|0;
   $21 = $20 & 1;
   $22 = (($19) - ($21))|0;
   HEAP32[$0>>2] = $22;
  } else {
   $23 = HEAP32[$combined_flags>>2]|0;
   $24 = $23 & 12;
   $25 = ($24|0)!=(0);
   if (!($25)) {
    $117 = HEAP32[$combined_flags>>2]|0;
    $118 = $117 & 2;
    $119 = ($118|0)!=(0);
    if ($119) {
     $120 = HEAP32[$f1>>2]|0;
     $121 = $120 & 2;
     $122 = ($121|0)==(0);
     if ($122) {
      HEAP32[$0>>2] = 1;
      break;
     }
     $123 = HEAP32[$f2>>2]|0;
     $124 = $123 & 2;
     $125 = ($124|0)==(0);
     if ($125) {
      HEAP32[$0>>2] = -1;
      break;
     }
     $126 = HEAP32[$3>>2]|0;
     $127 = ($126|0)!=(0|0);
     if ($127) {
      $128 = HEAP32[$1>>2]|0;
      $129 = HEAP32[$2>>2]|0;
      $130 = HEAP32[$3>>2]|0;
      $131 = (_vdbeCompareMemString($128,$129,$130,0)|0);
      HEAP32[$0>>2] = $131;
      break;
     }
    }
    $132 = HEAP32[$1>>2]|0;
    $133 = HEAP32[$2>>2]|0;
    $134 = (_sqlite3BlobCompare($132,$133)|0);
    HEAP32[$0>>2] = $134;
    break;
   }
   $26 = HEAP32[$f1>>2]|0;
   $27 = HEAP32[$f2>>2]|0;
   $28 = $26 & $27;
   $29 = $28 & 4;
   $30 = ($29|0)!=(0);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    $32 = $31;
    $33 = $32;
    $34 = HEAP32[$33>>2]|0;
    $35 = (($32) + 4)|0;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$2>>2]|0;
    $39 = $38;
    $40 = $39;
    $41 = HEAP32[$40>>2]|0;
    $42 = (($39) + 4)|0;
    $43 = $42;
    $44 = HEAP32[$43>>2]|0;
    $45 = ($37|0)<($44|0);
    $46 = ($34>>>0)<($41>>>0);
    $47 = ($37|0)==($44|0);
    $48 = $47 & $46;
    $49 = $45 | $48;
    if ($49) {
     HEAP32[$0>>2] = -1;
     break;
    }
    $50 = HEAP32[$1>>2]|0;
    $51 = $50;
    $52 = $51;
    $53 = HEAP32[$52>>2]|0;
    $54 = (($51) + 4)|0;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = HEAP32[$2>>2]|0;
    $58 = $57;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = (($58) + 4)|0;
    $62 = $61;
    $63 = HEAP32[$62>>2]|0;
    $64 = ($56|0)>($63|0);
    $65 = ($53>>>0)>($60>>>0);
    $66 = ($56|0)==($63|0);
    $67 = $66 & $65;
    $68 = $64 | $67;
    if ($68) {
     HEAP32[$0>>2] = 1;
     break;
    } else {
     HEAP32[$0>>2] = 0;
     break;
    }
   }
   $69 = HEAP32[$f1>>2]|0;
   $70 = HEAP32[$f2>>2]|0;
   $71 = $69 & $70;
   $72 = $71 & 8;
   $73 = ($72|0)!=(0);
   if ($73) {
    $74 = HEAP32[$1>>2]|0;
    $75 = +HEAPF64[$74>>3];
    $76 = HEAP32[$2>>2]|0;
    $77 = +HEAPF64[$76>>3];
    $78 = $75 < $77;
    if ($78) {
     HEAP32[$0>>2] = -1;
     break;
    }
    $79 = HEAP32[$1>>2]|0;
    $80 = +HEAPF64[$79>>3];
    $81 = HEAP32[$2>>2]|0;
    $82 = +HEAPF64[$81>>3];
    $83 = $80 > $82;
    if ($83) {
     HEAP32[$0>>2] = 1;
     break;
    } else {
     HEAP32[$0>>2] = 0;
     break;
    }
   }
   $84 = HEAP32[$f1>>2]|0;
   $85 = $84 & 4;
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = HEAP32[$f2>>2]|0;
    $88 = $87 & 8;
    $89 = ($88|0)!=(0);
    if ($89) {
     $90 = HEAP32[$1>>2]|0;
     $91 = $90;
     $92 = $91;
     $93 = HEAP32[$92>>2]|0;
     $94 = (($91) + 4)|0;
     $95 = $94;
     $96 = HEAP32[$95>>2]|0;
     $97 = HEAP32[$2>>2]|0;
     $98 = +HEAPF64[$97>>3];
     $99 = (_sqlite3IntFloatCompare($93,$96,$98)|0);
     HEAP32[$0>>2] = $99;
     break;
    } else {
     HEAP32[$0>>2] = -1;
     break;
    }
   }
   $100 = HEAP32[$f1>>2]|0;
   $101 = $100 & 8;
   $102 = ($101|0)!=(0);
   if (!($102)) {
    HEAP32[$0>>2] = 1;
    break;
   }
   $103 = HEAP32[$f2>>2]|0;
   $104 = $103 & 4;
   $105 = ($104|0)!=(0);
   if ($105) {
    $106 = HEAP32[$2>>2]|0;
    $107 = $106;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$1>>2]|0;
    $114 = +HEAPF64[$113>>3];
    $115 = (_sqlite3IntFloatCompare($109,$112,$114)|0);
    $116 = (0 - ($115))|0;
    HEAP32[$0>>2] = $116;
    break;
   } else {
    HEAP32[$0>>2] = -1;
    break;
   }
  }
 } while(0);
 $135 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _sqlite3VdbeCursorMoveto($pp,$piCol) {
 $pp = $pp|0;
 $piCol = $piCol|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $iMap = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $iMap = sp;
 HEAP32[$1>>2] = $pp;
 HEAP32[$2>>2] = $piCol;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(0);
 do {
  if ($8) {
   $9 = HEAP32[$p>>2]|0;
   $10 = ((($9)) + 3|0);
   $11 = HEAP8[$10>>0]|0;
   $12 = ($11<<24>>24)!=(0);
   $13 = HEAP32[$p>>2]|0;
   if (!($12)) {
    $35 = ((($13)) + 16|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = (_sqlite3BtreeCursorHasMoved($36)|0);
    $38 = ($37|0)!=(0);
    if (!($38)) {
     break;
    }
    $39 = HEAP32[$p>>2]|0;
    $40 = (_handleMovedCursor($39)|0);
    HEAP32[$0>>2] = $40;
    $41 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($41|0);
   }
   $14 = ((($13)) + 52|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   if ($16) {
    $17 = HEAP32[$2>>2]|0;
    $18 = HEAP32[$17>>2]|0;
    $19 = (1 + ($18))|0;
    $20 = HEAP32[$p>>2]|0;
    $21 = ((($20)) + 52|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = (($22) + ($19<<2)|0);
    $24 = HEAP32[$23>>2]|0;
    HEAP32[$iMap>>2] = $24;
    $25 = ($24|0)>(0);
    if ($25) {
     $26 = HEAP32[$p>>2]|0;
     $27 = ((($26)) + 48|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = HEAP32[$1>>2]|0;
     HEAP32[$29>>2] = $28;
     $30 = HEAP32[$iMap>>2]|0;
     $31 = (($30) - 1)|0;
     $32 = HEAP32[$2>>2]|0;
     HEAP32[$32>>2] = $31;
     HEAP32[$0>>2] = 0;
     $41 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($41|0);
    }
   }
   $33 = HEAP32[$p>>2]|0;
   $34 = (_handleDeferredMoveto($33)|0);
   HEAP32[$0>>2] = $34;
   $41 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($41|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $41 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($41|0);
}
function _sqlite3BtreePayloadFetch($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_fetchPayload($2,$3)|0);
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeMemFromBtree($pCur,$offset,$amt,$key,$pMem) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $key = $key|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $available = 0, $rc = 0, $zData = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $zData = sp + 8|0;
 $available = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $key;
 HEAP32[$4>>2] = $pMem;
 HEAP32[$available>>2] = 0;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3BtreePayloadFetch($5,$available)|0);
 HEAP32[$zData>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($7) + ($8))|0;
 $10 = HEAP32[$available>>2]|0;
 $11 = ($9>>>0)<=($10>>>0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = HEAP32[$zData>>2]|0;
  $14 = (($13) + ($12)|0);
  $15 = HEAP32[$4>>2]|0;
  $16 = ((($15)) + 16|0);
  HEAP32[$16>>2] = $14;
  $17 = HEAP32[$4>>2]|0;
  $18 = ((($17)) + 8|0);
  HEAP16[$18>>1] = 4112;
  $19 = HEAP32[$2>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = ((($20)) + 12|0);
  HEAP32[$21>>2] = $19;
  $28 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($28|0);
 } else {
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$3>>2]|0;
  $26 = HEAP32[$4>>2]|0;
  $27 = (_vdbeMemFromBtreeResize($22,$23,$24,$25,$26)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _sqlite3VdbeOneByteSerialTypeLen($serial_type) {
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP8[$0>>0] = $serial_type;
 $1 = HEAP8[$0>>0]|0;
 $2 = $1&255;
 $3 = (35061 + ($2)|0);
 $4 = HEAP8[$3>>0]|0;
 STACKTOP = sp;return ($4|0);
}
function _sqlite3VdbeSerialType($pMem,$file_format,$pLen) {
 $pMem = $pMem|0;
 $file_format = $file_format|0;
 $pLen = $pLen|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $flags = 0;
 var $i = 0, $n = 0, $or$cond = 0, $u = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $flags = sp + 20|0;
 $n = sp + 16|0;
 $i = sp + 8|0;
 $u = sp;
 HEAP32[$1>>2] = $pMem;
 HEAP32[$2>>2] = $file_format;
 HEAP32[$3>>2] = $pLen;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP16[$5>>1]|0;
 $7 = $6&65535;
 HEAP32[$flags>>2] = $7;
 $8 = HEAP32[$flags>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$3>>2]|0;
  HEAP32[$11>>2] = 0;
  HEAP32[$0>>2] = 0;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $12 = HEAP32[$flags>>2]|0;
 $13 = $12 & 4;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $134 = HEAP32[$flags>>2]|0;
  $135 = $134 & 8;
  $136 = ($135|0)!=(0);
  if ($136) {
   $137 = HEAP32[$3>>2]|0;
   HEAP32[$137>>2] = 8;
   HEAP32[$0>>2] = 7;
   $158 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($158|0);
  }
  $138 = HEAP32[$1>>2]|0;
  $139 = ((($138)) + 12|0);
  $140 = HEAP32[$139>>2]|0;
  HEAP32[$n>>2] = $140;
  $141 = HEAP32[$flags>>2]|0;
  $142 = $141 & 16384;
  $143 = ($142|0)!=(0);
  if ($143) {
   $144 = HEAP32[$1>>2]|0;
   $145 = HEAP32[$144>>2]|0;
   $146 = HEAP32[$n>>2]|0;
   $147 = (($146) + ($145))|0;
   HEAP32[$n>>2] = $147;
  }
  $148 = HEAP32[$n>>2]|0;
  $149 = HEAP32[$3>>2]|0;
  HEAP32[$149>>2] = $148;
  $150 = HEAP32[$n>>2]|0;
  $151 = $150<<1;
  $152 = (($151) + 12)|0;
  $153 = HEAP32[$flags>>2]|0;
  $154 = $153 & 2;
  $155 = ($154|0)!=(0);
  $156 = $155&1;
  $157 = (($152) + ($156))|0;
  HEAP32[$0>>2] = $157;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = $15;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = $i;
 $23 = $22;
 HEAP32[$23>>2] = $18;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = $i;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = ($31|0)<(0);
 $33 = $i;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 if ($32) {
  $39 = $35 ^ -1;
  $40 = $38 ^ -1;
  $41 = $u;
  $42 = $41;
  HEAP32[$42>>2] = $39;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $40;
 } else {
  $45 = $u;
  $46 = $45;
  HEAP32[$46>>2] = $35;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $38;
 }
 $49 = $u;
 $50 = $49;
 $51 = HEAP32[$50>>2]|0;
 $52 = (($49) + 4)|0;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = ($54>>>0)<(0);
 $56 = ($51>>>0)<=(127);
 $57 = ($54|0)==(0);
 $58 = $57 & $56;
 $59 = $55 | $58;
 if ($59) {
  $60 = $i;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (($60) + 4)|0;
  $64 = $63;
  $65 = HEAP32[$64>>2]|0;
  $66 = $62 & 1;
  $67 = $i;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = (($67) + 4)|0;
  $71 = $70;
  $72 = HEAP32[$71>>2]|0;
  $73 = ($66|0)==($69|0);
  $74 = (0)==($72|0);
  $75 = $73 & $74;
  $76 = HEAP32[$2>>2]|0;
  $77 = ($76|0)>=(4);
  $or$cond = $75 & $77;
  $78 = HEAP32[$3>>2]|0;
  if ($or$cond) {
   HEAP32[$78>>2] = 0;
   $79 = $u;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (($79) + 4)|0;
   $83 = $82;
   $84 = HEAP32[$83>>2]|0;
   $85 = (8 + ($81))|0;
   HEAP32[$0>>2] = $85;
   $158 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($158|0);
  } else {
   HEAP32[$78>>2] = 1;
   HEAP32[$0>>2] = 1;
   $158 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($158|0);
  }
 }
 $86 = $u;
 $87 = $86;
 $88 = HEAP32[$87>>2]|0;
 $89 = (($86) + 4)|0;
 $90 = $89;
 $91 = HEAP32[$90>>2]|0;
 $92 = ($91>>>0)<(0);
 $93 = ($88>>>0)<=(32767);
 $94 = ($91|0)==(0);
 $95 = $94 & $93;
 $96 = $92 | $95;
 if ($96) {
  $97 = HEAP32[$3>>2]|0;
  HEAP32[$97>>2] = 2;
  HEAP32[$0>>2] = 2;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $98 = $u;
 $99 = $98;
 $100 = HEAP32[$99>>2]|0;
 $101 = (($98) + 4)|0;
 $102 = $101;
 $103 = HEAP32[$102>>2]|0;
 $104 = ($103>>>0)<(0);
 $105 = ($100>>>0)<=(8388607);
 $106 = ($103|0)==(0);
 $107 = $106 & $105;
 $108 = $104 | $107;
 if ($108) {
  $109 = HEAP32[$3>>2]|0;
  HEAP32[$109>>2] = 3;
  HEAP32[$0>>2] = 3;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $110 = $u;
 $111 = $110;
 $112 = HEAP32[$111>>2]|0;
 $113 = (($110) + 4)|0;
 $114 = $113;
 $115 = HEAP32[$114>>2]|0;
 $116 = ($115>>>0)<(0);
 $117 = ($112>>>0)<=(2147483647);
 $118 = ($115|0)==(0);
 $119 = $118 & $117;
 $120 = $116 | $119;
 if ($120) {
  $121 = HEAP32[$3>>2]|0;
  HEAP32[$121>>2] = 4;
  HEAP32[$0>>2] = 4;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 $122 = $u;
 $123 = $122;
 $124 = HEAP32[$123>>2]|0;
 $125 = (($122) + 4)|0;
 $126 = $125;
 $127 = HEAP32[$126>>2]|0;
 $128 = ($127>>>0)<(32767);
 $129 = ($124>>>0)<=(4294967295);
 $130 = ($127|0)==(32767);
 $131 = $130 & $129;
 $132 = $128 | $131;
 $133 = HEAP32[$3>>2]|0;
 if ($132) {
  HEAP32[$133>>2] = 6;
  HEAP32[$0>>2] = 5;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 } else {
  HEAP32[$133>>2] = 8;
  HEAP32[$0>>2] = 6;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 return (0)|0;
}
function _sqlite3PutVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14>>>0)<(0);
 $16 = ($11>>>0)<=(127);
 $17 = ($14|0)==(0);
 $18 = $17 & $16;
 $19 = $15 | $18;
 $20 = $4;
 $21 = $20;
 $22 = HEAP32[$21>>2]|0;
 $23 = (($20) + 4)|0;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 if ($19) {
  $26 = $22 & 127;
  $27 = $26&255;
  $28 = HEAP32[$3>>2]|0;
  HEAP8[$28>>0] = $27;
  HEAP32[$2>>2] = 1;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $29 = ($25>>>0)<(0);
 $30 = ($22>>>0)<=(16383);
 $31 = ($25|0)==(0);
 $32 = $31 & $30;
 $33 = $29 | $32;
 if ($33) {
  $34 = $4;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  $37 = (($34) + 4)|0;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (_bitshift64Lshr(($36|0),($39|0),7)|0);
  $41 = tempRet0;
  $42 = $40 & 127;
  $43 = $42 | 128;
  $44 = $43&255;
  $45 = HEAP32[$3>>2]|0;
  HEAP8[$45>>0] = $44;
  $46 = $4;
  $47 = $46;
  $48 = HEAP32[$47>>2]|0;
  $49 = (($46) + 4)|0;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = $48 & 127;
  $53 = $52&255;
  $54 = HEAP32[$3>>2]|0;
  $55 = ((($54)) + 1|0);
  HEAP8[$55>>0] = $53;
  HEAP32[$2>>2] = 2;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  $56 = HEAP32[$3>>2]|0;
  $57 = $4;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_putVarint64($56,$59,$62)|0);
  HEAP32[$2>>2] = $63;
  $64 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSerialPut($buf,$pMem,$serial_type) {
 $buf = $buf|0;
 $pMem = $pMem|0;
 $serial_type = $serial_type|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $len = 0, $or$cond = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $len = sp + 12|0;
 $v = sp;
 $i = sp + 8|0;
 HEAP32[$1>>2] = $buf;
 HEAP32[$2>>2] = $pMem;
 HEAP32[$3>>2] = $serial_type;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4>>>0)<=(7);
 $6 = HEAP32[$3>>2]|0;
 $7 = ($6>>>0)>(0);
 $or$cond = $5 & $7;
 $8 = HEAP32[$3>>2]|0;
 if ($or$cond) {
  $9 = ($8|0)==(7);
  if ($9) {
   $10 = HEAP32[$2>>2]|0;
   ;HEAP32[$v>>2]=HEAP32[$10>>2]|0;HEAP32[$v+4>>2]=HEAP32[$10+4>>2]|0;
  } else {
   $11 = HEAP32[$2>>2]|0;
   $12 = $11;
   $13 = $12;
   $14 = HEAP32[$13>>2]|0;
   $15 = (($12) + 4)|0;
   $16 = $15;
   $17 = HEAP32[$16>>2]|0;
   $18 = $v;
   $19 = $18;
   HEAP32[$19>>2] = $14;
   $20 = (($18) + 4)|0;
   $21 = $20;
   HEAP32[$21>>2] = $17;
  }
  $22 = HEAP32[$3>>2]|0;
  $23 = (35061 + ($22)|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  HEAP32[$i>>2] = $25;
  HEAP32[$len>>2] = $25;
  while(1) {
   $26 = $v;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (($26) + 4)|0;
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = $28 & 255;
   $33 = $32&255;
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + -1)|0;
   HEAP32[$i>>2] = $35;
   $36 = HEAP32[$1>>2]|0;
   $37 = (($36) + ($35)|0);
   HEAP8[$37>>0] = $33;
   $38 = $v;
   $39 = $38;
   $40 = HEAP32[$39>>2]|0;
   $41 = (($38) + 4)|0;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (_bitshift64Lshr(($40|0),($43|0),8)|0);
   $45 = tempRet0;
   $46 = $v;
   $47 = $46;
   HEAP32[$47>>2] = $44;
   $48 = (($46) + 4)|0;
   $49 = $48;
   HEAP32[$49>>2] = $45;
   $50 = HEAP32[$i>>2]|0;
   $51 = ($50|0)!=(0);
   if (!($51)) {
    break;
   }
  }
  $52 = HEAP32[$len>>2]|0;
  HEAP32[$0>>2] = $52;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 } else {
  $53 = ($8>>>0)>=(12);
  if (!($53)) {
   HEAP32[$0>>2] = 0;
   $65 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($65|0);
  }
  $54 = HEAP32[$2>>2]|0;
  $55 = ((($54)) + 12|0);
  $56 = HEAP32[$55>>2]|0;
  HEAP32[$len>>2] = $56;
  $57 = HEAP32[$len>>2]|0;
  $58 = ($57>>>0)>(0);
  if ($58) {
   $59 = HEAP32[$1>>2]|0;
   $60 = HEAP32[$2>>2]|0;
   $61 = ((($60)) + 16|0);
   $62 = HEAP32[$61>>2]|0;
   $63 = HEAP32[$len>>2]|0;
   _memcpy(($59|0),($62|0),($63|0))|0;
  }
  $64 = HEAP32[$len>>2]|0;
  HEAP32[$0>>2] = $64;
  $65 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($65|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCount($pCur,$pnEntry) {
 $pCur = $pCur|0;
 $pnEntry = $pnEntry|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iIdx = 0, $nEntry = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $nEntry = sp;
 $rc = sp + 16|0;
 $iIdx = sp + 12|0;
 $pPage = sp + 8|0;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pnEntry;
 $3 = $nEntry;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = (($3) + 4)|0;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0);
 if ($10) {
  $11 = HEAP32[$2>>2]|0;
  $12 = $11;
  $13 = $12;
  HEAP32[$13>>2] = 0;
  $14 = (($12) + 4)|0;
  $15 = $14;
  HEAP32[$15>>2] = 0;
  HEAP32[$0>>2] = 0;
  $171 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($171|0);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = (_moveToRoot($16)|0);
 HEAP32[$rc>>2] = $17;
 L5: while(1) {
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)==(0);
  if (!($19)) {
   label = 16;
   break;
  }
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 68|0);
  $22 = HEAP8[$21>>0]|0;
  $23 = $22 << 24 >> 24;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 120|0);
  $26 = (($25) + ($23<<2)|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$pPage>>2] = $27;
  $28 = HEAP32[$pPage>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = $30&255;
  $32 = ($31|0)!=(0);
  if ($32) {
   label = 7;
  } else {
   $33 = HEAP32[$pPage>>2]|0;
   $34 = ((($33)) + 2|0);
   $35 = HEAP8[$34>>0]|0;
   $36 = ($35<<24>>24)!=(0);
   if (!($36)) {
    label = 7;
   }
  }
  if ((label|0) == 7) {
   label = 0;
   $37 = HEAP32[$pPage>>2]|0;
   $38 = ((($37)) + 18|0);
   $39 = HEAP16[$38>>1]|0;
   $40 = $39&65535;
   $41 = $nEntry;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($41) + 4)|0;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = (_i64Add(($43|0),($46|0),($40|0),0)|0);
   $48 = tempRet0;
   $49 = $nEntry;
   $50 = $49;
   HEAP32[$50>>2] = $47;
   $51 = (($49) + 4)|0;
   $52 = $51;
   HEAP32[$52>>2] = $48;
  }
  $53 = HEAP32[$pPage>>2]|0;
  $54 = ((($53)) + 4|0);
  $55 = HEAP8[$54>>0]|0;
  $56 = ($55<<24>>24)!=(0);
  if ($56) {
   while(1) {
    $57 = HEAP32[$1>>2]|0;
    $58 = ((($57)) + 68|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59 << 24 >> 24;
    $61 = ($60|0)==(0);
    if ($61) {
     label = 10;
     break L5;
    }
    $75 = HEAP32[$1>>2]|0;
    _moveToParent($75);
    $76 = HEAP32[$1>>2]|0;
    $77 = ((($76)) + 68|0);
    $78 = HEAP8[$77>>0]|0;
    $79 = $78 << 24 >> 24;
    $80 = HEAP32[$1>>2]|0;
    $81 = ((($80)) + 80|0);
    $82 = (($81) + ($79<<1)|0);
    $83 = HEAP16[$82>>1]|0;
    $84 = $83&65535;
    $85 = HEAP32[$1>>2]|0;
    $86 = ((($85)) + 68|0);
    $87 = HEAP8[$86>>0]|0;
    $88 = $87 << 24 >> 24;
    $89 = HEAP32[$1>>2]|0;
    $90 = ((($89)) + 120|0);
    $91 = (($90) + ($88<<2)|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ((($92)) + 18|0);
    $94 = HEAP16[$93>>1]|0;
    $95 = $94&65535;
    $96 = ($84|0)>=($95|0);
    if (!($96)) {
     break;
    }
   }
   $97 = HEAP32[$1>>2]|0;
   $98 = ((($97)) + 68|0);
   $99 = HEAP8[$98>>0]|0;
   $100 = $99 << 24 >> 24;
   $101 = HEAP32[$1>>2]|0;
   $102 = ((($101)) + 80|0);
   $103 = (($102) + ($100<<1)|0);
   $104 = HEAP16[$103>>1]|0;
   $105 = (($104) + 1)<<16>>16;
   HEAP16[$103>>1] = $105;
   $106 = HEAP32[$1>>2]|0;
   $107 = ((($106)) + 68|0);
   $108 = HEAP8[$107>>0]|0;
   $109 = $108 << 24 >> 24;
   $110 = HEAP32[$1>>2]|0;
   $111 = ((($110)) + 120|0);
   $112 = (($111) + ($109<<2)|0);
   $113 = HEAP32[$112>>2]|0;
   HEAP32[$pPage>>2] = $113;
  }
  $114 = HEAP32[$1>>2]|0;
  $115 = ((($114)) + 68|0);
  $116 = HEAP8[$115>>0]|0;
  $117 = $116 << 24 >> 24;
  $118 = HEAP32[$1>>2]|0;
  $119 = ((($118)) + 80|0);
  $120 = (($119) + ($117<<1)|0);
  $121 = HEAP16[$120>>1]|0;
  $122 = $121&65535;
  HEAP32[$iIdx>>2] = $122;
  $123 = HEAP32[$iIdx>>2]|0;
  $124 = HEAP32[$pPage>>2]|0;
  $125 = ((($124)) + 18|0);
  $126 = HEAP16[$125>>1]|0;
  $127 = $126&65535;
  $128 = ($123|0)==($127|0);
  $129 = HEAP32[$1>>2]|0;
  $130 = HEAP32[$pPage>>2]|0;
  if ($128) {
   $131 = ((($130)) + 5|0);
   $132 = HEAP8[$131>>0]|0;
   $133 = $132&255;
   $134 = (($133) + 8)|0;
   $135 = HEAP32[$pPage>>2]|0;
   $136 = ((($135)) + 56|0);
   $137 = HEAP32[$136>>2]|0;
   $138 = (($137) + ($134)|0);
   $139 = (_sqlite3Get4byte($138)|0);
   $140 = (_moveToChild($129,$139)|0);
   HEAP32[$rc>>2] = $140;
   continue;
  } else {
   $141 = ((($130)) + 56|0);
   $142 = HEAP32[$141>>2]|0;
   $143 = HEAP32[$pPage>>2]|0;
   $144 = ((($143)) + 20|0);
   $145 = HEAP16[$144>>1]|0;
   $146 = $145&65535;
   $147 = HEAP32[$iIdx>>2]|0;
   $148 = $147<<1;
   $149 = HEAP32[$pPage>>2]|0;
   $150 = ((($149)) + 64|0);
   $151 = HEAP32[$150>>2]|0;
   $152 = (($151) + ($148)|0);
   $153 = HEAP8[$152>>0]|0;
   $154 = $153&255;
   $155 = $154 << 8;
   $156 = HEAP32[$iIdx>>2]|0;
   $157 = $156<<1;
   $158 = HEAP32[$pPage>>2]|0;
   $159 = ((($158)) + 64|0);
   $160 = HEAP32[$159>>2]|0;
   $161 = (($160) + ($157)|0);
   $162 = ((($161)) + 1|0);
   $163 = HEAP8[$162>>0]|0;
   $164 = $163&255;
   $165 = $155 | $164;
   $166 = $146 & $165;
   $167 = (($142) + ($166)|0);
   $168 = (_sqlite3Get4byte($167)|0);
   $169 = (_moveToChild($129,$168)|0);
   HEAP32[$rc>>2] = $169;
   continue;
  }
 }
 if ((label|0) == 10) {
  $62 = $nEntry;
  $63 = $62;
  $64 = HEAP32[$63>>2]|0;
  $65 = (($62) + 4)|0;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $68 = HEAP32[$2>>2]|0;
  $69 = $68;
  $70 = $69;
  HEAP32[$70>>2] = $64;
  $71 = (($69) + 4)|0;
  $72 = $71;
  HEAP32[$72>>2] = $67;
  $73 = HEAP32[$1>>2]|0;
  $74 = (_moveToRoot($73)|0);
  HEAP32[$0>>2] = $74;
  $171 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($171|0);
 }
 else if ((label|0) == 16) {
  $170 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $170;
  $171 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($171|0);
 }
 return (0)|0;
}
function _sqlite3VtabSavepoint($db,$op,$iSavepoint) {
 $db = $db|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $i = 0, $pMod = 0, $pVTab = 0, $rc = 0, $xMethod = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $i = sp + 12|0;
 $pVTab = sp + 8|0;
 $pMod = sp + 4|0;
 $xMethod = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 340|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 15;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 316|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   label = 15;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 340|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($14<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVTab>>2] = $19;
  $20 = HEAP32[$pVTab>>2]|0;
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pMod>>2] = $23;
  $24 = HEAP32[$pVTab>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)!=(0|0);
  if ($27) {
   $28 = HEAP32[$pMod>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ($29|0)>=(2);
   if ($30) {
    $31 = HEAP32[$1>>2]|0;
    switch ($31|0) {
    case 0:  {
     $32 = HEAP32[$pMod>>2]|0;
     $33 = ((($32)) + 80|0);
     $34 = HEAP32[$33>>2]|0;
     HEAP32[$xMethod>>2] = $34;
     $35 = HEAP32[$2>>2]|0;
     $36 = (($35) + 1)|0;
     $37 = HEAP32[$pVTab>>2]|0;
     $38 = ((($37)) + 20|0);
     HEAP32[$38>>2] = $36;
     break;
    }
    case 2:  {
     $39 = HEAP32[$pMod>>2]|0;
     $40 = ((($39)) + 88|0);
     $41 = HEAP32[$40>>2]|0;
     HEAP32[$xMethod>>2] = $41;
     break;
    }
    default: {
     $42 = HEAP32[$pMod>>2]|0;
     $43 = ((($42)) + 84|0);
     $44 = HEAP32[$43>>2]|0;
     HEAP32[$xMethod>>2] = $44;
    }
    }
    $45 = HEAP32[$xMethod>>2]|0;
    $46 = ($45|0)!=(0|0);
    if ($46) {
     $47 = HEAP32[$pVTab>>2]|0;
     $48 = ((($47)) + 20|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = HEAP32[$2>>2]|0;
     $51 = ($49|0)>($50|0);
     if ($51) {
      $52 = HEAP32[$xMethod>>2]|0;
      $53 = HEAP32[$pVTab>>2]|0;
      $54 = ((($53)) + 8|0);
      $55 = HEAP32[$54>>2]|0;
      $56 = HEAP32[$2>>2]|0;
      $57 = (FUNCTION_TABLE_iii[$52 & 255]($55,$56)|0);
      HEAP32[$rc>>2] = $57;
     }
    }
   }
  }
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
 }
 if ((label|0) == 15) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 return (0)|0;
}
function _sqlite3BtreeTripAllCursors($pBtree,$errCode,$writeOnly) {
 $pBtree = $pBtree|0;
 $errCode = $errCode|0;
 $writeOnly = $writeOnly|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pBtree;
 HEAP32[$1>>2] = $errCode;
 HEAP32[$2>>2] = $writeOnly;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$p>>2] = $10;
 L4: while(1) {
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  if (!($12)) {
   label = 15;
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)!=(0);
  do {
   if ($14) {
    $15 = HEAP32[$p>>2]|0;
    $16 = ((($15)) + 64|0);
    $17 = HEAP8[$16>>0]|0;
    $18 = $17&255;
    $19 = $18 & 1;
    $20 = ($19|0)==(0);
    if ($20) {
     $21 = HEAP32[$p>>2]|0;
     $22 = ((($21)) + 66|0);
     $23 = HEAP8[$22>>0]|0;
     $24 = $23&255;
     $25 = ($24|0)==(1);
     if (!($25)) {
      $26 = HEAP32[$p>>2]|0;
      $27 = ((($26)) + 66|0);
      $28 = HEAP8[$27>>0]|0;
      $29 = $28&255;
      $30 = ($29|0)==(2);
      if (!($30)) {
       break;
      }
     }
     $31 = HEAP32[$p>>2]|0;
     $32 = (_saveCursorPosition($31)|0);
     HEAP32[$rc>>2] = $32;
     $33 = HEAP32[$rc>>2]|0;
     $34 = ($33|0)!=(0);
     if ($34) {
      break L4;
     }
    } else {
     label = 10;
    }
   } else {
    label = 10;
   }
  } while(0);
  if ((label|0) == 10) {
   label = 0;
   $37 = HEAP32[$p>>2]|0;
   _sqlite3BtreeClearCursor($37);
   $38 = HEAP32[$p>>2]|0;
   $39 = ((($38)) + 66|0);
   HEAP8[$39>>0] = 4;
   $40 = HEAP32[$1>>2]|0;
   $41 = HEAP32[$p>>2]|0;
   $42 = ((($41)) + 60|0);
   HEAP32[$42>>2] = $40;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $43 = HEAP32[$i>>2]|0;
   $44 = HEAP32[$p>>2]|0;
   $45 = ((($44)) + 68|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = $46 << 24 >> 24;
   $48 = ($43|0)<=($47|0);
   if (!($48)) {
    break;
   }
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$p>>2]|0;
   $51 = ((($50)) + 120|0);
   $52 = (($51) + ($49<<2)|0);
   $53 = HEAP32[$52>>2]|0;
   _releasePage($53);
   $54 = HEAP32[$i>>2]|0;
   $55 = HEAP32[$p>>2]|0;
   $56 = ((($55)) + 120|0);
   $57 = (($56) + ($54<<2)|0);
   HEAP32[$57>>2] = 0;
   $58 = HEAP32[$i>>2]|0;
   $59 = (($58) + 1)|0;
   HEAP32[$i>>2] = $59;
  }
  $60 = HEAP32[$p>>2]|0;
  $61 = ((($60)) + 8|0);
  $62 = HEAP32[$61>>2]|0;
  HEAP32[$p>>2] = $62;
 }
 if ((label|0) == 15) {
  $63 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$rc>>2]|0;
 (_sqlite3BtreeTripAllCursors($35,$36,0)|0);
 $63 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _sqlite3BtreeSavepoint($p,$op,$iSavepoint) {
 $p = $p|0;
 $op = $op|0;
 $iSavepoint = $iSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $op;
 HEAP32[$2>>2] = $iSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if (!($9)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pBt>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($13);
 $14 = HEAP32[$pBt>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = HEAP32[$2>>2]|0;
 $18 = (_sqlite3PagerSavepoint($15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)==(0);
 if (!($20)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = ($21|0)<(0);
 if ($22) {
  $23 = HEAP32[$pBt>>2]|0;
  $24 = ((($23)) + 22|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = $26 & 8;
  $28 = ($27|0)!=(0);
  if ($28) {
   $29 = HEAP32[$pBt>>2]|0;
   $30 = ((($29)) + 44|0);
   HEAP32[$30>>2] = 0;
  }
 }
 $31 = HEAP32[$pBt>>2]|0;
 $32 = (_newDatabase($31)|0);
 HEAP32[$rc>>2] = $32;
 $33 = HEAP32[$pBt>>2]|0;
 $34 = ((($33)) + 12|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ((($35)) + 56|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = ((($37)) + 28|0);
 $39 = (_sqlite3Get4byte($38)|0);
 $40 = HEAP32[$pBt>>2]|0;
 $41 = ((($40)) + 44|0);
 HEAP32[$41>>2] = $39;
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3RollbackAll($db,$tripCode) {
 $db = $db|0;
 $tripCode = $tripCode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $i = 0, $inTrans = 0, $p = 0, $schemaChange = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $i = sp + 12|0;
 $inTrans = sp + 8|0;
 $schemaChange = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $tripCode;
 HEAP32[$inTrans>>2] = 0;
 _sqlite3BeginBenignMalloc();
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnterAll($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 2;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 148|0);
  $10 = ((($9)) + 5|0);
  $11 = HEAP8[$10>>0]|0;
  $12 = $11&255;
  $13 = ($12|0)==(0);
  $15 = $13;
 } else {
  $15 = 0;
 }
 $14 = $15&1;
 HEAP32[$schemaChange>>2] = $14;
 HEAP32[$i>>2] = 0;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 20|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($16|0)<($19|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 16|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = (($24) + ($21<<4)|0);
  $26 = ((($25)) + 4|0);
  $27 = HEAP32[$26>>2]|0;
  HEAP32[$p>>2] = $27;
  $28 = HEAP32[$p>>2]|0;
  $29 = ($28|0)!=(0|0);
  if ($29) {
   $30 = HEAP32[$p>>2]|0;
   $31 = (_sqlite3BtreeIsInTrans($30)|0);
   $32 = ($31|0)!=(0);
   if ($32) {
    HEAP32[$inTrans>>2] = 1;
   }
   $33 = HEAP32[$p>>2]|0;
   $34 = HEAP32[$1>>2]|0;
   $35 = HEAP32[$schemaChange>>2]|0;
   $36 = ($35|0)!=(0);
   $37 = $36 ^ 1;
   $38 = $37&1;
   (_sqlite3BtreeRollback($33,$34,$38)|0);
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = (($39) + 1)|0;
  HEAP32[$i>>2] = $40;
 }
 $41 = HEAP32[$0>>2]|0;
 (_sqlite3VtabRollback($41)|0);
 _sqlite3EndBenignMalloc();
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 24|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = $44 & 2;
 $46 = ($45|0)!=(0);
 if ($46) {
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 148|0);
  $49 = ((($48)) + 5|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = ($51|0)==(0);
  if ($52) {
   $53 = HEAP32[$0>>2]|0;
   _sqlite3ExpirePreparedStatements($53);
   $54 = HEAP32[$0>>2]|0;
   _sqlite3ResetAllSchemasOfConnection($54);
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 440|0);
 $57 = $56;
 $58 = $57;
 HEAP32[$58>>2] = 0;
 $59 = (($57) + 4)|0;
 $60 = $59;
 HEAP32[$60>>2] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 448|0);
 $63 = $62;
 $64 = $63;
 HEAP32[$64>>2] = 0;
 $65 = (($63) + 4)|0;
 $66 = $65;
 HEAP32[$66>>2] = 0;
 $67 = HEAP32[$0>>2]|0;
 $68 = ((($67)) + 24|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = $69 & -33554433;
 HEAP32[$68>>2] = $70;
 $71 = HEAP32[$0>>2]|0;
 $72 = ((($71)) + 212|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = ($73|0)!=(0|0);
 if (!($74)) {
  STACKTOP = sp;return;
 }
 $75 = HEAP32[$inTrans>>2]|0;
 $76 = ($75|0)!=(0);
 if (!($76)) {
  $77 = HEAP32[$0>>2]|0;
  $78 = ((($77)) + 67|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = ($79<<24>>24)!=(0);
  if ($80) {
   STACKTOP = sp;return;
  }
 }
 $81 = HEAP32[$0>>2]|0;
 $82 = ((($81)) + 212|0);
 $83 = HEAP32[$82>>2]|0;
 $84 = HEAP32[$0>>2]|0;
 $85 = ((($84)) + 208|0);
 $86 = HEAP32[$85>>2]|0;
 FUNCTION_TABLE_vi[$83 & 255]($86);
 STACKTOP = sp;return;
}
function _sqlite3CloseSavepoints($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $pTmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pTmp = sp;
 HEAP32[$0>>2] = $db;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ((($1)) + 424|0);
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)!=(0|0);
  $5 = HEAP32[$0>>2]|0;
  if (!($4)) {
   break;
  }
  $6 = ((($5)) + 424|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pTmp>>2] = $7;
  $8 = HEAP32[$pTmp>>2]|0;
  $9 = ((($8)) + 24|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 424|0);
  HEAP32[$12>>2] = $10;
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$pTmp>>2]|0;
  _sqlite3DbFree($13,$14);
 }
 $15 = ((($5)) + 432|0);
 HEAP32[$15>>2] = 0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 436|0);
 HEAP32[$17>>2] = 0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 75|0);
 HEAP8[$19>>0] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeBeginStmt($p,$iStatement) {
 $p = $p|0;
 $iStatement = $iStatement|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iStatement;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pBt>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($5);
 $6 = HEAP32[$pBt>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_sqlite3PagerOpenSavepoint($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _allocateCursor($p,$iCur,$nField,$iDb,$eCurType) {
 $p = $p|0;
 $iCur = $iCur|0;
 $nField = $nField|0;
 $iDb = $iDb|0;
 $eCurType = $eCurType|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $nByte = 0, $pCx = 0, $pMem = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 28|0;
 $pMem = sp + 8|0;
 $nByte = sp + 4|0;
 $pCx = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iCur;
 HEAP32[$2>>2] = $nField;
 HEAP32[$3>>2] = $iDb;
 HEAP8[$4>>0] = $eCurType;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)>(0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = ((($7)) + 24|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = (($9) - ($10))|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 92|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + (($11*40)|0)|0);
  $18 = $15;
 } else {
  $16 = ((($7)) + 92|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = $17;
 }
 HEAP32[$pMem>>2] = $18;
 HEAP32[$pCx>>2] = 0;
 $19 = HEAP32[$2>>2]|0;
 $20 = $19<<3;
 $21 = (88 + ($20))|0;
 $22 = HEAP8[$4>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)==(0);
 if ($24) {
  $25 = (_sqlite3BtreeCursorSize()|0);
  $27 = $25;
 } else {
  $27 = 0;
 }
 $26 = (($21) + ($27))|0;
 HEAP32[$nByte>>2] = $26;
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 112|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = (($31) + ($28<<2)|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0|0);
 if ($34) {
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = HEAP32[$0>>2]|0;
  $38 = ((($37)) + 112|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = (($39) + ($36<<2)|0);
  $41 = HEAP32[$40>>2]|0;
  _sqlite3VdbeFreeCursor($35,$41);
  $42 = HEAP32[$1>>2]|0;
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 112|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + ($42<<2)|0);
  HEAP32[$46>>2] = 0;
 }
 $47 = HEAP32[$pMem>>2]|0;
 $48 = HEAP32[$nByte>>2]|0;
 $49 = (_sqlite3VdbeMemClearAndResize($47,$48)|0);
 $50 = (0)==($49|0);
 if (!($50)) {
  $91 = HEAP32[$pCx>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $51 = HEAP32[$pMem>>2]|0;
 $52 = ((($51)) + 16|0);
 $53 = HEAP32[$52>>2]|0;
 HEAP32[$pCx>>2] = $53;
 $54 = HEAP32[$1>>2]|0;
 $55 = HEAP32[$0>>2]|0;
 $56 = ((($55)) + 112|0);
 $57 = HEAP32[$56>>2]|0;
 $58 = (($57) + ($54<<2)|0);
 HEAP32[$58>>2] = $53;
 $59 = HEAP32[$pCx>>2]|0;
 dest=$59; stop=dest+88|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $60 = HEAP8[$4>>0]|0;
 $61 = HEAP32[$pCx>>2]|0;
 HEAP8[$61>>0] = $60;
 $62 = HEAP32[$3>>2]|0;
 $63 = $62&255;
 $64 = HEAP32[$pCx>>2]|0;
 $65 = ((($64)) + 1|0);
 HEAP8[$65>>0] = $63;
 $66 = HEAP32[$2>>2]|0;
 $67 = $66&65535;
 $68 = HEAP32[$pCx>>2]|0;
 $69 = ((($68)) + 12|0);
 HEAP16[$69>>1] = $67;
 $70 = HEAP32[$2>>2]|0;
 $71 = HEAP32[$pCx>>2]|0;
 $72 = ((($71)) + 80|0);
 $73 = (($72) + ($70<<2)|0);
 $74 = HEAP32[$pCx>>2]|0;
 $75 = ((($74)) + 76|0);
 HEAP32[$75>>2] = $73;
 $76 = HEAP8[$4>>0]|0;
 $77 = $76&255;
 $78 = ($77|0)==(0);
 if (!($78)) {
  $91 = HEAP32[$pCx>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $79 = HEAP32[$2>>2]|0;
 $80 = $79<<3;
 $81 = (88 + ($80))|0;
 $82 = HEAP32[$pMem>>2]|0;
 $83 = ((($82)) + 16|0);
 $84 = HEAP32[$83>>2]|0;
 $85 = (($84) + ($81)|0);
 $86 = HEAP32[$pCx>>2]|0;
 $87 = ((($86)) + 16|0);
 HEAP32[$87>>2] = $85;
 $88 = HEAP32[$pCx>>2]|0;
 $89 = ((($88)) + 16|0);
 $90 = HEAP32[$89>>2]|0;
 _sqlite3BtreeCursorZero($90);
 $91 = HEAP32[$pCx>>2]|0;
 STACKTOP = sp;return ($91|0);
}
function _sqlite3BtreeCursor($p,$iTable,$wrFlag,$pKeyInfo,$pCur) {
 $p = $p|0;
 $iTable = $iTable|0;
 $wrFlag = $wrFlag|0;
 $pKeyInfo = $pKeyInfo|0;
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $wrFlag;
 HEAP32[$3>>2] = $pKeyInfo;
 HEAP32[$4>>2] = $pCur;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)<(1);
 if ($6) {
  $7 = (_sqlite3CorruptError(62410)|0);
  HEAP32[$rc>>2] = $7;
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 } else {
  $8 = HEAP32[$0>>2]|0;
  _sqlite3BtreeEnter($8);
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = HEAP32[$3>>2]|0;
  $13 = HEAP32[$4>>2]|0;
  $14 = (_btreeCursor($9,$10,$11,$12,$13)|0);
  HEAP32[$rc>>2] = $14;
  $15 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($15|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCursorHintFlags($pCur,$x) {
 $pCur = $pCur|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2&255;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 67|0);
 HEAP8[$5>>0] = $3;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCreateTable($p,$piTable,$flags) {
 $p = $p|0;
 $piTable = $piTable|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piTable;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_btreeCreateTable($4,$5,$6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3VdbeSorterInit($db,$nField,$pCsr) {
 $db = $db|0;
 $nField = $nField|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $mxCache = 0, $pKeyInfo = 0, $pSorter = 0, $pTask = 0, $pgsz = 0, $rc = 0, $sz = 0, $szKeyInfo = 0, $szPma = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $pgsz = sp + 40|0;
 $i = sp + 36|0;
 $pSorter = sp + 32|0;
 $pKeyInfo = sp + 28|0;
 $szKeyInfo = sp + 24|0;
 $sz = sp + 20|0;
 $rc = sp + 16|0;
 $pTask = sp + 12|0;
 $mxCache = sp;
 $szPma = sp + 8|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $nField;
 HEAP32[$2>>2] = $pCsr;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 6|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = $7&65535;
 $9 = (($8) - 1)|0;
 $10 = $9<<2;
 $11 = (24 + ($10))|0;
 HEAP32[$szKeyInfo>>2] = $11;
 HEAP32[$sz>>2] = 136;
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$sz>>2]|0;
 $14 = HEAP32[$szKeyInfo>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = ($15|0)<(0);
 $17 = $16 << 31 >> 31;
 $18 = (_sqlite3DbMallocZero($12,$15,$17)|0);
 HEAP32[$pSorter>>2] = $18;
 $19 = HEAP32[$pSorter>>2]|0;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 16|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pSorter>>2]|0;
 $23 = ($22|0)==(0|0);
 if ($23) {
  HEAP32[$rc>>2] = 7;
  $225 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($225|0);
 }
 $24 = HEAP32[$pSorter>>2]|0;
 $25 = HEAP32[$sz>>2]|0;
 $26 = (($24) + ($25)|0);
 HEAP32[$pKeyInfo>>2] = $26;
 $27 = HEAP32[$pSorter>>2]|0;
 $28 = ((($27)) + 28|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$pKeyInfo>>2]|0;
 $30 = HEAP32[$2>>2]|0;
 $31 = ((($30)) + 24|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$szKeyInfo>>2]|0;
 _memcpy(($29|0),($32|0),($33|0))|0;
 $34 = HEAP32[$pKeyInfo>>2]|0;
 $35 = ((($34)) + 12|0);
 HEAP32[$35>>2] = 0;
 $36 = HEAP32[$1>>2]|0;
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$pKeyInfo>>2]|0;
  $39 = ((($38)) + 6|0);
  $40 = HEAP16[$39>>1]|0;
  $41 = $40&65535;
  $42 = HEAP32[$1>>2]|0;
  $43 = (($41) - ($42))|0;
  $44 = HEAP32[$pKeyInfo>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = HEAP16[$45>>1]|0;
  $47 = $46&65535;
  $48 = (($47) + ($43))|0;
  $49 = $48&65535;
  HEAP16[$45>>1] = $49;
  $50 = HEAP32[$1>>2]|0;
  $51 = $50&65535;
  $52 = HEAP32[$pKeyInfo>>2]|0;
  $53 = ((($52)) + 6|0);
  HEAP16[$53>>1] = $51;
 }
 $54 = HEAP32[$0>>2]|0;
 $55 = ((($54)) + 16|0);
 $56 = HEAP32[$55>>2]|0;
 $57 = ((($56)) + 4|0);
 $58 = HEAP32[$57>>2]|0;
 $59 = (_sqlite3BtreeGetPageSize($58)|0);
 HEAP32[$pgsz>>2] = $59;
 $60 = HEAP32[$pSorter>>2]|0;
 $61 = ((($60)) + 12|0);
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$pSorter>>2]|0;
 $63 = ((($62)) + 59|0);
 HEAP8[$63>>0] = 1;
 $64 = HEAP32[$pSorter>>2]|0;
 $65 = ((($64)) + 58|0);
 HEAP8[$65>>0] = -1;
 $66 = HEAP32[$pSorter>>2]|0;
 $67 = ((($66)) + 59|0);
 $68 = HEAP8[$67>>0]|0;
 $69 = $68&255;
 $70 = ($69|0)>(1);
 $71 = $70&1;
 $72 = $71&255;
 $73 = HEAP32[$pSorter>>2]|0;
 $74 = ((($73)) + 57|0);
 HEAP8[$74>>0] = $72;
 $75 = HEAP32[$0>>2]|0;
 $76 = HEAP32[$pSorter>>2]|0;
 $77 = ((($76)) + 24|0);
 HEAP32[$77>>2] = $75;
 HEAP32[$i>>2] = 0;
 while(1) {
  $78 = HEAP32[$i>>2]|0;
  $79 = HEAP32[$pSorter>>2]|0;
  $80 = ((($79)) + 59|0);
  $81 = HEAP8[$80>>0]|0;
  $82 = $81&255;
  $83 = ($78|0)<($82|0);
  if (!($83)) {
   break;
  }
  $84 = HEAP32[$i>>2]|0;
  $85 = HEAP32[$pSorter>>2]|0;
  $86 = ((($85)) + 64|0);
  $87 = (($86) + (($84*72)|0)|0);
  HEAP32[$pTask>>2] = $87;
  $88 = HEAP32[$pSorter>>2]|0;
  $89 = HEAP32[$pTask>>2]|0;
  $90 = ((($89)) + 8|0);
  HEAP32[$90>>2] = $88;
  $91 = HEAP32[$i>>2]|0;
  $92 = (($91) + 1)|0;
  HEAP32[$i>>2] = $92;
 }
 $93 = HEAP32[$0>>2]|0;
 $94 = (_sqlite3TempInMemory($93)|0);
 $95 = ($94|0)!=(0);
 if (!($95)) {
  $96 = HEAP32[(232)>>2]|0;
  HEAP32[$szPma>>2] = $96;
  $97 = HEAP32[$szPma>>2]|0;
  $98 = HEAP32[$pgsz>>2]|0;
  $99 = Math_imul($97, $98)|0;
  $100 = HEAP32[$pSorter>>2]|0;
  HEAP32[$100>>2] = $99;
  $101 = HEAP32[$0>>2]|0;
  $102 = ((($101)) + 16|0);
  $103 = HEAP32[$102>>2]|0;
  $104 = ((($103)) + 12|0);
  $105 = HEAP32[$104>>2]|0;
  $106 = ((($105)) + 80|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = ($107|0)<(0);
  $109 = $108 << 31 >> 31;
  $110 = $mxCache;
  $111 = $110;
  HEAP32[$111>>2] = $107;
  $112 = (($110) + 4)|0;
  $113 = $112;
  HEAP32[$113>>2] = $109;
  $114 = $mxCache;
  $115 = $114;
  $116 = HEAP32[$115>>2]|0;
  $117 = (($114) + 4)|0;
  $118 = $117;
  $119 = HEAP32[$118>>2]|0;
  $120 = ($119|0)<(0);
  $121 = $mxCache;
  $122 = $121;
  $123 = HEAP32[$122>>2]|0;
  $124 = (($121) + 4)|0;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  if ($120) {
   $127 = (___muldi3(($123|0),($126|0),-1024,-1)|0);
   $128 = tempRet0;
   $129 = $mxCache;
   $130 = $129;
   HEAP32[$130>>2] = $127;
   $131 = (($129) + 4)|0;
   $132 = $131;
   HEAP32[$132>>2] = $128;
  } else {
   $133 = HEAP32[$pgsz>>2]|0;
   $134 = ($133|0)<(0);
   $135 = $134 << 31 >> 31;
   $136 = (___muldi3(($123|0),($126|0),($133|0),($135|0))|0);
   $137 = tempRet0;
   $138 = $mxCache;
   $139 = $138;
   HEAP32[$139>>2] = $136;
   $140 = (($138) + 4)|0;
   $141 = $140;
   HEAP32[$141>>2] = $137;
  }
  $142 = $mxCache;
  $143 = $142;
  $144 = HEAP32[$143>>2]|0;
  $145 = (($142) + 4)|0;
  $146 = $145;
  $147 = HEAP32[$146>>2]|0;
  $148 = ($147|0)<(0);
  $149 = ($144>>>0)<(536870912);
  $150 = ($147|0)==(0);
  $151 = $150 & $149;
  $152 = $148 | $151;
  $153 = $mxCache;
  $154 = $153;
  $155 = HEAP32[$154>>2]|0;
  $156 = (($153) + 4)|0;
  $157 = $156;
  $158 = HEAP32[$157>>2]|0;
  $159 = $152 ? $155 : 536870912;
  $160 = $152 ? $158 : 0;
  $161 = $mxCache;
  $162 = $161;
  HEAP32[$162>>2] = $159;
  $163 = (($161) + 4)|0;
  $164 = $163;
  HEAP32[$164>>2] = $160;
  $165 = HEAP32[$pSorter>>2]|0;
  $166 = HEAP32[$165>>2]|0;
  $167 = $mxCache;
  $168 = $167;
  $169 = HEAP32[$168>>2]|0;
  $170 = (($167) + 4)|0;
  $171 = $170;
  $172 = HEAP32[$171>>2]|0;
  $173 = ($166|0)>($169|0);
  if ($173) {
   $174 = HEAP32[$pSorter>>2]|0;
   $175 = HEAP32[$174>>2]|0;
   $184 = $175;
  } else {
   $176 = $mxCache;
   $177 = $176;
   $178 = HEAP32[$177>>2]|0;
   $179 = (($176) + 4)|0;
   $180 = $179;
   $181 = HEAP32[$180>>2]|0;
   $184 = $178;
  }
  $182 = HEAP32[$pSorter>>2]|0;
  $183 = ((($182)) + 4|0);
  HEAP32[$183>>2] = $184;
  $185 = HEAP32[(200)>>2]|0;
  $186 = ($185|0)==(0|0);
  if ($186) {
   $187 = HEAP32[$pgsz>>2]|0;
   $188 = HEAP32[$pSorter>>2]|0;
   $189 = ((($188)) + 52|0);
   HEAP32[$189>>2] = $187;
   $190 = HEAP32[$pgsz>>2]|0;
   $191 = ($190|0)<(0);
   $192 = $191 << 31 >> 31;
   $193 = (_sqlite3Malloc($190,$192)|0);
   $194 = HEAP32[$pSorter>>2]|0;
   $195 = ((($194)) + 36|0);
   $196 = ((($195)) + 4|0);
   HEAP32[$196>>2] = $193;
   $197 = HEAP32[$pSorter>>2]|0;
   $198 = ((($197)) + 36|0);
   $199 = ((($198)) + 4|0);
   $200 = HEAP32[$199>>2]|0;
   $201 = ($200|0)!=(0|0);
   if (!($201)) {
    HEAP32[$rc>>2] = 7;
   }
  }
 }
 $202 = HEAP32[$pKeyInfo>>2]|0;
 $203 = ((($202)) + 6|0);
 $204 = HEAP16[$203>>1]|0;
 $205 = $204&65535;
 $206 = HEAP32[$pKeyInfo>>2]|0;
 $207 = ((($206)) + 8|0);
 $208 = HEAP16[$207>>1]|0;
 $209 = $208&65535;
 $210 = (($205) + ($209))|0;
 $211 = ($210|0)<(13);
 if (!($211)) {
  $225 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($225|0);
 }
 $212 = HEAP32[$pKeyInfo>>2]|0;
 $213 = ((($212)) + 20|0);
 $214 = HEAP32[$213>>2]|0;
 $215 = ($214|0)==(0|0);
 if (!($215)) {
  $216 = HEAP32[$pKeyInfo>>2]|0;
  $217 = ((($216)) + 20|0);
  $218 = HEAP32[$217>>2]|0;
  $219 = HEAP32[$0>>2]|0;
  $220 = ((($219)) + 8|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = ($218|0)==($221|0);
  if (!($222)) {
   $225 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($225|0);
  }
 }
 $223 = HEAP32[$pSorter>>2]|0;
 $224 = ((($223)) + 60|0);
 HEAP8[$224>>0] = 3;
 $225 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($225|0);
}
function _sqlite3VdbeFreeCursor($p,$pCx) {
 $p = $p|0;
 $pCx = $pCx|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pModule = 0, $pVCur = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pVCur = sp + 4|0;
 $pModule = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pCx;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(0|0);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 switch ($6|0) {
 case 1:  {
  $7 = HEAP32[$0>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  _sqlite3VdbeSorterClose($8,$9);
  STACKTOP = sp;return;
  break;
 }
 case 0:  {
  $10 = HEAP32[$1>>2]|0;
  $11 = ((($10)) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0|0);
  $14 = HEAP32[$1>>2]|0;
  if ($13) {
   $15 = ((($14)) + 20|0);
   $16 = HEAP32[$15>>2]|0;
   (_sqlite3BtreeClose($16)|0);
   STACKTOP = sp;return;
  } else {
   $17 = ((($14)) + 16|0);
   $18 = HEAP32[$17>>2]|0;
   (_sqlite3BtreeCloseCursor($18)|0);
   STACKTOP = sp;return;
  }
  break;
 }
 case 2:  {
  $19 = HEAP32[$1>>2]|0;
  $20 = ((($19)) + 16|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pVCur>>2] = $21;
  $22 = HEAP32[$pVCur>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pModule>>2] = $24;
  $25 = HEAP32[$pVCur>>2]|0;
  $26 = HEAP32[$25>>2]|0;
  $27 = ((($26)) + 4|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + -1)|0;
  HEAP32[$27>>2] = $29;
  $30 = HEAP32[$pModule>>2]|0;
  $31 = ((($30)) + 28|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$pVCur>>2]|0;
  (FUNCTION_TABLE_ii[$32 & 255]($33)|0);
  STACKTOP = sp;return;
  break;
 }
 default: {
  STACKTOP = sp;return;
 }
 }
}
function _sqlite3BtreeCursorHasHint($pCsr,$mask) {
 $pCsr = $pCsr|0;
 $mask = $mask|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCsr;
 HEAP32[$1>>2] = $mask;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 67|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = HEAP32[$1>>2]|0;
 $7 = $5 & $6;
 $8 = ($7|0)!=(0);
 $9 = $8&1;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3BtreeEof($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 66|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = (1)!=($4|0);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3BtreeLast($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pRes;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = (1)==($6|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 64|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = $11 & 8;
  $13 = ($12|0)!=(0);
  if ($13) {
   HEAP32[$0>>2] = 0;
   $37 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($37|0);
  }
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = (_moveToRoot($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 do {
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 66|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = (0)==($21|0);
   $23 = HEAP32[$2>>2]|0;
   if ($22) {
    HEAP32[$23>>2] = 1;
    break;
   }
   HEAP32[$23>>2] = 0;
   $24 = HEAP32[$1>>2]|0;
   $25 = (_moveToRightmost($24)|0);
   HEAP32[$rc>>2] = $25;
   $26 = HEAP32[$rc>>2]|0;
   $27 = ($26|0)==(0);
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 64|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   if ($27) {
    $32 = $31 | 8;
    $33 = $32&255;
    HEAP8[$29>>0] = $33;
    break;
   } else {
    $34 = $31 & -9;
    $35 = $34&255;
    HEAP8[$29>>0] = $35;
    break;
   }
  }
 } while(0);
 $36 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $36;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3BtreeInsert($pCur,$pX,$appendBias,$seekResult) {
 $pCur = $pCur|0;
 $pX = $pX|0;
 $appendBias = $appendBias|0;
 $seekResult = $seekResult|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $idx = 0, $loc = 0, $newCell = 0, $oldCell = 0, $p = 0, $pBt = 0, $pPage = 0, $rc = 0, $szNew = 0, $szOld = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $rc = sp + 32|0;
 $loc = sp + 28|0;
 $szNew = sp + 24|0;
 $idx = sp + 20|0;
 $pPage = sp + 16|0;
 $p = sp + 12|0;
 $pBt = sp + 8|0;
 $oldCell = sp + 4|0;
 $newCell = sp;
 $szOld = sp + 56|0;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pX;
 HEAP32[$3>>2] = $appendBias;
 HEAP32[$4>>2] = $seekResult;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$loc>>2] = $5;
 HEAP32[$szNew>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pBt>>2] = $10;
 HEAP32[$newCell>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 66|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(4);
 $16 = HEAP32[$1>>2]|0;
 if ($15) {
  $17 = ((($16)) + 60|0);
  $18 = HEAP32[$17>>2]|0;
  HEAP32[$0>>2] = $18;
  $245 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($245|0);
 }
 $19 = ((($16)) + 64|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = $21 & 32;
 $23 = ($22|0)!=(0);
 if ($23) {
  $24 = HEAP32[$pBt>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = ((($25)) + 52|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (_saveAllCursors($24,$27,$28)|0);
  HEAP32[$rc>>2] = $29;
  $30 = HEAP32[$rc>>2]|0;
  $31 = ($30|0)!=(0);
  if ($31) {
   $32 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $32;
   $245 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($245|0);
  }
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 72|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = ($35|0)==(0|0);
 do {
  if ($36) {
   $37 = HEAP32[$p>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = ((($38)) + 8|0);
   $40 = $39;
   $41 = $40;
   $42 = HEAP32[$41>>2]|0;
   $43 = (($40) + 4)|0;
   $44 = $43;
   $45 = HEAP32[$44>>2]|0;
   _invalidateIncrblobCursors($37,$42,$45,0);
   $46 = HEAP32[$1>>2]|0;
   $47 = ((($46)) + 64|0);
   $48 = HEAP8[$47>>0]|0;
   $49 = $48&255;
   $50 = $49 & 2;
   $51 = ($50|0)!=(0);
   if ($51) {
    $52 = HEAP32[$2>>2]|0;
    $53 = ((($52)) + 8|0);
    $54 = $53;
    $55 = $54;
    $56 = HEAP32[$55>>2]|0;
    $57 = (($54) + 4)|0;
    $58 = $57;
    $59 = HEAP32[$58>>2]|0;
    $60 = ($59|0)>(0);
    $61 = ($56>>>0)>(0);
    $62 = ($59|0)==(0);
    $63 = $62 & $61;
    $64 = $60 | $63;
    if ($64) {
     $65 = HEAP32[$1>>2]|0;
     $66 = ((($65)) + 16|0);
     $67 = $66;
     $68 = $67;
     $69 = HEAP32[$68>>2]|0;
     $70 = (($67) + 4)|0;
     $71 = $70;
     $72 = HEAP32[$71>>2]|0;
     $73 = HEAP32[$2>>2]|0;
     $74 = ((($73)) + 8|0);
     $75 = $74;
     $76 = $75;
     $77 = HEAP32[$76>>2]|0;
     $78 = (($75) + 4)|0;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $81 = (_i64Subtract(($77|0),($80|0),1,0)|0);
     $82 = tempRet0;
     $83 = ($69|0)==($81|0);
     $84 = ($72|0)==($82|0);
     $85 = $83 & $84;
     if ($85) {
      HEAP32[$loc>>2] = -1;
      break;
     }
    }
   }
   $86 = HEAP32[$loc>>2]|0;
   $87 = ($86|0)==(0);
   if ($87) {
    $88 = HEAP32[$1>>2]|0;
    $89 = HEAP32[$2>>2]|0;
    $90 = ((($89)) + 8|0);
    $91 = $90;
    $92 = $91;
    $93 = HEAP32[$92>>2]|0;
    $94 = (($91) + 4)|0;
    $95 = $94;
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$3>>2]|0;
    $98 = (_sqlite3BtreeMovetoUnpacked($88,0,$93,$96,$97,$loc)|0);
    HEAP32[$rc>>2] = $98;
    $99 = HEAP32[$rc>>2]|0;
    $100 = ($99|0)!=(0);
    if ($100) {
     $101 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $101;
     $245 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($245|0);
    }
   }
  } else {
   $102 = HEAP32[$loc>>2]|0;
   $103 = ($102|0)==(0);
   if ($103) {
    $104 = HEAP32[$1>>2]|0;
    $105 = HEAP32[$2>>2]|0;
    $106 = HEAP32[$105>>2]|0;
    $107 = HEAP32[$2>>2]|0;
    $108 = ((($107)) + 8|0);
    $109 = $108;
    $110 = $109;
    $111 = HEAP32[$110>>2]|0;
    $112 = (($109) + 4)|0;
    $113 = $112;
    $114 = HEAP32[$113>>2]|0;
    $115 = HEAP32[$3>>2]|0;
    $116 = (_btreeMoveto($104,$106,$111,$114,$115,$loc)|0);
    HEAP32[$rc>>2] = $116;
    $117 = HEAP32[$rc>>2]|0;
    $118 = ($117|0)!=(0);
    if ($118) {
     $119 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $119;
     $245 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($245|0);
    }
   }
  }
 } while(0);
 $120 = HEAP32[$1>>2]|0;
 $121 = ((($120)) + 68|0);
 $122 = HEAP8[$121>>0]|0;
 $123 = $122 << 24 >> 24;
 $124 = HEAP32[$1>>2]|0;
 $125 = ((($124)) + 120|0);
 $126 = (($125) + ($123<<2)|0);
 $127 = HEAP32[$126>>2]|0;
 HEAP32[$pPage>>2] = $127;
 $128 = HEAP32[$pBt>>2]|0;
 $129 = ((($128)) + 80|0);
 $130 = HEAP32[$129>>2]|0;
 HEAP32[$newCell>>2] = $130;
 $131 = HEAP32[$pPage>>2]|0;
 $132 = HEAP32[$newCell>>2]|0;
 $133 = HEAP32[$2>>2]|0;
 $134 = (_fillInCell($131,$132,$133,$szNew)|0);
 HEAP32[$rc>>2] = $134;
 $135 = HEAP32[$rc>>2]|0;
 $136 = ($135|0)!=(0);
 do {
  if (!($136)) {
   $137 = HEAP32[$1>>2]|0;
   $138 = ((($137)) + 68|0);
   $139 = HEAP8[$138>>0]|0;
   $140 = $139 << 24 >> 24;
   $141 = HEAP32[$1>>2]|0;
   $142 = ((($141)) + 80|0);
   $143 = (($142) + ($140<<1)|0);
   $144 = HEAP16[$143>>1]|0;
   $145 = $144&65535;
   HEAP32[$idx>>2] = $145;
   $146 = HEAP32[$loc>>2]|0;
   $147 = ($146|0)==(0);
   if ($147) {
    $148 = HEAP32[$pPage>>2]|0;
    $149 = ((($148)) + 72|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = (_sqlite3PagerWrite($150)|0);
    HEAP32[$rc>>2] = $151;
    $152 = HEAP32[$rc>>2]|0;
    $153 = ($152|0)!=(0);
    if ($153) {
     break;
    }
    $154 = HEAP32[$pPage>>2]|0;
    $155 = ((($154)) + 56|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = HEAP32[$pPage>>2]|0;
    $158 = ((($157)) + 20|0);
    $159 = HEAP16[$158>>1]|0;
    $160 = $159&65535;
    $161 = HEAP32[$idx>>2]|0;
    $162 = $161<<1;
    $163 = HEAP32[$pPage>>2]|0;
    $164 = ((($163)) + 64|0);
    $165 = HEAP32[$164>>2]|0;
    $166 = (($165) + ($162)|0);
    $167 = HEAP8[$166>>0]|0;
    $168 = $167&255;
    $169 = $168 << 8;
    $170 = HEAP32[$idx>>2]|0;
    $171 = $170<<1;
    $172 = HEAP32[$pPage>>2]|0;
    $173 = ((($172)) + 64|0);
    $174 = HEAP32[$173>>2]|0;
    $175 = (($174) + ($171)|0);
    $176 = ((($175)) + 1|0);
    $177 = HEAP8[$176>>0]|0;
    $178 = $177&255;
    $179 = $169 | $178;
    $180 = $160 & $179;
    $181 = (($156) + ($180)|0);
    HEAP32[$oldCell>>2] = $181;
    $182 = HEAP32[$pPage>>2]|0;
    $183 = ((($182)) + 4|0);
    $184 = HEAP8[$183>>0]|0;
    $185 = ($184<<24>>24)!=(0);
    if (!($185)) {
     $186 = HEAP32[$newCell>>2]|0;
     $187 = HEAP32[$oldCell>>2]|0;
     ;HEAP8[$186>>0]=HEAP8[$187>>0]|0;HEAP8[$186+1>>0]=HEAP8[$187+1>>0]|0;HEAP8[$186+2>>0]=HEAP8[$187+2>>0]|0;HEAP8[$186+3>>0]=HEAP8[$187+3>>0]|0;
    }
    $188 = HEAP32[$pPage>>2]|0;
    $189 = HEAP32[$oldCell>>2]|0;
    $190 = (_clearCell($188,$189,$szOld)|0);
    HEAP32[$rc>>2] = $190;
    $191 = HEAP32[$pPage>>2]|0;
    $192 = HEAP32[$idx>>2]|0;
    $193 = HEAP16[$szOld>>1]|0;
    $194 = $193&65535;
    _dropCell($191,$192,$194,$rc);
    $195 = HEAP32[$rc>>2]|0;
    $196 = ($195|0)!=(0);
    if ($196) {
     break;
    }
   } else {
    $197 = HEAP32[$loc>>2]|0;
    $198 = ($197|0)<(0);
    if ($198) {
     $199 = HEAP32[$pPage>>2]|0;
     $200 = ((($199)) + 18|0);
     $201 = HEAP16[$200>>1]|0;
     $202 = $201&65535;
     $203 = ($202|0)>(0);
     if ($203) {
      $204 = HEAP32[$1>>2]|0;
      $205 = ((($204)) + 68|0);
      $206 = HEAP8[$205>>0]|0;
      $207 = $206 << 24 >> 24;
      $208 = HEAP32[$1>>2]|0;
      $209 = ((($208)) + 80|0);
      $210 = (($209) + ($207<<1)|0);
      $211 = HEAP16[$210>>1]|0;
      $212 = (($211) + 1)<<16>>16;
      HEAP16[$210>>1] = $212;
      $213 = $212&65535;
      HEAP32[$idx>>2] = $213;
     }
    }
   }
   $214 = HEAP32[$pPage>>2]|0;
   $215 = HEAP32[$idx>>2]|0;
   $216 = HEAP32[$newCell>>2]|0;
   $217 = HEAP32[$szNew>>2]|0;
   _insertCell($214,$215,$216,$217,0,0,$rc);
   $218 = HEAP32[$1>>2]|0;
   $219 = ((($218)) + 16|0);
   $220 = ((($219)) + 18|0);
   HEAP16[$220>>1] = 0;
   $221 = HEAP32[$pPage>>2]|0;
   $222 = ((($221)) + 1|0);
   $223 = HEAP8[$222>>0]|0;
   $224 = ($223<<24>>24)!=(0);
   if ($224) {
    $225 = HEAP32[$1>>2]|0;
    $226 = ((($225)) + 64|0);
    $227 = HEAP8[$226>>0]|0;
    $228 = $227&255;
    $229 = $228 & -3;
    $230 = $229&255;
    HEAP8[$226>>0] = $230;
    $231 = HEAP32[$1>>2]|0;
    $232 = (_balance($231)|0);
    HEAP32[$rc>>2] = $232;
    $233 = HEAP32[$1>>2]|0;
    $234 = ((($233)) + 68|0);
    $235 = HEAP8[$234>>0]|0;
    $236 = $235 << 24 >> 24;
    $237 = HEAP32[$1>>2]|0;
    $238 = ((($237)) + 120|0);
    $239 = (($238) + ($236<<2)|0);
    $240 = HEAP32[$239>>2]|0;
    $241 = ((($240)) + 1|0);
    HEAP8[$241>>0] = 0;
    $242 = HEAP32[$1>>2]|0;
    $243 = ((($242)) + 66|0);
    HEAP8[$243>>0] = 0;
   }
  }
 } while(0);
 $244 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $244;
 $245 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($245|0);
}
function _sqlite3BtreeDelete($pCur,$flags) {
 $pCur = $pCur|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0;
 var $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0;
 var $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0;
 var $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0;
 var $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bPreserve = 0, $bSkipnext = 0, $iCellDepth = 0, $iCellIdx = 0, $n = 0, $nCell = 0, $notUsed = 0, $p = 0, $pBt = 0, $pCell = 0;
 var $pLeaf = 0, $pPage = 0, $pTmp = 0, $rc = 0, $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 63|0;
 $p = sp + 48|0;
 $pBt = sp + 44|0;
 $rc = sp + 40|0;
 $pPage = sp + 36|0;
 $pCell = sp + 32|0;
 $iCellIdx = sp + 28|0;
 $iCellDepth = sp + 24|0;
 $szCell = sp + 60|0;
 $bSkipnext = sp + 20|0;
 $bPreserve = sp + 62|0;
 $notUsed = sp + 16|0;
 $pLeaf = sp + 12|0;
 $nCell = sp + 8|0;
 $n = sp + 4|0;
 $pTmp = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP8[$2>>0] = $flags;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 HEAP32[$bSkipnext>>2] = 0;
 $8 = HEAP8[$2>>0]|0;
 $9 = $8&255;
 $10 = $9 & 2;
 $11 = $10&255;
 HEAP8[$bPreserve>>0] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 68|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14 << 24 >> 24;
 HEAP32[$iCellDepth>>2] = $15;
 $16 = HEAP32[$iCellDepth>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 80|0);
 $19 = (($18) + ($16<<1)|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 HEAP32[$iCellIdx>>2] = $21;
 $22 = HEAP32[$iCellDepth>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 120|0);
 $25 = (($24) + ($22<<2)|0);
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pPage>>2] = $26;
 $27 = HEAP32[$pPage>>2]|0;
 $28 = ((($27)) + 56|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$pPage>>2]|0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP16[$31>>1]|0;
 $33 = $32&65535;
 $34 = HEAP32[$iCellIdx>>2]|0;
 $35 = $34<<1;
 $36 = HEAP32[$pPage>>2]|0;
 $37 = ((($36)) + 64|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = (($38) + ($35)|0);
 $40 = HEAP8[$39>>0]|0;
 $41 = $40&255;
 $42 = $41 << 8;
 $43 = HEAP32[$iCellIdx>>2]|0;
 $44 = $43<<1;
 $45 = HEAP32[$pPage>>2]|0;
 $46 = ((($45)) + 64|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44)|0);
 $49 = ((($48)) + 1|0);
 $50 = HEAP8[$49>>0]|0;
 $51 = $50&255;
 $52 = $42 | $51;
 $53 = $33 & $52;
 $54 = (($29) + ($53)|0);
 HEAP32[$pCell>>2] = $54;
 $55 = HEAP8[$bPreserve>>0]|0;
 $56 = ($55<<24>>24)!=(0);
 do {
  if ($56) {
   $57 = HEAP32[$pPage>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = ($59<<24>>24)!=(0);
   if ($60) {
    $61 = HEAP32[$pPage>>2]|0;
    $62 = ((($61)) + 16|0);
    $63 = HEAP16[$62>>1]|0;
    $64 = $63&65535;
    $65 = HEAP32[$pPage>>2]|0;
    $66 = HEAP32[$pCell>>2]|0;
    $67 = (_cellSizePtr($65,$66)|0);
    $68 = $67&65535;
    $69 = (($64) + ($68))|0;
    $70 = (($69) + 2)|0;
    $71 = HEAP32[$pBt>>2]|0;
    $72 = ((($71)) + 36|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = $73<<1;
    $75 = (($74>>>0) / 3)&-1;
    $76 = ($70|0)>($75|0);
    if (!($76)) {
     HEAP32[$bSkipnext>>2] = 1;
     break;
    }
   }
   $77 = HEAP32[$1>>2]|0;
   $78 = (_saveCursorKey($77)|0);
   HEAP32[$rc>>2] = $78;
   $79 = HEAP32[$rc>>2]|0;
   $80 = ($79|0)!=(0);
   if ($80) {
    $81 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $81;
    $289 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($289|0);
   }
  }
 } while(0);
 $82 = HEAP32[$pPage>>2]|0;
 $83 = ((($82)) + 4|0);
 $84 = HEAP8[$83>>0]|0;
 $85 = ($84<<24>>24)!=(0);
 if (!($85)) {
  HEAP32[$notUsed>>2] = 0;
  $86 = HEAP32[$1>>2]|0;
  $87 = (_sqlite3BtreePrevious($86,$notUsed)|0);
  HEAP32[$rc>>2] = $87;
  $88 = HEAP32[$rc>>2]|0;
  $89 = ($88|0)!=(0);
  if ($89) {
   $90 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $90;
   $289 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($289|0);
  }
 }
 $91 = HEAP32[$1>>2]|0;
 $92 = ((($91)) + 64|0);
 $93 = HEAP8[$92>>0]|0;
 $94 = $93&255;
 $95 = $94 & 32;
 $96 = ($95|0)!=(0);
 if ($96) {
  $97 = HEAP32[$pBt>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = ((($98)) + 52|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = HEAP32[$1>>2]|0;
  $102 = (_saveAllCursors($97,$100,$101)|0);
  HEAP32[$rc>>2] = $102;
  $103 = HEAP32[$rc>>2]|0;
  $104 = ($103|0)!=(0);
  if ($104) {
   $105 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $105;
   $289 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($289|0);
  }
 }
 $106 = HEAP32[$1>>2]|0;
 $107 = ((($106)) + 72|0);
 $108 = HEAP32[$107>>2]|0;
 $109 = ($108|0)==(0|0);
 if ($109) {
  $110 = HEAP32[$p>>2]|0;
  $111 = HEAP32[$1>>2]|0;
  $112 = ((($111)) + 16|0);
  $113 = $112;
  $114 = $113;
  $115 = HEAP32[$114>>2]|0;
  $116 = (($113) + 4)|0;
  $117 = $116;
  $118 = HEAP32[$117>>2]|0;
  _invalidateIncrblobCursors($110,$115,$118,0);
 }
 $119 = HEAP32[$pPage>>2]|0;
 $120 = ((($119)) + 72|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = (_sqlite3PagerWrite($121)|0);
 HEAP32[$rc>>2] = $122;
 $123 = HEAP32[$rc>>2]|0;
 $124 = ($123|0)!=(0);
 if ($124) {
  $125 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $125;
  $289 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($289|0);
 }
 $126 = HEAP32[$pPage>>2]|0;
 $127 = HEAP32[$pCell>>2]|0;
 $128 = (_clearCell($126,$127,$szCell)|0);
 HEAP32[$rc>>2] = $128;
 $129 = HEAP32[$pPage>>2]|0;
 $130 = HEAP32[$iCellIdx>>2]|0;
 $131 = HEAP16[$szCell>>1]|0;
 $132 = $131&65535;
 _dropCell($129,$130,$132,$rc);
 $133 = HEAP32[$rc>>2]|0;
 $134 = ($133|0)!=(0);
 if ($134) {
  $135 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $135;
  $289 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($289|0);
 }
 $136 = HEAP32[$pPage>>2]|0;
 $137 = ((($136)) + 4|0);
 $138 = HEAP8[$137>>0]|0;
 $139 = ($138<<24>>24)!=(0);
 if (!($139)) {
  $140 = HEAP32[$1>>2]|0;
  $141 = ((($140)) + 68|0);
  $142 = HEAP8[$141>>0]|0;
  $143 = $142 << 24 >> 24;
  $144 = HEAP32[$1>>2]|0;
  $145 = ((($144)) + 120|0);
  $146 = (($145) + ($143<<2)|0);
  $147 = HEAP32[$146>>2]|0;
  HEAP32[$pLeaf>>2] = $147;
  $148 = HEAP32[$iCellDepth>>2]|0;
  $149 = (($148) + 1)|0;
  $150 = HEAP32[$1>>2]|0;
  $151 = ((($150)) + 120|0);
  $152 = (($151) + ($149<<2)|0);
  $153 = HEAP32[$152>>2]|0;
  $154 = ((($153)) + 84|0);
  $155 = HEAP32[$154>>2]|0;
  HEAP32[$n>>2] = $155;
  $156 = HEAP32[$pLeaf>>2]|0;
  $157 = ((($156)) + 56|0);
  $158 = HEAP32[$157>>2]|0;
  $159 = HEAP32[$pLeaf>>2]|0;
  $160 = ((($159)) + 20|0);
  $161 = HEAP16[$160>>1]|0;
  $162 = $161&65535;
  $163 = HEAP32[$pLeaf>>2]|0;
  $164 = ((($163)) + 18|0);
  $165 = HEAP16[$164>>1]|0;
  $166 = $165&65535;
  $167 = (($166) - 1)|0;
  $168 = $167<<1;
  $169 = HEAP32[$pLeaf>>2]|0;
  $170 = ((($169)) + 64|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = (($171) + ($168)|0);
  $173 = HEAP8[$172>>0]|0;
  $174 = $173&255;
  $175 = $174 << 8;
  $176 = HEAP32[$pLeaf>>2]|0;
  $177 = ((($176)) + 18|0);
  $178 = HEAP16[$177>>1]|0;
  $179 = $178&65535;
  $180 = (($179) - 1)|0;
  $181 = $180<<1;
  $182 = HEAP32[$pLeaf>>2]|0;
  $183 = ((($182)) + 64|0);
  $184 = HEAP32[$183>>2]|0;
  $185 = (($184) + ($181)|0);
  $186 = ((($185)) + 1|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $187&255;
  $189 = $175 | $188;
  $190 = $162 & $189;
  $191 = (($158) + ($190)|0);
  HEAP32[$pCell>>2] = $191;
  $192 = HEAP32[$pCell>>2]|0;
  $193 = HEAP32[$pLeaf>>2]|0;
  $194 = ((($193)) + 56|0);
  $195 = HEAP32[$194>>2]|0;
  $196 = ((($195)) + 4|0);
  $197 = ($192>>>0)<($196>>>0);
  if ($197) {
   $198 = (_sqlite3CorruptError(66449)|0);
   HEAP32[$0>>2] = $198;
   $289 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($289|0);
  }
  $199 = HEAP32[$pLeaf>>2]|0;
  $200 = ((($199)) + 76|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = HEAP32[$pLeaf>>2]|0;
  $203 = HEAP32[$pCell>>2]|0;
  $204 = (FUNCTION_TABLE_iii[$201 & 255]($202,$203)|0);
  $205 = $204&65535;
  HEAP32[$nCell>>2] = $205;
  $206 = HEAP32[$pBt>>2]|0;
  $207 = ((($206)) + 80|0);
  $208 = HEAP32[$207>>2]|0;
  HEAP32[$pTmp>>2] = $208;
  $209 = HEAP32[$pLeaf>>2]|0;
  $210 = ((($209)) + 72|0);
  $211 = HEAP32[$210>>2]|0;
  $212 = (_sqlite3PagerWrite($211)|0);
  HEAP32[$rc>>2] = $212;
  $213 = HEAP32[$rc>>2]|0;
  $214 = ($213|0)==(0);
  if ($214) {
   $215 = HEAP32[$pPage>>2]|0;
   $216 = HEAP32[$iCellIdx>>2]|0;
   $217 = HEAP32[$pCell>>2]|0;
   $218 = ((($217)) + -4|0);
   $219 = HEAP32[$nCell>>2]|0;
   $220 = (($219) + 4)|0;
   $221 = HEAP32[$pTmp>>2]|0;
   $222 = HEAP32[$n>>2]|0;
   _insertCell($215,$216,$218,$220,$221,$222,$rc);
  }
  $223 = HEAP32[$pLeaf>>2]|0;
  $224 = HEAP32[$pLeaf>>2]|0;
  $225 = ((($224)) + 18|0);
  $226 = HEAP16[$225>>1]|0;
  $227 = $226&65535;
  $228 = (($227) - 1)|0;
  $229 = HEAP32[$nCell>>2]|0;
  _dropCell($223,$228,$229,$rc);
  $230 = HEAP32[$rc>>2]|0;
  $231 = ($230|0)!=(0);
  if ($231) {
   $232 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $232;
   $289 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($289|0);
  }
 }
 $233 = HEAP32[$1>>2]|0;
 $234 = (_balance($233)|0);
 HEAP32[$rc>>2] = $234;
 $235 = HEAP32[$rc>>2]|0;
 $236 = ($235|0)==(0);
 if ($236) {
  $237 = HEAP32[$1>>2]|0;
  $238 = ((($237)) + 68|0);
  $239 = HEAP8[$238>>0]|0;
  $240 = $239 << 24 >> 24;
  $241 = HEAP32[$iCellDepth>>2]|0;
  $242 = ($240|0)>($241|0);
  if ($242) {
   while(1) {
    $243 = HEAP32[$1>>2]|0;
    $244 = ((($243)) + 68|0);
    $245 = HEAP8[$244>>0]|0;
    $246 = $245 << 24 >> 24;
    $247 = HEAP32[$iCellDepth>>2]|0;
    $248 = ($246|0)>($247|0);
    $249 = HEAP32[$1>>2]|0;
    if (!($248)) {
     break;
    }
    $250 = ((($249)) + 68|0);
    $251 = HEAP8[$250>>0]|0;
    $252 = (($251) + -1)<<24>>24;
    HEAP8[$250>>0] = $252;
    $253 = $251 << 24 >> 24;
    $254 = HEAP32[$1>>2]|0;
    $255 = ((($254)) + 120|0);
    $256 = (($255) + ($253<<2)|0);
    $257 = HEAP32[$256>>2]|0;
    _releasePage($257);
   }
   $258 = (_balance($249)|0);
   HEAP32[$rc>>2] = $258;
  }
 }
 $259 = HEAP32[$rc>>2]|0;
 $260 = ($259|0)==(0);
 do {
  if ($260) {
   $261 = HEAP32[$bSkipnext>>2]|0;
   $262 = ($261|0)!=(0);
   $263 = HEAP32[$1>>2]|0;
   if (!($262)) {
    $283 = (_moveToRoot($263)|0);
    HEAP32[$rc>>2] = $283;
    $284 = HEAP8[$bPreserve>>0]|0;
    $285 = ($284<<24>>24)!=(0);
    if (!($285)) {
     break;
    }
    $286 = HEAP32[$1>>2]|0;
    $287 = ((($286)) + 66|0);
    HEAP8[$287>>0] = 3;
    break;
   }
   $264 = ((($263)) + 66|0);
   HEAP8[$264>>0] = 2;
   $265 = HEAP32[$iCellIdx>>2]|0;
   $266 = HEAP32[$pPage>>2]|0;
   $267 = ((($266)) + 18|0);
   $268 = HEAP16[$267>>1]|0;
   $269 = $268&65535;
   $270 = ($265|0)>=($269|0);
   $271 = HEAP32[$1>>2]|0;
   $272 = ((($271)) + 60|0);
   if ($270) {
    HEAP32[$272>>2] = -1;
    $273 = HEAP32[$pPage>>2]|0;
    $274 = ((($273)) + 18|0);
    $275 = HEAP16[$274>>1]|0;
    $276 = $275&65535;
    $277 = (($276) - 1)|0;
    $278 = $277&65535;
    $279 = HEAP32[$iCellDepth>>2]|0;
    $280 = HEAP32[$1>>2]|0;
    $281 = ((($280)) + 80|0);
    $282 = (($281) + ($279<<1)|0);
    HEAP16[$282>>1] = $278;
    break;
   } else {
    HEAP32[$272>>2] = 1;
    break;
   }
  }
 } while(0);
 $288 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $288;
 $289 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($289|0);
}
function _sqlite3VdbeSorterCompare($pCsr,$pVal,$nKeyCol,$pRes) {
 $pCsr = $pCsr|0;
 $pVal = $pVal|0;
 $nKeyCol = $nKeyCol|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nKey = 0;
 var $p = 0, $pKey = 0, $pKeyInfo = 0, $pSorter = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $pSorter = sp + 24|0;
 $r2 = sp + 20|0;
 $pKeyInfo = sp + 16|0;
 $i = sp + 12|0;
 $pKey = sp + 8|0;
 $nKey = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pVal;
 HEAP32[$3>>2] = $nKeyCol;
 HEAP32[$4>>2] = $pRes;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pSorter>>2] = $7;
 $8 = HEAP32[$pSorter>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$r2>>2] = $10;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pKeyInfo>>2] = $13;
 $14 = HEAP32[$r2>>2]|0;
 $15 = ($14|0)==(0|0);
 do {
  if ($15) {
   $16 = HEAP32[$pKeyInfo>>2]|0;
   $17 = (_sqlite3VdbeAllocUnpackedRecord($16,0,0,$p)|0);
   $18 = HEAP32[$pSorter>>2]|0;
   $19 = ((($18)) + 32|0);
   HEAP32[$19>>2] = $17;
   HEAP32[$r2>>2] = $17;
   $20 = HEAP32[$r2>>2]|0;
   $21 = ($20|0)==(0|0);
   if (!($21)) {
    $22 = HEAP32[$3>>2]|0;
    $23 = $22&65535;
    $24 = HEAP32[$r2>>2]|0;
    $25 = ((($24)) + 8|0);
    HEAP16[$25>>1] = $23;
    break;
   }
   HEAP32[$0>>2] = 7;
   $57 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($57|0);
  }
 } while(0);
 $26 = HEAP32[$pSorter>>2]|0;
 $27 = (_vdbeSorterRowkey($26,$nKey)|0);
 HEAP32[$pKey>>2] = $27;
 $28 = HEAP32[$pKeyInfo>>2]|0;
 $29 = HEAP32[$nKey>>2]|0;
 $30 = HEAP32[$pKey>>2]|0;
 $31 = HEAP32[$r2>>2]|0;
 _sqlite3VdbeRecordUnpack($28,$29,$30,$31);
 HEAP32[$i>>2] = 0;
 while(1) {
  $32 = HEAP32[$i>>2]|0;
  $33 = HEAP32[$3>>2]|0;
  $34 = ($32|0)<($33|0);
  if (!($34)) {
   label = 10;
   break;
  }
  $35 = HEAP32[$i>>2]|0;
  $36 = HEAP32[$r2>>2]|0;
  $37 = ((($36)) + 4|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = (($38) + (($35*40)|0)|0);
  $40 = ((($39)) + 8|0);
  $41 = HEAP16[$40>>1]|0;
  $42 = $41&65535;
  $43 = $42 & 1;
  $44 = ($43|0)!=(0);
  if ($44) {
   label = 8;
   break;
  }
  $46 = HEAP32[$i>>2]|0;
  $47 = (($46) + 1)|0;
  HEAP32[$i>>2] = $47;
 }
 if ((label|0) == 8) {
  $45 = HEAP32[$4>>2]|0;
  HEAP32[$45>>2] = -1;
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 else if ((label|0) == 10) {
  $48 = HEAP32[$2>>2]|0;
  $49 = ((($48)) + 12|0);
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$2>>2]|0;
  $52 = ((($51)) + 16|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$r2>>2]|0;
  $55 = (_sqlite3VdbeRecordCompare($50,$53,$54)|0);
  $56 = HEAP32[$4>>2]|0;
  HEAP32[$56>>2] = $55;
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSorterRowkey($pCsr,$pOut) {
 $pCsr = $pCsr|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nKey = 0, $pKey = 0, $pSorter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSorter = sp + 8|0;
 $pKey = sp + 4|0;
 $nKey = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = (_vdbeSorterRowkey($6,$nKey)|0);
 HEAP32[$pKey>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$nKey>>2]|0;
 $10 = (_sqlite3VdbeMemClearAndResize($8,$9)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  HEAP32[$0>>2] = 7;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 } else {
  $12 = HEAP32[$nKey>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 12|0);
  HEAP32[$14>>2] = $12;
  $15 = HEAP32[$2>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP16[$16>>1]|0;
  $18 = $17&65535;
  $19 = $18 & -49664;
  $20 = $19 | 16;
  $21 = $20&65535;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 8|0);
  HEAP16[$23>>1] = $21;
  $24 = HEAP32[$2>>2]|0;
  $25 = ((($24)) + 16|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$pKey>>2]|0;
  $28 = HEAP32[$nKey>>2]|0;
  _memcpy(($26|0),($27|0),($28|0))|0;
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _sqlite3VtabImportErrmsg($p,$pVtab) {
 $p = $p|0;
 $pVtab = $pVtab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pVtab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$db>>2] = $7;
 $8 = HEAP32[$db>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 108|0);
 $11 = HEAP32[$10>>2]|0;
 _sqlite3DbFree($8,$11);
 $12 = HEAP32[$db>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3DbStrDup($12,$15)|0);
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 108|0);
 HEAP32[$18>>2] = $16;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 8|0);
 $21 = HEAP32[$20>>2]|0;
 _sqlite3_free($21);
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 8|0);
 HEAP32[$23>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3VdbeCursorRestore($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3BtreeCursorHasMoved($4)|0);
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = (_handleMovedCursor($7)|0);
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  HEAP32[$0>>2] = 0;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSorterRewind($pCsr,$pbEof) {
 $pCsr = $pCsr|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSorter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $pSorter = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pbEof;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = ((($6)) + 56|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = ($9|0)==(0);
 $11 = HEAP32[$pSorter>>2]|0;
 if (!($10)) {
  $22 = (_vdbeSorterFlushPMA($11)|0);
  HEAP32[$rc>>2] = $22;
  $23 = HEAP32[$rc>>2]|0;
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(0);
  if ($25) {
   $26 = HEAP32[$pSorter>>2]|0;
   $27 = (_vdbeSorterSetupMerge($26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$2>>2]|0;
   HEAP32[$28>>2] = 0;
  }
  $29 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $29;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $12 = ((($11)) + 36|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 $15 = HEAP32[$2>>2]|0;
 if ($14) {
  HEAP32[$15>>2] = 0;
  $16 = HEAP32[$pSorter>>2]|0;
  $17 = ((($16)) + 64|0);
  $18 = HEAP32[$pSorter>>2]|0;
  $19 = ((($18)) + 36|0);
  $20 = (_vdbeSorterSort($17,$19)|0);
  HEAP32[$rc>>2] = $20;
 } else {
  HEAP32[$15>>2] = 1;
 }
 $21 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $21;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _sqlite3BtreeFirst($pCur,$pRes) {
 $pCur = $pCur|0;
 $pRes = $pRes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pRes;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_moveToRoot($2)|0);
 HEAP32[$rc>>2] = $3;
 $4 = HEAP32[$rc>>2]|0;
 $5 = ($4|0)==(0);
 do {
  if ($5) {
   $6 = HEAP32[$0>>2]|0;
   $7 = ((($6)) + 66|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = ($9|0)==(0);
   $11 = HEAP32[$1>>2]|0;
   if ($10) {
    HEAP32[$11>>2] = 1;
    break;
   } else {
    HEAP32[$11>>2] = 0;
    $12 = HEAP32[$0>>2]|0;
    $13 = (_moveToLeftmost($12)|0);
    HEAP32[$rc>>2] = $13;
    break;
   }
  }
 } while(0);
 $14 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($14|0);
}
function _sqlite3VdbeSorterNext($db,$pCsr,$pbEof) {
 $db = $db|0;
 $pCsr = $pCsr|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFree = 0;
 var $pSorter = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pSorter = sp + 8|0;
 $rc = sp + 4|0;
 $pFree = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pbEof;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$pSorter>>2]|0;
 $7 = ((($6)) + 56|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 $10 = HEAP32[$pSorter>>2]|0;
 if ($9) {
  $11 = ((($10)) + 20|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = (_vdbeMergeEngineStep($12,$13)|0);
  HEAP32[$rc>>2] = $14;
  $38 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $15 = ((($10)) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pFree>>2] = $16;
 $17 = HEAP32[$pFree>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = HEAP32[$pSorter>>2]|0;
 $21 = ((($20)) + 36|0);
 HEAP32[$21>>2] = $19;
 $22 = HEAP32[$pFree>>2]|0;
 $23 = ((($22)) + 4|0);
 HEAP32[$23>>2] = 0;
 $24 = HEAP32[$pSorter>>2]|0;
 $25 = ((($24)) + 36|0);
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$pFree>>2]|0;
  _vdbeSorterRecordFree($29,$30);
 }
 $31 = HEAP32[$pSorter>>2]|0;
 $32 = ((($31)) + 36|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)!=(0|0);
 $35 = $34 ^ 1;
 $36 = $35&1;
 $37 = HEAP32[$2>>2]|0;
 HEAP32[$37>>2] = $36;
 HEAP32[$rc>>2] = 0;
 $38 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _sqlite3VdbeSorterWrite($pCsr,$pVal) {
 $pCsr = $pCsr|0;
 $pVal = $pVal|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $bFlush = 0, $iListOff = 0, $nMin = 0, $nNew = 0, $nPMA = 0, $nReq = 0, $or$cond = 0, $or$cond3 = 0, $pNew = 0, $pSorter = 0;
 var $rc = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $pSorter = sp + 40|0;
 $rc = sp + 36|0;
 $pNew = sp + 32|0;
 $bFlush = sp + 28|0;
 $nReq = sp + 24|0;
 $nPMA = sp + 20|0;
 $t = sp + 16|0;
 $nMin = sp + 12|0;
 $aNew = sp + 8|0;
 $iListOff = sp + 4|0;
 $nNew = sp;
 HEAP32[$1>>2] = $pCsr;
 HEAP32[$2>>2] = $pVal;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pSorter>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 1|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)<(128);
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 1|0);
 if ($12) {
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  HEAP32[$t>>2] = $18;
 } else {
  (_sqlite3GetVarint32($16,$t)|0);
 }
 $19 = HEAP32[$t>>2]|0;
 $20 = ($19|0)>(0);
 $21 = HEAP32[$t>>2]|0;
 $22 = ($21|0)<(10);
 $or$cond = $20 & $22;
 $23 = HEAP32[$t>>2]|0;
 $24 = ($23|0)!=(7);
 $or$cond3 = $or$cond & $24;
 do {
  if ($or$cond3) {
   $25 = HEAP32[$pSorter>>2]|0;
   $26 = ((($25)) + 60|0);
   $27 = HEAP8[$26>>0]|0;
   $28 = $27&255;
   $29 = $28 & 1;
   $30 = $29&255;
   HEAP8[$26>>0] = $30;
  } else {
   $31 = HEAP32[$t>>2]|0;
   $32 = ($31|0)>(10);
   if ($32) {
    $33 = HEAP32[$t>>2]|0;
    $34 = $33 & 1;
    $35 = ($34|0)!=(0);
    if ($35) {
     $36 = HEAP32[$pSorter>>2]|0;
     $37 = ((($36)) + 60|0);
     $38 = HEAP8[$37>>0]|0;
     $39 = $38&255;
     $40 = $39 & 2;
     $41 = $40&255;
     HEAP8[$37>>0] = $41;
     break;
    }
   }
   $42 = HEAP32[$pSorter>>2]|0;
   $43 = ((($42)) + 60|0);
   HEAP8[$43>>0] = 0;
  }
 } while(0);
 $44 = HEAP32[$2>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) + 8)|0;
 HEAP32[$nReq>>2] = $47;
 $48 = HEAP32[$2>>2]|0;
 $49 = ((($48)) + 12|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = HEAP32[$2>>2]|0;
 $52 = ((($51)) + 12|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = ($53|0)<(0);
 $55 = $54 << 31 >> 31;
 $56 = (_sqlite3VarintLen($53,$55)|0);
 $57 = (($50) + ($56))|0;
 HEAP32[$nPMA>>2] = $57;
 $58 = HEAP32[$pSorter>>2]|0;
 $59 = ((($58)) + 4|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)!=(0);
 if ($61) {
  $62 = HEAP32[$pSorter>>2]|0;
  $63 = ((($62)) + 36|0);
  $64 = ((($63)) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)!=(0|0);
  $67 = HEAP32[$pSorter>>2]|0;
  if ($66) {
   $68 = ((($67)) + 48|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ($69|0)!=(0);
   if ($70) {
    $71 = HEAP32[$pSorter>>2]|0;
    $72 = ((($71)) + 48|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = HEAP32[$nReq>>2]|0;
    $75 = (($73) + ($74))|0;
    $76 = HEAP32[$pSorter>>2]|0;
    $77 = ((($76)) + 4|0);
    $78 = HEAP32[$77>>2]|0;
    $79 = ($75|0)>($78|0);
    $81 = $79;
   } else {
    $81 = 0;
   }
   $80 = $81&1;
   HEAP32[$bFlush>>2] = $80;
  } else {
   $82 = ((($67)) + 36|0);
   $83 = ((($82)) + 8|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$pSorter>>2]|0;
   $86 = ((($85)) + 4|0);
   $87 = HEAP32[$86>>2]|0;
   $88 = ($84|0)>($87|0);
   if ($88) {
    $99 = 1;
   } else {
    $89 = HEAP32[$pSorter>>2]|0;
    $90 = ((($89)) + 36|0);
    $91 = ((($90)) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = HEAP32[$pSorter>>2]|0;
    $94 = HEAP32[$93>>2]|0;
    $95 = ($92|0)>($94|0);
    if ($95) {
     $96 = (_sqlite3HeapNearlyFull()|0);
     $97 = ($96|0)!=(0);
     $99 = $97;
    } else {
     $99 = 0;
    }
   }
   $98 = $99&1;
   HEAP32[$bFlush>>2] = $98;
  }
  $100 = HEAP32[$bFlush>>2]|0;
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$pSorter>>2]|0;
   $103 = (_vdbeSorterFlushPMA($102)|0);
   HEAP32[$rc>>2] = $103;
   $104 = HEAP32[$pSorter>>2]|0;
   $105 = ((($104)) + 36|0);
   $106 = ((($105)) + 8|0);
   HEAP32[$106>>2] = 0;
   $107 = HEAP32[$pSorter>>2]|0;
   $108 = ((($107)) + 48|0);
   HEAP32[$108>>2] = 0;
  }
 }
 $109 = HEAP32[$nPMA>>2]|0;
 $110 = HEAP32[$pSorter>>2]|0;
 $111 = ((($110)) + 36|0);
 $112 = ((($111)) + 8|0);
 $113 = HEAP32[$112>>2]|0;
 $114 = (($113) + ($109))|0;
 HEAP32[$112>>2] = $114;
 $115 = HEAP32[$nPMA>>2]|0;
 $116 = HEAP32[$pSorter>>2]|0;
 $117 = ((($116)) + 8|0);
 $118 = HEAP32[$117>>2]|0;
 $119 = ($115|0)>($118|0);
 if ($119) {
  $120 = HEAP32[$nPMA>>2]|0;
  $121 = HEAP32[$pSorter>>2]|0;
  $122 = ((($121)) + 8|0);
  HEAP32[$122>>2] = $120;
 }
 $123 = HEAP32[$pSorter>>2]|0;
 $124 = ((($123)) + 36|0);
 $125 = ((($124)) + 4|0);
 $126 = HEAP32[$125>>2]|0;
 $127 = ($126|0)!=(0|0);
 do {
  if ($127) {
   $128 = HEAP32[$pSorter>>2]|0;
   $129 = ((($128)) + 48|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = HEAP32[$nReq>>2]|0;
   $132 = (($130) + ($131))|0;
   HEAP32[$nMin>>2] = $132;
   $133 = HEAP32[$nMin>>2]|0;
   $134 = HEAP32[$pSorter>>2]|0;
   $135 = ((($134)) + 52|0);
   $136 = HEAP32[$135>>2]|0;
   $137 = ($133|0)>($136|0);
   do {
    if ($137) {
     $138 = HEAP32[$pSorter>>2]|0;
     $139 = ((($138)) + 36|0);
     $140 = HEAP32[$139>>2]|0;
     $141 = HEAP32[$pSorter>>2]|0;
     $142 = ((($141)) + 36|0);
     $143 = ((($142)) + 4|0);
     $144 = HEAP32[$143>>2]|0;
     $145 = $140;
     $146 = $144;
     $147 = (($145) - ($146))|0;
     HEAP32[$iListOff>>2] = $147;
     $148 = HEAP32[$pSorter>>2]|0;
     $149 = ((($148)) + 52|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = $150<<1;
     HEAP32[$nNew>>2] = $151;
     while(1) {
      $152 = HEAP32[$nNew>>2]|0;
      $153 = HEAP32[$nMin>>2]|0;
      $154 = ($152|0)<($153|0);
      $155 = HEAP32[$nNew>>2]|0;
      if (!($154)) {
       break;
      }
      $156 = $155<<1;
      HEAP32[$nNew>>2] = $156;
     }
     $157 = HEAP32[$pSorter>>2]|0;
     $158 = ((($157)) + 4|0);
     $159 = HEAP32[$158>>2]|0;
     $160 = ($155|0)>($159|0);
     if ($160) {
      $161 = HEAP32[$pSorter>>2]|0;
      $162 = ((($161)) + 4|0);
      $163 = HEAP32[$162>>2]|0;
      HEAP32[$nNew>>2] = $163;
     }
     $164 = HEAP32[$nNew>>2]|0;
     $165 = HEAP32[$nMin>>2]|0;
     $166 = ($164|0)<($165|0);
     if ($166) {
      $167 = HEAP32[$nMin>>2]|0;
      HEAP32[$nNew>>2] = $167;
     }
     $168 = HEAP32[$pSorter>>2]|0;
     $169 = ((($168)) + 36|0);
     $170 = ((($169)) + 4|0);
     $171 = HEAP32[$170>>2]|0;
     $172 = HEAP32[$nNew>>2]|0;
     $173 = ($172|0)<(0);
     $174 = $173 << 31 >> 31;
     $175 = (_sqlite3Realloc($171,$172,$174)|0);
     HEAP32[$aNew>>2] = $175;
     $176 = HEAP32[$aNew>>2]|0;
     $177 = ($176|0)!=(0|0);
     if ($177) {
      $178 = HEAP32[$iListOff>>2]|0;
      $179 = HEAP32[$aNew>>2]|0;
      $180 = (($179) + ($178)|0);
      $181 = HEAP32[$pSorter>>2]|0;
      $182 = ((($181)) + 36|0);
      HEAP32[$182>>2] = $180;
      $183 = HEAP32[$aNew>>2]|0;
      $184 = HEAP32[$pSorter>>2]|0;
      $185 = ((($184)) + 36|0);
      $186 = ((($185)) + 4|0);
      HEAP32[$186>>2] = $183;
      $187 = HEAP32[$nNew>>2]|0;
      $188 = HEAP32[$pSorter>>2]|0;
      $189 = ((($188)) + 52|0);
      HEAP32[$189>>2] = $187;
      break;
     }
     HEAP32[$0>>2] = 7;
     $248 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($248|0);
    }
   } while(0);
   $190 = HEAP32[$pSorter>>2]|0;
   $191 = ((($190)) + 48|0);
   $192 = HEAP32[$191>>2]|0;
   $193 = HEAP32[$pSorter>>2]|0;
   $194 = ((($193)) + 36|0);
   $195 = ((($194)) + 4|0);
   $196 = HEAP32[$195>>2]|0;
   $197 = (($196) + ($192)|0);
   HEAP32[$pNew>>2] = $197;
   $198 = HEAP32[$nReq>>2]|0;
   $199 = (($198) + 7)|0;
   $200 = $199 & -8;
   $201 = HEAP32[$pSorter>>2]|0;
   $202 = ((($201)) + 48|0);
   $203 = HEAP32[$202>>2]|0;
   $204 = (($203) + ($200))|0;
   HEAP32[$202>>2] = $204;
   $205 = HEAP32[$pSorter>>2]|0;
   $206 = ((($205)) + 36|0);
   $207 = HEAP32[$206>>2]|0;
   $208 = ($207|0)!=(0|0);
   if ($208) {
    $209 = HEAP32[$pSorter>>2]|0;
    $210 = ((($209)) + 36|0);
    $211 = HEAP32[$210>>2]|0;
    $212 = HEAP32[$pSorter>>2]|0;
    $213 = ((($212)) + 36|0);
    $214 = ((($213)) + 4|0);
    $215 = HEAP32[$214>>2]|0;
    $216 = $211;
    $217 = $215;
    $218 = (($216) - ($217))|0;
    $219 = HEAP32[$pNew>>2]|0;
    $220 = ((($219)) + 4|0);
    HEAP32[$220>>2] = $218;
   }
  } else {
   $221 = HEAP32[$nReq>>2]|0;
   $222 = ($221|0)<(0);
   $223 = $222 << 31 >> 31;
   $224 = (_sqlite3Malloc($221,$223)|0);
   HEAP32[$pNew>>2] = $224;
   $225 = HEAP32[$pNew>>2]|0;
   $226 = ($225|0)==(0|0);
   if (!($226)) {
    $227 = HEAP32[$pSorter>>2]|0;
    $228 = ((($227)) + 36|0);
    $229 = HEAP32[$228>>2]|0;
    $230 = HEAP32[$pNew>>2]|0;
    $231 = ((($230)) + 4|0);
    HEAP32[$231>>2] = $229;
    break;
   }
   HEAP32[$0>>2] = 7;
   $248 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($248|0);
  }
 } while(0);
 $232 = HEAP32[$pNew>>2]|0;
 $233 = ((($232)) + 8|0);
 $234 = HEAP32[$2>>2]|0;
 $235 = ((($234)) + 16|0);
 $236 = HEAP32[$235>>2]|0;
 $237 = HEAP32[$2>>2]|0;
 $238 = ((($237)) + 12|0);
 $239 = HEAP32[$238>>2]|0;
 _memcpy(($233|0),($236|0),($239|0))|0;
 $240 = HEAP32[$2>>2]|0;
 $241 = ((($240)) + 12|0);
 $242 = HEAP32[$241>>2]|0;
 $243 = HEAP32[$pNew>>2]|0;
 HEAP32[$243>>2] = $242;
 $244 = HEAP32[$pNew>>2]|0;
 $245 = HEAP32[$pSorter>>2]|0;
 $246 = ((($245)) + 36|0);
 HEAP32[$246>>2] = $244;
 $247 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $247;
 $248 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($248|0);
}
function _sqlite3VdbeIdxRowid($db,$pCur,$rowid) {
 $db = $db|0;
 $pCur = $pCur|0;
 $rowid = $rowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $lenRowid = 0, $m = 0, $nCellKey = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $szHdr = 0;
 var $typeRowid = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $nCellKey = sp + 80|0;
 $rc = sp + 100|0;
 $szHdr = sp + 96|0;
 $typeRowid = sp + 92|0;
 $lenRowid = sp + 88|0;
 $m = sp + 40|0;
 $v = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pCur;
 HEAP32[$3>>2] = $rowid;
 $4 = $nCellKey;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3BtreePayloadSize($8)|0);
 $10 = $nCellKey;
 $11 = $10;
 HEAP32[$11>>2] = $9;
 $12 = (($10) + 4)|0;
 $13 = $12;
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemInit($m,$14,0);
 $15 = HEAP32[$2>>2]|0;
 $16 = $nCellKey;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = (($16) + 4)|0;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (_sqlite3VdbeMemFromBtree($15,0,$18,1,$m)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $25;
  $92 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($92|0);
 }
 $26 = ((($m)) + 16|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = ($29|0)<(128);
 $31 = ((($m)) + 16|0);
 $32 = HEAP32[$31>>2]|0;
 if ($30) {
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  HEAP32[$szHdr>>2] = $34;
 } else {
  (_sqlite3GetVarint32($32,$szHdr)|0);
 }
 $35 = HEAP32[$szHdr>>2]|0;
 $36 = ($35>>>0)<(3);
 if (!($36)) {
  $37 = HEAP32[$szHdr>>2]|0;
  $38 = ((($m)) + 12|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = ($37|0)>($39|0);
  if (!($40)) {
   $41 = HEAP32[$szHdr>>2]|0;
   $42 = (($41) - 1)|0;
   $43 = ((($m)) + 16|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = (($44) + ($42)|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = $46&255;
   $48 = ($47|0)<(128);
   $49 = HEAP32[$szHdr>>2]|0;
   $50 = (($49) - 1)|0;
   $51 = ((($m)) + 16|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = (($52) + ($50)|0);
   if ($48) {
    $54 = HEAP8[$53>>0]|0;
    $55 = $54&255;
    HEAP32[$typeRowid>>2] = $55;
   } else {
    (_sqlite3GetVarint32($53,$typeRowid)|0);
   }
   $56 = HEAP32[$typeRowid>>2]|0;
   $57 = ($56>>>0)<(1);
   $58 = HEAP32[$typeRowid>>2]|0;
   $59 = ($58>>>0)>(9);
   $or$cond = $57 | $59;
   $60 = HEAP32[$typeRowid>>2]|0;
   $61 = ($60|0)==(7);
   $or$cond3 = $or$cond | $61;
   if (!($or$cond3)) {
    $62 = HEAP32[$typeRowid>>2]|0;
    $63 = (35061 + ($62)|0);
    $64 = HEAP8[$63>>0]|0;
    $65 = $64&255;
    HEAP32[$lenRowid>>2] = $65;
    $66 = ((($m)) + 12|0);
    $67 = HEAP32[$66>>2]|0;
    $68 = HEAP32[$szHdr>>2]|0;
    $69 = HEAP32[$lenRowid>>2]|0;
    $70 = (($68) + ($69))|0;
    $71 = ($67>>>0)<($70>>>0);
    if (!($71)) {
     $72 = ((($m)) + 12|0);
     $73 = HEAP32[$72>>2]|0;
     $74 = HEAP32[$lenRowid>>2]|0;
     $75 = (($73) - ($74))|0;
     $76 = ((($m)) + 16|0);
     $77 = HEAP32[$76>>2]|0;
     $78 = (($77) + ($75)|0);
     $79 = HEAP32[$typeRowid>>2]|0;
     (_sqlite3VdbeSerialGet($78,$79,$v)|0);
     $80 = $v;
     $81 = $80;
     $82 = HEAP32[$81>>2]|0;
     $83 = (($80) + 4)|0;
     $84 = $83;
     $85 = HEAP32[$84>>2]|0;
     $86 = HEAP32[$3>>2]|0;
     $87 = $86;
     $88 = $87;
     HEAP32[$88>>2] = $82;
     $89 = (($87) + 4)|0;
     $90 = $89;
     HEAP32[$90>>2] = $85;
     _sqlite3VdbeMemRelease($m);
     HEAP32[$0>>2] = 0;
     $92 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($92|0);
    }
   }
  }
 }
 _sqlite3VdbeMemRelease($m);
 $91 = (_sqlite3CorruptError(74890)|0);
 HEAP32[$0>>2] = $91;
 $92 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($92|0);
}
function _sqlite3VdbeIdxKeyCompare($db,$pC,$pUnpacked,$res) {
 $db = $db|0;
 $pC = $pC|0;
 $pUnpacked = $pUnpacked|0;
 $res = $res|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $m = 0, $nCellKey = 0, $or$cond = 0, $pCur = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 72|0;
 $1 = sp + 68|0;
 $2 = sp + 64|0;
 $3 = sp + 60|0;
 $4 = sp + 56|0;
 $nCellKey = sp + 40|0;
 $rc = sp + 52|0;
 $pCur = sp + 48|0;
 $m = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pC;
 HEAP32[$3>>2] = $pUnpacked;
 HEAP32[$4>>2] = $res;
 $5 = $nCellKey;
 $6 = $5;
 HEAP32[$6>>2] = 0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 16|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pCur>>2] = $11;
 $12 = HEAP32[$pCur>>2]|0;
 $13 = (_sqlite3BtreePayloadSize($12)|0);
 $14 = $nCellKey;
 $15 = $14;
 HEAP32[$15>>2] = $13;
 $16 = (($14) + 4)|0;
 $17 = $16;
 HEAP32[$17>>2] = 0;
 $18 = $nCellKey;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($18) + 4)|0;
 $22 = $21;
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)<(0);
 $25 = ($20>>>0)<=(0);
 $26 = ($23|0)==(0);
 $27 = $26 & $25;
 $28 = $24 | $27;
 $29 = $nCellKey;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (($29) + 4)|0;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)>(0);
 $36 = ($31>>>0)>(2147483647);
 $37 = ($34|0)==(0);
 $38 = $37 & $36;
 $39 = $35 | $38;
 $or$cond = $28 | $39;
 if ($or$cond) {
  $40 = HEAP32[$4>>2]|0;
  HEAP32[$40>>2] = 0;
  $41 = (_sqlite3CorruptError(74923)|0);
  HEAP32[$0>>2] = $41;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 $42 = HEAP32[$1>>2]|0;
 _sqlite3VdbeMemInit($m,$42,0);
 $43 = HEAP32[$pCur>>2]|0;
 $44 = $nCellKey;
 $45 = $44;
 $46 = HEAP32[$45>>2]|0;
 $47 = (($44) + 4)|0;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (_sqlite3VdbeMemFromBtree($43,0,$46,1,$m)|0);
 HEAP32[$rc>>2] = $50;
 $51 = HEAP32[$rc>>2]|0;
 $52 = ($51|0)!=(0);
 if ($52) {
  $53 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $53;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 } else {
  $54 = ((($m)) + 12|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ((($m)) + 16|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = HEAP32[$3>>2]|0;
  $59 = (_sqlite3VdbeRecordCompare($55,$57,$58)|0);
  $60 = HEAP32[$4>>2]|0;
  HEAP32[$60>>2] = $59;
  _sqlite3VdbeMemRelease($m);
  HEAP32[$0>>2] = 0;
  $61 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($61|0);
 }
 return (0)|0;
}
function _sqlite3BtreeDropTable($p,$iTable,$piMoved) {
 $p = $p|0;
 $iTable = $iTable|0;
 $piMoved = $piMoved|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $piMoved;
 $3 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = (_btreeDropTable($4,$5,$6)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($8|0);
}
function _sqlite3RootPageMoved($db,$iDb,$iFrom,$iTo) {
 $db = $db|0;
 $iDb = $iDb|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDb = 0, $pElem = 0, $pHash = 0, $pIdx = 0, $pTab = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pElem = sp + 16|0;
 $pHash = sp + 12|0;
 $pDb = sp + 8|0;
 $pTab = sp + 4|0;
 $pIdx = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $iFrom;
 HEAP32[$3>>2] = $iTo;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (($7) + ($4<<4)|0);
 HEAP32[$pDb>>2] = $8;
 $9 = HEAP32[$pDb>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($11)) + 8|0);
 HEAP32[$pHash>>2] = $12;
 $13 = HEAP32[$pHash>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 HEAP32[$pElem>>2] = $15;
 while(1) {
  $16 = HEAP32[$pElem>>2]|0;
  $17 = ($16|0)!=(0|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$pElem>>2]|0;
  $19 = ((($18)) + 8|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$pTab>>2] = $20;
  $21 = HEAP32[$pTab>>2]|0;
  $22 = ((($21)) + 28|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP32[$2>>2]|0;
  $25 = ($23|0)==($24|0);
  if ($25) {
   $26 = HEAP32[$3>>2]|0;
   $27 = HEAP32[$pTab>>2]|0;
   $28 = ((($27)) + 28|0);
   HEAP32[$28>>2] = $26;
  }
  $29 = HEAP32[$pElem>>2]|0;
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pElem>>2] = $30;
 }
 $31 = HEAP32[$pDb>>2]|0;
 $32 = ((($31)) + 12|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ((($33)) + 24|0);
 HEAP32[$pHash>>2] = $34;
 $35 = HEAP32[$pHash>>2]|0;
 $36 = ((($35)) + 8|0);
 $37 = HEAP32[$36>>2]|0;
 HEAP32[$pElem>>2] = $37;
 while(1) {
  $38 = HEAP32[$pElem>>2]|0;
  $39 = ($38|0)!=(0|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$pElem>>2]|0;
  $41 = ((($40)) + 8|0);
  $42 = HEAP32[$41>>2]|0;
  HEAP32[$pIdx>>2] = $42;
  $43 = HEAP32[$pIdx>>2]|0;
  $44 = ((($43)) + 44|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = HEAP32[$2>>2]|0;
  $47 = ($45|0)==($46|0);
  if ($47) {
   $48 = HEAP32[$3>>2]|0;
   $49 = HEAP32[$pIdx>>2]|0;
   $50 = ((($49)) + 44|0);
   HEAP32[$50>>2] = $48;
  }
  $51 = HEAP32[$pElem>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  HEAP32[$pElem>>2] = $52;
 }
 STACKTOP = sp;return;
}
function _sqlite3BtreeClearTable($p,$iTable,$pnChange) {
 $p = $p|0;
 $iTable = $iTable|0;
 $pnChange = $pnChange|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pBt = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTable;
 HEAP32[$2>>2] = $pnChange;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($6);
 $7 = HEAP32[$pBt>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_saveAllCursors($7,$8,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = (0)==($10|0);
 if (!($11)) {
  $17 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($17|0);
 }
 $12 = HEAP32[$0>>2]|0;
 _invalidateIncrblobCursors($12,0,0,1);
 $13 = HEAP32[$pBt>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (_clearDatabasePage($13,$14,0,$15)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($17|0);
}
function _sqlite3VdbeSorterReset($db,$pSorter) {
 $db = $db|0;
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pTask = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $i = sp + 4|0;
 $pTask = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSorter;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 _vdbeMergeEngineFree($4);
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 20|0);
 HEAP32[$6>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 59|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($7|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 64|0);
  $16 = (($15) + (($13*72)|0)|0);
  HEAP32[$pTask>>2] = $16;
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$pTask>>2]|0;
  _vdbeSortSubtaskCleanup($17,$18);
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP32[$pTask>>2]|0;
  $21 = ((($20)) + 8|0);
  HEAP32[$21>>2] = $19;
  $22 = HEAP32[$i>>2]|0;
  $23 = (($22) + 1)|0;
  HEAP32[$i>>2] = $23;
 }
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 36|0);
 $26 = ((($25)) + 4|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ($27|0)==(0|0);
 if ($28) {
  $29 = HEAP32[$1>>2]|0;
  $30 = ((($29)) + 36|0);
  $31 = HEAP32[$30>>2]|0;
  _vdbeSorterRecordFree(0,$31);
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 36|0);
 HEAP32[$33>>2] = 0;
 $34 = HEAP32[$1>>2]|0;
 $35 = ((($34)) + 36|0);
 $36 = ((($35)) + 8|0);
 HEAP32[$36>>2] = 0;
 $37 = HEAP32[$1>>2]|0;
 $38 = ((($37)) + 56|0);
 HEAP8[$38>>0] = 0;
 $39 = HEAP32[$1>>2]|0;
 $40 = ((($39)) + 48|0);
 HEAP32[$40>>2] = 0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 8|0);
 HEAP32[$42>>2] = 0;
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 32|0);
 $46 = HEAP32[$45>>2]|0;
 _sqlite3DbFree($43,$46);
 $47 = HEAP32[$1>>2]|0;
 $48 = ((($47)) + 32|0);
 HEAP32[$48>>2] = 0;
 STACKTOP = sp;return;
}
function _sqlite3BtreeClearTableOfCursor($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3BtreeClearTable($2,$5,0)|0);
 STACKTOP = sp;return ($6|0);
}
function _sqlite3UnlinkAndDeleteTable($db,$iDb,$zTabName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTabName = $zTabName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pDb = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zTabName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 HEAP32[$pDb>>2] = $7;
 $8 = HEAP32[$pDb>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$p>>2] = $13;
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$p>>2]|0;
 _sqlite3DeleteTable($14,$15);
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = $18 | 2;
 HEAP32[$17>>2] = $19;
 STACKTOP = sp;return;
}
function _sqlite3UnlinkAndDeleteIndex($db,$iDb,$zIdxName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zIdxName = $zIdxName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $p = 0, $pHash = 0, $pIndex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $pIndex = sp + 8|0;
 $pHash = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zIdxName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 HEAP32[$pHash>>2] = $10;
 $11 = HEAP32[$pHash>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$pIndex>>2] = $13;
 $14 = HEAP32[$pIndex>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  $57 = HEAP32[$0>>2]|0;
  $58 = ((($57)) + 24|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = $59 | 2;
  HEAP32[$58>>2] = $60;
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$pIndex>>2]|0;
 $17 = ((($16)) + 12|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pIndex>>2]|0;
 $22 = ($20|0)==($21|0);
 $23 = HEAP32[$pIndex>>2]|0;
 if ($22) {
  $24 = ((($23)) + 20|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$pIndex>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 8|0);
  HEAP32[$29>>2] = $25;
 } else {
  $30 = ((($23)) + 12|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  HEAP32[$p>>2] = $33;
  while(1) {
   $34 = HEAP32[$p>>2]|0;
   $35 = ($34|0)!=(0|0);
   if ($35) {
    $36 = HEAP32[$p>>2]|0;
    $37 = ((($36)) + 20|0);
    $38 = HEAP32[$37>>2]|0;
    $39 = HEAP32[$pIndex>>2]|0;
    $40 = ($38|0)!=($39|0);
    $61 = $40;
   } else {
    $61 = 0;
   }
   $41 = HEAP32[$p>>2]|0;
   if (!($61)) {
    break;
   }
   $42 = ((($41)) + 20|0);
   $43 = HEAP32[$42>>2]|0;
   HEAP32[$p>>2] = $43;
  }
  $44 = ($41|0)!=(0|0);
  if ($44) {
   $45 = HEAP32[$p>>2]|0;
   $46 = ((($45)) + 20|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = HEAP32[$pIndex>>2]|0;
   $49 = ($47|0)==($48|0);
   if ($49) {
    $50 = HEAP32[$pIndex>>2]|0;
    $51 = ((($50)) + 20|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = HEAP32[$p>>2]|0;
    $54 = ((($53)) + 20|0);
    HEAP32[$54>>2] = $52;
   }
  }
 }
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$pIndex>>2]|0;
 _freeIndex($55,$56);
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 24|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = $59 | 2;
 HEAP32[$58>>2] = $60;
 STACKTOP = sp;return;
}
function _sqlite3UnlinkAndDeleteTrigger($db,$iDb,$zName) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pHash = 0, $pTab = 0, $pTrigger = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pTrigger = sp + 12|0;
 $pHash = sp + 8|0;
 $pTab = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zName;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<4)|0);
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 40|0);
 HEAP32[$pHash>>2] = $10;
 $11 = HEAP32[$pHash>>2]|0;
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3HashInsert($11,$12,0)|0);
 HEAP32[$pTrigger>>2] = $13;
 $14 = HEAP32[$pTrigger>>2]|0;
 $15 = ($14|0)!=(0|0);
 if (!($15)) {
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$pTrigger>>2]|0;
 $17 = ((($16)) + 20|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$pTrigger>>2]|0;
 $20 = ((($19)) + 24|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($18|0)==($21|0);
 if ($22) {
  $23 = HEAP32[$pTrigger>>2]|0;
  $24 = (_tableOfTrigger($23)|0);
  HEAP32[$pTab>>2] = $24;
  $25 = HEAP32[$pTab>>2]|0;
  $26 = ((($25)) + 60|0);
  HEAP32[$pp>>2] = $26;
  while(1) {
   $27 = HEAP32[$pp>>2]|0;
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$pTrigger>>2]|0;
   $30 = ($28|0)!=($29|0);
   $31 = HEAP32[$pp>>2]|0;
   $32 = HEAP32[$31>>2]|0;
   $33 = ((($32)) + 32|0);
   if (!($30)) {
    break;
   }
   HEAP32[$pp>>2] = $33;
  }
  $34 = HEAP32[$33>>2]|0;
  $35 = HEAP32[$pp>>2]|0;
  HEAP32[$35>>2] = $34;
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$pTrigger>>2]|0;
 _sqlite3DeleteTrigger($36,$37);
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 24|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = $40 | 2;
 HEAP32[$39>>2] = $41;
 STACKTOP = sp;return;
}
function _sqlite3BtreeIntegrityCheck($p,$aRoot,$nRoot,$mxErr,$pnErr) {
 $p = $p|0;
 $aRoot = $aRoot|0;
 $nRoot = $nRoot|0;
 $mxErr = $mxErr|0;
 $pnErr = $pnErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0;
 var $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0;
 var $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0;
 var $98 = 0, $99 = 0, $i = 0, $notUsed = 0, $pBt = 0, $sCheck = 0, $savedDbFlags = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $vararg_buffer1 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $3 = sp + 112|0;
 $4 = sp + 108|0;
 $i = sp + 104|0;
 $sCheck = sp + 32|0;
 $pBt = sp + 24|0;
 $savedDbFlags = sp + 20|0;
 $zErr = sp + 128|0;
 $notUsed = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $aRoot;
 HEAP32[$2>>2] = $nRoot;
 HEAP32[$3>>2] = $mxErr;
 HEAP32[$4>>2] = $pnErr;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$pBt>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 24|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$savedDbFlags>>2] = $12;
 $13 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($13);
 $14 = HEAP32[$pBt>>2]|0;
 HEAP32[$sCheck>>2] = $14;
 $15 = HEAP32[$pBt>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = ((($sCheck)) + 4|0);
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$sCheck>>2]|0;
 $19 = (_btreePagecount($18)|0);
 $20 = ((($sCheck)) + 12|0);
 HEAP32[$20>>2] = $19;
 $21 = HEAP32[$3>>2]|0;
 $22 = ((($sCheck)) + 16|0);
 HEAP32[$22>>2] = $21;
 $23 = ((($sCheck)) + 20|0);
 HEAP32[$23>>2] = 0;
 $24 = ((($sCheck)) + 24|0);
 HEAP32[$24>>2] = 0;
 $25 = ((($sCheck)) + 28|0);
 HEAP32[$25>>2] = 0;
 $26 = ((($sCheck)) + 32|0);
 HEAP32[$26>>2] = 0;
 $27 = ((($sCheck)) + 36|0);
 HEAP32[$27>>2] = 0;
 $28 = ((($sCheck)) + 8|0);
 HEAP32[$28>>2] = 0;
 $29 = ((($sCheck)) + 68|0);
 HEAP32[$29>>2] = 0;
 $30 = ((($sCheck)) + 40|0);
 _sqlite3StrAccumInit($30,0,$zErr,100,1000000000);
 $31 = ((($sCheck)) + 40|0);
 $32 = ((($31)) + 25|0);
 HEAP8[$32>>0] = 1;
 $33 = ((($sCheck)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ($34|0)==(0);
 L1: do {
  if (!($35)) {
   $36 = ((($sCheck)) + 12|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = (($37>>>0) / 8)&-1;
   $39 = (($38) + 1)|0;
   $40 = (_sqlite3MallocZero($39,0)|0);
   $41 = ((($sCheck)) + 8|0);
   HEAP32[$41>>2] = $40;
   $42 = ((($sCheck)) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)!=(0|0);
   if (!($44)) {
    $45 = ((($sCheck)) + 24|0);
    HEAP32[$45>>2] = 1;
    break;
   }
   $46 = HEAP32[$pBt>>2]|0;
   $47 = ((($46)) + 32|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (_sqlite3PageMalloc($48)|0);
   $50 = ((($sCheck)) + 68|0);
   HEAP32[$50>>2] = $49;
   $51 = ((($sCheck)) + 68|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)==(0|0);
   if ($53) {
    $54 = ((($sCheck)) + 24|0);
    HEAP32[$54>>2] = 1;
    break;
   }
   $55 = HEAP32[1536>>2]|0;
   $56 = HEAP32[$pBt>>2]|0;
   $57 = ((($56)) + 32|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = (($55>>>0) / ($58>>>0))&-1;
   $60 = (($59) + 1)|0;
   HEAP32[$i>>2] = $60;
   $61 = HEAP32[$i>>2]|0;
   $62 = ((($sCheck)) + 12|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = ($61>>>0)<=($63>>>0);
   if ($64) {
    $65 = HEAP32[$i>>2]|0;
    _setPageReferenced($sCheck,$65);
   }
   $66 = ((($sCheck)) + 28|0);
   HEAP32[$66>>2] = 41095;
   $67 = HEAP32[$pBt>>2]|0;
   $68 = ((($67)) + 12|0);
   $69 = HEAP32[$68>>2]|0;
   $70 = ((($69)) + 56|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = ((($71)) + 32|0);
   $73 = (_sqlite3Get4byte($72)|0);
   $74 = HEAP32[$pBt>>2]|0;
   $75 = ((($74)) + 12|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = ((($76)) + 56|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ((($78)) + 36|0);
   $80 = (_sqlite3Get4byte($79)|0);
   _checkList($sCheck,1,$73,$80);
   $81 = ((($sCheck)) + 28|0);
   HEAP32[$81>>2] = 0;
   $82 = HEAP32[$pBt>>2]|0;
   $83 = ((($82)) + 4|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ((($84)) + 24|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = $86 & -536870913;
   HEAP32[$85>>2] = $87;
   HEAP32[$i>>2] = 0;
   while(1) {
    $88 = HEAP32[$i>>2]|0;
    $89 = HEAP32[$2>>2]|0;
    $90 = ($88|0)<($89|0);
    $91 = ((($sCheck)) + 16|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)!=(0);
    $94 = $90 ? $93 : 0;
    if (!($94)) {
     break;
    }
    $95 = HEAP32[$i>>2]|0;
    $96 = HEAP32[$1>>2]|0;
    $97 = (($96) + ($95<<2)|0);
    $98 = HEAP32[$97>>2]|0;
    $99 = ($98|0)==(0);
    if (!($99)) {
     $100 = HEAP32[$pBt>>2]|0;
     $101 = ((($100)) + 17|0);
     $102 = HEAP8[$101>>0]|0;
     $103 = $102&255;
     $104 = ($103|0)!=(0);
     if ($104) {
      $105 = HEAP32[$i>>2]|0;
      $106 = HEAP32[$1>>2]|0;
      $107 = (($106) + ($105<<2)|0);
      $108 = HEAP32[$107>>2]|0;
      $109 = ($108|0)>(1);
      if ($109) {
       $110 = HEAP32[$i>>2]|0;
       $111 = HEAP32[$1>>2]|0;
       $112 = (($111) + ($110<<2)|0);
       $113 = HEAP32[$112>>2]|0;
       _checkPtrmap($sCheck,$113,1,0);
      }
     }
     $114 = HEAP32[$i>>2]|0;
     $115 = HEAP32[$1>>2]|0;
     $116 = (($115) + ($114<<2)|0);
     $117 = HEAP32[$116>>2]|0;
     (_checkTreePage($sCheck,$117,$notUsed,-1,2147483647)|0);
    }
    $118 = HEAP32[$i>>2]|0;
    $119 = (($118) + 1)|0;
    HEAP32[$i>>2] = $119;
   }
   $120 = HEAP32[$savedDbFlags>>2]|0;
   $121 = HEAP32[$pBt>>2]|0;
   $122 = ((($121)) + 4|0);
   $123 = HEAP32[$122>>2]|0;
   $124 = ((($123)) + 24|0);
   HEAP32[$124>>2] = $120;
   HEAP32[$i>>2] = 1;
   while(1) {
    $125 = HEAP32[$i>>2]|0;
    $126 = ((($sCheck)) + 12|0);
    $127 = HEAP32[$126>>2]|0;
    $128 = ($125>>>0)<=($127>>>0);
    $129 = ((($sCheck)) + 16|0);
    $130 = HEAP32[$129>>2]|0;
    $131 = ($130|0)!=(0);
    $132 = $128 ? $131 : 0;
    if (!($132)) {
     break L1;
    }
    $133 = HEAP32[$i>>2]|0;
    $134 = (_getPageReferenced($sCheck,$133)|0);
    $135 = ($134|0)==(0);
    do {
     if ($135) {
      $136 = HEAP32[$pBt>>2]|0;
      $137 = HEAP32[$i>>2]|0;
      $138 = (_ptrmapPageno($136,$137)|0);
      $139 = HEAP32[$i>>2]|0;
      $140 = ($138|0)!=($139|0);
      if (!($140)) {
       $141 = HEAP32[$pBt>>2]|0;
       $142 = ((($141)) + 17|0);
       $143 = HEAP8[$142>>0]|0;
       $144 = ($143<<24>>24)!=(0);
       if ($144) {
        break;
       }
      }
      $145 = HEAP32[$i>>2]|0;
      HEAP32[$vararg_buffer>>2] = $145;
      _checkAppendMsg($sCheck,41111,$vararg_buffer);
     }
    } while(0);
    $146 = HEAP32[$i>>2]|0;
    $147 = (_getPageReferenced($sCheck,$146)|0);
    $148 = ($147|0)!=(0);
    if ($148) {
     $149 = HEAP32[$pBt>>2]|0;
     $150 = HEAP32[$i>>2]|0;
     $151 = (_ptrmapPageno($149,$150)|0);
     $152 = HEAP32[$i>>2]|0;
     $153 = ($151|0)==($152|0);
     if ($153) {
      $154 = HEAP32[$pBt>>2]|0;
      $155 = ((($154)) + 17|0);
      $156 = HEAP8[$155>>0]|0;
      $157 = $156&255;
      $158 = ($157|0)!=(0);
      if ($158) {
       $159 = HEAP32[$i>>2]|0;
       HEAP32[$vararg_buffer1>>2] = $159;
       _checkAppendMsg($sCheck,41133,$vararg_buffer1);
      }
     }
    }
    $160 = HEAP32[$i>>2]|0;
    $161 = (($160) + 1)|0;
    HEAP32[$i>>2] = $161;
   }
  }
 } while(0);
 $162 = ((($sCheck)) + 68|0);
 $163 = HEAP32[$162>>2]|0;
 _sqlite3PageFree($163);
 $164 = ((($sCheck)) + 8|0);
 $165 = HEAP32[$164>>2]|0;
 _sqlite3_free($165);
 $166 = ((($sCheck)) + 24|0);
 $167 = HEAP32[$166>>2]|0;
 $168 = ($167|0)!=(0);
 if ($168) {
  $169 = ((($sCheck)) + 40|0);
  _sqlite3StrAccumReset($169);
  $170 = ((($sCheck)) + 20|0);
  $171 = HEAP32[$170>>2]|0;
  $172 = (($171) + 1)|0;
  HEAP32[$170>>2] = $172;
 }
 $173 = ((($sCheck)) + 20|0);
 $174 = HEAP32[$173>>2]|0;
 $175 = HEAP32[$4>>2]|0;
 HEAP32[$175>>2] = $174;
 $176 = ((($sCheck)) + 20|0);
 $177 = HEAP32[$176>>2]|0;
 $178 = ($177|0)==(0);
 if (!($178)) {
  $180 = ((($sCheck)) + 40|0);
  $181 = (_sqlite3StrAccumFinish($180)|0);
  STACKTOP = sp;return ($181|0);
 }
 $179 = ((($sCheck)) + 40|0);
 _sqlite3StrAccumReset($179);
 $180 = ((($sCheck)) + 40|0);
 $181 = (_sqlite3StrAccumFinish($180)|0);
 STACKTOP = sp;return ($181|0);
}
function _sqlite3VdbeMemSetRowSet($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $pMem;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 32|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$db>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemRelease($4);
 $5 = HEAP32[$db>>2]|0;
 $6 = (_sqlite3DbMallocRawNN($5,64,0)|0);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 20|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$db>>2]|0;
 $10 = ((($9)) + 69|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = ($11<<24>>24)!=(0);
 if ($12) {
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 8|0);
  HEAP16[$14>>1] = 1;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 24|0);
  HEAP32[$16>>2] = 0;
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$db>>2]|0;
  $18 = HEAP32[$0>>2]|0;
  $19 = ((($18)) + 20|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3DbMallocSize($17,$20)|0);
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 24|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$db>>2]|0;
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 20|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 24|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3RowSetInit($24,$27,$30)|0);
  $32 = HEAP32[$0>>2]|0;
  HEAP32[$32>>2] = $31;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 8|0);
  HEAP16[$34>>1] = 32;
  STACKTOP = sp;return;
 }
}
function _sqlite3RowSetInsert($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pEntry = 0, $pLast = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $pEntry = sp + 12|0;
 $pLast = sp + 8|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_rowSetEntryAlloc($8)|0);
 HEAP32[$pEntry>>2] = $9;
 $10 = HEAP32[$pEntry>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = $3;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$pEntry>>2]|0;
 $19 = $18;
 $20 = $19;
 HEAP32[$20>>2] = $14;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = $17;
 $23 = HEAP32[$pEntry>>2]|0;
 $24 = ((($23)) + 8|0);
 HEAP32[$24>>2] = 0;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 HEAP32[$pLast>>2] = $27;
 $28 = HEAP32[$pLast>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = $3;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = HEAP32[$pLast>>2]|0;
  $37 = $36;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = ($35|0)<($42|0);
  $44 = ($32>>>0)<=($39>>>0);
  $45 = ($35|0)==($42|0);
  $46 = $45 & $44;
  $47 = $43 | $46;
  if ($47) {
   $48 = HEAP32[$2>>2]|0;
   $49 = ((($48)) + 26|0);
   $50 = HEAP16[$49>>1]|0;
   $51 = $50&65535;
   $52 = $51 & -2;
   $53 = $52&65535;
   HEAP16[$49>>1] = $53;
  }
  $54 = HEAP32[$pEntry>>2]|0;
  $55 = HEAP32[$pLast>>2]|0;
  $56 = ((($55)) + 8|0);
  HEAP32[$56>>2] = $54;
 } else {
  $57 = HEAP32[$pEntry>>2]|0;
  $58 = HEAP32[$2>>2]|0;
  $59 = ((($58)) + 8|0);
  HEAP32[$59>>2] = $57;
 }
 $60 = HEAP32[$pEntry>>2]|0;
 $61 = HEAP32[$2>>2]|0;
 $62 = ((($61)) + 12|0);
 HEAP32[$62>>2] = $60;
 STACKTOP = sp;return;
}
function _sqlite3RowSetNext($p,$pRowid) {
 $p = $p|0;
 $pRowid = $pRowid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pRowid;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 26|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 2;
 $8 = ($7|0)==(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 26|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = $12 & 1;
  $14 = ($13|0)==(0);
  if ($14) {
   $15 = HEAP32[$1>>2]|0;
   $16 = ((($15)) + 8|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = (_rowSetEntrySort($17)|0);
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 8|0);
   HEAP32[$20>>2] = $18;
  }
  $21 = HEAP32[$1>>2]|0;
  $22 = ((($21)) + 26|0);
  $23 = HEAP16[$22>>1]|0;
  $24 = $23&65535;
  $25 = $24 | 3;
  $26 = $25&65535;
  HEAP16[$22>>1] = $26;
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 8|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)!=(0|0);
 if (!($30)) {
  HEAP32[$0>>2] = 0;
  $57 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($57|0);
 }
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($31)) + 8|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = $33;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = $40;
 $42 = $41;
 HEAP32[$42>>2] = $36;
 $43 = (($41) + 4)|0;
 $44 = $43;
 HEAP32[$44>>2] = $39;
 $45 = HEAP32[$1>>2]|0;
 $46 = ((($45)) + 8|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = ((($47)) + 8|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = HEAP32[$1>>2]|0;
 $51 = ((($50)) + 8|0);
 HEAP32[$51>>2] = $49;
 $52 = HEAP32[$1>>2]|0;
 $53 = ((($52)) + 8|0);
 $54 = HEAP32[$53>>2]|0;
 $55 = ($54|0)==(0|0);
 if ($55) {
  $56 = HEAP32[$1>>2]|0;
  _sqlite3RowSetClear($56);
 }
 HEAP32[$0>>2] = 1;
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _sqlite3RowSetTest($pRowSet,$iBatch,$0,$1) {
 $pRowSet = $pRowSet|0;
 $iBatch = $iBatch|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $p = 0, $pAux = 0, $pTail = 0, $pTree = 0, $ppPrevTree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp;
 $p = sp + 24|0;
 $pTree = sp + 20|0;
 $ppPrevTree = sp + 16|0;
 $pAux = sp + 12|0;
 $pTail = sp + 8|0;
 HEAP32[$3>>2] = $pRowSet;
 HEAP32[$4>>2] = $iBatch;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$4>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)!=($13|0);
 if ($14) {
  $15 = HEAP32[$3>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$p>>2] = $17;
  $18 = HEAP32[$p>>2]|0;
  $19 = ($18|0)!=(0|0);
  if ($19) {
   $20 = HEAP32[$3>>2]|0;
   $21 = ((($20)) + 20|0);
   HEAP32[$ppPrevTree>>2] = $21;
   $22 = HEAP32[$3>>2]|0;
   $23 = ((($22)) + 26|0);
   $24 = HEAP16[$23>>1]|0;
   $25 = $24&65535;
   $26 = $25 & 1;
   $27 = ($26|0)==(0);
   if ($27) {
    $28 = HEAP32[$p>>2]|0;
    $29 = (_rowSetEntrySort($28)|0);
    HEAP32[$p>>2] = $29;
   }
   $30 = HEAP32[$3>>2]|0;
   $31 = ((($30)) + 20|0);
   $32 = HEAP32[$31>>2]|0;
   HEAP32[$pTree>>2] = $32;
   while(1) {
    $33 = HEAP32[$pTree>>2]|0;
    $34 = ($33|0)!=(0|0);
    if (!($34)) {
     break;
    }
    $35 = HEAP32[$pTree>>2]|0;
    $36 = ((($35)) + 8|0);
    HEAP32[$ppPrevTree>>2] = $36;
    $37 = HEAP32[$pTree>>2]|0;
    $38 = ((($37)) + 12|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = ($39|0)==(0|0);
    if ($40) {
     label = 8;
     break;
    }
    $45 = HEAP32[$pTree>>2]|0;
    $46 = ((($45)) + 12|0);
    $47 = HEAP32[$46>>2]|0;
    _rowSetTreeToList($47,$pAux,$pTail);
    $48 = HEAP32[$pTree>>2]|0;
    $49 = ((($48)) + 12|0);
    HEAP32[$49>>2] = 0;
    $50 = HEAP32[$pAux>>2]|0;
    $51 = HEAP32[$p>>2]|0;
    $52 = (_rowSetEntryMerge($50,$51)|0);
    HEAP32[$p>>2] = $52;
    $53 = HEAP32[$pTree>>2]|0;
    $54 = ((($53)) + 8|0);
    $55 = HEAP32[$54>>2]|0;
    HEAP32[$pTree>>2] = $55;
   }
   if ((label|0) == 8) {
    $41 = HEAP32[$p>>2]|0;
    $42 = (_rowSetListToTree($41)|0);
    $43 = HEAP32[$pTree>>2]|0;
    $44 = ((($43)) + 12|0);
    HEAP32[$44>>2] = $42;
   }
   $56 = HEAP32[$pTree>>2]|0;
   $57 = ($56|0)==(0|0);
   if ($57) {
    $58 = HEAP32[$3>>2]|0;
    $59 = (_rowSetEntryAlloc($58)|0);
    HEAP32[$pTree>>2] = $59;
    $60 = HEAP32[$ppPrevTree>>2]|0;
    HEAP32[$60>>2] = $59;
    $61 = HEAP32[$pTree>>2]|0;
    $62 = ($61|0)!=(0|0);
    if ($62) {
     $63 = HEAP32[$pTree>>2]|0;
     $64 = $63;
     $65 = $64;
     HEAP32[$65>>2] = 0;
     $66 = (($64) + 4)|0;
     $67 = $66;
     HEAP32[$67>>2] = 0;
     $68 = HEAP32[$pTree>>2]|0;
     $69 = ((($68)) + 8|0);
     HEAP32[$69>>2] = 0;
     $70 = HEAP32[$p>>2]|0;
     $71 = (_rowSetListToTree($70)|0);
     $72 = HEAP32[$pTree>>2]|0;
     $73 = ((($72)) + 12|0);
     HEAP32[$73>>2] = $71;
    }
   }
   $74 = HEAP32[$3>>2]|0;
   $75 = ((($74)) + 8|0);
   HEAP32[$75>>2] = 0;
   $76 = HEAP32[$3>>2]|0;
   $77 = ((($76)) + 12|0);
   HEAP32[$77>>2] = 0;
   $78 = HEAP32[$3>>2]|0;
   $79 = ((($78)) + 26|0);
   $80 = HEAP16[$79>>1]|0;
   $81 = $80&65535;
   $82 = $81 | 1;
   $83 = $82&65535;
   HEAP16[$79>>1] = $83;
  }
  $84 = HEAP32[$4>>2]|0;
  $85 = HEAP32[$3>>2]|0;
  $86 = ((($85)) + 28|0);
  HEAP32[$86>>2] = $84;
 }
 $87 = HEAP32[$3>>2]|0;
 $88 = ((($87)) + 20|0);
 $89 = HEAP32[$88>>2]|0;
 HEAP32[$pTree>>2] = $89;
 L21: while(1) {
  $90 = HEAP32[$pTree>>2]|0;
  $91 = ($90|0)!=(0|0);
  if (!($91)) {
   label = 25;
   break;
  }
  $92 = HEAP32[$pTree>>2]|0;
  $93 = ((($92)) + 12|0);
  $94 = HEAP32[$93>>2]|0;
  HEAP32[$p>>2] = $94;
  while(1) {
   $95 = HEAP32[$p>>2]|0;
   $96 = ($95|0)!=(0|0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[$p>>2]|0;
   $98 = $97;
   $99 = $98;
   $100 = HEAP32[$99>>2]|0;
   $101 = (($98) + 4)|0;
   $102 = $101;
   $103 = HEAP32[$102>>2]|0;
   $104 = $5;
   $105 = $104;
   $106 = HEAP32[$105>>2]|0;
   $107 = (($104) + 4)|0;
   $108 = $107;
   $109 = HEAP32[$108>>2]|0;
   $110 = ($103|0)<($109|0);
   $111 = ($100>>>0)<($106>>>0);
   $112 = ($103|0)==($109|0);
   $113 = $112 & $111;
   $114 = $110 | $113;
   $115 = HEAP32[$p>>2]|0;
   if ($114) {
    $116 = ((($115)) + 8|0);
    $117 = HEAP32[$116>>2]|0;
    HEAP32[$p>>2] = $117;
    continue;
   }
   $118 = $115;
   $119 = $118;
   $120 = HEAP32[$119>>2]|0;
   $121 = (($118) + 4)|0;
   $122 = $121;
   $123 = HEAP32[$122>>2]|0;
   $124 = $5;
   $125 = $124;
   $126 = HEAP32[$125>>2]|0;
   $127 = (($124) + 4)|0;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = ($123|0)>($129|0);
   $131 = ($120>>>0)>($126>>>0);
   $132 = ($123|0)==($129|0);
   $133 = $132 & $131;
   $134 = $130 | $133;
   if (!($134)) {
    label = 23;
    break L21;
   }
   $135 = HEAP32[$p>>2]|0;
   $136 = ((($135)) + 12|0);
   $137 = HEAP32[$136>>2]|0;
   HEAP32[$p>>2] = $137;
  }
  $138 = HEAP32[$pTree>>2]|0;
  $139 = ((($138)) + 8|0);
  $140 = HEAP32[$139>>2]|0;
  HEAP32[$pTree>>2] = $140;
 }
 if ((label|0) == 23) {
  HEAP32[$2>>2] = 1;
  $141 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 else if ((label|0) == 25) {
  HEAP32[$2>>2] = 0;
  $141 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($141|0);
 }
 return (0)|0;
}
function _sqlite3PagerOkToChangeJournalMode($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pPager;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 17|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)>=(3);
 if ($6) {
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 68|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 80|0);
  $14 = $13;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)>(0);
  $21 = ($16>>>0)>(0);
  $22 = ($19|0)==(0);
  $23 = $22 & $21;
  $24 = $20 | $23;
  if ($24) {
   HEAP32[$0>>2] = 0;
   $25 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($25|0);
  }
 }
 HEAP32[$0>>2] = 1;
 $25 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _sqlite3PagerWalSupported($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMethods = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pMethods = sp;
 HEAP32[$1>>2] = $pPager;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 64|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pMethods>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 14|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = ($8<<24>>24)!=(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 4|0);
 $12 = HEAP8[$11>>0]|0;
 $13 = $12&255;
 $14 = ($13|0)!=(0);
 if ($14) {
  $23 = 1;
 } else {
  $15 = HEAP32[$pMethods>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)>=(2);
  if ($17) {
   $18 = HEAP32[$pMethods>>2]|0;
   $19 = ((($18)) + 52|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = ($20|0)!=(0|0);
   $23 = $21;
  } else {
   $23 = 0;
  }
 }
 $22 = $23&1;
 HEAP32[$0>>2] = $22;
 $24 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($24|0);
}
function _sqlite3PagerCloseWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $logexists = 0, $or$cond = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $logexists = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  HEAP32[$logexists>>2] = 0;
  $5 = HEAP32[$0>>2]|0;
  $6 = (_pagerLockDb($5,1)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$0>>2]|0;
   $12 = ((($11)) + 220|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = (_sqlite3OsAccess($10,$13,0,$logexists)|0);
   HEAP32[$rc>>2] = $14;
  }
  $15 = HEAP32[$rc>>2]|0;
  $16 = ($15|0)==(0);
  $17 = HEAP32[$logexists>>2]|0;
  $18 = ($17|0)!=(0);
  $or$cond = $16 & $18;
  if ($or$cond) {
   $19 = HEAP32[$0>>2]|0;
   $20 = (_pagerOpenWal($19)|0);
   HEAP32[$rc>>2] = $20;
  }
 }
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if (!($22)) {
  $55 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 216|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ($25|0)!=(0|0);
 if (!($26)) {
  $55 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $27 = HEAP32[$0>>2]|0;
 $28 = (_pagerExclusiveLock($27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 $30 = ($29|0)==(0);
 if (!($30)) {
  $55 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 216|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 10|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = HEAP32[$0>>2]|0;
 $39 = ((($38)) + 160|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = HEAP32[$0>>2]|0;
 $42 = ((($41)) + 208|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3WalClose($33,$37,$40,$43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 216|0);
 HEAP32[$46>>2] = 0;
 $47 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($47);
 $48 = HEAP32[$rc>>2]|0;
 $49 = ($48|0)!=(0);
 if (!($49)) {
  $55 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 4|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = ($52<<24>>24)!=(0);
 if ($53) {
  $55 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($55|0);
 }
 $54 = HEAP32[$0>>2]|0;
 (_pagerUnlockDb($54,1)|0);
 $55 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3PagerSetJournalMode($pPager,$eMode) {
 $pPager = $pPager|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $eOld = 0, $or$cond = 0, $or$cond3 = 0, $rc = 0, $state = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $eOld = sp + 16|0;
 $rc = sp + 4|0;
 $state = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 5|0);
 $4 = HEAP8[$3>>0]|0;
 HEAP8[$eOld>>0] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$1>>2]|0;
  $10 = ($9|0)!=(4);
  $11 = HEAP32[$1>>2]|0;
  $12 = ($11|0)!=(2);
  $or$cond = $10 & $12;
  if ($or$cond) {
   $13 = HEAP8[$eOld>>0]|0;
   $14 = $13&255;
   HEAP32[$1>>2] = $14;
  }
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP8[$eOld>>0]|0;
 $17 = $16&255;
 $18 = ($15|0)!=($17|0);
 do {
  if ($18) {
   $19 = HEAP32[$1>>2]|0;
   $20 = $19&255;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 5|0);
   HEAP8[$22>>0] = $20;
   $23 = HEAP32[$0>>2]|0;
   $24 = ((($23)) + 4|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = ($25<<24>>24)!=(0);
   if (!($26)) {
    $27 = HEAP8[$eOld>>0]|0;
    $28 = $27&255;
    $29 = $28 & 5;
    $30 = ($29|0)==(1);
    if ($30) {
     $31 = HEAP32[$1>>2]|0;
     $32 = $31 & 1;
     $33 = ($32|0)==(0);
     if ($33) {
      $34 = HEAP32[$0>>2]|0;
      $35 = ((($34)) + 68|0);
      $36 = HEAP32[$35>>2]|0;
      _sqlite3OsClose($36);
      $37 = HEAP32[$0>>2]|0;
      $38 = ((($37)) + 18|0);
      $39 = HEAP8[$38>>0]|0;
      $40 = $39&255;
      $41 = ($40|0)>=(2);
      if ($41) {
       $42 = HEAP32[$0>>2]|0;
       $43 = HEAP32[$42>>2]|0;
       $44 = HEAP32[$0>>2]|0;
       $45 = ((($44)) + 180|0);
       $46 = HEAP32[$45>>2]|0;
       (_sqlite3OsDelete($43,$46,0)|0);
       break;
      }
      HEAP32[$rc>>2] = 0;
      $47 = HEAP32[$0>>2]|0;
      $48 = ((($47)) + 17|0);
      $49 = HEAP8[$48>>0]|0;
      $50 = $49&255;
      HEAP32[$state>>2] = $50;
      $51 = HEAP32[$state>>2]|0;
      $52 = ($51|0)==(0);
      if ($52) {
       $53 = HEAP32[$0>>2]|0;
       $54 = (_sqlite3PagerSharedLock($53)|0);
       HEAP32[$rc>>2] = $54;
      }
      $55 = HEAP32[$0>>2]|0;
      $56 = ((($55)) + 17|0);
      $57 = HEAP8[$56>>0]|0;
      $58 = $57&255;
      $59 = ($58|0)==(1);
      if ($59) {
       $60 = HEAP32[$0>>2]|0;
       $61 = (_pagerLockDb($60,2)|0);
       HEAP32[$rc>>2] = $61;
      }
      $62 = HEAP32[$rc>>2]|0;
      $63 = ($62|0)==(0);
      if ($63) {
       $64 = HEAP32[$0>>2]|0;
       $65 = HEAP32[$64>>2]|0;
       $66 = HEAP32[$0>>2]|0;
       $67 = ((($66)) + 180|0);
       $68 = HEAP32[$67>>2]|0;
       (_sqlite3OsDelete($65,$68,0)|0);
      }
      $69 = HEAP32[$rc>>2]|0;
      $70 = ($69|0)==(0);
      $71 = HEAP32[$state>>2]|0;
      $72 = ($71|0)==(1);
      $or$cond3 = $70 & $72;
      if ($or$cond3) {
       $73 = HEAP32[$0>>2]|0;
       (_pagerUnlockDb($73,1)|0);
       break;
      }
      $74 = HEAP32[$state>>2]|0;
      $75 = ($74|0)==(0);
      if (!($75)) {
       break;
      }
      $76 = HEAP32[$0>>2]|0;
      _pager_unlock($76);
      break;
     }
    }
   }
   $77 = HEAP32[$1>>2]|0;
   $78 = ($77|0)==(2);
   if ($78) {
    $79 = HEAP32[$0>>2]|0;
    $80 = ((($79)) + 68|0);
    $81 = HEAP32[$80>>2]|0;
    _sqlite3OsClose($81);
   }
  }
 } while(0);
 $82 = HEAP32[$0>>2]|0;
 $83 = ((($82)) + 5|0);
 $84 = HEAP8[$83>>0]|0;
 $85 = $84&255;
 STACKTOP = sp;return ($85|0);
}
function _sqlite3RunVacuum($pzErrMsg,$db,$iDb) {
 $pzErrMsg = $pzErrMsg|0;
 $db = $db|0;
 $iDb = $iDb|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $isMemDb = 0, $meta = 0, $nDb = 0, $nRes = 0, $pDb = 0, $pMain = 0, $pTemp = 0, $rc = 0, $saved_flags = 0, $saved_mTrace = 0, $saved_nChange = 0, $saved_nTotalChange = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $zDbMain = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $rc = sp + 76|0;
 $pMain = sp + 72|0;
 $pTemp = sp + 68|0;
 $saved_flags = sp + 64|0;
 $saved_nChange = sp + 60|0;
 $saved_nTotalChange = sp + 56|0;
 $saved_mTrace = sp + 96|0;
 $pDb = sp + 52|0;
 $isMemDb = sp + 48|0;
 $nRes = sp + 44|0;
 $nDb = sp + 40|0;
 $zDbMain = sp + 36|0;
 $meta = sp + 32|0;
 $i = sp + 28|0;
 HEAP32[$1>>2] = $pzErrMsg;
 HEAP32[$2>>2] = $db;
 HEAP32[$3>>2] = $iDb;
 HEAP32[$rc>>2] = 0;
 HEAP32[$pDb>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 67|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if (!($7)) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  _sqlite3SetString($8,$9,40519);
  HEAP32[$0>>2] = 1;
  $240 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($240|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 156|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(1);
 if ($13) {
  $14 = HEAP32[$1>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  _sqlite3SetString($14,$15,40559);
  HEAP32[$0>>2] = 1;
  $240 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($240|0);
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 24|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$saved_flags>>2] = $18;
 $19 = HEAP32[$2>>2]|0;
 $20 = ((($19)) + 88|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$saved_nChange>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = ((($22)) + 92|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$saved_nTotalChange>>2] = $24;
 $25 = HEAP32[$2>>2]|0;
 $26 = ((($25)) + 76|0);
 $27 = HEAP8[$26>>0]|0;
 HEAP8[$saved_mTrace>>0] = $27;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 24|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = $30 | 270542848;
 HEAP32[$29>>2] = $31;
 $32 = HEAP32[$2>>2]|0;
 $33 = ((($32)) + 24|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = $34 & -655489;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$2>>2]|0;
 $37 = ((($36)) + 76|0);
 HEAP8[$37>>0] = 0;
 $38 = HEAP32[$3>>2]|0;
 $39 = HEAP32[$2>>2]|0;
 $40 = ((($39)) + 16|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + ($38<<4)|0);
 $43 = HEAP32[$42>>2]|0;
 HEAP32[$zDbMain>>2] = $43;
 $44 = HEAP32[$3>>2]|0;
 $45 = HEAP32[$2>>2]|0;
 $46 = ((($45)) + 16|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = (($47) + ($44<<4)|0);
 $49 = ((($48)) + 4|0);
 $50 = HEAP32[$49>>2]|0;
 HEAP32[$pMain>>2] = $50;
 $51 = HEAP32[$pMain>>2]|0;
 $52 = (_sqlite3BtreePager($51)|0);
 $53 = (_sqlite3PagerIsMemdb($52)|0);
 HEAP32[$isMemDb>>2] = $53;
 $54 = HEAP32[$2>>2]|0;
 $55 = ((($54)) + 20|0);
 $56 = HEAP32[$55>>2]|0;
 HEAP32[$nDb>>2] = $56;
 $57 = HEAP32[$2>>2]|0;
 $58 = HEAP32[$1>>2]|0;
 $59 = (_execSql($57,$58,40602)|0);
 HEAP32[$rc>>2] = $59;
 $60 = HEAP32[$rc>>2]|0;
 $61 = ($60|0)!=(0);
 L9: do {
  if (!($61)) {
   $62 = HEAP32[$nDb>>2]|0;
   $63 = HEAP32[$2>>2]|0;
   $64 = ((($63)) + 16|0);
   $65 = HEAP32[$64>>2]|0;
   $66 = (($65) + ($62<<4)|0);
   HEAP32[$pDb>>2] = $66;
   $67 = HEAP32[$pDb>>2]|0;
   $68 = ((($67)) + 4|0);
   $69 = HEAP32[$68>>2]|0;
   HEAP32[$pTemp>>2] = $69;
   $70 = HEAP32[$pTemp>>2]|0;
   (_sqlite3BtreeCommit($70)|0);
   $71 = HEAP32[$pMain>>2]|0;
   $72 = (_sqlite3BtreeGetOptimalReserve($71)|0);
   HEAP32[$nRes>>2] = $72;
   $73 = HEAP32[$pTemp>>2]|0;
   $74 = HEAP32[$3>>2]|0;
   $75 = HEAP32[$2>>2]|0;
   $76 = ((($75)) + 16|0);
   $77 = HEAP32[$76>>2]|0;
   $78 = (($77) + ($74<<4)|0);
   $79 = ((($78)) + 12|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = ((($80)) + 80|0);
   $82 = HEAP32[$81>>2]|0;
   (_sqlite3BtreeSetCacheSize($73,$82)|0);
   $83 = HEAP32[$pTemp>>2]|0;
   $84 = HEAP32[$pMain>>2]|0;
   $85 = (_sqlite3BtreeSetSpillSize($84,0)|0);
   (_sqlite3BtreeSetSpillSize($83,$85)|0);
   $86 = HEAP32[$pTemp>>2]|0;
   (_sqlite3BtreeSetPagerFlags($86,33)|0);
   $87 = HEAP32[$2>>2]|0;
   $88 = HEAP32[$1>>2]|0;
   $89 = (_execSql($87,$88,37527)|0);
   HEAP32[$rc>>2] = $89;
   $90 = HEAP32[$rc>>2]|0;
   $91 = ($90|0)!=(0);
   if (!($91)) {
    $92 = HEAP32[$pMain>>2]|0;
    $93 = (_sqlite3BtreeBeginTrans($92,2)|0);
    HEAP32[$rc>>2] = $93;
    $94 = HEAP32[$rc>>2]|0;
    $95 = ($94|0)!=(0);
    if (!($95)) {
     $96 = HEAP32[$pMain>>2]|0;
     $97 = (_sqlite3BtreePager($96)|0);
     $98 = (_sqlite3PagerGetJournalMode($97)|0);
     $99 = ($98|0)==(5);
     if ($99) {
      $100 = HEAP32[$2>>2]|0;
      $101 = ((($100)) + 80|0);
      HEAP32[$101>>2] = 0;
     }
     $102 = HEAP32[$pTemp>>2]|0;
     $103 = HEAP32[$pMain>>2]|0;
     $104 = (_sqlite3BtreeGetPageSize($103)|0);
     $105 = HEAP32[$nRes>>2]|0;
     $106 = (_sqlite3BtreeSetPageSize($102,$104,$105,0)|0);
     $107 = ($106|0)!=(0);
     do {
      if (!($107)) {
       $108 = HEAP32[$isMemDb>>2]|0;
       $109 = ($108|0)!=(0);
       if (!($109)) {
        $110 = HEAP32[$pTemp>>2]|0;
        $111 = HEAP32[$2>>2]|0;
        $112 = ((($111)) + 80|0);
        $113 = HEAP32[$112>>2]|0;
        $114 = HEAP32[$nRes>>2]|0;
        $115 = (_sqlite3BtreeSetPageSize($110,$113,$114,0)|0);
        $116 = ($115|0)!=(0);
        if ($116) {
         break;
        }
       }
       $117 = HEAP32[$2>>2]|0;
       $118 = ((($117)) + 69|0);
       $119 = HEAP8[$118>>0]|0;
       $120 = $119&255;
       $121 = ($120|0)!=(0);
       if (!($121)) {
        $122 = HEAP32[$pTemp>>2]|0;
        $123 = HEAP32[$2>>2]|0;
        $124 = ((($123)) + 72|0);
        $125 = HEAP8[$124>>0]|0;
        $126 = $125 << 24 >> 24;
        $127 = ($126|0)>=(0);
        if ($127) {
         $128 = HEAP32[$2>>2]|0;
         $129 = ((($128)) + 72|0);
         $130 = HEAP8[$129>>0]|0;
         $131 = $130 << 24 >> 24;
         $134 = $131;
        } else {
         $132 = HEAP32[$pMain>>2]|0;
         $133 = (_sqlite3BtreeGetAutoVacuum($132)|0);
         $134 = $133;
        }
        (_sqlite3BtreeSetAutoVacuum($122,$134)|0);
        $135 = HEAP32[$nDb>>2]|0;
        $136 = $135&255;
        $137 = HEAP32[$2>>2]|0;
        $138 = ((($137)) + 148|0);
        $139 = ((($138)) + 4|0);
        HEAP8[$139>>0] = $136;
        $140 = HEAP32[$2>>2]|0;
        $141 = HEAP32[$1>>2]|0;
        $142 = HEAP32[$zDbMain>>2]|0;
        HEAP32[$vararg_buffer>>2] = $142;
        $143 = (_execSqlF($140,$141,40623,$vararg_buffer)|0);
        HEAP32[$rc>>2] = $143;
        $144 = HEAP32[$rc>>2]|0;
        $145 = ($144|0)!=(0);
        if ($145) {
         break L9;
        }
        $146 = HEAP32[$2>>2]|0;
        $147 = HEAP32[$1>>2]|0;
        $148 = HEAP32[$zDbMain>>2]|0;
        HEAP32[$vararg_buffer1>>2] = $148;
        $149 = (_execSqlF($146,$147,40731,$vararg_buffer1)|0);
        HEAP32[$rc>>2] = $149;
        $150 = HEAP32[$rc>>2]|0;
        $151 = ($150|0)!=(0);
        if ($151) {
         break L9;
        }
        $152 = HEAP32[$2>>2]|0;
        $153 = ((($152)) + 148|0);
        $154 = ((($153)) + 4|0);
        HEAP8[$154>>0] = 0;
        $155 = HEAP32[$2>>2]|0;
        $156 = HEAP32[$1>>2]|0;
        $157 = HEAP32[$zDbMain>>2]|0;
        HEAP32[$vararg_buffer4>>2] = $157;
        $158 = (_execSqlF($155,$156,40804,$vararg_buffer4)|0);
        HEAP32[$rc>>2] = $158;
        $159 = HEAP32[$2>>2]|0;
        $160 = ((($159)) + 24|0);
        $161 = HEAP32[$160>>2]|0;
        $162 = $161 & -268435457;
        HEAP32[$160>>2] = $162;
        $163 = HEAP32[$rc>>2]|0;
        $164 = ($163|0)!=(0);
        if ($164) {
         break L9;
        }
        $165 = HEAP32[$2>>2]|0;
        $166 = HEAP32[$1>>2]|0;
        $167 = HEAP32[$zDbMain>>2]|0;
        HEAP32[$vararg_buffer7>>2] = $167;
        $168 = (_execSqlF($165,$166,40955,$vararg_buffer7)|0);
        HEAP32[$rc>>2] = $168;
        $169 = HEAP32[$rc>>2]|0;
        $170 = ($169|0)!=(0);
        if ($170) {
         break L9;
        }
        HEAP32[$i>>2] = 0;
        while(1) {
         $171 = HEAP32[$i>>2]|0;
         $172 = ($171|0)<(10);
         $173 = HEAP32[$pMain>>2]|0;
         if (!($172)) {
          break;
         }
         $174 = HEAP32[$i>>2]|0;
         $175 = (41085 + ($174)|0);
         $176 = HEAP8[$175>>0]|0;
         $177 = $176&255;
         _sqlite3BtreeGetMeta($173,$177,$meta);
         $178 = HEAP32[$pTemp>>2]|0;
         $179 = HEAP32[$i>>2]|0;
         $180 = (41085 + ($179)|0);
         $181 = HEAP8[$180>>0]|0;
         $182 = $181&255;
         $183 = HEAP32[$meta>>2]|0;
         $184 = HEAP32[$i>>2]|0;
         $185 = (($184) + 1)|0;
         $186 = (41085 + ($185)|0);
         $187 = HEAP8[$186>>0]|0;
         $188 = $187&255;
         $189 = (($183) + ($188))|0;
         $190 = (_sqlite3BtreeUpdateMeta($178,$182,$189)|0);
         HEAP32[$rc>>2] = $190;
         $191 = HEAP32[$rc>>2]|0;
         $192 = ($191|0)!=(0);
         if ($192) {
          break L9;
         }
         $193 = HEAP32[$i>>2]|0;
         $194 = (($193) + 2)|0;
         HEAP32[$i>>2] = $194;
        }
        $195 = HEAP32[$pTemp>>2]|0;
        $196 = (_sqlite3BtreeCopyFile($173,$195)|0);
        HEAP32[$rc>>2] = $196;
        $197 = HEAP32[$rc>>2]|0;
        $198 = ($197|0)!=(0);
        if ($198) {
         break L9;
        }
        $199 = HEAP32[$pTemp>>2]|0;
        $200 = (_sqlite3BtreeCommit($199)|0);
        HEAP32[$rc>>2] = $200;
        $201 = HEAP32[$rc>>2]|0;
        $202 = ($201|0)!=(0);
        if ($202) {
         break L9;
        }
        $203 = HEAP32[$pMain>>2]|0;
        $204 = HEAP32[$pTemp>>2]|0;
        $205 = (_sqlite3BtreeGetAutoVacuum($204)|0);
        (_sqlite3BtreeSetAutoVacuum($203,$205)|0);
        $206 = HEAP32[$pMain>>2]|0;
        $207 = HEAP32[$pTemp>>2]|0;
        $208 = (_sqlite3BtreeGetPageSize($207)|0);
        $209 = HEAP32[$nRes>>2]|0;
        $210 = (_sqlite3BtreeSetPageSize($206,$208,$209,1)|0);
        HEAP32[$rc>>2] = $210;
        break L9;
       }
      }
     } while(0);
     HEAP32[$rc>>2] = 7;
    }
   }
  }
 } while(0);
 $211 = HEAP32[$2>>2]|0;
 $212 = ((($211)) + 148|0);
 $213 = ((($212)) + 4|0);
 HEAP8[$213>>0] = 0;
 $214 = HEAP32[$saved_flags>>2]|0;
 $215 = HEAP32[$2>>2]|0;
 $216 = ((($215)) + 24|0);
 HEAP32[$216>>2] = $214;
 $217 = HEAP32[$saved_nChange>>2]|0;
 $218 = HEAP32[$2>>2]|0;
 $219 = ((($218)) + 88|0);
 HEAP32[$219>>2] = $217;
 $220 = HEAP32[$saved_nTotalChange>>2]|0;
 $221 = HEAP32[$2>>2]|0;
 $222 = ((($221)) + 92|0);
 HEAP32[$222>>2] = $220;
 $223 = HEAP8[$saved_mTrace>>0]|0;
 $224 = HEAP32[$2>>2]|0;
 $225 = ((($224)) + 76|0);
 HEAP8[$225>>0] = $223;
 $226 = HEAP32[$pMain>>2]|0;
 (_sqlite3BtreeSetPageSize($226,-1,-1,1)|0);
 $227 = HEAP32[$2>>2]|0;
 $228 = ((($227)) + 67|0);
 HEAP8[$228>>0] = 1;
 $229 = HEAP32[$pDb>>2]|0;
 $230 = ($229|0)!=(0|0);
 if ($230) {
  $231 = HEAP32[$pDb>>2]|0;
  $232 = ((($231)) + 4|0);
  $233 = HEAP32[$232>>2]|0;
  (_sqlite3BtreeClose($233)|0);
  $234 = HEAP32[$pDb>>2]|0;
  $235 = ((($234)) + 4|0);
  HEAP32[$235>>2] = 0;
  $236 = HEAP32[$pDb>>2]|0;
  $237 = ((($236)) + 12|0);
  HEAP32[$237>>2] = 0;
 }
 $238 = HEAP32[$2>>2]|0;
 _sqlite3ResetAllSchemasOfConnection($238);
 $239 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $239;
 $240 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($240|0);
}
function _sqlite3BtreeIncrVacuum($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nFin = 0, $nFree = 0, $nOrig = 0, $pBt = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $rc = sp + 16|0;
 $pBt = sp + 12|0;
 $nOrig = sp + 8|0;
 $nFree = sp + 4|0;
 $nFin = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($4);
 $5 = HEAP32[$pBt>>2]|0;
 $6 = ((($5)) + 17|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if (!($8)) {
  HEAP32[$rc>>2] = 101;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $9 = HEAP32[$pBt>>2]|0;
 $10 = (_btreePagecount($9)|0);
 HEAP32[$nOrig>>2] = $10;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 36|0);
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$nFree>>2] = $17;
 $18 = HEAP32[$pBt>>2]|0;
 $19 = HEAP32[$nOrig>>2]|0;
 $20 = HEAP32[$nFree>>2]|0;
 $21 = (_finalDbSize($18,$19,$20)|0);
 HEAP32[$nFin>>2] = $21;
 $22 = HEAP32[$nOrig>>2]|0;
 $23 = HEAP32[$nFin>>2]|0;
 $24 = ($22>>>0)<($23>>>0);
 if ($24) {
  $25 = (_sqlite3CorruptError(61866)|0);
  HEAP32[$rc>>2] = $25;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $26 = HEAP32[$nFree>>2]|0;
 $27 = ($26>>>0)>(0);
 if (!($27)) {
  HEAP32[$rc>>2] = 101;
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $28 = HEAP32[$pBt>>2]|0;
 $29 = (_saveAllCursors($28,0,0)|0);
 HEAP32[$rc>>2] = $29;
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(0);
 if ($31) {
  $32 = HEAP32[$pBt>>2]|0;
  _invalidateAllOverflowCache($32);
  $33 = HEAP32[$pBt>>2]|0;
  $34 = HEAP32[$nFin>>2]|0;
  $35 = HEAP32[$nOrig>>2]|0;
  $36 = (_incrVacuumStep($33,$34,$35,0)|0);
  HEAP32[$rc>>2] = $36;
 }
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if (!($38)) {
  $54 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($54|0);
 }
 $39 = HEAP32[$pBt>>2]|0;
 $40 = ((($39)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ((($41)) + 72|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (_sqlite3PagerWrite($43)|0);
 HEAP32[$rc>>2] = $44;
 $45 = HEAP32[$pBt>>2]|0;
 $46 = ((($45)) + 12|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = ((($47)) + 56|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ((($49)) + 28|0);
 $51 = HEAP32[$pBt>>2]|0;
 $52 = ((($51)) + 44|0);
 $53 = HEAP32[$52>>2]|0;
 _sqlite3Put4byte($50,$53);
 $54 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($54|0);
}
function _sqlite3BtreeLockTable($p,$iTab,$isWriteLock) {
 $p = $p|0;
 $iTab = $iTab|0;
 $isWriteLock = $isWriteLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $lockType = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp + 13|0;
 $rc = sp;
 $lockType = sp + 12|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $iTab;
 HEAP8[$2>>0] = $isWriteLock;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 9|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 if (!($6)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $7 = HEAP8[$2>>0]|0;
 $8 = $7&255;
 $9 = (1 + ($8))|0;
 $10 = $9&255;
 HEAP8[$lockType>>0] = $10;
 $11 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($11);
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP8[$lockType>>0]|0;
 $15 = (_querySharedCacheTableLock($12,$13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if (!($17)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP8[$lockType>>0]|0;
 $21 = (_setSharedCacheTableLock($18,$19,$20)|0);
 HEAP32[$rc>>2] = $21;
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _sqlite3VtabBegin($db,$pVTab) {
 $db = $db|0;
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $i = 0, $iSvpt = 0;
 var $pModule = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $pModule = sp + 8|0;
 $i = sp + 4|0;
 $iSvpt = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pVTab;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 316|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)>(0);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 340|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(0|0);
  if ($10) {
   HEAP32[$0>>2] = 6;
   $77 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($77|0);
  }
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  HEAP32[$0>>2] = 0;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$pModule>>2] = $16;
 $17 = HEAP32[$pModule>>2]|0;
 $18 = ((($17)) + 56|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 316|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = ($21|0)<($24|0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$i>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 340|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + ($26<<2)|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$2>>2]|0;
   $33 = ($31|0)==($32|0);
   if ($33) {
    label = 10;
    break;
   }
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + 1)|0;
   HEAP32[$i>>2] = $35;
  }
  if ((label|0) == 10) {
   HEAP32[$0>>2] = 0;
   $77 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($77|0);
  }
  $36 = HEAP32[$1>>2]|0;
  $37 = (_growVTrans($36)|0);
  HEAP32[$rc>>2] = $37;
  $38 = HEAP32[$rc>>2]|0;
  $39 = ($38|0)==(0);
  if ($39) {
   $40 = HEAP32[$pModule>>2]|0;
   $41 = ((($40)) + 56|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = HEAP32[$2>>2]|0;
   $44 = ((($43)) + 8|0);
   $45 = HEAP32[$44>>2]|0;
   $46 = (FUNCTION_TABLE_ii[$42 & 255]($45)|0);
   HEAP32[$rc>>2] = $46;
   $47 = HEAP32[$rc>>2]|0;
   $48 = ($47|0)==(0);
   if ($48) {
    $49 = HEAP32[$1>>2]|0;
    $50 = ((($49)) + 436|0);
    $51 = HEAP32[$50>>2]|0;
    $52 = HEAP32[$1>>2]|0;
    $53 = ((($52)) + 432|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = (($51) + ($54))|0;
    HEAP32[$iSvpt>>2] = $55;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$2>>2]|0;
    _addToVTrans($56,$57);
    $58 = HEAP32[$iSvpt>>2]|0;
    $59 = ($58|0)!=(0);
    if ($59) {
     $60 = HEAP32[$pModule>>2]|0;
     $61 = ((($60)) + 80|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = ($62|0)!=(0|0);
     if ($63) {
      $64 = HEAP32[$iSvpt>>2]|0;
      $65 = HEAP32[$2>>2]|0;
      $66 = ((($65)) + 20|0);
      HEAP32[$66>>2] = $64;
      $67 = HEAP32[$pModule>>2]|0;
      $68 = ((($67)) + 80|0);
      $69 = HEAP32[$68>>2]|0;
      $70 = HEAP32[$2>>2]|0;
      $71 = ((($70)) + 8|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = HEAP32[$iSvpt>>2]|0;
      $74 = (($73) - 1)|0;
      $75 = (FUNCTION_TABLE_iii[$69 & 255]($72,$74)|0);
      HEAP32[$rc>>2] = $75;
     }
    }
   }
  }
 }
 $76 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $76;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _sqlite3VtabCallCreate($db,$iDb,$zTab,$pzErr) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTab = $zTab|0;
 $pzErr = $pzErr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pMod = 0, $pTab = 0, $rc = 0, $vararg_buffer = 0, $zMod = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $rc = sp + 16|0;
 $pTab = sp + 12|0;
 $pMod = sp + 8|0;
 $zMod = sp + 4|0;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $iDb;
 HEAP32[$2>>2] = $zTab;
 HEAP32[$3>>2] = $pzErr;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3FindTable($4,$5,$11)|0);
 HEAP32[$pTab>>2] = $12;
 $13 = HEAP32[$pTab>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$zMod>>2] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 320|0);
 $19 = HEAP32[$zMod>>2]|0;
 $20 = (_sqlite3HashFind($18,$19)|0);
 HEAP32[$pMod>>2] = $20;
 $21 = HEAP32[$pMod>>2]|0;
 $22 = ($21|0)==(0|0);
 if ($22) {
  label = 4;
 } else {
  $23 = HEAP32[$pMod>>2]|0;
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 4|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = ($26|0)==(0|0);
  if ($27) {
   label = 4;
  } else {
   $28 = HEAP32[$pMod>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ((($29)) + 20|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)==(0|0);
   if ($32) {
    label = 4;
   } else {
    $37 = HEAP32[$0>>2]|0;
    $38 = HEAP32[$pTab>>2]|0;
    $39 = HEAP32[$pMod>>2]|0;
    $40 = HEAP32[$pMod>>2]|0;
    $41 = HEAP32[$40>>2]|0;
    $42 = ((($41)) + 4|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = HEAP32[$3>>2]|0;
    $45 = (_vtabCallConstructor($37,$38,$39,$43,$44)|0);
    HEAP32[$rc>>2] = $45;
   }
  }
 }
 if ((label|0) == 4) {
  $33 = HEAP32[$0>>2]|0;
  $34 = HEAP32[$zMod>>2]|0;
  HEAP32[$vararg_buffer>>2] = $34;
  $35 = (_sqlite3MPrintf($33,30631,$vararg_buffer)|0);
  $36 = HEAP32[$3>>2]|0;
  HEAP32[$36>>2] = $35;
  HEAP32[$rc>>2] = 1;
 }
 $46 = HEAP32[$rc>>2]|0;
 $47 = ($46|0)==(0);
 if (!($47)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $48 = HEAP32[$0>>2]|0;
 $49 = HEAP32[$pTab>>2]|0;
 $50 = (_sqlite3GetVTable($48,$49)|0);
 $51 = ($50|0)!=(0|0);
 if (!($51)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $52 = HEAP32[$0>>2]|0;
 $53 = (_growVTrans($52)|0);
 HEAP32[$rc>>2] = $53;
 $54 = HEAP32[$rc>>2]|0;
 $55 = ($54|0)==(0);
 if (!($55)) {
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $56 = HEAP32[$0>>2]|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = HEAP32[$pTab>>2]|0;
 $59 = (_sqlite3GetVTable($57,$58)|0);
 _addToVTrans($56,$59);
 $60 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _sqlite3VtabCallDestroy($db,$iDb,$zTab) {
 $db = $db|0;
 $iDb = $iDb|0;
 $zTab = $zTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pTab = 0, $rc = 0, $xDestroy = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $rc = sp + 12|0;
 $pTab = sp + 8|0;
 $p = sp + 4|0;
 $xDestroy = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $iDb;
 HEAP32[$3>>2] = $zTab;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6<<4)|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3FindTable($4,$5,$11)|0);
 HEAP32[$pTab>>2] = $12;
 $13 = HEAP32[$pTab>>2]|0;
 $14 = ($13|0)!=(0|0);
 do {
  if ($14) {
   $15 = HEAP32[$pTab>>2]|0;
   $16 = ((($15)) + 56|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = ($17|0)!=(0|0);
   if ($18) {
    $19 = HEAP32[$pTab>>2]|0;
    $20 = ((($19)) + 56|0);
    $21 = HEAP32[$20>>2]|0;
    HEAP32[$p>>2] = $21;
    while(1) {
     $22 = HEAP32[$p>>2]|0;
     $23 = ($22|0)!=(0|0);
     if (!($23)) {
      label = 8;
      break;
     }
     $24 = HEAP32[$p>>2]|0;
     $25 = ((($24)) + 8|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ((($26)) + 4|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = ($28|0)>(0);
     if ($29) {
      label = 6;
      break;
     }
     $30 = HEAP32[$p>>2]|0;
     $31 = ((($30)) + 24|0);
     $32 = HEAP32[$31>>2]|0;
     HEAP32[$p>>2] = $32;
    }
    if ((label|0) == 6) {
     HEAP32[$0>>2] = 6;
     $55 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($55|0);
    }
    else if ((label|0) == 8) {
     $33 = HEAP32[$1>>2]|0;
     $34 = HEAP32[$pTab>>2]|0;
     $35 = (_vtabDisconnectAll($33,$34)|0);
     HEAP32[$p>>2] = $35;
     $36 = HEAP32[$p>>2]|0;
     $37 = ((($36)) + 4|0);
     $38 = HEAP32[$37>>2]|0;
     $39 = HEAP32[$38>>2]|0;
     $40 = ((($39)) + 20|0);
     $41 = HEAP32[$40>>2]|0;
     HEAP32[$xDestroy>>2] = $41;
     $42 = HEAP32[$xDestroy>>2]|0;
     $43 = HEAP32[$p>>2]|0;
     $44 = ((($43)) + 8|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (FUNCTION_TABLE_ii[$42 & 255]($45)|0);
     HEAP32[$rc>>2] = $46;
     $47 = HEAP32[$rc>>2]|0;
     $48 = ($47|0)==(0);
     if (!($48)) {
      break;
     }
     $49 = HEAP32[$p>>2]|0;
     $50 = ((($49)) + 8|0);
     HEAP32[$50>>2] = 0;
     $51 = HEAP32[$pTab>>2]|0;
     $52 = ((($51)) + 56|0);
     HEAP32[$52>>2] = 0;
     $53 = HEAP32[$p>>2]|0;
     _sqlite3VtabUnlock($53);
     break;
    }
   }
  }
 } while(0);
 $54 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($55|0);
}
function _sqlite3BtreeMaxPageCount($p,$mxPage) {
 $p = $p|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($2);
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (_sqlite3PagerMaxPageCount($6,$7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3SystemError($db,$rc) {
 $db = $db|0;
 $rc = $rc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $rc;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)==(3082);
 if ($3) {
  STACKTOP = sp;return;
 }
 $4 = HEAP32[$1>>2]|0;
 $5 = $4 & 255;
 HEAP32[$1>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)==(14);
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(10);
 $or$cond = $7 | $9;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 $12 = (_sqlite3OsGetLastError($11)|0);
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 60|0);
 HEAP32[$14>>2] = $12;
 STACKTOP = sp;return;
}
function _sqlite3OsGetLastError($pVfs) {
 $pVfs = $pVfs|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pVfs;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 68|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $10 = 0;
  STACKTOP = sp;return ($10|0);
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 68|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = (FUNCTION_TABLE_iiii[$7 & 255]($8,0,0)|0);
 $10 = $9;
 STACKTOP = sp;return ($10|0);
}
function _sqlite3PagerMaxPageCount($pPager,$mxPage) {
 $pPager = $pPager|0;
 $mxPage = $mxPage|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $mxPage;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)>(0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 164|0);
  HEAP32[$6>>2] = $4;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 164|0);
 $9 = HEAP32[$8>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _vtabDisconnectAll($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db2 = 0, $pNext = 0, $pRet = 0, $pVTable = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pRet = sp + 12|0;
 $pVTable = sp + 8|0;
 $db2 = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$pRet>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pVTable>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 56|0);
 HEAP32[$6>>2] = 0;
 while(1) {
  $7 = HEAP32[$pVTable>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$pVTable>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$db2>>2] = $10;
  $11 = HEAP32[$pVTable>>2]|0;
  $12 = ((($11)) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  HEAP32[$pNext>>2] = $13;
  $14 = HEAP32[$db2>>2]|0;
  $15 = HEAP32[$0>>2]|0;
  $16 = ($14|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$pVTable>>2]|0;
   HEAP32[$pRet>>2] = $17;
   $18 = HEAP32[$pRet>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ((($19)) + 56|0);
   HEAP32[$20>>2] = $18;
   $21 = HEAP32[$pRet>>2]|0;
   $22 = ((($21)) + 24|0);
   HEAP32[$22>>2] = 0;
  } else {
   $23 = HEAP32[$db2>>2]|0;
   $24 = ((($23)) + 344|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = HEAP32[$pVTable>>2]|0;
   $27 = ((($26)) + 24|0);
   HEAP32[$27>>2] = $25;
   $28 = HEAP32[$pVTable>>2]|0;
   $29 = HEAP32[$db2>>2]|0;
   $30 = ((($29)) + 344|0);
   HEAP32[$30>>2] = $28;
  }
  $31 = HEAP32[$pNext>>2]|0;
  HEAP32[$pVTable>>2] = $31;
 }
 $32 = HEAP32[$pRet>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _growVTrans($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ARRAY_INCR = 0, $aVTrans = 0, $nBytes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $ARRAY_INCR = sp + 8|0;
 $aVTrans = sp + 4|0;
 $nBytes = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$ARRAY_INCR>>2] = 5;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 316|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4|0) % 5)&-1;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 316|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = (($9) + 5)|0;
   $11 = $10<<2;
   HEAP32[$nBytes>>2] = $11;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$1>>2]|0;
   $14 = ((($13)) + 340|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$nBytes>>2]|0;
   $17 = ($16|0)<(0);
   $18 = $17 << 31 >> 31;
   $19 = (_sqlite3DbRealloc($12,$15,$16,$18)|0);
   HEAP32[$aVTrans>>2] = $19;
   $20 = HEAP32[$aVTrans>>2]|0;
   $21 = ($20|0)!=(0|0);
   if ($21) {
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 316|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = HEAP32[$aVTrans>>2]|0;
    $26 = (($25) + ($24<<2)|0);
    ;HEAP32[$26>>2]=0|0;HEAP32[$26+4>>2]=0|0;HEAP32[$26+8>>2]=0|0;HEAP32[$26+12>>2]=0|0;HEAP32[$26+16>>2]=0|0;
    $27 = HEAP32[$aVTrans>>2]|0;
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 340|0);
    HEAP32[$29>>2] = $27;
    break;
   }
   HEAP32[$0>>2] = 7;
   $30 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($30|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $30 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _addToVTrans($db,$pVTab) {
 $db = $db|0;
 $pVTab = $pVTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pVTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 316|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 1)|0;
 HEAP32[$4>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 340|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($5<<2)|0);
 HEAP32[$10>>2] = $2;
 $11 = HEAP32[$1>>2]|0;
 _sqlite3VtabLock($11);
 STACKTOP = sp;return;
}
function _setSharedCacheTableLock($p,$iTable,$eLock) {
 $p = $p|0;
 $iTable = $iTable|0;
 $eLock = $eLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pIter = 0, $pLock = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 24|0;
 $pBt = sp + 8|0;
 $pLock = sp + 4|0;
 $pIter = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP8[$3>>0] = $eLock;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 HEAP32[$pLock>>2] = 0;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 72|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pIter>>2] = $9;
 while(1) {
  $10 = HEAP32[$pIter>>2]|0;
  $11 = ($10|0)!=(0|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$pIter>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$2>>2]|0;
  $16 = ($14|0)==($15|0);
  if ($16) {
   $17 = HEAP32[$pIter>>2]|0;
   $18 = HEAP32[$17>>2]|0;
   $19 = HEAP32[$1>>2]|0;
   $20 = ($18|0)==($19|0);
   if ($20) {
    label = 5;
    break;
   }
  }
  $22 = HEAP32[$pIter>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$pIter>>2] = $24;
 }
 if ((label|0) == 5) {
  $21 = HEAP32[$pIter>>2]|0;
  HEAP32[$pLock>>2] = $21;
 }
 $25 = HEAP32[$pLock>>2]|0;
 $26 = ($25|0)!=(0|0);
 do {
  if (!($26)) {
   $27 = (_sqlite3MallocZero(16,0)|0);
   HEAP32[$pLock>>2] = $27;
   $28 = HEAP32[$pLock>>2]|0;
   $29 = ($28|0)!=(0|0);
   if ($29) {
    $30 = HEAP32[$2>>2]|0;
    $31 = HEAP32[$pLock>>2]|0;
    $32 = ((($31)) + 4|0);
    HEAP32[$32>>2] = $30;
    $33 = HEAP32[$1>>2]|0;
    $34 = HEAP32[$pLock>>2]|0;
    HEAP32[$34>>2] = $33;
    $35 = HEAP32[$pBt>>2]|0;
    $36 = ((($35)) + 72|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = HEAP32[$pLock>>2]|0;
    $39 = ((($38)) + 12|0);
    HEAP32[$39>>2] = $37;
    $40 = HEAP32[$pLock>>2]|0;
    $41 = HEAP32[$pBt>>2]|0;
    $42 = ((($41)) + 72|0);
    HEAP32[$42>>2] = $40;
    break;
   }
   HEAP32[$0>>2] = 7;
   $53 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($53|0);
  }
 } while(0);
 $43 = HEAP8[$3>>0]|0;
 $44 = $43&255;
 $45 = HEAP32[$pLock>>2]|0;
 $46 = ((($45)) + 8|0);
 $47 = HEAP8[$46>>0]|0;
 $48 = $47&255;
 $49 = ($44|0)>($48|0);
 if ($49) {
  $50 = HEAP8[$3>>0]|0;
  $51 = HEAP32[$pLock>>2]|0;
  $52 = ((($51)) + 8|0);
  HEAP8[$52>>0] = $50;
 }
 HEAP32[$0>>2] = 0;
 $53 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($53|0);
}
function _finalDbSize($pBt,$nOrig,$nFree) {
 $pBt = $pBt|0;
 $nOrig = $nOrig|0;
 $nFree = $nFree|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nEntry = 0, $nFin = 0, $nPtrmap = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $nEntry = sp + 8|0;
 $nPtrmap = sp + 4|0;
 $nFin = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $nOrig;
 HEAP32[$2>>2] = $nFree;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5>>>0) / 5)&-1;
 HEAP32[$nEntry>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (($7) - ($8))|0;
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (_ptrmapPageno($10,$11)|0);
 $13 = (($9) + ($12))|0;
 $14 = HEAP32[$nEntry>>2]|0;
 $15 = (($13) + ($14))|0;
 $16 = HEAP32[$nEntry>>2]|0;
 $17 = (($15>>>0) / ($16>>>0))&-1;
 HEAP32[$nPtrmap>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (($18) - ($19))|0;
 $21 = HEAP32[$nPtrmap>>2]|0;
 $22 = (($20) - ($21))|0;
 HEAP32[$nFin>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[1536>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 32|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($24>>>0) / ($27>>>0))&-1;
 $29 = (($28) + 1)|0;
 $30 = ($23>>>0)>($29>>>0);
 if ($30) {
  $31 = HEAP32[$nFin>>2]|0;
  $32 = HEAP32[1536>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($32>>>0) / ($35>>>0))&-1;
  $37 = (($36) + 1)|0;
  $38 = ($31>>>0)<($37>>>0);
  if ($38) {
   $39 = HEAP32[$nFin>>2]|0;
   $40 = (($39) + -1)|0;
   HEAP32[$nFin>>2] = $40;
  }
 }
 while(1) {
  $41 = HEAP32[$0>>2]|0;
  $42 = HEAP32[$nFin>>2]|0;
  $43 = (_ptrmapPageno($41,$42)|0);
  $44 = HEAP32[$nFin>>2]|0;
  $45 = ($43|0)==($44|0);
  if ($45) {
   $56 = 1;
  } else {
   $46 = HEAP32[$nFin>>2]|0;
   $47 = HEAP32[1536>>2]|0;
   $48 = HEAP32[$0>>2]|0;
   $49 = ((($48)) + 32|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = (($47>>>0) / ($50>>>0))&-1;
   $52 = (($51) + 1)|0;
   $53 = ($46|0)==($52|0);
   $56 = $53;
  }
  $54 = HEAP32[$nFin>>2]|0;
  if (!($56)) {
   break;
  }
  $55 = (($54) + -1)|0;
  HEAP32[$nFin>>2] = $55;
 }
 STACKTOP = sp;return ($54|0);
}
function _invalidateAllOverflowCache($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$p>>2] = $3;
 while(1) {
  $4 = HEAP32[$p>>2]|0;
  $5 = ($4|0)!=(0|0);
  if (!($5)) {
   break;
  }
  $6 = HEAP32[$p>>2]|0;
  $7 = ((($6)) + 64|0);
  $8 = HEAP8[$7>>0]|0;
  $9 = $8&255;
  $10 = $9 & -5;
  $11 = $10&255;
  HEAP8[$7>>0] = $11;
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$p>>2] = $14;
 }
 STACKTOP = sp;return;
}
function _incrVacuumStep($pBt,$nFin,$iLastPg,$bCommit) {
 $pBt = $pBt|0;
 $nFin = $nFin|0;
 $iLastPg = $iLastPg|0;
 $bCommit = $bCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0;
 var $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0;
 var $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0;
 var $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0;
 var $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0;
 var $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eMode = 0, $eType = 0, $iFreePg = 0, $iFreePg1 = 0, $iNear = 0, $iPtrPage = 0, $nFreeList = 0, $pFreePg = 0, $pFreePg2 = 0, $pLastPg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $nFreeList = sp + 32|0;
 $rc = sp + 28|0;
 $eType = sp + 57|0;
 $iPtrPage = sp + 24|0;
 $iFreePg = sp + 20|0;
 $pFreePg = sp + 16|0;
 $iFreePg1 = sp + 12|0;
 $pLastPg = sp + 8|0;
 $eMode = sp + 56|0;
 $iNear = sp + 4|0;
 $pFreePg2 = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $nFin;
 HEAP32[$3>>2] = $iLastPg;
 HEAP32[$4>>2] = $bCommit;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$3>>2]|0;
 $7 = (_ptrmapPageno($5,$6)|0);
 $8 = HEAP32[$3>>2]|0;
 $9 = ($7|0)==($8|0);
 do {
  if (!($9)) {
   $10 = HEAP32[$3>>2]|0;
   $11 = HEAP32[1536>>2]|0;
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 32|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = (($11>>>0) / ($14>>>0))&-1;
   $16 = (($15) + 1)|0;
   $17 = ($10|0)!=($16|0);
   if ($17) {
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 12|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = ((($20)) + 56|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ((($22)) + 36|0);
    $24 = (_sqlite3Get4byte($23)|0);
    HEAP32[$nFreeList>>2] = $24;
    $25 = HEAP32[$nFreeList>>2]|0;
    $26 = ($25|0)==(0);
    if ($26) {
     HEAP32[$0>>2] = 101;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $27 = HEAP32[$1>>2]|0;
    $28 = HEAP32[$3>>2]|0;
    $29 = (_ptrmapGet($27,$28,$eType,$iPtrPage)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$rc>>2]|0;
    $31 = ($30|0)!=(0);
    if ($31) {
     $32 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $32;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $33 = HEAP8[$eType>>0]|0;
    $34 = $33&255;
    $35 = ($34|0)==(1);
    if ($35) {
     $36 = (_sqlite3CorruptError(61751)|0);
     HEAP32[$0>>2] = $36;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $37 = HEAP8[$eType>>0]|0;
    $38 = $37&255;
    $39 = ($38|0)==(2);
    if ($39) {
     $40 = HEAP32[$4>>2]|0;
     $41 = ($40|0)==(0);
     if (!($41)) {
      break;
     }
     $42 = HEAP32[$1>>2]|0;
     $43 = HEAP32[$3>>2]|0;
     $44 = (_allocateBtreePage($42,$pFreePg,$iFreePg,$43,1)|0);
     HEAP32[$rc>>2] = $44;
     $45 = HEAP32[$rc>>2]|0;
     $46 = ($45|0)!=(0);
     if (!($46)) {
      $48 = HEAP32[$pFreePg>>2]|0;
      _releasePage($48);
      break;
     }
     $47 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $47;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    HEAP8[$eMode>>0] = 0;
    HEAP32[$iNear>>2] = 0;
    $49 = HEAP32[$1>>2]|0;
    $50 = HEAP32[$3>>2]|0;
    $51 = (_btreeGetPage($49,$50,$pLastPg,0)|0);
    HEAP32[$rc>>2] = $51;
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)!=(0);
    if ($53) {
     $54 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $54;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $55 = HEAP32[$4>>2]|0;
    $56 = ($55|0)==(0);
    if ($56) {
     HEAP8[$eMode>>0] = 2;
     $57 = HEAP32[$2>>2]|0;
     HEAP32[$iNear>>2] = $57;
    }
    while(1) {
     $58 = HEAP32[$1>>2]|0;
     $59 = HEAP32[$iNear>>2]|0;
     $60 = HEAP8[$eMode>>0]|0;
     $61 = (_allocateBtreePage($58,$pFreePg2,$iFreePg1,$59,$60)|0);
     HEAP32[$rc>>2] = $61;
     $62 = HEAP32[$rc>>2]|0;
     $63 = ($62|0)!=(0);
     if ($63) {
      label = 19;
      break;
     }
     $66 = HEAP32[$pFreePg2>>2]|0;
     _releasePage($66);
     $67 = HEAP32[$4>>2]|0;
     $68 = ($67|0)!=(0);
     if (!($68)) {
      break;
     }
     $69 = HEAP32[$iFreePg1>>2]|0;
     $70 = HEAP32[$2>>2]|0;
     $71 = ($69>>>0)>($70>>>0);
     if (!($71)) {
      break;
     }
    }
    if ((label|0) == 19) {
     $64 = HEAP32[$pLastPg>>2]|0;
     _releasePage($64);
     $65 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $65;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$pLastPg>>2]|0;
    $74 = HEAP8[$eType>>0]|0;
    $75 = HEAP32[$iPtrPage>>2]|0;
    $76 = HEAP32[$iFreePg1>>2]|0;
    $77 = HEAP32[$4>>2]|0;
    $78 = (_relocatePage($72,$73,$74,$75,$76,$77)|0);
    HEAP32[$rc>>2] = $78;
    $79 = HEAP32[$pLastPg>>2]|0;
    _releasePage($79);
    $80 = HEAP32[$rc>>2]|0;
    $81 = ($80|0)!=(0);
    if ($81) {
     $82 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $82;
     $105 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($105|0);
    }
   }
  }
 } while(0);
 $83 = HEAP32[$4>>2]|0;
 $84 = ($83|0)==(0);
 if ($84) {
  while(1) {
   $85 = HEAP32[$3>>2]|0;
   $86 = (($85) + -1)|0;
   HEAP32[$3>>2] = $86;
   $87 = HEAP32[$3>>2]|0;
   $88 = HEAP32[1536>>2]|0;
   $89 = HEAP32[$1>>2]|0;
   $90 = ((($89)) + 32|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = (($88>>>0) / ($91>>>0))&-1;
   $93 = (($92) + 1)|0;
   $94 = ($87|0)==($93|0);
   if ($94) {
    continue;
   }
   $95 = HEAP32[$1>>2]|0;
   $96 = HEAP32[$3>>2]|0;
   $97 = (_ptrmapPageno($95,$96)|0);
   $98 = HEAP32[$3>>2]|0;
   $99 = ($97|0)==($98|0);
   if (!($99)) {
    break;
   }
  }
  $100 = HEAP32[$1>>2]|0;
  $101 = ((($100)) + 19|0);
  HEAP8[$101>>0] = 1;
  $102 = HEAP32[$3>>2]|0;
  $103 = HEAP32[$1>>2]|0;
  $104 = ((($103)) + 44|0);
  HEAP32[$104>>2] = $102;
 }
 HEAP32[$0>>2] = 0;
 $105 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($105|0);
}
function _allocateBtreePage($pBt,$ppPage,$pPgno,$nearby,$eMode) {
 $pBt = $pBt|0;
 $ppPage = $ppPage|0;
 $pPgno = $pPgno|0;
 $nearby = $nearby|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0;
 var $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0;
 var $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0;
 var $bNoContent = 0, $closest = 0, $d2 = 0, $dist = 0, $eType = 0, $i = 0, $iNewTrunk = 0, $iPage = 0, $iTrunk = 0, $k = 0, $mxPage = 0, $n = 0, $nSearch = 0, $noContent = 0, $or$cond = 0, $pNewTrunk = 0, $pPage1 = 0, $pPg = 0, $pPrevTrunk = 0, $pTrunk = 0;
 var $rc = 0, $searchList = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 96|0;
 $1 = sp + 92|0;
 $2 = sp + 88|0;
 $3 = sp + 84|0;
 $4 = sp + 80|0;
 $5 = sp + 102|0;
 $pPage1 = sp + 76|0;
 $rc = sp + 72|0;
 $n = sp + 68|0;
 $k = sp + 64|0;
 $pTrunk = sp + 60|0;
 $pPrevTrunk = sp + 56|0;
 $mxPage = sp + 52|0;
 $iTrunk = sp + 48|0;
 $searchList = sp + 101|0;
 $nSearch = sp + 44|0;
 $eType = sp + 100|0;
 $pNewTrunk = sp + 40|0;
 $iNewTrunk = sp + 36|0;
 $closest = sp + 32|0;
 $iPage = sp + 28|0;
 $aData = sp + 24|0;
 $i = sp + 20|0;
 $dist = sp + 16|0;
 $d2 = sp + 12|0;
 $noContent = sp + 8|0;
 $bNoContent = sp + 4|0;
 $pPg = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $ppPage;
 HEAP32[$3>>2] = $pPgno;
 HEAP32[$4>>2] = $nearby;
 HEAP8[$5>>0] = $eMode;
 HEAP32[$pTrunk>>2] = 0;
 HEAP32[$pPrevTrunk>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pPage1>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = (_btreePagecount($9)|0);
 HEAP32[$mxPage>>2] = $10;
 $11 = HEAP32[$pPage1>>2]|0;
 $12 = ((($11)) + 56|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 36|0);
 $15 = (_sqlite3Get4byte($14)|0);
 HEAP32[$n>>2] = $15;
 $16 = HEAP32[$n>>2]|0;
 $17 = HEAP32[$mxPage>>2]|0;
 $18 = ($16>>>0)>=($17>>>0);
 if ($18) {
  $19 = (_sqlite3CorruptError(63793)|0);
  HEAP32[$0>>2] = $19;
  $458 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($458|0);
 }
 $20 = HEAP32[$n>>2]|0;
 $21 = ($20>>>0)>(0);
 L5: do {
  if ($21) {
   HEAP8[$searchList>>0] = 0;
   HEAP32[$nSearch>>2] = 0;
   $22 = HEAP8[$5>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)==(1);
   do {
    if ($24) {
     $25 = HEAP32[$4>>2]|0;
     $26 = HEAP32[$mxPage>>2]|0;
     $27 = ($25>>>0)<=($26>>>0);
     if ($27) {
      $28 = HEAP32[$1>>2]|0;
      $29 = HEAP32[$4>>2]|0;
      $30 = (_ptrmapGet($28,$29,$eType,0)|0);
      HEAP32[$rc>>2] = $30;
      $31 = HEAP32[$rc>>2]|0;
      $32 = ($31|0)!=(0);
      if ($32) {
       $33 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $33;
       $458 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($458|0);
      } else {
       $34 = HEAP8[$eType>>0]|0;
       $35 = $34&255;
       $36 = ($35|0)==(2);
       if (!($36)) {
        break;
       }
       HEAP8[$searchList>>0] = 1;
       break;
      }
     }
    } else {
     $37 = HEAP8[$5>>0]|0;
     $38 = $37&255;
     $39 = ($38|0)==(2);
     if ($39) {
      HEAP8[$searchList>>0] = 1;
     }
    }
   } while(0);
   $40 = HEAP32[$pPage1>>2]|0;
   $41 = ((($40)) + 72|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = (_sqlite3PagerWrite($42)|0);
   HEAP32[$rc>>2] = $43;
   $44 = HEAP32[$rc>>2]|0;
   $45 = ($44|0)!=(0);
   if ($45) {
    $46 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $46;
    $458 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($458|0);
   }
   $47 = HEAP32[$pPage1>>2]|0;
   $48 = ((($47)) + 56|0);
   $49 = HEAP32[$48>>2]|0;
   $50 = ((($49)) + 36|0);
   $51 = HEAP32[$n>>2]|0;
   $52 = (($51) - 1)|0;
   _sqlite3Put4byte($50,$52);
   L22: while(1) {
    $53 = HEAP32[$pTrunk>>2]|0;
    HEAP32[$pPrevTrunk>>2] = $53;
    $54 = HEAP32[$pPrevTrunk>>2]|0;
    $55 = ($54|0)!=(0|0);
    if ($55) {
     $56 = HEAP32[$pPrevTrunk>>2]|0;
     $57 = ((($56)) + 56|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = (_sqlite3Get4byte($58)|0);
     HEAP32[$iTrunk>>2] = $59;
    } else {
     $60 = HEAP32[$pPage1>>2]|0;
     $61 = ((($60)) + 56|0);
     $62 = HEAP32[$61>>2]|0;
     $63 = ((($62)) + 32|0);
     $64 = (_sqlite3Get4byte($63)|0);
     HEAP32[$iTrunk>>2] = $64;
    }
    $65 = HEAP32[$iTrunk>>2]|0;
    $66 = HEAP32[$mxPage>>2]|0;
    $67 = ($65>>>0)>($66>>>0);
    if ($67) {
     label = 20;
    } else {
     $68 = HEAP32[$nSearch>>2]|0;
     $69 = (($68) + 1)|0;
     HEAP32[$nSearch>>2] = $69;
     $70 = HEAP32[$n>>2]|0;
     $71 = ($68>>>0)>($70>>>0);
     if ($71) {
      label = 20;
     } else {
      $73 = HEAP32[$1>>2]|0;
      $74 = HEAP32[$iTrunk>>2]|0;
      $75 = (_btreeGetUnusedPage($73,$74,$pTrunk,0)|0);
      HEAP32[$rc>>2] = $75;
     }
    }
    if ((label|0) == 20) {
     label = 0;
     $72 = (_sqlite3CorruptError(63849)|0);
     HEAP32[$rc>>2] = $72;
    }
    $76 = HEAP32[$rc>>2]|0;
    $77 = ($76|0)!=(0);
    if ($77) {
     label = 23;
     break;
    }
    $78 = HEAP32[$pTrunk>>2]|0;
    $79 = ((($78)) + 56|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = ((($80)) + 4|0);
    $82 = (_sqlite3Get4byte($81)|0);
    HEAP32[$k>>2] = $82;
    $83 = HEAP32[$k>>2]|0;
    $84 = ($83|0)!=(0);
    $85 = HEAP8[$searchList>>0]|0;
    $86 = ($85<<24>>24)!=(0);
    $or$cond = $84 | $86;
    L35: do {
     if ($or$cond) {
      $104 = HEAP32[$k>>2]|0;
      $105 = HEAP32[$1>>2]|0;
      $106 = ((($105)) + 36|0);
      $107 = HEAP32[$106>>2]|0;
      $108 = (($107>>>0) / 4)&-1;
      $109 = (($108) - 2)|0;
      $110 = ($104>>>0)>($109>>>0);
      if ($110) {
       label = 28;
       break L22;
      }
      $112 = HEAP8[$searchList>>0]|0;
      $113 = $112&255;
      $114 = ($113|0)!=(0);
      do {
       if ($114) {
        $115 = HEAP32[$4>>2]|0;
        $116 = HEAP32[$iTrunk>>2]|0;
        $117 = ($115|0)==($116|0);
        if (!($117)) {
         $118 = HEAP32[$iTrunk>>2]|0;
         $119 = HEAP32[$4>>2]|0;
         $120 = ($118>>>0)<($119>>>0);
         if (!($120)) {
          break;
         }
         $121 = HEAP8[$5>>0]|0;
         $122 = $121&255;
         $123 = ($122|0)==(2);
         if (!($123)) {
          break;
         }
        }
        $124 = HEAP32[$iTrunk>>2]|0;
        $125 = HEAP32[$3>>2]|0;
        HEAP32[$125>>2] = $124;
        $126 = HEAP32[$pTrunk>>2]|0;
        $127 = HEAP32[$2>>2]|0;
        HEAP32[$127>>2] = $126;
        HEAP8[$searchList>>0] = 0;
        $128 = HEAP32[$pTrunk>>2]|0;
        $129 = ((($128)) + 72|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = (_sqlite3PagerWrite($130)|0);
        HEAP32[$rc>>2] = $131;
        $132 = HEAP32[$rc>>2]|0;
        $133 = ($132|0)!=(0);
        if ($133) {
         break L5;
        }
        $134 = HEAP32[$k>>2]|0;
        $135 = ($134|0)==(0);
        do {
         if ($135) {
          $136 = HEAP32[$pPrevTrunk>>2]|0;
          $137 = ($136|0)!=(0|0);
          if (!($137)) {
           $138 = HEAP32[$pPage1>>2]|0;
           $139 = ((($138)) + 56|0);
           $140 = HEAP32[$139>>2]|0;
           $141 = ((($140)) + 32|0);
           $142 = HEAP32[$pTrunk>>2]|0;
           $143 = ((($142)) + 56|0);
           $144 = HEAP32[$143>>2]|0;
           ;HEAP8[$141>>0]=HEAP8[$144>>0]|0;HEAP8[$141+1>>0]=HEAP8[$144+1>>0]|0;HEAP8[$141+2>>0]=HEAP8[$144+2>>0]|0;HEAP8[$141+3>>0]=HEAP8[$144+3>>0]|0;
           break;
          }
          $145 = HEAP32[$pPrevTrunk>>2]|0;
          $146 = ((($145)) + 72|0);
          $147 = HEAP32[$146>>2]|0;
          $148 = (_sqlite3PagerWrite($147)|0);
          HEAP32[$rc>>2] = $148;
          $149 = HEAP32[$rc>>2]|0;
          $150 = ($149|0)!=(0);
          if ($150) {
           break L5;
          }
          $151 = HEAP32[$pPrevTrunk>>2]|0;
          $152 = ((($151)) + 56|0);
          $153 = HEAP32[$152>>2]|0;
          $154 = HEAP32[$pTrunk>>2]|0;
          $155 = ((($154)) + 56|0);
          $156 = HEAP32[$155>>2]|0;
          ;HEAP8[$153>>0]=HEAP8[$156>>0]|0;HEAP8[$153+1>>0]=HEAP8[$156+1>>0]|0;HEAP8[$153+2>>0]=HEAP8[$156+2>>0]|0;HEAP8[$153+3>>0]=HEAP8[$156+3>>0]|0;
         } else {
          $157 = HEAP32[$pTrunk>>2]|0;
          $158 = ((($157)) + 56|0);
          $159 = HEAP32[$158>>2]|0;
          $160 = ((($159)) + 8|0);
          $161 = (_sqlite3Get4byte($160)|0);
          HEAP32[$iNewTrunk>>2] = $161;
          $162 = HEAP32[$iNewTrunk>>2]|0;
          $163 = HEAP32[$mxPage>>2]|0;
          $164 = ($162>>>0)>($163>>>0);
          if ($164) {
           label = 40;
           break L22;
          }
          $166 = HEAP32[$1>>2]|0;
          $167 = HEAP32[$iNewTrunk>>2]|0;
          $168 = (_btreeGetUnusedPage($166,$167,$pNewTrunk,0)|0);
          HEAP32[$rc>>2] = $168;
          $169 = HEAP32[$rc>>2]|0;
          $170 = ($169|0)!=(0);
          if ($170) {
           break L5;
          }
          $171 = HEAP32[$pNewTrunk>>2]|0;
          $172 = ((($171)) + 72|0);
          $173 = HEAP32[$172>>2]|0;
          $174 = (_sqlite3PagerWrite($173)|0);
          HEAP32[$rc>>2] = $174;
          $175 = HEAP32[$rc>>2]|0;
          $176 = ($175|0)!=(0);
          $177 = HEAP32[$pNewTrunk>>2]|0;
          if ($176) {
           label = 43;
           break L22;
          }
          $178 = ((($177)) + 56|0);
          $179 = HEAP32[$178>>2]|0;
          $180 = HEAP32[$pTrunk>>2]|0;
          $181 = ((($180)) + 56|0);
          $182 = HEAP32[$181>>2]|0;
          ;HEAP8[$179>>0]=HEAP8[$182>>0]|0;HEAP8[$179+1>>0]=HEAP8[$182+1>>0]|0;HEAP8[$179+2>>0]=HEAP8[$182+2>>0]|0;HEAP8[$179+3>>0]=HEAP8[$182+3>>0]|0;
          $183 = HEAP32[$pNewTrunk>>2]|0;
          $184 = ((($183)) + 56|0);
          $185 = HEAP32[$184>>2]|0;
          $186 = ((($185)) + 4|0);
          $187 = HEAP32[$k>>2]|0;
          $188 = (($187) - 1)|0;
          _sqlite3Put4byte($186,$188);
          $189 = HEAP32[$pNewTrunk>>2]|0;
          $190 = ((($189)) + 56|0);
          $191 = HEAP32[$190>>2]|0;
          $192 = ((($191)) + 8|0);
          $193 = HEAP32[$pTrunk>>2]|0;
          $194 = ((($193)) + 56|0);
          $195 = HEAP32[$194>>2]|0;
          $196 = ((($195)) + 12|0);
          $197 = HEAP32[$k>>2]|0;
          $198 = (($197) - 1)|0;
          $199 = $198<<2;
          _memcpy(($192|0),($196|0),($199|0))|0;
          $200 = HEAP32[$pNewTrunk>>2]|0;
          _releasePage($200);
          $201 = HEAP32[$pPrevTrunk>>2]|0;
          $202 = ($201|0)!=(0|0);
          if (!($202)) {
           $203 = HEAP32[$pPage1>>2]|0;
           $204 = ((($203)) + 56|0);
           $205 = HEAP32[$204>>2]|0;
           $206 = ((($205)) + 32|0);
           $207 = HEAP32[$iNewTrunk>>2]|0;
           _sqlite3Put4byte($206,$207);
           break;
          }
          $208 = HEAP32[$pPrevTrunk>>2]|0;
          $209 = ((($208)) + 72|0);
          $210 = HEAP32[$209>>2]|0;
          $211 = (_sqlite3PagerWrite($210)|0);
          HEAP32[$rc>>2] = $211;
          $212 = HEAP32[$rc>>2]|0;
          $213 = ($212|0)!=(0);
          if ($213) {
           break L5;
          }
          $214 = HEAP32[$pPrevTrunk>>2]|0;
          $215 = ((($214)) + 56|0);
          $216 = HEAP32[$215>>2]|0;
          $217 = HEAP32[$iNewTrunk>>2]|0;
          _sqlite3Put4byte($216,$217);
         }
        } while(0);
        HEAP32[$pTrunk>>2] = 0;
        break L35;
       }
      } while(0);
      $218 = HEAP32[$k>>2]|0;
      $219 = ($218>>>0)>(0);
      if ($219) {
       $220 = HEAP32[$pTrunk>>2]|0;
       $221 = ((($220)) + 56|0);
       $222 = HEAP32[$221>>2]|0;
       HEAP32[$aData>>2] = $222;
       $223 = HEAP32[$4>>2]|0;
       $224 = ($223>>>0)>(0);
       HEAP32[$closest>>2] = 0;
       L62: do {
        if ($224) {
         $225 = HEAP8[$5>>0]|0;
         $226 = $225&255;
         $227 = ($226|0)==(2);
         if ($227) {
          HEAP32[$i>>2] = 0;
          while(1) {
           $228 = HEAP32[$i>>2]|0;
           $229 = HEAP32[$k>>2]|0;
           $230 = ($228>>>0)<($229>>>0);
           if (!($230)) {
            break L62;
           }
           $231 = HEAP32[$i>>2]|0;
           $232 = $231<<2;
           $233 = (8 + ($232))|0;
           $234 = HEAP32[$aData>>2]|0;
           $235 = (($234) + ($233)|0);
           $236 = (_sqlite3Get4byte($235)|0);
           HEAP32[$iPage>>2] = $236;
           $237 = HEAP32[$iPage>>2]|0;
           $238 = HEAP32[$4>>2]|0;
           $239 = ($237>>>0)<=($238>>>0);
           $240 = HEAP32[$i>>2]|0;
           if ($239) {
            break;
           }
           $241 = (($240) + 1)|0;
           HEAP32[$i>>2] = $241;
          }
          HEAP32[$closest>>2] = $240;
          break;
         } else {
          $242 = HEAP32[$aData>>2]|0;
          $243 = ((($242)) + 8|0);
          $244 = (_sqlite3Get4byte($243)|0);
          $245 = HEAP32[$4>>2]|0;
          $246 = (($244) - ($245))|0;
          $247 = (_sqlite3AbsInt32($246)|0);
          HEAP32[$dist>>2] = $247;
          HEAP32[$i>>2] = 1;
          while(1) {
           $248 = HEAP32[$i>>2]|0;
           $249 = HEAP32[$k>>2]|0;
           $250 = ($248>>>0)<($249>>>0);
           if (!($250)) {
            break L62;
           }
           $251 = HEAP32[$i>>2]|0;
           $252 = $251<<2;
           $253 = (8 + ($252))|0;
           $254 = HEAP32[$aData>>2]|0;
           $255 = (($254) + ($253)|0);
           $256 = (_sqlite3Get4byte($255)|0);
           $257 = HEAP32[$4>>2]|0;
           $258 = (($256) - ($257))|0;
           $259 = (_sqlite3AbsInt32($258)|0);
           HEAP32[$d2>>2] = $259;
           $260 = HEAP32[$d2>>2]|0;
           $261 = HEAP32[$dist>>2]|0;
           $262 = ($260|0)<($261|0);
           if ($262) {
            $263 = HEAP32[$i>>2]|0;
            HEAP32[$closest>>2] = $263;
            $264 = HEAP32[$d2>>2]|0;
            HEAP32[$dist>>2] = $264;
           }
           $265 = HEAP32[$i>>2]|0;
           $266 = (($265) + 1)|0;
           HEAP32[$i>>2] = $266;
          }
         }
        }
       } while(0);
       $267 = HEAP32[$closest>>2]|0;
       $268 = $267<<2;
       $269 = (8 + ($268))|0;
       $270 = HEAP32[$aData>>2]|0;
       $271 = (($270) + ($269)|0);
       $272 = (_sqlite3Get4byte($271)|0);
       HEAP32[$iPage>>2] = $272;
       $273 = HEAP32[$iPage>>2]|0;
       $274 = HEAP32[$mxPage>>2]|0;
       $275 = ($273>>>0)>($274>>>0);
       if ($275) {
        label = 63;
        break L22;
       }
       $277 = HEAP8[$searchList>>0]|0;
       $278 = ($277<<24>>24)!=(0);
       if ($278) {
        $279 = HEAP32[$iPage>>2]|0;
        $280 = HEAP32[$4>>2]|0;
        $281 = ($279|0)==($280|0);
        if (!($281)) {
         $282 = HEAP32[$iPage>>2]|0;
         $283 = HEAP32[$4>>2]|0;
         $284 = ($282>>>0)<($283>>>0);
         if (!($284)) {
          break;
         }
         $285 = HEAP8[$5>>0]|0;
         $286 = $285&255;
         $287 = ($286|0)==(2);
         if (!($287)) {
          break;
         }
        }
       }
       $288 = HEAP32[$iPage>>2]|0;
       $289 = HEAP32[$3>>2]|0;
       HEAP32[$289>>2] = $288;
       $290 = HEAP32[$pTrunk>>2]|0;
       $291 = ((($290)) + 72|0);
       $292 = HEAP32[$291>>2]|0;
       $293 = (_sqlite3PagerWrite($292)|0);
       HEAP32[$rc>>2] = $293;
       $294 = HEAP32[$rc>>2]|0;
       $295 = ($294|0)!=(0);
       if ($295) {
        break L5;
       }
       $296 = HEAP32[$closest>>2]|0;
       $297 = HEAP32[$k>>2]|0;
       $298 = (($297) - 1)|0;
       $299 = ($296>>>0)<($298>>>0);
       if ($299) {
        $300 = HEAP32[$closest>>2]|0;
        $301 = $300<<2;
        $302 = (8 + ($301))|0;
        $303 = HEAP32[$aData>>2]|0;
        $304 = (($303) + ($302)|0);
        $305 = HEAP32[$k>>2]|0;
        $306 = $305<<2;
        $307 = (4 + ($306))|0;
        $308 = HEAP32[$aData>>2]|0;
        $309 = (($308) + ($307)|0);
        ;HEAP8[$304>>0]=HEAP8[$309>>0]|0;HEAP8[$304+1>>0]=HEAP8[$309+1>>0]|0;HEAP8[$304+2>>0]=HEAP8[$309+2>>0]|0;HEAP8[$304+3>>0]=HEAP8[$309+3>>0]|0;
       }
       $310 = HEAP32[$aData>>2]|0;
       $311 = ((($310)) + 4|0);
       $312 = HEAP32[$k>>2]|0;
       $313 = (($312) - 1)|0;
       _sqlite3Put4byte($311,$313);
       $314 = HEAP32[$1>>2]|0;
       $315 = HEAP32[$3>>2]|0;
       $316 = HEAP32[$315>>2]|0;
       $317 = (_btreeGetHasContent($314,$316)|0);
       $318 = ($317|0)!=(0);
       $319 = $318 ^ 1;
       $320 = $319 ? 1 : 0;
       HEAP32[$noContent>>2] = $320;
       $321 = HEAP32[$1>>2]|0;
       $322 = HEAP32[$3>>2]|0;
       $323 = HEAP32[$322>>2]|0;
       $324 = HEAP32[$2>>2]|0;
       $325 = HEAP32[$noContent>>2]|0;
       $326 = (_btreeGetUnusedPage($321,$323,$324,$325)|0);
       HEAP32[$rc>>2] = $326;
       $327 = HEAP32[$rc>>2]|0;
       $328 = ($327|0)==(0);
       do {
        if ($328) {
         $329 = HEAP32[$2>>2]|0;
         $330 = HEAP32[$329>>2]|0;
         $331 = ((($330)) + 72|0);
         $332 = HEAP32[$331>>2]|0;
         $333 = (_sqlite3PagerWrite($332)|0);
         HEAP32[$rc>>2] = $333;
         $334 = HEAP32[$rc>>2]|0;
         $335 = ($334|0)!=(0);
         if (!($335)) {
          break;
         }
         $336 = HEAP32[$2>>2]|0;
         $337 = HEAP32[$336>>2]|0;
         _releasePage($337);
         $338 = HEAP32[$2>>2]|0;
         HEAP32[$338>>2] = 0;
        }
       } while(0);
       HEAP8[$searchList>>0] = 0;
      }
     } else {
      $87 = HEAP32[$pTrunk>>2]|0;
      $88 = ((($87)) + 72|0);
      $89 = HEAP32[$88>>2]|0;
      $90 = (_sqlite3PagerWrite($89)|0);
      HEAP32[$rc>>2] = $90;
      $91 = HEAP32[$rc>>2]|0;
      $92 = ($91|0)!=(0);
      if ($92) {
       break L5;
      }
      $93 = HEAP32[$iTrunk>>2]|0;
      $94 = HEAP32[$3>>2]|0;
      HEAP32[$94>>2] = $93;
      $95 = HEAP32[$pPage1>>2]|0;
      $96 = ((($95)) + 56|0);
      $97 = HEAP32[$96>>2]|0;
      $98 = ((($97)) + 32|0);
      $99 = HEAP32[$pTrunk>>2]|0;
      $100 = ((($99)) + 56|0);
      $101 = HEAP32[$100>>2]|0;
      ;HEAP8[$98>>0]=HEAP8[$101>>0]|0;HEAP8[$98+1>>0]=HEAP8[$101+1>>0]|0;HEAP8[$98+2>>0]=HEAP8[$101+2>>0]|0;HEAP8[$98+3>>0]=HEAP8[$101+3>>0]|0;
      $102 = HEAP32[$pTrunk>>2]|0;
      $103 = HEAP32[$2>>2]|0;
      HEAP32[$103>>2] = $102;
      HEAP32[$pTrunk>>2] = 0;
     }
    } while(0);
    $339 = HEAP32[$pPrevTrunk>>2]|0;
    _releasePage($339);
    HEAP32[$pPrevTrunk>>2] = 0;
    $340 = HEAP8[$searchList>>0]|0;
    $341 = ($340<<24>>24)!=(0);
    if (!($341)) {
     break L5;
    }
   }
   if ((label|0) == 23) {
    HEAP32[$pTrunk>>2] = 0;
    break;
   }
   else if ((label|0) == 28) {
    $111 = (_sqlite3CorruptError(63878)|0);
    HEAP32[$rc>>2] = $111;
    break;
   }
   else if ((label|0) == 40) {
    $165 = (_sqlite3CorruptError(63912)|0);
    HEAP32[$rc>>2] = $165;
    break;
   }
   else if ((label|0) == 43) {
    _releasePage($177);
    break;
   }
   else if ((label|0) == 63) {
    $276 = (_sqlite3CorruptError(63977)|0);
    HEAP32[$rc>>2] = $276;
    break;
   }
  } else {
   $342 = HEAP32[$1>>2]|0;
   $343 = ((($342)) + 19|0);
   $344 = HEAP8[$343>>0]|0;
   $345 = $344&255;
   $346 = (0)==($345|0);
   $347 = $346 ? 1 : 0;
   HEAP32[$bNoContent>>2] = $347;
   $348 = HEAP32[$1>>2]|0;
   $349 = ((($348)) + 12|0);
   $350 = HEAP32[$349>>2]|0;
   $351 = ((($350)) + 72|0);
   $352 = HEAP32[$351>>2]|0;
   $353 = (_sqlite3PagerWrite($352)|0);
   HEAP32[$rc>>2] = $353;
   $354 = HEAP32[$rc>>2]|0;
   $355 = ($354|0)!=(0);
   if ($355) {
    $356 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $356;
    $458 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($458|0);
   }
   $357 = HEAP32[$1>>2]|0;
   $358 = ((($357)) + 44|0);
   $359 = HEAP32[$358>>2]|0;
   $360 = (($359) + 1)|0;
   HEAP32[$358>>2] = $360;
   $361 = HEAP32[$1>>2]|0;
   $362 = ((($361)) + 44|0);
   $363 = HEAP32[$362>>2]|0;
   $364 = HEAP32[1536>>2]|0;
   $365 = HEAP32[$1>>2]|0;
   $366 = ((($365)) + 32|0);
   $367 = HEAP32[$366>>2]|0;
   $368 = (($364>>>0) / ($367>>>0))&-1;
   $369 = (($368) + 1)|0;
   $370 = ($363|0)==($369|0);
   if ($370) {
    $371 = HEAP32[$1>>2]|0;
    $372 = ((($371)) + 44|0);
    $373 = HEAP32[$372>>2]|0;
    $374 = (($373) + 1)|0;
    HEAP32[$372>>2] = $374;
   }
   $375 = HEAP32[$1>>2]|0;
   $376 = ((($375)) + 17|0);
   $377 = HEAP8[$376>>0]|0;
   $378 = $377&255;
   $379 = ($378|0)!=(0);
   do {
    if ($379) {
     $380 = HEAP32[$1>>2]|0;
     $381 = HEAP32[$1>>2]|0;
     $382 = ((($381)) + 44|0);
     $383 = HEAP32[$382>>2]|0;
     $384 = (_ptrmapPageno($380,$383)|0);
     $385 = HEAP32[$1>>2]|0;
     $386 = ((($385)) + 44|0);
     $387 = HEAP32[$386>>2]|0;
     $388 = ($384|0)==($387|0);
     if ($388) {
      HEAP32[$pPg>>2] = 0;
      $389 = HEAP32[$1>>2]|0;
      $390 = HEAP32[$1>>2]|0;
      $391 = ((($390)) + 44|0);
      $392 = HEAP32[$391>>2]|0;
      $393 = HEAP32[$bNoContent>>2]|0;
      $394 = (_btreeGetUnusedPage($389,$392,$pPg,$393)|0);
      HEAP32[$rc>>2] = $394;
      $395 = HEAP32[$rc>>2]|0;
      $396 = ($395|0)==(0);
      if ($396) {
       $397 = HEAP32[$pPg>>2]|0;
       $398 = ((($397)) + 72|0);
       $399 = HEAP32[$398>>2]|0;
       $400 = (_sqlite3PagerWrite($399)|0);
       HEAP32[$rc>>2] = $400;
       $401 = HEAP32[$pPg>>2]|0;
       _releasePage($401);
      }
      $402 = HEAP32[$rc>>2]|0;
      $403 = ($402|0)!=(0);
      if ($403) {
       $404 = HEAP32[$rc>>2]|0;
       HEAP32[$0>>2] = $404;
       $458 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($458|0);
      } else {
       $405 = HEAP32[$1>>2]|0;
       $406 = ((($405)) + 44|0);
       $407 = HEAP32[$406>>2]|0;
       $408 = (($407) + 1)|0;
       HEAP32[$406>>2] = $408;
       $409 = HEAP32[$1>>2]|0;
       $410 = ((($409)) + 44|0);
       $411 = HEAP32[$410>>2]|0;
       $412 = HEAP32[1536>>2]|0;
       $413 = HEAP32[$1>>2]|0;
       $414 = ((($413)) + 32|0);
       $415 = HEAP32[$414>>2]|0;
       $416 = (($412>>>0) / ($415>>>0))&-1;
       $417 = (($416) + 1)|0;
       $418 = ($411|0)==($417|0);
       if (!($418)) {
        break;
       }
       $419 = HEAP32[$1>>2]|0;
       $420 = ((($419)) + 44|0);
       $421 = HEAP32[$420>>2]|0;
       $422 = (($421) + 1)|0;
       HEAP32[$420>>2] = $422;
       break;
      }
     }
    }
   } while(0);
   $423 = HEAP32[$1>>2]|0;
   $424 = ((($423)) + 12|0);
   $425 = HEAP32[$424>>2]|0;
   $426 = ((($425)) + 56|0);
   $427 = HEAP32[$426>>2]|0;
   $428 = ((($427)) + 28|0);
   $429 = HEAP32[$1>>2]|0;
   $430 = ((($429)) + 44|0);
   $431 = HEAP32[$430>>2]|0;
   _sqlite3Put4byte($428,$431);
   $432 = HEAP32[$1>>2]|0;
   $433 = ((($432)) + 44|0);
   $434 = HEAP32[$433>>2]|0;
   $435 = HEAP32[$3>>2]|0;
   HEAP32[$435>>2] = $434;
   $436 = HEAP32[$1>>2]|0;
   $437 = HEAP32[$3>>2]|0;
   $438 = HEAP32[$437>>2]|0;
   $439 = HEAP32[$2>>2]|0;
   $440 = HEAP32[$bNoContent>>2]|0;
   $441 = (_btreeGetUnusedPage($436,$438,$439,$440)|0);
   HEAP32[$rc>>2] = $441;
   $442 = HEAP32[$rc>>2]|0;
   $443 = ($442|0)!=(0);
   if ($443) {
    $444 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $444;
    $458 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($458|0);
   } else {
    $445 = HEAP32[$2>>2]|0;
    $446 = HEAP32[$445>>2]|0;
    $447 = ((($446)) + 72|0);
    $448 = HEAP32[$447>>2]|0;
    $449 = (_sqlite3PagerWrite($448)|0);
    HEAP32[$rc>>2] = $449;
    $450 = HEAP32[$rc>>2]|0;
    $451 = ($450|0)!=(0);
    if (!($451)) {
     break;
    }
    $452 = HEAP32[$2>>2]|0;
    $453 = HEAP32[$452>>2]|0;
    _releasePage($453);
    $454 = HEAP32[$2>>2]|0;
    HEAP32[$454>>2] = 0;
    break;
   }
  }
 } while(0);
 $455 = HEAP32[$pTrunk>>2]|0;
 _releasePage($455);
 $456 = HEAP32[$pPrevTrunk>>2]|0;
 _releasePage($456);
 $457 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $457;
 $458 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($458|0);
}
function _relocatePage($pBt,$pDbPage,$eType,$iPtrPage,$iFreePage,$isCommit) {
 $pBt = $pBt|0;
 $pDbPage = $pDbPage|0;
 $eType = $eType|0;
 $iPtrPage = $iPtrPage|0;
 $iFreePage = $iFreePage|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $8 = 0, $9 = 0, $iDbPage = 0, $nextOvfl = 0;
 var $pPager = 0, $pPtrPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 44|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $pPtrPage = sp + 16|0;
 $iDbPage = sp + 12|0;
 $pPager = sp + 8|0;
 $rc = sp + 4|0;
 $nextOvfl = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pDbPage;
 HEAP8[$3>>0] = $eType;
 HEAP32[$4>>2] = $iPtrPage;
 HEAP32[$5>>2] = $iFreePage;
 HEAP32[$6>>2] = $isCommit;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 84|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$iDbPage>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pPager>>2] = $11;
 $12 = HEAP32[$pPager>>2]|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 72|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$5>>2]|0;
 $17 = HEAP32[$6>>2]|0;
 $18 = (_sqlite3PagerMovepage($12,$15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $21;
  $77 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($77|0);
 }
 $22 = HEAP32[$5>>2]|0;
 $23 = HEAP32[$2>>2]|0;
 $24 = ((($23)) + 84|0);
 HEAP32[$24>>2] = $22;
 $25 = HEAP8[$3>>0]|0;
 $26 = $25&255;
 $27 = ($26|0)==(5);
 if ($27) {
  label = 5;
 } else {
  $28 = HEAP8[$3>>0]|0;
  $29 = $28&255;
  $30 = ($29|0)==(1);
  if ($30) {
   label = 5;
  } else {
   $36 = HEAP32[$2>>2]|0;
   $37 = ((($36)) + 56|0);
   $38 = HEAP32[$37>>2]|0;
   $39 = (_sqlite3Get4byte($38)|0);
   HEAP32[$nextOvfl>>2] = $39;
   $40 = HEAP32[$nextOvfl>>2]|0;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$1>>2]|0;
    $43 = HEAP32[$nextOvfl>>2]|0;
    $44 = HEAP32[$5>>2]|0;
    _ptrmapPut($42,$43,4,$44,$rc);
    $45 = HEAP32[$rc>>2]|0;
    $46 = ($45|0)!=(0);
    if ($46) {
     $47 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $47;
     $77 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($77|0);
    }
   }
  }
 }
 if ((label|0) == 5) {
  $31 = HEAP32[$2>>2]|0;
  $32 = (_setChildPtrmaps($31)|0);
  HEAP32[$rc>>2] = $32;
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $35;
   $77 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($77|0);
  }
 }
 $48 = HEAP8[$3>>0]|0;
 $49 = $48&255;
 $50 = ($49|0)!=(1);
 do {
  if ($50) {
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$4>>2]|0;
   $53 = (_btreeGetPage($51,$52,$pPtrPage,0)|0);
   HEAP32[$rc>>2] = $53;
   $54 = HEAP32[$rc>>2]|0;
   $55 = ($54|0)!=(0);
   if ($55) {
    $56 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $56;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   }
   $57 = HEAP32[$pPtrPage>>2]|0;
   $58 = ((($57)) + 72|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (_sqlite3PagerWrite($59)|0);
   HEAP32[$rc>>2] = $60;
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)!=(0);
   $63 = HEAP32[$pPtrPage>>2]|0;
   if ($62) {
    _releasePage($63);
    $64 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $64;
    $77 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($77|0);
   } else {
    $65 = HEAP32[$iDbPage>>2]|0;
    $66 = HEAP32[$5>>2]|0;
    $67 = HEAP8[$3>>0]|0;
    $68 = (_modifyPagePointer($63,$65,$66,$67)|0);
    HEAP32[$rc>>2] = $68;
    $69 = HEAP32[$pPtrPage>>2]|0;
    _releasePage($69);
    $70 = HEAP32[$rc>>2]|0;
    $71 = ($70|0)==(0);
    if (!($71)) {
     break;
    }
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$5>>2]|0;
    $74 = HEAP8[$3>>0]|0;
    $75 = HEAP32[$4>>2]|0;
    _ptrmapPut($72,$73,$74,$75,$rc);
    break;
   }
  }
 } while(0);
 $76 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $76;
 $77 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($77|0);
}
function _sqlite3PagerMovepage($pPager,$pPg,$pgno,$isCommit) {
 $pPager = $pPager|0;
 $pPg = $pPg|0;
 $pgno = $pgno|0;
 $isCommit = $isCommit|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $needSyncPgno = 0, $or$cond = 0, $or$cond3 = 0, $origPgno = 0, $pPgHdr = 0, $pPgOld = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pPgOld = sp + 16|0;
 $needSyncPgno = sp + 12|0;
 $rc = sp + 8|0;
 $origPgno = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pPg;
 HEAP32[$3>>2] = $pgno;
 HEAP32[$4>>2] = $isCommit;
 HEAP32[$needSyncPgno>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 13|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$2>>2]|0;
  $10 = (_sqlite3PagerWrite($9)|0);
  HEAP32[$rc>>2] = $10;
  $11 = HEAP32[$rc>>2]|0;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $13;
   $110 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($110|0);
  }
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 24|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = $17 & 2;
 $19 = ($18|0)!=(0);
 if ($19) {
  $20 = HEAP32[$2>>2]|0;
  $21 = (_subjournalPageIfRequired($20)|0);
  HEAP32[$rc>>2] = $21;
  $22 = (0)!=($21|0);
  if ($22) {
   $23 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $23;
   $110 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($110|0);
  }
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 8;
 $29 = ($28|0)==(0);
 $30 = HEAP32[$4>>2]|0;
 $31 = ($30|0)!=(0);
 $or$cond = $29 | $31;
 if (!($or$cond)) {
  $32 = HEAP32[$2>>2]|0;
  $33 = ((($32)) + 20|0);
  $34 = HEAP32[$33>>2]|0;
  HEAP32[$needSyncPgno>>2] = $34;
 }
 $35 = HEAP32[$2>>2]|0;
 $36 = ((($35)) + 24|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37&65535;
 $39 = $38 & -9;
 $40 = $39&65535;
 HEAP16[$36>>1] = $40;
 $41 = HEAP32[$1>>2]|0;
 $42 = HEAP32[$3>>2]|0;
 $43 = (_sqlite3PagerLookup($41,$42)|0);
 HEAP32[$pPgOld>>2] = $43;
 $44 = HEAP32[$pPgOld>>2]|0;
 $45 = ($44|0)!=(0|0);
 do {
  if ($45) {
   $46 = HEAP32[$pPgOld>>2]|0;
   $47 = ((($46)) + 24|0);
   $48 = HEAP16[$47>>1]|0;
   $49 = $48&65535;
   $50 = $49 & 8;
   $51 = HEAP32[$2>>2]|0;
   $52 = ((($51)) + 24|0);
   $53 = HEAP16[$52>>1]|0;
   $54 = $53&65535;
   $55 = $54 | $50;
   $56 = $55&65535;
   HEAP16[$52>>1] = $56;
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 13|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = ($59<<24>>24)!=(0);
   $61 = HEAP32[$pPgOld>>2]|0;
   if ($60) {
    $62 = HEAP32[$1>>2]|0;
    $63 = ((($62)) + 28|0);
    $64 = HEAP32[$63>>2]|0;
    $65 = (($64) + 1)|0;
    _sqlite3PcacheMove($61,$65);
    break;
   } else {
    _sqlite3PcacheDrop($61);
    break;
   }
  }
 } while(0);
 $66 = HEAP32[$2>>2]|0;
 $67 = ((($66)) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 HEAP32[$origPgno>>2] = $68;
 $69 = HEAP32[$2>>2]|0;
 $70 = HEAP32[$3>>2]|0;
 _sqlite3PcacheMove($69,$70);
 $71 = HEAP32[$2>>2]|0;
 _sqlite3PcacheMakeDirty($71);
 $72 = HEAP32[$1>>2]|0;
 $73 = ((($72)) + 13|0);
 $74 = HEAP8[$73>>0]|0;
 $75 = $74&255;
 $76 = ($75|0)!=(0);
 $77 = HEAP32[$pPgOld>>2]|0;
 $78 = ($77|0)!=(0|0);
 $or$cond3 = $76 & $78;
 if ($or$cond3) {
  $79 = HEAP32[$pPgOld>>2]|0;
  $80 = HEAP32[$origPgno>>2]|0;
  _sqlite3PcacheMove($79,$80);
  $81 = HEAP32[$pPgOld>>2]|0;
  _sqlite3PagerUnrefNotNull($81);
 }
 $82 = HEAP32[$needSyncPgno>>2]|0;
 $83 = ($82|0)!=(0);
 do {
  if ($83) {
   $84 = HEAP32[$1>>2]|0;
   $85 = HEAP32[$needSyncPgno>>2]|0;
   $86 = (_sqlite3PagerGet($84,$85,$pPgHdr,0)|0);
   HEAP32[$rc>>2] = $86;
   $87 = HEAP32[$rc>>2]|0;
   $88 = ($87|0)!=(0);
   if (!($88)) {
    $102 = HEAP32[$pPgHdr>>2]|0;
    $103 = ((($102)) + 24|0);
    $104 = HEAP16[$103>>1]|0;
    $105 = $104&65535;
    $106 = $105 | 8;
    $107 = $106&65535;
    HEAP16[$103>>1] = $107;
    $108 = HEAP32[$pPgHdr>>2]|0;
    _sqlite3PcacheMakeDirty($108);
    $109 = HEAP32[$pPgHdr>>2]|0;
    _sqlite3PagerUnrefNotNull($109);
    break;
   }
   $89 = HEAP32[$needSyncPgno>>2]|0;
   $90 = HEAP32[$1>>2]|0;
   $91 = ((($90)) + 32|0);
   $92 = HEAP32[$91>>2]|0;
   $93 = ($89>>>0)<=($92>>>0);
   if ($93) {
    $94 = HEAP32[$1>>2]|0;
    $95 = ((($94)) + 60|0);
    $96 = HEAP32[$95>>2]|0;
    $97 = HEAP32[$needSyncPgno>>2]|0;
    $98 = HEAP32[$1>>2]|0;
    $99 = ((($98)) + 208|0);
    $100 = HEAP32[$99>>2]|0;
    _sqlite3BitvecClear($96,$97,$100);
   }
   $101 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $101;
   $110 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($110|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $110 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($110|0);
}
function _setChildPtrmaps($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $childPgno = 0, $childPgno1 = 0, $i = 0, $isInitOrig = 0, $nCell = 0, $pBt = 0, $pCell = 0, $pgno = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $i = sp + 28|0;
 $nCell = sp + 24|0;
 $rc = sp + 20|0;
 $pBt = sp + 16|0;
 $isInitOrig = sp + 36|0;
 $pgno = sp + 12|0;
 $pCell = sp + 8|0;
 $childPgno = sp + 4|0;
 $childPgno1 = sp;
 HEAP32[$0>>2] = $pPage;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 52|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP8[$4>>0]|0;
 HEAP8[$isInitOrig>>0] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 84|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pgno>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = (_btreeInitPage($9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $77 = HEAP8[$isInitOrig>>0]|0;
  $78 = HEAP32[$0>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 18|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 HEAP32[$nCell>>2] = $16;
 HEAP32[$i>>2] = 0;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$nCell>>2]|0;
  $19 = ($17|0)<($18|0);
  $20 = HEAP32[$0>>2]|0;
  if (!($19)) {
   break;
  }
  $21 = ((($20)) + 56|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 20|0);
  $25 = HEAP16[$24>>1]|0;
  $26 = $25&65535;
  $27 = HEAP32[$i>>2]|0;
  $28 = $27<<1;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 64|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28)|0);
  $33 = HEAP8[$32>>0]|0;
  $34 = $33&255;
  $35 = $34 << 8;
  $36 = HEAP32[$i>>2]|0;
  $37 = $36<<1;
  $38 = HEAP32[$0>>2]|0;
  $39 = ((($38)) + 64|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = (($40) + ($37)|0);
  $42 = ((($41)) + 1|0);
  $43 = HEAP8[$42>>0]|0;
  $44 = $43&255;
  $45 = $35 | $44;
  $46 = $26 & $45;
  $47 = (($22) + ($46)|0);
  HEAP32[$pCell>>2] = $47;
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$pCell>>2]|0;
  _ptrmapPutOvflPtr($48,$49,$rc);
  $50 = HEAP32[$0>>2]|0;
  $51 = ((($50)) + 4|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = ($52<<24>>24)!=(0);
  if (!($53)) {
   $54 = HEAP32[$pCell>>2]|0;
   $55 = (_sqlite3Get4byte($54)|0);
   HEAP32[$childPgno>>2] = $55;
   $56 = HEAP32[$pBt>>2]|0;
   $57 = HEAP32[$childPgno>>2]|0;
   $58 = HEAP32[$pgno>>2]|0;
   _ptrmapPut($56,$57,5,$58,$rc);
  }
  $59 = HEAP32[$i>>2]|0;
  $60 = (($59) + 1)|0;
  HEAP32[$i>>2] = $60;
 }
 $61 = ((($20)) + 4|0);
 $62 = HEAP8[$61>>0]|0;
 $63 = ($62<<24>>24)!=(0);
 if ($63) {
  $77 = HEAP8[$isInitOrig>>0]|0;
  $78 = HEAP32[$0>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($79|0);
 }
 $64 = HEAP32[$0>>2]|0;
 $65 = ((($64)) + 5|0);
 $66 = HEAP8[$65>>0]|0;
 $67 = $66&255;
 $68 = (($67) + 8)|0;
 $69 = HEAP32[$0>>2]|0;
 $70 = ((($69)) + 56|0);
 $71 = HEAP32[$70>>2]|0;
 $72 = (($71) + ($68)|0);
 $73 = (_sqlite3Get4byte($72)|0);
 HEAP32[$childPgno1>>2] = $73;
 $74 = HEAP32[$pBt>>2]|0;
 $75 = HEAP32[$childPgno1>>2]|0;
 $76 = HEAP32[$pgno>>2]|0;
 _ptrmapPut($74,$75,5,$76,$rc);
 $77 = HEAP8[$isInitOrig>>0]|0;
 $78 = HEAP32[$0>>2]|0;
 HEAP8[$78>>0] = $77;
 $79 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($79|0);
}
function _ptrmapPut($pBt,$key,$eType,$parent,$pRC) {
 $pBt = $pBt|0;
 $key = $key|0;
 $eType = $eType|0;
 $parent = $parent|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $iPtrmap = 0, $offset = 0, $pDbPage = 0, $pPtrmap = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 36|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pDbPage = sp + 16|0;
 $pPtrmap = sp + 12|0;
 $iPtrmap = sp + 8|0;
 $offset = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $key;
 HEAP8[$2>>0] = $eType;
 HEAP32[$3>>2] = $parent;
 HEAP32[$4>>2] = $pRC;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  $10 = (_sqlite3CorruptError(59184)|0);
  $11 = HEAP32[$4>>2]|0;
  HEAP32[$11>>2] = $10;
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = (_ptrmapPageno($12,$13)|0);
 HEAP32[$iPtrmap>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$iPtrmap>>2]|0;
 $18 = (_sqlite3PagerGet($16,$17,$pDbPage,0)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$rc>>2]|0;
  $22 = HEAP32[$4>>2]|0;
  HEAP32[$22>>2] = $21;
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$1>>2]|0;
 $24 = HEAP32[$iPtrmap>>2]|0;
 $25 = (($23) - ($24))|0;
 $26 = (($25) - 1)|0;
 $27 = ($26*5)|0;
 HEAP32[$offset>>2] = $27;
 $28 = HEAP32[$offset>>2]|0;
 $29 = ($28|0)<(0);
 do {
  if ($29) {
   $30 = (_sqlite3CorruptError(59195)|0);
   $31 = HEAP32[$4>>2]|0;
   HEAP32[$31>>2] = $30;
  } else {
   $32 = HEAP32[$pDbPage>>2]|0;
   $33 = (_sqlite3PagerGetData($32)|0);
   HEAP32[$pPtrmap>>2] = $33;
   $34 = HEAP8[$2>>0]|0;
   $35 = $34&255;
   $36 = HEAP32[$offset>>2]|0;
   $37 = HEAP32[$pPtrmap>>2]|0;
   $38 = (($37) + ($36)|0);
   $39 = HEAP8[$38>>0]|0;
   $40 = $39&255;
   $41 = ($35|0)!=($40|0);
   if (!($41)) {
    $42 = HEAP32[$offset>>2]|0;
    $43 = (($42) + 1)|0;
    $44 = HEAP32[$pPtrmap>>2]|0;
    $45 = (($44) + ($43)|0);
    $46 = (_sqlite3Get4byte($45)|0);
    $47 = HEAP32[$3>>2]|0;
    $48 = ($46|0)!=($47|0);
    if (!($48)) {
     break;
    }
   }
   $49 = HEAP32[$pDbPage>>2]|0;
   $50 = (_sqlite3PagerWrite($49)|0);
   HEAP32[$rc>>2] = $50;
   $51 = HEAP32[$4>>2]|0;
   HEAP32[$51>>2] = $50;
   $52 = HEAP32[$rc>>2]|0;
   $53 = ($52|0)==(0);
   if ($53) {
    $54 = HEAP8[$2>>0]|0;
    $55 = HEAP32[$offset>>2]|0;
    $56 = HEAP32[$pPtrmap>>2]|0;
    $57 = (($56) + ($55)|0);
    HEAP8[$57>>0] = $54;
    $58 = HEAP32[$offset>>2]|0;
    $59 = (($58) + 1)|0;
    $60 = HEAP32[$pPtrmap>>2]|0;
    $61 = (($60) + ($59)|0);
    $62 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($61,$62);
   }
  }
 } while(0);
 $63 = HEAP32[$pDbPage>>2]|0;
 _sqlite3PagerUnref($63);
 STACKTOP = sp;return;
}
function _modifyPagePointer($pPage,$iFrom,$iTo,$eType) {
 $pPage = $pPage|0;
 $iFrom = $iFrom|0;
 $iTo = $iTo|0;
 $eType = $eType|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $info = 0, $isInitOrig = 0, $nCell = 0, $pCell = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 57|0;
 $isInitOrig = sp + 56|0;
 $i = sp + 36|0;
 $nCell = sp + 32|0;
 $rc = sp + 28|0;
 $pCell = sp + 24|0;
 $info = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $iFrom;
 HEAP32[$3>>2] = $iTo;
 HEAP8[$4>>0] = $eType;
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)==(4);
 $8 = HEAP32[$1>>2]|0;
 do {
  if ($7) {
   $9 = ((($8)) + 56|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3Get4byte($10)|0);
   $12 = HEAP32[$2>>2]|0;
   $13 = ($11|0)!=($12|0);
   if (!($13)) {
    $15 = HEAP32[$1>>2]|0;
    $16 = ((($15)) + 56|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($17,$18);
    break;
   }
   $14 = (_sqlite3CorruptError(61581)|0);
   HEAP32[$0>>2] = $14;
   $144 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($144|0);
  } else {
   $19 = HEAP8[$8>>0]|0;
   HEAP8[$isInitOrig>>0] = $19;
   $20 = HEAP32[$1>>2]|0;
   $21 = (_btreeInitPage($20)|0);
   HEAP32[$rc>>2] = $21;
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)!=(0);
   if ($23) {
    $24 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $24;
    $144 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($144|0);
   }
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 18|0);
   $27 = HEAP16[$26>>1]|0;
   $28 = $27&65535;
   HEAP32[$nCell>>2] = $28;
   HEAP32[$i>>2] = 0;
   while(1) {
    $29 = HEAP32[$i>>2]|0;
    $30 = HEAP32[$nCell>>2]|0;
    $31 = ($29|0)<($30|0);
    if (!($31)) {
     break;
    }
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 56|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = HEAP32[$1>>2]|0;
    $36 = ((($35)) + 20|0);
    $37 = HEAP16[$36>>1]|0;
    $38 = $37&65535;
    $39 = HEAP32[$i>>2]|0;
    $40 = $39<<1;
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 64|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = (($43) + ($40)|0);
    $45 = HEAP8[$44>>0]|0;
    $46 = $45&255;
    $47 = $46 << 8;
    $48 = HEAP32[$i>>2]|0;
    $49 = $48<<1;
    $50 = HEAP32[$1>>2]|0;
    $51 = ((($50)) + 64|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = (($52) + ($49)|0);
    $54 = ((($53)) + 1|0);
    $55 = HEAP8[$54>>0]|0;
    $56 = $55&255;
    $57 = $47 | $56;
    $58 = $38 & $57;
    $59 = (($34) + ($58)|0);
    HEAP32[$pCell>>2] = $59;
    $60 = HEAP8[$4>>0]|0;
    $61 = $60&255;
    $62 = ($61|0)==(3);
    if ($62) {
     $63 = HEAP32[$1>>2]|0;
     $64 = ((($63)) + 80|0);
     $65 = HEAP32[$64>>2]|0;
     $66 = HEAP32[$1>>2]|0;
     $67 = HEAP32[$pCell>>2]|0;
     FUNCTION_TABLE_viii[$65 & 255]($66,$67,$info);
     $68 = ((($info)) + 16|0);
     $69 = HEAP16[$68>>1]|0;
     $70 = $69&65535;
     $71 = ((($info)) + 12|0);
     $72 = HEAP32[$71>>2]|0;
     $73 = ($70>>>0)<($72>>>0);
     if ($73) {
      $74 = HEAP32[$pCell>>2]|0;
      $75 = ((($info)) + 18|0);
      $76 = HEAP16[$75>>1]|0;
      $77 = $76&65535;
      $78 = (($74) + ($77)|0);
      $79 = ((($78)) + -1|0);
      $80 = HEAP32[$1>>2]|0;
      $81 = ((($80)) + 56|0);
      $82 = HEAP32[$81>>2]|0;
      $83 = HEAP32[$1>>2]|0;
      $84 = ((($83)) + 20|0);
      $85 = HEAP16[$84>>1]|0;
      $86 = $85&65535;
      $87 = (($82) + ($86)|0);
      $88 = ($79>>>0)<=($87>>>0);
      if ($88) {
       $89 = HEAP32[$2>>2]|0;
       $90 = HEAP32[$pCell>>2]|0;
       $91 = ((($info)) + 18|0);
       $92 = HEAP16[$91>>1]|0;
       $93 = $92&65535;
       $94 = (($90) + ($93)|0);
       $95 = ((($94)) + -4|0);
       $96 = (_sqlite3Get4byte($95)|0);
       $97 = ($89|0)==($96|0);
       if ($97) {
        label = 13;
        break;
       }
      }
     }
    } else {
     $105 = HEAP32[$pCell>>2]|0;
     $106 = (_sqlite3Get4byte($105)|0);
     $107 = HEAP32[$2>>2]|0;
     $108 = ($106|0)==($107|0);
     if ($108) {
      label = 15;
      break;
     }
    }
    $111 = HEAP32[$i>>2]|0;
    $112 = (($111) + 1)|0;
    HEAP32[$i>>2] = $112;
   }
   if ((label|0) == 13) {
    $98 = HEAP32[$pCell>>2]|0;
    $99 = ((($info)) + 18|0);
    $100 = HEAP16[$99>>1]|0;
    $101 = $100&65535;
    $102 = (($98) + ($101)|0);
    $103 = ((($102)) + -4|0);
    $104 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($103,$104);
   }
   else if ((label|0) == 15) {
    $109 = HEAP32[$pCell>>2]|0;
    $110 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($109,$110);
   }
   $113 = HEAP32[$i>>2]|0;
   $114 = HEAP32[$nCell>>2]|0;
   $115 = ($113|0)==($114|0);
   do {
    if ($115) {
     $116 = HEAP8[$4>>0]|0;
     $117 = $116&255;
     $118 = ($117|0)!=(5);
     if (!($118)) {
      $119 = HEAP32[$1>>2]|0;
      $120 = ((($119)) + 5|0);
      $121 = HEAP8[$120>>0]|0;
      $122 = $121&255;
      $123 = (($122) + 8)|0;
      $124 = HEAP32[$1>>2]|0;
      $125 = ((($124)) + 56|0);
      $126 = HEAP32[$125>>2]|0;
      $127 = (($126) + ($123)|0);
      $128 = (_sqlite3Get4byte($127)|0);
      $129 = HEAP32[$2>>2]|0;
      $130 = ($128|0)!=($129|0);
      if (!($130)) {
       $132 = HEAP32[$1>>2]|0;
       $133 = ((($132)) + 5|0);
       $134 = HEAP8[$133>>0]|0;
       $135 = $134&255;
       $136 = (($135) + 8)|0;
       $137 = HEAP32[$1>>2]|0;
       $138 = ((($137)) + 56|0);
       $139 = HEAP32[$138>>2]|0;
       $140 = (($139) + ($136)|0);
       $141 = HEAP32[$3>>2]|0;
       _sqlite3Put4byte($140,$141);
       break;
      }
     }
     $131 = (_sqlite3CorruptError(61617)|0);
     HEAP32[$0>>2] = $131;
     $144 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($144|0);
    }
   } while(0);
   $142 = HEAP8[$isInitOrig>>0]|0;
   $143 = HEAP32[$1>>2]|0;
   HEAP8[$143>>0] = $142;
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $144 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($144|0);
}
function _ptrmapPutOvflPtr($pPage,$pCell,$pRC) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $info = 0, $ovfl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $info = sp;
 $ovfl = sp + 24|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pCell;
 HEAP32[$2>>2] = $pRC;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if ($5) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 80|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 FUNCTION_TABLE_viii[$8 & 255]($9,$10,$info);
 $11 = ((($info)) + 16|0);
 $12 = HEAP16[$11>>1]|0;
 $13 = $12&65535;
 $14 = ((($info)) + 12|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($13>>>0)<($15>>>0);
 if (!($16)) {
  STACKTOP = sp;return;
 }
 $17 = ((($info)) + 18|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = (($19) - 4)|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = (($21) + ($20)|0);
 $23 = (_sqlite3Get4byte($22)|0);
 HEAP32[$ovfl>>2] = $23;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 52|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$ovfl>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 84|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$2>>2]|0;
 _ptrmapPut($26,$27,3,$30,$31);
 STACKTOP = sp;return;
}
function _sqlite3PcacheMove($p,$newPgno) {
 $p = $p|0;
 $newPgno = $newPgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pCache = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pCache = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $newPgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 28|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pCache>>2] = $4;
 $5 = HEAP32[(152)>>2]|0;
 $6 = HEAP32[$pCache>>2]|0;
 $7 = ((($6)) + 44|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 FUNCTION_TABLE_viiii[$5 & 255]($8,$10,$13,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$0>>2]|0;
 $17 = ((($16)) + 20|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 24|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 & 2;
 $23 = ($22|0)!=(0);
 if (!($23)) {
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 8;
 $29 = ($28|0)!=(0);
 if (!($29)) {
  STACKTOP = sp;return;
 }
 $30 = HEAP32[$0>>2]|0;
 _pcacheManageDirtyList($30,3);
 STACKTOP = sp;return;
}
function _btreeGetUnusedPage($pBt,$pgno,$ppPage,$flags) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 HEAP32[$4>>2] = $flags;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$3>>2]|0;
 $8 = HEAP32[$4>>2]|0;
 $9 = (_btreeGetPage($5,$6,$7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 $12 = HEAP32[$3>>2]|0;
 do {
  if ($11) {
   $13 = HEAP32[$12>>2]|0;
   $14 = ((($13)) + 72|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = (_sqlite3PagerPageRefcount($15)|0);
   $17 = ($16|0)>(1);
   $18 = HEAP32[$3>>2]|0;
   $19 = HEAP32[$18>>2]|0;
   if (!($17)) {
    HEAP8[$19>>0] = 0;
    break;
   }
   _releasePage($19);
   $20 = HEAP32[$3>>2]|0;
   HEAP32[$20>>2] = 0;
   $21 = (_sqlite3CorruptError(60316)|0);
   HEAP32[$0>>2] = $21;
   $23 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($23|0);
  } else {
   HEAP32[$12>>2] = 0;
  }
 } while(0);
 $22 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _btreeGetHasContent($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pgno;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $16 = 0;
  $15 = $16&1;
  STACKTOP = sp;return ($15|0);
 }
 $7 = HEAP32[$1>>2]|0;
 $8 = HEAP32[$p>>2]|0;
 $9 = (_sqlite3BitvecSize($8)|0);
 $10 = ($7>>>0)>($9>>>0);
 if ($10) {
  $16 = 1;
  $15 = $16&1;
  STACKTOP = sp;return ($15|0);
 }
 $11 = HEAP32[$p>>2]|0;
 $12 = HEAP32[$1>>2]|0;
 $13 = (_sqlite3BitvecTest($11,$12)|0);
 $14 = ($13|0)!=(0);
 $16 = $14;
 $15 = $16&1;
 STACKTOP = sp;return ($15|0);
}
function _sqlite3PagerIsMemdb($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 13|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 STACKTOP = sp;return ($4|0);
}
function _execSql($db,$pzErrMsg,$zSql) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 $zSql = $zSql|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pStmt = 0, $rc = 0, $zSubSql = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $pStmt = sp + 8|0;
 $rc = sp + 4|0;
 $zSubSql = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pzErrMsg;
 HEAP32[$3>>2] = $zSql;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_sqlite3_prepare_v2($4,$5,-1,$pStmt,0)|0);
 HEAP32[$rc>>2] = $6;
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)!=(0);
 if ($8) {
  $9 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $9;
  $33 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 while(1) {
  $10 = HEAP32[$pStmt>>2]|0;
  $11 = (_sqlite3_step($10)|0);
  HEAP32[$rc>>2] = $11;
  $12 = (100)==($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$pStmt>>2]|0;
  $14 = (_sqlite3_column_text($13,0)|0);
  HEAP32[$zSubSql>>2] = $14;
  $15 = HEAP32[$zSubSql>>2]|0;
  $16 = ($15|0)!=(0|0);
  if (!($16)) {
   continue;
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$zSubSql>>2]|0;
  $20 = (_execSql($17,$18,$19)|0);
  HEAP32[$rc>>2] = $20;
  $21 = HEAP32[$rc>>2]|0;
  $22 = ($21|0)!=(0);
  if ($22) {
   break;
  }
 }
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)==(101);
 if ($24) {
  HEAP32[$rc>>2] = 0;
 }
 $25 = HEAP32[$rc>>2]|0;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  $30 = (_sqlite3_errmsg($29)|0);
  _sqlite3SetString($27,$28,$30);
 }
 $31 = HEAP32[$pStmt>>2]|0;
 (_sqlite3_finalize($31)|0);
 $32 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $32;
 $33 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _sqlite3BtreeGetOptimalReserve($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = (_sqlite3BtreeGetReserveNoMutex($2)|0);
 HEAP32[$n>>2] = $3;
 $4 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _execSqlF($db,$pzErrMsg,$zSql,$varargs) {
 $db = $db|0;
 $pzErrMsg = $pzErrMsg|0;
 $zSql = $zSql|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $rc = 0, $z = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $z = sp + 24|0;
 $ap = sp + 8|0;
 $rc = sp;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $pzErrMsg;
 HEAP32[$3>>2] = $zSql;
 HEAP32[$ap>>2] = $varargs;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_sqlite3VMPrintf($4,$5,$ap)|0);
 HEAP32[$z>>2] = $6;
 $7 = HEAP32[$z>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 7;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 } else {
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$z>>2]|0;
  $12 = (_execSql($9,$10,$11)|0);
  HEAP32[$rc>>2] = $12;
  $13 = HEAP32[$1>>2]|0;
  $14 = HEAP32[$z>>2]|0;
  _sqlite3DbFree($13,$14);
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _sqlite3BtreeCopyFile($pTo,$pFrom) {
 $pTo = $pTo|0;
 $pFrom = $pFrom|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $b = 0, $nByte = 0, $pFd = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $rc = sp + 60|0;
 $pFd = sp + 56|0;
 $b = sp + 8|0;
 $nByte = sp;
 HEAP32[$0>>2] = $pTo;
 HEAP32[$1>>2] = $pFrom;
 $2 = HEAP32[$0>>2]|0;
 _sqlite3BtreeEnter($2);
 $3 = HEAP32[$1>>2]|0;
 _sqlite3BtreeEnter($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = (_sqlite3BtreePager($4)|0);
 $6 = (_sqlite3PagerFile($5)|0);
 HEAP32[$pFd>>2] = $6;
 $7 = HEAP32[$pFd>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)!=(0|0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (_sqlite3BtreeGetPageSize($10)|0);
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = HEAP32[$1>>2]|0;
  $15 = (_sqlite3BtreeLastPage($14)|0);
  $16 = (___muldi3(($11|0),($13|0),($15|0),0)|0);
  $17 = tempRet0;
  $18 = $nByte;
  $19 = $18;
  HEAP32[$19>>2] = $16;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = $17;
  $22 = HEAP32[$pFd>>2]|0;
  $23 = (_sqlite3OsFileControl($22,11,$nByte)|0);
  HEAP32[$rc>>2] = $23;
  $24 = HEAP32[$rc>>2]|0;
  $25 = ($24|0)==(12);
  $$ = $25 ? 0 : $23;
  HEAP32[$rc>>2] = $$;
  $26 = HEAP32[$rc>>2]|0;
  $27 = ($26|0)!=(0);
  if ($27) {
   $50 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($50|0);
  }
 }
 dest=$b; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $28 = HEAP32[$1>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ((($b)) + 20|0);
 HEAP32[$30>>2] = $29;
 $31 = HEAP32[$1>>2]|0;
 $32 = ((($b)) + 24|0);
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($b)) + 4|0);
 HEAP32[$34>>2] = $33;
 $35 = ((($b)) + 16|0);
 HEAP32[$35>>2] = 1;
 (_sqlite3_backup_step($b,2147483647)|0);
 $36 = (_sqlite3_backup_finish($b)|0);
 HEAP32[$rc>>2] = $36;
 $37 = HEAP32[$rc>>2]|0;
 $38 = ($37|0)==(0);
 if ($38) {
  $39 = HEAP32[$0>>2]|0;
  $40 = ((($39)) + 4|0);
  $41 = HEAP32[$40>>2]|0;
  $42 = ((($41)) + 22|0);
  $43 = HEAP16[$42>>1]|0;
  $44 = $43&65535;
  $45 = $44 & -3;
  $46 = $45&65535;
  HEAP16[$42>>1] = $46;
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 } else {
  $47 = ((($b)) + 4|0);
  $48 = HEAP32[$47>>2]|0;
  $49 = (_sqlite3BtreePager($48)|0);
  _sqlite3PagerClearCache($49);
  $50 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($50|0);
 }
 return (0)|0;
}
function _sqlite3PagerClearCache($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 13|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 _pager_reset($6);
 STACKTOP = sp;return;
}
function _sqlite3BtreeGetReserveNoMutex($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ((($8)) + 36|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($5) - ($10))|0;
 HEAP32[$n>>2] = $11;
 $12 = HEAP32[$n>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _sqlite3PagerSharedLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bExists = 0, $bHotJournal = 0, $dbFileVers = 0, $f = 0, $fout = 0, $nPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0;
 var $pVfs = 0, $rc = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $rc = sp + 24|0;
 $bHotJournal = sp + 20|0;
 $pVfs = sp + 16|0;
 $bExists = sp + 12|0;
 $fout = sp + 8|0;
 $f = sp + 4|0;
 $nPage = sp;
 $dbFileVers = sp + 32|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3PagerUseWal($1)|0);
 $3 = ($2|0)!=(0);
 do {
  if ($3) {
   label = 32;
  } else {
   $4 = HEAP32[$0>>2]|0;
   $5 = ((($4)) + 17|0);
   $6 = HEAP8[$5>>0]|0;
   $7 = $6&255;
   $8 = ($7|0)==(0);
   if ($8) {
    HEAP32[$bHotJournal>>2] = 1;
    $9 = HEAP32[$0>>2]|0;
    $10 = (_pager_wait_on_lock($9,1)|0);
    HEAP32[$rc>>2] = $10;
    $11 = HEAP32[$rc>>2]|0;
    $12 = ($11|0)!=(0);
    if (!($12)) {
     $13 = HEAP32[$0>>2]|0;
     $14 = ((($13)) + 18|0);
     $15 = HEAP8[$14>>0]|0;
     $16 = $15&255;
     $17 = ($16|0)<=(1);
     if ($17) {
      $18 = HEAP32[$0>>2]|0;
      $19 = (_hasHotJournal($18,$bHotJournal)|0);
      HEAP32[$rc>>2] = $19;
     }
     $20 = HEAP32[$rc>>2]|0;
     $21 = ($20|0)!=(0);
     if (!($21)) {
      $22 = HEAP32[$bHotJournal>>2]|0;
      $23 = ($22|0)!=(0);
      if ($23) {
       $24 = HEAP32[$0>>2]|0;
       $25 = ((($24)) + 15|0);
       $26 = HEAP8[$25>>0]|0;
       $27 = ($26<<24>>24)!=(0);
       if ($27) {
        HEAP32[$rc>>2] = 776;
        break;
       }
       $28 = HEAP32[$0>>2]|0;
       $29 = (_pagerLockDb($28,4)|0);
       HEAP32[$rc>>2] = $29;
       $30 = HEAP32[$rc>>2]|0;
       $31 = ($30|0)!=(0);
       if ($31) {
        break;
       }
       $32 = HEAP32[$0>>2]|0;
       $33 = ((($32)) + 68|0);
       $34 = HEAP32[$33>>2]|0;
       $35 = HEAP32[$34>>2]|0;
       $36 = ($35|0)!=(0|0);
       if (!($36)) {
        $37 = HEAP32[$0>>2]|0;
        $38 = HEAP32[$37>>2]|0;
        HEAP32[$pVfs>>2] = $38;
        $39 = HEAP32[$pVfs>>2]|0;
        $40 = HEAP32[$0>>2]|0;
        $41 = ((($40)) + 180|0);
        $42 = HEAP32[$41>>2]|0;
        $43 = (_sqlite3OsAccess($39,$42,0,$bExists)|0);
        HEAP32[$rc>>2] = $43;
        $44 = HEAP32[$rc>>2]|0;
        $45 = ($44|0)==(0);
        $46 = HEAP32[$bExists>>2]|0;
        $47 = ($46|0)!=(0);
        $or$cond = $45 & $47;
        if ($or$cond) {
         HEAP32[$fout>>2] = 0;
         HEAP32[$f>>2] = 2050;
         $48 = HEAP32[$pVfs>>2]|0;
         $49 = HEAP32[$0>>2]|0;
         $50 = ((($49)) + 180|0);
         $51 = HEAP32[$50>>2]|0;
         $52 = HEAP32[$0>>2]|0;
         $53 = ((($52)) + 68|0);
         $54 = HEAP32[$53>>2]|0;
         $55 = HEAP32[$f>>2]|0;
         $56 = (_sqlite3OsOpen($48,$51,$54,$55,$fout)|0);
         HEAP32[$rc>>2] = $56;
         $57 = HEAP32[$rc>>2]|0;
         $58 = ($57|0)==(0);
         if ($58) {
          $59 = HEAP32[$fout>>2]|0;
          $60 = $59 & 1;
          $61 = ($60|0)!=(0);
          if ($61) {
           $62 = (_sqlite3CantopenError(51483)|0);
           HEAP32[$rc>>2] = $62;
           $63 = HEAP32[$0>>2]|0;
           $64 = ((($63)) + 68|0);
           $65 = HEAP32[$64>>2]|0;
           _sqlite3OsClose($65);
          }
         }
        }
       }
       $66 = HEAP32[$0>>2]|0;
       $67 = ((($66)) + 68|0);
       $68 = HEAP32[$67>>2]|0;
       $69 = HEAP32[$68>>2]|0;
       $70 = ($69|0)!=(0|0);
       $71 = HEAP32[$0>>2]|0;
       if ($70) {
        $72 = (_pagerSyncHotJournal($71)|0);
        HEAP32[$rc>>2] = $72;
        $73 = HEAP32[$rc>>2]|0;
        $74 = ($73|0)==(0);
        if ($74) {
         $75 = HEAP32[$0>>2]|0;
         $76 = HEAP32[$0>>2]|0;
         $77 = ((($76)) + 13|0);
         $78 = HEAP8[$77>>0]|0;
         $79 = ($78<<24>>24)!=(0);
         $80 = $79 ^ 1;
         $81 = $80&1;
         $82 = (_pager_playback($75,$81)|0);
         HEAP32[$rc>>2] = $82;
         $83 = HEAP32[$0>>2]|0;
         $84 = ((($83)) + 17|0);
         HEAP8[$84>>0] = 0;
        }
       } else {
        $85 = ((($71)) + 4|0);
        $86 = HEAP8[$85>>0]|0;
        $87 = ($86<<24>>24)!=(0);
        if (!($87)) {
         $88 = HEAP32[$0>>2]|0;
         (_pagerUnlockDb($88,1)|0);
        }
       }
       $89 = HEAP32[$rc>>2]|0;
       $90 = ($89|0)!=(0);
       if ($90) {
        $91 = HEAP32[$0>>2]|0;
        $92 = HEAP32[$rc>>2]|0;
        (_pager_error($91,$92)|0);
        break;
       }
      }
      $93 = HEAP32[$0>>2]|0;
      $94 = ((($93)) + 13|0);
      $95 = HEAP8[$94>>0]|0;
      $96 = ($95<<24>>24)!=(0);
      if (!($96)) {
       $97 = HEAP32[$0>>2]|0;
       $98 = ((($97)) + 24|0);
       $99 = HEAP8[$98>>0]|0;
       $100 = $99&255;
       $101 = ($100|0)!=(0);
       if ($101) {
        HEAP32[$nPage>>2] = 0;
        $102 = HEAP32[$0>>2]|0;
        $103 = (_pagerPagecount($102,$nPage)|0);
        HEAP32[$rc>>2] = $103;
        $104 = HEAP32[$rc>>2]|0;
        $105 = ($104|0)!=(0);
        if ($105) {
         break;
        }
        $106 = HEAP32[$nPage>>2]|0;
        $107 = ($106>>>0)>(0);
        if ($107) {
         $108 = HEAP32[$0>>2]|0;
         $109 = ((($108)) + 64|0);
         $110 = HEAP32[$109>>2]|0;
         $111 = (_sqlite3OsRead($110,$dbFileVers,16,24,0)|0);
         HEAP32[$rc>>2] = $111;
         $112 = HEAP32[$rc>>2]|0;
         $113 = ($112|0)!=(0);
         $114 = HEAP32[$rc>>2]|0;
         $115 = ($114|0)!=(522);
         $or$cond3 = $113 & $115;
         if ($or$cond3) {
          break;
         }
        } else {
         dest=$dbFileVers; stop=dest+16|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
        }
        $116 = HEAP32[$0>>2]|0;
        $117 = ((($116)) + 112|0);
        $118 = (_memcmp($117,$dbFileVers,16)|0);
        $119 = ($118|0)!=(0);
        if ($119) {
         $120 = HEAP32[$0>>2]|0;
         _pager_reset($120);
        }
       }
      }
      $121 = HEAP32[$0>>2]|0;
      $122 = (_pagerOpenWalIfPresent($121)|0);
      HEAP32[$rc>>2] = $122;
      label = 32;
     }
    }
   } else {
    label = 32;
   }
  }
 } while(0);
 if ((label|0) == 32) {
  $123 = HEAP32[$0>>2]|0;
  $124 = (_sqlite3PagerUseWal($123)|0);
  $125 = ($124|0)!=(0);
  if ($125) {
   $126 = HEAP32[$0>>2]|0;
   $127 = (_pagerBeginReadTransaction($126)|0);
   HEAP32[$rc>>2] = $127;
  }
  $128 = HEAP32[$0>>2]|0;
  $129 = ((($128)) + 13|0);
  $130 = HEAP8[$129>>0]|0;
  $131 = $130&255;
  $132 = ($131|0)==(0);
  if ($132) {
   $133 = HEAP32[$0>>2]|0;
   $134 = ((($133)) + 17|0);
   $135 = HEAP8[$134>>0]|0;
   $136 = $135&255;
   $137 = ($136|0)==(0);
   $138 = HEAP32[$rc>>2]|0;
   $139 = ($138|0)==(0);
   $or$cond5 = $137 & $139;
   if ($or$cond5) {
    $140 = HEAP32[$0>>2]|0;
    $141 = HEAP32[$0>>2]|0;
    $142 = ((($141)) + 28|0);
    $143 = (_pagerPagecount($140,$142)|0);
    HEAP32[$rc>>2] = $143;
   }
  }
 }
 $144 = HEAP32[$rc>>2]|0;
 $145 = ($144|0)!=(0);
 $146 = HEAP32[$0>>2]|0;
 if ($145) {
  _pager_unlock($146);
  $150 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($150|0);
 } else {
  $147 = ((($146)) + 17|0);
  HEAP8[$147>>0] = 1;
  $148 = HEAP32[$0>>2]|0;
  $149 = ((($148)) + 24|0);
  HEAP8[$149>>0] = 1;
  $150 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($150|0);
 }
 return (0)|0;
}
function _hasHotJournal($pPager,$pExists) {
 $pPager = $pPager|0;
 $pExists = $pExists|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $exists = 0, $f = 0, $first = 0, $jrnlOpen = 0, $locked = 0, $nPage = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $pVfs = sp + 24|0;
 $rc = sp + 20|0;
 $exists = sp + 16|0;
 $jrnlOpen = sp + 12|0;
 $locked = sp + 8|0;
 $nPage = sp + 4|0;
 $f = sp;
 $first = sp + 36|0;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $pExists;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 HEAP32[$rc>>2] = 0;
 HEAP32[$exists>>2] = 1;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 68|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = $8 ^ 1;
 $10 = $9 ^ 1;
 $11 = $10&1;
 HEAP32[$jrnlOpen>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 HEAP32[$12>>2] = 0;
 $13 = HEAP32[$jrnlOpen>>2]|0;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $15 = HEAP32[$pVfs>>2]|0;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 180|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3OsAccess($15,$18,0,$exists)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 $22 = HEAP32[$exists>>2]|0;
 $23 = ($22|0)!=(0);
 $or$cond = $21 & $23;
 if (!($or$cond)) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 HEAP32[$locked>>2] = 0;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 64|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3OsCheckReservedLock($26,$locked)|0);
 HEAP32[$rc>>2] = $27;
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$locked>>2]|0;
 $31 = ($30|0)!=(0);
 $or$cond3 = $29 | $31;
 if ($or$cond3) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = (_pagerPagecount($32,$nPage)|0);
 HEAP32[$rc>>2] = $33;
 $34 = HEAP32[$rc>>2]|0;
 $35 = ($34|0)==(0);
 if (!($35)) {
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $36 = HEAP32[$nPage>>2]|0;
 $37 = ($36|0)!=(0);
 $38 = HEAP32[$jrnlOpen>>2]|0;
 $39 = ($38|0)!=(0);
 $or$cond5 = $37 | $39;
 if (!($or$cond5)) {
  _sqlite3BeginBenignMalloc();
  $40 = HEAP32[$0>>2]|0;
  $41 = (_pagerLockDb($40,2)|0);
  $42 = ($41|0)==(0);
  if ($42) {
   $43 = HEAP32[$pVfs>>2]|0;
   $44 = HEAP32[$0>>2]|0;
   $45 = ((($44)) + 180|0);
   $46 = HEAP32[$45>>2]|0;
   (_sqlite3OsDelete($43,$46,0)|0);
   $47 = HEAP32[$0>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP8[$48>>0]|0;
   $50 = ($49<<24>>24)!=(0);
   if (!($50)) {
    $51 = HEAP32[$0>>2]|0;
    (_pagerUnlockDb($51,1)|0);
   }
  }
  _sqlite3EndBenignMalloc();
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 $52 = HEAP32[$jrnlOpen>>2]|0;
 $53 = ($52|0)!=(0);
 if (!($53)) {
  HEAP32[$f>>2] = 2049;
  $54 = HEAP32[$pVfs>>2]|0;
  $55 = HEAP32[$0>>2]|0;
  $56 = ((($55)) + 180|0);
  $57 = HEAP32[$56>>2]|0;
  $58 = HEAP32[$0>>2]|0;
  $59 = ((($58)) + 68|0);
  $60 = HEAP32[$59>>2]|0;
  $61 = HEAP32[$f>>2]|0;
  $62 = (_sqlite3OsOpen($54,$57,$60,$61,$f)|0);
  HEAP32[$rc>>2] = $62;
 }
 $63 = HEAP32[$rc>>2]|0;
 $64 = ($63|0)==(0);
 if ($64) {
  HEAP8[$first>>0] = 0;
  $65 = HEAP32[$0>>2]|0;
  $66 = ((($65)) + 68|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = (_sqlite3OsRead($67,$first,1,0,0)|0);
  HEAP32[$rc>>2] = $68;
  $69 = HEAP32[$rc>>2]|0;
  $70 = ($69|0)==(522);
  $$ = $70 ? 0 : $68;
  HEAP32[$rc>>2] = $$;
  $71 = HEAP32[$jrnlOpen>>2]|0;
  $72 = ($71|0)!=(0);
  if (!($72)) {
   $73 = HEAP32[$0>>2]|0;
   $74 = ((($73)) + 68|0);
   $75 = HEAP32[$74>>2]|0;
   _sqlite3OsClose($75);
  }
  $76 = HEAP8[$first>>0]|0;
  $77 = $76&255;
  $78 = ($77|0)!=(0);
  $79 = $78&1;
  $80 = HEAP32[$1>>2]|0;
  HEAP32[$80>>2] = $79;
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 } else {
  $81 = HEAP32[$rc>>2]|0;
  $82 = ($81|0)==(14);
  if (!($82)) {
   $84 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($84|0);
  }
  $83 = HEAP32[$1>>2]|0;
  HEAP32[$83>>2] = 1;
  HEAP32[$rc>>2] = 0;
  $84 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($84|0);
 }
 return (0)|0;
}
function _pagerPagecount($pPager,$pnPage) {
 $pPager = $pPager|0;
 $pnPage = $pnPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $nPage = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $nPage = sp + 12|0;
 $n = sp;
 $rc = sp + 8|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pnPage;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 216|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3WalDbsize($5)|0);
 HEAP32[$nPage>>2] = $6;
 $7 = HEAP32[$nPage>>2]|0;
 $8 = ($7|0)==(0);
 do {
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = ((($9)) + 64|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$11>>2]|0;
   $13 = ($12|0)!=(0|0);
   if ($13) {
    $14 = $n;
    $15 = $14;
    HEAP32[$15>>2] = 0;
    $16 = (($14) + 4)|0;
    $17 = $16;
    HEAP32[$17>>2] = 0;
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 64|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (_sqlite3OsFileSize($20,$n)|0);
    HEAP32[$rc>>2] = $21;
    $22 = HEAP32[$rc>>2]|0;
    $23 = ($22|0)!=(0);
    if (!($23)) {
     $25 = $n;
     $26 = $25;
     $27 = HEAP32[$26>>2]|0;
     $28 = (($25) + 4)|0;
     $29 = $28;
     $30 = HEAP32[$29>>2]|0;
     $31 = HEAP32[$1>>2]|0;
     $32 = ((($31)) + 160|0);
     $33 = HEAP32[$32>>2]|0;
     $34 = ($33|0)<(0);
     $35 = $34 << 31 >> 31;
     $36 = (_i64Add(($27|0),($30|0),($33|0),($35|0))|0);
     $37 = tempRet0;
     $38 = (_i64Subtract(($36|0),($37|0),1,0)|0);
     $39 = tempRet0;
     $40 = HEAP32[$1>>2]|0;
     $41 = ((($40)) + 160|0);
     $42 = HEAP32[$41>>2]|0;
     $43 = ($42|0)<(0);
     $44 = $43 << 31 >> 31;
     $45 = (___divdi3(($38|0),($39|0),($42|0),($44|0))|0);
     $46 = tempRet0;
     HEAP32[$nPage>>2] = $45;
     break;
    }
    $24 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $24;
    $57 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($57|0);
   }
  }
 } while(0);
 $47 = HEAP32[$nPage>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 164|0);
 $50 = HEAP32[$49>>2]|0;
 $51 = ($47>>>0)>($50>>>0);
 if ($51) {
  $52 = HEAP32[$nPage>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = ((($53)) + 164|0);
  HEAP32[$54>>2] = $52;
 }
 $55 = HEAP32[$nPage>>2]|0;
 $56 = HEAP32[$2>>2]|0;
 HEAP32[$56>>2] = $55;
 HEAP32[$0>>2] = 0;
 $57 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($57|0);
}
function _pagerOpenWalIfPresent($pPager) {
 $pPager = $pPager|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $isWal = 0, $nPage = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $isWal = sp + 4|0;
 $nPage = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 13|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 do {
  if (!($5)) {
   $6 = HEAP32[$1>>2]|0;
   $7 = (_pagerPagecount($6,$nPage)|0);
   HEAP32[$rc>>2] = $7;
   $8 = HEAP32[$rc>>2]|0;
   $9 = ($8|0)!=(0);
   if ($9) {
    $10 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $10;
    $35 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($35|0);
   }
   $11 = HEAP32[$nPage>>2]|0;
   $12 = ($11|0)==(0);
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$1>>2]|0;
   $16 = ((($15)) + 220|0);
   $17 = HEAP32[$16>>2]|0;
   if ($12) {
    $18 = (_sqlite3OsDelete($14,$17,0)|0);
    HEAP32[$rc>>2] = $18;
    $19 = HEAP32[$rc>>2]|0;
    $20 = ($19|0)==(5898);
    $$ = $20 ? 0 : $18;
    HEAP32[$rc>>2] = $$;
    HEAP32[$isWal>>2] = 0;
   } else {
    $21 = (_sqlite3OsAccess($14,$17,0,$isWal)|0);
    HEAP32[$rc>>2] = $21;
   }
   $22 = HEAP32[$rc>>2]|0;
   $23 = ($22|0)==(0);
   if ($23) {
    $24 = HEAP32[$isWal>>2]|0;
    $25 = ($24|0)!=(0);
    $26 = HEAP32[$1>>2]|0;
    if ($25) {
     $27 = (_sqlite3PagerOpenWal($26,0)|0);
     HEAP32[$rc>>2] = $27;
     break;
    }
    $28 = ((($26)) + 5|0);
    $29 = HEAP8[$28>>0]|0;
    $30 = $29&255;
    $31 = ($30|0)==(5);
    if ($31) {
     $32 = HEAP32[$1>>2]|0;
     $33 = ((($32)) + 5|0);
     HEAP8[$33>>0] = 0;
    }
   }
  }
 } while(0);
 $34 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $34;
 $35 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($35|0);
}
function _pagerBeginReadTransaction($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $changed = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $changed = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$changed>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 216|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3WalEndReadTransaction($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 216|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3WalBeginReadTransaction($6,$changed)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 $10 = HEAP32[$changed>>2]|0;
 $11 = ($10|0)!=(0);
 $or$cond = $9 | $11;
 if (!($or$cond)) {
  $13 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 $12 = HEAP32[$0>>2]|0;
 _pager_reset($12);
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3WalBeginReadTransaction($pWal,$pChanged) {
 $pWal = $pWal|0;
 $pChanged = $pChanged|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $cnt = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $pChanged;
 HEAP32[$cnt>>2] = 0;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = HEAP32[$cnt>>2]|0;
  $5 = (($4) + 1)|0;
  HEAP32[$cnt>>2] = $5;
  $6 = (_walTryBeginRead($2,$3,0,$5)|0);
  HEAP32[$rc>>2] = $6;
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(-1);
  if (!($8)) {
   break;
  }
 }
 $9 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($9|0);
}
function _sqlite3PagerOpenWal($pPager,$pbOpen) {
 $pPager = $pPager|0;
 $pbOpen = $pbOpen|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pbOpen;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 13|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = ($5<<24>>24)!=(0);
 do {
  if ($6) {
   label = 7;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 216|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = ($9|0)!=(0|0);
   if ($10) {
    label = 7;
   } else {
    $11 = HEAP32[$1>>2]|0;
    $12 = (_sqlite3PagerWalSupported($11)|0);
    $13 = ($12|0)!=(0);
    if ($13) {
     $14 = HEAP32[$1>>2]|0;
     $15 = ((($14)) + 68|0);
     $16 = HEAP32[$15>>2]|0;
     _sqlite3OsClose($16);
     $17 = HEAP32[$1>>2]|0;
     $18 = (_pagerOpenWal($17)|0);
     HEAP32[$rc>>2] = $18;
     $19 = HEAP32[$rc>>2]|0;
     $20 = ($19|0)==(0);
     if (!($20)) {
      break;
     }
     $21 = HEAP32[$1>>2]|0;
     $22 = ((($21)) + 5|0);
     HEAP8[$22>>0] = 5;
     $23 = HEAP32[$1>>2]|0;
     $24 = ((($23)) + 17|0);
     HEAP8[$24>>0] = 0;
     break;
    } else {
     HEAP32[$0>>2] = 14;
     $27 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($27|0);
    }
   }
  }
 } while(0);
 if ((label|0) == 7) {
  $25 = HEAP32[$2>>2]|0;
  HEAP32[$25>>2] = 1;
 }
 $26 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $26;
 $27 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($27|0);
}
function _pagerOpenWal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = (_pagerExclusiveLock($5)|0);
  HEAP32[$rc>>2] = $6;
 }
 $7 = HEAP32[$rc>>2]|0;
 $8 = ($7|0)==(0);
 if (!($8)) {
  $32 = HEAP32[$0>>2]|0;
  _pagerFixMaplimit($32);
  $33 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 64|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 220|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 168|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 216|0);
 $31 = (_sqlite3WalOpen($10,$13,$16,$20,$25,$28,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$0>>2]|0;
 _pagerFixMaplimit($32);
 $33 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _pagerExclusiveLock($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_pagerLockDb($1,4)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$rc>>2]|0;
 $4 = ($3|0)!=(0);
 if (!($4)) {
  $6 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($6|0);
 }
 $5 = HEAP32[$0>>2]|0;
 (_pagerUnlockDb($5,1)|0);
 $6 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($6|0);
}
function _sqlite3WalOpen($pVfs,$pDbFd,$zWalName,$bNoShm,$0,$1,$ppWal) {
 $pVfs = $pVfs|0;
 $pDbFd = $pDbFd|0;
 $zWalName = $zWalName|0;
 $bNoShm = $bNoShm|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppWal = $ppWal|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $flags = 0, $iDC = 0, $pRet = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $6 = sp + 28|0;
 $7 = sp;
 $8 = sp + 24|0;
 $rc = sp + 20|0;
 $pRet = sp + 16|0;
 $flags = sp + 12|0;
 $iDC = sp + 8|0;
 HEAP32[$3>>2] = $pVfs;
 HEAP32[$4>>2] = $pDbFd;
 HEAP32[$5>>2] = $zWalName;
 HEAP32[$6>>2] = $bNoShm;
 $9 = $7;
 $10 = $9;
 HEAP32[$10>>2] = $0;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $1;
 HEAP32[$8>>2] = $ppWal;
 $13 = HEAP32[$8>>2]|0;
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$3>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = (120 + ($16))|0;
 $18 = (_sqlite3MallocZero($17,0)|0);
 HEAP32[$pRet>>2] = $18;
 $19 = HEAP32[$pRet>>2]|0;
 $20 = ($19|0)!=(0|0);
 if (!($20)) {
  HEAP32[$2>>2] = 7;
  $93 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($93|0);
 }
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$pRet>>2]|0;
 HEAP32[$22>>2] = $21;
 $23 = HEAP32[$pRet>>2]|0;
 $24 = ((($23)) + 120|0);
 $25 = HEAP32[$pRet>>2]|0;
 $26 = ((($25)) + 8|0);
 HEAP32[$26>>2] = $24;
 $27 = HEAP32[$4>>2]|0;
 $28 = HEAP32[$pRet>>2]|0;
 $29 = ((($28)) + 4|0);
 HEAP32[$29>>2] = $27;
 $30 = HEAP32[$pRet>>2]|0;
 $31 = ((($30)) + 40|0);
 HEAP16[$31>>1] = -1;
 $32 = $7;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$pRet>>2]|0;
 $39 = ((($38)) + 16|0);
 $40 = $39;
 $41 = $40;
 HEAP32[$41>>2] = $34;
 $42 = (($40) + 4)|0;
 $43 = $42;
 HEAP32[$43>>2] = $37;
 $44 = HEAP32[$5>>2]|0;
 $45 = HEAP32[$pRet>>2]|0;
 $46 = ((($45)) + 108|0);
 HEAP32[$46>>2] = $44;
 $47 = HEAP32[$pRet>>2]|0;
 $48 = ((($47)) + 48|0);
 HEAP8[$48>>0] = 1;
 $49 = HEAP32[$pRet>>2]|0;
 $50 = ((($49)) + 49|0);
 HEAP8[$50>>0] = 1;
 $51 = HEAP32[$6>>2]|0;
 $52 = ($51|0)!=(0);
 $53 = $52 ? 2 : 0;
 $54 = $53&255;
 $55 = HEAP32[$pRet>>2]|0;
 $56 = ((($55)) + 43|0);
 HEAP8[$56>>0] = $54;
 HEAP32[$flags>>2] = 524294;
 $57 = HEAP32[$3>>2]|0;
 $58 = HEAP32[$5>>2]|0;
 $59 = HEAP32[$pRet>>2]|0;
 $60 = ((($59)) + 8|0);
 $61 = HEAP32[$60>>2]|0;
 $62 = HEAP32[$flags>>2]|0;
 $63 = (_sqlite3OsOpen($57,$58,$61,$62,$flags)|0);
 HEAP32[$rc>>2] = $63;
 $64 = HEAP32[$rc>>2]|0;
 $65 = ($64|0)==(0);
 if ($65) {
  $66 = HEAP32[$flags>>2]|0;
  $67 = $66 & 1;
  $68 = ($67|0)!=(0);
  if ($68) {
   $69 = HEAP32[$pRet>>2]|0;
   $70 = ((($69)) + 46|0);
   HEAP8[$70>>0] = 1;
  }
 }
 $71 = HEAP32[$rc>>2]|0;
 $72 = ($71|0)!=(0);
 if ($72) {
  $73 = HEAP32[$pRet>>2]|0;
  _walIndexClose($73,0);
  $74 = HEAP32[$pRet>>2]|0;
  $75 = ((($74)) + 8|0);
  $76 = HEAP32[$75>>2]|0;
  _sqlite3OsClose($76);
  $77 = HEAP32[$pRet>>2]|0;
  _sqlite3_free($77);
 } else {
  $78 = HEAP32[$4>>2]|0;
  $79 = (_sqlite3OsDeviceCharacteristics($78)|0);
  HEAP32[$iDC>>2] = $79;
  $80 = HEAP32[$iDC>>2]|0;
  $81 = $80 & 1024;
  $82 = ($81|0)!=(0);
  if ($82) {
   $83 = HEAP32[$pRet>>2]|0;
   $84 = ((($83)) + 48|0);
   HEAP8[$84>>0] = 0;
  }
  $85 = HEAP32[$iDC>>2]|0;
  $86 = $85 & 4096;
  $87 = ($86|0)!=(0);
  if ($87) {
   $88 = HEAP32[$pRet>>2]|0;
   $89 = ((($88)) + 49|0);
   HEAP8[$89>>0] = 0;
  }
  $90 = HEAP32[$pRet>>2]|0;
  $91 = HEAP32[$8>>2]|0;
  HEAP32[$91>>2] = $90;
 }
 $92 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $92;
 $93 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($93|0);
}
function _sqlite3WalDbsize($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pWal;
 $2 = HEAP32[$1>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$1>>2]|0;
  $5 = ((($4)) + 40|0);
  $6 = HEAP16[$5>>1]|0;
  $7 = $6 << 16 >> 16;
  $8 = ($7|0)>=(0);
  if ($8) {
   $9 = HEAP32[$1>>2]|0;
   $10 = ((($9)) + 52|0);
   $11 = ((($10)) + 20|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$0>>2] = $12;
   $13 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($13|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $13 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3OsCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$5 & 255]($6,$7)|0);
 STACKTOP = sp;return ($8|0);
}
function _rowSetEntrySort($pIn) {
 $pIn = $pIn|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aBucket = 0, $i = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 168|0;
 $i = sp + 164|0;
 $pNext = sp + 160|0;
 $aBucket = sp;
 HEAP32[$0>>2] = $pIn;
 _memset(($aBucket|0),0,160)|0;
 while(1) {
  $1 = HEAP32[$0>>2]|0;
  $2 = ($1|0)!=(0|0);
  if (!($2)) {
   break;
  }
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 8|0);
  $5 = HEAP32[$4>>2]|0;
  HEAP32[$pNext>>2] = $5;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 8|0);
  HEAP32[$7>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $8 = HEAP32[$i>>2]|0;
   $9 = (($aBucket) + ($8<<2)|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if (!($11)) {
    break;
   }
   $12 = HEAP32[$i>>2]|0;
   $13 = (($aBucket) + ($12<<2)|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = (_rowSetEntryMerge($14,$15)|0);
   HEAP32[$0>>2] = $16;
   $17 = HEAP32[$i>>2]|0;
   $18 = (($aBucket) + ($17<<2)|0);
   HEAP32[$18>>2] = 0;
   $19 = HEAP32[$i>>2]|0;
   $20 = (($19) + 1)|0;
   HEAP32[$i>>2] = $20;
  }
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$i>>2]|0;
  $23 = (($aBucket) + ($22<<2)|0);
  HEAP32[$23>>2] = $21;
  $24 = HEAP32[$pNext>>2]|0;
  HEAP32[$0>>2] = $24;
 }
 $25 = HEAP32[$aBucket>>2]|0;
 HEAP32[$0>>2] = $25;
 HEAP32[$i>>2] = 1;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = ($26>>>0)<(40);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = (($aBucket) + ($28<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)==(0|0);
  if (!($31)) {
   $32 = HEAP32[$0>>2]|0;
   $33 = ($32|0)!=(0|0);
   if ($33) {
    $34 = HEAP32[$0>>2]|0;
    $35 = HEAP32[$i>>2]|0;
    $36 = (($aBucket) + ($35<<2)|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = (_rowSetEntryMerge($34,$37)|0);
    $42 = $38;
   } else {
    $39 = HEAP32[$i>>2]|0;
    $40 = (($aBucket) + ($39<<2)|0);
    $41 = HEAP32[$40>>2]|0;
    $42 = $41;
   }
   HEAP32[$0>>2] = $42;
  }
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 $45 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _rowSetListToTree($pList) {
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iDepth = 0, $p = 0, $pLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $iDepth = sp + 8|0;
 $p = sp + 4|0;
 $pLeft = sp;
 HEAP32[$0>>2] = $pList;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$0>>2] = $4;
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$p>>2]|0;
 $8 = ((($7)) + 12|0);
 HEAP32[$8>>2] = 0;
 HEAP32[$iDepth>>2] = 1;
 while(1) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ($9|0)!=(0|0);
  $11 = HEAP32[$p>>2]|0;
  if (!($10)) {
   break;
  }
  HEAP32[$pLeft>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  HEAP32[$p>>2] = $12;
  $13 = HEAP32[$p>>2]|0;
  $14 = ((($13)) + 8|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$0>>2] = $15;
  $16 = HEAP32[$pLeft>>2]|0;
  $17 = HEAP32[$p>>2]|0;
  $18 = ((($17)) + 12|0);
  HEAP32[$18>>2] = $16;
  $19 = HEAP32[$iDepth>>2]|0;
  $20 = (_rowSetNDeepTree($0,$19)|0);
  $21 = HEAP32[$p>>2]|0;
  $22 = ((($21)) + 8|0);
  HEAP32[$22>>2] = $20;
  $23 = HEAP32[$iDepth>>2]|0;
  $24 = (($23) + 1)|0;
  HEAP32[$iDepth>>2] = $24;
 }
 STACKTOP = sp;return ($11|0);
}
function _rowSetTreeToList($pIn,$ppFirst,$ppLast) {
 $pIn = $pIn|0;
 $ppFirst = $ppFirst|0;
 $ppLast = $ppLast|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pIn;
 HEAP32[$1>>2] = $ppFirst;
 HEAP32[$2>>2] = $ppLast;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$0>>2]|0;
 if ($6) {
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  _rowSetTreeToList($9,$10,$p);
  $11 = HEAP32[$0>>2]|0;
  $12 = HEAP32[$p>>2]|0;
  $13 = ((($12)) + 8|0);
  HEAP32[$13>>2] = $11;
 } else {
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$14>>2] = $7;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$0>>2]|0;
 if ($18) {
  $20 = ((($19)) + 8|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 8|0);
  $24 = HEAP32[$2>>2]|0;
  _rowSetTreeToList($21,$23,$24);
  STACKTOP = sp;return;
 } else {
  $25 = HEAP32[$2>>2]|0;
  HEAP32[$25>>2] = $19;
  STACKTOP = sp;return;
 }
}
function _rowSetEntryMerge($pA,$pB) {
 $pA = $pA|0;
 $pB = $pB|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $head = 0, $pTail = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $head = sp;
 $pTail = sp + 16|0;
 HEAP32[$0>>2] = $pA;
 HEAP32[$1>>2] = $pB;
 HEAP32[$pTail>>2] = $head;
 while(1) {
  $2 = HEAP32[$0>>2]|0;
  $3 = $2;
  $4 = $3;
  $5 = HEAP32[$4>>2]|0;
  $6 = (($3) + 4)|0;
  $7 = $6;
  $8 = HEAP32[$7>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = $9;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = ($8|0)<($15|0);
  $17 = ($5>>>0)<=($12>>>0);
  $18 = ($8|0)==($15|0);
  $19 = $18 & $17;
  $20 = $16 | $19;
  if (!($20)) {
   $51 = HEAP32[$1>>2]|0;
   $52 = HEAP32[$pTail>>2]|0;
   $53 = ((($52)) + 8|0);
   HEAP32[$53>>2] = $51;
   HEAP32[$pTail>>2] = $51;
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 8|0);
   $56 = HEAP32[$55>>2]|0;
   HEAP32[$1>>2] = $56;
   $57 = HEAP32[$1>>2]|0;
   $58 = ($57|0)==(0|0);
   if ($58) {
    label = 8;
    break;
   } else {
    continue;
   }
  }
  $21 = HEAP32[$0>>2]|0;
  $22 = $21;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = $28;
  $30 = $29;
  $31 = HEAP32[$30>>2]|0;
  $32 = (($29) + 4)|0;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = ($27|0)<($34|0);
  $36 = ($24>>>0)<($31>>>0);
  $37 = ($27|0)==($34|0);
  $38 = $37 & $36;
  $39 = $35 | $38;
  if ($39) {
   $40 = HEAP32[$0>>2]|0;
   $41 = HEAP32[$pTail>>2]|0;
   $42 = ((($41)) + 8|0);
   HEAP32[$42>>2] = $40;
   HEAP32[$pTail>>2] = $40;
  }
  $43 = HEAP32[$0>>2]|0;
  $44 = ((($43)) + 8|0);
  $45 = HEAP32[$44>>2]|0;
  HEAP32[$0>>2] = $45;
  $46 = HEAP32[$0>>2]|0;
  $47 = ($46|0)==(0|0);
  if ($47) {
   label = 6;
   break;
  }
 }
 if ((label|0) == 6) {
  $48 = HEAP32[$1>>2]|0;
  $49 = HEAP32[$pTail>>2]|0;
  $50 = ((($49)) + 8|0);
  HEAP32[$50>>2] = $48;
  $62 = ((($head)) + 8|0);
  $63 = HEAP32[$62>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 else if ((label|0) == 8) {
  $59 = HEAP32[$0>>2]|0;
  $60 = HEAP32[$pTail>>2]|0;
  $61 = ((($60)) + 8|0);
  HEAP32[$61>>2] = $59;
  $62 = ((($head)) + 8|0);
  $63 = HEAP32[$62>>2]|0;
  STACKTOP = sp;return ($63|0);
 }
 return (0)|0;
}
function _rowSetEntryAlloc($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pNew = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 24|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = ($5|0)==(0);
 do {
  if ($6) {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 4|0);
   $9 = HEAP32[$8>>2]|0;
   $10 = (_sqlite3DbMallocRawNN($9,1016,0)|0);
   HEAP32[$pNew>>2] = $10;
   $11 = HEAP32[$pNew>>2]|0;
   $12 = ($11|0)==(0|0);
   if (!($12)) {
    $13 = HEAP32[$1>>2]|0;
    $14 = HEAP32[$13>>2]|0;
    $15 = HEAP32[$pNew>>2]|0;
    HEAP32[$15>>2] = $14;
    $16 = HEAP32[$pNew>>2]|0;
    $17 = HEAP32[$1>>2]|0;
    HEAP32[$17>>2] = $16;
    $18 = HEAP32[$pNew>>2]|0;
    $19 = ((($18)) + 8|0);
    $20 = HEAP32[$1>>2]|0;
    $21 = ((($20)) + 16|0);
    HEAP32[$21>>2] = $19;
    $22 = HEAP32[$1>>2]|0;
    $23 = ((($22)) + 24|0);
    HEAP16[$23>>1] = 63;
    break;
   }
   HEAP32[$0>>2] = 0;
   $32 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($32|0);
  }
 } while(0);
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = (($26) + -1)<<16>>16;
 HEAP16[$25>>1] = $27;
 $28 = HEAP32[$1>>2]|0;
 $29 = ((($28)) + 16|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = ((($30)) + 16|0);
 HEAP32[$29>>2] = $31;
 HEAP32[$0>>2] = $30;
 $32 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($32|0);
}
function _rowSetNDeepTree($ppList,$iDepth) {
 $ppList = $ppList|0;
 $iDepth = $iDepth|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $p = 0, $pLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $p = sp + 4|0;
 $pLeft = sp;
 HEAP32[$1>>2] = $ppList;
 HEAP32[$2>>2] = $iDepth;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $39 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($39|0);
 }
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)>(1);
 $8 = HEAP32[$1>>2]|0;
 do {
  if ($7) {
   $9 = HEAP32[$2>>2]|0;
   $10 = (($9) - 1)|0;
   $11 = (_rowSetNDeepTree($8,$10)|0);
   HEAP32[$pLeft>>2] = $11;
   $12 = HEAP32[$1>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   HEAP32[$p>>2] = $13;
   $14 = HEAP32[$p>>2]|0;
   $15 = ($14|0)==(0|0);
   $16 = HEAP32[$pLeft>>2]|0;
   if (!($15)) {
    $17 = HEAP32[$p>>2]|0;
    $18 = ((($17)) + 12|0);
    HEAP32[$18>>2] = $16;
    $19 = HEAP32[$p>>2]|0;
    $20 = ((($19)) + 8|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = HEAP32[$1>>2]|0;
    HEAP32[$22>>2] = $21;
    $23 = HEAP32[$1>>2]|0;
    $24 = HEAP32[$2>>2]|0;
    $25 = (($24) - 1)|0;
    $26 = (_rowSetNDeepTree($23,$25)|0);
    $27 = HEAP32[$p>>2]|0;
    $28 = ((($27)) + 8|0);
    HEAP32[$28>>2] = $26;
    break;
   }
   HEAP32[$0>>2] = $16;
   $39 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($39|0);
  } else {
   $29 = HEAP32[$8>>2]|0;
   HEAP32[$p>>2] = $29;
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 8|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = HEAP32[$1>>2]|0;
   HEAP32[$33>>2] = $32;
   $34 = HEAP32[$p>>2]|0;
   $35 = ((($34)) + 8|0);
   HEAP32[$35>>2] = 0;
   $36 = HEAP32[$p>>2]|0;
   $37 = ((($36)) + 12|0);
   HEAP32[$37>>2] = 0;
  }
 } while(0);
 $38 = HEAP32[$p>>2]|0;
 HEAP32[$0>>2] = $38;
 $39 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($39|0);
}
function _sqlite3RowSetInit($db,$pSpace,$N) {
 $db = $db|0;
 $pSpace = $pSpace|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pSpace;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $3;
 $4 = HEAP32[$p>>2]|0;
 HEAP32[$4>>2] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$p>>2]|0;
 $7 = ((($6)) + 4|0);
 HEAP32[$7>>2] = $5;
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 8|0);
 HEAP32[$9>>2] = 0;
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 12|0);
 HEAP32[$11>>2] = 0;
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 20|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$p>>2]|0;
 $15 = ((($14)) + 32|0);
 $16 = HEAP32[$p>>2]|0;
 $17 = ((($16)) + 16|0);
 HEAP32[$17>>2] = $15;
 $18 = HEAP32[$2>>2]|0;
 $19 = (($18) - 32)|0;
 $20 = (($19>>>0) / 16)&-1;
 $21 = $20&65535;
 $22 = HEAP32[$p>>2]|0;
 $23 = ((($22)) + 24|0);
 HEAP16[$23>>1] = $21;
 $24 = HEAP32[$p>>2]|0;
 $25 = ((($24)) + 26|0);
 HEAP16[$25>>1] = 1;
 $26 = HEAP32[$p>>2]|0;
 $27 = ((($26)) + 28|0);
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$p>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _setPageReferenced($pCheck,$iPg) {
 $pCheck = $pCheck|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 7;
 $4 = 1 << $3;
 $5 = HEAP32[$1>>2]|0;
 $6 = (($5>>>0) / 8)&-1;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + ($6)|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = $12 | $4;
 $14 = $13&255;
 HEAP8[$10>>0] = $14;
 STACKTOP = sp;return;
}
function _checkList($pCheck,$isFreeList,$iPage,$N) {
 $pCheck = $pCheck|0;
 $isFreeList = $isFreeList|0;
 $iPage = $iPage|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $expected = 0, $i = 0, $iFirst = 0, $iFreePage = 0, $n = 0, $or$cond = 0, $pOvflData = 0, $pOvflPage = 0, $vararg_buffer = 0, $vararg_buffer10 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer10 = sp + 32|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $i = sp + 60|0;
 $expected = sp + 56|0;
 $iFirst = sp + 52|0;
 $pOvflPage = sp + 48|0;
 $pOvflData = sp + 44|0;
 $n = sp + 40|0;
 $iFreePage = sp + 36|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $isFreeList;
 HEAP32[$2>>2] = $iPage;
 HEAP32[$3>>2] = $N;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$expected>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$iFirst>>2] = $5;
 while(1) {
  $6 = HEAP32[$3>>2]|0;
  $7 = (($6) + -1)|0;
  HEAP32[$3>>2] = $7;
  $8 = ($6|0)>(0);
  if (!($8)) {
   label = 25;
   break;
  }
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)!=(0);
  if (!($12)) {
   label = 25;
   break;
  }
  $13 = HEAP32[$2>>2]|0;
  $14 = ($13|0)<(1);
  $15 = HEAP32[$0>>2]|0;
  if ($14) {
   label = 5;
   break;
  }
  $20 = HEAP32[$2>>2]|0;
  $21 = (_checkRef($15,$20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   label = 25;
   break;
  }
  $23 = HEAP32[$0>>2]|0;
  $24 = ((($23)) + 4|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (_sqlite3PagerGet($25,$26,$pOvflPage,0)|0);
  $28 = ($27|0)!=(0);
  if ($28) {
   label = 8;
   break;
  }
  $31 = HEAP32[$pOvflPage>>2]|0;
  $32 = (_sqlite3PagerGetData($31)|0);
  HEAP32[$pOvflData>>2] = $32;
  $33 = HEAP32[$1>>2]|0;
  $34 = ($33|0)!=(0);
  do {
   if ($34) {
    $35 = HEAP32[$pOvflData>>2]|0;
    $36 = ((($35)) + 4|0);
    $37 = (_sqlite3Get4byte($36)|0);
    HEAP32[$n>>2] = $37;
    $38 = HEAP32[$0>>2]|0;
    $39 = HEAP32[$38>>2]|0;
    $40 = ((($39)) + 17|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = ($41<<24>>24)!=(0);
    if ($42) {
     $43 = HEAP32[$0>>2]|0;
     $44 = HEAP32[$2>>2]|0;
     _checkPtrmap($43,$44,2,0);
    }
    $45 = HEAP32[$n>>2]|0;
    $46 = HEAP32[$0>>2]|0;
    $47 = HEAP32[$46>>2]|0;
    $48 = ((($47)) + 36|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = (($49|0) / 4)&-1;
    $51 = (($50) - 2)|0;
    $52 = ($45|0)>($51|0);
    if ($52) {
     $53 = HEAP32[$0>>2]|0;
     $54 = HEAP32[$2>>2]|0;
     HEAP32[$vararg_buffer7>>2] = $54;
     _checkAppendMsg($53,41711,$vararg_buffer7);
     $55 = HEAP32[$3>>2]|0;
     $56 = (($55) + -1)|0;
     HEAP32[$3>>2] = $56;
     break;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $57 = HEAP32[$i>>2]|0;
     $58 = HEAP32[$n>>2]|0;
     $59 = ($57|0)<($58|0);
     if (!($59)) {
      break;
     }
     $60 = HEAP32[$i>>2]|0;
     $61 = $60<<2;
     $62 = (8 + ($61))|0;
     $63 = HEAP32[$pOvflData>>2]|0;
     $64 = (($63) + ($62)|0);
     $65 = (_sqlite3Get4byte($64)|0);
     HEAP32[$iFreePage>>2] = $65;
     $66 = HEAP32[$0>>2]|0;
     $67 = HEAP32[$66>>2]|0;
     $68 = ((($67)) + 17|0);
     $69 = HEAP8[$68>>0]|0;
     $70 = ($69<<24>>24)!=(0);
     if ($70) {
      $71 = HEAP32[$0>>2]|0;
      $72 = HEAP32[$iFreePage>>2]|0;
      _checkPtrmap($71,$72,2,0);
     }
     $73 = HEAP32[$0>>2]|0;
     $74 = HEAP32[$iFreePage>>2]|0;
     (_checkRef($73,$74)|0);
     $75 = HEAP32[$i>>2]|0;
     $76 = (($75) + 1)|0;
     HEAP32[$i>>2] = $76;
    }
    $77 = HEAP32[$n>>2]|0;
    $78 = HEAP32[$3>>2]|0;
    $79 = (($78) - ($77))|0;
    HEAP32[$3>>2] = $79;
   } else {
    $80 = HEAP32[$0>>2]|0;
    $81 = HEAP32[$80>>2]|0;
    $82 = ((($81)) + 17|0);
    $83 = HEAP8[$82>>0]|0;
    $84 = $83&255;
    $85 = ($84|0)!=(0);
    $86 = HEAP32[$3>>2]|0;
    $87 = ($86|0)>(0);
    $or$cond = $85 & $87;
    if ($or$cond) {
     $88 = HEAP32[$pOvflData>>2]|0;
     $89 = (_sqlite3Get4byte($88)|0);
     HEAP32[$i>>2] = $89;
     $90 = HEAP32[$0>>2]|0;
     $91 = HEAP32[$i>>2]|0;
     $92 = HEAP32[$2>>2]|0;
     _checkPtrmap($90,$91,4,$92);
    }
   }
  } while(0);
  $93 = HEAP32[$pOvflData>>2]|0;
  $94 = (_sqlite3Get4byte($93)|0);
  HEAP32[$2>>2] = $94;
  $95 = HEAP32[$pOvflPage>>2]|0;
  _sqlite3PagerUnref($95);
  $96 = HEAP32[$1>>2]|0;
  $97 = ($96|0)!=(0);
  if (!($97)) {
   continue;
  }
  $98 = HEAP32[$3>>2]|0;
  $99 = HEAP32[$2>>2]|0;
  $100 = ($99|0)!=(0);
  $101 = $100&1;
  $102 = ($98|0)<($101|0);
  if (!($102)) {
   continue;
  }
  $103 = HEAP32[$0>>2]|0;
  _checkAppendMsg($103,41750,$vararg_buffer10);
 }
 if ((label|0) == 5) {
  $16 = HEAP32[$3>>2]|0;
  $17 = (($16) + 1)|0;
  $18 = HEAP32[$expected>>2]|0;
  $19 = HEAP32[$iFirst>>2]|0;
  HEAP32[$vararg_buffer>>2] = $17;
  $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr2>>2] = $18;
  $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr3>>2] = $19;
  _checkAppendMsg($15,41632,$vararg_buffer);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 8) {
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$2>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $30;
  _checkAppendMsg($29,41689,$vararg_buffer4);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 25) {
  STACKTOP = sp;return;
 }
}
function _checkPtrmap($pCheck,$iChild,$eType,$iParent) {
 $pCheck = $pCheck|0;
 $iChild = $iChild|0;
 $eType = $eType|0;
 $iParent = $iParent|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ePtrmapType = 0, $iPtrmapParent = 0, $or$cond = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer2 = 0;
 var $vararg_ptr5 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 49|0;
 $3 = sp + 36|0;
 $rc = sp + 32|0;
 $ePtrmapType = sp + 48|0;
 $iPtrmapParent = sp + 28|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iChild;
 HEAP8[$2>>0] = $eType;
 HEAP32[$3>>2] = $iParent;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_ptrmapGet($5,$6,$ePtrmapType,$iPtrmapParent)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$rc>>2]|0;
  $11 = ($10|0)==(7);
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)==(3082);
  $or$cond = $11 | $13;
  if ($or$cond) {
   $14 = HEAP32[$0>>2]|0;
   $15 = ((($14)) + 24|0);
   HEAP32[$15>>2] = 1;
  }
  $16 = HEAP32[$0>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $17;
  _checkAppendMsg($16,41549,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $18 = HEAP8[$ePtrmapType>>0]|0;
 $19 = $18&255;
 $20 = HEAP8[$2>>0]|0;
 $21 = $20&255;
 $22 = ($19|0)!=($21|0);
 if (!($22)) {
  $23 = HEAP32[$iPtrmapParent>>2]|0;
  $24 = HEAP32[$3>>2]|0;
  $25 = ($23|0)!=($24|0);
  if (!($25)) {
   STACKTOP = sp;return;
  }
 }
 $26 = HEAP32[$0>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = HEAP8[$2>>0]|0;
 $29 = $28&255;
 $30 = HEAP32[$3>>2]|0;
 $31 = HEAP8[$ePtrmapType>>0]|0;
 $32 = $31&255;
 $33 = HEAP32[$iPtrmapParent>>2]|0;
 HEAP32[$vararg_buffer2>>2] = $27;
 $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
 HEAP32[$vararg_ptr5>>2] = $29;
 $vararg_ptr6 = ((($vararg_buffer2)) + 8|0);
 HEAP32[$vararg_ptr6>>2] = $30;
 $vararg_ptr7 = ((($vararg_buffer2)) + 12|0);
 HEAP32[$vararg_ptr7>>2] = $32;
 $vararg_ptr8 = ((($vararg_buffer2)) + 16|0);
 HEAP32[$vararg_ptr8>>2] = $33;
 _checkAppendMsg($26,41578,$vararg_buffer2);
 STACKTOP = sp;return;
}
function _checkTreePage($pCheck,$iPage,$piMinKey,$0,$1) {
 $pCheck = $pCheck|0;
 $iPage = $iPage|0;
 $piMinKey = $piMinKey|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0;
 var $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0;
 var $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0;
 var $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0;
 var $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0;
 var $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0;
 var $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0;
 var $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0;
 var $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0;
 var $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0;
 var $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0;
 var $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0;
 var $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0;
 var $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0;
 var $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0;
 var $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0;
 var $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0;
 var $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0;
 var $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0;
 var $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cellStart = 0;
 var $contentOffset = 0, $d2 = 0, $data = 0, $depth = 0, $doCoverageCheck = 0, $hdr = 0, $heap = 0, $i = 0, $info = 0, $j = 0, $keyCanBeEqual = 0, $nCell = 0, $nFrag = 0, $nPage = 0, $pBt = 0, $pCell = 0, $pCellIdx = 0, $pPage = 0, $pc = 0, $pgno = 0;
 var $pgnoOvfl = 0, $prev = 0, $rc = 0, $savedIsInit = 0, $saved_v1 = 0, $saved_v2 = 0, $saved_zPfx = 0, $size = 0, $size1 = 0, $usableSize = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer11 = 0, $vararg_buffer14 = 0, $vararg_buffer16 = 0, $vararg_buffer20 = 0, $vararg_buffer4 = 0, $vararg_buffer9 = 0, $vararg_ptr19 = 0, $vararg_ptr23 = 0;
 var $vararg_ptr24 = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $vararg_buffer20 = sp + 96|0;
 $vararg_buffer16 = sp + 88|0;
 $vararg_buffer14 = sp + 80|0;
 $vararg_buffer11 = sp + 72|0;
 $vararg_buffer9 = sp + 64|0;
 $vararg_buffer4 = sp + 48|0;
 $vararg_buffer1 = sp + 40|0;
 $vararg_buffer = sp + 32|0;
 $2 = sp + 240|0;
 $3 = sp + 236|0;
 $4 = sp + 232|0;
 $5 = sp + 228|0;
 $6 = sp + 24|0;
 $pPage = sp + 224|0;
 $i = sp + 220|0;
 $rc = sp + 216|0;
 $depth = sp + 212|0;
 $d2 = sp + 208|0;
 $pgno = sp + 204|0;
 $nFrag = sp + 200|0;
 $hdr = sp + 196|0;
 $cellStart = sp + 192|0;
 $nCell = sp + 188|0;
 $doCoverageCheck = sp + 184|0;
 $keyCanBeEqual = sp + 180|0;
 $data = sp + 176|0;
 $pCell = sp + 172|0;
 $pCellIdx = sp + 168|0;
 $pBt = sp + 164|0;
 $pc = sp + 160|0;
 $usableSize = sp + 156|0;
 $contentOffset = sp + 152|0;
 $heap = sp + 148|0;
 $x = sp + 144|0;
 $prev = sp + 140|0;
 $saved_zPfx = sp + 136|0;
 $saved_v1 = sp + 132|0;
 $saved_v2 = sp + 128|0;
 $savedIsInit = sp + 244|0;
 $info = sp;
 $nPage = sp + 124|0;
 $pgnoOvfl = sp + 120|0;
 $size = sp + 116|0;
 $size1 = sp + 112|0;
 $j = sp + 108|0;
 HEAP32[$3>>2] = $pCheck;
 HEAP32[$4>>2] = $iPage;
 HEAP32[$5>>2] = $piMinKey;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$pPage>>2] = 0;
 HEAP32[$depth>>2] = -1;
 HEAP32[$doCoverageCheck>>2] = 1;
 HEAP32[$keyCanBeEqual>>2] = 1;
 HEAP32[$heap>>2] = 0;
 HEAP32[$prev>>2] = 0;
 $11 = HEAP32[$3>>2]|0;
 $12 = ((($11)) + 28|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$saved_zPfx>>2] = $13;
 $14 = HEAP32[$3>>2]|0;
 $15 = ((($14)) + 32|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$saved_v1>>2] = $16;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 36|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$saved_v2>>2] = $19;
 HEAP8[$savedIsInit>>0] = 0;
 $20 = HEAP32[$3>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pBt>>2] = $21;
 $22 = HEAP32[$pBt>>2]|0;
 $23 = ((($22)) + 36|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$usableSize>>2] = $24;
 $25 = HEAP32[$4>>2]|0;
 $26 = ($25|0)==(0);
 if ($26) {
  HEAP32[$2>>2] = 0;
  $500 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($500|0);
 }
 $27 = HEAP32[$3>>2]|0;
 $28 = HEAP32[$4>>2]|0;
 $29 = (_checkRef($27,$28)|0);
 $30 = ($29|0)!=(0);
 if ($30) {
  HEAP32[$2>>2] = 0;
  $500 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($500|0);
 }
 $31 = HEAP32[$3>>2]|0;
 $32 = ((($31)) + 28|0);
 HEAP32[$32>>2] = 41169;
 $33 = HEAP32[$4>>2]|0;
 $34 = HEAP32[$3>>2]|0;
 $35 = ((($34)) + 32|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$pBt>>2]|0;
 $37 = HEAP32[$4>>2]|0;
 $38 = (_btreeGetPage($36,$37,$pPage,0)|0);
 HEAP32[$rc>>2] = $38;
 $39 = ($38|0)!=(0);
 do {
  if ($39) {
   $40 = HEAP32[$3>>2]|0;
   $41 = HEAP32[$rc>>2]|0;
   HEAP32[$vararg_buffer>>2] = $41;
   _checkAppendMsg($40,41179,$vararg_buffer);
  } else {
   $42 = HEAP32[$pPage>>2]|0;
   $43 = HEAP8[$42>>0]|0;
   HEAP8[$savedIsInit>>0] = $43;
   $44 = HEAP32[$pPage>>2]|0;
   HEAP8[$44>>0] = 0;
   $45 = HEAP32[$pPage>>2]|0;
   $46 = (_btreeInitPage($45)|0);
   HEAP32[$rc>>2] = $46;
   $47 = ($46|0)!=(0);
   if ($47) {
    $48 = HEAP32[$3>>2]|0;
    $49 = HEAP32[$rc>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $49;
    _checkAppendMsg($48,41217,$vararg_buffer1);
    break;
   }
   $50 = HEAP32[$pPage>>2]|0;
   $51 = ((($50)) + 56|0);
   $52 = HEAP32[$51>>2]|0;
   HEAP32[$data>>2] = $52;
   $53 = HEAP32[$pPage>>2]|0;
   $54 = ((($53)) + 5|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55&255;
   HEAP32[$hdr>>2] = $56;
   $57 = HEAP32[$3>>2]|0;
   $58 = ((($57)) + 28|0);
   HEAP32[$58>>2] = 41255;
   $59 = HEAP32[$hdr>>2]|0;
   $60 = (($59) + 5)|0;
   $61 = HEAP32[$data>>2]|0;
   $62 = (($61) + ($60)|0);
   $63 = HEAP8[$62>>0]|0;
   $64 = $63&255;
   $65 = $64 << 8;
   $66 = HEAP32[$hdr>>2]|0;
   $67 = (($66) + 5)|0;
   $68 = HEAP32[$data>>2]|0;
   $69 = (($68) + ($67)|0);
   $70 = ((($69)) + 1|0);
   $71 = HEAP8[$70>>0]|0;
   $72 = $71&255;
   $73 = $65 | $72;
   $74 = (($73) - 1)|0;
   $75 = $74 & 65535;
   $76 = (($75) + 1)|0;
   HEAP32[$contentOffset>>2] = $76;
   $77 = HEAP32[$hdr>>2]|0;
   $78 = (($77) + 3)|0;
   $79 = HEAP32[$data>>2]|0;
   $80 = (($79) + ($78)|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = $81&255;
   $83 = $82 << 8;
   $84 = HEAP32[$hdr>>2]|0;
   $85 = (($84) + 3)|0;
   $86 = HEAP32[$data>>2]|0;
   $87 = (($86) + ($85)|0);
   $88 = ((($87)) + 1|0);
   $89 = HEAP8[$88>>0]|0;
   $90 = $89&255;
   $91 = $83 | $90;
   HEAP32[$nCell>>2] = $91;
   $92 = HEAP32[$hdr>>2]|0;
   $93 = (($92) + 12)|0;
   $94 = HEAP32[$pPage>>2]|0;
   $95 = ((($94)) + 4|0);
   $96 = HEAP8[$95>>0]|0;
   $97 = $96&255;
   $98 = $97<<2;
   $99 = (($93) - ($98))|0;
   HEAP32[$cellStart>>2] = $99;
   $100 = HEAP32[$cellStart>>2]|0;
   $101 = HEAP32[$nCell>>2]|0;
   $102 = (($101) - 1)|0;
   $103 = $102<<1;
   $104 = (($100) + ($103))|0;
   $105 = HEAP32[$data>>2]|0;
   $106 = (($105) + ($104)|0);
   HEAP32[$pCellIdx>>2] = $106;
   $107 = HEAP32[$pPage>>2]|0;
   $108 = ((($107)) + 4|0);
   $109 = HEAP8[$108>>0]|0;
   $110 = ($109<<24>>24)!=(0);
   if ($110) {
    $134 = HEAP32[$3>>2]|0;
    $135 = ((($134)) + 68|0);
    $136 = HEAP32[$135>>2]|0;
    HEAP32[$heap>>2] = $136;
    $137 = HEAP32[$heap>>2]|0;
    HEAP32[$137>>2] = 0;
   } else {
    $111 = HEAP32[$hdr>>2]|0;
    $112 = (($111) + 8)|0;
    $113 = HEAP32[$data>>2]|0;
    $114 = (($113) + ($112)|0);
    $115 = (_sqlite3Get4byte($114)|0);
    HEAP32[$pgno>>2] = $115;
    $116 = HEAP32[$pBt>>2]|0;
    $117 = ((($116)) + 17|0);
    $118 = HEAP8[$117>>0]|0;
    $119 = ($118<<24>>24)!=(0);
    if ($119) {
     $120 = HEAP32[$3>>2]|0;
     $121 = ((($120)) + 28|0);
     HEAP32[$121>>2] = 41281;
     $122 = HEAP32[$3>>2]|0;
     $123 = HEAP32[$pgno>>2]|0;
     $124 = HEAP32[$4>>2]|0;
     _checkPtrmap($122,$123,5,$124);
    }
    $125 = HEAP32[$3>>2]|0;
    $126 = HEAP32[$pgno>>2]|0;
    $127 = $6;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = (_checkTreePage($125,$126,$6,$129,$132)|0);
    HEAP32[$depth>>2] = $133;
    HEAP32[$keyCanBeEqual>>2] = 0;
   }
   $138 = HEAP32[$nCell>>2]|0;
   $139 = (($138) - 1)|0;
   HEAP32[$i>>2] = $139;
   while(1) {
    $140 = HEAP32[$i>>2]|0;
    $141 = ($140|0)>=(0);
    if (!($141)) {
     break;
    }
    $142 = HEAP32[$3>>2]|0;
    $143 = ((($142)) + 16|0);
    $144 = HEAP32[$143>>2]|0;
    $145 = ($144|0)!=(0);
    if (!($145)) {
     break;
    }
    $146 = HEAP32[$i>>2]|0;
    $147 = HEAP32[$3>>2]|0;
    $148 = ((($147)) + 36|0);
    HEAP32[$148>>2] = $146;
    $149 = HEAP32[$pCellIdx>>2]|0;
    $150 = HEAP8[$149>>0]|0;
    $151 = $150&255;
    $152 = $151 << 8;
    $153 = HEAP32[$pCellIdx>>2]|0;
    $154 = ((($153)) + 1|0);
    $155 = HEAP8[$154>>0]|0;
    $156 = $155&255;
    $157 = $152 | $156;
    HEAP32[$pc>>2] = $157;
    $158 = HEAP32[$pCellIdx>>2]|0;
    $159 = ((($158)) + -2|0);
    HEAP32[$pCellIdx>>2] = $159;
    $160 = HEAP32[$pc>>2]|0;
    $161 = HEAP32[$contentOffset>>2]|0;
    $162 = ($160>>>0)<($161>>>0);
    do {
     if ($162) {
      label = 19;
     } else {
      $163 = HEAP32[$pc>>2]|0;
      $164 = HEAP32[$usableSize>>2]|0;
      $165 = (($164) - 4)|0;
      $166 = ($163>>>0)>($165>>>0);
      if ($166) {
       label = 19;
      } else {
       $172 = HEAP32[$pc>>2]|0;
       $173 = HEAP32[$data>>2]|0;
       $174 = (($173) + ($172)|0);
       HEAP32[$pCell>>2] = $174;
       $175 = HEAP32[$pPage>>2]|0;
       $176 = ((($175)) + 80|0);
       $177 = HEAP32[$176>>2]|0;
       $178 = HEAP32[$pPage>>2]|0;
       $179 = HEAP32[$pCell>>2]|0;
       FUNCTION_TABLE_viii[$177 & 255]($178,$179,$info);
       $180 = HEAP32[$pc>>2]|0;
       $181 = ((($info)) + 18|0);
       $182 = HEAP16[$181>>1]|0;
       $183 = $182&65535;
       $184 = (($180) + ($183))|0;
       $185 = HEAP32[$usableSize>>2]|0;
       $186 = ($184>>>0)>($185>>>0);
       if ($186) {
        $187 = HEAP32[$3>>2]|0;
        _checkAppendMsg($187,41339,$vararg_buffer9);
        HEAP32[$doCoverageCheck>>2] = 0;
        break;
       }
       $188 = HEAP32[$pPage>>2]|0;
       $189 = ((($188)) + 2|0);
       $190 = HEAP8[$189>>0]|0;
       $191 = ($190<<24>>24)!=(0);
       if ($191) {
        $192 = HEAP32[$keyCanBeEqual>>2]|0;
        $193 = ($192|0)!=(0);
        $194 = $info;
        $195 = $194;
        $196 = HEAP32[$195>>2]|0;
        $197 = (($194) + 4)|0;
        $198 = $197;
        $199 = HEAP32[$198>>2]|0;
        $200 = $6;
        $201 = $200;
        $202 = HEAP32[$201>>2]|0;
        $203 = (($200) + 4)|0;
        $204 = $203;
        $205 = HEAP32[$204>>2]|0;
        if ($193) {
         $206 = ($199|0)>($205|0);
         $207 = ($196>>>0)>($202>>>0);
         $208 = ($199|0)==($205|0);
         $209 = $208 & $207;
         $210 = $206 | $209;
         if ($210) {
          label = 26;
         }
        } else {
         $211 = ($199|0)>($205|0);
         $212 = ($196>>>0)>=($202>>>0);
         $213 = ($199|0)==($205|0);
         $214 = $213 & $212;
         $215 = $211 | $214;
         if ($215) {
          label = 26;
         }
        }
        if ((label|0) == 26) {
         label = 0;
         $216 = HEAP32[$3>>2]|0;
         $217 = $info;
         $218 = $217;
         $219 = HEAP32[$218>>2]|0;
         $220 = (($217) + 4)|0;
         $221 = $220;
         $222 = HEAP32[$221>>2]|0;
         $223 = $vararg_buffer11;
         $224 = $223;
         HEAP32[$224>>2] = $219;
         $225 = (($223) + 4)|0;
         $226 = $225;
         HEAP32[$226>>2] = $222;
         _checkAppendMsg($216,41363,$vararg_buffer11);
        }
        $227 = $info;
        $228 = $227;
        $229 = HEAP32[$228>>2]|0;
        $230 = (($227) + 4)|0;
        $231 = $230;
        $232 = HEAP32[$231>>2]|0;
        $233 = $6;
        $234 = $233;
        HEAP32[$234>>2] = $229;
        $235 = (($233) + 4)|0;
        $236 = $235;
        HEAP32[$236>>2] = $232;
       }
       $237 = ((($info)) + 12|0);
       $238 = HEAP32[$237>>2]|0;
       $239 = ((($info)) + 16|0);
       $240 = HEAP16[$239>>1]|0;
       $241 = $240&65535;
       $242 = ($238>>>0)>($241>>>0);
       if ($242) {
        $243 = ((($info)) + 12|0);
        $244 = HEAP32[$243>>2]|0;
        $245 = ((($info)) + 16|0);
        $246 = HEAP16[$245>>1]|0;
        $247 = $246&65535;
        $248 = (($244) - ($247))|0;
        $249 = HEAP32[$usableSize>>2]|0;
        $250 = (($248) + ($249))|0;
        $251 = (($250) - 5)|0;
        $252 = HEAP32[$usableSize>>2]|0;
        $253 = (($252) - 4)|0;
        $254 = (($251>>>0) / ($253>>>0))&-1;
        HEAP32[$nPage>>2] = $254;
        $255 = ((($info)) + 18|0);
        $256 = HEAP16[$255>>1]|0;
        $257 = $256&65535;
        $258 = (($257) - 4)|0;
        $259 = HEAP32[$pCell>>2]|0;
        $260 = (($259) + ($258)|0);
        $261 = (_sqlite3Get4byte($260)|0);
        HEAP32[$pgnoOvfl>>2] = $261;
        $262 = HEAP32[$pBt>>2]|0;
        $263 = ((($262)) + 17|0);
        $264 = HEAP8[$263>>0]|0;
        $265 = ($264<<24>>24)!=(0);
        if ($265) {
         $266 = HEAP32[$3>>2]|0;
         $267 = HEAP32[$pgnoOvfl>>2]|0;
         $268 = HEAP32[$4>>2]|0;
         _checkPtrmap($266,$267,3,$268);
        }
        $269 = HEAP32[$3>>2]|0;
        $270 = HEAP32[$pgnoOvfl>>2]|0;
        $271 = HEAP32[$nPage>>2]|0;
        _checkList($269,0,$270,$271);
       }
       $272 = HEAP32[$pPage>>2]|0;
       $273 = ((($272)) + 4|0);
       $274 = HEAP8[$273>>0]|0;
       $275 = ($274<<24>>24)!=(0);
       if ($275) {
        $299 = HEAP32[$heap>>2]|0;
        $300 = HEAP32[$pc>>2]|0;
        $301 = $300 << 16;
        $302 = HEAP32[$pc>>2]|0;
        $303 = ((($info)) + 18|0);
        $304 = HEAP16[$303>>1]|0;
        $305 = $304&65535;
        $306 = (($302) + ($305))|0;
        $307 = (($306) - 1)|0;
        $308 = $301 | $307;
        _btreeHeapInsert($299,$308);
        break;
       }
       $276 = HEAP32[$pCell>>2]|0;
       $277 = (_sqlite3Get4byte($276)|0);
       HEAP32[$pgno>>2] = $277;
       $278 = HEAP32[$pBt>>2]|0;
       $279 = ((($278)) + 17|0);
       $280 = HEAP8[$279>>0]|0;
       $281 = ($280<<24>>24)!=(0);
       if ($281) {
        $282 = HEAP32[$3>>2]|0;
        $283 = HEAP32[$pgno>>2]|0;
        $284 = HEAP32[$4>>2]|0;
        _checkPtrmap($282,$283,5,$284);
       }
       $285 = HEAP32[$3>>2]|0;
       $286 = HEAP32[$pgno>>2]|0;
       $287 = $6;
       $288 = $287;
       $289 = HEAP32[$288>>2]|0;
       $290 = (($287) + 4)|0;
       $291 = $290;
       $292 = HEAP32[$291>>2]|0;
       $293 = (_checkTreePage($285,$286,$6,$289,$292)|0);
       HEAP32[$d2>>2] = $293;
       HEAP32[$keyCanBeEqual>>2] = 0;
       $294 = HEAP32[$d2>>2]|0;
       $295 = HEAP32[$depth>>2]|0;
       $296 = ($294|0)!=($295|0);
       if ($296) {
        $297 = HEAP32[$3>>2]|0;
        _checkAppendMsg($297,41387,$vararg_buffer14);
        $298 = HEAP32[$d2>>2]|0;
        HEAP32[$depth>>2] = $298;
       }
      }
     }
    } while(0);
    if ((label|0) == 19) {
     label = 0;
     $167 = HEAP32[$3>>2]|0;
     $168 = HEAP32[$pc>>2]|0;
     $169 = HEAP32[$contentOffset>>2]|0;
     $170 = HEAP32[$usableSize>>2]|0;
     $171 = (($170) - 4)|0;
     HEAP32[$vararg_buffer4>>2] = $168;
     $vararg_ptr7 = ((($vararg_buffer4)) + 4|0);
     HEAP32[$vararg_ptr7>>2] = $169;
     $vararg_ptr8 = ((($vararg_buffer4)) + 8|0);
     HEAP32[$vararg_ptr8>>2] = $171;
     _checkAppendMsg($167,41309,$vararg_buffer4);
     HEAP32[$doCoverageCheck>>2] = 0;
    }
    $309 = HEAP32[$i>>2]|0;
    $310 = (($309) + -1)|0;
    HEAP32[$i>>2] = $310;
   }
   $311 = $6;
   $312 = $311;
   $313 = HEAP32[$312>>2]|0;
   $314 = (($311) + 4)|0;
   $315 = $314;
   $316 = HEAP32[$315>>2]|0;
   $317 = HEAP32[$5>>2]|0;
   $318 = $317;
   $319 = $318;
   HEAP32[$319>>2] = $313;
   $320 = (($318) + 4)|0;
   $321 = $320;
   HEAP32[$321>>2] = $316;
   $322 = HEAP32[$3>>2]|0;
   $323 = ((($322)) + 28|0);
   HEAP32[$323>>2] = 0;
   $324 = HEAP32[$doCoverageCheck>>2]|0;
   $325 = ($324|0)!=(0);
   if ($325) {
    $326 = HEAP32[$3>>2]|0;
    $327 = ((($326)) + 16|0);
    $328 = HEAP32[$327>>2]|0;
    $329 = ($328|0)>(0);
    if ($329) {
     $330 = HEAP32[$pPage>>2]|0;
     $331 = ((($330)) + 4|0);
     $332 = HEAP8[$331>>0]|0;
     $333 = ($332<<24>>24)!=(0);
     L60: do {
      if (!($333)) {
       $334 = HEAP32[$3>>2]|0;
       $335 = ((($334)) + 68|0);
       $336 = HEAP32[$335>>2]|0;
       HEAP32[$heap>>2] = $336;
       $337 = HEAP32[$heap>>2]|0;
       HEAP32[$337>>2] = 0;
       $338 = HEAP32[$nCell>>2]|0;
       $339 = (($338) - 1)|0;
       HEAP32[$i>>2] = $339;
       while(1) {
        $340 = HEAP32[$i>>2]|0;
        $341 = ($340|0)>=(0);
        if (!($341)) {
         break L60;
        }
        $342 = HEAP32[$cellStart>>2]|0;
        $343 = HEAP32[$i>>2]|0;
        $344 = $343<<1;
        $345 = (($342) + ($344))|0;
        $346 = HEAP32[$data>>2]|0;
        $347 = (($346) + ($345)|0);
        $348 = HEAP8[$347>>0]|0;
        $349 = $348&255;
        $350 = $349 << 8;
        $351 = HEAP32[$cellStart>>2]|0;
        $352 = HEAP32[$i>>2]|0;
        $353 = $352<<1;
        $354 = (($351) + ($353))|0;
        $355 = HEAP32[$data>>2]|0;
        $356 = (($355) + ($354)|0);
        $357 = ((($356)) + 1|0);
        $358 = HEAP8[$357>>0]|0;
        $359 = $358&255;
        $360 = $350 | $359;
        HEAP32[$pc>>2] = $360;
        $361 = HEAP32[$pPage>>2]|0;
        $362 = ((($361)) + 76|0);
        $363 = HEAP32[$362>>2]|0;
        $364 = HEAP32[$pPage>>2]|0;
        $365 = HEAP32[$pc>>2]|0;
        $366 = HEAP32[$data>>2]|0;
        $367 = (($366) + ($365)|0);
        $368 = (FUNCTION_TABLE_iii[$363 & 255]($364,$367)|0);
        $369 = $368&65535;
        HEAP32[$size>>2] = $369;
        $370 = HEAP32[$heap>>2]|0;
        $371 = HEAP32[$pc>>2]|0;
        $372 = $371 << 16;
        $373 = HEAP32[$pc>>2]|0;
        $374 = HEAP32[$size>>2]|0;
        $375 = (($373) + ($374))|0;
        $376 = (($375) - 1)|0;
        $377 = $372 | $376;
        _btreeHeapInsert($370,$377);
        $378 = HEAP32[$i>>2]|0;
        $379 = (($378) + -1)|0;
        HEAP32[$i>>2] = $379;
       }
      }
     } while(0);
     $380 = HEAP32[$hdr>>2]|0;
     $381 = (($380) + 1)|0;
     $382 = HEAP32[$data>>2]|0;
     $383 = (($382) + ($381)|0);
     $384 = HEAP8[$383>>0]|0;
     $385 = $384&255;
     $386 = $385 << 8;
     $387 = HEAP32[$hdr>>2]|0;
     $388 = (($387) + 1)|0;
     $389 = HEAP32[$data>>2]|0;
     $390 = (($389) + ($388)|0);
     $391 = ((($390)) + 1|0);
     $392 = HEAP8[$391>>0]|0;
     $393 = $392&255;
     $394 = $386 | $393;
     HEAP32[$i>>2] = $394;
     while(1) {
      $395 = HEAP32[$i>>2]|0;
      $396 = ($395|0)>(0);
      if (!($396)) {
       break;
      }
      $397 = HEAP32[$i>>2]|0;
      $398 = (($397) + 2)|0;
      $399 = HEAP32[$data>>2]|0;
      $400 = (($399) + ($398)|0);
      $401 = HEAP8[$400>>0]|0;
      $402 = $401&255;
      $403 = $402 << 8;
      $404 = HEAP32[$i>>2]|0;
      $405 = (($404) + 2)|0;
      $406 = HEAP32[$data>>2]|0;
      $407 = (($406) + ($405)|0);
      $408 = ((($407)) + 1|0);
      $409 = HEAP8[$408>>0]|0;
      $410 = $409&255;
      $411 = $403 | $410;
      HEAP32[$size1>>2] = $411;
      $412 = HEAP32[$heap>>2]|0;
      $413 = HEAP32[$i>>2]|0;
      $414 = $413 << 16;
      $415 = HEAP32[$i>>2]|0;
      $416 = HEAP32[$size1>>2]|0;
      $417 = (($415) + ($416))|0;
      $418 = (($417) - 1)|0;
      $419 = $414 | $418;
      _btreeHeapInsert($412,$419);
      $420 = HEAP32[$i>>2]|0;
      $421 = HEAP32[$data>>2]|0;
      $422 = (($421) + ($420)|0);
      $423 = HEAP8[$422>>0]|0;
      $424 = $423&255;
      $425 = $424 << 8;
      $426 = HEAP32[$i>>2]|0;
      $427 = HEAP32[$data>>2]|0;
      $428 = (($427) + ($426)|0);
      $429 = ((($428)) + 1|0);
      $430 = HEAP8[$429>>0]|0;
      $431 = $430&255;
      $432 = $425 | $431;
      HEAP32[$j>>2] = $432;
      $433 = HEAP32[$j>>2]|0;
      HEAP32[$i>>2] = $433;
     }
     HEAP32[$nFrag>>2] = 0;
     $434 = HEAP32[$contentOffset>>2]|0;
     $435 = (($434) - 1)|0;
     HEAP32[$prev>>2] = $435;
     while(1) {
      $436 = HEAP32[$heap>>2]|0;
      $437 = (_btreeHeapPull($436,$x)|0);
      $438 = ($437|0)!=(0);
      if (!($438)) {
       break;
      }
      $439 = HEAP32[$prev>>2]|0;
      $440 = $439 & 65535;
      $441 = HEAP32[$x>>2]|0;
      $442 = $441 >>> 16;
      $443 = ($440>>>0)>=($442>>>0);
      if ($443) {
       label = 51;
       break;
      }
      $448 = HEAP32[$x>>2]|0;
      $449 = $448 >>> 16;
      $450 = HEAP32[$prev>>2]|0;
      $451 = $450 & 65535;
      $452 = (($449) - ($451))|0;
      $453 = (($452) - 1)|0;
      $454 = HEAP32[$nFrag>>2]|0;
      $455 = (($454) + ($453))|0;
      HEAP32[$nFrag>>2] = $455;
      $456 = HEAP32[$x>>2]|0;
      HEAP32[$prev>>2] = $456;
     }
     if ((label|0) == 51) {
      $444 = HEAP32[$3>>2]|0;
      $445 = HEAP32[$x>>2]|0;
      $446 = $445 >>> 16;
      $447 = HEAP32[$4>>2]|0;
      HEAP32[$vararg_buffer16>>2] = $446;
      $vararg_ptr19 = ((($vararg_buffer16)) + 4|0);
      HEAP32[$vararg_ptr19>>2] = $447;
      _checkAppendMsg($444,41412,$vararg_buffer16);
     }
     $457 = HEAP32[$usableSize>>2]|0;
     $458 = HEAP32[$prev>>2]|0;
     $459 = $458 & 65535;
     $460 = (($457) - ($459))|0;
     $461 = (($460) - 1)|0;
     $462 = HEAP32[$nFrag>>2]|0;
     $463 = (($462) + ($461))|0;
     HEAP32[$nFrag>>2] = $463;
     $464 = HEAP32[$heap>>2]|0;
     $465 = HEAP32[$464>>2]|0;
     $466 = ($465|0)==(0);
     if ($466) {
      $467 = HEAP32[$nFrag>>2]|0;
      $468 = HEAP32[$hdr>>2]|0;
      $469 = (($468) + 7)|0;
      $470 = HEAP32[$data>>2]|0;
      $471 = (($470) + ($469)|0);
      $472 = HEAP8[$471>>0]|0;
      $473 = $472&255;
      $474 = ($467|0)!=($473|0);
      if ($474) {
       $475 = HEAP32[$3>>2]|0;
       $476 = HEAP32[$nFrag>>2]|0;
       $477 = HEAP32[$hdr>>2]|0;
       $478 = (($477) + 7)|0;
       $479 = HEAP32[$data>>2]|0;
       $480 = (($479) + ($478)|0);
       $481 = HEAP8[$480>>0]|0;
       $482 = $481&255;
       $483 = HEAP32[$4>>2]|0;
       HEAP32[$vararg_buffer20>>2] = $476;
       $vararg_ptr23 = ((($vararg_buffer20)) + 4|0);
       HEAP32[$vararg_ptr23>>2] = $482;
       $vararg_ptr24 = ((($vararg_buffer20)) + 8|0);
       HEAP32[$vararg_ptr24>>2] = $483;
       _checkAppendMsg($475,41449,$vararg_buffer20);
      }
     }
    }
   }
  }
 } while(0);
 $484 = HEAP32[$doCoverageCheck>>2]|0;
 $485 = ($484|0)!=(0);
 if (!($485)) {
  $486 = HEAP8[$savedIsInit>>0]|0;
  $487 = HEAP32[$pPage>>2]|0;
  HEAP8[$487>>0] = $486;
 }
 $488 = HEAP32[$pPage>>2]|0;
 _releasePage($488);
 $489 = HEAP32[$saved_zPfx>>2]|0;
 $490 = HEAP32[$3>>2]|0;
 $491 = ((($490)) + 28|0);
 HEAP32[$491>>2] = $489;
 $492 = HEAP32[$saved_v1>>2]|0;
 $493 = HEAP32[$3>>2]|0;
 $494 = ((($493)) + 32|0);
 HEAP32[$494>>2] = $492;
 $495 = HEAP32[$saved_v2>>2]|0;
 $496 = HEAP32[$3>>2]|0;
 $497 = ((($496)) + 36|0);
 HEAP32[$497>>2] = $495;
 $498 = HEAP32[$depth>>2]|0;
 $499 = (($498) + 1)|0;
 HEAP32[$2>>2] = $499;
 $500 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($500|0);
}
function _getPageReferenced($pCheck,$iPg) {
 $pCheck = $pCheck|0;
 $iPg = $iPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $iPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2>>>0) / 8)&-1;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3)|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8&255;
 $10 = HEAP32[$1>>2]|0;
 $11 = $10 & 7;
 $12 = 1 << $11;
 $13 = $9 & $12;
 STACKTOP = sp;return ($13|0);
}
function _checkAppendMsg($pCheck,$zFormat,$varargs) {
 $pCheck = $pCheck|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 HEAP32[$0>>2] = $pCheck;
 HEAP32[$1>>2] = $zFormat;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$0>>2]|0;
 $11 = ((($10)) + 20|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (($12) + 1)|0;
 HEAP32[$11>>2] = $13;
 HEAP32[$ap>>2] = $varargs;
 $14 = HEAP32[$0>>2]|0;
 $15 = ((($14)) + 40|0);
 $16 = ((($15)) + 12|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 40|0);
  _sqlite3StrAccumAppend($20,41167,1);
 }
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 28|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ($23|0)!=(0|0);
 if ($24) {
  $25 = HEAP32[$0>>2]|0;
  $26 = ((($25)) + 40|0);
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 28|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 32|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = ((($33)) + 36|0);
  $35 = HEAP32[$34>>2]|0;
  HEAP32[$vararg_buffer>>2] = $32;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $35;
  _sqlite3XPrintf($26,$29,$vararg_buffer);
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 40|0);
 $38 = HEAP32[$1>>2]|0;
 _sqlite3VXPrintf($37,$38,$ap);
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 40|0);
 $41 = ((($40)) + 24|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = ($43|0)==(1);
 if (!($44)) {
  STACKTOP = sp;return;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 24|0);
 HEAP32[$46>>2] = 1;
 STACKTOP = sp;return;
}
function _checkRef($pCheck,$iPage) {
 $pCheck = $pCheck|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 HEAP32[$1>>2] = $pCheck;
 HEAP32[$2>>2] = $iPage;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   HEAP32[$0>>2] = 1;
  } else {
   $5 = HEAP32[$2>>2]|0;
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 12|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = ($5>>>0)>($8>>>0);
   $10 = HEAP32[$1>>2]|0;
   $11 = HEAP32[$2>>2]|0;
   if ($9) {
    HEAP32[$vararg_buffer>>2] = $11;
    _checkAppendMsg($10,41501,$vararg_buffer);
    HEAP32[$0>>2] = 1;
    break;
   }
   $12 = (_getPageReferenced($10,$11)|0);
   $13 = ($12|0)!=(0);
   $14 = HEAP32[$1>>2]|0;
   $15 = HEAP32[$2>>2]|0;
   if ($13) {
    HEAP32[$vararg_buffer1>>2] = $15;
    _checkAppendMsg($14,41524,$vararg_buffer1);
    HEAP32[$0>>2] = 1;
    break;
   } else {
    _setPageReferenced($14,$15);
    HEAP32[$0>>2] = 0;
    break;
   }
  }
 } while(0);
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _btreeHeapInsert($aHeap,$x) {
 $aHeap = $aHeap|0;
 $x = $x|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $j = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $aHeap;
 HEAP32[$1>>2] = $x;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) + 1)|0;
 HEAP32[$2>>2] = $4;
 HEAP32[$i>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$i>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = (($7) + ($6<<2)|0);
 HEAP32[$8>>2] = $5;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = (($9>>>0) / 2)&-1;
  HEAP32[$j>>2] = $10;
  $11 = ($10>>>0)>(0);
  if (!($11)) {
   label = 5;
   break;
  }
  $12 = HEAP32[$j>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = (($13) + ($12<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$0>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = ($15>>>0)>($19>>>0);
  if (!($20)) {
   label = 5;
   break;
  }
  $21 = HEAP32[$j>>2]|0;
  $22 = HEAP32[$0>>2]|0;
  $23 = (($22) + ($21<<2)|0);
  $24 = HEAP32[$23>>2]|0;
  HEAP32[$1>>2] = $24;
  $25 = HEAP32[$i>>2]|0;
  $26 = HEAP32[$0>>2]|0;
  $27 = (($26) + ($25<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$j>>2]|0;
  $30 = HEAP32[$0>>2]|0;
  $31 = (($30) + ($29<<2)|0);
  HEAP32[$31>>2] = $28;
  $32 = HEAP32[$1>>2]|0;
  $33 = HEAP32[$i>>2]|0;
  $34 = HEAP32[$0>>2]|0;
  $35 = (($34) + ($33<<2)|0);
  HEAP32[$35>>2] = $32;
  $36 = HEAP32[$j>>2]|0;
  HEAP32[$i>>2] = $36;
 }
 if ((label|0) == 5) {
  STACKTOP = sp;return;
 }
}
function _btreeHeapPull($aHeap,$pOut) {
 $aHeap = $aHeap|0;
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $j = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $j = sp + 8|0;
 $i = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $aHeap;
 HEAP32[$2>>2] = $pOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$x>>2] = $4;
 $5 = ($4|0)==(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $64 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 HEAP32[$9>>2] = $8;
 $10 = HEAP32[$x>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) + ($10<<2)|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 4|0);
 HEAP32[$15>>2] = $13;
 $16 = HEAP32[$x>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = (($17) + ($16<<2)|0);
 HEAP32[$18>>2] = -1;
 $19 = HEAP32[$1>>2]|0;
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + -1)|0;
 HEAP32[$19>>2] = $21;
 HEAP32[$i>>2] = 1;
 while(1) {
  $22 = HEAP32[$i>>2]|0;
  $23 = $22<<1;
  HEAP32[$j>>2] = $23;
  $24 = HEAP32[$1>>2]|0;
  $25 = HEAP32[$24>>2]|0;
  $26 = ($23>>>0)<=($25>>>0);
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$j>>2]|0;
  $28 = HEAP32[$1>>2]|0;
  $29 = (($28) + ($27<<2)|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$j>>2]|0;
  $32 = (($31) + 1)|0;
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + ($32<<2)|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ($30>>>0)>($35>>>0);
  if ($36) {
   $37 = HEAP32[$j>>2]|0;
   $38 = (($37) + 1)|0;
   HEAP32[$j>>2] = $38;
  }
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$1>>2]|0;
  $41 = (($40) + ($39<<2)|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$j>>2]|0;
  $44 = HEAP32[$1>>2]|0;
  $45 = (($44) + ($43<<2)|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = ($42>>>0)<($46>>>0);
  if ($47) {
   break;
  }
  $48 = HEAP32[$i>>2]|0;
  $49 = HEAP32[$1>>2]|0;
  $50 = (($49) + ($48<<2)|0);
  $51 = HEAP32[$50>>2]|0;
  HEAP32[$x>>2] = $51;
  $52 = HEAP32[$j>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = (($53) + ($52<<2)|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = (($57) + ($56<<2)|0);
  HEAP32[$58>>2] = $55;
  $59 = HEAP32[$x>>2]|0;
  $60 = HEAP32[$j>>2]|0;
  $61 = HEAP32[$1>>2]|0;
  $62 = (($61) + ($60<<2)|0);
  HEAP32[$62>>2] = $59;
  $63 = HEAP32[$j>>2]|0;
  HEAP32[$i>>2] = $63;
 }
 HEAP32[$0>>2] = 1;
 $64 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($64|0);
}
function _vdbeMergeEngineFree($pMerger) {
 $pMerger = $pMerger|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pMerger;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_free($14);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = HEAP32[$0>>2]|0;
  $5 = HEAP32[$4>>2]|0;
  $6 = ($3|0)<($5|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$i>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 12|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + (($7*56)|0)|0);
  _vdbePmaReaderClear($11);
  $12 = HEAP32[$i>>2]|0;
  $13 = (($12) + 1)|0;
  HEAP32[$i>>2] = $13;
 }
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_free($14);
 STACKTOP = sp;return;
}
function _vdbeSortSubtaskCleanup($db,$pTask) {
 $db = $db|0;
 $pTask = $pTask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTask;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3DbFree($2,$5);
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 _vdbeSorterRecordFree(0,$8);
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 40|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3OsCloseFree($15);
 }
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 56|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  $23 = HEAP32[$1>>2]|0;
  dest=$23; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 56|0);
 $22 = HEAP32[$21>>2]|0;
 _sqlite3OsCloseFree($22);
 $23 = HEAP32[$1>>2]|0;
 dest=$23; stop=dest+72|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _vdbeSorterRecordFree($db,$pRecord) {
 $db = $db|0;
 $pRecord = $pRecord|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pRecord;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$p>>2] = $2;
 while(1) {
  $3 = HEAP32[$p>>2]|0;
  $4 = ($3|0)!=(0|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$p>>2]|0;
  $6 = ((($5)) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pNext>>2] = $7;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$p>>2]|0;
  _sqlite3DbFree($8,$9);
  $10 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $10;
 }
 STACKTOP = sp;return;
}
function _sqlite3OsCloseFree($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3OsClose($1);
 $2 = HEAP32[$0>>2]|0;
 _sqlite3_free($2);
 STACKTOP = sp;return;
}
function _vdbePmaReaderClear($pReadr) {
 $pReadr = $pReadr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pReadr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3_free($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3_free($6);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 44|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 24|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 44|0);
  $16 = HEAP32[$15>>2]|0;
  (_sqlite3OsUnfetch($13,0,0,$16)|0);
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 48|0);
 $19 = HEAP32[$18>>2]|0;
 _vdbeIncrFree($19);
 $20 = HEAP32[$0>>2]|0;
 dest=$20; stop=dest+56|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _vdbeIncrFree($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pIncr;
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)!=(0|0);
 if (!($2)) {
  STACKTOP = sp;return;
 }
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 _vdbeMergeEngineFree($5);
 $6 = HEAP32[$0>>2]|0;
 _sqlite3_free($6);
 STACKTOP = sp;return;
}
function _invalidateIncrblobCursors($pBtree,$0,$1,$isClearTable) {
 $pBtree = $pBtree|0;
 $0 = $0|0;
 $1 = $1|0;
 $isClearTable = $isClearTable|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $p = sp + 8|0;
 HEAP32[$2>>2] = $pBtree;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $isClearTable;
 $9 = HEAP32[$2>>2]|0;
 $10 = ((($9)) + 11|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 $13 = ($12|0)==(0);
 if ($13) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 11|0);
 HEAP8[$15>>0] = 0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($16)) + 4|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ((($18)) + 8|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$p>>2] = $20;
 while(1) {
  $21 = HEAP32[$p>>2]|0;
  $22 = ($21|0)!=(0|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$p>>2]|0;
  $24 = ((($23)) + 64|0);
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  $27 = $26 & 16;
  $28 = ($27|0)!=(0);
  do {
   if ($28) {
    $29 = HEAP32[$2>>2]|0;
    $30 = ((($29)) + 11|0);
    HEAP8[$30>>0] = 1;
    $31 = HEAP32[$4>>2]|0;
    $32 = ($31|0)!=(0);
    if (!($32)) {
     $33 = HEAP32[$p>>2]|0;
     $34 = ((($33)) + 16|0);
     $35 = $34;
     $36 = $35;
     $37 = HEAP32[$36>>2]|0;
     $38 = (($35) + 4)|0;
     $39 = $38;
     $40 = HEAP32[$39>>2]|0;
     $41 = $3;
     $42 = $41;
     $43 = HEAP32[$42>>2]|0;
     $44 = (($41) + 4)|0;
     $45 = $44;
     $46 = HEAP32[$45>>2]|0;
     $47 = ($37|0)==($43|0);
     $48 = ($40|0)==($46|0);
     $49 = $47 & $48;
     if (!($49)) {
      break;
     }
    }
    $50 = HEAP32[$p>>2]|0;
    $51 = ((($50)) + 66|0);
    HEAP8[$51>>0] = 0;
   }
  } while(0);
  $52 = HEAP32[$p>>2]|0;
  $53 = ((($52)) + 8|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$p>>2] = $54;
 }
 STACKTOP = sp;return;
}
function _clearDatabasePage($pBt,$pgno,$freePageFlag,$pnChange) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 $freePageFlag = $freePageFlag|0;
 $pnChange = $pnChange|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $hdr = 0, $i = 0, $pCell = 0, $pPage = 0, $rc = 0, $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $pPage = sp + 16|0;
 $rc = sp + 12|0;
 $pCell = sp + 8|0;
 $i = sp + 4|0;
 $hdr = sp;
 $szCell = sp + 40|0;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $freePageFlag;
 HEAP32[$4>>2] = $pnChange;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = (_btreePagecount($6)|0);
 $8 = ($5>>>0)>($7>>>0);
 if ($8) {
  $9 = (_sqlite3CorruptError(66688)|0);
  HEAP32[$0>>2] = $9;
  $123 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (_getAndInitPage($10,$11,$pPage,0,0)|0);
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if ($14) {
  $15 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $15;
  $123 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $16 = HEAP32[$pPage>>2]|0;
 $17 = ((($16)) + 8|0);
 $18 = HEAP8[$17>>0]|0;
 $19 = ($18<<24>>24)!=(0);
 L9: do {
  if ($19) {
   $20 = (_sqlite3CorruptError(66693)|0);
   HEAP32[$rc>>2] = $20;
  } else {
   $21 = HEAP32[$pPage>>2]|0;
   $22 = ((($21)) + 8|0);
   HEAP8[$22>>0] = 1;
   $23 = HEAP32[$pPage>>2]|0;
   $24 = ((($23)) + 5|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   HEAP32[$hdr>>2] = $26;
   HEAP32[$i>>2] = 0;
   while(1) {
    $27 = HEAP32[$i>>2]|0;
    $28 = HEAP32[$pPage>>2]|0;
    $29 = ((($28)) + 18|0);
    $30 = HEAP16[$29>>1]|0;
    $31 = $30&65535;
    $32 = ($27|0)<($31|0);
    $33 = HEAP32[$pPage>>2]|0;
    if (!($32)) {
     break;
    }
    $34 = ((($33)) + 56|0);
    $35 = HEAP32[$34>>2]|0;
    $36 = HEAP32[$pPage>>2]|0;
    $37 = ((($36)) + 20|0);
    $38 = HEAP16[$37>>1]|0;
    $39 = $38&65535;
    $40 = HEAP32[$i>>2]|0;
    $41 = $40<<1;
    $42 = HEAP32[$pPage>>2]|0;
    $43 = ((($42)) + 64|0);
    $44 = HEAP32[$43>>2]|0;
    $45 = (($44) + ($41)|0);
    $46 = HEAP8[$45>>0]|0;
    $47 = $46&255;
    $48 = $47 << 8;
    $49 = HEAP32[$i>>2]|0;
    $50 = $49<<1;
    $51 = HEAP32[$pPage>>2]|0;
    $52 = ((($51)) + 64|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (($53) + ($50)|0);
    $55 = ((($54)) + 1|0);
    $56 = HEAP8[$55>>0]|0;
    $57 = $56&255;
    $58 = $48 | $57;
    $59 = $39 & $58;
    $60 = (($35) + ($59)|0);
    HEAP32[$pCell>>2] = $60;
    $61 = HEAP32[$pPage>>2]|0;
    $62 = ((($61)) + 4|0);
    $63 = HEAP8[$62>>0]|0;
    $64 = ($63<<24>>24)!=(0);
    if (!($64)) {
     $65 = HEAP32[$1>>2]|0;
     $66 = HEAP32[$pCell>>2]|0;
     $67 = (_sqlite3Get4byte($66)|0);
     $68 = HEAP32[$4>>2]|0;
     $69 = (_clearDatabasePage($65,$67,1,$68)|0);
     HEAP32[$rc>>2] = $69;
     $70 = HEAP32[$rc>>2]|0;
     $71 = ($70|0)!=(0);
     if ($71) {
      break L9;
     }
    }
    $72 = HEAP32[$pPage>>2]|0;
    $73 = HEAP32[$pCell>>2]|0;
    $74 = (_clearCell($72,$73,$szCell)|0);
    HEAP32[$rc>>2] = $74;
    $75 = HEAP32[$rc>>2]|0;
    $76 = ($75|0)!=(0);
    if ($76) {
     break L9;
    }
    $77 = HEAP32[$i>>2]|0;
    $78 = (($77) + 1)|0;
    HEAP32[$i>>2] = $78;
   }
   $79 = ((($33)) + 4|0);
   $80 = HEAP8[$79>>0]|0;
   $81 = ($80<<24>>24)!=(0);
   if ($81) {
    $94 = HEAP32[$4>>2]|0;
    $95 = ($94|0)!=(0|0);
    if ($95) {
     $96 = HEAP32[$pPage>>2]|0;
     $97 = ((($96)) + 18|0);
     $98 = HEAP16[$97>>1]|0;
     $99 = $98&65535;
     $100 = HEAP32[$4>>2]|0;
     $101 = HEAP32[$100>>2]|0;
     $102 = (($101) + ($99))|0;
     HEAP32[$100>>2] = $102;
    }
   } else {
    $82 = HEAP32[$1>>2]|0;
    $83 = HEAP32[$hdr>>2]|0;
    $84 = (($83) + 8)|0;
    $85 = HEAP32[$pPage>>2]|0;
    $86 = ((($85)) + 56|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = (($87) + ($84)|0);
    $89 = (_sqlite3Get4byte($88)|0);
    $90 = HEAP32[$4>>2]|0;
    $91 = (_clearDatabasePage($82,$89,1,$90)|0);
    HEAP32[$rc>>2] = $91;
    $92 = HEAP32[$rc>>2]|0;
    $93 = ($92|0)!=(0);
    if ($93) {
     break;
    }
   }
   $103 = HEAP32[$3>>2]|0;
   $104 = ($103|0)!=(0);
   $105 = HEAP32[$pPage>>2]|0;
   if ($104) {
    _freePage($105,$rc);
    break;
   }
   $106 = ((($105)) + 72|0);
   $107 = HEAP32[$106>>2]|0;
   $108 = (_sqlite3PagerWrite($107)|0);
   HEAP32[$rc>>2] = $108;
   $109 = ($108|0)==(0);
   if ($109) {
    $110 = HEAP32[$pPage>>2]|0;
    $111 = HEAP32[$hdr>>2]|0;
    $112 = HEAP32[$pPage>>2]|0;
    $113 = ((($112)) + 56|0);
    $114 = HEAP32[$113>>2]|0;
    $115 = (($114) + ($111)|0);
    $116 = HEAP8[$115>>0]|0;
    $117 = $116&255;
    $118 = $117 | 8;
    _zeroPage($110,$118);
   }
  }
 } while(0);
 $119 = HEAP32[$pPage>>2]|0;
 $120 = ((($119)) + 8|0);
 HEAP8[$120>>0] = 0;
 $121 = HEAP32[$pPage>>2]|0;
 _releasePage($121);
 $122 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $122;
 $123 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _clearCell($pPage,$pCell,$pnSize) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pnSize = $pnSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $iNext = 0, $info = 0, $nOvfl = 0, $ovflPageSize = 0, $ovflPgno = 0, $pBt = 0, $pOvfl = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $3 = sp + 52|0;
 $pBt = sp + 48|0;
 $info = sp;
 $ovflPgno = sp + 44|0;
 $rc = sp + 40|0;
 $nOvfl = sp + 36|0;
 $ovflPageSize = sp + 32|0;
 $iNext = sp + 28|0;
 $pOvfl = sp + 24|0;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $pCell;
 HEAP32[$3>>2] = $pnSize;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 80|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 FUNCTION_TABLE_viii[$9 & 255]($10,$11,$info);
 $12 = ((($info)) + 18|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = HEAP32[$3>>2]|0;
 HEAP16[$14>>1] = $13;
 $15 = ((($info)) + 16|0);
 $16 = HEAP16[$15>>1]|0;
 $17 = $16&65535;
 $18 = ((($info)) + 12|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($17|0)==($19|0);
 if ($20) {
  HEAP32[$0>>2] = 0;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($info)) + 18|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = $23&65535;
 $25 = (($21) + ($24)|0);
 $26 = ((($25)) + -1|0);
 $27 = HEAP32[$1>>2]|0;
 $28 = ((($27)) + 56|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP16[$31>>1]|0;
 $33 = $32&65535;
 $34 = (($29) + ($33)|0);
 $35 = ($26>>>0)>($34>>>0);
 if ($35) {
  $36 = (_sqlite3CorruptError(64249)|0);
  HEAP32[$0>>2] = $36;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 $37 = HEAP32[$2>>2]|0;
 $38 = ((($info)) + 18|0);
 $39 = HEAP16[$38>>1]|0;
 $40 = $39&65535;
 $41 = (($37) + ($40)|0);
 $42 = ((($41)) + -4|0);
 $43 = (_sqlite3Get4byte($42)|0);
 HEAP32[$ovflPgno>>2] = $43;
 $44 = HEAP32[$pBt>>2]|0;
 $45 = ((($44)) + 36|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = (($46) - 4)|0;
 HEAP32[$ovflPageSize>>2] = $47;
 $48 = ((($info)) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ((($info)) + 16|0);
 $51 = HEAP16[$50>>1]|0;
 $52 = $51&65535;
 $53 = (($49) - ($52))|0;
 $54 = HEAP32[$ovflPageSize>>2]|0;
 $55 = (($53) + ($54))|0;
 $56 = (($55) - 1)|0;
 $57 = HEAP32[$ovflPageSize>>2]|0;
 $58 = (($56>>>0) / ($57>>>0))&-1;
 HEAP32[$nOvfl>>2] = $58;
 while(1) {
  $59 = HEAP32[$nOvfl>>2]|0;
  $60 = (($59) + -1)|0;
  HEAP32[$nOvfl>>2] = $60;
  $61 = ($59|0)!=(0);
  if (!($61)) {
   label = 23;
   break;
  }
  HEAP32[$iNext>>2] = 0;
  HEAP32[$pOvfl>>2] = 0;
  $62 = HEAP32[$ovflPgno>>2]|0;
  $63 = ($62>>>0)<(2);
  if ($63) {
   label = 9;
   break;
  }
  $64 = HEAP32[$ovflPgno>>2]|0;
  $65 = HEAP32[$pBt>>2]|0;
  $66 = (_btreePagecount($65)|0);
  $67 = ($64>>>0)>($66>>>0);
  if ($67) {
   label = 9;
   break;
  }
  $69 = HEAP32[$nOvfl>>2]|0;
  $70 = ($69|0)!=(0);
  if ($70) {
   $71 = HEAP32[$pBt>>2]|0;
   $72 = HEAP32[$ovflPgno>>2]|0;
   $73 = (_getOverflowPage($71,$72,$pOvfl,$iNext)|0);
   HEAP32[$rc>>2] = $73;
   $74 = HEAP32[$rc>>2]|0;
   $75 = ($74|0)!=(0);
   if ($75) {
    label = 12;
    break;
   }
  }
  $77 = HEAP32[$pOvfl>>2]|0;
  $78 = ($77|0)!=(0|0);
  if ($78) {
   label = 15;
  } else {
   $79 = HEAP32[$pBt>>2]|0;
   $80 = HEAP32[$ovflPgno>>2]|0;
   $81 = (_btreePageLookup($79,$80)|0);
   HEAP32[$pOvfl>>2] = $81;
   $82 = ($81|0)!=(0|0);
   if ($82) {
    label = 15;
   } else {
    label = 17;
   }
  }
  if ((label|0) == 15) {
   label = 0;
   $83 = HEAP32[$pOvfl>>2]|0;
   $84 = ((($83)) + 72|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = (_sqlite3PagerPageRefcount($85)|0);
   $87 = ($86|0)!=(1);
   if ($87) {
    $88 = (_sqlite3CorruptError(64285)|0);
    HEAP32[$rc>>2] = $88;
   } else {
    label = 17;
   }
  }
  if ((label|0) == 17) {
   label = 0;
   $89 = HEAP32[$pBt>>2]|0;
   $90 = HEAP32[$pOvfl>>2]|0;
   $91 = HEAP32[$ovflPgno>>2]|0;
   $92 = (_freePage2($89,$90,$91)|0);
   HEAP32[$rc>>2] = $92;
  }
  $93 = HEAP32[$pOvfl>>2]|0;
  $94 = ($93|0)!=(0|0);
  if ($94) {
   $95 = HEAP32[$pOvfl>>2]|0;
   $96 = ((($95)) + 72|0);
   $97 = HEAP32[$96>>2]|0;
   _sqlite3PagerUnref($97);
  }
  $98 = HEAP32[$rc>>2]|0;
  $99 = ($98|0)!=(0);
  if ($99) {
   label = 21;
   break;
  }
  $101 = HEAP32[$iNext>>2]|0;
  HEAP32[$ovflPgno>>2] = $101;
 }
 if ((label|0) == 9) {
  $68 = (_sqlite3CorruptError(64265)|0);
  HEAP32[$0>>2] = $68;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 else if ((label|0) == 12) {
  $76 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $76;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 else if ((label|0) == 21) {
  $100 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $100;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 else if ((label|0) == 23) {
  HEAP32[$0>>2] = 0;
  $102 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($102|0);
 }
 return (0)|0;
}
function _freePage($pPage,$pRC) {
 $pPage = $pPage|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $pRC;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 84|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = (_freePage2($7,$8,$11)|0);
 $13 = HEAP32[$1>>2]|0;
 HEAP32[$13>>2] = $12;
 STACKTOP = sp;return;
}
function _zeroPage($pPage,$flags) {
 $pPage = $pPage|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $first = 0;
 var $hdr = 0, $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $data = sp + 4|0;
 $pBt = sp;
 $hdr = sp + 18|0;
 $first = sp + 16|0;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $flags;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$data>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 5|0);
 $10 = HEAP8[$9>>0]|0;
 HEAP8[$hdr>>0] = $10;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 22|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & 4;
 $16 = ($15|0)!=(0);
 if ($16) {
  $17 = HEAP8[$hdr>>0]|0;
  $18 = $17&255;
  $19 = HEAP32[$data>>2]|0;
  $20 = (($19) + ($18)|0);
  $21 = HEAP32[$pBt>>2]|0;
  $22 = ((($21)) + 36|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = HEAP8[$hdr>>0]|0;
  $25 = $24&255;
  $26 = (($23) - ($25))|0;
  _memset(($20|0),0,($26|0))|0;
 }
 $27 = HEAP32[$1>>2]|0;
 $28 = $27&255;
 $29 = HEAP8[$hdr>>0]|0;
 $30 = $29&255;
 $31 = HEAP32[$data>>2]|0;
 $32 = (($31) + ($30)|0);
 HEAP8[$32>>0] = $28;
 $33 = HEAP8[$hdr>>0]|0;
 $34 = $33&255;
 $35 = HEAP32[$1>>2]|0;
 $36 = $35 & 8;
 $37 = ($36|0)==(0);
 $38 = $37 ? 12 : 8;
 $39 = (($34) + ($38))|0;
 $40 = $39&65535;
 HEAP16[$first>>1] = $40;
 $41 = HEAP8[$hdr>>0]|0;
 $42 = $41&255;
 $43 = (($42) + 1)|0;
 $44 = HEAP32[$data>>2]|0;
 $45 = (($44) + ($43)|0);
 ;HEAP8[$45>>0]=0|0;HEAP8[$45+1>>0]=0|0;HEAP8[$45+2>>0]=0|0;HEAP8[$45+3>>0]=0|0;
 $46 = HEAP8[$hdr>>0]|0;
 $47 = $46&255;
 $48 = (($47) + 7)|0;
 $49 = HEAP32[$data>>2]|0;
 $50 = (($49) + ($48)|0);
 HEAP8[$50>>0] = 0;
 $51 = HEAP32[$pBt>>2]|0;
 $52 = ((($51)) + 36|0);
 $53 = HEAP32[$52>>2]|0;
 $54 = $53 >>> 8;
 $55 = $54&255;
 $56 = HEAP8[$hdr>>0]|0;
 $57 = $56&255;
 $58 = (($57) + 5)|0;
 $59 = HEAP32[$data>>2]|0;
 $60 = (($59) + ($58)|0);
 HEAP8[$60>>0] = $55;
 $61 = HEAP32[$pBt>>2]|0;
 $62 = ((($61)) + 36|0);
 $63 = HEAP32[$62>>2]|0;
 $64 = $63&255;
 $65 = HEAP8[$hdr>>0]|0;
 $66 = $65&255;
 $67 = (($66) + 5)|0;
 $68 = HEAP32[$data>>2]|0;
 $69 = (($68) + ($67)|0);
 $70 = ((($69)) + 1|0);
 HEAP8[$70>>0] = $64;
 $71 = HEAP32[$pBt>>2]|0;
 $72 = ((($71)) + 36|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = HEAP16[$first>>1]|0;
 $75 = $74&65535;
 $76 = (($73) - ($75))|0;
 $77 = $76&65535;
 $78 = HEAP32[$0>>2]|0;
 $79 = ((($78)) + 16|0);
 HEAP16[$79>>1] = $77;
 $80 = HEAP32[$0>>2]|0;
 $81 = HEAP32[$1>>2]|0;
 (_decodeFlags($80,$81)|0);
 $82 = HEAP16[$first>>1]|0;
 $83 = HEAP32[$0>>2]|0;
 $84 = ((($83)) + 14|0);
 HEAP16[$84>>1] = $82;
 $85 = HEAP32[$pBt>>2]|0;
 $86 = ((($85)) + 36|0);
 $87 = HEAP32[$86>>2]|0;
 $88 = HEAP32[$data>>2]|0;
 $89 = (($88) + ($87)|0);
 $90 = HEAP32[$0>>2]|0;
 $91 = ((($90)) + 60|0);
 HEAP32[$91>>2] = $89;
 $92 = HEAP16[$first>>1]|0;
 $93 = $92&65535;
 $94 = HEAP32[$data>>2]|0;
 $95 = (($94) + ($93)|0);
 $96 = HEAP32[$0>>2]|0;
 $97 = ((($96)) + 64|0);
 HEAP32[$97>>2] = $95;
 $98 = HEAP32[$0>>2]|0;
 $99 = ((($98)) + 6|0);
 $100 = HEAP8[$99>>0]|0;
 $101 = $100&255;
 $102 = HEAP32[$data>>2]|0;
 $103 = (($102) + ($101)|0);
 $104 = HEAP32[$0>>2]|0;
 $105 = ((($104)) + 68|0);
 HEAP32[$105>>2] = $103;
 $106 = HEAP32[$0>>2]|0;
 $107 = ((($106)) + 1|0);
 HEAP8[$107>>0] = 0;
 $108 = HEAP32[$pBt>>2]|0;
 $109 = ((($108)) + 32|0);
 $110 = HEAP32[$109>>2]|0;
 $111 = (($110) - 1)|0;
 $112 = $111&65535;
 $113 = HEAP32[$0>>2]|0;
 $114 = ((($113)) + 20|0);
 HEAP16[$114>>1] = $112;
 $115 = HEAP32[$0>>2]|0;
 $116 = ((($115)) + 18|0);
 HEAP16[$116>>1] = 0;
 $117 = HEAP32[$0>>2]|0;
 HEAP8[$117>>0] = 1;
 STACKTOP = sp;return;
}
function _freePage2($pBt,$pMemPage,$iPage) {
 $pBt = $pBt|0;
 $pMemPage = $pMemPage|0;
 $iPage = $iPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iTrunk = 0, $nFree = 0, $nLeaf = 0, $pPage = 0, $pPage1 = 0, $pTrunk = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $pTrunk = sp + 24|0;
 $iTrunk = sp + 20|0;
 $pPage1 = sp + 16|0;
 $pPage = sp + 12|0;
 $rc = sp + 8|0;
 $nFree = sp + 4|0;
 $nLeaf = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pMemPage;
 HEAP32[$3>>2] = $iPage;
 HEAP32[$pTrunk>>2] = 0;
 HEAP32[$iTrunk>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pPage1>>2] = $6;
 $7 = HEAP32[$3>>2]|0;
 $8 = ($7>>>0)<(2);
 if ($8) {
  $9 = (_sqlite3CorruptError(64103)|0);
  HEAP32[$0>>2] = $9;
  $166 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($166|0);
 }
 $10 = HEAP32[$2>>2]|0;
 $11 = ($10|0)!=(0|0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  HEAP32[$pPage>>2] = $12;
  $13 = HEAP32[$pPage>>2]|0;
  $14 = ((($13)) + 72|0);
  $15 = HEAP32[$14>>2]|0;
  _sqlite3PagerRef($15);
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  $18 = (_btreePageLookup($16,$17)|0);
  HEAP32[$pPage>>2] = $18;
 }
 $19 = HEAP32[$pPage1>>2]|0;
 $20 = ((($19)) + 72|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (_sqlite3PagerWrite($21)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)!=(0);
 do {
  if (!($24)) {
   $25 = HEAP32[$pPage1>>2]|0;
   $26 = ((($25)) + 56|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ((($27)) + 36|0);
   $29 = (_sqlite3Get4byte($28)|0);
   HEAP32[$nFree>>2] = $29;
   $30 = HEAP32[$pPage1>>2]|0;
   $31 = ((($30)) + 56|0);
   $32 = HEAP32[$31>>2]|0;
   $33 = ((($32)) + 36|0);
   $34 = HEAP32[$nFree>>2]|0;
   $35 = (($34) + 1)|0;
   _sqlite3Put4byte($33,$35);
   $36 = HEAP32[$1>>2]|0;
   $37 = ((($36)) + 22|0);
   $38 = HEAP16[$37>>1]|0;
   $39 = $38&65535;
   $40 = $39 & 4;
   $41 = ($40|0)!=(0);
   if ($41) {
    $42 = HEAP32[$pPage>>2]|0;
    $43 = ($42|0)!=(0|0);
    if (!($43)) {
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP32[$3>>2]|0;
     $46 = (_btreeGetPage($44,$45,$pPage,0)|0);
     HEAP32[$rc>>2] = $46;
     $47 = ($46|0)!=(0);
     if ($47) {
      break;
     }
    }
    $48 = HEAP32[$pPage>>2]|0;
    $49 = ((($48)) + 72|0);
    $50 = HEAP32[$49>>2]|0;
    $51 = (_sqlite3PagerWrite($50)|0);
    HEAP32[$rc>>2] = $51;
    $52 = ($51|0)!=(0);
    if ($52) {
     break;
    }
    $53 = HEAP32[$pPage>>2]|0;
    $54 = ((($53)) + 56|0);
    $55 = HEAP32[$54>>2]|0;
    $56 = HEAP32[$pPage>>2]|0;
    $57 = ((($56)) + 52|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = ((($58)) + 32|0);
    $60 = HEAP32[$59>>2]|0;
    _memset(($55|0),0,($60|0))|0;
   }
   $61 = HEAP32[$1>>2]|0;
   $62 = ((($61)) + 17|0);
   $63 = HEAP8[$62>>0]|0;
   $64 = ($63<<24>>24)!=(0);
   if ($64) {
    $65 = HEAP32[$1>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    _ptrmapPut($65,$66,2,0,$rc);
    $67 = HEAP32[$rc>>2]|0;
    $68 = ($67|0)!=(0);
    if ($68) {
     break;
    }
   }
   $69 = HEAP32[$nFree>>2]|0;
   $70 = ($69|0)!=(0);
   if ($70) {
    $71 = HEAP32[$pPage1>>2]|0;
    $72 = ((($71)) + 56|0);
    $73 = HEAP32[$72>>2]|0;
    $74 = ((($73)) + 32|0);
    $75 = (_sqlite3Get4byte($74)|0);
    HEAP32[$iTrunk>>2] = $75;
    $76 = HEAP32[$1>>2]|0;
    $77 = HEAP32[$iTrunk>>2]|0;
    $78 = (_btreeGetPage($76,$77,$pTrunk,0)|0);
    HEAP32[$rc>>2] = $78;
    $79 = HEAP32[$rc>>2]|0;
    $80 = ($79|0)!=(0);
    if ($80) {
     break;
    }
    $81 = HEAP32[$pTrunk>>2]|0;
    $82 = ((($81)) + 56|0);
    $83 = HEAP32[$82>>2]|0;
    $84 = ((($83)) + 4|0);
    $85 = (_sqlite3Get4byte($84)|0);
    HEAP32[$nLeaf>>2] = $85;
    $86 = HEAP32[$nLeaf>>2]|0;
    $87 = HEAP32[$1>>2]|0;
    $88 = ((($87)) + 36|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = (($89>>>0) / 4)&-1;
    $91 = (($90) - 2)|0;
    $92 = ($86>>>0)>($91>>>0);
    if ($92) {
     $93 = (_sqlite3CorruptError(64156)|0);
     HEAP32[$rc>>2] = $93;
     break;
    }
    $94 = HEAP32[$nLeaf>>2]|0;
    $95 = HEAP32[$1>>2]|0;
    $96 = ((($95)) + 36|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = (($97>>>0) / 4)&-1;
    $99 = (($98) - 8)|0;
    $100 = ($94>>>0)<($99>>>0);
    if ($100) {
     $101 = HEAP32[$pTrunk>>2]|0;
     $102 = ((($101)) + 72|0);
     $103 = HEAP32[$102>>2]|0;
     $104 = (_sqlite3PagerWrite($103)|0);
     HEAP32[$rc>>2] = $104;
     $105 = HEAP32[$rc>>2]|0;
     $106 = ($105|0)==(0);
     if (!($106)) {
      break;
     }
     $107 = HEAP32[$pTrunk>>2]|0;
     $108 = ((($107)) + 56|0);
     $109 = HEAP32[$108>>2]|0;
     $110 = ((($109)) + 4|0);
     $111 = HEAP32[$nLeaf>>2]|0;
     $112 = (($111) + 1)|0;
     _sqlite3Put4byte($110,$112);
     $113 = HEAP32[$nLeaf>>2]|0;
     $114 = $113<<2;
     $115 = (8 + ($114))|0;
     $116 = HEAP32[$pTrunk>>2]|0;
     $117 = ((($116)) + 56|0);
     $118 = HEAP32[$117>>2]|0;
     $119 = (($118) + ($115)|0);
     $120 = HEAP32[$3>>2]|0;
     _sqlite3Put4byte($119,$120);
     $121 = HEAP32[$pPage>>2]|0;
     $122 = ($121|0)!=(0|0);
     if ($122) {
      $123 = HEAP32[$1>>2]|0;
      $124 = ((($123)) + 22|0);
      $125 = HEAP16[$124>>1]|0;
      $126 = $125&65535;
      $127 = $126 & 4;
      $128 = ($127|0)==(0);
      if ($128) {
       $129 = HEAP32[$pPage>>2]|0;
       $130 = ((($129)) + 72|0);
       $131 = HEAP32[$130>>2]|0;
       _sqlite3PagerDontWrite($131);
      }
     }
     $132 = HEAP32[$1>>2]|0;
     $133 = HEAP32[$3>>2]|0;
     $134 = (_btreeSetHasContent($132,$133)|0);
     HEAP32[$rc>>2] = $134;
     break;
    }
   }
   $135 = HEAP32[$pPage>>2]|0;
   $136 = ($135|0)==(0|0);
   if ($136) {
    $137 = HEAP32[$1>>2]|0;
    $138 = HEAP32[$3>>2]|0;
    $139 = (_btreeGetPage($137,$138,$pPage,0)|0);
    HEAP32[$rc>>2] = $139;
    $140 = (0)!=($139|0);
    if ($140) {
     break;
    }
   }
   $141 = HEAP32[$pPage>>2]|0;
   $142 = ((($141)) + 72|0);
   $143 = HEAP32[$142>>2]|0;
   $144 = (_sqlite3PagerWrite($143)|0);
   HEAP32[$rc>>2] = $144;
   $145 = HEAP32[$rc>>2]|0;
   $146 = ($145|0)!=(0);
   if (!($146)) {
    $147 = HEAP32[$pPage>>2]|0;
    $148 = ((($147)) + 56|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = HEAP32[$iTrunk>>2]|0;
    _sqlite3Put4byte($149,$150);
    $151 = HEAP32[$pPage>>2]|0;
    $152 = ((($151)) + 56|0);
    $153 = HEAP32[$152>>2]|0;
    $154 = ((($153)) + 4|0);
    _sqlite3Put4byte($154,0);
    $155 = HEAP32[$pPage1>>2]|0;
    $156 = ((($155)) + 56|0);
    $157 = HEAP32[$156>>2]|0;
    $158 = ((($157)) + 32|0);
    $159 = HEAP32[$3>>2]|0;
    _sqlite3Put4byte($158,$159);
   }
  }
 } while(0);
 $160 = HEAP32[$pPage>>2]|0;
 $161 = ($160|0)!=(0|0);
 if ($161) {
  $162 = HEAP32[$pPage>>2]|0;
  HEAP8[$162>>0] = 0;
 }
 $163 = HEAP32[$pPage>>2]|0;
 _releasePage($163);
 $164 = HEAP32[$pTrunk>>2]|0;
 _releasePage($164);
 $165 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $165;
 $166 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($166|0);
}
function _sqlite3PagerRef($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3PcacheRef($1);
 STACKTOP = sp;return;
}
function _btreePageLookup($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDbPage = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pDbPage = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$2>>2] = $pgno;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_sqlite3PagerLookup($4,$5)|0);
 HEAP32[$pDbPage>>2] = $6;
 $7 = HEAP32[$pDbPage>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$pDbPage>>2]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = (_btreePageFromDbPage($9,$10,$11)|0);
  HEAP32[$0>>2] = $12;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 } else {
  HEAP32[$0>>2] = 0;
  $13 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($13|0);
 }
 return (0)|0;
}
function _sqlite3PagerDontWrite($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pPager = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pPager = sp;
 HEAP32[$0>>2] = $pPg;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 13|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = ($6<<24>>24)!=(0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP16[$9>>1]|0;
 $11 = $10&65535;
 $12 = $11 & 2;
 $13 = ($12|0)!=(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$pPager>>2]|0;
 $15 = ((($14)) + 104|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)==(0);
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 24|0);
 $20 = HEAP16[$19>>1]|0;
 $21 = $20&65535;
 $22 = $21 | 16;
 $23 = $22&65535;
 HEAP16[$19>>1] = $23;
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 24|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & -5;
 $29 = $28&65535;
 HEAP16[$25>>1] = $29;
 STACKTOP = sp;return;
}
function _btreeSetHasContent($pBt,$pgno) {
 $pBt = $pBt|0;
 $pgno = $pgno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pBt;
 HEAP32[$1>>2] = $pgno;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3BitvecCreate($8)|0);
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 60|0);
  HEAP32[$11>>2] = $9;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 60|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)!=(0|0);
  if (!($15)) {
   HEAP32[$rc>>2] = 7;
  }
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if (!($17)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 60|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = (_sqlite3BitvecSize($21)|0);
 $23 = ($18>>>0)<=($22>>>0);
 if (!($23)) {
  $29 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $24 = HEAP32[$0>>2]|0;
 $25 = ((($24)) + 60|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$1>>2]|0;
 $28 = (_sqlite3BitvecSet($26,$27)|0);
 HEAP32[$rc>>2] = $28;
 $29 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _sqlite3PcacheRef($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 26|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = (($3) + 1)<<16>>16;
 HEAP16[$2>>1] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 28|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 12|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + 1)|0;
 HEAP32[$8>>2] = $10;
 STACKTOP = sp;return;
}
function _btreeDropTable($p,$iTable,$piMoved) {
 $p = $p|0;
 $iTable = $iTable|0;
 $piMoved = $piMoved|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $9 = 0, $maxRootPgno = 0, $pBt = 0, $pMove = 0, $pPage = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $rc = sp + 16|0;
 $pPage = sp + 12|0;
 $pBt = sp + 8|0;
 $maxRootPgno = sp + 4|0;
 $pMove = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $piMoved;
 HEAP32[$pPage>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)!=(0|0);
 if ($10) {
  HEAP32[$0>>2] = 262;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $11 = HEAP32[$2>>2]|0;
 $12 = ($11>>>0)<(2);
 if ($12) {
  $13 = (_sqlite3CorruptError(66814)|0);
  HEAP32[$0>>2] = $13;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $14 = HEAP32[$pBt>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = (_btreeGetPage($14,$15,$pPage,0)|0);
 HEAP32[$rc>>2] = $16;
 $17 = HEAP32[$rc>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $19;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $20 = HEAP32[$1>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = (_sqlite3BtreeClearTable($20,$21,0)|0);
 HEAP32[$rc>>2] = $22;
 $23 = HEAP32[$rc>>2]|0;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$pPage>>2]|0;
  _releasePage($25);
  $26 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $26;
  $88 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($88|0);
 }
 $27 = HEAP32[$3>>2]|0;
 HEAP32[$27>>2] = 0;
 $28 = HEAP32[$pBt>>2]|0;
 $29 = ((($28)) + 17|0);
 $30 = HEAP8[$29>>0]|0;
 $31 = ($30<<24>>24)!=(0);
 if ($31) {
  $32 = HEAP32[$1>>2]|0;
  _sqlite3BtreeGetMeta($32,4,$maxRootPgno);
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$maxRootPgno>>2]|0;
  $35 = ($33|0)==($34|0);
  $36 = HEAP32[$pPage>>2]|0;
  do {
   if ($35) {
    _freePage($36,$rc);
    $37 = HEAP32[$pPage>>2]|0;
    _releasePage($37);
    $38 = HEAP32[$rc>>2]|0;
    $39 = ($38|0)!=(0);
    if ($39) {
     $40 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $40;
     $88 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($88|0);
    }
   } else {
    _releasePage($36);
    $41 = HEAP32[$pBt>>2]|0;
    $42 = HEAP32[$maxRootPgno>>2]|0;
    $43 = (_btreeGetPage($41,$42,$pMove,0)|0);
    HEAP32[$rc>>2] = $43;
    $44 = HEAP32[$rc>>2]|0;
    $45 = ($44|0)!=(0);
    if ($45) {
     $46 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $46;
     $88 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($88|0);
    }
    $47 = HEAP32[$pBt>>2]|0;
    $48 = HEAP32[$pMove>>2]|0;
    $49 = HEAP32[$2>>2]|0;
    $50 = (_relocatePage($47,$48,1,0,$49,0)|0);
    HEAP32[$rc>>2] = $50;
    $51 = HEAP32[$pMove>>2]|0;
    _releasePage($51);
    $52 = HEAP32[$rc>>2]|0;
    $53 = ($52|0)!=(0);
    if ($53) {
     $54 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $54;
     $88 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($88|0);
    }
    HEAP32[$pMove>>2] = 0;
    $55 = HEAP32[$pBt>>2]|0;
    $56 = HEAP32[$maxRootPgno>>2]|0;
    $57 = (_btreeGetPage($55,$56,$pMove,0)|0);
    HEAP32[$rc>>2] = $57;
    $58 = HEAP32[$pMove>>2]|0;
    _freePage($58,$rc);
    $59 = HEAP32[$pMove>>2]|0;
    _releasePage($59);
    $60 = HEAP32[$rc>>2]|0;
    $61 = ($60|0)!=(0);
    if (!($61)) {
     $63 = HEAP32[$maxRootPgno>>2]|0;
     $64 = HEAP32[$3>>2]|0;
     HEAP32[$64>>2] = $63;
     break;
    }
    $62 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $62;
    $88 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($88|0);
   }
  } while(0);
  $65 = HEAP32[$maxRootPgno>>2]|0;
  $66 = (($65) + -1)|0;
  HEAP32[$maxRootPgno>>2] = $66;
  while(1) {
   $67 = HEAP32[$maxRootPgno>>2]|0;
   $68 = HEAP32[1536>>2]|0;
   $69 = HEAP32[$pBt>>2]|0;
   $70 = ((($69)) + 32|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($68>>>0) / ($71>>>0))&-1;
   $73 = (($72) + 1)|0;
   $74 = ($67|0)==($73|0);
   if (!($74)) {
    $75 = HEAP32[$pBt>>2]|0;
    $76 = HEAP32[$maxRootPgno>>2]|0;
    $77 = (_ptrmapPageno($75,$76)|0);
    $78 = HEAP32[$maxRootPgno>>2]|0;
    $79 = ($77|0)==($78|0);
    if (!($79)) {
     break;
    }
   }
   $80 = HEAP32[$maxRootPgno>>2]|0;
   $81 = (($80) + -1)|0;
   HEAP32[$maxRootPgno>>2] = $81;
  }
  $82 = HEAP32[$1>>2]|0;
  $83 = HEAP32[$maxRootPgno>>2]|0;
  $84 = (_sqlite3BtreeUpdateMeta($82,4,$83)|0);
  HEAP32[$rc>>2] = $84;
 } else {
  $85 = HEAP32[$pPage>>2]|0;
  _freePage($85,$rc);
  $86 = HEAP32[$pPage>>2]|0;
  _releasePage($86);
 }
 $87 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $87;
 $88 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($88|0);
}
function _vdbeSorterFlushPMA($pSorter) {
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pSorter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 56|0);
 HEAP8[$2>>0] = 1;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 64|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = (_vdbeSorterListToPMA($4,$6)|0);
 STACKTOP = sp;return ($7|0);
}
function _vdbeSorterListToPMA($pTask,$pList) {
 $pTask = $pTask|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $8 = 0, $9 = 0, $db = 0, $p = 0, $pNext = 0, $rc = 0, $writer = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $db = sp + 52|0;
 $rc = sp + 48|0;
 $writer = sp;
 $p = sp + 44|0;
 $pNext = sp + 40|0;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pList;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$db>>2] = $6;
 HEAP32[$rc>>2] = 0;
 dest=$writer; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 40|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$db>>2]|0;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 40|0);
  $14 = (_vdbeSorterOpenTempFile($11,0,0,$13)|0);
  HEAP32[$rc>>2] = $14;
 }
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = HEAP32[$0>>2]|0;
  $18 = HEAP32[$1>>2]|0;
  $19 = (_vdbeSorterSort($17,$18)|0);
  HEAP32[$rc>>2] = $19;
 }
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)==(0);
 if (!($21)) {
  $74 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($74|0);
 }
 HEAP32[$pNext>>2] = 0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 40|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 8|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 40|0);
 $32 = ((($31)) + 8|0);
 $33 = $32;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 $36 = (($33) + 4)|0;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 _vdbePmaWriterInit($24,$writer,$29,$35,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 28|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = (($41) + 1)|0;
 HEAP32[$40>>2] = $42;
 $43 = HEAP32[$1>>2]|0;
 $44 = ((($43)) + 8|0);
 $45 = HEAP32[$44>>2]|0;
 $46 = ($45|0)<(0);
 $47 = $46 << 31 >> 31;
 _vdbePmaWriteVarint($writer,$45,$47);
 $48 = HEAP32[$1>>2]|0;
 $49 = HEAP32[$48>>2]|0;
 HEAP32[$p>>2] = $49;
 while(1) {
  $50 = HEAP32[$p>>2]|0;
  $51 = ($50|0)!=(0|0);
  $52 = HEAP32[$p>>2]|0;
  if (!($51)) {
   break;
  }
  $53 = ((($52)) + 4|0);
  $54 = HEAP32[$53>>2]|0;
  HEAP32[$pNext>>2] = $54;
  $55 = HEAP32[$p>>2]|0;
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)<(0);
  $58 = $57 << 31 >> 31;
  _vdbePmaWriteVarint($writer,$56,$58);
  $59 = HEAP32[$p>>2]|0;
  $60 = ((($59)) + 8|0);
  $61 = HEAP32[$p>>2]|0;
  $62 = HEAP32[$61>>2]|0;
  _vdbePmaWriteBlob($writer,$60,$62);
  $63 = HEAP32[$1>>2]|0;
  $64 = ((($63)) + 4|0);
  $65 = HEAP32[$64>>2]|0;
  $66 = ($65|0)==(0|0);
  if ($66) {
   $67 = HEAP32[$p>>2]|0;
   _sqlite3_free($67);
  }
  $68 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $68;
 }
 $69 = HEAP32[$1>>2]|0;
 HEAP32[$69>>2] = $52;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 40|0);
 $72 = ((($71)) + 8|0);
 $73 = (_vdbePmaWriterFinish($writer,$72)|0);
 HEAP32[$rc>>2] = $73;
 $74 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($74|0);
}
function _vdbeSorterOpenTempFile($db,$0,$1,$ppFd) {
 $db = $db|0;
 $0 = $0|0;
 $1 = $1|0;
 $ppFd = $ppFd|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $max = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 8|0;
 $5 = sp + 20|0;
 $rc = sp + 16|0;
 $max = sp;
 HEAP32[$3>>2] = $db;
 $6 = $4;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$5>>2] = $ppFd;
 $10 = (_sqlite3FaultSim(202)|0);
 $11 = ($10|0)!=(0);
 if ($11) {
  HEAP32[$2>>2] = 3338;
  $25 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = HEAP32[$5>>2]|0;
 $15 = (_sqlite3OsOpenMalloc($13,0,$14,4126,$rc)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = $max;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$5>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  _sqlite3OsFileControlHint($23,18,$max);
 }
 $24 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $24;
 $25 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($25|0);
}
function _vdbeSorterSort($pTask,$pList) {
 $pTask = $pTask|0;
 $pList = $pList|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $aSlot = 0, $i = 0, $p = 0, $pNext = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $aSlot = sp + 12|0;
 $p = sp + 8|0;
 $rc = sp + 4|0;
 $pNext = sp;
 HEAP32[$1>>2] = $pTask;
 HEAP32[$2>>2] = $pList;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_vdbeSortAllocUnpacked($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$p>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_vdbeSorterGetCompare($12)|0);
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 32|0);
 HEAP32[$15>>2] = $13;
 $16 = (_sqlite3MallocZero(256,0)|0);
 HEAP32[$aSlot>>2] = $16;
 $17 = HEAP32[$aSlot>>2]|0;
 $18 = ($17|0)!=(0|0);
 if (!($18)) {
  HEAP32[$0>>2] = 7;
  $95 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($95|0);
 }
 while(1) {
  $19 = HEAP32[$p>>2]|0;
  $20 = ($19|0)!=(0|0);
  if (!($20)) {
   break;
  }
  $21 = HEAP32[$2>>2]|0;
  $22 = ((($21)) + 4|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = ($23|0)!=(0|0);
  $25 = HEAP32[$p>>2]|0;
  do {
   if ($24) {
    $26 = HEAP32[$2>>2]|0;
    $27 = ((($26)) + 4|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = ($25|0)==($28|0);
    if ($29) {
     HEAP32[$pNext>>2] = 0;
     break;
    } else {
     $30 = HEAP32[$p>>2]|0;
     $31 = ((($30)) + 4|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = HEAP32[$2>>2]|0;
     $34 = ((($33)) + 4|0);
     $35 = HEAP32[$34>>2]|0;
     $36 = (($35) + ($32)|0);
     HEAP32[$pNext>>2] = $36;
     break;
    }
   } else {
    $37 = ((($25)) + 4|0);
    $38 = HEAP32[$37>>2]|0;
    HEAP32[$pNext>>2] = $38;
   }
  } while(0);
  $39 = HEAP32[$p>>2]|0;
  $40 = ((($39)) + 4|0);
  HEAP32[$40>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $41 = HEAP32[$i>>2]|0;
   $42 = HEAP32[$aSlot>>2]|0;
   $43 = (($42) + ($41<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)!=(0|0);
   if (!($45)) {
    break;
   }
   $46 = HEAP32[$1>>2]|0;
   $47 = HEAP32[$p>>2]|0;
   $48 = HEAP32[$i>>2]|0;
   $49 = HEAP32[$aSlot>>2]|0;
   $50 = (($49) + ($48<<2)|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = (_vdbeSorterMerge($46,$47,$51)|0);
   HEAP32[$p>>2] = $52;
   $53 = HEAP32[$i>>2]|0;
   $54 = HEAP32[$aSlot>>2]|0;
   $55 = (($54) + ($53<<2)|0);
   HEAP32[$55>>2] = 0;
   $56 = HEAP32[$i>>2]|0;
   $57 = (($56) + 1)|0;
   HEAP32[$i>>2] = $57;
  }
  $58 = HEAP32[$p>>2]|0;
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$aSlot>>2]|0;
  $61 = (($60) + ($59<<2)|0);
  HEAP32[$61>>2] = $58;
  $62 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $62;
 }
 HEAP32[$p>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $63 = HEAP32[$i>>2]|0;
  $64 = ($63|0)<(64);
  if (!($64)) {
   break;
  }
  $65 = HEAP32[$i>>2]|0;
  $66 = HEAP32[$aSlot>>2]|0;
  $67 = (($66) + ($65<<2)|0);
  $68 = HEAP32[$67>>2]|0;
  $69 = ($68|0)==(0|0);
  if (!($69)) {
   $70 = HEAP32[$p>>2]|0;
   $71 = ($70|0)!=(0|0);
   if ($71) {
    $72 = HEAP32[$1>>2]|0;
    $73 = HEAP32[$p>>2]|0;
    $74 = HEAP32[$i>>2]|0;
    $75 = HEAP32[$aSlot>>2]|0;
    $76 = (($75) + ($74<<2)|0);
    $77 = HEAP32[$76>>2]|0;
    $78 = (_vdbeSorterMerge($72,$73,$77)|0);
    $83 = $78;
   } else {
    $79 = HEAP32[$i>>2]|0;
    $80 = HEAP32[$aSlot>>2]|0;
    $81 = (($80) + ($79<<2)|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = $82;
   }
   HEAP32[$p>>2] = $83;
  }
  $84 = HEAP32[$i>>2]|0;
  $85 = (($84) + 1)|0;
  HEAP32[$i>>2] = $85;
 }
 $86 = HEAP32[$p>>2]|0;
 $87 = HEAP32[$2>>2]|0;
 HEAP32[$87>>2] = $86;
 $88 = HEAP32[$aSlot>>2]|0;
 _sqlite3_free($88);
 $89 = HEAP32[$1>>2]|0;
 $90 = ((($89)) + 12|0);
 $91 = HEAP32[$90>>2]|0;
 $92 = ((($91)) + 11|0);
 $93 = HEAP8[$92>>0]|0;
 $94 = $93&255;
 HEAP32[$0>>2] = $94;
 $95 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($95|0);
}
function _vdbePmaWriterInit($pFd,$p,$nBuf,$0,$1) {
 $pFd = $pFd|0;
 $p = $p|0;
 $nBuf = $nBuf|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0, $8 = 0, $9 = 0;
 var dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp;
 HEAP32[$2>>2] = $pFd;
 HEAP32[$3>>2] = $p;
 HEAP32[$4>>2] = $nBuf;
 $6 = $5;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 $10 = HEAP32[$3>>2]|0;
 dest=$10; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $11 = HEAP32[$4>>2]|0;
 $12 = ($11|0)<(0);
 $13 = $12 << 31 >> 31;
 $14 = (_sqlite3Malloc($11,$13)|0);
 $15 = HEAP32[$3>>2]|0;
 $16 = ((($15)) + 4|0);
 HEAP32[$16>>2] = $14;
 $17 = HEAP32[$3>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $22 = $5;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (($22) + 4)|0;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$4>>2]|0;
  $29 = ($28|0)<(0);
  $30 = $29 << 31 >> 31;
  $31 = (___remdi3(($24|0),($27|0),($28|0),($30|0))|0);
  $32 = tempRet0;
  $33 = HEAP32[$3>>2]|0;
  $34 = ((($33)) + 12|0);
  HEAP32[$34>>2] = $31;
  $35 = HEAP32[$3>>2]|0;
  $36 = ((($35)) + 16|0);
  HEAP32[$36>>2] = $31;
  $37 = $5;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$3>>2]|0;
  $44 = ((($43)) + 12|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = ($45|0)<(0);
  $47 = $46 << 31 >> 31;
  $48 = (_i64Subtract(($39|0),($42|0),($45|0),($47|0))|0);
  $49 = tempRet0;
  $50 = HEAP32[$3>>2]|0;
  $51 = ((($50)) + 24|0);
  $52 = $51;
  $53 = $52;
  HEAP32[$53>>2] = $48;
  $54 = (($52) + 4)|0;
  $55 = $54;
  HEAP32[$55>>2] = $49;
  $56 = HEAP32[$4>>2]|0;
  $57 = HEAP32[$3>>2]|0;
  $58 = ((($57)) + 8|0);
  HEAP32[$58>>2] = $56;
  $59 = HEAP32[$2>>2]|0;
  $60 = HEAP32[$3>>2]|0;
  $61 = ((($60)) + 32|0);
  HEAP32[$61>>2] = $59;
  STACKTOP = sp;return;
 } else {
  $21 = HEAP32[$3>>2]|0;
  HEAP32[$21>>2] = 7;
  STACKTOP = sp;return;
 }
}
function _vdbePmaWriteVarint($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aByte = 0, $nByte = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 12|0;
 $3 = sp;
 $nByte = sp + 8|0;
 $aByte = sp + 16|0;
 HEAP32[$2>>2] = $p;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = $3;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3PutVarint($aByte,$10,$13)|0);
 HEAP32[$nByte>>2] = $14;
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$nByte>>2]|0;
 _vdbePmaWriteBlob($15,$aByte,$16);
 STACKTOP = sp;return;
}
function _vdbePmaWriteBlob($p,$pData,$nData) {
 $p = $p|0;
 $pData = $pData|0;
 $nData = $nData|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nCopy = 0, $nRem = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $nRem = sp + 4|0;
 $nCopy = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $pData;
 HEAP32[$2>>2] = $nData;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$nRem>>2] = $3;
 while(1) {
  $4 = HEAP32[$nRem>>2]|0;
  $5 = ($4|0)>(0);
  if (!($5)) {
   label = 9;
   break;
  }
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 9;
   break;
  }
  $9 = HEAP32[$nRem>>2]|0;
  HEAP32[$nCopy>>2] = $9;
  $10 = HEAP32[$nCopy>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($13) - ($16))|0;
  $18 = ($10|0)>($17|0);
  if ($18) {
   $19 = HEAP32[$0>>2]|0;
   $20 = ((($19)) + 8|0);
   $21 = HEAP32[$20>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 16|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($21) - ($24))|0;
   HEAP32[$nCopy>>2] = $25;
  }
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 16|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$0>>2]|0;
  $30 = ((($29)) + 4|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = (($31) + ($28)|0);
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$nRem>>2]|0;
  $35 = (($33) - ($34))|0;
  $36 = HEAP32[$1>>2]|0;
  $37 = (($36) + ($35)|0);
  $38 = HEAP32[$nCopy>>2]|0;
  _memcpy(($32|0),($37|0),($38|0))|0;
  $39 = HEAP32[$nCopy>>2]|0;
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 16|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = (($42) + ($39))|0;
  HEAP32[$41>>2] = $43;
  $44 = HEAP32[$0>>2]|0;
  $45 = ((($44)) + 16|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 8|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = ($46|0)==($49|0);
  if ($50) {
   $51 = HEAP32[$0>>2]|0;
   $52 = ((($51)) + 32|0);
   $53 = HEAP32[$52>>2]|0;
   $54 = HEAP32[$0>>2]|0;
   $55 = ((($54)) + 12|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$0>>2]|0;
   $58 = ((($57)) + 4|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = (($59) + ($56)|0);
   $61 = HEAP32[$0>>2]|0;
   $62 = ((($61)) + 16|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$0>>2]|0;
   $65 = ((($64)) + 12|0);
   $66 = HEAP32[$65>>2]|0;
   $67 = (($63) - ($66))|0;
   $68 = HEAP32[$0>>2]|0;
   $69 = ((($68)) + 24|0);
   $70 = $69;
   $71 = $70;
   $72 = HEAP32[$71>>2]|0;
   $73 = (($70) + 4)|0;
   $74 = $73;
   $75 = HEAP32[$74>>2]|0;
   $76 = HEAP32[$0>>2]|0;
   $77 = ((($76)) + 12|0);
   $78 = HEAP32[$77>>2]|0;
   $79 = ($78|0)<(0);
   $80 = $79 << 31 >> 31;
   $81 = (_i64Add(($72|0),($75|0),($78|0),($80|0))|0);
   $82 = tempRet0;
   $83 = (_sqlite3OsWrite($53,$60,$67,$81,$82)|0);
   $84 = HEAP32[$0>>2]|0;
   HEAP32[$84>>2] = $83;
   $85 = HEAP32[$0>>2]|0;
   $86 = ((($85)) + 16|0);
   HEAP32[$86>>2] = 0;
   $87 = HEAP32[$0>>2]|0;
   $88 = ((($87)) + 12|0);
   HEAP32[$88>>2] = 0;
   $89 = HEAP32[$0>>2]|0;
   $90 = ((($89)) + 8|0);
   $91 = HEAP32[$90>>2]|0;
   $92 = ($91|0)<(0);
   $93 = $92 << 31 >> 31;
   $94 = HEAP32[$0>>2]|0;
   $95 = ((($94)) + 24|0);
   $96 = $95;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = (($96) + 4)|0;
   $100 = $99;
   $101 = HEAP32[$100>>2]|0;
   $102 = (_i64Add(($98|0),($101|0),($91|0),($93|0))|0);
   $103 = tempRet0;
   $104 = $95;
   $105 = $104;
   HEAP32[$105>>2] = $102;
   $106 = (($104) + 4)|0;
   $107 = $106;
   HEAP32[$107>>2] = $103;
  }
  $108 = HEAP32[$nCopy>>2]|0;
  $109 = HEAP32[$nRem>>2]|0;
  $110 = (($109) - ($108))|0;
  HEAP32[$nRem>>2] = $110;
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
}
function _vdbePmaWriterFinish($p,$piEof) {
 $p = $p|0;
 $piEof = $piEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $rc = 0, dest = 0, label = 0;
 var sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $piEof;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 4|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 16|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 12|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($11|0)>($14|0);
   if ($15) {
    $16 = HEAP32[$0>>2]|0;
    $17 = ((($16)) + 32|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = HEAP32[$0>>2]|0;
    $20 = ((($19)) + 12|0);
    $21 = HEAP32[$20>>2]|0;
    $22 = HEAP32[$0>>2]|0;
    $23 = ((($22)) + 4|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($24) + ($21)|0);
    $26 = HEAP32[$0>>2]|0;
    $27 = ((($26)) + 16|0);
    $28 = HEAP32[$27>>2]|0;
    $29 = HEAP32[$0>>2]|0;
    $30 = ((($29)) + 12|0);
    $31 = HEAP32[$30>>2]|0;
    $32 = (($28) - ($31))|0;
    $33 = HEAP32[$0>>2]|0;
    $34 = ((($33)) + 24|0);
    $35 = $34;
    $36 = $35;
    $37 = HEAP32[$36>>2]|0;
    $38 = (($35) + 4)|0;
    $39 = $38;
    $40 = HEAP32[$39>>2]|0;
    $41 = HEAP32[$0>>2]|0;
    $42 = ((($41)) + 12|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (_i64Add(($37|0),($40|0),($43|0),($45|0))|0);
    $47 = tempRet0;
    $48 = (_sqlite3OsWrite($18,$25,$32,$46,$47)|0);
    $49 = HEAP32[$0>>2]|0;
    HEAP32[$49>>2] = $48;
   }
  }
 }
 $50 = HEAP32[$0>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = $51;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 16|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ($60|0)<(0);
 $62 = $61 << 31 >> 31;
 $63 = (_i64Add(($54|0),($57|0),($60|0),($62|0))|0);
 $64 = tempRet0;
 $65 = HEAP32[$1>>2]|0;
 $66 = $65;
 $67 = $66;
 HEAP32[$67>>2] = $63;
 $68 = (($66) + 4)|0;
 $69 = $68;
 HEAP32[$69>>2] = $64;
 $70 = HEAP32[$0>>2]|0;
 $71 = ((($70)) + 4|0);
 $72 = HEAP32[$71>>2]|0;
 _sqlite3_free($72);
 $73 = HEAP32[$0>>2]|0;
 $74 = HEAP32[$73>>2]|0;
 HEAP32[$rc>>2] = $74;
 $75 = HEAP32[$0>>2]|0;
 dest=$75; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $76 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _vdbeSortAllocUnpacked($pTask) {
 $pTask = $pTask|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pFree = sp;
 HEAP32[$1>>2] = $pTask;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0|0);
 do {
  if ($5) {
   $6 = HEAP32[$1>>2]|0;
   $7 = ((($6)) + 8|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = ((($8)) + 28|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = (_sqlite3VdbeAllocUnpackedRecord($10,0,0,$pFree)|0);
   $12 = HEAP32[$1>>2]|0;
   $13 = ((($12)) + 12|0);
   HEAP32[$13>>2] = $11;
   $14 = HEAP32[$pFree>>2]|0;
   $15 = ($14|0)==(0|0);
   if (!($15)) {
    $16 = HEAP32[$1>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ((($18)) + 28|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = ((($20)) + 6|0);
    $22 = HEAP16[$21>>1]|0;
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 12|0);
    $25 = HEAP32[$24>>2]|0;
    $26 = ((($25)) + 8|0);
    HEAP16[$26>>1] = $22;
    $27 = HEAP32[$1>>2]|0;
    $28 = ((($27)) + 12|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = ((($29)) + 11|0);
    HEAP8[$30>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 7;
   $31 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($31|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $31 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _vdbeSorterGetCompare($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 60|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4&255;
 $6 = ($5|0)==(1);
 do {
  if ($6) {
   HEAP32[$0>>2] = 138;
  } else {
   $7 = HEAP32[$1>>2]|0;
   $8 = ((($7)) + 60|0);
   $9 = HEAP8[$8>>0]|0;
   $10 = $9&255;
   $11 = ($10|0)==(2);
   if ($11) {
    HEAP32[$0>>2] = 139;
    break;
   } else {
    HEAP32[$0>>2] = 140;
    break;
   }
  }
 } while(0);
 $12 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _vdbeSorterMerge($pTask,$p1,$p2) {
 $pTask = $pTask|0;
 $p1 = $p1|0;
 $p2 = $p2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $bCached = 0, $pFinal = 0, $pp = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pFinal = sp + 12|0;
 $pp = sp + 8|0;
 $bCached = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $p1;
 HEAP32[$2>>2] = $p2;
 HEAP32[$pFinal>>2] = 0;
 HEAP32[$pp>>2] = $pFinal;
 HEAP32[$bCached>>2] = 0;
 while(1) {
  $3 = HEAP32[$0>>2]|0;
  $4 = ((($3)) + 32|0);
  $5 = HEAP32[$4>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 8|0);
  $9 = HEAP32[$1>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$2>>2]|0;
  $12 = ((($11)) + 8|0);
  $13 = HEAP32[$2>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = (FUNCTION_TABLE_iiiiiii[$5 & 255]($6,$bCached,$8,$10,$12,$14)|0);
  HEAP32[$res>>2] = $15;
  $16 = HEAP32[$res>>2]|0;
  $17 = ($16|0)<=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = HEAP32[$pp>>2]|0;
   HEAP32[$19>>2] = $18;
   $20 = HEAP32[$1>>2]|0;
   $21 = ((($20)) + 4|0);
   HEAP32[$pp>>2] = $21;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 4|0);
   $24 = HEAP32[$23>>2]|0;
   HEAP32[$1>>2] = $24;
   $25 = HEAP32[$1>>2]|0;
   $26 = ($25|0)==(0|0);
   if ($26) {
    label = 4;
    break;
   } else {
    continue;
   }
  } else {
   $29 = HEAP32[$2>>2]|0;
   $30 = HEAP32[$pp>>2]|0;
   HEAP32[$30>>2] = $29;
   $31 = HEAP32[$2>>2]|0;
   $32 = ((($31)) + 4|0);
   HEAP32[$pp>>2] = $32;
   $33 = HEAP32[$2>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   HEAP32[$2>>2] = $35;
   HEAP32[$bCached>>2] = 0;
   $36 = HEAP32[$2>>2]|0;
   $37 = ($36|0)==(0|0);
   if ($37) {
    label = 6;
    break;
   } else {
    continue;
   }
  }
 }
 if ((label|0) == 4) {
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$pp>>2]|0;
  HEAP32[$28>>2] = $27;
  $40 = HEAP32[$pFinal>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 else if ((label|0) == 6) {
  $38 = HEAP32[$1>>2]|0;
  $39 = HEAP32[$pp>>2]|0;
  HEAP32[$39>>2] = $38;
  $40 = HEAP32[$pFinal>>2]|0;
  STACKTOP = sp;return ($40|0);
 }
 return (0)|0;
}
function _vdbeSorterCompareInt($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $12 = 0, $13 = 0, $14 = 0;
 var $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $or$cond = 0, $p1 = 0, $p2 = 0, $res = 0, $s1 = 0, $s2 = 0;
 var $v1 = 0, $v2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $5 = sp + 32|0;
 $p1 = sp + 28|0;
 $p2 = sp + 24|0;
 $s1 = sp + 20|0;
 $s2 = sp + 16|0;
 $v1 = sp + 12|0;
 $v2 = sp + 8|0;
 $res = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p1>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 HEAP32[$p2>>2] = $7;
 $8 = HEAP32[$p1>>2]|0;
 $9 = ((($8)) + 1|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$s1>>2] = $11;
 $12 = HEAP32[$p2>>2]|0;
 $13 = ((($12)) + 1|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 HEAP32[$s2>>2] = $15;
 $16 = HEAP32[$p1>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = HEAP32[$p1>>2]|0;
 $20 = (($19) + ($18)|0);
 HEAP32[$v1>>2] = $20;
 $21 = HEAP32[$p2>>2]|0;
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = HEAP32[$p2>>2]|0;
 $25 = (($24) + ($23)|0);
 HEAP32[$v2>>2] = $25;
 $26 = HEAP32[$s1>>2]|0;
 $27 = ($26|0)>(7);
 $28 = HEAP32[$s2>>2]|0;
 $29 = ($28|0)>(7);
 $or$cond = $27 & $29;
 $30 = HEAP32[$s1>>2]|0;
 $31 = HEAP32[$s2>>2]|0;
 L1: do {
  if ($or$cond) {
   $32 = (($30) - ($31))|0;
   HEAP32[$res>>2] = $32;
  } else {
   $33 = ($30|0)==($31|0);
   if ($33) {
    $34 = HEAP32[$v1>>2]|0;
    $35 = HEAP8[$34>>0]|0;
    $36 = $35&255;
    $37 = HEAP32[$v2>>2]|0;
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = $36 ^ $39;
    $41 = $40 & 128;
    $42 = ($41|0)!=(0);
    if ($42) {
     $43 = HEAP32[$v1>>2]|0;
     $44 = HEAP8[$43>>0]|0;
     $45 = $44&255;
     $46 = $45 & 128;
     $47 = ($46|0)!=(0);
     $48 = $47 ? -1 : 1;
     HEAP32[$res>>2] = $48;
     break;
    }
    HEAP32[$res>>2] = 0;
    HEAP32[$i>>2] = 0;
    while(1) {
     $49 = HEAP32[$i>>2]|0;
     $50 = HEAP32[$s1>>2]|0;
     $51 = (41789 + ($50)|0);
     $52 = HEAP8[$51>>0]|0;
     $53 = $52&255;
     $54 = ($49|0)<($53|0);
     if (!($54)) {
      break L1;
     }
     $55 = HEAP32[$i>>2]|0;
     $56 = HEAP32[$v1>>2]|0;
     $57 = (($56) + ($55)|0);
     $58 = HEAP8[$57>>0]|0;
     $59 = $58&255;
     $60 = HEAP32[$i>>2]|0;
     $61 = HEAP32[$v2>>2]|0;
     $62 = (($61) + ($60)|0);
     $63 = HEAP8[$62>>0]|0;
     $64 = $63&255;
     $65 = (($59) - ($64))|0;
     HEAP32[$res>>2] = $65;
     $66 = ($65|0)!=(0);
     if ($66) {
      break L1;
     }
     $67 = HEAP32[$i>>2]|0;
     $68 = (($67) + 1)|0;
     HEAP32[$i>>2] = $68;
    }
   }
   $69 = HEAP32[$s2>>2]|0;
   $70 = ($69|0)>(7);
   do {
    if ($70) {
     HEAP32[$res>>2] = 1;
    } else {
     $71 = HEAP32[$s1>>2]|0;
     $72 = ($71|0)>(7);
     if ($72) {
      HEAP32[$res>>2] = -1;
      break;
     } else {
      $73 = HEAP32[$s1>>2]|0;
      $74 = HEAP32[$s2>>2]|0;
      $75 = (($73) - ($74))|0;
      HEAP32[$res>>2] = $75;
      break;
     }
    }
   } while(0);
   $76 = HEAP32[$res>>2]|0;
   $77 = ($76|0)>(0);
   if ($77) {
    $78 = HEAP32[$v1>>2]|0;
    $79 = HEAP8[$78>>0]|0;
    $80 = $79&255;
    $81 = $80 & 128;
    $82 = ($81|0)!=(0);
    if (!($82)) {
     break;
    }
    HEAP32[$res>>2] = -1;
    break;
   } else {
    $83 = HEAP32[$v2>>2]|0;
    $84 = HEAP8[$83>>0]|0;
    $85 = $84&255;
    $86 = $85 & 128;
    $87 = ($86|0)!=(0);
    if (!($87)) {
     break;
    }
    HEAP32[$res>>2] = 1;
    break;
   }
  }
 } while(0);
 $88 = HEAP32[$res>>2]|0;
 $89 = ($88|0)==(0);
 $90 = HEAP32[$0>>2]|0;
 $91 = ((($90)) + 8|0);
 $92 = HEAP32[$91>>2]|0;
 $93 = ((($92)) + 28|0);
 $94 = HEAP32[$93>>2]|0;
 if ($89) {
  $95 = ((($94)) + 6|0);
  $96 = HEAP16[$95>>1]|0;
  $97 = $96&65535;
  $98 = ($97|0)>(1);
  if (!($98)) {
   $112 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($112|0);
  }
  $99 = HEAP32[$0>>2]|0;
  $100 = HEAP32[$1>>2]|0;
  $101 = HEAP32[$2>>2]|0;
  $102 = HEAP32[$3>>2]|0;
  $103 = HEAP32[$4>>2]|0;
  $104 = HEAP32[$5>>2]|0;
  $105 = (_vdbeSorterCompareTail($99,$100,$101,$102,$103,$104)|0);
  HEAP32[$res>>2] = $105;
  $112 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($112|0);
 } else {
  $106 = ((($94)) + 16|0);
  $107 = HEAP32[$106>>2]|0;
  $108 = HEAP8[$107>>0]|0;
  $109 = ($108<<24>>24)!=(0);
  if (!($109)) {
   $112 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($112|0);
  }
  $110 = HEAP32[$res>>2]|0;
  $111 = Math_imul($110, -1)|0;
  HEAP32[$res>>2] = $111;
  $112 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($112|0);
 }
 return (0)|0;
}
function _vdbeSorterCompareText($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $n1 = 0, $n2 = 0, $p1 = 0, $p2 = 0, $res = 0, $v1 = 0, $v2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $5 = sp + 28|0;
 $p1 = sp + 24|0;
 $p2 = sp + 20|0;
 $v1 = sp + 16|0;
 $v2 = sp + 12|0;
 $n1 = sp + 8|0;
 $n2 = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$p1>>2] = $6;
 $7 = HEAP32[$4>>2]|0;
 HEAP32[$p2>>2] = $7;
 $8 = HEAP32[$p1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$p1>>2]|0;
 $12 = (($11) + ($10)|0);
 HEAP32[$v1>>2] = $12;
 $13 = HEAP32[$p2>>2]|0;
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = HEAP32[$p2>>2]|0;
 $17 = (($16) + ($15)|0);
 HEAP32[$v2>>2] = $17;
 $18 = HEAP32[$p1>>2]|0;
 $19 = ((($18)) + 1|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = $20&255;
 $22 = ($21|0)<(128);
 $23 = HEAP32[$p1>>2]|0;
 $24 = ((($23)) + 1|0);
 if ($22) {
  $25 = HEAP8[$24>>0]|0;
  $26 = $25&255;
  HEAP32[$n1>>2] = $26;
 } else {
  (_sqlite3GetVarint32($24,$n1)|0);
 }
 $27 = HEAP32[$n1>>2]|0;
 $28 = (($27) - 13)|0;
 $29 = (($28|0) / 2)&-1;
 HEAP32[$n1>>2] = $29;
 $30 = HEAP32[$p2>>2]|0;
 $31 = ((($30)) + 1|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = ($33|0)<(128);
 $35 = HEAP32[$p2>>2]|0;
 $36 = ((($35)) + 1|0);
 if ($34) {
  $37 = HEAP8[$36>>0]|0;
  $38 = $37&255;
  HEAP32[$n2>>2] = $38;
 } else {
  (_sqlite3GetVarint32($36,$n2)|0);
 }
 $39 = HEAP32[$n2>>2]|0;
 $40 = (($39) - 13)|0;
 $41 = (($40|0) / 2)&-1;
 HEAP32[$n2>>2] = $41;
 $42 = HEAP32[$v1>>2]|0;
 $43 = HEAP32[$v2>>2]|0;
 $44 = HEAP32[$n1>>2]|0;
 $45 = HEAP32[$n2>>2]|0;
 $46 = ($44|0)<($45|0);
 $47 = HEAP32[$n1>>2]|0;
 $48 = HEAP32[$n2>>2]|0;
 $49 = $46 ? $47 : $48;
 $50 = (_memcmp($42,$43,$49)|0);
 HEAP32[$res>>2] = $50;
 $51 = HEAP32[$res>>2]|0;
 $52 = ($51|0)==(0);
 if ($52) {
  $53 = HEAP32[$n1>>2]|0;
  $54 = HEAP32[$n2>>2]|0;
  $55 = (($53) - ($54))|0;
  HEAP32[$res>>2] = $55;
 }
 $56 = HEAP32[$res>>2]|0;
 $57 = ($56|0)==(0);
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 8|0);
 $60 = HEAP32[$59>>2]|0;
 $61 = ((($60)) + 28|0);
 $62 = HEAP32[$61>>2]|0;
 if ($57) {
  $63 = ((($62)) + 6|0);
  $64 = HEAP16[$63>>1]|0;
  $65 = $64&65535;
  $66 = ($65|0)>(1);
  if (!($66)) {
   $80 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($80|0);
  }
  $67 = HEAP32[$0>>2]|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = HEAP32[$2>>2]|0;
  $70 = HEAP32[$3>>2]|0;
  $71 = HEAP32[$4>>2]|0;
  $72 = HEAP32[$5>>2]|0;
  $73 = (_vdbeSorterCompareTail($67,$68,$69,$70,$71,$72)|0);
  HEAP32[$res>>2] = $73;
  $80 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($80|0);
 } else {
  $74 = ((($62)) + 16|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = HEAP8[$75>>0]|0;
  $77 = ($76<<24>>24)!=(0);
  if (!($77)) {
   $80 = HEAP32[$res>>2]|0;
   STACKTOP = sp;return ($80|0);
  }
  $78 = HEAP32[$res>>2]|0;
  $79 = Math_imul($78, -1)|0;
  HEAP32[$res>>2] = $79;
  $80 = HEAP32[$res>>2]|0;
  STACKTOP = sp;return ($80|0);
 }
 return (0)|0;
}
function _vdbeSorterCompare($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $r2 = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$r2>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$r2>>2]|0;
  $24 = (_sqlite3VdbeRecordCompare($21,$22,$23)|0);
  STACKTOP = sp;return ($24|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 28|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$5>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$r2>>2]|0;
 _sqlite3VdbeRecordUnpack($16,$17,$18,$19);
 $20 = HEAP32[$1>>2]|0;
 HEAP32[$20>>2] = 1;
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$r2>>2]|0;
 $24 = (_sqlite3VdbeRecordCompare($21,$22,$23)|0);
 STACKTOP = sp;return ($24|0);
}
function _vdbeSorterCompareTail($pTask,$pbKey2Cached,$pKey1,$nKey1,$pKey2,$nKey2) {
 $pTask = $pTask|0;
 $pbKey2Cached = $pbKey2Cached|0;
 $pKey1 = $pKey1|0;
 $nKey1 = $nKey1|0;
 $pKey2 = $pKey2|0;
 $nKey2 = $nKey2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $5 = sp + 4|0;
 $r2 = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pbKey2Cached;
 HEAP32[$2>>2] = $pKey1;
 HEAP32[$3>>2] = $nKey1;
 HEAP32[$4>>2] = $pKey2;
 HEAP32[$5>>2] = $nKey2;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$r2>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  $21 = HEAP32[$3>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$r2>>2]|0;
  $24 = (_sqlite3VdbeRecordCompareWithSkip($21,$22,$23,1)|0);
  STACKTOP = sp;return ($24|0);
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 28|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$5>>2]|0;
 $18 = HEAP32[$4>>2]|0;
 $19 = HEAP32[$r2>>2]|0;
 _sqlite3VdbeRecordUnpack($16,$17,$18,$19);
 $20 = HEAP32[$1>>2]|0;
 HEAP32[$20>>2] = 1;
 $21 = HEAP32[$3>>2]|0;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$r2>>2]|0;
 $24 = (_sqlite3VdbeRecordCompareWithSkip($21,$22,$23,1)|0);
 STACKTOP = sp;return ($24|0);
}
function _sqlite3OsOpenMalloc($pVfs,$zFile,$ppFile,$flags,$pOutFlags) {
 $pVfs = $pVfs|0;
 $zFile = $zFile|0;
 $ppFile = $ppFile|0;
 $flags = $flags|0;
 $pOutFlags = $pOutFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $4 = sp + 8|0;
 $rc = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zFile;
 HEAP32[$2>>2] = $ppFile;
 HEAP32[$3>>2] = $flags;
 HEAP32[$4>>2] = $pOutFlags;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 4|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)<(0);
 $9 = $8 << 31 >> 31;
 $10 = (_sqlite3MallocZero($7,$9)|0);
 HEAP32[$pFile>>2] = $10;
 $11 = HEAP32[$pFile>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  HEAP32[$rc>>2] = 7;
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$1>>2]|0;
 $15 = HEAP32[$pFile>>2]|0;
 $16 = HEAP32[$3>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = (_sqlite3OsOpen($13,$14,$15,$16,$17)|0);
 HEAP32[$rc>>2] = $18;
 $19 = HEAP32[$rc>>2]|0;
 $20 = ($19|0)!=(0);
 $21 = HEAP32[$pFile>>2]|0;
 if ($20) {
  _sqlite3_free($21);
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 } else {
  $22 = HEAP32[$2>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 return (0)|0;
}
function _vdbeMergeEngineStep($pMerger,$pbEof) {
 $pMerger = $pMerger|0;
 $pbEof = $pbEof|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $bCached = 0, $i = 0, $iPrev = 0, $iRes = 0, $pReadr1 = 0, $pReadr2 = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $rc = sp + 28|0;
 $iPrev = sp + 24|0;
 $pTask = sp + 20|0;
 $i = sp + 16|0;
 $pReadr1 = sp + 12|0;
 $pReadr2 = sp + 8|0;
 $bCached = sp + 4|0;
 $iRes = sp;
 HEAP32[$0>>2] = $pMerger;
 HEAP32[$1>>2] = $pbEof;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$iPrev>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pTask>>2] = $9;
 $10 = HEAP32[$iPrev>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 12|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + (($10*56)|0)|0);
 $15 = (_vdbePmaReaderNext($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  HEAP32[$bCached>>2] = 0;
  $18 = HEAP32[$iPrev>>2]|0;
  $19 = $18 & 65534;
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 12|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + (($19*56)|0)|0);
  HEAP32[$pReadr1>>2] = $23;
  $24 = HEAP32[$iPrev>>2]|0;
  $25 = $24 | 1;
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 12|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + (($25*56)|0)|0);
  HEAP32[$pReadr2>>2] = $29;
  $30 = HEAP32[$0>>2]|0;
  $31 = HEAP32[$30>>2]|0;
  $32 = HEAP32[$iPrev>>2]|0;
  $33 = (($31) + ($32))|0;
  $34 = (($33|0) / 2)&-1;
  HEAP32[$i>>2] = $34;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = ($35|0)>(0);
   if (!($36)) {
    break;
   }
   $37 = HEAP32[$pReadr1>>2]|0;
   $38 = ((($37)) + 24|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)==(0|0);
   do {
    if ($40) {
     HEAP32[$iRes>>2] = 1;
    } else {
     $41 = HEAP32[$pReadr2>>2]|0;
     $42 = ((($41)) + 24|0);
     $43 = HEAP32[$42>>2]|0;
     $44 = ($43|0)==(0|0);
     if ($44) {
      HEAP32[$iRes>>2] = -1;
      break;
     } else {
      $45 = HEAP32[$pTask>>2]|0;
      $46 = ((($45)) + 32|0);
      $47 = HEAP32[$46>>2]|0;
      $48 = HEAP32[$pTask>>2]|0;
      $49 = HEAP32[$pReadr1>>2]|0;
      $50 = ((($49)) + 32|0);
      $51 = HEAP32[$50>>2]|0;
      $52 = HEAP32[$pReadr1>>2]|0;
      $53 = ((($52)) + 20|0);
      $54 = HEAP32[$53>>2]|0;
      $55 = HEAP32[$pReadr2>>2]|0;
      $56 = ((($55)) + 32|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = HEAP32[$pReadr2>>2]|0;
      $59 = ((($58)) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = (FUNCTION_TABLE_iiiiiii[$47 & 255]($48,$bCached,$51,$54,$57,$60)|0);
      HEAP32[$iRes>>2] = $61;
      break;
     }
    }
   } while(0);
   $62 = HEAP32[$iRes>>2]|0;
   $63 = ($62|0)<(0);
   do {
    if ($63) {
     label = 12;
    } else {
     $64 = HEAP32[$iRes>>2]|0;
     $65 = ($64|0)==(0);
     if ($65) {
      $66 = HEAP32[$pReadr1>>2]|0;
      $67 = HEAP32[$pReadr2>>2]|0;
      $68 = ($66>>>0)<($67>>>0);
      if ($68) {
       label = 12;
       break;
      }
     }
     $93 = HEAP32[$pReadr1>>2]|0;
     $94 = ((($93)) + 24|0);
     $95 = HEAP32[$94>>2]|0;
     $96 = ($95|0)!=(0|0);
     if ($96) {
      HEAP32[$bCached>>2] = 0;
     }
     $97 = HEAP32[$pReadr2>>2]|0;
     $98 = HEAP32[$0>>2]|0;
     $99 = ((($98)) + 12|0);
     $100 = HEAP32[$99>>2]|0;
     $101 = $97;
     $102 = $100;
     $103 = (($101) - ($102))|0;
     $104 = (($103|0) / 56)&-1;
     $105 = HEAP32[$i>>2]|0;
     $106 = HEAP32[$0>>2]|0;
     $107 = ((($106)) + 8|0);
     $108 = HEAP32[$107>>2]|0;
     $109 = (($108) + ($105<<2)|0);
     HEAP32[$109>>2] = $104;
     $110 = HEAP32[$i>>2]|0;
     $111 = $110 ^ 1;
     $112 = HEAP32[$0>>2]|0;
     $113 = ((($112)) + 8|0);
     $114 = HEAP32[$113>>2]|0;
     $115 = (($114) + ($111<<2)|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$0>>2]|0;
     $118 = ((($117)) + 12|0);
     $119 = HEAP32[$118>>2]|0;
     $120 = (($119) + (($116*56)|0)|0);
     HEAP32[$pReadr1>>2] = $120;
    }
   } while(0);
   if ((label|0) == 12) {
    label = 0;
    $69 = HEAP32[$pReadr1>>2]|0;
    $70 = HEAP32[$0>>2]|0;
    $71 = ((($70)) + 12|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = $69;
    $74 = $72;
    $75 = (($73) - ($74))|0;
    $76 = (($75|0) / 56)&-1;
    $77 = HEAP32[$i>>2]|0;
    $78 = HEAP32[$0>>2]|0;
    $79 = ((($78)) + 8|0);
    $80 = HEAP32[$79>>2]|0;
    $81 = (($80) + ($77<<2)|0);
    HEAP32[$81>>2] = $76;
    $82 = HEAP32[$i>>2]|0;
    $83 = $82 ^ 1;
    $84 = HEAP32[$0>>2]|0;
    $85 = ((($84)) + 8|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = (($86) + ($83<<2)|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = HEAP32[$0>>2]|0;
    $90 = ((($89)) + 12|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = (($91) + (($88*56)|0)|0);
    HEAP32[$pReadr2>>2] = $92;
    HEAP32[$bCached>>2] = 0;
   }
   $121 = HEAP32[$i>>2]|0;
   $122 = (($121|0) / 2)&-1;
   HEAP32[$i>>2] = $122;
  }
  $123 = HEAP32[$0>>2]|0;
  $124 = ((($123)) + 8|0);
  $125 = HEAP32[$124>>2]|0;
  $126 = ((($125)) + 4|0);
  $127 = HEAP32[$126>>2]|0;
  $128 = HEAP32[$0>>2]|0;
  $129 = ((($128)) + 12|0);
  $130 = HEAP32[$129>>2]|0;
  $131 = (($130) + (($127*56)|0)|0);
  $132 = ((($131)) + 24|0);
  $133 = HEAP32[$132>>2]|0;
  $134 = ($133|0)==(0|0);
  $135 = $134&1;
  $136 = HEAP32[$1>>2]|0;
  HEAP32[$136>>2] = $135;
 }
 $137 = HEAP32[$rc>>2]|0;
 $138 = ($137|0)==(0);
 if ($138) {
  $139 = HEAP32[$pTask>>2]|0;
  $140 = ((($139)) + 12|0);
  $141 = HEAP32[$140>>2]|0;
  $142 = ((($141)) + 11|0);
  $143 = HEAP8[$142>>0]|0;
  $144 = $143&255;
  $146 = $144;
  STACKTOP = sp;return ($146|0);
 } else {
  $145 = HEAP32[$rc>>2]|0;
  $146 = $145;
  STACKTOP = sp;return ($146|0);
 }
 return (0)|0;
}
function _vdbePmaReaderNext($pReadr) {
 $pReadr = $pReadr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $9 = 0, $bEof = 0, $nRec = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $nRec = sp;
 $pIncr = sp + 12|0;
 $bEof = sp + 8|0;
 HEAP32[$1>>2] = $pReadr;
 HEAP32[$rc>>2] = 0;
 $2 = $nRec;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = $14;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = (($15) + 4)|0;
 $19 = $18;
 $20 = HEAP32[$19>>2]|0;
 $21 = ($12|0)>($20|0);
 $22 = ($9>>>0)>=($17>>>0);
 $23 = ($12|0)==($20|0);
 $24 = $23 & $22;
 $25 = $21 | $24;
 if ($25) {
  $26 = HEAP32[$1>>2]|0;
  $27 = ((($26)) + 48|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$pIncr>>2] = $28;
  HEAP32[$bEof>>2] = 1;
  $29 = HEAP32[$pIncr>>2]|0;
  $30 = ($29|0)!=(0|0);
  if ($30) {
   $31 = HEAP32[$pIncr>>2]|0;
   $32 = (_vdbeIncrSwap($31)|0);
   HEAP32[$rc>>2] = $32;
   $33 = HEAP32[$rc>>2]|0;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$pIncr>>2]|0;
    $36 = ((($35)) + 20|0);
    $37 = HEAP32[$36>>2]|0;
    $38 = ($37|0)==(0);
    if ($38) {
     $39 = HEAP32[$pIncr>>2]|0;
     $40 = HEAP32[$39>>2]|0;
     $41 = HEAP32[$1>>2]|0;
     $42 = HEAP32[$pIncr>>2]|0;
     $43 = ((($42)) + 32|0);
     $44 = HEAP32[$pIncr>>2]|0;
     $45 = ((($44)) + 8|0);
     $46 = $45;
     $47 = $46;
     $48 = HEAP32[$47>>2]|0;
     $49 = (($46) + 4)|0;
     $50 = $49;
     $51 = HEAP32[$50>>2]|0;
     $52 = (_vdbePmaReaderSeek($40,$41,$43,$48,$51)|0);
     HEAP32[$rc>>2] = $52;
     HEAP32[$bEof>>2] = 0;
    }
   }
  }
  $53 = HEAP32[$bEof>>2]|0;
  $54 = ($53|0)!=(0);
  if ($54) {
   $55 = HEAP32[$1>>2]|0;
   _vdbePmaReaderClear($55);
   $56 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $56;
   $82 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($82|0);
  }
 }
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)==(0);
 if ($58) {
  $59 = HEAP32[$1>>2]|0;
  $60 = (_vdbePmaReadVarint($59,$nRec)|0);
  HEAP32[$rc>>2] = $60;
 }
 $61 = HEAP32[$rc>>2]|0;
 $62 = ($61|0)==(0);
 if ($62) {
  $63 = $nRec;
  $64 = $63;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($63) + 4)|0;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 20|0);
  HEAP32[$70>>2] = $65;
  $71 = HEAP32[$1>>2]|0;
  $72 = $nRec;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = (($72) + 4)|0;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 32|0);
  $80 = (_vdbePmaReadBlob($71,$74,$79)|0);
  HEAP32[$rc>>2] = $80;
 }
 $81 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $81;
 $82 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($82|0);
}
function _vdbeIncrSwap($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pIncr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_vdbeIncrPopulate($1)|0);
 HEAP32[$rc>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 32|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 32|0);
 $7 = ((($6)) + 16|0);
 ;HEAP32[$4>>2]=HEAP32[$7>>2]|0;HEAP32[$4+4>>2]=HEAP32[$7+4>>2]|0;HEAP32[$4+8>>2]=HEAP32[$7+8>>2]|0;HEAP32[$4+12>>2]=HEAP32[$7+12>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 32|0);
 $10 = ((($9)) + 8|0);
 $11 = $10;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = $18;
 $20 = $19;
 $21 = HEAP32[$20>>2]|0;
 $22 = (($19) + 4)|0;
 $23 = $22;
 $24 = HEAP32[$23>>2]|0;
 $25 = ($13|0)==($21|0);
 $26 = ($16|0)==($24|0);
 $27 = $25 & $26;
 if (!($27)) {
  $30 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 20|0);
 HEAP32[$29>>2] = 1;
 $30 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($30|0);
}
function _vdbePmaReaderSeek($pTask,$pReadr,$pFile,$0,$1) {
 $pTask = $pTask|0;
 $pReadr = $pReadr|0;
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $iBuf = 0, $nRead = 0, $or$cond = 0, $pgsz = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp;
 $rc = sp + 20|0;
 $pgsz = sp + 16|0;
 $iBuf = sp + 12|0;
 $nRead = sp + 8|0;
 HEAP32[$3>>2] = $pTask;
 HEAP32[$4>>2] = $pReadr;
 HEAP32[$5>>2] = $pFile;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $11 = (_sqlite3FaultSim(201)|0);
 $12 = ($11|0)!=(0);
 if ($12) {
  HEAP32[$2>>2] = 266;
  $166 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($166|0);
 }
 $13 = HEAP32[$4>>2]|0;
 $14 = ((($13)) + 44|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)!=(0|0);
 if ($16) {
  $17 = HEAP32[$4>>2]|0;
  $18 = ((($17)) + 24|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$4>>2]|0;
  $21 = ((($20)) + 44|0);
  $22 = HEAP32[$21>>2]|0;
  (_sqlite3OsUnfetch($19,0,0,$22)|0);
  $23 = HEAP32[$4>>2]|0;
  $24 = ((($23)) + 44|0);
  HEAP32[$24>>2] = 0;
 }
 $25 = $6;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = HEAP32[$4>>2]|0;
 $32 = $31;
 $33 = $32;
 HEAP32[$33>>2] = $27;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $30;
 $36 = HEAP32[$5>>2]|0;
 $37 = ((($36)) + 8|0);
 $38 = $37;
 $39 = $38;
 $40 = HEAP32[$39>>2]|0;
 $41 = (($38) + 4)|0;
 $42 = $41;
 $43 = HEAP32[$42>>2]|0;
 $44 = HEAP32[$4>>2]|0;
 $45 = ((($44)) + 8|0);
 $46 = $45;
 $47 = $46;
 HEAP32[$47>>2] = $40;
 $48 = (($46) + 4)|0;
 $49 = $48;
 HEAP32[$49>>2] = $43;
 $50 = HEAP32[$5>>2]|0;
 $51 = HEAP32[$50>>2]|0;
 $52 = HEAP32[$4>>2]|0;
 $53 = ((($52)) + 24|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$3>>2]|0;
 $55 = HEAP32[$5>>2]|0;
 $56 = HEAP32[$4>>2]|0;
 $57 = ((($56)) + 44|0);
 $58 = (_vdbeSorterMapFile($54,$55,$57)|0);
 HEAP32[$rc>>2] = $58;
 $59 = HEAP32[$rc>>2]|0;
 $60 = ($59|0)==(0);
 if ($60) {
  $61 = HEAP32[$4>>2]|0;
  $62 = ((($61)) + 44|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($63|0)==(0|0);
  if ($64) {
   $65 = HEAP32[$3>>2]|0;
   $66 = ((($65)) + 8|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = ((($67)) + 12|0);
   $69 = HEAP32[$68>>2]|0;
   HEAP32[$pgsz>>2] = $69;
   $70 = HEAP32[$4>>2]|0;
   $71 = $70;
   $72 = $71;
   $73 = HEAP32[$72>>2]|0;
   $74 = (($71) + 4)|0;
   $75 = $74;
   $76 = HEAP32[$75>>2]|0;
   $77 = HEAP32[$pgsz>>2]|0;
   $78 = ($77|0)<(0);
   $79 = $78 << 31 >> 31;
   $80 = (___remdi3(($73|0),($76|0),($77|0),($79|0))|0);
   $81 = tempRet0;
   HEAP32[$iBuf>>2] = $80;
   $82 = HEAP32[$4>>2]|0;
   $83 = ((($82)) + 36|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = ($84|0)==(0|0);
   if ($85) {
    $86 = HEAP32[$pgsz>>2]|0;
    $87 = ($86|0)<(0);
    $88 = $87 << 31 >> 31;
    $89 = (_sqlite3Malloc($86,$88)|0);
    $90 = HEAP32[$4>>2]|0;
    $91 = ((($90)) + 36|0);
    HEAP32[$91>>2] = $89;
    $92 = HEAP32[$4>>2]|0;
    $93 = ((($92)) + 36|0);
    $94 = HEAP32[$93>>2]|0;
    $95 = ($94|0)==(0|0);
    if ($95) {
     HEAP32[$rc>>2] = 7;
    }
    $96 = HEAP32[$pgsz>>2]|0;
    $97 = HEAP32[$4>>2]|0;
    $98 = ((($97)) + 40|0);
    HEAP32[$98>>2] = $96;
   }
   $99 = HEAP32[$rc>>2]|0;
   $100 = ($99|0)==(0);
   $101 = HEAP32[$iBuf>>2]|0;
   $102 = ($101|0)!=(0);
   $or$cond = $100 & $102;
   if ($or$cond) {
    $103 = HEAP32[$pgsz>>2]|0;
    $104 = HEAP32[$iBuf>>2]|0;
    $105 = (($103) - ($104))|0;
    HEAP32[$nRead>>2] = $105;
    $106 = HEAP32[$4>>2]|0;
    $107 = $106;
    $108 = $107;
    $109 = HEAP32[$108>>2]|0;
    $110 = (($107) + 4)|0;
    $111 = $110;
    $112 = HEAP32[$111>>2]|0;
    $113 = HEAP32[$nRead>>2]|0;
    $114 = ($113|0)<(0);
    $115 = $114 << 31 >> 31;
    $116 = (_i64Add(($109|0),($112|0),($113|0),($115|0))|0);
    $117 = tempRet0;
    $118 = HEAP32[$4>>2]|0;
    $119 = ((($118)) + 8|0);
    $120 = $119;
    $121 = $120;
    $122 = HEAP32[$121>>2]|0;
    $123 = (($120) + 4)|0;
    $124 = $123;
    $125 = HEAP32[$124>>2]|0;
    $126 = ($117|0)>($125|0);
    $127 = ($116>>>0)>($122>>>0);
    $128 = ($117|0)==($125|0);
    $129 = $128 & $127;
    $130 = $126 | $129;
    if ($130) {
     $131 = HEAP32[$4>>2]|0;
     $132 = ((($131)) + 8|0);
     $133 = $132;
     $134 = $133;
     $135 = HEAP32[$134>>2]|0;
     $136 = (($133) + 4)|0;
     $137 = $136;
     $138 = HEAP32[$137>>2]|0;
     $139 = HEAP32[$4>>2]|0;
     $140 = $139;
     $141 = $140;
     $142 = HEAP32[$141>>2]|0;
     $143 = (($140) + 4)|0;
     $144 = $143;
     $145 = HEAP32[$144>>2]|0;
     $146 = (_i64Subtract(($135|0),($138|0),($142|0),($145|0))|0);
     $147 = tempRet0;
     HEAP32[$nRead>>2] = $146;
    }
    $148 = HEAP32[$4>>2]|0;
    $149 = ((($148)) + 24|0);
    $150 = HEAP32[$149>>2]|0;
    $151 = HEAP32[$iBuf>>2]|0;
    $152 = HEAP32[$4>>2]|0;
    $153 = ((($152)) + 36|0);
    $154 = HEAP32[$153>>2]|0;
    $155 = (($154) + ($151)|0);
    $156 = HEAP32[$nRead>>2]|0;
    $157 = HEAP32[$4>>2]|0;
    $158 = $157;
    $159 = $158;
    $160 = HEAP32[$159>>2]|0;
    $161 = (($158) + 4)|0;
    $162 = $161;
    $163 = HEAP32[$162>>2]|0;
    $164 = (_sqlite3OsRead($150,$155,$156,$160,$163)|0);
    HEAP32[$rc>>2] = $164;
   }
  }
 }
 $165 = HEAP32[$rc>>2]|0;
 HEAP32[$2>>2] = $165;
 $166 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($166|0);
}
function _vdbePmaReadVarint($p,$pnOut) {
 $p = $p|0;
 $pnOut = $pnOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $9 = 0, $a = 0, $aVarint = 0, $i = 0, $iBuf = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $iBuf = sp + 12|0;
 $aVarint = sp + 32|0;
 $a = sp + 8|0;
 $i = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pnOut;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 44|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0|0);
 $7 = HEAP32[$1>>2]|0;
 $8 = $7;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = (($8) + 4)|0;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 do {
  if ($6) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 44|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (($16) + ($10)|0);
   $18 = HEAP32[$2>>2]|0;
   $19 = (_sqlite3GetVarint($17,$18)|0);
   $20 = $19&255;
   $21 = HEAP32[$1>>2]|0;
   $22 = $21;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = (($22) + 4)|0;
   $26 = $25;
   $27 = HEAP32[$26>>2]|0;
   $28 = (_i64Add(($24|0),($27|0),($20|0),0)|0);
   $29 = tempRet0;
   $30 = $21;
   $31 = $30;
   HEAP32[$31>>2] = $28;
   $32 = (($30) + 4)|0;
   $33 = $32;
   HEAP32[$33>>2] = $29;
  } else {
   $34 = HEAP32[$1>>2]|0;
   $35 = ((($34)) + 40|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)<(0);
   $38 = $37 << 31 >> 31;
   $39 = (___remdi3(($10|0),($13|0),($36|0),($38|0))|0);
   $40 = tempRet0;
   HEAP32[$iBuf>>2] = $39;
   $41 = HEAP32[$iBuf>>2]|0;
   $42 = ($41|0)!=(0);
   if ($42) {
    $43 = HEAP32[$1>>2]|0;
    $44 = ((($43)) + 40|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = HEAP32[$iBuf>>2]|0;
    $47 = (($45) - ($46))|0;
    $48 = ($47|0)>=(9);
    if ($48) {
     $49 = HEAP32[$iBuf>>2]|0;
     $50 = HEAP32[$1>>2]|0;
     $51 = ((($50)) + 36|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = (($52) + ($49)|0);
     $54 = HEAP32[$2>>2]|0;
     $55 = (_sqlite3GetVarint($53,$54)|0);
     $56 = $55&255;
     $57 = HEAP32[$1>>2]|0;
     $58 = $57;
     $59 = $58;
     $60 = HEAP32[$59>>2]|0;
     $61 = (($58) + 4)|0;
     $62 = $61;
     $63 = HEAP32[$62>>2]|0;
     $64 = (_i64Add(($60|0),($63|0),($56|0),0)|0);
     $65 = tempRet0;
     $66 = $57;
     $67 = $66;
     HEAP32[$67>>2] = $64;
     $68 = (($66) + 4)|0;
     $69 = $68;
     HEAP32[$69>>2] = $65;
     break;
    }
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $70 = HEAP32[$1>>2]|0;
    $71 = (_vdbePmaReadBlob($70,1,$a)|0);
    HEAP32[$rc>>2] = $71;
    $72 = HEAP32[$rc>>2]|0;
    $73 = ($72|0)!=(0);
    if ($73) {
     break;
    }
    $75 = HEAP32[$a>>2]|0;
    $76 = HEAP8[$75>>0]|0;
    $77 = HEAP32[$i>>2]|0;
    $78 = (($77) + 1)|0;
    HEAP32[$i>>2] = $78;
    $79 = $77 & 15;
    $80 = (($aVarint) + ($79)|0);
    HEAP8[$80>>0] = $76;
    $81 = HEAP32[$a>>2]|0;
    $82 = HEAP8[$81>>0]|0;
    $83 = $82&255;
    $84 = $83 & 128;
    $85 = ($84|0)!=(0);
    if (!($85)) {
     label = 10;
     break;
    }
   }
   if ((label|0) == 10) {
    $86 = HEAP32[$2>>2]|0;
    (_sqlite3GetVarint($aVarint,$86)|0);
    break;
   }
   $74 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $74;
   $87 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($87|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $87 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($87|0);
}
function _vdbePmaReadBlob($p,$nByte,$ppOut) {
 $p = $p|0;
 $nByte = $nByte|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $aNext = 0, $iBuf = 0, $nAvail = 0;
 var $nCopy = 0, $nNew = 0, $nRead = 0, $nRem = 0, $rc = 0, $rc1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $iBuf = sp + 36|0;
 $nAvail = sp + 32|0;
 $nRead = sp + 28|0;
 $rc = sp + 24|0;
 $nRem = sp + 20|0;
 $aNew = sp + 16|0;
 $nNew = sp + 12|0;
 $rc1 = sp + 8|0;
 $nCopy = sp + 4|0;
 $aNext = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $ppOut;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 $8 = HEAP32[$1>>2]|0;
 $9 = $8;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 if ($7) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 44|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + ($11)|0);
  $19 = HEAP32[$3>>2]|0;
  HEAP32[$19>>2] = $18;
  $20 = HEAP32[$2>>2]|0;
  $21 = ($20|0)<(0);
  $22 = $21 << 31 >> 31;
  $23 = HEAP32[$1>>2]|0;
  $24 = $23;
  $25 = $24;
  $26 = HEAP32[$25>>2]|0;
  $27 = (($24) + 4)|0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (_i64Add(($26|0),($29|0),($20|0),($22|0))|0);
  $31 = tempRet0;
  $32 = $23;
  $33 = $32;
  HEAP32[$33>>2] = $30;
  $34 = (($32) + 4)|0;
  $35 = $34;
  HEAP32[$35>>2] = $31;
  HEAP32[$0>>2] = 0;
  $235 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($235|0);
 }
 $36 = HEAP32[$1>>2]|0;
 $37 = ((($36)) + 40|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)<(0);
 $40 = $39 << 31 >> 31;
 $41 = (___remdi3(($11|0),($14|0),($38|0),($40|0))|0);
 $42 = tempRet0;
 HEAP32[$iBuf>>2] = $41;
 $43 = HEAP32[$iBuf>>2]|0;
 $44 = ($43|0)==(0);
 if ($44) {
  $45 = HEAP32[$1>>2]|0;
  $46 = ((($45)) + 8|0);
  $47 = $46;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  $50 = (($47) + 4)|0;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = HEAP32[$1>>2]|0;
  $54 = $53;
  $55 = $54;
  $56 = HEAP32[$55>>2]|0;
  $57 = (($54) + 4)|0;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (_i64Subtract(($49|0),($52|0),($56|0),($59|0))|0);
  $61 = tempRet0;
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 40|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = ($64|0)<(0);
  $66 = $65 << 31 >> 31;
  $67 = ($61|0)>($66|0);
  $68 = ($60>>>0)>($64>>>0);
  $69 = ($61|0)==($66|0);
  $70 = $69 & $68;
  $71 = $67 | $70;
  $72 = HEAP32[$1>>2]|0;
  if ($71) {
   $73 = ((($72)) + 40|0);
   $74 = HEAP32[$73>>2]|0;
   HEAP32[$nRead>>2] = $74;
  } else {
   $75 = ((($72)) + 8|0);
   $76 = $75;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = HEAP32[$1>>2]|0;
   $83 = $82;
   $84 = $83;
   $85 = HEAP32[$84>>2]|0;
   $86 = (($83) + 4)|0;
   $87 = $86;
   $88 = HEAP32[$87>>2]|0;
   $89 = (_i64Subtract(($78|0),($81|0),($85|0),($88|0))|0);
   $90 = tempRet0;
   HEAP32[$nRead>>2] = $89;
  }
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 24|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$1>>2]|0;
  $95 = ((($94)) + 36|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = HEAP32[$nRead>>2]|0;
  $98 = HEAP32[$1>>2]|0;
  $99 = $98;
  $100 = $99;
  $101 = HEAP32[$100>>2]|0;
  $102 = (($99) + 4)|0;
  $103 = $102;
  $104 = HEAP32[$103>>2]|0;
  $105 = (_sqlite3OsRead($93,$96,$97,$101,$104)|0);
  HEAP32[$rc>>2] = $105;
  $106 = HEAP32[$rc>>2]|0;
  $107 = ($106|0)!=(0);
  if ($107) {
   $108 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $108;
   $235 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($235|0);
  }
 }
 $109 = HEAP32[$1>>2]|0;
 $110 = ((($109)) + 40|0);
 $111 = HEAP32[$110>>2]|0;
 $112 = HEAP32[$iBuf>>2]|0;
 $113 = (($111) - ($112))|0;
 HEAP32[$nAvail>>2] = $113;
 $114 = HEAP32[$2>>2]|0;
 $115 = HEAP32[$nAvail>>2]|0;
 $116 = ($114|0)<=($115|0);
 do {
  if ($116) {
   $117 = HEAP32[$iBuf>>2]|0;
   $118 = HEAP32[$1>>2]|0;
   $119 = ((($118)) + 36|0);
   $120 = HEAP32[$119>>2]|0;
   $121 = (($120) + ($117)|0);
   $122 = HEAP32[$3>>2]|0;
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[$2>>2]|0;
   $124 = ($123|0)<(0);
   $125 = $124 << 31 >> 31;
   $126 = HEAP32[$1>>2]|0;
   $127 = $126;
   $128 = $127;
   $129 = HEAP32[$128>>2]|0;
   $130 = (($127) + 4)|0;
   $131 = $130;
   $132 = HEAP32[$131>>2]|0;
   $133 = (_i64Add(($129|0),($132|0),($123|0),($125|0))|0);
   $134 = tempRet0;
   $135 = $126;
   $136 = $135;
   HEAP32[$136>>2] = $133;
   $137 = (($135) + 4)|0;
   $138 = $137;
   HEAP32[$138>>2] = $134;
  } else {
   $139 = HEAP32[$1>>2]|0;
   $140 = ((($139)) + 16|0);
   $141 = HEAP32[$140>>2]|0;
   $142 = HEAP32[$2>>2]|0;
   $143 = ($141|0)<($142|0);
   do {
    if ($143) {
     $144 = HEAP32[$1>>2]|0;
     $145 = ((($144)) + 16|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = $146<<1;
     $148 = (128)>($147|0);
     if ($148) {
      $153 = 128;
     } else {
      $149 = HEAP32[$1>>2]|0;
      $150 = ((($149)) + 16|0);
      $151 = HEAP32[$150>>2]|0;
      $152 = $151<<1;
      $153 = $152;
     }
     HEAP32[$nNew>>2] = $153;
     while(1) {
      $154 = HEAP32[$2>>2]|0;
      $155 = HEAP32[$nNew>>2]|0;
      $156 = ($154|0)>($155|0);
      if (!($156)) {
       break;
      }
      $157 = HEAP32[$nNew>>2]|0;
      $158 = $157<<1;
      HEAP32[$nNew>>2] = $158;
     }
     $159 = HEAP32[$1>>2]|0;
     $160 = ((($159)) + 28|0);
     $161 = HEAP32[$160>>2]|0;
     $162 = HEAP32[$nNew>>2]|0;
     $163 = ($162|0)<(0);
     $164 = $163 << 31 >> 31;
     $165 = (_sqlite3Realloc($161,$162,$164)|0);
     HEAP32[$aNew>>2] = $165;
     $166 = HEAP32[$aNew>>2]|0;
     $167 = ($166|0)!=(0|0);
     if ($167) {
      $168 = HEAP32[$nNew>>2]|0;
      $169 = HEAP32[$1>>2]|0;
      $170 = ((($169)) + 16|0);
      HEAP32[$170>>2] = $168;
      $171 = HEAP32[$aNew>>2]|0;
      $172 = HEAP32[$1>>2]|0;
      $173 = ((($172)) + 28|0);
      HEAP32[$173>>2] = $171;
      break;
     }
     HEAP32[$0>>2] = 7;
     $235 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($235|0);
    }
   } while(0);
   $174 = HEAP32[$1>>2]|0;
   $175 = ((($174)) + 28|0);
   $176 = HEAP32[$175>>2]|0;
   $177 = HEAP32[$iBuf>>2]|0;
   $178 = HEAP32[$1>>2]|0;
   $179 = ((($178)) + 36|0);
   $180 = HEAP32[$179>>2]|0;
   $181 = (($180) + ($177)|0);
   $182 = HEAP32[$nAvail>>2]|0;
   _memcpy(($176|0),($181|0),($182|0))|0;
   $183 = HEAP32[$nAvail>>2]|0;
   $184 = ($183|0)<(0);
   $185 = $184 << 31 >> 31;
   $186 = HEAP32[$1>>2]|0;
   $187 = $186;
   $188 = $187;
   $189 = HEAP32[$188>>2]|0;
   $190 = (($187) + 4)|0;
   $191 = $190;
   $192 = HEAP32[$191>>2]|0;
   $193 = (_i64Add(($189|0),($192|0),($183|0),($185|0))|0);
   $194 = tempRet0;
   $195 = $186;
   $196 = $195;
   HEAP32[$196>>2] = $193;
   $197 = (($195) + 4)|0;
   $198 = $197;
   HEAP32[$198>>2] = $194;
   $199 = HEAP32[$2>>2]|0;
   $200 = HEAP32[$nAvail>>2]|0;
   $201 = (($199) - ($200))|0;
   HEAP32[$nRem>>2] = $201;
   while(1) {
    $202 = HEAP32[$nRem>>2]|0;
    $203 = ($202|0)>(0);
    if (!($203)) {
     label = 27;
     break;
    }
    $204 = HEAP32[$nRem>>2]|0;
    HEAP32[$nCopy>>2] = $204;
    $205 = HEAP32[$nRem>>2]|0;
    $206 = HEAP32[$1>>2]|0;
    $207 = ((($206)) + 40|0);
    $208 = HEAP32[$207>>2]|0;
    $209 = ($205|0)>($208|0);
    if ($209) {
     $210 = HEAP32[$1>>2]|0;
     $211 = ((($210)) + 40|0);
     $212 = HEAP32[$211>>2]|0;
     HEAP32[$nCopy>>2] = $212;
    }
    $213 = HEAP32[$1>>2]|0;
    $214 = HEAP32[$nCopy>>2]|0;
    $215 = (_vdbePmaReadBlob($213,$214,$aNext)|0);
    HEAP32[$rc1>>2] = $215;
    $216 = HEAP32[$rc1>>2]|0;
    $217 = ($216|0)!=(0);
    if ($217) {
     break;
    }
    $219 = HEAP32[$2>>2]|0;
    $220 = HEAP32[$nRem>>2]|0;
    $221 = (($219) - ($220))|0;
    $222 = HEAP32[$1>>2]|0;
    $223 = ((($222)) + 28|0);
    $224 = HEAP32[$223>>2]|0;
    $225 = (($224) + ($221)|0);
    $226 = HEAP32[$aNext>>2]|0;
    $227 = HEAP32[$nCopy>>2]|0;
    _memcpy(($225|0),($226|0),($227|0))|0;
    $228 = HEAP32[$nCopy>>2]|0;
    $229 = HEAP32[$nRem>>2]|0;
    $230 = (($229) - ($228))|0;
    HEAP32[$nRem>>2] = $230;
   }
   if ((label|0) == 27) {
    $231 = HEAP32[$1>>2]|0;
    $232 = ((($231)) + 28|0);
    $233 = HEAP32[$232>>2]|0;
    $234 = HEAP32[$3>>2]|0;
    HEAP32[$234>>2] = $233;
    break;
   }
   $218 = HEAP32[$rc1>>2]|0;
   HEAP32[$0>>2] = $218;
   $235 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($235|0);
  }
 } while(0);
 HEAP32[$0>>2] = 0;
 $235 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($235|0);
}
function _vdbeSorterMapFile($pTask,$pFile,$pp) {
 $pTask = $pTask|0;
 $pFile = $pFile|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pFd = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pFd = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $pp;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($13)) + 24|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 144|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)<(0);
 $19 = $18 << 31 >> 31;
 $20 = ($10|0)<($19|0);
 $21 = ($7>>>0)<=($17>>>0);
 $22 = ($10|0)==($19|0);
 $23 = $22 & $21;
 $24 = $20 | $23;
 if (!($24)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 HEAP32[$pFd>>2] = $26;
 $27 = HEAP32[$pFd>>2]|0;
 $28 = HEAP32[$27>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 $30 = ($29|0)>=(3);
 if (!($30)) {
  $42 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 $31 = HEAP32[$pFd>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 8|0);
 $34 = $33;
 $35 = $34;
 $36 = HEAP32[$35>>2]|0;
 $37 = (($34) + 4)|0;
 $38 = $37;
 $39 = HEAP32[$38>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = (_sqlite3OsFetch($31,0,0,$36,$40)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($42|0);
}
function _sqlite3OsFetch($id,$0,$1,$iAmt,$pp) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $iAmt = $iAmt|0;
 $pp = $pp|0;
 var $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $iAmt;
 HEAP32[$5>>2] = $pp;
 $10 = HEAP32[$5>>2]|0;
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return 0;
}
function _vdbeIncrPopulate($pIncr) {
 $pIncr = $pIncr|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0;
 var $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0;
 var $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0;
 var $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0;
 var $96 = 0, $97 = 0, $98 = 0, $99 = 0, $dummy = 0, $iEof = 0, $iStart = 0, $nKey = 0, $pMerger = 0, $pOut = 0, $pReader = 0, $pTask = 0, $rc = 0, $rc2 = 0, $writer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $rc = sp + 84|0;
 $rc2 = sp + 80|0;
 $iStart = sp + 48|0;
 $pOut = sp + 76|0;
 $pTask = sp + 72|0;
 $pMerger = sp + 68|0;
 $writer = sp + 8|0;
 $dummy = sp + 64|0;
 $pReader = sp + 60|0;
 $nKey = sp + 56|0;
 $iEof = sp;
 HEAP32[$0>>2] = $pIncr;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = $2;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = $iStart;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = (($9) + 4)|0;
 $12 = $11;
 HEAP32[$12>>2] = $8;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 32|0);
 $15 = ((($14)) + 16|0);
 HEAP32[$pOut>>2] = $15;
 $16 = HEAP32[$0>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 HEAP32[$pTask>>2] = $17;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 HEAP32[$pMerger>>2] = $20;
 $21 = HEAP32[$pOut>>2]|0;
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pTask>>2]|0;
 $24 = ((($23)) + 8|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = ((($25)) + 12|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = $iStart;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (($28) + 4)|0;
 $32 = $31;
 $33 = HEAP32[$32>>2]|0;
 _vdbePmaWriterInit($22,$writer,$27,$30,$33);
 while(1) {
  $34 = HEAP32[$rc>>2]|0;
  $35 = ($34|0)==(0);
  if (!($35)) {
   break;
  }
  $36 = HEAP32[$pMerger>>2]|0;
  $37 = ((($36)) + 8|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ((($38)) + 4|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$pMerger>>2]|0;
  $42 = ((($41)) + 12|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = (($43) + (($40*56)|0)|0);
  HEAP32[$pReader>>2] = $44;
  $45 = HEAP32[$pReader>>2]|0;
  $46 = ((($45)) + 20|0);
  $47 = HEAP32[$46>>2]|0;
  HEAP32[$nKey>>2] = $47;
  $48 = ((($writer)) + 24|0);
  $49 = $48;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = ((($writer)) + 16|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ($56|0)<(0);
  $58 = $57 << 31 >> 31;
  $59 = (_i64Add(($51|0),($54|0),($56|0),($58|0))|0);
  $60 = tempRet0;
  $61 = $iEof;
  $62 = $61;
  HEAP32[$62>>2] = $59;
  $63 = (($61) + 4)|0;
  $64 = $63;
  HEAP32[$64>>2] = $60;
  $65 = HEAP32[$pReader>>2]|0;
  $66 = ((($65)) + 24|0);
  $67 = HEAP32[$66>>2]|0;
  $68 = ($67|0)==(0|0);
  if ($68) {
   break;
  }
  $69 = $iEof;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = HEAP32[$nKey>>2]|0;
  $76 = ($75|0)<(0);
  $77 = $76 << 31 >> 31;
  $78 = (_i64Add(($71|0),($74|0),($75|0),($77|0))|0);
  $79 = tempRet0;
  $80 = HEAP32[$nKey>>2]|0;
  $81 = ($80|0)<(0);
  $82 = $81 << 31 >> 31;
  $83 = (_sqlite3VarintLen($80,$82)|0);
  $84 = ($83|0)<(0);
  $85 = $84 << 31 >> 31;
  $86 = (_i64Add(($78|0),($79|0),($83|0),($85|0))|0);
  $87 = tempRet0;
  $88 = $iStart;
  $89 = $88;
  $90 = HEAP32[$89>>2]|0;
  $91 = (($88) + 4)|0;
  $92 = $91;
  $93 = HEAP32[$92>>2]|0;
  $94 = HEAP32[$0>>2]|0;
  $95 = ((($94)) + 16|0);
  $96 = HEAP32[$95>>2]|0;
  $97 = ($96|0)<(0);
  $98 = $97 << 31 >> 31;
  $99 = (_i64Add(($90|0),($93|0),($96|0),($98|0))|0);
  $100 = tempRet0;
  $101 = ($87|0)>($100|0);
  $102 = ($86>>>0)>($99>>>0);
  $103 = ($87|0)==($100|0);
  $104 = $103 & $102;
  $105 = $101 | $104;
  if ($105) {
   break;
  }
  $106 = HEAP32[$nKey>>2]|0;
  $107 = ($106|0)<(0);
  $108 = $107 << 31 >> 31;
  _vdbePmaWriteVarint($writer,$106,$108);
  $109 = HEAP32[$pReader>>2]|0;
  $110 = ((($109)) + 32|0);
  $111 = HEAP32[$110>>2]|0;
  $112 = HEAP32[$nKey>>2]|0;
  _vdbePmaWriteBlob($writer,$111,$112);
  $113 = HEAP32[$0>>2]|0;
  $114 = ((($113)) + 4|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = (_vdbeMergeEngineStep($115,$dummy)|0);
  HEAP32[$rc>>2] = $116;
 }
 $117 = HEAP32[$pOut>>2]|0;
 $118 = ((($117)) + 8|0);
 $119 = (_vdbePmaWriterFinish($writer,$118)|0);
 HEAP32[$rc2>>2] = $119;
 $120 = HEAP32[$rc>>2]|0;
 $121 = ($120|0)==(0);
 if (!($121)) {
  $123 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($123|0);
 }
 $122 = HEAP32[$rc2>>2]|0;
 HEAP32[$rc>>2] = $122;
 $123 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($123|0);
}
function _vdbeSorterSetupMerge($pSorter) {
 $pSorter = $pSorter|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pMain = 0, $pTask0 = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $rc = sp + 8|0;
 $pTask0 = sp + 4|0;
 $pMain = sp;
 HEAP32[$0>>2] = $pSorter;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 64|0);
 HEAP32[$pTask0>>2] = $2;
 HEAP32[$pMain>>2] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_vdbeSorterMergeTreeBuild($3,$pMain)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $7 = HEAP32[$pTask0>>2]|0;
  $8 = HEAP32[$pMain>>2]|0;
  $9 = (_vdbeMergeEngineInit($7,$8,0)|0);
  HEAP32[$rc>>2] = $9;
  $10 = HEAP32[$pMain>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 20|0);
  HEAP32[$12>>2] = $10;
  HEAP32[$pMain>>2] = 0;
 }
 $13 = HEAP32[$rc>>2]|0;
 $14 = ($13|0)!=(0);
 if (!($14)) {
  $16 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 $15 = HEAP32[$pMain>>2]|0;
 _vdbeMergeEngineFree($15);
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _vdbeSorterMergeTreeBuild($pSorter,$ppOut) {
 $pSorter = $pSorter|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $i = 0;
 var $iReadOff = 0, $iSeq = 0, $iTask = 0, $nDepth = 0, $nReader = 0, $pMain = 0, $pMerger = 0, $pRoot = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $pMain = sp + 44|0;
 $rc = sp + 40|0;
 $iTask = sp + 36|0;
 $pTask = sp + 32|0;
 $pRoot = sp + 28|0;
 $nDepth = sp + 24|0;
 $iReadOff = sp;
 $i = sp + 20|0;
 $iSeq = sp + 16|0;
 $pMerger = sp + 12|0;
 $nReader = sp + 8|0;
 HEAP32[$0>>2] = $pSorter;
 HEAP32[$1>>2] = $ppOut;
 HEAP32[$pMain>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iTask>>2] = 0;
 while(1) {
  $2 = HEAP32[$rc>>2]|0;
  $3 = ($2|0)==(0);
  if (!($3)) {
   break;
  }
  $4 = HEAP32[$iTask>>2]|0;
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 59|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($4|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$iTask>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 64|0);
  $13 = (($12) + (($10*72)|0)|0);
  HEAP32[$pTask>>2] = $13;
  HEAP32[$pRoot>>2] = 0;
  $14 = HEAP32[$pTask>>2]|0;
  $15 = ((($14)) + 28|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_vdbeSorterTreeDepth($16)|0);
  HEAP32[$nDepth>>2] = $17;
  $18 = $iReadOff;
  $19 = $18;
  HEAP32[$19>>2] = 0;
  $20 = (($18) + 4)|0;
  $21 = $20;
  HEAP32[$21>>2] = 0;
  $22 = HEAP32[$pTask>>2]|0;
  $23 = ((($22)) + 28|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ($24|0)<=(16);
  L5: do {
   if ($25) {
    $26 = HEAP32[$pTask>>2]|0;
    $27 = HEAP32[$pTask>>2]|0;
    $28 = ((($27)) + 28|0);
    $29 = HEAP32[$28>>2]|0;
    $30 = (_vdbeMergeEngineLevel0($26,$29,$iReadOff,$pRoot)|0);
    HEAP32[$rc>>2] = $30;
   } else {
    HEAP32[$iSeq>>2] = 0;
    $31 = (_vdbeMergeEngineNew(16)|0);
    HEAP32[$pRoot>>2] = $31;
    $32 = HEAP32[$pRoot>>2]|0;
    $33 = ($32|0)==(0|0);
    if ($33) {
     HEAP32[$rc>>2] = 7;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $34 = HEAP32[$i>>2]|0;
     $35 = HEAP32[$pTask>>2]|0;
     $36 = ((($35)) + 28|0);
     $37 = HEAP32[$36>>2]|0;
     $38 = ($34|0)<($37|0);
     $39 = HEAP32[$rc>>2]|0;
     $40 = ($39|0)==(0);
     $41 = $38 ? $40 : 0;
     if (!($41)) {
      break L5;
     }
     HEAP32[$pMerger>>2] = 0;
     $42 = HEAP32[$pTask>>2]|0;
     $43 = ((($42)) + 28|0);
     $44 = HEAP32[$43>>2]|0;
     $45 = HEAP32[$i>>2]|0;
     $46 = (($44) - ($45))|0;
     $47 = ($46|0)<(16);
     if ($47) {
      $48 = HEAP32[$pTask>>2]|0;
      $49 = ((($48)) + 28|0);
      $50 = HEAP32[$49>>2]|0;
      $51 = HEAP32[$i>>2]|0;
      $52 = (($50) - ($51))|0;
      $53 = $52;
     } else {
      $53 = 16;
     }
     HEAP32[$nReader>>2] = $53;
     $54 = HEAP32[$pTask>>2]|0;
     $55 = HEAP32[$nReader>>2]|0;
     $56 = (_vdbeMergeEngineLevel0($54,$55,$iReadOff,$pMerger)|0);
     HEAP32[$rc>>2] = $56;
     $57 = HEAP32[$rc>>2]|0;
     $58 = ($57|0)==(0);
     if ($58) {
      $59 = HEAP32[$pTask>>2]|0;
      $60 = HEAP32[$nDepth>>2]|0;
      $61 = HEAP32[$iSeq>>2]|0;
      $62 = (($61) + 1)|0;
      HEAP32[$iSeq>>2] = $62;
      $63 = HEAP32[$pRoot>>2]|0;
      $64 = HEAP32[$pMerger>>2]|0;
      $65 = (_vdbeSorterAddToTree($59,$60,$61,$63,$64)|0);
      HEAP32[$rc>>2] = $65;
     }
     $66 = HEAP32[$i>>2]|0;
     $67 = (($66) + 16)|0;
     HEAP32[$i>>2] = $67;
    }
   }
  } while(0);
  $68 = HEAP32[$rc>>2]|0;
  $69 = ($68|0)==(0);
  $70 = HEAP32[$pRoot>>2]|0;
  if ($69) {
   HEAP32[$pMain>>2] = $70;
  } else {
   _vdbeMergeEngineFree($70);
  }
  $71 = HEAP32[$iTask>>2]|0;
  $72 = (($71) + 1)|0;
  HEAP32[$iTask>>2] = $72;
 }
 $73 = HEAP32[$rc>>2]|0;
 $74 = ($73|0)!=(0);
 if (!($74)) {
  $76 = HEAP32[$pMain>>2]|0;
  $77 = HEAP32[$1>>2]|0;
  HEAP32[$77>>2] = $76;
  $78 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $75 = HEAP32[$pMain>>2]|0;
 _vdbeMergeEngineFree($75);
 HEAP32[$pMain>>2] = 0;
 $76 = HEAP32[$pMain>>2]|0;
 $77 = HEAP32[$1>>2]|0;
 HEAP32[$77>>2] = $76;
 $78 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _vdbeMergeEngineInit($pTask,$pMerger,$eMode) {
 $pTask = $pTask|0;
 $pMerger = $pMerger|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0;
 var $nTree = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $rc = sp + 8|0;
 $i = sp + 4|0;
 $nTree = sp;
 HEAP32[$1>>2] = $pTask;
 HEAP32[$2>>2] = $pMerger;
 HEAP32[$3>>2] = $eMode;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nTree>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 HEAP32[$8>>2] = $6;
 HEAP32[$i>>2] = 0;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$nTree>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$2>>2]|0;
  $14 = ((($13)) + 12|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (($15) + (($12*56)|0)|0);
  $17 = (_vdbePmaReaderIncrInit($16,0)|0);
  HEAP32[$rc>>2] = $17;
  $18 = HEAP32[$rc>>2]|0;
  $19 = ($18|0)!=(0);
  if ($19) {
   label = 4;
   break;
  }
  $21 = HEAP32[$i>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$i>>2] = $22;
 }
 if ((label|0) == 4) {
  $20 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $20;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$23>>2]|0;
 $25 = (($24) - 1)|0;
 HEAP32[$i>>2] = $25;
 while(1) {
  $26 = HEAP32[$i>>2]|0;
  $27 = ($26|0)>(0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$2>>2]|0;
  $29 = HEAP32[$i>>2]|0;
  _vdbeMergeEngineCompare($28,$29);
  $30 = HEAP32[$i>>2]|0;
  $31 = (($30) + -1)|0;
  HEAP32[$i>>2] = $31;
 }
 $32 = HEAP32[$1>>2]|0;
 $33 = ((($32)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = ((($34)) + 11|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 HEAP32[$0>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _vdbePmaReaderIncrInit($pReadr,$eMode) {
 $pReadr = $pReadr|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pIncr = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pReadr;
 HEAP32[$1>>2] = $eMode;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIncr>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pIncr>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $10 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = (_vdbePmaReaderIncrMergeInit($7,$8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _vdbeMergeEngineCompare($pMerger,$iOut) {
 $pMerger = $pMerger|0;
 $iOut = $iOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $bCached = 0;
 var $i1 = 0, $i2 = 0, $iRes = 0, $p1 = 0, $p2 = 0, $pTask = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $i1 = sp + 28|0;
 $i2 = sp + 24|0;
 $iRes = sp + 20|0;
 $p1 = sp + 16|0;
 $p2 = sp + 12|0;
 $pTask = sp + 8|0;
 $bCached = sp + 4|0;
 $res = sp;
 HEAP32[$0>>2] = $pMerger;
 HEAP32[$1>>2] = $iOut;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (($4|0) / 2)&-1;
 $6 = ($2|0)>=($5|0);
 $7 = HEAP32[$1>>2]|0;
 if ($6) {
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$8>>2]|0;
  $10 = (($9|0) / 2)&-1;
  $11 = (($7) - ($10))|0;
  $12 = $11<<1;
  HEAP32[$i1>>2] = $12;
  $13 = HEAP32[$i1>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$i2>>2] = $14;
 } else {
  $15 = $7<<1;
  $16 = HEAP32[$0>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($15<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  HEAP32[$i1>>2] = $20;
  $21 = HEAP32[$1>>2]|0;
  $22 = $21<<1;
  $23 = (($22) + 1)|0;
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (($26) + ($23<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$i2>>2] = $28;
 }
 $29 = HEAP32[$i1>>2]|0;
 $30 = HEAP32[$0>>2]|0;
 $31 = ((($30)) + 12|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = (($32) + (($29*56)|0)|0);
 HEAP32[$p1>>2] = $33;
 $34 = HEAP32[$i2>>2]|0;
 $35 = HEAP32[$0>>2]|0;
 $36 = ((($35)) + 12|0);
 $37 = HEAP32[$36>>2]|0;
 $38 = (($37) + (($34*56)|0)|0);
 HEAP32[$p2>>2] = $38;
 $39 = HEAP32[$p1>>2]|0;
 $40 = ((($39)) + 24|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ($41|0)==(0|0);
 do {
  if ($42) {
   $43 = HEAP32[$i2>>2]|0;
   HEAP32[$iRes>>2] = $43;
  } else {
   $44 = HEAP32[$p2>>2]|0;
   $45 = ((($44)) + 24|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==(0|0);
   if ($47) {
    $48 = HEAP32[$i1>>2]|0;
    HEAP32[$iRes>>2] = $48;
    break;
   }
   $49 = HEAP32[$0>>2]|0;
   $50 = ((($49)) + 4|0);
   $51 = HEAP32[$50>>2]|0;
   HEAP32[$pTask>>2] = $51;
   HEAP32[$bCached>>2] = 0;
   $52 = HEAP32[$pTask>>2]|0;
   $53 = ((($52)) + 32|0);
   $54 = HEAP32[$53>>2]|0;
   $55 = HEAP32[$pTask>>2]|0;
   $56 = HEAP32[$p1>>2]|0;
   $57 = ((($56)) + 32|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$p1>>2]|0;
   $60 = ((($59)) + 20|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = HEAP32[$p2>>2]|0;
   $63 = ((($62)) + 32|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = HEAP32[$p2>>2]|0;
   $66 = ((($65)) + 20|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = (FUNCTION_TABLE_iiiiiii[$54 & 255]($55,$bCached,$58,$61,$64,$67)|0);
   HEAP32[$res>>2] = $68;
   $69 = HEAP32[$res>>2]|0;
   $70 = ($69|0)<=(0);
   if ($70) {
    $71 = HEAP32[$i1>>2]|0;
    HEAP32[$iRes>>2] = $71;
    break;
   } else {
    $72 = HEAP32[$i2>>2]|0;
    HEAP32[$iRes>>2] = $72;
    break;
   }
  }
 } while(0);
 $73 = HEAP32[$iRes>>2]|0;
 $74 = HEAP32[$1>>2]|0;
 $75 = HEAP32[$0>>2]|0;
 $76 = ((($75)) + 8|0);
 $77 = HEAP32[$76>>2]|0;
 $78 = (($77) + ($74<<2)|0);
 HEAP32[$78>>2] = $73;
 STACKTOP = sp;return;
}
function _vdbePmaReaderIncrMergeInit($pReadr,$eMode) {
 $pReadr = $pReadr|0;
 $eMode = $eMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $db = 0, $mxSz = 0, $pIncr = 0, $pTask = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $rc = sp + 16|0;
 $pIncr = sp + 12|0;
 $pTask = sp + 8|0;
 $db = sp + 4|0;
 $mxSz = sp;
 HEAP32[$0>>2] = $pReadr;
 HEAP32[$1>>2] = $eMode;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 48|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIncr>>2] = $4;
 $5 = HEAP32[$pIncr>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pTask>>2] = $6;
 $7 = HEAP32[$pTask>>2]|0;
 $8 = ((($7)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$db>>2] = $11;
 $12 = HEAP32[$pTask>>2]|0;
 $13 = HEAP32[$pIncr>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$1>>2]|0;
 $17 = (_vdbeMergeEngineInit($12,$15,$16)|0);
 HEAP32[$rc>>2] = $17;
 $18 = HEAP32[$rc>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$pIncr>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$mxSz>>2] = $22;
  $23 = HEAP32[$pTask>>2]|0;
  $24 = ((($23)) + 56|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = ($25|0)==(0|0);
  if ($26) {
   $27 = HEAP32[$db>>2]|0;
   $28 = HEAP32[$pTask>>2]|0;
   $29 = ((($28)) + 56|0);
   $30 = ((($29)) + 8|0);
   $31 = $30;
   $32 = $31;
   $33 = HEAP32[$32>>2]|0;
   $34 = (($31) + 4)|0;
   $35 = $34;
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pTask>>2]|0;
   $38 = ((($37)) + 56|0);
   $39 = (_vdbeSorterOpenTempFile($27,$33,$36,$38)|0);
   HEAP32[$rc>>2] = $39;
   $40 = HEAP32[$pTask>>2]|0;
   $41 = ((($40)) + 56|0);
   $42 = ((($41)) + 8|0);
   $43 = $42;
   $44 = $43;
   HEAP32[$44>>2] = 0;
   $45 = (($43) + 4)|0;
   $46 = $45;
   HEAP32[$46>>2] = 0;
  }
  $47 = HEAP32[$rc>>2]|0;
  $48 = ($47|0)==(0);
  if ($48) {
   $49 = HEAP32[$pTask>>2]|0;
   $50 = ((($49)) + 56|0);
   $51 = HEAP32[$50>>2]|0;
   $52 = HEAP32[$pIncr>>2]|0;
   $53 = ((($52)) + 32|0);
   $54 = ((($53)) + 16|0);
   HEAP32[$54>>2] = $51;
   $55 = HEAP32[$pTask>>2]|0;
   $56 = ((($55)) + 56|0);
   $57 = ((($56)) + 8|0);
   $58 = $57;
   $59 = $58;
   $60 = HEAP32[$59>>2]|0;
   $61 = (($58) + 4)|0;
   $62 = $61;
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$pIncr>>2]|0;
   $65 = ((($64)) + 8|0);
   $66 = $65;
   $67 = $66;
   HEAP32[$67>>2] = $60;
   $68 = (($66) + 4)|0;
   $69 = $68;
   HEAP32[$69>>2] = $63;
   $70 = HEAP32[$mxSz>>2]|0;
   $71 = ($70|0)<(0);
   $72 = $71 << 31 >> 31;
   $73 = HEAP32[$pTask>>2]|0;
   $74 = ((($73)) + 56|0);
   $75 = ((($74)) + 8|0);
   $76 = $75;
   $77 = $76;
   $78 = HEAP32[$77>>2]|0;
   $79 = (($76) + 4)|0;
   $80 = $79;
   $81 = HEAP32[$80>>2]|0;
   $82 = (_i64Add(($78|0),($81|0),($70|0),($72|0))|0);
   $83 = tempRet0;
   $84 = $75;
   $85 = $84;
   HEAP32[$85>>2] = $82;
   $86 = (($84) + 4)|0;
   $87 = $86;
   HEAP32[$87>>2] = $83;
  }
 }
 $88 = HEAP32[$rc>>2]|0;
 $89 = ($88|0)==(0);
 if (!($89)) {
  $92 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($92|0);
 }
 $90 = HEAP32[$0>>2]|0;
 $91 = (_vdbePmaReaderNext($90)|0);
 HEAP32[$rc>>2] = $91;
 $92 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($92|0);
}
function _vdbeSorterTreeDepth($nPMA) {
 $nPMA = $nPMA|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nDepth = 0, $nDiv = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $nDepth = sp + 8|0;
 $nDiv = sp;
 HEAP32[$0>>2] = $nPMA;
 HEAP32[$nDepth>>2] = 0;
 $1 = $nDiv;
 $2 = $1;
 HEAP32[$2>>2] = 16;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 while(1) {
  $5 = $nDiv;
  $6 = $5;
  $7 = HEAP32[$6>>2]|0;
  $8 = (($5) + 4)|0;
  $9 = $8;
  $10 = HEAP32[$9>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = ($10|0)<($13|0);
  $15 = ($7>>>0)<($11>>>0);
  $16 = ($10|0)==($13|0);
  $17 = $16 & $15;
  $18 = $14 | $17;
  if (!($18)) {
   break;
  }
  $19 = $nDiv;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = (___muldi3(($21|0),($24|0),16,0)|0);
  $26 = tempRet0;
  $27 = $nDiv;
  $28 = $27;
  HEAP32[$28>>2] = $25;
  $29 = (($27) + 4)|0;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = HEAP32[$nDepth>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$nDepth>>2] = $32;
 }
 $33 = HEAP32[$nDepth>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _vdbeMergeEngineLevel0($pTask,$nPMA,$piOffset,$ppOut) {
 $pTask = $pTask|0;
 $nPMA = $nPMA|0;
 $piOffset = $piOffset|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $i = 0, $iOff = 0, $nDummy = 0, $pNew = 0;
 var $pReadr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pNew = sp + 28|0;
 $iOff = sp + 8|0;
 $i = sp + 24|0;
 $rc = sp + 20|0;
 $nDummy = sp;
 $pReadr = sp + 16|0;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $nPMA;
 HEAP32[$2>>2] = $piOffset;
 HEAP32[$3>>2] = $ppOut;
 $4 = HEAP32[$2>>2]|0;
 $5 = $4;
 $6 = $5;
 $7 = HEAP32[$6>>2]|0;
 $8 = (($5) + 4)|0;
 $9 = $8;
 $10 = HEAP32[$9>>2]|0;
 $11 = $iOff;
 $12 = $11;
 HEAP32[$12>>2] = $7;
 $13 = (($11) + 4)|0;
 $14 = $13;
 HEAP32[$14>>2] = $10;
 HEAP32[$rc>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = (_vdbeMergeEngineNew($15)|0);
 HEAP32[$pNew>>2] = $16;
 $17 = HEAP32[$3>>2]|0;
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ($18|0)==(0|0);
 if ($19) {
  HEAP32[$rc>>2] = 7;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$1>>2]|0;
  $22 = ($20|0)<($21|0);
  $23 = HEAP32[$rc>>2]|0;
  $24 = ($23|0)==(0);
  $25 = $22 ? $24 : 0;
  if (!($25)) {
   break;
  }
  $26 = $nDummy;
  $27 = $26;
  HEAP32[$27>>2] = 0;
  $28 = (($26) + 4)|0;
  $29 = $28;
  HEAP32[$29>>2] = 0;
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$pNew>>2]|0;
  $32 = ((($31)) + 12|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + (($30*56)|0)|0);
  HEAP32[$pReadr>>2] = $34;
  $35 = HEAP32[$0>>2]|0;
  $36 = HEAP32[$0>>2]|0;
  $37 = ((($36)) + 40|0);
  $38 = $iOff;
  $39 = $38;
  $40 = HEAP32[$39>>2]|0;
  $41 = (($38) + 4)|0;
  $42 = $41;
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$pReadr>>2]|0;
  $45 = (_vdbePmaReaderInit($35,$37,$40,$43,$44,$nDummy)|0);
  HEAP32[$rc>>2] = $45;
  $46 = HEAP32[$pReadr>>2]|0;
  $47 = ((($46)) + 8|0);
  $48 = $47;
  $49 = $48;
  $50 = HEAP32[$49>>2]|0;
  $51 = (($48) + 4)|0;
  $52 = $51;
  $53 = HEAP32[$52>>2]|0;
  $54 = $iOff;
  $55 = $54;
  HEAP32[$55>>2] = $50;
  $56 = (($54) + 4)|0;
  $57 = $56;
  HEAP32[$57>>2] = $53;
  $58 = HEAP32[$i>>2]|0;
  $59 = (($58) + 1)|0;
  HEAP32[$i>>2] = $59;
 }
 $60 = HEAP32[$rc>>2]|0;
 $61 = ($60|0)!=(0);
 if (!($61)) {
  $64 = $iOff;
  $65 = $64;
  $66 = HEAP32[$65>>2]|0;
  $67 = (($64) + 4)|0;
  $68 = $67;
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = $70;
  $72 = $71;
  HEAP32[$72>>2] = $66;
  $73 = (($71) + 4)|0;
  $74 = $73;
  HEAP32[$74>>2] = $69;
  $75 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 $62 = HEAP32[$pNew>>2]|0;
 _vdbeMergeEngineFree($62);
 $63 = HEAP32[$3>>2]|0;
 HEAP32[$63>>2] = 0;
 $64 = $iOff;
 $65 = $64;
 $66 = HEAP32[$65>>2]|0;
 $67 = (($64) + 4)|0;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = HEAP32[$2>>2]|0;
 $71 = $70;
 $72 = $71;
 HEAP32[$72>>2] = $66;
 $73 = (($71) + 4)|0;
 $74 = $73;
 HEAP32[$74>>2] = $69;
 $75 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($75|0);
}
function _vdbeMergeEngineNew($nReader) {
 $nReader = $nReader|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $N = 0, $nByte = 0, $pNew = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $N = sp + 8|0;
 $nByte = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $nReader;
 HEAP32[$N>>2] = 2;
 while(1) {
  $1 = HEAP32[$N>>2]|0;
  $2 = HEAP32[$0>>2]|0;
  $3 = ($1|0)<($2|0);
  $4 = HEAP32[$N>>2]|0;
  if (!($3)) {
   break;
  }
  $5 = HEAP32[$N>>2]|0;
  $6 = (($5) + ($4))|0;
  HEAP32[$N>>2] = $6;
 }
 $7 = ($4*60)|0;
 $8 = (16 + ($7))|0;
 HEAP32[$nByte>>2] = $8;
 $9 = (_sqlite3FaultSim(100)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $15 = 0;
 } else {
  $11 = HEAP32[$nByte>>2]|0;
  $12 = ($11|0)<(0);
  $13 = $12 << 31 >> 31;
  $14 = (_sqlite3MallocZero($11,$13)|0);
  $15 = $14;
 }
 HEAP32[$pNew>>2] = $15;
 $16 = HEAP32[$pNew>>2]|0;
 $17 = ($16|0)!=(0|0);
 if (!($17)) {
  $33 = HEAP32[$pNew>>2]|0;
  STACKTOP = sp;return ($33|0);
 }
 $18 = HEAP32[$N>>2]|0;
 $19 = HEAP32[$pNew>>2]|0;
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[$pNew>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$21>>2] = 0;
 $22 = HEAP32[$pNew>>2]|0;
 $23 = ((($22)) + 16|0);
 $24 = HEAP32[$pNew>>2]|0;
 $25 = ((($24)) + 12|0);
 HEAP32[$25>>2] = $23;
 $26 = HEAP32[$N>>2]|0;
 $27 = HEAP32[$pNew>>2]|0;
 $28 = ((($27)) + 12|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + (($26*56)|0)|0);
 $31 = HEAP32[$pNew>>2]|0;
 $32 = ((($31)) + 8|0);
 HEAP32[$32>>2] = $30;
 $33 = HEAP32[$pNew>>2]|0;
 STACKTOP = sp;return ($33|0);
}
function _vdbeSorterAddToTree($pTask,$nDepth,$iSeq,$pRoot,$pLeaf) {
 $pTask = $pTask|0;
 $nDepth = $nDepth|0;
 $iSeq = $iSeq|0;
 $pRoot = $pRoot|0;
 $pLeaf = $pLeaf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $iIter = 0, $nDiv = 0, $p = 0, $pIncr = 0, $pNew = 0, $pReadr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $3 = sp + 36|0;
 $4 = sp + 32|0;
 $rc = sp + 28|0;
 $nDiv = sp + 24|0;
 $i = sp + 20|0;
 $p = sp + 16|0;
 $pIncr = sp + 12|0;
 $iIter = sp + 8|0;
 $pReadr = sp + 4|0;
 $pNew = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $nDepth;
 HEAP32[$2>>2] = $iSeq;
 HEAP32[$3>>2] = $pRoot;
 HEAP32[$4>>2] = $pLeaf;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nDiv>>2] = 1;
 $5 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$4>>2]|0;
 $8 = (_vdbeIncrMergerNew($6,$7,$pIncr)|0);
 HEAP32[$rc>>2] = $8;
 HEAP32[$i>>2] = 1;
 while(1) {
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$1>>2]|0;
  $11 = ($9|0)<($10|0);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$nDiv>>2]|0;
  $13 = $12<<4;
  HEAP32[$nDiv>>2] = $13;
  $14 = HEAP32[$i>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 HEAP32[$i>>2] = 1;
 while(1) {
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = ($16|0)<($17|0);
  $19 = HEAP32[$rc>>2]|0;
  $20 = ($19|0)==(0);
  $21 = $18 ? $20 : 0;
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$nDiv>>2]|0;
  $24 = (($22|0) / ($23|0))&-1;
  $25 = (($24|0) % 16)&-1;
  HEAP32[$iIter>>2] = $25;
  $26 = HEAP32[$iIter>>2]|0;
  $27 = HEAP32[$p>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + (($26*56)|0)|0);
  HEAP32[$pReadr>>2] = $30;
  $31 = HEAP32[$pReadr>>2]|0;
  $32 = ((($31)) + 48|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = ($33|0)==(0|0);
  do {
   if ($34) {
    $35 = (_vdbeMergeEngineNew(16)|0);
    HEAP32[$pNew>>2] = $35;
    $36 = HEAP32[$pNew>>2]|0;
    $37 = ($36|0)==(0|0);
    if ($37) {
     HEAP32[$rc>>2] = 7;
     break;
    } else {
     $38 = HEAP32[$0>>2]|0;
     $39 = HEAP32[$pNew>>2]|0;
     $40 = HEAP32[$pReadr>>2]|0;
     $41 = ((($40)) + 48|0);
     $42 = (_vdbeIncrMergerNew($38,$39,$41)|0);
     HEAP32[$rc>>2] = $42;
     break;
    }
   }
  } while(0);
  $43 = HEAP32[$rc>>2]|0;
  $44 = ($43|0)==(0);
  if ($44) {
   $45 = HEAP32[$pReadr>>2]|0;
   $46 = ((($45)) + 48|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = ((($47)) + 4|0);
   $49 = HEAP32[$48>>2]|0;
   HEAP32[$p>>2] = $49;
   $50 = HEAP32[$nDiv>>2]|0;
   $51 = (($50|0) / 16)&-1;
   HEAP32[$nDiv>>2] = $51;
  }
  $52 = HEAP32[$i>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$i>>2] = $53;
 }
 $54 = HEAP32[$rc>>2]|0;
 $55 = ($54|0)==(0);
 $56 = HEAP32[$pIncr>>2]|0;
 if ($55) {
  $57 = HEAP32[$2>>2]|0;
  $58 = (($57|0) % 16)&-1;
  $59 = HEAP32[$p>>2]|0;
  $60 = ((($59)) + 12|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = (($61) + (($58*56)|0)|0);
  $63 = ((($62)) + 48|0);
  HEAP32[$63>>2] = $56;
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 } else {
  _vdbeIncrFree($56);
  $64 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($64|0);
 }
 return (0)|0;
}
function _vdbeIncrMergerNew($pTask,$pMerger,$ppOut) {
 $pTask = $pTask|0;
 $pMerger = $pMerger|0;
 $ppOut = $ppOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $pIncr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $rc = sp + 4|0;
 $pIncr = sp;
 HEAP32[$0>>2] = $pTask;
 HEAP32[$1>>2] = $pMerger;
 HEAP32[$2>>2] = $ppOut;
 HEAP32[$rc>>2] = 0;
 $3 = (_sqlite3FaultSim(100)|0);
 $4 = ($3|0)!=(0);
 if ($4) {
  $7 = 0;
 } else {
  $5 = (_sqlite3MallocZero(64,0)|0);
  $7 = $5;
 }
 $6 = HEAP32[$2>>2]|0;
 HEAP32[$6>>2] = $7;
 HEAP32[$pIncr>>2] = $7;
 $8 = HEAP32[$pIncr>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$1>>2]|0;
 if (!($9)) {
  _vdbeMergeEngineFree($10);
  HEAP32[$rc>>2] = 7;
  $60 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($60|0);
 }
 $11 = HEAP32[$pIncr>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pIncr>>2]|0;
 HEAP32[$14>>2] = $13;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 8|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($17)) + 8|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) + 9)|0;
 $21 = HEAP32[$0>>2]|0;
 $22 = ((($21)) + 8|0);
 $23 = HEAP32[$22>>2]|0;
 $24 = ((($23)) + 4|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25|0) / 2)&-1;
 $27 = ($20|0)>($26|0);
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 if ($27) {
  $31 = ((($30)) + 8|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (($32) + 9)|0;
  $39 = $33;
 } else {
  $34 = ((($30)) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (($35|0) / 2)&-1;
  $39 = $36;
 }
 $37 = HEAP32[$pIncr>>2]|0;
 $38 = ((($37)) + 16|0);
 HEAP32[$38>>2] = $39;
 $40 = HEAP32[$pIncr>>2]|0;
 $41 = ((($40)) + 16|0);
 $42 = HEAP32[$41>>2]|0;
 $43 = ($42|0)<(0);
 $44 = $43 << 31 >> 31;
 $45 = HEAP32[$0>>2]|0;
 $46 = ((($45)) + 56|0);
 $47 = ((($46)) + 8|0);
 $48 = $47;
 $49 = $48;
 $50 = HEAP32[$49>>2]|0;
 $51 = (($48) + 4)|0;
 $52 = $51;
 $53 = HEAP32[$52>>2]|0;
 $54 = (_i64Add(($50|0),($53|0),($42|0),($44|0))|0);
 $55 = tempRet0;
 $56 = $47;
 $57 = $56;
 HEAP32[$57>>2] = $54;
 $58 = (($56) + 4)|0;
 $59 = $58;
 HEAP32[$59>>2] = $55;
 $60 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($60|0);
}
function _vdbePmaReaderInit($pTask,$pFile,$0,$1,$pReadr,$pnByte) {
 $pTask = $pTask|0;
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 $pReadr = $pReadr|0;
 $pnByte = $pnByte|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $nByte = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 8|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $rc = sp + 16|0;
 $nByte = sp;
 HEAP32[$2>>2] = $pTask;
 HEAP32[$3>>2] = $pFile;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $pReadr;
 HEAP32[$6>>2] = $pnByte;
 $11 = HEAP32[$2>>2]|0;
 $12 = HEAP32[$5>>2]|0;
 $13 = HEAP32[$3>>2]|0;
 $14 = $4;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_vdbePmaReaderSeek($11,$12,$13,$16,$19)|0);
 HEAP32[$rc>>2] = $20;
 $21 = HEAP32[$rc>>2]|0;
 $22 = ($21|0)==(0);
 if ($22) {
  $23 = $nByte;
  $24 = $23;
  HEAP32[$24>>2] = 0;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = 0;
  $27 = HEAP32[$5>>2]|0;
  $28 = (_vdbePmaReadVarint($27,$nByte)|0);
  HEAP32[$rc>>2] = $28;
  $29 = HEAP32[$5>>2]|0;
  $30 = $29;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = (($30) + 4)|0;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = $nByte;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (($36) + 4)|0;
  $40 = $39;
  $41 = HEAP32[$40>>2]|0;
  $42 = (_i64Add(($32|0),($35|0),($38|0),($41|0))|0);
  $43 = tempRet0;
  $44 = HEAP32[$5>>2]|0;
  $45 = ((($44)) + 8|0);
  $46 = $45;
  $47 = $46;
  HEAP32[$47>>2] = $42;
  $48 = (($46) + 4)|0;
  $49 = $48;
  HEAP32[$49>>2] = $43;
  $50 = $nByte;
  $51 = $50;
  $52 = HEAP32[$51>>2]|0;
  $53 = (($50) + 4)|0;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$6>>2]|0;
  $57 = $56;
  $58 = $57;
  $59 = HEAP32[$58>>2]|0;
  $60 = (($57) + 4)|0;
  $61 = $60;
  $62 = HEAP32[$61>>2]|0;
  $63 = (_i64Add(($59|0),($62|0),($52|0),($55|0))|0);
  $64 = tempRet0;
  $65 = $56;
  $66 = $65;
  HEAP32[$66>>2] = $63;
  $67 = (($65) + 4)|0;
  $68 = $67;
  HEAP32[$68>>2] = $64;
 }
 $69 = HEAP32[$rc>>2]|0;
 $70 = ($69|0)==(0);
 if (!($70)) {
  $73 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($73|0);
 }
 $71 = HEAP32[$5>>2]|0;
 $72 = (_vdbePmaReaderNext($71)|0);
 HEAP32[$rc>>2] = $72;
 $73 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($73|0);
}
function _sqlite3BtreeCursorHasMoved($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCur;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 66|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = $3&255;
 $5 = ($4|0)!=(1);
 $6 = $5&1;
 STACKTOP = sp;return ($6|0);
}
function _handleMovedCursor($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $isDifferentRow = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $isDifferentRow = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3BtreeCursorRestore($3,$isDifferentRow)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 56|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$isDifferentRow>>2]|0;
 $8 = ($7|0)!=(0);
 if (!($8)) {
  $11 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($11|0);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 2|0);
 HEAP8[$10>>0] = 1;
 $11 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($11|0);
}
function _sqlite3BtreeCursorRestore($pCur,$pDifferentRow) {
 $pCur = $pCur|0;
 $pDifferentRow = $pDifferentRow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCur;
 HEAP32[$2>>2] = $pDifferentRow;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 66|0);
 $5 = HEAP8[$4>>0]|0;
 $6 = $5&255;
 $7 = ($6|0)>=(3);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = (_btreeRestoreCursorPosition($8)|0);
  $10 = $9;
 } else {
  $10 = 0;
 }
 HEAP32[$rc>>2] = $10;
 $11 = HEAP32[$rc>>2]|0;
 $12 = ($11|0)!=(0);
 if ($12) {
  $13 = HEAP32[$2>>2]|0;
  HEAP32[$13>>2] = 1;
  $14 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $14;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 66|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)!=(1);
 $20 = HEAP32[$2>>2]|0;
 if ($19) {
  HEAP32[$20>>2] = 1;
 } else {
  HEAP32[$20>>2] = 0;
 }
 HEAP32[$0>>2] = 0;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _vdbeSorterRowkey($pSorter,$pnKey) {
 $pSorter = $pSorter|0;
 $pnKey = $pnKey|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pKey = 0, $pReader = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pKey = sp + 4|0;
 $pReader = sp;
 HEAP32[$0>>2] = $pSorter;
 HEAP32[$1>>2] = $pnKey;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 $6 = HEAP32[$0>>2]|0;
 if ($5) {
  $7 = ((($6)) + 20|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ((($8)) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 4|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 20|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ((($15)) + 12|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = (($17) + (($12*56)|0)|0);
  HEAP32[$pReader>>2] = $18;
  $19 = HEAP32[$pReader>>2]|0;
  $20 = ((($19)) + 20|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = HEAP32[$1>>2]|0;
  HEAP32[$22>>2] = $21;
  $23 = HEAP32[$pReader>>2]|0;
  $24 = ((($23)) + 32|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pKey>>2] = $25;
  $34 = HEAP32[$pKey>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $26 = ((($6)) + 36|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = HEAP32[$27>>2]|0;
  $29 = HEAP32[$1>>2]|0;
  HEAP32[$29>>2] = $28;
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 36|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ((($32)) + 8|0);
  HEAP32[$pKey>>2] = $33;
  $34 = HEAP32[$pKey>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return (0)|0;
}
function _dropCell($pPage,$idx,$sz,$pRC) {
 $pPage = $pPage|0;
 $idx = $idx|0;
 $sz = $sz|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $data = 0, $hdr = 0, $pc = 0, $ptr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pc = sp + 16|0;
 $data = sp + 12|0;
 $ptr = sp + 8|0;
 $rc = sp + 4|0;
 $hdr = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $sz;
 HEAP32[$3>>2] = $pRC;
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 56|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$data>>2] = $9;
 $10 = HEAP32[$1>>2]|0;
 $11 = $10<<1;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 64|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (($14) + ($11)|0);
 HEAP32[$ptr>>2] = $15;
 $16 = HEAP32[$ptr>>2]|0;
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = $18 << 8;
 $20 = HEAP32[$ptr>>2]|0;
 $21 = ((($20)) + 1|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = $19 | $23;
 HEAP32[$pc>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = ((($25)) + 5|0);
 $27 = HEAP8[$26>>0]|0;
 $28 = $27&255;
 HEAP32[$hdr>>2] = $28;
 $29 = HEAP32[$pc>>2]|0;
 $30 = HEAP32[$hdr>>2]|0;
 $31 = (($30) + 5)|0;
 $32 = HEAP32[$data>>2]|0;
 $33 = (($32) + ($31)|0);
 $34 = HEAP8[$33>>0]|0;
 $35 = $34&255;
 $36 = $35 << 8;
 $37 = HEAP32[$hdr>>2]|0;
 $38 = (($37) + 5)|0;
 $39 = HEAP32[$data>>2]|0;
 $40 = (($39) + ($38)|0);
 $41 = ((($40)) + 1|0);
 $42 = HEAP8[$41>>0]|0;
 $43 = $42&255;
 $44 = $36 | $43;
 $45 = ($29>>>0)<($44>>>0);
 if (!($45)) {
  $46 = HEAP32[$pc>>2]|0;
  $47 = HEAP32[$2>>2]|0;
  $48 = (($46) + ($47))|0;
  $49 = HEAP32[$0>>2]|0;
  $50 = ((($49)) + 52|0);
  $51 = HEAP32[$50>>2]|0;
  $52 = ((($51)) + 36|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ($48>>>0)>($53>>>0);
  if (!($54)) {
   $57 = HEAP32[$0>>2]|0;
   $58 = HEAP32[$pc>>2]|0;
   $59 = $58&65535;
   $60 = HEAP32[$2>>2]|0;
   $61 = $60&65535;
   $62 = (_freeSpace($57,$59,$61)|0);
   HEAP32[$rc>>2] = $62;
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)!=(0);
   if ($64) {
    $65 = HEAP32[$rc>>2]|0;
    $66 = HEAP32[$3>>2]|0;
    HEAP32[$66>>2] = $65;
    STACKTOP = sp;return;
   }
   $67 = HEAP32[$0>>2]|0;
   $68 = ((($67)) + 18|0);
   $69 = HEAP16[$68>>1]|0;
   $70 = (($69) + -1)<<16>>16;
   HEAP16[$68>>1] = $70;
   $71 = HEAP32[$0>>2]|0;
   $72 = ((($71)) + 18|0);
   $73 = HEAP16[$72>>1]|0;
   $74 = $73&65535;
   $75 = ($74|0)==(0);
   if ($75) {
    $76 = HEAP32[$hdr>>2]|0;
    $77 = (($76) + 1)|0;
    $78 = HEAP32[$data>>2]|0;
    $79 = (($78) + ($77)|0);
    ;HEAP8[$79>>0]=0|0;HEAP8[$79+1>>0]=0|0;HEAP8[$79+2>>0]=0|0;HEAP8[$79+3>>0]=0|0;
    $80 = HEAP32[$hdr>>2]|0;
    $81 = (($80) + 7)|0;
    $82 = HEAP32[$data>>2]|0;
    $83 = (($82) + ($81)|0);
    HEAP8[$83>>0] = 0;
    $84 = HEAP32[$0>>2]|0;
    $85 = ((($84)) + 52|0);
    $86 = HEAP32[$85>>2]|0;
    $87 = ((($86)) + 36|0);
    $88 = HEAP32[$87>>2]|0;
    $89 = $88 >>> 8;
    $90 = $89&255;
    $91 = HEAP32[$hdr>>2]|0;
    $92 = (($91) + 5)|0;
    $93 = HEAP32[$data>>2]|0;
    $94 = (($93) + ($92)|0);
    HEAP8[$94>>0] = $90;
    $95 = HEAP32[$0>>2]|0;
    $96 = ((($95)) + 52|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ((($97)) + 36|0);
    $99 = HEAP32[$98>>2]|0;
    $100 = $99&255;
    $101 = HEAP32[$hdr>>2]|0;
    $102 = (($101) + 5)|0;
    $103 = HEAP32[$data>>2]|0;
    $104 = (($103) + ($102)|0);
    $105 = ((($104)) + 1|0);
    HEAP8[$105>>0] = $100;
    $106 = HEAP32[$0>>2]|0;
    $107 = ((($106)) + 52|0);
    $108 = HEAP32[$107>>2]|0;
    $109 = ((($108)) + 36|0);
    $110 = HEAP32[$109>>2]|0;
    $111 = HEAP32[$0>>2]|0;
    $112 = ((($111)) + 5|0);
    $113 = HEAP8[$112>>0]|0;
    $114 = $113&255;
    $115 = (($110) - ($114))|0;
    $116 = HEAP32[$0>>2]|0;
    $117 = ((($116)) + 6|0);
    $118 = HEAP8[$117>>0]|0;
    $119 = $118&255;
    $120 = (($115) - ($119))|0;
    $121 = (($120) - 8)|0;
    $122 = $121&65535;
    $123 = HEAP32[$0>>2]|0;
    $124 = ((($123)) + 16|0);
    HEAP16[$124>>1] = $122;
    STACKTOP = sp;return;
   } else {
    $125 = HEAP32[$ptr>>2]|0;
    $126 = HEAP32[$ptr>>2]|0;
    $127 = ((($126)) + 2|0);
    $128 = HEAP32[$0>>2]|0;
    $129 = ((($128)) + 18|0);
    $130 = HEAP16[$129>>1]|0;
    $131 = $130&65535;
    $132 = HEAP32[$1>>2]|0;
    $133 = (($131) - ($132))|0;
    $134 = $133<<1;
    _memmove(($125|0),($127|0),($134|0))|0;
    $135 = HEAP32[$0>>2]|0;
    $136 = ((($135)) + 18|0);
    $137 = HEAP16[$136>>1]|0;
    $138 = $137&65535;
    $139 = $138 >> 8;
    $140 = $139&255;
    $141 = HEAP32[$hdr>>2]|0;
    $142 = (($141) + 3)|0;
    $143 = HEAP32[$data>>2]|0;
    $144 = (($143) + ($142)|0);
    HEAP8[$144>>0] = $140;
    $145 = HEAP32[$0>>2]|0;
    $146 = ((($145)) + 18|0);
    $147 = HEAP16[$146>>1]|0;
    $148 = $147&255;
    $149 = HEAP32[$hdr>>2]|0;
    $150 = (($149) + 3)|0;
    $151 = HEAP32[$data>>2]|0;
    $152 = (($151) + ($150)|0);
    $153 = ((($152)) + 1|0);
    HEAP8[$153>>0] = $148;
    $154 = HEAP32[$0>>2]|0;
    $155 = ((($154)) + 16|0);
    $156 = HEAP16[$155>>1]|0;
    $157 = $156&65535;
    $158 = (($157) + 2)|0;
    $159 = $158&65535;
    HEAP16[$155>>1] = $159;
    STACKTOP = sp;return;
   }
  }
 }
 $55 = (_sqlite3CorruptError(64508)|0);
 $56 = HEAP32[$3>>2]|0;
 HEAP32[$56>>2] = $55;
 STACKTOP = sp;return;
}
function _insertCell($pPage,$i,$pCell,$sz,$pTemp,$iChild,$pRC) {
 $pPage = $pPage|0;
 $i = $i|0;
 $pCell = $pCell|0;
 $sz = $sz|0;
 $pTemp = $pTemp|0;
 $iChild = $iChild|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $data = 0, $idx = 0, $j = 0, $pIns = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $idx = sp + 16|0;
 $j = sp + 12|0;
 $data = sp + 8|0;
 $pIns = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pPage;
 HEAP32[$1>>2] = $i;
 HEAP32[$2>>2] = $pCell;
 HEAP32[$3>>2] = $sz;
 HEAP32[$4>>2] = $pTemp;
 HEAP32[$5>>2] = $iChild;
 HEAP32[$6>>2] = $pRC;
 HEAP32[$idx>>2] = 0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 1|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $12 = HEAP32[$3>>2]|0;
  $13 = (($12) + 2)|0;
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16&65535;
  $18 = ($13|0)>($17|0);
  if (!($18)) {
   $45 = HEAP32[$0>>2]|0;
   $46 = ((($45)) + 72|0);
   $47 = HEAP32[$46>>2]|0;
   $48 = (_sqlite3PagerWrite($47)|0);
   HEAP32[$rc>>2] = $48;
   $49 = HEAP32[$rc>>2]|0;
   $50 = ($49|0)!=(0);
   if ($50) {
    $51 = HEAP32[$rc>>2]|0;
    $52 = HEAP32[$6>>2]|0;
    HEAP32[$52>>2] = $51;
    STACKTOP = sp;return;
   }
   $53 = HEAP32[$0>>2]|0;
   $54 = ((($53)) + 56|0);
   $55 = HEAP32[$54>>2]|0;
   HEAP32[$data>>2] = $55;
   $56 = HEAP32[$0>>2]|0;
   $57 = HEAP32[$3>>2]|0;
   $58 = (_allocateSpace($56,$57,$idx)|0);
   HEAP32[$rc>>2] = $58;
   $59 = HEAP32[$rc>>2]|0;
   $60 = ($59|0)!=(0);
   if ($60) {
    $61 = HEAP32[$rc>>2]|0;
    $62 = HEAP32[$6>>2]|0;
    HEAP32[$62>>2] = $61;
    STACKTOP = sp;return;
   }
   $63 = HEAP32[$3>>2]|0;
   $64 = (2 + ($63))|0;
   $65 = $64&65535;
   $66 = $65&65535;
   $67 = HEAP32[$0>>2]|0;
   $68 = ((($67)) + 16|0);
   $69 = HEAP16[$68>>1]|0;
   $70 = $69&65535;
   $71 = (($70) - ($66))|0;
   $72 = $71&65535;
   HEAP16[$68>>1] = $72;
   $73 = HEAP32[$idx>>2]|0;
   $74 = HEAP32[$data>>2]|0;
   $75 = (($74) + ($73)|0);
   $76 = HEAP32[$2>>2]|0;
   $77 = HEAP32[$3>>2]|0;
   _memcpy(($75|0),($76|0),($77|0))|0;
   $78 = HEAP32[$5>>2]|0;
   $79 = ($78|0)!=(0);
   if ($79) {
    $80 = HEAP32[$idx>>2]|0;
    $81 = HEAP32[$data>>2]|0;
    $82 = (($81) + ($80)|0);
    $83 = HEAP32[$5>>2]|0;
    _sqlite3Put4byte($82,$83);
   }
   $84 = HEAP32[$0>>2]|0;
   $85 = ((($84)) + 64|0);
   $86 = HEAP32[$85>>2]|0;
   $87 = HEAP32[$1>>2]|0;
   $88 = $87<<1;
   $89 = (($86) + ($88)|0);
   HEAP32[$pIns>>2] = $89;
   $90 = HEAP32[$pIns>>2]|0;
   $91 = ((($90)) + 2|0);
   $92 = HEAP32[$pIns>>2]|0;
   $93 = HEAP32[$0>>2]|0;
   $94 = ((($93)) + 18|0);
   $95 = HEAP16[$94>>1]|0;
   $96 = $95&65535;
   $97 = HEAP32[$1>>2]|0;
   $98 = (($96) - ($97))|0;
   $99 = $98<<1;
   _memmove(($91|0),($92|0),($99|0))|0;
   $100 = HEAP32[$idx>>2]|0;
   $101 = $100 >> 8;
   $102 = $101&255;
   $103 = HEAP32[$pIns>>2]|0;
   HEAP8[$103>>0] = $102;
   $104 = HEAP32[$idx>>2]|0;
   $105 = $104&255;
   $106 = HEAP32[$pIns>>2]|0;
   $107 = ((($106)) + 1|0);
   HEAP8[$107>>0] = $105;
   $108 = HEAP32[$0>>2]|0;
   $109 = ((($108)) + 18|0);
   $110 = HEAP16[$109>>1]|0;
   $111 = (($110) + 1)<<16>>16;
   HEAP16[$109>>1] = $111;
   $112 = HEAP32[$0>>2]|0;
   $113 = ((($112)) + 5|0);
   $114 = HEAP8[$113>>0]|0;
   $115 = $114&255;
   $116 = (($115) + 4)|0;
   $117 = HEAP32[$data>>2]|0;
   $118 = (($117) + ($116)|0);
   $119 = HEAP8[$118>>0]|0;
   $120 = (($119) + 1)<<24>>24;
   HEAP8[$118>>0] = $120;
   $121 = $120&255;
   $122 = ($121|0)==(0);
   if ($122) {
    $123 = HEAP32[$0>>2]|0;
    $124 = ((($123)) + 5|0);
    $125 = HEAP8[$124>>0]|0;
    $126 = $125&255;
    $127 = (($126) + 3)|0;
    $128 = HEAP32[$data>>2]|0;
    $129 = (($128) + ($127)|0);
    $130 = HEAP8[$129>>0]|0;
    $131 = (($130) + 1)<<24>>24;
    HEAP8[$129>>0] = $131;
   }
   $132 = HEAP32[$0>>2]|0;
   $133 = ((($132)) + 52|0);
   $134 = HEAP32[$133>>2]|0;
   $135 = ((($134)) + 17|0);
   $136 = HEAP8[$135>>0]|0;
   $137 = ($136<<24>>24)!=(0);
   if (!($137)) {
    STACKTOP = sp;return;
   }
   $138 = HEAP32[$0>>2]|0;
   $139 = HEAP32[$2>>2]|0;
   $140 = HEAP32[$6>>2]|0;
   _ptrmapPutOvflPtr($138,$139,$140);
   STACKTOP = sp;return;
  }
 }
 $19 = HEAP32[$4>>2]|0;
 $20 = ($19|0)!=(0|0);
 if ($20) {
  $21 = HEAP32[$4>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$3>>2]|0;
  _memcpy(($21|0),($22|0),($23|0))|0;
  $24 = HEAP32[$4>>2]|0;
  HEAP32[$2>>2] = $24;
 }
 $25 = HEAP32[$5>>2]|0;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP32[$2>>2]|0;
  $28 = HEAP32[$5>>2]|0;
  _sqlite3Put4byte($27,$28);
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 1|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = (($31) + 1)<<24>>24;
 HEAP8[$30>>0] = $32;
 $33 = $31&255;
 HEAP32[$j>>2] = $33;
 $34 = HEAP32[$2>>2]|0;
 $35 = HEAP32[$j>>2]|0;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 32|0);
 $38 = (($37) + ($35<<2)|0);
 HEAP32[$38>>2] = $34;
 $39 = HEAP32[$1>>2]|0;
 $40 = $39&65535;
 $41 = HEAP32[$j>>2]|0;
 $42 = HEAP32[$0>>2]|0;
 $43 = ((($42)) + 22|0);
 $44 = (($43) + ($41<<1)|0);
 HEAP16[$44>>1] = $40;
 STACKTOP = sp;return;
}
function _balance($pCur) {
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aBalanceQuickSpace = 0, $iIdx = 0, $iPage = 0, $nMin = 0, $pFree = 0, $pPage = 0, $pParent = 0, $pSpace = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 32|0;
 $rc = sp + 28|0;
 $nMin = sp + 24|0;
 $aBalanceQuickSpace = sp + 36|0;
 $pFree = sp + 20|0;
 $iPage = sp + 16|0;
 $pPage = sp + 12|0;
 $pParent = sp + 8|0;
 $iIdx = sp + 4|0;
 $pSpace = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 36|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5<<1;
 $7 = (($6>>>0) / 3)&-1;
 HEAP32[$nMin>>2] = $7;
 HEAP32[$pFree>>2] = 0;
 while(1) {
  $8 = HEAP32[$0>>2]|0;
  $9 = ((($8)) + 68|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10 << 24 >> 24;
  HEAP32[$iPage>>2] = $11;
  $12 = HEAP32[$iPage>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 120|0);
  $15 = (($14) + ($12<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$pPage>>2] = $16;
  $17 = HEAP32[$iPage>>2]|0;
  $18 = ($17|0)==(0);
  $19 = HEAP32[$pPage>>2]|0;
  $20 = ((($19)) + 1|0);
  $21 = HEAP8[$20>>0]|0;
  if ($18) {
   $22 = ($21<<24>>24)!=(0);
   if (!($22)) {
    break;
   }
   $23 = HEAP32[$pPage>>2]|0;
   $24 = HEAP32[$0>>2]|0;
   $25 = ((($24)) + 120|0);
   $26 = ((($25)) + 4|0);
   $27 = (_balance_deeper($23,$26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$rc>>2]|0;
   $29 = ($28|0)==(0);
   if ($29) {
    $30 = HEAP32[$0>>2]|0;
    $31 = ((($30)) + 68|0);
    HEAP8[$31>>0] = 1;
    $32 = HEAP32[$0>>2]|0;
    $33 = ((($32)) + 80|0);
    HEAP16[$33>>1] = 0;
    $34 = HEAP32[$0>>2]|0;
    $35 = ((($34)) + 80|0);
    $36 = ((($35)) + 2|0);
    HEAP16[$36>>1] = 0;
   }
  } else {
   $37 = $21&255;
   $38 = ($37|0)==(0);
   if ($38) {
    $39 = HEAP32[$pPage>>2]|0;
    $40 = ((($39)) + 16|0);
    $41 = HEAP16[$40>>1]|0;
    $42 = $41&65535;
    $43 = HEAP32[$nMin>>2]|0;
    $44 = ($42|0)<=($43|0);
    if ($44) {
     break;
    }
   }
   $45 = HEAP32[$iPage>>2]|0;
   $46 = (($45) - 1)|0;
   $47 = HEAP32[$0>>2]|0;
   $48 = ((($47)) + 120|0);
   $49 = (($48) + ($46<<2)|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pParent>>2] = $50;
   $51 = HEAP32[$iPage>>2]|0;
   $52 = (($51) - 1)|0;
   $53 = HEAP32[$0>>2]|0;
   $54 = ((($53)) + 80|0);
   $55 = (($54) + ($52<<1)|0);
   $56 = HEAP16[$55>>1]|0;
   $57 = $56&65535;
   HEAP32[$iIdx>>2] = $57;
   $58 = HEAP32[$pParent>>2]|0;
   $59 = ((($58)) + 72|0);
   $60 = HEAP32[$59>>2]|0;
   $61 = (_sqlite3PagerWrite($60)|0);
   HEAP32[$rc>>2] = $61;
   $62 = HEAP32[$rc>>2]|0;
   $63 = ($62|0)==(0);
   do {
    if ($63) {
     $64 = HEAP32[$pPage>>2]|0;
     $65 = ((($64)) + 3|0);
     $66 = HEAP8[$65>>0]|0;
     $67 = $66&255;
     $68 = ($67|0)!=(0);
     if ($68) {
      $69 = HEAP32[$pPage>>2]|0;
      $70 = ((($69)) + 1|0);
      $71 = HEAP8[$70>>0]|0;
      $72 = $71&255;
      $73 = ($72|0)==(1);
      if ($73) {
       $74 = HEAP32[$pPage>>2]|0;
       $75 = ((($74)) + 22|0);
       $76 = HEAP16[$75>>1]|0;
       $77 = $76&65535;
       $78 = HEAP32[$pPage>>2]|0;
       $79 = ((($78)) + 18|0);
       $80 = HEAP16[$79>>1]|0;
       $81 = $80&65535;
       $82 = ($77|0)==($81|0);
       if ($82) {
        $83 = HEAP32[$pParent>>2]|0;
        $84 = ((($83)) + 84|0);
        $85 = HEAP32[$84>>2]|0;
        $86 = ($85|0)!=(1);
        if ($86) {
         $87 = HEAP32[$pParent>>2]|0;
         $88 = ((($87)) + 18|0);
         $89 = HEAP16[$88>>1]|0;
         $90 = $89&65535;
         $91 = HEAP32[$iIdx>>2]|0;
         $92 = ($90|0)==($91|0);
         if ($92) {
          $93 = HEAP32[$pParent>>2]|0;
          $94 = HEAP32[$pPage>>2]|0;
          $95 = (_balance_quick($93,$94,$aBalanceQuickSpace)|0);
          HEAP32[$rc>>2] = $95;
          break;
         }
        }
       }
      }
     }
     $96 = HEAP32[$0>>2]|0;
     $97 = ((($96)) + 4|0);
     $98 = HEAP32[$97>>2]|0;
     $99 = ((($98)) + 32|0);
     $100 = HEAP32[$99>>2]|0;
     $101 = (_sqlite3PageMalloc($100)|0);
     HEAP32[$pSpace>>2] = $101;
     $102 = HEAP32[$pParent>>2]|0;
     $103 = HEAP32[$iIdx>>2]|0;
     $104 = HEAP32[$pSpace>>2]|0;
     $105 = HEAP32[$iPage>>2]|0;
     $106 = ($105|0)==(1);
     $107 = $106&1;
     $108 = HEAP32[$0>>2]|0;
     $109 = ((($108)) + 67|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     $112 = $111 & 1;
     $113 = (_balance_nonroot($102,$103,$104,$107,$112)|0);
     HEAP32[$rc>>2] = $113;
     $114 = HEAP32[$pFree>>2]|0;
     $115 = ($114|0)!=(0|0);
     if ($115) {
      $116 = HEAP32[$pFree>>2]|0;
      _sqlite3PageFree($116);
     }
     $117 = HEAP32[$pSpace>>2]|0;
     HEAP32[$pFree>>2] = $117;
    }
   } while(0);
   $118 = HEAP32[$pPage>>2]|0;
   $119 = ((($118)) + 1|0);
   HEAP8[$119>>0] = 0;
   $120 = HEAP32[$pPage>>2]|0;
   _releasePage($120);
   $121 = HEAP32[$0>>2]|0;
   $122 = ((($121)) + 68|0);
   $123 = HEAP8[$122>>0]|0;
   $124 = (($123) + -1)<<24>>24;
   HEAP8[$122>>0] = $124;
  }
  $125 = HEAP32[$rc>>2]|0;
  $126 = ($125|0)==(0);
  if (!($126)) {
   break;
  }
 }
 $127 = HEAP32[$pFree>>2]|0;
 $128 = ($127|0)!=(0|0);
 if (!($128)) {
  $130 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($130|0);
 }
 $129 = HEAP32[$pFree>>2]|0;
 _sqlite3PageFree($129);
 $130 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($130|0);
}
function _balance_deeper($pRoot,$ppChild) {
 $pRoot = $pRoot|0;
 $ppChild = $ppChild|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $8 = 0, $9 = 0, $pBt = 0, $pChild = 0, $pgnoChild = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $pChild = sp + 8|0;
 $pgnoChild = sp + 4|0;
 $pBt = sp;
 HEAP32[$1>>2] = $pRoot;
 HEAP32[$2>>2] = $ppChild;
 HEAP32[$pChild>>2] = 0;
 HEAP32[$pgnoChild>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 52|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pBt>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 72|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3PagerWrite($8)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = HEAP32[$pBt>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 84|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = (_allocateBtreePage($12,$pChild,$pgnoChild,$15,0)|0);
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$1>>2]|0;
  $18 = HEAP32[$pChild>>2]|0;
  _copyNodeContent($17,$18,$rc);
  $19 = HEAP32[$pBt>>2]|0;
  $20 = ((($19)) + 17|0);
  $21 = HEAP8[$20>>0]|0;
  $22 = ($21<<24>>24)!=(0);
  if ($22) {
   $23 = HEAP32[$pBt>>2]|0;
   $24 = HEAP32[$pgnoChild>>2]|0;
   $25 = HEAP32[$1>>2]|0;
   $26 = ((($25)) + 84|0);
   $27 = HEAP32[$26>>2]|0;
   _ptrmapPut($23,$24,5,$27,$rc);
  }
 }
 $28 = HEAP32[$rc>>2]|0;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[$2>>2]|0;
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$pChild>>2]|0;
  _releasePage($31);
  $32 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $32;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 } else {
  $33 = HEAP32[$pChild>>2]|0;
  $34 = ((($33)) + 22|0);
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 22|0);
  $37 = HEAP32[$1>>2]|0;
  $38 = ((($37)) + 1|0);
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = $40<<1;
  _memcpy(($34|0),($36|0),($41|0))|0;
  $42 = HEAP32[$pChild>>2]|0;
  $43 = ((($42)) + 32|0);
  $44 = HEAP32[$1>>2]|0;
  $45 = ((($44)) + 32|0);
  $46 = HEAP32[$1>>2]|0;
  $47 = ((($46)) + 1|0);
  $48 = HEAP8[$47>>0]|0;
  $49 = $48&255;
  $50 = $49<<2;
  _memcpy(($43|0),($45|0),($50|0))|0;
  $51 = HEAP32[$1>>2]|0;
  $52 = ((($51)) + 1|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = HEAP32[$pChild>>2]|0;
  $55 = ((($54)) + 1|0);
  HEAP8[$55>>0] = $53;
  $56 = HEAP32[$1>>2]|0;
  $57 = HEAP32[$pChild>>2]|0;
  $58 = ((($57)) + 56|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = HEAP8[$59>>0]|0;
  $61 = $60&255;
  $62 = $61 & -9;
  _zeroPage($56,$62);
  $63 = HEAP32[$1>>2]|0;
  $64 = ((($63)) + 5|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = (($66) + 8)|0;
  $68 = HEAP32[$1>>2]|0;
  $69 = ((($68)) + 56|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (($70) + ($67)|0);
  $72 = HEAP32[$pgnoChild>>2]|0;
  _sqlite3Put4byte($71,$72);
  $73 = HEAP32[$pChild>>2]|0;
  $74 = HEAP32[$2>>2]|0;
  HEAP32[$74>>2] = $73;
  HEAP32[$0>>2] = 0;
  $75 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($75|0);
 }
 return (0)|0;
}
function _balance_quick($pParent,$pPage,$pSpace) {
 $pParent = $pParent|0;
 $pPage = $pPage|0;
 $pSpace = $pSpace|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $pBt = 0, $pCell = 0, $pNew = 0, $pOut = 0;
 var $pStop = 0, $pgnoNew = 0, $rc = 0, $szCell = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $pBt = sp + 24|0;
 $pNew = sp + 20|0;
 $rc = sp + 16|0;
 $pgnoNew = sp + 12|0;
 $pOut = sp + 8|0;
 $pCell = sp + 4|0;
 $szCell = sp + 44|0;
 $pStop = sp;
 HEAP32[$1>>2] = $pParent;
 HEAP32[$2>>2] = $pPage;
 HEAP32[$3>>2] = $pSpace;
 $4 = HEAP32[$2>>2]|0;
 $5 = ((($4)) + 52|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 18|0);
 $9 = HEAP16[$8>>1]|0;
 $10 = $9&65535;
 $11 = ($10|0)==(0);
 if ($11) {
  $12 = (_sqlite3CorruptError(65006)|0);
  HEAP32[$0>>2] = $12;
  $155 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($155|0);
 }
 $13 = HEAP32[$pBt>>2]|0;
 $14 = (_allocateBtreePage($13,$pNew,$pgnoNew,0,0)|0);
 HEAP32[$rc>>2] = $14;
 $15 = HEAP32[$rc>>2]|0;
 $16 = ($15|0)==(0);
 if ($16) {
  $17 = HEAP32[$3>>2]|0;
  $18 = ((($17)) + 4|0);
  HEAP32[$pOut>>2] = $18;
  $19 = HEAP32[$2>>2]|0;
  $20 = ((($19)) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pCell>>2] = $21;
  $22 = HEAP32[$2>>2]|0;
  $23 = ((($22)) + 76|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = HEAP32[$2>>2]|0;
  $26 = HEAP32[$pCell>>2]|0;
  $27 = (FUNCTION_TABLE_iii[$24 & 255]($25,$26)|0);
  HEAP16[$szCell>>1] = $27;
  $28 = HEAP32[$pNew>>2]|0;
  _zeroPage($28,13);
  $29 = HEAP32[$pNew>>2]|0;
  $30 = (_rebuildPage($29,1,$pCell,$szCell)|0);
  HEAP32[$rc>>2] = $30;
  $31 = HEAP32[$rc>>2]|0;
  $32 = ($31|0)!=(0);
  if ($32) {
   $33 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $33;
   $155 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($155|0);
  }
  $34 = HEAP32[$pBt>>2]|0;
  $35 = ((($34)) + 36|0);
  $36 = HEAP32[$35>>2]|0;
  $37 = HEAP32[$pNew>>2]|0;
  $38 = ((($37)) + 14|0);
  $39 = HEAP16[$38>>1]|0;
  $40 = $39&65535;
  $41 = (($36) - ($40))|0;
  $42 = (($41) - 2)|0;
  $43 = HEAP16[$szCell>>1]|0;
  $44 = $43&65535;
  $45 = (($42) - ($44))|0;
  $46 = $45&65535;
  $47 = HEAP32[$pNew>>2]|0;
  $48 = ((($47)) + 16|0);
  HEAP16[$48>>1] = $46;
  $49 = HEAP32[$pBt>>2]|0;
  $50 = ((($49)) + 17|0);
  $51 = HEAP8[$50>>0]|0;
  $52 = ($51<<24>>24)!=(0);
  if ($52) {
   $53 = HEAP32[$pBt>>2]|0;
   $54 = HEAP32[$pgnoNew>>2]|0;
   $55 = HEAP32[$1>>2]|0;
   $56 = ((($55)) + 84|0);
   $57 = HEAP32[$56>>2]|0;
   _ptrmapPut($53,$54,5,$57,$rc);
   $58 = HEAP16[$szCell>>1]|0;
   $59 = $58&65535;
   $60 = HEAP32[$pNew>>2]|0;
   $61 = ((($60)) + 12|0);
   $62 = HEAP16[$61>>1]|0;
   $63 = $62&65535;
   $64 = ($59|0)>($63|0);
   if ($64) {
    $65 = HEAP32[$pNew>>2]|0;
    $66 = HEAP32[$pCell>>2]|0;
    _ptrmapPutOvflPtr($65,$66,$rc);
   }
  }
  $67 = HEAP32[$2>>2]|0;
  $68 = ((($67)) + 56|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$2>>2]|0;
  $71 = ((($70)) + 20|0);
  $72 = HEAP16[$71>>1]|0;
  $73 = $72&65535;
  $74 = HEAP32[$2>>2]|0;
  $75 = ((($74)) + 18|0);
  $76 = HEAP16[$75>>1]|0;
  $77 = $76&65535;
  $78 = (($77) - 1)|0;
  $79 = $78<<1;
  $80 = HEAP32[$2>>2]|0;
  $81 = ((($80)) + 64|0);
  $82 = HEAP32[$81>>2]|0;
  $83 = (($82) + ($79)|0);
  $84 = HEAP8[$83>>0]|0;
  $85 = $84&255;
  $86 = $85 << 8;
  $87 = HEAP32[$2>>2]|0;
  $88 = ((($87)) + 18|0);
  $89 = HEAP16[$88>>1]|0;
  $90 = $89&65535;
  $91 = (($90) - 1)|0;
  $92 = $91<<1;
  $93 = HEAP32[$2>>2]|0;
  $94 = ((($93)) + 64|0);
  $95 = HEAP32[$94>>2]|0;
  $96 = (($95) + ($92)|0);
  $97 = ((($96)) + 1|0);
  $98 = HEAP8[$97>>0]|0;
  $99 = $98&255;
  $100 = $86 | $99;
  $101 = $73 & $100;
  $102 = (($69) + ($101)|0);
  HEAP32[$pCell>>2] = $102;
  $103 = HEAP32[$pCell>>2]|0;
  $104 = ((($103)) + 9|0);
  HEAP32[$pStop>>2] = $104;
  while(1) {
   $105 = HEAP32[$pCell>>2]|0;
   $106 = ((($105)) + 1|0);
   HEAP32[$pCell>>2] = $106;
   $107 = HEAP8[$105>>0]|0;
   $108 = $107&255;
   $109 = $108 & 128;
   $110 = ($109|0)!=(0);
   if (!($110)) {
    break;
   }
   $111 = HEAP32[$pCell>>2]|0;
   $112 = HEAP32[$pStop>>2]|0;
   $113 = ($111>>>0)<($112>>>0);
   if (!($113)) {
    break;
   }
  }
  $114 = HEAP32[$pCell>>2]|0;
  $115 = ((($114)) + 9|0);
  HEAP32[$pStop>>2] = $115;
  while(1) {
   $116 = HEAP32[$pCell>>2]|0;
   $117 = ((($116)) + 1|0);
   HEAP32[$pCell>>2] = $117;
   $118 = HEAP8[$116>>0]|0;
   $119 = HEAP32[$pOut>>2]|0;
   $120 = ((($119)) + 1|0);
   HEAP32[$pOut>>2] = $120;
   HEAP8[$119>>0] = $118;
   $121 = $118&255;
   $122 = $121 & 128;
   $123 = ($122|0)!=(0);
   if (!($123)) {
    break;
   }
   $124 = HEAP32[$pCell>>2]|0;
   $125 = HEAP32[$pStop>>2]|0;
   $126 = ($124>>>0)<($125>>>0);
   if (!($126)) {
    break;
   }
  }
  $127 = HEAP32[$rc>>2]|0;
  $128 = ($127|0)==(0);
  if ($128) {
   $129 = HEAP32[$1>>2]|0;
   $130 = HEAP32[$1>>2]|0;
   $131 = ((($130)) + 18|0);
   $132 = HEAP16[$131>>1]|0;
   $133 = $132&65535;
   $134 = HEAP32[$3>>2]|0;
   $135 = HEAP32[$pOut>>2]|0;
   $136 = HEAP32[$3>>2]|0;
   $137 = $135;
   $138 = $136;
   $139 = (($137) - ($138))|0;
   $140 = HEAP32[$2>>2]|0;
   $141 = ((($140)) + 84|0);
   $142 = HEAP32[$141>>2]|0;
   _insertCell($129,$133,$134,$139,0,$142,$rc);
  }
  $143 = HEAP32[$1>>2]|0;
  $144 = ((($143)) + 5|0);
  $145 = HEAP8[$144>>0]|0;
  $146 = $145&255;
  $147 = (($146) + 8)|0;
  $148 = HEAP32[$1>>2]|0;
  $149 = ((($148)) + 56|0);
  $150 = HEAP32[$149>>2]|0;
  $151 = (($150) + ($147)|0);
  $152 = HEAP32[$pgnoNew>>2]|0;
  _sqlite3Put4byte($151,$152);
  $153 = HEAP32[$pNew>>2]|0;
  _releasePage($153);
 }
 $154 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $154;
 $155 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($155|0);
}
function _balance_nonroot($pParent,$iParentIdx,$aOvflSpace,$isRoot,$bBulk) {
 $pParent = $pParent|0;
 $iParentIdx = $iParentIdx|0;
 $aOvflSpace = $aOvflSpace|0;
 $isRoot = $isRoot|0;
 $bBulk = $bBulk|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0, $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0;
 var $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0, $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0;
 var $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0, $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0;
 var $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0, $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0;
 var $107 = 0, $1070 = 0, $1071 = 0, $1072 = 0, $1073 = 0, $1074 = 0, $1075 = 0, $1076 = 0, $1077 = 0, $1078 = 0, $1079 = 0, $108 = 0, $1080 = 0, $1081 = 0, $1082 = 0, $1083 = 0, $1084 = 0, $1085 = 0, $1086 = 0, $1087 = 0;
 var $1088 = 0, $1089 = 0, $109 = 0, $1090 = 0, $1091 = 0, $1092 = 0, $1093 = 0, $1094 = 0, $1095 = 0, $1096 = 0, $1097 = 0, $1098 = 0, $1099 = 0, $11 = 0, $110 = 0, $1100 = 0, $1101 = 0, $1102 = 0, $1103 = 0, $1104 = 0;
 var $1105 = 0, $1106 = 0, $1107 = 0, $1108 = 0, $1109 = 0, $111 = 0, $1110 = 0, $1111 = 0, $1112 = 0, $1113 = 0, $1114 = 0, $1115 = 0, $1116 = 0, $1117 = 0, $1118 = 0, $1119 = 0, $112 = 0, $1120 = 0, $1121 = 0, $1122 = 0;
 var $1123 = 0, $1124 = 0, $1125 = 0, $1126 = 0, $1127 = 0, $1128 = 0, $1129 = 0, $113 = 0, $1130 = 0, $1131 = 0, $1132 = 0, $1133 = 0, $1134 = 0, $1135 = 0, $1136 = 0, $1137 = 0, $1138 = 0, $1139 = 0, $114 = 0, $1140 = 0;
 var $1141 = 0, $1142 = 0, $1143 = 0, $1144 = 0, $1145 = 0, $1146 = 0, $1147 = 0, $1148 = 0, $1149 = 0, $115 = 0, $1150 = 0, $1151 = 0, $1152 = 0, $1153 = 0, $1154 = 0, $1155 = 0, $1156 = 0, $1157 = 0, $1158 = 0, $1159 = 0;
 var $116 = 0, $1160 = 0, $1161 = 0, $1162 = 0, $1163 = 0, $1164 = 0, $1165 = 0, $1166 = 0, $1167 = 0, $1168 = 0, $1169 = 0, $117 = 0, $1170 = 0, $1171 = 0, $1172 = 0, $1173 = 0, $1174 = 0, $1175 = 0, $1176 = 0, $1177 = 0;
 var $1178 = 0, $1179 = 0, $118 = 0, $1180 = 0, $1181 = 0, $1182 = 0, $1183 = 0, $1184 = 0, $1185 = 0, $1186 = 0, $1187 = 0, $1188 = 0, $1189 = 0, $119 = 0, $1190 = 0, $1191 = 0, $1192 = 0, $1193 = 0, $1194 = 0, $1195 = 0;
 var $1196 = 0, $1197 = 0, $1198 = 0, $1199 = 0, $12 = 0, $120 = 0, $1200 = 0, $1201 = 0, $1202 = 0, $1203 = 0, $1204 = 0, $1205 = 0, $1206 = 0, $1207 = 0, $1208 = 0, $1209 = 0, $121 = 0, $1210 = 0, $1211 = 0, $1212 = 0;
 var $1213 = 0, $1214 = 0, $1215 = 0, $1216 = 0, $1217 = 0, $1218 = 0, $1219 = 0, $122 = 0, $1220 = 0, $1221 = 0, $1222 = 0, $1223 = 0, $1224 = 0, $1225 = 0, $1226 = 0, $1227 = 0, $1228 = 0, $1229 = 0, $123 = 0, $1230 = 0;
 var $1231 = 0, $1232 = 0, $1233 = 0, $1234 = 0, $1235 = 0, $1236 = 0, $1237 = 0, $1238 = 0, $1239 = 0, $124 = 0, $1240 = 0, $1241 = 0, $1242 = 0, $1243 = 0, $1244 = 0, $1245 = 0, $1246 = 0, $1247 = 0, $1248 = 0, $1249 = 0;
 var $125 = 0, $1250 = 0, $1251 = 0, $1252 = 0, $1253 = 0, $1254 = 0, $1255 = 0, $1256 = 0, $1257 = 0, $1258 = 0, $1259 = 0, $126 = 0, $1260 = 0, $1261 = 0, $1262 = 0, $1263 = 0, $1264 = 0, $1265 = 0, $1266 = 0, $1267 = 0;
 var $1268 = 0, $1269 = 0, $127 = 0, $1270 = 0, $1271 = 0, $1272 = 0, $1273 = 0, $1274 = 0, $1275 = 0, $1276 = 0, $1277 = 0, $1278 = 0, $1279 = 0, $128 = 0, $1280 = 0, $1281 = 0, $1282 = 0, $1283 = 0, $1284 = 0, $1285 = 0;
 var $1286 = 0, $1287 = 0, $1288 = 0, $1289 = 0, $129 = 0, $1290 = 0, $1291 = 0, $1292 = 0, $1293 = 0, $1294 = 0, $1295 = 0, $1296 = 0, $1297 = 0, $1298 = 0, $1299 = 0, $13 = 0, $130 = 0, $1300 = 0, $1301 = 0, $1302 = 0;
 var $1303 = 0, $1304 = 0, $1305 = 0, $1306 = 0, $1307 = 0, $1308 = 0, $1309 = 0, $131 = 0, $1310 = 0, $1311 = 0, $1312 = 0, $1313 = 0, $1314 = 0, $1315 = 0, $1316 = 0, $1317 = 0, $1318 = 0, $1319 = 0, $132 = 0, $1320 = 0;
 var $1321 = 0, $1322 = 0, $1323 = 0, $1324 = 0, $1325 = 0, $1326 = 0, $1327 = 0, $1328 = 0, $1329 = 0, $133 = 0, $1330 = 0, $1331 = 0, $1332 = 0, $1333 = 0, $1334 = 0, $1335 = 0, $1336 = 0, $1337 = 0, $1338 = 0, $1339 = 0;
 var $134 = 0, $1340 = 0, $1341 = 0, $1342 = 0, $1343 = 0, $1344 = 0, $1345 = 0, $1346 = 0, $1347 = 0, $1348 = 0, $1349 = 0, $135 = 0, $1350 = 0, $1351 = 0, $1352 = 0, $1353 = 0, $1354 = 0, $1355 = 0, $1356 = 0, $1357 = 0;
 var $1358 = 0, $1359 = 0, $136 = 0, $1360 = 0, $1361 = 0, $1362 = 0, $1363 = 0, $1364 = 0, $1365 = 0, $1366 = 0, $1367 = 0, $1368 = 0, $1369 = 0, $137 = 0, $1370 = 0, $1371 = 0, $1372 = 0, $1373 = 0, $1374 = 0, $1375 = 0;
 var $1376 = 0, $1377 = 0, $1378 = 0, $1379 = 0, $138 = 0, $1380 = 0, $1381 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0;
 var $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0;
 var $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0;
 var $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0;
 var $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0;
 var $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0;
 var $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0;
 var $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0;
 var $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0;
 var $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0;
 var $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0;
 var $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0;
 var $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0;
 var $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0;
 var $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0;
 var $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0;
 var $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0;
 var $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0, $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0;
 var $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0;
 var $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0;
 var $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0;
 var $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0, $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0;
 var $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0;
 var $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0;
 var $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0, $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0;
 var $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0;
 var $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0;
 var $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0, $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0;
 var $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0;
 var $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0, $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0;
 var $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0, $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0;
 var $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0, $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0;
 var $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0, $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0;
 var $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0, $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0;
 var $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0, $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0;
 var $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0, $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0;
 var $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0, $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0;
 var $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0, $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0;
 var $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0, $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0;
 var $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0, $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0;
 var $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0, $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0;
 var $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0, $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0;
 var $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0, $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0;
 var $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0, $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0;
 var $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0, $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0;
 var $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0, $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0;
 var $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0, $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0;
 var $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0, $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0;
 var $998 = 0, $999 = 0, $aData = 0, $aOld = 0, $aPgFlags = 0, $aPgOrder = 0, $aPgno = 0, $aSpace1 = 0, $abDone = 0, $apDiv = 0, $apNew = 0, $apOld = 0, $b = 0, $cntNew = 0, $cntOld = 0, $cntOldNext = 0, $d = 0, $i = 0, $iBest = 0, $iNew = 0;
 var $iNew9 = 0, $iOff = 0, $iOld = 0, $iOld10 = 0, $iOvflSpace = 0, $iPg = 0, $iSpace1 = 0, $info = 0, $j = 0, $k = 0, $key = 0, $leafCorrection = 0, $leafData = 0, $limit = 0, $maskPage = 0, $nMaxCells = 0, $nNew = 0, $nNewCell = 0, $nOld = 0, $nxDiv = 0;
 var $or$cond = 0, $or$cond3 = 0, $p = 0, $pBt = 0, $pCell = 0, $pCell5 = 0, $pNew = 0, $pNew3 = 0, $pNew8 = 0, $pOld = 0, $pOld2 = 0, $pOld4 = 0, $pRight = 0, $pTemp = 0, $pTemp6 = 0, $pageFlags = 0, $pgno = 0, $piCell = 0, $piEnd = 0, $r = 0;
 var $rc = 0, $sz = 0, $sz1 = 0, $sz7 = 0, $szLeft = 0, $szNew = 0, $szRight = 0, $szScratch = 0, $usableSize = 0, $usableSpace = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 448|0;
 $0 = sp + 412|0;
 $1 = sp + 408|0;
 $2 = sp + 404|0;
 $3 = sp + 400|0;
 $4 = sp + 396|0;
 $5 = sp + 392|0;
 $pBt = sp + 388|0;
 $nMaxCells = sp + 384|0;
 $nNew = sp + 380|0;
 $nOld = sp + 376|0;
 $i = sp + 372|0;
 $j = sp + 368|0;
 $k = sp + 364|0;
 $nxDiv = sp + 360|0;
 $rc = sp + 356|0;
 $leafCorrection = sp + 430|0;
 $leafData = sp + 352|0;
 $usableSpace = sp + 348|0;
 $pageFlags = sp + 344|0;
 $iSpace1 = sp + 340|0;
 $iOvflSpace = sp + 336|0;
 $szScratch = sp + 332|0;
 $apOld = sp + 320|0;
 $apNew = sp + 300|0;
 $pRight = sp + 296|0;
 $apDiv = sp + 288|0;
 $cntNew = sp + 264|0;
 $cntOld = sp + 244|0;
 $szNew = sp + 224|0;
 $aSpace1 = sp + 220|0;
 $pgno = sp + 216|0;
 $abDone = sp + 432|0;
 $aPgno = sp + 196|0;
 $aPgOrder = sp + 176|0;
 $aPgFlags = sp + 420|0;
 $b = sp + 160|0;
 $iOff = sp + 152|0;
 $pOld = sp + 148|0;
 $limit = sp + 144|0;
 $aData = sp + 140|0;
 $maskPage = sp + 418|0;
 $piCell = sp + 136|0;
 $piEnd = sp + 132|0;
 $sz = sp + 416|0;
 $pTemp = sp + 128|0;
 $p = sp + 124|0;
 $sz1 = sp + 120|0;
 $szRight = sp + 116|0;
 $szLeft = sp + 112|0;
 $r = sp + 108|0;
 $d = sp + 104|0;
 $pNew = sp + 100|0;
 $iBest = sp + 96|0;
 $pOld2 = sp + 92|0;
 $pNew3 = sp + 88|0;
 $aOld = sp + 84|0;
 $cntOldNext = sp + 80|0;
 $usableSize = sp + 76|0;
 $iNew = sp + 72|0;
 $iOld = sp + 68|0;
 $pCell = sp + 64|0;
 $pOld4 = sp + 60|0;
 $pCell5 = sp + 56|0;
 $pTemp6 = sp + 52|0;
 $sz7 = sp + 48|0;
 $pNew8 = sp + 44|0;
 $info = sp;
 $iPg = sp + 40|0;
 $iNew9 = sp + 36|0;
 $iOld10 = sp + 32|0;
 $nNewCell = sp + 28|0;
 $key = sp + 24|0;
 HEAP32[$1>>2] = $pParent;
 HEAP32[$2>>2] = $iParentIdx;
 HEAP32[$3>>2] = $aOvflSpace;
 HEAP32[$4>>2] = $isRoot;
 HEAP32[$5>>2] = $bBulk;
 HEAP32[$nMaxCells>>2] = 0;
 HEAP32[$nNew>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$iSpace1>>2] = 0;
 HEAP32[$iOvflSpace>>2] = 0;
 ;HEAP8[$abDone>>0]=0|0;HEAP8[$abDone+1>>0]=0|0;HEAP8[$abDone+2>>0]=0|0;HEAP8[$abDone+3>>0]=0|0;HEAP8[$abDone+4>>0]=0|0;
 HEAP32[$b>>2] = 0;
 $6 = ((($b)) + 8|0);
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$1>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pBt>>2] = $9;
 $10 = HEAP32[$3>>2]|0;
 $11 = ($10|0)!=(0|0);
 if (!($11)) {
  HEAP32[$0>>2] = 7;
  $1381 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($1381|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 1|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = $14&255;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 18|0);
 $18 = HEAP16[$17>>1]|0;
 $19 = $18&65535;
 $20 = (($15) + ($19))|0;
 HEAP32[$i>>2] = $20;
 $21 = HEAP32[$i>>2]|0;
 $22 = ($21|0)<(2);
 if ($22) {
  HEAP32[$nxDiv>>2] = 0;
 } else {
  $23 = HEAP32[$2>>2]|0;
  $24 = ($23|0)==(0);
  do {
   if ($24) {
    HEAP32[$nxDiv>>2] = 0;
   } else {
    $25 = HEAP32[$2>>2]|0;
    $26 = HEAP32[$i>>2]|0;
    $27 = ($25|0)==($26|0);
    if ($27) {
     $28 = HEAP32[$i>>2]|0;
     $29 = (($28) - 2)|0;
     $30 = HEAP32[$5>>2]|0;
     $31 = (($29) + ($30))|0;
     HEAP32[$nxDiv>>2] = $31;
     break;
    } else {
     $32 = HEAP32[$2>>2]|0;
     $33 = (($32) - 1)|0;
     HEAP32[$nxDiv>>2] = $33;
     break;
    }
   }
  } while(0);
  $34 = HEAP32[$5>>2]|0;
  $35 = (2 - ($34))|0;
  HEAP32[$i>>2] = $35;
 }
 $36 = HEAP32[$i>>2]|0;
 $37 = (($36) + 1)|0;
 HEAP32[$nOld>>2] = $37;
 $38 = HEAP32[$i>>2]|0;
 $39 = HEAP32[$nxDiv>>2]|0;
 $40 = (($38) + ($39))|0;
 $41 = HEAP32[$1>>2]|0;
 $42 = ((($41)) + 1|0);
 $43 = HEAP8[$42>>0]|0;
 $44 = $43&255;
 $45 = (($40) - ($44))|0;
 $46 = HEAP32[$1>>2]|0;
 $47 = ((($46)) + 18|0);
 $48 = HEAP16[$47>>1]|0;
 $49 = $48&65535;
 $50 = ($45|0)==($49|0);
 $51 = HEAP32[$1>>2]|0;
 if ($50) {
  $52 = ((($51)) + 5|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = (($54) + 8)|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 56|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (($58) + ($55)|0);
  HEAP32[$pRight>>2] = $59;
 } else {
  $60 = ((($51)) + 56|0);
  $61 = HEAP32[$60>>2]|0;
  $62 = HEAP32[$1>>2]|0;
  $63 = ((($62)) + 20|0);
  $64 = HEAP16[$63>>1]|0;
  $65 = $64&65535;
  $66 = HEAP32[$i>>2]|0;
  $67 = HEAP32[$nxDiv>>2]|0;
  $68 = (($66) + ($67))|0;
  $69 = HEAP32[$1>>2]|0;
  $70 = ((($69)) + 1|0);
  $71 = HEAP8[$70>>0]|0;
  $72 = $71&255;
  $73 = (($68) - ($72))|0;
  $74 = $73<<1;
  $75 = HEAP32[$1>>2]|0;
  $76 = ((($75)) + 64|0);
  $77 = HEAP32[$76>>2]|0;
  $78 = (($77) + ($74)|0);
  $79 = HEAP8[$78>>0]|0;
  $80 = $79&255;
  $81 = $80 << 8;
  $82 = HEAP32[$i>>2]|0;
  $83 = HEAP32[$nxDiv>>2]|0;
  $84 = (($82) + ($83))|0;
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 1|0);
  $87 = HEAP8[$86>>0]|0;
  $88 = $87&255;
  $89 = (($84) - ($88))|0;
  $90 = $89<<1;
  $91 = HEAP32[$1>>2]|0;
  $92 = ((($91)) + 64|0);
  $93 = HEAP32[$92>>2]|0;
  $94 = (($93) + ($90)|0);
  $95 = ((($94)) + 1|0);
  $96 = HEAP8[$95>>0]|0;
  $97 = $96&255;
  $98 = $81 | $97;
  $99 = $65 & $98;
  $100 = (($61) + ($99)|0);
  HEAP32[$pRight>>2] = $100;
 }
 $101 = HEAP32[$pRight>>2]|0;
 $102 = (_sqlite3Get4byte($101)|0);
 HEAP32[$pgno>>2] = $102;
 while(1) {
  $103 = HEAP32[$pBt>>2]|0;
  $104 = HEAP32[$pgno>>2]|0;
  $105 = HEAP32[$i>>2]|0;
  $106 = (($apOld) + ($105<<2)|0);
  $107 = (_getAndInitPage($103,$104,$106,0,0)|0);
  HEAP32[$rc>>2] = $107;
  $108 = HEAP32[$rc>>2]|0;
  $109 = ($108|0)!=(0);
  if ($109) {
   label = 16;
   break;
  }
  $113 = HEAP32[$i>>2]|0;
  $114 = (($apOld) + ($113<<2)|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = ((($115)) + 18|0);
  $117 = HEAP16[$116>>1]|0;
  $118 = $117&65535;
  $119 = (1 + ($118))|0;
  $120 = HEAP32[$i>>2]|0;
  $121 = (($apOld) + ($120<<2)|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = ((($122)) + 1|0);
  $124 = HEAP8[$123>>0]|0;
  $125 = $124&255;
  $126 = (($119) + ($125))|0;
  $127 = HEAP32[$nMaxCells>>2]|0;
  $128 = (($127) + ($126))|0;
  HEAP32[$nMaxCells>>2] = $128;
  $129 = HEAP32[$i>>2]|0;
  $130 = (($129) + -1)|0;
  HEAP32[$i>>2] = $130;
  $131 = ($129|0)==(0);
  if ($131) {
   label = 26;
   break;
  }
  $132 = HEAP32[$i>>2]|0;
  $133 = HEAP32[$nxDiv>>2]|0;
  $134 = (($132) + ($133))|0;
  $135 = HEAP32[$1>>2]|0;
  $136 = ((($135)) + 22|0);
  $137 = HEAP16[$136>>1]|0;
  $138 = $137&65535;
  $139 = ($134|0)==($138|0);
  if ($139) {
   $140 = HEAP32[$1>>2]|0;
   $141 = ((($140)) + 1|0);
   $142 = HEAP8[$141>>0]|0;
   $143 = $142&255;
   $144 = ($143|0)!=(0);
   if ($144) {
    $145 = HEAP32[$1>>2]|0;
    $146 = ((($145)) + 32|0);
    $147 = HEAP32[$146>>2]|0;
    $148 = HEAP32[$i>>2]|0;
    $149 = (($apDiv) + ($148<<2)|0);
    HEAP32[$149>>2] = $147;
    $150 = HEAP32[$i>>2]|0;
    $151 = (($apDiv) + ($150<<2)|0);
    $152 = HEAP32[$151>>2]|0;
    $153 = (_sqlite3Get4byte($152)|0);
    HEAP32[$pgno>>2] = $153;
    $154 = HEAP32[$1>>2]|0;
    $155 = ((($154)) + 76|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = HEAP32[$1>>2]|0;
    $158 = HEAP32[$i>>2]|0;
    $159 = (($apDiv) + ($158<<2)|0);
    $160 = HEAP32[$159>>2]|0;
    $161 = (FUNCTION_TABLE_iii[$156 & 255]($157,$160)|0);
    $162 = $161&65535;
    $163 = HEAP32[$i>>2]|0;
    $164 = (($szNew) + ($163<<2)|0);
    HEAP32[$164>>2] = $162;
    $165 = HEAP32[$1>>2]|0;
    $166 = ((($165)) + 1|0);
    HEAP8[$166>>0] = 0;
    continue;
   }
  }
  $167 = HEAP32[$1>>2]|0;
  $168 = ((($167)) + 56|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = HEAP32[$1>>2]|0;
  $171 = ((($170)) + 20|0);
  $172 = HEAP16[$171>>1]|0;
  $173 = $172&65535;
  $174 = HEAP32[$i>>2]|0;
  $175 = HEAP32[$nxDiv>>2]|0;
  $176 = (($174) + ($175))|0;
  $177 = HEAP32[$1>>2]|0;
  $178 = ((($177)) + 1|0);
  $179 = HEAP8[$178>>0]|0;
  $180 = $179&255;
  $181 = (($176) - ($180))|0;
  $182 = $181<<1;
  $183 = HEAP32[$1>>2]|0;
  $184 = ((($183)) + 64|0);
  $185 = HEAP32[$184>>2]|0;
  $186 = (($185) + ($182)|0);
  $187 = HEAP8[$186>>0]|0;
  $188 = $187&255;
  $189 = $188 << 8;
  $190 = HEAP32[$i>>2]|0;
  $191 = HEAP32[$nxDiv>>2]|0;
  $192 = (($190) + ($191))|0;
  $193 = HEAP32[$1>>2]|0;
  $194 = ((($193)) + 1|0);
  $195 = HEAP8[$194>>0]|0;
  $196 = $195&255;
  $197 = (($192) - ($196))|0;
  $198 = $197<<1;
  $199 = HEAP32[$1>>2]|0;
  $200 = ((($199)) + 64|0);
  $201 = HEAP32[$200>>2]|0;
  $202 = (($201) + ($198)|0);
  $203 = ((($202)) + 1|0);
  $204 = HEAP8[$203>>0]|0;
  $205 = $204&255;
  $206 = $189 | $205;
  $207 = $173 & $206;
  $208 = (($169) + ($207)|0);
  $209 = HEAP32[$i>>2]|0;
  $210 = (($apDiv) + ($209<<2)|0);
  HEAP32[$210>>2] = $208;
  $211 = HEAP32[$i>>2]|0;
  $212 = (($apDiv) + ($211<<2)|0);
  $213 = HEAP32[$212>>2]|0;
  $214 = (_sqlite3Get4byte($213)|0);
  HEAP32[$pgno>>2] = $214;
  $215 = HEAP32[$1>>2]|0;
  $216 = ((($215)) + 76|0);
  $217 = HEAP32[$216>>2]|0;
  $218 = HEAP32[$1>>2]|0;
  $219 = HEAP32[$i>>2]|0;
  $220 = (($apDiv) + ($219<<2)|0);
  $221 = HEAP32[$220>>2]|0;
  $222 = (FUNCTION_TABLE_iii[$217 & 255]($218,$221)|0);
  $223 = $222&65535;
  $224 = HEAP32[$i>>2]|0;
  $225 = (($szNew) + ($224<<2)|0);
  HEAP32[$225>>2] = $223;
  $226 = HEAP32[$pBt>>2]|0;
  $227 = ((($226)) + 22|0);
  $228 = HEAP16[$227>>1]|0;
  $229 = $228&65535;
  $230 = $229 & 4;
  $231 = ($230|0)!=(0);
  if ($231) {
   $232 = HEAP32[$i>>2]|0;
   $233 = (($apDiv) + ($232<<2)|0);
   $234 = HEAP32[$233>>2]|0;
   $235 = $234;
   $236 = HEAP32[$1>>2]|0;
   $237 = ((($236)) + 56|0);
   $238 = HEAP32[$237>>2]|0;
   $239 = $238;
   $240 = (($235) - ($239))|0;
   HEAP32[$iOff>>2] = $240;
   $241 = HEAP32[$iOff>>2]|0;
   $242 = HEAP32[$i>>2]|0;
   $243 = (($szNew) + ($242<<2)|0);
   $244 = HEAP32[$243>>2]|0;
   $245 = (($241) + ($244))|0;
   $246 = HEAP32[$pBt>>2]|0;
   $247 = ((($246)) + 36|0);
   $248 = HEAP32[$247>>2]|0;
   $249 = ($245|0)>($248|0);
   if ($249) {
    label = 23;
    break;
   }
   $254 = HEAP32[$iOff>>2]|0;
   $255 = HEAP32[$3>>2]|0;
   $256 = (($255) + ($254)|0);
   $257 = HEAP32[$i>>2]|0;
   $258 = (($apDiv) + ($257<<2)|0);
   $259 = HEAP32[$258>>2]|0;
   $260 = HEAP32[$i>>2]|0;
   $261 = (($szNew) + ($260<<2)|0);
   $262 = HEAP32[$261>>2]|0;
   _memcpy(($256|0),($259|0),($262|0))|0;
   $263 = HEAP32[$i>>2]|0;
   $264 = (($apDiv) + ($263<<2)|0);
   $265 = HEAP32[$264>>2]|0;
   $266 = HEAP32[$1>>2]|0;
   $267 = ((($266)) + 56|0);
   $268 = HEAP32[$267>>2]|0;
   $269 = $265;
   $270 = $268;
   $271 = (($269) - ($270))|0;
   $272 = HEAP32[$3>>2]|0;
   $273 = (($272) + ($271)|0);
   $274 = HEAP32[$i>>2]|0;
   $275 = (($apDiv) + ($274<<2)|0);
   HEAP32[$275>>2] = $273;
  }
  $276 = HEAP32[$1>>2]|0;
  $277 = HEAP32[$i>>2]|0;
  $278 = HEAP32[$nxDiv>>2]|0;
  $279 = (($277) + ($278))|0;
  $280 = HEAP32[$1>>2]|0;
  $281 = ((($280)) + 1|0);
  $282 = HEAP8[$281>>0]|0;
  $283 = $282&255;
  $284 = (($279) - ($283))|0;
  $285 = HEAP32[$i>>2]|0;
  $286 = (($szNew) + ($285<<2)|0);
  $287 = HEAP32[$286>>2]|0;
  _dropCell($276,$284,$287,$rc);
 }
 L32: do {
  if ((label|0) == 16) {
   $110 = HEAP32[$i>>2]|0;
   $111 = (($110) + 1)|0;
   $112 = $111<<2;
   _memset(($apOld|0),0,($112|0))|0;
  }
  else if ((label|0) == 23) {
   $250 = (_sqlite3CorruptError(65347)|0);
   HEAP32[$rc>>2] = $250;
   $251 = HEAP32[$i>>2]|0;
   $252 = (($251) + 1)|0;
   $253 = $252<<2;
   _memset(($apOld|0),0,($253|0))|0;
  }
  else if ((label|0) == 26) {
   $288 = HEAP32[$nMaxCells>>2]|0;
   $289 = (($288) + 3)|0;
   $290 = $289 & -4;
   HEAP32[$nMaxCells>>2] = $290;
   $291 = HEAP32[$nMaxCells>>2]|0;
   $292 = $291<<2;
   $293 = HEAP32[$nMaxCells>>2]|0;
   $294 = $293<<1;
   $295 = (($292) + ($294))|0;
   $296 = HEAP32[$pBt>>2]|0;
   $297 = ((($296)) + 32|0);
   $298 = HEAP32[$297>>2]|0;
   $299 = (($295) + ($298))|0;
   HEAP32[$szScratch>>2] = $299;
   $300 = HEAP32[$szScratch>>2]|0;
   $301 = (_sqlite3ScratchMalloc($300)|0);
   $302 = ((($b)) + 8|0);
   HEAP32[$302>>2] = $301;
   $303 = ((($b)) + 8|0);
   $304 = HEAP32[$303>>2]|0;
   $305 = ($304|0)==(0|0);
   if ($305) {
    HEAP32[$rc>>2] = 7;
    break;
   }
   $306 = HEAP32[$nMaxCells>>2]|0;
   $307 = ((($b)) + 8|0);
   $308 = HEAP32[$307>>2]|0;
   $309 = (($308) + ($306<<2)|0);
   $310 = ((($b)) + 12|0);
   HEAP32[$310>>2] = $309;
   $311 = HEAP32[$nMaxCells>>2]|0;
   $312 = ((($b)) + 12|0);
   $313 = HEAP32[$312>>2]|0;
   $314 = (($313) + ($311<<1)|0);
   HEAP32[$aSpace1>>2] = $314;
   $315 = HEAP32[$apOld>>2]|0;
   $316 = ((($b)) + 4|0);
   HEAP32[$316>>2] = $315;
   $317 = ((($b)) + 4|0);
   $318 = HEAP32[$317>>2]|0;
   $319 = ((($318)) + 4|0);
   $320 = HEAP8[$319>>0]|0;
   $321 = $320&255;
   $322 = $321<<2;
   $323 = $322&65535;
   HEAP16[$leafCorrection>>1] = $323;
   $324 = ((($b)) + 4|0);
   $325 = HEAP32[$324>>2]|0;
   $326 = ((($325)) + 3|0);
   $327 = HEAP8[$326>>0]|0;
   $328 = $327&255;
   HEAP32[$leafData>>2] = $328;
   HEAP32[$i>>2] = 0;
   while(1) {
    $329 = HEAP32[$i>>2]|0;
    $330 = HEAP32[$nOld>>2]|0;
    $331 = ($329|0)<($330|0);
    if (!($331)) {
     break;
    }
    $332 = HEAP32[$i>>2]|0;
    $333 = (($apOld) + ($332<<2)|0);
    $334 = HEAP32[$333>>2]|0;
    HEAP32[$pOld>>2] = $334;
    $335 = HEAP32[$pOld>>2]|0;
    $336 = ((($335)) + 18|0);
    $337 = HEAP16[$336>>1]|0;
    $338 = $337&65535;
    HEAP32[$limit>>2] = $338;
    $339 = HEAP32[$pOld>>2]|0;
    $340 = ((($339)) + 56|0);
    $341 = HEAP32[$340>>2]|0;
    HEAP32[$aData>>2] = $341;
    $342 = HEAP32[$pOld>>2]|0;
    $343 = ((($342)) + 20|0);
    $344 = HEAP16[$343>>1]|0;
    HEAP16[$maskPage>>1] = $344;
    $345 = HEAP32[$aData>>2]|0;
    $346 = HEAP32[$pOld>>2]|0;
    $347 = ((($346)) + 14|0);
    $348 = HEAP16[$347>>1]|0;
    $349 = $348&65535;
    $350 = (($345) + ($349)|0);
    HEAP32[$piCell>>2] = $350;
    $351 = HEAP32[$pOld>>2]|0;
    $352 = ((($351)) + 56|0);
    $353 = HEAP32[$352>>2]|0;
    $354 = HEAP8[$353>>0]|0;
    $355 = $354&255;
    $356 = HEAP32[$apOld>>2]|0;
    $357 = ((($356)) + 56|0);
    $358 = HEAP32[$357>>2]|0;
    $359 = HEAP8[$358>>0]|0;
    $360 = $359&255;
    $361 = ($355|0)!=($360|0);
    if ($361) {
     label = 31;
     break;
    }
    $363 = HEAP32[$b>>2]|0;
    $364 = ((($b)) + 12|0);
    $365 = HEAP32[$364>>2]|0;
    $366 = (($365) + ($363<<1)|0);
    $367 = HEAP32[$limit>>2]|0;
    $368 = HEAP32[$pOld>>2]|0;
    $369 = ((($368)) + 1|0);
    $370 = HEAP8[$369>>0]|0;
    $371 = $370&255;
    $372 = (($367) + ($371))|0;
    $373 = $372<<1;
    _memset(($366|0),0,($373|0))|0;
    $374 = HEAP32[$pOld>>2]|0;
    $375 = ((($374)) + 1|0);
    $376 = HEAP8[$375>>0]|0;
    $377 = $376&255;
    $378 = ($377|0)>(0);
    L43: do {
     if ($378) {
      $379 = HEAP32[$pOld>>2]|0;
      $380 = ((($379)) + 22|0);
      $381 = HEAP16[$380>>1]|0;
      $382 = $381&65535;
      HEAP32[$limit>>2] = $382;
      HEAP32[$j>>2] = 0;
      while(1) {
       $383 = HEAP32[$j>>2]|0;
       $384 = HEAP32[$limit>>2]|0;
       $385 = ($383|0)<($384|0);
       if (!($385)) {
        break;
       }
       $386 = HEAP32[$aData>>2]|0;
       $387 = HEAP16[$maskPage>>1]|0;
       $388 = $387&65535;
       $389 = HEAP32[$piCell>>2]|0;
       $390 = HEAP8[$389>>0]|0;
       $391 = $390&255;
       $392 = $391 << 8;
       $393 = HEAP32[$piCell>>2]|0;
       $394 = ((($393)) + 1|0);
       $395 = HEAP8[$394>>0]|0;
       $396 = $395&255;
       $397 = $392 | $396;
       $398 = $388 & $397;
       $399 = (($386) + ($398)|0);
       $400 = HEAP32[$b>>2]|0;
       $401 = ((($b)) + 8|0);
       $402 = HEAP32[$401>>2]|0;
       $403 = (($402) + ($400<<2)|0);
       HEAP32[$403>>2] = $399;
       $404 = HEAP32[$piCell>>2]|0;
       $405 = ((($404)) + 2|0);
       HEAP32[$piCell>>2] = $405;
       $406 = HEAP32[$b>>2]|0;
       $407 = (($406) + 1)|0;
       HEAP32[$b>>2] = $407;
       $408 = HEAP32[$j>>2]|0;
       $409 = (($408) + 1)|0;
       HEAP32[$j>>2] = $409;
      }
      HEAP32[$k>>2] = 0;
      while(1) {
       $410 = HEAP32[$k>>2]|0;
       $411 = HEAP32[$pOld>>2]|0;
       $412 = ((($411)) + 1|0);
       $413 = HEAP8[$412>>0]|0;
       $414 = $413&255;
       $415 = ($410|0)<($414|0);
       if (!($415)) {
        break L43;
       }
       $416 = HEAP32[$k>>2]|0;
       $417 = HEAP32[$pOld>>2]|0;
       $418 = ((($417)) + 32|0);
       $419 = (($418) + ($416<<2)|0);
       $420 = HEAP32[$419>>2]|0;
       $421 = HEAP32[$b>>2]|0;
       $422 = ((($b)) + 8|0);
       $423 = HEAP32[$422>>2]|0;
       $424 = (($423) + ($421<<2)|0);
       HEAP32[$424>>2] = $420;
       $425 = HEAP32[$b>>2]|0;
       $426 = (($425) + 1)|0;
       HEAP32[$b>>2] = $426;
       $427 = HEAP32[$k>>2]|0;
       $428 = (($427) + 1)|0;
       HEAP32[$k>>2] = $428;
      }
     }
    } while(0);
    $429 = HEAP32[$aData>>2]|0;
    $430 = HEAP32[$pOld>>2]|0;
    $431 = ((($430)) + 14|0);
    $432 = HEAP16[$431>>1]|0;
    $433 = $432&65535;
    $434 = (($429) + ($433)|0);
    $435 = HEAP32[$pOld>>2]|0;
    $436 = ((($435)) + 18|0);
    $437 = HEAP16[$436>>1]|0;
    $438 = $437&65535;
    $439 = $438<<1;
    $440 = (($434) + ($439)|0);
    HEAP32[$piEnd>>2] = $440;
    while(1) {
     $441 = HEAP32[$piCell>>2]|0;
     $442 = HEAP32[$piEnd>>2]|0;
     $443 = ($441>>>0)<($442>>>0);
     if (!($443)) {
      break;
     }
     $444 = HEAP32[$aData>>2]|0;
     $445 = HEAP16[$maskPage>>1]|0;
     $446 = $445&65535;
     $447 = HEAP32[$piCell>>2]|0;
     $448 = HEAP8[$447>>0]|0;
     $449 = $448&255;
     $450 = $449 << 8;
     $451 = HEAP32[$piCell>>2]|0;
     $452 = ((($451)) + 1|0);
     $453 = HEAP8[$452>>0]|0;
     $454 = $453&255;
     $455 = $450 | $454;
     $456 = $446 & $455;
     $457 = (($444) + ($456)|0);
     $458 = HEAP32[$b>>2]|0;
     $459 = ((($b)) + 8|0);
     $460 = HEAP32[$459>>2]|0;
     $461 = (($460) + ($458<<2)|0);
     HEAP32[$461>>2] = $457;
     $462 = HEAP32[$piCell>>2]|0;
     $463 = ((($462)) + 2|0);
     HEAP32[$piCell>>2] = $463;
     $464 = HEAP32[$b>>2]|0;
     $465 = (($464) + 1)|0;
     HEAP32[$b>>2] = $465;
    }
    $466 = HEAP32[$b>>2]|0;
    $467 = HEAP32[$i>>2]|0;
    $468 = (($cntOld) + ($467<<2)|0);
    HEAP32[$468>>2] = $466;
    $469 = HEAP32[$i>>2]|0;
    $470 = HEAP32[$nOld>>2]|0;
    $471 = (($470) - 1)|0;
    $472 = ($469|0)>=($471|0);
    $473 = HEAP32[$leafData>>2]|0;
    $474 = ($473|0)!=(0);
    $or$cond = $472 | $474;
    if (!($or$cond)) {
     $475 = HEAP32[$i>>2]|0;
     $476 = (($szNew) + ($475<<2)|0);
     $477 = HEAP32[$476>>2]|0;
     $478 = $477&65535;
     HEAP16[$sz>>1] = $478;
     $479 = HEAP16[$sz>>1]|0;
     $480 = HEAP32[$b>>2]|0;
     $481 = ((($b)) + 12|0);
     $482 = HEAP32[$481>>2]|0;
     $483 = (($482) + ($480<<1)|0);
     HEAP16[$483>>1] = $479;
     $484 = HEAP32[$iSpace1>>2]|0;
     $485 = HEAP32[$aSpace1>>2]|0;
     $486 = (($485) + ($484)|0);
     HEAP32[$pTemp>>2] = $486;
     $487 = HEAP16[$sz>>1]|0;
     $488 = $487&65535;
     $489 = HEAP32[$iSpace1>>2]|0;
     $490 = (($489) + ($488))|0;
     HEAP32[$iSpace1>>2] = $490;
     $491 = HEAP32[$pTemp>>2]|0;
     $492 = HEAP32[$i>>2]|0;
     $493 = (($apDiv) + ($492<<2)|0);
     $494 = HEAP32[$493>>2]|0;
     $495 = HEAP16[$sz>>1]|0;
     $496 = $495&65535;
     _memcpy(($491|0),($494|0),($496|0))|0;
     $497 = HEAP32[$pTemp>>2]|0;
     $498 = HEAP16[$leafCorrection>>1]|0;
     $499 = $498&65535;
     $500 = (($497) + ($499)|0);
     $501 = HEAP32[$b>>2]|0;
     $502 = ((($b)) + 8|0);
     $503 = HEAP32[$502>>2]|0;
     $504 = (($503) + ($501<<2)|0);
     HEAP32[$504>>2] = $500;
     $505 = HEAP32[$b>>2]|0;
     $506 = ((($b)) + 12|0);
     $507 = HEAP32[$506>>2]|0;
     $508 = (($507) + ($505<<1)|0);
     $509 = HEAP16[$508>>1]|0;
     $510 = $509&65535;
     $511 = HEAP16[$leafCorrection>>1]|0;
     $512 = $511&65535;
     $513 = (($510) - ($512))|0;
     $514 = $513&65535;
     $515 = HEAP32[$b>>2]|0;
     $516 = ((($b)) + 12|0);
     $517 = HEAP32[$516>>2]|0;
     $518 = (($517) + ($515<<1)|0);
     HEAP16[$518>>1] = $514;
     $519 = HEAP32[$pOld>>2]|0;
     $520 = ((($519)) + 4|0);
     $521 = HEAP8[$520>>0]|0;
     $522 = ($521<<24>>24)!=(0);
     L59: do {
      if ($522) {
       while(1) {
        $532 = HEAP32[$b>>2]|0;
        $533 = ((($b)) + 12|0);
        $534 = HEAP32[$533>>2]|0;
        $535 = (($534) + ($532<<1)|0);
        $536 = HEAP16[$535>>1]|0;
        $537 = $536&65535;
        $538 = ($537|0)<(4);
        if (!($538)) {
         break L59;
        }
        $539 = HEAP32[$iSpace1>>2]|0;
        $540 = (($539) + 1)|0;
        HEAP32[$iSpace1>>2] = $540;
        $541 = HEAP32[$aSpace1>>2]|0;
        $542 = (($541) + ($539)|0);
        HEAP8[$542>>0] = 0;
        $543 = HEAP32[$b>>2]|0;
        $544 = ((($b)) + 12|0);
        $545 = HEAP32[$544>>2]|0;
        $546 = (($545) + ($543<<1)|0);
        $547 = HEAP16[$546>>1]|0;
        $548 = (($547) + 1)<<16>>16;
        HEAP16[$546>>1] = $548;
       }
      } else {
       $523 = HEAP32[$b>>2]|0;
       $524 = ((($b)) + 8|0);
       $525 = HEAP32[$524>>2]|0;
       $526 = (($525) + ($523<<2)|0);
       $527 = HEAP32[$526>>2]|0;
       $528 = HEAP32[$pOld>>2]|0;
       $529 = ((($528)) + 56|0);
       $530 = HEAP32[$529>>2]|0;
       $531 = ((($530)) + 8|0);
       ;HEAP8[$527>>0]=HEAP8[$531>>0]|0;HEAP8[$527+1>>0]=HEAP8[$531+1>>0]|0;HEAP8[$527+2>>0]=HEAP8[$531+2>>0]|0;HEAP8[$527+3>>0]=HEAP8[$531+3>>0]|0;
      }
     } while(0);
     $549 = HEAP32[$b>>2]|0;
     $550 = (($549) + 1)|0;
     HEAP32[$b>>2] = $550;
    }
    $551 = HEAP32[$i>>2]|0;
    $552 = (($551) + 1)|0;
    HEAP32[$i>>2] = $552;
   }
   if ((label|0) == 31) {
    $362 = (_sqlite3CorruptError(65414)|0);
    HEAP32[$rc>>2] = $362;
    break;
   }
   $553 = HEAP32[$pBt>>2]|0;
   $554 = ((($553)) + 36|0);
   $555 = HEAP32[$554>>2]|0;
   $556 = (($555) - 12)|0;
   $557 = HEAP16[$leafCorrection>>1]|0;
   $558 = $557&65535;
   $559 = (($556) + ($558))|0;
   HEAP32[$usableSpace>>2] = $559;
   HEAP32[$i>>2] = 0;
   while(1) {
    $560 = HEAP32[$i>>2]|0;
    $561 = HEAP32[$nOld>>2]|0;
    $562 = ($560|0)<($561|0);
    if (!($562)) {
     break;
    }
    $563 = HEAP32[$i>>2]|0;
    $564 = (($apOld) + ($563<<2)|0);
    $565 = HEAP32[$564>>2]|0;
    HEAP32[$p>>2] = $565;
    $566 = HEAP32[$usableSpace>>2]|0;
    $567 = HEAP32[$p>>2]|0;
    $568 = ((($567)) + 16|0);
    $569 = HEAP16[$568>>1]|0;
    $570 = $569&65535;
    $571 = (($566) - ($570))|0;
    $572 = HEAP32[$i>>2]|0;
    $573 = (($szNew) + ($572<<2)|0);
    HEAP32[$573>>2] = $571;
    $574 = HEAP32[$i>>2]|0;
    $575 = (($szNew) + ($574<<2)|0);
    $576 = HEAP32[$575>>2]|0;
    $577 = ($576|0)<(0);
    if ($577) {
     label = 52;
     break;
    }
    HEAP32[$j>>2] = 0;
    while(1) {
     $579 = HEAP32[$j>>2]|0;
     $580 = HEAP32[$p>>2]|0;
     $581 = ((($580)) + 1|0);
     $582 = HEAP8[$581>>0]|0;
     $583 = $582&255;
     $584 = ($579|0)<($583|0);
     if (!($584)) {
      break;
     }
     $585 = HEAP32[$p>>2]|0;
     $586 = ((($585)) + 76|0);
     $587 = HEAP32[$586>>2]|0;
     $588 = HEAP32[$p>>2]|0;
     $589 = HEAP32[$j>>2]|0;
     $590 = HEAP32[$p>>2]|0;
     $591 = ((($590)) + 32|0);
     $592 = (($591) + ($589<<2)|0);
     $593 = HEAP32[$592>>2]|0;
     $594 = (FUNCTION_TABLE_iii[$587 & 255]($588,$593)|0);
     $595 = $594&65535;
     $596 = (2 + ($595))|0;
     $597 = HEAP32[$i>>2]|0;
     $598 = (($szNew) + ($597<<2)|0);
     $599 = HEAP32[$598>>2]|0;
     $600 = (($599) + ($596))|0;
     HEAP32[$598>>2] = $600;
     $601 = HEAP32[$j>>2]|0;
     $602 = (($601) + 1)|0;
     HEAP32[$j>>2] = $602;
    }
    $603 = HEAP32[$i>>2]|0;
    $604 = (($cntOld) + ($603<<2)|0);
    $605 = HEAP32[$604>>2]|0;
    $606 = HEAP32[$i>>2]|0;
    $607 = (($cntNew) + ($606<<2)|0);
    HEAP32[$607>>2] = $605;
    $608 = HEAP32[$i>>2]|0;
    $609 = (($608) + 1)|0;
    HEAP32[$i>>2] = $609;
   }
   if ((label|0) == 52) {
    $578 = (_sqlite3CorruptError(65512)|0);
    HEAP32[$rc>>2] = $578;
    break;
   }
   $610 = HEAP32[$nOld>>2]|0;
   HEAP32[$k>>2] = $610;
   HEAP32[$i>>2] = 0;
   L80: while(1) {
    $611 = HEAP32[$i>>2]|0;
    $612 = HEAP32[$k>>2]|0;
    $613 = ($611|0)<($612|0);
    if (!($613)) {
     label = 83;
     break;
    }
    while(1) {
     $614 = HEAP32[$i>>2]|0;
     $615 = (($szNew) + ($614<<2)|0);
     $616 = HEAP32[$615>>2]|0;
     $617 = HEAP32[$usableSpace>>2]|0;
     $618 = ($616|0)>($617|0);
     if (!($618)) {
      break;
     }
     $619 = HEAP32[$i>>2]|0;
     $620 = (($619) + 1)|0;
     $621 = HEAP32[$k>>2]|0;
     $622 = ($620|0)>=($621|0);
     if ($622) {
      $623 = HEAP32[$i>>2]|0;
      $624 = (($623) + 2)|0;
      HEAP32[$k>>2] = $624;
      $625 = HEAP32[$k>>2]|0;
      $626 = ($625|0)>(5);
      if ($626) {
       label = 62;
       break L80;
      }
      $628 = HEAP32[$k>>2]|0;
      $629 = (($628) - 1)|0;
      $630 = (($szNew) + ($629<<2)|0);
      HEAP32[$630>>2] = 0;
      $631 = HEAP32[$b>>2]|0;
      $632 = HEAP32[$k>>2]|0;
      $633 = (($632) - 1)|0;
      $634 = (($cntNew) + ($633<<2)|0);
      HEAP32[$634>>2] = $631;
     }
     $635 = HEAP32[$i>>2]|0;
     $636 = (($cntNew) + ($635<<2)|0);
     $637 = HEAP32[$636>>2]|0;
     $638 = (($637) - 1)|0;
     $639 = (_cachedCellSize($b,$638)|0);
     $640 = $639&65535;
     $641 = (2 + ($640))|0;
     HEAP32[$sz1>>2] = $641;
     $642 = HEAP32[$sz1>>2]|0;
     $643 = HEAP32[$i>>2]|0;
     $644 = (($szNew) + ($643<<2)|0);
     $645 = HEAP32[$644>>2]|0;
     $646 = (($645) - ($642))|0;
     HEAP32[$644>>2] = $646;
     $647 = HEAP32[$leafData>>2]|0;
     $648 = ($647|0)!=(0);
     do {
      if (!($648)) {
       $649 = HEAP32[$i>>2]|0;
       $650 = (($cntNew) + ($649<<2)|0);
       $651 = HEAP32[$650>>2]|0;
       $652 = HEAP32[$b>>2]|0;
       $653 = ($651|0)<($652|0);
       if ($653) {
        $654 = HEAP32[$i>>2]|0;
        $655 = (($cntNew) + ($654<<2)|0);
        $656 = HEAP32[$655>>2]|0;
        $657 = (_cachedCellSize($b,$656)|0);
        $658 = $657&65535;
        $659 = (2 + ($658))|0;
        HEAP32[$sz1>>2] = $659;
        break;
       } else {
        HEAP32[$sz1>>2] = 0;
        break;
       }
      }
     } while(0);
     $660 = HEAP32[$sz1>>2]|0;
     $661 = HEAP32[$i>>2]|0;
     $662 = (($661) + 1)|0;
     $663 = (($szNew) + ($662<<2)|0);
     $664 = HEAP32[$663>>2]|0;
     $665 = (($664) + ($660))|0;
     HEAP32[$663>>2] = $665;
     $666 = HEAP32[$i>>2]|0;
     $667 = (($cntNew) + ($666<<2)|0);
     $668 = HEAP32[$667>>2]|0;
     $669 = (($668) + -1)|0;
     HEAP32[$667>>2] = $669;
    }
    while(1) {
     $670 = HEAP32[$i>>2]|0;
     $671 = (($cntNew) + ($670<<2)|0);
     $672 = HEAP32[$671>>2]|0;
     $673 = HEAP32[$b>>2]|0;
     $674 = ($672|0)<($673|0);
     if (!($674)) {
      break;
     }
     $675 = HEAP32[$i>>2]|0;
     $676 = (($cntNew) + ($675<<2)|0);
     $677 = HEAP32[$676>>2]|0;
     $678 = (_cachedCellSize($b,$677)|0);
     $679 = $678&65535;
     $680 = (2 + ($679))|0;
     HEAP32[$sz1>>2] = $680;
     $681 = HEAP32[$i>>2]|0;
     $682 = (($szNew) + ($681<<2)|0);
     $683 = HEAP32[$682>>2]|0;
     $684 = HEAP32[$sz1>>2]|0;
     $685 = (($683) + ($684))|0;
     $686 = HEAP32[$usableSpace>>2]|0;
     $687 = ($685|0)>($686|0);
     if ($687) {
      break;
     }
     $688 = HEAP32[$sz1>>2]|0;
     $689 = HEAP32[$i>>2]|0;
     $690 = (($szNew) + ($689<<2)|0);
     $691 = HEAP32[$690>>2]|0;
     $692 = (($691) + ($688))|0;
     HEAP32[$690>>2] = $692;
     $693 = HEAP32[$i>>2]|0;
     $694 = (($cntNew) + ($693<<2)|0);
     $695 = HEAP32[$694>>2]|0;
     $696 = (($695) + 1)|0;
     HEAP32[$694>>2] = $696;
     $697 = HEAP32[$leafData>>2]|0;
     $698 = ($697|0)!=(0);
     do {
      if (!($698)) {
       $699 = HEAP32[$i>>2]|0;
       $700 = (($cntNew) + ($699<<2)|0);
       $701 = HEAP32[$700>>2]|0;
       $702 = HEAP32[$b>>2]|0;
       $703 = ($701|0)<($702|0);
       if ($703) {
        $704 = HEAP32[$i>>2]|0;
        $705 = (($cntNew) + ($704<<2)|0);
        $706 = HEAP32[$705>>2]|0;
        $707 = (_cachedCellSize($b,$706)|0);
        $708 = $707&65535;
        $709 = (2 + ($708))|0;
        HEAP32[$sz1>>2] = $709;
        break;
       } else {
        HEAP32[$sz1>>2] = 0;
        break;
       }
      }
     } while(0);
     $710 = HEAP32[$sz1>>2]|0;
     $711 = HEAP32[$i>>2]|0;
     $712 = (($711) + 1)|0;
     $713 = (($szNew) + ($712<<2)|0);
     $714 = HEAP32[$713>>2]|0;
     $715 = (($714) - ($710))|0;
     HEAP32[$713>>2] = $715;
    }
    $716 = HEAP32[$i>>2]|0;
    $717 = (($cntNew) + ($716<<2)|0);
    $718 = HEAP32[$717>>2]|0;
    $719 = HEAP32[$b>>2]|0;
    $720 = ($718|0)>=($719|0);
    $721 = HEAP32[$i>>2]|0;
    if ($720) {
     $722 = (($721) + 1)|0;
     HEAP32[$k>>2] = $722;
    } else {
     $723 = (($cntNew) + ($721<<2)|0);
     $724 = HEAP32[$723>>2]|0;
     $725 = HEAP32[$i>>2]|0;
     $726 = ($725|0)>(0);
     if ($726) {
      $727 = HEAP32[$i>>2]|0;
      $728 = (($727) - 1)|0;
      $729 = (($cntNew) + ($728<<2)|0);
      $730 = HEAP32[$729>>2]|0;
      $732 = $730;
     } else {
      $732 = 0;
     }
     $731 = ($724|0)<=($732|0);
     if ($731) {
      label = 81;
      break;
     }
    }
    $734 = HEAP32[$i>>2]|0;
    $735 = (($734) + 1)|0;
    HEAP32[$i>>2] = $735;
   }
   if ((label|0) == 62) {
    $627 = (_sqlite3CorruptError(65524)|0);
    HEAP32[$rc>>2] = $627;
    break;
   }
   else if ((label|0) == 81) {
    $733 = (_sqlite3CorruptError(65557)|0);
    HEAP32[$rc>>2] = $733;
    break;
   }
   else if ((label|0) == 83) {
    $736 = HEAP32[$k>>2]|0;
    $737 = (($736) - 1)|0;
    HEAP32[$i>>2] = $737;
    while(1) {
     $738 = HEAP32[$i>>2]|0;
     $739 = ($738|0)>(0);
     if (!($739)) {
      break;
     }
     $740 = HEAP32[$i>>2]|0;
     $741 = (($szNew) + ($740<<2)|0);
     $742 = HEAP32[$741>>2]|0;
     HEAP32[$szRight>>2] = $742;
     $743 = HEAP32[$i>>2]|0;
     $744 = (($743) - 1)|0;
     $745 = (($szNew) + ($744<<2)|0);
     $746 = HEAP32[$745>>2]|0;
     HEAP32[$szLeft>>2] = $746;
     $747 = HEAP32[$i>>2]|0;
     $748 = (($747) - 1)|0;
     $749 = (($cntNew) + ($748<<2)|0);
     $750 = HEAP32[$749>>2]|0;
     $751 = (($750) - 1)|0;
     HEAP32[$r>>2] = $751;
     $752 = HEAP32[$r>>2]|0;
     $753 = (($752) + 1)|0;
     $754 = HEAP32[$leafData>>2]|0;
     $755 = (($753) - ($754))|0;
     HEAP32[$d>>2] = $755;
     $756 = HEAP32[$d>>2]|0;
     (_cachedCellSize($b,$756)|0);
     while(1) {
      $757 = HEAP32[$r>>2]|0;
      (_cachedCellSize($b,$757)|0);
      $758 = HEAP32[$szRight>>2]|0;
      $759 = ($758|0)!=(0);
      if ($759) {
       $760 = HEAP32[$5>>2]|0;
       $761 = ($760|0)!=(0);
       if ($761) {
        break;
       }
       $762 = HEAP32[$szRight>>2]|0;
       $763 = HEAP32[$d>>2]|0;
       $764 = ((($b)) + 12|0);
       $765 = HEAP32[$764>>2]|0;
       $766 = (($765) + ($763<<1)|0);
       $767 = HEAP16[$766>>1]|0;
       $768 = $767&65535;
       $769 = (($762) + ($768))|0;
       $770 = (($769) + 2)|0;
       $771 = HEAP32[$szLeft>>2]|0;
       $772 = HEAP32[$r>>2]|0;
       $773 = ((($b)) + 12|0);
       $774 = HEAP32[$773>>2]|0;
       $775 = (($774) + ($772<<1)|0);
       $776 = HEAP16[$775>>1]|0;
       $777 = $776&65535;
       $778 = HEAP32[$i>>2]|0;
       $779 = HEAP32[$k>>2]|0;
       $780 = (($779) - 1)|0;
       $781 = ($778|0)==($780|0);
       $782 = $781 ? 0 : 2;
       $783 = (($777) + ($782))|0;
       $784 = (($771) - ($783))|0;
       $785 = ($770|0)>($784|0);
       if ($785) {
        break;
       }
      }
      $786 = HEAP32[$d>>2]|0;
      $787 = ((($b)) + 12|0);
      $788 = HEAP32[$787>>2]|0;
      $789 = (($788) + ($786<<1)|0);
      $790 = HEAP16[$789>>1]|0;
      $791 = $790&65535;
      $792 = (($791) + 2)|0;
      $793 = HEAP32[$szRight>>2]|0;
      $794 = (($793) + ($792))|0;
      HEAP32[$szRight>>2] = $794;
      $795 = HEAP32[$r>>2]|0;
      $796 = ((($b)) + 12|0);
      $797 = HEAP32[$796>>2]|0;
      $798 = (($797) + ($795<<1)|0);
      $799 = HEAP16[$798>>1]|0;
      $800 = $799&65535;
      $801 = (($800) + 2)|0;
      $802 = HEAP32[$szLeft>>2]|0;
      $803 = (($802) - ($801))|0;
      HEAP32[$szLeft>>2] = $803;
      $804 = HEAP32[$r>>2]|0;
      $805 = HEAP32[$i>>2]|0;
      $806 = (($805) - 1)|0;
      $807 = (($cntNew) + ($806<<2)|0);
      HEAP32[$807>>2] = $804;
      $808 = HEAP32[$r>>2]|0;
      $809 = (($808) + -1)|0;
      HEAP32[$r>>2] = $809;
      $810 = HEAP32[$d>>2]|0;
      $811 = (($810) + -1)|0;
      HEAP32[$d>>2] = $811;
      $812 = HEAP32[$r>>2]|0;
      $813 = ($812|0)>=(0);
      if (!($813)) {
       break;
      }
     }
     $814 = HEAP32[$szRight>>2]|0;
     $815 = HEAP32[$i>>2]|0;
     $816 = (($szNew) + ($815<<2)|0);
     HEAP32[$816>>2] = $814;
     $817 = HEAP32[$szLeft>>2]|0;
     $818 = HEAP32[$i>>2]|0;
     $819 = (($818) - 1)|0;
     $820 = (($szNew) + ($819<<2)|0);
     HEAP32[$820>>2] = $817;
     $821 = HEAP32[$i>>2]|0;
     $822 = (($821) - 1)|0;
     $823 = (($cntNew) + ($822<<2)|0);
     $824 = HEAP32[$823>>2]|0;
     $825 = HEAP32[$i>>2]|0;
     $826 = ($825|0)>(1);
     if ($826) {
      $827 = HEAP32[$i>>2]|0;
      $828 = (($827) - 2)|0;
      $829 = (($cntNew) + ($828<<2)|0);
      $830 = HEAP32[$829>>2]|0;
      $832 = $830;
     } else {
      $832 = 0;
     }
     $831 = ($824|0)<=($832|0);
     if ($831) {
      label = 93;
      break;
     }
     $834 = HEAP32[$i>>2]|0;
     $835 = (($834) + -1)|0;
     HEAP32[$i>>2] = $835;
    }
    if ((label|0) == 93) {
     $833 = (_sqlite3CorruptError(65599)|0);
     HEAP32[$rc>>2] = $833;
     break;
    }
    $836 = HEAP32[$apOld>>2]|0;
    $837 = ((($836)) + 56|0);
    $838 = HEAP32[$837>>2]|0;
    $839 = HEAP8[$838>>0]|0;
    $840 = $839&255;
    HEAP32[$pageFlags>>2] = $840;
    HEAP32[$i>>2] = 0;
    while(1) {
     $841 = HEAP32[$i>>2]|0;
     $842 = HEAP32[$k>>2]|0;
     $843 = ($841|0)<($842|0);
     if (!($843)) {
      break;
     }
     $844 = HEAP32[$i>>2]|0;
     $845 = HEAP32[$nOld>>2]|0;
     $846 = ($844|0)<($845|0);
     if ($846) {
      $847 = HEAP32[$i>>2]|0;
      $848 = (($apOld) + ($847<<2)|0);
      $849 = HEAP32[$848>>2]|0;
      $850 = HEAP32[$i>>2]|0;
      $851 = (($apNew) + ($850<<2)|0);
      HEAP32[$851>>2] = $849;
      HEAP32[$pNew>>2] = $849;
      $852 = HEAP32[$i>>2]|0;
      $853 = (($apOld) + ($852<<2)|0);
      HEAP32[$853>>2] = 0;
      $854 = HEAP32[$pNew>>2]|0;
      $855 = ((($854)) + 72|0);
      $856 = HEAP32[$855>>2]|0;
      $857 = (_sqlite3PagerWrite($856)|0);
      HEAP32[$rc>>2] = $857;
      $858 = HEAP32[$nNew>>2]|0;
      $859 = (($858) + 1)|0;
      HEAP32[$nNew>>2] = $859;
      $860 = HEAP32[$rc>>2]|0;
      $861 = ($860|0)!=(0);
      if ($861) {
       break L32;
      }
     } else {
      $862 = HEAP32[$pBt>>2]|0;
      $863 = HEAP32[$5>>2]|0;
      $864 = ($863|0)!=(0);
      $865 = HEAP32[$pgno>>2]|0;
      $866 = $864 ? 1 : $865;
      $867 = (_allocateBtreePage($862,$pNew,$pgno,$866,0)|0);
      HEAP32[$rc>>2] = $867;
      $868 = HEAP32[$rc>>2]|0;
      $869 = ($868|0)!=(0);
      if ($869) {
       break L32;
      }
      $870 = HEAP32[$pNew>>2]|0;
      $871 = HEAP32[$pageFlags>>2]|0;
      _zeroPage($870,$871);
      $872 = HEAP32[$pNew>>2]|0;
      $873 = HEAP32[$i>>2]|0;
      $874 = (($apNew) + ($873<<2)|0);
      HEAP32[$874>>2] = $872;
      $875 = HEAP32[$nNew>>2]|0;
      $876 = (($875) + 1)|0;
      HEAP32[$nNew>>2] = $876;
      $877 = HEAP32[$b>>2]|0;
      $878 = HEAP32[$i>>2]|0;
      $879 = (($cntOld) + ($878<<2)|0);
      HEAP32[$879>>2] = $877;
      $880 = HEAP32[$pBt>>2]|0;
      $881 = ((($880)) + 17|0);
      $882 = HEAP8[$881>>0]|0;
      $883 = ($882<<24>>24)!=(0);
      if ($883) {
       $884 = HEAP32[$pBt>>2]|0;
       $885 = HEAP32[$pNew>>2]|0;
       $886 = ((($885)) + 84|0);
       $887 = HEAP32[$886>>2]|0;
       $888 = HEAP32[$1>>2]|0;
       $889 = ((($888)) + 84|0);
       $890 = HEAP32[$889>>2]|0;
       _ptrmapPut($884,$887,5,$890,$rc);
       $891 = HEAP32[$rc>>2]|0;
       $892 = ($891|0)!=(0);
       if ($892) {
        break L32;
       }
      }
     }
     $893 = HEAP32[$i>>2]|0;
     $894 = (($893) + 1)|0;
     HEAP32[$i>>2] = $894;
    }
    HEAP32[$i>>2] = 0;
    L142: while(1) {
     $895 = HEAP32[$i>>2]|0;
     $896 = HEAP32[$nNew>>2]|0;
     $897 = ($895|0)<($896|0);
     if (!($897)) {
      break;
     }
     $898 = HEAP32[$i>>2]|0;
     $899 = (($apNew) + ($898<<2)|0);
     $900 = HEAP32[$899>>2]|0;
     $901 = ((($900)) + 84|0);
     $902 = HEAP32[$901>>2]|0;
     $903 = HEAP32[$i>>2]|0;
     $904 = (($aPgno) + ($903<<2)|0);
     HEAP32[$904>>2] = $902;
     $905 = HEAP32[$i>>2]|0;
     $906 = (($aPgOrder) + ($905<<2)|0);
     HEAP32[$906>>2] = $902;
     $907 = HEAP32[$i>>2]|0;
     $908 = (($apNew) + ($907<<2)|0);
     $909 = HEAP32[$908>>2]|0;
     $910 = ((($909)) + 72|0);
     $911 = HEAP32[$910>>2]|0;
     $912 = ((($911)) + 24|0);
     $913 = HEAP16[$912>>1]|0;
     $914 = HEAP32[$i>>2]|0;
     $915 = (($aPgFlags) + ($914<<1)|0);
     HEAP16[$915>>1] = $913;
     HEAP32[$j>>2] = 0;
     while(1) {
      $916 = HEAP32[$j>>2]|0;
      $917 = HEAP32[$i>>2]|0;
      $918 = ($916|0)<($917|0);
      if (!($918)) {
       break;
      }
      $919 = HEAP32[$j>>2]|0;
      $920 = (($aPgno) + ($919<<2)|0);
      $921 = HEAP32[$920>>2]|0;
      $922 = HEAP32[$i>>2]|0;
      $923 = (($aPgno) + ($922<<2)|0);
      $924 = HEAP32[$923>>2]|0;
      $925 = ($921|0)==($924|0);
      if ($925) {
       label = 108;
       break L142;
      }
      $927 = HEAP32[$j>>2]|0;
      $928 = (($927) + 1)|0;
      HEAP32[$j>>2] = $928;
     }
     $929 = HEAP32[$i>>2]|0;
     $930 = (($929) + 1)|0;
     HEAP32[$i>>2] = $930;
    }
    if ((label|0) == 108) {
     $926 = (_sqlite3CorruptError(65673)|0);
     HEAP32[$rc>>2] = $926;
     break;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $931 = HEAP32[$i>>2]|0;
     $932 = HEAP32[$nNew>>2]|0;
     $933 = ($931|0)<($932|0);
     if (!($933)) {
      break;
     }
     HEAP32[$iBest>>2] = 0;
     HEAP32[$j>>2] = 1;
     while(1) {
      $934 = HEAP32[$j>>2]|0;
      $935 = HEAP32[$nNew>>2]|0;
      $936 = ($934|0)<($935|0);
      if (!($936)) {
       break;
      }
      $937 = HEAP32[$j>>2]|0;
      $938 = (($aPgOrder) + ($937<<2)|0);
      $939 = HEAP32[$938>>2]|0;
      $940 = HEAP32[$iBest>>2]|0;
      $941 = (($aPgOrder) + ($940<<2)|0);
      $942 = HEAP32[$941>>2]|0;
      $943 = ($939>>>0)<($942>>>0);
      if ($943) {
       $944 = HEAP32[$j>>2]|0;
       HEAP32[$iBest>>2] = $944;
      }
      $945 = HEAP32[$j>>2]|0;
      $946 = (($945) + 1)|0;
      HEAP32[$j>>2] = $946;
     }
     $947 = HEAP32[$iBest>>2]|0;
     $948 = (($aPgOrder) + ($947<<2)|0);
     $949 = HEAP32[$948>>2]|0;
     HEAP32[$pgno>>2] = $949;
     $950 = HEAP32[$iBest>>2]|0;
     $951 = (($aPgOrder) + ($950<<2)|0);
     HEAP32[$951>>2] = -1;
     $952 = HEAP32[$iBest>>2]|0;
     $953 = HEAP32[$i>>2]|0;
     $954 = ($952|0)!=($953|0);
     if ($954) {
      $955 = HEAP32[$iBest>>2]|0;
      $956 = HEAP32[$i>>2]|0;
      $957 = ($955|0)>($956|0);
      if ($957) {
       $958 = HEAP32[$iBest>>2]|0;
       $959 = (($apNew) + ($958<<2)|0);
       $960 = HEAP32[$959>>2]|0;
       $961 = ((($960)) + 72|0);
       $962 = HEAP32[$961>>2]|0;
       $963 = HEAP32[$pBt>>2]|0;
       $964 = ((($963)) + 44|0);
       $965 = HEAP32[$964>>2]|0;
       $966 = HEAP32[$iBest>>2]|0;
       $967 = (($965) + ($966))|0;
       $968 = (($967) + 1)|0;
       _sqlite3PagerRekey($962,$968,0);
      }
      $969 = HEAP32[$i>>2]|0;
      $970 = (($apNew) + ($969<<2)|0);
      $971 = HEAP32[$970>>2]|0;
      $972 = ((($971)) + 72|0);
      $973 = HEAP32[$972>>2]|0;
      $974 = HEAP32[$pgno>>2]|0;
      $975 = HEAP32[$iBest>>2]|0;
      $976 = (($aPgFlags) + ($975<<1)|0);
      $977 = HEAP16[$976>>1]|0;
      _sqlite3PagerRekey($973,$974,$977);
      $978 = HEAP32[$pgno>>2]|0;
      $979 = HEAP32[$i>>2]|0;
      $980 = (($apNew) + ($979<<2)|0);
      $981 = HEAP32[$980>>2]|0;
      $982 = ((($981)) + 84|0);
      HEAP32[$982>>2] = $978;
     }
     $983 = HEAP32[$i>>2]|0;
     $984 = (($983) + 1)|0;
     HEAP32[$i>>2] = $984;
    }
    $985 = HEAP32[$pRight>>2]|0;
    $986 = HEAP32[$nNew>>2]|0;
    $987 = (($986) - 1)|0;
    $988 = (($apNew) + ($987<<2)|0);
    $989 = HEAP32[$988>>2]|0;
    $990 = ((($989)) + 84|0);
    $991 = HEAP32[$990>>2]|0;
    _sqlite3Put4byte($985,$991);
    $992 = HEAP32[$pageFlags>>2]|0;
    $993 = $992 & 8;
    $994 = ($993|0)==(0);
    if ($994) {
     $995 = HEAP32[$nOld>>2]|0;
     $996 = HEAP32[$nNew>>2]|0;
     $997 = ($995|0)!=($996|0);
     if ($997) {
      $998 = HEAP32[$nOld>>2]|0;
      $999 = (($998) - 1)|0;
      $1000 = HEAP32[$nNew>>2]|0;
      $1001 = HEAP32[$nOld>>2]|0;
      $1002 = ($1000|0)>($1001|0);
      $1003 = $1002 ? $apNew : $apOld;
      $1004 = (($1003) + ($999<<2)|0);
      $1005 = HEAP32[$1004>>2]|0;
      HEAP32[$pOld2>>2] = $1005;
      $1006 = HEAP32[$nNew>>2]|0;
      $1007 = (($1006) - 1)|0;
      $1008 = (($apNew) + ($1007<<2)|0);
      $1009 = HEAP32[$1008>>2]|0;
      $1010 = ((($1009)) + 56|0);
      $1011 = HEAP32[$1010>>2]|0;
      $1012 = ((($1011)) + 8|0);
      $1013 = HEAP32[$pOld2>>2]|0;
      $1014 = ((($1013)) + 56|0);
      $1015 = HEAP32[$1014>>2]|0;
      $1016 = ((($1015)) + 8|0);
      ;HEAP8[$1012>>0]=HEAP8[$1016>>0]|0;HEAP8[$1012+1>>0]=HEAP8[$1016+1>>0]|0;HEAP8[$1012+2>>0]=HEAP8[$1016+2>>0]|0;HEAP8[$1012+3>>0]=HEAP8[$1016+3>>0]|0;
     }
    }
    $1017 = HEAP32[$pBt>>2]|0;
    $1018 = ((($1017)) + 17|0);
    $1019 = HEAP8[$1018>>0]|0;
    $1020 = ($1019<<24>>24)!=(0);
    L174: do {
     if ($1020) {
      $1021 = HEAP32[$apNew>>2]|0;
      HEAP32[$pNew3>>2] = $1021;
      $1022 = HEAP32[$pNew3>>2]|0;
      $1023 = ((($1022)) + 56|0);
      $1024 = HEAP32[$1023>>2]|0;
      HEAP32[$aOld>>2] = $1024;
      $1025 = HEAP32[$pNew3>>2]|0;
      $1026 = ((($1025)) + 18|0);
      $1027 = HEAP16[$1026>>1]|0;
      $1028 = $1027&65535;
      $1029 = HEAP32[$pNew3>>2]|0;
      $1030 = ((($1029)) + 1|0);
      $1031 = HEAP8[$1030>>0]|0;
      $1032 = $1031&255;
      $1033 = (($1028) + ($1032))|0;
      HEAP32[$cntOldNext>>2] = $1033;
      $1034 = HEAP32[$pBt>>2]|0;
      $1035 = ((($1034)) + 36|0);
      $1036 = HEAP32[$1035>>2]|0;
      HEAP32[$usableSize>>2] = $1036;
      HEAP32[$iNew>>2] = 0;
      HEAP32[$iOld>>2] = 0;
      HEAP32[$i>>2] = 0;
      while(1) {
       $1037 = HEAP32[$i>>2]|0;
       $1038 = HEAP32[$b>>2]|0;
       $1039 = ($1037|0)<($1038|0);
       if (!($1039)) {
        break L174;
       }
       $1040 = HEAP32[$i>>2]|0;
       $1041 = ((($b)) + 8|0);
       $1042 = HEAP32[$1041>>2]|0;
       $1043 = (($1042) + ($1040<<2)|0);
       $1044 = HEAP32[$1043>>2]|0;
       HEAP32[$pCell>>2] = $1044;
       $1045 = HEAP32[$i>>2]|0;
       $1046 = HEAP32[$cntOldNext>>2]|0;
       $1047 = ($1045|0)==($1046|0);
       if ($1047) {
        $1048 = HEAP32[$iOld>>2]|0;
        $1049 = (($1048) + 1)|0;
        HEAP32[$iOld>>2] = $1049;
        $1050 = HEAP32[$nNew>>2]|0;
        $1051 = ($1049|0)<($1050|0);
        $1052 = HEAP32[$iOld>>2]|0;
        if ($1051) {
         $1053 = (($apNew) + ($1052<<2)|0);
         $1054 = HEAP32[$1053>>2]|0;
         $1057 = $1054;
        } else {
         $1055 = (($apOld) + ($1052<<2)|0);
         $1056 = HEAP32[$1055>>2]|0;
         $1057 = $1056;
        }
        HEAP32[$pOld4>>2] = $1057;
        $1058 = HEAP32[$pOld4>>2]|0;
        $1059 = ((($1058)) + 18|0);
        $1060 = HEAP16[$1059>>1]|0;
        $1061 = $1060&65535;
        $1062 = HEAP32[$pOld4>>2]|0;
        $1063 = ((($1062)) + 1|0);
        $1064 = HEAP8[$1063>>0]|0;
        $1065 = $1064&255;
        $1066 = (($1061) + ($1065))|0;
        $1067 = HEAP32[$leafData>>2]|0;
        $1068 = ($1067|0)!=(0);
        $1069 = $1068 ^ 1;
        $1070 = $1069&1;
        $1071 = (($1066) + ($1070))|0;
        $1072 = HEAP32[$cntOldNext>>2]|0;
        $1073 = (($1072) + ($1071))|0;
        HEAP32[$cntOldNext>>2] = $1073;
        $1074 = HEAP32[$pOld4>>2]|0;
        $1075 = ((($1074)) + 56|0);
        $1076 = HEAP32[$1075>>2]|0;
        HEAP32[$aOld>>2] = $1076;
       }
       $1077 = HEAP32[$i>>2]|0;
       $1078 = HEAP32[$iNew>>2]|0;
       $1079 = (($cntNew) + ($1078<<2)|0);
       $1080 = HEAP32[$1079>>2]|0;
       $1081 = ($1077|0)==($1080|0);
       if ($1081) {
        $1082 = HEAP32[$iNew>>2]|0;
        $1083 = (($1082) + 1)|0;
        HEAP32[$iNew>>2] = $1083;
        $1084 = (($apNew) + ($1083<<2)|0);
        $1085 = HEAP32[$1084>>2]|0;
        HEAP32[$pNew3>>2] = $1085;
        $1086 = HEAP32[$leafData>>2]|0;
        $1087 = ($1086|0)!=(0);
        if ($1087) {
         label = 136;
        }
       } else {
        label = 136;
       }
       do {
        if ((label|0) == 136) {
         label = 0;
         $1088 = HEAP32[$iOld>>2]|0;
         $1089 = HEAP32[$nNew>>2]|0;
         $1090 = ($1088|0)>=($1089|0);
         if (!($1090)) {
          $1091 = HEAP32[$pNew3>>2]|0;
          $1092 = ((($1091)) + 84|0);
          $1093 = HEAP32[$1092>>2]|0;
          $1094 = HEAP32[$iOld>>2]|0;
          $1095 = (($aPgno) + ($1094<<2)|0);
          $1096 = HEAP32[$1095>>2]|0;
          $1097 = ($1093|0)!=($1096|0);
          if (!($1097)) {
           $1098 = HEAP32[$pCell>>2]|0;
           $1099 = $1098;
           $1100 = HEAP32[$aOld>>2]|0;
           $1101 = $1100;
           $1102 = ($1099>>>0)>=($1101>>>0);
           if ($1102) {
            $1103 = HEAP32[$pCell>>2]|0;
            $1104 = $1103;
            $1105 = HEAP32[$usableSize>>2]|0;
            $1106 = HEAP32[$aOld>>2]|0;
            $1107 = (($1106) + ($1105)|0);
            $1108 = $1107;
            $1109 = ($1104>>>0)<($1108>>>0);
            if ($1109) {
             break;
            }
           }
          }
         }
         $1110 = HEAP16[$leafCorrection>>1]|0;
         $1111 = ($1110<<16>>16)!=(0);
         if (!($1111)) {
          $1112 = HEAP32[$pBt>>2]|0;
          $1113 = HEAP32[$pCell>>2]|0;
          $1114 = (_sqlite3Get4byte($1113)|0);
          $1115 = HEAP32[$pNew3>>2]|0;
          $1116 = ((($1115)) + 84|0);
          $1117 = HEAP32[$1116>>2]|0;
          _ptrmapPut($1112,$1114,5,$1117,$rc);
         }
         $1118 = HEAP32[$i>>2]|0;
         $1119 = (_cachedCellSize($b,$1118)|0);
         $1120 = $1119&65535;
         $1121 = HEAP32[$pNew3>>2]|0;
         $1122 = ((($1121)) + 12|0);
         $1123 = HEAP16[$1122>>1]|0;
         $1124 = $1123&65535;
         $1125 = ($1120|0)>($1124|0);
         if ($1125) {
          $1126 = HEAP32[$pNew3>>2]|0;
          $1127 = HEAP32[$pCell>>2]|0;
          _ptrmapPutOvflPtr($1126,$1127,$rc);
         }
         $1128 = HEAP32[$rc>>2]|0;
         $1129 = ($1128|0)!=(0);
         if ($1129) {
          break L32;
         }
        }
       } while(0);
       $1130 = HEAP32[$i>>2]|0;
       $1131 = (($1130) + 1)|0;
       HEAP32[$i>>2] = $1131;
      }
     }
    } while(0);
    HEAP32[$i>>2] = 0;
    while(1) {
     $1132 = HEAP32[$i>>2]|0;
     $1133 = HEAP32[$nNew>>2]|0;
     $1134 = (($1133) - 1)|0;
     $1135 = ($1132|0)<($1134|0);
     if (!($1135)) {
      break;
     }
     $1136 = HEAP32[$i>>2]|0;
     $1137 = (($apNew) + ($1136<<2)|0);
     $1138 = HEAP32[$1137>>2]|0;
     HEAP32[$pNew8>>2] = $1138;
     $1139 = HEAP32[$i>>2]|0;
     $1140 = (($cntNew) + ($1139<<2)|0);
     $1141 = HEAP32[$1140>>2]|0;
     HEAP32[$j>>2] = $1141;
     $1142 = HEAP32[$j>>2]|0;
     $1143 = ((($b)) + 8|0);
     $1144 = HEAP32[$1143>>2]|0;
     $1145 = (($1144) + ($1142<<2)|0);
     $1146 = HEAP32[$1145>>2]|0;
     HEAP32[$pCell5>>2] = $1146;
     $1147 = HEAP32[$j>>2]|0;
     $1148 = ((($b)) + 12|0);
     $1149 = HEAP32[$1148>>2]|0;
     $1150 = (($1149) + ($1147<<1)|0);
     $1151 = HEAP16[$1150>>1]|0;
     $1152 = $1151&65535;
     $1153 = HEAP16[$leafCorrection>>1]|0;
     $1154 = $1153&65535;
     $1155 = (($1152) + ($1154))|0;
     HEAP32[$sz7>>2] = $1155;
     $1156 = HEAP32[$iOvflSpace>>2]|0;
     $1157 = HEAP32[$3>>2]|0;
     $1158 = (($1157) + ($1156)|0);
     HEAP32[$pTemp6>>2] = $1158;
     $1159 = HEAP32[$pNew8>>2]|0;
     $1160 = ((($1159)) + 4|0);
     $1161 = HEAP8[$1160>>0]|0;
     $1162 = ($1161<<24>>24)!=(0);
     do {
      if ($1162) {
       $1168 = HEAP32[$leafData>>2]|0;
       $1169 = ($1168|0)!=(0);
       if ($1169) {
        $1170 = HEAP32[$j>>2]|0;
        $1171 = (($1170) + -1)|0;
        HEAP32[$j>>2] = $1171;
        $1172 = HEAP32[$pNew8>>2]|0;
        $1173 = ((($1172)) + 80|0);
        $1174 = HEAP32[$1173>>2]|0;
        $1175 = HEAP32[$pNew8>>2]|0;
        $1176 = HEAP32[$j>>2]|0;
        $1177 = ((($b)) + 8|0);
        $1178 = HEAP32[$1177>>2]|0;
        $1179 = (($1178) + ($1176<<2)|0);
        $1180 = HEAP32[$1179>>2]|0;
        FUNCTION_TABLE_viii[$1174 & 255]($1175,$1180,$info);
        $1181 = HEAP32[$pTemp6>>2]|0;
        HEAP32[$pCell5>>2] = $1181;
        $1182 = HEAP32[$pCell5>>2]|0;
        $1183 = ((($1182)) + 4|0);
        $1184 = $info;
        $1185 = $1184;
        $1186 = HEAP32[$1185>>2]|0;
        $1187 = (($1184) + 4)|0;
        $1188 = $1187;
        $1189 = HEAP32[$1188>>2]|0;
        $1190 = (_sqlite3PutVarint($1183,$1186,$1189)|0);
        $1191 = (4 + ($1190))|0;
        HEAP32[$sz7>>2] = $1191;
        HEAP32[$pTemp6>>2] = 0;
        break;
       }
       $1192 = HEAP32[$pCell5>>2]|0;
       $1193 = ((($1192)) + -4|0);
       HEAP32[$pCell5>>2] = $1193;
       $1194 = HEAP32[$j>>2]|0;
       $1195 = ((($b)) + 12|0);
       $1196 = HEAP32[$1195>>2]|0;
       $1197 = (($1196) + ($1194<<1)|0);
       $1198 = HEAP16[$1197>>1]|0;
       $1199 = $1198&65535;
       $1200 = ($1199|0)==(4);
       if ($1200) {
        $1201 = HEAP32[$1>>2]|0;
        $1202 = ((($1201)) + 76|0);
        $1203 = HEAP32[$1202>>2]|0;
        $1204 = HEAP32[$1>>2]|0;
        $1205 = HEAP32[$pCell5>>2]|0;
        $1206 = (FUNCTION_TABLE_iii[$1203 & 255]($1204,$1205)|0);
        $1207 = $1206&65535;
        HEAP32[$sz7>>2] = $1207;
       }
      } else {
       $1163 = HEAP32[$pNew8>>2]|0;
       $1164 = ((($1163)) + 56|0);
       $1165 = HEAP32[$1164>>2]|0;
       $1166 = ((($1165)) + 8|0);
       $1167 = HEAP32[$pCell5>>2]|0;
       ;HEAP8[$1166>>0]=HEAP8[$1167>>0]|0;HEAP8[$1166+1>>0]=HEAP8[$1167+1>>0]|0;HEAP8[$1166+2>>0]=HEAP8[$1167+2>>0]|0;HEAP8[$1166+3>>0]=HEAP8[$1167+3>>0]|0;
      }
     } while(0);
     $1208 = HEAP32[$sz7>>2]|0;
     $1209 = HEAP32[$iOvflSpace>>2]|0;
     $1210 = (($1209) + ($1208))|0;
     HEAP32[$iOvflSpace>>2] = $1210;
     $1211 = HEAP32[$1>>2]|0;
     $1212 = HEAP32[$nxDiv>>2]|0;
     $1213 = HEAP32[$i>>2]|0;
     $1214 = (($1212) + ($1213))|0;
     $1215 = HEAP32[$pCell5>>2]|0;
     $1216 = HEAP32[$sz7>>2]|0;
     $1217 = HEAP32[$pTemp6>>2]|0;
     $1218 = HEAP32[$pNew8>>2]|0;
     $1219 = ((($1218)) + 84|0);
     $1220 = HEAP32[$1219>>2]|0;
     _insertCell($1211,$1214,$1215,$1216,$1217,$1220,$rc);
     $1221 = HEAP32[$rc>>2]|0;
     $1222 = ($1221|0)!=(0);
     if ($1222) {
      break L32;
     }
     $1223 = HEAP32[$i>>2]|0;
     $1224 = (($1223) + 1)|0;
     HEAP32[$i>>2] = $1224;
    }
    $1225 = HEAP32[$nNew>>2]|0;
    $1226 = (1 - ($1225))|0;
    HEAP32[$i>>2] = $1226;
    while(1) {
     $1227 = HEAP32[$i>>2]|0;
     $1228 = HEAP32[$nNew>>2]|0;
     $1229 = ($1227|0)<($1228|0);
     if (!($1229)) {
      break;
     }
     $1230 = HEAP32[$i>>2]|0;
     $1231 = ($1230|0)<(0);
     $1232 = HEAP32[$i>>2]|0;
     $1233 = (0 - ($1232))|0;
     $1234 = $1231 ? $1233 : $1232;
     HEAP32[$iPg>>2] = $1234;
     $1235 = HEAP32[$iPg>>2]|0;
     $1236 = (($abDone) + ($1235)|0);
     $1237 = HEAP8[$1236>>0]|0;
     $1238 = ($1237<<24>>24)!=(0);
     do {
      if (!($1238)) {
       $1239 = HEAP32[$i>>2]|0;
       $1240 = ($1239|0)>=(0);
       if (!($1240)) {
        $1241 = HEAP32[$iPg>>2]|0;
        $1242 = (($1241) - 1)|0;
        $1243 = (($cntOld) + ($1242<<2)|0);
        $1244 = HEAP32[$1243>>2]|0;
        $1245 = HEAP32[$iPg>>2]|0;
        $1246 = (($1245) - 1)|0;
        $1247 = (($cntNew) + ($1246<<2)|0);
        $1248 = HEAP32[$1247>>2]|0;
        $1249 = ($1244|0)>=($1248|0);
        if (!($1249)) {
         break;
        }
       }
       $1250 = HEAP32[$iPg>>2]|0;
       $1251 = ($1250|0)==(0);
       if ($1251) {
        HEAP32[$iOld10>>2] = 0;
        HEAP32[$iNew9>>2] = 0;
        $1252 = HEAP32[$cntNew>>2]|0;
        HEAP32[$nNewCell>>2] = $1252;
       } else {
        $1253 = HEAP32[$iPg>>2]|0;
        $1254 = HEAP32[$nOld>>2]|0;
        $1255 = ($1253|0)<($1254|0);
        if ($1255) {
         $1256 = HEAP32[$iPg>>2]|0;
         $1257 = (($1256) - 1)|0;
         $1258 = (($cntOld) + ($1257<<2)|0);
         $1259 = HEAP32[$1258>>2]|0;
         $1260 = HEAP32[$leafData>>2]|0;
         $1261 = ($1260|0)!=(0);
         $1262 = $1261 ^ 1;
         $1263 = $1262&1;
         $1264 = (($1259) + ($1263))|0;
         $1266 = $1264;
        } else {
         $1265 = HEAP32[$b>>2]|0;
         $1266 = $1265;
        }
        HEAP32[$iOld10>>2] = $1266;
        $1267 = HEAP32[$iPg>>2]|0;
        $1268 = (($1267) - 1)|0;
        $1269 = (($cntNew) + ($1268<<2)|0);
        $1270 = HEAP32[$1269>>2]|0;
        $1271 = HEAP32[$leafData>>2]|0;
        $1272 = ($1271|0)!=(0);
        $1273 = $1272 ^ 1;
        $1274 = $1273&1;
        $1275 = (($1270) + ($1274))|0;
        HEAP32[$iNew9>>2] = $1275;
        $1276 = HEAP32[$iPg>>2]|0;
        $1277 = (($cntNew) + ($1276<<2)|0);
        $1278 = HEAP32[$1277>>2]|0;
        $1279 = HEAP32[$iNew9>>2]|0;
        $1280 = (($1278) - ($1279))|0;
        HEAP32[$nNewCell>>2] = $1280;
       }
       $1281 = HEAP32[$iPg>>2]|0;
       $1282 = (($apNew) + ($1281<<2)|0);
       $1283 = HEAP32[$1282>>2]|0;
       $1284 = HEAP32[$iOld10>>2]|0;
       $1285 = HEAP32[$iNew9>>2]|0;
       $1286 = HEAP32[$nNewCell>>2]|0;
       $1287 = (_editPage($1283,$1284,$1285,$1286,$b)|0);
       HEAP32[$rc>>2] = $1287;
       $1288 = HEAP32[$rc>>2]|0;
       $1289 = ($1288|0)!=(0);
       if ($1289) {
        break L32;
       }
       $1290 = HEAP32[$iPg>>2]|0;
       $1291 = (($abDone) + ($1290)|0);
       $1292 = HEAP8[$1291>>0]|0;
       $1293 = (($1292) + 1)<<24>>24;
       HEAP8[$1291>>0] = $1293;
       $1294 = HEAP32[$usableSpace>>2]|0;
       $1295 = HEAP32[$iPg>>2]|0;
       $1296 = (($szNew) + ($1295<<2)|0);
       $1297 = HEAP32[$1296>>2]|0;
       $1298 = (($1294) - ($1297))|0;
       $1299 = $1298&65535;
       $1300 = HEAP32[$iPg>>2]|0;
       $1301 = (($apNew) + ($1300<<2)|0);
       $1302 = HEAP32[$1301>>2]|0;
       $1303 = ((($1302)) + 16|0);
       HEAP16[$1303>>1] = $1299;
      }
     } while(0);
     $1304 = HEAP32[$i>>2]|0;
     $1305 = (($1304) + 1)|0;
     HEAP32[$i>>2] = $1305;
    }
    $1306 = HEAP32[$4>>2]|0;
    $1307 = ($1306|0)!=(0);
    if ($1307) {
     $1308 = HEAP32[$1>>2]|0;
     $1309 = ((($1308)) + 18|0);
     $1310 = HEAP16[$1309>>1]|0;
     $1311 = $1310&65535;
     $1312 = ($1311|0)==(0);
     if ($1312) {
      $1313 = HEAP32[$1>>2]|0;
      $1314 = ((($1313)) + 5|0);
      $1315 = HEAP8[$1314>>0]|0;
      $1316 = $1315&255;
      $1317 = HEAP32[$apNew>>2]|0;
      $1318 = ((($1317)) + 16|0);
      $1319 = HEAP16[$1318>>1]|0;
      $1320 = $1319&65535;
      $1321 = ($1316|0)<=($1320|0);
      if ($1321) {
       $1322 = HEAP32[$apNew>>2]|0;
       $1323 = (_defragmentPage($1322)|0);
       HEAP32[$rc>>2] = $1323;
       $1324 = HEAP32[$apNew>>2]|0;
       $1325 = HEAP32[$1>>2]|0;
       _copyNodeContent($1324,$1325,$rc);
       $1326 = HEAP32[$apNew>>2]|0;
       _freePage($1326,$rc);
      } else {
       label = 174;
      }
     } else {
      label = 174;
     }
    } else {
     label = 174;
    }
    L239: do {
     if ((label|0) == 174) {
      $1327 = HEAP32[$pBt>>2]|0;
      $1328 = ((($1327)) + 17|0);
      $1329 = HEAP8[$1328>>0]|0;
      $1330 = $1329&255;
      $1331 = ($1330|0)==(0);
      $1332 = HEAP16[$leafCorrection>>1]|0;
      $1333 = ($1332<<16>>16)!=(0);
      $or$cond3 = $1331 | $1333;
      if (!($or$cond3)) {
       HEAP32[$i>>2] = 0;
       while(1) {
        $1334 = HEAP32[$i>>2]|0;
        $1335 = HEAP32[$nNew>>2]|0;
        $1336 = ($1334|0)<($1335|0);
        if (!($1336)) {
         break L239;
        }
        $1337 = HEAP32[$i>>2]|0;
        $1338 = (($apNew) + ($1337<<2)|0);
        $1339 = HEAP32[$1338>>2]|0;
        $1340 = ((($1339)) + 56|0);
        $1341 = HEAP32[$1340>>2]|0;
        $1342 = ((($1341)) + 8|0);
        $1343 = (_sqlite3Get4byte($1342)|0);
        HEAP32[$key>>2] = $1343;
        $1344 = HEAP32[$pBt>>2]|0;
        $1345 = HEAP32[$key>>2]|0;
        $1346 = HEAP32[$i>>2]|0;
        $1347 = (($apNew) + ($1346<<2)|0);
        $1348 = HEAP32[$1347>>2]|0;
        $1349 = ((($1348)) + 84|0);
        $1350 = HEAP32[$1349>>2]|0;
        _ptrmapPut($1344,$1345,5,$1350,$rc);
        $1351 = HEAP32[$i>>2]|0;
        $1352 = (($1351) + 1)|0;
        HEAP32[$i>>2] = $1352;
       }
      }
     }
    } while(0);
    $1353 = HEAP32[$nNew>>2]|0;
    HEAP32[$i>>2] = $1353;
    while(1) {
     $1354 = HEAP32[$i>>2]|0;
     $1355 = HEAP32[$nOld>>2]|0;
     $1356 = ($1354|0)<($1355|0);
     if (!($1356)) {
      break L32;
     }
     $1357 = HEAP32[$i>>2]|0;
     $1358 = (($apOld) + ($1357<<2)|0);
     $1359 = HEAP32[$1358>>2]|0;
     _freePage($1359,$rc);
     $1360 = HEAP32[$i>>2]|0;
     $1361 = (($1360) + 1)|0;
     HEAP32[$i>>2] = $1361;
    }
   }
  }
 } while(0);
 $1362 = ((($b)) + 8|0);
 $1363 = HEAP32[$1362>>2]|0;
 _sqlite3ScratchFree($1363);
 HEAP32[$i>>2] = 0;
 while(1) {
  $1364 = HEAP32[$i>>2]|0;
  $1365 = HEAP32[$nOld>>2]|0;
  $1366 = ($1364|0)<($1365|0);
  if (!($1366)) {
   break;
  }
  $1367 = HEAP32[$i>>2]|0;
  $1368 = (($apOld) + ($1367<<2)|0);
  $1369 = HEAP32[$1368>>2]|0;
  _releasePage($1369);
  $1370 = HEAP32[$i>>2]|0;
  $1371 = (($1370) + 1)|0;
  HEAP32[$i>>2] = $1371;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $1372 = HEAP32[$i>>2]|0;
  $1373 = HEAP32[$nNew>>2]|0;
  $1374 = ($1372|0)<($1373|0);
  if (!($1374)) {
   break;
  }
  $1375 = HEAP32[$i>>2]|0;
  $1376 = (($apNew) + ($1375<<2)|0);
  $1377 = HEAP32[$1376>>2]|0;
  _releasePage($1377);
  $1378 = HEAP32[$i>>2]|0;
  $1379 = (($1378) + 1)|0;
  HEAP32[$i>>2] = $1379;
 }
 $1380 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $1380;
 $1381 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($1381|0);
}
function _cachedCellSize($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $N;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (($6) + ($3<<1)|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = ($8<<16>>16)!=(0);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($10<<1)|0);
  $15 = HEAP16[$14>>1]|0;
  HEAP16[$0>>1] = $15;
  $19 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($19|0);
 } else {
  $16 = HEAP32[$1>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (_computeCellSize($16,$17)|0);
  HEAP16[$0>>1] = $18;
  $19 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($19|0);
 }
 return (0)|0;
}
function _sqlite3PagerRekey($pPg,$iNew,$flags) {
 $pPg = $pPg|0;
 $iNew = $iNew|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$1>>2] = $iNew;
 HEAP16[$2>>1] = $flags;
 $3 = HEAP16[$2>>1]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 24|0);
 HEAP16[$5>>1] = $3;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$1>>2]|0;
 _sqlite3PcacheMove($6,$7);
 STACKTOP = sp;return;
}
function _editPage($pPg,$iOld,$iNew,$nNew,$pCArray) {
 $pPg = $pPg|0;
 $iOld = $iOld|0;
 $iNew = $iNew|0;
 $nNew = $nNew|0;
 $pCArray = $pCArray|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0;
 var $hdr = 0, $i = 0, $iCell = 0, $iNewEnd = 0, $iOldEnd = 0, $nAdd = 0, $nCell = 0, $nShift = 0, $pBegin = 0, $pCellptr = 0, $pData = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $3 = sp + 56|0;
 $4 = sp + 52|0;
 $5 = sp + 48|0;
 $aData = sp + 44|0;
 $hdr = sp + 40|0;
 $pBegin = sp + 36|0;
 $nCell = sp + 32|0;
 $pData = sp + 28|0;
 $pCellptr = sp + 24|0;
 $i = sp + 20|0;
 $iOldEnd = sp + 16|0;
 $iNewEnd = sp + 12|0;
 $nShift = sp + 8|0;
 $nAdd = sp + 4|0;
 $iCell = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $iOld;
 HEAP32[$3>>2] = $iNew;
 HEAP32[$4>>2] = $nNew;
 HEAP32[$5>>2] = $pCArray;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 56|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$aData>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 5|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = $11&255;
 HEAP32[$hdr>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 $14 = $13<<1;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 64|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = (($17) + ($14)|0);
 HEAP32[$pBegin>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 18|0);
 $21 = HEAP16[$20>>1]|0;
 $22 = $21&65535;
 HEAP32[$nCell>>2] = $22;
 $23 = HEAP32[$2>>2]|0;
 $24 = HEAP32[$1>>2]|0;
 $25 = ((($24)) + 18|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = (($23) + ($27))|0;
 $29 = HEAP32[$1>>2]|0;
 $30 = ((($29)) + 1|0);
 $31 = HEAP8[$30>>0]|0;
 $32 = $31&255;
 $33 = (($28) + ($32))|0;
 HEAP32[$iOldEnd>>2] = $33;
 $34 = HEAP32[$3>>2]|0;
 $35 = HEAP32[$4>>2]|0;
 $36 = (($34) + ($35))|0;
 HEAP32[$iNewEnd>>2] = $36;
 $37 = HEAP32[$2>>2]|0;
 $38 = HEAP32[$3>>2]|0;
 $39 = ($37|0)<($38|0);
 if ($39) {
  $40 = HEAP32[$1>>2]|0;
  $41 = HEAP32[$2>>2]|0;
  $42 = HEAP32[$3>>2]|0;
  $43 = HEAP32[$2>>2]|0;
  $44 = (($42) - ($43))|0;
  $45 = HEAP32[$5>>2]|0;
  $46 = (_pageFreeArray($40,$41,$44,$45)|0);
  HEAP32[$nShift>>2] = $46;
  $47 = HEAP32[$1>>2]|0;
  $48 = ((($47)) + 64|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$nShift>>2]|0;
  $51 = $50<<1;
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 64|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51)|0);
  $56 = HEAP32[$nCell>>2]|0;
  $57 = $56<<1;
  _memmove(($49|0),($55|0),($57|0))|0;
  $58 = HEAP32[$nShift>>2]|0;
  $59 = HEAP32[$nCell>>2]|0;
  $60 = (($59) - ($58))|0;
  HEAP32[$nCell>>2] = $60;
 }
 $61 = HEAP32[$iNewEnd>>2]|0;
 $62 = HEAP32[$iOldEnd>>2]|0;
 $63 = ($61|0)<($62|0);
 if ($63) {
  $64 = HEAP32[$1>>2]|0;
  $65 = HEAP32[$iNewEnd>>2]|0;
  $66 = HEAP32[$iOldEnd>>2]|0;
  $67 = HEAP32[$iNewEnd>>2]|0;
  $68 = (($66) - ($67))|0;
  $69 = HEAP32[$5>>2]|0;
  $70 = (_pageFreeArray($64,$65,$68,$69)|0);
  $71 = HEAP32[$nCell>>2]|0;
  $72 = (($71) - ($70))|0;
  HEAP32[$nCell>>2] = $72;
 }
 $73 = HEAP32[$hdr>>2]|0;
 $74 = (($73) + 5)|0;
 $75 = HEAP32[$aData>>2]|0;
 $76 = (($75) + ($74)|0);
 $77 = HEAP8[$76>>0]|0;
 $78 = $77&255;
 $79 = $78 << 8;
 $80 = HEAP32[$hdr>>2]|0;
 $81 = (($80) + 5)|0;
 $82 = HEAP32[$aData>>2]|0;
 $83 = (($82) + ($81)|0);
 $84 = ((($83)) + 1|0);
 $85 = HEAP8[$84>>0]|0;
 $86 = $85&255;
 $87 = $79 | $86;
 $88 = (($87) - 1)|0;
 $89 = $88 & 65535;
 $90 = (($89) + 1)|0;
 $91 = HEAP32[$aData>>2]|0;
 $92 = (($91) + ($90)|0);
 HEAP32[$pData>>2] = $92;
 $93 = HEAP32[$pData>>2]|0;
 $94 = HEAP32[$pBegin>>2]|0;
 $95 = ($93>>>0)<($94>>>0);
 L7: do {
  if (!($95)) {
   $96 = HEAP32[$3>>2]|0;
   $97 = HEAP32[$2>>2]|0;
   $98 = ($96|0)<($97|0);
   if ($98) {
    $99 = HEAP32[$4>>2]|0;
    $100 = HEAP32[$2>>2]|0;
    $101 = HEAP32[$3>>2]|0;
    $102 = (($100) - ($101))|0;
    $103 = ($99|0)<($102|0);
    if ($103) {
     $104 = HEAP32[$4>>2]|0;
     $108 = $104;
    } else {
     $105 = HEAP32[$2>>2]|0;
     $106 = HEAP32[$3>>2]|0;
     $107 = (($105) - ($106))|0;
     $108 = $107;
    }
    HEAP32[$nAdd>>2] = $108;
    $109 = HEAP32[$1>>2]|0;
    $110 = ((($109)) + 64|0);
    $111 = HEAP32[$110>>2]|0;
    HEAP32[$pCellptr>>2] = $111;
    $112 = HEAP32[$nAdd>>2]|0;
    $113 = $112<<1;
    $114 = HEAP32[$pCellptr>>2]|0;
    $115 = (($114) + ($113)|0);
    $116 = HEAP32[$pCellptr>>2]|0;
    $117 = HEAP32[$nCell>>2]|0;
    $118 = $117<<1;
    _memmove(($115|0),($116|0),($118|0))|0;
    $119 = HEAP32[$1>>2]|0;
    $120 = HEAP32[$pBegin>>2]|0;
    $121 = HEAP32[$pCellptr>>2]|0;
    $122 = HEAP32[$3>>2]|0;
    $123 = HEAP32[$nAdd>>2]|0;
    $124 = HEAP32[$5>>2]|0;
    $125 = (_pageInsertArray($119,$120,$pData,$121,$122,$123,$124)|0);
    $126 = ($125|0)!=(0);
    if ($126) {
     break;
    }
    $127 = HEAP32[$nAdd>>2]|0;
    $128 = HEAP32[$nCell>>2]|0;
    $129 = (($128) + ($127))|0;
    HEAP32[$nCell>>2] = $129;
   }
   HEAP32[$i>>2] = 0;
   while(1) {
    $130 = HEAP32[$i>>2]|0;
    $131 = HEAP32[$1>>2]|0;
    $132 = ((($131)) + 1|0);
    $133 = HEAP8[$132>>0]|0;
    $134 = $133&255;
    $135 = ($130|0)<($134|0);
    if (!($135)) {
     break;
    }
    $136 = HEAP32[$2>>2]|0;
    $137 = HEAP32[$i>>2]|0;
    $138 = HEAP32[$1>>2]|0;
    $139 = ((($138)) + 22|0);
    $140 = (($139) + ($137<<1)|0);
    $141 = HEAP16[$140>>1]|0;
    $142 = $141&65535;
    $143 = (($136) + ($142))|0;
    $144 = HEAP32[$3>>2]|0;
    $145 = (($143) - ($144))|0;
    HEAP32[$iCell>>2] = $145;
    $146 = HEAP32[$iCell>>2]|0;
    $147 = ($146|0)>=(0);
    if ($147) {
     $148 = HEAP32[$iCell>>2]|0;
     $149 = HEAP32[$4>>2]|0;
     $150 = ($148|0)<($149|0);
     if ($150) {
      $151 = HEAP32[$iCell>>2]|0;
      $152 = $151<<1;
      $153 = HEAP32[$1>>2]|0;
      $154 = ((($153)) + 64|0);
      $155 = HEAP32[$154>>2]|0;
      $156 = (($155) + ($152)|0);
      HEAP32[$pCellptr>>2] = $156;
      $157 = HEAP32[$pCellptr>>2]|0;
      $158 = ((($157)) + 2|0);
      $159 = HEAP32[$pCellptr>>2]|0;
      $160 = HEAP32[$nCell>>2]|0;
      $161 = HEAP32[$iCell>>2]|0;
      $162 = (($160) - ($161))|0;
      $163 = $162<<1;
      _memmove(($158|0),($159|0),($163|0))|0;
      $164 = HEAP32[$nCell>>2]|0;
      $165 = (($164) + 1)|0;
      HEAP32[$nCell>>2] = $165;
      $166 = HEAP32[$1>>2]|0;
      $167 = HEAP32[$pBegin>>2]|0;
      $168 = HEAP32[$pCellptr>>2]|0;
      $169 = HEAP32[$iCell>>2]|0;
      $170 = HEAP32[$3>>2]|0;
      $171 = (($169) + ($170))|0;
      $172 = HEAP32[$5>>2]|0;
      $173 = (_pageInsertArray($166,$167,$pData,$168,$171,1,$172)|0);
      $174 = ($173|0)!=(0);
      if ($174) {
       break L7;
      }
     }
    }
    $175 = HEAP32[$i>>2]|0;
    $176 = (($175) + 1)|0;
    HEAP32[$i>>2] = $176;
   }
   $177 = HEAP32[$nCell>>2]|0;
   $178 = $177<<1;
   $179 = HEAP32[$1>>2]|0;
   $180 = ((($179)) + 64|0);
   $181 = HEAP32[$180>>2]|0;
   $182 = (($181) + ($178)|0);
   HEAP32[$pCellptr>>2] = $182;
   $183 = HEAP32[$1>>2]|0;
   $184 = HEAP32[$pBegin>>2]|0;
   $185 = HEAP32[$pCellptr>>2]|0;
   $186 = HEAP32[$3>>2]|0;
   $187 = HEAP32[$nCell>>2]|0;
   $188 = (($186) + ($187))|0;
   $189 = HEAP32[$4>>2]|0;
   $190 = HEAP32[$nCell>>2]|0;
   $191 = (($189) - ($190))|0;
   $192 = HEAP32[$5>>2]|0;
   $193 = (_pageInsertArray($183,$184,$pData,$185,$188,$191,$192)|0);
   $194 = ($193|0)!=(0);
   if (!($194)) {
    $195 = HEAP32[$4>>2]|0;
    $196 = $195&65535;
    $197 = HEAP32[$1>>2]|0;
    $198 = ((($197)) + 18|0);
    HEAP16[$198>>1] = $196;
    $199 = HEAP32[$1>>2]|0;
    $200 = ((($199)) + 1|0);
    HEAP8[$200>>0] = 0;
    $201 = HEAP32[$1>>2]|0;
    $202 = ((($201)) + 18|0);
    $203 = HEAP16[$202>>1]|0;
    $204 = $203&65535;
    $205 = $204 >> 8;
    $206 = $205&255;
    $207 = HEAP32[$hdr>>2]|0;
    $208 = (($207) + 3)|0;
    $209 = HEAP32[$aData>>2]|0;
    $210 = (($209) + ($208)|0);
    HEAP8[$210>>0] = $206;
    $211 = HEAP32[$1>>2]|0;
    $212 = ((($211)) + 18|0);
    $213 = HEAP16[$212>>1]|0;
    $214 = $213&255;
    $215 = HEAP32[$hdr>>2]|0;
    $216 = (($215) + 3)|0;
    $217 = HEAP32[$aData>>2]|0;
    $218 = (($217) + ($216)|0);
    $219 = ((($218)) + 1|0);
    HEAP8[$219>>0] = $214;
    $220 = HEAP32[$pData>>2]|0;
    $221 = HEAP32[$aData>>2]|0;
    $222 = $220;
    $223 = $221;
    $224 = (($222) - ($223))|0;
    $225 = $224 >> 8;
    $226 = $225&255;
    $227 = HEAP32[$hdr>>2]|0;
    $228 = (($227) + 5)|0;
    $229 = HEAP32[$aData>>2]|0;
    $230 = (($229) + ($228)|0);
    HEAP8[$230>>0] = $226;
    $231 = HEAP32[$pData>>2]|0;
    $232 = HEAP32[$aData>>2]|0;
    $233 = $231;
    $234 = $232;
    $235 = (($233) - ($234))|0;
    $236 = $235&255;
    $237 = HEAP32[$hdr>>2]|0;
    $238 = (($237) + 5)|0;
    $239 = HEAP32[$aData>>2]|0;
    $240 = (($239) + ($238)|0);
    $241 = ((($240)) + 1|0);
    HEAP8[$241>>0] = $236;
    HEAP32[$0>>2] = 0;
    $258 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($258|0);
   }
  }
 } while(0);
 $242 = HEAP32[$5>>2]|0;
 $243 = HEAP32[$3>>2]|0;
 $244 = HEAP32[$4>>2]|0;
 _populateCellCache($242,$243,$244);
 $245 = HEAP32[$1>>2]|0;
 $246 = HEAP32[$4>>2]|0;
 $247 = HEAP32[$3>>2]|0;
 $248 = HEAP32[$5>>2]|0;
 $249 = ((($248)) + 8|0);
 $250 = HEAP32[$249>>2]|0;
 $251 = (($250) + ($247<<2)|0);
 $252 = HEAP32[$3>>2]|0;
 $253 = HEAP32[$5>>2]|0;
 $254 = ((($253)) + 12|0);
 $255 = HEAP32[$254>>2]|0;
 $256 = (($255) + ($252<<1)|0);
 $257 = (_rebuildPage($245,$246,$251,$256)|0);
 HEAP32[$0>>2] = $257;
 $258 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($258|0);
}
function _defragmentPage($pPage) {
 $pPage = $pPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cbrk = 0, $cellOffset = 0, $data = 0, $hdr = 0, $i = 0, $iCellFirst = 0, $iCellLast = 0, $nCell = 0, $pAddr = 0;
 var $pc = 0, $size = 0, $src = 0, $temp = 0, $usableSize = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $i = sp + 56|0;
 $pc = sp + 52|0;
 $hdr = sp + 48|0;
 $size = sp + 44|0;
 $usableSize = sp + 40|0;
 $cellOffset = sp + 36|0;
 $cbrk = sp + 32|0;
 $nCell = sp + 28|0;
 $data = sp + 24|0;
 $temp = sp + 20|0;
 $src = sp + 16|0;
 $iCellFirst = sp + 12|0;
 $iCellLast = sp + 8|0;
 $pAddr = sp + 4|0;
 $x = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$temp>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 56|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$data>>2] = $4;
 HEAP32[$src>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 5|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$hdr>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 14|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 HEAP32[$cellOffset>>2] = $12;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 18|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 HEAP32[$nCell>>2] = $16;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 52|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = ((($19)) + 36|0);
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$usableSize>>2] = $21;
 $22 = HEAP32[$usableSize>>2]|0;
 HEAP32[$cbrk>>2] = $22;
 $23 = HEAP32[$cellOffset>>2]|0;
 $24 = HEAP32[$nCell>>2]|0;
 $25 = $24<<1;
 $26 = (($23) + ($25))|0;
 HEAP32[$iCellFirst>>2] = $26;
 $27 = HEAP32[$usableSize>>2]|0;
 $28 = (($27) - 4)|0;
 HEAP32[$iCellLast>>2] = $28;
 HEAP32[$i>>2] = 0;
 while(1) {
  $29 = HEAP32[$i>>2]|0;
  $30 = HEAP32[$nCell>>2]|0;
  $31 = ($29|0)<($30|0);
  if (!($31)) {
   label = 14;
   break;
  }
  $32 = HEAP32[$cellOffset>>2]|0;
  $33 = HEAP32[$i>>2]|0;
  $34 = $33<<1;
  $35 = (($32) + ($34))|0;
  $36 = HEAP32[$data>>2]|0;
  $37 = (($36) + ($35)|0);
  HEAP32[$pAddr>>2] = $37;
  $38 = HEAP32[$pAddr>>2]|0;
  $39 = HEAP8[$38>>0]|0;
  $40 = $39&255;
  $41 = $40 << 8;
  $42 = HEAP32[$pAddr>>2]|0;
  $43 = ((($42)) + 1|0);
  $44 = HEAP8[$43>>0]|0;
  $45 = $44&255;
  $46 = $41 | $45;
  HEAP32[$pc>>2] = $46;
  $47 = HEAP32[$pc>>2]|0;
  $48 = HEAP32[$iCellFirst>>2]|0;
  $49 = ($47|0)<($48|0);
  if ($49) {
   label = 5;
   break;
  }
  $50 = HEAP32[$pc>>2]|0;
  $51 = HEAP32[$iCellLast>>2]|0;
  $52 = ($50|0)>($51|0);
  if ($52) {
   label = 5;
   break;
  }
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 76|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$1>>2]|0;
  $58 = HEAP32[$pc>>2]|0;
  $59 = HEAP32[$src>>2]|0;
  $60 = (($59) + ($58)|0);
  $61 = (FUNCTION_TABLE_iii[$56 & 255]($57,$60)|0);
  $62 = $61&65535;
  HEAP32[$size>>2] = $62;
  $63 = HEAP32[$size>>2]|0;
  $64 = HEAP32[$cbrk>>2]|0;
  $65 = (($64) - ($63))|0;
  HEAP32[$cbrk>>2] = $65;
  $66 = HEAP32[$cbrk>>2]|0;
  $67 = HEAP32[$iCellFirst>>2]|0;
  $68 = ($66|0)<($67|0);
  if ($68) {
   label = 8;
   break;
  }
  $69 = HEAP32[$pc>>2]|0;
  $70 = HEAP32[$size>>2]|0;
  $71 = (($69) + ($70))|0;
  $72 = HEAP32[$usableSize>>2]|0;
  $73 = ($71|0)>($72|0);
  if ($73) {
   label = 8;
   break;
  }
  $75 = HEAP32[$cbrk>>2]|0;
  $76 = $75 >> 8;
  $77 = $76&255;
  $78 = HEAP32[$pAddr>>2]|0;
  HEAP8[$78>>0] = $77;
  $79 = HEAP32[$cbrk>>2]|0;
  $80 = $79&255;
  $81 = HEAP32[$pAddr>>2]|0;
  $82 = ((($81)) + 1|0);
  HEAP8[$82>>0] = $80;
  $83 = HEAP32[$temp>>2]|0;
  $84 = ($83|0)==(0|0);
  if ($84) {
   $85 = HEAP32[$cbrk>>2]|0;
   $86 = HEAP32[$pc>>2]|0;
   $87 = ($85|0)==($86|0);
   if (!($87)) {
    $88 = HEAP32[$1>>2]|0;
    $89 = ((($88)) + 52|0);
    $90 = HEAP32[$89>>2]|0;
    $91 = HEAP32[$90>>2]|0;
    $92 = (_sqlite3PagerTempSpace($91)|0);
    HEAP32[$temp>>2] = $92;
    $93 = HEAP32[$hdr>>2]|0;
    $94 = (($93) + 5)|0;
    $95 = HEAP32[$data>>2]|0;
    $96 = (($95) + ($94)|0);
    $97 = HEAP8[$96>>0]|0;
    $98 = $97&255;
    $99 = $98 << 8;
    $100 = HEAP32[$hdr>>2]|0;
    $101 = (($100) + 5)|0;
    $102 = HEAP32[$data>>2]|0;
    $103 = (($102) + ($101)|0);
    $104 = ((($103)) + 1|0);
    $105 = HEAP8[$104>>0]|0;
    $106 = $105&255;
    $107 = $99 | $106;
    HEAP32[$x>>2] = $107;
    $108 = HEAP32[$x>>2]|0;
    $109 = HEAP32[$temp>>2]|0;
    $110 = (($109) + ($108)|0);
    $111 = HEAP32[$x>>2]|0;
    $112 = HEAP32[$data>>2]|0;
    $113 = (($112) + ($111)|0);
    $114 = HEAP32[$cbrk>>2]|0;
    $115 = HEAP32[$size>>2]|0;
    $116 = (($114) + ($115))|0;
    $117 = HEAP32[$x>>2]|0;
    $118 = (($116) - ($117))|0;
    _memcpy(($110|0),($113|0),($118|0))|0;
    $119 = HEAP32[$temp>>2]|0;
    HEAP32[$src>>2] = $119;
    label = 12;
   }
  } else {
   label = 12;
  }
  if ((label|0) == 12) {
   label = 0;
   $120 = HEAP32[$cbrk>>2]|0;
   $121 = HEAP32[$data>>2]|0;
   $122 = (($121) + ($120)|0);
   $123 = HEAP32[$pc>>2]|0;
   $124 = HEAP32[$src>>2]|0;
   $125 = (($124) + ($123)|0);
   $126 = HEAP32[$size>>2]|0;
   _memcpy(($122|0),($125|0),($126|0))|0;
  }
  $127 = HEAP32[$i>>2]|0;
  $128 = (($127) + 1)|0;
  HEAP32[$i>>2] = $128;
 }
 if ((label|0) == 5) {
  $53 = (_sqlite3CorruptError(59617)|0);
  HEAP32[$0>>2] = $53;
  $170 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($170|0);
 }
 else if ((label|0) == 8) {
  $74 = (_sqlite3CorruptError(59623)|0);
  HEAP32[$0>>2] = $74;
  $170 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($170|0);
 }
 else if ((label|0) == 14) {
  $129 = HEAP32[$cbrk>>2]|0;
  $130 = $129 >> 8;
  $131 = $130&255;
  $132 = HEAP32[$hdr>>2]|0;
  $133 = (($132) + 5)|0;
  $134 = HEAP32[$data>>2]|0;
  $135 = (($134) + ($133)|0);
  HEAP8[$135>>0] = $131;
  $136 = HEAP32[$cbrk>>2]|0;
  $137 = $136&255;
  $138 = HEAP32[$hdr>>2]|0;
  $139 = (($138) + 5)|0;
  $140 = HEAP32[$data>>2]|0;
  $141 = (($140) + ($139)|0);
  $142 = ((($141)) + 1|0);
  HEAP8[$142>>0] = $137;
  $143 = HEAP32[$hdr>>2]|0;
  $144 = (($143) + 1)|0;
  $145 = HEAP32[$data>>2]|0;
  $146 = (($145) + ($144)|0);
  HEAP8[$146>>0] = 0;
  $147 = HEAP32[$hdr>>2]|0;
  $148 = (($147) + 2)|0;
  $149 = HEAP32[$data>>2]|0;
  $150 = (($149) + ($148)|0);
  HEAP8[$150>>0] = 0;
  $151 = HEAP32[$hdr>>2]|0;
  $152 = (($151) + 7)|0;
  $153 = HEAP32[$data>>2]|0;
  $154 = (($153) + ($152)|0);
  HEAP8[$154>>0] = 0;
  $155 = HEAP32[$iCellFirst>>2]|0;
  $156 = HEAP32[$data>>2]|0;
  $157 = (($156) + ($155)|0);
  $158 = HEAP32[$cbrk>>2]|0;
  $159 = HEAP32[$iCellFirst>>2]|0;
  $160 = (($158) - ($159))|0;
  _memset(($157|0),0,($160|0))|0;
  $161 = HEAP32[$cbrk>>2]|0;
  $162 = HEAP32[$iCellFirst>>2]|0;
  $163 = (($161) - ($162))|0;
  $164 = HEAP32[$1>>2]|0;
  $165 = ((($164)) + 16|0);
  $166 = HEAP16[$165>>1]|0;
  $167 = $166&65535;
  $168 = ($163|0)!=($167|0);
  if ($168) {
   $169 = (_sqlite3CorruptError(59647)|0);
   HEAP32[$0>>2] = $169;
   $170 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($170|0);
  } else {
   HEAP32[$0>>2] = 0;
   $170 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($170|0);
  }
 }
 return (0)|0;
}
function _copyNodeContent($pFrom,$pTo,$pRC) {
 $pFrom = $pFrom|0;
 $pTo = $pTo|0;
 $pRC = $pRC|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $9 = 0;
 var $aFrom = 0, $aTo = 0, $iData = 0, $iFromHdr = 0, $iToHdr = 0, $pBt = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $pBt = sp + 24|0;
 $aFrom = sp + 20|0;
 $aTo = sp + 16|0;
 $iFromHdr = sp + 12|0;
 $iToHdr = sp + 8|0;
 $rc = sp + 4|0;
 $iData = sp;
 HEAP32[$0>>2] = $pFrom;
 HEAP32[$1>>2] = $pTo;
 HEAP32[$2>>2] = $pRC;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)==(0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 52|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$aFrom>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 56|0);
 $14 = HEAP32[$13>>2]|0;
 HEAP32[$aTo>>2] = $14;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 5|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 HEAP32[$iFromHdr>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 84|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ($21|0)==(1);
 $23 = $22 ? 100 : 0;
 HEAP32[$iToHdr>>2] = $23;
 $24 = HEAP32[$iFromHdr>>2]|0;
 $25 = (($24) + 5)|0;
 $26 = HEAP32[$aFrom>>2]|0;
 $27 = (($26) + ($25)|0);
 $28 = HEAP8[$27>>0]|0;
 $29 = $28&255;
 $30 = $29 << 8;
 $31 = HEAP32[$iFromHdr>>2]|0;
 $32 = (($31) + 5)|0;
 $33 = HEAP32[$aFrom>>2]|0;
 $34 = (($33) + ($32)|0);
 $35 = ((($34)) + 1|0);
 $36 = HEAP8[$35>>0]|0;
 $37 = $36&255;
 $38 = $30 | $37;
 HEAP32[$iData>>2] = $38;
 $39 = HEAP32[$iData>>2]|0;
 $40 = HEAP32[$aTo>>2]|0;
 $41 = (($40) + ($39)|0);
 $42 = HEAP32[$iData>>2]|0;
 $43 = HEAP32[$aFrom>>2]|0;
 $44 = (($43) + ($42)|0);
 $45 = HEAP32[$pBt>>2]|0;
 $46 = ((($45)) + 36|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$iData>>2]|0;
 $49 = (($47) - ($48))|0;
 _memcpy(($41|0),($44|0),($49|0))|0;
 $50 = HEAP32[$iToHdr>>2]|0;
 $51 = HEAP32[$aTo>>2]|0;
 $52 = (($51) + ($50)|0);
 $53 = HEAP32[$iFromHdr>>2]|0;
 $54 = HEAP32[$aFrom>>2]|0;
 $55 = (($54) + ($53)|0);
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 14|0);
 $58 = HEAP16[$57>>1]|0;
 $59 = $58&65535;
 $60 = HEAP32[$0>>2]|0;
 $61 = ((($60)) + 18|0);
 $62 = HEAP16[$61>>1]|0;
 $63 = $62&65535;
 $64 = $63<<1;
 $65 = (($59) + ($64))|0;
 _memcpy(($52|0),($55|0),($65|0))|0;
 $66 = HEAP32[$1>>2]|0;
 HEAP8[$66>>0] = 0;
 $67 = HEAP32[$1>>2]|0;
 $68 = (_btreeInitPage($67)|0);
 HEAP32[$rc>>2] = $68;
 $69 = HEAP32[$rc>>2]|0;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$rc>>2]|0;
  $72 = HEAP32[$2>>2]|0;
  HEAP32[$72>>2] = $71;
  STACKTOP = sp;return;
 }
 $73 = HEAP32[$pBt>>2]|0;
 $74 = ((($73)) + 17|0);
 $75 = HEAP8[$74>>0]|0;
 $76 = ($75<<24>>24)!=(0);
 if (!($76)) {
  STACKTOP = sp;return;
 }
 $77 = HEAP32[$1>>2]|0;
 $78 = (_setChildPtrmaps($77)|0);
 $79 = HEAP32[$2>>2]|0;
 HEAP32[$79>>2] = $78;
 STACKTOP = sp;return;
}
function _sqlite3PagerTempSpace($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 208|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _pageFreeArray($pPg,$iFirst,$nCell,$pCArray) {
 $pPg = $pPg|0;
 $iFirst = $iFirst|0;
 $nCell = $nCell|0;
 $pCArray = $pCArray|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $aData = 0, $i = 0, $iEnd = 0, $nRet = 0, $pCell = 0, $pEnd = 0, $pFree = 0, $pStart = 0, $sz = 0, $szFree = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $aData = sp + 36|0;
 $pEnd = sp + 32|0;
 $pStart = sp + 28|0;
 $nRet = sp + 24|0;
 $i = sp + 20|0;
 $iEnd = sp + 16|0;
 $pFree = sp + 12|0;
 $szFree = sp + 8|0;
 $pCell = sp + 4|0;
 $sz = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $iFirst;
 HEAP32[$3>>2] = $nCell;
 HEAP32[$4>>2] = $pCArray;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 56|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$aData>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 52|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ((($10)) + 36|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$aData>>2]|0;
 $14 = (($13) + ($12)|0);
 HEAP32[$pEnd>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 5|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = (($18) + 8)|0;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 6|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = (($19) + ($23))|0;
 $25 = HEAP32[$aData>>2]|0;
 $26 = (($25) + ($24)|0);
 HEAP32[$pStart>>2] = $26;
 HEAP32[$nRet>>2] = 0;
 $27 = HEAP32[$2>>2]|0;
 $28 = HEAP32[$3>>2]|0;
 $29 = (($27) + ($28))|0;
 HEAP32[$iEnd>>2] = $29;
 HEAP32[$pFree>>2] = 0;
 HEAP32[$szFree>>2] = 0;
 $30 = HEAP32[$2>>2]|0;
 HEAP32[$i>>2] = $30;
 while(1) {
  $31 = HEAP32[$i>>2]|0;
  $32 = HEAP32[$iEnd>>2]|0;
  $33 = ($31|0)<($32|0);
  if (!($33)) {
   break;
  }
  $34 = HEAP32[$i>>2]|0;
  $35 = HEAP32[$4>>2]|0;
  $36 = ((($35)) + 8|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = (($37) + ($34<<2)|0);
  $39 = HEAP32[$38>>2]|0;
  HEAP32[$pCell>>2] = $39;
  $40 = HEAP32[$pCell>>2]|0;
  $41 = $40;
  $42 = HEAP32[$pStart>>2]|0;
  $43 = $42;
  $44 = ($41>>>0)>=($43>>>0);
  if ($44) {
   $45 = HEAP32[$pCell>>2]|0;
   $46 = $45;
   $47 = HEAP32[$pEnd>>2]|0;
   $48 = $47;
   $49 = ($46>>>0)<($48>>>0);
   if ($49) {
    $50 = HEAP32[$i>>2]|0;
    $51 = HEAP32[$4>>2]|0;
    $52 = ((($51)) + 12|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (($53) + ($50<<1)|0);
    $55 = HEAP16[$54>>1]|0;
    $56 = $55&65535;
    HEAP32[$sz>>2] = $56;
    $57 = HEAP32[$pFree>>2]|0;
    $58 = HEAP32[$pCell>>2]|0;
    $59 = HEAP32[$sz>>2]|0;
    $60 = (($58) + ($59)|0);
    $61 = ($57|0)!=($60|0);
    if ($61) {
     $62 = HEAP32[$pFree>>2]|0;
     $63 = ($62|0)!=(0|0);
     if ($63) {
      $64 = HEAP32[$1>>2]|0;
      $65 = HEAP32[$pFree>>2]|0;
      $66 = HEAP32[$aData>>2]|0;
      $67 = $65;
      $68 = $66;
      $69 = (($67) - ($68))|0;
      $70 = $69&65535;
      $71 = HEAP32[$szFree>>2]|0;
      $72 = $71&65535;
      (_freeSpace($64,$70,$72)|0);
     }
     $73 = HEAP32[$pCell>>2]|0;
     HEAP32[$pFree>>2] = $73;
     $74 = HEAP32[$sz>>2]|0;
     HEAP32[$szFree>>2] = $74;
     $75 = HEAP32[$pFree>>2]|0;
     $76 = HEAP32[$sz>>2]|0;
     $77 = (($75) + ($76)|0);
     $78 = HEAP32[$pEnd>>2]|0;
     $79 = ($77>>>0)>($78>>>0);
     if ($79) {
      label = 9;
      break;
     }
    } else {
     $80 = HEAP32[$pCell>>2]|0;
     HEAP32[$pFree>>2] = $80;
     $81 = HEAP32[$sz>>2]|0;
     $82 = HEAP32[$szFree>>2]|0;
     $83 = (($82) + ($81))|0;
     HEAP32[$szFree>>2] = $83;
    }
    $84 = HEAP32[$nRet>>2]|0;
    $85 = (($84) + 1)|0;
    HEAP32[$nRet>>2] = $85;
   }
  }
  $86 = HEAP32[$i>>2]|0;
  $87 = (($86) + 1)|0;
  HEAP32[$i>>2] = $87;
 }
 if ((label|0) == 9) {
  HEAP32[$0>>2] = 0;
  $100 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($100|0);
 }
 $88 = HEAP32[$pFree>>2]|0;
 $89 = ($88|0)!=(0|0);
 if ($89) {
  $90 = HEAP32[$1>>2]|0;
  $91 = HEAP32[$pFree>>2]|0;
  $92 = HEAP32[$aData>>2]|0;
  $93 = $91;
  $94 = $92;
  $95 = (($93) - ($94))|0;
  $96 = $95&65535;
  $97 = HEAP32[$szFree>>2]|0;
  $98 = $97&65535;
  (_freeSpace($90,$96,$98)|0);
 }
 $99 = HEAP32[$nRet>>2]|0;
 HEAP32[$0>>2] = $99;
 $100 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($100|0);
}
function _pageInsertArray($pPg,$pBegin,$ppData,$pCellptr,$iFirst,$nCell,$pCArray) {
 $pPg = $pPg|0;
 $pBegin = $pBegin|0;
 $ppData = $ppData|0;
 $pCellptr = $pCellptr|0;
 $iFirst = $iFirst|0;
 $nCell = $nCell|0;
 $pCArray = $pCArray|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $9 = 0, $aData = 0, $i = 0, $iEnd = 0, $pData = 0, $pSlot = 0, $rc = 0, $sz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $3 = sp + 44|0;
 $4 = sp + 40|0;
 $5 = sp + 36|0;
 $6 = sp + 32|0;
 $7 = sp + 28|0;
 $i = sp + 24|0;
 $aData = sp + 20|0;
 $pData = sp + 16|0;
 $iEnd = sp + 12|0;
 $sz = sp + 8|0;
 $rc = sp + 4|0;
 $pSlot = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $pBegin;
 HEAP32[$3>>2] = $ppData;
 HEAP32[$4>>2] = $pCellptr;
 HEAP32[$5>>2] = $iFirst;
 HEAP32[$6>>2] = $nCell;
 HEAP32[$7>>2] = $pCArray;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$aData>>2] = $10;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pData>>2] = $12;
 $13 = HEAP32[$5>>2]|0;
 $14 = HEAP32[$6>>2]|0;
 $15 = (($13) + ($14))|0;
 HEAP32[$iEnd>>2] = $15;
 $16 = HEAP32[$5>>2]|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = HEAP32[$iEnd>>2]|0;
  $19 = ($17|0)<($18|0);
  if (!($19)) {
   label = 10;
   break;
  }
  $20 = HEAP32[$7>>2]|0;
  $21 = HEAP32[$i>>2]|0;
  $22 = (_cachedCellSize($20,$21)|0);
  $23 = $22&65535;
  HEAP32[$sz>>2] = $23;
  $24 = HEAP32[$aData>>2]|0;
  $25 = ((($24)) + 1|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = ($27|0)==(0);
  if ($28) {
   $29 = HEAP32[$aData>>2]|0;
   $30 = ((($29)) + 2|0);
   $31 = HEAP8[$30>>0]|0;
   $32 = $31&255;
   $33 = ($32|0)==(0);
   if ($33) {
    label = 6;
   } else {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $34 = HEAP32[$1>>2]|0;
   $35 = HEAP32[$sz>>2]|0;
   $36 = (_pageFindSlot($34,$35,$rc)|0);
   HEAP32[$pSlot>>2] = $36;
   $37 = ($36|0)==(0|0);
   if ($37) {
    label = 6;
   }
  }
  if ((label|0) == 6) {
   label = 0;
   $38 = HEAP32[$pData>>2]|0;
   $39 = HEAP32[$2>>2]|0;
   $40 = $38;
   $41 = $39;
   $42 = (($40) - ($41))|0;
   $43 = HEAP32[$sz>>2]|0;
   $44 = ($42|0)<($43|0);
   if ($44) {
    label = 7;
    break;
   }
   $45 = HEAP32[$sz>>2]|0;
   $46 = HEAP32[$pData>>2]|0;
   $47 = (0 - ($45))|0;
   $48 = (($46) + ($47)|0);
   HEAP32[$pData>>2] = $48;
   $49 = HEAP32[$pData>>2]|0;
   HEAP32[$pSlot>>2] = $49;
  }
  $50 = HEAP32[$pSlot>>2]|0;
  $51 = HEAP32[$i>>2]|0;
  $52 = HEAP32[$7>>2]|0;
  $53 = ((($52)) + 8|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = (($54) + ($51<<2)|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = HEAP32[$sz>>2]|0;
  _memmove(($50|0),($56|0),($57|0))|0;
  $58 = HEAP32[$pSlot>>2]|0;
  $59 = HEAP32[$aData>>2]|0;
  $60 = $58;
  $61 = $59;
  $62 = (($60) - ($61))|0;
  $63 = $62 >> 8;
  $64 = $63&255;
  $65 = HEAP32[$4>>2]|0;
  HEAP8[$65>>0] = $64;
  $66 = HEAP32[$pSlot>>2]|0;
  $67 = HEAP32[$aData>>2]|0;
  $68 = $66;
  $69 = $67;
  $70 = (($68) - ($69))|0;
  $71 = $70&255;
  $72 = HEAP32[$4>>2]|0;
  $73 = ((($72)) + 1|0);
  HEAP8[$73>>0] = $71;
  $74 = HEAP32[$4>>2]|0;
  $75 = ((($74)) + 2|0);
  HEAP32[$4>>2] = $75;
  $76 = HEAP32[$i>>2]|0;
  $77 = (($76) + 1)|0;
  HEAP32[$i>>2] = $77;
 }
 if ((label|0) == 7) {
  HEAP32[$0>>2] = 1;
  $80 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($80|0);
 }
 else if ((label|0) == 10) {
  $78 = HEAP32[$pData>>2]|0;
  $79 = HEAP32[$3>>2]|0;
  HEAP32[$79>>2] = $78;
  HEAP32[$0>>2] = 0;
  $80 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($80|0);
 }
 return (0)|0;
}
function _populateCellCache($p,$idx,$N) {
 $p = $p|0;
 $idx = $idx|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $idx;
 HEAP32[$2>>2] = $N;
 while(1) {
  $3 = HEAP32[$2>>2]|0;
  $4 = ($3|0)>(0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$1>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 12|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (($8) + ($5<<1)|0);
  $10 = HEAP16[$9>>1]|0;
  $11 = $10&65535;
  $12 = ($11|0)==(0);
  if ($12) {
   $13 = HEAP32[$0>>2]|0;
   $14 = ((($13)) + 4|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ((($15)) + 76|0);
   $17 = HEAP32[$16>>2]|0;
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 4|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = HEAP32[$1>>2]|0;
   $22 = HEAP32[$0>>2]|0;
   $23 = ((($22)) + 8|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<2)|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = (FUNCTION_TABLE_iii[$17 & 255]($20,$26)|0);
   $28 = HEAP32[$1>>2]|0;
   $29 = HEAP32[$0>>2]|0;
   $30 = ((($29)) + 12|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = (($31) + ($28<<1)|0);
   HEAP16[$32>>1] = $27;
  }
  $33 = HEAP32[$1>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$1>>2] = $34;
  $35 = HEAP32[$2>>2]|0;
  $36 = (($35) + -1)|0;
  HEAP32[$2>>2] = $36;
 }
 STACKTOP = sp;return;
}
function _rebuildPage($pPg,$nCell,$apCell,$szCell) {
 $pPg = $pPg|0;
 $nCell = $nCell|0;
 $apCell = $apCell|0;
 $szCell = $szCell|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aData = 0, $hdr = 0, $i = 0;
 var $pCell = 0, $pCellptr = 0, $pData = 0, $pEnd = 0, $pTmp = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $3 = sp + 40|0;
 $4 = sp + 36|0;
 $hdr = sp + 32|0;
 $aData = sp + 28|0;
 $usableSize = sp + 24|0;
 $pEnd = sp + 20|0;
 $i = sp + 16|0;
 $pCellptr = sp + 12|0;
 $pTmp = sp + 8|0;
 $pData = sp + 4|0;
 $pCell = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $nCell;
 HEAP32[$3>>2] = $apCell;
 HEAP32[$4>>2] = $szCell;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 5|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 HEAP32[$hdr>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$aData>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 52|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ((($14)) + 36|0);
 $16 = HEAP32[$15>>2]|0;
 HEAP32[$usableSize>>2] = $16;
 $17 = HEAP32[$usableSize>>2]|0;
 $18 = HEAP32[$aData>>2]|0;
 $19 = (($18) + ($17)|0);
 HEAP32[$pEnd>>2] = $19;
 $20 = HEAP32[$1>>2]|0;
 $21 = ((($20)) + 64|0);
 $22 = HEAP32[$21>>2]|0;
 HEAP32[$pCellptr>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 52|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3PagerTempSpace($26)|0);
 HEAP32[$pTmp>>2] = $27;
 $28 = HEAP32[$hdr>>2]|0;
 $29 = (($28) + 5)|0;
 $30 = HEAP32[$aData>>2]|0;
 $31 = (($30) + ($29)|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = $33 << 8;
 $35 = HEAP32[$hdr>>2]|0;
 $36 = (($35) + 5)|0;
 $37 = HEAP32[$aData>>2]|0;
 $38 = (($37) + ($36)|0);
 $39 = ((($38)) + 1|0);
 $40 = HEAP8[$39>>0]|0;
 $41 = $40&255;
 $42 = $34 | $41;
 HEAP32[$i>>2] = $42;
 $43 = HEAP32[$i>>2]|0;
 $44 = HEAP32[$pTmp>>2]|0;
 $45 = (($44) + ($43)|0);
 $46 = HEAP32[$i>>2]|0;
 $47 = HEAP32[$aData>>2]|0;
 $48 = (($47) + ($46)|0);
 $49 = HEAP32[$usableSize>>2]|0;
 $50 = HEAP32[$i>>2]|0;
 $51 = (($49) - ($50))|0;
 _memcpy(($45|0),($48|0),($51|0))|0;
 $52 = HEAP32[$pEnd>>2]|0;
 HEAP32[$pData>>2] = $52;
 HEAP32[$i>>2] = 0;
 while(1) {
  $53 = HEAP32[$i>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  $55 = ($53|0)<($54|0);
  if (!($55)) {
   label = 9;
   break;
  }
  $56 = HEAP32[$i>>2]|0;
  $57 = HEAP32[$3>>2]|0;
  $58 = (($57) + ($56<<2)|0);
  $59 = HEAP32[$58>>2]|0;
  HEAP32[$pCell>>2] = $59;
  $60 = HEAP32[$pCell>>2]|0;
  $61 = $60;
  $62 = HEAP32[$aData>>2]|0;
  $63 = $62;
  $64 = ($61>>>0)>=($63>>>0);
  if ($64) {
   $65 = HEAP32[$pCell>>2]|0;
   $66 = $65;
   $67 = HEAP32[$pEnd>>2]|0;
   $68 = $67;
   $69 = ($66>>>0)<($68>>>0);
   if ($69) {
    $70 = HEAP32[$pCell>>2]|0;
    $71 = HEAP32[$aData>>2]|0;
    $72 = $70;
    $73 = $71;
    $74 = (($72) - ($73))|0;
    $75 = HEAP32[$pTmp>>2]|0;
    $76 = (($75) + ($74)|0);
    HEAP32[$pCell>>2] = $76;
   }
  }
  $77 = HEAP32[$i>>2]|0;
  $78 = HEAP32[$4>>2]|0;
  $79 = (($78) + ($77<<1)|0);
  $80 = HEAP16[$79>>1]|0;
  $81 = $80&65535;
  $82 = HEAP32[$pData>>2]|0;
  $83 = (0 - ($81))|0;
  $84 = (($82) + ($83)|0);
  HEAP32[$pData>>2] = $84;
  $85 = HEAP32[$pData>>2]|0;
  $86 = HEAP32[$aData>>2]|0;
  $87 = $85;
  $88 = $86;
  $89 = (($87) - ($88))|0;
  $90 = $89 >> 8;
  $91 = $90&255;
  $92 = HEAP32[$pCellptr>>2]|0;
  HEAP8[$92>>0] = $91;
  $93 = HEAP32[$pData>>2]|0;
  $94 = HEAP32[$aData>>2]|0;
  $95 = $93;
  $96 = $94;
  $97 = (($95) - ($96))|0;
  $98 = $97&255;
  $99 = HEAP32[$pCellptr>>2]|0;
  $100 = ((($99)) + 1|0);
  HEAP8[$100>>0] = $98;
  $101 = HEAP32[$pCellptr>>2]|0;
  $102 = ((($101)) + 2|0);
  HEAP32[$pCellptr>>2] = $102;
  $103 = HEAP32[$pData>>2]|0;
  $104 = HEAP32[$pCellptr>>2]|0;
  $105 = ($103>>>0)<($104>>>0);
  if ($105) {
   label = 7;
   break;
  }
  $107 = HEAP32[$pData>>2]|0;
  $108 = HEAP32[$pCell>>2]|0;
  $109 = HEAP32[$i>>2]|0;
  $110 = HEAP32[$4>>2]|0;
  $111 = (($110) + ($109<<1)|0);
  $112 = HEAP16[$111>>1]|0;
  $113 = $112&65535;
  _memcpy(($107|0),($108|0),($113|0))|0;
  $114 = HEAP32[$i>>2]|0;
  $115 = (($114) + 1)|0;
  HEAP32[$i>>2] = $115;
 }
 if ((label|0) == 7) {
  $106 = (_sqlite3CorruptError(64716)|0);
  HEAP32[$0>>2] = $106;
  $176 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 else if ((label|0) == 9) {
  $116 = HEAP32[$2>>2]|0;
  $117 = $116&65535;
  $118 = HEAP32[$1>>2]|0;
  $119 = ((($118)) + 18|0);
  HEAP16[$119>>1] = $117;
  $120 = HEAP32[$1>>2]|0;
  $121 = ((($120)) + 1|0);
  HEAP8[$121>>0] = 0;
  $122 = HEAP32[$hdr>>2]|0;
  $123 = (($122) + 1)|0;
  $124 = HEAP32[$aData>>2]|0;
  $125 = (($124) + ($123)|0);
  HEAP8[$125>>0] = 0;
  $126 = HEAP32[$hdr>>2]|0;
  $127 = (($126) + 1)|0;
  $128 = HEAP32[$aData>>2]|0;
  $129 = (($128) + ($127)|0);
  $130 = ((($129)) + 1|0);
  HEAP8[$130>>0] = 0;
  $131 = HEAP32[$1>>2]|0;
  $132 = ((($131)) + 18|0);
  $133 = HEAP16[$132>>1]|0;
  $134 = $133&65535;
  $135 = $134 >> 8;
  $136 = $135&255;
  $137 = HEAP32[$hdr>>2]|0;
  $138 = (($137) + 3)|0;
  $139 = HEAP32[$aData>>2]|0;
  $140 = (($139) + ($138)|0);
  HEAP8[$140>>0] = $136;
  $141 = HEAP32[$1>>2]|0;
  $142 = ((($141)) + 18|0);
  $143 = HEAP16[$142>>1]|0;
  $144 = $143&255;
  $145 = HEAP32[$hdr>>2]|0;
  $146 = (($145) + 3)|0;
  $147 = HEAP32[$aData>>2]|0;
  $148 = (($147) + ($146)|0);
  $149 = ((($148)) + 1|0);
  HEAP8[$149>>0] = $144;
  $150 = HEAP32[$pData>>2]|0;
  $151 = HEAP32[$aData>>2]|0;
  $152 = $150;
  $153 = $151;
  $154 = (($152) - ($153))|0;
  $155 = $154 >> 8;
  $156 = $155&255;
  $157 = HEAP32[$hdr>>2]|0;
  $158 = (($157) + 5)|0;
  $159 = HEAP32[$aData>>2]|0;
  $160 = (($159) + ($158)|0);
  HEAP8[$160>>0] = $156;
  $161 = HEAP32[$pData>>2]|0;
  $162 = HEAP32[$aData>>2]|0;
  $163 = $161;
  $164 = $162;
  $165 = (($163) - ($164))|0;
  $166 = $165&255;
  $167 = HEAP32[$hdr>>2]|0;
  $168 = (($167) + 5)|0;
  $169 = HEAP32[$aData>>2]|0;
  $170 = (($169) + ($168)|0);
  $171 = ((($170)) + 1|0);
  HEAP8[$171>>0] = $166;
  $172 = HEAP32[$hdr>>2]|0;
  $173 = (($172) + 7)|0;
  $174 = HEAP32[$aData>>2]|0;
  $175 = (($174) + ($173)|0);
  HEAP8[$175>>0] = 0;
  HEAP32[$0>>2] = 0;
  $176 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($176|0);
 }
 return (0)|0;
}
function _pageFindSlot($pPg,$nByte,$pRc) {
 $pPg = $pPg|0;
 $nByte = $nByte|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0;
 var $aData = 0, $hdr = 0, $iAddr = 0, $pc = 0, $size = 0, $usableSize = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $hdr = sp + 24|0;
 $aData = sp + 20|0;
 $iAddr = sp + 16|0;
 $pc = sp + 12|0;
 $x = sp + 8|0;
 $usableSize = sp + 4|0;
 $size = sp;
 HEAP32[$1>>2] = $pPg;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $pRc;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 HEAP32[$hdr>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$aData>>2] = $10;
 $11 = HEAP32[$hdr>>2]|0;
 $12 = (($11) + 1)|0;
 HEAP32[$iAddr>>2] = $12;
 $13 = HEAP32[$iAddr>>2]|0;
 $14 = HEAP32[$aData>>2]|0;
 $15 = (($14) + ($13)|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = $17 << 8;
 $19 = HEAP32[$iAddr>>2]|0;
 $20 = HEAP32[$aData>>2]|0;
 $21 = (($20) + ($19)|0);
 $22 = ((($21)) + 1|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = $23&255;
 $25 = $18 | $24;
 HEAP32[$pc>>2] = $25;
 $26 = HEAP32[$1>>2]|0;
 $27 = ((($26)) + 52|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ((($28)) + 36|0);
 $30 = HEAP32[$29>>2]|0;
 HEAP32[$usableSize>>2] = $30;
 while(1) {
  $31 = HEAP32[$pc>>2]|0;
  $32 = HEAP32[$usableSize>>2]|0;
  $33 = (($32) - 4)|0;
  $34 = ($31|0)>($33|0);
  if ($34) {
   label = 4;
   break;
  }
  $35 = HEAP32[$pc>>2]|0;
  $36 = HEAP32[$iAddr>>2]|0;
  $37 = (($36) + 4)|0;
  $38 = ($35|0)<($37|0);
  if ($38) {
   label = 4;
   break;
  }
  $41 = HEAP32[$pc>>2]|0;
  $42 = (($41) + 2)|0;
  $43 = HEAP32[$aData>>2]|0;
  $44 = (($43) + ($42)|0);
  $45 = HEAP8[$44>>0]|0;
  $46 = $45&255;
  $47 = $46 << 8;
  $48 = HEAP32[$pc>>2]|0;
  $49 = (($48) + 2)|0;
  $50 = HEAP32[$aData>>2]|0;
  $51 = (($50) + ($49)|0);
  $52 = ((($51)) + 1|0);
  $53 = HEAP8[$52>>0]|0;
  $54 = $53&255;
  $55 = $47 | $54;
  HEAP32[$size>>2] = $55;
  $56 = HEAP32[$size>>2]|0;
  $57 = HEAP32[$2>>2]|0;
  $58 = (($56) - ($57))|0;
  HEAP32[$x>>2] = $58;
  $59 = ($58|0)>=(0);
  $60 = HEAP32[$pc>>2]|0;
  if ($59) {
   label = 6;
   break;
  }
  HEAP32[$iAddr>>2] = $60;
  $124 = HEAP32[$pc>>2]|0;
  $125 = HEAP32[$aData>>2]|0;
  $126 = (($125) + ($124)|0);
  $127 = HEAP8[$126>>0]|0;
  $128 = $127&255;
  $129 = $128 << 8;
  $130 = HEAP32[$pc>>2]|0;
  $131 = HEAP32[$aData>>2]|0;
  $132 = (($131) + ($130)|0);
  $133 = ((($132)) + 1|0);
  $134 = HEAP8[$133>>0]|0;
  $135 = $134&255;
  $136 = $129 | $135;
  HEAP32[$pc>>2] = $136;
  $137 = HEAP32[$pc>>2]|0;
  $138 = ($137|0)!=(0);
  if (!($138)) {
   label = 16;
   break;
  }
 }
 if ((label|0) == 4) {
  $39 = (_sqlite3CorruptError(59680)|0);
  $40 = HEAP32[$3>>2]|0;
  HEAP32[$40>>2] = $39;
  HEAP32[$0>>2] = 0;
  $139 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 else if ((label|0) == 6) {
  $61 = HEAP32[$1>>2]|0;
  $62 = ((($61)) + 14|0);
  $63 = HEAP16[$62>>1]|0;
  $64 = $63&65535;
  $65 = HEAP32[$1>>2]|0;
  $66 = ((($65)) + 18|0);
  $67 = HEAP16[$66>>1]|0;
  $68 = $67&65535;
  $69 = $68<<1;
  $70 = (($64) + ($69))|0;
  $71 = ($60|0)<($70|0);
  if (!($71)) {
   $72 = HEAP32[$size>>2]|0;
   $73 = HEAP32[$pc>>2]|0;
   $74 = (($72) + ($73))|0;
   $75 = HEAP32[$usableSize>>2]|0;
   $76 = ($74|0)>($75|0);
   if (!($76)) {
    $79 = HEAP32[$x>>2]|0;
    $80 = ($79|0)<(4);
    do {
     if ($80) {
      $81 = HEAP32[$hdr>>2]|0;
      $82 = (($81) + 7)|0;
      $83 = HEAP32[$aData>>2]|0;
      $84 = (($83) + ($82)|0);
      $85 = HEAP8[$84>>0]|0;
      $86 = $85&255;
      $87 = ($86|0)>(57);
      if (!($87)) {
       $88 = HEAP32[$iAddr>>2]|0;
       $89 = HEAP32[$aData>>2]|0;
       $90 = (($89) + ($88)|0);
       $91 = HEAP32[$pc>>2]|0;
       $92 = HEAP32[$aData>>2]|0;
       $93 = (($92) + ($91)|0);
       ;HEAP8[$90>>0]=HEAP8[$93>>0]|0;HEAP8[$90+1>>0]=HEAP8[$93+1>>0]|0;
       $94 = HEAP32[$x>>2]|0;
       $95 = $94&255;
       $96 = $95&255;
       $97 = HEAP32[$hdr>>2]|0;
       $98 = (($97) + 7)|0;
       $99 = HEAP32[$aData>>2]|0;
       $100 = (($99) + ($98)|0);
       $101 = HEAP8[$100>>0]|0;
       $102 = $101&255;
       $103 = (($102) + ($96))|0;
       $104 = $103&255;
       HEAP8[$100>>0] = $104;
       break;
      }
      HEAP32[$0>>2] = 0;
      $139 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($139|0);
     } else {
      $105 = HEAP32[$x>>2]|0;
      $106 = $105 >> 8;
      $107 = $106&255;
      $108 = HEAP32[$pc>>2]|0;
      $109 = (($108) + 2)|0;
      $110 = HEAP32[$aData>>2]|0;
      $111 = (($110) + ($109)|0);
      HEAP8[$111>>0] = $107;
      $112 = HEAP32[$x>>2]|0;
      $113 = $112&255;
      $114 = HEAP32[$pc>>2]|0;
      $115 = (($114) + 2)|0;
      $116 = HEAP32[$aData>>2]|0;
      $117 = (($116) + ($115)|0);
      $118 = ((($117)) + 1|0);
      HEAP8[$118>>0] = $113;
     }
    } while(0);
    $119 = HEAP32[$pc>>2]|0;
    $120 = HEAP32[$x>>2]|0;
    $121 = (($119) + ($120))|0;
    $122 = HEAP32[$aData>>2]|0;
    $123 = (($122) + ($121)|0);
    HEAP32[$0>>2] = $123;
    $139 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($139|0);
   }
  }
  $77 = (_sqlite3CorruptError(59691)|0);
  $78 = HEAP32[$3>>2]|0;
  HEAP32[$78>>2] = $77;
  HEAP32[$0>>2] = 0;
  $139 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 else if ((label|0) == 16) {
  HEAP32[$0>>2] = 0;
  $139 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($139|0);
 }
 return (0)|0;
}
function _freeSpace($pPage,$iStart,$iSize) {
 $pPage = $pPage|0;
 $iStart = $iStart|0;
 $iSize = $iSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $hdr = 0, $iEnd = 0, $iFreeBlk = 0, $iLast = 0, $iOrigSize = 0, $iPtr = 0, $iPtrEnd = 0, $nFrag = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 32|0;
 $3 = sp + 30|0;
 $iPtr = sp + 28|0;
 $iFreeBlk = sp + 26|0;
 $hdr = sp + 35|0;
 $nFrag = sp + 34|0;
 $iOrigSize = sp + 24|0;
 $iLast = sp + 12|0;
 $iEnd = sp + 8|0;
 $data = sp + 4|0;
 $iPtrEnd = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP16[$2>>1] = $iStart;
 HEAP16[$3>>1] = $iSize;
 HEAP8[$nFrag>>0] = 0;
 $4 = HEAP16[$3>>1]|0;
 HEAP16[$iOrigSize>>1] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 36|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) - 4)|0;
 HEAP32[$iLast>>2] = $10;
 $11 = HEAP16[$2>>1]|0;
 $12 = $11&65535;
 $13 = HEAP16[$3>>1]|0;
 $14 = $13&65535;
 $15 = (($12) + ($14))|0;
 HEAP32[$iEnd>>2] = $15;
 $16 = HEAP32[$1>>2]|0;
 $17 = ((($16)) + 56|0);
 $18 = HEAP32[$17>>2]|0;
 HEAP32[$data>>2] = $18;
 $19 = HEAP32[$1>>2]|0;
 $20 = ((($19)) + 52|0);
 $21 = HEAP32[$20>>2]|0;
 $22 = ((($21)) + 22|0);
 $23 = HEAP16[$22>>1]|0;
 $24 = $23&65535;
 $25 = $24 & 4;
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP16[$2>>1]|0;
  $28 = $27&65535;
  $29 = HEAP32[$data>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP16[$3>>1]|0;
  $32 = $31&65535;
  _memset(($30|0),0,($32|0))|0;
 }
 $33 = HEAP32[$1>>2]|0;
 $34 = ((($33)) + 5|0);
 $35 = HEAP8[$34>>0]|0;
 HEAP8[$hdr>>0] = $35;
 $36 = HEAP8[$hdr>>0]|0;
 $37 = $36&255;
 $38 = (($37) + 1)|0;
 $39 = $38&65535;
 HEAP16[$iPtr>>1] = $39;
 $40 = HEAP16[$iPtr>>1]|0;
 $41 = $40&65535;
 $42 = (($41) + 1)|0;
 $43 = HEAP32[$data>>2]|0;
 $44 = (($43) + ($42)|0);
 $45 = HEAP8[$44>>0]|0;
 $46 = $45&255;
 $47 = ($46|0)==(0);
 if ($47) {
  $48 = HEAP16[$iPtr>>1]|0;
  $49 = $48&65535;
  $50 = HEAP32[$data>>2]|0;
  $51 = (($50) + ($49)|0);
  $52 = HEAP8[$51>>0]|0;
  $53 = $52&255;
  $54 = ($53|0)==(0);
  if ($54) {
   HEAP16[$iFreeBlk>>1] = 0;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 do {
  if ((label|0) == 6) {
   while(1) {
    label = 0;
    $55 = HEAP16[$iPtr>>1]|0;
    $56 = $55&65535;
    $57 = HEAP32[$data>>2]|0;
    $58 = (($57) + ($56)|0);
    $59 = HEAP8[$58>>0]|0;
    $60 = $59&255;
    $61 = $60 << 8;
    $62 = HEAP16[$iPtr>>1]|0;
    $63 = $62&65535;
    $64 = HEAP32[$data>>2]|0;
    $65 = (($64) + ($63)|0);
    $66 = ((($65)) + 1|0);
    $67 = HEAP8[$66>>0]|0;
    $68 = $67&255;
    $69 = $61 | $68;
    $70 = $69&65535;
    HEAP16[$iFreeBlk>>1] = $70;
    $71 = $70&65535;
    $72 = HEAP16[$2>>1]|0;
    $73 = $72&65535;
    $74 = ($71|0)<($73|0);
    if (!($74)) {
     break;
    }
    $75 = HEAP16[$iFreeBlk>>1]|0;
    $76 = $75&65535;
    $77 = HEAP16[$iPtr>>1]|0;
    $78 = $77&65535;
    $79 = (($78) + 4)|0;
    $80 = ($76|0)<($79|0);
    $81 = HEAP16[$iFreeBlk>>1]|0;
    if ($80) {
     label = 8;
     break;
    }
    HEAP16[$iPtr>>1] = $81;
    label = 6;
   }
   if ((label|0) == 8) {
    $82 = $81&65535;
    $83 = ($82|0)==(0);
    if (!($83)) {
     $84 = (_sqlite3CorruptError(59854)|0);
     HEAP32[$0>>2] = $84;
     $344 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($344|0);
    }
   }
   $85 = HEAP16[$iFreeBlk>>1]|0;
   $86 = $85&65535;
   $87 = HEAP32[$iLast>>2]|0;
   $88 = ($86>>>0)>($87>>>0);
   if ($88) {
    $89 = (_sqlite3CorruptError(59858)|0);
    HEAP32[$0>>2] = $89;
    $344 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($344|0);
   }
   $90 = HEAP16[$iFreeBlk>>1]|0;
   $91 = $90&65535;
   $92 = ($91|0)!=(0);
   do {
    if ($92) {
     $93 = HEAP32[$iEnd>>2]|0;
     $94 = (($93) + 3)|0;
     $95 = HEAP16[$iFreeBlk>>1]|0;
     $96 = $95&65535;
     $97 = ($94>>>0)>=($96>>>0);
     if ($97) {
      $98 = HEAP16[$iFreeBlk>>1]|0;
      $99 = $98&65535;
      $100 = HEAP32[$iEnd>>2]|0;
      $101 = (($99) - ($100))|0;
      $102 = $101&255;
      HEAP8[$nFrag>>0] = $102;
      $103 = HEAP32[$iEnd>>2]|0;
      $104 = HEAP16[$iFreeBlk>>1]|0;
      $105 = $104&65535;
      $106 = ($103>>>0)>($105>>>0);
      if ($106) {
       $107 = (_sqlite3CorruptError(59869)|0);
       HEAP32[$0>>2] = $107;
       $344 = HEAP32[$0>>2]|0;
       STACKTOP = sp;return ($344|0);
      }
      $108 = HEAP16[$iFreeBlk>>1]|0;
      $109 = $108&65535;
      $110 = HEAP16[$iFreeBlk>>1]|0;
      $111 = $110&65535;
      $112 = (($111) + 2)|0;
      $113 = HEAP32[$data>>2]|0;
      $114 = (($113) + ($112)|0);
      $115 = HEAP8[$114>>0]|0;
      $116 = $115&255;
      $117 = $116 << 8;
      $118 = HEAP16[$iFreeBlk>>1]|0;
      $119 = $118&65535;
      $120 = (($119) + 2)|0;
      $121 = HEAP32[$data>>2]|0;
      $122 = (($121) + ($120)|0);
      $123 = ((($122)) + 1|0);
      $124 = HEAP8[$123>>0]|0;
      $125 = $124&255;
      $126 = $117 | $125;
      $127 = (($109) + ($126))|0;
      HEAP32[$iEnd>>2] = $127;
      $128 = HEAP32[$iEnd>>2]|0;
      $129 = HEAP32[$1>>2]|0;
      $130 = ((($129)) + 52|0);
      $131 = HEAP32[$130>>2]|0;
      $132 = ((($131)) + 36|0);
      $133 = HEAP32[$132>>2]|0;
      $134 = ($128>>>0)>($133>>>0);
      if (!($134)) {
       $136 = HEAP32[$iEnd>>2]|0;
       $137 = HEAP16[$2>>1]|0;
       $138 = $137&65535;
       $139 = (($136) - ($138))|0;
       $140 = $139&65535;
       HEAP16[$3>>1] = $140;
       $141 = HEAP16[$iFreeBlk>>1]|0;
       $142 = $141&65535;
       $143 = HEAP32[$data>>2]|0;
       $144 = (($143) + ($142)|0);
       $145 = HEAP8[$144>>0]|0;
       $146 = $145&255;
       $147 = $146 << 8;
       $148 = HEAP16[$iFreeBlk>>1]|0;
       $149 = $148&65535;
       $150 = HEAP32[$data>>2]|0;
       $151 = (($150) + ($149)|0);
       $152 = ((($151)) + 1|0);
       $153 = HEAP8[$152>>0]|0;
       $154 = $153&255;
       $155 = $147 | $154;
       $156 = $155&65535;
       HEAP16[$iFreeBlk>>1] = $156;
       break;
      }
      $135 = (_sqlite3CorruptError(59871)|0);
      HEAP32[$0>>2] = $135;
      $344 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($344|0);
     }
    }
   } while(0);
   $157 = HEAP16[$iPtr>>1]|0;
   $158 = $157&65535;
   $159 = HEAP8[$hdr>>0]|0;
   $160 = $159&255;
   $161 = (($160) + 1)|0;
   $162 = ($158|0)>($161|0);
   do {
    if ($162) {
     $163 = HEAP16[$iPtr>>1]|0;
     $164 = $163&65535;
     $165 = HEAP16[$iPtr>>1]|0;
     $166 = $165&65535;
     $167 = (($166) + 2)|0;
     $168 = HEAP32[$data>>2]|0;
     $169 = (($168) + ($167)|0);
     $170 = HEAP8[$169>>0]|0;
     $171 = $170&255;
     $172 = $171 << 8;
     $173 = HEAP16[$iPtr>>1]|0;
     $174 = $173&65535;
     $175 = (($174) + 2)|0;
     $176 = HEAP32[$data>>2]|0;
     $177 = (($176) + ($175)|0);
     $178 = ((($177)) + 1|0);
     $179 = HEAP8[$178>>0]|0;
     $180 = $179&255;
     $181 = $172 | $180;
     $182 = (($164) + ($181))|0;
     HEAP32[$iPtrEnd>>2] = $182;
     $183 = HEAP32[$iPtrEnd>>2]|0;
     $184 = (($183) + 3)|0;
     $185 = HEAP16[$2>>1]|0;
     $186 = $185&65535;
     $187 = ($184|0)>=($186|0);
     if ($187) {
      $188 = HEAP32[$iPtrEnd>>2]|0;
      $189 = HEAP16[$2>>1]|0;
      $190 = $189&65535;
      $191 = ($188|0)>($190|0);
      if (!($191)) {
       $193 = HEAP16[$2>>1]|0;
       $194 = $193&65535;
       $195 = HEAP32[$iPtrEnd>>2]|0;
       $196 = (($194) - ($195))|0;
       $197 = HEAP8[$nFrag>>0]|0;
       $198 = $197&255;
       $199 = (($198) + ($196))|0;
       $200 = $199&255;
       HEAP8[$nFrag>>0] = $200;
       $201 = HEAP32[$iEnd>>2]|0;
       $202 = HEAP16[$iPtr>>1]|0;
       $203 = $202&65535;
       $204 = (($201) - ($203))|0;
       $205 = $204&65535;
       HEAP16[$3>>1] = $205;
       $206 = HEAP16[$iPtr>>1]|0;
       HEAP16[$2>>1] = $206;
       break;
      }
      $192 = (_sqlite3CorruptError(59883)|0);
      HEAP32[$0>>2] = $192;
      $344 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($344|0);
     }
    }
   } while(0);
   $207 = HEAP8[$nFrag>>0]|0;
   $208 = $207&255;
   $209 = HEAP8[$hdr>>0]|0;
   $210 = $209&255;
   $211 = (($210) + 7)|0;
   $212 = HEAP32[$data>>2]|0;
   $213 = (($212) + ($211)|0);
   $214 = HEAP8[$213>>0]|0;
   $215 = $214&255;
   $216 = ($208|0)>($215|0);
   if (!($216)) {
    $218 = HEAP8[$nFrag>>0]|0;
    $219 = $218&255;
    $220 = HEAP8[$hdr>>0]|0;
    $221 = $220&255;
    $222 = (($221) + 7)|0;
    $223 = HEAP32[$data>>2]|0;
    $224 = (($223) + ($222)|0);
    $225 = HEAP8[$224>>0]|0;
    $226 = $225&255;
    $227 = (($226) - ($219))|0;
    $228 = $227&255;
    HEAP8[$224>>0] = $228;
    break;
   }
   $217 = (_sqlite3CorruptError(59889)|0);
   HEAP32[$0>>2] = $217;
   $344 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($344|0);
  }
 } while(0);
 $229 = HEAP16[$2>>1]|0;
 $230 = $229&65535;
 $231 = HEAP8[$hdr>>0]|0;
 $232 = $231&255;
 $233 = (($232) + 5)|0;
 $234 = HEAP32[$data>>2]|0;
 $235 = (($234) + ($233)|0);
 $236 = HEAP8[$235>>0]|0;
 $237 = $236&255;
 $238 = $237 << 8;
 $239 = HEAP8[$hdr>>0]|0;
 $240 = $239&255;
 $241 = (($240) + 5)|0;
 $242 = HEAP32[$data>>2]|0;
 $243 = (($242) + ($241)|0);
 $244 = ((($243)) + 1|0);
 $245 = HEAP8[$244>>0]|0;
 $246 = $245&255;
 $247 = $238 | $246;
 $248 = ($230|0)==($247|0);
 do {
  if ($248) {
   $249 = HEAP16[$iPtr>>1]|0;
   $250 = $249&65535;
   $251 = HEAP8[$hdr>>0]|0;
   $252 = $251&255;
   $253 = (($252) + 1)|0;
   $254 = ($250|0)!=($253|0);
   if (!($254)) {
    $256 = HEAP16[$iFreeBlk>>1]|0;
    $257 = $256&65535;
    $258 = $257 >> 8;
    $259 = $258&255;
    $260 = HEAP8[$hdr>>0]|0;
    $261 = $260&255;
    $262 = (($261) + 1)|0;
    $263 = HEAP32[$data>>2]|0;
    $264 = (($263) + ($262)|0);
    HEAP8[$264>>0] = $259;
    $265 = HEAP16[$iFreeBlk>>1]|0;
    $266 = $265&255;
    $267 = HEAP8[$hdr>>0]|0;
    $268 = $267&255;
    $269 = (($268) + 1)|0;
    $270 = HEAP32[$data>>2]|0;
    $271 = (($270) + ($269)|0);
    $272 = ((($271)) + 1|0);
    HEAP8[$272>>0] = $266;
    $273 = HEAP32[$iEnd>>2]|0;
    $274 = $273 >>> 8;
    $275 = $274&255;
    $276 = HEAP8[$hdr>>0]|0;
    $277 = $276&255;
    $278 = (($277) + 5)|0;
    $279 = HEAP32[$data>>2]|0;
    $280 = (($279) + ($278)|0);
    HEAP8[$280>>0] = $275;
    $281 = HEAP32[$iEnd>>2]|0;
    $282 = $281&255;
    $283 = HEAP8[$hdr>>0]|0;
    $284 = $283&255;
    $285 = (($284) + 5)|0;
    $286 = HEAP32[$data>>2]|0;
    $287 = (($286) + ($285)|0);
    $288 = ((($287)) + 1|0);
    HEAP8[$288>>0] = $282;
    break;
   }
   $255 = (_sqlite3CorruptError(59896)|0);
   HEAP32[$0>>2] = $255;
   $344 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($344|0);
  } else {
   $289 = HEAP16[$2>>1]|0;
   $290 = $289&65535;
   $291 = $290 >> 8;
   $292 = $291&255;
   $293 = HEAP16[$iPtr>>1]|0;
   $294 = $293&65535;
   $295 = HEAP32[$data>>2]|0;
   $296 = (($295) + ($294)|0);
   HEAP8[$296>>0] = $292;
   $297 = HEAP16[$2>>1]|0;
   $298 = $297&255;
   $299 = HEAP16[$iPtr>>1]|0;
   $300 = $299&65535;
   $301 = HEAP32[$data>>2]|0;
   $302 = (($301) + ($300)|0);
   $303 = ((($302)) + 1|0);
   HEAP8[$303>>0] = $298;
   $304 = HEAP16[$iFreeBlk>>1]|0;
   $305 = $304&65535;
   $306 = $305 >> 8;
   $307 = $306&255;
   $308 = HEAP16[$2>>1]|0;
   $309 = $308&65535;
   $310 = HEAP32[$data>>2]|0;
   $311 = (($310) + ($309)|0);
   HEAP8[$311>>0] = $307;
   $312 = HEAP16[$iFreeBlk>>1]|0;
   $313 = $312&255;
   $314 = HEAP16[$2>>1]|0;
   $315 = $314&65535;
   $316 = HEAP32[$data>>2]|0;
   $317 = (($316) + ($315)|0);
   $318 = ((($317)) + 1|0);
   HEAP8[$318>>0] = $313;
   $319 = HEAP16[$3>>1]|0;
   $320 = $319&65535;
   $321 = $320 >> 8;
   $322 = $321&255;
   $323 = HEAP16[$2>>1]|0;
   $324 = $323&65535;
   $325 = (($324) + 2)|0;
   $326 = HEAP32[$data>>2]|0;
   $327 = (($326) + ($325)|0);
   HEAP8[$327>>0] = $322;
   $328 = HEAP16[$3>>1]|0;
   $329 = $328&255;
   $330 = HEAP16[$2>>1]|0;
   $331 = $330&65535;
   $332 = (($331) + 2)|0;
   $333 = HEAP32[$data>>2]|0;
   $334 = (($333) + ($332)|0);
   $335 = ((($334)) + 1|0);
   HEAP8[$335>>0] = $329;
  }
 } while(0);
 $336 = HEAP16[$iOrigSize>>1]|0;
 $337 = $336&65535;
 $338 = HEAP32[$1>>2]|0;
 $339 = ((($338)) + 16|0);
 $340 = HEAP16[$339>>1]|0;
 $341 = $340&65535;
 $342 = (($341) + ($337))|0;
 $343 = $342&65535;
 HEAP16[$339>>1] = $343;
 HEAP32[$0>>2] = 0;
 $344 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($344|0);
}
function _computeCellSize($p,$N) {
 $p = $p|0;
 $N = $N|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 4|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ((($4)) + 76|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (($13) + ($10<<2)|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = (FUNCTION_TABLE_iii[$6 & 255]($9,$15)|0);
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (($20) + ($17<<1)|0);
 HEAP16[$21>>1] = $16;
 $22 = HEAP32[$1>>2]|0;
 $23 = HEAP32[$0>>2]|0;
 $24 = ((($23)) + 12|0);
 $25 = HEAP32[$24>>2]|0;
 $26 = (($25) + ($22<<1)|0);
 $27 = HEAP16[$26>>1]|0;
 STACKTOP = sp;return ($27|0);
}
function _allocateSpace($pPage,$nByte,$pIdx) {
 $pPage = $pPage|0;
 $nByte = $nByte|0;
 $pIdx = $pIdx|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0;
 var $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0;
 var $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0;
 var $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0;
 var $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $data = 0, $gap = 0, $hdr = 0, $pSpace = 0, $rc = 0, $top = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $hdr = sp + 20|0;
 $data = sp + 16|0;
 $top = sp + 12|0;
 $rc = sp + 8|0;
 $gap = sp + 4|0;
 $pSpace = sp;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $nByte;
 HEAP32[$3>>2] = $pIdx;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 5|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 HEAP32[$hdr>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 56|0);
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$data>>2] = $10;
 HEAP32[$rc>>2] = 0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 14|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 18|0);
 $17 = HEAP16[$16>>1]|0;
 $18 = $17&65535;
 $19 = $18<<1;
 $20 = (($14) + ($19))|0;
 HEAP32[$gap>>2] = $20;
 $21 = HEAP32[$hdr>>2]|0;
 $22 = (($21) + 5)|0;
 $23 = HEAP32[$data>>2]|0;
 $24 = (($23) + ($22)|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = $25&255;
 $27 = $26 << 8;
 $28 = HEAP32[$hdr>>2]|0;
 $29 = (($28) + 5)|0;
 $30 = HEAP32[$data>>2]|0;
 $31 = (($30) + ($29)|0);
 $32 = ((($31)) + 1|0);
 $33 = HEAP8[$32>>0]|0;
 $34 = $33&255;
 $35 = $27 | $34;
 HEAP32[$top>>2] = $35;
 $36 = HEAP32[$gap>>2]|0;
 $37 = HEAP32[$top>>2]|0;
 $38 = ($36|0)>($37|0);
 do {
  if ($38) {
   $39 = HEAP32[$top>>2]|0;
   $40 = ($39|0)==(0);
   if ($40) {
    $41 = HEAP32[$1>>2]|0;
    $42 = ((($41)) + 52|0);
    $43 = HEAP32[$42>>2]|0;
    $44 = ((($43)) + 36|0);
    $45 = HEAP32[$44>>2]|0;
    $46 = ($45|0)==(65536);
    if ($46) {
     HEAP32[$top>>2] = 65536;
     break;
    }
   }
   $47 = (_sqlite3CorruptError(59758)|0);
   HEAP32[$0>>2] = $47;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $48 = HEAP32[$hdr>>2]|0;
 $49 = (($48) + 2)|0;
 $50 = HEAP32[$data>>2]|0;
 $51 = (($50) + ($49)|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = $52&255;
 $54 = ($53|0)!=(0);
 if ($54) {
  label = 8;
 } else {
  $55 = HEAP32[$hdr>>2]|0;
  $56 = (($55) + 1)|0;
  $57 = HEAP32[$data>>2]|0;
  $58 = (($57) + ($56)|0);
  $59 = HEAP8[$58>>0]|0;
  $60 = $59&255;
  $61 = ($60|0)!=(0);
  if ($61) {
   label = 8;
  }
 }
 if ((label|0) == 8) {
  $62 = HEAP32[$gap>>2]|0;
  $63 = (($62) + 2)|0;
  $64 = HEAP32[$top>>2]|0;
  $65 = ($63|0)<=($64|0);
  if ($65) {
   $66 = HEAP32[$1>>2]|0;
   $67 = HEAP32[$2>>2]|0;
   $68 = (_pageFindSlot($66,$67,$rc)|0);
   HEAP32[$pSpace>>2] = $68;
   $69 = HEAP32[$pSpace>>2]|0;
   $70 = ($69|0)!=(0|0);
   if ($70) {
    $71 = HEAP32[$pSpace>>2]|0;
    $72 = HEAP32[$data>>2]|0;
    $73 = $71;
    $74 = $72;
    $75 = (($73) - ($74))|0;
    $76 = HEAP32[$3>>2]|0;
    HEAP32[$76>>2] = $75;
    HEAP32[$0>>2] = 0;
    $128 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($128|0);
   }
   $77 = HEAP32[$rc>>2]|0;
   $78 = ($77|0)!=(0);
   if ($78) {
    $79 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $79;
    $128 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($128|0);
   }
  }
 }
 $80 = HEAP32[$gap>>2]|0;
 $81 = (($80) + 2)|0;
 $82 = HEAP32[$2>>2]|0;
 $83 = (($81) + ($82))|0;
 $84 = HEAP32[$top>>2]|0;
 $85 = ($83|0)>($84|0);
 do {
  if ($85) {
   $86 = HEAP32[$1>>2]|0;
   $87 = (_defragmentPage($86)|0);
   HEAP32[$rc>>2] = $87;
   $88 = HEAP32[$rc>>2]|0;
   $89 = ($88|0)!=(0);
   if (!($89)) {
    $91 = HEAP32[$hdr>>2]|0;
    $92 = (($91) + 5)|0;
    $93 = HEAP32[$data>>2]|0;
    $94 = (($93) + ($92)|0);
    $95 = HEAP8[$94>>0]|0;
    $96 = $95&255;
    $97 = $96 << 8;
    $98 = HEAP32[$hdr>>2]|0;
    $99 = (($98) + 5)|0;
    $100 = HEAP32[$data>>2]|0;
    $101 = (($100) + ($99)|0);
    $102 = ((($101)) + 1|0);
    $103 = HEAP8[$102>>0]|0;
    $104 = $103&255;
    $105 = $97 | $104;
    $106 = (($105) - 1)|0;
    $107 = $106 & 65535;
    $108 = (($107) + 1)|0;
    HEAP32[$top>>2] = $108;
    break;
   }
   $90 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $90;
   $128 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($128|0);
  }
 } while(0);
 $109 = HEAP32[$2>>2]|0;
 $110 = HEAP32[$top>>2]|0;
 $111 = (($110) - ($109))|0;
 HEAP32[$top>>2] = $111;
 $112 = HEAP32[$top>>2]|0;
 $113 = $112 >> 8;
 $114 = $113&255;
 $115 = HEAP32[$hdr>>2]|0;
 $116 = (($115) + 5)|0;
 $117 = HEAP32[$data>>2]|0;
 $118 = (($117) + ($116)|0);
 HEAP8[$118>>0] = $114;
 $119 = HEAP32[$top>>2]|0;
 $120 = $119&255;
 $121 = HEAP32[$hdr>>2]|0;
 $122 = (($121) + 5)|0;
 $123 = HEAP32[$data>>2]|0;
 $124 = (($123) + ($122)|0);
 $125 = ((($124)) + 1|0);
 HEAP8[$125>>0] = $120;
 $126 = HEAP32[$top>>2]|0;
 $127 = HEAP32[$3>>2]|0;
 HEAP32[$127>>2] = $126;
 HEAP32[$0>>2] = 0;
 $128 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($128|0);
}
function _fillInCell($pPage,$pCell,$pX,$pnSize) {
 $pPage = $pPage|0;
 $pCell = $pCell|0;
 $pX = $pX|0;
 $pnSize = $pnSize|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $eType = 0, $mn = 0, $n = 0, $nHeader = 0, $nPayload = 0, $nSrc = 0, $or$cond = 0, $pBt = 0, $pOvfl = 0, $pPayload = 0, $pPrior = 0, $pSrc = 0, $pToRelease = 0, $pgnoOvfl = 0, $pgnoPtrmap = 0, $rc = 0, $spaceLeft = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $3 = sp + 64|0;
 $4 = sp + 60|0;
 $nPayload = sp + 56|0;
 $pSrc = sp + 52|0;
 $nSrc = sp + 48|0;
 $n = sp + 44|0;
 $rc = sp + 40|0;
 $spaceLeft = sp + 36|0;
 $pOvfl = sp + 32|0;
 $pToRelease = sp + 28|0;
 $pPrior = sp + 24|0;
 $pPayload = sp + 20|0;
 $pBt = sp + 16|0;
 $pgnoOvfl = sp + 12|0;
 $nHeader = sp + 8|0;
 $mn = sp + 4|0;
 $pgnoPtrmap = sp;
 $eType = sp + 80|0;
 HEAP32[$1>>2] = $pPage;
 HEAP32[$2>>2] = $pCell;
 HEAP32[$3>>2] = $pX;
 HEAP32[$4>>2] = $pnSize;
 HEAP32[$pOvfl>>2] = 0;
 HEAP32[$pToRelease>>2] = 0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 52|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$pBt>>2] = $7;
 HEAP32[$pgnoOvfl>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = ((($8)) + 6|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 HEAP32[$nHeader>>2] = $11;
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 2|0);
 $14 = HEAP8[$13>>0]|0;
 $15 = ($14<<24>>24)!=(0);
 $16 = HEAP32[$3>>2]|0;
 if ($15) {
  $17 = ((($16)) + 20|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$3>>2]|0;
  $20 = ((($19)) + 24|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = (($18) + ($21))|0;
  HEAP32[$nPayload>>2] = $22;
  $23 = HEAP32[$3>>2]|0;
  $24 = ((($23)) + 16|0);
  $25 = HEAP32[$24>>2]|0;
  HEAP32[$pSrc>>2] = $25;
  $26 = HEAP32[$3>>2]|0;
  $27 = ((($26)) + 20|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$nSrc>>2] = $28;
  $29 = HEAP32[$nPayload>>2]|0;
  $30 = ($29>>>0)<(128);
  if ($30) {
   $31 = HEAP32[$nPayload>>2]|0;
   $32 = $31&255;
   $33 = HEAP32[$nHeader>>2]|0;
   $34 = HEAP32[$2>>2]|0;
   $35 = (($34) + ($33)|0);
   HEAP8[$35>>0] = $32;
   $44 = 1;
  } else {
   $36 = HEAP32[$nHeader>>2]|0;
   $37 = HEAP32[$2>>2]|0;
   $38 = (($37) + ($36)|0);
   $39 = HEAP32[$nPayload>>2]|0;
   $40 = ($39|0)<(0);
   $41 = $40 << 31 >> 31;
   $42 = (_sqlite3PutVarint($38,$39,$41)|0);
   $44 = $42;
  }
  $43 = $44&255;
  $45 = $43&255;
  $46 = HEAP32[$nHeader>>2]|0;
  $47 = (($46) + ($45))|0;
  HEAP32[$nHeader>>2] = $47;
  $48 = HEAP32[$nHeader>>2]|0;
  $49 = HEAP32[$2>>2]|0;
  $50 = (($49) + ($48)|0);
  $51 = HEAP32[$3>>2]|0;
  $52 = ((($51)) + 8|0);
  $53 = $52;
  $54 = $53;
  $55 = HEAP32[$54>>2]|0;
  $56 = (($53) + 4)|0;
  $57 = $56;
  $58 = HEAP32[$57>>2]|0;
  $59 = (_sqlite3PutVarint($50,$55,$58)|0);
  $60 = HEAP32[$nHeader>>2]|0;
  $61 = (($60) + ($59))|0;
  HEAP32[$nHeader>>2] = $61;
 } else {
  $62 = ((($16)) + 8|0);
  $63 = $62;
  $64 = $63;
  $65 = HEAP32[$64>>2]|0;
  $66 = (($63) + 4)|0;
  $67 = $66;
  $68 = HEAP32[$67>>2]|0;
  HEAP32[$nPayload>>2] = $65;
  HEAP32[$nSrc>>2] = $65;
  $69 = HEAP32[$3>>2]|0;
  $70 = HEAP32[$69>>2]|0;
  HEAP32[$pSrc>>2] = $70;
  $71 = HEAP32[$nPayload>>2]|0;
  $72 = ($71>>>0)<(128);
  if ($72) {
   $73 = HEAP32[$nPayload>>2]|0;
   $74 = $73&255;
   $75 = HEAP32[$nHeader>>2]|0;
   $76 = HEAP32[$2>>2]|0;
   $77 = (($76) + ($75)|0);
   HEAP8[$77>>0] = $74;
   $86 = 1;
  } else {
   $78 = HEAP32[$nHeader>>2]|0;
   $79 = HEAP32[$2>>2]|0;
   $80 = (($79) + ($78)|0);
   $81 = HEAP32[$nPayload>>2]|0;
   $82 = ($81|0)<(0);
   $83 = $82 << 31 >> 31;
   $84 = (_sqlite3PutVarint($80,$81,$83)|0);
   $86 = $84;
  }
  $85 = $86&255;
  $87 = $85&255;
  $88 = HEAP32[$nHeader>>2]|0;
  $89 = (($88) + ($87))|0;
  HEAP32[$nHeader>>2] = $89;
 }
 $90 = HEAP32[$nPayload>>2]|0;
 $91 = HEAP32[$1>>2]|0;
 $92 = ((($91)) + 10|0);
 $93 = HEAP16[$92>>1]|0;
 $94 = $93&65535;
 $95 = ($90|0)<=($94|0);
 if ($95) {
  $96 = HEAP32[$nHeader>>2]|0;
  $97 = HEAP32[$nPayload>>2]|0;
  $98 = (($96) + ($97))|0;
  HEAP32[$n>>2] = $98;
  $99 = HEAP32[$n>>2]|0;
  $100 = ($99|0)<(4);
  $$ = $100 ? 4 : $98;
  HEAP32[$n>>2] = $$;
  $101 = HEAP32[$n>>2]|0;
  $102 = HEAP32[$4>>2]|0;
  HEAP32[$102>>2] = $101;
  $103 = HEAP32[$nPayload>>2]|0;
  HEAP32[$spaceLeft>>2] = $103;
  $104 = HEAP32[$2>>2]|0;
  HEAP32[$pPrior>>2] = $104;
 } else {
  $105 = HEAP32[$1>>2]|0;
  $106 = ((($105)) + 12|0);
  $107 = HEAP16[$106>>1]|0;
  $108 = $107&65535;
  HEAP32[$mn>>2] = $108;
  $109 = HEAP32[$mn>>2]|0;
  $110 = HEAP32[$nPayload>>2]|0;
  $111 = HEAP32[$mn>>2]|0;
  $112 = (($110) - ($111))|0;
  $113 = HEAP32[$1>>2]|0;
  $114 = ((($113)) + 52|0);
  $115 = HEAP32[$114>>2]|0;
  $116 = ((($115)) + 36|0);
  $117 = HEAP32[$116>>2]|0;
  $118 = (($117) - 4)|0;
  $119 = (($112>>>0) % ($118>>>0))&-1;
  $120 = (($109) + ($119))|0;
  HEAP32[$n>>2] = $120;
  $121 = HEAP32[$n>>2]|0;
  $122 = HEAP32[$1>>2]|0;
  $123 = ((($122)) + 10|0);
  $124 = HEAP16[$123>>1]|0;
  $125 = $124&65535;
  $126 = ($121|0)>($125|0);
  if ($126) {
   $127 = HEAP32[$mn>>2]|0;
   HEAP32[$n>>2] = $127;
  }
  $128 = HEAP32[$n>>2]|0;
  HEAP32[$spaceLeft>>2] = $128;
  $129 = HEAP32[$n>>2]|0;
  $130 = HEAP32[$nHeader>>2]|0;
  $131 = (($129) + ($130))|0;
  $132 = (($131) + 4)|0;
  $133 = HEAP32[$4>>2]|0;
  HEAP32[$133>>2] = $132;
  $134 = HEAP32[$nHeader>>2]|0;
  $135 = HEAP32[$n>>2]|0;
  $136 = (($134) + ($135))|0;
  $137 = HEAP32[$2>>2]|0;
  $138 = (($137) + ($136)|0);
  HEAP32[$pPrior>>2] = $138;
 }
 $139 = HEAP32[$nHeader>>2]|0;
 $140 = HEAP32[$2>>2]|0;
 $141 = (($140) + ($139)|0);
 HEAP32[$pPayload>>2] = $141;
 while(1) {
  $142 = HEAP32[$nPayload>>2]|0;
  $143 = ($142|0)>(0);
  if (!($143)) {
   label = 35;
   break;
  }
  $144 = HEAP32[$spaceLeft>>2]|0;
  $145 = ($144|0)==(0);
  if ($145) {
   $146 = HEAP32[$pgnoOvfl>>2]|0;
   HEAP32[$pgnoPtrmap>>2] = $146;
   $147 = HEAP32[$pBt>>2]|0;
   $148 = ((($147)) + 17|0);
   $149 = HEAP8[$148>>0]|0;
   $150 = ($149<<24>>24)!=(0);
   if ($150) {
    while(1) {
     $151 = HEAP32[$pgnoOvfl>>2]|0;
     $152 = (($151) + 1)|0;
     HEAP32[$pgnoOvfl>>2] = $152;
     $153 = HEAP32[$pBt>>2]|0;
     $154 = HEAP32[$pgnoOvfl>>2]|0;
     $155 = (_ptrmapPageno($153,$154)|0);
     $156 = HEAP32[$pgnoOvfl>>2]|0;
     $157 = ($155|0)==($156|0);
     if ($157) {
      continue;
     }
     $158 = HEAP32[$pgnoOvfl>>2]|0;
     $159 = HEAP32[1536>>2]|0;
     $160 = HEAP32[$pBt>>2]|0;
     $161 = ((($160)) + 32|0);
     $162 = HEAP32[$161>>2]|0;
     $163 = (($159>>>0) / ($162>>>0))&-1;
     $164 = (($163) + 1)|0;
     $165 = ($158|0)==($164|0);
     if (!($165)) {
      break;
     }
    }
   }
   $166 = HEAP32[$pBt>>2]|0;
   $167 = HEAP32[$pgnoOvfl>>2]|0;
   $168 = (_allocateBtreePage($166,$pOvfl,$pgnoOvfl,$167,0)|0);
   HEAP32[$rc>>2] = $168;
   $169 = HEAP32[$pBt>>2]|0;
   $170 = ((($169)) + 17|0);
   $171 = HEAP8[$170>>0]|0;
   $172 = $171&255;
   $173 = ($172|0)!=(0);
   $174 = HEAP32[$rc>>2]|0;
   $175 = ($174|0)==(0);
   $or$cond = $173 & $175;
   if ($or$cond) {
    $176 = HEAP32[$pgnoPtrmap>>2]|0;
    $177 = ($176|0)!=(0);
    $178 = $177 ? 4 : 3;
    $179 = $178&255;
    HEAP8[$eType>>0] = $179;
    $180 = HEAP32[$pBt>>2]|0;
    $181 = HEAP32[$pgnoOvfl>>2]|0;
    $182 = HEAP8[$eType>>0]|0;
    $183 = HEAP32[$pgnoPtrmap>>2]|0;
    _ptrmapPut($180,$181,$182,$183,$rc);
    $184 = HEAP32[$rc>>2]|0;
    $185 = ($184|0)!=(0);
    if ($185) {
     $186 = HEAP32[$pOvfl>>2]|0;
     _releasePage($186);
    }
   }
   $187 = HEAP32[$rc>>2]|0;
   $188 = ($187|0)!=(0);
   if ($188) {
    label = 25;
    break;
   }
   $191 = HEAP32[$pPrior>>2]|0;
   $192 = HEAP32[$pgnoOvfl>>2]|0;
   _sqlite3Put4byte($191,$192);
   $193 = HEAP32[$pToRelease>>2]|0;
   _releasePage($193);
   $194 = HEAP32[$pOvfl>>2]|0;
   HEAP32[$pToRelease>>2] = $194;
   $195 = HEAP32[$pOvfl>>2]|0;
   $196 = ((($195)) + 56|0);
   $197 = HEAP32[$196>>2]|0;
   HEAP32[$pPrior>>2] = $197;
   $198 = HEAP32[$pPrior>>2]|0;
   _sqlite3Put4byte($198,0);
   $199 = HEAP32[$pOvfl>>2]|0;
   $200 = ((($199)) + 56|0);
   $201 = HEAP32[$200>>2]|0;
   $202 = ((($201)) + 4|0);
   HEAP32[$pPayload>>2] = $202;
   $203 = HEAP32[$pBt>>2]|0;
   $204 = ((($203)) + 36|0);
   $205 = HEAP32[$204>>2]|0;
   $206 = (($205) - 4)|0;
   HEAP32[$spaceLeft>>2] = $206;
  }
  $207 = HEAP32[$nPayload>>2]|0;
  HEAP32[$n>>2] = $207;
  $208 = HEAP32[$n>>2]|0;
  $209 = HEAP32[$spaceLeft>>2]|0;
  $210 = ($208|0)>($209|0);
  if ($210) {
   $211 = HEAP32[$spaceLeft>>2]|0;
   HEAP32[$n>>2] = $211;
  }
  $212 = HEAP32[$nSrc>>2]|0;
  $213 = ($212|0)>(0);
  if ($213) {
   $214 = HEAP32[$n>>2]|0;
   $215 = HEAP32[$nSrc>>2]|0;
   $216 = ($214|0)>($215|0);
   if ($216) {
    $217 = HEAP32[$nSrc>>2]|0;
    HEAP32[$n>>2] = $217;
   }
   $218 = HEAP32[$pPayload>>2]|0;
   $219 = HEAP32[$pSrc>>2]|0;
   $220 = HEAP32[$n>>2]|0;
   _memcpy(($218|0),($219|0),($220|0))|0;
  } else {
   $221 = HEAP32[$pPayload>>2]|0;
   $222 = HEAP32[$n>>2]|0;
   _memset(($221|0),0,($222|0))|0;
  }
  $223 = HEAP32[$n>>2]|0;
  $224 = HEAP32[$nPayload>>2]|0;
  $225 = (($224) - ($223))|0;
  HEAP32[$nPayload>>2] = $225;
  $226 = HEAP32[$n>>2]|0;
  $227 = HEAP32[$pPayload>>2]|0;
  $228 = (($227) + ($226)|0);
  HEAP32[$pPayload>>2] = $228;
  $229 = HEAP32[$n>>2]|0;
  $230 = HEAP32[$pSrc>>2]|0;
  $231 = (($230) + ($229)|0);
  HEAP32[$pSrc>>2] = $231;
  $232 = HEAP32[$n>>2]|0;
  $233 = HEAP32[$nSrc>>2]|0;
  $234 = (($233) - ($232))|0;
  HEAP32[$nSrc>>2] = $234;
  $235 = HEAP32[$n>>2]|0;
  $236 = HEAP32[$spaceLeft>>2]|0;
  $237 = (($236) - ($235))|0;
  HEAP32[$spaceLeft>>2] = $237;
 }
 if ((label|0) == 25) {
  $189 = HEAP32[$pToRelease>>2]|0;
  _releasePage($189);
  $190 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $190;
  $239 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($239|0);
 }
 else if ((label|0) == 35) {
  $238 = HEAP32[$pToRelease>>2]|0;
  _releasePage($238);
  HEAP32[$0>>2] = 0;
  $239 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($239|0);
 }
 return (0)|0;
}
function _sqlite3VdbeSorterClose($db,$pCsr) {
 $db = $db|0;
 $pCsr = $pCsr|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pSorter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $pSorter = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pCsr;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pSorter>>2] = $4;
 $5 = HEAP32[$pSorter>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$pSorter>>2]|0;
 _sqlite3VdbeSorterReset($7,$8);
 $9 = HEAP32[$pSorter>>2]|0;
 $10 = ((($9)) + 36|0);
 $11 = ((($10)) + 4|0);
 $12 = HEAP32[$11>>2]|0;
 _sqlite3_free($12);
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$pSorter>>2]|0;
 _sqlite3DbFree($13,$14);
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 16|0);
 HEAP32[$16>>2] = 0;
 STACKTOP = sp;return;
}
function _btreeCreateTable($p,$piTable,$createTabFlags) {
 $p = $p|0;
 $piTable = $piTable|0;
 $createTabFlags = $createTabFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $eType = 0, $iPtrPage = 0;
 var $pBt = 0, $pPageMove = 0, $pRoot = 0, $pgnoMove = 0, $pgnoRoot = 0, $ptfFlags = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pBt = sp + 28|0;
 $pRoot = sp + 24|0;
 $pgnoRoot = sp + 20|0;
 $rc = sp + 16|0;
 $ptfFlags = sp + 12|0;
 $pgnoMove = sp + 8|0;
 $pPageMove = sp + 4|0;
 $eType = sp + 48|0;
 $iPtrPage = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $piTable;
 HEAP32[$3>>2] = $createTabFlags;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$pBt>>2] = $6;
 $7 = HEAP32[$pBt>>2]|0;
 $8 = ((($7)) + 17|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = ($9<<24>>24)!=(0);
 $11 = HEAP32[$pBt>>2]|0;
 if ($10) {
  _invalidateAllOverflowCache($11);
  $12 = HEAP32[$1>>2]|0;
  _sqlite3BtreeGetMeta($12,4,$pgnoRoot);
  $13 = HEAP32[$pgnoRoot>>2]|0;
  $14 = (($13) + 1)|0;
  HEAP32[$pgnoRoot>>2] = $14;
  while(1) {
   $15 = HEAP32[$pgnoRoot>>2]|0;
   $16 = HEAP32[$pBt>>2]|0;
   $17 = HEAP32[$pgnoRoot>>2]|0;
   $18 = (_ptrmapPageno($16,$17)|0);
   $19 = ($15|0)==($18|0);
   if (!($19)) {
    $20 = HEAP32[$pgnoRoot>>2]|0;
    $21 = HEAP32[1536>>2]|0;
    $22 = HEAP32[$pBt>>2]|0;
    $23 = ((($22)) + 32|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = (($21>>>0) / ($24>>>0))&-1;
    $26 = (($25) + 1)|0;
    $27 = ($20|0)==($26|0);
    if (!($27)) {
     break;
    }
   }
   $28 = HEAP32[$pgnoRoot>>2]|0;
   $29 = (($28) + 1)|0;
   HEAP32[$pgnoRoot>>2] = $29;
  }
  $30 = HEAP32[$pBt>>2]|0;
  $31 = HEAP32[$pgnoRoot>>2]|0;
  $32 = (_allocateBtreePage($30,$pPageMove,$pgnoMove,$31,1)|0);
  HEAP32[$rc>>2] = $32;
  $33 = HEAP32[$rc>>2]|0;
  $34 = ($33|0)!=(0);
  if ($34) {
   $35 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $35;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
  $36 = HEAP32[$pgnoMove>>2]|0;
  $37 = HEAP32[$pgnoRoot>>2]|0;
  $38 = ($36|0)!=($37|0);
  if ($38) {
   HEAP8[$eType>>0] = 0;
   HEAP32[$iPtrPage>>2] = 0;
   $39 = HEAP32[$pBt>>2]|0;
   $40 = (_saveAllCursors($39,0,0)|0);
   HEAP32[$rc>>2] = $40;
   $41 = HEAP32[$pPageMove>>2]|0;
   _releasePage($41);
   $42 = HEAP32[$rc>>2]|0;
   $43 = ($42|0)!=(0);
   if ($43) {
    $44 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $44;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $45 = HEAP32[$pBt>>2]|0;
   $46 = HEAP32[$pgnoRoot>>2]|0;
   $47 = (_btreeGetPage($45,$46,$pRoot,0)|0);
   HEAP32[$rc>>2] = $47;
   $48 = HEAP32[$rc>>2]|0;
   $49 = ($48|0)!=(0);
   if ($49) {
    $50 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $50;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $51 = HEAP32[$pBt>>2]|0;
   $52 = HEAP32[$pgnoRoot>>2]|0;
   $53 = (_ptrmapGet($51,$52,$eType,$iPtrPage)|0);
   HEAP32[$rc>>2] = $53;
   $54 = HEAP8[$eType>>0]|0;
   $55 = $54&255;
   $56 = ($55|0)==(1);
   if ($56) {
    label = 15;
   } else {
    $57 = HEAP8[$eType>>0]|0;
    $58 = $57&255;
    $59 = ($58|0)==(2);
    if ($59) {
     label = 15;
    }
   }
   if ((label|0) == 15) {
    $60 = (_sqlite3CorruptError(66598)|0);
    HEAP32[$rc>>2] = $60;
   }
   $61 = HEAP32[$rc>>2]|0;
   $62 = ($61|0)!=(0);
   if ($62) {
    $63 = HEAP32[$pRoot>>2]|0;
    _releasePage($63);
    $64 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $64;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $65 = HEAP32[$pBt>>2]|0;
   $66 = HEAP32[$pRoot>>2]|0;
   $67 = HEAP8[$eType>>0]|0;
   $68 = HEAP32[$iPtrPage>>2]|0;
   $69 = HEAP32[$pgnoMove>>2]|0;
   $70 = (_relocatePage($65,$66,$67,$68,$69,0)|0);
   HEAP32[$rc>>2] = $70;
   $71 = HEAP32[$pRoot>>2]|0;
   _releasePage($71);
   $72 = HEAP32[$rc>>2]|0;
   $73 = ($72|0)!=(0);
   if ($73) {
    $74 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $74;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $75 = HEAP32[$pBt>>2]|0;
   $76 = HEAP32[$pgnoRoot>>2]|0;
   $77 = (_btreeGetPage($75,$76,$pRoot,0)|0);
   HEAP32[$rc>>2] = $77;
   $78 = HEAP32[$rc>>2]|0;
   $79 = ($78|0)!=(0);
   if ($79) {
    $80 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $80;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
   $81 = HEAP32[$pRoot>>2]|0;
   $82 = ((($81)) + 72|0);
   $83 = HEAP32[$82>>2]|0;
   $84 = (_sqlite3PagerWrite($83)|0);
   HEAP32[$rc>>2] = $84;
   $85 = HEAP32[$rc>>2]|0;
   $86 = ($85|0)!=(0);
   if ($86) {
    $87 = HEAP32[$pRoot>>2]|0;
    _releasePage($87);
    $88 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $88;
    $117 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($117|0);
   }
  } else {
   $89 = HEAP32[$pPageMove>>2]|0;
   HEAP32[$pRoot>>2] = $89;
  }
  $90 = HEAP32[$pBt>>2]|0;
  $91 = HEAP32[$pgnoRoot>>2]|0;
  _ptrmapPut($90,$91,1,0,$rc);
  $92 = HEAP32[$rc>>2]|0;
  $93 = ($92|0)!=(0);
  if ($93) {
   $94 = HEAP32[$pRoot>>2]|0;
   _releasePage($94);
   $95 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $95;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
  $96 = HEAP32[$1>>2]|0;
  $97 = HEAP32[$pgnoRoot>>2]|0;
  $98 = (_sqlite3BtreeUpdateMeta($96,4,$97)|0);
  HEAP32[$rc>>2] = $98;
  $99 = HEAP32[$rc>>2]|0;
  $100 = ($99|0)!=(0);
  if ($100) {
   $101 = HEAP32[$pRoot>>2]|0;
   _releasePage($101);
   $102 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $102;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
 } else {
  $103 = (_allocateBtreePage($11,$pRoot,$pgnoRoot,1,0)|0);
  HEAP32[$rc>>2] = $103;
  $104 = HEAP32[$rc>>2]|0;
  $105 = ($104|0)!=(0);
  if ($105) {
   $106 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $106;
   $117 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($117|0);
  }
 }
 $107 = HEAP32[$3>>2]|0;
 $108 = $107 & 1;
 $109 = ($108|0)!=(0);
 if ($109) {
  HEAP32[$ptfFlags>>2] = 13;
 } else {
  HEAP32[$ptfFlags>>2] = 10;
 }
 $110 = HEAP32[$pRoot>>2]|0;
 $111 = HEAP32[$ptfFlags>>2]|0;
 _zeroPage($110,$111);
 $112 = HEAP32[$pRoot>>2]|0;
 $113 = ((($112)) + 72|0);
 $114 = HEAP32[$113>>2]|0;
 _sqlite3PagerUnref($114);
 $115 = HEAP32[$pgnoRoot>>2]|0;
 $116 = HEAP32[$2>>2]|0;
 HEAP32[$116>>2] = $115;
 HEAP32[$0>>2] = 0;
 $117 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($117|0);
}
function _btreeCursor($p,$iTable,$wrFlag,$pKeyInfo,$pCur) {
 $p = $p|0;
 $iTable = $iTable|0;
 $wrFlag = $wrFlag|0;
 $pKeyInfo = $pKeyInfo|0;
 $pCur = $pCur|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $pBt = 0, $pX = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 $pBt = sp + 4|0;
 $pX = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $iTable;
 HEAP32[$3>>2] = $wrFlag;
 HEAP32[$4>>2] = $pKeyInfo;
 HEAP32[$5>>2] = $pCur;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pBt>>2] = $8;
 $9 = HEAP32[$3>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$pBt>>2]|0;
  _allocateTempSpace($11);
  $12 = HEAP32[$pBt>>2]|0;
  $13 = ((($12)) + 80|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = ($14|0)==(0|0);
  if ($15) {
   HEAP32[$0>>2] = 7;
   $81 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($81|0);
  }
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = ($16|0)==(1);
 if ($17) {
  $18 = HEAP32[$pBt>>2]|0;
  $19 = (_btreePagecount($18)|0);
  $20 = ($19|0)==(0);
  if ($20) {
   HEAP32[$2>>2] = 0;
  }
 }
 $21 = HEAP32[$2>>2]|0;
 $22 = HEAP32[$5>>2]|0;
 $23 = ((($22)) + 52|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$5>>2]|0;
 $25 = ((($24)) + 68|0);
 HEAP8[$25>>0] = -1;
 $26 = HEAP32[$4>>2]|0;
 $27 = HEAP32[$5>>2]|0;
 $28 = ((($27)) + 72|0);
 HEAP32[$28>>2] = $26;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$5>>2]|0;
 HEAP32[$30>>2] = $29;
 $31 = HEAP32[$pBt>>2]|0;
 $32 = HEAP32[$5>>2]|0;
 $33 = ((($32)) + 4|0);
 HEAP32[$33>>2] = $31;
 $34 = HEAP32[$3>>2]|0;
 $35 = ($34|0)!=(0);
 $36 = $35 ? 1 : 0;
 $37 = $36&255;
 $38 = HEAP32[$5>>2]|0;
 $39 = ((($38)) + 64|0);
 HEAP8[$39>>0] = $37;
 $40 = HEAP32[$3>>2]|0;
 $41 = ($40|0)!=(0);
 $42 = $41 ? 0 : 2;
 $43 = $42&255;
 $44 = HEAP32[$5>>2]|0;
 $45 = ((($44)) + 65|0);
 HEAP8[$45>>0] = $43;
 $46 = HEAP32[$pBt>>2]|0;
 $47 = ((($46)) + 8|0);
 $48 = HEAP32[$47>>2]|0;
 HEAP32[$pX>>2] = $48;
 while(1) {
  $49 = HEAP32[$pX>>2]|0;
  $50 = ($49|0)!=(0|0);
  if (!($50)) {
   break;
  }
  $51 = HEAP32[$pX>>2]|0;
  $52 = ((($51)) + 52|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = HEAP32[$2>>2]|0;
  $55 = ($53|0)==($54|0);
  if ($55) {
   $56 = HEAP32[$pX>>2]|0;
   $57 = ((($56)) + 64|0);
   $58 = HEAP8[$57>>0]|0;
   $59 = $58&255;
   $60 = $59 | 32;
   $61 = $60&255;
   HEAP8[$57>>0] = $61;
   $62 = HEAP32[$5>>2]|0;
   $63 = ((($62)) + 64|0);
   $64 = HEAP8[$63>>0]|0;
   $65 = $64&255;
   $66 = $65 | 32;
   $67 = $66&255;
   HEAP8[$63>>0] = $67;
  }
  $68 = HEAP32[$pX>>2]|0;
  $69 = ((($68)) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  HEAP32[$pX>>2] = $70;
 }
 $71 = HEAP32[$pBt>>2]|0;
 $72 = ((($71)) + 8|0);
 $73 = HEAP32[$72>>2]|0;
 $74 = HEAP32[$5>>2]|0;
 $75 = ((($74)) + 8|0);
 HEAP32[$75>>2] = $73;
 $76 = HEAP32[$5>>2]|0;
 $77 = HEAP32[$pBt>>2]|0;
 $78 = ((($77)) + 8|0);
 HEAP32[$78>>2] = $76;
 $79 = HEAP32[$5>>2]|0;
 $80 = ((($79)) + 66|0);
 HEAP8[$80>>0] = 0;
 HEAP32[$0>>2] = 0;
 $81 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($81|0);
}
function _allocateTempSpace($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 32|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3PageMalloc($7)|0);
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 80|0);
 HEAP32[$10>>2] = $8;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 80|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 80|0);
 $17 = HEAP32[$16>>2]|0;
 ;HEAP8[$17>>0]=0|0;HEAP8[$17+1>>0]=0|0;HEAP8[$17+2>>0]=0|0;HEAP8[$17+3>>0]=0|0;HEAP8[$17+4>>0]=0|0;HEAP8[$17+5>>0]=0|0;HEAP8[$17+6>>0]=0|0;HEAP8[$17+7>>0]=0|0;
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 80|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ((($20)) + 4|0);
 HEAP32[$19>>2] = $21;
 STACKTOP = sp;return;
}
function _sqlite3BtreeCursorSize() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 200;
}
function _sqlite3BtreeCursorZero($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 dest=$1; stop=dest+68|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return;
}
function _sqlite3PagerOpenSavepoint($pPager,$nSavepoint) {
 $pPager = $pPager|0;
 $nSavepoint = $nSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $nSavepoint;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 104|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)>($6|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 6|0);
  $10 = HEAP8[$9>>0]|0;
  $11 = $10&255;
  $12 = ($11|0)!=(0);
  if ($12) {
   $13 = HEAP32[$1>>2]|0;
   $14 = HEAP32[$2>>2]|0;
   $15 = (_pagerOpenSavepoint($13,$14)|0);
   HEAP32[$0>>2] = $15;
   $16 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($16|0);
  }
 }
 HEAP32[$0>>2] = 0;
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _pagerOpenSavepoint($pPager,$nSavepoint) {
 $pPager = $pPager|0;
 $nSavepoint = $nSavepoint|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aNew = 0, $ii = 0, $nCurrent = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $rc = sp + 12|0;
 $nCurrent = sp + 8|0;
 $ii = sp + 4|0;
 $aNew = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $nSavepoint;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 104|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$nCurrent>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 100|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($9*48)|0;
 $11 = (_sqlite3Realloc($8,$10,0)|0);
 HEAP32[$aNew>>2] = $11;
 $12 = HEAP32[$aNew>>2]|0;
 $13 = ($12|0)!=(0|0);
 if (!($13)) {
  HEAP32[$0>>2] = 7;
  $116 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 $14 = HEAP32[$nCurrent>>2]|0;
 $15 = HEAP32[$aNew>>2]|0;
 $16 = (($15) + (($14*48)|0)|0);
 $17 = HEAP32[$2>>2]|0;
 $18 = HEAP32[$nCurrent>>2]|0;
 $19 = (($17) - ($18))|0;
 $20 = ($19*48)|0;
 _memset(($16|0),0,($20|0))|0;
 $21 = HEAP32[$aNew>>2]|0;
 $22 = HEAP32[$1>>2]|0;
 $23 = ((($22)) + 100|0);
 HEAP32[$23>>2] = $21;
 $24 = HEAP32[$nCurrent>>2]|0;
 HEAP32[$ii>>2] = $24;
 while(1) {
  $25 = HEAP32[$ii>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = ($25|0)<($26|0);
  if (!($27)) {
   label = 14;
   break;
  }
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 28|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = HEAP32[$ii>>2]|0;
  $32 = HEAP32[$aNew>>2]|0;
  $33 = (($32) + (($31*48)|0)|0);
  $34 = ((($33)) + 20|0);
  HEAP32[$34>>2] = $30;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 68|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)!=(0|0);
  if ($39) {
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 80|0);
   $42 = $41;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = ($47|0)>(0);
   $49 = ($44>>>0)>(0);
   $50 = ($47|0)==(0);
   $51 = $50 & $49;
   $52 = $48 | $51;
   if ($52) {
    $53 = HEAP32[$1>>2]|0;
    $54 = ((($53)) + 80|0);
    $55 = $54;
    $56 = $55;
    $57 = HEAP32[$56>>2]|0;
    $58 = (($55) + 4)|0;
    $59 = $58;
    $60 = HEAP32[$59>>2]|0;
    $61 = HEAP32[$ii>>2]|0;
    $62 = HEAP32[$aNew>>2]|0;
    $63 = (($62) + (($61*48)|0)|0);
    $64 = $63;
    $65 = $64;
    HEAP32[$65>>2] = $57;
    $66 = (($64) + 4)|0;
    $67 = $66;
    HEAP32[$67>>2] = $60;
   } else {
    label = 8;
   }
  } else {
   label = 8;
  }
  if ((label|0) == 8) {
   label = 0;
   $68 = HEAP32[$1>>2]|0;
   $69 = ((($68)) + 156|0);
   $70 = HEAP32[$69>>2]|0;
   $71 = HEAP32[$ii>>2]|0;
   $72 = HEAP32[$aNew>>2]|0;
   $73 = (($72) + (($71*48)|0)|0);
   $74 = $73;
   $75 = $74;
   HEAP32[$75>>2] = $70;
   $76 = (($74) + 4)|0;
   $77 = $76;
   HEAP32[$77>>2] = 0;
  }
  $78 = HEAP32[$1>>2]|0;
  $79 = ((($78)) + 56|0);
  $80 = HEAP32[$79>>2]|0;
  $81 = HEAP32[$ii>>2]|0;
  $82 = HEAP32[$aNew>>2]|0;
  $83 = (($82) + (($81*48)|0)|0);
  $84 = ((($83)) + 24|0);
  HEAP32[$84>>2] = $80;
  $85 = HEAP32[$1>>2]|0;
  $86 = ((($85)) + 28|0);
  $87 = HEAP32[$86>>2]|0;
  $88 = (_sqlite3BitvecCreate($87)|0);
  $89 = HEAP32[$ii>>2]|0;
  $90 = HEAP32[$aNew>>2]|0;
  $91 = (($90) + (($89*48)|0)|0);
  $92 = ((($91)) + 16|0);
  HEAP32[$92>>2] = $88;
  $93 = HEAP32[$ii>>2]|0;
  $94 = HEAP32[$aNew>>2]|0;
  $95 = (($94) + (($93*48)|0)|0);
  $96 = ((($95)) + 16|0);
  $97 = HEAP32[$96>>2]|0;
  $98 = ($97|0)!=(0|0);
  if (!($98)) {
   label = 10;
   break;
  }
  $99 = HEAP32[$1>>2]|0;
  $100 = (_sqlite3PagerUseWal($99)|0);
  $101 = ($100|0)!=(0);
  if ($101) {
   $102 = HEAP32[$1>>2]|0;
   $103 = ((($102)) + 216|0);
   $104 = HEAP32[$103>>2]|0;
   $105 = HEAP32[$ii>>2]|0;
   $106 = HEAP32[$aNew>>2]|0;
   $107 = (($106) + (($105*48)|0)|0);
   $108 = ((($107)) + 28|0);
   _sqlite3WalSavepoint($104,$108);
  }
  $109 = HEAP32[$ii>>2]|0;
  $110 = (($109) + 1)|0;
  $111 = HEAP32[$1>>2]|0;
  $112 = ((($111)) + 104|0);
  HEAP32[$112>>2] = $110;
  $113 = HEAP32[$ii>>2]|0;
  $114 = (($113) + 1)|0;
  HEAP32[$ii>>2] = $114;
 }
 if ((label|0) == 10) {
  HEAP32[$0>>2] = 7;
  $116 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 else if ((label|0) == 14) {
  $115 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $115;
  $116 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($116|0);
 }
 return (0)|0;
}
function _sqlite3WalSavepoint($pWal,$aWalData) {
 $pWal = $pWal|0;
 $aWalData = $aWalData|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pWal;
 HEAP32[$1>>2] = $aWalData;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 52|0);
 $4 = ((($3)) + 16|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$6>>2] = $5;
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 52|0);
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 4|0);
 HEAP32[$12>>2] = $10;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = ((($14)) + 24|0);
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 8|0);
 HEAP32[$19>>2] = $17;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 112|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 12|0);
 HEAP32[$24>>2] = $22;
 STACKTOP = sp;return;
}
function _newDatabase($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $9 = 0, $data = 0, $pP1 = 0, $rc = 0, dest = 0, label = 0, sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pP1 = sp + 8|0;
 $data = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pBt;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4>>>0)>(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pP1>>2] = $8;
 $9 = HEAP32[$pP1>>2]|0;
 $10 = ((($9)) + 56|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$data>>2] = $11;
 $12 = HEAP32[$pP1>>2]|0;
 $13 = ((($12)) + 72|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3PagerWrite($14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 if ($17) {
  $18 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $18;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 } else {
  $19 = HEAP32[$data>>2]|0;
  dest=$19; src=41796; stop=dest+16|0; do { HEAP8[dest>>0]=HEAP8[src>>0]|0; dest=dest+1|0; src=src+1|0; } while ((dest|0) < (stop|0));
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = $22 >>> 8;
  $24 = $23 & 255;
  $25 = $24&255;
  $26 = HEAP32[$data>>2]|0;
  $27 = ((($26)) + 16|0);
  HEAP8[$27>>0] = $25;
  $28 = HEAP32[$1>>2]|0;
  $29 = ((($28)) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = $30 >>> 16;
  $32 = $31 & 255;
  $33 = $32&255;
  $34 = HEAP32[$data>>2]|0;
  $35 = ((($34)) + 17|0);
  HEAP8[$35>>0] = $33;
  $36 = HEAP32[$data>>2]|0;
  $37 = ((($36)) + 18|0);
  HEAP8[$37>>0] = 1;
  $38 = HEAP32[$data>>2]|0;
  $39 = ((($38)) + 19|0);
  HEAP8[$39>>0] = 1;
  $40 = HEAP32[$1>>2]|0;
  $41 = ((($40)) + 32|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = HEAP32[$1>>2]|0;
  $44 = ((($43)) + 36|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($42) - ($45))|0;
  $47 = $46&255;
  $48 = HEAP32[$data>>2]|0;
  $49 = ((($48)) + 20|0);
  HEAP8[$49>>0] = $47;
  $50 = HEAP32[$data>>2]|0;
  $51 = ((($50)) + 21|0);
  HEAP8[$51>>0] = 64;
  $52 = HEAP32[$data>>2]|0;
  $53 = ((($52)) + 22|0);
  HEAP8[$53>>0] = 32;
  $54 = HEAP32[$data>>2]|0;
  $55 = ((($54)) + 23|0);
  HEAP8[$55>>0] = 32;
  $56 = HEAP32[$data>>2]|0;
  $57 = ((($56)) + 24|0);
  dest=$57; stop=dest+76|0; do { HEAP8[dest>>0]=0|0; dest=dest+1|0; } while ((dest|0) < (stop|0));
  $58 = HEAP32[$pP1>>2]|0;
  _zeroPage($58,13);
  $59 = HEAP32[$1>>2]|0;
  $60 = ((($59)) + 22|0);
  $61 = HEAP16[$60>>1]|0;
  $62 = $61&65535;
  $63 = $62 | 2;
  $64 = $63&65535;
  HEAP16[$60>>1] = $64;
  $65 = HEAP32[$data>>2]|0;
  $66 = ((($65)) + 52|0);
  $67 = HEAP32[$1>>2]|0;
  $68 = ((($67)) + 17|0);
  $69 = HEAP8[$68>>0]|0;
  $70 = $69&255;
  _sqlite3Put4byte($66,$70);
  $71 = HEAP32[$data>>2]|0;
  $72 = ((($71)) + 64|0);
  $73 = HEAP32[$1>>2]|0;
  $74 = ((($73)) + 18|0);
  $75 = HEAP8[$74>>0]|0;
  $76 = $75&255;
  _sqlite3Put4byte($72,$76);
  $77 = HEAP32[$1>>2]|0;
  $78 = ((($77)) + 44|0);
  HEAP32[$78>>2] = 1;
  $79 = HEAP32[$data>>2]|0;
  $80 = ((($79)) + 31|0);
  HEAP8[$80>>0] = 1;
  HEAP32[$0>>2] = 0;
  $81 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($81|0);
 }
 return (0)|0;
}
function _putVarint64($p,$0,$1) {
 $p = $p|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $buf = 0;
 var $i = 0, $j = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $4 = sp;
 $i = sp + 16|0;
 $j = sp + 12|0;
 $n = sp + 8|0;
 $buf = sp + 28|0;
 HEAP32[$3>>2] = $p;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = $4;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = $14 & -16777216;
 $16 = (0)!=(0);
 $17 = ($15|0)!=(0);
 $18 = $16 | $17;
 if ($18) {
  $19 = $4;
  $20 = $19;
  $21 = HEAP32[$20>>2]|0;
  $22 = (($19) + 4)|0;
  $23 = $22;
  $24 = HEAP32[$23>>2]|0;
  $25 = $21&255;
  $26 = HEAP32[$3>>2]|0;
  $27 = ((($26)) + 8|0);
  HEAP8[$27>>0] = $25;
  $28 = $4;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = (($28) + 4)|0;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (_bitshift64Lshr(($30|0),($33|0),8)|0);
  $35 = tempRet0;
  $36 = $4;
  $37 = $36;
  HEAP32[$37>>2] = $34;
  $38 = (($36) + 4)|0;
  $39 = $38;
  HEAP32[$39>>2] = $35;
  HEAP32[$i>>2] = 7;
  while(1) {
   $40 = HEAP32[$i>>2]|0;
   $41 = ($40|0)>=(0);
   if (!($41)) {
    break;
   }
   $42 = $4;
   $43 = $42;
   $44 = HEAP32[$43>>2]|0;
   $45 = (($42) + 4)|0;
   $46 = $45;
   $47 = HEAP32[$46>>2]|0;
   $48 = $44 & 127;
   $49 = $48 | 128;
   $50 = $49&255;
   $51 = HEAP32[$i>>2]|0;
   $52 = HEAP32[$3>>2]|0;
   $53 = (($52) + ($51)|0);
   HEAP8[$53>>0] = $50;
   $54 = $4;
   $55 = $54;
   $56 = HEAP32[$55>>2]|0;
   $57 = (($54) + 4)|0;
   $58 = $57;
   $59 = HEAP32[$58>>2]|0;
   $60 = (_bitshift64Lshr(($56|0),($59|0),7)|0);
   $61 = tempRet0;
   $62 = $4;
   $63 = $62;
   HEAP32[$63>>2] = $60;
   $64 = (($62) + 4)|0;
   $65 = $64;
   HEAP32[$65>>2] = $61;
   $66 = HEAP32[$i>>2]|0;
   $67 = (($66) + -1)|0;
   HEAP32[$i>>2] = $67;
  }
  HEAP32[$2>>2] = 9;
  $120 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($120|0);
 }
 HEAP32[$n>>2] = 0;
 while(1) {
  $68 = $4;
  $69 = $68;
  $70 = HEAP32[$69>>2]|0;
  $71 = (($68) + 4)|0;
  $72 = $71;
  $73 = HEAP32[$72>>2]|0;
  $74 = $70 & 127;
  $75 = $74 | 128;
  $76 = $75&255;
  $77 = HEAP32[$n>>2]|0;
  $78 = (($77) + 1)|0;
  HEAP32[$n>>2] = $78;
  $79 = (($buf) + ($77)|0);
  HEAP8[$79>>0] = $76;
  $80 = $4;
  $81 = $80;
  $82 = HEAP32[$81>>2]|0;
  $83 = (($80) + 4)|0;
  $84 = $83;
  $85 = HEAP32[$84>>2]|0;
  $86 = (_bitshift64Lshr(($82|0),($85|0),7)|0);
  $87 = tempRet0;
  $88 = $4;
  $89 = $88;
  HEAP32[$89>>2] = $86;
  $90 = (($88) + 4)|0;
  $91 = $90;
  HEAP32[$91>>2] = $87;
  $92 = $4;
  $93 = $92;
  $94 = HEAP32[$93>>2]|0;
  $95 = (($92) + 4)|0;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = ($94|0)!=(0);
  $99 = ($97|0)!=(0);
  $100 = $98 | $99;
  if (!($100)) {
   break;
  }
 }
 $101 = HEAP8[$buf>>0]|0;
 $102 = $101&255;
 $103 = $102 & 127;
 $104 = $103&255;
 HEAP8[$buf>>0] = $104;
 HEAP32[$i>>2] = 0;
 $105 = HEAP32[$n>>2]|0;
 $106 = (($105) - 1)|0;
 HEAP32[$j>>2] = $106;
 while(1) {
  $107 = HEAP32[$j>>2]|0;
  $108 = ($107|0)>=(0);
  if (!($108)) {
   break;
  }
  $109 = HEAP32[$j>>2]|0;
  $110 = (($buf) + ($109)|0);
  $111 = HEAP8[$110>>0]|0;
  $112 = HEAP32[$i>>2]|0;
  $113 = HEAP32[$3>>2]|0;
  $114 = (($113) + ($112)|0);
  HEAP8[$114>>0] = $111;
  $115 = HEAP32[$j>>2]|0;
  $116 = (($115) + -1)|0;
  HEAP32[$j>>2] = $116;
  $117 = HEAP32[$i>>2]|0;
  $118 = (($117) + 1)|0;
  HEAP32[$i>>2] = $118;
 }
 $119 = HEAP32[$n>>2]|0;
 HEAP32[$2>>2] = $119;
 $120 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($120|0);
}
function _vdbeMemFromBtreeResize($pCur,$offset,$amt,$key,$pMem) {
 $pCur = $pCur|0;
 $offset = $offset|0;
 $amt = $amt|0;
 $key = $key|0;
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $offset;
 HEAP32[$2>>2] = $amt;
 HEAP32[$3>>2] = $key;
 HEAP32[$4>>2] = $pMem;
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 8|0);
 HEAP16[$6>>1] = 1;
 $7 = HEAP32[$4>>2]|0;
 $8 = HEAP32[$2>>2]|0;
 $9 = (($8) + 2)|0;
 $10 = (_sqlite3VdbeMemClearAndResize($7,$9)|0);
 HEAP32[$rc>>2] = $10;
 $11 = (0)==($10|0);
 if (!($11)) {
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 $12 = HEAP32[$3>>2]|0;
 $13 = ($12|0)!=(0);
 $14 = HEAP32[$0>>2]|0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$4>>2]|0;
 $18 = ((($17)) + 16|0);
 $19 = HEAP32[$18>>2]|0;
 if ($13) {
  $20 = (_sqlite3BtreeKey($14,$15,$16,$19)|0);
  HEAP32[$rc>>2] = $20;
 } else {
  $21 = (_sqlite3BtreeData($14,$15,$16,$19)|0);
  HEAP32[$rc>>2] = $21;
 }
 $22 = HEAP32[$rc>>2]|0;
 $23 = ($22|0)==(0);
 if ($23) {
  $24 = HEAP32[$2>>2]|0;
  $25 = HEAP32[$4>>2]|0;
  $26 = ((($25)) + 16|0);
  $27 = HEAP32[$26>>2]|0;
  $28 = (($27) + ($24)|0);
  HEAP8[$28>>0] = 0;
  $29 = HEAP32[$2>>2]|0;
  $30 = (($29) + 1)|0;
  $31 = HEAP32[$4>>2]|0;
  $32 = ((($31)) + 16|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($30)|0);
  HEAP8[$34>>0] = 0;
  $35 = HEAP32[$4>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP16[$36>>1] = 528;
  $37 = HEAP32[$2>>2]|0;
  $38 = HEAP32[$4>>2]|0;
  $39 = ((($38)) + 12|0);
  HEAP32[$39>>2] = $37;
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 } else {
  $40 = HEAP32[$4>>2]|0;
  _sqlite3VdbeMemRelease($40);
  $41 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($41|0);
 }
 return (0)|0;
}
function _fetchPayload($pCur,$pAmt) {
 $pCur = $pCur|0;
 $pAmt = $pAmt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $amt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $amt = sp;
 HEAP32[$0>>2] = $pCur;
 HEAP32[$1>>2] = $pAmt;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 68|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 120|0);
 $8 = (($7) + ($5<<2)|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($9)) + 60|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $11;
 $17 = $15;
 $18 = (($16) - ($17))|0;
 HEAP32[$amt>>2] = $18;
 $19 = HEAP32[$0>>2]|0;
 $20 = ((($19)) + 16|0);
 $21 = ((($20)) + 16|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = HEAP32[$amt>>2]|0;
 $25 = ($23>>>0)<($24>>>0);
 if ($25) {
  $26 = HEAP32[$0>>2]|0;
  $27 = ((($26)) + 16|0);
  $28 = ((($27)) + 16|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  HEAP32[$amt>>2] = $30;
 }
 $31 = HEAP32[$amt>>2]|0;
 $32 = HEAP32[$1>>2]|0;
 HEAP32[$32>>2] = $31;
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 16|0);
 $35 = ((($34)) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 STACKTOP = sp;return ($36|0);
}
function _handleDeferredMoveto($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, $res = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $res = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 40|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3BtreeMovetoUnpacked($4,0,$9,$12,0,$res)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)!=(0);
 if ($15) {
  $16 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $16;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 $17 = HEAP32[$res>>2]|0;
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = (_sqlite3CorruptError(73491)|0);
  HEAP32[$0>>2] = $19;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 3|0);
  HEAP8[$21>>0] = 0;
  $22 = HEAP32[$1>>2]|0;
  $23 = ((($22)) + 56|0);
  HEAP32[$23>>2] = 0;
  HEAP32[$0>>2] = 0;
  $24 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _sqlite3BlobCompare($pB1,$pB2) {
 $pB1 = $pB1|0;
 $pB2 = $pB2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $9 = 0, $c = 0, $n1 = 0, $n2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $c = sp + 8|0;
 $n1 = sp + 4|0;
 $n2 = sp;
 HEAP32[$1>>2] = $pB1;
 HEAP32[$2>>2] = $pB2;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$n1>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$n2>>2] = $8;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 8|0);
 $11 = HEAP16[$10>>1]|0;
 $12 = $11&65535;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = $15&65535;
 $17 = $12 | $16;
 $18 = $17 & 16384;
 $19 = ($18|0)!=(0);
 $20 = HEAP32[$1>>2]|0;
 if (!($19)) {
  $65 = ((($20)) + 16|0);
  $66 = HEAP32[$65>>2]|0;
  $67 = HEAP32[$2>>2]|0;
  $68 = ((($67)) + 16|0);
  $69 = HEAP32[$68>>2]|0;
  $70 = HEAP32[$n1>>2]|0;
  $71 = HEAP32[$n2>>2]|0;
  $72 = ($70|0)>($71|0);
  $73 = HEAP32[$n2>>2]|0;
  $74 = HEAP32[$n1>>2]|0;
  $75 = $72 ? $73 : $74;
  $76 = (_memcmp($66,$69,$75)|0);
  HEAP32[$c>>2] = $76;
  $77 = HEAP32[$c>>2]|0;
  $78 = ($77|0)!=(0);
  if ($78) {
   $79 = HEAP32[$c>>2]|0;
   HEAP32[$0>>2] = $79;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  } else {
   $80 = HEAP32[$n1>>2]|0;
   $81 = HEAP32[$n2>>2]|0;
   $82 = (($80) - ($81))|0;
   HEAP32[$0>>2] = $82;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  }
 }
 $21 = ((($20)) + 8|0);
 $22 = HEAP16[$21>>1]|0;
 $23 = $22&65535;
 $24 = HEAP32[$2>>2]|0;
 $25 = ((($24)) + 8|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $23 & $27;
 $29 = $28 & 16384;
 $30 = ($29|0)!=(0);
 $31 = HEAP32[$1>>2]|0;
 if ($30) {
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$2>>2]|0;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) - ($34))|0;
  HEAP32[$0>>2] = $35;
  $83 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($83|0);
 }
 $36 = ((($31)) + 8|0);
 $37 = HEAP16[$36>>1]|0;
 $38 = $37&65535;
 $39 = $38 & 16384;
 $40 = ($39|0)!=(0);
 if ($40) {
  $41 = HEAP32[$2>>2]|0;
  $42 = ((($41)) + 16|0);
  $43 = HEAP32[$42>>2]|0;
  $44 = HEAP32[$2>>2]|0;
  $45 = ((($44)) + 12|0);
  $46 = HEAP32[$45>>2]|0;
  $47 = (_isAllZero($43,$46)|0);
  $48 = ($47|0)!=(0);
  if ($48) {
   $49 = HEAP32[$1>>2]|0;
   $50 = HEAP32[$49>>2]|0;
   $51 = HEAP32[$n2>>2]|0;
   $52 = (($50) - ($51))|0;
   HEAP32[$0>>2] = $52;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  } else {
   HEAP32[$0>>2] = -1;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  }
 } else {
  $53 = HEAP32[$1>>2]|0;
  $54 = ((($53)) + 16|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = HEAP32[$1>>2]|0;
  $57 = ((($56)) + 12|0);
  $58 = HEAP32[$57>>2]|0;
  $59 = (_isAllZero($55,$58)|0);
  $60 = ($59|0)!=(0);
  if ($60) {
   $61 = HEAP32[$n1>>2]|0;
   $62 = HEAP32[$2>>2]|0;
   $63 = HEAP32[$62>>2]|0;
   $64 = (($61) - ($63))|0;
   HEAP32[$0>>2] = $64;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  } else {
   HEAP32[$0>>2] = 1;
   $83 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($83|0);
  }
 }
 return (0)|0;
}
function _computeNumericType($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = HEAP32[$1>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 10|0);
 $11 = HEAP8[$10>>0]|0;
 $12 = (_sqlite3AtoF($4,$5,$8,$11)|0);
 $13 = ($12|0)==(0);
 if ($13) {
  HEAP16[$0>>1] = 0;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 }
 $14 = HEAP32[$1>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP32[$15>>2]|0;
 $17 = HEAP32[$1>>2]|0;
 $18 = HEAP32[$1>>2]|0;
 $19 = ((($18)) + 12|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$1>>2]|0;
 $22 = ((($21)) + 10|0);
 $23 = HEAP8[$22>>0]|0;
 $24 = (_sqlite3Atoi64($16,$17,$20,$23)|0);
 $25 = ($24|0)==(0);
 if ($25) {
  HEAP16[$0>>1] = 4;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 } else {
  HEAP16[$0>>1] = 8;
  $26 = HEAP16[$0>>1]|0;
  STACKTOP = sp;return ($26|0);
 }
 return (0)|0;
}
function _vdbeMemAddTerminator($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $pMem;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (($5) + 2)|0;
 $7 = (_sqlite3VdbeMemGrow($2,$6,1)|0);
 $8 = ($7|0)!=(0);
 if ($8) {
  HEAP32[$0>>2] = 7;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 } else {
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 12|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 16|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($11)|0);
  HEAP8[$15>>0] = 0;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 12|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + 1)|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 16|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = (($22) + ($19)|0);
  HEAP8[$23>>0] = 0;
  $24 = HEAP32[$1>>2]|0;
  $25 = ((($24)) + 8|0);
  $26 = HEAP16[$25>>1]|0;
  $27 = $26&65535;
  $28 = $27 | 512;
  $29 = $28&65535;
  HEAP16[$25>>1] = $29;
  HEAP32[$0>>2] = 0;
  $30 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($30|0);
 }
 return (0)|0;
}
function _out2PrereleaseWithClear($pOut) {
 $pOut = $pOut|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pOut;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($1);
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 8|0);
 HEAP16[$3>>1] = 4;
 $4 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _closeAllCursors($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pDel = 0, $pFrame = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pFrame = sp + 4|0;
 $pDel = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 184|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 184|0);
  $7 = HEAP32[$6>>2]|0;
  HEAP32[$pFrame>>2] = $7;
  while(1) {
   $8 = HEAP32[$pFrame>>2]|0;
   $9 = ((($8)) + 4|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   $12 = HEAP32[$pFrame>>2]|0;
   if (!($11)) {
    break;
   }
   $13 = ((($12)) + 4|0);
   $14 = HEAP32[$13>>2]|0;
   HEAP32[$pFrame>>2] = $14;
  }
  (_sqlite3VdbeFrameRestore($12)|0);
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 184|0);
  HEAP32[$16>>2] = 0;
  $17 = HEAP32[$0>>2]|0;
  $18 = ((($17)) + 192|0);
  HEAP32[$18>>2] = 0;
 }
 $19 = HEAP32[$0>>2]|0;
 _closeCursorsInFrame($19);
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 92|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = ($22|0)!=(0|0);
 if ($23) {
  $24 = HEAP32[$0>>2]|0;
  $25 = ((($24)) + 92|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 24|0);
  $29 = HEAP32[$28>>2]|0;
  _releaseMemArray($26,$29);
 }
 while(1) {
  $30 = HEAP32[$0>>2]|0;
  $31 = ((($30)) + 188|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)!=(0|0);
  $34 = HEAP32[$0>>2]|0;
  if (!($33)) {
   break;
  }
  $35 = ((($34)) + 188|0);
  $36 = HEAP32[$35>>2]|0;
  HEAP32[$pDel>>2] = $36;
  $37 = HEAP32[$pDel>>2]|0;
  $38 = ((($37)) + 4|0);
  $39 = HEAP32[$38>>2]|0;
  $40 = HEAP32[$0>>2]|0;
  $41 = ((($40)) + 188|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$pDel>>2]|0;
  _sqlite3VdbeFrameDelete($42);
 }
 $43 = ((($34)) + 204|0);
 $44 = HEAP32[$43>>2]|0;
 $45 = ($44|0)!=(0|0);
 if (!($45)) {
  STACKTOP = sp;return;
 }
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$0>>2]|0;
 $49 = ((($48)) + 204|0);
 _sqlite3VdbeDeleteAuxData($47,$49,-1,0);
 STACKTOP = sp;return;
}
function _vdbeCommit($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $i = 0, $iRandom = 0, $nMainFile = 0, $nTrans = 0, $needXcommit = 0, $offset = 0, $or$cond = 0, $pBt = 0, $pBt1 = 0, $pBt2 = 0, $pBt3 = 0, $pBt4 = 0, $pBt5 = 0, $pMaster = 0, $pPager = 0, $pVfs = 0, $rc = 0, $res = 0, $retryCount = 0;
 var $vararg_buffer = 0, $vararg_buffer11 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0, $vararg_ptr14 = 0, $zFile = 0, $zMainFile = 0, $zMaster = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer11 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 128|0;
 $1 = sp + 124|0;
 $2 = sp + 120|0;
 $i = sp + 116|0;
 $nTrans = sp + 112|0;
 $rc = sp + 108|0;
 $needXcommit = sp + 104|0;
 $pBt = sp + 100|0;
 $pPager = sp + 96|0;
 $pBt1 = sp + 92|0;
 $pBt2 = sp + 88|0;
 $pVfs = sp + 84|0;
 $zMaster = sp + 80|0;
 $zMainFile = sp + 76|0;
 $pMaster = sp + 72|0;
 $offset = sp;
 $res = sp + 68|0;
 $retryCount = sp + 64|0;
 $nMainFile = sp + 60|0;
 $iRandom = sp + 56|0;
 $pBt3 = sp + 52|0;
 $zFile = sp + 48|0;
 $pBt4 = sp + 44|0;
 $pBt5 = sp + 40|0;
 HEAP32[$1>>2] = $db;
 HEAP32[$2>>2] = $p;
 HEAP32[$nTrans>>2] = 0;
 HEAP32[$rc>>2] = 0;
 HEAP32[$needXcommit>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP32[$2>>2]|0;
 $5 = (_sqlite3VtabSync($3,$4)|0);
 HEAP32[$rc>>2] = $5;
 HEAP32[$i>>2] = 0;
 while(1) {
  $6 = HEAP32[$rc>>2]|0;
  $7 = ($6|0)==(0);
  if (!($7)) {
   break;
  }
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = ((($9)) + 20|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 16|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (($16) + ($13<<4)|0);
  $18 = ((($17)) + 4|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pBt>>2] = $19;
  $20 = HEAP32[$pBt>>2]|0;
  $21 = (_sqlite3BtreeIsInTrans($20)|0);
  $22 = ($21|0)!=(0);
  if ($22) {
   HEAP32[$needXcommit>>2] = 1;
   $23 = HEAP32[$pBt>>2]|0;
   _sqlite3BtreeEnter($23);
   $24 = HEAP32[$pBt>>2]|0;
   $25 = (_sqlite3BtreePager($24)|0);
   HEAP32[$pPager>>2] = $25;
   $26 = HEAP32[$i>>2]|0;
   $27 = HEAP32[$1>>2]|0;
   $28 = ((($27)) + 16|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (($29) + ($26<<4)|0);
   $31 = ((($30)) + 8|0);
   $32 = HEAP8[$31>>0]|0;
   $33 = $32&255;
   $34 = ($33|0)!=(1);
   if ($34) {
    $35 = HEAP32[$pPager>>2]|0;
    $36 = (_sqlite3PagerGetJournalMode($35)|0);
    $37 = (41812 + ($36)|0);
    $38 = HEAP8[$37>>0]|0;
    $39 = $38&255;
    $40 = ($39|0)!=(0);
    if ($40) {
     $41 = HEAP32[$nTrans>>2]|0;
     $42 = (($41) + 1)|0;
     HEAP32[$nTrans>>2] = $42;
    }
   }
   $43 = HEAP32[$pPager>>2]|0;
   $44 = (_sqlite3PagerExclusiveLock($43)|0);
   HEAP32[$rc>>2] = $44;
  }
  $45 = HEAP32[$i>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$i>>2] = $46;
 }
 $47 = HEAP32[$rc>>2]|0;
 $48 = ($47|0)!=(0);
 if ($48) {
  $49 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $49;
  $300 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($300|0);
 }
 $50 = HEAP32[$needXcommit>>2]|0;
 $51 = ($50|0)!=(0);
 if ($51) {
  $52 = HEAP32[$1>>2]|0;
  $53 = ((($52)) + 204|0);
  $54 = HEAP32[$53>>2]|0;
  $55 = ($54|0)!=(0|0);
  if ($55) {
   $56 = HEAP32[$1>>2]|0;
   $57 = ((($56)) + 204|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 200|0);
   $61 = HEAP32[$60>>2]|0;
   $62 = (FUNCTION_TABLE_ii[$58 & 255]($61)|0);
   HEAP32[$rc>>2] = $62;
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)!=(0);
   if ($64) {
    HEAP32[$0>>2] = 531;
    $300 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($300|0);
   }
  }
 }
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 16|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = ((($67)) + 4|0);
 $69 = HEAP32[$68>>2]|0;
 $70 = (_sqlite3BtreeGetFilename($69)|0);
 $71 = (_sqlite3Strlen30($70)|0);
 $72 = (0)==($71|0);
 $73 = HEAP32[$nTrans>>2]|0;
 $74 = ($73|0)<=(1);
 $or$cond = $72 | $74;
 if ($or$cond) {
  HEAP32[$i>>2] = 0;
  while(1) {
   $75 = HEAP32[$rc>>2]|0;
   $76 = ($75|0)==(0);
   if (!($76)) {
    break;
   }
   $77 = HEAP32[$i>>2]|0;
   $78 = HEAP32[$1>>2]|0;
   $79 = ((($78)) + 20|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = ($77|0)<($80|0);
   if (!($81)) {
    break;
   }
   $82 = HEAP32[$i>>2]|0;
   $83 = HEAP32[$1>>2]|0;
   $84 = ((($83)) + 16|0);
   $85 = HEAP32[$84>>2]|0;
   $86 = (($85) + ($82<<4)|0);
   $87 = ((($86)) + 4|0);
   $88 = HEAP32[$87>>2]|0;
   HEAP32[$pBt1>>2] = $88;
   $89 = HEAP32[$pBt1>>2]|0;
   $90 = ($89|0)!=(0|0);
   if ($90) {
    $91 = HEAP32[$pBt1>>2]|0;
    $92 = (_sqlite3BtreeCommitPhaseOne($91,0)|0);
    HEAP32[$rc>>2] = $92;
   }
   $93 = HEAP32[$i>>2]|0;
   $94 = (($93) + 1)|0;
   HEAP32[$i>>2] = $94;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $95 = HEAP32[$rc>>2]|0;
   $96 = ($95|0)==(0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[$i>>2]|0;
   $98 = HEAP32[$1>>2]|0;
   $99 = ((($98)) + 20|0);
   $100 = HEAP32[$99>>2]|0;
   $101 = ($97|0)<($100|0);
   if (!($101)) {
    break;
   }
   $102 = HEAP32[$i>>2]|0;
   $103 = HEAP32[$1>>2]|0;
   $104 = ((($103)) + 16|0);
   $105 = HEAP32[$104>>2]|0;
   $106 = (($105) + ($102<<4)|0);
   $107 = ((($106)) + 4|0);
   $108 = HEAP32[$107>>2]|0;
   HEAP32[$pBt2>>2] = $108;
   $109 = HEAP32[$pBt2>>2]|0;
   $110 = ($109|0)!=(0|0);
   if ($110) {
    $111 = HEAP32[$pBt2>>2]|0;
    $112 = (_sqlite3BtreeCommitPhaseTwo($111,0)|0);
    HEAP32[$rc>>2] = $112;
   }
   $113 = HEAP32[$i>>2]|0;
   $114 = (($113) + 1)|0;
   HEAP32[$i>>2] = $114;
  }
  $115 = HEAP32[$rc>>2]|0;
  $116 = ($115|0)==(0);
  if ($116) {
   $117 = HEAP32[$1>>2]|0;
   (_sqlite3VtabCommit($117)|0);
  }
 } else {
  $118 = HEAP32[$1>>2]|0;
  $119 = HEAP32[$118>>2]|0;
  HEAP32[$pVfs>>2] = $119;
  HEAP32[$zMaster>>2] = 0;
  $120 = HEAP32[$1>>2]|0;
  $121 = ((($120)) + 16|0);
  $122 = HEAP32[$121>>2]|0;
  $123 = ((($122)) + 4|0);
  $124 = HEAP32[$123>>2]|0;
  $125 = (_sqlite3BtreeGetFilename($124)|0);
  HEAP32[$zMainFile>>2] = $125;
  HEAP32[$pMaster>>2] = 0;
  $126 = $offset;
  $127 = $126;
  HEAP32[$127>>2] = 0;
  $128 = (($126) + 4)|0;
  $129 = $128;
  HEAP32[$129>>2] = 0;
  HEAP32[$retryCount>>2] = 0;
  $130 = HEAP32[$zMainFile>>2]|0;
  $131 = (_sqlite3Strlen30($130)|0);
  HEAP32[$nMainFile>>2] = $131;
  $132 = HEAP32[$1>>2]|0;
  $133 = HEAP32[$zMainFile>>2]|0;
  HEAP32[$vararg_buffer>>2] = $133;
  $134 = (_sqlite3MPrintf($132,41818,$vararg_buffer)|0);
  HEAP32[$zMaster>>2] = $134;
  $135 = HEAP32[$zMaster>>2]|0;
  $136 = ($135|0)==(0|0);
  if ($136) {
   HEAP32[$0>>2] = 7;
   $300 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($300|0);
  }
  while(1) {
   $137 = HEAP32[$retryCount>>2]|0;
   $138 = ($137|0)!=(0);
   if ($138) {
    $139 = HEAP32[$retryCount>>2]|0;
    $140 = ($139|0)>(100);
    if ($140) {
     label = 35;
     break;
    }
    $144 = HEAP32[$retryCount>>2]|0;
    $145 = ($144|0)==(1);
    if ($145) {
     $146 = HEAP32[$zMaster>>2]|0;
     HEAP32[$vararg_buffer8>>2] = $146;
     _sqlite3_log(13,41848,$vararg_buffer8);
    }
   }
   $147 = HEAP32[$retryCount>>2]|0;
   $148 = (($147) + 1)|0;
   HEAP32[$retryCount>>2] = $148;
   _sqlite3_randomness(4,$iRandom);
   $149 = HEAP32[$nMainFile>>2]|0;
   $150 = HEAP32[$zMaster>>2]|0;
   $151 = (($150) + ($149)|0);
   $152 = HEAP32[$iRandom>>2]|0;
   $153 = $152 >>> 8;
   $154 = $153 & 16777215;
   $155 = HEAP32[$iRandom>>2]|0;
   $156 = $155 & 255;
   HEAP32[$vararg_buffer11>>2] = $154;
   $vararg_ptr14 = ((($vararg_buffer11)) + 4|0);
   HEAP32[$vararg_ptr14>>2] = $156;
   (_sqlite3_snprintf(13,$151,41863,$vararg_buffer11)|0);
   $157 = HEAP32[$pVfs>>2]|0;
   $158 = HEAP32[$zMaster>>2]|0;
   $159 = (_sqlite3OsAccess($157,$158,0,$res)|0);
   HEAP32[$rc>>2] = $159;
   $160 = HEAP32[$rc>>2]|0;
   $161 = ($160|0)==(0);
   $162 = HEAP32[$res>>2]|0;
   $163 = ($162|0)!=(0);
   $164 = $161 ? $163 : 0;
   if (!($164)) {
    break;
   }
  }
  if ((label|0) == 35) {
   $141 = HEAP32[$zMaster>>2]|0;
   HEAP32[$vararg_buffer5>>2] = $141;
   _sqlite3_log(13,41834,$vararg_buffer5);
   $142 = HEAP32[$pVfs>>2]|0;
   $143 = HEAP32[$zMaster>>2]|0;
   (_sqlite3OsDelete($142,$143,0)|0);
  }
  $165 = HEAP32[$rc>>2]|0;
  $166 = ($165|0)==(0);
  if ($166) {
   $167 = HEAP32[$pVfs>>2]|0;
   $168 = HEAP32[$zMaster>>2]|0;
   $169 = (_sqlite3OsOpenMalloc($167,$168,$pMaster,16406,0)|0);
   HEAP32[$rc>>2] = $169;
  }
  $170 = HEAP32[$rc>>2]|0;
  $171 = ($170|0)!=(0);
  if ($171) {
   $172 = HEAP32[$1>>2]|0;
   $173 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($172,$173);
   $174 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $174;
   $300 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($300|0);
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $175 = HEAP32[$i>>2]|0;
   $176 = HEAP32[$1>>2]|0;
   $177 = ((($176)) + 20|0);
   $178 = HEAP32[$177>>2]|0;
   $179 = ($175|0)<($178|0);
   if (!($179)) {
    break;
   }
   $180 = HEAP32[$i>>2]|0;
   $181 = HEAP32[$1>>2]|0;
   $182 = ((($181)) + 16|0);
   $183 = HEAP32[$182>>2]|0;
   $184 = (($183) + ($180<<4)|0);
   $185 = ((($184)) + 4|0);
   $186 = HEAP32[$185>>2]|0;
   HEAP32[$pBt3>>2] = $186;
   $187 = HEAP32[$pBt3>>2]|0;
   $188 = (_sqlite3BtreeIsInTrans($187)|0);
   $189 = ($188|0)!=(0);
   if ($189) {
    $190 = HEAP32[$pBt3>>2]|0;
    $191 = (_sqlite3BtreeGetJournalname($190)|0);
    HEAP32[$zFile>>2] = $191;
    $192 = HEAP32[$zFile>>2]|0;
    $193 = ($192|0)==(0|0);
    if (!($193)) {
     $194 = HEAP32[$pMaster>>2]|0;
     $195 = HEAP32[$zFile>>2]|0;
     $196 = HEAP32[$zFile>>2]|0;
     $197 = (_sqlite3Strlen30($196)|0);
     $198 = (($197) + 1)|0;
     $199 = $offset;
     $200 = $199;
     $201 = HEAP32[$200>>2]|0;
     $202 = (($199) + 4)|0;
     $203 = $202;
     $204 = HEAP32[$203>>2]|0;
     $205 = (_sqlite3OsWrite($194,$195,$198,$201,$204)|0);
     HEAP32[$rc>>2] = $205;
     $206 = HEAP32[$zFile>>2]|0;
     $207 = (_sqlite3Strlen30($206)|0);
     $208 = (($207) + 1)|0;
     $209 = ($208|0)<(0);
     $210 = $209 << 31 >> 31;
     $211 = $offset;
     $212 = $211;
     $213 = HEAP32[$212>>2]|0;
     $214 = (($211) + 4)|0;
     $215 = $214;
     $216 = HEAP32[$215>>2]|0;
     $217 = (_i64Add(($213|0),($216|0),($208|0),($210|0))|0);
     $218 = tempRet0;
     $219 = $offset;
     $220 = $219;
     HEAP32[$220>>2] = $217;
     $221 = (($219) + 4)|0;
     $222 = $221;
     HEAP32[$222>>2] = $218;
     $223 = HEAP32[$rc>>2]|0;
     $224 = ($223|0)!=(0);
     if ($224) {
      label = 48;
      break;
     }
    }
   }
   $231 = HEAP32[$i>>2]|0;
   $232 = (($231) + 1)|0;
   HEAP32[$i>>2] = $232;
  }
  if ((label|0) == 48) {
   $225 = HEAP32[$pMaster>>2]|0;
   _sqlite3OsCloseFree($225);
   $226 = HEAP32[$pVfs>>2]|0;
   $227 = HEAP32[$zMaster>>2]|0;
   (_sqlite3OsDelete($226,$227,0)|0);
   $228 = HEAP32[$1>>2]|0;
   $229 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($228,$229);
   $230 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $230;
   $300 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($300|0);
  }
  $233 = HEAP32[$pMaster>>2]|0;
  $234 = (_sqlite3OsDeviceCharacteristics($233)|0);
  $235 = $234 & 1024;
  $236 = (0)==($235|0);
  if ($236) {
   $237 = HEAP32[$pMaster>>2]|0;
   $238 = (_sqlite3OsSync($237,2)|0);
   HEAP32[$rc>>2] = $238;
   $239 = (0)!=($238|0);
   if ($239) {
    $240 = HEAP32[$pMaster>>2]|0;
    _sqlite3OsCloseFree($240);
    $241 = HEAP32[$pVfs>>2]|0;
    $242 = HEAP32[$zMaster>>2]|0;
    (_sqlite3OsDelete($241,$242,0)|0);
    $243 = HEAP32[$1>>2]|0;
    $244 = HEAP32[$zMaster>>2]|0;
    _sqlite3DbFree($243,$244);
    $245 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $245;
    $300 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($300|0);
   }
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $246 = HEAP32[$rc>>2]|0;
   $247 = ($246|0)==(0);
   if (!($247)) {
    break;
   }
   $248 = HEAP32[$i>>2]|0;
   $249 = HEAP32[$1>>2]|0;
   $250 = ((($249)) + 20|0);
   $251 = HEAP32[$250>>2]|0;
   $252 = ($248|0)<($251|0);
   if (!($252)) {
    break;
   }
   $253 = HEAP32[$i>>2]|0;
   $254 = HEAP32[$1>>2]|0;
   $255 = ((($254)) + 16|0);
   $256 = HEAP32[$255>>2]|0;
   $257 = (($256) + ($253<<4)|0);
   $258 = ((($257)) + 4|0);
   $259 = HEAP32[$258>>2]|0;
   HEAP32[$pBt4>>2] = $259;
   $260 = HEAP32[$pBt4>>2]|0;
   $261 = ($260|0)!=(0|0);
   if ($261) {
    $262 = HEAP32[$pBt4>>2]|0;
    $263 = HEAP32[$zMaster>>2]|0;
    $264 = (_sqlite3BtreeCommitPhaseOne($262,$263)|0);
    HEAP32[$rc>>2] = $264;
   }
   $265 = HEAP32[$i>>2]|0;
   $266 = (($265) + 1)|0;
   HEAP32[$i>>2] = $266;
  }
  $267 = HEAP32[$pMaster>>2]|0;
  _sqlite3OsCloseFree($267);
  $268 = HEAP32[$rc>>2]|0;
  $269 = ($268|0)!=(0);
  if ($269) {
   $270 = HEAP32[$1>>2]|0;
   $271 = HEAP32[$zMaster>>2]|0;
   _sqlite3DbFree($270,$271);
   $272 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $272;
   $300 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($300|0);
  }
  $273 = HEAP32[$pVfs>>2]|0;
  $274 = HEAP32[$zMaster>>2]|0;
  $275 = (_sqlite3OsDelete($273,$274,1)|0);
  HEAP32[$rc>>2] = $275;
  $276 = HEAP32[$1>>2]|0;
  $277 = HEAP32[$zMaster>>2]|0;
  _sqlite3DbFree($276,$277);
  HEAP32[$zMaster>>2] = 0;
  $278 = HEAP32[$rc>>2]|0;
  $279 = ($278|0)!=(0);
  if ($279) {
   $280 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $280;
   $300 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($300|0);
  }
  _sqlite3BeginBenignMalloc();
  HEAP32[$i>>2] = 0;
  while(1) {
   $281 = HEAP32[$i>>2]|0;
   $282 = HEAP32[$1>>2]|0;
   $283 = ((($282)) + 20|0);
   $284 = HEAP32[$283>>2]|0;
   $285 = ($281|0)<($284|0);
   if (!($285)) {
    break;
   }
   $286 = HEAP32[$i>>2]|0;
   $287 = HEAP32[$1>>2]|0;
   $288 = ((($287)) + 16|0);
   $289 = HEAP32[$288>>2]|0;
   $290 = (($289) + ($286<<4)|0);
   $291 = ((($290)) + 4|0);
   $292 = HEAP32[$291>>2]|0;
   HEAP32[$pBt5>>2] = $292;
   $293 = HEAP32[$pBt5>>2]|0;
   $294 = ($293|0)!=(0|0);
   if ($294) {
    $295 = HEAP32[$pBt5>>2]|0;
    (_sqlite3BtreeCommitPhaseTwo($295,1)|0);
   }
   $296 = HEAP32[$i>>2]|0;
   $297 = (($296) + 1)|0;
   HEAP32[$i>>2] = $297;
  }
  _sqlite3EndBenignMalloc();
  $298 = HEAP32[$1>>2]|0;
  (_sqlite3VtabCommit($298)|0);
 }
 $299 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $299;
 $300 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($300|0);
}
function _sqlite3VtabSync($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aVTrans = 0, $i = 0;
 var $pVtab = 0, $rc = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $i = sp + 16|0;
 $rc = sp + 12|0;
 $aVTrans = sp + 8|0;
 $x = sp + 4|0;
 $pVtab = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 340|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$aVTrans>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 340|0);
 HEAP32[$6>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $7 = HEAP32[$rc>>2]|0;
  $8 = ($7|0)==(0);
  if (!($8)) {
   label = 8;
   break;
  }
  $9 = HEAP32[$i>>2]|0;
  $10 = HEAP32[$0>>2]|0;
  $11 = ((($10)) + 316|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($9|0)<($12|0);
  if (!($13)) {
   label = 8;
   break;
  }
  $14 = HEAP32[$i>>2]|0;
  $15 = HEAP32[$aVTrans>>2]|0;
  $16 = (($15) + ($14<<2)|0);
  $17 = HEAP32[$16>>2]|0;
  $18 = ((($17)) + 8|0);
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$pVtab>>2] = $19;
  $20 = HEAP32[$pVtab>>2]|0;
  $21 = ($20|0)!=(0|0);
  if ($21) {
   $22 = HEAP32[$pVtab>>2]|0;
   $23 = HEAP32[$22>>2]|0;
   $24 = ((($23)) + 60|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$x>>2] = $25;
   $26 = ($25|0)!=(0|0);
   if ($26) {
    $27 = HEAP32[$x>>2]|0;
    $28 = HEAP32[$pVtab>>2]|0;
    $29 = (FUNCTION_TABLE_ii[$27 & 255]($28)|0);
    HEAP32[$rc>>2] = $29;
    $30 = HEAP32[$1>>2]|0;
    $31 = HEAP32[$pVtab>>2]|0;
    _sqlite3VtabImportErrmsg($30,$31);
   }
  }
  $32 = HEAP32[$i>>2]|0;
  $33 = (($32) + 1)|0;
  HEAP32[$i>>2] = $33;
 }
 if ((label|0) == 8) {
  $34 = HEAP32[$aVTrans>>2]|0;
  $35 = HEAP32[$0>>2]|0;
  $36 = ((($35)) + 340|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 return (0)|0;
}
function _sqlite3VtabCommit($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _callFinaliser($1,64);
 STACKTOP = sp;return 0;
}
function _sqlite3BtreeGetJournalname($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3PagerJournalname($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3PagerJournalname($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 180|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _closeCursorsInFrame($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $pC = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $i = sp + 4|0;
 $pC = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 112|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 28|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = ($5|0)<($8|0);
  if (!($9)) {
   break;
  }
  $10 = HEAP32[$i>>2]|0;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 112|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = (($13) + ($10<<2)|0);
  $15 = HEAP32[$14>>2]|0;
  HEAP32[$pC>>2] = $15;
  $16 = HEAP32[$pC>>2]|0;
  $17 = ($16|0)!=(0|0);
  if ($17) {
   $18 = HEAP32[$0>>2]|0;
   $19 = HEAP32[$pC>>2]|0;
   _sqlite3VdbeFreeCursor($18,$19);
   $20 = HEAP32[$i>>2]|0;
   $21 = HEAP32[$0>>2]|0;
   $22 = ((($21)) + 112|0);
   $23 = HEAP32[$22>>2]|0;
   $24 = (($23) + ($20<<2)|0);
   HEAP32[$24>>2] = 0;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _sqlite3VdbeFrameDelete($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $aMem = 0, $apCsr = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $i = sp + 8|0;
 $aMem = sp + 4|0;
 $apCsr = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 80|0);
 HEAP32[$aMem>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 60|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$aMem>>2]|0;
 $7 = (($6) + (($5*40)|0)|0);
 HEAP32[$apCsr>>2] = $7;
 HEAP32[$i>>2] = 0;
 while(1) {
  $8 = HEAP32[$i>>2]|0;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 64|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($8|0)<($11|0);
  if (!($12)) {
   break;
  }
  $13 = HEAP32[$0>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$apCsr>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  _sqlite3VdbeFreeCursor($14,$18);
  $19 = HEAP32[$i>>2]|0;
  $20 = (($19) + 1)|0;
  HEAP32[$i>>2] = $20;
 }
 $21 = HEAP32[$aMem>>2]|0;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 60|0);
 $24 = HEAP32[$23>>2]|0;
 _releaseMemArray($21,$24);
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$26>>2]|0;
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 40|0);
 _sqlite3VdbeDeleteAuxData($27,$29,-1,0);
 $30 = HEAP32[$0>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$31>>2]|0;
 $33 = HEAP32[$0>>2]|0;
 _sqlite3DbFree($32,$33);
 STACKTOP = sp;return;
}
function _sqlite3OpcodeName($i) {
 $i = $i|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $i;
 $1 = HEAP32[$0>>2]|0;
 $2 = (5588 + ($1<<2)|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _displayP4($pOp,$zTemp,$nTemp) {
 $pOp = $pOp|0;
 $zTemp = $zTemp|0;
 $nTemp = $nTemp|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0.0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0.0, $82 = 0, $83 = 0;
 var $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ai = 0, $i = 0, $j = 0;
 var $n = 0, $pColl = 0, $pColl1 = 0, $pDef = 0, $pKeyInfo = 0, $pMem = 0, $pVtab = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer12 = 0, $vararg_buffer15 = 0, $vararg_buffer18 = 0, $vararg_buffer21 = 0, $vararg_buffer24 = 0, $vararg_buffer27 = 0, $vararg_buffer30 = 0, $vararg_buffer33 = 0, $vararg_buffer35 = 0, $vararg_buffer5 = 0, $vararg_buffer8 = 0;
 var $vararg_ptr11 = 0, $vararg_ptr4 = 0, $x = 0, $zColl = 0, $zP4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer35 = sp + 96|0;
 $vararg_buffer33 = sp + 88|0;
 $vararg_buffer30 = sp + 80|0;
 $vararg_buffer27 = sp + 72|0;
 $vararg_buffer24 = sp + 64|0;
 $vararg_buffer21 = sp + 56|0;
 $vararg_buffer18 = sp + 48|0;
 $vararg_buffer15 = sp + 40|0;
 $vararg_buffer12 = sp + 32|0;
 $vararg_buffer8 = sp + 24|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 184|0;
 $1 = sp + 180|0;
 $2 = sp + 176|0;
 $zP4 = sp + 172|0;
 $x = sp + 144|0;
 $j = sp + 140|0;
 $pKeyInfo = sp + 136|0;
 $pColl = sp + 132|0;
 $zColl = sp + 128|0;
 $pColl1 = sp + 124|0;
 $pDef = sp + 120|0;
 $pMem = sp + 116|0;
 $pVtab = sp + 112|0;
 $i = sp + 108|0;
 $ai = sp + 104|0;
 $n = sp + 100|0;
 HEAP32[$0>>2] = $pOp;
 HEAP32[$1>>2] = $zTemp;
 HEAP32[$2>>2] = $nTemp;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$zP4>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 _sqlite3StrAccumInit($x,0,$4,$5,0);
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 1|0);
 $8 = HEAP8[$7>>0]|0;
 $9 = $8 << 24 >> 24;
 L1: do {
  switch ($9|0) {
  case -6:  {
   $10 = HEAP32[$0>>2]|0;
   $11 = ((($10)) + 16|0);
   $12 = HEAP32[$11>>2]|0;
   HEAP32[$pKeyInfo>>2] = $12;
   $13 = HEAP32[$pKeyInfo>>2]|0;
   $14 = ((($13)) + 6|0);
   $15 = HEAP16[$14>>1]|0;
   $16 = $15&65535;
   HEAP32[$vararg_buffer>>2] = $16;
   _sqlite3XPrintf($x,41915,$vararg_buffer);
   HEAP32[$j>>2] = 0;
   while(1) {
    $17 = HEAP32[$j>>2]|0;
    $18 = HEAP32[$pKeyInfo>>2]|0;
    $19 = ((($18)) + 6|0);
    $20 = HEAP16[$19>>1]|0;
    $21 = $20&65535;
    $22 = ($17|0)<($21|0);
    if (!($22)) {
     break;
    }
    $23 = HEAP32[$j>>2]|0;
    $24 = HEAP32[$pKeyInfo>>2]|0;
    $25 = ((($24)) + 20|0);
    $26 = (($25) + ($23<<2)|0);
    $27 = HEAP32[$26>>2]|0;
    HEAP32[$pColl>>2] = $27;
    $28 = HEAP32[$pColl>>2]|0;
    $29 = ($28|0)!=(0|0);
    if ($29) {
     $30 = HEAP32[$pColl>>2]|0;
     $31 = HEAP32[$30>>2]|0;
     $32 = $31;
    } else {
     $32 = 18402;
    }
    HEAP32[$zColl>>2] = $32;
    $33 = HEAP32[$zColl>>2]|0;
    $34 = (_strcmp($33,34834)|0);
    $35 = ($34|0)==(0);
    $$ = $35 ? 41920 : $32;
    HEAP32[$zColl>>2] = $$;
    $36 = HEAP32[$j>>2]|0;
    $37 = HEAP32[$pKeyInfo>>2]|0;
    $38 = ((($37)) + 16|0);
    $39 = HEAP32[$38>>2]|0;
    $40 = (($39) + ($36)|0);
    $41 = HEAP8[$40>>0]|0;
    $42 = $41&255;
    $43 = ($42|0)!=(0);
    $44 = $43 ? 28264 : 18402;
    $45 = HEAP32[$zColl>>2]|0;
    HEAP32[$vararg_buffer1>>2] = $44;
    $vararg_ptr4 = ((($vararg_buffer1)) + 4|0);
    HEAP32[$vararg_ptr4>>2] = $45;
    _sqlite3XPrintf($x,41922,$vararg_buffer1);
    $46 = HEAP32[$j>>2]|0;
    $47 = (($46) + 1)|0;
    HEAP32[$j>>2] = $47;
   }
   _sqlite3StrAccumAppend($x,34706,1);
   break;
  }
  case -4:  {
   $48 = HEAP32[$0>>2]|0;
   $49 = ((($48)) + 16|0);
   $50 = HEAP32[$49>>2]|0;
   HEAP32[$pColl1>>2] = $50;
   $51 = HEAP32[$pColl1>>2]|0;
   $52 = HEAP32[$51>>2]|0;
   HEAP32[$vararg_buffer5>>2] = $52;
   _sqlite3XPrintf($x,41928,$vararg_buffer5);
   break;
  }
  case -5:  {
   $53 = HEAP32[$0>>2]|0;
   $54 = ((($53)) + 16|0);
   $55 = HEAP32[$54>>2]|0;
   HEAP32[$pDef>>2] = $55;
   $56 = HEAP32[$pDef>>2]|0;
   $57 = ((($56)) + 20|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$pDef>>2]|0;
   $60 = HEAP8[$59>>0]|0;
   $61 = $60 << 24 >> 24;
   HEAP32[$vararg_buffer8>>2] = $58;
   $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
   HEAP32[$vararg_ptr11>>2] = $61;
   _sqlite3XPrintf($x,41936,$vararg_buffer8);
   break;
  }
  case -13:  {
   $62 = HEAP32[$0>>2]|0;
   $63 = ((($62)) + 16|0);
   $64 = HEAP32[$63>>2]|0;
   $65 = $64;
   $66 = $65;
   $67 = HEAP32[$66>>2]|0;
   $68 = (($65) + 4)|0;
   $69 = $68;
   $70 = HEAP32[$69>>2]|0;
   $71 = $vararg_buffer12;
   $72 = $71;
   HEAP32[$72>>2] = $67;
   $73 = (($71) + 4)|0;
   $74 = $73;
   HEAP32[$74>>2] = $70;
   _sqlite3XPrintf($x,28266,$vararg_buffer12);
   break;
  }
  case -14:  {
   $75 = HEAP32[$0>>2]|0;
   $76 = ((($75)) + 16|0);
   $77 = HEAP32[$76>>2]|0;
   HEAP32[$vararg_buffer15>>2] = $77;
   _sqlite3XPrintf($x,41943,$vararg_buffer15);
   break;
  }
  case -12:  {
   $78 = HEAP32[$0>>2]|0;
   $79 = ((($78)) + 16|0);
   $80 = HEAP32[$79>>2]|0;
   $81 = +HEAPF64[$80>>3];
   HEAPF64[$vararg_buffer18>>3] = $81;
   _sqlite3XPrintf($x,41946,$vararg_buffer18);
   break;
  }
  case -8:  {
   $82 = HEAP32[$0>>2]|0;
   $83 = ((($82)) + 16|0);
   $84 = HEAP32[$83>>2]|0;
   HEAP32[$pMem>>2] = $84;
   $85 = HEAP32[$pMem>>2]|0;
   $86 = ((($85)) + 8|0);
   $87 = HEAP16[$86>>1]|0;
   $88 = $87&65535;
   $89 = $88 & 2;
   $90 = ($89|0)!=(0);
   $91 = HEAP32[$pMem>>2]|0;
   if ($90) {
    $92 = ((($91)) + 16|0);
    $93 = HEAP32[$92>>2]|0;
    HEAP32[$zP4>>2] = $93;
    break L1;
   }
   $94 = ((($91)) + 8|0);
   $95 = HEAP16[$94>>1]|0;
   $96 = $95&65535;
   $97 = $96 & 4;
   $98 = ($97|0)!=(0);
   $99 = HEAP32[$pMem>>2]|0;
   if ($98) {
    $100 = $99;
    $101 = $100;
    $102 = HEAP32[$101>>2]|0;
    $103 = (($100) + 4)|0;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = $vararg_buffer21;
    $107 = $106;
    HEAP32[$107>>2] = $102;
    $108 = (($106) + 4)|0;
    $109 = $108;
    HEAP32[$109>>2] = $105;
    _sqlite3XPrintf($x,28266,$vararg_buffer21);
    break L1;
   }
   $110 = ((($99)) + 8|0);
   $111 = HEAP16[$110>>1]|0;
   $112 = $111&65535;
   $113 = $112 & 8;
   $114 = ($113|0)!=(0);
   $115 = HEAP32[$pMem>>2]|0;
   if ($114) {
    $116 = +HEAPF64[$115>>3];
    HEAPF64[$vararg_buffer24>>3] = $116;
    _sqlite3XPrintf($x,41946,$vararg_buffer24);
    break L1;
   }
   $117 = ((($115)) + 8|0);
   $118 = HEAP16[$117>>1]|0;
   $119 = $118&65535;
   $120 = $119 & 1;
   $121 = ($120|0)!=(0);
   if ($121) {
    HEAP32[$zP4>>2] = 39691;
    break L1;
   } else {
    HEAP32[$zP4>>2] = 41952;
    break L1;
   }
   break;
  }
  case -10:  {
   $122 = HEAP32[$0>>2]|0;
   $123 = ((($122)) + 16|0);
   $124 = HEAP32[$123>>2]|0;
   $125 = ((($124)) + 8|0);
   $126 = HEAP32[$125>>2]|0;
   HEAP32[$pVtab>>2] = $126;
   $127 = HEAP32[$pVtab>>2]|0;
   HEAP32[$vararg_buffer27>>2] = $127;
   _sqlite3XPrintf($x,41959,$vararg_buffer27);
   break;
  }
  case -15:  {
   $128 = HEAP32[$0>>2]|0;
   $129 = ((($128)) + 16|0);
   $130 = HEAP32[$129>>2]|0;
   HEAP32[$ai>>2] = $130;
   $131 = HEAP32[$ai>>2]|0;
   $132 = HEAP32[$131>>2]|0;
   HEAP32[$n>>2] = $132;
   HEAP32[$i>>2] = 1;
   while(1) {
    $133 = HEAP32[$i>>2]|0;
    $134 = HEAP32[$n>>2]|0;
    $135 = ($133|0)<($134|0);
    if (!($135)) {
     break;
    }
    $136 = HEAP32[$i>>2]|0;
    $137 = HEAP32[$ai>>2]|0;
    $138 = (($137) + ($136<<2)|0);
    $139 = HEAP32[$138>>2]|0;
    HEAP32[$vararg_buffer30>>2] = $139;
    _sqlite3XPrintf($x,41967,$vararg_buffer30);
    $140 = HEAP32[$i>>2]|0;
    $141 = (($140) + 1)|0;
    HEAP32[$i>>2] = $141;
   }
   $142 = HEAP32[$1>>2]|0;
   HEAP8[$142>>0] = 91;
   _sqlite3StrAccumAppend($x,41971,1);
   break;
  }
  case -18:  {
   _sqlite3XPrintf($x,41973,$vararg_buffer33);
   break;
  }
  case -19:  {
   $143 = HEAP32[$1>>2]|0;
   HEAP8[$143>>0] = 0;
   break;
  }
  case -20:  {
   $144 = HEAP32[$0>>2]|0;
   $145 = ((($144)) + 16|0);
   $146 = HEAP32[$145>>2]|0;
   $147 = HEAP32[$146>>2]|0;
   HEAP32[$vararg_buffer35>>2] = $147;
   _sqlite3XPrintf($x,18391,$vararg_buffer35);
   break;
  }
  default: {
   $148 = HEAP32[$0>>2]|0;
   $149 = ((($148)) + 16|0);
   $150 = HEAP32[$149>>2]|0;
   HEAP32[$zP4>>2] = $150;
   $151 = HEAP32[$zP4>>2]|0;
   $152 = ($151|0)==(0|0);
   if ($152) {
    $153 = HEAP32[$1>>2]|0;
    HEAP32[$zP4>>2] = $153;
    $154 = HEAP32[$1>>2]|0;
    HEAP8[$154>>0] = 0;
   }
  }
  }
 } while(0);
 (_sqlite3StrAccumFinish($x)|0);
 $155 = HEAP32[$zP4>>2]|0;
 STACKTOP = sp;return ($155|0);
}
function _vdbeReleaseAndSetInt64($pMem,$0,$1) {
 $pMem = $pMem|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 8|0;
 $3 = sp;
 HEAP32[$2>>2] = $pMem;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 _sqlite3VdbeMemSetNull($8);
 $9 = $3;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$2>>2]|0;
 $16 = $15;
 $17 = $16;
 HEAP32[$17>>2] = $11;
 $18 = (($16) + 4)|0;
 $19 = $18;
 HEAP32[$19>>2] = $14;
 $20 = HEAP32[$2>>2]|0;
 $21 = ((($20)) + 8|0);
 HEAP16[$21>>1] = 4;
 STACKTOP = sp;return;
}
function _sqlite3VdbeMemHandleBom($pMem) {
 $pMem = $pMem|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $8 = 0, $9 = 0, $b1 = 0, $b2 = 0, $bom = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $rc = sp;
 $bom = sp + 10|0;
 $b1 = sp + 9|0;
 $b2 = sp + 8|0;
 HEAP32[$0>>2] = $pMem;
 HEAP32[$rc>>2] = 0;
 HEAP8[$bom>>0] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)>(1);
 if ($4) {
  $5 = HEAP32[$0>>2]|0;
  $6 = ((($5)) + 16|0);
  $7 = HEAP32[$6>>2]|0;
  $8 = HEAP8[$7>>0]|0;
  HEAP8[$b1>>0] = $8;
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 16|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ((($11)) + 1|0);
  $13 = HEAP8[$12>>0]|0;
  HEAP8[$b2>>0] = $13;
  $14 = HEAP8[$b1>>0]|0;
  $15 = $14&255;
  $16 = ($15|0)==(254);
  if ($16) {
   $17 = HEAP8[$b2>>0]|0;
   $18 = $17&255;
   $19 = ($18|0)==(255);
   if ($19) {
    HEAP8[$bom>>0] = 3;
   }
  }
  $20 = HEAP8[$b1>>0]|0;
  $21 = $20&255;
  $22 = ($21|0)==(255);
  if ($22) {
   $23 = HEAP8[$b2>>0]|0;
   $24 = $23&255;
   $25 = ($24|0)==(254);
   if ($25) {
    HEAP8[$bom>>0] = 2;
   }
  }
 }
 $26 = HEAP8[$bom>>0]|0;
 $27 = ($26<<24>>24)!=(0);
 if (!($27)) {
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $28 = HEAP32[$0>>2]|0;
 $29 = (_sqlite3VdbeMemMakeWriteable($28)|0);
 HEAP32[$rc>>2] = $29;
 $30 = HEAP32[$rc>>2]|0;
 $31 = ($30|0)==(0);
 if (!($31)) {
  $70 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($70|0);
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = ((($32)) + 12|0);
 $34 = HEAP32[$33>>2]|0;
 $35 = (($34) - 2)|0;
 HEAP32[$33>>2] = $35;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 16|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = HEAP32[$0>>2]|0;
 $40 = ((($39)) + 16|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = ((($41)) + 2|0);
 $43 = HEAP32[$0>>2]|0;
 $44 = ((($43)) + 12|0);
 $45 = HEAP32[$44>>2]|0;
 _memmove(($38|0),($42|0),($45|0))|0;
 $46 = HEAP32[$0>>2]|0;
 $47 = ((($46)) + 12|0);
 $48 = HEAP32[$47>>2]|0;
 $49 = HEAP32[$0>>2]|0;
 $50 = ((($49)) + 16|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = (($51) + ($48)|0);
 HEAP8[$52>>0] = 0;
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 12|0);
 $55 = HEAP32[$54>>2]|0;
 $56 = (($55) + 1)|0;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 16|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = (($59) + ($56)|0);
 HEAP8[$60>>0] = 0;
 $61 = HEAP32[$0>>2]|0;
 $62 = ((($61)) + 8|0);
 $63 = HEAP16[$62>>1]|0;
 $64 = $63&65535;
 $65 = $64 | 512;
 $66 = $65&65535;
 HEAP16[$62>>1] = $66;
 $67 = HEAP8[$bom>>0]|0;
 $68 = HEAP32[$0>>2]|0;
 $69 = ((($68)) + 10|0);
 HEAP8[$69>>0] = $67;
 $70 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($70|0);
}
function _valueBytes($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 4|0;
 HEAP32[$0>>2] = $pVal;
 HEAP8[$1>>0] = $enc;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP8[$1>>0]|0;
 $4 = (_valueToText($2,$3)|0);
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $9 = 0;
  STACKTOP = sp;return ($9|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 12|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8;
 STACKTOP = sp;return ($9|0);
}
function _valueToText($pVal,$enc) {
 $pVal = $pVal|0;
 $enc = $enc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$1>>2] = $pVal;
 HEAP8[$2>>0] = $enc;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP16[$4>>1]|0;
 $6 = $5&65535;
 $7 = $6 & 18;
 $8 = ($7|0)!=(0);
 $9 = HEAP32[$1>>2]|0;
 if ($8) {
  $10 = ((($9)) + 8|0);
  $11 = HEAP16[$10>>1]|0;
  $12 = $11&65535;
  $13 = $12 | 2;
  $14 = $13&65535;
  HEAP16[$10>>1] = $14;
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 10|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = $17&255;
  $19 = HEAP8[$2>>0]|0;
  $20 = $19&255;
  $21 = $20 & -9;
  $22 = ($18|0)!=($21|0);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = HEAP8[$2>>0]|0;
   $25 = $24&255;
   $26 = $25 & -9;
   (_sqlite3VdbeChangeEncoding($23,$26)|0);
  }
  $27 = HEAP8[$2>>0]|0;
  $28 = $27&255;
  $29 = $28 & 8;
  $30 = ($29|0)!=(0);
  if ($30) {
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 16|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = $33;
   $35 = 1 & $34;
   $36 = (1)==($35|0);
   if ($36) {
    $37 = HEAP32[$1>>2]|0;
    $38 = (_sqlite3VdbeMemMakeWriteable($37)|0);
    $39 = ($38|0)!=(0);
    if ($39) {
     HEAP32[$0>>2] = 0;
     $53 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($53|0);
    }
   }
  }
  $40 = HEAP32[$1>>2]|0;
  (_sqlite3VdbeMemNulTerminate($40)|0);
 } else {
  $41 = HEAP8[$2>>0]|0;
  (_sqlite3VdbeMemStringify($9,$41,0)|0);
 }
 $42 = HEAP32[$1>>2]|0;
 $43 = ((($42)) + 10|0);
 $44 = HEAP8[$43>>0]|0;
 $45 = $44&255;
 $46 = HEAP8[$2>>0]|0;
 $47 = $46&255;
 $48 = $47 & -9;
 $49 = ($45|0)==($48|0);
 if ($49) {
  $50 = HEAP32[$1>>2]|0;
  $51 = ((($50)) + 16|0);
  $52 = HEAP32[$51>>2]|0;
  HEAP32[$0>>2] = $52;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 } else {
  HEAP32[$0>>2] = 0;
  $53 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($53|0);
 }
 return (0)|0;
}
function _vdbeMemClear($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 9312;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$0>>2]|0;
  _vdbeMemClearExternAndSetNull($7);
 }
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 24|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 16|0);
  HEAP32[$21>>2] = 0;
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = ((($12)) + 32|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 20|0);
 $17 = HEAP32[$16>>2]|0;
 _sqlite3DbFree($14,$17);
 $18 = HEAP32[$0>>2]|0;
 $19 = ((($18)) + 24|0);
 HEAP32[$19>>2] = 0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 16|0);
 HEAP32[$21>>2] = 0;
 STACKTOP = sp;return;
}
function _Cleanup($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$db>>2] = $2;
 $3 = HEAP32[$db>>2]|0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 108|0);
 $6 = HEAP32[$5>>2]|0;
 _sqlite3DbFree($3,$6);
 $7 = HEAP32[$0>>2]|0;
 $8 = ((($7)) + 108|0);
 HEAP32[$8>>2] = 0;
 $9 = HEAP32[$0>>2]|0;
 $10 = ((($9)) + 104|0);
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return;
}
function _apiOomError($db) {
 $db = $db|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $db;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3OomClear($1);
 $2 = HEAP32[$0>>2]|0;
 _sqlite3Error($2,7);
 STACKTOP = sp;return 7;
}
function _btreeClearHasContent($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pBt;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 60|0);
 $3 = HEAP32[$2>>2]|0;
 _sqlite3BitvecDestroy($3);
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 60|0);
 HEAP32[$5>>2] = 0;
 STACKTOP = sp;return;
}
function _btreeEndTransaction($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0;
 var $pBt = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$db>>2] = $5;
 $6 = HEAP32[$pBt>>2]|0;
 $7 = ((($6)) + 19|0);
 HEAP8[$7>>0] = 0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 8|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)>(0);
 if ($12) {
  $13 = HEAP32[$db>>2]|0;
  $14 = ((($13)) + 160|0);
  $15 = HEAP32[$14>>2]|0;
  $16 = ($15|0)>(1);
  if ($16) {
   $17 = HEAP32[$0>>2]|0;
   _downgradeAllSharedCacheTableLocks($17);
   $18 = HEAP32[$0>>2]|0;
   $19 = ((($18)) + 8|0);
   HEAP8[$19>>0] = 1;
   STACKTOP = sp;return;
  }
 }
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 8|0);
 $22 = HEAP8[$21>>0]|0;
 $23 = $22&255;
 $24 = ($23|0)!=(0);
 if ($24) {
  $25 = HEAP32[$0>>2]|0;
  _clearAllSharedCacheTableLocks($25);
  $26 = HEAP32[$pBt>>2]|0;
  $27 = ((($26)) + 40|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = (($28) + -1)|0;
  HEAP32[$27>>2] = $29;
  $30 = HEAP32[$pBt>>2]|0;
  $31 = ((($30)) + 40|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = (0)==($32|0);
  if ($33) {
   $34 = HEAP32[$pBt>>2]|0;
   $35 = ((($34)) + 20|0);
   HEAP8[$35>>0] = 0;
  }
 }
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 8|0);
 HEAP8[$37>>0] = 0;
 $38 = HEAP32[$pBt>>2]|0;
 _unlockBtreeIfUnused($38);
 STACKTOP = sp;return;
}
function _downgradeAllSharedCacheTableLocks($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pLock = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pBt = sp + 4|0;
 $pLock = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$pBt>>2]|0;
 $5 = ((($4)) + 76|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = ($6|0)==($7|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$pBt>>2]|0;
 $10 = ((($9)) + 76|0);
 HEAP32[$10>>2] = 0;
 $11 = HEAP32[$pBt>>2]|0;
 $12 = ((($11)) + 22|0);
 $13 = HEAP16[$12>>1]|0;
 $14 = $13&65535;
 $15 = $14 & -97;
 $16 = $15&65535;
 HEAP16[$12>>1] = $16;
 $17 = HEAP32[$pBt>>2]|0;
 $18 = ((($17)) + 72|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pLock>>2] = $19;
 while(1) {
  $20 = HEAP32[$pLock>>2]|0;
  $21 = ($20|0)!=(0|0);
  if (!($21)) {
   break;
  }
  $22 = HEAP32[$pLock>>2]|0;
  $23 = ((($22)) + 8|0);
  HEAP8[$23>>0] = 1;
  $24 = HEAP32[$pLock>>2]|0;
  $25 = ((($24)) + 12|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$pLock>>2] = $26;
 }
 STACKTOP = sp;return;
}
function _clearAllSharedCacheTableLocks($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pBt = 0, $pLock = 0, $ppIter = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pBt = sp + 8|0;
 $ppIter = sp + 4|0;
 $pLock = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 4|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pBt>>2] = $3;
 $4 = HEAP32[$pBt>>2]|0;
 $5 = ((($4)) + 72|0);
 HEAP32[$ppIter>>2] = $5;
 while(1) {
  $6 = HEAP32[$ppIter>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$ppIter>>2]|0;
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$pLock>>2] = $10;
  $11 = HEAP32[$pLock>>2]|0;
  $12 = HEAP32[$11>>2]|0;
  $13 = HEAP32[$0>>2]|0;
  $14 = ($12|0)==($13|0);
  $15 = HEAP32[$pLock>>2]|0;
  $16 = ((($15)) + 12|0);
  if (!($14)) {
   HEAP32[$ppIter>>2] = $16;
   continue;
  }
  $17 = HEAP32[$16>>2]|0;
  $18 = HEAP32[$ppIter>>2]|0;
  HEAP32[$18>>2] = $17;
  $19 = HEAP32[$pLock>>2]|0;
  $20 = ((($19)) + 4|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)!=(1);
  if (!($22)) {
   continue;
  }
  $23 = HEAP32[$pLock>>2]|0;
  _sqlite3_free($23);
 }
 $24 = HEAP32[$pBt>>2]|0;
 $25 = ((($24)) + 76|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = HEAP32[$0>>2]|0;
 $28 = ($26|0)==($27|0);
 $29 = HEAP32[$pBt>>2]|0;
 if ($28) {
  $30 = ((($29)) + 76|0);
  HEAP32[$30>>2] = 0;
  $31 = HEAP32[$pBt>>2]|0;
  $32 = ((($31)) + 22|0);
  $33 = HEAP16[$32>>1]|0;
  $34 = $33&65535;
  $35 = $34 & -97;
  $36 = $35&65535;
  HEAP16[$32>>1] = $36;
  STACKTOP = sp;return;
 }
 $37 = ((($29)) + 40|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)==(2);
 if (!($39)) {
  STACKTOP = sp;return;
 }
 $40 = HEAP32[$pBt>>2]|0;
 $41 = ((($40)) + 22|0);
 $42 = HEAP16[$41>>1]|0;
 $43 = $42&65535;
 $44 = $43 & -65;
 $45 = $44&65535;
 HEAP16[$41>>1] = $45;
 STACKTOP = sp;return;
}
function _autoVacuumCommit($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $iFree = 0;
 var $nFin = 0, $nFree = 0, $nOrig = 0, $or$cond = 0, $or$cond3 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $rc = sp + 20|0;
 $pPager = sp + 16|0;
 $nFin = sp + 12|0;
 $nFree = sp + 8|0;
 $iFree = sp + 4|0;
 $nOrig = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 _invalidateAllOverflowCache($4);
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 18|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 do {
  if (!($8)) {
   $9 = HEAP32[$1>>2]|0;
   $10 = (_btreePagecount($9)|0);
   HEAP32[$nOrig>>2] = $10;
   $11 = HEAP32[$1>>2]|0;
   $12 = HEAP32[$nOrig>>2]|0;
   $13 = (_ptrmapPageno($11,$12)|0);
   $14 = HEAP32[$nOrig>>2]|0;
   $15 = ($13|0)==($14|0);
   if (!($15)) {
    $16 = HEAP32[$nOrig>>2]|0;
    $17 = HEAP32[1536>>2]|0;
    $18 = HEAP32[$1>>2]|0;
    $19 = ((($18)) + 32|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = (($17>>>0) / ($20>>>0))&-1;
    $22 = (($21) + 1)|0;
    $23 = ($16|0)==($22|0);
    if (!($23)) {
     $25 = HEAP32[$1>>2]|0;
     $26 = ((($25)) + 12|0);
     $27 = HEAP32[$26>>2]|0;
     $28 = ((($27)) + 56|0);
     $29 = HEAP32[$28>>2]|0;
     $30 = ((($29)) + 36|0);
     $31 = (_sqlite3Get4byte($30)|0);
     HEAP32[$nFree>>2] = $31;
     $32 = HEAP32[$1>>2]|0;
     $33 = HEAP32[$nOrig>>2]|0;
     $34 = HEAP32[$nFree>>2]|0;
     $35 = (_finalDbSize($32,$33,$34)|0);
     HEAP32[$nFin>>2] = $35;
     $36 = HEAP32[$nFin>>2]|0;
     $37 = HEAP32[$nOrig>>2]|0;
     $38 = ($36>>>0)>($37>>>0);
     if ($38) {
      $39 = (_sqlite3CorruptError(61919)|0);
      HEAP32[$0>>2] = $39;
      $98 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($98|0);
     }
     $40 = HEAP32[$nFin>>2]|0;
     $41 = HEAP32[$nOrig>>2]|0;
     $42 = ($40>>>0)<($41>>>0);
     if ($42) {
      $43 = HEAP32[$1>>2]|0;
      $44 = (_saveAllCursors($43,0,0)|0);
      HEAP32[$rc>>2] = $44;
     }
     $45 = HEAP32[$nOrig>>2]|0;
     HEAP32[$iFree>>2] = $45;
     while(1) {
      $46 = HEAP32[$iFree>>2]|0;
      $47 = HEAP32[$nFin>>2]|0;
      $48 = ($46>>>0)>($47>>>0);
      $49 = HEAP32[$rc>>2]|0;
      $50 = ($49|0)==(0);
      $51 = $48 ? $50 : 0;
      if (!($51)) {
       break;
      }
      $52 = HEAP32[$1>>2]|0;
      $53 = HEAP32[$nFin>>2]|0;
      $54 = HEAP32[$iFree>>2]|0;
      $55 = (_incrVacuumStep($52,$53,$54,1)|0);
      HEAP32[$rc>>2] = $55;
      $56 = HEAP32[$iFree>>2]|0;
      $57 = (($56) + -1)|0;
      HEAP32[$iFree>>2] = $57;
     }
     $58 = HEAP32[$rc>>2]|0;
     $59 = ($58|0)==(101);
     $60 = HEAP32[$rc>>2]|0;
     $61 = ($60|0)==(0);
     $or$cond = $59 | $61;
     $62 = HEAP32[$nFree>>2]|0;
     $63 = ($62>>>0)>(0);
     $or$cond3 = $or$cond & $63;
     if ($or$cond3) {
      $64 = HEAP32[$1>>2]|0;
      $65 = ((($64)) + 12|0);
      $66 = HEAP32[$65>>2]|0;
      $67 = ((($66)) + 72|0);
      $68 = HEAP32[$67>>2]|0;
      $69 = (_sqlite3PagerWrite($68)|0);
      HEAP32[$rc>>2] = $69;
      $70 = HEAP32[$1>>2]|0;
      $71 = ((($70)) + 12|0);
      $72 = HEAP32[$71>>2]|0;
      $73 = ((($72)) + 56|0);
      $74 = HEAP32[$73>>2]|0;
      $75 = ((($74)) + 32|0);
      _sqlite3Put4byte($75,0);
      $76 = HEAP32[$1>>2]|0;
      $77 = ((($76)) + 12|0);
      $78 = HEAP32[$77>>2]|0;
      $79 = ((($78)) + 56|0);
      $80 = HEAP32[$79>>2]|0;
      $81 = ((($80)) + 36|0);
      _sqlite3Put4byte($81,0);
      $82 = HEAP32[$1>>2]|0;
      $83 = ((($82)) + 12|0);
      $84 = HEAP32[$83>>2]|0;
      $85 = ((($84)) + 56|0);
      $86 = HEAP32[$85>>2]|0;
      $87 = ((($86)) + 28|0);
      $88 = HEAP32[$nFin>>2]|0;
      _sqlite3Put4byte($87,$88);
      $89 = HEAP32[$1>>2]|0;
      $90 = ((($89)) + 19|0);
      HEAP8[$90>>0] = 1;
      $91 = HEAP32[$nFin>>2]|0;
      $92 = HEAP32[$1>>2]|0;
      $93 = ((($92)) + 44|0);
      HEAP32[$93>>2] = $91;
     }
     $94 = HEAP32[$rc>>2]|0;
     $95 = ($94|0)!=(0);
     if (!($95)) {
      break;
     }
     $96 = HEAP32[$pPager>>2]|0;
     (_sqlite3PagerRollback($96)|0);
     break;
    }
   }
   $24 = (_sqlite3CorruptError(61914)|0);
   HEAP32[$0>>2] = $24;
   $98 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($98|0);
  }
 } while(0);
 $97 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $97;
 $98 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($98|0);
}
function _sqlite3PagerCommitPhaseTwo($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0);
 $6 = HEAP32[$1>>2]|0;
 if ($5) {
  $7 = ((($6)) + 44|0);
  $8 = HEAP32[$7>>2]|0;
  HEAP32[$0>>2] = $8;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $9 = ((($6)) + 17|0);
 $10 = HEAP8[$9>>0]|0;
 $11 = $10&255;
 $12 = ($11|0)==(2);
 if ($12) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 4|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(0);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 5|0);
   $20 = HEAP8[$19>>0]|0;
   $21 = $20&255;
   $22 = ($21|0)==(1);
   if ($22) {
    $23 = HEAP32[$1>>2]|0;
    $24 = ((($23)) + 17|0);
    HEAP8[$24>>0] = 1;
    HEAP32[$0>>2] = 0;
    $38 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($38|0);
   }
  }
 }
 $25 = HEAP32[$1>>2]|0;
 $26 = ((($25)) + 108|0);
 $27 = HEAP32[$26>>2]|0;
 $28 = (($27) + 1)|0;
 HEAP32[$26>>2] = $28;
 $29 = HEAP32[$1>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 20|0);
 $32 = HEAP8[$31>>0]|0;
 $33 = $32&255;
 $34 = (_pager_end_transaction($29,$33,1)|0);
 HEAP32[$rc>>2] = $34;
 $35 = HEAP32[$1>>2]|0;
 $36 = HEAP32[$rc>>2]|0;
 $37 = (_pager_error($35,$36)|0);
 HEAP32[$0>>2] = $37;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _pager_incr_changecounter($pPager,$isDirectMode) {
 $pPager = $pPager|0;
 $isDirectMode = $isDirectMode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $pPgHdr = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pPgHdr = sp;
 HEAP32[$0>>2] = $pPager;
 HEAP32[$1>>2] = $isDirectMode;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 19|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 28|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8>>>0)>(0);
 if (!($9)) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = (_sqlite3PagerGet($10,1,$pPgHdr,0)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)==(0);
 if ($13) {
  $14 = HEAP32[$pPgHdr>>2]|0;
  $15 = (_sqlite3PagerWrite($14)|0);
  HEAP32[$rc>>2] = $15;
 }
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)==(0);
 if ($17) {
  $18 = HEAP32[$pPgHdr>>2]|0;
  _pager_write_changecounter($18);
  $19 = HEAP32[$0>>2]|0;
  $20 = ((($19)) + 19|0);
  HEAP8[$20>>0] = 1;
 }
 $21 = HEAP32[$pPgHdr>>2]|0;
 _sqlite3PagerUnref($21);
 $22 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($22|0);
}
function _writeMasterJournal($pPager,$zMaster) {
 $pPager = $pPager|0;
 $zMaster = $zMaster|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $21 = 0;
 var $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0;
 var $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0;
 var $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum = 0, $iHdrOff = 0, $jrnlSize = 0, $nMaster = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $rc = sp + 24|0;
 $nMaster = sp + 20|0;
 $iHdrOff = sp + 8|0;
 $jrnlSize = sp;
 $cksum = sp + 16|0;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $zMaster;
 HEAP32[$cksum>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if ($4) {
  $5 = HEAP32[$1>>2]|0;
  $6 = ((($5)) + 5|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = ($8|0)==(4);
  if (!($9)) {
   $10 = HEAP32[$1>>2]|0;
   $11 = ((($10)) + 68|0);
   $12 = HEAP32[$11>>2]|0;
   $13 = HEAP32[$12>>2]|0;
   $14 = ($13|0)!=(0|0);
   if ($14) {
    $15 = HEAP32[$1>>2]|0;
    $16 = ((($15)) + 20|0);
    HEAP8[$16>>0] = 1;
    HEAP32[$nMaster>>2] = 0;
    while(1) {
     $17 = HEAP32[$nMaster>>2]|0;
     $18 = HEAP32[$2>>2]|0;
     $19 = (($18) + ($17)|0);
     $20 = HEAP8[$19>>0]|0;
     $21 = ($20<<24>>24)!=(0);
     if (!($21)) {
      break;
     }
     $22 = HEAP32[$nMaster>>2]|0;
     $23 = HEAP32[$2>>2]|0;
     $24 = (($23) + ($22)|0);
     $25 = HEAP8[$24>>0]|0;
     $26 = $25 << 24 >> 24;
     $27 = HEAP32[$cksum>>2]|0;
     $28 = (($27) + ($26))|0;
     HEAP32[$cksum>>2] = $28;
     $29 = HEAP32[$nMaster>>2]|0;
     $30 = (($29) + 1)|0;
     HEAP32[$nMaster>>2] = $30;
    }
    $31 = HEAP32[$1>>2]|0;
    $32 = ((($31)) + 8|0);
    $33 = HEAP8[$32>>0]|0;
    $34 = ($33<<24>>24)!=(0);
    if ($34) {
     $35 = HEAP32[$1>>2]|0;
     $36 = (_journalHdrOffset($35)|0);
     $37 = tempRet0;
     $38 = HEAP32[$1>>2]|0;
     $39 = ((($38)) + 80|0);
     $40 = $39;
     $41 = $40;
     HEAP32[$41>>2] = $36;
     $42 = (($40) + 4)|0;
     $43 = $42;
     HEAP32[$43>>2] = $37;
    }
    $44 = HEAP32[$1>>2]|0;
    $45 = ((($44)) + 80|0);
    $46 = $45;
    $47 = $46;
    $48 = HEAP32[$47>>2]|0;
    $49 = (($46) + 4)|0;
    $50 = $49;
    $51 = HEAP32[$50>>2]|0;
    $52 = $iHdrOff;
    $53 = $52;
    HEAP32[$53>>2] = $48;
    $54 = (($52) + 4)|0;
    $55 = $54;
    HEAP32[$55>>2] = $51;
    $56 = HEAP32[$1>>2]|0;
    $57 = ((($56)) + 68|0);
    $58 = HEAP32[$57>>2]|0;
    $59 = $iHdrOff;
    $60 = $59;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59) + 4)|0;
    $63 = $62;
    $64 = HEAP32[$63>>2]|0;
    $65 = HEAP32[1536>>2]|0;
    $66 = HEAP32[$1>>2]|0;
    $67 = ((($66)) + 160|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (($65|0) / ($68|0))&-1;
    $70 = (($69) + 1)|0;
    $71 = (_write32bits($58,$61,$64,$70)|0);
    HEAP32[$rc>>2] = $71;
    $72 = (0)!=($71|0);
    if (!($72)) {
     $73 = HEAP32[$1>>2]|0;
     $74 = ((($73)) + 68|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = HEAP32[$2>>2]|0;
     $77 = HEAP32[$nMaster>>2]|0;
     $78 = $iHdrOff;
     $79 = $78;
     $80 = HEAP32[$79>>2]|0;
     $81 = (($78) + 4)|0;
     $82 = $81;
     $83 = HEAP32[$82>>2]|0;
     $84 = (_i64Add(($80|0),($83|0),4,0)|0);
     $85 = tempRet0;
     $86 = (_sqlite3OsWrite($75,$76,$77,$84,$85)|0);
     HEAP32[$rc>>2] = $86;
     $87 = (0)!=($86|0);
     if (!($87)) {
      $88 = HEAP32[$1>>2]|0;
      $89 = ((($88)) + 68|0);
      $90 = HEAP32[$89>>2]|0;
      $91 = $iHdrOff;
      $92 = $91;
      $93 = HEAP32[$92>>2]|0;
      $94 = (($91) + 4)|0;
      $95 = $94;
      $96 = HEAP32[$95>>2]|0;
      $97 = (_i64Add(($93|0),($96|0),4,0)|0);
      $98 = tempRet0;
      $99 = HEAP32[$nMaster>>2]|0;
      $100 = ($99|0)<(0);
      $101 = $100 << 31 >> 31;
      $102 = (_i64Add(($97|0),($98|0),($99|0),($101|0))|0);
      $103 = tempRet0;
      $104 = HEAP32[$nMaster>>2]|0;
      $105 = (_write32bits($90,$102,$103,$104)|0);
      HEAP32[$rc>>2] = $105;
      $106 = (0)!=($105|0);
      if (!($106)) {
       $107 = HEAP32[$1>>2]|0;
       $108 = ((($107)) + 68|0);
       $109 = HEAP32[$108>>2]|0;
       $110 = $iHdrOff;
       $111 = $110;
       $112 = HEAP32[$111>>2]|0;
       $113 = (($110) + 4)|0;
       $114 = $113;
       $115 = HEAP32[$114>>2]|0;
       $116 = (_i64Add(($112|0),($115|0),4,0)|0);
       $117 = tempRet0;
       $118 = HEAP32[$nMaster>>2]|0;
       $119 = ($118|0)<(0);
       $120 = $119 << 31 >> 31;
       $121 = (_i64Add(($116|0),($117|0),($118|0),($120|0))|0);
       $122 = tempRet0;
       $123 = (_i64Add(($121|0),($122|0),4,0)|0);
       $124 = tempRet0;
       $125 = HEAP32[$cksum>>2]|0;
       $126 = (_write32bits($109,$123,$124,$125)|0);
       HEAP32[$rc>>2] = $126;
       $127 = (0)!=($126|0);
       if (!($127)) {
        $128 = HEAP32[$1>>2]|0;
        $129 = ((($128)) + 68|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = $iHdrOff;
        $132 = $131;
        $133 = HEAP32[$132>>2]|0;
        $134 = (($131) + 4)|0;
        $135 = $134;
        $136 = HEAP32[$135>>2]|0;
        $137 = (_i64Add(($133|0),($136|0),4,0)|0);
        $138 = tempRet0;
        $139 = HEAP32[$nMaster>>2]|0;
        $140 = ($139|0)<(0);
        $141 = $140 << 31 >> 31;
        $142 = (_i64Add(($137|0),($138|0),($139|0),($141|0))|0);
        $143 = tempRet0;
        $144 = (_i64Add(($142|0),($143|0),8,0)|0);
        $145 = tempRet0;
        $146 = (_sqlite3OsWrite($130,25901,8,$144,$145)|0);
        HEAP32[$rc>>2] = $146;
        $147 = (0)!=($146|0);
        if (!($147)) {
         $149 = HEAP32[$nMaster>>2]|0;
         $150 = (($149) + 20)|0;
         $151 = ($150|0)<(0);
         $152 = $151 << 31 >> 31;
         $153 = HEAP32[$1>>2]|0;
         $154 = ((($153)) + 80|0);
         $155 = $154;
         $156 = $155;
         $157 = HEAP32[$156>>2]|0;
         $158 = (($155) + 4)|0;
         $159 = $158;
         $160 = HEAP32[$159>>2]|0;
         $161 = (_i64Add(($157|0),($160|0),($150|0),($152|0))|0);
         $162 = tempRet0;
         $163 = $154;
         $164 = $163;
         HEAP32[$164>>2] = $161;
         $165 = (($163) + 4)|0;
         $166 = $165;
         HEAP32[$166>>2] = $162;
         $167 = HEAP32[$1>>2]|0;
         $168 = ((($167)) + 68|0);
         $169 = HEAP32[$168>>2]|0;
         $170 = (_sqlite3OsFileSize($169,$jrnlSize)|0);
         HEAP32[$rc>>2] = $170;
         $171 = (0)==($170|0);
         if ($171) {
          $172 = $jrnlSize;
          $173 = $172;
          $174 = HEAP32[$173>>2]|0;
          $175 = (($172) + 4)|0;
          $176 = $175;
          $177 = HEAP32[$176>>2]|0;
          $178 = HEAP32[$1>>2]|0;
          $179 = ((($178)) + 80|0);
          $180 = $179;
          $181 = $180;
          $182 = HEAP32[$181>>2]|0;
          $183 = (($180) + 4)|0;
          $184 = $183;
          $185 = HEAP32[$184>>2]|0;
          $186 = ($177|0)>($185|0);
          $187 = ($174>>>0)>($182>>>0);
          $188 = ($177|0)==($185|0);
          $189 = $188 & $187;
          $190 = $186 | $189;
          if ($190) {
           $191 = HEAP32[$1>>2]|0;
           $192 = ((($191)) + 68|0);
           $193 = HEAP32[$192>>2]|0;
           $194 = HEAP32[$1>>2]|0;
           $195 = ((($194)) + 80|0);
           $196 = $195;
           $197 = $196;
           $198 = HEAP32[$197>>2]|0;
           $199 = (($196) + 4)|0;
           $200 = $199;
           $201 = HEAP32[$200>>2]|0;
           $202 = (_sqlite3OsTruncate($193,$198,$201)|0);
           HEAP32[$rc>>2] = $202;
          }
         }
         $203 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $203;
         $204 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($204|0);
        }
       }
      }
     }
    }
    $148 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $148;
    $204 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($204|0);
   }
  }
 }
 HEAP32[$0>>2] = 0;
 $204 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($204|0);
}
function _pagerWriteLargeSector($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0;
 var $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ii = 0, $nPage = 0, $nPageCount = 0, $nPagePerSector = 0;
 var $needSync = 0, $or$cond = 0, $pPage = 0, $pPage1 = 0, $pPager = 0, $pg = 0, $pg1 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $rc = sp + 40|0;
 $nPageCount = sp + 36|0;
 $pg1 = sp + 32|0;
 $nPage = sp + 28|0;
 $ii = sp + 24|0;
 $needSync = sp + 20|0;
 $pPager = sp + 16|0;
 $nPagePerSector = sp + 12|0;
 $pg = sp + 8|0;
 $pPage = sp + 4|0;
 $pPage1 = sp;
 HEAP32[$0>>2] = $pPg;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nPage>>2] = 0;
 HEAP32[$needSync>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pPager>>2] = $3;
 $4 = HEAP32[$pPager>>2]|0;
 $5 = ((($4)) + 156|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$pPager>>2]|0;
 $8 = ((($7)) + 160|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($6>>>0) / ($9>>>0))&-1;
 HEAP32[$nPagePerSector>>2] = $10;
 $11 = HEAP32[$pPager>>2]|0;
 $12 = ((($11)) + 21|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = $14 | 4;
 $16 = $15&255;
 HEAP8[$12>>0] = $16;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 20|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (($19) - 1)|0;
 $21 = HEAP32[$nPagePerSector>>2]|0;
 $22 = (($21) - 1)|0;
 $23 = $22 ^ -1;
 $24 = $20 & $23;
 $25 = (($24) + 1)|0;
 HEAP32[$pg1>>2] = $25;
 $26 = HEAP32[$pPager>>2]|0;
 $27 = ((($26)) + 28|0);
 $28 = HEAP32[$27>>2]|0;
 HEAP32[$nPageCount>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = HEAP32[$nPageCount>>2]|0;
 $33 = ($31>>>0)>($32>>>0);
 do {
  if ($33) {
   $34 = HEAP32[$0>>2]|0;
   $35 = ((($34)) + 20|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = HEAP32[$pg1>>2]|0;
   $38 = (($36) - ($37))|0;
   $39 = (($38) + 1)|0;
   HEAP32[$nPage>>2] = $39;
  } else {
   $40 = HEAP32[$pg1>>2]|0;
   $41 = HEAP32[$nPagePerSector>>2]|0;
   $42 = (($40) + ($41))|0;
   $43 = (($42) - 1)|0;
   $44 = HEAP32[$nPageCount>>2]|0;
   $45 = ($43>>>0)>($44>>>0);
   if ($45) {
    $46 = HEAP32[$nPageCount>>2]|0;
    $47 = (($46) + 1)|0;
    $48 = HEAP32[$pg1>>2]|0;
    $49 = (($47) - ($48))|0;
    HEAP32[$nPage>>2] = $49;
    break;
   } else {
    $50 = HEAP32[$nPagePerSector>>2]|0;
    HEAP32[$nPage>>2] = $50;
    break;
   }
  }
 } while(0);
 HEAP32[$ii>>2] = 0;
 while(1) {
  $51 = HEAP32[$ii>>2]|0;
  $52 = HEAP32[$nPage>>2]|0;
  $53 = ($51|0)<($52|0);
  $54 = HEAP32[$rc>>2]|0;
  $55 = ($54|0)==(0);
  $56 = $53 ? $55 : 0;
  if (!($56)) {
   break;
  }
  $57 = HEAP32[$pg1>>2]|0;
  $58 = HEAP32[$ii>>2]|0;
  $59 = (($57) + ($58))|0;
  HEAP32[$pg>>2] = $59;
  $60 = HEAP32[$pg>>2]|0;
  $61 = HEAP32[$0>>2]|0;
  $62 = ((($61)) + 20|0);
  $63 = HEAP32[$62>>2]|0;
  $64 = ($60|0)==($63|0);
  if ($64) {
   label = 10;
  } else {
   $65 = HEAP32[$pPager>>2]|0;
   $66 = ((($65)) + 60|0);
   $67 = HEAP32[$66>>2]|0;
   $68 = HEAP32[$pg>>2]|0;
   $69 = (_sqlite3BitvecTest($67,$68)|0);
   $70 = ($69|0)!=(0);
   if ($70) {
    $93 = HEAP32[$pPager>>2]|0;
    $94 = HEAP32[$pg>>2]|0;
    $95 = (_sqlite3PagerLookup($93,$94)|0);
    HEAP32[$pPage>>2] = $95;
    $96 = ($95|0)!=(0|0);
    if ($96) {
     $97 = HEAP32[$pPage>>2]|0;
     $98 = ((($97)) + 24|0);
     $99 = HEAP16[$98>>1]|0;
     $100 = $99&65535;
     $101 = $100 & 8;
     $102 = ($101|0)!=(0);
     if ($102) {
      HEAP32[$needSync>>2] = 1;
     }
     $103 = HEAP32[$pPage>>2]|0;
     _sqlite3PagerUnrefNotNull($103);
    }
   } else {
    label = 10;
   }
  }
  if ((label|0) == 10) {
   label = 0;
   $71 = HEAP32[$pg>>2]|0;
   $72 = HEAP32[1536>>2]|0;
   $73 = HEAP32[$pPager>>2]|0;
   $74 = ((($73)) + 160|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = (($72|0) / ($75|0))&-1;
   $77 = (($76) + 1)|0;
   $78 = ($71|0)!=($77|0);
   if ($78) {
    $79 = HEAP32[$pPager>>2]|0;
    $80 = HEAP32[$pg>>2]|0;
    $81 = (_sqlite3PagerGet($79,$80,$pPage,0)|0);
    HEAP32[$rc>>2] = $81;
    $82 = HEAP32[$rc>>2]|0;
    $83 = ($82|0)==(0);
    if ($83) {
     $84 = HEAP32[$pPage>>2]|0;
     $85 = (_pager_write($84)|0);
     HEAP32[$rc>>2] = $85;
     $86 = HEAP32[$pPage>>2]|0;
     $87 = ((($86)) + 24|0);
     $88 = HEAP16[$87>>1]|0;
     $89 = $88&65535;
     $90 = $89 & 8;
     $91 = ($90|0)!=(0);
     if ($91) {
      HEAP32[$needSync>>2] = 1;
     }
     $92 = HEAP32[$pPage>>2]|0;
     _sqlite3PagerUnrefNotNull($92);
    }
   }
  }
  $104 = HEAP32[$ii>>2]|0;
  $105 = (($104) + 1)|0;
  HEAP32[$ii>>2] = $105;
 }
 $106 = HEAP32[$rc>>2]|0;
 $107 = ($106|0)==(0);
 $108 = HEAP32[$needSync>>2]|0;
 $109 = ($108|0)!=(0);
 $or$cond = $107 & $109;
 if (!($or$cond)) {
  $129 = HEAP32[$pPager>>2]|0;
  $130 = ((($129)) + 21|0);
  $131 = HEAP8[$130>>0]|0;
  $132 = $131&255;
  $133 = $132 & -5;
  $134 = $133&255;
  HEAP8[$130>>0] = $134;
  $135 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($135|0);
 }
 HEAP32[$ii>>2] = 0;
 while(1) {
  $110 = HEAP32[$ii>>2]|0;
  $111 = HEAP32[$nPage>>2]|0;
  $112 = ($110|0)<($111|0);
  if (!($112)) {
   break;
  }
  $113 = HEAP32[$pPager>>2]|0;
  $114 = HEAP32[$pg1>>2]|0;
  $115 = HEAP32[$ii>>2]|0;
  $116 = (($114) + ($115))|0;
  $117 = (_sqlite3PagerLookup($113,$116)|0);
  HEAP32[$pPage1>>2] = $117;
  $118 = HEAP32[$pPage1>>2]|0;
  $119 = ($118|0)!=(0|0);
  if ($119) {
   $120 = HEAP32[$pPage1>>2]|0;
   $121 = ((($120)) + 24|0);
   $122 = HEAP16[$121>>1]|0;
   $123 = $122&65535;
   $124 = $123 | 8;
   $125 = $124&65535;
   HEAP16[$121>>1] = $125;
   $126 = HEAP32[$pPage1>>2]|0;
   _sqlite3PagerUnrefNotNull($126);
  }
  $127 = HEAP32[$ii>>2]|0;
  $128 = (($127) + 1)|0;
  HEAP32[$ii>>2] = $128;
 }
 $129 = HEAP32[$pPager>>2]|0;
 $130 = ((($129)) + 21|0);
 $131 = HEAP8[$130>>0]|0;
 $132 = $131&255;
 $133 = $132 & -5;
 $134 = $133&255;
 HEAP8[$130>>0] = $134;
 $135 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($135|0);
}
function _pager_write($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $pPager = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $pPager = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pPager>>2]|0;
 $6 = ((($5)) + 17|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$pPager>>2]|0;
  $11 = (_pager_open_journal($10)|0);
  HEAP32[$rc>>2] = $11;
  $12 = HEAP32[$rc>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $14;
   $76 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($76|0);
  }
 }
 $15 = HEAP32[$1>>2]|0;
 _sqlite3PcacheMakeDirty($15);
 $16 = HEAP32[$pPager>>2]|0;
 $17 = ((($16)) + 60|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)!=(0|0);
 do {
  if ($19) {
   $20 = HEAP32[$pPager>>2]|0;
   $21 = ((($20)) + 60|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 20|0);
   $25 = HEAP32[$24>>2]|0;
   $26 = (_sqlite3BitvecTestNotNull($22,$25)|0);
   $27 = ($26|0)==(0);
   if ($27) {
    $28 = HEAP32[$1>>2]|0;
    $29 = ((($28)) + 20|0);
    $30 = HEAP32[$29>>2]|0;
    $31 = HEAP32[$pPager>>2]|0;
    $32 = ((($31)) + 32|0);
    $33 = HEAP32[$32>>2]|0;
    $34 = ($30>>>0)<=($33>>>0);
    if (!($34)) {
     $40 = HEAP32[$pPager>>2]|0;
     $41 = ((($40)) + 17|0);
     $42 = HEAP8[$41>>0]|0;
     $43 = $42&255;
     $44 = ($43|0)!=(4);
     if (!($44)) {
      break;
     }
     $45 = HEAP32[$1>>2]|0;
     $46 = ((($45)) + 24|0);
     $47 = HEAP16[$46>>1]|0;
     $48 = $47&65535;
     $49 = $48 | 8;
     $50 = $49&65535;
     HEAP16[$46>>1] = $50;
     break;
    }
    $35 = HEAP32[$1>>2]|0;
    $36 = (_pagerAddPageToRollbackJournal($35)|0);
    HEAP32[$rc>>2] = $36;
    $37 = HEAP32[$rc>>2]|0;
    $38 = ($37|0)!=(0);
    if ($38) {
     $39 = HEAP32[$rc>>2]|0;
     HEAP32[$0>>2] = $39;
     $76 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($76|0);
    }
   }
  }
 } while(0);
 $51 = HEAP32[$1>>2]|0;
 $52 = ((($51)) + 24|0);
 $53 = HEAP16[$52>>1]|0;
 $54 = $53&65535;
 $55 = $54 | 4;
 $56 = $55&65535;
 HEAP16[$52>>1] = $56;
 $57 = HEAP32[$pPager>>2]|0;
 $58 = ((($57)) + 104|0);
 $59 = HEAP32[$58>>2]|0;
 $60 = ($59|0)>(0);
 if ($60) {
  $61 = HEAP32[$1>>2]|0;
  $62 = (_subjournalPageIfRequired($61)|0);
  HEAP32[$rc>>2] = $62;
 }
 $63 = HEAP32[$pPager>>2]|0;
 $64 = ((($63)) + 28|0);
 $65 = HEAP32[$64>>2]|0;
 $66 = HEAP32[$1>>2]|0;
 $67 = ((($66)) + 20|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ($65>>>0)<($68>>>0);
 if ($69) {
  $70 = HEAP32[$1>>2]|0;
  $71 = ((($70)) + 20|0);
  $72 = HEAP32[$71>>2]|0;
  $73 = HEAP32[$pPager>>2]|0;
  $74 = ((($73)) + 28|0);
  HEAP32[$74>>2] = $72;
 }
 $75 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $75;
 $76 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($76|0);
}
function _pager_open_journal($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $flags = 0, $nSpill = 0, $pVfs = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $rc = sp + 12|0;
 $pVfs = sp + 8|0;
 $flags = sp + 4|0;
 $nSpill = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pVfs>>2] = $3;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 $8 = HEAP32[$1>>2]|0;
 if ($7) {
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $92 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($92|0);
 }
 $11 = (_sqlite3PagerUseWal($8)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 5|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = ($16|0)!=(2);
  if ($17) {
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 28|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (_sqlite3BitvecCreate($20)|0);
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 60|0);
   HEAP32[$23>>2] = $21;
   $24 = HEAP32[$1>>2]|0;
   $25 = ((($24)) + 60|0);
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)==(0|0);
   if ($27) {
    HEAP32[$0>>2] = 7;
    $92 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($92|0);
   }
   $28 = HEAP32[$1>>2]|0;
   $29 = ((($28)) + 68|0);
   $30 = HEAP32[$29>>2]|0;
   $31 = HEAP32[$30>>2]|0;
   $32 = ($31|0)!=(0|0);
   do {
    if (!($32)) {
     $33 = HEAP32[$1>>2]|0;
     $34 = ((($33)) + 5|0);
     $35 = HEAP8[$34>>0]|0;
     $36 = $35&255;
     $37 = ($36|0)==(4);
     if ($37) {
      $38 = HEAP32[$1>>2]|0;
      $39 = ((($38)) + 68|0);
      $40 = HEAP32[$39>>2]|0;
      _sqlite3MemJournalOpen($40);
      break;
     }
     HEAP32[$flags>>2] = 6;
     $41 = HEAP32[$1>>2]|0;
     $42 = ((($41)) + 13|0);
     $43 = HEAP8[$42>>0]|0;
     $44 = ($43<<24>>24)!=(0);
     $45 = HEAP32[$flags>>2]|0;
     if ($44) {
      $46 = $45 | 4104;
      HEAP32[$flags>>2] = $46;
      $47 = HEAP32[(44)>>2]|0;
      HEAP32[$nSpill>>2] = $47;
     } else {
      $48 = $45 | 2048;
      HEAP32[$flags>>2] = $48;
      HEAP32[$nSpill>>2] = 0;
     }
     $49 = HEAP32[$1>>2]|0;
     $50 = (_databaseIsUnmoved($49)|0);
     HEAP32[$rc>>2] = $50;
     $51 = HEAP32[$rc>>2]|0;
     $52 = ($51|0)==(0);
     if ($52) {
      $53 = HEAP32[$pVfs>>2]|0;
      $54 = HEAP32[$1>>2]|0;
      $55 = ((($54)) + 180|0);
      $56 = HEAP32[$55>>2]|0;
      $57 = HEAP32[$1>>2]|0;
      $58 = ((($57)) + 68|0);
      $59 = HEAP32[$58>>2]|0;
      $60 = HEAP32[$flags>>2]|0;
      $61 = HEAP32[$nSpill>>2]|0;
      $62 = (_sqlite3JournalOpen($53,$56,$59,$60,$61)|0);
      HEAP32[$rc>>2] = $62;
     }
    }
   } while(0);
   $63 = HEAP32[$rc>>2]|0;
   $64 = ($63|0)==(0);
   if ($64) {
    $65 = HEAP32[$1>>2]|0;
    $66 = ((($65)) + 48|0);
    HEAP32[$66>>2] = 0;
    $67 = HEAP32[$1>>2]|0;
    $68 = ((($67)) + 80|0);
    $69 = $68;
    $70 = $69;
    HEAP32[$70>>2] = 0;
    $71 = (($69) + 4)|0;
    $72 = $71;
    HEAP32[$72>>2] = 0;
    $73 = HEAP32[$1>>2]|0;
    $74 = ((($73)) + 20|0);
    HEAP8[$74>>0] = 0;
    $75 = HEAP32[$1>>2]|0;
    $76 = ((($75)) + 88|0);
    $77 = $76;
    $78 = $77;
    HEAP32[$78>>2] = 0;
    $79 = (($77) + 4)|0;
    $80 = $79;
    HEAP32[$80>>2] = 0;
    $81 = HEAP32[$1>>2]|0;
    $82 = (_writeJournalHdr($81)|0);
    HEAP32[$rc>>2] = $82;
   }
  }
 }
 $83 = HEAP32[$rc>>2]|0;
 $84 = ($83|0)!=(0);
 $85 = HEAP32[$1>>2]|0;
 if ($84) {
  $86 = ((($85)) + 60|0);
  $87 = HEAP32[$86>>2]|0;
  _sqlite3BitvecDestroy($87);
  $88 = HEAP32[$1>>2]|0;
  $89 = ((($88)) + 60|0);
  HEAP32[$89>>2] = 0;
 } else {
  $90 = ((($85)) + 17|0);
  HEAP8[$90>>0] = 3;
 }
 $91 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $91;
 $92 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($92|0);
}
function _pagerAddPageToRollbackJournal($pPg) {
 $pPg = $pPg|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0;
 var $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0;
 var $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0;
 var $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0;
 var $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0;
 var $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $cksum = 0, $iOff = 0, $pData2 = 0, $pPager = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $pPager = sp + 20|0;
 $rc = sp + 16|0;
 $cksum = sp + 12|0;
 $pData2 = sp + 8|0;
 $iOff = sp;
 HEAP32[$1>>2] = $pPg;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pPager>>2] = $4;
 $5 = HEAP32[$pPager>>2]|0;
 $6 = ((($5)) + 80|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $iOff;
 $14 = $13;
 HEAP32[$14>>2] = $9;
 $15 = (($13) + 4)|0;
 $16 = $15;
 HEAP32[$16>>2] = $12;
 $17 = HEAP32[$1>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$pData2>>2] = $19;
 $20 = HEAP32[$pPager>>2]|0;
 $21 = HEAP32[$pData2>>2]|0;
 $22 = (_pager_cksum($20,$21)|0);
 HEAP32[$cksum>>2] = $22;
 $23 = HEAP32[$1>>2]|0;
 $24 = ((($23)) + 24|0);
 $25 = HEAP16[$24>>1]|0;
 $26 = $25&65535;
 $27 = $26 | 8;
 $28 = $27&65535;
 HEAP16[$24>>1] = $28;
 $29 = HEAP32[$pPager>>2]|0;
 $30 = ((($29)) + 68|0);
 $31 = HEAP32[$30>>2]|0;
 $32 = $iOff;
 $33 = $32;
 $34 = HEAP32[$33>>2]|0;
 $35 = (($32) + 4)|0;
 $36 = $35;
 $37 = HEAP32[$36>>2]|0;
 $38 = HEAP32[$1>>2]|0;
 $39 = ((($38)) + 20|0);
 $40 = HEAP32[$39>>2]|0;
 $41 = (_write32bits($31,$34,$37,$40)|0);
 HEAP32[$rc>>2] = $41;
 $42 = HEAP32[$rc>>2]|0;
 $43 = ($42|0)!=(0);
 if ($43) {
  $44 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $44;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 }
 $45 = HEAP32[$pPager>>2]|0;
 $46 = ((($45)) + 68|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$pData2>>2]|0;
 $49 = HEAP32[$pPager>>2]|0;
 $50 = ((($49)) + 160|0);
 $51 = HEAP32[$50>>2]|0;
 $52 = $iOff;
 $53 = $52;
 $54 = HEAP32[$53>>2]|0;
 $55 = (($52) + 4)|0;
 $56 = $55;
 $57 = HEAP32[$56>>2]|0;
 $58 = (_i64Add(($54|0),($57|0),4,0)|0);
 $59 = tempRet0;
 $60 = (_sqlite3OsWrite($47,$48,$51,$58,$59)|0);
 HEAP32[$rc>>2] = $60;
 $61 = HEAP32[$rc>>2]|0;
 $62 = ($61|0)!=(0);
 if ($62) {
  $63 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $63;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 }
 $64 = HEAP32[$pPager>>2]|0;
 $65 = ((($64)) + 68|0);
 $66 = HEAP32[$65>>2]|0;
 $67 = $iOff;
 $68 = $67;
 $69 = HEAP32[$68>>2]|0;
 $70 = (($67) + 4)|0;
 $71 = $70;
 $72 = HEAP32[$71>>2]|0;
 $73 = HEAP32[$pPager>>2]|0;
 $74 = ((($73)) + 160|0);
 $75 = HEAP32[$74>>2]|0;
 $76 = ($75|0)<(0);
 $77 = $76 << 31 >> 31;
 $78 = (_i64Add(($69|0),($72|0),($75|0),($77|0))|0);
 $79 = tempRet0;
 $80 = (_i64Add(($78|0),($79|0),4,0)|0);
 $81 = tempRet0;
 $82 = HEAP32[$cksum>>2]|0;
 $83 = (_write32bits($66,$80,$81,$82)|0);
 HEAP32[$rc>>2] = $83;
 $84 = HEAP32[$rc>>2]|0;
 $85 = ($84|0)!=(0);
 if ($85) {
  $86 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $86;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 } else {
  $87 = HEAP32[$pPager>>2]|0;
  $88 = ((($87)) + 160|0);
  $89 = HEAP32[$88>>2]|0;
  $90 = (8 + ($89))|0;
  $91 = ($90|0)<(0);
  $92 = $91 << 31 >> 31;
  $93 = HEAP32[$pPager>>2]|0;
  $94 = ((($93)) + 80|0);
  $95 = $94;
  $96 = $95;
  $97 = HEAP32[$96>>2]|0;
  $98 = (($95) + 4)|0;
  $99 = $98;
  $100 = HEAP32[$99>>2]|0;
  $101 = (_i64Add(($97|0),($100|0),($90|0),($92|0))|0);
  $102 = tempRet0;
  $103 = $94;
  $104 = $103;
  HEAP32[$104>>2] = $101;
  $105 = (($103) + 4)|0;
  $106 = $105;
  HEAP32[$106>>2] = $102;
  $107 = HEAP32[$pPager>>2]|0;
  $108 = ((($107)) + 48|0);
  $109 = HEAP32[$108>>2]|0;
  $110 = (($109) + 1)|0;
  HEAP32[$108>>2] = $110;
  $111 = HEAP32[$pPager>>2]|0;
  $112 = ((($111)) + 60|0);
  $113 = HEAP32[$112>>2]|0;
  $114 = HEAP32[$1>>2]|0;
  $115 = ((($114)) + 20|0);
  $116 = HEAP32[$115>>2]|0;
  $117 = (_sqlite3BitvecSet($113,$116)|0);
  HEAP32[$rc>>2] = $117;
  $118 = HEAP32[$pPager>>2]|0;
  $119 = HEAP32[$1>>2]|0;
  $120 = ((($119)) + 20|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = (_addToSavepointBitvecs($118,$121)|0);
  $123 = HEAP32[$rc>>2]|0;
  $124 = $123 | $122;
  HEAP32[$rc>>2] = $124;
  $125 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $125;
  $126 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($126|0);
 }
 return (0)|0;
}
function _sqlite3MemJournalOpen($pJfd) {
 $pJfd = $pJfd|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pJfd;
 $1 = HEAP32[$0>>2]|0;
 (_sqlite3JournalOpen(0,0,$1,0,-1)|0);
 STACKTOP = sp;return;
}
function _databaseIsUnmoved($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $bHasMoved = 0, $or$cond = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $bHasMoved = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$bHasMoved>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 13|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 28|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)==(0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $21 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($21|0);
 }
 $10 = HEAP32[$1>>2]|0;
 $11 = ((($10)) + 64|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3OsFileControl($12,20,$bHasMoved)|0);
 HEAP32[$rc>>2] = $13;
 $14 = HEAP32[$rc>>2]|0;
 $15 = ($14|0)==(12);
 if ($15) {
  HEAP32[$rc>>2] = 0;
 } else {
  $16 = HEAP32[$rc>>2]|0;
  $17 = ($16|0)==(0);
  $18 = HEAP32[$bHasMoved>>2]|0;
  $19 = ($18|0)!=(0);
  $or$cond = $17 & $19;
  if ($or$cond) {
   HEAP32[$rc>>2] = 1032;
  }
 }
 $20 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $20;
 $21 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _pagerAcquireMapPage($pPager,$pgno,$pData,$ppPage) {
 $pPager = $pPager|0;
 $pgno = $pgno|0;
 $pData = $pData|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $4 = sp + 4|0;
 $p = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $pData;
 HEAP32[$4>>2] = $ppPage;
 $5 = HEAP32[$1>>2]|0;
 $6 = ((($5)) + 144|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ($7|0)!=(0|0);
 $9 = HEAP32[$1>>2]|0;
 do {
  if ($8) {
   $10 = ((($9)) + 144|0);
   $11 = HEAP32[$10>>2]|0;
   HEAP32[$p>>2] = $11;
   $12 = HEAP32[$4>>2]|0;
   HEAP32[$12>>2] = $11;
   $13 = HEAP32[$p>>2]|0;
   $14 = ((($13)) + 12|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 144|0);
   HEAP32[$17>>2] = $15;
   $18 = HEAP32[$p>>2]|0;
   $19 = ((($18)) + 12|0);
   HEAP32[$19>>2] = 0;
   $20 = HEAP32[$p>>2]|0;
   $21 = ((($20)) + 8|0);
   $22 = HEAP32[$21>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 148|0);
   $25 = HEAP16[$24>>1]|0;
   $26 = $25&65535;
   _memset(($22|0),0,($26|0))|0;
  } else {
   $27 = ((($9)) + 148|0);
   $28 = HEAP16[$27>>1]|0;
   $29 = $28&65535;
   $30 = (40 + ($29))|0;
   $31 = (_sqlite3MallocZero($30,0)|0);
   HEAP32[$p>>2] = $31;
   $32 = HEAP32[$4>>2]|0;
   HEAP32[$32>>2] = $31;
   $33 = HEAP32[$p>>2]|0;
   $34 = ($33|0)==(0|0);
   if (!($34)) {
    $48 = HEAP32[$p>>2]|0;
    $49 = ((($48)) + 40|0);
    $50 = HEAP32[$p>>2]|0;
    $51 = ((($50)) + 8|0);
    HEAP32[$51>>2] = $49;
    $52 = HEAP32[$p>>2]|0;
    $53 = ((($52)) + 24|0);
    HEAP16[$53>>1] = 32;
    $54 = HEAP32[$p>>2]|0;
    $55 = ((($54)) + 26|0);
    HEAP16[$55>>1] = 1;
    $56 = HEAP32[$1>>2]|0;
    $57 = HEAP32[$p>>2]|0;
    $58 = ((($57)) + 16|0);
    HEAP32[$58>>2] = $56;
    break;
   }
   $35 = HEAP32[$1>>2]|0;
   $36 = ((($35)) + 64|0);
   $37 = HEAP32[$36>>2]|0;
   $38 = HEAP32[$2>>2]|0;
   $39 = (($38) - 1)|0;
   $40 = HEAP32[$1>>2]|0;
   $41 = ((($40)) + 160|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)<(0);
   $44 = $43 << 31 >> 31;
   $45 = (___muldi3(($39|0),0,($42|0),($44|0))|0);
   $46 = tempRet0;
   $47 = HEAP32[$3>>2]|0;
   (_sqlite3OsUnfetch($37,$45,$46,$47)|0);
   HEAP32[$0>>2] = 7;
   $69 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($69|0);
  }
 } while(0);
 $59 = HEAP32[$2>>2]|0;
 $60 = HEAP32[$p>>2]|0;
 $61 = ((($60)) + 20|0);
 HEAP32[$61>>2] = $59;
 $62 = HEAP32[$3>>2]|0;
 $63 = HEAP32[$p>>2]|0;
 $64 = ((($63)) + 4|0);
 HEAP32[$64>>2] = $62;
 $65 = HEAP32[$1>>2]|0;
 $66 = ((($65)) + 128|0);
 $67 = HEAP32[$66>>2]|0;
 $68 = (($67) + 1)|0;
 HEAP32[$66>>2] = $68;
 HEAP32[$0>>2] = 0;
 $69 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($69|0);
}
function _sqlite3PcacheFetchStress($pCache,$pgno,$ppPage) {
 $pCache = $pCache|0;
 $pgno = $pgno|0;
 $ppPage = $ppPage|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $or$cond = 0;
 var $pPg = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $pPg = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pCache;
 HEAP32[$2>>2] = $pgno;
 HEAP32[$3>>2] = $ppPage;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 33|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = ($7|0)==(2);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = (_sqlite3PcachePagecount($9)|0);
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 20|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)>($13|0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = ((($15)) + 8|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$pPg>>2] = $17;
  while(1) {
   $18 = HEAP32[$pPg>>2]|0;
   $19 = ($18|0)!=(0|0);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$pPg>>2]|0;
   $21 = ((($20)) + 26|0);
   $22 = HEAP16[$21>>1]|0;
   $23 = $22 << 16 >> 16;
   $24 = ($23|0)!=(0);
   if (!($24)) {
    $25 = HEAP32[$pPg>>2]|0;
    $26 = ((($25)) + 24|0);
    $27 = HEAP16[$26>>1]|0;
    $28 = $27&65535;
    $29 = $28 & 8;
    $30 = ($29|0)!=(0);
    if (!($30)) {
     break;
    }
   }
   $31 = HEAP32[$pPg>>2]|0;
   $32 = ((($31)) + 36|0);
   $33 = HEAP32[$32>>2]|0;
   HEAP32[$pPg>>2] = $33;
  }
  $34 = HEAP32[$pPg>>2]|0;
  $35 = HEAP32[$1>>2]|0;
  $36 = ((($35)) + 8|0);
  HEAP32[$36>>2] = $34;
  $37 = HEAP32[$pPg>>2]|0;
  $38 = ($37|0)!=(0|0);
  L14: do {
   if (!($38)) {
    $39 = HEAP32[$1>>2]|0;
    $40 = ((($39)) + 4|0);
    $41 = HEAP32[$40>>2]|0;
    HEAP32[$pPg>>2] = $41;
    while(1) {
     $42 = HEAP32[$pPg>>2]|0;
     $43 = ($42|0)!=(0|0);
     if (!($43)) {
      break L14;
     }
     $44 = HEAP32[$pPg>>2]|0;
     $45 = ((($44)) + 26|0);
     $46 = HEAP16[$45>>1]|0;
     $47 = $46 << 16 >> 16;
     $48 = ($47|0)!=(0);
     if (!($48)) {
      break L14;
     }
     $49 = HEAP32[$pPg>>2]|0;
     $50 = ((($49)) + 36|0);
     $51 = HEAP32[$50>>2]|0;
     HEAP32[$pPg>>2] = $51;
    }
   }
  } while(0);
  $52 = HEAP32[$pPg>>2]|0;
  $53 = ($52|0)!=(0|0);
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 36|0);
   $56 = HEAP32[$55>>2]|0;
   $57 = HEAP32[$1>>2]|0;
   $58 = ((($57)) + 40|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = HEAP32[$pPg>>2]|0;
   $61 = (FUNCTION_TABLE_iii[$56 & 255]($59,$60)|0);
   HEAP32[$rc>>2] = $61;
   $62 = HEAP32[$rc>>2]|0;
   $63 = ($62|0)!=(0);
   $64 = HEAP32[$rc>>2]|0;
   $65 = ($64|0)!=(5);
   $or$cond = $63 & $65;
   if ($or$cond) {
    $66 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $66;
    $78 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($78|0);
   }
  }
 }
 $67 = HEAP32[(144)>>2]|0;
 $68 = HEAP32[$1>>2]|0;
 $69 = ((($68)) + 44|0);
 $70 = HEAP32[$69>>2]|0;
 $71 = HEAP32[$2>>2]|0;
 $72 = (FUNCTION_TABLE_iiii[$67 & 255]($70,$71,2)|0);
 $73 = HEAP32[$3>>2]|0;
 HEAP32[$73>>2] = $72;
 $74 = HEAP32[$3>>2]|0;
 $75 = HEAP32[$74>>2]|0;
 $76 = ($75|0)==(0|0);
 $77 = $76 ? 7 : 0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3PcachePagecount($pCache) {
 $pCache = $pCache|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pCache;
 $1 = HEAP32[(140)>>2]|0;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 44|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = (FUNCTION_TABLE_ii[$1 & 255]($4)|0);
 STACKTOP = sp;return ($5|0);
}
function _sqlite3PagerDataVersion($pPager) {
 $pPager = $pPager|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $pPager;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 108|0);
 $3 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($3|0);
}
function _lockBtree($pBt) {
 $pBt = $pBt|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $isOpen = 0, $nPage = 0, $nPageFile = 0, $nPageHeader = 0, $or$cond = 0, $or$cond3 = 0, $pPage1 = 0, $page1 = 0, $pageSize = 0, $rc = 0, $usableSize = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $rc = sp + 32|0;
 $pPage1 = sp + 28|0;
 $nPage = sp + 24|0;
 $nPageFile = sp + 20|0;
 $nPageHeader = sp + 16|0;
 $pageSize = sp + 12|0;
 $usableSize = sp + 8|0;
 $page1 = sp + 4|0;
 $isOpen = sp;
 HEAP32[$1>>2] = $pBt;
 HEAP32[$nPageFile>>2] = 0;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = (_sqlite3PagerSharedLock($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 $6 = ($5|0)!=(0);
 if ($6) {
  $7 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $7;
  $222 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = (_btreeGetPage($8,1,$pPage1,0)|0);
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 $11 = ($10|0)!=(0);
 if ($11) {
  $12 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $12;
  $222 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($222|0);
 }
 $13 = HEAP32[$pPage1>>2]|0;
 $14 = ((($13)) + 56|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 28|0);
 $17 = (_sqlite3Get4byte($16)|0);
 HEAP32[$nPageHeader>>2] = $17;
 HEAP32[$nPage>>2] = $17;
 $18 = HEAP32[$1>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 _sqlite3PagerPagecount($19,$nPageFile);
 $20 = HEAP32[$nPage>>2]|0;
 $21 = ($20|0)==(0);
 if ($21) {
  label = 7;
 } else {
  $22 = HEAP32[$pPage1>>2]|0;
  $23 = ((($22)) + 56|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 24|0);
  $26 = HEAP32[$pPage1>>2]|0;
  $27 = ((($26)) + 56|0);
  $28 = HEAP32[$27>>2]|0;
  $29 = ((($28)) + 92|0);
  $30 = (_memcmp($25,$29,4)|0);
  $31 = ($30|0)!=(0);
  if ($31) {
   label = 7;
  }
 }
 if ((label|0) == 7) {
  $32 = HEAP32[$nPageFile>>2]|0;
  HEAP32[$nPage>>2] = $32;
 }
 $33 = HEAP32[$nPage>>2]|0;
 $34 = ($33|0)>(0);
 L14: do {
  if ($34) {
   $35 = HEAP32[$pPage1>>2]|0;
   $36 = ((($35)) + 56|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$page1>>2] = $37;
   HEAP32[$rc>>2] = 26;
   $38 = HEAP32[$page1>>2]|0;
   $39 = (_memcmp($38,41796,16)|0);
   $40 = ($39|0)!=(0);
   L16: do {
    if (!($40)) {
     $41 = HEAP32[$page1>>2]|0;
     $42 = ((($41)) + 18|0);
     $43 = HEAP8[$42>>0]|0;
     $44 = $43&255;
     $45 = ($44|0)>(2);
     if ($45) {
      $46 = HEAP32[$1>>2]|0;
      $47 = ((($46)) + 22|0);
      $48 = HEAP16[$47>>1]|0;
      $49 = $48&65535;
      $50 = $49 | 1;
      $51 = $50&65535;
      HEAP16[$47>>1] = $51;
     }
     $52 = HEAP32[$page1>>2]|0;
     $53 = ((($52)) + 19|0);
     $54 = HEAP8[$53>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)>(2);
     if (!($56)) {
      $57 = HEAP32[$page1>>2]|0;
      $58 = ((($57)) + 19|0);
      $59 = HEAP8[$58>>0]|0;
      $60 = $59&255;
      $61 = ($60|0)==(2);
      do {
       if ($61) {
        $62 = HEAP32[$1>>2]|0;
        $63 = ((($62)) + 22|0);
        $64 = HEAP16[$63>>1]|0;
        $65 = $64&65535;
        $66 = $65 & 16;
        $67 = ($66|0)==(0);
        if ($67) {
         HEAP32[$isOpen>>2] = 0;
         $68 = HEAP32[$1>>2]|0;
         $69 = HEAP32[$68>>2]|0;
         $70 = (_sqlite3PagerOpenWal($69,$isOpen)|0);
         HEAP32[$rc>>2] = $70;
         $71 = HEAP32[$rc>>2]|0;
         $72 = ($71|0)!=(0);
         if ($72) {
          break L16;
         }
         $73 = HEAP32[$isOpen>>2]|0;
         $74 = ($73|0)==(0);
         if (!($74)) {
          HEAP32[$rc>>2] = 26;
          break;
         }
         $75 = HEAP32[$pPage1>>2]|0;
         _releasePage($75);
         HEAP32[$0>>2] = 0;
         $222 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($222|0);
        }
       }
      } while(0);
      $76 = HEAP32[$page1>>2]|0;
      $77 = ((($76)) + 21|0);
      $78 = (_memcmp($77,44049,3)|0);
      $79 = ($78|0)!=(0);
      if (!($79)) {
       $80 = HEAP32[$page1>>2]|0;
       $81 = ((($80)) + 16|0);
       $82 = HEAP8[$81>>0]|0;
       $83 = $82&255;
       $84 = $83 << 8;
       $85 = HEAP32[$page1>>2]|0;
       $86 = ((($85)) + 17|0);
       $87 = HEAP8[$86>>0]|0;
       $88 = $87&255;
       $89 = $88 << 16;
       $90 = $84 | $89;
       HEAP32[$pageSize>>2] = $90;
       $91 = HEAP32[$pageSize>>2]|0;
       $92 = (($91) - 1)|0;
       $93 = HEAP32[$pageSize>>2]|0;
       $94 = $92 & $93;
       $95 = ($94|0)!=(0);
       $96 = HEAP32[$pageSize>>2]|0;
       $97 = ($96>>>0)>(65536);
       $or$cond = $95 | $97;
       $98 = HEAP32[$pageSize>>2]|0;
       $99 = ($98>>>0)<=(256);
       $or$cond3 = $or$cond | $99;
       if (!($or$cond3)) {
        $100 = HEAP32[$pageSize>>2]|0;
        $101 = HEAP32[$page1>>2]|0;
        $102 = ((($101)) + 20|0);
        $103 = HEAP8[$102>>0]|0;
        $104 = $103&255;
        $105 = (($100) - ($104))|0;
        HEAP32[$usableSize>>2] = $105;
        $106 = HEAP32[$pageSize>>2]|0;
        $107 = HEAP32[$1>>2]|0;
        $108 = ((($107)) + 32|0);
        $109 = HEAP32[$108>>2]|0;
        $110 = ($106|0)!=($109|0);
        if ($110) {
         $111 = HEAP32[$pPage1>>2]|0;
         _releasePage($111);
         $112 = HEAP32[$usableSize>>2]|0;
         $113 = HEAP32[$1>>2]|0;
         $114 = ((($113)) + 36|0);
         HEAP32[$114>>2] = $112;
         $115 = HEAP32[$pageSize>>2]|0;
         $116 = HEAP32[$1>>2]|0;
         $117 = ((($116)) + 32|0);
         HEAP32[$117>>2] = $115;
         $118 = HEAP32[$1>>2]|0;
         _freeTempSpace($118);
         $119 = HEAP32[$1>>2]|0;
         $120 = HEAP32[$119>>2]|0;
         $121 = HEAP32[$1>>2]|0;
         $122 = ((($121)) + 32|0);
         $123 = HEAP32[$pageSize>>2]|0;
         $124 = HEAP32[$usableSize>>2]|0;
         $125 = (($123) - ($124))|0;
         $126 = (_sqlite3PagerSetPagesize($120,$122,$125)|0);
         HEAP32[$rc>>2] = $126;
         $127 = HEAP32[$rc>>2]|0;
         HEAP32[$0>>2] = $127;
         $222 = HEAP32[$0>>2]|0;
         STACKTOP = sp;return ($222|0);
        }
        $128 = HEAP32[$1>>2]|0;
        $129 = ((($128)) + 4|0);
        $130 = HEAP32[$129>>2]|0;
        $131 = ((($130)) + 24|0);
        $132 = HEAP32[$131>>2]|0;
        $133 = $132 & 65536;
        $134 = ($133|0)==(0);
        if ($134) {
         $135 = HEAP32[$nPage>>2]|0;
         $136 = HEAP32[$nPageFile>>2]|0;
         $137 = ($135|0)>($136|0);
         if ($137) {
          $138 = (_sqlite3CorruptError(61191)|0);
          HEAP32[$rc>>2] = $138;
          break;
         }
        }
        $139 = HEAP32[$usableSize>>2]|0;
        $140 = ($139>>>0)<(480);
        if (!($140)) {
         $141 = HEAP32[$pageSize>>2]|0;
         $142 = HEAP32[$1>>2]|0;
         $143 = ((($142)) + 32|0);
         HEAP32[$143>>2] = $141;
         $144 = HEAP32[$usableSize>>2]|0;
         $145 = HEAP32[$1>>2]|0;
         $146 = ((($145)) + 36|0);
         HEAP32[$146>>2] = $144;
         $147 = HEAP32[$page1>>2]|0;
         $148 = ((($147)) + 52|0);
         $149 = (_sqlite3Get4byte($148)|0);
         $150 = ($149|0)!=(0);
         $151 = $150 ? 1 : 0;
         $152 = $151&255;
         $153 = HEAP32[$1>>2]|0;
         $154 = ((($153)) + 17|0);
         HEAP8[$154>>0] = $152;
         $155 = HEAP32[$page1>>2]|0;
         $156 = ((($155)) + 64|0);
         $157 = (_sqlite3Get4byte($156)|0);
         $158 = ($157|0)!=(0);
         $159 = $158 ? 1 : 0;
         $160 = $159&255;
         $161 = HEAP32[$1>>2]|0;
         $162 = ((($161)) + 18|0);
         HEAP8[$162>>0] = $160;
         break L14;
        }
       }
      }
     }
    }
   } while(0);
   $218 = HEAP32[$pPage1>>2]|0;
   _releasePage($218);
   $219 = HEAP32[$1>>2]|0;
   $220 = ((($219)) + 12|0);
   HEAP32[$220>>2] = 0;
   $221 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $221;
   $222 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($222|0);
  }
 } while(0);
 $163 = HEAP32[$1>>2]|0;
 $164 = ((($163)) + 36|0);
 $165 = HEAP32[$164>>2]|0;
 $166 = (($165) - 12)|0;
 $167 = $166<<6;
 $168 = (($167>>>0) / 255)&-1;
 $169 = (($168) - 23)|0;
 $170 = $169&65535;
 $171 = HEAP32[$1>>2]|0;
 $172 = ((($171)) + 24|0);
 HEAP16[$172>>1] = $170;
 $173 = HEAP32[$1>>2]|0;
 $174 = ((($173)) + 36|0);
 $175 = HEAP32[$174>>2]|0;
 $176 = (($175) - 12)|0;
 $177 = $176<<5;
 $178 = (($177>>>0) / 255)&-1;
 $179 = (($178) - 23)|0;
 $180 = $179&65535;
 $181 = HEAP32[$1>>2]|0;
 $182 = ((($181)) + 26|0);
 HEAP16[$182>>1] = $180;
 $183 = HEAP32[$1>>2]|0;
 $184 = ((($183)) + 36|0);
 $185 = HEAP32[$184>>2]|0;
 $186 = (($185) - 35)|0;
 $187 = $186&65535;
 $188 = HEAP32[$1>>2]|0;
 $189 = ((($188)) + 28|0);
 HEAP16[$189>>1] = $187;
 $190 = HEAP32[$1>>2]|0;
 $191 = ((($190)) + 36|0);
 $192 = HEAP32[$191>>2]|0;
 $193 = (($192) - 12)|0;
 $194 = $193<<5;
 $195 = (($194>>>0) / 255)&-1;
 $196 = (($195) - 23)|0;
 $197 = $196&65535;
 $198 = HEAP32[$1>>2]|0;
 $199 = ((($198)) + 30|0);
 HEAP16[$199>>1] = $197;
 $200 = HEAP32[$1>>2]|0;
 $201 = ((($200)) + 24|0);
 $202 = HEAP16[$201>>1]|0;
 $203 = $202&65535;
 $204 = ($203|0)>(127);
 $205 = HEAP32[$1>>2]|0;
 if ($204) {
  $206 = ((($205)) + 21|0);
  HEAP8[$206>>0] = 127;
 } else {
  $207 = ((($205)) + 24|0);
  $208 = HEAP16[$207>>1]|0;
  $209 = $208&255;
  $210 = HEAP32[$1>>2]|0;
  $211 = ((($210)) + 21|0);
  HEAP8[$211>>0] = $209;
 }
 $212 = HEAP32[$pPage1>>2]|0;
 $213 = HEAP32[$1>>2]|0;
 $214 = ((($213)) + 12|0);
 HEAP32[$214>>2] = $212;
 $215 = HEAP32[$nPage>>2]|0;
 $216 = HEAP32[$1>>2]|0;
 $217 = ((($216)) + 44|0);
 HEAP32[$217>>2] = $215;
 HEAP32[$0>>2] = 0;
 $222 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($222|0);
}
function _sqlite3PagerBegin($pPager,$exFlag,$subjInMemory) {
 $pPager = $pPager|0;
 $exFlag = $exFlag|0;
 $subjInMemory = $subjInMemory|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $8 = 0, $9 = 0, $or$cond = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $3 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pPager;
 HEAP32[$2>>2] = $exFlag;
 HEAP32[$3>>2] = $subjInMemory;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 44|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 44|0);
  $10 = HEAP32[$9>>2]|0;
  HEAP32[$0>>2] = $10;
  $78 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($78|0);
 }
 $11 = HEAP32[$3>>2]|0;
 $12 = $11&255;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 22|0);
 HEAP8[$14>>0] = $12;
 $15 = HEAP32[$1>>2]|0;
 $16 = ((($15)) + 17|0);
 $17 = HEAP8[$16>>0]|0;
 $18 = $17&255;
 $19 = ($18|0)==(1);
 if ($19) {
  $20 = HEAP32[$1>>2]|0;
  $21 = (_sqlite3PagerUseWal($20)|0);
  $22 = ($21|0)!=(0);
  $23 = HEAP32[$1>>2]|0;
  if ($22) {
   $24 = ((($23)) + 4|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = ($26|0)!=(0);
   do {
    if ($27) {
     $28 = HEAP32[$1>>2]|0;
     $29 = ((($28)) + 216|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = (_sqlite3WalExclusiveMode($30,-1)|0);
     $32 = ($31|0)!=(0);
     if ($32) {
      $33 = HEAP32[$1>>2]|0;
      $34 = (_pagerLockDb($33,4)|0);
      HEAP32[$rc>>2] = $34;
      $35 = HEAP32[$rc>>2]|0;
      $36 = ($35|0)!=(0);
      if (!($36)) {
       $38 = HEAP32[$1>>2]|0;
       $39 = ((($38)) + 216|0);
       $40 = HEAP32[$39>>2]|0;
       (_sqlite3WalExclusiveMode($40,1)|0);
       break;
      }
      $37 = HEAP32[$rc>>2]|0;
      HEAP32[$0>>2] = $37;
      $78 = HEAP32[$0>>2]|0;
      STACKTOP = sp;return ($78|0);
     }
    }
   } while(0);
   $41 = HEAP32[$1>>2]|0;
   $42 = ((($41)) + 216|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = (_sqlite3WalBeginWriteTransaction($43)|0);
   HEAP32[$rc>>2] = $44;
  } else {
   $45 = (_pagerLockDb($23,2)|0);
   HEAP32[$rc>>2] = $45;
   $46 = HEAP32[$rc>>2]|0;
   $47 = ($46|0)==(0);
   $48 = HEAP32[$2>>2]|0;
   $49 = ($48|0)!=(0);
   $or$cond = $47 & $49;
   if ($or$cond) {
    $50 = HEAP32[$1>>2]|0;
    $51 = (_pager_wait_on_lock($50,4)|0);
    HEAP32[$rc>>2] = $51;
   }
  }
  $52 = HEAP32[$rc>>2]|0;
  $53 = ($52|0)==(0);
  if ($53) {
   $54 = HEAP32[$1>>2]|0;
   $55 = ((($54)) + 17|0);
   HEAP8[$55>>0] = 2;
   $56 = HEAP32[$1>>2]|0;
   $57 = ((($56)) + 28|0);
   $58 = HEAP32[$57>>2]|0;
   $59 = HEAP32[$1>>2]|0;
   $60 = ((($59)) + 40|0);
   HEAP32[$60>>2] = $58;
   $61 = HEAP32[$1>>2]|0;
   $62 = ((($61)) + 28|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = HEAP32[$1>>2]|0;
   $65 = ((($64)) + 36|0);
   HEAP32[$65>>2] = $63;
   $66 = HEAP32[$1>>2]|0;
   $67 = ((($66)) + 28|0);
   $68 = HEAP32[$67>>2]|0;
   $69 = HEAP32[$1>>2]|0;
   $70 = ((($69)) + 32|0);
   HEAP32[$70>>2] = $68;
   $71 = HEAP32[$1>>2]|0;
   $72 = ((($71)) + 80|0);
   $73 = $72;
   $74 = $73;
   HEAP32[$74>>2] = 0;
   $75 = (($73) + 4)|0;
   $76 = $75;
   HEAP32[$76>>2] = 0;
  }
 }
 $77 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $77;
 $78 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($78|0);
}
function _sqlite3WalBeginWriteTransaction($pWal) {
 $pWal = $pWal|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $rc = sp;
 HEAP32[$1>>2] = $pWal;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 46|0);
 $4 = HEAP8[$3>>0]|0;
 $5 = ($4<<24>>24)!=(0);
 if ($5) {
  HEAP32[$0>>2] = 8;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = (_walLockExclusive($6,0,1)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $10;
  $23 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = ((($11)) + 44|0);
 HEAP8[$12>>0] = 1;
 $13 = HEAP32[$1>>2]|0;
 $14 = ((($13)) + 52|0);
 $15 = HEAP32[$1>>2]|0;
 $16 = (_walIndexHdr($15)|0);
 $17 = (_memcmp($14,$16,48)|0);
 $18 = ($17|0)!=(0);
 if ($18) {
  $19 = HEAP32[$1>>2]|0;
  _walUnlockExclusive($19,0,1);
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 44|0);
  HEAP8[$21>>0] = 0;
  HEAP32[$rc>>2] = 517;
 }
 $22 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $22;
 $23 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _sqlite3ErrorFinish($db,$err_code) {
 $db = $db|0;
 $err_code = $err_code|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $err_code;
 $2 = HEAP32[$0>>2]|0;
 $3 = ((($2)) + 244|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)!=(0|0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 244|0);
  $8 = HEAP32[$7>>2]|0;
  _sqlite3ValueSetNull($8);
 }
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$1>>2]|0;
 _sqlite3SystemError($9,$10);
 STACKTOP = sp;return;
}
function _sqlite3ValueSetNull($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _sqlite3VdbeMemSetNull($1);
 STACKTOP = sp;return;
}
function _unixOpen($pVfs,$zPath,$pFile,$flags,$pOutFlags) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $pFile = $pFile|0;
 $flags = $flags|0;
 $pOutFlags = $pOutFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $ctrlFlags = 0, $eType = 0, $fd = 0, $gid = 0, $isCreate = 0, $isDelete = 0, $isExclusive = 0;
 var $isReadWrite = 0, $isReadonly = 0, $noLock = 0, $openFlags = 0, $openMode = 0, $or$cond = 0, $or$cond3 = 0, $p = 0, $pUnused = 0, $rc = 0, $syncDir = 0, $uid = 0, $zName = 0, $zTmpname = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 624|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $4 = sp + 76|0;
 $5 = sp + 72|0;
 $p = sp + 68|0;
 $fd = sp + 64|0;
 $openFlags = sp + 60|0;
 $eType = sp + 56|0;
 $noLock = sp + 52|0;
 $rc = sp + 48|0;
 $ctrlFlags = sp + 44|0;
 $isExclusive = sp + 40|0;
 $isDelete = sp + 36|0;
 $isCreate = sp + 32|0;
 $isReadonly = sp + 28|0;
 $isReadWrite = sp + 24|0;
 $syncDir = sp + 20|0;
 $zTmpname = sp + 96|0;
 $zName = sp + 16|0;
 $pUnused = sp + 12|0;
 $openMode = sp + 8|0;
 $uid = sp + 4|0;
 $gid = sp;
 HEAP32[$1>>2] = $pVfs;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $pFile;
 HEAP32[$4>>2] = $flags;
 HEAP32[$5>>2] = $pOutFlags;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$p>>2] = $6;
 HEAP32[$fd>>2] = -1;
 HEAP32[$openFlags>>2] = 0;
 $7 = HEAP32[$4>>2]|0;
 $8 = $7 & -256;
 HEAP32[$eType>>2] = $8;
 HEAP32[$rc>>2] = 0;
 HEAP32[$ctrlFlags>>2] = 0;
 $9 = HEAP32[$4>>2]|0;
 $10 = $9 & 16;
 HEAP32[$isExclusive>>2] = $10;
 $11 = HEAP32[$4>>2]|0;
 $12 = $11 & 8;
 HEAP32[$isDelete>>2] = $12;
 $13 = HEAP32[$4>>2]|0;
 $14 = $13 & 4;
 HEAP32[$isCreate>>2] = $14;
 $15 = HEAP32[$4>>2]|0;
 $16 = $15 & 1;
 HEAP32[$isReadonly>>2] = $16;
 $17 = HEAP32[$4>>2]|0;
 $18 = $17 & 2;
 HEAP32[$isReadWrite>>2] = $18;
 $19 = HEAP32[$isCreate>>2]|0;
 $20 = ($19|0)!=(0);
 if ($20) {
  $21 = HEAP32[$eType>>2]|0;
  $22 = ($21|0)==(16384);
  $23 = HEAP32[$eType>>2]|0;
  $24 = ($23|0)==(2048);
  $or$cond = $22 | $24;
  if ($or$cond) {
   $28 = 1;
  } else {
   $25 = HEAP32[$eType>>2]|0;
   $26 = ($25|0)==(524288);
   $28 = $26;
  }
 } else {
  $28 = 0;
 }
 $27 = $28&1;
 HEAP32[$syncDir>>2] = $27;
 $29 = HEAP32[$2>>2]|0;
 HEAP32[$zName>>2] = $29;
 $30 = HEAP32[6872>>2]|0;
 $31 = (_getpid()|0);
 $32 = ($30|0)!=($31|0);
 if ($32) {
  $33 = (_getpid()|0);
  HEAP32[6872>>2] = $33;
  _sqlite3_randomness(0,0);
 }
 $34 = HEAP32[$p>>2]|0;
 dest=$34; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $35 = HEAP32[$eType>>2]|0;
 $36 = ($35|0)==(256);
 $37 = HEAP32[$zName>>2]|0;
 do {
  if ($36) {
   $38 = HEAP32[$4>>2]|0;
   $39 = (_findReusableFd($37,$38)|0);
   HEAP32[$pUnused>>2] = $39;
   $40 = HEAP32[$pUnused>>2]|0;
   $41 = ($40|0)!=(0|0);
   if ($41) {
    $42 = HEAP32[$pUnused>>2]|0;
    $43 = HEAP32[$42>>2]|0;
    HEAP32[$fd>>2] = $43;
   } else {
    $44 = (_sqlite3_malloc64(12,0)|0);
    HEAP32[$pUnused>>2] = $44;
    $45 = HEAP32[$pUnused>>2]|0;
    $46 = ($45|0)!=(0|0);
    if (!($46)) {
     HEAP32[$0>>2] = 7;
     $172 = HEAP32[$0>>2]|0;
     STACKTOP = sp;return ($172|0);
    }
   }
   $47 = HEAP32[$pUnused>>2]|0;
   $48 = HEAP32[$p>>2]|0;
   $49 = ((($48)) + 28|0);
   HEAP32[$49>>2] = $47;
  } else {
   $50 = ($37|0)!=(0|0);
   if (!($50)) {
    $51 = HEAP32[$1>>2]|0;
    $52 = ((($51)) + 8|0);
    $53 = HEAP32[$52>>2]|0;
    $54 = (_unixGetTempname($53,$zTmpname)|0);
    HEAP32[$rc>>2] = $54;
    $55 = HEAP32[$rc>>2]|0;
    $56 = ($55|0)!=(0);
    if (!($56)) {
     HEAP32[$zName>>2] = $zTmpname;
     break;
    }
    $57 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $57;
    $172 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($172|0);
   }
  }
 } while(0);
 $58 = HEAP32[$isReadonly>>2]|0;
 $59 = ($58|0)!=(0);
 if ($59) {
  $60 = HEAP32[$openFlags>>2]|0;
  HEAP32[$openFlags>>2] = $60;
 }
 $61 = HEAP32[$isReadWrite>>2]|0;
 $62 = ($61|0)!=(0);
 if ($62) {
  $63 = HEAP32[$openFlags>>2]|0;
  $64 = $63 | 2;
  HEAP32[$openFlags>>2] = $64;
 }
 $65 = HEAP32[$isCreate>>2]|0;
 $66 = ($65|0)!=(0);
 if ($66) {
  $67 = HEAP32[$openFlags>>2]|0;
  $68 = $67 | 64;
  HEAP32[$openFlags>>2] = $68;
 }
 $69 = HEAP32[$isExclusive>>2]|0;
 $70 = ($69|0)!=(0);
 if ($70) {
  $71 = HEAP32[$openFlags>>2]|0;
  $72 = $71 | 131200;
  HEAP32[$openFlags>>2] = $72;
 }
 $73 = HEAP32[$openFlags>>2]|0;
 HEAP32[$openFlags>>2] = $73;
 $74 = HEAP32[$fd>>2]|0;
 $75 = ($74|0)<(0);
 do {
  if ($75) {
   $76 = HEAP32[$zName>>2]|0;
   $77 = HEAP32[$4>>2]|0;
   $78 = (_findCreateFileMode($76,$77,$openMode,$uid,$gid)|0);
   HEAP32[$rc>>2] = $78;
   $79 = HEAP32[$rc>>2]|0;
   $80 = ($79|0)!=(0);
   if ($80) {
    $81 = HEAP32[$rc>>2]|0;
    HEAP32[$0>>2] = $81;
    $172 = HEAP32[$0>>2]|0;
    STACKTOP = sp;return ($172|0);
   }
   $82 = HEAP32[$zName>>2]|0;
   $83 = HEAP32[$openFlags>>2]|0;
   $84 = HEAP32[$openMode>>2]|0;
   $85 = (_robust_open($82,$83,$84)|0);
   HEAP32[$fd>>2] = $85;
   $86 = HEAP32[$fd>>2]|0;
   $87 = ($86|0)<(0);
   if ($87) {
    $88 = (___errno_location()|0);
    $89 = HEAP32[$88>>2]|0;
    $90 = ($89|0)!=(21);
    $91 = HEAP32[$isReadWrite>>2]|0;
    $92 = ($91|0)!=(0);
    $or$cond3 = $90 & $92;
    if ($or$cond3) {
     $93 = HEAP32[$4>>2]|0;
     $94 = $93 & -7;
     HEAP32[$4>>2] = $94;
     $95 = HEAP32[$openFlags>>2]|0;
     $96 = $95 & -67;
     HEAP32[$openFlags>>2] = $96;
     $97 = HEAP32[$4>>2]|0;
     $98 = $97 | 1;
     HEAP32[$4>>2] = $98;
     $99 = HEAP32[$openFlags>>2]|0;
     HEAP32[$openFlags>>2] = $99;
     HEAP32[$isReadonly>>2] = 1;
     $100 = HEAP32[$zName>>2]|0;
     $101 = HEAP32[$openFlags>>2]|0;
     $102 = HEAP32[$openMode>>2]|0;
     $103 = (_robust_open($100,$101,$102)|0);
     HEAP32[$fd>>2] = $103;
    }
   }
   $104 = HEAP32[$fd>>2]|0;
   $105 = ($104|0)<(0);
   if ($105) {
    $106 = (_sqlite3CantopenError(35186)|0);
    $107 = HEAP32[$zName>>2]|0;
    $108 = (_unixLogErrorAtLine($106,44128,$107,35186)|0);
    HEAP32[$rc>>2] = $108;
    break;
   }
   $109 = HEAP32[$4>>2]|0;
   $110 = $109 & 526336;
   $111 = ($110|0)!=(0);
   if ($111) {
    $112 = HEAP32[$fd>>2]|0;
    $113 = HEAP32[$uid>>2]|0;
    $114 = HEAP32[$gid>>2]|0;
    (_robustFchown($112,$113,$114)|0);
    label = 34;
   } else {
    label = 34;
   }
  } else {
   label = 34;
  }
 } while(0);
 if ((label|0) == 34) {
  $115 = HEAP32[$5>>2]|0;
  $116 = ($115|0)!=(0|0);
  if ($116) {
   $117 = HEAP32[$4>>2]|0;
   $118 = HEAP32[$5>>2]|0;
   HEAP32[$118>>2] = $117;
  }
  $119 = HEAP32[$p>>2]|0;
  $120 = ((($119)) + 28|0);
  $121 = HEAP32[$120>>2]|0;
  $122 = ($121|0)!=(0|0);
  if ($122) {
   $123 = HEAP32[$fd>>2]|0;
   $124 = HEAP32[$p>>2]|0;
   $125 = ((($124)) + 28|0);
   $126 = HEAP32[$125>>2]|0;
   HEAP32[$126>>2] = $123;
   $127 = HEAP32[$4>>2]|0;
   $128 = HEAP32[$p>>2]|0;
   $129 = ((($128)) + 28|0);
   $130 = HEAP32[$129>>2]|0;
   $131 = ((($130)) + 4|0);
   HEAP32[$131>>2] = $127;
  }
  $132 = HEAP32[$isDelete>>2]|0;
  $133 = ($132|0)!=(0);
  if ($133) {
   $134 = HEAP32[(6632)>>2]|0;
   $135 = HEAP32[$zName>>2]|0;
   (FUNCTION_TABLE_ii[$134 & 255]($135)|0);
  }
  $136 = HEAP32[$isDelete>>2]|0;
  $137 = ($136|0)!=(0);
  if ($137) {
   $138 = HEAP32[$ctrlFlags>>2]|0;
   $139 = $138 | 32;
   HEAP32[$ctrlFlags>>2] = $139;
  }
  $140 = HEAP32[$isReadonly>>2]|0;
  $141 = ($140|0)!=(0);
  if ($141) {
   $142 = HEAP32[$ctrlFlags>>2]|0;
   $143 = $142 | 2;
   HEAP32[$ctrlFlags>>2] = $143;
  }
  $144 = HEAP32[$eType>>2]|0;
  $145 = ($144|0)!=(256);
  $146 = $145&1;
  HEAP32[$noLock>>2] = $146;
  $147 = HEAP32[$noLock>>2]|0;
  $148 = ($147|0)!=(0);
  if ($148) {
   $149 = HEAP32[$ctrlFlags>>2]|0;
   $150 = $149 | 128;
   HEAP32[$ctrlFlags>>2] = $150;
  }
  $151 = HEAP32[$syncDir>>2]|0;
  $152 = ($151|0)!=(0);
  if ($152) {
   $153 = HEAP32[$ctrlFlags>>2]|0;
   $154 = $153 | 8;
   HEAP32[$ctrlFlags>>2] = $154;
  }
  $155 = HEAP32[$4>>2]|0;
  $156 = $155 & 64;
  $157 = ($156|0)!=(0);
  if ($157) {
   $158 = HEAP32[$ctrlFlags>>2]|0;
   $159 = $158 | 64;
   HEAP32[$ctrlFlags>>2] = $159;
  }
  $160 = HEAP32[$1>>2]|0;
  $161 = HEAP32[$fd>>2]|0;
  $162 = HEAP32[$3>>2]|0;
  $163 = HEAP32[$2>>2]|0;
  $164 = HEAP32[$ctrlFlags>>2]|0;
  $165 = (_fillInUnixFile($160,$161,$162,$163,$164)|0);
  HEAP32[$rc>>2] = $165;
 }
 $166 = HEAP32[$rc>>2]|0;
 $167 = ($166|0)!=(0);
 if ($167) {
  $168 = HEAP32[$p>>2]|0;
  $169 = ((($168)) + 28|0);
  $170 = HEAP32[$169>>2]|0;
  _sqlite3_free($170);
 }
 $171 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $171;
 $172 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($172|0);
}
function _unixDelete($NotUsed,$zPath,$dirSync) {
 $NotUsed = $NotUsed|0;
 $zPath = $zPath|0;
 $dirSync = $dirSync|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $rc = sp + 4|0;
 $fd = sp;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $dirSync;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[(6632)>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (FUNCTION_TABLE_ii[$4 & 255]($5)|0);
 $7 = ($6|0)==(-1);
 if ($7) {
  $8 = (___errno_location()|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)==(2);
  if ($10) {
   HEAP32[$rc>>2] = 5898;
  } else {
   $11 = HEAP32[$2>>2]|0;
   $12 = (_unixLogErrorAtLine(2570,44225,$11,35312)|0);
   HEAP32[$rc>>2] = $12;
  }
  $13 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $13;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 $14 = HEAP32[$3>>2]|0;
 $15 = $14 & 1;
 $16 = ($15|0)!=(0);
 do {
  if ($16) {
   $17 = HEAP32[(6644)>>2]|0;
   $18 = HEAP32[$2>>2]|0;
   $19 = (FUNCTION_TABLE_iii[$17 & 255]($18,$fd)|0);
   HEAP32[$rc>>2] = $19;
   $20 = HEAP32[$rc>>2]|0;
   $21 = ($20|0)==(0);
   if (!($21)) {
    HEAP32[$rc>>2] = 0;
    break;
   }
   $22 = HEAP32[$fd>>2]|0;
   $23 = (_full_fsync($22,0,0)|0);
   $24 = ($23|0)!=(0);
   if ($24) {
    $25 = HEAP32[$2>>2]|0;
    $26 = (_unixLogErrorAtLine(1290,44459,$25,35322)|0);
    HEAP32[$rc>>2] = $26;
   }
   $27 = HEAP32[$fd>>2]|0;
   _robust_close(0,$27,35324);
  }
 } while(0);
 $28 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $28;
 $29 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($29|0);
}
function _unixAccess($NotUsed,$zPath,$flags,$pResOut) {
 $NotUsed = $NotUsed|0;
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $3 = sp + 76|0;
 $buf = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $flags;
 HEAP32[$3>>2] = $pResOut;
 $4 = HEAP32[$2>>2]|0;
 $5 = ($4|0)==(0);
 if ($5) {
  $6 = HEAP32[(6488)>>2]|0;
  $7 = HEAP32[$1>>2]|0;
  $8 = (FUNCTION_TABLE_iii[$6 & 255]($7,$buf)|0);
  $9 = (0)==($8|0);
  $10 = ((($buf)) + 36|0);
  $11 = HEAP32[$10>>2]|0;
  $12 = ($11|0)>(0);
  $13 = $9 ? $12 : 0;
  $14 = $13&1;
  $15 = HEAP32[$3>>2]|0;
  HEAP32[$15>>2] = $14;
  STACKTOP = sp;return 0;
 } else {
  $16 = HEAP32[(6464)>>2]|0;
  $17 = HEAP32[$1>>2]|0;
  $18 = (FUNCTION_TABLE_iii[$16 & 255]($17,6)|0);
  $19 = ($18|0)==(0);
  $20 = $19&1;
  $21 = HEAP32[$3>>2]|0;
  HEAP32[$21>>2] = $20;
  STACKTOP = sp;return 0;
 }
 return (0)|0;
}
function _unixFullPathname($pVfs,$zPath,$nOut,$zOut) {
 $pVfs = $pVfs|0;
 $zPath = $zPath|0;
 $nOut = $nOut|0;
 $zOut = $zOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $bLink = 0, $buf = 0, $n = 0, $nByte = 0, $nLink = 0, $rc = 0, $zDel = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $2 = sp + 108|0;
 $3 = sp + 104|0;
 $rc = sp + 100|0;
 $nByte = sp + 96|0;
 $nLink = sp + 92|0;
 $zIn = sp + 88|0;
 $zDel = sp + 84|0;
 $bLink = sp + 80|0;
 $buf = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $nOut;
 HEAP32[$3>>2] = $zOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$nLink>>2] = 1;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$zIn>>2] = $4;
 HEAP32[$zDel>>2] = 0;
 while(1) {
  HEAP32[$bLink>>2] = 0;
  $5 = HEAP32[(6764)>>2]|0;
  $6 = HEAP32[$zIn>>2]|0;
  $7 = (FUNCTION_TABLE_iii[$5 & 255]($6,$buf)|0);
  $8 = ($7|0)!=(0);
  if ($8) {
   $9 = (___errno_location()|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(2);
   if ($11) {
    $12 = (_sqlite3CantopenError(35436)|0);
    $13 = HEAP32[$zIn>>2]|0;
    $14 = (_unixLogErrorAtLine($12,44313,$13,35436)|0);
    HEAP32[$rc>>2] = $14;
   }
  } else {
   $15 = ((($buf)) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = $16 & 61440;
   $18 = ($17|0)==(40960);
   $19 = $18&1;
   HEAP32[$bLink>>2] = $19;
  }
  $20 = HEAP32[$bLink>>2]|0;
  $21 = ($20|0)!=(0);
  if ($21) {
   $22 = HEAP32[$zDel>>2]|0;
   $23 = ($22|0)==(0|0);
   if ($23) {
    $24 = HEAP32[$2>>2]|0;
    $25 = (_sqlite3_malloc($24)|0);
    HEAP32[$zDel>>2] = $25;
    $26 = HEAP32[$zDel>>2]|0;
    $27 = ($26|0)==(0|0);
    if ($27) {
     HEAP32[$rc>>2] = 7;
    }
   } else {
    $28 = HEAP32[$nLink>>2]|0;
    $29 = (($28) + 1)|0;
    HEAP32[$nLink>>2] = $29;
    $30 = ($29|0)>(100);
    if ($30) {
     $31 = (_sqlite3CantopenError(35447)|0);
     HEAP32[$rc>>2] = $31;
    }
   }
   $32 = HEAP32[$rc>>2]|0;
   $33 = ($32|0)==(0);
   do {
    if ($33) {
     $34 = HEAP32[(6752)>>2]|0;
     $35 = HEAP32[$zIn>>2]|0;
     $36 = HEAP32[$zDel>>2]|0;
     $37 = HEAP32[$2>>2]|0;
     $38 = (($37) - 1)|0;
     $39 = (FUNCTION_TABLE_iiii[$34 & 255]($35,$36,$38)|0);
     HEAP32[$nByte>>2] = $39;
     $40 = HEAP32[$nByte>>2]|0;
     $41 = ($40|0)<(0);
     if ($41) {
      $42 = (_sqlite3CantopenError(35453)|0);
      $43 = HEAP32[$zIn>>2]|0;
      $44 = (_unixLogErrorAtLine($42,44304,$43,35453)|0);
      HEAP32[$rc>>2] = $44;
      break;
     }
     $45 = HEAP32[$zDel>>2]|0;
     $46 = HEAP8[$45>>0]|0;
     $47 = $46 << 24 >> 24;
     $48 = ($47|0)!=(47);
     do {
      if ($48) {
       $49 = HEAP32[$zIn>>2]|0;
       $50 = (_sqlite3Strlen30($49)|0);
       HEAP32[$n>>2] = $50;
       while(1) {
        $51 = HEAP32[$n>>2]|0;
        $52 = ($51|0)>(0);
        if (!($52)) {
         break;
        }
        $53 = HEAP32[$n>>2]|0;
        $54 = (($53) - 1)|0;
        $55 = HEAP32[$zIn>>2]|0;
        $56 = (($55) + ($54)|0);
        $57 = HEAP8[$56>>0]|0;
        $58 = $57 << 24 >> 24;
        $59 = ($58|0)!=(47);
        if (!($59)) {
         break;
        }
        $60 = HEAP32[$n>>2]|0;
        $61 = (($60) + -1)|0;
        HEAP32[$n>>2] = $61;
       }
       $62 = HEAP32[$nByte>>2]|0;
       $63 = HEAP32[$n>>2]|0;
       $64 = (($62) + ($63))|0;
       $65 = (($64) + 1)|0;
       $66 = HEAP32[$2>>2]|0;
       $67 = ($65|0)>($66|0);
       if ($67) {
        $68 = (_sqlite3CantopenError(35459)|0);
        HEAP32[$rc>>2] = $68;
        break;
       } else {
        $69 = HEAP32[$n>>2]|0;
        $70 = HEAP32[$zDel>>2]|0;
        $71 = (($70) + ($69)|0);
        $72 = HEAP32[$zDel>>2]|0;
        $73 = HEAP32[$nByte>>2]|0;
        $74 = (($73) + 1)|0;
        _memmove(($71|0),($72|0),($74|0))|0;
        $75 = HEAP32[$zDel>>2]|0;
        $76 = HEAP32[$zIn>>2]|0;
        $77 = HEAP32[$n>>2]|0;
        _memcpy(($75|0),($76|0),($77|0))|0;
        $78 = HEAP32[$n>>2]|0;
        $79 = HEAP32[$nByte>>2]|0;
        $80 = (($79) + ($78))|0;
        HEAP32[$nByte>>2] = $80;
        break;
       }
      }
     } while(0);
     $81 = HEAP32[$nByte>>2]|0;
     $82 = HEAP32[$zDel>>2]|0;
     $83 = (($82) + ($81)|0);
     HEAP8[$83>>0] = 0;
    }
   } while(0);
   $84 = HEAP32[$zDel>>2]|0;
   HEAP32[$zIn>>2] = $84;
  }
  $85 = HEAP32[$rc>>2]|0;
  $86 = ($85|0)==(0);
  if ($86) {
   $87 = HEAP32[$zIn>>2]|0;
   $88 = HEAP32[$3>>2]|0;
   $89 = ($87|0)!=($88|0);
   if ($89) {
    $90 = HEAP32[$zIn>>2]|0;
    $91 = HEAP32[$3>>2]|0;
    $92 = HEAP32[$2>>2]|0;
    $93 = (_mkFullPathname($90,$91,$92)|0);
    HEAP32[$rc>>2] = $93;
   }
  }
  $94 = HEAP32[$bLink>>2]|0;
  $95 = ($94|0)==(0);
  if ($95) {
   label = 30;
   break;
  }
  $96 = HEAP32[$3>>2]|0;
  HEAP32[$zIn>>2] = $96;
  $97 = HEAP32[$rc>>2]|0;
  $98 = ($97|0)==(0);
  if (!($98)) {
   label = 30;
   break;
  }
 }
 if ((label|0) == 30) {
  $99 = HEAP32[$zDel>>2]|0;
  _sqlite3_free($99);
  $100 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($100|0);
 }
 return (0)|0;
}
function _unixRandomness($NotUsed,$nBuf,$zBuf) {
 $NotUsed = $NotUsed|0;
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $got = 0, $t = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $fd = sp + 8|0;
 $got = sp + 4|0;
 $t = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $nBuf;
 HEAP32[$2>>2] = $zBuf;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 _memset(($3|0),0,($4|0))|0;
 $5 = (_getpid()|0);
 HEAP32[6872>>2] = $5;
 $6 = (_robust_open(44446,0,0)|0);
 HEAP32[$fd>>2] = $6;
 $7 = HEAP32[$fd>>2]|0;
 $8 = ($7|0)<(0);
 if ($8) {
  (_time(($t|0))|0);
  $9 = HEAP32[$2>>2]|0;
  ;HEAP8[$9>>0]=HEAP8[$t>>0]|0;HEAP8[$9+1>>0]=HEAP8[$t+1>>0]|0;HEAP8[$9+2>>0]=HEAP8[$t+2>>0]|0;HEAP8[$9+3>>0]=HEAP8[$t+3>>0]|0;
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 4|0);
  ;HEAP8[$11>>0]=HEAP8[6872>>0]|0;HEAP8[$11+1>>0]=HEAP8[6872+1>>0]|0;HEAP8[$11+2>>0]=HEAP8[6872+2>>0]|0;HEAP8[$11+3>>0]=HEAP8[6872+3>>0]|0;
  HEAP32[$1>>2] = 8;
  $23 = HEAP32[$1>>2]|0;
  STACKTOP = sp;return ($23|0);
 }
 while(1) {
  $12 = HEAP32[(6536)>>2]|0;
  $13 = HEAP32[$fd>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  $15 = HEAP32[$1>>2]|0;
  $16 = (FUNCTION_TABLE_iiii[$12 & 255]($13,$14,$15)|0);
  HEAP32[$got>>2] = $16;
  $17 = HEAP32[$got>>2]|0;
  $18 = ($17|0)<(0);
  if (!($18)) {
   break;
  }
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(4);
  if (!($21)) {
   break;
  }
 }
 $22 = HEAP32[$fd>>2]|0;
 _robust_close(0,$22,35583);
 $23 = HEAP32[$1>>2]|0;
 STACKTOP = sp;return ($23|0);
}
function _unixSleep($NotUsed,$microseconds) {
 $NotUsed = $NotUsed|0;
 $microseconds = $microseconds|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $seconds = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $seconds = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $microseconds;
 $2 = HEAP32[$1>>2]|0;
 $3 = (($2) + 999999)|0;
 $4 = (($3|0) / 1000000)&-1;
 HEAP32[$seconds>>2] = $4;
 $5 = HEAP32[$seconds>>2]|0;
 (_sleep($5)|0);
 $6 = HEAP32[$seconds>>2]|0;
 $7 = ($6*1000000)|0;
 STACKTOP = sp;return ($7|0);
}
function _unixCurrentTime($NotUsed,$prNow) {
 $NotUsed = $NotUsed|0;
 $prNow = $prNow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rc = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $i = sp;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $prNow;
 $2 = $i;
 $3 = $2;
 HEAP32[$3>>2] = 0;
 $4 = (($2) + 4)|0;
 $5 = $4;
 HEAP32[$5>>2] = 0;
 $6 = (_unixCurrentTimeInt64(0,$i)|0);
 HEAP32[$rc>>2] = $6;
 $7 = $i;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (+($9>>>0)) + (4294967296.0*(+($12|0)));
 $14 = $13 / 8.64E+7;
 $15 = HEAP32[$1>>2]|0;
 HEAPF64[$15>>3] = $14;
 $16 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _unixGetLastError($NotUsed,$NotUsed2,$NotUsed3) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 $NotUsed3 = $NotUsed3|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 HEAP32[$2>>2] = $NotUsed3;
 $3 = (___errno_location()|0);
 $4 = HEAP32[$3>>2]|0;
 STACKTOP = sp;return ($4|0);
}
function _unixCurrentTimeInt64($NotUsed,$piNow) {
 $NotUsed = $NotUsed|0;
 $piNow = $piNow|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $rc = 0, $sNow = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $sNow = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $piNow;
 HEAP32[$rc>>2] = 0;
 (_gettimeofday(($sNow|0),(0|0))|0);
 $2 = HEAP32[$sNow>>2]|0;
 $3 = ($2|0)<(0);
 $4 = $3 << 31 >> 31;
 $5 = (___muldi3(1000,0,($2|0),($4|0))|0);
 $6 = tempRet0;
 $7 = (_i64Add(1045635584,49096,($5|0),($6|0))|0);
 $8 = tempRet0;
 $9 = ((($sNow)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (($10|0) / 1000)&-1;
 $12 = ($11|0)<(0);
 $13 = $12 << 31 >> 31;
 $14 = (_i64Add(($7|0),($8|0),($11|0),($13|0))|0);
 $15 = tempRet0;
 $16 = HEAP32[$1>>2]|0;
 $17 = $16;
 $18 = $17;
 HEAP32[$18>>2] = $14;
 $19 = (($17) + 4)|0;
 $20 = $19;
 HEAP32[$20>>2] = $15;
 $21 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($21|0);
}
function _unixSetSystemCall($pNotUsed,$zName,$pNewFunc) {
 $pNotUsed = $pNotUsed|0;
 $zName = $zName|0;
 $pNewFunc = $pNewFunc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $i = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $pNotUsed;
 HEAP32[$1>>2] = $zName;
 HEAP32[$2>>2] = $pNewFunc;
 HEAP32[$rc>>2] = 12;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$rc>>2] = 0;
  HEAP32[$i>>2] = 0;
  while(1) {
   $5 = HEAP32[$i>>2]|0;
   $6 = ($5>>>0)<(28);
   if (!($6)) {
    break;
   }
   $7 = HEAP32[$i>>2]|0;
   $8 = (6436 + (($7*12)|0)|0);
   $9 = ((($8)) + 8|0);
   $10 = HEAP32[$9>>2]|0;
   $11 = ($10|0)!=(0|0);
   if ($11) {
    $12 = HEAP32[$i>>2]|0;
    $13 = (6436 + (($12*12)|0)|0);
    $14 = ((($13)) + 8|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = HEAP32[$i>>2]|0;
    $17 = (6436 + (($16*12)|0)|0);
    $18 = ((($17)) + 4|0);
    HEAP32[$18>>2] = $15;
   }
   $19 = HEAP32[$i>>2]|0;
   $20 = (($19) + 1)|0;
   HEAP32[$i>>2] = $20;
  }
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $21 = HEAP32[$i>>2]|0;
  $22 = ($21>>>0)<(28);
  if (!($22)) {
   label = 16;
   break;
  }
  $23 = HEAP32[$1>>2]|0;
  $24 = HEAP32[$i>>2]|0;
  $25 = (6436 + (($24*12)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  $27 = (_strcmp($23,$26)|0);
  $28 = ($27|0)==(0);
  $29 = HEAP32[$i>>2]|0;
  if ($28) {
   break;
  }
  $51 = (($29) + 1)|0;
  HEAP32[$i>>2] = $51;
 }
 if ((label|0) == 16) {
  $52 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($52|0);
 }
 $30 = (6436 + (($29*12)|0)|0);
 $31 = ((($30)) + 8|0);
 $32 = HEAP32[$31>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  $34 = HEAP32[$i>>2]|0;
  $35 = (6436 + (($34*12)|0)|0);
  $36 = ((($35)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = HEAP32[$i>>2]|0;
  $39 = (6436 + (($38*12)|0)|0);
  $40 = ((($39)) + 8|0);
  HEAP32[$40>>2] = $37;
 }
 HEAP32[$rc>>2] = 0;
 $41 = HEAP32[$2>>2]|0;
 $42 = ($41|0)==(0|0);
 if ($42) {
  $43 = HEAP32[$i>>2]|0;
  $44 = (6436 + (($43*12)|0)|0);
  $45 = ((($44)) + 8|0);
  $46 = HEAP32[$45>>2]|0;
  HEAP32[$2>>2] = $46;
 }
 $47 = HEAP32[$2>>2]|0;
 $48 = HEAP32[$i>>2]|0;
 $49 = (6436 + (($48*12)|0)|0);
 $50 = ((($49)) + 4|0);
 HEAP32[$50>>2] = $47;
 $52 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($52|0);
}
function _unixGetSystemCall($pNotUsed,$zName) {
 $pNotUsed = $pNotUsed|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $pNotUsed;
 HEAP32[$2>>2] = $zName;
 HEAP32[$i>>2] = 0;
 while(1) {
  $3 = HEAP32[$i>>2]|0;
  $4 = ($3>>>0)<(28);
  if (!($4)) {
   label = 6;
   break;
  }
  $5 = HEAP32[$2>>2]|0;
  $6 = HEAP32[$i>>2]|0;
  $7 = (6436 + (($6*12)|0)|0);
  $8 = HEAP32[$7>>2]|0;
  $9 = (_strcmp($5,$8)|0);
  $10 = ($9|0)==(0);
  $11 = HEAP32[$i>>2]|0;
  if ($10) {
   label = 4;
   break;
  }
  $15 = (($11) + 1)|0;
  HEAP32[$i>>2] = $15;
 }
 if ((label|0) == 4) {
  $12 = (6436 + (($11*12)|0)|0);
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$0>>2] = $14;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 else if ((label|0) == 6) {
  HEAP32[$0>>2] = 0;
  $16 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($16|0);
 }
 return (0)|0;
}
function _unixNextSystemCall($p,$zName) {
 $p = $p|0;
 $zName = $zName|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $i = sp;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $zName;
 HEAP32[$i>>2] = -1;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 L1: do {
  if ($4) {
   HEAP32[$i>>2] = 0;
   while(1) {
    $5 = HEAP32[$i>>2]|0;
    $6 = ($5|0)<(27);
    if (!($6)) {
     break L1;
    }
    $7 = HEAP32[$2>>2]|0;
    $8 = HEAP32[$i>>2]|0;
    $9 = (6436 + (($8*12)|0)|0);
    $10 = HEAP32[$9>>2]|0;
    $11 = (_strcmp($7,$10)|0);
    $12 = ($11|0)==(0);
    if ($12) {
     break L1;
    }
    $13 = HEAP32[$i>>2]|0;
    $14 = (($13) + 1)|0;
    HEAP32[$i>>2] = $14;
   }
  }
 } while(0);
 $15 = HEAP32[$i>>2]|0;
 $16 = (($15) + 1)|0;
 HEAP32[$i>>2] = $16;
 while(1) {
  $17 = HEAP32[$i>>2]|0;
  $18 = ($17|0)<(28);
  if (!($18)) {
   label = 11;
   break;
  }
  $19 = HEAP32[$i>>2]|0;
  $20 = (6436 + (($19*12)|0)|0);
  $21 = ((($20)) + 4|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)!=(0|0);
  $24 = HEAP32[$i>>2]|0;
  if ($23) {
   label = 9;
   break;
  }
  $27 = (($24) + 1)|0;
  HEAP32[$i>>2] = $27;
 }
 if ((label|0) == 9) {
  $25 = (6436 + (($24*12)|0)|0);
  $26 = HEAP32[$25>>2]|0;
  HEAP32[$0>>2] = $26;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 else if ((label|0) == 11) {
  HEAP32[$0>>2] = 0;
  $28 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 return (0)|0;
}
function _dotlockIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (6360|0);
}
function _dotlockClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $pFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$0>>2]|0;
 (_dotlockUnlock($2,0)|0);
 $3 = HEAP32[$pFile>>2]|0;
 $4 = ((($3)) + 24|0);
 $5 = HEAP32[$4>>2]|0;
 _sqlite3_free($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = (_closeUnixFile($6)|0);
 STACKTOP = sp;return ($7|0);
}
function _unixRead($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $got = 0, $pFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $pFile = sp + 12|0;
 $got = sp + 8|0;
 HEAP32[$3>>2] = $id;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $amt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $11;
 $12 = HEAP32[$pFile>>2]|0;
 $13 = $6;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = (_seekAndRead($12,$15,$18,$19,$20)|0);
 HEAP32[$got>>2] = $21;
 $22 = HEAP32[$got>>2]|0;
 $23 = HEAP32[$5>>2]|0;
 $24 = ($22|0)==($23|0);
 if ($24) {
  HEAP32[$2>>2] = 0;
  $34 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 $25 = HEAP32[$got>>2]|0;
 $26 = ($25|0)<(0);
 if ($26) {
  HEAP32[$2>>2] = 266;
  $34 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($34|0);
 } else {
  $27 = HEAP32[$pFile>>2]|0;
  _storeLastErrno($27,0);
  $28 = HEAP32[$got>>2]|0;
  $29 = HEAP32[$4>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP32[$5>>2]|0;
  $32 = HEAP32[$got>>2]|0;
  $33 = (($31) - ($32))|0;
  _memset(($30|0),0,($33|0))|0;
  HEAP32[$2>>2] = 522;
  $34 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($34|0);
 }
 return (0)|0;
}
function _unixWrite($id,$pBuf,$amt,$0,$1) {
 $id = $id|0;
 $pBuf = $pBuf|0;
 $amt = $amt|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $wrote = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 28|0;
 $3 = sp + 24|0;
 $4 = sp + 20|0;
 $5 = sp + 16|0;
 $6 = sp;
 $pFile = sp + 12|0;
 $wrote = sp + 8|0;
 HEAP32[$3>>2] = $id;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $amt;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 $11 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $11;
 HEAP32[$wrote>>2] = 0;
 while(1) {
  $12 = HEAP32[$pFile>>2]|0;
  $13 = $6;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (($13) + 4)|0;
  $17 = $16;
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$4>>2]|0;
  $20 = HEAP32[$5>>2]|0;
  $21 = (_seekAndWrite($12,$15,$18,$19,$20)|0);
  HEAP32[$wrote>>2] = $21;
  $22 = HEAP32[$5>>2]|0;
  $23 = ($21|0)<($22|0);
  $24 = HEAP32[$wrote>>2]|0;
  $25 = ($24|0)>(0);
  $26 = $23 ? $25 : 0;
  if (!($26)) {
   break;
  }
  $27 = HEAP32[$wrote>>2]|0;
  $28 = HEAP32[$5>>2]|0;
  $29 = (($28) - ($27))|0;
  HEAP32[$5>>2] = $29;
  $30 = HEAP32[$wrote>>2]|0;
  $31 = ($30|0)<(0);
  $32 = $31 << 31 >> 31;
  $33 = $6;
  $34 = $33;
  $35 = HEAP32[$34>>2]|0;
  $36 = (($33) + 4)|0;
  $37 = $36;
  $38 = HEAP32[$37>>2]|0;
  $39 = (_i64Add(($35|0),($38|0),($30|0),($32|0))|0);
  $40 = tempRet0;
  $41 = $6;
  $42 = $41;
  HEAP32[$42>>2] = $39;
  $43 = (($41) + 4)|0;
  $44 = $43;
  HEAP32[$44>>2] = $40;
  $45 = HEAP32[$wrote>>2]|0;
  $46 = HEAP32[$4>>2]|0;
  $47 = (($46) + ($45)|0);
  HEAP32[$4>>2] = $47;
 }
 $48 = HEAP32[$5>>2]|0;
 $49 = HEAP32[$wrote>>2]|0;
 $50 = ($48|0)>($49|0);
 if (!($50)) {
  HEAP32[$2>>2] = 0;
  $58 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($58|0);
 }
 $51 = HEAP32[$wrote>>2]|0;
 $52 = ($51|0)<(0);
 if ($52) {
  $53 = HEAP32[$pFile>>2]|0;
  $54 = ((($53)) + 20|0);
  $55 = HEAP32[$54>>2]|0;
  $56 = ($55|0)!=(28);
  if ($56) {
   HEAP32[$2>>2] = 778;
   $58 = HEAP32[$2>>2]|0;
   STACKTOP = sp;return ($58|0);
  }
 }
 $57 = HEAP32[$pFile>>2]|0;
 _storeLastErrno($57,0);
 HEAP32[$2>>2] = 13;
 $58 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($58|0);
}
function _unixTruncate($id,$0,$1) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp;
 $pFile = sp + 12|0;
 $rc = sp + 8|0;
 HEAP32[$3>>2] = $id;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 HEAP32[$pFile>>2] = $9;
 $10 = HEAP32[$pFile>>2]|0;
 $11 = ((($10)) + 40|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 if ($13) {
  $14 = $4;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$pFile>>2]|0;
  $21 = ((($20)) + 40|0);
  $22 = HEAP32[$21>>2]|0;
  $23 = ($22|0)<(0);
  $24 = $23 << 31 >> 31;
  $25 = (_i64Add(($16|0),($19|0),($22|0),($24|0))|0);
  $26 = tempRet0;
  $27 = (_i64Subtract(($25|0),($26|0),1,0)|0);
  $28 = tempRet0;
  $29 = HEAP32[$pFile>>2]|0;
  $30 = ((($29)) + 40|0);
  $31 = HEAP32[$30>>2]|0;
  $32 = ($31|0)<(0);
  $33 = $32 << 31 >> 31;
  $34 = (___divdi3(($27|0),($28|0),($31|0),($33|0))|0);
  $35 = tempRet0;
  $36 = HEAP32[$pFile>>2]|0;
  $37 = ((($36)) + 40|0);
  $38 = HEAP32[$37>>2]|0;
  $39 = ($38|0)<(0);
  $40 = $39 << 31 >> 31;
  $41 = (___muldi3(($34|0),($35|0),($38|0),($40|0))|0);
  $42 = tempRet0;
  $43 = $4;
  $44 = $43;
  HEAP32[$44>>2] = $41;
  $45 = (($43) + 4)|0;
  $46 = $45;
  HEAP32[$46>>2] = $42;
 }
 $47 = HEAP32[$pFile>>2]|0;
 $48 = ((($47)) + 12|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = $4;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + 4)|0;
 $54 = $53;
 $55 = HEAP32[$54>>2]|0;
 $56 = (_robust_ftruncate($49,$52,$55)|0);
 HEAP32[$rc>>2] = $56;
 $57 = HEAP32[$rc>>2]|0;
 $58 = ($57|0)!=(0);
 if ($58) {
  $59 = HEAP32[$pFile>>2]|0;
  $60 = (___errno_location()|0);
  $61 = HEAP32[$60>>2]|0;
  _storeLastErrno($59,$61);
  $62 = HEAP32[$pFile>>2]|0;
  $63 = ((($62)) + 32|0);
  $64 = HEAP32[$63>>2]|0;
  $65 = (_unixLogErrorAtLine(1546,44151,$64,32998)|0);
  HEAP32[$2>>2] = $65;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 } else {
  HEAP32[$2>>2] = 0;
  $66 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($66|0);
 }
 return (0)|0;
}
function _unixSync($id,$flags) {
 $id = $id|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $dirfd = 0, $isDataOnly = 0, $isFullsync = 0, $pFile = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $rc = sp + 16|0;
 $pFile = sp + 12|0;
 $isDataOnly = sp + 8|0;
 $isFullsync = sp + 4|0;
 $dirfd = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $flags;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$2>>2]|0;
 $5 = $4 & 16;
 HEAP32[$isDataOnly>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = $6 & 15;
 $8 = ($7|0)==(3);
 $9 = $8&1;
 HEAP32[$isFullsync>>2] = $9;
 $10 = HEAP32[$pFile>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$isFullsync>>2]|0;
 $14 = HEAP32[$isDataOnly>>2]|0;
 $15 = (_full_fsync($12,$13,$14)|0);
 HEAP32[$rc>>2] = $15;
 $16 = HEAP32[$rc>>2]|0;
 $17 = ($16|0)!=(0);
 $18 = HEAP32[$pFile>>2]|0;
 if ($17) {
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  _storeLastErrno($18,$20);
  $21 = HEAP32[$pFile>>2]|0;
  $22 = ((($21)) + 32|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (_unixLogErrorAtLine(1034,44435,$23,32953)|0);
  HEAP32[$0>>2] = $24;
  $47 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($47|0);
 }
 $25 = ((($18)) + 18|0);
 $26 = HEAP16[$25>>1]|0;
 $27 = $26&65535;
 $28 = $27 & 8;
 $29 = ($28|0)!=(0);
 if ($29) {
  $30 = HEAP32[(6644)>>2]|0;
  $31 = HEAP32[$pFile>>2]|0;
  $32 = ((($31)) + 32|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (FUNCTION_TABLE_iii[$30 & 255]($33,$dirfd)|0);
  HEAP32[$rc>>2] = $34;
  $35 = HEAP32[$rc>>2]|0;
  $36 = ($35|0)==(0);
  if ($36) {
   $37 = HEAP32[$dirfd>>2]|0;
   (_full_fsync($37,0,0)|0);
   $38 = HEAP32[$pFile>>2]|0;
   $39 = HEAP32[$dirfd>>2]|0;
   _robust_close($38,$39,32967);
  } else {
   HEAP32[$rc>>2] = 0;
  }
  $40 = HEAP32[$pFile>>2]|0;
  $41 = ((($40)) + 18|0);
  $42 = HEAP16[$41>>1]|0;
  $43 = $42&65535;
  $44 = $43 & -9;
  $45 = $44&65535;
  HEAP16[$41>>1] = $45;
 }
 $46 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $46;
 $47 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($47|0);
}
function _unixFileSize($id,$pSize) {
 $id = $id|0;
 $pSize = $pSize|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $rc = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 88|0;
 $1 = sp + 84|0;
 $2 = sp + 80|0;
 $rc = sp + 76|0;
 $buf = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $pSize;
 $3 = HEAP32[(6500)>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 12|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$3 & 255]($6,$buf)|0);
 HEAP32[$rc>>2] = $7;
 $8 = HEAP32[$rc>>2]|0;
 $9 = ($8|0)!=(0);
 if ($9) {
  $10 = HEAP32[$1>>2]|0;
  $11 = (___errno_location()|0);
  $12 = HEAP32[$11>>2]|0;
  _storeLastErrno($10,$12);
  HEAP32[$0>>2] = 1802;
  $37 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $13 = ((($buf)) + 36|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = ($14|0)<(0);
 $16 = $15 << 31 >> 31;
 $17 = HEAP32[$2>>2]|0;
 $18 = $17;
 $19 = $18;
 HEAP32[$19>>2] = $14;
 $20 = (($18) + 4)|0;
 $21 = $20;
 HEAP32[$21>>2] = $16;
 $22 = HEAP32[$2>>2]|0;
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = ($25|0)==(1);
 $30 = ($28|0)==(0);
 $31 = $29 & $30;
 if ($31) {
  $32 = HEAP32[$2>>2]|0;
  $33 = $32;
  $34 = $33;
  HEAP32[$34>>2] = 0;
  $35 = (($33) + 4)|0;
  $36 = $35;
  HEAP32[$36>>2] = 0;
 }
 HEAP32[$0>>2] = 0;
 $37 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _dotlockLock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0;
 var $rc = 0, $tErrno = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pFile = sp + 12|0;
 $zLockFile = sp + 8|0;
 $rc = sp + 4|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$zLockFile>>2] = $6;
 HEAP32[$rc>>2] = 0;
 $7 = HEAP32[$pFile>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)>(0);
 if ($11) {
  $12 = HEAP32[$2>>2]|0;
  $13 = $12&255;
  $14 = HEAP32[$pFile>>2]|0;
  $15 = ((($14)) + 16|0);
  HEAP8[$15>>0] = $13;
  $16 = HEAP32[$zLockFile>>2]|0;
  (_utimes(($16|0),(0|0))|0);
  HEAP32[$0>>2] = 0;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $17 = HEAP32[(6656)>>2]|0;
 $18 = HEAP32[$zLockFile>>2]|0;
 $19 = (FUNCTION_TABLE_iii[$17 & 255]($18,511)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)<(0);
 if (!($21)) {
  $33 = HEAP32[$2>>2]|0;
  $34 = $33&255;
  $35 = HEAP32[$pFile>>2]|0;
  $36 = ((($35)) + 16|0);
  HEAP8[$36>>0] = $34;
  $37 = HEAP32[$rc>>2]|0;
  HEAP32[$0>>2] = $37;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 $22 = (___errno_location()|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$tErrno>>2] = $23;
 $24 = HEAP32[$tErrno>>2]|0;
 $25 = (17)==($24|0);
 if ($25) {
  HEAP32[$rc>>2] = 5;
 } else {
  $26 = HEAP32[$tErrno>>2]|0;
  $27 = (_sqliteErrorFromPosixError($26,3850)|0);
  HEAP32[$rc>>2] = $27;
  $28 = HEAP32[$rc>>2]|0;
  $29 = ($28|0)!=(5);
  if ($29) {
   $30 = HEAP32[$pFile>>2]|0;
   $31 = HEAP32[$tErrno>>2]|0;
   _storeLastErrno($30,$31);
  }
 }
 $32 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $32;
 $38 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($38|0);
}
function _dotlockUnlock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, $tErrno = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $pFile = sp + 12|0;
 $zLockFile = sp + 8|0;
 $rc = sp + 4|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$zLockFile>>2] = $6;
 $7 = HEAP32[$pFile>>2]|0;
 $8 = ((($7)) + 16|0);
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = HEAP32[$2>>2]|0;
 $12 = ($10|0)==($11|0);
 if ($12) {
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ($13|0)==(1);
 if ($14) {
  $15 = HEAP32[$pFile>>2]|0;
  $16 = ((($15)) + 16|0);
  HEAP8[$16>>0] = 1;
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $17 = HEAP32[(6668)>>2]|0;
 $18 = HEAP32[$zLockFile>>2]|0;
 $19 = (FUNCTION_TABLE_ii[$17 & 255]($18)|0);
 HEAP32[$rc>>2] = $19;
 $20 = HEAP32[$rc>>2]|0;
 $21 = ($20|0)<(0);
 if (!($21)) {
  $29 = HEAP32[$pFile>>2]|0;
  $30 = ((($29)) + 16|0);
  HEAP8[$30>>0] = 0;
  HEAP32[$0>>2] = 0;
  $31 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($31|0);
 }
 $22 = (___errno_location()|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$tErrno>>2] = $23;
 $24 = HEAP32[$tErrno>>2]|0;
 $25 = ($24|0)==(2);
 if ($25) {
  HEAP32[$rc>>2] = 0;
 } else {
  HEAP32[$rc>>2] = 2058;
  $26 = HEAP32[$pFile>>2]|0;
  $27 = HEAP32[$tErrno>>2]|0;
  _storeLastErrno($26,$27);
 }
 $28 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $28;
 $31 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($31|0);
}
function _dotlockCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, $rc = 0, $reserved = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $rc = sp + 8|0;
 $reserved = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$reserved>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $2;
 $3 = HEAP32[(6464)>>2]|0;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 24|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = (FUNCTION_TABLE_iii[$3 & 255]($6,0)|0);
 $8 = ($7|0)==(0);
 $9 = $8&1;
 HEAP32[$reserved>>2] = $9;
 $10 = HEAP32[$reserved>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 HEAP32[$11>>2] = $10;
 $12 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($12|0);
}
function _unixFileControl($id,$op,$pArg) {
 $id = $id|0;
 $op = $op|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $pFile = 0, $rc = 0, $vararg_buffer = 0, $zTFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $pFile = sp + 12|0;
 $rc = sp + 8|0;
 $zTFile = sp + 4|0;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $op;
 HEAP32[$3>>2] = $pArg;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 do {
  switch ($5|0) {
  case 1:  {
   $6 = HEAP32[$pFile>>2]|0;
   $7 = ((($6)) + 16|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8&255;
   $10 = HEAP32[$3>>2]|0;
   HEAP32[$10>>2] = $9;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 4:  {
   $11 = HEAP32[$pFile>>2]|0;
   $12 = ((($11)) + 20|0);
   $13 = HEAP32[$12>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   HEAP32[$14>>2] = $13;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 6:  {
   $15 = HEAP32[$3>>2]|0;
   $16 = HEAP32[$15>>2]|0;
   $17 = HEAP32[$pFile>>2]|0;
   $18 = ((($17)) + 40|0);
   HEAP32[$18>>2] = $16;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 5:  {
   $19 = HEAP32[$pFile>>2]|0;
   $20 = HEAP32[$3>>2]|0;
   $21 = $20;
   $22 = $21;
   $23 = HEAP32[$22>>2]|0;
   $24 = (($21) + 4)|0;
   $25 = $24;
   $26 = HEAP32[$25>>2]|0;
   $27 = (_fcntlSizeHint($19,$23,$26)|0);
   HEAP32[$rc>>2] = $27;
   $28 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $28;
   break;
  }
  case 10:  {
   $29 = HEAP32[$pFile>>2]|0;
   $30 = HEAP32[$3>>2]|0;
   _unixModeBit($29,4,$30);
   HEAP32[$0>>2] = 0;
   break;
  }
  case 13:  {
   $31 = HEAP32[$pFile>>2]|0;
   $32 = HEAP32[$3>>2]|0;
   _unixModeBit($31,16,$32);
   HEAP32[$0>>2] = 0;
   break;
  }
  case 12:  {
   $33 = HEAP32[$pFile>>2]|0;
   $34 = ((($33)) + 4|0);
   $35 = HEAP32[$34>>2]|0;
   $36 = ((($35)) + 16|0);
   $37 = HEAP32[$36>>2]|0;
   HEAP32[$vararg_buffer>>2] = $37;
   $38 = (_sqlite3_mprintf(18391,$vararg_buffer)|0);
   $39 = HEAP32[$3>>2]|0;
   HEAP32[$39>>2] = $38;
   HEAP32[$0>>2] = 0;
   break;
  }
  case 16:  {
   $40 = HEAP32[$pFile>>2]|0;
   $41 = ((($40)) + 4|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ((($42)) + 8|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = ($44|0)<(0);
   $46 = $45 << 31 >> 31;
   $47 = (_sqlite3_malloc64($44,$46)|0);
   HEAP32[$zTFile>>2] = $47;
   $48 = HEAP32[$zTFile>>2]|0;
   $49 = ($48|0)!=(0|0);
   if ($49) {
    $50 = HEAP32[$pFile>>2]|0;
    $51 = ((($50)) + 4|0);
    $52 = HEAP32[$51>>2]|0;
    $53 = ((($52)) + 8|0);
    $54 = HEAP32[$53>>2]|0;
    $55 = HEAP32[$zTFile>>2]|0;
    (_unixGetTempname($54,$55)|0);
    $56 = HEAP32[$zTFile>>2]|0;
    $57 = HEAP32[$3>>2]|0;
    HEAP32[$57>>2] = $56;
   }
   HEAP32[$0>>2] = 0;
   break;
  }
  case 20:  {
   $58 = HEAP32[$pFile>>2]|0;
   $59 = (_fileHasMoved($58)|0);
   $60 = HEAP32[$3>>2]|0;
   HEAP32[$60>>2] = $59;
   HEAP32[$0>>2] = 0;
   break;
  }
  default: {
   HEAP32[$0>>2] = 12;
  }
  }
 } while(0);
 $61 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($61|0);
}
function _unixSectorSize($NotUsed) {
 $NotUsed = $NotUsed|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $NotUsed;
 STACKTOP = sp;return 4096;
}
function _unixDeviceCharacteristics($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $p = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$p>>2] = $1;
 HEAP32[$rc>>2] = 0;
 $2 = HEAP32[$p>>2]|0;
 $3 = ((($2)) + 18|0);
 $4 = HEAP16[$3>>1]|0;
 $5 = $4&65535;
 $6 = $5 & 16;
 $7 = ($6|0)!=(0);
 if (!($7)) {
  $10 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($10|0);
 }
 $8 = HEAP32[$rc>>2]|0;
 $9 = $8 | 4096;
 HEAP32[$rc>>2] = $9;
 $10 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($10|0);
}
function _unixShmLock($fd,$ofst,$n,$flags) {
 $fd = $fd|0;
 $ofst = $ofst|0;
 $n = $n|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $allMask = 0, $allShared = 0, $mask = 0, $p = 0, $pDbFd = 0, $pShmNode = 0, $pX = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $3 = sp + 20|0;
 $pDbFd = sp + 16|0;
 $p = sp + 12|0;
 $pX = sp + 8|0;
 $pShmNode = sp + 4|0;
 $rc = sp;
 $mask = sp + 40|0;
 $allMask = sp + 38|0;
 $allShared = sp + 36|0;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $ofst;
 HEAP32[$2>>2] = $n;
 HEAP32[$3>>2] = $flags;
 $4 = HEAP32[$0>>2]|0;
 HEAP32[$pDbFd>>2] = $4;
 $5 = HEAP32[$pDbFd>>2]|0;
 $6 = ((($5)) + 36|0);
 $7 = HEAP32[$6>>2]|0;
 HEAP32[$p>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 HEAP32[$pShmNode>>2] = $9;
 HEAP32[$rc>>2] = 0;
 $10 = HEAP32[$1>>2]|0;
 $11 = HEAP32[$2>>2]|0;
 $12 = (($10) + ($11))|0;
 $13 = 1 << $12;
 $14 = HEAP32[$1>>2]|0;
 $15 = 1 << $14;
 $16 = (($13) - ($15))|0;
 $17 = $16&65535;
 HEAP16[$mask>>1] = $17;
 $18 = HEAP32[$3>>2]|0;
 $19 = $18 & 1;
 $20 = ($19|0)!=(0);
 if ($20) {
  HEAP16[$allMask>>1] = 0;
  $21 = HEAP32[$pShmNode>>2]|0;
  $22 = ((($21)) + 32|0);
  $23 = HEAP32[$22>>2]|0;
  HEAP32[$pX>>2] = $23;
  while(1) {
   $24 = HEAP32[$pX>>2]|0;
   $25 = ($24|0)!=(0|0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$pX>>2]|0;
   $27 = HEAP32[$p>>2]|0;
   $28 = ($26|0)==($27|0);
   if (!($28)) {
    $29 = HEAP32[$pX>>2]|0;
    $30 = ((($29)) + 10|0);
    $31 = HEAP16[$30>>1]|0;
    $32 = $31&65535;
    $33 = HEAP16[$allMask>>1]|0;
    $34 = $33&65535;
    $35 = $34 | $32;
    $36 = $35&65535;
    HEAP16[$allMask>>1] = $36;
   }
   $37 = HEAP32[$pX>>2]|0;
   $38 = ((($37)) + 4|0);
   $39 = HEAP32[$38>>2]|0;
   HEAP32[$pX>>2] = $39;
  }
  $40 = HEAP16[$mask>>1]|0;
  $41 = $40&65535;
  $42 = HEAP16[$allMask>>1]|0;
  $43 = $42&65535;
  $44 = $41 & $43;
  $45 = ($44|0)==(0);
  if ($45) {
   $46 = HEAP32[$pDbFd>>2]|0;
   $47 = HEAP32[$1>>2]|0;
   $48 = (($47) + 120)|0;
   $49 = HEAP32[$2>>2]|0;
   $50 = (_unixShmSystemLock($46,2,$48,$49)|0);
   HEAP32[$rc>>2] = $50;
  } else {
   HEAP32[$rc>>2] = 0;
  }
  $51 = HEAP32[$rc>>2]|0;
  $52 = ($51|0)==(0);
  if (!($52)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $53 = HEAP16[$mask>>1]|0;
  $54 = $53&65535;
  $55 = $54 ^ -1;
  $56 = HEAP32[$p>>2]|0;
  $57 = ((($56)) + 12|0);
  $58 = HEAP16[$57>>1]|0;
  $59 = $58&65535;
  $60 = $59 & $55;
  $61 = $60&65535;
  HEAP16[$57>>1] = $61;
  $62 = HEAP16[$mask>>1]|0;
  $63 = $62&65535;
  $64 = $63 ^ -1;
  $65 = HEAP32[$p>>2]|0;
  $66 = ((($65)) + 10|0);
  $67 = HEAP16[$66>>1]|0;
  $68 = $67&65535;
  $69 = $68 & $64;
  $70 = $69&65535;
  HEAP16[$66>>1] = $70;
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $71 = HEAP32[$3>>2]|0;
 $72 = $71 & 4;
 $73 = ($72|0)!=(0);
 if (!($73)) {
  $121 = HEAP32[$pShmNode>>2]|0;
  $122 = ((($121)) + 32|0);
  $123 = HEAP32[$122>>2]|0;
  HEAP32[$pX>>2] = $123;
  while(1) {
   $124 = HEAP32[$pX>>2]|0;
   $125 = ($124|0)!=(0|0);
   if (!($125)) {
    break;
   }
   $126 = HEAP32[$pX>>2]|0;
   $127 = ((($126)) + 12|0);
   $128 = HEAP16[$127>>1]|0;
   $129 = $128&65535;
   $130 = HEAP16[$mask>>1]|0;
   $131 = $130&65535;
   $132 = $129 & $131;
   $133 = ($132|0)!=(0);
   if ($133) {
    label = 28;
    break;
   }
   $134 = HEAP32[$pX>>2]|0;
   $135 = ((($134)) + 10|0);
   $136 = HEAP16[$135>>1]|0;
   $137 = $136&65535;
   $138 = HEAP16[$mask>>1]|0;
   $139 = $138&65535;
   $140 = $137 & $139;
   $141 = ($140|0)!=(0);
   if ($141) {
    label = 28;
    break;
   }
   $142 = HEAP32[$pX>>2]|0;
   $143 = ((($142)) + 4|0);
   $144 = HEAP32[$143>>2]|0;
   HEAP32[$pX>>2] = $144;
  }
  if ((label|0) == 28) {
   HEAP32[$rc>>2] = 5;
  }
  $145 = HEAP32[$rc>>2]|0;
  $146 = ($145|0)==(0);
  if (!($146)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $147 = HEAP32[$pDbFd>>2]|0;
  $148 = HEAP32[$1>>2]|0;
  $149 = (($148) + 120)|0;
  $150 = HEAP32[$2>>2]|0;
  $151 = (_unixShmSystemLock($147,1,$149,$150)|0);
  HEAP32[$rc>>2] = $151;
  $152 = HEAP32[$rc>>2]|0;
  $153 = ($152|0)==(0);
  if (!($153)) {
   $162 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($162|0);
  }
  $154 = HEAP16[$mask>>1]|0;
  $155 = $154&65535;
  $156 = HEAP32[$p>>2]|0;
  $157 = ((($156)) + 12|0);
  $158 = HEAP16[$157>>1]|0;
  $159 = $158&65535;
  $160 = $159 | $155;
  $161 = $160&65535;
  HEAP16[$157>>1] = $161;
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 HEAP16[$allShared>>1] = 0;
 $74 = HEAP32[$pShmNode>>2]|0;
 $75 = ((($74)) + 32|0);
 $76 = HEAP32[$75>>2]|0;
 HEAP32[$pX>>2] = $76;
 while(1) {
  $77 = HEAP32[$pX>>2]|0;
  $78 = ($77|0)!=(0|0);
  if (!($78)) {
   break;
  }
  $79 = HEAP32[$pX>>2]|0;
  $80 = ((($79)) + 12|0);
  $81 = HEAP16[$80>>1]|0;
  $82 = $81&65535;
  $83 = HEAP16[$mask>>1]|0;
  $84 = $83&65535;
  $85 = $82 & $84;
  $86 = ($85|0)!=(0);
  if ($86) {
   label = 16;
   break;
  }
  $87 = HEAP32[$pX>>2]|0;
  $88 = ((($87)) + 10|0);
  $89 = HEAP16[$88>>1]|0;
  $90 = $89&65535;
  $91 = HEAP16[$allShared>>1]|0;
  $92 = $91&65535;
  $93 = $92 | $90;
  $94 = $93&65535;
  HEAP16[$allShared>>1] = $94;
  $95 = HEAP32[$pX>>2]|0;
  $96 = ((($95)) + 4|0);
  $97 = HEAP32[$96>>2]|0;
  HEAP32[$pX>>2] = $97;
 }
 if ((label|0) == 16) {
  HEAP32[$rc>>2] = 5;
 }
 $98 = HEAP32[$rc>>2]|0;
 $99 = ($98|0)==(0);
 do {
  if ($99) {
   $100 = HEAP16[$allShared>>1]|0;
   $101 = $100&65535;
   $102 = HEAP16[$mask>>1]|0;
   $103 = $102&65535;
   $104 = $101 & $103;
   $105 = ($104|0)==(0);
   if ($105) {
    $106 = HEAP32[$pDbFd>>2]|0;
    $107 = HEAP32[$1>>2]|0;
    $108 = (($107) + 120)|0;
    $109 = HEAP32[$2>>2]|0;
    $110 = (_unixShmSystemLock($106,0,$108,$109)|0);
    HEAP32[$rc>>2] = $110;
    break;
   } else {
    HEAP32[$rc>>2] = 0;
    break;
   }
  }
 } while(0);
 $111 = HEAP32[$rc>>2]|0;
 $112 = ($111|0)==(0);
 if (!($112)) {
  $162 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($162|0);
 }
 $113 = HEAP16[$mask>>1]|0;
 $114 = $113&65535;
 $115 = HEAP32[$p>>2]|0;
 $116 = ((($115)) + 10|0);
 $117 = HEAP16[$116>>1]|0;
 $118 = $117&65535;
 $119 = $118 | $114;
 $120 = $119&65535;
 HEAP16[$116>>1] = $120;
 $162 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _unixShmBarrier($fd) {
 $fd = $fd|0;
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $fd;
 _unixEnterMutex();
 _unixLeaveMutex();
 STACKTOP = sp;return;
}
function _unixShmUnmap($fd,$deleteFlag) {
 $fd = $fd|0;
 $deleteFlag = $deleteFlag|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pDbFd = 0, $pShmNode = 0, $pp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $pShmNode = sp + 8|0;
 $pp = sp + 4|0;
 $pDbFd = sp;
 HEAP32[$1>>2] = $fd;
 HEAP32[$2>>2] = $deleteFlag;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pDbFd>>2] = $3;
 $4 = HEAP32[$pDbFd>>2]|0;
 $5 = ((($4)) + 36|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$p>>2]|0;
 $8 = ($7|0)==(0|0);
 if ($8) {
  HEAP32[$0>>2] = 0;
  $46 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $9 = HEAP32[$p>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 HEAP32[$pShmNode>>2] = $10;
 $11 = HEAP32[$pShmNode>>2]|0;
 $12 = ((($11)) + 32|0);
 HEAP32[$pp>>2] = $12;
 while(1) {
  $13 = HEAP32[$pp>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  $15 = HEAP32[$p>>2]|0;
  $16 = ($14|0)!=($15|0);
  if (!($16)) {
   break;
  }
  $17 = HEAP32[$pp>>2]|0;
  $18 = HEAP32[$17>>2]|0;
  $19 = ((($18)) + 4|0);
  HEAP32[$pp>>2] = $19;
 }
 $20 = HEAP32[$p>>2]|0;
 $21 = ((($20)) + 4|0);
 $22 = HEAP32[$21>>2]|0;
 $23 = HEAP32[$pp>>2]|0;
 HEAP32[$23>>2] = $22;
 $24 = HEAP32[$p>>2]|0;
 _sqlite3_free($24);
 $25 = HEAP32[$pDbFd>>2]|0;
 $26 = ((($25)) + 36|0);
 HEAP32[$26>>2] = 0;
 _unixEnterMutex();
 $27 = HEAP32[$pShmNode>>2]|0;
 $28 = ((($27)) + 28|0);
 $29 = HEAP32[$28>>2]|0;
 $30 = (($29) + -1)|0;
 HEAP32[$28>>2] = $30;
 $31 = HEAP32[$pShmNode>>2]|0;
 $32 = ((($31)) + 28|0);
 $33 = HEAP32[$32>>2]|0;
 $34 = ($33|0)==(0);
 if ($34) {
  $35 = HEAP32[$2>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$pShmNode>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = ($39|0)>=(0);
   if ($40) {
    $41 = HEAP32[(6632)>>2]|0;
    $42 = HEAP32[$pShmNode>>2]|0;
    $43 = ((($42)) + 8|0);
    $44 = HEAP32[$43>>2]|0;
    (FUNCTION_TABLE_ii[$41 & 255]($44)|0);
   }
  }
  $45 = HEAP32[$pDbFd>>2]|0;
  _unixShmPurge($45);
 }
 _unixLeaveMutex();
 HEAP32[$0>>2] = 0;
 $46 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _unixFetch($fd,$0,$1,$nAmt,$pp) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $nAmt = $nAmt|0;
 $pp = $pp|0;
 var $10 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $nAmt;
 HEAP32[$5>>2] = $pp;
 $10 = HEAP32[$5>>2]|0;
 HEAP32[$10>>2] = 0;
 STACKTOP = sp;return 0;
}
function _unixUnfetch($fd,$0,$1,$p) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $p = $p|0;
 var $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $4 = sp + 8|0;
 HEAP32[$2>>2] = $fd;
 $5 = $3;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 HEAP32[$4>>2] = $p;
 STACKTOP = sp;return 0;
}
function _unixEnterMutex() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _unixShmPurge($pFd) {
 $pFd = $pFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $nShmPerMap = 0;
 var $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $p = sp + 8|0;
 $nShmPerMap = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $pFd;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 20|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$p>>2] = $5;
 $6 = HEAP32[$p>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$p>>2]|0;
 $9 = ((($8)) + 28|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = (_unixShmRegionPerMap()|0);
 HEAP32[$nShmPerMap>>2] = $12;
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$p>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP16[$15>>1]|0;
  $17 = $16&65535;
  $18 = ($13|0)<($17|0);
  $19 = HEAP32[$p>>2]|0;
  if (!($18)) {
   break;
  }
  $20 = ((($19)) + 12|0);
  $21 = HEAP32[$20>>2]|0;
  $22 = ($21|0)>=(0);
  if ($22) {
   $23 = HEAP32[(6716)>>2]|0;
   $24 = HEAP32[$i>>2]|0;
   $25 = HEAP32[$p>>2]|0;
   $26 = ((($25)) + 24|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = (($27) + ($24<<2)|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = HEAP32[$p>>2]|0;
   $31 = ((($30)) + 16|0);
   $32 = HEAP32[$31>>2]|0;
   (FUNCTION_TABLE_iii[$23 & 255]($29,$32)|0);
  } else {
   $33 = HEAP32[$i>>2]|0;
   $34 = HEAP32[$p>>2]|0;
   $35 = ((($34)) + 24|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = (($36) + ($33<<2)|0);
   $38 = HEAP32[$37>>2]|0;
   _sqlite3_free($38);
  }
  $39 = HEAP32[$nShmPerMap>>2]|0;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + ($39))|0;
  HEAP32[$i>>2] = $41;
 }
 $42 = ((($19)) + 24|0);
 $43 = HEAP32[$42>>2]|0;
 _sqlite3_free($43);
 $44 = HEAP32[$p>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 $47 = ($46|0)>=(0);
 if ($47) {
  $48 = HEAP32[$0>>2]|0;
  $49 = HEAP32[$p>>2]|0;
  $50 = ((($49)) + 12|0);
  $51 = HEAP32[$50>>2]|0;
  _robust_close($48,$51,33574);
  $52 = HEAP32[$p>>2]|0;
  $53 = ((($52)) + 12|0);
  HEAP32[$53>>2] = -1;
 }
 $54 = HEAP32[$p>>2]|0;
 $55 = HEAP32[$54>>2]|0;
 $56 = ((($55)) + 20|0);
 HEAP32[$56>>2] = 0;
 $57 = HEAP32[$p>>2]|0;
 _sqlite3_free($57);
 STACKTOP = sp;return;
}
function _unixLeaveMutex() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _unixShmRegionPerMap() {
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pgsz = 0, $shmsz = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $shmsz = sp + 4|0;
 $pgsz = sp;
 HEAP32[$shmsz>>2] = 32768;
 $1 = HEAP32[(6740)>>2]|0;
 $2 = (FUNCTION_TABLE_i[$1 & 255]()|0);
 HEAP32[$pgsz>>2] = $2;
 $3 = HEAP32[$pgsz>>2]|0;
 $4 = HEAP32[$shmsz>>2]|0;
 $5 = ($3|0)<($4|0);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 } else {
  $6 = HEAP32[$pgsz>>2]|0;
  $7 = HEAP32[$shmsz>>2]|0;
  $8 = (($6|0) / ($7|0))&-1;
  HEAP32[$0>>2] = $8;
  $9 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($9|0);
 }
 return (0)|0;
}
function _robust_close($pFile,$h,$lineno) {
 $pFile = $pFile|0;
 $h = $h|0;
 $lineno = $lineno|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$1>>2] = $h;
 HEAP32[$2>>2] = $lineno;
 $3 = HEAP32[(6452)>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = (FUNCTION_TABLE_ii[$3 & 255]($4)|0);
 $6 = ($5|0)!=(0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = ($7|0)!=(0|0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 32|0);
  $11 = HEAP32[$10>>2]|0;
  $13 = $11;
 } else {
  $13 = 0;
 }
 $12 = HEAP32[$2>>2]|0;
 (_unixLogErrorAtLine(4106,44091,$13,$12)|0);
 STACKTOP = sp;return;
}
function _unixLogErrorAtLine($errcode,$zFunc,$zPath,$iLine) {
 $errcode = $errcode|0;
 $zFunc = $zFunc|0;
 $zPath = $zPath|0;
 $iLine = $iLine|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iErrno = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $zErr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $zErr = sp + 24|0;
 $iErrno = sp + 20|0;
 HEAP32[$0>>2] = $errcode;
 HEAP32[$1>>2] = $zFunc;
 HEAP32[$2>>2] = $zPath;
 HEAP32[$3>>2] = $iLine;
 $4 = (___errno_location()|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$iErrno>>2] = $5;
 $6 = HEAP32[$iErrno>>2]|0;
 $7 = (_strerror($6)|0);
 HEAP32[$zErr>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  HEAP32[$2>>2] = 18402;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$iErrno>>2]|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$zErr>>2]|0;
 HEAP32[$vararg_buffer>>2] = $11;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $12;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $14;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $15;
 _sqlite3_log($10,44097,$vararg_buffer);
 $16 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($16|0);
}
function _posixOpen($zFile,$flags,$mode) {
 $zFile = $zFile|0;
 $flags = $flags|0;
 $mode = $mode|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $zFile;
 HEAP32[$1>>2] = $flags;
 HEAP32[$2>>2] = $mode;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$vararg_buffer>>2] = $5;
 $6 = (_open($3,$4,$vararg_buffer)|0);
 STACKTOP = sp;return ($6|0);
}
function _openDirectory($zFilename,$pFd) {
 $zFilename = $zFilename|0;
 $pFd = $pFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $ii = 0, $vararg_buffer = 0, $zDirname = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 544|0;
 $vararg_buffer = sp;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $ii = sp + 8|0;
 $fd = sp + 4|0;
 $zDirname = sp + 24|0;
 HEAP32[$1>>2] = $zFilename;
 HEAP32[$2>>2] = $pFd;
 HEAP32[$fd>>2] = -1;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $3;
 (_sqlite3_snprintf(512,$zDirname,18391,$vararg_buffer)|0);
 $4 = (_strlen($zDirname)|0);
 HEAP32[$ii>>2] = $4;
 while(1) {
  $5 = HEAP32[$ii>>2]|0;
  $6 = ($5|0)>(0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$ii>>2]|0;
  $8 = (($zDirname) + ($7)|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9 << 24 >> 24;
  $11 = ($10|0)!=(47);
  if (!($11)) {
   break;
  }
  $12 = HEAP32[$ii>>2]|0;
  $13 = (($12) + -1)|0;
  HEAP32[$ii>>2] = $13;
 }
 $14 = HEAP32[$ii>>2]|0;
 $15 = ($14|0)>(0);
 if ($15) {
  $16 = HEAP32[$ii>>2]|0;
  $17 = (($zDirname) + ($16)|0);
  HEAP8[$17>>0] = 0;
 } else {
  $18 = HEAP8[$zDirname>>0]|0;
  $19 = $18 << 24 >> 24;
  $20 = ($19|0)!=(47);
  if ($20) {
   HEAP8[$zDirname>>0] = 46;
  }
  $21 = ((($zDirname)) + 1|0);
  HEAP8[$21>>0] = 0;
 }
 $22 = (_robust_open($zDirname,0,0)|0);
 HEAP32[$fd>>2] = $22;
 $23 = HEAP32[$fd>>2]|0;
 $24 = HEAP32[$2>>2]|0;
 HEAP32[$24>>2] = $23;
 $25 = HEAP32[$fd>>2]|0;
 $26 = ($25|0)>=(0);
 if ($26) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 } else {
  $27 = (_sqlite3CantopenError(32912)|0);
  $28 = (_unixLogErrorAtLine($27,44232,$zDirname,32912)|0);
  HEAP32[$0>>2] = $28;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _unixGetpagesize() {
 var $0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_sysconf(30)|0);
 return ($0|0);
}
function _robust_open($z,$f,$m) {
 $z = $z|0;
 $f = $f|0;
 $m = $m|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $m2 = 0, $or$cond = 0, $statbuf = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer = sp;
 $0 = sp + 100|0;
 $1 = sp + 96|0;
 $2 = sp + 92|0;
 $fd = sp + 88|0;
 $m2 = sp + 84|0;
 $statbuf = sp + 8|0;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $f;
 HEAP32[$2>>2] = $m;
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0);
 $5 = HEAP32[$2>>2]|0;
 $6 = $4 ? $5 : 420;
 HEAP32[$m2>>2] = $6;
 while(1) {
  $7 = HEAP32[(6440)>>2]|0;
  $8 = HEAP32[$0>>2]|0;
  $9 = HEAP32[$1>>2]|0;
  $10 = $9 | 524288;
  $11 = HEAP32[$m2>>2]|0;
  $12 = (FUNCTION_TABLE_iiii[$7 & 255]($8,$10,$11)|0);
  HEAP32[$fd>>2] = $12;
  $13 = HEAP32[$fd>>2]|0;
  $14 = ($13|0)<(0);
  if ($14) {
   $15 = (___errno_location()|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = ($16|0)==(4);
   if ($17) {
    continue;
   } else {
    break;
   }
  }
  $18 = HEAP32[$fd>>2]|0;
  $19 = ($18|0)>=(3);
  if ($19) {
   break;
  }
  $20 = HEAP32[(6452)>>2]|0;
  $21 = HEAP32[$fd>>2]|0;
  (FUNCTION_TABLE_ii[$20 & 255]($21)|0);
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$fd>>2]|0;
  HEAP32[$vararg_buffer>>2] = $22;
  $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr2>>2] = $23;
  _sqlite3_log(28,44319,$vararg_buffer);
  HEAP32[$fd>>2] = -1;
  $24 = HEAP32[(6440)>>2]|0;
  $25 = HEAP32[$1>>2]|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (FUNCTION_TABLE_iiii[$24 & 255](44362,$25,$26)|0);
  $28 = ($27|0)<(0);
  if ($28) {
   break;
  }
 }
 $29 = HEAP32[$fd>>2]|0;
 $30 = ($29|0)>=(0);
 $31 = HEAP32[$2>>2]|0;
 $32 = ($31|0)!=(0);
 $or$cond = $30 & $32;
 if (!($or$cond)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $33 = HEAP32[(6500)>>2]|0;
 $34 = HEAP32[$fd>>2]|0;
 $35 = (FUNCTION_TABLE_iii[$33 & 255]($34,$statbuf)|0);
 $36 = ($35|0)==(0);
 if (!($36)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $37 = ((($statbuf)) + 36|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38|0)==(0);
 if (!($39)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $40 = ((($statbuf)) + 12|0);
 $41 = HEAP32[$40>>2]|0;
 $42 = $41 & 511;
 $43 = HEAP32[$2>>2]|0;
 $44 = ($42|0)!=($43|0);
 if (!($44)) {
  $48 = HEAP32[$fd>>2]|0;
  STACKTOP = sp;return ($48|0);
 }
 $45 = HEAP32[(6608)>>2]|0;
 $46 = HEAP32[$fd>>2]|0;
 $47 = HEAP32[$2>>2]|0;
 (FUNCTION_TABLE_iii[$45 & 255]($46,$47)|0);
 $48 = HEAP32[$fd>>2]|0;
 STACKTOP = sp;return ($48|0);
}
function _unixShmSystemLock($pFile,$lockType,$ofst,$n) {
 $pFile = $pFile|0;
 $lockType = $lockType|0;
 $ofst = $ofst|0;
 $n = $n|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $f = 0, $pShmNode = 0, $rc = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $pShmNode = sp + 24|0;
 $f = sp + 8|0;
 $rc = sp + 4|0;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$1>>2] = $lockType;
 HEAP32[$2>>2] = $ofst;
 HEAP32[$3>>2] = $n;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ((($6)) + 20|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pShmNode>>2] = $8;
 $9 = HEAP32[$pShmNode>>2]|0;
 $10 = ((($9)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>=(0);
 if (!($12)) {
  $28 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($28|0);
 }
 ;HEAP32[$f>>2]=0|0;HEAP32[$f+4>>2]=0|0;HEAP32[$f+8>>2]=0|0;HEAP32[$f+12>>2]=0|0;
 $13 = HEAP32[$1>>2]|0;
 $14 = $13&65535;
 HEAP16[$f>>1] = $14;
 $15 = ((($f)) + 2|0);
 HEAP16[$15>>1] = 0;
 $16 = HEAP32[$2>>2]|0;
 $17 = ((($f)) + 4|0);
 HEAP32[$17>>2] = $16;
 $18 = HEAP32[$3>>2]|0;
 $19 = ((($f)) + 8|0);
 HEAP32[$19>>2] = $18;
 $20 = HEAP32[(6524)>>2]|0;
 $21 = HEAP32[$pShmNode>>2]|0;
 $22 = ((($21)) + 12|0);
 $23 = HEAP32[$22>>2]|0;
 HEAP32[$vararg_buffer>>2] = $f;
 $24 = (FUNCTION_TABLE_iiii[$20 & 255]($23,13,$vararg_buffer)|0);
 HEAP32[$rc>>2] = $24;
 $25 = HEAP32[$rc>>2]|0;
 $26 = ($25|0)!=(-1);
 $27 = $26 ? 0 : 5;
 HEAP32[$rc>>2] = $27;
 $28 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($28|0);
}
function _fcntlSizeHint($pFile,$0,$1) {
 $pFile = $pFile|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0;
 var $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0;
 var $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0;
 var $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0;
 var $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0;
 var $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0;
 var $99 = 0, $buf = 0, $iWrite = 0, $nBlk = 0, $nSize = 0, $nWrite = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $2 = sp + 112|0;
 $3 = sp + 108|0;
 $4 = sp + 16|0;
 $nSize = sp + 8|0;
 $buf = sp + 32|0;
 $nBlk = sp + 28|0;
 $nWrite = sp + 24|0;
 $iWrite = sp;
 HEAP32[$3>>2] = $pFile;
 $5 = $4;
 $6 = $5;
 HEAP32[$6>>2] = $0;
 $7 = (($5) + 4)|0;
 $8 = $7;
 HEAP32[$8>>2] = $1;
 $9 = HEAP32[$3>>2]|0;
 $10 = ((($9)) + 40|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 L1: do {
  if ($12) {
   $13 = HEAP32[(6500)>>2]|0;
   $14 = HEAP32[$3>>2]|0;
   $15 = ((($14)) + 12|0);
   $16 = HEAP32[$15>>2]|0;
   $17 = (FUNCTION_TABLE_iii[$13 & 255]($16,$buf)|0);
   $18 = ($17|0)!=(0);
   if ($18) {
    HEAP32[$2>>2] = 1802;
    $162 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($162|0);
   }
   $19 = $4;
   $20 = $19;
   $21 = HEAP32[$20>>2]|0;
   $22 = (($19) + 4)|0;
   $23 = $22;
   $24 = HEAP32[$23>>2]|0;
   $25 = HEAP32[$3>>2]|0;
   $26 = ((($25)) + 40|0);
   $27 = HEAP32[$26>>2]|0;
   $28 = ($27|0)<(0);
   $29 = $28 << 31 >> 31;
   $30 = (_i64Add(($21|0),($24|0),($27|0),($29|0))|0);
   $31 = tempRet0;
   $32 = (_i64Subtract(($30|0),($31|0),1,0)|0);
   $33 = tempRet0;
   $34 = HEAP32[$3>>2]|0;
   $35 = ((($34)) + 40|0);
   $36 = HEAP32[$35>>2]|0;
   $37 = ($36|0)<(0);
   $38 = $37 << 31 >> 31;
   $39 = (___divdi3(($32|0),($33|0),($36|0),($38|0))|0);
   $40 = tempRet0;
   $41 = HEAP32[$3>>2]|0;
   $42 = ((($41)) + 40|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)<(0);
   $45 = $44 << 31 >> 31;
   $46 = (___muldi3(($39|0),($40|0),($43|0),($45|0))|0);
   $47 = tempRet0;
   $48 = $nSize;
   $49 = $48;
   HEAP32[$49>>2] = $46;
   $50 = (($48) + 4)|0;
   $51 = $50;
   HEAP32[$51>>2] = $47;
   $52 = $nSize;
   $53 = $52;
   $54 = HEAP32[$53>>2]|0;
   $55 = (($52) + 4)|0;
   $56 = $55;
   $57 = HEAP32[$56>>2]|0;
   $58 = ((($buf)) + 36|0);
   $59 = HEAP32[$58>>2]|0;
   $60 = ($59|0)<(0);
   $61 = $60 << 31 >> 31;
   $62 = ($57|0)>($61|0);
   $63 = ($54>>>0)>($59>>>0);
   $64 = ($57|0)==($61|0);
   $65 = $64 & $63;
   $66 = $62 | $65;
   if ($66) {
    $67 = ((($buf)) + 40|0);
    $68 = HEAP32[$67>>2]|0;
    HEAP32[$nBlk>>2] = $68;
    HEAP32[$nWrite>>2] = 0;
    $69 = ((($buf)) + 36|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = HEAP32[$nBlk>>2]|0;
    $72 = (($70|0) / ($71|0))&-1;
    $73 = HEAP32[$nBlk>>2]|0;
    $74 = Math_imul($72, $73)|0;
    $75 = HEAP32[$nBlk>>2]|0;
    $76 = (($74) + ($75))|0;
    $77 = (($76) - 1)|0;
    $78 = ($77|0)<(0);
    $79 = $78 << 31 >> 31;
    $80 = $iWrite;
    $81 = $80;
    HEAP32[$81>>2] = $77;
    $82 = (($80) + 4)|0;
    $83 = $82;
    HEAP32[$83>>2] = $79;
    while(1) {
     $84 = $iWrite;
     $85 = $84;
     $86 = HEAP32[$85>>2]|0;
     $87 = (($84) + 4)|0;
     $88 = $87;
     $89 = HEAP32[$88>>2]|0;
     $90 = $nSize;
     $91 = $90;
     $92 = HEAP32[$91>>2]|0;
     $93 = (($90) + 4)|0;
     $94 = $93;
     $95 = HEAP32[$94>>2]|0;
     $96 = HEAP32[$nBlk>>2]|0;
     $97 = ($96|0)<(0);
     $98 = $97 << 31 >> 31;
     $99 = (_i64Add(($92|0),($95|0),($96|0),($98|0))|0);
     $100 = tempRet0;
     $101 = (_i64Subtract(($99|0),($100|0),1,0)|0);
     $102 = tempRet0;
     $103 = ($89|0)<($102|0);
     $104 = ($86>>>0)<($101>>>0);
     $105 = ($89|0)==($102|0);
     $106 = $105 & $104;
     $107 = $103 | $106;
     if (!($107)) {
      break L1;
     }
     $108 = $iWrite;
     $109 = $108;
     $110 = HEAP32[$109>>2]|0;
     $111 = (($108) + 4)|0;
     $112 = $111;
     $113 = HEAP32[$112>>2]|0;
     $114 = $nSize;
     $115 = $114;
     $116 = HEAP32[$115>>2]|0;
     $117 = (($114) + 4)|0;
     $118 = $117;
     $119 = HEAP32[$118>>2]|0;
     $120 = ($113|0)>($119|0);
     $121 = ($110>>>0)>=($116>>>0);
     $122 = ($113|0)==($119|0);
     $123 = $122 & $121;
     $124 = $120 | $123;
     if ($124) {
      $125 = $nSize;
      $126 = $125;
      $127 = HEAP32[$126>>2]|0;
      $128 = (($125) + 4)|0;
      $129 = $128;
      $130 = HEAP32[$129>>2]|0;
      $131 = (_i64Subtract(($127|0),($130|0),1,0)|0);
      $132 = tempRet0;
      $133 = $iWrite;
      $134 = $133;
      HEAP32[$134>>2] = $131;
      $135 = (($133) + 4)|0;
      $136 = $135;
      HEAP32[$136>>2] = $132;
     }
     $137 = HEAP32[$3>>2]|0;
     $138 = $iWrite;
     $139 = $138;
     $140 = HEAP32[$139>>2]|0;
     $141 = (($138) + 4)|0;
     $142 = $141;
     $143 = HEAP32[$142>>2]|0;
     $144 = (_seekAndWrite($137,$140,$143,18402,1)|0);
     HEAP32[$nWrite>>2] = $144;
     $145 = HEAP32[$nWrite>>2]|0;
     $146 = ($145|0)!=(1);
     if ($146) {
      break;
     }
     $147 = HEAP32[$nBlk>>2]|0;
     $148 = ($147|0)<(0);
     $149 = $148 << 31 >> 31;
     $150 = $iWrite;
     $151 = $150;
     $152 = HEAP32[$151>>2]|0;
     $153 = (($150) + 4)|0;
     $154 = $153;
     $155 = HEAP32[$154>>2]|0;
     $156 = (_i64Add(($152|0),($155|0),($147|0),($149|0))|0);
     $157 = tempRet0;
     $158 = $iWrite;
     $159 = $158;
     HEAP32[$159>>2] = $156;
     $160 = (($158) + 4)|0;
     $161 = $160;
     HEAP32[$161>>2] = $157;
    }
    HEAP32[$2>>2] = 778;
    $162 = HEAP32[$2>>2]|0;
    STACKTOP = sp;return ($162|0);
   }
  }
 } while(0);
 HEAP32[$2>>2] = 0;
 $162 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($162|0);
}
function _unixModeBit($pFile,$mask,$pArg) {
 $pFile = $pFile|0;
 $mask = $mask|0;
 $pArg = $pArg|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp + 8|0;
 $2 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP8[$1>>0] = $mask;
 HEAP32[$2>>2] = $pArg;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<(0);
 if ($5) {
  $6 = HEAP32[$0>>2]|0;
  $7 = ((($6)) + 18|0);
  $8 = HEAP16[$7>>1]|0;
  $9 = $8&65535;
  $10 = HEAP8[$1>>0]|0;
  $11 = $10&255;
  $12 = $9 & $11;
  $13 = ($12|0)!=(0);
  $14 = $13&1;
  $15 = HEAP32[$2>>2]|0;
  HEAP32[$15>>2] = $14;
  STACKTOP = sp;return;
 }
 $16 = HEAP32[$2>>2]|0;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)==(0);
 $19 = HEAP8[$1>>0]|0;
 $20 = $19&255;
 if ($18) {
  $21 = $20 ^ -1;
  $22 = HEAP32[$0>>2]|0;
  $23 = ((($22)) + 18|0);
  $24 = HEAP16[$23>>1]|0;
  $25 = $24&65535;
  $26 = $25 & $21;
  $27 = $26&65535;
  HEAP16[$23>>1] = $27;
  STACKTOP = sp;return;
 } else {
  $28 = HEAP32[$0>>2]|0;
  $29 = ((($28)) + 18|0);
  $30 = HEAP16[$29>>1]|0;
  $31 = $30&65535;
  $32 = $31 | $20;
  $33 = $32&65535;
  HEAP16[$29>>1] = $33;
  STACKTOP = sp;return;
 }
}
function _unixGetTempname($nBuf,$zBuf) {
 $nBuf = $nBuf|0;
 $zBuf = $zBuf|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iLimit = 0;
 var $r = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $zDir = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $zDir = sp + 36|0;
 $iLimit = sp + 32|0;
 $r = sp;
 HEAP32[$1>>2] = $nBuf;
 HEAP32[$2>>2] = $zBuf;
 HEAP32[$iLimit>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 HEAP8[$3>>0] = 0;
 $4 = (_unixTempFileDir()|0);
 HEAP32[$zDir>>2] = $4;
 $5 = HEAP32[$zDir>>2]|0;
 $6 = ($5|0)==(0|0);
 if ($6) {
  HEAP32[$0>>2] = 6410;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 while(1) {
  _sqlite3_randomness(8,$r);
  $7 = HEAP32[$1>>2]|0;
  $8 = (($7) - 2)|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = (($9) + ($8)|0);
  HEAP8[$10>>0] = 0;
  $11 = HEAP32[$1>>2]|0;
  $12 = HEAP32[$2>>2]|0;
  $13 = HEAP32[$zDir>>2]|0;
  $14 = $r;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  HEAP32[$vararg_buffer>>2] = $13;
  $vararg_ptr1 = ((($vararg_buffer)) + 8|0);
  $20 = $vararg_ptr1;
  $21 = $20;
  HEAP32[$21>>2] = $16;
  $22 = (($20) + 4)|0;
  $23 = $22;
  HEAP32[$23>>2] = $19;
  $vararg_ptr2 = ((($vararg_buffer)) + 16|0);
  HEAP32[$vararg_ptr2>>2] = 0;
  (_sqlite3_snprintf($11,$12,44372,$vararg_buffer)|0);
  $24 = HEAP32[$1>>2]|0;
  $25 = (($24) - 2)|0;
  $26 = HEAP32[$2>>2]|0;
  $27 = (($26) + ($25)|0);
  $28 = HEAP8[$27>>0]|0;
  $29 = $28 << 24 >> 24;
  $30 = ($29|0)!=(0);
  if ($30) {
   label = 5;
   break;
  }
  $31 = HEAP32[$iLimit>>2]|0;
  $32 = (($31) + 1)|0;
  HEAP32[$iLimit>>2] = $32;
  $33 = ($31|0)>(10);
  if ($33) {
   label = 5;
   break;
  }
  $34 = HEAP32[(6464)>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = (FUNCTION_TABLE_iii[$34 & 255]($35,0)|0);
  $37 = ($36|0)==(0);
  if (!($37)) {
   label = 7;
   break;
  }
 }
 if ((label|0) == 5) {
  HEAP32[$0>>2] = 1;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 else if ((label|0) == 7) {
  HEAP32[$0>>2] = 0;
  $38 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($38|0);
 }
 return (0)|0;
}
function _fileHasMoved($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $buf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 76|0;
 $buf = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $20 = 0;
  $19 = $20&1;
  STACKTOP = sp;return ($19|0);
 }
 $5 = HEAP32[(6488)>>2]|0;
 $6 = HEAP32[$0>>2]|0;
 $7 = ((($6)) + 32|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (FUNCTION_TABLE_iii[$5 & 255]($8,$buf)|0);
 $10 = ($9|0)!=(0);
 if ($10) {
  $20 = 1;
  $19 = $20&1;
  STACKTOP = sp;return ($19|0);
 }
 $11 = ((($buf)) + 72|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = HEAP32[$0>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($15)) + 4|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($12|0)!=($17|0);
 $20 = $18;
 $19 = $20&1;
 STACKTOP = sp;return ($19|0);
}
function _unixTempFileDir() {
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $i = 0, $zDir = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 84|0;
 $i = sp + 80|0;
 $buf = sp + 4|0;
 $zDir = sp;
 HEAP32[$i>>2] = 0;
 $1 = HEAP32[972>>2]|0;
 HEAP32[$zDir>>2] = $1;
 $2 = HEAP32[6772>>2]|0;
 $3 = ($2|0)!=(0|0);
 if (!($3)) {
  $4 = (_getenv((44389|0))|0);
  HEAP32[6772>>2] = $4;
 }
 $5 = HEAP32[(6776)>>2]|0;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  $7 = (_getenv((44403|0))|0);
  HEAP32[(6776)>>2] = $7;
 }
 while(1) {
  $8 = HEAP32[$zDir>>2]|0;
  $9 = ($8|0)!=(0|0);
  if ($9) {
   $10 = HEAP32[(6488)>>2]|0;
   $11 = HEAP32[$zDir>>2]|0;
   $12 = (FUNCTION_TABLE_iii[$10 & 255]($11,$buf)|0);
   $13 = ($12|0)==(0);
   if ($13) {
    $14 = ((($buf)) + 12|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = $15 & 61440;
    $17 = ($16|0)==(16384);
    if ($17) {
     $18 = HEAP32[(6464)>>2]|0;
     $19 = HEAP32[$zDir>>2]|0;
     $20 = (FUNCTION_TABLE_iii[$18 & 255]($19,3)|0);
     $21 = ($20|0)==(0);
     if ($21) {
      label = 9;
      break;
     }
    }
   }
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = ($23>>>0)>=(6);
  if ($24) {
   label = 12;
   break;
  }
  $25 = HEAP32[$i>>2]|0;
  $26 = (($25) + 1)|0;
  HEAP32[$i>>2] = $26;
  $27 = (6772 + ($25<<2)|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$zDir>>2] = $28;
 }
 if ((label|0) == 9) {
  $22 = HEAP32[$zDir>>2]|0;
  HEAP32[$0>>2] = $22;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 else if ((label|0) == 12) {
  HEAP32[$0>>2] = 0;
  $29 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _seekAndWrite($id,$0,$1,$pBuf,$cnt) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $cnt = $cnt|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $4 = sp + 12|0;
 $5 = sp + 8|0;
 HEAP32[$2>>2] = $id;
 $6 = $3;
 $7 = $6;
 HEAP32[$7>>2] = $0;
 $8 = (($6) + 4)|0;
 $9 = $8;
 HEAP32[$9>>2] = $1;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $cnt;
 $10 = HEAP32[$2>>2]|0;
 $11 = ((($10)) + 12|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = $3;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = (($13) + 4)|0;
 $17 = $16;
 $18 = HEAP32[$17>>2]|0;
 $19 = HEAP32[$4>>2]|0;
 $20 = HEAP32[$5>>2]|0;
 $21 = HEAP32[$2>>2]|0;
 $22 = ((($21)) + 20|0);
 $23 = (_seekAndWriteFd($12,$15,$18,$19,$20,$22)|0);
 STACKTOP = sp;return ($23|0);
}
function _seekAndWriteFd($fd,$0,$1,$pBuf,$nBuf,$piErrno) {
 $fd = $fd|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $nBuf = $nBuf|0;
 $piErrno = $piErrno|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $iSeek = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 32|0;
 $3 = sp + 8|0;
 $4 = sp + 28|0;
 $5 = sp + 24|0;
 $6 = sp + 20|0;
 $rc = sp + 16|0;
 $iSeek = sp;
 HEAP32[$2>>2] = $fd;
 $7 = $3;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$4>>2] = $pBuf;
 HEAP32[$5>>2] = $nBuf;
 HEAP32[$6>>2] = $piErrno;
 HEAP32[$rc>>2] = 0;
 $11 = HEAP32[$5>>2]|0;
 $12 = $11 & 131071;
 HEAP32[$5>>2] = $12;
 while(1) {
  $13 = HEAP32[$2>>2]|0;
  $14 = $3;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_lseek($13,$16,0)|0);
  $21 = ($20|0)<(0);
  $22 = $21 << 31 >> 31;
  $23 = $iSeek;
  $24 = $23;
  HEAP32[$24>>2] = $20;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $iSeek;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)<(0);
  if ($33) {
   label = 3;
   break;
  }
  $34 = HEAP32[(6572)>>2]|0;
  $35 = HEAP32[$2>>2]|0;
  $36 = HEAP32[$4>>2]|0;
  $37 = HEAP32[$5>>2]|0;
  $38 = (FUNCTION_TABLE_iiii[$34 & 255]($35,$36,$37)|0);
  HEAP32[$rc>>2] = $38;
  $39 = HEAP32[$rc>>2]|0;
  $40 = ($39|0)<(0);
  if (!($40)) {
   break;
  }
  $41 = (___errno_location()|0);
  $42 = HEAP32[$41>>2]|0;
  $43 = ($42|0)==(4);
  if (!($43)) {
   break;
  }
 }
 if ((label|0) == 3) {
  HEAP32[$rc>>2] = -1;
 }
 $44 = HEAP32[$rc>>2]|0;
 $45 = ($44|0)<(0);
 if (!($45)) {
  $49 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($49|0);
 }
 $46 = (___errno_location()|0);
 $47 = HEAP32[$46>>2]|0;
 $48 = HEAP32[$6>>2]|0;
 HEAP32[$48>>2] = $47;
 $49 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($49|0);
}
function _storeLastErrno($pFile,$error) {
 $pFile = $pFile|0;
 $error = $error|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $pFile;
 HEAP32[$1>>2] = $error;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 20|0);
 HEAP32[$4>>2] = $2;
 STACKTOP = sp;return;
}
function _sqliteErrorFromPosixError($posixError,$sqliteIOErr) {
 $posixError = $posixError|0;
 $sqliteIOErr = $sqliteIOErr|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $posixError;
 HEAP32[$2>>2] = $sqliteIOErr;
 $3 = HEAP32[$1>>2]|0;
 switch ($3|0) {
 case 37: case 4: case 16: case 110: case 11: case 13:  {
  HEAP32[$0>>2] = 5;
  break;
 }
 case 1:  {
  HEAP32[$0>>2] = 3;
  break;
 }
 default: {
  $4 = HEAP32[$2>>2]|0;
  HEAP32[$0>>2] = $4;
 }
 }
 $5 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _full_fsync($fd,$fullSync,$dataOnly) {
 $fd = $fd|0;
 $fullSync = $fullSync|0;
 $dataOnly = $dataOnly|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $fullSync;
 HEAP32[$2>>2] = $dataOnly;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_fsync($3)|0);
 HEAP32[$rc>>2] = $4;
 $5 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($5|0);
}
function _robust_ftruncate($h,$0,$1) {
 $h = $h|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $2 = sp + 12|0;
 $3 = sp;
 $rc = sp + 8|0;
 HEAP32[$2>>2] = $h;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 while(1) {
  $8 = HEAP32[(6512)>>2]|0;
  $9 = HEAP32[$2>>2]|0;
  $10 = $3;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = (FUNCTION_TABLE_iii[$8 & 255]($9,$12)|0);
  HEAP32[$rc>>2] = $16;
  $17 = HEAP32[$rc>>2]|0;
  $18 = ($17|0)<(0);
  if (!($18)) {
   label = 4;
   break;
  }
  $19 = (___errno_location()|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = ($20|0)==(4);
  if (!($21)) {
   label = 4;
   break;
  }
 }
 if ((label|0) == 4) {
  $22 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($22|0);
 }
 return (0)|0;
}
function _seekAndRead($id,$0,$1,$pBuf,$cnt) {
 $id = $id|0;
 $0 = $0|0;
 $1 = $1|0;
 $pBuf = $pBuf|0;
 $cnt = $cnt|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $9 = 0, $got = 0, $newOffset = 0, $prior = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $4 = sp + 8|0;
 $5 = sp + 28|0;
 $6 = sp + 24|0;
 $got = sp + 20|0;
 $prior = sp + 16|0;
 $newOffset = sp;
 HEAP32[$3>>2] = $id;
 $7 = $4;
 $8 = $7;
 HEAP32[$8>>2] = $0;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $1;
 HEAP32[$5>>2] = $pBuf;
 HEAP32[$6>>2] = $cnt;
 HEAP32[$prior>>2] = 0;
 while(1) {
  $11 = HEAP32[$3>>2]|0;
  $12 = ((($11)) + 12|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = $4;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_lseek($13,$16,0)|0);
  $21 = ($20|0)<(0);
  $22 = $21 << 31 >> 31;
  $23 = $newOffset;
  $24 = $23;
  HEAP32[$24>>2] = $20;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $newOffset;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  $33 = ($32|0)<(0);
  if ($33) {
   label = 3;
   break;
  }
  $37 = HEAP32[(6536)>>2]|0;
  $38 = HEAP32[$3>>2]|0;
  $39 = ((($38)) + 12|0);
  $40 = HEAP32[$39>>2]|0;
  $41 = HEAP32[$5>>2]|0;
  $42 = HEAP32[$6>>2]|0;
  $43 = (FUNCTION_TABLE_iiii[$37 & 255]($40,$41,$42)|0);
  HEAP32[$got>>2] = $43;
  $44 = HEAP32[$got>>2]|0;
  $45 = HEAP32[$6>>2]|0;
  $46 = ($44|0)==($45|0);
  if ($46) {
   break;
  }
  $47 = HEAP32[$got>>2]|0;
  $48 = ($47|0)<(0);
  if ($48) {
   $49 = (___errno_location()|0);
   $50 = HEAP32[$49>>2]|0;
   $51 = ($50|0)==(4);
   if (!($51)) {
    label = 8;
    break;
   }
   HEAP32[$got>>2] = 1;
  } else {
   $55 = HEAP32[$got>>2]|0;
   $56 = ($55|0)>(0);
   if ($56) {
    $57 = HEAP32[$got>>2]|0;
    $58 = HEAP32[$6>>2]|0;
    $59 = (($58) - ($57))|0;
    HEAP32[$6>>2] = $59;
    $60 = HEAP32[$got>>2]|0;
    $61 = ($60|0)<(0);
    $62 = $61 << 31 >> 31;
    $63 = $4;
    $64 = $63;
    $65 = HEAP32[$64>>2]|0;
    $66 = (($63) + 4)|0;
    $67 = $66;
    $68 = HEAP32[$67>>2]|0;
    $69 = (_i64Add(($65|0),($68|0),($60|0),($62|0))|0);
    $70 = tempRet0;
    $71 = $4;
    $72 = $71;
    HEAP32[$72>>2] = $69;
    $73 = (($71) + 4)|0;
    $74 = $73;
    HEAP32[$74>>2] = $70;
    $75 = HEAP32[$got>>2]|0;
    $76 = HEAP32[$prior>>2]|0;
    $77 = (($76) + ($75))|0;
    HEAP32[$prior>>2] = $77;
    $78 = HEAP32[$got>>2]|0;
    $79 = HEAP32[$5>>2]|0;
    $80 = (($79) + ($78)|0);
    HEAP32[$5>>2] = $80;
   }
  }
  $81 = HEAP32[$got>>2]|0;
  $82 = ($81|0)>(0);
  if (!($82)) {
   break;
  }
 }
 if ((label|0) == 3) {
  $34 = HEAP32[$3>>2]|0;
  $35 = (___errno_location()|0);
  $36 = HEAP32[$35>>2]|0;
  _storeLastErrno($34,$36);
  HEAP32[$2>>2] = -1;
  $86 = HEAP32[$2>>2]|0;
  STACKTOP = sp;return ($86|0);
 }
 else if ((label|0) == 8) {
  HEAP32[$prior>>2] = 0;
  $52 = HEAP32[$3>>2]|0;
  $53 = (___errno_location()|0);
  $54 = HEAP32[$53>>2]|0;
  _storeLastErrno($52,$54);
 }
 $83 = HEAP32[$got>>2]|0;
 $84 = HEAP32[$prior>>2]|0;
 $85 = (($83) + ($84))|0;
 HEAP32[$2>>2] = $85;
 $86 = HEAP32[$2>>2]|0;
 STACKTOP = sp;return ($86|0);
}
function _closeUnixFile($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0, dest = 0, label = 0, sp = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$pFile>>2]|0;
 $3 = ((($2)) + 12|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)>=(0);
 if ($5) {
  $6 = HEAP32[$pFile>>2]|0;
  $7 = HEAP32[$pFile>>2]|0;
  $8 = ((($7)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  _robust_close($6,$9,31321);
  $10 = HEAP32[$pFile>>2]|0;
  $11 = ((($10)) + 12|0);
  HEAP32[$11>>2] = -1;
 }
 $12 = HEAP32[$pFile>>2]|0;
 $13 = ((($12)) + 28|0);
 $14 = HEAP32[$13>>2]|0;
 _sqlite3_free($14);
 $15 = HEAP32[$pFile>>2]|0;
 dest=$15; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 STACKTOP = sp;return 0;
}
function _nolockIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (6796|0);
}
function _nolockClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $id;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_closeUnixFile($1)|0);
 STACKTOP = sp;return ($2|0);
}
function _nolockLock($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _nolockUnlock($NotUsed,$NotUsed2) {
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $NotUsed2;
 STACKTOP = sp;return 0;
}
function _nolockCheckReservedLock($NotUsed,$pResOut) {
 $NotUsed = $NotUsed|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $NotUsed;
 HEAP32[$1>>2] = $pResOut;
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$2>>2] = 0;
 STACKTOP = sp;return 0;
}
function _mkFullPathname($zPath,$zOut,$nOut) {
 $zPath = $zPath|0;
 $zOut = $zOut|0;
 $nOut = $nOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $iOff = 0, $nPath = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $3 = sp + 12|0;
 $nPath = sp + 8|0;
 $iOff = sp + 4|0;
 HEAP32[$1>>2] = $zPath;
 HEAP32[$2>>2] = $zOut;
 HEAP32[$3>>2] = $nOut;
 $4 = HEAP32[$1>>2]|0;
 $5 = (_sqlite3Strlen30($4)|0);
 HEAP32[$nPath>>2] = $5;
 HEAP32[$iOff>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP8[$6>>0]|0;
 $8 = $7 << 24 >> 24;
 $9 = ($8|0)!=(47);
 do {
  if ($9) {
   $10 = HEAP32[(6476)>>2]|0;
   $11 = HEAP32[$2>>2]|0;
   $12 = HEAP32[$3>>2]|0;
   $13 = (($12) - 2)|0;
   $14 = (FUNCTION_TABLE_iii[$10 & 255]($11,$13)|0);
   $15 = ($14|0)==(0|0);
   if (!($15)) {
    $19 = HEAP32[$2>>2]|0;
    $20 = (_sqlite3Strlen30($19)|0);
    HEAP32[$iOff>>2] = $20;
    $21 = HEAP32[$iOff>>2]|0;
    $22 = (($21) + 1)|0;
    HEAP32[$iOff>>2] = $22;
    $23 = HEAP32[$2>>2]|0;
    $24 = (($23) + ($21)|0);
    HEAP8[$24>>0] = 47;
    break;
   }
   $16 = (_sqlite3CantopenError(35379)|0);
   $17 = HEAP32[$1>>2]|0;
   $18 = (_unixLogErrorAtLine($16,44133,$17,35379)|0);
   HEAP32[$0>>2] = $18;
   $42 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($42|0);
  }
 } while(0);
 $25 = HEAP32[$iOff>>2]|0;
 $26 = HEAP32[$nPath>>2]|0;
 $27 = (($25) + ($26))|0;
 $28 = (($27) + 1)|0;
 $29 = HEAP32[$3>>2]|0;
 $30 = ($28|0)>($29|0);
 if ($30) {
  $31 = HEAP32[$iOff>>2]|0;
  $32 = HEAP32[$2>>2]|0;
  $33 = (($32) + ($31)|0);
  HEAP8[$33>>0] = 0;
  $34 = (_sqlite3CantopenError(35388)|0);
  HEAP32[$0>>2] = $34;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 } else {
  $35 = HEAP32[$3>>2]|0;
  $36 = HEAP32[$iOff>>2]|0;
  $37 = (($35) - ($36))|0;
  $38 = HEAP32[$iOff>>2]|0;
  $39 = HEAP32[$2>>2]|0;
  $40 = (($39) + ($38)|0);
  $41 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer>>2] = $41;
  (_sqlite3_snprintf($37,$40,18391,$vararg_buffer)|0);
  HEAP32[$0>>2] = 0;
  $42 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($42|0);
 }
 return (0)|0;
}
function _findReusableFd($zPath,$flags) {
 $zPath = $zPath|0;
 $flags = $flags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInode = 0, $pUnused = 0, $pp = 0, $sStat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $pUnused = sp + 84|0;
 $sStat = sp + 8|0;
 $pInode = sp + 4|0;
 $pp = sp;
 HEAP32[$0>>2] = $zPath;
 HEAP32[$1>>2] = $flags;
 HEAP32[$pUnused>>2] = 0;
 $2 = HEAP32[(6488)>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (FUNCTION_TABLE_iii[$2 & 255]($3,$sStat)|0);
 $5 = (0)==($4|0);
 if (!($5)) {
  $45 = HEAP32[$pUnused>>2]|0;
  STACKTOP = sp;return ($45|0);
 }
 _unixEnterMutex();
 $6 = HEAP32[6952>>2]|0;
 HEAP32[$pInode>>2] = $6;
 while(1) {
  $7 = HEAP32[$pInode>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$pInode>>2]|0;
   $10 = HEAP32[$9>>2]|0;
   $11 = HEAP32[$sStat>>2]|0;
   $12 = ($10|0)!=($11|0);
   if ($12) {
    $46 = 1;
   } else {
    $13 = HEAP32[$pInode>>2]|0;
    $14 = ((($13)) + 4|0);
    $15 = HEAP32[$14>>2]|0;
    $16 = ((($sStat)) + 72|0);
    $17 = HEAP32[$16>>2]|0;
    $18 = ($15|0)!=($17|0);
    $46 = $18;
   }
  } else {
   $46 = 0;
  }
  $19 = HEAP32[$pInode>>2]|0;
  if (!($46)) {
   break;
  }
  $20 = ((($19)) + 32|0);
  $21 = HEAP32[$20>>2]|0;
  HEAP32[$pInode>>2] = $21;
 }
 $22 = ($19|0)!=(0|0);
 if ($22) {
  $23 = HEAP32[$pInode>>2]|0;
  $24 = ((($23)) + 28|0);
  HEAP32[$pp>>2] = $24;
  while(1) {
   $25 = HEAP32[$pp>>2]|0;
   $26 = HEAP32[$25>>2]|0;
   $27 = ($26|0)!=(0|0);
   if (!($27)) {
    break;
   }
   $28 = HEAP32[$pp>>2]|0;
   $29 = HEAP32[$28>>2]|0;
   $30 = ((($29)) + 4|0);
   $31 = HEAP32[$30>>2]|0;
   $32 = HEAP32[$1>>2]|0;
   $33 = ($31|0)!=($32|0);
   if (!($33)) {
    break;
   }
   $34 = HEAP32[$pp>>2]|0;
   $35 = HEAP32[$34>>2]|0;
   $36 = ((($35)) + 8|0);
   HEAP32[$pp>>2] = $36;
  }
  $37 = HEAP32[$pp>>2]|0;
  $38 = HEAP32[$37>>2]|0;
  HEAP32[$pUnused>>2] = $38;
  $39 = HEAP32[$pUnused>>2]|0;
  $40 = ($39|0)!=(0|0);
  if ($40) {
   $41 = HEAP32[$pUnused>>2]|0;
   $42 = ((($41)) + 8|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = HEAP32[$pp>>2]|0;
   HEAP32[$44>>2] = $43;
  }
 }
 _unixLeaveMutex();
 $45 = HEAP32[$pUnused>>2]|0;
 STACKTOP = sp;return ($45|0);
}
function _findCreateFileMode($zPath,$flags,$pMode,$pUid,$pGid) {
 $zPath = $zPath|0;
 $flags = $flags|0;
 $pMode = $pMode|0;
 $pUid = $pUid|0;
 $pGid = $pGid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nDb = 0, $rc = 0, $z = 0, $zDb = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 560|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $rc = sp + 8|0;
 $zDb = sp + 32|0;
 $nDb = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $zPath;
 HEAP32[$1>>2] = $flags;
 HEAP32[$2>>2] = $pMode;
 HEAP32[$3>>2] = $pUid;
 HEAP32[$4>>2] = $pGid;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$5>>2] = 0;
 $6 = HEAP32[$3>>2]|0;
 HEAP32[$6>>2] = 0;
 $7 = HEAP32[$4>>2]|0;
 HEAP32[$7>>2] = 0;
 $8 = HEAP32[$1>>2]|0;
 $9 = $8 & 526336;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  $12 = (_sqlite3Strlen30($11)|0);
  $13 = (($12) - 1)|0;
  HEAP32[$nDb>>2] = $13;
  while(1) {
   $14 = HEAP32[$nDb>>2]|0;
   $15 = HEAP32[$0>>2]|0;
   $16 = (($15) + ($14)|0);
   $17 = HEAP8[$16>>0]|0;
   $18 = $17 << 24 >> 24;
   $19 = ($18|0)!=(45);
   if (!($19)) {
    break;
   }
   $20 = HEAP32[$nDb>>2]|0;
   $21 = (($20) + -1)|0;
   HEAP32[$nDb>>2] = $21;
  }
  $22 = HEAP32[$0>>2]|0;
  $23 = HEAP32[$nDb>>2]|0;
  _memcpy(($zDb|0),($22|0),($23|0))|0;
  $24 = HEAP32[$nDb>>2]|0;
  $25 = (($zDb) + ($24)|0);
  HEAP8[$25>>0] = 0;
  $26 = HEAP32[$2>>2]|0;
  $27 = HEAP32[$3>>2]|0;
  $28 = HEAP32[$4>>2]|0;
  $29 = (_getFileMode($zDb,$26,$27,$28)|0);
  HEAP32[$rc>>2] = $29;
  $46 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $30 = HEAP32[$1>>2]|0;
 $31 = $30 & 8;
 $32 = ($31|0)!=(0);
 if ($32) {
  $33 = HEAP32[$2>>2]|0;
  HEAP32[$33>>2] = 384;
  $46 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $34 = HEAP32[$1>>2]|0;
 $35 = $34 & 64;
 $36 = ($35|0)!=(0);
 if (!($36)) {
  $46 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $37 = HEAP32[$0>>2]|0;
 $38 = (_sqlite3_uri_parameter($37,44606)|0);
 HEAP32[$z>>2] = $38;
 $39 = HEAP32[$z>>2]|0;
 $40 = ($39|0)!=(0|0);
 if (!($40)) {
  $46 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($46|0);
 }
 $41 = HEAP32[$z>>2]|0;
 $42 = HEAP32[$2>>2]|0;
 $43 = HEAP32[$3>>2]|0;
 $44 = HEAP32[$4>>2]|0;
 $45 = (_getFileMode($41,$42,$43,$44)|0);
 HEAP32[$rc>>2] = $45;
 $46 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($46|0);
}
function _robustFchown($fd,$uid,$gid) {
 $fd = $fd|0;
 $uid = $uid|0;
 $gid = $gid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $fd;
 HEAP32[$1>>2] = $uid;
 HEAP32[$2>>2] = $gid;
 $3 = HEAP32[(6692)>>2]|0;
 $4 = (FUNCTION_TABLE_i[$3 & 255]()|0);
 $5 = ($4|0)!=(0);
 if ($5) {
  $11 = 0;
  STACKTOP = sp;return ($11|0);
 }
 $6 = HEAP32[(6680)>>2]|0;
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP32[$2>>2]|0;
 $10 = (FUNCTION_TABLE_iiii[$6 & 255]($7,$8,$9)|0);
 $11 = $10;
 STACKTOP = sp;return ($11|0);
}
function _fillInUnixFile($pVfs,$h,$pId,$zFilename,$ctrlFlags) {
 $pVfs = $pVfs|0;
 $h = $h|0;
 $pId = $pId|0;
 $zFilename = $zFilename|0;
 $ctrlFlags = $ctrlFlags|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $nFilename = 0, $pLockingStyle = 0, $pNew = 0, $rc = 0, $vararg_buffer = 0, $zLockFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $3 = sp + 28|0;
 $4 = sp + 24|0;
 $pLockingStyle = sp + 20|0;
 $pNew = sp + 16|0;
 $rc = sp + 12|0;
 $zLockFile = sp + 8|0;
 $nFilename = sp + 4|0;
 HEAP32[$0>>2] = $pVfs;
 HEAP32[$1>>2] = $h;
 HEAP32[$2>>2] = $pId;
 HEAP32[$3>>2] = $zFilename;
 HEAP32[$4>>2] = $ctrlFlags;
 $5 = HEAP32[$2>>2]|0;
 HEAP32[$pNew>>2] = $5;
 HEAP32[$rc>>2] = 0;
 $6 = HEAP32[$1>>2]|0;
 $7 = HEAP32[$pNew>>2]|0;
 $8 = ((($7)) + 12|0);
 HEAP32[$8>>2] = $6;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$pNew>>2]|0;
 $11 = ((($10)) + 4|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$3>>2]|0;
 $13 = HEAP32[$pNew>>2]|0;
 $14 = ((($13)) + 32|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$4>>2]|0;
 $16 = $15&255;
 $17 = $16&255;
 $18 = HEAP32[$pNew>>2]|0;
 $19 = ((($18)) + 18|0);
 HEAP16[$19>>1] = $17;
 $20 = HEAP32[$4>>2]|0;
 $21 = $20 & 64;
 $22 = ($21|0)!=(0);
 $23 = HEAP32[$3>>2]|0;
 $24 = $22 ? $23 : 0;
 $25 = (_sqlite3_uri_boolean($24,44465,1)|0);
 $26 = ($25|0)!=(0);
 if ($26) {
  $27 = HEAP32[$pNew>>2]|0;
  $28 = ((($27)) + 18|0);
  $29 = HEAP16[$28>>1]|0;
  $30 = $29&65535;
  $31 = $30 | 16;
  $32 = $31&65535;
  HEAP16[$28>>1] = $32;
 }
 $33 = HEAP32[$0>>2]|0;
 $34 = ((($33)) + 16|0);
 $35 = HEAP32[$34>>2]|0;
 $36 = (_strcmp($35,44081)|0);
 $37 = ($36|0)==(0);
 if ($37) {
  $38 = HEAP32[$pNew>>2]|0;
  $39 = ((($38)) + 18|0);
  $40 = HEAP16[$39>>1]|0;
  $41 = $40&65535;
  $42 = $41 | 1;
  $43 = $42&65535;
  HEAP16[$39>>1] = $43;
 }
 $44 = HEAP32[$4>>2]|0;
 $45 = $44 & 128;
 $46 = ($45|0)!=(0);
 if ($46) {
  HEAP32[$pLockingStyle>>2] = 6796;
 } else {
  $47 = HEAP32[$0>>2]|0;
  $48 = ((($47)) + 20|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = HEAP32[$49>>2]|0;
  $51 = HEAP32[$3>>2]|0;
  $52 = HEAP32[$pNew>>2]|0;
  $53 = (FUNCTION_TABLE_iii[$50 & 255]($51,$52)|0);
  HEAP32[$pLockingStyle>>2] = $53;
 }
 $54 = HEAP32[$pLockingStyle>>2]|0;
 $55 = ($54|0)==(6876|0);
 if ($55) {
  _unixEnterMutex();
  $56 = HEAP32[$pNew>>2]|0;
  $57 = HEAP32[$pNew>>2]|0;
  $58 = ((($57)) + 8|0);
  $59 = (_findInodeInfo($56,$58)|0);
  HEAP32[$rc>>2] = $59;
  $60 = HEAP32[$rc>>2]|0;
  $61 = ($60|0)!=(0);
  if ($61) {
   $62 = HEAP32[$pNew>>2]|0;
   $63 = HEAP32[$1>>2]|0;
   _robust_close($62,$63,34692);
   HEAP32[$1>>2] = -1;
  }
  _unixLeaveMutex();
 } else {
  $64 = HEAP32[$pLockingStyle>>2]|0;
  $65 = ($64|0)==(6360|0);
  if ($65) {
   $66 = HEAP32[$3>>2]|0;
   $67 = (_strlen($66)|0);
   $68 = (($67) + 6)|0;
   HEAP32[$nFilename>>2] = $68;
   $69 = HEAP32[$nFilename>>2]|0;
   $70 = ($69|0)<(0);
   $71 = $70 << 31 >> 31;
   $72 = (_sqlite3_malloc64($69,$71)|0);
   HEAP32[$zLockFile>>2] = $72;
   $73 = HEAP32[$zLockFile>>2]|0;
   $74 = ($73|0)==(0|0);
   if ($74) {
    HEAP32[$rc>>2] = 7;
   } else {
    $75 = HEAP32[$nFilename>>2]|0;
    $76 = HEAP32[$zLockFile>>2]|0;
    $77 = HEAP32[$3>>2]|0;
    HEAP32[$vararg_buffer>>2] = $77;
    (_sqlite3_snprintf($75,$76,44470,$vararg_buffer)|0);
   }
   $78 = HEAP32[$zLockFile>>2]|0;
   $79 = HEAP32[$pNew>>2]|0;
   $80 = ((($79)) + 24|0);
   HEAP32[$80>>2] = $78;
  }
 }
 $81 = HEAP32[$pNew>>2]|0;
 _storeLastErrno($81,0);
 $82 = HEAP32[$rc>>2]|0;
 $83 = ($82|0)!=(0);
 if (!($83)) {
  $88 = HEAP32[$pLockingStyle>>2]|0;
  $89 = HEAP32[$pNew>>2]|0;
  HEAP32[$89>>2] = $88;
  $90 = HEAP32[$pNew>>2]|0;
  _verifyDbFile($90);
  $91 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $84 = HEAP32[$1>>2]|0;
 $85 = ($84|0)>=(0);
 if (!($85)) {
  $91 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($91|0);
 }
 $86 = HEAP32[$pNew>>2]|0;
 $87 = HEAP32[$1>>2]|0;
 _robust_close($86,$87,34777);
 $91 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($91|0);
}
function _findInodeInfo($pFile,$ppInode) {
 $pFile = $pFile|0;
 $ppInode = $ppInode|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $fd = 0, $fileId = 0;
 var $pInode = 0, $rc = 0, $statbuf = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $0 = sp + 104|0;
 $1 = sp + 100|0;
 $2 = sp + 96|0;
 $rc = sp + 92|0;
 $fd = sp + 88|0;
 $fileId = sp + 80|0;
 $statbuf = sp + 4|0;
 $pInode = sp;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $ppInode;
 HEAP32[$pInode>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$fd>>2] = $5;
 $6 = HEAP32[(6500)>>2]|0;
 $7 = HEAP32[$fd>>2]|0;
 $8 = (FUNCTION_TABLE_iii[$6 & 255]($7,$statbuf)|0);
 HEAP32[$rc>>2] = $8;
 $9 = HEAP32[$rc>>2]|0;
 $10 = ($9|0)!=(0);
 if ($10) {
  $11 = HEAP32[$1>>2]|0;
  $12 = (___errno_location()|0);
  $13 = HEAP32[$12>>2]|0;
  _storeLastErrno($11,$13);
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 20|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($16|0)==(75);
  if ($17) {
   HEAP32[$0>>2] = 22;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  } else {
   HEAP32[$0>>2] = 10;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 }
 ;HEAP32[$fileId>>2]=0|0;HEAP32[$fileId+4>>2]=0|0;
 $18 = HEAP32[$statbuf>>2]|0;
 HEAP32[$fileId>>2] = $18;
 $19 = ((($statbuf)) + 72|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = ((($fileId)) + 4|0);
 HEAP32[$21>>2] = $20;
 $22 = HEAP32[6952>>2]|0;
 HEAP32[$pInode>>2] = $22;
 while(1) {
  $23 = HEAP32[$pInode>>2]|0;
  $24 = ($23|0)!=(0|0);
  if ($24) {
   $25 = HEAP32[$pInode>>2]|0;
   $26 = (_memcmp($fileId,$25,8)|0);
   $27 = ($26|0)!=(0);
   $57 = $27;
  } else {
   $57 = 0;
  }
  $28 = HEAP32[$pInode>>2]|0;
  if (!($57)) {
   break;
  }
  $29 = ((($28)) + 32|0);
  $30 = HEAP32[$29>>2]|0;
  HEAP32[$pInode>>2] = $30;
 }
 $31 = ($28|0)==(0|0);
 if ($31) {
  $32 = (_sqlite3_malloc64(40,0)|0);
  HEAP32[$pInode>>2] = $32;
  $33 = HEAP32[$pInode>>2]|0;
  $34 = ($33|0)==(0|0);
  if ($34) {
   HEAP32[$0>>2] = 7;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
  $35 = HEAP32[$pInode>>2]|0;
  dest=$35; stop=dest+40|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
  $36 = HEAP32[$pInode>>2]|0;
  ;HEAP32[$36>>2]=HEAP32[$fileId>>2]|0;HEAP32[$36+4>>2]=HEAP32[$fileId+4>>2]|0;
  $37 = HEAP32[$pInode>>2]|0;
  $38 = ((($37)) + 16|0);
  HEAP32[$38>>2] = 1;
  $39 = HEAP32[6952>>2]|0;
  $40 = HEAP32[$pInode>>2]|0;
  $41 = ((($40)) + 32|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$pInode>>2]|0;
  $43 = ((($42)) + 36|0);
  HEAP32[$43>>2] = 0;
  $44 = HEAP32[6952>>2]|0;
  $45 = ($44|0)!=(0|0);
  if ($45) {
   $46 = HEAP32[$pInode>>2]|0;
   $47 = HEAP32[6952>>2]|0;
   $48 = ((($47)) + 36|0);
   HEAP32[$48>>2] = $46;
  }
  $49 = HEAP32[$pInode>>2]|0;
  HEAP32[6952>>2] = $49;
 } else {
  $50 = HEAP32[$pInode>>2]|0;
  $51 = ((($50)) + 16|0);
  $52 = HEAP32[$51>>2]|0;
  $53 = (($52) + 1)|0;
  HEAP32[$51>>2] = $53;
 }
 $54 = HEAP32[$pInode>>2]|0;
 $55 = HEAP32[$2>>2]|0;
 HEAP32[$55>>2] = $54;
 HEAP32[$0>>2] = 0;
 $56 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _verifyDbFile($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $buf = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 112|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer4 = sp + 16|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 108|0;
 $buf = sp + 32|0;
 $rc = sp + 28|0;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 18|0);
 $3 = HEAP16[$2>>1]|0;
 $4 = $3&65535;
 $5 = $4 & 128;
 $6 = ($5|0)!=(0);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[(6500)>>2]|0;
 $8 = HEAP32[$0>>2]|0;
 $9 = ((($8)) + 12|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (FUNCTION_TABLE_iii[$7 & 255]($10,$buf)|0);
 HEAP32[$rc>>2] = $11;
 $12 = HEAP32[$rc>>2]|0;
 $13 = ($12|0)!=(0);
 if ($13) {
  $14 = HEAP32[$0>>2]|0;
  $15 = ((($14)) + 32|0);
  $16 = HEAP32[$15>>2]|0;
  HEAP32[$vararg_buffer>>2] = $16;
  _sqlite3_log(28,44478,$vararg_buffer);
  STACKTOP = sp;return;
 }
 $17 = ((($buf)) + 16|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)==(0);
 if ($19) {
  $20 = HEAP32[$0>>2]|0;
  $21 = ((($20)) + 32|0);
  $22 = HEAP32[$21>>2]|0;
  HEAP32[$vararg_buffer1>>2] = $22;
  _sqlite3_log(28,44502,$vararg_buffer1);
  STACKTOP = sp;return;
 }
 $23 = ((($buf)) + 16|0);
 $24 = HEAP32[$23>>2]|0;
 $25 = ($24>>>0)>(1);
 $26 = HEAP32[$0>>2]|0;
 if ($25) {
  $27 = ((($26)) + 32|0);
  $28 = HEAP32[$27>>2]|0;
  HEAP32[$vararg_buffer4>>2] = $28;
  _sqlite3_log(28,44531,$vararg_buffer4);
  STACKTOP = sp;return;
 }
 $29 = (_fileHasMoved($26)|0);
 $30 = ($29|0)!=(0);
 if (!($30)) {
  STACKTOP = sp;return;
 }
 $31 = HEAP32[$0>>2]|0;
 $32 = ((($31)) + 32|0);
 $33 = HEAP32[$32>>2]|0;
 HEAP32[$vararg_buffer7>>2] = $33;
 _sqlite3_log(28,44558,$vararg_buffer7);
 STACKTOP = sp;return;
}
function _unixClose($id) {
 $id = $id|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pFile = 0;
 var $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $pFile = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$rc>>2] = 0;
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $1;
 $2 = HEAP32[$pFile>>2]|0;
 _verifyDbFile($2);
 $3 = HEAP32[$0>>2]|0;
 (_unixUnlock($3,0)|0);
 _unixEnterMutex();
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$pFile>>2]|0;
  $9 = ((($8)) + 8|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($10)) + 24|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($12|0)!=(0);
  if ($13) {
   $14 = HEAP32[$pFile>>2]|0;
   _setPendingFd($14);
  }
 }
 $15 = HEAP32[$pFile>>2]|0;
 _releaseInodeInfo($15);
 $16 = HEAP32[$0>>2]|0;
 $17 = (_closeUnixFile($16)|0);
 HEAP32[$rc>>2] = $17;
 _unixLeaveMutex();
 $18 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($18|0);
}
function _unixLock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0;
 var $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0;
 var $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0;
 var $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0;
 var $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $lock = 0;
 var $or$cond = 0, $or$cond3 = 0, $pFile = 0, $pInode = 0, $rc = 0, $tErrno = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $rc = sp + 32|0;
 $pFile = sp + 28|0;
 $pInode = sp + 24|0;
 $lock = sp + 8|0;
 $tErrno = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 HEAP32[$rc>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $3;
 HEAP32[$tErrno>>2] = 0;
 $4 = HEAP32[$pFile>>2]|0;
 $5 = ((($4)) + 16|0);
 $6 = HEAP8[$5>>0]|0;
 $7 = $6&255;
 $8 = HEAP32[$2>>2]|0;
 $9 = ($7|0)>=($8|0);
 if ($9) {
  HEAP32[$0>>2] = 0;
  $158 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($158|0);
 }
 _unixEnterMutex();
 $10 = HEAP32[$pFile>>2]|0;
 $11 = ((($10)) + 8|0);
 $12 = HEAP32[$11>>2]|0;
 HEAP32[$pInode>>2] = $12;
 $13 = HEAP32[$pFile>>2]|0;
 $14 = ((($13)) + 16|0);
 $15 = HEAP8[$14>>0]|0;
 $16 = $15&255;
 $17 = HEAP32[$pInode>>2]|0;
 $18 = ((($17)) + 12|0);
 $19 = HEAP8[$18>>0]|0;
 $20 = $19&255;
 $21 = ($16|0)!=($20|0);
 if ($21) {
  $22 = HEAP32[$pInode>>2]|0;
  $23 = ((($22)) + 12|0);
  $24 = HEAP8[$23>>0]|0;
  $25 = $24&255;
  $26 = ($25|0)>=(3);
  $27 = HEAP32[$2>>2]|0;
  $28 = ($27|0)>(1);
  $or$cond = $26 | $28;
  if ($or$cond) {
   HEAP32[$rc>>2] = 5;
  } else {
   label = 6;
  }
 } else {
  label = 6;
 }
 L8: do {
  if ((label|0) == 6) {
   $29 = HEAP32[$2>>2]|0;
   $30 = ($29|0)==(1);
   do {
    if ($30) {
     $31 = HEAP32[$pInode>>2]|0;
     $32 = ((($31)) + 12|0);
     $33 = HEAP8[$32>>0]|0;
     $34 = $33&255;
     $35 = ($34|0)==(1);
     if (!($35)) {
      $36 = HEAP32[$pInode>>2]|0;
      $37 = ((($36)) + 12|0);
      $38 = HEAP8[$37>>0]|0;
      $39 = $38&255;
      $40 = ($39|0)==(2);
      if (!($40)) {
       break;
      }
     }
     $41 = HEAP32[$pFile>>2]|0;
     $42 = ((($41)) + 16|0);
     HEAP8[$42>>0] = 1;
     $43 = HEAP32[$pInode>>2]|0;
     $44 = ((($43)) + 8|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = (($45) + 1)|0;
     HEAP32[$44>>2] = $46;
     $47 = HEAP32[$pInode>>2]|0;
     $48 = ((($47)) + 24|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = (($49) + 1)|0;
     HEAP32[$48>>2] = $50;
     break L8;
    }
   } while(0);
   $51 = ((($lock)) + 8|0);
   HEAP32[$51>>2] = 1;
   $52 = ((($lock)) + 2|0);
   HEAP16[$52>>1] = 0;
   $53 = HEAP32[$2>>2]|0;
   $54 = ($53|0)==(1);
   if ($54) {
    label = 13;
   } else {
    $55 = HEAP32[$2>>2]|0;
    $56 = ($55|0)==(4);
    if ($56) {
     $57 = HEAP32[$pFile>>2]|0;
     $58 = ((($57)) + 16|0);
     $59 = HEAP8[$58>>0]|0;
     $60 = $59&255;
     $61 = ($60|0)<(3);
     if ($61) {
      label = 13;
     }
    }
   }
   if ((label|0) == 13) {
    $62 = HEAP32[$2>>2]|0;
    $63 = ($62|0)==(1);
    $64 = $63 ? 0 : 1;
    $65 = $64&65535;
    HEAP16[$lock>>1] = $65;
    $66 = HEAP32[1536>>2]|0;
    $67 = ((($lock)) + 4|0);
    HEAP32[$67>>2] = $66;
    $68 = HEAP32[$pFile>>2]|0;
    $69 = (_unixFileLock($68,$lock)|0);
    $70 = ($69|0)!=(0);
    if ($70) {
     $71 = (___errno_location()|0);
     $72 = HEAP32[$71>>2]|0;
     HEAP32[$tErrno>>2] = $72;
     $73 = HEAP32[$tErrno>>2]|0;
     $74 = (_sqliteErrorFromPosixError($73,3850)|0);
     HEAP32[$rc>>2] = $74;
     $75 = HEAP32[$rc>>2]|0;
     $76 = ($75|0)!=(5);
     if (!($76)) {
      break;
     }
     $77 = HEAP32[$pFile>>2]|0;
     $78 = HEAP32[$tErrno>>2]|0;
     _storeLastErrno($77,$78);
     break;
    }
   }
   $79 = HEAP32[$2>>2]|0;
   $80 = ($79|0)==(1);
   do {
    if ($80) {
     $81 = HEAP32[1536>>2]|0;
     $82 = (($81) + 2)|0;
     $83 = ((($lock)) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = ((($lock)) + 8|0);
     HEAP32[$84>>2] = 510;
     $85 = HEAP32[$pFile>>2]|0;
     $86 = (_unixFileLock($85,$lock)|0);
     $87 = ($86|0)!=(0);
     if ($87) {
      $88 = (___errno_location()|0);
      $89 = HEAP32[$88>>2]|0;
      HEAP32[$tErrno>>2] = $89;
      $90 = HEAP32[$tErrno>>2]|0;
      $91 = (_sqliteErrorFromPosixError($90,3850)|0);
      HEAP32[$rc>>2] = $91;
     }
     $92 = HEAP32[1536>>2]|0;
     $93 = ((($lock)) + 4|0);
     HEAP32[$93>>2] = $92;
     $94 = ((($lock)) + 8|0);
     HEAP32[$94>>2] = 1;
     HEAP16[$lock>>1] = 2;
     $95 = HEAP32[$pFile>>2]|0;
     $96 = (_unixFileLock($95,$lock)|0);
     $97 = ($96|0)!=(0);
     $98 = HEAP32[$rc>>2]|0;
     $99 = ($98|0)==(0);
     $or$cond3 = $97 & $99;
     if ($or$cond3) {
      $100 = (___errno_location()|0);
      $101 = HEAP32[$100>>2]|0;
      HEAP32[$tErrno>>2] = $101;
      HEAP32[$rc>>2] = 2058;
     }
     $102 = HEAP32[$rc>>2]|0;
     $103 = ($102|0)!=(0);
     if (!($103)) {
      $108 = HEAP32[$pFile>>2]|0;
      $109 = ((($108)) + 16|0);
      HEAP8[$109>>0] = 1;
      $110 = HEAP32[$pInode>>2]|0;
      $111 = ((($110)) + 24|0);
      $112 = HEAP32[$111>>2]|0;
      $113 = (($112) + 1)|0;
      HEAP32[$111>>2] = $113;
      $114 = HEAP32[$pInode>>2]|0;
      $115 = ((($114)) + 8|0);
      HEAP32[$115>>2] = 1;
      break;
     }
     $104 = HEAP32[$rc>>2]|0;
     $105 = ($104|0)!=(5);
     if (!($105)) {
      break L8;
     }
     $106 = HEAP32[$pFile>>2]|0;
     $107 = HEAP32[$tErrno>>2]|0;
     _storeLastErrno($106,$107);
     break L8;
    } else {
     $116 = HEAP32[$2>>2]|0;
     $117 = ($116|0)==(4);
     if ($117) {
      $118 = HEAP32[$pInode>>2]|0;
      $119 = ((($118)) + 8|0);
      $120 = HEAP32[$119>>2]|0;
      $121 = ($120|0)>(1);
      if ($121) {
       HEAP32[$rc>>2] = 5;
       break;
      }
     }
     HEAP16[$lock>>1] = 1;
     $122 = HEAP32[$2>>2]|0;
     $123 = ($122|0)==(2);
     $124 = HEAP32[1536>>2]|0;
     if ($123) {
      $125 = (($124) + 1)|0;
      $126 = ((($lock)) + 4|0);
      HEAP32[$126>>2] = $125;
      $127 = ((($lock)) + 8|0);
      HEAP32[$127>>2] = 1;
     } else {
      $128 = (($124) + 2)|0;
      $129 = ((($lock)) + 4|0);
      HEAP32[$129>>2] = $128;
      $130 = ((($lock)) + 8|0);
      HEAP32[$130>>2] = 510;
     }
     $131 = HEAP32[$pFile>>2]|0;
     $132 = (_unixFileLock($131,$lock)|0);
     $133 = ($132|0)!=(0);
     if ($133) {
      $134 = (___errno_location()|0);
      $135 = HEAP32[$134>>2]|0;
      HEAP32[$tErrno>>2] = $135;
      $136 = HEAP32[$tErrno>>2]|0;
      $137 = (_sqliteErrorFromPosixError($136,3850)|0);
      HEAP32[$rc>>2] = $137;
      $138 = HEAP32[$rc>>2]|0;
      $139 = ($138|0)!=(5);
      if ($139) {
       $140 = HEAP32[$pFile>>2]|0;
       $141 = HEAP32[$tErrno>>2]|0;
       _storeLastErrno($140,$141);
      }
     }
    }
   } while(0);
   $142 = HEAP32[$rc>>2]|0;
   $143 = ($142|0)==(0);
   $144 = HEAP32[$2>>2]|0;
   if ($143) {
    $145 = $144&255;
    $146 = HEAP32[$pFile>>2]|0;
    $147 = ((($146)) + 16|0);
    HEAP8[$147>>0] = $145;
    $148 = HEAP32[$2>>2]|0;
    $149 = $148&255;
    $150 = HEAP32[$pInode>>2]|0;
    $151 = ((($150)) + 12|0);
    HEAP8[$151>>0] = $149;
    break;
   }
   $152 = ($144|0)==(4);
   if ($152) {
    $153 = HEAP32[$pFile>>2]|0;
    $154 = ((($153)) + 16|0);
    HEAP8[$154>>0] = 3;
    $155 = HEAP32[$pInode>>2]|0;
    $156 = ((($155)) + 12|0);
    HEAP8[$156>>0] = 3;
   }
  }
 } while(0);
 _unixLeaveMutex();
 $157 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $157;
 $158 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($158|0);
}
function _unixUnlock($id,$eFileLock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $eFileLock;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_posixUnlock($2,$3,0)|0);
 STACKTOP = sp;return ($4|0);
}
function _unixCheckReservedLock($id,$pResOut) {
 $id = $id|0;
 $pResOut = $pResOut|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lock = 0, $pFile = 0;
 var $rc = 0, $reserved = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $rc = sp + 32|0;
 $reserved = sp + 28|0;
 $pFile = sp + 24|0;
 $lock = sp + 8|0;
 HEAP32[$0>>2] = $id;
 HEAP32[$1>>2] = $pResOut;
 HEAP32[$rc>>2] = 0;
 HEAP32[$reserved>>2] = 0;
 $2 = HEAP32[$0>>2]|0;
 HEAP32[$pFile>>2] = $2;
 _unixEnterMutex();
 $3 = HEAP32[$pFile>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)>(1);
 if ($9) {
  HEAP32[$reserved>>2] = 1;
 }
 $10 = HEAP32[$reserved>>2]|0;
 $11 = ($10|0)!=(0);
 do {
  if (!($11)) {
   $12 = HEAP32[$pFile>>2]|0;
   $13 = ((($12)) + 8|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ((($14)) + 13|0);
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)!=(0);
   if (!($17)) {
    $18 = ((($lock)) + 2|0);
    HEAP16[$18>>1] = 0;
    $19 = HEAP32[1536>>2]|0;
    $20 = (($19) + 1)|0;
    $21 = ((($lock)) + 4|0);
    HEAP32[$21>>2] = $20;
    $22 = ((($lock)) + 8|0);
    HEAP32[$22>>2] = 1;
    HEAP16[$lock>>1] = 1;
    $23 = HEAP32[(6524)>>2]|0;
    $24 = HEAP32[$pFile>>2]|0;
    $25 = ((($24)) + 12|0);
    $26 = HEAP32[$25>>2]|0;
    HEAP32[$vararg_buffer>>2] = $lock;
    $27 = (FUNCTION_TABLE_iiii[$23 & 255]($26,12,$vararg_buffer)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$rc>>2] = 3594;
     $29 = HEAP32[$pFile>>2]|0;
     $30 = (___errno_location()|0);
     $31 = HEAP32[$30>>2]|0;
     _storeLastErrno($29,$31);
     break;
    }
    $32 = HEAP16[$lock>>1]|0;
    $33 = $32 << 16 >> 16;
    $34 = ($33|0)!=(2);
    if ($34) {
     HEAP32[$reserved>>2] = 1;
    }
   }
  }
 } while(0);
 _unixLeaveMutex();
 $35 = HEAP32[$reserved>>2]|0;
 $36 = HEAP32[$1>>2]|0;
 HEAP32[$36>>2] = $35;
 $37 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _unixShmMap($fd,$iRegion,$szRegion,$bExtend,$pp) {
 $fd = $fd|0;
 $iRegion = $iRegion|0;
 $szRegion = $szRegion|0;
 $bExtend = $bExtend|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $apNew = 0, $i = 0, $iPg = 0, $nByte = 0, $nMap = 0, $nReqRegion = 0, $nShmPerMap = 0, $or$cond = 0, $p = 0;
 var $pDbFd = 0, $pMem = 0, $pShmNode = 0, $rc = 0, $sStat = 0, $x = 0, $zFile = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 152|0;
 $1 = sp + 148|0;
 $2 = sp + 144|0;
 $3 = sp + 140|0;
 $4 = sp + 136|0;
 $5 = sp + 132|0;
 $pDbFd = sp + 128|0;
 $p = sp + 124|0;
 $pShmNode = sp + 120|0;
 $rc = sp + 116|0;
 $nShmPerMap = sp + 112|0;
 $nReqRegion = sp + 108|0;
 $apNew = sp + 104|0;
 $nByte = sp + 100|0;
 $sStat = sp + 24|0;
 $iPg = sp + 20|0;
 $x = sp + 16|0;
 $zFile = sp + 12|0;
 $nMap = sp + 8|0;
 $i = sp + 4|0;
 $pMem = sp;
 HEAP32[$1>>2] = $fd;
 HEAP32[$2>>2] = $iRegion;
 HEAP32[$3>>2] = $szRegion;
 HEAP32[$4>>2] = $bExtend;
 HEAP32[$5>>2] = $pp;
 $6 = HEAP32[$1>>2]|0;
 HEAP32[$pDbFd>>2] = $6;
 HEAP32[$rc>>2] = 0;
 $7 = (_unixShmRegionPerMap()|0);
 HEAP32[$nShmPerMap>>2] = $7;
 $8 = HEAP32[$pDbFd>>2]|0;
 $9 = ((($8)) + 36|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  $12 = HEAP32[$pDbFd>>2]|0;
  $13 = (_unixOpenSharedMemory($12)|0);
  HEAP32[$rc>>2] = $13;
  $14 = HEAP32[$rc>>2]|0;
  $15 = ($14|0)!=(0);
  if ($15) {
   $16 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $16;
   $190 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($190|0);
  }
 }
 $17 = HEAP32[$pDbFd>>2]|0;
 $18 = ((($17)) + 36|0);
 $19 = HEAP32[$18>>2]|0;
 HEAP32[$p>>2] = $19;
 $20 = HEAP32[$p>>2]|0;
 $21 = HEAP32[$20>>2]|0;
 HEAP32[$pShmNode>>2] = $21;
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$nShmPerMap>>2]|0;
 $24 = (($22) + ($23))|0;
 $25 = HEAP32[$nShmPerMap>>2]|0;
 $26 = (($24|0) / ($25|0))&-1;
 $27 = HEAP32[$nShmPerMap>>2]|0;
 $28 = Math_imul($26, $27)|0;
 HEAP32[$nReqRegion>>2] = $28;
 $29 = HEAP32[$pShmNode>>2]|0;
 $30 = ((($29)) + 20|0);
 $31 = HEAP16[$30>>1]|0;
 $32 = $31&65535;
 $33 = HEAP32[$nReqRegion>>2]|0;
 $34 = ($32|0)<($33|0);
 L6: do {
  if ($34) {
   $35 = HEAP32[$nReqRegion>>2]|0;
   $36 = HEAP32[$3>>2]|0;
   $37 = Math_imul($35, $36)|0;
   HEAP32[$nByte>>2] = $37;
   $38 = HEAP32[$3>>2]|0;
   $39 = HEAP32[$pShmNode>>2]|0;
   $40 = ((($39)) + 16|0);
   HEAP32[$40>>2] = $38;
   $41 = HEAP32[$pShmNode>>2]|0;
   $42 = ((($41)) + 12|0);
   $43 = HEAP32[$42>>2]|0;
   $44 = ($43|0)>=(0);
   L8: do {
    if ($44) {
     $45 = HEAP32[(6500)>>2]|0;
     $46 = HEAP32[$pShmNode>>2]|0;
     $47 = ((($46)) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = (FUNCTION_TABLE_iii[$45 & 255]($48,$sStat)|0);
     $50 = ($49|0)!=(0);
     if ($50) {
      HEAP32[$rc>>2] = 4874;
      break L6;
     }
     $51 = ((($sStat)) + 36|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = HEAP32[$nByte>>2]|0;
     $54 = ($52|0)<($53|0);
     if ($54) {
      $55 = HEAP32[$4>>2]|0;
      $56 = ($55|0)!=(0);
      if (!($56)) {
       break L6;
      }
      $57 = ((($sStat)) + 36|0);
      $58 = HEAP32[$57>>2]|0;
      $59 = (($58|0) / 4096)&-1;
      HEAP32[$iPg>>2] = $59;
      while(1) {
       $60 = HEAP32[$iPg>>2]|0;
       $61 = HEAP32[$nByte>>2]|0;
       $62 = (($61|0) / 4096)&-1;
       $63 = ($60|0)<($62|0);
       if (!($63)) {
        break L8;
       }
       HEAP32[$x>>2] = 0;
       $64 = HEAP32[$pShmNode>>2]|0;
       $65 = ((($64)) + 12|0);
       $66 = HEAP32[$65>>2]|0;
       $67 = HEAP32[$iPg>>2]|0;
       $68 = $67<<12;
       $69 = (($68) + 4096)|0;
       $70 = (($69) - 1)|0;
       $71 = ($70|0)<(0);
       $72 = $71 << 31 >> 31;
       $73 = (_seekAndWriteFd($66,$70,$72,18402,1,$x)|0);
       $74 = ($73|0)!=(1);
       if ($74) {
        break;
       }
       $80 = HEAP32[$iPg>>2]|0;
       $81 = (($80) + 1)|0;
       HEAP32[$iPg>>2] = $81;
      }
      $75 = HEAP32[$pShmNode>>2]|0;
      $76 = ((($75)) + 8|0);
      $77 = HEAP32[$76>>2]|0;
      HEAP32[$zFile>>2] = $77;
      $78 = HEAP32[$zFile>>2]|0;
      $79 = (_unixLogErrorAtLine(4874,44186,$78,33839)|0);
      HEAP32[$rc>>2] = $79;
      break L6;
     }
    }
   } while(0);
   $82 = HEAP32[$pShmNode>>2]|0;
   $83 = ((($82)) + 24|0);
   $84 = HEAP32[$83>>2]|0;
   $85 = HEAP32[$nReqRegion>>2]|0;
   $86 = $85<<2;
   $87 = (_sqlite3_realloc($84,$86)|0);
   HEAP32[$apNew>>2] = $87;
   $88 = HEAP32[$apNew>>2]|0;
   $89 = ($88|0)!=(0|0);
   if (!($89)) {
    HEAP32[$rc>>2] = 3082;
    break;
   }
   $90 = HEAP32[$apNew>>2]|0;
   $91 = HEAP32[$pShmNode>>2]|0;
   $92 = ((($91)) + 24|0);
   HEAP32[$92>>2] = $90;
   while(1) {
    $93 = HEAP32[$pShmNode>>2]|0;
    $94 = ((($93)) + 20|0);
    $95 = HEAP16[$94>>1]|0;
    $96 = $95&65535;
    $97 = HEAP32[$nReqRegion>>2]|0;
    $98 = ($96|0)<($97|0);
    if (!($98)) {
     break L6;
    }
    $99 = HEAP32[$3>>2]|0;
    $100 = HEAP32[$nShmPerMap>>2]|0;
    $101 = Math_imul($99, $100)|0;
    HEAP32[$nMap>>2] = $101;
    $102 = HEAP32[$pShmNode>>2]|0;
    $103 = ((($102)) + 12|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = ($104|0)>=(0);
    if ($105) {
     $106 = HEAP32[(6704)>>2]|0;
     $107 = HEAP32[$nMap>>2]|0;
     $108 = HEAP32[$pShmNode>>2]|0;
     $109 = ((($108)) + 22|0);
     $110 = HEAP8[$109>>0]|0;
     $111 = $110&255;
     $112 = ($111|0)!=(0);
     $113 = $112 ? 1 : 3;
     $114 = HEAP32[$pShmNode>>2]|0;
     $115 = ((($114)) + 12|0);
     $116 = HEAP32[$115>>2]|0;
     $117 = HEAP32[$3>>2]|0;
     $118 = ($117|0)<(0);
     $119 = $118 << 31 >> 31;
     $120 = HEAP32[$pShmNode>>2]|0;
     $121 = ((($120)) + 20|0);
     $122 = HEAP16[$121>>1]|0;
     $123 = $122&65535;
     $124 = (___muldi3(($117|0),($119|0),($123|0),0)|0);
     $125 = tempRet0;
     $126 = (FUNCTION_TABLE_iiiiiii[$106 & 255](0,$107,$113,1,$116,$124)|0);
     HEAP32[$pMem>>2] = $126;
     $127 = HEAP32[$pMem>>2]|0;
     $128 = ($127|0)==((-1)|0);
     if ($128) {
      label = 21;
      break;
     }
    } else {
     $133 = HEAP32[$3>>2]|0;
     $134 = ($133|0)<(0);
     $135 = $134 << 31 >> 31;
     $136 = (_sqlite3_malloc64($133,$135)|0);
     HEAP32[$pMem>>2] = $136;
     $137 = HEAP32[$pMem>>2]|0;
     $138 = ($137|0)==(0|0);
     if ($138) {
      label = 23;
      break;
     }
     $139 = HEAP32[$pMem>>2]|0;
     $140 = HEAP32[$3>>2]|0;
     _memset(($139|0),0,($140|0))|0;
    }
    HEAP32[$i>>2] = 0;
    while(1) {
     $141 = HEAP32[$i>>2]|0;
     $142 = HEAP32[$nShmPerMap>>2]|0;
     $143 = ($141|0)<($142|0);
     if (!($143)) {
      break;
     }
     $144 = HEAP32[$3>>2]|0;
     $145 = HEAP32[$i>>2]|0;
     $146 = Math_imul($144, $145)|0;
     $147 = HEAP32[$pMem>>2]|0;
     $148 = (($147) + ($146)|0);
     $149 = HEAP32[$pShmNode>>2]|0;
     $150 = ((($149)) + 20|0);
     $151 = HEAP16[$150>>1]|0;
     $152 = $151&65535;
     $153 = HEAP32[$i>>2]|0;
     $154 = (($152) + ($153))|0;
     $155 = HEAP32[$pShmNode>>2]|0;
     $156 = ((($155)) + 24|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = (($157) + ($154<<2)|0);
     HEAP32[$158>>2] = $148;
     $159 = HEAP32[$i>>2]|0;
     $160 = (($159) + 1)|0;
     HEAP32[$i>>2] = $160;
    }
    $161 = HEAP32[$nShmPerMap>>2]|0;
    $162 = HEAP32[$pShmNode>>2]|0;
    $163 = ((($162)) + 20|0);
    $164 = HEAP16[$163>>1]|0;
    $165 = $164&65535;
    $166 = (($165) + ($161))|0;
    $167 = $166&65535;
    HEAP16[$163>>1] = $167;
   }
   if ((label|0) == 21) {
    $129 = HEAP32[$pShmNode>>2]|0;
    $130 = ((($129)) + 8|0);
    $131 = HEAP32[$130>>2]|0;
    $132 = (_unixLogErrorAtLine(5386,44273,$131,33866)|0);
    HEAP32[$rc>>2] = $132;
    break;
   }
   else if ((label|0) == 23) {
    HEAP32[$rc>>2] = 7;
    break;
   }
  }
 } while(0);
 $168 = HEAP32[$pShmNode>>2]|0;
 $169 = ((($168)) + 20|0);
 $170 = HEAP16[$169>>1]|0;
 $171 = $170&65535;
 $172 = HEAP32[$2>>2]|0;
 $173 = ($171|0)>($172|0);
 if ($173) {
  $174 = HEAP32[$2>>2]|0;
  $175 = HEAP32[$pShmNode>>2]|0;
  $176 = ((($175)) + 24|0);
  $177 = HEAP32[$176>>2]|0;
  $178 = (($177) + ($174<<2)|0);
  $179 = HEAP32[$178>>2]|0;
  $180 = HEAP32[$5>>2]|0;
  HEAP32[$180>>2] = $179;
 } else {
  $181 = HEAP32[$5>>2]|0;
  HEAP32[$181>>2] = 0;
 }
 $182 = HEAP32[$pShmNode>>2]|0;
 $183 = ((($182)) + 22|0);
 $184 = HEAP8[$183>>0]|0;
 $185 = $184&255;
 $186 = ($185|0)!=(0);
 $187 = HEAP32[$rc>>2]|0;
 $188 = ($187|0)==(0);
 $or$cond = $186 & $188;
 if ($or$cond) {
  HEAP32[$rc>>2] = 8;
 }
 $189 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $189;
 $190 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($190|0);
}
function _unixOpenSharedMemory($pDbFd) {
 $pDbFd = $pDbFd|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $14 = 0, $15 = 0, $16 = 0;
 var $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $nShmFilename = 0, $openFlags = 0, $p = 0, $pInode = 0, $pShmNode = 0, $rc = 0, $sStat = 0, $vararg_buffer = 0, $zBasePath = 0;
 var $zShmFilename = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 128|0;
 $vararg_buffer = sp;
 $0 = sp + 116|0;
 $1 = sp + 112|0;
 $p = sp + 108|0;
 $pShmNode = sp + 104|0;
 $rc = sp + 100|0;
 $pInode = sp + 96|0;
 $zShmFilename = sp + 92|0;
 $nShmFilename = sp + 88|0;
 $sStat = sp + 12|0;
 $zBasePath = sp + 8|0;
 $openFlags = sp + 4|0;
 HEAP32[$1>>2] = $pDbFd;
 HEAP32[$p>>2] = 0;
 $2 = (_sqlite3_malloc64(16,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)==(0|0);
 if ($4) {
  HEAP32[$0>>2] = 7;
  $130 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($130|0);
 }
 $5 = HEAP32[$p>>2]|0;
 ;HEAP32[$5>>2]=0|0;HEAP32[$5+4>>2]=0|0;HEAP32[$5+8>>2]=0|0;HEAP32[$5+12>>2]=0|0;
 _unixEnterMutex();
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 8|0);
 $8 = HEAP32[$7>>2]|0;
 HEAP32[$pInode>>2] = $8;
 $9 = HEAP32[$pInode>>2]|0;
 $10 = ((($9)) + 20|0);
 $11 = HEAP32[$10>>2]|0;
 HEAP32[$pShmNode>>2] = $11;
 $12 = HEAP32[$pShmNode>>2]|0;
 $13 = ($12|0)==(0|0);
 L5: do {
  if ($13) {
   $14 = HEAP32[$1>>2]|0;
   $15 = ((($14)) + 32|0);
   $16 = HEAP32[$15>>2]|0;
   HEAP32[$zBasePath>>2] = $16;
   $17 = HEAP32[(6500)>>2]|0;
   $18 = HEAP32[$1>>2]|0;
   $19 = ((($18)) + 12|0);
   $20 = HEAP32[$19>>2]|0;
   $21 = (FUNCTION_TABLE_iii[$17 & 255]($20,$sStat)|0);
   $22 = ($21|0)!=(0);
   do {
    if ($22) {
     HEAP32[$rc>>2] = 1802;
    } else {
     $23 = HEAP32[$zBasePath>>2]|0;
     $24 = (_strlen($23)|0);
     $25 = (6 + ($24))|0;
     HEAP32[$nShmFilename>>2] = $25;
     $26 = HEAP32[$nShmFilename>>2]|0;
     $27 = (36 + ($26))|0;
     $28 = (_sqlite3_malloc64($27,0)|0);
     HEAP32[$pShmNode>>2] = $28;
     $29 = HEAP32[$pShmNode>>2]|0;
     $30 = ($29|0)==(0|0);
     if ($30) {
      HEAP32[$rc>>2] = 7;
      break;
     }
     $31 = HEAP32[$pShmNode>>2]|0;
     $32 = HEAP32[$nShmFilename>>2]|0;
     $33 = (36 + ($32))|0;
     _memset(($31|0),0,($33|0))|0;
     $34 = HEAP32[$pShmNode>>2]|0;
     $35 = ((($34)) + 36|0);
     $36 = HEAP32[$pShmNode>>2]|0;
     $37 = ((($36)) + 8|0);
     HEAP32[$37>>2] = $35;
     HEAP32[$zShmFilename>>2] = $35;
     $38 = HEAP32[$nShmFilename>>2]|0;
     $39 = HEAP32[$zShmFilename>>2]|0;
     $40 = HEAP32[$zBasePath>>2]|0;
     HEAP32[$vararg_buffer>>2] = $40;
     (_sqlite3_snprintf($38,$39,44586,$vararg_buffer)|0);
     $41 = HEAP32[$pShmNode>>2]|0;
     $42 = ((($41)) + 12|0);
     HEAP32[$42>>2] = -1;
     $43 = HEAP32[$pShmNode>>2]|0;
     $44 = HEAP32[$1>>2]|0;
     $45 = ((($44)) + 8|0);
     $46 = HEAP32[$45>>2]|0;
     $47 = ((($46)) + 20|0);
     HEAP32[$47>>2] = $43;
     $48 = HEAP32[$1>>2]|0;
     $49 = ((($48)) + 8|0);
     $50 = HEAP32[$49>>2]|0;
     $51 = HEAP32[$pShmNode>>2]|0;
     HEAP32[$51>>2] = $50;
     $52 = HEAP32[(12)>>2]|0;
     $53 = ($52|0)!=(0);
     if ($53) {
      $54 = HEAP32[$pShmNode>>2]|0;
      $55 = ((($54)) + 4|0);
      HEAP32[$55>>2] = (8);
      $56 = HEAP32[$pShmNode>>2]|0;
      $57 = ((($56)) + 4|0);
      $58 = HEAP32[$57>>2]|0;
      $59 = ($58|0)==(0|0);
      if ($59) {
       HEAP32[$rc>>2] = 7;
       break;
      }
     }
     $60 = HEAP32[$pInode>>2]|0;
     $61 = ((($60)) + 13|0);
     $62 = HEAP8[$61>>0]|0;
     $63 = $62&255;
     $64 = ($63|0)==(0);
     if (!($64)) {
      break L5;
     }
     HEAP32[$openFlags>>2] = 66;
     $65 = HEAP32[$1>>2]|0;
     $66 = ((($65)) + 32|0);
     $67 = HEAP32[$66>>2]|0;
     $68 = (_sqlite3_uri_boolean($67,44593,0)|0);
     $69 = ($68|0)!=(0);
     if ($69) {
      HEAP32[$openFlags>>2] = 0;
      $70 = HEAP32[$pShmNode>>2]|0;
      $71 = ((($70)) + 22|0);
      HEAP8[$71>>0] = 1;
     }
     $72 = HEAP32[$zShmFilename>>2]|0;
     $73 = HEAP32[$openFlags>>2]|0;
     $74 = ((($sStat)) + 12|0);
     $75 = HEAP32[$74>>2]|0;
     $76 = $75 & 511;
     $77 = (_robust_open($72,$73,$76)|0);
     $78 = HEAP32[$pShmNode>>2]|0;
     $79 = ((($78)) + 12|0);
     HEAP32[$79>>2] = $77;
     $80 = HEAP32[$pShmNode>>2]|0;
     $81 = ((($80)) + 12|0);
     $82 = HEAP32[$81>>2]|0;
     $83 = ($82|0)<(0);
     if ($83) {
      $84 = (_sqlite3CantopenError(33691)|0);
      $85 = HEAP32[$zShmFilename>>2]|0;
      $86 = (_unixLogErrorAtLine($84,44128,$85,33691)|0);
      HEAP32[$rc>>2] = $86;
      break;
     }
     $87 = HEAP32[$pShmNode>>2]|0;
     $88 = ((($87)) + 12|0);
     $89 = HEAP32[$88>>2]|0;
     $90 = ((($sStat)) + 20|0);
     $91 = HEAP32[$90>>2]|0;
     $92 = ((($sStat)) + 24|0);
     $93 = HEAP32[$92>>2]|0;
     (_robustFchown($89,$91,$93)|0);
     HEAP32[$rc>>2] = 0;
     $94 = HEAP32[$1>>2]|0;
     $95 = (_unixShmSystemLock($94,1,128,1)|0);
     $96 = ($95|0)==(0);
     if ($96) {
      $97 = HEAP32[$pShmNode>>2]|0;
      $98 = ((($97)) + 12|0);
      $99 = HEAP32[$98>>2]|0;
      $100 = (_robust_ftruncate($99,0,0)|0);
      $101 = ($100|0)!=(0);
      if ($101) {
       $102 = HEAP32[$zShmFilename>>2]|0;
       $103 = (_unixLogErrorAtLine(4618,44151,$102,33707)|0);
       HEAP32[$rc>>2] = $103;
      }
     }
     $104 = HEAP32[$rc>>2]|0;
     $105 = ($104|0)==(0);
     if ($105) {
      $106 = HEAP32[$1>>2]|0;
      $107 = (_unixShmSystemLock($106,0,128,1)|0);
      HEAP32[$rc>>2] = $107;
     }
     $108 = HEAP32[$rc>>2]|0;
     $109 = ($108|0)!=(0);
     if (!($109)) {
      break L5;
     }
    }
   } while(0);
   $127 = HEAP32[$1>>2]|0;
   _unixShmPurge($127);
   $128 = HEAP32[$p>>2]|0;
   _sqlite3_free($128);
   _unixLeaveMutex();
   $129 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $129;
   $130 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($130|0);
  }
 } while(0);
 $110 = HEAP32[$pShmNode>>2]|0;
 $111 = HEAP32[$p>>2]|0;
 HEAP32[$111>>2] = $110;
 $112 = HEAP32[$pShmNode>>2]|0;
 $113 = ((($112)) + 28|0);
 $114 = HEAP32[$113>>2]|0;
 $115 = (($114) + 1)|0;
 HEAP32[$113>>2] = $115;
 $116 = HEAP32[$p>>2]|0;
 $117 = HEAP32[$1>>2]|0;
 $118 = ((($117)) + 36|0);
 HEAP32[$118>>2] = $116;
 _unixLeaveMutex();
 $119 = HEAP32[$pShmNode>>2]|0;
 $120 = ((($119)) + 32|0);
 $121 = HEAP32[$120>>2]|0;
 $122 = HEAP32[$p>>2]|0;
 $123 = ((($122)) + 4|0);
 HEAP32[$123>>2] = $121;
 $124 = HEAP32[$p>>2]|0;
 $125 = HEAP32[$pShmNode>>2]|0;
 $126 = ((($125)) + 32|0);
 HEAP32[$126>>2] = $124;
 HEAP32[$0>>2] = 0;
 $130 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($130|0);
}
function _posixUnlock($id,$eFileLock,$handleNFSUnlock) {
 $id = $id|0;
 $eFileLock = $eFileLock|0;
 $handleNFSUnlock = $handleNFSUnlock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $lock = 0, $pFile = 0, $pInode = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $3 = sp + 32|0;
 $pFile = sp + 28|0;
 $pInode = sp + 24|0;
 $lock = sp + 8|0;
 $rc = sp;
 HEAP32[$1>>2] = $id;
 HEAP32[$2>>2] = $eFileLock;
 HEAP32[$3>>2] = $handleNFSUnlock;
 $4 = HEAP32[$1>>2]|0;
 HEAP32[$pFile>>2] = $4;
 HEAP32[$rc>>2] = 0;
 $5 = HEAP32[$pFile>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = HEAP32[$2>>2]|0;
 $10 = ($8|0)<=($9|0);
 if ($10) {
  HEAP32[$0>>2] = 0;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 _unixEnterMutex();
 $11 = HEAP32[$pFile>>2]|0;
 $12 = ((($11)) + 8|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$pInode>>2] = $13;
 $14 = HEAP32[$pFile>>2]|0;
 $15 = ((($14)) + 16|0);
 $16 = HEAP8[$15>>0]|0;
 $17 = $16&255;
 $18 = ($17|0)>(1);
 do {
  if ($18) {
   $19 = HEAP32[$2>>2]|0;
   $20 = ($19|0)==(1);
   if ($20) {
    HEAP16[$lock>>1] = 0;
    $21 = ((($lock)) + 2|0);
    HEAP16[$21>>1] = 0;
    $22 = HEAP32[1536>>2]|0;
    $23 = (($22) + 2)|0;
    $24 = ((($lock)) + 4|0);
    HEAP32[$24>>2] = $23;
    $25 = ((($lock)) + 8|0);
    HEAP32[$25>>2] = 510;
    $26 = HEAP32[$pFile>>2]|0;
    $27 = (_unixFileLock($26,$lock)|0);
    $28 = ($27|0)!=(0);
    if ($28) {
     HEAP32[$rc>>2] = 2314;
     $29 = HEAP32[$pFile>>2]|0;
     $30 = (___errno_location()|0);
     $31 = HEAP32[$30>>2]|0;
     _storeLastErrno($29,$31);
     break;
    }
   }
   HEAP16[$lock>>1] = 2;
   $32 = ((($lock)) + 2|0);
   HEAP16[$32>>1] = 0;
   $33 = HEAP32[1536>>2]|0;
   $34 = ((($lock)) + 4|0);
   HEAP32[$34>>2] = $33;
   $35 = ((($lock)) + 8|0);
   HEAP32[$35>>2] = 2;
   $36 = HEAP32[$pFile>>2]|0;
   $37 = (_unixFileLock($36,$lock)|0);
   $38 = ($37|0)==(0);
   if ($38) {
    $39 = HEAP32[$pInode>>2]|0;
    $40 = ((($39)) + 12|0);
    HEAP8[$40>>0] = 1;
    label = 10;
    break;
   } else {
    HEAP32[$rc>>2] = 2058;
    $41 = HEAP32[$pFile>>2]|0;
    $42 = (___errno_location()|0);
    $43 = HEAP32[$42>>2]|0;
    _storeLastErrno($41,$43);
    break;
   }
  } else {
   label = 10;
  }
 } while(0);
 if ((label|0) == 10) {
  $44 = HEAP32[$2>>2]|0;
  $45 = ($44|0)==(0);
  if ($45) {
   $46 = HEAP32[$pInode>>2]|0;
   $47 = ((($46)) + 8|0);
   $48 = HEAP32[$47>>2]|0;
   $49 = (($48) + -1)|0;
   HEAP32[$47>>2] = $49;
   $50 = HEAP32[$pInode>>2]|0;
   $51 = ((($50)) + 8|0);
   $52 = HEAP32[$51>>2]|0;
   $53 = ($52|0)==(0);
   do {
    if ($53) {
     HEAP16[$lock>>1] = 2;
     $54 = ((($lock)) + 2|0);
     HEAP16[$54>>1] = 0;
     $55 = ((($lock)) + 8|0);
     HEAP32[$55>>2] = 0;
     $56 = ((($lock)) + 4|0);
     HEAP32[$56>>2] = 0;
     $57 = HEAP32[$pFile>>2]|0;
     $58 = (_unixFileLock($57,$lock)|0);
     $59 = ($58|0)==(0);
     if ($59) {
      $60 = HEAP32[$pInode>>2]|0;
      $61 = ((($60)) + 12|0);
      HEAP8[$61>>0] = 0;
      break;
     } else {
      HEAP32[$rc>>2] = 2058;
      $62 = HEAP32[$pFile>>2]|0;
      $63 = (___errno_location()|0);
      $64 = HEAP32[$63>>2]|0;
      _storeLastErrno($62,$64);
      $65 = HEAP32[$pInode>>2]|0;
      $66 = ((($65)) + 12|0);
      HEAP8[$66>>0] = 0;
      $67 = HEAP32[$pFile>>2]|0;
      $68 = ((($67)) + 16|0);
      HEAP8[$68>>0] = 0;
      break;
     }
    }
   } while(0);
   $69 = HEAP32[$pInode>>2]|0;
   $70 = ((($69)) + 24|0);
   $71 = HEAP32[$70>>2]|0;
   $72 = (($71) + -1)|0;
   HEAP32[$70>>2] = $72;
   $73 = HEAP32[$pInode>>2]|0;
   $74 = ((($73)) + 24|0);
   $75 = HEAP32[$74>>2]|0;
   $76 = ($75|0)==(0);
   if ($76) {
    $77 = HEAP32[$pFile>>2]|0;
    _closePendingFds($77);
   }
  }
 }
 _unixLeaveMutex();
 $78 = HEAP32[$rc>>2]|0;
 $79 = ($78|0)==(0);
 if ($79) {
  $80 = HEAP32[$2>>2]|0;
  $81 = $80&255;
  $82 = HEAP32[$pFile>>2]|0;
  $83 = ((($82)) + 16|0);
  HEAP8[$83>>0] = $81;
 }
 $84 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $84;
 $85 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($85|0);
}
function _unixFileLock($pFile,$pLock) {
 $pFile = $pFile|0;
 $pLock = $pLock|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $lock = 0, $pInode = 0, $rc = 0, $vararg_buffer = 0, $vararg_buffer1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer1 = sp + 8|0;
 $vararg_buffer = sp;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $rc = sp + 36|0;
 $pInode = sp + 32|0;
 $lock = sp + 16|0;
 HEAP32[$1>>2] = $pFile;
 HEAP32[$2>>2] = $pLock;
 $3 = HEAP32[$1>>2]|0;
 $4 = ((($3)) + 8|0);
 $5 = HEAP32[$4>>2]|0;
 HEAP32[$pInode>>2] = $5;
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 18|0);
 $8 = HEAP16[$7>>1]|0;
 $9 = $8&65535;
 $10 = $9 & 3;
 $11 = ($10|0)==(1);
 do {
  if ($11) {
   $12 = HEAP32[$pInode>>2]|0;
   $13 = ((($12)) + 13|0);
   $14 = HEAP8[$13>>0]|0;
   $15 = $14&255;
   $16 = ($15|0)==(0);
   if (!($16)) {
    HEAP32[$rc>>2] = 0;
    break;
   }
   $17 = ((($lock)) + 2|0);
   HEAP16[$17>>1] = 0;
   $18 = HEAP32[1536>>2]|0;
   $19 = (($18) + 2)|0;
   $20 = ((($lock)) + 4|0);
   HEAP32[$20>>2] = $19;
   $21 = ((($lock)) + 8|0);
   HEAP32[$21>>2] = 510;
   HEAP16[$lock>>1] = 1;
   $22 = HEAP32[(6524)>>2]|0;
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 12|0);
   $25 = HEAP32[$24>>2]|0;
   HEAP32[$vararg_buffer>>2] = $lock;
   $26 = (FUNCTION_TABLE_iiii[$22 & 255]($25,13,$vararg_buffer)|0);
   HEAP32[$rc>>2] = $26;
   $27 = HEAP32[$rc>>2]|0;
   $28 = ($27|0)<(0);
   if (!($28)) {
    $30 = HEAP32[$pInode>>2]|0;
    $31 = ((($30)) + 13|0);
    HEAP8[$31>>0] = 1;
    $32 = HEAP32[$pInode>>2]|0;
    $33 = ((($32)) + 24|0);
    $34 = HEAP32[$33>>2]|0;
    $35 = (($34) + 1)|0;
    HEAP32[$33>>2] = $35;
    break;
   }
   $29 = HEAP32[$rc>>2]|0;
   HEAP32[$0>>2] = $29;
   $43 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($43|0);
  } else {
   $36 = HEAP32[(6524)>>2]|0;
   $37 = HEAP32[$1>>2]|0;
   $38 = ((($37)) + 12|0);
   $39 = HEAP32[$38>>2]|0;
   $40 = HEAP32[$2>>2]|0;
   HEAP32[$vararg_buffer1>>2] = $40;
   $41 = (FUNCTION_TABLE_iiii[$36 & 255]($39,13,$vararg_buffer1)|0);
   HEAP32[$rc>>2] = $41;
  }
 } while(0);
 $42 = HEAP32[$rc>>2]|0;
 HEAP32[$0>>2] = $42;
 $43 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($43|0);
}
function _closePendingFds($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pInode = 0, $pNext = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $pInode = sp + 8|0;
 $p = sp + 4|0;
 $pNext = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$pInode>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 while(1) {
  $7 = HEAP32[$p>>2]|0;
  $8 = ($7|0)!=(0|0);
  if (!($8)) {
   break;
  }
  $9 = HEAP32[$p>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$pNext>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = HEAP32[$p>>2]|0;
  $14 = HEAP32[$13>>2]|0;
  _robust_close($12,$14,30592);
  $15 = HEAP32[$p>>2]|0;
  _sqlite3_free($15);
  $16 = HEAP32[$pNext>>2]|0;
  HEAP32[$p>>2] = $16;
 }
 $17 = HEAP32[$pInode>>2]|0;
 $18 = ((($17)) + 28|0);
 HEAP32[$18>>2] = 0;
 STACKTOP = sp;return;
}
function _setPendingFd($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0;
 var $pInode = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $pInode = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$0>>2]|0;
 $5 = ((($4)) + 28|0);
 $6 = HEAP32[$5>>2]|0;
 HEAP32[$p>>2] = $6;
 $7 = HEAP32[$pInode>>2]|0;
 $8 = ((($7)) + 28|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$p>>2]|0;
 $11 = ((($10)) + 8|0);
 HEAP32[$11>>2] = $9;
 $12 = HEAP32[$p>>2]|0;
 $13 = HEAP32[$pInode>>2]|0;
 $14 = ((($13)) + 28|0);
 HEAP32[$14>>2] = $12;
 $15 = HEAP32[$0>>2]|0;
 $16 = ((($15)) + 12|0);
 HEAP32[$16>>2] = -1;
 $17 = HEAP32[$0>>2]|0;
 $18 = ((($17)) + 28|0);
 HEAP32[$18>>2] = 0;
 STACKTOP = sp;return;
}
function _releaseInodeInfo($pFile) {
 $pFile = $pFile|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pInode = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pInode = sp;
 HEAP32[$0>>2] = $pFile;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 8|0);
 $3 = HEAP32[$2>>2]|0;
 HEAP32[$pInode>>2] = $3;
 $4 = HEAP32[$pInode>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$pInode>>2]|0;
 $7 = ((($6)) + 16|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + -1)|0;
 HEAP32[$7>>2] = $9;
 $10 = HEAP32[$pInode>>2]|0;
 $11 = ((($10)) + 16|0);
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)==(0);
 if (!($13)) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$0>>2]|0;
 _closePendingFds($14);
 $15 = HEAP32[$pInode>>2]|0;
 $16 = ((($15)) + 36|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)!=(0|0);
 $19 = HEAP32[$pInode>>2]|0;
 $20 = ((($19)) + 32|0);
 $21 = HEAP32[$20>>2]|0;
 if ($18) {
  $22 = HEAP32[$pInode>>2]|0;
  $23 = ((($22)) + 36|0);
  $24 = HEAP32[$23>>2]|0;
  $25 = ((($24)) + 32|0);
  HEAP32[$25>>2] = $21;
 } else {
  HEAP32[6952>>2] = $21;
 }
 $26 = HEAP32[$pInode>>2]|0;
 $27 = ((($26)) + 32|0);
 $28 = HEAP32[$27>>2]|0;
 $29 = ($28|0)!=(0|0);
 if ($29) {
  $30 = HEAP32[$pInode>>2]|0;
  $31 = ((($30)) + 36|0);
  $32 = HEAP32[$31>>2]|0;
  $33 = HEAP32[$pInode>>2]|0;
  $34 = ((($33)) + 32|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = ((($35)) + 36|0);
  HEAP32[$36>>2] = $32;
 }
 $37 = HEAP32[$pInode>>2]|0;
 _sqlite3_free($37);
 STACKTOP = sp;return;
}
function _getFileMode($zFile,$pMode,$pUid,$pGid) {
 $zFile = $zFile|0;
 $pMode = $pMode|0;
 $pUid = $pUid|0;
 $pGid = $pGid|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $rc = 0;
 var $sStat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 96|0;
 $0 = sp + 92|0;
 $1 = sp + 88|0;
 $2 = sp + 84|0;
 $3 = sp + 80|0;
 $sStat = sp + 4|0;
 $rc = sp;
 HEAP32[$0>>2] = $zFile;
 HEAP32[$1>>2] = $pMode;
 HEAP32[$2>>2] = $pUid;
 HEAP32[$3>>2] = $pGid;
 HEAP32[$rc>>2] = 0;
 $4 = HEAP32[(6488)>>2]|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = (FUNCTION_TABLE_iii[$4 & 255]($5,$sStat)|0);
 $7 = (0)==($6|0);
 if ($7) {
  $8 = ((($sStat)) + 12|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = $9 & 511;
  $11 = HEAP32[$1>>2]|0;
  HEAP32[$11>>2] = $10;
  $12 = ((($sStat)) + 20|0);
  $13 = HEAP32[$12>>2]|0;
  $14 = HEAP32[$2>>2]|0;
  HEAP32[$14>>2] = $13;
  $15 = ((($sStat)) + 24|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = HEAP32[$3>>2]|0;
  HEAP32[$17>>2] = $16;
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  HEAP32[$rc>>2] = 1802;
  $18 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _posixIoFinderImpl($z,$p) {
 $z = $z|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $z;
 HEAP32[$1>>2] = $p;
 STACKTOP = sp;return (6876|0);
}
function _getIntArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  $8 = $0;
  $9 = $8;
  HEAP32[$9>>2] = 0;
  $10 = (($8) + 4)|0;
  $11 = $10;
  HEAP32[$11>>2] = 0;
  $27 = $0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  tempRet0 = ($32);
  STACKTOP = sp;return ($29|0);
 } else {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 4|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + 1)|0;
  HEAP32[$13>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = ((($16)) + 8|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (($18) + ($14<<2)|0);
  $20 = HEAP32[$19>>2]|0;
  $21 = (_sqlite3_value_int64($20)|0);
  $22 = tempRet0;
  $23 = $0;
  $24 = $23;
  HEAP32[$24>>2] = $21;
  $25 = (($23) + 4)|0;
  $26 = $25;
  HEAP32[$26>>2] = $22;
  $27 = $0;
  $28 = $27;
  $29 = HEAP32[$28>>2]|0;
  $30 = (($27) + 4)|0;
  $31 = $30;
  $32 = HEAP32[$31>>2]|0;
  tempRet0 = ($32);
  STACKTOP = sp;return ($29|0);
 }
 return (0)|0;
}
function _getDoubleArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 $1 = sp + 8|0;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  HEAPF64[$0>>3] = 0.0;
  $18 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$18);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$9>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($10<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (+_sqlite3_value_double($16));
  HEAPF64[$0>>3] = $17;
  $18 = +HEAPF64[$0>>3];
  STACKTOP = sp;return (+$18);
 }
 return +(0.0);
}
function _et_getdigit($val,$cnt) {
 $val = $val|0;
 $cnt = $cnt|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $d = 0, $digit = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $digit = sp + 8|0;
 $d = sp;
 HEAP32[$1>>2] = $val;
 HEAP32[$2>>2] = $cnt;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = ($4|0)<=(0);
 if ($5) {
  HEAP8[$0>>0] = 48;
  $24 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($24|0);
 } else {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (($7) + -1)|0;
  HEAP32[$6>>2] = $8;
  $9 = HEAP32[$1>>2]|0;
  $10 = +HEAPF64[$9>>3];
  $11 = (~~(($10)));
  HEAP32[$digit>>2] = $11;
  $12 = HEAP32[$digit>>2]|0;
  $13 = (+($12|0));
  HEAPF64[$d>>3] = $13;
  $14 = HEAP32[$digit>>2]|0;
  $15 = (($14) + 48)|0;
  HEAP32[$digit>>2] = $15;
  $16 = HEAP32[$1>>2]|0;
  $17 = +HEAPF64[$16>>3];
  $18 = +HEAPF64[$d>>3];
  $19 = $17 - $18;
  $20 = $19 * 10.0;
  $21 = HEAP32[$1>>2]|0;
  HEAPF64[$21>>3] = $20;
  $22 = HEAP32[$digit>>2]|0;
  $23 = $22&255;
  HEAP8[$0>>0] = $23;
  $24 = HEAP8[$0>>0]|0;
  STACKTOP = sp;return ($24|0);
 }
 return (0)|0;
}
function _getTextArg($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$1>>2]|0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = ((($4)) + 4|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($3|0)<=($6|0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 } else {
  $8 = HEAP32[$1>>2]|0;
  $9 = ((($8)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = (($10) + 1)|0;
  HEAP32[$9>>2] = $11;
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 8|0);
  $14 = HEAP32[$13>>2]|0;
  $15 = (($14) + ($10<<2)|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = (_sqlite3_value_text($16)|0);
  HEAP32[$0>>2] = $17;
  $18 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($18|0);
 }
 return (0)|0;
}
function _sqlite3AppendChar($p,$N,$c) {
 $p = $p|0;
 $N = $N|0;
 $c = $c|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 $2 = sp + 8|0;
 HEAP32[$0>>2] = $p;
 HEAP32[$1>>2] = $N;
 HEAP8[$2>>0] = $c;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 12|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = HEAP32[$1>>2]|0;
 $7 = ($6|0)<(0);
 $8 = $7 << 31 >> 31;
 $9 = (_i64Add(($5|0),0,($6|0),($8|0))|0);
 $10 = tempRet0;
 $11 = HEAP32[$0>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ($10|0)>(0);
 $15 = ($9>>>0)>=($13>>>0);
 $16 = ($10|0)==(0);
 $17 = $16 & $15;
 $18 = $14 | $17;
 if ($18) {
  $19 = HEAP32[$0>>2]|0;
  $20 = HEAP32[$1>>2]|0;
  $21 = (_sqlite3StrAccumEnlarge($19,$20)|0);
  HEAP32[$1>>2] = $21;
  $22 = ($21|0)<=(0);
  if ($22) {
   STACKTOP = sp;return;
  }
 }
 while(1) {
  $23 = HEAP32[$1>>2]|0;
  $24 = (($23) + -1)|0;
  HEAP32[$1>>2] = $24;
  $25 = ($23|0)>(0);
  if (!($25)) {
   break;
  }
  $26 = HEAP8[$2>>0]|0;
  $27 = HEAP32[$0>>2]|0;
  $28 = ((($27)) + 12|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = (($29) + 1)|0;
  HEAP32[$28>>2] = $30;
  $31 = HEAP32[$0>>2]|0;
  $32 = ((($31)) + 8|0);
  $33 = HEAP32[$32>>2]|0;
  $34 = (($33) + ($29)|0);
  HEAP8[$34>>0] = $26;
 }
 STACKTOP = sp;return;
}
function _sqlite3MallocAlarm($nByte) {
 $nByte = $nByte|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $nByte;
 $1 = (296);
 $2 = $1;
 $3 = HEAP32[$2>>2]|0;
 $4 = (($1) + 4)|0;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)<(0);
 $8 = ($3>>>0)<=(0);
 $9 = ($6|0)==(0);
 $10 = $9 & $8;
 $11 = $7 | $10;
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 (_sqlite3_release_memory($12)|0);
 STACKTOP = sp;return;
}
function _mallocWithAlarm($n,$pp) {
 $n = $n|0;
 $pp = $pp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $nFull = 0, $nUsed = 0, $p = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $nFull = sp + 12|0;
 $p = sp + 8|0;
 $nUsed = sp;
 HEAP32[$0>>2] = $n;
 HEAP32[$1>>2] = $pp;
 $2 = HEAP32[(64)>>2]|0;
 $3 = HEAP32[$0>>2]|0;
 $4 = (FUNCTION_TABLE_ii[$2 & 255]($3)|0);
 HEAP32[$nFull>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 _sqlite3StatusHighwater(5,$5);
 $6 = (296);
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($6) + 4)|0;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)>(0);
 $13 = ($8>>>0)>(0);
 $14 = ($11|0)==(0);
 $15 = $14 & $13;
 $16 = $12 | $15;
 do {
  if ($16) {
   $17 = (_sqlite3StatusValue(0)|0);
   $18 = tempRet0;
   $19 = $nUsed;
   $20 = $19;
   HEAP32[$20>>2] = $17;
   $21 = (($19) + 4)|0;
   $22 = $21;
   HEAP32[$22>>2] = $18;
   $23 = $nUsed;
   $24 = $23;
   $25 = HEAP32[$24>>2]|0;
   $26 = (($23) + 4)|0;
   $27 = $26;
   $28 = HEAP32[$27>>2]|0;
   $29 = (296);
   $30 = $29;
   $31 = HEAP32[$30>>2]|0;
   $32 = (($29) + 4)|0;
   $33 = $32;
   $34 = HEAP32[$33>>2]|0;
   $35 = HEAP32[$nFull>>2]|0;
   $36 = ($35|0)<(0);
   $37 = $36 << 31 >> 31;
   $38 = (_i64Subtract(($31|0),($34|0),($35|0),($37|0))|0);
   $39 = tempRet0;
   $40 = ($28|0)>($39|0);
   $41 = ($25>>>0)>=($38>>>0);
   $42 = ($28|0)==($39|0);
   $43 = $42 & $41;
   $44 = $40 | $43;
   if ($44) {
    HEAP32[(316)>>2] = 1;
    $45 = HEAP32[$nFull>>2]|0;
    _sqlite3MallocAlarm($45);
    break;
   } else {
    HEAP32[(316)>>2] = 0;
    break;
   }
  }
 } while(0);
 $46 = HEAP32[(48)>>2]|0;
 $47 = HEAP32[$nFull>>2]|0;
 $48 = (FUNCTION_TABLE_ii[$46 & 255]($47)|0);
 HEAP32[$p>>2] = $48;
 $49 = HEAP32[$p>>2]|0;
 $50 = ($49|0)!=(0|0);
 if (!($50)) {
  $54 = HEAP32[$p>>2]|0;
  $55 = HEAP32[$1>>2]|0;
  HEAP32[$55>>2] = $54;
  $56 = HEAP32[$nFull>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $51 = HEAP32[$p>>2]|0;
 $52 = (_sqlite3MallocSize($51)|0);
 HEAP32[$nFull>>2] = $52;
 $53 = HEAP32[$nFull>>2]|0;
 _sqlite3StatusUp(0,$53);
 _sqlite3StatusUp(9,1);
 $54 = HEAP32[$p>>2]|0;
 $55 = HEAP32[$1>>2]|0;
 HEAP32[$55>>2] = $54;
 $56 = HEAP32[$nFull>>2]|0;
 STACKTOP = sp;return ($56|0);
}
function _sqlite3AlterFunctions() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 _sqlite3InsertBuiltinFuncs(8780,3);
 return;
}
function _sqlite3RegisterDateTimeFunctions() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 _sqlite3InsertBuiltinFuncs(8556,8);
 return;
}
function _sqlite3InsertBuiltinFuncs($aDef,$nDef) {
 $aDef = $aDef|0;
 $nDef = $nDef|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $h = 0, $i = 0, $nName = 0, $pOther = 0;
 var $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $i = sp + 16|0;
 $pOther = sp + 12|0;
 $zName = sp + 8|0;
 $nName = sp + 4|0;
 $h = sp;
 HEAP32[$0>>2] = $aDef;
 HEAP32[$1>>2] = $nDef;
 HEAP32[$i>>2] = 0;
 while(1) {
  $2 = HEAP32[$i>>2]|0;
  $3 = HEAP32[$1>>2]|0;
  $4 = ($2|0)<($3|0);
  if (!($4)) {
   break;
  }
  $5 = HEAP32[$i>>2]|0;
  $6 = HEAP32[$0>>2]|0;
  $7 = (($6) + (($5*28)|0)|0);
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$zName>>2] = $9;
  $10 = HEAP32[$zName>>2]|0;
  $11 = (_sqlite3Strlen30($10)|0);
  HEAP32[$nName>>2] = $11;
  $12 = HEAP32[$zName>>2]|0;
  $13 = HEAP8[$12>>0]|0;
  $14 = $13&255;
  $15 = (17613 + ($14)|0);
  $16 = HEAP8[$15>>0]|0;
  $17 = $16&255;
  $18 = HEAP32[$nName>>2]|0;
  $19 = (($17) + ($18))|0;
  $20 = (($19|0) % 23)&-1;
  HEAP32[$h>>2] = $20;
  $21 = HEAP32[$h>>2]|0;
  $22 = HEAP32[$zName>>2]|0;
  $23 = (_functionSearch($21,$22)|0);
  HEAP32[$pOther>>2] = $23;
  $24 = HEAP32[$pOther>>2]|0;
  $25 = ($24|0)!=(0|0);
  if ($25) {
   $26 = HEAP32[$pOther>>2]|0;
   $27 = ((($26)) + 8|0);
   $28 = HEAP32[$27>>2]|0;
   $29 = HEAP32[$i>>2]|0;
   $30 = HEAP32[$0>>2]|0;
   $31 = (($30) + (($29*28)|0)|0);
   $32 = ((($31)) + 8|0);
   HEAP32[$32>>2] = $28;
   $33 = HEAP32[$i>>2]|0;
   $34 = HEAP32[$0>>2]|0;
   $35 = (($34) + (($33*28)|0)|0);
   $36 = HEAP32[$pOther>>2]|0;
   $37 = ((($36)) + 8|0);
   HEAP32[$37>>2] = $35;
  } else {
   $38 = HEAP32[$i>>2]|0;
   $39 = HEAP32[$0>>2]|0;
   $40 = (($39) + (($38*28)|0)|0);
   $41 = ((($40)) + 8|0);
   HEAP32[$41>>2] = 0;
   $42 = HEAP32[$h>>2]|0;
   $43 = (1092 + ($42<<2)|0);
   $44 = HEAP32[$43>>2]|0;
   $45 = HEAP32[$i>>2]|0;
   $46 = HEAP32[$0>>2]|0;
   $47 = (($46) + (($45*28)|0)|0);
   $48 = ((($47)) + 24|0);
   HEAP32[$48>>2] = $44;
   $49 = HEAP32[$i>>2]|0;
   $50 = HEAP32[$0>>2]|0;
   $51 = (($50) + (($49*28)|0)|0);
   $52 = HEAP32[$h>>2]|0;
   $53 = (1092 + ($52<<2)|0);
   HEAP32[$53>>2] = $51;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 STACKTOP = sp;return;
}
function _compileoptionusedFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $zOptName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $zOptName = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zOptName>>2] = $5;
 $6 = ($5|0)!=(0|0);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = HEAP32[$zOptName>>2]|0;
 $9 = (_sqlite3_compileoption_used($8)|0);
 _sqlite3_result_int($7,$9);
 STACKTOP = sp;return;
}
function _compileoptiongetFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $n = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$n>>2]|0;
 $8 = (_sqlite3_compileoption_get($7)|0);
 _sqlite3_result_text($6,$8,-1,0);
 STACKTOP = sp;return;
}
function _versionFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_libversion()|0);
 _sqlite3_result_text($3,$4,-1,0);
 STACKTOP = sp;return;
}
function _trimFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$old = 0, $$old1 = 0, $$old2 = 0, $$old3 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0;
 var $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0;
 var $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0;
 var $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0;
 var $167 = 0, $168 = 0, $169 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $aLen = 0, $azChar = 0, $flags = 0, $i = 0, $len = 0, $len1 = 0;
 var $nChar = 0, $nIn = 0, $or$cond = 0, $or$cond4 = 0, $z = 0, $zCharSet = 0, $zIn = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 52|0;
 $1 = sp + 48|0;
 $2 = sp + 44|0;
 $zIn = sp + 40|0;
 $zCharSet = sp + 36|0;
 $nIn = sp + 32|0;
 $flags = sp + 28|0;
 $i = sp + 24|0;
 $aLen = sp + 20|0;
 $azChar = sp + 16|0;
 $nChar = sp + 12|0;
 $z = sp + 8|0;
 $len = sp + 4|0;
 $len1 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$aLen>>2] = 0;
 HEAP32[$azChar>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 $6 = ($5|0)==(5);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zIn>>2] = $9;
 $10 = HEAP32[$zIn>>2]|0;
 $11 = ($10|0)==(0|0);
 if ($11) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_value_bytes($13)|0);
 HEAP32[$nIn>>2] = $14;
 $15 = HEAP32[$1>>2]|0;
 $16 = ($15|0)==(1);
 L7: do {
  if ($16) {
   HEAP32[$nChar>>2] = 1;
   HEAP32[$aLen>>2] = 45220;
   HEAP32[$azChar>>2] = 8552;
   HEAP32[$zCharSet>>2] = 0;
  } else {
   $17 = HEAP32[$2>>2]|0;
   $18 = ((($17)) + 4|0);
   $19 = HEAP32[$18>>2]|0;
   $20 = (_sqlite3_value_text($19)|0);
   HEAP32[$zCharSet>>2] = $20;
   $21 = ($20|0)==(0|0);
   if ($21) {
    STACKTOP = sp;return;
   }
   $22 = HEAP32[$zCharSet>>2]|0;
   HEAP32[$z>>2] = $22;
   HEAP32[$nChar>>2] = 0;
   while(1) {
    $23 = HEAP32[$z>>2]|0;
    $24 = HEAP8[$23>>0]|0;
    $25 = ($24<<24>>24)!=(0);
    if (!($25)) {
     break;
    }
    $26 = HEAP32[$z>>2]|0;
    $27 = ((($26)) + 1|0);
    HEAP32[$z>>2] = $27;
    $28 = HEAP8[$26>>0]|0;
    $29 = $28&255;
    $30 = ($29|0)>=(192);
    L16: do {
     if ($30) {
      while(1) {
       $31 = HEAP32[$z>>2]|0;
       $32 = HEAP8[$31>>0]|0;
       $33 = $32&255;
       $34 = $33 & 192;
       $35 = ($34|0)==(128);
       if (!($35)) {
        break L16;
       }
       $36 = HEAP32[$z>>2]|0;
       $37 = ((($36)) + 1|0);
       HEAP32[$z>>2] = $37;
      }
     }
    } while(0);
    $38 = HEAP32[$nChar>>2]|0;
    $39 = (($38) + 1)|0;
    HEAP32[$nChar>>2] = $39;
   }
   $40 = HEAP32[$nChar>>2]|0;
   $41 = ($40|0)>(0);
   if ($41) {
    $42 = HEAP32[$0>>2]|0;
    $43 = HEAP32[$nChar>>2]|0;
    $44 = ($43|0)<(0);
    $45 = $44 << 31 >> 31;
    $46 = (___muldi3(($43|0),($45|0),5,0)|0);
    $47 = tempRet0;
    $48 = (_contextMalloc($42,$46,$47)|0);
    HEAP32[$azChar>>2] = $48;
    $49 = HEAP32[$azChar>>2]|0;
    $50 = ($49|0)==(0|0);
    if ($50) {
     STACKTOP = sp;return;
    }
    $51 = HEAP32[$nChar>>2]|0;
    $52 = HEAP32[$azChar>>2]|0;
    $53 = (($52) + ($51<<2)|0);
    HEAP32[$aLen>>2] = $53;
    $54 = HEAP32[$zCharSet>>2]|0;
    HEAP32[$z>>2] = $54;
    HEAP32[$nChar>>2] = 0;
    while(1) {
     $55 = HEAP32[$z>>2]|0;
     $56 = HEAP8[$55>>0]|0;
     $57 = ($56<<24>>24)!=(0);
     if (!($57)) {
      break L7;
     }
     $58 = HEAP32[$z>>2]|0;
     $59 = HEAP32[$nChar>>2]|0;
     $60 = HEAP32[$azChar>>2]|0;
     $61 = (($60) + ($59<<2)|0);
     HEAP32[$61>>2] = $58;
     $62 = HEAP32[$z>>2]|0;
     $63 = ((($62)) + 1|0);
     HEAP32[$z>>2] = $63;
     $64 = HEAP8[$62>>0]|0;
     $65 = $64&255;
     $66 = ($65|0)>=(192);
     L29: do {
      if ($66) {
       while(1) {
        $67 = HEAP32[$z>>2]|0;
        $68 = HEAP8[$67>>0]|0;
        $69 = $68&255;
        $70 = $69 & 192;
        $71 = ($70|0)==(128);
        if (!($71)) {
         break L29;
        }
        $72 = HEAP32[$z>>2]|0;
        $73 = ((($72)) + 1|0);
        HEAP32[$z>>2] = $73;
       }
      }
     } while(0);
     $74 = HEAP32[$z>>2]|0;
     $75 = HEAP32[$nChar>>2]|0;
     $76 = HEAP32[$azChar>>2]|0;
     $77 = (($76) + ($75<<2)|0);
     $78 = HEAP32[$77>>2]|0;
     $79 = $74;
     $80 = $78;
     $81 = (($79) - ($80))|0;
     $82 = $81&255;
     $83 = HEAP32[$nChar>>2]|0;
     $84 = HEAP32[$aLen>>2]|0;
     $85 = (($84) + ($83)|0);
     HEAP8[$85>>0] = $82;
     $86 = HEAP32[$nChar>>2]|0;
     $87 = (($86) + 1)|0;
     HEAP32[$nChar>>2] = $87;
    }
   }
  }
 } while(0);
 $88 = HEAP32[$nChar>>2]|0;
 $89 = ($88|0)>(0);
 if ($89) {
  $90 = HEAP32[$0>>2]|0;
  $91 = (_sqlite3_user_data($90)|0);
  $92 = $91;
  HEAP32[$flags>>2] = $92;
  $93 = HEAP32[$flags>>2]|0;
  $94 = $93 & 1;
  $95 = ($94|0)!=(0);
  $96 = HEAP32[$nIn>>2]|0;
  $97 = ($96|0)>(0);
  $or$cond = $95 & $97;
  L37: do {
   if ($or$cond) {
    while(1) {
     HEAP32[$len>>2] = 0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $98 = HEAP32[$i>>2]|0;
      $99 = HEAP32[$nChar>>2]|0;
      $100 = ($98|0)<($99|0);
      if (!($100)) {
       break;
      }
      $101 = HEAP32[$i>>2]|0;
      $102 = HEAP32[$aLen>>2]|0;
      $103 = (($102) + ($101)|0);
      $104 = HEAP8[$103>>0]|0;
      $105 = $104&255;
      HEAP32[$len>>2] = $105;
      $106 = HEAP32[$len>>2]|0;
      $107 = HEAP32[$nIn>>2]|0;
      $108 = ($106|0)<=($107|0);
      if ($108) {
       $109 = HEAP32[$zIn>>2]|0;
       $110 = HEAP32[$i>>2]|0;
       $111 = HEAP32[$azChar>>2]|0;
       $112 = (($111) + ($110<<2)|0);
       $113 = HEAP32[$112>>2]|0;
       $114 = HEAP32[$len>>2]|0;
       $115 = (_memcmp($109,$113,$114)|0);
       $116 = ($115|0)==(0);
       if ($116) {
        break;
       }
      }
      $117 = HEAP32[$i>>2]|0;
      $118 = (($117) + 1)|0;
      HEAP32[$i>>2] = $118;
     }
     $119 = HEAP32[$i>>2]|0;
     $120 = HEAP32[$nChar>>2]|0;
     $121 = ($119|0)>=($120|0);
     if ($121) {
      break L37;
     }
     $122 = HEAP32[$len>>2]|0;
     $123 = HEAP32[$zIn>>2]|0;
     $124 = (($123) + ($122)|0);
     HEAP32[$zIn>>2] = $124;
     $125 = HEAP32[$len>>2]|0;
     $126 = HEAP32[$nIn>>2]|0;
     $127 = (($126) - ($125))|0;
     HEAP32[$nIn>>2] = $127;
     $$old = HEAP32[$nIn>>2]|0;
     $$old1 = ($$old|0)>(0);
     if (!($$old1)) {
      break;
     }
    }
   }
  } while(0);
  $128 = HEAP32[$flags>>2]|0;
  $129 = $128 & 2;
  $130 = ($129|0)!=(0);
  $131 = HEAP32[$nIn>>2]|0;
  $132 = ($131|0)>(0);
  $or$cond4 = $130 & $132;
  L49: do {
   if ($or$cond4) {
    while(1) {
     HEAP32[$len1>>2] = 0;
     HEAP32[$i>>2] = 0;
     while(1) {
      $133 = HEAP32[$i>>2]|0;
      $134 = HEAP32[$nChar>>2]|0;
      $135 = ($133|0)<($134|0);
      if (!($135)) {
       break;
      }
      $136 = HEAP32[$i>>2]|0;
      $137 = HEAP32[$aLen>>2]|0;
      $138 = (($137) + ($136)|0);
      $139 = HEAP8[$138>>0]|0;
      $140 = $139&255;
      HEAP32[$len1>>2] = $140;
      $141 = HEAP32[$len1>>2]|0;
      $142 = HEAP32[$nIn>>2]|0;
      $143 = ($141|0)<=($142|0);
      if ($143) {
       $144 = HEAP32[$nIn>>2]|0;
       $145 = HEAP32[$len1>>2]|0;
       $146 = (($144) - ($145))|0;
       $147 = HEAP32[$zIn>>2]|0;
       $148 = (($147) + ($146)|0);
       $149 = HEAP32[$i>>2]|0;
       $150 = HEAP32[$azChar>>2]|0;
       $151 = (($150) + ($149<<2)|0);
       $152 = HEAP32[$151>>2]|0;
       $153 = HEAP32[$len1>>2]|0;
       $154 = (_memcmp($148,$152,$153)|0);
       $155 = ($154|0)==(0);
       if ($155) {
        break;
       }
      }
      $156 = HEAP32[$i>>2]|0;
      $157 = (($156) + 1)|0;
      HEAP32[$i>>2] = $157;
     }
     $158 = HEAP32[$i>>2]|0;
     $159 = HEAP32[$nChar>>2]|0;
     $160 = ($158|0)>=($159|0);
     if ($160) {
      break L49;
     }
     $161 = HEAP32[$len1>>2]|0;
     $162 = HEAP32[$nIn>>2]|0;
     $163 = (($162) - ($161))|0;
     HEAP32[$nIn>>2] = $163;
     $$old2 = HEAP32[$nIn>>2]|0;
     $$old3 = ($$old2|0)>(0);
     if (!($$old3)) {
      break;
     }
    }
   }
  } while(0);
  $164 = HEAP32[$zCharSet>>2]|0;
  $165 = ($164|0)!=(0|0);
  if ($165) {
   $166 = HEAP32[$azChar>>2]|0;
   _sqlite3_free($166);
  }
 }
 $167 = HEAP32[$0>>2]|0;
 $168 = HEAP32[$zIn>>2]|0;
 $169 = HEAP32[$nIn>>2]|0;
 _sqlite3_result_text($167,$168,$169,(-1));
 STACKTOP = sp;return;
}
function _minmaxFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $i = 0, $iBest = 0, $mask = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $i = sp + 12|0;
 $mask = sp + 8|0;
 $iBest = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_user_data($3)|0);
 $5 = ($4|0)==(0|0);
 $6 = $5 ? 0 : -1;
 HEAP32[$mask>>2] = $6;
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3GetFuncCollSeq($7)|0);
 HEAP32[$pColl>>2] = $8;
 HEAP32[$iBest>>2] = 0;
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 $12 = ($11|0)==(5);
 if ($12) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 1;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ($13|0)<($14|0);
  if (!($15)) {
   break;
  }
  $16 = HEAP32[$i>>2]|0;
  $17 = HEAP32[$2>>2]|0;
  $18 = (($17) + ($16<<2)|0);
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3_value_type($19)|0);
  $21 = ($20|0)==(5);
  if ($21) {
   label = 9;
   break;
  }
  $22 = HEAP32[$iBest>>2]|0;
  $23 = HEAP32[$2>>2]|0;
  $24 = (($23) + ($22<<2)|0);
  $25 = HEAP32[$24>>2]|0;
  $26 = HEAP32[$i>>2]|0;
  $27 = HEAP32[$2>>2]|0;
  $28 = (($27) + ($26<<2)|0);
  $29 = HEAP32[$28>>2]|0;
  $30 = HEAP32[$pColl>>2]|0;
  $31 = (_sqlite3MemCompare($25,$29,$30)|0);
  $32 = HEAP32[$mask>>2]|0;
  $33 = $31 ^ $32;
  $34 = ($33|0)>=(0);
  if ($34) {
   $35 = HEAP32[$i>>2]|0;
   HEAP32[$iBest>>2] = $35;
  }
  $36 = HEAP32[$i>>2]|0;
  $37 = (($36) + 1)|0;
  HEAP32[$i>>2] = $37;
 }
 if ((label|0) == 9) {
  STACKTOP = sp;return;
 }
 $38 = HEAP32[$0>>2]|0;
 $39 = HEAP32[$iBest>>2]|0;
 $40 = HEAP32[$2>>2]|0;
 $41 = (($40) + ($39<<2)|0);
 $42 = HEAP32[$41>>2]|0;
 _sqlite3_result_value($38,$42);
 STACKTOP = sp;return;
}
function _minmaxStep($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cmp = 0, $max = 0, $or$cond = 0, $or$cond3 = 0, $pArg = 0, $pBest = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $pArg = sp + 16|0;
 $pBest = sp + 12|0;
 $max = sp + 8|0;
 $cmp = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pArg>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = (_sqlite3_aggregate_context($5,40)|0);
 HEAP32[$pBest>>2] = $6;
 $7 = HEAP32[$pBest>>2]|0;
 $8 = ($7|0)!=(0|0);
 if (!($8)) {
  STACKTOP = sp;return;
 }
 $9 = HEAP32[$2>>2]|0;
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_type($10)|0);
 $12 = ($11|0)==(5);
 $13 = HEAP32[$pBest>>2]|0;
 $14 = ((($13)) + 8|0);
 $15 = HEAP16[$14>>1]|0;
 $16 = ($15<<16>>16)!=(0);
 if ($12) {
  if (!($16)) {
   STACKTOP = sp;return;
  }
  $17 = HEAP32[$0>>2]|0;
  _sqlite3SkipAccumulatorLoad($17);
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 if (!($16)) {
  $39 = (_sqlite3_context_db_handle($18)|0);
  $40 = HEAP32[$pBest>>2]|0;
  $41 = ((($40)) + 32|0);
  HEAP32[$41>>2] = $39;
  $42 = HEAP32[$pBest>>2]|0;
  $43 = HEAP32[$pArg>>2]|0;
  (_sqlite3VdbeMemCopy($42,$43)|0);
  STACKTOP = sp;return;
 }
 $19 = (_sqlite3GetFuncCollSeq($18)|0);
 HEAP32[$pColl>>2] = $19;
 $20 = HEAP32[$0>>2]|0;
 $21 = (_sqlite3_user_data($20)|0);
 $22 = ($21|0)!=(0|0);
 $23 = $22&1;
 HEAP32[$max>>2] = $23;
 $24 = HEAP32[$pBest>>2]|0;
 $25 = HEAP32[$pArg>>2]|0;
 $26 = HEAP32[$pColl>>2]|0;
 $27 = (_sqlite3MemCompare($24,$25,$26)|0);
 HEAP32[$cmp>>2] = $27;
 $28 = HEAP32[$max>>2]|0;
 $29 = ($28|0)!=(0);
 $30 = HEAP32[$cmp>>2]|0;
 $31 = ($30|0)<(0);
 $or$cond = $29 & $31;
 if (!($or$cond)) {
  $32 = HEAP32[$max>>2]|0;
  $33 = ($32|0)==(0);
  $34 = HEAP32[$cmp>>2]|0;
  $35 = ($34|0)>(0);
  $or$cond3 = $33 & $35;
  if (!($or$cond3)) {
   $38 = HEAP32[$0>>2]|0;
   _sqlite3SkipAccumulatorLoad($38);
   STACKTOP = sp;return;
  }
 }
 $36 = HEAP32[$pBest>>2]|0;
 $37 = HEAP32[$pArg>>2]|0;
 (_sqlite3VdbeMemCopy($36,$37)|0);
 STACKTOP = sp;return;
}
function _minMaxFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pRes = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pRes = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$pRes>>2] = $2;
 $3 = HEAP32[$pRes>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$pRes>>2]|0;
 $6 = ((($5)) + 8|0);
 $7 = HEAP16[$6>>1]|0;
 $8 = ($7<<16>>16)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = HEAP32[$pRes>>2]|0;
  _sqlite3_result_value($9,$10);
 }
 $11 = HEAP32[$pRes>>2]|0;
 _sqlite3VdbeMemRelease($11);
 STACKTOP = sp;return;
}
function _typeofFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 HEAP32[$z>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 1:  {
  HEAP32[$z>>2] = 39710;
  break;
 }
 case 3:  {
  HEAP32[$z>>2] = 45210;
  break;
 }
 case 2:  {
  HEAP32[$z>>2] = 39705;
  break;
 }
 case 4:  {
  HEAP32[$z>>2] = 45215;
  break;
 }
 default: {
  HEAP32[$z>>2] = 39718;
 }
 }
 $6 = HEAP32[$0>>2]|0;
 $7 = HEAP32[$z>>2]|0;
 _sqlite3_result_text($6,$7,-1,0);
 STACKTOP = sp;return;
}
function _lengthFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $len = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $len = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 2: case 1: case 4:  {
  $6 = HEAP32[$0>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_bytes($8)|0);
  _sqlite3_result_int($6,$9);
  STACKTOP = sp;return;
  break;
 }
 case 3:  {
  $10 = HEAP32[$2>>2]|0;
  $11 = HEAP32[$10>>2]|0;
  $12 = (_sqlite3_value_text($11)|0);
  HEAP32[$z>>2] = $12;
  $13 = HEAP32[$z>>2]|0;
  $14 = ($13|0)==(0|0);
  if ($14) {
   STACKTOP = sp;return;
  }
  HEAP32[$len>>2] = 0;
  L10: while(1) {
   $15 = HEAP32[$z>>2]|0;
   $16 = HEAP8[$15>>0]|0;
   $17 = ($16<<24>>24)!=(0);
   if (!($17)) {
    break;
   }
   $18 = HEAP32[$len>>2]|0;
   $19 = (($18) + 1)|0;
   HEAP32[$len>>2] = $19;
   $20 = HEAP32[$z>>2]|0;
   $21 = ((($20)) + 1|0);
   HEAP32[$z>>2] = $21;
   $22 = HEAP8[$20>>0]|0;
   $23 = $22&255;
   $24 = ($23|0)>=(192);
   if (!($24)) {
    continue;
   }
   while(1) {
    $25 = HEAP32[$z>>2]|0;
    $26 = HEAP8[$25>>0]|0;
    $27 = $26&255;
    $28 = $27 & 192;
    $29 = ($28|0)==(128);
    if (!($29)) {
     continue L10;
    }
    $30 = HEAP32[$z>>2]|0;
    $31 = ((($30)) + 1|0);
    HEAP32[$z>>2] = $31;
   }
  }
  $32 = HEAP32[$0>>2]|0;
  $33 = HEAP32[$len>>2]|0;
  _sqlite3_result_int($32,$33);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $34 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($34);
  STACKTOP = sp;return;
 }
 }
}
function _instrFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $N = 0, $isText = 0, $nHaystack = 0, $nNeedle = 0, $or$cond = 0, $or$cond3 = 0, $typeHaystack = 0, $typeNeedle = 0, $zHaystack = 0, $zNeedle = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $zHaystack = sp + 28|0;
 $zNeedle = sp + 24|0;
 $nHaystack = sp + 20|0;
 $nNeedle = sp + 16|0;
 $typeHaystack = sp + 12|0;
 $typeNeedle = sp + 8|0;
 $N = sp + 4|0;
 $isText = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$N>>2] = 1;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 HEAP32[$typeHaystack>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_type($8)|0);
 HEAP32[$typeNeedle>>2] = $9;
 $10 = HEAP32[$typeHaystack>>2]|0;
 $11 = ($10|0)==(5);
 $12 = HEAP32[$typeNeedle>>2]|0;
 $13 = ($12|0)==(5);
 $or$cond = $11 | $13;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 $16 = (_sqlite3_value_bytes($15)|0);
 HEAP32[$nHaystack>>2] = $16;
 $17 = HEAP32[$2>>2]|0;
 $18 = ((($17)) + 4|0);
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3_value_bytes($19)|0);
 HEAP32[$nNeedle>>2] = $20;
 $21 = HEAP32[$typeHaystack>>2]|0;
 $22 = ($21|0)==(4);
 $23 = HEAP32[$typeNeedle>>2]|0;
 $24 = ($23|0)==(4);
 $or$cond3 = $22 & $24;
 $25 = HEAP32[$2>>2]|0;
 $26 = HEAP32[$25>>2]|0;
 if ($or$cond3) {
  $27 = (_sqlite3_value_blob($26)|0);
  HEAP32[$zHaystack>>2] = $27;
  $28 = HEAP32[$2>>2]|0;
  $29 = ((($28)) + 4|0);
  $30 = HEAP32[$29>>2]|0;
  $31 = (_sqlite3_value_blob($30)|0);
  HEAP32[$zNeedle>>2] = $31;
  HEAP32[$isText>>2] = 0;
 } else {
  $32 = (_sqlite3_value_text($26)|0);
  HEAP32[$zHaystack>>2] = $32;
  $33 = HEAP32[$2>>2]|0;
  $34 = ((($33)) + 4|0);
  $35 = HEAP32[$34>>2]|0;
  $36 = (_sqlite3_value_text($35)|0);
  HEAP32[$zNeedle>>2] = $36;
  HEAP32[$isText>>2] = 1;
 }
 L7: while(1) {
  $37 = HEAP32[$nNeedle>>2]|0;
  $38 = HEAP32[$nHaystack>>2]|0;
  $39 = ($37|0)<=($38|0);
  if (!($39)) {
   break;
  }
  $40 = HEAP32[$zHaystack>>2]|0;
  $41 = HEAP32[$zNeedle>>2]|0;
  $42 = HEAP32[$nNeedle>>2]|0;
  $43 = (_memcmp($40,$41,$42)|0);
  $44 = ($43|0)!=(0);
  if (!($44)) {
   break;
  }
  $45 = HEAP32[$N>>2]|0;
  $46 = (($45) + 1)|0;
  HEAP32[$N>>2] = $46;
  while(1) {
   $47 = HEAP32[$nHaystack>>2]|0;
   $48 = (($47) + -1)|0;
   HEAP32[$nHaystack>>2] = $48;
   $49 = HEAP32[$zHaystack>>2]|0;
   $50 = ((($49)) + 1|0);
   HEAP32[$zHaystack>>2] = $50;
   $51 = HEAP32[$isText>>2]|0;
   $52 = ($51|0)!=(0);
   if (!($52)) {
    continue L7;
   }
   $53 = HEAP32[$zHaystack>>2]|0;
   $54 = HEAP8[$53>>0]|0;
   $55 = $54&255;
   $56 = $55 & 192;
   $57 = ($56|0)==(128);
   if (!($57)) {
    continue L7;
   }
  }
 }
 $58 = HEAP32[$nNeedle>>2]|0;
 $59 = HEAP32[$nHaystack>>2]|0;
 $60 = ($58|0)>($59|0);
 if ($60) {
  HEAP32[$N>>2] = 0;
 }
 $61 = HEAP32[$0>>2]|0;
 $62 = HEAP32[$N>>2]|0;
 _sqlite3_result_int($61,$62);
 STACKTOP = sp;return;
}
function _printfFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $n = 0, $str = 0, $vararg_buffer = 0, $x = 0, $zFormat = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $x = sp + 44|0;
 $str = sp + 16|0;
 $zFormat = sp + 12|0;
 $n = sp + 8|0;
 $db = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)>=(1);
 if (!($6)) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$2>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zFormat>>2] = $9;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) - 1)|0;
 HEAP32[$x>>2] = $12;
 $13 = ((($x)) + 4|0);
 HEAP32[$13>>2] = 0;
 $14 = HEAP32[$2>>2]|0;
 $15 = ((($14)) + 4|0);
 $16 = ((($x)) + 8|0);
 HEAP32[$16>>2] = $15;
 $17 = HEAP32[$db>>2]|0;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 96|0);
 $20 = HEAP32[$19>>2]|0;
 _sqlite3StrAccumInit($str,$17,0,0,$20);
 $21 = ((($str)) + 25|0);
 HEAP8[$21>>0] = 2;
 $22 = HEAP32[$zFormat>>2]|0;
 HEAP32[$vararg_buffer>>2] = $x;
 _sqlite3XPrintf($str,$22,$vararg_buffer);
 $23 = ((($str)) + 12|0);
 $24 = HEAP32[$23>>2]|0;
 HEAP32[$n>>2] = $24;
 $25 = HEAP32[$0>>2]|0;
 $26 = (_sqlite3StrAccumFinish($str)|0);
 $27 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($25,$26,$27,170);
 STACKTOP = sp;return;
}
function _unicodeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z>>2] = $5;
 $6 = HEAP32[$z>>2]|0;
 $7 = ($6|0)!=(0|0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$z>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9&255;
 $11 = ($10|0)!=(0);
 if (!($11)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$0>>2]|0;
 $13 = (_sqlite3Utf8Read($z)|0);
 _sqlite3_result_int($12,$13);
 STACKTOP = sp;return;
}
function _charFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0;
 var $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $c = 0, $i = 0, $or$cond = 0, $x = 0, $z = 0, $zOut = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $z = sp + 20|0;
 $zOut = sp + 16|0;
 $i = sp + 12|0;
 $x = sp;
 $c = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$1>>2]|0;
 $4 = $3<<2;
 $5 = (($4) + 1)|0;
 $6 = ($5|0)<(0);
 $7 = $6 << 31 >> 31;
 $8 = (_sqlite3_malloc64($5,$7)|0);
 HEAP32[$z>>2] = $8;
 HEAP32[$zOut>>2] = $8;
 $9 = HEAP32[$z>>2]|0;
 $10 = ($9|0)==(0|0);
 if ($10) {
  $11 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($11);
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $12 = HEAP32[$i>>2]|0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ($12|0)<($13|0);
  if (!($14)) {
   break;
  }
  $15 = HEAP32[$i>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = (($16) + ($15<<2)|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = (_sqlite3_value_int64($18)|0);
  $20 = tempRet0;
  $21 = $x;
  $22 = $21;
  HEAP32[$22>>2] = $19;
  $23 = (($21) + 4)|0;
  $24 = $23;
  HEAP32[$24>>2] = $20;
  $25 = $x;
  $26 = $25;
  $27 = HEAP32[$26>>2]|0;
  $28 = (($25) + 4)|0;
  $29 = $28;
  $30 = HEAP32[$29>>2]|0;
  $31 = ($30|0)<(0);
  $32 = $x;
  $33 = $32;
  $34 = HEAP32[$33>>2]|0;
  $35 = (($32) + 4)|0;
  $36 = $35;
  $37 = HEAP32[$36>>2]|0;
  $38 = ($37|0)>(0);
  $39 = ($34>>>0)>(1114111);
  $40 = ($37|0)==(0);
  $41 = $40 & $39;
  $42 = $38 | $41;
  $or$cond = $31 | $42;
  $43 = $or$cond ? 65533 : $19;
  $44 = $or$cond ? 0 : $20;
  $45 = $x;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  $49 = $x;
  $50 = $49;
  $51 = HEAP32[$50>>2]|0;
  $52 = (($49) + 4)|0;
  $53 = $52;
  $54 = HEAP32[$53>>2]|0;
  $55 = $51 & 2097151;
  HEAP32[$c>>2] = $55;
  $56 = HEAP32[$c>>2]|0;
  $57 = ($56>>>0)<(128);
  $58 = HEAP32[$c>>2]|0;
  do {
   if ($57) {
    $59 = $58 & 255;
    $60 = $59&255;
    $61 = HEAP32[$zOut>>2]|0;
    $62 = ((($61)) + 1|0);
    HEAP32[$zOut>>2] = $62;
    HEAP8[$61>>0] = $60;
   } else {
    $63 = ($58>>>0)<(2048);
    $64 = HEAP32[$c>>2]|0;
    if ($63) {
     $65 = $64 >>> 6;
     $66 = $65 & 31;
     $67 = $66&255;
     $68 = $67&255;
     $69 = (192 + ($68))|0;
     $70 = $69&255;
     $71 = HEAP32[$zOut>>2]|0;
     $72 = ((($71)) + 1|0);
     HEAP32[$zOut>>2] = $72;
     HEAP8[$71>>0] = $70;
     $73 = HEAP32[$c>>2]|0;
     $74 = $73 & 63;
     $75 = $74&255;
     $76 = $75&255;
     $77 = (128 + ($76))|0;
     $78 = $77&255;
     $79 = HEAP32[$zOut>>2]|0;
     $80 = ((($79)) + 1|0);
     HEAP32[$zOut>>2] = $80;
     HEAP8[$79>>0] = $78;
     break;
    }
    $81 = ($64>>>0)<(65536);
    $82 = HEAP32[$c>>2]|0;
    if ($81) {
     $83 = $82 >>> 12;
     $84 = $83 & 15;
     $85 = $84&255;
     $86 = $85&255;
     $87 = (224 + ($86))|0;
     $88 = $87&255;
     $89 = HEAP32[$zOut>>2]|0;
     $90 = ((($89)) + 1|0);
     HEAP32[$zOut>>2] = $90;
     HEAP8[$89>>0] = $88;
     $91 = HEAP32[$c>>2]|0;
     $92 = $91 >>> 6;
     $93 = $92 & 63;
     $94 = $93&255;
     $95 = $94&255;
     $96 = (128 + ($95))|0;
     $97 = $96&255;
     $98 = HEAP32[$zOut>>2]|0;
     $99 = ((($98)) + 1|0);
     HEAP32[$zOut>>2] = $99;
     HEAP8[$98>>0] = $97;
     $100 = HEAP32[$c>>2]|0;
     $101 = $100 & 63;
     $102 = $101&255;
     $103 = $102&255;
     $104 = (128 + ($103))|0;
     $105 = $104&255;
     $106 = HEAP32[$zOut>>2]|0;
     $107 = ((($106)) + 1|0);
     HEAP32[$zOut>>2] = $107;
     HEAP8[$106>>0] = $105;
     break;
    } else {
     $108 = $82 >>> 18;
     $109 = $108 & 7;
     $110 = $109&255;
     $111 = $110&255;
     $112 = (240 + ($111))|0;
     $113 = $112&255;
     $114 = HEAP32[$zOut>>2]|0;
     $115 = ((($114)) + 1|0);
     HEAP32[$zOut>>2] = $115;
     HEAP8[$114>>0] = $113;
     $116 = HEAP32[$c>>2]|0;
     $117 = $116 >>> 12;
     $118 = $117 & 63;
     $119 = $118&255;
     $120 = $119&255;
     $121 = (128 + ($120))|0;
     $122 = $121&255;
     $123 = HEAP32[$zOut>>2]|0;
     $124 = ((($123)) + 1|0);
     HEAP32[$zOut>>2] = $124;
     HEAP8[$123>>0] = $122;
     $125 = HEAP32[$c>>2]|0;
     $126 = $125 >>> 6;
     $127 = $126 & 63;
     $128 = $127&255;
     $129 = $128&255;
     $130 = (128 + ($129))|0;
     $131 = $130&255;
     $132 = HEAP32[$zOut>>2]|0;
     $133 = ((($132)) + 1|0);
     HEAP32[$zOut>>2] = $133;
     HEAP8[$132>>0] = $131;
     $134 = HEAP32[$c>>2]|0;
     $135 = $134 & 63;
     $136 = $135&255;
     $137 = $136&255;
     $138 = (128 + ($137))|0;
     $139 = $138&255;
     $140 = HEAP32[$zOut>>2]|0;
     $141 = ((($140)) + 1|0);
     HEAP32[$zOut>>2] = $141;
     HEAP8[$140>>0] = $139;
     break;
    }
   }
  } while(0);
  $142 = HEAP32[$i>>2]|0;
  $143 = (($142) + 1)|0;
  HEAP32[$i>>2] = $143;
 }
 $144 = HEAP32[$0>>2]|0;
 $145 = HEAP32[$z>>2]|0;
 $146 = HEAP32[$zOut>>2]|0;
 $147 = HEAP32[$z>>2]|0;
 $148 = $146;
 $149 = $147;
 $150 = (($148) - ($149))|0;
 $151 = ($150|0)<(0);
 $152 = $151 << 31 >> 31;
 _sqlite3_result_text64($144,$145,$150,$152,148,1);
 STACKTOP = sp;return;
}
function _absFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0.0, $54 = 0.0, $55 = 0, $56 = 0.0, $57 = 0.0, $58 = 0, $59 = 0.0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $iVal = 0, $rVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $iVal = sp + 8|0;
 $rVal = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 1:  {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (_sqlite3_value_int64($7)|0);
  $9 = tempRet0;
  $10 = $iVal;
  $11 = $10;
  HEAP32[$11>>2] = $8;
  $12 = (($10) + 4)|0;
  $13 = $12;
  HEAP32[$13>>2] = $9;
  $14 = $iVal;
  $15 = $14;
  $16 = HEAP32[$15>>2]|0;
  $17 = (($14) + 4)|0;
  $18 = $17;
  $19 = HEAP32[$18>>2]|0;
  $20 = ($19|0)<(0);
  do {
   if ($20) {
    $21 = $iVal;
    $22 = $21;
    $23 = HEAP32[$22>>2]|0;
    $24 = (($21) + 4)|0;
    $25 = $24;
    $26 = HEAP32[$25>>2]|0;
    $27 = ($23|0)==(0);
    $28 = ($26|0)==(-2147483648);
    $29 = $27 & $28;
    if (!($29)) {
     $31 = $iVal;
     $32 = $31;
     $33 = HEAP32[$32>>2]|0;
     $34 = (($31) + 4)|0;
     $35 = $34;
     $36 = HEAP32[$35>>2]|0;
     $37 = (_i64Subtract(0,0,($33|0),($36|0))|0);
     $38 = tempRet0;
     $39 = $iVal;
     $40 = $39;
     HEAP32[$40>>2] = $37;
     $41 = (($39) + 4)|0;
     $42 = $41;
     HEAP32[$42>>2] = $38;
     break;
    }
    $30 = HEAP32[$0>>2]|0;
    _sqlite3_result_error($30,45165,-1);
    STACKTOP = sp;return;
   }
  } while(0);
  $43 = HEAP32[$0>>2]|0;
  $44 = $iVal;
  $45 = $44;
  $46 = HEAP32[$45>>2]|0;
  $47 = (($44) + 4)|0;
  $48 = $47;
  $49 = HEAP32[$48>>2]|0;
  _sqlite3_result_int64($43,$46,$49);
  STACKTOP = sp;return;
  break;
 }
 case 5:  {
  $50 = HEAP32[$0>>2]|0;
  _sqlite3_result_null($50);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $51 = HEAP32[$2>>2]|0;
  $52 = HEAP32[$51>>2]|0;
  $53 = (+_sqlite3_value_double($52));
  HEAPF64[$rVal>>3] = $53;
  $54 = +HEAPF64[$rVal>>3];
  $55 = $54 < 0.0;
  if ($55) {
   $56 = +HEAPF64[$rVal>>3];
   $57 = -$56;
   HEAPF64[$rVal>>3] = $57;
  }
  $58 = HEAP32[$0>>2]|0;
  $59 = +HEAPF64[$rVal>>3];
  _sqlite3_result_double($58,$59);
  STACKTOP = sp;return;
 }
 }
}
function _roundFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $$$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0;
 var $25 = 0, $26 = 0, $27 = 0.0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0.0, $32 = 0.0, $33 = 0, $34 = 0, $35 = 0.0, $36 = 0, $37 = 0, $38 = 0.0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0;
 var $43 = 0.0, $44 = 0.0, $45 = 0.0, $46 = 0, $47 = 0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0;
 var $61 = 0.0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond5 = 0, $r = 0, $vararg_buffer = 0, $vararg_ptr6 = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp + 8|0;
 $0 = sp + 40|0;
 $1 = sp + 36|0;
 $2 = sp + 32|0;
 $n = sp + 28|0;
 $r = sp;
 $zBuf = sp + 24|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$n>>2] = 0;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(2);
 do {
  if ($4) {
   $5 = HEAP32[$2>>2]|0;
   $6 = ((($5)) + 4|0);
   $7 = HEAP32[$6>>2]|0;
   $8 = (_sqlite3_value_type($7)|0);
   $9 = (5)==($8|0);
   if ($9) {
    STACKTOP = sp;return;
   } else {
    $10 = HEAP32[$2>>2]|0;
    $11 = ((($10)) + 4|0);
    $12 = HEAP32[$11>>2]|0;
    $13 = (_sqlite3_value_int($12)|0);
    HEAP32[$n>>2] = $13;
    $14 = HEAP32[$n>>2]|0;
    $15 = ($14|0)>(30);
    $$ = $15 ? 30 : $13;
    HEAP32[$n>>2] = $$;
    $16 = HEAP32[$n>>2]|0;
    $17 = ($16|0)<(0);
    $$$ = $17 ? 0 : $$;
    HEAP32[$n>>2] = $$$;
    break;
   }
  }
 } while(0);
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_sqlite3_value_type($19)|0);
 $21 = ($20|0)==(5);
 if ($21) {
  STACKTOP = sp;return;
 }
 $22 = HEAP32[$2>>2]|0;
 $23 = HEAP32[$22>>2]|0;
 $24 = (+_sqlite3_value_double($23));
 HEAPF64[$r>>3] = $24;
 $25 = HEAP32[$n>>2]|0;
 $26 = ($25|0)==(0);
 $27 = +HEAPF64[$r>>3];
 $28 = $27 >= 0.0;
 $or$cond = $26 & $28;
 $29 = +HEAPF64[$r>>3];
 $30 = $29 < 9.2233720368547758E+18;
 $or$cond3 = $or$cond & $30;
 do {
  if ($or$cond3) {
   $31 = +HEAPF64[$r>>3];
   $32 = $31 + 0.5;
   $33 = (~~$32)>>>0;
   $34 = +Math_abs($32) >= 1.0 ? $32 > 0.0 ? (~~+Math_min(+Math_floor($32 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($32 - +(~~$32 >>> 0)) / 4294967296.0) >>> 0 : 0;
   $35 = (+($33>>>0)) + (4294967296.0*(+($34|0)));
   HEAPF64[$r>>3] = $35;
  } else {
   $36 = HEAP32[$n>>2]|0;
   $37 = ($36|0)==(0);
   $38 = +HEAPF64[$r>>3];
   $39 = $38 < 0.0;
   $or$cond5 = $37 & $39;
   if ($or$cond5) {
    $40 = +HEAPF64[$r>>3];
    $41 = -$40;
    $42 = $41 < 9.2233720368547758E+18;
    if ($42) {
     $43 = +HEAPF64[$r>>3];
     $44 = -$43;
     $45 = $44 + 0.5;
     $46 = (~~$45)>>>0;
     $47 = +Math_abs($45) >= 1.0 ? $45 > 0.0 ? (~~+Math_min(+Math_floor($45 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($45 - +(~~$45 >>> 0)) / 4294967296.0) >>> 0 : 0;
     $48 = (+($46>>>0)) + (4294967296.0*(+($47|0)));
     $49 = -$48;
     HEAPF64[$r>>3] = $49;
     break;
    }
   }
   $50 = HEAP32[$n>>2]|0;
   $51 = +HEAPF64[$r>>3];
   HEAP32[$vararg_buffer>>2] = $50;
   $vararg_ptr6 = ((($vararg_buffer)) + 8|0);
   HEAPF64[$vararg_ptr6>>3] = $51;
   $52 = (_sqlite3_mprintf(45205,$vararg_buffer)|0);
   HEAP32[$zBuf>>2] = $52;
   $53 = HEAP32[$zBuf>>2]|0;
   $54 = ($53|0)==(0|0);
   if (!($54)) {
    $56 = HEAP32[$zBuf>>2]|0;
    $57 = HEAP32[$zBuf>>2]|0;
    $58 = (_sqlite3Strlen30($57)|0);
    (_sqlite3AtoF($56,$r,$58,1)|0);
    $59 = HEAP32[$zBuf>>2]|0;
    _sqlite3_free($59);
    break;
   }
   $55 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_nomem($55);
   STACKTOP = sp;return;
  }
 } while(0);
 $60 = HEAP32[$0>>2]|0;
 $61 = +HEAPF64[$r>>3];
 _sqlite3_result_double($60,$61);
 STACKTOP = sp;return;
}
function _upperFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0, $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z1 = sp + 12|0;
 $z2 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$z2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$n>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_i64Add(($12|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($11,$15,$16)|0);
 HEAP32[$z1>>2] = $17;
 $18 = HEAP32[$z1>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$z2>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26 << 24 >> 24;
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$z2>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31&255;
  $33 = (18403 + ($32)|0);
  $34 = HEAP8[$33>>0]|0;
  $35 = $34&255;
  $36 = $35 & 32;
  $37 = $36 ^ -1;
  $38 = $27 & $37;
  $39 = $38&255;
  $40 = HEAP32[$i>>2]|0;
  $41 = HEAP32[$z1>>2]|0;
  $42 = (($41) + ($40)|0);
  HEAP8[$42>>0] = $39;
  $43 = HEAP32[$i>>2]|0;
  $44 = (($43) + 1)|0;
  HEAP32[$i>>2] = $44;
 }
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$z1>>2]|0;
 $47 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($45,$46,$47,148);
 STACKTOP = sp;return;
}
function _lowerFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, $n = 0;
 var $z1 = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $z1 = sp + 12|0;
 $z2 = sp + 8|0;
 $i = sp + 4|0;
 $n = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$z2>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$z2>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = HEAP32[$n>>2]|0;
 $13 = ($12|0)<(0);
 $14 = $13 << 31 >> 31;
 $15 = (_i64Add(($12|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($11,$15,$16)|0);
 HEAP32[$z1>>2] = $17;
 $18 = HEAP32[$z1>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$z2>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = $26&255;
  $28 = (17613 + ($27)|0);
  $29 = HEAP8[$28>>0]|0;
  $30 = HEAP32[$i>>2]|0;
  $31 = HEAP32[$z1>>2]|0;
  $32 = (($31) + ($30)|0);
  HEAP8[$32>>0] = $29;
  $33 = HEAP32[$i>>2]|0;
  $34 = (($33) + 1)|0;
  HEAP32[$i>>2] = $34;
 }
 $35 = HEAP32[$0>>2]|0;
 $36 = HEAP32[$z1>>2]|0;
 $37 = HEAP32[$n>>2]|0;
 _sqlite3_result_text($35,$36,$37,148);
 STACKTOP = sp;return;
}
function _hexFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $i = 0, $n = 0, $pBlob = 0, $z = 0, $zHex = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $i = sp + 16|0;
 $n = sp + 12|0;
 $pBlob = sp + 8|0;
 $zHex = sp + 4|0;
 $z = sp;
 $c = sp + 32|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_blob($4)|0);
 HEAP32[$pBlob>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = HEAP32[$6>>2]|0;
 $8 = (_sqlite3_value_bytes($7)|0);
 HEAP32[$n>>2] = $8;
 $9 = HEAP32[$0>>2]|0;
 $10 = HEAP32[$n>>2]|0;
 $11 = ($10|0)<(0);
 $12 = $11 << 31 >> 31;
 $13 = (___muldi3(($10|0),($12|0),2,0)|0);
 $14 = tempRet0;
 $15 = (_i64Add(($13|0),($14|0),1,0)|0);
 $16 = tempRet0;
 $17 = (_contextMalloc($9,$15,$16)|0);
 HEAP32[$zHex>>2] = $17;
 HEAP32[$z>>2] = $17;
 $18 = HEAP32[$zHex>>2]|0;
 $19 = ($18|0)!=(0|0);
 if (!($19)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $20 = HEAP32[$i>>2]|0;
  $21 = HEAP32[$n>>2]|0;
  $22 = ($20|0)<($21|0);
  if (!($22)) {
   break;
  }
  $23 = HEAP32[$pBlob>>2]|0;
  $24 = HEAP8[$23>>0]|0;
  HEAP8[$c>>0] = $24;
  $25 = HEAP8[$c>>0]|0;
  $26 = $25&255;
  $27 = $26 >> 4;
  $28 = $27 & 15;
  $29 = (45189 + ($28)|0);
  $30 = HEAP8[$29>>0]|0;
  $31 = HEAP32[$z>>2]|0;
  $32 = ((($31)) + 1|0);
  HEAP32[$z>>2] = $32;
  HEAP8[$31>>0] = $30;
  $33 = HEAP8[$c>>0]|0;
  $34 = $33&255;
  $35 = $34 & 15;
  $36 = (45189 + ($35)|0);
  $37 = HEAP8[$36>>0]|0;
  $38 = HEAP32[$z>>2]|0;
  $39 = ((($38)) + 1|0);
  HEAP32[$z>>2] = $39;
  HEAP8[$38>>0] = $37;
  $40 = HEAP32[$i>>2]|0;
  $41 = (($40) + 1)|0;
  HEAP32[$i>>2] = $41;
  $42 = HEAP32[$pBlob>>2]|0;
  $43 = ((($42)) + 1|0);
  HEAP32[$pBlob>>2] = $43;
 }
 $44 = HEAP32[$z>>2]|0;
 HEAP8[$44>>0] = 0;
 $45 = HEAP32[$0>>2]|0;
 $46 = HEAP32[$zHex>>2]|0;
 $47 = HEAP32[$n>>2]|0;
 $48 = $47<<1;
 _sqlite3_result_text($45,$46,$48,148);
 STACKTOP = sp;return;
}
function _randomFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $r = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 _sqlite3_randomness(8,$r);
 $3 = $r;
 $4 = $3;
 $5 = HEAP32[$4>>2]|0;
 $6 = (($3) + 4)|0;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = ($8|0)<(0);
 if ($9) {
  $10 = $r;
  $11 = $10;
  $12 = HEAP32[$11>>2]|0;
  $13 = (($10) + 4)|0;
  $14 = $13;
  $15 = HEAP32[$14>>2]|0;
  $16 = $15 & 2147483647;
  $17 = (_i64Subtract(0,0,($12|0),($16|0))|0);
  $18 = tempRet0;
  $19 = $r;
  $20 = $19;
  HEAP32[$20>>2] = $17;
  $21 = (($19) + 4)|0;
  $22 = $21;
  HEAP32[$22>>2] = $18;
 }
 $23 = HEAP32[$0>>2]|0;
 $24 = $r;
 $25 = $24;
 $26 = HEAP32[$25>>2]|0;
 $27 = (($24) + 4)|0;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 _sqlite3_result_int64($23,$26,$29);
 STACKTOP = sp;return;
}
function _randomBlob($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $n = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $2 = sp + 8|0;
 $n = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 HEAP32[$n>>2] = $5;
 $6 = HEAP32[$n>>2]|0;
 $7 = ($6|0)<(1);
 $$ = $7 ? 1 : $5;
 HEAP32[$n>>2] = $$;
 $8 = HEAP32[$0>>2]|0;
 $9 = HEAP32[$n>>2]|0;
 $10 = ($9|0)<(0);
 $11 = $10 << 31 >> 31;
 $12 = (_contextMalloc($8,$9,$11)|0);
 HEAP32[$p>>2] = $12;
 $13 = HEAP32[$p>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 $15 = HEAP32[$n>>2]|0;
 $16 = HEAP32[$p>>2]|0;
 _sqlite3_randomness($15,$16);
 $17 = HEAP32[$0>>2]|0;
 $18 = HEAP32[$p>>2]|0;
 $19 = HEAP32[$n>>2]|0;
 _sqlite3_result_blob($17,$18,$19,148);
 STACKTOP = sp;return;
}
function _nullifFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pColl = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pColl = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3GetFuncCollSeq($3)|0);
 HEAP32[$pColl>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = HEAP32[$2>>2]|0;
 $8 = ((($7)) + 4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = HEAP32[$pColl>>2]|0;
 $11 = (_sqlite3MemCompare($6,$9,$10)|0);
 $12 = ($11|0)!=(0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 $13 = HEAP32[$0>>2]|0;
 $14 = HEAP32[$2>>2]|0;
 $15 = HEAP32[$14>>2]|0;
 _sqlite3_result_value($13,$15);
 STACKTOP = sp;return;
}
function _sourceidFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_sourceid()|0);
 _sqlite3_result_text($3,$4,-1,0);
 STACKTOP = sp;return;
}
function _errlogFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int($4)|0);
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$vararg_buffer>>2] = $9;
 _sqlite3_log($5,18391,$vararg_buffer);
 STACKTOP = sp;return;
}
function _quoteFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0.0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0.0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0.0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0;
 var $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0;
 var $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0;
 var $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0.0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0;
 var $9 = 0.0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $i = 0, $i1 = 0, $j = 0, $n = 0, $nBlob = 0, $r1 = 0, $r2 = 0, $vararg_buffer = 0, $vararg_buffer1 = 0;
 var $z = 0, $zArg = 0, $zBlob = 0, $zBuf = 0, $zText = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 144|0;
 $vararg_buffer1 = sp + 32|0;
 $vararg_buffer = sp + 24|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $r1 = sp + 16|0;
 $r2 = sp + 8|0;
 $zBuf = sp + 84|0;
 $zText = sp + 68|0;
 $zBlob = sp + 64|0;
 $nBlob = sp + 60|0;
 $i = sp + 56|0;
 $i1 = sp + 52|0;
 $j = sp + 48|0;
 $n = sp;
 $zArg = sp + 44|0;
 $z = sp + 40|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 switch ($5|0) {
 case 2:  {
  $6 = HEAP32[$2>>2]|0;
  $7 = HEAP32[$6>>2]|0;
  $8 = (+_sqlite3_value_double($7));
  HEAPF64[$r1>>3] = $8;
  $9 = +HEAPF64[$r1>>3];
  HEAPF64[$vararg_buffer>>3] = $9;
  (_sqlite3_snprintf(50,$zBuf,28271,$vararg_buffer)|0);
  (_sqlite3AtoF($zBuf,$r2,20,1)|0);
  $10 = +HEAPF64[$r1>>3];
  $11 = +HEAPF64[$r2>>3];
  $12 = $10 != $11;
  if ($12) {
   $13 = +HEAPF64[$r1>>3];
   HEAPF64[$vararg_buffer1>>3] = $13;
   (_sqlite3_snprintf(50,$zBuf,45182,$vararg_buffer1)|0);
  }
  $14 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($14,$zBuf,-1,(-1));
  STACKTOP = sp;return;
  break;
 }
 case 1:  {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$2>>2]|0;
  $17 = HEAP32[$16>>2]|0;
  _sqlite3_result_value($15,$17);
  STACKTOP = sp;return;
  break;
 }
 case 4:  {
  HEAP32[$zText>>2] = 0;
  $18 = HEAP32[$2>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = (_sqlite3_value_blob($19)|0);
  HEAP32[$zBlob>>2] = $20;
  $21 = HEAP32[$2>>2]|0;
  $22 = HEAP32[$21>>2]|0;
  $23 = (_sqlite3_value_bytes($22)|0);
  HEAP32[$nBlob>>2] = $23;
  $24 = HEAP32[$0>>2]|0;
  $25 = HEAP32[$nBlob>>2]|0;
  $26 = ($25|0)<(0);
  $27 = $26 << 31 >> 31;
  $28 = (___muldi3(2,0,($25|0),($27|0))|0);
  $29 = tempRet0;
  $30 = (_i64Add(($28|0),($29|0),4,0)|0);
  $31 = tempRet0;
  $32 = (_contextMalloc($24,$30,$31)|0);
  HEAP32[$zText>>2] = $32;
  $33 = HEAP32[$zText>>2]|0;
  $34 = ($33|0)!=(0|0);
  if (!($34)) {
   STACKTOP = sp;return;
  }
  HEAP32[$i>>2] = 0;
  while(1) {
   $35 = HEAP32[$i>>2]|0;
   $36 = HEAP32[$nBlob>>2]|0;
   $37 = ($35|0)<($36|0);
   if (!($37)) {
    break;
   }
   $38 = HEAP32[$i>>2]|0;
   $39 = HEAP32[$zBlob>>2]|0;
   $40 = (($39) + ($38)|0);
   $41 = HEAP8[$40>>0]|0;
   $42 = $41 << 24 >> 24;
   $43 = $42 >> 4;
   $44 = $43 & 15;
   $45 = (45189 + ($44)|0);
   $46 = HEAP8[$45>>0]|0;
   $47 = HEAP32[$i>>2]|0;
   $48 = $47<<1;
   $49 = (($48) + 2)|0;
   $50 = HEAP32[$zText>>2]|0;
   $51 = (($50) + ($49)|0);
   HEAP8[$51>>0] = $46;
   $52 = HEAP32[$i>>2]|0;
   $53 = HEAP32[$zBlob>>2]|0;
   $54 = (($53) + ($52)|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = $56 & 15;
   $58 = (45189 + ($57)|0);
   $59 = HEAP8[$58>>0]|0;
   $60 = HEAP32[$i>>2]|0;
   $61 = $60<<1;
   $62 = (($61) + 3)|0;
   $63 = HEAP32[$zText>>2]|0;
   $64 = (($63) + ($62)|0);
   HEAP8[$64>>0] = $59;
   $65 = HEAP32[$i>>2]|0;
   $66 = (($65) + 1)|0;
   HEAP32[$i>>2] = $66;
  }
  $67 = HEAP32[$nBlob>>2]|0;
  $68 = $67<<1;
  $69 = (($68) + 2)|0;
  $70 = HEAP32[$zText>>2]|0;
  $71 = (($70) + ($69)|0);
  HEAP8[$71>>0] = 39;
  $72 = HEAP32[$nBlob>>2]|0;
  $73 = $72<<1;
  $74 = (($73) + 3)|0;
  $75 = HEAP32[$zText>>2]|0;
  $76 = (($75) + ($74)|0);
  HEAP8[$76>>0] = 0;
  $77 = HEAP32[$zText>>2]|0;
  HEAP8[$77>>0] = 88;
  $78 = HEAP32[$zText>>2]|0;
  $79 = ((($78)) + 1|0);
  HEAP8[$79>>0] = 39;
  $80 = HEAP32[$0>>2]|0;
  $81 = HEAP32[$zText>>2]|0;
  _sqlite3_result_text($80,$81,-1,(-1));
  $82 = HEAP32[$zText>>2]|0;
  _sqlite3_free($82);
  STACKTOP = sp;return;
  break;
 }
 case 3:  {
  $83 = HEAP32[$2>>2]|0;
  $84 = HEAP32[$83>>2]|0;
  $85 = (_sqlite3_value_text($84)|0);
  HEAP32[$zArg>>2] = $85;
  $86 = HEAP32[$zArg>>2]|0;
  $87 = ($86|0)==(0|0);
  if ($87) {
   STACKTOP = sp;return;
  }
  HEAP32[$i1>>2] = 0;
  $88 = $n;
  $89 = $88;
  HEAP32[$89>>2] = 0;
  $90 = (($88) + 4)|0;
  $91 = $90;
  HEAP32[$91>>2] = 0;
  while(1) {
   $92 = HEAP32[$i1>>2]|0;
   $93 = HEAP32[$zArg>>2]|0;
   $94 = (($93) + ($92)|0);
   $95 = HEAP8[$94>>0]|0;
   $96 = ($95<<24>>24)!=(0);
   if (!($96)) {
    break;
   }
   $97 = HEAP32[$i1>>2]|0;
   $98 = HEAP32[$zArg>>2]|0;
   $99 = (($98) + ($97)|0);
   $100 = HEAP8[$99>>0]|0;
   $101 = $100&255;
   $102 = ($101|0)==(39);
   if ($102) {
    $103 = $n;
    $104 = $103;
    $105 = HEAP32[$104>>2]|0;
    $106 = (($103) + 4)|0;
    $107 = $106;
    $108 = HEAP32[$107>>2]|0;
    $109 = (_i64Add(($105|0),($108|0),1,0)|0);
    $110 = tempRet0;
    $111 = $n;
    $112 = $111;
    HEAP32[$112>>2] = $109;
    $113 = (($111) + 4)|0;
    $114 = $113;
    HEAP32[$114>>2] = $110;
   }
   $115 = HEAP32[$i1>>2]|0;
   $116 = (($115) + 1)|0;
   HEAP32[$i1>>2] = $116;
  }
  $117 = HEAP32[$0>>2]|0;
  $118 = HEAP32[$i1>>2]|0;
  $119 = ($118|0)<(0);
  $120 = $119 << 31 >> 31;
  $121 = $n;
  $122 = $121;
  $123 = HEAP32[$122>>2]|0;
  $124 = (($121) + 4)|0;
  $125 = $124;
  $126 = HEAP32[$125>>2]|0;
  $127 = (_i64Add(($118|0),($120|0),($123|0),($126|0))|0);
  $128 = tempRet0;
  $129 = (_i64Add(($127|0),($128|0),3,0)|0);
  $130 = tempRet0;
  $131 = (_contextMalloc($117,$129,$130)|0);
  HEAP32[$z>>2] = $131;
  $132 = HEAP32[$z>>2]|0;
  $133 = ($132|0)!=(0|0);
  if (!($133)) {
   STACKTOP = sp;return;
  }
  $134 = HEAP32[$z>>2]|0;
  HEAP8[$134>>0] = 39;
  HEAP32[$i1>>2] = 0;
  HEAP32[$j>>2] = 1;
  while(1) {
   $135 = HEAP32[$i1>>2]|0;
   $136 = HEAP32[$zArg>>2]|0;
   $137 = (($136) + ($135)|0);
   $138 = HEAP8[$137>>0]|0;
   $139 = ($138<<24>>24)!=(0);
   if (!($139)) {
    break;
   }
   $140 = HEAP32[$i1>>2]|0;
   $141 = HEAP32[$zArg>>2]|0;
   $142 = (($141) + ($140)|0);
   $143 = HEAP8[$142>>0]|0;
   $144 = HEAP32[$j>>2]|0;
   $145 = (($144) + 1)|0;
   HEAP32[$j>>2] = $145;
   $146 = HEAP32[$z>>2]|0;
   $147 = (($146) + ($144)|0);
   HEAP8[$147>>0] = $143;
   $148 = HEAP32[$i1>>2]|0;
   $149 = HEAP32[$zArg>>2]|0;
   $150 = (($149) + ($148)|0);
   $151 = HEAP8[$150>>0]|0;
   $152 = $151&255;
   $153 = ($152|0)==(39);
   if ($153) {
    $154 = HEAP32[$j>>2]|0;
    $155 = (($154) + 1)|0;
    HEAP32[$j>>2] = $155;
    $156 = HEAP32[$z>>2]|0;
    $157 = (($156) + ($154)|0);
    HEAP8[$157>>0] = 39;
   }
   $158 = HEAP32[$i1>>2]|0;
   $159 = (($158) + 1)|0;
   HEAP32[$i1>>2] = $159;
  }
  $160 = HEAP32[$j>>2]|0;
  $161 = (($160) + 1)|0;
  HEAP32[$j>>2] = $161;
  $162 = HEAP32[$z>>2]|0;
  $163 = (($162) + ($160)|0);
  HEAP8[$163>>0] = 39;
  $164 = HEAP32[$j>>2]|0;
  $165 = HEAP32[$z>>2]|0;
  $166 = (($165) + ($164)|0);
  HEAP8[$166>>0] = 0;
  $167 = HEAP32[$0>>2]|0;
  $168 = HEAP32[$z>>2]|0;
  $169 = HEAP32[$j>>2]|0;
  _sqlite3_result_text($167,$168,$169,148);
  STACKTOP = sp;return;
  break;
 }
 default: {
  $170 = HEAP32[$0>>2]|0;
  _sqlite3_result_text($170,39691,4,0);
  STACKTOP = sp;return;
 }
 }
}
function _last_insert_rowid($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_last_insert_rowid($6)|0);
 $8 = tempRet0;
 _sqlite3_result_int64($5,$7,$8);
 STACKTOP = sp;return;
}
function _changes($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_changes($6)|0);
 _sqlite3_result_int($5,$7);
 STACKTOP = sp;return;
}
function _total_changes($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $db = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $db = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 $5 = HEAP32[$0>>2]|0;
 $6 = HEAP32[$db>>2]|0;
 $7 = (_sqlite3_total_changes($6)|0);
 _sqlite3_result_int($5,$7);
 STACKTOP = sp;return;
}
function _replaceFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0;
 var $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $loopLimit = 0, $nOut = 0, $nPattern = 0;
 var $nRep = 0, $nStr = 0, $zOld = 0, $zOut = 0, $zPattern = 0, $zRep = 0, $zStr = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $0 = sp + 64|0;
 $1 = sp + 60|0;
 $2 = sp + 56|0;
 $zStr = sp + 52|0;
 $zPattern = sp + 48|0;
 $zRep = sp + 44|0;
 $zOut = sp + 40|0;
 $nStr = sp + 36|0;
 $nPattern = sp + 32|0;
 $nRep = sp + 28|0;
 $nOut = sp;
 $loopLimit = sp + 24|0;
 $i = sp + 20|0;
 $j = sp + 16|0;
 $zOld = sp + 12|0;
 $db = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zStr>>2] = $5;
 $6 = HEAP32[$zStr>>2]|0;
 $7 = ($6|0)==(0|0);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$8>>2]|0;
 $10 = (_sqlite3_value_bytes($9)|0);
 HEAP32[$nStr>>2] = $10;
 $11 = HEAP32[$2>>2]|0;
 $12 = ((($11)) + 4|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = (_sqlite3_value_text($13)|0);
 HEAP32[$zPattern>>2] = $14;
 $15 = HEAP32[$zPattern>>2]|0;
 $16 = ($15|0)==(0|0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$zPattern>>2]|0;
 $18 = HEAP8[$17>>0]|0;
 $19 = $18&255;
 $20 = ($19|0)==(0);
 if ($20) {
  $21 = HEAP32[$0>>2]|0;
  $22 = HEAP32[$2>>2]|0;
  $23 = HEAP32[$22>>2]|0;
  _sqlite3_result_value($21,$23);
  STACKTOP = sp;return;
 }
 $24 = HEAP32[$2>>2]|0;
 $25 = ((($24)) + 4|0);
 $26 = HEAP32[$25>>2]|0;
 $27 = (_sqlite3_value_bytes($26)|0);
 HEAP32[$nPattern>>2] = $27;
 $28 = HEAP32[$2>>2]|0;
 $29 = ((($28)) + 8|0);
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3_value_text($30)|0);
 HEAP32[$zRep>>2] = $31;
 $32 = HEAP32[$zRep>>2]|0;
 $33 = ($32|0)==(0|0);
 if ($33) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$2>>2]|0;
 $35 = ((($34)) + 8|0);
 $36 = HEAP32[$35>>2]|0;
 $37 = (_sqlite3_value_bytes($36)|0);
 HEAP32[$nRep>>2] = $37;
 $38 = HEAP32[$nStr>>2]|0;
 $39 = (($38) + 1)|0;
 $40 = ($39|0)<(0);
 $41 = $40 << 31 >> 31;
 $42 = $nOut;
 $43 = $42;
 HEAP32[$43>>2] = $39;
 $44 = (($42) + 4)|0;
 $45 = $44;
 HEAP32[$45>>2] = $41;
 $46 = HEAP32[$0>>2]|0;
 $47 = $nOut;
 $48 = $47;
 $49 = HEAP32[$48>>2]|0;
 $50 = (($47) + 4)|0;
 $51 = $50;
 $52 = HEAP32[$51>>2]|0;
 $53 = (_contextMalloc($46,$49,$52)|0);
 HEAP32[$zOut>>2] = $53;
 $54 = HEAP32[$zOut>>2]|0;
 $55 = ($54|0)==(0|0);
 if ($55) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$nStr>>2]|0;
 $57 = HEAP32[$nPattern>>2]|0;
 $58 = (($56) - ($57))|0;
 HEAP32[$loopLimit>>2] = $58;
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $59 = HEAP32[$i>>2]|0;
  $60 = HEAP32[$loopLimit>>2]|0;
  $61 = ($59|0)<=($60|0);
  if (!($61)) {
   label = 18;
   break;
  }
  $62 = HEAP32[$i>>2]|0;
  $63 = HEAP32[$zStr>>2]|0;
  $64 = (($63) + ($62)|0);
  $65 = HEAP8[$64>>0]|0;
  $66 = $65&255;
  $67 = HEAP32[$zPattern>>2]|0;
  $68 = HEAP8[$67>>0]|0;
  $69 = $68&255;
  $70 = ($66|0)!=($69|0);
  if ($70) {
   label = 11;
  } else {
   $71 = HEAP32[$i>>2]|0;
   $72 = HEAP32[$zStr>>2]|0;
   $73 = (($72) + ($71)|0);
   $74 = HEAP32[$zPattern>>2]|0;
   $75 = HEAP32[$nPattern>>2]|0;
   $76 = (_memcmp($73,$74,$75)|0);
   $77 = ($76|0)!=(0);
   if ($77) {
    label = 11;
   } else {
    $86 = HEAP32[$0>>2]|0;
    $87 = (_sqlite3_context_db_handle($86)|0);
    HEAP32[$db>>2] = $87;
    $88 = HEAP32[$nRep>>2]|0;
    $89 = HEAP32[$nPattern>>2]|0;
    $90 = (($88) - ($89))|0;
    $91 = ($90|0)<(0);
    $92 = $91 << 31 >> 31;
    $93 = $nOut;
    $94 = $93;
    $95 = HEAP32[$94>>2]|0;
    $96 = (($93) + 4)|0;
    $97 = $96;
    $98 = HEAP32[$97>>2]|0;
    $99 = (_i64Add(($95|0),($98|0),($90|0),($92|0))|0);
    $100 = tempRet0;
    $101 = $nOut;
    $102 = $101;
    HEAP32[$102>>2] = $99;
    $103 = (($101) + 4)|0;
    $104 = $103;
    HEAP32[$104>>2] = $100;
    $105 = $nOut;
    $106 = $105;
    $107 = HEAP32[$106>>2]|0;
    $108 = (($105) + 4)|0;
    $109 = $108;
    $110 = HEAP32[$109>>2]|0;
    $111 = (_i64Subtract(($107|0),($110|0),1,0)|0);
    $112 = tempRet0;
    $113 = HEAP32[$db>>2]|0;
    $114 = ((($113)) + 96|0);
    $115 = HEAP32[$114>>2]|0;
    $116 = ($115|0)<(0);
    $117 = $116 << 31 >> 31;
    $118 = ($112|0)>($117|0);
    $119 = ($111>>>0)>($115>>>0);
    $120 = ($112|0)==($117|0);
    $121 = $120 & $119;
    $122 = $118 | $121;
    if ($122) {
     label = 13;
     break;
    }
    $125 = HEAP32[$zOut>>2]|0;
    HEAP32[$zOld>>2] = $125;
    $126 = HEAP32[$zOut>>2]|0;
    $127 = $nOut;
    $128 = $127;
    $129 = HEAP32[$128>>2]|0;
    $130 = (($127) + 4)|0;
    $131 = $130;
    $132 = HEAP32[$131>>2]|0;
    $133 = ($129|0)<(0);
    $134 = $133 << 31 >> 31;
    $135 = (_sqlite3_realloc64($126,$129,$134)|0);
    HEAP32[$zOut>>2] = $135;
    $136 = HEAP32[$zOut>>2]|0;
    $137 = ($136|0)==(0|0);
    if ($137) {
     label = 15;
     break;
    }
    $140 = HEAP32[$j>>2]|0;
    $141 = HEAP32[$zOut>>2]|0;
    $142 = (($141) + ($140)|0);
    $143 = HEAP32[$zRep>>2]|0;
    $144 = HEAP32[$nRep>>2]|0;
    _memcpy(($142|0),($143|0),($144|0))|0;
    $145 = HEAP32[$nRep>>2]|0;
    $146 = HEAP32[$j>>2]|0;
    $147 = (($146) + ($145))|0;
    HEAP32[$j>>2] = $147;
    $148 = HEAP32[$nPattern>>2]|0;
    $149 = (($148) - 1)|0;
    $150 = HEAP32[$i>>2]|0;
    $151 = (($150) + ($149))|0;
    HEAP32[$i>>2] = $151;
   }
  }
  if ((label|0) == 11) {
   label = 0;
   $78 = HEAP32[$i>>2]|0;
   $79 = HEAP32[$zStr>>2]|0;
   $80 = (($79) + ($78)|0);
   $81 = HEAP8[$80>>0]|0;
   $82 = HEAP32[$j>>2]|0;
   $83 = (($82) + 1)|0;
   HEAP32[$j>>2] = $83;
   $84 = HEAP32[$zOut>>2]|0;
   $85 = (($84) + ($82)|0);
   HEAP8[$85>>0] = $81;
  }
  $152 = HEAP32[$i>>2]|0;
  $153 = (($152) + 1)|0;
  HEAP32[$i>>2] = $153;
 }
 if ((label|0) == 13) {
  $123 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_toobig($123);
  $124 = HEAP32[$zOut>>2]|0;
  _sqlite3_free($124);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 15) {
  $138 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_nomem($138);
  $139 = HEAP32[$zOld>>2]|0;
  _sqlite3_free($139);
  STACKTOP = sp;return;
 }
 else if ((label|0) == 18) {
  $154 = HEAP32[$j>>2]|0;
  $155 = HEAP32[$zOut>>2]|0;
  $156 = (($155) + ($154)|0);
  $157 = HEAP32[$i>>2]|0;
  $158 = HEAP32[$zStr>>2]|0;
  $159 = (($158) + ($157)|0);
  $160 = HEAP32[$nStr>>2]|0;
  $161 = HEAP32[$i>>2]|0;
  $162 = (($160) - ($161))|0;
  _memcpy(($156|0),($159|0),($162|0))|0;
  $163 = HEAP32[$nStr>>2]|0;
  $164 = HEAP32[$i>>2]|0;
  $165 = (($163) - ($164))|0;
  $166 = HEAP32[$j>>2]|0;
  $167 = (($166) + ($165))|0;
  HEAP32[$j>>2] = $167;
  $168 = HEAP32[$j>>2]|0;
  $169 = HEAP32[$zOut>>2]|0;
  $170 = (($169) + ($168)|0);
  HEAP8[$170>>0] = 0;
  $171 = HEAP32[$0>>2]|0;
  $172 = HEAP32[$zOut>>2]|0;
  $173 = HEAP32[$j>>2]|0;
  _sqlite3_result_text($171,$172,$173,148);
  STACKTOP = sp;return;
 }
}
function _zeroblobFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $n = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $n = sp;
 $rc = sp + 8|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_int64($4)|0);
 $6 = tempRet0;
 $7 = $n;
 $8 = $7;
 HEAP32[$8>>2] = $5;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $6;
 $11 = $n;
 $12 = $11;
 $13 = HEAP32[$12>>2]|0;
 $14 = (($11) + 4)|0;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = ($16|0)<(0);
 $18 = $17 ? 0 : $5;
 $19 = $17 ? 0 : $6;
 $20 = $n;
 $21 = $20;
 HEAP32[$21>>2] = $18;
 $22 = (($20) + 4)|0;
 $23 = $22;
 HEAP32[$23>>2] = $19;
 $24 = HEAP32[$0>>2]|0;
 $25 = $n;
 $26 = $25;
 $27 = HEAP32[$26>>2]|0;
 $28 = (($25) + 4)|0;
 $29 = $28;
 $30 = HEAP32[$29>>2]|0;
 $31 = (_sqlite3_result_zeroblob64($24,$27,$30)|0);
 HEAP32[$rc>>2] = $31;
 $32 = HEAP32[$rc>>2]|0;
 $33 = ($32|0)!=(0);
 if (!($33)) {
  STACKTOP = sp;return;
 }
 $34 = HEAP32[$0>>2]|0;
 $35 = HEAP32[$rc>>2]|0;
 _sqlite3_result_error_code($34,$35);
 STACKTOP = sp;return;
}
function _substrFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0;
 var $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0;
 var $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0;
 var $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0;
 var $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $len = 0, $negP2 = 0;
 var $p0type = 0, $p1 = 0, $p2 = 0, $z = 0, $z2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 44|0;
 $1 = sp + 40|0;
 $2 = sp + 36|0;
 $z = sp + 32|0;
 $z2 = sp + 28|0;
 $len = sp + 24|0;
 $p0type = sp + 20|0;
 $p1 = sp + 8|0;
 $p2 = sp;
 $negP2 = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 HEAP32[$negP2>>2] = 0;
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = (_sqlite3_value_type($5)|0);
 $7 = ($6|0)==(5);
 if ($7) {
  STACKTOP = sp;return;
 }
 $8 = HEAP32[$1>>2]|0;
 $9 = ($8|0)==(3);
 if ($9) {
  $10 = HEAP32[$2>>2]|0;
  $11 = ((($10)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = (_sqlite3_value_type($12)|0);
  $14 = ($13|0)==(5);
  if ($14) {
   STACKTOP = sp;return;
  }
 }
 $15 = HEAP32[$2>>2]|0;
 $16 = HEAP32[$15>>2]|0;
 $17 = (_sqlite3_value_type($16)|0);
 HEAP32[$p0type>>2] = $17;
 $18 = HEAP32[$2>>2]|0;
 $19 = ((($18)) + 4|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = (_sqlite3_value_int($20)|0);
 $22 = ($21|0)<(0);
 $23 = $22 << 31 >> 31;
 $24 = $p1;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = (($24) + 4)|0;
 $27 = $26;
 HEAP32[$27>>2] = $23;
 $28 = HEAP32[$p0type>>2]|0;
 $29 = ($28|0)==(4);
 $30 = HEAP32[$2>>2]|0;
 $31 = HEAP32[$30>>2]|0;
 L8: do {
  if ($29) {
   $32 = (_sqlite3_value_bytes($31)|0);
   HEAP32[$len>>2] = $32;
   $33 = HEAP32[$2>>2]|0;
   $34 = HEAP32[$33>>2]|0;
   $35 = (_sqlite3_value_blob($34)|0);
   HEAP32[$z>>2] = $35;
   $36 = HEAP32[$z>>2]|0;
   $37 = ($36|0)==(0|0);
   if ($37) {
    STACKTOP = sp;return;
   }
  } else {
   $38 = (_sqlite3_value_text($31)|0);
   HEAP32[$z>>2] = $38;
   $39 = HEAP32[$z>>2]|0;
   $40 = ($39|0)==(0|0);
   if ($40) {
    STACKTOP = sp;return;
   }
   HEAP32[$len>>2] = 0;
   $41 = $p1;
   $42 = $41;
   $43 = HEAP32[$42>>2]|0;
   $44 = (($41) + 4)|0;
   $45 = $44;
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)<(0);
   if ($47) {
    $48 = HEAP32[$z>>2]|0;
    HEAP32[$z2>>2] = $48;
    while(1) {
     $49 = HEAP32[$z2>>2]|0;
     $50 = HEAP8[$49>>0]|0;
     $51 = ($50<<24>>24)!=(0);
     if (!($51)) {
      break L8;
     }
     $52 = HEAP32[$z2>>2]|0;
     $53 = ((($52)) + 1|0);
     HEAP32[$z2>>2] = $53;
     $54 = HEAP8[$52>>0]|0;
     $55 = $54&255;
     $56 = ($55|0)>=(192);
     L19: do {
      if ($56) {
       while(1) {
        $57 = HEAP32[$z2>>2]|0;
        $58 = HEAP8[$57>>0]|0;
        $59 = $58&255;
        $60 = $59 & 192;
        $61 = ($60|0)==(128);
        if (!($61)) {
         break L19;
        }
        $62 = HEAP32[$z2>>2]|0;
        $63 = ((($62)) + 1|0);
        HEAP32[$z2>>2] = $63;
       }
      }
     } while(0);
     $64 = HEAP32[$len>>2]|0;
     $65 = (($64) + 1)|0;
     HEAP32[$len>>2] = $65;
    }
   }
  }
 } while(0);
 $66 = HEAP32[$1>>2]|0;
 $67 = ($66|0)==(3);
 if ($67) {
  $68 = HEAP32[$2>>2]|0;
  $69 = ((($68)) + 8|0);
  $70 = HEAP32[$69>>2]|0;
  $71 = (_sqlite3_value_int($70)|0);
  $72 = ($71|0)<(0);
  $73 = $72 << 31 >> 31;
  $74 = $p2;
  $75 = $74;
  HEAP32[$75>>2] = $71;
  $76 = (($74) + 4)|0;
  $77 = $76;
  HEAP32[$77>>2] = $73;
  $78 = $p2;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (($78) + 4)|0;
  $82 = $81;
  $83 = HEAP32[$82>>2]|0;
  $84 = ($83|0)<(0);
  if ($84) {
   $85 = $p2;
   $86 = $85;
   $87 = HEAP32[$86>>2]|0;
   $88 = (($85) + 4)|0;
   $89 = $88;
   $90 = HEAP32[$89>>2]|0;
   $91 = (_i64Subtract(0,0,($87|0),($90|0))|0);
   $92 = tempRet0;
   $93 = $p2;
   $94 = $93;
   HEAP32[$94>>2] = $91;
   $95 = (($93) + 4)|0;
   $96 = $95;
   HEAP32[$96>>2] = $92;
   HEAP32[$negP2>>2] = 1;
  }
 } else {
  $97 = HEAP32[$0>>2]|0;
  $98 = (_sqlite3_context_db_handle($97)|0);
  $99 = ((($98)) + 96|0);
  $100 = HEAP32[$99>>2]|0;
  $101 = ($100|0)<(0);
  $102 = $101 << 31 >> 31;
  $103 = $p2;
  $104 = $103;
  HEAP32[$104>>2] = $100;
  $105 = (($103) + 4)|0;
  $106 = $105;
  HEAP32[$106>>2] = $102;
 }
 $107 = $p1;
 $108 = $107;
 $109 = HEAP32[$108>>2]|0;
 $110 = (($107) + 4)|0;
 $111 = $110;
 $112 = HEAP32[$111>>2]|0;
 $113 = ($112|0)<(0);
 do {
  if ($113) {
   $114 = HEAP32[$len>>2]|0;
   $115 = ($114|0)<(0);
   $116 = $115 << 31 >> 31;
   $117 = $p1;
   $118 = $117;
   $119 = HEAP32[$118>>2]|0;
   $120 = (($117) + 4)|0;
   $121 = $120;
   $122 = HEAP32[$121>>2]|0;
   $123 = (_i64Add(($119|0),($122|0),($114|0),($116|0))|0);
   $124 = tempRet0;
   $125 = $p1;
   $126 = $125;
   HEAP32[$126>>2] = $123;
   $127 = (($125) + 4)|0;
   $128 = $127;
   HEAP32[$128>>2] = $124;
   $129 = $p1;
   $130 = $129;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($129) + 4)|0;
   $133 = $132;
   $134 = HEAP32[$133>>2]|0;
   $135 = ($134|0)<(0);
   if ($135) {
    $136 = $p1;
    $137 = $136;
    $138 = HEAP32[$137>>2]|0;
    $139 = (($136) + 4)|0;
    $140 = $139;
    $141 = HEAP32[$140>>2]|0;
    $142 = $p2;
    $143 = $142;
    $144 = HEAP32[$143>>2]|0;
    $145 = (($142) + 4)|0;
    $146 = $145;
    $147 = HEAP32[$146>>2]|0;
    $148 = (_i64Add(($144|0),($147|0),($138|0),($141|0))|0);
    $149 = tempRet0;
    $150 = $p2;
    $151 = $150;
    HEAP32[$151>>2] = $148;
    $152 = (($150) + 4)|0;
    $153 = $152;
    HEAP32[$153>>2] = $149;
    $154 = $p2;
    $155 = $154;
    $156 = HEAP32[$155>>2]|0;
    $157 = (($154) + 4)|0;
    $158 = $157;
    $159 = HEAP32[$158>>2]|0;
    $160 = ($159|0)<(0);
    $161 = $160 ? 0 : $148;
    $162 = $160 ? 0 : $149;
    $163 = $p2;
    $164 = $163;
    HEAP32[$164>>2] = $161;
    $165 = (($163) + 4)|0;
    $166 = $165;
    HEAP32[$166>>2] = $162;
    $167 = $p1;
    $168 = $167;
    HEAP32[$168>>2] = 0;
    $169 = (($167) + 4)|0;
    $170 = $169;
    HEAP32[$170>>2] = 0;
   }
  } else {
   $171 = $p1;
   $172 = $171;
   $173 = HEAP32[$172>>2]|0;
   $174 = (($171) + 4)|0;
   $175 = $174;
   $176 = HEAP32[$175>>2]|0;
   $177 = ($176|0)>(0);
   $178 = ($173>>>0)>(0);
   $179 = ($176|0)==(0);
   $180 = $179 & $178;
   $181 = $177 | $180;
   if ($181) {
    $182 = $p1;
    $183 = $182;
    $184 = HEAP32[$183>>2]|0;
    $185 = (($182) + 4)|0;
    $186 = $185;
    $187 = HEAP32[$186>>2]|0;
    $188 = (_i64Add(($184|0),($187|0),-1,-1)|0);
    $189 = tempRet0;
    $190 = $p1;
    $191 = $190;
    HEAP32[$191>>2] = $188;
    $192 = (($190) + 4)|0;
    $193 = $192;
    HEAP32[$193>>2] = $189;
    break;
   }
   $194 = $p2;
   $195 = $194;
   $196 = HEAP32[$195>>2]|0;
   $197 = (($194) + 4)|0;
   $198 = $197;
   $199 = HEAP32[$198>>2]|0;
   $200 = ($199|0)>(0);
   $201 = ($196>>>0)>(0);
   $202 = ($199|0)==(0);
   $203 = $202 & $201;
   $204 = $200 | $203;
   if ($204) {
    $205 = $p2;
    $206 = $205;
    $207 = HEAP32[$206>>2]|0;
    $208 = (($205) + 4)|0;
    $209 = $208;
    $210 = HEAP32[$209>>2]|0;
    $211 = (_i64Add(($207|0),($210|0),-1,-1)|0);
    $212 = tempRet0;
    $213 = $p2;
    $214 = $213;
    HEAP32[$214>>2] = $211;
    $215 = (($213) + 4)|0;
    $216 = $215;
    HEAP32[$216>>2] = $212;
   }
  }
 } while(0);
 $217 = HEAP32[$negP2>>2]|0;
 $218 = ($217|0)!=(0);
 if ($218) {
  $219 = $p2;
  $220 = $219;
  $221 = HEAP32[$220>>2]|0;
  $222 = (($219) + 4)|0;
  $223 = $222;
  $224 = HEAP32[$223>>2]|0;
  $225 = $p1;
  $226 = $225;
  $227 = HEAP32[$226>>2]|0;
  $228 = (($225) + 4)|0;
  $229 = $228;
  $230 = HEAP32[$229>>2]|0;
  $231 = (_i64Subtract(($227|0),($230|0),($221|0),($224|0))|0);
  $232 = tempRet0;
  $233 = $p1;
  $234 = $233;
  HEAP32[$234>>2] = $231;
  $235 = (($233) + 4)|0;
  $236 = $235;
  HEAP32[$236>>2] = $232;
  $237 = $p1;
  $238 = $237;
  $239 = HEAP32[$238>>2]|0;
  $240 = (($237) + 4)|0;
  $241 = $240;
  $242 = HEAP32[$241>>2]|0;
  $243 = ($242|0)<(0);
  if ($243) {
   $244 = $p1;
   $245 = $244;
   $246 = HEAP32[$245>>2]|0;
   $247 = (($244) + 4)|0;
   $248 = $247;
   $249 = HEAP32[$248>>2]|0;
   $250 = $p2;
   $251 = $250;
   $252 = HEAP32[$251>>2]|0;
   $253 = (($250) + 4)|0;
   $254 = $253;
   $255 = HEAP32[$254>>2]|0;
   $256 = (_i64Add(($252|0),($255|0),($246|0),($249|0))|0);
   $257 = tempRet0;
   $258 = $p2;
   $259 = $258;
   HEAP32[$259>>2] = $256;
   $260 = (($258) + 4)|0;
   $261 = $260;
   HEAP32[$261>>2] = $257;
   $262 = $p1;
   $263 = $262;
   HEAP32[$263>>2] = 0;
   $264 = (($262) + 4)|0;
   $265 = $264;
   HEAP32[$265>>2] = 0;
  }
 }
 $266 = HEAP32[$p0type>>2]|0;
 $267 = ($266|0)!=(4);
 if (!($267)) {
  $353 = $p1;
  $354 = $353;
  $355 = HEAP32[$354>>2]|0;
  $356 = (($353) + 4)|0;
  $357 = $356;
  $358 = HEAP32[$357>>2]|0;
  $359 = $p2;
  $360 = $359;
  $361 = HEAP32[$360>>2]|0;
  $362 = (($359) + 4)|0;
  $363 = $362;
  $364 = HEAP32[$363>>2]|0;
  $365 = (_i64Add(($355|0),($358|0),($361|0),($364|0))|0);
  $366 = tempRet0;
  $367 = HEAP32[$len>>2]|0;
  $368 = ($367|0)<(0);
  $369 = $368 << 31 >> 31;
  $370 = ($366|0)>($369|0);
  $371 = ($365>>>0)>($367>>>0);
  $372 = ($366|0)==($369|0);
  $373 = $372 & $371;
  $374 = $370 | $373;
  if ($374) {
   $375 = HEAP32[$len>>2]|0;
   $376 = ($375|0)<(0);
   $377 = $376 << 31 >> 31;
   $378 = $p1;
   $379 = $378;
   $380 = HEAP32[$379>>2]|0;
   $381 = (($378) + 4)|0;
   $382 = $381;
   $383 = HEAP32[$382>>2]|0;
   $384 = (_i64Subtract(($375|0),($377|0),($380|0),($383|0))|0);
   $385 = tempRet0;
   $386 = $p2;
   $387 = $386;
   HEAP32[$387>>2] = $384;
   $388 = (($386) + 4)|0;
   $389 = $388;
   HEAP32[$389>>2] = $385;
   $390 = $p2;
   $391 = $390;
   $392 = HEAP32[$391>>2]|0;
   $393 = (($390) + 4)|0;
   $394 = $393;
   $395 = HEAP32[$394>>2]|0;
   $396 = ($395|0)<(0);
   $397 = $396 ? 0 : $384;
   $398 = $396 ? 0 : $385;
   $399 = $p2;
   $400 = $399;
   HEAP32[$400>>2] = $397;
   $401 = (($399) + 4)|0;
   $402 = $401;
   HEAP32[$402>>2] = $398;
  }
  $403 = HEAP32[$0>>2]|0;
  $404 = $p1;
  $405 = $404;
  $406 = HEAP32[$405>>2]|0;
  $407 = (($404) + 4)|0;
  $408 = $407;
  $409 = HEAP32[$408>>2]|0;
  $410 = HEAP32[$z>>2]|0;
  $411 = (($410) + ($406)|0);
  $412 = $p2;
  $413 = $412;
  $414 = HEAP32[$413>>2]|0;
  $415 = (($412) + 4)|0;
  $416 = $415;
  $417 = HEAP32[$416>>2]|0;
  _sqlite3_result_blob64($403,$411,$414,$417,(-1));
  STACKTOP = sp;return;
 }
 while(1) {
  $268 = HEAP32[$z>>2]|0;
  $269 = HEAP8[$268>>0]|0;
  $270 = $269&255;
  $271 = ($270|0)!=(0);
  $272 = $p1;
  $273 = $272;
  $274 = HEAP32[$273>>2]|0;
  $275 = (($272) + 4)|0;
  $276 = $275;
  $277 = HEAP32[$276>>2]|0;
  $278 = ($274|0)!=(0);
  $279 = ($277|0)!=(0);
  $280 = $278 | $279;
  $281 = $271 ? $280 : 0;
  $282 = HEAP32[$z>>2]|0;
  if (!($281)) {
   break;
  }
  $283 = ((($282)) + 1|0);
  HEAP32[$z>>2] = $283;
  $284 = HEAP8[$282>>0]|0;
  $285 = $284&255;
  $286 = ($285|0)>=(192);
  L52: do {
   if ($286) {
    while(1) {
     $287 = HEAP32[$z>>2]|0;
     $288 = HEAP8[$287>>0]|0;
     $289 = $288&255;
     $290 = $289 & 192;
     $291 = ($290|0)==(128);
     if (!($291)) {
      break L52;
     }
     $292 = HEAP32[$z>>2]|0;
     $293 = ((($292)) + 1|0);
     HEAP32[$z>>2] = $293;
    }
   }
  } while(0);
  $294 = $p1;
  $295 = $294;
  $296 = HEAP32[$295>>2]|0;
  $297 = (($294) + 4)|0;
  $298 = $297;
  $299 = HEAP32[$298>>2]|0;
  $300 = (_i64Add(($296|0),($299|0),-1,-1)|0);
  $301 = tempRet0;
  $302 = $p1;
  $303 = $302;
  HEAP32[$303>>2] = $300;
  $304 = (($302) + 4)|0;
  $305 = $304;
  HEAP32[$305>>2] = $301;
 }
 HEAP32[$z2>>2] = $282;
 while(1) {
  $306 = HEAP32[$z2>>2]|0;
  $307 = HEAP8[$306>>0]|0;
  $308 = $307&255;
  $309 = ($308|0)!=(0);
  $310 = $p2;
  $311 = $310;
  $312 = HEAP32[$311>>2]|0;
  $313 = (($310) + 4)|0;
  $314 = $313;
  $315 = HEAP32[$314>>2]|0;
  $316 = ($312|0)!=(0);
  $317 = ($315|0)!=(0);
  $318 = $316 | $317;
  $319 = $309 ? $318 : 0;
  if (!($319)) {
   break;
  }
  $320 = HEAP32[$z2>>2]|0;
  $321 = ((($320)) + 1|0);
  HEAP32[$z2>>2] = $321;
  $322 = HEAP8[$320>>0]|0;
  $323 = $322&255;
  $324 = ($323|0)>=(192);
  L61: do {
   if ($324) {
    while(1) {
     $325 = HEAP32[$z2>>2]|0;
     $326 = HEAP8[$325>>0]|0;
     $327 = $326&255;
     $328 = $327 & 192;
     $329 = ($328|0)==(128);
     if (!($329)) {
      break L61;
     }
     $330 = HEAP32[$z2>>2]|0;
     $331 = ((($330)) + 1|0);
     HEAP32[$z2>>2] = $331;
    }
   }
  } while(0);
  $332 = $p2;
  $333 = $332;
  $334 = HEAP32[$333>>2]|0;
  $335 = (($332) + 4)|0;
  $336 = $335;
  $337 = HEAP32[$336>>2]|0;
  $338 = (_i64Add(($334|0),($337|0),-1,-1)|0);
  $339 = tempRet0;
  $340 = $p2;
  $341 = $340;
  HEAP32[$341>>2] = $338;
  $342 = (($340) + 4)|0;
  $343 = $342;
  HEAP32[$343>>2] = $339;
 }
 $344 = HEAP32[$0>>2]|0;
 $345 = HEAP32[$z>>2]|0;
 $346 = HEAP32[$z2>>2]|0;
 $347 = HEAP32[$z>>2]|0;
 $348 = $346;
 $349 = $347;
 $350 = (($348) - ($349))|0;
 $351 = ($350|0)<(0);
 $352 = $351 << 31 >> 31;
 _sqlite3_result_text64($344,$345,$350,$352,(-1),1);
 STACKTOP = sp;return;
}
function _sumStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0, $44 = 0.0;
 var $45 = 0.0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0, $7 = 0, $70 = 0.0, $71 = 0.0, $72 = 0, $73 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, $type = 0, $v = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 24|0;
 $1 = sp + 20|0;
 $2 = sp + 16|0;
 $p = sp + 12|0;
 $type = sp + 8|0;
 $v = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_aggregate_context($3,32)|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_numeric_type($6)|0);
 HEAP32[$type>>2] = $7;
 $8 = HEAP32[$p>>2]|0;
 $9 = ($8|0)!=(0|0);
 $10 = HEAP32[$type>>2]|0;
 $11 = ($10|0)!=(5);
 $or$cond = $9 & $11;
 if (!($or$cond)) {
  STACKTOP = sp;return;
 }
 $12 = HEAP32[$p>>2]|0;
 $13 = ((($12)) + 16|0);
 $14 = $13;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 $22 = $13;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 $26 = HEAP32[$type>>2]|0;
 $27 = ($26|0)==(1);
 $28 = HEAP32[$2>>2]|0;
 $29 = HEAP32[$28>>2]|0;
 if (!($27)) {
  $68 = (+_sqlite3_value_double($29));
  $69 = HEAP32[$p>>2]|0;
  $70 = +HEAPF64[$69>>3];
  $71 = $70 + $68;
  HEAPF64[$69>>3] = $71;
  $72 = HEAP32[$p>>2]|0;
  $73 = ((($72)) + 25|0);
  HEAP8[$73>>0] = 1;
  STACKTOP = sp;return;
 }
 $30 = (_sqlite3_value_int64($29)|0);
 $31 = tempRet0;
 $32 = $v;
 $33 = $32;
 HEAP32[$33>>2] = $30;
 $34 = (($32) + 4)|0;
 $35 = $34;
 HEAP32[$35>>2] = $31;
 $36 = $v;
 $37 = $36;
 $38 = HEAP32[$37>>2]|0;
 $39 = (($36) + 4)|0;
 $40 = $39;
 $41 = HEAP32[$40>>2]|0;
 $42 = (+($38>>>0)) + (4294967296.0*(+($41|0)));
 $43 = HEAP32[$p>>2]|0;
 $44 = +HEAPF64[$43>>3];
 $45 = $44 + $42;
 HEAPF64[$43>>3] = $45;
 $46 = HEAP32[$p>>2]|0;
 $47 = ((($46)) + 25|0);
 $48 = HEAP8[$47>>0]|0;
 $49 = $48&255;
 $50 = HEAP32[$p>>2]|0;
 $51 = ((($50)) + 24|0);
 $52 = HEAP8[$51>>0]|0;
 $53 = $52&255;
 $54 = $49 | $53;
 $55 = ($54|0)==(0);
 if (!($55)) {
  STACKTOP = sp;return;
 }
 $56 = HEAP32[$p>>2]|0;
 $57 = ((($56)) + 8|0);
 $58 = $v;
 $59 = $58;
 $60 = HEAP32[$59>>2]|0;
 $61 = (($58) + 4)|0;
 $62 = $61;
 $63 = HEAP32[$62>>2]|0;
 $64 = (_sqlite3AddInt64($57,$60,$63)|0);
 $65 = ($64|0)!=(0);
 if (!($65)) {
  STACKTOP = sp;return;
 }
 $66 = HEAP32[$p>>2]|0;
 $67 = ((($66)) + 24|0);
 HEAP8[$67>>0] = 1;
 STACKTOP = sp;return;
}
function _sumFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 $14 = ($9>>>0)>(0);
 $15 = ($12|0)==(0);
 $16 = $15 & $14;
 $17 = $13 | $16;
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$p>>2]|0;
 $19 = ((($18)) + 24|0);
 $20 = HEAP8[$19>>0]|0;
 $21 = ($20<<24>>24)!=(0);
 if ($21) {
  $22 = HEAP32[$0>>2]|0;
  _sqlite3_result_error($22,45165,-1);
  STACKTOP = sp;return;
 }
 $23 = HEAP32[$p>>2]|0;
 $24 = ((($23)) + 25|0);
 $25 = HEAP8[$24>>0]|0;
 $26 = ($25<<24>>24)!=(0);
 $27 = HEAP32[$0>>2]|0;
 $28 = HEAP32[$p>>2]|0;
 if ($26) {
  $29 = +HEAPF64[$28>>3];
  _sqlite3_result_double($27,$29);
  STACKTOP = sp;return;
 } else {
  $30 = ((($28)) + 8|0);
  $31 = $30;
  $32 = $31;
  $33 = HEAP32[$32>>2]|0;
  $34 = (($31) + 4)|0;
  $35 = $34;
  $36 = HEAP32[$35>>2]|0;
  _sqlite3_result_int64($27,$33,$36);
  STACKTOP = sp;return;
 }
}
function _totalFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0.0, $8 = 0.0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$p>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $8 = 0.0;
  _sqlite3_result_double($3,$8);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = +HEAPF64[$6>>3];
 $8 = $7;
 _sqlite3_result_double($3,$8);
 STACKTOP = sp;return;
}
function _avgFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$p>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$p>>2]|0;
 $6 = ((($5)) + 16|0);
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = ($12|0)>(0);
 $14 = ($9>>>0)>(0);
 $15 = ($12|0)==(0);
 $16 = $15 & $14;
 $17 = $13 | $16;
 if (!($17)) {
  STACKTOP = sp;return;
 }
 $18 = HEAP32[$0>>2]|0;
 $19 = HEAP32[$p>>2]|0;
 $20 = +HEAPF64[$19>>3];
 $21 = HEAP32[$p>>2]|0;
 $22 = ((($21)) + 16|0);
 $23 = $22;
 $24 = $23;
 $25 = HEAP32[$24>>2]|0;
 $26 = (($23) + 4)|0;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (+($25>>>0)) + (4294967296.0*(+($28|0)));
 $30 = $20 / $29;
 _sqlite3_result_double($18,$30);
 STACKTOP = sp;return;
}
function _countStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $$old = 0, $$old1 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0;
 var $25 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_aggregate_context($3,8)|0);
 HEAP32[$p>>2] = $4;
 $5 = HEAP32[$1>>2]|0;
 $6 = ($5|0)==(0);
 if ($6) {
  $$old = HEAP32[$p>>2]|0;
  $$old1 = ($$old|0)!=(0|0);
  if (!($$old1)) {
   STACKTOP = sp;return;
  }
 } else {
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$7>>2]|0;
  $9 = (_sqlite3_value_type($8)|0);
  $10 = (5)!=($9|0);
  $11 = HEAP32[$p>>2]|0;
  $12 = ($11|0)!=(0|0);
  $or$cond = $10 & $12;
  if (!($or$cond)) {
   STACKTOP = sp;return;
  }
 }
 $13 = HEAP32[$p>>2]|0;
 $14 = $13;
 $15 = $14;
 $16 = HEAP32[$15>>2]|0;
 $17 = (($14) + 4)|0;
 $18 = $17;
 $19 = HEAP32[$18>>2]|0;
 $20 = (_i64Add(($16|0),($19|0),1,0)|0);
 $21 = tempRet0;
 $22 = $13;
 $23 = $22;
 HEAP32[$23>>2] = $20;
 $24 = (($22) + 4)|0;
 $25 = $24;
 HEAP32[$25>>2] = $21;
 STACKTOP = sp;return;
}
function _countFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $p = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$p>>2] = $2;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$p>>2]|0;
 $5 = ($4|0)!=(0|0);
 if (!($5)) {
  $13 = 0;$14 = 0;
  _sqlite3_result_int64($3,$13,$14);
  STACKTOP = sp;return;
 }
 $6 = HEAP32[$p>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = $9;$14 = $12;
 _sqlite3_result_int64($3,$13,$14);
 STACKTOP = sp;return;
}
function _groupConcatStep($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $firstTerm = 0, $nSep = 0, $nVal = 0, $pAccum = 0, $zSep = 0, $zVal = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $zVal = sp + 24|0;
 $pAccum = sp + 20|0;
 $zSep = sp + 16|0;
 $nVal = sp + 12|0;
 $nSep = sp + 8|0;
 $db = sp + 4|0;
 $firstTerm = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_type($4)|0);
 $6 = ($5|0)==(5);
 if ($6) {
  STACKTOP = sp;return;
 }
 $7 = HEAP32[$0>>2]|0;
 $8 = (_sqlite3_aggregate_context($7,28)|0);
 HEAP32[$pAccum>>2] = $8;
 $9 = HEAP32[$pAccum>>2]|0;
 $10 = ($9|0)!=(0|0);
 if (!($10)) {
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$pAccum>>2]|0;
 $14 = ((($13)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ($15|0)==(0);
 $17 = $16&1;
 HEAP32[$firstTerm>>2] = $17;
 $18 = HEAP32[$db>>2]|0;
 $19 = ((($18)) + 96|0);
 $20 = HEAP32[$19>>2]|0;
 $21 = HEAP32[$pAccum>>2]|0;
 $22 = ((($21)) + 20|0);
 HEAP32[$22>>2] = $20;
 $23 = HEAP32[$firstTerm>>2]|0;
 $24 = ($23|0)!=(0);
 if (!($24)) {
  $25 = HEAP32[$1>>2]|0;
  $26 = ($25|0)==(2);
  if ($26) {
   $27 = HEAP32[$2>>2]|0;
   $28 = ((($27)) + 4|0);
   $29 = HEAP32[$28>>2]|0;
   $30 = (_sqlite3_value_text($29)|0);
   HEAP32[$zSep>>2] = $30;
   $31 = HEAP32[$2>>2]|0;
   $32 = ((($31)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (_sqlite3_value_bytes($33)|0);
   HEAP32[$nSep>>2] = $34;
  } else {
   HEAP32[$zSep>>2] = 34710;
   HEAP32[$nSep>>2] = 1;
  }
  $35 = HEAP32[$nSep>>2]|0;
  $36 = ($35|0)!=(0);
  if ($36) {
   $37 = HEAP32[$pAccum>>2]|0;
   $38 = HEAP32[$zSep>>2]|0;
   $39 = HEAP32[$nSep>>2]|0;
   _sqlite3StrAccumAppend($37,$38,$39);
  }
 }
 $40 = HEAP32[$2>>2]|0;
 $41 = HEAP32[$40>>2]|0;
 $42 = (_sqlite3_value_text($41)|0);
 HEAP32[$zVal>>2] = $42;
 $43 = HEAP32[$2>>2]|0;
 $44 = HEAP32[$43>>2]|0;
 $45 = (_sqlite3_value_bytes($44)|0);
 HEAP32[$nVal>>2] = $45;
 $46 = HEAP32[$zVal>>2]|0;
 $47 = ($46|0)!=(0|0);
 if (!($47)) {
  STACKTOP = sp;return;
 }
 $48 = HEAP32[$pAccum>>2]|0;
 $49 = HEAP32[$zVal>>2]|0;
 $50 = HEAP32[$nVal>>2]|0;
 _sqlite3StrAccumAppend($48,$49,$50);
 STACKTOP = sp;return;
}
function _groupConcatFinalize($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pAccum = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pAccum = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = (_sqlite3_aggregate_context($1,0)|0);
 HEAP32[$pAccum>>2] = $2;
 $3 = HEAP32[$pAccum>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$pAccum>>2]|0;
 $6 = ((($5)) + 24|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = $7&255;
 $9 = ($8|0)==(2);
 if ($9) {
  $10 = HEAP32[$0>>2]|0;
  _sqlite3_result_error_toobig($10);
  STACKTOP = sp;return;
 }
 $11 = HEAP32[$pAccum>>2]|0;
 $12 = ((($11)) + 24|0);
 $13 = HEAP8[$12>>0]|0;
 $14 = $13&255;
 $15 = ($14|0)==(1);
 $16 = HEAP32[$0>>2]|0;
 if ($15) {
  _sqlite3_result_error_nomem($16);
  STACKTOP = sp;return;
 } else {
  $17 = HEAP32[$pAccum>>2]|0;
  $18 = (_sqlite3StrAccumFinish($17)|0);
  _sqlite3_result_text($16,$18,-1,148);
  STACKTOP = sp;return;
 }
}
function _contextMalloc($context,$0,$1) {
 $context = $context|0;
 $0 = $0|0;
 $1 = $1|0;
 var $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0;
 var $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $2 = sp + 16|0;
 $3 = sp;
 $z = sp + 12|0;
 $db = sp + 8|0;
 HEAP32[$2>>2] = $context;
 $4 = $3;
 $5 = $4;
 HEAP32[$5>>2] = $0;
 $6 = (($4) + 4)|0;
 $7 = $6;
 HEAP32[$7>>2] = $1;
 $8 = HEAP32[$2>>2]|0;
 $9 = (_sqlite3_context_db_handle($8)|0);
 HEAP32[$db>>2] = $9;
 $10 = $3;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (($10) + 4)|0;
 $14 = $13;
 $15 = HEAP32[$14>>2]|0;
 $16 = HEAP32[$db>>2]|0;
 $17 = ((($16)) + 96|0);
 $18 = HEAP32[$17>>2]|0;
 $19 = ($18|0)<(0);
 $20 = $19 << 31 >> 31;
 $21 = ($15|0)>($20|0);
 $22 = ($12>>>0)>($18>>>0);
 $23 = ($15|0)==($20|0);
 $24 = $23 & $22;
 $25 = $21 | $24;
 if ($25) {
  $26 = HEAP32[$2>>2]|0;
  _sqlite3_result_error_toobig($26);
  HEAP32[$z>>2] = 0;
  $37 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $27 = $3;
 $28 = $27;
 $29 = HEAP32[$28>>2]|0;
 $30 = (($27) + 4)|0;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (_sqlite3Malloc($29,$32)|0);
 HEAP32[$z>>2] = $33;
 $34 = HEAP32[$z>>2]|0;
 $35 = ($34|0)!=(0|0);
 if ($35) {
  $37 = HEAP32[$z>>2]|0;
  STACKTOP = sp;return ($37|0);
 }
 $36 = HEAP32[$2>>2]|0;
 _sqlite3_result_error_nomem($36);
 $37 = HEAP32[$z>>2]|0;
 STACKTOP = sp;return ($37|0);
}
function _sqlite3GetFuncCollSeq($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 16|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = (($3) - 1)|0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 12|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = ((($7)) + 88|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = (($9) + (($4*20)|0)|0);
 HEAP32[$pOp>>2] = $10;
 $11 = HEAP32[$pOp>>2]|0;
 $12 = ((($11)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _sqlite3SkipAccumulatorLoad($context) {
 $context = $context|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $context;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 24|0);
 HEAP8[$2>>0] = 1;
 STACKTOP = sp;return;
}
function _juliandayFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0.0, $16 = 0.0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $x = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $0 = sp + 56|0;
 $1 = sp + 52|0;
 $2 = sp + 48|0;
 $x = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeJD($x);
 $8 = HEAP32[$0>>2]|0;
 $9 = $x;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = (($9) + 4)|0;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (+($11>>>0)) + (4294967296.0*(+($14|0)));
 $16 = $15 / 8.64E+7;
 _sqlite3_result_double($8,$16);
 STACKTOP = sp;return;
}
function _dateFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $x = 0, $zBuf = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $x = sp;
 $zBuf = sp + 72|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeYMD($x);
 $8 = ((($x)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($x)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($x)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 (_sqlite3_snprintf(100,$zBuf,45506,$vararg_buffer)|0);
 $14 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($14,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _timeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $x = 0;
 var $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $x = sp;
 $zBuf = sp + 72|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeHMS($x);
 $8 = ((($x)) + 20|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($x)) + 24|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($x)) + 32|0);
 $13 = +HEAPF64[$12>>3];
 $14 = (~~(($13)));
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $14;
 (_sqlite3_snprintf(100,$zBuf,45491,$vararg_buffer)|0);
 $15 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($15,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _datetimeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0;
 var $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, $x = 0, $zBuf = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer = sp + 48|0;
 $0 = sp + 80|0;
 $1 = sp + 76|0;
 $2 = sp + 72|0;
 $x = sp;
 $zBuf = sp + 84|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = HEAP32[$1>>2]|0;
 $5 = HEAP32[$2>>2]|0;
 $6 = (_isDate($3,$4,$5,$x)|0);
 $7 = ($6|0)==(0);
 if (!($7)) {
  STACKTOP = sp;return;
 }
 _computeYMD_HMS($x);
 $8 = ((($x)) + 8|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = ((($x)) + 12|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ((($x)) + 16|0);
 $13 = HEAP32[$12>>2]|0;
 $14 = ((($x)) + 20|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = ((($x)) + 24|0);
 $17 = HEAP32[$16>>2]|0;
 $18 = ((($x)) + 32|0);
 $19 = +HEAPF64[$18>>3];
 $20 = (~~(($19)));
 HEAP32[$vararg_buffer>>2] = $9;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $11;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $13;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $15;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $17;
 $vararg_ptr5 = ((($vararg_buffer)) + 20|0);
 HEAP32[$vararg_ptr5>>2] = $20;
 (_sqlite3_snprintf(100,$zBuf,45461,$vararg_buffer)|0);
 $21 = HEAP32[$0>>2]|0;
 _sqlite3_result_text($21,$zBuf,-1,(-1));
 STACKTOP = sp;return;
}
function _strftimeFunc($context,$argc,$argv) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0;
 var $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0;
 var $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0;
 var $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0;
 var $189 = 0, $19 = 0, $190 = 0.0, $191 = 0.0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0.0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0;
 var $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0;
 var $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0;
 var $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0;
 var $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0.0, $276 = 0.0, $277 = 0, $278 = 0;
 var $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0;
 var $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0;
 var $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0.0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0;
 var $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0;
 var $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0;
 var $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0;
 var $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0;
 var $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0;
 var $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $db = 0, $i = 0, $j = 0, $n = 0, $nDay = 0, $s = 0, $vararg_buffer = 0;
 var $vararg_buffer1 = 0, $vararg_buffer10 = 0, $vararg_buffer13 = 0, $vararg_buffer16 = 0, $vararg_buffer19 = 0, $vararg_buffer22 = 0, $vararg_buffer25 = 0, $vararg_buffer28 = 0, $vararg_buffer4 = 0, $vararg_buffer7 = 0, $wd = 0, $x = 0, $y = 0, $z = 0, $zBuf = 0, $zFmt = 0, dest = 0, label = 0, sp = 0, src = 0;
 var stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 336|0;
 $vararg_buffer28 = sp + 192|0;
 $vararg_buffer25 = sp + 184|0;
 $vararg_buffer22 = sp + 176|0;
 $vararg_buffer19 = sp + 168|0;
 $vararg_buffer16 = sp + 160|0;
 $vararg_buffer13 = sp + 152|0;
 $vararg_buffer10 = sp + 144|0;
 $vararg_buffer7 = sp + 136|0;
 $vararg_buffer4 = sp + 128|0;
 $vararg_buffer1 = sp + 120|0;
 $vararg_buffer = sp + 112|0;
 $0 = sp + 232|0;
 $1 = sp + 228|0;
 $2 = sp + 224|0;
 $x = sp + 64|0;
 $n = sp + 56|0;
 $i = sp + 220|0;
 $j = sp + 216|0;
 $z = sp + 212|0;
 $db = sp + 208|0;
 $zFmt = sp + 204|0;
 $zBuf = sp + 236|0;
 $s = sp + 48|0;
 $nDay = sp + 200|0;
 $y = sp;
 $wd = sp + 196|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $argc;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)==(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zFmt>>2] = $7;
 $8 = HEAP32[$zFmt>>2]|0;
 $9 = ($8|0)==(0|0);
 if ($9) {
  STACKTOP = sp;return;
 }
 $10 = HEAP32[$0>>2]|0;
 $11 = HEAP32[$1>>2]|0;
 $12 = (($11) - 1)|0;
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 4|0);
 $15 = (_isDate($10,$12,$14,$x)|0);
 $16 = ($15|0)!=(0);
 if ($16) {
  STACKTOP = sp;return;
 }
 $17 = HEAP32[$0>>2]|0;
 $18 = (_sqlite3_context_db_handle($17)|0);
 HEAP32[$db>>2] = $18;
 HEAP32[$i>>2] = 0;
 $19 = $n;
 $20 = $19;
 HEAP32[$20>>2] = 1;
 $21 = (($19) + 4)|0;
 $22 = $21;
 HEAP32[$22>>2] = 0;
 L10: while(1) {
  $23 = HEAP32[$i>>2]|0;
  $24 = HEAP32[$zFmt>>2]|0;
  $25 = (($24) + ($23)|0);
  $26 = HEAP8[$25>>0]|0;
  $27 = ($26<<24>>24)!=(0);
  if (!($27)) {
   break;
  }
  $28 = HEAP32[$i>>2]|0;
  $29 = HEAP32[$zFmt>>2]|0;
  $30 = (($29) + ($28)|0);
  $31 = HEAP8[$30>>0]|0;
  $32 = $31 << 24 >> 24;
  $33 = ($32|0)==(37);
  if ($33) {
   $34 = HEAP32[$i>>2]|0;
   $35 = (($34) + 1)|0;
   $36 = HEAP32[$zFmt>>2]|0;
   $37 = (($36) + ($35)|0);
   $38 = HEAP8[$37>>0]|0;
   $39 = $38 << 24 >> 24;
   switch ($39|0) {
   case 87: case 83: case 77: case 109: case 72: case 100:  {
    $40 = $n;
    $41 = $40;
    $42 = HEAP32[$41>>2]|0;
    $43 = (($40) + 4)|0;
    $44 = $43;
    $45 = HEAP32[$44>>2]|0;
    $46 = (_i64Add(($42|0),($45|0),1,0)|0);
    $47 = tempRet0;
    $48 = $n;
    $49 = $48;
    HEAP32[$49>>2] = $46;
    $50 = (($48) + 4)|0;
    $51 = $50;
    HEAP32[$51>>2] = $47;
    break;
   }
   case 37: case 119:  {
    break;
   }
   case 102:  {
    $52 = $n;
    $53 = $52;
    $54 = HEAP32[$53>>2]|0;
    $55 = (($52) + 4)|0;
    $56 = $55;
    $57 = HEAP32[$56>>2]|0;
    $58 = (_i64Add(($54|0),($57|0),8,0)|0);
    $59 = tempRet0;
    $60 = $n;
    $61 = $60;
    HEAP32[$61>>2] = $58;
    $62 = (($60) + 4)|0;
    $63 = $62;
    HEAP32[$63>>2] = $59;
    break;
   }
   case 106:  {
    $64 = $n;
    $65 = $64;
    $66 = HEAP32[$65>>2]|0;
    $67 = (($64) + 4)|0;
    $68 = $67;
    $69 = HEAP32[$68>>2]|0;
    $70 = (_i64Add(($66|0),($69|0),3,0)|0);
    $71 = tempRet0;
    $72 = $n;
    $73 = $72;
    HEAP32[$73>>2] = $70;
    $74 = (($72) + 4)|0;
    $75 = $74;
    HEAP32[$75>>2] = $71;
    break;
   }
   case 89:  {
    $76 = $n;
    $77 = $76;
    $78 = HEAP32[$77>>2]|0;
    $79 = (($76) + 4)|0;
    $80 = $79;
    $81 = HEAP32[$80>>2]|0;
    $82 = (_i64Add(($78|0),($81|0),8,0)|0);
    $83 = tempRet0;
    $84 = $n;
    $85 = $84;
    HEAP32[$85>>2] = $82;
    $86 = (($84) + 4)|0;
    $87 = $86;
    HEAP32[$87>>2] = $83;
    break;
   }
   case 74: case 115:  {
    $88 = $n;
    $89 = $88;
    $90 = HEAP32[$89>>2]|0;
    $91 = (($88) + 4)|0;
    $92 = $91;
    $93 = HEAP32[$92>>2]|0;
    $94 = (_i64Add(($90|0),($93|0),50,0)|0);
    $95 = tempRet0;
    $96 = $n;
    $97 = $96;
    HEAP32[$97>>2] = $94;
    $98 = (($96) + 4)|0;
    $99 = $98;
    HEAP32[$99>>2] = $95;
    break;
   }
   default: {
    label = 44;
    break L10;
   }
   }
   $100 = HEAP32[$i>>2]|0;
   $101 = (($100) + 1)|0;
   HEAP32[$i>>2] = $101;
  }
  $102 = HEAP32[$i>>2]|0;
  $103 = (($102) + 1)|0;
  HEAP32[$i>>2] = $103;
  $104 = $n;
  $105 = $104;
  $106 = HEAP32[$105>>2]|0;
  $107 = (($104) + 4)|0;
  $108 = $107;
  $109 = HEAP32[$108>>2]|0;
  $110 = (_i64Add(($106|0),($109|0),1,0)|0);
  $111 = tempRet0;
  $112 = $n;
  $113 = $112;
  HEAP32[$113>>2] = $110;
  $114 = (($112) + 4)|0;
  $115 = $114;
  HEAP32[$115>>2] = $111;
 }
 if ((label|0) == 44) {
  STACKTOP = sp;return;
 }
 $116 = $n;
 $117 = $116;
 $118 = HEAP32[$117>>2]|0;
 $119 = (($116) + 4)|0;
 $120 = $119;
 $121 = HEAP32[$120>>2]|0;
 $122 = ($121>>>0)<(0);
 $123 = ($118>>>0)<(100);
 $124 = ($121|0)==(0);
 $125 = $124 & $123;
 $126 = $122 | $125;
 if ($126) {
  HEAP32[$z>>2] = $zBuf;
 } else {
  $127 = $n;
  $128 = $127;
  $129 = HEAP32[$128>>2]|0;
  $130 = (($127) + 4)|0;
  $131 = $130;
  $132 = HEAP32[$131>>2]|0;
  $133 = HEAP32[$db>>2]|0;
  $134 = ((($133)) + 96|0);
  $135 = HEAP32[$134>>2]|0;
  $136 = ($135|0)<(0);
  $137 = $136 << 31 >> 31;
  $138 = ($132>>>0)>($137>>>0);
  $139 = ($129>>>0)>($135>>>0);
  $140 = ($132|0)==($137|0);
  $141 = $140 & $139;
  $142 = $138 | $141;
  if ($142) {
   $143 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_toobig($143);
   STACKTOP = sp;return;
  }
  $144 = HEAP32[$db>>2]|0;
  $145 = $n;
  $146 = $145;
  $147 = HEAP32[$146>>2]|0;
  $148 = (($145) + 4)|0;
  $149 = $148;
  $150 = HEAP32[$149>>2]|0;
  $151 = ($147|0)<(0);
  $152 = $151 << 31 >> 31;
  $153 = (_sqlite3DbMallocRawNN($144,$147,$152)|0);
  HEAP32[$z>>2] = $153;
  $154 = HEAP32[$z>>2]|0;
  $155 = ($154|0)==(0|0);
  if ($155) {
   $156 = HEAP32[$0>>2]|0;
   _sqlite3_result_error_nomem($156);
   STACKTOP = sp;return;
  }
 }
 _computeJD($x);
 _computeYMD_HMS($x);
 HEAP32[$j>>2] = 0;
 HEAP32[$i>>2] = 0;
 while(1) {
  $157 = HEAP32[$i>>2]|0;
  $158 = HEAP32[$zFmt>>2]|0;
  $159 = (($158) + ($157)|0);
  $160 = HEAP8[$159>>0]|0;
  $161 = ($160<<24>>24)!=(0);
  if (!($161)) {
   break;
  }
  $162 = HEAP32[$i>>2]|0;
  $163 = HEAP32[$zFmt>>2]|0;
  $164 = (($163) + ($162)|0);
  $165 = HEAP8[$164>>0]|0;
  $166 = $165 << 24 >> 24;
  $167 = ($166|0)!=(37);
  $168 = HEAP32[$i>>2]|0;
  L39: do {
   if ($167) {
    $169 = HEAP32[$zFmt>>2]|0;
    $170 = (($169) + ($168)|0);
    $171 = HEAP8[$170>>0]|0;
    $172 = HEAP32[$j>>2]|0;
    $173 = (($172) + 1)|0;
    HEAP32[$j>>2] = $173;
    $174 = HEAP32[$z>>2]|0;
    $175 = (($174) + ($172)|0);
    HEAP8[$175>>0] = $171;
   } else {
    $176 = (($168) + 1)|0;
    HEAP32[$i>>2] = $176;
    $177 = HEAP32[$i>>2]|0;
    $178 = HEAP32[$zFmt>>2]|0;
    $179 = (($178) + ($177)|0);
    $180 = HEAP8[$179>>0]|0;
    $181 = $180 << 24 >> 24;
    do {
     switch ($181|0) {
     case 100:  {
      $182 = HEAP32[$j>>2]|0;
      $183 = HEAP32[$z>>2]|0;
      $184 = (($183) + ($182)|0);
      $185 = ((($x)) + 16|0);
      $186 = HEAP32[$185>>2]|0;
      HEAP32[$vararg_buffer>>2] = $186;
      (_sqlite3_snprintf(3,$184,45303,$vararg_buffer)|0);
      $187 = HEAP32[$j>>2]|0;
      $188 = (($187) + 2)|0;
      HEAP32[$j>>2] = $188;
      break L39;
      break;
     }
     case 102:  {
      $189 = ((($x)) + 32|0);
      $190 = +HEAPF64[$189>>3];
      HEAPF64[$s>>3] = $190;
      $191 = +HEAPF64[$s>>3];
      $192 = $191 > 59.999000000000002;
      if ($192) {
       HEAPF64[$s>>3] = 59.999000000000002;
      }
      $193 = HEAP32[$j>>2]|0;
      $194 = HEAP32[$z>>2]|0;
      $195 = (($194) + ($193)|0);
      $196 = +HEAPF64[$s>>3];
      HEAPF64[$vararg_buffer1>>3] = $196;
      (_sqlite3_snprintf(7,$195,45308,$vararg_buffer1)|0);
      $197 = HEAP32[$j>>2]|0;
      $198 = HEAP32[$z>>2]|0;
      $199 = (($198) + ($197)|0);
      $200 = (_sqlite3Strlen30($199)|0);
      $201 = HEAP32[$j>>2]|0;
      $202 = (($201) + ($200))|0;
      HEAP32[$j>>2] = $202;
      break L39;
      break;
     }
     case 72:  {
      $203 = HEAP32[$j>>2]|0;
      $204 = HEAP32[$z>>2]|0;
      $205 = (($204) + ($203)|0);
      $206 = ((($x)) + 20|0);
      $207 = HEAP32[$206>>2]|0;
      HEAP32[$vararg_buffer4>>2] = $207;
      (_sqlite3_snprintf(3,$205,45303,$vararg_buffer4)|0);
      $208 = HEAP32[$j>>2]|0;
      $209 = (($208) + 2)|0;
      HEAP32[$j>>2] = $209;
      break L39;
      break;
     }
     case 106: case 87:  {
      dest=$y; src=$x; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
      $210 = ((($y)) + 42|0);
      HEAP8[$210>>0] = 0;
      $211 = ((($y)) + 12|0);
      HEAP32[$211>>2] = 1;
      $212 = ((($y)) + 16|0);
      HEAP32[$212>>2] = 1;
      _computeJD($y);
      $213 = $x;
      $214 = $213;
      $215 = HEAP32[$214>>2]|0;
      $216 = (($213) + 4)|0;
      $217 = $216;
      $218 = HEAP32[$217>>2]|0;
      $219 = $y;
      $220 = $219;
      $221 = HEAP32[$220>>2]|0;
      $222 = (($219) + 4)|0;
      $223 = $222;
      $224 = HEAP32[$223>>2]|0;
      $225 = (_i64Subtract(($215|0),($218|0),($221|0),($224|0))|0);
      $226 = tempRet0;
      $227 = (_i64Add(($225|0),($226|0),43200000,0)|0);
      $228 = tempRet0;
      $229 = (___divdi3(($227|0),($228|0),86400000,0)|0);
      $230 = tempRet0;
      HEAP32[$nDay>>2] = $229;
      $231 = HEAP32[$i>>2]|0;
      $232 = HEAP32[$zFmt>>2]|0;
      $233 = (($232) + ($231)|0);
      $234 = HEAP8[$233>>0]|0;
      $235 = $234 << 24 >> 24;
      $236 = ($235|0)==(87);
      if ($236) {
       $237 = $x;
       $238 = $237;
       $239 = HEAP32[$238>>2]|0;
       $240 = (($237) + 4)|0;
       $241 = $240;
       $242 = HEAP32[$241>>2]|0;
       $243 = (_i64Add(($239|0),($242|0),43200000,0)|0);
       $244 = tempRet0;
       $245 = (___divdi3(($243|0),($244|0),86400000,0)|0);
       $246 = tempRet0;
       $247 = (___remdi3(($245|0),($246|0),7,0)|0);
       $248 = tempRet0;
       HEAP32[$wd>>2] = $247;
       $249 = HEAP32[$j>>2]|0;
       $250 = HEAP32[$z>>2]|0;
       $251 = (($250) + ($249)|0);
       $252 = HEAP32[$nDay>>2]|0;
       $253 = (($252) + 7)|0;
       $254 = HEAP32[$wd>>2]|0;
       $255 = (($253) - ($254))|0;
       $256 = (($255|0) / 7)&-1;
       HEAP32[$vararg_buffer7>>2] = $256;
       (_sqlite3_snprintf(3,$251,45303,$vararg_buffer7)|0);
       $257 = HEAP32[$j>>2]|0;
       $258 = (($257) + 2)|0;
       HEAP32[$j>>2] = $258;
       break L39;
      } else {
       $259 = HEAP32[$j>>2]|0;
       $260 = HEAP32[$z>>2]|0;
       $261 = (($260) + ($259)|0);
       $262 = HEAP32[$nDay>>2]|0;
       $263 = (($262) + 1)|0;
       HEAP32[$vararg_buffer10>>2] = $263;
       (_sqlite3_snprintf(4,$261,45315,$vararg_buffer10)|0);
       $264 = HEAP32[$j>>2]|0;
       $265 = (($264) + 3)|0;
       HEAP32[$j>>2] = $265;
       break L39;
      }
      break;
     }
     case 74:  {
      $266 = HEAP32[$j>>2]|0;
      $267 = HEAP32[$z>>2]|0;
      $268 = (($267) + ($266)|0);
      $269 = $x;
      $270 = $269;
      $271 = HEAP32[$270>>2]|0;
      $272 = (($269) + 4)|0;
      $273 = $272;
      $274 = HEAP32[$273>>2]|0;
      $275 = (+($271>>>0)) + (4294967296.0*(+($274|0)));
      $276 = $275 / 8.64E+7;
      HEAPF64[$vararg_buffer13>>3] = $276;
      (_sqlite3_snprintf(20,$268,41946,$vararg_buffer13)|0);
      $277 = HEAP32[$j>>2]|0;
      $278 = HEAP32[$z>>2]|0;
      $279 = (($278) + ($277)|0);
      $280 = (_sqlite3Strlen30($279)|0);
      $281 = HEAP32[$j>>2]|0;
      $282 = (($281) + ($280))|0;
      HEAP32[$j>>2] = $282;
      break L39;
      break;
     }
     case 109:  {
      $283 = HEAP32[$j>>2]|0;
      $284 = HEAP32[$z>>2]|0;
      $285 = (($284) + ($283)|0);
      $286 = ((($x)) + 12|0);
      $287 = HEAP32[$286>>2]|0;
      HEAP32[$vararg_buffer16>>2] = $287;
      (_sqlite3_snprintf(3,$285,45303,$vararg_buffer16)|0);
      $288 = HEAP32[$j>>2]|0;
      $289 = (($288) + 2)|0;
      HEAP32[$j>>2] = $289;
      break L39;
      break;
     }
     case 77:  {
      $290 = HEAP32[$j>>2]|0;
      $291 = HEAP32[$z>>2]|0;
      $292 = (($291) + ($290)|0);
      $293 = ((($x)) + 24|0);
      $294 = HEAP32[$293>>2]|0;
      HEAP32[$vararg_buffer19>>2] = $294;
      (_sqlite3_snprintf(3,$292,45303,$vararg_buffer19)|0);
      $295 = HEAP32[$j>>2]|0;
      $296 = (($295) + 2)|0;
      HEAP32[$j>>2] = $296;
      break L39;
      break;
     }
     case 115:  {
      $297 = HEAP32[$j>>2]|0;
      $298 = HEAP32[$z>>2]|0;
      $299 = (($298) + ($297)|0);
      $300 = $x;
      $301 = $300;
      $302 = HEAP32[$301>>2]|0;
      $303 = (($300) + 4)|0;
      $304 = $303;
      $305 = HEAP32[$304>>2]|0;
      $306 = (___divdi3(($302|0),($305|0),1000,0)|0);
      $307 = tempRet0;
      $308 = (_i64Subtract(($306|0),($307|0),413362496,49)|0);
      $309 = tempRet0;
      $310 = $vararg_buffer22;
      $311 = $310;
      HEAP32[$311>>2] = $308;
      $312 = (($310) + 4)|0;
      $313 = $312;
      HEAP32[$313>>2] = $309;
      (_sqlite3_snprintf(30,$299,28266,$vararg_buffer22)|0);
      $314 = HEAP32[$j>>2]|0;
      $315 = HEAP32[$z>>2]|0;
      $316 = (($315) + ($314)|0);
      $317 = (_sqlite3Strlen30($316)|0);
      $318 = HEAP32[$j>>2]|0;
      $319 = (($318) + ($317))|0;
      HEAP32[$j>>2] = $319;
      break L39;
      break;
     }
     case 83:  {
      $320 = HEAP32[$j>>2]|0;
      $321 = HEAP32[$z>>2]|0;
      $322 = (($321) + ($320)|0);
      $323 = ((($x)) + 32|0);
      $324 = +HEAPF64[$323>>3];
      $325 = (~~(($324)));
      HEAP32[$vararg_buffer25>>2] = $325;
      (_sqlite3_snprintf(3,$322,45303,$vararg_buffer25)|0);
      $326 = HEAP32[$j>>2]|0;
      $327 = (($326) + 2)|0;
      HEAP32[$j>>2] = $327;
      break L39;
      break;
     }
     case 119:  {
      $328 = $x;
      $329 = $328;
      $330 = HEAP32[$329>>2]|0;
      $331 = (($328) + 4)|0;
      $332 = $331;
      $333 = HEAP32[$332>>2]|0;
      $334 = (_i64Add(($330|0),($333|0),129600000,0)|0);
      $335 = tempRet0;
      $336 = (___divdi3(($334|0),($335|0),86400000,0)|0);
      $337 = tempRet0;
      $338 = (___remdi3(($336|0),($337|0),7,0)|0);
      $339 = tempRet0;
      $340 = $338&255;
      $341 = $340 << 24 >> 24;
      $342 = (($341) + 48)|0;
      $343 = $342&255;
      $344 = HEAP32[$j>>2]|0;
      $345 = (($344) + 1)|0;
      HEAP32[$j>>2] = $345;
      $346 = HEAP32[$z>>2]|0;
      $347 = (($346) + ($344)|0);
      HEAP8[$347>>0] = $343;
      break L39;
      break;
     }
     case 89:  {
      $348 = HEAP32[$j>>2]|0;
      $349 = HEAP32[$z>>2]|0;
      $350 = (($349) + ($348)|0);
      $351 = ((($x)) + 8|0);
      $352 = HEAP32[$351>>2]|0;
      HEAP32[$vararg_buffer28>>2] = $352;
      (_sqlite3_snprintf(5,$350,45320,$vararg_buffer28)|0);
      $353 = HEAP32[$j>>2]|0;
      $354 = HEAP32[$z>>2]|0;
      $355 = (($354) + ($353)|0);
      $356 = (_sqlite3Strlen30($355)|0);
      $357 = HEAP32[$j>>2]|0;
      $358 = (($357) + ($356))|0;
      HEAP32[$j>>2] = $358;
      break L39;
      break;
     }
     default: {
      $359 = HEAP32[$j>>2]|0;
      $360 = (($359) + 1)|0;
      HEAP32[$j>>2] = $360;
      $361 = HEAP32[$z>>2]|0;
      $362 = (($361) + ($359)|0);
      HEAP8[$362>>0] = 37;
      break L39;
     }
     }
    } while(0);
   }
  } while(0);
  $363 = HEAP32[$i>>2]|0;
  $364 = (($363) + 1)|0;
  HEAP32[$i>>2] = $364;
 }
 $365 = HEAP32[$j>>2]|0;
 $366 = HEAP32[$z>>2]|0;
 $367 = (($366) + ($365)|0);
 HEAP8[$367>>0] = 0;
 $368 = HEAP32[$0>>2]|0;
 $369 = HEAP32[$z>>2]|0;
 $370 = HEAP32[$z>>2]|0;
 $371 = ($370|0)==($zBuf|0);
 $372 = $371 ? (-1) : 170;
 _sqlite3_result_text($368,$369,-1,$372);
 STACKTOP = sp;return;
}
function _ctimeFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _timeFunc($3,0,0);
 STACKTOP = sp;return;
}
function _ctimestampFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _datetimeFunc($3,0,0);
 STACKTOP = sp;return;
}
function _cdateFunc($context,$NotUsed,$NotUsed2) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $NotUsed2 = $NotUsed2|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $NotUsed2;
 $3 = HEAP32[$0>>2]|0;
 _dateFunc($3,0,0);
 STACKTOP = sp;return;
}
function _isDate($context,$argc,$argv,$p) {
 $context = $context|0;
 $argc = $argc|0;
 $argv = $argv|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $eType = 0, $i = 0, $or$cond = 0;
 var $z = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $2 = sp + 20|0;
 $3 = sp + 16|0;
 $4 = sp + 12|0;
 $i = sp + 8|0;
 $z = sp + 4|0;
 $eType = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $argc;
 HEAP32[$3>>2] = $argv;
 HEAP32[$4>>2] = $p;
 $5 = HEAP32[$4>>2]|0;
 dest=$5; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $6 = HEAP32[$2>>2]|0;
 $7 = ($6|0)==(0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$4>>2]|0;
  $10 = (_setDateTimeToCurrent($8,$9)|0);
  HEAP32[$0>>2] = $10;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 $11 = HEAP32[$3>>2]|0;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_sqlite3_value_type($12)|0);
 HEAP32[$eType>>2] = $13;
 $14 = ($13|0)==(2);
 $15 = HEAP32[$eType>>2]|0;
 $16 = ($15|0)==(1);
 $or$cond = $14 | $16;
 $17 = HEAP32[$3>>2]|0;
 $18 = HEAP32[$17>>2]|0;
 do {
  if ($or$cond) {
   $19 = (+_sqlite3_value_double($18));
   $20 = $19 * 8.64E+7;
   $21 = $20 + 0.5;
   $22 = (~~$21)>>>0;
   $23 = +Math_abs($21) >= 1.0 ? $21 > 0.0 ? (~~+Math_min(+Math_floor($21 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($21 - +(~~$21 >>> 0)) / 4294967296.0) >>> 0 : 0;
   $24 = HEAP32[$4>>2]|0;
   $25 = $24;
   $26 = $25;
   HEAP32[$26>>2] = $22;
   $27 = (($25) + 4)|0;
   $28 = $27;
   HEAP32[$28>>2] = $23;
   $29 = HEAP32[$4>>2]|0;
   $30 = ((($29)) + 42|0);
   HEAP8[$30>>0] = 1;
  } else {
   $31 = (_sqlite3_value_text($18)|0);
   HEAP32[$z>>2] = $31;
   $32 = HEAP32[$z>>2]|0;
   $33 = ($32|0)!=(0|0);
   if ($33) {
    $34 = HEAP32[$1>>2]|0;
    $35 = HEAP32[$z>>2]|0;
    $36 = HEAP32[$4>>2]|0;
    $37 = (_parseDateOrTime($34,$35,$36)|0);
    $38 = ($37|0)!=(0);
    if (!($38)) {
     break;
    }
   }
   HEAP32[$0>>2] = 1;
   $56 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($56|0);
  }
 } while(0);
 HEAP32[$i>>2] = 1;
 while(1) {
  $39 = HEAP32[$i>>2]|0;
  $40 = HEAP32[$2>>2]|0;
  $41 = ($39|0)<($40|0);
  if (!($41)) {
   label = 14;
   break;
  }
  $42 = HEAP32[$i>>2]|0;
  $43 = HEAP32[$3>>2]|0;
  $44 = (($43) + ($42<<2)|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (_sqlite3_value_text($45)|0);
  HEAP32[$z>>2] = $46;
  $47 = HEAP32[$z>>2]|0;
  $48 = ($47|0)==(0|0);
  if ($48) {
   label = 12;
   break;
  }
  $49 = HEAP32[$1>>2]|0;
  $50 = HEAP32[$z>>2]|0;
  $51 = HEAP32[$4>>2]|0;
  $52 = (_parseModifier($49,$50,$51)|0);
  $53 = ($52|0)!=(0);
  if ($53) {
   label = 12;
   break;
  }
  $54 = HEAP32[$i>>2]|0;
  $55 = (($54) + 1)|0;
  HEAP32[$i>>2] = $55;
 }
 if ((label|0) == 12) {
  HEAP32[$0>>2] = 1;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 else if ((label|0) == 14) {
  HEAP32[$0>>2] = 0;
  $56 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($56|0);
 }
 return (0)|0;
}
function _computeJD($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $A = 0, $B = 0, $D = 0, $M = 0, $X1 = 0, $X2 = 0, $Y = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $Y = sp + 24|0;
 $M = sp + 20|0;
 $D = sp + 16|0;
 $A = sp + 12|0;
 $B = sp + 8|0;
 $X1 = sp + 4|0;
 $X2 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 42|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 40|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 if ($8) {
  $9 = HEAP32[$0>>2]|0;
  $10 = ((($9)) + 8|0);
  $11 = HEAP32[$10>>2]|0;
  HEAP32[$Y>>2] = $11;
  $12 = HEAP32[$0>>2]|0;
  $13 = ((($12)) + 12|0);
  $14 = HEAP32[$13>>2]|0;
  HEAP32[$M>>2] = $14;
  $15 = HEAP32[$0>>2]|0;
  $16 = ((($15)) + 16|0);
  $17 = HEAP32[$16>>2]|0;
  HEAP32[$D>>2] = $17;
 } else {
  HEAP32[$Y>>2] = 2000;
  HEAP32[$M>>2] = 1;
  HEAP32[$D>>2] = 1;
 }
 $18 = HEAP32[$M>>2]|0;
 $19 = ($18|0)<=(2);
 if ($19) {
  $20 = HEAP32[$Y>>2]|0;
  $21 = (($20) + -1)|0;
  HEAP32[$Y>>2] = $21;
  $22 = HEAP32[$M>>2]|0;
  $23 = (($22) + 12)|0;
  HEAP32[$M>>2] = $23;
 }
 $24 = HEAP32[$Y>>2]|0;
 $25 = (($24|0) / 100)&-1;
 HEAP32[$A>>2] = $25;
 $26 = HEAP32[$A>>2]|0;
 $27 = (2 - ($26))|0;
 $28 = HEAP32[$A>>2]|0;
 $29 = (($28|0) / 4)&-1;
 $30 = (($27) + ($29))|0;
 HEAP32[$B>>2] = $30;
 $31 = HEAP32[$Y>>2]|0;
 $32 = (($31) + 4716)|0;
 $33 = ($32*36525)|0;
 $34 = (($33|0) / 100)&-1;
 HEAP32[$X1>>2] = $34;
 $35 = HEAP32[$M>>2]|0;
 $36 = (($35) + 1)|0;
 $37 = ($36*306001)|0;
 $38 = (($37|0) / 10000)&-1;
 HEAP32[$X2>>2] = $38;
 $39 = HEAP32[$X1>>2]|0;
 $40 = HEAP32[$X2>>2]|0;
 $41 = (($39) + ($40))|0;
 $42 = HEAP32[$D>>2]|0;
 $43 = (($41) + ($42))|0;
 $44 = HEAP32[$B>>2]|0;
 $45 = (($43) + ($44))|0;
 $46 = (+($45|0));
 $47 = $46 - 1524.5;
 $48 = $47 * 8.64E+7;
 $49 = (~~$48)>>>0;
 $50 = +Math_abs($48) >= 1.0 ? $48 > 0.0 ? (~~+Math_min(+Math_floor($48 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($48 - +(~~$48 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $51 = HEAP32[$0>>2]|0;
 $52 = $51;
 $53 = $52;
 HEAP32[$53>>2] = $49;
 $54 = (($52) + 4)|0;
 $55 = $54;
 HEAP32[$55>>2] = $50;
 $56 = HEAP32[$0>>2]|0;
 $57 = ((($56)) + 42|0);
 HEAP8[$57>>0] = 1;
 $58 = HEAP32[$0>>2]|0;
 $59 = ((($58)) + 41|0);
 $60 = HEAP8[$59>>0]|0;
 $61 = ($60<<24>>24)!=(0);
 if (!($61)) {
  STACKTOP = sp;return;
 }
 $62 = HEAP32[$0>>2]|0;
 $63 = ((($62)) + 20|0);
 $64 = HEAP32[$63>>2]|0;
 $65 = Math_imul($64, 3600000)|0;
 $66 = HEAP32[$0>>2]|0;
 $67 = ((($66)) + 24|0);
 $68 = HEAP32[$67>>2]|0;
 $69 = ($68*60000)|0;
 $70 = (($65) + ($69))|0;
 $71 = ($70|0)<(0);
 $72 = $71 << 31 >> 31;
 $73 = HEAP32[$0>>2]|0;
 $74 = ((($73)) + 32|0);
 $75 = +HEAPF64[$74>>3];
 $76 = $75 * 1000.0;
 $77 = (~~$76)>>>0;
 $78 = +Math_abs($76) >= 1.0 ? $76 > 0.0 ? (~~+Math_min(+Math_floor($76 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($76 - +(~~$76 >>> 0)) / 4294967296.0) >>> 0 : 0;
 $79 = (_i64Add(($70|0),($72|0),($77|0),($78|0))|0);
 $80 = tempRet0;
 $81 = HEAP32[$0>>2]|0;
 $82 = $81;
 $83 = $82;
 $84 = HEAP32[$83>>2]|0;
 $85 = (($82) + 4)|0;
 $86 = $85;
 $87 = HEAP32[$86>>2]|0;
 $88 = (_i64Add(($84|0),($87|0),($79|0),($80|0))|0);
 $89 = tempRet0;
 $90 = $81;
 $91 = $90;
 HEAP32[$91>>2] = $88;
 $92 = (($90) + 4)|0;
 $93 = $92;
 HEAP32[$93>>2] = $89;
 $94 = HEAP32[$0>>2]|0;
 $95 = ((($94)) + 43|0);
 $96 = HEAP8[$95>>0]|0;
 $97 = ($96<<24>>24)!=(0);
 if (!($97)) {
  STACKTOP = sp;return;
 }
 $98 = HEAP32[$0>>2]|0;
 $99 = ((($98)) + 28|0);
 $100 = HEAP32[$99>>2]|0;
 $101 = ($100*60000)|0;
 $102 = ($101|0)<(0);
 $103 = $102 << 31 >> 31;
 $104 = HEAP32[$0>>2]|0;
 $105 = $104;
 $106 = $105;
 $107 = HEAP32[$106>>2]|0;
 $108 = (($105) + 4)|0;
 $109 = $108;
 $110 = HEAP32[$109>>2]|0;
 $111 = (_i64Subtract(($107|0),($110|0),($101|0),($103|0))|0);
 $112 = tempRet0;
 $113 = $104;
 $114 = $113;
 HEAP32[$114>>2] = $111;
 $115 = (($113) + 4)|0;
 $116 = $115;
 HEAP32[$116>>2] = $112;
 $117 = HEAP32[$0>>2]|0;
 $118 = ((($117)) + 40|0);
 HEAP8[$118>>0] = 0;
 $119 = HEAP32[$0>>2]|0;
 $120 = ((($119)) + 41|0);
 HEAP8[$120>>0] = 0;
 $121 = HEAP32[$0>>2]|0;
 $122 = ((($121)) + 43|0);
 HEAP8[$122>>0] = 0;
 STACKTOP = sp;return;
}
function _computeYMD_HMS($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 _computeYMD($1);
 $2 = HEAP32[$0>>2]|0;
 _computeHMS($2);
 STACKTOP = sp;return;
}
function _computeYMD($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0.0;
 var $27 = 0.0, $28 = 0.0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0.0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $A = 0, $B = 0, $C = 0, $D = 0, $E = 0, $X1 = 0, $Z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 28|0;
 $Z = sp + 24|0;
 $A = sp + 20|0;
 $B = sp + 16|0;
 $C = sp + 12|0;
 $D = sp + 8|0;
 $E = sp + 4|0;
 $X1 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 42|0);
 $7 = HEAP8[$6>>0]|0;
 $8 = ($7<<24>>24)!=(0);
 $9 = HEAP32[$0>>2]|0;
 if ($8) {
  $15 = $9;
  $16 = $15;
  $17 = HEAP32[$16>>2]|0;
  $18 = (($15) + 4)|0;
  $19 = $18;
  $20 = HEAP32[$19>>2]|0;
  $21 = (_i64Add(($17|0),($20|0),43200000,0)|0);
  $22 = tempRet0;
  $23 = (___divdi3(($21|0),($22|0),86400000,0)|0);
  $24 = tempRet0;
  HEAP32[$Z>>2] = $23;
  $25 = HEAP32[$Z>>2]|0;
  $26 = (+($25|0));
  $27 = $26 - 1867216.25;
  $28 = $27 / 36524.25;
  $29 = (~~(($28)));
  HEAP32[$A>>2] = $29;
  $30 = HEAP32[$Z>>2]|0;
  $31 = (($30) + 1)|0;
  $32 = HEAP32[$A>>2]|0;
  $33 = (($31) + ($32))|0;
  $34 = HEAP32[$A>>2]|0;
  $35 = (($34|0) / 4)&-1;
  $36 = (($33) - ($35))|0;
  HEAP32[$A>>2] = $36;
  $37 = HEAP32[$A>>2]|0;
  $38 = (($37) + 1524)|0;
  HEAP32[$B>>2] = $38;
  $39 = HEAP32[$B>>2]|0;
  $40 = (+($39|0));
  $41 = $40 - 122.09999999999999;
  $42 = $41 / 365.25;
  $43 = (~~(($42)));
  HEAP32[$C>>2] = $43;
  $44 = HEAP32[$C>>2]|0;
  $45 = $44 & 32767;
  $46 = ($45*36525)|0;
  $47 = (($46|0) / 100)&-1;
  HEAP32[$D>>2] = $47;
  $48 = HEAP32[$B>>2]|0;
  $49 = HEAP32[$D>>2]|0;
  $50 = (($48) - ($49))|0;
  $51 = (+($50|0));
  $52 = $51 / 30.600100000000001;
  $53 = (~~(($52)));
  HEAP32[$E>>2] = $53;
  $54 = HEAP32[$E>>2]|0;
  $55 = (+($54|0));
  $56 = 30.600100000000001 * $55;
  $57 = (~~(($56)));
  HEAP32[$X1>>2] = $57;
  $58 = HEAP32[$B>>2]|0;
  $59 = HEAP32[$D>>2]|0;
  $60 = (($58) - ($59))|0;
  $61 = HEAP32[$X1>>2]|0;
  $62 = (($60) - ($61))|0;
  $63 = HEAP32[$0>>2]|0;
  $64 = ((($63)) + 16|0);
  HEAP32[$64>>2] = $62;
  $65 = HEAP32[$E>>2]|0;
  $66 = ($65|0)<(14);
  $67 = HEAP32[$E>>2]|0;
  $68 = (($67) - 1)|0;
  $69 = (($67) - 13)|0;
  $70 = $66 ? $68 : $69;
  $71 = HEAP32[$0>>2]|0;
  $72 = ((($71)) + 12|0);
  HEAP32[$72>>2] = $70;
  $73 = HEAP32[$0>>2]|0;
  $74 = ((($73)) + 12|0);
  $75 = HEAP32[$74>>2]|0;
  $76 = ($75|0)>(2);
  $77 = HEAP32[$C>>2]|0;
  $78 = (($77) - 4716)|0;
  $79 = (($77) - 4715)|0;
  $80 = $76 ? $78 : $79;
  $81 = HEAP32[$0>>2]|0;
  $82 = ((($81)) + 8|0);
  HEAP32[$82>>2] = $80;
 } else {
  $10 = ((($9)) + 8|0);
  HEAP32[$10>>2] = 2000;
  $11 = HEAP32[$0>>2]|0;
  $12 = ((($11)) + 12|0);
  HEAP32[$12>>2] = 1;
  $13 = HEAP32[$0>>2]|0;
  $14 = ((($13)) + 16|0);
  HEAP32[$14>>2] = 1;
 }
 $83 = HEAP32[$0>>2]|0;
 $84 = ((($83)) + 40|0);
 HEAP8[$84>>0] = 1;
 STACKTOP = sp;return;
}
function _computeHMS($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0.0, $25 = 0, $26 = 0;
 var $27 = 0.0, $28 = 0, $29 = 0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0.0, $53 = 0, $54 = 0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $s = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $s = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 41|0);
 $3 = HEAP8[$2>>0]|0;
 $4 = ($3<<24>>24)!=(0);
 if ($4) {
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$0>>2]|0;
 _computeJD($5);
 $6 = HEAP32[$0>>2]|0;
 $7 = $6;
 $8 = $7;
 $9 = HEAP32[$8>>2]|0;
 $10 = (($7) + 4)|0;
 $11 = $10;
 $12 = HEAP32[$11>>2]|0;
 $13 = (_i64Add(($9|0),($12|0),43200000,0)|0);
 $14 = tempRet0;
 $15 = (___remdi3(($13|0),($14|0),86400000,0)|0);
 $16 = tempRet0;
 HEAP32[$s>>2] = $15;
 $17 = HEAP32[$s>>2]|0;
 $18 = (+($17|0));
 $19 = $18 / 1000.0;
 $20 = HEAP32[$0>>2]|0;
 $21 = ((($20)) + 32|0);
 HEAPF64[$21>>3] = $19;
 $22 = HEAP32[$0>>2]|0;
 $23 = ((($22)) + 32|0);
 $24 = +HEAPF64[$23>>3];
 $25 = (~~(($24)));
 HEAP32[$s>>2] = $25;
 $26 = HEAP32[$s>>2]|0;
 $27 = (+($26|0));
 $28 = HEAP32[$0>>2]|0;
 $29 = ((($28)) + 32|0);
 $30 = +HEAPF64[$29>>3];
 $31 = $30 - $27;
 HEAPF64[$29>>3] = $31;
 $32 = HEAP32[$s>>2]|0;
 $33 = (($32|0) / 3600)&-1;
 $34 = HEAP32[$0>>2]|0;
 $35 = ((($34)) + 20|0);
 HEAP32[$35>>2] = $33;
 $36 = HEAP32[$0>>2]|0;
 $37 = ((($36)) + 20|0);
 $38 = HEAP32[$37>>2]|0;
 $39 = ($38*3600)|0;
 $40 = HEAP32[$s>>2]|0;
 $41 = (($40) - ($39))|0;
 HEAP32[$s>>2] = $41;
 $42 = HEAP32[$s>>2]|0;
 $43 = (($42|0) / 60)&-1;
 $44 = HEAP32[$0>>2]|0;
 $45 = ((($44)) + 24|0);
 HEAP32[$45>>2] = $43;
 $46 = HEAP32[$s>>2]|0;
 $47 = HEAP32[$0>>2]|0;
 $48 = ((($47)) + 24|0);
 $49 = HEAP32[$48>>2]|0;
 $50 = ($49*60)|0;
 $51 = (($46) - ($50))|0;
 $52 = (+($51|0));
 $53 = HEAP32[$0>>2]|0;
 $54 = ((($53)) + 32|0);
 $55 = +HEAPF64[$54>>3];
 $56 = $55 + $52;
 HEAPF64[$54>>3] = $56;
 $57 = HEAP32[$0>>2]|0;
 $58 = ((($57)) + 41|0);
 HEAP8[$58>>0] = 1;
 STACKTOP = sp;return;
}
function _setDateTimeToCurrent($context,$p) {
 $context = $context|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $3 = 0;
 var $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $2 = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3StmtCurrentTime($3)|0);
 $5 = tempRet0;
 $6 = HEAP32[$2>>2]|0;
 $7 = $6;
 $8 = $7;
 HEAP32[$8>>2] = $4;
 $9 = (($7) + 4)|0;
 $10 = $9;
 HEAP32[$10>>2] = $5;
 $11 = HEAP32[$2>>2]|0;
 $12 = $11;
 $13 = $12;
 $14 = HEAP32[$13>>2]|0;
 $15 = (($12) + 4)|0;
 $16 = $15;
 $17 = HEAP32[$16>>2]|0;
 $18 = ($17|0)>(0);
 $19 = ($14>>>0)>(0);
 $20 = ($17|0)==(0);
 $21 = $20 & $19;
 $22 = $18 | $21;
 if ($22) {
  $23 = HEAP32[$2>>2]|0;
  $24 = ((($23)) + 42|0);
  HEAP8[$24>>0] = 1;
  HEAP32[$0>>2] = 0;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 } else {
  HEAP32[$0>>2] = 1;
  $25 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($25|0);
 }
 return (0)|0;
}
function _parseDateOrTime($context,$zDate,$p) {
 $context = $context|0;
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0.0, $24 = 0.0, $25 = 0.0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $r = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $2 = sp + 12|0;
 $3 = sp + 8|0;
 $r = sp;
 HEAP32[$1>>2] = $context;
 HEAP32[$2>>2] = $zDate;
 HEAP32[$3>>2] = $p;
 $4 = HEAP32[$2>>2]|0;
 $5 = HEAP32[$3>>2]|0;
 $6 = (_parseYyyyMmDd($4,$5)|0);
 $7 = ($6|0)==(0);
 if ($7) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $8 = HEAP32[$2>>2]|0;
 $9 = HEAP32[$3>>2]|0;
 $10 = (_parseHhMmSs($8,$9)|0);
 $11 = ($10|0)==(0);
 if ($11) {
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $12 = HEAP32[$2>>2]|0;
 $13 = (_sqlite3StrICmp($12,45445)|0);
 $14 = ($13|0)==(0);
 if ($14) {
  $15 = HEAP32[$1>>2]|0;
  $16 = HEAP32[$3>>2]|0;
  $17 = (_setDateTimeToCurrent($15,$16)|0);
  HEAP32[$0>>2] = $17;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 $18 = HEAP32[$2>>2]|0;
 $19 = HEAP32[$2>>2]|0;
 $20 = (_sqlite3Strlen30($19)|0);
 $21 = (_sqlite3AtoF($18,$r,$20,1)|0);
 $22 = ($21|0)!=(0);
 if ($22) {
  $23 = +HEAPF64[$r>>3];
  $24 = $23 * 8.64E+7;
  $25 = $24 + 0.5;
  $26 = (~~$25)>>>0;
  $27 = +Math_abs($25) >= 1.0 ? $25 > 0.0 ? (~~+Math_min(+Math_floor($25 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($25 - +(~~$25 >>> 0)) / 4294967296.0) >>> 0 : 0;
  $28 = HEAP32[$3>>2]|0;
  $29 = $28;
  $30 = $29;
  HEAP32[$30>>2] = $26;
  $31 = (($29) + 4)|0;
  $32 = $31;
  HEAP32[$32>>2] = $27;
  $33 = HEAP32[$3>>2]|0;
  $34 = ((($33)) + 42|0);
  HEAP8[$34>>0] = 1;
  HEAP32[$0>>2] = 0;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 } else {
  HEAP32[$0>>2] = 1;
  $35 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($35|0);
 }
 return (0)|0;
}
function _parseModifier($pCtx,$zMod,$p) {
 $pCtx = $pCtx|0;
 $zMod = $zMod|0;
 $p = $p|0;
 var $$sink = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0.0, $154 = 0, $155 = 0.0, $156 = 0.0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0.0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0;
 var $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0;
 var $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0;
 var $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0;
 var $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0;
 var $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0;
 var $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0;
 var $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0;
 var $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0;
 var $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0, $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0;
 var $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0, $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0;
 var $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0, $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0;
 var $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0, $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0;
 var $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0, $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0;
 var $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0, $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0.0, $435 = 0, $436 = 0.0, $437 = 0, $438 = 0, $439 = 0;
 var $44 = 0, $440 = 0, $441 = 0, $442 = 0.0, $443 = 0.0, $444 = 0.0, $445 = 0.0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0;
 var $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0, $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0.0, $467 = 0.0, $468 = 0.0, $469 = 0.0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0;
 var $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0, $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0.0, $491 = 0.0, $492 = 0.0, $493 = 0.0;
 var $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0, $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0;
 var $511 = 0, $512 = 0, $513 = 0, $514 = 0.0, $515 = 0.0, $516 = 0.0, $517 = 0.0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0;
 var $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0, $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0.0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0;
 var $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0, $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0;
 var $566 = 0, $567 = 0, $568 = 0, $569 = 0.0, $57 = 0, $570 = 0, $571 = 0, $572 = 0.0, $573 = 0.0, $574 = 0, $575 = 0.0, $576 = 0, $577 = 0.0, $578 = 0.0, $579 = 0.0, $58 = 0, $580 = 0.0, $581 = 0.0, $582 = 0.0, $583 = 0;
 var $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0, $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0;
 var $601 = 0, $602 = 0, $603 = 0.0, $604 = 0, $605 = 0, $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0.0, $616 = 0.0, $617 = 0, $618 = 0.0, $619 = 0;
 var $62 = 0, $620 = 0.0, $621 = 0.0, $622 = 0.0, $623 = 0.0, $624 = 0.0, $625 = 0.0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0;
 var $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0, $642 = 0, $643 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0;
 var $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0;
 var $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $Z = 0, $c1 = 0, $day = 0, $n = 0, $or$cond = 0, $or$cond3 = 0, $or$cond6 = 0, $r = 0, $rRounder = 0, $rc = 0, $tx = 0, $x = 0, $y = 0, $y1 = 0, $z = 0;
 var $z2 = 0, $zBuf = 0, dest = 0, label = 0, sp = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 160|0;
 $0 = sp + 124|0;
 $1 = sp + 120|0;
 $2 = sp + 116|0;
 $rc = sp + 112|0;
 $n = sp + 108|0;
 $r = sp + 80|0;
 $z = sp + 104|0;
 $zBuf = sp + 128|0;
 $c1 = sp + 72|0;
 $Z = sp + 64|0;
 $rRounder = sp + 56|0;
 $z2 = sp + 100|0;
 $tx = sp + 8|0;
 $day = sp;
 $x = sp + 96|0;
 $y = sp + 92|0;
 $y1 = sp + 88|0;
 HEAP32[$0>>2] = $pCtx;
 HEAP32[$1>>2] = $zMod;
 HEAP32[$2>>2] = $p;
 HEAP32[$rc>>2] = 1;
 HEAP32[$z>>2] = $zBuf;
 HEAP32[$n>>2] = 0;
 while(1) {
  $3 = HEAP32[$n>>2]|0;
  $4 = ($3|0)<(29);
  if ($4) {
   $5 = HEAP32[$n>>2]|0;
   $6 = HEAP32[$1>>2]|0;
   $7 = (($6) + ($5)|0);
   $8 = HEAP8[$7>>0]|0;
   $9 = $8 << 24 >> 24;
   $10 = ($9|0)!=(0);
   $643 = $10;
  } else {
   $643 = 0;
  }
  $11 = HEAP32[$n>>2]|0;
  if (!($643)) {
   break;
  }
  $12 = HEAP32[$1>>2]|0;
  $13 = (($12) + ($11)|0);
  $14 = HEAP8[$13>>0]|0;
  $15 = $14&255;
  $16 = (17613 + ($15)|0);
  $17 = HEAP8[$16>>0]|0;
  $18 = HEAP32[$n>>2]|0;
  $19 = HEAP32[$z>>2]|0;
  $20 = (($19) + ($18)|0);
  HEAP8[$20>>0] = $17;
  $21 = HEAP32[$n>>2]|0;
  $22 = (($21) + 1)|0;
  HEAP32[$n>>2] = $22;
 }
 $23 = HEAP32[$z>>2]|0;
 $24 = (($23) + ($11)|0);
 HEAP8[$24>>0] = 0;
 $25 = HEAP32[$z>>2]|0;
 $26 = HEAP8[$25>>0]|0;
 $27 = $26 << 24 >> 24;
 switch ($27|0) {
 case 108:  {
  $28 = HEAP32[$z>>2]|0;
  $29 = (_strcmp($28,45325)|0);
  $30 = ($29|0)==(0);
  if (!($30)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $31 = HEAP32[$2>>2]|0;
  _computeJD($31);
  $32 = HEAP32[$2>>2]|0;
  $33 = HEAP32[$0>>2]|0;
  $34 = (_localtimeOffset($32,$33,$rc)|0);
  $35 = tempRet0;
  $36 = HEAP32[$2>>2]|0;
  $37 = $36;
  $38 = $37;
  $39 = HEAP32[$38>>2]|0;
  $40 = (($37) + 4)|0;
  $41 = $40;
  $42 = HEAP32[$41>>2]|0;
  $43 = (_i64Add(($39|0),($42|0),($34|0),($35|0))|0);
  $44 = tempRet0;
  $45 = $36;
  $46 = $45;
  HEAP32[$46>>2] = $43;
  $47 = (($45) + 4)|0;
  $48 = $47;
  HEAP32[$48>>2] = $44;
  $49 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($49);
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 117:  {
  $50 = HEAP32[$z>>2]|0;
  $51 = (_strcmp($50,45335)|0);
  $52 = ($51|0)==(0);
  if ($52) {
   $53 = HEAP32[$2>>2]|0;
   $54 = ((($53)) + 42|0);
   $55 = HEAP8[$54>>0]|0;
   $56 = $55 << 24 >> 24;
   $57 = ($56|0)!=(0);
   if ($57) {
    $58 = HEAP32[$2>>2]|0;
    $59 = $58;
    $60 = $59;
    $61 = HEAP32[$60>>2]|0;
    $62 = (($59) + 4)|0;
    $63 = $62;
    $64 = HEAP32[$63>>2]|0;
    $65 = (_i64Add(($61|0),($64|0),43200,0)|0);
    $66 = tempRet0;
    $67 = (___divdi3(($65|0),($66|0),86400,0)|0);
    $68 = tempRet0;
    $69 = (_i64Add(($67|0),($68|0),1045635584,49096)|0);
    $70 = tempRet0;
    $71 = HEAP32[$2>>2]|0;
    $72 = $71;
    $73 = $72;
    HEAP32[$73>>2] = $69;
    $74 = (($72) + 4)|0;
    $75 = $74;
    HEAP32[$75>>2] = $70;
    $76 = HEAP32[$2>>2]|0;
    _clearYMD_HMS_TZ($76);
    HEAP32[$rc>>2] = 0;
    $642 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($642|0);
   }
  }
  $77 = HEAP32[$z>>2]|0;
  $78 = (_strcmp($77,45345)|0);
  $79 = ($78|0)==(0);
  if (!($79)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $80 = HEAP32[$2>>2]|0;
  $81 = ((($80)) + 44|0);
  $82 = HEAP8[$81>>0]|0;
  $83 = $82 << 24 >> 24;
  $84 = ($83|0)==(0);
  if (!($84)) {
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $85 = HEAP32[$2>>2]|0;
  _computeJD($85);
  $86 = HEAP32[$2>>2]|0;
  $87 = HEAP32[$0>>2]|0;
  $88 = (_localtimeOffset($86,$87,$rc)|0);
  $89 = tempRet0;
  $90 = $c1;
  $91 = $90;
  HEAP32[$91>>2] = $88;
  $92 = (($90) + 4)|0;
  $93 = $92;
  HEAP32[$93>>2] = $89;
  $94 = HEAP32[$rc>>2]|0;
  $95 = ($94|0)==(0);
  if ($95) {
   $96 = $c1;
   $97 = $96;
   $98 = HEAP32[$97>>2]|0;
   $99 = (($96) + 4)|0;
   $100 = $99;
   $101 = HEAP32[$100>>2]|0;
   $102 = HEAP32[$2>>2]|0;
   $103 = $102;
   $104 = $103;
   $105 = HEAP32[$104>>2]|0;
   $106 = (($103) + 4)|0;
   $107 = $106;
   $108 = HEAP32[$107>>2]|0;
   $109 = (_i64Subtract(($105|0),($108|0),($98|0),($101|0))|0);
   $110 = tempRet0;
   $111 = $102;
   $112 = $111;
   HEAP32[$112>>2] = $109;
   $113 = (($111) + 4)|0;
   $114 = $113;
   HEAP32[$114>>2] = $110;
   $115 = HEAP32[$2>>2]|0;
   _clearYMD_HMS_TZ($115);
   $116 = $c1;
   $117 = $116;
   $118 = HEAP32[$117>>2]|0;
   $119 = (($116) + 4)|0;
   $120 = $119;
   $121 = HEAP32[$120>>2]|0;
   $122 = HEAP32[$2>>2]|0;
   $123 = HEAP32[$0>>2]|0;
   $124 = (_localtimeOffset($122,$123,$rc)|0);
   $125 = tempRet0;
   $126 = (_i64Subtract(($118|0),($121|0),($124|0),($125|0))|0);
   $127 = tempRet0;
   $128 = HEAP32[$2>>2]|0;
   $129 = $128;
   $130 = $129;
   $131 = HEAP32[$130>>2]|0;
   $132 = (($129) + 4)|0;
   $133 = $132;
   $134 = HEAP32[$133>>2]|0;
   $135 = (_i64Add(($131|0),($134|0),($126|0),($127|0))|0);
   $136 = tempRet0;
   $137 = $128;
   $138 = $137;
   HEAP32[$138>>2] = $135;
   $139 = (($137) + 4)|0;
   $140 = $139;
   HEAP32[$140>>2] = $136;
  }
  $141 = HEAP32[$2>>2]|0;
  $142 = ((($141)) + 44|0);
  HEAP8[$142>>0] = 1;
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 119:  {
  $143 = HEAP32[$z>>2]|0;
  $144 = (_strncmp($143,45349,8)|0);
  $145 = ($144|0)==(0);
  if (!($145)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $146 = HEAP32[$z>>2]|0;
  $147 = ((($146)) + 8|0);
  $148 = HEAP32[$z>>2]|0;
  $149 = ((($148)) + 8|0);
  $150 = (_sqlite3Strlen30($149)|0);
  $151 = (_sqlite3AtoF($147,$r,$150,1)|0);
  $152 = ($151|0)!=(0);
  if (!($152)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $153 = +HEAPF64[$r>>3];
  $154 = (~~(($153)));
  HEAP32[$n>>2] = $154;
  $155 = (+($154|0));
  $156 = +HEAPF64[$r>>3];
  $157 = $155 == $156;
  $158 = HEAP32[$n>>2]|0;
  $159 = ($158|0)>=(0);
  $or$cond = $157 & $159;
  $160 = +HEAPF64[$r>>3];
  $161 = $160 < 7.0;
  $or$cond3 = $or$cond & $161;
  if (!($or$cond3)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $162 = HEAP32[$2>>2]|0;
  _computeYMD_HMS($162);
  $163 = HEAP32[$2>>2]|0;
  $164 = ((($163)) + 43|0);
  HEAP8[$164>>0] = 0;
  $165 = HEAP32[$2>>2]|0;
  $166 = ((($165)) + 42|0);
  HEAP8[$166>>0] = 0;
  $167 = HEAP32[$2>>2]|0;
  _computeJD($167);
  $168 = HEAP32[$2>>2]|0;
  $169 = $168;
  $170 = $169;
  $171 = HEAP32[$170>>2]|0;
  $172 = (($169) + 4)|0;
  $173 = $172;
  $174 = HEAP32[$173>>2]|0;
  $175 = (_i64Add(($171|0),($174|0),129600000,0)|0);
  $176 = tempRet0;
  $177 = (___divdi3(($175|0),($176|0),86400000,0)|0);
  $178 = tempRet0;
  $179 = (___remdi3(($177|0),($178|0),7,0)|0);
  $180 = tempRet0;
  $181 = $Z;
  $182 = $181;
  HEAP32[$182>>2] = $179;
  $183 = (($181) + 4)|0;
  $184 = $183;
  HEAP32[$184>>2] = $180;
  $185 = $Z;
  $186 = $185;
  $187 = HEAP32[$186>>2]|0;
  $188 = (($185) + 4)|0;
  $189 = $188;
  $190 = HEAP32[$189>>2]|0;
  $191 = HEAP32[$n>>2]|0;
  $192 = ($191|0)<(0);
  $193 = $192 << 31 >> 31;
  $194 = ($190|0)>($193|0);
  $195 = ($187>>>0)>($191>>>0);
  $196 = ($190|0)==($193|0);
  $197 = $196 & $195;
  $198 = $194 | $197;
  if ($198) {
   $199 = $Z;
   $200 = $199;
   $201 = HEAP32[$200>>2]|0;
   $202 = (($199) + 4)|0;
   $203 = $202;
   $204 = HEAP32[$203>>2]|0;
   $205 = (_i64Subtract(($201|0),($204|0),7,0)|0);
   $206 = tempRet0;
   $207 = $Z;
   $208 = $207;
   HEAP32[$208>>2] = $205;
   $209 = (($207) + 4)|0;
   $210 = $209;
   HEAP32[$210>>2] = $206;
  }
  $211 = HEAP32[$n>>2]|0;
  $212 = ($211|0)<(0);
  $213 = $212 << 31 >> 31;
  $214 = $Z;
  $215 = $214;
  $216 = HEAP32[$215>>2]|0;
  $217 = (($214) + 4)|0;
  $218 = $217;
  $219 = HEAP32[$218>>2]|0;
  $220 = (_i64Subtract(($211|0),($213|0),($216|0),($219|0))|0);
  $221 = tempRet0;
  $222 = (___muldi3(($220|0),($221|0),86400000,0)|0);
  $223 = tempRet0;
  $224 = HEAP32[$2>>2]|0;
  $225 = $224;
  $226 = $225;
  $227 = HEAP32[$226>>2]|0;
  $228 = (($225) + 4)|0;
  $229 = $228;
  $230 = HEAP32[$229>>2]|0;
  $231 = (_i64Add(($227|0),($230|0),($222|0),($223|0))|0);
  $232 = tempRet0;
  $233 = $224;
  $234 = $233;
  HEAP32[$234>>2] = $231;
  $235 = (($233) + 4)|0;
  $236 = $235;
  HEAP32[$236>>2] = $232;
  $237 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($237);
  HEAP32[$rc>>2] = 0;
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 115:  {
  $238 = HEAP32[$z>>2]|0;
  $239 = (_strncmp($238,45358,9)|0);
  $240 = ($239|0)!=(0);
  if ($240) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $241 = HEAP32[$z>>2]|0;
  $242 = ((($241)) + 9|0);
  HEAP32[$z>>2] = $242;
  $243 = HEAP32[$2>>2]|0;
  _computeYMD($243);
  $244 = HEAP32[$2>>2]|0;
  $245 = ((($244)) + 41|0);
  HEAP8[$245>>0] = 1;
  $246 = HEAP32[$2>>2]|0;
  $247 = ((($246)) + 24|0);
  HEAP32[$247>>2] = 0;
  $248 = HEAP32[$2>>2]|0;
  $249 = ((($248)) + 20|0);
  HEAP32[$249>>2] = 0;
  $250 = HEAP32[$2>>2]|0;
  $251 = ((($250)) + 32|0);
  HEAPF64[$251>>3] = 0.0;
  $252 = HEAP32[$2>>2]|0;
  $253 = ((($252)) + 43|0);
  HEAP8[$253>>0] = 0;
  $254 = HEAP32[$2>>2]|0;
  $255 = ((($254)) + 42|0);
  HEAP8[$255>>0] = 0;
  $256 = HEAP32[$z>>2]|0;
  $257 = (_strcmp($256,45368)|0);
  $258 = ($257|0)==(0);
  if ($258) {
   $259 = HEAP32[$2>>2]|0;
   $260 = ((($259)) + 16|0);
   HEAP32[$260>>2] = 1;
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $261 = HEAP32[$z>>2]|0;
  $262 = (_strcmp($261,45374)|0);
  $263 = ($262|0)==(0);
  if ($263) {
   $264 = HEAP32[$2>>2]|0;
   _computeYMD($264);
   $265 = HEAP32[$2>>2]|0;
   $266 = ((($265)) + 12|0);
   HEAP32[$266>>2] = 1;
   $267 = HEAP32[$2>>2]|0;
   $268 = ((($267)) + 16|0);
   HEAP32[$268>>2] = 1;
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $269 = HEAP32[$z>>2]|0;
  $270 = (_strcmp($269,45379)|0);
  $271 = ($270|0)==(0);
  if (!($271)) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  HEAP32[$rc>>2] = 0;
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 case 57: case 56: case 55: case 54: case 53: case 52: case 51: case 50: case 49: case 48: case 45: case 43:  {
  HEAP32[$n>>2] = 1;
  while(1) {
   $272 = HEAP32[$n>>2]|0;
   $273 = HEAP32[$z>>2]|0;
   $274 = (($273) + ($272)|0);
   $275 = HEAP8[$274>>0]|0;
   $276 = $275 << 24 >> 24;
   $277 = ($276|0)!=(0);
   if (!($277)) {
    break;
   }
   $278 = HEAP32[$n>>2]|0;
   $279 = HEAP32[$z>>2]|0;
   $280 = (($279) + ($278)|0);
   $281 = HEAP8[$280>>0]|0;
   $282 = $281 << 24 >> 24;
   $283 = ($282|0)!=(58);
   if (!($283)) {
    break;
   }
   $284 = HEAP32[$n>>2]|0;
   $285 = HEAP32[$z>>2]|0;
   $286 = (($285) + ($284)|0);
   $287 = HEAP8[$286>>0]|0;
   $288 = $287&255;
   $289 = (18403 + ($288)|0);
   $290 = HEAP8[$289>>0]|0;
   $291 = $290&255;
   $292 = $291 & 1;
   $293 = ($292|0)!=(0);
   $294 = $293 ^ 1;
   if (!($294)) {
    break;
   }
   $295 = HEAP32[$n>>2]|0;
   $296 = (($295) + 1)|0;
   HEAP32[$n>>2] = $296;
  }
  $297 = HEAP32[$z>>2]|0;
  $298 = HEAP32[$n>>2]|0;
  $299 = (_sqlite3AtoF($297,$r,$298,1)|0);
  $300 = ($299|0)!=(0);
  if (!($300)) {
   HEAP32[$rc>>2] = 1;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $301 = HEAP32[$n>>2]|0;
  $302 = HEAP32[$z>>2]|0;
  $303 = (($302) + ($301)|0);
  $304 = HEAP8[$303>>0]|0;
  $305 = $304 << 24 >> 24;
  $306 = ($305|0)==(58);
  if ($306) {
   $307 = HEAP32[$z>>2]|0;
   HEAP32[$z2>>2] = $307;
   $308 = HEAP32[$z2>>2]|0;
   $309 = HEAP8[$308>>0]|0;
   $310 = $309&255;
   $311 = (18403 + ($310)|0);
   $312 = HEAP8[$311>>0]|0;
   $313 = $312&255;
   $314 = $313 & 4;
   $315 = ($314|0)!=(0);
   if (!($315)) {
    $316 = HEAP32[$z2>>2]|0;
    $317 = ((($316)) + 1|0);
    HEAP32[$z2>>2] = $317;
   }
   dest=$tx; stop=dest+48|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
   $318 = HEAP32[$z2>>2]|0;
   $319 = (_parseHhMmSs($318,$tx)|0);
   $320 = ($319|0)!=(0);
   if ($320) {
    $642 = HEAP32[$rc>>2]|0;
    STACKTOP = sp;return ($642|0);
   }
   _computeJD($tx);
   $321 = $tx;
   $322 = $321;
   $323 = HEAP32[$322>>2]|0;
   $324 = (($321) + 4)|0;
   $325 = $324;
   $326 = HEAP32[$325>>2]|0;
   $327 = (_i64Subtract(($323|0),($326|0),43200000,0)|0);
   $328 = tempRet0;
   $329 = $tx;
   $330 = $329;
   HEAP32[$330>>2] = $327;
   $331 = (($329) + 4)|0;
   $332 = $331;
   HEAP32[$332>>2] = $328;
   $333 = $tx;
   $334 = $333;
   $335 = HEAP32[$334>>2]|0;
   $336 = (($333) + 4)|0;
   $337 = $336;
   $338 = HEAP32[$337>>2]|0;
   $339 = (___divdi3(($335|0),($338|0),86400000,0)|0);
   $340 = tempRet0;
   $341 = $day;
   $342 = $341;
   HEAP32[$342>>2] = $339;
   $343 = (($341) + 4)|0;
   $344 = $343;
   HEAP32[$344>>2] = $340;
   $345 = $day;
   $346 = $345;
   $347 = HEAP32[$346>>2]|0;
   $348 = (($345) + 4)|0;
   $349 = $348;
   $350 = HEAP32[$349>>2]|0;
   $351 = (___muldi3(($347|0),($350|0),86400000,0)|0);
   $352 = tempRet0;
   $353 = $tx;
   $354 = $353;
   $355 = HEAP32[$354>>2]|0;
   $356 = (($353) + 4)|0;
   $357 = $356;
   $358 = HEAP32[$357>>2]|0;
   $359 = (_i64Subtract(($355|0),($358|0),($351|0),($352|0))|0);
   $360 = tempRet0;
   $361 = $tx;
   $362 = $361;
   HEAP32[$362>>2] = $359;
   $363 = (($361) + 4)|0;
   $364 = $363;
   HEAP32[$364>>2] = $360;
   $365 = HEAP32[$z>>2]|0;
   $366 = HEAP8[$365>>0]|0;
   $367 = $366 << 24 >> 24;
   $368 = ($367|0)==(45);
   if ($368) {
    $369 = $tx;
    $370 = $369;
    $371 = HEAP32[$370>>2]|0;
    $372 = (($369) + 4)|0;
    $373 = $372;
    $374 = HEAP32[$373>>2]|0;
    $375 = (_i64Subtract(0,0,($371|0),($374|0))|0);
    $376 = tempRet0;
    $377 = $tx;
    $378 = $377;
    HEAP32[$378>>2] = $375;
    $379 = (($377) + 4)|0;
    $380 = $379;
    HEAP32[$380>>2] = $376;
   }
   $381 = HEAP32[$2>>2]|0;
   _computeJD($381);
   $382 = HEAP32[$2>>2]|0;
   _clearYMD_HMS_TZ($382);
   $383 = $tx;
   $384 = $383;
   $385 = HEAP32[$384>>2]|0;
   $386 = (($383) + 4)|0;
   $387 = $386;
   $388 = HEAP32[$387>>2]|0;
   $389 = HEAP32[$2>>2]|0;
   $390 = $389;
   $391 = $390;
   $392 = HEAP32[$391>>2]|0;
   $393 = (($390) + 4)|0;
   $394 = $393;
   $395 = HEAP32[$394>>2]|0;
   $396 = (_i64Add(($392|0),($395|0),($385|0),($388|0))|0);
   $397 = tempRet0;
   $398 = $389;
   $399 = $398;
   HEAP32[$399>>2] = $396;
   $400 = (($398) + 4)|0;
   $401 = $400;
   HEAP32[$401>>2] = $397;
   HEAP32[$rc>>2] = 0;
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $402 = HEAP32[$n>>2]|0;
  $403 = HEAP32[$z>>2]|0;
  $404 = (($403) + ($402)|0);
  HEAP32[$z>>2] = $404;
  while(1) {
   $405 = HEAP32[$z>>2]|0;
   $406 = HEAP8[$405>>0]|0;
   $407 = $406&255;
   $408 = (18403 + ($407)|0);
   $409 = HEAP8[$408>>0]|0;
   $410 = $409&255;
   $411 = $410 & 1;
   $412 = ($411|0)!=(0);
   $413 = HEAP32[$z>>2]|0;
   if (!($412)) {
    break;
   }
   $414 = ((($413)) + 1|0);
   HEAP32[$z>>2] = $414;
  }
  $415 = (_sqlite3Strlen30($413)|0);
  HEAP32[$n>>2] = $415;
  $416 = HEAP32[$n>>2]|0;
  $417 = ($416|0)>(10);
  $418 = HEAP32[$n>>2]|0;
  $419 = ($418|0)<(3);
  $or$cond6 = $417 | $419;
  if ($or$cond6) {
   $642 = HEAP32[$rc>>2]|0;
   STACKTOP = sp;return ($642|0);
  }
  $420 = HEAP32[$n>>2]|0;
  $421 = (($420) - 1)|0;
  $422 = HEAP32[$z>>2]|0;
  $423 = (($422) + ($421)|0);
  $424 = HEAP8[$423>>0]|0;
  $425 = $424 << 24 >> 24;
  $426 = ($425|0)==(115);
  if ($426) {
   $427 = HEAP32[$n>>2]|0;
   $428 = (($427) - 1)|0;
   $429 = HEAP32[$z>>2]|0;
   $430 = (($429) + ($428)|0);
   HEAP8[$430>>0] = 0;
   $431 = HEAP32[$n>>2]|0;
   $432 = (($431) + -1)|0;
   HEAP32[$n>>2] = $432;
  }
  $433 = HEAP32[$2>>2]|0;
  _computeJD($433);
  HEAP32[$rc>>2] = 0;
  $434 = +HEAPF64[$r>>3];
  $435 = $434 < 0.0;
  $436 = $435 ? -0.5 : 0.5;
  HEAPF64[$rRounder>>3] = $436;
  $437 = HEAP32[$n>>2]|0;
  $438 = ($437|0)==(3);
  if ($438) {
   $439 = HEAP32[$z>>2]|0;
   $440 = (_strcmp($439,45379)|0);
   $441 = ($440|0)==(0);
   if ($441) {
    $442 = +HEAPF64[$r>>3];
    $443 = $442 * 8.64E+7;
    $444 = +HEAPF64[$rRounder>>3];
    $445 = $443 + $444;
    $446 = (~~$445)>>>0;
    $447 = +Math_abs($445) >= 1.0 ? $445 > 0.0 ? (~~+Math_min(+Math_floor($445 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($445 - +(~~$445 >>> 0)) / 4294967296.0) >>> 0 : 0;
    $448 = HEAP32[$2>>2]|0;
    $449 = $448;
    $450 = $449;
    $451 = HEAP32[$450>>2]|0;
    $452 = (($449) + 4)|0;
    $453 = $452;
    $454 = HEAP32[$453>>2]|0;
    $455 = (_i64Add(($451|0),($454|0),($446|0),($447|0))|0);
    $456 = tempRet0;
    $457 = $448;
    $458 = $457;
    HEAP32[$458>>2] = $455;
    $459 = (($457) + 4)|0;
    $460 = $459;
    HEAP32[$460>>2] = $456;
   } else {
    label = 54;
   }
  } else {
   label = 54;
  }
  do {
   if ((label|0) == 54) {
    $461 = HEAP32[$n>>2]|0;
    $462 = ($461|0)==(4);
    if ($462) {
     $463 = HEAP32[$z>>2]|0;
     $464 = (_strcmp($463,45383)|0);
     $465 = ($464|0)==(0);
     if ($465) {
      $466 = +HEAPF64[$r>>3];
      $467 = $466 * 3.6E+6;
      $468 = +HEAPF64[$rRounder>>3];
      $469 = $467 + $468;
      $470 = (~~$469)>>>0;
      $471 = +Math_abs($469) >= 1.0 ? $469 > 0.0 ? (~~+Math_min(+Math_floor($469 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($469 - +(~~$469 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $472 = HEAP32[$2>>2]|0;
      $473 = $472;
      $474 = $473;
      $475 = HEAP32[$474>>2]|0;
      $476 = (($473) + 4)|0;
      $477 = $476;
      $478 = HEAP32[$477>>2]|0;
      $479 = (_i64Add(($475|0),($478|0),($470|0),($471|0))|0);
      $480 = tempRet0;
      $481 = $472;
      $482 = $481;
      HEAP32[$482>>2] = $479;
      $483 = (($481) + 4)|0;
      $484 = $483;
      HEAP32[$484>>2] = $480;
      break;
     }
    }
    $485 = HEAP32[$n>>2]|0;
    $486 = ($485|0)==(6);
    if ($486) {
     $487 = HEAP32[$z>>2]|0;
     $488 = (_strcmp($487,45388)|0);
     $489 = ($488|0)==(0);
     if ($489) {
      $490 = +HEAPF64[$r>>3];
      $491 = $490 * 6.0E+4;
      $492 = +HEAPF64[$rRounder>>3];
      $493 = $491 + $492;
      $494 = (~~$493)>>>0;
      $495 = +Math_abs($493) >= 1.0 ? $493 > 0.0 ? (~~+Math_min(+Math_floor($493 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($493 - +(~~$493 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $496 = HEAP32[$2>>2]|0;
      $497 = $496;
      $498 = $497;
      $499 = HEAP32[$498>>2]|0;
      $500 = (($497) + 4)|0;
      $501 = $500;
      $502 = HEAP32[$501>>2]|0;
      $503 = (_i64Add(($499|0),($502|0),($494|0),($495|0))|0);
      $504 = tempRet0;
      $505 = $496;
      $506 = $505;
      HEAP32[$506>>2] = $503;
      $507 = (($505) + 4)|0;
      $508 = $507;
      HEAP32[$508>>2] = $504;
      break;
     }
    }
    $509 = HEAP32[$n>>2]|0;
    $510 = ($509|0)==(6);
    if ($510) {
     $511 = HEAP32[$z>>2]|0;
     $512 = (_strcmp($511,45395)|0);
     $513 = ($512|0)==(0);
     if ($513) {
      $514 = +HEAPF64[$r>>3];
      $515 = $514 * 1000.0;
      $516 = +HEAPF64[$rRounder>>3];
      $517 = $515 + $516;
      $518 = (~~$517)>>>0;
      $519 = +Math_abs($517) >= 1.0 ? $517 > 0.0 ? (~~+Math_min(+Math_floor($517 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($517 - +(~~$517 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $520 = HEAP32[$2>>2]|0;
      $521 = $520;
      $522 = $521;
      $523 = HEAP32[$522>>2]|0;
      $524 = (($521) + 4)|0;
      $525 = $524;
      $526 = HEAP32[$525>>2]|0;
      $527 = (_i64Add(($523|0),($526|0),($518|0),($519|0))|0);
      $528 = tempRet0;
      $529 = $520;
      $530 = $529;
      HEAP32[$530>>2] = $527;
      $531 = (($529) + 4)|0;
      $532 = $531;
      HEAP32[$532>>2] = $528;
      break;
     }
    }
    $533 = HEAP32[$n>>2]|0;
    $534 = ($533|0)==(5);
    if ($534) {
     $535 = HEAP32[$z>>2]|0;
     $536 = (_strcmp($535,45368)|0);
     $537 = ($536|0)==(0);
     if ($537) {
      $538 = HEAP32[$2>>2]|0;
      _computeYMD_HMS($538);
      $539 = +HEAPF64[$r>>3];
      $540 = (~~(($539)));
      $541 = HEAP32[$2>>2]|0;
      $542 = ((($541)) + 12|0);
      $543 = HEAP32[$542>>2]|0;
      $544 = (($543) + ($540))|0;
      HEAP32[$542>>2] = $544;
      $545 = HEAP32[$2>>2]|0;
      $546 = ((($545)) + 12|0);
      $547 = HEAP32[$546>>2]|0;
      $548 = ($547|0)>(0);
      $549 = HEAP32[$2>>2]|0;
      $550 = ((($549)) + 12|0);
      $551 = HEAP32[$550>>2]|0;
      $552 = (($551) - 1)|0;
      $553 = (($551) - 12)|0;
      $$sink = $548 ? $552 : $553;
      $554 = (($$sink|0) / 12)&-1;
      HEAP32[$x>>2] = $554;
      $555 = HEAP32[$x>>2]|0;
      $556 = HEAP32[$2>>2]|0;
      $557 = ((($556)) + 8|0);
      $558 = HEAP32[$557>>2]|0;
      $559 = (($558) + ($555))|0;
      HEAP32[$557>>2] = $559;
      $560 = HEAP32[$x>>2]|0;
      $561 = ($560*12)|0;
      $562 = HEAP32[$2>>2]|0;
      $563 = ((($562)) + 12|0);
      $564 = HEAP32[$563>>2]|0;
      $565 = (($564) - ($561))|0;
      HEAP32[$563>>2] = $565;
      $566 = HEAP32[$2>>2]|0;
      $567 = ((($566)) + 42|0);
      HEAP8[$567>>0] = 0;
      $568 = HEAP32[$2>>2]|0;
      _computeJD($568);
      $569 = +HEAPF64[$r>>3];
      $570 = (~~(($569)));
      HEAP32[$y>>2] = $570;
      $571 = HEAP32[$y>>2]|0;
      $572 = (+($571|0));
      $573 = +HEAPF64[$r>>3];
      $574 = $572 != $573;
      if (!($574)) {
       break;
      }
      $575 = +HEAPF64[$r>>3];
      $576 = HEAP32[$y>>2]|0;
      $577 = (+($576|0));
      $578 = $575 - $577;
      $579 = $578 * 30.0;
      $580 = $579 * 8.64E+7;
      $581 = +HEAPF64[$rRounder>>3];
      $582 = $580 + $581;
      $583 = (~~$582)>>>0;
      $584 = +Math_abs($582) >= 1.0 ? $582 > 0.0 ? (~~+Math_min(+Math_floor($582 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($582 - +(~~$582 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $585 = HEAP32[$2>>2]|0;
      $586 = $585;
      $587 = $586;
      $588 = HEAP32[$587>>2]|0;
      $589 = (($586) + 4)|0;
      $590 = $589;
      $591 = HEAP32[$590>>2]|0;
      $592 = (_i64Add(($588|0),($591|0),($583|0),($584|0))|0);
      $593 = tempRet0;
      $594 = $585;
      $595 = $594;
      HEAP32[$595>>2] = $592;
      $596 = (($594) + 4)|0;
      $597 = $596;
      HEAP32[$597>>2] = $593;
      break;
     }
    }
    $598 = HEAP32[$n>>2]|0;
    $599 = ($598|0)==(4);
    if ($599) {
     $600 = HEAP32[$z>>2]|0;
     $601 = (_strcmp($600,45374)|0);
     $602 = ($601|0)==(0);
     if ($602) {
      $603 = +HEAPF64[$r>>3];
      $604 = (~~(($603)));
      HEAP32[$y1>>2] = $604;
      $605 = HEAP32[$2>>2]|0;
      _computeYMD_HMS($605);
      $606 = HEAP32[$y1>>2]|0;
      $607 = HEAP32[$2>>2]|0;
      $608 = ((($607)) + 8|0);
      $609 = HEAP32[$608>>2]|0;
      $610 = (($609) + ($606))|0;
      HEAP32[$608>>2] = $610;
      $611 = HEAP32[$2>>2]|0;
      $612 = ((($611)) + 42|0);
      HEAP8[$612>>0] = 0;
      $613 = HEAP32[$2>>2]|0;
      _computeJD($613);
      $614 = HEAP32[$y1>>2]|0;
      $615 = (+($614|0));
      $616 = +HEAPF64[$r>>3];
      $617 = $615 != $616;
      if (!($617)) {
       break;
      }
      $618 = +HEAPF64[$r>>3];
      $619 = HEAP32[$y1>>2]|0;
      $620 = (+($619|0));
      $621 = $618 - $620;
      $622 = $621 * 365.0;
      $623 = $622 * 8.64E+7;
      $624 = +HEAPF64[$rRounder>>3];
      $625 = $623 + $624;
      $626 = (~~$625)>>>0;
      $627 = +Math_abs($625) >= 1.0 ? $625 > 0.0 ? (~~+Math_min(+Math_floor($625 / 4294967296.0), 4294967295.0)) >>> 0 : ~~+Math_ceil(($625 - +(~~$625 >>> 0)) / 4294967296.0) >>> 0 : 0;
      $628 = HEAP32[$2>>2]|0;
      $629 = $628;
      $630 = $629;
      $631 = HEAP32[$630>>2]|0;
      $632 = (($629) + 4)|0;
      $633 = $632;
      $634 = HEAP32[$633>>2]|0;
      $635 = (_i64Add(($631|0),($634|0),($626|0),($627|0))|0);
      $636 = tempRet0;
      $637 = $628;
      $638 = $637;
      HEAP32[$638>>2] = $635;
      $639 = (($637) + 4)|0;
      $640 = $639;
      HEAP32[$640>>2] = $636;
      break;
     }
    }
    HEAP32[$rc>>2] = 1;
   }
  } while(0);
  $641 = HEAP32[$2>>2]|0;
  _clearYMD_HMS_TZ($641);
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
  break;
 }
 default: {
  $642 = HEAP32[$rc>>2]|0;
  STACKTOP = sp;return ($642|0);
 }
 }
 return (0)|0;
}
function _localtimeOffset($p,$pCtx,$pRc) {
 $p = $p|0;
 $pCtx = $pCtx|0;
 $pRc = $pRc|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0, $21 = 0, $22 = 0.0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0.0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $s = 0, $sLocal = 0, $t = 0, $x = 0, $y = 0, dest = 0, label = 0;
 var sp = 0, src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 176|0;
 $0 = sp + 96|0;
 $1 = sp + 164|0;
 $2 = sp + 160|0;
 $3 = sp + 156|0;
 $x = sp + 48|0;
 $y = sp;
 $t = sp + 152|0;
 $sLocal = sp + 108|0;
 $s = sp + 104|0;
 HEAP32[$1>>2] = $p;
 HEAP32[$2>>2] = $pCtx;
 HEAP32[$3>>2] = $pRc;
 dest=$sLocal; stop=dest+44|0; do { HEAP32[dest>>2]=0|0; dest=dest+4|0; } while ((dest|0) < (stop|0));
 $4 = HEAP32[$1>>2]|0;
 dest=$x; src=$4; stop=dest+48|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 _computeYMD_HMS($x);
 $5 = ((($x)) + 8|0);
 $6 = HEAP32[$5>>2]|0;
 $7 = ($6|0)<(1971);
 if ($7) {
  label = 3;
 } else {
  $8 = ((($x)) + 8|0);
  $9 = HEAP32[$8>>2]|0;
  $10 = ($9|0)>=(2038);
  if ($10) {
   label = 3;
  } else {
   $17 = ((($x)) + 32|0);
   $18 = +HEAPF64[$17>>3];
   $19 = $18 + 0.5;
   $20 = (~~(($19)));
   HEAP32[$s>>2] = $20;
   $21 = HEAP32[$s>>2]|0;
   $22 = (+($21|0));
   $23 = ((($x)) + 32|0);
   HEAPF64[$23>>3] = $22;
  }
 }
 if ((label|0) == 3) {
  $11 = ((($x)) + 8|0);
  HEAP32[$11>>2] = 2000;
  $12 = ((($x)) + 12|0);
  HEAP32[$12>>2] = 1;
  $13 = ((($x)) + 16|0);
  HEAP32[$13>>2] = 1;
  $14 = ((($x)) + 20|0);
  HEAP32[$14>>2] = 0;
  $15 = ((($x)) + 24|0);
  HEAP32[$15>>2] = 0;
  $16 = ((($x)) + 32|0);
  HEAPF64[$16>>3] = 0.0;
 }
 $24 = ((($x)) + 28|0);
 HEAP32[$24>>2] = 0;
 $25 = ((($x)) + 42|0);
 HEAP8[$25>>0] = 0;
 _computeJD($x);
 $26 = $x;
 $27 = $26;
 $28 = HEAP32[$27>>2]|0;
 $29 = (($26) + 4)|0;
 $30 = $29;
 $31 = HEAP32[$30>>2]|0;
 $32 = (___divdi3(($28|0),($31|0),1000,0)|0);
 $33 = tempRet0;
 $34 = (_i64Subtract(($32|0),($33|0),413362496,49)|0);
 $35 = tempRet0;
 HEAP32[$t>>2] = $34;
 $36 = (_osLocaltime($t,$sLocal)|0);
 $37 = ($36|0)!=(0);
 if ($37) {
  $38 = HEAP32[$2>>2]|0;
  _sqlite3_result_error($38,45422,-1);
  $39 = HEAP32[$3>>2]|0;
  HEAP32[$39>>2] = 1;
  $40 = $0;
  $41 = $40;
  HEAP32[$41>>2] = 0;
  $42 = (($40) + 4)|0;
  $43 = $42;
  HEAP32[$43>>2] = 0;
  $87 = $0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  tempRet0 = ($92);
  STACKTOP = sp;return ($89|0);
 } else {
  $44 = ((($sLocal)) + 20|0);
  $45 = HEAP32[$44>>2]|0;
  $46 = (($45) + 1900)|0;
  $47 = ((($y)) + 8|0);
  HEAP32[$47>>2] = $46;
  $48 = ((($sLocal)) + 16|0);
  $49 = HEAP32[$48>>2]|0;
  $50 = (($49) + 1)|0;
  $51 = ((($y)) + 12|0);
  HEAP32[$51>>2] = $50;
  $52 = ((($sLocal)) + 12|0);
  $53 = HEAP32[$52>>2]|0;
  $54 = ((($y)) + 16|0);
  HEAP32[$54>>2] = $53;
  $55 = ((($sLocal)) + 8|0);
  $56 = HEAP32[$55>>2]|0;
  $57 = ((($y)) + 20|0);
  HEAP32[$57>>2] = $56;
  $58 = ((($sLocal)) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  $60 = ((($y)) + 24|0);
  HEAP32[$60>>2] = $59;
  $61 = HEAP32[$sLocal>>2]|0;
  $62 = (+($61|0));
  $63 = ((($y)) + 32|0);
  HEAPF64[$63>>3] = $62;
  $64 = ((($y)) + 40|0);
  HEAP8[$64>>0] = 1;
  $65 = ((($y)) + 41|0);
  HEAP8[$65>>0] = 1;
  $66 = ((($y)) + 42|0);
  HEAP8[$66>>0] = 0;
  $67 = ((($y)) + 43|0);
  HEAP8[$67>>0] = 0;
  _computeJD($y);
  $68 = HEAP32[$3>>2]|0;
  HEAP32[$68>>2] = 0;
  $69 = $y;
  $70 = $69;
  $71 = HEAP32[$70>>2]|0;
  $72 = (($69) + 4)|0;
  $73 = $72;
  $74 = HEAP32[$73>>2]|0;
  $75 = $x;
  $76 = $75;
  $77 = HEAP32[$76>>2]|0;
  $78 = (($75) + 4)|0;
  $79 = $78;
  $80 = HEAP32[$79>>2]|0;
  $81 = (_i64Subtract(($71|0),($74|0),($77|0),($80|0))|0);
  $82 = tempRet0;
  $83 = $0;
  $84 = $83;
  HEAP32[$84>>2] = $81;
  $85 = (($83) + 4)|0;
  $86 = $85;
  HEAP32[$86>>2] = $82;
  $87 = $0;
  $88 = $87;
  $89 = HEAP32[$88>>2]|0;
  $90 = (($87) + 4)|0;
  $91 = $90;
  $92 = HEAP32[$91>>2]|0;
  tempRet0 = ($92);
  STACKTOP = sp;return ($89|0);
 }
 return (0)|0;
}
function _clearYMD_HMS_TZ($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 40|0);
 HEAP8[$2>>0] = 0;
 $3 = HEAP32[$0>>2]|0;
 $4 = ((($3)) + 41|0);
 HEAP8[$4>>0] = 0;
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 43|0);
 HEAP8[$6>>0] = 0;
 STACKTOP = sp;return;
}
function _parseHhMmSs($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0.0, $43 = 0.0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0, $53 = 0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0.0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $9 = 0, $h = 0, $m = 0, $ms = 0, $rScale = 0, $s = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer2 = sp + 24|0;
 $vararg_buffer = sp + 16|0;
 $0 = sp + 48|0;
 $1 = sp + 44|0;
 $2 = sp + 40|0;
 $h = sp + 36|0;
 $m = sp + 32|0;
 $s = sp + 28|0;
 $ms = sp + 8|0;
 $rScale = sp;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 HEAPF64[$ms>>3] = 0.0;
 $3 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $h;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $m;
 $4 = (_getDigits($3,45402,$vararg_buffer)|0);
 $5 = ($4|0)!=(2);
 if ($5) {
  HEAP32[$0>>2] = 1;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 $6 = HEAP32[$1>>2]|0;
 $7 = ((($6)) + 5|0);
 HEAP32[$1>>2] = $7;
 $8 = HEAP32[$1>>2]|0;
 $9 = HEAP8[$8>>0]|0;
 $10 = $9 << 24 >> 24;
 $11 = ($10|0)==(58);
 if ($11) {
  $12 = HEAP32[$1>>2]|0;
  $13 = ((($12)) + 1|0);
  HEAP32[$1>>2] = $13;
  $14 = HEAP32[$1>>2]|0;
  HEAP32[$vararg_buffer2>>2] = $s;
  $15 = (_getDigits($14,45410,$vararg_buffer2)|0);
  $16 = ($15|0)!=(1);
  if ($16) {
   HEAP32[$0>>2] = 1;
   $85 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($85|0);
  }
  $17 = HEAP32[$1>>2]|0;
  $18 = ((($17)) + 2|0);
  HEAP32[$1>>2] = $18;
  $19 = HEAP32[$1>>2]|0;
  $20 = HEAP8[$19>>0]|0;
  $21 = $20 << 24 >> 24;
  $22 = ($21|0)==(46);
  if ($22) {
   $23 = HEAP32[$1>>2]|0;
   $24 = ((($23)) + 1|0);
   $25 = HEAP8[$24>>0]|0;
   $26 = $25&255;
   $27 = (18403 + ($26)|0);
   $28 = HEAP8[$27>>0]|0;
   $29 = $28&255;
   $30 = $29 & 4;
   $31 = ($30|0)!=(0);
   if ($31) {
    HEAPF64[$rScale>>3] = 1.0;
    $32 = HEAP32[$1>>2]|0;
    $33 = ((($32)) + 1|0);
    HEAP32[$1>>2] = $33;
    while(1) {
     $34 = HEAP32[$1>>2]|0;
     $35 = HEAP8[$34>>0]|0;
     $36 = $35&255;
     $37 = (18403 + ($36)|0);
     $38 = HEAP8[$37>>0]|0;
     $39 = $38&255;
     $40 = $39 & 4;
     $41 = ($40|0)!=(0);
     if (!($41)) {
      break;
     }
     $42 = +HEAPF64[$ms>>3];
     $43 = $42 * 10.0;
     $44 = HEAP32[$1>>2]|0;
     $45 = HEAP8[$44>>0]|0;
     $46 = $45 << 24 >> 24;
     $47 = (+($46|0));
     $48 = $43 + $47;
     $49 = $48 - 48.0;
     HEAPF64[$ms>>3] = $49;
     $50 = +HEAPF64[$rScale>>3];
     $51 = $50 * 10.0;
     HEAPF64[$rScale>>3] = $51;
     $52 = HEAP32[$1>>2]|0;
     $53 = ((($52)) + 1|0);
     HEAP32[$1>>2] = $53;
    }
    $54 = +HEAPF64[$rScale>>3];
    $55 = +HEAPF64[$ms>>3];
    $56 = $55 / $54;
    HEAPF64[$ms>>3] = $56;
   }
  }
 } else {
  HEAP32[$s>>2] = 0;
 }
 $57 = HEAP32[$2>>2]|0;
 $58 = ((($57)) + 42|0);
 HEAP8[$58>>0] = 0;
 $59 = HEAP32[$2>>2]|0;
 $60 = ((($59)) + 41|0);
 HEAP8[$60>>0] = 1;
 $61 = HEAP32[$h>>2]|0;
 $62 = HEAP32[$2>>2]|0;
 $63 = ((($62)) + 20|0);
 HEAP32[$63>>2] = $61;
 $64 = HEAP32[$m>>2]|0;
 $65 = HEAP32[$2>>2]|0;
 $66 = ((($65)) + 24|0);
 HEAP32[$66>>2] = $64;
 $67 = HEAP32[$s>>2]|0;
 $68 = (+($67|0));
 $69 = +HEAPF64[$ms>>3];
 $70 = $68 + $69;
 $71 = HEAP32[$2>>2]|0;
 $72 = ((($71)) + 32|0);
 HEAPF64[$72>>3] = $70;
 $73 = HEAP32[$1>>2]|0;
 $74 = HEAP32[$2>>2]|0;
 $75 = (_parseTimezone($73,$74)|0);
 $76 = ($75|0)!=(0);
 if ($76) {
  HEAP32[$0>>2] = 1;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 } else {
  $77 = HEAP32[$2>>2]|0;
  $78 = ((($77)) + 28|0);
  $79 = HEAP32[$78>>2]|0;
  $80 = ($79|0)!=(0);
  $81 = $80 ? 1 : 0;
  $82 = $81&255;
  $83 = HEAP32[$2>>2]|0;
  $84 = ((($83)) + 43|0);
  HEAP8[$84>>0] = $82;
  HEAP32[$0>>2] = 0;
  $85 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($85|0);
 }
 return (0)|0;
}
function _getDigits($zDate,$zFormat,$varargs) {
 $zDate = $zDate|0;
 $zFormat = $zFormat|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $8 = 0, $9 = 0, $N = 0, $ap = 0, $arglist_current = 0;
 var $arglist_next = 0, $cnt = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $max = 0, $min = 0, $nextC = 0, $val = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $0 = sp + 28|0;
 $1 = sp + 24|0;
 $ap = sp + 8|0;
 $cnt = sp + 4|0;
 $nextC = sp + 36|0;
 $N = sp + 35|0;
 $min = sp + 34|0;
 $val = sp;
 $max = sp + 32|0;
 HEAP32[$0>>2] = $zDate;
 HEAP32[$1>>2] = $zFormat;
 HEAP32[$cnt>>2] = 0;
 HEAP32[$ap>>2] = $varargs;
 L1: while(1) {
  $2 = HEAP32[$1>>2]|0;
  $3 = HEAP8[$2>>0]|0;
  $4 = $3 << 24 >> 24;
  $5 = (($4) - 48)|0;
  $6 = $5&255;
  HEAP8[$N>>0] = $6;
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 1|0);
  $9 = HEAP8[$8>>0]|0;
  $10 = $9 << 24 >> 24;
  $11 = (($10) - 48)|0;
  $12 = $11&255;
  HEAP8[$min>>0] = $12;
  HEAP32[$val>>2] = 0;
  $13 = HEAP32[$1>>2]|0;
  $14 = ((($13)) + 2|0);
  $15 = HEAP8[$14>>0]|0;
  $16 = $15 << 24 >> 24;
  $17 = (($16) - 97)|0;
  $18 = (16422 + ($17<<1)|0);
  $19 = HEAP16[$18>>1]|0;
  HEAP16[$max>>1] = $19;
  $20 = HEAP32[$1>>2]|0;
  $21 = ((($20)) + 3|0);
  $22 = HEAP8[$21>>0]|0;
  HEAP8[$nextC>>0] = $22;
  HEAP32[$val>>2] = 0;
  while(1) {
   $23 = HEAP8[$N>>0]|0;
   $24 = (($23) + -1)<<24>>24;
   HEAP8[$N>>0] = $24;
   $25 = ($23<<24>>24)!=(0);
   if (!($25)) {
    break;
   }
   $26 = HEAP32[$0>>2]|0;
   $27 = HEAP8[$26>>0]|0;
   $28 = $27&255;
   $29 = (18403 + ($28)|0);
   $30 = HEAP8[$29>>0]|0;
   $31 = $30&255;
   $32 = $31 & 4;
   $33 = ($32|0)!=(0);
   if (!($33)) {
    label = 11;
    break L1;
   }
   $34 = HEAP32[$val>>2]|0;
   $35 = ($34*10)|0;
   $36 = HEAP32[$0>>2]|0;
   $37 = HEAP8[$36>>0]|0;
   $38 = $37 << 24 >> 24;
   $39 = (($35) + ($38))|0;
   $40 = (($39) - 48)|0;
   HEAP32[$val>>2] = $40;
   $41 = HEAP32[$0>>2]|0;
   $42 = ((($41)) + 1|0);
   HEAP32[$0>>2] = $42;
  }
  $43 = HEAP32[$val>>2]|0;
  $44 = HEAP8[$min>>0]|0;
  $45 = $44 << 24 >> 24;
  $46 = ($43|0)<($45|0);
  if ($46) {
   label = 11;
   break;
  }
  $47 = HEAP32[$val>>2]|0;
  $48 = HEAP16[$max>>1]|0;
  $49 = $48&65535;
  $50 = ($47|0)>($49|0);
  if ($50) {
   label = 11;
   break;
  }
  $51 = HEAP8[$nextC>>0]|0;
  $52 = $51 << 24 >> 24;
  $53 = ($52|0)!=(0);
  if ($53) {
   $54 = HEAP8[$nextC>>0]|0;
   $55 = $54 << 24 >> 24;
   $56 = HEAP32[$0>>2]|0;
   $57 = HEAP8[$56>>0]|0;
   $58 = $57 << 24 >> 24;
   $59 = ($55|0)!=($58|0);
   if ($59) {
    label = 11;
    break;
   }
  }
  $60 = HEAP32[$val>>2]|0;
  $arglist_current = HEAP32[$ap>>2]|0;
  $61 = $arglist_current;
  $62 = ((0) + 4|0);
  $expanded2 = $62;
  $expanded = (($expanded2) - 1)|0;
  $63 = (($61) + ($expanded))|0;
  $64 = ((0) + 4|0);
  $expanded6 = $64;
  $expanded5 = (($expanded6) - 1)|0;
  $expanded4 = $expanded5 ^ -1;
  $65 = $63 & $expanded4;
  $66 = $65;
  $67 = HEAP32[$66>>2]|0;
  $arglist_next = ((($66)) + 4|0);
  HEAP32[$ap>>2] = $arglist_next;
  HEAP32[$67>>2] = $60;
  $68 = HEAP32[$0>>2]|0;
  $69 = ((($68)) + 1|0);
  HEAP32[$0>>2] = $69;
  $70 = HEAP32[$cnt>>2]|0;
  $71 = (($70) + 1)|0;
  HEAP32[$cnt>>2] = $71;
  $72 = HEAP32[$1>>2]|0;
  $73 = ((($72)) + 4|0);
  HEAP32[$1>>2] = $73;
  $74 = HEAP8[$nextC>>0]|0;
  $75 = ($74<<24>>24)!=(0);
  if (!($75)) {
   label = 11;
   break;
  }
 }
 if ((label|0) == 11) {
  $76 = HEAP32[$cnt>>2]|0;
  STACKTOP = sp;return ($76|0);
 }
 return (0)|0;
}
function _parseTimezone($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $7 = 0, $8 = 0, $9 = 0, $c = 0, $nHr = 0, $nMn = 0, $or$cond = 0, $sgn = 0, $vararg_buffer = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 32|0;
 $1 = sp + 28|0;
 $2 = sp + 24|0;
 $sgn = sp + 20|0;
 $nHr = sp + 16|0;
 $nMn = sp + 12|0;
 $c = sp + 8|0;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 HEAP32[$sgn>>2] = 0;
 while(1) {
  $3 = HEAP32[$1>>2]|0;
  $4 = HEAP8[$3>>0]|0;
  $5 = $4&255;
  $6 = (18403 + ($5)|0);
  $7 = HEAP8[$6>>0]|0;
  $8 = $7&255;
  $9 = $8 & 1;
  $10 = ($9|0)!=(0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$1>>2]|0;
  $12 = ((($11)) + 1|0);
  HEAP32[$1>>2] = $12;
 }
 $13 = HEAP32[$2>>2]|0;
 $14 = ((($13)) + 28|0);
 HEAP32[$14>>2] = 0;
 $15 = HEAP32[$1>>2]|0;
 $16 = HEAP8[$15>>0]|0;
 $17 = $16 << 24 >> 24;
 HEAP32[$c>>2] = $17;
 $18 = HEAP32[$c>>2]|0;
 $19 = ($18|0)==(45);
 do {
  if ($19) {
   HEAP32[$sgn>>2] = -1;
   label = 11;
  } else {
   $20 = HEAP32[$c>>2]|0;
   $21 = ($20|0)==(43);
   if ($21) {
    HEAP32[$sgn>>2] = 1;
    label = 11;
    break;
   }
   $22 = HEAP32[$c>>2]|0;
   $23 = ($22|0)==(90);
   $24 = HEAP32[$c>>2]|0;
   $25 = ($24|0)==(122);
   $or$cond = $23 | $25;
   if ($or$cond) {
    $26 = HEAP32[$1>>2]|0;
    $27 = ((($26)) + 1|0);
    HEAP32[$1>>2] = $27;
    break;
   }
   $28 = HEAP32[$c>>2]|0;
   $29 = ($28|0)!=(0);
   $30 = $29&1;
   HEAP32[$0>>2] = $30;
   $63 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
 } while(0);
 do {
  if ((label|0) == 11) {
   $31 = HEAP32[$1>>2]|0;
   $32 = ((($31)) + 1|0);
   HEAP32[$1>>2] = $32;
   $33 = HEAP32[$1>>2]|0;
   HEAP32[$vararg_buffer>>2] = $nHr;
   $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr2>>2] = $nMn;
   $34 = (_getDigits($33,45414,$vararg_buffer)|0);
   $35 = ($34|0)!=(2);
   if (!($35)) {
    $36 = HEAP32[$1>>2]|0;
    $37 = ((($36)) + 5|0);
    HEAP32[$1>>2] = $37;
    $38 = HEAP32[$sgn>>2]|0;
    $39 = HEAP32[$nMn>>2]|0;
    $40 = HEAP32[$nHr>>2]|0;
    $41 = ($40*60)|0;
    $42 = (($39) + ($41))|0;
    $43 = Math_imul($38, $42)|0;
    $44 = HEAP32[$2>>2]|0;
    $45 = ((($44)) + 28|0);
    HEAP32[$45>>2] = $43;
    break;
   }
   HEAP32[$0>>2] = 1;
   $63 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($63|0);
  }
 } while(0);
 while(1) {
  $46 = HEAP32[$1>>2]|0;
  $47 = HEAP8[$46>>0]|0;
  $48 = $47&255;
  $49 = (18403 + ($48)|0);
  $50 = HEAP8[$49>>0]|0;
  $51 = $50&255;
  $52 = $51 & 1;
  $53 = ($52|0)!=(0);
  if (!($53)) {
   break;
  }
  $54 = HEAP32[$1>>2]|0;
  $55 = ((($54)) + 1|0);
  HEAP32[$1>>2] = $55;
 }
 $56 = HEAP32[$2>>2]|0;
 $57 = ((($56)) + 44|0);
 HEAP8[$57>>0] = 1;
 $58 = HEAP32[$1>>2]|0;
 $59 = HEAP8[$58>>0]|0;
 $60 = $59 << 24 >> 24;
 $61 = ($60|0)!=(0);
 $62 = $61&1;
 HEAP32[$0>>2] = $62;
 $63 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($63|0);
}
function _osLocaltime($t,$pTm) {
 $t = $t|0;
 $pTm = $pTm|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pX = 0, $rc = 0, dest = 0, label = 0, sp = 0;
 var src = 0, stop = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $rc = sp + 4|0;
 $pX = sp;
 HEAP32[$0>>2] = $t;
 HEAP32[$1>>2] = $pTm;
 $2 = HEAP32[$0>>2]|0;
 $3 = (_localtime(($2|0))|0);
 HEAP32[$pX>>2] = $3;
 $4 = HEAP32[(276)>>2]|0;
 $5 = ($4|0)!=(0);
 $$ = $5 ? 0 : $3;
 HEAP32[$pX>>2] = $$;
 $6 = HEAP32[$pX>>2]|0;
 $7 = ($6|0)!=(0|0);
 if ($7) {
  $8 = HEAP32[$1>>2]|0;
  $9 = HEAP32[$pX>>2]|0;
  dest=$8; src=$9; stop=dest+44|0; do { HEAP32[dest>>2]=HEAP32[src>>2]|0; dest=dest+4|0; src=src+4|0; } while ((dest|0) < (stop|0));
 }
 $10 = HEAP32[$pX>>2]|0;
 $11 = ($10|0)==(0|0);
 $12 = $11&1;
 HEAP32[$rc>>2] = $12;
 $13 = HEAP32[$rc>>2]|0;
 STACKTOP = sp;return ($13|0);
}
function _parseYyyyMmDd($zDate,$p) {
 $zDate = $zDate|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $D = 0, $M = 0, $Y = 0, $neg = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer = sp;
 $0 = sp + 36|0;
 $1 = sp + 32|0;
 $2 = sp + 28|0;
 $Y = sp + 24|0;
 $M = sp + 20|0;
 $D = sp + 16|0;
 $neg = sp + 12|0;
 HEAP32[$1>>2] = $zDate;
 HEAP32[$2>>2] = $p;
 $3 = HEAP32[$1>>2]|0;
 $4 = HEAP8[$3>>0]|0;
 $5 = $4 << 24 >> 24;
 $6 = ($5|0)==(45);
 if ($6) {
  $7 = HEAP32[$1>>2]|0;
  $8 = ((($7)) + 1|0);
  HEAP32[$1>>2] = $8;
  HEAP32[$neg>>2] = 1;
 } else {
  HEAP32[$neg>>2] = 0;
 }
 $9 = HEAP32[$1>>2]|0;
 HEAP32[$vararg_buffer>>2] = $Y;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $M;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $D;
 $10 = (_getDigits($9,45449,$vararg_buffer)|0);
 $11 = ($10|0)!=(3);
 if ($11) {
  HEAP32[$0>>2] = 1;
  $59 = HEAP32[$0>>2]|0;
  STACKTOP = sp;return ($59|0);
 }
 $12 = HEAP32[$1>>2]|0;
 $13 = ((($12)) + 10|0);
 HEAP32[$1>>2] = $13;
 while(1) {
  $14 = HEAP32[$1>>2]|0;
  $15 = HEAP8[$14>>0]|0;
  $16 = $15&255;
  $17 = (18403 + ($16)|0);
  $18 = HEAP8[$17>>0]|0;
  $19 = $18&255;
  $20 = $19 & 1;
  $21 = ($20|0)!=(0);
  if ($21) {
   $60 = 1;
  } else {
   $22 = HEAP32[$1>>2]|0;
   $23 = HEAP8[$22>>0]|0;
   $24 = $23&255;
   $25 = (84)==($24|0);
   $60 = $25;
  }
  $26 = HEAP32[$1>>2]|0;
  if (!($60)) {
   break;
  }
  $27 = ((($26)) + 1|0);
  HEAP32[$1>>2] = $27;
 }
 $28 = HEAP32[$2>>2]|0;
 $29 = (_parseHhMmSs($26,$28)|0);
 $30 = ($29|0)==(0);
 do {
  if (!($30)) {
   $31 = HEAP32[$1>>2]|0;
   $32 = HEAP8[$31>>0]|0;
   $33 = $32 << 24 >> 24;
   $34 = ($33|0)==(0);
   if ($34) {
    $35 = HEAP32[$2>>2]|0;
    $36 = ((($35)) + 41|0);
    HEAP8[$36>>0] = 0;
    break;
   }
   HEAP32[$0>>2] = 1;
   $59 = HEAP32[$0>>2]|0;
   STACKTOP = sp;return ($59|0);
  }
 } while(0);
 $37 = HEAP32[$2>>2]|0;
 $38 = ((($37)) + 42|0);
 HEAP8[$38>>0] = 0;
 $39 = HEAP32[$2>>2]|0;
 $40 = ((($39)) + 40|0);
 HEAP8[$40>>0] = 1;
 $41 = HEAP32[$neg>>2]|0;
 $42 = ($41|0)!=(0);
 $43 = HEAP32[$Y>>2]|0;
 $44 = (0 - ($43))|0;
 $45 = $42 ? $44 : $43;
 $46 = HEAP32[$2>>2]|0;
 $47 = ((($46)) + 8|0);
 HEAP32[$47>>2] = $45;
 $48 = HEAP32[$M>>2]|0;
 $49 = HEAP32[$2>>2]|0;
 $50 = ((($49)) + 12|0);
 HEAP32[$50>>2] = $48;
 $51 = HEAP32[$D>>2]|0;
 $52 = HEAP32[$2>>2]|0;
 $53 = ((($52)) + 16|0);
 HEAP32[$53>>2] = $51;
 $54 = HEAP32[$2>>2]|0;
 $55 = ((($54)) + 43|0);
 $56 = HEAP8[$55>>0]|0;
 $57 = ($56<<24>>24)!=(0);
 if ($57) {
  $58 = HEAP32[$2>>2]|0;
  _computeJD($58);
 }
 HEAP32[$0>>2] = 0;
 $59 = HEAP32[$0>>2]|0;
 STACKTOP = sp;return ($59|0);
}
function _sqlite3StmtCurrentTime($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $piTime = 0, $rc = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $rc = sp + 4|0;
 $piTime = sp;
 HEAP32[$0>>2] = $p;
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($1)) + 12|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ((($3)) + 56|0);
 HEAP32[$piTime>>2] = $4;
 $5 = HEAP32[$piTime>>2]|0;
 $6 = $5;
 $7 = $6;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($6) + 4)|0;
 $10 = $9;
 $11 = HEAP32[$10>>2]|0;
 $12 = ($8|0)==(0);
 $13 = ($11|0)==(0);
 $14 = $12 & $13;
 if ($14) {
  $15 = HEAP32[$0>>2]|0;
  $16 = HEAP32[$15>>2]|0;
  $17 = ((($16)) + 32|0);
  $18 = HEAP32[$17>>2]|0;
  $19 = HEAP32[$18>>2]|0;
  $20 = HEAP32[$piTime>>2]|0;
  $21 = (_sqlite3OsCurrentTimeInt64($19,$20)|0);
  HEAP32[$rc>>2] = $21;
  $22 = HEAP32[$rc>>2]|0;
  $23 = ($22|0)!=(0);
  if ($23) {
   $24 = HEAP32[$piTime>>2]|0;
   $25 = $24;
   $26 = $25;
   HEAP32[$26>>2] = 0;
   $27 = (($25) + 4)|0;
   $28 = $27;
   HEAP32[$28>>2] = 0;
  }
 }
 $29 = HEAP32[$piTime>>2]|0;
 $30 = $29;
 $31 = $30;
 $32 = HEAP32[$31>>2]|0;
 $33 = (($30) + 4)|0;
 $34 = $33;
 $35 = HEAP32[$34>>2]|0;
 tempRet0 = ($35);
 STACKTOP = sp;return ($32|0);
}
function _renameTableFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $len = 0, $tname = 0, $token = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $zCsr = 0, $zRet = 0, $zSql = 0, $zTableName = 0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer = sp;
 $0 = sp + 60|0;
 $1 = sp + 56|0;
 $2 = sp + 52|0;
 $zSql = sp + 48|0;
 $zTableName = sp + 44|0;
 $token = sp + 40|0;
 $tname = sp + 32|0;
 $zCsr = sp + 28|0;
 $len = sp + 24|0;
 $zRet = sp + 20|0;
 $db = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zSql>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zTableName>>2] = $9;
 $10 = HEAP32[$zSql>>2]|0;
 HEAP32[$zCsr>>2] = $10;
 HEAP32[$len>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$zSql>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $15 = HEAP32[$zCsr>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   label = 7;
   break;
  }
  $18 = HEAP32[$zCsr>>2]|0;
  HEAP32[$tname>>2] = $18;
  $19 = HEAP32[$len>>2]|0;
  $20 = ((($tname)) + 4|0);
  HEAP32[$20>>2] = $19;
  while(1) {
   $21 = HEAP32[$len>>2]|0;
   $22 = HEAP32[$zCsr>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zCsr>>2] = $23;
   $24 = HEAP32[$zCsr>>2]|0;
   $25 = (_sqlite3GetToken($24,$token)|0);
   HEAP32[$len>>2] = $25;
   $26 = HEAP32[$token>>2]|0;
   $27 = ($26|0)==(162);
   if (!($27)) {
    break;
   }
  }
  $28 = HEAP32[$token>>2]|0;
  $29 = ($28|0)!=(22);
  $30 = HEAP32[$token>>2]|0;
  $31 = ($30|0)!=(125);
  $32 = $29 ? $31 : 0;
  if (!($32)) {
   break;
  }
 }
 if ((label|0) == 7) {
  STACKTOP = sp;return;
 }
 $33 = HEAP32[$db>>2]|0;
 $34 = HEAP32[$tname>>2]|0;
 $35 = HEAP32[$zSql>>2]|0;
 $36 = $34;
 $37 = $35;
 $38 = (($36) - ($37))|0;
 $39 = HEAP32[$zSql>>2]|0;
 $40 = HEAP32[$zTableName>>2]|0;
 $41 = HEAP32[$tname>>2]|0;
 $42 = ((($tname)) + 4|0);
 $43 = HEAP32[$42>>2]|0;
 $44 = (($41) + ($43)|0);
 HEAP32[$vararg_buffer>>2] = $38;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $39;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $40;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $44;
 $45 = (_sqlite3MPrintf($33,45595,$vararg_buffer)|0);
 HEAP32[$zRet>>2] = $45;
 $46 = HEAP32[$0>>2]|0;
 $47 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($46,$47,-1,170);
 STACKTOP = sp;return;
}
function _renameTriggerFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0;
 var $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $db = 0, $dist = 0;
 var $len = 0, $or$cond = 0, $or$cond3 = 0, $or$cond6 = 0, $tname = 0, $token = 0, $vararg_buffer = 0, $vararg_ptr7 = 0, $vararg_ptr8 = 0, $vararg_ptr9 = 0, $zCsr = 0, $zRet = 0, $zSql = 0, $zTableName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $0 = sp + 68|0;
 $1 = sp + 64|0;
 $2 = sp + 60|0;
 $zSql = sp + 56|0;
 $zTableName = sp + 52|0;
 $token = sp + 48|0;
 $tname = sp + 40|0;
 $dist = sp + 32|0;
 $zCsr = sp + 28|0;
 $len = sp + 24|0;
 $zRet = sp + 20|0;
 $db = sp + 16|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$2>>2]|0;
 $4 = HEAP32[$3>>2]|0;
 $5 = (_sqlite3_value_text($4)|0);
 HEAP32[$zSql>>2] = $5;
 $6 = HEAP32[$2>>2]|0;
 $7 = ((($6)) + 4|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = (_sqlite3_value_text($8)|0);
 HEAP32[$zTableName>>2] = $9;
 HEAP32[$dist>>2] = 3;
 $10 = HEAP32[$zSql>>2]|0;
 HEAP32[$zCsr>>2] = $10;
 HEAP32[$len>>2] = 0;
 $11 = HEAP32[$0>>2]|0;
 $12 = (_sqlite3_context_db_handle($11)|0);
 HEAP32[$db>>2] = $12;
 $13 = HEAP32[$zSql>>2]|0;
 $14 = ($13|0)!=(0|0);
 if (!($14)) {
  STACKTOP = sp;return;
 }
 while(1) {
  $15 = HEAP32[$zCsr>>2]|0;
  $16 = HEAP8[$15>>0]|0;
  $17 = ($16<<24>>24)!=(0);
  if (!($17)) {
   label = 8;
   break;
  }
  $18 = HEAP32[$zCsr>>2]|0;
  HEAP32[$tname>>2] = $18;
  $19 = HEAP32[$len>>2]|0;
  $20 = ((($tname)) + 4|0);
  HEAP32[$20>>2] = $19;
  while(1) {
   $21 = HEAP32[$len>>2]|0;
   $22 = HEAP32[$zCsr>>2]|0;
   $23 = (($22) + ($21)|0);
   HEAP32[$zCsr>>2] = $23;
   $24 = HEAP32[$zCsr>>2]|0;
   $25 = (_sqlite3GetToken($24,$token)|0);
   HEAP32[$len>>2] = $25;
   $26 = HEAP32[$token>>2]|0;
   $27 = ($26|0)==(162);
   if (!($27)) {
    break;
   }
  }
  $28 = HEAP32[$dist>>2]|0;
  $29 = (($28) + 1)|0;
  HEAP32[$dist>>2] = $29;
  $30 = HEAP32[$token>>2]|0;
  $31 = ($30|0)==(122);
  $32 = HEAP32[$token>>2]|0;
  $33 = ($32|0)==(107);
  $or$cond = $31 | $33;
  $$ = $or$cond ? 0 : $29;
  HEAP32[$dist>>2] = $$;
  $34 = HEAP32[$dist>>2]|0;
  $35 = ($34|0)!=(2);
  if ($35) {
   continue;
  }
  $36 = HEAP32[$token>>2]|0;
  $37 = ($36|0)!=(137);
  $38 = HEAP32[$token>>2]|0;
  $39 = ($38|0)!=(74);
  $or$cond3 = $37 & $39;
  $40 = HEAP32[$token>>2]|0;
  $41 = ($40|0)!=(5);
  $or$cond6 = $or$cond3 & $41;
  if (!($or$cond6)) {
   break;
  }
 }
 if ((label|0) == 8) {
  STACKTOP = sp;return;
 }
 $42 = HEAP32[$db>>2]|0;
 $43 = HEAP32[$tname>>2]|0;
 $44 = HEAP32[$zSql>>2]|0;
 $45 = $43;
 $46 = $44;
 $47 = (($45) - ($46))|0;
 $48 = HEAP32[$zSql>>2]|0;
 $49 = HEAP32[$zTableName>>2]|0;
 $50 = HEAP32[$tname>>2]|0;
 $51 = ((($tname)) + 4|0);
 $52 = HEAP32[$51>>2]|0;
 $53 = (($50) + ($52)|0);
 HEAP32[$vararg_buffer>>2] = $47;
 $vararg_ptr7 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr7>>2] = $48;
 $vararg_ptr8 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr8>>2] = $49;
 $vararg_ptr9 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr9>>2] = $53;
 $54 = (_sqlite3MPrintf($42,45595,$vararg_buffer)|0);
 HEAP32[$zRet>>2] = $54;
 $55 = HEAP32[$0>>2]|0;
 $56 = HEAP32[$zRet>>2]|0;
 _sqlite3_result_text($55,$56,-1,170);
 STACKTOP = sp;return;
}
function _renameParentFunc($context,$NotUsed,$argv) {
 $context = $context|0;
 $NotUsed = $NotUsed|0;
 $argv = $argv|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0;
 var $81 = 0, $82 = 0, $83 = 0, $84 = 0, $9 = 0, $db = 0, $n = 0, $or$cond = 0, $token = 0, $vararg_buffer = 0, $vararg_buffer5 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr8 = 0, $z = 0, $zInput = 0, $zNew = 0, $zOld = 0, $zOut = 0;
 var $zOutput = 0, $zParent = 0, $zResult = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer5 = sp + 16|0;
 $vararg_buffer = sp;
 $0 = sp + 76|0;
 $1 = sp + 72|0;
 $2 = sp + 68|0;
 $db = sp + 64|0;
 $zOutput = sp + 60|0;
 $zResult = sp + 56|0;
 $zInput = sp + 52|0;
 $zOld = sp + 48|0;
 $zNew = sp + 44|0;
 $z = sp + 40|0;
 $n = sp + 36|0;
 $token = sp + 32|0;
 $zParent = sp + 28|0;
 $zOut = sp + 24|0;
 HEAP32[$0>>2] = $context;
 HEAP32[$1>>2] = $NotUsed;
 HEAP32[$2>>2] = $argv;
 $3 = HEAP32[$0>>2]|0;
 $4 = (_sqlite3_context_db_handle($3)|0);
 HEAP32[$db>>2] = $4;
 HEAP32[$zOutput>>2] = 0;
 $5 = HEAP32[$2>>2]|0;
 $6 = HEAP32[$5>>2]|0;
 $7 = (_sqlite3_value_text($6)|0);
 HEAP32[$zInput>>2] = $7;
 $8 = HEAP32[$2>>2]|0;
 $9 = ((($8)) + 4|0);
 $10 = HEAP32[$9>>2]|0;
 $11 = (_sqlite3_value_text($10)|0);
 HEAP32[$zOld>>2] = $11;
 $12 = HEAP32[$2>>2]|0;
 $13 = ((($12)) + 8|0);
 $14 = HEAP32[$13>>2]|0;
 $15 = (_sqlite3_value_text($14)|0);
 HEAP32[$zNew>>2] = $15;
 $16 = HEAP32[$zInput>>2]|0;
 $17 = ($16|0)==(0|0);
 $18 = HEAP32[$zOld>>2]|0;
 $19 = ($18|0)==(0|0);
 $or$cond = $17 | $19;
 if ($or$cond) {
  STACKTOP = sp;return;
 }
 $20 = HEAP32[$zInput>>2]|0;
 HEAP32[$z>>2] = $20;
 while(1) {
  $21 = HEAP32[$z>>2]|0;
  $22 = HEAP8[$21>>0]|0;
  $23 = ($22<<24>>24)!=(0);
  if (!($23)) {
   break;
  }
  $24 = HEAP32[$z>>2]|0;
  $25 = (_sqlite3GetToken($24,$token)|0);
  HEAP32[$n>>2] = $25;
  $26 = HEAP32[$token>>2]|0;
  $27 = ($26|0)==(105);
  if ($27) {
   while(1) {
    $28 = HEAP32[$n>>2]|0;
    $29 = HEAP32[$z>>2]|0;
    $30 = (($29) + ($28)|0);
    HEAP32[$z>>2] = $30;
    $31 = HEAP32[$z>>2]|0;
    $32 = (_sqlite3GetToken($31,$token)|0);
    HEAP32[$n>>2] = $32;
    $33 = HEAP32[$token>>2]|0;
    $34 = ($33|0)==(162);
    if (!($34)) {
     break;
    }
   }
   $35 = HEAP32[$token>>2]|0;
   $36 = ($35|0)==(163);
   if ($36) {
    break;
   }
   $37 = HEAP32[$db>>2]|0;
   $38 = HEAP32[$z>>2]|0;
   $39 = HEAP32[$n>>2]|0;
   $40 = ($39|0)<(0);
   $41 = $40 << 31 >> 31;
   $42 = (_sqlite3DbStrNDup($37,$38,$39,$41)|0);
   HEAP32[$zParent>>2] = $42;
   $43 = HEAP32[$zParent>>2]|0;
   $44 = ($43|0)==(0|0);
   if ($44) {
    break;
   }
   $45 = HEAP32[$zParent>>2]|0;
   _sqlite3Dequote($45);
   $46 = HEAP32[$zOld>>2]|0;
   $47 = HEAP32[$zParent>>2]|0;
   $48 = (_sqlite3StrICmp($46,$47)|0);
   $49 = (0)==($48|0);
   if ($49) {
    $50 = HEAP32[$db>>2]|0;
    $51 = HEAP32[$zOutput>>2]|0;
    $52 = ($51|0)!=(0|0);
    $53 = HEAP32[$zOutput>>2]|0;
    $54 = $52 ? $53 : 18402;
    $55 = HEAP32[$z>>2]|0;
    $56 = HEAP32[$zInput>>2]|0;
    $57 = $55;
    $58 = $56;
    $59 = (($57) - ($58))|0;
    $60 = HEAP32[$zInput>>2]|0;
    $61 = HEAP32[$zNew>>2]|0;
    HEAP32[$vararg_buffer>>2] = $54;
    $vararg_ptr2 = ((($vararg_buffer)) + 4|0);
    HEAP32[$vararg_ptr2>>2] = $59;
    $vararg_ptr3 = ((($vararg_buffer)) + 8|0);
    HEAP32[$vararg_ptr3>>2] = $60;
    $vararg_ptr4 = ((($vararg_buffer)) + 12|0);
    HEAP32[$vararg_ptr4>>2] = $61;
    $62 = (_sqlite3MPrintf($50,45584,$vararg_buffer)|0);
    HEAP32[$zOut>>2] = $62;
    $63 = HEAP32[$db>>2]|0;
    $64 = HEAP32[$zOutput>>2]|0;
    _sqlite3DbFree($63,$64);
    $65 = HEAP32[$zOut>>2]|0;
    HEAP32[$zOutput>>2] = $65;
    $66 = HEAP32[$n>>2]|0;
    $67 = HEAP32[$z>>2]|0;
    $68 = (($67) + ($66)|0);
    HEAP32[$zInput>>2] = $68;
   }
   $69 = HEAP32[$db>>2]|0;
   $70 = HEAP32[$zParent>>2]|0;
   _sqlite3DbFree($69,$70);
  }
  $71 = HEAP32[$z>>2]|0;
  $72 = HEAP32[$n>>2]|0;
  $73 = (($71) + ($72)|0);
  HEAP32[$z>>2] = $73;
 }
 $74 = HEAP32[$db>>2]|0;
 $75 = HEAP32[$zOutput>>2]|0;
 $76 = ($75|0)!=(0|0);
 $77 = HEAP32[$zOutput>>2]|0;
 $78 = $76 ? $77 : 18402;
 $79 = HEAP32[$zInput>>2]|0;
 HEAP32[$vararg_buffer5>>2] = $78;
 $vararg_ptr8 = ((($vararg_buffer5)) + 4|0);
 HEAP32[$vararg_ptr8>>2] = $79;
 $80 = (_sqlite3MPrintf($74,24705,$vararg_buffer5)|0);
 HEAP32[$zResult>>2] = $80;
 $81 = HEAP32[$0>>2]|0;
 $82 = HEAP32[$zResult>>2]|0;
 _sqlite3_result_text($81,$82,-1,170);
 $83 = HEAP32[$db>>2]|0;
 $84 = HEAP32[$zOutput>>2]|0;
 _sqlite3DbFree($83,$84);
 STACKTOP = sp;return;
}
function _measureAllocationSize($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 4|0;
 $1 = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = HEAP32[$1>>2]|0;
 $4 = (_sqlite3DbMallocSize($2,$3)|0);
 $5 = HEAP32[$0>>2]|0;
 $6 = ((($5)) + 456|0);
 $7 = HEAP32[$6>>2]|0;
 $8 = HEAP32[$7>>2]|0;
 $9 = (($8) + ($4))|0;
 HEAP32[$7>>2] = $9;
 STACKTOP = sp;return;
}
function _vdbeFreeOpArray($db,$aOp,$nOp) {
 $db = $db|0;
 $aOp = $aOp|0;
 $nOp = $nOp|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pOp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 12|0;
 $1 = sp + 8|0;
 $2 = sp + 4|0;
 $pOp = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $aOp;
 HEAP32[$2>>2] = $nOp;
 $3 = HEAP32[$1>>2]|0;
 $4 = ($3|0)!=(0|0);
 if (!($4)) {
  $25 = HEAP32[$0>>2]|0;
  $26 = HEAP32[$1>>2]|0;
  _sqlite3DbFree($25,$26);
  STACKTOP = sp;return;
 }
 $5 = HEAP32[$1>>2]|0;
 HEAP32[$pOp>>2] = $5;
 while(1) {
  $6 = HEAP32[$pOp>>2]|0;
  $7 = HEAP32[$2>>2]|0;
  $8 = HEAP32[$1>>2]|0;
  $9 = (($8) + (($7*20)|0)|0);
  $10 = ($6>>>0)<($9>>>0);
  if (!($10)) {
   break;
  }
  $11 = HEAP32[$pOp>>2]|0;
  $12 = ((($11)) + 1|0);
  $13 = HEAP8[$12>>0]|0;
  $14 = ($13<<24>>24)!=(0);
  if ($14) {
   $15 = HEAP32[$0>>2]|0;
   $16 = HEAP32[$pOp>>2]|0;
   $17 = ((($16)) + 1|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18 << 24 >> 24;
   $20 = HEAP32[$pOp>>2]|0;
   $21 = ((($20)) + 16|0);
   $22 = HEAP32[$21>>2]|0;
   _freeP4($15,$19,$22);
  }
  $23 = HEAP32[$pOp>>2]|0;
  $24 = ((($23)) + 20|0);
  HEAP32[$pOp>>2] = $24;
 }
 $25 = HEAP32[$0>>2]|0;
 $26 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($25,$26);
 STACKTOP = sp;return;
}
function _deleteTable($db,$pTable) {
 $db = $db|0;
 $pTable = $pTable|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $pIndex = 0, $pNext = 0, $zName = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 16|0;
 $1 = sp + 12|0;
 $pIndex = sp + 8|0;
 $pNext = sp + 4|0;
 $zName = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTable;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 8|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pIndex>>2] = $4;
 while(1) {
  $5 = HEAP32[$pIndex>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$pIndex>>2]|0;
  $8 = ((($7)) + 20|0);
  $9 = HEAP32[$8>>2]|0;
  HEAP32[$pNext>>2] = $9;
  $10 = HEAP32[$0>>2]|0;
  $11 = ($10|0)==(0|0);
  if ($11) {
   label = 5;
  } else {
   $12 = HEAP32[$0>>2]|0;
   $13 = ((($12)) + 456|0);
   $14 = HEAP32[$13>>2]|0;
   $15 = ($14|0)==(0|0);
   if ($15) {
    label = 5;
   }
  }
  if ((label|0) == 5) {
   label = 0;
   $16 = HEAP32[$1>>2]|0;
   $17 = ((($16)) + 42|0);
   $18 = HEAP8[$17>>0]|0;
   $19 = $18&255;
   $20 = $19 & 16;
   $21 = ($20|0)!=(0);
   if (!($21)) {
    $22 = HEAP32[$pIndex>>2]|0;
    $23 = HEAP32[$22>>2]|0;
    HEAP32[$zName>>2] = $23;
    $24 = HEAP32[$pIndex>>2]|0;
    $25 = ((($24)) + 24|0);
    $26 = HEAP32[$25>>2]|0;
    $27 = ((($26)) + 24|0);
    $28 = HEAP32[$zName>>2]|0;
    (_sqlite3HashInsert($27,$28,0)|0);
   }
  }
  $29 = HEAP32[$0>>2]|0;
  $30 = HEAP32[$pIndex>>2]|0;
  _freeIndex($29,$30);
  $31 = HEAP32[$pNext>>2]|0;
  HEAP32[$pIndex>>2] = $31;
 }
 $32 = HEAP32[$0>>2]|0;
 $33 = HEAP32[$1>>2]|0;
 _sqlite3FkDelete($32,$33);
 $34 = HEAP32[$0>>2]|0;
 $35 = HEAP32[$1>>2]|0;
 _sqlite3DeleteColumnNames($34,$35);
 $36 = HEAP32[$0>>2]|0;
 $37 = HEAP32[$1>>2]|0;
 $38 = HEAP32[$37>>2]|0;
 _sqlite3DbFree($36,$38);
 $39 = HEAP32[$0>>2]|0;
 $40 = HEAP32[$1>>2]|0;
 $41 = ((($40)) + 20|0);
 $42 = HEAP32[$41>>2]|0;
 _sqlite3DbFree($39,$42);
 $43 = HEAP32[$0>>2]|0;
 $44 = HEAP32[$1>>2]|0;
 $45 = ((($44)) + 12|0);
 $46 = HEAP32[$45>>2]|0;
 _sqlite3SelectDelete($43,$46);
 $47 = HEAP32[$0>>2]|0;
 $48 = HEAP32[$1>>2]|0;
 $49 = ((($48)) + 24|0);
 $50 = HEAP32[$49>>2]|0;
 _sqlite3ExprListDelete($47,$50);
 $51 = HEAP32[$0>>2]|0;
 $52 = HEAP32[$1>>2]|0;
 _sqlite3VtabClear($51,$52);
 $53 = HEAP32[$0>>2]|0;
 $54 = HEAP32[$1>>2]|0;
 _sqlite3DbFree($53,$54);
 STACKTOP = sp;return;
}
function _sqlite3FkDelete($db,$pTab) {
 $db = $db|0;
 $pTab = $pTab|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0;
 var $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0;
 var $63 = 0, $64 = 0, $65 = 0, $7 = 0, $8 = 0, $9 = 0, $p = 0, $pFKey = 0, $pNext = 0, $z = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $0 = sp + 20|0;
 $1 = sp + 16|0;
 $pFKey = sp + 12|0;
 $pNext = sp + 8|0;
 $p = sp + 4|0;
 $z = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $pTab;
 $2 = HEAP32[$1>>2]|0;
 $3 = ((($2)) + 16|0);
 $4 = HEAP32[$3>>2]|0;
 HEAP32[$pFKey>>2] = $4;
 while(1) {
  $5 = HEAP32[$pFKey>>2]|0;
  $6 = ($5|0)!=(0|0);
  if (!($6)) {
   break;
  }
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)!=(0|0);
  if ($8) {
   $9 = HEAP32[$0>>2]|0;
   $10 = ((($9)) + 456|0);
   $11 = HEAP32[$10>>2]|0;
   $12 = ($11|0)==(0|0);
   if ($12) {
    label = 5;
   }
  } else {
   label = 5;
  }
  if ((label|0) == 5) {
   label = 0;
   $13 = HEAP32[$pFKey>>2]|0;
   $14 = ((($13)) + 16|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = ($15|0)!=(0|0);
   $17 = HEAP32[$pFKey>>2]|0;
   $18 = ((($17)) + 12|0);
   $19 = HEAP32[$18>>2]|0;
   if ($16) {
    $20 = HEAP32[$pFKey>>2]|0;
    $21 = ((($20)) + 16|0);
    $22 = HEAP32[$21>>2]|0;
    $23 = ((($22)) + 12|0);
    HEAP32[$23>>2] = $19;
   } else {
    HEAP32[$p>>2] = $19;
    $24 = HEAP32[$p>>2]|0;
    $25 = ($24|0)!=(0|0);
    $26 = HEAP32[$pFKey>>2]|0;
    if ($25) {
     $27 = ((($26)) + 12|0);
     $28 = HEAP32[$27>>2]|0;
     $29 = ((($28)) + 8|0);
     $30 = HEAP32[$29>>2]|0;
     $33 = $30;
    } else {
     $31 = ((($26)) + 8|0);
     $32 = HEAP32[$31>>2]|0;
     $33 = $32;
    }
    HEAP32[$z>>2] = $33;
    $34 = HEAP32[$1>>2]|0;
    $35 = ((($34)) + 64|0);
    $36 = HEAP32[$35>>2]|0;
    $37 = ((($36)) + 56|0);
    $38 = HEAP32[$z>>2]|0;
    $39 = HEAP32[$p>>2]|0;
    (_sqlite3HashInsert($37,$38,$39)|0);
   }
   $40 = HEAP32[$pFKey>>2]|0;
   $41 = ((($40)) + 12|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)!=(0|0);
   if ($43) {
    $44 = HEAP32[$pFKey>>2]|0;
    $45 = ((($44)) + 16|0);
    $46 = HEAP32[$45>>2]|0;
    $47 = HEAP32[$pFKey>>2]|0;
    $48 = ((($47)) + 12|0);
    $49 = HEAP32[$48>>2]|0;
    $50 = ((($49)) + 16|0);
    HEAP32[$50>>2] = $46;
   }
  }
  $51 = HEAP32[$0>>2]|0;
  $52 = HEAP32[$pFKey>>2]|0;
  $53 = ((($52)) + 28|0);
  $54 = HEAP32[$53>>2]|0;
  _fkTriggerDelete($51,$54);
  $55 = HEAP32[$0>>2]|0;
  $56 = HEAP32[$pFKey>>2]|0;
  $57 = ((($56)) + 28|0);
  $58 = ((($57)) + 4|0);
  $59 = HEAP32[$58>>2]|0;
  _fkTriggerDelete($55,$59);
  $60 = HEAP32[$pFKey>>2]|0;
  $61 = ((($60)) + 4|0);
  $62 = HEAP32[$61>>2]|0;
  HEAP32[$pNext>>2] = $62;
  $63 = HEAP32[$0>>2]|0;
  $64 = HEAP32[$pFKey>>2]|0;
  _sqlite3DbFree($63,$64);
  $65 = HEAP32[$pNext>>2]|0;
  HEAP32[$pFKey>>2] = $65;
 }
 STACKTOP = sp;return;
}
function _sqlite3VtabClear($db,$p) {
 $db = $db|0;
 $p = $p|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $0 = sp + 8|0;
 $1 = sp + 4|0;
 $i = sp;
 HEAP32[$0>>2] = $db;
 HEAP32[$1>>2] = $p;
 $2 = HEAP32[$0>>2]|0;
 $3 = ($2|0)!=(0|0);
 if ($3) {
  $4 = HEAP32[$0>>2]|0;
  $5 = ((($4)) + 456|0);
  $6 = HEAP32[$5>>2]|0;
  $7 = ($6|0)==(0|0);
  if ($7) {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $8 = HEAP32[$1>>2]|0;
  (_vtabDisconnectAll(0,$8)|0);
 }
 $9 = HEAP32[$1>>2]|0;
 $10 = ((($9)) + 52|0);
 $11 = HEAP32[$10>>2]|0;
 $12 = ($11|0)!=(0|0);
 if (!($12)) {
  STACKTOP = sp;return;
 }
 HEAP32[$i>>2] = 0;
 while(1) {
  $13 = HEAP32[$i>>2]|0;
  $14 = HEAP32[$1>>2]|0;
  $15 = ((($14)) + 48|0);
  $16 = HEAP32[$15>>2]|0;
  $17 = ($13|0)<($16|0);
  if (!($17)) {
   break;
  }
  $18 = HEAP32[$i>>2]|0;
  $19 = ($18|0)!=(1);
  if ($19) {
   $20 = HEAP32[$0>>2]|0;
   $21 = HEAP32[$i>>2]|0;
   $22 = HEAP32[$1>>2]|0;
   $23 = ((($22)) + 52|0);
   $24 = HEAP32[$23>>2]|0;
   $25 = (($24) + ($21<<2)|0);
   $26 = HEAP32[$25>>2]|0;
   _sqlite3DbFree($20,$26);
  }
  $27 = HEAP32[$i>>2]|0;
  $28 = (($27) + 1)|0;
  HEAP32[$i>>2] = $28;
 }
 $29 = HEAP32[$0>>2]|0;
 $30 = HEAP32[$1>>2]|0;
 $31 = ((($30)) + 52|0);
 $32 = HEAP32[$31>>2]|0;
 _sqlite3DbFree($29,$32);
 STACKTOP = sp;return;
}
function _isalpha($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $c | 32;
 $1 = (($0) + -97)|0;
 $2 = ($1>>>0)<(26);
 $3 = $2&1;
 return ($3|0);
}
function _isblank($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($c|0)==(32);
 $1 = ($c|0)==(9);
 $2 = $0 | $1;
 $3 = $2&1;
 return ($3|0);
}
function _islower($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($c) + -97)|0;
 $1 = ($0>>>0)<(26);
 $2 = $1&1;
 return ($2|0);
}
function _isupper($c) {
 $c = $c|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($c) + -65)|0;
 $1 = ($0>>>0)<(26);
 $2 = $1&1;
 return ($2|0);
}
function _tolower($c) {
 $c = $c|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_isupper($c)|0);
 $1 = ($0|0)==(0);
 $2 = $c | 32;
 $$0 = $1 ? $c : $2;
 return ($$0|0);
}
function _toupper($c) {
 $c = $c|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_islower($c)|0);
 $1 = ($0|0)==(0);
 $2 = $c & 95;
 $$0 = $1 ? $c : $2;
 return ($$0|0);
}
function ___errno_location() {
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = HEAP32[8864>>2]|0;
 $1 = ($0|0)==(0|0);
 if ($1) {
  $$0 = 8912;
 } else {
  $2 = (_pthread_self()|0);
  $3 = ((($2)) + 60|0);
  $4 = HEAP32[$3>>2]|0;
  $$0 = $4;
 }
 return ($$0|0);
}
function _strerror($e) {
 $e = $e|0;
 var $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $i$03 = 0, $i$03$lcssa = 0, $i$12 = 0, $s$0$lcssa = 0, $s$01 = 0, $s$1 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $i$03 = 0;
 while(1) {
  $1 = (45740 + ($i$03)|0);
  $2 = HEAP8[$1>>0]|0;
  $3 = $2&255;
  $4 = ($3|0)==($e|0);
  if ($4) {
   $i$03$lcssa = $i$03;
   label = 2;
   break;
  }
  $5 = (($i$03) + 1)|0;
  $6 = ($5|0)==(87);
  if ($6) {
   $i$12 = 87;$s$01 = 45828;
   label = 5;
   break;
  } else {
   $i$03 = $5;
  }
 }
 if ((label|0) == 2) {
  $0 = ($i$03$lcssa|0)==(0);
  if ($0) {
   $s$0$lcssa = 45828;
  } else {
   $i$12 = $i$03$lcssa;$s$01 = 45828;
   label = 5;
  }
 }
 if ((label|0) == 5) {
  while(1) {
   label = 0;
   $s$1 = $s$01;
   while(1) {
    $7 = HEAP8[$s$1>>0]|0;
    $8 = ($7<<24>>24)==(0);
    $9 = ((($s$1)) + 1|0);
    if ($8) {
     $$lcssa = $9;
     break;
    } else {
     $s$1 = $9;
    }
   }
   $10 = (($i$12) + -1)|0;
   $11 = ($10|0)==(0);
   if ($11) {
    $s$0$lcssa = $$lcssa;
    break;
   } else {
    $i$12 = $10;$s$01 = $$lcssa;
    label = 5;
   }
  }
 }
 return ($s$0$lcssa|0);
}
function _fcntl($fd,$cmd,$varargs) {
 $fd = $fd|0;
 $cmd = $cmd|0;
 $varargs = $varargs|0;
 var $$ = 0, $$0 = 0, $$off = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, $ap = 0, $arglist_current = 0, $arglist_next = 0, $ex = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $switch = 0, $vararg_buffer = 0, $vararg_buffer13 = 0, $vararg_buffer18 = 0, $vararg_buffer23 = 0, $vararg_buffer28 = 0, $vararg_buffer3 = 0, $vararg_buffer31 = 0, $vararg_buffer36 = 0, $vararg_buffer41 = 0;
 var $vararg_buffer46 = 0, $vararg_buffer8 = 0, $vararg_ptr1 = 0, $vararg_ptr11 = 0, $vararg_ptr12 = 0, $vararg_ptr16 = 0, $vararg_ptr17 = 0, $vararg_ptr2 = 0, $vararg_ptr21 = 0, $vararg_ptr22 = 0, $vararg_ptr26 = 0, $vararg_ptr27 = 0, $vararg_ptr34 = 0, $vararg_ptr35 = 0, $vararg_ptr39 = 0, $vararg_ptr40 = 0, $vararg_ptr44 = 0, $vararg_ptr45 = 0, $vararg_ptr49 = 0, $vararg_ptr50 = 0;
 var $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 192|0;
 $vararg_buffer46 = sp + 152|0;
 $vararg_buffer41 = sp + 136|0;
 $vararg_buffer36 = sp + 120|0;
 $vararg_buffer31 = sp + 104|0;
 $vararg_buffer28 = sp + 96|0;
 $vararg_buffer23 = sp + 80|0;
 $vararg_buffer18 = sp + 64|0;
 $vararg_buffer13 = sp + 48|0;
 $vararg_buffer8 = sp + 32|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $ap = sp + 176|0;
 $ex = sp + 168|0;
 HEAP32[$ap>>2] = $varargs;
 $arglist_current = HEAP32[$ap>>2]|0;
 $0 = $arglist_current;
 $1 = ((0) + 4|0);
 $expanded2 = $1;
 $expanded = (($expanded2) - 1)|0;
 $2 = (($0) + ($expanded))|0;
 $3 = ((0) + 4|0);
 $expanded6 = $3;
 $expanded5 = (($expanded6) - 1)|0;
 $expanded4 = $expanded5 ^ -1;
 $4 = $2 & $expanded4;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $arglist_next = ((($5)) + 4|0);
 HEAP32[$ap>>2] = $arglist_next;
 $7 = ($cmd|0)==(4);
 $8 = $6 | 32768;
 $$ = $7 ? $8 : $6;
 L1: do {
  switch ($cmd|0) {
  case 14:  {
   $9 = $$;
   HEAP32[$vararg_buffer>>2] = $fd;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = 14;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $9;
   $10 = (___syscall221(221,($vararg_buffer|0))|0);
   $11 = (___syscall_ret($10)|0);
   $$0 = $11;
   break;
  }
  case 9:  {
   HEAP32[$vararg_buffer3>>2] = $fd;
   $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
   HEAP32[$vararg_ptr6>>2] = 16;
   $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
   HEAP32[$vararg_ptr7>>2] = $ex;
   $12 = (___syscall221(221,($vararg_buffer3|0))|0);
   switch ($12|0) {
   case -22:  {
    $13 = $$;
    HEAP32[$vararg_buffer8>>2] = $fd;
    $vararg_ptr11 = ((($vararg_buffer8)) + 4|0);
    HEAP32[$vararg_ptr11>>2] = 9;
    $vararg_ptr12 = ((($vararg_buffer8)) + 8|0);
    HEAP32[$vararg_ptr12>>2] = $13;
    $14 = (___syscall221(221,($vararg_buffer8|0))|0);
    $$0 = $14;
    break L1;
    break;
   }
   case 0:  {
    $16 = HEAP32[$ex>>2]|0;
    $17 = ($16|0)==(2);
    $18 = ((($ex)) + 4|0);
    $19 = HEAP32[$18>>2]|0;
    $20 = (0 - ($19))|0;
    $21 = $17 ? $20 : $19;
    $$0 = $21;
    break L1;
    break;
   }
   default: {
    $15 = (___syscall_ret($12)|0);
    $$0 = $15;
    break L1;
   }
   }
   break;
  }
  case 1030:  {
   HEAP32[$vararg_buffer13>>2] = $fd;
   $vararg_ptr16 = ((($vararg_buffer13)) + 4|0);
   HEAP32[$vararg_ptr16>>2] = 1030;
   $vararg_ptr17 = ((($vararg_buffer13)) + 8|0);
   HEAP32[$vararg_ptr17>>2] = $$;
   $22 = (___syscall221(221,($vararg_buffer13|0))|0);
   $23 = ($22|0)==(-22);
   if (!($23)) {
    $24 = ($22|0)>(-1);
    if ($24) {
     HEAP32[$vararg_buffer18>>2] = $22;
     $vararg_ptr21 = ((($vararg_buffer18)) + 4|0);
     HEAP32[$vararg_ptr21>>2] = 2;
     $vararg_ptr22 = ((($vararg_buffer18)) + 8|0);
     HEAP32[$vararg_ptr22>>2] = 1;
     (___syscall221(221,($vararg_buffer18|0))|0);
    }
    $25 = (___syscall_ret($22)|0);
    $$0 = $25;
    break L1;
   }
   HEAP32[$vararg_buffer23>>2] = $fd;
   $vararg_ptr26 = ((($vararg_buffer23)) + 4|0);
   HEAP32[$vararg_ptr26>>2] = 1030;
   $vararg_ptr27 = ((($vararg_buffer23)) + 8|0);
   HEAP32[$vararg_ptr27>>2] = 0;
   $26 = (___syscall221(221,($vararg_buffer23|0))|0);
   $27 = ($26|0)==(-22);
   if ($27) {
    HEAP32[$vararg_buffer31>>2] = $fd;
    $vararg_ptr34 = ((($vararg_buffer31)) + 4|0);
    HEAP32[$vararg_ptr34>>2] = 0;
    $vararg_ptr35 = ((($vararg_buffer31)) + 8|0);
    HEAP32[$vararg_ptr35>>2] = $$;
    $30 = (___syscall221(221,($vararg_buffer31|0))|0);
    $31 = ($30|0)>(-1);
    if ($31) {
     HEAP32[$vararg_buffer36>>2] = $30;
     $vararg_ptr39 = ((($vararg_buffer36)) + 4|0);
     HEAP32[$vararg_ptr39>>2] = 2;
     $vararg_ptr40 = ((($vararg_buffer36)) + 8|0);
     HEAP32[$vararg_ptr40>>2] = 1;
     (___syscall221(221,($vararg_buffer36|0))|0);
    }
    $32 = (___syscall_ret($30)|0);
    $$0 = $32;
    break L1;
   } else {
    $28 = ($26|0)>(-1);
    if ($28) {
     HEAP32[$vararg_buffer28>>2] = $26;
     (___syscall6(6,($vararg_buffer28|0))|0);
    }
    $29 = (___syscall_ret(-22)|0);
    $$0 = $29;
    break L1;
   }
   break;
  }
  default: {
   $$off = (($cmd) + -12)|0;
   $switch = ($$off>>>0)<(5);
   if ($switch) {
    $33 = $$;
    HEAP32[$vararg_buffer41>>2] = $fd;
    $vararg_ptr44 = ((($vararg_buffer41)) + 4|0);
    HEAP32[$vararg_ptr44>>2] = $cmd;
    $vararg_ptr45 = ((($vararg_buffer41)) + 8|0);
    HEAP32[$vararg_ptr45>>2] = $33;
    $34 = (___syscall221(221,($vararg_buffer41|0))|0);
    $35 = (___syscall_ret($34)|0);
    $$0 = $35;
    break L1;
   } else {
    HEAP32[$vararg_buffer46>>2] = $fd;
    $vararg_ptr49 = ((($vararg_buffer46)) + 4|0);
    HEAP32[$vararg_ptr49>>2] = $cmd;
    $vararg_ptr50 = ((($vararg_buffer46)) + 8|0);
    HEAP32[$vararg_ptr50>>2] = $$;
    $36 = (___syscall221(221,($vararg_buffer46|0))|0);
    $37 = (___syscall_ret($36)|0);
    $$0 = $37;
    break L1;
   }
  }
  }
 } while(0);
 STACKTOP = sp;return ($$0|0);
}
function _open($filename,$flags,$varargs) {
 $filename = $filename|0;
 $flags = $flags|0;
 $varargs = $varargs|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ap = 0, $arglist_current = 0, $arglist_next = 0, $expanded = 0, $expanded2 = 0, $expanded4 = 0, $expanded5 = 0, $expanded6 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0;
 var $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $ap = sp + 16|0;
 HEAP32[$ap>>2] = $varargs;
 $arglist_current = HEAP32[$ap>>2]|0;
 $0 = $arglist_current;
 $1 = ((0) + 4|0);
 $expanded2 = $1;
 $expanded = (($expanded2) - 1)|0;
 $2 = (($0) + ($expanded))|0;
 $3 = ((0) + 4|0);
 $expanded6 = $3;
 $expanded5 = (($expanded6) - 1)|0;
 $expanded4 = $expanded5 ^ -1;
 $4 = $2 & $expanded4;
 $5 = $4;
 $6 = HEAP32[$5>>2]|0;
 $arglist_next = ((($5)) + 4|0);
 HEAP32[$ap>>2] = $arglist_next;
 $7 = $flags | 32768;
 HEAP32[$vararg_buffer>>2] = $filename;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $7;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $6;
 $8 = (___syscall5(5,($vararg_buffer|0))|0);
 $9 = (___syscall_ret($8)|0);
 STACKTOP = sp;return ($9|0);
}
function ___procfdname($buf,$fd) {
 $buf = $buf|0;
 $fd = $fd|0;
 var $$01 = 0, $$lcssa = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $i$0 = 0, $i$13 = 0, $i$22 = 0, $j$04 = 0, $scevgep = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $i$0 = 0;
 while(1) {
  $0 = (47632 + ($i$0)|0);
  $1 = HEAP8[$0>>0]|0;
  $2 = (($buf) + ($i$0)|0);
  HEAP8[$2>>0] = $1;
  $3 = ($i$0|0)==(14);
  $4 = (($i$0) + 1)|0;
  if ($3) {
   break;
  } else {
   $i$0 = $4;
  }
 }
 $5 = ($fd|0)==(0);
 if ($5) {
  $scevgep = ((($buf)) + 14|0);
  HEAP8[$scevgep>>0] = 48;
  $6 = ((($buf)) + 15|0);
  HEAP8[$6>>0] = 0;
 } else {
  $i$13 = 14;$j$04 = $fd;
  while(1) {
   $7 = (($j$04>>>0) / 10)&-1;
   $8 = (($i$13) + 1)|0;
   $9 = ($j$04>>>0)<(10);
   if ($9) {
    $$lcssa = $8;
    break;
   } else {
    $i$13 = $8;$j$04 = $7;
   }
  }
  $10 = (($buf) + ($$lcssa)|0);
  HEAP8[$10>>0] = 0;
  $$01 = $fd;$i$22 = $$lcssa;
  while(1) {
   $11 = (($$01>>>0) % 10)&-1;
   $12 = $11 | 48;
   $13 = $12&255;
   $14 = (($i$22) + -1)|0;
   $15 = (($buf) + ($14)|0);
   HEAP8[$15>>0] = $13;
   $16 = (($$01>>>0) / 10)&-1;
   $17 = ($$01>>>0)<(10);
   if ($17) {
    break;
   } else {
    $$01 = $16;$i$22 = $14;
   }
  }
 }
 return;
}
function ___syscall_ret($r) {
 $r = $r|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($r>>>0)>(4294963200);
 if ($0) {
  $1 = (0 - ($r))|0;
  $2 = (___errno_location()|0);
  HEAP32[$2>>2] = $1;
  $$0 = -1;
 } else {
  $$0 = $r;
 }
 return ($$0|0);
}
function ___expo2($x) {
 $x = +$x;
 var $0 = 0.0, $1 = 0.0, $2 = 0.0, $3 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $x + -1416.0996898839683;
 $1 = (+Math_exp((+$0)));
 $2 = $1 * 2.2471164185778949E+307;
 $3 = $2 * 2.2471164185778949E+307;
 return (+$3);
}
function _acosh($x) {
 $x = +$x;
 var $$0 = 0.0, $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $3 = 0, $4 = 0;
 var $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0.0, $9 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 $5 = ($4>>>0)<(1024);
 do {
  if ($5) {
   $6 = $x + -1.0;
   $7 = $6 * $6;
   $8 = $6 * 2.0;
   $9 = $7 + $8;
   $10 = (+Math_sqrt((+$9)));
   $11 = $6 + $10;
   $12 = (+_log1p($11));
   $$0 = $12;
  } else {
   $13 = ($4>>>0)<(1049);
   if ($13) {
    $14 = $x * 2.0;
    $15 = $x * $x;
    $16 = $15 + -1.0;
    $17 = (+Math_sqrt((+$16)));
    $18 = $17 + $x;
    $19 = 1.0 / $18;
    $20 = $14 - $19;
    $21 = (+Math_log((+$20)));
    $$0 = $21;
    break;
   } else {
    $22 = (+Math_log((+$x)));
    $23 = $22 + 0.69314718055994529;
    $$0 = $23;
    break;
   }
  }
 } while(0);
 return (+$$0);
}
function _asinh($x) {
 $x = +$x;
 var $$0 = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0;
 var $26 = 0.0, $27 = 0, $28 = 0.0, $29 = 0.0, $3 = 0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0, $8 = 0.0, $9 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 $5 = $1 & 2147483647;
 HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $5;$6 = +HEAPF64[tempDoublePtr>>3];
 $7 = ($4>>>0)>(1048);
 do {
  if ($7) {
   $8 = (+Math_log((+$6)));
   $9 = $8 + 0.69314718055994529;
   $$0 = $9;
  } else {
   $10 = ($4>>>0)>(1023);
   if ($10) {
    $11 = $6 * 2.0;
    $12 = $6 * $6;
    $13 = $12 + 1.0;
    $14 = (+Math_sqrt((+$13)));
    $15 = $6 + $14;
    $16 = 1.0 / $15;
    $17 = $11 + $16;
    $18 = (+Math_log((+$17)));
    $$0 = $18;
    break;
   }
   $19 = ($4>>>0)>(996);
   if ($19) {
    $20 = $6 * $6;
    $21 = $20 + 1.0;
    $22 = (+Math_sqrt((+$21)));
    $23 = $22 + 1.0;
    $24 = $20 / $23;
    $25 = $6 + $24;
    $26 = (+_log1p($25));
    $$0 = $26;
   } else {
    $$0 = $6;
   }
  }
 } while(0);
 $27 = ($1|0)<(0);
 $28 = -$$0;
 $29 = $27 ? $28 : $$0;
 return (+$29);
}
function _atanh($x) {
 $x = +$x;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0.0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0.0, $7 = 0, $8 = 0, $9 = 0.0, $y$0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = (_bitshift64Lshr(($0|0),($1|0),52)|0);
 $3 = tempRet0;
 $4 = $2 & 2047;
 $5 = $1 & 2147483647;
 HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $5;$6 = +HEAPF64[tempDoublePtr>>3];
 $7 = ($4>>>0)<(1022);
 if ($7) {
  $8 = ($4>>>0)<(991);
  if ($8) {
   $y$0 = $6;
  } else {
   $9 = $6 * 2.0;
   $10 = $6 * $9;
   $11 = 1.0 - $6;
   $12 = $10 / $11;
   $13 = $9 + $12;
   $14 = (+_log1p($13));
   $15 = $14 * 0.5;
   $y$0 = $15;
  }
 } else {
  $16 = 1.0 - $6;
  $17 = $6 / $16;
  $18 = $17 * 2.0;
  $19 = (+_log1p($18));
  $20 = $19 * 0.5;
  $y$0 = $20;
 }
 $21 = ($1|0)<(0);
 $22 = -$y$0;
 $23 = $21 ? $22 : $y$0;
 return (+$23);
}
function _cosh($x) {
 $x = +$x;
 var $$0 = 0.0, $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $2 = 0, $3 = 0.0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0.0, $9 = 0.0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = $1 & 2147483647;
 HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $2;$3 = +HEAPF64[tempDoublePtr>>3];
 $4 = ($2>>>0)<(1072049730);
 do {
  if ($4) {
   $5 = ($2>>>0)<(1045430272);
   if ($5) {
    $$0 = 1.0;
   } else {
    $6 = (+_expm1($3));
    $7 = $6 * $6;
    $8 = $6 + 1.0;
    $9 = $8 * 2.0;
    $10 = $7 / $9;
    $11 = $10 + 1.0;
    $$0 = $11;
   }
  } else {
   $12 = ($2>>>0)<(1082535490);
   if ($12) {
    $13 = (+Math_exp((+$3)));
    $14 = 1.0 / $13;
    $15 = $13 + $14;
    $16 = $15 * 0.5;
    $$0 = $16;
    break;
   } else {
    $17 = (+___expo2($3));
    $$0 = $17;
    break;
   }
  }
 } while(0);
 return (+$$0);
}
function _expm1($x) {
 $x = +$x;
 var $$0 = 0.0, $$02 = 0.0, $$pn = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0.0, $15 = 0, $16 = 0, $17 = 0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0, $22 = 0.0, $23 = 0.0;
 var $24 = 0, $25 = 0.0, $26 = 0.0, $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0;
 var $42 = 0.0, $43 = 0.0, $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0, $53 = 0.0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0;
 var $60 = 0.0, $61 = 0.0, $62 = 0.0, $63 = 0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0.0, $74 = 0, $75 = 0.0, $76 = 0.0, $77 = 0, $78 = 0.0;
 var $79 = 0.0, $8 = 0, $80 = 0.0, $81 = 0.0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0.0, $87 = 0.0, $88 = 0.0, $89 = 0.0, $9 = 0, $90 = 0.0, $91 = 0.0, $92 = 0.0, $93 = 0.0, $c$0 = 0.0, $hi$0 = 0.0, $k$0 = 0;
 var $k$1 = 0, $lo$0 = 0.0, $y$0 = 0.0, $y$1 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = $1 & 2147483647;
 $3 = (_bitshift64Lshr(($0|0),($1|0),63)|0);
 $4 = tempRet0;
 $5 = ($2>>>0)>(1078159481);
 do {
  if ($5) {
   $6 = $1 & 2147483647;
   $7 = ($6>>>0)>(2146435072);
   $8 = ($0>>>0)>(0);
   $9 = ($6|0)==(2146435072);
   $10 = $9 & $8;
   $11 = $7 | $10;
   if ($11) {
    $$0 = $x;
   } else {
    $12 = ($3|0)==(0);
    if ($12) {
     $13 = $x > 709.78271289338397;
     if ($13) {
      $14 = $x * 8.9884656743115795E+307;
      $$0 = $14;
     } else {
      label = 11;
     }
    } else {
     $$0 = -1.0;
    }
   }
  } else {
   $15 = ($2>>>0)>(1071001154);
   if (!($15)) {
    $32 = ($2>>>0)<(1016070144);
    if ($32) {
     $$0 = $x;
     break;
    } else {
     $$02 = $x;$c$0 = 0.0;$k$1 = 0;
     label = 14;
     break;
    }
   }
   $16 = ($2>>>0)<(1072734898);
   if ($16) {
    $17 = ($3|0)==(0);
    if ($17) {
     $18 = $x + -0.69314718036912382;
     $hi$0 = $18;$k$0 = 1;$lo$0 = 1.9082149292705877E-10;
     label = 12;
     break;
    } else {
     $19 = $x + 0.69314718036912382;
     $hi$0 = $19;$k$0 = -1;$lo$0 = -1.9082149292705877E-10;
     label = 12;
     break;
    }
   } else {
    label = 11;
   }
  }
 } while(0);
 if ((label|0) == 11) {
  $20 = $x * 1.4426950408889634;
  $21 = ($3|0)!=(0);
  $22 = $21 ? -0.5 : 0.5;
  $23 = $20 + $22;
  $24 = (~~(($23)));
  $25 = (+($24|0));
  $26 = $25 * 0.69314718036912382;
  $27 = $x - $26;
  $28 = $25 * 1.9082149292705877E-10;
  $hi$0 = $27;$k$0 = $24;$lo$0 = $28;
  label = 12;
 }
 if ((label|0) == 12) {
  $29 = $hi$0 - $lo$0;
  $30 = $hi$0 - $29;
  $31 = $30 - $lo$0;
  $$02 = $29;$c$0 = $31;$k$1 = $k$0;
  label = 14;
 }
 L18: do {
  if ((label|0) == 14) {
   $33 = $$02 * 0.5;
   $34 = $$02 * $33;
   $35 = $34 * 2.0109921818362437E-7;
   $36 = 4.0082178273293624E-6 - $35;
   $37 = $34 * $36;
   $38 = $37 + -7.9365075786748794E-5;
   $39 = $34 * $38;
   $40 = $39 + 0.0015873015872548146;
   $41 = $34 * $40;
   $42 = $41 + -0.033333333333333132;
   $43 = $34 * $42;
   $44 = $43 + 1.0;
   $45 = $33 * $44;
   $46 = 3.0 - $45;
   $47 = $44 - $46;
   $48 = $$02 * $46;
   $49 = 6.0 - $48;
   $50 = $47 / $49;
   $51 = $34 * $50;
   $52 = ($k$1|0)==(0);
   if ($52) {
    $53 = $$02 * $51;
    $54 = $53 - $34;
    $55 = $$02 - $54;
    $$0 = $55;
    break;
   }
   $56 = $51 - $c$0;
   $57 = $$02 * $56;
   $58 = $57 - $c$0;
   $59 = $58 - $34;
   switch ($k$1|0) {
   case -1:  {
    $60 = $$02 - $59;
    $61 = $60 * 0.5;
    $62 = $61 + -0.5;
    $$0 = $62;
    break L18;
    break;
   }
   case 1:  {
    $63 = $$02 < -0.25;
    if ($63) {
     $64 = $$02 + 0.5;
     $65 = $59 - $64;
     $66 = $65 * -2.0;
     $$0 = $66;
     break L18;
    } else {
     $67 = $$02 - $59;
     $68 = $67 * 2.0;
     $69 = $68 + 1.0;
     $$0 = $69;
     break L18;
    }
    break;
   }
   default: {
    $70 = (($k$1) + 1023)|0;
    $71 = (_bitshift64Shl(($70|0),0,52)|0);
    $72 = tempRet0;
    HEAP32[tempDoublePtr>>2] = $71;HEAP32[tempDoublePtr+4>>2] = $72;$73 = +HEAPF64[tempDoublePtr>>3];
    $74 = ($k$1>>>0)>(56);
    if ($74) {
     $75 = $$02 - $59;
     $76 = $75 + 1.0;
     $77 = ($k$1|0)==(1024);
     $78 = $76 * 2.0;
     $79 = $78 * 8.9884656743115795E+307;
     $80 = $73 * $76;
     $y$0 = $77 ? $79 : $80;
     $81 = $y$0 + -1.0;
     $$0 = $81;
     break L18;
    }
    $82 = (1023 - ($k$1))|0;
    $83 = (_bitshift64Shl(($82|0),0,52)|0);
    $84 = tempRet0;
    $85 = ($k$1|0)<(20);
    if ($85) {
     $86 = $$02 - $59;
     HEAP32[tempDoublePtr>>2] = $83;HEAP32[tempDoublePtr+4>>2] = $84;$87 = +HEAPF64[tempDoublePtr>>3];
     $88 = 1.0 - $87;
     $89 = $88 + $86;
     $$pn = $89;
    } else {
     HEAP32[tempDoublePtr>>2] = $83;HEAP32[tempDoublePtr+4>>2] = $84;$90 = +HEAPF64[tempDoublePtr>>3];
     $91 = $90 + $59;
     $92 = $$02 - $91;
     $93 = $92 + 1.0;
     $$pn = $93;
    }
    $y$1 = $73 * $$pn;
    $$0 = $y$1;
    break L18;
   }
   }
  }
 } while(0);
 return (+$$0);
}
function _log10($x) {
 $x = +$x;
 var $$0 = 0.0, $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0;
 var $26 = 0.0, $27 = 0.0, $28 = 0.0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0.0;
 var $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0, $49 = 0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $59 = 0.0, $6 = 0, $60 = 0.0, $61 = 0.0;
 var $62 = 0.0, $63 = 0.0, $64 = 0.0, $65 = 0.0, $66 = 0.0, $67 = 0.0, $68 = 0.0, $69 = 0.0, $7 = 0, $70 = 0, $8 = 0.0, $9 = 0.0, $hx$0 = 0, $k$0 = 0, $or$cond = 0, $or$cond4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = ($1>>>0)<(1048576);
 $3 = ($1|0)<(0);
 $or$cond = $3 | $2;
 do {
  if ($or$cond) {
   $4 = $1 & 2147483647;
   $5 = ($0|0)==(0);
   $6 = ($4|0)==(0);
   $7 = $5 & $6;
   if ($7) {
    $8 = $x * $x;
    $9 = -1.0 / $8;
    $$0 = $9;
    break;
   }
   if ($3) {
    $10 = $x - $x;
    $11 = $10 / 0.0;
    $$0 = $11;
    break;
   } else {
    $12 = $x * 18014398509481984.0;
    HEAPF64[tempDoublePtr>>3] = $12;$13 = HEAP32[tempDoublePtr>>2]|0;
    $14 = HEAP32[tempDoublePtr+4>>2]|0;
    $25 = $13;$70 = $14;$hx$0 = $14;$k$0 = -1077;
    label = 9;
    break;
   }
  } else {
   $15 = ($1>>>0)>(2146435071);
   if ($15) {
    $$0 = $x;
   } else {
    $16 = ($1|0)==(1072693248);
    $17 = ($0|0)==(0);
    $18 = (0)==(0);
    $19 = $17 & $18;
    $or$cond4 = $19 & $16;
    if ($or$cond4) {
     $$0 = 0.0;
    } else {
     $25 = $0;$70 = $1;$hx$0 = $1;$k$0 = -1023;
     label = 9;
    }
   }
  }
 } while(0);
 if ((label|0) == 9) {
  $20 = (($hx$0) + 614242)|0;
  $21 = $20 >>> 20;
  $22 = (($k$0) + ($21))|0;
  $23 = $20 & 1048575;
  $24 = (($23) + 1072079006)|0;
  HEAP32[tempDoublePtr>>2] = $25;HEAP32[tempDoublePtr+4>>2] = $24;$26 = +HEAPF64[tempDoublePtr>>3];
  $27 = $26 + -1.0;
  $28 = $27 * 0.5;
  $29 = $27 * $28;
  $30 = $27 + 2.0;
  $31 = $27 / $30;
  $32 = $31 * $31;
  $33 = $32 * $32;
  $34 = $33 * 0.15313837699209373;
  $35 = $34 + 0.22222198432149784;
  $36 = $33 * $35;
  $37 = $36 + 0.39999999999409419;
  $38 = $33 * $37;
  $39 = $33 * 0.14798198605116586;
  $40 = $39 + 0.1818357216161805;
  $41 = $33 * $40;
  $42 = $41 + 0.28571428743662391;
  $43 = $33 * $42;
  $44 = $43 + 0.66666666666667351;
  $45 = $32 * $44;
  $46 = $38 + $45;
  $47 = $27 - $29;
  HEAPF64[tempDoublePtr>>3] = $47;$48 = HEAP32[tempDoublePtr>>2]|0;
  $49 = HEAP32[tempDoublePtr+4>>2]|0;
  HEAP32[tempDoublePtr>>2] = 0;HEAP32[tempDoublePtr+4>>2] = $49;$50 = +HEAPF64[tempDoublePtr>>3];
  $51 = $27 - $50;
  $52 = $51 - $29;
  $53 = $29 + $46;
  $54 = $31 * $53;
  $55 = $54 + $52;
  $56 = $50 * 0.43429448187816888;
  $57 = (+($22|0));
  $58 = $57 * 0.30102999566361177;
  $59 = $57 * 3.6942390771589308E-13;
  $60 = $50 + $55;
  $61 = $60 * 2.5082946711645275E-11;
  $62 = $59 + $61;
  $63 = $55 * 0.43429448187816888;
  $64 = $63 + $62;
  $65 = $58 + $56;
  $66 = $58 - $65;
  $67 = $56 + $66;
  $68 = $67 + $64;
  $69 = $65 + $68;
  $$0 = $69;
 }
 return (+$$0);
}
function _log1p($x) {
 $x = +$x;
 var $$0 = 0.0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0.0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0.0, $25 = 0.0;
 var $26 = 0.0, $27 = 0, $28 = 0, $29 = 0.0, $3 = 0, $30 = 0.0, $31 = 0.0, $32 = 0.0, $33 = 0.0, $34 = 0.0, $35 = 0.0, $36 = 0.0, $37 = 0.0, $38 = 0.0, $39 = 0.0, $4 = 0, $40 = 0.0, $41 = 0.0, $42 = 0.0, $43 = 0.0;
 var $44 = 0.0, $45 = 0.0, $46 = 0.0, $47 = 0.0, $48 = 0.0, $49 = 0.0, $5 = 0, $50 = 0.0, $51 = 0.0, $52 = 0.0, $53 = 0.0, $54 = 0.0, $55 = 0.0, $56 = 0.0, $57 = 0.0, $58 = 0.0, $6 = 0.0, $7 = 0.0, $8 = 0, $9 = 0;
 var $c$1 = 0.0, $c$2 = 0.0, $f$1 = 0.0, $k$1 = 0.0, $or$cond = 0, $phitmp = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = ($1>>>0)<(1071284858);
 $3 = ($1|0)<(0);
 $or$cond = $3 | $2;
 do {
  if ($or$cond) {
   $4 = ($1>>>0)>(3220176895);
   if ($4) {
    $5 = $x == -1.0;
    if ($5) {
     $$0 = -inf;
     break;
    }
    $6 = $x - $x;
    $7 = $6 / 0.0;
    $$0 = $7;
    break;
   } else {
    $8 = (_bitshift64Shl(($1|0),0,1)|0);
    $9 = tempRet0;
    $10 = ($8>>>0)<(2034237440);
    if ($10) {
     $$0 = $x;
     break;
    }
    $11 = ($1>>>0)<(3218259653);
    if ($11) {
     $c$2 = 0.0;$f$1 = $x;$k$1 = 0.0;
     label = 11;
     break;
    } else {
     label = 8;
     break;
    }
   }
  } else {
   $12 = ($1>>>0)>(2146435071);
   if ($12) {
    $$0 = $x;
   } else {
    label = 8;
   }
  }
 } while(0);
 if ((label|0) == 8) {
  $13 = $x + 1.0;
  HEAPF64[tempDoublePtr>>3] = $13;$14 = HEAP32[tempDoublePtr>>2]|0;
  $15 = HEAP32[tempDoublePtr+4>>2]|0;
  $16 = (($15) + 614242)|0;
  $17 = $16 >>> 20;
  $18 = (($17) + -1023)|0;
  $19 = ($18|0)<(54);
  if ($19) {
   $20 = ($18|0)>(1);
   $21 = $13 - $x;
   $22 = 1.0 - $21;
   $23 = $13 + -1.0;
   $24 = $x - $23;
   $25 = $20 ? $22 : $24;
   $26 = $25 / $13;
   $c$1 = $26;
  } else {
   $c$1 = 0.0;
  }
  $27 = $16 & 1048575;
  $28 = (($27) + 1072079006)|0;
  HEAP32[tempDoublePtr>>2] = $14;HEAP32[tempDoublePtr+4>>2] = $28;$29 = +HEAPF64[tempDoublePtr>>3];
  $30 = $29 + -1.0;
  $phitmp = (+($18|0));
  $c$2 = $c$1;$f$1 = $30;$k$1 = $phitmp;
  label = 11;
 }
 if ((label|0) == 11) {
  $31 = $f$1 * 0.5;
  $32 = $f$1 * $31;
  $33 = $f$1 + 2.0;
  $34 = $f$1 / $33;
  $35 = $34 * $34;
  $36 = $35 * $35;
  $37 = $36 * 0.15313837699209373;
  $38 = $37 + 0.22222198432149784;
  $39 = $36 * $38;
  $40 = $39 + 0.39999999999409419;
  $41 = $36 * $40;
  $42 = $36 * 0.14798198605116586;
  $43 = $42 + 0.1818357216161805;
  $44 = $36 * $43;
  $45 = $44 + 0.28571428743662391;
  $46 = $36 * $45;
  $47 = $46 + 0.66666666666667351;
  $48 = $35 * $47;
  $49 = $41 + $48;
  $50 = $32 + $49;
  $51 = $34 * $50;
  $52 = $k$1 * 1.9082149292705877E-10;
  $53 = $c$2 + $52;
  $54 = $53 + $51;
  $55 = $54 - $32;
  $56 = $f$1 + $55;
  $57 = $k$1 * 0.69314718036912382;
  $58 = $57 + $56;
  $$0 = $58;
 }
 return (+$$0);
}
function _sinh($x) {
 $x = +$x;
 var $$ = 0.0, $$0 = 0.0, $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0.0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0.0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $3 = 0, $4 = 0.0, $5 = 0;
 var $6 = 0.0, $7 = 0, $8 = 0, $9 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = ($1|0)<(0);
 $$ = $2 ? -0.5 : 0.5;
 $3 = $1 & 2147483647;
 HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $3;$4 = +HEAPF64[tempDoublePtr>>3];
 $5 = ($3>>>0)<(1082535490);
 do {
  if ($5) {
   $6 = (+_expm1($4));
   $7 = ($3>>>0)<(1072693248);
   if (!($7)) {
    $15 = $6 + 1.0;
    $16 = $6 / $15;
    $17 = $6 + $16;
    $18 = $$ * $17;
    $$0 = $18;
    break;
   }
   $8 = ($3>>>0)<(1045430272);
   if ($8) {
    $$0 = $x;
   } else {
    $9 = $6 * 2.0;
    $10 = $6 * $6;
    $11 = $6 + 1.0;
    $12 = $10 / $11;
    $13 = $9 - $12;
    $14 = $$ * $13;
    $$0 = $14;
   }
  } else {
   $19 = $$ * 2.0;
   $20 = (+___expo2($4));
   $21 = $19 * $20;
   $$0 = $21;
  }
 } while(0);
 return (+$$0);
}
function _tanh($x) {
 $x = +$x;
 var $0 = 0, $1 = 0, $10 = 0.0, $11 = 0.0, $12 = 0.0, $13 = 0, $14 = 0.0, $15 = 0.0, $16 = 0.0, $17 = 0.0, $18 = 0, $19 = 0.0, $2 = 0, $20 = 0.0, $21 = 0.0, $22 = 0.0, $23 = 0.0, $24 = 0, $25 = 0.0, $26 = 0.0;
 var $3 = 0.0, $4 = 0, $5 = 0, $6 = 0.0, $7 = 0.0, $8 = 0.0, $9 = 0.0, $t$0 = 0.0, label = 0, sp = 0;
 sp = STACKTOP;
 HEAPF64[tempDoublePtr>>3] = $x;$0 = HEAP32[tempDoublePtr>>2]|0;
 $1 = HEAP32[tempDoublePtr+4>>2]|0;
 $2 = $1 & 2147483647;
 HEAP32[tempDoublePtr>>2] = $0;HEAP32[tempDoublePtr+4>>2] = $2;$3 = +HEAPF64[tempDoublePtr>>3];
 $4 = ($2>>>0)>(1071748074);
 do {
  if ($4) {
   $5 = ($2>>>0)>(1077149696);
   if ($5) {
    $6 = 0.0 / $3;
    $7 = 1.0 - $6;
    $t$0 = $7;
    break;
   } else {
    $8 = $3 * 2.0;
    $9 = (+_expm1($8));
    $10 = $9 + 2.0;
    $11 = 2.0 / $10;
    $12 = 1.0 - $11;
    $t$0 = $12;
    break;
   }
  } else {
   $13 = ($2>>>0)>(1070618798);
   if ($13) {
    $14 = $3 * 2.0;
    $15 = (+_expm1($14));
    $16 = $15 + 2.0;
    $17 = $15 / $16;
    $t$0 = $17;
    break;
   }
   $18 = ($2>>>0)>(1048575);
   if ($18) {
    $19 = $3 * -2.0;
    $20 = (+_expm1($19));
    $21 = -$20;
    $22 = $20 + 2.0;
    $23 = $21 / $22;
    $t$0 = $23;
   } else {
    $t$0 = $3;
   }
  }
 } while(0);
 $24 = ($1|0)<(0);
 $25 = -$t$0;
 $26 = $24 ? $25 : $t$0;
 return (+$26);
}
function ___mmap($start,$len,$prot,$flags,$fd,$off) {
 $start = $start|0;
 $len = $len|0;
 $prot = $prot|0;
 $flags = $flags|0;
 $fd = $fd|0;
 $off = $off|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0;
 var $vararg_ptr3 = 0, $vararg_ptr4 = 0, $vararg_ptr5 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $0 = ($off|0)<(0);
 $1 = $0 << 31 >> 31;
 $2 = $off & 4095;
 $3 = $1 & -4096;
 $4 = ($2|0)==(0);
 $5 = ($3|0)==(0);
 $6 = $4 & $5;
 do {
  if ($6) {
   $8 = ($len>>>0)>(2147483646);
   if ($8) {
    $9 = (___errno_location()|0);
    HEAP32[$9>>2] = 12;
    $$0 = (-1);
    break;
   }
   $10 = $flags & 16;
   $11 = ($10|0)!=(0);
   if ($11) {
    _dummy1(-1);
   }
   $12 = $off >> 12;
   HEAP32[$vararg_buffer>>2] = $start;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $len;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $prot;
   $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
   HEAP32[$vararg_ptr3>>2] = $flags;
   $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
   HEAP32[$vararg_ptr4>>2] = $fd;
   $vararg_ptr5 = ((($vararg_buffer)) + 20|0);
   HEAP32[$vararg_ptr5>>2] = $12;
   $13 = (___syscall192(192,($vararg_buffer|0))|0);
   $14 = (___syscall_ret($13)|0);
   $15 = $14;
   if ($11) {
    _dummy0();
    $$0 = $15;
   } else {
    $$0 = $15;
   }
  } else {
   $7 = (___errno_location()|0);
   HEAP32[$7>>2] = 22;
   $$0 = (-1);
  }
 } while(0);
 STACKTOP = sp;return ($$0|0);
}
function ___munmap($start,$len) {
 $start = $start|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 _dummy1(-1);
 HEAP32[$vararg_buffer>>2] = $start;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $len;
 $0 = (___syscall91(91,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 _dummy0();
 STACKTOP = sp;return ($1|0);
}
function _fchmod($fd,$mode) {
 $fd = $fd|0;
 $mode = $mode|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $buf = sp + 24|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $mode;
 $0 = (___syscall94(94,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-9);
 if ($1) {
  HEAP32[$vararg_buffer2>>2] = $fd;
  $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
  HEAP32[$vararg_ptr5>>2] = 1;
  $2 = (___syscall221(221,($vararg_buffer2|0))|0);
  $3 = ($2|0)<(0);
  if ($3) {
   label = 3;
  } else {
   ___procfdname($buf,$fd);
   HEAP32[$vararg_buffer6>>2] = $buf;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $mode;
   $5 = (___syscall15(15,($vararg_buffer6|0))|0);
   $6 = (___syscall_ret($5)|0);
   $$0 = $6;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $4 = (___syscall_ret($0)|0);
  $$0 = $4;
 }
 STACKTOP = sp;return ($$0|0);
}
function _fstat($fd,$st) {
 $fd = $fd|0;
 $st = $st|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $vararg_buffer = 0, $vararg_buffer2 = 0, $vararg_buffer6 = 0, $vararg_ptr1 = 0, $vararg_ptr5 = 0, $vararg_ptr9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer6 = sp + 16|0;
 $vararg_buffer2 = sp + 8|0;
 $vararg_buffer = sp;
 $buf = sp + 24|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $st;
 $0 = (___syscall197(197,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-9);
 if ($1) {
  HEAP32[$vararg_buffer2>>2] = $fd;
  $vararg_ptr5 = ((($vararg_buffer2)) + 4|0);
  HEAP32[$vararg_ptr5>>2] = 1;
  $2 = (___syscall221(221,($vararg_buffer2|0))|0);
  $3 = ($2|0)<(0);
  if ($3) {
   label = 3;
  } else {
   ___procfdname($buf,$fd);
   HEAP32[$vararg_buffer6>>2] = $buf;
   $vararg_ptr9 = ((($vararg_buffer6)) + 4|0);
   HEAP32[$vararg_ptr9>>2] = $st;
   $5 = (___syscall195(195,($vararg_buffer6|0))|0);
   $6 = (___syscall_ret($5)|0);
   $$0 = $6;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $4 = (___syscall_ret($0)|0);
  $$0 = $4;
 }
 STACKTOP = sp;return ($$0|0);
}
function _lstat($path,$buf) {
 $path = $path|0;
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $0 = (___syscall196(196,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _mkdir($path,$mode) {
 $path = $path|0;
 $mode = $mode|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $mode;
 $0 = (___syscall39(39,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _stat($path,$buf) {
 $path = $path|0;
 $buf = $buf|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $0 = (___syscall195(195,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function ___lockfile($f) {
 $f = $f|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return 0;
}
function ___unlockfile($f) {
 $f = $f|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function ___stdio_close($f) {
 $f = $f|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = ((($f)) + 60|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 $2 = (___syscall6(6,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 STACKTOP = sp;return ($3|0);
}
function ___stdio_seek($f,$off,$whence) {
 $f = $f|0;
 $off = $off|0;
 $whence = $whence|0;
 var $$pre = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $ret = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $ret = sp + 20|0;
 $0 = ((($f)) + 60|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$vararg_buffer>>2] = $1;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $off;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $ret;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $whence;
 $2 = (___syscall140(140,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 $4 = ($3|0)<(0);
 if ($4) {
  HEAP32[$ret>>2] = -1;
  $5 = -1;
 } else {
  $$pre = HEAP32[$ret>>2]|0;
  $5 = $$pre;
 }
 STACKTOP = sp;return ($5|0);
}
function ___stdio_write($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $$0 = 0, $$phi$trans$insert = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0;
 var $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $cnt$0 = 0, $cnt$1 = 0, $iov$0 = 0, $iov$0$lcssa11 = 0, $iov$1 = 0, $iovcnt$0 = 0;
 var $iovcnt$0$lcssa12 = 0, $iovcnt$1 = 0, $iovs = 0, $rem$0 = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, $vararg_ptr7 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 48|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $iovs = sp + 32|0;
 $0 = ((($f)) + 28|0);
 $1 = HEAP32[$0>>2]|0;
 HEAP32[$iovs>>2] = $1;
 $2 = ((($iovs)) + 4|0);
 $3 = ((($f)) + 20|0);
 $4 = HEAP32[$3>>2]|0;
 $5 = $4;
 $6 = (($5) - ($1))|0;
 HEAP32[$2>>2] = $6;
 $7 = ((($iovs)) + 8|0);
 HEAP32[$7>>2] = $buf;
 $8 = ((($iovs)) + 12|0);
 HEAP32[$8>>2] = $len;
 $9 = (($6) + ($len))|0;
 $10 = ((($f)) + 60|0);
 $11 = ((($f)) + 44|0);
 $iov$0 = $iovs;$iovcnt$0 = 2;$rem$0 = $9;
 while(1) {
  $12 = HEAP32[8864>>2]|0;
  $13 = ($12|0)==(0|0);
  if ($13) {
   $17 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer3>>2] = $17;
   $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
   HEAP32[$vararg_ptr6>>2] = $iov$0;
   $vararg_ptr7 = ((($vararg_buffer3)) + 8|0);
   HEAP32[$vararg_ptr7>>2] = $iovcnt$0;
   $18 = (___syscall146(146,($vararg_buffer3|0))|0);
   $19 = (___syscall_ret($18)|0);
   $cnt$0 = $19;
  } else {
   _pthread_cleanup_push((154|0),($f|0));
   $14 = HEAP32[$10>>2]|0;
   HEAP32[$vararg_buffer>>2] = $14;
   $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
   HEAP32[$vararg_ptr1>>2] = $iov$0;
   $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
   HEAP32[$vararg_ptr2>>2] = $iovcnt$0;
   $15 = (___syscall146(146,($vararg_buffer|0))|0);
   $16 = (___syscall_ret($15)|0);
   _pthread_cleanup_pop(0);
   $cnt$0 = $16;
  }
  $20 = ($rem$0|0)==($cnt$0|0);
  if ($20) {
   label = 6;
   break;
  }
  $27 = ($cnt$0|0)<(0);
  if ($27) {
   $iov$0$lcssa11 = $iov$0;$iovcnt$0$lcssa12 = $iovcnt$0;
   label = 8;
   break;
  }
  $35 = (($rem$0) - ($cnt$0))|0;
  $36 = ((($iov$0)) + 4|0);
  $37 = HEAP32[$36>>2]|0;
  $38 = ($cnt$0>>>0)>($37>>>0);
  if ($38) {
   $39 = HEAP32[$11>>2]|0;
   HEAP32[$0>>2] = $39;
   HEAP32[$3>>2] = $39;
   $40 = (($cnt$0) - ($37))|0;
   $41 = ((($iov$0)) + 8|0);
   $42 = (($iovcnt$0) + -1)|0;
   $$phi$trans$insert = ((($iov$0)) + 12|0);
   $$pre = HEAP32[$$phi$trans$insert>>2]|0;
   $50 = $$pre;$cnt$1 = $40;$iov$1 = $41;$iovcnt$1 = $42;
  } else {
   $43 = ($iovcnt$0|0)==(2);
   if ($43) {
    $44 = HEAP32[$0>>2]|0;
    $45 = (($44) + ($cnt$0)|0);
    HEAP32[$0>>2] = $45;
    $50 = $37;$cnt$1 = $cnt$0;$iov$1 = $iov$0;$iovcnt$1 = 2;
   } else {
    $50 = $37;$cnt$1 = $cnt$0;$iov$1 = $iov$0;$iovcnt$1 = $iovcnt$0;
   }
  }
  $46 = HEAP32[$iov$1>>2]|0;
  $47 = (($46) + ($cnt$1)|0);
  HEAP32[$iov$1>>2] = $47;
  $48 = ((($iov$1)) + 4|0);
  $49 = (($50) - ($cnt$1))|0;
  HEAP32[$48>>2] = $49;
  $iov$0 = $iov$1;$iovcnt$0 = $iovcnt$1;$rem$0 = $35;
 }
 if ((label|0) == 6) {
  $21 = HEAP32[$11>>2]|0;
  $22 = ((($f)) + 48|0);
  $23 = HEAP32[$22>>2]|0;
  $24 = (($21) + ($23)|0);
  $25 = ((($f)) + 16|0);
  HEAP32[$25>>2] = $24;
  $26 = $21;
  HEAP32[$0>>2] = $26;
  HEAP32[$3>>2] = $26;
  $$0 = $len;
 }
 else if ((label|0) == 8) {
  $28 = ((($f)) + 16|0);
  HEAP32[$28>>2] = 0;
  HEAP32[$0>>2] = 0;
  HEAP32[$3>>2] = 0;
  $29 = HEAP32[$f>>2]|0;
  $30 = $29 | 32;
  HEAP32[$f>>2] = $30;
  $31 = ($iovcnt$0$lcssa12|0)==(2);
  if ($31) {
   $$0 = 0;
  } else {
   $32 = ((($iov$0$lcssa11)) + 4|0);
   $33 = HEAP32[$32>>2]|0;
   $34 = (($len) - ($33))|0;
   $$0 = $34;
  }
 }
 STACKTOP = sp;return ($$0|0);
}
function ___stdout_write($f,$buf,$len) {
 $f = $f|0;
 $buf = $buf|0;
 $len = $len|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $tio = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 80|0;
 $vararg_buffer = sp;
 $tio = sp + 12|0;
 $0 = ((($f)) + 36|0);
 HEAP32[$0>>2] = 163;
 $1 = HEAP32[$f>>2]|0;
 $2 = $1 & 64;
 $3 = ($2|0)==(0);
 if ($3) {
  $4 = ((($f)) + 60|0);
  $5 = HEAP32[$4>>2]|0;
  HEAP32[$vararg_buffer>>2] = $5;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = 21505;
  $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
  HEAP32[$vararg_ptr2>>2] = $tio;
  $6 = (___syscall54(54,($vararg_buffer|0))|0);
  $7 = ($6|0)==(0);
  if (!($7)) {
   $8 = ((($f)) + 75|0);
   HEAP8[$8>>0] = -1;
  }
 }
 $9 = (___stdio_write($f,$buf,$len)|0);
 STACKTOP = sp;return ($9|0);
}
function _fflush($f) {
 $f = $f|0;
 var $$0 = 0, $$01 = 0, $$012 = 0, $$014 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0;
 var $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $phitmp = 0, $r$0$lcssa = 0, $r$03 = 0, $r$1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($f|0)==(0|0);
 do {
  if ($0) {
   $7 = HEAP32[8908>>2]|0;
   $8 = ($7|0)==(0|0);
   if ($8) {
    $27 = 0;
   } else {
    $9 = HEAP32[8908>>2]|0;
    $10 = (_fflush($9)|0);
    $27 = $10;
   }
   ___lock(((8892)|0));
   $$012 = HEAP32[(8888)>>2]|0;
   $11 = ($$012|0)==(0|0);
   if ($11) {
    $r$0$lcssa = $27;
   } else {
    $$014 = $$012;$r$03 = $27;
    while(1) {
     $12 = ((($$014)) + 76|0);
     $13 = HEAP32[$12>>2]|0;
     $14 = ($13|0)>(-1);
     if ($14) {
      $15 = (___lockfile($$014)|0);
      $24 = $15;
     } else {
      $24 = 0;
     }
     $16 = ((($$014)) + 20|0);
     $17 = HEAP32[$16>>2]|0;
     $18 = ((($$014)) + 28|0);
     $19 = HEAP32[$18>>2]|0;
     $20 = ($17>>>0)>($19>>>0);
     if ($20) {
      $21 = (___fflush_unlocked($$014)|0);
      $22 = $21 | $r$03;
      $r$1 = $22;
     } else {
      $r$1 = $r$03;
     }
     $23 = ($24|0)==(0);
     if (!($23)) {
      ___unlockfile($$014);
     }
     $25 = ((($$014)) + 56|0);
     $$01 = HEAP32[$25>>2]|0;
     $26 = ($$01|0)==(0|0);
     if ($26) {
      $r$0$lcssa = $r$1;
      break;
     } else {
      $$014 = $$01;$r$03 = $r$1;
     }
    }
   }
   ___unlock(((8892)|0));
   $$0 = $r$0$lcssa;
  } else {
   $1 = ((($f)) + 76|0);
   $2 = HEAP32[$1>>2]|0;
   $3 = ($2|0)>(-1);
   if (!($3)) {
    $4 = (___fflush_unlocked($f)|0);
    $$0 = $4;
    break;
   }
   $5 = (___lockfile($f)|0);
   $phitmp = ($5|0)==(0);
   $6 = (___fflush_unlocked($f)|0);
   if ($phitmp) {
    $$0 = $6;
   } else {
    ___unlockfile($f);
    $$0 = $6;
   }
  }
 } while(0);
 return ($$0|0);
}
function _qsort($base,$nel,$width,$cmp) {
 $base = $base|0;
 $nel = $nel|0;
 $width = $width|0;
 $cmp = $cmp|0;
 var $$0$i = 0, $$0$i30 = 0, $$02$i$i = 0, $$02$i3$i = 0, $$lcssa = 0, $$lcssa57 = 0, $$phi$trans$insert$i = 0, $$pre = 0, $$pre$i = 0, $$pre$i11 = 0, $$pre$i20 = 0, $$pre$i5 = 0, $$pre$i8 = 0, $$pre1$i = 0, $$pre1$i12 = 0, $$pre1$i27$pre = 0, $$pre1$i6 = 0, $$pre1$i9 = 0, $$sum = 0, $$sum2 = 0;
 var $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0;
 var $116 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0;
 var $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0;
 var $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0;
 var $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $8$phi = 0, $80 = 0, $81 = 0, $82 = 0;
 var $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $head$0$lcssa = 0, $head$036 = 0;
 var $head$1$be = 0, $head$153 = 0, $i$0 = 0, $lp = 0, $nTrailingZeros$03$i$i = 0, $nTrailingZeros$03$i2$i = 0, $nTrailingZeros$03$i2$i$lcssa = 0, $or$cond = 0, $or$cond48 = 0, $or$cond4852 = 0, $or$cond51 = 0, $p = 0, $pshift$0$lcssa = 0, $pshift$037 = 0, $pshift$1 = 0, $pshift$2$be = 0, $pshift$254 = 0, $sum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 208|0;
 $lp = sp + 8|0;
 $p = sp;
 $0 = Math_imul($width, $nel)|0;
 $1 = $p;
 $2 = $1;
 HEAP32[$2>>2] = 1;
 $3 = (($1) + 4)|0;
 $4 = $3;
 HEAP32[$4>>2] = 0;
 $5 = ($0|0)==(0);
 if (!($5)) {
  $$sum = (($0) - ($width))|0;
  $6 = ((($lp)) + 4|0);
  HEAP32[$6>>2] = $width;
  HEAP32[$lp>>2] = $width;
  $10 = $width;$8 = $width;$i$0 = 2;
  while(1) {
   $7 = (($8) + ($width))|0;
   $9 = (($7) + ($10))|0;
   $11 = (($lp) + ($i$0<<2)|0);
   HEAP32[$11>>2] = $9;
   $12 = ($9>>>0)<($0>>>0);
   $13 = (($i$0) + 1)|0;
   if ($12) {
    $8$phi = $10;$10 = $9;$i$0 = $13;$8 = $8$phi;
   } else {
    break;
   }
  }
  $14 = (0 - ($width))|0;
  $15 = (($base) + ($$sum)|0);
  $16 = ($$sum|0)>(0);
  $$phi$trans$insert$i = ((($p)) + 4|0);
  if ($16) {
   $17 = $15;
   $19 = 1;$head$036 = $base;$pshift$037 = 1;
   while(1) {
    $18 = $19 & 3;
    $20 = ($18|0)==(3);
    do {
     if ($20) {
      _sift($head$036,$width,$cmp,$pshift$037,$lp);
      $$pre$i = HEAP32[$p>>2]|0;
      $$pre1$i = HEAP32[$$phi$trans$insert$i>>2]|0;
      $21 = $$pre$i >>> 2;
      $22 = $$pre1$i << 30;
      $23 = $22 | $21;
      HEAP32[$p>>2] = $23;
      $24 = $$pre1$i >>> 2;
      HEAP32[$$phi$trans$insert$i>>2] = $24;
      $25 = (($pshift$037) + 2)|0;
      $48 = $23;$pshift$1 = $25;
     } else {
      $26 = (($pshift$037) + -1)|0;
      $27 = (($lp) + ($26<<2)|0);
      $28 = HEAP32[$27>>2]|0;
      $29 = $head$036;
      $30 = (($17) - ($29))|0;
      $31 = ($28>>>0)<($30>>>0);
      if ($31) {
       _sift($head$036,$width,$cmp,$pshift$037,$lp);
      } else {
       _trinkle($head$036,$width,$cmp,$p,$pshift$037,0,$lp);
      }
      $32 = ($pshift$037|0)==(1);
      if ($32) {
       $$pre$i5 = HEAP32[$$phi$trans$insert$i>>2]|0;
       $$pre1$i6 = HEAP32[$p>>2]|0;
       $33 = $$pre$i5 << 1;
       $34 = $$pre1$i6 >>> 31;
       $35 = $34 | $33;
       HEAP32[$$phi$trans$insert$i>>2] = $35;
       $36 = $$pre1$i6 << 1;
       HEAP32[$p>>2] = $36;
       $48 = $36;$pshift$1 = 0;
       break;
      }
      $37 = ($26>>>0)>(31);
      if ($37) {
       $38 = (($pshift$037) + -33)|0;
       $39 = HEAP32[$p>>2]|0;
       HEAP32[$$phi$trans$insert$i>>2] = $39;
       HEAP32[$p>>2] = 0;
       $$0$i = $38;$41 = $39;$44 = 0;
      } else {
       $$pre$i11 = HEAP32[$$phi$trans$insert$i>>2]|0;
       $$pre1$i12 = HEAP32[$p>>2]|0;
       $$0$i = $26;$41 = $$pre$i11;$44 = $$pre1$i12;
      }
      $40 = $41 << $$0$i;
      $42 = (32 - ($$0$i))|0;
      $43 = $44 >>> $42;
      $45 = $43 | $40;
      HEAP32[$$phi$trans$insert$i>>2] = $45;
      $46 = $44 << $$0$i;
      HEAP32[$p>>2] = $46;
      $48 = $46;$pshift$1 = 1;
     }
    } while(0);
    $47 = $48 | 1;
    HEAP32[$p>>2] = $47;
    $49 = (($head$036) + ($width)|0);
    $50 = ($49>>>0)<($15>>>0);
    if ($50) {
     $19 = $47;$head$036 = $49;$pshift$037 = $pshift$1;
    } else {
     $head$0$lcssa = $49;$pshift$0$lcssa = $pshift$1;
     break;
    }
   }
  } else {
   $head$0$lcssa = $base;$pshift$0$lcssa = 1;
  }
  _trinkle($head$0$lcssa,$width,$cmp,$p,$pshift$0$lcssa,0,$lp);
  $51 = ((($p)) + 4|0);
  $52 = ($pshift$0$lcssa|0)==(1);
  $53 = HEAP32[$p>>2]|0;
  $54 = ($53|0)==(1);
  $or$cond51 = $52 & $54;
  $55 = HEAP32[$51>>2]|0;
  $56 = ($55|0)==(0);
  $or$cond4852 = $or$cond51 & $56;
  if (!($or$cond4852)) {
   $59 = $53;$head$153 = $head$0$lcssa;$pshift$254 = $pshift$0$lcssa;
   while(1) {
    $57 = ($pshift$254|0)<(2);
    if ($57) {
     $58 = (($59) + -1)|0;
     $60 = ($58|0)==(0);
     do {
      if ($60) {
       $81 = 32;
       label = 30;
      } else {
       $61 = $58 & 1;
       $62 = ($61|0)==(0);
       if ($62) {
        $$02$i$i = $58;$nTrailingZeros$03$i$i = 0;
        while(1) {
         $63 = (($nTrailingZeros$03$i$i) + 1)|0;
         $64 = $$02$i$i >>> 1;
         $65 = $64 & 1;
         $66 = ($65|0)==(0);
         if ($66) {
          $$02$i$i = $64;$nTrailingZeros$03$i$i = $63;
         } else {
          $$lcssa = $63;
          break;
         }
        }
        $67 = ($$lcssa|0)==(0);
        if ($67) {
         label = 24;
        } else {
         $79 = $$lcssa;
        }
       } else {
        label = 24;
       }
       if ((label|0) == 24) {
        label = 0;
        $68 = HEAP32[$$phi$trans$insert$i>>2]|0;
        $69 = ($68|0)==(0);
        if ($69) {
         $81 = 64;
         label = 30;
         break;
        }
        $70 = $68 & 1;
        $71 = ($70|0)==(0);
        if ($71) {
         $$02$i3$i = $68;$nTrailingZeros$03$i2$i = 0;
        } else {
         $$0$i30 = 0;$84 = $59;$87 = $68;$91 = 0;
         break;
        }
        while(1) {
         $72 = (($nTrailingZeros$03$i2$i) + 1)|0;
         $73 = $$02$i3$i >>> 1;
         $74 = $73 & 1;
         $75 = ($74|0)==(0);
         if ($75) {
          $$02$i3$i = $73;$nTrailingZeros$03$i2$i = $72;
         } else {
          $$lcssa57 = $72;$nTrailingZeros$03$i2$i$lcssa = $nTrailingZeros$03$i2$i;
          break;
         }
        }
        $76 = (($nTrailingZeros$03$i2$i$lcssa) + 33)|0;
        $77 = ($$lcssa57|0)==(0);
        if ($77) {
         $$0$i30 = 0;$84 = $59;$87 = $68;$91 = 0;
         break;
        } else {
         $79 = $76;
        }
       }
       $78 = ($79>>>0)>(31);
       if ($78) {
        $81 = $79;
        label = 30;
       } else {
        $$pre1$i27$pre = HEAP32[$$phi$trans$insert$i>>2]|0;
        $$0$i30 = $79;$84 = $59;$87 = $$pre1$i27$pre;$91 = $79;
       }
      }
     } while(0);
     if ((label|0) == 30) {
      label = 0;
      $80 = (($81) + -32)|0;
      $82 = HEAP32[$$phi$trans$insert$i>>2]|0;
      HEAP32[$p>>2] = $82;
      HEAP32[$$phi$trans$insert$i>>2] = 0;
      $$0$i30 = $80;$84 = $82;$87 = 0;$91 = $81;
     }
     $83 = $84 >>> $$0$i30;
     $85 = (32 - ($$0$i30))|0;
     $86 = $87 << $85;
     $88 = $86 | $83;
     HEAP32[$p>>2] = $88;
     $89 = $87 >>> $$0$i30;
     HEAP32[$$phi$trans$insert$i>>2] = $89;
     $90 = (($91) + ($pshift$254))|0;
     $$pre = (($head$153) + ($14)|0);
     $head$1$be = $$pre;$pshift$2$be = $90;
    } else {
     $$pre$i20 = HEAP32[$$phi$trans$insert$i>>2]|0;
     $92 = $$pre$i20 << 2;
     $93 = $59 >>> 30;
     $94 = $93 | $92;
     $95 = (($pshift$254) + -2)|0;
     $96 = $59 << 1;
     $97 = $96 & 2147483646;
     $98 = $93 << 31;
     $99 = $97 | $98;
     $100 = $99 ^ 3;
     HEAP32[$p>>2] = $100;
     $101 = $94 >>> 1;
     HEAP32[$$phi$trans$insert$i>>2] = $101;
     $102 = (($lp) + ($95<<2)|0);
     $103 = HEAP32[$102>>2]|0;
     $sum = (($103) + ($width))|0;
     $$sum2 = (0 - ($sum))|0;
     $104 = (($head$153) + ($$sum2)|0);
     $105 = (($pshift$254) + -1)|0;
     _trinkle($104,$width,$cmp,$p,$105,1,$lp);
     $$pre$i8 = HEAP32[$$phi$trans$insert$i>>2]|0;
     $$pre1$i9 = HEAP32[$p>>2]|0;
     $106 = $$pre$i8 << 1;
     $107 = $$pre1$i9 >>> 31;
     $108 = $107 | $106;
     HEAP32[$$phi$trans$insert$i>>2] = $108;
     $109 = $$pre1$i9 << 1;
     $110 = $109 | 1;
     HEAP32[$p>>2] = $110;
     $111 = (($head$153) + ($14)|0);
     _trinkle($111,$width,$cmp,$p,$95,1,$lp);
     $head$1$be = $111;$pshift$2$be = $95;
    }
    $112 = ($pshift$2$be|0)==(1);
    $113 = HEAP32[$p>>2]|0;
    $114 = ($113|0)==(1);
    $or$cond = $112 & $114;
    $115 = HEAP32[$51>>2]|0;
    $116 = ($115|0)==(0);
    $or$cond48 = $or$cond & $116;
    if ($or$cond48) {
     break;
    } else {
     $59 = $113;$head$153 = $head$1$be;$pshift$254 = $pshift$2$be;
    }
   }
  }
 }
 STACKTOP = sp;return;
}
function _memcmp($vl,$vr,$n) {
 $vl = $vl|0;
 $vr = $vr|0;
 $n = $n|0;
 var $$03 = 0, $$lcssa = 0, $$lcssa19 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $l$04 = 0, $r$05 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 L1: do {
  if ($0) {
   $11 = 0;
  } else {
   $$03 = $n;$l$04 = $vl;$r$05 = $vr;
   while(1) {
    $1 = HEAP8[$l$04>>0]|0;
    $2 = HEAP8[$r$05>>0]|0;
    $3 = ($1<<24>>24)==($2<<24>>24);
    if (!($3)) {
     $$lcssa = $1;$$lcssa19 = $2;
     break;
    }
    $4 = (($$03) + -1)|0;
    $5 = ((($l$04)) + 1|0);
    $6 = ((($r$05)) + 1|0);
    $7 = ($4|0)==(0);
    if ($7) {
     $11 = 0;
     break L1;
    } else {
     $$03 = $4;$l$04 = $5;$r$05 = $6;
    }
   }
   $8 = $$lcssa&255;
   $9 = $$lcssa19&255;
   $10 = (($8) - ($9))|0;
   $11 = $10;
  }
 } while(0);
 return ($11|0);
}
function ___stpcpy($d,$s) {
 $d = $d|0;
 $s = $s|0;
 var $$0$lcssa = 0, $$01$lcssa = 0, $$0115 = 0, $$016 = 0, $$03 = 0, $$1$ph = 0, $$12$ph = 0, $$128 = 0, $$19 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0;
 var $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $wd$0$lcssa = 0, $wd$010 = 0, $ws$0$lcssa = 0, $ws$011 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $s;
 $1 = $d;
 $2 = $0 ^ $1;
 $3 = $2 & 3;
 $4 = ($3|0)==(0);
 L1: do {
  if ($4) {
   $5 = $0 & 3;
   $6 = ($5|0)==(0);
   if ($6) {
    $$0$lcssa = $s;$$01$lcssa = $d;
   } else {
    $$0115 = $d;$$016 = $s;
    while(1) {
     $7 = HEAP8[$$016>>0]|0;
     HEAP8[$$0115>>0] = $7;
     $8 = ($7<<24>>24)==(0);
     if ($8) {
      $$03 = $$0115;
      break L1;
     }
     $9 = ((($$016)) + 1|0);
     $10 = ((($$0115)) + 1|0);
     $11 = $9;
     $12 = $11 & 3;
     $13 = ($12|0)==(0);
     if ($13) {
      $$0$lcssa = $9;$$01$lcssa = $10;
      break;
     } else {
      $$0115 = $10;$$016 = $9;
     }
    }
   }
   $14 = HEAP32[$$0$lcssa>>2]|0;
   $15 = (($14) + -16843009)|0;
   $16 = $14 & -2139062144;
   $17 = $16 ^ -2139062144;
   $18 = $17 & $15;
   $19 = ($18|0)==(0);
   if ($19) {
    $22 = $14;$wd$010 = $$01$lcssa;$ws$011 = $$0$lcssa;
    while(1) {
     $20 = ((($ws$011)) + 4|0);
     $21 = ((($wd$010)) + 4|0);
     HEAP32[$wd$010>>2] = $22;
     $23 = HEAP32[$20>>2]|0;
     $24 = (($23) + -16843009)|0;
     $25 = $23 & -2139062144;
     $26 = $25 ^ -2139062144;
     $27 = $26 & $24;
     $28 = ($27|0)==(0);
     if ($28) {
      $22 = $23;$wd$010 = $21;$ws$011 = $20;
     } else {
      $wd$0$lcssa = $21;$ws$0$lcssa = $20;
      break;
     }
    }
   } else {
    $wd$0$lcssa = $$01$lcssa;$ws$0$lcssa = $$0$lcssa;
   }
   $$1$ph = $ws$0$lcssa;$$12$ph = $wd$0$lcssa;
   label = 8;
  } else {
   $$1$ph = $s;$$12$ph = $d;
   label = 8;
  }
 } while(0);
 if ((label|0) == 8) {
  $29 = HEAP8[$$1$ph>>0]|0;
  HEAP8[$$12$ph>>0] = $29;
  $30 = ($29<<24>>24)==(0);
  if ($30) {
   $$03 = $$12$ph;
  } else {
   $$128 = $$12$ph;$$19 = $$1$ph;
   while(1) {
    $31 = ((($$19)) + 1|0);
    $32 = ((($$128)) + 1|0);
    $33 = HEAP8[$31>>0]|0;
    HEAP8[$32>>0] = $33;
    $34 = ($33<<24>>24)==(0);
    if ($34) {
     $$03 = $32;
     break;
    } else {
     $$128 = $32;$$19 = $31;
    }
   }
  }
 }
 return ($$03|0);
}
function ___stpncpy($d,$s,$n) {
 $d = $d|0;
 $s = $s|0;
 $n = $n|0;
 var $$0$lcssa = 0, $$0$lcssa60 = 0, $$031 = 0, $$04$lcssa = 0, $$04$lcssa61 = 0, $$0430 = 0, $$06$lcssa = 0, $$06$lcssa62 = 0, $$0629 = 0, $$1$lcssa = 0, $$117 = 0, $$15$ph = 0, $$1511 = 0, $$17$ph = 0, $$1710 = 0, $$2$ph = 0, $$212 = 0, $$28 = 0, $$3 = 0, $$lcssa = 0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0;
 var $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond28 = 0, $wd$0$lcssa = 0;
 var $wd$018 = 0, $ws$0$lcssa = 0, $ws$019 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $s;
 $1 = $d;
 $2 = $0 ^ $1;
 $3 = $2 & 3;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   $5 = $0 & 3;
   $6 = ($5|0)!=(0);
   $7 = ($n|0)!=(0);
   $or$cond28 = $7 & $6;
   L3: do {
    if ($or$cond28) {
     $$031 = $n;$$0430 = $s;$$0629 = $d;
     while(1) {
      $8 = HEAP8[$$0430>>0]|0;
      HEAP8[$$0629>>0] = $8;
      $9 = ($8<<24>>24)==(0);
      if ($9) {
       $$0$lcssa60 = $$031;$$04$lcssa61 = $$0430;$$06$lcssa62 = $$0629;
       break L3;
      }
      $10 = (($$031) + -1)|0;
      $11 = ((($$0430)) + 1|0);
      $12 = ((($$0629)) + 1|0);
      $13 = $11;
      $14 = $13 & 3;
      $15 = ($14|0)!=(0);
      $16 = ($10|0)!=(0);
      $or$cond = $16 & $15;
      if ($or$cond) {
       $$031 = $10;$$0430 = $11;$$0629 = $12;
      } else {
       $$0$lcssa = $10;$$04$lcssa = $11;$$06$lcssa = $12;$$lcssa = $16;
       label = 5;
       break;
      }
     }
    } else {
     $$0$lcssa = $n;$$04$lcssa = $s;$$06$lcssa = $d;$$lcssa = $7;
     label = 5;
    }
   } while(0);
   if ((label|0) == 5) {
    if ($$lcssa) {
     $$0$lcssa60 = $$0$lcssa;$$04$lcssa61 = $$04$lcssa;$$06$lcssa62 = $$06$lcssa;
    } else {
     $$28 = $$06$lcssa;$$3 = 0;
     break;
    }
   }
   $17 = HEAP8[$$04$lcssa61>>0]|0;
   $18 = ($17<<24>>24)==(0);
   if ($18) {
    $$28 = $$06$lcssa62;$$3 = $$0$lcssa60;
   } else {
    $19 = ($$0$lcssa60>>>0)>(3);
    L11: do {
     if ($19) {
      $$117 = $$0$lcssa60;$wd$018 = $$06$lcssa62;$ws$019 = $$04$lcssa61;
      while(1) {
       $20 = HEAP32[$ws$019>>2]|0;
       $21 = (($20) + -16843009)|0;
       $22 = $20 & -2139062144;
       $23 = $22 ^ -2139062144;
       $24 = $23 & $21;
       $25 = ($24|0)==(0);
       if (!($25)) {
        $$1$lcssa = $$117;$wd$0$lcssa = $wd$018;$ws$0$lcssa = $ws$019;
        break L11;
       }
       HEAP32[$wd$018>>2] = $20;
       $26 = (($$117) + -4)|0;
       $27 = ((($ws$019)) + 4|0);
       $28 = ((($wd$018)) + 4|0);
       $29 = ($26>>>0)>(3);
       if ($29) {
        $$117 = $26;$wd$018 = $28;$ws$019 = $27;
       } else {
        $$1$lcssa = $26;$wd$0$lcssa = $28;$ws$0$lcssa = $27;
        break;
       }
      }
     } else {
      $$1$lcssa = $$0$lcssa60;$wd$0$lcssa = $$06$lcssa62;$ws$0$lcssa = $$04$lcssa61;
     }
    } while(0);
    $$15$ph = $ws$0$lcssa;$$17$ph = $wd$0$lcssa;$$2$ph = $$1$lcssa;
    label = 11;
   }
  } else {
   $$15$ph = $s;$$17$ph = $d;$$2$ph = $n;
   label = 11;
  }
 } while(0);
 L16: do {
  if ((label|0) == 11) {
   $30 = ($$2$ph|0)==(0);
   if ($30) {
    $$28 = $$17$ph;$$3 = 0;
   } else {
    $$1511 = $$15$ph;$$1710 = $$17$ph;$$212 = $$2$ph;
    while(1) {
     $31 = HEAP8[$$1511>>0]|0;
     HEAP8[$$1710>>0] = $31;
     $32 = ($31<<24>>24)==(0);
     if ($32) {
      $$28 = $$1710;$$3 = $$212;
      break L16;
     }
     $33 = (($$212) + -1)|0;
     $34 = ((($$1511)) + 1|0);
     $35 = ((($$1710)) + 1|0);
     $36 = ($33|0)==(0);
     if ($36) {
      $$28 = $35;$$3 = 0;
      break;
     } else {
      $$1511 = $34;$$1710 = $35;$$212 = $33;
     }
    }
   }
  }
 } while(0);
 _memset(($$28|0),0,($$3|0))|0;
 return ($$28|0);
}
function _strcmp($l,$r) {
 $l = $l|0;
 $r = $r|0;
 var $$014 = 0, $$05 = 0, $$lcssa = 0, $$lcssa2 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $or$cond = 0, $or$cond3 = 0, label = 0;
 var sp = 0;
 sp = STACKTOP;
 $0 = HEAP8[$l>>0]|0;
 $1 = HEAP8[$r>>0]|0;
 $2 = ($0<<24>>24)!=($1<<24>>24);
 $3 = ($0<<24>>24)==(0);
 $or$cond3 = $3 | $2;
 if ($or$cond3) {
  $$lcssa = $0;$$lcssa2 = $1;
 } else {
  $$014 = $l;$$05 = $r;
  while(1) {
   $4 = ((($$014)) + 1|0);
   $5 = ((($$05)) + 1|0);
   $6 = HEAP8[$4>>0]|0;
   $7 = HEAP8[$5>>0]|0;
   $8 = ($6<<24>>24)!=($7<<24>>24);
   $9 = ($6<<24>>24)==(0);
   $or$cond = $9 | $8;
   if ($or$cond) {
    $$lcssa = $6;$$lcssa2 = $7;
    break;
   } else {
    $$014 = $4;$$05 = $5;
   }
  }
 }
 $10 = $$lcssa&255;
 $11 = $$lcssa2&255;
 $12 = (($10) - ($11))|0;
 return ($12|0);
}
function _strcpy($dest,$src) {
 $dest = $dest|0;
 $src = $src|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 (___stpcpy($dest,$src)|0);
 return ($dest|0);
}
function ___strdup($s) {
 $s = $s|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (_strlen($s)|0);
 $1 = (($0) + 1)|0;
 $2 = (_malloc($1)|0);
 $3 = ($2|0)==(0|0);
 if ($3) {
  $$0 = 0;
 } else {
  _memcpy(($2|0),($s|0),($1|0))|0;
  $$0 = $2;
 }
 return ($$0|0);
}
function _strlen($s) {
 $s = $s|0;
 var $$0 = 0, $$01$lcssa = 0, $$014 = 0, $$1$lcssa = 0, $$lcssa20 = 0, $$pn = 0, $$pn15 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0;
 var $2 = 0, $20 = 0, $21 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $w$0 = 0, $w$0$lcssa = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = $s;
 $1 = $0 & 3;
 $2 = ($1|0)==(0);
 L1: do {
  if ($2) {
   $$01$lcssa = $s;
   label = 4;
  } else {
   $$014 = $s;$21 = $0;
   while(1) {
    $3 = HEAP8[$$014>>0]|0;
    $4 = ($3<<24>>24)==(0);
    if ($4) {
     $$pn = $21;
     break L1;
    }
    $5 = ((($$014)) + 1|0);
    $6 = $5;
    $7 = $6 & 3;
    $8 = ($7|0)==(0);
    if ($8) {
     $$01$lcssa = $5;
     label = 4;
     break;
    } else {
     $$014 = $5;$21 = $6;
    }
   }
  }
 } while(0);
 if ((label|0) == 4) {
  $w$0 = $$01$lcssa;
  while(1) {
   $9 = HEAP32[$w$0>>2]|0;
   $10 = (($9) + -16843009)|0;
   $11 = $9 & -2139062144;
   $12 = $11 ^ -2139062144;
   $13 = $12 & $10;
   $14 = ($13|0)==(0);
   $15 = ((($w$0)) + 4|0);
   if ($14) {
    $w$0 = $15;
   } else {
    $$lcssa20 = $9;$w$0$lcssa = $w$0;
    break;
   }
  }
  $16 = $$lcssa20&255;
  $17 = ($16<<24>>24)==(0);
  if ($17) {
   $$1$lcssa = $w$0$lcssa;
  } else {
   $$pn15 = $w$0$lcssa;
   while(1) {
    $18 = ((($$pn15)) + 1|0);
    $$pre = HEAP8[$18>>0]|0;
    $19 = ($$pre<<24>>24)==(0);
    if ($19) {
     $$1$lcssa = $18;
     break;
    } else {
     $$pn15 = $18;
    }
   }
  }
  $20 = $$1$lcssa;
  $$pn = $20;
 }
 $$0 = (($$pn) - ($0))|0;
 return ($$0|0);
}
function _strncmp($_l,$_r,$n) {
 $_l = $_l|0;
 $_r = $_r|0;
 $n = $n|0;
 var $$03 = 0, $$08 = 0, $$08$in = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0;
 var $l$06 = 0, $or$cond = 0, $or$cond4 = 0, $r$0$lcssa = 0, $r$07 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n|0)==(0);
 if ($0) {
  $$03 = 0;
 } else {
  $1 = HEAP8[$_l>>0]|0;
  $2 = ($1<<24>>24)==(0);
  L3: do {
   if ($2) {
    $13 = 0;$r$0$lcssa = $_r;
   } else {
    $$08$in = $n;$7 = $1;$l$06 = $_l;$r$07 = $_r;
    while(1) {
     $$08 = (($$08$in) + -1)|0;
     $3 = HEAP8[$r$07>>0]|0;
     $4 = ($3<<24>>24)!=(0);
     $5 = ($$08|0)!=(0);
     $or$cond = $5 & $4;
     $6 = ($7<<24>>24)==($3<<24>>24);
     $or$cond4 = $6 & $or$cond;
     if (!($or$cond4)) {
      $13 = $7;$r$0$lcssa = $r$07;
      break L3;
     }
     $8 = ((($l$06)) + 1|0);
     $9 = ((($r$07)) + 1|0);
     $10 = HEAP8[$8>>0]|0;
     $11 = ($10<<24>>24)==(0);
     if ($11) {
      $13 = 0;$r$0$lcssa = $9;
      break;
     } else {
      $$08$in = $$08;$7 = $10;$l$06 = $8;$r$07 = $9;
     }
    }
   }
  } while(0);
  $12 = $13&255;
  $14 = HEAP8[$r$0$lcssa>>0]|0;
  $15 = $14&255;
  $16 = (($12) - ($15))|0;
  $$03 = $16;
 }
 return ($$03|0);
}
function _strncpy($d,$s,$n) {
 $d = $d|0;
 $s = $s|0;
 $n = $n|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 (___stpncpy($d,$s,$n)|0);
 return ($d|0);
}
function _access($filename,$amode) {
 $filename = $filename|0;
 $amode = $amode|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $filename;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $amode;
 $0 = (___syscall33(33,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _close($fd) {
 $fd = $fd|0;
 var $$ = 0, $0 = 0, $1 = 0, $2 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $0 = (___syscall6(6,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-4);
 $$ = $1 ? -115 : $0;
 $2 = (___syscall_ret($$)|0);
 STACKTOP = sp;return ($2|0);
}
function _fchown($fd,$uid,$gid) {
 $fd = $fd|0;
 $uid = $uid|0;
 $gid = $gid|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $buf = 0, $vararg_buffer = 0, $vararg_buffer3 = 0, $vararg_buffer7 = 0, $vararg_ptr1 = 0, $vararg_ptr10 = 0, $vararg_ptr11 = 0, $vararg_ptr2 = 0, $vararg_ptr6 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 64|0;
 $vararg_buffer7 = sp + 24|0;
 $vararg_buffer3 = sp + 16|0;
 $vararg_buffer = sp;
 $buf = sp + 36|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $uid;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $gid;
 $0 = (___syscall207(207,($vararg_buffer|0))|0);
 $1 = ($0|0)==(-9);
 if ($1) {
  HEAP32[$vararg_buffer3>>2] = $fd;
  $vararg_ptr6 = ((($vararg_buffer3)) + 4|0);
  HEAP32[$vararg_ptr6>>2] = 1;
  $2 = (___syscall221(221,($vararg_buffer3|0))|0);
  $3 = ($2|0)<(0);
  if ($3) {
   label = 3;
  } else {
   ___procfdname($buf,$fd);
   HEAP32[$vararg_buffer7>>2] = $buf;
   $vararg_ptr10 = ((($vararg_buffer7)) + 4|0);
   HEAP32[$vararg_ptr10>>2] = $uid;
   $vararg_ptr11 = ((($vararg_buffer7)) + 8|0);
   HEAP32[$vararg_ptr11>>2] = $gid;
   $5 = (___syscall212(212,($vararg_buffer7|0))|0);
   $6 = (___syscall_ret($5)|0);
   $$0 = $6;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $4 = (___syscall_ret($0)|0);
  $$0 = $4;
 }
 STACKTOP = sp;return ($$0|0);
}
function _fsync($fd) {
 $fd = $fd|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $0 = (___syscall118(118,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _ftruncate($fd,$length) {
 $fd = $fd|0;
 $length = $length|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = ($length|0)<(0);
 $1 = $0 << 31 >> 31;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $length;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $1;
 $2 = (___syscall194(194,($vararg_buffer|0))|0);
 $3 = (___syscall_ret($2)|0);
 STACKTOP = sp;return ($3|0);
}
function _getcwd($buf,$size) {
 $buf = $buf|0;
 $size = $size|0;
 var $$0 = 0, $$01 = 0, $$02 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $tmp = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 4112|0;
 $vararg_buffer = sp;
 $tmp = sp + 8|0;
 $0 = ($buf|0)==(0|0);
 if ($0) {
  $$0 = 4096;$$01 = $tmp;
  label = 4;
 } else {
  $1 = ($size|0)==(0);
  if ($1) {
   $2 = (___errno_location()|0);
   HEAP32[$2>>2] = 22;
   $$02 = 0;
  } else {
   $$0 = $size;$$01 = $buf;
   label = 4;
  }
 }
 if ((label|0) == 4) {
  HEAP32[$vararg_buffer>>2] = $$01;
  $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
  HEAP32[$vararg_ptr1>>2] = $$0;
  $3 = (___syscall183(183,($vararg_buffer|0))|0);
  $4 = (___syscall_ret($3)|0);
  $5 = ($4|0)<(0);
  if ($5) {
   $$02 = 0;
  } else {
   $6 = ($$01|0)==($tmp|0);
   if ($6) {
    $7 = (___strdup($tmp)|0);
    $$02 = $7;
   } else {
    $$02 = $$01;
   }
  }
 }
 STACKTOP = sp;return ($$02|0);
}
function _geteuid() {
 var $0 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = (___syscall201(201,($vararg_buffer|0))|0);
 STACKTOP = sp;return ($0|0);
}
function _getpid() {
 var $0 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 $0 = (___syscall20(20,($vararg_buffer|0))|0);
 STACKTOP = sp;return ($0|0);
}
function _lseek($fd,$offset,$whence) {
 $fd = $fd|0;
 $offset = $offset|0;
 $whence = $whence|0;
 var $0 = 0, $1 = 0, $2 = 0, $3 = 0, $4 = 0, $result = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, $vararg_ptr3 = 0, $vararg_ptr4 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 32|0;
 $vararg_buffer = sp;
 $result = sp + 20|0;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = 0;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $offset;
 $vararg_ptr3 = ((($vararg_buffer)) + 12|0);
 HEAP32[$vararg_ptr3>>2] = $result;
 $vararg_ptr4 = ((($vararg_buffer)) + 16|0);
 HEAP32[$vararg_ptr4>>2] = $whence;
 $0 = (___syscall140(140,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 $2 = ($1|0)!=(0);
 $3 = HEAP32[$result>>2]|0;
 $4 = $2 ? -1 : $3;
 STACKTOP = sp;return ($4|0);
}
function _read($fd,$buf,$count) {
 $fd = $fd|0;
 $buf = $buf|0;
 $count = $count|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $count;
 $0 = (___syscall3(3,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _readlink($path,$buf,$bufsize) {
 $path = $path|0;
 $buf = $buf|0;
 $bufsize = $bufsize|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $bufsize;
 $0 = (___syscall85(85,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _rmdir($path) {
 $path = $path|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $0 = (___syscall40(40,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _sleep($seconds) {
 $seconds = $seconds|0;
 var $$0 = 0, $0 = 0, $1 = 0, $2 = 0, $3 = 0, $tv = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $tv = sp;
 HEAP32[$tv>>2] = $seconds;
 $0 = ((($tv)) + 4|0);
 HEAP32[$0>>2] = 0;
 $1 = (_nanosleep(($tv|0),($tv|0))|0);
 $2 = ($1|0)==(0);
 $3 = HEAP32[$tv>>2]|0;
 $$0 = $2 ? 0 : $3;
 STACKTOP = sp;return ($$0|0);
}
function _unlink($path) {
 $path = $path|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $path;
 $0 = (___syscall10(10,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _write($fd,$buf,$count) {
 $fd = $fd|0;
 $buf = $buf|0;
 $count = $count|0;
 var $0 = 0, $1 = 0, $vararg_buffer = 0, $vararg_ptr1 = 0, $vararg_ptr2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 16|0;
 $vararg_buffer = sp;
 HEAP32[$vararg_buffer>>2] = $fd;
 $vararg_ptr1 = ((($vararg_buffer)) + 4|0);
 HEAP32[$vararg_ptr1>>2] = $buf;
 $vararg_ptr2 = ((($vararg_buffer)) + 8|0);
 HEAP32[$vararg_ptr2>>2] = $count;
 $0 = (___syscall4(4,($vararg_buffer|0))|0);
 $1 = (___syscall_ret($0)|0);
 STACKTOP = sp;return ($1|0);
}
function _cleanup392($p) {
 $p = $p|0;
 var $0 = 0, $1 = 0, $2 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($p)) + 68|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ($1|0)==(0);
 if ($2) {
  ___unlockfile($p);
 }
 return;
}
function ___fflush_unlocked($f) {
 $f = $f|0;
 var $$0 = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0;
 var $9 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($f)) + 20|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = ((($f)) + 28|0);
 $3 = HEAP32[$2>>2]|0;
 $4 = ($1>>>0)>($3>>>0);
 if ($4) {
  $5 = ((($f)) + 36|0);
  $6 = HEAP32[$5>>2]|0;
  (FUNCTION_TABLE_iiii[$6 & 255]($f,0,0)|0);
  $7 = HEAP32[$0>>2]|0;
  $8 = ($7|0)==(0|0);
  if ($8) {
   $$0 = -1;
  } else {
   label = 3;
  }
 } else {
  label = 3;
 }
 if ((label|0) == 3) {
  $9 = ((($f)) + 4|0);
  $10 = HEAP32[$9>>2]|0;
  $11 = ((($f)) + 8|0);
  $12 = HEAP32[$11>>2]|0;
  $13 = ($10>>>0)<($12>>>0);
  if ($13) {
   $14 = ((($f)) + 40|0);
   $15 = HEAP32[$14>>2]|0;
   $16 = $10;
   $17 = $12;
   $18 = (($16) - ($17))|0;
   (FUNCTION_TABLE_iiii[$15 & 255]($f,$18,1)|0);
  }
  $19 = ((($f)) + 16|0);
  HEAP32[$19>>2] = 0;
  HEAP32[$2>>2] = 0;
  HEAP32[$0>>2] = 0;
  HEAP32[$11>>2] = 0;
  HEAP32[$9>>2] = 0;
  $$0 = 0;
 }
 return ($$0|0);
}
function _sift($head,$width,$cmp,$pshift,$lp) {
 $head = $head|0;
 $width = $width|0;
 $cmp = $cmp|0;
 $pshift = $pshift|0;
 $lp = $lp|0;
 var $$0$be = 0, $$01$be = 0, $$012 = 0, $$03 = 0, $$pre = 0, $$sum = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0;
 var $6 = 0, $7 = 0, $8 = 0, $9 = 0, $ar = 0, $i$0$lcssa = 0, $i$04 = 0, $sum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $ar = sp;
 HEAP32[$ar>>2] = $head;
 $0 = ($pshift|0)>(1);
 L1: do {
  if ($0) {
   $1 = (0 - ($width))|0;
   $$012 = $pshift;$$03 = $head;$7 = $head;$i$04 = 1;
   while(1) {
    $2 = (($$03) + ($1)|0);
    $3 = (($$012) + -2)|0;
    $4 = (($lp) + ($3<<2)|0);
    $5 = HEAP32[$4>>2]|0;
    $sum = (($5) + ($width))|0;
    $$sum = (0 - ($sum))|0;
    $6 = (($$03) + ($$sum)|0);
    $8 = (FUNCTION_TABLE_iii[$cmp & 255]($7,$6)|0);
    $9 = ($8|0)>(-1);
    if ($9) {
     $10 = (FUNCTION_TABLE_iii[$cmp & 255]($7,$2)|0);
     $11 = ($10|0)>(-1);
     if ($11) {
      $i$0$lcssa = $i$04;
      break L1;
     }
    }
    $12 = (FUNCTION_TABLE_iii[$cmp & 255]($6,$2)|0);
    $13 = ($12|0)>(-1);
    $14 = (($i$04) + 1)|0;
    $15 = (($ar) + ($i$04<<2)|0);
    if ($13) {
     HEAP32[$15>>2] = $6;
     $16 = (($$012) + -1)|0;
     $$0$be = $6;$$01$be = $16;
    } else {
     HEAP32[$15>>2] = $2;
     $$0$be = $2;$$01$be = $3;
    }
    $17 = ($$01$be|0)>(1);
    if (!($17)) {
     $i$0$lcssa = $14;
     break L1;
    }
    $$pre = HEAP32[$ar>>2]|0;
    $$012 = $$01$be;$$03 = $$0$be;$7 = $$pre;$i$04 = $14;
   }
  } else {
   $i$0$lcssa = 1;
  }
 } while(0);
 _cycle($width,$ar,$i$0$lcssa);
 STACKTOP = sp;return;
}
function _trinkle($head,$width,$cmp,$pp,$pshift,$trusty,$lp) {
 $head = $head|0;
 $width = $width|0;
 $cmp = $cmp|0;
 $pp = $pp|0;
 $pshift = $pshift|0;
 $trusty = $trusty|0;
 $lp = $lp|0;
 var $$0$i = 0, $$0$lcssa = 0, $$0$lcssa49 = 0, $$01162 = 0, $$01162$phi = 0, $$02$i$i = 0, $$02$i3$i = 0, $$02$lcssa = 0, $$02$lcssa51 = 0, $$02964 = 0, $$03$lcssa = 0, $$03865 = 0, $$lcssa = 0, $$lcssa75 = 0, $$pre = 0, $$sum = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0;
 var $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0;
 var $30 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0;
 var $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0;
 var $67 = 0, $68 = 0, $7 = 0, $8 = 0, $9 = 0, $ar = 0, $i$0$lcssa = 0, $i$0$lcssa50 = 0, $i$01063 = 0, $nTrailingZeros$03$i$i = 0, $nTrailingZeros$03$i2$i = 0, $nTrailingZeros$03$i2$i$lcssa = 0, $or$cond = 0, $phitmp = 0, $sum = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 240|0;
 $ar = sp;
 $0 = HEAP32[$pp>>2]|0;
 $1 = ((($pp)) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 HEAP32[$ar>>2] = $head;
 $3 = (0 - ($width))|0;
 $4 = ($0|0)!=(1);
 $5 = ($2|0)!=(0);
 $6 = $5 | $4;
 L1: do {
  if ($6) {
   $7 = (($lp) + ($pshift<<2)|0);
   $8 = HEAP32[$7>>2]|0;
   $9 = (0 - ($8))|0;
   $10 = (($head) + ($9)|0);
   $11 = (FUNCTION_TABLE_iii[$cmp & 255]($10,$head)|0);
   $12 = ($11|0)<(1);
   if ($12) {
    $$0$lcssa = $head;$$02$lcssa = $pshift;$$03$lcssa = $trusty;$i$0$lcssa = 1;
    label = 19;
   } else {
    $phitmp = ($trusty|0)==(0);
    $$01162 = $head;$$02964 = $pshift;$$03865 = $phitmp;$18 = $10;$27 = $0;$37 = $2;$i$01063 = 1;
    while(1) {
     $13 = ($$02964|0)>(1);
     $or$cond = $$03865 & $13;
     if ($or$cond) {
      $14 = (($$01162) + ($3)|0);
      $15 = (($$02964) + -2)|0;
      $16 = (($lp) + ($15<<2)|0);
      $17 = HEAP32[$16>>2]|0;
      $19 = (FUNCTION_TABLE_iii[$cmp & 255]($14,$18)|0);
      $20 = ($19|0)>(-1);
      if ($20) {
       $$0$lcssa49 = $$01162;$$02$lcssa51 = $$02964;$i$0$lcssa50 = $i$01063;
       label = 20;
       break L1;
      }
      $sum = (($17) + ($width))|0;
      $$sum = (0 - ($sum))|0;
      $21 = (($$01162) + ($$sum)|0);
      $22 = (FUNCTION_TABLE_iii[$cmp & 255]($21,$18)|0);
      $23 = ($22|0)>(-1);
      if ($23) {
       $$0$lcssa49 = $$01162;$$02$lcssa51 = $$02964;$i$0$lcssa50 = $i$01063;
       label = 20;
       break L1;
      }
     }
     $24 = (($i$01063) + 1)|0;
     $25 = (($ar) + ($i$01063<<2)|0);
     HEAP32[$25>>2] = $18;
     $26 = (($27) + -1)|0;
     $28 = ($26|0)==(0);
     do {
      if ($28) {
       $49 = 32;
       label = 16;
      } else {
       $29 = $26 & 1;
       $30 = ($29|0)==(0);
       if ($30) {
        $$02$i$i = $26;$nTrailingZeros$03$i$i = 0;
        while(1) {
         $31 = (($nTrailingZeros$03$i$i) + 1)|0;
         $32 = $$02$i$i >>> 1;
         $33 = $32 & 1;
         $34 = ($33|0)==(0);
         if ($34) {
          $$02$i$i = $32;$nTrailingZeros$03$i$i = $31;
         } else {
          $$lcssa = $31;
          break;
         }
        }
        $35 = ($$lcssa|0)==(0);
        if ($35) {
         label = 11;
        } else {
         $47 = $$lcssa;
        }
       } else {
        label = 11;
       }
       if ((label|0) == 11) {
        label = 0;
        $36 = ($37|0)==(0);
        if ($36) {
         $49 = 64;
         label = 16;
         break;
        }
        $38 = $37 & 1;
        $39 = ($38|0)==(0);
        if ($39) {
         $$02$i3$i = $37;$nTrailingZeros$03$i2$i = 0;
        } else {
         $$0$i = 0;$51 = $27;$54 = $37;$58 = 0;
         break;
        }
        while(1) {
         $40 = (($nTrailingZeros$03$i2$i) + 1)|0;
         $41 = $$02$i3$i >>> 1;
         $42 = $41 & 1;
         $43 = ($42|0)==(0);
         if ($43) {
          $$02$i3$i = $41;$nTrailingZeros$03$i2$i = $40;
         } else {
          $$lcssa75 = $40;$nTrailingZeros$03$i2$i$lcssa = $nTrailingZeros$03$i2$i;
          break;
         }
        }
        $44 = (($nTrailingZeros$03$i2$i$lcssa) + 33)|0;
        $45 = ($$lcssa75|0)==(0);
        if ($45) {
         $$0$i = 0;$51 = $27;$54 = $37;$58 = 0;
         break;
        } else {
         $47 = $44;
        }
       }
       $46 = ($47>>>0)>(31);
       if ($46) {
        $49 = $47;
        label = 16;
       } else {
        $$0$i = $47;$51 = $27;$54 = $37;$58 = $47;
       }
      }
     } while(0);
     if ((label|0) == 16) {
      label = 0;
      $48 = (($49) + -32)|0;
      $$0$i = $48;$51 = $37;$54 = 0;$58 = $49;
     }
     $50 = $51 >>> $$0$i;
     $52 = (32 - ($$0$i))|0;
     $53 = $54 << $52;
     $55 = $53 | $50;
     $56 = $54 >>> $$0$i;
     $57 = (($58) + ($$02964))|0;
     $59 = ($55|0)!=(1);
     $60 = ($56|0)!=(0);
     $61 = $60 | $59;
     if (!($61)) {
      $$0$lcssa49 = $18;$$02$lcssa51 = $57;$i$0$lcssa50 = $24;
      label = 20;
      break L1;
     }
     $$pre = HEAP32[$ar>>2]|0;
     $62 = (($lp) + ($57<<2)|0);
     $63 = HEAP32[$62>>2]|0;
     $64 = (0 - ($63))|0;
     $65 = (($18) + ($64)|0);
     $66 = (FUNCTION_TABLE_iii[$cmp & 255]($65,$$pre)|0);
     $67 = ($66|0)<(1);
     if ($67) {
      $$0$lcssa = $18;$$02$lcssa = $57;$$03$lcssa = 0;$i$0$lcssa = $24;
      label = 19;
      break;
     } else {
      $$01162$phi = $18;$$02964 = $57;$$03865 = 1;$18 = $65;$27 = $55;$37 = $56;$i$01063 = $24;$$01162 = $$01162$phi;
     }
    }
   }
  } else {
   $$0$lcssa = $head;$$02$lcssa = $pshift;$$03$lcssa = $trusty;$i$0$lcssa = 1;
   label = 19;
  }
 } while(0);
 if ((label|0) == 19) {
  $68 = ($$03$lcssa|0)==(0);
  if ($68) {
   $$0$lcssa49 = $$0$lcssa;$$02$lcssa51 = $$02$lcssa;$i$0$lcssa50 = $i$0$lcssa;
   label = 20;
  }
 }
 if ((label|0) == 20) {
  _cycle($width,$ar,$i$0$lcssa50);
  _sift($$0$lcssa49,$width,$cmp,$$02$lcssa51,$lp);
 }
 STACKTOP = sp;return;
}
function _dummy1($x) {
 $x = $x|0;
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _dummy0() {
 var label = 0, sp = 0;
 sp = STACKTOP;
 return;
}
function _cycle($width,$ar,$n) {
 $width = $width|0;
 $ar = $ar|0;
 $n = $n|0;
 var $$02 = 0, $$pre = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $exitcond = 0, $i$01 = 0;
 var $tmp = 0, label = 0, sp = 0;
 sp = STACKTOP;
 STACKTOP = STACKTOP + 256|0;
 $tmp = sp;
 $0 = ($n|0)<(2);
 L1: do {
  if (!($0)) {
   $1 = (($ar) + ($n<<2)|0);
   HEAP32[$1>>2] = $tmp;
   $2 = ($width|0)==(0);
   if (!($2)) {
    $$02 = $width;$6 = $tmp;
    while(1) {
     $3 = ($$02>>>0)>(256);
     $4 = $3 ? 256 : $$02;
     $5 = HEAP32[$ar>>2]|0;
     _memcpy(($6|0),($5|0),($4|0))|0;
     $i$01 = 0;
     while(1) {
      $7 = (($ar) + ($i$01<<2)|0);
      $8 = HEAP32[$7>>2]|0;
      $9 = (($i$01) + 1)|0;
      $10 = (($ar) + ($9<<2)|0);
      $11 = HEAP32[$10>>2]|0;
      _memcpy(($8|0),($11|0),($4|0))|0;
      $12 = HEAP32[$7>>2]|0;
      $13 = (($12) + ($4)|0);
      HEAP32[$7>>2] = $13;
      $exitcond = ($9|0)==($n|0);
      if ($exitcond) {
       break;
      } else {
       $i$01 = $9;
      }
     }
     $14 = ($$02|0)==($4|0);
     if ($14) {
      break L1;
     }
     $15 = (($$02) - ($4))|0;
     $$pre = HEAP32[$1>>2]|0;
     $$02 = $15;$6 = $$pre;
    }
   }
  }
 } while(0);
 STACKTOP = sp;return;
}
function _malloc($bytes) {
 $bytes = $bytes|0;
 var $$3$i = 0, $$lcssa = 0, $$lcssa211 = 0, $$lcssa215 = 0, $$lcssa216 = 0, $$lcssa217 = 0, $$lcssa219 = 0, $$lcssa222 = 0, $$lcssa224 = 0, $$lcssa226 = 0, $$lcssa228 = 0, $$lcssa230 = 0, $$lcssa232 = 0, $$pre = 0, $$pre$i = 0, $$pre$i$i = 0, $$pre$i22$i = 0, $$pre$i25 = 0, $$pre$phi$i$iZ2D = 0, $$pre$phi$i23$iZ2D = 0;
 var $$pre$phi$i26Z2D = 0, $$pre$phi$iZ2D = 0, $$pre$phi58$i$iZ2D = 0, $$pre$phiZ2D = 0, $$pre105 = 0, $$pre106 = 0, $$pre14$i$i = 0, $$pre43$i = 0, $$pre56$i$i = 0, $$pre57$i$i = 0, $$pre8$i = 0, $$rsize$0$i = 0, $$rsize$3$i = 0, $$sum = 0, $$sum$i$i = 0, $$sum$i$i$i = 0, $$sum$i13$i = 0, $$sum$i14$i = 0, $$sum$i17$i = 0, $$sum$i19$i = 0;
 var $$sum$i2334 = 0, $$sum$i32 = 0, $$sum$i35 = 0, $$sum1 = 0, $$sum1$i = 0, $$sum1$i$i = 0, $$sum1$i15$i = 0, $$sum1$i20$i = 0, $$sum1$i24 = 0, $$sum10 = 0, $$sum10$i = 0, $$sum10$i$i = 0, $$sum11$i = 0, $$sum11$i$i = 0, $$sum1112 = 0, $$sum112$i = 0, $$sum113$i = 0, $$sum114$i = 0, $$sum115$i = 0, $$sum116$i = 0;
 var $$sum117$i = 0, $$sum118$i = 0, $$sum119$i = 0, $$sum12$i = 0, $$sum12$i$i = 0, $$sum120$i = 0, $$sum121$i = 0, $$sum122$i = 0, $$sum123$i = 0, $$sum124$i = 0, $$sum125$i = 0, $$sum13$i = 0, $$sum13$i$i = 0, $$sum14$i$i = 0, $$sum15$i = 0, $$sum15$i$i = 0, $$sum16$i = 0, $$sum16$i$i = 0, $$sum17$i = 0, $$sum17$i$i = 0;
 var $$sum18$i = 0, $$sum1819$i$i = 0, $$sum2 = 0, $$sum2$i = 0, $$sum2$i$i = 0, $$sum2$i$i$i = 0, $$sum2$i16$i = 0, $$sum2$i18$i = 0, $$sum2$i21$i = 0, $$sum20$i$i = 0, $$sum21$i$i = 0, $$sum22$i$i = 0, $$sum23$i$i = 0, $$sum24$i$i = 0, $$sum25$i$i = 0, $$sum27$i$i = 0, $$sum28$i$i = 0, $$sum29$i$i = 0, $$sum3$i = 0, $$sum3$i27 = 0;
 var $$sum30$i$i = 0, $$sum3132$i$i = 0, $$sum34$i$i = 0, $$sum3536$i$i = 0, $$sum3738$i$i = 0, $$sum39$i$i = 0, $$sum4 = 0, $$sum4$i = 0, $$sum4$i$i = 0, $$sum4$i28 = 0, $$sum40$i$i = 0, $$sum41$i$i = 0, $$sum42$i$i = 0, $$sum5$i = 0, $$sum5$i$i = 0, $$sum56 = 0, $$sum6$i = 0, $$sum67$i$i = 0, $$sum7$i = 0, $$sum8$i = 0;
 var $$sum9 = 0, $$sum9$i = 0, $$sum9$i$i = 0, $$tsize$1$i = 0, $$v$0$i = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $1000 = 0, $1001 = 0, $1002 = 0, $1003 = 0, $1004 = 0, $1005 = 0, $1006 = 0, $1007 = 0, $1008 = 0, $1009 = 0, $101 = 0;
 var $1010 = 0, $1011 = 0, $1012 = 0, $1013 = 0, $1014 = 0, $1015 = 0, $1016 = 0, $1017 = 0, $1018 = 0, $1019 = 0, $102 = 0, $1020 = 0, $1021 = 0, $1022 = 0, $1023 = 0, $1024 = 0, $1025 = 0, $1026 = 0, $1027 = 0, $1028 = 0;
 var $1029 = 0, $103 = 0, $1030 = 0, $1031 = 0, $1032 = 0, $1033 = 0, $1034 = 0, $1035 = 0, $1036 = 0, $1037 = 0, $1038 = 0, $1039 = 0, $104 = 0, $1040 = 0, $1041 = 0, $1042 = 0, $1043 = 0, $1044 = 0, $1045 = 0, $1046 = 0;
 var $1047 = 0, $1048 = 0, $1049 = 0, $105 = 0, $1050 = 0, $1051 = 0, $1052 = 0, $1053 = 0, $1054 = 0, $1055 = 0, $1056 = 0, $1057 = 0, $1058 = 0, $1059 = 0, $106 = 0, $1060 = 0, $1061 = 0, $1062 = 0, $1063 = 0, $1064 = 0;
 var $1065 = 0, $1066 = 0, $1067 = 0, $1068 = 0, $1069 = 0, $107 = 0, $1070 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0;
 var $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0;
 var $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0;
 var $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0;
 var $174 = 0, $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0;
 var $192 = 0, $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0;
 var $21 = 0, $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0;
 var $228 = 0, $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0;
 var $246 = 0, $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0;
 var $264 = 0, $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0;
 var $282 = 0, $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0;
 var $30 = 0, $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0;
 var $318 = 0, $319 = 0, $32 = 0, $320 = 0, $321 = 0, $322 = 0, $323 = 0, $324 = 0, $325 = 0, $326 = 0, $327 = 0, $328 = 0, $329 = 0, $33 = 0, $330 = 0, $331 = 0, $332 = 0, $333 = 0, $334 = 0, $335 = 0;
 var $336 = 0, $337 = 0, $338 = 0, $339 = 0, $34 = 0, $340 = 0, $341 = 0, $342 = 0, $343 = 0, $344 = 0, $345 = 0, $346 = 0, $347 = 0, $348 = 0, $349 = 0, $35 = 0, $350 = 0, $351 = 0, $352 = 0, $353 = 0;
 var $354 = 0, $355 = 0, $356 = 0, $357 = 0, $358 = 0, $359 = 0, $36 = 0, $360 = 0, $361 = 0, $362 = 0, $363 = 0, $364 = 0, $365 = 0, $366 = 0, $367 = 0, $368 = 0, $369 = 0, $37 = 0, $370 = 0, $371 = 0;
 var $372 = 0, $373 = 0, $374 = 0, $375 = 0, $376 = 0, $377 = 0, $378 = 0, $379 = 0, $38 = 0, $380 = 0, $381 = 0, $382 = 0, $383 = 0, $384 = 0, $385 = 0, $386 = 0, $387 = 0, $388 = 0, $389 = 0, $39 = 0;
 var $390 = 0, $391 = 0, $392 = 0, $393 = 0, $394 = 0, $395 = 0, $396 = 0, $397 = 0, $398 = 0, $399 = 0, $4 = 0, $40 = 0, $400 = 0, $401 = 0, $402 = 0, $403 = 0, $404 = 0, $405 = 0, $406 = 0, $407 = 0;
 var $408 = 0, $409 = 0, $41 = 0, $410 = 0, $411 = 0, $412 = 0, $413 = 0, $414 = 0, $415 = 0, $416 = 0, $417 = 0, $418 = 0, $419 = 0, $42 = 0, $420 = 0, $421 = 0, $422 = 0, $423 = 0, $424 = 0, $425 = 0;
 var $426 = 0, $427 = 0, $428 = 0, $429 = 0, $43 = 0, $430 = 0, $431 = 0, $432 = 0, $433 = 0, $434 = 0, $435 = 0, $436 = 0, $437 = 0, $438 = 0, $439 = 0, $44 = 0, $440 = 0, $441 = 0, $442 = 0, $443 = 0;
 var $444 = 0, $445 = 0, $446 = 0, $447 = 0, $448 = 0, $449 = 0, $45 = 0, $450 = 0, $451 = 0, $452 = 0, $453 = 0, $454 = 0, $455 = 0, $456 = 0, $457 = 0, $458 = 0, $459 = 0, $46 = 0, $460 = 0, $461 = 0;
 var $462 = 0, $463 = 0, $464 = 0, $465 = 0, $466 = 0, $467 = 0, $468 = 0, $469 = 0, $47 = 0, $470 = 0, $471 = 0, $472 = 0, $473 = 0, $474 = 0, $475 = 0, $476 = 0, $477 = 0, $478 = 0, $479 = 0, $48 = 0;
 var $480 = 0, $481 = 0, $482 = 0, $483 = 0, $484 = 0, $485 = 0, $486 = 0, $487 = 0, $488 = 0, $489 = 0, $49 = 0, $490 = 0, $491 = 0, $492 = 0, $493 = 0, $494 = 0, $495 = 0, $496 = 0, $497 = 0, $498 = 0;
 var $499 = 0, $5 = 0, $50 = 0, $500 = 0, $501 = 0, $502 = 0, $503 = 0, $504 = 0, $505 = 0, $506 = 0, $507 = 0, $508 = 0, $509 = 0, $51 = 0, $510 = 0, $511 = 0, $512 = 0, $513 = 0, $514 = 0, $515 = 0;
 var $516 = 0, $517 = 0, $518 = 0, $519 = 0, $52 = 0, $520 = 0, $521 = 0, $522 = 0, $523 = 0, $524 = 0, $525 = 0, $526 = 0, $527 = 0, $528 = 0, $529 = 0, $53 = 0, $530 = 0, $531 = 0, $532 = 0, $533 = 0;
 var $534 = 0, $535 = 0, $536 = 0, $537 = 0, $538 = 0, $539 = 0, $54 = 0, $540 = 0, $541 = 0, $542 = 0, $543 = 0, $544 = 0, $545 = 0, $546 = 0, $547 = 0, $548 = 0, $549 = 0, $55 = 0, $550 = 0, $551 = 0;
 var $552 = 0, $553 = 0, $554 = 0, $555 = 0, $556 = 0, $557 = 0, $558 = 0, $559 = 0, $56 = 0, $560 = 0, $561 = 0, $562 = 0, $563 = 0, $564 = 0, $565 = 0, $566 = 0, $567 = 0, $568 = 0, $569 = 0, $57 = 0;
 var $570 = 0, $571 = 0, $572 = 0, $573 = 0, $574 = 0, $575 = 0, $576 = 0, $577 = 0, $578 = 0, $579 = 0, $58 = 0, $580 = 0, $581 = 0, $582 = 0, $583 = 0, $584 = 0, $585 = 0, $586 = 0, $587 = 0, $588 = 0;
 var $589 = 0, $59 = 0, $590 = 0, $591 = 0, $592 = 0, $593 = 0, $594 = 0, $595 = 0, $596 = 0, $597 = 0, $598 = 0, $599 = 0, $6 = 0, $60 = 0, $600 = 0, $601 = 0, $602 = 0, $603 = 0, $604 = 0, $605 = 0;
 var $606 = 0, $607 = 0, $608 = 0, $609 = 0, $61 = 0, $610 = 0, $611 = 0, $612 = 0, $613 = 0, $614 = 0, $615 = 0, $616 = 0, $617 = 0, $618 = 0, $619 = 0, $62 = 0, $620 = 0, $621 = 0, $622 = 0, $623 = 0;
 var $624 = 0, $625 = 0, $626 = 0, $627 = 0, $628 = 0, $629 = 0, $63 = 0, $630 = 0, $631 = 0, $632 = 0, $633 = 0, $634 = 0, $635 = 0, $636 = 0, $637 = 0, $638 = 0, $639 = 0, $64 = 0, $640 = 0, $641 = 0;
 var $642 = 0, $643 = 0, $644 = 0, $645 = 0, $646 = 0, $647 = 0, $648 = 0, $649 = 0, $65 = 0, $650 = 0, $651 = 0, $652 = 0, $653 = 0, $654 = 0, $655 = 0, $656 = 0, $657 = 0, $658 = 0, $659 = 0, $66 = 0;
 var $660 = 0, $661 = 0, $662 = 0, $663 = 0, $664 = 0, $665 = 0, $666 = 0, $667 = 0, $668 = 0, $669 = 0, $67 = 0, $670 = 0, $671 = 0, $672 = 0, $673 = 0, $674 = 0, $675 = 0, $676 = 0, $677 = 0, $678 = 0;
 var $679 = 0, $68 = 0, $680 = 0, $681 = 0, $682 = 0, $683 = 0, $684 = 0, $685 = 0, $686 = 0, $687 = 0, $688 = 0, $689 = 0, $69 = 0, $690 = 0, $691 = 0, $692 = 0, $693 = 0, $694 = 0, $695 = 0, $696 = 0;
 var $697 = 0, $698 = 0, $699 = 0, $7 = 0, $70 = 0, $700 = 0, $701 = 0, $702 = 0, $703 = 0, $704 = 0, $705 = 0, $706 = 0, $707 = 0, $708 = 0, $709 = 0, $71 = 0, $710 = 0, $711 = 0, $712 = 0, $713 = 0;
 var $714 = 0, $715 = 0, $716 = 0, $717 = 0, $718 = 0, $719 = 0, $72 = 0, $720 = 0, $721 = 0, $722 = 0, $723 = 0, $724 = 0, $725 = 0, $726 = 0, $727 = 0, $728 = 0, $729 = 0, $73 = 0, $730 = 0, $731 = 0;
 var $732 = 0, $733 = 0, $734 = 0, $735 = 0, $736 = 0, $737 = 0, $738 = 0, $739 = 0, $74 = 0, $740 = 0, $741 = 0, $742 = 0, $743 = 0, $744 = 0, $745 = 0, $746 = 0, $747 = 0, $748 = 0, $749 = 0, $75 = 0;
 var $750 = 0, $751 = 0, $752 = 0, $753 = 0, $754 = 0, $755 = 0, $756 = 0, $757 = 0, $758 = 0, $759 = 0, $76 = 0, $760 = 0, $761 = 0, $762 = 0, $763 = 0, $764 = 0, $765 = 0, $766 = 0, $767 = 0, $768 = 0;
 var $769 = 0, $77 = 0, $770 = 0, $771 = 0, $772 = 0, $773 = 0, $774 = 0, $775 = 0, $776 = 0, $777 = 0, $778 = 0, $779 = 0, $78 = 0, $780 = 0, $781 = 0, $782 = 0, $783 = 0, $784 = 0, $785 = 0, $786 = 0;
 var $787 = 0, $788 = 0, $789 = 0, $79 = 0, $790 = 0, $791 = 0, $792 = 0, $793 = 0, $794 = 0, $795 = 0, $796 = 0, $797 = 0, $798 = 0, $799 = 0, $8 = 0, $80 = 0, $800 = 0, $801 = 0, $802 = 0, $803 = 0;
 var $804 = 0, $805 = 0, $806 = 0, $807 = 0, $808 = 0, $809 = 0, $81 = 0, $810 = 0, $811 = 0, $812 = 0, $813 = 0, $814 = 0, $815 = 0, $816 = 0, $817 = 0, $818 = 0, $819 = 0, $82 = 0, $820 = 0, $821 = 0;
 var $822 = 0, $823 = 0, $824 = 0, $825 = 0, $826 = 0, $827 = 0, $828 = 0, $829 = 0, $83 = 0, $830 = 0, $831 = 0, $832 = 0, $833 = 0, $834 = 0, $835 = 0, $836 = 0, $837 = 0, $838 = 0, $839 = 0, $84 = 0;
 var $840 = 0, $841 = 0, $842 = 0, $843 = 0, $844 = 0, $845 = 0, $846 = 0, $847 = 0, $848 = 0, $849 = 0, $85 = 0, $850 = 0, $851 = 0, $852 = 0, $853 = 0, $854 = 0, $855 = 0, $856 = 0, $857 = 0, $858 = 0;
 var $859 = 0, $86 = 0, $860 = 0, $861 = 0, $862 = 0, $863 = 0, $864 = 0, $865 = 0, $866 = 0, $867 = 0, $868 = 0, $869 = 0, $87 = 0, $870 = 0, $871 = 0, $872 = 0, $873 = 0, $874 = 0, $875 = 0, $876 = 0;
 var $877 = 0, $878 = 0, $879 = 0, $88 = 0, $880 = 0, $881 = 0, $882 = 0, $883 = 0, $884 = 0, $885 = 0, $886 = 0, $887 = 0, $888 = 0, $889 = 0, $89 = 0, $890 = 0, $891 = 0, $892 = 0, $893 = 0, $894 = 0;
 var $895 = 0, $896 = 0, $897 = 0, $898 = 0, $899 = 0, $9 = 0, $90 = 0, $900 = 0, $901 = 0, $902 = 0, $903 = 0, $904 = 0, $905 = 0, $906 = 0, $907 = 0, $908 = 0, $909 = 0, $91 = 0, $910 = 0, $911 = 0;
 var $912 = 0, $913 = 0, $914 = 0, $915 = 0, $916 = 0, $917 = 0, $918 = 0, $919 = 0, $92 = 0, $920 = 0, $921 = 0, $922 = 0, $923 = 0, $924 = 0, $925 = 0, $926 = 0, $927 = 0, $928 = 0, $929 = 0, $93 = 0;
 var $930 = 0, $931 = 0, $932 = 0, $933 = 0, $934 = 0, $935 = 0, $936 = 0, $937 = 0, $938 = 0, $939 = 0, $94 = 0, $940 = 0, $941 = 0, $942 = 0, $943 = 0, $944 = 0, $945 = 0, $946 = 0, $947 = 0, $948 = 0;
 var $949 = 0, $95 = 0, $950 = 0, $951 = 0, $952 = 0, $953 = 0, $954 = 0, $955 = 0, $956 = 0, $957 = 0, $958 = 0, $959 = 0, $96 = 0, $960 = 0, $961 = 0, $962 = 0, $963 = 0, $964 = 0, $965 = 0, $966 = 0;
 var $967 = 0, $968 = 0, $969 = 0, $97 = 0, $970 = 0, $971 = 0, $972 = 0, $973 = 0, $974 = 0, $975 = 0, $976 = 0, $977 = 0, $978 = 0, $979 = 0, $98 = 0, $980 = 0, $981 = 0, $982 = 0, $983 = 0, $984 = 0;
 var $985 = 0, $986 = 0, $987 = 0, $988 = 0, $989 = 0, $99 = 0, $990 = 0, $991 = 0, $992 = 0, $993 = 0, $994 = 0, $995 = 0, $996 = 0, $997 = 0, $998 = 0, $999 = 0, $F$0$i$i = 0, $F1$0$i = 0, $F4$0 = 0, $F4$0$i$i = 0;
 var $F5$0$i = 0, $I1$0$i$i = 0, $I7$0$i = 0, $I7$0$i$i = 0, $K12$029$i = 0, $K2$07$i$i = 0, $K8$051$i$i = 0, $R$0$i = 0, $R$0$i$i = 0, $R$0$i$i$lcssa = 0, $R$0$i$lcssa = 0, $R$0$i18 = 0, $R$0$i18$lcssa = 0, $R$1$i = 0, $R$1$i$i = 0, $R$1$i20 = 0, $RP$0$i = 0, $RP$0$i$i = 0, $RP$0$i$i$lcssa = 0, $RP$0$i$lcssa = 0;
 var $RP$0$i17 = 0, $RP$0$i17$lcssa = 0, $T$0$lcssa$i = 0, $T$0$lcssa$i$i = 0, $T$0$lcssa$i25$i = 0, $T$028$i = 0, $T$028$i$lcssa = 0, $T$050$i$i = 0, $T$050$i$i$lcssa = 0, $T$06$i$i = 0, $T$06$i$i$lcssa = 0, $br$0$ph$i = 0, $cond$i = 0, $cond$i$i = 0, $cond$i21 = 0, $exitcond$i$i = 0, $i$02$i$i = 0, $idx$0$i = 0, $mem$0 = 0, $nb$0 = 0;
 var $not$$i = 0, $not$$i$i = 0, $not$$i26$i = 0, $oldfirst$0$i$i = 0, $or$cond$i = 0, $or$cond$i30 = 0, $or$cond1$i = 0, $or$cond19$i = 0, $or$cond2$i = 0, $or$cond3$i = 0, $or$cond5$i = 0, $or$cond57$i = 0, $or$cond6$i = 0, $or$cond8$i = 0, $or$cond9$i = 0, $qsize$0$i$i = 0, $rsize$0$i = 0, $rsize$0$i$lcssa = 0, $rsize$0$i15 = 0, $rsize$1$i = 0;
 var $rsize$2$i = 0, $rsize$3$lcssa$i = 0, $rsize$331$i = 0, $rst$0$i = 0, $rst$1$i = 0, $sizebits$0$i = 0, $sp$0$i$i = 0, $sp$0$i$i$i = 0, $sp$084$i = 0, $sp$084$i$lcssa = 0, $sp$183$i = 0, $sp$183$i$lcssa = 0, $ssize$0$$i = 0, $ssize$0$i = 0, $ssize$1$ph$i = 0, $ssize$2$i = 0, $t$0$i = 0, $t$0$i14 = 0, $t$1$i = 0, $t$2$ph$i = 0;
 var $t$2$v$3$i = 0, $t$230$i = 0, $tbase$255$i = 0, $tsize$0$ph$i = 0, $tsize$0323944$i = 0, $tsize$1$i = 0, $tsize$254$i = 0, $v$0$i = 0, $v$0$i$lcssa = 0, $v$0$i16 = 0, $v$1$i = 0, $v$2$i = 0, $v$3$lcssa$i = 0, $v$3$ph$i = 0, $v$332$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($bytes>>>0)<(245);
 do {
  if ($0) {
   $1 = ($bytes>>>0)<(11);
   $2 = (($bytes) + 11)|0;
   $3 = $2 & -8;
   $4 = $1 ? 16 : $3;
   $5 = $4 >>> 3;
   $6 = HEAP32[9028>>2]|0;
   $7 = $6 >>> $5;
   $8 = $7 & 3;
   $9 = ($8|0)==(0);
   if (!($9)) {
    $10 = $7 & 1;
    $11 = $10 ^ 1;
    $12 = (($11) + ($5))|0;
    $13 = $12 << 1;
    $14 = (9068 + ($13<<2)|0);
    $$sum10 = (($13) + 2)|0;
    $15 = (9068 + ($$sum10<<2)|0);
    $16 = HEAP32[$15>>2]|0;
    $17 = ((($16)) + 8|0);
    $18 = HEAP32[$17>>2]|0;
    $19 = ($14|0)==($18|0);
    do {
     if ($19) {
      $20 = 1 << $12;
      $21 = $20 ^ -1;
      $22 = $6 & $21;
      HEAP32[9028>>2] = $22;
     } else {
      $23 = HEAP32[(9044)>>2]|0;
      $24 = ($18>>>0)<($23>>>0);
      if ($24) {
       _abort();
       // unreachable;
      }
      $25 = ((($18)) + 12|0);
      $26 = HEAP32[$25>>2]|0;
      $27 = ($26|0)==($16|0);
      if ($27) {
       HEAP32[$25>>2] = $14;
       HEAP32[$15>>2] = $18;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $28 = $12 << 3;
    $29 = $28 | 3;
    $30 = ((($16)) + 4|0);
    HEAP32[$30>>2] = $29;
    $$sum1112 = $28 | 4;
    $31 = (($16) + ($$sum1112)|0);
    $32 = HEAP32[$31>>2]|0;
    $33 = $32 | 1;
    HEAP32[$31>>2] = $33;
    $mem$0 = $17;
    return ($mem$0|0);
   }
   $34 = HEAP32[(9036)>>2]|0;
   $35 = ($4>>>0)>($34>>>0);
   if ($35) {
    $36 = ($7|0)==(0);
    if (!($36)) {
     $37 = $7 << $5;
     $38 = 2 << $5;
     $39 = (0 - ($38))|0;
     $40 = $38 | $39;
     $41 = $37 & $40;
     $42 = (0 - ($41))|0;
     $43 = $41 & $42;
     $44 = (($43) + -1)|0;
     $45 = $44 >>> 12;
     $46 = $45 & 16;
     $47 = $44 >>> $46;
     $48 = $47 >>> 5;
     $49 = $48 & 8;
     $50 = $49 | $46;
     $51 = $47 >>> $49;
     $52 = $51 >>> 2;
     $53 = $52 & 4;
     $54 = $50 | $53;
     $55 = $51 >>> $53;
     $56 = $55 >>> 1;
     $57 = $56 & 2;
     $58 = $54 | $57;
     $59 = $55 >>> $57;
     $60 = $59 >>> 1;
     $61 = $60 & 1;
     $62 = $58 | $61;
     $63 = $59 >>> $61;
     $64 = (($62) + ($63))|0;
     $65 = $64 << 1;
     $66 = (9068 + ($65<<2)|0);
     $$sum4 = (($65) + 2)|0;
     $67 = (9068 + ($$sum4<<2)|0);
     $68 = HEAP32[$67>>2]|0;
     $69 = ((($68)) + 8|0);
     $70 = HEAP32[$69>>2]|0;
     $71 = ($66|0)==($70|0);
     do {
      if ($71) {
       $72 = 1 << $64;
       $73 = $72 ^ -1;
       $74 = $6 & $73;
       HEAP32[9028>>2] = $74;
       $89 = $34;
      } else {
       $75 = HEAP32[(9044)>>2]|0;
       $76 = ($70>>>0)<($75>>>0);
       if ($76) {
        _abort();
        // unreachable;
       }
       $77 = ((($70)) + 12|0);
       $78 = HEAP32[$77>>2]|0;
       $79 = ($78|0)==($68|0);
       if ($79) {
        HEAP32[$77>>2] = $66;
        HEAP32[$67>>2] = $70;
        $$pre = HEAP32[(9036)>>2]|0;
        $89 = $$pre;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $80 = $64 << 3;
     $81 = (($80) - ($4))|0;
     $82 = $4 | 3;
     $83 = ((($68)) + 4|0);
     HEAP32[$83>>2] = $82;
     $84 = (($68) + ($4)|0);
     $85 = $81 | 1;
     $$sum56 = $4 | 4;
     $86 = (($68) + ($$sum56)|0);
     HEAP32[$86>>2] = $85;
     $87 = (($68) + ($80)|0);
     HEAP32[$87>>2] = $81;
     $88 = ($89|0)==(0);
     if (!($88)) {
      $90 = HEAP32[(9048)>>2]|0;
      $91 = $89 >>> 3;
      $92 = $91 << 1;
      $93 = (9068 + ($92<<2)|0);
      $94 = HEAP32[9028>>2]|0;
      $95 = 1 << $91;
      $96 = $94 & $95;
      $97 = ($96|0)==(0);
      if ($97) {
       $98 = $94 | $95;
       HEAP32[9028>>2] = $98;
       $$pre105 = (($92) + 2)|0;
       $$pre106 = (9068 + ($$pre105<<2)|0);
       $$pre$phiZ2D = $$pre106;$F4$0 = $93;
      } else {
       $$sum9 = (($92) + 2)|0;
       $99 = (9068 + ($$sum9<<2)|0);
       $100 = HEAP32[$99>>2]|0;
       $101 = HEAP32[(9044)>>2]|0;
       $102 = ($100>>>0)<($101>>>0);
       if ($102) {
        _abort();
        // unreachable;
       } else {
        $$pre$phiZ2D = $99;$F4$0 = $100;
       }
      }
      HEAP32[$$pre$phiZ2D>>2] = $90;
      $103 = ((($F4$0)) + 12|0);
      HEAP32[$103>>2] = $90;
      $104 = ((($90)) + 8|0);
      HEAP32[$104>>2] = $F4$0;
      $105 = ((($90)) + 12|0);
      HEAP32[$105>>2] = $93;
     }
     HEAP32[(9036)>>2] = $81;
     HEAP32[(9048)>>2] = $84;
     $mem$0 = $69;
     return ($mem$0|0);
    }
    $106 = HEAP32[(9032)>>2]|0;
    $107 = ($106|0)==(0);
    if ($107) {
     $nb$0 = $4;
    } else {
     $108 = (0 - ($106))|0;
     $109 = $106 & $108;
     $110 = (($109) + -1)|0;
     $111 = $110 >>> 12;
     $112 = $111 & 16;
     $113 = $110 >>> $112;
     $114 = $113 >>> 5;
     $115 = $114 & 8;
     $116 = $115 | $112;
     $117 = $113 >>> $115;
     $118 = $117 >>> 2;
     $119 = $118 & 4;
     $120 = $116 | $119;
     $121 = $117 >>> $119;
     $122 = $121 >>> 1;
     $123 = $122 & 2;
     $124 = $120 | $123;
     $125 = $121 >>> $123;
     $126 = $125 >>> 1;
     $127 = $126 & 1;
     $128 = $124 | $127;
     $129 = $125 >>> $127;
     $130 = (($128) + ($129))|0;
     $131 = (9332 + ($130<<2)|0);
     $132 = HEAP32[$131>>2]|0;
     $133 = ((($132)) + 4|0);
     $134 = HEAP32[$133>>2]|0;
     $135 = $134 & -8;
     $136 = (($135) - ($4))|0;
     $rsize$0$i = $136;$t$0$i = $132;$v$0$i = $132;
     while(1) {
      $137 = ((($t$0$i)) + 16|0);
      $138 = HEAP32[$137>>2]|0;
      $139 = ($138|0)==(0|0);
      if ($139) {
       $140 = ((($t$0$i)) + 20|0);
       $141 = HEAP32[$140>>2]|0;
       $142 = ($141|0)==(0|0);
       if ($142) {
        $rsize$0$i$lcssa = $rsize$0$i;$v$0$i$lcssa = $v$0$i;
        break;
       } else {
        $144 = $141;
       }
      } else {
       $144 = $138;
      }
      $143 = ((($144)) + 4|0);
      $145 = HEAP32[$143>>2]|0;
      $146 = $145 & -8;
      $147 = (($146) - ($4))|0;
      $148 = ($147>>>0)<($rsize$0$i>>>0);
      $$rsize$0$i = $148 ? $147 : $rsize$0$i;
      $$v$0$i = $148 ? $144 : $v$0$i;
      $rsize$0$i = $$rsize$0$i;$t$0$i = $144;$v$0$i = $$v$0$i;
     }
     $149 = HEAP32[(9044)>>2]|0;
     $150 = ($v$0$i$lcssa>>>0)<($149>>>0);
     if ($150) {
      _abort();
      // unreachable;
     }
     $151 = (($v$0$i$lcssa) + ($4)|0);
     $152 = ($v$0$i$lcssa>>>0)<($151>>>0);
     if (!($152)) {
      _abort();
      // unreachable;
     }
     $153 = ((($v$0$i$lcssa)) + 24|0);
     $154 = HEAP32[$153>>2]|0;
     $155 = ((($v$0$i$lcssa)) + 12|0);
     $156 = HEAP32[$155>>2]|0;
     $157 = ($156|0)==($v$0$i$lcssa|0);
     do {
      if ($157) {
       $167 = ((($v$0$i$lcssa)) + 20|0);
       $168 = HEAP32[$167>>2]|0;
       $169 = ($168|0)==(0|0);
       if ($169) {
        $170 = ((($v$0$i$lcssa)) + 16|0);
        $171 = HEAP32[$170>>2]|0;
        $172 = ($171|0)==(0|0);
        if ($172) {
         $R$1$i = 0;
         break;
        } else {
         $R$0$i = $171;$RP$0$i = $170;
        }
       } else {
        $R$0$i = $168;$RP$0$i = $167;
       }
       while(1) {
        $173 = ((($R$0$i)) + 20|0);
        $174 = HEAP32[$173>>2]|0;
        $175 = ($174|0)==(0|0);
        if (!($175)) {
         $R$0$i = $174;$RP$0$i = $173;
         continue;
        }
        $176 = ((($R$0$i)) + 16|0);
        $177 = HEAP32[$176>>2]|0;
        $178 = ($177|0)==(0|0);
        if ($178) {
         $R$0$i$lcssa = $R$0$i;$RP$0$i$lcssa = $RP$0$i;
         break;
        } else {
         $R$0$i = $177;$RP$0$i = $176;
        }
       }
       $179 = ($RP$0$i$lcssa>>>0)<($149>>>0);
       if ($179) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$RP$0$i$lcssa>>2] = 0;
        $R$1$i = $R$0$i$lcssa;
        break;
       }
      } else {
       $158 = ((($v$0$i$lcssa)) + 8|0);
       $159 = HEAP32[$158>>2]|0;
       $160 = ($159>>>0)<($149>>>0);
       if ($160) {
        _abort();
        // unreachable;
       }
       $161 = ((($159)) + 12|0);
       $162 = HEAP32[$161>>2]|0;
       $163 = ($162|0)==($v$0$i$lcssa|0);
       if (!($163)) {
        _abort();
        // unreachable;
       }
       $164 = ((($156)) + 8|0);
       $165 = HEAP32[$164>>2]|0;
       $166 = ($165|0)==($v$0$i$lcssa|0);
       if ($166) {
        HEAP32[$161>>2] = $156;
        HEAP32[$164>>2] = $159;
        $R$1$i = $156;
        break;
       } else {
        _abort();
        // unreachable;
       }
      }
     } while(0);
     $180 = ($154|0)==(0|0);
     do {
      if (!($180)) {
       $181 = ((($v$0$i$lcssa)) + 28|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = (9332 + ($182<<2)|0);
       $184 = HEAP32[$183>>2]|0;
       $185 = ($v$0$i$lcssa|0)==($184|0);
       if ($185) {
        HEAP32[$183>>2] = $R$1$i;
        $cond$i = ($R$1$i|0)==(0|0);
        if ($cond$i) {
         $186 = 1 << $182;
         $187 = $186 ^ -1;
         $188 = HEAP32[(9032)>>2]|0;
         $189 = $188 & $187;
         HEAP32[(9032)>>2] = $189;
         break;
        }
       } else {
        $190 = HEAP32[(9044)>>2]|0;
        $191 = ($154>>>0)<($190>>>0);
        if ($191) {
         _abort();
         // unreachable;
        }
        $192 = ((($154)) + 16|0);
        $193 = HEAP32[$192>>2]|0;
        $194 = ($193|0)==($v$0$i$lcssa|0);
        if ($194) {
         HEAP32[$192>>2] = $R$1$i;
        } else {
         $195 = ((($154)) + 20|0);
         HEAP32[$195>>2] = $R$1$i;
        }
        $196 = ($R$1$i|0)==(0|0);
        if ($196) {
         break;
        }
       }
       $197 = HEAP32[(9044)>>2]|0;
       $198 = ($R$1$i>>>0)<($197>>>0);
       if ($198) {
        _abort();
        // unreachable;
       }
       $199 = ((($R$1$i)) + 24|0);
       HEAP32[$199>>2] = $154;
       $200 = ((($v$0$i$lcssa)) + 16|0);
       $201 = HEAP32[$200>>2]|0;
       $202 = ($201|0)==(0|0);
       do {
        if (!($202)) {
         $203 = ($201>>>0)<($197>>>0);
         if ($203) {
          _abort();
          // unreachable;
         } else {
          $204 = ((($R$1$i)) + 16|0);
          HEAP32[$204>>2] = $201;
          $205 = ((($201)) + 24|0);
          HEAP32[$205>>2] = $R$1$i;
          break;
         }
        }
       } while(0);
       $206 = ((($v$0$i$lcssa)) + 20|0);
       $207 = HEAP32[$206>>2]|0;
       $208 = ($207|0)==(0|0);
       if (!($208)) {
        $209 = HEAP32[(9044)>>2]|0;
        $210 = ($207>>>0)<($209>>>0);
        if ($210) {
         _abort();
         // unreachable;
        } else {
         $211 = ((($R$1$i)) + 20|0);
         HEAP32[$211>>2] = $207;
         $212 = ((($207)) + 24|0);
         HEAP32[$212>>2] = $R$1$i;
         break;
        }
       }
      }
     } while(0);
     $213 = ($rsize$0$i$lcssa>>>0)<(16);
     if ($213) {
      $214 = (($rsize$0$i$lcssa) + ($4))|0;
      $215 = $214 | 3;
      $216 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$216>>2] = $215;
      $$sum4$i = (($214) + 4)|0;
      $217 = (($v$0$i$lcssa) + ($$sum4$i)|0);
      $218 = HEAP32[$217>>2]|0;
      $219 = $218 | 1;
      HEAP32[$217>>2] = $219;
     } else {
      $220 = $4 | 3;
      $221 = ((($v$0$i$lcssa)) + 4|0);
      HEAP32[$221>>2] = $220;
      $222 = $rsize$0$i$lcssa | 1;
      $$sum$i35 = $4 | 4;
      $223 = (($v$0$i$lcssa) + ($$sum$i35)|0);
      HEAP32[$223>>2] = $222;
      $$sum1$i = (($rsize$0$i$lcssa) + ($4))|0;
      $224 = (($v$0$i$lcssa) + ($$sum1$i)|0);
      HEAP32[$224>>2] = $rsize$0$i$lcssa;
      $225 = HEAP32[(9036)>>2]|0;
      $226 = ($225|0)==(0);
      if (!($226)) {
       $227 = HEAP32[(9048)>>2]|0;
       $228 = $225 >>> 3;
       $229 = $228 << 1;
       $230 = (9068 + ($229<<2)|0);
       $231 = HEAP32[9028>>2]|0;
       $232 = 1 << $228;
       $233 = $231 & $232;
       $234 = ($233|0)==(0);
       if ($234) {
        $235 = $231 | $232;
        HEAP32[9028>>2] = $235;
        $$pre$i = (($229) + 2)|0;
        $$pre8$i = (9068 + ($$pre$i<<2)|0);
        $$pre$phi$iZ2D = $$pre8$i;$F1$0$i = $230;
       } else {
        $$sum3$i = (($229) + 2)|0;
        $236 = (9068 + ($$sum3$i<<2)|0);
        $237 = HEAP32[$236>>2]|0;
        $238 = HEAP32[(9044)>>2]|0;
        $239 = ($237>>>0)<($238>>>0);
        if ($239) {
         _abort();
         // unreachable;
        } else {
         $$pre$phi$iZ2D = $236;$F1$0$i = $237;
        }
       }
       HEAP32[$$pre$phi$iZ2D>>2] = $227;
       $240 = ((($F1$0$i)) + 12|0);
       HEAP32[$240>>2] = $227;
       $241 = ((($227)) + 8|0);
       HEAP32[$241>>2] = $F1$0$i;
       $242 = ((($227)) + 12|0);
       HEAP32[$242>>2] = $230;
      }
      HEAP32[(9036)>>2] = $rsize$0$i$lcssa;
      HEAP32[(9048)>>2] = $151;
     }
     $243 = ((($v$0$i$lcssa)) + 8|0);
     $mem$0 = $243;
     return ($mem$0|0);
    }
   } else {
    $nb$0 = $4;
   }
  } else {
   $244 = ($bytes>>>0)>(4294967231);
   if ($244) {
    $nb$0 = -1;
   } else {
    $245 = (($bytes) + 11)|0;
    $246 = $245 & -8;
    $247 = HEAP32[(9032)>>2]|0;
    $248 = ($247|0)==(0);
    if ($248) {
     $nb$0 = $246;
    } else {
     $249 = (0 - ($246))|0;
     $250 = $245 >>> 8;
     $251 = ($250|0)==(0);
     if ($251) {
      $idx$0$i = 0;
     } else {
      $252 = ($246>>>0)>(16777215);
      if ($252) {
       $idx$0$i = 31;
      } else {
       $253 = (($250) + 1048320)|0;
       $254 = $253 >>> 16;
       $255 = $254 & 8;
       $256 = $250 << $255;
       $257 = (($256) + 520192)|0;
       $258 = $257 >>> 16;
       $259 = $258 & 4;
       $260 = $259 | $255;
       $261 = $256 << $259;
       $262 = (($261) + 245760)|0;
       $263 = $262 >>> 16;
       $264 = $263 & 2;
       $265 = $260 | $264;
       $266 = (14 - ($265))|0;
       $267 = $261 << $264;
       $268 = $267 >>> 15;
       $269 = (($266) + ($268))|0;
       $270 = $269 << 1;
       $271 = (($269) + 7)|0;
       $272 = $246 >>> $271;
       $273 = $272 & 1;
       $274 = $273 | $270;
       $idx$0$i = $274;
      }
     }
     $275 = (9332 + ($idx$0$i<<2)|0);
     $276 = HEAP32[$275>>2]|0;
     $277 = ($276|0)==(0|0);
     L123: do {
      if ($277) {
       $rsize$2$i = $249;$t$1$i = 0;$v$2$i = 0;
       label = 86;
      } else {
       $278 = ($idx$0$i|0)==(31);
       $279 = $idx$0$i >>> 1;
       $280 = (25 - ($279))|0;
       $281 = $278 ? 0 : $280;
       $282 = $246 << $281;
       $rsize$0$i15 = $249;$rst$0$i = 0;$sizebits$0$i = $282;$t$0$i14 = $276;$v$0$i16 = 0;
       while(1) {
        $283 = ((($t$0$i14)) + 4|0);
        $284 = HEAP32[$283>>2]|0;
        $285 = $284 & -8;
        $286 = (($285) - ($246))|0;
        $287 = ($286>>>0)<($rsize$0$i15>>>0);
        if ($287) {
         $288 = ($285|0)==($246|0);
         if ($288) {
          $rsize$331$i = $286;$t$230$i = $t$0$i14;$v$332$i = $t$0$i14;
          label = 90;
          break L123;
         } else {
          $rsize$1$i = $286;$v$1$i = $t$0$i14;
         }
        } else {
         $rsize$1$i = $rsize$0$i15;$v$1$i = $v$0$i16;
        }
        $289 = ((($t$0$i14)) + 20|0);
        $290 = HEAP32[$289>>2]|0;
        $291 = $sizebits$0$i >>> 31;
        $292 = (((($t$0$i14)) + 16|0) + ($291<<2)|0);
        $293 = HEAP32[$292>>2]|0;
        $294 = ($290|0)==(0|0);
        $295 = ($290|0)==($293|0);
        $or$cond19$i = $294 | $295;
        $rst$1$i = $or$cond19$i ? $rst$0$i : $290;
        $296 = ($293|0)==(0|0);
        $297 = $sizebits$0$i << 1;
        if ($296) {
         $rsize$2$i = $rsize$1$i;$t$1$i = $rst$1$i;$v$2$i = $v$1$i;
         label = 86;
         break;
        } else {
         $rsize$0$i15 = $rsize$1$i;$rst$0$i = $rst$1$i;$sizebits$0$i = $297;$t$0$i14 = $293;$v$0$i16 = $v$1$i;
        }
       }
      }
     } while(0);
     if ((label|0) == 86) {
      $298 = ($t$1$i|0)==(0|0);
      $299 = ($v$2$i|0)==(0|0);
      $or$cond$i = $298 & $299;
      if ($or$cond$i) {
       $300 = 2 << $idx$0$i;
       $301 = (0 - ($300))|0;
       $302 = $300 | $301;
       $303 = $247 & $302;
       $304 = ($303|0)==(0);
       if ($304) {
        $nb$0 = $246;
        break;
       }
       $305 = (0 - ($303))|0;
       $306 = $303 & $305;
       $307 = (($306) + -1)|0;
       $308 = $307 >>> 12;
       $309 = $308 & 16;
       $310 = $307 >>> $309;
       $311 = $310 >>> 5;
       $312 = $311 & 8;
       $313 = $312 | $309;
       $314 = $310 >>> $312;
       $315 = $314 >>> 2;
       $316 = $315 & 4;
       $317 = $313 | $316;
       $318 = $314 >>> $316;
       $319 = $318 >>> 1;
       $320 = $319 & 2;
       $321 = $317 | $320;
       $322 = $318 >>> $320;
       $323 = $322 >>> 1;
       $324 = $323 & 1;
       $325 = $321 | $324;
       $326 = $322 >>> $324;
       $327 = (($325) + ($326))|0;
       $328 = (9332 + ($327<<2)|0);
       $329 = HEAP32[$328>>2]|0;
       $t$2$ph$i = $329;$v$3$ph$i = 0;
      } else {
       $t$2$ph$i = $t$1$i;$v$3$ph$i = $v$2$i;
      }
      $330 = ($t$2$ph$i|0)==(0|0);
      if ($330) {
       $rsize$3$lcssa$i = $rsize$2$i;$v$3$lcssa$i = $v$3$ph$i;
      } else {
       $rsize$331$i = $rsize$2$i;$t$230$i = $t$2$ph$i;$v$332$i = $v$3$ph$i;
       label = 90;
      }
     }
     if ((label|0) == 90) {
      while(1) {
       label = 0;
       $331 = ((($t$230$i)) + 4|0);
       $332 = HEAP32[$331>>2]|0;
       $333 = $332 & -8;
       $334 = (($333) - ($246))|0;
       $335 = ($334>>>0)<($rsize$331$i>>>0);
       $$rsize$3$i = $335 ? $334 : $rsize$331$i;
       $t$2$v$3$i = $335 ? $t$230$i : $v$332$i;
       $336 = ((($t$230$i)) + 16|0);
       $337 = HEAP32[$336>>2]|0;
       $338 = ($337|0)==(0|0);
       if (!($338)) {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $337;$v$332$i = $t$2$v$3$i;
        label = 90;
        continue;
       }
       $339 = ((($t$230$i)) + 20|0);
       $340 = HEAP32[$339>>2]|0;
       $341 = ($340|0)==(0|0);
       if ($341) {
        $rsize$3$lcssa$i = $$rsize$3$i;$v$3$lcssa$i = $t$2$v$3$i;
        break;
       } else {
        $rsize$331$i = $$rsize$3$i;$t$230$i = $340;$v$332$i = $t$2$v$3$i;
        label = 90;
       }
      }
     }
     $342 = ($v$3$lcssa$i|0)==(0|0);
     if ($342) {
      $nb$0 = $246;
     } else {
      $343 = HEAP32[(9036)>>2]|0;
      $344 = (($343) - ($246))|0;
      $345 = ($rsize$3$lcssa$i>>>0)<($344>>>0);
      if ($345) {
       $346 = HEAP32[(9044)>>2]|0;
       $347 = ($v$3$lcssa$i>>>0)<($346>>>0);
       if ($347) {
        _abort();
        // unreachable;
       }
       $348 = (($v$3$lcssa$i) + ($246)|0);
       $349 = ($v$3$lcssa$i>>>0)<($348>>>0);
       if (!($349)) {
        _abort();
        // unreachable;
       }
       $350 = ((($v$3$lcssa$i)) + 24|0);
       $351 = HEAP32[$350>>2]|0;
       $352 = ((($v$3$lcssa$i)) + 12|0);
       $353 = HEAP32[$352>>2]|0;
       $354 = ($353|0)==($v$3$lcssa$i|0);
       do {
        if ($354) {
         $364 = ((($v$3$lcssa$i)) + 20|0);
         $365 = HEAP32[$364>>2]|0;
         $366 = ($365|0)==(0|0);
         if ($366) {
          $367 = ((($v$3$lcssa$i)) + 16|0);
          $368 = HEAP32[$367>>2]|0;
          $369 = ($368|0)==(0|0);
          if ($369) {
           $R$1$i20 = 0;
           break;
          } else {
           $R$0$i18 = $368;$RP$0$i17 = $367;
          }
         } else {
          $R$0$i18 = $365;$RP$0$i17 = $364;
         }
         while(1) {
          $370 = ((($R$0$i18)) + 20|0);
          $371 = HEAP32[$370>>2]|0;
          $372 = ($371|0)==(0|0);
          if (!($372)) {
           $R$0$i18 = $371;$RP$0$i17 = $370;
           continue;
          }
          $373 = ((($R$0$i18)) + 16|0);
          $374 = HEAP32[$373>>2]|0;
          $375 = ($374|0)==(0|0);
          if ($375) {
           $R$0$i18$lcssa = $R$0$i18;$RP$0$i17$lcssa = $RP$0$i17;
           break;
          } else {
           $R$0$i18 = $374;$RP$0$i17 = $373;
          }
         }
         $376 = ($RP$0$i17$lcssa>>>0)<($346>>>0);
         if ($376) {
          _abort();
          // unreachable;
         } else {
          HEAP32[$RP$0$i17$lcssa>>2] = 0;
          $R$1$i20 = $R$0$i18$lcssa;
          break;
         }
        } else {
         $355 = ((($v$3$lcssa$i)) + 8|0);
         $356 = HEAP32[$355>>2]|0;
         $357 = ($356>>>0)<($346>>>0);
         if ($357) {
          _abort();
          // unreachable;
         }
         $358 = ((($356)) + 12|0);
         $359 = HEAP32[$358>>2]|0;
         $360 = ($359|0)==($v$3$lcssa$i|0);
         if (!($360)) {
          _abort();
          // unreachable;
         }
         $361 = ((($353)) + 8|0);
         $362 = HEAP32[$361>>2]|0;
         $363 = ($362|0)==($v$3$lcssa$i|0);
         if ($363) {
          HEAP32[$358>>2] = $353;
          HEAP32[$361>>2] = $356;
          $R$1$i20 = $353;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $377 = ($351|0)==(0|0);
       do {
        if (!($377)) {
         $378 = ((($v$3$lcssa$i)) + 28|0);
         $379 = HEAP32[$378>>2]|0;
         $380 = (9332 + ($379<<2)|0);
         $381 = HEAP32[$380>>2]|0;
         $382 = ($v$3$lcssa$i|0)==($381|0);
         if ($382) {
          HEAP32[$380>>2] = $R$1$i20;
          $cond$i21 = ($R$1$i20|0)==(0|0);
          if ($cond$i21) {
           $383 = 1 << $379;
           $384 = $383 ^ -1;
           $385 = HEAP32[(9032)>>2]|0;
           $386 = $385 & $384;
           HEAP32[(9032)>>2] = $386;
           break;
          }
         } else {
          $387 = HEAP32[(9044)>>2]|0;
          $388 = ($351>>>0)<($387>>>0);
          if ($388) {
           _abort();
           // unreachable;
          }
          $389 = ((($351)) + 16|0);
          $390 = HEAP32[$389>>2]|0;
          $391 = ($390|0)==($v$3$lcssa$i|0);
          if ($391) {
           HEAP32[$389>>2] = $R$1$i20;
          } else {
           $392 = ((($351)) + 20|0);
           HEAP32[$392>>2] = $R$1$i20;
          }
          $393 = ($R$1$i20|0)==(0|0);
          if ($393) {
           break;
          }
         }
         $394 = HEAP32[(9044)>>2]|0;
         $395 = ($R$1$i20>>>0)<($394>>>0);
         if ($395) {
          _abort();
          // unreachable;
         }
         $396 = ((($R$1$i20)) + 24|0);
         HEAP32[$396>>2] = $351;
         $397 = ((($v$3$lcssa$i)) + 16|0);
         $398 = HEAP32[$397>>2]|0;
         $399 = ($398|0)==(0|0);
         do {
          if (!($399)) {
           $400 = ($398>>>0)<($394>>>0);
           if ($400) {
            _abort();
            // unreachable;
           } else {
            $401 = ((($R$1$i20)) + 16|0);
            HEAP32[$401>>2] = $398;
            $402 = ((($398)) + 24|0);
            HEAP32[$402>>2] = $R$1$i20;
            break;
           }
          }
         } while(0);
         $403 = ((($v$3$lcssa$i)) + 20|0);
         $404 = HEAP32[$403>>2]|0;
         $405 = ($404|0)==(0|0);
         if (!($405)) {
          $406 = HEAP32[(9044)>>2]|0;
          $407 = ($404>>>0)<($406>>>0);
          if ($407) {
           _abort();
           // unreachable;
          } else {
           $408 = ((($R$1$i20)) + 20|0);
           HEAP32[$408>>2] = $404;
           $409 = ((($404)) + 24|0);
           HEAP32[$409>>2] = $R$1$i20;
           break;
          }
         }
        }
       } while(0);
       $410 = ($rsize$3$lcssa$i>>>0)<(16);
       L199: do {
        if ($410) {
         $411 = (($rsize$3$lcssa$i) + ($246))|0;
         $412 = $411 | 3;
         $413 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$413>>2] = $412;
         $$sum18$i = (($411) + 4)|0;
         $414 = (($v$3$lcssa$i) + ($$sum18$i)|0);
         $415 = HEAP32[$414>>2]|0;
         $416 = $415 | 1;
         HEAP32[$414>>2] = $416;
        } else {
         $417 = $246 | 3;
         $418 = ((($v$3$lcssa$i)) + 4|0);
         HEAP32[$418>>2] = $417;
         $419 = $rsize$3$lcssa$i | 1;
         $$sum$i2334 = $246 | 4;
         $420 = (($v$3$lcssa$i) + ($$sum$i2334)|0);
         HEAP32[$420>>2] = $419;
         $$sum1$i24 = (($rsize$3$lcssa$i) + ($246))|0;
         $421 = (($v$3$lcssa$i) + ($$sum1$i24)|0);
         HEAP32[$421>>2] = $rsize$3$lcssa$i;
         $422 = $rsize$3$lcssa$i >>> 3;
         $423 = ($rsize$3$lcssa$i>>>0)<(256);
         if ($423) {
          $424 = $422 << 1;
          $425 = (9068 + ($424<<2)|0);
          $426 = HEAP32[9028>>2]|0;
          $427 = 1 << $422;
          $428 = $426 & $427;
          $429 = ($428|0)==(0);
          if ($429) {
           $430 = $426 | $427;
           HEAP32[9028>>2] = $430;
           $$pre$i25 = (($424) + 2)|0;
           $$pre43$i = (9068 + ($$pre$i25<<2)|0);
           $$pre$phi$i26Z2D = $$pre43$i;$F5$0$i = $425;
          } else {
           $$sum17$i = (($424) + 2)|0;
           $431 = (9068 + ($$sum17$i<<2)|0);
           $432 = HEAP32[$431>>2]|0;
           $433 = HEAP32[(9044)>>2]|0;
           $434 = ($432>>>0)<($433>>>0);
           if ($434) {
            _abort();
            // unreachable;
           } else {
            $$pre$phi$i26Z2D = $431;$F5$0$i = $432;
           }
          }
          HEAP32[$$pre$phi$i26Z2D>>2] = $348;
          $435 = ((($F5$0$i)) + 12|0);
          HEAP32[$435>>2] = $348;
          $$sum15$i = (($246) + 8)|0;
          $436 = (($v$3$lcssa$i) + ($$sum15$i)|0);
          HEAP32[$436>>2] = $F5$0$i;
          $$sum16$i = (($246) + 12)|0;
          $437 = (($v$3$lcssa$i) + ($$sum16$i)|0);
          HEAP32[$437>>2] = $425;
          break;
         }
         $438 = $rsize$3$lcssa$i >>> 8;
         $439 = ($438|0)==(0);
         if ($439) {
          $I7$0$i = 0;
         } else {
          $440 = ($rsize$3$lcssa$i>>>0)>(16777215);
          if ($440) {
           $I7$0$i = 31;
          } else {
           $441 = (($438) + 1048320)|0;
           $442 = $441 >>> 16;
           $443 = $442 & 8;
           $444 = $438 << $443;
           $445 = (($444) + 520192)|0;
           $446 = $445 >>> 16;
           $447 = $446 & 4;
           $448 = $447 | $443;
           $449 = $444 << $447;
           $450 = (($449) + 245760)|0;
           $451 = $450 >>> 16;
           $452 = $451 & 2;
           $453 = $448 | $452;
           $454 = (14 - ($453))|0;
           $455 = $449 << $452;
           $456 = $455 >>> 15;
           $457 = (($454) + ($456))|0;
           $458 = $457 << 1;
           $459 = (($457) + 7)|0;
           $460 = $rsize$3$lcssa$i >>> $459;
           $461 = $460 & 1;
           $462 = $461 | $458;
           $I7$0$i = $462;
          }
         }
         $463 = (9332 + ($I7$0$i<<2)|0);
         $$sum2$i = (($246) + 28)|0;
         $464 = (($v$3$lcssa$i) + ($$sum2$i)|0);
         HEAP32[$464>>2] = $I7$0$i;
         $$sum3$i27 = (($246) + 16)|0;
         $465 = (($v$3$lcssa$i) + ($$sum3$i27)|0);
         $$sum4$i28 = (($246) + 20)|0;
         $466 = (($v$3$lcssa$i) + ($$sum4$i28)|0);
         HEAP32[$466>>2] = 0;
         HEAP32[$465>>2] = 0;
         $467 = HEAP32[(9032)>>2]|0;
         $468 = 1 << $I7$0$i;
         $469 = $467 & $468;
         $470 = ($469|0)==(0);
         if ($470) {
          $471 = $467 | $468;
          HEAP32[(9032)>>2] = $471;
          HEAP32[$463>>2] = $348;
          $$sum5$i = (($246) + 24)|0;
          $472 = (($v$3$lcssa$i) + ($$sum5$i)|0);
          HEAP32[$472>>2] = $463;
          $$sum6$i = (($246) + 12)|0;
          $473 = (($v$3$lcssa$i) + ($$sum6$i)|0);
          HEAP32[$473>>2] = $348;
          $$sum7$i = (($246) + 8)|0;
          $474 = (($v$3$lcssa$i) + ($$sum7$i)|0);
          HEAP32[$474>>2] = $348;
          break;
         }
         $475 = HEAP32[$463>>2]|0;
         $476 = ((($475)) + 4|0);
         $477 = HEAP32[$476>>2]|0;
         $478 = $477 & -8;
         $479 = ($478|0)==($rsize$3$lcssa$i|0);
         L217: do {
          if ($479) {
           $T$0$lcssa$i = $475;
          } else {
           $480 = ($I7$0$i|0)==(31);
           $481 = $I7$0$i >>> 1;
           $482 = (25 - ($481))|0;
           $483 = $480 ? 0 : $482;
           $484 = $rsize$3$lcssa$i << $483;
           $K12$029$i = $484;$T$028$i = $475;
           while(1) {
            $491 = $K12$029$i >>> 31;
            $492 = (((($T$028$i)) + 16|0) + ($491<<2)|0);
            $487 = HEAP32[$492>>2]|0;
            $493 = ($487|0)==(0|0);
            if ($493) {
             $$lcssa232 = $492;$T$028$i$lcssa = $T$028$i;
             break;
            }
            $485 = $K12$029$i << 1;
            $486 = ((($487)) + 4|0);
            $488 = HEAP32[$486>>2]|0;
            $489 = $488 & -8;
            $490 = ($489|0)==($rsize$3$lcssa$i|0);
            if ($490) {
             $T$0$lcssa$i = $487;
             break L217;
            } else {
             $K12$029$i = $485;$T$028$i = $487;
            }
           }
           $494 = HEAP32[(9044)>>2]|0;
           $495 = ($$lcssa232>>>0)<($494>>>0);
           if ($495) {
            _abort();
            // unreachable;
           } else {
            HEAP32[$$lcssa232>>2] = $348;
            $$sum11$i = (($246) + 24)|0;
            $496 = (($v$3$lcssa$i) + ($$sum11$i)|0);
            HEAP32[$496>>2] = $T$028$i$lcssa;
            $$sum12$i = (($246) + 12)|0;
            $497 = (($v$3$lcssa$i) + ($$sum12$i)|0);
            HEAP32[$497>>2] = $348;
            $$sum13$i = (($246) + 8)|0;
            $498 = (($v$3$lcssa$i) + ($$sum13$i)|0);
            HEAP32[$498>>2] = $348;
            break L199;
           }
          }
         } while(0);
         $499 = ((($T$0$lcssa$i)) + 8|0);
         $500 = HEAP32[$499>>2]|0;
         $501 = HEAP32[(9044)>>2]|0;
         $502 = ($500>>>0)>=($501>>>0);
         $not$$i = ($T$0$lcssa$i>>>0)>=($501>>>0);
         $503 = $502 & $not$$i;
         if ($503) {
          $504 = ((($500)) + 12|0);
          HEAP32[$504>>2] = $348;
          HEAP32[$499>>2] = $348;
          $$sum8$i = (($246) + 8)|0;
          $505 = (($v$3$lcssa$i) + ($$sum8$i)|0);
          HEAP32[$505>>2] = $500;
          $$sum9$i = (($246) + 12)|0;
          $506 = (($v$3$lcssa$i) + ($$sum9$i)|0);
          HEAP32[$506>>2] = $T$0$lcssa$i;
          $$sum10$i = (($246) + 24)|0;
          $507 = (($v$3$lcssa$i) + ($$sum10$i)|0);
          HEAP32[$507>>2] = 0;
          break;
         } else {
          _abort();
          // unreachable;
         }
        }
       } while(0);
       $508 = ((($v$3$lcssa$i)) + 8|0);
       $mem$0 = $508;
       return ($mem$0|0);
      } else {
       $nb$0 = $246;
      }
     }
    }
   }
  }
 } while(0);
 $509 = HEAP32[(9036)>>2]|0;
 $510 = ($509>>>0)<($nb$0>>>0);
 if (!($510)) {
  $511 = (($509) - ($nb$0))|0;
  $512 = HEAP32[(9048)>>2]|0;
  $513 = ($511>>>0)>(15);
  if ($513) {
   $514 = (($512) + ($nb$0)|0);
   HEAP32[(9048)>>2] = $514;
   HEAP32[(9036)>>2] = $511;
   $515 = $511 | 1;
   $$sum2 = (($nb$0) + 4)|0;
   $516 = (($512) + ($$sum2)|0);
   HEAP32[$516>>2] = $515;
   $517 = (($512) + ($509)|0);
   HEAP32[$517>>2] = $511;
   $518 = $nb$0 | 3;
   $519 = ((($512)) + 4|0);
   HEAP32[$519>>2] = $518;
  } else {
   HEAP32[(9036)>>2] = 0;
   HEAP32[(9048)>>2] = 0;
   $520 = $509 | 3;
   $521 = ((($512)) + 4|0);
   HEAP32[$521>>2] = $520;
   $$sum1 = (($509) + 4)|0;
   $522 = (($512) + ($$sum1)|0);
   $523 = HEAP32[$522>>2]|0;
   $524 = $523 | 1;
   HEAP32[$522>>2] = $524;
  }
  $525 = ((($512)) + 8|0);
  $mem$0 = $525;
  return ($mem$0|0);
 }
 $526 = HEAP32[(9040)>>2]|0;
 $527 = ($526>>>0)>($nb$0>>>0);
 if ($527) {
  $528 = (($526) - ($nb$0))|0;
  HEAP32[(9040)>>2] = $528;
  $529 = HEAP32[(9052)>>2]|0;
  $530 = (($529) + ($nb$0)|0);
  HEAP32[(9052)>>2] = $530;
  $531 = $528 | 1;
  $$sum = (($nb$0) + 4)|0;
  $532 = (($529) + ($$sum)|0);
  HEAP32[$532>>2] = $531;
  $533 = $nb$0 | 3;
  $534 = ((($529)) + 4|0);
  HEAP32[$534>>2] = $533;
  $535 = ((($529)) + 8|0);
  $mem$0 = $535;
  return ($mem$0|0);
 }
 $536 = HEAP32[9500>>2]|0;
 $537 = ($536|0)==(0);
 do {
  if ($537) {
   $538 = (_sysconf(30)|0);
   $539 = (($538) + -1)|0;
   $540 = $539 & $538;
   $541 = ($540|0)==(0);
   if ($541) {
    HEAP32[(9508)>>2] = $538;
    HEAP32[(9504)>>2] = $538;
    HEAP32[(9512)>>2] = -1;
    HEAP32[(9516)>>2] = -1;
    HEAP32[(9520)>>2] = 0;
    HEAP32[(9472)>>2] = 0;
    $542 = (_time((0|0))|0);
    $543 = $542 & -16;
    $544 = $543 ^ 1431655768;
    HEAP32[9500>>2] = $544;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $545 = (($nb$0) + 48)|0;
 $546 = HEAP32[(9508)>>2]|0;
 $547 = (($nb$0) + 47)|0;
 $548 = (($546) + ($547))|0;
 $549 = (0 - ($546))|0;
 $550 = $548 & $549;
 $551 = ($550>>>0)>($nb$0>>>0);
 if (!($551)) {
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $552 = HEAP32[(9468)>>2]|0;
 $553 = ($552|0)==(0);
 if (!($553)) {
  $554 = HEAP32[(9460)>>2]|0;
  $555 = (($554) + ($550))|0;
  $556 = ($555>>>0)<=($554>>>0);
  $557 = ($555>>>0)>($552>>>0);
  $or$cond1$i = $556 | $557;
  if ($or$cond1$i) {
   $mem$0 = 0;
   return ($mem$0|0);
  }
 }
 $558 = HEAP32[(9472)>>2]|0;
 $559 = $558 & 4;
 $560 = ($559|0)==(0);
 L258: do {
  if ($560) {
   $561 = HEAP32[(9052)>>2]|0;
   $562 = ($561|0)==(0|0);
   L260: do {
    if ($562) {
     label = 174;
    } else {
     $sp$0$i$i = (9476);
     while(1) {
      $563 = HEAP32[$sp$0$i$i>>2]|0;
      $564 = ($563>>>0)>($561>>>0);
      if (!($564)) {
       $565 = ((($sp$0$i$i)) + 4|0);
       $566 = HEAP32[$565>>2]|0;
       $567 = (($563) + ($566)|0);
       $568 = ($567>>>0)>($561>>>0);
       if ($568) {
        $$lcssa228 = $sp$0$i$i;$$lcssa230 = $565;
        break;
       }
      }
      $569 = ((($sp$0$i$i)) + 8|0);
      $570 = HEAP32[$569>>2]|0;
      $571 = ($570|0)==(0|0);
      if ($571) {
       label = 174;
       break L260;
      } else {
       $sp$0$i$i = $570;
      }
     }
     $594 = HEAP32[(9040)>>2]|0;
     $595 = (($548) - ($594))|0;
     $596 = $595 & $549;
     $597 = ($596>>>0)<(2147483647);
     if ($597) {
      $598 = (_sbrk(($596|0))|0);
      $599 = HEAP32[$$lcssa228>>2]|0;
      $600 = HEAP32[$$lcssa230>>2]|0;
      $601 = (($599) + ($600)|0);
      $602 = ($598|0)==($601|0);
      $$3$i = $602 ? $596 : 0;
      if ($602) {
       $603 = ($598|0)==((-1)|0);
       if ($603) {
        $tsize$0323944$i = $$3$i;
       } else {
        $tbase$255$i = $598;$tsize$254$i = $$3$i;
        label = 194;
        break L258;
       }
      } else {
       $br$0$ph$i = $598;$ssize$1$ph$i = $596;$tsize$0$ph$i = $$3$i;
       label = 184;
      }
     } else {
      $tsize$0323944$i = 0;
     }
    }
   } while(0);
   do {
    if ((label|0) == 174) {
     $572 = (_sbrk(0)|0);
     $573 = ($572|0)==((-1)|0);
     if ($573) {
      $tsize$0323944$i = 0;
     } else {
      $574 = $572;
      $575 = HEAP32[(9504)>>2]|0;
      $576 = (($575) + -1)|0;
      $577 = $576 & $574;
      $578 = ($577|0)==(0);
      if ($578) {
       $ssize$0$i = $550;
      } else {
       $579 = (($576) + ($574))|0;
       $580 = (0 - ($575))|0;
       $581 = $579 & $580;
       $582 = (($550) - ($574))|0;
       $583 = (($582) + ($581))|0;
       $ssize$0$i = $583;
      }
      $584 = HEAP32[(9460)>>2]|0;
      $585 = (($584) + ($ssize$0$i))|0;
      $586 = ($ssize$0$i>>>0)>($nb$0>>>0);
      $587 = ($ssize$0$i>>>0)<(2147483647);
      $or$cond$i30 = $586 & $587;
      if ($or$cond$i30) {
       $588 = HEAP32[(9468)>>2]|0;
       $589 = ($588|0)==(0);
       if (!($589)) {
        $590 = ($585>>>0)<=($584>>>0);
        $591 = ($585>>>0)>($588>>>0);
        $or$cond2$i = $590 | $591;
        if ($or$cond2$i) {
         $tsize$0323944$i = 0;
         break;
        }
       }
       $592 = (_sbrk(($ssize$0$i|0))|0);
       $593 = ($592|0)==($572|0);
       $ssize$0$$i = $593 ? $ssize$0$i : 0;
       if ($593) {
        $tbase$255$i = $572;$tsize$254$i = $ssize$0$$i;
        label = 194;
        break L258;
       } else {
        $br$0$ph$i = $592;$ssize$1$ph$i = $ssize$0$i;$tsize$0$ph$i = $ssize$0$$i;
        label = 184;
       }
      } else {
       $tsize$0323944$i = 0;
      }
     }
    }
   } while(0);
   L280: do {
    if ((label|0) == 184) {
     $604 = (0 - ($ssize$1$ph$i))|0;
     $605 = ($br$0$ph$i|0)!=((-1)|0);
     $606 = ($ssize$1$ph$i>>>0)<(2147483647);
     $or$cond5$i = $606 & $605;
     $607 = ($545>>>0)>($ssize$1$ph$i>>>0);
     $or$cond6$i = $607 & $or$cond5$i;
     do {
      if ($or$cond6$i) {
       $608 = HEAP32[(9508)>>2]|0;
       $609 = (($547) - ($ssize$1$ph$i))|0;
       $610 = (($609) + ($608))|0;
       $611 = (0 - ($608))|0;
       $612 = $610 & $611;
       $613 = ($612>>>0)<(2147483647);
       if ($613) {
        $614 = (_sbrk(($612|0))|0);
        $615 = ($614|0)==((-1)|0);
        if ($615) {
         (_sbrk(($604|0))|0);
         $tsize$0323944$i = $tsize$0$ph$i;
         break L280;
        } else {
         $616 = (($612) + ($ssize$1$ph$i))|0;
         $ssize$2$i = $616;
         break;
        }
       } else {
        $ssize$2$i = $ssize$1$ph$i;
       }
      } else {
       $ssize$2$i = $ssize$1$ph$i;
      }
     } while(0);
     $617 = ($br$0$ph$i|0)==((-1)|0);
     if ($617) {
      $tsize$0323944$i = $tsize$0$ph$i;
     } else {
      $tbase$255$i = $br$0$ph$i;$tsize$254$i = $ssize$2$i;
      label = 194;
      break L258;
     }
    }
   } while(0);
   $618 = HEAP32[(9472)>>2]|0;
   $619 = $618 | 4;
   HEAP32[(9472)>>2] = $619;
   $tsize$1$i = $tsize$0323944$i;
   label = 191;
  } else {
   $tsize$1$i = 0;
   label = 191;
  }
 } while(0);
 if ((label|0) == 191) {
  $620 = ($550>>>0)<(2147483647);
  if ($620) {
   $621 = (_sbrk(($550|0))|0);
   $622 = (_sbrk(0)|0);
   $623 = ($621|0)!=((-1)|0);
   $624 = ($622|0)!=((-1)|0);
   $or$cond3$i = $623 & $624;
   $625 = ($621>>>0)<($622>>>0);
   $or$cond8$i = $625 & $or$cond3$i;
   if ($or$cond8$i) {
    $626 = $622;
    $627 = $621;
    $628 = (($626) - ($627))|0;
    $629 = (($nb$0) + 40)|0;
    $630 = ($628>>>0)>($629>>>0);
    $$tsize$1$i = $630 ? $628 : $tsize$1$i;
    if ($630) {
     $tbase$255$i = $621;$tsize$254$i = $$tsize$1$i;
     label = 194;
    }
   }
  }
 }
 if ((label|0) == 194) {
  $631 = HEAP32[(9460)>>2]|0;
  $632 = (($631) + ($tsize$254$i))|0;
  HEAP32[(9460)>>2] = $632;
  $633 = HEAP32[(9464)>>2]|0;
  $634 = ($632>>>0)>($633>>>0);
  if ($634) {
   HEAP32[(9464)>>2] = $632;
  }
  $635 = HEAP32[(9052)>>2]|0;
  $636 = ($635|0)==(0|0);
  L299: do {
   if ($636) {
    $637 = HEAP32[(9044)>>2]|0;
    $638 = ($637|0)==(0|0);
    $639 = ($tbase$255$i>>>0)<($637>>>0);
    $or$cond9$i = $638 | $639;
    if ($or$cond9$i) {
     HEAP32[(9044)>>2] = $tbase$255$i;
    }
    HEAP32[(9476)>>2] = $tbase$255$i;
    HEAP32[(9480)>>2] = $tsize$254$i;
    HEAP32[(9488)>>2] = 0;
    $640 = HEAP32[9500>>2]|0;
    HEAP32[(9064)>>2] = $640;
    HEAP32[(9060)>>2] = -1;
    $i$02$i$i = 0;
    while(1) {
     $641 = $i$02$i$i << 1;
     $642 = (9068 + ($641<<2)|0);
     $$sum$i$i = (($641) + 3)|0;
     $643 = (9068 + ($$sum$i$i<<2)|0);
     HEAP32[$643>>2] = $642;
     $$sum1$i$i = (($641) + 2)|0;
     $644 = (9068 + ($$sum1$i$i<<2)|0);
     HEAP32[$644>>2] = $642;
     $645 = (($i$02$i$i) + 1)|0;
     $exitcond$i$i = ($645|0)==(32);
     if ($exitcond$i$i) {
      break;
     } else {
      $i$02$i$i = $645;
     }
    }
    $646 = (($tsize$254$i) + -40)|0;
    $647 = ((($tbase$255$i)) + 8|0);
    $648 = $647;
    $649 = $648 & 7;
    $650 = ($649|0)==(0);
    $651 = (0 - ($648))|0;
    $652 = $651 & 7;
    $653 = $650 ? 0 : $652;
    $654 = (($tbase$255$i) + ($653)|0);
    $655 = (($646) - ($653))|0;
    HEAP32[(9052)>>2] = $654;
    HEAP32[(9040)>>2] = $655;
    $656 = $655 | 1;
    $$sum$i13$i = (($653) + 4)|0;
    $657 = (($tbase$255$i) + ($$sum$i13$i)|0);
    HEAP32[$657>>2] = $656;
    $$sum2$i$i = (($tsize$254$i) + -36)|0;
    $658 = (($tbase$255$i) + ($$sum2$i$i)|0);
    HEAP32[$658>>2] = 40;
    $659 = HEAP32[(9516)>>2]|0;
    HEAP32[(9056)>>2] = $659;
   } else {
    $sp$084$i = (9476);
    while(1) {
     $660 = HEAP32[$sp$084$i>>2]|0;
     $661 = ((($sp$084$i)) + 4|0);
     $662 = HEAP32[$661>>2]|0;
     $663 = (($660) + ($662)|0);
     $664 = ($tbase$255$i|0)==($663|0);
     if ($664) {
      $$lcssa222 = $660;$$lcssa224 = $661;$$lcssa226 = $662;$sp$084$i$lcssa = $sp$084$i;
      label = 204;
      break;
     }
     $665 = ((($sp$084$i)) + 8|0);
     $666 = HEAP32[$665>>2]|0;
     $667 = ($666|0)==(0|0);
     if ($667) {
      break;
     } else {
      $sp$084$i = $666;
     }
    }
    if ((label|0) == 204) {
     $668 = ((($sp$084$i$lcssa)) + 12|0);
     $669 = HEAP32[$668>>2]|0;
     $670 = $669 & 8;
     $671 = ($670|0)==(0);
     if ($671) {
      $672 = ($635>>>0)>=($$lcssa222>>>0);
      $673 = ($635>>>0)<($tbase$255$i>>>0);
      $or$cond57$i = $673 & $672;
      if ($or$cond57$i) {
       $674 = (($$lcssa226) + ($tsize$254$i))|0;
       HEAP32[$$lcssa224>>2] = $674;
       $675 = HEAP32[(9040)>>2]|0;
       $676 = (($675) + ($tsize$254$i))|0;
       $677 = ((($635)) + 8|0);
       $678 = $677;
       $679 = $678 & 7;
       $680 = ($679|0)==(0);
       $681 = (0 - ($678))|0;
       $682 = $681 & 7;
       $683 = $680 ? 0 : $682;
       $684 = (($635) + ($683)|0);
       $685 = (($676) - ($683))|0;
       HEAP32[(9052)>>2] = $684;
       HEAP32[(9040)>>2] = $685;
       $686 = $685 | 1;
       $$sum$i17$i = (($683) + 4)|0;
       $687 = (($635) + ($$sum$i17$i)|0);
       HEAP32[$687>>2] = $686;
       $$sum2$i18$i = (($676) + 4)|0;
       $688 = (($635) + ($$sum2$i18$i)|0);
       HEAP32[$688>>2] = 40;
       $689 = HEAP32[(9516)>>2]|0;
       HEAP32[(9056)>>2] = $689;
       break;
      }
     }
    }
    $690 = HEAP32[(9044)>>2]|0;
    $691 = ($tbase$255$i>>>0)<($690>>>0);
    if ($691) {
     HEAP32[(9044)>>2] = $tbase$255$i;
     $755 = $tbase$255$i;
    } else {
     $755 = $690;
    }
    $692 = (($tbase$255$i) + ($tsize$254$i)|0);
    $sp$183$i = (9476);
    while(1) {
     $693 = HEAP32[$sp$183$i>>2]|0;
     $694 = ($693|0)==($692|0);
     if ($694) {
      $$lcssa219 = $sp$183$i;$sp$183$i$lcssa = $sp$183$i;
      label = 212;
      break;
     }
     $695 = ((($sp$183$i)) + 8|0);
     $696 = HEAP32[$695>>2]|0;
     $697 = ($696|0)==(0|0);
     if ($697) {
      $sp$0$i$i$i = (9476);
      break;
     } else {
      $sp$183$i = $696;
     }
    }
    if ((label|0) == 212) {
     $698 = ((($sp$183$i$lcssa)) + 12|0);
     $699 = HEAP32[$698>>2]|0;
     $700 = $699 & 8;
     $701 = ($700|0)==(0);
     if ($701) {
      HEAP32[$$lcssa219>>2] = $tbase$255$i;
      $702 = ((($sp$183$i$lcssa)) + 4|0);
      $703 = HEAP32[$702>>2]|0;
      $704 = (($703) + ($tsize$254$i))|0;
      HEAP32[$702>>2] = $704;
      $705 = ((($tbase$255$i)) + 8|0);
      $706 = $705;
      $707 = $706 & 7;
      $708 = ($707|0)==(0);
      $709 = (0 - ($706))|0;
      $710 = $709 & 7;
      $711 = $708 ? 0 : $710;
      $712 = (($tbase$255$i) + ($711)|0);
      $$sum112$i = (($tsize$254$i) + 8)|0;
      $713 = (($tbase$255$i) + ($$sum112$i)|0);
      $714 = $713;
      $715 = $714 & 7;
      $716 = ($715|0)==(0);
      $717 = (0 - ($714))|0;
      $718 = $717 & 7;
      $719 = $716 ? 0 : $718;
      $$sum113$i = (($719) + ($tsize$254$i))|0;
      $720 = (($tbase$255$i) + ($$sum113$i)|0);
      $721 = $720;
      $722 = $712;
      $723 = (($721) - ($722))|0;
      $$sum$i19$i = (($711) + ($nb$0))|0;
      $724 = (($tbase$255$i) + ($$sum$i19$i)|0);
      $725 = (($723) - ($nb$0))|0;
      $726 = $nb$0 | 3;
      $$sum1$i20$i = (($711) + 4)|0;
      $727 = (($tbase$255$i) + ($$sum1$i20$i)|0);
      HEAP32[$727>>2] = $726;
      $728 = ($720|0)==($635|0);
      L324: do {
       if ($728) {
        $729 = HEAP32[(9040)>>2]|0;
        $730 = (($729) + ($725))|0;
        HEAP32[(9040)>>2] = $730;
        HEAP32[(9052)>>2] = $724;
        $731 = $730 | 1;
        $$sum42$i$i = (($$sum$i19$i) + 4)|0;
        $732 = (($tbase$255$i) + ($$sum42$i$i)|0);
        HEAP32[$732>>2] = $731;
       } else {
        $733 = HEAP32[(9048)>>2]|0;
        $734 = ($720|0)==($733|0);
        if ($734) {
         $735 = HEAP32[(9036)>>2]|0;
         $736 = (($735) + ($725))|0;
         HEAP32[(9036)>>2] = $736;
         HEAP32[(9048)>>2] = $724;
         $737 = $736 | 1;
         $$sum40$i$i = (($$sum$i19$i) + 4)|0;
         $738 = (($tbase$255$i) + ($$sum40$i$i)|0);
         HEAP32[$738>>2] = $737;
         $$sum41$i$i = (($736) + ($$sum$i19$i))|0;
         $739 = (($tbase$255$i) + ($$sum41$i$i)|0);
         HEAP32[$739>>2] = $736;
         break;
        }
        $$sum2$i21$i = (($tsize$254$i) + 4)|0;
        $$sum114$i = (($$sum2$i21$i) + ($719))|0;
        $740 = (($tbase$255$i) + ($$sum114$i)|0);
        $741 = HEAP32[$740>>2]|0;
        $742 = $741 & 3;
        $743 = ($742|0)==(1);
        if ($743) {
         $744 = $741 & -8;
         $745 = $741 >>> 3;
         $746 = ($741>>>0)<(256);
         L332: do {
          if ($746) {
           $$sum3738$i$i = $719 | 8;
           $$sum124$i = (($$sum3738$i$i) + ($tsize$254$i))|0;
           $747 = (($tbase$255$i) + ($$sum124$i)|0);
           $748 = HEAP32[$747>>2]|0;
           $$sum39$i$i = (($tsize$254$i) + 12)|0;
           $$sum125$i = (($$sum39$i$i) + ($719))|0;
           $749 = (($tbase$255$i) + ($$sum125$i)|0);
           $750 = HEAP32[$749>>2]|0;
           $751 = $745 << 1;
           $752 = (9068 + ($751<<2)|0);
           $753 = ($748|0)==($752|0);
           do {
            if (!($753)) {
             $754 = ($748>>>0)<($755>>>0);
             if ($754) {
              _abort();
              // unreachable;
             }
             $756 = ((($748)) + 12|0);
             $757 = HEAP32[$756>>2]|0;
             $758 = ($757|0)==($720|0);
             if ($758) {
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $759 = ($750|0)==($748|0);
           if ($759) {
            $760 = 1 << $745;
            $761 = $760 ^ -1;
            $762 = HEAP32[9028>>2]|0;
            $763 = $762 & $761;
            HEAP32[9028>>2] = $763;
            break;
           }
           $764 = ($750|0)==($752|0);
           do {
            if ($764) {
             $$pre57$i$i = ((($750)) + 8|0);
             $$pre$phi58$i$iZ2D = $$pre57$i$i;
            } else {
             $765 = ($750>>>0)<($755>>>0);
             if ($765) {
              _abort();
              // unreachable;
             }
             $766 = ((($750)) + 8|0);
             $767 = HEAP32[$766>>2]|0;
             $768 = ($767|0)==($720|0);
             if ($768) {
              $$pre$phi58$i$iZ2D = $766;
              break;
             }
             _abort();
             // unreachable;
            }
           } while(0);
           $769 = ((($748)) + 12|0);
           HEAP32[$769>>2] = $750;
           HEAP32[$$pre$phi58$i$iZ2D>>2] = $748;
          } else {
           $$sum34$i$i = $719 | 24;
           $$sum115$i = (($$sum34$i$i) + ($tsize$254$i))|0;
           $770 = (($tbase$255$i) + ($$sum115$i)|0);
           $771 = HEAP32[$770>>2]|0;
           $$sum5$i$i = (($tsize$254$i) + 12)|0;
           $$sum116$i = (($$sum5$i$i) + ($719))|0;
           $772 = (($tbase$255$i) + ($$sum116$i)|0);
           $773 = HEAP32[$772>>2]|0;
           $774 = ($773|0)==($720|0);
           do {
            if ($774) {
             $$sum67$i$i = $719 | 16;
             $$sum122$i = (($$sum2$i21$i) + ($$sum67$i$i))|0;
             $784 = (($tbase$255$i) + ($$sum122$i)|0);
             $785 = HEAP32[$784>>2]|0;
             $786 = ($785|0)==(0|0);
             if ($786) {
              $$sum123$i = (($$sum67$i$i) + ($tsize$254$i))|0;
              $787 = (($tbase$255$i) + ($$sum123$i)|0);
              $788 = HEAP32[$787>>2]|0;
              $789 = ($788|0)==(0|0);
              if ($789) {
               $R$1$i$i = 0;
               break;
              } else {
               $R$0$i$i = $788;$RP$0$i$i = $787;
              }
             } else {
              $R$0$i$i = $785;$RP$0$i$i = $784;
             }
             while(1) {
              $790 = ((($R$0$i$i)) + 20|0);
              $791 = HEAP32[$790>>2]|0;
              $792 = ($791|0)==(0|0);
              if (!($792)) {
               $R$0$i$i = $791;$RP$0$i$i = $790;
               continue;
              }
              $793 = ((($R$0$i$i)) + 16|0);
              $794 = HEAP32[$793>>2]|0;
              $795 = ($794|0)==(0|0);
              if ($795) {
               $R$0$i$i$lcssa = $R$0$i$i;$RP$0$i$i$lcssa = $RP$0$i$i;
               break;
              } else {
               $R$0$i$i = $794;$RP$0$i$i = $793;
              }
             }
             $796 = ($RP$0$i$i$lcssa>>>0)<($755>>>0);
             if ($796) {
              _abort();
              // unreachable;
             } else {
              HEAP32[$RP$0$i$i$lcssa>>2] = 0;
              $R$1$i$i = $R$0$i$i$lcssa;
              break;
             }
            } else {
             $$sum3536$i$i = $719 | 8;
             $$sum117$i = (($$sum3536$i$i) + ($tsize$254$i))|0;
             $775 = (($tbase$255$i) + ($$sum117$i)|0);
             $776 = HEAP32[$775>>2]|0;
             $777 = ($776>>>0)<($755>>>0);
             if ($777) {
              _abort();
              // unreachable;
             }
             $778 = ((($776)) + 12|0);
             $779 = HEAP32[$778>>2]|0;
             $780 = ($779|0)==($720|0);
             if (!($780)) {
              _abort();
              // unreachable;
             }
             $781 = ((($773)) + 8|0);
             $782 = HEAP32[$781>>2]|0;
             $783 = ($782|0)==($720|0);
             if ($783) {
              HEAP32[$778>>2] = $773;
              HEAP32[$781>>2] = $776;
              $R$1$i$i = $773;
              break;
             } else {
              _abort();
              // unreachable;
             }
            }
           } while(0);
           $797 = ($771|0)==(0|0);
           if ($797) {
            break;
           }
           $$sum30$i$i = (($tsize$254$i) + 28)|0;
           $$sum118$i = (($$sum30$i$i) + ($719))|0;
           $798 = (($tbase$255$i) + ($$sum118$i)|0);
           $799 = HEAP32[$798>>2]|0;
           $800 = (9332 + ($799<<2)|0);
           $801 = HEAP32[$800>>2]|0;
           $802 = ($720|0)==($801|0);
           do {
            if ($802) {
             HEAP32[$800>>2] = $R$1$i$i;
             $cond$i$i = ($R$1$i$i|0)==(0|0);
             if (!($cond$i$i)) {
              break;
             }
             $803 = 1 << $799;
             $804 = $803 ^ -1;
             $805 = HEAP32[(9032)>>2]|0;
             $806 = $805 & $804;
             HEAP32[(9032)>>2] = $806;
             break L332;
            } else {
             $807 = HEAP32[(9044)>>2]|0;
             $808 = ($771>>>0)<($807>>>0);
             if ($808) {
              _abort();
              // unreachable;
             }
             $809 = ((($771)) + 16|0);
             $810 = HEAP32[$809>>2]|0;
             $811 = ($810|0)==($720|0);
             if ($811) {
              HEAP32[$809>>2] = $R$1$i$i;
             } else {
              $812 = ((($771)) + 20|0);
              HEAP32[$812>>2] = $R$1$i$i;
             }
             $813 = ($R$1$i$i|0)==(0|0);
             if ($813) {
              break L332;
             }
            }
           } while(0);
           $814 = HEAP32[(9044)>>2]|0;
           $815 = ($R$1$i$i>>>0)<($814>>>0);
           if ($815) {
            _abort();
            // unreachable;
           }
           $816 = ((($R$1$i$i)) + 24|0);
           HEAP32[$816>>2] = $771;
           $$sum3132$i$i = $719 | 16;
           $$sum119$i = (($$sum3132$i$i) + ($tsize$254$i))|0;
           $817 = (($tbase$255$i) + ($$sum119$i)|0);
           $818 = HEAP32[$817>>2]|0;
           $819 = ($818|0)==(0|0);
           do {
            if (!($819)) {
             $820 = ($818>>>0)<($814>>>0);
             if ($820) {
              _abort();
              // unreachable;
             } else {
              $821 = ((($R$1$i$i)) + 16|0);
              HEAP32[$821>>2] = $818;
              $822 = ((($818)) + 24|0);
              HEAP32[$822>>2] = $R$1$i$i;
              break;
             }
            }
           } while(0);
           $$sum120$i = (($$sum2$i21$i) + ($$sum3132$i$i))|0;
           $823 = (($tbase$255$i) + ($$sum120$i)|0);
           $824 = HEAP32[$823>>2]|0;
           $825 = ($824|0)==(0|0);
           if ($825) {
            break;
           }
           $826 = HEAP32[(9044)>>2]|0;
           $827 = ($824>>>0)<($826>>>0);
           if ($827) {
            _abort();
            // unreachable;
           } else {
            $828 = ((($R$1$i$i)) + 20|0);
            HEAP32[$828>>2] = $824;
            $829 = ((($824)) + 24|0);
            HEAP32[$829>>2] = $R$1$i$i;
            break;
           }
          }
         } while(0);
         $$sum9$i$i = $744 | $719;
         $$sum121$i = (($$sum9$i$i) + ($tsize$254$i))|0;
         $830 = (($tbase$255$i) + ($$sum121$i)|0);
         $831 = (($744) + ($725))|0;
         $oldfirst$0$i$i = $830;$qsize$0$i$i = $831;
        } else {
         $oldfirst$0$i$i = $720;$qsize$0$i$i = $725;
        }
        $832 = ((($oldfirst$0$i$i)) + 4|0);
        $833 = HEAP32[$832>>2]|0;
        $834 = $833 & -2;
        HEAP32[$832>>2] = $834;
        $835 = $qsize$0$i$i | 1;
        $$sum10$i$i = (($$sum$i19$i) + 4)|0;
        $836 = (($tbase$255$i) + ($$sum10$i$i)|0);
        HEAP32[$836>>2] = $835;
        $$sum11$i$i = (($qsize$0$i$i) + ($$sum$i19$i))|0;
        $837 = (($tbase$255$i) + ($$sum11$i$i)|0);
        HEAP32[$837>>2] = $qsize$0$i$i;
        $838 = $qsize$0$i$i >>> 3;
        $839 = ($qsize$0$i$i>>>0)<(256);
        if ($839) {
         $840 = $838 << 1;
         $841 = (9068 + ($840<<2)|0);
         $842 = HEAP32[9028>>2]|0;
         $843 = 1 << $838;
         $844 = $842 & $843;
         $845 = ($844|0)==(0);
         do {
          if ($845) {
           $846 = $842 | $843;
           HEAP32[9028>>2] = $846;
           $$pre$i22$i = (($840) + 2)|0;
           $$pre56$i$i = (9068 + ($$pre$i22$i<<2)|0);
           $$pre$phi$i23$iZ2D = $$pre56$i$i;$F4$0$i$i = $841;
          } else {
           $$sum29$i$i = (($840) + 2)|0;
           $847 = (9068 + ($$sum29$i$i<<2)|0);
           $848 = HEAP32[$847>>2]|0;
           $849 = HEAP32[(9044)>>2]|0;
           $850 = ($848>>>0)<($849>>>0);
           if (!($850)) {
            $$pre$phi$i23$iZ2D = $847;$F4$0$i$i = $848;
            break;
           }
           _abort();
           // unreachable;
          }
         } while(0);
         HEAP32[$$pre$phi$i23$iZ2D>>2] = $724;
         $851 = ((($F4$0$i$i)) + 12|0);
         HEAP32[$851>>2] = $724;
         $$sum27$i$i = (($$sum$i19$i) + 8)|0;
         $852 = (($tbase$255$i) + ($$sum27$i$i)|0);
         HEAP32[$852>>2] = $F4$0$i$i;
         $$sum28$i$i = (($$sum$i19$i) + 12)|0;
         $853 = (($tbase$255$i) + ($$sum28$i$i)|0);
         HEAP32[$853>>2] = $841;
         break;
        }
        $854 = $qsize$0$i$i >>> 8;
        $855 = ($854|0)==(0);
        do {
         if ($855) {
          $I7$0$i$i = 0;
         } else {
          $856 = ($qsize$0$i$i>>>0)>(16777215);
          if ($856) {
           $I7$0$i$i = 31;
           break;
          }
          $857 = (($854) + 1048320)|0;
          $858 = $857 >>> 16;
          $859 = $858 & 8;
          $860 = $854 << $859;
          $861 = (($860) + 520192)|0;
          $862 = $861 >>> 16;
          $863 = $862 & 4;
          $864 = $863 | $859;
          $865 = $860 << $863;
          $866 = (($865) + 245760)|0;
          $867 = $866 >>> 16;
          $868 = $867 & 2;
          $869 = $864 | $868;
          $870 = (14 - ($869))|0;
          $871 = $865 << $868;
          $872 = $871 >>> 15;
          $873 = (($870) + ($872))|0;
          $874 = $873 << 1;
          $875 = (($873) + 7)|0;
          $876 = $qsize$0$i$i >>> $875;
          $877 = $876 & 1;
          $878 = $877 | $874;
          $I7$0$i$i = $878;
         }
        } while(0);
        $879 = (9332 + ($I7$0$i$i<<2)|0);
        $$sum12$i$i = (($$sum$i19$i) + 28)|0;
        $880 = (($tbase$255$i) + ($$sum12$i$i)|0);
        HEAP32[$880>>2] = $I7$0$i$i;
        $$sum13$i$i = (($$sum$i19$i) + 16)|0;
        $881 = (($tbase$255$i) + ($$sum13$i$i)|0);
        $$sum14$i$i = (($$sum$i19$i) + 20)|0;
        $882 = (($tbase$255$i) + ($$sum14$i$i)|0);
        HEAP32[$882>>2] = 0;
        HEAP32[$881>>2] = 0;
        $883 = HEAP32[(9032)>>2]|0;
        $884 = 1 << $I7$0$i$i;
        $885 = $883 & $884;
        $886 = ($885|0)==(0);
        if ($886) {
         $887 = $883 | $884;
         HEAP32[(9032)>>2] = $887;
         HEAP32[$879>>2] = $724;
         $$sum15$i$i = (($$sum$i19$i) + 24)|0;
         $888 = (($tbase$255$i) + ($$sum15$i$i)|0);
         HEAP32[$888>>2] = $879;
         $$sum16$i$i = (($$sum$i19$i) + 12)|0;
         $889 = (($tbase$255$i) + ($$sum16$i$i)|0);
         HEAP32[$889>>2] = $724;
         $$sum17$i$i = (($$sum$i19$i) + 8)|0;
         $890 = (($tbase$255$i) + ($$sum17$i$i)|0);
         HEAP32[$890>>2] = $724;
         break;
        }
        $891 = HEAP32[$879>>2]|0;
        $892 = ((($891)) + 4|0);
        $893 = HEAP32[$892>>2]|0;
        $894 = $893 & -8;
        $895 = ($894|0)==($qsize$0$i$i|0);
        L418: do {
         if ($895) {
          $T$0$lcssa$i25$i = $891;
         } else {
          $896 = ($I7$0$i$i|0)==(31);
          $897 = $I7$0$i$i >>> 1;
          $898 = (25 - ($897))|0;
          $899 = $896 ? 0 : $898;
          $900 = $qsize$0$i$i << $899;
          $K8$051$i$i = $900;$T$050$i$i = $891;
          while(1) {
           $907 = $K8$051$i$i >>> 31;
           $908 = (((($T$050$i$i)) + 16|0) + ($907<<2)|0);
           $903 = HEAP32[$908>>2]|0;
           $909 = ($903|0)==(0|0);
           if ($909) {
            $$lcssa = $908;$T$050$i$i$lcssa = $T$050$i$i;
            break;
           }
           $901 = $K8$051$i$i << 1;
           $902 = ((($903)) + 4|0);
           $904 = HEAP32[$902>>2]|0;
           $905 = $904 & -8;
           $906 = ($905|0)==($qsize$0$i$i|0);
           if ($906) {
            $T$0$lcssa$i25$i = $903;
            break L418;
           } else {
            $K8$051$i$i = $901;$T$050$i$i = $903;
           }
          }
          $910 = HEAP32[(9044)>>2]|0;
          $911 = ($$lcssa>>>0)<($910>>>0);
          if ($911) {
           _abort();
           // unreachable;
          } else {
           HEAP32[$$lcssa>>2] = $724;
           $$sum23$i$i = (($$sum$i19$i) + 24)|0;
           $912 = (($tbase$255$i) + ($$sum23$i$i)|0);
           HEAP32[$912>>2] = $T$050$i$i$lcssa;
           $$sum24$i$i = (($$sum$i19$i) + 12)|0;
           $913 = (($tbase$255$i) + ($$sum24$i$i)|0);
           HEAP32[$913>>2] = $724;
           $$sum25$i$i = (($$sum$i19$i) + 8)|0;
           $914 = (($tbase$255$i) + ($$sum25$i$i)|0);
           HEAP32[$914>>2] = $724;
           break L324;
          }
         }
        } while(0);
        $915 = ((($T$0$lcssa$i25$i)) + 8|0);
        $916 = HEAP32[$915>>2]|0;
        $917 = HEAP32[(9044)>>2]|0;
        $918 = ($916>>>0)>=($917>>>0);
        $not$$i26$i = ($T$0$lcssa$i25$i>>>0)>=($917>>>0);
        $919 = $918 & $not$$i26$i;
        if ($919) {
         $920 = ((($916)) + 12|0);
         HEAP32[$920>>2] = $724;
         HEAP32[$915>>2] = $724;
         $$sum20$i$i = (($$sum$i19$i) + 8)|0;
         $921 = (($tbase$255$i) + ($$sum20$i$i)|0);
         HEAP32[$921>>2] = $916;
         $$sum21$i$i = (($$sum$i19$i) + 12)|0;
         $922 = (($tbase$255$i) + ($$sum21$i$i)|0);
         HEAP32[$922>>2] = $T$0$lcssa$i25$i;
         $$sum22$i$i = (($$sum$i19$i) + 24)|0;
         $923 = (($tbase$255$i) + ($$sum22$i$i)|0);
         HEAP32[$923>>2] = 0;
         break;
        } else {
         _abort();
         // unreachable;
        }
       }
      } while(0);
      $$sum1819$i$i = $711 | 8;
      $924 = (($tbase$255$i) + ($$sum1819$i$i)|0);
      $mem$0 = $924;
      return ($mem$0|0);
     } else {
      $sp$0$i$i$i = (9476);
     }
    }
    while(1) {
     $925 = HEAP32[$sp$0$i$i$i>>2]|0;
     $926 = ($925>>>0)>($635>>>0);
     if (!($926)) {
      $927 = ((($sp$0$i$i$i)) + 4|0);
      $928 = HEAP32[$927>>2]|0;
      $929 = (($925) + ($928)|0);
      $930 = ($929>>>0)>($635>>>0);
      if ($930) {
       $$lcssa215 = $925;$$lcssa216 = $928;$$lcssa217 = $929;
       break;
      }
     }
     $931 = ((($sp$0$i$i$i)) + 8|0);
     $932 = HEAP32[$931>>2]|0;
     $sp$0$i$i$i = $932;
    }
    $$sum$i14$i = (($$lcssa216) + -47)|0;
    $$sum1$i15$i = (($$lcssa216) + -39)|0;
    $933 = (($$lcssa215) + ($$sum1$i15$i)|0);
    $934 = $933;
    $935 = $934 & 7;
    $936 = ($935|0)==(0);
    $937 = (0 - ($934))|0;
    $938 = $937 & 7;
    $939 = $936 ? 0 : $938;
    $$sum2$i16$i = (($$sum$i14$i) + ($939))|0;
    $940 = (($$lcssa215) + ($$sum2$i16$i)|0);
    $941 = ((($635)) + 16|0);
    $942 = ($940>>>0)<($941>>>0);
    $943 = $942 ? $635 : $940;
    $944 = ((($943)) + 8|0);
    $945 = (($tsize$254$i) + -40)|0;
    $946 = ((($tbase$255$i)) + 8|0);
    $947 = $946;
    $948 = $947 & 7;
    $949 = ($948|0)==(0);
    $950 = (0 - ($947))|0;
    $951 = $950 & 7;
    $952 = $949 ? 0 : $951;
    $953 = (($tbase$255$i) + ($952)|0);
    $954 = (($945) - ($952))|0;
    HEAP32[(9052)>>2] = $953;
    HEAP32[(9040)>>2] = $954;
    $955 = $954 | 1;
    $$sum$i$i$i = (($952) + 4)|0;
    $956 = (($tbase$255$i) + ($$sum$i$i$i)|0);
    HEAP32[$956>>2] = $955;
    $$sum2$i$i$i = (($tsize$254$i) + -36)|0;
    $957 = (($tbase$255$i) + ($$sum2$i$i$i)|0);
    HEAP32[$957>>2] = 40;
    $958 = HEAP32[(9516)>>2]|0;
    HEAP32[(9056)>>2] = $958;
    $959 = ((($943)) + 4|0);
    HEAP32[$959>>2] = 27;
    ;HEAP32[$944>>2]=HEAP32[(9476)>>2]|0;HEAP32[$944+4>>2]=HEAP32[(9476)+4>>2]|0;HEAP32[$944+8>>2]=HEAP32[(9476)+8>>2]|0;HEAP32[$944+12>>2]=HEAP32[(9476)+12>>2]|0;
    HEAP32[(9476)>>2] = $tbase$255$i;
    HEAP32[(9480)>>2] = $tsize$254$i;
    HEAP32[(9488)>>2] = 0;
    HEAP32[(9484)>>2] = $944;
    $960 = ((($943)) + 28|0);
    HEAP32[$960>>2] = 7;
    $961 = ((($943)) + 32|0);
    $962 = ($961>>>0)<($$lcssa217>>>0);
    if ($962) {
     $964 = $960;
     while(1) {
      $963 = ((($964)) + 4|0);
      HEAP32[$963>>2] = 7;
      $965 = ((($964)) + 8|0);
      $966 = ($965>>>0)<($$lcssa217>>>0);
      if ($966) {
       $964 = $963;
      } else {
       break;
      }
     }
    }
    $967 = ($943|0)==($635|0);
    if (!($967)) {
     $968 = $943;
     $969 = $635;
     $970 = (($968) - ($969))|0;
     $971 = HEAP32[$959>>2]|0;
     $972 = $971 & -2;
     HEAP32[$959>>2] = $972;
     $973 = $970 | 1;
     $974 = ((($635)) + 4|0);
     HEAP32[$974>>2] = $973;
     HEAP32[$943>>2] = $970;
     $975 = $970 >>> 3;
     $976 = ($970>>>0)<(256);
     if ($976) {
      $977 = $975 << 1;
      $978 = (9068 + ($977<<2)|0);
      $979 = HEAP32[9028>>2]|0;
      $980 = 1 << $975;
      $981 = $979 & $980;
      $982 = ($981|0)==(0);
      if ($982) {
       $983 = $979 | $980;
       HEAP32[9028>>2] = $983;
       $$pre$i$i = (($977) + 2)|0;
       $$pre14$i$i = (9068 + ($$pre$i$i<<2)|0);
       $$pre$phi$i$iZ2D = $$pre14$i$i;$F$0$i$i = $978;
      } else {
       $$sum4$i$i = (($977) + 2)|0;
       $984 = (9068 + ($$sum4$i$i<<2)|0);
       $985 = HEAP32[$984>>2]|0;
       $986 = HEAP32[(9044)>>2]|0;
       $987 = ($985>>>0)<($986>>>0);
       if ($987) {
        _abort();
        // unreachable;
       } else {
        $$pre$phi$i$iZ2D = $984;$F$0$i$i = $985;
       }
      }
      HEAP32[$$pre$phi$i$iZ2D>>2] = $635;
      $988 = ((($F$0$i$i)) + 12|0);
      HEAP32[$988>>2] = $635;
      $989 = ((($635)) + 8|0);
      HEAP32[$989>>2] = $F$0$i$i;
      $990 = ((($635)) + 12|0);
      HEAP32[$990>>2] = $978;
      break;
     }
     $991 = $970 >>> 8;
     $992 = ($991|0)==(0);
     if ($992) {
      $I1$0$i$i = 0;
     } else {
      $993 = ($970>>>0)>(16777215);
      if ($993) {
       $I1$0$i$i = 31;
      } else {
       $994 = (($991) + 1048320)|0;
       $995 = $994 >>> 16;
       $996 = $995 & 8;
       $997 = $991 << $996;
       $998 = (($997) + 520192)|0;
       $999 = $998 >>> 16;
       $1000 = $999 & 4;
       $1001 = $1000 | $996;
       $1002 = $997 << $1000;
       $1003 = (($1002) + 245760)|0;
       $1004 = $1003 >>> 16;
       $1005 = $1004 & 2;
       $1006 = $1001 | $1005;
       $1007 = (14 - ($1006))|0;
       $1008 = $1002 << $1005;
       $1009 = $1008 >>> 15;
       $1010 = (($1007) + ($1009))|0;
       $1011 = $1010 << 1;
       $1012 = (($1010) + 7)|0;
       $1013 = $970 >>> $1012;
       $1014 = $1013 & 1;
       $1015 = $1014 | $1011;
       $I1$0$i$i = $1015;
      }
     }
     $1016 = (9332 + ($I1$0$i$i<<2)|0);
     $1017 = ((($635)) + 28|0);
     HEAP32[$1017>>2] = $I1$0$i$i;
     $1018 = ((($635)) + 20|0);
     HEAP32[$1018>>2] = 0;
     HEAP32[$941>>2] = 0;
     $1019 = HEAP32[(9032)>>2]|0;
     $1020 = 1 << $I1$0$i$i;
     $1021 = $1019 & $1020;
     $1022 = ($1021|0)==(0);
     if ($1022) {
      $1023 = $1019 | $1020;
      HEAP32[(9032)>>2] = $1023;
      HEAP32[$1016>>2] = $635;
      $1024 = ((($635)) + 24|0);
      HEAP32[$1024>>2] = $1016;
      $1025 = ((($635)) + 12|0);
      HEAP32[$1025>>2] = $635;
      $1026 = ((($635)) + 8|0);
      HEAP32[$1026>>2] = $635;
      break;
     }
     $1027 = HEAP32[$1016>>2]|0;
     $1028 = ((($1027)) + 4|0);
     $1029 = HEAP32[$1028>>2]|0;
     $1030 = $1029 & -8;
     $1031 = ($1030|0)==($970|0);
     L459: do {
      if ($1031) {
       $T$0$lcssa$i$i = $1027;
      } else {
       $1032 = ($I1$0$i$i|0)==(31);
       $1033 = $I1$0$i$i >>> 1;
       $1034 = (25 - ($1033))|0;
       $1035 = $1032 ? 0 : $1034;
       $1036 = $970 << $1035;
       $K2$07$i$i = $1036;$T$06$i$i = $1027;
       while(1) {
        $1043 = $K2$07$i$i >>> 31;
        $1044 = (((($T$06$i$i)) + 16|0) + ($1043<<2)|0);
        $1039 = HEAP32[$1044>>2]|0;
        $1045 = ($1039|0)==(0|0);
        if ($1045) {
         $$lcssa211 = $1044;$T$06$i$i$lcssa = $T$06$i$i;
         break;
        }
        $1037 = $K2$07$i$i << 1;
        $1038 = ((($1039)) + 4|0);
        $1040 = HEAP32[$1038>>2]|0;
        $1041 = $1040 & -8;
        $1042 = ($1041|0)==($970|0);
        if ($1042) {
         $T$0$lcssa$i$i = $1039;
         break L459;
        } else {
         $K2$07$i$i = $1037;$T$06$i$i = $1039;
        }
       }
       $1046 = HEAP32[(9044)>>2]|0;
       $1047 = ($$lcssa211>>>0)<($1046>>>0);
       if ($1047) {
        _abort();
        // unreachable;
       } else {
        HEAP32[$$lcssa211>>2] = $635;
        $1048 = ((($635)) + 24|0);
        HEAP32[$1048>>2] = $T$06$i$i$lcssa;
        $1049 = ((($635)) + 12|0);
        HEAP32[$1049>>2] = $635;
        $1050 = ((($635)) + 8|0);
        HEAP32[$1050>>2] = $635;
        break L299;
       }
      }
     } while(0);
     $1051 = ((($T$0$lcssa$i$i)) + 8|0);
     $1052 = HEAP32[$1051>>2]|0;
     $1053 = HEAP32[(9044)>>2]|0;
     $1054 = ($1052>>>0)>=($1053>>>0);
     $not$$i$i = ($T$0$lcssa$i$i>>>0)>=($1053>>>0);
     $1055 = $1054 & $not$$i$i;
     if ($1055) {
      $1056 = ((($1052)) + 12|0);
      HEAP32[$1056>>2] = $635;
      HEAP32[$1051>>2] = $635;
      $1057 = ((($635)) + 8|0);
      HEAP32[$1057>>2] = $1052;
      $1058 = ((($635)) + 12|0);
      HEAP32[$1058>>2] = $T$0$lcssa$i$i;
      $1059 = ((($635)) + 24|0);
      HEAP32[$1059>>2] = 0;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   }
  } while(0);
  $1060 = HEAP32[(9040)>>2]|0;
  $1061 = ($1060>>>0)>($nb$0>>>0);
  if ($1061) {
   $1062 = (($1060) - ($nb$0))|0;
   HEAP32[(9040)>>2] = $1062;
   $1063 = HEAP32[(9052)>>2]|0;
   $1064 = (($1063) + ($nb$0)|0);
   HEAP32[(9052)>>2] = $1064;
   $1065 = $1062 | 1;
   $$sum$i32 = (($nb$0) + 4)|0;
   $1066 = (($1063) + ($$sum$i32)|0);
   HEAP32[$1066>>2] = $1065;
   $1067 = $nb$0 | 3;
   $1068 = ((($1063)) + 4|0);
   HEAP32[$1068>>2] = $1067;
   $1069 = ((($1063)) + 8|0);
   $mem$0 = $1069;
   return ($mem$0|0);
  }
 }
 $1070 = (___errno_location()|0);
 HEAP32[$1070>>2] = 12;
 $mem$0 = 0;
 return ($mem$0|0);
}
function _free($mem) {
 $mem = $mem|0;
 var $$lcssa = 0, $$pre = 0, $$pre$phi59Z2D = 0, $$pre$phi61Z2D = 0, $$pre$phiZ2D = 0, $$pre57 = 0, $$pre58 = 0, $$pre60 = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum1718 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0;
 var $$sum25 = 0, $$sum26 = 0, $$sum27 = 0, $$sum28 = 0, $$sum29 = 0, $$sum3 = 0, $$sum30 = 0, $$sum31 = 0, $$sum5 = 0, $$sum67 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0;
 var $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0;
 var $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0;
 var $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0;
 var $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0, $175 = 0, $176 = 0, $177 = 0;
 var $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0, $193 = 0, $194 = 0, $195 = 0;
 var $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0, $210 = 0, $211 = 0, $212 = 0;
 var $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0, $229 = 0, $23 = 0, $230 = 0;
 var $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0, $247 = 0, $248 = 0, $249 = 0;
 var $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0, $265 = 0, $266 = 0, $267 = 0;
 var $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0, $283 = 0, $284 = 0, $285 = 0;
 var $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0, $300 = 0, $301 = 0, $302 = 0;
 var $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $308 = 0, $309 = 0, $31 = 0, $310 = 0, $311 = 0, $312 = 0, $313 = 0, $314 = 0, $315 = 0, $316 = 0, $317 = 0, $318 = 0, $319 = 0, $32 = 0, $320 = 0;
 var $321 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0;
 var $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0;
 var $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0;
 var $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I18$0 = 0, $K19$052 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0;
 var $R7$0 = 0, $R7$0$lcssa = 0, $R7$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $RP9$0 = 0, $RP9$0$lcssa = 0, $T$0$lcssa = 0, $T$051 = 0, $T$051$lcssa = 0, $cond = 0, $cond47 = 0, $not$ = 0, $p$0 = 0, $psize$0 = 0, $psize$1 = 0, $sp$0$i = 0, $sp$0$in$i = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($mem|0)==(0|0);
 if ($0) {
  return;
 }
 $1 = ((($mem)) + -8|0);
 $2 = HEAP32[(9044)>>2]|0;
 $3 = ($1>>>0)<($2>>>0);
 if ($3) {
  _abort();
  // unreachable;
 }
 $4 = ((($mem)) + -4|0);
 $5 = HEAP32[$4>>2]|0;
 $6 = $5 & 3;
 $7 = ($6|0)==(1);
 if ($7) {
  _abort();
  // unreachable;
 }
 $8 = $5 & -8;
 $$sum = (($8) + -8)|0;
 $9 = (($mem) + ($$sum)|0);
 $10 = $5 & 1;
 $11 = ($10|0)==(0);
 do {
  if ($11) {
   $12 = HEAP32[$1>>2]|0;
   $13 = ($6|0)==(0);
   if ($13) {
    return;
   }
   $$sum2 = (-8 - ($12))|0;
   $14 = (($mem) + ($$sum2)|0);
   $15 = (($12) + ($8))|0;
   $16 = ($14>>>0)<($2>>>0);
   if ($16) {
    _abort();
    // unreachable;
   }
   $17 = HEAP32[(9048)>>2]|0;
   $18 = ($14|0)==($17|0);
   if ($18) {
    $$sum3 = (($8) + -4)|0;
    $103 = (($mem) + ($$sum3)|0);
    $104 = HEAP32[$103>>2]|0;
    $105 = $104 & 3;
    $106 = ($105|0)==(3);
    if (!($106)) {
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    HEAP32[(9036)>>2] = $15;
    $107 = $104 & -2;
    HEAP32[$103>>2] = $107;
    $108 = $15 | 1;
    $$sum20 = (($$sum2) + 4)|0;
    $109 = (($mem) + ($$sum20)|0);
    HEAP32[$109>>2] = $108;
    HEAP32[$9>>2] = $15;
    return;
   }
   $19 = $12 >>> 3;
   $20 = ($12>>>0)<(256);
   if ($20) {
    $$sum30 = (($$sum2) + 8)|0;
    $21 = (($mem) + ($$sum30)|0);
    $22 = HEAP32[$21>>2]|0;
    $$sum31 = (($$sum2) + 12)|0;
    $23 = (($mem) + ($$sum31)|0);
    $24 = HEAP32[$23>>2]|0;
    $25 = $19 << 1;
    $26 = (9068 + ($25<<2)|0);
    $27 = ($22|0)==($26|0);
    if (!($27)) {
     $28 = ($22>>>0)<($2>>>0);
     if ($28) {
      _abort();
      // unreachable;
     }
     $29 = ((($22)) + 12|0);
     $30 = HEAP32[$29>>2]|0;
     $31 = ($30|0)==($14|0);
     if (!($31)) {
      _abort();
      // unreachable;
     }
    }
    $32 = ($24|0)==($22|0);
    if ($32) {
     $33 = 1 << $19;
     $34 = $33 ^ -1;
     $35 = HEAP32[9028>>2]|0;
     $36 = $35 & $34;
     HEAP32[9028>>2] = $36;
     $p$0 = $14;$psize$0 = $15;
     break;
    }
    $37 = ($24|0)==($26|0);
    if ($37) {
     $$pre60 = ((($24)) + 8|0);
     $$pre$phi61Z2D = $$pre60;
    } else {
     $38 = ($24>>>0)<($2>>>0);
     if ($38) {
      _abort();
      // unreachable;
     }
     $39 = ((($24)) + 8|0);
     $40 = HEAP32[$39>>2]|0;
     $41 = ($40|0)==($14|0);
     if ($41) {
      $$pre$phi61Z2D = $39;
     } else {
      _abort();
      // unreachable;
     }
    }
    $42 = ((($22)) + 12|0);
    HEAP32[$42>>2] = $24;
    HEAP32[$$pre$phi61Z2D>>2] = $22;
    $p$0 = $14;$psize$0 = $15;
    break;
   }
   $$sum22 = (($$sum2) + 24)|0;
   $43 = (($mem) + ($$sum22)|0);
   $44 = HEAP32[$43>>2]|0;
   $$sum23 = (($$sum2) + 12)|0;
   $45 = (($mem) + ($$sum23)|0);
   $46 = HEAP32[$45>>2]|0;
   $47 = ($46|0)==($14|0);
   do {
    if ($47) {
     $$sum25 = (($$sum2) + 20)|0;
     $57 = (($mem) + ($$sum25)|0);
     $58 = HEAP32[$57>>2]|0;
     $59 = ($58|0)==(0|0);
     if ($59) {
      $$sum24 = (($$sum2) + 16)|0;
      $60 = (($mem) + ($$sum24)|0);
      $61 = HEAP32[$60>>2]|0;
      $62 = ($61|0)==(0|0);
      if ($62) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $61;$RP$0 = $60;
      }
     } else {
      $R$0 = $58;$RP$0 = $57;
     }
     while(1) {
      $63 = ((($R$0)) + 20|0);
      $64 = HEAP32[$63>>2]|0;
      $65 = ($64|0)==(0|0);
      if (!($65)) {
       $R$0 = $64;$RP$0 = $63;
       continue;
      }
      $66 = ((($R$0)) + 16|0);
      $67 = HEAP32[$66>>2]|0;
      $68 = ($67|0)==(0|0);
      if ($68) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $67;$RP$0 = $66;
      }
     }
     $69 = ($RP$0$lcssa>>>0)<($2>>>0);
     if ($69) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum29 = (($$sum2) + 8)|0;
     $48 = (($mem) + ($$sum29)|0);
     $49 = HEAP32[$48>>2]|0;
     $50 = ($49>>>0)<($2>>>0);
     if ($50) {
      _abort();
      // unreachable;
     }
     $51 = ((($49)) + 12|0);
     $52 = HEAP32[$51>>2]|0;
     $53 = ($52|0)==($14|0);
     if (!($53)) {
      _abort();
      // unreachable;
     }
     $54 = ((($46)) + 8|0);
     $55 = HEAP32[$54>>2]|0;
     $56 = ($55|0)==($14|0);
     if ($56) {
      HEAP32[$51>>2] = $46;
      HEAP32[$54>>2] = $49;
      $R$1 = $46;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $70 = ($44|0)==(0|0);
   if ($70) {
    $p$0 = $14;$psize$0 = $15;
   } else {
    $$sum26 = (($$sum2) + 28)|0;
    $71 = (($mem) + ($$sum26)|0);
    $72 = HEAP32[$71>>2]|0;
    $73 = (9332 + ($72<<2)|0);
    $74 = HEAP32[$73>>2]|0;
    $75 = ($14|0)==($74|0);
    if ($75) {
     HEAP32[$73>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $76 = 1 << $72;
      $77 = $76 ^ -1;
      $78 = HEAP32[(9032)>>2]|0;
      $79 = $78 & $77;
      HEAP32[(9032)>>2] = $79;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    } else {
     $80 = HEAP32[(9044)>>2]|0;
     $81 = ($44>>>0)<($80>>>0);
     if ($81) {
      _abort();
      // unreachable;
     }
     $82 = ((($44)) + 16|0);
     $83 = HEAP32[$82>>2]|0;
     $84 = ($83|0)==($14|0);
     if ($84) {
      HEAP32[$82>>2] = $R$1;
     } else {
      $85 = ((($44)) + 20|0);
      HEAP32[$85>>2] = $R$1;
     }
     $86 = ($R$1|0)==(0|0);
     if ($86) {
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
    $87 = HEAP32[(9044)>>2]|0;
    $88 = ($R$1>>>0)<($87>>>0);
    if ($88) {
     _abort();
     // unreachable;
    }
    $89 = ((($R$1)) + 24|0);
    HEAP32[$89>>2] = $44;
    $$sum27 = (($$sum2) + 16)|0;
    $90 = (($mem) + ($$sum27)|0);
    $91 = HEAP32[$90>>2]|0;
    $92 = ($91|0)==(0|0);
    do {
     if (!($92)) {
      $93 = ($91>>>0)<($87>>>0);
      if ($93) {
       _abort();
       // unreachable;
      } else {
       $94 = ((($R$1)) + 16|0);
       HEAP32[$94>>2] = $91;
       $95 = ((($91)) + 24|0);
       HEAP32[$95>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum28 = (($$sum2) + 20)|0;
    $96 = (($mem) + ($$sum28)|0);
    $97 = HEAP32[$96>>2]|0;
    $98 = ($97|0)==(0|0);
    if ($98) {
     $p$0 = $14;$psize$0 = $15;
    } else {
     $99 = HEAP32[(9044)>>2]|0;
     $100 = ($97>>>0)<($99>>>0);
     if ($100) {
      _abort();
      // unreachable;
     } else {
      $101 = ((($R$1)) + 20|0);
      HEAP32[$101>>2] = $97;
      $102 = ((($97)) + 24|0);
      HEAP32[$102>>2] = $R$1;
      $p$0 = $14;$psize$0 = $15;
      break;
     }
    }
   }
  } else {
   $p$0 = $1;$psize$0 = $8;
  }
 } while(0);
 $110 = ($p$0>>>0)<($9>>>0);
 if (!($110)) {
  _abort();
  // unreachable;
 }
 $$sum19 = (($8) + -4)|0;
 $111 = (($mem) + ($$sum19)|0);
 $112 = HEAP32[$111>>2]|0;
 $113 = $112 & 1;
 $114 = ($113|0)==(0);
 if ($114) {
  _abort();
  // unreachable;
 }
 $115 = $112 & 2;
 $116 = ($115|0)==(0);
 if ($116) {
  $117 = HEAP32[(9052)>>2]|0;
  $118 = ($9|0)==($117|0);
  if ($118) {
   $119 = HEAP32[(9040)>>2]|0;
   $120 = (($119) + ($psize$0))|0;
   HEAP32[(9040)>>2] = $120;
   HEAP32[(9052)>>2] = $p$0;
   $121 = $120 | 1;
   $122 = ((($p$0)) + 4|0);
   HEAP32[$122>>2] = $121;
   $123 = HEAP32[(9048)>>2]|0;
   $124 = ($p$0|0)==($123|0);
   if (!($124)) {
    return;
   }
   HEAP32[(9048)>>2] = 0;
   HEAP32[(9036)>>2] = 0;
   return;
  }
  $125 = HEAP32[(9048)>>2]|0;
  $126 = ($9|0)==($125|0);
  if ($126) {
   $127 = HEAP32[(9036)>>2]|0;
   $128 = (($127) + ($psize$0))|0;
   HEAP32[(9036)>>2] = $128;
   HEAP32[(9048)>>2] = $p$0;
   $129 = $128 | 1;
   $130 = ((($p$0)) + 4|0);
   HEAP32[$130>>2] = $129;
   $131 = (($p$0) + ($128)|0);
   HEAP32[$131>>2] = $128;
   return;
  }
  $132 = $112 & -8;
  $133 = (($132) + ($psize$0))|0;
  $134 = $112 >>> 3;
  $135 = ($112>>>0)<(256);
  do {
   if ($135) {
    $136 = (($mem) + ($8)|0);
    $137 = HEAP32[$136>>2]|0;
    $$sum1718 = $8 | 4;
    $138 = (($mem) + ($$sum1718)|0);
    $139 = HEAP32[$138>>2]|0;
    $140 = $134 << 1;
    $141 = (9068 + ($140<<2)|0);
    $142 = ($137|0)==($141|0);
    if (!($142)) {
     $143 = HEAP32[(9044)>>2]|0;
     $144 = ($137>>>0)<($143>>>0);
     if ($144) {
      _abort();
      // unreachable;
     }
     $145 = ((($137)) + 12|0);
     $146 = HEAP32[$145>>2]|0;
     $147 = ($146|0)==($9|0);
     if (!($147)) {
      _abort();
      // unreachable;
     }
    }
    $148 = ($139|0)==($137|0);
    if ($148) {
     $149 = 1 << $134;
     $150 = $149 ^ -1;
     $151 = HEAP32[9028>>2]|0;
     $152 = $151 & $150;
     HEAP32[9028>>2] = $152;
     break;
    }
    $153 = ($139|0)==($141|0);
    if ($153) {
     $$pre58 = ((($139)) + 8|0);
     $$pre$phi59Z2D = $$pre58;
    } else {
     $154 = HEAP32[(9044)>>2]|0;
     $155 = ($139>>>0)<($154>>>0);
     if ($155) {
      _abort();
      // unreachable;
     }
     $156 = ((($139)) + 8|0);
     $157 = HEAP32[$156>>2]|0;
     $158 = ($157|0)==($9|0);
     if ($158) {
      $$pre$phi59Z2D = $156;
     } else {
      _abort();
      // unreachable;
     }
    }
    $159 = ((($137)) + 12|0);
    HEAP32[$159>>2] = $139;
    HEAP32[$$pre$phi59Z2D>>2] = $137;
   } else {
    $$sum5 = (($8) + 16)|0;
    $160 = (($mem) + ($$sum5)|0);
    $161 = HEAP32[$160>>2]|0;
    $$sum67 = $8 | 4;
    $162 = (($mem) + ($$sum67)|0);
    $163 = HEAP32[$162>>2]|0;
    $164 = ($163|0)==($9|0);
    do {
     if ($164) {
      $$sum9 = (($8) + 12)|0;
      $175 = (($mem) + ($$sum9)|0);
      $176 = HEAP32[$175>>2]|0;
      $177 = ($176|0)==(0|0);
      if ($177) {
       $$sum8 = (($8) + 8)|0;
       $178 = (($mem) + ($$sum8)|0);
       $179 = HEAP32[$178>>2]|0;
       $180 = ($179|0)==(0|0);
       if ($180) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $179;$RP9$0 = $178;
       }
      } else {
       $R7$0 = $176;$RP9$0 = $175;
      }
      while(1) {
       $181 = ((($R7$0)) + 20|0);
       $182 = HEAP32[$181>>2]|0;
       $183 = ($182|0)==(0|0);
       if (!($183)) {
        $R7$0 = $182;$RP9$0 = $181;
        continue;
       }
       $184 = ((($R7$0)) + 16|0);
       $185 = HEAP32[$184>>2]|0;
       $186 = ($185|0)==(0|0);
       if ($186) {
        $R7$0$lcssa = $R7$0;$RP9$0$lcssa = $RP9$0;
        break;
       } else {
        $R7$0 = $185;$RP9$0 = $184;
       }
      }
      $187 = HEAP32[(9044)>>2]|0;
      $188 = ($RP9$0$lcssa>>>0)<($187>>>0);
      if ($188) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0$lcssa>>2] = 0;
       $R7$1 = $R7$0$lcssa;
       break;
      }
     } else {
      $165 = (($mem) + ($8)|0);
      $166 = HEAP32[$165>>2]|0;
      $167 = HEAP32[(9044)>>2]|0;
      $168 = ($166>>>0)<($167>>>0);
      if ($168) {
       _abort();
       // unreachable;
      }
      $169 = ((($166)) + 12|0);
      $170 = HEAP32[$169>>2]|0;
      $171 = ($170|0)==($9|0);
      if (!($171)) {
       _abort();
       // unreachable;
      }
      $172 = ((($163)) + 8|0);
      $173 = HEAP32[$172>>2]|0;
      $174 = ($173|0)==($9|0);
      if ($174) {
       HEAP32[$169>>2] = $163;
       HEAP32[$172>>2] = $166;
       $R7$1 = $163;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $189 = ($161|0)==(0|0);
    if (!($189)) {
     $$sum12 = (($8) + 20)|0;
     $190 = (($mem) + ($$sum12)|0);
     $191 = HEAP32[$190>>2]|0;
     $192 = (9332 + ($191<<2)|0);
     $193 = HEAP32[$192>>2]|0;
     $194 = ($9|0)==($193|0);
     if ($194) {
      HEAP32[$192>>2] = $R7$1;
      $cond47 = ($R7$1|0)==(0|0);
      if ($cond47) {
       $195 = 1 << $191;
       $196 = $195 ^ -1;
       $197 = HEAP32[(9032)>>2]|0;
       $198 = $197 & $196;
       HEAP32[(9032)>>2] = $198;
       break;
      }
     } else {
      $199 = HEAP32[(9044)>>2]|0;
      $200 = ($161>>>0)<($199>>>0);
      if ($200) {
       _abort();
       // unreachable;
      }
      $201 = ((($161)) + 16|0);
      $202 = HEAP32[$201>>2]|0;
      $203 = ($202|0)==($9|0);
      if ($203) {
       HEAP32[$201>>2] = $R7$1;
      } else {
       $204 = ((($161)) + 20|0);
       HEAP32[$204>>2] = $R7$1;
      }
      $205 = ($R7$1|0)==(0|0);
      if ($205) {
       break;
      }
     }
     $206 = HEAP32[(9044)>>2]|0;
     $207 = ($R7$1>>>0)<($206>>>0);
     if ($207) {
      _abort();
      // unreachable;
     }
     $208 = ((($R7$1)) + 24|0);
     HEAP32[$208>>2] = $161;
     $$sum13 = (($8) + 8)|0;
     $209 = (($mem) + ($$sum13)|0);
     $210 = HEAP32[$209>>2]|0;
     $211 = ($210|0)==(0|0);
     do {
      if (!($211)) {
       $212 = ($210>>>0)<($206>>>0);
       if ($212) {
        _abort();
        // unreachable;
       } else {
        $213 = ((($R7$1)) + 16|0);
        HEAP32[$213>>2] = $210;
        $214 = ((($210)) + 24|0);
        HEAP32[$214>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum14 = (($8) + 12)|0;
     $215 = (($mem) + ($$sum14)|0);
     $216 = HEAP32[$215>>2]|0;
     $217 = ($216|0)==(0|0);
     if (!($217)) {
      $218 = HEAP32[(9044)>>2]|0;
      $219 = ($216>>>0)<($218>>>0);
      if ($219) {
       _abort();
       // unreachable;
      } else {
       $220 = ((($R7$1)) + 20|0);
       HEAP32[$220>>2] = $216;
       $221 = ((($216)) + 24|0);
       HEAP32[$221>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $222 = $133 | 1;
  $223 = ((($p$0)) + 4|0);
  HEAP32[$223>>2] = $222;
  $224 = (($p$0) + ($133)|0);
  HEAP32[$224>>2] = $133;
  $225 = HEAP32[(9048)>>2]|0;
  $226 = ($p$0|0)==($225|0);
  if ($226) {
   HEAP32[(9036)>>2] = $133;
   return;
  } else {
   $psize$1 = $133;
  }
 } else {
  $227 = $112 & -2;
  HEAP32[$111>>2] = $227;
  $228 = $psize$0 | 1;
  $229 = ((($p$0)) + 4|0);
  HEAP32[$229>>2] = $228;
  $230 = (($p$0) + ($psize$0)|0);
  HEAP32[$230>>2] = $psize$0;
  $psize$1 = $psize$0;
 }
 $231 = $psize$1 >>> 3;
 $232 = ($psize$1>>>0)<(256);
 if ($232) {
  $233 = $231 << 1;
  $234 = (9068 + ($233<<2)|0);
  $235 = HEAP32[9028>>2]|0;
  $236 = 1 << $231;
  $237 = $235 & $236;
  $238 = ($237|0)==(0);
  if ($238) {
   $239 = $235 | $236;
   HEAP32[9028>>2] = $239;
   $$pre = (($233) + 2)|0;
   $$pre57 = (9068 + ($$pre<<2)|0);
   $$pre$phiZ2D = $$pre57;$F16$0 = $234;
  } else {
   $$sum11 = (($233) + 2)|0;
   $240 = (9068 + ($$sum11<<2)|0);
   $241 = HEAP32[$240>>2]|0;
   $242 = HEAP32[(9044)>>2]|0;
   $243 = ($241>>>0)<($242>>>0);
   if ($243) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $240;$F16$0 = $241;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $p$0;
  $244 = ((($F16$0)) + 12|0);
  HEAP32[$244>>2] = $p$0;
  $245 = ((($p$0)) + 8|0);
  HEAP32[$245>>2] = $F16$0;
  $246 = ((($p$0)) + 12|0);
  HEAP32[$246>>2] = $234;
  return;
 }
 $247 = $psize$1 >>> 8;
 $248 = ($247|0)==(0);
 if ($248) {
  $I18$0 = 0;
 } else {
  $249 = ($psize$1>>>0)>(16777215);
  if ($249) {
   $I18$0 = 31;
  } else {
   $250 = (($247) + 1048320)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 8;
   $253 = $247 << $252;
   $254 = (($253) + 520192)|0;
   $255 = $254 >>> 16;
   $256 = $255 & 4;
   $257 = $256 | $252;
   $258 = $253 << $256;
   $259 = (($258) + 245760)|0;
   $260 = $259 >>> 16;
   $261 = $260 & 2;
   $262 = $257 | $261;
   $263 = (14 - ($262))|0;
   $264 = $258 << $261;
   $265 = $264 >>> 15;
   $266 = (($263) + ($265))|0;
   $267 = $266 << 1;
   $268 = (($266) + 7)|0;
   $269 = $psize$1 >>> $268;
   $270 = $269 & 1;
   $271 = $270 | $267;
   $I18$0 = $271;
  }
 }
 $272 = (9332 + ($I18$0<<2)|0);
 $273 = ((($p$0)) + 28|0);
 HEAP32[$273>>2] = $I18$0;
 $274 = ((($p$0)) + 16|0);
 $275 = ((($p$0)) + 20|0);
 HEAP32[$275>>2] = 0;
 HEAP32[$274>>2] = 0;
 $276 = HEAP32[(9032)>>2]|0;
 $277 = 1 << $I18$0;
 $278 = $276 & $277;
 $279 = ($278|0)==(0);
 L199: do {
  if ($279) {
   $280 = $276 | $277;
   HEAP32[(9032)>>2] = $280;
   HEAP32[$272>>2] = $p$0;
   $281 = ((($p$0)) + 24|0);
   HEAP32[$281>>2] = $272;
   $282 = ((($p$0)) + 12|0);
   HEAP32[$282>>2] = $p$0;
   $283 = ((($p$0)) + 8|0);
   HEAP32[$283>>2] = $p$0;
  } else {
   $284 = HEAP32[$272>>2]|0;
   $285 = ((($284)) + 4|0);
   $286 = HEAP32[$285>>2]|0;
   $287 = $286 & -8;
   $288 = ($287|0)==($psize$1|0);
   L202: do {
    if ($288) {
     $T$0$lcssa = $284;
    } else {
     $289 = ($I18$0|0)==(31);
     $290 = $I18$0 >>> 1;
     $291 = (25 - ($290))|0;
     $292 = $289 ? 0 : $291;
     $293 = $psize$1 << $292;
     $K19$052 = $293;$T$051 = $284;
     while(1) {
      $300 = $K19$052 >>> 31;
      $301 = (((($T$051)) + 16|0) + ($300<<2)|0);
      $296 = HEAP32[$301>>2]|0;
      $302 = ($296|0)==(0|0);
      if ($302) {
       $$lcssa = $301;$T$051$lcssa = $T$051;
       break;
      }
      $294 = $K19$052 << 1;
      $295 = ((($296)) + 4|0);
      $297 = HEAP32[$295>>2]|0;
      $298 = $297 & -8;
      $299 = ($298|0)==($psize$1|0);
      if ($299) {
       $T$0$lcssa = $296;
       break L202;
      } else {
       $K19$052 = $294;$T$051 = $296;
      }
     }
     $303 = HEAP32[(9044)>>2]|0;
     $304 = ($$lcssa>>>0)<($303>>>0);
     if ($304) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$$lcssa>>2] = $p$0;
      $305 = ((($p$0)) + 24|0);
      HEAP32[$305>>2] = $T$051$lcssa;
      $306 = ((($p$0)) + 12|0);
      HEAP32[$306>>2] = $p$0;
      $307 = ((($p$0)) + 8|0);
      HEAP32[$307>>2] = $p$0;
      break L199;
     }
    }
   } while(0);
   $308 = ((($T$0$lcssa)) + 8|0);
   $309 = HEAP32[$308>>2]|0;
   $310 = HEAP32[(9044)>>2]|0;
   $311 = ($309>>>0)>=($310>>>0);
   $not$ = ($T$0$lcssa>>>0)>=($310>>>0);
   $312 = $311 & $not$;
   if ($312) {
    $313 = ((($309)) + 12|0);
    HEAP32[$313>>2] = $p$0;
    HEAP32[$308>>2] = $p$0;
    $314 = ((($p$0)) + 8|0);
    HEAP32[$314>>2] = $309;
    $315 = ((($p$0)) + 12|0);
    HEAP32[$315>>2] = $T$0$lcssa;
    $316 = ((($p$0)) + 24|0);
    HEAP32[$316>>2] = 0;
    break;
   } else {
    _abort();
    // unreachable;
   }
  }
 } while(0);
 $317 = HEAP32[(9060)>>2]|0;
 $318 = (($317) + -1)|0;
 HEAP32[(9060)>>2] = $318;
 $319 = ($318|0)==(0);
 if ($319) {
  $sp$0$in$i = (9484);
 } else {
  return;
 }
 while(1) {
  $sp$0$i = HEAP32[$sp$0$in$i>>2]|0;
  $320 = ($sp$0$i|0)==(0|0);
  $321 = ((($sp$0$i)) + 8|0);
  if ($320) {
   break;
  } else {
   $sp$0$in$i = $321;
  }
 }
 HEAP32[(9060)>>2] = -1;
 return;
}
function _calloc($n_elements,$elem_size) {
 $n_elements = $n_elements|0;
 $elem_size = $elem_size|0;
 var $$ = 0, $0 = 0, $1 = 0, $10 = 0, $11 = 0, $2 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0, $7 = 0, $8 = 0, $9 = 0, $req$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($n_elements|0)==(0);
 if ($0) {
  $req$0 = 0;
 } else {
  $1 = Math_imul($elem_size, $n_elements)|0;
  $2 = $elem_size | $n_elements;
  $3 = ($2>>>0)>(65535);
  if ($3) {
   $4 = (($1>>>0) / ($n_elements>>>0))&-1;
   $5 = ($4|0)==($elem_size|0);
   $$ = $5 ? $1 : -1;
   $req$0 = $$;
  } else {
   $req$0 = $1;
  }
 }
 $6 = (_malloc($req$0)|0);
 $7 = ($6|0)==(0|0);
 if ($7) {
  return ($6|0);
 }
 $8 = ((($6)) + -4|0);
 $9 = HEAP32[$8>>2]|0;
 $10 = $9 & 3;
 $11 = ($10|0)==(0);
 if ($11) {
  return ($6|0);
 }
 _memset(($6|0),0,($req$0|0))|0;
 return ($6|0);
}
function _realloc($oldmem,$bytes) {
 $oldmem = $oldmem|0;
 $bytes = $bytes|0;
 var $0 = 0, $1 = 0, $10 = 0, $11 = 0, $12 = 0, $13 = 0, $14 = 0, $15 = 0, $16 = 0, $17 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $3 = 0, $4 = 0, $5 = 0, $6 = 0;
 var $7 = 0, $8 = 0, $9 = 0, $mem$0 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ($oldmem|0)==(0|0);
 if ($0) {
  $1 = (_malloc($bytes)|0);
  $mem$0 = $1;
  return ($mem$0|0);
 }
 $2 = ($bytes>>>0)>(4294967231);
 if ($2) {
  $3 = (___errno_location()|0);
  HEAP32[$3>>2] = 12;
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $4 = ($bytes>>>0)<(11);
 $5 = (($bytes) + 11)|0;
 $6 = $5 & -8;
 $7 = $4 ? 16 : $6;
 $8 = ((($oldmem)) + -8|0);
 $9 = (_try_realloc_chunk($8,$7)|0);
 $10 = ($9|0)==(0|0);
 if (!($10)) {
  $11 = ((($9)) + 8|0);
  $mem$0 = $11;
  return ($mem$0|0);
 }
 $12 = (_malloc($bytes)|0);
 $13 = ($12|0)==(0|0);
 if ($13) {
  $mem$0 = 0;
  return ($mem$0|0);
 }
 $14 = ((($oldmem)) + -4|0);
 $15 = HEAP32[$14>>2]|0;
 $16 = $15 & -8;
 $17 = $15 & 3;
 $18 = ($17|0)==(0);
 $19 = $18 ? 8 : 4;
 $20 = (($16) - ($19))|0;
 $21 = ($20>>>0)<($bytes>>>0);
 $22 = $21 ? $20 : $bytes;
 _memcpy(($12|0),($oldmem|0),($22|0))|0;
 _free($oldmem);
 $mem$0 = $12;
 return ($mem$0|0);
}
function _try_realloc_chunk($p,$nb) {
 $p = $p|0;
 $nb = $nb|0;
 var $$pre = 0, $$pre$phiZ2D = 0, $$sum = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum15 = 0, $$sum16 = 0, $$sum17 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum22 = 0, $$sum23 = 0, $$sum2728 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum78 = 0;
 var $$sum910 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0, $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0;
 var $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0, $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0;
 var $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0, $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0;
 var $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0, $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0;
 var $17 = 0, $170 = 0, $18 = 0, $19 = 0, $2 = 0, $20 = 0, $21 = 0, $22 = 0, $23 = 0, $24 = 0, $25 = 0, $26 = 0, $27 = 0, $28 = 0, $29 = 0, $3 = 0, $30 = 0, $31 = 0, $32 = 0, $33 = 0;
 var $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0, $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0;
 var $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0, $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0;
 var $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0, $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0;
 var $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0, $97 = 0, $98 = 0, $99 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $cond = 0, $newp$0 = 0, $notlhs = 0;
 var $notrhs = 0, $or$cond$not = 0, $or$cond30 = 0, $storemerge = 0, $storemerge21 = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = ((($p)) + 4|0);
 $1 = HEAP32[$0>>2]|0;
 $2 = $1 & -8;
 $3 = (($p) + ($2)|0);
 $4 = HEAP32[(9044)>>2]|0;
 $5 = $1 & 3;
 $notlhs = ($p>>>0)>=($4>>>0);
 $notrhs = ($5|0)!=(1);
 $or$cond$not = $notrhs & $notlhs;
 $6 = ($p>>>0)<($3>>>0);
 $or$cond30 = $or$cond$not & $6;
 if (!($or$cond30)) {
  _abort();
  // unreachable;
 }
 $$sum2728 = $2 | 4;
 $7 = (($p) + ($$sum2728)|0);
 $8 = HEAP32[$7>>2]|0;
 $9 = $8 & 1;
 $10 = ($9|0)==(0);
 if ($10) {
  _abort();
  // unreachable;
 }
 $11 = ($5|0)==(0);
 if ($11) {
  $12 = ($nb>>>0)<(256);
  if ($12) {
   $newp$0 = 0;
   return ($newp$0|0);
  }
  $13 = (($nb) + 4)|0;
  $14 = ($2>>>0)<($13>>>0);
  if (!($14)) {
   $15 = (($2) - ($nb))|0;
   $16 = HEAP32[(9508)>>2]|0;
   $17 = $16 << 1;
   $18 = ($15>>>0)>($17>>>0);
   if (!($18)) {
    $newp$0 = $p;
    return ($newp$0|0);
   }
  }
  $newp$0 = 0;
  return ($newp$0|0);
 }
 $19 = ($2>>>0)<($nb>>>0);
 if (!($19)) {
  $20 = (($2) - ($nb))|0;
  $21 = ($20>>>0)>(15);
  if (!($21)) {
   $newp$0 = $p;
   return ($newp$0|0);
  }
  $22 = (($p) + ($nb)|0);
  $23 = $1 & 1;
  $24 = $23 | $nb;
  $25 = $24 | 2;
  HEAP32[$0>>2] = $25;
  $$sum23 = (($nb) + 4)|0;
  $26 = (($p) + ($$sum23)|0);
  $27 = $20 | 3;
  HEAP32[$26>>2] = $27;
  $28 = HEAP32[$7>>2]|0;
  $29 = $28 | 1;
  HEAP32[$7>>2] = $29;
  _dispose_chunk($22,$20);
  $newp$0 = $p;
  return ($newp$0|0);
 }
 $30 = HEAP32[(9052)>>2]|0;
 $31 = ($3|0)==($30|0);
 if ($31) {
  $32 = HEAP32[(9040)>>2]|0;
  $33 = (($32) + ($2))|0;
  $34 = ($33>>>0)>($nb>>>0);
  if (!($34)) {
   $newp$0 = 0;
   return ($newp$0|0);
  }
  $35 = (($33) - ($nb))|0;
  $36 = (($p) + ($nb)|0);
  $37 = $1 & 1;
  $38 = $37 | $nb;
  $39 = $38 | 2;
  HEAP32[$0>>2] = $39;
  $$sum22 = (($nb) + 4)|0;
  $40 = (($p) + ($$sum22)|0);
  $41 = $35 | 1;
  HEAP32[$40>>2] = $41;
  HEAP32[(9052)>>2] = $36;
  HEAP32[(9040)>>2] = $35;
  $newp$0 = $p;
  return ($newp$0|0);
 }
 $42 = HEAP32[(9048)>>2]|0;
 $43 = ($3|0)==($42|0);
 if ($43) {
  $44 = HEAP32[(9036)>>2]|0;
  $45 = (($44) + ($2))|0;
  $46 = ($45>>>0)<($nb>>>0);
  if ($46) {
   $newp$0 = 0;
   return ($newp$0|0);
  }
  $47 = (($45) - ($nb))|0;
  $48 = ($47>>>0)>(15);
  if ($48) {
   $49 = (($p) + ($nb)|0);
   $50 = (($p) + ($45)|0);
   $51 = $1 & 1;
   $52 = $51 | $nb;
   $53 = $52 | 2;
   HEAP32[$0>>2] = $53;
   $$sum19 = (($nb) + 4)|0;
   $54 = (($p) + ($$sum19)|0);
   $55 = $47 | 1;
   HEAP32[$54>>2] = $55;
   HEAP32[$50>>2] = $47;
   $$sum20 = (($45) + 4)|0;
   $56 = (($p) + ($$sum20)|0);
   $57 = HEAP32[$56>>2]|0;
   $58 = $57 & -2;
   HEAP32[$56>>2] = $58;
   $storemerge = $49;$storemerge21 = $47;
  } else {
   $59 = $1 & 1;
   $60 = $59 | $45;
   $61 = $60 | 2;
   HEAP32[$0>>2] = $61;
   $$sum17 = (($45) + 4)|0;
   $62 = (($p) + ($$sum17)|0);
   $63 = HEAP32[$62>>2]|0;
   $64 = $63 | 1;
   HEAP32[$62>>2] = $64;
   $storemerge = 0;$storemerge21 = 0;
  }
  HEAP32[(9036)>>2] = $storemerge21;
  HEAP32[(9048)>>2] = $storemerge;
  $newp$0 = $p;
  return ($newp$0|0);
 }
 $65 = $8 & 2;
 $66 = ($65|0)==(0);
 if (!($66)) {
  $newp$0 = 0;
  return ($newp$0|0);
 }
 $67 = $8 & -8;
 $68 = (($67) + ($2))|0;
 $69 = ($68>>>0)<($nb>>>0);
 if ($69) {
  $newp$0 = 0;
  return ($newp$0|0);
 }
 $70 = (($68) - ($nb))|0;
 $71 = $8 >>> 3;
 $72 = ($8>>>0)<(256);
 do {
  if ($72) {
   $$sum15 = (($2) + 8)|0;
   $73 = (($p) + ($$sum15)|0);
   $74 = HEAP32[$73>>2]|0;
   $$sum16 = (($2) + 12)|0;
   $75 = (($p) + ($$sum16)|0);
   $76 = HEAP32[$75>>2]|0;
   $77 = $71 << 1;
   $78 = (9068 + ($77<<2)|0);
   $79 = ($74|0)==($78|0);
   if (!($79)) {
    $80 = ($74>>>0)<($4>>>0);
    if ($80) {
     _abort();
     // unreachable;
    }
    $81 = ((($74)) + 12|0);
    $82 = HEAP32[$81>>2]|0;
    $83 = ($82|0)==($3|0);
    if (!($83)) {
     _abort();
     // unreachable;
    }
   }
   $84 = ($76|0)==($74|0);
   if ($84) {
    $85 = 1 << $71;
    $86 = $85 ^ -1;
    $87 = HEAP32[9028>>2]|0;
    $88 = $87 & $86;
    HEAP32[9028>>2] = $88;
    break;
   }
   $89 = ($76|0)==($78|0);
   if ($89) {
    $$pre = ((($76)) + 8|0);
    $$pre$phiZ2D = $$pre;
   } else {
    $90 = ($76>>>0)<($4>>>0);
    if ($90) {
     _abort();
     // unreachable;
    }
    $91 = ((($76)) + 8|0);
    $92 = HEAP32[$91>>2]|0;
    $93 = ($92|0)==($3|0);
    if ($93) {
     $$pre$phiZ2D = $91;
    } else {
     _abort();
     // unreachable;
    }
   }
   $94 = ((($74)) + 12|0);
   HEAP32[$94>>2] = $76;
   HEAP32[$$pre$phiZ2D>>2] = $74;
  } else {
   $$sum = (($2) + 24)|0;
   $95 = (($p) + ($$sum)|0);
   $96 = HEAP32[$95>>2]|0;
   $$sum2 = (($2) + 12)|0;
   $97 = (($p) + ($$sum2)|0);
   $98 = HEAP32[$97>>2]|0;
   $99 = ($98|0)==($3|0);
   do {
    if ($99) {
     $$sum4 = (($2) + 20)|0;
     $109 = (($p) + ($$sum4)|0);
     $110 = HEAP32[$109>>2]|0;
     $111 = ($110|0)==(0|0);
     if ($111) {
      $$sum3 = (($2) + 16)|0;
      $112 = (($p) + ($$sum3)|0);
      $113 = HEAP32[$112>>2]|0;
      $114 = ($113|0)==(0|0);
      if ($114) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $113;$RP$0 = $112;
      }
     } else {
      $R$0 = $110;$RP$0 = $109;
     }
     while(1) {
      $115 = ((($R$0)) + 20|0);
      $116 = HEAP32[$115>>2]|0;
      $117 = ($116|0)==(0|0);
      if (!($117)) {
       $R$0 = $116;$RP$0 = $115;
       continue;
      }
      $118 = ((($R$0)) + 16|0);
      $119 = HEAP32[$118>>2]|0;
      $120 = ($119|0)==(0|0);
      if ($120) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $119;$RP$0 = $118;
      }
     }
     $121 = ($RP$0$lcssa>>>0)<($4>>>0);
     if ($121) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum14 = (($2) + 8)|0;
     $100 = (($p) + ($$sum14)|0);
     $101 = HEAP32[$100>>2]|0;
     $102 = ($101>>>0)<($4>>>0);
     if ($102) {
      _abort();
      // unreachable;
     }
     $103 = ((($101)) + 12|0);
     $104 = HEAP32[$103>>2]|0;
     $105 = ($104|0)==($3|0);
     if (!($105)) {
      _abort();
      // unreachable;
     }
     $106 = ((($98)) + 8|0);
     $107 = HEAP32[$106>>2]|0;
     $108 = ($107|0)==($3|0);
     if ($108) {
      HEAP32[$103>>2] = $98;
      HEAP32[$106>>2] = $101;
      $R$1 = $98;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $122 = ($96|0)==(0|0);
   if (!($122)) {
    $$sum11 = (($2) + 28)|0;
    $123 = (($p) + ($$sum11)|0);
    $124 = HEAP32[$123>>2]|0;
    $125 = (9332 + ($124<<2)|0);
    $126 = HEAP32[$125>>2]|0;
    $127 = ($3|0)==($126|0);
    if ($127) {
     HEAP32[$125>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $128 = 1 << $124;
      $129 = $128 ^ -1;
      $130 = HEAP32[(9032)>>2]|0;
      $131 = $130 & $129;
      HEAP32[(9032)>>2] = $131;
      break;
     }
    } else {
     $132 = HEAP32[(9044)>>2]|0;
     $133 = ($96>>>0)<($132>>>0);
     if ($133) {
      _abort();
      // unreachable;
     }
     $134 = ((($96)) + 16|0);
     $135 = HEAP32[$134>>2]|0;
     $136 = ($135|0)==($3|0);
     if ($136) {
      HEAP32[$134>>2] = $R$1;
     } else {
      $137 = ((($96)) + 20|0);
      HEAP32[$137>>2] = $R$1;
     }
     $138 = ($R$1|0)==(0|0);
     if ($138) {
      break;
     }
    }
    $139 = HEAP32[(9044)>>2]|0;
    $140 = ($R$1>>>0)<($139>>>0);
    if ($140) {
     _abort();
     // unreachable;
    }
    $141 = ((($R$1)) + 24|0);
    HEAP32[$141>>2] = $96;
    $$sum12 = (($2) + 16)|0;
    $142 = (($p) + ($$sum12)|0);
    $143 = HEAP32[$142>>2]|0;
    $144 = ($143|0)==(0|0);
    do {
     if (!($144)) {
      $145 = ($143>>>0)<($139>>>0);
      if ($145) {
       _abort();
       // unreachable;
      } else {
       $146 = ((($R$1)) + 16|0);
       HEAP32[$146>>2] = $143;
       $147 = ((($143)) + 24|0);
       HEAP32[$147>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum13 = (($2) + 20)|0;
    $148 = (($p) + ($$sum13)|0);
    $149 = HEAP32[$148>>2]|0;
    $150 = ($149|0)==(0|0);
    if (!($150)) {
     $151 = HEAP32[(9044)>>2]|0;
     $152 = ($149>>>0)<($151>>>0);
     if ($152) {
      _abort();
      // unreachable;
     } else {
      $153 = ((($R$1)) + 20|0);
      HEAP32[$153>>2] = $149;
      $154 = ((($149)) + 24|0);
      HEAP32[$154>>2] = $R$1;
      break;
     }
    }
   }
  }
 } while(0);
 $155 = ($70>>>0)<(16);
 if ($155) {
  $156 = $1 & 1;
  $157 = $68 | $156;
  $158 = $157 | 2;
  HEAP32[$0>>2] = $158;
  $$sum910 = $68 | 4;
  $159 = (($p) + ($$sum910)|0);
  $160 = HEAP32[$159>>2]|0;
  $161 = $160 | 1;
  HEAP32[$159>>2] = $161;
  $newp$0 = $p;
  return ($newp$0|0);
 } else {
  $162 = (($p) + ($nb)|0);
  $163 = $1 & 1;
  $164 = $163 | $nb;
  $165 = $164 | 2;
  HEAP32[$0>>2] = $165;
  $$sum5 = (($nb) + 4)|0;
  $166 = (($p) + ($$sum5)|0);
  $167 = $70 | 3;
  HEAP32[$166>>2] = $167;
  $$sum78 = $68 | 4;
  $168 = (($p) + ($$sum78)|0);
  $169 = HEAP32[$168>>2]|0;
  $170 = $169 | 1;
  HEAP32[$168>>2] = $170;
  _dispose_chunk($162,$70);
  $newp$0 = $p;
  return ($newp$0|0);
 }
 return (0)|0;
}
function _dispose_chunk($p,$psize) {
 $p = $p|0;
 $psize = $psize|0;
 var $$0 = 0, $$02 = 0, $$1 = 0, $$lcssa = 0, $$pre = 0, $$pre$phi50Z2D = 0, $$pre$phi52Z2D = 0, $$pre$phiZ2D = 0, $$pre48 = 0, $$pre49 = 0, $$pre51 = 0, $$sum = 0, $$sum1 = 0, $$sum10 = 0, $$sum11 = 0, $$sum12 = 0, $$sum13 = 0, $$sum14 = 0, $$sum16 = 0, $$sum17 = 0;
 var $$sum18 = 0, $$sum19 = 0, $$sum2 = 0, $$sum20 = 0, $$sum21 = 0, $$sum22 = 0, $$sum23 = 0, $$sum24 = 0, $$sum25 = 0, $$sum3 = 0, $$sum4 = 0, $$sum5 = 0, $$sum7 = 0, $$sum8 = 0, $$sum9 = 0, $0 = 0, $1 = 0, $10 = 0, $100 = 0, $101 = 0;
 var $102 = 0, $103 = 0, $104 = 0, $105 = 0, $106 = 0, $107 = 0, $108 = 0, $109 = 0, $11 = 0, $110 = 0, $111 = 0, $112 = 0, $113 = 0, $114 = 0, $115 = 0, $116 = 0, $117 = 0, $118 = 0, $119 = 0, $12 = 0;
 var $120 = 0, $121 = 0, $122 = 0, $123 = 0, $124 = 0, $125 = 0, $126 = 0, $127 = 0, $128 = 0, $129 = 0, $13 = 0, $130 = 0, $131 = 0, $132 = 0, $133 = 0, $134 = 0, $135 = 0, $136 = 0, $137 = 0, $138 = 0;
 var $139 = 0, $14 = 0, $140 = 0, $141 = 0, $142 = 0, $143 = 0, $144 = 0, $145 = 0, $146 = 0, $147 = 0, $148 = 0, $149 = 0, $15 = 0, $150 = 0, $151 = 0, $152 = 0, $153 = 0, $154 = 0, $155 = 0, $156 = 0;
 var $157 = 0, $158 = 0, $159 = 0, $16 = 0, $160 = 0, $161 = 0, $162 = 0, $163 = 0, $164 = 0, $165 = 0, $166 = 0, $167 = 0, $168 = 0, $169 = 0, $17 = 0, $170 = 0, $171 = 0, $172 = 0, $173 = 0, $174 = 0;
 var $175 = 0, $176 = 0, $177 = 0, $178 = 0, $179 = 0, $18 = 0, $180 = 0, $181 = 0, $182 = 0, $183 = 0, $184 = 0, $185 = 0, $186 = 0, $187 = 0, $188 = 0, $189 = 0, $19 = 0, $190 = 0, $191 = 0, $192 = 0;
 var $193 = 0, $194 = 0, $195 = 0, $196 = 0, $197 = 0, $198 = 0, $199 = 0, $2 = 0, $20 = 0, $200 = 0, $201 = 0, $202 = 0, $203 = 0, $204 = 0, $205 = 0, $206 = 0, $207 = 0, $208 = 0, $209 = 0, $21 = 0;
 var $210 = 0, $211 = 0, $212 = 0, $213 = 0, $214 = 0, $215 = 0, $216 = 0, $217 = 0, $218 = 0, $219 = 0, $22 = 0, $220 = 0, $221 = 0, $222 = 0, $223 = 0, $224 = 0, $225 = 0, $226 = 0, $227 = 0, $228 = 0;
 var $229 = 0, $23 = 0, $230 = 0, $231 = 0, $232 = 0, $233 = 0, $234 = 0, $235 = 0, $236 = 0, $237 = 0, $238 = 0, $239 = 0, $24 = 0, $240 = 0, $241 = 0, $242 = 0, $243 = 0, $244 = 0, $245 = 0, $246 = 0;
 var $247 = 0, $248 = 0, $249 = 0, $25 = 0, $250 = 0, $251 = 0, $252 = 0, $253 = 0, $254 = 0, $255 = 0, $256 = 0, $257 = 0, $258 = 0, $259 = 0, $26 = 0, $260 = 0, $261 = 0, $262 = 0, $263 = 0, $264 = 0;
 var $265 = 0, $266 = 0, $267 = 0, $268 = 0, $269 = 0, $27 = 0, $270 = 0, $271 = 0, $272 = 0, $273 = 0, $274 = 0, $275 = 0, $276 = 0, $277 = 0, $278 = 0, $279 = 0, $28 = 0, $280 = 0, $281 = 0, $282 = 0;
 var $283 = 0, $284 = 0, $285 = 0, $286 = 0, $287 = 0, $288 = 0, $289 = 0, $29 = 0, $290 = 0, $291 = 0, $292 = 0, $293 = 0, $294 = 0, $295 = 0, $296 = 0, $297 = 0, $298 = 0, $299 = 0, $3 = 0, $30 = 0;
 var $300 = 0, $301 = 0, $302 = 0, $303 = 0, $304 = 0, $305 = 0, $306 = 0, $307 = 0, $31 = 0, $32 = 0, $33 = 0, $34 = 0, $35 = 0, $36 = 0, $37 = 0, $38 = 0, $39 = 0, $4 = 0, $40 = 0, $41 = 0;
 var $42 = 0, $43 = 0, $44 = 0, $45 = 0, $46 = 0, $47 = 0, $48 = 0, $49 = 0, $5 = 0, $50 = 0, $51 = 0, $52 = 0, $53 = 0, $54 = 0, $55 = 0, $56 = 0, $57 = 0, $58 = 0, $59 = 0, $6 = 0;
 var $60 = 0, $61 = 0, $62 = 0, $63 = 0, $64 = 0, $65 = 0, $66 = 0, $67 = 0, $68 = 0, $69 = 0, $7 = 0, $70 = 0, $71 = 0, $72 = 0, $73 = 0, $74 = 0, $75 = 0, $76 = 0, $77 = 0, $78 = 0;
 var $79 = 0, $8 = 0, $80 = 0, $81 = 0, $82 = 0, $83 = 0, $84 = 0, $85 = 0, $86 = 0, $87 = 0, $88 = 0, $89 = 0, $9 = 0, $90 = 0, $91 = 0, $92 = 0, $93 = 0, $94 = 0, $95 = 0, $96 = 0;
 var $97 = 0, $98 = 0, $99 = 0, $F16$0 = 0, $I19$0 = 0, $K20$043 = 0, $R$0 = 0, $R$0$lcssa = 0, $R$1 = 0, $R7$0 = 0, $R7$0$lcssa = 0, $R7$1 = 0, $RP$0 = 0, $RP$0$lcssa = 0, $RP9$0 = 0, $RP9$0$lcssa = 0, $T$0$lcssa = 0, $T$042 = 0, $T$042$lcssa = 0, $cond = 0;
 var $cond39 = 0, $not$ = 0, label = 0, sp = 0;
 sp = STACKTOP;
 $0 = (($p) + ($psize)|0);
 $1 = ((($p)) + 4|0);
 $2 = HEAP32[$1>>2]|0;
 $3 = $2 & 1;
 $4 = ($3|0)==(0);
 do {
  if ($4) {
   $5 = HEAP32[$p>>2]|0;
   $6 = $2 & 3;
   $7 = ($6|0)==(0);
   if ($7) {
    return;
   }
   $8 = (0 - ($5))|0;
   $9 = (($p) + ($8)|0);
   $10 = (($5) + ($psize))|0;
   $11 = HEAP32[(9044)>>2]|0;
   $12 = ($9>>>0)<($11>>>0);
   if ($12) {
    _abort();
    // unreachable;
   }
   $13 = HEAP32[(9048)>>2]|0;
   $14 = ($9|0)==($13|0);
   if ($14) {
    $$sum = (($psize) + 4)|0;
    $99 = (($p) + ($$sum)|0);
    $100 = HEAP32[$99>>2]|0;
    $101 = $100 & 3;
    $102 = ($101|0)==(3);
    if (!($102)) {
     $$0 = $9;$$02 = $10;
     break;
    }
    HEAP32[(9036)>>2] = $10;
    $103 = $100 & -2;
    HEAP32[$99>>2] = $103;
    $104 = $10 | 1;
    $$sum14 = (4 - ($5))|0;
    $105 = (($p) + ($$sum14)|0);
    HEAP32[$105>>2] = $104;
    HEAP32[$0>>2] = $10;
    return;
   }
   $15 = $5 >>> 3;
   $16 = ($5>>>0)<(256);
   if ($16) {
    $$sum24 = (8 - ($5))|0;
    $17 = (($p) + ($$sum24)|0);
    $18 = HEAP32[$17>>2]|0;
    $$sum25 = (12 - ($5))|0;
    $19 = (($p) + ($$sum25)|0);
    $20 = HEAP32[$19>>2]|0;
    $21 = $15 << 1;
    $22 = (9068 + ($21<<2)|0);
    $23 = ($18|0)==($22|0);
    if (!($23)) {
     $24 = ($18>>>0)<($11>>>0);
     if ($24) {
      _abort();
      // unreachable;
     }
     $25 = ((($18)) + 12|0);
     $26 = HEAP32[$25>>2]|0;
     $27 = ($26|0)==($9|0);
     if (!($27)) {
      _abort();
      // unreachable;
     }
    }
    $28 = ($20|0)==($18|0);
    if ($28) {
     $29 = 1 << $15;
     $30 = $29 ^ -1;
     $31 = HEAP32[9028>>2]|0;
     $32 = $31 & $30;
     HEAP32[9028>>2] = $32;
     $$0 = $9;$$02 = $10;
     break;
    }
    $33 = ($20|0)==($22|0);
    if ($33) {
     $$pre51 = ((($20)) + 8|0);
     $$pre$phi52Z2D = $$pre51;
    } else {
     $34 = ($20>>>0)<($11>>>0);
     if ($34) {
      _abort();
      // unreachable;
     }
     $35 = ((($20)) + 8|0);
     $36 = HEAP32[$35>>2]|0;
     $37 = ($36|0)==($9|0);
     if ($37) {
      $$pre$phi52Z2D = $35;
     } else {
      _abort();
      // unreachable;
     }
    }
    $38 = ((($18)) + 12|0);
    HEAP32[$38>>2] = $20;
    HEAP32[$$pre$phi52Z2D>>2] = $18;
    $$0 = $9;$$02 = $10;
    break;
   }
   $$sum16 = (24 - ($5))|0;
   $39 = (($p) + ($$sum16)|0);
   $40 = HEAP32[$39>>2]|0;
   $$sum17 = (12 - ($5))|0;
   $41 = (($p) + ($$sum17)|0);
   $42 = HEAP32[$41>>2]|0;
   $43 = ($42|0)==($9|0);
   do {
    if ($43) {
     $$sum18 = (16 - ($5))|0;
     $$sum19 = (($$sum18) + 4)|0;
     $53 = (($p) + ($$sum19)|0);
     $54 = HEAP32[$53>>2]|0;
     $55 = ($54|0)==(0|0);
     if ($55) {
      $56 = (($p) + ($$sum18)|0);
      $57 = HEAP32[$56>>2]|0;
      $58 = ($57|0)==(0|0);
      if ($58) {
       $R$1 = 0;
       break;
      } else {
       $R$0 = $57;$RP$0 = $56;
      }
     } else {
      $R$0 = $54;$RP$0 = $53;
     }
     while(1) {
      $59 = ((($R$0)) + 20|0);
      $60 = HEAP32[$59>>2]|0;
      $61 = ($60|0)==(0|0);
      if (!($61)) {
       $R$0 = $60;$RP$0 = $59;
       continue;
      }
      $62 = ((($R$0)) + 16|0);
      $63 = HEAP32[$62>>2]|0;
      $64 = ($63|0)==(0|0);
      if ($64) {
       $R$0$lcssa = $R$0;$RP$0$lcssa = $RP$0;
       break;
      } else {
       $R$0 = $63;$RP$0 = $62;
      }
     }
     $65 = ($RP$0$lcssa>>>0)<($11>>>0);
     if ($65) {
      _abort();
      // unreachable;
     } else {
      HEAP32[$RP$0$lcssa>>2] = 0;
      $R$1 = $R$0$lcssa;
      break;
     }
    } else {
     $$sum23 = (8 - ($5))|0;
     $44 = (($p) + ($$sum23)|0);
     $45 = HEAP32[$44>>2]|0;
     $46 = ($45>>>0)<($11>>>0);
     if ($46) {
      _abort();
      // unreachable;
     }
     $47 = ((($45)) + 12|0);
     $48 = HEAP32[$47>>2]|0;
     $49 = ($48|0)==($9|0);
     if (!($49)) {
      _abort();
      // unreachable;
     }
     $50 = ((($42)) + 8|0);
     $51 = HEAP32[$50>>2]|0;
     $52 = ($51|0)==($9|0);
     if ($52) {
      HEAP32[$47>>2] = $42;
      HEAP32[$50>>2] = $45;
      $R$1 = $42;
      break;
     } else {
      _abort();
      // unreachable;
     }
    }
   } while(0);
   $66 = ($40|0)==(0|0);
   if ($66) {
    $$0 = $9;$$02 = $10;
   } else {
    $$sum20 = (28 - ($5))|0;
    $67 = (($p) + ($$sum20)|0);
    $68 = HEAP32[$67>>2]|0;
    $69 = (9332 + ($68<<2)|0);
    $70 = HEAP32[$69>>2]|0;
    $71 = ($9|0)==($70|0);
    if ($71) {
     HEAP32[$69>>2] = $R$1;
     $cond = ($R$1|0)==(0|0);
     if ($cond) {
      $72 = 1 << $68;
      $73 = $72 ^ -1;
      $74 = HEAP32[(9032)>>2]|0;
      $75 = $74 & $73;
      HEAP32[(9032)>>2] = $75;
      $$0 = $9;$$02 = $10;
      break;
     }
    } else {
     $76 = HEAP32[(9044)>>2]|0;
     $77 = ($40>>>0)<($76>>>0);
     if ($77) {
      _abort();
      // unreachable;
     }
     $78 = ((($40)) + 16|0);
     $79 = HEAP32[$78>>2]|0;
     $80 = ($79|0)==($9|0);
     if ($80) {
      HEAP32[$78>>2] = $R$1;
     } else {
      $81 = ((($40)) + 20|0);
      HEAP32[$81>>2] = $R$1;
     }
     $82 = ($R$1|0)==(0|0);
     if ($82) {
      $$0 = $9;$$02 = $10;
      break;
     }
    }
    $83 = HEAP32[(9044)>>2]|0;
    $84 = ($R$1>>>0)<($83>>>0);
    if ($84) {
     _abort();
     // unreachable;
    }
    $85 = ((($R$1)) + 24|0);
    HEAP32[$85>>2] = $40;
    $$sum21 = (16 - ($5))|0;
    $86 = (($p) + ($$sum21)|0);
    $87 = HEAP32[$86>>2]|0;
    $88 = ($87|0)==(0|0);
    do {
     if (!($88)) {
      $89 = ($87>>>0)<($83>>>0);
      if ($89) {
       _abort();
       // unreachable;
      } else {
       $90 = ((($R$1)) + 16|0);
       HEAP32[$90>>2] = $87;
       $91 = ((($87)) + 24|0);
       HEAP32[$91>>2] = $R$1;
       break;
      }
     }
    } while(0);
    $$sum22 = (($$sum21) + 4)|0;
    $92 = (($p) + ($$sum22)|0);
    $93 = HEAP32[$92>>2]|0;
    $94 = ($93|0)==(0|0);
    if ($94) {
     $$0 = $9;$$02 = $10;
    } else {
     $95 = HEAP32[(9044)>>2]|0;
     $96 = ($93>>>0)<($95>>>0);
     if ($96) {
      _abort();
      // unreachable;
     } else {
      $97 = ((($R$1)) + 20|0);
      HEAP32[$97>>2] = $93;
      $98 = ((($93)) + 24|0);
      HEAP32[$98>>2] = $R$1;
      $$0 = $9;$$02 = $10;
      break;
     }
    }
   }
  } else {
   $$0 = $p;$$02 = $psize;
  }
 } while(0);
 $106 = HEAP32[(9044)>>2]|0;
 $107 = ($0>>>0)<($106>>>0);
 if ($107) {
  _abort();
  // unreachable;
 }
 $$sum1 = (($psize) + 4)|0;
 $108 = (($p) + ($$sum1)|0);
 $109 = HEAP32[$108>>2]|0;
 $110 = $109 & 2;
 $111 = ($110|0)==(0);
 if ($111) {
  $112 = HEAP32[(9052)>>2]|0;
  $113 = ($0|0)==($112|0);
  if ($113) {
   $114 = HEAP32[(9040)>>2]|0;
   $115 = (($114) + ($$02))|0;
   HEAP32[(9040)>>2] = $115;
   HEAP32[(9052)>>2] = $$0;
   $116 = $115 | 1;
   $117 = ((($$0)) + 4|0);
   HEAP32[$117>>2] = $116;
   $118 = HEAP32[(9048)>>2]|0;
   $119 = ($$0|0)==($118|0);
   if (!($119)) {
    return;
   }
   HEAP32[(9048)>>2] = 0;
   HEAP32[(9036)>>2] = 0;
   return;
  }
  $120 = HEAP32[(9048)>>2]|0;
  $121 = ($0|0)==($120|0);
  if ($121) {
   $122 = HEAP32[(9036)>>2]|0;
   $123 = (($122) + ($$02))|0;
   HEAP32[(9036)>>2] = $123;
   HEAP32[(9048)>>2] = $$0;
   $124 = $123 | 1;
   $125 = ((($$0)) + 4|0);
   HEAP32[$125>>2] = $124;
   $126 = (($$0) + ($123)|0);
   HEAP32[$126>>2] = $123;
   return;
  }
  $127 = $109 & -8;
  $128 = (($127) + ($$02))|0;
  $129 = $109 >>> 3;
  $130 = ($109>>>0)<(256);
  do {
   if ($130) {
    $$sum12 = (($psize) + 8)|0;
    $131 = (($p) + ($$sum12)|0);
    $132 = HEAP32[$131>>2]|0;
    $$sum13 = (($psize) + 12)|0;
    $133 = (($p) + ($$sum13)|0);
    $134 = HEAP32[$133>>2]|0;
    $135 = $129 << 1;
    $136 = (9068 + ($135<<2)|0);
    $137 = ($132|0)==($136|0);
    if (!($137)) {
     $138 = ($132>>>0)<($106>>>0);
     if ($138) {
      _abort();
      // unreachable;
     }
     $139 = ((($132)) + 12|0);
     $140 = HEAP32[$139>>2]|0;
     $141 = ($140|0)==($0|0);
     if (!($141)) {
      _abort();
      // unreachable;
     }
    }
    $142 = ($134|0)==($132|0);
    if ($142) {
     $143 = 1 << $129;
     $144 = $143 ^ -1;
     $145 = HEAP32[9028>>2]|0;
     $146 = $145 & $144;
     HEAP32[9028>>2] = $146;
     break;
    }
    $147 = ($134|0)==($136|0);
    if ($147) {
     $$pre49 = ((($134)) + 8|0);
     $$pre$phi50Z2D = $$pre49;
    } else {
     $148 = ($134>>>0)<($106>>>0);
     if ($148) {
      _abort();
      // unreachable;
     }
     $149 = ((($134)) + 8|0);
     $150 = HEAP32[$149>>2]|0;
     $151 = ($150|0)==($0|0);
     if ($151) {
      $$pre$phi50Z2D = $149;
     } else {
      _abort();
      // unreachable;
     }
    }
    $152 = ((($132)) + 12|0);
    HEAP32[$152>>2] = $134;
    HEAP32[$$pre$phi50Z2D>>2] = $132;
   } else {
    $$sum2 = (($psize) + 24)|0;
    $153 = (($p) + ($$sum2)|0);
    $154 = HEAP32[$153>>2]|0;
    $$sum3 = (($psize) + 12)|0;
    $155 = (($p) + ($$sum3)|0);
    $156 = HEAP32[$155>>2]|0;
    $157 = ($156|0)==($0|0);
    do {
     if ($157) {
      $$sum5 = (($psize) + 20)|0;
      $167 = (($p) + ($$sum5)|0);
      $168 = HEAP32[$167>>2]|0;
      $169 = ($168|0)==(0|0);
      if ($169) {
       $$sum4 = (($psize) + 16)|0;
       $170 = (($p) + ($$sum4)|0);
       $171 = HEAP32[$170>>2]|0;
       $172 = ($171|0)==(0|0);
       if ($172) {
        $R7$1 = 0;
        break;
       } else {
        $R7$0 = $171;$RP9$0 = $170;
       }
      } else {
       $R7$0 = $168;$RP9$0 = $167;
      }
      while(1) {
       $173 = ((($R7$0)) + 20|0);
       $174 = HEAP32[$173>>2]|0;
       $175 = ($174|0)==(0|0);
       if (!($175)) {
        $R7$0 = $174;$RP9$0 = $173;
        continue;
       }
       $176 = ((($R7$0)) + 16|0);
       $177 = HEAP32[$176>>2]|0;
       $178 = ($177|0)==(0|0);
       if ($178) {
        $R7$0$lcssa = $R7$0;$RP9$0$lcssa = $RP9$0;
        break;
       } else {
        $R7$0 = $177;$RP9$0 = $176;
       }
      }
      $179 = ($RP9$0$lcssa>>>0)<($106>>>0);
      if ($179) {
       _abort();
       // unreachable;
      } else {
       HEAP32[$RP9$0$lcssa>>2] = 0;
       $R7$1 = $R7$0$lcssa;
       break;
      }
     } else {
      $$sum11 = (($psize) + 8)|0;
      $158 = (($p) + ($$sum11)|0);
      $159 = HEAP32[$158>>2]|0;
      $160 = ($159>>>0)<($106>>>0);
      if ($160) {
       _abort();
       // unreachable;
      }
      $161 = ((($159)) + 12|0);
      $162 = HEAP32[$161>>2]|0;
      $163 = ($162|0)==($0|0);
      if (!($163)) {
       _abort();
       // unreachable;
      }
      $164 = ((($156)) + 8|0);
      $165 = HEAP32[$164>>2]|0;
      $166 = ($165|0)==($0|0);
      if ($166) {
       HEAP32[$161>>2] = $156;
       HEAP32[$164>>2] = $159;
       $R7$1 = $156;
       break;
      } else {
       _abort();
       // unreachable;
      }
     }
    } while(0);
    $180 = ($154|0)==(0|0);
    if (!($180)) {
     $$sum8 = (($psize) + 28)|0;
     $181 = (($p) + ($$sum8)|0);
     $182 = HEAP32[$181>>2]|0;
     $183 = (9332 + ($182<<2)|0);
     $184 = HEAP32[$183>>2]|0;
     $185 = ($0|0)==($184|0);
     if ($185) {
      HEAP32[$183>>2] = $R7$1;
      $cond39 = ($R7$1|0)==(0|0);
      if ($cond39) {
       $186 = 1 << $182;
       $187 = $186 ^ -1;
       $188 = HEAP32[(9032)>>2]|0;
       $189 = $188 & $187;
       HEAP32[(9032)>>2] = $189;
       break;
      }
     } else {
      $190 = HEAP32[(9044)>>2]|0;
      $191 = ($154>>>0)<($190>>>0);
      if ($191) {
       _abort();
       // unreachable;
      }
      $192 = ((($154)) + 16|0);
      $193 = HEAP32[$192>>2]|0;
      $194 = ($193|0)==($0|0);
      if ($194) {
       HEAP32[$192>>2] = $R7$1;
      } else {
       $195 = ((($154)) + 20|0);
       HEAP32[$195>>2] = $R7$1;
      }
      $196 = ($R7$1|0)==(0|0);
      if ($196) {
       break;
      }
     }
     $197 = HEAP32[(9044)>>2]|0;
     $198 = ($R7$1>>>0)<($197>>>0);
     if ($198) {
      _abort();
      // unreachable;
     }
     $199 = ((($R7$1)) + 24|0);
     HEAP32[$199>>2] = $154;
     $$sum9 = (($psize) + 16)|0;
     $200 = (($p) + ($$sum9)|0);
     $201 = HEAP32[$200>>2]|0;
     $202 = ($201|0)==(0|0);
     do {
      if (!($202)) {
       $203 = ($201>>>0)<($197>>>0);
       if ($203) {
        _abort();
        // unreachable;
       } else {
        $204 = ((($R7$1)) + 16|0);
        HEAP32[$204>>2] = $201;
        $205 = ((($201)) + 24|0);
        HEAP32[$205>>2] = $R7$1;
        break;
       }
      }
     } while(0);
     $$sum10 = (($psize) + 20)|0;
     $206 = (($p) + ($$sum10)|0);
     $207 = HEAP32[$206>>2]|0;
     $208 = ($207|0)==(0|0);
     if (!($208)) {
      $209 = HEAP32[(9044)>>2]|0;
      $210 = ($207>>>0)<($209>>>0);
      if ($210) {
       _abort();
       // unreachable;
      } else {
       $211 = ((($R7$1)) + 20|0);
       HEAP32[$211>>2] = $207;
       $212 = ((($207)) + 24|0);
       HEAP32[$212>>2] = $R7$1;
       break;
      }
     }
    }
   }
  } while(0);
  $213 = $128 | 1;
  $214 = ((($$0)) + 4|0);
  HEAP32[$214>>2] = $213;
  $215 = (($$0) + ($128)|0);
  HEAP32[$215>>2] = $128;
  $216 = HEAP32[(9048)>>2]|0;
  $217 = ($$0|0)==($216|0);
  if ($217) {
   HEAP32[(9036)>>2] = $128;
   return;
  } else {
   $$1 = $128;
  }
 } else {
  $218 = $109 & -2;
  HEAP32[$108>>2] = $218;
  $219 = $$02 | 1;
  $220 = ((($$0)) + 4|0);
  HEAP32[$220>>2] = $219;
  $221 = (($$0) + ($$02)|0);
  HEAP32[$221>>2] = $$02;
  $$1 = $$02;
 }
 $222 = $$1 >>> 3;
 $223 = ($$1>>>0)<(256);
 if ($223) {
  $224 = $222 << 1;
  $225 = (9068 + ($224<<2)|0);
  $226 = HEAP32[9028>>2]|0;
  $227 = 1 << $222;
  $228 = $226 & $227;
  $229 = ($228|0)==(0);
  if ($229) {
   $230 = $226 | $227;
   HEAP32[9028>>2] = $230;
   $$pre = (($224) + 2)|0;
   $$pre48 = (9068 + ($$pre<<2)|0);
   $$pre$phiZ2D = $$pre48;$F16$0 = $225;
  } else {
   $$sum7 = (($224) + 2)|0;
   $231 = (9068 + ($$sum7<<2)|0);
   $232 = HEAP32[$231>>2]|0;
   $233 = HEAP32[(9044)>>2]|0;
   $234 = ($232>>>0)<($233>>>0);
   if ($234) {
    _abort();
    // unreachable;
   } else {
    $$pre$phiZ2D = $231;$F16$0 = $232;
   }
  }
  HEAP32[$$pre$phiZ2D>>2] = $$0;
  $235 = ((($F16$0)) + 12|0);
  HEAP32[$235>>2] = $$0;
  $236 = ((($$0)) + 8|0);
  HEAP32[$236>>2] = $F16$0;
  $237 = ((($$0)) + 12|0);
  HEAP32[$237>>2] = $225;
  return;
 }
 $238 = $$1 >>> 8;
 $239 = ($238|0)==(0);
 if ($239) {
  $I19$0 = 0;
 } else {
  $240 = ($$1>>>0)>(16777215);
  if ($240) {
   $I19$0 = 31;
  } else {
   $241 = (($238) + 1048320)|0;
   $242 = $241 >>> 16;
   $243 = $242 & 8;
   $244 = $238 << $243;
   $245 = (($244) + 520192)|0;
   $246 = $245 >>> 16;
   $247 = $246 & 4;
   $248 = $247 | $243;
   $249 = $244 << $247;
   $250 = (($249) + 245760)|0;
   $251 = $250 >>> 16;
   $252 = $251 & 2;
   $253 = $248 | $252;
   $254 = (14 - ($253))|0;
   $255 = $249 << $252;
   $256 = $255 >>> 15;
   $257 = (($254) + ($256))|0;
   $258 = $257 << 1;
   $259 = (($257) + 7)|0;
   $260 = $$1 >>> $259;
   $261 = $260 & 1;
   $262 = $261 | $258;
   $I19$0 = $262;
  }
 }
 $263 = (9332 + ($I19$0<<2)|0);
 $264 = ((($$0)) + 28|0);
 HEAP32[$264>>2] = $I19$0;
 $265 = ((($$0)) + 16|0);
 $266 = ((($$0)) + 20|0);
 HEAP32[$266>>2] = 0;
 HEAP32[$265>>2] = 0;
 $267 = HEAP32[(9032)>>2]|0;
 $268 = 1 << $I19$0;
 $269 = $267 & $268;
 $270 = ($269|0)==(0);
 if ($270) {
  $271 = $267 | $268;
  HEAP32[(9032)>>2] = $271;
  HEAP32[$263>>2] = $$0;
  $272 = ((($$0)) + 24|0);
  HEAP32[$272>>2] = $263;
  $273 = ((($$0)) + 12|0);
  HEAP32[$273>>2] = $$0;
  $274 = ((($$0)) + 8|0);
  HEAP32[$274>>2] = $$0;
  return;
 }
 $275 = HEAP32[$263>>2]|0;
 $276 = ((($275)) + 4|0);
 $277 = HEAP32[$276>>2]|0;
 $278 = $277 & -8;
 $279 = ($278|0)==($$1|0);
 L191: do {
  if ($279) {
   $T$0$lcssa = $275;
  } else {
   $280 = ($I19$0|0)==(31);
   $281 = $I19$0 >>> 1;
   $282 = (25 - ($281))|0;
   $283 = $280 ? 0 : $282;
   $284 = $$1 << $283;
   $K20$043 = $284;$T$042 = $275;
   while(1) {
    $291 = $K20$043 >>> 31;
    $292 = (((($T$042)) + 16|0) + ($291<<2)|0);
    $287 = HEAP32[$292>>2]|0;
    $293 = ($287|0)==(0|0);
    if ($293) {
     $$lcssa = $292;$T$042$lcssa = $T$042;
     break;
    }
    $285 = $K20$043 << 1;
    $286 = ((($287)) + 4|0);
    $288 = HEAP32[$286>>2]|0;
    $289 = $288 & -8;
    $290 = ($289|0)==($$1|0);
    if ($290) {
     $T$0$lcssa = $287;
     break L191;
    } else {
     $K20$043 = $285;$T$042 = $287;
    }
   }
   $294 = HEAP32[(9044)>>2]|0;
   $295 = ($$lcssa>>>0)<($294>>>0);
   if ($295) {
    _abort();
    // unreachable;
   }
   HEAP32[$$lcssa>>2] = $$0;
   $296 = ((($$0)) + 24|0);
   HEAP32[$296>>2] = $T$042$lcssa;
   $297 = ((($$0)) + 12|0);
   HEAP32[$297>>2] = $$0;
   $298 = ((($$0)) + 8|0);
   HEAP32[$298>>2] = $$0;
   return;
  }
 } while(0);
 $299 = ((($T$0$lcssa)) + 8|0);
 $300 = HEAP32[$299>>2]|0;
 $301 = HEAP32[(9044)>>2]|0;
 $302 = ($300>>>0)>=($301>>>0);
 $not$ = ($T$0$lcssa>>>0)>=($301>>>0);
 $303 = $302 & $not$;
 if (!($303)) {
  _abort();
  // unreachable;
 }
 $304 = ((($300)) + 12|0);
 HEAP32[$304>>2] = $$0;
 HEAP32[$299>>2] = $$0;
 $305 = ((($$0)) + 8|0);
 HEAP32[$305>>2] = $300;
 $306 = ((($$0)) + 12|0);
 HEAP32[$306>>2] = $T$0$lcssa;
 $307 = ((($$0)) + 24|0);
 HEAP32[$307>>2] = 0;
 return;
}
function runPostSets() {
}
function _i64Subtract(a, b, c, d) {
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a - c)>>>0;
    h = (b - d)>>>0;
    h = (b - d - (((c>>>0) > (a>>>0))|0))>>>0; // Borrow one from high word to low word on underflow.
    return ((tempRet0 = h,l|0)|0);
}
function _i64Add(a, b, c, d) {
    /*
      x = a + b*2^32
      y = c + d*2^32
      result = l + h*2^32
    */
    a = a|0; b = b|0; c = c|0; d = d|0;
    var l = 0, h = 0;
    l = (a + c)>>>0;
    h = (b + d + (((l>>>0) < (a>>>0))|0))>>>0; // Add carry from low word to high word on overflow.
    return ((tempRet0 = h,l|0)|0);
}
function _bitshift64Lshr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >>> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = 0;
    return (high >>> (bits - 32))|0;
}
function _memcpy(dest, src, num) {
    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if ((num|0) >= 4096) return _emscripten_memcpy_big(dest|0, src|0, num|0)|0;
    ret = dest|0;
    if ((dest&3) == (src&3)) {
      while (dest & 3) {
        if ((num|0) == 0) return ret|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
        dest = (dest+1)|0;
        src = (src+1)|0;
        num = (num-1)|0;
      }
      while ((num|0) >= 4) {
        HEAP32[((dest)>>2)]=((HEAP32[((src)>>2)])|0);
        dest = (dest+4)|0;
        src = (src+4)|0;
        num = (num-4)|0;
      }
    }
    while ((num|0) > 0) {
      HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      dest = (dest+1)|0;
      src = (src+1)|0;
      num = (num-1)|0;
    }
    return ret|0;
}
function _memmove(dest, src, num) {
    dest = dest|0; src = src|0; num = num|0;
    var ret = 0;
    if (((src|0) < (dest|0)) & ((dest|0) < ((src + num)|0))) {
      // Unlikely case: Copy backwards in a safe manner
      ret = dest;
      src = (src + num)|0;
      dest = (dest + num)|0;
      while ((num|0) > 0) {
        dest = (dest - 1)|0;
        src = (src - 1)|0;
        num = (num - 1)|0;
        HEAP8[((dest)>>0)]=((HEAP8[((src)>>0)])|0);
      }
      dest = ret;
    } else {
      _memcpy(dest, src, num) | 0;
    }
    return dest | 0;
}
function _memset(ptr, value, num) {
    ptr = ptr|0; value = value|0; num = num|0;
    var stop = 0, value4 = 0, stop4 = 0, unaligned = 0;
    stop = (ptr + num)|0;
    if ((num|0) >= 20) {
      // This is unaligned, but quite large, so work hard to get to aligned settings
      value = value & 0xff;
      unaligned = ptr & 3;
      value4 = value | (value << 8) | (value << 16) | (value << 24);
      stop4 = stop & ~3;
      if (unaligned) {
        unaligned = (ptr + 4 - unaligned)|0;
        while ((ptr|0) < (unaligned|0)) { // no need to check for stop, since we have large num
          HEAP8[((ptr)>>0)]=value;
          ptr = (ptr+1)|0;
        }
      }
      while ((ptr|0) < (stop4|0)) {
        HEAP32[((ptr)>>2)]=value4;
        ptr = (ptr+4)|0;
      }
    }
    while ((ptr|0) < (stop|0)) {
      HEAP8[((ptr)>>0)]=value;
      ptr = (ptr+1)|0;
    }
    return (ptr-num)|0;
}
function _bitshift64Shl(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = (high << bits) | ((low&(ander << (32 - bits))) >>> (32 - bits));
      return low << bits;
    }
    tempRet0 = low << (bits - 32);
    return 0;
}
function _bitshift64Ashr(low, high, bits) {
    low = low|0; high = high|0; bits = bits|0;
    var ander = 0;
    if ((bits|0) < 32) {
      ander = ((1 << bits) - 1)|0;
      tempRet0 = high >> bits;
      return (low >>> bits) | ((high&ander) << (32 - bits));
    }
    tempRet0 = (high|0) < 0 ? -1 : 0;
    return (high >> (bits - 32))|0;
  }
function _llvm_cttz_i32(x) {
    x = x|0;
    var ret = 0;
    ret = ((HEAP8[(((cttz_i8)+(x & 0xff))>>0)])|0);
    if ((ret|0) < 8) return ret|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 8)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 8)|0;
    ret = ((HEAP8[(((cttz_i8)+((x >> 16)&0xff))>>0)])|0);
    if ((ret|0) < 8) return (ret + 16)|0;
    return (((HEAP8[(((cttz_i8)+(x >>> 24))>>0)])|0) + 24)|0;
  }

// ======== compiled code from system/lib/compiler-rt , see readme therein
function ___muldsi3($a, $b) {
  $a = $a | 0;
  $b = $b | 0;
  var $1 = 0, $2 = 0, $3 = 0, $6 = 0, $8 = 0, $11 = 0, $12 = 0;
  $1 = $a & 65535;
  $2 = $b & 65535;
  $3 = Math_imul($2, $1) | 0;
  $6 = $a >>> 16;
  $8 = ($3 >>> 16) + (Math_imul($2, $6) | 0) | 0;
  $11 = $b >>> 16;
  $12 = Math_imul($11, $1) | 0;
  return (tempRet0 = (($8 >>> 16) + (Math_imul($11, $6) | 0) | 0) + ((($8 & 65535) + $12 | 0) >>> 16) | 0, 0 | ($8 + $12 << 16 | $3 & 65535)) | 0;
}
function ___divdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $7$0 = 0, $7$1 = 0, $8$0 = 0, $10$0 = 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  $7$0 = $2$0 ^ $1$0;
  $7$1 = $2$1 ^ $1$1;
  $8$0 = ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, 0) | 0;
  $10$0 = _i64Subtract($8$0 ^ $7$0, tempRet0 ^ $7$1, $7$0, $7$1) | 0;
  return $10$0 | 0;
}
function ___remdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, $1$0 = 0, $1$1 = 0, $2$0 = 0, $2$1 = 0, $4$0 = 0, $4$1 = 0, $6$0 = 0, $10$0 = 0, $10$1 = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 16 | 0;
  $rem = __stackBase__ | 0;
  $1$0 = $a$1 >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $1$1 = (($a$1 | 0) < 0 ? -1 : 0) >> 31 | (($a$1 | 0) < 0 ? -1 : 0) << 1;
  $2$0 = $b$1 >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $2$1 = (($b$1 | 0) < 0 ? -1 : 0) >> 31 | (($b$1 | 0) < 0 ? -1 : 0) << 1;
  $4$0 = _i64Subtract($1$0 ^ $a$0, $1$1 ^ $a$1, $1$0, $1$1) | 0;
  $4$1 = tempRet0;
  $6$0 = _i64Subtract($2$0 ^ $b$0, $2$1 ^ $b$1, $2$0, $2$1) | 0;
  ___udivmoddi4($4$0, $4$1, $6$0, tempRet0, $rem) | 0;
  $10$0 = _i64Subtract(HEAP32[$rem >> 2] ^ $1$0, HEAP32[$rem + 4 >> 2] ^ $1$1, $1$0, $1$1) | 0;
  $10$1 = tempRet0;
  STACKTOP = __stackBase__;
  return (tempRet0 = $10$1, $10$0) | 0;
}
function ___muldi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $x_sroa_0_0_extract_trunc = 0, $y_sroa_0_0_extract_trunc = 0, $1$0 = 0, $1$1 = 0, $2 = 0;
  $x_sroa_0_0_extract_trunc = $a$0;
  $y_sroa_0_0_extract_trunc = $b$0;
  $1$0 = ___muldsi3($x_sroa_0_0_extract_trunc, $y_sroa_0_0_extract_trunc) | 0;
  $1$1 = tempRet0;
  $2 = Math_imul($a$1, $y_sroa_0_0_extract_trunc) | 0;
  return (tempRet0 = ((Math_imul($b$1, $x_sroa_0_0_extract_trunc) | 0) + $2 | 0) + $1$1 | $1$1 & 0, 0 | $1$0 & -1) | 0;
}
function ___udivdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $1$0 = 0;
  $1$0 = ___udivmoddi4($a$0, $a$1, $b$0, $b$1, 0) | 0;
  return $1$0 | 0;
}
function ___uremdi3($a$0, $a$1, $b$0, $b$1) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  var $rem = 0, __stackBase__ = 0;
  __stackBase__ = STACKTOP;
  STACKTOP = STACKTOP + 16 | 0;
  $rem = __stackBase__ | 0;
  ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) | 0;
  STACKTOP = __stackBase__;
  return (tempRet0 = HEAP32[$rem + 4 >> 2] | 0, HEAP32[$rem >> 2] | 0) | 0;
}
function ___udivmoddi4($a$0, $a$1, $b$0, $b$1, $rem) {
  $a$0 = $a$0 | 0;
  $a$1 = $a$1 | 0;
  $b$0 = $b$0 | 0;
  $b$1 = $b$1 | 0;
  $rem = $rem | 0;
  var $n_sroa_0_0_extract_trunc = 0, $n_sroa_1_4_extract_shift$0 = 0, $n_sroa_1_4_extract_trunc = 0, $d_sroa_0_0_extract_trunc = 0, $d_sroa_1_4_extract_shift$0 = 0, $d_sroa_1_4_extract_trunc = 0, $4 = 0, $17 = 0, $37 = 0, $49 = 0, $51 = 0, $57 = 0, $58 = 0, $66 = 0, $78 = 0, $86 = 0, $88 = 0, $89 = 0, $91 = 0, $92 = 0, $95 = 0, $105 = 0, $117 = 0, $119 = 0, $125 = 0, $126 = 0, $130 = 0, $q_sroa_1_1_ph = 0, $q_sroa_0_1_ph = 0, $r_sroa_1_1_ph = 0, $r_sroa_0_1_ph = 0, $sr_1_ph = 0, $d_sroa_0_0_insert_insert99$0 = 0, $d_sroa_0_0_insert_insert99$1 = 0, $137$0 = 0, $137$1 = 0, $carry_0203 = 0, $sr_1202 = 0, $r_sroa_0_1201 = 0, $r_sroa_1_1200 = 0, $q_sroa_0_1199 = 0, $q_sroa_1_1198 = 0, $147 = 0, $149 = 0, $r_sroa_0_0_insert_insert42$0 = 0, $r_sroa_0_0_insert_insert42$1 = 0, $150$1 = 0, $151$0 = 0, $152 = 0, $154$0 = 0, $r_sroa_0_0_extract_trunc = 0, $r_sroa_1_4_extract_trunc = 0, $155 = 0, $carry_0_lcssa$0 = 0, $carry_0_lcssa$1 = 0, $r_sroa_0_1_lcssa = 0, $r_sroa_1_1_lcssa = 0, $q_sroa_0_1_lcssa = 0, $q_sroa_1_1_lcssa = 0, $q_sroa_0_0_insert_ext75$0 = 0, $q_sroa_0_0_insert_ext75$1 = 0, $q_sroa_0_0_insert_insert77$1 = 0, $_0$0 = 0, $_0$1 = 0;
  $n_sroa_0_0_extract_trunc = $a$0;
  $n_sroa_1_4_extract_shift$0 = $a$1;
  $n_sroa_1_4_extract_trunc = $n_sroa_1_4_extract_shift$0;
  $d_sroa_0_0_extract_trunc = $b$0;
  $d_sroa_1_4_extract_shift$0 = $b$1;
  $d_sroa_1_4_extract_trunc = $d_sroa_1_4_extract_shift$0;
  if (($n_sroa_1_4_extract_trunc | 0) == 0) {
    $4 = ($rem | 0) != 0;
    if (($d_sroa_1_4_extract_trunc | 0) == 0) {
      if ($4) {
        HEAP32[$rem >> 2] = ($n_sroa_0_0_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
        HEAP32[$rem + 4 >> 2] = 0;
      }
      $_0$1 = 0;
      $_0$0 = ($n_sroa_0_0_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$4) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    }
  }
  $17 = ($d_sroa_1_4_extract_trunc | 0) == 0;
  do {
    if (($d_sroa_0_0_extract_trunc | 0) == 0) {
      if ($17) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_0_0_extract_trunc >>> 0);
          HEAP32[$rem + 4 >> 2] = 0;
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_0_0_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      if (($n_sroa_0_0_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0;
          HEAP32[$rem + 4 >> 2] = ($n_sroa_1_4_extract_trunc >>> 0) % ($d_sroa_1_4_extract_trunc >>> 0);
        }
        $_0$1 = 0;
        $_0$0 = ($n_sroa_1_4_extract_trunc >>> 0) / ($d_sroa_1_4_extract_trunc >>> 0) >>> 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $37 = $d_sroa_1_4_extract_trunc - 1 | 0;
      if (($37 & $d_sroa_1_4_extract_trunc | 0) == 0) {
        if (($rem | 0) != 0) {
          HEAP32[$rem >> 2] = 0 | $a$0 & -1;
          HEAP32[$rem + 4 >> 2] = $37 & $n_sroa_1_4_extract_trunc | $a$1 & 0;
        }
        $_0$1 = 0;
        $_0$0 = $n_sroa_1_4_extract_trunc >>> ((_llvm_cttz_i32($d_sroa_1_4_extract_trunc | 0) | 0) >>> 0);
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $49 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
      $51 = $49 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
      if ($51 >>> 0 <= 30) {
        $57 = $51 + 1 | 0;
        $58 = 31 - $51 | 0;
        $sr_1_ph = $57;
        $r_sroa_0_1_ph = $n_sroa_1_4_extract_trunc << $58 | $n_sroa_0_0_extract_trunc >>> ($57 >>> 0);
        $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($57 >>> 0);
        $q_sroa_0_1_ph = 0;
        $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $58;
        break;
      }
      if (($rem | 0) == 0) {
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      HEAP32[$rem >> 2] = 0 | $a$0 & -1;
      HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
      $_0$1 = 0;
      $_0$0 = 0;
      return (tempRet0 = $_0$1, $_0$0) | 0;
    } else {
      if (!$17) {
        $117 = Math_clz32($d_sroa_1_4_extract_trunc | 0) | 0;
        $119 = $117 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        if ($119 >>> 0 <= 31) {
          $125 = $119 + 1 | 0;
          $126 = 31 - $119 | 0;
          $130 = $119 - 31 >> 31;
          $sr_1_ph = $125;
          $r_sroa_0_1_ph = $n_sroa_0_0_extract_trunc >>> ($125 >>> 0) & $130 | $n_sroa_1_4_extract_trunc << $126;
          $r_sroa_1_1_ph = $n_sroa_1_4_extract_trunc >>> ($125 >>> 0) & $130;
          $q_sroa_0_1_ph = 0;
          $q_sroa_1_1_ph = $n_sroa_0_0_extract_trunc << $126;
          break;
        }
        if (($rem | 0) == 0) {
          $_0$1 = 0;
          $_0$0 = 0;
          return (tempRet0 = $_0$1, $_0$0) | 0;
        }
        HEAP32[$rem >> 2] = 0 | $a$0 & -1;
        HEAP32[$rem + 4 >> 2] = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$1 = 0;
        $_0$0 = 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
      $66 = $d_sroa_0_0_extract_trunc - 1 | 0;
      if (($66 & $d_sroa_0_0_extract_trunc | 0) != 0) {
        $86 = (Math_clz32($d_sroa_0_0_extract_trunc | 0) | 0) + 33 | 0;
        $88 = $86 - (Math_clz32($n_sroa_1_4_extract_trunc | 0) | 0) | 0;
        $89 = 64 - $88 | 0;
        $91 = 32 - $88 | 0;
        $92 = $91 >> 31;
        $95 = $88 - 32 | 0;
        $105 = $95 >> 31;
        $sr_1_ph = $88;
        $r_sroa_0_1_ph = $91 - 1 >> 31 & $n_sroa_1_4_extract_trunc >>> ($95 >>> 0) | ($n_sroa_1_4_extract_trunc << $91 | $n_sroa_0_0_extract_trunc >>> ($88 >>> 0)) & $105;
        $r_sroa_1_1_ph = $105 & $n_sroa_1_4_extract_trunc >>> ($88 >>> 0);
        $q_sroa_0_1_ph = $n_sroa_0_0_extract_trunc << $89 & $92;
        $q_sroa_1_1_ph = ($n_sroa_1_4_extract_trunc << $89 | $n_sroa_0_0_extract_trunc >>> ($95 >>> 0)) & $92 | $n_sroa_0_0_extract_trunc << $91 & $88 - 33 >> 31;
        break;
      }
      if (($rem | 0) != 0) {
        HEAP32[$rem >> 2] = $66 & $n_sroa_0_0_extract_trunc;
        HEAP32[$rem + 4 >> 2] = 0;
      }
      if (($d_sroa_0_0_extract_trunc | 0) == 1) {
        $_0$1 = $n_sroa_1_4_extract_shift$0 | $a$1 & 0;
        $_0$0 = 0 | $a$0 & -1;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      } else {
        $78 = _llvm_cttz_i32($d_sroa_0_0_extract_trunc | 0) | 0;
        $_0$1 = 0 | $n_sroa_1_4_extract_trunc >>> ($78 >>> 0);
        $_0$0 = $n_sroa_1_4_extract_trunc << 32 - $78 | $n_sroa_0_0_extract_trunc >>> ($78 >>> 0) | 0;
        return (tempRet0 = $_0$1, $_0$0) | 0;
      }
    }
  } while (0);
  if (($sr_1_ph | 0) == 0) {
    $q_sroa_1_1_lcssa = $q_sroa_1_1_ph;
    $q_sroa_0_1_lcssa = $q_sroa_0_1_ph;
    $r_sroa_1_1_lcssa = $r_sroa_1_1_ph;
    $r_sroa_0_1_lcssa = $r_sroa_0_1_ph;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = 0;
  } else {
    $d_sroa_0_0_insert_insert99$0 = 0 | $b$0 & -1;
    $d_sroa_0_0_insert_insert99$1 = $d_sroa_1_4_extract_shift$0 | $b$1 & 0;
    $137$0 = _i64Add($d_sroa_0_0_insert_insert99$0 | 0, $d_sroa_0_0_insert_insert99$1 | 0, -1, -1) | 0;
    $137$1 = tempRet0;
    $q_sroa_1_1198 = $q_sroa_1_1_ph;
    $q_sroa_0_1199 = $q_sroa_0_1_ph;
    $r_sroa_1_1200 = $r_sroa_1_1_ph;
    $r_sroa_0_1201 = $r_sroa_0_1_ph;
    $sr_1202 = $sr_1_ph;
    $carry_0203 = 0;
    while (1) {
      $147 = $q_sroa_0_1199 >>> 31 | $q_sroa_1_1198 << 1;
      $149 = $carry_0203 | $q_sroa_0_1199 << 1;
      $r_sroa_0_0_insert_insert42$0 = 0 | ($r_sroa_0_1201 << 1 | $q_sroa_1_1198 >>> 31);
      $r_sroa_0_0_insert_insert42$1 = $r_sroa_0_1201 >>> 31 | $r_sroa_1_1200 << 1 | 0;
      _i64Subtract($137$0, $137$1, $r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1) | 0;
      $150$1 = tempRet0;
      $151$0 = $150$1 >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1;
      $152 = $151$0 & 1;
      $154$0 = _i64Subtract($r_sroa_0_0_insert_insert42$0, $r_sroa_0_0_insert_insert42$1, $151$0 & $d_sroa_0_0_insert_insert99$0, ((($150$1 | 0) < 0 ? -1 : 0) >> 31 | (($150$1 | 0) < 0 ? -1 : 0) << 1) & $d_sroa_0_0_insert_insert99$1) | 0;
      $r_sroa_0_0_extract_trunc = $154$0;
      $r_sroa_1_4_extract_trunc = tempRet0;
      $155 = $sr_1202 - 1 | 0;
      if (($155 | 0) == 0) {
        break;
      } else {
        $q_sroa_1_1198 = $147;
        $q_sroa_0_1199 = $149;
        $r_sroa_1_1200 = $r_sroa_1_4_extract_trunc;
        $r_sroa_0_1201 = $r_sroa_0_0_extract_trunc;
        $sr_1202 = $155;
        $carry_0203 = $152;
      }
    }
    $q_sroa_1_1_lcssa = $147;
    $q_sroa_0_1_lcssa = $149;
    $r_sroa_1_1_lcssa = $r_sroa_1_4_extract_trunc;
    $r_sroa_0_1_lcssa = $r_sroa_0_0_extract_trunc;
    $carry_0_lcssa$1 = 0;
    $carry_0_lcssa$0 = $152;
  }
  $q_sroa_0_0_insert_ext75$0 = $q_sroa_0_1_lcssa;
  $q_sroa_0_0_insert_ext75$1 = 0;
  $q_sroa_0_0_insert_insert77$1 = $q_sroa_1_1_lcssa | $q_sroa_0_0_insert_ext75$1;
  if (($rem | 0) != 0) {
    HEAP32[$rem >> 2] = 0 | $r_sroa_0_1_lcssa;
    HEAP32[$rem + 4 >> 2] = $r_sroa_1_1_lcssa | 0;
  }
  $_0$1 = (0 | $q_sroa_0_0_insert_ext75$0) >>> 31 | $q_sroa_0_0_insert_insert77$1 << 1 | ($q_sroa_0_0_insert_ext75$1 << 1 | $q_sroa_0_0_insert_ext75$0 >>> 31) & 0 | $carry_0_lcssa$1;
  $_0$0 = ($q_sroa_0_0_insert_ext75$0 << 1 | 0 >>> 31) & -2 | $carry_0_lcssa$0;
  return (tempRet0 = $_0$1, $_0$0) | 0;
}
// =======================================================================



  
function dynCall_iiii(index,a1,a2,a3) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0;
  return FUNCTION_TABLE_iiii[index&255](a1|0,a2|0,a3|0)|0;
}


function jsCall_iiii_0(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(0,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_1(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(1,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_2(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(2,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_3(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(3,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_4(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(4,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_5(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(5,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_6(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(6,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_7(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(7,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_8(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(8,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_9(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(9,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_10(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(10,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_11(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(11,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_12(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(12,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_13(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(13,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_14(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(14,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_15(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(15,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_16(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(16,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_17(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(17,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_18(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(18,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_19(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(19,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_20(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(20,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_21(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(21,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_22(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(22,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_23(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(23,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_24(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(24,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_25(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(25,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_26(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(26,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_27(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(27,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_28(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(28,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_29(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(29,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_30(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(30,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_31(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(31,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_32(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(32,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_33(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(33,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_34(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(34,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_35(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(35,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_36(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(36,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_37(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(37,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_38(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(38,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_39(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(39,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_40(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(40,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_41(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(41,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_42(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(42,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_43(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(43,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_44(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(44,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_45(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(45,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_46(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(46,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_47(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(47,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_48(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(48,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_49(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(49,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_50(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(50,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_51(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(51,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_52(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(52,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_53(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(53,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_54(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(54,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_55(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(55,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_56(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(56,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_57(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(57,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_58(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(58,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_59(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(59,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_60(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(60,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_61(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(61,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_62(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(62,a1|0,a2|0,a3|0)|0;
}



function jsCall_iiii_63(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  return jsCall_iiii(63,a1|0,a2|0,a3|0)|0;
}



function dynCall_i(index) {
  index = index|0;
  
  return FUNCTION_TABLE_i[index&255]()|0;
}


function jsCall_i_0() {
  
  return jsCall_i(0)|0;
}



function jsCall_i_1() {
  
  return jsCall_i(1)|0;
}



function jsCall_i_2() {
  
  return jsCall_i(2)|0;
}



function jsCall_i_3() {
  
  return jsCall_i(3)|0;
}



function jsCall_i_4() {
  
  return jsCall_i(4)|0;
}



function jsCall_i_5() {
  
  return jsCall_i(5)|0;
}



function jsCall_i_6() {
  
  return jsCall_i(6)|0;
}



function jsCall_i_7() {
  
  return jsCall_i(7)|0;
}



function jsCall_i_8() {
  
  return jsCall_i(8)|0;
}



function jsCall_i_9() {
  
  return jsCall_i(9)|0;
}



function jsCall_i_10() {
  
  return jsCall_i(10)|0;
}



function jsCall_i_11() {
  
  return jsCall_i(11)|0;
}



function jsCall_i_12() {
  
  return jsCall_i(12)|0;
}



function jsCall_i_13() {
  
  return jsCall_i(13)|0;
}



function jsCall_i_14() {
  
  return jsCall_i(14)|0;
}



function jsCall_i_15() {
  
  return jsCall_i(15)|0;
}



function jsCall_i_16() {
  
  return jsCall_i(16)|0;
}



function jsCall_i_17() {
  
  return jsCall_i(17)|0;
}



function jsCall_i_18() {
  
  return jsCall_i(18)|0;
}



function jsCall_i_19() {
  
  return jsCall_i(19)|0;
}



function jsCall_i_20() {
  
  return jsCall_i(20)|0;
}



function jsCall_i_21() {
  
  return jsCall_i(21)|0;
}



function jsCall_i_22() {
  
  return jsCall_i(22)|0;
}



function jsCall_i_23() {
  
  return jsCall_i(23)|0;
}



function jsCall_i_24() {
  
  return jsCall_i(24)|0;
}



function jsCall_i_25() {
  
  return jsCall_i(25)|0;
}



function jsCall_i_26() {
  
  return jsCall_i(26)|0;
}



function jsCall_i_27() {
  
  return jsCall_i(27)|0;
}



function jsCall_i_28() {
  
  return jsCall_i(28)|0;
}



function jsCall_i_29() {
  
  return jsCall_i(29)|0;
}



function jsCall_i_30() {
  
  return jsCall_i(30)|0;
}



function jsCall_i_31() {
  
  return jsCall_i(31)|0;
}



function jsCall_i_32() {
  
  return jsCall_i(32)|0;
}



function jsCall_i_33() {
  
  return jsCall_i(33)|0;
}



function jsCall_i_34() {
  
  return jsCall_i(34)|0;
}



function jsCall_i_35() {
  
  return jsCall_i(35)|0;
}



function jsCall_i_36() {
  
  return jsCall_i(36)|0;
}



function jsCall_i_37() {
  
  return jsCall_i(37)|0;
}



function jsCall_i_38() {
  
  return jsCall_i(38)|0;
}



function jsCall_i_39() {
  
  return jsCall_i(39)|0;
}



function jsCall_i_40() {
  
  return jsCall_i(40)|0;
}



function jsCall_i_41() {
  
  return jsCall_i(41)|0;
}



function jsCall_i_42() {
  
  return jsCall_i(42)|0;
}



function jsCall_i_43() {
  
  return jsCall_i(43)|0;
}



function jsCall_i_44() {
  
  return jsCall_i(44)|0;
}



function jsCall_i_45() {
  
  return jsCall_i(45)|0;
}



function jsCall_i_46() {
  
  return jsCall_i(46)|0;
}



function jsCall_i_47() {
  
  return jsCall_i(47)|0;
}



function jsCall_i_48() {
  
  return jsCall_i(48)|0;
}



function jsCall_i_49() {
  
  return jsCall_i(49)|0;
}



function jsCall_i_50() {
  
  return jsCall_i(50)|0;
}



function jsCall_i_51() {
  
  return jsCall_i(51)|0;
}



function jsCall_i_52() {
  
  return jsCall_i(52)|0;
}



function jsCall_i_53() {
  
  return jsCall_i(53)|0;
}



function jsCall_i_54() {
  
  return jsCall_i(54)|0;
}



function jsCall_i_55() {
  
  return jsCall_i(55)|0;
}



function jsCall_i_56() {
  
  return jsCall_i(56)|0;
}



function jsCall_i_57() {
  
  return jsCall_i(57)|0;
}



function jsCall_i_58() {
  
  return jsCall_i(58)|0;
}



function jsCall_i_59() {
  
  return jsCall_i(59)|0;
}



function jsCall_i_60() {
  
  return jsCall_i(60)|0;
}



function jsCall_i_61() {
  
  return jsCall_i(61)|0;
}



function jsCall_i_62() {
  
  return jsCall_i(62)|0;
}



function jsCall_i_63() {
  
  return jsCall_i(63)|0;
}



function dynCall_vi(index,a1) {
  index = index|0;
  a1=a1|0;
  FUNCTION_TABLE_vi[index&255](a1|0);
}


function jsCall_vi_0(a1) {
  a1=a1|0;
  jsCall_vi(0,a1|0);
}



function jsCall_vi_1(a1) {
  a1=a1|0;
  jsCall_vi(1,a1|0);
}



function jsCall_vi_2(a1) {
  a1=a1|0;
  jsCall_vi(2,a1|0);
}



function jsCall_vi_3(a1) {
  a1=a1|0;
  jsCall_vi(3,a1|0);
}



function jsCall_vi_4(a1) {
  a1=a1|0;
  jsCall_vi(4,a1|0);
}



function jsCall_vi_5(a1) {
  a1=a1|0;
  jsCall_vi(5,a1|0);
}



function jsCall_vi_6(a1) {
  a1=a1|0;
  jsCall_vi(6,a1|0);
}



function jsCall_vi_7(a1) {
  a1=a1|0;
  jsCall_vi(7,a1|0);
}



function jsCall_vi_8(a1) {
  a1=a1|0;
  jsCall_vi(8,a1|0);
}



function jsCall_vi_9(a1) {
  a1=a1|0;
  jsCall_vi(9,a1|0);
}



function jsCall_vi_10(a1) {
  a1=a1|0;
  jsCall_vi(10,a1|0);
}



function jsCall_vi_11(a1) {
  a1=a1|0;
  jsCall_vi(11,a1|0);
}



function jsCall_vi_12(a1) {
  a1=a1|0;
  jsCall_vi(12,a1|0);
}



function jsCall_vi_13(a1) {
  a1=a1|0;
  jsCall_vi(13,a1|0);
}



function jsCall_vi_14(a1) {
  a1=a1|0;
  jsCall_vi(14,a1|0);
}



function jsCall_vi_15(a1) {
  a1=a1|0;
  jsCall_vi(15,a1|0);
}



function jsCall_vi_16(a1) {
  a1=a1|0;
  jsCall_vi(16,a1|0);
}



function jsCall_vi_17(a1) {
  a1=a1|0;
  jsCall_vi(17,a1|0);
}



function jsCall_vi_18(a1) {
  a1=a1|0;
  jsCall_vi(18,a1|0);
}



function jsCall_vi_19(a1) {
  a1=a1|0;
  jsCall_vi(19,a1|0);
}



function jsCall_vi_20(a1) {
  a1=a1|0;
  jsCall_vi(20,a1|0);
}



function jsCall_vi_21(a1) {
  a1=a1|0;
  jsCall_vi(21,a1|0);
}



function jsCall_vi_22(a1) {
  a1=a1|0;
  jsCall_vi(22,a1|0);
}



function jsCall_vi_23(a1) {
  a1=a1|0;
  jsCall_vi(23,a1|0);
}



function jsCall_vi_24(a1) {
  a1=a1|0;
  jsCall_vi(24,a1|0);
}



function jsCall_vi_25(a1) {
  a1=a1|0;
  jsCall_vi(25,a1|0);
}



function jsCall_vi_26(a1) {
  a1=a1|0;
  jsCall_vi(26,a1|0);
}



function jsCall_vi_27(a1) {
  a1=a1|0;
  jsCall_vi(27,a1|0);
}



function jsCall_vi_28(a1) {
  a1=a1|0;
  jsCall_vi(28,a1|0);
}



function jsCall_vi_29(a1) {
  a1=a1|0;
  jsCall_vi(29,a1|0);
}



function jsCall_vi_30(a1) {
  a1=a1|0;
  jsCall_vi(30,a1|0);
}



function jsCall_vi_31(a1) {
  a1=a1|0;
  jsCall_vi(31,a1|0);
}



function jsCall_vi_32(a1) {
  a1=a1|0;
  jsCall_vi(32,a1|0);
}



function jsCall_vi_33(a1) {
  a1=a1|0;
  jsCall_vi(33,a1|0);
}



function jsCall_vi_34(a1) {
  a1=a1|0;
  jsCall_vi(34,a1|0);
}



function jsCall_vi_35(a1) {
  a1=a1|0;
  jsCall_vi(35,a1|0);
}



function jsCall_vi_36(a1) {
  a1=a1|0;
  jsCall_vi(36,a1|0);
}



function jsCall_vi_37(a1) {
  a1=a1|0;
  jsCall_vi(37,a1|0);
}



function jsCall_vi_38(a1) {
  a1=a1|0;
  jsCall_vi(38,a1|0);
}



function jsCall_vi_39(a1) {
  a1=a1|0;
  jsCall_vi(39,a1|0);
}



function jsCall_vi_40(a1) {
  a1=a1|0;
  jsCall_vi(40,a1|0);
}



function jsCall_vi_41(a1) {
  a1=a1|0;
  jsCall_vi(41,a1|0);
}



function jsCall_vi_42(a1) {
  a1=a1|0;
  jsCall_vi(42,a1|0);
}



function jsCall_vi_43(a1) {
  a1=a1|0;
  jsCall_vi(43,a1|0);
}



function jsCall_vi_44(a1) {
  a1=a1|0;
  jsCall_vi(44,a1|0);
}



function jsCall_vi_45(a1) {
  a1=a1|0;
  jsCall_vi(45,a1|0);
}



function jsCall_vi_46(a1) {
  a1=a1|0;
  jsCall_vi(46,a1|0);
}



function jsCall_vi_47(a1) {
  a1=a1|0;
  jsCall_vi(47,a1|0);
}



function jsCall_vi_48(a1) {
  a1=a1|0;
  jsCall_vi(48,a1|0);
}



function jsCall_vi_49(a1) {
  a1=a1|0;
  jsCall_vi(49,a1|0);
}



function jsCall_vi_50(a1) {
  a1=a1|0;
  jsCall_vi(50,a1|0);
}



function jsCall_vi_51(a1) {
  a1=a1|0;
  jsCall_vi(51,a1|0);
}



function jsCall_vi_52(a1) {
  a1=a1|0;
  jsCall_vi(52,a1|0);
}



function jsCall_vi_53(a1) {
  a1=a1|0;
  jsCall_vi(53,a1|0);
}



function jsCall_vi_54(a1) {
  a1=a1|0;
  jsCall_vi(54,a1|0);
}



function jsCall_vi_55(a1) {
  a1=a1|0;
  jsCall_vi(55,a1|0);
}



function jsCall_vi_56(a1) {
  a1=a1|0;
  jsCall_vi(56,a1|0);
}



function jsCall_vi_57(a1) {
  a1=a1|0;
  jsCall_vi(57,a1|0);
}



function jsCall_vi_58(a1) {
  a1=a1|0;
  jsCall_vi(58,a1|0);
}



function jsCall_vi_59(a1) {
  a1=a1|0;
  jsCall_vi(59,a1|0);
}



function jsCall_vi_60(a1) {
  a1=a1|0;
  jsCall_vi(60,a1|0);
}



function jsCall_vi_61(a1) {
  a1=a1|0;
  jsCall_vi(61,a1|0);
}



function jsCall_vi_62(a1) {
  a1=a1|0;
  jsCall_vi(62,a1|0);
}



function jsCall_vi_63(a1) {
  a1=a1|0;
  jsCall_vi(63,a1|0);
}



function dynCall_vii(index,a1,a2) {
  index = index|0;
  a1=a1|0; a2=a2|0;
  FUNCTION_TABLE_vii[index&255](a1|0,a2|0);
}


function jsCall_vii_0(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(0,a1|0,a2|0);
}



function jsCall_vii_1(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(1,a1|0,a2|0);
}



function jsCall_vii_2(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(2,a1|0,a2|0);
}



function jsCall_vii_3(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(3,a1|0,a2|0);
}



function jsCall_vii_4(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(4,a1|0,a2|0);
}



function jsCall_vii_5(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(5,a1|0,a2|0);
}



function jsCall_vii_6(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(6,a1|0,a2|0);
}



function jsCall_vii_7(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(7,a1|0,a2|0);
}



function jsCall_vii_8(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(8,a1|0,a2|0);
}



function jsCall_vii_9(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(9,a1|0,a2|0);
}



function jsCall_vii_10(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(10,a1|0,a2|0);
}



function jsCall_vii_11(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(11,a1|0,a2|0);
}



function jsCall_vii_12(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(12,a1|0,a2|0);
}



function jsCall_vii_13(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(13,a1|0,a2|0);
}



function jsCall_vii_14(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(14,a1|0,a2|0);
}



function jsCall_vii_15(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(15,a1|0,a2|0);
}



function jsCall_vii_16(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(16,a1|0,a2|0);
}



function jsCall_vii_17(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(17,a1|0,a2|0);
}



function jsCall_vii_18(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(18,a1|0,a2|0);
}



function jsCall_vii_19(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(19,a1|0,a2|0);
}



function jsCall_vii_20(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(20,a1|0,a2|0);
}



function jsCall_vii_21(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(21,a1|0,a2|0);
}



function jsCall_vii_22(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(22,a1|0,a2|0);
}



function jsCall_vii_23(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(23,a1|0,a2|0);
}



function jsCall_vii_24(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(24,a1|0,a2|0);
}



function jsCall_vii_25(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(25,a1|0,a2|0);
}



function jsCall_vii_26(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(26,a1|0,a2|0);
}



function jsCall_vii_27(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(27,a1|0,a2|0);
}



function jsCall_vii_28(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(28,a1|0,a2|0);
}



function jsCall_vii_29(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(29,a1|0,a2|0);
}



function jsCall_vii_30(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(30,a1|0,a2|0);
}



function jsCall_vii_31(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(31,a1|0,a2|0);
}



function jsCall_vii_32(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(32,a1|0,a2|0);
}



function jsCall_vii_33(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(33,a1|0,a2|0);
}



function jsCall_vii_34(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(34,a1|0,a2|0);
}



function jsCall_vii_35(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(35,a1|0,a2|0);
}



function jsCall_vii_36(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(36,a1|0,a2|0);
}



function jsCall_vii_37(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(37,a1|0,a2|0);
}



function jsCall_vii_38(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(38,a1|0,a2|0);
}



function jsCall_vii_39(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(39,a1|0,a2|0);
}



function jsCall_vii_40(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(40,a1|0,a2|0);
}



function jsCall_vii_41(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(41,a1|0,a2|0);
}



function jsCall_vii_42(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(42,a1|0,a2|0);
}



function jsCall_vii_43(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(43,a1|0,a2|0);
}



function jsCall_vii_44(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(44,a1|0,a2|0);
}



function jsCall_vii_45(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(45,a1|0,a2|0);
}



function jsCall_vii_46(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(46,a1|0,a2|0);
}



function jsCall_vii_47(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(47,a1|0,a2|0);
}



function jsCall_vii_48(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(48,a1|0,a2|0);
}



function jsCall_vii_49(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(49,a1|0,a2|0);
}



function jsCall_vii_50(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(50,a1|0,a2|0);
}



function jsCall_vii_51(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(51,a1|0,a2|0);
}



function jsCall_vii_52(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(52,a1|0,a2|0);
}



function jsCall_vii_53(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(53,a1|0,a2|0);
}



function jsCall_vii_54(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(54,a1|0,a2|0);
}



function jsCall_vii_55(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(55,a1|0,a2|0);
}



function jsCall_vii_56(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(56,a1|0,a2|0);
}



function jsCall_vii_57(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(57,a1|0,a2|0);
}



function jsCall_vii_58(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(58,a1|0,a2|0);
}



function jsCall_vii_59(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(59,a1|0,a2|0);
}



function jsCall_vii_60(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(60,a1|0,a2|0);
}



function jsCall_vii_61(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(61,a1|0,a2|0);
}



function jsCall_vii_62(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(62,a1|0,a2|0);
}



function jsCall_vii_63(a1,a2) {
  a1=a1|0; a2=a2|0;
  jsCall_vii(63,a1|0,a2|0);
}



function dynCall_iiiiiii(index,a1,a2,a3,a4,a5,a6) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return FUNCTION_TABLE_iiiiiii[index&255](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}


function jsCall_iiiiiii_0(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(0,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_1(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(1,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_2(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(2,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_3(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(3,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_4(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(4,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_5(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(5,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_6(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(6,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_7(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(7,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_8(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(8,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_9(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(9,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_10(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(10,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_11(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(11,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_12(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(12,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_13(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(13,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_14(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(14,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_15(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(15,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_16(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(16,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_17(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(17,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_18(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(18,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_19(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(19,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_20(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(20,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_21(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(21,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_22(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(22,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_23(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(23,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_24(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(24,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_25(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(25,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_26(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(26,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_27(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(27,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_28(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(28,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_29(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(29,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_30(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(30,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_31(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(31,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_32(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(32,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_33(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(33,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_34(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(34,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_35(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(35,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_36(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(36,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_37(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(37,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_38(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(38,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_39(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(39,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_40(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(40,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_41(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(41,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_42(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(42,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_43(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(43,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_44(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(44,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_45(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(45,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_46(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(46,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_47(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(47,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_48(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(48,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_49(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(49,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_50(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(50,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_51(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(51,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_52(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(52,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_53(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(53,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_54(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(54,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_55(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(55,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_56(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(56,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_57(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(57,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_58(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(58,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_59(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(59,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_60(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(60,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_61(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(61,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_62(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(62,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function jsCall_iiiiiii_63(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  return jsCall_iiiiiii(63,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0)|0;
}



function dynCall_ii(index,a1) {
  index = index|0;
  a1=a1|0;
  return FUNCTION_TABLE_ii[index&255](a1|0)|0;
}


function jsCall_ii_0(a1) {
  a1=a1|0;
  return jsCall_ii(0,a1|0)|0;
}



function jsCall_ii_1(a1) {
  a1=a1|0;
  return jsCall_ii(1,a1|0)|0;
}



function jsCall_ii_2(a1) {
  a1=a1|0;
  return jsCall_ii(2,a1|0)|0;
}



function jsCall_ii_3(a1) {
  a1=a1|0;
  return jsCall_ii(3,a1|0)|0;
}



function jsCall_ii_4(a1) {
  a1=a1|0;
  return jsCall_ii(4,a1|0)|0;
}



function jsCall_ii_5(a1) {
  a1=a1|0;
  return jsCall_ii(5,a1|0)|0;
}



function jsCall_ii_6(a1) {
  a1=a1|0;
  return jsCall_ii(6,a1|0)|0;
}



function jsCall_ii_7(a1) {
  a1=a1|0;
  return jsCall_ii(7,a1|0)|0;
}



function jsCall_ii_8(a1) {
  a1=a1|0;
  return jsCall_ii(8,a1|0)|0;
}



function jsCall_ii_9(a1) {
  a1=a1|0;
  return jsCall_ii(9,a1|0)|0;
}



function jsCall_ii_10(a1) {
  a1=a1|0;
  return jsCall_ii(10,a1|0)|0;
}



function jsCall_ii_11(a1) {
  a1=a1|0;
  return jsCall_ii(11,a1|0)|0;
}



function jsCall_ii_12(a1) {
  a1=a1|0;
  return jsCall_ii(12,a1|0)|0;
}



function jsCall_ii_13(a1) {
  a1=a1|0;
  return jsCall_ii(13,a1|0)|0;
}



function jsCall_ii_14(a1) {
  a1=a1|0;
  return jsCall_ii(14,a1|0)|0;
}



function jsCall_ii_15(a1) {
  a1=a1|0;
  return jsCall_ii(15,a1|0)|0;
}



function jsCall_ii_16(a1) {
  a1=a1|0;
  return jsCall_ii(16,a1|0)|0;
}



function jsCall_ii_17(a1) {
  a1=a1|0;
  return jsCall_ii(17,a1|0)|0;
}



function jsCall_ii_18(a1) {
  a1=a1|0;
  return jsCall_ii(18,a1|0)|0;
}



function jsCall_ii_19(a1) {
  a1=a1|0;
  return jsCall_ii(19,a1|0)|0;
}



function jsCall_ii_20(a1) {
  a1=a1|0;
  return jsCall_ii(20,a1|0)|0;
}



function jsCall_ii_21(a1) {
  a1=a1|0;
  return jsCall_ii(21,a1|0)|0;
}



function jsCall_ii_22(a1) {
  a1=a1|0;
  return jsCall_ii(22,a1|0)|0;
}



function jsCall_ii_23(a1) {
  a1=a1|0;
  return jsCall_ii(23,a1|0)|0;
}



function jsCall_ii_24(a1) {
  a1=a1|0;
  return jsCall_ii(24,a1|0)|0;
}



function jsCall_ii_25(a1) {
  a1=a1|0;
  return jsCall_ii(25,a1|0)|0;
}



function jsCall_ii_26(a1) {
  a1=a1|0;
  return jsCall_ii(26,a1|0)|0;
}



function jsCall_ii_27(a1) {
  a1=a1|0;
  return jsCall_ii(27,a1|0)|0;
}



function jsCall_ii_28(a1) {
  a1=a1|0;
  return jsCall_ii(28,a1|0)|0;
}



function jsCall_ii_29(a1) {
  a1=a1|0;
  return jsCall_ii(29,a1|0)|0;
}



function jsCall_ii_30(a1) {
  a1=a1|0;
  return jsCall_ii(30,a1|0)|0;
}



function jsCall_ii_31(a1) {
  a1=a1|0;
  return jsCall_ii(31,a1|0)|0;
}



function jsCall_ii_32(a1) {
  a1=a1|0;
  return jsCall_ii(32,a1|0)|0;
}



function jsCall_ii_33(a1) {
  a1=a1|0;
  return jsCall_ii(33,a1|0)|0;
}



function jsCall_ii_34(a1) {
  a1=a1|0;
  return jsCall_ii(34,a1|0)|0;
}



function jsCall_ii_35(a1) {
  a1=a1|0;
  return jsCall_ii(35,a1|0)|0;
}



function jsCall_ii_36(a1) {
  a1=a1|0;
  return jsCall_ii(36,a1|0)|0;
}



function jsCall_ii_37(a1) {
  a1=a1|0;
  return jsCall_ii(37,a1|0)|0;
}



function jsCall_ii_38(a1) {
  a1=a1|0;
  return jsCall_ii(38,a1|0)|0;
}



function jsCall_ii_39(a1) {
  a1=a1|0;
  return jsCall_ii(39,a1|0)|0;
}



function jsCall_ii_40(a1) {
  a1=a1|0;
  return jsCall_ii(40,a1|0)|0;
}



function jsCall_ii_41(a1) {
  a1=a1|0;
  return jsCall_ii(41,a1|0)|0;
}



function jsCall_ii_42(a1) {
  a1=a1|0;
  return jsCall_ii(42,a1|0)|0;
}



function jsCall_ii_43(a1) {
  a1=a1|0;
  return jsCall_ii(43,a1|0)|0;
}



function jsCall_ii_44(a1) {
  a1=a1|0;
  return jsCall_ii(44,a1|0)|0;
}



function jsCall_ii_45(a1) {
  a1=a1|0;
  return jsCall_ii(45,a1|0)|0;
}



function jsCall_ii_46(a1) {
  a1=a1|0;
  return jsCall_ii(46,a1|0)|0;
}



function jsCall_ii_47(a1) {
  a1=a1|0;
  return jsCall_ii(47,a1|0)|0;
}



function jsCall_ii_48(a1) {
  a1=a1|0;
  return jsCall_ii(48,a1|0)|0;
}



function jsCall_ii_49(a1) {
  a1=a1|0;
  return jsCall_ii(49,a1|0)|0;
}



function jsCall_ii_50(a1) {
  a1=a1|0;
  return jsCall_ii(50,a1|0)|0;
}



function jsCall_ii_51(a1) {
  a1=a1|0;
  return jsCall_ii(51,a1|0)|0;
}



function jsCall_ii_52(a1) {
  a1=a1|0;
  return jsCall_ii(52,a1|0)|0;
}



function jsCall_ii_53(a1) {
  a1=a1|0;
  return jsCall_ii(53,a1|0)|0;
}



function jsCall_ii_54(a1) {
  a1=a1|0;
  return jsCall_ii(54,a1|0)|0;
}



function jsCall_ii_55(a1) {
  a1=a1|0;
  return jsCall_ii(55,a1|0)|0;
}



function jsCall_ii_56(a1) {
  a1=a1|0;
  return jsCall_ii(56,a1|0)|0;
}



function jsCall_ii_57(a1) {
  a1=a1|0;
  return jsCall_ii(57,a1|0)|0;
}



function jsCall_ii_58(a1) {
  a1=a1|0;
  return jsCall_ii(58,a1|0)|0;
}



function jsCall_ii_59(a1) {
  a1=a1|0;
  return jsCall_ii(59,a1|0)|0;
}



function jsCall_ii_60(a1) {
  a1=a1|0;
  return jsCall_ii(60,a1|0)|0;
}



function jsCall_ii_61(a1) {
  a1=a1|0;
  return jsCall_ii(61,a1|0)|0;
}



function jsCall_ii_62(a1) {
  a1=a1|0;
  return jsCall_ii(62,a1|0)|0;
}



function jsCall_ii_63(a1) {
  a1=a1|0;
  return jsCall_ii(63,a1|0)|0;
}



function dynCall_viii(index,a1,a2,a3) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0;
  FUNCTION_TABLE_viii[index&255](a1|0,a2|0,a3|0);
}


function jsCall_viii_0(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(0,a1|0,a2|0,a3|0);
}



function jsCall_viii_1(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(1,a1|0,a2|0,a3|0);
}



function jsCall_viii_2(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(2,a1|0,a2|0,a3|0);
}



function jsCall_viii_3(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(3,a1|0,a2|0,a3|0);
}



function jsCall_viii_4(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(4,a1|0,a2|0,a3|0);
}



function jsCall_viii_5(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(5,a1|0,a2|0,a3|0);
}



function jsCall_viii_6(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(6,a1|0,a2|0,a3|0);
}



function jsCall_viii_7(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(7,a1|0,a2|0,a3|0);
}



function jsCall_viii_8(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(8,a1|0,a2|0,a3|0);
}



function jsCall_viii_9(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(9,a1|0,a2|0,a3|0);
}



function jsCall_viii_10(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(10,a1|0,a2|0,a3|0);
}



function jsCall_viii_11(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(11,a1|0,a2|0,a3|0);
}



function jsCall_viii_12(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(12,a1|0,a2|0,a3|0);
}



function jsCall_viii_13(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(13,a1|0,a2|0,a3|0);
}



function jsCall_viii_14(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(14,a1|0,a2|0,a3|0);
}



function jsCall_viii_15(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(15,a1|0,a2|0,a3|0);
}



function jsCall_viii_16(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(16,a1|0,a2|0,a3|0);
}



function jsCall_viii_17(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(17,a1|0,a2|0,a3|0);
}



function jsCall_viii_18(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(18,a1|0,a2|0,a3|0);
}



function jsCall_viii_19(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(19,a1|0,a2|0,a3|0);
}



function jsCall_viii_20(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(20,a1|0,a2|0,a3|0);
}



function jsCall_viii_21(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(21,a1|0,a2|0,a3|0);
}



function jsCall_viii_22(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(22,a1|0,a2|0,a3|0);
}



function jsCall_viii_23(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(23,a1|0,a2|0,a3|0);
}



function jsCall_viii_24(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(24,a1|0,a2|0,a3|0);
}



function jsCall_viii_25(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(25,a1|0,a2|0,a3|0);
}



function jsCall_viii_26(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(26,a1|0,a2|0,a3|0);
}



function jsCall_viii_27(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(27,a1|0,a2|0,a3|0);
}



function jsCall_viii_28(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(28,a1|0,a2|0,a3|0);
}



function jsCall_viii_29(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(29,a1|0,a2|0,a3|0);
}



function jsCall_viii_30(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(30,a1|0,a2|0,a3|0);
}



function jsCall_viii_31(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(31,a1|0,a2|0,a3|0);
}



function jsCall_viii_32(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(32,a1|0,a2|0,a3|0);
}



function jsCall_viii_33(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(33,a1|0,a2|0,a3|0);
}



function jsCall_viii_34(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(34,a1|0,a2|0,a3|0);
}



function jsCall_viii_35(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(35,a1|0,a2|0,a3|0);
}



function jsCall_viii_36(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(36,a1|0,a2|0,a3|0);
}



function jsCall_viii_37(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(37,a1|0,a2|0,a3|0);
}



function jsCall_viii_38(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(38,a1|0,a2|0,a3|0);
}



function jsCall_viii_39(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(39,a1|0,a2|0,a3|0);
}



function jsCall_viii_40(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(40,a1|0,a2|0,a3|0);
}



function jsCall_viii_41(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(41,a1|0,a2|0,a3|0);
}



function jsCall_viii_42(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(42,a1|0,a2|0,a3|0);
}



function jsCall_viii_43(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(43,a1|0,a2|0,a3|0);
}



function jsCall_viii_44(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(44,a1|0,a2|0,a3|0);
}



function jsCall_viii_45(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(45,a1|0,a2|0,a3|0);
}



function jsCall_viii_46(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(46,a1|0,a2|0,a3|0);
}



function jsCall_viii_47(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(47,a1|0,a2|0,a3|0);
}



function jsCall_viii_48(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(48,a1|0,a2|0,a3|0);
}



function jsCall_viii_49(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(49,a1|0,a2|0,a3|0);
}



function jsCall_viii_50(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(50,a1|0,a2|0,a3|0);
}



function jsCall_viii_51(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(51,a1|0,a2|0,a3|0);
}



function jsCall_viii_52(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(52,a1|0,a2|0,a3|0);
}



function jsCall_viii_53(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(53,a1|0,a2|0,a3|0);
}



function jsCall_viii_54(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(54,a1|0,a2|0,a3|0);
}



function jsCall_viii_55(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(55,a1|0,a2|0,a3|0);
}



function jsCall_viii_56(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(56,a1|0,a2|0,a3|0);
}



function jsCall_viii_57(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(57,a1|0,a2|0,a3|0);
}



function jsCall_viii_58(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(58,a1|0,a2|0,a3|0);
}



function jsCall_viii_59(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(59,a1|0,a2|0,a3|0);
}



function jsCall_viii_60(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(60,a1|0,a2|0,a3|0);
}



function jsCall_viii_61(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(61,a1|0,a2|0,a3|0);
}



function jsCall_viii_62(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(62,a1|0,a2|0,a3|0);
}



function jsCall_viii_63(a1,a2,a3) {
  a1=a1|0; a2=a2|0; a3=a3|0;
  jsCall_viii(63,a1|0,a2|0,a3|0);
}



function dynCall_v(index) {
  index = index|0;
  
  FUNCTION_TABLE_v[index&255]();
}


function jsCall_v_0() {
  
  jsCall_v(0);
}



function jsCall_v_1() {
  
  jsCall_v(1);
}



function jsCall_v_2() {
  
  jsCall_v(2);
}



function jsCall_v_3() {
  
  jsCall_v(3);
}



function jsCall_v_4() {
  
  jsCall_v(4);
}



function jsCall_v_5() {
  
  jsCall_v(5);
}



function jsCall_v_6() {
  
  jsCall_v(6);
}



function jsCall_v_7() {
  
  jsCall_v(7);
}



function jsCall_v_8() {
  
  jsCall_v(8);
}



function jsCall_v_9() {
  
  jsCall_v(9);
}



function jsCall_v_10() {
  
  jsCall_v(10);
}



function jsCall_v_11() {
  
  jsCall_v(11);
}



function jsCall_v_12() {
  
  jsCall_v(12);
}



function jsCall_v_13() {
  
  jsCall_v(13);
}



function jsCall_v_14() {
  
  jsCall_v(14);
}



function jsCall_v_15() {
  
  jsCall_v(15);
}



function jsCall_v_16() {
  
  jsCall_v(16);
}



function jsCall_v_17() {
  
  jsCall_v(17);
}



function jsCall_v_18() {
  
  jsCall_v(18);
}



function jsCall_v_19() {
  
  jsCall_v(19);
}



function jsCall_v_20() {
  
  jsCall_v(20);
}



function jsCall_v_21() {
  
  jsCall_v(21);
}



function jsCall_v_22() {
  
  jsCall_v(22);
}



function jsCall_v_23() {
  
  jsCall_v(23);
}



function jsCall_v_24() {
  
  jsCall_v(24);
}



function jsCall_v_25() {
  
  jsCall_v(25);
}



function jsCall_v_26() {
  
  jsCall_v(26);
}



function jsCall_v_27() {
  
  jsCall_v(27);
}



function jsCall_v_28() {
  
  jsCall_v(28);
}



function jsCall_v_29() {
  
  jsCall_v(29);
}



function jsCall_v_30() {
  
  jsCall_v(30);
}



function jsCall_v_31() {
  
  jsCall_v(31);
}



function jsCall_v_32() {
  
  jsCall_v(32);
}



function jsCall_v_33() {
  
  jsCall_v(33);
}



function jsCall_v_34() {
  
  jsCall_v(34);
}



function jsCall_v_35() {
  
  jsCall_v(35);
}



function jsCall_v_36() {
  
  jsCall_v(36);
}



function jsCall_v_37() {
  
  jsCall_v(37);
}



function jsCall_v_38() {
  
  jsCall_v(38);
}



function jsCall_v_39() {
  
  jsCall_v(39);
}



function jsCall_v_40() {
  
  jsCall_v(40);
}



function jsCall_v_41() {
  
  jsCall_v(41);
}



function jsCall_v_42() {
  
  jsCall_v(42);
}



function jsCall_v_43() {
  
  jsCall_v(43);
}



function jsCall_v_44() {
  
  jsCall_v(44);
}



function jsCall_v_45() {
  
  jsCall_v(45);
}



function jsCall_v_46() {
  
  jsCall_v(46);
}



function jsCall_v_47() {
  
  jsCall_v(47);
}



function jsCall_v_48() {
  
  jsCall_v(48);
}



function jsCall_v_49() {
  
  jsCall_v(49);
}



function jsCall_v_50() {
  
  jsCall_v(50);
}



function jsCall_v_51() {
  
  jsCall_v(51);
}



function jsCall_v_52() {
  
  jsCall_v(52);
}



function jsCall_v_53() {
  
  jsCall_v(53);
}



function jsCall_v_54() {
  
  jsCall_v(54);
}



function jsCall_v_55() {
  
  jsCall_v(55);
}



function jsCall_v_56() {
  
  jsCall_v(56);
}



function jsCall_v_57() {
  
  jsCall_v(57);
}



function jsCall_v_58() {
  
  jsCall_v(58);
}



function jsCall_v_59() {
  
  jsCall_v(59);
}



function jsCall_v_60() {
  
  jsCall_v(60);
}



function jsCall_v_61() {
  
  jsCall_v(61);
}



function jsCall_v_62() {
  
  jsCall_v(62);
}



function jsCall_v_63() {
  
  jsCall_v(63);
}



function dynCall_iiiii(index,a1,a2,a3,a4) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return FUNCTION_TABLE_iiiii[index&255](a1|0,a2|0,a3|0,a4|0)|0;
}


function jsCall_iiiii_0(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(0,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_1(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(1,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_2(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(2,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_3(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(3,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_4(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(4,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_5(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(5,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_6(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(6,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_7(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(7,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_8(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(8,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_9(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(9,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_10(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(10,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_11(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(11,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_12(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(12,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_13(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(13,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_14(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(14,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_15(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(15,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_16(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(16,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_17(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(17,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_18(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(18,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_19(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(19,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_20(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(20,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_21(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(21,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_22(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(22,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_23(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(23,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_24(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(24,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_25(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(25,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_26(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(26,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_27(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(27,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_28(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(28,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_29(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(29,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_30(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(30,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_31(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(31,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_32(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(32,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_33(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(33,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_34(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(34,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_35(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(35,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_36(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(36,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_37(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(37,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_38(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(38,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_39(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(39,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_40(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(40,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_41(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(41,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_42(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(42,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_43(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(43,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_44(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(44,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_45(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(45,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_46(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(46,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_47(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(47,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_48(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(48,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_49(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(49,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_50(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(50,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_51(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(51,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_52(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(52,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_53(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(53,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_54(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(54,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_55(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(55,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_56(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(56,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_57(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(57,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_58(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(58,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_59(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(59,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_60(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(60,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_61(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(61,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_62(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(62,a1|0,a2|0,a3|0,a4|0)|0;
}



function jsCall_iiiii_63(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  return jsCall_iiiii(63,a1|0,a2|0,a3|0,a4|0)|0;
}



function dynCall_viiiiii(index,a1,a2,a3,a4,a5,a6) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  FUNCTION_TABLE_viiiiii[index&255](a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}


function jsCall_viiiiii_0(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(0,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_1(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(1,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_2(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(2,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_3(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(3,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_4(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(4,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_5(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(5,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_6(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(6,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_7(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(7,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_8(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(8,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_9(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(9,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_10(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(10,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_11(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(11,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_12(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(12,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_13(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(13,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_14(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(14,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_15(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(15,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_16(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(16,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_17(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(17,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_18(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(18,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_19(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(19,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_20(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(20,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_21(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(21,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_22(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(22,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_23(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(23,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_24(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(24,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_25(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(25,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_26(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(26,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_27(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(27,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_28(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(28,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_29(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(29,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_30(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(30,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_31(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(31,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_32(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(32,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_33(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(33,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_34(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(34,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_35(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(35,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_36(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(36,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_37(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(37,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_38(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(38,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_39(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(39,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_40(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(40,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_41(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(41,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_42(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(42,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_43(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(43,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_44(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(44,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_45(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(45,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_46(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(46,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_47(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(47,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_48(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(48,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_49(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(49,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_50(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(50,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_51(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(51,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_52(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(52,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_53(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(53,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_54(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(54,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_55(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(55,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_56(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(56,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_57(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(57,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_58(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(58,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_59(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(59,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_60(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(60,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_61(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(61,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_62(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(62,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function jsCall_viiiiii_63(a1,a2,a3,a4,a5,a6) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0; a6=a6|0;
  jsCall_viiiiii(63,a1|0,a2|0,a3|0,a4|0,a5|0,a6|0);
}



function dynCall_iii(index,a1,a2) {
  index = index|0;
  a1=a1|0; a2=a2|0;
  return FUNCTION_TABLE_iii[index&255](a1|0,a2|0)|0;
}


function jsCall_iii_0(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(0,a1|0,a2|0)|0;
}



function jsCall_iii_1(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(1,a1|0,a2|0)|0;
}



function jsCall_iii_2(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(2,a1|0,a2|0)|0;
}



function jsCall_iii_3(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(3,a1|0,a2|0)|0;
}



function jsCall_iii_4(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(4,a1|0,a2|0)|0;
}



function jsCall_iii_5(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(5,a1|0,a2|0)|0;
}



function jsCall_iii_6(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(6,a1|0,a2|0)|0;
}



function jsCall_iii_7(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(7,a1|0,a2|0)|0;
}



function jsCall_iii_8(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(8,a1|0,a2|0)|0;
}



function jsCall_iii_9(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(9,a1|0,a2|0)|0;
}



function jsCall_iii_10(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(10,a1|0,a2|0)|0;
}



function jsCall_iii_11(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(11,a1|0,a2|0)|0;
}



function jsCall_iii_12(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(12,a1|0,a2|0)|0;
}



function jsCall_iii_13(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(13,a1|0,a2|0)|0;
}



function jsCall_iii_14(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(14,a1|0,a2|0)|0;
}



function jsCall_iii_15(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(15,a1|0,a2|0)|0;
}



function jsCall_iii_16(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(16,a1|0,a2|0)|0;
}



function jsCall_iii_17(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(17,a1|0,a2|0)|0;
}



function jsCall_iii_18(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(18,a1|0,a2|0)|0;
}



function jsCall_iii_19(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(19,a1|0,a2|0)|0;
}



function jsCall_iii_20(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(20,a1|0,a2|0)|0;
}



function jsCall_iii_21(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(21,a1|0,a2|0)|0;
}



function jsCall_iii_22(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(22,a1|0,a2|0)|0;
}



function jsCall_iii_23(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(23,a1|0,a2|0)|0;
}



function jsCall_iii_24(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(24,a1|0,a2|0)|0;
}



function jsCall_iii_25(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(25,a1|0,a2|0)|0;
}



function jsCall_iii_26(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(26,a1|0,a2|0)|0;
}



function jsCall_iii_27(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(27,a1|0,a2|0)|0;
}



function jsCall_iii_28(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(28,a1|0,a2|0)|0;
}



function jsCall_iii_29(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(29,a1|0,a2|0)|0;
}



function jsCall_iii_30(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(30,a1|0,a2|0)|0;
}



function jsCall_iii_31(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(31,a1|0,a2|0)|0;
}



function jsCall_iii_32(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(32,a1|0,a2|0)|0;
}



function jsCall_iii_33(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(33,a1|0,a2|0)|0;
}



function jsCall_iii_34(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(34,a1|0,a2|0)|0;
}



function jsCall_iii_35(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(35,a1|0,a2|0)|0;
}



function jsCall_iii_36(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(36,a1|0,a2|0)|0;
}



function jsCall_iii_37(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(37,a1|0,a2|0)|0;
}



function jsCall_iii_38(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(38,a1|0,a2|0)|0;
}



function jsCall_iii_39(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(39,a1|0,a2|0)|0;
}



function jsCall_iii_40(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(40,a1|0,a2|0)|0;
}



function jsCall_iii_41(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(41,a1|0,a2|0)|0;
}



function jsCall_iii_42(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(42,a1|0,a2|0)|0;
}



function jsCall_iii_43(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(43,a1|0,a2|0)|0;
}



function jsCall_iii_44(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(44,a1|0,a2|0)|0;
}



function jsCall_iii_45(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(45,a1|0,a2|0)|0;
}



function jsCall_iii_46(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(46,a1|0,a2|0)|0;
}



function jsCall_iii_47(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(47,a1|0,a2|0)|0;
}



function jsCall_iii_48(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(48,a1|0,a2|0)|0;
}



function jsCall_iii_49(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(49,a1|0,a2|0)|0;
}



function jsCall_iii_50(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(50,a1|0,a2|0)|0;
}



function jsCall_iii_51(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(51,a1|0,a2|0)|0;
}



function jsCall_iii_52(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(52,a1|0,a2|0)|0;
}



function jsCall_iii_53(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(53,a1|0,a2|0)|0;
}



function jsCall_iii_54(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(54,a1|0,a2|0)|0;
}



function jsCall_iii_55(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(55,a1|0,a2|0)|0;
}



function jsCall_iii_56(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(56,a1|0,a2|0)|0;
}



function jsCall_iii_57(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(57,a1|0,a2|0)|0;
}



function jsCall_iii_58(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(58,a1|0,a2|0)|0;
}



function jsCall_iii_59(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(59,a1|0,a2|0)|0;
}



function jsCall_iii_60(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(60,a1|0,a2|0)|0;
}



function jsCall_iii_61(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(61,a1|0,a2|0)|0;
}



function jsCall_iii_62(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(62,a1|0,a2|0)|0;
}



function jsCall_iii_63(a1,a2) {
  a1=a1|0; a2=a2|0;
  return jsCall_iii(63,a1|0,a2|0)|0;
}



function dynCall_iiiiii(index,a1,a2,a3,a4,a5) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return FUNCTION_TABLE_iiiiii[index&255](a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}


function jsCall_iiiiii_0(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(0,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_1(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(1,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_2(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(2,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_3(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(3,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_4(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(4,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_5(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(5,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_6(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(6,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_7(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(7,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_8(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(8,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_9(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(9,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_10(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(10,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_11(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(11,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_12(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(12,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_13(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(13,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_14(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(14,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_15(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(15,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_16(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(16,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_17(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(17,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_18(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(18,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_19(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(19,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_20(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(20,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_21(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(21,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_22(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(22,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_23(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(23,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_24(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(24,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_25(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(25,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_26(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(26,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_27(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(27,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_28(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(28,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_29(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(29,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_30(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(30,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_31(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(31,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_32(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(32,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_33(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(33,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_34(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(34,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_35(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(35,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_36(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(36,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_37(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(37,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_38(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(38,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_39(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(39,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_40(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(40,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_41(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(41,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_42(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(42,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_43(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(43,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_44(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(44,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_45(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(45,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_46(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(46,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_47(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(47,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_48(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(48,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_49(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(49,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_50(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(50,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_51(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(51,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_52(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(52,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_53(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(53,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_54(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(54,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_55(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(55,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_56(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(56,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_57(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(57,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_58(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(58,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_59(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(59,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_60(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(60,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_61(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(61,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_62(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(62,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function jsCall_iiiiii_63(a1,a2,a3,a4,a5) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0; a5=a5|0;
  return jsCall_iiiiii(63,a1|0,a2|0,a3|0,a4|0,a5|0)|0;
}



function dynCall_viiii(index,a1,a2,a3,a4) {
  index = index|0;
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  FUNCTION_TABLE_viiii[index&255](a1|0,a2|0,a3|0,a4|0);
}


function jsCall_viiii_0(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(0,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_1(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(1,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_2(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(2,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_3(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(3,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_4(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(4,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_5(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(5,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_6(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(6,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_7(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(7,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_8(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(8,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_9(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(9,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_10(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(10,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_11(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(11,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_12(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(12,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_13(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(13,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_14(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(14,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_15(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(15,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_16(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(16,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_17(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(17,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_18(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(18,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_19(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(19,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_20(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(20,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_21(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(21,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_22(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(22,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_23(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(23,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_24(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(24,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_25(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(25,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_26(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(26,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_27(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(27,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_28(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(28,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_29(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(29,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_30(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(30,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_31(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(31,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_32(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(32,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_33(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(33,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_34(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(34,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_35(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(35,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_36(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(36,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_37(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(37,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_38(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(38,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_39(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(39,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_40(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(40,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_41(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(41,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_42(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(42,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_43(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(43,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_44(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(44,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_45(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(45,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_46(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(46,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_47(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(47,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_48(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(48,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_49(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(49,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_50(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(50,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_51(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(51,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_52(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(52,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_53(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(53,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_54(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(54,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_55(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(55,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_56(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(56,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_57(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(57,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_58(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(58,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_59(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(59,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_60(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(60,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_61(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(61,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_62(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(62,a1|0,a2|0,a3|0,a4|0);
}



function jsCall_viiii_63(a1,a2,a3,a4) {
  a1=a1|0; a2=a2|0; a3=a3|0; a4=a4|0;
  jsCall_viiii(63,a1|0,a2|0,a3|0,a4|0);
}


function b0(p0,p1,p2) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0; abort(0);return 0;
}
function b1() {
 ; abort(1);return 0;
}
function b2(p0) {
 p0 = p0|0; abort(2);
}
function b3(p0,p1) {
 p0 = p0|0;p1 = p1|0; abort(3);
}
function b4(p0,p1,p2,p3,p4,p5) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; abort(4);return 0;
}
function b5(p0) {
 p0 = p0|0; abort(5);return 0;
}
function b6(p0,p1,p2) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0; abort(6);
}
function b7() {
 ; abort(7);
}
function b8(p0,p1,p2,p3) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; abort(8);return 0;
}
function b9(p0,p1,p2,p3,p4,p5) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0;p5 = p5|0; abort(9);
}
function b10(p0,p1) {
 p0 = p0|0;p1 = p1|0; abort(10);return 0;
}
function b11(p0,p1,p2,p3,p4) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0;p4 = p4|0; abort(11);return 0;
}
function b12(p0,p1,p2,p3) {
 p0 = p0|0;p1 = p1|0;p2 = p2|0;p3 = p3|0; abort(12);
}

// EMSCRIPTEN_END_FUNCS
var FUNCTION_TABLE_iiii = [b0,b0,jsCall_iiii_0,b0,jsCall_iiii_1,b0,jsCall_iiii_2,b0,jsCall_iiii_3,b0,jsCall_iiii_4,b0,jsCall_iiii_5,b0,jsCall_iiii_6,b0,jsCall_iiii_7,b0,jsCall_iiii_8,b0,jsCall_iiii_9,b0,jsCall_iiii_10,b0,jsCall_iiii_11,b0,jsCall_iiii_12,b0,jsCall_iiii_13
,b0,jsCall_iiii_14,b0,jsCall_iiii_15,b0,jsCall_iiii_16,b0,jsCall_iiii_17,b0,jsCall_iiii_18,b0,jsCall_iiii_19,b0,jsCall_iiii_20,b0,jsCall_iiii_21,b0,jsCall_iiii_22,b0,jsCall_iiii_23,b0,jsCall_iiii_24,b0,jsCall_iiii_25,b0,jsCall_iiii_26,b0,jsCall_iiii_27,b0,jsCall_iiii_28
,b0,jsCall_iiii_29,b0,jsCall_iiii_30,b0,jsCall_iiii_31,b0,jsCall_iiii_32,b0,jsCall_iiii_33,b0,jsCall_iiii_34,b0,jsCall_iiii_35,b0,jsCall_iiii_36,b0,jsCall_iiii_37,b0,jsCall_iiii_38,b0,jsCall_iiii_39,b0,jsCall_iiii_40,b0,jsCall_iiii_41,b0,jsCall_iiii_42,b0,jsCall_iiii_43
,b0,jsCall_iiii_44,b0,jsCall_iiii_45,b0,jsCall_iiii_46,b0,jsCall_iiii_47,b0,jsCall_iiii_48,b0,jsCall_iiii_49,b0,jsCall_iiii_50,b0,jsCall_iiii_51,b0,jsCall_iiii_52,b0,jsCall_iiii_53,b0,jsCall_iiii_54,b0,jsCall_iiii_55,b0,jsCall_iiii_56,b0,jsCall_iiii_57,b0,jsCall_iiii_58
,b0,jsCall_iiii_59,b0,jsCall_iiii_60,b0,jsCall_iiii_61,b0,jsCall_iiii_62,b0,jsCall_iiii_63,b0,_unixDelete,_unixRandomness,_unixGetLastError,_unixSetSystemCall,_fts3ColumnMethod,_fts3tokColumnMethod,_porterCreate,_simpleCreate,_fts3auxColumnMethod,_unicodeCreate,_memjrnlTruncate,_pcache1Create,_pcache1Fetch,_unixTruncate,_unixFileControl,_posixOpen,_fcntl,_read,_write
,_fchown,_readlink,___stdout_write,___stdio_seek,_fts3ExprPhraseCountCb,_fts3ExprGlobalHitsCb,_fts3ExprLocalHitsCb,_fts3ExprLoadDoclistsCb,_fts3MatchinfoLcsCb,_fts3ExprTermOffsetInit,_fts3SnippetFindPositions,_vdbeRecordCompareInt,_vdbeRecordCompareString,_sqlite3VdbeRecordCompare,___stdio_write,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0
,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0,b0];
var FUNCTION_TABLE_i = [b1,b1,jsCall_i_0,b1,jsCall_i_1,b1,jsCall_i_2,b1,jsCall_i_3,b1,jsCall_i_4,b1,jsCall_i_5,b1,jsCall_i_6,b1,jsCall_i_7,b1,jsCall_i_8,b1,jsCall_i_9,b1,jsCall_i_10,b1,jsCall_i_11,b1,jsCall_i_12,b1,jsCall_i_13
,b1,jsCall_i_14,b1,jsCall_i_15,b1,jsCall_i_16,b1,jsCall_i_17,b1,jsCall_i_18,b1,jsCall_i_19,b1,jsCall_i_20,b1,jsCall_i_21,b1,jsCall_i_22,b1,jsCall_i_23,b1,jsCall_i_24,b1,jsCall_i_25,b1,jsCall_i_26,b1,jsCall_i_27,b1,jsCall_i_28
,b1,jsCall_i_29,b1,jsCall_i_30,b1,jsCall_i_31,b1,jsCall_i_32,b1,jsCall_i_33,b1,jsCall_i_34,b1,jsCall_i_35,b1,jsCall_i_36,b1,jsCall_i_37,b1,jsCall_i_38,b1,jsCall_i_39,b1,jsCall_i_40,b1,jsCall_i_41,b1,jsCall_i_42,b1,jsCall_i_43
,b1,jsCall_i_44,b1,jsCall_i_45,b1,jsCall_i_46,b1,jsCall_i_47,b1,jsCall_i_48,b1,jsCall_i_49,b1,jsCall_i_50,b1,jsCall_i_51,b1,jsCall_i_52,b1,jsCall_i_53,b1,jsCall_i_54,b1,jsCall_i_55,b1,jsCall_i_56,b1,jsCall_i_57,b1,jsCall_i_58
,b1,jsCall_i_59,b1,jsCall_i_60,b1,jsCall_i_61,b1,jsCall_i_62,b1,jsCall_i_63,b1,_geteuid,_unixGetpagesize,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1
,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1,b1];
var FUNCTION_TABLE_vi = [b2,b2,jsCall_vi_0,b2,jsCall_vi_1,b2,jsCall_vi_2,b2,jsCall_vi_3,b2,jsCall_vi_4,b2,jsCall_vi_5,b2,jsCall_vi_6,b2,jsCall_vi_7,b2,jsCall_vi_8,b2,jsCall_vi_9,b2,jsCall_vi_10,b2,jsCall_vi_11,b2,jsCall_vi_12,b2,jsCall_vi_13
,b2,jsCall_vi_14,b2,jsCall_vi_15,b2,jsCall_vi_16,b2,jsCall_vi_17,b2,jsCall_vi_18,b2,jsCall_vi_19,b2,jsCall_vi_20,b2,jsCall_vi_21,b2,jsCall_vi_22,b2,jsCall_vi_23,b2,jsCall_vi_24,b2,jsCall_vi_25,b2,jsCall_vi_26,b2,jsCall_vi_27,b2,jsCall_vi_28
,b2,jsCall_vi_29,b2,jsCall_vi_30,b2,jsCall_vi_31,b2,jsCall_vi_32,b2,jsCall_vi_33,b2,jsCall_vi_34,b2,jsCall_vi_35,b2,jsCall_vi_36,b2,jsCall_vi_37,b2,jsCall_vi_38,b2,jsCall_vi_39,b2,jsCall_vi_40,b2,jsCall_vi_41,b2,jsCall_vi_42,b2,jsCall_vi_43
,b2,jsCall_vi_44,b2,jsCall_vi_45,b2,jsCall_vi_46,b2,jsCall_vi_47,b2,jsCall_vi_48,b2,jsCall_vi_49,b2,jsCall_vi_50,b2,jsCall_vi_51,b2,jsCall_vi_52,b2,jsCall_vi_53,b2,jsCall_vi_54,b2,jsCall_vi_55,b2,jsCall_vi_56,b2,jsCall_vi_57,b2,jsCall_vi_58
,b2,jsCall_vi_59,b2,jsCall_vi_60,b2,jsCall_vi_61,b2,jsCall_vi_62,b2,jsCall_vi_63,b2,_stdevFinalize,_varianceFinalize,_modeFinalize,_medianFinalize,_lower_quartileFinalize,_upper_quartileFinalize,_pcache1Shutdown,_pcache1Destroy,_pcache1Shrink,_sqlite3MemFree,_sqlite3MemShutdown,_unixShmBarrier,_minMaxFinalize,_sumFinalize,_totalFinalize,_avgFinalize,_countFinalize,_groupConcatFinalize,_sqlite3_free
,_pageReinit,_sqlite3SchemaClear,_hashDestroy,_fts3MIBufferFree,_stat4Destructor,_cleanup392,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2
,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2,b2];
var FUNCTION_TABLE_vii = [b3,b3,jsCall_vii_0,b3,jsCall_vii_1,b3,jsCall_vii_2,b3,jsCall_vii_3,b3,jsCall_vii_4,b3,jsCall_vii_5,b3,jsCall_vii_6,b3,jsCall_vii_7,b3,jsCall_vii_8,b3,jsCall_vii_9,b3,jsCall_vii_10,b3,jsCall_vii_11,b3,jsCall_vii_12,b3,jsCall_vii_13
,b3,jsCall_vii_14,b3,jsCall_vii_15,b3,jsCall_vii_16,b3,jsCall_vii_17,b3,jsCall_vii_18,b3,jsCall_vii_19,b3,jsCall_vii_20,b3,jsCall_vii_21,b3,jsCall_vii_22,b3,jsCall_vii_23,b3,jsCall_vii_24,b3,jsCall_vii_25,b3,jsCall_vii_26,b3,jsCall_vii_27,b3,jsCall_vii_28
,b3,jsCall_vii_29,b3,jsCall_vii_30,b3,jsCall_vii_31,b3,jsCall_vii_32,b3,jsCall_vii_33,b3,jsCall_vii_34,b3,jsCall_vii_35,b3,jsCall_vii_36,b3,jsCall_vii_37,b3,jsCall_vii_38,b3,jsCall_vii_39,b3,jsCall_vii_40,b3,jsCall_vii_41,b3,jsCall_vii_42,b3,jsCall_vii_43
,b3,jsCall_vii_44,b3,jsCall_vii_45,b3,jsCall_vii_46,b3,jsCall_vii_47,b3,jsCall_vii_48,b3,jsCall_vii_49,b3,jsCall_vii_50,b3,jsCall_vii_51,b3,jsCall_vii_52,b3,jsCall_vii_53,b3,jsCall_vii_54,b3,jsCall_vii_55,b3,jsCall_vii_56,b3,jsCall_vii_57,b3,jsCall_vii_58
,b3,jsCall_vii_59,b3,jsCall_vii_60,b3,jsCall_vii_61,b3,jsCall_vii_62,b3,jsCall_vii_63,b3,_pcache1Cachesize,_pcache1Truncate,_selectPopWith,_selectAddSubqueryTypeInfo,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3
,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3,b3];
var FUNCTION_TABLE_iiiiiii = [b4,b4,jsCall_iiiiiii_0,b4,jsCall_iiiiiii_1,b4,jsCall_iiiiiii_2,b4,jsCall_iiiiiii_3,b4,jsCall_iiiiiii_4,b4,jsCall_iiiiiii_5,b4,jsCall_iiiiiii_6,b4,jsCall_iiiiiii_7,b4,jsCall_iiiiiii_8,b4,jsCall_iiiiiii_9,b4,jsCall_iiiiiii_10,b4,jsCall_iiiiiii_11,b4,jsCall_iiiiiii_12,b4,jsCall_iiiiiii_13
,b4,jsCall_iiiiiii_14,b4,jsCall_iiiiiii_15,b4,jsCall_iiiiiii_16,b4,jsCall_iiiiiii_17,b4,jsCall_iiiiiii_18,b4,jsCall_iiiiiii_19,b4,jsCall_iiiiiii_20,b4,jsCall_iiiiiii_21,b4,jsCall_iiiiiii_22,b4,jsCall_iiiiiii_23,b4,jsCall_iiiiiii_24,b4,jsCall_iiiiiii_25,b4,jsCall_iiiiiii_26,b4,jsCall_iiiiiii_27,b4,jsCall_iiiiiii_28
,b4,jsCall_iiiiiii_29,b4,jsCall_iiiiiii_30,b4,jsCall_iiiiiii_31,b4,jsCall_iiiiiii_32,b4,jsCall_iiiiiii_33,b4,jsCall_iiiiiii_34,b4,jsCall_iiiiiii_35,b4,jsCall_iiiiiii_36,b4,jsCall_iiiiiii_37,b4,jsCall_iiiiiii_38,b4,jsCall_iiiiiii_39,b4,jsCall_iiiiiii_40,b4,jsCall_iiiiiii_41,b4,jsCall_iiiiiii_42,b4,jsCall_iiiiiii_43
,b4,jsCall_iiiiiii_44,b4,jsCall_iiiiiii_45,b4,jsCall_iiiiiii_46,b4,jsCall_iiiiiii_47,b4,jsCall_iiiiiii_48,b4,jsCall_iiiiiii_49,b4,jsCall_iiiiiii_50,b4,jsCall_iiiiiii_51,b4,jsCall_iiiiiii_52,b4,jsCall_iiiiiii_53,b4,jsCall_iiiiiii_54,b4,jsCall_iiiiiii_55,b4,jsCall_iiiiiii_56,b4,jsCall_iiiiiii_57,b4,jsCall_iiiiiii_58
,b4,jsCall_iiiiiii_59,b4,jsCall_iiiiiii_60,b4,jsCall_iiiiiii_61,b4,jsCall_iiiiiii_62,b4,jsCall_iiiiiii_63,b4,_fts3CreateMethod,_fts3ConnectMethod,_fts3tokConnectMethod,_porterNext,_simpleNext,_fts3auxConnectMethod,_unicodeNext,___mmap,_vdbeSorterCompareInt,_vdbeSorterCompareText,_vdbeSorterCompare,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4
,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4,b4];
var FUNCTION_TABLE_ii = [b5,b5,jsCall_ii_0,b5,jsCall_ii_1,b5,jsCall_ii_2,b5,jsCall_ii_3,b5,jsCall_ii_4,b5,jsCall_ii_5,b5,jsCall_ii_6,b5,jsCall_ii_7,b5,jsCall_ii_8,b5,jsCall_ii_9,b5,jsCall_ii_10,b5,jsCall_ii_11,b5,jsCall_ii_12,b5,jsCall_ii_13
,b5,jsCall_ii_14,b5,jsCall_ii_15,b5,jsCall_ii_16,b5,jsCall_ii_17,b5,jsCall_ii_18,b5,jsCall_ii_19,b5,jsCall_ii_20,b5,jsCall_ii_21,b5,jsCall_ii_22,b5,jsCall_ii_23,b5,jsCall_ii_24,b5,jsCall_ii_25,b5,jsCall_ii_26,b5,jsCall_ii_27,b5,jsCall_ii_28
,b5,jsCall_ii_29,b5,jsCall_ii_30,b5,jsCall_ii_31,b5,jsCall_ii_32,b5,jsCall_ii_33,b5,jsCall_ii_34,b5,jsCall_ii_35,b5,jsCall_ii_36,b5,jsCall_ii_37,b5,jsCall_ii_38,b5,jsCall_ii_39,b5,jsCall_ii_40,b5,jsCall_ii_41,b5,jsCall_ii_42,b5,jsCall_ii_43
,b5,jsCall_ii_44,b5,jsCall_ii_45,b5,jsCall_ii_46,b5,jsCall_ii_47,b5,jsCall_ii_48,b5,jsCall_ii_49,b5,jsCall_ii_50,b5,jsCall_ii_51,b5,jsCall_ii_52,b5,jsCall_ii_53,b5,jsCall_ii_54,b5,jsCall_ii_55,b5,jsCall_ii_56,b5,jsCall_ii_57,b5,jsCall_ii_58
,b5,jsCall_ii_59,b5,jsCall_ii_60,b5,jsCall_ii_61,b5,jsCall_ii_62,b5,jsCall_ii_63,b5,_fts3DisconnectMethod,_fts3DestroyMethod,_fts3CloseMethod,_fts3NextMethod,_fts3EofMethod,_fts3BeginMethod,_fts3SyncMethod,_fts3CommitMethod,_fts3RollbackMethod,_fts3tokDisconnectMethod,_fts3tokCloseMethod,_fts3tokNextMethod,_fts3tokEofMethod,_porterDestroy,_porterClose,_simpleDestroy,_simpleClose,_fts3auxDisconnectMethod,_fts3auxCloseMethod
,_fts3auxNextMethod,_fts3auxEofMethod,_unicodeDestroy,_unicodeClose,_memjrnlClose,_pcache1Init,_pcache1Pagecount,_sqlite3MemMalloc,_sqlite3MemSize,_sqlite3MemRoundup,_sqlite3MemInit,_dotlockClose,_unixSectorSize,_unixDeviceCharacteristics,_close,_unlink,_rmdir,_nolockClose,_unixClose,___stdio_close,_sqlite3_value_text,_sqlite3MallocSize,_btreeInvokeBusyHandler,_m_gt_0,_hasVowel,_m_gt_1,b5,b5,b5,b5
,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5
,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5,b5];
var FUNCTION_TABLE_viii = [b6,b6,jsCall_viii_0,b6,jsCall_viii_1,b6,jsCall_viii_2,b6,jsCall_viii_3,b6,jsCall_viii_4,b6,jsCall_viii_5,b6,jsCall_viii_6,b6,jsCall_viii_7,b6,jsCall_viii_8,b6,jsCall_viii_9,b6,jsCall_viii_10,b6,jsCall_viii_11,b6,jsCall_viii_12,b6,jsCall_viii_13
,b6,jsCall_viii_14,b6,jsCall_viii_15,b6,jsCall_viii_16,b6,jsCall_viii_17,b6,jsCall_viii_18,b6,jsCall_viii_19,b6,jsCall_viii_20,b6,jsCall_viii_21,b6,jsCall_viii_22,b6,jsCall_viii_23,b6,jsCall_viii_24,b6,jsCall_viii_25,b6,jsCall_viii_26,b6,jsCall_viii_27,b6,jsCall_viii_28
,b6,jsCall_viii_29,b6,jsCall_viii_30,b6,jsCall_viii_31,b6,jsCall_viii_32,b6,jsCall_viii_33,b6,jsCall_viii_34,b6,jsCall_viii_35,b6,jsCall_viii_36,b6,jsCall_viii_37,b6,jsCall_viii_38,b6,jsCall_viii_39,b6,jsCall_viii_40,b6,jsCall_viii_41,b6,jsCall_viii_42,b6,jsCall_viii_43
,b6,jsCall_viii_44,b6,jsCall_viii_45,b6,jsCall_viii_46,b6,jsCall_viii_47,b6,jsCall_viii_48,b6,jsCall_viii_49,b6,jsCall_viii_50,b6,jsCall_viii_51,b6,jsCall_viii_52,b6,jsCall_viii_53,b6,jsCall_viii_54,b6,jsCall_viii_55,b6,jsCall_viii_56,b6,jsCall_viii_57,b6,jsCall_viii_58
,b6,jsCall_viii_59,b6,jsCall_viii_60,b6,jsCall_viii_61,b6,jsCall_viii_62,b6,jsCall_viii_63,b6,_acosFunc,_asinFunc,_atanFunc,_atn2Func,_acoshFunc,_asinhFunc,_atanhFunc,_differenceFunc,_rad2degFunc,_deg2radFunc,_cosFunc,_sinFunc,_tanFunc,_cotFunc,_coshFunc,_sinhFunc,_tanhFunc,_cothFunc,_expFunc
,_logFunc,_log10Func,_powerFunc,_signFunc,_sqrtFunc,_squareFunc,_ceilFunc,_floorFunc,_piFunc,_replicateFunc,_charindexFunc,_leftFunc,_rightFunc,_reverseFunc,_properFunc,_padlFunc,_padrFunc,_padcFunc,_strfilterFunc,_varianceStep,_modeStep,_fts3SnippetFunc,_fts3OffsetsFunc,_fts3OptimizeFunc,_fts3MatchinfoFunc,_pcache1Unpin,_statInit,_statPush,_statGet,_detachFunc
,_attachFunc,_compileoptionusedFunc,_compileoptiongetFunc,_versionFunc,_trimFunc,_minmaxFunc,_minmaxStep,_typeofFunc,_lengthFunc,_instrFunc,_printfFunc,_unicodeFunc,_charFunc,_absFunc,_roundFunc,_upperFunc,_lowerFunc,_hexFunc,_randomFunc,_randomBlob,_nullifFunc,_sourceidFunc,_errlogFunc,_quoteFunc,_last_insert_rowid,_changes,_total_changes,_replaceFunc,_zeroblobFunc,_substrFunc
,_sumStep,_countStep,_groupConcatStep,_likeFunc,_juliandayFunc,_dateFunc,_timeFunc,_datetimeFunc,_strftimeFunc,_ctimeFunc,_ctimestampFunc,_cdateFunc,_renameTableFunc,_renameTriggerFunc,_renameParentFunc,_sqlite3InvalidFunction,_fts3TokenizerFunc,_btreeParseCellPtr,_btreeParseCellPtrNoPayload,_btreeParseCellPtrIndex,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6
,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6,b6];
var FUNCTION_TABLE_v = [b7,b7,jsCall_v_0,b7,jsCall_v_1,b7,jsCall_v_2,b7,jsCall_v_3,b7,jsCall_v_4,b7,jsCall_v_5,b7,jsCall_v_6,b7,jsCall_v_7,b7,jsCall_v_8,b7,jsCall_v_9,b7,jsCall_v_10,b7,jsCall_v_11,b7,jsCall_v_12,b7,jsCall_v_13
,b7,jsCall_v_14,b7,jsCall_v_15,b7,jsCall_v_16,b7,jsCall_v_17,b7,jsCall_v_18,b7,jsCall_v_19,b7,jsCall_v_20,b7,jsCall_v_21,b7,jsCall_v_22,b7,jsCall_v_23,b7,jsCall_v_24,b7,jsCall_v_25,b7,jsCall_v_26,b7,jsCall_v_27,b7,jsCall_v_28
,b7,jsCall_v_29,b7,jsCall_v_30,b7,jsCall_v_31,b7,jsCall_v_32,b7,jsCall_v_33,b7,jsCall_v_34,b7,jsCall_v_35,b7,jsCall_v_36,b7,jsCall_v_37,b7,jsCall_v_38,b7,jsCall_v_39,b7,jsCall_v_40,b7,jsCall_v_41,b7,jsCall_v_42,b7,jsCall_v_43
,b7,jsCall_v_44,b7,jsCall_v_45,b7,jsCall_v_46,b7,jsCall_v_47,b7,jsCall_v_48,b7,jsCall_v_49,b7,jsCall_v_50,b7,jsCall_v_51,b7,jsCall_v_52,b7,jsCall_v_53,b7,jsCall_v_54,b7,jsCall_v_55,b7,jsCall_v_56,b7,jsCall_v_57,b7,jsCall_v_58
,b7,jsCall_v_59,b7,jsCall_v_60,b7,jsCall_v_61,b7,jsCall_v_62,b7,jsCall_v_63,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7
,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7,b7];
var FUNCTION_TABLE_iiiii = [b8,b8,jsCall_iiiii_0,b8,jsCall_iiiii_1,b8,jsCall_iiiii_2,b8,jsCall_iiiii_3,b8,jsCall_iiiii_4,b8,jsCall_iiiii_5,b8,jsCall_iiiii_6,b8,jsCall_iiiii_7,b8,jsCall_iiiii_8,b8,jsCall_iiiii_9,b8,jsCall_iiiii_10,b8,jsCall_iiiii_11,b8,jsCall_iiiii_12,b8,jsCall_iiiii_13
,b8,jsCall_iiiii_14,b8,jsCall_iiiii_15,b8,jsCall_iiiii_16,b8,jsCall_iiiii_17,b8,jsCall_iiiii_18,b8,jsCall_iiiii_19,b8,jsCall_iiiii_20,b8,jsCall_iiiii_21,b8,jsCall_iiiii_22,b8,jsCall_iiiii_23,b8,jsCall_iiiii_24,b8,jsCall_iiiii_25,b8,jsCall_iiiii_26,b8,jsCall_iiiii_27,b8,jsCall_iiiii_28
,b8,jsCall_iiiii_29,b8,jsCall_iiiii_30,b8,jsCall_iiiii_31,b8,jsCall_iiiii_32,b8,jsCall_iiiii_33,b8,jsCall_iiiii_34,b8,jsCall_iiiii_35,b8,jsCall_iiiii_36,b8,jsCall_iiiii_37,b8,jsCall_iiiii_38,b8,jsCall_iiiii_39,b8,jsCall_iiiii_40,b8,jsCall_iiiii_41,b8,jsCall_iiiii_42,b8,jsCall_iiiii_43
,b8,jsCall_iiiii_44,b8,jsCall_iiiii_45,b8,jsCall_iiiii_46,b8,jsCall_iiiii_47,b8,jsCall_iiiii_48,b8,jsCall_iiiii_49,b8,jsCall_iiiii_50,b8,jsCall_iiiii_51,b8,jsCall_iiiii_52,b8,jsCall_iiiii_53,b8,jsCall_iiiii_54,b8,jsCall_iiiii_55,b8,jsCall_iiiii_56,b8,jsCall_iiiii_57,b8,jsCall_iiiii_58
,b8,jsCall_iiiii_59,b8,jsCall_iiiii_60,b8,jsCall_iiiii_61,b8,jsCall_iiiii_62,b8,jsCall_iiiii_63,b8,_unixAccess,_unixFullPathname,_fts3UpdateMethod,_porterOpen,_simpleOpen,_unicodeOpen,_unixShmLock,_unixUnfetch,_sqlite3BtreeData,_sqlite3WalDefaultHook,_sqlite3InitCallback,_analysisLoader,_fts3StrCompare,_fts3BinCompare,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8
,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8,b8];
var FUNCTION_TABLE_viiiiii = [b9,b9,jsCall_viiiiii_0,b9,jsCall_viiiiii_1,b9,jsCall_viiiiii_2,b9,jsCall_viiiiii_3,b9,jsCall_viiiiii_4,b9,jsCall_viiiiii_5,b9,jsCall_viiiiii_6,b9,jsCall_viiiiii_7,b9,jsCall_viiiiii_8,b9,jsCall_viiiiii_9,b9,jsCall_viiiiii_10,b9,jsCall_viiiiii_11,b9,jsCall_viiiiii_12,b9,jsCall_viiiiii_13
,b9,jsCall_viiiiii_14,b9,jsCall_viiiiii_15,b9,jsCall_viiiiii_16,b9,jsCall_viiiiii_17,b9,jsCall_viiiiii_18,b9,jsCall_viiiiii_19,b9,jsCall_viiiiii_20,b9,jsCall_viiiiii_21,b9,jsCall_viiiiii_22,b9,jsCall_viiiiii_23,b9,jsCall_viiiiii_24,b9,jsCall_viiiiii_25,b9,jsCall_viiiiii_26,b9,jsCall_viiiiii_27,b9,jsCall_viiiiii_28
,b9,jsCall_viiiiii_29,b9,jsCall_viiiiii_30,b9,jsCall_viiiiii_31,b9,jsCall_viiiiii_32,b9,jsCall_viiiiii_33,b9,jsCall_viiiiii_34,b9,jsCall_viiiiii_35,b9,jsCall_viiiiii_36,b9,jsCall_viiiiii_37,b9,jsCall_viiiiii_38,b9,jsCall_viiiiii_39,b9,jsCall_viiiiii_40,b9,jsCall_viiiiii_41,b9,jsCall_viiiiii_42,b9,jsCall_viiiiii_43
,b9,jsCall_viiiiii_44,b9,jsCall_viiiiii_45,b9,jsCall_viiiiii_46,b9,jsCall_viiiiii_47,b9,jsCall_viiiiii_48,b9,jsCall_viiiiii_49,b9,jsCall_viiiiii_50,b9,jsCall_viiiiii_51,b9,jsCall_viiiiii_52,b9,jsCall_viiiiii_53,b9,jsCall_viiiiii_54,b9,jsCall_viiiiii_55,b9,jsCall_viiiiii_56,b9,jsCall_viiiiii_57,b9,jsCall_viiiiii_58
,b9,jsCall_viiiiii_59,b9,jsCall_viiiiii_60,b9,jsCall_viiiiii_61,b9,jsCall_viiiiii_62,b9,jsCall_viiiiii_63,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9
,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9,b9];
var FUNCTION_TABLE_iii = [b10,b10,jsCall_iii_0,b10,jsCall_iii_1,b10,jsCall_iii_2,b10,jsCall_iii_3,b10,jsCall_iii_4,b10,jsCall_iii_5,b10,jsCall_iii_6,b10,jsCall_iii_7,b10,jsCall_iii_8,b10,jsCall_iii_9,b10,jsCall_iii_10,b10,jsCall_iii_11,b10,jsCall_iii_12,b10,jsCall_iii_13
,b10,jsCall_iii_14,b10,jsCall_iii_15,b10,jsCall_iii_16,b10,jsCall_iii_17,b10,jsCall_iii_18,b10,jsCall_iii_19,b10,jsCall_iii_20,b10,jsCall_iii_21,b10,jsCall_iii_22,b10,jsCall_iii_23,b10,jsCall_iii_24,b10,jsCall_iii_25,b10,jsCall_iii_26,b10,jsCall_iii_27,b10,jsCall_iii_28
,b10,jsCall_iii_29,b10,jsCall_iii_30,b10,jsCall_iii_31,b10,jsCall_iii_32,b10,jsCall_iii_33,b10,jsCall_iii_34,b10,jsCall_iii_35,b10,jsCall_iii_36,b10,jsCall_iii_37,b10,jsCall_iii_38,b10,jsCall_iii_39,b10,jsCall_iii_40,b10,jsCall_iii_41,b10,jsCall_iii_42,b10,jsCall_iii_43
,b10,jsCall_iii_44,b10,jsCall_iii_45,b10,jsCall_iii_46,b10,jsCall_iii_47,b10,jsCall_iii_48,b10,jsCall_iii_49,b10,jsCall_iii_50,b10,jsCall_iii_51,b10,jsCall_iii_52,b10,jsCall_iii_53,b10,jsCall_iii_54,b10,jsCall_iii_55,b10,jsCall_iii_56,b10,jsCall_iii_57,b10,jsCall_iii_58
,b10,jsCall_iii_59,b10,jsCall_iii_60,b10,jsCall_iii_61,b10,jsCall_iii_62,b10,jsCall_iii_63,b10,_unixSleep,_unixCurrentTime,_unixCurrentTimeInt64,_unixGetSystemCall,_unixNextSystemCall,_fts3BestIndexMethod,_fts3OpenMethod,_fts3RowidMethod,_fts3RenameMethod,_fts3SavepointMethod,_fts3ReleaseMethod,_fts3RollbackToMethod,_fts3tokBestIndexMethod,_fts3tokOpenMethod,_fts3tokRowidMethod,_fts3auxBestIndexMethod,_fts3auxOpenMethod,_fts3auxRowidMethod,_memjrnlSync
,_memjrnlFileSize,_sqlite3MemRealloc,_posixIoFinderImpl,_nolockIoFinderImpl,_dotlockIoFinderImpl,_unixSync,_unixFileSize,_dotlockLock,_dotlockUnlock,_dotlockCheckReservedLock,_unixShmUnmap,_access,_getcwd,_stat,_fstat,_ftruncate,_fchmod,_openDirectory,_mkdir,___munmap,_lstat,_nolockLock,_nolockUnlock,_nolockCheckReservedLock,_unixLock,_unixUnlock,_unixCheckReservedLock,_int_cmp,_double_cmp,_sqliteDefaultBusyCallback
,_sqlite3Malloc,_fts3StrHash,_fts3BinHash,_fts3SegReaderDoclistCmpRev,_fts3SegReaderDoclistCmp,_fts3SegReaderCmp,_fts3CompareElemByTerm,_pagerStress,_pagerUndoCallback,_cellSizePtr,_cellSizePtrNoPayload,_sqlite3ExprWalkNoop,_convertCompoundSelectToSubquery,_selectExpander,_resolveExprStep,_resolveSelectStep,_incrAggDepth,_exprSrcCount,_exprNodeIsConstant,_selectNodeIsConstant,_exprIdxCover,_checkConstraintExprNode,_sqlite3BtreeNext,_sqlite3BtreePrevious,_analyzeAggregate,_analyzeAggregatesInSelect,b10,b10,b10,b10
,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10
,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10,b10];
var FUNCTION_TABLE_iiiiii = [b11,b11,jsCall_iiiiii_0,b11,jsCall_iiiiii_1,b11,jsCall_iiiiii_2,b11,jsCall_iiiiii_3,b11,jsCall_iiiiii_4,b11,jsCall_iiiiii_5,b11,jsCall_iiiiii_6,b11,jsCall_iiiiii_7,b11,jsCall_iiiiii_8,b11,jsCall_iiiiii_9,b11,jsCall_iiiiii_10,b11,jsCall_iiiiii_11,b11,jsCall_iiiiii_12,b11,jsCall_iiiiii_13
,b11,jsCall_iiiiii_14,b11,jsCall_iiiiii_15,b11,jsCall_iiiiii_16,b11,jsCall_iiiiii_17,b11,jsCall_iiiiii_18,b11,jsCall_iiiiii_19,b11,jsCall_iiiiii_20,b11,jsCall_iiiiii_21,b11,jsCall_iiiiii_22,b11,jsCall_iiiiii_23,b11,jsCall_iiiiii_24,b11,jsCall_iiiiii_25,b11,jsCall_iiiiii_26,b11,jsCall_iiiiii_27,b11,jsCall_iiiiii_28
,b11,jsCall_iiiiii_29,b11,jsCall_iiiiii_30,b11,jsCall_iiiiii_31,b11,jsCall_iiiiii_32,b11,jsCall_iiiiii_33,b11,jsCall_iiiiii_34,b11,jsCall_iiiiii_35,b11,jsCall_iiiiii_36,b11,jsCall_iiiiii_37,b11,jsCall_iiiiii_38,b11,jsCall_iiiiii_39,b11,jsCall_iiiiii_40,b11,jsCall_iiiiii_41,b11,jsCall_iiiiii_42,b11,jsCall_iiiiii_43
,b11,jsCall_iiiiii_44,b11,jsCall_iiiiii_45,b11,jsCall_iiiiii_46,b11,jsCall_iiiiii_47,b11,jsCall_iiiiii_48,b11,jsCall_iiiiii_49,b11,jsCall_iiiiii_50,b11,jsCall_iiiiii_51,b11,jsCall_iiiiii_52,b11,jsCall_iiiiii_53,b11,jsCall_iiiiii_54,b11,jsCall_iiiiii_55,b11,jsCall_iiiiii_56,b11,jsCall_iiiiii_57,b11,jsCall_iiiiii_58
,b11,jsCall_iiiiii_59,b11,jsCall_iiiiii_60,b11,jsCall_iiiiii_61,b11,jsCall_iiiiii_62,b11,jsCall_iiiiii_63,b11,_unixOpen,_fts3FilterMethod,_fts3FindFunctionMethod,_fts3tokFilterMethod,_fts3auxFilterMethod,_memjrnlRead,_memjrnlWrite,_unixRead,_unixWrite,_unixFetch,_unixShmMap,_binCollFunc,_nocaseCollatingFunc,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11
,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11,b11];
var FUNCTION_TABLE_viiii = [b12,b12,jsCall_viiii_0,b12,jsCall_viiii_1,b12,jsCall_viiii_2,b12,jsCall_viiii_3,b12,jsCall_viiii_4,b12,jsCall_viiii_5,b12,jsCall_viiii_6,b12,jsCall_viiii_7,b12,jsCall_viiii_8,b12,jsCall_viiii_9,b12,jsCall_viiii_10,b12,jsCall_viiii_11,b12,jsCall_viiii_12,b12,jsCall_viiii_13
,b12,jsCall_viiii_14,b12,jsCall_viiii_15,b12,jsCall_viiii_16,b12,jsCall_viiii_17,b12,jsCall_viiii_18,b12,jsCall_viiii_19,b12,jsCall_viiii_20,b12,jsCall_viiii_21,b12,jsCall_viiii_22,b12,jsCall_viiii_23,b12,jsCall_viiii_24,b12,jsCall_viiii_25,b12,jsCall_viiii_26,b12,jsCall_viiii_27,b12,jsCall_viiii_28
,b12,jsCall_viiii_29,b12,jsCall_viiii_30,b12,jsCall_viiii_31,b12,jsCall_viiii_32,b12,jsCall_viiii_33,b12,jsCall_viiii_34,b12,jsCall_viiii_35,b12,jsCall_viiii_36,b12,jsCall_viiii_37,b12,jsCall_viiii_38,b12,jsCall_viiii_39,b12,jsCall_viiii_40,b12,jsCall_viiii_41,b12,jsCall_viiii_42,b12,jsCall_viiii_43
,b12,jsCall_viiii_44,b12,jsCall_viiii_45,b12,jsCall_viiii_46,b12,jsCall_viiii_47,b12,jsCall_viiii_48,b12,jsCall_viiii_49,b12,jsCall_viiii_50,b12,jsCall_viiii_51,b12,jsCall_viiii_52,b12,jsCall_viiii_53,b12,jsCall_viiii_54,b12,jsCall_viiii_55,b12,jsCall_viiii_56,b12,jsCall_viiii_57,b12,jsCall_viiii_58
,b12,jsCall_viiii_59,b12,jsCall_viiii_60,b12,jsCall_viiii_61,b12,jsCall_viiii_62,b12,jsCall_viiii_63,b12,_pcache1Rekey,_modeIterate,_medianIterate,_sqlite3ExprIfFalse,_sqlite3ExprIfTrue,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12
,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12,b12];

  return { _sqlite3_value_blob: _sqlite3_value_blob, _sqlite3_column_name: _sqlite3_column_name, _sqlite3_reset: _sqlite3_reset, _sqlite3_column_type: _sqlite3_column_type, _sqlite3_exec: _sqlite3_exec, _sqlite3_result_null: _sqlite3_result_null, _sqlite3_step: _sqlite3_step, _bitshift64Lshr: _bitshift64Lshr, _sqlite3_prepare_v2: _sqlite3_prepare_v2, _sqlite3_close_v2: _sqlite3_close_v2, _sqlite3_open: _sqlite3_open, _bitshift64Shl: _bitshift64Shl, _sqlite3_result_text: _sqlite3_result_text, _fflush: _fflush, _sqlite3_column_bytes: _sqlite3_column_bytes, _sqlite3_bind_int: _sqlite3_bind_int, _sqlite3_bind_blob: _sqlite3_bind_blob, _memset: _memset, _sqlite3_value_double: _sqlite3_value_double, _memcpy: _memcpy, _sqlite3_result_double: _sqlite3_result_double, _sqlite3_value_text: _sqlite3_value_text, _sqlite3_changes: _sqlite3_changes, _sqlite3_column_blob: _sqlite3_column_blob, _sqlite3_bind_parameter_index: _sqlite3_bind_parameter_index, _sqlite3_value_type: _sqlite3_value_type, _i64Subtract: _i64Subtract, _sqlite3_column_text: _sqlite3_column_text, _i64Add: _i64Add, _sqlite3_value_bytes: _sqlite3_value_bytes, _sqlite3_finalize: _sqlite3_finalize, _sqlite3_column_double: _sqlite3_column_double, _sqlite3_create_function_v2: _sqlite3_create_function_v2, _sqlite3_free: _sqlite3_free, _sqlite3_value_int: _sqlite3_value_int, _sqlite3_data_count: _sqlite3_data_count, _sqlite3_bind_text: _sqlite3_bind_text, _sqlite3_bind_double: _sqlite3_bind_double, ___errno_location: ___errno_location, _RegisterExtensionFunctions: _RegisterExtensionFunctions, _free: _free, _memmove: _memmove, _sqlite3_errmsg: _sqlite3_errmsg, _sqlite3_clear_bindings: _sqlite3_clear_bindings, _malloc: _malloc, runPostSets: runPostSets, stackAlloc: stackAlloc, stackSave: stackSave, stackRestore: stackRestore, establishStackSpace: establishStackSpace, setThrew: setThrew, setTempRet0: setTempRet0, getTempRet0: getTempRet0, dynCall_iiii: dynCall_iiii, dynCall_i: dynCall_i, dynCall_vi: dynCall_vi, dynCall_vii: dynCall_vii, dynCall_iiiiiii: dynCall_iiiiiii, dynCall_ii: dynCall_ii, dynCall_viii: dynCall_viii, dynCall_v: dynCall_v, dynCall_iiiii: dynCall_iiiii, dynCall_viiiiii: dynCall_viiiiii, dynCall_iii: dynCall_iii, dynCall_iiiiii: dynCall_iiiiii, dynCall_viiii: dynCall_viiii };
})
// EMSCRIPTEN_END_ASM
(Module.asmGlobalArg, Module.asmLibraryArg, buffer);
var _sqlite3_value_blob = Module["_sqlite3_value_blob"] = asm["_sqlite3_value_blob"];
var _sqlite3_column_name = Module["_sqlite3_column_name"] = asm["_sqlite3_column_name"];
var _sqlite3_reset = Module["_sqlite3_reset"] = asm["_sqlite3_reset"];
var _sqlite3_column_type = Module["_sqlite3_column_type"] = asm["_sqlite3_column_type"];
var _sqlite3_exec = Module["_sqlite3_exec"] = asm["_sqlite3_exec"];
var _sqlite3_result_null = Module["_sqlite3_result_null"] = asm["_sqlite3_result_null"];
var _sqlite3_step = Module["_sqlite3_step"] = asm["_sqlite3_step"];
var _bitshift64Lshr = Module["_bitshift64Lshr"] = asm["_bitshift64Lshr"];
var _sqlite3_prepare_v2 = Module["_sqlite3_prepare_v2"] = asm["_sqlite3_prepare_v2"];
var _sqlite3_close_v2 = Module["_sqlite3_close_v2"] = asm["_sqlite3_close_v2"];
var _sqlite3_open = Module["_sqlite3_open"] = asm["_sqlite3_open"];
var _bitshift64Shl = Module["_bitshift64Shl"] = asm["_bitshift64Shl"];
var _sqlite3_result_text = Module["_sqlite3_result_text"] = asm["_sqlite3_result_text"];
var _fflush = Module["_fflush"] = asm["_fflush"];
var _sqlite3_column_bytes = Module["_sqlite3_column_bytes"] = asm["_sqlite3_column_bytes"];
var _sqlite3_bind_int = Module["_sqlite3_bind_int"] = asm["_sqlite3_bind_int"];
var _sqlite3_bind_blob = Module["_sqlite3_bind_blob"] = asm["_sqlite3_bind_blob"];
var _memset = Module["_memset"] = asm["_memset"];
var _sqlite3_value_double = Module["_sqlite3_value_double"] = asm["_sqlite3_value_double"];
var _memcpy = Module["_memcpy"] = asm["_memcpy"];
var _sqlite3_result_double = Module["_sqlite3_result_double"] = asm["_sqlite3_result_double"];
var _sqlite3_value_text = Module["_sqlite3_value_text"] = asm["_sqlite3_value_text"];
var _sqlite3_changes = Module["_sqlite3_changes"] = asm["_sqlite3_changes"];
var _sqlite3_column_blob = Module["_sqlite3_column_blob"] = asm["_sqlite3_column_blob"];
var _sqlite3_bind_parameter_index = Module["_sqlite3_bind_parameter_index"] = asm["_sqlite3_bind_parameter_index"];
var _sqlite3_value_type = Module["_sqlite3_value_type"] = asm["_sqlite3_value_type"];
var _i64Subtract = Module["_i64Subtract"] = asm["_i64Subtract"];
var _sqlite3_column_text = Module["_sqlite3_column_text"] = asm["_sqlite3_column_text"];
var _i64Add = Module["_i64Add"] = asm["_i64Add"];
var _sqlite3_value_bytes = Module["_sqlite3_value_bytes"] = asm["_sqlite3_value_bytes"];
var _sqlite3_finalize = Module["_sqlite3_finalize"] = asm["_sqlite3_finalize"];
var _sqlite3_column_double = Module["_sqlite3_column_double"] = asm["_sqlite3_column_double"];
var _sqlite3_create_function_v2 = Module["_sqlite3_create_function_v2"] = asm["_sqlite3_create_function_v2"];
var _sqlite3_free = Module["_sqlite3_free"] = asm["_sqlite3_free"];
var _sqlite3_value_int = Module["_sqlite3_value_int"] = asm["_sqlite3_value_int"];
var _sqlite3_data_count = Module["_sqlite3_data_count"] = asm["_sqlite3_data_count"];
var _sqlite3_bind_text = Module["_sqlite3_bind_text"] = asm["_sqlite3_bind_text"];
var _sqlite3_bind_double = Module["_sqlite3_bind_double"] = asm["_sqlite3_bind_double"];
var ___errno_location = Module["___errno_location"] = asm["___errno_location"];
var _RegisterExtensionFunctions = Module["_RegisterExtensionFunctions"] = asm["_RegisterExtensionFunctions"];
var _free = Module["_free"] = asm["_free"];
var runPostSets = Module["runPostSets"] = asm["runPostSets"];
var _memmove = Module["_memmove"] = asm["_memmove"];
var _sqlite3_errmsg = Module["_sqlite3_errmsg"] = asm["_sqlite3_errmsg"];
var _sqlite3_clear_bindings = Module["_sqlite3_clear_bindings"] = asm["_sqlite3_clear_bindings"];
var _malloc = Module["_malloc"] = asm["_malloc"];
var dynCall_iiii = Module["dynCall_iiii"] = asm["dynCall_iiii"];
var dynCall_i = Module["dynCall_i"] = asm["dynCall_i"];
var dynCall_vi = Module["dynCall_vi"] = asm["dynCall_vi"];
var dynCall_vii = Module["dynCall_vii"] = asm["dynCall_vii"];
var dynCall_iiiiiii = Module["dynCall_iiiiiii"] = asm["dynCall_iiiiiii"];
var dynCall_ii = Module["dynCall_ii"] = asm["dynCall_ii"];
var dynCall_viii = Module["dynCall_viii"] = asm["dynCall_viii"];
var dynCall_v = Module["dynCall_v"] = asm["dynCall_v"];
var dynCall_iiiii = Module["dynCall_iiiii"] = asm["dynCall_iiiii"];
var dynCall_viiiiii = Module["dynCall_viiiiii"] = asm["dynCall_viiiiii"];
var dynCall_iii = Module["dynCall_iii"] = asm["dynCall_iii"];
var dynCall_iiiiii = Module["dynCall_iiiiii"] = asm["dynCall_iiiiii"];
var dynCall_viiii = Module["dynCall_viiii"] = asm["dynCall_viiii"];
;

Runtime.stackAlloc = asm['stackAlloc'];
Runtime.stackSave = asm['stackSave'];
Runtime.stackRestore = asm['stackRestore'];
Runtime.establishStackSpace = asm['establishStackSpace'];

Runtime.setTempRet0 = asm['setTempRet0'];
Runtime.getTempRet0 = asm['getTempRet0'];



// === Auto-generated postamble setup entry stuff ===


function ExitStatus(status) {
  this.name = "ExitStatus";
  this.message = "Program terminated with exit(" + status + ")";
  this.status = status;
};
ExitStatus.prototype = new Error();
ExitStatus.prototype.constructor = ExitStatus;

var initialStackTop;
var preloadStartTime = null;
var calledMain = false;

dependenciesFulfilled = function runCaller() {
  // If run has never been called, and we should call run (INVOKE_RUN is true, and Module.noInitialRun is not false)
  if (!Module['calledRun']) run();
  if (!Module['calledRun']) dependenciesFulfilled = runCaller; // try this again later, after new deps are fulfilled
}

Module['callMain'] = Module.callMain = function callMain(args) {
  assert(runDependencies == 0, 'cannot call main when async dependencies remain! (listen on __ATMAIN__)');
  assert(__ATPRERUN__.length == 0, 'cannot call main when preRun functions remain to be called');

  args = args || [];

  ensureInitRuntime();

  var argc = args.length+1;
  function pad() {
    for (var i = 0; i < 4-1; i++) {
      argv.push(0);
    }
  }
  var argv = [allocate(intArrayFromString(Module['thisProgram']), 'i8', ALLOC_NORMAL) ];
  pad();
  for (var i = 0; i < argc-1; i = i + 1) {
    argv.push(allocate(intArrayFromString(args[i]), 'i8', ALLOC_NORMAL));
    pad();
  }
  argv.push(0);
  argv = allocate(argv, 'i32', ALLOC_NORMAL);


  try {

    var ret = Module['_main'](argc, argv, 0);


    // if we're not running an evented main loop, it's time to exit
    exit(ret, /* implicit = */ true);
  }
  catch(e) {
    if (e instanceof ExitStatus) {
      // exit() throws this once it's done to make sure execution
      // has been stopped completely
      return;
    } else if (e == 'SimulateInfiniteLoop') {
      // running an evented main loop, don't immediately exit
      Module['noExitRuntime'] = true;
      return;
    } else {
      if (e && typeof e === 'object' && e.stack) Module.printErr('exception thrown: ' + [e, e.stack]);
      throw e;
    }
  } finally {
    calledMain = true;
  }
}




function run(args) {
  args = args || Module['arguments'];

  if (preloadStartTime === null) preloadStartTime = Date.now();

  if (runDependencies > 0) {
    return;
  }

  preRun();

  if (runDependencies > 0) return; // a preRun added a dependency, run will be called later
  if (Module['calledRun']) return; // run may have just been called through dependencies being fulfilled just in this very frame

  function doRun() {
    if (Module['calledRun']) return; // run may have just been called while the async setStatus time below was happening
    Module['calledRun'] = true;

    if (ABORT) return; 

    ensureInitRuntime();

    preMain();


    if (Module['onRuntimeInitialized']) Module['onRuntimeInitialized']();

    if (Module['_main'] && shouldRunNow) Module['callMain'](args);

    postRun();
  }

  if (Module['setStatus']) {
    Module['setStatus']('Running...');
    setTimeout(function() {
      setTimeout(function() {
        Module['setStatus']('');
      }, 1);
      doRun();
    }, 1);
  } else {
    doRun();
  }
}
Module['run'] = Module.run = run;

function exit(status, implicit) {
  if (implicit && Module['noExitRuntime']) {
    return;
  }

  if (Module['noExitRuntime']) {
  } else {

    ABORT = true;
    EXITSTATUS = status;
    STACKTOP = initialStackTop;

    exitRuntime();

    if (Module['onExit']) Module['onExit'](status);
  }

  if (ENVIRONMENT_IS_NODE) {
    // Work around a node.js bug where stdout buffer is not flushed at process exit:
    // Instead of process.exit() directly, wait for stdout flush event.
    // See https://github.com/joyent/node/issues/1669 and https://github.com/kripken/emscripten/issues/2582
    // Workaround is based on https://github.com/RReverser/acorn/commit/50ab143cecc9ed71a2d66f78b4aec3bb2e9844f6
    process['stdout']['once']('drain', function () {
      process['exit'](status);
    });
    console.log(' '); // Make sure to print something to force the drain event to occur, in case the stdout buffer was empty.
    // Work around another node bug where sometimes 'drain' is never fired - make another effort
    // to emit the exit status, after a significant delay (if node hasn't fired drain by then, give up)
    setTimeout(function() {
      process['exit'](status);
    }, 500);
  } else
  if (ENVIRONMENT_IS_SHELL && typeof quit === 'function') {
    quit(status);
  }
  // if we reach here, we must throw an exception to halt the current execution
  throw new ExitStatus(status);
}
Module['exit'] = Module.exit = exit;

var abortDecorators = [];

function abort(what) {
  if (what !== undefined) {
    Module.print(what);
    Module.printErr(what);
    what = JSON.stringify(what)
  } else {
    what = '';
  }

  ABORT = true;
  EXITSTATUS = 1;

  var extra = '\nIf this abort() is unexpected, build with -s ASSERTIONS=1 which can give more information.';

  var output = 'abort(' + what + ') at ' + stackTrace() + extra;
  if (abortDecorators) {
    abortDecorators.forEach(function(decorator) {
      output = decorator(output, what);
    });
  }
  throw output;
}
Module['abort'] = Module.abort = abort;

// {{PRE_RUN_ADDITIONS}}

if (Module['preInit']) {
  if (typeof Module['preInit'] == 'function') Module['preInit'] = [Module['preInit']];
  while (Module['preInit'].length > 0) {
    Module['preInit'].pop()();
  }
}

// shouldRunNow refers to calling main(), not run().
var shouldRunNow = true;
if (Module['noInitialRun']) {
  shouldRunNow = false;
}


run();

// {{POST_RUN_ADDITIONS}}






// {{MODULE_ADDITIONS}}



// Generated by CoffeeScript 1.9.2
var Database, NULL, RegisterExtensionFunctions, Runtime, SQLite, Statement, apiTemp, i, sqlite3_bind_blob, sqlite3_bind_double, sqlite3_bind_int, sqlite3_bind_parameter_index, sqlite3_bind_text, sqlite3_changes, sqlite3_clear_bindings, sqlite3_close_v2, sqlite3_column_blob, sqlite3_column_bytes, sqlite3_column_double, sqlite3_column_name, sqlite3_column_text, sqlite3_column_type, sqlite3_create_function_v2, sqlite3_data_count, sqlite3_errmsg, sqlite3_exec, sqlite3_finalize, sqlite3_free, sqlite3_open, sqlite3_prepare_v2, sqlite3_prepare_v2_sqlptr, sqlite3_reset, sqlite3_result_double, sqlite3_result_null, sqlite3_result_text, sqlite3_step, sqlite3_value_blob, sqlite3_value_bytes, sqlite3_value_double, sqlite3_value_int, sqlite3_value_text, sqlite3_value_type;

Runtime = Module['Runtime'];

apiTemp = Runtime.stackAlloc(4);

SQLite = {};


/* Represents an prepared statement.

Prepared statements allow you to have a template sql string,
that you can execute multiple times with different parameters.

You can't instantiate this class directly, you have to use a [Database](Database.html)
object in order to create a statement.

**Warning**: When you close a database (using db.close()), all its statements are
closed too and become unusable.

@see Database.html#prepare-dynamic
@see https://en.wikipedia.org/wiki/Prepared_statement
 */

Statement = (function() {
  function Statement(stmt1, db) {
    this.stmt = stmt1;
    this.db = db;
    this.pos = 1;
    this.allocatedmem = [];
  }


  /* Bind values to the parameters, after having reseted the statement
  
  SQL statements can have parameters, named *'?', '?NNN', ':VVV', '@VVV', '$VVV'*,
  where NNN is a number and VVV a string.
  This function binds these parameters to the given values.
  
  *Warning*: ':', '@', and '$' are included in the parameters names
  
  ## Binding values to named parameters
  @example Bind values to named parameters
      var stmt = db.prepare("UPDATE test SET a=@newval WHERE id BETWEEN $mini AND $maxi");
      stmt.bind({$mini:10, $maxi:20, '@newval':5});
  - Create a statement that contains parameters like '$VVV', ':VVV', '@VVV'
  - Call Statement.bind with an object as parameter
  
  ## Binding values to parameters
  @example Bind values to anonymous parameters
      var stmt = db.prepare("UPDATE test SET a=? WHERE id BETWEEN ? AND ?");
      stmt.bind([5, 10, 20]);
   - Create a statement that contains parameters like '?', '?NNN'
   - Call Statement.bind with an array as parameter
  
  ## Value types
  Javascript type | SQLite type
  --- | ---
  number | REAL, INTEGER
  boolean | INTEGER
  string | TEXT
  Array, Uint8Array | BLOB
  null | NULL
  @see http://www.sqlite.org/datatype3.html
  
  @see http://www.sqlite.org/lang_expr.html#varparam
  @param values [Array,Object] The values to bind
  @return [Boolean] true if it worked
  @throw [String] SQLite Error
   */

  Statement.prototype['bind'] = function(values) {
    if (!this.stmt) {
      throw "Statement closed";
    }
    this['reset']();
    if (Array.isArray(values)) {
      return this.bindFromArray(values);
    } else {
      return this.bindFromObject(values);
    }
  };


  /* Execute the statement, fetching the the next line of result,
  that can be retrieved with [Statement.get()](#get-dynamic) .
  
  @return [Boolean] true if a row of result available
  @throw [String] SQLite Error
   */

  Statement.prototype['step'] = function() {
    var ret;
    if (!this.stmt) {
      throw "Statement closed";
    }
    this.pos = 1;
    switch (ret = sqlite3_step(this.stmt)) {
      case SQLite.ROW:
        return true;
      case SQLite.DONE:
        return false;
      default:
        return this.db.handleError(ret);
    }
  };

  Statement.prototype.getNumber = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_column_double(this.stmt, pos);
  };

  Statement.prototype.getString = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_column_text(this.stmt, pos);
  };

  Statement.prototype.getBlob = function(pos) {
    var i, k, ptr, ref, result, size;
    if (pos == null) {
      pos = this.pos++;
    }
    size = sqlite3_column_bytes(this.stmt, pos);
    ptr = sqlite3_column_blob(this.stmt, pos);
    result = new Uint8Array(size);
    for (i = k = 0, ref = size; 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
      result[i] = HEAP8[ptr + i];
    }
    return result;
  };


  /* Get one row of results of a statement.
  If the first parameter is not provided, step must have been called before get.
  @param [Array,Object] Optional: If set, the values will be bound to the statement, and it will be executed
  @return [Array<String,Number,Uint8Array,null>] One row of result
  
  @example Print all the rows of the table test to the console
  
      var stmt = db.prepare("SELECT * FROM test");
      while (stmt.step()) console.log(stmt.get());
   */

  Statement.prototype['get'] = function(params) {
    var field, k, ref, results1;
    if (params != null) {
      this['bind'](params) && this['step']();
    }
    results1 = [];
    for (field = k = 0, ref = sqlite3_data_count(this.stmt); 0 <= ref ? k < ref : k > ref; field = 0 <= ref ? ++k : --k) {
      switch (sqlite3_column_type(this.stmt, field)) {
        case SQLite.INTEGER:
        case SQLite.FLOAT:
          results1.push(this.getNumber(field));
          break;
        case SQLite.TEXT:
          results1.push(this.getString(field));
          break;
        case SQLite.BLOB:
          results1.push(this.getBlob(field));
          break;
        default:
          results1.push(null);
      }
    }
    return results1;
  };


  /* Get the list of column names of a row of result of a statement.
  @return [Array<String>] The names of the columns
  @example
  
      var stmt = db.prepare("SELECT 5 AS nbr, x'616200' AS data, NULL AS nothing;");
      stmt.step(); // Execute the statement
      console.log(stmt.getColumnNames()); // Will print ['nbr','data','nothing']
   */

  Statement.prototype['getColumnNames'] = function() {
    var i, k, ref, results1;
    results1 = [];
    for (i = k = 0, ref = sqlite3_data_count(this.stmt); 0 <= ref ? k < ref : k > ref; i = 0 <= ref ? ++k : --k) {
      results1.push(sqlite3_column_name(this.stmt, i));
    }
    return results1;
  };


  /* Get one row of result as a javascript object, associating column names with
  their value in the current row.
  @param [Array,Object] Optional: If set, the values will be bound to the statement, and it will be executed
  @return [Object] The row of result
  @see [Statement.get](#get-dynamic)
  
  @example
  
      var stmt = db.prepare("SELECT 5 AS nbr, x'616200' AS data, NULL AS nothing;");
      stmt.step(); // Execute the statement
      console.log(stmt.getAsObject()); // Will print {nbr:5, data: Uint8Array([1,2,3]), nothing:null}
   */

  Statement.prototype['getAsObject'] = function(params) {
    var i, k, len, name, names, rowObject, values;
    values = this['get'](params);
    names = this['getColumnNames']();
    rowObject = {};
    for (i = k = 0, len = names.length; k < len; i = ++k) {
      name = names[i];
      rowObject[name] = values[i];
    }
    return rowObject;
  };


  /* Shorthand for bind + step + reset
  Bind the values, execute the statement, ignoring the rows it returns, and resets it
  @param [Array,Object] Value to bind to the statement
   */

  Statement.prototype['run'] = function(values) {
    if (values != null) {
      this['bind'](values);
    }
    this['step']();
    return this['reset']();
  };

  Statement.prototype.bindString = function(string, pos) {
    var bytes, strptr;
    if (pos == null) {
      pos = this.pos++;
    }
    bytes = intArrayFromString(string);
    this.allocatedmem.push(strptr = allocate(bytes, 'i8', ALLOC_NORMAL));
    this.db.handleError(sqlite3_bind_text(this.stmt, pos, strptr, bytes.length - 1, 0));
    return true;
  };

  Statement.prototype.bindBlob = function(array, pos) {
    var blobptr;
    if (pos == null) {
      pos = this.pos++;
    }
    this.allocatedmem.push(blobptr = allocate(array, 'i8', ALLOC_NORMAL));
    this.db.handleError(sqlite3_bind_blob(this.stmt, pos, blobptr, array.length, 0));
    return true;
  };

  Statement.prototype.bindNumber = function(num, pos) {
    var bindfunc;
    if (pos == null) {
      pos = this.pos++;
    }
    bindfunc = num === (num | 0) ? sqlite3_bind_int : sqlite3_bind_double;
    this.db.handleError(bindfunc(this.stmt, pos, num));
    return true;
  };

  Statement.prototype.bindNull = function(pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    return sqlite3_bind_blob(this.stmt, pos, 0, 0, 0) === SQLite.OK;
  };

  Statement.prototype.bindValue = function(val, pos) {
    if (pos == null) {
      pos = this.pos++;
    }
    switch (typeof val) {
      case "string":
        return this.bindString(val, pos);
      case "number":
      case "boolean":
        return this.bindNumber(val + 0, pos);
      case "object":
        if (val === null) {
          return this.bindNull(pos);
        } else if (val.length != null) {
          return this.bindBlob(val, pos);
        } else {
          throw "Wrong API use : tried to bind a value of an unknown type (" + val + ").";
        }
    }
  };


  /* Bind names and values of an object to the named parameters of the statement
  @param [Object]
  @private
  @nodoc
   */

  Statement.prototype.bindFromObject = function(valuesObj) {
    var name, num, value;
    for (name in valuesObj) {
      value = valuesObj[name];
      num = sqlite3_bind_parameter_index(this.stmt, name);
      if (num !== 0) {
        this.bindValue(value, num);
      }
    }
    return true;
  };


  /* Bind values to numbered parameters
  @param [Array]
  @private
  @nodoc
   */

  Statement.prototype.bindFromArray = function(values) {
    var k, len, num, value;
    for (num = k = 0, len = values.length; k < len; num = ++k) {
      value = values[num];
      this.bindValue(value, num + 1);
    }
    return true;
  };


  /* Reset a statement, so that it's parameters can be bound to new values
  It also clears all previous bindings, freeing the memory used by bound parameters.
   */

  Statement.prototype['reset'] = function() {
    this.freemem();
    return sqlite3_clear_bindings(this.stmt) === SQLite.OK && sqlite3_reset(this.stmt) === SQLite.OK;
  };


  /* Free the memory allocated during parameter binding
   */

  Statement.prototype.freemem = function() {
    var mem;
    while (mem = this.allocatedmem.pop()) {
      _free(mem);
    }
    return null;
  };


  /* Free the memory used by the statement
  @return [Boolean] true in case of success
   */

  Statement.prototype['free'] = function() {
    var res;
    this.freemem();
    res = sqlite3_finalize(this.stmt) === SQLite.OK;
    delete this.db.statements[this.stmt];
    this.stmt = NULL;
    return res;
  };

  return Statement;

})();

Database = (function() {
  function Database(data) {
    this.filename = 'dbfile_' + (0xffffffff * Math.random() >>> 0);
    if (data != null) {
      FS.createDataFile('/', this.filename, data, true, true);
    }
    this.handleError(sqlite3_open(this.filename, apiTemp));
    this.db = getValue(apiTemp, 'i32');
    RegisterExtensionFunctions(this.db);
    this.statements = {};
  }


  /* Execute an SQL query, ignoring the rows it returns.
  
  @param sql [String] a string containing some SQL text to execute
  @param params [Array] (*optional*) When the SQL statement contains placeholders, you can pass them in here. They will be bound to the statement before it is executed.
  
  If you use the params argument, you **cannot** provide an sql string that contains several
  queries (separated by ';')
  
  @example Insert values in a table
      db.run("INSERT INTO test VALUES (:age, :name)", {':age':18, ':name':'John'});
  
  @return [Database] The database object (useful for method chaining)
   */

  Database.prototype['run'] = function(sql, params) {
    var stmt;
    if (!this.db) {
      throw "Database closed";
    }
    if (params) {
      stmt = this['prepare'](sql, params);
      stmt['step']();
      stmt['free']();
    } else {
      this.handleError(sqlite3_exec(this.db, sql, 0, 0, apiTemp));
    }
    return this;
  };


  /* Execute an SQL query, and returns the result.
  
  This is a wrapper against Database.prepare, Statement.step, Statement.get,
  and Statement.free.
  
  The result is an array of result elements. There are as many result elements
  as the number of statements in your sql string (statements are separated by a semicolon)
  
  Each result element is an object with two properties:
      'columns' : the name of the columns of the result (as returned by Statement.getColumnNames())
      'values' : an array of rows. Each row is itself an array of values
  
  ## Example use
  We have the following table, named *test* :
  
  | id | age |  name  |
  |:--:|:---:|:------:|
  | 1  |  1  | Ling   |
  | 2  |  18 | Paul   |
  | 3  |  3  | Markus |
  
  
  We query it like that:
  ```javascript
  var db = new SQL.Database();
  var res = db.exec("SELECT id FROM test; SELECT age,name FROM test;");
  ```
  
  `res` is now :
  ```javascript
      [
          {columns: ['id'], values:[[1],[2],[3]]},
          {columns: ['age','name'], values:[[1,'Ling'],[18,'Paul'],[3,'Markus']]}
      ]
  ```
  
  @param sql [String] a string containing some SQL text to execute
  @return [Array<QueryResults>] An array of results.
   */

  Database.prototype['exec'] = function(sql) {
    var curresult, nextSqlPtr, pStmt, pzTail, results, stack, stmt;
    if (!this.db) {
      throw "Database closed";
    }
    stack = Runtime.stackSave();
    nextSqlPtr = Runtime.stackAlloc(sql.length << 2 + 1);
    writeStringToMemory(sql, nextSqlPtr);
    pzTail = Runtime.stackAlloc(4);
    results = [];
    while (getValue(nextSqlPtr, 'i8') !== NULL) {
      setValue(apiTemp, 0, 'i32');
      setValue(pzTail, 0, 'i32');
      this.handleError(sqlite3_prepare_v2_sqlptr(this.db, nextSqlPtr, -1, apiTemp, pzTail));
      pStmt = getValue(apiTemp, 'i32');
      nextSqlPtr = getValue(pzTail, 'i32');
      if (pStmt === NULL) {
        continue;
      }
      stmt = new Statement(pStmt, this);
      curresult = null;
      while (stmt['step']()) {
        if (curresult === null) {
          curresult = {
            'columns': stmt['getColumnNames'](),
            'values': []
          };
          results.push(curresult);
        }
        curresult['values'].push(stmt['get']());
      }
      stmt['free']();
    }
    Runtime.stackRestore(stack);
    return results;
  };


  /* Execute an sql statement, and call a callback for each row of result.
  
  **Currently** this method is synchronous, it will not return until the callback has
  been called on every row of the result. But this might change.
  
  @param sql [String] A string of SQL text. Can contain placeholders that will be
  bound to the parameters given as the second argument
  @param params [Array<String,Number,null,Uint8Array>] (*optional*) Parameters to bind
  to the query
  @param callback [Function(Object)] A function that will be called on each row of result
  @param done [Function] A function that will be called when all rows have been retrieved
  
  @return [Database] The database object. Useful for method chaining
  
  @example Read values from a table
      db.each("SELECT name,age FROM users WHERE age >= $majority",
                      {$majority:18},
                      function(row){console.log(row.name + " is a grown-up.")}
                  );
   */

  Database.prototype['each'] = function(sql, params, callback, done) {
    var stmt;
    if (typeof params === 'function') {
      done = callback;
      callback = params;
      params = void 0;
    }
    stmt = this['prepare'](sql, params);
    while (stmt['step']()) {
      callback(stmt['getAsObject']());
    }
    stmt['free']();
    if (typeof done === 'function') {
      return done();
    }
  };


  /* Prepare an SQL statement
  @param sql [String] a string of SQL, that can contain placeholders ('?', ':VVV', ':AAA', '@AAA')
  @param params [Array] (*optional*) values to bind to placeholders
  @return [Statement] the resulting statement
  @throw [String] SQLite error
   */

  Database.prototype['prepare'] = function(sql, params) {
    var pStmt, stmt;
    setValue(apiTemp, 0, 'i32');
    this.handleError(sqlite3_prepare_v2(this.db, sql, -1, apiTemp, NULL));
    pStmt = getValue(apiTemp, 'i32');
    if (pStmt === NULL) {
      throw 'Nothing to prepare';
    }
    stmt = new Statement(pStmt, this);
    if (params != null) {
      stmt.bind(params);
    }
    this.statements[pStmt] = stmt;
    return stmt;
  };


  /* Exports the contents of the database to a binary array
  @return [Uint8Array] An array of bytes of the SQLite3 database file
   */

  Database.prototype['export'] = function() {
    var _, binaryDb, ref, stmt;
    ref = this.statements;
    for (_ in ref) {
      stmt = ref[_];
      stmt['free']();
    }
    this.handleError(sqlite3_close_v2(this.db));
    binaryDb = FS.readFile(this.filename, {
      encoding: 'binary'
    });
    this.handleError(sqlite3_open(this.filename, apiTemp));
    this.db = getValue(apiTemp, 'i32');
    return binaryDb;
  };


  /* Close the database, and all associated prepared statements.
  
  The memory associated to the database and all associated statements
  will be freed.
  
  **Warning**: A statement belonging to a database that has been closed cannot
  be used anymore.
  
  Databases **must** be closed, when you're finished with them, or the
  memory consumption will grow forever
   */

  Database.prototype['close'] = function() {
    var _, ref, stmt;
    ref = this.statements;
    for (_ in ref) {
      stmt = ref[_];
      stmt['free']();
    }
    this.handleError(sqlite3_close_v2(this.db));
    FS.unlink('/' + this.filename);
    return this.db = null;
  };


  /* Analyze a result code, return null if no error occured, and throw
  an error with a descriptive message otherwise
  @nodoc
   */

  Database.prototype.handleError = function(returnCode) {
    var errmsg;
    if (returnCode === SQLite.OK) {
      return null;
    } else {
      errmsg = sqlite3_errmsg(this.db);
      throw new Error(errmsg);
    }
  };


  /* Returns the number of rows modified, inserted or deleted by the
  most recently completed INSERT, UPDATE or DELETE statement on the
  database Executing any other type of SQL statement does not modify
  the value returned by this function.
  
  @return [Number] the number of rows modified
   */

  Database.prototype['getRowsModified'] = function() {
    return sqlite3_changes(this.db);
  };


  /* Register a custom function with SQLite
  @example Register a simple function
      db.create_function("addOne", function(x) {return x+1;})
      db.exec("SELECT addOne(1)") // = 2
  
  @param name [String] the name of the function as referenced in SQL statements.
  @param func [Function] the actual function to be executed.
   */

  Database.prototype['create_function'] = function(name, func) {
    var func_ptr, wrapped_func;
    wrapped_func = function(cx, argc, argv) {
      var arg, args, data_func, i, k, ref, result, value_ptr, value_type;
      args = [];
      for (i = k = 0, ref = argc; 0 <= ref ? k <= ref : k >= ref; i = 0 <= ref ? ++k : --k) {
        value_ptr = getValue(argv + (4 * i), 'i32');
        value_type = sqlite3_value_type(value_ptr);
        data_func = (function() {
          switch (false) {
            case value_type !== 1:
              return sqlite3_value_int;
            case value_type !== 2:
              return sqlite3_value_double;
            case value_type !== 3:
              return sqlite3_value_text;
            case value_type !== 4:
              return function(ptr) {
                var blob_arg, blob_ptr, j, l, ref1, size;
                size = sqlite3_value_bytes(ptr);
                blob_ptr = sqlite3_value_blob(ptr);
                blob_arg = new Uint8Array(size);
                for (j = l = 0, ref1 = size; 0 <= ref1 ? l < ref1 : l > ref1; j = 0 <= ref1 ? ++l : --l) {
                  blob_arg[j] = HEAP8[blob_ptr + j];
                }
                return blob_arg;
              };
            default:
              return function(ptr) {
                return null;
              };
          }
        })();
        arg = data_func(value_ptr);
        args.push(arg);
      }
      result = func.apply(null, args);
      if (!result) {
        return sqlite3_result_null(cx);
      } else {
        switch (typeof result) {
          case 'number':
            return sqlite3_result_double(cx, result);
          case 'string':
            return sqlite3_result_text(cx, result, -1, -1);
        }
      }
    };
    func_ptr = Runtime.addFunction(wrapped_func);
    this.handleError(sqlite3_create_function_v2(this.db, name, func.length, SQLite.UTF8, 0, func_ptr, 0, 0, 0));
    return this;
  };

  return Database;

})();

sqlite3_open = Module['cwrap']('sqlite3_open', 'number', ['string', 'number']);

sqlite3_close_v2 = Module['cwrap']('sqlite3_close_v2', 'number', ['number']);

sqlite3_exec = Module['cwrap']('sqlite3_exec', 'number', ['number', 'string', 'number', 'number', 'number']);

sqlite3_free = Module['cwrap']('sqlite3_free', '', ['number']);

sqlite3_changes = Module['cwrap']('sqlite3_changes', 'number', ['number']);

sqlite3_prepare_v2 = Module['cwrap']('sqlite3_prepare_v2', 'number', ['number', 'string', 'number', 'number', 'number']);

sqlite3_prepare_v2_sqlptr = Module['cwrap']('sqlite3_prepare_v2', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_text = Module['cwrap']('sqlite3_bind_text', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_blob = Module['cwrap']('sqlite3_bind_blob', 'number', ['number', 'number', 'number', 'number', 'number']);

sqlite3_bind_double = Module['cwrap']('sqlite3_bind_double', 'number', ['number', 'number', 'number']);

sqlite3_bind_int = Module['cwrap']('sqlite3_bind_int', 'number', ['number', 'number', 'number']);

sqlite3_bind_parameter_index = Module['cwrap']('sqlite3_bind_parameter_index', 'number', ['number', 'string']);

sqlite3_step = Module['cwrap']('sqlite3_step', 'number', ['number']);

sqlite3_errmsg = Module['cwrap']('sqlite3_errmsg', 'string', ['number']);

sqlite3_data_count = Module['cwrap']('sqlite3_data_count', 'number', ['number']);

sqlite3_column_double = Module['cwrap']('sqlite3_column_double', 'number', ['number', 'number']);

sqlite3_column_text = Module['cwrap']('sqlite3_column_text', 'string', ['number', 'number']);

sqlite3_column_blob = Module['cwrap']('sqlite3_column_blob', 'number', ['number', 'number']);

sqlite3_column_bytes = Module['cwrap']('sqlite3_column_bytes', 'number', ['number', 'number']);

sqlite3_column_type = Module['cwrap']('sqlite3_column_type', 'number', ['number', 'number']);

sqlite3_column_name = Module['cwrap']('sqlite3_column_name', 'string', ['number', 'number']);

sqlite3_reset = Module['cwrap']('sqlite3_reset', 'number', ['number']);

sqlite3_clear_bindings = Module['cwrap']('sqlite3_clear_bindings', 'number', ['number']);

sqlite3_finalize = Module['cwrap']('sqlite3_finalize', 'number', ['number']);

sqlite3_create_function_v2 = Module['cwrap']('sqlite3_create_function_v2', 'number', ['number', 'string', 'number', 'number', 'number', 'number', 'number', 'number', 'number']);

sqlite3_value_type = Module['cwrap']('sqlite3_value_type', 'number', ['number']);

sqlite3_value_bytes = Module['cwrap']('sqlite3_value_bytes', 'number', ['number']);

sqlite3_value_text = Module['cwrap']('sqlite3_value_text', 'string', ['number']);

sqlite3_value_int = Module['cwrap']('sqlite3_value_int', 'number', ['number']);

sqlite3_value_blob = Module['cwrap']('sqlite3_value_blob', 'number', ['number']);

sqlite3_value_double = Module['cwrap']('sqlite3_value_double', 'number', ['number']);

sqlite3_result_double = Module['cwrap']('sqlite3_result_double', '', ['number', 'number']);

sqlite3_result_null = Module['cwrap']('sqlite3_result_null', '', ['number']);

sqlite3_result_text = Module['cwrap']('sqlite3_result_text', '', ['number', 'string', 'number', 'number']);

RegisterExtensionFunctions = Module['cwrap']('RegisterExtensionFunctions', 'number', ['number']);

this['SQL'] = {
  'Database': Database
};

for (i in this['SQL']) {
  Module[i] = this['SQL'][i];
}

NULL = 0;

SQLite.OK = 0;

SQLite.ERROR = 1;

SQLite.INTERNAL = 2;

SQLite.PERM = 3;

SQLite.ABORT = 4;

SQLite.BUSY = 5;

SQLite.LOCKED = 6;

SQLite.NOMEM = 7;

SQLite.READONLY = 8;

SQLite.INTERRUPT = 9;

SQLite.IOERR = 10;

SQLite.CORRUPT = 11;

SQLite.NOTFOUND = 12;

SQLite.FULL = 13;

SQLite.CANTOPEN = 14;

SQLite.PROTOCOL = 15;

SQLite.EMPTY = 16;

SQLite.SCHEMA = 17;

SQLite.TOOBIG = 18;

SQLite.CONSTRAINT = 19;

SQLite.MISMATCH = 20;

SQLite.MISUSE = 21;

SQLite.NOLFS = 22;

SQLite.AUTH = 23;

SQLite.FORMAT = 24;

SQLite.RANGE = 25;

SQLite.NOTADB = 26;

SQLite.NOTICE = 27;

SQLite.WARNING = 28;

SQLite.ROW = 100;

SQLite.DONE = 101;

SQLite.INTEGER = 1;

SQLite.FLOAT = 2;

SQLite.TEXT = 3;

SQLite.BLOB = 4;

SQLite.NULL = 5;

SQLite.UTF8 = 1;

return this['SQL'];
})();
if (typeof module !== 'undefined') module.exports = SQL;
if (typeof define === 'function') define(SQL);
